// This prevents pollution of the global namespace
var SQL = (function () {
// The Module object: Our interface to the outside world. We import
// and export values on it, and do the work to get that through
// closure compiler if necessary. There are various ways Module can be used:
// 1. Not defined. We create it here
// 2. A function parameter, function(Module) { ..generated code.. }
// 3. pre-run appended it, var Module = {}; ..generated code..
// 4. External script tag defines var Module.
// We need to do an eval in order to handle the closure compiler
// case, where this code here is minified but Module was defined
// elsewhere (e.g. case 4 above). We also need to check if Module
// already exists (e.g. case 3 above).
// Note that if you want to run closure, and also to use Module
// after the generated code, you will need to define   var Module = {};
// before the code. Then that object will be used in the code, and you
// can continue to use Module afterwards as well.
var Module;
if (!Module) Module = (typeof Module !== 'undefined' ? Module : null) || {};

// Sometimes an existing Module object exists with properties
// meant to overwrite the default module functionality. Here
// we collect those properties and reapply _after_ we configure
// the current environment's defaults to avoid having to be so
// defensive during initialization.
var moduleOverrides = {};
for (var key in Module) {
  if (Module.hasOwnProperty(key)) {
    moduleOverrides[key] = Module[key];
  }
}

// The environment setup code below is customized to use Module.
// *** Environment setup code ***
var ENVIRONMENT_IS_NODE = typeof process === 'object' && typeof require === 'function';
var ENVIRONMENT_IS_WEB = typeof window === 'object';
var ENVIRONMENT_IS_WORKER = typeof importScripts === 'function';
var ENVIRONMENT_IS_SHELL = !ENVIRONMENT_IS_WEB && !ENVIRONMENT_IS_NODE && !ENVIRONMENT_IS_WORKER;

if (ENVIRONMENT_IS_NODE) {
  // Expose functionality in the same simple way that the shells work
  // Note that we pollute the global namespace here, otherwise we break in node
  if (!Module['print']) Module['print'] = function print(x) {
    process['stdout'].write(x + '\n');
  };
  if (!Module['printErr']) Module['printErr'] = function printErr(x) {
    process['stderr'].write(x + '\n');
  };

  var nodeFS = require('fs');
  var nodePath = require('path');

  Module['read'] = function read(filename, binary) {
    filename = nodePath['normalize'](filename);
    var ret = nodeFS['readFileSync'](filename);
    // The path is absolute if the normalized version is the same as the resolved.
    if (!ret && filename != nodePath['resolve'](filename)) {
      filename = path.join(__dirname, '..', 'src', filename);
      ret = nodeFS['readFileSync'](filename);
    }
    if (ret && !binary) ret = ret.toString();
    return ret;
  };

  Module['readBinary'] = function readBinary(filename) { return Module['read'](filename, true) };

  Module['load'] = function load(f) {
    globalEval(read(f));
  };

  if (process['argv'].length > 1) {
    Module['thisProgram'] = process['argv'][1].replace(/\\/g, '/');
  } else {
    Module['thisProgram'] = 'unknown-program';
  }

  Module['arguments'] = process['argv'].slice(2);

  if (typeof module !== 'undefined') {
    module['exports'] = Module;
  }

  process['on']('uncaughtException', function(ex) {
    // suppress ExitStatus exceptions from showing an error
    if (!(ex instanceof ExitStatus)) {
      throw ex;
    }
  });
}
else if (ENVIRONMENT_IS_SHELL) {
  if (!Module['print']) Module['print'] = print;
  if (typeof printErr != 'undefined') Module['printErr'] = printErr; // not present in v8 or older sm

  if (typeof read != 'undefined') {
    Module['read'] = read;
  } else {
    Module['read'] = function read() { throw 'no read() available (jsc?)' };
  }

  Module['readBinary'] = function readBinary(f) {
    if (typeof readbuffer === 'function') {
      return new Uint8Array(readbuffer(f));
    }
    var data = read(f, 'binary');
    assert(typeof data === 'object');
    return data;
  };

  if (typeof scriptArgs != 'undefined') {
    Module['arguments'] = scriptArgs;
  } else if (typeof arguments != 'undefined') {
    Module['arguments'] = arguments;
  }

  this['Module'] = Module;

}
else if (ENVIRONMENT_IS_WEB || ENVIRONMENT_IS_WORKER) {
  Module['read'] = function read(url) {
    var xhr = new XMLHttpRequest();
    xhr.open('GET', url, false);
    xhr.send(null);
    return xhr.responseText;
  };

  if (typeof arguments != 'undefined') {
    Module['arguments'] = arguments;
  }

  if (typeof console !== 'undefined') {
    if (!Module['print']) Module['print'] = function print(x) {
      console.log(x);
    };
    if (!Module['printErr']) Module['printErr'] = function printErr(x) {
      console.log(x);
    };
  } else {
    // Probably a worker, and without console.log. We can do very little here...
    var TRY_USE_DUMP = false;
    if (!Module['print']) Module['print'] = (TRY_USE_DUMP && (typeof(dump) !== "undefined") ? (function(x) {
      dump(x);
    }) : (function(x) {
      // self.postMessage(x); // enable this if you want stdout to be sent as messages
    }));
  }

  if (ENVIRONMENT_IS_WEB) {
    window['Module'] = Module;
  } else {
    Module['load'] = importScripts;
  }
}
else {
  // Unreachable because SHELL is dependant on the others
  throw 'Unknown runtime environment. Where are we?';
}

function globalEval(x) {
  eval.call(null, x);
}
if (!Module['load'] && Module['read']) {
  Module['load'] = function load(f) {
    globalEval(Module['read'](f));
  };
}
if (!Module['print']) {
  Module['print'] = function(){};
}
if (!Module['printErr']) {
  Module['printErr'] = Module['print'];
}
if (!Module['arguments']) {
  Module['arguments'] = [];
}
if (!Module['thisProgram']) {
  Module['thisProgram'] = './this.program';
}

// *** Environment setup code ***

// Closure helpers
Module.print = Module['print'];
Module.printErr = Module['printErr'];

// Callbacks
Module['preRun'] = [];
Module['postRun'] = [];

// Merge back in the overrides
for (var key in moduleOverrides) {
  if (moduleOverrides.hasOwnProperty(key)) {
    Module[key] = moduleOverrides[key];
  }
}



// === Preamble library stuff ===

// Documentation for the public APIs defined in this file must be updated in: 
//    site/source/docs/api_reference/preamble.js.rst
// A prebuilt local version of the documentation is available at: 
//    site/build/text/docs/api_reference/preamble.js.txt
// You can also build docs locally as HTML or other formats in site/
// An online HTML version (which may be of a different version of Emscripten)
//    is up at http://kripken.github.io/emscripten-site/docs/api_reference/preamble.js.html

//========================================
// Runtime code shared with compiler
//========================================

var Runtime = {
  setTempRet0: function (value) {
    tempRet0 = value;
  },
  getTempRet0: function () {
    return tempRet0;
  },
  stackSave: function () {
    return STACKTOP;
  },
  stackRestore: function (stackTop) {
    STACKTOP = stackTop;
  },
  getNativeTypeSize: function (type) {
    switch (type) {
      case 'i1': case 'i8': return 1;
      case 'i16': return 2;
      case 'i32': return 4;
      case 'i64': return 8;
      case 'float': return 4;
      case 'double': return 8;
      default: {
        if (type[type.length-1] === '*') {
          return Runtime.QUANTUM_SIZE; // A pointer
        } else if (type[0] === 'i') {
          var bits = parseInt(type.substr(1));
          assert(bits % 8 === 0);
          return bits/8;
        } else {
          return 0;
        }
      }
    }
  },
  getNativeFieldSize: function (type) {
    return Math.max(Runtime.getNativeTypeSize(type), Runtime.QUANTUM_SIZE);
  },
  STACK_ALIGN: 16,
  getAlignSize: function (type, size, vararg) {
    // we align i64s and doubles on 64-bit boundaries, unlike x86
    if (!vararg && (type == 'i64' || type == 'double')) return 8;
    if (!type) return Math.min(size, 8); // align structures internally to 64 bits
    return Math.min(size || (type ? Runtime.getNativeFieldSize(type) : 0), Runtime.QUANTUM_SIZE);
  },
  dynCall: function (sig, ptr, args) {
    if (args && args.length) {
      if (!args.splice) args = Array.prototype.slice.call(args);
      args.splice(0, 0, ptr);
      return Module['dynCall_' + sig].apply(null, args);
    } else {
      return Module['dynCall_' + sig].call(null, ptr);
    }
  },
  functionPointers: [],
  addFunction: function (func) {
    for (var i = 0; i < Runtime.functionPointers.length; i++) {
      if (!Runtime.functionPointers[i]) {
        Runtime.functionPointers[i] = func;
        return 2*(1 + i);
      }
    }
    throw 'Finished up all reserved function pointers. Use a higher value for RESERVED_FUNCTION_POINTERS.';
  },
  removeFunction: function (index) {
    Runtime.functionPointers[(index-2)/2] = null;
  },
  getAsmConst: function (code, numArgs) {
    // code is a constant string on the heap, so we can cache these
    if (!Runtime.asmConstCache) Runtime.asmConstCache = {};
    var func = Runtime.asmConstCache[code];
    if (func) return func;
    var args = [];
    for (var i = 0; i < numArgs; i++) {
      args.push(String.fromCharCode(36) + i); // $0, $1 etc
    }
    var source = Pointer_stringify(code);
    if (source[0] === '"') {
      // tolerate EM_ASM("..code..") even though EM_ASM(..code..) is correct
      if (source.indexOf('"', 1) === source.length-1) {
        source = source.substr(1, source.length-2);
      } else {
        // something invalid happened, e.g. EM_ASM("..code($0)..", input)
        abort('invalid EM_ASM input |' + source + '|. Please use EM_ASM(..code..) (no quotes) or EM_ASM({ ..code($0).. }, input) (to input values)');
      }
    }
    try {
      // Module is the only 'upvar', which we provide directly. We also provide FS for legacy support.
      var evalled = eval('(function(Module, FS) { return function(' + args.join(',') + '){ ' + source + ' } })')(Module, typeof FS !== 'undefined' ? FS : null);
    } catch(e) {
      Module.printErr('error in executing inline EM_ASM code: ' + e + ' on: \n\n' + source + '\n\nwith args |' + args + '| (make sure to use the right one out of EM_ASM, EM_ASM_ARGS, etc.)');
      throw e;
    }
    return Runtime.asmConstCache[code] = evalled;
  },
  warnOnce: function (text) {
    if (!Runtime.warnOnce.shown) Runtime.warnOnce.shown = {};
    if (!Runtime.warnOnce.shown[text]) {
      Runtime.warnOnce.shown[text] = 1;
      Module.printErr(text);
    }
  },
  funcWrappers: {},
  getFuncWrapper: function (func, sig) {
    assert(sig);
    if (!Runtime.funcWrappers[sig]) {
      Runtime.funcWrappers[sig] = {};
    }
    var sigCache = Runtime.funcWrappers[sig];
    if (!sigCache[func]) {
      sigCache[func] = function dynCall_wrapper() {
        return Runtime.dynCall(sig, func, arguments);
      };
    }
    return sigCache[func];
  },
  UTF8Processor: function () {
    var buffer = [];
    var needed = 0;
    this.processCChar = function (code) {
      code = code & 0xFF;

      if (buffer.length == 0) {
        if ((code & 0x80) == 0x00) {        // 0xxxxxxx
          return String.fromCharCode(code);
        }
        buffer.push(code);
        if ((code & 0xE0) == 0xC0) {        // 110xxxxx
          needed = 1;
        } else if ((code & 0xF0) == 0xE0) { // 1110xxxx
          needed = 2;
        } else {                            // 11110xxx
          needed = 3;
        }
        return '';
      }

      if (needed) {
        buffer.push(code);
        needed--;
        if (needed > 0) return '';
      }

      var c1 = buffer[0];
      var c2 = buffer[1];
      var c3 = buffer[2];
      var c4 = buffer[3];
      var ret;
      if (buffer.length == 2) {
        ret = String.fromCharCode(((c1 & 0x1F) << 6)  | (c2 & 0x3F));
      } else if (buffer.length == 3) {
        ret = String.fromCharCode(((c1 & 0x0F) << 12) | ((c2 & 0x3F) << 6)  | (c3 & 0x3F));
      } else {
        // http://mathiasbynens.be/notes/javascript-encoding#surrogate-formulae
        var codePoint = ((c1 & 0x07) << 18) | ((c2 & 0x3F) << 12) |
                        ((c3 & 0x3F) << 6)  | (c4 & 0x3F);
        ret = String.fromCharCode(
          (((codePoint - 0x10000) / 0x400)|0) + 0xD800,
          (codePoint - 0x10000) % 0x400 + 0xDC00);
      }
      buffer.length = 0;
      return ret;
    }
    this.processJSString = function processJSString(string) {
      /* TODO: use TextEncoder when present,
        var encoder = new TextEncoder();
        encoder['encoding'] = "utf-8";
        var utf8Array = encoder['encode'](aMsg.data);
      */
      string = unescape(encodeURIComponent(string));
      var ret = [];
      for (var i = 0; i < string.length; i++) {
        ret.push(string.charCodeAt(i));
      }
      return ret;
    }
  },
  getCompilerSetting: function (name) {
    throw 'You must build with -s RETAIN_COMPILER_SETTINGS=1 for Runtime.getCompilerSetting or emscripten_get_compiler_setting to work';
  },
  stackAlloc: function (size) { var ret = STACKTOP;STACKTOP = (STACKTOP + size)|0;STACKTOP = (((STACKTOP)+15)&-16); return ret; },
  staticAlloc: function (size) { var ret = STATICTOP;STATICTOP = (STATICTOP + size)|0;STATICTOP = (((STATICTOP)+15)&-16); return ret; },
  dynamicAlloc: function (size) { var ret = DYNAMICTOP;DYNAMICTOP = (DYNAMICTOP + size)|0;DYNAMICTOP = (((DYNAMICTOP)+15)&-16); if (DYNAMICTOP >= TOTAL_MEMORY) enlargeMemory();; return ret; },
  alignMemory: function (size,quantum) { var ret = size = Math.ceil((size)/(quantum ? quantum : 16))*(quantum ? quantum : 16); return ret; },
  makeBigInt: function (low,high,unsigned) { var ret = (unsigned ? ((+((low>>>0)))+((+((high>>>0)))*4294967296.0)) : ((+((low>>>0)))+((+((high|0)))*4294967296.0))); return ret; },
  GLOBAL_BASE: 8,
  QUANTUM_SIZE: 4,
  __dummy__: 0
}


Module['Runtime'] = Runtime;









//========================================
// Runtime essentials
//========================================

var __THREW__ = 0; // Used in checking for thrown exceptions.

var ABORT = false; // whether we are quitting the application. no code should run after this. set in exit() and abort()
var EXITSTATUS = 0;

var undef = 0;
// tempInt is used for 32-bit signed values or smaller. tempBigInt is used
// for 32-bit unsigned values or more than 32 bits. TODO: audit all uses of tempInt
var tempValue, tempInt, tempBigInt, tempInt2, tempBigInt2, tempPair, tempBigIntI, tempBigIntR, tempBigIntS, tempBigIntP, tempBigIntD, tempDouble, tempFloat;
var tempI64, tempI64b;
var tempRet0, tempRet1, tempRet2, tempRet3, tempRet4, tempRet5, tempRet6, tempRet7, tempRet8, tempRet9;

function assert(condition, text) {
  if (!condition) {
    abort('Assertion failed: ' + text);
  }
}

var globalScope = this;

// Returns the C function with a specified identifier (for C++, you need to do manual name mangling)
function getCFunc(ident) {
  var func = Module['_' + ident]; // closure exported function
  if (!func) {
    try {
      func = eval('_' + ident); // explicit lookup
    } catch(e) {}
  }
  assert(func, 'Cannot call unknown function ' + ident + ' (perhaps LLVM optimizations or closure removed it?)');
  return func;
}

var cwrap, ccall;
(function(){
  var JSfuncs = {
    // Helpers for cwrap -- it can't refer to Runtime directly because it might
    // be renamed by closure, instead it calls JSfuncs['stackSave'].body to find
    // out what the minified function name is.
    'stackSave': function() {
      Runtime.stackSave()
    },
    'stackRestore': function() {
      Runtime.stackRestore()
    },
    // type conversion from js to c
    'arrayToC' : function(arr) {
      var ret = Runtime.stackAlloc(arr.length);
      writeArrayToMemory(arr, ret);
      return ret;
    },
    'stringToC' : function(str) {
      var ret = 0;
      if (str !== null && str !== undefined && str !== 0) { // null string
        // at most 4 bytes per UTF-8 code point, +1 for the trailing '\0'
        ret = Runtime.stackAlloc((str.length << 2) + 1);
        writeStringToMemory(str, ret);
      }
      return ret;
    }
  };
  // For fast lookup of conversion functions
  var toC = {'string' : JSfuncs['stringToC'], 'array' : JSfuncs['arrayToC']};

  // C calling interface. 
  ccall = function ccallFunc(ident, returnType, argTypes, args) {
    var func = getCFunc(ident);
    var cArgs = [];
    var stack = 0;
    if (args) {
      for (var i = 0; i < args.length; i++) {
        var converter = toC[argTypes[i]];
        if (converter) {
          if (stack === 0) stack = Runtime.stackSave();
          cArgs[i] = converter(args[i]);
        } else {
          cArgs[i] = args[i];
        }
      }
    }
    var ret = func.apply(null, cArgs);
    if (returnType === 'string') ret = Pointer_stringify(ret);
    if (stack !== 0) Runtime.stackRestore(stack);
    return ret;
  }

  var sourceRegex = /^function\s*\(([^)]*)\)\s*{\s*([^*]*?)[\s;]*(?:return\s*(.*?)[;\s]*)?}$/;
  function parseJSFunc(jsfunc) {
    // Match the body and the return value of a javascript function source
    var parsed = jsfunc.toString().match(sourceRegex).slice(1);
    return {arguments : parsed[0], body : parsed[1], returnValue: parsed[2]}
  }
  var JSsource = {};
  for (var fun in JSfuncs) {
    if (JSfuncs.hasOwnProperty(fun)) {
      // Elements of toCsource are arrays of three items:
      // the code, and the return value
      JSsource[fun] = parseJSFunc(JSfuncs[fun]);
    }
  }

  
  cwrap = function cwrap(ident, returnType, argTypes) {
    argTypes = argTypes || [];
    var cfunc = getCFunc(ident);
    // When the function takes numbers and returns a number, we can just return
    // the original function
    var numericArgs = argTypes.every(function(type){ return type === 'number'});
    var numericRet = (returnType !== 'string');
    if ( numericRet && numericArgs) {
      return cfunc;
    }
    // Creation of the arguments list (["$1","$2",...,"$nargs"])
    var argNames = argTypes.map(function(x,i){return '$'+i});
    var funcstr = "(function(" + argNames.join(',') + ") {";
    var nargs = argTypes.length;
    if (!numericArgs) {
      // Generate the code needed to convert the arguments from javascript
      // values to pointers
      funcstr += 'var stack = ' + JSsource['stackSave'].body + ';';
      for (var i = 0; i < nargs; i++) {
        var arg = argNames[i], type = argTypes[i];
        if (type === 'number') continue;
        var convertCode = JSsource[type + 'ToC']; // [code, return]
        funcstr += 'var ' + convertCode.arguments + ' = ' + arg + ';';
        funcstr += convertCode.body + ';';
        funcstr += arg + '=' + convertCode.returnValue + ';';
      }
    }

    // When the code is compressed, the name of cfunc is not literally 'cfunc' anymore
    var cfuncname = parseJSFunc(function(){return cfunc}).returnValue;
    // Call the function
    funcstr += 'var ret = ' + cfuncname + '(' + argNames.join(',') + ');';
    if (!numericRet) { // Return type can only by 'string' or 'number'
      // Convert the result to a string
      var strgfy = parseJSFunc(function(){return Pointer_stringify}).returnValue;
      funcstr += 'ret = ' + strgfy + '(ret);';
    }
    if (!numericArgs) {
      // If we had a stack, restore it
      funcstr += JSsource['stackRestore'].body.replace('()', '(stack)') + ';';
    }
    funcstr += 'return ret})';
    return eval(funcstr);
  };
})();
Module["cwrap"] = cwrap;
Module["ccall"] = ccall;


function setValue(ptr, value, type, noSafe) {
  type = type || 'i8';
  if (type.charAt(type.length-1) === '*') type = 'i32'; // pointers are 32-bit
    switch(type) {
      case 'i1': HEAP8[((ptr)>>0)]=value; break;
      case 'i8': HEAP8[((ptr)>>0)]=value; break;
      case 'i16': HEAP16[((ptr)>>1)]=value; break;
      case 'i32': HEAP32[((ptr)>>2)]=value; break;
      case 'i64': (tempI64 = [value>>>0,(tempDouble=value,(+(Math_abs(tempDouble))) >= 1.0 ? (tempDouble > 0.0 ? ((Math_min((+(Math_floor((tempDouble)/4294967296.0))), 4294967295.0))|0)>>>0 : (~~((+(Math_ceil((tempDouble - +(((~~(tempDouble)))>>>0))/4294967296.0)))))>>>0) : 0)],HEAP32[((ptr)>>2)]=tempI64[0],HEAP32[(((ptr)+(4))>>2)]=tempI64[1]); break;
      case 'float': HEAPF32[((ptr)>>2)]=value; break;
      case 'double': HEAPF64[((ptr)>>3)]=value; break;
      default: abort('invalid type for setValue: ' + type);
    }
}
Module['setValue'] = setValue;


function getValue(ptr, type, noSafe) {
  type = type || 'i8';
  if (type.charAt(type.length-1) === '*') type = 'i32'; // pointers are 32-bit
    switch(type) {
      case 'i1': return HEAP8[((ptr)>>0)];
      case 'i8': return HEAP8[((ptr)>>0)];
      case 'i16': return HEAP16[((ptr)>>1)];
      case 'i32': return HEAP32[((ptr)>>2)];
      case 'i64': return HEAP32[((ptr)>>2)];
      case 'float': return HEAPF32[((ptr)>>2)];
      case 'double': return HEAPF64[((ptr)>>3)];
      default: abort('invalid type for setValue: ' + type);
    }
  return null;
}
Module['getValue'] = getValue;

var ALLOC_NORMAL = 0; // Tries to use _malloc()
var ALLOC_STACK = 1; // Lives for the duration of the current function call
var ALLOC_STATIC = 2; // Cannot be freed
var ALLOC_DYNAMIC = 3; // Cannot be freed except through sbrk
var ALLOC_NONE = 4; // Do not allocate
Module['ALLOC_NORMAL'] = ALLOC_NORMAL;
Module['ALLOC_STACK'] = ALLOC_STACK;
Module['ALLOC_STATIC'] = ALLOC_STATIC;
Module['ALLOC_DYNAMIC'] = ALLOC_DYNAMIC;
Module['ALLOC_NONE'] = ALLOC_NONE;

// allocate(): This is for internal use. You can use it yourself as well, but the interface
//             is a little tricky (see docs right below). The reason is that it is optimized
//             for multiple syntaxes to save space in generated code. So you should
//             normally not use allocate(), and instead allocate memory using _malloc(),
//             initialize it with setValue(), and so forth.
// @slab: An array of data, or a number. If a number, then the size of the block to allocate,
//        in *bytes* (note that this is sometimes confusing: the next parameter does not
//        affect this!)
// @types: Either an array of types, one for each byte (or 0 if no type at that position),
//         or a single type which is used for the entire block. This only matters if there
//         is initial data - if @slab is a number, then this does not matter at all and is
//         ignored.
// @allocator: How to allocate memory, see ALLOC_*
function allocate(slab, types, allocator, ptr) {
  var zeroinit, size;
  if (typeof slab === 'number') {
    zeroinit = true;
    size = slab;
  } else {
    zeroinit = false;
    size = slab.length;
  }

  var singleType = typeof types === 'string' ? types : null;

  var ret;
  if (allocator == ALLOC_NONE) {
    ret = ptr;
  } else {
    ret = [_malloc, Runtime.stackAlloc, Runtime.staticAlloc, Runtime.dynamicAlloc][allocator === undefined ? ALLOC_STATIC : allocator](Math.max(size, singleType ? 1 : types.length));
  }

  if (zeroinit) {
    var ptr = ret, stop;
    assert((ret & 3) == 0);
    stop = ret + (size & ~3);
    for (; ptr < stop; ptr += 4) {
      HEAP32[((ptr)>>2)]=0;
    }
    stop = ret + size;
    while (ptr < stop) {
      HEAP8[((ptr++)>>0)]=0;
    }
    return ret;
  }

  if (singleType === 'i8') {
    if (slab.subarray || slab.slice) {
      HEAPU8.set(slab, ret);
    } else {
      HEAPU8.set(new Uint8Array(slab), ret);
    }
    return ret;
  }

  var i = 0, type, typeSize, previousType;
  while (i < size) {
    var curr = slab[i];

    if (typeof curr === 'function') {
      curr = Runtime.getFunctionIndex(curr);
    }

    type = singleType || types[i];
    if (type === 0) {
      i++;
      continue;
    }

    if (type == 'i64') type = 'i32'; // special case: we have one i32 here, and one i32 later

    setValue(ret+i, curr, type);

    // no need to look up size unless type changes, so cache it
    if (previousType !== type) {
      typeSize = Runtime.getNativeTypeSize(type);
      previousType = type;
    }
    i += typeSize;
  }

  return ret;
}
Module['allocate'] = allocate;

function Pointer_stringify(ptr, /* optional */ length) {
  if (length === 0 || !ptr) return '';
  // TODO: use TextDecoder
  // Find the length, and check for UTF while doing so
  var hasUtf = false;
  var t;
  var i = 0;
  while (1) {
    t = HEAPU8[(((ptr)+(i))>>0)];
    if (t >= 128) hasUtf = true;
    else if (t == 0 && !length) break;
    i++;
    if (length && i == length) break;
  }
  if (!length) length = i;

  var ret = '';

  if (!hasUtf) {
    var MAX_CHUNK = 1024; // split up into chunks, because .apply on a huge string can overflow the stack
    var curr;
    while (length > 0) {
      curr = String.fromCharCode.apply(String, HEAPU8.subarray(ptr, ptr + Math.min(length, MAX_CHUNK)));
      ret = ret ? ret + curr : curr;
      ptr += MAX_CHUNK;
      length -= MAX_CHUNK;
    }
    return ret;
  }

  var utf8 = new Runtime.UTF8Processor();
  for (i = 0; i < length; i++) {
    t = HEAPU8[(((ptr)+(i))>>0)];
    ret += utf8.processCChar(t);
  }
  return ret;
}
Module['Pointer_stringify'] = Pointer_stringify;

function UTF16ToString(ptr) {
  var i = 0;

  var str = '';
  while (1) {
    var codeUnit = HEAP16[(((ptr)+(i*2))>>1)];
    if (codeUnit == 0)
      return str;
    ++i;
    // fromCharCode constructs a character from a UTF-16 code unit, so we can pass the UTF16 string right through.
    str += String.fromCharCode(codeUnit);
  }
}
Module['UTF16ToString'] = UTF16ToString;


function stringToUTF16(str, outPtr) {
  for(var i = 0; i < str.length; ++i) {
    // charCodeAt returns a UTF-16 encoded code unit, so it can be directly written to the HEAP.
    var codeUnit = str.charCodeAt(i); // possibly a lead surrogate
    HEAP16[(((outPtr)+(i*2))>>1)]=codeUnit;
  }
  // Null-terminate the pointer to the HEAP.
  HEAP16[(((outPtr)+(str.length*2))>>1)]=0;
}
Module['stringToUTF16'] = stringToUTF16;


function UTF32ToString(ptr) {
  var i = 0;

  var str = '';
  while (1) {
    var utf32 = HEAP32[(((ptr)+(i*4))>>2)];
    if (utf32 == 0)
      return str;
    ++i;
    // Gotcha: fromCharCode constructs a character from a UTF-16 encoded code (pair), not from a Unicode code point! So encode the code point to UTF-16 for constructing.
    if (utf32 >= 0x10000) {
      var ch = utf32 - 0x10000;
      str += String.fromCharCode(0xD800 | (ch >> 10), 0xDC00 | (ch & 0x3FF));
    } else {
      str += String.fromCharCode(utf32);
    }
  }
}
Module['UTF32ToString'] = UTF32ToString;


function stringToUTF32(str, outPtr) {
  var iChar = 0;
  for(var iCodeUnit = 0; iCodeUnit < str.length; ++iCodeUnit) {
    // Gotcha: charCodeAt returns a 16-bit word that is a UTF-16 encoded code unit, not a Unicode code point of the character! We must decode the string to UTF-32 to the heap.
    var codeUnit = str.charCodeAt(iCodeUnit); // possibly a lead surrogate
    if (codeUnit >= 0xD800 && codeUnit <= 0xDFFF) {
      var trailSurrogate = str.charCodeAt(++iCodeUnit);
      codeUnit = 0x10000 + ((codeUnit & 0x3FF) << 10) | (trailSurrogate & 0x3FF);
    }
    HEAP32[(((outPtr)+(iChar*4))>>2)]=codeUnit;
    ++iChar;
  }
  // Null-terminate the pointer to the HEAP.
  HEAP32[(((outPtr)+(iChar*4))>>2)]=0;
}
Module['stringToUTF32'] = stringToUTF32;

function demangle(func) {
  var hasLibcxxabi = !!Module['___cxa_demangle'];
  if (hasLibcxxabi) {
    try {
      var buf = _malloc(func.length);
      writeStringToMemory(func.substr(1), buf);
      var status = _malloc(4);
      var ret = Module['___cxa_demangle'](buf, 0, 0, status);
      if (getValue(status, 'i32') === 0 && ret) {
        return Pointer_stringify(ret);
      }
      // otherwise, libcxxabi failed, we can try ours which may return a partial result
    } catch(e) {
      // failure when using libcxxabi, we can try ours which may return a partial result
    } finally {
      if (buf) _free(buf);
      if (status) _free(status);
      if (ret) _free(ret);
    }
  }
  var i = 3;
  // params, etc.
  var basicTypes = {
    'v': 'void',
    'b': 'bool',
    'c': 'char',
    's': 'short',
    'i': 'int',
    'l': 'long',
    'f': 'float',
    'd': 'double',
    'w': 'wchar_t',
    'a': 'signed char',
    'h': 'unsigned char',
    't': 'unsigned short',
    'j': 'unsigned int',
    'm': 'unsigned long',
    'x': 'long long',
    'y': 'unsigned long long',
    'z': '...'
  };
  var subs = [];
  var first = true;
  function dump(x) {
    //return;
    if (x) Module.print(x);
    Module.print(func);
    var pre = '';
    for (var a = 0; a < i; a++) pre += ' ';
    Module.print (pre + '^');
  }
  function parseNested() {
    i++;
    if (func[i] === 'K') i++; // ignore const
    var parts = [];
    while (func[i] !== 'E') {
      if (func[i] === 'S') { // substitution
        i++;
        var next = func.indexOf('_', i);
        var num = func.substring(i, next) || 0;
        parts.push(subs[num] || '?');
        i = next+1;
        continue;
      }
      if (func[i] === 'C') { // constructor
        parts.push(parts[parts.length-1]);
        i += 2;
        continue;
      }
      var size = parseInt(func.substr(i));
      var pre = size.toString().length;
      if (!size || !pre) { i--; break; } // counter i++ below us
      var curr = func.substr(i + pre, size);
      parts.push(curr);
      subs.push(curr);
      i += pre + size;
    }
    i++; // skip E
    return parts;
  }
  function parse(rawList, limit, allowVoid) { // main parser
    limit = limit || Infinity;
    var ret = '', list = [];
    function flushList() {
      return '(' + list.join(', ') + ')';
    }
    var name;
    if (func[i] === 'N') {
      // namespaced N-E
      name = parseNested().join('::');
      limit--;
      if (limit === 0) return rawList ? [name] : name;
    } else {
      // not namespaced
      if (func[i] === 'K' || (first && func[i] === 'L')) i++; // ignore const and first 'L'
      var size = parseInt(func.substr(i));
      if (size) {
        var pre = size.toString().length;
        name = func.substr(i + pre, size);
        i += pre + size;
      }
    }
    first = false;
    if (func[i] === 'I') {
      i++;
      var iList = parse(true);
      var iRet = parse(true, 1, true);
      ret += iRet[0] + ' ' + name + '<' + iList.join(', ') + '>';
    } else {
      ret = name;
    }
    paramLoop: while (i < func.length && limit-- > 0) {
      //dump('paramLoop');
      var c = func[i++];
      if (c in basicTypes) {
        list.push(basicTypes[c]);
      } else {
        switch (c) {
          case 'P': list.push(parse(true, 1, true)[0] + '*'); break; // pointer
          case 'R': list.push(parse(true, 1, true)[0] + '&'); break; // reference
          case 'L': { // literal
            i++; // skip basic type
            var end = func.indexOf('E', i);
            var size = end - i;
            list.push(func.substr(i, size));
            i += size + 2; // size + 'EE'
            break;
          }
          case 'A': { // array
            var size = parseInt(func.substr(i));
            i += size.toString().length;
            if (func[i] !== '_') throw '?';
            i++; // skip _
            list.push(parse(true, 1, true)[0] + ' [' + size + ']');
            break;
          }
          case 'E': break paramLoop;
          default: ret += '?' + c; break paramLoop;
        }
      }
    }
    if (!allowVoid && list.length === 1 && list[0] === 'void') list = []; // avoid (void)
    if (rawList) {
      if (ret) {
        list.push(ret + '?');
      }
      return list;
    } else {
      return ret + flushList();
    }
  }
  var parsed = func;
  try {
    // Special-case the entry point, since its name differs from other name mangling.
    if (func == 'Object._main' || func == '_main') {
      return 'main()';
    }
    if (typeof func === 'number') func = Pointer_stringify(func);
    if (func[0] !== '_') return func;
    if (func[1] !== '_') return func; // C function
    if (func[2] !== 'Z') return func;
    switch (func[3]) {
      case 'n': return 'operator new()';
      case 'd': return 'operator delete()';
    }
    parsed = parse();
  } catch(e) {
    parsed += '?';
  }
  if (parsed.indexOf('?') >= 0 && !hasLibcxxabi) {
    Runtime.warnOnce('warning: a problem occurred in builtin C++ name demangling; build with  -s DEMANGLE_SUPPORT=1  to link in libcxxabi demangling');
  }
  return parsed;
}

function demangleAll(text) {
  return text.replace(/__Z[\w\d_]+/g, function(x) { var y = demangle(x); return x === y ? x : (x + ' [' + y + ']') });
}

function jsStackTrace() {
  var err = new Error();
  if (!err.stack) {
    // IE10+ special cases: It does have callstack info, but it is only populated if an Error object is thrown,
    // so try that as a special-case.
    try {
      throw new Error(0);
    } catch(e) {
      err = e;
    }
    if (!err.stack) {
      return '(no stack trace available)';
    }
  }
  return err.stack.toString();
}

function stackTrace() {
  return demangleAll(jsStackTrace());
}
Module['stackTrace'] = stackTrace;

// Memory management

var PAGE_SIZE = 4096;
function alignMemoryPage(x) {
  return (x+4095)&-4096;
}

var HEAP;
var HEAP8, HEAPU8, HEAP16, HEAPU16, HEAP32, HEAPU32, HEAPF32, HEAPF64;

var STATIC_BASE = 0, STATICTOP = 0, staticSealed = false; // static area
var STACK_BASE = 0, STACKTOP = 0, STACK_MAX = 0; // stack area
var DYNAMIC_BASE = 0, DYNAMICTOP = 0; // dynamic area handled by sbrk

function enlargeMemory() {
  abort('Cannot enlarge memory arrays. Either (1) compile with -s TOTAL_MEMORY=X with X higher than the current value ' + TOTAL_MEMORY + ', (2) compile with ALLOW_MEMORY_GROWTH which adjusts the size at runtime but prevents some optimizations, or (3) set Module.TOTAL_MEMORY before the program runs.');
}


var TOTAL_STACK = Module['TOTAL_STACK'] || 5242880;
var TOTAL_MEMORY = Module['TOTAL_MEMORY'] || 16777216;
var FAST_MEMORY = Module['FAST_MEMORY'] || 2097152;

var totalMemory = 64*1024;
while (totalMemory < TOTAL_MEMORY || totalMemory < 2*TOTAL_STACK) {
  if (totalMemory < 16*1024*1024) {
    totalMemory *= 2;
  } else {
    totalMemory += 16*1024*1024
  }
}
if (totalMemory !== TOTAL_MEMORY) {
  Module.printErr('increasing TOTAL_MEMORY to ' + totalMemory + ' to be compliant with the asm.js spec');
  TOTAL_MEMORY = totalMemory;
}

// Initialize the runtime's memory
// check for full engine support (use string 'subarray' to avoid closure compiler confusion)
assert(typeof Int32Array !== 'undefined' && typeof Float64Array !== 'undefined' && !!(new Int32Array(1)['subarray']) && !!(new Int32Array(1)['set']),
       'JS engine does not provide full typed array support');

var buffer = new ArrayBuffer(TOTAL_MEMORY);
HEAP8 = new Int8Array(buffer);
HEAP16 = new Int16Array(buffer);
HEAP32 = new Int32Array(buffer);
HEAPU8 = new Uint8Array(buffer);
HEAPU16 = new Uint16Array(buffer);
HEAPU32 = new Uint32Array(buffer);
HEAPF32 = new Float32Array(buffer);
HEAPF64 = new Float64Array(buffer);

// Endianness check (note: assumes compiler arch was little-endian)
HEAP32[0] = 255;
assert(HEAPU8[0] === 255 && HEAPU8[3] === 0, 'Typed arrays 2 must be run on a little-endian system');

Module['HEAP'] = HEAP;
Module['buffer'] = buffer;
Module['HEAP8'] = HEAP8;
Module['HEAP16'] = HEAP16;
Module['HEAP32'] = HEAP32;
Module['HEAPU8'] = HEAPU8;
Module['HEAPU16'] = HEAPU16;
Module['HEAPU32'] = HEAPU32;
Module['HEAPF32'] = HEAPF32;
Module['HEAPF64'] = HEAPF64;

function callRuntimeCallbacks(callbacks) {
  while(callbacks.length > 0) {
    var callback = callbacks.shift();
    if (typeof callback == 'function') {
      callback();
      continue;
    }
    var func = callback.func;
    if (typeof func === 'number') {
      if (callback.arg === undefined) {
        Runtime.dynCall('v', func);
      } else {
        Runtime.dynCall('vi', func, [callback.arg]);
      }
    } else {
      func(callback.arg === undefined ? null : callback.arg);
    }
  }
}

var __ATPRERUN__  = []; // functions called before the runtime is initialized
var __ATINIT__    = []; // functions called during startup
var __ATMAIN__    = []; // functions called when main() is to be run
var __ATEXIT__    = []; // functions called during shutdown
var __ATPOSTRUN__ = []; // functions called after the runtime has exited

var runtimeInitialized = false;
var runtimeExited = false;

function preRun() {
  // compatibility - merge in anything from Module['preRun'] at this time
  if (Module['preRun']) {
    if (typeof Module['preRun'] == 'function') Module['preRun'] = [Module['preRun']];
    while (Module['preRun'].length) {
      addOnPreRun(Module['preRun'].shift());
    }
  }
  callRuntimeCallbacks(__ATPRERUN__);
}

function ensureInitRuntime() {
  if (runtimeInitialized) return;
  runtimeInitialized = true;
  callRuntimeCallbacks(__ATINIT__);
}

function preMain() {
  callRuntimeCallbacks(__ATMAIN__);
}

function exitRuntime() {
  callRuntimeCallbacks(__ATEXIT__);
  runtimeExited = true;
}

function postRun() {
  // compatibility - merge in anything from Module['postRun'] at this time
  if (Module['postRun']) {
    if (typeof Module['postRun'] == 'function') Module['postRun'] = [Module['postRun']];
    while (Module['postRun'].length) {
      addOnPostRun(Module['postRun'].shift());
    }
  }
  callRuntimeCallbacks(__ATPOSTRUN__);
}

function addOnPreRun(cb) {
  __ATPRERUN__.unshift(cb);
}
Module['addOnPreRun'] = Module.addOnPreRun = addOnPreRun;

function addOnInit(cb) {
  __ATINIT__.unshift(cb);
}
Module['addOnInit'] = Module.addOnInit = addOnInit;

function addOnPreMain(cb) {
  __ATMAIN__.unshift(cb);
}
Module['addOnPreMain'] = Module.addOnPreMain = addOnPreMain;

function addOnExit(cb) {
  __ATEXIT__.unshift(cb);
}
Module['addOnExit'] = Module.addOnExit = addOnExit;

function addOnPostRun(cb) {
  __ATPOSTRUN__.unshift(cb);
}
Module['addOnPostRun'] = Module.addOnPostRun = addOnPostRun;

// Tools


function intArrayFromString(stringy, dontAddNull, length /* optional */) {
  var ret = (new Runtime.UTF8Processor()).processJSString(stringy);
  if (length) {
    ret.length = length;
  }
  if (!dontAddNull) {
    ret.push(0);
  }
  return ret;
}
Module['intArrayFromString'] = intArrayFromString;

function intArrayToString(array) {
  var ret = [];
  for (var i = 0; i < array.length; i++) {
    var chr = array[i];
    if (chr > 0xFF) {
      chr &= 0xFF;
    }
    ret.push(String.fromCharCode(chr));
  }
  return ret.join('');
}
Module['intArrayToString'] = intArrayToString;

function writeStringToMemory(string, buffer, dontAddNull) {
  var array = intArrayFromString(string, dontAddNull);
  var i = 0;
  while (i < array.length) {
    var chr = array[i];
    HEAP8[(((buffer)+(i))>>0)]=chr;
    i = i + 1;
  }
}
Module['writeStringToMemory'] = writeStringToMemory;

function writeArrayToMemory(array, buffer) {
  for (var i = 0; i < array.length; i++) {
    HEAP8[(((buffer)+(i))>>0)]=array[i];
  }
}
Module['writeArrayToMemory'] = writeArrayToMemory;

function writeAsciiToMemory(str, buffer, dontAddNull) {
  for (var i = 0; i < str.length; i++) {
    HEAP8[(((buffer)+(i))>>0)]=str.charCodeAt(i);
  }
  if (!dontAddNull) HEAP8[(((buffer)+(str.length))>>0)]=0;
}
Module['writeAsciiToMemory'] = writeAsciiToMemory;

function unSign(value, bits, ignore) {
  if (value >= 0) {
    return value;
  }
  return bits <= 32 ? 2*Math.abs(1 << (bits-1)) + value // Need some trickery, since if bits == 32, we are right at the limit of the bits JS uses in bitshifts
                    : Math.pow(2, bits)         + value;
}
function reSign(value, bits, ignore) {
  if (value <= 0) {
    return value;
  }
  var half = bits <= 32 ? Math.abs(1 << (bits-1)) // abs is needed if bits == 32
                        : Math.pow(2, bits-1);
  if (value >= half && (bits <= 32 || value > half)) { // for huge values, we can hit the precision limit and always get true here. so don't do that
                                                       // but, in general there is no perfect solution here. With 64-bit ints, we get rounding and errors
                                                       // TODO: In i64 mode 1, resign the two parts separately and safely
    value = -2*half + value; // Cannot bitshift half, as it may be at the limit of the bits JS uses in bitshifts
  }
  return value;
}

// check for imul support, and also for correctness ( https://bugs.webkit.org/show_bug.cgi?id=126345 )
if (!Math['imul'] || Math['imul'](0xffffffff, 5) !== -5) Math['imul'] = function imul(a, b) {
  var ah  = a >>> 16;
  var al = a & 0xffff;
  var bh  = b >>> 16;
  var bl = b & 0xffff;
  return (al*bl + ((ah*bl + al*bh) << 16))|0;
};
Math.imul = Math['imul'];


var Math_abs = Math.abs;
var Math_cos = Math.cos;
var Math_sin = Math.sin;
var Math_tan = Math.tan;
var Math_acos = Math.acos;
var Math_asin = Math.asin;
var Math_atan = Math.atan;
var Math_atan2 = Math.atan2;
var Math_exp = Math.exp;
var Math_log = Math.log;
var Math_sqrt = Math.sqrt;
var Math_ceil = Math.ceil;
var Math_floor = Math.floor;
var Math_pow = Math.pow;
var Math_imul = Math.imul;
var Math_fround = Math.fround;
var Math_min = Math.min;

// A counter of dependencies for calling run(). If we need to
// do asynchronous work before running, increment this and
// decrement it. Incrementing must happen in a place like
// PRE_RUN_ADDITIONS (used by emcc to add file preloading).
// Note that you can add dependencies in preRun, even though
// it happens right before run - run will be postponed until
// the dependencies are met.
var runDependencies = 0;
var runDependencyWatcher = null;
var dependenciesFulfilled = null; // overridden to take different actions when all run dependencies are fulfilled

function addRunDependency(id) {
  runDependencies++;
  if (Module['monitorRunDependencies']) {
    Module['monitorRunDependencies'](runDependencies);
  }
}
Module['addRunDependency'] = addRunDependency;
function removeRunDependency(id) {
  runDependencies--;
  if (Module['monitorRunDependencies']) {
    Module['monitorRunDependencies'](runDependencies);
  }
  if (runDependencies == 0) {
    if (runDependencyWatcher !== null) {
      clearInterval(runDependencyWatcher);
      runDependencyWatcher = null;
    }
    if (dependenciesFulfilled) {
      var callback = dependenciesFulfilled;
      dependenciesFulfilled = null;
      callback(); // can add another dependenciesFulfilled
    }
  }
}
Module['removeRunDependency'] = removeRunDependency;

Module["preloadedImages"] = {}; // maps url to image data
Module["preloadedAudios"] = {}; // maps url to audio data


var memoryInitializer = null;

// === Body ===





STATIC_BASE = 8;

STATICTOP = STATIC_BASE + 39504;
  /* global initializers */ __ATINIT__.push();
  

/* memory initializer */ allocate([51,46,56,46,55,46,52,0,83,81,76,73,84,69,95,0,0,152,0,0,16,152,0,0,32,152,0,0,56,152,0,0,72,152,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,2,3,4,5,6,7,8,9,10,11,12,13,14,15,16,17,18,19,20,21,22,23,24,25,26,27,28,29,30,31,32,33,34,35,36,37,38,39,40,41,42,43,44,45,46,47,48,49,50,51,52,53,54,55,56,57,58,59,60,61,62,63,64,97,98,99,100,101,102,103,104,105,106,107,108,109,110,111,112,113,114,115,116,117,118,119,120,121,122,91,92,93,94,95,96,97,98,99,100,101,102,103,104,105,106,107,108,109,110,111,112,113,114,115,116,117,118,119,120,121,122,123,124,125,126,127,128,129,130,131,132,133,134,135,136,137,138,139,140,141,142,143,144,145,146,147,148,149,150,151,152,153,154,155,156,157,158,159,160,161,162,163,164,165,166,167,168,169,170,171,172,173,174,175,176,177,178,179,180,181,182,183,184,185,186,187,188,189,190,191,192,193,194,195,196,197,198,199,200,201,202,203,204,205,206,207,208,209,210,211,212,213,214,215,216,217,218,219,220,221,222,223,224,225,226,227,228,229,230,231,232,233,234,235,236,237,238,239,240,241,242,243,244,245,246,247,248,249,250,251,252,253,254,255,3,0,0,0,44,0,0,0,0,2,0,0,0,0,0,0,24,4,0,0,32,4,0,0,1,0,0,0,1,0,0,0,1,0,0,0,2,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,2,0,0,0,1,0,0,0,2,0,0,0,3,0,0,0,3,0,0,0,4,0,0,0,4,0,0,0,5,0,0,0,3,0,0,0,44,0,0,0,0,2,0,0,0,0,0,0,40,4,0,0,56,4,0,0,1,0,0,0,1,0,0,0,1,0,0,0,2,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,2,0,0,0,1,0,0,0,2,0,0,0,3,0,0,0,3,0,0,0,4,0,0,0,4,0,0,0,5,0,0,0,3,0,0,0,44,0,0,0,0,2,0,0,0,0,0,0,64,4,0,0,80,4,0,0,1,0,0,0,1,0,0,0,1,0,0,0,2,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,2,0,0,0,1,0,0,0,2,0,0,0,3,0,0,0,3,0,0,0,4,0,0,0,4,0,0,0,5,0,0,0,3,0,0,0,44,0,0,0,0,2,0,0,0,0,0,0,88,4,0,0,32,4,0,0,1,0,0,0,1,0,0,0,1,0,0,0,2,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,2,0,0,0,1,0,0,0,2,0,0,0,3,0,0,0,3,0,0,0,4,0,0,0,4,0,0,0,5,0,0,0,117,110,105,120,0,0,0,0,6,0,0,0,0,0,0,0,117,110,105,120,45,110,111,110,101,0,0,0,0,0,0,0,7,0,0,0,0,0,0,0,117,110,105,120,45,100,111,116,102,105,108,101,0,0,0,0,8,0,0,0,0,0,0,0,117,110,105,120,45,101,120,99,108,0,0,0,0,0,0,0,0,0,0,64,0,0,0,0,1,0,0,0,0,0,0,0,4,5,3,5,1,5,1,5,2,5,2,5,1,5,1,5,4,5,3,5,1,5,1,5,2,5,2,5,1,5,1,5,115,116,114,105,110,103,32,111,114,32,98,108,111,98,32,116,111,111,32,98,105,103,0,0,37,115,0,0,0,0,0,0,42,63,91,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,1,1,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,64,0,0,0,0,0,0,0,0,0,0,0,12,12,12,12,12,12,12,12,12,12,0,0,0,0,0,0,0,10,10,10,10,10,10,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,0,0,0,0,64,0,42,42,42,42,42,42,34,34,34,34,34,34,34,34,34,34,34,34,34,34,34,34,34,34,34,34,0,0,0,0,0,64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,0,0,0,0,0,0,0,0,116,114,105,103,103,101,114,0,116,101,109,112,0,0,0,0,50,48,49,52,45,49,50,45,48,57,32,48,49,58,51,52,58,51,54,32,102,54,54,102,55,97,49,55,98,55,56,98,97,54,49,55,97,99,100,101,57,48,102,99,56,49,48,49,48,55,102,51,52,102,49,97,49,102,50,101,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,117,110,107,110,111,119,110,32,100,97,116,97,98,97,115,101,58,32,37,115,0,0,0,0,0,202,154,59,0,202,154,59,208,7,0,0,232,3,0,0,244,1,0,0,168,97,0,0,127,0,0,0,10,0,0,0,80,195,0,0,231,3,0,0,232,3,0,0,0,0,0,0,57,50,50,51,51,55,50,48,51,54,56,53,52,55,55,53,56,48,0,0,0,0,0,0,111,110,111,102,102,97,108,115,101,121,101,115,116,114,117,101,102,117,108,108,0,0,0,0,0,1,2,4,9,12,16,0,2,2,3,5,3,4,4,0,1,0,0,0,1,1,2,0,82,69,73,78,68,69,88,69,68,69,83,67,65,80,69,65,67,72,69,67,75,69,89,66,69,70,79,82,69,73,71,78,79,82,69,71,69,88,80,76,65,73,78,83,84,69,65,68,68,65,84,65,66,65,83,69,76,69,67,84,65,66,76,69,70,84,72,69,78,68,69,70,69,82,82,65,66,76,69,76,83,69,88,67,69,80,84,82,65,78,83,65,67,84,73,79,78,65,84,85,82,65,76,84,69,82,65,73,83,69,88,67,76,85,83,73,86,69,88,73,83,84,83,65,86,69,80,79,73,78,84,69,82,83,69,67,84,82,73,71,71,69,82,69,70,69,82,69,78,67,69,83,67,79,78,83,84,82,65,73,78,84,79,70,70,83,69,84,69,77,80,79,82,65,82,89,85,78,73,81,85,69,82,89,87,73,84,72,79,85,84,69,82,69,76,69,65,83,69,65,84,84,65,67,72,65,86,73,78,71,82,79,85,80,68,65,84,69,66,69,71,73,78,78,69,82,69,67,85,82,83,73,86,69,66,69,84,87,69,69,78,79,84,78,85,76,76,73,75,69,67,65,83,67,65,68,69,76,69,84,69,67,65,83,69,67,79,76,76,65,84,69,67,82,69,65,84,69,67,85,82,82,69,78,84,95,68,65,84,69,68,69,84,65,67,72,73,77,77,69,68,73,65,84,69,74,79,73,78,83,69,82,84,77,65,84,67,72,80,76,65,78,65,76,89,90,69,80,82,65,71,77,65,66,79,82,84,86,65,76,85,69,83,86,73,82,84,85,65,76,73,77,73,84,87,72,69,78,87,72,69,82,69,78,65,77,69,65,70,84,69,82,69,80,76,65,67,69,65,78,68,69,70,65,85,76,84,65,85,84,79,73,78,67,82,69,77,69,78,84,67,65,83,84,67,79,76,85,77,78,67,79,77,77,73,84,67,79,78,70,76,73,67,84,67,82,79,83,83,67,85,82,82,69,78,84,95,84,73,77,69,83,84,65,77,80,82,73,77,65,82,89,68,69,70,69,82,82,69,68,73,83,84,73,78,67,84,68,82,79,80,70,65,73,76,70,82,79,77,70,85,76,76,71,76,79,66,89,73,70,73,83,78,85,76,76,79,82,68,69,82,69,83,84,82,73,67,84,82,73,71,72,84,82,79,76,76,66,65,67,75,82,79,87,85,78,73,79,78,85,83,73,78,71,86,65,67,85,85,77,86,73,69,87,73,78,73,84,73,65,76,76,89,0,0,0,0,0,0,0,76,105,117,74,0,45,0,0,82,0,77,0,0,42,12,78,15,0,116,85,54,112,0,19,0,0,121,0,119,115,0,22,93,0,9,0,0,70,71,0,69,6,0,48,90,102,0,118,101,0,0,44,0,103,24,0,17,0,122,53,23,0,5,110,25,96,0,0,124,106,60,123,57,28,55,0,91,0,100,26,0,99,0,0,0,95,92,97,88,109,14,39,108,0,81,0,18,89,111,32,0,120,80,113,62,46,84,0,0,94,40,59,114,0,36,0,0,29,0,86,63,64,0,20,61,0,56,0,0,0,0,0,4,0,0,0,0,0,0,0,0,0,2,0,0,0,0,0,0,13,0,0,0,0,0,7,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,33,0,21,0,0,0,0,0,50,0,43,3,47,0,0,0,0,30,0,58,0,38,0,0,0,1,66,0,0,67,0,41,0,0,0,0,0,0,49,65,0,0,0,0,31,52,16,34,10,0,0,0,0,0,0,0,11,72,79,0,8,0,104,98,0,107,0,87,0,75,51,0,27,37,73,83,0,35,68,0,0,0,0,0,0,7,7,5,4,6,4,5,3,6,7,3,6,6,7,7,3,8,2,6,5,4,4,3,10,4,6,11,6,2,7,5,5,9,6,9,9,7,10,10,4,6,2,3,9,4,2,6,5,7,4,5,7,6,6,5,6,5,5,9,7,7,3,2,4,4,7,3,6,4,7,6,12,6,9,4,6,5,4,7,6,5,6,7,5,4,5,6,5,7,3,7,13,2,2,4,6,6,8,5,17,12,7,8,8,2,4,4,4,4,4,2,2,6,5,8,5,8,3,5,5,6,4,9,3,0,0,0,0,0,0,2,0,2,0,8,0,9,0,14,0,16,0,20,0,23,0,25,0,25,0,29,0,33,0,36,0,41,0,46,0,48,0,53,0,54,0,59,0,62,0,65,0,67,0,69,0,78,0,81,0,86,0,91,0,95,0,96,0,101,0,105,0,109,0,117,0,122,0,128,0,136,0,142,0,152,0,159,0,162,0,162,0,165,0,167,0,167,0,171,0,176,0,179,0,184,0,184,0,188,0,192,0,199,0,204,0,209,0,212,0,218,0,221,0,225,0,234,0,240,0,240,0,240,0,243,0,246,0,250,0,251,0,255,0,5,1,9,1,16,1,22,1,34,1,40,1,49,1,51,1,57,1,62,1,64,1,71,1,76,1,81,1,87,1,93,1,98,1,102,1,105,1,111,1,115,1,122,1,124,1,131,1,133,1,135,1,144,1,148,1,154,1,160,1,168,1,173,1,173,1,189,1,196,1,203,1,204,1,211,1,215,1,219,1,223,1,227,1,230,1,232,1,234,1,240,1,244,1,252,1,1,2,9,2,12,2,17,2,22,2,28,2,32,2,37,2,67,28,140,42,84,44,104,53,35,113,46,47,50,2,49,142,41,24,119,16,98,138,11,112,139,117,6,30,107,98,141,57,9,20,13,118,62,105,99,131,55,54,111,21,21,71,103,3,25,66,98,14,34,128,127,110,5,98,58,74,77,19,52,101,50,37,33,109,136,95,17,69,43,8,124,108,51,4,32,56,29,120,65,129,137,130,68,31,59,72,100,106,15,75,38,39,10,40,98,69,69,102,7,121,73,114,45,123,98,50,36,18,76,126,60,98,12,61,115,125,63,64,48,116,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,109,105,115,117,115,101,32,97,116,32,108,105,110,101,32,37,100,32,111,102,32,91,37,46,49,48,115,93,0,0,0,0,117,110,97,98,108,101,32,116,111,32,100,101,108,101,116,101,47,109,111,100,105,102,121,32,99,111,108,108,97,116,105,111,110,32,115,101,113,117,101,110,99,101,32,100,117,101,32,116,111,32,97,99,116,105,118,101,32,115,116,97,116,101,109,101,110,116,115,0,0,0,0,0,66,73,78,65,82,89,0,0,82,84,82,73,77,0,0,0,78,79,67,65,83,69,0,0,109,97,105,110,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,97,117,116,111,109,97,116,105,99,32,101,120,116,101,110,115,105,111,110,32,108,111,97,100,105,110,103,32,102,97,105,108,101,100,58,32,37,115,0,0,77,65,84,67,72,0,0,0,58,109,101,109,111,114,121,58,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,1,0,0,0,2,0,0,0,3,0,0,0,5,0,0,0,9,0,0,0,10,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,114,101,99,111,118,101,114,101,100,32,37,100,32,112,97,103,101,115,32,102,114,111,109,32,37,115,0,0,0,0,0,0,217,213,5,249,32,161,99,215,100,97,116,97,98,97,115,101,32,99,111,114,114,117,112,116,105,111,110,32,97,116,32,108,105,110,101,32,37,100,32,111,102,32,91,37,46,49,48,115,93,0,0,0,0,0,0,0,99,97,110,110,111,116,32,108,105,109,105,116,32,87,65,76,32,115,105,122,101,58,32,37,115,0,0,0,0,0,0,0,99,97,110,110,111,116,32,111,112,101,110,32,102,105,108,101,32,97,116,32,108,105,110,101,32,37,100,32,111,102,32,91,37,46,49,48,115,93,0,0,114,101,99,111,118,101,114,101,100,32,37,100,32,102,114,97,109,101,115,32,102,114,111,109,32,87,65,76,32,102,105,108,101,32,37,115,0,0,0,0,45,106,111,117,114,110,97,108,0,0,0,0,0,0,0,0,45,119,97,108,0,0,0,0,110,111,108,111,99,107,0,0,105,109,109,117,116,97,98,108,101,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,102,105,108,101,58,0,0,0,108,111,99,97,108,104,111,115,116,0,0,0,0,0,0,0,105,110,118,97,108,105,100,32,117,114,105,32,97,117,116,104,111,114,105,116,121,58,32,37,46,42,115,0,0,0,0,0,118,102,115,0,0,0,0,0,99,97,99,104,101,0,0,0,96,18,0,0,0,0,2,0,104,18,0,0,0,0,4,0,0,0,0,0,0,0,0,0,115,104,97,114,101,100,0,0,112,114,105,118,97,116,101,0,109,111,100,101,0,0,0,0,160,18,0,0,1,0,0,0,168,18,0,0,2,0,0,0,176,18,0,0,6,0,0,0,184,18,0,0,128,0,0,0,0,0,0,0,0,0,0,0,114,111,0,0,0,0,0,0,114,119,0,0,0,0,0,0,114,119,99,0,0,0,0,0,109,101,109,111,114,121,0,0,97,99,99,101,115,115,0,0,110,111,32,115,117,99,104,32,37,115,32,109,111,100,101,58,32,37,115,0,0,0,0,0,37,115,32,109,111,100,101,32,110,111,116,32,97,108,108,111,119,101,100,58,32,37,115,0,110,111,32,115,117,99,104,32,118,102,115,58,32,37,115,0,105,110,118,97,108,105,100,0,65,80,73,32,99,97,108,108,32,119,105,116,104,32,37,115,32,100,97,116,97,98,97,115,101,32,99,111,110,110,101,99,116,105,111,110,32,112,111,105,110,116,101,114,0,0,0,0,117,110,97,98,108,101,32,116,111,32,117,115,101,32,102,117,110,99,116,105,111,110,32,37,115,32,105,110,32,116,104,101,32,114,101,113,117,101,115,116,101,100,32,99,111,110,116,101,120,116,0,0,0,0,0,0,0,1,2,3,4,5,6,7,8,9,10,11,12,13,14,15,16,17,18,19,20,21,22,23,24,25,26,27,28,29,30,31,0,1,2,3,4,5,6,7,8,9,10,11,12,13,14,15,0,1,2,3,4,5,6,7,0,1,2,3,0,1,0,0,117,110,97,98,108,101,32,116,111,32,100,101,108,101,116,101,47,109,111,100,105,102,121,32,117,115,101,114,45,102,117,110,99,116,105,111,110,32,100,117,101,32,116,111,32,97,99,116,105,118,101,32,115,116,97,116,101,109,101,110,116,115,0,0,104,20,0,0,120,20,0,0,0,0,0,0,160,20,0,0,192,20,0,0,224,20,0,0,248,20,0,0,24,21,0,0,40,21,0,0,80,21,0,0,96,21,0,0,112,21,0,0,152,21,0,0,176,21,0,0,208,21,0,0,240,21,0,0,8,22,0,0,32,22,0,0,152,4,0,0,64,22,0,0,88,22,0,0,112,22,0,0,152,22,0,0,184,22,0,0,208,22,0,0,240,22,0,0,24,23,0,0,0,0,0,0,110,111,116,32,97,110,32,101,114,114,111,114,0,0,0,0,83,81,76,32,108,111,103,105,99,32,101,114,114,111,114,32,111,114,32,109,105,115,115,105,110,103,32,100,97,116,97,98,97,115,101,0,0,0,0,0,97,99,99,101,115,115,32,112,101,114,109,105,115,115,105,111,110,32,100,101,110,105,101,100,0,0,0,0,0,0,0,0,99,97,108,108,98,97,99,107,32,114,101,113,117,101,115,116,101,100,32,113,117,101,114,121,32,97,98,111,114,116,0,0,100,97,116,97,98,97,115,101,32,105,115,32,108,111,99,107,101,100,0,0,0,0,0,0,100,97,116,97,98,97,115,101,32,116,97,98,108,101,32,105,115,32,108,111,99,107,101,100,0,0,0,0,0,0,0,0,111,117,116,32,111,102,32,109,101,109,111,114,121,0,0,0,97,116,116,101,109,112,116,32,116,111,32,119,114,105,116,101,32,97,32,114,101,97,100,111,110,108,121,32,100,97,116,97,98,97,115,101,0,0,0,0,105,110,116,101,114,114,117,112,116,101,100,0,0,0,0,0,100,105,115,107,32,73,47,79,32,101,114,114,111,114,0,0,100,97,116,97,98,97,115,101,32,100,105,115,107,32,105,109,97,103,101,32,105,115,32,109,97,108,102,111,114,109,101,100,0,0,0,0,0,0,0,0,117,110,107,110,111,119,110,32,111,112,101,114,97,116,105,111,110,0,0,0,0,0,0,0,100,97,116,97,98,97,115,101,32,111,114,32,100,105,115,107,32,105,115,32,102,117,108,108,0,0,0,0,0,0,0,0,117,110,97,98,108,101,32,116,111,32,111,112,101,110,32,100,97,116,97,98,97,115,101,32,102,105,108,101,0,0,0,0,108,111,99,107,105,110,103,32,112,114,111,116,111,99,111,108,0,0,0,0,0,0,0,0,116,97,98,108,101,32,99,111,110,116,97,105,110,115,32,110,111,32,100,97,116,97,0,0,100,97,116,97,98,97,115,101,32,115,99,104,101,109,97,32,104,97,115,32,99,104,97,110,103,101,100,0,0,0,0,0,99,111,110,115,116,114,97,105,110,116,32,102,97,105,108,101,100,0,0,0,0,0,0,0,100,97,116,97,116,121,112,101,32,109,105,115,109,97,116,99,104,0,0,0,0,0,0,0,108,105,98,114,97,114,121,32,114,111,117,116,105,110,101,32,99,97,108,108,101,100,32,111,117,116,32,111,102,32,115,101,113,117,101,110,99,101,0,0,108,97,114,103,101,32,102,105,108,101,32,115,117,112,112,111,114,116,32,105,115,32,100,105,115,97,98,108,101,100,0,0,97,117,116,104,111,114,105,122,97,116,105,111,110,32,100,101,110,105,101,100,0,0,0,0,97,117,120,105,108,105,97,114,121,32,100,97,116,97,98,97,115,101,32,102,111,114,109,97,116,32,101,114,114,111,114,0,98,105,110,100,32,111,114,32,99,111,108,117,109,110,32,105,110,100,101,120,32,111,117,116,32,111,102,32,114,97,110,103,101,0,0,0,0,0,0,0,102,105,108,101,32,105,115,32,101,110,99,114,121,112,116,101,100,32,111,114,32,105,115,32,110,111,116,32,97,32,100,97,116,97,98,97,115,101,0,0,117,110,107,110,111,119,110,32,101,114,114,111,114,0,0,0,97,98,111,114,116,32,100,117,101,32,116,111,32,82,79,76,76,66,65,67,75,0,0,0,117,110,97,98,108,101,32,116,111,32,99,108,111,115,101,32,100,117,101,32,116,111,32,117,110,102,105,110,97,108,105,122,101,100,32,115,116,97,116,101,109,101,110,116,115,32,111,114,32,117,110,102,105,110,105,115,104,101,100,32,98,97,99,107,117,112,115,0,0,0,0,0,1,0,0,0,0,0,0,0,2,0,0,0,1,0,0,0,6,0,0,0,1,0,0,0,3,0,0,0,7,0,0,0,1,0,0,0,1,0,0,0,2,0,0,0,2,0,0,0,3,0,0,0,0,0,0,0,4,0,0,0,4,0,0,0,11,0,0,0,5,0,0,0,6,0,0,0,7,0,0,0,5,0,0,0,0,0,0,0,102,97,105,108,101,100,32,109,101,109,111,114,121,32,114,101,115,105,122,101,32,37,117,32,116,111,32,37,117,32,98,121,116,101,115,0,0,0,0,0,102,97,105,108,101,100,32,116,111,32,97,108,108,111,99,97,116,101,32,37,117,32,98,121,116,101,115,32,111,102,32,109,101,109,111,114,121,0,0,0,1,0,1,8,1,0,0,0,0,0,0,0,2,0,0,0,0,0,0,0,0,0,0,0,96,32,0,0,0,0,0,0,0,0,0,0,2,0,1,8,1,0,0,0,0,0,0,0,2,0,0,0,0,0,0,0,0,0,0,0,96,32,0,0,0,0,0,0,0,0,0,0,1,0,1,8,2,0,0,0,0,0,0,0,2,0,0,0,0,0,0,0,0,0,0,0,104,32,0,0,0,0,0,0,0,0,0,0,2,0,1,8,2,0,0,0,0,0,0,0,2,0,0,0,0,0,0,0,0,0,0,0,104,32,0,0,0,0,0,0,0,0,0,0,1,0,1,8,3,0,0,0,0,0,0,0,2,0,0,0,0,0,0,0,0,0,0,0,112,32,0,0,0,0,0,0,0,0,0,0,2,0,1,8,3,0,0,0,0,0,0,0,2,0,0,0,0,0,0,0,0,0,0,0,112,32,0,0,0,0,0,0,0,0,0,0,255,255,33,8,0,0,0,0,0,0,0,0,3,0,0,0,0,0,0,0,0,0,0,0,120,32,0,0,0,0,0,0,0,0,0,0,0,0,33,8,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,120,32,0,0,0,0,0,0,0,0,0,0,1,0,33,16,0,0,0,0,0,0,0,0,0,0,0,0,4,0,0,0,6,0,0,0,120,32,0,0,0,0,0,0,0,0,0,0,255,255,33,8,1,0,0,0,0,0,0,0,3,0,0,0,0,0,0,0,0,0,0,0,128,32,0,0,0,0,0,0,0,0,0,0,0,0,33,8,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,128,32,0,0,0,0,0,0,0,0,0,0,1,0,33,16,1,0,0,0,0,0,0,0,0,0,0,0,4,0,0,0,6,0,0,0,128,32,0,0,0,0,0,0,0,0,0,0,1,0,129,8,0,0,0,0,0,0,0,0,5,0,0,0,0,0,0,0,0,0,0,0,136,32,0,0,0,0,0,0,0,0,0,0,1,0,65,8,0,0,0,0,0,0,0,0,6,0,0,0,0,0,0,0,0,0,0,0,144,32,0,0,0,0,0,0,0,0,0,0,2,0,1,8,0,0,0,0,0,0,0,0,7,0,0,0,0,0,0,0,0,0,0,0,152,32,0,0,0,0,0,0,0,0,0,0,2,0,1,8,0,0,0,0,0,0,0,0,8,0,0,0,0,0,0,0,0,0,0,0,160,32,0,0,0,0,0,0,0,0,0,0,3,0,1,8,0,0,0,0,0,0,0,0,8,0,0,0,0,0,0,0,0,0,0,0,160,32,0,0,0,0,0,0,0,0,0,0,255,255,1,8,0,0,0,0,0,0,0,0,9,0,0,0,0,0,0,0,0,0,0,0,168,32,0,0,0,0,0,0,0,0,0,0,1,0,1,8,0,0,0,0,0,0,0,0,10,0,0,0,0,0,0,0,0,0,0,0,176,32,0,0,0,0,0,0,0,0,0,0,255,255,1,8,0,0,0,0,0,0,0,0,11,0,0,0,0,0,0,0,0,0,0,0,184,32,0,0,0,0,0,0,0,0,0,0,1,0,1,8,0,0,0,0,0,0,0,0,12,0,0,0,0,0,0,0,0,0,0,0,192,32,0,0,0,0,0,0,0,0,0,0,1,0,1,8,0,0,0,0,0,0,0,0,13,0,0,0,0,0,0,0,0,0,0,0,200,32,0,0,0,0,0,0,0,0,0,0,2,0,1,8,0,0,0,0,0,0,0,0,13,0,0,0,0,0,0,0,0,0,0,0,200,32,0,0,0,0,0,0,0,0,0,0,1,0,1,8,0,0,0,0,0,0,0,0,14,0,0,0,0,0,0,0,0,0,0,0,208,32,0,0,0,0,0,0,0,0,0,0,1,0,1,8,0,0,0,0,0,0,0,0,15,0,0,0,0,0,0,0,0,0,0,0,216,32,0,0,0,0,0,0,0,0,0,0,1,0,1,8,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,224,32,0,0,0,0,0,0,0,0,0,0,0,0,1,8,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,224,32,0,0,0,0,0,0,0,0,0,0,255,255,1,10,0,0,0,0,0,0,0,0,16,0,0,0,0,0,0,0,0,0,0,0,224,32,0,0,0,0,0,0,0,0,0,0,1,0,1,8,0,0,0,0,0,0,0,0,17,0,0,0,0,0,0,0,0,0,0,0,240,32,0,0,0,0,0,0,0,0,0,0,2,0,1,10,0,0,0,0,0,0,0,0,16,0,0,0,0,0,0,0,0,0,0,0,248,32,0,0,0,0,0,0,0,0,0,0,1,0,1,12,0,0,0,0,0,0,0,0,16,0,0,0,0,0,0,0,0,0,0,0,0,33,0,0,0,0,0,0,0,0,0,0,2,0,1,12,0,0,0,0,0,0,0,0,16,0,0,0,0,0,0,0,0,0,0,0,16,33,0,0,0,0,0,0,0,0,0,0,1,0,1,12,0,0,0,0,0,0,0,0,16,0,0,0,0,0,0,0,0,0,0,0,32,33,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,0,0,18,0,0,0,0,0,0,0,0,0,0,0,40,33,0,0,0,0,0,0,0,0,0,0,1,0,1,0,0,0,0,0,0,0,0,0,19,0,0,0,0,0,0,0,0,0,0,0,48,33,0,0,0,0,0,0,0,0,0,0,2,0,33,8,0,0,0,0,0,0,0,0,20,0,0,0,0,0,0,0,0,0,0,0,64,33,0,0,0,0,0,0,0,0,0,0,0,0,1,8,0,0,0,0,0,0,0,0,16,0,0,0,0,0,0,0,0,0,0,0,72,33,0,0,0,0,0,0,0,0,0,0,0,0,1,8,0,0,0,0,0,0,0,0,21,0,0,0,0,0,0,0,0,0,0,0,88,33,0,0,0,0,0,0,0,0,0,0,2,0,1,8,0,0,0,0,0,0,0,0,22,0,0,0,0,0,0,0,0,0,0,0,112,33,0,0,0,0,0,0,0,0,0,0,1,0,1,8,0,0,0,0,0,0,0,0,23,0,0,0,0,0,0,0,0,0,0,0,128,33,0,0,0,0,0,0,0,0,0,0,1,0,1,8,0,0,0,0,0,0,0,0,24,0,0,0,0,0,0,0,0,0,0,0,160,33,0,0,0,0,0,0,0,0,0,0,1,0,1,8,0,0,0,0,0,0,0,0,25,0,0,0,0,0,0,0,0,0,0,0,192,33,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,0,0,26,0,0,0,0,0,0,0,0,0,0,0,200,33,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,0,0,27,0,0,0,0,0,0,0,0,0,0,0,224,33,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,0,0,28,0,0,0,0,0,0,0,0,0,0,0,232,33,0,0,0,0,0,0,0,0,0,0,3,0,1,8,0,0,0,0,0,0,0,0,29,0,0,0,0,0,0,0,0,0,0,0,248,33,0,0,0,0,0,0,0,0,0,0,1,0,1,8,0,0,0,0,0,0,0,0,30,0,0,0,0,0,0,0,0,0,0,0,0,34,0,0,0,0,0,0,0,0,0,0,1,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,31,0,0,0,7,0,0,0,16,34,0,0,0,0,0,0,0,0,0,0,1,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,31,0,0,0,8,0,0,0,24,34,0,0,0,0,0,0,0,0,0,0,1,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,31,0,0,0,9,0,0,0,32,34,0,0,0,0,0,0,0,0,0,0,0,0,1,1,0,0,0,0,0,0,0,0,0,0,0,0,32,0,0,0,10,0,0,0,40,34,0,0,0,0,0,0,0,0,0,0,1,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,32,0,0,0,10,0,0,0,40,34,0,0,0,0,0,0,0,0,0,0,1,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,33,0,0,0,11,0,0,0,48,34,0,0,0,0,0,0,0,0,0,0,2,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,33,0,0,0,11,0,0,0,48,34,0,0,0,0,0,0,0,0,0,0,2,0,13,8,184,4,0,0,0,0,0,0,34,0,0,0,0,0,0,0,0,0,0,0,64,34,0,0,0,0,0,0,0,0,0,0,2,0,5,8,72,34,0,0,0,0,0,0,34,0,0,0,0,0,0,0,0,0,0,0,80,34,0,0,0,0,0,0,0,0,0,0,3,0,5,8,72,34,0,0,0,0,0,0,34,0,0,0,0,0,0,0,0,0,0,0,80,34,0,0,0,0,0,0,0,0,0,0,0,0,0,0,108,116,114,105,109,0,0,0,114,116,114,105,109,0,0,0,116,114,105,109,0,0,0,0,109,105,110,0,0,0,0,0,109,97,120,0,0,0,0,0,116,121,112,101,111,102,0,0,108,101,110,103,116,104,0,0,105,110,115,116,114,0,0,0,115,117,98,115,116,114,0,0,112,114,105,110,116,102,0,0,117,110,105,99,111,100,101,0,99,104,97,114,0,0,0,0,97,98,115,0,0,0,0,0,114,111,117,110,100,0,0,0,117,112,112,101,114,0,0,0,108,111,119,101,114,0,0,0,99,111,97,108,101,115,99,101,0,0,0,0,0,0,0,0,104,101,120,0,0,0,0,0,105,102,110,117,108,108,0,0,117,110,108,105,107,101,108,121,0,0,0,0,0,0,0,0,108,105,107,101,108,105,104,111,111,100,0,0,0,0,0,0,108,105,107,101,108,121,0,0,114,97,110,100,111,109,0,0,114,97,110,100,111,109,98,108,111,98,0,0,0,0,0,0,110,117,108,108,105,102,0,0,115,113,108,105,116,101,95,118,101,114,115,105,111,110,0,0,115,113,108,105,116,101,95,115,111,117,114,99,101,95,105,100,0,0,0,0,0,0,0,0,115,113,108,105,116,101,95,108,111,103,0,0,0,0,0,0,115,113,108,105,116,101,95,99,111,109,112,105,108,101,111,112,116,105,111,110,95,117,115,101,100,0,0,0,0,0,0,0,115,113,108,105,116,101,95,99,111,109,112,105,108,101,111,112,116,105,111,110,95,103,101,116,0,0,0,0,0,0,0,0,113,117,111,116,101,0,0,0,108,97,115,116,95,105,110,115,101,114,116,95,114,111,119,105,100,0,0,0,0,0,0,0,99,104,97,110,103,101,115,0,116,111,116,97,108,95,99,104,97,110,103,101,115,0,0,0,114,101,112,108,97,99,101,0,122,101,114,111,98,108,111,98,0,0,0,0,0,0,0,0,115,117,109,0,0,0,0,0,116,111,116,97,108,0,0,0,97,118,103,0,0,0,0,0,99,111,117,110,116,0,0,0,103,114,111,117,112,95,99,111,110,99,97,116,0,0,0,0,103,108,111,98,0,0,0,0,37,95,0,1,0,0,0,0,108,105,107,101,0,0,0,0,2,0,1,8,0,0,0,0,0,0,0,0,35,0,0,0,0,0,0,0,0,0,0,0,200,34,0,0,0,0,0,0,0,0,0,0,2,0,1,8,0,0,0,0,0,0,0,0,36,0,0,0,0,0,0,0,0,0,0,0,224,34,0,0,0,0,0,0,0,0,0,0,3,0,1,8,0,0,0,0,0,0,0,0,37,0,0,0,0,0,0,0,0,0,0,0,248,34,0,0,0,0,0,0,0,0,0,0,0,0,0,0,115,113,108,105,116,101,95,114,101,110,97,109,101,95,116,97,98,108,101,0,0,0,0,0,115,113,108,105,116,101,95,114,101,110,97,109,101,95,116,114,105,103,103,101,114,0,0,0,115,113,108,105,116,101,95,114,101,110,97,109,101,95,112,97,114,101,110,116,0,0,0,0,37,115,37,46,42,115,34,37,119,34,0,0,0,0,0,0,37,115,37,115,0,0,0,0,37,46,42,115,34,37,119,34,37,115,0,0,0,0,0,0,255,255,1,8,0,0,0,0,0,0,0,0,38,0,0,0,0,0,0,0,0,0,0,0,88,36,0,0,0,0,0,0,0,0,0,0,255,255,1,8,0,0,0,0,0,0,0,0,39,0,0,0,0,0,0,0,0,0,0,0,104,36,0,0,0,0,0,0,0,0,0,0,255,255,1,8,0,0,0,0,0,0,0,0,40,0,0,0,0,0,0,0,0,0,0,0,112,36,0,0,0,0,0,0,0,0,0,0,255,255,1,8,0,0,0,0,0,0,0,0,41,0,0,0,0,0,0,0,0,0,0,0,120,36,0,0,0,0,0,0,0,0,0,0,255,255,1,8,0,0,0,0,0,0,0,0,42,0,0,0,0,0,0,0,0,0,0,0,136,36,0,0,0,0,0,0,0,0,0,0,0,0,1,8,0,0,0,0,0,0,0,0,43,0,0,0,0,0,0,0,0,0,0,0,152,36,0,0,0,0,0,0,0,0,0,0,0,0,1,8,0,0,0,0,0,0,0,0,44,0,0,0,0,0,0,0,0,0,0,0,168,36,0,0,0,0,0,0,0,0,0,0,0,0,1,8,0,0,0,0,0,0,0,0,45,0,0,0,0,0,0,0,0,0,0,0,192,36,0,0,0,0,0,0,0,0,0,0,106,117,108,105,97,110,100,97,121,0,0,0,0,0,0,0,100,97,116,101,0,0,0,0,116,105,109,101,0,0,0,0,100,97,116,101,116,105,109,101,0,0,0,0,0,0,0,0,115,116,114,102,116,105,109,101,0,0,0,0,0,0,0,0,99,117,114,114,101,110,116,95,116,105,109,101,0,0,0,0,99,117,114,114,101,110,116,95,116,105,109,101,115,116,97,109,112,0,0,0,0,0,0,0,99,117,114,114,101,110,116,95,100,97,116,101,0,0,0,0,37,48,50,100,0,0,0,0,37,48,54,46,51,102,0,0,37,48,51,100,0,0,0,0,37,46,49,54,103,0,0,0,37,108,108,100,0,0,0,0,37,48,52,100,0,0,0,0,108,111,99,97,108,116,105,109,101,0,0,0,0,0,0,0,117,110,105,120,101,112,111,99,104,0,0,0,0,0,0,0,117,116,99,0,0,0,0,0,119,101,101,107,100,97,121,32,0,0,0,0,0,0,0,0,115,116,97,114,116,32,111,102,32,0,0,0,0,0,0,0,109,111,110,116,104,0,0,0,121,101,97,114,0,0,0,0,100,97,121,0,0,0,0,0,104,111,117,114,0,0,0,0,109,105,110,117,116,101,0,0,115,101,99,111,110,100,0,0,108,111,99,97,108,32,116,105,109,101,32,117,110,97,118,97,105,108,97,98,108,101,0,0,110,111,119,0,0,0,0,0,37,48,52,100,45,37,48,50,100,45,37,48,50,100,32,37,48,50,100,58,37,48,50,100,58,37,48,50,100,0,0,0,37,48,50,100,58,37,48,50,100,58,37,48,50,100,0,0,37,48,52,100,45,37,48,50,100,45,37,48,50,100,0,0,76,73,75,69,32,111,114,32,71,76,79,66,32,112,97,116,116,101,114,110,32,116,111,111,32,99,111,109,112,108,101,120,0,0,0,0,0,0,0,0,69,83,67,65,80,69,32,101,120,112,114,101,115,115,105,111,110,32,109,117,115,116,32,98,101,32,97,32,115,105,110,103,108,101,32,99,104,97,114,97,99,116,101,114,0,0,0,0,44,0,0,0,0,0,0,0,105,110,116,101,103,101,114,32,111,118,101,114,102,108,111,119,0,0,0,0,0,0,0,0,37,33,46,49,53,103,0,0,37,33,46,50,48,101,0,0,48,49,50,51,52,53,54,55,56,57,65,66,67,68,69,70,78,85,76,76,0,0,0,0,37,46,42,102,0,0,0,0,105,110,116,101,103,101,114,0,116,101,120,116,0,0,0,0,114,101,97,108,0,0,0,0,98,108,111,98,0,0,0,0,110,117,108,108,0,0,0,0,1,0,0,0,0,0,0,0,184,38,0,0,0,0,0,0,32,0,0,0,0,0,0,0,105,110,116,101,114,114,117,112,116,0,0,0,0,0,0,0,117,110,114,101,99,111,103,110,105,122,101,100,32,116,111,107,101,110,58,32,34,37,84,34,0,0,0,0,0,0,0,0,110,101,97,114,32,34,37,84,34,58,32,115,121,110,116,97,120,32,101,114,114,111,114,0,114,111,119,105,100,0,0,0,117,110,107,110,111,119,110,32,116,97,98,108,101,32,111,112,116,105,111,110,58,32,37,46,42,115,0,0,0,0,0,0,9,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,116,111,111,32,109,97,110,121,32,116,101,114,109,115,32,105,110,32,99,111,109,112,111,117,110,100,32,83,69,76,69,67,84,0,0,0,0,0,0,0,115,101,116,32,108,105,115,116,0,0,0,0,0,0,0,0,116,111,111,32,109,97,110,121,32,97,114,103,117,109,101,110,116,115,32,111,110,32,102,117,110,99,116,105,111,110,32,37,84,0,0,0,0,0,0,0,56,91,0,0,1,0,0,0,48,73,0,0,1,0,0,0,105,110,100,101,120,0,0,0,113,117,97,108,105,102,105,101,100,32,116,97,98,108,101,32,110,97,109,101,115,32,97,114,101,32,110,111,116,32,97,108,108,111,119,101,100,32,111,110,32,73,78,83,69,82,84,44,32,85,80,68,65,84,69,44,32,97,110,100,32,68,69,76,69,84,69,32,115,116,97,116], "i8", ALLOC_NONE, Runtime.GLOBAL_BASE);
/* memory initializer */ allocate([101,109,101,110,116,115,32,119,105,116,104,105,110,32,116,114,105,103,103,101,114,115,0,0,116,104,101,32,73,78,68,69,88,69,68,32,66,89,32,99,108,97,117,115,101,32,105,115,32,110,111,116,32,97,108,108,111,119,101,100,32,111,110,32,85,80,68,65,84,69,32,111,114,32,68,69,76,69,84,69,32,115,116,97,116,101,109,101,110,116,115,32,119,105,116,104,105,110,32,116,114,105,103,103,101,114,115,0,0,0,0,0,116,104,101,32,78,79,84,32,73,78,68,69,88,69,68,32,99,108,97,117,115,101,32,105,115,32,110,111,116,32,97,108,108,111,119,101,100,32,111,110,32,85,80,68,65,84,69,32,111,114,32,68,69,76,69,84,69,32,115,116,97,116,101,109,101,110,116,115,32,119,105,116,104,105,110,32,116,114,105,103,103,101,114,115,0,0,0,0,144,1,145,2,145,1,146,1,146,3,147,0,147,1,147,3,148,1,149,3,151,0,151,1,151,2,150,0,150,1,150,1,150,1,149,2,149,2,149,2,153,1,153,0,149,2,149,3,149,5,149,2,154,6,156,1,158,0,158,3,157,1,157,0,155,5,155,2,162,0,162,2,160,3,160,1,164,3,165,1,152,1,152,1,152,1,166,0,166,1,168,1,168,4,168,6,169,1,169,2,170,1,170,1,167,2,167,0,173,2,173,2,173,4,173,3,173,3,173,2,173,2,173,3,173,5,173,2,173,4,173,4,173,1,173,2,178,0,178,1,180,0,180,2,182,2,182,3,182,3,182,3,183,2,183,2,183,1,183,1,183,2,181,3,181,2,184,0,184,2,184,2,161,0,161,2,185,3,185,1,186,1,186,0,187,2,187,7,187,5,187,5,187,10,189,0,189,1,176,0,176,3,190,0,190,2,191,1,191,1,191,1,149,4,193,2,193,0,149,8,149,4,149,1,163,2,195,1,195,3,198,1,198,2,198,1,196,9,196,1,207,4,207,5,199,1,199,1,199,0,210,2,210,0,200,3,200,2,200,4,211,2,211,1,211,0,201,0,201,2,213,2,213,0,212,7,212,7,212,7,159,0,159,2,194,2,214,1,214,2,214,3,214,4,216,2,216,0,215,0,215,3,215,2,217,4,217,0,205,0,205,3,220,4,220,2,177,1,177,1,177,0,203,0,203,3,204,0,204,2,206,0,206,2,206,4,206,4,149,6,202,0,202,2,149,8,221,5,221,3,149,6,149,7,222,2,222,1,223,0,223,3,219,3,219,1,175,1,175,3,174,1,175,1,175,1,175,3,175,5,174,1,174,1,175,1,175,3,175,6,175,5,175,4,174,1,175,3,175,3,175,3,175,3,175,3,175,3,175,3,175,3,224,1,224,2,175,3,175,5,175,2,175,3,175,3,175,4,175,2,175,2,175,2,175,2,225,1,225,2,175,5,226,1,226,2,175,5,175,3,175,5,175,4,175,4,175,5,228,5,228,4,229,2,229,0,227,1,227,0,209,1,209,0,208,3,208,1,149,12,230,1,230,0,179,0,179,3,188,5,188,3,231,0,231,2,149,4,149,1,149,2,149,3,149,5,149,6,149,5,149,6,232,1,232,1,232,1,232,1,232,1,171,2,171,1,172,2,149,5,233,11,235,1,235,1,235,2,235,0,236,1,236,1,236,3,237,0,237,3,238,0,238,2,234,3,234,2,240,1,240,3,241,0,241,3,241,2,239,7,239,5,239,5,239,1,175,4,175,6,192,1,192,1,192,1,149,4,149,6,149,3,243,0,243,2,242,1,242,0,149,1,149,3,149,1,149,3,149,6,149,6,244,1,245,0,245,1,149,1,149,4,246,8,247,1,247,3,248,0,248,2,249,1,249,3,250,1,251,0,251,4,251,2,197,0,197,2,197,3,252,6,252,8,0,0,238,0,186,3,213,0,33,1,54,1,234,0,144,0,61,1,147,255,126,1,121,1,47,1,205,1,133,1,122,1,112,1,46,1,38,1,253,0,139,1,37,1,68,1,147,1,147,1,147,1,147,1,147,1,147,1,147,1,147,1,147,1,147,1,147,1,147,1,147,1,147,1,147,1,147,1,147,1,147,1,147,1,147,1,147,1,147,1,147,1,147,1,147,1,147,1,147,1,147,1,147,1,254,3,244,3,237,3,230,3,195,3,193,3,191,3,189,3,182,3,179,3,162,3,144,3,105,3,93,3,55,3,42,3,3,3,247,2,208,2,196,2,158,2,145,2,107,2,102,2,100,2,98,2,96,2,94,2,92,2,86,2,83,2,81,2,68,2,30,2,28,2,25,2,23,2,21,2,19,2,17,2,15,2,247,1,130,1,147,1,147,1,147,1,147,1,147,1,147,1,147,1,147,1,147,1,95,0,191,1,82,0,78,1,248,1,211,1,147,1,221,1,208,1,147,1,147,1,147,1,147,1,92,3,235,2,232,2,17,3,126,2,126,2,158,3,123,3,132,3,131,3,119,3,76,3,72,3,67,3,80,3,62,3,75,3,61,3,24,3,71,3,58,3,225,2,70,3,27,3,21,3,47,0,222,2,18,2,184,2,9,3,199,2,165,2,221,2,218,2,217,2,216,2,215,2,115,2,192,1,64,0,187,0,25,5,22,5,228,4,10,5,249,4,43,5,42,5,41,5,39,5,38,5,36,5,35,5,34,5,33,5,32,5,31,5,30,5,28,5,27,5,24,5,23,5,21,5,18,5,14,5,12,5,9,5,242,4,240,4,235,4,8,5,7,5,214,4,5,5,1,5,0,5,255,4,254,4,227,4,253,4,252,4,251,4,249,4,244,4,243,4,241,4,239,4,237,4,233,4,224,4,213,4,223,4,222,4,219,4,214,4,216,4,211,4,225,4,210,4,209,4,206,4,196,4,190,4,186,4,201,4,195,4,208,4,207,4,173,4,171,4,203,4,200,4,177,4,184,4,218,4,113,4,212,4,205,4,169,4,157,4,197,4,153,4,172,4,155,4,167,4,166,4,162,4,158,4,194,4,192,4,152,4,138,4,159,4,156,4,136,4,175,4,179,4,109,4,71,4,174,4,170,4,164,4,168,4,147,4,145,4,144,4,149,4,150,4,142,4,135,4,117,4,146,4,134,4,143,4,133,4,108,4,121,4,119,4,100,4,104,4,79,4,78,4,76,4,72,4,126,4,125,4,61,4,101,4,56,4,40,4,96,4,73,4,58,4,54,4,49,4,43,4,85,4,83,4,95,4,93,4,92,4,89,4,87,4,84,4,239,3,232,3,234,3,52,4,51,4,57,4,0,0,0,0,50,1,212,0,176,1,187,3,127,2,191,0,187,3,39,1,47,2,88,0,88,0,88,0,88,0,81,0,86,0,86,0,86,0,86,0,85,0,85,0,84,0,84,0,84,0,83,0,74,1,185,0,184,0,183,0,123,2,123,2,36,1,94,2,94,2,88,0,88,0,88,0,88,0,171,2,86,0,86,0,86,0,86,0,85,0,85,0,84,0,84,0,84,0,83,0,74,1,16,0,180,1,85,2,89,0,90,0,80,0,88,2,87,2,89,2,89,2,87,0,87,0,88,0,88,0,88,0,88,0,172,2,86,0,86,0,86,0,86,0,85,0,85,0,84,0,84,0,84,0,83,0,74,1,50,1,47,2,84,0,84,0,84,0,83,0,74,1,65,0,86,0,86,0,86,0,86,0,85,0,85,0,84,0,84,0,84,0,83,0,74,1,123,2,123,2,122,2,121,2,182,0,170,2,38,2,123,1,120,1,119,1,17,0,66,1,94,2,94,2,115,1,198,0,223,1,91,0,118,1,82,0,79,0,165,0,85,0,85,0,84,0,84,0,84,0,83,0,74,1,86,2,123,2,123,2,107,0,89,0,90,0,80,0,88,2,87,2,89,2,89,2,87,0,87,0,88,0,88,0,88,0,88,0,186,0,86,0,86,0,86,0,86,0,85,0,85,0,84,0,84,0,84,0,83,0,74,1,50,1,82,2,82,2,142,0,72,1,71,1,228,1,249,0,88,1,238,0,123,2,123,2,122,2,121,2,73,2,192,1,14,2,13,2,229,0,132,1,1,0,138,1,194,1,72,2,193,1,123,2,123,2,123,2,123,2,63,1,139,1,94,2,94,2,199,0,157,0,17,1,126,1,12,1,125,1,187,0,123,2,123,2,122,2,121,2,55,1,43,2,10,1,81,2,81,2,10,1,91,1,76,2,89,0,90,0,80,0,88,2,87,2,89,2,89,2,87,0,87,0,88,0,88,0,88,0,88,0,222,1,86,0,86,0,86,0,86,0,85,0,85,0,84,0,84,0,84,0,83,0,74,1,50,1,16,1,24,2,122,2,121,2,146,0,98,2,197,0,54,1,63,2,182,0,226,1,15,1,123,1,120,1,119,1,250,1,21,0,122,2,121,2,122,2,121,2,123,2,123,2,118,1,99,2,62,2,36,2,184,1,111,0,51,2,94,2,94,2,122,2,121,2,68,1,223,1,96,2,96,2,96,2,44,1,179,1,61,2,119,0,151,1,210,0,162,0,50,2,115,3,80,2,80,2,50,1,89,0,90,0,80,0,88,2,87,2,89,2,89,2,87,0,87,0,88,0,88,0,88,0,88,0,250,1,86,0,86,0,86,0,86,0,85,0,85,0,84,0,84,0,84,0,83,0,74,1,108,2,111,0,123,2,123,2,105,1,94,2,94,2,102,1,249,0,93,1,248,0,177,1,243,0,223,1,74,2,122,2,121,2,195,0,99,2,93,0,119,0,221,0,63,2,241,1,22,2,22,2,89,0,90,0,80,0,88,2,87,2,89,2,89,2,87,0,87,0,88,0,88,0,88,0,88,0,62,2,86,0,86,0,86,0,86,0,85,0,85,0,84,0,84,0,84,0,83,0,74,1,50,1,77,0,173,1,126,2,61,2,77,2,18,2,240,0,230,0,242,0,105,0,249,0,93,1,248,0,3,2,76,2,208,0,204,1,17,2,52,2,173,0,122,2,121,2,202,3,144,0,174,1,2,0,168,1,228,0,124,1,45,2,94,2,94,2,190,0,153,0,159,0,158,0,2,2,51,0,120,2,119,2,118,2,71,0,24,2,176,1,186,3,196,0,98,2,186,3,102,2,45,0,18,0,89,0,90,0,80,0,88,2,87,2,89,2,89,2,87,0,87,0,88,0,88,0,88,0,88,0,5,1,86,0,86,0,86,0,86,0,85,0,85,0,84,0,84,0,84,0,83,0,74,1,50,1,96,2,96,2,96,2,30,2,168,1,146,1,129,1,241,0,250,1,195,1,64,1,211,0,31,2,164,0,180,1,130,1,37,1,195,1,75,2,108,0,240,1,111,0,78,1,135,1,79,2,168,1,102,2,27,0,196,1,197,1,94,2,94,2,72,0,1,1,70,0,3,1,196,1,83,1,86,1,52,2,70,2,68,0,159,1,213,1,72,1,71,1,62,0,102,2,45,0,110,0,137,1,89,0,90,0,80,0,88,2,87,2,89,2,89,2,87,0,87,0,88,0,88,0,88,0,88,0,152,0,86,0,86,0,86,0,86,0,85,0,85,0,84,0,84,0,84,0,83,0,74,1,50,1,110,0,243,1,8,2,26,2,146,1,133,1,168,1,110,0,54,2,244,1,81,2,81,2,198,1,82,0,79,0,165,0,168,1,79,2,128,1,52,2,84,1,103,2,188,0,162,0,168,1,94,1,104,2,168,1,102,2,44,0,94,2,94,2,189,1,70,2,44,1,178,1,151,0,19,0,102,2,9,0,56,2,68,2,92,1,103,2,213,1,55,2,102,2,26,0,104,2,102,2,45,0,89,0,90,0,80,0,88,2,87,2,89,2,89,2,87,0,87,0,88,0,88,0,88,0,88,0,155,1,86,0,86,0,86,0,86,0,85,0,85,0,84,0,84,0,84,0,83,0,74,1,50,1,67,2,110,0,66,2,9,2,26,1,177,1,142,1,144,1,255,0,230,1,82,0,79,0,165,0,231,1,164,0,82,0,79,0,165,0,232,1,232,1,108,1,131,1,168,1,32,2,32,2,253,1,94,1,106,1,155,0,191,0,94,2,94,2,47,2,130,2,128,2,77,1,82,0,79,0,165,0,49,1,52,2,251,1,56,1,101,1,102,2,45,0,73,1,84,2,83,2,194,0,81,1,89,0,90,0,80,0,88,2,87,2,89,2,89,2,87,0,87,0,88,0,88,0,88,0,88,0,168,1,86,0,86,0,86,0,86,0,85,0,85,0,84,0,84,0,84,0,83,0,74,1,50,1,20,0,67,1,150,0,7,1,211,0,31,2,165,1,84,2,83,2,102,2,22,0,168,1,193,0,168,1,28,1,168,1,135,1,168,1,253,1,168,1,65,2,168,1,186,0,79,1,168,1,47,2,168,1,57,1,120,0,34,2,94,2,94,2,67,0,102,2,47,0,102,2,50,0,102,2,48,0,102,2,100,0,102,2,99,0,102,2,101,0,64,2,102,2,102,0,102,2,109,0,70,1,89,0,90,0,80,0,88,2,87,2,89,2,89,2,87,0,87,0,88,0,88,0,88,0,88,0,168,1,86,0,86,0,86,0,86,0,85,0,85,0,84,0,84,0,84,0,83,0,74,1,50,1,168,1,55,1,168,1,73,2,54,0,168,1,4,2,5,2,78,2,102,2,112,0,168,1,72,2,168,1,60,2,168,1,195,0,168,1,59,2,168,1,67,0,168,1,102,2,94,0,102,2,98,0,168,1,102,2,97,0,8,1,94,2,94,2,195,0,102,2,46,0,102,2,96,0,102,2,30,0,102,2,49,0,102,2,115,0,102,2,114,0,162,1,229,0,132,1,102,2,113,0,50,1,89,0,90,0,80,0,88,2,87,2,89,2,89,2,87,0,87,0,88,0,88,0,88,0,88,0,168,1,86,0,86,0,86,0,86,0,85,0,85,0,84,0,84,0,84,0,83,0,74,1,119,0,168,1,78,2,110,0,116,1,94,2,94,2,195,0,53,0,250,0,102,2,29,0,195,0,216,1,182,1,217,2,190,0,46,1,242,1,14,0,11,2,129,2,2,0,102,2,43,0,50,1,89,0,90,0,80,0,88,2,87,2,89,2,89,2,87,0,87,0,88,0,88,0,88,0,88,0,168,1,86,0,86,0,86,0,86,0,85,0,85,0,84,0,84,0,84,0,83,0,74,1,168,1,101,2,196,3,196,3,98,1,94,2,94,2,164,1,56,1,64,0,102,2,42,0,135,1,99,1,27,1,181,1,45,1,255,0,158,1,154,1,239,1,236,1,102,2,28,0,215,1,50,1,89,0,90,0,80,0,88,2,87,2,89,2,89,2,87,0,87,0,88,0,88,0,88,0,88,0,168,1,86,0,86,0,86,0,86,0,85,0,85,0,84,0,84,0,84,0,83,0,74,1,168,1,110,0,110,0,110,0,110,0,94,2,94,2,110,0,254,0,13,0,102,2,41,0,20,2,19,2,27,1,225,1,19,2,201,1,28,1,119,0,49,2,100,1,102,2,40,0,28,1,50,1,89,0,78,0,80,0,88,2,87,2,89,2,89,2,87,0,87,0,88,0,88,0,88,0,88,0,168,1,86,0,86,0,86,0,86,0,85,0,85,0,84,0,84,0,84,0,83,0,74,1,110,0,168,1,85,1,220,0,43,2,94,2,94,2,95,1,43,2,62,1,102,2,95,0,157,1,255,0,83,0,74,1,28,1,28,1,255,0,128,2,77,1,100,1,255,0,102,2,39,0,50,1,100,1,90,0,80,0,88,2,87,2,89,2,89,2,87,0,87,0,88,0,88,0,88,0,88,0,168,1,86,0,86,0,86,0,86,0,85,0,85,0,84,0,84,0,84,0,83,0,74,1,168,1,61,1,60,1,141,0,209,1,94,2,94,2,219,0,107,2,207,1,102,2,10,0,161,1,206,1,255,0,189,0,254,1,41,2,95,1,207,0,107,1,161,0,102,2,38,0,59,1,218,0,255,0,255,0,80,0,88,2,87,2,89,2,89,2,87,0,87,0,88,0,88,0,88,0,88,0,168,1,86,0,86,0,86,0,86,0,85,0,85,0,84,0,84,0,84,0,83,0,74,1,76,0,163,1,255,0,3,0,110,3,205,1,168,1,247,0,75,1,75,1,102,2,37,0,217,0,76,0,163,1,134,1,3,0,216,0,215,0,166,1,4,0,75,1,75,1,168,1,35,2,12,0,168,1,33,2,102,2,36,0,168,1,29,2,166,1,168,1,28,2,168,1,214,0,168,1,152,1,168,1,27,2,147,1,93,2,93,2,237,0,102,2,25,0,119,0,102,2,24,0,76,2,152,1,102,2,45,0,118,0,102,2,35,0,102,2,34,0,102,2,33,0,102,2,23,0,76,2,60,0,223,0,91,2,90,2,1,2,122,1,73,0,74,0,140,0,139,0,168,1,110,0,9,1,75,0,170,1,169,1,59,0,168,1,98,2,73,0,74,0,37,2,146,1,148,1,168,1,117,1,75,0,170,1,169,1,92,2,138,0,98,2,102,2,11,0,136,1,76,0,163,1,181,0,3,0,102,2,32,0,15,1,113,1,75,1,75,1,237,1,102,2,31,0,149,0,96,2,96,2,96,2,95,2,15,0,166,1,109,1,102,2,8,0,137,0,233,1,136,0,190,0,96,2,96,2,96,2,95,2,15,0,229,1,176,0,135,0,7,0,252,0,221,1,152,1,174,0,133,0,175,0,218,1,57,0,56,0,132,0,130,0,119,0,76,0,163,1,76,2,3,0,212,1,245,0,208,1,171,0,75,1,75,1,125,0,123,0,200,1,191,1,122,0,190,1,104,0,80,1,231,0,166,1,166,0,154,0,73,0,74,0,76,1,116,0,175,1,121,0,53,1,75,0,170,1,169,1,222,0,106,0,98,2,52,1,125,2,204,0,152,1,117,2,115,2,116,2,6,0,200,0,172,1,171,1,34,1,203,0,110,2,201,0,76,2,62,0,63,0,33,1,66,0,163,1,143,1,3,0,145,1,32,1,92,0,143,0,75,1,75,1,31,1,96,2,96,2,96,2,95,2,15,0,73,0,74,0,227,0,166,1,69,1,69,0,160,1,75,0,170,1,169,1,100,2,156,1,98,2,192,0,61,0,57,2,209,0,140,1,226,0,22,1,225,0,127,1,152,1,15,2,46,2,20,1,21,2,40,2,16,2,65,1,11,2,114,1,252,1,180,0,76,2,238,1,179,0,110,1,117,0,253,0,13,1,10,2,247,1,96,2,96,2,96,2,95,2,15,0,39,2,246,1,58,0,18,1,12,2,178,0,73,0,74,0,48,1,245,1,112,1,47,1,206,0,75,0,170,1,169,1,235,1,104,1,98,2,213,0,177,0,227,1,131,0,89,1,42,1,41,1,40,1,202,0,38,1,224,1,234,1,210,1,134,0,172,0,129,0,188,1,90,1,214,1,128,0,58,1,203,1,103,0,127,0,126,0,148,0,124,0,167,0,187,1,235,0,96,2,96,2,96,2,95,2,15,0,186,1,183,1,111,2,234,0,43,1,145,0,71,2,35,1,121,1,69,2,160,0,119,0,156,0,14,1,124,2,203,3,169,0,23,1,114,2,8,2,113,2,217,1,112,2,170,0,109,2,106,2,119,0,168,0,55,0,153,1,167,1,25,2,97,2,30,1,29,1,149,1,58,2,48,2,44,2,5,0,52,0,202,1,42,2,147,0,11,1,7,2,248,1,6,2,150,1,6,1,239,0,4,1,0,2,87,1,255,1,2,1,97,1,53,2,0,1,224,0,251,0,103,1,21,1,19,1,220,1,219,1,246,0,96,1,244,0,211,1,199,1,236,0,233,0,232,0,51,1,185,1,25,1,205,0,163,0,141,1,24,1,23,2,249,1,74,1,105,2,203,3,203,3,203,3,203,3,111,1,203,3,203,3,203,3,203,3,203,3,203,3,203,3,203,3,203,3,203,3,203,3,82,1,0,0,0,0,0,0,100,117,112,108,105,99,97,116,101,32,87,73,84,72,32,116,97,98,108,101,32,110,97,109,101,58,32,37,115,0,0,0,110,111,116,32,97,117,116,104,111,114,105,122,101,100,0,0,97,117,116,104,111,114,105,122,101,114,32,109,97,108,102,117,110,99,116,105,111,110,0,0,67,82,69,65,84,69,32,86,73,82,84,85,65,76,32,84,65,66,76,69,32,37,84,0,85,80,68,65,84,69,32,37,81,46,37,115,32,83,69,84,32,116,121,112,101,61,39,116,97,98,108,101,39,44,32,110,97,109,101,61,37,81,44,32,116,98,108,95,110,97,109,101,61,37,81,44,32,114,111,111,116,112,97,103,101,61,48,44,32,115,113,108,61,37,81,32,87,72,69,82,69,32,114,111,119,105,100,61,35,37,100,0,115,113,108,105,116,101,95,116,101,109,112,95,109,97,115,116,101,114,0,0,0,0,0,0,115,113,108,105,116,101,95,109,97,115,116,101,114,0,0,0,110,97,109,101,61,39,37,113,39,32,65,78,68,32,116,121,112,101,61,39,116,97,98,108,101,39,0,0,0,0,0,0,118,105,114,116,117,97,108,32,116,97,98,108,101,115,32,109,97,121,32,110,111,116,32,98,101,32,97,108,116,101,114,101,100,0,0,0,0,0,0,0,67,97,110,110,111,116,32,97,100,100,32,97,32,99,111,108,117,109,110,32,116,111,32,97,32,118,105,101,119,0,0,0,115,113,108,105,116,101,95,97,108,116,101,114,116,97,98,95,37,115,0,0,0,0,0,0,117,110,97,98,108,101,32,116,111,32,111,112,101,110,32,97,32,116,101,109,112,111,114,97,114,121,32,100,97,116,97,98,97,115,101,32,102,105,108,101,32,102,111,114,32,115,116,111,114,105,110,103,32,116,101,109,112,111,114,97,114,121,32,116,97,98,108,101,115,0,0,0,115,113,108,105,116,101,95,0,116,97,98,108,101,32,37,115,32,109,97,121,32,110,111,116,32,98,101,32,97,108,116,101,114,101,100,0,0,0,0,0,67,97,110,110,111,116,32,97,100,100,32,97,32,80,82,73,77,65,82,89,32,75,69,89,32,99,111,108,117,109,110,0,67,97,110,110,111,116,32,97,100,100,32,97,32,85,78,73,81,85,69,32,99,111,108,117,109,110,0,0,0,0,0,0,67,97,110,110,111,116,32,97,100,100,32,97,32,82,69,70,69,82,69,78,67,69,83,32,99,111,108,117,109,110,32,119,105,116,104,32,110,111,110,45,78,85,76,76,32,100,101,102,97,117,108,116,32,118,97,108,117,101,0,0,0,0,0,0,67,97,110,110,111,116,32,97,100,100,32,97,32,78,79,84,32,78,85,76,76,32,99,111,108,117,109,110,32,119,105,116,104,32,100,101,102,97,117,108,116,32,118,97,108,117,101,32,78,85,76,76,0,0,0,0,67,97,110,110,111,116,32,97,100,100,32,97,32,99,111,108,117,109,110,32,119,105,116,104,32,110,111,110,45,99,111,110,115,116,97,110,116,32,100,101,102,97,117,108,116,0,0,0,85,80,68,65,84,69,32,34,37,119,34,46,37,115,32,83,69,84,32,115,113,108,32,61,32,115,117,98,115,116,114,40,115,113,108,44,49,44,37,100,41,32,124,124,32,39,44,32,39,32,124,124,32,37,81,32,124,124,32,115,117,98,115,116,114,40,115,113,108,44,37,100,41,32,87,72,69,82,69,32,116,121,112,101,32,61,32,39,116,97,98,108,101,39,32,65,78,68,32,110,97,109,101,32,61,32,37,81,0,0,0,0,116,98,108,95,110,97,109,101,61,37,81,0,0,0,0,0,116,121,112,101,61,39,116,114,105,103,103,101,114,39,32,65,78,68,32,40,37,115,41,0,110,97,109,101,61,37,81,0,37,115,32,79,82,32,110,97,109,101,61,37,81,0,0,0,45,0,0,0,0,0,0,0,116,104,101,114,101,32,105,115,32,97,108,114,101,97,100,121,32,97,110,111,116,104,101,114,32,116,97,98,108,101,32,111,114,32,105,110,100,101,120,32,119,105,116,104,32,116,104,105,115,32,110,97,109,101,58,32,37,115,0,0,0,0,0,0,118,105,101,119,32,37,115,32,109,97,121,32,110,111,116,32,98,101,32,97,108,116,101,114,101,100,0,0,0,0,0,0,85,80,68,65,84,69,32,34,37,119,34,46,37,115,32,83,69,84,32,115,113,108,32,61,32,115,113,108,105,116,101,95,114,101,110,97,109,101,95,112,97,114,101,110,116,40,115,113,108,44,32,37,81,44,32,37,81,41,32,87,72,69,82,69,32,37,115,59,0,0,0,0,85,80,68,65,84,69,32,37,81,46,37,115,32,83,69,84,32,115,113,108,32,61,32,67,65,83,69,32,87,72,69,78,32,116,121,112,101,32,61,32,39,116,114,105,103,103,101,114,39,32,84,72,69,78,32,115,113,108,105,116,101,95,114,101,110,97,109,101,95,116,114,105,103,103,101,114,40,115,113,108,44,32,37,81,41,69,76,83,69,32,115,113,108,105,116,101,95,114,101,110,97,109,101,95,116,97,98,108,101,40,115,113,108,44,32,37,81,41,32,69,78,68,44,32,116,98,108,95,110,97,109,101,32,61,32,37,81,44,32,110,97,109,101,32,61,32,67,65,83,69,32,87,72,69,78,32,116,121,112,101,61,39,116,97,98,108,101,39,32,84,72,69,78,32,37,81,32,87,72,69,78,32,110,97,109,101,32,76,73,75,69,32,39,115,113,108,105,116,101,95,97,117,116,111,105,110,100,101,120,37,37,39,32,65,78,68,32,116,121,112,101,61,39,105,110,100,101,120,39,32,84,72,69,78,32,39,115,113,108,105,116,101,95,97,117,116,111,105,110,100,101,120,95,39,32,124,124,32,37,81,32,124,124,32,115,117,98,115,116,114,40,110,97,109,101,44,37,100,43,49,56,41,32,69,76,83,69,32,110,97,109,101,32,69,78,68,32,87,72,69,82,69,32,116,98,108,95,110,97,109,101,61,37,81,32,67,79,76,76,65,84,69,32,110,111,99,97,115,101,32,65,78,68,32,40,116,121,112,101,61,39,116,97,98,108,101,39,32,79,82,32,116,121,112,101,61,39,105,110,100,101,120,39,32,79,82,32,116,121,112,101,61,39,116,114,105,103,103,101,114,39,41,59,0,115,113,108,105,116,101,95,115,101,113,117,101,110,99,101,0,85,80,68,65,84,69,32,34,37,119,34,46,115,113,108,105,116,101,95,115,101,113,117,101,110,99,101,32,115,101,116,32,110,97,109,101,32,61,32,37,81,32,87,72,69,82,69,32,110,97,109,101,32,61,32,37,81,0,0,0,0,0,0,0,85,80,68,65,84,69,32,115,113,108,105,116,101,95,116,101,109,112,95,109,97,115,116,101,114,32,83,69,84,32,115,113,108,32,61,32,115,113,108,105,116,101,95,114,101,110,97,109,101,95,116,114,105,103,103,101,114,40,115,113,108,44,32,37,81,41,44,32,116,98,108,95,110,97,109,101,32,61,32,37,81,32,87,72,69,82,69,32,37,115,59,0,0,0,0,0,118,105,101,119,32,37,115,32,105,115,32,99,105,114,99,117,108,97,114,108,121,32,100,101,102,105,110,101,100,0,0,0,0,0,2,0,3,0,5,0,6,0,7,0,8,0,9,0,110,111,32,115,117,99,104,32,99,111,108,108,97,116,105,111,110,32,115,101,113,117,101,110,99,101,58,32,37,115,0,0,3,2,1,0,0,0,0,0,73,78,84,69,71,69,82,0,37,115,58,37,100,0,0,0,97,32,71,82,79,85,80,32,66,89,32,99,108,97,117,115,101,32,105,115,32,114,101,113,117,105,114,101,100,32,98,101,102,111,114,101,32,72,65,86,73,78,71,0,0,0,0,0,79,82,68,69,82,0,0,0,71,82,79,85,80,0,0,0,97,103,103,114,101,103,97,116,101,32,102,117,110,99,116,105,111,110,115,32,97,114,101,32,110,111,116,32,97,108,108,111,119,101,100,32,105,110,32,116,104,101,32,71,82,79,85,80,32,66,89,32,99,108,97,117,115,101,0,0,0,0,0,0,116,111,111,32,109,97,110,121,32,116,101,114,109,115,32,105,110,32,79,82,68,69,82,32,66,89,32,99,108,97,117,115,101,0,0,0,0,0,0,0,37,114,32,79,82,68,69,82,32,66,89,32,116,101,114,109,32,100,111,101,115,32,110,111,116,32,109,97,116,99,104,32,97,110,121,32,99,111,108,117,109,110,32,105,110,32,116,104,101,32,114,101,115,117,108,116,32,115,101,116,0,0,0,0,37,114,32,37,115,32,66,89,32,116,101,114,109,32,111,117,116,32,111,102,32,114,97,110,103,101,32,45,32,115,104,111,117,108,100,32,98,101,32,98,101,116,119,101,101,110,32,49,32,97,110,100,32,37,100,0,116,111,111,32,109,97,110,121,32,116,101,114,109,115,32,105,110,32,37,115,32,66,89,32,99,108,97,117,115,101,0,0,69,120,112,114,101,115,115,105,111,110,32,116,114,101,101,32,105,115,32,116,111,111,32,108,97,114,103,101,32,40,109,97,120,105,109,117,109,32,100,101,112,116,104,32,37,100,41,0,102,117,110,99,116,105,111,110,115,0,0,0,0,0,0,0,115,101,99,111,110,100,32,97,114,103,117,109,101,110,116,32,116,111,32,108,105,107,101,108,105,104,111,111,100,40,41,32,109,117,115,116,32,98,101,32,97,32,99,111,110,115,116,97,110,116,32,98,101,116,119,101,101,110,32,48,46,48,32,97,110,100,32,49,46,48,0,0,110,111,116,32,97,117,116,104,111,114,105,122,101,100,32,116,111,32,117,115,101,32,102,117,110,99,116,105,111,110,58,32,37,115,0,0,0,0,0,0,109,105,115,117,115,101,32,111,102,32,97,103,103,114,101,103,97,116,101,32,102,117,110,99,116,105,111,110,32,37,46,42,115,40,41,0,0,0,0,0,110,111,32,115,117,99,104,32,102,117,110,99,116,105,111,110,58,32,37,46,42,115,0,0,119,114,111,110,103,32,110,117,109,98,101,114,32,111,102,32,97,114,103,117,109,101,110,116,115,32,116,111,32,102,117,110,99,116,105,111,110,32,37,46,42,115,40,41,0,0,0,0,115,117,98,113,117,101,114,105,101,115,0,0,0,0,0,0,112,97,114,97,109,101,116,101,114,115,0,0,0,0,0,0,37,115,32,112,114,111,104,105,98,105,116,101,100,32,105,110,32,67,72,69,67,75,32,99,111,110,115,116,114,97,105,110,116,115,0,0,0,0,0,0,37,115,32,112,114,111,104,105,98,105,116,101,100,32,105,110,32,112,97,114,116,105,97,108,32,105,110,100,101,120,32,87,72,69,82,69,32,99,108,97,117,115,101,115,0,0,0,0,110,101,119,0,0,0,0,0,111,108,100,0,0,0,0,0,109,105,115,117,115,101,32,111,102,32,97,108,105,97,115,101,100,32,97,103,103,114,101,103,97,116,101,32,37,115,0,0,110,111,32,115,117,99,104,32,99,111,108,117,109,110,0,0,97,109,98,105,103,117,111,117,115,32,99,111,108,117,109,110,32,110,97,109,101,0,0,0,37,115,58,32,37,115,46,37,115,46,37,115,0,0,0,0,37,115,58,32,37,115,46,37,115,0,0,0,0,0,0,0,37,115,58,32,37,115,0,0,82,79,87,73,68,0,0,0,97,99,99,101,115,115,32,116,111,32,37,115,46,37,115,46,37,115,32,105,115,32,112,114,111,104,105,98,105,116,101,100,0,0,0,0,0,0,0,0,97,99,99,101,115,115,32,116,111,32,37,115,46,37,115,32,105,115,32,112,114,111,104,105,98,105,116,101,100,0,0,0,95,82,79,87,73,68,95,0,79,73,68,0,0,0,0,0,115,113,108,105,116,101,95,115,113,95,37,112,0,0,0,0,116,111,111,32,109,97,110,121,32,114,101,102,101,114,101,110,99,101,115,32,116,111,32,34,37,115,34,58,32,109,97,120,32,54,53,53,51,53,0,0,42,0,0,0,0,0,0,0,37,115,46,37,115,0,0,0,37,115,46,37,115,46,37,115,0,0,0,0,0,0,0,0,110,111,32,115,117,99,104,32,116,97,98,108,101,58,32,37,115,0,0,0,0,0,0,0,110,111,32,116,97,98,108,101,115,32,115,112,101,99,105,102,105,101,100,0,0,0,0,0,116,111,111,32,109,97,110,121,32,99,111,108,117,109,110,115,32,105,110,32,114,101,115,117,108,116,32,115,101,116,0,0,97,32,78,65,84,85,82,65,76,32,106,111,105,110,32,109,97,121,32,110,111,116,32,104,97,118,101,32,97,110,32,79,78,32,111,114,32,85,83,73,78,71,32,99,108,97,117,115,101,0,0,0,0,0,0,0,99,97,110,110,111,116,32,104,97,118,101,32,98,111,116,104,32,79,78,32,97,110,100,32,85,83,73,78,71,32,99,108,97,117,115,101,115,32,105,110,32,116,104,101,32,115,97,109,101,32,106,111,105,110,0,0,99,97,110,110,111,116,32,106,111,105,110,32,117,115,105,110,103,32,99,111,108,117,109,110,32,37,115,32,45,32,99,111,108,117,109,110,32,110,111,116,32,112,114,101,115,101,110,116,32,105,110,32,98,111,116,104,32,116,97,98,108,101,115,0,110,111,32,115,117,99,104,32,105,110,100,101,120,58,32,37,115,0,0,0,0,0,0,0,109,117,108,116,105,112,108,101,32,114,101,102,101,114,101,110,99,101,115,32,116,111,32,114,101,99,117,114,115,105,118,101,32,116,97,98,108,101,58,32,37,115,0,0,0,0,0,0,99,105,114,99,117,108,97,114,32,114,101,102,101,114,101,110,99,101,58,32,37,115,0,0,116,97,98,108,101,32,37,115,32,104,97,115,32,37,100,32,118,97,108,117,101,115,32,102,111,114,32,37,100,32,99,111,108,117,109,110,115,0,0,0,109,117,108,116,105,112,108,101,32,114,101,99,117,114,115,105,118,101,32,114,101,102,101,114,101,110,99,101,115,58,32,37,115,0,0,0,0,0,0,0,114,101,99,117,114,115,105,118,101,32,114,101,102,101,114,101,110,99,101,32,105,110,32,97,32,115,117,98,113,117,101,114,121,58,32,37,115,0,0,0,110,111,32,115,117,99,104,32,109,111,100,117,108,101,58,32,37,115,0,0,0,0,0,0,118,116,97,98,108,101,32,99,111,110,115,116,114,117,99,116,111,114,32,102,97,105,108,101,100,58,32,37,115,0,0,0,118,116,97,98,108,101,32,99,111,110,115,116,114,117,99,116,111,114,32,100,105,100,32,110,111,116,32,100,101,99,108,97,114,101,32,115,99,104,101,109,97,58,32,37,115,0,0,0,104,105,100,100,101,110,0,0,32,104,105,100,100,101,110,0,111,98,106,101,99,116,32,110,97,109,101,32,114,101,115,101,114,118,101,100,32,102,111,114,32,105,110,116,101,114,110,97,108,32,117,115,101,58,32,37,115,0,0,0,0,0,0,0,99,111,114,114,117,112,116,32,100,97,116,97,98,97,115,101,0,0,0,0,0,0,0,0,117,110,107,110,111,119,110,32,100,97,116,97,98,97,115,101,32,37,84,0,0,0,0,0,105,100,120,0,0,0,0,0,116,98,108,0,0,0,0,0,2,0,1,0,0,0,0,0,0,0,0,0,46,0,0,0,0,0,0,0,0,0,0,0,176,71,0,0,0,0,0,0,0,0,0,0,0,0,0,0,2,0,1,0,0,0,0,0,0,0,0,0,47,0,0,0,0,0,0,0,0,0,0,0,160,71,0,0,0,0,0,0,0,0,0,0,0,0,0,0,66,66,66,0,0,0,0,0,1,0,1,0,0,0,0,0,0,0,0,0,48,0,0,0,0,0,0,0,0,0,0,0,128,71,0,0,0,0,0,0,0,0,0,0,0,0,0,0,115,116,97,116,95,103,101,116,0,0,0,0,0,0,0,0,37,108,108,117,0,0,0,0,32,37,108,108,117,0,0,0,115,116,97,116,95,112,117,115,104,0,0,0,0,0,0,0,115,116,97,116,95,105,110,105,116,0,0,0,0,0,0,0,216,71,0,0,232,71,0,0,248,71,0,0,0,0,0,0,8,72,0,0,0,0,0,0,115,113,108,105,116,101,95,115,116,97,116,49,0,0,0,0,116,98,108,44,105,100,120,44,115,116,97,116,0,0,0,0,115,113,108,105,116,101,95,115,116,97,116,51,0,0,0,0,115,113,108,105,116,101,95,115,116,97,116,52,0,0,0,0,67,82,69,65,84,69,32,84,65,66,76,69,32,37,81,46,37,115,40,37,115,41,0,0,68,69,76,69,84,69,32,70,82,79,77,32,37,81,46,37,115,32,87,72,69,82,69,32,37,115,61,37,81,0,0,0,67,82,69,65,84,69,32,84,65,66,76,69,32,115,113,108,105,116,101,95,109,97,115,116,101,114,40,10,32,32,116,121,112,101,32,116,101,120,116,44,10,32,32,110,97,109,101,32,116,101,120,116,44,10,32,32,116,98,108,95,110,97,109,101,32,116,101,120,116,44,10,32,32,114,111,111,116,112,97,103,101,32,105,110,116,101,103,101,114,44,10,32,32,115,113,108,32,116,101,120,116,10,41,0,67,82,69,65,84,69,32,84,69,77,80,32,84,65,66,76,69,32,115,113,108,105,116,101,95,116,101,109,112,95,109,97,115,116,101,114,40,10,32,32,116,121,112,101,32,116,101,120,116,44,10,32,32,110,97,109,101,32,116,101,120,116,44,10,32,32,116,98,108,95,110,97,109,101,32,116,101,120,116,44,10,32,32,114,111,111,116,112,97,103,101,32,105,110,116,101,103,101,114,44,10,32,32,115,113,108,32,116,101,120,116,10,41,0,0,0,0,0,0,0,49,0,0,0,0,0,0,0,97,116,116,97,99,104,101,100,32,100,97,116,97,98,97,115,101,115,32,109,117,115,116,32,117,115,101,32,116,104,101,32,115,97,109,101,32,116,101,120,116,32,101,110,99,111,100,105,110,103,32,97,115,32,109,97,105,110,32,100,97,116,97,98,97,115,101,0,0,0,0,0,117,110,115,117,112,112,111,114,116,101,100,32,102,105,108,101,32,102,111,114,109,97,116,0,83,69,76,69,67,84,32,110,97,109,101,44,32,114,111,111,116,112,97,103,101,44,32,115,113,108,32,70,82,79,77,32,39,37,113,39,46,37,115,32,79,82,68,69,82,32,66,89,32,114,111,119,105,100,0,0,83,69,76,69,67,84,32,116,98,108,44,105,100,120,44,115,116,97,116,32,70,82,79,77,32,37,81,46,115,113,108,105,116,101,95,115,116,97,116,49,0,0,0,0,0,0,0,0,117,110,111,114,100,101,114,101,100,42,0,0,0,0,0,0,115,122,61,91,48,45,57,93,42,0,0,0,0,0,0,0,33,0,32,0,30,0,28,0,26,0,0,0,0,0,0,0,105,110,118,97,108,105,100,32,114,111,111,116,112,97,103,101,0,0,0,0,0,0,0,0,63,0,0,0,0,0,0,0,109,97,108,102,111,114,109,101,100,32,100,97,116,97,98,97,115,101,32,115,99,104,101,109,97,32,40,37,115,41,0,0,37,115,32,45,32,37,115,0,117,110,97,98,108,101,32,116,111,32,105,100,101,110,116,105,102,121,32,116,104,101,32,111,98,106,101,99,116,32,116,111,32,98,101,32,114,101,105,110,100,101,120,101,100,0,0,0,44,32,0,0,0,0,0,0,46,0,0,0,0,0,0,0,109,105,115,117,115,101,32,111,102,32,97,103,103,114,101,103,97,116,101,58,32,37,115,40,41,0,0,0,0,0,0,0,117,110,107,110,111,119,110,32,102,117,110,99,116,105,111,110,58,32,37,46,42,115,40,41,0,0,0,0,0,0,0,0,82,65,73,83,69,40,41,32,109,97,121,32,111,110,108,121,32,98,101,32,117,115,101,100,32,119,105,116,104,105,110,32,97,32,116,114,105,103,103,101,114,45,112,114,111,103,114,97,109,0,0,0,0,0,0,0,69,88,69,67,85,84,69,32,37,115,37,115,32,83,85,66,81,85,69,82,89,32,37,100,0,0,0,0,0,0,0,0,67,79,82,82,69,76,65,84,69,68,32,0,0,0,0,0,76,73,83,84,0,0,0,0,83,67,65,76,65,82,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,48,120,0,0,0,0,0,0,104,101,120,32,108,105,116,101,114,97,108,32,116,111,111,32,98,105,103,58,32,37,115,0,1,0,1,0,0,0,0,0,0,0,0,0,49,0,0,0,0,0,0,0,0,0,0,0,208,75,0,0,0,0,0,0,0,0,0,0,0,0,0,0,115,113,108,105,116,101,95,100,101,116,97,99,104,0,0,0,110,111,32,115,117,99,104,32,100,97,116,97,98,97,115,101,58,32,37,115,0,0,0,0,99,97,110,110,111,116,32,100,101,116,97,99,104,32,100,97,116,97,98,97,115,101,32,37,115,0,0,0,0,0,0,0,99,97,110,110,111,116,32,68,69,84,65,67,72,32,100,97,116,97,98,97,115,101,32,119,105,116,104,105,110,32,116,114,97,110,115,97,99,116,105,111,110,0,0,0,0,0,0,0,100,97,116,97,98,97,115,101,32,37,115,32,105,115,32,108,111,99,107,101,100,0,0,0,3,0,1,0,0,0,0,0,0,0,0,0,50,0,0,0,0,0,0,0,0,0,0,0,136,76,0,0,0,0,0,0,0,0,0,0,0,0,0,0,115,113,108,105,116,101,95,97,116,116,97,99,104,0,0,0,116,111,111,32,109,97,110,121,32,97,116,116,97,99,104,101,100,32,100,97,116,97,98,97,115,101,115,32,45,32,109,97,120,32,37,100,0,0,0,0,99,97,110,110,111,116,32,65,84,84,65,67,72,32,100,97,116,97,98,97,115,101,32,119,105,116,104,105,110,32,116,114,97,110,115,97,99,116,105,111,110,0,0,0,0,0,0,0,100,97,116,97,98,97,115,101,32,37,115,32,105,115,32,97,108,114,101,97,100,121,32,105,110,32,117,115,101,0,0,0,100,97,116,97,98,97,115,101,32,105,115,32,97,108,114,101,97,100,121,32,97,116,116,97,99,104,101,100,0,0,0,0,117,110,97,98,108,101,32,116,111,32,111,112,101,110,32,100,97,116,97,98,97,115,101,58,32,37,115,0,0,0,0,0,110,111,32,115,117,99,104,32,116,114,105,103,103,101,114,58,32,37,83,0,0,0,0,0,108,0,246,0,97,0,1,0,47,0,1,2,78,2,247,1,97,0,1,0,47,0,0,2,78,2,247,1,95,0,0,0,9,0,254,0,0,0,0,0,116,101,109,112,111,114,97,114,121,32,116,114,105,103,103,101,114,32,109,97,121,32,110,111,116,32,104,97,118,101,32,113,117,97,108,105,102,105,101,100,32,110,97,109,101,0,0,0,99,97,110,110,111,116,32,99,114,101,97,116,101,32,116,114,105,103,103,101,114,115,32,111,110,32,118,105,114,116,117,97,108,32,116,97,98,108,101,115,0,0,0,0,0,0,0,0,116,114,105,103,103,101,114,32,37,84,32,97,108,114,101,97,100,121,32,101,120,105,115,116,115,0,0,0,0,0,0,0,99,97,110,110,111,116,32,99,114,101,97,116,101,32,116,114,105,103,103,101,114,32,111,110,32,115,121,115,116,101,109,32,116,97,98,108,101,0,0,0,99,97,110,110,111,116,32,99,114,101,97,116,101,32,37,115,32,116,114,105,103,103,101,114,32,111,110,32,118,105,101,119,58,32,37,83,0,0,0,0,66,69,70,79,82,69,0,0,65,70,84,69,82,0,0,0,99,97,110,110,111,116,32,99,114,101,97,116,101,32,73,78,83,84,69,65,68,32,79,70,32,116,114,105,103,103,101,114,32,111,110,32,116,97,98,108,101,58,32,37,83,0,0,0,37,115,32,37,84,32,99,97,110,110,111,116,32,114,101,102,101,114,101,110,99,101,32,111,98,106,101,99,116,115,32,105,110,32,100,97,116,97,98,97,115,101,32,37,115,0,0,0,37,115,32,99,97,110,110,111,116,32,117,115,101,32,118,97,114,105,97,98,108,101,115,0,73,78,83,69,82,84,32,73,78,84,79,32,37,81,46,37,115,32,86,65,76,85,69,83,40,39,116,114,105,103,103,101,114,39,44,37,81,44,37,81,44,48,44,39,67,82,69,65,84,69,32,84,82,73,71,71,69,82,32,37,113,39,41,0,116,121,112,101,61,39,116,114,105,103,103,101,114,39,32,65,78,68,32,110,97,109,101,61,39,37,113,39,0,0,0,0,45,37,84,0,0,0,0,0,114,101,115,117,108,116,0,0,96,87,0,0,0,0,0,0,0,0,0,0,152,82,0,0,1,1,0,0,0,0,0,0,112,87,0,0,2,0,0,0,0,0,16,0,128,87,0,0,3,0,0,0,0,0,0,0,24,82,0,0,4,1,0,0,0,0,0,0,144,87,0,0,2,0,0,0,16,0,0,0,160,87,0,0,5,0,0,0,0,0,0,0,184,87,0,0,2,0,0,0,8,0,0,0,208,87,0,0,6,0,0,0,0,0,0,0,224,87,0,0,7,0,0,0,0,0,0,0,240,87,0,0,2,0,0,0,128,0,0,0,0,88,0,0,9,1,0,0,0,0,0,0,16,88,0,0,10,1,0,0,0,0,0,0,40,88,0,0,2,0,0,0,0,0,0,1,64,88,0,0,2], "i8", ALLOC_NONE, Runtime.GLOBAL_BASE+10240);
/* memory initializer */ allocate([1,0,0,120,85,0,0,11,0,0,0,0,0,0,0,88,88,0,0,12,1,0,0,0,0,0,0,112,88,0,0,13,1,0,0,0,0,0,0,136,88,0,0,2,0,0,0,0,0,8,0,152,88,0,0,0,0,0,0,0,0,0,0,168,88,0,0,2,0,0,0,32,0,0,0,192,88,0,0,2,0,0,0,4,0,0,0,208,88,0,0,2,0,0,0,0,32,0,0,240,88,0,0,14,1,0,0,0,0,0,0,8,89,0,0,15,1,0,0,0,0,0,0,24,89,0,0,16,1,0,0,0,0,0,0,56,84,0,0,17,1,0,0,0,0,0,0,112,82,0,0,18,1,0,0,0,0,0,0,128,82,0,0,19,0,0,0,0,0,0,0,40,89,0,0,2,0,0,0,0,128,0,0,96,82,0,0,21,0,0,0,0,0,0,0,64,89,0,0,22,1,0,0,0,0,0,0,192,82,0,0,23,0,0,0,0,0,0,0,80,89,0,0,22,1,0,0,0,0,0,0,40,82,0,0,24,0,0,0,0,0,0,0,96,89,0,0,2,0,0,0,0,0,0,2,112,89,0,0,17,1,0,0,0,0,0,0,128,89,0,0,2,0,0,0,0,64,0,0,152,89,0,0,2,0,0,0,0,0,4,0,176,89,0,0,2,0,0,0,0,0,2,0,208,89,0,0,0,0,0,0,0,0,0,0,56,82,0,0,25,0,0,0,0,0,0,0,224,89,0,0,2,0,0,0,64,0,0,0,248,89,0,0,26,0,0,0,0,0,0,0,40,86,0,0,27,0,0,0,0,0,0,0,8,90,0,0,28,1,0,0,0,0,0,0,24,83,0,0,29,1,0,0,0,0,0,0,16,90,0,0,30,1,0,0,0,0,0,0,208,82,0,0,31,0,0,0,0,0,0,0,224,82,0,0,32,0,0,0,0,0,0,0,56,86,0,0,33,0,0,0,0,0,0,0,32,90,0,0,0,0,0,0,0,0,0,0,8,86,0,0,34,0,0,0,0,0,0,0,48,90,0,0,35,1,0,0,0,0,0,0,64,90,0,0,2,0,0,0,0,8,1,0,0,0,0,0,4,0,0,0,51,0,1,3,137,1,8,0,25,0,2,0,90,1,2,1,137,1,8,0,25,0,1,0,154,0,0,0,35,1,1,0,0,0,0,0,99,97,99,104,101,95,115,105,122,101,0,0,0,0,0,0,112,97,103,101,95,115,105,122,101,0,0,0,0,0,0,0,115,101,99,117,114,101,95,100,101,108,101,116,101,0,0,0,110,111,114,109,97,108,0,0,101,120,99,108,117,115,105,118,101,0,0,0,0,0,0,0,108,111,99,107,105,110,103,95,109,111,100,101,0,0,0,0,106,111,117,114,110,97,108,95,109,111,100,101,0,0,0,0,106,111,117,114,110,97,108,95,115,105,122,101,95,108,105,109,105,116,0,0,0,0,0,0,97,117,116,111,95,118,97,99,117,117,109,0,0,0,0,0,4,0,1,0,51,0,1,4,45,1,0,0,24,0,2,0,25,0,1,0,52,0,7,1,109,109,97,112,95,115,105,122,101,0,0,0,0,0,0,0,116,101,109,112,95,115,116,111,114,101,0,0,0,0,0,0,116,101,109,112,95,115,116,111,114,101,95,100,105,114,101,99,116,111,114,121,0,0,0,0,110,111,116,32,97,32,119,114,105,116,97,98,108,101,32,100,105,114,101,99,116,111,114,121,0,0,0,0,0,0,0,0,115,121,110,99,104,114,111,110,111,117,115,0,0,0,0,0,83,97,102,101,116,121,32,108,101,118,101,108,32,109,97,121,32,110,111,116,32,98,101,32,99,104,97,110,103,101,100,32,105,110,115,105,100,101,32,97,32,116,114,97,110,115,97,99,116,105,111,110,0,0,0,0,99,105,100,0,0,0,0,0,110,97,109,101,0,0,0,0,116,121,112,101,0,0,0,0,110,111,116,110,117,108,108,0,100,102,108,116,95,118,97,108,117,101,0,0,0,0,0,0,112,107,0,0,0,0,0,0,116,97,98,108,101,0,0,0,119,105,100,116,104,0,0,0,104,101,105,103,104,116,0,0,115,101,113,110,111,0,0,0,115,101,113,0,0,0,0,0,117,110,105,113,117,101,0,0,102,105,108,101,0,0,0,0,105,100,0,0,0,0,0,0,102,114,111,109,0,0,0,0,116,111,0,0,0,0,0,0,111,110,95,117,112,100,97,116,101,0,0,0,0,0,0,0,111,110,95,100,101,108,101,116,101,0,0,0,0,0,0,0,109,97,116,99,104,0,0,0,78,79,78,69,0,0,0,0,112,97,114,101,110,116,0,0,102,107,105,100,0,0,0,0,138,1,0,0,97,0,3,0,35,3,1,0,0,0,0,0,105,110,116,101,103,114,105,116,121,95,99,104,101,99,107,0,42,42,42,32,105,110,32,100,97,116,97,98,97,115,101,32,37,115,32,42,42,42,10,0,78,85,76,76,32,118,97,108,117,101,32,105,110,32,37,115,46,37,115,0,0,0,0,0,114,111,119,32,0,0,0,0,32,109,105,115,115,105,110,103,32,102,114,111,109,32,105,110,100,101,120,32,0,0,0,0,110,111,110,45,117,110,105,113,117,101,32,101,110,116,114,121,32,105,110,32,105,110,100,101,120,32,0,0,0,0,0,0,119,114,111,110,103,32,35,32,111,102,32,101,110,116,114,105,101,115,32,105,110,32,105,110,100,101,120,32,0,0,0,0,111,107,0,0,0,0,0,0,40,85,0,0,1,0,0,0,48,85,0,0,1,0,0,0,56,85,0,0,2,0,0,0,72,85,0,0,3,0,0,0,88,85,0,0,2,0,0,0,96,85,0,0,3,0,0,0,104,85,0,0,0,0,0,0,112,85,0,0,0,0,0,0,0,0,0,0,0,0,0,0,85,84,70,56,0,0,0,0,85,84,70,45,56,0,0,0,85,84,70,45,49,54,108,101,0,0,0,0,0,0,0,0,85,84,70,45,49,54,98,101,0,0,0,0,0,0,0,0,85,84,70,49,54,108,101,0,85,84,70,49,54,98,101,0,85,84,70,45,49,54,0,0,85,84,70,49,54,0,0,0,101,110,99,111,100,105,110,103,0,0,0,0,0,0,0,0,117,110,115,117,112,112,111,114,116,101,100,32,101,110,99,111,100,105,110,103,58,32,37,115,0,0,0,0,0,0,0,0,4,0,1,0,25,0,1,0,52,0,0,1,0,0,0,0,4,0,0,0,51,0,1,0,35,1,1,0,0,0,0,0,99,111,109,112,105,108,101,95,111,112,116,105,111,110,0,0,102,117,108,108,0,0,0,0,114,101,115,116,97,114,116,0,98,117,115,121,0,0,0,0,108,111,103,0,0,0,0,0,99,104,101,99,107,112,111,105,110,116,101,100,0,0,0,0,119,97,108,95,97,117,116,111,99,104,101,99,107,112,111,105,110,116,0,0,0,0,0,0,116,105,109,101,111,117,116,0,115,111,102,116,95,104,101,97,112,95,108,105,109,105,116,0,116,104,114,101,97,100,115,0,37,95,0,0,0,0,0,0,102,111,114,101,105,103,110,32,107,101,121,32,109,105,115,109,97,116,99,104,32,45,32,34,37,119,34,32,114,101,102,101,114,101,110,99,105,110,103,32,34,37,119,34,0,0,0,0,83,69,84,32,78,85,76,76,0,0,0,0,0,0,0,0,83,69,84,32,68,69,70,65,85,76,84,0,0,0,0,0,67,65,83,67,65,68,69,0,82,69,83,84,82,73,67,84,0,0,0,0,0,0,0,0,78,79,32,65,67,84,73,79,78,0,0,0,0,0,0,0,116,101,109,112,111,114,97,114,121,32,115,116,111,114,97,103,101,32,99,97,110,110,111,116,32,98,101,32,99,104,97,110,103,101,100,32,102,114,111,109,32,119,105,116,104,105,110,32,97,32,116,114,97,110,115,97,99,116,105,111,110,0,0,0,110,111,110,101,0,0,0,0,105,110,99,114,101,109,101,110,116,97,108,0,0,0,0,0,48,87,0,0,56,87,0,0,64,87,0,0,72,87,0,0,184,18,0,0,88,87,0,0,100,101,108,101,116,101,0,0,112,101,114,115,105,115,116,0,111,102,102,0,0,0,0,0,116,114,117,110,99,97,116,101,0,0,0,0,0,0,0,0,119,97,108,0,0,0,0,0,97,112,112,108,105,99,97,116,105,111,110,95,105,100,0,0,97,117,116,111,109,97,116,105,99,95,105,110,100,101,120,0,98,117,115,121,95,116,105,109,101,111,117,116,0,0,0,0,99,97,99,104,101,95,115,112,105,108,108,0,0,0,0,0,99,97,115,101,95,115,101,110,115,105,116,105,118,101,95,108,105,107,101,0,0,0,0,0,99,104,101,99,107,112,111,105,110,116,95,102,117,108,108,102,115,121,110,99,0,0,0,0,99,111,108,108,97,116,105,111,110,95,108,105,115,116,0,0,99,111,109,112,105,108,101,95,111,112,116,105,111,110,115,0,99,111,117,110,116,95,99,104,97,110,103,101,115,0,0,0,100,97,116,97,98,97,115,101,95,108,105,115,116,0,0,0,100,101,102,97,117,108,116,95,99,97,99,104,101,95,115,105,122,101,0,0,0,0,0,0,100,101,102,101,114,95,102,111,114,101,105,103,110,95,107,101,121,115,0,0,0,0,0,0,101,109,112,116,121,95,114,101,115,117,108,116,95,99,97,108,108,98,97,99,107,115,0,0,102,111,114,101,105,103,110,95,107,101,121,95,99,104,101,99,107,0,0,0,0,0,0,0,102,111,114,101,105,103,110,95,107,101,121,95,108,105,115,116,0,0,0,0,0,0,0,0,102,111,114,101,105,103,110,95,107,101,121,115,0,0,0,0,102,114,101,101,108,105,115,116,95,99,111,117,110,116,0,0,102,117,108,108,95,99,111,108,117,109,110,95,110,97,109,101,115,0,0,0,0,0,0,0,102,117,108,108,102,115,121,110,99,0,0,0,0,0,0,0,105,103,110,111,114,101,95,99,104,101,99,107,95,99,111,110,115,116,114,97,105,110,116,115,0,0,0,0,0,0,0,0,105,110,99,114,101,109,101,110,116,97,108,95,118,97,99,117,117,109,0,0,0,0,0,0,105,110,100,101,120,95,105,110,102,111,0,0,0,0,0,0,105,110,100,101,120,95,108,105,115,116,0,0,0,0,0,0,108,101,103,97,99,121,95,102,105,108,101,95,102,111,114,109,97,116,0,0,0,0,0,0,109,97,120,95,112,97,103,101,95,99,111,117,110,116,0,0,112,97,103,101,95,99,111,117,110,116,0,0,0,0,0,0,113,117,101,114,121,95,111,110,108,121,0,0,0,0,0,0,113,117,105,99,107,95,99,104,101,99,107,0,0,0,0,0,114,101,97,100,95,117,110,99,111,109,109,105,116,116,101,100,0,0,0,0,0,0,0,0,114,101,99,117,114,115,105,118,101,95,116,114,105,103,103,101,114,115,0,0,0,0,0,0,114,101,118,101,114,115,101,95,117,110,111,114,100,101,114,101,100,95,115,101,108,101,99,116,115,0,0,0,0,0,0,0,115,99,104,101,109,97,95,118,101,114,115,105,111,110,0,0,115,104,111,114,116,95,99,111,108,117,109,110,95,110,97,109,101,115,0,0,0,0,0,0,115,104,114,105,110,107,95,109,101,109,111,114,121,0,0,0,115,116,97,116,115,0,0,0,116,97,98,108,101,95,105,110,102,111,0,0,0,0,0,0,117,115,101,114,95,118,101,114,115,105,111,110,0,0,0,0,119,97,108,95,99,104,101,99,107,112,111,105,110,116,0,0,119,114,105,116,97,98,108,101,95,115,99,104,101,109,97,0,110,111,32,115,117,99,104,32,105,110,100,101,120,58,32,37,83,0,0,0,0,0,0,0,105,110,100,101,120,32,97,115,115,111,99,105,97,116,101,100,32,119,105,116,104,32,85,78,73,81,85,69,32,111,114,32,80,82,73,77,65,82,89,32,75,69,89,32,99,111,110,115,116,114,97,105,110,116,32,99,97,110,110,111,116,32,98,101,32,100,114,111,112,112,101,100,0,0,0,0,0,0,0,0,68,69,76,69,84,69,32,70,82,79,77,32,37,81,46,37,115,32,87,72,69,82,69,32,110,97,109,101,61,37,81,32,65,78,68,32,116,121,112,101,61,39,105,110,100,101,120,39,0,0,0,0,0,0,0,0,85,80,68,65,84,69,32,37,81,46,37,115,32,83,69,84,32,114,111,111,116,112,97,103,101,61,37,100,32,87,72,69,82,69,32,35,37,100,32,65,78,68,32,114,111,111,116,112,97,103,101,61,35,37,100,0,115,113,108,105,116,101,95,115,116,97,116,37,100,0,0,0,48,0,0,0,0,0,0,0,118,97,114,105,97,98,108,101,32,110,117,109,98,101,114,32,109,117,115,116,32,98,101,32,98,101,116,119,101,101,110,32,63,49,32,97,110,100,32,63,37,100,0,0,0,0,0,0,116,111,111,32,109,97,110,121,32,83,81,76,32,118,97,114,105,97,98,108,101,115,0,0,116,97,98,108,101,32,37,83,32,104,97,115,32,110,111,32,99,111,108,117,109,110,32,110,97,109,101,100,32,37,115,0,116,97,98,108,101,32,37,83,32,104,97,115,32,37,100,32,99,111,108,117,109,110,115,32,98,117,116,32,37,100,32,118,97,108,117,101,115,32,119,101,114,101,32,115,117,112,112,108,105,101,100,0,0,0,0,0,37,100,32,118,97,108,117,101,115,32,102,111,114,32,37,100,32,99,111,108,117,109,110,115,0,0,0,0,0,0,0,0,114,111,119,115,32,105,110,115,101,114,116,101,100,0,0,0,97,116,32,109,111,115,116,32,37,100,32,116,97,98,108,101,115,32,105,110,32,97,32,106,111,105,110,0,0,0,0,0,65,63,62,64,0,0,0,0,0,0,108,105,65,62,64,63,116,114,113,115,0,0,0,0,9,8,0,0,0,0,0,0,108,105,0,0,0,0,0,0,83,69,65,82,67,72,0,0,83,67,65,78,0,0,0,0,32,83,85,66,81,85,69,82,89,32,37,100,0,0,0,0,32,84,65,66,76,69,32,37,115,0,0,0,0,0,0,0,32,65,83,32,37,115,0,0,80,82,73,77,65,82,89,32,75,69,89,0,0,0,0,0,65,85,84,79,77,65,84,73,67,32,67,79,86,69,82,73,78,71,32,73,78,68,69,88,0,0,0,0,0,0,0,0,67,79,86,69,82,73,78,71,32,73,78,68,69,88,32,37,115,0,0,0,0,0,0,0,73,78,68,69,88,32,37,115,0,0,0,0,0,0,0,0,32,85,83,73,78,71,32,0,40,114,111,119,105,100,61,63,41,0,0,0,0,0,0,0,40,114,111,119,105,100,62,63,32,65,78,68,32,114,111,119,105,100,60,63,41,0,0,0,40,114,111,119,105,100,62,63,41,0,0,0,0,0,0,0,40,114,111,119,105,100,60,63,41,0,0,0,0,0,0,0,32,85,83,73,78,71,32,73,78,84,69,71,69,82,32,80,82,73,77,65,82,89,32,75,69,89,32,0,0,0,0,0,32,86,73,82,84,85,65,76,32,84,65,66,76,69,32,73,78,68,69,88,32,37,100,58,37,115,0,0,0,0,0,0,32,40,0,0,0,0,0,0,61,0,0,0,0,0,0,0,32,65,78,68,32,0,0,0,65,78,89,40,37,115,41,0,62,0,0,0,0,0,0,0,60,0,0,0,0,0,0,0,41,0,0,0,0,0,0,0,97,117,116,111,109,97,116,105,99,32,105,110,100,101,120,32,111,110,32,37,115,40,37,115,41,0,0,0,0,0,0,0,97,117,116,111,45,105,110,100,101,120,0,0,0,0,0,0,110,111,32,113,117,101,114,121,32,115,111,108,117,116,105,111,110,0,0,0,0,0,0,0,10,10,9,9,8,8,7,7,7,6,6,6,5,5,5,4,4,4,4,3,3,3,3,3,3,2,2,2,2,2,2,2,37,115,46,120,66,101,115,116,73,110,100,101,120,40,41,32,109,97,108,102,117,110,99,116,105,111,110,0,0,0,0,0,116,97,98,108,101,32,37,115,58,32,120,66,101,115,116,73,110,100,101,120,32,114,101,116,117,114,110,101,100,32,97,110,32,105,110,118,97,108,105,100,32,112,108,97,110,0,0,0,83,81,0,0,0,0,0,0,45,45,32,84,82,73,71,71,69,82,32,37,115,0,0,0,0,0,0,0,0,1,1,1,1,1,0,0,2,0,1,0,1,1,4,36,1,4,5,16,0,2,2,2,2,0,2,2,0,0,32,0,0,4,5,4,4,0,0,1,1,5,5,0,0,0,2,2,16,0,0,0,0,0,0,0,0,0,17,17,17,17,8,17,17,17,17,76,76,2,2,0,5,5,21,21,21,21,21,21,0,76,76,76,76,76,76,76,76,76,76,0,36,2,0,0,0,0,0,2,0,1,1,1,1,8,8,0,2,1,1,1,1,2,0,0,2,2,0,0,0,0,0,0,12,69,21,1,2,2,0,1,8,5,5,5,0,1,0,0,0,0,0,0,0,1,0,2,2,1,0,0,0,0,0,0,16,16,2,16,0,0,0,0,0,1,2,3,4,6,8,8,0,0,0,0,0,0,0,0,16,67,0,0,3,0,0,0,8,67,0,0,3,0,0,0,111,105,100,0,0,0,0,0,70,79,82,69,73,71,78,32,75,69,89,32,99,111,110,115,116,114,97,105,110,116,32,102,97,105,108,101,100,0,0,0,37,115,46,114,111,119,105,100,0,0,0,0,0,0,0,0,116,97,98,108,101,32,37,115,32,109,97,121,32,110,111,116,32,98,101,32,109,111,100,105,102,105,101,100,0,0,0,0,99,97,110,110,111,116,32,109,111,100,105,102,121,32,37,115,32,98,101,99,97,117,115,101,32,105,116,32,105,115,32,97,32,118,105,101,119,0,0,0,110,111,32,115,117,99,104,32,99,111,108,117,109,110,58,32,37,115,0,0,0,0,0,0,114,111,119,115,32,117,112,100,97,116,101,100,0,0,0,0,95,114,111,119,105,100,95,0,116,111,111,32,109,97,110,121,32,99,111,108,117,109,110,115,32,105,110,32,37,115,0,0,114,111,119,115,32,100,101,108,101,116,101,100,0,0,0,0,110,97,116,117,114,97,108,101,102,116,111,117,116,101,114,105,103,104,116,102,117,108,108,105,110,110,101,114,99,114,111,115,115,0,0,0,0,0,0,0,0,7,4,6,4,40,10,5,32,14,5,48,19,4,56,23,5,1,28,5,3,0,0,0,117,110,107,110,111,119,110,32,111,114,32,117,110,115,117,112,112,111,114,116,101,100,32,106,111,105,110,32,116,121,112,101,58,32,37,84,32,37,84,37,115,37,84,0,0,0,0,0,82,73,71,72,84,32,97,110,100,32,70,85,76,76,32,79,85,84,69,82,32,74,79,73,78,115,32,97,114,101,32,110,111,116,32,99,117,114,114,101,110,116,108,121,32,115,117,112,112,111,114,116,101,100,0,0,97,32,74,79,73,78,32,99,108,97,117,115,101,32,105,115,32,114,101,113,117,105,114,101,100,32,98,101,102,111,114,101,32,37,115,0,0,0,0,0,79,78,0,0,0,0,0,0,85,83,73,78,71,0,0,0,68,73,83,84,73,78,67,84,0,0,0,0,0,0,0,0,71,82,79,85,80,32,66,89,0,0,0,0,0,0,0,0,82,73,71,72,84,32,80,65,82,84,32,79,70,32,79,82,68,69,82,32,66,89,0,0,79,82,68,69,82,32,66,89,0,0,0,0,0,0,0,0,99,111,108,117,109,110,37,100,0,0,0,0,0,0,0,0,83,67,65,78,32,84,65,66,76,69,32,37,115,37,115,37,115,0,0,0,0,0,0,0,32,85,83,73,78,71,32,67,79,86,69,82,73,78,71,32,73,78,68,69,88,32,0,0,68,73,83,84,73,78,67,84,32,97,103,103,114,101,103,97,116,101,115,32,109,117,115,116,32,104,97,118,101,32,101,120,97,99,116,108,121,32,111,110,101,32,97,114,103,117,109,101,110,116,0,0,0,0,0,0,85,83,69,32,84,69,77,80,32,66,45,84,82,69,69,32,70,79,82,32,37,115,0,0,79,82,68,69,82,32,66,89,32,99,108,97,117,115,101,32,115,104,111,117,108,100,32,99,111,109,101,32,97,102,116,101,114,32,37,115,32,110,111,116,32,98,101,102,111,114,101,0,76,73,77,73,84,32,99,108,97,117,115,101,32,115,104,111,117,108,100,32,99,111,109,101,32,97,102,116,101,114,32,37,115,32,110,111,116,32,98,101,102,111,114,101,0,0,0,0,97,108,108,32,86,65,76,85,69,83,32,109,117,115,116,32,104,97,118,101,32,116,104,101,32,115,97,109,101,32,110,117,109,98,101,114,32,111,102,32,116,101,114,109,115,0,0,0,83,69,76,69,67,84,115,32,116,111,32,116,104,101,32,108,101,102,116,32,97,110,100,32,114,105,103,104,116,32,111,102,32,37,115,32,100,111,32,110,111,116,32,104,97,118,101,32,116,104,101,32,115,97,109,101,32,110,117,109,98,101,114,32,111,102,32,114,101,115,117,108,116,32,99,111,108,117,109,110,115,0,0,0,0,0,0,0,67,79,77,80,79,85,78,68,32,83,85,66,81,85,69,82,73,69,83,32,37,100,32,65,78,68,32,37,100,32,37,115,40,37,115,41,0,0,0,0,85,83,73,78,71,32,84,69,77,80,32,66,45,84,82,69,69,32,0,0,0,0,0,0,85,78,73,79,78,32,65,76,76,0,0,0,0,0,0,0,73,78,84,69,82,83,69,67,84,0,0,0,0,0,0,0,69,88,67,69,80,84,0,0,85,78,73,79,78,0,0,0,111,110,108,121,32,97,32,115,105,110,103,108,101,32,114,101,115,117,108,116,32,97,108,108,111,119,101,100,32,102,111,114,32,97,32,83,69,76,69,67,84,32,116,104,97,116,32,105,115,32,112,97,114,116,32,111,102,32,97,110,32,101,120,112,114,101,115,115,105,111,110,0,112,97,114,97,109,101,116,101,114,115,32,97,114,101,32,110,111,116,32,97,108,108,111,119,101,100,32,105,110,32,118,105,101,119,115,0,0,0,0,0,118,105,101,119,0,0,0,0,115,113,108,105,116,101,95,115,116,97,116,0,0,0,0,0,116,97,98,108,101,32,37,115,32,109,97,121,32,110,111,116,32,98,101,32,100,114,111,112,112,101,100,0,0,0,0,0,117,115,101,32,68,82,79,80,32,84,65,66,76,69,32,116,111,32,100,101,108,101,116,101,32,116,97,98,108,101,32,37,115,0,0,0,0,0,0,0,117,115,101,32,68,82,79,80,32,86,73,69,87,32,116,111,32,100,101,108,101,116,101,32,118,105,101,119,32,37,115,0,68,69,76,69,84,69,32,70,82,79,77,32,37,81,46,115,113,108,105,116,101,95,115,101,113,117,101,110,99,101,32,87,72,69,82,69,32,110,97,109,101,61,37,81,0,0,0,0,68,69,76,69,84,69,32,70,82,79,77,32,37,81,46,37,115,32,87,72,69,82,69,32,116,98,108,95,110,97,109,101,61,37,81,32,97,110,100,32,116,121,112,101,33,61,39,116,114,105,103,103,101,114,39,0,102,111,114,101,105,103,110,32,107,101,121,32,111,110,32,37,115,32,115,104,111,117,108,100,32,114,101,102,101,114,101,110,99,101,32,111,110,108,121,32,111,110,101,32,99,111,108,117,109,110,32,111,102,32,116,97,98,108,101,32,37,84,0,0,110,117,109,98,101,114,32,111,102,32,99,111,108,117,109,110,115,32,105,110,32,102,111,114,101,105,103,110,32,107,101,121,32,100,111,101,115,32,110,111,116,32,109,97,116,99,104,32,116,104,101,32,110,117,109,98,101,114,32,111,102,32,99,111,108,117,109,110,115,32,105,110,32,116,104,101,32,114,101,102,101,114,101,110,99,101,100,32,116,97,98,108,101,0,0,0,117,110,107,110,111,119,110,32,99,111,108,117,109,110,32,34,37,115,34,32,105,110,32,102,111,114,101,105,103,110,32,107,101,121,32,100,101,102,105,110,105,116,105,111,110,0,0,0,99,97,110,110,111,116,32,99,114,101,97,116,101,32,97,32,84,69,77,80,32,105,110,100,101,120,32,111,110,32,110,111,110,45,84,69,77,80,32,116,97,98,108,101,32,34,37,115,34,0,0,0,0,0,0,0,97,108,116,101,114,116,97,98,95,0,0,0,0,0,0,0,116,97,98,108,101,32,37,115,32,109,97,121,32,110,111,116,32,98,101,32,105,110,100,101,120,101,100,0,0,0,0,0,118,105,101,119,115,32,109,97,121,32,110,111,116,32,98,101,32,105,110,100,101,120,101,100,0,0,0,0,0,0,0,0,118,105,114,116,117,97,108,32,116,97,98,108,101,115,32,109,97,121,32,110,111,116,32,98,101,32,105,110,100,101,120,101,100,0,0,0,0,0,0,0,116,104,101,114,101,32,105,115,32,97,108,114,101,97,100,121,32,97,32,116,97,98,108,101,32,110,97,109,101,100,32,37,115,0,0,0,0,0,0,0,105,110,100,101,120,32,37,115,32,97,108,114,101,97,100,121,32,101,120,105,115,116,115,0,115,113,108,105,116,101,95,97,117,116,111,105,110,100,101,120,95,37,115,95,37,100,0,0,116,97,98,108,101,32,37,115,32,104,97,115,32,110,111,32,99,111,108,117,109,110,32,110,97,109,101,100,32,37,115,0,99,111,110,102,108,105,99,116,105,110,103,32,79,78,32,67,79,78,70,76,73,67,84,32,99,108,97,117,115,101,115,32,115,112,101,99,105,102,105,101,100,0,0,0,0,0,0,0,67,82,69,65,84,69,37,115,32,73,78,68,69,88,32,37,46,42,115,0,0,0,0,0,32,85,78,73,81,85,69,0,73,78,83,69,82,84,32,73,78,84,79,32,37,81,46,37,115,32,86,65,76,85,69,83,40,39,105,110,100,101,120,39,44,37,81,44,37,81,44,35,37,100,44,37,81,41,59,0,110,97,109,101,61,39,37,113,39,32,65,78,68,32,116,121,112,101,61,39,105,110,100,101,120,39,0,0,0,0,0,0,116,97,98,108,101,32,34,37,115,34,32,104,97,115,32,109,111,114,101,32,116,104,97,110,32,111,110,101,32,112,114,105,109,97,114,121,32,107,101,121,0,0,0,0,0,0,0,0,65,85,84,79,73,78,67,82,69,77,69,78,84,32,105,115,32,111,110,108,121,32,97,108,108,111,119,101,100,32,111,110,32,97,110,32,73,78,84,69,71,69,82,32,80,82,73,77,65,82,89,32,75,69,89,0,100,101,102,97,117,108,116,32,118,97,108,117,101,32,111,102,32,99,111,108,117,109,110,32,91,37,115,93,32,105,115,32,110,111,116,32,99,111,110,115,116,97,110,116,0,0,0,0,116,111,111,32,109,97,110,121,32,99,111,108,117,109,110,115,32,111,110,32,37,115,0,0,100,117,112,108,105,99,97,116,101,32,99,111,108,117,109,110,32,110,97,109,101,58,32,37,115,0,0,0,0,0,0,0,65,85,84,79,73,78,67,82,69,77,69,78,84,32,110,111,116,32,97,108,108,111,119,101,100,32,111,110,32,87,73,84,72,79,85,84,32,82,79,87,73,68,32,116,97,98,108,101,115,0,0,0,0,0,0,0,80,82,73,77,65,82,89,32,75,69,89,32,109,105,115,115,105,110,103,32,111,110,32,116,97,98,108,101,32,37,115,0,84,65,66,76,69,0,0,0,86,73,69,87,0,0,0,0,67,82,69,65,84,69,32,37,115,32,37,46,42,115,0,0,85,80,68,65,84,69,32,37,81,46,37,115,32,83,69,84,32,116,121,112,101,61,39,37,115,39,44,32,110,97,109,101,61,37,81,44,32,116,98,108,95,110,97,109,101,61,37,81,44,32,114,111,111,116,112,97,103,101,61,35,37,100,44,32,115,113,108,61,37,81,32,87,72,69,82,69,32,114,111,119,105,100,61,35,37,100,0,0,67,82,69,65,84,69,32,84,65,66,76,69,32,37,81,46,115,113,108,105,116,101,95,115,101,113,117,101,110,99,101,40,110,97,109,101,44,115,101,113,41,0,0,0,0,0,0,0,116,98,108,95,110,97,109,101,61,39,37,113,39,32,65,78,68,32,116,121,112,101,33,61,39,116,114,105,103,103,101,114,39,0,0,0,0,0,0,0,10,32,32,0,0,0,0,0,44,10,32,32,0,0,0,0,10,41,0,0,0,0,0,0,67,82,69,65,84,69,32,84,65,66,76,69,32,0,0,0,192,4,0,0,88,105,0,0,96,105,0,0,104,105,0,0,112,105,0,0,0,0,0,0,32,84,69,88,84,0,0,0,32,78,85,77,0,0,0,0,32,73,78,84,0,0,0,0,32,82,69,65,76,0,0,0,116,101,109,112,111,114,97,114,121,32,116,97,98,108,101,32,110,97,109,101,32,109,117,115,116,32,98,101,32,117,110,113,117,97,108,105,102,105,101,100,0,0,0,0,0,0,0,0,116,97,98,108,101,32,37,84,32,97,108,114,101,97,100,121,32,101,120,105,115,116,115,0,116,104,101,114,101,32,105,115,32,97,108,114,101,97,100,121,32,97,110,32,105,110,100,101,120,32,110,97,109,101,100,32,37,115,0,0,0,0,0,0,248,105,0,0,0,106,0,0,8,106,0,0,0,0,0,0,66,69,71,73,78,0,0,0,82,69,76,69,65,83,69,0,82,79,76,76,66,65,67,75,0,0,0,0,0,0,0,0,67,79,77,77,73,84,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,112,97,114,115,101,114,32,115,116,97,99,107,32,111,118,101,114,102,108,111,119,0,0,0,224,3,33,4,75,5,132,4,180,4,180,4,1,0,6,1,237,255,135,0,135,0,8,3,180,4,180,4,180,4,180,4,69,0,69,0,53,0,208,0,27,1,243,2,58,0,213,2,136,2,59,2,238,1,161,1,84,1,7,1,212,0,59,3,59,3,59,3,59,3,59,3,59,3,59,3,59,3,59,3,59,3,59,3,59,3,59,3,59,3,59,3,110,3,59,3,161,3,212,3,212,3,46,4,180,4,180,4,180,4,180,4,180,4,180,4,180,4,180,4,180,4,180,4,180,4,180,4,180,4,180,4,180,4,180,4,180,4,180,4,180,4,180,4,180,4,180,4,180,4,180,4,180,4,180,4,180,4,180,4,234,4,180,4,180,4,180,4,180,4,180,4,180,4,180,4,180,4,180,4,180,4,180,4,180,4,180,4,185,255,209,255,209,255,209,255,209,255,209,255,0,0,29,0,244,255,27,1,27,1,139,0,91,0,136,1,136,1,126,3,160,2,214,2,103,5,170,255,170,255,170,255,88,0,62,1,62,1,99,0,125,1,236,255,27,1,27,1,27,1,27,1,27,1,27,1,27,1,27,1,27,1,27,1,27,1,27,1,27,1,27,1,27,1,27,1,27,1,27,1,27,1,27,1,27,1,27,1,112,2,108,3,214,2,160,2,60,5,60,5,60,5,60,5,60,5,60,5,170,255,170,255,170,255,49,1,136,0,136,0,142,0,167,0,226,0,154,0,137,0,152,0,27,1,27,1,27,1,27,1,27,1,27,1,27,1,27,1,27,1,27,1,27,1,27,1,27,1,27,1,27,1,27,1,27,1,80,1,80,1,80,1,27,1,27,1,96,1,27,1,27,1,27,1,27,1,27,1,228,0,27,1,27,1,27,1,27,1,27,1,27,1,27,1,27,1,27,1,27,1,245,1,57,2,84,2,84,2,84,2,251,1,241,1,185,1,135,1,97,1,156,0,156,0,89,3,97,1,89,3,223,2,45,3,127,2,203,2,156,0,76,1,203,2,203,2,240,1,163,1,134,2,77,5,160,4,160,4,55,5,55,5,160,4,61,5,236,4,120,4,66,5,66,5,66,5,66,5,160,4,26,5,120,4,61,5,236,4,236,4,120,4,160,4,26,5,182,4,4,5,160,4,160,4,26,5,160,4,26,5,160,4,26,5,238,4,183,4,183,4,183,4,250,4,238,4,183,4,193,4,183,4,250,4,183,4,183,4,161,4,176,4,161,4,176,4,161,4,176,4,160,4,160,4,137,4,238,4,178,4,178,4,238,4,130,4,131,4,123,4,128,4,120,4,217,4,215,4,226,4,226,4,230,4,230,4,230,4,230,4,170,255,170,255,170,255,170,255,170,255,170,255,44,4,48,1,14,2,249,0,152,1,173,255,178,1,44,3,27,0,43,3,39,3,34,3,239,2,77,2,139,2,163,0,131,0,162,2,110,1,194,1,43,1,148,0,23,0,102,0,229,0,235,255,221,4,220,4,198,4,75,4,204,4,148,4,199,4,191,4,189,4,91,4,82,4,99,4,86,4,185,4,81,4,188,4,202,4,74,4,65,4,116,4,115,4,80,4,165,4,154,4,70,4,187,4,181,4,163,4,77,4,47,4,129,4,151,4,122,4,114,4,127,4,67,4,140,4,141,4,139,4,45,4,48,4,124,4,88,4,110,4,103,4,105,4,102,4,68,4,90,4,94,4,64,4,66,4,69,4,63,4,60,4,219,3,55,4,53,4,50,4,41,4,156,3,253,3,246,3,236,3,238,3,51,3,227,2,128,3,87,3,36,3,227,2,228,2,224,2,178,2,142,2,153,2,106,2,70,2,56,2,16,2,42,2,123,1,20,2,223,1,199,1,123,1,176,1,115,1,85,1,28,0,82,1,116,0,245,255,199,255,171,255,7,0,248,255,3,0,0,0,0,0,135,2,196,3,196,3,196,3,110,3,110,3,201,3,196,3,6,3,34,3,34,3,170,3,201,3,201,3,201,3,108,3,201,3,201,3,201,3,196,3,201,3,10,3,40,3,201,3,201,3,201,3,201,3,201,3,201,3,201,3,201,3,169,3,171,3,48,3,47,3,150,3,21,3,45,3,38,3,42,3,111,3,104,3,105,3,103,3,107,3,112,3,201,3,41,3,73,3,88,3,72,3,201,3,201,3,201,3,201,3,201,3,201,3,201,3,201,3,201,3,201,3,201,3,201,3,201,3,201,3,201,3,201,3,201,3,201,3,201,3,201,3,201,3,201,3,201,3,201,3,201,3,201,3,201,3,201,3,201,3,201,3,201,3,201,3,201,3,201,3,201,3,201,3,201,3,201,3,201,3,201,3,201,3,201,3,201,3,82,3,87,3,94,3,86,3,83,3,75,3,74,3,76,3,77,3,201,3,201,3,161,2,227,2,201,3,201,3,78,3,201,3,173,2,79,3,91,3,90,3,89,3,168,2,201,3,201,3,201,3,201,3,201,3,201,3,201,3,201,3,201,3,201,3,201,3,201,3,201,3,201,3,201,3,201,3,201,3,201,3,201,3,201,3,201,3,201,3,201,3,201,3,201,3,201,3,201,3,135,2,196,3,201,3,201,3,196,3,196,3,196,3,196,3,196,3,196,3,188,3,10,3,0,3,201,3,201,3,201,3,201,3,201,3,201,3,201,3,201,3,201,3,201,3,176,3,174,3,201,3,123,3,201,3,201,3,201,3,201,3,201,3,201,3,201,3,201,3,201,3,201,3,201,3,201,3,201,3,201,3,201,3,201,3,201,3,201,3,201,3,201,3,201,3,201,3,201,3,201,3,201,3,201,3,201,3,201,3,201,3,201,3,201,3,201,3,201,3,141,2,201,3,143,3,6,3,6,3,6,3,8,3,242,2,254,2,143,2,44,3,23,3,23,3,155,3,44,3,155,3,198,2,221,2,195,2,34,3,23,3,106,3,34,3,34,3,7,3,254,2,201,3,181,3,14,3,14,3,173,3,173,3,14,3,53,3,231,2,44,3,238,2,238,2,238,2,238,2,14,3,158,2,44,3,53,3,231,2,231,2,44,3,14,3,158,2,149,3,147,3,14,3,14,3,158,2,14,3,158,2,14,3,158,2,116,3,229,2,229,2,229,2,213,2,116,3,229,2,198,2,229,2,213,2,229,2,229,2,27,3,22,3,27,3,22,3,27,3,22,3,14,3,14,3,201,3,116,3,120,3,120,3,116,3,39,3,28,3,37,3,35,3,44,3,164,2,216,2,151,2,151,2,140,2,140,2,140,2,140,2,193,3,193,3,188,3,200,2,200,2,183,2,201,3,201,3,201,3,201,3,201,3,201,3,175,2,201,3,125,3,201,3,201,3,201,3,201,3,201,3,201,3,201,3,201,3,201,3,201,3,201,3,201,3,201,3,201,3,201,3,201,3,60,3,201,3,136,2,183,3,201,3,201,3,180,3,201,3,201,3,201,3,201,3,201,3,201,3,201,3,201,3,201,3,201,3,201,3,201,3,201,3,201,3,201,3,201,3,201,3,201,3,153,3,201,3,201,3,201,3,201,3,201,3,201,3,146,3,145,3,201,3,201,3,201,3,201,3,201,3,201,3,201,3,201,3,201,3,201,3,201,3,201,3,201,3,201,3,201,3,201,3,201,3,201,3,201,3,201,3,201,3,201,3,245,2,201,3,201,3,201,3,249,2,201,3,201,3,201,3,201,3,201,3,201,3,36,3,201,3,29,3,201,3,109,3,201,3,201,3,201,3,201,3,201,3,201,3,201,3,201,3,201,3,201,3,198,3,201,3,201,3,201,3,197,3,201,3,201,3,201,3,201,3,201,3,62,3,201,3,61,3,65,3,201,3,149,2,201,3,132,2,137,2,192,3,195,3,194,3,191,3,190,3,189,3,184,3,182,3,179,3,178,3,177,3,175,3,172,3,168,3,129,3,127,3,134,3,133,3,132,3,131,3,130,3,128,3,126,3,124,3,50,3,49,3,46,3,43,3,241,2,167,3,122,3,240,2,237,2,236,2,157,2,185,3,152,3,161,3,160,3,159,3,54,3,158,3,157,3,156,3,154,3,151,3,138,3,52,3,51,3,232,2,114,3,113,3,160,2,142,3,141,3,140,3,144,3,148,3,139,3,16,3,239,2,159,2,156,2,163,2,167,2,219,2,220,2,228,2,226,2,225,2,224,2,223,2,222,2,218,2,169,2,174,2,212,2,197,2,196,2,205,2,204,2,210,2,209,2,208,2,207,2,206,2,203,2,202,2,201,2,194,2,193,2,199,2,192,2,215,2,214,2,211,2,191,2,235,2,234,2,233,2,230,2,190,2,189,2,188,2,65,3,187,2,186,2,70,3,69,3,98,3,58,3,243,2,247,2,246,2,250,2,251,2,3,3,2,3,1,3,12,3,13,3,25,3,24,3,56,3,55,3,26,3,11,3,5,3,4,3,20,3,19,3,18,3,17,3,9,3,255,2,31,3,30,3,100,3,15,3,99,3,97,3,166,3,165,3,164,3,163,3,162,3,102,3,199,3,200,3,119,3,121,3,118,3,33,3,32,3,117,3,101,3,71,3,68,3,178,2,179,2,137,3,136,3,135,3,181,2,180,2,177,2,176,2,95,3,92,3,84,3,96,3,93,3,85,3,81,3,80,3,66,3,64,3,63,3,59,3,67,3,248,2,244,2,57,3,253,2,252,2,185,2,184,2,182,2,166,2,165,2,162,2,155,2,153,2,152,2,154,2,150,2,148,2,147,2,146,2,145,2,144,2,172,2,171,2,170,2,142,2,139,2,138,2,134,2,133,2,131,2,0,0,0,0,19,22,22,23,1,24,26,15,27,80,81,82,83,84,85,86,87,88,89,90,91,92,93,94,95,108,109,110,27,28,23,50,51,80,81,82,83,122,85,86,87,88,89,90,91,92,93,94,95,22,70,23,71,72,73,74,75,76,77,78,79,80,81,82,83,122,85,86,87,88,89,90,91,92,93,94,95,19,97,91,92,93,94,95,26,85,86,87,88,89,90,91,92,93,94,95,27,28,97,98,99,122,211,102,103,104,79,19,50,51,19,122,59,55,113,224,225,226,89,90,91,92,93,94,95,23,27,28,26,71,72,73,74,75,76,77,78,79,80,81,82,83,51,85,86,87,88,89,90,91,92,93,94,95,19,132,133,58,89,90,21,108,109,110,27,28,97,98,33,100,7,8,119,120,22,19,107,42,109,27,28,27,28,95,28,50,51,99,100,101,102,103,104,105,27,28,97,98,107,152,112,132,133,112,65,69,71,72,73,74,75,76,77,78,79,80,81,82,83,11,85,86,87,88,89,90,91,92,93,94,95,19,101,97,97,98,24,101,122,157,12,99,103,112,102,103,104,152,22,97,98,97,98,27,28,113,27,29,91,164,165,124,50,51,97,98,219,59,132,133,134,22,23,45,66,47,212,213,124,140,132,133,19,71,72,73,74,75,76,77,78,79,80,81,82,83,152,85,86,87,88,89,90,91,92,93,94,95,164,165,27,28,230,50,51,233,108,109,110,70,16,59,23,97,98,26,97,22,66,185,12,187,27,28,71,72,73,74,75,76,77,78,79,80,81,82,83,29,85,86,87,88,89,90,91,92,93,94,95,19,22,148,149,45,23,47,62,154,64,156,108,109,110,37,69,23,163,59,26,26,97,98,144,145,146,147,152,200,52,23,50,51,26,22,89,90,60,210,7,8,9,138,97,22,23,26,101,26,174,175,197,71,72,73,74,75,76,77,78,79,80,81,82,83,16,85,86,87,88,89,90,91,92,93,94,95,19,132,133,134,23,152,208,209,140,152,152,111,195,196,98,70,163,160,152,23,22,164,165,246,207,27,152,174,175,171,172,50,51,137,62,139,64,171,172,222,124,27,138,24,163,89,90,130,174,175,197,163,71,72,73,74,75,76,77,78,79,80,81,82,83,22,85,86,87,88,89,90,91,92,93,94,95,19,197,181,182,23,208,209,152,197,26,189,132,133,232,224,225,226,152,97,91,26,232,116,212,213,152,222,121,152,174,175,50,51,243,97,22,23,22,234,174,175,177,23,239,116,163,177,174,175,121,174,175,71,72,73,74,75,76,77,78,79,80,81,82,83,24,85,86,87,88,89,90,91,92,93,94,95,19,23,197,11,23,227,70,208,220,152,31,224,225,226,35,98,224,225,226,108,109,110,115,152,117,118,27,222,49,123,24,50,51,27,0,1,2,224,225,226,166,124,168,169,239,174,175,170,171,172,22,194,71,72,73,74,75,76,77,78,79,80,81,82,83,152,85,86,87,88,89,90,91,92,93,94,95,19,22,208,24,23,195,196,170,171,172,174,175,152,26,152,152,152,207,152,97,152,23,152,51,244,152,97,152,247,248,23,50,51,26,174,175,174,175,174,175,174,175,174,175,174,175,23,174,175,174,175,188,71,72,73,74,75,76,77,78,79,80,81,82,83,152,85,86,87,88,89,90,91,92,93,94,95,19,152,107,152,33,24,152,100,101,27,174,175,152,42,152,23,152,26,152,23,152,26,152,174,175,174,175,152,174,175,23,50,51,26,174,175,174,175,174,175,174,175,174,175,174,175,163,119,120,174,175,19,71,72,73,74,75,76,77,78,79,80,81,82,83,152,85,86,87,88,89,90,91,92,93,94,95,66,152,97,197,23,50,51,26,53,23,174,175,26,23,23,23,26,26,26,36,106,146,147,174,175,19,71,72,73,74,75,76,77,78,79,80,81,82,83,152,85,86,87,88,89,90,91,92,93,94,95,152,196,119,120,19,50,51,168,169,26,174,175,207,28,152,249,250,152,163,163,163,163,174,175,163,19,71,72,73,74,75,76,77,78,79,80,81,82,83,152,85,86,87,88,89,90,91,92,93,94,95,152,197,197,197,197,50,51,197,194,36,174,175,191,192,152,191,192,163,152,66,124,152,174,175,152,19,71,72,73,74,75,76,77,78,79,80,81,82,83,152,85,86,87,88,89,90,91,92,93,94,95,197,152,100,188,152,50,51,152,152,188,174,175,252,152,94,95,152,152,152,1,2,152,152,174,175,19,152,72,73,74,75,76,77,78,79,80,81,82,83,152,85,86,87,88,89,90,91,92,93,94,95,152,188,188,22,194,50,51,240,173,194,174,175,252,194,152,36,181,28,152,23,219,122,174,175,219,221,152,152,73,74,75,76,77,78,79,80,81,82,83,152,85,86,87,88,89,90,91,92,93,94,95,19,20,152,22,23,194,152,240,27,28,174,175,240,19,20,26,22,194,194,38,22,27,28,152,23,22,152,116,174,175,152,23,38,152,23,152,221,152,57,152,23,163,50,51,194,174,175,66,174,175,69,57,174,175,40,174,175,174,175,174,175,174,175,69,22,53,74,75,30,53,89,90,22,22,152,197,23,96,97,98,22,152,101,89,90,91,208,209,152,53,96,97,98,101,22,101,174,175,152,19,20,105,22,174,175,112,19,27,28,20,174,175,24,132,133,134,135,136,38,44,174,175,107,61,54,26,132,133,134,135,136,54,107,22,5,140,1,57,36,111,122,28,79,79,131,123,66,19,20,69,22,1,16,20,125,27,28,123,111,120,23,131,23,16,68,142,38,15,22,89,90,3,167,4,248,251,96,97,98,180,180,101,251,151,6,57,151,13,151,26,25,151,161,202,153,162,153,69,130,128,203,19,20,127,22,126,204,129,22,27,28,205,132,133,134,135,136,89,90,231,38,95,137,179,96,97,98,206,179,101,122,107,159,159,125,231,216,228,107,57,184,217,216,176,217,176,48,106,18,184,158,69,159,158,46,71,237,176,176,176,132,133,134,135,136,217,176,137,216,178,158,89,90,179,176,159,179,159,96,97,98,159,159,101,5,158,202,22,18,10,11,12,13], "i8", ALLOC_NONE, Runtime.GLOBAL_BASE+20481);
/* memory initializer */ allocate([14,190,238,17,190,158,193,41,159,202,193,159,202,245,193,193,223,190,32,159,34,132,133,134,135,136,159,39,155,43,150,223,177,201,178,177,186,66,199,177,152,253,56,215,152,182,152,202,152,63,152,152,66,67,242,229,152,174,152,152,152,152,152,152,152,199,242,202,152,198,152,152,152,183,192,152,215,152,183,215,183,152,241,214,152,211,152,152,211,211,152,152,241,152,152,152,152,152,152,152,114,152,152,235,152,152,152,174,187,95,174,253,253,253,253,236,253,253,253,253,253,253,253,253,253,253,253,141,0,0,0,0,0,0,0,0,0,27,27,27,27,0,27,27,27,0,27,27,27,27,0,0,0,27,0,0,27,0,0,0,27,0,0,0,27,27,27,27,27,27,27,27,27,27,27,27,27,27,27,27,27,27,27,27,27,27,27,27,27,27,27,27,27,27,27,27,27,27,27,27,27,27,27,27,27,0,0,100,97,116,97,98,97,115,101,32,115,99,104,101,109,97,32,105,115,32,108,111,99,107,101,100,58,32,37,115,0,0,0,115,116,97,116,101,109,101,110,116,32,116,111,111,32,108,111,110,103,0,0,0,0,0,0,64,121,0,0,72,121,0,0,80,121,0,0,88,121,0,0,96,121,0,0,104,121,0,0,112,121,0,0,120,121,0,0,128,121,0,0,144,121,0,0,216,83,0,0,152,121,0,0,97,100,100,114,0,0,0,0,111,112,99,111,100,101,0,0,112,49,0,0,0,0,0,0,112,50,0,0,0,0,0,0,112,51,0,0,0,0,0,0,112,52,0,0,0,0,0,0,112,53,0,0,0,0,0,0,99,111,109,109,101,110,116,0,115,101,108,101,99,116,105,100,0,0,0,0,0,0,0,0,111,114,100,101,114,0,0,0,100,101,116,97,105,108,0,0,117,110,111,112,101,110,101,100,0,0,0,0,0,0,0,0,110,111,32,115,117,99,104,32,118,105,101,119,0,0,0,0,110,111,32,115,117,99,104,32,116,97,98,108,101,0,0,0,98,105,110,100,32,111,110,32,97,32,98,117,115,121,32,112,114,101,112,97,114,101,100,32,115,116,97,116,101,109,101,110,116,58,32,91,37,115,93,0,0,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,48,122,0,0,64,122,0,0,72,122,0,0,80,122,0,0,78,79,84,32,78,85,76,76,0,0,0,0,0,0,0,0,85,78,73,81,85,69,0,0,67,72,69,67,75,0,0,0,70,79,82,69,73,71,78,32,75,69,89,0,0,0,0,0,97,98,111,114,116,32,97,116,32,37,100,32,105,110,32,91,37,115,93,58,32,37,115,0,37,115,32,99,111,110,115,116,114,97,105,110,116,32,102,97,105,108,101,100,58,32,37,115,0,0,0,0,0,0,0,0,37,115,32,99,111,110,115,116,114,97,105,110,116,32,102,97,105,108,101,100,0,0,0,0,0,0,0,0,1,2,0,2,2,0,0,0,0,0,0,0,0,1,2,1,1,1,2,1,2,0,0,0,0,0,0,0,99,97,110,110,111,116,32,111,112,101,110,32,115,97,118,101,112,111,105,110,116,32,45,32,83,81,76,32,115,116,97,116,101,109,101,110,116,115,32,105,110,32,112,114,111,103,114,101,115,115,0,0,0,0,0,0,110,111,32,115,117,99,104,32,115,97,118,101,112,111,105,110,116,58,32,37,115,0,0,0,99,97,110,110,111,116,32,114,101,108,101,97,115,101,32,115,97,118,101,112,111,105,110,116,32,45,32,83,81,76,32,115,116,97,116,101,109,101,110,116,115,32,105,110,32,112,114,111,103,114,101,115,115,0,0,0,99,97,110,110,111,116,32,99,111,109,109,105,116,32,116,114,97,110,115,97,99,116,105,111,110,32,45,32,83,81,76,32,115,116,97,116,101,109,101,110,116,115,32,105,110,32,112,114,111,103,114,101,115,115,0,0,99,97,110,110,111,116,32,115,116,97,114,116,32,97,32,116,114,97,110,115,97,99,116,105,111,110,32,119,105,116,104,105,110,32,97,32,116,114,97,110,115,97,99,116,105,111,110,0,99,97,110,110,111,116,32,114,111,108,108,98,97,99,107,32,45,32,110,111,32,116,114,97,110,115,97,99,116,105,111,110,32,105,115,32,97,99,116,105,118,101,0,0,0,0,0,0,99,97,110,110,111,116,32,99,111,109,109,105,116,32,45,32,110,111,32,116,114,97,110,115,97,99,116,105,111,110,32,105,115,32,97,99,116,105,118,101,0,0,0,0,0,0,0,0,83,69,76,69,67,84,32,110,97,109,101,44,32,114,111,111,116,112,97,103,101,44,32,115,113,108,32,70,82,79,77,32,39,37,113,39,46,37,115,32,87,72,69,82,69,32,37,115,32,79,82,68,69,82,32,66,89,32,114,111,119,105,100,0,116,111,111,32,109,97,110,121,32,108,101,118,101,108,115,32,111,102,32,116,114,105,103,103,101,114,32,114,101,99,117,114,115,105,111,110,0,0,0,0,99,97,110,110,111,116,32,99,104,97,110,103,101,32,37,115,32,119,97,108,32,109,111,100,101,32,102,114,111,109,32,119,105,116,104,105,110,32,97,32,116,114,97,110,115,97,99,116,105,111,110,0,0,0,0,0,105,110,116,111,0,0,0,0,111,117,116,32,111,102,0,0,100,97,116,97,98,97,115,101,32,116,97,98,108,101,32,105,115,32,108,111,99,107,101,100,58,32,37,115,0,0,0,0,115,116,97,116,101,109,101,110,116,32,97,98,111,114,116,115,32,97,116,32,37,100,58,32,91,37,115,93,32,37,115,0,45,45,32,0,0,0,0,0,39,37,46,42,113,39,0,0,122,101,114,111,98,108,111,98,40,37,100,41,0,0,0,0,120,39,0,0,0,0,0,0,37,48,50,120,0,0,0,0,39,0,0,0,0,0,0,0,99,97,110,110,111,116,32,86,65,67,85,85,77,32,102,114,111,109,32,119,105,116,104,105,110,32,97,32,116,114,97,110,115,97,99,116,105,111,110,0,99,97,110,110,111,116,32,86,65,67,85,85,77,32,45,32,83,81,76,32,115,116,97,116,101,109,101,110,116,115,32,105,110,32,112,114,111,103,114,101,115,115,0,0,0,0,0,0,65,84,84,65,67,72,32,39,58,109,101,109,111,114,121,58,39,32,65,83,32,118,97,99,117,117,109,95,100,98,59,0,65,84,84,65,67,72,32,39,39,32,65,83,32,118,97,99,117,117,109,95,100,98,59,0,80,82,65,71,77,65,32,118,97,99,117,117,109,95,100,98,46,115,121,110,99,104,114,111,110,111,117,115,61,79,70,70,0,0,0,0,0,0,0,0,66,69,71,73,78,59,0,0,83,69,76,69,67,84,32,39,67,82,69,65,84,69,32,84,65,66,76,69,32,118,97,99,117,117,109,95,100,98,46,39,32,124,124,32,115,117,98,115,116,114,40,115,113,108,44,49,52,41,32,32,32,70,82,79,77,32,115,113,108,105,116,101,95,109,97,115,116,101,114,32,87,72,69,82,69,32,116,121,112,101,61,39,116,97,98,108,101,39,32,65,78,68,32,110,97,109,101,33,61,39,115,113,108,105,116,101,95,115,101,113,117,101,110,99,101,39,32,32,32,65,78,68,32,99,111,97,108,101,115,99,101,40,114,111,111,116,112,97,103,101,44,49,41,62,48,0,0,0,0,0,83,69,76,69,67,84,32,39,67,82,69,65,84,69,32,73,78,68,69,88,32,118,97,99,117,117,109,95,100,98,46,39,32,124,124,32,115,117,98,115,116,114,40,115,113,108,44,49,52,41,32,32,70,82,79,77,32,115,113,108,105,116,101,95,109,97,115,116,101,114,32,87,72,69,82,69,32,115,113,108,32,76,73,75,69,32,39,67,82,69,65,84,69,32,73,78,68,69,88,32,37,39,32,0,83,69,76,69,67,84,32,39,67,82,69,65,84,69,32,85,78,73,81,85,69,32,73,78,68,69,88,32,118,97,99,117,117,109,95,100,98,46,39,32,124,124,32,115,117,98,115,116,114,40,115,113,108,44,50,49,41,32,32,32,70,82,79,77,32,115,113,108,105,116,101,95,109,97,115,116,101,114,32,87,72,69,82,69,32,115,113,108,32,76,73,75,69,32,39,67,82,69,65,84,69,32,85,78,73,81,85,69,32,73,78,68,69,88,32,37,39,0,0,0,83,69,76,69,67,84,32,39,73,78,83,69,82,84,32,73,78,84,79,32,118,97,99,117,117,109,95,100,98,46,39,32,124,124,32,113,117,111,116,101,40,110,97,109,101,41,32,124,124,32,39,32,83,69,76,69,67,84,32,42,32,70,82,79,77,32,109,97,105,110,46,39,32,124,124,32,113,117,111,116,101,40,110,97,109,101,41,32,124,124,32,39,59,39,70,82,79,77,32,109,97,105,110,46,115,113,108,105,116,101,95,109,97,115,116,101,114,32,87,72,69,82,69,32,116,121,112,101,32,61,32,39,116,97,98,108,101,39,32,65,78,68,32,110,97,109,101,33,61,39,115,113,108,105,116,101,95,115,101,113,117,101,110,99,101,39,32,32,32,65,78,68,32,99,111,97,108,101,115,99,101,40,114,111,111,116,112,97,103,101,44,49,41,62,48,0,0,0,0,0,83,69,76,69,67,84,32,39,68,69,76,69,84,69,32,70,82,79,77,32,118,97,99,117,117,109,95,100,98,46,39,32,124,124,32,113,117,111,116,101,40,110,97,109,101,41,32,124,124,32,39,59,39,32,70,82,79,77,32,118,97,99,117,117,109,95,100,98,46,115,113,108,105,116,101,95,109,97,115,116,101,114,32,87,72,69,82,69,32,110,97,109,101,61,39,115,113,108,105,116,101,95,115,101,113,117,101,110,99,101,39,32,0,0,0,0,0,0,0,0,83,69,76,69,67,84,32,39,73,78,83,69,82,84,32,73,78,84,79,32,118,97,99,117,117,109,95,100,98,46,39,32,124,124,32,113,117,111,116,101,40,110,97,109,101,41,32,124,124,32,39,32,83,69,76,69,67,84,32,42,32,70,82,79,77,32,109,97,105,110,46,39,32,124,124,32,113,117,111,116,101,40,110,97,109,101,41,32,124,124,32,39,59,39,32,70,82,79,77,32,118,97,99,117,117,109,95,100,98,46,115,113,108,105,116,101,95,109,97,115,116,101,114,32,87,72,69,82,69,32,110,97,109,101,61,61,39,115,113,108,105,116,101,95,115,101,113,117,101,110,99,101,39,59,0,0,0,0,0,0,73,78,83,69,82,84,32,73,78,84,79,32,118,97,99,117,117,109,95,100,98,46,115,113,108,105,116,101,95,109,97,115,116,101,114,32,32,32,83,69,76,69,67,84,32,116,121,112,101,44,32,110,97,109,101,44,32,116,98,108,95,110,97,109,101,44,32,114,111,111,116,112,97,103,101,44,32,115,113,108,32,32,32,32,70,82,79,77,32,109,97,105,110,46,115,113,108,105,116,101,95,109,97,115,116,101,114,32,32,32,87,72,69,82,69,32,116,121,112,101,61,39,118,105,101,119,39,32,79,82,32,116,121,112,101,61,39,116,114,105,103,103,101,114,39,32,32,32,32,32,32,79,82,32,40,116,121,112,101,61,39,116,97,98,108,101,39,32,65,78,68,32,114,111,111,116,112,97,103,101,61,48,41,0,1,1,3,0,5,0,6,0,8,0,0,0,0,0,0,0,77,97,105,110,32,102,114,101,101,108,105,115,116,58,32,0,76,105,115,116,32,111,102,32,116,114,101,101,32,114,111,111,116,115,58,32,0,0,0,0,80,97,103,101,32,37,100,32,105,115,32,110,101,118,101,114,32,117,115,101,100,0,0,0,80,111,105,110,116,101,114,32,109,97,112,32,112,97,103,101,32,37,100,32,105,115,32,114,101,102,101,114,101,110,99,101,100,0,0,0,0,0,0,0,79,117,116,115,116,97,110,100,105,110,103,32,112,97,103,101,32,99,111,117,110,116,32,103,111,101,115,32,102,114,111,109,32,37,100,32,116,111,32,37,100,32,100,117,114,105,110,103,32,116,104,105,115,32,97,110,97,108,121,115,105,115,0,0,10,0,0,0,0,0,0,0,80,97,103,101,32,37,100,58,32,0,0,0,0,0,0,0,117,110,97,98,108,101,32,116,111,32,103,101,116,32,116,104,101,32,112,97,103,101,46,32,101,114,114,111,114,32,99,111,100,101,61,37,100,0,0,0,98,116,114,101,101,73,110,105,116,80,97,103,101,40,41,32,114,101,116,117,114,110,115,32,101,114,114,111,114,32,99,111,100,101,32,37,100,0,0,0,79,110,32,116,114,101,101,32,112,97,103,101,32,37,100,32,99,101,108,108,32,37,100,58,32,0,0,0,0,0,0,0,82,111,119,105,100,32,37,108,108,100,32,111,117,116,32,111,102,32,111,114,100,101,114,32,40,112,114,101,118,105,111,117,115,32,119,97,115,32,37,108,108,100,41,0,0,0,0,0,67,104,105,108,100,32,112,97,103,101,32,100,101,112,116,104,32,100,105,102,102,101,114,115,0,0,0,0,0,0,0,0,79,110,32,112,97,103,101,32,37,100,32,97,116,32,114,105,103,104,116,32,99,104,105,108,100,58,32,0,0,0,0,0,82,111,119,105,100,32,37,108,108,100,32,111,117,116,32,111,102,32,111,114,100,101,114,32,40,109,97,120,32,108,97,114,103,101,114,32,116,104,97,110,32,112,97,114,101,110,116,32,109,105,110,32,111,102,32,37,108,108,100,41,0,0,0,0,82,111,119,105,100,32,37,108,108,100,32,111,117,116,32,111,102,32,111,114,100,101,114,32,40,109,105,110,32,108,101,115,115,32,116,104,97,110,32,112,97,114,101,110,116,32,109,105,110,32,111,102,32,37,108,108,100,41,0,0,0,0,0,0,82,111,119,105,100,32,37,108,108,100,32,111,117,116,32,111,102,32,111,114,100,101,114,32,40,109,97,120,32,108,97,114,103,101,114,32,116,104,97,110,32,112,97,114,101,110,116,32,109,97,120,32,111,102,32,37,108,108,100,41,0,0,0,0,82,111,119,105,100,32,37,108,108,100,32,111,117,116,32,111,102,32,111,114,100,101,114,32,40,109,105,110,32,108,101,115,115,32,116,104,97,110,32,112,97,114,101,110,116,32,109,97,120,32,111,102,32,37,108,108,100,41,0,0,0,0,0,0,67,111,114,114,117,112,116,105,111,110,32,100,101,116,101,99,116,101,100,32,105,110,32,99,101,108,108,32,37,100,32,111,110,32,112,97,103,101,32,37,100,0,0,0,0,0,0,0,77,117,108,116,105,112,108,101,32,117,115,101,115,32,102,111,114,32,98,121,116,101,32,37,100,32,111,102,32,112,97,103,101,32,37,100,0,0,0,0,70,114,97,103,109,101,110,116,97,116,105,111,110,32,111,102,32,37,100,32,98,121,116,101,115,32,114,101,112,111,114,116,101,100,32,97,115,32,37,100,32,111,110,32,112,97,103,101,32,37,100,0,0,0,0,0,105,110,118,97,108,105,100,32,112,97,103,101,32,110,117,109,98,101,114,32,37,100,0,0,50,110,100,32,114,101,102,101,114,101,110,99,101,32,116,111,32,112,97,103,101,32,37,100,0,0,0,0,0,0,0,0,70,97,105,108,101,100,32,116,111,32,114,101,97,100,32,112,116,114,109,97,112,32,107,101,121,61,37,100,0,0,0,0,66,97,100,32,112,116,114,32,109,97,112,32,101,110,116,114,121,32,107,101,121,61,37,100,32,101,120,112,101,99,116,101,100,61,40,37,100,44,37,100,41,32,103,111,116,61,40,37,100,44,37,100,41,0,0,0,37,100,32,111,102,32,37,100,32,112,97,103,101,115,32,109,105,115,115,105,110,103,32,102,114,111,109,32,111,118,101,114,102,108,111,119,32,108,105,115,116,32,115,116,97,114,116,105,110,103,32,97,116,32,37,100,0,0,0,0,0,0,0,0,102,97,105,108,101,100,32,116,111,32,103,101,116,32,112,97,103,101,32,37,100,0,0,0,102,114,101,101,108,105,115,116,32,108,101,97,102,32,99,111,117,110,116,32,116,111,111,32,98,105,103,32,111,110,32,112,97,103,101,32,37,100,0,0,83,81,76,105,116,101,32,102,111,114,109,97,116,32,51,0,37,115,45,109,106,88,88,88,88,88,88,57,88,88,122,0,77,74,32,100,101,108,101,116,101,58,32,37,115,0,0,0,77,74,32,99,111,108,108,105,100,101,58,32,37,115,0,0,45,109,106,37,48,54,88,57,37,48,50,88,0,0,0,0,37,46,50,120,0,0,0,0,107,40,37,100,0,0,0,0,110,105,108,0,0,0,0,0,66,0,0,0,0,0,0,0,44,46,46,46,0,0,0,0,40,37,46,50,48,115,41,0,37,115,40,37,100,41,0,0,37,100,0,0,0,0,0,0,40,98,108,111,98,41,0,0,118,116,97,98,58,37,112,58,37,112,0,0,0,0,0,0,105,110,116,97,114,114,97,121,0,0,0,0,0,0,0,0,112,114,111,103,114,97,109,0,72,74,0,0,152,137,0,0,168,137,0,0,184,137,0,0,200,137,0,0,216,137,0,0,232,137,0,0,248,137,0,0,8,138,0,0,16,138,0,0,24,138,0,0,32,138,0,0,48,138,0,0,64,138,0,0,72,138,0,0,80,138,0,0,88,138,0,0,96,138,0,0,104,138,0,0,112,138,0,0,120,138,0,0,136,138,0,0,152,138,0,0,160,138,0,0,176,138,0,0,184,138,0,0,192,138,0,0,200,138,0,0,208,138,0,0,216,138,0,0,232,138,0,0,240,138,0,0,0,139,0,0,8,139,0,0,16,139,0,0,24,139,0,0,40,139,0,0,48,139,0,0,56,139,0,0,72,139,0,0,88,139,0,0,96,139,0,0,112,139,0,0,120,139,0,0,128,139,0,0,136,139,0,0,144,139,0,0,152,139,0,0,160,139,0,0,176,139,0,0,192,139,0,0,200,139,0,0,216,139,0,0,232,139,0,0,248,139,0,0,8,140,0,0,24,140,0,0,40,140,0,0,56,140,0,0,72,140,0,0,88,140,0,0,104,140,0,0,112,140,0,0,120,140,0,0,128,140,0,0,136,140,0,0,144,140,0,0,152,140,0,0,168,140,0,0,184,140,0,0,192,140,0,0,208,140,0,0,216,140,0,0,224,140,0,0,240,140,0,0,0,141,0,0,8,141,0,0,16,141,0,0,24,141,0,0,32,141,0,0,40,141,0,0,48,141,0,0,56,141,0,0,64,141,0,0,72,141,0,0,88,141,0,0,96,141,0,0,104,141,0,0,120,141,0,0,136,141,0,0,144,141,0,0,160,141,0,0,176,141,0,0,184,141,0,0,200,141,0,0,208,141,0,0,216,141,0,0,224,141,0,0,232,141,0,0,248,141,0,0,8,142,0,0,24,142,0,0,32,142,0,0,40,142,0,0,48,142,0,0,56,142,0,0,64,142,0,0,80,142,0,0,88,142,0,0,96,142,0,0,112,142,0,0,128,142,0,0,144,142,0,0,160,142,0,0,168,142,0,0,176,142,0,0,184,142,0,0,192,142,0,0,200,142,0,0,208,142,0,0,224,142,0,0,240,142,0,0,0,143,0,0,16,143,0,0,32,143,0,0,48,143,0,0,64,143,0,0,80,143,0,0,96,143,0,0,112,143,0,0,128,143,0,0,144,143,0,0,152,143,0,0,160,143,0,0,168,143,0,0,184,143,0,0,200,143,0,0,208,143,0,0,216,143,0,0,224,143,0,0,232,143,0,0,248,143,0,0,8,144,0,0,16,144,0,0,32,144,0,0,40,144,0,0,48,144,0,0,64,144,0,0,72,144,0,0,80,144,0,0,88,144,0,0,96,144,0,0,112,144,0,0,128,144,0,0,136,144,0,0,144,144,0,0,70,117,110,99,116,105,111,110,0,0,0,0,0,0,0,0,83,97,118,101,112,111,105,110,116,0,0,0,0,0,0,0,65,117,116,111,67,111,109,109,105,116,0,0,0,0,0,0,84,114,97,110,115,97,99,116,105,111,110,0,0,0,0,0,83,111,114,116,101,114,78,101,120,116,0,0,0,0,0,0,80,114,101,118,73,102,79,112,101,110,0,0,0,0,0,0,78,101,120,116,73,102,79,112,101,110,0,0,0,0,0,0,80,114,101,118,0,0,0,0,78,101,120,116,0,0,0,0,65,103,103,83,116,101,112,0,67,104,101,99,107,112,111,105,110,116,0,0,0,0,0,0,74,111,117,114,110,97,108,77,111,100,101,0,0,0,0,0,86,97,99,117,117,109,0,0,86,70,105,108,116,101,114,0,86,85,112,100,97,116,101,0,71,111,116,111,0,0,0,0,71,111,115,117,98,0,0,0,82,101,116,117,114,110,0,0,78,111,116,0,0,0,0,0,73,110,105,116,67,111,114,111,117,116,105,110,101,0,0,0,69,110,100,67,111,114,111,117,116,105,110,101,0,0,0,0,89,105,101,108,100,0,0,0,72,97,108,116,73,102,78,117,108,108,0,0,0,0,0,0,72,97,108,116,0,0,0,0,73,110,116,101,103,101,114,0,73,110,116,54,52,0,0,0,83,116,114,105,110,103,0,0,78,117,108,108,0,0,0,0,83,111,102,116,78,117,108,108,0,0,0,0,0,0,0,0,66,108,111,98,0,0,0,0,86,97,114,105,97,98,108,101,0,0,0,0,0,0,0,0,77,111,118,101,0,0,0,0,67,111,112,121,0,0,0,0,83,67,111,112,121,0,0,0,82,101,115,117,108,116,82,111,119,0,0,0,0,0,0,0,67,111,108,108,83,101,113,0,65,100,100,73,109,109,0,0,77,117,115,116,66,101,73,110,116,0,0,0,0,0,0,0,82,101,97,108,65,102,102,105,110,105,116,121,0,0,0,0,67,97,115,116,0,0,0,0,80,101,114,109,117,116,97,116,105,111,110,0,0,0,0,0,67,111,109,112,97,114,101,0,74,117,109,112,0,0,0,0,79,110,99,101,0,0,0,0,73,102,0,0,0,0,0,0,73,102,78,111,116,0,0,0,67,111,108,117,109,110,0,0,65,102,102,105,110,105,116,121,0,0,0,0,0,0,0,0,77,97,107,101,82,101,99,111,114,100,0,0,0,0,0,0,67,111,117,110,116,0,0,0,82,101,97,100,67,111,111,107,105,101,0,0,0,0,0,0,83,101,116,67,111,111,107,105,101,0,0,0,0,0,0,0,82,101,111,112,101,110,73,100,120,0,0,0,0,0,0,0,79,112,101,110,82,101,97,100,0,0,0,0,0,0,0,0,79,112,101,110,87,114,105,116,101,0,0,0,0,0,0,0,79,112,101,110,65,117,116,111,105,110,100,101,120,0,0,0,79,112,101,110,69,112,104,101,109,101,114,97,108,0,0,0,83,111,114,116,101,114,79,112,101,110,0,0,0,0,0,0,83,101,113,117,101,110,99,101,84,101,115,116,0,0,0,0,79,112,101,110,80,115,101,117,100,111,0,0,0,0,0,0,67,108,111,115,101,0,0,0,83,101,101,107,76,84,0,0,83,101,101,107,76,69,0,0,83,101,101,107,71,69,0,0,83,101,101,107,71,84,0,0,83,101,101,107,0,0,0,0,78,111,67,111,110,102,108,105,99,116,0,0,0,0,0,0,78,111,116,70,111,117,110,100,0,0,0,0,0,0,0,0,70,111,117,110,100,0,0,0,78,111,116,69,120,105,115,116,115,0,0,0,0,0,0,0,79,114,0,0,0,0,0,0,65,110,100,0,0,0,0,0,83,101,113,117,101,110,99,101,0,0,0,0,0,0,0,0,78,101,119,82,111,119,105,100,0,0,0,0,0,0,0,0,73,110,115,101,114,116,0,0,73,115,78,117,108,108,0,0,78,111,116,78,117,108,108,0,78,101,0,0,0,0,0,0,69,113,0,0,0,0,0,0,71,116,0,0,0,0,0,0,76,101,0,0,0,0,0,0,76,116,0,0,0,0,0,0,71,101,0,0,0,0,0,0,73,110,115,101,114,116,73,110,116,0,0,0,0,0,0,0,66,105,116,65,110,100,0,0,66,105,116,79,114,0,0,0,83,104,105,102,116,76,101,102,116,0,0,0,0,0,0,0,83,104,105,102,116,82,105,103,104,116,0,0,0,0,0,0,65,100,100,0,0,0,0,0,83,117,98,116,114,97,99,116,0,0,0,0,0,0,0,0,77,117,108,116,105,112,108,121,0,0,0,0,0,0,0,0,68,105,118,105,100,101,0,0,82,101,109,97,105,110,100,101,114,0,0,0,0,0,0,0,67,111,110,99,97,116,0,0,68,101,108,101,116,101,0,0,66,105,116,78,111,116,0,0,83,116,114,105,110,103,56,0,82,101,115,101,116,67,111,117,110,116,0,0,0,0,0,0,83,111,114,116,101,114,67,111,109,112,97,114,101,0,0,0,83,111,114,116,101,114,68,97,116,97,0,0,0,0,0,0,82,111,119,75,101,121,0,0,82,111,119,68,97,116,97,0,82,111,119,105,100,0,0,0,78,117,108,108,82,111,119,0,76,97,115,116,0,0,0,0,83,111,114,116,101,114,83,111,114,116,0,0,0,0,0,0,83,111,114,116,0,0,0,0,82,101,119,105,110,100,0,0,83,111,114,116,101,114,73,110,115,101,114,116,0,0,0,0,73,100,120,73,110,115,101,114,116,0,0,0,0,0,0,0,73,100,120,68,101,108,101,116,101,0,0,0,0,0,0,0,73,100,120,82,111,119,105,100,0,0,0,0,0,0,0,0,73,100,120,76,69,0,0,0,73,100,120,71,84,0,0,0,73,100,120,76,84,0,0,0,73,100,120,71,69,0,0,0,68,101,115,116,114,111,121,0,67,108,101,97,114,0,0,0,82,101,115,101,116,83,111,114,116,101,114,0,0,0,0,0,67,114,101,97,116,101,73,110,100,101,120,0,0,0,0,0,67,114,101,97,116,101,84,97,98,108,101,0,0,0,0,0,80,97,114,115,101,83,99,104,101,109,97,0,0,0,0,0,76,111,97,100,65,110,97,108,121,115,105,115,0,0,0,0,68,114,111,112,84,97,98,108,101,0,0,0,0,0,0,0,68,114,111,112,73,110,100,101,120,0,0,0,0,0,0,0,68,114,111,112,84,114,105,103,103,101,114,0,0,0,0,0,73,110,116,101,103,114,105,116,121,67,107,0,0,0,0,0,82,111,119,83,101,116,65,100,100,0,0,0,0,0,0,0,82,111,119,83,101,116,82,101,97,100,0,0,0,0,0,0,82,111,119,83,101,116,84,101,115,116,0,0,0,0,0,0,80,114,111,103,114,97,109,0,80,97,114,97,109,0,0,0,82,101,97,108,0,0,0,0,70,107,67,111,117,110,116,101,114,0,0,0,0,0,0,0,70,107,73,102,90,101,114,111,0,0,0,0,0,0,0,0,77,101,109,77,97,120,0,0,73,102,80,111,115,0,0,0,73,102,78,101,103,0,0,0,73,102,90,101,114,111,0,0,65,103,103,70,105,110,97,108,0,0,0,0,0,0,0,0,73,110,99,114,86,97,99,117,117,109,0,0,0,0,0,0,69,120,112,105,114,101,0,0,84,97,98,108,101,76,111,99,107,0,0,0,0,0,0,0,86,66,101,103,105,110,0,0,86,67,114,101,97,116,101,0,86,68,101,115,116,114,111,121,0,0,0,0,0,0,0,0,86,79,112,101,110,0,0,0,86,67,111,108,117,109,110,0,86,78,101,120,116,0,0,0,86,82,101,110,97,109,101,0,80,97,103,101,99,111,117,110,116,0,0,0,0,0,0,0,77,97,120,80,103,99,110,116,0,0,0,0,0,0,0,0,73,110,105,116,0,0,0,0,78,111,111,112,0,0,0,0,69,120,112,108,97,105,110,0,65,80,73,32,99,97,108,108,101,100,32,119,105,116,104,32,78,85,76,76,32,112,114,101,112,97,114,101,100,32,115,116,97,116,101,109,101,110,116,0,65,80,73,32,99,97,108,108,101,100,32,119,105,116,104,32,102,105,110,97,108,105,122,101,100,32,112,114,101,112,97,114,101,100,32,115,116,97,116,101,109,101,110,116,0,0,0,0,64,32,32,0,0,0,0,0,1,0,0,0,8,0,0,0,4,0,0,0,5,0,0,0,8,0,0,0,12,0,0,0,13,0,0,0,14,0,0,0,15,0,0,0,16,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,0,0,0,0,3,0,0,0,12,0,0,0,17,0,0,0,6,0,0,0,4,0,0,0,0,0,0,0,160,146,0,0,10,0,0,0,0,0,0,0,120,146,0,0,11,0,0,0,0,0,0,0,192,18,0,0,18,0,0,0,0,0,0,0,168,146,0,0,19,0,0,0,0,0,0,0,176,146,0,0,20,0,0,0,0,0,0,0,184,146,0,0,21,0,0,0,0,0,0,0,192,146,0,0,22,0,0,0,0,0,0,0,208,146,0,0,11,0,0,0,0,0,0,0,216,146,0,0,12,0,0,0,0,0,0,0,224,146,0,0,0,0,0,0,0,0,0,0,232,146,0,0,0,0,0,0,0,0,0,0,240,146,0,0,13,0,0,0,0,0,0,0,248,146,0,0,0,0,0,0,0,0,0,0,0,147,0,0,0,0,0,0,0,0,0,0,16,147,0,0,23,0,0,0,0,0,0,0,24,147,0,0,0,0,0,0,0,0,0,0,40,147,0,0,12,0,0,0,0,0,0,0,48,147,0,0,24,0,0,0,0,0,0,0,64,147,0,0,25,0,0,0,0,0,0,0,72,147,0,0,13,0,0,0,0,0,0,0,80,147,0,0,14,0,0,0,0,0,0,0,88,147,0,0,1,0,0,0,0,0,0,0,96,147,0,0,26,0,0,0,0,0,0,0,104,147,0,0,0,0,0,0,0,0,0,0,112,147,0,0,1,0,0,0,0,0,0,0,0,0,0,0,99,108,111,115,101,0,0,0,111,115,95,117,110,105,120,46,99,58,37,100,58,32,40,37,100,41,32,37,115,40,37,115,41,32,45,32,37,115,0,0,111,112,101,110,0,0,0,0,103,101,116,99,119,100,0,0,115,116,97,116,0,0,0,0,102,115,116,97,116,0,0,0,102,116,114,117,110,99,97,116,101,0,0,0,0,0,0,0,102,99,110,116,108,0,0,0,114,101,97,100,0,0,0,0,112,114,101,97,100,0,0,0,112,114,101,97,100,54,52,0,119,114,105,116,101,0,0,0,112,119,114,105,116,101,0,0,112,119,114,105,116,101,54,52,0,0,0,0,0,0,0,0,102,99,104,109,111,100,0,0,102,97,108,108,111,99,97,116,101,0,0,0,0,0,0,0,117,110,108,105,110,107,0,0,111,112,101,110,68,105,114,101,99,116,111,114,121,0,0,0,109,107,100,105,114,0,0,0,114,109,100,105,114,0,0,0,102,99,104,111,119,110,0,0,109,109,97,112,0,0,0,0,109,117,110,109,97,112,0,0,109,114,101,109,97,112,0,0,103,101,116,112,97,103,101,115,105,122,101,0,0,0,0,0,97,116,116,101,109,112,116,32,116,111,32,111,112,101,110,32,34,37,115,34,32,97,115,32,102,105,108,101,32,100,101,115,99,114,105,112,116,111,114,32,37,100,0,0,0,0,0,0,47,100,101,118,47,110,117,108,108,0,0,0,0,0,0,0,97,98,99,100,101,102,103,104,105,106,107,108,109,110,111,112,113,114,115,116,117,118,119,120,121,122,65,66,67,68,69,70,71,72,73,74,75,76,77,78,79,80,81,82,83,84,85,86,87,88,89,90,48,49,50,51,52,53,54,55,56,57,0,0,37,115,47,101,116,105,108,113,115,95,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,48,148,0,0,64,148,0,0,80,148,0,0,0,0,0,0,0,0,0,0,47,118,97,114,47,116,109,112,0,0,0,0,0,0,0,0,47,117,115,114,47,116,109,112,0,0,0,0,0,0,0,0,47,116,109,112,0,0,0,0,83,81,76,73,84,69,95,84,77,80,68,73,82,0,0,0,84,77,80,68,73,82,0,0,102,117,108,108,95,102,115,121,110,99,0,0,0,0,0,0,3,0,0,0,14,0,0,0,4,0,0,0,5,0,0,0,8,0,0,0,12,0,0,0,13,0,0,0,27,0,0,0,28,0,0,0,29,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,0,0,0,0,3,0,0,0,12,0,0,0,17,0,0,0,6,0,0,0,4,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,47,100,101,118,47,117,114,97,110,100,111,109,0,0,0,0,47,37,115,0,0,0,0,0,102,115,121,110,99,0,0,0,112,115,111,119,0,0,0,0,3,0,0,0,15,0,0,0,4,0,0,0,5,0,0,0,8,0,0,0,12,0,0,0,13,0,0,0,30,0,0,0,31,0,0,0,32,0,0,0,9,0,0,0,9,0,0,0,10,0,0,0,7,0,0,0,3,0,0,0,12,0,0,0,17,0,0,0,6,0,0,0,4,0,0,0,0,0,0,0,37,115,46,108,111,99,107,0,99,97,110,110,111,116,32,102,115,116,97,116,32,100,98,32,102,105,108,101,32,37,115,0,102,105,108,101,32,117,110,108,105,110,107,101,100,32,119,104,105,108,101,32,111,112,101,110,58,32,37,115,0,0,0,0,109,117,108,116,105,112,108,101,32,108,105,110,107,115,32,116,111,32,102,105,108,101,58,32,37,115,0,0,0,0,0,0,102,105,108,101,32,114,101,110,97,109,101,100,32,119,104,105,108,101,32,111,112,101,110,58,32,37,115,0,0,0,0,0,0,0,0,0,0,0,0,0,37,115,45,115,104,109,0,0,114,101,97,100,111,110,108,121,95,115,104,109,0,0,0,0,37,0,0,0,0,0,0,0,100,10,1,1,0,0,115,0,4,6,0,0,103,0,1,4,30,0,122,0,4,7,0,0,113,0,4,10,0,0,81,0,4,11,0,0,119,0,4,15,0,0,99,0,0,9,0,0,111,8,0,1,0,2,117,10,0,1,0,0,120,16,0,1,16,1,88,16,0,1,0,4,102,0,1,2,0,0,101,0,1,3,30,0,69,0,1,3,14,0,71,0,1,4,14,0,105,10,1,1,0,0,110,0,0,5,0,0,37,0,0,8,0,0,112,16,0,14,0,1,84,0,2,12,0,0,83,0,2,13,0,0,114,10,3,16,0,0,0,0,0,0,0,0,116,104,115,116,110,100,114,100,0,0,0,0,0,0,0,0,48,49,50,51,52,53,54,55,56,57,65,66,67,68,69,70,48,49,50,51,52,53,54,55,56,57,97,98,99,100,101,102,0,0,0,0,0,0,0,0,45,120,48,0,88,48,0,0,78,97,78,0,0,0,0,0,45,73,110,102,0,0,0,0,43,73,110,102,0,0,0,0,73,110,102,0,0,0,0,0,40,78,85,76,76,41,0,0,1,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,254,255,255,127,0,0,0,0,128,0,0,0,244,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,68,73,83,65,66,76,69,95,76,70,83,0,0,0,0,0,73,78,84,54,52,95,84,89,80,69,0,0,0,0,0,0,79,77,73,84,95,76,79,65,68,95,69,88,84,69,78,83,73,79,78,0,0,0,0,0,83,89,83,84,69,77,95,77,65,76,76,79,67,0,0,0,84,72,82,69,65,68,83,65,70,69,61,48,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0], "i8", ALLOC_NONE, Runtime.GLOBAL_BASE+30721);




var tempDoublePtr = Runtime.alignMemory(allocate(12, "i8", ALLOC_STATIC), 8);

assert(tempDoublePtr % 8 == 0);

function copyTempFloat(ptr) { // functions, because inlining this code increases code size too much

  HEAP8[tempDoublePtr] = HEAP8[ptr];

  HEAP8[tempDoublePtr+1] = HEAP8[ptr+1];

  HEAP8[tempDoublePtr+2] = HEAP8[ptr+2];

  HEAP8[tempDoublePtr+3] = HEAP8[ptr+3];

}

function copyTempDouble(ptr) {

  HEAP8[tempDoublePtr] = HEAP8[ptr];

  HEAP8[tempDoublePtr+1] = HEAP8[ptr+1];

  HEAP8[tempDoublePtr+2] = HEAP8[ptr+2];

  HEAP8[tempDoublePtr+3] = HEAP8[ptr+3];

  HEAP8[tempDoublePtr+4] = HEAP8[ptr+4];

  HEAP8[tempDoublePtr+5] = HEAP8[ptr+5];

  HEAP8[tempDoublePtr+6] = HEAP8[ptr+6];

  HEAP8[tempDoublePtr+7] = HEAP8[ptr+7];

}


  
  
  var ERRNO_CODES={EPERM:1,ENOENT:2,ESRCH:3,EINTR:4,EIO:5,ENXIO:6,E2BIG:7,ENOEXEC:8,EBADF:9,ECHILD:10,EAGAIN:11,EWOULDBLOCK:11,ENOMEM:12,EACCES:13,EFAULT:14,ENOTBLK:15,EBUSY:16,EEXIST:17,EXDEV:18,ENODEV:19,ENOTDIR:20,EISDIR:21,EINVAL:22,ENFILE:23,EMFILE:24,ENOTTY:25,ETXTBSY:26,EFBIG:27,ENOSPC:28,ESPIPE:29,EROFS:30,EMLINK:31,EPIPE:32,EDOM:33,ERANGE:34,ENOMSG:42,EIDRM:43,ECHRNG:44,EL2NSYNC:45,EL3HLT:46,EL3RST:47,ELNRNG:48,EUNATCH:49,ENOCSI:50,EL2HLT:51,EDEADLK:35,ENOLCK:37,EBADE:52,EBADR:53,EXFULL:54,ENOANO:55,EBADRQC:56,EBADSLT:57,EDEADLOCK:35,EBFONT:59,ENOSTR:60,ENODATA:61,ETIME:62,ENOSR:63,ENONET:64,ENOPKG:65,EREMOTE:66,ENOLINK:67,EADV:68,ESRMNT:69,ECOMM:70,EPROTO:71,EMULTIHOP:72,EDOTDOT:73,EBADMSG:74,ENOTUNIQ:76,EBADFD:77,EREMCHG:78,ELIBACC:79,ELIBBAD:80,ELIBSCN:81,ELIBMAX:82,ELIBEXEC:83,ENOSYS:38,ENOTEMPTY:39,ENAMETOOLONG:36,ELOOP:40,EOPNOTSUPP:95,EPFNOSUPPORT:96,ECONNRESET:104,ENOBUFS:105,EAFNOSUPPORT:97,EPROTOTYPE:91,ENOTSOCK:88,ENOPROTOOPT:92,ESHUTDOWN:108,ECONNREFUSED:111,EADDRINUSE:98,ECONNABORTED:103,ENETUNREACH:101,ENETDOWN:100,ETIMEDOUT:110,EHOSTDOWN:112,EHOSTUNREACH:113,EINPROGRESS:115,EALREADY:114,EDESTADDRREQ:89,EMSGSIZE:90,EPROTONOSUPPORT:93,ESOCKTNOSUPPORT:94,EADDRNOTAVAIL:99,ENETRESET:102,EISCONN:106,ENOTCONN:107,ETOOMANYREFS:109,EUSERS:87,EDQUOT:122,ESTALE:116,ENOTSUP:95,ENOMEDIUM:123,EILSEQ:84,EOVERFLOW:75,ECANCELED:125,ENOTRECOVERABLE:131,EOWNERDEAD:130,ESTRPIPE:86};
  
  var ERRNO_MESSAGES={0:"Success",1:"Not super-user",2:"No such file or directory",3:"No such process",4:"Interrupted system call",5:"I/O error",6:"No such device or address",7:"Arg list too long",8:"Exec format error",9:"Bad file number",10:"No children",11:"No more processes",12:"Not enough core",13:"Permission denied",14:"Bad address",15:"Block device required",16:"Mount device busy",17:"File exists",18:"Cross-device link",19:"No such device",20:"Not a directory",21:"Is a directory",22:"Invalid argument",23:"Too many open files in system",24:"Too many open files",25:"Not a typewriter",26:"Text file busy",27:"File too large",28:"No space left on device",29:"Illegal seek",30:"Read only file system",31:"Too many links",32:"Broken pipe",33:"Math arg out of domain of func",34:"Math result not representable",35:"File locking deadlock error",36:"File or path name too long",37:"No record locks available",38:"Function not implemented",39:"Directory not empty",40:"Too many symbolic links",42:"No message of desired type",43:"Identifier removed",44:"Channel number out of range",45:"Level 2 not synchronized",46:"Level 3 halted",47:"Level 3 reset",48:"Link number out of range",49:"Protocol driver not attached",50:"No CSI structure available",51:"Level 2 halted",52:"Invalid exchange",53:"Invalid request descriptor",54:"Exchange full",55:"No anode",56:"Invalid request code",57:"Invalid slot",59:"Bad font file fmt",60:"Device not a stream",61:"No data (for no delay io)",62:"Timer expired",63:"Out of streams resources",64:"Machine is not on the network",65:"Package not installed",66:"The object is remote",67:"The link has been severed",68:"Advertise error",69:"Srmount error",70:"Communication error on send",71:"Protocol error",72:"Multihop attempted",73:"Cross mount point (not really error)",74:"Trying to read unreadable message",75:"Value too large for defined data type",76:"Given log. name not unique",77:"f.d. invalid for this operation",78:"Remote address changed",79:"Can   access a needed shared lib",80:"Accessing a corrupted shared lib",81:".lib section in a.out corrupted",82:"Attempting to link in too many libs",83:"Attempting to exec a shared library",84:"Illegal byte sequence",86:"Streams pipe error",87:"Too many users",88:"Socket operation on non-socket",89:"Destination address required",90:"Message too long",91:"Protocol wrong type for socket",92:"Protocol not available",93:"Unknown protocol",94:"Socket type not supported",95:"Not supported",96:"Protocol family not supported",97:"Address family not supported by protocol family",98:"Address already in use",99:"Address not available",100:"Network interface is not configured",101:"Network is unreachable",102:"Connection reset by network",103:"Connection aborted",104:"Connection reset by peer",105:"No buffer space available",106:"Socket is already connected",107:"Socket is not connected",108:"Can't send after socket shutdown",109:"Too many references",110:"Connection timed out",111:"Connection refused",112:"Host is down",113:"Host is unreachable",114:"Socket already connected",115:"Connection already in progress",116:"Stale file handle",122:"Quota exceeded",123:"No medium (in tape drive)",125:"Operation canceled",130:"Previous owner died",131:"State not recoverable"};
  
  
  var ___errno_state=0;function ___setErrNo(value) {
      // For convenient setting and returning of errno.
      HEAP32[((___errno_state)>>2)]=value;
      return value;
    }
  
  var PATH={splitPath:function (filename) {
        var splitPathRe = /^(\/?|)([\s\S]*?)((?:\.{1,2}|[^\/]+?|)(\.[^.\/]*|))(?:[\/]*)$/;
        return splitPathRe.exec(filename).slice(1);
      },normalizeArray:function (parts, allowAboveRoot) {
        // if the path tries to go above the root, `up` ends up > 0
        var up = 0;
        for (var i = parts.length - 1; i >= 0; i--) {
          var last = parts[i];
          if (last === '.') {
            parts.splice(i, 1);
          } else if (last === '..') {
            parts.splice(i, 1);
            up++;
          } else if (up) {
            parts.splice(i, 1);
            up--;
          }
        }
        // if the path is allowed to go above the root, restore leading ..s
        if (allowAboveRoot) {
          for (; up--; up) {
            parts.unshift('..');
          }
        }
        return parts;
      },normalize:function (path) {
        var isAbsolute = path.charAt(0) === '/',
            trailingSlash = path.substr(-1) === '/';
        // Normalize the path
        path = PATH.normalizeArray(path.split('/').filter(function(p) {
          return !!p;
        }), !isAbsolute).join('/');
        if (!path && !isAbsolute) {
          path = '.';
        }
        if (path && trailingSlash) {
          path += '/';
        }
        return (isAbsolute ? '/' : '') + path;
      },dirname:function (path) {
        var result = PATH.splitPath(path),
            root = result[0],
            dir = result[1];
        if (!root && !dir) {
          // No dirname whatsoever
          return '.';
        }
        if (dir) {
          // It has a dirname, strip trailing slash
          dir = dir.substr(0, dir.length - 1);
        }
        return root + dir;
      },basename:function (path) {
        // EMSCRIPTEN return '/'' for '/', not an empty string
        if (path === '/') return '/';
        var lastSlash = path.lastIndexOf('/');
        if (lastSlash === -1) return path;
        return path.substr(lastSlash+1);
      },extname:function (path) {
        return PATH.splitPath(path)[3];
      },join:function () {
        var paths = Array.prototype.slice.call(arguments, 0);
        return PATH.normalize(paths.join('/'));
      },join2:function (l, r) {
        return PATH.normalize(l + '/' + r);
      },resolve:function () {
        var resolvedPath = '',
          resolvedAbsolute = false;
        for (var i = arguments.length - 1; i >= -1 && !resolvedAbsolute; i--) {
          var path = (i >= 0) ? arguments[i] : FS.cwd();
          // Skip empty and invalid entries
          if (typeof path !== 'string') {
            throw new TypeError('Arguments to path.resolve must be strings');
          } else if (!path) {
            return ''; // an invalid portion invalidates the whole thing
          }
          resolvedPath = path + '/' + resolvedPath;
          resolvedAbsolute = path.charAt(0) === '/';
        }
        // At this point the path should be resolved to a full absolute path, but
        // handle relative paths to be safe (might happen when process.cwd() fails)
        resolvedPath = PATH.normalizeArray(resolvedPath.split('/').filter(function(p) {
          return !!p;
        }), !resolvedAbsolute).join('/');
        return ((resolvedAbsolute ? '/' : '') + resolvedPath) || '.';
      },relative:function (from, to) {
        from = PATH.resolve(from).substr(1);
        to = PATH.resolve(to).substr(1);
        function trim(arr) {
          var start = 0;
          for (; start < arr.length; start++) {
            if (arr[start] !== '') break;
          }
          var end = arr.length - 1;
          for (; end >= 0; end--) {
            if (arr[end] !== '') break;
          }
          if (start > end) return [];
          return arr.slice(start, end - start + 1);
        }
        var fromParts = trim(from.split('/'));
        var toParts = trim(to.split('/'));
        var length = Math.min(fromParts.length, toParts.length);
        var samePartsLength = length;
        for (var i = 0; i < length; i++) {
          if (fromParts[i] !== toParts[i]) {
            samePartsLength = i;
            break;
          }
        }
        var outputParts = [];
        for (var i = samePartsLength; i < fromParts.length; i++) {
          outputParts.push('..');
        }
        outputParts = outputParts.concat(toParts.slice(samePartsLength));
        return outputParts.join('/');
      }};
  
  var TTY={ttys:[],init:function () {
        // https://github.com/kripken/emscripten/pull/1555
        // if (ENVIRONMENT_IS_NODE) {
        //   // currently, FS.init does not distinguish if process.stdin is a file or TTY
        //   // device, it always assumes it's a TTY device. because of this, we're forcing
        //   // process.stdin to UTF8 encoding to at least make stdin reading compatible
        //   // with text files until FS.init can be refactored.
        //   process['stdin']['setEncoding']('utf8');
        // }
      },shutdown:function () {
        // https://github.com/kripken/emscripten/pull/1555
        // if (ENVIRONMENT_IS_NODE) {
        //   // inolen: any idea as to why node -e 'process.stdin.read()' wouldn't exit immediately (with process.stdin being a tty)?
        //   // isaacs: because now it's reading from the stream, you've expressed interest in it, so that read() kicks off a _read() which creates a ReadReq operation
        //   // inolen: I thought read() in that case was a synchronous operation that just grabbed some amount of buffered data if it exists?
        //   // isaacs: it is. but it also triggers a _read() call, which calls readStart() on the handle
        //   // isaacs: do process.stdin.pause() and i'd think it'd probably close the pending call
        //   process['stdin']['pause']();
        // }
      },register:function (dev, ops) {
        TTY.ttys[dev] = { input: [], output: [], ops: ops };
        FS.registerDevice(dev, TTY.stream_ops);
      },stream_ops:{open:function (stream) {
          var tty = TTY.ttys[stream.node.rdev];
          if (!tty) {
            throw new FS.ErrnoError(ERRNO_CODES.ENODEV);
          }
          stream.tty = tty;
          stream.seekable = false;
        },close:function (stream) {
          // flush any pending line data
          stream.tty.ops.flush(stream.tty);
        },flush:function (stream) {
          stream.tty.ops.flush(stream.tty);
        },read:function (stream, buffer, offset, length, pos /* ignored */) {
          if (!stream.tty || !stream.tty.ops.get_char) {
            throw new FS.ErrnoError(ERRNO_CODES.ENXIO);
          }
          var bytesRead = 0;
          for (var i = 0; i < length; i++) {
            var result;
            try {
              result = stream.tty.ops.get_char(stream.tty);
            } catch (e) {
              throw new FS.ErrnoError(ERRNO_CODES.EIO);
            }
            if (result === undefined && bytesRead === 0) {
              throw new FS.ErrnoError(ERRNO_CODES.EAGAIN);
            }
            if (result === null || result === undefined) break;
            bytesRead++;
            buffer[offset+i] = result;
          }
          if (bytesRead) {
            stream.node.timestamp = Date.now();
          }
          return bytesRead;
        },write:function (stream, buffer, offset, length, pos) {
          if (!stream.tty || !stream.tty.ops.put_char) {
            throw new FS.ErrnoError(ERRNO_CODES.ENXIO);
          }
          for (var i = 0; i < length; i++) {
            try {
              stream.tty.ops.put_char(stream.tty, buffer[offset+i]);
            } catch (e) {
              throw new FS.ErrnoError(ERRNO_CODES.EIO);
            }
          }
          if (length) {
            stream.node.timestamp = Date.now();
          }
          return i;
        }},default_tty_ops:{get_char:function (tty) {
          if (!tty.input.length) {
            var result = null;
            if (ENVIRONMENT_IS_NODE) {
              result = process['stdin']['read']();
              if (!result) {
                if (process['stdin']['_readableState'] && process['stdin']['_readableState']['ended']) {
                  return null;  // EOF
                }
                return undefined;  // no data available
              }
            } else if (typeof window != 'undefined' &&
              typeof window.prompt == 'function') {
              // Browser.
              result = window.prompt('Input: ');  // returns null on cancel
              if (result !== null) {
                result += '\n';
              }
            } else if (typeof readline == 'function') {
              // Command line.
              result = readline();
              if (result !== null) {
                result += '\n';
              }
            }
            if (!result) {
              return null;
            }
            tty.input = intArrayFromString(result, true);
          }
          return tty.input.shift();
        },flush:function (tty) {
          if (tty.output && tty.output.length > 0) {
            Module['print'](tty.output.join(''));
            tty.output = [];
          }
        },put_char:function (tty, val) {
          if (val === null || val === 10) {
            Module['print'](tty.output.join(''));
            tty.output = [];
          } else {
            tty.output.push(TTY.utf8.processCChar(val));
          }
        }},default_tty1_ops:{put_char:function (tty, val) {
          if (val === null || val === 10) {
            Module['printErr'](tty.output.join(''));
            tty.output = [];
          } else {
            tty.output.push(TTY.utf8.processCChar(val));
          }
        },flush:function (tty) {
          if (tty.output && tty.output.length > 0) {
            Module['printErr'](tty.output.join(''));
            tty.output = [];
          }
        }}};
  
  var MEMFS={ops_table:null,mount:function (mount) {
        return MEMFS.createNode(null, '/', 16384 | 511 /* 0777 */, 0);
      },createNode:function (parent, name, mode, dev) {
        if (FS.isBlkdev(mode) || FS.isFIFO(mode)) {
          // no supported
          throw new FS.ErrnoError(ERRNO_CODES.EPERM);
        }
        if (!MEMFS.ops_table) {
          MEMFS.ops_table = {
            dir: {
              node: {
                getattr: MEMFS.node_ops.getattr,
                setattr: MEMFS.node_ops.setattr,
                lookup: MEMFS.node_ops.lookup,
                mknod: MEMFS.node_ops.mknod,
                rename: MEMFS.node_ops.rename,
                unlink: MEMFS.node_ops.unlink,
                rmdir: MEMFS.node_ops.rmdir,
                readdir: MEMFS.node_ops.readdir,
                symlink: MEMFS.node_ops.symlink
              },
              stream: {
                llseek: MEMFS.stream_ops.llseek
              }
            },
            file: {
              node: {
                getattr: MEMFS.node_ops.getattr,
                setattr: MEMFS.node_ops.setattr
              },
              stream: {
                llseek: MEMFS.stream_ops.llseek,
                read: MEMFS.stream_ops.read,
                write: MEMFS.stream_ops.write,
                allocate: MEMFS.stream_ops.allocate,
                mmap: MEMFS.stream_ops.mmap
              }
            },
            link: {
              node: {
                getattr: MEMFS.node_ops.getattr,
                setattr: MEMFS.node_ops.setattr,
                readlink: MEMFS.node_ops.readlink
              },
              stream: {}
            },
            chrdev: {
              node: {
                getattr: MEMFS.node_ops.getattr,
                setattr: MEMFS.node_ops.setattr
              },
              stream: FS.chrdev_stream_ops
            }
          };
        }
        var node = FS.createNode(parent, name, mode, dev);
        if (FS.isDir(node.mode)) {
          node.node_ops = MEMFS.ops_table.dir.node;
          node.stream_ops = MEMFS.ops_table.dir.stream;
          node.contents = {};
        } else if (FS.isFile(node.mode)) {
          node.node_ops = MEMFS.ops_table.file.node;
          node.stream_ops = MEMFS.ops_table.file.stream;
          node.usedBytes = 0; // The actual number of bytes used in the typed array, as opposed to contents.buffer.byteLength which gives the whole capacity.
          // When the byte data of the file is populated, this will point to either a typed array, or a normal JS array. Typed arrays are preferred
          // for performance, and used by default. However, typed arrays are not resizable like normal JS arrays are, so there is a small disk size
          // penalty involved for appending file writes that continuously grow a file similar to std::vector capacity vs used -scheme.
          node.contents = null; 
        } else if (FS.isLink(node.mode)) {
          node.node_ops = MEMFS.ops_table.link.node;
          node.stream_ops = MEMFS.ops_table.link.stream;
        } else if (FS.isChrdev(node.mode)) {
          node.node_ops = MEMFS.ops_table.chrdev.node;
          node.stream_ops = MEMFS.ops_table.chrdev.stream;
        }
        node.timestamp = Date.now();
        // add the new node to the parent
        if (parent) {
          parent.contents[name] = node;
        }
        return node;
      },getFileDataAsRegularArray:function (node) {
        if (node.contents && node.contents.subarray) {
          var arr = [];
          for (var i = 0; i < node.usedBytes; ++i) arr.push(node.contents[i]);
          return arr; // Returns a copy of the original data.
        }
        return node.contents; // No-op, the file contents are already in a JS array. Return as-is.
      },getFileDataAsTypedArray:function (node) {
        if (!node.contents) return new Uint8Array;
        if (node.contents.subarray) return node.contents.subarray(0, node.usedBytes); // Make sure to not return excess unused bytes.
        return new Uint8Array(node.contents);
      },expandFileStorage:function (node, newCapacity) {
  
        // If we are asked to expand the size of a file that already exists, revert to using a standard JS array to store the file
        // instead of a typed array. This makes resizing the array more flexible because we can just .push() elements at the back to
        // increase the size.
        if (node.contents && node.contents.subarray && newCapacity > node.contents.length) {
          node.contents = MEMFS.getFileDataAsRegularArray(node);
          node.usedBytes = node.contents.length; // We might be writing to a lazy-loaded file which had overridden this property, so force-reset it.
        }
  
        if (!node.contents || node.contents.subarray) { // Keep using a typed array if creating a new storage, or if old one was a typed array as well.
          var prevCapacity = node.contents ? node.contents.buffer.byteLength : 0;
          if (prevCapacity >= newCapacity) return; // No need to expand, the storage was already large enough.
          // Don't expand strictly to the given requested limit if it's only a very small increase, but instead geometrically grow capacity.
          // For small filesizes (<1MB), perform size*2 geometric increase, but for large sizes, do a much more conservative size*1.125 increase to
          // avoid overshooting the allocation cap by a very large margin.
          var CAPACITY_DOUBLING_MAX = 1024 * 1024;
          newCapacity = Math.max(newCapacity, (prevCapacity * (prevCapacity < CAPACITY_DOUBLING_MAX ? 2.0 : 1.125)) | 0);
          if (prevCapacity != 0) newCapacity = Math.max(newCapacity, 256); // At minimum allocate 256b for each file when expanding.
          var oldContents = node.contents;
          node.contents = new Uint8Array(newCapacity); // Allocate new storage.
          if (node.usedBytes > 0) node.contents.set(oldContents.subarray(0, node.usedBytes), 0); // Copy old data over to the new storage.
          return;
        }
        // Not using a typed array to back the file storage. Use a standard JS array instead.
        if (!node.contents && newCapacity > 0) node.contents = [];
        while (node.contents.length < newCapacity) node.contents.push(0);
      },resizeFileStorage:function (node, newSize) {
        if (node.usedBytes == newSize) return;
        if (newSize == 0) {
          node.contents = null; // Fully decommit when requesting a resize to zero.
          node.usedBytes = 0;
          return;
        }
  
        if (!node.contents || node.contents.subarray) { // Resize a typed array if that is being used as the backing store.
          var oldContents = node.contents;
          node.contents = new Uint8Array(new ArrayBuffer(newSize)); // Allocate new storage.
          if (oldContents) {
            node.contents.set(oldContents.subarray(0, Math.min(newSize, node.usedBytes))); // Copy old data over to the new storage.
          }
          node.usedBytes = newSize;
          return;
        }
        // Backing with a JS array.
        if (!node.contents) node.contents = [];
        if (node.contents.length > newSize) node.contents.length = newSize;
        else while (node.contents.length < newSize) node.contents.push(0);
        node.usedBytes = newSize;
      },node_ops:{getattr:function (node) {
          var attr = {};
          // device numbers reuse inode numbers.
          attr.dev = FS.isChrdev(node.mode) ? node.id : 1;
          attr.ino = node.id;
          attr.mode = node.mode;
          attr.nlink = 1;
          attr.uid = 0;
          attr.gid = 0;
          attr.rdev = node.rdev;
          if (FS.isDir(node.mode)) {
            attr.size = 4096;
          } else if (FS.isFile(node.mode)) {
            attr.size = node.usedBytes;
          } else if (FS.isLink(node.mode)) {
            attr.size = node.link.length;
          } else {
            attr.size = 0;
          }
          attr.atime = new Date(node.timestamp);
          attr.mtime = new Date(node.timestamp);
          attr.ctime = new Date(node.timestamp);
          // NOTE: In our implementation, st_blocks = Math.ceil(st_size/st_blksize),
          //       but this is not required by the standard.
          attr.blksize = 4096;
          attr.blocks = Math.ceil(attr.size / attr.blksize);
          return attr;
        },setattr:function (node, attr) {
          if (attr.mode !== undefined) {
            node.mode = attr.mode;
          }
          if (attr.timestamp !== undefined) {
            node.timestamp = attr.timestamp;
          }
          if (attr.size !== undefined) {
            MEMFS.resizeFileStorage(node, attr.size);
          }
        },lookup:function (parent, name) {
          throw FS.genericErrors[ERRNO_CODES.ENOENT];
        },mknod:function (parent, name, mode, dev) {
          return MEMFS.createNode(parent, name, mode, dev);
        },rename:function (old_node, new_dir, new_name) {
          // if we're overwriting a directory at new_name, make sure it's empty.
          if (FS.isDir(old_node.mode)) {
            var new_node;
            try {
              new_node = FS.lookupNode(new_dir, new_name);
            } catch (e) {
            }
            if (new_node) {
              for (var i in new_node.contents) {
                throw new FS.ErrnoError(ERRNO_CODES.ENOTEMPTY);
              }
            }
          }
          // do the internal rewiring
          delete old_node.parent.contents[old_node.name];
          old_node.name = new_name;
          new_dir.contents[new_name] = old_node;
          old_node.parent = new_dir;
        },unlink:function (parent, name) {
          delete parent.contents[name];
        },rmdir:function (parent, name) {
          var node = FS.lookupNode(parent, name);
          for (var i in node.contents) {
            throw new FS.ErrnoError(ERRNO_CODES.ENOTEMPTY);
          }
          delete parent.contents[name];
        },readdir:function (node) {
          var entries = ['.', '..']
          for (var key in node.contents) {
            if (!node.contents.hasOwnProperty(key)) {
              continue;
            }
            entries.push(key);
          }
          return entries;
        },symlink:function (parent, newname, oldpath) {
          var node = MEMFS.createNode(parent, newname, 511 /* 0777 */ | 40960, 0);
          node.link = oldpath;
          return node;
        },readlink:function (node) {
          if (!FS.isLink(node.mode)) {
            throw new FS.ErrnoError(ERRNO_CODES.EINVAL);
          }
          return node.link;
        }},stream_ops:{read:function (stream, buffer, offset, length, position) {
          var contents = stream.node.contents;
          if (position >= stream.node.usedBytes) return 0;
          var size = Math.min(stream.node.usedBytes - position, length);
          assert(size >= 0);
          if (size > 8 && contents.subarray) { // non-trivial, and typed array
            buffer.set(contents.subarray(position, position + size), offset);
          } else
          {
            for (var i = 0; i < size; i++) buffer[offset + i] = contents[position + i];
          }
          return size;
        },write:function (stream, buffer, offset, length, position, canOwn) {
          if (!length) return 0;
          var node = stream.node;
          node.timestamp = Date.now();
  
          if (buffer.subarray && (!node.contents || node.contents.subarray)) { // This write is from a typed array to a typed array?
            if (canOwn) { // Can we just reuse the buffer we are given?
              node.contents = buffer.subarray(offset, offset + length);
              node.usedBytes = length;
              return length;
            } else if (node.usedBytes === 0 && position === 0) { // If this is a simple first write to an empty file, do a fast set since we don't need to care about old data.
              node.contents = new Uint8Array(buffer.subarray(offset, offset + length));
              node.usedBytes = length;
              return length;
            } else if (position + length <= node.usedBytes) { // Writing to an already allocated and used subrange of the file?
              node.contents.set(buffer.subarray(offset, offset + length), position);
              return length;
            }
          }
          // Appending to an existing file and we need to reallocate, or source data did not come as a typed array.
          MEMFS.expandFileStorage(node, position+length);
          if (node.contents.subarray && buffer.subarray) node.contents.set(buffer.subarray(offset, offset + length), position); // Use typed array write if available.
          else
            for (var i = 0; i < length; i++) {
             node.contents[position + i] = buffer[offset + i]; // Or fall back to manual write if not.
            }
          node.usedBytes = Math.max(node.usedBytes, position+length);
          return length;
        },llseek:function (stream, offset, whence) {
          var position = offset;
          if (whence === 1) {  // SEEK_CUR.
            position += stream.position;
          } else if (whence === 2) {  // SEEK_END.
            if (FS.isFile(stream.node.mode)) {
              position += stream.node.usedBytes;
            }
          }
          if (position < 0) {
            throw new FS.ErrnoError(ERRNO_CODES.EINVAL);
          }
          return position;
        },allocate:function (stream, offset, length) {
          MEMFS.expandFileStorage(stream.node, offset + length);
          stream.node.usedBytes = Math.max(stream.node.usedBytes, offset + length);
        },mmap:function (stream, buffer, offset, length, position, prot, flags) {
          if (!FS.isFile(stream.node.mode)) {
            throw new FS.ErrnoError(ERRNO_CODES.ENODEV);
          }
          var ptr;
          var allocated;
          var contents = stream.node.contents;
          // Only make a new copy when MAP_PRIVATE is specified.
          if ( !(flags & 2) &&
                (contents.buffer === buffer || contents.buffer === buffer.buffer) ) {
            // We can't emulate MAP_SHARED when the file is not backed by the buffer
            // we're mapping to (e.g. the HEAP buffer).
            allocated = false;
            ptr = contents.byteOffset;
          } else {
            // Try to avoid unnecessary slices.
            if (position > 0 || position + length < stream.node.usedBytes) {
              if (contents.subarray) {
                contents = contents.subarray(position, position + length);
              } else {
                contents = Array.prototype.slice.call(contents, position, position + length);
              }
            }
            allocated = true;
            ptr = _malloc(length);
            if (!ptr) {
              throw new FS.ErrnoError(ERRNO_CODES.ENOMEM);
            }
            buffer.set(contents, ptr);
          }
          return { ptr: ptr, allocated: allocated };
        }}};
  
  var IDBFS={dbs:{},indexedDB:function () {
        if (typeof indexedDB !== 'undefined') return indexedDB;
        var ret = null;
        if (typeof window === 'object') ret = window.indexedDB || window.mozIndexedDB || window.webkitIndexedDB || window.msIndexedDB;
        assert(ret, 'IDBFS used, but indexedDB not supported');
        return ret;
      },DB_VERSION:21,DB_STORE_NAME:"FILE_DATA",mount:function (mount) {
        // reuse all of the core MEMFS functionality
        return MEMFS.mount.apply(null, arguments);
      },syncfs:function (mount, populate, callback) {
        IDBFS.getLocalSet(mount, function(err, local) {
          if (err) return callback(err);
  
          IDBFS.getRemoteSet(mount, function(err, remote) {
            if (err) return callback(err);
  
            var src = populate ? remote : local;
            var dst = populate ? local : remote;
  
            IDBFS.reconcile(src, dst, callback);
          });
        });
      },getDB:function (name, callback) {
        // check the cache first
        var db = IDBFS.dbs[name];
        if (db) {
          return callback(null, db);
        }
  
        var req;
        try {
          req = IDBFS.indexedDB().open(name, IDBFS.DB_VERSION);
        } catch (e) {
          return callback(e);
        }
        req.onupgradeneeded = function(e) {
          var db = e.target.result;
          var transaction = e.target.transaction;
  
          var fileStore;
  
          if (db.objectStoreNames.contains(IDBFS.DB_STORE_NAME)) {
            fileStore = transaction.objectStore(IDBFS.DB_STORE_NAME);
          } else {
            fileStore = db.createObjectStore(IDBFS.DB_STORE_NAME);
          }
  
          fileStore.createIndex('timestamp', 'timestamp', { unique: false });
        };
        req.onsuccess = function() {
          db = req.result;
  
          // add to the cache
          IDBFS.dbs[name] = db;
          callback(null, db);
        };
        req.onerror = function() {
          callback(this.error);
        };
      },getLocalSet:function (mount, callback) {
        var entries = {};
  
        function isRealDir(p) {
          return p !== '.' && p !== '..';
        };
        function toAbsolute(root) {
          return function(p) {
            return PATH.join2(root, p);
          }
        };
  
        var check = FS.readdir(mount.mountpoint).filter(isRealDir).map(toAbsolute(mount.mountpoint));
  
        while (check.length) {
          var path = check.pop();
          var stat;
  
          try {
            stat = FS.stat(path);
          } catch (e) {
            return callback(e);
          }
  
          if (FS.isDir(stat.mode)) {
            check.push.apply(check, FS.readdir(path).filter(isRealDir).map(toAbsolute(path)));
          }
  
          entries[path] = { timestamp: stat.mtime };
        }
  
        return callback(null, { type: 'local', entries: entries });
      },getRemoteSet:function (mount, callback) {
        var entries = {};
  
        IDBFS.getDB(mount.mountpoint, function(err, db) {
          if (err) return callback(err);
  
          var transaction = db.transaction([IDBFS.DB_STORE_NAME], 'readonly');
          transaction.onerror = function() { callback(this.error); };
  
          var store = transaction.objectStore(IDBFS.DB_STORE_NAME);
          var index = store.index('timestamp');
  
          index.openKeyCursor().onsuccess = function(event) {
            var cursor = event.target.result;
  
            if (!cursor) {
              return callback(null, { type: 'remote', db: db, entries: entries });
            }
  
            entries[cursor.primaryKey] = { timestamp: cursor.key };
  
            cursor.continue();
          };
        });
      },loadLocalEntry:function (path, callback) {
        var stat, node;
  
        try {
          var lookup = FS.lookupPath(path);
          node = lookup.node;
          stat = FS.stat(path);
        } catch (e) {
          return callback(e);
        }
  
        if (FS.isDir(stat.mode)) {
          return callback(null, { timestamp: stat.mtime, mode: stat.mode });
        } else if (FS.isFile(stat.mode)) {
          // Performance consideration: storing a normal JavaScript array to a IndexedDB is much slower than storing a typed array.
          // Therefore always convert the file contents to a typed array first before writing the data to IndexedDB.
          node.contents = MEMFS.getFileDataAsTypedArray(node);
          return callback(null, { timestamp: stat.mtime, mode: stat.mode, contents: node.contents });
        } else {
          return callback(new Error('node type not supported'));
        }
      },storeLocalEntry:function (path, entry, callback) {
        try {
          if (FS.isDir(entry.mode)) {
            FS.mkdir(path, entry.mode);
          } else if (FS.isFile(entry.mode)) {
            FS.writeFile(path, entry.contents, { encoding: 'binary', canOwn: true });
          } else {
            return callback(new Error('node type not supported'));
          }
  
          FS.chmod(path, entry.mode);
          FS.utime(path, entry.timestamp, entry.timestamp);
        } catch (e) {
          return callback(e);
        }
  
        callback(null);
      },removeLocalEntry:function (path, callback) {
        try {
          var lookup = FS.lookupPath(path);
          var stat = FS.stat(path);
  
          if (FS.isDir(stat.mode)) {
            FS.rmdir(path);
          } else if (FS.isFile(stat.mode)) {
            FS.unlink(path);
          }
        } catch (e) {
          return callback(e);
        }
  
        callback(null);
      },loadRemoteEntry:function (store, path, callback) {
        var req = store.get(path);
        req.onsuccess = function(event) { callback(null, event.target.result); };
        req.onerror = function() { callback(this.error); };
      },storeRemoteEntry:function (store, path, entry, callback) {
        var req = store.put(entry, path);
        req.onsuccess = function() { callback(null); };
        req.onerror = function() { callback(this.error); };
      },removeRemoteEntry:function (store, path, callback) {
        var req = store.delete(path);
        req.onsuccess = function() { callback(null); };
        req.onerror = function() { callback(this.error); };
      },reconcile:function (src, dst, callback) {
        var total = 0;
  
        var create = [];
        Object.keys(src.entries).forEach(function (key) {
          var e = src.entries[key];
          var e2 = dst.entries[key];
          if (!e2 || e.timestamp > e2.timestamp) {
            create.push(key);
            total++;
          }
        });
  
        var remove = [];
        Object.keys(dst.entries).forEach(function (key) {
          var e = dst.entries[key];
          var e2 = src.entries[key];
          if (!e2) {
            remove.push(key);
            total++;
          }
        });
  
        if (!total) {
          return callback(null);
        }
  
        var errored = false;
        var completed = 0;
        var db = src.type === 'remote' ? src.db : dst.db;
        var transaction = db.transaction([IDBFS.DB_STORE_NAME], 'readwrite');
        var store = transaction.objectStore(IDBFS.DB_STORE_NAME);
  
        function done(err) {
          if (err) {
            if (!done.errored) {
              done.errored = true;
              return callback(err);
            }
            return;
          }
          if (++completed >= total) {
            return callback(null);
          }
        };
  
        transaction.onerror = function() { done(this.error); };
  
        // sort paths in ascending order so directory entries are created
        // before the files inside them
        create.sort().forEach(function (path) {
          if (dst.type === 'local') {
            IDBFS.loadRemoteEntry(store, path, function (err, entry) {
              if (err) return done(err);
              IDBFS.storeLocalEntry(path, entry, done);
            });
          } else {
            IDBFS.loadLocalEntry(path, function (err, entry) {
              if (err) return done(err);
              IDBFS.storeRemoteEntry(store, path, entry, done);
            });
          }
        });
  
        // sort paths in descending order so files are deleted before their
        // parent directories
        remove.sort().reverse().forEach(function(path) {
          if (dst.type === 'local') {
            IDBFS.removeLocalEntry(path, done);
          } else {
            IDBFS.removeRemoteEntry(store, path, done);
          }
        });
      }};
  
  var NODEFS={isWindows:false,staticInit:function () {
        NODEFS.isWindows = !!process.platform.match(/^win/);
      },mount:function (mount) {
        assert(ENVIRONMENT_IS_NODE);
        return NODEFS.createNode(null, '/', NODEFS.getMode(mount.opts.root), 0);
      },createNode:function (parent, name, mode, dev) {
        if (!FS.isDir(mode) && !FS.isFile(mode) && !FS.isLink(mode)) {
          throw new FS.ErrnoError(ERRNO_CODES.EINVAL);
        }
        var node = FS.createNode(parent, name, mode);
        node.node_ops = NODEFS.node_ops;
        node.stream_ops = NODEFS.stream_ops;
        return node;
      },getMode:function (path) {
        var stat;
        try {
          stat = fs.lstatSync(path);
          if (NODEFS.isWindows) {
            // On Windows, directories return permission bits 'rw-rw-rw-', even though they have 'rwxrwxrwx', so 
            // propagate write bits to execute bits.
            stat.mode = stat.mode | ((stat.mode & 146) >> 1);
          }
        } catch (e) {
          if (!e.code) throw e;
          throw new FS.ErrnoError(ERRNO_CODES[e.code]);
        }
        return stat.mode;
      },realPath:function (node) {
        var parts = [];
        while (node.parent !== node) {
          parts.push(node.name);
          node = node.parent;
        }
        parts.push(node.mount.opts.root);
        parts.reverse();
        return PATH.join.apply(null, parts);
      },flagsToPermissionStringMap:{0:"r",1:"r+",2:"r+",64:"r",65:"r+",66:"r+",129:"rx+",193:"rx+",514:"w+",577:"w",578:"w+",705:"wx",706:"wx+",1024:"a",1025:"a",1026:"a+",1089:"a",1090:"a+",1153:"ax",1154:"ax+",1217:"ax",1218:"ax+",4096:"rs",4098:"rs+"},flagsToPermissionString:function (flags) {
        if (flags in NODEFS.flagsToPermissionStringMap) {
          return NODEFS.flagsToPermissionStringMap[flags];
        } else {
          return flags;
        }
      },node_ops:{getattr:function (node) {
          var path = NODEFS.realPath(node);
          var stat;
          try {
            stat = fs.lstatSync(path);
          } catch (e) {
            if (!e.code) throw e;
            throw new FS.ErrnoError(ERRNO_CODES[e.code]);
          }
          // node.js v0.10.20 doesn't report blksize and blocks on Windows. Fake them with default blksize of 4096.
          // See http://support.microsoft.com/kb/140365
          if (NODEFS.isWindows && !stat.blksize) {
            stat.blksize = 4096;
          }
          if (NODEFS.isWindows && !stat.blocks) {
            stat.blocks = (stat.size+stat.blksize-1)/stat.blksize|0;
          }
          return {
            dev: stat.dev,
            ino: stat.ino,
            mode: stat.mode,
            nlink: stat.nlink,
            uid: stat.uid,
            gid: stat.gid,
            rdev: stat.rdev,
            size: stat.size,
            atime: stat.atime,
            mtime: stat.mtime,
            ctime: stat.ctime,
            blksize: stat.blksize,
            blocks: stat.blocks
          };
        },setattr:function (node, attr) {
          var path = NODEFS.realPath(node);
          try {
            if (attr.mode !== undefined) {
              fs.chmodSync(path, attr.mode);
              // update the common node structure mode as well
              node.mode = attr.mode;
            }
            if (attr.timestamp !== undefined) {
              var date = new Date(attr.timestamp);
              fs.utimesSync(path, date, date);
            }
            if (attr.size !== undefined) {
              fs.truncateSync(path, attr.size);
            }
          } catch (e) {
            if (!e.code) throw e;
            throw new FS.ErrnoError(ERRNO_CODES[e.code]);
          }
        },lookup:function (parent, name) {
          var path = PATH.join2(NODEFS.realPath(parent), name);
          var mode = NODEFS.getMode(path);
          return NODEFS.createNode(parent, name, mode);
        },mknod:function (parent, name, mode, dev) {
          var node = NODEFS.createNode(parent, name, mode, dev);
          // create the backing node for this in the fs root as well
          var path = NODEFS.realPath(node);
          try {
            if (FS.isDir(node.mode)) {
              fs.mkdirSync(path, node.mode);
            } else {
              fs.writeFileSync(path, '', { mode: node.mode });
            }
          } catch (e) {
            if (!e.code) throw e;
            throw new FS.ErrnoError(ERRNO_CODES[e.code]);
          }
          return node;
        },rename:function (oldNode, newDir, newName) {
          var oldPath = NODEFS.realPath(oldNode);
          var newPath = PATH.join2(NODEFS.realPath(newDir), newName);
          try {
            fs.renameSync(oldPath, newPath);
          } catch (e) {
            if (!e.code) throw e;
            throw new FS.ErrnoError(ERRNO_CODES[e.code]);
          }
        },unlink:function (parent, name) {
          var path = PATH.join2(NODEFS.realPath(parent), name);
          try {
            fs.unlinkSync(path);
          } catch (e) {
            if (!e.code) throw e;
            throw new FS.ErrnoError(ERRNO_CODES[e.code]);
          }
        },rmdir:function (parent, name) {
          var path = PATH.join2(NODEFS.realPath(parent), name);
          try {
            fs.rmdirSync(path);
          } catch (e) {
            if (!e.code) throw e;
            throw new FS.ErrnoError(ERRNO_CODES[e.code]);
          }
        },readdir:function (node) {
          var path = NODEFS.realPath(node);
          try {
            return fs.readdirSync(path);
          } catch (e) {
            if (!e.code) throw e;
            throw new FS.ErrnoError(ERRNO_CODES[e.code]);
          }
        },symlink:function (parent, newName, oldPath) {
          var newPath = PATH.join2(NODEFS.realPath(parent), newName);
          try {
            fs.symlinkSync(oldPath, newPath);
          } catch (e) {
            if (!e.code) throw e;
            throw new FS.ErrnoError(ERRNO_CODES[e.code]);
          }
        },readlink:function (node) {
          var path = NODEFS.realPath(node);
          try {
            return fs.readlinkSync(path);
          } catch (e) {
            if (!e.code) throw e;
            throw new FS.ErrnoError(ERRNO_CODES[e.code]);
          }
        }},stream_ops:{open:function (stream) {
          var path = NODEFS.realPath(stream.node);
          try {
            if (FS.isFile(stream.node.mode)) {
              stream.nfd = fs.openSync(path, NODEFS.flagsToPermissionString(stream.flags));
            }
          } catch (e) {
            if (!e.code) throw e;
            throw new FS.ErrnoError(ERRNO_CODES[e.code]);
          }
        },close:function (stream) {
          try {
            if (FS.isFile(stream.node.mode) && stream.nfd) {
              fs.closeSync(stream.nfd);
            }
          } catch (e) {
            if (!e.code) throw e;
            throw new FS.ErrnoError(ERRNO_CODES[e.code]);
          }
        },read:function (stream, buffer, offset, length, position) {
          if (length === 0) return 0; // node errors on 0 length reads
          // FIXME this is terrible.
          var nbuffer = new Buffer(length);
          var res;
          try {
            res = fs.readSync(stream.nfd, nbuffer, 0, length, position);
          } catch (e) {
            throw new FS.ErrnoError(ERRNO_CODES[e.code]);
          }
          if (res > 0) {
            for (var i = 0; i < res; i++) {
              buffer[offset + i] = nbuffer[i];
            }
          }
          return res;
        },write:function (stream, buffer, offset, length, position) {
          // FIXME this is terrible.
          var nbuffer = new Buffer(buffer.subarray(offset, offset + length));
          var res;
          try {
            res = fs.writeSync(stream.nfd, nbuffer, 0, length, position);
          } catch (e) {
            throw new FS.ErrnoError(ERRNO_CODES[e.code]);
          }
          return res;
        },llseek:function (stream, offset, whence) {
          var position = offset;
          if (whence === 1) {  // SEEK_CUR.
            position += stream.position;
          } else if (whence === 2) {  // SEEK_END.
            if (FS.isFile(stream.node.mode)) {
              try {
                var stat = fs.fstatSync(stream.nfd);
                position += stat.size;
              } catch (e) {
                throw new FS.ErrnoError(ERRNO_CODES[e.code]);
              }
            }
          }
  
          if (position < 0) {
            throw new FS.ErrnoError(ERRNO_CODES.EINVAL);
          }
  
          return position;
        }}};
  
  var _stdin=allocate(1, "i32*", ALLOC_STATIC);
  
  var _stdout=allocate(1, "i32*", ALLOC_STATIC);
  
  var _stderr=allocate(1, "i32*", ALLOC_STATIC);
  
  function _fflush(stream) {
      // int fflush(FILE *stream);
      // http://pubs.opengroup.org/onlinepubs/000095399/functions/fflush.html
  
      /*
      // Disabled, see https://github.com/kripken/emscripten/issues/2770
      stream = FS.getStreamFromPtr(stream);
      if (stream.stream_ops.flush) {
        stream.stream_ops.flush(stream);
      }
      */
    }var FS={root:null,mounts:[],devices:[null],streams:[],nextInode:1,nameTable:null,currentPath:"/",initialized:false,ignorePermissions:true,trackingDelegate:{},tracking:{openFlags:{READ:1,WRITE:2}},ErrnoError:null,genericErrors:{},handleFSError:function (e) {
        if (!(e instanceof FS.ErrnoError)) throw e + ' : ' + stackTrace();
        return ___setErrNo(e.errno);
      },lookupPath:function (path, opts) {
        path = PATH.resolve(FS.cwd(), path);
        opts = opts || {};
  
        if (!path) return { path: '', node: null };
  
        var defaults = {
          follow_mount: true,
          recurse_count: 0
        };
        for (var key in defaults) {
          if (opts[key] === undefined) {
            opts[key] = defaults[key];
          }
        }
  
        if (opts.recurse_count > 8) {  // max recursive lookup of 8
          throw new FS.ErrnoError(ERRNO_CODES.ELOOP);
        }
  
        // split the path
        var parts = PATH.normalizeArray(path.split('/').filter(function(p) {
          return !!p;
        }), false);
  
        // start at the root
        var current = FS.root;
        var current_path = '/';
  
        for (var i = 0; i < parts.length; i++) {
          var islast = (i === parts.length-1);
          if (islast && opts.parent) {
            // stop resolving
            break;
          }
  
          current = FS.lookupNode(current, parts[i]);
          current_path = PATH.join2(current_path, parts[i]);
  
          // jump to the mount's root node if this is a mountpoint
          if (FS.isMountpoint(current)) {
            if (!islast || (islast && opts.follow_mount)) {
              current = current.mounted.root;
            }
          }
  
          // by default, lookupPath will not follow a symlink if it is the final path component.
          // setting opts.follow = true will override this behavior.
          if (!islast || opts.follow) {
            var count = 0;
            while (FS.isLink(current.mode)) {
              var link = FS.readlink(current_path);
              current_path = PATH.resolve(PATH.dirname(current_path), link);
              
              var lookup = FS.lookupPath(current_path, { recurse_count: opts.recurse_count });
              current = lookup.node;
  
              if (count++ > 40) {  // limit max consecutive symlinks to 40 (SYMLOOP_MAX).
                throw new FS.ErrnoError(ERRNO_CODES.ELOOP);
              }
            }
          }
        }
  
        return { path: current_path, node: current };
      },getPath:function (node) {
        var path;
        while (true) {
          if (FS.isRoot(node)) {
            var mount = node.mount.mountpoint;
            if (!path) return mount;
            return mount[mount.length-1] !== '/' ? mount + '/' + path : mount + path;
          }
          path = path ? node.name + '/' + path : node.name;
          node = node.parent;
        }
      },hashName:function (parentid, name) {
        var hash = 0;
  
  
        for (var i = 0; i < name.length; i++) {
          hash = ((hash << 5) - hash + name.charCodeAt(i)) | 0;
        }
        return ((parentid + hash) >>> 0) % FS.nameTable.length;
      },hashAddNode:function (node) {
        var hash = FS.hashName(node.parent.id, node.name);
        node.name_next = FS.nameTable[hash];
        FS.nameTable[hash] = node;
      },hashRemoveNode:function (node) {
        var hash = FS.hashName(node.parent.id, node.name);
        if (FS.nameTable[hash] === node) {
          FS.nameTable[hash] = node.name_next;
        } else {
          var current = FS.nameTable[hash];
          while (current) {
            if (current.name_next === node) {
              current.name_next = node.name_next;
              break;
            }
            current = current.name_next;
          }
        }
      },lookupNode:function (parent, name) {
        var err = FS.mayLookup(parent);
        if (err) {
          throw new FS.ErrnoError(err, parent);
        }
        var hash = FS.hashName(parent.id, name);
        for (var node = FS.nameTable[hash]; node; node = node.name_next) {
          var nodeName = node.name;
          if (node.parent.id === parent.id && nodeName === name) {
            return node;
          }
        }
        // if we failed to find it in the cache, call into the VFS
        return FS.lookup(parent, name);
      },createNode:function (parent, name, mode, rdev) {
        if (!FS.FSNode) {
          FS.FSNode = function(parent, name, mode, rdev) {
            if (!parent) {
              parent = this;  // root node sets parent to itself
            }
            this.parent = parent;
            this.mount = parent.mount;
            this.mounted = null;
            this.id = FS.nextInode++;
            this.name = name;
            this.mode = mode;
            this.node_ops = {};
            this.stream_ops = {};
            this.rdev = rdev;
          };
  
          FS.FSNode.prototype = {};
  
          // compatibility
          var readMode = 292 | 73;
          var writeMode = 146;
  
          // NOTE we must use Object.defineProperties instead of individual calls to
          // Object.defineProperty in order to make closure compiler happy
          Object.defineProperties(FS.FSNode.prototype, {
            read: {
              get: function() { return (this.mode & readMode) === readMode; },
              set: function(val) { val ? this.mode |= readMode : this.mode &= ~readMode; }
            },
            write: {
              get: function() { return (this.mode & writeMode) === writeMode; },
              set: function(val) { val ? this.mode |= writeMode : this.mode &= ~writeMode; }
            },
            isFolder: {
              get: function() { return FS.isDir(this.mode); }
            },
            isDevice: {
              get: function() { return FS.isChrdev(this.mode); }
            }
          });
        }
  
        var node = new FS.FSNode(parent, name, mode, rdev);
  
        FS.hashAddNode(node);
  
        return node;
      },destroyNode:function (node) {
        FS.hashRemoveNode(node);
      },isRoot:function (node) {
        return node === node.parent;
      },isMountpoint:function (node) {
        return !!node.mounted;
      },isFile:function (mode) {
        return (mode & 61440) === 32768;
      },isDir:function (mode) {
        return (mode & 61440) === 16384;
      },isLink:function (mode) {
        return (mode & 61440) === 40960;
      },isChrdev:function (mode) {
        return (mode & 61440) === 8192;
      },isBlkdev:function (mode) {
        return (mode & 61440) === 24576;
      },isFIFO:function (mode) {
        return (mode & 61440) === 4096;
      },isSocket:function (mode) {
        return (mode & 49152) === 49152;
      },flagModes:{"r":0,"rs":1052672,"r+":2,"w":577,"wx":705,"xw":705,"w+":578,"wx+":706,"xw+":706,"a":1089,"ax":1217,"xa":1217,"a+":1090,"ax+":1218,"xa+":1218},modeStringToFlags:function (str) {
        var flags = FS.flagModes[str];
        if (typeof flags === 'undefined') {
          throw new Error('Unknown file open mode: ' + str);
        }
        return flags;
      },flagsToPermissionString:function (flag) {
        var accmode = flag & 2097155;
        var perms = ['r', 'w', 'rw'][accmode];
        if ((flag & 512)) {
          perms += 'w';
        }
        return perms;
      },nodePermissions:function (node, perms) {
        if (FS.ignorePermissions) {
          return 0;
        }
        // return 0 if any user, group or owner bits are set.
        if (perms.indexOf('r') !== -1 && !(node.mode & 292)) {
          return ERRNO_CODES.EACCES;
        } else if (perms.indexOf('w') !== -1 && !(node.mode & 146)) {
          return ERRNO_CODES.EACCES;
        } else if (perms.indexOf('x') !== -1 && !(node.mode & 73)) {
          return ERRNO_CODES.EACCES;
        }
        return 0;
      },mayLookup:function (dir) {
        var err = FS.nodePermissions(dir, 'x');
        if (err) return err;
        if (!dir.node_ops.lookup) return ERRNO_CODES.EACCES;
        return 0;
      },mayCreate:function (dir, name) {
        try {
          var node = FS.lookupNode(dir, name);
          return ERRNO_CODES.EEXIST;
        } catch (e) {
        }
        return FS.nodePermissions(dir, 'wx');
      },mayDelete:function (dir, name, isdir) {
        var node;
        try {
          node = FS.lookupNode(dir, name);
        } catch (e) {
          return e.errno;
        }
        var err = FS.nodePermissions(dir, 'wx');
        if (err) {
          return err;
        }
        if (isdir) {
          if (!FS.isDir(node.mode)) {
            return ERRNO_CODES.ENOTDIR;
          }
          if (FS.isRoot(node) || FS.getPath(node) === FS.cwd()) {
            return ERRNO_CODES.EBUSY;
          }
        } else {
          if (FS.isDir(node.mode)) {
            return ERRNO_CODES.EISDIR;
          }
        }
        return 0;
      },mayOpen:function (node, flags) {
        if (!node) {
          return ERRNO_CODES.ENOENT;
        }
        if (FS.isLink(node.mode)) {
          return ERRNO_CODES.ELOOP;
        } else if (FS.isDir(node.mode)) {
          if ((flags & 2097155) !== 0 ||  // opening for write
              (flags & 512)) {
            return ERRNO_CODES.EISDIR;
          }
        }
        return FS.nodePermissions(node, FS.flagsToPermissionString(flags));
      },MAX_OPEN_FDS:4096,nextfd:function (fd_start, fd_end) {
        fd_start = fd_start || 0;
        fd_end = fd_end || FS.MAX_OPEN_FDS;
        for (var fd = fd_start; fd <= fd_end; fd++) {
          if (!FS.streams[fd]) {
            return fd;
          }
        }
        throw new FS.ErrnoError(ERRNO_CODES.EMFILE);
      },getStream:function (fd) {
        return FS.streams[fd];
      },createStream:function (stream, fd_start, fd_end) {
        if (!FS.FSStream) {
          FS.FSStream = function(){};
          FS.FSStream.prototype = {};
          // compatibility
          Object.defineProperties(FS.FSStream.prototype, {
            object: {
              get: function() { return this.node; },
              set: function(val) { this.node = val; }
            },
            isRead: {
              get: function() { return (this.flags & 2097155) !== 1; }
            },
            isWrite: {
              get: function() { return (this.flags & 2097155) !== 0; }
            },
            isAppend: {
              get: function() { return (this.flags & 1024); }
            }
          });
        }
        // clone it, so we can return an instance of FSStream
        var newStream = new FS.FSStream();
        for (var p in stream) {
          newStream[p] = stream[p];
        }
        stream = newStream;
        var fd = FS.nextfd(fd_start, fd_end);
        stream.fd = fd;
        FS.streams[fd] = stream;
        return stream;
      },closeStream:function (fd) {
        FS.streams[fd] = null;
      },getStreamFromPtr:function (ptr) {
        return FS.streams[ptr - 1];
      },getPtrForStream:function (stream) {
        return stream ? stream.fd + 1 : 0;
      },chrdev_stream_ops:{open:function (stream) {
          var device = FS.getDevice(stream.node.rdev);
          // override node's stream ops with the device's
          stream.stream_ops = device.stream_ops;
          // forward the open call
          if (stream.stream_ops.open) {
            stream.stream_ops.open(stream);
          }
        },llseek:function () {
          throw new FS.ErrnoError(ERRNO_CODES.ESPIPE);
        }},major:function (dev) {
        return ((dev) >> 8);
      },minor:function (dev) {
        return ((dev) & 0xff);
      },makedev:function (ma, mi) {
        return ((ma) << 8 | (mi));
      },registerDevice:function (dev, ops) {
        FS.devices[dev] = { stream_ops: ops };
      },getDevice:function (dev) {
        return FS.devices[dev];
      },getMounts:function (mount) {
        var mounts = [];
        var check = [mount];
  
        while (check.length) {
          var m = check.pop();
  
          mounts.push(m);
  
          check.push.apply(check, m.mounts);
        }
  
        return mounts;
      },syncfs:function (populate, callback) {
        if (typeof(populate) === 'function') {
          callback = populate;
          populate = false;
        }
  
        var mounts = FS.getMounts(FS.root.mount);
        var completed = 0;
  
        function done(err) {
          if (err) {
            if (!done.errored) {
              done.errored = true;
              return callback(err);
            }
            return;
          }
          if (++completed >= mounts.length) {
            callback(null);
          }
        };
  
        // sync all mounts
        mounts.forEach(function (mount) {
          if (!mount.type.syncfs) {
            return done(null);
          }
          mount.type.syncfs(mount, populate, done);
        });
      },mount:function (type, opts, mountpoint) {
        var root = mountpoint === '/';
        var pseudo = !mountpoint;
        var node;
  
        if (root && FS.root) {
          throw new FS.ErrnoError(ERRNO_CODES.EBUSY);
        } else if (!root && !pseudo) {
          var lookup = FS.lookupPath(mountpoint, { follow_mount: false });
  
          mountpoint = lookup.path;  // use the absolute path
          node = lookup.node;
  
          if (FS.isMountpoint(node)) {
            throw new FS.ErrnoError(ERRNO_CODES.EBUSY);
          }
  
          if (!FS.isDir(node.mode)) {
            throw new FS.ErrnoError(ERRNO_CODES.ENOTDIR);
          }
        }
  
        var mount = {
          type: type,
          opts: opts,
          mountpoint: mountpoint,
          mounts: []
        };
  
        // create a root node for the fs
        var mountRoot = type.mount(mount);
        mountRoot.mount = mount;
        mount.root = mountRoot;
  
        if (root) {
          FS.root = mountRoot;
        } else if (node) {
          // set as a mountpoint
          node.mounted = mount;
  
          // add the new mount to the current mount's children
          if (node.mount) {
            node.mount.mounts.push(mount);
          }
        }
  
        return mountRoot;
      },unmount:function (mountpoint) {
        var lookup = FS.lookupPath(mountpoint, { follow_mount: false });
  
        if (!FS.isMountpoint(lookup.node)) {
          throw new FS.ErrnoError(ERRNO_CODES.EINVAL);
        }
  
        // destroy the nodes for this mount, and all its child mounts
        var node = lookup.node;
        var mount = node.mounted;
        var mounts = FS.getMounts(mount);
  
        Object.keys(FS.nameTable).forEach(function (hash) {
          var current = FS.nameTable[hash];
  
          while (current) {
            var next = current.name_next;
  
            if (mounts.indexOf(current.mount) !== -1) {
              FS.destroyNode(current);
            }
  
            current = next;
          }
        });
  
        // no longer a mountpoint
        node.mounted = null;
  
        // remove this mount from the child mounts
        var idx = node.mount.mounts.indexOf(mount);
        assert(idx !== -1);
        node.mount.mounts.splice(idx, 1);
      },lookup:function (parent, name) {
        return parent.node_ops.lookup(parent, name);
      },mknod:function (path, mode, dev) {
        var lookup = FS.lookupPath(path, { parent: true });
        var parent = lookup.node;
        var name = PATH.basename(path);
        if (!name || name === '.' || name === '..') {
          throw new FS.ErrnoError(ERRNO_CODES.EINVAL);
        }
        var err = FS.mayCreate(parent, name);
        if (err) {
          throw new FS.ErrnoError(err);
        }
        if (!parent.node_ops.mknod) {
          throw new FS.ErrnoError(ERRNO_CODES.EPERM);
        }
        return parent.node_ops.mknod(parent, name, mode, dev);
      },create:function (path, mode) {
        mode = mode !== undefined ? mode : 438 /* 0666 */;
        mode &= 4095;
        mode |= 32768;
        return FS.mknod(path, mode, 0);
      },mkdir:function (path, mode) {
        mode = mode !== undefined ? mode : 511 /* 0777 */;
        mode &= 511 | 512;
        mode |= 16384;
        return FS.mknod(path, mode, 0);
      },mkdev:function (path, mode, dev) {
        if (typeof(dev) === 'undefined') {
          dev = mode;
          mode = 438 /* 0666 */;
        }
        mode |= 8192;
        return FS.mknod(path, mode, dev);
      },symlink:function (oldpath, newpath) {
        if (!PATH.resolve(oldpath)) {
          throw new FS.ErrnoError(ERRNO_CODES.ENOENT);
        }
        var lookup = FS.lookupPath(newpath, { parent: true });
        var parent = lookup.node;
        if (!parent) {
          throw new FS.ErrnoError(ERRNO_CODES.ENOENT);
        }
        var newname = PATH.basename(newpath);
        var err = FS.mayCreate(parent, newname);
        if (err) {
          throw new FS.ErrnoError(err);
        }
        if (!parent.node_ops.symlink) {
          throw new FS.ErrnoError(ERRNO_CODES.EPERM);
        }
        return parent.node_ops.symlink(parent, newname, oldpath);
      },rename:function (old_path, new_path) {
        var old_dirname = PATH.dirname(old_path);
        var new_dirname = PATH.dirname(new_path);
        var old_name = PATH.basename(old_path);
        var new_name = PATH.basename(new_path);
        // parents must exist
        var lookup, old_dir, new_dir;
        try {
          lookup = FS.lookupPath(old_path, { parent: true });
          old_dir = lookup.node;
          lookup = FS.lookupPath(new_path, { parent: true });
          new_dir = lookup.node;
        } catch (e) {
          throw new FS.ErrnoError(ERRNO_CODES.EBUSY);
        }
        if (!old_dir || !new_dir) throw new FS.ErrnoError(ERRNO_CODES.ENOENT);
        // need to be part of the same mount
        if (old_dir.mount !== new_dir.mount) {
          throw new FS.ErrnoError(ERRNO_CODES.EXDEV);
        }
        // source must exist
        var old_node = FS.lookupNode(old_dir, old_name);
        // old path should not be an ancestor of the new path
        var relative = PATH.relative(old_path, new_dirname);
        if (relative.charAt(0) !== '.') {
          throw new FS.ErrnoError(ERRNO_CODES.EINVAL);
        }
        // new path should not be an ancestor of the old path
        relative = PATH.relative(new_path, old_dirname);
        if (relative.charAt(0) !== '.') {
          throw new FS.ErrnoError(ERRNO_CODES.ENOTEMPTY);
        }
        // see if the new path already exists
        var new_node;
        try {
          new_node = FS.lookupNode(new_dir, new_name);
        } catch (e) {
          // not fatal
        }
        // early out if nothing needs to change
        if (old_node === new_node) {
          return;
        }
        // we'll need to delete the old entry
        var isdir = FS.isDir(old_node.mode);
        var err = FS.mayDelete(old_dir, old_name, isdir);
        if (err) {
          throw new FS.ErrnoError(err);
        }
        // need delete permissions if we'll be overwriting.
        // need create permissions if new doesn't already exist.
        err = new_node ?
          FS.mayDelete(new_dir, new_name, isdir) :
          FS.mayCreate(new_dir, new_name);
        if (err) {
          throw new FS.ErrnoError(err);
        }
        if (!old_dir.node_ops.rename) {
          throw new FS.ErrnoError(ERRNO_CODES.EPERM);
        }
        if (FS.isMountpoint(old_node) || (new_node && FS.isMountpoint(new_node))) {
          throw new FS.ErrnoError(ERRNO_CODES.EBUSY);
        }
        // if we are going to change the parent, check write permissions
        if (new_dir !== old_dir) {
          err = FS.nodePermissions(old_dir, 'w');
          if (err) {
            throw new FS.ErrnoError(err);
          }
        }
        try {
          if (FS.trackingDelegate['willMovePath']) {
            FS.trackingDelegate['willMovePath'](old_path, new_path);
          }
        } catch(e) {
          console.log("FS.trackingDelegate['willMovePath']('"+old_path+"', '"+new_path+"') threw an exception: " + e.message);
        }
        // remove the node from the lookup hash
        FS.hashRemoveNode(old_node);
        // do the underlying fs rename
        try {
          old_dir.node_ops.rename(old_node, new_dir, new_name);
        } catch (e) {
          throw e;
        } finally {
          // add the node back to the hash (in case node_ops.rename
          // changed its name)
          FS.hashAddNode(old_node);
        }
        try {
          if (FS.trackingDelegate['onMovePath']) FS.trackingDelegate['onMovePath'](old_path, new_path);
        } catch(e) {
          console.log("FS.trackingDelegate['onMovePath']('"+old_path+"', '"+new_path+"') threw an exception: " + e.message);
        }
      },rmdir:function (path) {
        var lookup = FS.lookupPath(path, { parent: true });
        var parent = lookup.node;
        var name = PATH.basename(path);
        var node = FS.lookupNode(parent, name);
        var err = FS.mayDelete(parent, name, true);
        if (err) {
          throw new FS.ErrnoError(err);
        }
        if (!parent.node_ops.rmdir) {
          throw new FS.ErrnoError(ERRNO_CODES.EPERM);
        }
        if (FS.isMountpoint(node)) {
          throw new FS.ErrnoError(ERRNO_CODES.EBUSY);
        }
        try {
          if (FS.trackingDelegate['willDeletePath']) {
            FS.trackingDelegate['willDeletePath'](path);
          }
        } catch(e) {
          console.log("FS.trackingDelegate['willDeletePath']('"+path+"') threw an exception: " + e.message);
        }
        parent.node_ops.rmdir(parent, name);
        FS.destroyNode(node);
        try {
          if (FS.trackingDelegate['onDeletePath']) FS.trackingDelegate['onDeletePath'](path);
        } catch(e) {
          console.log("FS.trackingDelegate['onDeletePath']('"+path+"') threw an exception: " + e.message);
        }
      },readdir:function (path) {
        var lookup = FS.lookupPath(path, { follow: true });
        var node = lookup.node;
        if (!node.node_ops.readdir) {
          throw new FS.ErrnoError(ERRNO_CODES.ENOTDIR);
        }
        return node.node_ops.readdir(node);
      },unlink:function (path) {
        var lookup = FS.lookupPath(path, { parent: true });
        var parent = lookup.node;
        var name = PATH.basename(path);
        var node = FS.lookupNode(parent, name);
        var err = FS.mayDelete(parent, name, false);
        if (err) {
          // POSIX says unlink should set EPERM, not EISDIR
          if (err === ERRNO_CODES.EISDIR) err = ERRNO_CODES.EPERM;
          throw new FS.ErrnoError(err);
        }
        if (!parent.node_ops.unlink) {
          throw new FS.ErrnoError(ERRNO_CODES.EPERM);
        }
        if (FS.isMountpoint(node)) {
          throw new FS.ErrnoError(ERRNO_CODES.EBUSY);
        }
        try {
          if (FS.trackingDelegate['willDeletePath']) {
            FS.trackingDelegate['willDeletePath'](path);
          }
        } catch(e) {
          console.log("FS.trackingDelegate['willDeletePath']('"+path+"') threw an exception: " + e.message);
        }
        parent.node_ops.unlink(parent, name);
        FS.destroyNode(node);
        try {
          if (FS.trackingDelegate['onDeletePath']) FS.trackingDelegate['onDeletePath'](path);
        } catch(e) {
          console.log("FS.trackingDelegate['onDeletePath']('"+path+"') threw an exception: " + e.message);
        }
      },readlink:function (path) {
        var lookup = FS.lookupPath(path);
        var link = lookup.node;
        if (!link) {
          throw new FS.ErrnoError(ERRNO_CODES.ENOENT);
        }
        if (!link.node_ops.readlink) {
          throw new FS.ErrnoError(ERRNO_CODES.EINVAL);
        }
        return link.node_ops.readlink(link);
      },stat:function (path, dontFollow) {
        var lookup = FS.lookupPath(path, { follow: !dontFollow });
        var node = lookup.node;
        if (!node) {
          throw new FS.ErrnoError(ERRNO_CODES.ENOENT);
        }
        if (!node.node_ops.getattr) {
          throw new FS.ErrnoError(ERRNO_CODES.EPERM);
        }
        return node.node_ops.getattr(node);
      },lstat:function (path) {
        return FS.stat(path, true);
      },chmod:function (path, mode, dontFollow) {
        var node;
        if (typeof path === 'string') {
          var lookup = FS.lookupPath(path, { follow: !dontFollow });
          node = lookup.node;
        } else {
          node = path;
        }
        if (!node.node_ops.setattr) {
          throw new FS.ErrnoError(ERRNO_CODES.EPERM);
        }
        node.node_ops.setattr(node, {
          mode: (mode & 4095) | (node.mode & ~4095),
          timestamp: Date.now()
        });
      },lchmod:function (path, mode) {
        FS.chmod(path, mode, true);
      },fchmod:function (fd, mode) {
        var stream = FS.getStream(fd);
        if (!stream) {
          throw new FS.ErrnoError(ERRNO_CODES.EBADF);
        }
        FS.chmod(stream.node, mode);
      },chown:function (path, uid, gid, dontFollow) {
        var node;
        if (typeof path === 'string') {
          var lookup = FS.lookupPath(path, { follow: !dontFollow });
          node = lookup.node;
        } else {
          node = path;
        }
        if (!node.node_ops.setattr) {
          throw new FS.ErrnoError(ERRNO_CODES.EPERM);
        }
        node.node_ops.setattr(node, {
          timestamp: Date.now()
          // we ignore the uid / gid for now
        });
      },lchown:function (path, uid, gid) {
        FS.chown(path, uid, gid, true);
      },fchown:function (fd, uid, gid) {
        var stream = FS.getStream(fd);
        if (!stream) {
          throw new FS.ErrnoError(ERRNO_CODES.EBADF);
        }
        FS.chown(stream.node, uid, gid);
      },truncate:function (path, len) {
        if (len < 0) {
          throw new FS.ErrnoError(ERRNO_CODES.EINVAL);
        }
        var node;
        if (typeof path === 'string') {
          var lookup = FS.lookupPath(path, { follow: true });
          node = lookup.node;
        } else {
          node = path;
        }
        if (!node.node_ops.setattr) {
          throw new FS.ErrnoError(ERRNO_CODES.EPERM);
        }
        if (FS.isDir(node.mode)) {
          throw new FS.ErrnoError(ERRNO_CODES.EISDIR);
        }
        if (!FS.isFile(node.mode)) {
          throw new FS.ErrnoError(ERRNO_CODES.EINVAL);
        }
        var err = FS.nodePermissions(node, 'w');
        if (err) {
          throw new FS.ErrnoError(err);
        }
        node.node_ops.setattr(node, {
          size: len,
          timestamp: Date.now()
        });
      },ftruncate:function (fd, len) {
        var stream = FS.getStream(fd);
        if (!stream) {
          throw new FS.ErrnoError(ERRNO_CODES.EBADF);
        }
        if ((stream.flags & 2097155) === 0) {
          throw new FS.ErrnoError(ERRNO_CODES.EINVAL);
        }
        FS.truncate(stream.node, len);
      },utime:function (path, atime, mtime) {
        var lookup = FS.lookupPath(path, { follow: true });
        var node = lookup.node;
        node.node_ops.setattr(node, {
          timestamp: Math.max(atime, mtime)
        });
      },open:function (path, flags, mode, fd_start, fd_end) {
        if (path === "") {
          throw new FS.ErrnoError(ERRNO_CODES.ENOENT);
        }
        flags = typeof flags === 'string' ? FS.modeStringToFlags(flags) : flags;
        mode = typeof mode === 'undefined' ? 438 /* 0666 */ : mode;
        if ((flags & 64)) {
          mode = (mode & 4095) | 32768;
        } else {
          mode = 0;
        }
        var node;
        if (typeof path === 'object') {
          node = path;
        } else {
          path = PATH.normalize(path);
          try {
            var lookup = FS.lookupPath(path, {
              follow: !(flags & 131072)
            });
            node = lookup.node;
          } catch (e) {
            // ignore
          }
        }
        // perhaps we need to create the node
        var created = false;
        if ((flags & 64)) {
          if (node) {
            // if O_CREAT and O_EXCL are set, error out if the node already exists
            if ((flags & 128)) {
              throw new FS.ErrnoError(ERRNO_CODES.EEXIST);
            }
          } else {
            // node doesn't exist, try to create it
            node = FS.mknod(path, mode, 0);
            created = true;
          }
        }
        if (!node) {
          throw new FS.ErrnoError(ERRNO_CODES.ENOENT);
        }
        // can't truncate a device
        if (FS.isChrdev(node.mode)) {
          flags &= ~512;
        }
        // check permissions, if this is not a file we just created now (it is ok to
        // create and write to a file with read-only permissions; it is read-only
        // for later use)
        if (!created) {
          var err = FS.mayOpen(node, flags);
          if (err) {
            throw new FS.ErrnoError(err);
          }
        }
        // do truncation if necessary
        if ((flags & 512)) {
          FS.truncate(node, 0);
        }
        // we've already handled these, don't pass down to the underlying vfs
        flags &= ~(128 | 512);
  
        // register the stream with the filesystem
        var stream = FS.createStream({
          node: node,
          path: FS.getPath(node),  // we want the absolute path to the node
          flags: flags,
          seekable: true,
          position: 0,
          stream_ops: node.stream_ops,
          // used by the file family libc calls (fopen, fwrite, ferror, etc.)
          ungotten: [],
          error: false
        }, fd_start, fd_end);
        // call the new stream's open function
        if (stream.stream_ops.open) {
          stream.stream_ops.open(stream);
        }
        if (Module['logReadFiles'] && !(flags & 1)) {
          if (!FS.readFiles) FS.readFiles = {};
          if (!(path in FS.readFiles)) {
            FS.readFiles[path] = 1;
            Module['printErr']('read file: ' + path);
          }
        }
        try {
          if (FS.trackingDelegate['onOpenFile']) {
            var trackingFlags = 0;
            if ((flags & 2097155) !== 1) {
              trackingFlags |= FS.tracking.openFlags.READ;
            }
            if ((flags & 2097155) !== 0) {
              trackingFlags |= FS.tracking.openFlags.WRITE;
            }
            FS.trackingDelegate['onOpenFile'](path, trackingFlags);
          }
        } catch(e) {
          console.log("FS.trackingDelegate['onOpenFile']('"+path+"', flags) threw an exception: " + e.message);
        }
        return stream;
      },close:function (stream) {
        try {
          if (stream.stream_ops.close) {
            stream.stream_ops.close(stream);
          }
        } catch (e) {
          throw e;
        } finally {
          FS.closeStream(stream.fd);
        }
      },llseek:function (stream, offset, whence) {
        if (!stream.seekable || !stream.stream_ops.llseek) {
          throw new FS.ErrnoError(ERRNO_CODES.ESPIPE);
        }
        stream.position = stream.stream_ops.llseek(stream, offset, whence);
        stream.ungotten = [];
        return stream.position;
      },read:function (stream, buffer, offset, length, position) {
        if (length < 0 || position < 0) {
          throw new FS.ErrnoError(ERRNO_CODES.EINVAL);
        }
        if ((stream.flags & 2097155) === 1) {
          throw new FS.ErrnoError(ERRNO_CODES.EBADF);
        }
        if (FS.isDir(stream.node.mode)) {
          throw new FS.ErrnoError(ERRNO_CODES.EISDIR);
        }
        if (!stream.stream_ops.read) {
          throw new FS.ErrnoError(ERRNO_CODES.EINVAL);
        }
        var seeking = true;
        if (typeof position === 'undefined') {
          position = stream.position;
          seeking = false;
        } else if (!stream.seekable) {
          throw new FS.ErrnoError(ERRNO_CODES.ESPIPE);
        }
        var bytesRead = stream.stream_ops.read(stream, buffer, offset, length, position);
        if (!seeking) stream.position += bytesRead;
        return bytesRead;
      },write:function (stream, buffer, offset, length, position, canOwn) {
        if (length < 0 || position < 0) {
          throw new FS.ErrnoError(ERRNO_CODES.EINVAL);
        }
        if ((stream.flags & 2097155) === 0) {
          throw new FS.ErrnoError(ERRNO_CODES.EBADF);
        }
        if (FS.isDir(stream.node.mode)) {
          throw new FS.ErrnoError(ERRNO_CODES.EISDIR);
        }
        if (!stream.stream_ops.write) {
          throw new FS.ErrnoError(ERRNO_CODES.EINVAL);
        }
        if (stream.flags & 1024) {
          // seek to the end before writing in append mode
          FS.llseek(stream, 0, 2);
        }
        var seeking = true;
        if (typeof position === 'undefined') {
          position = stream.position;
          seeking = false;
        } else if (!stream.seekable) {
          throw new FS.ErrnoError(ERRNO_CODES.ESPIPE);
        }
        var bytesWritten = stream.stream_ops.write(stream, buffer, offset, length, position, canOwn);
        if (!seeking) stream.position += bytesWritten;
        try {
          if (stream.path && FS.trackingDelegate['onWriteToFile']) FS.trackingDelegate['onWriteToFile'](stream.path);
        } catch(e) {
          console.log("FS.trackingDelegate['onWriteToFile']('"+path+"') threw an exception: " + e.message);
        }
        return bytesWritten;
      },allocate:function (stream, offset, length) {
        if (offset < 0 || length <= 0) {
          throw new FS.ErrnoError(ERRNO_CODES.EINVAL);
        }
        if ((stream.flags & 2097155) === 0) {
          throw new FS.ErrnoError(ERRNO_CODES.EBADF);
        }
        if (!FS.isFile(stream.node.mode) && !FS.isDir(node.mode)) {
          throw new FS.ErrnoError(ERRNO_CODES.ENODEV);
        }
        if (!stream.stream_ops.allocate) {
          throw new FS.ErrnoError(ERRNO_CODES.EOPNOTSUPP);
        }
        stream.stream_ops.allocate(stream, offset, length);
      },mmap:function (stream, buffer, offset, length, position, prot, flags) {
        // TODO if PROT is PROT_WRITE, make sure we have write access
        if ((stream.flags & 2097155) === 1) {
          throw new FS.ErrnoError(ERRNO_CODES.EACCES);
        }
        if (!stream.stream_ops.mmap) {
          throw new FS.ErrnoError(ERRNO_CODES.ENODEV);
        }
        return stream.stream_ops.mmap(stream, buffer, offset, length, position, prot, flags);
      },ioctl:function (stream, cmd, arg) {
        if (!stream.stream_ops.ioctl) {
          throw new FS.ErrnoError(ERRNO_CODES.ENOTTY);
        }
        return stream.stream_ops.ioctl(stream, cmd, arg);
      },readFile:function (path, opts) {
        opts = opts || {};
        opts.flags = opts.flags || 'r';
        opts.encoding = opts.encoding || 'binary';
        if (opts.encoding !== 'utf8' && opts.encoding !== 'binary') {
          throw new Error('Invalid encoding type "' + opts.encoding + '"');
        }
        var ret;
        var stream = FS.open(path, opts.flags);
        var stat = FS.stat(path);
        var length = stat.size;
        var buf = new Uint8Array(length);
        FS.read(stream, buf, 0, length, 0);
        if (opts.encoding === 'utf8') {
          ret = '';
          var utf8 = new Runtime.UTF8Processor();
          for (var i = 0; i < length; i++) {
            ret += utf8.processCChar(buf[i]);
          }
        } else if (opts.encoding === 'binary') {
          ret = buf;
        }
        FS.close(stream);
        return ret;
      },writeFile:function (path, data, opts) {
        opts = opts || {};
        opts.flags = opts.flags || 'w';
        opts.encoding = opts.encoding || 'utf8';
        if (opts.encoding !== 'utf8' && opts.encoding !== 'binary') {
          throw new Error('Invalid encoding type "' + opts.encoding + '"');
        }
        var stream = FS.open(path, opts.flags, opts.mode);
        if (opts.encoding === 'utf8') {
          var utf8 = new Runtime.UTF8Processor();
          var buf = new Uint8Array(utf8.processJSString(data));
          FS.write(stream, buf, 0, buf.length, 0, opts.canOwn);
        } else if (opts.encoding === 'binary') {
          FS.write(stream, data, 0, data.length, 0, opts.canOwn);
        }
        FS.close(stream);
      },cwd:function () {
        return FS.currentPath;
      },chdir:function (path) {
        var lookup = FS.lookupPath(path, { follow: true });
        if (!FS.isDir(lookup.node.mode)) {
          throw new FS.ErrnoError(ERRNO_CODES.ENOTDIR);
        }
        var err = FS.nodePermissions(lookup.node, 'x');
        if (err) {
          throw new FS.ErrnoError(err);
        }
        FS.currentPath = lookup.path;
      },createDefaultDirectories:function () {
        FS.mkdir('/tmp');
        FS.mkdir('/home');
        FS.mkdir('/home/web_user');
      },createDefaultDevices:function () {
        // create /dev
        FS.mkdir('/dev');
        // setup /dev/null
        FS.registerDevice(FS.makedev(1, 3), {
          read: function() { return 0; },
          write: function() { return 0; }
        });
        FS.mkdev('/dev/null', FS.makedev(1, 3));
        // setup /dev/tty and /dev/tty1
        // stderr needs to print output using Module['printErr']
        // so we register a second tty just for it.
        TTY.register(FS.makedev(5, 0), TTY.default_tty_ops);
        TTY.register(FS.makedev(6, 0), TTY.default_tty1_ops);
        FS.mkdev('/dev/tty', FS.makedev(5, 0));
        FS.mkdev('/dev/tty1', FS.makedev(6, 0));
        // setup /dev/[u]random
        var random_device;
        if (typeof crypto !== 'undefined') {
          // for modern web browsers
          var randomBuffer = new Uint8Array(1);
          random_device = function() { crypto.getRandomValues(randomBuffer); return randomBuffer[0]; };
        } else if (ENVIRONMENT_IS_NODE) {
          // for nodejs
          random_device = function() { return require('crypto').randomBytes(1)[0]; };
        } else {
          // default for ES5 platforms
          random_device = function() { return (Math.random()*256)|0; };
        }
        FS.createDevice('/dev', 'random', random_device);
        FS.createDevice('/dev', 'urandom', random_device);
        // we're not going to emulate the actual shm device,
        // just create the tmp dirs that reside in it commonly
        FS.mkdir('/dev/shm');
        FS.mkdir('/dev/shm/tmp');
      },createStandardStreams:function () {
        // TODO deprecate the old functionality of a single
        // input / output callback and that utilizes FS.createDevice
        // and instead require a unique set of stream ops
  
        // by default, we symlink the standard streams to the
        // default tty devices. however, if the standard streams
        // have been overwritten we create a unique device for
        // them instead.
        if (Module['stdin']) {
          FS.createDevice('/dev', 'stdin', Module['stdin']);
        } else {
          FS.symlink('/dev/tty', '/dev/stdin');
        }
        if (Module['stdout']) {
          FS.createDevice('/dev', 'stdout', null, Module['stdout']);
        } else {
          FS.symlink('/dev/tty', '/dev/stdout');
        }
        if (Module['stderr']) {
          FS.createDevice('/dev', 'stderr', null, Module['stderr']);
        } else {
          FS.symlink('/dev/tty1', '/dev/stderr');
        }
  
        // open default streams for the stdin, stdout and stderr devices
        var stdin = FS.open('/dev/stdin', 'r');
        HEAP32[((_stdin)>>2)]=FS.getPtrForStream(stdin);
        assert(stdin.fd === 0, 'invalid handle for stdin (' + stdin.fd + ')');
  
        var stdout = FS.open('/dev/stdout', 'w');
        HEAP32[((_stdout)>>2)]=FS.getPtrForStream(stdout);
        assert(stdout.fd === 1, 'invalid handle for stdout (' + stdout.fd + ')');
  
        var stderr = FS.open('/dev/stderr', 'w');
        HEAP32[((_stderr)>>2)]=FS.getPtrForStream(stderr);
        assert(stderr.fd === 2, 'invalid handle for stderr (' + stderr.fd + ')');
      },ensureErrnoError:function () {
        if (FS.ErrnoError) return;
        FS.ErrnoError = function ErrnoError(errno, node) {
          this.node = node;
          this.setErrno = function(errno) {
            this.errno = errno;
            for (var key in ERRNO_CODES) {
              if (ERRNO_CODES[key] === errno) {
                this.code = key;
                break;
              }
            }
          };
          this.setErrno(errno);
          this.message = ERRNO_MESSAGES[errno];
        };
        FS.ErrnoError.prototype = new Error();
        FS.ErrnoError.prototype.constructor = FS.ErrnoError;
        // Some errors may happen quite a bit, to avoid overhead we reuse them (and suffer a lack of stack info)
        [ERRNO_CODES.ENOENT].forEach(function(code) {
          FS.genericErrors[code] = new FS.ErrnoError(code);
          FS.genericErrors[code].stack = '<generic error, no stack>';
        });
      },staticInit:function () {
        FS.ensureErrnoError();
  
        FS.nameTable = new Array(4096);
  
        FS.mount(MEMFS, {}, '/');
  
        FS.createDefaultDirectories();
        FS.createDefaultDevices();
      },init:function (input, output, error) {
        assert(!FS.init.initialized, 'FS.init was previously called. If you want to initialize later with custom parameters, remove any earlier calls (note that one is automatically added to the generated code)');
        FS.init.initialized = true;
  
        FS.ensureErrnoError();
  
        // Allow Module.stdin etc. to provide defaults, if none explicitly passed to us here
        Module['stdin'] = input || Module['stdin'];
        Module['stdout'] = output || Module['stdout'];
        Module['stderr'] = error || Module['stderr'];
  
        FS.createStandardStreams();
      },quit:function () {
        FS.init.initialized = false;
        for (var i = 0; i < FS.streams.length; i++) {
          var stream = FS.streams[i];
          if (!stream) {
            continue;
          }
          FS.close(stream);
        }
      },getMode:function (canRead, canWrite) {
        var mode = 0;
        if (canRead) mode |= 292 | 73;
        if (canWrite) mode |= 146;
        return mode;
      },joinPath:function (parts, forceRelative) {
        var path = PATH.join.apply(null, parts);
        if (forceRelative && path[0] == '/') path = path.substr(1);
        return path;
      },absolutePath:function (relative, base) {
        return PATH.resolve(base, relative);
      },standardizePath:function (path) {
        return PATH.normalize(path);
      },findObject:function (path, dontResolveLastLink) {
        var ret = FS.analyzePath(path, dontResolveLastLink);
        if (ret.exists) {
          return ret.object;
        } else {
          ___setErrNo(ret.error);
          return null;
        }
      },analyzePath:function (path, dontResolveLastLink) {
        // operate from within the context of the symlink's target
        try {
          var lookup = FS.lookupPath(path, { follow: !dontResolveLastLink });
          path = lookup.path;
        } catch (e) {
        }
        var ret = {
          isRoot: false, exists: false, error: 0, name: null, path: null, object: null,
          parentExists: false, parentPath: null, parentObject: null
        };
        try {
          var lookup = FS.lookupPath(path, { parent: true });
          ret.parentExists = true;
          ret.parentPath = lookup.path;
          ret.parentObject = lookup.node;
          ret.name = PATH.basename(path);
          lookup = FS.lookupPath(path, { follow: !dontResolveLastLink });
          ret.exists = true;
          ret.path = lookup.path;
          ret.object = lookup.node;
          ret.name = lookup.node.name;
          ret.isRoot = lookup.path === '/';
        } catch (e) {
          ret.error = e.errno;
        };
        return ret;
      },createFolder:function (parent, name, canRead, canWrite) {
        var path = PATH.join2(typeof parent === 'string' ? parent : FS.getPath(parent), name);
        var mode = FS.getMode(canRead, canWrite);
        return FS.mkdir(path, mode);
      },createPath:function (parent, path, canRead, canWrite) {
        parent = typeof parent === 'string' ? parent : FS.getPath(parent);
        var parts = path.split('/').reverse();
        while (parts.length) {
          var part = parts.pop();
          if (!part) continue;
          var current = PATH.join2(parent, part);
          try {
            FS.mkdir(current);
          } catch (e) {
            // ignore EEXIST
          }
          parent = current;
        }
        return current;
      },createFile:function (parent, name, properties, canRead, canWrite) {
        var path = PATH.join2(typeof parent === 'string' ? parent : FS.getPath(parent), name);
        var mode = FS.getMode(canRead, canWrite);
        return FS.create(path, mode);
      },createDataFile:function (parent, name, data, canRead, canWrite, canOwn) {
        var path = name ? PATH.join2(typeof parent === 'string' ? parent : FS.getPath(parent), name) : parent;
        var mode = FS.getMode(canRead, canWrite);
        var node = FS.create(path, mode);
        if (data) {
          if (typeof data === 'string') {
            var arr = new Array(data.length);
            for (var i = 0, len = data.length; i < len; ++i) arr[i] = data.charCodeAt(i);
            data = arr;
          }
          // make sure we can write to the file
          FS.chmod(node, mode | 146);
          var stream = FS.open(node, 'w');
          FS.write(stream, data, 0, data.length, 0, canOwn);
          FS.close(stream);
          FS.chmod(node, mode);
        }
        return node;
      },createDevice:function (parent, name, input, output) {
        var path = PATH.join2(typeof parent === 'string' ? parent : FS.getPath(parent), name);
        var mode = FS.getMode(!!input, !!output);
        if (!FS.createDevice.major) FS.createDevice.major = 64;
        var dev = FS.makedev(FS.createDevice.major++, 0);
        // Create a fake device that a set of stream ops to emulate
        // the old behavior.
        FS.registerDevice(dev, {
          open: function(stream) {
            stream.seekable = false;
          },
          close: function(stream) {
            // flush any pending line data
            if (output && output.buffer && output.buffer.length) {
              output(10);
            }
          },
          read: function(stream, buffer, offset, length, pos /* ignored */) {
            var bytesRead = 0;
            for (var i = 0; i < length; i++) {
              var result;
              try {
                result = input();
              } catch (e) {
                throw new FS.ErrnoError(ERRNO_CODES.EIO);
              }
              if (result === undefined && bytesRead === 0) {
                throw new FS.ErrnoError(ERRNO_CODES.EAGAIN);
              }
              if (result === null || result === undefined) break;
              bytesRead++;
              buffer[offset+i] = result;
            }
            if (bytesRead) {
              stream.node.timestamp = Date.now();
            }
            return bytesRead;
          },
          write: function(stream, buffer, offset, length, pos) {
            for (var i = 0; i < length; i++) {
              try {
                output(buffer[offset+i]);
              } catch (e) {
                throw new FS.ErrnoError(ERRNO_CODES.EIO);
              }
            }
            if (length) {
              stream.node.timestamp = Date.now();
            }
            return i;
          }
        });
        return FS.mkdev(path, mode, dev);
      },createLink:function (parent, name, target, canRead, canWrite) {
        var path = PATH.join2(typeof parent === 'string' ? parent : FS.getPath(parent), name);
        return FS.symlink(target, path);
      },forceLoadFile:function (obj) {
        if (obj.isDevice || obj.isFolder || obj.link || obj.contents) return true;
        var success = true;
        if (typeof XMLHttpRequest !== 'undefined') {
          throw new Error("Lazy loading should have been performed (contents set) in createLazyFile, but it was not. Lazy loading only works in web workers. Use --embed-file or --preload-file in emcc on the main thread.");
        } else if (Module['read']) {
          // Command-line.
          try {
            // WARNING: Can't read binary files in V8's d8 or tracemonkey's js, as
            //          read() will try to parse UTF8.
            obj.contents = intArrayFromString(Module['read'](obj.url), true);
            obj.usedBytes = obj.contents.length;
          } catch (e) {
            success = false;
          }
        } else {
          throw new Error('Cannot load without read() or XMLHttpRequest.');
        }
        if (!success) ___setErrNo(ERRNO_CODES.EIO);
        return success;
      },createLazyFile:function (parent, name, url, canRead, canWrite) {
        // Lazy chunked Uint8Array (implements get and length from Uint8Array). Actual getting is abstracted away for eventual reuse.
        function LazyUint8Array() {
          this.lengthKnown = false;
          this.chunks = []; // Loaded chunks. Index is the chunk number
        }
        LazyUint8Array.prototype.get = function LazyUint8Array_get(idx) {
          if (idx > this.length-1 || idx < 0) {
            return undefined;
          }
          var chunkOffset = idx % this.chunkSize;
          var chunkNum = (idx / this.chunkSize)|0;
          return this.getter(chunkNum)[chunkOffset];
        }
        LazyUint8Array.prototype.setDataGetter = function LazyUint8Array_setDataGetter(getter) {
          this.getter = getter;
        }
        LazyUint8Array.prototype.cacheLength = function LazyUint8Array_cacheLength() {
          // Find length
          var xhr = new XMLHttpRequest();
          xhr.open('HEAD', url, false);
          xhr.send(null);
          if (!(xhr.status >= 200 && xhr.status < 300 || xhr.status === 304)) throw new Error("Couldn't load " + url + ". Status: " + xhr.status);
          var datalength = Number(xhr.getResponseHeader("Content-length"));
          var header;
          var hasByteServing = (header = xhr.getResponseHeader("Accept-Ranges")) && header === "bytes";
          var chunkSize = 1024*1024; // Chunk size in bytes
  
          if (!hasByteServing) chunkSize = datalength;
  
          // Function to get a range from the remote URL.
          var doXHR = (function(from, to) {
            if (from > to) throw new Error("invalid range (" + from + ", " + to + ") or no bytes requested!");
            if (to > datalength-1) throw new Error("only " + datalength + " bytes available! programmer error!");
  
            // TODO: Use mozResponseArrayBuffer, responseStream, etc. if available.
            var xhr = new XMLHttpRequest();
            xhr.open('GET', url, false);
            if (datalength !== chunkSize) xhr.setRequestHeader("Range", "bytes=" + from + "-" + to);
  
            // Some hints to the browser that we want binary data.
            if (typeof Uint8Array != 'undefined') xhr.responseType = 'arraybuffer';
            if (xhr.overrideMimeType) {
              xhr.overrideMimeType('text/plain; charset=x-user-defined');
            }
  
            xhr.send(null);
            if (!(xhr.status >= 200 && xhr.status < 300 || xhr.status === 304)) throw new Error("Couldn't load " + url + ". Status: " + xhr.status);
            if (xhr.response !== undefined) {
              return new Uint8Array(xhr.response || []);
            } else {
              return intArrayFromString(xhr.responseText || '', true);
            }
          });
          var lazyArray = this;
          lazyArray.setDataGetter(function(chunkNum) {
            var start = chunkNum * chunkSize;
            var end = (chunkNum+1) * chunkSize - 1; // including this byte
            end = Math.min(end, datalength-1); // if datalength-1 is selected, this is the last block
            if (typeof(lazyArray.chunks[chunkNum]) === "undefined") {
              lazyArray.chunks[chunkNum] = doXHR(start, end);
            }
            if (typeof(lazyArray.chunks[chunkNum]) === "undefined") throw new Error("doXHR failed!");
            return lazyArray.chunks[chunkNum];
          });
  
          this._length = datalength;
          this._chunkSize = chunkSize;
          this.lengthKnown = true;
        }
        if (typeof XMLHttpRequest !== 'undefined') {
          if (!ENVIRONMENT_IS_WORKER) throw 'Cannot do synchronous binary XHRs outside webworkers in modern browsers. Use --embed-file or --preload-file in emcc';
          var lazyArray = new LazyUint8Array();
          Object.defineProperty(lazyArray, "length", {
              get: function() {
                  if(!this.lengthKnown) {
                      this.cacheLength();
                  }
                  return this._length;
              }
          });
          Object.defineProperty(lazyArray, "chunkSize", {
              get: function() {
                  if(!this.lengthKnown) {
                      this.cacheLength();
                  }
                  return this._chunkSize;
              }
          });
  
          var properties = { isDevice: false, contents: lazyArray };
        } else {
          var properties = { isDevice: false, url: url };
        }
  
        var node = FS.createFile(parent, name, properties, canRead, canWrite);
        // This is a total hack, but I want to get this lazy file code out of the
        // core of MEMFS. If we want to keep this lazy file concept I feel it should
        // be its own thin LAZYFS proxying calls to MEMFS.
        if (properties.contents) {
          node.contents = properties.contents;
        } else if (properties.url) {
          node.contents = null;
          node.url = properties.url;
        }
        // Add a function that defers querying the file size until it is asked the first time.
        Object.defineProperty(node, "usedBytes", {
            get: function() { return this.contents.length; }
        });
        // override each stream op with one that tries to force load the lazy file first
        var stream_ops = {};
        var keys = Object.keys(node.stream_ops);
        keys.forEach(function(key) {
          var fn = node.stream_ops[key];
          stream_ops[key] = function forceLoadLazyFile() {
            if (!FS.forceLoadFile(node)) {
              throw new FS.ErrnoError(ERRNO_CODES.EIO);
            }
            return fn.apply(null, arguments);
          };
        });
        // use a custom read function
        stream_ops.read = function stream_ops_read(stream, buffer, offset, length, position) {
          if (!FS.forceLoadFile(node)) {
            throw new FS.ErrnoError(ERRNO_CODES.EIO);
          }
          var contents = stream.node.contents;
          if (position >= contents.length)
            return 0;
          var size = Math.min(contents.length - position, length);
          assert(size >= 0);
          if (contents.slice) { // normal array
            for (var i = 0; i < size; i++) {
              buffer[offset + i] = contents[position + i];
            }
          } else {
            for (var i = 0; i < size; i++) { // LazyUint8Array from sync binary XHR
              buffer[offset + i] = contents.get(position + i);
            }
          }
          return size;
        };
        node.stream_ops = stream_ops;
        return node;
      },createPreloadedFile:function (parent, name, url, canRead, canWrite, onload, onerror, dontCreateFile, canOwn) {
        Browser.init();
        // TODO we should allow people to just pass in a complete filename instead
        // of parent and name being that we just join them anyways
        var fullname = name ? PATH.resolve(PATH.join2(parent, name)) : parent;
        function processData(byteArray) {
          function finish(byteArray) {
            if (!dontCreateFile) {
              FS.createDataFile(parent, name, byteArray, canRead, canWrite, canOwn);
            }
            if (onload) onload();
            removeRunDependency('cp ' + fullname);
          }
          var handled = false;
          Module['preloadPlugins'].forEach(function(plugin) {
            if (handled) return;
            if (plugin['canHandle'](fullname)) {
              plugin['handle'](byteArray, fullname, finish, function() {
                if (onerror) onerror();
                removeRunDependency('cp ' + fullname);
              });
              handled = true;
            }
          });
          if (!handled) finish(byteArray);
        }
        addRunDependency('cp ' + fullname);
        if (typeof url == 'string') {
          Browser.asyncLoad(url, function(byteArray) {
            processData(byteArray);
          }, onerror);
        } else {
          processData(url);
        }
      },indexedDB:function () {
        return window.indexedDB || window.mozIndexedDB || window.webkitIndexedDB || window.msIndexedDB;
      },DB_NAME:function () {
        return 'EM_FS_' + window.location.pathname;
      },DB_VERSION:20,DB_STORE_NAME:"FILE_DATA",saveFilesToDB:function (paths, onload, onerror) {
        onload = onload || function(){};
        onerror = onerror || function(){};
        var indexedDB = FS.indexedDB();
        try {
          var openRequest = indexedDB.open(FS.DB_NAME(), FS.DB_VERSION);
        } catch (e) {
          return onerror(e);
        }
        openRequest.onupgradeneeded = function openRequest_onupgradeneeded() {
          console.log('creating db');
          var db = openRequest.result;
          db.createObjectStore(FS.DB_STORE_NAME);
        };
        openRequest.onsuccess = function openRequest_onsuccess() {
          var db = openRequest.result;
          var transaction = db.transaction([FS.DB_STORE_NAME], 'readwrite');
          var files = transaction.objectStore(FS.DB_STORE_NAME);
          var ok = 0, fail = 0, total = paths.length;
          function finish() {
            if (fail == 0) onload(); else onerror();
          }
          paths.forEach(function(path) {
            var putRequest = files.put(FS.analyzePath(path).object.contents, path);
            putRequest.onsuccess = function putRequest_onsuccess() { ok++; if (ok + fail == total) finish() };
            putRequest.onerror = function putRequest_onerror() { fail++; if (ok + fail == total) finish() };
          });
          transaction.onerror = onerror;
        };
        openRequest.onerror = onerror;
      },loadFilesFromDB:function (paths, onload, onerror) {
        onload = onload || function(){};
        onerror = onerror || function(){};
        var indexedDB = FS.indexedDB();
        try {
          var openRequest = indexedDB.open(FS.DB_NAME(), FS.DB_VERSION);
        } catch (e) {
          return onerror(e);
        }
        openRequest.onupgradeneeded = onerror; // no database to load from
        openRequest.onsuccess = function openRequest_onsuccess() {
          var db = openRequest.result;
          try {
            var transaction = db.transaction([FS.DB_STORE_NAME], 'readonly');
          } catch(e) {
            onerror(e);
            return;
          }
          var files = transaction.objectStore(FS.DB_STORE_NAME);
          var ok = 0, fail = 0, total = paths.length;
          function finish() {
            if (fail == 0) onload(); else onerror();
          }
          paths.forEach(function(path) {
            var getRequest = files.get(path);
            getRequest.onsuccess = function getRequest_onsuccess() {
              if (FS.analyzePath(path).exists) {
                FS.unlink(path);
              }
              FS.createDataFile(PATH.dirname(path), PATH.basename(path), getRequest.result, true, true, true);
              ok++;
              if (ok + fail == total) finish();
            };
            getRequest.onerror = function getRequest_onerror() { fail++; if (ok + fail == total) finish() };
          });
          transaction.onerror = onerror;
        };
        openRequest.onerror = onerror;
      }};function _lseek(fildes, offset, whence) {
      // off_t lseek(int fildes, off_t offset, int whence);
      // http://pubs.opengroup.org/onlinepubs/000095399/functions/lseek.html
      var stream = FS.getStream(fildes);
      if (!stream) {
        ___setErrNo(ERRNO_CODES.EBADF);
        return -1;
      }
      try {
        return FS.llseek(stream, offset, whence);
      } catch (e) {
        FS.handleFSError(e);
        return -1;
      }
    }

   
  Module["_i64Subtract"] = _i64Subtract;

  var _DtoILow=true;

  function _sysconf(name) {
      // long sysconf(int name);
      // http://pubs.opengroup.org/onlinepubs/009695399/functions/sysconf.html
      switch(name) {
        case 30: return PAGE_SIZE;
        case 132:
        case 133:
        case 12:
        case 137:
        case 138:
        case 15:
        case 235:
        case 16:
        case 17:
        case 18:
        case 19:
        case 20:
        case 149:
        case 13:
        case 10:
        case 236:
        case 153:
        case 9:
        case 21:
        case 22:
        case 159:
        case 154:
        case 14:
        case 77:
        case 78:
        case 139:
        case 80:
        case 81:
        case 79:
        case 82:
        case 68:
        case 67:
        case 164:
        case 11:
        case 29:
        case 47:
        case 48:
        case 95:
        case 52:
        case 51:
        case 46:
          return 200809;
        case 27:
        case 246:
        case 127:
        case 128:
        case 23:
        case 24:
        case 160:
        case 161:
        case 181:
        case 182:
        case 242:
        case 183:
        case 184:
        case 243:
        case 244:
        case 245:
        case 165:
        case 178:
        case 179:
        case 49:
        case 50:
        case 168:
        case 169:
        case 175:
        case 170:
        case 171:
        case 172:
        case 97:
        case 76:
        case 32:
        case 173:
        case 35:
          return -1;
        case 176:
        case 177:
        case 7:
        case 155:
        case 8:
        case 157:
        case 125:
        case 126:
        case 92:
        case 93:
        case 129:
        case 130:
        case 131:
        case 94:
        case 91:
          return 1;
        case 74:
        case 60:
        case 69:
        case 70:
        case 4:
          return 1024;
        case 31:
        case 42:
        case 72:
          return 32;
        case 87:
        case 26:
        case 33:
          return 2147483647;
        case 34:
        case 1:
          return 47839;
        case 38:
        case 36:
          return 99;
        case 43:
        case 37:
          return 2048;
        case 0: return 2097152;
        case 3: return 65536;
        case 28: return 32768;
        case 44: return 32767;
        case 75: return 16384;
        case 39: return 1000;
        case 89: return 700;
        case 71: return 256;
        case 40: return 255;
        case 2: return 100;
        case 180: return 64;
        case 25: return 20;
        case 5: return 16;
        case 6: return 6;
        case 73: return 4;
        case 84: {
          if (typeof navigator === 'object') return navigator['hardwareConcurrency'] || 1;
          return 1;
        }
      }
      ___setErrNo(ERRNO_CODES.EINVAL);
      return -1;
    }

  var _SItoD=true;

   
  Module["_memset"] = _memset;

  function _stat(path, buf, dontResolveLastLink) {
      // http://pubs.opengroup.org/onlinepubs/7908799/xsh/stat.html
      // int stat(const char *path, struct stat *buf);
      // NOTE: dontResolveLastLink is a shortcut for lstat(). It should never be
      //       used in client code.
      path = typeof path !== 'string' ? Pointer_stringify(path) : path;
      try {
        var stat = dontResolveLastLink ? FS.lstat(path) : FS.stat(path);
        HEAP32[((buf)>>2)]=stat.dev;
        HEAP32[(((buf)+(4))>>2)]=0;
        HEAP32[(((buf)+(8))>>2)]=stat.ino;
        HEAP32[(((buf)+(12))>>2)]=stat.mode;
        HEAP32[(((buf)+(16))>>2)]=stat.nlink;
        HEAP32[(((buf)+(20))>>2)]=stat.uid;
        HEAP32[(((buf)+(24))>>2)]=stat.gid;
        HEAP32[(((buf)+(28))>>2)]=stat.rdev;
        HEAP32[(((buf)+(32))>>2)]=0;
        HEAP32[(((buf)+(36))>>2)]=stat.size;
        HEAP32[(((buf)+(40))>>2)]=4096;
        HEAP32[(((buf)+(44))>>2)]=stat.blocks;
        HEAP32[(((buf)+(48))>>2)]=(stat.atime.getTime() / 1000)|0;
        HEAP32[(((buf)+(52))>>2)]=0;
        HEAP32[(((buf)+(56))>>2)]=(stat.mtime.getTime() / 1000)|0;
        HEAP32[(((buf)+(60))>>2)]=0;
        HEAP32[(((buf)+(64))>>2)]=(stat.ctime.getTime() / 1000)|0;
        HEAP32[(((buf)+(68))>>2)]=0;
        HEAP32[(((buf)+(72))>>2)]=stat.ino;
        return 0;
      } catch (e) {
        if (e.node && PATH.normalize(path) !== PATH.normalize(FS.getPath(e.node))) {
          // an error occurred while trying to look up the path; we should just report ENOTDIR
          e.setErrno(ERRNO_CODES.ENOTDIR);
        }
        FS.handleFSError(e);
        return -1;
      }
    }

  var _DtoIHigh=true;

   
  Module["_bitshift64Shl"] = _bitshift64Shl;

  function _abort() {
      Module['abort']();
    }

  
  function _usleep(useconds) {
      // int usleep(useconds_t useconds);
      // http://pubs.opengroup.org/onlinepubs/000095399/functions/usleep.html
      // We're single-threaded, so use a busy loop. Super-ugly.
      var msec = useconds / 1000;
      if ((ENVIRONMENT_IS_WEB || ENVIRONMENT_IS_WORKER) && self['performance'] && self['performance']['now']) {
        var start = self['performance']['now']();
        while (self['performance']['now']() - start < msec) {
          // Do nothing.
        }
      } else {
        var start = Date.now();
        while (Date.now() - start < msec) {
          // Do nothing.
        }
      }
      return 0;
    }function _sleep(seconds) {
      // unsigned sleep(unsigned seconds);
      // http://pubs.opengroup.org/onlinepubs/000095399/functions/sleep.html
      return _usleep(seconds * 1e6);
    }

  function _close(fildes) {
      // int close(int fildes);
      // http://pubs.opengroup.org/onlinepubs/000095399/functions/close.html
      var stream = FS.getStream(fildes);
      if (!stream) {
        ___setErrNo(ERRNO_CODES.EBADF);
        return -1;
      }
      try {
        FS.close(stream);
        return 0;
      } catch (e) {
        FS.handleFSError(e);
        return -1;
      }
    }

   
  Module["_strlen"] = _strlen;

  function _access(path, amode) {
      // int access(const char *path, int amode);
      // http://pubs.opengroup.org/onlinepubs/000095399/functions/access.html
      path = Pointer_stringify(path);
      if (amode & ~7) {
        // need a valid mode
        ___setErrNo(ERRNO_CODES.EINVAL);
        return -1;
      }
      var node;
      try {
        var lookup = FS.lookupPath(path, { follow: true });
        node = lookup.node;
      } catch (e) {
        FS.handleFSError(e);
        return -1;
      }
      var perms = '';
      if (amode & 4) perms += 'r';
      if (amode & 2) perms += 'w';
      if (amode & 1) perms += 'x';
      if (perms /* otherwise, they've just passed F_OK */ && FS.nodePermissions(node, perms)) {
        ___setErrNo(ERRNO_CODES.EACCES);
        return -1;
      }
      return 0;
    }

  function _unlink(path) {
      // int unlink(const char *path);
      // http://pubs.opengroup.org/onlinepubs/000095399/functions/unlink.html
      path = Pointer_stringify(path);
      try {
        FS.unlink(path);
        return 0;
      } catch (e) {
        FS.handleFSError(e);
        return -1;
      }
    }

   
  Module["_i64Add"] = _i64Add;

  function _open(path, oflag, varargs) {
      // int open(const char *path, int oflag, ...);
      // http://pubs.opengroup.org/onlinepubs/009695399/functions/open.html
      var mode = HEAP32[((varargs)>>2)];
      path = Pointer_stringify(path);
      try {
        var stream = FS.open(path, oflag, mode);
        return stream.fd;
      } catch (e) {
        FS.handleFSError(e);
        return -1;
      }
    }

  
  function _getuid() { return 0; }function _geteuid() {
  return _getuid.apply(null, arguments)
  }

  
  function _mknod(path, mode, dev) {
      // int mknod(const char *path, mode_t mode, dev_t dev);
      // http://pubs.opengroup.org/onlinepubs/7908799/xsh/mknod.html
      path = Pointer_stringify(path);
      // we don't want this in the JS API as the JS API
      // uses mknod to create all nodes.
      switch (mode & 61440) {
        case 32768:
        case 8192:
        case 24576:
        case 4096:
        case 49152:
          break;
        default:
          ___setErrNo(ERRNO_CODES.EINVAL);
          return -1;
      }
      try {
        FS.mknod(path, mode, dev);
        return 0;
      } catch (e) {
        FS.handleFSError(e);
        return -1;
      }
    }function _mkdir(path, mode) {
      // int mkdir(const char *path, mode_t mode);
      // http://pubs.opengroup.org/onlinepubs/7908799/xsh/mkdir.html
      path = Pointer_stringify(path);
      // remove a trailing slash, if one - /a/b/ has basename of '', but
      // we want to create b in the context of this function
      path = PATH.normalize(path);
      if (path[path.length-1] === '/') path = path.substr(0, path.length-1);
      try {
        FS.mkdir(path, mode, 0);
        return 0;
      } catch (e) {
        FS.handleFSError(e);
        return -1;
      }
    }

  
  
  
  function _mkport() { throw 'TODO' }var SOCKFS={mount:function (mount) {
        // If Module['websocket'] has already been defined (e.g. for configuring
        // the subprotocol/url) use that, if not initialise it to a new object.
        Module['websocket'] = (Module['websocket'] && 
                               ('object' === typeof Module['websocket'])) ? Module['websocket'] : {};
  
        // Add the Event registration mechanism to the exported websocket configuration
        // object so we can register network callbacks from native JavaScript too.
        // For more documentation see system/include/emscripten/emscripten.h
        Module['websocket']._callbacks = {};
        Module['websocket']['on'] = function(event, callback) {
  	    if ('function' === typeof callback) {
  		  this._callbacks[event] = callback;
          }
  	    return this;
        };
  
        Module['websocket'].emit = function(event, param) {
  	    if ('function' === typeof this._callbacks[event]) {
  		  this._callbacks[event].call(this, param);
          }
        };
  
        // If debug is enabled register simple default logging callbacks for each Event.
  
        return FS.createNode(null, '/', 16384 | 511 /* 0777 */, 0);
      },createSocket:function (family, type, protocol) {
        var streaming = type == 1;
        if (protocol) {
          assert(streaming == (protocol == 6)); // if SOCK_STREAM, must be tcp
        }
  
        // create our internal socket structure
        var sock = {
          family: family,
          type: type,
          protocol: protocol,
          server: null,
          error: null, // Used in getsockopt for SOL_SOCKET/SO_ERROR test
          peers: {},
          pending: [],
          recv_queue: [],
          sock_ops: SOCKFS.websocket_sock_ops
        };
  
        // create the filesystem node to store the socket structure
        var name = SOCKFS.nextname();
        var node = FS.createNode(SOCKFS.root, name, 49152, 0);
        node.sock = sock;
  
        // and the wrapping stream that enables library functions such
        // as read and write to indirectly interact with the socket
        var stream = FS.createStream({
          path: name,
          node: node,
          flags: FS.modeStringToFlags('r+'),
          seekable: false,
          stream_ops: SOCKFS.stream_ops
        });
  
        // map the new stream to the socket structure (sockets have a 1:1
        // relationship with a stream)
        sock.stream = stream;
  
        return sock;
      },getSocket:function (fd) {
        var stream = FS.getStream(fd);
        if (!stream || !FS.isSocket(stream.node.mode)) {
          return null;
        }
        return stream.node.sock;
      },stream_ops:{poll:function (stream) {
          var sock = stream.node.sock;
          return sock.sock_ops.poll(sock);
        },ioctl:function (stream, request, varargs) {
          var sock = stream.node.sock;
          return sock.sock_ops.ioctl(sock, request, varargs);
        },read:function (stream, buffer, offset, length, position /* ignored */) {
          var sock = stream.node.sock;
          var msg = sock.sock_ops.recvmsg(sock, length);
          if (!msg) {
            // socket is closed
            return 0;
          }
          buffer.set(msg.buffer, offset);
          return msg.buffer.length;
        },write:function (stream, buffer, offset, length, position /* ignored */) {
          var sock = stream.node.sock;
          return sock.sock_ops.sendmsg(sock, buffer, offset, length);
        },close:function (stream) {
          var sock = stream.node.sock;
          sock.sock_ops.close(sock);
        }},nextname:function () {
        if (!SOCKFS.nextname.current) {
          SOCKFS.nextname.current = 0;
        }
        return 'socket[' + (SOCKFS.nextname.current++) + ']';
      },websocket_sock_ops:{createPeer:function (sock, addr, port) {
          var ws;
  
          if (typeof addr === 'object') {
            ws = addr;
            addr = null;
            port = null;
          }
  
          if (ws) {
            // for sockets that've already connected (e.g. we're the server)
            // we can inspect the _socket property for the address
            if (ws._socket) {
              addr = ws._socket.remoteAddress;
              port = ws._socket.remotePort;
            }
            // if we're just now initializing a connection to the remote,
            // inspect the url property
            else {
              var result = /ws[s]?:\/\/([^:]+):(\d+)/.exec(ws.url);
              if (!result) {
                throw new Error('WebSocket URL must be in the format ws(s)://address:port');
              }
              addr = result[1];
              port = parseInt(result[2], 10);
            }
          } else {
            // create the actual websocket object and connect
            try {
              // runtimeConfig gets set to true if WebSocket runtime configuration is available.
              var runtimeConfig = (Module['websocket'] && ('object' === typeof Module['websocket']));
  
              // The default value is 'ws://' the replace is needed because the compiler replaces '//' comments with '#'
              // comments without checking context, so we'd end up with ws:#, the replace swaps the '#' for '//' again.
              var url = 'ws:#'.replace('#', '//');
  
              if (runtimeConfig) {
                if ('string' === typeof Module['websocket']['url']) {
                  url = Module['websocket']['url']; // Fetch runtime WebSocket URL config.
                }
              }
  
              if (url === 'ws://' || url === 'wss://') { // Is the supplied URL config just a prefix, if so complete it.
                var parts = addr.split('/');
                url = url + parts[0] + ":" + port + "/" + parts.slice(1).join('/');
              }
  
              // Make the WebSocket subprotocol (Sec-WebSocket-Protocol) default to binary if no configuration is set.
              var subProtocols = 'binary'; // The default value is 'binary'
  
              if (runtimeConfig) {
                if ('string' === typeof Module['websocket']['subprotocol']) {
                  subProtocols = Module['websocket']['subprotocol']; // Fetch runtime WebSocket subprotocol config.
                }
              }
  
              // The regex trims the string (removes spaces at the beginning and end, then splits the string by
              // <any space>,<any space> into an Array. Whitespace removal is important for Websockify and ws.
              subProtocols = subProtocols.replace(/^ +| +$/g,"").split(/ *, */);
  
              // The node ws library API for specifying optional subprotocol is slightly different than the browser's.
              var opts = ENVIRONMENT_IS_NODE ? {'protocol': subProtocols.toString()} : subProtocols;
  
              // If node we use the ws library.
              var WebSocket = ENVIRONMENT_IS_NODE ? require('ws') : window['WebSocket'];
              ws = new WebSocket(url, opts);
              ws.binaryType = 'arraybuffer';
            } catch (e) {
              throw new FS.ErrnoError(ERRNO_CODES.EHOSTUNREACH);
            }
          }
  
  
          var peer = {
            addr: addr,
            port: port,
            socket: ws,
            dgram_send_queue: []
          };
  
          SOCKFS.websocket_sock_ops.addPeer(sock, peer);
          SOCKFS.websocket_sock_ops.handlePeerEvents(sock, peer);
  
          // if this is a bound dgram socket, send the port number first to allow
          // us to override the ephemeral port reported to us by remotePort on the
          // remote end.
          if (sock.type === 2 && typeof sock.sport !== 'undefined') {
            peer.dgram_send_queue.push(new Uint8Array([
                255, 255, 255, 255,
                'p'.charCodeAt(0), 'o'.charCodeAt(0), 'r'.charCodeAt(0), 't'.charCodeAt(0),
                ((sock.sport & 0xff00) >> 8) , (sock.sport & 0xff)
            ]));
          }
  
          return peer;
        },getPeer:function (sock, addr, port) {
          return sock.peers[addr + ':' + port];
        },addPeer:function (sock, peer) {
          sock.peers[peer.addr + ':' + peer.port] = peer;
        },removePeer:function (sock, peer) {
          delete sock.peers[peer.addr + ':' + peer.port];
        },handlePeerEvents:function (sock, peer) {
          var first = true;
  
          var handleOpen = function () {
  
            Module['websocket'].emit('open', sock.stream.fd);
  
            try {
              var queued = peer.dgram_send_queue.shift();
              while (queued) {
                peer.socket.send(queued);
                queued = peer.dgram_send_queue.shift();
              }
            } catch (e) {
              // not much we can do here in the way of proper error handling as we've already
              // lied and said this data was sent. shut it down.
              peer.socket.close();
            }
          };
  
          function handleMessage(data) {
            assert(typeof data !== 'string' && data.byteLength !== undefined);  // must receive an ArrayBuffer
            data = new Uint8Array(data);  // make a typed array view on the array buffer
  
  
            // if this is the port message, override the peer's port with it
            var wasfirst = first;
            first = false;
            if (wasfirst &&
                data.length === 10 &&
                data[0] === 255 && data[1] === 255 && data[2] === 255 && data[3] === 255 &&
                data[4] === 'p'.charCodeAt(0) && data[5] === 'o'.charCodeAt(0) && data[6] === 'r'.charCodeAt(0) && data[7] === 't'.charCodeAt(0)) {
              // update the peer's port and it's key in the peer map
              var newport = ((data[8] << 8) | data[9]);
              SOCKFS.websocket_sock_ops.removePeer(sock, peer);
              peer.port = newport;
              SOCKFS.websocket_sock_ops.addPeer(sock, peer);
              return;
            }
  
            sock.recv_queue.push({ addr: peer.addr, port: peer.port, data: data });
            Module['websocket'].emit('message', sock.stream.fd);
          };
  
          if (ENVIRONMENT_IS_NODE) {
            peer.socket.on('open', handleOpen);
            peer.socket.on('message', function(data, flags) {
              if (!flags.binary) {
                return;
              }
              handleMessage((new Uint8Array(data)).buffer);  // copy from node Buffer -> ArrayBuffer
            });
            peer.socket.on('close', function() {
              Module['websocket'].emit('close', sock.stream.fd);
            });
            peer.socket.on('error', function(error) {
              // Although the ws library may pass errors that may be more descriptive than
              // ECONNREFUSED they are not necessarily the expected error code e.g. 
              // ENOTFOUND on getaddrinfo seems to be node.js specific, so using ECONNREFUSED
              // is still probably the most useful thing to do.
              sock.error = ERRNO_CODES.ECONNREFUSED; // Used in getsockopt for SOL_SOCKET/SO_ERROR test.
              Module['websocket'].emit('error', [sock.stream.fd, sock.error, 'ECONNREFUSED: Connection refused']);
              // don't throw
            });
          } else {
            peer.socket.onopen = handleOpen;
            peer.socket.onclose = function() {
              Module['websocket'].emit('close', sock.stream.fd);
            };
            peer.socket.onmessage = function peer_socket_onmessage(event) {
              handleMessage(event.data);
            };
            peer.socket.onerror = function(error) {
              // The WebSocket spec only allows a 'simple event' to be thrown on error,
              // so we only really know as much as ECONNREFUSED.
              sock.error = ERRNO_CODES.ECONNREFUSED; // Used in getsockopt for SOL_SOCKET/SO_ERROR test.
              Module['websocket'].emit('error', [sock.stream.fd, sock.error, 'ECONNREFUSED: Connection refused']);
            };
          }
        },poll:function (sock) {
          if (sock.type === 1 && sock.server) {
            // listen sockets should only say they're available for reading
            // if there are pending clients.
            return sock.pending.length ? (64 | 1) : 0;
          }
  
          var mask = 0;
          var dest = sock.type === 1 ?  // we only care about the socket state for connection-based sockets
            SOCKFS.websocket_sock_ops.getPeer(sock, sock.daddr, sock.dport) :
            null;
  
          if (sock.recv_queue.length ||
              !dest ||  // connection-less sockets are always ready to read
              (dest && dest.socket.readyState === dest.socket.CLOSING) ||
              (dest && dest.socket.readyState === dest.socket.CLOSED)) {  // let recv return 0 once closed
            mask |= (64 | 1);
          }
  
          if (!dest ||  // connection-less sockets are always ready to write
              (dest && dest.socket.readyState === dest.socket.OPEN)) {
            mask |= 4;
          }
  
          if ((dest && dest.socket.readyState === dest.socket.CLOSING) ||
              (dest && dest.socket.readyState === dest.socket.CLOSED)) {
            mask |= 16;
          }
  
          return mask;
        },ioctl:function (sock, request, arg) {
          switch (request) {
            case 21531:
              var bytes = 0;
              if (sock.recv_queue.length) {
                bytes = sock.recv_queue[0].data.length;
              }
              HEAP32[((arg)>>2)]=bytes;
              return 0;
            default:
              return ERRNO_CODES.EINVAL;
          }
        },close:function (sock) {
          // if we've spawned a listen server, close it
          if (sock.server) {
            try {
              sock.server.close();
            } catch (e) {
            }
            sock.server = null;
          }
          // close any peer connections
          var peers = Object.keys(sock.peers);
          for (var i = 0; i < peers.length; i++) {
            var peer = sock.peers[peers[i]];
            try {
              peer.socket.close();
            } catch (e) {
            }
            SOCKFS.websocket_sock_ops.removePeer(sock, peer);
          }
          return 0;
        },bind:function (sock, addr, port) {
          if (typeof sock.saddr !== 'undefined' || typeof sock.sport !== 'undefined') {
            throw new FS.ErrnoError(ERRNO_CODES.EINVAL);  // already bound
          }
          sock.saddr = addr;
          sock.sport = port || _mkport();
          // in order to emulate dgram sockets, we need to launch a listen server when
          // binding on a connection-less socket
          // note: this is only required on the server side
          if (sock.type === 2) {
            // close the existing server if it exists
            if (sock.server) {
              sock.server.close();
              sock.server = null;
            }
            // swallow error operation not supported error that occurs when binding in the
            // browser where this isn't supported
            try {
              sock.sock_ops.listen(sock, 0);
            } catch (e) {
              if (!(e instanceof FS.ErrnoError)) throw e;
              if (e.errno !== ERRNO_CODES.EOPNOTSUPP) throw e;
            }
          }
        },connect:function (sock, addr, port) {
          if (sock.server) {
            throw new FS.ErrnoError(ERRNO_CODES.EOPNOTSUPP);
          }
  
          // TODO autobind
          // if (!sock.addr && sock.type == 2) {
          // }
  
          // early out if we're already connected / in the middle of connecting
          if (typeof sock.daddr !== 'undefined' && typeof sock.dport !== 'undefined') {
            var dest = SOCKFS.websocket_sock_ops.getPeer(sock, sock.daddr, sock.dport);
            if (dest) {
              if (dest.socket.readyState === dest.socket.CONNECTING) {
                throw new FS.ErrnoError(ERRNO_CODES.EALREADY);
              } else {
                throw new FS.ErrnoError(ERRNO_CODES.EISCONN);
              }
            }
          }
  
          // add the socket to our peer list and set our
          // destination address / port to match
          var peer = SOCKFS.websocket_sock_ops.createPeer(sock, addr, port);
          sock.daddr = peer.addr;
          sock.dport = peer.port;
  
          // always "fail" in non-blocking mode
          throw new FS.ErrnoError(ERRNO_CODES.EINPROGRESS);
        },listen:function (sock, backlog) {
          if (!ENVIRONMENT_IS_NODE) {
            throw new FS.ErrnoError(ERRNO_CODES.EOPNOTSUPP);
          }
          if (sock.server) {
             throw new FS.ErrnoError(ERRNO_CODES.EINVAL);  // already listening
          }
          var WebSocketServer = require('ws').Server;
          var host = sock.saddr;
          sock.server = new WebSocketServer({
            host: host,
            port: sock.sport
            // TODO support backlog
          });
          Module['websocket'].emit('listen', sock.stream.fd); // Send Event with listen fd.
  
          sock.server.on('connection', function(ws) {
            if (sock.type === 1) {
              var newsock = SOCKFS.createSocket(sock.family, sock.type, sock.protocol);
  
              // create a peer on the new socket
              var peer = SOCKFS.websocket_sock_ops.createPeer(newsock, ws);
              newsock.daddr = peer.addr;
              newsock.dport = peer.port;
  
              // push to queue for accept to pick up
              sock.pending.push(newsock);
              Module['websocket'].emit('connection', newsock.stream.fd);
            } else {
              // create a peer on the listen socket so calling sendto
              // with the listen socket and an address will resolve
              // to the correct client
              SOCKFS.websocket_sock_ops.createPeer(sock, ws);
              Module['websocket'].emit('connection', sock.stream.fd);
            }
          });
          sock.server.on('closed', function() {
            Module['websocket'].emit('close', sock.stream.fd);
            sock.server = null;
          });
          sock.server.on('error', function(error) {
            // Although the ws library may pass errors that may be more descriptive than
            // ECONNREFUSED they are not necessarily the expected error code e.g. 
            // ENOTFOUND on getaddrinfo seems to be node.js specific, so using EHOSTUNREACH
            // is still probably the most useful thing to do. This error shouldn't
            // occur in a well written app as errors should get trapped in the compiled
            // app's own getaddrinfo call.
            sock.error = ERRNO_CODES.EHOSTUNREACH; // Used in getsockopt for SOL_SOCKET/SO_ERROR test.
            Module['websocket'].emit('error', [sock.stream.fd, sock.error, 'EHOSTUNREACH: Host is unreachable']);
            // don't throw
          });
        },accept:function (listensock) {
          if (!listensock.server) {
            throw new FS.ErrnoError(ERRNO_CODES.EINVAL);
          }
          var newsock = listensock.pending.shift();
          newsock.stream.flags = listensock.stream.flags;
          return newsock;
        },getname:function (sock, peer) {
          var addr, port;
          if (peer) {
            if (sock.daddr === undefined || sock.dport === undefined) {
              throw new FS.ErrnoError(ERRNO_CODES.ENOTCONN);
            }
            addr = sock.daddr;
            port = sock.dport;
          } else {
            // TODO saddr and sport will be set for bind()'d UDP sockets, but what
            // should we be returning for TCP sockets that've been connect()'d?
            addr = sock.saddr || 0;
            port = sock.sport || 0;
          }
          return { addr: addr, port: port };
        },sendmsg:function (sock, buffer, offset, length, addr, port) {
          if (sock.type === 2) {
            // connection-less sockets will honor the message address,
            // and otherwise fall back to the bound destination address
            if (addr === undefined || port === undefined) {
              addr = sock.daddr;
              port = sock.dport;
            }
            // if there was no address to fall back to, error out
            if (addr === undefined || port === undefined) {
              throw new FS.ErrnoError(ERRNO_CODES.EDESTADDRREQ);
            }
          } else {
            // connection-based sockets will only use the bound
            addr = sock.daddr;
            port = sock.dport;
          }
  
          // find the peer for the destination address
          var dest = SOCKFS.websocket_sock_ops.getPeer(sock, addr, port);
  
          // early out if not connected with a connection-based socket
          if (sock.type === 1) {
            if (!dest || dest.socket.readyState === dest.socket.CLOSING || dest.socket.readyState === dest.socket.CLOSED) {
              throw new FS.ErrnoError(ERRNO_CODES.ENOTCONN);
            } else if (dest.socket.readyState === dest.socket.CONNECTING) {
              throw new FS.ErrnoError(ERRNO_CODES.EAGAIN);
            }
          }
  
          // create a copy of the incoming data to send, as the WebSocket API
          // doesn't work entirely with an ArrayBufferView, it'll just send
          // the entire underlying buffer
          var data;
          if (buffer instanceof Array || buffer instanceof ArrayBuffer) {
            data = buffer.slice(offset, offset + length);
          } else {  // ArrayBufferView
            data = buffer.buffer.slice(buffer.byteOffset + offset, buffer.byteOffset + offset + length);
          }
  
          // if we're emulating a connection-less dgram socket and don't have
          // a cached connection, queue the buffer to send upon connect and
          // lie, saying the data was sent now.
          if (sock.type === 2) {
            if (!dest || dest.socket.readyState !== dest.socket.OPEN) {
              // if we're not connected, open a new connection
              if (!dest || dest.socket.readyState === dest.socket.CLOSING || dest.socket.readyState === dest.socket.CLOSED) {
                dest = SOCKFS.websocket_sock_ops.createPeer(sock, addr, port);
              }
              dest.dgram_send_queue.push(data);
              return length;
            }
          }
  
          try {
            // send the actual data
            dest.socket.send(data);
            return length;
          } catch (e) {
            throw new FS.ErrnoError(ERRNO_CODES.EINVAL);
          }
        },recvmsg:function (sock, length) {
          // http://pubs.opengroup.org/onlinepubs/7908799/xns/recvmsg.html
          if (sock.type === 1 && sock.server) {
            // tcp servers should not be recv()'ing on the listen socket
            throw new FS.ErrnoError(ERRNO_CODES.ENOTCONN);
          }
  
          var queued = sock.recv_queue.shift();
          if (!queued) {
            if (sock.type === 1) {
              var dest = SOCKFS.websocket_sock_ops.getPeer(sock, sock.daddr, sock.dport);
  
              if (!dest) {
                // if we have a destination address but are not connected, error out
                throw new FS.ErrnoError(ERRNO_CODES.ENOTCONN);
              }
              else if (dest.socket.readyState === dest.socket.CLOSING || dest.socket.readyState === dest.socket.CLOSED) {
                // return null if the socket has closed
                return null;
              }
              else {
                // else, our socket is in a valid state but truly has nothing available
                throw new FS.ErrnoError(ERRNO_CODES.EAGAIN);
              }
            } else {
              throw new FS.ErrnoError(ERRNO_CODES.EAGAIN);
            }
          }
  
          // queued.data will be an ArrayBuffer if it's unadulterated, but if it's
          // requeued TCP data it'll be an ArrayBufferView
          var queuedLength = queued.data.byteLength || queued.data.length;
          var queuedOffset = queued.data.byteOffset || 0;
          var queuedBuffer = queued.data.buffer || queued.data;
          var bytesRead = Math.min(length, queuedLength);
          var res = {
            buffer: new Uint8Array(queuedBuffer, queuedOffset, bytesRead),
            addr: queued.addr,
            port: queued.port
          };
  
  
          // push back any unread data for TCP connections
          if (sock.type === 1 && bytesRead < queuedLength) {
            var bytesRemaining = queuedLength - bytesRead;
            queued.data = new Uint8Array(queuedBuffer, queuedOffset + bytesRead, bytesRemaining);
            sock.recv_queue.unshift(queued);
          }
  
          return res;
        }}};function _send(fd, buf, len, flags) {
      var sock = SOCKFS.getSocket(fd);
      if (!sock) {
        ___setErrNo(ERRNO_CODES.EBADF);
        return -1;
      }
      // TODO honor flags
      return _write(fd, buf, len);
    }
  
  function _pwrite(fildes, buf, nbyte, offset) {
      // ssize_t pwrite(int fildes, const void *buf, size_t nbyte, off_t offset);
      // http://pubs.opengroup.org/onlinepubs/000095399/functions/write.html
      var stream = FS.getStream(fildes);
      if (!stream) {
        ___setErrNo(ERRNO_CODES.EBADF);
        return -1;
      }
      try {
        var slab = HEAP8;
        return FS.write(stream, slab, buf, nbyte, offset);
      } catch (e) {
        FS.handleFSError(e);
        return -1;
      }
    }function _write(fildes, buf, nbyte) {
      // ssize_t write(int fildes, const void *buf, size_t nbyte);
      // http://pubs.opengroup.org/onlinepubs/000095399/functions/write.html
      var stream = FS.getStream(fildes);
      if (!stream) {
        ___setErrNo(ERRNO_CODES.EBADF);
        return -1;
      }
  
  
      try {
        var slab = HEAP8;
        return FS.write(stream, slab, buf, nbyte);
      } catch (e) {
        FS.handleFSError(e);
        return -1;
      }
    }

  
  function _chmod(path, mode, dontResolveLastLink) {
      // int chmod(const char *path, mode_t mode);
      // http://pubs.opengroup.org/onlinepubs/7908799/xsh/chmod.html
      // NOTE: dontResolveLastLink is a shortcut for lchmod(). It should never be
      //       used in client code.
      path = typeof path !== 'string' ? Pointer_stringify(path) : path;
      try {
        FS.chmod(path, mode);
        return 0;
      } catch (e) {
        FS.handleFSError(e);
        return -1;
      }
    }function _fchmod(fildes, mode) {
      // int fchmod(int fildes, mode_t mode);
      // http://pubs.opengroup.org/onlinepubs/7908799/xsh/fchmod.html
      try {
        FS.fchmod(fildes, mode);
        return 0;
      } catch (e) {
        FS.handleFSError(e);
        return -1;
      }
    }

  
  
  function _emscripten_set_main_loop_timing(mode, value) {
      Browser.mainLoop.timingMode = mode;
      Browser.mainLoop.timingValue = value;
  
      if (!Browser.mainLoop.func) {
        return 1; // Return non-zero on failure, can't set timing mode when there is no main loop.
      }
  
      if (mode == 0 /*EM_TIMING_SETTIMEOUT*/) {
        Browser.mainLoop.scheduler = function Browser_mainLoop_scheduler() {
          setTimeout(Browser.mainLoop.runner, value); // doing this each time means that on exception, we stop
        };
        Browser.mainLoop.method = 'timeout';
      } else if (mode == 1 /*EM_TIMING_RAF*/) {
        Browser.mainLoop.scheduler = function Browser_mainLoop_scheduler() {
          Browser.requestAnimationFrame(Browser.mainLoop.runner);
        };
        Browser.mainLoop.method = 'rAF';
      }
      return 0;
    }function _emscripten_set_main_loop(func, fps, simulateInfiniteLoop, arg) {
      Module['noExitRuntime'] = true;
  
      assert(!Browser.mainLoop.func, 'emscripten_set_main_loop: there can only be one main loop function at once: call emscripten_cancel_main_loop to cancel the previous one before setting a new one with different parameters.');
  
      Browser.mainLoop.func = func;
      Browser.mainLoop.arg = arg;
  
      var thisMainLoopId = Browser.mainLoop.currentlyRunningMainloop;
  
      Browser.mainLoop.runner = function Browser_mainLoop_runner() {
        if (ABORT) return;
        if (Browser.mainLoop.queue.length > 0) {
          var start = Date.now();
          var blocker = Browser.mainLoop.queue.shift();
          blocker.func(blocker.arg);
          if (Browser.mainLoop.remainingBlockers) {
            var remaining = Browser.mainLoop.remainingBlockers;
            var next = remaining%1 == 0 ? remaining-1 : Math.floor(remaining);
            if (blocker.counted) {
              Browser.mainLoop.remainingBlockers = next;
            } else {
              // not counted, but move the progress along a tiny bit
              next = next + 0.5; // do not steal all the next one's progress
              Browser.mainLoop.remainingBlockers = (8*remaining + next)/9;
            }
          }
          console.log('main loop blocker "' + blocker.name + '" took ' + (Date.now() - start) + ' ms'); //, left: ' + Browser.mainLoop.remainingBlockers);
          Browser.mainLoop.updateStatus();
          setTimeout(Browser.mainLoop.runner, 0);
          return;
        }
  
        // catch pauses from non-main loop sources
        if (thisMainLoopId < Browser.mainLoop.currentlyRunningMainloop) return;
  
        // Implement very basic swap interval control
        Browser.mainLoop.currentFrameNumber = Browser.mainLoop.currentFrameNumber + 1 | 0;
        if (Browser.mainLoop.timingMode == 1/*EM_TIMING_RAF*/ && Browser.mainLoop.timingValue > 1 && Browser.mainLoop.currentFrameNumber % Browser.mainLoop.timingValue != 0) {
          // Not the scheduled time to render this frame - skip.
          Browser.mainLoop.scheduler();
          return;
        }
  
        // Signal GL rendering layer that processing of a new frame is about to start. This helps it optimize
        // VBO double-buffering and reduce GPU stalls.
  
        if (Browser.mainLoop.method === 'timeout' && Module.ctx) {
          Module.printErr('Looks like you are rendering without using requestAnimationFrame for the main loop. You should use 0 for the frame rate in emscripten_set_main_loop in order to use requestAnimationFrame, as that can greatly improve your frame rates!');
          Browser.mainLoop.method = ''; // just warn once per call to set main loop
        }
  
        Browser.mainLoop.runIter(function() {
          if (typeof arg !== 'undefined') {
            Runtime.dynCall('vi', func, [arg]);
          } else {
            Runtime.dynCall('v', func);
          }
        });
  
        // catch pauses from the main loop itself
        if (thisMainLoopId < Browser.mainLoop.currentlyRunningMainloop) return;
  
        // Queue new audio data. This is important to be right after the main loop invocation, so that we will immediately be able
        // to queue the newest produced audio samples.
        // TODO: Consider adding pre- and post- rAF callbacks so that GL.newRenderingFrameStarted() and SDL.audio.queueNewAudioData()
        //       do not need to be hardcoded into this function, but can be more generic.
        if (typeof SDL === 'object' && SDL.audio && SDL.audio.queueNewAudioData) SDL.audio.queueNewAudioData();
  
        Browser.mainLoop.scheduler();
      }
  
      if (fps && fps > 0) _emscripten_set_main_loop_timing(0/*EM_TIMING_SETTIMEOUT*/, 1000.0 / fps);
      else _emscripten_set_main_loop_timing(1/*EM_TIMING_RAF*/, 1); // Do rAF by rendering each frame (no decimating)
  
      Browser.mainLoop.scheduler();
  
      if (simulateInfiniteLoop) {
        throw 'SimulateInfiniteLoop';
      }
    }var Browser={mainLoop:{scheduler:null,method:"",currentlyRunningMainloop:0,func:null,arg:0,timingMode:0,timingValue:0,currentFrameNumber:0,queue:[],pause:function () {
          Browser.mainLoop.scheduler = null;
          Browser.mainLoop.currentlyRunningMainloop++; // Incrementing this signals the previous main loop that it's now become old, and it must return.
        },resume:function () {
          Browser.mainLoop.currentlyRunningMainloop++;
          var timingMode = Browser.mainLoop.timingMode;
          var timingValue = Browser.mainLoop.timingValue;
          var func = Browser.mainLoop.func;
          Browser.mainLoop.func = null;
          _emscripten_set_main_loop(func, 0, false, Browser.mainLoop.arg);
          _emscripten_set_main_loop_timing(timingMode, timingValue);
        },updateStatus:function () {
          if (Module['setStatus']) {
            var message = Module['statusMessage'] || 'Please wait...';
            var remaining = Browser.mainLoop.remainingBlockers;
            var expected = Browser.mainLoop.expectedBlockers;
            if (remaining) {
              if (remaining < expected) {
                Module['setStatus'](message + ' (' + (expected - remaining) + '/' + expected + ')');
              } else {
                Module['setStatus'](message);
              }
            } else {
              Module['setStatus']('');
            }
          }
        },runIter:function (func) {
          if (ABORT) return;
          if (Module['preMainLoop']) {
            var preRet = Module['preMainLoop']();
            if (preRet === false) {
              return; // |return false| skips a frame
            }
          }
          try {
            func();
          } catch (e) {
            if (e instanceof ExitStatus) {
              return;
            } else {
              if (e && typeof e === 'object' && e.stack) Module.printErr('exception thrown: ' + [e, e.stack]);
              throw e;
            }
          }
          if (Module['postMainLoop']) Module['postMainLoop']();
        }},isFullScreen:false,pointerLock:false,moduleContextCreatedCallbacks:[],workers:[],init:function () {
        if (!Module["preloadPlugins"]) Module["preloadPlugins"] = []; // needs to exist even in workers
  
        if (Browser.initted) return;
        Browser.initted = true;
  
        try {
          new Blob();
          Browser.hasBlobConstructor = true;
        } catch(e) {
          Browser.hasBlobConstructor = false;
          console.log("warning: no blob constructor, cannot create blobs with mimetypes");
        }
        Browser.BlobBuilder = typeof MozBlobBuilder != "undefined" ? MozBlobBuilder : (typeof WebKitBlobBuilder != "undefined" ? WebKitBlobBuilder : (!Browser.hasBlobConstructor ? console.log("warning: no BlobBuilder") : null));
        Browser.URLObject = typeof window != "undefined" ? (window.URL ? window.URL : window.webkitURL) : undefined;
        if (!Module.noImageDecoding && typeof Browser.URLObject === 'undefined') {
          console.log("warning: Browser does not support creating object URLs. Built-in browser image decoding will not be available.");
          Module.noImageDecoding = true;
        }
  
        // Support for plugins that can process preloaded files. You can add more of these to
        // your app by creating and appending to Module.preloadPlugins.
        //
        // Each plugin is asked if it can handle a file based on the file's name. If it can,
        // it is given the file's raw data. When it is done, it calls a callback with the file's
        // (possibly modified) data. For example, a plugin might decompress a file, or it
        // might create some side data structure for use later (like an Image element, etc.).
  
        var imagePlugin = {};
        imagePlugin['canHandle'] = function imagePlugin_canHandle(name) {
          return !Module.noImageDecoding && /\.(jpg|jpeg|png|bmp)$/i.test(name);
        };
        imagePlugin['handle'] = function imagePlugin_handle(byteArray, name, onload, onerror) {
          var b = null;
          if (Browser.hasBlobConstructor) {
            try {
              b = new Blob([byteArray], { type: Browser.getMimetype(name) });
              if (b.size !== byteArray.length) { // Safari bug #118630
                // Safari's Blob can only take an ArrayBuffer
                b = new Blob([(new Uint8Array(byteArray)).buffer], { type: Browser.getMimetype(name) });
              }
            } catch(e) {
              Runtime.warnOnce('Blob constructor present but fails: ' + e + '; falling back to blob builder');
            }
          }
          if (!b) {
            var bb = new Browser.BlobBuilder();
            bb.append((new Uint8Array(byteArray)).buffer); // we need to pass a buffer, and must copy the array to get the right data range
            b = bb.getBlob();
          }
          var url = Browser.URLObject.createObjectURL(b);
          var img = new Image();
          img.onload = function img_onload() {
            assert(img.complete, 'Image ' + name + ' could not be decoded');
            var canvas = document.createElement('canvas');
            canvas.width = img.width;
            canvas.height = img.height;
            var ctx = canvas.getContext('2d');
            ctx.drawImage(img, 0, 0);
            Module["preloadedImages"][name] = canvas;
            Browser.URLObject.revokeObjectURL(url);
            if (onload) onload(byteArray);
          };
          img.onerror = function img_onerror(event) {
            console.log('Image ' + url + ' could not be decoded');
            if (onerror) onerror();
          };
          img.src = url;
        };
        Module['preloadPlugins'].push(imagePlugin);
  
        var audioPlugin = {};
        audioPlugin['canHandle'] = function audioPlugin_canHandle(name) {
          return !Module.noAudioDecoding && name.substr(-4) in { '.ogg': 1, '.wav': 1, '.mp3': 1 };
        };
        audioPlugin['handle'] = function audioPlugin_handle(byteArray, name, onload, onerror) {
          var done = false;
          function finish(audio) {
            if (done) return;
            done = true;
            Module["preloadedAudios"][name] = audio;
            if (onload) onload(byteArray);
          }
          function fail() {
            if (done) return;
            done = true;
            Module["preloadedAudios"][name] = new Audio(); // empty shim
            if (onerror) onerror();
          }
          if (Browser.hasBlobConstructor) {
            try {
              var b = new Blob([byteArray], { type: Browser.getMimetype(name) });
            } catch(e) {
              return fail();
            }
            var url = Browser.URLObject.createObjectURL(b); // XXX we never revoke this!
            var audio = new Audio();
            audio.addEventListener('canplaythrough', function() { finish(audio) }, false); // use addEventListener due to chromium bug 124926
            audio.onerror = function audio_onerror(event) {
              if (done) return;
              console.log('warning: browser could not fully decode audio ' + name + ', trying slower base64 approach');
              function encode64(data) {
                var BASE = 'ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789+/';
                var PAD = '=';
                var ret = '';
                var leftchar = 0;
                var leftbits = 0;
                for (var i = 0; i < data.length; i++) {
                  leftchar = (leftchar << 8) | data[i];
                  leftbits += 8;
                  while (leftbits >= 6) {
                    var curr = (leftchar >> (leftbits-6)) & 0x3f;
                    leftbits -= 6;
                    ret += BASE[curr];
                  }
                }
                if (leftbits == 2) {
                  ret += BASE[(leftchar&3) << 4];
                  ret += PAD + PAD;
                } else if (leftbits == 4) {
                  ret += BASE[(leftchar&0xf) << 2];
                  ret += PAD;
                }
                return ret;
              }
              audio.src = 'data:audio/x-' + name.substr(-3) + ';base64,' + encode64(byteArray);
              finish(audio); // we don't wait for confirmation this worked - but it's worth trying
            };
            audio.src = url;
            // workaround for chrome bug 124926 - we do not always get oncanplaythrough or onerror
            Browser.safeSetTimeout(function() {
              finish(audio); // try to use it even though it is not necessarily ready to play
            }, 10000);
          } else {
            return fail();
          }
        };
        Module['preloadPlugins'].push(audioPlugin);
  
        // Canvas event setup
  
        var canvas = Module['canvas'];
        function pointerLockChange() {
          Browser.pointerLock = document['pointerLockElement'] === canvas ||
                                document['mozPointerLockElement'] === canvas ||
                                document['webkitPointerLockElement'] === canvas ||
                                document['msPointerLockElement'] === canvas;
        }
        if (canvas) {
          // forced aspect ratio can be enabled by defining 'forcedAspectRatio' on Module
          // Module['forcedAspectRatio'] = 4 / 3;
          
          canvas.requestPointerLock = canvas['requestPointerLock'] ||
                                      canvas['mozRequestPointerLock'] ||
                                      canvas['webkitRequestPointerLock'] ||
                                      canvas['msRequestPointerLock'] ||
                                      function(){};
          canvas.exitPointerLock = document['exitPointerLock'] ||
                                   document['mozExitPointerLock'] ||
                                   document['webkitExitPointerLock'] ||
                                   document['msExitPointerLock'] ||
                                   function(){}; // no-op if function does not exist
          canvas.exitPointerLock = canvas.exitPointerLock.bind(document);
  
  
          document.addEventListener('pointerlockchange', pointerLockChange, false);
          document.addEventListener('mozpointerlockchange', pointerLockChange, false);
          document.addEventListener('webkitpointerlockchange', pointerLockChange, false);
          document.addEventListener('mspointerlockchange', pointerLockChange, false);
  
          if (Module['elementPointerLock']) {
            canvas.addEventListener("click", function(ev) {
              if (!Browser.pointerLock && canvas.requestPointerLock) {
                canvas.requestPointerLock();
                ev.preventDefault();
              }
            }, false);
          }
        }
      },createContext:function (canvas, useWebGL, setInModule, webGLContextAttributes) {
        if (useWebGL && Module.ctx && canvas == Module.canvas) return Module.ctx; // no need to recreate GL context if it's already been created for this canvas.
  
        var ctx;
        var contextHandle;
        if (useWebGL) {
          // For GLES2/desktop GL compatibility, adjust a few defaults to be different to WebGL defaults, so that they align better with the desktop defaults.
          var contextAttributes = {
            antialias: false,
            alpha: false
          };
  
          if (webGLContextAttributes) {
            for (var attribute in webGLContextAttributes) {
              contextAttributes[attribute] = webGLContextAttributes[attribute];
            }
          }
  
          contextHandle = GL.createContext(canvas, contextAttributes);
          if (contextHandle) {
            ctx = GL.getContext(contextHandle).GLctx;
          }
          // Set the background of the WebGL canvas to black
          canvas.style.backgroundColor = "black";
        } else {
          ctx = canvas.getContext('2d');
        }
  
        if (!ctx) return null;
  
        if (setInModule) {
          if (!useWebGL) assert(typeof GLctx === 'undefined', 'cannot set in module if GLctx is used, but we are a non-GL context that would replace it');
  
          Module.ctx = ctx;
          if (useWebGL) GL.makeContextCurrent(contextHandle);
          Module.useWebGL = useWebGL;
          Browser.moduleContextCreatedCallbacks.forEach(function(callback) { callback() });
          Browser.init();
        }
        return ctx;
      },destroyContext:function (canvas, useWebGL, setInModule) {},fullScreenHandlersInstalled:false,lockPointer:undefined,resizeCanvas:undefined,requestFullScreen:function (lockPointer, resizeCanvas) {
        Browser.lockPointer = lockPointer;
        Browser.resizeCanvas = resizeCanvas;
        if (typeof Browser.lockPointer === 'undefined') Browser.lockPointer = true;
        if (typeof Browser.resizeCanvas === 'undefined') Browser.resizeCanvas = false;
  
        var canvas = Module['canvas'];
        function fullScreenChange() {
          Browser.isFullScreen = false;
          var canvasContainer = canvas.parentNode;
          if ((document['webkitFullScreenElement'] || document['webkitFullscreenElement'] ||
               document['mozFullScreenElement'] || document['mozFullscreenElement'] ||
               document['fullScreenElement'] || document['fullscreenElement'] ||
               document['msFullScreenElement'] || document['msFullscreenElement'] ||
               document['webkitCurrentFullScreenElement']) === canvasContainer) {
            canvas.cancelFullScreen = document['cancelFullScreen'] ||
                                      document['mozCancelFullScreen'] ||
                                      document['webkitCancelFullScreen'] ||
                                      document['msExitFullscreen'] ||
                                      document['exitFullscreen'] ||
                                      function() {};
            canvas.cancelFullScreen = canvas.cancelFullScreen.bind(document);
            if (Browser.lockPointer) canvas.requestPointerLock();
            Browser.isFullScreen = true;
            if (Browser.resizeCanvas) Browser.setFullScreenCanvasSize();
          } else {
            
            // remove the full screen specific parent of the canvas again to restore the HTML structure from before going full screen
            canvasContainer.parentNode.insertBefore(canvas, canvasContainer);
            canvasContainer.parentNode.removeChild(canvasContainer);
            
            if (Browser.resizeCanvas) Browser.setWindowedCanvasSize();
          }
          if (Module['onFullScreen']) Module['onFullScreen'](Browser.isFullScreen);
          Browser.updateCanvasDimensions(canvas);
        }
  
        if (!Browser.fullScreenHandlersInstalled) {
          Browser.fullScreenHandlersInstalled = true;
          document.addEventListener('fullscreenchange', fullScreenChange, false);
          document.addEventListener('mozfullscreenchange', fullScreenChange, false);
          document.addEventListener('webkitfullscreenchange', fullScreenChange, false);
          document.addEventListener('MSFullscreenChange', fullScreenChange, false);
        }
  
        // create a new parent to ensure the canvas has no siblings. this allows browsers to optimize full screen performance when its parent is the full screen root
        var canvasContainer = document.createElement("div");
        canvas.parentNode.insertBefore(canvasContainer, canvas);
        canvasContainer.appendChild(canvas);
        
        // use parent of canvas as full screen root to allow aspect ratio correction (Firefox stretches the root to screen size)
        canvasContainer.requestFullScreen = canvasContainer['requestFullScreen'] ||
                                            canvasContainer['mozRequestFullScreen'] ||
                                            canvasContainer['msRequestFullscreen'] ||
                                           (canvasContainer['webkitRequestFullScreen'] ? function() { canvasContainer['webkitRequestFullScreen'](Element['ALLOW_KEYBOARD_INPUT']) } : null);
        canvasContainer.requestFullScreen();
      },nextRAF:0,fakeRequestAnimationFrame:function (func) {
        // try to keep 60fps between calls to here
        var now = Date.now();
        if (Browser.nextRAF === 0) {
          Browser.nextRAF = now + 1000/60;
        } else {
          while (now + 2 >= Browser.nextRAF) { // fudge a little, to avoid timer jitter causing us to do lots of delay:0
            Browser.nextRAF += 1000/60;
          }
        }
        var delay = Math.max(Browser.nextRAF - now, 0);
        setTimeout(func, delay);
      },requestAnimationFrame:function requestAnimationFrame(func) {
        if (typeof window === 'undefined') { // Provide fallback to setTimeout if window is undefined (e.g. in Node.js)
          Browser.fakeRequestAnimationFrame(func);
        } else {
          if (!window.requestAnimationFrame) {
            window.requestAnimationFrame = window['requestAnimationFrame'] ||
                                           window['mozRequestAnimationFrame'] ||
                                           window['webkitRequestAnimationFrame'] ||
                                           window['msRequestAnimationFrame'] ||
                                           window['oRequestAnimationFrame'] ||
                                           Browser.fakeRequestAnimationFrame;
          }
          window.requestAnimationFrame(func);
        }
      },safeCallback:function (func) {
        return function() {
          if (!ABORT) return func.apply(null, arguments);
        };
      },safeRequestAnimationFrame:function (func) {
        return Browser.requestAnimationFrame(function() {
          if (!ABORT) func();
        });
      },safeSetTimeout:function (func, timeout) {
        Module['noExitRuntime'] = true;
        return setTimeout(function() {
          if (!ABORT) func();
        }, timeout);
      },safeSetInterval:function (func, timeout) {
        Module['noExitRuntime'] = true;
        return setInterval(function() {
          if (!ABORT) func();
        }, timeout);
      },getMimetype:function (name) {
        return {
          'jpg': 'image/jpeg',
          'jpeg': 'image/jpeg',
          'png': 'image/png',
          'bmp': 'image/bmp',
          'ogg': 'audio/ogg',
          'wav': 'audio/wav',
          'mp3': 'audio/mpeg'
        }[name.substr(name.lastIndexOf('.')+1)];
      },getUserMedia:function (func) {
        if(!window.getUserMedia) {
          window.getUserMedia = navigator['getUserMedia'] ||
                                navigator['mozGetUserMedia'];
        }
        window.getUserMedia(func);
      },getMovementX:function (event) {
        return event['movementX'] ||
               event['mozMovementX'] ||
               event['webkitMovementX'] ||
               0;
      },getMovementY:function (event) {
        return event['movementY'] ||
               event['mozMovementY'] ||
               event['webkitMovementY'] ||
               0;
      },getMouseWheelDelta:function (event) {
        var delta = 0;
        switch (event.type) {
          case 'DOMMouseScroll': 
            delta = event.detail;
            break;
          case 'mousewheel': 
            delta = event.wheelDelta;
            break;
          case 'wheel': 
            delta = event['deltaY'];
            break;
          default:
            throw 'unrecognized mouse wheel event: ' + event.type;
        }
        return delta;
      },mouseX:0,mouseY:0,mouseMovementX:0,mouseMovementY:0,touches:{},lastTouches:{},calculateMouseEvent:function (event) { // event should be mousemove, mousedown or mouseup
        if (Browser.pointerLock) {
          // When the pointer is locked, calculate the coordinates
          // based on the movement of the mouse.
          // Workaround for Firefox bug 764498
          if (event.type != 'mousemove' &&
              ('mozMovementX' in event)) {
            Browser.mouseMovementX = Browser.mouseMovementY = 0;
          } else {
            Browser.mouseMovementX = Browser.getMovementX(event);
            Browser.mouseMovementY = Browser.getMovementY(event);
          }
          
          // check if SDL is available
          if (typeof SDL != "undefined") {
          	Browser.mouseX = SDL.mouseX + Browser.mouseMovementX;
          	Browser.mouseY = SDL.mouseY + Browser.mouseMovementY;
          } else {
          	// just add the mouse delta to the current absolut mouse position
          	// FIXME: ideally this should be clamped against the canvas size and zero
          	Browser.mouseX += Browser.mouseMovementX;
          	Browser.mouseY += Browser.mouseMovementY;
          }        
        } else {
          // Otherwise, calculate the movement based on the changes
          // in the coordinates.
          var rect = Module["canvas"].getBoundingClientRect();
          var cw = Module["canvas"].width;
          var ch = Module["canvas"].height;
  
          // Neither .scrollX or .pageXOffset are defined in a spec, but
          // we prefer .scrollX because it is currently in a spec draft.
          // (see: http://www.w3.org/TR/2013/WD-cssom-view-20131217/)
          var scrollX = ((typeof window.scrollX !== 'undefined') ? window.scrollX : window.pageXOffset);
          var scrollY = ((typeof window.scrollY !== 'undefined') ? window.scrollY : window.pageYOffset);
  
          if (event.type === 'touchstart' || event.type === 'touchend' || event.type === 'touchmove') {
            var touch = event.touch;
            if (touch === undefined) {
              return; // the "touch" property is only defined in SDL
  
            }
            var adjustedX = touch.pageX - (scrollX + rect.left);
            var adjustedY = touch.pageY - (scrollY + rect.top);
  
            adjustedX = adjustedX * (cw / rect.width);
            adjustedY = adjustedY * (ch / rect.height);
  
            var coords = { x: adjustedX, y: adjustedY };
            
            if (event.type === 'touchstart') {
              Browser.lastTouches[touch.identifier] = coords;
              Browser.touches[touch.identifier] = coords;
            } else if (event.type === 'touchend' || event.type === 'touchmove') {
              Browser.lastTouches[touch.identifier] = Browser.touches[touch.identifier];
              Browser.touches[touch.identifier] = { x: adjustedX, y: adjustedY };
            } 
            return;
          }
  
          var x = event.pageX - (scrollX + rect.left);
          var y = event.pageY - (scrollY + rect.top);
  
          // the canvas might be CSS-scaled compared to its backbuffer;
          // SDL-using content will want mouse coordinates in terms
          // of backbuffer units.
          x = x * (cw / rect.width);
          y = y * (ch / rect.height);
  
          Browser.mouseMovementX = x - Browser.mouseX;
          Browser.mouseMovementY = y - Browser.mouseY;
          Browser.mouseX = x;
          Browser.mouseY = y;
        }
      },xhrLoad:function (url, onload, onerror) {
        var xhr = new XMLHttpRequest();
        xhr.open('GET', url, true);
        xhr.responseType = 'arraybuffer';
        xhr.onload = function xhr_onload() {
          if (xhr.status == 200 || (xhr.status == 0 && xhr.response)) { // file URLs can return 0
            onload(xhr.response);
          } else {
            onerror();
          }
        };
        xhr.onerror = onerror;
        xhr.send(null);
      },asyncLoad:function (url, onload, onerror, noRunDep) {
        Browser.xhrLoad(url, function(arrayBuffer) {
          assert(arrayBuffer, 'Loading data file "' + url + '" failed (no arrayBuffer).');
          onload(new Uint8Array(arrayBuffer));
          if (!noRunDep) removeRunDependency('al ' + url);
        }, function(event) {
          if (onerror) {
            onerror();
          } else {
            throw 'Loading data file "' + url + '" failed.';
          }
        });
        if (!noRunDep) addRunDependency('al ' + url);
      },resizeListeners:[],updateResizeListeners:function () {
        var canvas = Module['canvas'];
        Browser.resizeListeners.forEach(function(listener) {
          listener(canvas.width, canvas.height);
        });
      },setCanvasSize:function (width, height, noUpdates) {
        var canvas = Module['canvas'];
        Browser.updateCanvasDimensions(canvas, width, height);
        if (!noUpdates) Browser.updateResizeListeners();
      },windowedWidth:0,windowedHeight:0,setFullScreenCanvasSize:function () {
        // check if SDL is available   
        if (typeof SDL != "undefined") {
        	var flags = HEAPU32[((SDL.screen+Runtime.QUANTUM_SIZE*0)>>2)];
        	flags = flags | 0x00800000; // set SDL_FULLSCREEN flag
        	HEAP32[((SDL.screen+Runtime.QUANTUM_SIZE*0)>>2)]=flags
        }
        Browser.updateResizeListeners();
      },setWindowedCanvasSize:function () {
        // check if SDL is available       
        if (typeof SDL != "undefined") {
        	var flags = HEAPU32[((SDL.screen+Runtime.QUANTUM_SIZE*0)>>2)];
        	flags = flags & ~0x00800000; // clear SDL_FULLSCREEN flag
        	HEAP32[((SDL.screen+Runtime.QUANTUM_SIZE*0)>>2)]=flags
        }
        Browser.updateResizeListeners();
      },updateCanvasDimensions:function (canvas, wNative, hNative) {
        if (wNative && hNative) {
          canvas.widthNative = wNative;
          canvas.heightNative = hNative;
        } else {
          wNative = canvas.widthNative;
          hNative = canvas.heightNative;
        }
        var w = wNative;
        var h = hNative;
        if (Module['forcedAspectRatio'] && Module['forcedAspectRatio'] > 0) {
          if (w/h < Module['forcedAspectRatio']) {
            w = Math.round(h * Module['forcedAspectRatio']);
          } else {
            h = Math.round(w / Module['forcedAspectRatio']);
          }
        }
        if (((document['webkitFullScreenElement'] || document['webkitFullscreenElement'] ||
             document['mozFullScreenElement'] || document['mozFullscreenElement'] ||
             document['fullScreenElement'] || document['fullscreenElement'] ||
             document['msFullScreenElement'] || document['msFullscreenElement'] ||
             document['webkitCurrentFullScreenElement']) === canvas.parentNode) && (typeof screen != 'undefined')) {
           var factor = Math.min(screen.width / w, screen.height / h);
           w = Math.round(w * factor);
           h = Math.round(h * factor);
        }
        if (Browser.resizeCanvas) {
          if (canvas.width  != w) canvas.width  = w;
          if (canvas.height != h) canvas.height = h;
          if (typeof canvas.style != 'undefined') {
            canvas.style.removeProperty( "width");
            canvas.style.removeProperty("height");
          }
        } else {
          if (canvas.width  != wNative) canvas.width  = wNative;
          if (canvas.height != hNative) canvas.height = hNative;
          if (typeof canvas.style != 'undefined') {
            if (w != wNative || h != hNative) {
              canvas.style.setProperty( "width", w + "px", "important");
              canvas.style.setProperty("height", h + "px", "important");
            } else {
              canvas.style.removeProperty( "width");
              canvas.style.removeProperty("height");
            }
          }
        }
      },wgetRequests:{},nextWgetRequestHandle:0,getNextWgetRequestHandle:function () {
        var handle = Browser.nextWgetRequestHandle;
        Browser.nextWgetRequestHandle++;
        return handle;
      }};

  function _rmdir(path) {
      // int rmdir(const char *path);
      // http://pubs.opengroup.org/onlinepubs/000095399/functions/rmdir.html
      path = Pointer_stringify(path);
      try {
        FS.rmdir(path);
        return 0;
      } catch (e) {
        FS.handleFSError(e);
        return -1;
      }
    }

  function _munmap(start, num) {
      if (!_mmap.mappings) _mmap.mappings = {};
      // TODO: support unmmap'ing parts of allocations
      var info = _mmap.mappings[start];
      if (!info) return 0;
      if (num == info.num) {
        _mmap.mappings[start] = null;
        if (info.allocated) {
          _free(info.malloc);
        }
      }
      return 0;
    }

  function _fstat(fildes, buf) {
      // int fstat(int fildes, struct stat *buf);
      // http://pubs.opengroup.org/onlinepubs/7908799/xsh/fstat.html
      var stream = FS.getStream(fildes);
      if (!stream) {
        ___setErrNo(ERRNO_CODES.EBADF);
        return -1;
      }
      return _stat(stream.path, buf);
    }

  
  var ___tm_current=allocate(44, "i8", ALLOC_STATIC);
  
  
  var ___tm_timezone=allocate(intArrayFromString("GMT"), "i8", ALLOC_STATIC);
  
  
  var _tzname=allocate(8, "i32*", ALLOC_STATIC);
  
  var _daylight=allocate(1, "i32*", ALLOC_STATIC);
  
  var _timezone=allocate(1, "i32*", ALLOC_STATIC);function _tzset() {
      // TODO: Use (malleable) environment variables instead of system settings.
      if (_tzset.called) return;
      _tzset.called = true;
  
      HEAP32[((_timezone)>>2)]=-(new Date()).getTimezoneOffset() * 60;
  
      var winter = new Date(2000, 0, 1);
      var summer = new Date(2000, 6, 1);
      HEAP32[((_daylight)>>2)]=Number(winter.getTimezoneOffset() != summer.getTimezoneOffset());
  
      function extractZone(date) {
        var match = date.toTimeString().match(/\(([A-Za-z ]+)\)$/);
        return match ? match[1] : "GMT";
      };
      var winterName = extractZone(winter);
      var summerName = extractZone(summer);
      var winterNamePtr = allocate(intArrayFromString(winterName), 'i8', ALLOC_NORMAL);
      var summerNamePtr = allocate(intArrayFromString(summerName), 'i8', ALLOC_NORMAL);
      if (summer.getTimezoneOffset() < winter.getTimezoneOffset()) {
        // Northern hemisphere
        HEAP32[((_tzname)>>2)]=winterNamePtr;
        HEAP32[(((_tzname)+(4))>>2)]=summerNamePtr;
      } else {
        HEAP32[((_tzname)>>2)]=summerNamePtr;
        HEAP32[(((_tzname)+(4))>>2)]=winterNamePtr;
      }
    }function _localtime_r(time, tmPtr) {
      _tzset();
      var date = new Date(HEAP32[((time)>>2)]*1000);
      HEAP32[((tmPtr)>>2)]=date.getSeconds();
      HEAP32[(((tmPtr)+(4))>>2)]=date.getMinutes();
      HEAP32[(((tmPtr)+(8))>>2)]=date.getHours();
      HEAP32[(((tmPtr)+(12))>>2)]=date.getDate();
      HEAP32[(((tmPtr)+(16))>>2)]=date.getMonth();
      HEAP32[(((tmPtr)+(20))>>2)]=date.getFullYear()-1900;
      HEAP32[(((tmPtr)+(24))>>2)]=date.getDay();
  
      var start = new Date(date.getFullYear(), 0, 1);
      var yday = ((date.getTime() - start.getTime()) / (1000 * 60 * 60 * 24))|0;
      HEAP32[(((tmPtr)+(28))>>2)]=yday;
      HEAP32[(((tmPtr)+(36))>>2)]=-(date.getTimezoneOffset() * 60);
  
      // DST is in December in South
      var summerOffset = new Date(2000, 6, 1).getTimezoneOffset();
      var winterOffset = start.getTimezoneOffset();
      var dst = (date.getTimezoneOffset() == Math.min(winterOffset, summerOffset))|0;
      HEAP32[(((tmPtr)+(32))>>2)]=dst;
  
      var zonePtr = HEAP32[(((_tzname)+(dst ? Runtime.QUANTUM_SIZE : 0))>>2)];
      HEAP32[(((tmPtr)+(40))>>2)]=zonePtr;
  
      return tmPtr;
    }function _localtime(time) {
      return _localtime_r(time, ___tm_current);
    }

  function _fcntl(fildes, cmd, varargs, dup2) {
      // int fcntl(int fildes, int cmd, ...);
      // http://pubs.opengroup.org/onlinepubs/009695399/functions/fcntl.html
      var stream = FS.getStream(fildes);
      if (!stream) {
        ___setErrNo(ERRNO_CODES.EBADF);
        return -1;
      }
      switch (cmd) {
        case 0:
          var arg = HEAP32[((varargs)>>2)];
          if (arg < 0) {
            ___setErrNo(ERRNO_CODES.EINVAL);
            return -1;
          }
          var newStream;
          try {
            newStream = FS.open(stream.path, stream.flags, 0, arg);
          } catch (e) {
            FS.handleFSError(e);
            return -1;
          }
          return newStream.fd;
        case 1:
        case 2:
          return 0;  // FD_CLOEXEC makes no sense for a single process.
        case 3:
          return stream.flags;
        case 4:
          var arg = HEAP32[((varargs)>>2)];
          stream.flags |= arg;
          return 0;
        case 12:
        case 12:
          var arg = HEAP32[((varargs)>>2)];
          var offset = 0;
          // We're always unlocked.
          HEAP16[(((arg)+(offset))>>1)]=2;
          return 0;
        case 13:
        case 14:
        case 13:
        case 14:
          // Pretend that the locking is successful.
          return 0;
        case 8:
        case 9:
          // These are for sockets. We don't have them fully implemented yet.
          ___setErrNo(ERRNO_CODES.EINVAL);
          return -1;
        default:
          ___setErrNo(ERRNO_CODES.EINVAL);
          return -1;
      }
      // Should never be reached. Only to silence strict warnings.
      return -1;
    }

   
  Module["_bitshift64Lshr"] = _bitshift64Lshr;

  
  function _recv(fd, buf, len, flags) {
      var sock = SOCKFS.getSocket(fd);
      if (!sock) {
        ___setErrNo(ERRNO_CODES.EBADF);
        return -1;
      }
      // TODO honor flags
      return _read(fd, buf, len);
    }
  
  function _pread(fildes, buf, nbyte, offset) {
      // ssize_t pread(int fildes, void *buf, size_t nbyte, off_t offset);
      // http://pubs.opengroup.org/onlinepubs/000095399/functions/read.html
      var stream = FS.getStream(fildes);
      if (!stream) {
        ___setErrNo(ERRNO_CODES.EBADF);
        return -1;
      }
      try {
        var slab = HEAP8;
        return FS.read(stream, slab, buf, nbyte, offset);
      } catch (e) {
        FS.handleFSError(e);
        return -1;
      }
    }function _read(fildes, buf, nbyte) {
      // ssize_t read(int fildes, void *buf, size_t nbyte);
      // http://pubs.opengroup.org/onlinepubs/000095399/functions/read.html
      var stream = FS.getStream(fildes);
      if (!stream) {
        ___setErrNo(ERRNO_CODES.EBADF);
        return -1;
      }
  
  
      try {
        var slab = HEAP8;
        return FS.read(stream, slab, buf, nbyte);
      } catch (e) {
        FS.handleFSError(e);
        return -1;
      }
    }

  function _time(ptr) {
      var ret = (Date.now()/1000)|0;
      if (ptr) {
        HEAP32[((ptr)>>2)]=ret;
      }
      return ret;
    }

  function _getcwd(buf, size) {
      // char *getcwd(char *buf, size_t size);
      // http://pubs.opengroup.org/onlinepubs/000095399/functions/getcwd.html
      if (size == 0) {
        ___setErrNo(ERRNO_CODES.EINVAL);
        return 0;
      }
      var cwd = FS.cwd();
      if (size < cwd.length + 1) {
        ___setErrNo(ERRNO_CODES.ERANGE);
        return 0;
      } else {
        writeAsciiToMemory(cwd, buf);
        return buf;
      }
    }

  
  var PROCINFO={ppid:1,pid:42,sid:42,pgid:42};function _getpid() {  return PROCINFO.pid;  }

  
  
  
  
  var _environ=allocate(1, "i32*", ALLOC_STATIC);var ___environ=_environ;function ___buildEnvironment(env) {
      // WARNING: Arbitrary limit!
      var MAX_ENV_VALUES = 64;
      var TOTAL_ENV_SIZE = 1024;
  
      // Statically allocate memory for the environment.
      var poolPtr;
      var envPtr;
      if (!___buildEnvironment.called) {
        ___buildEnvironment.called = true;
        // Set default values. Use string keys for Closure Compiler compatibility.
        ENV['USER'] = 'web_user';
        ENV['PATH'] = '/';
        ENV['PWD'] = '/';
        ENV['HOME'] = '/home/web_user';
        ENV['LANG'] = 'C';
        ENV['_'] = Module['thisProgram'];
        // Allocate memory.
        poolPtr = allocate(TOTAL_ENV_SIZE, 'i8', ALLOC_STATIC);
        envPtr = allocate(MAX_ENV_VALUES * 4,
                          'i8*', ALLOC_STATIC);
        HEAP32[((envPtr)>>2)]=poolPtr;
        HEAP32[((_environ)>>2)]=envPtr;
      } else {
        envPtr = HEAP32[((_environ)>>2)];
        poolPtr = HEAP32[((envPtr)>>2)];
      }
  
      // Collect key=value lines.
      var strings = [];
      var totalSize = 0;
      for (var key in env) {
        if (typeof env[key] === 'string') {
          var line = key + '=' + env[key];
          strings.push(line);
          totalSize += line.length;
        }
      }
      if (totalSize > TOTAL_ENV_SIZE) {
        throw new Error('Environment size exceeded TOTAL_ENV_SIZE!');
      }
  
      // Make new.
      var ptrSize = 4;
      for (var i = 0; i < strings.length; i++) {
        var line = strings[i];
        writeAsciiToMemory(line, poolPtr);
        HEAP32[(((envPtr)+(i * ptrSize))>>2)]=poolPtr;
        poolPtr += line.length + 1;
      }
      HEAP32[(((envPtr)+(strings.length * ptrSize))>>2)]=0;
    }var ENV={};function _getenv(name) {
      // char *getenv(const char *name);
      // http://pubs.opengroup.org/onlinepubs/009695399/functions/getenv.html
      if (name === 0) return 0;
      name = Pointer_stringify(name);
      if (!ENV.hasOwnProperty(name)) return 0;
  
      if (_getenv.ret) _free(_getenv.ret);
      _getenv.ret = allocate(intArrayFromString(ENV[name]), 'i8', ALLOC_NORMAL);
      return _getenv.ret;
    }

  function _gettimeofday(ptr) {
      var now = Date.now();
      HEAP32[((ptr)>>2)]=(now/1000)|0; // seconds
      HEAP32[(((ptr)+(4))>>2)]=((now % 1000)*1000)|0; // microseconds
      return 0;
    }

  
  function _chown(path, owner, group, dontResolveLastLink) {
      // int chown(const char *path, uid_t owner, gid_t group);
      // http://pubs.opengroup.org/onlinepubs/000095399/functions/chown.html
      // We don't support multiple users, so changing ownership makes no sense.
      // NOTE: The path argument may be a string, to simplify fchown().
      // NOTE: dontResolveLastLink is a shortcut for lchown(). It should never be
      //       used in client code.
      if (typeof path !== 'string') path = Pointer_stringify(path);
      try {
        FS.chown(path, owner, group);
        return 0;
      } catch (e) {
        FS.handleFSError(e);
        return -1;
      }
    }function _fchown(fildes, owner, group) {
      // int fchown(int fildes, uid_t owner, gid_t group);
      // http://pubs.opengroup.org/onlinepubs/000095399/functions/fchown.html
      try {
        FS.fchown(fildes, owner, group);
        return 0;
      } catch (e) {
        FS.handleFSError(e);
        return -1;
      }
    }

  
  function _emscripten_memcpy_big(dest, src, num) {
      HEAPU8.set(HEAPU8.subarray(src, src+num), dest);
      return dest;
    } 
  Module["_memcpy"] = _memcpy;

  function _fsync(fildes) {
      // int fsync(int fildes);
      // http://pubs.opengroup.org/onlinepubs/000095399/functions/fsync.html
      var stream = FS.getStream(fildes);
      if (stream) {
        // We write directly to the file system, so there's nothing to do here.
        return 0;
      } else {
        ___setErrNo(ERRNO_CODES.EBADF);
        return -1;
      }
    }

  function _sbrk(bytes) {
      // Implement a Linux-like 'memory area' for our 'process'.
      // Changes the size of the memory area by |bytes|; returns the
      // address of the previous top ('break') of the memory area
      // We control the "dynamic" memory - DYNAMIC_BASE to DYNAMICTOP
      var self = _sbrk;
      if (!self.called) {
        DYNAMICTOP = alignMemoryPage(DYNAMICTOP); // make sure we start out aligned
        self.called = true;
        assert(Runtime.dynamicAlloc);
        self.alloc = Runtime.dynamicAlloc;
        Runtime.dynamicAlloc = function() { abort('cannot dynamically allocate, sbrk now has control') };
      }
      var ret = DYNAMICTOP;
      if (bytes != 0) self.alloc(bytes);
      return ret;  // Previous break location.
    }

   
  Module["_memmove"] = _memmove;

  function ___errno_location() {
      return ___errno_state;
    }

  
  function _strerror_r(errnum, strerrbuf, buflen) {
      if (errnum in ERRNO_MESSAGES) {
        if (ERRNO_MESSAGES[errnum].length > buflen - 1) {
          return ___setErrNo(ERRNO_CODES.ERANGE);
        } else {
          var msg = ERRNO_MESSAGES[errnum];
          writeAsciiToMemory(msg, strerrbuf);
          return 0;
        }
      } else {
        return ___setErrNo(ERRNO_CODES.EINVAL);
      }
    }function _strerror(errnum) {
      if (!_strerror.buffer) _strerror.buffer = _malloc(256);
      _strerror_r(errnum, _strerror.buffer, 256);
      return _strerror.buffer;
    }

  function _mmap(start, num, prot, flags, fd, offset) {
      /* FIXME: Since mmap is normally implemented at the kernel level,
       * this implementation simply uses malloc underneath the call to
       * mmap.
       */
      var MAP_PRIVATE = 2;
      var ptr;
      var allocated = false;
  
      if (!_mmap.mappings) _mmap.mappings = {};
  
      if (fd == -1) {
        ptr = _malloc(num);
        if (!ptr) return -1;
        _memset(ptr, 0, num);
        allocated = true;
      } else {
        var info = FS.getStream(fd);
        if (!info) return -1;
        try {
          var res = FS.mmap(info, HEAPU8, start, num, offset, prot, flags);
          ptr = res.ptr;
          allocated = res.allocated;
        } catch (e) {
          FS.handleFSError(e);
          return -1;
        }
      }
  
      _mmap.mappings[ptr] = { malloc: ptr, num: num, allocated: allocated };
      return ptr;
    }

  
  function _truncate(path, length) {
      // int truncate(const char *path, off_t length);
      // http://pubs.opengroup.org/onlinepubs/000095399/functions/truncate.html
      // NOTE: The path argument may be a string, to simplify ftruncate().
      if (typeof path !== 'string') path = Pointer_stringify(path);
      try {
        FS.truncate(path, length);
        return 0;
      } catch (e) {
        FS.handleFSError(e);
        return -1;
      }
    }function _ftruncate(fildes, length) {
      // int ftruncate(int fildes, off_t length);
      // http://pubs.opengroup.org/onlinepubs/000095399/functions/ftruncate.html
      try {
        FS.ftruncate(fildes, length);
        return 0;
      } catch (e) {
        FS.handleFSError(e);
        return -1;
      }
    }

  function _utimes(path, times) {
      var time;
      if (times) {
        var offset = 8 + 0;
        time = HEAP32[(((times)+(offset))>>2)] * 1000;
        offset = 8 + 4;
        time += HEAP32[(((times)+(offset))>>2)] / 1000;
      } else {
        time = Date.now();
      }
      path = Pointer_stringify(path);
      try {
        FS.utime(path, time, time);
        return 0;
      } catch (e) {
        FS.handleFSError(e);
        return -1;
      }
    }
FS.staticInit();__ATINIT__.unshift({ func: function() { if (!Module["noFSInit"] && !FS.init.initialized) FS.init() } });__ATMAIN__.push({ func: function() { FS.ignorePermissions = false } });__ATEXIT__.push({ func: function() { FS.quit() } });Module["FS_createFolder"] = FS.createFolder;Module["FS_createPath"] = FS.createPath;Module["FS_createDataFile"] = FS.createDataFile;Module["FS_createPreloadedFile"] = FS.createPreloadedFile;Module["FS_createLazyFile"] = FS.createLazyFile;Module["FS_createLink"] = FS.createLink;Module["FS_createDevice"] = FS.createDevice;
___errno_state = Runtime.staticAlloc(4); HEAP32[((___errno_state)>>2)]=0;
__ATINIT__.unshift({ func: function() { TTY.init() } });__ATEXIT__.push({ func: function() { TTY.shutdown() } });TTY.utf8 = new Runtime.UTF8Processor();
if (ENVIRONMENT_IS_NODE) { var fs = require("fs"); NODEFS.staticInit(); }
__ATINIT__.push({ func: function() { SOCKFS.root = FS.mount(SOCKFS, {}, null); } });
Module["requestFullScreen"] = function Module_requestFullScreen(lockPointer, resizeCanvas) { Browser.requestFullScreen(lockPointer, resizeCanvas) };
  Module["requestAnimationFrame"] = function Module_requestAnimationFrame(func) { Browser.requestAnimationFrame(func) };
  Module["setCanvasSize"] = function Module_setCanvasSize(width, height, noUpdates) { Browser.setCanvasSize(width, height, noUpdates) };
  Module["pauseMainLoop"] = function Module_pauseMainLoop() { Browser.mainLoop.pause() };
  Module["resumeMainLoop"] = function Module_resumeMainLoop() { Browser.mainLoop.resume() };
  Module["getUserMedia"] = function Module_getUserMedia() { Browser.getUserMedia() }
___buildEnvironment(ENV);
STACK_BASE = STACKTOP = Runtime.alignMemory(STATICTOP);

staticSealed = true; // seal the static portion of memory

STACK_MAX = STACK_BASE + TOTAL_STACK;

DYNAMIC_BASE = DYNAMICTOP = Runtime.alignMemory(STACK_MAX);

assert(DYNAMIC_BASE < TOTAL_MEMORY, "TOTAL_MEMORY not big enough for stack");

 var ctlz_i8 = allocate([8,7,6,6,5,5,5,5,4,4,4,4,4,4,4,4,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0], "i8", ALLOC_DYNAMIC);
 var cttz_i8 = allocate([8,0,1,0,2,0,1,0,3,0,1,0,2,0,1,0,4,0,1,0,2,0,1,0,3,0,1,0,2,0,1,0,5,0,1,0,2,0,1,0,3,0,1,0,2,0,1,0,4,0,1,0,2,0,1,0,3,0,1,0,2,0,1,0,6,0,1,0,2,0,1,0,3,0,1,0,2,0,1,0,4,0,1,0,2,0,1,0,3,0,1,0,2,0,1,0,5,0,1,0,2,0,1,0,3,0,1,0,2,0,1,0,4,0,1,0,2,0,1,0,3,0,1,0,2,0,1,0,7,0,1,0,2,0,1,0,3,0,1,0,2,0,1,0,4,0,1,0,2,0,1,0,3,0,1,0,2,0,1,0,5,0,1,0,2,0,1,0,3,0,1,0,2,0,1,0,4,0,1,0,2,0,1,0,3,0,1,0,2,0,1,0,6,0,1,0,2,0,1,0,3,0,1,0,2,0,1,0,4,0,1,0,2,0,1,0,3,0,1,0,2,0,1,0,5,0,1,0,2,0,1,0,3,0,1,0,2,0,1,0,4,0,1,0,2,0,1,0,3,0,1,0,2,0,1,0], "i8", ALLOC_DYNAMIC);


function invoke_iiii(index,a1,a2,a3) {
  try {
    return Module["dynCall_iiii"](index,a1,a2,a3);
  } catch(e) {
    if (typeof e !== 'number' && e !== 'longjmp') throw e;
    asm["setThrew"](1, 0);
  }
}

function invoke_i(index) {
  try {
    return Module["dynCall_i"](index);
  } catch(e) {
    if (typeof e !== 'number' && e !== 'longjmp') throw e;
    asm["setThrew"](1, 0);
  }
}

function invoke_vi(index,a1) {
  try {
    Module["dynCall_vi"](index,a1);
  } catch(e) {
    if (typeof e !== 'number' && e !== 'longjmp') throw e;
    asm["setThrew"](1, 0);
  }
}

function invoke_vii(index,a1,a2) {
  try {
    Module["dynCall_vii"](index,a1,a2);
  } catch(e) {
    if (typeof e !== 'number' && e !== 'longjmp') throw e;
    asm["setThrew"](1, 0);
  }
}

function invoke_iiiiiii(index,a1,a2,a3,a4,a5,a6) {
  try {
    return Module["dynCall_iiiiiii"](index,a1,a2,a3,a4,a5,a6);
  } catch(e) {
    if (typeof e !== 'number' && e !== 'longjmp') throw e;
    asm["setThrew"](1, 0);
  }
}

function invoke_ii(index,a1) {
  try {
    return Module["dynCall_ii"](index,a1);
  } catch(e) {
    if (typeof e !== 'number' && e !== 'longjmp') throw e;
    asm["setThrew"](1, 0);
  }
}

function invoke_viii(index,a1,a2,a3) {
  try {
    Module["dynCall_viii"](index,a1,a2,a3);
  } catch(e) {
    if (typeof e !== 'number' && e !== 'longjmp') throw e;
    asm["setThrew"](1, 0);
  }
}

function invoke_v(index) {
  try {
    Module["dynCall_v"](index);
  } catch(e) {
    if (typeof e !== 'number' && e !== 'longjmp') throw e;
    asm["setThrew"](1, 0);
  }
}

function invoke_iiiii(index,a1,a2,a3,a4) {
  try {
    return Module["dynCall_iiiii"](index,a1,a2,a3,a4);
  } catch(e) {
    if (typeof e !== 'number' && e !== 'longjmp') throw e;
    asm["setThrew"](1, 0);
  }
}

function invoke_viiiiii(index,a1,a2,a3,a4,a5,a6) {
  try {
    Module["dynCall_viiiiii"](index,a1,a2,a3,a4,a5,a6);
  } catch(e) {
    if (typeof e !== 'number' && e !== 'longjmp') throw e;
    asm["setThrew"](1, 0);
  }
}

function invoke_iii(index,a1,a2) {
  try {
    return Module["dynCall_iii"](index,a1,a2);
  } catch(e) {
    if (typeof e !== 'number' && e !== 'longjmp') throw e;
    asm["setThrew"](1, 0);
  }
}

function invoke_iiiiii(index,a1,a2,a3,a4,a5) {
  try {
    return Module["dynCall_iiiiii"](index,a1,a2,a3,a4,a5);
  } catch(e) {
    if (typeof e !== 'number' && e !== 'longjmp') throw e;
    asm["setThrew"](1, 0);
  }
}

function invoke_viiii(index,a1,a2,a3,a4) {
  try {
    Module["dynCall_viiii"](index,a1,a2,a3,a4);
  } catch(e) {
    if (typeof e !== 'number' && e !== 'longjmp') throw e;
    asm["setThrew"](1, 0);
  }
}

Module.asmGlobalArg = { "Math": Math, "Int8Array": Int8Array, "Int16Array": Int16Array, "Int32Array": Int32Array, "Uint8Array": Uint8Array, "Uint16Array": Uint16Array, "Uint32Array": Uint32Array, "Float32Array": Float32Array, "Float64Array": Float64Array };
Module.asmLibraryArg = { "abort": abort, "assert": assert, "min": Math_min, "invoke_iiii": invoke_iiii, "invoke_i": invoke_i, "invoke_vi": invoke_vi, "invoke_vii": invoke_vii, "invoke_iiiiiii": invoke_iiiiiii, "invoke_ii": invoke_ii, "invoke_viii": invoke_viii, "invoke_v": invoke_v, "invoke_iiiii": invoke_iiiii, "invoke_viiiiii": invoke_viiiiii, "invoke_iii": invoke_iii, "invoke_iiiiii": invoke_iiiiii, "invoke_viiii": invoke_viiii, "_utimes": _utimes, "_getuid": _getuid, "_send": _send, "_getcwd": _getcwd, "_mknod": _mknod, "_lseek": _lseek, "_emscripten_set_main_loop_timing": _emscripten_set_main_loop_timing, "_access": _access, "_fstat": _fstat, "_chmod": _chmod, "_rmdir": _rmdir, "_chown": _chown, "_fcntl": _fcntl, "___buildEnvironment": ___buildEnvironment, "_fflush": _fflush, "_pwrite": _pwrite, "_strerror_r": _strerror_r, "_localtime_r": _localtime_r, "_tzset": _tzset, "_open": _open, "_getpid": _getpid, "_sbrk": _sbrk, "_usleep": _usleep, "_emscripten_memcpy_big": _emscripten_memcpy_big, "_unlink": _unlink, "_sysconf": _sysconf, "_fchmod": _fchmod, "___setErrNo": ___setErrNo, "_ftruncate": _ftruncate, "_mkdir": _mkdir, "_pread": _pread, "_mkport": _mkport, "_write": _write, "_fsync": _fsync, "___errno_location": ___errno_location, "_stat": _stat, "_recv": _recv, "_geteuid": _geteuid, "_getenv": _getenv, "_sleep": _sleep, "_emscripten_set_main_loop": _emscripten_set_main_loop, "_abort": _abort, "_time": _time, "_fchown": _fchown, "_strerror": _strerror, "_gettimeofday": _gettimeofday, "_munmap": _munmap, "_mmap": _mmap, "_localtime": _localtime, "_close": _close, "_read": _read, "_truncate": _truncate, "STACKTOP": STACKTOP, "STACK_MAX": STACK_MAX, "tempDoublePtr": tempDoublePtr, "ABORT": ABORT, "cttz_i8": cttz_i8, "ctlz_i8": ctlz_i8, "NaN": NaN, "Infinity": Infinity };
// EMSCRIPTEN_START_ASM
var asm = (function(global, env, buffer) {
  'use asm';
  
  var HEAP8 = new global.Int8Array(buffer);
  var HEAP16 = new global.Int16Array(buffer);
  var HEAP32 = new global.Int32Array(buffer);
  var HEAPU8 = new global.Uint8Array(buffer);
  var HEAPU16 = new global.Uint16Array(buffer);
  var HEAPU32 = new global.Uint32Array(buffer);
  var HEAPF32 = new global.Float32Array(buffer);
  var HEAPF64 = new global.Float64Array(buffer);


  var STACKTOP=env.STACKTOP|0;
  var STACK_MAX=env.STACK_MAX|0;
  var tempDoublePtr=env.tempDoublePtr|0;
  var ABORT=env.ABORT|0;
  var cttz_i8=env.cttz_i8|0;
  var ctlz_i8=env.ctlz_i8|0;

  var __THREW__ = 0;
  var threwValue = 0;
  var setjmpId = 0;
  var undef = 0;
  var nan = +env.NaN, inf = +env.Infinity;
  var tempInt = 0, tempBigInt = 0, tempBigIntP = 0, tempBigIntS = 0, tempBigIntR = 0.0, tempBigIntI = 0, tempBigIntD = 0, tempValue = 0, tempDouble = 0.0;

  var tempRet0 = 0;
  var tempRet1 = 0;
  var tempRet2 = 0;
  var tempRet3 = 0;
  var tempRet4 = 0;
  var tempRet5 = 0;
  var tempRet6 = 0;
  var tempRet7 = 0;
  var tempRet8 = 0;
  var tempRet9 = 0;
  var Math_floor=global.Math.floor;
  var Math_abs=global.Math.abs;
  var Math_sqrt=global.Math.sqrt;
  var Math_pow=global.Math.pow;
  var Math_cos=global.Math.cos;
  var Math_sin=global.Math.sin;
  var Math_tan=global.Math.tan;
  var Math_acos=global.Math.acos;
  var Math_asin=global.Math.asin;
  var Math_atan=global.Math.atan;
  var Math_atan2=global.Math.atan2;
  var Math_exp=global.Math.exp;
  var Math_log=global.Math.log;
  var Math_ceil=global.Math.ceil;
  var Math_imul=global.Math.imul;
  var abort=env.abort;
  var assert=env.assert;
  var Math_min=env.min;
  var invoke_iiii=env.invoke_iiii;
  var invoke_i=env.invoke_i;
  var invoke_vi=env.invoke_vi;
  var invoke_vii=env.invoke_vii;
  var invoke_iiiiiii=env.invoke_iiiiiii;
  var invoke_ii=env.invoke_ii;
  var invoke_viii=env.invoke_viii;
  var invoke_v=env.invoke_v;
  var invoke_iiiii=env.invoke_iiiii;
  var invoke_viiiiii=env.invoke_viiiiii;
  var invoke_iii=env.invoke_iii;
  var invoke_iiiiii=env.invoke_iiiiii;
  var invoke_viiii=env.invoke_viiii;
  var _utimes=env._utimes;
  var _getuid=env._getuid;
  var _send=env._send;
  var _getcwd=env._getcwd;
  var _mknod=env._mknod;
  var _lseek=env._lseek;
  var _emscripten_set_main_loop_timing=env._emscripten_set_main_loop_timing;
  var _access=env._access;
  var _fstat=env._fstat;
  var _chmod=env._chmod;
  var _rmdir=env._rmdir;
  var _chown=env._chown;
  var _fcntl=env._fcntl;
  var ___buildEnvironment=env.___buildEnvironment;
  var _fflush=env._fflush;
  var _pwrite=env._pwrite;
  var _strerror_r=env._strerror_r;
  var _localtime_r=env._localtime_r;
  var _tzset=env._tzset;
  var _open=env._open;
  var _getpid=env._getpid;
  var _sbrk=env._sbrk;
  var _usleep=env._usleep;
  var _emscripten_memcpy_big=env._emscripten_memcpy_big;
  var _unlink=env._unlink;
  var _sysconf=env._sysconf;
  var _fchmod=env._fchmod;
  var ___setErrNo=env.___setErrNo;
  var _ftruncate=env._ftruncate;
  var _mkdir=env._mkdir;
  var _pread=env._pread;
  var _mkport=env._mkport;
  var _write=env._write;
  var _fsync=env._fsync;
  var ___errno_location=env.___errno_location;
  var _stat=env._stat;
  var _recv=env._recv;
  var _geteuid=env._geteuid;
  var _getenv=env._getenv;
  var _sleep=env._sleep;
  var _emscripten_set_main_loop=env._emscripten_set_main_loop;
  var _abort=env._abort;
  var _time=env._time;
  var _fchown=env._fchown;
  var _strerror=env._strerror;
  var _gettimeofday=env._gettimeofday;
  var _munmap=env._munmap;
  var _mmap=env._mmap;
  var _localtime=env._localtime;
  var _close=env._close;
  var _read=env._read;
  var _truncate=env._truncate;
  var tempFloat = 0.0;

// EMSCRIPTEN_START_FUNCS
function stackAlloc(size) {
  size = size|0;
  var ret = 0;
  ret = STACKTOP;
  STACKTOP = (STACKTOP + size)|0;
STACKTOP = (STACKTOP + 15)&-16;

  return ret|0;
}
function stackSave() {
  return STACKTOP|0;
}
function stackRestore(top) {
  top = top|0;
  STACKTOP = top;
}

function setThrew(threw, value) {
  threw = threw|0;
  value = value|0;
  if ((__THREW__|0) == 0) {
    __THREW__ = threw;
    threwValue = value;
  }
}
function copyTempFloat(ptr) {
  ptr = ptr|0;
  HEAP8[tempDoublePtr>>0] = HEAP8[ptr>>0];
  HEAP8[tempDoublePtr+1>>0] = HEAP8[ptr+1>>0];
  HEAP8[tempDoublePtr+2>>0] = HEAP8[ptr+2>>0];
  HEAP8[tempDoublePtr+3>>0] = HEAP8[ptr+3>>0];
}
function copyTempDouble(ptr) {
  ptr = ptr|0;
  HEAP8[tempDoublePtr>>0] = HEAP8[ptr>>0];
  HEAP8[tempDoublePtr+1>>0] = HEAP8[ptr+1>>0];
  HEAP8[tempDoublePtr+2>>0] = HEAP8[ptr+2>>0];
  HEAP8[tempDoublePtr+3>>0] = HEAP8[ptr+3>>0];
  HEAP8[tempDoublePtr+4>>0] = HEAP8[ptr+4>>0];
  HEAP8[tempDoublePtr+5>>0] = HEAP8[ptr+5>>0];
  HEAP8[tempDoublePtr+6>>0] = HEAP8[ptr+6>>0];
  HEAP8[tempDoublePtr+7>>0] = HEAP8[ptr+7>>0];
}
function setTempRet0(value) {
  value = value|0;
  tempRet0 = value;
}
function getTempRet0() {
  return tempRet0|0;
}

function _sqlite3_compileoption_used($zOptName) {
 $zOptName = $zOptName|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $i = 0, $n = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $1 = sp + 8|0;
 $i = sp + 4|0;
 $n = sp;
 HEAP32[$1>>2] = $zOptName;
 $2 = HEAP32[$1>>2]|0;
 $3 = (_sqlite3_strnicmp($2,16,7)|0);
 $4 = ($3|0)==(0);
 if ($4) {
  $5 = HEAP32[$1>>2]|0;
  $6 = (($5) + 7|0);
  HEAP32[$1>>2] = $6;
 }
 $7 = HEAP32[$1>>2]|0;
 $8 = (_sqlite3Strlen30($7)|0);
 HEAP32[$n>>2] = $8;
 HEAP32[$i>>2] = 0;
 while(1) {
  $9 = HEAP32[$i>>2]|0;
  $10 = ($9|0)<(5);
  if (!($10)) {
   label = 9;
   break;
  }
  $11 = HEAP32[$1>>2]|0;
  $12 = HEAP32[$i>>2]|0;
  $13 = (24 + ($12<<2)|0);
  $14 = HEAP32[$13>>2]|0;
  $15 = HEAP32[$n>>2]|0;
  $16 = (_sqlite3_strnicmp($11,$14,$15)|0);
  $17 = ($16|0)==(0);
  if ($17) {
   $18 = HEAP32[$n>>2]|0;
   $19 = HEAP32[$i>>2]|0;
   $20 = (24 + ($19<<2)|0);
   $21 = HEAP32[$20>>2]|0;
   $22 = (($21) + ($18)|0);
   $23 = HEAP8[$22>>0]|0;
   $24 = (_sqlite3IsIdChar($23)|0);
   $25 = ($24|0)==(0);
   if ($25) {
    label = 7;
    break;
   }
  }
  $26 = HEAP32[$i>>2]|0;
  $27 = (($26) + 1)|0;
  HEAP32[$i>>2] = $27;
 }
 if ((label|0) == 7) {
  HEAP32[$0>>2] = 1;
  $28 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($28|0);
 }
 else if ((label|0) == 9) {
  HEAP32[$0>>2] = 0;
  $28 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($28|0);
 }
 return 0|0;
}
function _sqlite3_strnicmp($zLeft,$zRight,$N) {
 $zLeft = $zLeft|0;
 $zRight = $zRight|0;
 $N = $N|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $a = 0, $b = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 16|0;
 $1 = sp + 12|0;
 $2 = sp + 8|0;
 $a = sp + 4|0;
 $b = sp;
 HEAP32[$0>>2] = $zLeft;
 HEAP32[$1>>2] = $zRight;
 HEAP32[$2>>2] = $N;
 $3 = HEAP32[$0>>2]|0;
 HEAP32[$a>>2] = $3;
 $4 = HEAP32[$1>>2]|0;
 HEAP32[$b>>2] = $4;
 while(1) {
  $5 = HEAP32[$2>>2]|0;
  $6 = (($5) + -1)|0;
  HEAP32[$2>>2] = $6;
  $7 = ($5|0)>(0);
  if (!($7)) {
   break;
  }
  $8 = HEAP32[$a>>2]|0;
  $9 = HEAP8[$8>>0]|0;
  $10 = $9&255;
  $11 = ($10|0)!=(0);
  if (!($11)) {
   break;
  }
  $12 = HEAP32[$a>>2]|0;
  $13 = HEAP8[$12>>0]|0;
  $14 = $13&255;
  $15 = (440 + ($14)|0);
  $16 = HEAP8[$15>>0]|0;
  $17 = $16&255;
  $18 = HEAP32[$b>>2]|0;
  $19 = HEAP8[$18>>0]|0;
  $20 = $19&255;
  $21 = (440 + ($20)|0);
  $22 = HEAP8[$21>>0]|0;
  $23 = $22&255;
  $24 = ($17|0)==($23|0);
  if (!($24)) {
   break;
  }
  $25 = HEAP32[$a>>2]|0;
  $26 = (($25) + 1|0);
  HEAP32[$a>>2] = $26;
  $27 = HEAP32[$b>>2]|0;
  $28 = (($27) + 1|0);
  HEAP32[$b>>2] = $28;
 }
 $29 = HEAP32[$2>>2]|0;
 $30 = ($29|0)<(0);
 if ($30) {
  $44 = 0;
  STACKTOP = sp;return ($44|0);
 }
 $31 = HEAP32[$a>>2]|0;
 $32 = HEAP8[$31>>0]|0;
 $33 = $32&255;
 $34 = (440 + ($33)|0);
 $35 = HEAP8[$34>>0]|0;
 $36 = $35&255;
 $37 = HEAP32[$b>>2]|0;
 $38 = HEAP8[$37>>0]|0;
 $39 = $38&255;
 $40 = (440 + ($39)|0);
 $41 = HEAP8[$40>>0]|0;
 $42 = $41&255;
 $43 = (($36) - ($42))|0;
 $44 = $43;
 STACKTOP = sp;return ($44|0);
}
function _sqlite3Strlen30($z) {
 $z = $z|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $z2 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $1 = sp + 4|0;
 $z2 = sp;
 HEAP32[$1>>2] = $z;
 $2 = HEAP32[$1>>2]|0;
 HEAP32[$z2>>2] = $2;
 $3 = HEAP32[$1>>2]|0;
 $4 = ($3|0)==(0|0);
 if ($4) {
  HEAP32[$0>>2] = 0;
  $15 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($15|0);
 }
 while(1) {
  $5 = HEAP32[$z2>>2]|0;
  $6 = HEAP8[$5>>0]|0;
  $7 = ($6<<24>>24)!=(0);
  $8 = HEAP32[$z2>>2]|0;
  if (!($7)) {
   break;
  }
  $9 = (($8) + 1|0);
  HEAP32[$z2>>2] = $9;
 }
 $10 = HEAP32[$1>>2]|0;
 $11 = $8;
 $12 = $10;
 $13 = (($11) - ($12))|0;
 $14 = 1073741823 & $13;
 HEAP32[$0>>2] = $14;
 $15 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($15|0);
}
function _sqlite3IsIdChar($c) {
 $c = $c|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 HEAP8[$0>>0] = $c;
 $1 = HEAP8[$0>>0]|0;
 $2 = $1&255;
 $3 = (1224 + ($2)|0);
 $4 = HEAP8[$3>>0]|0;
 $5 = $4&255;
 $6 = $5 & 70;
 $7 = ($6|0)!=(0);
 $8 = $7&1;
 STACKTOP = sp;return ($8|0);
}
function _sqlite3_compileoption_get($N) {
 $N = $N|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $or$cond = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $1 = sp;
 HEAP32[$1>>2] = $N;
 $2 = HEAP32[$1>>2]|0;
 $3 = ($2|0)>=(0);
 $4 = HEAP32[$1>>2]|0;
 $5 = ($4|0)<(5);
 $or$cond = $3 & $5;
 if ($or$cond) {
  $6 = HEAP32[$1>>2]|0;
  $7 = (24 + ($6<<2)|0);
  $8 = HEAP32[$7>>2]|0;
  HEAP32[$0>>2] = $8;
  $9 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($9|0);
 } else {
  HEAP32[$0>>2] = 0;
  $9 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($9|0);
 }
 return 0|0;
}
function _sqlite3_status($op,$pCurrent,$pHighwater,$resetFlag) {
 $op = $op|0;
 $pCurrent = $pCurrent|0;
 $pHighwater = $pHighwater|0;
 $resetFlag = $resetFlag|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $3 = 0;
 var $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $or$cond = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 16|0;
 $1 = sp + 12|0;
 $2 = sp + 8|0;
 $3 = sp + 4|0;
 $4 = sp;
 HEAP32[$1>>2] = $op;
 HEAP32[$2>>2] = $pCurrent;
 HEAP32[$3>>2] = $pHighwater;
 HEAP32[$4>>2] = $resetFlag;
 $5 = HEAP32[$1>>2]|0;
 $6 = ($5|0)<(0);
 $7 = HEAP32[$1>>2]|0;
 $8 = ($7|0)>=(10);
 $or$cond = $6 | $8;
 if ($or$cond) {
  $9 = (_sqlite3MisuseError(14581)|0);
  HEAP32[$0>>2] = $9;
  $25 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($25|0);
 }
 $10 = HEAP32[$1>>2]|0;
 $11 = (48 + ($10<<2)|0);
 $12 = HEAP32[$11>>2]|0;
 $13 = HEAP32[$2>>2]|0;
 HEAP32[$13>>2] = $12;
 $14 = HEAP32[$1>>2]|0;
 $15 = (((48 + 40|0)) + ($14<<2)|0);
 $16 = HEAP32[$15>>2]|0;
 $17 = HEAP32[$3>>2]|0;
 HEAP32[$17>>2] = $16;
 $18 = HEAP32[$4>>2]|0;
 $19 = ($18|0)!=(0);
 if ($19) {
  $20 = HEAP32[$1>>2]|0;
  $21 = (48 + ($20<<2)|0);
  $22 = HEAP32[$21>>2]|0;
  $23 = HEAP32[$1>>2]|0;
  $24 = (((48 + 40|0)) + ($23<<2)|0);
  HEAP32[$24>>2] = $22;
 }
 HEAP32[$0>>2] = 0;
 $25 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($25|0);
}
function _sqlite3MisuseError($lineno) {
 $lineno = $lineno|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $vararg_buffer = 0, $vararg_ptr1 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $vararg_buffer = sp;
 $0 = sp + 8|0;
 HEAP32[$0>>2] = $lineno;
 $1 = HEAP32[$0>>2]|0;
 $2 = (_sqlite3_sourceid()|0);
 $3 = (($2) + 20|0);
 HEAP32[$vararg_buffer>>2] = $1;
 $vararg_ptr1 = (($vararg_buffer) + 4|0);
 HEAP32[$vararg_ptr1>>2] = $3;
 _sqlite3_log(21,3216,$vararg_buffer);
 STACKTOP = sp;return 21;
}
function _sqlite3BtreeEnterAll($db) {
 $db = $db|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0;
 var $7 = 0, $8 = 0, $9 = 0, $i = 0, $p = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $i = sp + 4|0;
 $p = sp;
 HEAP32[$0>>2] = $db;
 HEAP32[$i>>2] = 0;
 while(1) {
  $1 = HEAP32[$i>>2]|0;
  $2 = HEAP32[$0>>2]|0;
  $3 = (($2) + 20|0);
  $4 = HEAP32[$3>>2]|0;
  $5 = ($1|0)<($4|0);
  if (!($5)) {
   break;
  }
  $6 = HEAP32[$i>>2]|0;
  $7 = HEAP32[$0>>2]|0;
  $8 = (($7) + 16|0);
  $9 = HEAP32[$8>>2]|0;
  $10 = (($9) + ($6<<4)|0);
  $11 = (($10) + 4|0);
  $12 = HEAP32[$11>>2]|0;
  HEAP32[$p>>2] = $12;
  $13 = HEAP32[$p>>2]|0;
  $14 = ($13|0)!=(0|0);
  if ($14) {
   $15 = HEAP32[$p>>2]|0;
   $16 = HEAP32[$15>>2]|0;
   $17 = HEAP32[$p>>2]|0;
   $18 = (($17) + 4|0);
   $19 = HEAP32[$18>>2]|0;
   $20 = (($19) + 4|0);
   HEAP32[$20>>2] = $16;
  }
  $21 = HEAP32[$i>>2]|0;
  $22 = (($21) + 1)|0;
  HEAP32[$i>>2] = $22;
 }
 STACKTOP = sp;return;
}
function _sqlite3BtreePager($p) {
 $p = $p|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 HEAP32[$0>>2] = $p;
 $1 = HEAP32[$0>>2]|0;
 $2 = (($1) + 4|0);
 $3 = HEAP32[$2>>2]|0;
 $4 = HEAP32[$3>>2]|0;
 STACKTOP = sp;return ($4|0);
}
function _sqlite3MallocSize($p) {
 $p = $p|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 HEAP32[$0>>2] = $p;
 $1 = HEAP32[((38640 + 48|0))>>2]|0;
 $2 = HEAP32[$0>>2]|0;
 $3 = (FUNCTION_TABLE_ii[$1 & 31]($2)|0);
 STACKTOP = sp;return ($3|0);
}
function _sqlite3DeleteTrigger($db,$pTrigger) {
 $db = $db|0;
 $pTrigger = $pTrigger|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $3 = 0, $4 = 0;
 var $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $1 = sp;
 HEAP32[$0>>2] = $db;
 HEAP32[$1>>2] = $pTrigger;
 $2 = HEAP32[$1>>2]|0;
 $3 = ($2|0)==(0|0);
 if ($3) {
  STACKTOP = sp;return;
 }
 $4 = HEAP32[$0>>2]|0;
 $5 = HEAP32[$1>>2]|0;
 $6 = (($5) + 28|0);
 $7 = HEAP32[$6>>2]|0;
 _sqlite3DeleteTriggerStep($4,$7);
 $8 = HEAP32[$0>>2]|0;
 $9 = HEAP32[$1>>2]|0;
 $10 = HEAP32[$9>>2]|0;
 _sqlite3DbFree($8,$10);
 $11 = HEAP32[$0>>2]|0;
 $12 = HEAP32[$1>>2]|0;
 $13 = (($12) + 4|0);
 $14 = HEAP32[$13>>2]|0;
 _sqlite3DbFree($11,$14);
 $15 = HEAP32[$0>>2]|0;
 $16 = HEAP32[$1>>2]|0;
 $17 = (($16) + 12|0);
 $18 = HEAP32[$17>>2]|0;
 _sqlite3ExprDelete($15,$18);
 $19 = HEAP32[$0>>2]|0;
 $20 = HEAP32[$1>>2]|0;
 $21 = (($20) + 16|0);
 $22 = HEAP32[$21>>2]|0;
 _sqlite3IdListDelete($19,$22);
 $23 = HEAP32[$0>>2]|0;
 $24 = HEAP32[$1>>2]|0;
 _sqlite3DbFree($23,$24);
 STACKTOP = sp;return;
}
function _sqlite3DeleteTable($db,$pTable) {
 $db = $db|0;
 $pTable = $pTable|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $7 = 0, $8 = 0, $9 = 0, $pIndex = 0, $pNext = 0, $zName = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 16|0;
 $1 = sp + 12|0;
 $pIndex = sp + 8|0;
 $pNext = sp + 4|0;
 $zName = sp;
 HEAP32[$0>>2] = $db;
 HEAP32[$1>>2] = $pTable;
 $2 = HEAP32[$1>>2]|0;
 $3 = ($2|0)!=(0|0);
 if (!($3)) {
  STACKTOP = sp;return;
 }
 $4 = HEAP32[$0>>2]|0;
 $5 = ($4|0)!=(0|0);
 if ($5) {
  $6 = HEAP32[$0>>2]|0;
  $7 = (($6) + 520|0);
  $8 = HEAP32[$7>>2]|0;
  $9 = ($8|0)==(0|0);
  if ($9) {
   label = 4;
  }
 } else {
  label = 4;
 }
 if ((label|0) == 4) {
  $10 = HEAP32[$1>>2]|0;
  $11 = (($10) + 40|0);
  $12 = HEAP16[$11>>1]|0;
  $13 = (($12) + -1)<<16>>16;
  HEAP16[$11>>1] = $13;
  $14 = $13&65535;
  $15 = ($14|0)>(0);
  if ($15) {
   STACKTOP = sp;return;
  }
 }
 $16 = HEAP32[$1>>2]|0;
 $17 = (($16) + 8|0);
 $18 = HEAP32[$17>>2]|0;
 HEAP32[$pIndex>>2] = $18;
 while(1) {
  $19 = HEAP32[$pIndex>>2]|0;
  $20 = ($19|0)!=(0|0);
  if (!($20)) {
   break;
  }
  $21 = HEAP32[$pIndex>>2]|0;
  $22 = (($21) + 20|0);
  $23 = HEAP32[$22>>2]|0;
  HEAP32[$pNext>>2] = $23;
  $24 = HEAP32[$0>>2]|0;
  $25 = ($24|0)!=(0|0);
  if ($25) {
   $26 = HEAP32[$0>>2]|0;
   $27 = (($26) + 520|0);
   $28 = HEAP32[$27>>2]|0;
   $29 = ($28|0)==(0|0);
   if ($29) {
    label = 9;
   }
  } else {
   label = 9;
  }
  if ((label|0) == 9) {
   label = 0;
   $30 = HEAP32[$pIndex>>2]|0;
   $31 = HEAP32[$30>>2]|0;
   HEAP32[$zName>>2] = $31;
   $32 = HEAP32[$pIndex>>2]|0;
   $33 = (($32) + 24|0);
   $34 = HEAP32[$33>>2]|0;
   $35 = (($34) + 24|0);
   $36 = HEAP32[$zName>>2]|0;
   (_sqlite3HashInsert($35,$36,0)|0);
  }
  $37 = HEAP32[$0>>2]|0;
  $38 = HEAP32[$pIndex>>2]|0;
  _freeIndex($37,$38);
  $39 = HEAP32[$pNext>>2]|0;
  HEAP32[$pIndex>>2] = $39;
 }
 $40 = HEAP32[$0>>2]|0;
 $41 = HEAP32[$1>>2]|0;
 _sqlite3FkDelete($40,$41);
 $42 = HEAP32[$0>>2]|0;
 $43 = HEAP32[$1>>2]|0;
 _sqliteDeleteColumnNames($42,$43);
 $44 = HEAP32[$0>>2]|0;
 $45 = HEAP32[$1>>2]|0;
 $46 = HEAP32[$45>>2]|0;
 _sqlite3DbFree($44,$46);
 $47 = HEAP32[$0>>2]|0;
 $48 = HEAP32[$1>>2]|0;
 $49 = (($48) + 20|0);
 $50 = HEAP32[$49>>2]|0;
 _sqlite3DbFree($47,$50);
 $51 = HEAP32[$0>>2]|0;
 $52 = HEAP32[$1>>2]|0;
 $53 = (($52) + 12|0);
 $54 = HEAP32[$53>>2]|0;
 _sqlite3SelectDelete($51,$54);
 $55 = HEAP32[$0>>2]|0;
 $56 = HEAP32[$1>>2]|0;
 $57 = (($56) + 24|0);
 $58 = HEAP32[$57>>2]|0;
 _sqlite3ExprListDelete($55,$58);
 $59 = HEAP32[$0>>2]|0;
 $60 = HEAP32[$1>>2]|0;
 _sqlite3VtabClear($59,$60);
 $61 = HEAP32[$0>>2]|0;
 $62 = HEAP32[$1>>2]|0;
 _sqlite3DbFree($61,$62);
 STACKTOP = sp;return;
}
function _sqlite3VdbeClearObject($db,$p) {
 $db = $db|0;
 $p = $p|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $7 = 0, $8 = 0, $9 = 0, $i = 0, $pNext = 0, $pSub = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 16|0;
 $1 = sp + 12|0;
 $pSub = sp + 8|0;
 $pNext = sp + 4|0;
 $i = sp;
 HEAP32[$0>>2] = $db;
 HEAP32[$1>>2] = $p;
 $2 = HEAP32[$1>>2]|0;
 $3 = (($2) + 60|0);
 $4 = HEAP32[$3>>2]|0;
 $5 = HEAP32[$1>>2]|0;
 $6 = (($5) + 68|0);
 $7 = HEAP16[$6>>1]|0;
 $8 = $7 << 16 >> 16;
 _releaseMemArray($4,$8);
 $9 = HEAP32[$1>>2]|0;
 $10 = (($9) + 16|0);
 $11 = HEAP32[$10>>2]|0;
 $12 = HEAP32[$1>>2]|0;
 $13 = (($12) + 84|0);
 $14 = HEAP16[$13>>1]|0;
 $15 = $14&65535;
 $16 = $15<<1;
 _releaseMemArray($11,$16);
 $17 = HEAP32[$1>>2]|0;
 $18 = (($17) + 192|0);
 $19 = HEAP32[$18>>2]|0;
 HEAP32[$pSub>>2] = $19;
 while(1) {
  $20 = HEAP32[$pSub>>2]|0;
  $21 = ($20|0)!=(0|0);
  if (!($21)) {
   break;
  }
  $22 = HEAP32[$pSub>>2]|0;
  $23 = (($22) + 24|0);
  $24 = HEAP32[$23>>2]|0;
  HEAP32[$pNext>>2] = $24;
  $25 = HEAP32[$0>>2]|0;
  $26 = HEAP32[$pSub>>2]|0;
  $27 = HEAP32[$26>>2]|0;
  $28 = HEAP32[$pSub>>2]|0;
  $29 = (($28) + 4|0);
  $30 = HEAP32[$29>>2]|0;
  _vdbeFreeOpArray($25,$27,$30);
  $31 = HEAP32[$0>>2]|0;
  $32 = HEAP32[$pSub>>2]|0;
  _sqlite3DbFree($31,$32);
  $33 = HEAP32[$pNext>>2]|0;
  HEAP32[$pSub>>2] = $33;
 }
 $34 = HEAP32[$1>>2]|0;
 $35 = (($34) + 70|0);
 $36 = HEAP16[$35>>1]|0;
 $37 = $36 << 16 >> 16;
 $38 = (($37) - 1)|0;
 HEAP32[$i>>2] = $38;
 while(1) {
  $39 = HEAP32[$i>>2]|0;
  $40 = ($39|0)>=(0);
  $41 = HEAP32[$0>>2]|0;
  if (!($40)) {
   break;
  }
  $42 = HEAP32[$i>>2]|0;
  $43 = HEAP32[$1>>2]|0;
  $44 = (($43) + 64|0);
  $45 = HEAP32[$44>>2]|0;
  $46 = (($45) + ($42<<2)|0);
  $47 = HEAP32[$46>>2]|0;
  _sqlite3DbFree($41,$47);
  $48 = HEAP32[$i>>2]|0;
  $49 = (($48) + -1)|0;
  HEAP32[$i>>2] = $49;
 }
 $50 = HEAP32[$1>>2]|0;
 $51 = (($50) + 4|0);
 $52 = HEAP32[$51>>2]|0;
 $53 = HEAP32[$1>>2]|0;
 $54 = (($53) + 32|0);
 $55 = HEAP32[$54>>2]|0;
 _vdbeFreeOpArray($41,$52,$55);
 $56 = HEAP32[$0>>2]|0;
 $57 = HEAP32[$1>>2]|0;
 $58 = (($57) + 16|0);
 $59 = HEAP32[$58>>2]|0;
 _sqlite3DbFree($56,$59);
 $60 = HEAP32[$0>>2]|0;
 $61 = HEAP32[$1>>2]|0;
 $62 = (($61) + 168|0);
 $63 = HEAP32[$62>>2]|0;
 _sqlite3DbFree($60,$63);
 $64 = HEAP32[$0>>2]|0;
 $65 = HEAP32[$1>>2]|0;
 $66 = (($65) + 172|0);
 $67 = HEAP32[$66>>2]|0;
 _sqlite3DbFree($64,$67);
 STACKTOP = sp;return;
}
function _sqlite3DbFree($db,$p) {
 $db = $db|0;
 $p = $p|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $pBuf = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $1 = sp + 4|0;
 $pBuf = sp;
 HEAP32[$0>>2] = $db;
 HEAP32[$1>>2] = $p;
 $2 = HEAP32[$1>>2]|0;
 $3 = ($2|0)==(0|0);
 if ($3) {
  STACKTOP = sp;return;
 }
 $4 = HEAP32[$0>>2]|0;
 $5 = ($4|0)!=(0|0);
 if ($5) {
  $6 = HEAP32[$0>>2]|0;
  $7 = (($6) + 520|0);
  $8 = HEAP32[$7>>2]|0;
  $9 = ($8|0)!=(0|0);
  $10 = HEAP32[$0>>2]|0;
  $11 = HEAP32[$1>>2]|0;
  if ($9) {
   _measureAllocationSize($10,$11);
   STACKTOP = sp;return;
  }
  $12 = (_isLookaside($10,$11)|0);
  $13 = ($12|0)!=(0);
  if ($13) {
   $14 = HEAP32[$1>>2]|0;
   HEAP32[$pBuf>>2] = $14;
   $15 = HEAP32[$0>>2]|0;
   $16 = (($15) + 248|0);
   $17 = (($16) + 24|0);
   $18 = HEAP32[$17>>2]|0;
   $19 = HEAP32[$pBuf>>2]|0;
   HEAP32[$19>>2] = $18;
   $20 = HEAP32[$pBuf>>2]|0;
   $21 = HEAP32[$0>>2]|0;
   $22 = (($21) + 248|0);
   $23 = (($22) + 24|0);
   HEAP32[$23>>2] = $20;
   $24 = HEAP32[$0>>2]|0;
   $25 = (($24) + 248|0);
   $26 = (($25) + 4|0);
   $27 = HEAP32[$26>>2]|0;
   $28 = (($27) + -1)|0;
   HEAP32[$26>>2] = $28;
   STACKTOP = sp;return;
  }
 }
 $29 = HEAP32[$1>>2]|0;
 _sqlite3_free($29);
 STACKTOP = sp;return;
}
function _sqlite3_vfs_find($zVfs) {
 $zVfs = $zVfs|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0;
 var $9 = 0, $or$cond = 0, $pVfs = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $1 = sp + 8|0;
 $pVfs = sp + 4|0;
 $rc = sp;
 HEAP32[$1>>2] = $zVfs;
 HEAP32[$pVfs>>2] = 0;
 $2 = (_sqlite3_initialize()|0);
 HEAP32[$rc>>2] = $2;
 $3 = HEAP32[$rc>>2]|0;
 $4 = ($3|0)!=(0);
 if ($4) {
  HEAP32[$0>>2] = 0;
  $20 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($20|0);
 }
 $5 = HEAP32[128>>2]|0;
 HEAP32[$pVfs>>2] = $5;
 while(1) {
  $6 = HEAP32[$pVfs>>2]|0;
  $7 = ($6|0)==(0|0);
  $8 = HEAP32[$1>>2]|0;
  $9 = ($8|0)==(0|0);
  $or$cond = $7 | $9;
  if ($or$cond) {
   break;
  }
  $10 = HEAP32[$1>>2]|0;
  $11 = HEAP32[$pVfs>>2]|0;
  $12 = (($11) + 16|0);
  $13 = HEAP32[$12>>2]|0;
  $14 = (_strcmp($10,$13)|0);
  $15 = ($14|0)==(0);
  if ($15) {
   break;
  }
  $16 = HEAP32[$pVfs>>2]|0;
  $17 = (($16) + 12|0);
  $18 = HEAP32[$17>>2]|0;
  HEAP32[$pVfs>>2] = $18;
 }
 $19 = HEAP32[$pVfs>>2]|0;
 HEAP32[$0>>2] = $19;
 $20 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($20|0);
}
function _sqlite3_initialize() {
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $pHash = 0, $rc = 0, dest = 0, label = 0, sp = 0, stop = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $rc = sp + 4|0;
 $pHash = sp;
 $1 = HEAP32[((38640 + 224|0))>>2]|0;
 $2 = ($1|0)!=(0);
 if ($2) {
  HEAP32[$0>>2] = 0;
  $45 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($45|0);
 }
 HEAP32[$rc>>2] = 0;
 $3 = HEAP32[$rc>>2]|0;
 $4 = ($3|0)!=(0);
 if ($4) {
  $5 = HEAP32[$rc>>2]|0;
  HEAP32[$0>>2] = $5;
  $45 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($45|0);
 }
 HEAP32[((38640 + 232|0))>>2] = 1;
 $6 = HEAP32[((38640 + 236|0))>>2]|0;
 $7 = ($6|0)!=(0);
 if (!($7)) {
  $8 = (_sqlite3MallocInit()|0);
  HEAP32[$rc>>2] = $8;
 }
 $9 = HEAP32[$rc>>2]|0;
 $10 = ($9|0)==(0);
 if ($10) {
  HEAP32[((38640 + 236|0))>>2] = 1;
  $11 = HEAP32[((38640 + 248|0))>>2]|0;
  $12 = ($11|0)!=(0|0);
  if (!($12)) {
   HEAP32[((38640 + 248|0))>>2] = (8);
   $13 = HEAP32[((38640 + 4|0))>>2]|0;
   $14 = ($13|0)!=(0);
   if ($14) {
    $15 = HEAP32[((38640 + 248|0))>>2]|0;
    $16 = ($15|0)!=(0|0);
    if (!($16)) {
     HEAP32[$rc>>2] = 7;
    }
   }
  }
 }
 $17 = HEAP32[$rc>>2]|0;
 $18 = ($17|0)==(0);
 if ($18) {
  $19 = HEAP32[((38640 + 244|0))>>2]|0;
  $20 = (($19) + 1)|0;
  HEAP32[((38640 + 244|0))>>2] = $20;
 }
 $21 = HEAP32[$rc>>2]|0;
 $22 = ($21|0)!=(0);
 if ($22) {
  $23 = HEAP32[$rc>>2]|0;
  HEAP32[$0>>2] = $23;
  $45 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($45|0);
 }
 $24 = HEAP32[((38640 + 224|0))>>2]|0;
 $25 = ($24|0)==(0);
 if ($25) {
  $26 = HEAP32[((38640 + 228|0))>>2]|0;
  $27 = ($26|0)==(0);
  if ($27) {
   HEAP32[$pHash>>2] = 1576;
   HEAP32[((38640 + 228|0))>>2] = 1;
   $28 = HEAP32[$pHash>>2]|0;
   dest=$28+0|0; stop=dest+92|0; do { HEAP32[dest>>2]=0|0; dest=dest+4|0; } while ((dest|0) < (stop|0));
   _sqlite3RegisterGlobalFunctions();
   $29 = HEAP32[((38640 + 240|0))>>2]|0;
   $30 = ($29|0)==(0);
   if ($30) {
    $31 = (_sqlite3PcacheInitialize()|0);
    HEAP32[$rc>>2] = $31;
   }
   $32 = HEAP32[$rc>>2]|0;
   $33 = ($32|0)==(0);
   if ($33) {
    HEAP32[((38640 + 240|0))>>2] = 1;
    $34 = (_sqlite3OsInit()|0);
    HEAP32[$rc>>2] = $34;
   }
   $35 = HEAP32[$rc>>2]|0;
   $36 = ($35|0)==(0);
   if ($36) {
    $37 = HEAP32[((38640 + 204|0))>>2]|0;
    $38 = HEAP32[((38640 + 208|0))>>2]|0;
    $39 = HEAP32[((38640 + 212|0))>>2]|0;
    _sqlite3PCacheBufferSetup($37,$38,$39);
    HEAP32[((38640 + 224|0))>>2] = 1;
   }
   HEAP32[((38640 + 228|0))>>2] = 0;
  }
 }
 $40 = HEAP32[((38640 + 244|0))>>2]|0;
 $41 = (($40) + -1)|0;
 HEAP32[((38640 + 244|0))>>2] = $41;
 $42 = HEAP32[((38640 + 244|0))>>2]|0;
 $43 = ($42|0)<=(0);
 if ($43) {
  HEAP32[((38640 + 248|0))>>2] = 0;
 }
 $44 = HEAP32[$rc>>2]|0;
 HEAP32[$0>>2] = $44;
 $45 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($45|0);
}
function _sqlite3_vfs_register($pVfs,$makeDflt) {
 $pVfs = $pVfs|0;
 $makeDflt = $makeDflt|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $3 = 0, $4 = 0, $5 = 0;
 var $6 = 0, $7 = 0, $8 = 0, $9 = 0, $or$cond = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $1 = sp + 8|0;
 $2 = sp + 4|0;
 $rc = sp;
 HEAP32[$1>>2] = $pVfs;
 HEAP32[$2>>2] = $makeDflt;
 $3 = (_sqlite3_initialize()|0);
 HEAP32[$rc>>2] = $3;
 $4 = HEAP32[$rc>>2]|0;
 $5 = ($4|0)!=(0);
 if ($5) {
  $6 = HEAP32[$rc>>2]|0;
  HEAP32[$0>>2] = $6;
  $23 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($23|0);
 }
 $7 = HEAP32[$1>>2]|0;
 _vfsUnlink($7);
 $8 = HEAP32[$2>>2]|0;
 $9 = ($8|0)!=(0);
 $10 = HEAP32[128>>2]|0;
 $11 = ($10|0)==(0|0);
 $or$cond = $9 | $11;
 $12 = HEAP32[128>>2]|0;
 if ($or$cond) {
  $13 = HEAP32[$1>>2]|0;
  $14 = (($13) + 12|0);
  HEAP32[$14>>2] = $12;
  $15 = HEAP32[$1>>2]|0;
  HEAP32[128>>2] = $15;
 } else {
  $16 = (($12) + 12|0);
  $17 = HEAP32[$16>>2]|0;
  $18 = HEAP32[$1>>2]|0;
  $19 = (($18) + 12|0);
  HEAP32[$19>>2] = $17;
  $20 = HEAP32[$1>>2]|0;
  $21 = HEAP32[128>>2]|0;
  $22 = (($21) + 12|0);
  HEAP32[$22>>2] = $20;
 }
 HEAP32[$0>>2] = 0;
 $23 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($23|0);
}
function _vfsUnlink($pVfs) {
 $pVfs = $pVfs|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $p = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $p = sp;
 HEAP32[$0>>2] = $pVfs;
 $1 = HEAP32[$0>>2]|0;
 $2 = ($1|0)==(0|0);
 if ($2) {
  STACKTOP = sp;return;
 }
 $3 = HEAP32[128>>2]|0;
 $4 = HEAP32[$0>>2]|0;
 $5 = ($3|0)==($4|0);
 if ($5) {
  $6 = HEAP32[$0>>2]|0;
  $7 = (($6) + 12|0);
  $8 = HEAP32[$7>>2]|0;
  HEAP32[128>>2] = $8;
  STACKTOP = sp;return;
 }
 $9 = HEAP32[128>>2]|0;
 $10 = ($9|0)!=(0|0);
 if (!($10)) {
  STACKTOP = sp;return;
 }
 $11 = HEAP32[128>>2]|0;
 HEAP32[$p>>2] = $11;
 while(1) {
  $12 = HEAP32[$p>>2]|0;
  $13 = (($12) + 12|0);
  $14 = HEAP32[$13>>2]|0;
  $15 = ($14|0)!=(0|0);
  if ($15) {
   $16 = HEAP32[$p>>2]|0;
   $17 = (($16) + 12|0);
   $18 = HEAP32[$17>>2]|0;
   $19 = HEAP32[$0>>2]|0;
   $20 = ($18|0)!=($19|0);
   $31 = $20;
  } else {
   $31 = 0;
  }
  $21 = HEAP32[$p>>2]|0;
  $22 = (($21) + 12|0);
  $23 = HEAP32[$22>>2]|0;
  if (!($31)) {
   break;
  }
  HEAP32[$p>>2] = $23;
 }
 $24 = HEAP32[$0>>2]|0;
 $25 = ($23|0)==($24|0);
 if (!($25)) {
  STACKTOP = sp;return;
 }
 $26 = HEAP32[$0>>2]|0;
 $27 = (($26) + 12|0);
 $28 = HEAP32[$27>>2]|0;
 $29 = HEAP32[$p>>2]|0;
 $30 = (($29) + 12|0);
 HEAP32[$30>>2] = $28;
 STACKTOP = sp;return;
}
function _sqlite3_release_memory($n) {
 $n = $n|0;
 var $0 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 HEAP32[$0>>2] = $n;
 STACKTOP = sp;return 0;
}
function _sqlite3MemoryAlarm($xCallback,$pArg,$0,$1) {
 $xCallback = $xCallback|0;
 $pArg = $pArg|0;
 $0 = $0|0;
 $1 = $1|0;
 var $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0;
 var $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0;
 var $47 = 0, $48 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $nUsed = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $2 = sp + 16|0;
 $3 = sp + 12|0;
 $4 = sp;
 $nUsed = sp + 8|0;
 HEAP32[$2>>2] = $xCallback;
 HEAP32[$3>>2] = $pArg;
 $5 = $4;
 $6 = $5;
 HEAP32[$6>>2] = $0;
 $7 = (($5) + 4)|0;
 $8 = $7;
 HEAP32[$8>>2] = $1;
 $9 = HEAP32[$2>>2]|0;
 HEAP32[((136 + 16|0))>>2] = $9;
 $10 = HEAP32[$3>>2]|0;
 HEAP32[((136 + 20|0))>>2] = $10;
 $11 = $4;
 $12 = $11;
 $13 = HEAP32[$12>>2]|0;
 $14 = (($11) + 4)|0;
 $15 = $14;
 $16 = HEAP32[$15>>2]|0;
 $17 = ((136 + 8|0));
 $18 = $17;
 HEAP32[$18>>2] = $13;
 $19 = (($17) + 4)|0;
 $20 = $19;
 HEAP32[$20>>2] = $16;
 $21 = (_sqlite3StatusValue(0)|0);
 HEAP32[$nUsed>>2] = $21;
 $22 = $4;
 $23 = $22;
 $24 = HEAP32[$23>>2]|0;
 $25 = (($22) + 4)|0;
 $26 = $25;
 $27 = HEAP32[$26>>2]|0;
 $28 = ($27|0)>(0);
 $29 = ($24>>>0)>(0);
 $30 = ($27|0)==(0);
 $31 = $30 & $29;
 $32 = $28 | $31;
 if (!($32)) {
  $48 = 0;
  $47 = $48&1;
  HEAP32[((136 + 36|0))>>2] = $47;
  STACKTOP = sp;return 0;
 }
 $33 = $4;
 $34 = $33;
 $35 = HEAP32[$34>>2]|0;
 $36 = (($33) + 4)|0;
 $37 = $36;
 $38 = HEAP32[$37>>2]|0;
 $39 = HEAP32[$nUsed>>2]|0;
 $40 = ($39|0)<(0);
 $41 = $40 << 31 >> 31;
 $42 = ($38|0)<($41|0);
 $43 = ($35>>>0)<=($39>>>0);
 $44 = ($38|0)==($41|0);
 $45 = $44 & $43;
 $46 = $42 | $45;
 $48 = $46;
 $47 = $48&1;
 HEAP32[((136 + 36|0))>>2] = $47;
 STACKTOP = sp;return 0;
}
function _sqlite3_soft_heap_limit64($0,$1) {
 $0 = $0|0;
 $1 = $1|0;
 var $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0;
 var $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0;
 var $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0;
 var $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0;
 var $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0;
 var $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $excess = 0, $priorLimit = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $2 = sp + 24|0;
 $3 = sp + 16|0;
 $priorLimit = sp + 8|0;
 $excess = sp;
 $rc = sp + 32|0;
 $4 = $3;
 $5 = $4;
 HEAP32[$5>>2] = $0;
 $6 = (($4) + 4)|0;
 $7 = $6;
 HEAP32[$7>>2] = $1;
 $8 = (_sqlite3_initialize()|0);
 HEAP32[$rc>>2] = $8;
 $9 = HEAP32[$rc>>2]|0;
 $10 = ($9|0)!=(0);
 if ($10) {
  $11 = $2;
  $12 = $11;
  HEAP32[$12>>2] = -1;
  $13 = (($11) + 4)|0;
  $14 = $13;
  HEAP32[$14>>2] = -1;
  $101 = $2;
  $102 = $101;
  $103 = HEAP32[$102>>2]|0;
  $104 = (($101) + 4)|0;
  $105 = $104;
  $106 = HEAP32[$105>>2]|0;
  tempRet0 = $106;
  STACKTOP = sp;return ($103|0);
 }
 $15 = ((136 + 8|0));
 $16 = $15;
 $17 = HEAP32[$16>>2]|0;
 $18 = (($15) + 4)|0;
 $19 = $18;
 $20 = HEAP32[$19>>2]|0;
 $21 = $priorLimit;
 $22 = $21;
 HEAP32[$22>>2] = $17;
 $23 = (($21) + 4)|0;
 $24 = $23;
 HEAP32[$24>>2] = $20;
 $25 = $3;
 $26 = $25;
 $27 = HEAP32[$26>>2]|0;
 $28 = (($25) + 4)|0;
 $29 = $28;
 $30 = HEAP32[$29>>2]|0;
 $31 = ($30|0)<(0);
 if ($31) {
  $32 = $priorLimit;
  $33 = $32;
  $34 = HEAP32[$33>>2]|0;
  $35 = (($32) + 4)|0;
  $36 = $35;
  $37 = HEAP32[$36>>2]|0;
  $38 = $2;
  $39 = $38;
  HEAP32[$39>>2] = $34;
  $40 = (($38) + 4)|0;
  $41 = $40;
  HEAP32[$41>>2] = $37;
  $101 = $2;
  $102 = $101;
  $103 = HEAP32[$102>>2]|0;
  $104 = (($101) + 4)|0;
  $105 = $104;
  $106 = HEAP32[$105>>2]|0;
  tempRet0 = $106;
  STACKTOP = sp;return ($103|0);
 }
 $42 = $3;
 $43 = $42;
 $44 = HEAP32[$43>>2]|0;
 $45 = (($42) + 4)|0;
 $46 = $45;
 $47 = HEAP32[$46>>2]|0;
 $48 = ($47|0)>(0);
 $49 = ($44>>>0)>(0);
 $50 = ($47|0)==(0);
 $51 = $50 & $49;
 $52 = $48 | $51;
 if ($52) {
  $53 = $3;
  $54 = $53;
  $55 = HEAP32[$54>>2]|0;
  $56 = (($53) + 4)|0;
  $57 = $56;
  $58 = HEAP32[$57>>2]|0;
  (_sqlite3MemoryAlarm(2,0,$55,$58)|0);
 } else {
  (_sqlite3MemoryAlarm(0,0,0,0)|0);
 }
 $59 = (_sqlite3_memory_used()|0);
 $60 = tempRet0;
 $61 = $3;
 $62 = $61;
 $63 = HEAP32[$62>>2]|0;
 $64 = (($61) + 4)|0;
 $65 = $64;
 $66 = HEAP32[$65>>2]|0;
 $67 = (_i64Subtract(($59|0),($60|0),($63|0),($66|0))|0);
 $68 = tempRet0;
 $69 = $excess;
 $70 = $69;
 HEAP32[$70>>2] = $67;
 $71 = (($69) + 4)|0;
 $72 = $71;
 HEAP32[$72>>2] = $68;
 $73 = $excess;
 $74 = $73;
 $75 = HEAP32[$74>>2]|0;
 $76 = (($73) + 4)|0;
 $77 = $76;
 $78 = HEAP32[$77>>2]|0;
 $79 = ($78|0)>(0);
 $80 = ($75>>>0)>(0);
 $81 = ($78|0)==(0);
 $82 = $81 & $80;
 $83 = $79 | $82;
 if ($83) {
  $84 = $excess;
  $85 = $84;
  $86 = HEAP32[$85>>2]|0;
  $87 = (($84) + 4)|0;
  $88 = $87;
  $89 = HEAP32[$88>>2]|0;
  $90 = $86 & 2147483647;
  (_sqlite3_release_memory($90)|0);
 }
 $91 = $priorLimit;
 $92 = $91;
 $93 = HEAP32[$92>>2]|0;
 $94 = (($91) + 4)|0;
 $95 = $94;
 $96 = HEAP32[$95>>2]|0;
 $97 = $2;
 $98 = $97;
 HEAP32[$98>>2] = $93;
 $99 = (($97) + 4)|0;
 $100 = $99;
 HEAP32[$100>>2] = $96;
 $101 = $2;
 $102 = $101;
 $103 = HEAP32[$102>>2]|0;
 $104 = (($101) + 4)|0;
 $105 = $104;
 $106 = HEAP32[$105>>2]|0;
 tempRet0 = $106;
 STACKTOP = sp;return ($103|0);
}
function _softHeapLimitEnforcer($NotUsed,$0,$1,$allocSize) {
 $NotUsed = $NotUsed|0;
 $0 = $0|0;
 $1 = $1|0;
 $allocSize = $allocSize|0;
 var $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $2 = sp + 12|0;
 $3 = sp;
 $4 = sp + 8|0;
 HEAP32[$2>>2] = $NotUsed;
 $5 = $3;
 $6 = $5;
 HEAP32[$6>>2] = $0;
 $7 = (($5) + 4)|0;
 $8 = $7;
 HEAP32[$8>>2] = $1;
 HEAP32[$4>>2] = $allocSize;
 $9 = HEAP32[$4>>2]|0;
 (_sqlite3_release_memory($9)|0);
 STACKTOP = sp;return;
}
function _sqlite3_memory_used() {
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $mx = 0, $n = 0, $res = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $n = sp + 12|0;
 $mx = sp + 8|0;
 $res = sp;
 (_sqlite3_status(0,$n,$mx,0)|0);
 $0 = HEAP32[$n>>2]|0;
 $1 = ($0|0)<(0);
 $2 = $1 << 31 >> 31;
 $3 = $res;
 $4 = $3;
 HEAP32[$4>>2] = $0;
 $5 = (($3) + 4)|0;
 $6 = $5;
 HEAP32[$6>>2] = $2;
 $7 = $res;
 $8 = $7;
 $9 = HEAP32[$8>>2]|0;
 $10 = (($7) + 4)|0;
 $11 = $10;
 $12 = HEAP32[$11>>2]|0;
 tempRet0 = $12;
 STACKTOP = sp;return ($9|0);
}
function _sqlite3_malloc($n) {
 $n = $n|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $1 = sp;
 HEAP32[$1>>2] = $n;
 $2 = (_sqlite3_initialize()|0);
 $3 = ($2|0)!=(0);
 if ($3) {
  HEAP32[$0>>2] = 0;
  $11 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($11|0);
 }
 $4 = HEAP32[$1>>2]|0;
 $5 = ($4|0)<=(0);
 if ($5) {
  $10 = 0;
 } else {
  $6 = HEAP32[$1>>2]|0;
  $7 = ($6|0)<(0);
  $8 = $7 << 31 >> 31;
  $9 = (_sqlite3Malloc($6,$8)|0);
  $10 = $9;
 }
 HEAP32[$0>>2] = $10;
 $11 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($11|0);
}
function _sqlite3Malloc($0,$1) {
 $0 = $0|0;
 $1 = $1|0;
 var $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0;
 var $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $5 = 0, $6 = 0, $7 = 0;
 var $8 = 0, $9 = 0, $or$cond = 0, $p = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $2 = sp;
 $p = sp + 8|0;
 $3 = $2;
 $4 = $3;
 HEAP32[$4>>2] = $0;
 $5 = (($3) + 4)|0;
 $6 = $5;
 HEAP32[$6>>2] = $1;
 $7 = $2;
 $8 = $7;
 $9 = HEAP32[$8>>2]|0;
 $10 = (($7) + 4)|0;
 $11 = $10;
 $12 = HEAP32[$11>>2]|0;
 $13 = ($9|0)==(0);
 $14 = ($12|0)==(0);
 $15 = $13 & $14;
 $16 = $2;
 $17 = $16;
 $18 = HEAP32[$17>>2]|0;
 $19 = (($16) + 4)|0;
 $20 = $19;
 $21 = HEAP32[$20>>2]|0;
 $22 = ($21>>>0)>(0);
 $23 = ($18>>>0)>=(2147483392);
 $24 = ($21|0)==(0);
 $25 = $24 & $23;
 $26 = $22 | $25;
 $or$cond = $15 | $26;
 if ($or$cond) {
  HEAP32[$p>>2] = 0;
  $43 = HEAP32[$p>>2]|0;
  STACKTOP = sp;return ($43|0);
 }
 $27 = HEAP32[38640>>2]|0;
 $28 = ($27|0)!=(0);
 if ($28) {
  $29 = $2;
  $30 = $29;
  $31 = HEAP32[$30>>2]|0;
  $32 = (($29) + 4)|0;
  $33 = $32;
  $34 = HEAP32[$33>>2]|0;
  (_mallocWithAlarm($31,$p)|0);
  $43 = HEAP32[$p>>2]|0;
  STACKTOP = sp;return ($43|0);
 } else {
  $35 = HEAP32[((38640 + 36|0))>>2]|0;
  $36 = $2;
  $37 = $36;
  $38 = HEAP32[$37>>2]|0;
  $39 = (($36) + 4)|0;
  $40 = $39;
  $41 = HEAP32[$40>>2]|0;
  $42 = (FUNCTION_TABLE_ii[$35 & 31]($38)|0);
  HEAP32[$p>>2] = $42;
  $43 = HEAP32[$p>>2]|0;
  STACKTOP = sp;return ($43|0);
 }
 return 0|0;
}
function _sqlite3_free($p) {
 $p = $p|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 HEAP32[$0>>2] = $p;
 $1 = HEAP32[$0>>2]|0;
 $2 = ($1|0)==(0|0);
 if ($2) {
  STACKTOP = sp;return;
 }
 $3 = HEAP32[38640>>2]|0;
 $4 = ($3|0)!=(0);
 if ($4) {
  $5 = HEAP32[$0>>2]|0;
  $6 = (_sqlite3MallocSize($5)|0);
  $7 = (0 - ($6))|0;
  _sqlite3StatusAdd(0,$7);
  _sqlite3StatusAdd(9,-1);
  $8 = HEAP32[((38640 + 40|0))>>2]|0;
  $9 = HEAP32[$0>>2]|0;
  FUNCTION_TABLE_vi[$8 & 31]($9);
  STACKTOP = sp;return;
 } else {
  $10 = HEAP32[((38640 + 40|0))>>2]|0;
  $11 = HEAP32[$0>>2]|0;
  FUNCTION_TABLE_vi[$10 & 31]($11);
  STACKTOP = sp;return;
 }
}
function _sqlite3StatusAdd($op,$N) {
 $op = $op|0;
 $N = $N|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0;
 var sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $1 = sp;
 HEAP32[$0>>2] = $op;
 HEAP32[$1>>2] = $N;
 $2 = HEAP32[$1>>2]|0;
 $3 = HEAP32[$0>>2]|0;
 $4 = (48 + ($3<<2)|0);
 $5 = HEAP32[$4>>2]|0;
 $6 = (($5) + ($2))|0;
 HEAP32[$4>>2] = $6;
 $7 = HEAP32[$0>>2]|0;
 $8 = (48 + ($7<<2)|0);
 $9 = HEAP32[$8>>2]|0;
 $10 = HEAP32[$0>>2]|0;
 $11 = (((48 + 40|0)) + ($10<<2)|0);
 $12 = HEAP32[$11>>2]|0;
 $13 = ($9|0)>($12|0);
 if (!($13)) {
  STACKTOP = sp;return;
 }
 $14 = HEAP32[$0>>2]|0;
 $15 = (48 + ($14<<2)|0);
 $16 = HEAP32[$15>>2]|0;
 $17 = HEAP32[$0>>2]|0;
 $18 = (((48 + 40|0)) + ($17<<2)|0);
 HEAP32[$18>>2] = $16;
 STACKTOP = sp;return;
}
function _sqlite3_realloc($pOld,$n) {
 $pOld = $pOld|0;
 $n = $n|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $1 = sp + 4|0;
 $2 = sp;
 HEAP32[$1>>2] = $pOld;
 HEAP32[$2>>2] = $n;
 $3 = (_sqlite3_initialize()|0);
 $4 = ($3|0)!=(0);
 if ($4) {
  HEAP32[$0>>2] = 0;
  $12 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($12|0);
 }
 $5 = HEAP32[$2>>2]|0;
 $6 = ($5|0)<(0);
 if ($6) {
  HEAP32[$2>>2] = 0;
 }
 $7 = HEAP32[$1>>2]|0;
 $8 = HEAP32[$2>>2]|0;
 $9 = ($8|0)<(0);
 $10 = $9 << 31 >> 31;
 $11 = (_sqlite3Realloc($7,$8,$10)|0);
 HEAP32[$0>>2] = $11;
 $12 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($12|0);
}
function _sqlite3Realloc($pOld,$0,$1) {
 $pOld = $pOld|0;
 $0 = $0|0;
 $1 = $1|0;
 var $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0;
 var $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0;
 var $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0;
 var $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0;
 var $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0;
 var $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $nDiff = 0, $nNew = 0, $nOld = 0, $or$cond = 0, $pNew = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $2 = sp + 28|0;
 $3 = sp + 24|0;
 $4 = sp;
 $nOld = sp + 20|0;
 $nNew = sp + 16|0;
 $nDiff = sp + 12|0;
 $pNew = sp + 8|0;
 HEAP32[$3>>2] = $pOld;
 $5 = $4;
 $6 = $5;
 HEAP32[$6>>2] = $0;
 $7 = (($5) + 4)|0;
 $8 = $7;
 HEAP32[$8>>2] = $1;
 $9 = HEAP32[$3>>2]|0;
 $10 = ($9|0)==(0|0);
 $11 = $4;
 $12 = $11;
 $13 = HEAP32[$12>>2]|0;
 $14 = (($11) + 4)|0;
 $15 = $14;
 $16 = HEAP32[$15>>2]|0;
 if ($10) {
  $17 = (_sqlite3Malloc($13,$16)|0);
  HEAP32[$2>>2] = $17;
  $108 = HEAP32[$2>>2]|0;
  STACKTOP = sp;return ($108|0);
 }
 $18 = ($13|0)==(0);
 $19 = ($16|0)==(0);
 $20 = $18 & $19;
 if ($20) {
  $21 = HEAP32[$3>>2]|0;
  _sqlite3_free($21);
  HEAP32[$2>>2] = 0;
  $108 = HEAP32[$2>>2]|0;
  STACKTOP = sp;return ($108|0);
 }
 $22 = $4;
 $23 = $22;
 $24 = HEAP32[$23>>2]|0;
 $25 = (($22) + 4)|0;
 $26 = $25;
 $27 = HEAP32[$26>>2]|0;
 $28 = ($27>>>0)>(0);
 $29 = ($24>>>0)>=(2147483392);
 $30 = ($27|0)==(0);
 $31 = $30 & $29;
 $32 = $28 | $31;
 if ($32) {
  HEAP32[$2>>2] = 0;
  $108 = HEAP32[$2>>2]|0;
  STACKTOP = sp;return ($108|0);
 }
 $33 = HEAP32[$3>>2]|0;
 $34 = (_sqlite3MallocSize($33)|0);
 HEAP32[$nOld>>2] = $34;
 $35 = HEAP32[((38640 + 52|0))>>2]|0;
 $36 = $4;
 $37 = $36;
 $38 = HEAP32[$37>>2]|0;
 $39 = (($36) + 4)|0;
 $40 = $39;
 $41 = HEAP32[$40>>2]|0;
 $42 = (FUNCTION_TABLE_ii[$35 & 31]($38)|0);
 HEAP32[$nNew>>2] = $42;
 $43 = HEAP32[$nOld>>2]|0;
 $44 = HEAP32[$nNew>>2]|0;
 $45 = ($43|0)==($44|0);
 do {
  if ($45) {
   $46 = HEAP32[$3>>2]|0;
   HEAP32[$pNew>>2] = $46;
  } else {
   $47 = HEAP32[38640>>2]|0;
   $48 = ($47|0)!=(0);
   if (!($48)) {
    $103 = HEAP32[((38640 + 44|0))>>2]|0;
    $104 = HEAP32[$3>>2]|0;
    $105 = HEAP32[$nNew>>2]|0;
    $106 = (FUNCTION_TABLE_iii[$103 & 63]($104,$105)|0);
    HEAP32[$pNew>>2] = $106;
    break;
   }
   $49 = $4;
   $50 = $49;
   $51 = HEAP32[$50>>2]|0;
   $52 = (($49) + 4)|0;
   $53 = $52;
   $54 = HEAP32[$53>>2]|0;
   _sqlite3StatusSet(5,$51);
   $55 = HEAP32[$nNew>>2]|0;
   $56 = HEAP32[$nOld>>2]|0;
   $57 = (($55) - ($56))|0;
   HEAP32[$nDiff>>2] = $57;
   $58 = (_sqlite3StatusValue(0)|0);
   $59 = ($58|0)<(0);
   $60 = $59 << 31 >> 31;
   $61 = ((136 + 8|0));
   $62 = $61;
   $63 = HEAP32[$62>>2]|0;
   $64 = (($61) + 4)|0;
   $65 = $64;
   $66 = HEAP32[$65>>2]|0;
   $67 = HEAP32[$nDiff>>2]|0;
   $68 = ($67|0)<(0);
   $69 = $68 << 31 >> 31;
   $70 = (_i64Subtract(($63|0),($66|0),($67|0),($69|0))|0);
   $71 = tempRet0;
   $72 = ($60|0)>($71|0);
   $73 = ($58>>>0)>=($70>>>0);
   $74 = ($60|0)==($71|0);
   $75 = $74 & $73;
   $76 = $72 | $75;
   if ($76) {
    $77 = HEAP32[$nDiff>>2]|0;
    _sqlite3MallocAlarm($77);
   }
   $78 = HEAP32[((38640 + 44|0))>>2]|0;
   $79 = HEAP32[$3>>2]|0;
   $80 = HEAP32[$nNew>>2]|0;
   $81 = (FUNCTION_TABLE_iii[$78 & 63]($79,$80)|0);
   HEAP32[$pNew>>2] = $81;
   $82 = HEAP32[$pNew>>2]|0;
   $83 = ($82|0)==(0|0);
   $84 = HEAP32[((136 + 16|0))>>2]|0;
   $85 = ($84|0)!=(0|0);
   $or$cond = $83 & $85;
   if ($or$cond) {
    $86 = $4;
    $87 = $86;
    $88 = HEAP32[$87>>2]|0;
    $89 = (($86) + 4)|0;
    $90 = $89;
    $91 = HEAP32[$90>>2]|0;
    _sqlite3MallocAlarm($88);
    $92 = HEAP32[((38640 + 44|0))>>2]|0;
    $93 = HEAP32[$3>>2]|0;
    $94 = HEAP32[$nNew>>2]|0;
    $95 = (FUNCTION_TABLE_iii[$92 & 63]($93,$94)|0);
    HEAP32[$pNew>>2] = $95;
   }
   $96 = HEAP32[$pNew>>2]|0;
   $97 = ($96|0)!=(0|0);
   if ($97) {
    $98 = HEAP32[$pNew>>2]|0;
    $99 = (_sqlite3MallocSize($98)|0);
    HEAP32[$nNew>>2] = $99;
    $100 = HEAP32[$nNew>>2]|0;
    $101 = HEAP32[$nOld>>2]|0;
    $102 = (($100) - ($101))|0;
    _sqlite3StatusAdd(0,$102);
   }
  }
 } while(0);
 $107 = HEAP32[$pNew>>2]|0;
 HEAP32[$2>>2] = $107;
 $108 = HEAP32[$2>>2]|0;
 STACKTOP = sp;return ($108|0);
}
function _sqlite3_realloc64($pOld,$0,$1) {
 $pOld = $pOld|0;
 $0 = $0|0;
 $1 = $1|0;
 var $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $2 = sp + 12|0;
 $3 = sp + 8|0;
 $4 = sp;
 HEAP32[$3>>2] = $pOld;
 $5 = $4;
 $6 = $5;
 HEAP32[$6>>2] = $0;
 $7 = (($5) + 4)|0;
 $8 = $7;
 HEAP32[$8>>2] = $1;
 $9 = (_sqlite3_initialize()|0);
 $10 = ($9|0)!=(0);
 if ($10) {
  HEAP32[$2>>2] = 0;
  $19 = HEAP32[$2>>2]|0;
  STACKTOP = sp;return ($19|0);
 } else {
  $11 = HEAP32[$3>>2]|0;
  $12 = $4;
  $13 = $12;
  $14 = HEAP32[$13>>2]|0;
  $15 = (($12) + 4)|0;
  $16 = $15;
  $17 = HEAP32[$16>>2]|0;
  $18 = (_sqlite3Realloc($11,$14,$17)|0);
  HEAP32[$2>>2] = $18;
  $19 = HEAP32[$2>>2]|0;
  STACKTOP = sp;return ($19|0);
 }
 return 0|0;
}
function _sqlite3_vmprintf($zFormat,$ap) {
 $zFormat = $zFormat|0;
 $ap = $ap|0;
 var $0 = 0, $1 = 0, $10 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $acc = 0, $z = 0, $zBase = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 128|0;
 $0 = sp + 40|0;
 $1 = sp + 36|0;
 $2 = sp + 32|0;
 $z = sp + 28|0;
 $zBase = sp + 44|0;
 $acc = sp;
 HEAP32[$1>>2] = $zFormat;
 HEAP32[$2>>2] = $ap;
 $3 = (_sqlite3_initialize()|0);
 $4 = ($3|0)!=(0);
 if ($4) {
  HEAP32[$0>>2] = 0;
  $10 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($10|0);
 } else {
  _sqlite3StrAccumInit($acc,$zBase,70,1000000000);
  $5 = (($acc) + 24|0);
  HEAP8[$5>>0] = 2;
  $6 = HEAP32[$1>>2]|0;
  $7 = HEAP32[$2>>2]|0;
  _sqlite3VXPrintf($acc,0,$6,$7);
  $8 = (_sqlite3StrAccumFinish($acc)|0);
  HEAP32[$z>>2] = $8;
  $9 = HEAP32[$z>>2]|0;
  HEAP32[$0>>2] = $9;
  $10 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($10|0);
 }
 return 0|0;
}
function _sqlite3StrAccumInit($p,$zBase,$n,$mx) {
 $p = $p|0;
 $zBase = $zBase|0;
 $n = $n|0;
 $mx = $mx|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0;
 var $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $1 = sp + 8|0;
 $2 = sp + 4|0;
 $3 = sp;
 HEAP32[$0>>2] = $p;
 HEAP32[$1>>2] = $zBase;
 HEAP32[$2>>2] = $n;
 HEAP32[$3>>2] = $mx;
 $4 = HEAP32[$1>>2]|0;
 $5 = HEAP32[$0>>2]|0;
 $6 = (($5) + 4|0);
 HEAP32[$6>>2] = $4;
 $7 = HEAP32[$0>>2]|0;
 $8 = (($7) + 8|0);
 HEAP32[$8>>2] = $4;
 $9 = HEAP32[$0>>2]|0;
 HEAP32[$9>>2] = 0;
 $10 = HEAP32[$0>>2]|0;
 $11 = (($10) + 12|0);
 HEAP32[$11>>2] = 0;
 $12 = HEAP32[$2>>2]|0;
 $13 = HEAP32[$0>>2]|0;
 $14 = (($13) + 16|0);
 HEAP32[$14>>2] = $12;
 $15 = HEAP32[$3>>2]|0;
 $16 = HEAP32[$0>>2]|0;
 $17 = (($16) + 20|0);
 HEAP32[$17>>2] = $15;
 $18 = HEAP32[$0>>2]|0;
 $19 = (($18) + 24|0);
 HEAP8[$19>>0] = 1;
 $20 = HEAP32[$0>>2]|0;
 $21 = (($20) + 25|0);
 HEAP8[$21>>0] = 0;
 STACKTOP = sp;return;
}
function _sqlite3VXPrintf($pAccum,$bFlags,$fmt,$ap) {
 $pAccum = $pAccum|0;
 $bFlags = $bFlags|0;
 $fmt = $fmt|0;
 $ap = $ap|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0;
 var $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0, $188 = 0;
 var $189 = 0, $19 = 0, $190 = 0, $191 = 0, $192 = 0, $193 = 0, $194 = 0, $195 = 0, $196 = 0, $197 = 0, $198 = 0, $199 = 0, $2 = 0, $20 = 0, $200 = 0, $201 = 0, $202 = 0, $203 = 0, $204 = 0, $205 = 0;
 var $206 = 0, $207 = 0, $208 = 0, $209 = 0, $21 = 0, $210 = 0, $211 = 0, $212 = 0, $213 = 0, $214 = 0, $215 = 0, $216 = 0, $217 = 0, $218 = 0, $219 = 0, $22 = 0, $220 = 0, $221 = 0, $222 = 0, $223 = 0;
 var $224 = 0, $225 = 0, $226 = 0, $227 = 0, $228 = 0, $229 = 0, $23 = 0, $230 = 0, $231 = 0, $232 = 0, $233 = 0, $234 = 0, $235 = 0, $236 = 0, $237 = 0, $238 = 0, $239 = 0, $24 = 0, $240 = 0, $241 = 0;
 var $242 = 0, $243 = 0, $244 = 0, $245 = 0, $246 = 0, $247 = 0, $248 = 0, $249 = 0, $25 = 0, $250 = 0, $251 = 0, $252 = 0, $253 = 0, $254 = 0, $255 = 0, $256 = 0, $257 = 0, $258 = 0, $259 = 0, $26 = 0;
 var $260 = 0, $261 = 0, $262 = 0, $263 = 0, $264 = 0, $265 = 0, $266 = 0, $267 = 0, $268 = 0, $269 = 0, $27 = 0, $270 = 0, $271 = 0, $272 = 0, $273 = 0, $274 = 0, $275 = 0, $276 = 0, $277 = 0, $278 = 0;
 var $279 = 0, $28 = 0, $280 = 0, $281 = 0, $282 = 0, $283 = 0, $284 = 0, $285 = 0, $286 = 0, $287 = 0, $288 = 0, $289 = 0, $29 = 0, $290 = 0, $291 = 0, $292 = 0, $293 = 0, $294 = 0, $295 = 0, $296 = 0;
 var $297 = 0, $298 = 0, $299 = 0, $3 = 0, $30 = 0, $300 = 0, $301 = 0, $302 = 0, $303 = 0, $304 = 0, $305 = 0, $306 = 0, $307 = 0, $308 = 0, $309 = 0, $31 = 0, $310 = 0, $311 = 0, $312 = 0, $313 = 0;
 var $314 = 0, $315 = 0, $316 = 0, $317 = 0, $318 = 0, $319 = 0, $32 = 0, $320 = 0, $321 = 0, $322 = 0, $323 = 0, $324 = 0, $325 = 0, $326 = 0, $327 = 0, $328 = 0, $329 = 0, $33 = 0, $330 = 0, $331 = 0;
 var $332 = 0, $333 = 0, $334 = 0, $335 = 0, $336 = 0, $337 = 0, $338 = 0, $339 = 0, $34 = 0, $340 = 0, $341 = 0, $342 = 0, $343 = 0, $344 = 0, $345 = 0, $346 = 0, $347 = 0, $348 = 0, $349 = 0, $35 = 0;
 var $350 = 0, $351 = 0, $352 = 0, $353 = 0, $354 = 0, $355 = 0, $356 = 0, $357 = 0, $358 = 0, $359 = 0, $36 = 0, $360 = 0, $361 = 0, $362 = 0, $363 = 0, $364 = 0, $365 = 0, $366 = 0, $367 = 0, $368 = 0;
 var $369 = 0, $37 = 0, $370 = 0, $371 = 0, $372 = 0, $373 = 0, $374 = 0, $375 = 0, $376 = 0, $377 = 0, $378 = 0, $379 = 0, $38 = 0, $380 = 0, $381 = 0, $382 = 0, $383 = 0, $384 = 0, $385 = 0, $386 = 0;
 var $387 = 0, $388 = 0, $389 = 0, $39 = 0, $390 = 0, $391 = 0, $392 = 0, $393 = 0, $394 = 0, $395 = 0, $396 = 0, $397 = 0, $398 = 0, $399 = 0, $4 = 0, $40 = 0, $400 = 0, $401 = 0, $402 = 0, $403 = 0;
 var $404 = 0, $405 = 0, $406 = 0, $407 = 0, $408 = 0, $409 = 0, $41 = 0, $410 = 0, $411 = 0, $412 = 0, $413 = 0, $414 = 0, $415 = 0, $416 = 0, $417 = 0, $418 = 0, $419 = 0, $42 = 0, $420 = 0, $421 = 0;
 var $422 = 0, $423 = 0, $424 = 0, $425 = 0, $426 = 0, $427 = 0, $428 = 0, $429 = 0, $43 = 0, $430 = 0, $431 = 0, $432 = 0, $433 = 0, $434 = 0, $435 = 0, $436 = 0, $437 = 0, $438 = 0, $439 = 0, $44 = 0;
 var $440 = 0, $441 = 0, $442 = 0, $443 = 0, $444 = 0.0, $445 = 0, $446 = 0.0, $447 = 0, $448 = 0, $449 = 0.0, $45 = 0, $450 = 0, $451 = 0.0, $452 = 0.0, $453 = 0, $454 = 0, $455 = 0, $456 = 0, $457 = 0, $458 = 0;
 var $459 = 0, $46 = 0, $460 = 0, $461 = 0, $462 = 0, $463 = 0, $464 = 0, $465 = 0, $466 = 0, $467 = 0, $468 = 0, $469 = 0.0, $47 = 0, $470 = 0.0, $471 = 0, $472 = 0, $473 = 0, $474 = 0.0, $475 = 0.0, $476 = 0.0;
 var $477 = 0.0, $478 = 0, $479 = 0, $48 = 0, $480 = 0.0, $481 = 0, $482 = 0.0, $483 = 0.0, $484 = 0.0, $485 = 0, $486 = 0, $487 = 0, $488 = 0.0, $489 = 0.0, $49 = 0, $490 = 0, $491 = 0, $492 = 0.0, $493 = 0.0, $494 = 0.0;
 var $495 = 0, $496 = 0, $497 = 0, $498 = 0.0, $499 = 0.0, $5 = 0, $50 = 0, $500 = 0, $501 = 0, $502 = 0.0, $503 = 0.0, $504 = 0.0, $505 = 0, $506 = 0, $507 = 0, $508 = 0.0, $509 = 0.0, $51 = 0, $510 = 0, $511 = 0;
 var $512 = 0.0, $513 = 0.0, $514 = 0.0, $515 = 0, $516 = 0, $517 = 0, $518 = 0.0, $519 = 0.0, $52 = 0, $520 = 0, $521 = 0, $522 = 0.0, $523 = 0.0, $524 = 0.0, $525 = 0, $526 = 0.0, $527 = 0.0, $528 = 0, $529 = 0, $53 = 0;
 var $530 = 0.0, $531 = 0, $532 = 0.0, $533 = 0.0, $534 = 0, $535 = 0, $536 = 0, $537 = 0, $538 = 0, $539 = 0, $54 = 0, $540 = 0, $541 = 0, $542 = 0, $543 = 0, $544 = 0, $545 = 0, $546 = 0, $547 = 0, $548 = 0;
 var $549 = 0.0, $55 = 0, $550 = 0.0, $551 = 0.0, $552 = 0.0, $553 = 0, $554 = 0.0, $555 = 0.0, $556 = 0, $557 = 0, $558 = 0, $559 = 0, $56 = 0, $560 = 0, $561 = 0, $562 = 0, $563 = 0, $564 = 0, $565 = 0, $566 = 0;
 var $567 = 0, $568 = 0, $569 = 0, $57 = 0, $570 = 0, $571 = 0, $572 = 0, $573 = 0, $574 = 0, $575 = 0, $576 = 0, $577 = 0, $578 = 0, $579 = 0, $58 = 0, $580 = 0, $581 = 0, $582 = 0, $583 = 0, $584 = 0;
 var $585 = 0, $586 = 0, $587 = 0, $588 = 0, $589 = 0, $59 = 0, $590 = 0, $591 = 0, $592 = 0, $593 = 0, $594 = 0, $595 = 0, $596 = 0, $597 = 0, $598 = 0, $599 = 0, $6 = 0, $60 = 0, $600 = 0, $601 = 0;
 var $602 = 0, $603 = 0, $604 = 0, $605 = 0, $606 = 0, $607 = 0, $608 = 0, $609 = 0, $61 = 0, $610 = 0, $611 = 0, $612 = 0, $613 = 0, $614 = 0, $615 = 0, $616 = 0, $617 = 0, $618 = 0, $619 = 0, $62 = 0;
 var $620 = 0, $621 = 0, $622 = 0, $623 = 0, $624 = 0, $625 = 0, $626 = 0, $627 = 0, $628 = 0, $629 = 0, $63 = 0, $630 = 0, $631 = 0, $632 = 0, $633 = 0, $634 = 0, $635 = 0, $636 = 0, $637 = 0, $638 = 0;
 var $639 = 0, $64 = 0, $640 = 0, $641 = 0, $642 = 0, $643 = 0, $644 = 0, $645 = 0, $646 = 0, $647 = 0, $648 = 0, $649 = 0, $65 = 0, $650 = 0, $651 = 0, $652 = 0, $653 = 0, $654 = 0, $655 = 0, $656 = 0;
 var $657 = 0, $658 = 0, $659 = 0, $66 = 0, $660 = 0, $661 = 0, $662 = 0, $663 = 0, $664 = 0, $665 = 0, $666 = 0, $667 = 0, $668 = 0, $669 = 0, $67 = 0, $670 = 0, $671 = 0, $672 = 0, $673 = 0, $674 = 0;
 var $675 = 0, $676 = 0, $677 = 0, $678 = 0, $679 = 0, $68 = 0, $680 = 0, $681 = 0, $682 = 0, $683 = 0, $684 = 0, $685 = 0, $686 = 0, $687 = 0, $688 = 0, $689 = 0, $69 = 0, $690 = 0, $691 = 0, $692 = 0;
 var $693 = 0, $694 = 0, $695 = 0, $696 = 0, $697 = 0, $698 = 0, $699 = 0, $7 = 0, $70 = 0, $700 = 0, $701 = 0, $702 = 0, $703 = 0, $704 = 0, $705 = 0, $706 = 0, $707 = 0, $708 = 0, $709 = 0, $71 = 0;
 var $710 = 0, $711 = 0, $712 = 0, $713 = 0, $714 = 0, $715 = 0, $716 = 0, $717 = 0, $718 = 0, $719 = 0, $72 = 0, $720 = 0, $721 = 0, $722 = 0, $723 = 0, $724 = 0, $725 = 0, $726 = 0, $727 = 0, $728 = 0;
 var $729 = 0, $73 = 0, $730 = 0, $731 = 0, $732 = 0, $733 = 0, $734 = 0, $735 = 0, $736 = 0, $737 = 0, $738 = 0, $739 = 0, $74 = 0, $740 = 0, $741 = 0, $742 = 0, $743 = 0, $744 = 0, $745 = 0, $746 = 0;
 var $747 = 0, $748 = 0, $749 = 0, $75 = 0, $750 = 0, $751 = 0, $752 = 0, $753 = 0, $754 = 0, $755 = 0, $756 = 0, $757 = 0, $758 = 0, $759 = 0, $76 = 0, $760 = 0, $761 = 0, $762 = 0, $763 = 0, $764 = 0;
 var $765 = 0, $766 = 0, $767 = 0, $768 = 0, $769 = 0, $77 = 0, $770 = 0, $771 = 0, $772 = 0, $773 = 0, $774 = 0, $775 = 0, $776 = 0, $777 = 0, $778 = 0, $779 = 0, $78 = 0, $780 = 0, $781 = 0, $782 = 0;
 var $783 = 0, $784 = 0, $785 = 0, $786 = 0, $787 = 0, $788 = 0, $789 = 0, $79 = 0, $790 = 0, $791 = 0, $792 = 0, $793 = 0, $794 = 0, $795 = 0, $796 = 0, $797 = 0, $798 = 0, $799 = 0, $8 = 0, $80 = 0;
 var $800 = 0, $801 = 0, $802 = 0, $803 = 0, $804 = 0, $805 = 0, $806 = 0, $807 = 0, $808 = 0, $809 = 0, $81 = 0, $810 = 0, $811 = 0, $812 = 0, $813 = 0, $814 = 0, $815 = 0, $816 = 0, $817 = 0, $818 = 0;
 var $819 = 0, $82 = 0, $820 = 0, $821 = 0, $822 = 0, $823 = 0, $824 = 0, $825 = 0, $826 = 0, $827 = 0, $828 = 0, $829 = 0, $83 = 0, $830 = 0, $831 = 0, $832 = 0, $833 = 0, $834 = 0, $835 = 0, $836 = 0;
 var $837 = 0, $838 = 0, $839 = 0, $84 = 0, $840 = 0, $841 = 0, $842 = 0, $843 = 0, $844 = 0, $845 = 0, $846 = 0, $847 = 0, $848 = 0, $849 = 0, $85 = 0, $850 = 0, $851 = 0, $852 = 0, $853 = 0, $854 = 0;
 var $855 = 0, $856 = 0, $857 = 0, $858 = 0, $859 = 0, $86 = 0, $860 = 0, $861 = 0, $862 = 0, $863 = 0, $864 = 0, $865 = 0, $866 = 0, $867 = 0, $868 = 0, $869 = 0, $87 = 0, $870 = 0, $871 = 0, $872 = 0;
 var $873 = 0, $874 = 0, $875 = 0, $876 = 0, $877 = 0, $878 = 0, $879 = 0, $88 = 0, $880 = 0, $881 = 0, $882 = 0, $883 = 0, $884 = 0, $885 = 0, $886 = 0, $887 = 0, $888 = 0, $889 = 0, $89 = 0, $890 = 0;
 var $891 = 0, $892 = 0, $893 = 0, $894 = 0, $895 = 0, $896 = 0, $897 = 0, $898 = 0, $899 = 0, $9 = 0, $90 = 0, $900 = 0, $901 = 0, $902 = 0, $903 = 0, $904 = 0, $905 = 0, $906 = 0, $907 = 0, $908 = 0;
 var $909 = 0, $91 = 0, $910 = 0, $911 = 0, $912 = 0, $913 = 0, $914 = 0, $915 = 0, $916 = 0, $917 = 0, $918 = 0, $919 = 0, $92 = 0, $920 = 0, $921 = 0, $922 = 0, $923 = 0, $924 = 0, $925 = 0, $926 = 0;
 var $927 = 0, $928 = 0, $929 = 0, $93 = 0, $930 = 0, $931 = 0, $932 = 0, $933 = 0, $934 = 0, $935 = 0, $936 = 0, $937 = 0, $938 = 0, $939 = 0, $94 = 0, $940 = 0, $941 = 0, $942 = 0, $943 = 0, $944 = 0;
 var $945 = 0, $946 = 0, $947 = 0, $948 = 0, $949 = 0, $95 = 0, $950 = 0, $951 = 0, $952 = 0, $953 = 0, $954 = 0, $955 = 0, $956 = 0, $957 = 0, $958 = 0, $959 = 0, $96 = 0, $960 = 0, $961 = 0, $962 = 0;
 var $963 = 0, $964 = 0, $965 = 0, $966 = 0, $967 = 0, $968 = 0, $969 = 0, $97 = 0, $970 = 0, $971 = 0, $972 = 0, $973 = 0, $974 = 0, $975 = 0, $976 = 0, $977 = 0, $978 = 0, $979 = 0, $98 = 0, $980 = 0;
 var $981 = 0, $982 = 0, $983 = 0, $984 = 0, $985 = 0, $986 = 0, $987 = 0, $988 = 0, $989 = 0, $99 = 0, $990 = 0, $991 = 0, $arglist_current = 0, $arglist_current27 = 0, $arglist_current30 = 0, $arglist_current33 = 0, $arglist_current36 = 0, $arglist_current39 = 0, $arglist_current42 = 0, $arglist_current45 = 0;
 var $arglist_current48 = 0, $arglist_current51 = 0, $arglist_current54 = 0, $arglist_current57 = 0, $arglist_current60 = 0, $arglist_current63 = 0, $arglist_current66 = 0, $arglist_next = 0, $arglist_next28 = 0, $arglist_next31 = 0, $arglist_next34 = 0, $arglist_next37 = 0, $arglist_next40 = 0, $arglist_next43 = 0, $arglist_next46 = 0, $arglist_next49 = 0, $arglist_next52 = 0, $arglist_next55 = 0, $arglist_next58 = 0, $arglist_next61 = 0;
 var $arglist_next64 = 0, $arglist_next67 = 0, $bArgList = 0, $base = 0, $buf = 0, $bufpt = 0, $c = 0, $ch = 0, $cset = 0, $done = 0, $e2 = 0, $escarg = 0, $exp = 0, $flag_alternateform = 0, $flag_altform2 = 0, $flag_blanksign = 0, $flag_dp = 0, $flag_leftjustify = 0, $flag_long = 0, $flag_longlong = 0;
 var $flag_plussign = 0, $flag_rtz = 0, $flag_zeropad = 0, $i = 0, $i2 = 0, $idx = 0, $infop = 0, $isnull = 0, $j = 0, $k = 0, $k3 = 0, $length = 0, $longvalue = 0, $n = 0, $nOut = 0, $nPad = 0, $needQuote = 0, $nsd = 0, $or$cond = 0, $or$cond11 = 0;
 var $or$cond13 = 0, $or$cond15 = 0, $or$cond17 = 0, $or$cond19 = 0, $or$cond21 = 0, $or$cond23 = 0, $or$cond25 = 0, $or$cond9 = 0, $pArgList = 0, $pItem = 0, $pSrc = 0, $pToken = 0, $pre = 0, $precision = 0, $prefix = 0, $q = 0, $realvalue = 0, $rounder = 0, $scale = 0, $useIntern = 0;
 var $v = 0, $width = 0, $x = 0, $x1 = 0, $xtype = 0, $zExtra = 0, $zOut = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 272|0;
 $0 = sp + 172|0;
 $1 = sp + 168|0;
 $2 = sp + 164|0;
 $3 = sp + 160|0;
 $c = sp + 156|0;
 $bufpt = sp + 152|0;
 $precision = sp + 148|0;
 $length = sp + 144|0;
 $idx = sp + 140|0;
 $width = sp + 136|0;
 $flag_leftjustify = sp + 264|0;
 $flag_plussign = sp + 263|0;
 $flag_blanksign = sp + 262|0;
 $flag_alternateform = sp + 261|0;
 $flag_altform2 = sp + 260|0;
 $flag_zeropad = sp + 259|0;
 $flag_long = sp + 258|0;
 $flag_longlong = sp + 257|0;
 $done = sp + 256|0;
 $xtype = sp + 255|0;
 $bArgList = sp + 254|0;
 $useIntern = sp + 253|0;
 $prefix = sp + 252|0;
 $longvalue = sp + 32|0;
 $realvalue = sp + 24|0;
 $infop = sp + 132|0;
 $zOut = sp + 128|0;
 $nOut = sp + 124|0;
 $zExtra = sp + 120|0;
 $exp = sp + 116|0;
 $e2 = sp + 112|0;
 $nsd = sp + 108|0;
 $rounder = sp + 16|0;
 $flag_dp = sp + 251|0;
 $flag_rtz = sp + 250|0;
 $pArgList = sp + 104|0;
 $buf = sp + 180|0;
 $v = sp + 8|0;
 $x = sp + 100|0;
 $cset = sp + 96|0;
 $base = sp + 179|0;
 $pre = sp + 92|0;
 $x1 = sp + 178|0;
 $scale = sp;
 $i = sp + 88|0;
 $nPad = sp + 84|0;
 $i2 = sp + 80|0;
 $j = sp + 76|0;
 $k = sp + 72|0;
 $n = sp + 68|0;
 $isnull = sp + 64|0;
 $needQuote = sp + 60|0;
 $ch = sp + 177|0;
 $q = sp + 176|0;
 $escarg = sp + 56|0;
 $pToken = sp + 52|0;
 $pSrc = sp + 48|0;
 $k3 = sp + 44|0;
 $pItem = sp + 40|0;
 HEAP32[$0>>2] = $pAccum;
 HEAP32[$1>>2] = $bFlags;
 HEAP32[$2>>2] = $fmt;
 HEAP32[$3>>2] = $ap;
 HEAP8[$xtype>>0] = 0;
 HEAP32[$zExtra>>2] = 0;
 HEAP32[$pArgList>>2] = 0;
 HEAP32[$bufpt>>2] = 0;
 $4 = HEAP32[$1>>2]|0;
 $5 = ($4|0)!=(0);
 if ($5) {
  $6 = HEAP32[$1>>2]|0;
  $7 = $6 & 2;
  $8 = $7&255;
  HEAP8[$bArgList>>0] = $8;
  $9 = $8&255;
  $10 = ($9|0)!=(0);
  if ($10) {
   $11 = HEAP32[$3>>2]|0;
   $arglist_current = HEAP32[$11>>2]|0;
   $12 = HEAP32[$arglist_current>>2]|0;
   $arglist_next = (($arglist_current) + 4|0);
   HEAP32[$11>>2] = $arglist_next;
   HEAP32[$pArgList>>2] = $12;
  }
  $13 = HEAP32[$1>>2]|0;
  $14 = $13 & 1;
  $15 = $14&255;
  HEAP8[$useIntern>>0] = $15;
 } else {
  HEAP8[$useIntern>>0] = 0;
  HEAP8[$bArgList>>0] = 0;
 }
 L7: while(1) {
  $16 = HEAP32[$2>>2]|0;
  $17 = HEAP8[$16>>0]|0;
  $18 = $17 << 24 >> 24;
  HEAP32[$c>>2] = $18;
  $19 = ($18|0)!=(0);
  if (!($19)) {
   label = 274;
   break;
  }
  $20 = HEAP32[$c>>2]|0;
  $21 = ($20|0)!=(37);
  if ($21) {
   $22 = HEAP32[$2>>2]|0;
   HEAP32[$bufpt>>2] = $22;
   while(1) {
    $23 = HEAP32[$2>>2]|0;
    $24 = (($23) + 1|0);
    HEAP32[$2>>2] = $24;
    $25 = HEAP32[$2>>2]|0;
    $26 = HEAP8[$25>>0]|0;
    $27 = $26 << 24 >> 24;
    $28 = ($27|0)!=(0);
    if (!($28)) {
     break;
    }
    $29 = HEAP32[$2>>2]|0;
    $30 = HEAP8[$29>>0]|0;
    $31 = $30 << 24 >> 24;
    $32 = ($31|0)!=(37);
    if (!($32)) {
     break;
    }
   }
   $33 = HEAP32[$0>>2]|0;
   $34 = HEAP32[$bufpt>>2]|0;
   $35 = HEAP32[$2>>2]|0;
   $36 = HEAP32[$bufpt>>2]|0;
   $37 = $35;
   $38 = $36;
   $39 = (($37) - ($38))|0;
   _sqlite3StrAccumAppend($33,$34,$39);
   $40 = HEAP32[$2>>2]|0;
   $41 = HEAP8[$40>>0]|0;
   $42 = $41 << 24 >> 24;
   $43 = ($42|0)==(0);
   if ($43) {
    label = 274;
    break;
   }
  }
  $44 = HEAP32[$2>>2]|0;
  $45 = (($44) + 1|0);
  HEAP32[$2>>2] = $45;
  $46 = HEAP8[$45>>0]|0;
  $47 = $46 << 24 >> 24;
  HEAP32[$c>>2] = $47;
  $48 = ($47|0)==(0);
  if ($48) {
   label = 13;
   break;
  }
  HEAP8[$flag_zeropad>>0] = 0;
  HEAP8[$flag_altform2>>0] = 0;
  HEAP8[$flag_alternateform>>0] = 0;
  HEAP8[$flag_blanksign>>0] = 0;
  HEAP8[$flag_plussign>>0] = 0;
  HEAP8[$flag_leftjustify>>0] = 0;
  HEAP8[$done>>0] = 0;
  while(1) {
   $50 = HEAP32[$c>>2]|0;
   switch ($50|0) {
   case 45:  {
    HEAP8[$flag_leftjustify>>0] = 1;
    break;
   }
   case 43:  {
    HEAP8[$flag_plussign>>0] = 1;
    break;
   }
   case 32:  {
    HEAP8[$flag_blanksign>>0] = 1;
    break;
   }
   case 33:  {
    HEAP8[$flag_altform2>>0] = 1;
    break;
   }
   case 48:  {
    HEAP8[$flag_zeropad>>0] = 1;
    break;
   }
   case 35:  {
    HEAP8[$flag_alternateform>>0] = 1;
    break;
   }
   default: {
    HEAP8[$done>>0] = 1;
   }
   }
   $51 = HEAP8[$done>>0]|0;
   $52 = ($51<<24>>24)!=(0);
   if ($52) {
    break;
   }
   $53 = HEAP32[$2>>2]|0;
   $54 = (($53) + 1|0);
   HEAP32[$2>>2] = $54;
   $55 = HEAP8[$54>>0]|0;
   $56 = $55 << 24 >> 24;
   HEAP32[$c>>2] = $56;
   $57 = ($56|0)!=(0);
   if (!($57)) {
    break;
   }
  }
  HEAP32[$width>>2] = 0;
  $58 = HEAP32[$c>>2]|0;
  $59 = ($58|0)==(42);
  L31: do {
   if ($59) {
    $60 = HEAP8[$bArgList>>0]|0;
    $61 = ($60<<24>>24)!=(0);
    if ($61) {
     $62 = HEAP32[$pArgList>>2]|0;
     $63 = (_getIntArg($62)|0);
     $64 = tempRet0;
     HEAP32[$width>>2] = $63;
    } else {
     $65 = HEAP32[$3>>2]|0;
     $arglist_current27 = HEAP32[$65>>2]|0;
     $66 = HEAP32[$arglist_current27>>2]|0;
     $arglist_next28 = (($arglist_current27) + 4|0);
     HEAP32[$65>>2] = $arglist_next28;
     HEAP32[$width>>2] = $66;
    }
    $67 = HEAP32[$width>>2]|0;
    $68 = ($67|0)<(0);
    if ($68) {
     HEAP8[$flag_leftjustify>>0] = 1;
     $69 = HEAP32[$width>>2]|0;
     $70 = (0 - ($69))|0;
     HEAP32[$width>>2] = $70;
    }
    $71 = HEAP32[$2>>2]|0;
    $72 = (($71) + 1|0);
    HEAP32[$2>>2] = $72;
    $73 = HEAP8[$72>>0]|0;
    $74 = $73 << 24 >> 24;
    HEAP32[$c>>2] = $74;
   } else {
    while(1) {
     $75 = HEAP32[$c>>2]|0;
     $76 = ($75|0)>=(48);
     $77 = HEAP32[$c>>2]|0;
     $78 = ($77|0)<=(57);
     $or$cond17 = $76 & $78;
     if (!($or$cond17)) {
      break L31;
     }
     $79 = HEAP32[$width>>2]|0;
     $80 = ($79*10)|0;
     $81 = HEAP32[$c>>2]|0;
     $82 = (($80) + ($81))|0;
     $83 = (($82) - 48)|0;
     HEAP32[$width>>2] = $83;
     $84 = HEAP32[$2>>2]|0;
     $85 = (($84) + 1|0);
     HEAP32[$2>>2] = $85;
     $86 = HEAP8[$85>>0]|0;
     $87 = $86 << 24 >> 24;
     HEAP32[$c>>2] = $87;
    }
   }
  } while(0);
  $88 = HEAP32[$c>>2]|0;
  $89 = ($88|0)==(46);
  L44: do {
   if ($89) {
    HEAP32[$precision>>2] = 0;
    $90 = HEAP32[$2>>2]|0;
    $91 = (($90) + 1|0);
    HEAP32[$2>>2] = $91;
    $92 = HEAP8[$91>>0]|0;
    $93 = $92 << 24 >> 24;
    HEAP32[$c>>2] = $93;
    $94 = HEAP32[$c>>2]|0;
    $95 = ($94|0)==(42);
    if (!($95)) {
     while(1) {
      $111 = HEAP32[$c>>2]|0;
      $112 = ($111|0)>=(48);
      $113 = HEAP32[$c>>2]|0;
      $114 = ($113|0)<=(57);
      $or$cond19 = $112 & $114;
      if (!($or$cond19)) {
       break L44;
      }
      $115 = HEAP32[$precision>>2]|0;
      $116 = ($115*10)|0;
      $117 = HEAP32[$c>>2]|0;
      $118 = (($116) + ($117))|0;
      $119 = (($118) - 48)|0;
      HEAP32[$precision>>2] = $119;
      $120 = HEAP32[$2>>2]|0;
      $121 = (($120) + 1|0);
      HEAP32[$2>>2] = $121;
      $122 = HEAP8[$121>>0]|0;
      $123 = $122 << 24 >> 24;
      HEAP32[$c>>2] = $123;
     }
    }
    $96 = HEAP8[$bArgList>>0]|0;
    $97 = ($96<<24>>24)!=(0);
    if ($97) {
     $98 = HEAP32[$pArgList>>2]|0;
     $99 = (_getIntArg($98)|0);
     $100 = tempRet0;
     HEAP32[$precision>>2] = $99;
    } else {
     $101 = HEAP32[$3>>2]|0;
     $arglist_current30 = HEAP32[$101>>2]|0;
     $102 = HEAP32[$arglist_current30>>2]|0;
     $arglist_next31 = (($arglist_current30) + 4|0);
     HEAP32[$101>>2] = $arglist_next31;
     HEAP32[$precision>>2] = $102;
    }
    $103 = HEAP32[$precision>>2]|0;
    $104 = ($103|0)<(0);
    if ($104) {
     $105 = HEAP32[$precision>>2]|0;
     $106 = (0 - ($105))|0;
     HEAP32[$precision>>2] = $106;
    }
    $107 = HEAP32[$2>>2]|0;
    $108 = (($107) + 1|0);
    HEAP32[$2>>2] = $108;
    $109 = HEAP8[$108>>0]|0;
    $110 = $109 << 24 >> 24;
    HEAP32[$c>>2] = $110;
   } else {
    HEAP32[$precision>>2] = -1;
   }
  } while(0);
  $124 = HEAP32[$c>>2]|0;
  $125 = ($124|0)==(108);
  do {
   if ($125) {
    HEAP8[$flag_long>>0] = 1;
    $126 = HEAP32[$2>>2]|0;
    $127 = (($126) + 1|0);
    HEAP32[$2>>2] = $127;
    $128 = HEAP8[$127>>0]|0;
    $129 = $128 << 24 >> 24;
    HEAP32[$c>>2] = $129;
    $130 = HEAP32[$c>>2]|0;
    $131 = ($130|0)==(108);
    if ($131) {
     HEAP8[$flag_longlong>>0] = 1;
     $132 = HEAP32[$2>>2]|0;
     $133 = (($132) + 1|0);
     HEAP32[$2>>2] = $133;
     $134 = HEAP8[$133>>0]|0;
     $135 = $134 << 24 >> 24;
     HEAP32[$c>>2] = $135;
     break;
    } else {
     HEAP8[$flag_longlong>>0] = 0;
     break;
    }
   } else {
    HEAP8[$flag_longlong>>0] = 0;
    HEAP8[$flag_long>>0] = 0;
   }
  } while(0);
  HEAP32[$infop>>2] = 38392;
  HEAP8[$xtype>>0] = 0;
  HEAP32[$idx>>2] = 0;
  while(1) {
   $136 = HEAP32[$idx>>2]|0;
   $137 = ($136|0)<(23);
   if (!($137)) {
    break;
   }
   $138 = HEAP32[$c>>2]|0;
   $139 = HEAP32[$idx>>2]|0;
   $140 = (38392 + (($139*6)|0)|0);
   $141 = HEAP8[$140>>0]|0;
   $142 = $141 << 24 >> 24;
   $143 = ($138|0)==($142|0);
   $144 = HEAP32[$idx>>2]|0;
   if ($143) {
    label = 53;
    break;
   }
   $158 = (($144) + 1)|0;
   HEAP32[$idx>>2] = $158;
  }
  if ((label|0) == 53) {
   label = 0;
   $145 = (38392 + (($144*6)|0)|0);
   HEAP32[$infop>>2] = $145;
   $146 = HEAP8[$useIntern>>0]|0;
   $147 = $146&255;
   $148 = ($147|0)!=(0);
   if (!($148)) {
    $149 = HEAP32[$infop>>2]|0;
    $150 = (($149) + 2|0);
    $151 = HEAP8[$150>>0]|0;
    $152 = $151&255;
    $153 = $152 & 2;
    $154 = ($153|0)==(0);
    if (!($154)) {
     label = 274;
     break;
    }
   }
   $155 = HEAP32[$infop>>2]|0;
   $156 = (($155) + 3|0);
   $157 = HEAP8[$156>>0]|0;
   HEAP8[$xtype>>0] = $157;
  }
  $159 = HEAP8[$xtype>>0]|0;
  $160 = $159&255;
  L77: do {
   switch ($160|0) {
   case 14:  {
    HEAP8[$flag_longlong>>0] = 0;
    HEAP8[$flag_long>>0] = 1;
    label = 59;
    break;
   }
   case 1: case 16:  {
    label = 59;
    break;
   }
   case 4: case 3: case 2:  {
    $441 = HEAP8[$bArgList>>0]|0;
    $442 = ($441<<24>>24)!=(0);
    if ($442) {
     $443 = HEAP32[$pArgList>>2]|0;
     $444 = (+_getDoubleArg($443));
     HEAPF64[$realvalue>>3] = $444;
    } else {
     $445 = HEAP32[$3>>2]|0;
     $arglist_current45 = HEAP32[$445>>2]|0;
     HEAP32[tempDoublePtr>>2]=HEAP32[$arglist_current45>>2];HEAP32[tempDoublePtr+4>>2]=HEAP32[$arglist_current45+4>>2];$446 = +HEAPF64[tempDoublePtr>>3];
     $arglist_next46 = (($arglist_current45) + 8|0);
     HEAP32[$445>>2] = $arglist_next46;
     HEAPF64[$realvalue>>3] = $446;
    }
    $447 = HEAP32[$precision>>2]|0;
    $448 = ($447|0)<(0);
    if ($448) {
     HEAP32[$precision>>2] = 6;
    }
    $449 = +HEAPF64[$realvalue>>3];
    $450 = $449 < 0.0;
    do {
     if ($450) {
      $451 = +HEAPF64[$realvalue>>3];
      $452 = -$451;
      HEAPF64[$realvalue>>3] = $452;
      HEAP8[$prefix>>0] = 45;
     } else {
      $453 = HEAP8[$flag_plussign>>0]|0;
      $454 = ($453<<24>>24)!=(0);
      if ($454) {
       HEAP8[$prefix>>0] = 43;
       break;
      }
      $455 = HEAP8[$flag_blanksign>>0]|0;
      $456 = ($455<<24>>24)!=(0);
      if ($456) {
       HEAP8[$prefix>>0] = 32;
       break;
      } else {
       HEAP8[$prefix>>0] = 0;
       break;
      }
     }
    } while(0);
    $457 = HEAP8[$xtype>>0]|0;
    $458 = $457&255;
    $459 = ($458|0)==(4);
    $460 = HEAP32[$precision>>2]|0;
    $461 = ($460|0)>(0);
    $or$cond = $459 & $461;
    if ($or$cond) {
     $462 = HEAP32[$precision>>2]|0;
     $463 = (($462) + -1)|0;
     HEAP32[$precision>>2] = $463;
    }
    $464 = HEAP32[$precision>>2]|0;
    HEAP32[$idx>>2] = $464;
    HEAPF64[$rounder>>3] = 0.5;
    while(1) {
     $465 = HEAP32[$idx>>2]|0;
     $466 = ($465|0)>(0);
     if (!($466)) {
      break;
     }
     $467 = HEAP32[$idx>>2]|0;
     $468 = (($467) + -1)|0;
     HEAP32[$idx>>2] = $468;
     $469 = +HEAPF64[$rounder>>3];
     $470 = $469 * 0.10000000000000001;
     HEAPF64[$rounder>>3] = $470;
    }
    $471 = HEAP8[$xtype>>0]|0;
    $472 = $471&255;
    $473 = ($472|0)==(2);
    if ($473) {
     $474 = +HEAPF64[$rounder>>3];
     $475 = +HEAPF64[$realvalue>>3];
     $476 = $475 + $474;
     HEAPF64[$realvalue>>3] = $476;
    }
    HEAP32[$exp>>2] = 0;
    $477 = +HEAPF64[$realvalue>>3];
    $478 = (_sqlite3IsNaN($477)|0);
    $479 = ($478|0)!=(0);
    if ($479) {
     HEAP32[$bufpt>>2] = 38600;
     HEAP32[$length>>2] = 3;
     break L77;
    }
    $480 = +HEAPF64[$realvalue>>3];
    $481 = $480 > 0.0;
    if ($481) {
     HEAPF64[$scale>>3] = 1.0;
     while(1) {
      $482 = +HEAPF64[$realvalue>>3];
      $483 = +HEAPF64[$scale>>3];
      $484 = 1.0E+100 * $483;
      $485 = $482 >= $484;
      $486 = HEAP32[$exp>>2]|0;
      $487 = ($486|0)<=(350);
      $or$cond21 = $485 & $487;
      if (!($or$cond21)) {
       break;
      }
      $488 = +HEAPF64[$scale>>3];
      $489 = $488 * 1.0E+100;
      HEAPF64[$scale>>3] = $489;
      $490 = HEAP32[$exp>>2]|0;
      $491 = (($490) + 100)|0;
      HEAP32[$exp>>2] = $491;
     }
     while(1) {
      $492 = +HEAPF64[$realvalue>>3];
      $493 = +HEAPF64[$scale>>3];
      $494 = 1.0E+64 * $493;
      $495 = $492 >= $494;
      $496 = HEAP32[$exp>>2]|0;
      $497 = ($496|0)<=(350);
      $or$cond23 = $495 & $497;
      if (!($or$cond23)) {
       break;
      }
      $498 = +HEAPF64[$scale>>3];
      $499 = $498 * 1.0E+64;
      HEAPF64[$scale>>3] = $499;
      $500 = HEAP32[$exp>>2]|0;
      $501 = (($500) + 64)|0;
      HEAP32[$exp>>2] = $501;
     }
     while(1) {
      $502 = +HEAPF64[$realvalue>>3];
      $503 = +HEAPF64[$scale>>3];
      $504 = 1.0E+8 * $503;
      $505 = $502 >= $504;
      $506 = HEAP32[$exp>>2]|0;
      $507 = ($506|0)<=(350);
      $or$cond25 = $505 & $507;
      if (!($or$cond25)) {
       break;
      }
      $508 = +HEAPF64[$scale>>3];
      $509 = $508 * 1.0E+8;
      HEAPF64[$scale>>3] = $509;
      $510 = HEAP32[$exp>>2]|0;
      $511 = (($510) + 8)|0;
      HEAP32[$exp>>2] = $511;
     }
     while(1) {
      $512 = +HEAPF64[$realvalue>>3];
      $513 = +HEAPF64[$scale>>3];
      $514 = 10.0 * $513;
      $515 = $512 >= $514;
      if ($515) {
       $516 = HEAP32[$exp>>2]|0;
       $517 = ($516|0)<=(350);
       $991 = $517;
      } else {
       $991 = 0;
      }
      $518 = +HEAPF64[$scale>>3];
      if (!($991)) {
       break;
      }
      $519 = $518 * 10.0;
      HEAPF64[$scale>>3] = $519;
      $520 = HEAP32[$exp>>2]|0;
      $521 = (($520) + 1)|0;
      HEAP32[$exp>>2] = $521;
     }
     $522 = +HEAPF64[$realvalue>>3];
     $523 = $522 / $518;
     HEAPF64[$realvalue>>3] = $523;
     while(1) {
      $524 = +HEAPF64[$realvalue>>3];
      $525 = $524 < 1.0E-8;
      if (!($525)) {
       break;
      }
      $526 = +HEAPF64[$realvalue>>3];
      $527 = $526 * 1.0E+8;
      HEAPF64[$realvalue>>3] = $527;
      $528 = HEAP32[$exp>>2]|0;
      $529 = (($528) - 8)|0;
      HEAP32[$exp>>2] = $529;
     }
     while(1) {
      $530 = +HEAPF64[$realvalue>>3];
      $531 = $530 < 1.0;
      if (!($531)) {
       break;
      }
      $532 = +HEAPF64[$realvalue>>3];
      $533 = $532 * 10.0;
      HEAPF64[$realvalue>>3] = $533;
      $534 = HEAP32[$exp>>2]|0;
      $535 = (($534) + -1)|0;
      HEAP32[$exp>>2] = $535;
     }
     $536 = HEAP32[$exp>>2]|0;
     $537 = ($536|0)>(350);
     if ($537) {
      $538 = HEAP8[$prefix>>0]|0;
      $539 = $538 << 24 >> 24;
      $540 = ($539|0)==(45);
      do {
       if ($540) {
        HEAP32[$bufpt>>2] = 38608;
       } else {
        $541 = HEAP8[$prefix>>0]|0;
        $542 = $541 << 24 >> 24;
        $543 = ($542|0)==(43);
        if ($543) {
         HEAP32[$bufpt>>2] = 38616;
         break;
        } else {
         HEAP32[$bufpt>>2] = 38624;
         break;
        }
       }
      } while(0);
      $544 = HEAP32[$bufpt>>2]|0;
      $545 = (_sqlite3Strlen30($544)|0);
      HEAP32[$length>>2] = $545;
      break L77;
     }
    }
    HEAP32[$bufpt>>2] = $buf;
    $546 = HEAP8[$xtype>>0]|0;
    $547 = $546&255;
    $548 = ($547|0)!=(2);
    if ($548) {
     $549 = +HEAPF64[$rounder>>3];
     $550 = +HEAPF64[$realvalue>>3];
     $551 = $550 + $549;
     HEAPF64[$realvalue>>3] = $551;
     $552 = +HEAPF64[$realvalue>>3];
     $553 = $552 >= 10.0;
     if ($553) {
      $554 = +HEAPF64[$realvalue>>3];
      $555 = $554 * 0.10000000000000001;
      HEAPF64[$realvalue>>3] = $555;
      $556 = HEAP32[$exp>>2]|0;
      $557 = (($556) + 1)|0;
      HEAP32[$exp>>2] = $557;
     }
    }
    $558 = HEAP8[$xtype>>0]|0;
    $559 = $558&255;
    $560 = ($559|0)==(4);
    L148: do {
     if ($560) {
      $561 = HEAP8[$flag_alternateform>>0]|0;
      $562 = ($561<<24>>24)!=(0);
      $563 = $562 ^ 1;
      $564 = $563&1;
      $565 = $564&255;
      HEAP8[$flag_rtz>>0] = $565;
      $566 = HEAP32[$exp>>2]|0;
      $567 = ($566|0)<(-4);
      do {
       if (!($567)) {
        $568 = HEAP32[$exp>>2]|0;
        $569 = HEAP32[$precision>>2]|0;
        $570 = ($568|0)>($569|0);
        if ($570) {
         break;
        }
        $571 = HEAP32[$precision>>2]|0;
        $572 = HEAP32[$exp>>2]|0;
        $573 = (($571) - ($572))|0;
        HEAP32[$precision>>2] = $573;
        HEAP8[$xtype>>0] = 2;
        break L148;
       }
      } while(0);
      HEAP8[$xtype>>0] = 3;
     } else {
      $574 = HEAP8[$flag_altform2>>0]|0;
      HEAP8[$flag_rtz>>0] = $574;
     }
    } while(0);
    $575 = HEAP8[$xtype>>0]|0;
    $576 = $575&255;
    $577 = ($576|0)==(3);
    if ($577) {
     HEAP32[$e2>>2] = 0;
    } else {
     $578 = HEAP32[$exp>>2]|0;
     HEAP32[$e2>>2] = $578;
    }
    $579 = HEAP32[$e2>>2]|0;
    $580 = ($579|0)>(0);
    $581 = HEAP32[$e2>>2]|0;
    $582 = $580 ? $581 : 0;
    $583 = HEAP32[$precision>>2]|0;
    $584 = (($582) + ($583))|0;
    $585 = HEAP32[$width>>2]|0;
    $586 = (($584) + ($585))|0;
    $587 = ($586|0)>(55);
    if ($587) {
     $588 = HEAP32[$e2>>2]|0;
     $589 = ($588|0)>(0);
     $590 = HEAP32[$e2>>2]|0;
     $591 = $589 ? $590 : 0;
     $592 = HEAP32[$precision>>2]|0;
     $593 = (($591) + ($592))|0;
     $594 = HEAP32[$width>>2]|0;
     $595 = (($593) + ($594))|0;
     $596 = (($595) + 15)|0;
     $597 = ($596|0)<(0);
     $598 = $597 << 31 >> 31;
     $599 = (_sqlite3Malloc($596,$598)|0);
     HEAP32[$zExtra>>2] = $599;
     HEAP32[$bufpt>>2] = $599;
     $600 = HEAP32[$bufpt>>2]|0;
     $601 = ($600|0)==(0|0);
     if ($601) {
      label = 167;
      break L7;
     }
    }
    $603 = HEAP32[$bufpt>>2]|0;
    HEAP32[$zOut>>2] = $603;
    $604 = HEAP8[$flag_altform2>>0]|0;
    $605 = $604&255;
    $606 = ($605*10)|0;
    $607 = (16 + ($606))|0;
    HEAP32[$nsd>>2] = $607;
    $608 = HEAP32[$precision>>2]|0;
    $609 = ($608|0)>(0);
    $610 = $609 ? 1 : 0;
    $611 = HEAP8[$flag_alternateform>>0]|0;
    $612 = $611&255;
    $613 = $610 | $612;
    $614 = HEAP8[$flag_altform2>>0]|0;
    $615 = $614&255;
    $616 = $613 | $615;
    $617 = $616&255;
    HEAP8[$flag_dp>>0] = $617;
    $618 = HEAP8[$prefix>>0]|0;
    $619 = ($618<<24>>24)!=(0);
    if ($619) {
     $620 = HEAP8[$prefix>>0]|0;
     $621 = HEAP32[$bufpt>>2]|0;
     $622 = (($621) + 1|0);
     HEAP32[$bufpt>>2] = $622;
     HEAP8[$621>>0] = $620;
    }
    $623 = HEAP32[$e2>>2]|0;
    $624 = ($623|0)<(0);
    L166: do {
     if ($624) {
      $625 = HEAP32[$bufpt>>2]|0;
      $626 = (($625) + 1|0);
      HEAP32[$bufpt>>2] = $626;
      HEAP8[$625>>0] = 48;
     } else {
      while(1) {
       $627 = HEAP32[$e2>>2]|0;
       $628 = ($627|0)>=(0);
       if (!($628)) {
        break L166;
       }
       $629 = (_et_getdigit($realvalue,$nsd)|0);
       $630 = HEAP32[$bufpt>>2]|0;
       $631 = (($630) + 1|0);
       HEAP32[$bufpt>>2] = $631;
       HEAP8[$630>>0] = $629;
       $632 = HEAP32[$e2>>2]|0;
       $633 = (($632) + -1)|0;
       HEAP32[$e2>>2] = $633;
      }
     }
    } while(0);
    $634 = HEAP8[$flag_dp>>0]|0;
    $635 = ($634<<24>>24)!=(0);
    if ($635) {
     $636 = HEAP32[$bufpt>>2]|0;
     $637 = (($636) + 1|0);
     HEAP32[$bufpt>>2] = $637;
     HEAP8[$636>>0] = 46;
    }
    $638 = HEAP32[$e2>>2]|0;
    $639 = (($638) + 1)|0;
    HEAP32[$e2>>2] = $639;
    while(1) {
     $640 = HEAP32[$e2>>2]|0;
     $641 = ($640|0)<(0);
     if (!($641)) {
      break;
     }
     $642 = HEAP32[$bufpt>>2]|0;
     $643 = (($642) + 1|0);
     HEAP32[$bufpt>>2] = $643;
     HEAP8[$642>>0] = 48;
     $644 = HEAP32[$precision>>2]|0;
     $645 = (($644) + -1)|0;
     HEAP32[$precision>>2] = $645;
     $646 = HEAP32[$e2>>2]|0;
     $647 = (($646) + 1)|0;
     HEAP32[$e2>>2] = $647;
    }
    while(1) {
     $648 = HEAP32[$precision>>2]|0;
     $649 = (($648) + -1)|0;
     HEAP32[$precision>>2] = $649;
     $650 = ($648|0)>(0);
     if (!($650)) {
      break;
     }
     $651 = (_et_getdigit($realvalue,$nsd)|0);
     $652 = HEAP32[$bufpt>>2]|0;
     $653 = (($652) + 1|0);
     HEAP32[$bufpt>>2] = $653;
     HEAP8[$652>>0] = $651;
    }
    $654 = HEAP8[$flag_rtz>>0]|0;
    $655 = $654&255;
    $656 = ($655|0)!=(0);
    do {
     if ($656) {
      $657 = HEAP8[$flag_dp>>0]|0;
      $658 = $657&255;
      $659 = ($658|0)!=(0);
      if (!($659)) {
       break;
      }
      while(1) {
       $660 = HEAP32[$bufpt>>2]|0;
       $661 = (($660) + -1|0);
       $662 = HEAP8[$661>>0]|0;
       $663 = $662 << 24 >> 24;
       $664 = ($663|0)==(48);
       $665 = HEAP32[$bufpt>>2]|0;
       $666 = (($665) + -1|0);
       if (!($664)) {
        break;
       }
       HEAP32[$bufpt>>2] = $666;
       HEAP8[$666>>0] = 0;
      }
      $667 = HEAP8[$666>>0]|0;
      $668 = $667 << 24 >> 24;
      $669 = ($668|0)==(46);
      if (!($669)) {
       break;
      }
      $670 = HEAP8[$flag_altform2>>0]|0;
      $671 = ($670<<24>>24)!=(0);
      $672 = HEAP32[$bufpt>>2]|0;
      if ($671) {
       $673 = (($672) + 1|0);
       HEAP32[$bufpt>>2] = $673;
       HEAP8[$672>>0] = 48;
       break;
      } else {
       $674 = (($672) + -1|0);
       HEAP32[$bufpt>>2] = $674;
       HEAP8[$674>>0] = 0;
       break;
      }
     }
    } while(0);
    $675 = HEAP8[$xtype>>0]|0;
    $676 = $675&255;
    $677 = ($676|0)==(3);
    if ($677) {
     $678 = HEAP32[$infop>>2]|0;
     $679 = (($678) + 4|0);
     $680 = HEAP8[$679>>0]|0;
     $681 = $680&255;
     $682 = (38552 + ($681)|0);
     $683 = HEAP8[$682>>0]|0;
     $684 = HEAP32[$bufpt>>2]|0;
     $685 = (($684) + 1|0);
     HEAP32[$bufpt>>2] = $685;
     HEAP8[$684>>0] = $683;
     $686 = HEAP32[$exp>>2]|0;
     $687 = ($686|0)<(0);
     $688 = HEAP32[$bufpt>>2]|0;
     $689 = (($688) + 1|0);
     HEAP32[$bufpt>>2] = $689;
     if ($687) {
      HEAP8[$688>>0] = 45;
      $690 = HEAP32[$exp>>2]|0;
      $691 = (0 - ($690))|0;
      HEAP32[$exp>>2] = $691;
     } else {
      HEAP8[$688>>0] = 43;
     }
     $692 = HEAP32[$exp>>2]|0;
     $693 = ($692|0)>=(100);
     if ($693) {
      $694 = HEAP32[$exp>>2]|0;
      $695 = (($694|0) / 100)&-1;
      $696 = (($695) + 48)|0;
      $697 = $696&255;
      $698 = HEAP32[$bufpt>>2]|0;
      $699 = (($698) + 1|0);
      HEAP32[$bufpt>>2] = $699;
      HEAP8[$698>>0] = $697;
      $700 = HEAP32[$exp>>2]|0;
      $701 = (($700|0) % 100)&-1;
      HEAP32[$exp>>2] = $701;
     }
     $702 = HEAP32[$exp>>2]|0;
     $703 = (($702|0) / 10)&-1;
     $704 = (($703) + 48)|0;
     $705 = $704&255;
     $706 = HEAP32[$bufpt>>2]|0;
     $707 = (($706) + 1|0);
     HEAP32[$bufpt>>2] = $707;
     HEAP8[$706>>0] = $705;
     $708 = HEAP32[$exp>>2]|0;
     $709 = (($708|0) % 10)&-1;
     $710 = (($709) + 48)|0;
     $711 = $710&255;
     $712 = HEAP32[$bufpt>>2]|0;
     $713 = (($712) + 1|0);
     HEAP32[$bufpt>>2] = $713;
     HEAP8[$712>>0] = $711;
    }
    $714 = HEAP32[$bufpt>>2]|0;
    HEAP8[$714>>0] = 0;
    $715 = HEAP32[$bufpt>>2]|0;
    $716 = HEAP32[$zOut>>2]|0;
    $717 = $715;
    $718 = $716;
    $719 = (($717) - ($718))|0;
    HEAP32[$length>>2] = $719;
    $720 = HEAP32[$zOut>>2]|0;
    HEAP32[$bufpt>>2] = $720;
    $721 = HEAP8[$flag_zeropad>>0]|0;
    $722 = $721&255;
    $723 = ($722|0)==(0);
    $724 = HEAP8[$flag_leftjustify>>0]|0;
    $725 = ($724<<24>>24)!=(0);
    $or$cond9 = $723 | $725;
    if ($or$cond9) {
     break L77;
    }
    $726 = HEAP32[$length>>2]|0;
    $727 = HEAP32[$width>>2]|0;
    $728 = ($726|0)<($727|0);
    if (!($728)) {
     break L77;
    }
    $729 = HEAP32[$width>>2]|0;
    $730 = HEAP32[$length>>2]|0;
    $731 = (($729) - ($730))|0;
    HEAP32[$nPad>>2] = $731;
    $732 = HEAP32[$width>>2]|0;
    HEAP32[$i>>2] = $732;
    while(1) {
     $733 = HEAP32[$i>>2]|0;
     $734 = HEAP32[$nPad>>2]|0;
     $735 = ($733|0)>=($734|0);
     if (!($735)) {
      break;
     }
     $736 = HEAP32[$i>>2]|0;
     $737 = HEAP32[$nPad>>2]|0;
     $738 = (($736) - ($737))|0;
     $739 = HEAP32[$bufpt>>2]|0;
     $740 = (($739) + ($738)|0);
     $741 = HEAP8[$740>>0]|0;
     $742 = HEAP32[$i>>2]|0;
     $743 = HEAP32[$bufpt>>2]|0;
     $744 = (($743) + ($742)|0);
     HEAP8[$744>>0] = $741;
     $745 = HEAP32[$i>>2]|0;
     $746 = (($745) + -1)|0;
     HEAP32[$i>>2] = $746;
    }
    $747 = HEAP8[$prefix>>0]|0;
    $748 = $747 << 24 >> 24;
    $749 = ($748|0)!=(0);
    $750 = $749&1;
    HEAP32[$i>>2] = $750;
    while(1) {
     $751 = HEAP32[$nPad>>2]|0;
     $752 = (($751) + -1)|0;
     HEAP32[$nPad>>2] = $752;
     $753 = ($751|0)!=(0);
     if (!($753)) {
      break;
     }
     $754 = HEAP32[$i>>2]|0;
     $755 = (($754) + 1)|0;
     HEAP32[$i>>2] = $755;
     $756 = HEAP32[$bufpt>>2]|0;
     $757 = (($756) + ($754)|0);
     HEAP8[$757>>0] = 48;
    }
    $758 = HEAP32[$width>>2]|0;
    HEAP32[$length>>2] = $758;
    break;
   }
   case 5:  {
    $759 = HEAP8[$bArgList>>0]|0;
    $760 = ($759<<24>>24)!=(0);
    if (!($760)) {
     $761 = HEAP32[$0>>2]|0;
     $762 = (($761) + 12|0);
     $763 = HEAP32[$762>>2]|0;
     $764 = HEAP32[$3>>2]|0;
     $arglist_current48 = HEAP32[$764>>2]|0;
     $765 = HEAP32[$arglist_current48>>2]|0;
     $arglist_next49 = (($arglist_current48) + 4|0);
     HEAP32[$764>>2] = $arglist_next49;
     HEAP32[$765>>2] = $763;
    }
    HEAP32[$width>>2] = 0;
    HEAP32[$length>>2] = 0;
    break;
   }
   case 8:  {
    HEAP8[$buf>>0] = 37;
    HEAP32[$bufpt>>2] = $buf;
    HEAP32[$length>>2] = 1;
    break;
   }
   case 9:  {
    $766 = HEAP8[$bArgList>>0]|0;
    $767 = ($766<<24>>24)!=(0);
    if ($767) {
     $768 = HEAP32[$pArgList>>2]|0;
     $769 = (_getTextArg($768)|0);
     HEAP32[$bufpt>>2] = $769;
     $770 = HEAP32[$bufpt>>2]|0;
     $771 = ($770|0)!=(0|0);
     if ($771) {
      $772 = HEAP32[$bufpt>>2]|0;
      $773 = HEAP8[$772>>0]|0;
      $774 = $773 << 24 >> 24;
      $775 = $774;
     } else {
      $775 = 0;
     }
     HEAP32[$c>>2] = $775;
    } else {
     $776 = HEAP32[$3>>2]|0;
     $arglist_current51 = HEAP32[$776>>2]|0;
     $777 = HEAP32[$arglist_current51>>2]|0;
     $arglist_next52 = (($arglist_current51) + 4|0);
     HEAP32[$776>>2] = $arglist_next52;
     HEAP32[$c>>2] = $777;
    }
    $778 = HEAP32[$precision>>2]|0;
    $779 = ($778|0)>(1);
    if ($779) {
     $780 = HEAP32[$precision>>2]|0;
     $781 = (($780) - 1)|0;
     $782 = HEAP32[$width>>2]|0;
     $783 = (($782) - ($781))|0;
     HEAP32[$width>>2] = $783;
     $784 = HEAP32[$width>>2]|0;
     $785 = ($784|0)<=(1);
     $786 = HEAP8[$flag_leftjustify>>0]|0;
     $787 = ($786<<24>>24)!=(0);
     $or$cond11 = $785 | $787;
     if (!($or$cond11)) {
      $788 = HEAP32[$0>>2]|0;
      $789 = HEAP32[$width>>2]|0;
      $790 = (($789) - 1)|0;
      _sqlite3AppendChar($788,$790,32);
      HEAP32[$width>>2] = 0;
     }
     $791 = HEAP32[$0>>2]|0;
     $792 = HEAP32[$precision>>2]|0;
     $793 = (($792) - 1)|0;
     $794 = HEAP32[$c>>2]|0;
     $795 = $794&255;
     _sqlite3AppendChar($791,$793,$795);
    }
    HEAP32[$length>>2] = 1;
    $796 = HEAP32[$c>>2]|0;
    $797 = $796&255;
    HEAP8[$buf>>0] = $797;
    HEAP32[$bufpt>>2] = $buf;
    break;
   }
   case 7: case 6:  {
    $798 = HEAP8[$bArgList>>0]|0;
    $799 = ($798<<24>>24)!=(0);
    if ($799) {
     $800 = HEAP32[$pArgList>>2]|0;
     $801 = (_getTextArg($800)|0);
     HEAP32[$bufpt>>2] = $801;
    } else {
     $802 = HEAP32[$3>>2]|0;
     $arglist_current54 = HEAP32[$802>>2]|0;
     $803 = HEAP32[$arglist_current54>>2]|0;
     $arglist_next55 = (($arglist_current54) + 4|0);
     HEAP32[$802>>2] = $arglist_next55;
     HEAP32[$bufpt>>2] = $803;
    }
    $804 = HEAP32[$bufpt>>2]|0;
    $805 = ($804|0)==(0|0);
    if ($805) {
     HEAP32[$bufpt>>2] = 1216;
    } else {
     $806 = HEAP8[$xtype>>0]|0;
     $807 = $806&255;
     $808 = ($807|0)!=(7);
     $809 = HEAP8[$bArgList>>0]|0;
     $810 = ($809<<24>>24)!=(0);
     $or$cond13 = $808 | $810;
     if (!($or$cond13)) {
      $811 = HEAP32[$bufpt>>2]|0;
      HEAP32[$zExtra>>2] = $811;
     }
    }
    $812 = HEAP32[$precision>>2]|0;
    $813 = ($812|0)>=(0);
    if (!($813)) {
     $825 = HEAP32[$bufpt>>2]|0;
     $826 = (_sqlite3Strlen30($825)|0);
     HEAP32[$length>>2] = $826;
     break L77;
    }
    HEAP32[$length>>2] = 0;
    while(1) {
     $814 = HEAP32[$length>>2]|0;
     $815 = HEAP32[$precision>>2]|0;
     $816 = ($814|0)<($815|0);
     if (!($816)) {
      break L77;
     }
     $817 = HEAP32[$length>>2]|0;
     $818 = HEAP32[$bufpt>>2]|0;
     $819 = (($818) + ($817)|0);
     $820 = HEAP8[$819>>0]|0;
     $821 = $820 << 24 >> 24;
     $822 = ($821|0)!=(0);
     if (!($822)) {
      break L77;
     }
     $823 = HEAP32[$length>>2]|0;
     $824 = (($823) + 1)|0;
     HEAP32[$length>>2] = $824;
    }
    break;
   }
   case 15: case 11: case 10:  {
    $827 = HEAP8[$xtype>>0]|0;
    $828 = $827&255;
    $829 = ($828|0)==(15);
    $830 = $829 ? 34 : 39;
    $831 = $830&255;
    HEAP8[$q>>0] = $831;
    $832 = HEAP8[$bArgList>>0]|0;
    $833 = ($832<<24>>24)!=(0);
    if ($833) {
     $834 = HEAP32[$pArgList>>2]|0;
     $835 = (_getTextArg($834)|0);
     HEAP32[$escarg>>2] = $835;
    } else {
     $836 = HEAP32[$3>>2]|0;
     $arglist_current57 = HEAP32[$836>>2]|0;
     $837 = HEAP32[$arglist_current57>>2]|0;
     $arglist_next58 = (($arglist_current57) + 4|0);
     HEAP32[$836>>2] = $arglist_next58;
     HEAP32[$escarg>>2] = $837;
    }
    $838 = HEAP32[$escarg>>2]|0;
    $839 = ($838|0)==(0|0);
    $840 = $839&1;
    HEAP32[$isnull>>2] = $840;
    $841 = HEAP32[$isnull>>2]|0;
    $842 = ($841|0)!=(0);
    if ($842) {
     $843 = HEAP8[$xtype>>0]|0;
     $844 = $843&255;
     $845 = ($844|0)==(11);
     $846 = $845 ? 9840 : 38632;
     HEAP32[$escarg>>2] = $846;
    }
    $847 = HEAP32[$precision>>2]|0;
    HEAP32[$k>>2] = $847;
    HEAP32[$n>>2] = 0;
    HEAP32[$i2>>2] = 0;
    while(1) {
     $848 = HEAP32[$k>>2]|0;
     $849 = ($848|0)!=(0);
     if (!($849)) {
      break;
     }
     $850 = HEAP32[$i2>>2]|0;
     $851 = HEAP32[$escarg>>2]|0;
     $852 = (($851) + ($850)|0);
     $853 = HEAP8[$852>>0]|0;
     HEAP8[$ch>>0] = $853;
     $854 = $853 << 24 >> 24;
     $855 = ($854|0)!=(0);
     if (!($855)) {
      break;
     }
     $856 = HEAP8[$ch>>0]|0;
     $857 = $856 << 24 >> 24;
     $858 = HEAP8[$q>>0]|0;
     $859 = $858 << 24 >> 24;
     $860 = ($857|0)==($859|0);
     if ($860) {
      $861 = HEAP32[$n>>2]|0;
      $862 = (($861) + 1)|0;
      HEAP32[$n>>2] = $862;
     }
     $863 = HEAP32[$i2>>2]|0;
     $864 = (($863) + 1)|0;
     HEAP32[$i2>>2] = $864;
     $865 = HEAP32[$k>>2]|0;
     $866 = (($865) + -1)|0;
     HEAP32[$k>>2] = $866;
    }
    $867 = HEAP32[$isnull>>2]|0;
    $868 = ($867|0)!=(0);
    if ($868) {
     $873 = 0;
    } else {
     $869 = HEAP8[$xtype>>0]|0;
     $870 = $869&255;
     $871 = ($870|0)==(11);
     $873 = $871;
    }
    $872 = $873&1;
    HEAP32[$needQuote>>2] = $872;
    $874 = HEAP32[$i2>>2]|0;
    $875 = (($874) + 1)|0;
    $876 = HEAP32[$needQuote>>2]|0;
    $877 = $876<<1;
    $878 = (($875) + ($877))|0;
    $879 = HEAP32[$n>>2]|0;
    $880 = (($879) + ($878))|0;
    HEAP32[$n>>2] = $880;
    $881 = HEAP32[$n>>2]|0;
    $882 = ($881|0)>(70);
    if ($882) {
     $883 = HEAP32[$n>>2]|0;
     $884 = ($883|0)<(0);
     $885 = $884 << 31 >> 31;
     $886 = (_sqlite3Malloc($883,$885)|0);
     HEAP32[$zExtra>>2] = $886;
     HEAP32[$bufpt>>2] = $886;
     $887 = HEAP32[$bufpt>>2]|0;
     $888 = ($887|0)==(0|0);
     if ($888) {
      label = 247;
      break L7;
     }
    } else {
     HEAP32[$bufpt>>2] = $buf;
    }
    HEAP32[$j>>2] = 0;
    $890 = HEAP32[$needQuote>>2]|0;
    $891 = ($890|0)!=(0);
    if ($891) {
     $892 = HEAP8[$q>>0]|0;
     $893 = HEAP32[$j>>2]|0;
     $894 = (($893) + 1)|0;
     HEAP32[$j>>2] = $894;
     $895 = HEAP32[$bufpt>>2]|0;
     $896 = (($895) + ($893)|0);
     HEAP8[$896>>0] = $892;
    }
    $897 = HEAP32[$i2>>2]|0;
    HEAP32[$k>>2] = $897;
    HEAP32[$i2>>2] = 0;
    while(1) {
     $898 = HEAP32[$i2>>2]|0;
     $899 = HEAP32[$k>>2]|0;
     $900 = ($898|0)<($899|0);
     if (!($900)) {
      break;
     }
     $901 = HEAP32[$i2>>2]|0;
     $902 = HEAP32[$escarg>>2]|0;
     $903 = (($902) + ($901)|0);
     $904 = HEAP8[$903>>0]|0;
     HEAP8[$ch>>0] = $904;
     $905 = HEAP32[$j>>2]|0;
     $906 = (($905) + 1)|0;
     HEAP32[$j>>2] = $906;
     $907 = HEAP32[$bufpt>>2]|0;
     $908 = (($907) + ($905)|0);
     HEAP8[$908>>0] = $904;
     $909 = HEAP8[$ch>>0]|0;
     $910 = $909 << 24 >> 24;
     $911 = HEAP8[$q>>0]|0;
     $912 = $911 << 24 >> 24;
     $913 = ($910|0)==($912|0);
     if ($913) {
      $914 = HEAP8[$ch>>0]|0;
      $915 = HEAP32[$j>>2]|0;
      $916 = (($915) + 1)|0;
      HEAP32[$j>>2] = $916;
      $917 = HEAP32[$bufpt>>2]|0;
      $918 = (($917) + ($915)|0);
      HEAP8[$918>>0] = $914;
     }
     $919 = HEAP32[$i2>>2]|0;
     $920 = (($919) + 1)|0;
     HEAP32[$i2>>2] = $920;
    }
    $921 = HEAP32[$needQuote>>2]|0;
    $922 = ($921|0)!=(0);
    if ($922) {
     $923 = HEAP8[$q>>0]|0;
     $924 = HEAP32[$j>>2]|0;
     $925 = (($924) + 1)|0;
     HEAP32[$j>>2] = $925;
     $926 = HEAP32[$bufpt>>2]|0;
     $927 = (($926) + ($924)|0);
     HEAP8[$927>>0] = $923;
    }
    $928 = HEAP32[$j>>2]|0;
    $929 = HEAP32[$bufpt>>2]|0;
    $930 = (($929) + ($928)|0);
    HEAP8[$930>>0] = 0;
    $931 = HEAP32[$j>>2]|0;
    HEAP32[$length>>2] = $931;
    break;
   }
   case 12:  {
    $932 = HEAP32[$3>>2]|0;
    $arglist_current60 = HEAP32[$932>>2]|0;
    $933 = HEAP32[$arglist_current60>>2]|0;
    $arglist_next61 = (($arglist_current60) + 4|0);
    HEAP32[$932>>2] = $arglist_next61;
    HEAP32[$pToken>>2] = $933;
    $934 = HEAP32[$pToken>>2]|0;
    $935 = ($934|0)!=(0|0);
    if ($935) {
     $936 = HEAP32[$pToken>>2]|0;
     $937 = (($936) + 4|0);
     $938 = HEAP32[$937>>2]|0;
     $939 = ($938|0)!=(0);
     if ($939) {
      $940 = HEAP32[$0>>2]|0;
      $941 = HEAP32[$pToken>>2]|0;
      $942 = HEAP32[$941>>2]|0;
      $943 = HEAP32[$pToken>>2]|0;
      $944 = (($943) + 4|0);
      $945 = HEAP32[$944>>2]|0;
      _sqlite3StrAccumAppend($940,$942,$945);
     }
    }
    HEAP32[$width>>2] = 0;
    HEAP32[$length>>2] = 0;
    break;
   }
   case 13:  {
    $946 = HEAP32[$3>>2]|0;
    $arglist_current63 = HEAP32[$946>>2]|0;
    $947 = HEAP32[$arglist_current63>>2]|0;
    $arglist_next64 = (($arglist_current63) + 4|0);
    HEAP32[$946>>2] = $arglist_next64;
    HEAP32[$pSrc>>2] = $947;
    $948 = HEAP32[$3>>2]|0;
    $arglist_current66 = HEAP32[$948>>2]|0;
    $949 = HEAP32[$arglist_current66>>2]|0;
    $arglist_next67 = (($arglist_current66) + 4|0);
    HEAP32[$948>>2] = $arglist_next67;
    HEAP32[$k3>>2] = $949;
    $950 = HEAP32[$k3>>2]|0;
    $951 = HEAP32[$pSrc>>2]|0;
    $952 = (($951) + 8|0);
    $953 = (($952) + (($950*72)|0)|0);
    HEAP32[$pItem>>2] = $953;
    $954 = HEAP32[$pItem>>2]|0;
    $955 = (($954) + 4|0);
    $956 = HEAP32[$955>>2]|0;
    $957 = ($956|0)!=(0|0);
    if ($957) {
     $958 = HEAP32[$0>>2]|0;
     $959 = HEAP32[$pItem>>2]|0;
     $960 = (($959) + 4|0);
     $961 = HEAP32[$960>>2]|0;
     _sqlite3StrAccumAppendAll($958,$961);
     $962 = HEAP32[$0>>2]|0;
     _sqlite3StrAccumAppend($962,19120,1);
    }
    $963 = HEAP32[$0>>2]|0;
    $964 = HEAP32[$pItem>>2]|0;
    $965 = (($964) + 8|0);
    $966 = HEAP32[$965>>2]|0;
    _sqlite3StrAccumAppendAll($963,$966);
    HEAP32[$width>>2] = 0;
    HEAP32[$length>>2] = 0;
    break;
   }
   default: {
    label = 274;
    break L7;
   }
   }
  } while(0);
  if ((label|0) == 59) {
   label = 0;
   $161 = HEAP32[$infop>>2]|0;
   $162 = (($161) + 2|0);
   $163 = HEAP8[$162>>0]|0;
   $164 = $163&255;
   $165 = $164 & 1;
   $166 = ($165|0)!=(0);
   $167 = HEAP8[$bArgList>>0]|0;
   $168 = ($167<<24>>24)!=(0);
   do {
    if ($166) {
     do {
      if ($168) {
       $169 = HEAP32[$pArgList>>2]|0;
       $170 = (_getIntArg($169)|0);
       $171 = tempRet0;
       $172 = $v;
       $173 = $172;
       HEAP32[$173>>2] = $170;
       $174 = (($172) + 4)|0;
       $175 = $174;
       HEAP32[$175>>2] = $171;
      } else {
       $176 = HEAP8[$flag_longlong>>0]|0;
       $177 = ($176<<24>>24)!=(0);
       $178 = HEAP32[$3>>2]|0;
       if ($177) {
        $arglist_current33 = HEAP32[$178>>2]|0;
        $179 = $arglist_current33;
        $180 = $179;
        $181 = HEAP32[$180>>2]|0;
        $182 = (($179) + 4)|0;
        $183 = $182;
        $184 = HEAP32[$183>>2]|0;
        $arglist_next34 = (($arglist_current33) + 8|0);
        HEAP32[$178>>2] = $arglist_next34;
        $185 = $v;
        $186 = $185;
        HEAP32[$186>>2] = $181;
        $187 = (($185) + 4)|0;
        $188 = $187;
        HEAP32[$188>>2] = $184;
        break;
       } else {
        $arglist_current36 = HEAP32[$178>>2]|0;
        $189 = HEAP32[$arglist_current36>>2]|0;
        $arglist_next37 = (($arglist_current36) + 4|0);
        HEAP32[$178>>2] = $arglist_next37;
        $190 = ($189|0)<(0);
        $191 = $190 << 31 >> 31;
        $192 = $v;
        $193 = $192;
        HEAP32[$193>>2] = $189;
        $194 = (($192) + 4)|0;
        $195 = $194;
        HEAP32[$195>>2] = $191;
        break;
       }
      }
     } while(0);
     $196 = $v;
     $197 = $196;
     $198 = HEAP32[$197>>2]|0;
     $199 = (($196) + 4)|0;
     $200 = $199;
     $201 = HEAP32[$200>>2]|0;
     $202 = ($201|0)<(0);
     $203 = $v;
     $204 = $203;
     $205 = HEAP32[$204>>2]|0;
     $206 = (($203) + 4)|0;
     $207 = $206;
     $208 = HEAP32[$207>>2]|0;
     if ($202) {
      $209 = ($205|0)==(0);
      $210 = ($208|0)==(-2147483648);
      $211 = $209 & $210;
      if ($211) {
       $212 = $longvalue;
       $213 = $212;
       HEAP32[$213>>2] = 0;
       $214 = (($212) + 4)|0;
       $215 = $214;
       HEAP32[$215>>2] = -2147483648;
      } else {
       $216 = $v;
       $217 = $216;
       $218 = HEAP32[$217>>2]|0;
       $219 = (($216) + 4)|0;
       $220 = $219;
       $221 = HEAP32[$220>>2]|0;
       $222 = (_i64Subtract(0,0,($218|0),($221|0))|0);
       $223 = tempRet0;
       $224 = $longvalue;
       $225 = $224;
       HEAP32[$225>>2] = $222;
       $226 = (($224) + 4)|0;
       $227 = $226;
       HEAP32[$227>>2] = $223;
      }
      HEAP8[$prefix>>0] = 45;
      break;
     }
     $228 = $longvalue;
     $229 = $228;
     HEAP32[$229>>2] = $205;
     $230 = (($228) + 4)|0;
     $231 = $230;
     HEAP32[$231>>2] = $208;
     $232 = HEAP8[$flag_plussign>>0]|0;
     $233 = ($232<<24>>24)!=(0);
     if ($233) {
      HEAP8[$prefix>>0] = 43;
      break;
     }
     $234 = HEAP8[$flag_blanksign>>0]|0;
     $235 = ($234<<24>>24)!=(0);
     if ($235) {
      HEAP8[$prefix>>0] = 32;
      break;
     } else {
      HEAP8[$prefix>>0] = 0;
      break;
     }
    } else {
     do {
      if ($168) {
       $236 = HEAP32[$pArgList>>2]|0;
       $237 = (_getIntArg($236)|0);
       $238 = tempRet0;
       $239 = $longvalue;
       $240 = $239;
       HEAP32[$240>>2] = $237;
       $241 = (($239) + 4)|0;
       $242 = $241;
       HEAP32[$242>>2] = $238;
      } else {
       $243 = HEAP8[$flag_longlong>>0]|0;
       $244 = ($243<<24>>24)!=(0);
       $245 = HEAP32[$3>>2]|0;
       if ($244) {
        $arglist_current39 = HEAP32[$245>>2]|0;
        $246 = $arglist_current39;
        $247 = $246;
        $248 = HEAP32[$247>>2]|0;
        $249 = (($246) + 4)|0;
        $250 = $249;
        $251 = HEAP32[$250>>2]|0;
        $arglist_next40 = (($arglist_current39) + 8|0);
        HEAP32[$245>>2] = $arglist_next40;
        $252 = $longvalue;
        $253 = $252;
        HEAP32[$253>>2] = $248;
        $254 = (($252) + 4)|0;
        $255 = $254;
        HEAP32[$255>>2] = $251;
        break;
       } else {
        $arglist_current42 = HEAP32[$245>>2]|0;
        $256 = HEAP32[$arglist_current42>>2]|0;
        $arglist_next43 = (($arglist_current42) + 4|0);
        HEAP32[$245>>2] = $arglist_next43;
        $257 = $longvalue;
        $258 = $257;
        HEAP32[$258>>2] = $256;
        $259 = (($257) + 4)|0;
        $260 = $259;
        HEAP32[$260>>2] = 0;
        break;
       }
      }
     } while(0);
     HEAP8[$prefix>>0] = 0;
    }
   } while(0);
   $261 = $longvalue;
   $262 = $261;
   $263 = HEAP32[$262>>2]|0;
   $264 = (($261) + 4)|0;
   $265 = $264;
   $266 = HEAP32[$265>>2]|0;
   $267 = ($263|0)==(0);
   $268 = ($266|0)==(0);
   $269 = $267 & $268;
   if ($269) {
    HEAP8[$flag_alternateform>>0] = 0;
   }
   $270 = HEAP8[$flag_zeropad>>0]|0;
   $271 = $270&255;
   $272 = ($271|0)!=(0);
   if ($272) {
    $273 = HEAP32[$precision>>2]|0;
    $274 = HEAP32[$width>>2]|0;
    $275 = HEAP8[$prefix>>0]|0;
    $276 = $275 << 24 >> 24;
    $277 = ($276|0)!=(0);
    $278 = $277&1;
    $279 = (($274) - ($278))|0;
    $280 = ($273|0)<($279|0);
    if ($280) {
     $281 = HEAP32[$width>>2]|0;
     $282 = HEAP8[$prefix>>0]|0;
     $283 = $282 << 24 >> 24;
     $284 = ($283|0)!=(0);
     $285 = $284&1;
     $286 = (($281) - ($285))|0;
     HEAP32[$precision>>2] = $286;
    }
   }
   $287 = HEAP32[$precision>>2]|0;
   $288 = ($287|0)<(60);
   if ($288) {
    HEAP32[$nOut>>2] = 70;
    HEAP32[$zOut>>2] = $buf;
   } else {
    $289 = HEAP32[$precision>>2]|0;
    $290 = (($289) + 10)|0;
    HEAP32[$nOut>>2] = $290;
    $291 = HEAP32[$nOut>>2]|0;
    $292 = ($291|0)<(0);
    $293 = $292 << 31 >> 31;
    $294 = (_sqlite3Malloc($291,$293)|0);
    HEAP32[$zExtra>>2] = $294;
    HEAP32[$zOut>>2] = $294;
    $295 = HEAP32[$zOut>>2]|0;
    $296 = ($295|0)==(0|0);
    if ($296) {
     label = 89;
     break;
    }
   }
   $298 = HEAP32[$nOut>>2]|0;
   $299 = (($298) - 1)|0;
   $300 = HEAP32[$zOut>>2]|0;
   $301 = (($300) + ($299)|0);
   HEAP32[$bufpt>>2] = $301;
   $302 = HEAP8[$xtype>>0]|0;
   $303 = $302&255;
   $304 = ($303|0)==(16);
   if ($304) {
    $305 = $longvalue;
    $306 = $305;
    $307 = HEAP32[$306>>2]|0;
    $308 = (($305) + 4)|0;
    $309 = $308;
    $310 = HEAP32[$309>>2]|0;
    $311 = (___uremdi3(($307|0),($310|0),10,0)|0);
    $312 = tempRet0;
    HEAP32[$x>>2] = $311;
    $313 = HEAP32[$x>>2]|0;
    $314 = ($313|0)>=(4);
    if ($314) {
     label = 93;
    } else {
     $315 = $longvalue;
     $316 = $315;
     $317 = HEAP32[$316>>2]|0;
     $318 = (($315) + 4)|0;
     $319 = $318;
     $320 = HEAP32[$319>>2]|0;
     $321 = (___udivdi3(($317|0),($320|0),10,0)|0);
     $322 = tempRet0;
     $323 = (___uremdi3(($321|0),($322|0),10,0)|0);
     $324 = tempRet0;
     $325 = ($323|0)==(1);
     $326 = ($324|0)==(0);
     $327 = $325 & $326;
     if ($327) {
      label = 93;
     }
    }
    if ((label|0) == 93) {
     label = 0;
     HEAP32[$x>>2] = 0;
    }
    $328 = HEAP32[$x>>2]|0;
    $329 = $328<<1;
    $330 = (($329) + 1)|0;
    $331 = (38536 + ($330)|0);
    $332 = HEAP8[$331>>0]|0;
    $333 = HEAP32[$bufpt>>2]|0;
    $334 = (($333) + -1|0);
    HEAP32[$bufpt>>2] = $334;
    HEAP8[$334>>0] = $332;
    $335 = HEAP32[$x>>2]|0;
    $336 = $335<<1;
    $337 = (38536 + ($336)|0);
    $338 = HEAP8[$337>>0]|0;
    $339 = HEAP32[$bufpt>>2]|0;
    $340 = (($339) + -1|0);
    HEAP32[$bufpt>>2] = $340;
    HEAP8[$340>>0] = $338;
   }
   $341 = HEAP32[$infop>>2]|0;
   $342 = (($341) + 4|0);
   $343 = HEAP8[$342>>0]|0;
   $344 = $343&255;
   $345 = (38552 + ($344)|0);
   HEAP32[$cset>>2] = $345;
   $346 = HEAP32[$infop>>2]|0;
   $347 = (($346) + 1|0);
   $348 = HEAP8[$347>>0]|0;
   HEAP8[$base>>0] = $348;
   while(1) {
    $349 = $longvalue;
    $350 = $349;
    $351 = HEAP32[$350>>2]|0;
    $352 = (($349) + 4)|0;
    $353 = $352;
    $354 = HEAP32[$353>>2]|0;
    $355 = HEAP8[$base>>0]|0;
    $356 = $355&255;
    $357 = (___uremdi3(($351|0),($354|0),($356|0),0)|0);
    $358 = tempRet0;
    $359 = HEAP32[$cset>>2]|0;
    $360 = (($359) + ($357)|0);
    $361 = HEAP8[$360>>0]|0;
    $362 = HEAP32[$bufpt>>2]|0;
    $363 = (($362) + -1|0);
    HEAP32[$bufpt>>2] = $363;
    HEAP8[$363>>0] = $361;
    $364 = $longvalue;
    $365 = $364;
    $366 = HEAP32[$365>>2]|0;
    $367 = (($364) + 4)|0;
    $368 = $367;
    $369 = HEAP32[$368>>2]|0;
    $370 = HEAP8[$base>>0]|0;
    $371 = $370&255;
    $372 = (___udivdi3(($366|0),($369|0),($371|0),0)|0);
    $373 = tempRet0;
    $374 = $longvalue;
    $375 = $374;
    HEAP32[$375>>2] = $372;
    $376 = (($374) + 4)|0;
    $377 = $376;
    HEAP32[$377>>2] = $373;
    $378 = $longvalue;
    $379 = $378;
    $380 = HEAP32[$379>>2]|0;
    $381 = (($378) + 4)|0;
    $382 = $381;
    $383 = HEAP32[$382>>2]|0;
    $384 = ($383>>>0)>(0);
    $385 = ($380>>>0)>(0);
    $386 = ($383|0)==(0);
    $387 = $386 & $385;
    $388 = $384 | $387;
    if (!($388)) {
     break;
    }
   }
   $389 = HEAP32[$nOut>>2]|0;
   $390 = (($389) - 1)|0;
   $391 = HEAP32[$zOut>>2]|0;
   $392 = (($391) + ($390)|0);
   $393 = HEAP32[$bufpt>>2]|0;
   $394 = $392;
   $395 = $393;
   $396 = (($394) - ($395))|0;
   HEAP32[$length>>2] = $396;
   $397 = HEAP32[$precision>>2]|0;
   $398 = HEAP32[$length>>2]|0;
   $399 = (($397) - ($398))|0;
   HEAP32[$idx>>2] = $399;
   while(1) {
    $400 = HEAP32[$idx>>2]|0;
    $401 = ($400|0)>(0);
    if (!($401)) {
     break;
    }
    $402 = HEAP32[$bufpt>>2]|0;
    $403 = (($402) + -1|0);
    HEAP32[$bufpt>>2] = $403;
    HEAP8[$403>>0] = 48;
    $404 = HEAP32[$idx>>2]|0;
    $405 = (($404) + -1)|0;
    HEAP32[$idx>>2] = $405;
   }
   $406 = HEAP8[$prefix>>0]|0;
   $407 = ($406<<24>>24)!=(0);
   if ($407) {
    $408 = HEAP8[$prefix>>0]|0;
    $409 = HEAP32[$bufpt>>2]|0;
    $410 = (($409) + -1|0);
    HEAP32[$bufpt>>2] = $410;
    HEAP8[$410>>0] = $408;
   }
   $411 = HEAP8[$flag_alternateform>>0]|0;
   $412 = $411&255;
   $413 = ($412|0)!=(0);
   L356: do {
    if ($413) {
     $414 = HEAP32[$infop>>2]|0;
     $415 = (($414) + 5|0);
     $416 = HEAP8[$415>>0]|0;
     $417 = $416&255;
     $418 = ($417|0)!=(0);
     if ($418) {
      $419 = HEAP32[$infop>>2]|0;
      $420 = (($419) + 5|0);
      $421 = HEAP8[$420>>0]|0;
      $422 = $421&255;
      $423 = (38592 + ($422)|0);
      HEAP32[$pre>>2] = $423;
      while(1) {
       $424 = HEAP32[$pre>>2]|0;
       $425 = HEAP8[$424>>0]|0;
       HEAP8[$x1>>0] = $425;
       $426 = $425 << 24 >> 24;
       $427 = ($426|0)!=(0);
       if (!($427)) {
        break L356;
       }
       $428 = HEAP8[$x1>>0]|0;
       $429 = HEAP32[$bufpt>>2]|0;
       $430 = (($429) + -1|0);
       HEAP32[$bufpt>>2] = $430;
       HEAP8[$430>>0] = $428;
       $431 = HEAP32[$pre>>2]|0;
       $432 = (($431) + 1|0);
       HEAP32[$pre>>2] = $432;
      }
     }
    }
   } while(0);
   $433 = HEAP32[$nOut>>2]|0;
   $434 = (($433) - 1)|0;
   $435 = HEAP32[$zOut>>2]|0;
   $436 = (($435) + ($434)|0);
   $437 = HEAP32[$bufpt>>2]|0;
   $438 = $436;
   $439 = $437;
   $440 = (($438) - ($439))|0;
   HEAP32[$length>>2] = $440;
  }
  $967 = HEAP32[$length>>2]|0;
  $968 = HEAP32[$width>>2]|0;
  $969 = (($968) - ($967))|0;
  HEAP32[$width>>2] = $969;
  $970 = HEAP32[$width>>2]|0;
  $971 = ($970|0)<=(0);
  $972 = HEAP8[$flag_leftjustify>>0]|0;
  $973 = ($972<<24>>24)!=(0);
  $or$cond15 = $971 | $973;
  if (!($or$cond15)) {
   $974 = HEAP32[$0>>2]|0;
   $975 = HEAP32[$width>>2]|0;
   _sqlite3AppendChar($974,$975,32);
  }
  $976 = HEAP32[$0>>2]|0;
  $977 = HEAP32[$bufpt>>2]|0;
  $978 = HEAP32[$length>>2]|0;
  _sqlite3StrAccumAppend($976,$977,$978);
  $979 = HEAP32[$width>>2]|0;
  $980 = ($979|0)>(0);
  if ($980) {
   $981 = HEAP8[$flag_leftjustify>>0]|0;
   $982 = $981&255;
   $983 = ($982|0)!=(0);
   if ($983) {
    $984 = HEAP32[$0>>2]|0;
    $985 = HEAP32[$width>>2]|0;
    _sqlite3AppendChar($984,$985,32);
   }
  }
  $986 = HEAP32[$zExtra>>2]|0;
  $987 = ($986|0)!=(0|0);
  if ($987) {
   $988 = HEAP32[$zExtra>>2]|0;
   _sqlite3_free($988);
   HEAP32[$zExtra>>2] = 0;
  }
  $989 = HEAP32[$2>>2]|0;
  $990 = (($989) + 1|0);
  HEAP32[$2>>2] = $990;
 }
 if ((label|0) == 13) {
  $49 = HEAP32[$0>>2]|0;
  _sqlite3StrAccumAppend($49,38384,1);
  STACKTOP = sp;return;
 }
 else if ((label|0) == 89) {
  $297 = HEAP32[$0>>2]|0;
  _setStrAccumError($297,1);
  STACKTOP = sp;return;
 }
 else if ((label|0) == 167) {
  $602 = HEAP32[$0>>2]|0;
  _setStrAccumError($602,1);
  STACKTOP = sp;return;
 }
 else if ((label|0) == 247) {
  $889 = HEAP32[$0>>2]|0;
  _setStrAccumError($889,1);
  STACKTOP = sp;return;
 }
 else if ((label|0) == 274) {
  STACKTOP = sp;return;
 }
}
function _sqlite3StrAccumFinish($p) {
 $p = $p|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 HEAP32[$0>>2] = $p;
 $1 = HEAP32[$0>>2]|0;
 $2 = (($1) + 8|0);
 $3 = HEAP32[$2>>2]|0;
 $4 = ($3|0)!=(0|0);
 do {
  if ($4) {
   $5 = HEAP32[$0>>2]|0;
   $6 = (($5) + 12|0);
   $7 = HEAP32[$6>>2]|0;
   $8 = HEAP32[$0>>2]|0;
   $9 = (($8) + 8|0);
   $10 = HEAP32[$9>>2]|0;
   $11 = (($10) + ($7)|0);
   HEAP8[$11>>0] = 0;
   $12 = HEAP32[$0>>2]|0;
   $13 = (($12) + 24|0);
   $14 = HEAP8[$13>>0]|0;
   $15 = $14&255;
   $16 = ($15|0)!=(0);
   if ($16) {
    $17 = HEAP32[$0>>2]|0;
    $18 = (($17) + 8|0);
    $19 = HEAP32[$18>>2]|0;
    $20 = HEAP32[$0>>2]|0;
    $21 = (($20) + 4|0);
    $22 = HEAP32[$21>>2]|0;
    $23 = ($19|0)==($22|0);
    if ($23) {
     $24 = HEAP32[$0>>2]|0;
     $25 = (($24) + 24|0);
     $26 = HEAP8[$25>>0]|0;
     $27 = $26&255;
     $28 = ($27|0)==(1);
     $29 = HEAP32[$0>>2]|0;
     if ($28) {
      $30 = HEAP32[$29>>2]|0;
      $31 = HEAP32[$0>>2]|0;
      $32 = (($31) + 12|0);
      $33 = HEAP32[$32>>2]|0;
      $34 = (($33) + 1)|0;
      $35 = ($34|0)<(0);
      $36 = $35 << 31 >> 31;
      $37 = (_sqlite3DbMallocRaw($30,$34,$36)|0);
      $38 = HEAP32[$0>>2]|0;
      $39 = (($38) + 8|0);
      HEAP32[$39>>2] = $37;
     } else {
      $40 = (($29) + 12|0);
      $41 = HEAP32[$40>>2]|0;
      $42 = (($41) + 1)|0;
      $43 = (_sqlite3_malloc($42)|0);
      $44 = HEAP32[$0>>2]|0;
      $45 = (($44) + 8|0);
      HEAP32[$45>>2] = $43;
     }
     $46 = HEAP32[$0>>2]|0;
     $47 = (($46) + 8|0);
     $48 = HEAP32[$47>>2]|0;
     $49 = ($48|0)!=(0|0);
     $50 = HEAP32[$0>>2]|0;
     if ($49) {
      $51 = (($50) + 8|0);
      $52 = HEAP32[$51>>2]|0;
      $53 = HEAP32[$0>>2]|0;
      $54 = (($53) + 4|0);
      $55 = HEAP32[$54>>2]|0;
      $56 = HEAP32[$0>>2]|0;
      $57 = (($56) + 12|0);
      $58 = HEAP32[$57>>2]|0;
      $59 = (($58) + 1)|0;
      _memcpy(($52|0),($55|0),($59|0))|0;
      break;
     } else {
      _setStrAccumError($50,1);
      break;
     }
    }
   }
  }
 } while(0);
 $60 = HEAP32[$0>>2]|0;
 $61 = (($60) + 8|0);
 $62 = HEAP32[$61>>2]|0;
 STACKTOP = sp;return ($62|0);
}
function _sqlite3_mprintf($zFormat,$varargs) {
 $zFormat = $zFormat|0;
 $varargs = $varargs|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $ap = 0, $z = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 28|0;
 $1 = sp + 24|0;
 $ap = sp + 8|0;
 $z = sp;
 HEAP32[$1>>2] = $zFormat;
 $2 = (_sqlite3_initialize()|0);
 $3 = ($2|0)!=(0);
 if ($3) {
  HEAP32[$0>>2] = 0;
  $7 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($7|0);
 } else {
  HEAP32[$ap>>2] = $varargs;
  $4 = HEAP32[$1>>2]|0;
  $5 = (_sqlite3_vmprintf($4,$ap)|0);
  HEAP32[$z>>2] = $5;
  $6 = HEAP32[$z>>2]|0;
  HEAP32[$0>>2] = $6;
  $7 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($7|0);
 }
 return 0|0;
}
function _sqlite3_vsnprintf($n,$zBuf,$zFormat,$ap) {
 $n = $n|0;
 $zBuf = $zBuf|0;
 $zFormat = $zFormat|0;
 $ap = $ap|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $acc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $0 = sp + 44|0;
 $1 = sp + 40|0;
 $2 = sp + 36|0;
 $3 = sp + 32|0;
 $4 = sp + 28|0;
 $acc = sp;
 HEAP32[$1>>2] = $n;
 HEAP32[$2>>2] = $zBuf;
 HEAP32[$3>>2] = $zFormat;
 HEAP32[$4>>2] = $ap;
 $5 = HEAP32[$1>>2]|0;
 $6 = ($5|0)<=(0);
 $7 = HEAP32[$2>>2]|0;
 if ($6) {
  HEAP32[$0>>2] = $7;
  $13 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($13|0);
 } else {
  $8 = HEAP32[$1>>2]|0;
  _sqlite3StrAccumInit($acc,$7,$8,0);
  $9 = (($acc) + 24|0);
  HEAP8[$9>>0] = 0;
  $10 = HEAP32[$3>>2]|0;
  $11 = HEAP32[$4>>2]|0;
  _sqlite3VXPrintf($acc,0,$10,$11);
  $12 = (_sqlite3StrAccumFinish($acc)|0);
  HEAP32[$0>>2] = $12;
  $13 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($13|0);
 }
 return 0|0;
}
function _sqlite3_snprintf($n,$zBuf,$zFormat,$varargs) {
 $n = $n|0;
 $zBuf = $zBuf|0;
 $zFormat = $zFormat|0;
 $varargs = $varargs|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $ap = 0, $z = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 28|0;
 $1 = sp + 24|0;
 $2 = sp + 20|0;
 $z = sp + 16|0;
 $ap = sp;
 HEAP32[$0>>2] = $n;
 HEAP32[$1>>2] = $zBuf;
 HEAP32[$2>>2] = $zFormat;
 HEAP32[$ap>>2] = $varargs;
 $3 = HEAP32[$0>>2]|0;
 $4 = HEAP32[$1>>2]|0;
 $5 = HEAP32[$2>>2]|0;
 $6 = (_sqlite3_vsnprintf($3,$4,$5,$ap)|0);
 HEAP32[$z>>2] = $6;
 $7 = HEAP32[$z>>2]|0;
 STACKTOP = sp;return ($7|0);
}
function _sqlite3_log($iErrCode,$zFormat,$varargs) {
 $iErrCode = $iErrCode|0;
 $zFormat = $zFormat|0;
 $varargs = $varargs|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $ap = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 20|0;
 $1 = sp + 16|0;
 $ap = sp;
 HEAP32[$0>>2] = $iErrCode;
 HEAP32[$1>>2] = $zFormat;
 $2 = HEAP32[((38640 + 252|0))>>2]|0;
 $3 = ($2|0)!=(0|0);
 if (!($3)) {
  STACKTOP = sp;return;
 }
 HEAP32[$ap>>2] = $varargs;
 $4 = HEAP32[$0>>2]|0;
 $5 = HEAP32[$1>>2]|0;
 _renderLogMsg($4,$5,$ap);
 STACKTOP = sp;return;
}
function _renderLogMsg($iErrCode,$zFormat,$ap) {
 $iErrCode = $iErrCode|0;
 $zFormat = $zFormat|0;
 $ap = $ap|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $acc = 0, $zMsg = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 256|0;
 $0 = sp + 36|0;
 $1 = sp + 32|0;
 $2 = sp + 28|0;
 $acc = sp;
 $zMsg = sp + 40|0;
 HEAP32[$0>>2] = $iErrCode;
 HEAP32[$1>>2] = $zFormat;
 HEAP32[$2>>2] = $ap;
 _sqlite3StrAccumInit($acc,$zMsg,210,0);
 $3 = (($acc) + 24|0);
 HEAP8[$3>>0] = 0;
 $4 = HEAP32[$1>>2]|0;
 $5 = HEAP32[$2>>2]|0;
 _sqlite3VXPrintf($acc,0,$4,$5);
 $6 = HEAP32[((38640 + 252|0))>>2]|0;
 $7 = HEAP32[((38640 + 256|0))>>2]|0;
 $8 = HEAP32[$0>>2]|0;
 $9 = (_sqlite3StrAccumFinish($acc)|0);
 FUNCTION_TABLE_viii[$6 & 63]($7,$8,$9);
 STACKTOP = sp;return;
}
function _sqlite3_randomness($N,$pBuf) {
 $N = $N|0;
 $pBuf = $pBuf|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0;
 var $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $9 = 0, $i = 0, $k = 0, $t = 0, $zBuf = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 288|0;
 $0 = sp + 12|0;
 $1 = sp + 8|0;
 $t = sp + 272|0;
 $zBuf = sp + 4|0;
 $i = sp;
 $k = sp + 16|0;
 HEAP32[$0>>2] = $N;
 HEAP32[$1>>2] = $pBuf;
 $2 = HEAP32[$1>>2]|0;
 HEAP32[$zBuf>>2] = $2;
 $3 = HEAP32[$0>>2]|0;
 $4 = ($3|0)<=(0);
 if ($4) {
  HEAP8[176>>0] = 0;
  STACKTOP = sp;return;
 }
 $5 = HEAP8[176>>0]|0;
 $6 = ($5<<24>>24)!=(0);
 if (!($6)) {
  HEAP8[((176 + 2|0))>>0] = 0;
  HEAP8[((176 + 1|0))>>0] = 0;
  $7 = (_sqlite3_vfs_find(0)|0);
  (_sqlite3OsRandomness($7,256,$k)|0);
  HEAP32[$i>>2] = 0;
  while(1) {
   $8 = HEAP32[$i>>2]|0;
   $9 = ($8|0)<(256);
   if (!($9)) {
    break;
   }
   $10 = HEAP32[$i>>2]|0;
   $11 = $10&255;
   $12 = HEAP32[$i>>2]|0;
   $13 = (((176 + 3|0)) + ($12)|0);
   HEAP8[$13>>0] = $11;
   $14 = HEAP32[$i>>2]|0;
   $15 = (($14) + 1)|0;
   HEAP32[$i>>2] = $15;
  }
  HEAP32[$i>>2] = 0;
  while(1) {
   $16 = HEAP32[$i>>2]|0;
   $17 = ($16|0)<(256);
   if (!($17)) {
    break;
   }
   $18 = HEAP32[$i>>2]|0;
   $19 = (((176 + 3|0)) + ($18)|0);
   $20 = HEAP8[$19>>0]|0;
   $21 = $20&255;
   $22 = HEAP32[$i>>2]|0;
   $23 = (($k) + ($22)|0);
   $24 = HEAP8[$23>>0]|0;
   $25 = $24 << 24 >> 24;
   $26 = (($21) + ($25))|0;
   $27 = HEAP8[((176 + 2|0))>>0]|0;
   $28 = $27&255;
   $29 = (($28) + ($26))|0;
   $30 = $29&255;
   HEAP8[((176 + 2|0))>>0] = $30;
   $31 = HEAP8[((176 + 2|0))>>0]|0;
   $32 = $31&255;
   $33 = (((176 + 3|0)) + ($32)|0);
   $34 = HEAP8[$33>>0]|0;
   HEAP8[$t>>0] = $34;
   $35 = HEAP32[$i>>2]|0;
   $36 = (((176 + 3|0)) + ($35)|0);
   $37 = HEAP8[$36>>0]|0;
   $38 = HEAP8[((176 + 2|0))>>0]|0;
   $39 = $38&255;
   $40 = (((176 + 3|0)) + ($39)|0);
   HEAP8[$40>>0] = $37;
   $41 = HEAP8[$t>>0]|0;
   $42 = HEAP32[$i>>2]|0;
   $43 = (((176 + 3|0)) + ($42)|0);
   HEAP8[$43>>0] = $41;
   $44 = HEAP32[$i>>2]|0;
   $45 = (($44) + 1)|0;
   HEAP32[$i>>2] = $45;
  }
  HEAP8[176>>0] = 1;
 }
 while(1) {
  $46 = HEAP8[((176 + 1|0))>>0]|0;
  $47 = (($46) + 1)<<24>>24;
  HEAP8[((176 + 1|0))>>0] = $47;
  $48 = HEAP8[((176 + 1|0))>>0]|0;
  $49 = $48&255;
  $50 = (((176 + 3|0)) + ($49)|0);
  $51 = HEAP8[$50>>0]|0;
  HEAP8[$t>>0] = $51;
  $52 = HEAP8[$t>>0]|0;
  $53 = $52&255;
  $54 = HEAP8[((176 + 2|0))>>0]|0;
  $55 = $54&255;
  $56 = (($55) + ($53))|0;
  $57 = $56&255;
  HEAP8[((176 + 2|0))>>0] = $57;
  $58 = HEAP8[((176 + 2|0))>>0]|0;
  $59 = $58&255;
  $60 = (((176 + 3|0)) + ($59)|0);
  $61 = HEAP8[$60>>0]|0;
  $62 = HEAP8[((176 + 1|0))>>0]|0;
  $63 = $62&255;
  $64 = (((176 + 3|0)) + ($63)|0);
  HEAP8[$64>>0] = $61;
  $65 = HEAP8[$t>>0]|0;
  $66 = HEAP8[((176 + 2|0))>>0]|0;
  $67 = $66&255;
  $68 = (((176 + 3|0)) + ($67)|0);
  HEAP8[$68>>0] = $65;
  $69 = HEAP8[((176 + 1|0))>>0]|0;
  $70 = $69&255;
  $71 = (((176 + 3|0)) + ($70)|0);
  $72 = HEAP8[$71>>0]|0;
  $73 = $72&255;
  $74 = HEAP8[$t>>0]|0;
  $75 = $74&255;
  $76 = (($75) + ($73))|0;
  $77 = $76&255;
  HEAP8[$t>>0] = $77;
  $78 = HEAP8[$t>>0]|0;
  $79 = $78&255;
  $80 = (((176 + 3|0)) + ($79)|0);
  $81 = HEAP8[$80>>0]|0;
  $82 = HEAP32[$zBuf>>2]|0;
  $83 = (($82) + 1|0);
  HEAP32[$zBuf>>2] = $83;
  HEAP8[$82>>0] = $81;
  $84 = HEAP32[$0>>2]|0;
  $85 = (($84) + -1)|0;
  HEAP32[$0>>2] = $85;
  $86 = ($85|0)!=(0);
  if (!($86)) {
   break;
  }
 }
 STACKTOP = sp;return;
}
function _sqlite3OsRandomness($pVfs,$nByte,$zBufOut) {
 $pVfs = $pVfs|0;
 $nByte = $nByte|0;
 $zBufOut = $zBufOut|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $1 = sp + 4|0;
 $2 = sp;
 HEAP32[$0>>2] = $pVfs;
 HEAP32[$1>>2] = $nByte;
 HEAP32[$2>>2] = $zBufOut;
 $3 = HEAP32[$0>>2]|0;
 $4 = (($3) + 56|0);
 $5 = HEAP32[$4>>2]|0;
 $6 = HEAP32[$0>>2]|0;
 $7 = HEAP32[$1>>2]|0;
 $8 = HEAP32[$2>>2]|0;
 $9 = (FUNCTION_TABLE_iiii[$5 & 31]($6,$7,$8)|0);
 STACKTOP = sp;return ($9|0);
}
function _sqlite3_stricmp($zLeft,$zRight) {
 $zLeft = $zLeft|0;
 $zRight = $zRight|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $a = 0, $b = 0;
 var label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $1 = sp + 8|0;
 $a = sp + 4|0;
 $b = sp;
 HEAP32[$0>>2] = $zLeft;
 HEAP32[$1>>2] = $zRight;
 $2 = HEAP32[$0>>2]|0;
 HEAP32[$a>>2] = $2;
 $3 = HEAP32[$1>>2]|0;
 HEAP32[$b>>2] = $3;
 while(1) {
  $4 = HEAP32[$a>>2]|0;
  $5 = HEAP8[$4>>0]|0;
  $6 = $5&255;
  $7 = ($6|0)!=(0);
  if ($7) {
   $8 = HEAP32[$a>>2]|0;
   $9 = HEAP8[$8>>0]|0;
   $10 = $9&255;
   $11 = (440 + ($10)|0);
   $12 = HEAP8[$11>>0]|0;
   $13 = $12&255;
   $14 = HEAP32[$b>>2]|0;
   $15 = HEAP8[$14>>0]|0;
   $16 = $15&255;
   $17 = (440 + ($16)|0);
   $18 = HEAP8[$17>>0]|0;
   $19 = $18&255;
   $20 = ($13|0)==($19|0);
   $37 = $20;
  } else {
   $37 = 0;
  }
  $21 = HEAP32[$a>>2]|0;
  if (!($37)) {
   break;
  }
  $22 = (($21) + 1|0);
  HEAP32[$a>>2] = $22;
  $23 = HEAP32[$b>>2]|0;
  $24 = (($23) + 1|0);
  HEAP32[$b>>2] = $24;
 }
 $25 = HEAP8[$21>>0]|0;
 $26 = $25&255;
 $27 = (440 + ($26)|0);
 $28 = HEAP8[$27>>0]|0;
 $29 = $28&255;
 $30 = HEAP32[$b>>2]|0;
 $31 = HEAP8[$30>>0]|0;
 $32 = $31&255;
 $33 = (440 + ($32)|0);
 $34 = HEAP8[$33>>0]|0;
 $35 = $34&255;
 $36 = (($29) - ($35))|0;
 STACKTOP = sp;return ($36|0);
}
function _sqlite3_os_init() {
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $i = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $i = sp;
 HEAP32[$i>>2] = 0;
 while(1) {
  $0 = HEAP32[$i>>2]|0;
  $1 = ($0>>>0)<(4);
  if (!($1)) {
   break;
  }
  $2 = HEAP32[$i>>2]|0;
  $3 = (696 + (($2*88)|0)|0);
  $4 = HEAP32[$i>>2]|0;
  $5 = ($4|0)==(0);
  $6 = $5&1;
  (_sqlite3_vfs_register($3,$6)|0);
  $7 = HEAP32[$i>>2]|0;
  $8 = (($7) + 1)|0;
  HEAP32[$i>>2] = $8;
 }
 STACKTOP = sp;return 0;
}
function _unixOpen($pVfs,$zPath,$pFile,$flags,$pOutFlags) {
 $pVfs = $pVfs|0;
 $zPath = $zPath|0;
 $pFile = $pFile|0;
 $flags = $flags|0;
 $pOutFlags = $pOutFlags|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0;
 var $170 = 0, $171 = 0, $172 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0;
 var $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0;
 var $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0;
 var $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0;
 var $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $ctrlFlags = 0, $eType = 0, $fd = 0, $gid = 0, $isCreate = 0, $isDelete = 0, $isExclusive = 0;
 var $isReadWrite = 0, $isReadonly = 0, $noLock = 0, $openFlags = 0, $openMode = 0, $or$cond = 0, $or$cond3 = 0, $or$cond3$not = 0, $or$cond5 = 0, $p = 0, $pUnused = 0, $rc = 0, $syncDir = 0, $uid = 0, $zName = 0, $zTmpname = 0, dest = 0, label = 0, sp = 0, stop = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 624|0;
 $0 = sp + 92|0;
 $1 = sp + 88|0;
 $2 = sp + 84|0;
 $3 = sp + 80|0;
 $4 = sp + 76|0;
 $5 = sp + 72|0;
 $p = sp + 68|0;
 $fd = sp + 64|0;
 $openFlags = sp + 60|0;
 $eType = sp + 56|0;
 $noLock = sp + 52|0;
 $rc = sp + 48|0;
 $ctrlFlags = sp + 44|0;
 $isExclusive = sp + 40|0;
 $isDelete = sp + 36|0;
 $isCreate = sp + 32|0;
 $isReadonly = sp + 28|0;
 $isReadWrite = sp + 24|0;
 $syncDir = sp + 20|0;
 $zTmpname = sp + 96|0;
 $zName = sp + 16|0;
 $pUnused = sp + 12|0;
 $openMode = sp + 8|0;
 $uid = sp + 4|0;
 $gid = sp;
 HEAP32[$1>>2] = $pVfs;
 HEAP32[$2>>2] = $zPath;
 HEAP32[$3>>2] = $pFile;
 HEAP32[$4>>2] = $flags;
 HEAP32[$5>>2] = $pOutFlags;
 $6 = HEAP32[$3>>2]|0;
 HEAP32[$p>>2] = $6;
 HEAP32[$fd>>2] = -1;
 HEAP32[$openFlags>>2] = 0;
 $7 = HEAP32[$4>>2]|0;
 $8 = $7 & -256;
 HEAP32[$eType>>2] = $8;
 HEAP32[$rc>>2] = 0;
 HEAP32[$ctrlFlags>>2] = 0;
 $9 = HEAP32[$4>>2]|0;
 $10 = $9 & 16;
 HEAP32[$isExclusive>>2] = $10;
 $11 = HEAP32[$4>>2]|0;
 $12 = $11 & 8;
 HEAP32[$isDelete>>2] = $12;
 $13 = HEAP32[$4>>2]|0;
 $14 = $13 & 4;
 HEAP32[$isCreate>>2] = $14;
 $15 = HEAP32[$4>>2]|0;
 $16 = $15 & 1;
 HEAP32[$isReadonly>>2] = $16;
 $17 = HEAP32[$4>>2]|0;
 $18 = $17 & 2;
 HEAP32[$isReadWrite>>2] = $18;
 $19 = HEAP32[$isCreate>>2]|0;
 $20 = ($19|0)!=(0);
 if ($20) {
  $21 = HEAP32[$eType>>2]|0;
  $22 = ($21|0)==(16384);
  $23 = HEAP32[$eType>>2]|0;
  $24 = ($23|0)==(2048);
  $or$cond = $22 | $24;
  if ($or$cond) {
   $28 = 1;
  } else {
   $25 = HEAP32[$eType>>2]|0;
   $26 = ($25|0)==(524288);
   $28 = $26;
  }
 } else {
  $28 = 0;
 }
 $27 = $28&1;
 HEAP32[$syncDir>>2] = $27;
 $29 = HEAP32[$2>>2]|0;
 HEAP32[$zName>>2] = $29;
 $30 = HEAP32[38096>>2]|0;
 $31 = (_getpid()|0);
 $32 = ($30|0)!=($31|0);
 if ($32) {
  $33 = (_getpid()|0);
  HEAP32[38096>>2] = $33;
  _sqlite3_randomness(0,0);
 }
 $34 = HEAP32[$p>>2]|0;
 dest=$34+0|0; stop=dest+44|0; do { HEAP32[dest>>2]=0|0; dest=dest+4|0; } while ((dest|0) < (stop|0));
 $35 = HEAP32[$eType>>2]|0;
 $36 = ($35|0)==(256);
 $37 = HEAP32[$zName>>2]|0;
 do {
  if ($36) {
   $38 = HEAP32[$4>>2]|0;
   $39 = (_findReusableFd($37,$38)|0);
   HEAP32[$pUnused>>2] = $39;
   $40 = HEAP32[$pUnused>>2]|0;
   $41 = ($40|0)!=(0|0);
   if ($41) {
    $42 = HEAP32[$pUnused>>2]|0;
    $43 = HEAP32[$42>>2]|0;
    HEAP32[$fd>>2] = $43;
   } else {
    $44 = (_sqlite3_malloc(12)|0);
    HEAP32[$pUnused>>2] = $44;
    $45 = HEAP32[$pUnused>>2]|0;
    $46 = ($45|0)!=(0|0);
    if (!($46)) {
     HEAP32[$0>>2] = 7;
     $172 = HEAP32[$0>>2]|0;
     STACKTOP = sp;return ($172|0);
    }
   }
   $47 = HEAP32[$pUnused>>2]|0;
   $48 = HEAP32[$p>>2]|0;
   $49 = (($48) + 28|0);
   HEAP32[$49>>2] = $47;
  } else {
   $50 = ($37|0)!=(0|0);
   if (!($50)) {
    $51 = (_unixGetTempname(514,$zTmpname)|0);
    HEAP32[$rc>>2] = $51;
    $52 = HEAP32[$rc>>2]|0;
    $53 = ($52|0)!=(0);
    if (!($53)) {
     HEAP32[$zName>>2] = $zTmpname;
     break;
    }
    $54 = HEAP32[$rc>>2]|0;
    HEAP32[$0>>2] = $54;
    $172 = HEAP32[$0>>2]|0;
    STACKTOP = sp;return ($172|0);
   }
  }
 } while(0);
 $55 = HEAP32[$isReadonly>>2]|0;
 $56 = ($55|0)!=(0);
 if ($56) {
  $57 = HEAP32[$openFlags>>2]|0;
  HEAP32[$openFlags>>2] = $57;
 }
 $58 = HEAP32[$isReadWrite>>2]|0;
 $59 = ($58|0)!=(0);
 if ($59) {
  $60 = HEAP32[$openFlags>>2]|0;
  $61 = $60 | 2;
  HEAP32[$openFlags>>2] = $61;
 }
 $62 = HEAP32[$isCreate>>2]|0;
 $63 = ($62|0)!=(0);
 if ($63) {
  $64 = HEAP32[$openFlags>>2]|0;
  $65 = $64 | 64;
  HEAP32[$openFlags>>2] = $65;
 }
 $66 = HEAP32[$isExclusive>>2]|0;
 $67 = ($66|0)!=(0);
 if ($67) {
  $68 = HEAP32[$openFlags>>2]|0;
  $69 = $68 | 131200;
  HEAP32[$openFlags>>2] = $69;
 }
 $70 = HEAP32[$openFlags>>2]|0;
 HEAP32[$openFlags>>2] = $70;
 $71 = HEAP32[$fd>>2]|0;
 $72 = ($71|0)<(0);
 do {
  if ($72) {
   $73 = HEAP32[$zName>>2]|0;
   $74 = HEAP32[$4>>2]|0;
   $75 = (_findCreateFileMode($73,$74,$openMode,$uid,$gid)|0);
   HEAP32[$rc>>2] = $75;
   $76 = HEAP32[$rc>>2]|0;
   $77 = ($76|0)!=(0);
   if ($77) {
    $78 = HEAP32[$rc>>2]|0;
    HEAP32[$0>>2] = $78;
    $172 = HEAP32[$0>>2]|0;
    STACKTOP = sp;return ($172|0);
   }
   $79 = HEAP32[$zName>>2]|0;
   $80 = HEAP32[$openFlags>>2]|0;
   $81 = HEAP32[$openMode>>2]|0;
   $82 = (_robust_open($79,$80,$81)|0);
   HEAP32[$fd>>2] = $82;
   $83 = HEAP32[$fd>>2]|0;
   $84 = ($83|0)<(0);
   if ($84) {
    $85 = (___errno_location()|0);
    $86 = HEAP32[$85>>2]|0;
    $87 = ($86|0)!=(21);
    $88 = HEAP32[$isReadWrite>>2]|0;
    $89 = ($88|0)!=(0);
    $or$cond3 = $87 & $89;
    $or$cond3$not = $or$cond3 ^ 1;
    $90 = HEAP32[$isExclusive>>2]|0;
    $91 = ($90|0)!=(0);
    $or$cond5 = $or$cond3$not | $91;
    if (!($or$cond5)) {
     $92 = HEAP32[$4>>2]|0;
     $93 = $92 & -7;
     HEAP32[$4>>2] = $93;
     $94 = HEAP32[$openFlags>>2]|0;
     $95 = $94 & -67;
     HEAP32[$openFlags>>2] = $95;
     $96 = HEAP32[$4>>2]|0;
     $97 = $96 | 1;
     HEAP32[$4>>2] = $97;
     $98 = HEAP32[$openFlags>>2]|0;
     HEAP32[$openFlags>>2] = $98;
     HEAP32[$isReadonly>>2] = 1;
     $99 = HEAP32[$zName>>2]|0;
     $100 = HEAP32[$openFlags>>2]|0;
     $101 = HEAP32[$openMode>>2]|0;
     $102 = (_robust_open($99,$100,$101)|0);
     HEAP32[$fd>>2] = $102;
    }
   }
   $103 = HEAP32[$fd>>2]|0;
   $104 = ($103|0)<(0);
   if ($104) {
    $105 = (_sqlite3CantopenError(30591)|0);
    $106 = HEAP32[$zName>>2]|0;
    $107 = (_unixLogErrorAtLine($105,37536,$106,30591)|0);
    HEAP32[$rc>>2] = $107;
    break;
   }
   $108 = HEAP32[$4>>2]|0;
   $109 = $108 & 526336;
   $110 = ($109|0)!=(0);
   if ($110) {
    $111 = HEAP32[((37192 + 244|0))>>2]|0;
    $112 = HEAP32[$fd>>2]|0;
    $113 = HEAP32[$uid>>2]|0;
    $114 = HEAP32[$gid>>2]|0;
    (FUNCTION_TABLE_iiii[$111 & 31]($112,$113,$114)|0);
    label = 34;
   } else {
    label = 34;
   }
  } else {
   label = 34;
  }
 } while(0);
 if ((label|0) == 34) {
  $115 = HEAP32[$5>>2]|0;
  $116 = ($115|0)!=(0|0);
  if ($116) {
   $117 = HEAP32[$4>>2]|0;
   $118 = HEAP32[$5>>2]|0;
   HEAP32[$118>>2] = $117;
  }
  $119 = HEAP32[$p>>2]|0;
  $120 = (($119) + 28|0);
  $121 = HEAP32[$120>>2]|0;
  $122 = ($121|0)!=(0|0);
  if ($122) {
   $123 = HEAP32[$fd>>2]|0;
   $124 = HEAP32[$p>>2]|0;
   $125 = (($124) + 28|0);
   $126 = HEAP32[$125>>2]|0;
   HEAP32[$126>>2] = $123;
   $127 = HEAP32[$4>>2]|0;
   $128 = HEAP32[$p>>2]|0;
   $129 = (($128) + 28|0);
   $130 = HEAP32[$129>>2]|0;
   $131 = (($130) + 4|0);
   HEAP32[$131>>2] = $127;
  }
  $132 = HEAP32[$isDelete>>2]|0;
  $133 = ($132|0)!=(0);
  if ($133) {
   $134 = HEAP32[((37192 + 196|0))>>2]|0;
   $135 = HEAP32[$zName>>2]|0;
   (FUNCTION_TABLE_ii[$134 & 31]($135)|0);
  }
  $136 = HEAP32[$eType>>2]|0;
  $137 = ($136|0)!=(256);
  $138 = $137&1;
  HEAP32[$noLock>>2] = $138;
  $139 = HEAP32[$isDelete>>2]|0;
  $140 = ($139|0)!=(0);
  if ($140) {
   $141 = HEAP32[$ctrlFlags>>2]|0;
   $142 = $141 | 32;
   HEAP32[$ctrlFlags>>2] = $142;
  }
  $143 = HEAP32[$isReadonly>>2]|0;
  $144 = ($143|0)!=(0);
  if ($144) {
   $145 = HEAP32[$ctrlFlags>>2]|0;
   $146 = $145 | 2;
   HEAP32[$ctrlFlags>>2] = $146;
  }
  $147 = HEAP32[$noLock>>2]|0;
  $148 = ($147|0)!=(0);
  if ($148) {
   $149 = HEAP32[$ctrlFlags>>2]|0;
   $150 = $149 | 128;
   HEAP32[$ctrlFlags>>2] = $150;
  }
  $151 = HEAP32[$syncDir>>2]|0;
  $152 = ($151|0)!=(0);
  if ($152) {
   $153 = HEAP32[$ctrlFlags>>2]|0;
   $154 = $153 | 8;
   HEAP32[$ctrlFlags>>2] = $154;
  }
  $155 = HEAP32[$4>>2]|0;
  $156 = $155 & 64;
  $157 = ($156|0)!=(0);
  if ($157) {
   $158 = HEAP32[$ctrlFlags>>2]|0;
   $159 = $158 | 64;
   HEAP32[$ctrlFlags>>2] = $159;
  }
  $160 = HEAP32[$1>>2]|0;
  $161 = HEAP32[$fd>>2]|0;
  $162 = HEAP32[$3>>2]|0;
  $163 = HEAP32[$2>>2]|0;
  $164 = HEAP32[$ctrlFlags>>2]|0;
  $165 = (_fillInUnixFile($160,$161,$162,$163,$164)|0);
  HEAP32[$rc>>2] = $165;
 }
 $166 = HEAP32[$rc>>2]|0;
 $167 = ($166|0)!=(0);
 if ($167) {
  $168 = HEAP32[$p>>2]|0;
  $169 = (($168) + 28|0);
  $170 = HEAP32[$169>>2]|0;
  _sqlite3_free($170);
 }
 $171 = HEAP32[$rc>>2]|0;
 HEAP32[$0>>2] = $171;
 $172 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($172|0);
}
function _unixDelete($NotUsed,$zPath,$dirSync) {
 $NotUsed = $NotUsed|0;
 $zPath = $zPath|0;
 $dirSync = $dirSync|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $fd = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 20|0;
 $1 = sp + 16|0;
 $2 = sp + 12|0;
 $3 = sp + 8|0;
 $rc = sp + 4|0;
 $fd = sp;
 HEAP32[$1>>2] = $NotUsed;
 HEAP32[$2>>2] = $zPath;
 HEAP32[$3>>2] = $dirSync;
 HEAP32[$rc>>2] = 0;
 $4 = HEAP32[((37192 + 196|0))>>2]|0;
 $5 = HEAP32[$2>>2]|0;
 $6 = (FUNCTION_TABLE_ii[$4 & 31]($5)|0);
 $7 = ($6|0)==(-1);
 if ($7) {
  $8 = (___errno_location()|0);
  $9 = HEAP32[$8>>2]|0;
  $10 = ($9|0)==(2);
  if ($10) {
   HEAP32[$rc>>2] = 5898;
  } else {
   $11 = HEAP32[$2>>2]|0;
   $12 = (_unixLogErrorAtLine(2570,37672,$11,30729)|0);
   HEAP32[$rc>>2] = $12;
  }
  $13 = HEAP32[$rc>>2]|0;
  HEAP32[$0>>2] = $13;
  $31 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($31|0);
 }
 $14 = HEAP32[$3>>2]|0;
 $15 = $14 & 1;
 $16 = ($15|0)!=(0);
 do {
  if ($16) {
   $17 = HEAP32[((37192 + 208|0))>>2]|0;
   $18 = HEAP32[$2>>2]|0;
   $19 = (FUNCTION_TABLE_iii[$17 & 63]($18,$fd)|0);
   HEAP32[$rc>>2] = $19;
   $20 = HEAP32[$rc>>2]|0;
   $21 = ($20|0)==(0);
   if (!($21)) {
    $28 = HEAP32[$rc>>2]|0;
    $29 = ($28|0)==(14);
    if (!($29)) {
     break;
    }
    HEAP32[$rc>>2] = 0;
    break;
   }
   $22 = HEAP32[$fd>>2]|0;
   $23 = (_fsync(($22|0))|0);
   $24 = ($23|0)!=(0);
   if ($24) {
    $25 = HEAP32[$2>>2]|0;
    $26 = (_unixLogErrorAtLine(1290,38128,$25,30744)|0);
    HEAP32[$rc>>2] = $26;
   }
   $27 = HEAP32[$fd>>2]|0;
   _robust_close(0,$27,30746);
  }
 } while(0);
 $30 = HEAP32[$rc>>2]|0;
 HEAP32[$0>>2] = $30;
 $31 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($31|0);
}
function _unixAccess($NotUsed,$zPath,$flags,$pResOut) {
 $NotUsed = $NotUsed|0;
 $zPath = $zPath|0;
 $flags = $flags|0;
 $pResOut = $pResOut|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $3 = 0, $4 = 0;
 var $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $amode = 0, $buf = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 96|0;
 $0 = sp + 92|0;
 $1 = sp + 88|0;
 $2 = sp + 84|0;
 $3 = sp + 80|0;
 $amode = sp + 76|0;
 $buf = sp;
 HEAP32[$0>>2] = $NotUsed;
 HEAP32[$1>>2] = $zPath;
 HEAP32[$2>>2] = $flags;
 HEAP32[$3>>2] = $pResOut;
 HEAP32[$amode>>2] = 0;
 $4 = HEAP32[$2>>2]|0;
 if ((($4|0) == 2)) {
  HEAP32[$amode>>2] = 4;
 } else if ((($4|0) == 0)) {
  HEAP32[$amode>>2] = 0;
 } else if ((($4|0) == 1)) {
  HEAP32[$amode>>2] = 6;
 }
 $5 = HEAP32[((37192 + 28|0))>>2]|0;
 $6 = HEAP32[$1>>2]|0;
 $7 = HEAP32[$amode>>2]|0;
 $8 = (FUNCTION_TABLE_iii[$5 & 63]($6,$7)|0);
 $9 = ($8|0)==(0);
 $10 = $9&1;
 $11 = HEAP32[$3>>2]|0;
 HEAP32[$11>>2] = $10;
 $12 = HEAP32[$2>>2]|0;
 $13 = ($12|0)==(0);
 if (!($13)) {
  STACKTOP = sp;return 0;
 }
 $14 = HEAP32[$3>>2]|0;
 $15 = HEAP32[$14>>2]|0;
 $16 = ($15|0)!=(0);
 if (!($16)) {
  STACKTOP = sp;return 0;
 }
 $17 = HEAP32[((37192 + 52|0))>>2]|0;
 $18 = HEAP32[$1>>2]|0;
 $19 = (FUNCTION_TABLE_iii[$17 & 63]($18,$buf)|0);
 $20 = (0)==($19|0);
 if (!($20)) {
  STACKTOP = sp;return 0;
 }
 $21 = (($buf) + 36|0);
 $22 = HEAP32[$21>>2]|0;
 $23 = ($22|0)==(0);
 if (!($23)) {
  STACKTOP = sp;return 0;
 }
 $24 = HEAP32[$3>>2]|0;
 HEAP32[$24>>2] = 0;
 STACKTOP = sp;return 0;
}
function _unixFullPathname($pVfs,$zPath,$nOut,$zOut) {
 $pVfs = $pVfs|0;
 $zPath = $zPath|0;
 $nOut = $nOut|0;
 $zOut = $zOut|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $nCwd = 0, $vararg_buffer1 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $vararg_buffer1 = sp;
 $0 = sp + 24|0;
 $1 = sp + 20|0;
 $2 = sp + 16|0;
 $3 = sp + 12|0;
 $4 = sp + 8|0;
 $nCwd = sp + 4|0;
 HEAP32[$1>>2] = $pVfs;
 HEAP32[$2>>2] = $zPath;
 HEAP32[$3>>2] = $nOut;
 HEAP32[$4>>2] = $zOut;
 $5 = HEAP32[$3>>2]|0;
 $6 = (($5) - 1)|0;
 $7 = HEAP32[$4>>2]|0;
 $8 = (($7) + ($6)|0);
 HEAP8[$8>>0] = 0;
 $9 = HEAP32[$2>>2]|0;
 $10 = HEAP8[$9>>0]|0;
 $11 = $10 << 24 >> 24;
 $12 = ($11|0)==(47);
 do {
  if ($12) {
   $13 = HEAP32[$3>>2]|0;
   $14 = HEAP32[$4>>2]|0;
   $15 = HEAP32[$2>>2]|0;
   HEAP32[$vararg_buffer1>>2] = $15;
   (_sqlite3_snprintf($13,$14,1200,$vararg_buffer1)|0);
  } else {
   $16 = HEAP32[((37192 + 40|0))>>2]|0;
   $17 = HEAP32[$4>>2]|0;
   $18 = HEAP32[$3>>2]|0;
   $19 = (($18) - 1)|0;
   $20 = (FUNCTION_TABLE_iii[$16 & 63]($17,$19)|0);
   $21 = ($20|0)==(0|0);
   if (!($21)) {
    $25 = HEAP32[$4>>2]|0;
    $26 = (_strlen(($25|0))|0);
    HEAP32[$nCwd>>2] = $26;
    $27 = HEAP32[$3>>2]|0;
    $28 = HEAP32[$nCwd>>2]|0;
    $29 = (($27) - ($28))|0;
    $30 = HEAP32[$nCwd>>2]|0;
    $31 = HEAP32[$4>>2]|0;
    $32 = (($31) + ($30)|0);
    $33 = HEAP32[$2>>2]|0;
    HEAP32[$vararg_buffer1>>2] = $33;
    (_sqlite3_snprintf($29,$32,38120,$vararg_buffer1)|0);
    break;
   }
   $22 = (_sqlite3CantopenError(30831)|0);
   $23 = HEAP32[$2>>2]|0;
   $24 = (_unixLogErrorAtLine($22,37544,$23,30831)|0);
   HEAP32[$0>>2] = $24;
   $34 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($34|0);
  }
 } while(0);
 HEAP32[$0>>2] = 0;
 $34 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($34|0);
}
function _unixRandomness($NotUsed,$nBuf,$zBuf) {
 $NotUsed = $NotUsed|0;
 $nBuf = $nBuf|0;
 $zBuf = $zBuf|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $3 = 0, $4 = 0, $5 = 0;
 var $6 = 0, $7 = 0, $8 = 0, $9 = 0, $fd = 0, $got = 0, $t = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 20|0;
 $1 = sp + 16|0;
 $2 = sp + 12|0;
 $fd = sp + 8|0;
 $got = sp + 4|0;
 $t = sp;
 HEAP32[$0>>2] = $NotUsed;
 HEAP32[$1>>2] = $nBuf;
 HEAP32[$2>>2] = $zBuf;
 $3 = HEAP32[$2>>2]|0;
 $4 = HEAP32[$1>>2]|0;
 _memset(($3|0),0,($4|0))|0;
 $5 = (_getpid()|0);
 HEAP32[38096>>2] = $5;
 $6 = (_robust_open(38104,0,0)|0);
 HEAP32[$fd>>2] = $6;
 $7 = HEAP32[$fd>>2]|0;
 $8 = ($7|0)<(0);
 if ($8) {
  (_time(($t|0))|0);
  $9 = HEAP32[$2>>2]|0;
  ;HEAP8[$9+0>>0]=HEAP8[$t+0>>0]|0;HEAP8[$9+1>>0]=HEAP8[$t+1>>0]|0;HEAP8[$9+2>>0]=HEAP8[$t+2>>0]|0;HEAP8[$9+3>>0]=HEAP8[$t+3>>0]|0;
  $10 = HEAP32[$2>>2]|0;
  $11 = (($10) + 4|0);
  ;HEAP8[$11+0>>0]=HEAP8[38096+0>>0]|0;HEAP8[$11+1>>0]=HEAP8[38096+1>>0]|0;HEAP8[$11+2>>0]=HEAP8[38096+2>>0]|0;HEAP8[$11+3>>0]=HEAP8[38096+3>>0]|0;
  HEAP32[$1>>2] = 8;
  $23 = HEAP32[$1>>2]|0;
  STACKTOP = sp;return ($23|0);
 }
 while(1) {
  $12 = HEAP32[((37192 + 100|0))>>2]|0;
  $13 = HEAP32[$fd>>2]|0;
  $14 = HEAP32[$2>>2]|0;
  $15 = HEAP32[$1>>2]|0;
  $16 = (FUNCTION_TABLE_iiii[$12 & 31]($13,$14,$15)|0);
  HEAP32[$got>>2] = $16;
  $17 = HEAP32[$got>>2]|0;
  $18 = ($17|0)<(0);
  if (!($18)) {
   break;
  }
  $19 = (___errno_location()|0);
  $20 = HEAP32[$19>>2]|0;
  $21 = ($20|0)==(4);
  if (!($21)) {
   break;
  }
 }
 $22 = HEAP32[$fd>>2]|0;
 _robust_close(0,$22,30936);
 $23 = HEAP32[$1>>2]|0;
 STACKTOP = sp;return ($23|0);
}
function _unixSleep($NotUsed,$microseconds) {
 $NotUsed = $NotUsed|0;
 $microseconds = $microseconds|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $seconds = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $1 = sp + 4|0;
 $seconds = sp;
 HEAP32[$0>>2] = $NotUsed;
 HEAP32[$1>>2] = $microseconds;
 $2 = HEAP32[$1>>2]|0;
 $3 = (($2) + 999999)|0;
 $4 = (($3|0) / 1000000)&-1;
 HEAP32[$seconds>>2] = $4;
 $5 = HEAP32[$seconds>>2]|0;
 (_sleep(($5|0))|0);
 $6 = HEAP32[$seconds>>2]|0;
 $7 = ($6*1000000)|0;
 STACKTOP = sp;return ($7|0);
}
function _unixCurrentTime($NotUsed,$prNow) {
 $NotUsed = $NotUsed|0;
 $prNow = $prNow|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0.0, $14 = 0.0, $15 = 0, $16 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $i = 0, $rc = 0, label = 0;
 var sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 16|0;
 $1 = sp + 12|0;
 $i = sp;
 $rc = sp + 8|0;
 HEAP32[$0>>2] = $NotUsed;
 HEAP32[$1>>2] = $prNow;
 $2 = $i;
 $3 = $2;
 HEAP32[$3>>2] = 0;
 $4 = (($2) + 4)|0;
 $5 = $4;
 HEAP32[$5>>2] = 0;
 $6 = (_unixCurrentTimeInt64(0,$i)|0);
 HEAP32[$rc>>2] = $6;
 $7 = $i;
 $8 = $7;
 $9 = HEAP32[$8>>2]|0;
 $10 = (($7) + 4)|0;
 $11 = $10;
 $12 = HEAP32[$11>>2]|0;
 $13 = (+($9>>>0)) + (4294967296.0*(+($12|0)));
 $14 = $13 / 8.64E+7;
 $15 = HEAP32[$1>>2]|0;
 HEAPF64[$15>>3] = $14;
 $16 = HEAP32[$rc>>2]|0;
 STACKTOP = sp;return ($16|0);
}
function _unixGetLastError($NotUsed,$NotUsed2,$NotUsed3) {
 $NotUsed = $NotUsed|0;
 $NotUsed2 = $NotUsed2|0;
 $NotUsed3 = $NotUsed3|0;
 var $0 = 0, $1 = 0, $2 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $1 = sp + 4|0;
 $2 = sp;
 HEAP32[$0>>2] = $NotUsed;
 HEAP32[$1>>2] = $NotUsed2;
 HEAP32[$2>>2] = $NotUsed3;
 STACKTOP = sp;return 0;
}
function _unixCurrentTimeInt64($NotUsed,$piNow) {
 $NotUsed = $NotUsed|0;
 $piNow = $piNow|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $3 = 0, $4 = 0, $5 = 0;
 var $6 = 0, $7 = 0, $8 = 0, $9 = 0, $rc = 0, $sNow = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 16|0;
 $1 = sp + 12|0;
 $rc = sp + 8|0;
 $sNow = sp;
 HEAP32[$0>>2] = $NotUsed;
 HEAP32[$1>>2] = $piNow;
 HEAP32[$rc>>2] = 0;
 $2 = (_gettimeofday(($sNow|0),(0|0))|0);
 $3 = ($2|0)==(0);
 if ($3) {
  $4 = HEAP32[$sNow>>2]|0;
  $5 = ($4|0)<(0);
  $6 = $5 << 31 >> 31;
  $7 = (___muldi3(1000,0,($4|0),($6|0))|0);
  $8 = tempRet0;
  $9 = (_i64Add(1045635584,49096,($7|0),($8|0))|0);
  $10 = tempRet0;
  $11 = (($sNow) + 4|0);
  $12 = HEAP32[$11>>2]|0;
  $13 = (($12|0) / 1000)&-1;
  $14 = ($13|0)<(0);
  $15 = $14 << 31 >> 31;
  $16 = (_i64Add(($9|0),($10|0),($13|0),($15|0))|0);
  $17 = tempRet0;
  $18 = HEAP32[$1>>2]|0;
  $19 = $18;
  $20 = $19;
  HEAP32[$20>>2] = $16;
  $21 = (($19) + 4)|0;
  $22 = $21;
  HEAP32[$22>>2] = $17;
  $23 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($23|0);
 } else {
  HEAP32[$rc>>2] = 1;
  $23 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($23|0);
 }
 return 0|0;
}
function _unixSetSystemCall($pNotUsed,$zName,$pNewFunc) {
 $pNotUsed = $pNotUsed|0;
 $zName = $zName|0;
 $pNewFunc = $pNewFunc|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $i = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 16|0;
 $1 = sp + 12|0;
 $2 = sp + 8|0;
 $i = sp + 4|0;
 $rc = sp;
 HEAP32[$0>>2] = $pNotUsed;
 HEAP32[$1>>2] = $zName;
 HEAP32[$2>>2] = $pNewFunc;
 HEAP32[$rc>>2] = 12;
 $3 = HEAP32[$1>>2]|0;
 $4 = ($3|0)==(0|0);
 if ($4) {
  HEAP32[$rc>>2] = 0;
  HEAP32[$i>>2] = 0;
  while(1) {
   $5 = HEAP32[$i>>2]|0;
   $6 = ($5>>>0)<(25);
   if (!($6)) {
    break;
   }
   $7 = HEAP32[$i>>2]|0;
   $8 = (37192 + (($7*12)|0)|0);
   $9 = (($8) + 8|0);
   $10 = HEAP32[$9>>2]|0;
   $11 = ($10|0)!=(0|0);
   if ($11) {
    $12 = HEAP32[$i>>2]|0;
    $13 = (37192 + (($12*12)|0)|0);
    $14 = (($13) + 8|0);
    $15 = HEAP32[$14>>2]|0;
    $16 = HEAP32[$i>>2]|0;
    $17 = (37192 + (($16*12)|0)|0);
    $18 = (($17) + 4|0);
    HEAP32[$18>>2] = $15;
   }
   $19 = HEAP32[$i>>2]|0;
   $20 = (($19) + 1)|0;
   HEAP32[$i>>2] = $20;
  }
  $52 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($52|0);
 }
 HEAP32[$i>>2] = 0;
 while(1) {
  $21 = HEAP32[$i>>2]|0;
  $22 = ($21>>>0)<(25);
  if (!($22)) {
   label = 16;
   break;
  }
  $23 = HEAP32[$1>>2]|0;
  $24 = HEAP32[$i>>2]|0;
  $25 = (37192 + (($24*12)|0)|0);
  $26 = HEAP32[$25>>2]|0;
  $27 = (_strcmp($23,$26)|0);
  $28 = ($27|0)==(0);
  $29 = HEAP32[$i>>2]|0;
  if ($28) {
   break;
  }
  $51 = (($29) + 1)|0;
  HEAP32[$i>>2] = $51;
 }
 if ((label|0) == 16) {
  $52 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($52|0);
 }
 $30 = (37192 + (($29*12)|0)|0);
 $31 = (($30) + 8|0);
 $32 = HEAP32[$31>>2]|0;
 $33 = ($32|0)==(0|0);
 if ($33) {
  $34 = HEAP32[$i>>2]|0;
  $35 = (37192 + (($34*12)|0)|0);
  $36 = (($35) + 4|0);
  $37 = HEAP32[$36>>2]|0;
  $38 = HEAP32[$i>>2]|0;
  $39 = (37192 + (($38*12)|0)|0);
  $40 = (($39) + 8|0);
  HEAP32[$40>>2] = $37;
 }
 HEAP32[$rc>>2] = 0;
 $41 = HEAP32[$2>>2]|0;
 $42 = ($41|0)==(0|0);
 if ($42) {
  $43 = HEAP32[$i>>2]|0;
  $44 = (37192 + (($43*12)|0)|0);
  $45 = (($44) + 8|0);
  $46 = HEAP32[$45>>2]|0;
  HEAP32[$2>>2] = $46;
 }
 $47 = HEAP32[$2>>2]|0;
 $48 = HEAP32[$i>>2]|0;
 $49 = (37192 + (($48*12)|0)|0);
 $50 = (($49) + 4|0);
 HEAP32[$50>>2] = $47;
 $52 = HEAP32[$rc>>2]|0;
 STACKTOP = sp;return ($52|0);
}
function _unixGetSystemCall($pNotUsed,$zName) {
 $pNotUsed = $pNotUsed|0;
 $zName = $zName|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $i = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $1 = sp + 8|0;
 $2 = sp + 4|0;
 $i = sp;
 HEAP32[$1>>2] = $pNotUsed;
 HEAP32[$2>>2] = $zName;
 HEAP32[$i>>2] = 0;
 while(1) {
  $3 = HEAP32[$i>>2]|0;
  $4 = ($3>>>0)<(25);
  if (!($4)) {
   label = 6;
   break;
  }
  $5 = HEAP32[$2>>2]|0;
  $6 = HEAP32[$i>>2]|0;
  $7 = (37192 + (($6*12)|0)|0);
  $8 = HEAP32[$7>>2]|0;
  $9 = (_strcmp($5,$8)|0);
  $10 = ($9|0)==(0);
  $11 = HEAP32[$i>>2]|0;
  if ($10) {
   label = 4;
   break;
  }
  $15 = (($11) + 1)|0;
  HEAP32[$i>>2] = $15;
 }
 if ((label|0) == 4) {
  $12 = (37192 + (($11*12)|0)|0);
  $13 = (($12) + 4|0);
  $14 = HEAP32[$13>>2]|0;
  HEAP32[$0>>2] = $14;
  $16 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($16|0);
 }
 else if ((label|0) == 6) {
  HEAP32[$0>>2] = 0;
  $16 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($16|0);
 }
 return 0|0;
}
function _unixNextSystemCall($p,$zName) {
 $p = $p|0;
 $zName = $zName|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $i = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $1 = sp + 8|0;
 $2 = sp + 4|0;
 $i = sp;
 HEAP32[$1>>2] = $p;
 HEAP32[$2>>2] = $zName;
 HEAP32[$i>>2] = -1;
 $3 = HEAP32[$2>>2]|0;
 $4 = ($3|0)!=(0|0);
 L1: do {
  if ($4) {
   HEAP32[$i>>2] = 0;
   while(1) {
    $5 = HEAP32[$i>>2]|0;
    $6 = ($5|0)<(24);
    if (!($6)) {
     break L1;
    }
    $7 = HEAP32[$2>>2]|0;
    $8 = HEAP32[$i>>2]|0;
    $9 = (37192 + (($8*12)|0)|0);
    $10 = HEAP32[$9>>2]|0;
    $11 = (_strcmp($7,$10)|0);
    $12 = ($11|0)==(0);
    if ($12) {
     break L1;
    }
    $13 = HEAP32[$i>>2]|0;
    $14 = (($13) + 1)|0;
    HEAP32[$i>>2] = $14;
   }
  }
 } while(0);
 $15 = HEAP32[$i>>2]|0;
 $16 = (($15) + 1)|0;
 HEAP32[$i>>2] = $16;
 while(1) {
  $17 = HEAP32[$i>>2]|0;
  $18 = ($17|0)<(25);
  if (!($18)) {
   label = 11;
   break;
  }
  $19 = HEAP32[$i>>2]|0;
  $20 = (37192 + (($19*12)|0)|0);
  $21 = (($20) + 4|0);
  $22 = HEAP32[$21>>2]|0;
  $23 = ($22|0)!=(0|0);
  $24 = HEAP32[$i>>2]|0;
  if ($23) {
   label = 9;
   break;
  }
  $27 = (($24) + 1)|0;
  HEAP32[$i>>2] = $27;
 }
 if ((label|0) == 9) {
  $25 = (37192 + (($24*12)|0)|0);
  $26 = HEAP32[$25>>2]|0;
  HEAP32[$0>>2] = $26;
  $28 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($28|0);
 }
 else if ((label|0) == 11) {
  HEAP32[$0>>2] = 0;
  $28 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($28|0);
 }
 return 0|0;
}
function _sqlite3ErrorWithMsg($db,$err_code,$zFormat,$varargs) {
 $db = $db|0;
 $err_code = $err_code|0;
 $zFormat = $zFormat|0;
 $varargs = $varargs|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $3 = 0, $4 = 0;
 var $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $ap = 0, $z = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 28|0;
 $1 = sp + 24|0;
 $2 = sp + 20|0;
 $z = sp + 16|0;
 $ap = sp;
 HEAP32[$0>>2] = $db;
 HEAP32[$1>>2] = $err_code;
 HEAP32[$2>>2] = $zFormat;
 $3 = HEAP32[$1>>2]|0;
 $4 = HEAP32[$0>>2]|0;
 $5 = (($4) + 52|0);
 HEAP32[$5>>2] = $3;
 $6 = HEAP32[$2>>2]|0;
 $7 = ($6|0)==(0|0);
 $8 = HEAP32[$0>>2]|0;
 if ($7) {
  $9 = HEAP32[$1>>2]|0;
  _sqlite3Error($8,$9);
  STACKTOP = sp;return;
 }
 $10 = (($8) + 232|0);
 $11 = HEAP32[$10>>2]|0;
 $12 = ($11|0)!=(0|0);
 if (!($12)) {
  $13 = HEAP32[$0>>2]|0;
  $14 = (_sqlite3ValueNew($13)|0);
  $15 = HEAP32[$0>>2]|0;
  $16 = (($15) + 232|0);
  HEAP32[$16>>2] = $14;
  $17 = ($14|0)!=(0|0);
  if (!($17)) {
   STACKTOP = sp;return;
  }
 }
 HEAP32[$ap>>2] = $varargs;
 $18 = HEAP32[$0>>2]|0;
 $19 = HEAP32[$2>>2]|0;
 $20 = (_sqlite3VMPrintf($18,$19,$ap)|0);
 HEAP32[$z>>2] = $20;
 $21 = HEAP32[$0>>2]|0;
 $22 = (($21) + 232|0);
 $23 = HEAP32[$22>>2]|0;
 $24 = HEAP32[$z>>2]|0;
 _sqlite3ValueSetStr($23,-1,$24,1,16);
 STACKTOP = sp;return;
}
function _sqlite3MallocZero($0,$1) {
 $0 = $0|0;
 $1 = $1|0;
 var $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0;
 var $8 = 0, $9 = 0, $p = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $2 = sp;
 $p = sp + 8|0;
 $3 = $2;
 $4 = $3;
 HEAP32[$4>>2] = $0;
 $5 = (($3) + 4)|0;
 $6 = $5;
 HEAP32[$6>>2] = $1;
 $7 = $2;
 $8 = $7;
 $9 = HEAP32[$8>>2]|0;
 $10 = (($7) + 4)|0;
 $11 = $10;
 $12 = HEAP32[$11>>2]|0;
 $13 = (_sqlite3Malloc($9,$12)|0);
 HEAP32[$p>>2] = $13;
 $14 = HEAP32[$p>>2]|0;
 $15 = ($14|0)!=(0|0);
 if (!($15)) {
  $23 = HEAP32[$p>>2]|0;
  STACKTOP = sp;return ($23|0);
 }
 $16 = HEAP32[$p>>2]|0;
 $17 = $2;
 $18 = $17;
 $19 = HEAP32[$18>>2]|0;
 $20 = (($17) + 4)|0;
 $21 = $20;
 $22 = HEAP32[$21>>2]|0;
 _memset(($16|0),0,($19|0))|0;
 $23 = HEAP32[$p>>2]|0;
 STACKTOP = sp;return ($23|0);
}
function _sqlite3Error($db,$err_code) {
 $db = $db|0;
 $err_code = $err_code|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $1 = sp;
 HEAP32[$0>>2] = $db;
 HEAP32[$1>>2] = $err_code;
 $2 = HEAP32[$1>>2]|0;
 $3 = HEAP32[$0>>2]|0;
 $4 = (($3) + 52|0);
 HEAP32[$4>>2] = $2;
 $5 = HEAP32[$0>>2]|0;
 $6 = (($5) + 232|0);
 $7 = HEAP32[$6>>2]|0;
 $8 = ($7|0)!=(0|0);
 if (!($8)) {
  STACKTOP = sp;return;
 }
 $9 = HEAP32[$0>>2]|0;
 $10 = (($9) + 232|0);
 $11 = HEAP32[$10>>2]|0;
 _sqlite3ValueSetNull($11);
 STACKTOP = sp;return;
}
function _sqlite3_backup_step($p,$nPage) {
 $p = $p|0;
 $nPage = $nPage|0;
 var $$ = 0, $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0;
 var $115 = 0, $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0;
 var $133 = 0, $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0;
 var $151 = 0, $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0;
 var $17 = 0, $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0;
 var $188 = 0, $189 = 0, $19 = 0, $190 = 0, $191 = 0, $192 = 0, $193 = 0, $194 = 0, $195 = 0, $196 = 0, $197 = 0, $198 = 0, $199 = 0, $2 = 0, $20 = 0, $200 = 0, $201 = 0, $202 = 0, $203 = 0, $204 = 0;
 var $205 = 0, $206 = 0, $207 = 0, $208 = 0, $209 = 0, $21 = 0, $210 = 0, $211 = 0, $212 = 0, $213 = 0, $214 = 0, $215 = 0, $216 = 0, $217 = 0, $218 = 0, $219 = 0, $22 = 0, $220 = 0, $221 = 0, $222 = 0;
 var $223 = 0, $224 = 0, $225 = 0, $226 = 0, $227 = 0, $228 = 0, $229 = 0, $23 = 0, $230 = 0, $231 = 0, $232 = 0, $233 = 0, $234 = 0, $235 = 0, $236 = 0, $237 = 0, $238 = 0, $239 = 0, $24 = 0, $240 = 0;
 var $241 = 0, $242 = 0, $243 = 0, $244 = 0, $245 = 0, $246 = 0, $247 = 0, $248 = 0, $249 = 0, $25 = 0, $250 = 0, $251 = 0, $252 = 0, $253 = 0, $254 = 0, $255 = 0, $256 = 0, $257 = 0, $258 = 0, $259 = 0;
 var $26 = 0, $260 = 0, $261 = 0, $262 = 0, $263 = 0, $264 = 0, $265 = 0, $266 = 0, $267 = 0, $268 = 0, $269 = 0, $27 = 0, $270 = 0, $271 = 0, $272 = 0, $273 = 0, $274 = 0, $275 = 0, $276 = 0, $277 = 0;
 var $278 = 0, $279 = 0, $28 = 0, $280 = 0, $281 = 0, $282 = 0, $283 = 0, $284 = 0, $285 = 0, $286 = 0, $287 = 0, $288 = 0, $289 = 0, $29 = 0, $290 = 0, $291 = 0, $292 = 0, $293 = 0, $294 = 0, $295 = 0;
 var $296 = 0, $297 = 0, $298 = 0, $299 = 0, $3 = 0, $30 = 0, $300 = 0, $301 = 0, $302 = 0, $303 = 0, $304 = 0, $305 = 0, $306 = 0, $307 = 0, $308 = 0, $309 = 0, $31 = 0, $310 = 0, $311 = 0, $312 = 0;
 var $313 = 0, $314 = 0, $315 = 0, $316 = 0, $317 = 0, $318 = 0, $319 = 0, $32 = 0, $320 = 0, $321 = 0, $322 = 0, $323 = 0, $324 = 0, $325 = 0, $326 = 0, $327 = 0, $328 = 0, $329 = 0, $33 = 0, $330 = 0;
 var $331 = 0, $332 = 0, $333 = 0, $334 = 0, $335 = 0, $336 = 0, $337 = 0, $338 = 0, $339 = 0, $34 = 0, $340 = 0, $341 = 0, $342 = 0, $343 = 0, $344 = 0, $345 = 0, $346 = 0, $347 = 0, $348 = 0, $349 = 0;
 var $35 = 0, $350 = 0, $351 = 0, $352 = 0, $353 = 0, $354 = 0, $355 = 0, $356 = 0, $357 = 0, $358 = 0, $359 = 0, $36 = 0, $360 = 0, $361 = 0, $362 = 0, $363 = 0, $364 = 0, $365 = 0, $366 = 0, $367 = 0;
 var $368 = 0, $369 = 0, $37 = 0, $370 = 0, $371 = 0, $372 = 0, $373 = 0, $374 = 0, $375 = 0, $376 = 0, $377 = 0, $378 = 0, $379 = 0, $38 = 0, $380 = 0, $381 = 0, $382 = 0, $383 = 0, $384 = 0, $385 = 0;
 var $386 = 0, $387 = 0, $388 = 0, $389 = 0, $39 = 0, $390 = 0, $391 = 0, $392 = 0, $393 = 0, $394 = 0, $395 = 0, $396 = 0, $397 = 0, $398 = 0, $399 = 0, $4 = 0, $40 = 0, $400 = 0, $401 = 0, $402 = 0;
 var $403 = 0, $404 = 0, $405 = 0, $406 = 0, $407 = 0, $408 = 0, $409 = 0, $41 = 0, $410 = 0, $411 = 0, $412 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0;
 var $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0;
 var $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0;
 var $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $bCloseTrans = 0, $destMode = 0, $iEnd = 0, $iOff = 0, $iPg = 0, $iSize = 0;
 var $iSrcPg = 0, $iSrcPg2 = 0, $ii = 0, $nDestTruncate = 0, $nDstPage = 0, $nSrcPage = 0, $or$cond = 0, $or$cond3 = 0, $or$cond5 = 0, $pDestPager = 0, $pFile = 0, $pPg = 0, $pSrcPager = 0, $pSrcPg = 0, $pSrcPg1 = 0, $pgszDest = 0, $pgszSrc = 0, $ratio = 0, $rc = 0, $zData = 0;
 var label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 112|0;
 $0 = sp + 108|0;
 $1 = sp + 104|0;
 $rc = sp + 100|0;
 $destMode = sp + 96|0;
 $pgszSrc = sp + 92|0;
 $pgszDest = sp + 88|0;
 $pSrcPager = sp + 84|0;
 $pDestPager = sp + 80|0;
 $ii = sp + 76|0;
 $nSrcPage = sp + 72|0;
 $bCloseTrans = sp + 68|0;
 $iSrcPg = sp + 64|0;
 $pSrcPg = sp + 60|0;
 $nDestTruncate = sp + 56|0;
 $ratio = sp + 52|0;
 $iSize = sp + 16|0;
 $pFile = sp + 48|0;
 $iPg = sp + 44|0;
 $nDstPage = sp + 40|0;
 $iOff = sp + 8|0;
 $iEnd = sp;
 $pPg = sp + 36|0;
 $pSrcPg1 = sp + 32|0;
 $iSrcPg2 = sp + 28|0;
 $zData = sp + 24|0;
 HEAP32[$0>>2] = $p;
 HEAP32[$1>>2] = $nPage;
 HEAP32[$pgszSrc>>2] = 0;
 HEAP32[$pgszDest>>2] = 0;
 $2 = HEAP32[$0>>2]|0;
 $3 = (($2) + 24|0);
 $4 = HEAP32[$3>>2]|0;
 _sqlite3BtreeEnter($4);
 $5 = HEAP32[$0>>2]|0;
 $6 = (($5) + 28|0);
 $7 = HEAP32[$6>>2]|0;
 HEAP32[$rc>>2] = $7;
 $8 = HEAP32[$rc>>2]|0;
 $9 = (_isFatalError($8)|0);
 $10 = ($9|0)!=(0);
 if ($10) {
  $412 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($412|0);
 }
 $11 = HEAP32[$0>>2]|0;
 $12 = (($11) + 24|0);
 $13 = HEAP32[$12>>2]|0;
 $14 = (_sqlite3BtreePager($13)|0);
 HEAP32[$pSrcPager>>2] = $14;
 $15 = HEAP32[$0>>2]|0;
 $16 = (($15) + 4|0);
 $17 = HEAP32[$16>>2]|0;
 $18 = (_sqlite3BtreePager($17)|0);
 HEAP32[$pDestPager>>2] = $18;
 HEAP32[$nSrcPage>>2] = -1;
 HEAP32[$bCloseTrans>>2] = 0;
 $19 = HEAP32[$0>>2]|0;
 $20 = HEAP32[$19>>2]|0;
 $21 = ($20|0)!=(0|0);
 if ($21) {
  $22 = HEAP32[$0>>2]|0;
  $23 = (($22) + 24|0);
  $24 = HEAP32[$23>>2]|0;
  $25 = (($24) + 4|0);
  $26 = HEAP32[$25>>2]|0;
  $27 = (($26) + 20|0);
  $28 = HEAP8[$27>>0]|0;
  $29 = $28&255;
  $30 = ($29|0)==(2);
  if ($30) {
   HEAP32[$rc>>2] = 5;
  } else {
   label = 5;
  }
 } else {
  label = 5;
 }
 if ((label|0) == 5) {
  HEAP32[$rc>>2] = 0;
 }
 $31 = HEAP32[$rc>>2]|0;
 $32 = (0)==($31|0);
 if ($32) {
  $33 = HEAP32[$0>>2]|0;
  $34 = (($33) + 12|0);
  $35 = HEAP32[$34>>2]|0;
  $36 = ($35|0)==(0);
  if ($36) {
   $37 = HEAP32[$0>>2]|0;
   $38 = (($37) + 4|0);
   $39 = HEAP32[$38>>2]|0;
   $40 = (_sqlite3BtreeBeginTrans($39,2)|0);
   HEAP32[$rc>>2] = $40;
   $41 = (0)==($40|0);
   if ($41) {
    $42 = HEAP32[$0>>2]|0;
    $43 = (($42) + 12|0);
    HEAP32[$43>>2] = 1;
    $44 = HEAP32[$0>>2]|0;
    $45 = (($44) + 4|0);
    $46 = HEAP32[$45>>2]|0;
    $47 = HEAP32[$0>>2]|0;
    $48 = (($47) + 8|0);
    _sqlite3BtreeGetMeta($46,1,$48);
   }
  }
 }
 $49 = HEAP32[$rc>>2]|0;
 $50 = ($49|0)==(0);
 if ($50) {
  $51 = HEAP32[$0>>2]|0;
  $52 = (($51) + 24|0);
  $53 = HEAP32[$52>>2]|0;
  $54 = (_sqlite3BtreeIsInReadTrans($53)|0);
  $55 = (0)==($54|0);
  if ($55) {
   $56 = HEAP32[$0>>2]|0;
   $57 = (($56) + 24|0);
   $58 = HEAP32[$57>>2]|0;
   $59 = (_sqlite3BtreeBeginTrans($58,0)|0);
   HEAP32[$rc>>2] = $59;
   HEAP32[$bCloseTrans>>2] = 1;
  }
 }
 $60 = HEAP32[$0>>2]|0;
 $61 = (($60) + 24|0);
 $62 = HEAP32[$61>>2]|0;
 $63 = (_sqlite3BtreeGetPageSize($62)|0);
 HEAP32[$pgszSrc>>2] = $63;
 $64 = HEAP32[$0>>2]|0;
 $65 = (($64) + 4|0);
 $66 = HEAP32[$65>>2]|0;
 $67 = (_sqlite3BtreeGetPageSize($66)|0);
 HEAP32[$pgszDest>>2] = $67;
 $68 = HEAP32[$0>>2]|0;
 $69 = (($68) + 4|0);
 $70 = HEAP32[$69>>2]|0;
 $71 = (_sqlite3BtreePager($70)|0);
 $72 = (_sqlite3PagerGetJournalMode($71)|0);
 HEAP32[$destMode>>2] = $72;
 $73 = HEAP32[$rc>>2]|0;
 $74 = (0)==($73|0);
 $75 = HEAP32[$destMode>>2]|0;
 $76 = ($75|0)==(5);
 $or$cond = $74 & $76;
 if ($or$cond) {
  $77 = HEAP32[$pgszSrc>>2]|0;
  $78 = HEAP32[$pgszDest>>2]|0;
  $79 = ($77|0)!=($78|0);
  if ($79) {
   HEAP32[$rc>>2] = 8;
  }
 }
 $80 = HEAP32[$0>>2]|0;
 $81 = (($80) + 24|0);
 $82 = HEAP32[$81>>2]|0;
 $83 = (_sqlite3BtreeLastPage($82)|0);
 HEAP32[$nSrcPage>>2] = $83;
 HEAP32[$ii>>2] = 0;
 while(1) {
  $84 = HEAP32[$1>>2]|0;
  $85 = ($84|0)<(0);
  if (!($85)) {
   $86 = HEAP32[$ii>>2]|0;
   $87 = HEAP32[$1>>2]|0;
   $88 = ($86|0)<($87|0);
   if (!($88)) {
    break;
   }
  }
  $89 = HEAP32[$0>>2]|0;
  $90 = (($89) + 16|0);
  $91 = HEAP32[$90>>2]|0;
  $92 = HEAP32[$nSrcPage>>2]|0;
  $93 = ($91>>>0)>($92>>>0);
  $94 = HEAP32[$rc>>2]|0;
  $95 = ($94|0)!=(0);
  $or$cond3 = $93 | $95;
  if ($or$cond3) {
   break;
  }
  $96 = HEAP32[$0>>2]|0;
  $97 = (($96) + 16|0);
  $98 = HEAP32[$97>>2]|0;
  HEAP32[$iSrcPg>>2] = $98;
  $99 = HEAP32[$iSrcPg>>2]|0;
  $100 = HEAP32[1128>>2]|0;
  $101 = HEAP32[$0>>2]|0;
  $102 = (($101) + 24|0);
  $103 = HEAP32[$102>>2]|0;
  $104 = (($103) + 4|0);
  $105 = HEAP32[$104>>2]|0;
  $106 = (($105) + 32|0);
  $107 = HEAP32[$106>>2]|0;
  $108 = (($100>>>0) / ($107>>>0))&-1;
  $109 = (($108) + 1)|0;
  $110 = ($99|0)!=($109|0);
  if ($110) {
   $111 = HEAP32[$pSrcPager>>2]|0;
   $112 = HEAP32[$iSrcPg>>2]|0;
   $113 = (_sqlite3PagerAcquire($111,$112,$pSrcPg,2)|0);
   HEAP32[$rc>>2] = $113;
   $114 = HEAP32[$rc>>2]|0;
   $115 = ($114|0)==(0);
   if ($115) {
    $116 = HEAP32[$0>>2]|0;
    $117 = HEAP32[$iSrcPg>>2]|0;
    $118 = HEAP32[$pSrcPg>>2]|0;
    $119 = (_sqlite3PagerGetData($118)|0);
    $120 = (_backupOnePage($116,$117,$119,0)|0);
    HEAP32[$rc>>2] = $120;
    $121 = HEAP32[$pSrcPg>>2]|0;
    _sqlite3PagerUnref($121);
   }
  }
  $122 = HEAP32[$0>>2]|0;
  $123 = (($122) + 16|0);
  $124 = HEAP32[$123>>2]|0;
  $125 = (($124) + 1)|0;
  HEAP32[$123>>2] = $125;
  $126 = HEAP32[$ii>>2]|0;
  $127 = (($126) + 1)|0;
  HEAP32[$ii>>2] = $127;
 }
 $128 = HEAP32[$rc>>2]|0;
 $129 = ($128|0)==(0);
 do {
  if ($129) {
   $130 = HEAP32[$nSrcPage>>2]|0;
   $131 = HEAP32[$0>>2]|0;
   $132 = (($131) + 36|0);
   HEAP32[$132>>2] = $130;
   $133 = HEAP32[$nSrcPage>>2]|0;
   $134 = (($133) + 1)|0;
   $135 = HEAP32[$0>>2]|0;
   $136 = (($135) + 16|0);
   $137 = HEAP32[$136>>2]|0;
   $138 = (($134) - ($137))|0;
   $139 = HEAP32[$0>>2]|0;
   $140 = (($139) + 32|0);
   HEAP32[$140>>2] = $138;
   $141 = HEAP32[$0>>2]|0;
   $142 = (($141) + 16|0);
   $143 = HEAP32[$142>>2]|0;
   $144 = HEAP32[$nSrcPage>>2]|0;
   $145 = ($143>>>0)>($144>>>0);
   if ($145) {
    HEAP32[$rc>>2] = 101;
    break;
   }
   $146 = HEAP32[$0>>2]|0;
   $147 = (($146) + 40|0);
   $148 = HEAP32[$147>>2]|0;
   $149 = ($148|0)!=(0);
   if (!($149)) {
    $150 = HEAP32[$0>>2]|0;
    _attachBackupObject($150);
   }
  }
 } while(0);
 $151 = HEAP32[$rc>>2]|0;
 $152 = ($151|0)==(101);
 if ($152) {
  $153 = HEAP32[$nSrcPage>>2]|0;
  $154 = ($153|0)==(0);
  if ($154) {
   $155 = HEAP32[$0>>2]|0;
   $156 = (($155) + 4|0);
   $157 = HEAP32[$156>>2]|0;
   $158 = (_sqlite3BtreeNewDb($157)|0);
   HEAP32[$rc>>2] = $158;
   HEAP32[$nSrcPage>>2] = 1;
  }
  $159 = HEAP32[$rc>>2]|0;
  $160 = ($159|0)==(0);
  $161 = HEAP32[$rc>>2]|0;
  $162 = ($161|0)==(101);
  $or$cond5 = $160 | $162;
  if ($or$cond5) {
   $163 = HEAP32[$0>>2]|0;
   $164 = (($163) + 4|0);
   $165 = HEAP32[$164>>2]|0;
   $166 = HEAP32[$0>>2]|0;
   $167 = (($166) + 8|0);
   $168 = HEAP32[$167>>2]|0;
   $169 = (($168) + 1)|0;
   $170 = (_sqlite3BtreeUpdateMeta($165,1,$169)|0);
   HEAP32[$rc>>2] = $170;
  }
  $171 = HEAP32[$rc>>2]|0;
  $172 = ($171|0)==(0);
  if ($172) {
   $173 = HEAP32[$0>>2]|0;
   $174 = HEAP32[$173>>2]|0;
   $175 = ($174|0)!=(0|0);
   if ($175) {
    $176 = HEAP32[$0>>2]|0;
    $177 = HEAP32[$176>>2]|0;
    _sqlite3ResetAllSchemasOfConnection($177);
   }
   $178 = HEAP32[$destMode>>2]|0;
   $179 = ($178|0)==(5);
   if ($179) {
    $180 = HEAP32[$0>>2]|0;
    $181 = (($180) + 4|0);
    $182 = HEAP32[$181>>2]|0;
    $183 = (_sqlite3BtreeSetVersion($182,2)|0);
    HEAP32[$rc>>2] = $183;
   }
  }
  $184 = HEAP32[$rc>>2]|0;
  $185 = ($184|0)==(0);
  if ($185) {
   $186 = HEAP32[$pgszSrc>>2]|0;
   $187 = HEAP32[$pgszDest>>2]|0;
   $188 = ($186|0)<($187|0);
   if ($188) {
    $189 = HEAP32[$pgszDest>>2]|0;
    $190 = HEAP32[$pgszSrc>>2]|0;
    $191 = (($189|0) / ($190|0))&-1;
    HEAP32[$ratio>>2] = $191;
    $192 = HEAP32[$nSrcPage>>2]|0;
    $193 = HEAP32[$ratio>>2]|0;
    $194 = (($192) + ($193))|0;
    $195 = (($194) - 1)|0;
    $196 = HEAP32[$ratio>>2]|0;
    $197 = (($195|0) / ($196|0))&-1;
    HEAP32[$nDestTruncate>>2] = $197;
    $198 = HEAP32[$nDestTruncate>>2]|0;
    $199 = HEAP32[1128>>2]|0;
    $200 = HEAP32[$0>>2]|0;
    $201 = (($200) + 4|0);
    $202 = HEAP32[$201>>2]|0;
    $203 = (($202) + 4|0);
    $204 = HEAP32[$203>>2]|0;
    $205 = (($204) + 32|0);
    $206 = HEAP32[$205>>2]|0;
    $207 = (($199>>>0) / ($206>>>0))&-1;
    $208 = (($207) + 1)|0;
    $209 = ($198|0)==($208|0);
    if ($209) {
     $210 = HEAP32[$nDestTruncate>>2]|0;
     $211 = (($210) + -1)|0;
     HEAP32[$nDestTruncate>>2] = $211;
    }
   } else {
    $212 = HEAP32[$nSrcPage>>2]|0;
    $213 = HEAP32[$pgszSrc>>2]|0;
    $214 = HEAP32[$pgszDest>>2]|0;
    $215 = (($213|0) / ($214|0))&-1;
    $216 = Math_imul($212, $215)|0;
    HEAP32[$nDestTruncate>>2] = $216;
   }
   $217 = HEAP32[$pgszSrc>>2]|0;
   $218 = HEAP32[$pgszDest>>2]|0;
   $219 = ($217|0)<($218|0);
   if ($219) {
    $220 = HEAP32[$pgszSrc>>2]|0;
    $221 = ($220|0)<(0);
    $222 = $221 << 31 >> 31;
    $223 = HEAP32[$nSrcPage>>2]|0;
    $224 = ($223|0)<(0);
    $225 = $224 << 31 >> 31;
    $226 = (___muldi3(($220|0),($222|0),($223|0),($225|0))|0);
    $227 = tempRet0;
    $228 = $iSize;
    $229 = $228;
    HEAP32[$229>>2] = $226;
    $230 = (($228) + 4)|0;
    $231 = $230;
    HEAP32[$231>>2] = $227;
    $232 = HEAP32[$pDestPager>>2]|0;
    $233 = (_sqlite3PagerFile($232)|0);
    HEAP32[$pFile>>2] = $233;
    $234 = HEAP32[$pDestPager>>2]|0;
    _sqlite3PagerPagecount($234,$nDstPage);
    $235 = HEAP32[$nDestTruncate>>2]|0;
    HEAP32[$iPg>>2] = $235;
    while(1) {
     $236 = HEAP32[$rc>>2]|0;
     $237 = ($236|0)==(0);
     if (!($237)) {
      break;
     }
     $238 = HEAP32[$iPg>>2]|0;
     $239 = HEAP32[$nDstPage>>2]|0;
     $240 = ($238>>>0)<=($239>>>0);
     if (!($240)) {
      break;
     }
     $241 = HEAP32[$iPg>>2]|0;
     $242 = HEAP32[1128>>2]|0;
     $243 = HEAP32[$0>>2]|0;
     $244 = (($243) + 4|0);
     $245 = HEAP32[$244>>2]|0;
     $246 = (($245) + 4|0);
     $247 = HEAP32[$246>>2]|0;
     $248 = (($247) + 32|0);
     $249 = HEAP32[$248>>2]|0;
     $250 = (($242>>>0) / ($249>>>0))&-1;
     $251 = (($250) + 1)|0;
     $252 = ($241|0)!=($251|0);
     do {
      if ($252) {
       $253 = HEAP32[$pDestPager>>2]|0;
       $254 = HEAP32[$iPg>>2]|0;
       $255 = (_sqlite3PagerAcquire($253,$254,$pPg,0)|0);
       HEAP32[$rc>>2] = $255;
       $256 = HEAP32[$rc>>2]|0;
       $257 = ($256|0)==(0);
       if (!($257)) {
        break;
       }
       $258 = HEAP32[$pPg>>2]|0;
       $259 = (_sqlite3PagerWrite($258)|0);
       HEAP32[$rc>>2] = $259;
       $260 = HEAP32[$pPg>>2]|0;
       _sqlite3PagerUnref($260);
      }
     } while(0);
     $261 = HEAP32[$iPg>>2]|0;
     $262 = (($261) + 1)|0;
     HEAP32[$iPg>>2] = $262;
    }
    $263 = HEAP32[$rc>>2]|0;
    $264 = ($263|0)==(0);
    if ($264) {
     $265 = HEAP32[$pDestPager>>2]|0;
     $266 = (_sqlite3PagerCommitPhaseOne($265,0,1)|0);
     HEAP32[$rc>>2] = $266;
    }
    $267 = HEAP32[1128>>2]|0;
    $268 = HEAP32[$pgszDest>>2]|0;
    $269 = (($267) + ($268))|0;
    $270 = ($269|0)<(0);
    $271 = $270 << 31 >> 31;
    $272 = $iSize;
    $273 = $272;
    $274 = HEAP32[$273>>2]|0;
    $275 = (($272) + 4)|0;
    $276 = $275;
    $277 = HEAP32[$276>>2]|0;
    $278 = ($271|0)<($277|0);
    $279 = ($269>>>0)<($274>>>0);
    $280 = ($271|0)==($277|0);
    $281 = $280 & $279;
    $282 = $278 | $281;
    if ($282) {
     $283 = HEAP32[1128>>2]|0;
     $284 = HEAP32[$pgszDest>>2]|0;
     $285 = (($283) + ($284))|0;
     $286 = ($285|0)<(0);
     $287 = $286 << 31 >> 31;
     $296 = $285;$299 = $287;
    } else {
     $288 = $iSize;
     $289 = $288;
     $290 = HEAP32[$289>>2]|0;
     $291 = (($288) + 4)|0;
     $292 = $291;
     $293 = HEAP32[$292>>2]|0;
     $296 = $290;$299 = $293;
    }
    $294 = $iEnd;
    $295 = $294;
    HEAP32[$295>>2] = $296;
    $297 = (($294) + 4)|0;
    $298 = $297;
    HEAP32[$298>>2] = $299;
    $300 = HEAP32[1128>>2]|0;
    $301 = HEAP32[$pgszSrc>>2]|0;
    $302 = (($300) + ($301))|0;
    $303 = ($302|0)<(0);
    $304 = $303 << 31 >> 31;
    $305 = $iOff;
    $306 = $305;
    HEAP32[$306>>2] = $302;
    $307 = (($305) + 4)|0;
    $308 = $307;
    HEAP32[$308>>2] = $304;
    while(1) {
     $309 = HEAP32[$rc>>2]|0;
     $310 = ($309|0)==(0);
     if (!($310)) {
      break;
     }
     $311 = $iOff;
     $312 = $311;
     $313 = HEAP32[$312>>2]|0;
     $314 = (($311) + 4)|0;
     $315 = $314;
     $316 = HEAP32[$315>>2]|0;
     $317 = $iEnd;
     $318 = $317;
     $319 = HEAP32[$318>>2]|0;
     $320 = (($317) + 4)|0;
     $321 = $320;
     $322 = HEAP32[$321>>2]|0;
     $323 = ($316|0)<($322|0);
     $324 = ($313>>>0)<($319>>>0);
     $325 = ($316|0)==($322|0);
     $326 = $325 & $324;
     $327 = $323 | $326;
     if (!($327)) {
      break;
     }
     HEAP32[$pSrcPg1>>2] = 0;
     $328 = $iOff;
     $329 = $328;
     $330 = HEAP32[$329>>2]|0;
     $331 = (($328) + 4)|0;
     $332 = $331;
     $333 = HEAP32[$332>>2]|0;
     $334 = HEAP32[$pgszSrc>>2]|0;
     $335 = ($334|0)<(0);
     $336 = $335 << 31 >> 31;
     $337 = (___divdi3(($330|0),($333|0),($334|0),($336|0))|0);
     $338 = tempRet0;
     $339 = (_i64Add(($337|0),($338|0),1,0)|0);
     $340 = tempRet0;
     HEAP32[$iSrcPg2>>2] = $339;
     $341 = HEAP32[$pSrcPager>>2]|0;
     $342 = HEAP32[$iSrcPg2>>2]|0;
     $343 = (_sqlite3PagerAcquire($341,$342,$pSrcPg1,0)|0);
     HEAP32[$rc>>2] = $343;
     $344 = HEAP32[$rc>>2]|0;
     $345 = ($344|0)==(0);
     if ($345) {
      $346 = HEAP32[$pSrcPg1>>2]|0;
      $347 = (_sqlite3PagerGetData($346)|0);
      HEAP32[$zData>>2] = $347;
      $348 = HEAP32[$pFile>>2]|0;
      $349 = HEAP32[$zData>>2]|0;
      $350 = HEAP32[$pgszSrc>>2]|0;
      $351 = $iOff;
      $352 = $351;
      $353 = HEAP32[$352>>2]|0;
      $354 = (($351) + 4)|0;
      $355 = $354;
      $356 = HEAP32[$355>>2]|0;
      $357 = (_sqlite3OsWrite($348,$349,$350,$353,$356)|0);
      HEAP32[$rc>>2] = $357;
     }
     $358 = HEAP32[$pSrcPg1>>2]|0;
     _sqlite3PagerUnref($358);
     $359 = HEAP32[$pgszSrc>>2]|0;
     $360 = ($359|0)<(0);
     $361 = $360 << 31 >> 31;
     $362 = $iOff;
     $363 = $362;
     $364 = HEAP32[$363>>2]|0;
     $365 = (($362) + 4)|0;
     $366 = $365;
     $367 = HEAP32[$366>>2]|0;
     $368 = (_i64Add(($364|0),($367|0),($359|0),($361|0))|0);
     $369 = tempRet0;
     $370 = $iOff;
     $371 = $370;
     HEAP32[$371>>2] = $368;
     $372 = (($370) + 4)|0;
     $373 = $372;
     HEAP32[$373>>2] = $369;
    }
    $374 = HEAP32[$rc>>2]|0;
    $375 = ($374|0)==(0);
    if ($375) {
     $376 = HEAP32[$pFile>>2]|0;
     $377 = $iSize;
     $378 = $377;
     $379 = HEAP32[$378>>2]|0;
     $380 = (($377) + 4)|0;
     $381 = $380;
     $382 = HEAP32[$381>>2]|0;
     $383 = (_backupTruncateFile($376,$379,$382)|0);
     HEAP32[$rc>>2] = $383;
    }
    $384 = HEAP32[$rc>>2]|0;
    $385 = ($384|0)==(0);
    if ($385) {
     $386 = HEAP32[$pDestPager>>2]|0;
     $387 = (_sqlite3PagerSync($386,0)|0);
     HEAP32[$rc>>2] = $387;
    }
   } else {
    $388 = HEAP32[$pDestPager>>2]|0;
    $389 = HEAP32[$nDestTruncate>>2]|0;
    _sqlite3PagerTruncateImage($388,$389);
    $390 = HEAP32[$pDestPager>>2]|0;
    $391 = (_sqlite3PagerCommitPhaseOne($390,0,0)|0);
    HEAP32[$rc>>2] = $391;
   }
   $392 = HEAP32[$rc>>2]|0;
   $393 = (0)==($392|0);
   if ($393) {
    $394 = HEAP32[$0>>2]|0;
    $395 = (($394) + 4|0);
    $396 = HEAP32[$395>>2]|0;
    $397 = (_sqlite3BtreeCommitPhaseTwo($396,0)|0);
    HEAP32[$rc>>2] = $397;
    $398 = (0)==($397|0);
    $$ = $398 ? 101 : $397;
    HEAP32[$rc>>2] = $$;
   }
  }
 }
 $399 = HEAP32[$bCloseTrans>>2]|0;
 $400 = ($399|0)!=(0);
 if ($400) {
  $401 = HEAP32[$0>>2]|0;
  $402 = (($401) + 24|0);
  $403 = HEAP32[$402>>2]|0;
  (_sqlite3BtreeCommitPhaseOne($403,0)|0);
  $404 = HEAP32[$0>>2]|0;
  $405 = (($404) + 24|0);
  $406 = HEAP32[$405>>2]|0;
  (_sqlite3BtreeCommitPhaseTwo($406,0)|0);
 }
 $407 = HEAP32[$rc>>2]|0;
 $408 = ($407|0)==(3082);
 if ($408) {
  HEAP32[$rc>>2] = 7;
 }
 $409 = HEAP32[$rc>>2]|0;
 $410 = HEAP32[$0>>2]|0;
 $411 = (($410) + 28|0);
 HEAP32[$411>>2] = $409;
 $412 = HEAP32[$rc>>2]|0;
 STACKTOP = sp;return ($412|0);
}
function _sqlite3BtreeEnter($p) {
 $p = $p|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 HEAP32[$0>>2] = $p;
 $1 = HEAP32[$0>>2]|0;
 $2 = HEAP32[$1>>2]|0;
 $3 = HEAP32[$0>>2]|0;
 $4 = (($3) + 4|0);
 $5 = HEAP32[$4>>2]|0;
 $6 = (($5) + 4|0);
 HEAP32[$6>>2] = $2;
 STACKTOP = sp;return;
}
function _isFatalError($rc) {
 $rc = $rc|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $or$cond = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 HEAP32[$0>>2] = $rc;
 $1 = HEAP32[$0>>2]|0;
 $2 = ($1|0)!=(0);
 $3 = HEAP32[$0>>2]|0;
 $4 = ($3|0)!=(5);
 $or$cond = $2 & $4;
 if (!($or$cond)) {
  $8 = 0;
  $7 = $8&1;
  STACKTOP = sp;return ($7|0);
 }
 $5 = HEAP32[$0>>2]|0;
 $6 = ($5|0)!=(6);
 $8 = $6;
 $7 = $8&1;
 STACKTOP = sp;return ($7|0);
}
function _sqlite3BtreeBeginTrans($p,$wrflag) {
 $p = $p|0;
 $wrflag = $wrflag|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0;
 var $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0, $188 = 0;
 var $189 = 0, $19 = 0, $190 = 0, $191 = 0, $192 = 0, $193 = 0, $194 = 0, $195 = 0, $196 = 0, $197 = 0, $198 = 0, $199 = 0, $2 = 0, $20 = 0, $200 = 0, $201 = 0, $202 = 0, $203 = 0, $204 = 0, $205 = 0;
 var $206 = 0, $207 = 0, $208 = 0, $209 = 0, $21 = 0, $210 = 0, $211 = 0, $212 = 0, $213 = 0, $214 = 0, $215 = 0, $216 = 0, $217 = 0, $218 = 0, $219 = 0, $22 = 0, $220 = 0, $221 = 0, $222 = 0, $223 = 0;
 var $224 = 0, $225 = 0, $226 = 0, $227 = 0, $228 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0;
 var $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0;
 var $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0;
 var $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0;
 var $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $or$cond = 0, $or$cond3 = 0, $or$cond5 = 0, $or$cond9 = 0, $pBlock = 0, $pBt = 0, $pIter = 0, $pPage1 = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 24|0;
 $1 = sp + 20|0;
 $pBlock = sp + 16|0;
 $pBt = sp + 12|0;
 $rc = sp + 8|0;
 $pIter = sp + 4|0;
 $pPage1 = sp;
 HEAP32[$0>>2] = $p;
 HEAP32[$1>>2] = $wrflag;
 HEAP32[$pBlock>>2] = 0;
 $2 = HEAP32[$0>>2]|0;
 $3 = (($2) + 4|0);
 $4 = HEAP32[$3>>2]|0;
 HEAP32[$pBt>>2] = $4;
 HEAP32[$rc>>2] = 0;
 $5 = HEAP32[$0>>2]|0;
 _sqlite3BtreeEnter($5);
 $6 = HEAP32[$0>>2]|0;
 $7 = (($6) + 8|0);
 $8 = HEAP8[$7>>0]|0;
 $9 = $8&255;
 $10 = ($9|0)==(2);
 do {
  if (!($10)) {
   $11 = HEAP32[$0>>2]|0;
   $12 = (($11) + 8|0);
   $13 = HEAP8[$12>>0]|0;
   $14 = $13&255;
   $15 = ($14|0)!=(1);
   $16 = HEAP32[$1>>2]|0;
   $17 = ($16|0)!=(0);
   $or$cond = $15 | $17;
   if ($or$cond) {
    $18 = HEAP32[$pBt>>2]|0;
    $19 = (($18) + 22|0);
    $20 = HEAP16[$19>>1]|0;
    $21 = $20&65535;
    $22 = $21 & 1;
    $23 = ($22|0)!=(0);
    $24 = HEAP32[$1>>2]|0;
    $25 = ($24|0)!=(0);
    $or$cond3 = $23 & $25;
    if ($or$cond3) {
     HEAP32[$rc>>2] = 8;
     break;
    }
    $26 = HEAP32[$1>>2]|0;
    $27 = ($26|0)!=(0);
    if ($27) {
     $28 = HEAP32[$pBt>>2]|0;
     $29 = (($28) + 20|0);
     $30 = HEAP8[$29>>0]|0;
     $31 = $30&255;
     $32 = ($31|0)==(2);
     if ($32) {
      label = 8;
     } else {
      label = 7;
     }
    } else {
     label = 7;
    }
    L9: do {
     if ((label|0) == 7) {
      $33 = HEAP32[$pBt>>2]|0;
      $34 = (($33) + 22|0);
      $35 = HEAP16[$34>>1]|0;
      $36 = $35&65535;
      $37 = $36 & 64;
      $38 = ($37|0)!=(0);
      if ($38) {
       label = 8;
      } else {
       $43 = HEAP32[$1>>2]|0;
       $44 = ($43|0)>(1);
       if ($44) {
        $45 = HEAP32[$pBt>>2]|0;
        $46 = (($45) + 72|0);
        $47 = HEAP32[$46>>2]|0;
        HEAP32[$pIter>>2] = $47;
        while(1) {
         $48 = HEAP32[$pIter>>2]|0;
         $49 = ($48|0)!=(0|0);
         if (!($49)) {
          break L9;
         }
         $50 = HEAP32[$pIter>>2]|0;
         $51 = HEAP32[$50>>2]|0;
         $52 = HEAP32[$0>>2]|0;
         $53 = ($51|0)!=($52|0);
         $54 = HEAP32[$pIter>>2]|0;
         if ($53) {
          break;
         }
         $57 = (($54) + 12|0);
         $58 = HEAP32[$57>>2]|0;
         HEAP32[$pIter>>2] = $58;
        }
        $55 = HEAP32[$54>>2]|0;
        $56 = HEAP32[$55>>2]|0;
        HEAP32[$pBlock>>2] = $56;
       }
      }
     }
    } while(0);
    if ((label|0) == 8) {
     $39 = HEAP32[$pBt>>2]|0;
     $40 = (($39) + 76|0);
     $41 = HEAP32[$40>>2]|0;
     $42 = HEAP32[$41>>2]|0;
     HEAP32[$pBlock>>2] = $42;
    }
    $59 = HEAP32[$pBlock>>2]|0;
    $60 = ($59|0)!=(0|0);
    if ($60) {
     HEAP32[$rc>>2] = 262;
     break;
    }
    $61 = HEAP32[$0>>2]|0;
    $62 = (_querySharedCacheTableLock($61,1,1)|0);
    HEAP32[$rc>>2] = $62;
    $63 = HEAP32[$rc>>2]|0;
    $64 = (0)!=($63|0);
    if (!($64)) {
     $65 = HEAP32[$pBt>>2]|0;
     $66 = (($65) + 22|0);
     $67 = HEAP16[$66>>1]|0;
     $68 = $67&65535;
     $69 = $68 & -9;
     $70 = $69&65535;
     HEAP16[$66>>1] = $70;
     $71 = HEAP32[$pBt>>2]|0;
     $72 = (($71) + 44|0);
     $73 = HEAP32[$72>>2]|0;
     $74 = ($73|0)==(0);
     if ($74) {
      $75 = HEAP32[$pBt>>2]|0;
      $76 = (($75) + 22|0);
      $77 = HEAP16[$76>>1]|0;
      $78 = $77&65535;
      $79 = $78 | 8;
      $80 = $79&65535;
      HEAP16[$76>>1] = $80;
     }
     while(1) {
      $81 = HEAP32[$pBt>>2]|0;
      $82 = (($81) + 12|0);
      $83 = HEAP32[$82>>2]|0;
      $84 = ($83|0)==(0|0);
      if ($84) {
       $85 = HEAP32[$pBt>>2]|0;
       $86 = (_lockBtree($85)|0);
       HEAP32[$rc>>2] = $86;
       $87 = (0)==($86|0);
       if ($87) {
        continue;
       }
      }
      $88 = HEAP32[$rc>>2]|0;
      $89 = ($88|0)==(0);
      $90 = HEAP32[$1>>2]|0;
      $91 = ($90|0)!=(0);
      $or$cond5 = $89 & $91;
      do {
       if ($or$cond5) {
        $92 = HEAP32[$pBt>>2]|0;
        $93 = (($92) + 22|0);
        $94 = HEAP16[$93>>1]|0;
        $95 = $94&65535;
        $96 = $95 & 1;
        $97 = ($96|0)!=(0);
        if ($97) {
         HEAP32[$rc>>2] = 8;
         break;
        }
        $98 = HEAP32[$pBt>>2]|0;
        $99 = HEAP32[$98>>2]|0;
        $100 = HEAP32[$1>>2]|0;
        $101 = ($100|0)>(1);
        $102 = $101&1;
        $103 = HEAP32[$0>>2]|0;
        $104 = HEAP32[$103>>2]|0;
        $105 = (_sqlite3TempInMemory($104)|0);
        $106 = (_sqlite3PagerBegin($99,$102,$105)|0);
        HEAP32[$rc>>2] = $106;
        $107 = HEAP32[$rc>>2]|0;
        $108 = ($107|0)==(0);
        if ($108) {
         $109 = HEAP32[$pBt>>2]|0;
         $110 = (_newDatabase($109)|0);
         HEAP32[$rc>>2] = $110;
        }
       }
      } while(0);
      $111 = HEAP32[$rc>>2]|0;
      $112 = ($111|0)!=(0);
      if ($112) {
       $113 = HEAP32[$pBt>>2]|0;
       _unlockBtreeIfUnused($113);
      }
      $114 = HEAP32[$rc>>2]|0;
      $115 = $114 & 255;
      $116 = ($115|0)==(5);
      if (!($116)) {
       break;
      }
      $117 = HEAP32[$pBt>>2]|0;
      $118 = (($117) + 20|0);
      $119 = HEAP8[$118>>0]|0;
      $120 = $119&255;
      $121 = ($120|0)==(0);
      if (!($121)) {
       break;
      }
      $122 = HEAP32[$pBt>>2]|0;
      $123 = (_btreeInvokeBusyHandler($122)|0);
      $124 = ($123|0)!=(0);
      if (!($124)) {
       break;
      }
     }
     $125 = HEAP32[$rc>>2]|0;
     $126 = ($125|0)==(0);
     if ($126) {
      $127 = HEAP32[$0>>2]|0;
      $128 = (($127) + 8|0);
      $129 = HEAP8[$128>>0]|0;
      $130 = $129&255;
      $131 = ($130|0)==(0);
      if ($131) {
       $132 = HEAP32[$pBt>>2]|0;
       $133 = (($132) + 40|0);
       $134 = HEAP32[$133>>2]|0;
       $135 = (($134) + 1)|0;
       HEAP32[$133>>2] = $135;
       $136 = HEAP32[$0>>2]|0;
       $137 = (($136) + 9|0);
       $138 = HEAP8[$137>>0]|0;
       $139 = ($138<<24>>24)!=(0);
       if ($139) {
        $140 = HEAP32[$0>>2]|0;
        $141 = (($140) + 28|0);
        $142 = (($141) + 8|0);
        HEAP8[$142>>0] = 1;
        $143 = HEAP32[$pBt>>2]|0;
        $144 = (($143) + 72|0);
        $145 = HEAP32[$144>>2]|0;
        $146 = HEAP32[$0>>2]|0;
        $147 = (($146) + 28|0);
        $148 = (($147) + 12|0);
        HEAP32[$148>>2] = $145;
        $149 = HEAP32[$0>>2]|0;
        $150 = (($149) + 28|0);
        $151 = HEAP32[$pBt>>2]|0;
        $152 = (($151) + 72|0);
        HEAP32[$152>>2] = $150;
       }
      }
      $153 = HEAP32[$1>>2]|0;
      $154 = ($153|0)!=(0);
      $155 = $154 ? 2 : 1;
      $156 = $155&255;
      $157 = HEAP32[$0>>2]|0;
      $158 = (($157) + 8|0);
      HEAP8[$158>>0] = $156;
      $159 = HEAP32[$0>>2]|0;
      $160 = (($159) + 8|0);
      $161 = HEAP8[$160>>0]|0;
      $162 = $161&255;
      $163 = HEAP32[$pBt>>2]|0;
      $164 = (($163) + 20|0);
      $165 = HEAP8[$164>>0]|0;
      $166 = $165&255;
      $167 = ($162|0)>($166|0);
      if ($167) {
       $168 = HEAP32[$0>>2]|0;
       $169 = (($168) + 8|0);
       $170 = HEAP8[$169>>0]|0;
       $171 = HEAP32[$pBt>>2]|0;
       $172 = (($171) + 20|0);
       HEAP8[$172>>0] = $170;
      }
      $173 = HEAP32[$1>>2]|0;
      $174 = ($173|0)!=(0);
      if ($174) {
       $175 = HEAP32[$pBt>>2]|0;
       $176 = (($175) + 12|0);
       $177 = HEAP32[$176>>2]|0;
       HEAP32[$pPage1>>2] = $177;
       $178 = HEAP32[$0>>2]|0;
       $179 = HEAP32[$pBt>>2]|0;
       $180 = (($179) + 76|0);
       HEAP32[$180>>2] = $178;
       $181 = HEAP32[$pBt>>2]|0;
       $182 = (($181) + 22|0);
       $183 = HEAP16[$182>>1]|0;
       $184 = $183&65535;
       $185 = $184 & -33;
       $186 = $185&65535;
       HEAP16[$182>>1] = $186;
       $187 = HEAP32[$1>>2]|0;
       $188 = ($187|0)>(1);
       if ($188) {
        $189 = HEAP32[$pBt>>2]|0;
        $190 = (($189) + 22|0);
        $191 = HEAP16[$190>>1]|0;
        $192 = $191&65535;
        $193 = $192 | 32;
        $194 = $193&65535;
        HEAP16[$190>>1] = $194;
       }
       $195 = HEAP32[$pBt>>2]|0;
       $196 = (($195) + 44|0);
       $197 = HEAP32[$196>>2]|0;
       $198 = HEAP32[$pPage1>>2]|0;
       $199 = (($198) + 56|0);
       $200 = HEAP32[$199>>2]|0;
       $201 = (($200) + 28|0);
       $202 = (_sqlite3Get4byte($201)|0);
       $203 = ($197|0)!=($202|0);
       if ($203) {
        $204 = HEAP32[$pPage1>>2]|0;
        $205 = (($204) + 68|0);
        $206 = HEAP32[$205>>2]|0;
        $207 = (_sqlite3PagerWrite($206)|0);
        HEAP32[$rc>>2] = $207;
        $208 = HEAP32[$rc>>2]|0;
        $209 = ($208|0)==(0);
        if ($209) {
         $210 = HEAP32[$pPage1>>2]|0;
         $211 = (($210) + 56|0);
         $212 = HEAP32[$211>>2]|0;
         $213 = (($212) + 28|0);
         $214 = HEAP32[$pBt>>2]|0;
         $215 = (($214) + 44|0);
         $216 = HEAP32[$215>>2]|0;
         _sqlite3Put4byte($213,$216);
        }
       }
      }
     }
    }
   }
  }
 } while(0);
 $217 = HEAP32[$rc>>2]|0;
 $218 = ($217|0)==(0);
 $219 = HEAP32[$1>>2]|0;
 $220 = ($219|0)!=(0);
 $or$cond9 = $218 & $220;
 if (!($or$cond9)) {
  $228 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($228|0);
 }
 $221 = HEAP32[$pBt>>2]|0;
 $222 = HEAP32[$221>>2]|0;
 $223 = HEAP32[$0>>2]|0;
 $224 = HEAP32[$223>>2]|0;
 $225 = (($224) + 496|0);
 $226 = HEAP32[$225>>2]|0;
 $227 = (_sqlite3PagerOpenSavepoint($222,$226)|0);
 HEAP32[$rc>>2] = $227;
 $228 = HEAP32[$rc>>2]|0;
 STACKTOP = sp;return ($228|0);
}
function _sqlite3BtreeGetMeta($p,$idx,$pMeta) {
 $p = $p|0;
 $idx = $idx|0;
 $pMeta = $pMeta|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $pBt = 0, label = 0;
 var sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $1 = sp + 8|0;
 $2 = sp + 4|0;
 $pBt = sp;
 HEAP32[$0>>2] = $p;
 HEAP32[$1>>2] = $idx;
 HEAP32[$2>>2] = $pMeta;
 $3 = HEAP32[$0>>2]|0;
 $4 = (($3) + 4|0);
 $5 = HEAP32[$4>>2]|0;
 HEAP32[$pBt>>2] = $5;
 $6 = HEAP32[$0>>2]|0;
 _sqlite3BtreeEnter($6);
 $7 = HEAP32[$1>>2]|0;
 $8 = $7<<2;
 $9 = (36 + ($8))|0;
 $10 = HEAP32[$pBt>>2]|0;
 $11 = (($10) + 12|0);
 $12 = HEAP32[$11>>2]|0;
 $13 = (($12) + 56|0);
 $14 = HEAP32[$13>>2]|0;
 $15 = (($14) + ($9)|0);
 $16 = (_sqlite3Get4byte($15)|0);
 $17 = HEAP32[$2>>2]|0;
 HEAP32[$17>>2] = $16;
 STACKTOP = sp;return;
}
function _sqlite3BtreeIsInReadTrans($p) {
 $p = $p|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 HEAP32[$0>>2] = $p;
 $1 = HEAP32[$0>>2]|0;
 $2 = (($1) + 8|0);
 $3 = HEAP8[$2>>0]|0;
 $4 = $3&255;
 $5 = ($4|0)!=(0);
 $6 = $5&1;
 STACKTOP = sp;return ($6|0);
}
function _sqlite3BtreeGetPageSize($p) {
 $p = $p|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 HEAP32[$0>>2] = $p;
 $1 = HEAP32[$0>>2]|0;
 $2 = (($1) + 4|0);
 $3 = HEAP32[$2>>2]|0;
 $4 = (($3) + 32|0);
 $5 = HEAP32[$4>>2]|0;
 STACKTOP = sp;return ($5|0);
}
function _sqlite3PagerGetJournalMode($pPager) {
 $pPager = $pPager|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 HEAP32[$0>>2] = $pPager;
 $1 = HEAP32[$0>>2]|0;
 $2 = (($1) + 5|0);
 $3 = HEAP8[$2>>0]|0;
 $4 = $3&255;
 STACKTOP = sp;return ($4|0);
}
function _sqlite3BtreeLastPage($p) {
 $p = $p|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 HEAP32[$0>>2] = $p;
 $1 = HEAP32[$0>>2]|0;
 $2 = (($1) + 4|0);
 $3 = HEAP32[$2>>2]|0;
 $4 = (_btreePagecount($3)|0);
 STACKTOP = sp;return ($4|0);
}
function _sqlite3PagerAcquire($pPager,$pgno,$ppPage,$flags) {
 $pPager = $pPager|0;
 $pgno = $pgno|0;
 $ppPage = $ppPage|0;
 $flags = $flags|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0;
 var $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0, $188 = 0;
 var $189 = 0, $19 = 0, $190 = 0, $191 = 0, $192 = 0, $193 = 0, $194 = 0, $195 = 0, $196 = 0, $197 = 0, $198 = 0, $199 = 0, $2 = 0, $20 = 0, $200 = 0, $201 = 0, $202 = 0, $203 = 0, $204 = 0, $205 = 0;
 var $206 = 0, $207 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0;
 var $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0;
 var $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0;
 var $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0;
 var $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $bMmapOk = 0, $iFrame = 0, $noContent = 0, $or$cond = 0, $or$cond3 = 0, $or$cond5 = 0, $or$cond7 = 0, $or$cond9 = 0, $pBase = 0, $pData = 0, $pPg = 0, $rc = 0;
 var label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $0 = sp + 44|0;
 $1 = sp + 40|0;
 $2 = sp + 36|0;
 $3 = sp + 32|0;
 $4 = sp + 28|0;
 $rc = sp + 24|0;
 $pPg = sp + 20|0;
 $iFrame = sp + 16|0;
 $noContent = sp + 12|0;
 $bMmapOk = sp + 8|0;
 $pData = sp + 4|0;
 $pBase = sp;
 HEAP32[$1>>2] = $pPager;
 HEAP32[$2>>2] = $pgno;
 HEAP32[$3>>2] = $ppPage;
 HEAP32[$4>>2] = $flags;
 HEAP32[$rc>>2] = 0;
 HEAP32[$pPg>>2] = 0;
 HEAP32[$iFrame>>2] = 0;
 $5 = HEAP32[$4>>2]|0;
 $6 = $5 & 1;
 HEAP32[$noContent>>2] = $6;
 $7 = 0;
 HEAP32[$bMmapOk>>2] = $7;
 $8 = HEAP32[$2>>2]|0;
 $9 = ($8|0)==(0);
 if ($9) {
  $10 = (_sqlite3CorruptError(46160)|0);
  HEAP32[$0>>2] = $10;
  $207 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($207|0);
 }
 $11 = HEAP32[$1>>2]|0;
 $12 = (($11) + 40|0);
 $13 = HEAP32[$12>>2]|0;
 $14 = ($13|0)!=(0);
 do {
  if ($14) {
   $15 = HEAP32[$1>>2]|0;
   $16 = (($15) + 40|0);
   $17 = HEAP32[$16>>2]|0;
   HEAP32[$rc>>2] = $17;
   label = 22;
  } else {
   $18 = HEAP32[$bMmapOk>>2]|0;
   $19 = ($18|0)!=(0);
   if ($19) {
    $20 = HEAP32[$1>>2]|0;
    $21 = (_pagerUseWal($20)|0);
    $22 = ($21|0)!=(0);
    if ($22) {
     $23 = HEAP32[$1>>2]|0;
     $24 = (($23) + 208|0);
     $25 = HEAP32[$24>>2]|0;
     $26 = HEAP32[$2>>2]|0;
     $27 = (_sqlite3WalFindFrame($25,$26,$iFrame)|0);
     HEAP32[$rc>>2] = $27;
     $28 = HEAP32[$rc>>2]|0;
     $29 = ($28|0)!=(0);
     if ($29) {
      break;
     }
    }
   }
   $30 = HEAP32[$bMmapOk>>2]|0;
   $31 = ($30|0)!=(0);
   $32 = HEAP32[$iFrame>>2]|0;
   $33 = ($32|0)==(0);
   $or$cond = $31 & $33;
   if ($or$cond) {
    HEAP32[$pData>>2] = 0;
    $34 = HEAP32[$1>>2]|0;
    $35 = (($34) + 60|0);
    $36 = HEAP32[$35>>2]|0;
    $37 = HEAP32[$2>>2]|0;
    $38 = (($37) - 1)|0;
    $39 = HEAP32[$1>>2]|0;
    $40 = (($39) + 152|0);
    $41 = HEAP32[$40>>2]|0;
    $42 = ($41|0)<(0);
    $43 = $42 << 31 >> 31;
    $44 = (___muldi3(($38|0),0,($41|0),($43|0))|0);
    $45 = tempRet0;
    $46 = HEAP32[$1>>2]|0;
    $47 = (($46) + 152|0);
    $48 = HEAP32[$47>>2]|0;
    $49 = (_sqlite3OsFetch($36,$44,$45,$48,$pData)|0);
    HEAP32[$rc>>2] = $49;
    $50 = HEAP32[$rc>>2]|0;
    $51 = ($50|0)==(0);
    $52 = HEAP32[$pData>>2]|0;
    $53 = ($52|0)!=(0|0);
    $or$cond3 = $51 & $53;
    if ($or$cond3) {
     $54 = HEAP32[$1>>2]|0;
     $55 = (($54) + 16|0);
     $56 = HEAP8[$55>>0]|0;
     $57 = $56&255;
     $58 = ($57|0)>(1);
     if ($58) {
      $59 = HEAP32[$1>>2]|0;
      $60 = HEAP32[$2>>2]|0;
      $61 = (_sqlite3PagerLookup($59,$60)|0);
      HEAP32[$pPg>>2] = $61;
     }
     $62 = HEAP32[$pPg>>2]|0;
     $63 = ($62|0)==(0|0);
     $64 = HEAP32[$1>>2]|0;
     if ($63) {
      $65 = HEAP32[$2>>2]|0;
      $66 = HEAP32[$pData>>2]|0;
      $67 = (_pagerAcquireMapPage($64,$65,$66,$pPg)|0);
      HEAP32[$rc>>2] = $67;
     } else {
      $68 = (($64) + 60|0);
      $69 = HEAP32[$68>>2]|0;
      $70 = HEAP32[$2>>2]|0;
      $71 = (($70) - 1)|0;
      $72 = HEAP32[$1>>2]|0;
      $73 = (($72) + 152|0);
      $74 = HEAP32[$73>>2]|0;
      $75 = ($74|0)<(0);
      $76 = $75 << 31 >> 31;
      $77 = (___muldi3(($71|0),0,($74|0),($76|0))|0);
      $78 = tempRet0;
      $79 = HEAP32[$pData>>2]|0;
      (_sqlite3OsUnfetch($69,$77,$78,$79)|0);
     }
     $80 = HEAP32[$pPg>>2]|0;
     $81 = ($80|0)!=(0|0);
     if ($81) {
      $82 = HEAP32[$pPg>>2]|0;
      $83 = HEAP32[$3>>2]|0;
      HEAP32[$83>>2] = $82;
      HEAP32[$0>>2] = 0;
      $207 = HEAP32[$0>>2]|0;
      STACKTOP = sp;return ($207|0);
     }
    }
    $84 = HEAP32[$rc>>2]|0;
    $85 = ($84|0)!=(0);
    if ($85) {
     break;
    }
   }
   $86 = HEAP32[$1>>2]|0;
   $87 = (($86) + 204|0);
   $88 = HEAP32[$87>>2]|0;
   $89 = HEAP32[$2>>2]|0;
   $90 = (_sqlite3PcacheFetch($88,$89,3)|0);
   HEAP32[$pBase>>2] = $90;
   $91 = HEAP32[$pBase>>2]|0;
   $92 = ($91|0)==(0|0);
   if ($92) {
    $93 = HEAP32[$1>>2]|0;
    $94 = (($93) + 204|0);
    $95 = HEAP32[$94>>2]|0;
    $96 = HEAP32[$2>>2]|0;
    $97 = (_sqlite3PcacheFetchStress($95,$96,$pBase)|0);
    HEAP32[$rc>>2] = $97;
    $98 = HEAP32[$rc>>2]|0;
    $99 = ($98|0)!=(0);
    if ($99) {
     break;
    }
   }
   $100 = HEAP32[$1>>2]|0;
   $101 = (($100) + 204|0);
   $102 = HEAP32[$101>>2]|0;
   $103 = HEAP32[$2>>2]|0;
   $104 = HEAP32[$pBase>>2]|0;
   $105 = (_sqlite3PcacheFetchFinish($102,$103,$104)|0);
   $106 = HEAP32[$3>>2]|0;
   HEAP32[$106>>2] = $105;
   HEAP32[$pPg>>2] = $105;
   $107 = HEAP32[$pPg>>2]|0;
   $108 = ($107|0)==(0|0);
   if ($108) {
    HEAP32[$rc>>2] = 7;
    label = 22;
   } else {
    label = 22;
   }
  }
 } while(0);
 do {
  if ((label|0) == 22) {
   $109 = HEAP32[$rc>>2]|0;
   $110 = ($109|0)!=(0);
   if ($110) {
    HEAP32[$pPg>>2] = 0;
    break;
   }
   $111 = HEAP32[$3>>2]|0;
   $112 = HEAP32[$111>>2]|0;
   $113 = (($112) + 16|0);
   $114 = HEAP32[$113>>2]|0;
   $115 = ($114|0)==(0|0);
   $116 = HEAP32[$noContent>>2]|0;
   $117 = ($116|0)!=(0);
   $or$cond5 = $115 | $117;
   if (!($or$cond5)) {
    $118 = HEAP32[$1>>2]|0;
    $119 = (($118) + 184|0);
    $120 = HEAP32[$119>>2]|0;
    $121 = (($120) + 1)|0;
    HEAP32[$119>>2] = $121;
    HEAP32[$0>>2] = 0;
    $207 = HEAP32[$0>>2]|0;
    STACKTOP = sp;return ($207|0);
   }
   $122 = HEAP32[$3>>2]|0;
   $123 = HEAP32[$122>>2]|0;
   HEAP32[$pPg>>2] = $123;
   $124 = HEAP32[$1>>2]|0;
   $125 = HEAP32[$pPg>>2]|0;
   $126 = (($125) + 16|0);
   HEAP32[$126>>2] = $124;
   $127 = HEAP32[$2>>2]|0;
   $128 = ($127>>>0)>(2147483647);
   if (!($128)) {
    $129 = HEAP32[$2>>2]|0;
    $130 = HEAP32[1128>>2]|0;
    $131 = HEAP32[$1>>2]|0;
    $132 = (($131) + 152|0);
    $133 = HEAP32[$132>>2]|0;
    $134 = (($130|0) / ($133|0))&-1;
    $135 = (($134) + 1)|0;
    $136 = ($129|0)==($135|0);
    if (!($136)) {
     $138 = HEAP32[$1>>2]|0;
     $139 = (($138) + 15|0);
     $140 = HEAP8[$139>>0]|0;
     $141 = $140&255;
     $142 = ($141|0)!=(0);
     if ($142) {
      label = 32;
     } else {
      $143 = HEAP32[$1>>2]|0;
      $144 = (($143) + 24|0);
      $145 = HEAP32[$144>>2]|0;
      $146 = HEAP32[$2>>2]|0;
      $147 = ($145>>>0)<($146>>>0);
      $148 = HEAP32[$noContent>>2]|0;
      $149 = ($148|0)!=(0);
      $or$cond7 = $147 | $149;
      if ($or$cond7) {
       label = 32;
      } else {
       $150 = HEAP32[$1>>2]|0;
       $151 = (($150) + 60|0);
       $152 = HEAP32[$151>>2]|0;
       $153 = HEAP32[$152>>2]|0;
       $154 = ($153|0)!=(0|0);
       if ($154) {
        $179 = HEAP32[$1>>2]|0;
        $180 = (_pagerUseWal($179)|0);
        $181 = ($180|0)!=(0);
        $182 = HEAP32[$bMmapOk>>2]|0;
        $183 = ($182|0)==(0);
        $or$cond9 = $181 & $183;
        if ($or$cond9) {
         $184 = HEAP32[$1>>2]|0;
         $185 = (($184) + 208|0);
         $186 = HEAP32[$185>>2]|0;
         $187 = HEAP32[$2>>2]|0;
         $188 = (_sqlite3WalFindFrame($186,$187,$iFrame)|0);
         HEAP32[$rc>>2] = $188;
         $189 = HEAP32[$rc>>2]|0;
         $190 = ($189|0)!=(0);
         if ($190) {
          break;
         }
        }
        $191 = HEAP32[$1>>2]|0;
        $192 = (($191) + 184|0);
        $193 = (($192) + 4|0);
        $194 = HEAP32[$193>>2]|0;
        $195 = (($194) + 1)|0;
        HEAP32[$193>>2] = $195;
        $196 = HEAP32[$pPg>>2]|0;
        $197 = HEAP32[$iFrame>>2]|0;
        $198 = (_readDbPage($196,$197)|0);
        HEAP32[$rc>>2] = $198;
        $199 = HEAP32[$rc>>2]|0;
        $200 = ($199|0)!=(0);
        if ($200) {
         break;
        }
       } else {
        label = 32;
       }
      }
     }
     if ((label|0) == 32) {
      $155 = HEAP32[$2>>2]|0;
      $156 = HEAP32[$1>>2]|0;
      $157 = (($156) + 156|0);
      $158 = HEAP32[$157>>2]|0;
      $159 = ($155>>>0)>($158>>>0);
      if ($159) {
       HEAP32[$rc>>2] = 13;
       break;
      }
      $160 = HEAP32[$noContent>>2]|0;
      $161 = ($160|0)!=(0);
      if ($161) {
       _sqlite3BeginBenignMalloc();
       $162 = HEAP32[$2>>2]|0;
       $163 = HEAP32[$1>>2]|0;
       $164 = (($163) + 28|0);
       $165 = HEAP32[$164>>2]|0;
       $166 = ($162>>>0)<=($165>>>0);
       if ($166) {
        $167 = HEAP32[$1>>2]|0;
        $168 = (($167) + 56|0);
        $169 = HEAP32[$168>>2]|0;
        $170 = HEAP32[$2>>2]|0;
        (_sqlite3BitvecSet($169,$170)|0);
       }
       $171 = HEAP32[$1>>2]|0;
       $172 = HEAP32[$2>>2]|0;
       (_addToSavepointBitvecs($171,$172)|0);
       _sqlite3EndBenignMalloc();
      }
      $173 = HEAP32[$pPg>>2]|0;
      $174 = (($173) + 4|0);
      $175 = HEAP32[$174>>2]|0;
      $176 = HEAP32[$1>>2]|0;
      $177 = (($176) + 152|0);
      $178 = HEAP32[$177>>2]|0;
      _memset(($175|0),0,($178|0))|0;
     }
     HEAP32[$0>>2] = 0;
     $207 = HEAP32[$0>>2]|0;
     STACKTOP = sp;return ($207|0);
    }
   }
   $137 = (_sqlite3CorruptError(46239)|0);
   HEAP32[$rc>>2] = $137;
  }
 } while(0);
 $201 = HEAP32[$pPg>>2]|0;
 $202 = ($201|0)!=(0|0);
 if ($202) {
  $203 = HEAP32[$pPg>>2]|0;
  _sqlite3PcacheDrop($203);
 }
 $204 = HEAP32[$1>>2]|0;
 _pagerUnlockIfUnused($204);
 $205 = HEAP32[$3>>2]|0;
 HEAP32[$205>>2] = 0;
 $206 = HEAP32[$rc>>2]|0;
 HEAP32[$0>>2] = $206;
 $207 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($207|0);
}
function _backupOnePage($p,$iSrcPg,$zSrcData,$bUpdate) {
 $p = $p|0;
 $iSrcPg = $iSrcPg|0;
 $zSrcData = $zSrcData|0;
 $bUpdate = $bUpdate|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0;
 var $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0;
 var $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0;
 var $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0;
 var $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0;
 var $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $iDest = 0, $iEnd = 0, $iOff = 0, $nCopy = 0, $nDestPgsz = 0, $nSrcPgsz = 0, $or$cond = 0, $pDestPager = 0, $pDestPg = 0, $rc = 0;
 var $zDestData = 0, $zIn = 0, $zOut = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 80|0;
 $0 = sp + 68|0;
 $1 = sp + 64|0;
 $2 = sp + 60|0;
 $3 = sp + 56|0;
 $pDestPager = sp + 52|0;
 $nSrcPgsz = sp + 48|0;
 $nDestPgsz = sp + 44|0;
 $nCopy = sp + 40|0;
 $iEnd = sp + 8|0;
 $rc = sp + 36|0;
 $iOff = sp;
 $pDestPg = sp + 32|0;
 $iDest = sp + 28|0;
 $zIn = sp + 24|0;
 $zDestData = sp + 20|0;
 $zOut = sp + 16|0;
 HEAP32[$0>>2] = $p;
 HEAP32[$1>>2] = $iSrcPg;
 HEAP32[$2>>2] = $zSrcData;
 HEAP32[$3>>2] = $bUpdate;
 $4 = HEAP32[$0>>2]|0;
 $5 = (($4) + 4|0);
 $6 = HEAP32[$5>>2]|0;
 $7 = (_sqlite3BtreePager($6)|0);
 HEAP32[$pDestPager>>2] = $7;
 $8 = HEAP32[$0>>2]|0;
 $9 = (($8) + 24|0);
 $10 = HEAP32[$9>>2]|0;
 $11 = (_sqlite3BtreeGetPageSize($10)|0);
 HEAP32[$nSrcPgsz>>2] = $11;
 $12 = HEAP32[$0>>2]|0;
 $13 = (($12) + 4|0);
 $14 = HEAP32[$13>>2]|0;
 $15 = (_sqlite3BtreeGetPageSize($14)|0);
 HEAP32[$nDestPgsz>>2] = $15;
 $16 = HEAP32[$nSrcPgsz>>2]|0;
 $17 = HEAP32[$nDestPgsz>>2]|0;
 $18 = ($16|0)<($17|0);
 $19 = HEAP32[$nSrcPgsz>>2]|0;
 $20 = HEAP32[$nDestPgsz>>2]|0;
 $21 = $18 ? $19 : $20;
 HEAP32[$nCopy>>2] = $21;
 $22 = HEAP32[$1>>2]|0;
 $23 = HEAP32[$nSrcPgsz>>2]|0;
 $24 = ($23|0)<(0);
 $25 = $24 << 31 >> 31;
 $26 = (___muldi3(($22|0),0,($23|0),($25|0))|0);
 $27 = tempRet0;
 $28 = $iEnd;
 $29 = $28;
 HEAP32[$29>>2] = $26;
 $30 = (($28) + 4)|0;
 $31 = $30;
 HEAP32[$31>>2] = $27;
 HEAP32[$rc>>2] = 0;
 $32 = HEAP32[$nSrcPgsz>>2]|0;
 $33 = HEAP32[$nDestPgsz>>2]|0;
 $34 = ($32|0)!=($33|0);
 if ($34) {
  $35 = HEAP32[$pDestPager>>2]|0;
  $36 = (_sqlite3PagerIsMemdb($35)|0);
  $37 = ($36|0)!=(0);
  if ($37) {
   HEAP32[$rc>>2] = 8;
  }
 }
 $38 = $iEnd;
 $39 = $38;
 $40 = HEAP32[$39>>2]|0;
 $41 = (($38) + 4)|0;
 $42 = $41;
 $43 = HEAP32[$42>>2]|0;
 $44 = HEAP32[$nSrcPgsz>>2]|0;
 $45 = ($44|0)<(0);
 $46 = $45 << 31 >> 31;
 $47 = (_i64Subtract(($40|0),($43|0),($44|0),($46|0))|0);
 $48 = tempRet0;
 $49 = $iOff;
 $50 = $49;
 HEAP32[$50>>2] = $47;
 $51 = (($49) + 4)|0;
 $52 = $51;
 HEAP32[$52>>2] = $48;
 while(1) {
  $53 = HEAP32[$rc>>2]|0;
  $54 = ($53|0)==(0);
  if (!($54)) {
   label = 14;
   break;
  }
  $55 = $iOff;
  $56 = $55;
  $57 = HEAP32[$56>>2]|0;
  $58 = (($55) + 4)|0;
  $59 = $58;
  $60 = HEAP32[$59>>2]|0;
  $61 = $iEnd;
  $62 = $61;
  $63 = HEAP32[$62>>2]|0;
  $64 = (($61) + 4)|0;
  $65 = $64;
  $66 = HEAP32[$65>>2]|0;
  $67 = ($60|0)<($66|0);
  $68 = ($57>>>0)<($63>>>0);
  $69 = ($60|0)==($66|0);
  $70 = $69 & $68;
  $71 = $67 | $70;
  if (!($71)) {
   label = 14;
   break;
  }
  HEAP32[$pDestPg>>2] = 0;
  $72 = $iOff;
  $73 = $72;
  $74 = HEAP32[$73>>2]|0;
  $75 = (($72) + 4)|0;
  $76 = $75;
  $77 = HEAP32[$76>>2]|0;
  $78 = HEAP32[$nDestPgsz>>2]|0;
  $79 = ($78|0)<(0);
  $80 = $79 << 31 >> 31;
  $81 = (___divdi3(($74|0),($77|0),($78|0),($80|0))|0);
  $82 = tempRet0;
  $83 = (($81) + 1)|0;
  HEAP32[$iDest>>2] = $83;
  $84 = HEAP32[$iDest>>2]|0;
  $85 = HEAP32[1128>>2]|0;
  $86 = HEAP32[$0>>2]|0;
  $87 = (($86) + 4|0);
  $88 = HEAP32[$87>>2]|0;
  $89 = (($88) + 4|0);
  $90 = HEAP32[$89>>2]|0;
  $91 = (($90) + 32|0);
  $92 = HEAP32[$91>>2]|0;
  $93 = (($85>>>0) / ($92>>>0))&-1;
  $94 = (($93) + 1)|0;
  $95 = ($84|0)==($94|0);
  if (!($95)) {
   $96 = HEAP32[$pDestPager>>2]|0;
   $97 = HEAP32[$iDest>>2]|0;
   $98 = (_sqlite3PagerAcquire($96,$97,$pDestPg,0)|0);
   HEAP32[$rc>>2] = $98;
   $99 = (0)==($98|0);
   if ($99) {
    $100 = HEAP32[$pDestPg>>2]|0;
    $101 = (_sqlite3PagerWrite($100)|0);
    HEAP32[$rc>>2] = $101;
    $102 = (0)==($101|0);
    if ($102) {
     $103 = $iOff;
     $104 = $103;
     $105 = HEAP32[$104>>2]|0;
     $106 = (($103) + 4)|0;
     $107 = $106;
     $108 = HEAP32[$107>>2]|0;
     $109 = HEAP32[$nSrcPgsz>>2]|0;
     $110 = ($109|0)<(0);
     $111 = $110 << 31 >> 31;
     $112 = (___remdi3(($105|0),($108|0),($109|0),($111|0))|0);
     $113 = tempRet0;
     $114 = HEAP32[$2>>2]|0;
     $115 = (($114) + ($112)|0);
     HEAP32[$zIn>>2] = $115;
     $116 = HEAP32[$pDestPg>>2]|0;
     $117 = (_sqlite3PagerGetData($116)|0);
     HEAP32[$zDestData>>2] = $117;
     $118 = $iOff;
     $119 = $118;
     $120 = HEAP32[$119>>2]|0;
     $121 = (($118) + 4)|0;
     $122 = $121;
     $123 = HEAP32[$122>>2]|0;
     $124 = HEAP32[$nDestPgsz>>2]|0;
     $125 = ($124|0)<(0);
     $126 = $125 << 31 >> 31;
     $127 = (___remdi3(($120|0),($123|0),($124|0),($126|0))|0);
     $128 = tempRet0;
     $129 = HEAP32[$zDestData>>2]|0;
     $130 = (($129) + ($127)|0);
     HEAP32[$zOut>>2] = $130;
     $131 = HEAP32[$zOut>>2]|0;
     $132 = HEAP32[$zIn>>2]|0;
     $133 = HEAP32[$nCopy>>2]|0;
     _memcpy(($131|0),($132|0),($133|0))|0;
     $134 = HEAP32[$pDestPg>>2]|0;
     $135 = (_sqlite3PagerGetExtra($134)|0);
     HEAP8[$135>>0] = 0;
     $136 = $iOff;
     $137 = $136;
     $138 = HEAP32[$137>>2]|0;
     $139 = (($136) + 4)|0;
     $140 = $139;
     $141 = HEAP32[$140>>2]|0;
     $142 = ($138|0)==(0);
     $143 = ($141|0)==(0);
     $144 = $142 & $143;
     $145 = HEAP32[$3>>2]|0;
     $146 = ($145|0)==(0);
     $or$cond = $144 & $146;
     if ($or$cond) {
      $147 = HEAP32[$zOut>>2]|0;
      $148 = (($147) + 28|0);
      $149 = HEAP32[$0>>2]|0;
      $150 = (($149) + 24|0);
      $151 = HEAP32[$150>>2]|0;
      $152 = (_sqlite3BtreeLastPage($151)|0);
      _sqlite3Put4byte($148,$152);
     }
    }
   }
   $153 = HEAP32[$pDestPg>>2]|0;
   _sqlite3PagerUnref($153);
  }
  $154 = HEAP32[$nDestPgsz>>2]|0;
  $155 = ($154|0)<(0);
  $156 = $155 << 31 >> 31;
  $157 = $iOff;
  $158 = $157;
  $159 = HEAP32[$158>>2]|0;
  $160 = (($157) + 4)|0;
  $161 = $160;
  $162 = HEAP32[$161>>2]|0;
  $163 = (_i64Add(($159|0),($162|0),($154|0),($156|0))|0);
  $164 = tempRet0;
  $165 = $iOff;
  $166 = $165;
  HEAP32[$166>>2] = $163;
  $167 = (($165) + 4)|0;
  $168 = $167;
  HEAP32[$168>>2] = $164;
 }
 if ((label|0) == 14) {
  $169 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($169|0);
 }
 return 0|0;
}
function _sqlite3PagerGetData($pPg) {
 $pPg = $pPg|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 HEAP32[$0>>2] = $pPg;
 $1 = HEAP32[$0>>2]|0;
 $2 = (($1) + 4|0);
 $3 = HEAP32[$2>>2]|0;
 STACKTOP = sp;return ($3|0);
}
function _sqlite3PagerUnref($pPg) {
 $pPg = $pPg|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 HEAP32[$0>>2] = $pPg;
 $1 = HEAP32[$0>>2]|0;
 $2 = ($1|0)!=(0|0);
 if (!($2)) {
  STACKTOP = sp;return;
 }
 $3 = HEAP32[$0>>2]|0;
 _sqlite3PagerUnrefNotNull($3);
 STACKTOP = sp;return;
}
function _attachBackupObject($p) {
 $p = $p|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $pp = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $pp = sp;
 HEAP32[$0>>2] = $p;
 $1 = HEAP32[$0>>2]|0;
 $2 = (($1) + 24|0);
 $3 = HEAP32[$2>>2]|0;
 $4 = (_sqlite3BtreePager($3)|0);
 $5 = (_sqlite3PagerBackupPtr($4)|0);
 HEAP32[$pp>>2] = $5;
 $6 = HEAP32[$pp>>2]|0;
 $7 = HEAP32[$6>>2]|0;
 $8 = HEAP32[$0>>2]|0;
 $9 = (($8) + 44|0);
 HEAP32[$9>>2] = $7;
 $10 = HEAP32[$0>>2]|0;
 $11 = HEAP32[$pp>>2]|0;
 HEAP32[$11>>2] = $10;
 $12 = HEAP32[$0>>2]|0;
 $13 = (($12) + 40|0);
 HEAP32[$13>>2] = 1;
 STACKTOP = sp;return;
}
function _sqlite3BtreeNewDb($p) {
 $p = $p|0;
 var $0 = 0, $1 = 0, $10 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $rc = sp;
 HEAP32[$0>>2] = $p;
 $1 = HEAP32[$0>>2]|0;
 _sqlite3BtreeEnter($1);
 $2 = HEAP32[$0>>2]|0;
 $3 = (($2) + 4|0);
 $4 = HEAP32[$3>>2]|0;
 $5 = (($4) + 44|0);
 HEAP32[$5>>2] = 0;
 $6 = HEAP32[$0>>2]|0;
 $7 = (($6) + 4|0);
 $8 = HEAP32[$7>>2]|0;
 $9 = (_newDatabase($8)|0);
 HEAP32[$rc>>2] = $9;
 $10 = HEAP32[$rc>>2]|0;
 STACKTOP = sp;return ($10|0);
}
function _sqlite3BtreeUpdateMeta($p,$idx,$iMeta) {
 $p = $p|0;
 $idx = $idx|0;
 $iMeta = $iMeta|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $pBt = 0, $pP1 = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 20|0;
 $1 = sp + 16|0;
 $2 = sp + 12|0;
 $pBt = sp + 8|0;
 $pP1 = sp + 4|0;
 $rc = sp;
 HEAP32[$0>>2] = $p;
 HEAP32[$1>>2] = $idx;
 HEAP32[$2>>2] = $iMeta;
 $3 = HEAP32[$0>>2]|0;
 $4 = (($3) + 4|0);
 $5 = HEAP32[$4>>2]|0;
 HEAP32[$pBt>>2] = $5;
 $6 = HEAP32[$0>>2]|0;
 _sqlite3BtreeEnter($6);
 $7 = HEAP32[$pBt>>2]|0;
 $8 = (($7) + 12|0);
 $9 = HEAP32[$8>>2]|0;
 $10 = (($9) + 56|0);
 $11 = HEAP32[$10>>2]|0;
 HEAP32[$pP1>>2] = $11;
 $12 = HEAP32[$pBt>>2]|0;
 $13 = (($12) + 12|0);
 $14 = HEAP32[$13>>2]|0;
 $15 = (($14) + 68|0);
 $16 = HEAP32[$15>>2]|0;
 $17 = (_sqlite3PagerWrite($16)|0);
 HEAP32[$rc>>2] = $17;
 $18 = HEAP32[$rc>>2]|0;
 $19 = ($18|0)==(0);
 if (!($19)) {
  $32 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($32|0);
 }
 $20 = HEAP32[$1>>2]|0;
 $21 = $20<<2;
 $22 = (36 + ($21))|0;
 $23 = HEAP32[$pP1>>2]|0;
 $24 = (($23) + ($22)|0);
 $25 = HEAP32[$2>>2]|0;
 _sqlite3Put4byte($24,$25);
 $26 = HEAP32[$1>>2]|0;
 $27 = ($26|0)==(7);
 if (!($27)) {
  $32 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($32|0);
 }
 $28 = HEAP32[$2>>2]|0;
 $29 = $28&255;
 $30 = HEAP32[$pBt>>2]|0;
 $31 = (($30) + 18|0);
 HEAP8[$31>>0] = $29;
 $32 = HEAP32[$rc>>2]|0;
 STACKTOP = sp;return ($32|0);
}
function _sqlite3ResetAllSchemasOfConnection($db) {
 $db = $db|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $i = 0, $pDb = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $i = sp + 4|0;
 $pDb = sp;
 HEAP32[$0>>2] = $db;
 $1 = HEAP32[$0>>2]|0;
 _sqlite3BtreeEnterAll($1);
 HEAP32[$i>>2] = 0;
 while(1) {
  $2 = HEAP32[$i>>2]|0;
  $3 = HEAP32[$0>>2]|0;
  $4 = (($3) + 20|0);
  $5 = HEAP32[$4>>2]|0;
  $6 = ($2|0)<($5|0);
  if (!($6)) {
   break;
  }
  $7 = HEAP32[$i>>2]|0;
  $8 = HEAP32[$0>>2]|0;
  $9 = (($8) + 16|0);
  $10 = HEAP32[$9>>2]|0;
  $11 = (($10) + ($7<<4)|0);
  HEAP32[$pDb>>2] = $11;
  $12 = HEAP32[$pDb>>2]|0;
  $13 = (($12) + 12|0);
  $14 = HEAP32[$13>>2]|0;
  $15 = ($14|0)!=(0|0);
  if ($15) {
   $16 = HEAP32[$pDb>>2]|0;
   $17 = (($16) + 12|0);
   $18 = HEAP32[$17>>2]|0;
   _sqlite3SchemaClear($18);
  }
  $19 = HEAP32[$i>>2]|0;
  $20 = (($19) + 1)|0;
  HEAP32[$i>>2] = $20;
 }
 $21 = HEAP32[$0>>2]|0;
 $22 = (($21) + 24|0);
 $23 = HEAP32[$22>>2]|0;
 $24 = $23 & -3;
 HEAP32[$22>>2] = $24;
 $25 = HEAP32[$0>>2]|0;
 _sqlite3VtabUnlockList($25);
 $26 = HEAP32[$0>>2]|0;
 _sqlite3CollapseDatabaseArray($26);
 STACKTOP = sp;return;
}
function _sqlite3BtreeSetVersion($pBtree,$iVersion) {
 $pBtree = $pBtree|0;
 $iVersion = $iVersion|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $8 = 0, $9 = 0, $aData = 0, $pBt = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 16|0;
 $1 = sp + 12|0;
 $pBt = sp + 8|0;
 $rc = sp + 4|0;
 $aData = sp;
 HEAP32[$0>>2] = $pBtree;
 HEAP32[$1>>2] = $iVersion;
 $2 = HEAP32[$0>>2]|0;
 $3 = (($2) + 4|0);
 $4 = HEAP32[$3>>2]|0;
 HEAP32[$pBt>>2] = $4;
 $5 = HEAP32[$pBt>>2]|0;
 $6 = (($5) + 22|0);
 $7 = HEAP16[$6>>1]|0;
 $8 = $7&65535;
 $9 = $8 & -17;
 $10 = $9&65535;
 HEAP16[$6>>1] = $10;
 $11 = HEAP32[$1>>2]|0;
 $12 = ($11|0)==(1);
 if ($12) {
  $13 = HEAP32[$pBt>>2]|0;
  $14 = (($13) + 22|0);
  $15 = HEAP16[$14>>1]|0;
  $16 = $15&65535;
  $17 = $16 | 16;
  $18 = $17&65535;
  HEAP16[$14>>1] = $18;
 }
 $19 = HEAP32[$0>>2]|0;
 $20 = (_sqlite3BtreeBeginTrans($19,0)|0);
 HEAP32[$rc>>2] = $20;
 $21 = HEAP32[$rc>>2]|0;
 $22 = ($21|0)==(0);
 do {
  if ($22) {
   $23 = HEAP32[$pBt>>2]|0;
   $24 = (($23) + 12|0);
   $25 = HEAP32[$24>>2]|0;
   $26 = (($25) + 56|0);
   $27 = HEAP32[$26>>2]|0;
   HEAP32[$aData>>2] = $27;
   $28 = HEAP32[$aData>>2]|0;
   $29 = (($28) + 18|0);
   $30 = HEAP8[$29>>0]|0;
   $31 = $30&255;
   $32 = HEAP32[$1>>2]|0;
   $33 = $32&255;
   $34 = $33&255;
   $35 = ($31|0)!=($34|0);
   if (!($35)) {
    $36 = HEAP32[$aData>>2]|0;
    $37 = (($36) + 19|0);
    $38 = HEAP8[$37>>0]|0;
    $39 = $38&255;
    $40 = HEAP32[$1>>2]|0;
    $41 = $40&255;
    $42 = $41&255;
    $43 = ($39|0)!=($42|0);
    if (!($43)) {
     break;
    }
   }
   $44 = HEAP32[$0>>2]|0;
   $45 = (_sqlite3BtreeBeginTrans($44,2)|0);
   HEAP32[$rc>>2] = $45;
   $46 = HEAP32[$rc>>2]|0;
   $47 = ($46|0)==(0);
   if ($47) {
    $48 = HEAP32[$pBt>>2]|0;
    $49 = (($48) + 12|0);
    $50 = HEAP32[$49>>2]|0;
    $51 = (($50) + 68|0);
    $52 = HEAP32[$51>>2]|0;
    $53 = (_sqlite3PagerWrite($52)|0);
    HEAP32[$rc>>2] = $53;
    $54 = HEAP32[$rc>>2]|0;
    $55 = ($54|0)==(0);
    if ($55) {
     $56 = HEAP32[$1>>2]|0;
     $57 = $56&255;
     $58 = HEAP32[$aData>>2]|0;
     $59 = (($58) + 18|0);
     HEAP8[$59>>0] = $57;
     $60 = HEAP32[$1>>2]|0;
     $61 = $60&255;
     $62 = HEAP32[$aData>>2]|0;
     $63 = (($62) + 19|0);
     HEAP8[$63>>0] = $61;
    }
   }
  }
 } while(0);
 $64 = HEAP32[$pBt>>2]|0;
 $65 = (($64) + 22|0);
 $66 = HEAP16[$65>>1]|0;
 $67 = $66&65535;
 $68 = $67 & -17;
 $69 = $68&65535;
 HEAP16[$65>>1] = $69;
 $70 = HEAP32[$rc>>2]|0;
 STACKTOP = sp;return ($70|0);
}
function _sqlite3PagerFile($pPager) {
 $pPager = $pPager|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 HEAP32[$0>>2] = $pPager;
 $1 = HEAP32[$0>>2]|0;
 $2 = (($1) + 60|0);
 $3 = HEAP32[$2>>2]|0;
 STACKTOP = sp;return ($3|0);
}
function _sqlite3PagerPagecount($pPager,$pnPage) {
 $pPager = $pPager|0;
 $pnPage = $pnPage|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $1 = sp;
 HEAP32[$0>>2] = $pPager;
 HEAP32[$1>>2] = $pnPage;
 $2 = HEAP32[$0>>2]|0;
 $3 = (($2) + 24|0);
 $4 = HEAP32[$3>>2]|0;
 $5 = HEAP32[$1>>2]|0;
 HEAP32[$5>>2] = $4;
 STACKTOP = sp;return;
}
function _sqlite3PagerWrite($pPg) {
 $pPg = $pPg|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $1 = sp;
 HEAP32[$1>>2] = $pPg;
 $2 = HEAP32[$1>>2]|0;
 $3 = (($2) + 16|0);
 $4 = HEAP32[$3>>2]|0;
 $5 = (($4) + 148|0);
 $6 = HEAP32[$5>>2]|0;
 $7 = HEAP32[$1>>2]|0;
 $8 = (($7) + 16|0);
 $9 = HEAP32[$8>>2]|0;
 $10 = (($9) + 152|0);
 $11 = HEAP32[$10>>2]|0;
 $12 = ($6>>>0)>($11>>>0);
 $13 = HEAP32[$1>>2]|0;
 if ($12) {
  $14 = (_pagerWriteLargeSector($13)|0);
  HEAP32[$0>>2] = $14;
  $16 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($16|0);
 } else {
  $15 = (_pager_write($13)|0);
  HEAP32[$0>>2] = $15;
  $16 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($16|0);
 }
 return 0|0;
}
function _sqlite3PagerCommitPhaseOne($pPager,$zMaster,$noSync) {
 $pPager = $pPager|0;
 $zMaster = $zMaster|0;
 $noSync = $noSync|0;
 var $$old = 0, $$old1 = 0, $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $12 = 0;
 var $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0;
 var $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0;
 var $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0;
 var $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0;
 var $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $nNew = 0, $or$cond = 0, $pList = 0, $pPageOne = 0, $rc = 0;
 var label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 28|0;
 $1 = sp + 24|0;
 $2 = sp + 20|0;
 $3 = sp + 16|0;
 $rc = sp + 12|0;
 $pList = sp + 8|0;
 $pPageOne = sp + 4|0;
 $nNew = sp;
 HEAP32[$1>>2] = $pPager;
 HEAP32[$2>>2] = $zMaster;
 HEAP32[$3>>2] = $noSync;
 HEAP32[$rc>>2] = 0;
 $4 = HEAP32[$1>>2]|0;
 $5 = (($4) + 40|0);
 $6 = HEAP32[$5>>2]|0;
 $7 = ($6|0)!=(0);
 $8 = HEAP32[$1>>2]|0;
 if ($7) {
  $9 = (($8) + 40|0);
  $10 = HEAP32[$9>>2]|0;
  HEAP32[$0>>2] = $10;
  $112 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($112|0);
 }
 $11 = (($8) + 16|0);
 $12 = HEAP8[$11>>0]|0;
 $13 = $12&255;
 $14 = ($13|0)<(3);
 if ($14) {
  HEAP32[$0>>2] = 0;
  $112 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($112|0);
 }
 $15 = HEAP32[$1>>2]|0;
 $16 = (($15) + 15|0);
 $17 = HEAP8[$16>>0]|0;
 $18 = ($17<<24>>24)!=(0);
 $19 = HEAP32[$1>>2]|0;
 do {
  if ($18) {
   $20 = (($19) + 88|0);
   $21 = HEAP32[$20>>2]|0;
   _sqlite3BackupRestart($21);
  } else {
   $22 = (_pagerUseWal($19)|0);
   $23 = ($22|0)!=(0);
   $24 = HEAP32[$1>>2]|0;
   if ($23) {
    $25 = (($24) + 204|0);
    $26 = HEAP32[$25>>2]|0;
    $27 = (_sqlite3PcacheDirtyList($26)|0);
    HEAP32[$pList>>2] = $27;
    HEAP32[$pPageOne>>2] = 0;
    $28 = HEAP32[$pList>>2]|0;
    $29 = ($28|0)==(0|0);
    if ($29) {
     $30 = HEAP32[$1>>2]|0;
     $31 = (_sqlite3PagerAcquire($30,1,$pPageOne,0)|0);
     HEAP32[$rc>>2] = $31;
     $32 = HEAP32[$pPageOne>>2]|0;
     HEAP32[$pList>>2] = $32;
     $33 = HEAP32[$pList>>2]|0;
     $34 = (($33) + 12|0);
     HEAP32[$34>>2] = 0;
    }
    $35 = HEAP32[$pList>>2]|0;
    $36 = ($35|0)!=(0|0);
    if ($36) {
     $37 = HEAP32[$1>>2]|0;
     $38 = HEAP32[$pList>>2]|0;
     $39 = HEAP32[$1>>2]|0;
     $40 = (($39) + 24|0);
     $41 = HEAP32[$40>>2]|0;
     $42 = (_pagerWalFrames($37,$38,$41,1)|0);
     HEAP32[$rc>>2] = $42;
    }
    $43 = HEAP32[$pPageOne>>2]|0;
    _sqlite3PagerUnref($43);
    $44 = HEAP32[$rc>>2]|0;
    $45 = ($44|0)==(0);
    if (!($45)) {
     break;
    }
    $46 = HEAP32[$1>>2]|0;
    $47 = (($46) + 204|0);
    $48 = HEAP32[$47>>2]|0;
    _sqlite3PcacheCleanAll($48);
    break;
   }
   $49 = (_pager_incr_changecounter($24,0)|0);
   HEAP32[$rc>>2] = $49;
   $50 = HEAP32[$rc>>2]|0;
   $51 = ($50|0)!=(0);
   if (!($51)) {
    $52 = HEAP32[$1>>2]|0;
    $53 = HEAP32[$2>>2]|0;
    $54 = (_writeMasterJournal($52,$53)|0);
    HEAP32[$rc>>2] = $54;
    $55 = HEAP32[$rc>>2]|0;
    $56 = ($55|0)!=(0);
    if (!($56)) {
     $57 = HEAP32[$1>>2]|0;
     $58 = (_syncJournal($57,0)|0);
     HEAP32[$rc>>2] = $58;
     $59 = HEAP32[$rc>>2]|0;
     $60 = ($59|0)!=(0);
     if (!($60)) {
      $61 = HEAP32[$1>>2]|0;
      $62 = HEAP32[$1>>2]|0;
      $63 = (($62) + 204|0);
      $64 = HEAP32[$63>>2]|0;
      $65 = (_sqlite3PcacheDirtyList($64)|0);
      $66 = (_pager_write_pagelist($61,$65)|0);
      HEAP32[$rc>>2] = $66;
      $67 = HEAP32[$rc>>2]|0;
      $68 = ($67|0)!=(0);
      if (!($68)) {
       $69 = HEAP32[$1>>2]|0;
       $70 = (($69) + 204|0);
       $71 = HEAP32[$70>>2]|0;
       _sqlite3PcacheCleanAll($71);
       $72 = HEAP32[$1>>2]|0;
       $73 = (($72) + 24|0);
       $74 = HEAP32[$73>>2]|0;
       $75 = HEAP32[$1>>2]|0;
       $76 = (($75) + 32|0);
       $77 = HEAP32[$76>>2]|0;
       $78 = ($74>>>0)>($77>>>0);
       if ($78) {
        $79 = HEAP32[$1>>2]|0;
        $80 = (($79) + 24|0);
        $81 = HEAP32[$80>>2]|0;
        $82 = HEAP32[$1>>2]|0;
        $83 = (($82) + 24|0);
        $84 = HEAP32[$83>>2]|0;
        $85 = HEAP32[1128>>2]|0;
        $86 = HEAP32[$1>>2]|0;
        $87 = (($86) + 152|0);
        $88 = HEAP32[$87>>2]|0;
        $89 = (($85|0) / ($88|0))&-1;
        $90 = (($89) + 1)|0;
        $91 = ($84|0)==($90|0);
        $92 = $91&1;
        $93 = (($81) - ($92))|0;
        HEAP32[$nNew>>2] = $93;
        $94 = HEAP32[$1>>2]|0;
        $95 = HEAP32[$nNew>>2]|0;
        $96 = (_pager_truncate($94,$95)|0);
        HEAP32[$rc>>2] = $96;
        $97 = HEAP32[$rc>>2]|0;
        $98 = ($97|0)!=(0);
        $99 = HEAP32[$3>>2]|0;
        $100 = ($99|0)!=(0);
        $or$cond = $98 | $100;
        if ($or$cond) {
         break;
        }
       } else {
        $$old = HEAP32[$3>>2]|0;
        $$old1 = ($$old|0)!=(0);
        if ($$old1) {
         break;
        }
       }
       $101 = HEAP32[$1>>2]|0;
       $102 = HEAP32[$2>>2]|0;
       $103 = (_sqlite3PagerSync($101,$102)|0);
       HEAP32[$rc>>2] = $103;
      }
     }
    }
   }
  }
 } while(0);
 $104 = HEAP32[$rc>>2]|0;
 $105 = ($104|0)==(0);
 if ($105) {
  $106 = HEAP32[$1>>2]|0;
  $107 = (_pagerUseWal($106)|0);
  $108 = ($107|0)!=(0);
  if (!($108)) {
   $109 = HEAP32[$1>>2]|0;
   $110 = (($109) + 16|0);
   HEAP8[$110>>0] = 5;
  }
 }
 $111 = HEAP32[$rc>>2]|0;
 HEAP32[$0>>2] = $111;
 $112 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($112|0);
}
function _sqlite3OsWrite($id,$pBuf,$amt,$0,$1) {
 $id = $id|0;
 $pBuf = $pBuf|0;
 $amt = $amt|0;
 $0 = $0|0;
 $1 = $1|0;
 var $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0;
 var $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $2 = sp + 16|0;
 $3 = sp + 12|0;
 $4 = sp + 8|0;
 $5 = sp;
 HEAP32[$2>>2] = $id;
 HEAP32[$3>>2] = $pBuf;
 HEAP32[$4>>2] = $amt;
 $6 = $5;
 $7 = $6;
 HEAP32[$7>>2] = $0;
 $8 = (($6) + 4)|0;
 $9 = $8;
 HEAP32[$9>>2] = $1;
 $10 = HEAP32[$2>>2]|0;
 $11 = HEAP32[$10>>2]|0;
 $12 = (($11) + 12|0);
 $13 = HEAP32[$12>>2]|0;
 $14 = HEAP32[$2>>2]|0;
 $15 = HEAP32[$3>>2]|0;
 $16 = HEAP32[$4>>2]|0;
 $17 = $5;
 $18 = $17;
 $19 = HEAP32[$18>>2]|0;
 $20 = (($17) + 4)|0;
 $21 = $20;
 $22 = HEAP32[$21>>2]|0;
 $23 = (FUNCTION_TABLE_iiiiii[$13 & 15]($14,$15,$16,$19,$22)|0);
 STACKTOP = sp;return ($23|0);
}
function _backupTruncateFile($pFile,$0,$1) {
 $pFile = $pFile|0;
 $0 = $0|0;
 $1 = $1|0;
 var $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0;
 var $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $iCurrent = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $2 = sp + 20|0;
 $3 = sp + 8|0;
 $iCurrent = sp;
 $rc = sp + 16|0;
 HEAP32[$2>>2] = $pFile;
 $4 = $3;
 $5 = $4;
 HEAP32[$5>>2] = $0;
 $6 = (($4) + 4)|0;
 $7 = $6;
 HEAP32[$7>>2] = $1;
 $8 = HEAP32[$2>>2]|0;
 $9 = (_sqlite3OsFileSize($8,$iCurrent)|0);
 HEAP32[$rc>>2] = $9;
 $10 = HEAP32[$rc>>2]|0;
 $11 = ($10|0)==(0);
 if (!($11)) {
  $37 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($37|0);
 }
 $12 = $iCurrent;
 $13 = $12;
 $14 = HEAP32[$13>>2]|0;
 $15 = (($12) + 4)|0;
 $16 = $15;
 $17 = HEAP32[$16>>2]|0;
 $18 = $3;
 $19 = $18;
 $20 = HEAP32[$19>>2]|0;
 $21 = (($18) + 4)|0;
 $22 = $21;
 $23 = HEAP32[$22>>2]|0;
 $24 = ($17|0)>($23|0);
 $25 = ($14>>>0)>($20>>>0);
 $26 = ($17|0)==($23|0);
 $27 = $26 & $25;
 $28 = $24 | $27;
 if (!($28)) {
  $37 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($37|0);
 }
 $29 = HEAP32[$2>>2]|0;
 $30 = $3;
 $31 = $30;
 $32 = HEAP32[$31>>2]|0;
 $33 = (($30) + 4)|0;
 $34 = $33;
 $35 = HEAP32[$34>>2]|0;
 $36 = (_sqlite3OsTruncate($29,$32,$35)|0);
 HEAP32[$rc>>2] = $36;
 $37 = HEAP32[$rc>>2]|0;
 STACKTOP = sp;return ($37|0);
}
function _sqlite3PagerSync($pPager,$zMaster) {
 $pPager = $pPager|0;
 $zMaster = $zMaster|0;
 var $$ = 0, $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0;
 var $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $pArg = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $1 = sp + 8|0;
 $rc = sp + 4|0;
 $pArg = sp;
 HEAP32[$0>>2] = $pPager;
 HEAP32[$1>>2] = $zMaster;
 HEAP32[$rc>>2] = 0;
 $2 = HEAP32[$0>>2]|0;
 $3 = (($2) + 60|0);
 $4 = HEAP32[$3>>2]|0;
 $5 = HEAP32[$4>>2]|0;
 $6 = ($5|0)!=(0|0);
 if ($6) {
  $7 = HEAP32[$1>>2]|0;
  HEAP32[$pArg>>2] = $7;
  $8 = HEAP32[$0>>2]|0;
  $9 = (($8) + 60|0);
  $10 = HEAP32[$9>>2]|0;
  $11 = HEAP32[$pArg>>2]|0;
  $12 = (_sqlite3OsFileControl($10,21,$11)|0);
  HEAP32[$rc>>2] = $12;
  $13 = HEAP32[$rc>>2]|0;
  $14 = ($13|0)==(12);
  $$ = $14 ? 0 : $12;
  HEAP32[$rc>>2] = $$;
 }
 $15 = HEAP32[$rc>>2]|0;
 $16 = ($15|0)==(0);
 if (!($16)) {
  $29 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($29|0);
 }
 $17 = HEAP32[$0>>2]|0;
 $18 = (($17) + 7|0);
 $19 = HEAP8[$18>>0]|0;
 $20 = ($19<<24>>24)!=(0);
 if ($20) {
  $29 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($29|0);
 }
 $21 = HEAP32[$0>>2]|0;
 $22 = (($21) + 60|0);
 $23 = HEAP32[$22>>2]|0;
 $24 = HEAP32[$0>>2]|0;
 $25 = (($24) + 11|0);
 $26 = HEAP8[$25>>0]|0;
 $27 = $26&255;
 $28 = (_sqlite3OsSync($23,$27)|0);
 HEAP32[$rc>>2] = $28;
 $29 = HEAP32[$rc>>2]|0;
 STACKTOP = sp;return ($29|0);
}
function _sqlite3PagerTruncateImage($pPager,$nPage) {
 $pPager = $pPager|0;
 $nPage = $nPage|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $1 = sp;
 HEAP32[$0>>2] = $pPager;
 HEAP32[$1>>2] = $nPage;
 $2 = HEAP32[$1>>2]|0;
 $3 = HEAP32[$0>>2]|0;
 $4 = (($3) + 24|0);
 HEAP32[$4>>2] = $2;
 STACKTOP = sp;return;
}
function _sqlite3BtreeCommitPhaseTwo($p,$bCleanup) {
 $p = $p|0;
 $bCleanup = $bCleanup|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $or$cond = 0, $pBt = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 16|0;
 $1 = sp + 12|0;
 $2 = sp + 8|0;
 $rc = sp + 4|0;
 $pBt = sp;
 HEAP32[$1>>2] = $p;
 HEAP32[$2>>2] = $bCleanup;
 $3 = HEAP32[$1>>2]|0;
 $4 = (($3) + 8|0);
 $5 = HEAP8[$4>>0]|0;
 $6 = $5&255;
 $7 = ($6|0)==(0);
 if ($7) {
  HEAP32[$0>>2] = 0;
  $29 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($29|0);
 }
 $8 = HEAP32[$1>>2]|0;
 _sqlite3BtreeEnter($8);
 $9 = HEAP32[$1>>2]|0;
 $10 = (($9) + 8|0);
 $11 = HEAP8[$10>>0]|0;
 $12 = $11&255;
 $13 = ($12|0)==(2);
 do {
  if ($13) {
   $14 = HEAP32[$1>>2]|0;
   $15 = (($14) + 4|0);
   $16 = HEAP32[$15>>2]|0;
   HEAP32[$pBt>>2] = $16;
   $17 = HEAP32[$pBt>>2]|0;
   $18 = HEAP32[$17>>2]|0;
   $19 = (_sqlite3PagerCommitPhaseTwo($18)|0);
   HEAP32[$rc>>2] = $19;
   $20 = HEAP32[$rc>>2]|0;
   $21 = ($20|0)!=(0);
   $22 = HEAP32[$2>>2]|0;
   $23 = ($22|0)==(0);
   $or$cond = $21 & $23;
   if (!($or$cond)) {
    $25 = HEAP32[$pBt>>2]|0;
    $26 = (($25) + 20|0);
    HEAP8[$26>>0] = 1;
    $27 = HEAP32[$pBt>>2]|0;
    _btreeClearHasContent($27);
    break;
   }
   $24 = HEAP32[$rc>>2]|0;
   HEAP32[$0>>2] = $24;
   $29 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($29|0);
  }
 } while(0);
 $28 = HEAP32[$1>>2]|0;
 _btreeEndTransaction($28);
 HEAP32[$0>>2] = 0;
 $29 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($29|0);
}
function _sqlite3BtreeCommitPhaseOne($p,$zMaster) {
 $p = $p|0;
 $zMaster = $zMaster|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $pBt = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 16|0;
 $1 = sp + 12|0;
 $2 = sp + 8|0;
 $rc = sp + 4|0;
 $pBt = sp;
 HEAP32[$1>>2] = $p;
 HEAP32[$2>>2] = $zMaster;
 HEAP32[$rc>>2] = 0;
 $3 = HEAP32[$1>>2]|0;
 $4 = (($3) + 8|0);
 $5 = HEAP8[$4>>0]|0;
 $6 = $5&255;
 $7 = ($6|0)==(2);
 if ($7) {
  $8 = HEAP32[$1>>2]|0;
  $9 = (($8) + 4|0);
  $10 = HEAP32[$9>>2]|0;
  HEAP32[$pBt>>2] = $10;
  $11 = HEAP32[$1>>2]|0;
  _sqlite3BtreeEnter($11);
  $12 = HEAP32[$pBt>>2]|0;
  $13 = (($12) + 17|0);
  $14 = HEAP8[$13>>0]|0;
  $15 = ($14<<24>>24)!=(0);
  if ($15) {
   $16 = HEAP32[$pBt>>2]|0;
   $17 = (_autoVacuumCommit($16)|0);
   HEAP32[$rc>>2] = $17;
   $18 = HEAP32[$rc>>2]|0;
   $19 = ($18|0)!=(0);
   if ($19) {
    $20 = HEAP32[$rc>>2]|0;
    HEAP32[$0>>2] = $20;
    $35 = HEAP32[$0>>2]|0;
    STACKTOP = sp;return ($35|0);
   }
  }
  $21 = HEAP32[$pBt>>2]|0;
  $22 = (($21) + 19|0);
  $23 = HEAP8[$22>>0]|0;
  $24 = ($23<<24>>24)!=(0);
  if ($24) {
   $25 = HEAP32[$pBt>>2]|0;
   $26 = HEAP32[$25>>2]|0;
   $27 = HEAP32[$pBt>>2]|0;
   $28 = (($27) + 44|0);
   $29 = HEAP32[$28>>2]|0;
   _sqlite3PagerTruncateImage($26,$29);
  }
  $30 = HEAP32[$pBt>>2]|0;
  $31 = HEAP32[$30>>2]|0;
  $32 = HEAP32[$2>>2]|0;
  $33 = (_sqlite3PagerCommitPhaseOne($31,$32,0)|0);
  HEAP32[$rc>>2] = $33;
 }
 $34 = HEAP32[$rc>>2]|0;
 HEAP32[$0>>2] = $34;
 $35 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($35|0);
}
function _sqlite3_backup_finish($p) {
 $p = $p|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $7 = 0, $8 = 0, $9 = 0, $pSrcDb = 0, $pp = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 16|0;
 $1 = sp + 12|0;
 $pp = sp + 8|0;
 $pSrcDb = sp + 4|0;
 $rc = sp;
 HEAP32[$1>>2] = $p;
 $2 = HEAP32[$1>>2]|0;
 $3 = ($2|0)==(0|0);
 if ($3) {
  HEAP32[$0>>2] = 0;
  $64 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($64|0);
 }
 $4 = HEAP32[$1>>2]|0;
 $5 = (($4) + 20|0);
 $6 = HEAP32[$5>>2]|0;
 HEAP32[$pSrcDb>>2] = $6;
 $7 = HEAP32[$1>>2]|0;
 $8 = (($7) + 24|0);
 $9 = HEAP32[$8>>2]|0;
 _sqlite3BtreeEnter($9);
 $10 = HEAP32[$1>>2]|0;
 $11 = HEAP32[$10>>2]|0;
 $12 = ($11|0)!=(0|0);
 if ($12) {
  $13 = HEAP32[$1>>2]|0;
  $14 = (($13) + 24|0);
  $15 = HEAP32[$14>>2]|0;
  $16 = (($15) + 16|0);
  $17 = HEAP32[$16>>2]|0;
  $18 = (($17) + -1)|0;
  HEAP32[$16>>2] = $18;
 }
 $19 = HEAP32[$1>>2]|0;
 $20 = (($19) + 40|0);
 $21 = HEAP32[$20>>2]|0;
 $22 = ($21|0)!=(0);
 if ($22) {
  $23 = HEAP32[$1>>2]|0;
  $24 = (($23) + 24|0);
  $25 = HEAP32[$24>>2]|0;
  $26 = (_sqlite3BtreePager($25)|0);
  $27 = (_sqlite3PagerBackupPtr($26)|0);
  HEAP32[$pp>>2] = $27;
  while(1) {
   $28 = HEAP32[$pp>>2]|0;
   $29 = HEAP32[$28>>2]|0;
   $30 = HEAP32[$1>>2]|0;
   $31 = ($29|0)!=($30|0);
   if (!($31)) {
    break;
   }
   $32 = HEAP32[$pp>>2]|0;
   $33 = HEAP32[$32>>2]|0;
   $34 = (($33) + 44|0);
   HEAP32[$pp>>2] = $34;
  }
  $35 = HEAP32[$1>>2]|0;
  $36 = (($35) + 44|0);
  $37 = HEAP32[$36>>2]|0;
  $38 = HEAP32[$pp>>2]|0;
  HEAP32[$38>>2] = $37;
 }
 $39 = HEAP32[$1>>2]|0;
 $40 = (($39) + 4|0);
 $41 = HEAP32[$40>>2]|0;
 (_sqlite3BtreeRollback($41,0,0)|0);
 $42 = HEAP32[$1>>2]|0;
 $43 = (($42) + 28|0);
 $44 = HEAP32[$43>>2]|0;
 $45 = ($44|0)==(101);
 if ($45) {
  $49 = 0;
 } else {
  $46 = HEAP32[$1>>2]|0;
  $47 = (($46) + 28|0);
  $48 = HEAP32[$47>>2]|0;
  $49 = $48;
 }
 HEAP32[$rc>>2] = $49;
 $50 = HEAP32[$1>>2]|0;
 $51 = HEAP32[$50>>2]|0;
 $52 = ($51|0)!=(0|0);
 if ($52) {
  $53 = HEAP32[$1>>2]|0;
  $54 = HEAP32[$53>>2]|0;
  $55 = HEAP32[$rc>>2]|0;
  _sqlite3Error($54,$55);
  $56 = HEAP32[$1>>2]|0;
  $57 = HEAP32[$56>>2]|0;
  _sqlite3LeaveMutexAndCloseZombie($57);
 }
 $58 = HEAP32[$1>>2]|0;
 $59 = HEAP32[$58>>2]|0;
 $60 = ($59|0)!=(0|0);
 if ($60) {
  $61 = HEAP32[$1>>2]|0;
  _sqlite3_free($61);
 }
 $62 = HEAP32[$pSrcDb>>2]|0;
 _sqlite3LeaveMutexAndCloseZombie($62);
 $63 = HEAP32[$rc>>2]|0;
 HEAP32[$0>>2] = $63;
 $64 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($64|0);
}
function _sqlite3PagerBackupPtr($pPager) {
 $pPager = $pPager|0;
 var $0 = 0, $1 = 0, $2 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 HEAP32[$0>>2] = $pPager;
 $1 = HEAP32[$0>>2]|0;
 $2 = (($1) + 88|0);
 STACKTOP = sp;return ($2|0);
}
function _sqlite3BtreeRollback($p,$tripCode,$writeOnly) {
 $p = $p|0;
 $tripCode = $tripCode|0;
 $writeOnly = $writeOnly|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $nPage = 0, $pBt = 0, $pPage1 = 0, $rc = 0, $rc2 = 0, $rc21 = 0;
 var label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $0 = sp + 32|0;
 $1 = sp + 28|0;
 $2 = sp + 24|0;
 $rc = sp + 20|0;
 $pBt = sp + 16|0;
 $pPage1 = sp + 12|0;
 $rc2 = sp + 8|0;
 $rc21 = sp + 4|0;
 $nPage = sp;
 HEAP32[$0>>2] = $p;
 HEAP32[$1>>2] = $tripCode;
 HEAP32[$2>>2] = $writeOnly;
 $3 = HEAP32[$0>>2]|0;
 $4 = (($3) + 4|0);
 $5 = HEAP32[$4>>2]|0;
 HEAP32[$pBt>>2] = $5;
 $6 = HEAP32[$0>>2]|0;
 _sqlite3BtreeEnter($6);
 $7 = HEAP32[$1>>2]|0;
 $8 = ($7|0)==(0);
 if ($8) {
  $9 = HEAP32[$pBt>>2]|0;
  $10 = (_saveAllCursors($9,0,0)|0);
  HEAP32[$1>>2] = $10;
  HEAP32[$rc>>2] = $10;
  $11 = HEAP32[$rc>>2]|0;
  $12 = ($11|0)!=(0);
  if ($12) {
   HEAP32[$2>>2] = 0;
  }
 } else {
  HEAP32[$rc>>2] = 0;
 }
 $13 = HEAP32[$1>>2]|0;
 $14 = ($13|0)!=(0);
 if ($14) {
  $15 = HEAP32[$0>>2]|0;
  $16 = HEAP32[$1>>2]|0;
  $17 = HEAP32[$2>>2]|0;
  $18 = (_sqlite3BtreeTripAllCursors($15,$16,$17)|0);
  HEAP32[$rc2>>2] = $18;
  $19 = HEAP32[$rc2>>2]|0;
  $20 = ($19|0)!=(0);
  if ($20) {
   $21 = HEAP32[$rc2>>2]|0;
   HEAP32[$rc>>2] = $21;
  }
 }
 $22 = HEAP32[$0>>2]|0;
 $23 = (($22) + 8|0);
 $24 = HEAP8[$23>>0]|0;
 $25 = $24&255;
 $26 = ($25|0)==(2);
 if (!($26)) {
  $52 = HEAP32[$0>>2]|0;
  _btreeEndTransaction($52);
  $53 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($53|0);
 }
 $27 = HEAP32[$pBt>>2]|0;
 $28 = HEAP32[$27>>2]|0;
 $29 = (_sqlite3PagerRollback($28)|0);
 HEAP32[$rc21>>2] = $29;
 $30 = HEAP32[$rc21>>2]|0;
 $31 = ($30|0)!=(0);
 if ($31) {
  $32 = HEAP32[$rc21>>2]|0;
  HEAP32[$rc>>2] = $32;
 }
 $33 = HEAP32[$pBt>>2]|0;
 $34 = (_btreeGetPage($33,1,$pPage1,0)|0);
 $35 = ($34|0)==(0);
 if ($35) {
  $36 = HEAP32[$pPage1>>2]|0;
  $37 = (($36) + 56|0);
  $38 = HEAP32[$37>>2]|0;
  $39 = (($38) + 28|0);
  $40 = (_sqlite3Get4byte($39)|0);
  HEAP32[$nPage>>2] = $40;
  $41 = HEAP32[$nPage>>2]|0;
  $42 = ($41|0)==(0);
  if ($42) {
   $43 = HEAP32[$pBt>>2]|0;
   $44 = HEAP32[$43>>2]|0;
   _sqlite3PagerPagecount($44,$nPage);
  }
  $45 = HEAP32[$nPage>>2]|0;
  $46 = HEAP32[$pBt>>2]|0;
  $47 = (($46) + 44|0);
  HEAP32[$47>>2] = $45;
  $48 = HEAP32[$pPage1>>2]|0;
  _releasePage($48);
 }
 $49 = HEAP32[$pBt>>2]|0;
 $50 = (($49) + 20|0);
 HEAP8[$50>>0] = 1;
 $51 = HEAP32[$pBt>>2]|0;
 _btreeClearHasContent($51);
 $52 = HEAP32[$0>>2]|0;
 _btreeEndTransaction($52);
 $53 = HEAP32[$rc>>2]|0;
 STACKTOP = sp;return ($53|0);
}
function _sqlite3LeaveMutexAndCloseZombie($db) {
 $db = $db|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0;
 var $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0;
 var $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0;
 var $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0;
 var $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0;
 var $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0;
 var $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $i = 0, $j = 0, $p = 0, $pColl = 0, $pDb = 0, $pHash = 0, $pIdx = 0, $pMod = 0, $pNext = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $0 = sp + 36|0;
 $i = sp + 32|0;
 $j = sp + 28|0;
 $pDb = sp + 24|0;
 $pIdx = sp + 20|0;
 $pNext = sp + 16|0;
 $pHash = sp + 12|0;
 $p = sp + 8|0;
 $pColl = sp + 4|0;
 $pMod = sp;
 HEAP32[$0>>2] = $db;
 $1 = HEAP32[$0>>2]|0;
 $2 = (($1) + 76|0);
 $3 = HEAP32[$2>>2]|0;
 $4 = ($3|0)!=(1691352191);
 if ($4) {
  STACKTOP = sp;return;
 }
 $5 = HEAP32[$0>>2]|0;
 $6 = (_connectionIsBusy($5)|0);
 $7 = ($6|0)!=(0);
 if ($7) {
  STACKTOP = sp;return;
 }
 $8 = HEAP32[$0>>2]|0;
 _sqlite3RollbackAll($8,0);
 $9 = HEAP32[$0>>2]|0;
 _sqlite3CloseSavepoints($9);
 HEAP32[$j>>2] = 0;
 while(1) {
  $10 = HEAP32[$j>>2]|0;
  $11 = HEAP32[$0>>2]|0;
  $12 = (($11) + 20|0);
  $13 = HEAP32[$12>>2]|0;
  $14 = ($10|0)<($13|0);
  if (!($14)) {
   break;
  }
  $15 = HEAP32[$j>>2]|0;
  $16 = HEAP32[$0>>2]|0;
  $17 = (($16) + 16|0);
  $18 = HEAP32[$17>>2]|0;
  $19 = (($18) + ($15<<4)|0);
  HEAP32[$pDb>>2] = $19;
  $20 = HEAP32[$pDb>>2]|0;
  $21 = (($20) + 4|0);
  $22 = HEAP32[$21>>2]|0;
  $23 = ($22|0)!=(0|0);
  if ($23) {
   $24 = HEAP32[$pDb>>2]|0;
   $25 = (($24) + 12|0);
   $26 = HEAP32[$25>>2]|0;
   $27 = ($26|0)!=(0|0);
   L12: do {
    if ($27) {
     $28 = HEAP32[$pDb>>2]|0;
     $29 = (($28) + 4|0);
     $30 = HEAP32[$29>>2]|0;
     _sqlite3BtreeEnter($30);
     $31 = HEAP32[$pDb>>2]|0;
     $32 = (($31) + 12|0);
     $33 = HEAP32[$32>>2]|0;
     $34 = (($33) + 24|0);
     $35 = (($34) + 8|0);
     $36 = HEAP32[$35>>2]|0;
     HEAP32[$i>>2] = $36;
     while(1) {
      $37 = HEAP32[$i>>2]|0;
      $38 = ($37|0)!=(0|0);
      if (!($38)) {
       break L12;
      }
      $39 = HEAP32[$i>>2]|0;
      $40 = (($39) + 8|0);
      $41 = HEAP32[$40>>2]|0;
      HEAP32[$pIdx>>2] = $41;
      $42 = HEAP32[$pIdx>>2]|0;
      $43 = (($42) + 40|0);
      $44 = HEAP32[$43>>2]|0;
      _sqlite3KeyInfoUnref($44);
      $45 = HEAP32[$pIdx>>2]|0;
      $46 = (($45) + 40|0);
      HEAP32[$46>>2] = 0;
      $47 = HEAP32[$i>>2]|0;
      $48 = HEAP32[$47>>2]|0;
      HEAP32[$i>>2] = $48;
     }
    }
   } while(0);
   $49 = HEAP32[$pDb>>2]|0;
   $50 = (($49) + 4|0);
   $51 = HEAP32[$50>>2]|0;
   (_sqlite3BtreeClose($51)|0);
   $52 = HEAP32[$pDb>>2]|0;
   $53 = (($52) + 4|0);
   HEAP32[$53>>2] = 0;
   $54 = HEAP32[$j>>2]|0;
   $55 = ($54|0)!=(1);
   if ($55) {
    $56 = HEAP32[$pDb>>2]|0;
    $57 = (($56) + 12|0);
    HEAP32[$57>>2] = 0;
   }
  }
  $58 = HEAP32[$j>>2]|0;
  $59 = (($58) + 1)|0;
  HEAP32[$j>>2] = $59;
 }
 $60 = HEAP32[$0>>2]|0;
 $61 = (($60) + 16|0);
 $62 = HEAP32[$61>>2]|0;
 $63 = (($62) + 16|0);
 $64 = (($63) + 12|0);
 $65 = HEAP32[$64>>2]|0;
 $66 = ($65|0)!=(0|0);
 if ($66) {
  $67 = HEAP32[$0>>2]|0;
  $68 = (($67) + 16|0);
  $69 = HEAP32[$68>>2]|0;
  $70 = (($69) + 16|0);
  $71 = (($70) + 12|0);
  $72 = HEAP32[$71>>2]|0;
  _sqlite3SchemaClear($72);
 }
 $73 = HEAP32[$0>>2]|0;
 _sqlite3VtabUnlockList($73);
 $74 = HEAP32[$0>>2]|0;
 _sqlite3CollapseDatabaseArray($74);
 HEAP32[$j>>2] = 0;
 while(1) {
  $75 = HEAP32[$j>>2]|0;
  $76 = ($75|0)<(23);
  if (!($76)) {
   break;
  }
  $77 = HEAP32[$j>>2]|0;
  $78 = HEAP32[$0>>2]|0;
  $79 = (($78) + 336|0);
  $80 = (($79) + ($77<<2)|0);
  $81 = HEAP32[$80>>2]|0;
  HEAP32[$p>>2] = $81;
  while(1) {
   $82 = HEAP32[$p>>2]|0;
   $83 = ($82|0)!=(0|0);
   if (!($83)) {
    break;
   }
   $84 = HEAP32[$p>>2]|0;
   $85 = (($84) + 28|0);
   $86 = HEAP32[$85>>2]|0;
   HEAP32[$pHash>>2] = $86;
   while(1) {
    $87 = HEAP32[$p>>2]|0;
    $88 = ($87|0)!=(0|0);
    if (!($88)) {
     break;
    }
    $89 = HEAP32[$0>>2]|0;
    $90 = HEAP32[$p>>2]|0;
    _functionDestroy($89,$90);
    $91 = HEAP32[$p>>2]|0;
    $92 = (($91) + 8|0);
    $93 = HEAP32[$92>>2]|0;
    HEAP32[$pNext>>2] = $93;
    $94 = HEAP32[$0>>2]|0;
    $95 = HEAP32[$p>>2]|0;
    _sqlite3DbFree($94,$95);
    $96 = HEAP32[$pNext>>2]|0;
    HEAP32[$p>>2] = $96;
   }
   $97 = HEAP32[$pHash>>2]|0;
   HEAP32[$p>>2] = $97;
  }
  $98 = HEAP32[$j>>2]|0;
  $99 = (($98) + 1)|0;
  HEAP32[$j>>2] = $99;
 }
 $100 = HEAP32[$0>>2]|0;
 $101 = (($100) + 428|0);
 $102 = (($101) + 8|0);
 $103 = HEAP32[$102>>2]|0;
 HEAP32[$i>>2] = $103;
 while(1) {
  $104 = HEAP32[$i>>2]|0;
  $105 = ($104|0)!=(0|0);
  if (!($105)) {
   break;
  }
  $106 = HEAP32[$i>>2]|0;
  $107 = (($106) + 8|0);
  $108 = HEAP32[$107>>2]|0;
  HEAP32[$pColl>>2] = $108;
  HEAP32[$j>>2] = 0;
  while(1) {
   $109 = HEAP32[$j>>2]|0;
   $110 = ($109|0)<(3);
   if (!($110)) {
    break;
   }
   $111 = HEAP32[$j>>2]|0;
   $112 = HEAP32[$pColl>>2]|0;
   $113 = (($112) + (($111*20)|0)|0);
   $114 = (($113) + 16|0);
   $115 = HEAP32[$114>>2]|0;
   $116 = ($115|0)!=(0|0);
   if ($116) {
    $117 = HEAP32[$j>>2]|0;
    $118 = HEAP32[$pColl>>2]|0;
    $119 = (($118) + (($117*20)|0)|0);
    $120 = (($119) + 16|0);
    $121 = HEAP32[$120>>2]|0;
    $122 = HEAP32[$j>>2]|0;
    $123 = HEAP32[$pColl>>2]|0;
    $124 = (($123) + (($122*20)|0)|0);
    $125 = (($124) + 8|0);
    $126 = HEAP32[$125>>2]|0;
    FUNCTION_TABLE_vi[$121 & 31]($126);
   }
   $127 = HEAP32[$j>>2]|0;
   $128 = (($127) + 1)|0;
   HEAP32[$j>>2] = $128;
  }
  $129 = HEAP32[$0>>2]|0;
  $130 = HEAP32[$pColl>>2]|0;
  _sqlite3DbFree($129,$130);
  $131 = HEAP32[$i>>2]|0;
  $132 = HEAP32[$131>>2]|0;
  HEAP32[$i>>2] = $132;
 }
 $133 = HEAP32[$0>>2]|0;
 $134 = (($133) + 428|0);
 _sqlite3HashClear($134);
 $135 = HEAP32[$0>>2]|0;
 $136 = (($135) + 308|0);
 $137 = (($136) + 8|0);
 $138 = HEAP32[$137>>2]|0;
 HEAP32[$i>>2] = $138;
 while(1) {
  $139 = HEAP32[$i>>2]|0;
  $140 = ($139|0)!=(0|0);
  if (!($140)) {
   break;
  }
  $141 = HEAP32[$i>>2]|0;
  $142 = (($141) + 8|0);
  $143 = HEAP32[$142>>2]|0;
  HEAP32[$pMod>>2] = $143;
  $144 = HEAP32[$pMod>>2]|0;
  $145 = (($144) + 12|0);
  $146 = HEAP32[$145>>2]|0;
  $147 = ($146|0)!=(0|0);
  if ($147) {
   $148 = HEAP32[$pMod>>2]|0;
   $149 = (($148) + 12|0);
   $150 = HEAP32[$149>>2]|0;
   $151 = HEAP32[$pMod>>2]|0;
   $152 = (($151) + 8|0);
   $153 = HEAP32[$152>>2]|0;
   FUNCTION_TABLE_vi[$150 & 31]($153);
  }
  $154 = HEAP32[$0>>2]|0;
  $155 = HEAP32[$pMod>>2]|0;
  _sqlite3DbFree($154,$155);
  $156 = HEAP32[$i>>2]|0;
  $157 = HEAP32[$156>>2]|0;
  HEAP32[$i>>2] = $157;
 }
 $158 = HEAP32[$0>>2]|0;
 $159 = (($158) + 308|0);
 _sqlite3HashClear($159);
 $160 = HEAP32[$0>>2]|0;
 _sqlite3Error($160,0);
 $161 = HEAP32[$0>>2]|0;
 $162 = (($161) + 232|0);
 $163 = HEAP32[$162>>2]|0;
 _sqlite3ValueFree($163);
 $164 = HEAP32[$0>>2]|0;
 $165 = (($164) + 76|0);
 HEAP32[$165>>2] = -1254786768;
 $166 = HEAP32[$0>>2]|0;
 $167 = HEAP32[$0>>2]|0;
 $168 = (($167) + 16|0);
 $169 = HEAP32[$168>>2]|0;
 $170 = (($169) + 16|0);
 $171 = (($170) + 12|0);
 $172 = HEAP32[$171>>2]|0;
 _sqlite3DbFree($166,$172);
 $173 = HEAP32[$0>>2]|0;
 $174 = (($173) + 76|0);
 HEAP32[$174>>2] = -1623446221;
 $175 = HEAP32[$0>>2]|0;
 $176 = (($175) + 248|0);
 $177 = (($176) + 3|0);
 $178 = HEAP8[$177>>0]|0;
 $179 = ($178<<24>>24)!=(0);
 if ($179) {
  $180 = HEAP32[$0>>2]|0;
  $181 = (($180) + 248|0);
  $182 = (($181) + 28|0);
  $183 = HEAP32[$182>>2]|0;
  _sqlite3_free($183);
 }
 $184 = HEAP32[$0>>2]|0;
 _sqlite3_free($184);
 STACKTOP = sp;return;
}
function _sqlite3_sql($pStmt) {
 $pStmt = $pStmt|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $p = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $p = sp;
 HEAP32[$0>>2] = $pStmt;
 $1 = HEAP32[$0>>2]|0;
 HEAP32[$p>>2] = $1;
 $2 = HEAP32[$p>>2]|0;
 $3 = ($2|0)!=(0|0);
 if (!($3)) {
  $14 = 0;
  STACKTOP = sp;return ($14|0);
 }
 $4 = HEAP32[$p>>2]|0;
 $5 = (($4) + 88|0);
 $6 = HEAP16[$5>>1]|0;
 $7 = ($6&65535) >>> 10;
 $8 = $7 & 1;
 $9 = $8&65535;
 $10 = ($9|0)!=(0);
 if (!($10)) {
  $14 = 0;
  STACKTOP = sp;return ($14|0);
 }
 $11 = HEAP32[$p>>2]|0;
 $12 = (($11) + 168|0);
 $13 = HEAP32[$12>>2]|0;
 $14 = $13;
 STACKTOP = sp;return ($14|0);
}
function _sqlite3_finalize($pStmt) {
 $pStmt = $pStmt|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $db = 0;
 var $rc = 0, $v = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 16|0;
 $1 = sp + 12|0;
 $rc = sp + 8|0;
 $v = sp + 4|0;
 $db = sp;
 HEAP32[$1>>2] = $pStmt;
 $2 = HEAP32[$1>>2]|0;
 $3 = ($2|0)==(0|0);
 do {
  if ($3) {
   HEAP32[$rc>>2] = 0;
  } else {
   $4 = HEAP32[$1>>2]|0;
   HEAP32[$v>>2] = $4;
   $5 = HEAP32[$v>>2]|0;
   $6 = HEAP32[$5>>2]|0;
   HEAP32[$db>>2] = $6;
   $7 = HEAP32[$v>>2]|0;
   $8 = (_vdbeSafety($7)|0);
   $9 = ($8|0)!=(0);
   if (!($9)) {
    $11 = HEAP32[$v>>2]|0;
    $12 = (_sqlite3VdbeFinalize($11)|0);
    HEAP32[$rc>>2] = $12;
    $13 = HEAP32[$db>>2]|0;
    $14 = HEAP32[$rc>>2]|0;
    $15 = (_sqlite3ApiExit($13,$14)|0);
    HEAP32[$rc>>2] = $15;
    $16 = HEAP32[$db>>2]|0;
    _sqlite3LeaveMutexAndCloseZombie($16);
    break;
   }
   $10 = (_sqlite3MisuseError(67386)|0);
   HEAP32[$0>>2] = $10;
   $18 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($18|0);
  }
 } while(0);
 $17 = HEAP32[$rc>>2]|0;
 HEAP32[$0>>2] = $17;
 $18 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($18|0);
}
function _vdbeSafety($p) {
 $p = $p|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $vararg_buffer = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $vararg_buffer = sp;
 $0 = sp + 8|0;
 $1 = sp + 4|0;
 HEAP32[$1>>2] = $p;
 $2 = HEAP32[$1>>2]|0;
 $3 = HEAP32[$2>>2]|0;
 $4 = ($3|0)==(0|0);
 if ($4) {
  _sqlite3_log(21,37056,$vararg_buffer);
  HEAP32[$0>>2] = 1;
  $5 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($5|0);
 } else {
  HEAP32[$0>>2] = 0;
  $5 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($5|0);
 }
 return 0|0;
}
function _sqlite3VdbeFinalize($p) {
 $p = $p|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $rc = sp;
 HEAP32[$0>>2] = $p;
 HEAP32[$rc>>2] = 0;
 $1 = HEAP32[$0>>2]|0;
 $2 = (($1) + 40|0);
 $3 = HEAP32[$2>>2]|0;
 $4 = ($3|0)==(-1108210269);
 if ($4) {
  label = 3;
 } else {
  $5 = HEAP32[$0>>2]|0;
  $6 = (($5) + 40|0);
  $7 = HEAP32[$6>>2]|0;
  $8 = ($7|0)==(1369188723);
  if ($8) {
   label = 3;
  }
 }
 if ((label|0) == 3) {
  $9 = HEAP32[$0>>2]|0;
  $10 = (_sqlite3VdbeReset($9)|0);
  HEAP32[$rc>>2] = $10;
 }
 $11 = HEAP32[$0>>2]|0;
 _sqlite3VdbeDelete($11);
 $12 = HEAP32[$rc>>2]|0;
 STACKTOP = sp;return ($12|0);
}
function _sqlite3ApiExit($db,$rc) {
 $db = $db|0;
 $rc = $rc|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0;
 var $8 = 0, $9 = 0, $or$cond = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $1 = sp + 4|0;
 $2 = sp;
 HEAP32[$1>>2] = $db;
 HEAP32[$2>>2] = $rc;
 $3 = HEAP32[$1>>2]|0;
 $4 = ($3|0)==(0|0);
 if ($4) {
  $5 = HEAP32[$2>>2]|0;
  $6 = $5 & 255;
  HEAP32[$0>>2] = $6;
  $21 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($21|0);
 }
 $7 = HEAP32[$1>>2]|0;
 $8 = (($7) + 64|0);
 $9 = HEAP8[$8>>0]|0;
 $10 = $9&255;
 $11 = ($10|0)!=(0);
 $12 = HEAP32[$2>>2]|0;
 $13 = ($12|0)==(3082);
 $or$cond = $11 | $13;
 if ($or$cond) {
  $14 = HEAP32[$1>>2]|0;
  $15 = (_apiOomError($14)|0);
  HEAP32[$0>>2] = $15;
  $21 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($21|0);
 } else {
  $16 = HEAP32[$2>>2]|0;
  $17 = HEAP32[$1>>2]|0;
  $18 = (($17) + 56|0);
  $19 = HEAP32[$18>>2]|0;
  $20 = $16 & $19;
  HEAP32[$0>>2] = $20;
  $21 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($21|0);
 }
 return 0|0;
}
function _sqlite3_reset($pStmt) {
 $pStmt = $pStmt|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $rc = 0, $v = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $rc = sp + 4|0;
 $v = sp;
 HEAP32[$0>>2] = $pStmt;
 $1 = HEAP32[$0>>2]|0;
 $2 = ($1|0)==(0|0);
 if ($2) {
  HEAP32[$rc>>2] = 0;
  $11 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($11|0);
 } else {
  $3 = HEAP32[$0>>2]|0;
  HEAP32[$v>>2] = $3;
  $4 = HEAP32[$v>>2]|0;
  $5 = (_sqlite3VdbeReset($4)|0);
  HEAP32[$rc>>2] = $5;
  $6 = HEAP32[$v>>2]|0;
  _sqlite3VdbeRewind($6);
  $7 = HEAP32[$v>>2]|0;
  $8 = HEAP32[$7>>2]|0;
  $9 = HEAP32[$rc>>2]|0;
  $10 = (_sqlite3ApiExit($8,$9)|0);
  HEAP32[$rc>>2] = $10;
  $11 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($11|0);
 }
 return 0|0;
}
function _sqlite3VdbeReset($p) {
 $p = $p|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $8 = 0, $9 = 0, $db = 0, $vararg_buffer = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $vararg_buffer = sp;
 $0 = sp + 8|0;
 $db = sp + 4|0;
 HEAP32[$0>>2] = $p;
 $1 = HEAP32[$0>>2]|0;
 $2 = HEAP32[$1>>2]|0;
 HEAP32[$db>>2] = $2;
 $3 = HEAP32[$0>>2]|0;
 (_sqlite3VdbeHalt($3)|0);
 $4 = HEAP32[$0>>2]|0;
 $5 = (($4) + 76|0);
 $6 = HEAP32[$5>>2]|0;
 $7 = ($6|0)>=(0);
 $8 = HEAP32[$0>>2]|0;
 if ($7) {
  (_sqlite3VdbeTransferError($8)|0);
  $9 = HEAP32[$db>>2]|0;
  $10 = HEAP32[$0>>2]|0;
  $11 = (($10) + 44|0);
  $12 = HEAP32[$11>>2]|0;
  _sqlite3DbFree($9,$12);
  $13 = HEAP32[$0>>2]|0;
  $14 = (($13) + 44|0);
  HEAP32[$14>>2] = 0;
  $15 = HEAP32[$0>>2]|0;
  $16 = (($15) + 88|0);
  $17 = HEAP16[$16>>1]|0;
  $18 = ($17&65535) >>> 6;
  $19 = $18 & 1;
  $20 = $19&65535;
  $21 = ($20|0)!=(0);
  if ($21) {
   $22 = HEAP32[$0>>2]|0;
   $23 = (($22) + 88|0);
   $24 = HEAP16[$23>>1]|0;
   $25 = $24 & -33;
   $26 = $25 | 32;
   HEAP16[$23>>1] = $26;
  }
 } else {
  $27 = (($8) + 80|0);
  $28 = HEAP32[$27>>2]|0;
  $29 = ($28|0)!=(0);
  if ($29) {
   $30 = HEAP32[$0>>2]|0;
   $31 = (($30) + 88|0);
   $32 = HEAP16[$31>>1]|0;
   $33 = ($32&65535) >>> 5;
   $34 = $33 & 1;
   $35 = $34&65535;
   $36 = ($35|0)!=(0);
   if ($36) {
    $37 = HEAP32[$db>>2]|0;
    $38 = HEAP32[$0>>2]|0;
    $39 = (($38) + 80|0);
    $40 = HEAP32[$39>>2]|0;
    $41 = HEAP32[$0>>2]|0;
    $42 = (($41) + 44|0);
    $43 = HEAP32[$42>>2]|0;
    $44 = ($43|0)!=(0|0);
    $45 = $44 ? 1200 : 0;
    $46 = HEAP32[$0>>2]|0;
    $47 = (($46) + 44|0);
    $48 = HEAP32[$47>>2]|0;
    HEAP32[$vararg_buffer>>2] = $48;
    _sqlite3ErrorWithMsg($37,$40,$45,$vararg_buffer);
    $49 = HEAP32[$db>>2]|0;
    $50 = HEAP32[$0>>2]|0;
    $51 = (($50) + 44|0);
    $52 = HEAP32[$51>>2]|0;
    _sqlite3DbFree($49,$52);
    $53 = HEAP32[$0>>2]|0;
    $54 = (($53) + 44|0);
    HEAP32[$54>>2] = 0;
   }
  }
 }
 $55 = HEAP32[$0>>2]|0;
 _Cleanup($55);
 $56 = HEAP32[$0>>2]|0;
 $57 = (($56) + 136|0);
 $58 = $57;
 $59 = $58;
 HEAP32[$59>>2] = 0;
 $60 = (($58) + 4)|0;
 $61 = $60;
 HEAP32[$61>>2] = 0;
 $62 = HEAP32[$0>>2]|0;
 $63 = (($62) + 40|0);
 HEAP32[$63>>2] = 649915045;
 $64 = HEAP32[$0>>2]|0;
 $65 = (($64) + 80|0);
 $66 = HEAP32[$65>>2]|0;
 $67 = HEAP32[$db>>2]|0;
 $68 = (($67) + 56|0);
 $69 = HEAP32[$68>>2]|0;
 $70 = $66 & $69;
 STACKTOP = sp;return ($70|0);
}
function _sqlite3VdbeRewind($p) {
 $p = $p|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $3 = 0, $4 = 0;
 var $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 HEAP32[$0>>2] = $p;
 $1 = HEAP32[$0>>2]|0;
 $2 = (($1) + 40|0);
 HEAP32[$2>>2] = -1108210269;
 $3 = HEAP32[$0>>2]|0;
 $4 = (($3) + 76|0);
 HEAP32[$4>>2] = -1;
 $5 = HEAP32[$0>>2]|0;
 $6 = (($5) + 80|0);
 HEAP32[$6>>2] = 0;
 $7 = HEAP32[$0>>2]|0;
 $8 = (($7) + 86|0);
 HEAP8[$8>>0] = 2;
 $9 = HEAP32[$0>>2]|0;
 $10 = (($9) + 40|0);
 HEAP32[$10>>2] = -1108210269;
 $11 = HEAP32[$0>>2]|0;
 $12 = (($11) + 92|0);
 HEAP32[$12>>2] = 0;
 $13 = HEAP32[$0>>2]|0;
 $14 = (($13) + 72|0);
 HEAP32[$14>>2] = 1;
 $15 = HEAP32[$0>>2]|0;
 $16 = (($15) + 87|0);
 HEAP8[$16>>0] = -1;
 $17 = HEAP32[$0>>2]|0;
 $18 = (($17) + 104|0);
 HEAP32[$18>>2] = 0;
 $19 = HEAP32[$0>>2]|0;
 $20 = (($19) + 144|0);
 $21 = $20;
 $22 = $21;
 HEAP32[$22>>2] = 0;
 $23 = (($21) + 4)|0;
 $24 = $23;
 HEAP32[$24>>2] = 0;
 STACKTOP = sp;return;
}
function _sqlite3_clear_bindings($pStmt) {
 $pStmt = $pStmt|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $i = 0, $p = 0;
 var $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $i = sp + 8|0;
 $rc = sp + 4|0;
 $p = sp;
 HEAP32[$0>>2] = $pStmt;
 HEAP32[$rc>>2] = 0;
 $1 = HEAP32[$0>>2]|0;
 HEAP32[$p>>2] = $1;
 HEAP32[$i>>2] = 0;
 while(1) {
  $2 = HEAP32[$i>>2]|0;
  $3 = HEAP32[$p>>2]|0;
  $4 = (($3) + 68|0);
  $5 = HEAP16[$4>>1]|0;
  $6 = $5 << 16 >> 16;
  $7 = ($2|0)<($6|0);
  if (!($7)) {
   break;
  }
  $8 = HEAP32[$i>>2]|0;
  $9 = HEAP32[$p>>2]|0;
  $10 = (($9) + 60|0);
  $11 = HEAP32[$10>>2]|0;
  $12 = (($11) + (($8*40)|0)|0);
  _sqlite3VdbeMemRelease($12);
  $13 = HEAP32[$i>>2]|0;
  $14 = HEAP32[$p>>2]|0;
  $15 = (($14) + 60|0);
  $16 = HEAP32[$15>>2]|0;
  $17 = (($16) + (($13*40)|0)|0);
  $18 = (($17) + 8|0);
  HEAP16[$18>>1] = 1;
  $19 = HEAP32[$i>>2]|0;
  $20 = (($19) + 1)|0;
  HEAP32[$i>>2] = $20;
 }
 $21 = HEAP32[$p>>2]|0;
 $22 = (($21) + 88|0);
 $23 = HEAP16[$22>>1]|0;
 $24 = ($23&65535) >>> 10;
 $25 = $24 & 1;
 $26 = $25&65535;
 $27 = ($26|0)!=(0);
 if (!($27)) {
  $37 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($37|0);
 }
 $28 = HEAP32[$p>>2]|0;
 $29 = (($28) + 188|0);
 $30 = HEAP32[$29>>2]|0;
 $31 = ($30|0)!=(0);
 if (!($31)) {
  $37 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($37|0);
 }
 $32 = HEAP32[$p>>2]|0;
 $33 = (($32) + 88|0);
 $34 = HEAP16[$33>>1]|0;
 $35 = $34 & -33;
 $36 = $35 | 32;
 HEAP16[$33>>1] = $36;
 $37 = HEAP32[$rc>>2]|0;
 STACKTOP = sp;return ($37|0);
}
function _sqlite3VdbeMemRelease($p) {
 $p = $p|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 HEAP32[$0>>2] = $p;
 $1 = HEAP32[$0>>2]|0;
 $2 = (($1) + 8|0);
 $3 = HEAP16[$2>>1]|0;
 $4 = $3&65535;
 $5 = $4 & 9312;
 $6 = ($5|0)!=(0);
 if (!($6)) {
  $7 = HEAP32[$0>>2]|0;
  $8 = (($7) + 24|0);
  $9 = HEAP32[$8>>2]|0;
  $10 = ($9|0)!=(0);
  if (!($10)) {
   STACKTOP = sp;return;
  }
 }
 $11 = HEAP32[$0>>2]|0;
 _vdbeMemClear($11);
 STACKTOP = sp;return;
}
function _sqlite3_value_blob($pVal) {
 $pVal = $pVal|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $p = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $1 = sp + 4|0;
 $p = sp;
 HEAP32[$1>>2] = $pVal;
 $2 = HEAP32[$1>>2]|0;
 HEAP32[$p>>2] = $2;
 $3 = HEAP32[$p>>2]|0;
 $4 = (($3) + 8|0);
 $5 = HEAP16[$4>>1]|0;
 $6 = $5&65535;
 $7 = $6 & 18;
 $8 = ($7|0)!=(0);
 if (!($8)) {
  $24 = HEAP32[$1>>2]|0;
  $25 = (_sqlite3_value_text($24)|0);
  HEAP32[$0>>2] = $25;
  $26 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($26|0);
 }
 $9 = HEAP32[$p>>2]|0;
 (_sqlite3VdbeMemExpandBlob($9)|0);
 $10 = HEAP32[$p>>2]|0;
 $11 = (($10) + 8|0);
 $12 = HEAP16[$11>>1]|0;
 $13 = $12&65535;
 $14 = $13 | 16;
 $15 = $14&65535;
 HEAP16[$11>>1] = $15;
 $16 = HEAP32[$p>>2]|0;
 $17 = (($16) + 12|0);
 $18 = HEAP32[$17>>2]|0;
 $19 = ($18|0)!=(0);
 if ($19) {
  $20 = HEAP32[$p>>2]|0;
  $21 = (($20) + 16|0);
  $22 = HEAP32[$21>>2]|0;
  $23 = $22;
 } else {
  $23 = 0;
 }
 HEAP32[$0>>2] = $23;
 $26 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($26|0);
}
function _sqlite3VdbeMemExpandBlob($pMem) {
 $pMem = $pMem|0;
 var $$ = 0, $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0;
 var $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $5 = 0, $6 = 0;
 var $7 = 0, $8 = 0, $9 = 0, $nByte = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $1 = sp + 4|0;
 $nByte = sp;
 HEAP32[$1>>2] = $pMem;
 $2 = HEAP32[$1>>2]|0;
 $3 = (($2) + 8|0);
 $4 = HEAP16[$3>>1]|0;
 $5 = $4&65535;
 $6 = $5 & 16384;
 $7 = ($6|0)!=(0);
 do {
  if ($7) {
   $8 = HEAP32[$1>>2]|0;
   $9 = (($8) + 12|0);
   $10 = HEAP32[$9>>2]|0;
   $11 = HEAP32[$1>>2]|0;
   $12 = HEAP32[$11>>2]|0;
   $13 = (($10) + ($12))|0;
   HEAP32[$nByte>>2] = $13;
   $14 = HEAP32[$nByte>>2]|0;
   $15 = ($14|0)<=(0);
   $$ = $15 ? 1 : $13;
   HEAP32[$nByte>>2] = $$;
   $16 = HEAP32[$1>>2]|0;
   $17 = HEAP32[$nByte>>2]|0;
   $18 = (_sqlite3VdbeMemGrow($16,$17,1)|0);
   $19 = ($18|0)!=(0);
   if (!($19)) {
    $20 = HEAP32[$1>>2]|0;
    $21 = (($20) + 12|0);
    $22 = HEAP32[$21>>2]|0;
    $23 = HEAP32[$1>>2]|0;
    $24 = (($23) + 16|0);
    $25 = HEAP32[$24>>2]|0;
    $26 = (($25) + ($22)|0);
    $27 = HEAP32[$1>>2]|0;
    $28 = HEAP32[$27>>2]|0;
    _memset(($26|0),0,($28|0))|0;
    $29 = HEAP32[$1>>2]|0;
    $30 = HEAP32[$29>>2]|0;
    $31 = HEAP32[$1>>2]|0;
    $32 = (($31) + 12|0);
    $33 = HEAP32[$32>>2]|0;
    $34 = (($33) + ($30))|0;
    HEAP32[$32>>2] = $34;
    $35 = HEAP32[$1>>2]|0;
    $36 = (($35) + 8|0);
    $37 = HEAP16[$36>>1]|0;
    $38 = $37&65535;
    $39 = $38 & -16897;
    $40 = $39&65535;
    HEAP16[$36>>1] = $40;
    break;
   }
   HEAP32[$0>>2] = 7;
   $41 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($41|0);
  }
 } while(0);
 HEAP32[$0>>2] = 0;
 $41 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($41|0);
}
function _sqlite3_value_text($pVal) {
 $pVal = $pVal|0;
 var $0 = 0, $1 = 0, $2 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 HEAP32[$0>>2] = $pVal;
 $1 = HEAP32[$0>>2]|0;
 $2 = (_sqlite3ValueText($1,1)|0);
 STACKTOP = sp;return ($2|0);
}
function _sqlite3_value_bytes($pVal) {
 $pVal = $pVal|0;
 var $0 = 0, $1 = 0, $2 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 HEAP32[$0>>2] = $pVal;
 $1 = HEAP32[$0>>2]|0;
 $2 = (_sqlite3ValueBytes($1,1)|0);
 STACKTOP = sp;return ($2|0);
}
function _sqlite3ValueBytes($pVal,$enc) {
 $pVal = $pVal|0;
 $enc = $enc|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $p = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $1 = sp + 4|0;
 $2 = sp + 12|0;
 $p = sp;
 HEAP32[$1>>2] = $pVal;
 HEAP8[$2>>0] = $enc;
 $3 = HEAP32[$1>>2]|0;
 HEAP32[$p>>2] = $3;
 $4 = HEAP32[$p>>2]|0;
 $5 = (($4) + 8|0);
 $6 = HEAP16[$5>>1]|0;
 $7 = $6&65535;
 $8 = $7 & 16;
 $9 = ($8|0)!=(0);
 if (!($9)) {
  $10 = HEAP32[$1>>2]|0;
  $11 = HEAP8[$2>>0]|0;
  $12 = (_sqlite3ValueText($10,$11)|0);
  $13 = ($12|0)!=(0|0);
  if (!($13)) {
   HEAP32[$0>>2] = 0;
   $26 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($26|0);
  }
 }
 $14 = HEAP32[$p>>2]|0;
 $15 = (($14) + 8|0);
 $16 = HEAP16[$15>>1]|0;
 $17 = $16&65535;
 $18 = $17 & 16384;
 $19 = ($18|0)!=(0);
 $20 = HEAP32[$p>>2]|0;
 $21 = (($20) + 12|0);
 $22 = HEAP32[$21>>2]|0;
 if ($19) {
  $23 = HEAP32[$p>>2]|0;
  $24 = HEAP32[$23>>2]|0;
  $25 = (($22) + ($24))|0;
  HEAP32[$0>>2] = $25;
  $26 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($26|0);
 } else {
  HEAP32[$0>>2] = $22;
  $26 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($26|0);
 }
 return 0|0;
}
function _sqlite3_value_double($pVal) {
 $pVal = $pVal|0;
 var $0 = 0, $1 = 0, $2 = 0.0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 HEAP32[$0>>2] = $pVal;
 $1 = HEAP32[$0>>2]|0;
 $2 = (+_sqlite3VdbeRealValue($1));
 STACKTOP = sp;return (+$2);
}
function _sqlite3VdbeRealValue($pMem) {
 $pMem = $pMem|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0.0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0.0, $38 = 0.0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0.0, $val = 0;
 var label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 8|0;
 $1 = sp + 16|0;
 $val = sp;
 HEAP32[$1>>2] = $pMem;
 $2 = HEAP32[$1>>2]|0;
 $3 = (($2) + 8|0);
 $4 = HEAP16[$3>>1]|0;
 $5 = $4&65535;
 $6 = $5 & 8;
 $7 = ($6|0)!=(0);
 $8 = HEAP32[$1>>2]|0;
 if ($7) {
  $9 = +HEAPF64[$8>>3];
  HEAPF64[$0>>3] = $9;
  $38 = +HEAPF64[$0>>3];
  STACKTOP = sp;return (+$38);
 }
 $10 = (($8) + 8|0);
 $11 = HEAP16[$10>>1]|0;
 $12 = $11&65535;
 $13 = $12 & 4;
 $14 = ($13|0)!=(0);
 $15 = HEAP32[$1>>2]|0;
 if ($14) {
  $16 = $15;
  $17 = $16;
  $18 = HEAP32[$17>>2]|0;
  $19 = (($16) + 4)|0;
  $20 = $19;
  $21 = HEAP32[$20>>2]|0;
  $22 = (+($18>>>0)) + (4294967296.0*(+($21|0)));
  HEAPF64[$0>>3] = $22;
  $38 = +HEAPF64[$0>>3];
  STACKTOP = sp;return (+$38);
 }
 $23 = (($15) + 8|0);
 $24 = HEAP16[$23>>1]|0;
 $25 = $24&65535;
 $26 = $25 & 18;
 $27 = ($26|0)!=(0);
 if ($27) {
  HEAPF64[$val>>3] = 0.0;
  $28 = HEAP32[$1>>2]|0;
  $29 = (($28) + 16|0);
  $30 = HEAP32[$29>>2]|0;
  $31 = HEAP32[$1>>2]|0;
  $32 = (($31) + 12|0);
  $33 = HEAP32[$32>>2]|0;
  $34 = HEAP32[$1>>2]|0;
  $35 = (($34) + 10|0);
  $36 = HEAP8[$35>>0]|0;
  (_sqlite3AtoF($30,$val,$33,$36)|0);
  $37 = +HEAPF64[$val>>3];
  HEAPF64[$0>>3] = $37;
  $38 = +HEAPF64[$0>>3];
  STACKTOP = sp;return (+$38);
 } else {
  HEAPF64[$0>>3] = 0.0;
  $38 = +HEAPF64[$0>>3];
  STACKTOP = sp;return (+$38);
 }
 return +0;
}
function _sqlite3_value_int($pVal) {
 $pVal = $pVal|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 HEAP32[$0>>2] = $pVal;
 $1 = HEAP32[$0>>2]|0;
 $2 = (_sqlite3VdbeIntValue($1)|0);
 $3 = tempRet0;
 STACKTOP = sp;return ($2|0);
}
function _sqlite3VdbeIntValue($pMem) {
 $pMem = $pMem|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0.0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $7 = 0, $8 = 0, $9 = 0, $flags = 0, $value = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 8|0;
 $1 = sp + 20|0;
 $flags = sp + 16|0;
 $value = sp;
 HEAP32[$1>>2] = $pMem;
 $2 = HEAP32[$1>>2]|0;
 $3 = (($2) + 8|0);
 $4 = HEAP16[$3>>1]|0;
 $5 = $4&65535;
 HEAP32[$flags>>2] = $5;
 $6 = HEAP32[$flags>>2]|0;
 $7 = $6 & 4;
 $8 = ($7|0)!=(0);
 do {
  if ($8) {
   $9 = HEAP32[$1>>2]|0;
   $10 = $9;
   $11 = $10;
   $12 = HEAP32[$11>>2]|0;
   $13 = (($10) + 4)|0;
   $14 = $13;
   $15 = HEAP32[$14>>2]|0;
   $16 = $0;
   $17 = $16;
   HEAP32[$17>>2] = $12;
   $18 = (($16) + 4)|0;
   $19 = $18;
   HEAP32[$19>>2] = $15;
  } else {
   $20 = HEAP32[$flags>>2]|0;
   $21 = $20 & 8;
   $22 = ($21|0)!=(0);
   if ($22) {
    $23 = HEAP32[$1>>2]|0;
    $24 = +HEAPF64[$23>>3];
    $25 = (_doubleToInt64($24)|0);
    $26 = tempRet0;
    $27 = $0;
    $28 = $27;
    HEAP32[$28>>2] = $25;
    $29 = (($27) + 4)|0;
    $30 = $29;
    HEAP32[$30>>2] = $26;
    break;
   }
   $31 = HEAP32[$flags>>2]|0;
   $32 = $31 & 18;
   $33 = ($32|0)!=(0);
   if ($33) {
    $34 = $value;
    $35 = $34;
    HEAP32[$35>>2] = 0;
    $36 = (($34) + 4)|0;
    $37 = $36;
    HEAP32[$37>>2] = 0;
    $38 = HEAP32[$1>>2]|0;
    $39 = (($38) + 16|0);
    $40 = HEAP32[$39>>2]|0;
    $41 = HEAP32[$1>>2]|0;
    $42 = (($41) + 12|0);
    $43 = HEAP32[$42>>2]|0;
    $44 = HEAP32[$1>>2]|0;
    $45 = (($44) + 10|0);
    $46 = HEAP8[$45>>0]|0;
    (_sqlite3Atoi64($40,$value,$43,$46)|0);
    $47 = $value;
    $48 = $47;
    $49 = HEAP32[$48>>2]|0;
    $50 = (($47) + 4)|0;
    $51 = $50;
    $52 = HEAP32[$51>>2]|0;
    $53 = $0;
    $54 = $53;
    HEAP32[$54>>2] = $49;
    $55 = (($53) + 4)|0;
    $56 = $55;
    HEAP32[$56>>2] = $52;
    break;
   } else {
    $57 = $0;
    $58 = $57;
    HEAP32[$58>>2] = 0;
    $59 = (($57) + 4)|0;
    $60 = $59;
    HEAP32[$60>>2] = 0;
    break;
   }
  }
 } while(0);
 $61 = $0;
 $62 = $61;
 $63 = HEAP32[$62>>2]|0;
 $64 = (($61) + 4)|0;
 $65 = $64;
 $66 = HEAP32[$65>>2]|0;
 tempRet0 = $66;
 STACKTOP = sp;return ($63|0);
}
function _sqlite3_value_int64($pVal) {
 $pVal = $pVal|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 HEAP32[$0>>2] = $pVal;
 $1 = HEAP32[$0>>2]|0;
 $2 = (_sqlite3VdbeIntValue($1)|0);
 $3 = tempRet0;
 tempRet0 = $3;
 STACKTOP = sp;return ($2|0);
}
function _sqlite3ValueText($pVal,$enc) {
 $pVal = $pVal|0;
 $enc = $enc|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $1 = sp;
 $2 = sp + 8|0;
 HEAP32[$1>>2] = $pVal;
 HEAP8[$2>>0] = $enc;
 $3 = HEAP32[$1>>2]|0;
 $4 = ($3|0)!=(0|0);
 do {
  if ($4) {
   $5 = HEAP32[$1>>2]|0;
   $6 = (($5) + 8|0);
   $7 = HEAP16[$6>>1]|0;
   $8 = $7&65535;
   $9 = $8 & 514;
   $10 = ($9|0)==(514);
   if ($10) {
    $11 = HEAP32[$1>>2]|0;
    $12 = (($11) + 10|0);
    $13 = HEAP8[$12>>0]|0;
    $14 = $13&255;
    $15 = HEAP8[$2>>0]|0;
    $16 = $15&255;
    $17 = ($14|0)==($16|0);
    if ($17) {
     $18 = HEAP32[$1>>2]|0;
     $19 = (($18) + 16|0);
     $20 = HEAP32[$19>>2]|0;
     HEAP32[$0>>2] = $20;
     break;
    }
   }
   $21 = HEAP32[$1>>2]|0;
   $22 = (($21) + 8|0);
   $23 = HEAP16[$22>>1]|0;
   $24 = $23&65535;
   $25 = $24 & 1;
   $26 = ($25|0)!=(0);
   if ($26) {
    HEAP32[$0>>2] = 0;
    break;
   } else {
    $27 = HEAP32[$1>>2]|0;
    $28 = HEAP8[$2>>0]|0;
    $29 = (_valueToText($27,$28)|0);
    HEAP32[$0>>2] = $29;
    break;
   }
  } else {
   HEAP32[$0>>2] = 0;
  }
 } while(0);
 $30 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($30|0);
}
function _sqlite3_value_type($pVal) {
 $pVal = $pVal|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 HEAP32[$0>>2] = $pVal;
 $1 = HEAP32[$0>>2]|0;
 $2 = (($1) + 8|0);
 $3 = HEAP16[$2>>1]|0;
 $4 = $3&65535;
 $5 = $4 & 31;
 $6 = (1144 + ($5)|0);
 $7 = HEAP8[$6>>0]|0;
 $8 = $7&255;
 STACKTOP = sp;return ($8|0);
}
function _sqlite3_result_blob($pCtx,$z,$n,$xDel) {
 $pCtx = $pCtx|0;
 $z = $z|0;
 $n = $n|0;
 $xDel = $xDel|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $1 = sp + 8|0;
 $2 = sp + 4|0;
 $3 = sp;
 HEAP32[$0>>2] = $pCtx;
 HEAP32[$1>>2] = $z;
 HEAP32[$2>>2] = $n;
 HEAP32[$3>>2] = $xDel;
 $4 = HEAP32[$0>>2]|0;
 $5 = HEAP32[$1>>2]|0;
 $6 = HEAP32[$2>>2]|0;
 $7 = HEAP32[$3>>2]|0;
 _setResultStrOrError($4,$5,$6,0,$7);
 STACKTOP = sp;return;
}
function _setResultStrOrError($pCtx,$z,$n,$enc,$xDel) {
 $pCtx = $pCtx|0;
 $z = $z|0;
 $n = $n|0;
 $enc = $enc|0;
 $xDel = $xDel|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 12|0;
 $1 = sp + 8|0;
 $2 = sp + 4|0;
 $3 = sp + 16|0;
 $4 = sp;
 HEAP32[$0>>2] = $pCtx;
 HEAP32[$1>>2] = $z;
 HEAP32[$2>>2] = $n;
 HEAP8[$3>>0] = $enc;
 HEAP32[$4>>2] = $xDel;
 $5 = HEAP32[$0>>2]|0;
 $6 = HEAP32[$5>>2]|0;
 $7 = HEAP32[$1>>2]|0;
 $8 = HEAP32[$2>>2]|0;
 $9 = HEAP8[$3>>0]|0;
 $10 = HEAP32[$4>>2]|0;
 $11 = (_sqlite3VdbeMemSetStr($6,$7,$8,$9,$10)|0);
 $12 = ($11|0)==(18);
 if (!($12)) {
  STACKTOP = sp;return;
 }
 $13 = HEAP32[$0>>2]|0;
 _sqlite3_result_error_toobig($13);
 STACKTOP = sp;return;
}
function _sqlite3_result_blob64($pCtx,$z,$0,$1,$xDel) {
 $pCtx = $pCtx|0;
 $z = $z|0;
 $0 = $0|0;
 $1 = $1|0;
 $xDel = $xDel|0;
 var $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0;
 var $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $2 = sp + 16|0;
 $3 = sp + 12|0;
 $4 = sp;
 $5 = sp + 8|0;
 HEAP32[$2>>2] = $pCtx;
 HEAP32[$3>>2] = $z;
 $6 = $4;
 $7 = $6;
 HEAP32[$7>>2] = $0;
 $8 = (($6) + 4)|0;
 $9 = $8;
 HEAP32[$9>>2] = $1;
 HEAP32[$5>>2] = $xDel;
 $10 = $4;
 $11 = $10;
 $12 = HEAP32[$11>>2]|0;
 $13 = (($10) + 4)|0;
 $14 = $13;
 $15 = HEAP32[$14>>2]|0;
 $16 = ($15>>>0)>(0);
 $17 = ($12>>>0)>(2147483647);
 $18 = ($15|0)==(0);
 $19 = $18 & $17;
 $20 = $16 | $19;
 if ($20) {
  $21 = HEAP32[$3>>2]|0;
  $22 = HEAP32[$5>>2]|0;
  $23 = HEAP32[$2>>2]|0;
  (_invokeValueDestructor($21,$22,$23)|0);
  STACKTOP = sp;return;
 } else {
  $24 = HEAP32[$2>>2]|0;
  $25 = HEAP32[$3>>2]|0;
  $26 = $4;
  $27 = $26;
  $28 = HEAP32[$27>>2]|0;
  $29 = (($26) + 4)|0;
  $30 = $29;
  $31 = HEAP32[$30>>2]|0;
  $32 = HEAP32[$5>>2]|0;
  _setResultStrOrError($24,$25,$28,0,$32);
  STACKTOP = sp;return;
 }
}
function _invokeValueDestructor($p,$xDel,$pCtx) {
 $p = $p|0;
 $xDel = $xDel|0;
 $pCtx = $pCtx|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $or$cond = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $1 = sp + 4|0;
 $2 = sp;
 HEAP32[$0>>2] = $p;
 HEAP32[$1>>2] = $xDel;
 HEAP32[$2>>2] = $pCtx;
 $3 = HEAP32[$1>>2]|0;
 $4 = ($3|0)==(0|0);
 $5 = HEAP32[$1>>2]|0;
 $6 = ($5|0)==((-1)|0);
 $or$cond = $4 | $6;
 if (!($or$cond)) {
  $7 = HEAP32[$1>>2]|0;
  $8 = HEAP32[$0>>2]|0;
  FUNCTION_TABLE_vi[$7 & 31]($8);
 }
 $9 = HEAP32[$2>>2]|0;
 $10 = ($9|0)!=(0|0);
 if (!($10)) {
  STACKTOP = sp;return 18;
 }
 $11 = HEAP32[$2>>2]|0;
 _sqlite3_result_error_toobig($11);
 STACKTOP = sp;return 18;
}
function _sqlite3_result_double($pCtx,$rVal) {
 $pCtx = $pCtx|0;
 $rVal = +$rVal;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0.0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $1 = sp;
 HEAP32[$0>>2] = $pCtx;
 HEAPF64[$1>>3] = $rVal;
 $2 = HEAP32[$0>>2]|0;
 $3 = HEAP32[$2>>2]|0;
 $4 = +HEAPF64[$1>>3];
 _sqlite3VdbeMemSetDouble($3,$4);
 STACKTOP = sp;return;
}
function _sqlite3VdbeMemSetDouble($pMem,$val) {
 $pMem = $pMem|0;
 $val = +$val;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0.0, $4 = 0, $5 = 0, $6 = 0.0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $1 = sp;
 HEAP32[$0>>2] = $pMem;
 HEAPF64[$1>>3] = $val;
 $2 = HEAP32[$0>>2]|0;
 _sqlite3VdbeMemSetNull($2);
 $3 = +HEAPF64[$1>>3];
 $4 = (_sqlite3IsNaN($3)|0);
 $5 = ($4|0)!=(0);
 if ($5) {
  STACKTOP = sp;return;
 }
 $6 = +HEAPF64[$1>>3];
 $7 = HEAP32[$0>>2]|0;
 HEAPF64[$7>>3] = $6;
 $8 = HEAP32[$0>>2]|0;
 $9 = (($8) + 8|0);
 HEAP16[$9>>1] = 8;
 STACKTOP = sp;return;
}
function _sqlite3_result_error($pCtx,$z,$n) {
 $pCtx = $pCtx|0;
 $z = $z|0;
 $n = $n|0;
 var $0 = 0, $1 = 0, $10 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $1 = sp + 4|0;
 $2 = sp;
 HEAP32[$0>>2] = $pCtx;
 HEAP32[$1>>2] = $z;
 HEAP32[$2>>2] = $n;
 $3 = HEAP32[$0>>2]|0;
 $4 = (($3) + 20|0);
 HEAP32[$4>>2] = 1;
 $5 = HEAP32[$0>>2]|0;
 $6 = (($5) + 25|0);
 HEAP8[$6>>0] = 1;
 $7 = HEAP32[$0>>2]|0;
 $8 = HEAP32[$7>>2]|0;
 $9 = HEAP32[$1>>2]|0;
 $10 = HEAP32[$2>>2]|0;
 (_sqlite3VdbeMemSetStr($8,$9,$10,1,(-1))|0);
 STACKTOP = sp;return;
}
function _sqlite3VdbeMemSetStr($pMem,$z,$n,$enc,$xDel) {
 $pMem = $pMem|0;
 $z = $z|0;
 $n = $n|0;
 $enc = $enc|0;
 $xDel = $xDel|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0;
 var $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0;
 var $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0;
 var $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0;
 var $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $flags = 0;
 var $iLimit = 0, $nAlloc = 0, $nByte = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $0 = sp + 28|0;
 $1 = sp + 24|0;
 $2 = sp + 20|0;
 $3 = sp + 16|0;
 $4 = sp + 34|0;
 $5 = sp + 12|0;
 $nByte = sp + 8|0;
 $iLimit = sp + 4|0;
 $flags = sp + 32|0;
 $nAlloc = sp;
 HEAP32[$1>>2] = $pMem;
 HEAP32[$2>>2] = $z;
 HEAP32[$3>>2] = $n;
 HEAP8[$4>>0] = $enc;
 HEAP32[$5>>2] = $xDel;
 $6 = HEAP32[$3>>2]|0;
 HEAP32[$nByte>>2] = $6;
 HEAP16[$flags>>1] = 0;
 $7 = HEAP32[$2>>2]|0;
 $8 = ($7|0)!=(0|0);
 $9 = HEAP32[$1>>2]|0;
 if (!($8)) {
  _sqlite3VdbeMemSetNull($9);
  HEAP32[$0>>2] = 0;
  $138 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($138|0);
 }
 $10 = (($9) + 32|0);
 $11 = HEAP32[$10>>2]|0;
 $12 = ($11|0)!=(0|0);
 if ($12) {
  $13 = HEAP32[$1>>2]|0;
  $14 = (($13) + 32|0);
  $15 = HEAP32[$14>>2]|0;
  $16 = (($15) + 88|0);
  $17 = HEAP32[$16>>2]|0;
  HEAP32[$iLimit>>2] = $17;
 } else {
  HEAP32[$iLimit>>2] = 1000000000;
 }
 $18 = HEAP8[$4>>0]|0;
 $19 = $18&255;
 $20 = ($19|0)==(0);
 $21 = $20 ? 16 : 2;
 $22 = $21&65535;
 HEAP16[$flags>>1] = $22;
 $23 = HEAP32[$nByte>>2]|0;
 $24 = ($23|0)<(0);
 if ($24) {
  $25 = HEAP8[$4>>0]|0;
  $26 = $25&255;
  $27 = ($26|0)==(1);
  L11: do {
   if ($27) {
    $28 = HEAP32[$2>>2]|0;
    $29 = (_sqlite3Strlen30($28)|0);
    HEAP32[$nByte>>2] = $29;
    $30 = HEAP32[$nByte>>2]|0;
    $31 = HEAP32[$iLimit>>2]|0;
    $32 = ($30|0)>($31|0);
    if ($32) {
     $33 = HEAP32[$iLimit>>2]|0;
     $34 = (($33) + 1)|0;
     HEAP32[$nByte>>2] = $34;
    }
   } else {
    HEAP32[$nByte>>2] = 0;
    while(1) {
     $35 = HEAP32[$nByte>>2]|0;
     $36 = HEAP32[$iLimit>>2]|0;
     $37 = ($35|0)<=($36|0);
     if (!($37)) {
      break L11;
     }
     $38 = HEAP32[$nByte>>2]|0;
     $39 = HEAP32[$2>>2]|0;
     $40 = (($39) + ($38)|0);
     $41 = HEAP8[$40>>0]|0;
     $42 = $41 << 24 >> 24;
     $43 = HEAP32[$nByte>>2]|0;
     $44 = (($43) + 1)|0;
     $45 = HEAP32[$2>>2]|0;
     $46 = (($45) + ($44)|0);
     $47 = HEAP8[$46>>0]|0;
     $48 = $47 << 24 >> 24;
     $49 = $42 | $48;
     $50 = ($49|0)!=(0);
     if (!($50)) {
      break L11;
     }
     $51 = HEAP32[$nByte>>2]|0;
     $52 = (($51) + 2)|0;
     HEAP32[$nByte>>2] = $52;
    }
   }
  } while(0);
  $53 = HEAP16[$flags>>1]|0;
  $54 = $53&65535;
  $55 = $54 | 512;
  $56 = $55&65535;
  HEAP16[$flags>>1] = $56;
 }
 $57 = HEAP32[$5>>2]|0;
 $58 = ($57|0)==((-1)|0);
 do {
  if ($58) {
   $59 = HEAP32[$nByte>>2]|0;
   HEAP32[$nAlloc>>2] = $59;
   $60 = HEAP16[$flags>>1]|0;
   $61 = $60&65535;
   $62 = $61 & 512;
   $63 = ($62|0)!=(0);
   if ($63) {
    $64 = HEAP8[$4>>0]|0;
    $65 = $64&255;
    $66 = ($65|0)==(1);
    $67 = $66 ? 1 : 2;
    $68 = HEAP32[$nAlloc>>2]|0;
    $69 = (($68) + ($67))|0;
    HEAP32[$nAlloc>>2] = $69;
   }
   $70 = HEAP32[$nByte>>2]|0;
   $71 = HEAP32[$iLimit>>2]|0;
   $72 = ($70|0)>($71|0);
   if ($72) {
    HEAP32[$0>>2] = 18;
    $138 = HEAP32[$0>>2]|0;
    STACKTOP = sp;return ($138|0);
   }
   $73 = HEAP32[$1>>2]|0;
   $74 = HEAP32[$nAlloc>>2]|0;
   $75 = ($74|0)>(32);
   $76 = HEAP32[$nAlloc>>2]|0;
   $77 = $75 ? $76 : 32;
   $78 = (_sqlite3VdbeMemClearAndResize($73,$77)|0);
   $79 = ($78|0)!=(0);
   if (!($79)) {
    $80 = HEAP32[$1>>2]|0;
    $81 = (($80) + 16|0);
    $82 = HEAP32[$81>>2]|0;
    $83 = HEAP32[$2>>2]|0;
    $84 = HEAP32[$nAlloc>>2]|0;
    _memcpy(($82|0),($83|0),($84|0))|0;
    break;
   }
   HEAP32[$0>>2] = 7;
   $138 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($138|0);
  } else {
   $85 = HEAP32[$5>>2]|0;
   $86 = ($85|0)==(16|0);
   $87 = HEAP32[$1>>2]|0;
   _sqlite3VdbeMemRelease($87);
   $88 = HEAP32[$2>>2]|0;
   $89 = HEAP32[$1>>2]|0;
   $90 = (($89) + 16|0);
   HEAP32[$90>>2] = $88;
   if ($86) {
    $91 = HEAP32[$1>>2]|0;
    $92 = (($91) + 20|0);
    HEAP32[$92>>2] = $88;
    $93 = HEAP32[$1>>2]|0;
    $94 = (($93) + 32|0);
    $95 = HEAP32[$94>>2]|0;
    $96 = HEAP32[$1>>2]|0;
    $97 = (($96) + 20|0);
    $98 = HEAP32[$97>>2]|0;
    $99 = (_sqlite3DbMallocSize($95,$98)|0);
    $100 = HEAP32[$1>>2]|0;
    $101 = (($100) + 24|0);
    HEAP32[$101>>2] = $99;
    break;
   } else {
    $102 = HEAP32[$5>>2]|0;
    $103 = HEAP32[$1>>2]|0;
    $104 = (($103) + 36|0);
    HEAP32[$104>>2] = $102;
    $105 = HEAP32[$5>>2]|0;
    $106 = ($105|0)==(0|0);
    $107 = $106 ? 2048 : 1024;
    $108 = HEAP16[$flags>>1]|0;
    $109 = $108&65535;
    $110 = $109 | $107;
    $111 = $110&65535;
    HEAP16[$flags>>1] = $111;
    break;
   }
  }
 } while(0);
 $112 = HEAP32[$nByte>>2]|0;
 $113 = HEAP32[$1>>2]|0;
 $114 = (($113) + 12|0);
 HEAP32[$114>>2] = $112;
 $115 = HEAP16[$flags>>1]|0;
 $116 = HEAP32[$1>>2]|0;
 $117 = (($116) + 8|0);
 HEAP16[$117>>1] = $115;
 $118 = HEAP8[$4>>0]|0;
 $119 = $118&255;
 $120 = ($119|0)==(0);
 if ($120) {
  $124 = 1;
 } else {
  $121 = HEAP8[$4>>0]|0;
  $122 = $121&255;
  $124 = $122;
 }
 $123 = $124&255;
 $125 = HEAP32[$1>>2]|0;
 $126 = (($125) + 10|0);
 HEAP8[$126>>0] = $123;
 $127 = HEAP32[$1>>2]|0;
 $128 = (($127) + 10|0);
 $129 = HEAP8[$128>>0]|0;
 $130 = $129&255;
 $131 = ($130|0)!=(1);
 if ($131) {
  $132 = HEAP32[$1>>2]|0;
  $133 = (_sqlite3VdbeMemHandleBom($132)|0);
  $134 = ($133|0)!=(0);
  if ($134) {
   HEAP32[$0>>2] = 7;
   $138 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($138|0);
  }
 }
 $135 = HEAP32[$nByte>>2]|0;
 $136 = HEAP32[$iLimit>>2]|0;
 $137 = ($135|0)>($136|0);
 if ($137) {
  HEAP32[$0>>2] = 18;
  $138 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($138|0);
 } else {
  HEAP32[$0>>2] = 0;
  $138 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($138|0);
 }
 return 0|0;
}
function _sqlite3_result_int($pCtx,$iVal) {
 $pCtx = $pCtx|0;
 $iVal = $iVal|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $1 = sp;
 HEAP32[$0>>2] = $pCtx;
 HEAP32[$1>>2] = $iVal;
 $2 = HEAP32[$0>>2]|0;
 $3 = HEAP32[$2>>2]|0;
 $4 = HEAP32[$1>>2]|0;
 $5 = ($4|0)<(0);
 $6 = $5 << 31 >> 31;
 _sqlite3VdbeMemSetInt64($3,$4,$6);
 STACKTOP = sp;return;
}
function _sqlite3VdbeMemSetInt64($pMem,$0,$1) {
 $pMem = $pMem|0;
 $0 = $0|0;
 $1 = $1|0;
 var $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0;
 var $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $2 = sp + 8|0;
 $3 = sp;
 HEAP32[$2>>2] = $pMem;
 $4 = $3;
 $5 = $4;
 HEAP32[$5>>2] = $0;
 $6 = (($4) + 4)|0;
 $7 = $6;
 HEAP32[$7>>2] = $1;
 $8 = HEAP32[$2>>2]|0;
 $9 = (($8) + 8|0);
 $10 = HEAP16[$9>>1]|0;
 $11 = $10&65535;
 $12 = $11 & 9312;
 $13 = ($12|0)!=(0);
 if ($13) {
  $14 = HEAP32[$2>>2]|0;
  $15 = $3;
  $16 = $15;
  $17 = HEAP32[$16>>2]|0;
  $18 = (($15) + 4)|0;
  $19 = $18;
  $20 = HEAP32[$19>>2]|0;
  _vdbeReleaseAndSetInt64($14,$17,$20);
  STACKTOP = sp;return;
 } else {
  $21 = $3;
  $22 = $21;
  $23 = HEAP32[$22>>2]|0;
  $24 = (($21) + 4)|0;
  $25 = $24;
  $26 = HEAP32[$25>>2]|0;
  $27 = HEAP32[$2>>2]|0;
  $28 = $27;
  $29 = $28;
  HEAP32[$29>>2] = $23;
  $30 = (($28) + 4)|0;
  $31 = $30;
  HEAP32[$31>>2] = $26;
  $32 = HEAP32[$2>>2]|0;
  $33 = (($32) + 8|0);
  HEAP16[$33>>1] = 4;
  STACKTOP = sp;return;
 }
}
function _sqlite3_result_int64($pCtx,$0,$1) {
 $pCtx = $pCtx|0;
 $0 = $0|0;
 $1 = $1|0;
 var $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $2 = sp + 8|0;
 $3 = sp;
 HEAP32[$2>>2] = $pCtx;
 $4 = $3;
 $5 = $4;
 HEAP32[$5>>2] = $0;
 $6 = (($4) + 4)|0;
 $7 = $6;
 HEAP32[$7>>2] = $1;
 $8 = HEAP32[$2>>2]|0;
 $9 = HEAP32[$8>>2]|0;
 $10 = $3;
 $11 = $10;
 $12 = HEAP32[$11>>2]|0;
 $13 = (($10) + 4)|0;
 $14 = $13;
 $15 = HEAP32[$14>>2]|0;
 _sqlite3VdbeMemSetInt64($9,$12,$15);
 STACKTOP = sp;return;
}
function _sqlite3_result_null($pCtx) {
 $pCtx = $pCtx|0;
 var $0 = 0, $1 = 0, $2 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 HEAP32[$0>>2] = $pCtx;
 $1 = HEAP32[$0>>2]|0;
 $2 = HEAP32[$1>>2]|0;
 _sqlite3VdbeMemSetNull($2);
 STACKTOP = sp;return;
}
function _sqlite3VdbeMemSetNull($pMem) {
 $pMem = $pMem|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 HEAP32[$0>>2] = $pMem;
 $1 = HEAP32[$0>>2]|0;
 $2 = (($1) + 8|0);
 $3 = HEAP16[$2>>1]|0;
 $4 = $3&65535;
 $5 = $4 & 9312;
 $6 = ($5|0)!=(0);
 $7 = HEAP32[$0>>2]|0;
 if ($6) {
  _vdbeMemClearExternAndSetNull($7);
  STACKTOP = sp;return;
 } else {
  $8 = (($7) + 8|0);
  HEAP16[$8>>1] = 1;
  STACKTOP = sp;return;
 }
}
function _sqlite3_result_text($pCtx,$z,$n,$xDel) {
 $pCtx = $pCtx|0;
 $z = $z|0;
 $n = $n|0;
 $xDel = $xDel|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $1 = sp + 8|0;
 $2 = sp + 4|0;
 $3 = sp;
 HEAP32[$0>>2] = $pCtx;
 HEAP32[$1>>2] = $z;
 HEAP32[$2>>2] = $n;
 HEAP32[$3>>2] = $xDel;
 $4 = HEAP32[$0>>2]|0;
 $5 = HEAP32[$1>>2]|0;
 $6 = HEAP32[$2>>2]|0;
 $7 = HEAP32[$3>>2]|0;
 _setResultStrOrError($4,$5,$6,1,$7);
 STACKTOP = sp;return;
}
function _sqlite3_result_text64($pCtx,$z,$0,$1,$xDel,$enc) {
 $pCtx = $pCtx|0;
 $z = $z|0;
 $0 = $0|0;
 $1 = $1|0;
 $xDel = $xDel|0;
 $enc = $enc|0;
 var $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0;
 var $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0;
 var $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $2 = sp + 16|0;
 $3 = sp + 12|0;
 $4 = sp;
 $5 = sp + 8|0;
 $6 = sp + 20|0;
 HEAP32[$2>>2] = $pCtx;
 HEAP32[$3>>2] = $z;
 $7 = $4;
 $8 = $7;
 HEAP32[$8>>2] = $0;
 $9 = (($7) + 4)|0;
 $10 = $9;
 HEAP32[$10>>2] = $1;
 HEAP32[$5>>2] = $xDel;
 HEAP8[$6>>0] = $enc;
 $11 = HEAP8[$6>>0]|0;
 $12 = $11&255;
 $13 = ($12|0)==(4);
 if ($13) {
  $14 = HEAP8[1136>>0]|0;
  $15 = $14 << 24 >> 24;
  $16 = ($15|0)==(0);
  $17 = $16 ? 3 : 2;
  $18 = $17&255;
  HEAP8[$6>>0] = $18;
 }
 $19 = $4;
 $20 = $19;
 $21 = HEAP32[$20>>2]|0;
 $22 = (($19) + 4)|0;
 $23 = $22;
 $24 = HEAP32[$23>>2]|0;
 $25 = ($24>>>0)>(0);
 $26 = ($21>>>0)>(2147483647);
 $27 = ($24|0)==(0);
 $28 = $27 & $26;
 $29 = $25 | $28;
 if ($29) {
  $30 = HEAP32[$3>>2]|0;
  $31 = HEAP32[$5>>2]|0;
  $32 = HEAP32[$2>>2]|0;
  (_invokeValueDestructor($30,$31,$32)|0);
  STACKTOP = sp;return;
 } else {
  $33 = HEAP32[$2>>2]|0;
  $34 = HEAP32[$3>>2]|0;
  $35 = $4;
  $36 = $35;
  $37 = HEAP32[$36>>2]|0;
  $38 = (($35) + 4)|0;
  $39 = $38;
  $40 = HEAP32[$39>>2]|0;
  $41 = HEAP8[$6>>0]|0;
  $42 = HEAP32[$5>>2]|0;
  _setResultStrOrError($33,$34,$37,$41,$42);
  STACKTOP = sp;return;
 }
}
function _sqlite3_result_value($pCtx,$pValue) {
 $pCtx = $pCtx|0;
 $pValue = $pValue|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $1 = sp;
 HEAP32[$0>>2] = $pCtx;
 HEAP32[$1>>2] = $pValue;
 $2 = HEAP32[$0>>2]|0;
 $3 = HEAP32[$2>>2]|0;
 $4 = HEAP32[$1>>2]|0;
 (_sqlite3VdbeMemCopy($3,$4)|0);
 STACKTOP = sp;return;
}
function _sqlite3VdbeMemCopy($pTo,$pFrom) {
 $pTo = $pTo|0;
 $pFrom = $pFrom|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $rc = 0, label = 0;
 var sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $1 = sp + 4|0;
 $rc = sp;
 HEAP32[$0>>2] = $pTo;
 HEAP32[$1>>2] = $pFrom;
 HEAP32[$rc>>2] = 0;
 $2 = HEAP32[$0>>2]|0;
 $3 = (($2) + 8|0);
 $4 = HEAP16[$3>>1]|0;
 $5 = $4&65535;
 $6 = $5 & 9312;
 $7 = ($6|0)!=(0);
 if ($7) {
  $8 = HEAP32[$0>>2]|0;
  _vdbeMemClearExternAndSetNull($8);
 }
 $9 = HEAP32[$0>>2]|0;
 $10 = HEAP32[$1>>2]|0;
 ;HEAP32[$9+0>>2]=HEAP32[$10+0>>2]|0;HEAP32[$9+4>>2]=HEAP32[$10+4>>2]|0;HEAP32[$9+8>>2]=HEAP32[$10+8>>2]|0;HEAP32[$9+12>>2]=HEAP32[$10+12>>2]|0;HEAP32[$9+16>>2]=HEAP32[$10+16>>2]|0;
 $11 = HEAP32[$0>>2]|0;
 $12 = (($11) + 8|0);
 $13 = HEAP16[$12>>1]|0;
 $14 = $13&65535;
 $15 = $14 & -1025;
 $16 = $15&65535;
 HEAP16[$12>>1] = $16;
 $17 = HEAP32[$0>>2]|0;
 $18 = (($17) + 8|0);
 $19 = HEAP16[$18>>1]|0;
 $20 = $19&65535;
 $21 = $20 & 18;
 $22 = ($21|0)!=(0);
 if (!($22)) {
  $37 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($37|0);
 }
 $23 = HEAP32[$1>>2]|0;
 $24 = (($23) + 8|0);
 $25 = HEAP16[$24>>1]|0;
 $26 = $25&65535;
 $27 = $26 & 2048;
 $28 = (0)==($27|0);
 if (!($28)) {
  $37 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($37|0);
 }
 $29 = HEAP32[$0>>2]|0;
 $30 = (($29) + 8|0);
 $31 = HEAP16[$30>>1]|0;
 $32 = $31&65535;
 $33 = $32 | 4096;
 $34 = $33&65535;
 HEAP16[$30>>1] = $34;
 $35 = HEAP32[$0>>2]|0;
 $36 = (_sqlite3VdbeMemMakeWriteable($35)|0);
 HEAP32[$rc>>2] = $36;
 $37 = HEAP32[$rc>>2]|0;
 STACKTOP = sp;return ($37|0);
}
function _sqlite3_result_zeroblob($pCtx,$n) {
 $pCtx = $pCtx|0;
 $n = $n|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $1 = sp;
 HEAP32[$0>>2] = $pCtx;
 HEAP32[$1>>2] = $n;
 $2 = HEAP32[$0>>2]|0;
 $3 = HEAP32[$2>>2]|0;
 $4 = HEAP32[$1>>2]|0;
 _sqlite3VdbeMemSetZeroBlob($3,$4);
 STACKTOP = sp;return;
}
function _sqlite3VdbeMemSetZeroBlob($pMem,$n) {
 $pMem = $pMem|0;
 $n = $n|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $1 = sp;
 HEAP32[$0>>2] = $pMem;
 HEAP32[$1>>2] = $n;
 $2 = HEAP32[$0>>2]|0;
 _sqlite3VdbeMemRelease($2);
 $3 = HEAP32[$0>>2]|0;
 $4 = (($3) + 8|0);
 HEAP16[$4>>1] = 16400;
 $5 = HEAP32[$0>>2]|0;
 $6 = (($5) + 12|0);
 HEAP32[$6>>2] = 0;
 $7 = HEAP32[$1>>2]|0;
 $8 = ($7|0)<(0);
 if ($8) {
  HEAP32[$1>>2] = 0;
 }
 $9 = HEAP32[$1>>2]|0;
 $10 = HEAP32[$0>>2]|0;
 HEAP32[$10>>2] = $9;
 $11 = HEAP32[$0>>2]|0;
 $12 = (($11) + 10|0);
 HEAP8[$12>>0] = 1;
 $13 = HEAP32[$0>>2]|0;
 $14 = (($13) + 16|0);
 HEAP32[$14>>2] = 0;
 STACKTOP = sp;return;
}
function _sqlite3_result_error_code($pCtx,$errCode) {
 $pCtx = $pCtx|0;
 $errCode = $errCode|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $1 = sp;
 HEAP32[$0>>2] = $pCtx;
 HEAP32[$1>>2] = $errCode;
 $2 = HEAP32[$1>>2]|0;
 $3 = HEAP32[$0>>2]|0;
 $4 = (($3) + 20|0);
 HEAP32[$4>>2] = $2;
 $5 = HEAP32[$0>>2]|0;
 $6 = (($5) + 25|0);
 HEAP8[$6>>0] = 1;
 $7 = HEAP32[$0>>2]|0;
 $8 = HEAP32[$7>>2]|0;
 $9 = (($8) + 8|0);
 $10 = HEAP16[$9>>1]|0;
 $11 = $10&65535;
 $12 = $11 & 1;
 $13 = ($12|0)!=(0);
 if (!($13)) {
  STACKTOP = sp;return;
 }
 $14 = HEAP32[$0>>2]|0;
 $15 = HEAP32[$14>>2]|0;
 $16 = HEAP32[$1>>2]|0;
 $17 = (_sqlite3ErrStr($16)|0);
 (_sqlite3VdbeMemSetStr($15,$17,-1,1,0)|0);
 STACKTOP = sp;return;
}
function _sqlite3ErrStr($rc) {
 $rc = $rc|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $cond = 0, $or$cond = 0, $zErr = 0, label = 0;
 var sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $zErr = sp;
 HEAP32[$0>>2] = $rc;
 HEAP32[$zErr>>2] = 5952;
 $1 = HEAP32[$0>>2]|0;
 $cond = ($1|0)==(516);
 if ($cond) {
  HEAP32[$zErr>>2] = 5968;
 } else {
  $2 = HEAP32[$0>>2]|0;
  $3 = $2 & 255;
  HEAP32[$0>>2] = $3;
  $4 = HEAP32[$0>>2]|0;
  $5 = ($4|0)>=(0);
  $6 = HEAP32[$0>>2]|0;
  $7 = ($6|0)<(27);
  $or$cond = $5 & $7;
  if ($or$cond) {
   $8 = HEAP32[$0>>2]|0;
   $9 = (5112 + ($8<<2)|0);
   $10 = HEAP32[$9>>2]|0;
   $11 = ($10|0)!=(0|0);
   if ($11) {
    $12 = HEAP32[$0>>2]|0;
    $13 = (5112 + ($12<<2)|0);
    $14 = HEAP32[$13>>2]|0;
    HEAP32[$zErr>>2] = $14;
   }
  }
 }
 $15 = HEAP32[$zErr>>2]|0;
 STACKTOP = sp;return ($15|0);
}
function _sqlite3_result_error_toobig($pCtx) {
 $pCtx = $pCtx|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 HEAP32[$0>>2] = $pCtx;
 $1 = HEAP32[$0>>2]|0;
 $2 = (($1) + 20|0);
 HEAP32[$2>>2] = 18;
 $3 = HEAP32[$0>>2]|0;
 $4 = (($3) + 25|0);
 HEAP8[$4>>0] = 1;
 $5 = HEAP32[$0>>2]|0;
 $6 = HEAP32[$5>>2]|0;
 (_sqlite3VdbeMemSetStr($6,1176,-1,1,0)|0);
 STACKTOP = sp;return;
}
function _sqlite3_result_error_nomem($pCtx) {
 $pCtx = $pCtx|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 HEAP32[$0>>2] = $pCtx;
 $1 = HEAP32[$0>>2]|0;
 $2 = HEAP32[$1>>2]|0;
 _sqlite3VdbeMemSetNull($2);
 $3 = HEAP32[$0>>2]|0;
 $4 = (($3) + 20|0);
 HEAP32[$4>>2] = 7;
 $5 = HEAP32[$0>>2]|0;
 $6 = (($5) + 25|0);
 HEAP8[$6>>0] = 1;
 $7 = HEAP32[$0>>2]|0;
 $8 = HEAP32[$7>>2]|0;
 $9 = (($8) + 32|0);
 $10 = HEAP32[$9>>2]|0;
 $11 = (($10) + 64|0);
 HEAP8[$11>>0] = 1;
 STACKTOP = sp;return;
}
function _sqlite3_step($pStmt) {
 $pStmt = $pStmt|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $7 = 0, $8 = 0, $9 = 0, $cnt = 0, $db = 0, $rc = 0, $rc2 = 0, $savedPc = 0, $v = 0, $zErr = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $0 = sp + 32|0;
 $1 = sp + 28|0;
 $rc = sp + 24|0;
 $rc2 = sp + 20|0;
 $v = sp + 16|0;
 $cnt = sp + 12|0;
 $db = sp + 8|0;
 $savedPc = sp + 4|0;
 $zErr = sp;
 HEAP32[$1>>2] = $pStmt;
 HEAP32[$rc>>2] = 0;
 HEAP32[$rc2>>2] = 0;
 $2 = HEAP32[$1>>2]|0;
 HEAP32[$v>>2] = $2;
 HEAP32[$cnt>>2] = 0;
 $3 = HEAP32[$v>>2]|0;
 $4 = (_vdbeSafetyNotNull($3)|0);
 $5 = ($4|0)!=(0);
 if ($5) {
  $6 = (_sqlite3MisuseError(67871)|0);
  HEAP32[$0>>2] = $6;
  $64 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($64|0);
 }
 $7 = HEAP32[$v>>2]|0;
 $8 = HEAP32[$7>>2]|0;
 HEAP32[$db>>2] = $8;
 $9 = HEAP32[$v>>2]|0;
 $10 = (($9) + 88|0);
 $11 = HEAP16[$10>>1]|0;
 $12 = $11 & -2049;
 HEAP16[$10>>1] = $12;
 while(1) {
  $13 = HEAP32[$v>>2]|0;
  $14 = (_sqlite3Step($13)|0);
  HEAP32[$rc>>2] = $14;
  $15 = ($14|0)==(17);
  if (!($15)) {
   break;
  }
  $16 = HEAP32[$cnt>>2]|0;
  $17 = (($16) + 1)|0;
  HEAP32[$cnt>>2] = $17;
  $18 = ($16|0)<(50);
  if (!($18)) {
   break;
  }
  $19 = HEAP32[$v>>2]|0;
  $20 = (($19) + 76|0);
  $21 = HEAP32[$20>>2]|0;
  HEAP32[$savedPc>>2] = $21;
  $22 = HEAP32[$v>>2]|0;
  $23 = (_sqlite3Reprepare($22)|0);
  HEAP32[$rc>>2] = $23;
  HEAP32[$rc2>>2] = $23;
  $24 = HEAP32[$rc>>2]|0;
  $25 = ($24|0)!=(0);
  if ($25) {
   break;
  }
  $26 = HEAP32[$1>>2]|0;
  (_sqlite3_reset($26)|0);
  $27 = HEAP32[$savedPc>>2]|0;
  $28 = ($27|0)>=(0);
  if (!($28)) {
   continue;
  }
  $29 = HEAP32[$v>>2]|0;
  $30 = (($29) + 88|0);
  $31 = HEAP16[$30>>1]|0;
  $32 = $31 & -2049;
  $33 = $32 | 2048;
  HEAP16[$30>>1] = $33;
 }
 $34 = HEAP32[$rc2>>2]|0;
 $35 = ($34|0)!=(0);
 do {
  if ($35) {
   $36 = HEAP32[$db>>2]|0;
   $37 = (($36) + 232|0);
   $38 = HEAP32[$37>>2]|0;
   $39 = (_sqlite3_value_text($38)|0);
   HEAP32[$zErr>>2] = $39;
   $40 = HEAP32[$db>>2]|0;
   $41 = HEAP32[$v>>2]|0;
   $42 = (($41) + 44|0);
   $43 = HEAP32[$42>>2]|0;
   _sqlite3DbFree($40,$43);
   $44 = HEAP32[$db>>2]|0;
   $45 = (($44) + 64|0);
   $46 = HEAP8[$45>>0]|0;
   $47 = ($46<<24>>24)!=(0);
   if ($47) {
    $56 = HEAP32[$v>>2]|0;
    $57 = (($56) + 44|0);
    HEAP32[$57>>2] = 0;
    HEAP32[$rc>>2] = 7;
    $58 = HEAP32[$v>>2]|0;
    $59 = (($58) + 80|0);
    HEAP32[$59>>2] = 7;
    break;
   } else {
    $48 = HEAP32[$db>>2]|0;
    $49 = HEAP32[$zErr>>2]|0;
    $50 = (_sqlite3DbStrDup($48,$49)|0);
    $51 = HEAP32[$v>>2]|0;
    $52 = (($51) + 44|0);
    HEAP32[$52>>2] = $50;
    $53 = HEAP32[$rc2>>2]|0;
    $54 = HEAP32[$v>>2]|0;
    $55 = (($54) + 80|0);
    HEAP32[$55>>2] = $53;
    break;
   }
  }
 } while(0);
 $60 = HEAP32[$db>>2]|0;
 $61 = HEAP32[$rc>>2]|0;
 $62 = (_sqlite3ApiExit($60,$61)|0);
 HEAP32[$rc>>2] = $62;
 $63 = HEAP32[$rc>>2]|0;
 HEAP32[$0>>2] = $63;
 $64 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($64|0);
}
function _vdbeSafetyNotNull($p) {
 $p = $p|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $vararg_buffer = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $vararg_buffer = sp;
 $0 = sp + 8|0;
 $1 = sp + 4|0;
 HEAP32[$1>>2] = $p;
 $2 = HEAP32[$1>>2]|0;
 $3 = ($2|0)==(0|0);
 if ($3) {
  _sqlite3_log(21,37016,$vararg_buffer);
  HEAP32[$0>>2] = 1;
  $6 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($6|0);
 } else {
  $4 = HEAP32[$1>>2]|0;
  $5 = (_vdbeSafety($4)|0);
  HEAP32[$0>>2] = $5;
  $6 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($6|0);
 }
 return 0|0;
}
function _sqlite3Step($p) {
 $p = $p|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0;
 var $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0;
 var $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0;
 var $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0;
 var $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0;
 var $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0;
 var $99 = 0, $db = 0, $iNow = 0, $or$cond = 0, $or$cond3 = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 20|0;
 $1 = sp + 16|0;
 $db = sp + 12|0;
 $rc = sp + 8|0;
 $iNow = sp;
 HEAP32[$1>>2] = $p;
 $2 = HEAP32[$1>>2]|0;
 $3 = (($2) + 40|0);
 $4 = HEAP32[$3>>2]|0;
 $5 = ($4|0)!=(-1108210269);
 if ($5) {
  $6 = HEAP32[$1>>2]|0;
  (_sqlite3_reset($6)|0);
 }
 $7 = HEAP32[$1>>2]|0;
 $8 = HEAP32[$7>>2]|0;
 HEAP32[$db>>2] = $8;
 $9 = HEAP32[$db>>2]|0;
 $10 = (($9) + 64|0);
 $11 = HEAP8[$10>>0]|0;
 $12 = ($11<<24>>24)!=(0);
 $13 = HEAP32[$1>>2]|0;
 if ($12) {
  $14 = (($13) + 80|0);
  HEAP32[$14>>2] = 7;
  HEAP32[$0>>2] = 7;
  $180 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($180|0);
 }
 $15 = (($13) + 76|0);
 $16 = HEAP32[$15>>2]|0;
 $17 = ($16|0)<=(0);
 if ($17) {
  $18 = HEAP32[$1>>2]|0;
  $19 = (($18) + 88|0);
  $20 = HEAP16[$19>>1]|0;
  $21 = ($20&65535) >>> 5;
  $22 = $21 & 1;
  $23 = $22&65535;
  $24 = ($23|0)!=(0);
  if ($24) {
   $25 = HEAP32[$1>>2]|0;
   $26 = (($25) + 80|0);
   HEAP32[$26>>2] = 17;
   HEAP32[$rc>>2] = 1;
  } else {
   label = 8;
  }
 } else {
  label = 8;
 }
 if ((label|0) == 8) {
  $27 = HEAP32[$1>>2]|0;
  $28 = (($27) + 76|0);
  $29 = HEAP32[$28>>2]|0;
  $30 = ($29|0)<(0);
  if ($30) {
   $31 = HEAP32[$db>>2]|0;
   $32 = (($31) + 148|0);
   $33 = HEAP32[$32>>2]|0;
   $34 = ($33|0)==(0);
   if ($34) {
    $35 = HEAP32[$db>>2]|0;
    $36 = (($35) + 240|0);
    HEAP32[$36>>2] = 0;
   }
   $37 = HEAP32[$db>>2]|0;
   $38 = (($37) + 180|0);
   $39 = HEAP32[$38>>2]|0;
   $40 = ($39|0)!=(0|0);
   if ($40) {
    $41 = HEAP32[$db>>2]|0;
    $42 = (($41) + 140|0);
    $43 = (($42) + 5|0);
    $44 = HEAP8[$43>>0]|0;
    $45 = ($44<<24>>24)!=(0);
    if (!($45)) {
     $46 = HEAP32[$db>>2]|0;
     $47 = HEAP32[$46>>2]|0;
     $48 = HEAP32[$1>>2]|0;
     $49 = (($48) + 128|0);
     (_sqlite3OsCurrentTimeInt64($47,$49)|0);
    }
   }
   $50 = HEAP32[$db>>2]|0;
   $51 = (($50) + 148|0);
   $52 = HEAP32[$51>>2]|0;
   $53 = (($52) + 1)|0;
   HEAP32[$51>>2] = $53;
   $54 = HEAP32[$1>>2]|0;
   $55 = (($54) + 88|0);
   $56 = HEAP16[$55>>1]|0;
   $57 = ($56&65535) >>> 8;
   $58 = $57 & 1;
   $59 = $58&65535;
   $60 = ($59|0)==(0);
   if ($60) {
    $61 = HEAP32[$db>>2]|0;
    $62 = (($61) + 156|0);
    $63 = HEAP32[$62>>2]|0;
    $64 = (($63) + 1)|0;
    HEAP32[$62>>2] = $64;
   }
   $65 = HEAP32[$1>>2]|0;
   $66 = (($65) + 88|0);
   $67 = HEAP16[$66>>1]|0;
   $68 = ($67&65535) >>> 9;
   $69 = $68 & 1;
   $70 = $69&65535;
   $71 = ($70|0)!=(0);
   if ($71) {
    $72 = HEAP32[$db>>2]|0;
    $73 = (($72) + 152|0);
    $74 = HEAP32[$73>>2]|0;
    $75 = (($74) + 1)|0;
    HEAP32[$73>>2] = $75;
   }
   $76 = HEAP32[$1>>2]|0;
   $77 = (($76) + 76|0);
   HEAP32[$77>>2] = 0;
  }
  $78 = HEAP32[$1>>2]|0;
  $79 = (($78) + 88|0);
  $80 = HEAP16[$79>>1]|0;
  $81 = $80 & 3;
  $82 = $81&65535;
  $83 = ($82|0)!=(0);
  if ($83) {
   $84 = HEAP32[$1>>2]|0;
   $85 = (_sqlite3VdbeList($84)|0);
   HEAP32[$rc>>2] = $85;
  } else {
   $86 = HEAP32[$db>>2]|0;
   $87 = (($86) + 160|0);
   $88 = HEAP32[$87>>2]|0;
   $89 = (($88) + 1)|0;
   HEAP32[$87>>2] = $89;
   $90 = HEAP32[$1>>2]|0;
   $91 = (_sqlite3VdbeExec($90)|0);
   HEAP32[$rc>>2] = $91;
   $92 = HEAP32[$db>>2]|0;
   $93 = (($92) + 160|0);
   $94 = HEAP32[$93>>2]|0;
   $95 = (($94) + -1)|0;
   HEAP32[$93>>2] = $95;
  }
  $96 = HEAP32[$rc>>2]|0;
  $97 = ($96|0)!=(100);
  if ($97) {
   $98 = HEAP32[$db>>2]|0;
   $99 = (($98) + 180|0);
   $100 = HEAP32[$99>>2]|0;
   $101 = ($100|0)!=(0|0);
   if ($101) {
    $102 = HEAP32[$db>>2]|0;
    $103 = (($102) + 140|0);
    $104 = (($103) + 5|0);
    $105 = HEAP8[$104>>0]|0;
    $106 = ($105<<24>>24)!=(0);
    if (!($106)) {
     $107 = HEAP32[$1>>2]|0;
     $108 = (($107) + 168|0);
     $109 = HEAP32[$108>>2]|0;
     $110 = ($109|0)!=(0|0);
     if ($110) {
      $111 = HEAP32[$db>>2]|0;
      $112 = HEAP32[$111>>2]|0;
      (_sqlite3OsCurrentTimeInt64($112,$iNow)|0);
      $113 = HEAP32[$db>>2]|0;
      $114 = (($113) + 180|0);
      $115 = HEAP32[$114>>2]|0;
      $116 = HEAP32[$db>>2]|0;
      $117 = (($116) + 184|0);
      $118 = HEAP32[$117>>2]|0;
      $119 = HEAP32[$1>>2]|0;
      $120 = (($119) + 168|0);
      $121 = HEAP32[$120>>2]|0;
      $122 = $iNow;
      $123 = $122;
      $124 = HEAP32[$123>>2]|0;
      $125 = (($122) + 4)|0;
      $126 = $125;
      $127 = HEAP32[$126>>2]|0;
      $128 = HEAP32[$1>>2]|0;
      $129 = (($128) + 128|0);
      $130 = $129;
      $131 = $130;
      $132 = HEAP32[$131>>2]|0;
      $133 = (($130) + 4)|0;
      $134 = $133;
      $135 = HEAP32[$134>>2]|0;
      $136 = (_i64Subtract(($124|0),($127|0),($132|0),($135|0))|0);
      $137 = tempRet0;
      $138 = (___muldi3(($136|0),($137|0),1000000,0)|0);
      $139 = tempRet0;
      FUNCTION_TABLE_viiii[$115 & 3]($118,$121,$138,$139);
     }
    }
   }
  }
  $140 = HEAP32[$rc>>2]|0;
  $141 = ($140|0)==(101);
  if ($141) {
   $142 = HEAP32[$db>>2]|0;
   $143 = (_doWalCallbacks($142)|0);
   $144 = HEAP32[$1>>2]|0;
   $145 = (($144) + 80|0);
   HEAP32[$145>>2] = $143;
   $146 = HEAP32[$1>>2]|0;
   $147 = (($146) + 80|0);
   $148 = HEAP32[$147>>2]|0;
   $149 = ($148|0)!=(0);
   if ($149) {
    HEAP32[$rc>>2] = 1;
   }
  }
  $150 = HEAP32[$rc>>2]|0;
  $151 = HEAP32[$db>>2]|0;
  $152 = (($151) + 52|0);
  HEAP32[$152>>2] = $150;
  $153 = HEAP32[$1>>2]|0;
  $154 = HEAP32[$153>>2]|0;
  $155 = HEAP32[$1>>2]|0;
  $156 = (($155) + 80|0);
  $157 = HEAP32[$156>>2]|0;
  $158 = (_sqlite3ApiExit($154,$157)|0);
  $159 = (7)==($158|0);
  if ($159) {
   $160 = HEAP32[$1>>2]|0;
   $161 = (($160) + 80|0);
   HEAP32[$161>>2] = 7;
  }
 }
 $162 = HEAP32[$1>>2]|0;
 $163 = (($162) + 88|0);
 $164 = HEAP16[$163>>1]|0;
 $165 = ($164&65535) >>> 10;
 $166 = $165 & 1;
 $167 = $166&65535;
 $168 = ($167|0)!=(0);
 $169 = HEAP32[$rc>>2]|0;
 $170 = ($169|0)!=(100);
 $or$cond = $168 & $170;
 $171 = HEAP32[$rc>>2]|0;
 $172 = ($171|0)!=(101);
 $or$cond3 = $or$cond & $172;
 if ($or$cond3) {
  $173 = HEAP32[$1>>2]|0;
  $174 = (_sqlite3VdbeTransferError($173)|0);
  HEAP32[$rc>>2] = $174;
 }
 $175 = HEAP32[$rc>>2]|0;
 $176 = HEAP32[$db>>2]|0;
 $177 = (($176) + 56|0);
 $178 = HEAP32[$177>>2]|0;
 $179 = $175 & $178;
 HEAP32[$0>>2] = $179;
 $180 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($180|0);
}
function _sqlite3Reprepare($p) {
 $p = $p|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $3 = 0, $4 = 0, $5 = 0;
 var $6 = 0, $7 = 0, $8 = 0, $9 = 0, $db = 0, $pNew = 0, $rc = 0, $zSql = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 20|0;
 $1 = sp + 16|0;
 $rc = sp + 12|0;
 $pNew = sp + 8|0;
 $zSql = sp + 4|0;
 $db = sp;
 HEAP32[$1>>2] = $p;
 $2 = HEAP32[$1>>2]|0;
 $3 = (_sqlite3_sql($2)|0);
 HEAP32[$zSql>>2] = $3;
 $4 = HEAP32[$1>>2]|0;
 $5 = (_sqlite3VdbeDb($4)|0);
 HEAP32[$db>>2] = $5;
 $6 = HEAP32[$db>>2]|0;
 $7 = HEAP32[$zSql>>2]|0;
 $8 = HEAP32[$1>>2]|0;
 $9 = (_sqlite3LockAndPrepare($6,$7,-1,0,$8,$pNew,0)|0);
 HEAP32[$rc>>2] = $9;
 $10 = HEAP32[$rc>>2]|0;
 $11 = ($10|0)!=(0);
 if (!($11)) {
  $17 = HEAP32[$pNew>>2]|0;
  $18 = HEAP32[$1>>2]|0;
  _sqlite3VdbeSwap($17,$18);
  $19 = HEAP32[$pNew>>2]|0;
  $20 = HEAP32[$1>>2]|0;
  (_sqlite3TransferBindings($19,$20)|0);
  $21 = HEAP32[$pNew>>2]|0;
  _sqlite3VdbeResetStepResult($21);
  $22 = HEAP32[$pNew>>2]|0;
  (_sqlite3VdbeFinalize($22)|0);
  HEAP32[$0>>2] = 0;
  $23 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($23|0);
 }
 $12 = HEAP32[$rc>>2]|0;
 $13 = ($12|0)==(7);
 if ($13) {
  $14 = HEAP32[$db>>2]|0;
  $15 = (($14) + 64|0);
  HEAP8[$15>>0] = 1;
 }
 $16 = HEAP32[$rc>>2]|0;
 HEAP32[$0>>2] = $16;
 $23 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($23|0);
}
function _sqlite3DbStrDup($db,$z) {
 $db = $db|0;
 $z = $z|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0;
 var $n = 0, $zNew = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 16|0;
 $1 = sp + 12|0;
 $2 = sp + 8|0;
 $zNew = sp + 4|0;
 $n = sp;
 HEAP32[$1>>2] = $db;
 HEAP32[$2>>2] = $z;
 $3 = HEAP32[$2>>2]|0;
 $4 = ($3|0)==(0|0);
 if ($4) {
  HEAP32[$0>>2] = 0;
  $19 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($19|0);
 }
 $5 = HEAP32[$2>>2]|0;
 $6 = (_sqlite3Strlen30($5)|0);
 $7 = (($6) + 1)|0;
 HEAP32[$n>>2] = $7;
 $8 = HEAP32[$1>>2]|0;
 $9 = HEAP32[$n>>2]|0;
 $10 = ($9|0)<(0);
 $11 = $10 << 31 >> 31;
 $12 = (_sqlite3DbMallocRaw($8,$9,$11)|0);
 HEAP32[$zNew>>2] = $12;
 $13 = HEAP32[$zNew>>2]|0;
 $14 = ($13|0)!=(0|0);
 if ($14) {
  $15 = HEAP32[$zNew>>2]|0;
  $16 = HEAP32[$2>>2]|0;
  $17 = HEAP32[$n>>2]|0;
  _memcpy(($15|0),($16|0),($17|0))|0;
 }
 $18 = HEAP32[$zNew>>2]|0;
 HEAP32[$0>>2] = $18;
 $19 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($19|0);
}
function _sqlite3_user_data($p) {
 $p = $p|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 HEAP32[$0>>2] = $p;
 $1 = HEAP32[$0>>2]|0;
 $2 = (($1) + 4|0);
 $3 = HEAP32[$2>>2]|0;
 $4 = (($3) + 4|0);
 $5 = HEAP32[$4>>2]|0;
 STACKTOP = sp;return ($5|0);
}
function _sqlite3_context_db_handle($p) {
 $p = $p|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 HEAP32[$0>>2] = $p;
 $1 = HEAP32[$0>>2]|0;
 $2 = HEAP32[$1>>2]|0;
 $3 = (($2) + 32|0);
 $4 = HEAP32[$3>>2]|0;
 STACKTOP = sp;return ($4|0);
}
function _sqlite3_aggregate_context($p,$nByte) {
 $p = $p|0;
 $nByte = $nByte|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0;
 var sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $1 = sp + 4|0;
 $2 = sp;
 HEAP32[$1>>2] = $p;
 HEAP32[$2>>2] = $nByte;
 $3 = HEAP32[$1>>2]|0;
 $4 = (($3) + 8|0);
 $5 = HEAP32[$4>>2]|0;
 $6 = (($5) + 8|0);
 $7 = HEAP16[$6>>1]|0;
 $8 = $7&65535;
 $9 = $8 & 8192;
 $10 = ($9|0)==(0);
 $11 = HEAP32[$1>>2]|0;
 if ($10) {
  $12 = HEAP32[$2>>2]|0;
  $13 = (_createAggContext($11,$12)|0);
  HEAP32[$0>>2] = $13;
  $18 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($18|0);
 } else {
  $14 = (($11) + 8|0);
  $15 = HEAP32[$14>>2]|0;
  $16 = (($15) + 16|0);
  $17 = HEAP32[$16>>2]|0;
  HEAP32[$0>>2] = $17;
  $18 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($18|0);
 }
 return 0|0;
}
function _createAggContext($p,$nByte) {
 $p = $p|0;
 $nByte = $nByte|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $pMem = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $1 = sp + 4|0;
 $pMem = sp;
 HEAP32[$0>>2] = $p;
 HEAP32[$1>>2] = $nByte;
 $2 = HEAP32[$0>>2]|0;
 $3 = (($2) + 8|0);
 $4 = HEAP32[$3>>2]|0;
 HEAP32[$pMem>>2] = $4;
 $5 = HEAP32[$1>>2]|0;
 $6 = ($5|0)<=(0);
 $7 = HEAP32[$pMem>>2]|0;
 if ($6) {
  _sqlite3VdbeMemSetNull($7);
  $8 = HEAP32[$pMem>>2]|0;
  $9 = (($8) + 16|0);
  HEAP32[$9>>2] = 0;
 } else {
  $10 = HEAP32[$1>>2]|0;
  (_sqlite3VdbeMemClearAndResize($7,$10)|0);
  $11 = HEAP32[$pMem>>2]|0;
  $12 = (($11) + 8|0);
  HEAP16[$12>>1] = 8192;
  $13 = HEAP32[$0>>2]|0;
  $14 = (($13) + 4|0);
  $15 = HEAP32[$14>>2]|0;
  $16 = HEAP32[$pMem>>2]|0;
  HEAP32[$16>>2] = $15;
  $17 = HEAP32[$pMem>>2]|0;
  $18 = (($17) + 16|0);
  $19 = HEAP32[$18>>2]|0;
  $20 = ($19|0)!=(0|0);
  if ($20) {
   $21 = HEAP32[$pMem>>2]|0;
   $22 = (($21) + 16|0);
   $23 = HEAP32[$22>>2]|0;
   $24 = HEAP32[$1>>2]|0;
   _memset(($23|0),0,($24|0))|0;
  }
 }
 $25 = HEAP32[$pMem>>2]|0;
 $26 = (($25) + 16|0);
 $27 = HEAP32[$26>>2]|0;
 STACKTOP = sp;return ($27|0);
}
function _sqlite3DbMallocZero($db,$0,$1) {
 $db = $db|0;
 $0 = $0|0;
 $1 = $1|0;
 var $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $3 = 0, $4 = 0, $5 = 0;
 var $6 = 0, $7 = 0, $8 = 0, $9 = 0, $p = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $2 = sp + 12|0;
 $3 = sp;
 $p = sp + 8|0;
 HEAP32[$2>>2] = $db;
 $4 = $3;
 $5 = $4;
 HEAP32[$5>>2] = $0;
 $6 = (($4) + 4)|0;
 $7 = $6;
 HEAP32[$7>>2] = $1;
 $8 = HEAP32[$2>>2]|0;
 $9 = $3;
 $10 = $9;
 $11 = HEAP32[$10>>2]|0;
 $12 = (($9) + 4)|0;
 $13 = $12;
 $14 = HEAP32[$13>>2]|0;
 $15 = (_sqlite3DbMallocRaw($8,$11,$14)|0);
 HEAP32[$p>>2] = $15;
 $16 = HEAP32[$p>>2]|0;
 $17 = ($16|0)!=(0|0);
 if (!($17)) {
  $25 = HEAP32[$p>>2]|0;
  STACKTOP = sp;return ($25|0);
 }
 $18 = HEAP32[$p>>2]|0;
 $19 = $3;
 $20 = $19;
 $21 = HEAP32[$20>>2]|0;
 $22 = (($19) + 4)|0;
 $23 = $22;
 $24 = HEAP32[$23>>2]|0;
 _memset(($18|0),0,($21|0))|0;
 $25 = HEAP32[$p>>2]|0;
 STACKTOP = sp;return ($25|0);
}
function _sqlite3_column_count($pStmt) {
 $pStmt = $pStmt|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $pVm = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $pVm = sp;
 HEAP32[$0>>2] = $pStmt;
 $1 = HEAP32[$0>>2]|0;
 HEAP32[$pVm>>2] = $1;
 $2 = HEAP32[$pVm>>2]|0;
 $3 = ($2|0)!=(0|0);
 if (!($3)) {
  $8 = 0;
  STACKTOP = sp;return ($8|0);
 }
 $4 = HEAP32[$pVm>>2]|0;
 $5 = (($4) + 84|0);
 $6 = HEAP16[$5>>1]|0;
 $7 = $6&65535;
 $8 = $7;
 STACKTOP = sp;return ($8|0);
}
function _sqlite3_data_count($pStmt) {
 $pStmt = $pStmt|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $pVm = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $1 = sp + 4|0;
 $pVm = sp;
 HEAP32[$1>>2] = $pStmt;
 $2 = HEAP32[$1>>2]|0;
 HEAP32[$pVm>>2] = $2;
 $3 = HEAP32[$pVm>>2]|0;
 $4 = ($3|0)==(0|0);
 if (!($4)) {
  $5 = HEAP32[$pVm>>2]|0;
  $6 = (($5) + 20|0);
  $7 = HEAP32[$6>>2]|0;
  $8 = ($7|0)==(0|0);
  if (!($8)) {
   $9 = HEAP32[$pVm>>2]|0;
   $10 = (($9) + 84|0);
   $11 = HEAP16[$10>>1]|0;
   $12 = $11&65535;
   HEAP32[$0>>2] = $12;
   $13 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($13|0);
  }
 }
 HEAP32[$0>>2] = 0;
 $13 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($13|0);
}
function _sqlite3_column_blob($pStmt,$i) {
 $pStmt = $pStmt|0;
 $i = $i|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $val = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $1 = sp + 4|0;
 $val = sp;
 HEAP32[$0>>2] = $pStmt;
 HEAP32[$1>>2] = $i;
 $2 = HEAP32[$0>>2]|0;
 $3 = HEAP32[$1>>2]|0;
 $4 = (_columnMem($2,$3)|0);
 $5 = (_sqlite3_value_blob($4)|0);
 HEAP32[$val>>2] = $5;
 $6 = HEAP32[$0>>2]|0;
 _columnMallocFailure($6);
 $7 = HEAP32[$val>>2]|0;
 STACKTOP = sp;return ($7|0);
}
function _columnMem($pStmt,$i) {
 $pStmt = $pStmt|0;
 $i = $i|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $or$cond = 0, $pOut = 0, $pVm = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $1 = sp + 8|0;
 $pVm = sp + 4|0;
 $pOut = sp;
 HEAP32[$0>>2] = $pStmt;
 HEAP32[$1>>2] = $i;
 $2 = HEAP32[$0>>2]|0;
 HEAP32[$pVm>>2] = $2;
 $3 = HEAP32[$pVm>>2]|0;
 $4 = ($3|0)!=(0|0);
 if ($4) {
  $5 = HEAP32[$pVm>>2]|0;
  $6 = (($5) + 20|0);
  $7 = HEAP32[$6>>2]|0;
  $8 = ($7|0)!=(0|0);
  if ($8) {
   $9 = HEAP32[$1>>2]|0;
   $10 = HEAP32[$pVm>>2]|0;
   $11 = (($10) + 84|0);
   $12 = HEAP16[$11>>1]|0;
   $13 = $12&65535;
   $14 = ($9|0)<($13|0);
   $15 = HEAP32[$1>>2]|0;
   $16 = ($15|0)>=(0);
   $or$cond = $14 & $16;
   if ($or$cond) {
    $17 = HEAP32[$1>>2]|0;
    $18 = HEAP32[$pVm>>2]|0;
    $19 = (($18) + 20|0);
    $20 = HEAP32[$19>>2]|0;
    $21 = (($20) + (($17*40)|0)|0);
    HEAP32[$pOut>>2] = $21;
    $30 = HEAP32[$pOut>>2]|0;
    STACKTOP = sp;return ($30|0);
   }
  }
 }
 $22 = HEAP32[$pVm>>2]|0;
 $23 = ($22|0)!=(0|0);
 if ($23) {
  $24 = HEAP32[$pVm>>2]|0;
  $25 = HEAP32[$24>>2]|0;
  $26 = ($25|0)!=(0|0);
  if ($26) {
   $27 = HEAP32[$pVm>>2]|0;
   $28 = HEAP32[$27>>2]|0;
   _sqlite3Error($28,25);
  }
 }
 $29 = (_columnNullValue()|0);
 HEAP32[$pOut>>2] = $29;
 $30 = HEAP32[$pOut>>2]|0;
 STACKTOP = sp;return ($30|0);
}
function _columnMallocFailure($pStmt) {
 $pStmt = $pStmt|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $p = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $p = sp;
 HEAP32[$0>>2] = $pStmt;
 $1 = HEAP32[$0>>2]|0;
 HEAP32[$p>>2] = $1;
 $2 = HEAP32[$p>>2]|0;
 $3 = ($2|0)!=(0|0);
 if (!($3)) {
  STACKTOP = sp;return;
 }
 $4 = HEAP32[$p>>2]|0;
 $5 = HEAP32[$4>>2]|0;
 $6 = HEAP32[$p>>2]|0;
 $7 = (($6) + 80|0);
 $8 = HEAP32[$7>>2]|0;
 $9 = (_sqlite3ApiExit($5,$8)|0);
 $10 = HEAP32[$p>>2]|0;
 $11 = (($10) + 80|0);
 HEAP32[$11>>2] = $9;
 STACKTOP = sp;return;
}
function _sqlite3_column_bytes($pStmt,$i) {
 $pStmt = $pStmt|0;
 $i = $i|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $val = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $1 = sp + 4|0;
 $val = sp;
 HEAP32[$0>>2] = $pStmt;
 HEAP32[$1>>2] = $i;
 $2 = HEAP32[$0>>2]|0;
 $3 = HEAP32[$1>>2]|0;
 $4 = (_columnMem($2,$3)|0);
 $5 = (_sqlite3_value_bytes($4)|0);
 HEAP32[$val>>2] = $5;
 $6 = HEAP32[$0>>2]|0;
 _columnMallocFailure($6);
 $7 = HEAP32[$val>>2]|0;
 STACKTOP = sp;return ($7|0);
}
function _sqlite3_column_double($pStmt,$i) {
 $pStmt = $pStmt|0;
 $i = $i|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0.0, $6 = 0, $7 = 0.0, $val = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $1 = sp + 8|0;
 $val = sp;
 HEAP32[$0>>2] = $pStmt;
 HEAP32[$1>>2] = $i;
 $2 = HEAP32[$0>>2]|0;
 $3 = HEAP32[$1>>2]|0;
 $4 = (_columnMem($2,$3)|0);
 $5 = (+_sqlite3_value_double($4));
 HEAPF64[$val>>3] = $5;
 $6 = HEAP32[$0>>2]|0;
 _columnMallocFailure($6);
 $7 = +HEAPF64[$val>>3];
 STACKTOP = sp;return (+$7);
}
function _sqlite3_column_text($pStmt,$i) {
 $pStmt = $pStmt|0;
 $i = $i|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $val = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $1 = sp + 4|0;
 $val = sp;
 HEAP32[$0>>2] = $pStmt;
 HEAP32[$1>>2] = $i;
 $2 = HEAP32[$0>>2]|0;
 $3 = HEAP32[$1>>2]|0;
 $4 = (_columnMem($2,$3)|0);
 $5 = (_sqlite3_value_text($4)|0);
 HEAP32[$val>>2] = $5;
 $6 = HEAP32[$0>>2]|0;
 _columnMallocFailure($6);
 $7 = HEAP32[$val>>2]|0;
 STACKTOP = sp;return ($7|0);
}
function _sqlite3_column_type($pStmt,$i) {
 $pStmt = $pStmt|0;
 $i = $i|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $iType = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $1 = sp + 4|0;
 $iType = sp;
 HEAP32[$0>>2] = $pStmt;
 HEAP32[$1>>2] = $i;
 $2 = HEAP32[$0>>2]|0;
 $3 = HEAP32[$1>>2]|0;
 $4 = (_columnMem($2,$3)|0);
 $5 = (_sqlite3_value_type($4)|0);
 HEAP32[$iType>>2] = $5;
 $6 = HEAP32[$0>>2]|0;
 _columnMallocFailure($6);
 $7 = HEAP32[$iType>>2]|0;
 STACKTOP = sp;return ($7|0);
}
function _sqlite3_column_name($pStmt,$N) {
 $pStmt = $pStmt|0;
 $N = $N|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $1 = sp;
 HEAP32[$0>>2] = $pStmt;
 HEAP32[$1>>2] = $N;
 $2 = HEAP32[$0>>2]|0;
 $3 = HEAP32[$1>>2]|0;
 $4 = (_columnName($2,$3,17,0)|0);
 STACKTOP = sp;return ($4|0);
}
function _columnName($pStmt,$N,$xFunc,$useType) {
 $pStmt = $pStmt|0;
 $N = $N|0;
 $xFunc = $xFunc|0;
 $useType = $useType|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $db = 0, $n = 0, $or$cond = 0, $p = 0, $ret = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 28|0;
 $1 = sp + 24|0;
 $2 = sp + 20|0;
 $3 = sp + 16|0;
 $ret = sp + 12|0;
 $p = sp + 8|0;
 $n = sp + 4|0;
 $db = sp;
 HEAP32[$0>>2] = $pStmt;
 HEAP32[$1>>2] = $N;
 HEAP32[$2>>2] = $xFunc;
 HEAP32[$3>>2] = $useType;
 HEAP32[$ret>>2] = 0;
 $4 = HEAP32[$0>>2]|0;
 HEAP32[$p>>2] = $4;
 $5 = HEAP32[$p>>2]|0;
 $6 = HEAP32[$5>>2]|0;
 HEAP32[$db>>2] = $6;
 $7 = HEAP32[$0>>2]|0;
 $8 = (_sqlite3_column_count($7)|0);
 HEAP32[$n>>2] = $8;
 $9 = HEAP32[$1>>2]|0;
 $10 = HEAP32[$n>>2]|0;
 $11 = ($9|0)<($10|0);
 $12 = HEAP32[$1>>2]|0;
 $13 = ($12|0)>=(0);
 $or$cond = $11 & $13;
 if (!($or$cond)) {
  $32 = HEAP32[$ret>>2]|0;
  STACKTOP = sp;return ($32|0);
 }
 $14 = HEAP32[$3>>2]|0;
 $15 = HEAP32[$n>>2]|0;
 $16 = Math_imul($14, $15)|0;
 $17 = HEAP32[$1>>2]|0;
 $18 = (($17) + ($16))|0;
 HEAP32[$1>>2] = $18;
 $19 = HEAP32[$2>>2]|0;
 $20 = HEAP32[$1>>2]|0;
 $21 = HEAP32[$p>>2]|0;
 $22 = (($21) + 16|0);
 $23 = HEAP32[$22>>2]|0;
 $24 = (($23) + (($20*40)|0)|0);
 $25 = (FUNCTION_TABLE_ii[$19 & 31]($24)|0);
 HEAP32[$ret>>2] = $25;
 $26 = HEAP32[$db>>2]|0;
 $27 = (($26) + 64|0);
 $28 = HEAP8[$27>>0]|0;
 $29 = ($28<<24>>24)!=(0);
 if (!($29)) {
  $32 = HEAP32[$ret>>2]|0;
  STACKTOP = sp;return ($32|0);
 }
 $30 = HEAP32[$db>>2]|0;
 $31 = (($30) + 64|0);
 HEAP8[$31>>0] = 0;
 HEAP32[$ret>>2] = 0;
 $32 = HEAP32[$ret>>2]|0;
 STACKTOP = sp;return ($32|0);
}
function _sqlite3_bind_blob($pStmt,$i,$zData,$nData,$xDel) {
 $pStmt = $pStmt|0;
 $i = $i|0;
 $zData = $zData|0;
 $nData = $nData|0;
 $xDel = $xDel|0;
 var $0 = 0, $1 = 0, $10 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 16|0;
 $1 = sp + 12|0;
 $2 = sp + 8|0;
 $3 = sp + 4|0;
 $4 = sp;
 HEAP32[$0>>2] = $pStmt;
 HEAP32[$1>>2] = $i;
 HEAP32[$2>>2] = $zData;
 HEAP32[$3>>2] = $nData;
 HEAP32[$4>>2] = $xDel;
 $5 = HEAP32[$0>>2]|0;
 $6 = HEAP32[$1>>2]|0;
 $7 = HEAP32[$2>>2]|0;
 $8 = HEAP32[$3>>2]|0;
 $9 = HEAP32[$4>>2]|0;
 $10 = (_bindText($5,$6,$7,$8,$9,0)|0);
 STACKTOP = sp;return ($10|0);
}
function _bindText($pStmt,$i,$zData,$nData,$xDel,$encoding) {
 $pStmt = $pStmt|0;
 $i = $i|0;
 $zData = $zData|0;
 $nData = $nData|0;
 $xDel = $xDel|0;
 $encoding = $encoding|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $or$cond = 0, $p = 0, $pVar = 0, $rc = 0;
 var label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $0 = sp + 28|0;
 $1 = sp + 24|0;
 $2 = sp + 20|0;
 $3 = sp + 16|0;
 $4 = sp + 12|0;
 $5 = sp + 32|0;
 $p = sp + 8|0;
 $pVar = sp + 4|0;
 $rc = sp;
 HEAP32[$0>>2] = $pStmt;
 HEAP32[$1>>2] = $i;
 HEAP32[$2>>2] = $zData;
 HEAP32[$3>>2] = $nData;
 HEAP32[$4>>2] = $xDel;
 HEAP8[$5>>0] = $encoding;
 $6 = HEAP32[$0>>2]|0;
 HEAP32[$p>>2] = $6;
 $7 = HEAP32[$p>>2]|0;
 $8 = HEAP32[$1>>2]|0;
 $9 = (_vdbeUnbind($7,$8)|0);
 HEAP32[$rc>>2] = $9;
 $10 = HEAP32[$rc>>2]|0;
 $11 = ($10|0)==(0);
 if (!($11)) {
  $49 = HEAP32[$4>>2]|0;
  $50 = ($49|0)!=(0|0);
  $51 = HEAP32[$4>>2]|0;
  $52 = ($51|0)!=((-1)|0);
  $or$cond = $50 & $52;
  if (!($or$cond)) {
   $55 = HEAP32[$rc>>2]|0;
   STACKTOP = sp;return ($55|0);
  }
  $53 = HEAP32[$4>>2]|0;
  $54 = HEAP32[$2>>2]|0;
  FUNCTION_TABLE_vi[$53 & 31]($54);
  $55 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($55|0);
 }
 $12 = HEAP32[$2>>2]|0;
 $13 = ($12|0)!=(0|0);
 if (!($13)) {
  $55 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($55|0);
 }
 $14 = HEAP32[$1>>2]|0;
 $15 = (($14) - 1)|0;
 $16 = HEAP32[$p>>2]|0;
 $17 = (($16) + 60|0);
 $18 = HEAP32[$17>>2]|0;
 $19 = (($18) + (($15*40)|0)|0);
 HEAP32[$pVar>>2] = $19;
 $20 = HEAP32[$pVar>>2]|0;
 $21 = HEAP32[$2>>2]|0;
 $22 = HEAP32[$3>>2]|0;
 $23 = HEAP8[$5>>0]|0;
 $24 = HEAP32[$4>>2]|0;
 $25 = (_sqlite3VdbeMemSetStr($20,$21,$22,$23,$24)|0);
 HEAP32[$rc>>2] = $25;
 $26 = HEAP32[$rc>>2]|0;
 $27 = ($26|0)==(0);
 if ($27) {
  $28 = HEAP8[$5>>0]|0;
  $29 = $28&255;
  $30 = ($29|0)!=(0);
  if ($30) {
   $31 = HEAP32[$pVar>>2]|0;
   $32 = HEAP32[$p>>2]|0;
   $33 = HEAP32[$32>>2]|0;
   $34 = (($33) + 16|0);
   $35 = HEAP32[$34>>2]|0;
   $36 = (($35) + 12|0);
   $37 = HEAP32[$36>>2]|0;
   $38 = (($37) + 77|0);
   $39 = HEAP8[$38>>0]|0;
   $40 = $39&255;
   $41 = (_sqlite3VdbeChangeEncoding($31,$40)|0);
   HEAP32[$rc>>2] = $41;
  }
 }
 $42 = HEAP32[$p>>2]|0;
 $43 = HEAP32[$42>>2]|0;
 $44 = HEAP32[$rc>>2]|0;
 _sqlite3Error($43,$44);
 $45 = HEAP32[$p>>2]|0;
 $46 = HEAP32[$45>>2]|0;
 $47 = HEAP32[$rc>>2]|0;
 $48 = (_sqlite3ApiExit($46,$47)|0);
 HEAP32[$rc>>2] = $48;
 $55 = HEAP32[$rc>>2]|0;
 STACKTOP = sp;return ($55|0);
}
function _sqlite3_bind_double($pStmt,$i,$rValue) {
 $pStmt = $pStmt|0;
 $i = $i|0;
 $rValue = +$rValue;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0.0, $16 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $p = 0, $rc = 0, label = 0;
 var sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 20|0;
 $1 = sp + 16|0;
 $2 = sp;
 $rc = sp + 12|0;
 $p = sp + 8|0;
 HEAP32[$0>>2] = $pStmt;
 HEAP32[$1>>2] = $i;
 HEAPF64[$2>>3] = $rValue;
 $3 = HEAP32[$0>>2]|0;
 HEAP32[$p>>2] = $3;
 $4 = HEAP32[$p>>2]|0;
 $5 = HEAP32[$1>>2]|0;
 $6 = (_vdbeUnbind($4,$5)|0);
 HEAP32[$rc>>2] = $6;
 $7 = HEAP32[$rc>>2]|0;
 $8 = ($7|0)==(0);
 if (!($8)) {
  $16 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($16|0);
 }
 $9 = HEAP32[$1>>2]|0;
 $10 = (($9) - 1)|0;
 $11 = HEAP32[$p>>2]|0;
 $12 = (($11) + 60|0);
 $13 = HEAP32[$12>>2]|0;
 $14 = (($13) + (($10*40)|0)|0);
 $15 = +HEAPF64[$2>>3];
 _sqlite3VdbeMemSetDouble($14,$15);
 $16 = HEAP32[$rc>>2]|0;
 STACKTOP = sp;return ($16|0);
}
function _vdbeUnbind($p,$i) {
 $p = $p|0;
 $i = $i|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $7 = 0, $8 = 0, $9 = 0, $pVar = 0, $vararg_buffer = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $vararg_buffer = sp;
 $0 = sp + 16|0;
 $1 = sp + 12|0;
 $2 = sp + 8|0;
 $pVar = sp + 4|0;
 HEAP32[$1>>2] = $p;
 HEAP32[$2>>2] = $i;
 $3 = HEAP32[$1>>2]|0;
 $4 = (_vdbeSafetyNotNull($3)|0);
 $5 = ($4|0)!=(0);
 if ($5) {
  $6 = (_sqlite3MisuseError(68415)|0);
  HEAP32[$0>>2] = $6;
  $68 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($68|0);
 }
 $7 = HEAP32[$1>>2]|0;
 $8 = (($7) + 40|0);
 $9 = HEAP32[$8>>2]|0;
 $10 = ($9|0)!=(-1108210269);
 if (!($10)) {
  $11 = HEAP32[$1>>2]|0;
  $12 = (($11) + 76|0);
  $13 = HEAP32[$12>>2]|0;
  $14 = ($13|0)>=(0);
  if (!($14)) {
   $21 = HEAP32[$2>>2]|0;
   $22 = ($21|0)<(1);
   if (!($22)) {
    $23 = HEAP32[$2>>2]|0;
    $24 = HEAP32[$1>>2]|0;
    $25 = (($24) + 68|0);
    $26 = HEAP16[$25>>1]|0;
    $27 = $26 << 16 >> 16;
    $28 = ($23|0)>($27|0);
    if (!($28)) {
     $31 = HEAP32[$2>>2]|0;
     $32 = (($31) + -1)|0;
     HEAP32[$2>>2] = $32;
     $33 = HEAP32[$2>>2]|0;
     $34 = HEAP32[$1>>2]|0;
     $35 = (($34) + 60|0);
     $36 = HEAP32[$35>>2]|0;
     $37 = (($36) + (($33*40)|0)|0);
     HEAP32[$pVar>>2] = $37;
     $38 = HEAP32[$pVar>>2]|0;
     _sqlite3VdbeMemRelease($38);
     $39 = HEAP32[$pVar>>2]|0;
     $40 = (($39) + 8|0);
     HEAP16[$40>>1] = 1;
     $41 = HEAP32[$1>>2]|0;
     $42 = HEAP32[$41>>2]|0;
     _sqlite3Error($42,0);
     $43 = HEAP32[$1>>2]|0;
     $44 = (($43) + 88|0);
     $45 = HEAP16[$44>>1]|0;
     $46 = ($45&65535) >>> 10;
     $47 = $46 & 1;
     $48 = $47&65535;
     $49 = ($48|0)!=(0);
     do {
      if ($49) {
       $50 = HEAP32[$2>>2]|0;
       $51 = ($50|0)<(32);
       if ($51) {
        $52 = HEAP32[$1>>2]|0;
        $53 = (($52) + 188|0);
        $54 = HEAP32[$53>>2]|0;
        $55 = HEAP32[$2>>2]|0;
        $56 = 1 << $55;
        $57 = $54 & $56;
        $58 = ($57|0)!=(0);
        if (!($58)) {
         label = 12;
        }
       } else {
        label = 12;
       }
       if ((label|0) == 12) {
        $59 = HEAP32[$1>>2]|0;
        $60 = (($59) + 188|0);
        $61 = HEAP32[$60>>2]|0;
        $62 = ($61|0)==(-1);
        if (!($62)) {
         break;
        }
       }
       $63 = HEAP32[$1>>2]|0;
       $64 = (($63) + 88|0);
       $65 = HEAP16[$64>>1]|0;
       $66 = $65 & -33;
       $67 = $66 | 32;
       HEAP16[$64>>1] = $67;
      }
     } while(0);
     HEAP32[$0>>2] = 0;
     $68 = HEAP32[$0>>2]|0;
     STACKTOP = sp;return ($68|0);
    }
   }
   $29 = HEAP32[$1>>2]|0;
   $30 = HEAP32[$29>>2]|0;
   _sqlite3Error($30,25);
   HEAP32[$0>>2] = 25;
   $68 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($68|0);
  }
 }
 $15 = HEAP32[$1>>2]|0;
 $16 = HEAP32[$15>>2]|0;
 _sqlite3Error($16,21);
 $17 = HEAP32[$1>>2]|0;
 $18 = (($17) + 168|0);
 $19 = HEAP32[$18>>2]|0;
 HEAP32[$vararg_buffer>>2] = $19;
 _sqlite3_log(21,31184,$vararg_buffer);
 $20 = (_sqlite3MisuseError(68423)|0);
 HEAP32[$0>>2] = $20;
 $68 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($68|0);
}
function _sqlite3_bind_int($p,$i,$iValue) {
 $p = $p|0;
 $i = $i|0;
 $iValue = $iValue|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $1 = sp + 4|0;
 $2 = sp;
 HEAP32[$0>>2] = $p;
 HEAP32[$1>>2] = $i;
 HEAP32[$2>>2] = $iValue;
 $3 = HEAP32[$0>>2]|0;
 $4 = HEAP32[$1>>2]|0;
 $5 = HEAP32[$2>>2]|0;
 $6 = ($5|0)<(0);
 $7 = $6 << 31 >> 31;
 $8 = (_sqlite3_bind_int64($3,$4,$5,$7)|0);
 STACKTOP = sp;return ($8|0);
}
function _sqlite3_bind_int64($pStmt,$i,$0,$1) {
 $pStmt = $pStmt|0;
 $i = $i|0;
 $0 = $0|0;
 $1 = $1|0;
 var $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $3 = 0;
 var $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $p = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $2 = sp + 20|0;
 $3 = sp + 16|0;
 $4 = sp;
 $rc = sp + 12|0;
 $p = sp + 8|0;
 HEAP32[$2>>2] = $pStmt;
 HEAP32[$3>>2] = $i;
 $5 = $4;
 $6 = $5;
 HEAP32[$6>>2] = $0;
 $7 = (($5) + 4)|0;
 $8 = $7;
 HEAP32[$8>>2] = $1;
 $9 = HEAP32[$2>>2]|0;
 HEAP32[$p>>2] = $9;
 $10 = HEAP32[$p>>2]|0;
 $11 = HEAP32[$3>>2]|0;
 $12 = (_vdbeUnbind($10,$11)|0);
 HEAP32[$rc>>2] = $12;
 $13 = HEAP32[$rc>>2]|0;
 $14 = ($13|0)==(0);
 if (!($14)) {
  $27 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($27|0);
 }
 $15 = HEAP32[$3>>2]|0;
 $16 = (($15) - 1)|0;
 $17 = HEAP32[$p>>2]|0;
 $18 = (($17) + 60|0);
 $19 = HEAP32[$18>>2]|0;
 $20 = (($19) + (($16*40)|0)|0);
 $21 = $4;
 $22 = $21;
 $23 = HEAP32[$22>>2]|0;
 $24 = (($21) + 4)|0;
 $25 = $24;
 $26 = HEAP32[$25>>2]|0;
 _sqlite3VdbeMemSetInt64($20,$23,$26);
 $27 = HEAP32[$rc>>2]|0;
 STACKTOP = sp;return ($27|0);
}
function _sqlite3_bind_text($pStmt,$i,$zData,$nData,$xDel) {
 $pStmt = $pStmt|0;
 $i = $i|0;
 $zData = $zData|0;
 $nData = $nData|0;
 $xDel = $xDel|0;
 var $0 = 0, $1 = 0, $10 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 16|0;
 $1 = sp + 12|0;
 $2 = sp + 8|0;
 $3 = sp + 4|0;
 $4 = sp;
 HEAP32[$0>>2] = $pStmt;
 HEAP32[$1>>2] = $i;
 HEAP32[$2>>2] = $zData;
 HEAP32[$3>>2] = $nData;
 HEAP32[$4>>2] = $xDel;
 $5 = HEAP32[$0>>2]|0;
 $6 = HEAP32[$1>>2]|0;
 $7 = HEAP32[$2>>2]|0;
 $8 = HEAP32[$3>>2]|0;
 $9 = HEAP32[$4>>2]|0;
 $10 = (_bindText($5,$6,$7,$8,$9,1)|0);
 STACKTOP = sp;return ($10|0);
}
function _sqlite3_bind_parameter_index($pStmt,$zName) {
 $pStmt = $pStmt|0;
 $zName = $zName|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $1 = sp;
 HEAP32[$0>>2] = $pStmt;
 HEAP32[$1>>2] = $zName;
 $2 = HEAP32[$0>>2]|0;
 $3 = HEAP32[$1>>2]|0;
 $4 = HEAP32[$1>>2]|0;
 $5 = (_sqlite3Strlen30($4)|0);
 $6 = (_sqlite3VdbeParameterIndex($2,$3,$5)|0);
 STACKTOP = sp;return ($6|0);
}
function _sqlite3VdbeParameterIndex($p,$zName,$nName) {
 $p = $p|0;
 $zName = $zName|0;
 $nName = $nName|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $i = 0, $z = 0;
 var label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 20|0;
 $1 = sp + 16|0;
 $2 = sp + 12|0;
 $3 = sp + 8|0;
 $i = sp + 4|0;
 $z = sp;
 HEAP32[$1>>2] = $p;
 HEAP32[$2>>2] = $zName;
 HEAP32[$3>>2] = $nName;
 $4 = HEAP32[$1>>2]|0;
 $5 = ($4|0)==(0|0);
 if ($5) {
  HEAP32[$0>>2] = 0;
  $37 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($37|0);
 }
 $6 = HEAP32[$2>>2]|0;
 $7 = ($6|0)!=(0|0);
 L5: do {
  if ($7) {
   HEAP32[$i>>2] = 0;
   while(1) {
    $8 = HEAP32[$i>>2]|0;
    $9 = HEAP32[$1>>2]|0;
    $10 = (($9) + 70|0);
    $11 = HEAP16[$10>>1]|0;
    $12 = $11 << 16 >> 16;
    $13 = ($8|0)<($12|0);
    if (!($13)) {
     break L5;
    }
    $14 = HEAP32[$i>>2]|0;
    $15 = HEAP32[$1>>2]|0;
    $16 = (($15) + 64|0);
    $17 = HEAP32[$16>>2]|0;
    $18 = (($17) + ($14<<2)|0);
    $19 = HEAP32[$18>>2]|0;
    HEAP32[$z>>2] = $19;
    $20 = HEAP32[$z>>2]|0;
    $21 = ($20|0)!=(0|0);
    if ($21) {
     $22 = HEAP32[$z>>2]|0;
     $23 = HEAP32[$2>>2]|0;
     $24 = HEAP32[$3>>2]|0;
     $25 = (_strncmp($22,$23,$24)|0);
     $26 = ($25|0)==(0);
     if ($26) {
      $27 = HEAP32[$3>>2]|0;
      $28 = HEAP32[$z>>2]|0;
      $29 = (($28) + ($27)|0);
      $30 = HEAP8[$29>>0]|0;
      $31 = $30 << 24 >> 24;
      $32 = ($31|0)==(0);
      if ($32) {
       break;
      }
     }
    }
    $35 = HEAP32[$i>>2]|0;
    $36 = (($35) + 1)|0;
    HEAP32[$i>>2] = $36;
   }
   $33 = HEAP32[$i>>2]|0;
   $34 = (($33) + 1)|0;
   HEAP32[$0>>2] = $34;
   $37 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($37|0);
  }
 } while(0);
 HEAP32[$0>>2] = 0;
 $37 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($37|0);
}
function _sqlite3TransferBindings($pFromStmt,$pToStmt) {
 $pFromStmt = $pFromStmt|0;
 $pToStmt = $pToStmt|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0;
 var $8 = 0, $9 = 0, $i = 0, $pFrom = 0, $pTo = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 16|0;
 $1 = sp + 12|0;
 $pFrom = sp + 8|0;
 $pTo = sp + 4|0;
 $i = sp;
 HEAP32[$0>>2] = $pFromStmt;
 HEAP32[$1>>2] = $pToStmt;
 $2 = HEAP32[$0>>2]|0;
 HEAP32[$pFrom>>2] = $2;
 $3 = HEAP32[$1>>2]|0;
 HEAP32[$pTo>>2] = $3;
 HEAP32[$i>>2] = 0;
 while(1) {
  $4 = HEAP32[$i>>2]|0;
  $5 = HEAP32[$pFrom>>2]|0;
  $6 = (($5) + 68|0);
  $7 = HEAP16[$6>>1]|0;
  $8 = $7 << 16 >> 16;
  $9 = ($4|0)<($8|0);
  if (!($9)) {
   break;
  }
  $10 = HEAP32[$i>>2]|0;
  $11 = HEAP32[$pTo>>2]|0;
  $12 = (($11) + 60|0);
  $13 = HEAP32[$12>>2]|0;
  $14 = (($13) + (($10*40)|0)|0);
  $15 = HEAP32[$i>>2]|0;
  $16 = HEAP32[$pFrom>>2]|0;
  $17 = (($16) + 60|0);
  $18 = HEAP32[$17>>2]|0;
  $19 = (($18) + (($15*40)|0)|0);
  _sqlite3VdbeMemMove($14,$19);
  $20 = HEAP32[$i>>2]|0;
  $21 = (($20) + 1)|0;
  HEAP32[$i>>2] = $21;
 }
 STACKTOP = sp;return 0;
}
function _sqlite3_value_numeric_type($pVal) {
 $pVal = $pVal|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $eType = 0, $pMem = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $eType = sp + 4|0;
 $pMem = sp;
 HEAP32[$0>>2] = $pVal;
 $1 = HEAP32[$0>>2]|0;
 $2 = (_sqlite3_value_type($1)|0);
 HEAP32[$eType>>2] = $2;
 $3 = HEAP32[$eType>>2]|0;
 $4 = ($3|0)==(3);
 if (!($4)) {
  $9 = HEAP32[$eType>>2]|0;
  STACKTOP = sp;return ($9|0);
 }
 $5 = HEAP32[$0>>2]|0;
 HEAP32[$pMem>>2] = $5;
 $6 = HEAP32[$pMem>>2]|0;
 _applyNumericAffinity($6,0);
 $7 = HEAP32[$0>>2]|0;
 $8 = (_sqlite3_value_type($7)|0);
 HEAP32[$eType>>2] = $8;
 $9 = HEAP32[$eType>>2]|0;
 STACKTOP = sp;return ($9|0);
}
function _applyNumericAffinity($pRec,$bTryForInt) {
 $pRec = $pRec|0;
 $bTryForInt = $bTryForInt|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0.0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $enc = 0, $iValue = 0, $rValue = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 20|0;
 $1 = sp + 16|0;
 $rValue = sp + 8|0;
 $iValue = sp;
 $enc = sp + 24|0;
 HEAP32[$0>>2] = $pRec;
 HEAP32[$1>>2] = $bTryForInt;
 $2 = HEAP32[$0>>2]|0;
 $3 = (($2) + 10|0);
 $4 = HEAP8[$3>>0]|0;
 HEAP8[$enc>>0] = $4;
 $5 = HEAP32[$0>>2]|0;
 $6 = (($5) + 16|0);
 $7 = HEAP32[$6>>2]|0;
 $8 = HEAP32[$0>>2]|0;
 $9 = (($8) + 12|0);
 $10 = HEAP32[$9>>2]|0;
 $11 = HEAP8[$enc>>0]|0;
 $12 = (_sqlite3AtoF($7,$rValue,$10,$11)|0);
 $13 = ($12|0)==(0);
 if ($13) {
  STACKTOP = sp;return;
 }
 $14 = HEAP32[$0>>2]|0;
 $15 = (($14) + 16|0);
 $16 = HEAP32[$15>>2]|0;
 $17 = HEAP32[$0>>2]|0;
 $18 = (($17) + 12|0);
 $19 = HEAP32[$18>>2]|0;
 $20 = HEAP8[$enc>>0]|0;
 $21 = (_sqlite3Atoi64($16,$iValue,$19,$20)|0);
 $22 = (0)==($21|0);
 if ($22) {
  $23 = $iValue;
  $24 = $23;
  $25 = HEAP32[$24>>2]|0;
  $26 = (($23) + 4)|0;
  $27 = $26;
  $28 = HEAP32[$27>>2]|0;
  $29 = HEAP32[$0>>2]|0;
  $30 = $29;
  $31 = $30;
  HEAP32[$31>>2] = $25;
  $32 = (($30) + 4)|0;
  $33 = $32;
  HEAP32[$33>>2] = $28;
  $34 = HEAP32[$0>>2]|0;
  $35 = (($34) + 8|0);
  $36 = HEAP16[$35>>1]|0;
  $37 = $36&65535;
  $38 = $37 | 4;
  $39 = $38&65535;
  HEAP16[$35>>1] = $39;
  STACKTOP = sp;return;
 }
 $40 = +HEAPF64[$rValue>>3];
 $41 = HEAP32[$0>>2]|0;
 HEAPF64[$41>>3] = $40;
 $42 = HEAP32[$0>>2]|0;
 $43 = (($42) + 8|0);
 $44 = HEAP16[$43>>1]|0;
 $45 = $44&65535;
 $46 = $45 | 8;
 $47 = $46&65535;
 HEAP16[$43>>1] = $47;
 $48 = HEAP32[$1>>2]|0;
 $49 = ($48|0)!=(0);
 if (!($49)) {
  STACKTOP = sp;return;
 }
 $50 = HEAP32[$0>>2]|0;
 _sqlite3VdbeIntegerAffinity($50);
 STACKTOP = sp;return;
}
function _sqlite3DbMallocRaw($db,$0,$1) {
 $db = $db|0;
 $0 = $0|0;
 $1 = $1|0;
 var $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0;
 var $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0;
 var $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0;
 var $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0;
 var $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $or$cond = 0, $p = 0, $pBuf = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $2 = sp + 20|0;
 $3 = sp + 16|0;
 $4 = sp;
 $p = sp + 12|0;
 $pBuf = sp + 8|0;
 HEAP32[$3>>2] = $db;
 $5 = $4;
 $6 = $5;
 HEAP32[$6>>2] = $0;
 $7 = (($5) + 4)|0;
 $8 = $7;
 HEAP32[$8>>2] = $1;
 $9 = HEAP32[$3>>2]|0;
 $10 = ($9|0)!=(0|0);
 do {
  if ($10) {
   $11 = HEAP32[$3>>2]|0;
   $12 = (($11) + 64|0);
   $13 = HEAP8[$12>>0]|0;
   $14 = ($13<<24>>24)!=(0);
   if ($14) {
    HEAP32[$2>>2] = 0;
    $96 = HEAP32[$2>>2]|0;
    STACKTOP = sp;return ($96|0);
   }
   $15 = HEAP32[$3>>2]|0;
   $16 = (($15) + 248|0);
   $17 = (($16) + 2|0);
   $18 = HEAP8[$17>>0]|0;
   $19 = ($18<<24>>24)!=(0);
   if ($19) {
    $20 = $4;
    $21 = $20;
    $22 = HEAP32[$21>>2]|0;
    $23 = (($20) + 4)|0;
    $24 = $23;
    $25 = HEAP32[$24>>2]|0;
    $26 = HEAP32[$3>>2]|0;
    $27 = (($26) + 248|0);
    $28 = HEAP16[$27>>1]|0;
    $29 = $28&65535;
    $30 = ($25>>>0)>(0);
    $31 = ($22>>>0)>($29>>>0);
    $32 = ($25|0)==(0);
    $33 = $32 & $31;
    $34 = $30 | $33;
    $35 = HEAP32[$3>>2]|0;
    $36 = (($35) + 248|0);
    if ($34) {
     $37 = (($36) + 12|0);
     $38 = (($37) + 4|0);
     $39 = HEAP32[$38>>2]|0;
     $40 = (($39) + 1)|0;
     HEAP32[$38>>2] = $40;
     break;
    }
    $41 = (($36) + 24|0);
    $42 = HEAP32[$41>>2]|0;
    HEAP32[$pBuf>>2] = $42;
    $43 = ($42|0)==(0|0);
    if ($43) {
     $44 = HEAP32[$3>>2]|0;
     $45 = (($44) + 248|0);
     $46 = (($45) + 12|0);
     $47 = (($46) + 8|0);
     $48 = HEAP32[$47>>2]|0;
     $49 = (($48) + 1)|0;
     HEAP32[$47>>2] = $49;
     break;
    }
    $50 = HEAP32[$pBuf>>2]|0;
    $51 = HEAP32[$50>>2]|0;
    $52 = HEAP32[$3>>2]|0;
    $53 = (($52) + 248|0);
    $54 = (($53) + 24|0);
    HEAP32[$54>>2] = $51;
    $55 = HEAP32[$3>>2]|0;
    $56 = (($55) + 248|0);
    $57 = (($56) + 4|0);
    $58 = HEAP32[$57>>2]|0;
    $59 = (($58) + 1)|0;
    HEAP32[$57>>2] = $59;
    $60 = HEAP32[$3>>2]|0;
    $61 = (($60) + 248|0);
    $62 = (($61) + 12|0);
    $63 = HEAP32[$62>>2]|0;
    $64 = (($63) + 1)|0;
    HEAP32[$62>>2] = $64;
    $65 = HEAP32[$3>>2]|0;
    $66 = (($65) + 248|0);
    $67 = (($66) + 4|0);
    $68 = HEAP32[$67>>2]|0;
    $69 = HEAP32[$3>>2]|0;
    $70 = (($69) + 248|0);
    $71 = (($70) + 8|0);
    $72 = HEAP32[$71>>2]|0;
    $73 = ($68|0)>($72|0);
    if ($73) {
     $74 = HEAP32[$3>>2]|0;
     $75 = (($74) + 248|0);
     $76 = (($75) + 4|0);
     $77 = HEAP32[$76>>2]|0;
     $78 = HEAP32[$3>>2]|0;
     $79 = (($78) + 248|0);
     $80 = (($79) + 8|0);
     HEAP32[$80>>2] = $77;
    }
    $81 = HEAP32[$pBuf>>2]|0;
    HEAP32[$2>>2] = $81;
    $96 = HEAP32[$2>>2]|0;
    STACKTOP = sp;return ($96|0);
   }
  }
 } while(0);
 $82 = $4;
 $83 = $82;
 $84 = HEAP32[$83>>2]|0;
 $85 = (($82) + 4)|0;
 $86 = $85;
 $87 = HEAP32[$86>>2]|0;
 $88 = (_sqlite3Malloc($84,$87)|0);
 HEAP32[$p>>2] = $88;
 $89 = HEAP32[$p>>2]|0;
 $90 = ($89|0)==(0|0);
 $91 = HEAP32[$3>>2]|0;
 $92 = ($91|0)!=(0|0);
 $or$cond = $90 & $92;
 if ($or$cond) {
  $93 = HEAP32[$3>>2]|0;
  $94 = (($93) + 64|0);
  HEAP8[$94>>0] = 1;
 }
 $95 = HEAP32[$p>>2]|0;
 HEAP32[$2>>2] = $95;
 $96 = HEAP32[$2>>2]|0;
 STACKTOP = sp;return ($96|0);
}
function _sqlite3LocateTable($pParse,$isView,$zName,$zDbase) {
 $pParse = $pParse|0;
 $isView = $isView|0;
 $zName = $zName|0;
 $zDbase = $zDbase|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $p = 0, $vararg_buffer3 = 0, $vararg_ptr1 = 0, $vararg_ptr2 = 0, $vararg_ptr6 = 0, $zMsg = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $vararg_buffer3 = sp;
 $0 = sp + 36|0;
 $1 = sp + 32|0;
 $2 = sp + 28|0;
 $3 = sp + 24|0;
 $4 = sp + 20|0;
 $p = sp + 16|0;
 $zMsg = sp + 12|0;
 HEAP32[$1>>2] = $pParse;
 HEAP32[$2>>2] = $isView;
 HEAP32[$3>>2] = $zName;
 HEAP32[$4>>2] = $zDbase;
 $5 = HEAP32[$1>>2]|0;
 $6 = (_sqlite3ReadSchema($5)|0);
 $7 = (0)!=($6|0);
 if ($7) {
  HEAP32[$0>>2] = 0;
  $28 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($28|0);
 }
 $8 = HEAP32[$1>>2]|0;
 $9 = HEAP32[$8>>2]|0;
 $10 = HEAP32[$3>>2]|0;
 $11 = HEAP32[$4>>2]|0;
 $12 = (_sqlite3FindTable($9,$10,$11)|0);
 HEAP32[$p>>2] = $12;
 $13 = HEAP32[$p>>2]|0;
 $14 = ($13|0)==(0|0);
 if ($14) {
  $15 = HEAP32[$2>>2]|0;
  $16 = ($15|0)!=(0);
  $17 = $16 ? 31152 : 31168;
  HEAP32[$zMsg>>2] = $17;
  $18 = HEAP32[$4>>2]|0;
  $19 = ($18|0)!=(0|0);
  $20 = HEAP32[$1>>2]|0;
  $21 = HEAP32[$zMsg>>2]|0;
  if ($19) {
   $22 = HEAP32[$4>>2]|0;
   $23 = HEAP32[$3>>2]|0;
   HEAP32[$vararg_buffer3>>2] = $21;
   $vararg_ptr1 = (($vararg_buffer3) + 4|0);
   HEAP32[$vararg_ptr1>>2] = $22;
   $vararg_ptr2 = (($vararg_buffer3) + 8|0);
   HEAP32[$vararg_ptr2>>2] = $23;
   _sqlite3ErrorMsg($20,17264,$vararg_buffer3);
  } else {
   $24 = HEAP32[$3>>2]|0;
   HEAP32[$vararg_buffer3>>2] = $21;
   $vararg_ptr6 = (($vararg_buffer3) + 4|0);
   HEAP32[$vararg_ptr6>>2] = $24;
   _sqlite3ErrorMsg($20,17280,$vararg_buffer3);
  }
  $25 = HEAP32[$1>>2]|0;
  $26 = (($25) + 17|0);
  HEAP8[$26>>0] = 1;
 }
 $27 = HEAP32[$p>>2]|0;
 HEAP32[$0>>2] = $27;
 $28 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($28|0);
}
function _sqlite3ErrorMsg($pParse,$zFormat,$varargs) {
 $pParse = $pParse|0;
 $zFormat = $zFormat|0;
 $varargs = $varargs|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $3 = 0;
 var $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $ap = 0, $db = 0, $zMsg = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $0 = sp + 32|0;
 $1 = sp + 28|0;
 $zMsg = sp + 24|0;
 $ap = sp + 8|0;
 $db = sp;
 HEAP32[$0>>2] = $pParse;
 HEAP32[$1>>2] = $zFormat;
 $2 = HEAP32[$0>>2]|0;
 $3 = HEAP32[$2>>2]|0;
 HEAP32[$db>>2] = $3;
 HEAP32[$ap>>2] = $varargs;
 $4 = HEAP32[$db>>2]|0;
 $5 = HEAP32[$1>>2]|0;
 $6 = (_sqlite3VMPrintf($4,$5,$ap)|0);
 HEAP32[$zMsg>>2] = $6;
 $7 = HEAP32[$db>>2]|0;
 $8 = (($7) + 67|0);
 $9 = HEAP8[$8>>0]|0;
 $10 = ($9<<24>>24)!=(0);
 if ($10) {
  $11 = HEAP32[$db>>2]|0;
  $12 = HEAP32[$zMsg>>2]|0;
  _sqlite3DbFree($11,$12);
  STACKTOP = sp;return;
 } else {
  $13 = HEAP32[$0>>2]|0;
  $14 = (($13) + 64|0);
  $15 = HEAP32[$14>>2]|0;
  $16 = (($15) + 1)|0;
  HEAP32[$14>>2] = $16;
  $17 = HEAP32[$db>>2]|0;
  $18 = HEAP32[$0>>2]|0;
  $19 = (($18) + 4|0);
  $20 = HEAP32[$19>>2]|0;
  _sqlite3DbFree($17,$20);
  $21 = HEAP32[$zMsg>>2]|0;
  $22 = HEAP32[$0>>2]|0;
  $23 = (($22) + 4|0);
  HEAP32[$23>>2] = $21;
  $24 = HEAP32[$0>>2]|0;
  $25 = (($24) + 12|0);
  HEAP32[$25>>2] = 1;
  STACKTOP = sp;return;
 }
}
function _sqlite3MPrintf($db,$zFormat,$varargs) {
 $db = $db|0;
 $zFormat = $zFormat|0;
 $varargs = $varargs|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $ap = 0, $z = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 28|0;
 $1 = sp + 24|0;
 $ap = sp + 8|0;
 $z = sp;
 HEAP32[$0>>2] = $db;
 HEAP32[$1>>2] = $zFormat;
 HEAP32[$ap>>2] = $varargs;
 $2 = HEAP32[$0>>2]|0;
 $3 = HEAP32[$1>>2]|0;
 $4 = (_sqlite3VMPrintf($2,$3,$ap)|0);
 HEAP32[$z>>2] = $4;
 $5 = HEAP32[$z>>2]|0;
 STACKTOP = sp;return ($5|0);
}
function _sqlite3VdbeCreate($pParse) {
 $pParse = $pParse|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $db = 0, $p = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $1 = sp + 8|0;
 $db = sp + 4|0;
 $p = sp;
 HEAP32[$1>>2] = $pParse;
 $2 = HEAP32[$1>>2]|0;
 $3 = HEAP32[$2>>2]|0;
 HEAP32[$db>>2] = $3;
 $4 = HEAP32[$db>>2]|0;
 $5 = (_sqlite3DbMallocZero($4,208,0)|0);
 HEAP32[$p>>2] = $5;
 $6 = HEAP32[$p>>2]|0;
 $7 = ($6|0)==(0|0);
 if ($7) {
  HEAP32[$0>>2] = 0;
  $35 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($35|0);
 }
 $8 = HEAP32[$db>>2]|0;
 $9 = HEAP32[$p>>2]|0;
 HEAP32[$9>>2] = $8;
 $10 = HEAP32[$db>>2]|0;
 $11 = (($10) + 4|0);
 $12 = HEAP32[$11>>2]|0;
 $13 = ($12|0)!=(0|0);
 if ($13) {
  $14 = HEAP32[$p>>2]|0;
  $15 = HEAP32[$db>>2]|0;
  $16 = (($15) + 4|0);
  $17 = HEAP32[$16>>2]|0;
  $18 = (($17) + 48|0);
  HEAP32[$18>>2] = $14;
 }
 $19 = HEAP32[$db>>2]|0;
 $20 = (($19) + 4|0);
 $21 = HEAP32[$20>>2]|0;
 $22 = HEAP32[$p>>2]|0;
 $23 = (($22) + 52|0);
 HEAP32[$23>>2] = $21;
 $24 = HEAP32[$p>>2]|0;
 $25 = (($24) + 48|0);
 HEAP32[$25>>2] = 0;
 $26 = HEAP32[$p>>2]|0;
 $27 = HEAP32[$db>>2]|0;
 $28 = (($27) + 4|0);
 HEAP32[$28>>2] = $26;
 $29 = HEAP32[$p>>2]|0;
 $30 = (($29) + 40|0);
 HEAP32[$30>>2] = 649915045;
 $31 = HEAP32[$1>>2]|0;
 $32 = HEAP32[$p>>2]|0;
 $33 = (($32) + 24|0);
 HEAP32[$33>>2] = $31;
 $34 = HEAP32[$p>>2]|0;
 HEAP32[$0>>2] = $34;
 $35 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($35|0);
}
function _sqlite3SchemaToIndex($db,$pSchema) {
 $db = $db|0;
 $pSchema = $pSchema|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0;
 var $9 = 0, $i = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $1 = sp + 4|0;
 $i = sp;
 HEAP32[$0>>2] = $db;
 HEAP32[$1>>2] = $pSchema;
 HEAP32[$i>>2] = -1000000;
 $2 = HEAP32[$1>>2]|0;
 $3 = ($2|0)!=(0|0);
 if (!($3)) {
  $20 = HEAP32[$i>>2]|0;
  STACKTOP = sp;return ($20|0);
 }
 HEAP32[$i>>2] = 0;
 while(1) {
  $4 = HEAP32[$i>>2]|0;
  $5 = HEAP32[$0>>2]|0;
  $6 = (($5) + 20|0);
  $7 = HEAP32[$6>>2]|0;
  $8 = ($4|0)<($7|0);
  if (!($8)) {
   label = 6;
   break;
  }
  $9 = HEAP32[$i>>2]|0;
  $10 = HEAP32[$0>>2]|0;
  $11 = (($10) + 16|0);
  $12 = HEAP32[$11>>2]|0;
  $13 = (($12) + ($9<<4)|0);
  $14 = (($13) + 12|0);
  $15 = HEAP32[$14>>2]|0;
  $16 = HEAP32[$1>>2]|0;
  $17 = ($15|0)==($16|0);
  if ($17) {
   label = 6;
   break;
  }
  $18 = HEAP32[$i>>2]|0;
  $19 = (($18) + 1)|0;
  HEAP32[$i>>2] = $19;
 }
 if ((label|0) == 6) {
  $20 = HEAP32[$i>>2]|0;
  STACKTOP = sp;return ($20|0);
 }
 return 0|0;
}
function _sqlite3VdbeAddOp4Int($p,$op,$p1,$p2,$p3,$p4) {
 $p = $p|0;
 $op = $op|0;
 $p1 = $p1|0;
 $p2 = $p2|0;
 $p3 = $p3|0;
 $p4 = $p4|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $addr = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 24|0;
 $1 = sp + 20|0;
 $2 = sp + 16|0;
 $3 = sp + 12|0;
 $4 = sp + 8|0;
 $5 = sp + 4|0;
 $addr = sp;
 HEAP32[$0>>2] = $p;
 HEAP32[$1>>2] = $op;
 HEAP32[$2>>2] = $p1;
 HEAP32[$3>>2] = $p2;
 HEAP32[$4>>2] = $p3;
 HEAP32[$5>>2] = $p4;
 $6 = HEAP32[$0>>2]|0;
 $7 = HEAP32[$1>>2]|0;
 $8 = HEAP32[$2>>2]|0;
 $9 = HEAP32[$3>>2]|0;
 $10 = HEAP32[$4>>2]|0;
 $11 = (_sqlite3VdbeAddOp3($6,$7,$8,$9,$10)|0);
 HEAP32[$addr>>2] = $11;
 $12 = HEAP32[$0>>2]|0;
 $13 = HEAP32[$addr>>2]|0;
 $14 = HEAP32[$5>>2]|0;
 $15 = $14;
 _sqlite3VdbeChangeP4($12,$13,$15,-14);
 $16 = HEAP32[$addr>>2]|0;
 STACKTOP = sp;return ($16|0);
}
function _sqlite3VdbeChangeP5($p,$val) {
 $p = $p|0;
 $val = $val|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 $1 = sp + 4|0;
 HEAP32[$0>>2] = $p;
 HEAP8[$1>>0] = $val;
 $2 = HEAP32[$0>>2]|0;
 $3 = (($2) + 4|0);
 $4 = HEAP32[$3>>2]|0;
 $5 = ($4|0)!=(0|0);
 if (!($5)) {
  STACKTOP = sp;return;
 }
 $6 = HEAP8[$1>>0]|0;
 $7 = HEAP32[$0>>2]|0;
 $8 = (($7) + 32|0);
 $9 = HEAP32[$8>>2]|0;
 $10 = (($9) - 1)|0;
 $11 = HEAP32[$0>>2]|0;
 $12 = (($11) + 4|0);
 $13 = HEAP32[$12>>2]|0;
 $14 = (($13) + (($10*20)|0)|0);
 $15 = (($14) + 3|0);
 HEAP8[$15>>0] = $6;
 STACKTOP = sp;return;
}
function _sqlite3VdbeAddOpList($p,$nOp,$aOp,$iLineno) {
 $p = $p|0;
 $nOp = $nOp|0;
 $aOp = $aOp|0;
 $iLineno = $iLineno|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0;
 var $81 = 0, $82 = 0, $9 = 0, $addr = 0, $i = 0, $p2 = 0, $pIn = 0, $pOut = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $0 = sp + 36|0;
 $1 = sp + 32|0;
 $2 = sp + 28|0;
 $3 = sp + 24|0;
 $4 = sp + 20|0;
 $addr = sp + 16|0;
 $i = sp + 12|0;
 $pIn = sp + 8|0;
 $p2 = sp + 4|0;
 $pOut = sp;
 HEAP32[$1>>2] = $p;
 HEAP32[$2>>2] = $nOp;
 HEAP32[$3>>2] = $aOp;
 HEAP32[$4>>2] = $iLineno;
 $5 = HEAP32[$1>>2]|0;
 $6 = (($5) + 32|0);
 $7 = HEAP32[$6>>2]|0;
 $8 = HEAP32[$2>>2]|0;
 $9 = (($7) + ($8))|0;
 $10 = HEAP32[$1>>2]|0;
 $11 = (($10) + 24|0);
 $12 = HEAP32[$11>>2]|0;
 $13 = (($12) + 84|0);
 $14 = HEAP32[$13>>2]|0;
 $15 = ($9|0)>($14|0);
 if ($15) {
  $16 = HEAP32[$1>>2]|0;
  $17 = HEAP32[$2>>2]|0;
  $18 = (_growOpArray($16,$17)|0);
  $19 = ($18|0)!=(0);
  if ($19) {
   HEAP32[$0>>2] = 0;
   $82 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($82|0);
  }
 }
 $20 = HEAP32[$1>>2]|0;
 $21 = (($20) + 32|0);
 $22 = HEAP32[$21>>2]|0;
 HEAP32[$addr>>2] = $22;
 $23 = HEAP32[$2>>2]|0;
 $24 = ($23|0)>(0);
 if ($24) {
  $25 = HEAP32[$3>>2]|0;
  HEAP32[$pIn>>2] = $25;
  HEAP32[$i>>2] = 0;
  while(1) {
   $26 = HEAP32[$i>>2]|0;
   $27 = HEAP32[$2>>2]|0;
   $28 = ($26|0)<($27|0);
   if (!($28)) {
    break;
   }
   $29 = HEAP32[$pIn>>2]|0;
   $30 = (($29) + 2|0);
   $31 = HEAP8[$30>>0]|0;
   $32 = $31 << 24 >> 24;
   HEAP32[$p2>>2] = $32;
   $33 = HEAP32[$i>>2]|0;
   $34 = HEAP32[$addr>>2]|0;
   $35 = (($33) + ($34))|0;
   $36 = HEAP32[$1>>2]|0;
   $37 = (($36) + 4|0);
   $38 = HEAP32[$37>>2]|0;
   $39 = (($38) + (($35*20)|0)|0);
   HEAP32[$pOut>>2] = $39;
   $40 = HEAP32[$pIn>>2]|0;
   $41 = HEAP8[$40>>0]|0;
   $42 = HEAP32[$pOut>>2]|0;
   HEAP8[$42>>0] = $41;
   $43 = HEAP32[$pIn>>2]|0;
   $44 = (($43) + 1|0);
   $45 = HEAP8[$44>>0]|0;
   $46 = $45 << 24 >> 24;
   $47 = HEAP32[$pOut>>2]|0;
   $48 = (($47) + 4|0);
   HEAP32[$48>>2] = $46;
   $49 = HEAP32[$p2>>2]|0;
   $50 = ($49|0)<(0);
   if ($50) {
    $51 = HEAP32[$addr>>2]|0;
    $52 = HEAP32[$p2>>2]|0;
    $53 = (-1 - ($52))|0;
    $54 = (($51) + ($53))|0;
    $55 = HEAP32[$pOut>>2]|0;
    $56 = (($55) + 8|0);
    HEAP32[$56>>2] = $54;
   } else {
    $57 = HEAP32[$p2>>2]|0;
    $58 = HEAP32[$pOut>>2]|0;
    $59 = (($58) + 8|0);
    HEAP32[$59>>2] = $57;
   }
   $60 = HEAP32[$pIn>>2]|0;
   $61 = (($60) + 3|0);
   $62 = HEAP8[$61>>0]|0;
   $63 = $62 << 24 >> 24;
   $64 = HEAP32[$pOut>>2]|0;
   $65 = (($64) + 12|0);
   HEAP32[$65>>2] = $63;
   $66 = HEAP32[$pOut>>2]|0;
   $67 = (($66) + 1|0);
   HEAP8[$67>>0] = 0;
   $68 = HEAP32[$pOut>>2]|0;
   $69 = (($68) + 16|0);
   HEAP32[$69>>2] = 0;
   $70 = HEAP32[$pOut>>2]|0;
   $71 = (($70) + 3|0);
   HEAP8[$71>>0] = 0;
   $72 = HEAP32[$i>>2]|0;
   $73 = (($72) + 1)|0;
   HEAP32[$i>>2] = $73;
   $74 = HEAP32[$pIn>>2]|0;
   $75 = (($74) + 4|0);
   HEAP32[$pIn>>2] = $75;
  }
  $76 = HEAP32[$2>>2]|0;
  $77 = HEAP32[$1>>2]|0;
  $78 = (($77) + 32|0);
  $79 = HEAP32[$78>>2]|0;
  $80 = (($79) + ($76))|0;
  HEAP32[$78>>2] = $80;
 }
 $81 = HEAP32[$addr>>2]|0;
 HEAP32[$0>>2] = $81;
 $82 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($82|0);
}
function _sqlite3VdbeUsesBtree($p,$i) {
 $p = $p|0;
 $i = $i|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $1 = sp;
 HEAP32[$0>>2] = $p;
 HEAP32[$1>>2] = $i;
 $2 = HEAP32[$1>>2]|0;
 $3 = 1 << $2;
 $4 = HEAP32[$0>>2]|0;
 $5 = (($4) + 96|0);
 $6 = HEAP32[$5>>2]|0;
 $7 = $6 | $3;
 HEAP32[$5>>2] = $7;
 STACKTOP = sp;return;
}
function _sqlite3VdbeChangeP1($p,$addr,$val) {
 $p = $p|0;
 $addr = $addr|0;
 $val = $val|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $1 = sp + 4|0;
 $2 = sp;
 HEAP32[$0>>2] = $p;
 HEAP32[$1>>2] = $addr;
 HEAP32[$2>>2] = $val;
 $3 = HEAP32[$0>>2]|0;
 $4 = (($3) + 32|0);
 $5 = HEAP32[$4>>2]|0;
 $6 = HEAP32[$1>>2]|0;
 $7 = ($5>>>0)>($6>>>0);
 if (!($7)) {
  STACKTOP = sp;return;
 }
 $8 = HEAP32[$2>>2]|0;
 $9 = HEAP32[$1>>2]|0;
 $10 = HEAP32[$0>>2]|0;
 $11 = (($10) + 4|0);
 $12 = HEAP32[$11>>2]|0;
 $13 = (($12) + (($9*20)|0)|0);
 $14 = (($13) + 4|0);
 HEAP32[$14>>2] = $8;
 STACKTOP = sp;return;
}
function _sqlite3VdbeChangeP2($p,$addr,$val) {
 $p = $p|0;
 $addr = $addr|0;
 $val = $val|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $1 = sp + 4|0;
 $2 = sp;
 HEAP32[$0>>2] = $p;
 HEAP32[$1>>2] = $addr;
 HEAP32[$2>>2] = $val;
 $3 = HEAP32[$0>>2]|0;
 $4 = (($3) + 32|0);
 $5 = HEAP32[$4>>2]|0;
 $6 = HEAP32[$1>>2]|0;
 $7 = ($5>>>0)>($6>>>0);
 if (!($7)) {
  STACKTOP = sp;return;
 }
 $8 = HEAP32[$2>>2]|0;
 $9 = HEAP32[$1>>2]|0;
 $10 = HEAP32[$0>>2]|0;
 $11 = (($10) + 4|0);
 $12 = HEAP32[$11>>2]|0;
 $13 = (($12) + (($9*20)|0)|0);
 $14 = (($13) + 8|0);
 HEAP32[$14>>2] = $8;
 STACKTOP = sp;return;
}
function _sqlite3VdbeChangeP3($p,$addr,$val) {
 $p = $p|0;
 $addr = $addr|0;
 $val = $val|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $1 = sp + 4|0;
 $2 = sp;
 HEAP32[$0>>2] = $p;
 HEAP32[$1>>2] = $addr;
 HEAP32[$2>>2] = $val;
 $3 = HEAP32[$0>>2]|0;
 $4 = (($3) + 32|0);
 $5 = HEAP32[$4>>2]|0;
 $6 = HEAP32[$1>>2]|0;
 $7 = ($5>>>0)>($6>>>0);
 if (!($7)) {
  STACKTOP = sp;return;
 }
 $8 = HEAP32[$2>>2]|0;
 $9 = HEAP32[$1>>2]|0;
 $10 = HEAP32[$0>>2]|0;
 $11 = (($10) + 4|0);
 $12 = HEAP32[$11>>2]|0;
 $13 = (($12) + (($9*20)|0)|0);
 $14 = (($13) + 12|0);
 HEAP32[$14>>2] = $8;
 STACKTOP = sp;return;
}
function _sqlite3VdbeChangeP4($p,$addr,$zP4,$n) {
 $p = $p|0;
 $addr = $addr|0;
 $zP4 = $zP4|0;
 $n = $n|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0;
 var $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $db = 0, $pOp = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 20|0;
 $1 = sp + 16|0;
 $2 = sp + 12|0;
 $3 = sp + 8|0;
 $pOp = sp + 4|0;
 $db = sp;
 HEAP32[$0>>2] = $p;
 HEAP32[$1>>2] = $addr;
 HEAP32[$2>>2] = $zP4;
 HEAP32[$3>>2] = $n;
 $4 = HEAP32[$0>>2]|0;
 $5 = HEAP32[$4>>2]|0;
 HEAP32[$db>>2] = $5;
 $6 = HEAP32[$0>>2]|0;
 $7 = (($6) + 4|0);
 $8 = HEAP32[$7>>2]|0;
 $9 = ($8|0)==(0|0);
 if (!($9)) {
  $10 = HEAP32[$db>>2]|0;
  $11 = (($10) + 64|0);
  $12 = HEAP8[$11>>0]|0;
  $13 = $12&255;
  $14 = ($13|0)!=(0);
  if (!($14)) {
   $20 = HEAP32[$1>>2]|0;
   $21 = ($20|0)<(0);
   if ($21) {
    $22 = HEAP32[$0>>2]|0;
    $23 = (($22) + 32|0);
    $24 = HEAP32[$23>>2]|0;
    $25 = (($24) - 1)|0;
    HEAP32[$1>>2] = $25;
   }
   $26 = HEAP32[$1>>2]|0;
   $27 = HEAP32[$0>>2]|0;
   $28 = (($27) + 4|0);
   $29 = HEAP32[$28>>2]|0;
   $30 = (($29) + (($26*20)|0)|0);
   HEAP32[$pOp>>2] = $30;
   $31 = HEAP32[$db>>2]|0;
   $32 = HEAP32[$pOp>>2]|0;
   $33 = (($32) + 1|0);
   $34 = HEAP8[$33>>0]|0;
   $35 = $34 << 24 >> 24;
   $36 = HEAP32[$pOp>>2]|0;
   $37 = (($36) + 16|0);
   $38 = HEAP32[$37>>2]|0;
   _freeP4($31,$35,$38);
   $39 = HEAP32[$pOp>>2]|0;
   $40 = (($39) + 16|0);
   HEAP32[$40>>2] = 0;
   $41 = HEAP32[$3>>2]|0;
   $42 = ($41|0)==(-14);
   $43 = HEAP32[$2>>2]|0;
   if ($42) {
    $44 = $43;
    $45 = HEAP32[$pOp>>2]|0;
    $46 = (($45) + 16|0);
    HEAP32[$46>>2] = $44;
    $47 = HEAP32[$pOp>>2]|0;
    $48 = (($47) + 1|0);
    HEAP8[$48>>0] = -14;
    STACKTOP = sp;return;
   }
   $49 = ($43|0)==(0|0);
   if ($49) {
    $50 = HEAP32[$pOp>>2]|0;
    $51 = (($50) + 16|0);
    HEAP32[$51>>2] = 0;
    $52 = HEAP32[$pOp>>2]|0;
    $53 = (($52) + 1|0);
    HEAP8[$53>>0] = 0;
    STACKTOP = sp;return;
   }
   $54 = HEAP32[$3>>2]|0;
   $55 = ($54|0)==(-6);
   if ($55) {
    $56 = HEAP32[$2>>2]|0;
    $57 = HEAP32[$pOp>>2]|0;
    $58 = (($57) + 16|0);
    HEAP32[$58>>2] = $56;
    $59 = HEAP32[$pOp>>2]|0;
    $60 = (($59) + 1|0);
    HEAP8[$60>>0] = -6;
    STACKTOP = sp;return;
   }
   $61 = HEAP32[$3>>2]|0;
   $62 = ($61|0)==(-10);
   if ($62) {
    $63 = HEAP32[$2>>2]|0;
    $64 = HEAP32[$pOp>>2]|0;
    $65 = (($64) + 16|0);
    HEAP32[$65>>2] = $63;
    $66 = HEAP32[$pOp>>2]|0;
    $67 = (($66) + 1|0);
    HEAP8[$67>>0] = -10;
    $68 = HEAP32[$2>>2]|0;
    _sqlite3VtabLock($68);
    STACKTOP = sp;return;
   }
   $69 = HEAP32[$3>>2]|0;
   $70 = ($69|0)<(0);
   if ($70) {
    $71 = HEAP32[$2>>2]|0;
    $72 = HEAP32[$pOp>>2]|0;
    $73 = (($72) + 16|0);
    HEAP32[$73>>2] = $71;
    $74 = HEAP32[$3>>2]|0;
    $75 = $74&255;
    $76 = HEAP32[$pOp>>2]|0;
    $77 = (($76) + 1|0);
    HEAP8[$77>>0] = $75;
    STACKTOP = sp;return;
   }
   $78 = HEAP32[$3>>2]|0;
   $79 = ($78|0)==(0);
   if ($79) {
    $80 = HEAP32[$2>>2]|0;
    $81 = (_sqlite3Strlen30($80)|0);
    HEAP32[$3>>2] = $81;
   }
   $82 = HEAP32[$0>>2]|0;
   $83 = HEAP32[$82>>2]|0;
   $84 = HEAP32[$2>>2]|0;
   $85 = HEAP32[$3>>2]|0;
   $86 = ($85|0)<(0);
   $87 = $86 << 31 >> 31;
   $88 = (_sqlite3DbStrNDup($83,$84,$85,$87)|0);
   $89 = HEAP32[$pOp>>2]|0;
   $90 = (($89) + 16|0);
   HEAP32[$90>>2] = $88;
   $91 = HEAP32[$pOp>>2]|0;
   $92 = (($91) + 1|0);
   HEAP8[$92>>0] = -1;
   STACKTOP = sp;return;
  }
 }
 $15 = HEAP32[$3>>2]|0;
 $16 = ($15|0)!=(-10);
 if (!($16)) {
  STACKTOP = sp;return;
 }
 $17 = HEAP32[$db>>2]|0;
 $18 = HEAP32[$3>>2]|0;
 $19 = HEAP32[$2>>2]|0;
 _freeP4($17,$18,$19);
 STACKTOP = sp;return;
}
function _sqlite3VdbeChangeToNoop($p,$addr) {
 $p = $p|0;
 $addr = $addr|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $db = 0, $pOp = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $1 = sp + 8|0;
 $pOp = sp + 4|0;
 $db = sp;
 HEAP32[$0>>2] = $p;
 HEAP32[$1>>2] = $addr;
 $2 = HEAP32[$1>>2]|0;
 $3 = HEAP32[$0>>2]|0;
 $4 = (($3) + 32|0);
 $5 = HEAP32[$4>>2]|0;
 $6 = ($2|0)<($5|0);
 if (!($6)) {
  STACKTOP = sp;return;
 }
 $7 = HEAP32[$1>>2]|0;
 $8 = HEAP32[$0>>2]|0;
 $9 = (($8) + 4|0);
 $10 = HEAP32[$9>>2]|0;
 $11 = (($10) + (($7*20)|0)|0);
 HEAP32[$pOp>>2] = $11;
 $12 = HEAP32[$0>>2]|0;
 $13 = HEAP32[$12>>2]|0;
 HEAP32[$db>>2] = $13;
 $14 = HEAP32[$db>>2]|0;
 $15 = HEAP32[$pOp>>2]|0;
 $16 = (($15) + 1|0);
 $17 = HEAP8[$16>>0]|0;
 $18 = $17 << 24 >> 24;
 $19 = HEAP32[$pOp>>2]|0;
 $20 = (($19) + 16|0);
 $21 = HEAP32[$20>>2]|0;
 _freeP4($14,$18,$21);
 $22 = HEAP32[$pOp>>2]|0;
 ;HEAP32[$22+0>>2]=0|0;HEAP32[$22+4>>2]=0|0;HEAP32[$22+8>>2]=0|0;HEAP32[$22+12>>2]=0|0;HEAP32[$22+16>>2]=0|0;
 $23 = HEAP32[$pOp>>2]|0;
 HEAP8[$23>>0] = -102;
 $24 = HEAP32[$1>>2]|0;
 $25 = HEAP32[$0>>2]|0;
 $26 = (($25) + 32|0);
 $27 = HEAP32[$26>>2]|0;
 $28 = (($27) - 1)|0;
 $29 = ($24|0)==($28|0);
 if (!($29)) {
  STACKTOP = sp;return;
 }
 $30 = HEAP32[$0>>2]|0;
 $31 = (($30) + 32|0);
 $32 = HEAP32[$31>>2]|0;
 $33 = (($32) + -1)|0;
 HEAP32[$31>>2] = $33;
 STACKTOP = sp;return;
}
function _sqlite3VdbeMakeReady($p,$pParse) {
 $p = $p|0;
 $pParse = $pParse|0;
 var $$ = 0, $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0;
 var $115 = 0, $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0;
 var $133 = 0, $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0;
 var $151 = 0, $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0;
 var $17 = 0, $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0;
 var $188 = 0, $189 = 0, $19 = 0, $190 = 0, $191 = 0, $192 = 0, $193 = 0, $194 = 0, $195 = 0, $196 = 0, $197 = 0, $198 = 0, $199 = 0, $2 = 0, $20 = 0, $200 = 0, $201 = 0, $202 = 0, $203 = 0, $204 = 0;
 var $205 = 0, $206 = 0, $207 = 0, $208 = 0, $209 = 0, $21 = 0, $210 = 0, $211 = 0, $212 = 0, $213 = 0, $214 = 0, $215 = 0, $216 = 0, $217 = 0, $218 = 0, $219 = 0, $22 = 0, $220 = 0, $221 = 0, $222 = 0;
 var $223 = 0, $224 = 0, $225 = 0, $226 = 0, $227 = 0, $228 = 0, $229 = 0, $23 = 0, $230 = 0, $231 = 0, $232 = 0, $233 = 0, $234 = 0, $235 = 0, $236 = 0, $237 = 0, $238 = 0, $239 = 0, $24 = 0, $240 = 0;
 var $241 = 0, $242 = 0, $243 = 0, $244 = 0, $245 = 0, $246 = 0, $247 = 0, $248 = 0, $249 = 0, $25 = 0, $250 = 0, $251 = 0, $252 = 0, $253 = 0, $254 = 0, $255 = 0, $256 = 0, $257 = 0, $258 = 0, $259 = 0;
 var $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0;
 var $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0;
 var $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0;
 var $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0;
 var $99 = 0, $db = 0, $n = 0, $nArg = 0, $nByte = 0, $nCursor = 0, $nMem = 0, $nOnce = 0, $nVar = 0, $or$cond = 0, $zCsr = 0, $zEnd = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $0 = sp + 44|0;
 $1 = sp + 40|0;
 $db = sp + 36|0;
 $nVar = sp + 32|0;
 $nMem = sp + 28|0;
 $nCursor = sp + 24|0;
 $nArg = sp + 20|0;
 $nOnce = sp + 16|0;
 $n = sp + 12|0;
 $zCsr = sp + 8|0;
 $zEnd = sp + 4|0;
 $nByte = sp;
 HEAP32[$0>>2] = $p;
 HEAP32[$1>>2] = $pParse;
 $2 = HEAP32[$0>>2]|0;
 $3 = HEAP32[$2>>2]|0;
 HEAP32[$db>>2] = $3;
 $4 = HEAP32[$1>>2]|0;
 $5 = (($4) + 440|0);
 $6 = HEAP32[$5>>2]|0;
 HEAP32[$nVar>>2] = $6;
 $7 = HEAP32[$1>>2]|0;
 $8 = (($7) + 72|0);
 $9 = HEAP32[$8>>2]|0;
 HEAP32[$nMem>>2] = $9;
 $10 = HEAP32[$1>>2]|0;
 $11 = (($10) + 68|0);
 $12 = HEAP32[$11>>2]|0;
 HEAP32[$nCursor>>2] = $12;
 $13 = HEAP32[$1>>2]|0;
 $14 = (($13) + 392|0);
 $15 = HEAP32[$14>>2]|0;
 HEAP32[$nArg>>2] = $15;
 $16 = HEAP32[$1>>2]|0;
 $17 = (($16) + 80|0);
 $18 = HEAP32[$17>>2]|0;
 HEAP32[$nOnce>>2] = $18;
 $19 = HEAP32[$nOnce>>2]|0;
 $20 = ($19|0)==(0);
 $$ = $20 ? 1 : $18;
 HEAP32[$nOnce>>2] = $$;
 $21 = HEAP32[$nCursor>>2]|0;
 $22 = HEAP32[$nMem>>2]|0;
 $23 = (($22) + ($21))|0;
 HEAP32[$nMem>>2] = $23;
 $24 = HEAP32[$0>>2]|0;
 $25 = (($24) + 32|0);
 $26 = HEAP32[$25>>2]|0;
 $27 = HEAP32[$0>>2]|0;
 $28 = (($27) + 4|0);
 $29 = HEAP32[$28>>2]|0;
 $30 = (($29) + (($26*20)|0)|0);
 HEAP32[$zCsr>>2] = $30;
 $31 = HEAP32[$1>>2]|0;
 $32 = (($31) + 84|0);
 $33 = HEAP32[$32>>2]|0;
 $34 = HEAP32[$0>>2]|0;
 $35 = (($34) + 4|0);
 $36 = HEAP32[$35>>2]|0;
 $37 = (($36) + (($33*20)|0)|0);
 HEAP32[$zEnd>>2] = $37;
 $38 = HEAP32[$0>>2]|0;
 _resolveP2Values($38,$nArg);
 $39 = HEAP32[$1>>2]|0;
 $40 = (($39) + 20|0);
 $41 = HEAP8[$40>>0]|0;
 $42 = $41&255;
 $43 = ($42|0)!=(0);
 if ($43) {
  $44 = HEAP32[$1>>2]|0;
  $45 = (($44) + 21|0);
  $46 = HEAP8[$45>>0]|0;
  $47 = $46&255;
  $48 = ($47|0)!=(0);
  $50 = $48;
 } else {
  $50 = 0;
 }
 $49 = $50&1;
 $51 = $49&255;
 $52 = $51&255;
 $53 = HEAP32[$0>>2]|0;
 $54 = (($53) + 88|0);
 $55 = $52&65535;
 $56 = HEAP16[$54>>1]|0;
 $57 = $55 & 1;
 $58 = ($57 << 7)&65535;
 $59 = $56 & -129;
 $60 = $59 | $58;
 HEAP16[$54>>1] = $60;
 $61 = HEAP32[$1>>2]|0;
 $62 = (($61) + 450|0);
 $63 = HEAP8[$62>>0]|0;
 $64 = $63&255;
 $65 = ($64|0)!=(0);
 $66 = HEAP32[$nMem>>2]|0;
 $67 = ($66|0)<(10);
 $or$cond = $65 & $67;
 if ($or$cond) {
  HEAP32[$nMem>>2] = 10;
 }
 $68 = HEAP32[$zCsr>>2]|0;
 $69 = HEAP32[$zEnd>>2]|0;
 $70 = HEAP32[$zCsr>>2]|0;
 $71 = $69;
 $72 = $70;
 $73 = (($71) - ($72))|0;
 _memset(($68|0),0,($73|0))|0;
 $74 = HEAP32[$zCsr>>2]|0;
 $75 = $74;
 $76 = (($75) - 0)|0;
 $77 = $76 & 7;
 $78 = HEAP32[$zCsr>>2]|0;
 $79 = (($78) + ($77)|0);
 HEAP32[$zCsr>>2] = $79;
 $80 = HEAP32[$0>>2]|0;
 $81 = (($80) + 88|0);
 $82 = HEAP16[$81>>1]|0;
 $83 = $82 & -33;
 HEAP16[$81>>1] = $83;
 while(1) {
  HEAP32[$nByte>>2] = 0;
  $84 = HEAP32[$0>>2]|0;
  $85 = (($84) + 8|0);
  $86 = HEAP32[$85>>2]|0;
  $87 = HEAP32[$nMem>>2]|0;
  $88 = ($87*40)|0;
  $89 = HEAP32[$zEnd>>2]|0;
  $90 = (_allocSpace($86,$88,$zCsr,$89,$nByte)|0);
  $91 = HEAP32[$0>>2]|0;
  $92 = (($91) + 8|0);
  HEAP32[$92>>2] = $90;
  $93 = HEAP32[$0>>2]|0;
  $94 = (($93) + 60|0);
  $95 = HEAP32[$94>>2]|0;
  $96 = HEAP32[$nVar>>2]|0;
  $97 = ($96*40)|0;
  $98 = HEAP32[$zEnd>>2]|0;
  $99 = (_allocSpace($95,$97,$zCsr,$98,$nByte)|0);
  $100 = HEAP32[$0>>2]|0;
  $101 = (($100) + 60|0);
  HEAP32[$101>>2] = $99;
  $102 = HEAP32[$0>>2]|0;
  $103 = (($102) + 12|0);
  $104 = HEAP32[$103>>2]|0;
  $105 = HEAP32[$nArg>>2]|0;
  $106 = $105<<2;
  $107 = HEAP32[$zEnd>>2]|0;
  $108 = (_allocSpace($104,$106,$zCsr,$107,$nByte)|0);
  $109 = HEAP32[$0>>2]|0;
  $110 = (($109) + 12|0);
  HEAP32[$110>>2] = $108;
  $111 = HEAP32[$0>>2]|0;
  $112 = (($111) + 64|0);
  $113 = HEAP32[$112>>2]|0;
  $114 = HEAP32[$nVar>>2]|0;
  $115 = $114<<2;
  $116 = HEAP32[$zEnd>>2]|0;
  $117 = (_allocSpace($113,$115,$zCsr,$116,$nByte)|0);
  $118 = HEAP32[$0>>2]|0;
  $119 = (($118) + 64|0);
  HEAP32[$119>>2] = $117;
  $120 = HEAP32[$0>>2]|0;
  $121 = (($120) + 56|0);
  $122 = HEAP32[$121>>2]|0;
  $123 = HEAP32[$nCursor>>2]|0;
  $124 = $123<<2;
  $125 = HEAP32[$zEnd>>2]|0;
  $126 = (_allocSpace($122,$124,$zCsr,$125,$nByte)|0);
  $127 = HEAP32[$0>>2]|0;
  $128 = (($127) + 56|0);
  HEAP32[$128>>2] = $126;
  $129 = HEAP32[$0>>2]|0;
  $130 = (($129) + 200|0);
  $131 = HEAP32[$130>>2]|0;
  $132 = HEAP32[$nOnce>>2]|0;
  $133 = HEAP32[$zEnd>>2]|0;
  $134 = (_allocSpace($131,$132,$zCsr,$133,$nByte)|0);
  $135 = HEAP32[$0>>2]|0;
  $136 = (($135) + 200|0);
  HEAP32[$136>>2] = $134;
  $137 = HEAP32[$nByte>>2]|0;
  $138 = ($137|0)!=(0);
  if ($138) {
   $139 = HEAP32[$db>>2]|0;
   $140 = HEAP32[$nByte>>2]|0;
   $141 = ($140|0)<(0);
   $142 = $141 << 31 >> 31;
   $143 = (_sqlite3DbMallocZero($139,$140,$142)|0);
   $144 = HEAP32[$0>>2]|0;
   $145 = (($144) + 172|0);
   HEAP32[$145>>2] = $143;
  }
  $146 = HEAP32[$0>>2]|0;
  $147 = (($146) + 172|0);
  $148 = HEAP32[$147>>2]|0;
  HEAP32[$zCsr>>2] = $148;
  $149 = HEAP32[$nByte>>2]|0;
  $150 = HEAP32[$zCsr>>2]|0;
  $151 = (($150) + ($149)|0);
  HEAP32[$zEnd>>2] = $151;
  $152 = HEAP32[$nByte>>2]|0;
  $153 = ($152|0)!=(0);
  if (!($153)) {
   break;
  }
  $154 = HEAP32[$db>>2]|0;
  $155 = (($154) + 64|0);
  $156 = HEAP8[$155>>0]|0;
  $157 = ($156<<24>>24)!=(0);
  $158 = $157 ^ 1;
  if (!($158)) {
   break;
  }
 }
 $159 = HEAP32[$nCursor>>2]|0;
 $160 = HEAP32[$0>>2]|0;
 $161 = (($160) + 36|0);
 HEAP32[$161>>2] = $159;
 $162 = HEAP32[$nOnce>>2]|0;
 $163 = HEAP32[$0>>2]|0;
 $164 = (($163) + 196|0);
 HEAP32[$164>>2] = $162;
 $165 = HEAP32[$0>>2]|0;
 $166 = (($165) + 60|0);
 $167 = HEAP32[$166>>2]|0;
 $168 = ($167|0)!=(0|0);
 L14: do {
  if ($168) {
   $169 = HEAP32[$nVar>>2]|0;
   $170 = $169&65535;
   $171 = HEAP32[$0>>2]|0;
   $172 = (($171) + 68|0);
   HEAP16[$172>>1] = $170;
   HEAP32[$n>>2] = 0;
   while(1) {
    $173 = HEAP32[$n>>2]|0;
    $174 = HEAP32[$nVar>>2]|0;
    $175 = ($173|0)<($174|0);
    if (!($175)) {
     break L14;
    }
    $176 = HEAP32[$n>>2]|0;
    $177 = HEAP32[$0>>2]|0;
    $178 = (($177) + 60|0);
    $179 = HEAP32[$178>>2]|0;
    $180 = (($179) + (($176*40)|0)|0);
    $181 = (($180) + 8|0);
    HEAP16[$181>>1] = 1;
    $182 = HEAP32[$db>>2]|0;
    $183 = HEAP32[$n>>2]|0;
    $184 = HEAP32[$0>>2]|0;
    $185 = (($184) + 60|0);
    $186 = HEAP32[$185>>2]|0;
    $187 = (($186) + (($183*40)|0)|0);
    $188 = (($187) + 32|0);
    HEAP32[$188>>2] = $182;
    $189 = HEAP32[$n>>2]|0;
    $190 = (($189) + 1)|0;
    HEAP32[$n>>2] = $190;
   }
  }
 } while(0);
 $191 = HEAP32[$0>>2]|0;
 $192 = (($191) + 64|0);
 $193 = HEAP32[$192>>2]|0;
 $194 = ($193|0)!=(0|0);
 if ($194) {
  $195 = HEAP32[$1>>2]|0;
  $196 = (($195) + 444|0);
  $197 = HEAP32[$196>>2]|0;
  $198 = $197&65535;
  $199 = HEAP32[$0>>2]|0;
  $200 = (($199) + 70|0);
  HEAP16[$200>>1] = $198;
  $201 = HEAP32[$0>>2]|0;
  $202 = (($201) + 64|0);
  $203 = HEAP32[$202>>2]|0;
  $204 = HEAP32[$1>>2]|0;
  $205 = (($204) + 472|0);
  $206 = HEAP32[$205>>2]|0;
  $207 = HEAP32[$0>>2]|0;
  $208 = (($207) + 70|0);
  $209 = HEAP16[$208>>1]|0;
  $210 = $209 << 16 >> 16;
  $211 = $210<<2;
  _memcpy(($203|0),($206|0),($211|0))|0;
  $212 = HEAP32[$1>>2]|0;
  $213 = (($212) + 472|0);
  $214 = HEAP32[$213>>2]|0;
  $215 = HEAP32[$1>>2]|0;
  $216 = (($215) + 444|0);
  $217 = HEAP32[$216>>2]|0;
  $218 = $217<<2;
  _memset(($214|0),0,($218|0))|0;
 }
 $219 = HEAP32[$0>>2]|0;
 $220 = (($219) + 8|0);
 $221 = HEAP32[$220>>2]|0;
 $222 = ($221|0)!=(0|0);
 if (!($222)) {
  $248 = HEAP32[$1>>2]|0;
  $249 = (($248) + 450|0);
  $250 = HEAP8[$249>>0]|0;
  $251 = $250&255;
  $252 = HEAP32[$0>>2]|0;
  $253 = (($252) + 88|0);
  $254 = $251&65535;
  $255 = HEAP16[$253>>1]|0;
  $256 = $254 & 3;
  $257 = $255 & -4;
  $258 = $257 | $256;
  HEAP16[$253>>1] = $258;
  $259 = HEAP32[$0>>2]|0;
  _sqlite3VdbeRewind($259);
  STACKTOP = sp;return;
 }
 $223 = HEAP32[$0>>2]|0;
 $224 = (($223) + 8|0);
 $225 = HEAP32[$224>>2]|0;
 $226 = (($225) + -40|0);
 HEAP32[$224>>2] = $226;
 $227 = HEAP32[$nMem>>2]|0;
 $228 = HEAP32[$0>>2]|0;
 $229 = (($228) + 28|0);
 HEAP32[$229>>2] = $227;
 HEAP32[$n>>2] = 1;
 while(1) {
  $230 = HEAP32[$n>>2]|0;
  $231 = HEAP32[$nMem>>2]|0;
  $232 = ($230|0)<=($231|0);
  if (!($232)) {
   break;
  }
  $233 = HEAP32[$n>>2]|0;
  $234 = HEAP32[$0>>2]|0;
  $235 = (($234) + 8|0);
  $236 = HEAP32[$235>>2]|0;
  $237 = (($236) + (($233*40)|0)|0);
  $238 = (($237) + 8|0);
  HEAP16[$238>>1] = 128;
  $239 = HEAP32[$db>>2]|0;
  $240 = HEAP32[$n>>2]|0;
  $241 = HEAP32[$0>>2]|0;
  $242 = (($241) + 8|0);
  $243 = HEAP32[$242>>2]|0;
  $244 = (($243) + (($240*40)|0)|0);
  $245 = (($244) + 32|0);
  HEAP32[$245>>2] = $239;
  $246 = HEAP32[$n>>2]|0;
  $247 = (($246) + 1)|0;
  HEAP32[$n>>2] = $247;
 }
 $248 = HEAP32[$1>>2]|0;
 $249 = (($248) + 450|0);
 $250 = HEAP8[$249>>0]|0;
 $251 = $250&255;
 $252 = HEAP32[$0>>2]|0;
 $253 = (($252) + 88|0);
 $254 = $251&65535;
 $255 = HEAP16[$253>>1]|0;
 $256 = $254 & 3;
 $257 = $255 & -4;
 $258 = $257 | $256;
 HEAP16[$253>>1] = $258;
 $259 = HEAP32[$0>>2]|0;
 _sqlite3VdbeRewind($259);
 STACKTOP = sp;return;
}
function _sqlite3ParserReset($pParse) {
 $pParse = $pParse|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $db = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $db = sp;
 HEAP32[$0>>2] = $pParse;
 $1 = HEAP32[$0>>2]|0;
 $2 = ($1|0)!=(0|0);
 if (!($2)) {
  STACKTOP = sp;return;
 }
 $3 = HEAP32[$0>>2]|0;
 $4 = HEAP32[$3>>2]|0;
 HEAP32[$db>>2] = $4;
 $5 = HEAP32[$db>>2]|0;
 $6 = HEAP32[$0>>2]|0;
 $7 = (($6) + 112|0);
 $8 = HEAP32[$7>>2]|0;
 _sqlite3DbFree($5,$8);
 $9 = HEAP32[$db>>2]|0;
 $10 = HEAP32[$0>>2]|0;
 $11 = (($10) + 316|0);
 $12 = HEAP32[$11>>2]|0;
 _sqlite3ExprListDelete($9,$12);
 STACKTOP = sp;return;
}
function _sqlite3BtreeData($pCur,$offset,$amt,$pBuf) {
 $pCur = $pCur|0;
 $offset = $offset|0;
 $amt = $amt|0;
 $pBuf = $pBuf|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 20|0;
 $1 = sp + 16|0;
 $2 = sp + 12|0;
 $3 = sp + 8|0;
 $4 = sp + 4|0;
 $rc = sp;
 HEAP32[$1>>2] = $pCur;
 HEAP32[$2>>2] = $offset;
 HEAP32[$3>>2] = $amt;
 HEAP32[$4>>2] = $pBuf;
 $5 = HEAP32[$1>>2]|0;
 $6 = (($5) + 73|0);
 $7 = HEAP8[$6>>0]|0;
 $8 = $7&255;
 $9 = ($8|0)==(0);
 if ($9) {
  HEAP32[$0>>2] = 4;
  $26 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($26|0);
 }
 $10 = HEAP32[$1>>2]|0;
 $11 = (($10) + 73|0);
 $12 = HEAP8[$11>>0]|0;
 $13 = $12&255;
 $14 = ($13|0)>=(3);
 if ($14) {
  $15 = HEAP32[$1>>2]|0;
  $16 = (_btreeRestoreCursorPosition($15)|0);
  $17 = $16;
 } else {
  $17 = 0;
 }
 HEAP32[$rc>>2] = $17;
 $18 = HEAP32[$rc>>2]|0;
 $19 = ($18|0)==(0);
 if ($19) {
  $20 = HEAP32[$1>>2]|0;
  $21 = HEAP32[$2>>2]|0;
  $22 = HEAP32[$3>>2]|0;
  $23 = HEAP32[$4>>2]|0;
  $24 = (_accessPayload($20,$21,$22,$23,0)|0);
  HEAP32[$rc>>2] = $24;
 }
 $25 = HEAP32[$rc>>2]|0;
 HEAP32[$0>>2] = $25;
 $26 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($26|0);
}
function _sqlite3ExpirePreparedStatements($db) {
 $db = $db|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $p = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $p = sp;
 HEAP32[$0>>2] = $db;
 $1 = HEAP32[$0>>2]|0;
 $2 = (($1) + 4|0);
 $3 = HEAP32[$2>>2]|0;
 HEAP32[$p>>2] = $3;
 while(1) {
  $4 = HEAP32[$p>>2]|0;
  $5 = ($4|0)!=(0|0);
  if (!($5)) {
   break;
  }
  $6 = HEAP32[$p>>2]|0;
  $7 = (($6) + 88|0);
  $8 = HEAP16[$7>>1]|0;
  $9 = $8 & -33;
  $10 = $9 | 32;
  HEAP16[$7>>1] = $10;
  $11 = HEAP32[$p>>2]|0;
  $12 = (($11) + 52|0);
  $13 = HEAP32[$12>>2]|0;
  HEAP32[$p>>2] = $13;
 }
 STACKTOP = sp;return;
}
function _sqlite3_strglob($zGlobPattern,$zString) {
 $zGlobPattern = $zGlobPattern|0;
 $zString = $zString|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $1 = sp;
 HEAP32[$0>>2] = $zGlobPattern;
 HEAP32[$1>>2] = $zString;
 $2 = HEAP32[$0>>2]|0;
 $3 = HEAP32[$1>>2]|0;
 $4 = (_patternCompare($2,$3,1208,0)|0);
 $5 = ($4|0)==(0);
 $6 = $5&1;
 STACKTOP = sp;return ($6|0);
}
function _patternCompare($zPattern,$zString,$pInfo,$esc) {
 $zPattern = $zPattern|0;
 $zString = $zString|0;
 $pInfo = $pInfo|0;
 $esc = $esc|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0;
 var $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0, $188 = 0;
 var $189 = 0, $19 = 0, $190 = 0, $191 = 0, $192 = 0, $193 = 0, $194 = 0, $195 = 0, $196 = 0, $197 = 0, $198 = 0, $199 = 0, $2 = 0, $20 = 0, $200 = 0, $201 = 0, $202 = 0, $203 = 0, $204 = 0, $205 = 0;
 var $206 = 0, $207 = 0, $208 = 0, $209 = 0, $21 = 0, $210 = 0, $211 = 0, $212 = 0, $213 = 0, $214 = 0, $215 = 0, $216 = 0, $217 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0;
 var $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0;
 var $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0;
 var $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0;
 var $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $c = 0, $c2 = 0;
 var $cx = 0, $invert = 0, $matchAll = 0, $matchOne = 0, $matchOther = 0, $noCase = 0, $or$cond = 0, $or$cond3 = 0, $or$cond5 = 0, $or$cond7 = 0, $prior_c = 0, $seen = 0, $zEscaped = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 64|0;
 $0 = sp + 56|0;
 $1 = sp + 52|0;
 $2 = sp + 48|0;
 $3 = sp + 44|0;
 $4 = sp + 40|0;
 $c = sp + 36|0;
 $c2 = sp + 32|0;
 $matchOne = sp + 28|0;
 $matchAll = sp + 24|0;
 $matchOther = sp + 20|0;
 $noCase = sp + 60|0;
 $zEscaped = sp + 16|0;
 $cx = sp + 12|0;
 $prior_c = sp + 8|0;
 $seen = sp + 4|0;
 $invert = sp;
 HEAP32[$1>>2] = $zPattern;
 HEAP32[$2>>2] = $zString;
 HEAP32[$3>>2] = $pInfo;
 HEAP32[$4>>2] = $esc;
 $5 = HEAP32[$3>>2]|0;
 $6 = (($5) + 1|0);
 $7 = HEAP8[$6>>0]|0;
 $8 = $7&255;
 HEAP32[$matchOne>>2] = $8;
 $9 = HEAP32[$3>>2]|0;
 $10 = HEAP8[$9>>0]|0;
 $11 = $10&255;
 HEAP32[$matchAll>>2] = $11;
 $12 = HEAP32[$3>>2]|0;
 $13 = (($12) + 3|0);
 $14 = HEAP8[$13>>0]|0;
 HEAP8[$noCase>>0] = $14;
 HEAP32[$zEscaped>>2] = 0;
 $15 = HEAP32[$4>>2]|0;
 $16 = ($15|0)!=(0);
 if ($16) {
  $17 = HEAP32[$4>>2]|0;
  $22 = $17;
 } else {
  $18 = HEAP32[$3>>2]|0;
  $19 = (($18) + 2|0);
  $20 = HEAP8[$19>>0]|0;
  $21 = $20&255;
  $22 = $21;
 }
 HEAP32[$matchOther>>2] = $22;
 L5: while(1) {
  $23 = (_sqlite3Utf8Read($1)|0);
  HEAP32[$c>>2] = $23;
  $24 = ($23|0)!=(0);
  if (!($24)) {
   label = 76;
   break;
  }
  $25 = HEAP32[$c>>2]|0;
  $26 = HEAP32[$matchAll>>2]|0;
  $27 = ($25|0)==($26|0);
  if ($27) {
   label = 7;
   break;
  }
  $123 = HEAP32[$c>>2]|0;
  $124 = HEAP32[$matchOther>>2]|0;
  $125 = ($123|0)==($124|0);
  do {
   if ($125) {
    $126 = HEAP32[$4>>2]|0;
    $127 = ($126|0)!=(0);
    if ($127) {
     $128 = (_sqlite3Utf8Read($1)|0);
     HEAP32[$c>>2] = $128;
     $129 = HEAP32[$c>>2]|0;
     $130 = ($129|0)==(0);
     if ($130) {
      label = 43;
      break L5;
     }
     $131 = HEAP32[$1>>2]|0;
     HEAP32[$zEscaped>>2] = $131;
     break;
    }
    HEAP32[$prior_c>>2] = 0;
    HEAP32[$seen>>2] = 0;
    HEAP32[$invert>>2] = 0;
    $132 = (_sqlite3Utf8Read($2)|0);
    HEAP32[$c>>2] = $132;
    $133 = HEAP32[$c>>2]|0;
    $134 = ($133|0)==(0);
    if ($134) {
     label = 46;
     break L5;
    }
    $135 = (_sqlite3Utf8Read($1)|0);
    HEAP32[$c2>>2] = $135;
    $136 = HEAP32[$c2>>2]|0;
    $137 = ($136|0)==(94);
    if ($137) {
     HEAP32[$invert>>2] = 1;
     $138 = (_sqlite3Utf8Read($1)|0);
     HEAP32[$c2>>2] = $138;
    }
    $139 = HEAP32[$c2>>2]|0;
    $140 = ($139|0)==(93);
    if ($140) {
     $141 = HEAP32[$c>>2]|0;
     $142 = ($141|0)==(93);
     if ($142) {
      HEAP32[$seen>>2] = 1;
     }
     $143 = (_sqlite3Utf8Read($1)|0);
     HEAP32[$c2>>2] = $143;
    }
    while(1) {
     $144 = HEAP32[$c2>>2]|0;
     $145 = ($144|0)!=(0);
     if ($145) {
      $146 = HEAP32[$c2>>2]|0;
      $147 = ($146|0)!=(93);
      $217 = $147;
     } else {
      $217 = 0;
     }
     $148 = HEAP32[$c2>>2]|0;
     if (!($217)) {
      break;
     }
     $149 = ($148|0)==(45);
     if ($149) {
      $150 = HEAP32[$1>>2]|0;
      $151 = HEAP8[$150>>0]|0;
      $152 = $151&255;
      $153 = ($152|0)!=(93);
      if ($153) {
       $154 = HEAP32[$1>>2]|0;
       $155 = HEAP8[$154>>0]|0;
       $156 = $155&255;
       $157 = ($156|0)!=(0);
       $158 = HEAP32[$prior_c>>2]|0;
       $159 = ($158>>>0)>(0);
       $or$cond = $157 & $159;
       if ($or$cond) {
        $160 = (_sqlite3Utf8Read($1)|0);
        HEAP32[$c2>>2] = $160;
        $161 = HEAP32[$c>>2]|0;
        $162 = HEAP32[$prior_c>>2]|0;
        $163 = ($161>>>0)>=($162>>>0);
        if ($163) {
         $164 = HEAP32[$c>>2]|0;
         $165 = HEAP32[$c2>>2]|0;
         $166 = ($164>>>0)<=($165>>>0);
         if ($166) {
          HEAP32[$seen>>2] = 1;
         }
        }
        HEAP32[$prior_c>>2] = 0;
       } else {
        label = 63;
       }
      } else {
       label = 63;
      }
     } else {
      label = 63;
     }
     if ((label|0) == 63) {
      label = 0;
      $167 = HEAP32[$c>>2]|0;
      $168 = HEAP32[$c2>>2]|0;
      $169 = ($167|0)==($168|0);
      if ($169) {
       HEAP32[$seen>>2] = 1;
      }
      $170 = HEAP32[$c2>>2]|0;
      HEAP32[$prior_c>>2] = $170;
     }
     $171 = (_sqlite3Utf8Read($1)|0);
     HEAP32[$c2>>2] = $171;
    }
    $172 = ($148|0)==(0);
    if ($172) {
     label = 69;
     break L5;
    }
    $173 = HEAP32[$seen>>2]|0;
    $174 = HEAP32[$invert>>2]|0;
    $175 = $173 ^ $174;
    $176 = ($175|0)==(0);
    if ($176) {
     label = 69;
     break L5;
    } else {
     continue L5;
    }
   }
  } while(0);
  $177 = (_sqlite3Utf8Read($2)|0);
  HEAP32[$c2>>2] = $177;
  $178 = HEAP32[$c>>2]|0;
  $179 = HEAP32[$c2>>2]|0;
  $180 = ($178|0)==($179|0);
  if ($180) {
   continue;
  }
  $181 = HEAP8[$noCase>>0]|0;
  $182 = $181&255;
  $183 = ($182|0)!=(0);
  $184 = HEAP32[$c>>2]|0;
  $185 = ($184>>>0)<(128);
  $or$cond3 = $183 & $185;
  $186 = HEAP32[$c2>>2]|0;
  $187 = ($186>>>0)<(128);
  $or$cond5 = $or$cond3 & $187;
  if ($or$cond5) {
   $188 = HEAP32[$c>>2]|0;
   $189 = $188&255;
   $190 = $189&255;
   $191 = (440 + ($190)|0);
   $192 = HEAP8[$191>>0]|0;
   $193 = $192&255;
   $194 = HEAP32[$c2>>2]|0;
   $195 = $194&255;
   $196 = $195&255;
   $197 = (440 + ($196)|0);
   $198 = HEAP8[$197>>0]|0;
   $199 = $198&255;
   $200 = ($193|0)==($199|0);
   if ($200) {
    continue;
   }
  }
  $201 = HEAP32[$c>>2]|0;
  $202 = HEAP32[$matchOne>>2]|0;
  $203 = ($201|0)==($202|0);
  if (!($203)) {
   label = 75;
   break;
  }
  $204 = HEAP32[$1>>2]|0;
  $205 = HEAP32[$zEscaped>>2]|0;
  $206 = ($204|0)!=($205|0);
  $207 = HEAP32[$c2>>2]|0;
  $208 = ($207|0)!=(0);
  $or$cond7 = $206 & $208;
  if (!($or$cond7)) {
   label = 75;
   break;
  }
 }
 if ((label|0) == 7) {
  while(1) {
   label = 0;
   $28 = (_sqlite3Utf8Read($1)|0);
   HEAP32[$c>>2] = $28;
   $29 = HEAP32[$matchAll>>2]|0;
   $30 = ($28|0)==($29|0);
   if ($30) {
    $215 = 1;
   } else {
    $31 = HEAP32[$c>>2]|0;
    $32 = HEAP32[$matchOne>>2]|0;
    $33 = ($31|0)==($32|0);
    $215 = $33;
   }
   $34 = HEAP32[$c>>2]|0;
   if (!($215)) {
    break;
   }
   $35 = HEAP32[$matchOne>>2]|0;
   $36 = ($34|0)==($35|0);
   if (!($36)) {
    label = 7;
    continue;
   }
   $37 = (_sqlite3Utf8Read($2)|0);
   $38 = ($37|0)==(0);
   if ($38) {
    label = 12;
    break;
   } else {
    label = 7;
   }
  }
  if ((label|0) == 12) {
   HEAP32[$0>>2] = 0;
   $214 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($214|0);
  }
  $39 = ($34|0)==(0);
  if ($39) {
   HEAP32[$0>>2] = 1;
   $214 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($214|0);
  }
  $40 = HEAP32[$c>>2]|0;
  $41 = HEAP32[$matchOther>>2]|0;
  $42 = ($40|0)==($41|0);
  do {
   if ($42) {
    $43 = HEAP32[$4>>2]|0;
    $44 = ($43|0)!=(0);
    if ($44) {
     $45 = (_sqlite3Utf8Read($1)|0);
     HEAP32[$c>>2] = $45;
     $46 = HEAP32[$c>>2]|0;
     $47 = ($46|0)==(0);
     if (!($47)) {
      break;
     }
     HEAP32[$0>>2] = 0;
     $214 = HEAP32[$0>>2]|0;
     STACKTOP = sp;return ($214|0);
    }
    L82: while(1) {
     $48 = HEAP32[$2>>2]|0;
     $49 = HEAP8[$48>>0]|0;
     $50 = $49&255;
     $51 = ($50|0)!=(0);
     if ($51) {
      $52 = HEAP32[$1>>2]|0;
      $53 = (($52) + -1|0);
      $54 = HEAP32[$2>>2]|0;
      $55 = HEAP32[$3>>2]|0;
      $56 = HEAP32[$4>>2]|0;
      $57 = (_patternCompare($53,$54,$55,$56)|0);
      $58 = ($57|0)==(0);
      $216 = $58;
     } else {
      $216 = 0;
     }
     $59 = HEAP32[$2>>2]|0;
     if (!($216)) {
      break;
     }
     $60 = (($59) + 1|0);
     HEAP32[$2>>2] = $60;
     $61 = HEAP8[$59>>0]|0;
     $62 = $61&255;
     $63 = ($62|0)>=(192);
     if (!($63)) {
      continue;
     }
     while(1) {
      $64 = HEAP32[$2>>2]|0;
      $65 = HEAP8[$64>>0]|0;
      $66 = $65&255;
      $67 = $66 & 192;
      $68 = ($67|0)==(128);
      if (!($68)) {
       continue L82;
      }
      $69 = HEAP32[$2>>2]|0;
      $70 = (($69) + 1|0);
      HEAP32[$2>>2] = $70;
     }
    }
    $71 = HEAP8[$59>>0]|0;
    $72 = $71&255;
    $73 = ($72|0)!=(0);
    $74 = $73&1;
    HEAP32[$0>>2] = $74;
    $214 = HEAP32[$0>>2]|0;
    STACKTOP = sp;return ($214|0);
   }
  } while(0);
  $75 = HEAP32[$c>>2]|0;
  $76 = ($75>>>0)<=(128);
  L94: do {
   if ($76) {
    $77 = HEAP8[$noCase>>0]|0;
    $78 = ($77<<24>>24)!=(0);
    $79 = HEAP32[$c>>2]|0;
    if ($78) {
     $80 = HEAP32[$c>>2]|0;
     $81 = $80&255;
     $82 = $81&255;
     $83 = (1224 + ($82)|0);
     $84 = HEAP8[$83>>0]|0;
     $85 = $84&255;
     $86 = $85 & 32;
     $87 = $86 ^ -1;
     $88 = $79 & $87;
     HEAP32[$cx>>2] = $88;
     $89 = HEAP32[$c>>2]|0;
     $90 = $89&255;
     $91 = $90&255;
     $92 = (440 + ($91)|0);
     $93 = HEAP8[$92>>0]|0;
     $94 = $93&255;
     HEAP32[$c>>2] = $94;
    } else {
     HEAP32[$cx>>2] = $79;
    }
    while(1) {
     $95 = HEAP32[$2>>2]|0;
     $96 = (($95) + 1|0);
     HEAP32[$2>>2] = $96;
     $97 = HEAP8[$95>>0]|0;
     $98 = $97&255;
     HEAP32[$c2>>2] = $98;
     $99 = ($98|0)!=(0);
     if (!($99)) {
      break L94;
     }
     $100 = HEAP32[$c2>>2]|0;
     $101 = HEAP32[$c>>2]|0;
     $102 = ($100|0)!=($101|0);
     if ($102) {
      $103 = HEAP32[$c2>>2]|0;
      $104 = HEAP32[$cx>>2]|0;
      $105 = ($103|0)!=($104|0);
      if ($105) {
       continue;
      }
     }
     $106 = HEAP32[$1>>2]|0;
     $107 = HEAP32[$2>>2]|0;
     $108 = HEAP32[$3>>2]|0;
     $109 = HEAP32[$4>>2]|0;
     $110 = (_patternCompare($106,$107,$108,$109)|0);
     $111 = ($110|0)!=(0);
     if ($111) {
      break;
     }
    }
    HEAP32[$0>>2] = 1;
    $214 = HEAP32[$0>>2]|0;
    STACKTOP = sp;return ($214|0);
   } else {
    while(1) {
     $112 = (_sqlite3Utf8Read($2)|0);
     HEAP32[$c2>>2] = $112;
     $113 = ($112|0)!=(0);
     if (!($113)) {
      break L94;
     }
     $114 = HEAP32[$c2>>2]|0;
     $115 = HEAP32[$c>>2]|0;
     $116 = ($114|0)!=($115|0);
     if ($116) {
      continue;
     }
     $117 = HEAP32[$1>>2]|0;
     $118 = HEAP32[$2>>2]|0;
     $119 = HEAP32[$3>>2]|0;
     $120 = HEAP32[$4>>2]|0;
     $121 = (_patternCompare($117,$118,$119,$120)|0);
     $122 = ($121|0)!=(0);
     if ($122) {
      break;
     }
    }
    HEAP32[$0>>2] = 1;
    $214 = HEAP32[$0>>2]|0;
    STACKTOP = sp;return ($214|0);
   }
  } while(0);
  HEAP32[$0>>2] = 0;
  $214 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($214|0);
 }
 else if ((label|0) == 43) {
  HEAP32[$0>>2] = 0;
  $214 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($214|0);
 }
 else if ((label|0) == 46) {
  HEAP32[$0>>2] = 0;
  $214 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($214|0);
 }
 else if ((label|0) == 69) {
  HEAP32[$0>>2] = 0;
  $214 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($214|0);
 }
 else if ((label|0) == 75) {
  HEAP32[$0>>2] = 0;
  $214 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($214|0);
 }
 else if ((label|0) == 76) {
  $209 = HEAP32[$2>>2]|0;
  $210 = HEAP8[$209>>0]|0;
  $211 = $210&255;
  $212 = ($211|0)==(0);
  $213 = $212&1;
  HEAP32[$0>>2] = $213;
  $214 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($214|0);
 }
 return 0|0;
}
function _sqlite3_exec($db,$zSql,$xCallback,$pArg,$pzErrMsg) {
 $db = $db|0;
 $zSql = $zSql|0;
 $xCallback = $xCallback|0;
 $pArg = $pArg|0;
 $pzErrMsg = $pzErrMsg|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0;
 var $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0;
 var $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0;
 var $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0;
 var $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $azCols = 0, $azVals = 0, $callbackIsInit = 0, $i = 0, $nCol = 0, $nErrMsg = 0, $or$cond = 0;
 var $or$cond3 = 0, $pStmt = 0, $rc = 0, $zLeftover = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 64|0;
 $0 = sp + 56|0;
 $1 = sp + 52|0;
 $2 = sp + 48|0;
 $3 = sp + 44|0;
 $4 = sp + 40|0;
 $5 = sp + 36|0;
 $rc = sp + 32|0;
 $zLeftover = sp + 28|0;
 $pStmt = sp + 24|0;
 $azCols = sp + 20|0;
 $callbackIsInit = sp + 16|0;
 $nCol = sp + 12|0;
 $azVals = sp + 8|0;
 $i = sp + 4|0;
 $nErrMsg = sp;
 HEAP32[$1>>2] = $db;
 HEAP32[$2>>2] = $zSql;
 HEAP32[$3>>2] = $xCallback;
 HEAP32[$4>>2] = $pArg;
 HEAP32[$5>>2] = $pzErrMsg;
 HEAP32[$rc>>2] = 0;
 HEAP32[$pStmt>>2] = 0;
 HEAP32[$azCols>>2] = 0;
 $6 = HEAP32[$1>>2]|0;
 $7 = (_sqlite3SafetyCheckOk($6)|0);
 $8 = ($7|0)!=(0);
 if (!($8)) {
  $9 = (_sqlite3MisuseError(99392)|0);
  HEAP32[$0>>2] = $9;
  $152 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($152|0);
 }
 $10 = HEAP32[$2>>2]|0;
 $11 = ($10|0)==(0|0);
 if ($11) {
  HEAP32[$2>>2] = 1216;
 }
 $12 = HEAP32[$1>>2]|0;
 _sqlite3Error($12,0);
 L8: while(1) {
  $13 = HEAP32[$rc>>2]|0;
  $14 = ($13|0)==(0);
  if (!($14)) {
   break;
  }
  $15 = HEAP32[$2>>2]|0;
  $16 = HEAP8[$15>>0]|0;
  $17 = $16 << 24 >> 24;
  $18 = ($17|0)!=(0);
  if (!($18)) {
   break;
  }
  HEAP32[$azVals>>2] = 0;
  HEAP32[$pStmt>>2] = 0;
  $19 = HEAP32[$1>>2]|0;
  $20 = HEAP32[$2>>2]|0;
  $21 = (_sqlite3_prepare_v2($19,$20,-1,$pStmt,$zLeftover)|0);
  HEAP32[$rc>>2] = $21;
  $22 = HEAP32[$rc>>2]|0;
  $23 = ($22|0)!=(0);
  if ($23) {
   continue;
  }
  $24 = HEAP32[$pStmt>>2]|0;
  $25 = ($24|0)!=(0|0);
  if (!($25)) {
   $26 = HEAP32[$zLeftover>>2]|0;
   HEAP32[$2>>2] = $26;
   continue;
  }
  HEAP32[$callbackIsInit>>2] = 0;
  $27 = HEAP32[$pStmt>>2]|0;
  $28 = (_sqlite3_column_count($27)|0);
  HEAP32[$nCol>>2] = $28;
  while(1) {
   $29 = HEAP32[$pStmt>>2]|0;
   $30 = (_sqlite3_step($29)|0);
   HEAP32[$rc>>2] = $30;
   $31 = HEAP32[$3>>2]|0;
   $32 = ($31|0)!=(0|0);
   do {
    if ($32) {
     $33 = HEAP32[$rc>>2]|0;
     $34 = (100)==($33|0);
     if (!($34)) {
      $35 = HEAP32[$rc>>2]|0;
      $36 = (101)!=($35|0);
      $37 = HEAP32[$callbackIsInit>>2]|0;
      $38 = ($37|0)!=(0);
      $or$cond = $36 | $38;
      if ($or$cond) {
       break;
      }
      $39 = HEAP32[$1>>2]|0;
      $40 = (($39) + 24|0);
      $41 = HEAP32[$40>>2]|0;
      $42 = $41 & 256;
      $43 = ($42|0)!=(0);
      if (!($43)) {
       break;
      }
     }
     $44 = HEAP32[$callbackIsInit>>2]|0;
     $45 = ($44|0)!=(0);
     if (!($45)) {
      $46 = HEAP32[$1>>2]|0;
      $47 = HEAP32[$nCol>>2]|0;
      $48 = $47<<1;
      $49 = $48<<2;
      $50 = (($49) + 1)|0;
      $51 = (_sqlite3DbMallocZero($46,$50,0)|0);
      HEAP32[$azCols>>2] = $51;
      $52 = HEAP32[$azCols>>2]|0;
      $53 = ($52|0)==(0|0);
      if ($53) {
       break L8;
      }
      HEAP32[$i>>2] = 0;
      while(1) {
       $54 = HEAP32[$i>>2]|0;
       $55 = HEAP32[$nCol>>2]|0;
       $56 = ($54|0)<($55|0);
       if (!($56)) {
        break;
       }
       $57 = HEAP32[$pStmt>>2]|0;
       $58 = HEAP32[$i>>2]|0;
       $59 = (_sqlite3_column_name($57,$58)|0);
       $60 = HEAP32[$i>>2]|0;
       $61 = HEAP32[$azCols>>2]|0;
       $62 = (($61) + ($60<<2)|0);
       HEAP32[$62>>2] = $59;
       $63 = HEAP32[$i>>2]|0;
       $64 = (($63) + 1)|0;
       HEAP32[$i>>2] = $64;
      }
      HEAP32[$callbackIsInit>>2] = 1;
     }
     $65 = HEAP32[$rc>>2]|0;
     $66 = ($65|0)==(100);
     L32: do {
      if ($66) {
       $67 = HEAP32[$nCol>>2]|0;
       $68 = HEAP32[$azCols>>2]|0;
       $69 = (($68) + ($67<<2)|0);
       HEAP32[$azVals>>2] = $69;
       HEAP32[$i>>2] = 0;
       while(1) {
        $70 = HEAP32[$i>>2]|0;
        $71 = HEAP32[$nCol>>2]|0;
        $72 = ($70|0)<($71|0);
        if (!($72)) {
         break L32;
        }
        $73 = HEAP32[$pStmt>>2]|0;
        $74 = HEAP32[$i>>2]|0;
        $75 = (_sqlite3_column_text($73,$74)|0);
        $76 = HEAP32[$i>>2]|0;
        $77 = HEAP32[$azVals>>2]|0;
        $78 = (($77) + ($76<<2)|0);
        HEAP32[$78>>2] = $75;
        $79 = HEAP32[$i>>2]|0;
        $80 = HEAP32[$azVals>>2]|0;
        $81 = (($80) + ($79<<2)|0);
        $82 = HEAP32[$81>>2]|0;
        $83 = ($82|0)!=(0|0);
        if (!($83)) {
         $84 = HEAP32[$pStmt>>2]|0;
         $85 = HEAP32[$i>>2]|0;
         $86 = (_sqlite3_column_type($84,$85)|0);
         $87 = ($86|0)!=(5);
         if ($87) {
          label = 27;
          break L8;
         }
        }
        $90 = HEAP32[$i>>2]|0;
        $91 = (($90) + 1)|0;
        HEAP32[$i>>2] = $91;
       }
      }
     } while(0);
     $92 = HEAP32[$3>>2]|0;
     $93 = HEAP32[$4>>2]|0;
     $94 = HEAP32[$nCol>>2]|0;
     $95 = HEAP32[$azVals>>2]|0;
     $96 = HEAP32[$azCols>>2]|0;
     $97 = (FUNCTION_TABLE_iiiii[$92 & 7]($93,$94,$95,$96)|0);
     $98 = ($97|0)!=(0);
     if ($98) {
      label = 30;
      break L8;
     }
    }
   } while(0);
   $101 = HEAP32[$rc>>2]|0;
   $102 = ($101|0)!=(100);
   if ($102) {
    break;
   }
  }
  $103 = HEAP32[$pStmt>>2]|0;
  $104 = (_sqlite3VdbeFinalize($103)|0);
  HEAP32[$rc>>2] = $104;
  HEAP32[$pStmt>>2] = 0;
  $105 = HEAP32[$zLeftover>>2]|0;
  HEAP32[$2>>2] = $105;
  while(1) {
   $106 = HEAP32[$2>>2]|0;
   $107 = HEAP8[$106>>0]|0;
   $108 = $107&255;
   $109 = (1224 + ($108)|0);
   $110 = HEAP8[$109>>0]|0;
   $111 = $110&255;
   $112 = $111 & 1;
   $113 = ($112|0)!=(0);
   if (!($113)) {
    break;
   }
   $114 = HEAP32[$2>>2]|0;
   $115 = (($114) + 1|0);
   HEAP32[$2>>2] = $115;
  }
  $116 = HEAP32[$1>>2]|0;
  $117 = HEAP32[$azCols>>2]|0;
  _sqlite3DbFree($116,$117);
  HEAP32[$azCols>>2] = 0;
 }
 if ((label|0) == 27) {
  $88 = HEAP32[$1>>2]|0;
  $89 = (($88) + 64|0);
  HEAP8[$89>>0] = 1;
 }
 else if ((label|0) == 30) {
  HEAP32[$rc>>2] = 4;
  $99 = HEAP32[$pStmt>>2]|0;
  (_sqlite3VdbeFinalize($99)|0);
  HEAP32[$pStmt>>2] = 0;
  $100 = HEAP32[$1>>2]|0;
  _sqlite3Error($100,4);
 }
 $118 = HEAP32[$pStmt>>2]|0;
 $119 = ($118|0)!=(0|0);
 if ($119) {
  $120 = HEAP32[$pStmt>>2]|0;
  (_sqlite3VdbeFinalize($120)|0);
 }
 $121 = HEAP32[$1>>2]|0;
 $122 = HEAP32[$azCols>>2]|0;
 _sqlite3DbFree($121,$122);
 $123 = HEAP32[$1>>2]|0;
 $124 = HEAP32[$rc>>2]|0;
 $125 = (_sqlite3ApiExit($123,$124)|0);
 HEAP32[$rc>>2] = $125;
 $126 = HEAP32[$rc>>2]|0;
 $127 = ($126|0)!=(0);
 $128 = HEAP32[$5>>2]|0;
 $129 = ($128|0)!=(0|0);
 $or$cond3 = $127 & $129;
 do {
  if ($or$cond3) {
   $130 = HEAP32[$1>>2]|0;
   $131 = (_sqlite3_errmsg($130)|0);
   $132 = (_sqlite3Strlen30($131)|0);
   $133 = (1 + ($132))|0;
   HEAP32[$nErrMsg>>2] = $133;
   $134 = HEAP32[$nErrMsg>>2]|0;
   $135 = ($134|0)<(0);
   $136 = $135 << 31 >> 31;
   $137 = (_sqlite3Malloc($134,$136)|0);
   $138 = HEAP32[$5>>2]|0;
   HEAP32[$138>>2] = $137;
   $139 = HEAP32[$5>>2]|0;
   $140 = HEAP32[$139>>2]|0;
   $141 = ($140|0)!=(0|0);
   if ($141) {
    $142 = HEAP32[$5>>2]|0;
    $143 = HEAP32[$142>>2]|0;
    $144 = HEAP32[$1>>2]|0;
    $145 = (_sqlite3_errmsg($144)|0);
    $146 = HEAP32[$nErrMsg>>2]|0;
    _memcpy(($143|0),($145|0),($146|0))|0;
    break;
   } else {
    HEAP32[$rc>>2] = 7;
    $147 = HEAP32[$1>>2]|0;
    _sqlite3Error($147,7);
    break;
   }
  } else {
   $148 = HEAP32[$5>>2]|0;
   $149 = ($148|0)!=(0|0);
   if ($149) {
    $150 = HEAP32[$5>>2]|0;
    HEAP32[$150>>2] = 0;
   }
  }
 } while(0);
 $151 = HEAP32[$rc>>2]|0;
 HEAP32[$0>>2] = $151;
 $152 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($152|0);
}
function _sqlite3SafetyCheckOk($db) {
 $db = $db|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $magic = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $1 = sp + 4|0;
 $magic = sp;
 HEAP32[$1>>2] = $db;
 $2 = HEAP32[$1>>2]|0;
 $3 = ($2|0)==(0|0);
 do {
  if ($3) {
   _logBadConnection(9840);
   HEAP32[$0>>2] = 0;
  } else {
   $4 = HEAP32[$1>>2]|0;
   $5 = (($4) + 76|0);
   $6 = HEAP32[$5>>2]|0;
   HEAP32[$magic>>2] = $6;
   $7 = HEAP32[$magic>>2]|0;
   $8 = ($7|0)!=(-1607883113);
   if (!($8)) {
    HEAP32[$0>>2] = 1;
    break;
   }
   $9 = HEAP32[$1>>2]|0;
   $10 = (_sqlite3SafetyCheckSickOrOk($9)|0);
   $11 = ($10|0)!=(0);
   if ($11) {
    _logBadConnection(31136);
   }
   HEAP32[$0>>2] = 0;
  }
 } while(0);
 $12 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($12|0);
}
function _sqlite3_prepare_v2($db,$zSql,$nBytes,$ppStmt,$pzTail) {
 $db = $db|0;
 $zSql = $zSql|0;
 $nBytes = $nBytes|0;
 $ppStmt = $ppStmt|0;
 $pzTail = $pzTail|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 20|0;
 $1 = sp + 16|0;
 $2 = sp + 12|0;
 $3 = sp + 8|0;
 $4 = sp + 4|0;
 $rc = sp;
 HEAP32[$0>>2] = $db;
 HEAP32[$1>>2] = $zSql;
 HEAP32[$2>>2] = $nBytes;
 HEAP32[$3>>2] = $ppStmt;
 HEAP32[$4>>2] = $pzTail;
 $5 = HEAP32[$0>>2]|0;
 $6 = HEAP32[$1>>2]|0;
 $7 = HEAP32[$2>>2]|0;
 $8 = HEAP32[$3>>2]|0;
 $9 = HEAP32[$4>>2]|0;
 $10 = (_sqlite3LockAndPrepare($5,$6,$7,1,0,$8,$9)|0);
 HEAP32[$rc>>2] = $10;
 $11 = HEAP32[$rc>>2]|0;
 STACKTOP = sp;return ($11|0);
}
function _sqlite3_errmsg($db) {
 $db = $db|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $z = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $1 = sp + 4|0;
 $z = sp;
 HEAP32[$1>>2] = $db;
 $2 = HEAP32[$1>>2]|0;
 $3 = ($2|0)!=(0|0);
 if (!($3)) {
  $4 = (_sqlite3ErrStr(7)|0);
  HEAP32[$0>>2] = $4;
  $26 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($26|0);
 }
 $5 = HEAP32[$1>>2]|0;
 $6 = (_sqlite3SafetyCheckSickOrOk($5)|0);
 $7 = ($6|0)!=(0);
 if (!($7)) {
  $8 = (_sqlite3MisuseError(126852)|0);
  $9 = (_sqlite3ErrStr($8)|0);
  HEAP32[$0>>2] = $9;
  $26 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($26|0);
 }
 $10 = HEAP32[$1>>2]|0;
 $11 = (($10) + 64|0);
 $12 = HEAP8[$11>>0]|0;
 $13 = ($12<<24>>24)!=(0);
 if ($13) {
  $14 = (_sqlite3ErrStr(7)|0);
  HEAP32[$z>>2] = $14;
 } else {
  $15 = HEAP32[$1>>2]|0;
  $16 = (($15) + 232|0);
  $17 = HEAP32[$16>>2]|0;
  $18 = (_sqlite3_value_text($17)|0);
  HEAP32[$z>>2] = $18;
  $19 = HEAP32[$z>>2]|0;
  $20 = ($19|0)==(0|0);
  if ($20) {
   $21 = HEAP32[$1>>2]|0;
   $22 = (($21) + 52|0);
   $23 = HEAP32[$22>>2]|0;
   $24 = (_sqlite3ErrStr($23)|0);
   HEAP32[$z>>2] = $24;
  }
 }
 $25 = HEAP32[$z>>2]|0;
 HEAP32[$0>>2] = $25;
 $26 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($26|0);
}
function _sqlite3_prepare($db,$zSql,$nBytes,$ppStmt,$pzTail) {
 $db = $db|0;
 $zSql = $zSql|0;
 $nBytes = $nBytes|0;
 $ppStmt = $ppStmt|0;
 $pzTail = $pzTail|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 20|0;
 $1 = sp + 16|0;
 $2 = sp + 12|0;
 $3 = sp + 8|0;
 $4 = sp + 4|0;
 $rc = sp;
 HEAP32[$0>>2] = $db;
 HEAP32[$1>>2] = $zSql;
 HEAP32[$2>>2] = $nBytes;
 HEAP32[$3>>2] = $ppStmt;
 HEAP32[$4>>2] = $pzTail;
 $5 = HEAP32[$0>>2]|0;
 $6 = HEAP32[$1>>2]|0;
 $7 = HEAP32[$2>>2]|0;
 $8 = HEAP32[$3>>2]|0;
 $9 = HEAP32[$4>>2]|0;
 $10 = (_sqlite3LockAndPrepare($5,$6,$7,0,0,$8,$9)|0);
 HEAP32[$rc>>2] = $10;
 $11 = HEAP32[$rc>>2]|0;
 STACKTOP = sp;return ($11|0);
}
function _sqlite3LockAndPrepare($db,$zSql,$nBytes,$saveSqlFlag,$pOld,$ppStmt,$pzTail) {
 $db = $db|0;
 $zSql = $zSql|0;
 $nBytes = $nBytes|0;
 $saveSqlFlag = $saveSqlFlag|0;
 $pOld = $pOld|0;
 $ppStmt = $ppStmt|0;
 $pzTail = $pzTail|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $0 = sp + 32|0;
 $1 = sp + 28|0;
 $2 = sp + 24|0;
 $3 = sp + 20|0;
 $4 = sp + 16|0;
 $5 = sp + 12|0;
 $6 = sp + 8|0;
 $7 = sp + 4|0;
 $rc = sp;
 HEAP32[$1>>2] = $db;
 HEAP32[$2>>2] = $zSql;
 HEAP32[$3>>2] = $nBytes;
 HEAP32[$4>>2] = $saveSqlFlag;
 HEAP32[$5>>2] = $pOld;
 HEAP32[$6>>2] = $ppStmt;
 HEAP32[$7>>2] = $pzTail;
 $8 = HEAP32[$6>>2]|0;
 HEAP32[$8>>2] = 0;
 $9 = HEAP32[$1>>2]|0;
 $10 = (_sqlite3SafetyCheckOk($9)|0);
 $11 = ($10|0)!=(0);
 if (!($11)) {
  $12 = (_sqlite3MisuseError(103895)|0);
  HEAP32[$0>>2] = $12;
  $35 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($35|0);
 }
 $13 = HEAP32[$1>>2]|0;
 _sqlite3BtreeEnterAll($13);
 $14 = HEAP32[$1>>2]|0;
 $15 = HEAP32[$2>>2]|0;
 $16 = HEAP32[$3>>2]|0;
 $17 = HEAP32[$4>>2]|0;
 $18 = HEAP32[$5>>2]|0;
 $19 = HEAP32[$6>>2]|0;
 $20 = HEAP32[$7>>2]|0;
 $21 = (_sqlite3Prepare($14,$15,$16,$17,$18,$19,$20)|0);
 HEAP32[$rc>>2] = $21;
 $22 = HEAP32[$rc>>2]|0;
 $23 = ($22|0)==(17);
 if ($23) {
  $24 = HEAP32[$6>>2]|0;
  $25 = HEAP32[$24>>2]|0;
  (_sqlite3_finalize($25)|0);
  $26 = HEAP32[$1>>2]|0;
  $27 = HEAP32[$2>>2]|0;
  $28 = HEAP32[$3>>2]|0;
  $29 = HEAP32[$4>>2]|0;
  $30 = HEAP32[$5>>2]|0;
  $31 = HEAP32[$6>>2]|0;
  $32 = HEAP32[$7>>2]|0;
  $33 = (_sqlite3Prepare($26,$27,$28,$29,$30,$31,$32)|0);
  HEAP32[$rc>>2] = $33;
 }
 $34 = HEAP32[$rc>>2]|0;
 HEAP32[$0>>2] = $34;
 $35 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($35|0);
}
function _sqlite3RunParser($pParse,$zSql,$pzErrMsg) {
 $pParse = $pParse|0;
 $zSql = $zSql|0;
 $pzErrMsg = $pzErrMsg|0;
 var $$byval_copy11 = 0, $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0;
 var $115 = 0, $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0;
 var $133 = 0, $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0;
 var $151 = 0, $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0;
 var $17 = 0, $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0;
 var $188 = 0, $189 = 0, $19 = 0, $190 = 0, $191 = 0, $192 = 0, $193 = 0, $194 = 0, $195 = 0, $196 = 0, $197 = 0, $198 = 0, $199 = 0, $2 = 0, $20 = 0, $200 = 0, $201 = 0, $202 = 0, $203 = 0, $204 = 0;
 var $205 = 0, $206 = 0, $207 = 0, $208 = 0, $209 = 0, $21 = 0, $210 = 0, $211 = 0, $212 = 0, $213 = 0, $214 = 0, $215 = 0, $216 = 0, $217 = 0, $218 = 0, $219 = 0, $22 = 0, $220 = 0, $221 = 0, $222 = 0;
 var $223 = 0, $224 = 0, $225 = 0, $226 = 0, $227 = 0, $228 = 0, $229 = 0, $23 = 0, $230 = 0, $231 = 0, $232 = 0, $233 = 0, $234 = 0, $235 = 0, $236 = 0, $237 = 0, $238 = 0, $239 = 0, $24 = 0, $240 = 0;
 var $241 = 0, $242 = 0, $243 = 0, $244 = 0, $245 = 0, $246 = 0, $247 = 0, $248 = 0, $249 = 0, $25 = 0, $250 = 0, $251 = 0, $252 = 0, $253 = 0, $254 = 0, $255 = 0, $256 = 0, $257 = 0, $258 = 0, $259 = 0;
 var $26 = 0, $260 = 0, $261 = 0, $262 = 0, $263 = 0, $264 = 0, $265 = 0, $266 = 0, $267 = 0, $268 = 0, $269 = 0, $27 = 0, $270 = 0, $271 = 0, $272 = 0, $273 = 0, $274 = 0, $275 = 0, $276 = 0, $277 = 0;
 var $278 = 0, $279 = 0, $28 = 0, $280 = 0, $281 = 0, $282 = 0, $283 = 0, $284 = 0, $285 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0;
 var $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0;
 var $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0;
 var $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0;
 var $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $db = 0, $enableLookaside = 0, $i = 0, $lastTokenParsed = 0, $mxSqlLen = 0, $nErr = 0, $or$cond = 0, $p = 0, $p1 = 0, $pEngine = 0, $tokenType = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 64|0;
 $$byval_copy11 = sp;
 $0 = sp + 56|0;
 $1 = sp + 52|0;
 $2 = sp + 48|0;
 $3 = sp + 44|0;
 $nErr = sp + 40|0;
 $i = sp + 36|0;
 $pEngine = sp + 32|0;
 $tokenType = sp + 28|0;
 $lastTokenParsed = sp + 24|0;
 $enableLookaside = sp + 60|0;
 $db = sp + 20|0;
 $mxSqlLen = sp + 16|0;
 $p = sp + 12|0;
 $p1 = sp + 8|0;
 HEAP32[$1>>2] = $pParse;
 HEAP32[$2>>2] = $zSql;
 HEAP32[$3>>2] = $pzErrMsg;
 HEAP32[$nErr>>2] = 0;
 HEAP32[$lastTokenParsed>>2] = -1;
 $4 = HEAP32[$1>>2]|0;
 $5 = HEAP32[$4>>2]|0;
 HEAP32[$db>>2] = $5;
 $6 = HEAP32[$db>>2]|0;
 $7 = (($6) + 88|0);
 $8 = (($7) + 4|0);
 $9 = HEAP32[$8>>2]|0;
 HEAP32[$mxSqlLen>>2] = $9;
 $10 = HEAP32[$db>>2]|0;
 $11 = (($10) + 148|0);
 $12 = HEAP32[$11>>2]|0;
 $13 = ($12|0)==(0);
 if ($13) {
  $14 = HEAP32[$db>>2]|0;
  $15 = (($14) + 240|0);
  HEAP32[$15>>2] = 0;
 }
 $16 = HEAP32[$1>>2]|0;
 $17 = (($16) + 12|0);
 HEAP32[$17>>2] = 0;
 $18 = HEAP32[$2>>2]|0;
 $19 = HEAP32[$1>>2]|0;
 $20 = (($19) + 480|0);
 HEAP32[$20>>2] = $18;
 HEAP32[$i>>2] = 0;
 $21 = (_sqlite3ParserAlloc(33)|0);
 HEAP32[$pEngine>>2] = $21;
 $22 = HEAP32[$pEngine>>2]|0;
 $23 = ($22|0)==(0|0);
 $24 = HEAP32[$db>>2]|0;
 if ($23) {
  $25 = (($24) + 64|0);
  HEAP8[$25>>0] = 1;
  HEAP32[$0>>2] = 7;
  $285 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($285|0);
 }
 $26 = (($24) + 248|0);
 $27 = (($26) + 2|0);
 $28 = HEAP8[$27>>0]|0;
 HEAP8[$enableLookaside>>0] = $28;
 $29 = HEAP32[$db>>2]|0;
 $30 = (($29) + 248|0);
 $31 = (($30) + 28|0);
 $32 = HEAP32[$31>>2]|0;
 $33 = ($32|0)!=(0|0);
 if ($33) {
  $34 = HEAP32[$db>>2]|0;
  $35 = (($34) + 248|0);
  $36 = (($35) + 2|0);
  HEAP8[$36>>0] = 1;
 }
 while(1) {
  $37 = HEAP32[$db>>2]|0;
  $38 = (($37) + 64|0);
  $39 = HEAP8[$38>>0]|0;
  $40 = ($39<<24>>24)!=(0);
  if ($40) {
   break;
  }
  $41 = HEAP32[$i>>2]|0;
  $42 = HEAP32[$2>>2]|0;
  $43 = (($42) + ($41)|0);
  $44 = HEAP8[$43>>0]|0;
  $45 = $44 << 24 >> 24;
  $46 = ($45|0)!=(0);
  if (!($46)) {
   break;
  }
  $47 = HEAP32[$i>>2]|0;
  $48 = HEAP32[$2>>2]|0;
  $49 = (($48) + ($47)|0);
  $50 = HEAP32[$1>>2]|0;
  $51 = (($50) + 504|0);
  HEAP32[$51>>2] = $49;
  $52 = HEAP32[$i>>2]|0;
  $53 = HEAP32[$2>>2]|0;
  $54 = (($53) + ($52)|0);
  $55 = (_sqlite3GetToken($54,$tokenType)|0);
  $56 = HEAP32[$1>>2]|0;
  $57 = (($56) + 504|0);
  $58 = (($57) + 4|0);
  HEAP32[$58>>2] = $55;
  $59 = HEAP32[$1>>2]|0;
  $60 = (($59) + 504|0);
  $61 = (($60) + 4|0);
  $62 = HEAP32[$61>>2]|0;
  $63 = HEAP32[$i>>2]|0;
  $64 = (($63) + ($62))|0;
  HEAP32[$i>>2] = $64;
  $65 = HEAP32[$i>>2]|0;
  $66 = HEAP32[$mxSqlLen>>2]|0;
  $67 = ($65|0)>($66|0);
  if ($67) {
   label = 10;
   break;
  }
  $70 = HEAP32[$tokenType>>2]|0;
  if ((($70|0) == 151)) {
   $71 = HEAP32[$db>>2]|0;
   $72 = (($71) + 240|0);
   $73 = HEAP32[$72>>2]|0;
   $74 = ($73|0)!=(0);
   if ($74) {
    label = 13;
    break;
   } else {
    continue;
   }
  } else if ((($70|0) == 1)) {
   $88 = HEAP32[$i>>2]|0;
   $89 = HEAP32[$2>>2]|0;
   $90 = (($89) + ($88)|0);
   $91 = HEAP32[$1>>2]|0;
   $92 = (($91) + 480|0);
   HEAP32[$92>>2] = $90;
  } else if ((($70|0) == 150)) {
   label = 14;
   break;
  }
  $93 = HEAP32[$pEngine>>2]|0;
  $94 = HEAP32[$tokenType>>2]|0;
  $95 = HEAP32[$1>>2]|0;
  $96 = (($95) + 504|0);
  $97 = HEAP32[$1>>2]|0;
  ;HEAP32[$$byval_copy11+0>>2]=HEAP32[$96+0>>2]|0;HEAP32[$$byval_copy11+4>>2]=HEAP32[$96+4>>2]|0;
  _sqlite3Parser($93,$94,$$byval_copy11,$97);
  $98 = HEAP32[$tokenType>>2]|0;
  HEAP32[$lastTokenParsed>>2] = $98;
  $99 = HEAP32[$1>>2]|0;
  $100 = (($99) + 12|0);
  $101 = HEAP32[$100>>2]|0;
  $102 = ($101|0)!=(0);
  if ($102) {
   break;
  }
 }
 if ((label|0) == 10) {
  $68 = HEAP32[$1>>2]|0;
  $69 = (($68) + 12|0);
  HEAP32[$69>>2] = 18;
 }
 else if ((label|0) == 13) {
  $75 = HEAP32[$1>>2]|0;
  _sqlite3ErrorMsg($75,9920,$$byval_copy11);
  $76 = HEAP32[$1>>2]|0;
  $77 = (($76) + 12|0);
  HEAP32[$77>>2] = 9;
 }
 else if ((label|0) == 14) {
  $78 = HEAP32[$db>>2]|0;
  $79 = HEAP32[$3>>2]|0;
  $80 = HEAP32[$79>>2]|0;
  _sqlite3DbFree($78,$80);
  $81 = HEAP32[$db>>2]|0;
  $82 = HEAP32[$1>>2]|0;
  $83 = (($82) + 504|0);
  HEAP32[$$byval_copy11>>2] = $83;
  $84 = (_sqlite3MPrintf($81,9936,$$byval_copy11)|0);
  $85 = HEAP32[$3>>2]|0;
  HEAP32[$85>>2] = $84;
  $86 = HEAP32[$nErr>>2]|0;
  $87 = (($86) + 1)|0;
  HEAP32[$nErr>>2] = $87;
 }
 $103 = HEAP32[$i>>2]|0;
 $104 = HEAP32[$2>>2]|0;
 $105 = (($104) + ($103)|0);
 $106 = HEAP8[$105>>0]|0;
 $107 = $106 << 24 >> 24;
 $108 = ($107|0)==(0);
 $109 = HEAP32[$nErr>>2]|0;
 $110 = ($109|0)==(0);
 $or$cond = $108 & $110;
 if ($or$cond) {
  $111 = HEAP32[$1>>2]|0;
  $112 = (($111) + 12|0);
  $113 = HEAP32[$112>>2]|0;
  $114 = ($113|0)==(0);
  if ($114) {
   $115 = HEAP32[$lastTokenParsed>>2]|0;
   $116 = ($115|0)!=(1);
   if ($116) {
    $117 = HEAP32[$pEngine>>2]|0;
    $118 = HEAP32[$1>>2]|0;
    $119 = (($118) + 504|0);
    $120 = HEAP32[$1>>2]|0;
    ;HEAP32[$$byval_copy11+0>>2]=HEAP32[$119+0>>2]|0;HEAP32[$$byval_copy11+4>>2]=HEAP32[$119+4>>2]|0;
    _sqlite3Parser($117,1,$$byval_copy11,$120);
    $121 = HEAP32[$i>>2]|0;
    $122 = HEAP32[$2>>2]|0;
    $123 = (($122) + ($121)|0);
    $124 = HEAP32[$1>>2]|0;
    $125 = (($124) + 480|0);
    HEAP32[$125>>2] = $123;
   }
   $126 = HEAP32[$pEngine>>2]|0;
   $127 = HEAP32[$1>>2]|0;
   $128 = (($127) + 504|0);
   $129 = HEAP32[$1>>2]|0;
   ;HEAP32[$$byval_copy11+0>>2]=HEAP32[$128+0>>2]|0;HEAP32[$$byval_copy11+4>>2]=HEAP32[$128+4>>2]|0;
   _sqlite3Parser($126,0,$$byval_copy11,$129);
  }
 }
 $130 = HEAP32[$pEngine>>2]|0;
 _sqlite3ParserFree($130,13);
 $131 = HEAP8[$enableLookaside>>0]|0;
 $132 = HEAP32[$db>>2]|0;
 $133 = (($132) + 248|0);
 $134 = (($133) + 2|0);
 HEAP8[$134>>0] = $131;
 $135 = HEAP32[$db>>2]|0;
 $136 = (($135) + 64|0);
 $137 = HEAP8[$136>>0]|0;
 $138 = ($137<<24>>24)!=(0);
 if ($138) {
  $139 = HEAP32[$1>>2]|0;
  $140 = (($139) + 12|0);
  HEAP32[$140>>2] = 7;
 }
 $141 = HEAP32[$1>>2]|0;
 $142 = (($141) + 12|0);
 $143 = HEAP32[$142>>2]|0;
 $144 = ($143|0)!=(0);
 if ($144) {
  $145 = HEAP32[$1>>2]|0;
  $146 = (($145) + 12|0);
  $147 = HEAP32[$146>>2]|0;
  $148 = ($147|0)!=(101);
  if ($148) {
   $149 = HEAP32[$1>>2]|0;
   $150 = (($149) + 4|0);
   $151 = HEAP32[$150>>2]|0;
   $152 = ($151|0)==(0|0);
   if ($152) {
    $153 = HEAP32[$1>>2]|0;
    $154 = (($153) + 4|0);
    $155 = HEAP32[$db>>2]|0;
    $156 = HEAP32[$1>>2]|0;
    $157 = (($156) + 12|0);
    $158 = HEAP32[$157>>2]|0;
    $159 = (_sqlite3ErrStr($158)|0);
    HEAP32[$$byval_copy11>>2] = $159;
    _sqlite3SetString($154,$155,1200,$$byval_copy11);
   }
  }
 }
 $160 = HEAP32[$1>>2]|0;
 $161 = (($160) + 4|0);
 $162 = HEAP32[$161>>2]|0;
 $163 = ($162|0)!=(0|0);
 if ($163) {
  $164 = HEAP32[$1>>2]|0;
  $165 = (($164) + 4|0);
  $166 = HEAP32[$165>>2]|0;
  $167 = HEAP32[$3>>2]|0;
  HEAP32[$167>>2] = $166;
  $168 = HEAP32[$1>>2]|0;
  $169 = (($168) + 12|0);
  $170 = HEAP32[$169>>2]|0;
  $171 = HEAP32[$3>>2]|0;
  $172 = HEAP32[$171>>2]|0;
  HEAP32[$$byval_copy11>>2] = $172;
  _sqlite3_log($170,1200,$$byval_copy11);
  $173 = HEAP32[$1>>2]|0;
  $174 = (($173) + 4|0);
  HEAP32[$174>>2] = 0;
  $175 = HEAP32[$nErr>>2]|0;
  $176 = (($175) + 1)|0;
  HEAP32[$nErr>>2] = $176;
 }
 $177 = HEAP32[$1>>2]|0;
 $178 = (($177) + 8|0);
 $179 = HEAP32[$178>>2]|0;
 $180 = ($179|0)!=(0|0);
 if ($180) {
  $181 = HEAP32[$1>>2]|0;
  $182 = (($181) + 64|0);
  $183 = HEAP32[$182>>2]|0;
  $184 = ($183|0)>(0);
  if ($184) {
   $185 = HEAP32[$1>>2]|0;
   $186 = (($185) + 18|0);
   $187 = HEAP8[$186>>0]|0;
   $188 = $187&255;
   $189 = ($188|0)==(0);
   if ($189) {
    $190 = HEAP32[$1>>2]|0;
    $191 = (($190) + 8|0);
    $192 = HEAP32[$191>>2]|0;
    _sqlite3VdbeDelete($192);
    $193 = HEAP32[$1>>2]|0;
    $194 = (($193) + 8|0);
    HEAP32[$194>>2] = 0;
   }
  }
 }
 $195 = HEAP32[$1>>2]|0;
 $196 = (($195) + 18|0);
 $197 = HEAP8[$196>>0]|0;
 $198 = $197&255;
 $199 = ($198|0)==(0);
 if ($199) {
  $200 = HEAP32[$db>>2]|0;
  $201 = HEAP32[$1>>2]|0;
  $202 = (($201) + 400|0);
  $203 = HEAP32[$202>>2]|0;
  _sqlite3DbFree($200,$203);
  $204 = HEAP32[$1>>2]|0;
  $205 = (($204) + 400|0);
  HEAP32[$205>>2] = 0;
  $206 = HEAP32[$1>>2]|0;
  $207 = (($206) + 396|0);
  HEAP32[$207>>2] = 0;
 }
 $208 = HEAP32[$1>>2]|0;
 $209 = (($208) + 520|0);
 $210 = HEAP32[$209>>2]|0;
 _sqlite3_free($210);
 $211 = HEAP32[$1>>2]|0;
 $212 = (($211) + 451|0);
 $213 = HEAP8[$212>>0]|0;
 $214 = ($213<<24>>24)!=(0);
 if (!($214)) {
  $215 = HEAP32[$db>>2]|0;
  $216 = HEAP32[$1>>2]|0;
  $217 = (($216) + 484|0);
  $218 = HEAP32[$217>>2]|0;
  _sqlite3DeleteTable($215,$218);
 }
 $219 = HEAP32[$1>>2]|0;
 $220 = (($219) + 449|0);
 $221 = HEAP8[$220>>0]|0;
 $222 = ($221<<24>>24)!=(0);
 if ($222) {
  $223 = HEAP32[$db>>2]|0;
  $224 = HEAP32[$1>>2]|0;
  $225 = (($224) + 532|0);
  $226 = HEAP32[$225>>2]|0;
  _sqlite3WithDelete($223,$226);
 }
 $227 = HEAP32[$db>>2]|0;
 $228 = HEAP32[$1>>2]|0;
 $229 = (($228) + 488|0);
 $230 = HEAP32[$229>>2]|0;
 _sqlite3DeleteTrigger($227,$230);
 $231 = HEAP32[$1>>2]|0;
 $232 = (($231) + 444|0);
 $233 = HEAP32[$232>>2]|0;
 $234 = (($233) - 1)|0;
 HEAP32[$i>>2] = $234;
 while(1) {
  $235 = HEAP32[$i>>2]|0;
  $236 = ($235|0)>=(0);
  $237 = HEAP32[$db>>2]|0;
  if (!($236)) {
   break;
  }
  $238 = HEAP32[$i>>2]|0;
  $239 = HEAP32[$1>>2]|0;
  $240 = (($239) + 472|0);
  $241 = HEAP32[$240>>2]|0;
  $242 = (($241) + ($238<<2)|0);
  $243 = HEAP32[$242>>2]|0;
  _sqlite3DbFree($237,$243);
  $244 = HEAP32[$i>>2]|0;
  $245 = (($244) + -1)|0;
  HEAP32[$i>>2] = $245;
 }
 $246 = HEAP32[$1>>2]|0;
 $247 = (($246) + 472|0);
 $248 = HEAP32[$247>>2]|0;
 _sqlite3DbFree($237,$248);
 while(1) {
  $249 = HEAP32[$1>>2]|0;
  $250 = (($249) + 404|0);
  $251 = HEAP32[$250>>2]|0;
  $252 = ($251|0)!=(0|0);
  if (!($252)) {
   break;
  }
  $253 = HEAP32[$1>>2]|0;
  $254 = (($253) + 404|0);
  $255 = HEAP32[$254>>2]|0;
  HEAP32[$p>>2] = $255;
  $256 = HEAP32[$p>>2]|0;
  $257 = HEAP32[$256>>2]|0;
  $258 = HEAP32[$1>>2]|0;
  $259 = (($258) + 404|0);
  HEAP32[$259>>2] = $257;
  $260 = HEAP32[$db>>2]|0;
  $261 = HEAP32[$p>>2]|0;
  _sqlite3DbFree($260,$261);
 }
 while(1) {
  $262 = HEAP32[$1>>2]|0;
  $263 = (($262) + 524|0);
  $264 = HEAP32[$263>>2]|0;
  $265 = ($264|0)!=(0|0);
  if (!($265)) {
   break;
  }
  $266 = HEAP32[$1>>2]|0;
  $267 = (($266) + 524|0);
  $268 = HEAP32[$267>>2]|0;
  HEAP32[$p1>>2] = $268;
  $269 = HEAP32[$p1>>2]|0;
  $270 = (($269) + 72|0);
  $271 = HEAP32[$270>>2]|0;
  $272 = HEAP32[$1>>2]|0;
  $273 = (($272) + 524|0);
  HEAP32[$273>>2] = $271;
  $274 = HEAP32[$db>>2]|0;
  $275 = HEAP32[$p1>>2]|0;
  _sqlite3DeleteTable($274,$275);
 }
 $276 = HEAP32[$nErr>>2]|0;
 $277 = ($276|0)>(0);
 if ($277) {
  $278 = HEAP32[$1>>2]|0;
  $279 = (($278) + 12|0);
  $280 = HEAP32[$279>>2]|0;
  $281 = ($280|0)==(0);
  if ($281) {
   $282 = HEAP32[$1>>2]|0;
   $283 = (($282) + 12|0);
   HEAP32[$283>>2] = 1;
  }
 }
 $284 = HEAP32[$nErr>>2]|0;
 HEAP32[$0>>2] = $284;
 $285 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($285|0);
}
function _sqlite3ValueNew($db) {
 $db = $db|0;
 var $0 = 0, $1 = 0, $10 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $p = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $p = sp;
 HEAP32[$0>>2] = $db;
 $1 = HEAP32[$0>>2]|0;
 $2 = (_sqlite3DbMallocZero($1,40,0)|0);
 HEAP32[$p>>2] = $2;
 $3 = HEAP32[$p>>2]|0;
 $4 = ($3|0)!=(0|0);
 if (!($4)) {
  $10 = HEAP32[$p>>2]|0;
  STACKTOP = sp;return ($10|0);
 }
 $5 = HEAP32[$p>>2]|0;
 $6 = (($5) + 8|0);
 HEAP16[$6>>1] = 1;
 $7 = HEAP32[$0>>2]|0;
 $8 = HEAP32[$p>>2]|0;
 $9 = (($8) + 32|0);
 HEAP32[$9>>2] = $7;
 $10 = HEAP32[$p>>2]|0;
 STACKTOP = sp;return ($10|0);
}
function _sqlite3ValueSetStr($v,$n,$z,$enc,$xDel) {
 $v = $v|0;
 $n = $n|0;
 $z = $z|0;
 $enc = $enc|0;
 $xDel = $xDel|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 12|0;
 $1 = sp + 8|0;
 $2 = sp + 4|0;
 $3 = sp + 16|0;
 $4 = sp;
 HEAP32[$0>>2] = $v;
 HEAP32[$1>>2] = $n;
 HEAP32[$2>>2] = $z;
 HEAP8[$3>>0] = $enc;
 HEAP32[$4>>2] = $xDel;
 $5 = HEAP32[$0>>2]|0;
 $6 = ($5|0)!=(0|0);
 if (!($6)) {
  STACKTOP = sp;return;
 }
 $7 = HEAP32[$0>>2]|0;
 $8 = HEAP32[$2>>2]|0;
 $9 = HEAP32[$1>>2]|0;
 $10 = HEAP8[$3>>0]|0;
 $11 = HEAP32[$4>>2]|0;
 (_sqlite3VdbeMemSetStr($7,$8,$9,$10,$11)|0);
 STACKTOP = sp;return;
}
function _sqlite3ValueFree($v) {
 $v = $v|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 HEAP32[$0>>2] = $v;
 $1 = HEAP32[$0>>2]|0;
 $2 = ($1|0)!=(0|0);
 if (!($2)) {
  STACKTOP = sp;return;
 }
 $3 = HEAP32[$0>>2]|0;
 _sqlite3VdbeMemRelease($3);
 $4 = HEAP32[$0>>2]|0;
 $5 = (($4) + 32|0);
 $6 = HEAP32[$5>>2]|0;
 $7 = HEAP32[$0>>2]|0;
 _sqlite3DbFree($6,$7);
 STACKTOP = sp;return;
}
function _sqlite3_libversion() {
 var label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = sp;return (8|0);
}
function _sqlite3_sourceid() {
 var label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = sp;return (1504|0);
}
function _sqlite3MallocInit() {
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0;
 var $9 = 0, $i = 0, $n = 0, $pSlot = 0, $sz = 0, dest = 0, label = 0, sp = 0, stop = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $i = sp + 12|0;
 $n = sp + 8|0;
 $sz = sp + 4|0;
 $pSlot = sp;
 $0 = HEAP32[((38640 + 36|0))>>2]|0;
 $1 = ($0|0)==(0|0);
 if ($1) {
  _sqlite3MemSetDefault();
 }
 dest=136+0|0; stop=dest+40|0; do { HEAP32[dest>>2]=0|0; dest=dest+4|0; } while ((dest|0) < (stop|0));
 $2 = HEAP32[((38640 + 4|0))>>2]|0;
 $3 = ($2|0)!=(0);
 if ($3) {
  HEAP32[136>>2] = (8);
 }
 $4 = HEAP32[((38640 + 192|0))>>2]|0;
 $5 = ($4|0)!=(0|0);
 if ($5) {
  $6 = HEAP32[((38640 + 196|0))>>2]|0;
  $7 = ($6|0)>=(100);
  if ($7) {
   $8 = HEAP32[((38640 + 200|0))>>2]|0;
   $9 = ($8|0)>(0);
   if ($9) {
    $10 = HEAP32[((38640 + 196|0))>>2]|0;
    $11 = $10 & -8;
    HEAP32[$sz>>2] = $11;
    $12 = HEAP32[$sz>>2]|0;
    HEAP32[((38640 + 196|0))>>2] = $12;
    $13 = HEAP32[((38640 + 192|0))>>2]|0;
    HEAP32[$pSlot>>2] = $13;
    $14 = HEAP32[((38640 + 200|0))>>2]|0;
    HEAP32[$n>>2] = $14;
    $15 = HEAP32[$pSlot>>2]|0;
    HEAP32[((136 + 28|0))>>2] = $15;
    $16 = HEAP32[$n>>2]|0;
    HEAP32[((136 + 32|0))>>2] = $16;
    HEAP32[$i>>2] = 0;
    while(1) {
     $17 = HEAP32[$i>>2]|0;
     $18 = HEAP32[$n>>2]|0;
     $19 = (($18) - 1)|0;
     $20 = ($17|0)<($19|0);
     if (!($20)) {
      break;
     }
     $21 = HEAP32[$sz>>2]|0;
     $22 = HEAP32[$pSlot>>2]|0;
     $23 = (($22) + ($21)|0);
     $24 = HEAP32[$pSlot>>2]|0;
     HEAP32[$24>>2] = $23;
     $25 = HEAP32[$pSlot>>2]|0;
     $26 = HEAP32[$25>>2]|0;
     HEAP32[$pSlot>>2] = $26;
     $27 = HEAP32[$i>>2]|0;
     $28 = (($27) + 1)|0;
     HEAP32[$i>>2] = $28;
    }
    $29 = HEAP32[$pSlot>>2]|0;
    HEAP32[$29>>2] = 0;
    $30 = HEAP32[$pSlot>>2]|0;
    $31 = (($30) + 4|0);
    HEAP32[((136 + 24|0))>>2] = $31;
   } else {
    label = 12;
   }
  } else {
   label = 12;
  }
 } else {
  label = 12;
 }
 if ((label|0) == 12) {
  HEAP32[((136 + 24|0))>>2] = 0;
  HEAP32[((38640 + 192|0))>>2] = 0;
  HEAP32[((38640 + 196|0))>>2] = 0;
  HEAP32[((38640 + 200|0))>>2] = 0;
 }
 $32 = HEAP32[((38640 + 204|0))>>2]|0;
 $33 = ($32|0)==(0|0);
 if (!($33)) {
  $34 = HEAP32[((38640 + 208|0))>>2]|0;
  $35 = ($34|0)<(512);
  if (!($35)) {
   $36 = HEAP32[((38640 + 212|0))>>2]|0;
   $37 = ($36|0)<(1);
   if (!($37)) {
    $38 = HEAP32[((38640 + 56|0))>>2]|0;
    $39 = HEAP32[((38640 + 64|0))>>2]|0;
    $40 = (FUNCTION_TABLE_ii[$38 & 31]($39)|0);
    STACKTOP = sp;return ($40|0);
   }
  }
 }
 HEAP32[((38640 + 204|0))>>2] = 0;
 HEAP32[((38640 + 208|0))>>2] = 0;
 HEAP32[((38640 + 212|0))>>2] = 0;
 $38 = HEAP32[((38640 + 56|0))>>2]|0;
 $39 = HEAP32[((38640 + 64|0))>>2]|0;
 $40 = (FUNCTION_TABLE_ii[$38 & 31]($39)|0);
 STACKTOP = sp;return ($40|0);
}
function _sqlite3RegisterGlobalFunctions() {
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $aFunc = 0, $i = 0, $pHash = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $i = sp + 8|0;
 $pHash = sp + 4|0;
 $aFunc = sp;
 HEAP32[$pHash>>2] = 1576;
 HEAP32[$aFunc>>2] = 6232;
 HEAP32[$i>>2] = 0;
 while(1) {
  $0 = HEAP32[$i>>2]|0;
  $1 = ($0|0)<(57);
  if (!($1)) {
   break;
  }
  $2 = HEAP32[$pHash>>2]|0;
  $3 = HEAP32[$i>>2]|0;
  $4 = HEAP32[$aFunc>>2]|0;
  $5 = (($4) + (($3*36)|0)|0);
  _sqlite3FuncDefInsert($2,$5);
  $6 = HEAP32[$i>>2]|0;
  $7 = (($6) + 1)|0;
  HEAP32[$i>>2] = $7;
 }
 _sqlite3RegisterDateTimeFunctions();
 _sqlite3AlterFunctions();
 STACKTOP = sp;return;
}
function _sqlite3PcacheInitialize() {
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 $0 = HEAP32[((38640 + 112|0))>>2]|0;
 $1 = ($0|0)==(0|0);
 if ($1) {
  _sqlite3PCacheSetDefault();
 }
 $2 = HEAP32[((38640 + 112|0))>>2]|0;
 $3 = HEAP32[((38640 + 108|0))>>2]|0;
 $4 = (FUNCTION_TABLE_ii[$2 & 31]($3)|0);
 STACKTOP = sp;return ($4|0);
}
function _sqlite3OsInit() {
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $p = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $p = sp;
 $1 = (_sqlite3_malloc(10)|0);
 HEAP32[$p>>2] = $1;
 $2 = HEAP32[$p>>2]|0;
 $3 = ($2|0)==(0|0);
 if ($3) {
  HEAP32[$0>>2] = 7;
  $6 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($6|0);
 } else {
  $4 = HEAP32[$p>>2]|0;
  _sqlite3_free($4);
  $5 = (_sqlite3_os_init()|0);
  HEAP32[$0>>2] = $5;
  $6 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($6|0);
 }
 return 0|0;
}
function _sqlite3PCacheBufferSetup($pBuf,$sz,$n) {
 $pBuf = $pBuf|0;
 $sz = $sz|0;
 $n = $n|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $3 = 0;
 var $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $p = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $1 = sp + 8|0;
 $2 = sp + 4|0;
 $p = sp;
 HEAP32[$0>>2] = $pBuf;
 HEAP32[$1>>2] = $sz;
 HEAP32[$2>>2] = $n;
 $3 = HEAP32[((3136 + 28|0))>>2]|0;
 $4 = ($3|0)!=(0);
 if (!($4)) {
  STACKTOP = sp;return;
 }
 $5 = HEAP32[$1>>2]|0;
 $6 = $5 & -8;
 HEAP32[$1>>2] = $6;
 $7 = HEAP32[$1>>2]|0;
 HEAP32[((3136 + 32|0))>>2] = $7;
 $8 = HEAP32[$2>>2]|0;
 HEAP32[((3136 + 60|0))>>2] = $8;
 HEAP32[((3136 + 36|0))>>2] = $8;
 $9 = HEAP32[$2>>2]|0;
 $10 = ($9|0)>(90);
 if ($10) {
  $14 = 10;
 } else {
  $11 = HEAP32[$2>>2]|0;
  $12 = (($11|0) / 10)&-1;
  $13 = (($12) + 1)|0;
  $14 = $13;
 }
 HEAP32[((3136 + 40|0))>>2] = $14;
 $15 = HEAP32[$0>>2]|0;
 HEAP32[((3136 + 44|0))>>2] = $15;
 HEAP32[((3136 + 56|0))>>2] = 0;
 HEAP32[((3136 + 64|0))>>2] = 0;
 while(1) {
  $16 = HEAP32[$2>>2]|0;
  $17 = (($16) + -1)|0;
  HEAP32[$2>>2] = $17;
  $18 = ($16|0)!=(0);
  $19 = HEAP32[$0>>2]|0;
  if (!($18)) {
   break;
  }
  HEAP32[$p>>2] = $19;
  $20 = HEAP32[((3136 + 56|0))>>2]|0;
  $21 = HEAP32[$p>>2]|0;
  HEAP32[$21>>2] = $20;
  $22 = HEAP32[$p>>2]|0;
  HEAP32[((3136 + 56|0))>>2] = $22;
  $23 = HEAP32[$1>>2]|0;
  $24 = HEAP32[$0>>2]|0;
  $25 = (($24) + ($23)|0);
  HEAP32[$0>>2] = $25;
 }
 HEAP32[((3136 + 48|0))>>2] = $19;
 STACKTOP = sp;return;
}
function _sqlite3_config($op,$varargs) {
 $op = $op|0;
 $varargs = $varargs|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $14 = 0, $15 = 0, $16 = 0;
 var $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0;
 var $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0;
 var $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0;
 var $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0;
 var $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $ap = 0, $arglist_current = 0, $arglist_current12 = 0, $arglist_current15 = 0, $arglist_current18 = 0, $arglist_current21 = 0, $arglist_current24 = 0, $arglist_current27 = 0, $arglist_current3 = 0;
 var $arglist_current30 = 0, $arglist_current33 = 0, $arglist_current36 = 0, $arglist_current39 = 0, $arglist_current42 = 0, $arglist_current45 = 0, $arglist_current48 = 0, $arglist_current51 = 0, $arglist_current54 = 0, $arglist_current6 = 0, $arglist_current9 = 0, $arglist_next = 0, $arglist_next10 = 0, $arglist_next13 = 0, $arglist_next16 = 0, $arglist_next19 = 0, $arglist_next22 = 0, $arglist_next25 = 0, $arglist_next28 = 0, $arglist_next31 = 0;
 var $arglist_next34 = 0, $arglist_next37 = 0, $arglist_next4 = 0, $arglist_next40 = 0, $arglist_next43 = 0, $arglist_next46 = 0, $arglist_next49 = 0, $arglist_next52 = 0, $arglist_next55 = 0, $arglist_next7 = 0, $mxMmap = 0, $or$cond = 0, $rc = 0, $szMmap = 0, dest = 0, label = 0, sp = 0, src = 0, stop = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $0 = sp + 44|0;
 $1 = sp + 40|0;
 $ap = sp + 24|0;
 $rc = sp + 16|0;
 $szMmap = sp + 8|0;
 $mxMmap = sp;
 HEAP32[$1>>2] = $op;
 HEAP32[$rc>>2] = 0;
 $2 = HEAP32[((38640 + 224|0))>>2]|0;
 $3 = ($2|0)!=(0);
 if ($3) {
  $4 = (_sqlite3MisuseError(125290)|0);
  HEAP32[$0>>2] = $4;
  $130 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($130|0);
 }
 HEAP32[$ap>>2] = $varargs;
 $5 = HEAP32[$1>>2]|0;
 do {
  switch ($5|0) {
  case 5:  {
   $7 = HEAP32[((38640 + 36|0))>>2]|0;
   $8 = ($7|0)==(0|0);
   if ($8) {
    _sqlite3MemSetDefault();
   }
   $arglist_current3 = HEAP32[$ap>>2]|0;
   $9 = HEAP32[$arglist_current3>>2]|0;
   $arglist_next4 = (($arglist_current3) + 4|0);
   HEAP32[$ap>>2] = $arglist_next4;
   ;HEAP32[$9+0>>2]=HEAP32[((38640 + 36|0))+0>>2]|0;HEAP32[$9+4>>2]=HEAP32[((38640 + 36|0))+4>>2]|0;HEAP32[$9+8>>2]=HEAP32[((38640 + 36|0))+8>>2]|0;HEAP32[$9+12>>2]=HEAP32[((38640 + 36|0))+12>>2]|0;HEAP32[$9+16>>2]=HEAP32[((38640 + 36|0))+16>>2]|0;HEAP32[$9+20>>2]=HEAP32[((38640 + 36|0))+20>>2]|0;HEAP32[$9+24>>2]=HEAP32[((38640 + 36|0))+24>>2]|0;HEAP32[$9+28>>2]=HEAP32[((38640 + 36|0))+28>>2]|0;
   break;
  }
  case 17:  {
   $arglist_current45 = HEAP32[$ap>>2]|0;
   $25 = HEAP32[$arglist_current45>>2]|0;
   $arglist_next46 = (($arglist_current45) + 4|0);
   HEAP32[$ap>>2] = $arglist_next46;
   HEAP32[((38640 + 12|0))>>2] = $25;
   break;
  }
  case 9:  {
   $arglist_current6 = HEAP32[$ap>>2]|0;
   $10 = HEAP32[$arglist_current6>>2]|0;
   $arglist_next7 = (($arglist_current6) + 4|0);
   HEAP32[$ap>>2] = $arglist_next7;
   HEAP32[38640>>2] = $10;
   break;
  }
  case 20:  {
   $arglist_current48 = HEAP32[$ap>>2]|0;
   $26 = HEAP32[$arglist_current48>>2]|0;
   $arglist_next49 = (($arglist_current48) + 4|0);
   HEAP32[$ap>>2] = $arglist_next49;
   HEAP32[((38640 + 16|0))>>2] = $26;
   break;
  }
  case 19:  {
   $18 = HEAP32[((38640 + 112|0))>>2]|0;
   $19 = ($18|0)==(0|0);
   if ($19) {
    _sqlite3PCacheSetDefault();
   }
   $arglist_current30 = HEAP32[$ap>>2]|0;
   $20 = HEAP32[$arglist_current30>>2]|0;
   $arglist_next31 = (($arglist_current30) + 4|0);
   HEAP32[$ap>>2] = $arglist_next31;
   dest=$20+0|0; src=((38640 + 104|0))+0|0; stop=dest+52|0; do { HEAP32[dest>>2]=HEAP32[src>>2]|0; dest=dest+4|0; src=src+4|0; } while ((dest|0) < (stop|0));
   break;
  }
  case 15:  {
   HEAP32[$rc>>2] = 1;
   break;
  }
  case 16:  {
   $arglist_current39 = HEAP32[$ap>>2]|0;
   $23 = HEAP32[$arglist_current39>>2]|0;
   $arglist_next40 = (($arglist_current39) + 4|0);
   HEAP32[$ap>>2] = $arglist_next40;
   HEAP32[((38640 + 252|0))>>2] = $23;
   $arglist_current42 = HEAP32[$ap>>2]|0;
   $24 = HEAP32[$arglist_current42>>2]|0;
   $arglist_next43 = (($arglist_current42) + 4|0);
   HEAP32[$ap>>2] = $arglist_next43;
   HEAP32[((38640 + 256|0))>>2] = $24;
   break;
  }
  case 7:  {
   $arglist_current18 = HEAP32[$ap>>2]|0;
   $14 = HEAP32[$arglist_current18>>2]|0;
   $arglist_next19 = (($arglist_current18) + 4|0);
   HEAP32[$ap>>2] = $arglist_next19;
   HEAP32[((38640 + 204|0))>>2] = $14;
   $arglist_current21 = HEAP32[$ap>>2]|0;
   $15 = HEAP32[$arglist_current21>>2]|0;
   $arglist_next22 = (($arglist_current21) + 4|0);
   HEAP32[$ap>>2] = $arglist_next22;
   HEAP32[((38640 + 208|0))>>2] = $15;
   $arglist_current24 = HEAP32[$ap>>2]|0;
   $16 = HEAP32[$arglist_current24>>2]|0;
   $arglist_next25 = (($arglist_current24) + 4|0);
   HEAP32[$ap>>2] = $arglist_next25;
   HEAP32[((38640 + 212|0))>>2] = $16;
   break;
  }
  case 13:  {
   $arglist_current33 = HEAP32[$ap>>2]|0;
   $21 = HEAP32[$arglist_current33>>2]|0;
   $arglist_next34 = (($arglist_current33) + 4|0);
   HEAP32[$ap>>2] = $arglist_next34;
   HEAP32[((38640 + 28|0))>>2] = $21;
   $arglist_current36 = HEAP32[$ap>>2]|0;
   $22 = HEAP32[$arglist_current36>>2]|0;
   $arglist_next37 = (($arglist_current36) + 4|0);
   HEAP32[$ap>>2] = $arglist_next37;
   HEAP32[((38640 + 32|0))>>2] = $22;
   break;
  }
  case 4:  {
   $arglist_current = HEAP32[$ap>>2]|0;
   $6 = HEAP32[$arglist_current>>2]|0;
   $arglist_next = (($arglist_current) + 4|0);
   HEAP32[$ap>>2] = $arglist_next;
   ;HEAP32[((38640 + 36|0))+0>>2]=HEAP32[$6+0>>2]|0;HEAP32[((38640 + 36|0))+4>>2]=HEAP32[$6+4>>2]|0;HEAP32[((38640 + 36|0))+8>>2]=HEAP32[$6+8>>2]|0;HEAP32[((38640 + 36|0))+12>>2]=HEAP32[$6+12>>2]|0;HEAP32[((38640 + 36|0))+16>>2]=HEAP32[$6+16>>2]|0;HEAP32[((38640 + 36|0))+20>>2]=HEAP32[$6+20>>2]|0;HEAP32[((38640 + 36|0))+24>>2]=HEAP32[$6+24>>2]|0;HEAP32[((38640 + 36|0))+28>>2]=HEAP32[$6+28>>2]|0;
   break;
  }
  case 14:  {
   break;
  }
  case 6:  {
   $arglist_current9 = HEAP32[$ap>>2]|0;
   $11 = HEAP32[$arglist_current9>>2]|0;
   $arglist_next10 = (($arglist_current9) + 4|0);
   HEAP32[$ap>>2] = $arglist_next10;
   HEAP32[((38640 + 192|0))>>2] = $11;
   $arglist_current12 = HEAP32[$ap>>2]|0;
   $12 = HEAP32[$arglist_current12>>2]|0;
   $arglist_next13 = (($arglist_current12) + 4|0);
   HEAP32[$ap>>2] = $arglist_next13;
   HEAP32[((38640 + 196|0))>>2] = $12;
   $arglist_current15 = HEAP32[$ap>>2]|0;
   $13 = HEAP32[$arglist_current15>>2]|0;
   $arglist_next16 = (($arglist_current15) + 4|0);
   HEAP32[$ap>>2] = $arglist_next16;
   HEAP32[((38640 + 200|0))>>2] = $13;
   break;
  }
  case 18:  {
   $arglist_current27 = HEAP32[$ap>>2]|0;
   $17 = HEAP32[$arglist_current27>>2]|0;
   $arglist_next28 = (($arglist_current27) + 4|0);
   HEAP32[$ap>>2] = $arglist_next28;
   dest=((38640 + 104|0))+0|0; src=$17+0|0; stop=dest+52|0; do { HEAP32[dest>>2]=HEAP32[src>>2]|0; dest=dest+4|0; src=src+4|0; } while ((dest|0) < (stop|0));
   break;
  }
  case 22:  {
   $arglist_current51 = HEAP32[$ap>>2]|0;
   $27 = $arglist_current51;
   $28 = $27;
   $29 = HEAP32[$28>>2]|0;
   $30 = (($27) + 4)|0;
   $31 = $30;
   $32 = HEAP32[$31>>2]|0;
   $arglist_next52 = (($arglist_current51) + 8|0);
   HEAP32[$ap>>2] = $arglist_next52;
   $33 = $szMmap;
   $34 = $33;
   HEAP32[$34>>2] = $29;
   $35 = (($33) + 4)|0;
   $36 = $35;
   HEAP32[$36>>2] = $32;
   $arglist_current54 = HEAP32[$ap>>2]|0;
   $37 = $arglist_current54;
   $38 = $37;
   $39 = HEAP32[$38>>2]|0;
   $40 = (($37) + 4)|0;
   $41 = $40;
   $42 = HEAP32[$41>>2]|0;
   $arglist_next55 = (($arglist_current54) + 8|0);
   HEAP32[$ap>>2] = $arglist_next55;
   $43 = $mxMmap;
   $44 = $43;
   HEAP32[$44>>2] = $39;
   $45 = (($43) + 4)|0;
   $46 = $45;
   HEAP32[$46>>2] = $42;
   $47 = $mxMmap;
   $48 = $47;
   $49 = HEAP32[$48>>2]|0;
   $50 = (($47) + 4)|0;
   $51 = $50;
   $52 = HEAP32[$51>>2]|0;
   $53 = ($52|0)<(0);
   $54 = $mxMmap;
   $55 = $54;
   $56 = HEAP32[$55>>2]|0;
   $57 = (($54) + 4)|0;
   $58 = $57;
   $59 = HEAP32[$58>>2]|0;
   $60 = ($59|0)>(0);
   $61 = ($56>>>0)>(0);
   $62 = ($59|0)==(0);
   $63 = $62 & $61;
   $64 = $60 | $63;
   $or$cond = $53 | $64;
   $65 = $or$cond ? 0 : $39;
   $66 = $or$cond ? 0 : $42;
   $67 = $mxMmap;
   $68 = $67;
   HEAP32[$68>>2] = $65;
   $69 = (($67) + 4)|0;
   $70 = $69;
   HEAP32[$70>>2] = $66;
   $71 = $mxMmap;
   $72 = $71;
   $73 = HEAP32[$72>>2]|0;
   $74 = (($71) + 4)|0;
   $75 = $74;
   $76 = HEAP32[$75>>2]|0;
   $77 = ((38640 + 184|0));
   $78 = $77;
   HEAP32[$78>>2] = $73;
   $79 = (($77) + 4)|0;
   $80 = $79;
   HEAP32[$80>>2] = $76;
   $81 = $szMmap;
   $82 = $81;
   $83 = HEAP32[$82>>2]|0;
   $84 = (($81) + 4)|0;
   $85 = $84;
   $86 = HEAP32[$85>>2]|0;
   $87 = ($86|0)<(0);
   if ($87) {
    $88 = $szMmap;
    $89 = $88;
    HEAP32[$89>>2] = 0;
    $90 = (($88) + 4)|0;
    $91 = $90;
    HEAP32[$91>>2] = 0;
   }
   $92 = $szMmap;
   $93 = $92;
   $94 = HEAP32[$93>>2]|0;
   $95 = (($92) + 4)|0;
   $96 = $95;
   $97 = HEAP32[$96>>2]|0;
   $98 = $mxMmap;
   $99 = $98;
   $100 = HEAP32[$99>>2]|0;
   $101 = (($98) + 4)|0;
   $102 = $101;
   $103 = HEAP32[$102>>2]|0;
   $104 = ($97|0)>($103|0);
   $105 = ($94>>>0)>($100>>>0);
   $106 = ($97|0)==($103|0);
   $107 = $106 & $105;
   $108 = $104 | $107;
   if ($108) {
    $109 = $mxMmap;
    $110 = $109;
    $111 = HEAP32[$110>>2]|0;
    $112 = (($109) + 4)|0;
    $113 = $112;
    $114 = HEAP32[$113>>2]|0;
    $115 = $szMmap;
    $116 = $115;
    HEAP32[$116>>2] = $111;
    $117 = (($115) + 4)|0;
    $118 = $117;
    HEAP32[$118>>2] = $114;
   }
   $119 = $szMmap;
   $120 = $119;
   $121 = HEAP32[$120>>2]|0;
   $122 = (($119) + 4)|0;
   $123 = $122;
   $124 = HEAP32[$123>>2]|0;
   $125 = ((38640 + 176|0));
   $126 = $125;
   HEAP32[$126>>2] = $121;
   $127 = (($125) + 4)|0;
   $128 = $127;
   HEAP32[$128>>2] = $124;
   break;
  }
  default: {
   HEAP32[$rc>>2] = 1;
  }
  }
 } while(0);
 $129 = HEAP32[$rc>>2]|0;
 HEAP32[$0>>2] = $129;
 $130 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($130|0);
}
function _sqlite3MemSetDefault() {
 var $vararg_buffer = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $vararg_buffer = sp;
 HEAP32[$vararg_buffer>>2] = 6120;
 (_sqlite3_config(4,$vararg_buffer)|0);
 STACKTOP = sp;return;
}
function _sqlite3PCacheSetDefault() {
 var $vararg_buffer = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $vararg_buffer = sp;
 HEAP32[$vararg_buffer>>2] = 6064;
 (_sqlite3_config(18,$vararg_buffer)|0);
 STACKTOP = sp;return;
}
function _sqlite3_db_release_memory($db) {
 $db = $db|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0;
 var $9 = 0, $i = 0, $pBt = 0, $pPager = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $i = sp + 8|0;
 $pBt = sp + 4|0;
 $pPager = sp;
 HEAP32[$0>>2] = $db;
 $1 = HEAP32[$0>>2]|0;
 _sqlite3BtreeEnterAll($1);
 HEAP32[$i>>2] = 0;
 while(1) {
  $2 = HEAP32[$i>>2]|0;
  $3 = HEAP32[$0>>2]|0;
  $4 = (($3) + 20|0);
  $5 = HEAP32[$4>>2]|0;
  $6 = ($2|0)<($5|0);
  if (!($6)) {
   break;
  }
  $7 = HEAP32[$i>>2]|0;
  $8 = HEAP32[$0>>2]|0;
  $9 = (($8) + 16|0);
  $10 = HEAP32[$9>>2]|0;
  $11 = (($10) + ($7<<4)|0);
  $12 = (($11) + 4|0);
  $13 = HEAP32[$12>>2]|0;
  HEAP32[$pBt>>2] = $13;
  $14 = HEAP32[$pBt>>2]|0;
  $15 = ($14|0)!=(0|0);
  if ($15) {
   $16 = HEAP32[$pBt>>2]|0;
   $17 = (_sqlite3BtreePager($16)|0);
   HEAP32[$pPager>>2] = $17;
   $18 = HEAP32[$pPager>>2]|0;
   _sqlite3PagerShrink($18);
  }
  $19 = HEAP32[$i>>2]|0;
  $20 = (($19) + 1)|0;
  HEAP32[$i>>2] = $20;
 }
 STACKTOP = sp;return 0;
}
function _sqlite3PagerShrink($pPager) {
 $pPager = $pPager|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 HEAP32[$0>>2] = $pPager;
 $1 = HEAP32[$0>>2]|0;
 $2 = (($1) + 204|0);
 $3 = HEAP32[$2>>2]|0;
 _sqlite3PcacheShrink($3);
 STACKTOP = sp;return;
}
function _setupLookaside($db,$pBuf,$sz,$cnt) {
 $db = $db|0;
 $pBuf = $pBuf|0;
 $sz = $sz|0;
 $cnt = $cnt|0;
 var $$ = 0, $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0;
 var $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0;
 var $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0;
 var $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0;
 var $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0;
 var $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $i = 0, $or$cond = 0, $p = 0, $pStart = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 28|0;
 $1 = sp + 24|0;
 $2 = sp + 20|0;
 $3 = sp + 16|0;
 $4 = sp + 12|0;
 $pStart = sp + 8|0;
 $i = sp + 4|0;
 $p = sp;
 HEAP32[$1>>2] = $db;
 HEAP32[$2>>2] = $pBuf;
 HEAP32[$3>>2] = $sz;
 HEAP32[$4>>2] = $cnt;
 $5 = HEAP32[$1>>2]|0;
 $6 = (($5) + 248|0);
 $7 = (($6) + 4|0);
 $8 = HEAP32[$7>>2]|0;
 $9 = ($8|0)!=(0);
 if ($9) {
  HEAP32[$0>>2] = 5;
  $104 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($104|0);
 }
 $10 = HEAP32[$1>>2]|0;
 $11 = (($10) + 248|0);
 $12 = (($11) + 3|0);
 $13 = HEAP8[$12>>0]|0;
 $14 = ($13<<24>>24)!=(0);
 if ($14) {
  $15 = HEAP32[$1>>2]|0;
  $16 = (($15) + 248|0);
  $17 = (($16) + 28|0);
  $18 = HEAP32[$17>>2]|0;
  _sqlite3_free($18);
 }
 $19 = HEAP32[$3>>2]|0;
 $20 = $19 & -8;
 HEAP32[$3>>2] = $20;
 $21 = HEAP32[$3>>2]|0;
 $22 = ($21|0)<=(4);
 $$ = $22 ? 0 : $20;
 HEAP32[$3>>2] = $$;
 $23 = HEAP32[$4>>2]|0;
 $24 = ($23|0)<(0);
 if ($24) {
  HEAP32[$4>>2] = 0;
 }
 $25 = HEAP32[$3>>2]|0;
 $26 = ($25|0)==(0);
 $27 = HEAP32[$4>>2]|0;
 $28 = ($27|0)==(0);
 $or$cond = $26 | $28;
 do {
  if ($or$cond) {
   HEAP32[$3>>2] = 0;
   HEAP32[$pStart>>2] = 0;
  } else {
   $29 = HEAP32[$2>>2]|0;
   $30 = ($29|0)==(0|0);
   if (!($30)) {
    $43 = HEAP32[$2>>2]|0;
    HEAP32[$pStart>>2] = $43;
    break;
   }
   _sqlite3BeginBenignMalloc();
   $31 = HEAP32[$3>>2]|0;
   $32 = HEAP32[$4>>2]|0;
   $33 = Math_imul($31, $32)|0;
   $34 = ($33|0)<(0);
   $35 = $34 << 31 >> 31;
   $36 = (_sqlite3Malloc($33,$35)|0);
   HEAP32[$pStart>>2] = $36;
   _sqlite3EndBenignMalloc();
   $37 = HEAP32[$pStart>>2]|0;
   $38 = ($37|0)!=(0|0);
   if ($38) {
    $39 = HEAP32[$pStart>>2]|0;
    $40 = (_sqlite3MallocSize($39)|0);
    $41 = HEAP32[$3>>2]|0;
    $42 = (($40|0) / ($41|0))&-1;
    HEAP32[$4>>2] = $42;
   }
  }
 } while(0);
 $44 = HEAP32[$pStart>>2]|0;
 $45 = HEAP32[$1>>2]|0;
 $46 = (($45) + 248|0);
 $47 = (($46) + 28|0);
 HEAP32[$47>>2] = $44;
 $48 = HEAP32[$1>>2]|0;
 $49 = (($48) + 248|0);
 $50 = (($49) + 24|0);
 HEAP32[$50>>2] = 0;
 $51 = HEAP32[$3>>2]|0;
 $52 = $51&65535;
 $53 = HEAP32[$1>>2]|0;
 $54 = (($53) + 248|0);
 HEAP16[$54>>1] = $52;
 $55 = HEAP32[$pStart>>2]|0;
 $56 = ($55|0)!=(0|0);
 if ($56) {
  $57 = HEAP32[$pStart>>2]|0;
  HEAP32[$p>>2] = $57;
  $58 = HEAP32[$4>>2]|0;
  $59 = (($58) - 1)|0;
  HEAP32[$i>>2] = $59;
  while(1) {
   $60 = HEAP32[$i>>2]|0;
   $61 = ($60|0)>=(0);
   if (!($61)) {
    break;
   }
   $62 = HEAP32[$1>>2]|0;
   $63 = (($62) + 248|0);
   $64 = (($63) + 24|0);
   $65 = HEAP32[$64>>2]|0;
   $66 = HEAP32[$p>>2]|0;
   HEAP32[$66>>2] = $65;
   $67 = HEAP32[$p>>2]|0;
   $68 = HEAP32[$1>>2]|0;
   $69 = (($68) + 248|0);
   $70 = (($69) + 24|0);
   HEAP32[$70>>2] = $67;
   $71 = HEAP32[$3>>2]|0;
   $72 = HEAP32[$p>>2]|0;
   $73 = (($72) + ($71)|0);
   HEAP32[$p>>2] = $73;
   $74 = HEAP32[$i>>2]|0;
   $75 = (($74) + -1)|0;
   HEAP32[$i>>2] = $75;
  }
  $76 = HEAP32[$p>>2]|0;
  $77 = HEAP32[$1>>2]|0;
  $78 = (($77) + 248|0);
  $79 = (($78) + 32|0);
  HEAP32[$79>>2] = $76;
  $80 = HEAP32[$1>>2]|0;
  $81 = (($80) + 248|0);
  $82 = (($81) + 2|0);
  HEAP8[$82>>0] = 1;
  $83 = HEAP32[$2>>2]|0;
  $84 = ($83|0)==(0|0);
  $85 = $84 ? 1 : 0;
  $86 = $85&255;
  $87 = HEAP32[$1>>2]|0;
  $88 = (($87) + 248|0);
  $89 = (($88) + 3|0);
  HEAP8[$89>>0] = $86;
 } else {
  $90 = HEAP32[$1>>2]|0;
  $91 = HEAP32[$1>>2]|0;
  $92 = (($91) + 248|0);
  $93 = (($92) + 28|0);
  HEAP32[$93>>2] = $90;
  $94 = HEAP32[$1>>2]|0;
  $95 = HEAP32[$1>>2]|0;
  $96 = (($95) + 248|0);
  $97 = (($96) + 32|0);
  HEAP32[$97>>2] = $94;
  $98 = HEAP32[$1>>2]|0;
  $99 = (($98) + 248|0);
  $100 = (($99) + 2|0);
  HEAP8[$100>>0] = 0;
  $101 = HEAP32[$1>>2]|0;
  $102 = (($101) + 248|0);
  $103 = (($102) + 3|0);
  HEAP8[$103>>0] = 0;
 }
 HEAP32[$0>>2] = 0;
 $104 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($104|0);
}
function _sqlite3_last_insert_rowid($db) {
 $db = $db|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 HEAP32[$0>>2] = $db;
 $1 = HEAP32[$0>>2]|0;
 $2 = (($1) + 32|0);
 $3 = $2;
 $4 = $3;
 $5 = HEAP32[$4>>2]|0;
 $6 = (($3) + 4)|0;
 $7 = $6;
 $8 = HEAP32[$7>>2]|0;
 tempRet0 = $8;
 STACKTOP = sp;return ($5|0);
}
function _sqlite3_changes($db) {
 $db = $db|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 HEAP32[$0>>2] = $db;
 $1 = HEAP32[$0>>2]|0;
 $2 = (($1) + 80|0);
 $3 = HEAP32[$2>>2]|0;
 STACKTOP = sp;return ($3|0);
}
function _sqlite3_total_changes($db) {
 $db = $db|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 HEAP32[$0>>2] = $db;
 $1 = HEAP32[$0>>2]|0;
 $2 = (($1) + 84|0);
 $3 = HEAP32[$2>>2]|0;
 STACKTOP = sp;return ($3|0);
}
function _sqlite3_close($db) {
 $db = $db|0;
 var $0 = 0, $1 = 0, $2 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 HEAP32[$0>>2] = $db;
 $1 = HEAP32[$0>>2]|0;
 $2 = (_sqlite3Close($1,0)|0);
 STACKTOP = sp;return ($2|0);
}
function _sqlite3Close($db,$forceZombie) {
 $db = $db|0;
 $forceZombie = $forceZombie|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0;
 var $9 = 0, $vararg_buffer = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $vararg_buffer = sp;
 $0 = sp + 12|0;
 $1 = sp + 8|0;
 $2 = sp + 4|0;
 HEAP32[$1>>2] = $db;
 HEAP32[$2>>2] = $forceZombie;
 $3 = HEAP32[$1>>2]|0;
 $4 = ($3|0)!=(0|0);
 if (!($4)) {
  HEAP32[$0>>2] = 0;
  $20 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($20|0);
 }
 $5 = HEAP32[$1>>2]|0;
 $6 = (_sqlite3SafetyCheckSickOrOk($5)|0);
 $7 = ($6|0)!=(0);
 if (!($7)) {
  $8 = (_sqlite3MisuseError(125804)|0);
  HEAP32[$0>>2] = $8;
  $20 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($20|0);
 }
 $9 = HEAP32[$1>>2]|0;
 _disconnectAllVtab($9);
 $10 = HEAP32[$1>>2]|0;
 (_sqlite3VtabRollback($10)|0);
 $11 = HEAP32[$2>>2]|0;
 $12 = ($11|0)!=(0);
 if (!($12)) {
  $13 = HEAP32[$1>>2]|0;
  $14 = (_connectionIsBusy($13)|0);
  $15 = ($14|0)!=(0);
  if ($15) {
   $16 = HEAP32[$1>>2]|0;
   _sqlite3ErrorWithMsg($16,5,5992,$vararg_buffer);
   HEAP32[$0>>2] = 5;
   $20 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($20|0);
  }
 }
 $17 = HEAP32[$1>>2]|0;
 $18 = (($17) + 76|0);
 HEAP32[$18>>2] = 1691352191;
 $19 = HEAP32[$1>>2]|0;
 _sqlite3LeaveMutexAndCloseZombie($19);
 HEAP32[$0>>2] = 0;
 $20 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($20|0);
}
function _sqlite3_close_v2($db) {
 $db = $db|0;
 var $0 = 0, $1 = 0, $2 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 HEAP32[$0>>2] = $db;
 $1 = HEAP32[$0>>2]|0;
 $2 = (_sqlite3Close($1,1)|0);
 STACKTOP = sp;return ($2|0);
}
function _sqlite3_busy_handler($db,$xBusy,$pArg) {
 $db = $db|0;
 $xBusy = $xBusy|0;
 $pArg = $pArg|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $1 = sp + 4|0;
 $2 = sp;
 HEAP32[$0>>2] = $db;
 HEAP32[$1>>2] = $xBusy;
 HEAP32[$2>>2] = $pArg;
 $3 = HEAP32[$1>>2]|0;
 $4 = HEAP32[$0>>2]|0;
 $5 = (($4) + 444|0);
 HEAP32[$5>>2] = $3;
 $6 = HEAP32[$2>>2]|0;
 $7 = HEAP32[$0>>2]|0;
 $8 = (($7) + 444|0);
 $9 = (($8) + 4|0);
 HEAP32[$9>>2] = $6;
 $10 = HEAP32[$0>>2]|0;
 $11 = (($10) + 444|0);
 $12 = (($11) + 8|0);
 HEAP32[$12>>2] = 0;
 $13 = HEAP32[$0>>2]|0;
 $14 = (($13) + 492|0);
 HEAP32[$14>>2] = 0;
 STACKTOP = sp;return 0;
}
function _sqlite3_busy_timeout($db,$ms) {
 $db = $db|0;
 $ms = $ms|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $1 = sp;
 HEAP32[$0>>2] = $db;
 HEAP32[$1>>2] = $ms;
 $2 = HEAP32[$1>>2]|0;
 $3 = ($2|0)>(0);
 $4 = HEAP32[$0>>2]|0;
 if ($3) {
  $5 = HEAP32[$0>>2]|0;
  (_sqlite3_busy_handler($4,34,$5)|0);
  $6 = HEAP32[$1>>2]|0;
  $7 = HEAP32[$0>>2]|0;
  $8 = (($7) + 492|0);
  HEAP32[$8>>2] = $6;
  STACKTOP = sp;return 0;
 } else {
  (_sqlite3_busy_handler($4,0,0)|0);
  STACKTOP = sp;return 0;
 }
 return 0|0;
}
function _sqliteDefaultBusyCallback($ptr,$count) {
 $ptr = $ptr|0;
 $count = $count|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $db = 0, $timeout = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 16|0;
 $1 = sp + 12|0;
 $2 = sp + 8|0;
 $db = sp + 4|0;
 $timeout = sp;
 HEAP32[$1>>2] = $ptr;
 HEAP32[$2>>2] = $count;
 $3 = HEAP32[$1>>2]|0;
 HEAP32[$db>>2] = $3;
 $4 = HEAP32[$1>>2]|0;
 $5 = (($4) + 492|0);
 $6 = HEAP32[$5>>2]|0;
 HEAP32[$timeout>>2] = $6;
 $7 = HEAP32[$2>>2]|0;
 $8 = (($7) + 1)|0;
 $9 = ($8*1000)|0;
 $10 = HEAP32[$timeout>>2]|0;
 $11 = ($9|0)>($10|0);
 if ($11) {
  HEAP32[$0>>2] = 0;
  $14 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($14|0);
 } else {
  $12 = HEAP32[$db>>2]|0;
  $13 = HEAP32[$12>>2]|0;
  (_sqlite3OsSleep($13,1000000)|0);
  HEAP32[$0>>2] = 1;
  $14 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($14|0);
 }
 return 0|0;
}
function _sqlite3CreateFunc($db,$zFunctionName,$nArg,$enc,$pUserData,$xFunc,$xStep,$xFinal,$pDestructor) {
 $db = $db|0;
 $zFunctionName = $zFunctionName|0;
 $nArg = $nArg|0;
 $enc = $enc|0;
 $pUserData = $pUserData|0;
 $xFunc = $xFunc|0;
 $xStep = $xStep|0;
 $xFinal = $xFinal|0;
 $pDestructor = $pDestructor|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0;
 var $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0;
 var $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0;
 var $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0;
 var $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0;
 var $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $extraFlags = 0, $nName = 0, $or$cond = 0, $or$cond11 = 0, $or$cond13 = 0, $or$cond3 = 0, $or$cond3$not = 0, $or$cond5 = 0, $or$cond7 = 0, $or$cond7$not = 0, $or$cond9 = 0, $p = 0;
 var $rc = 0, $vararg_buffer = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 64|0;
 $vararg_buffer = sp;
 $0 = sp + 56|0;
 $1 = sp + 52|0;
 $2 = sp + 48|0;
 $3 = sp + 44|0;
 $4 = sp + 40|0;
 $5 = sp + 36|0;
 $6 = sp + 32|0;
 $7 = sp + 28|0;
 $8 = sp + 24|0;
 $9 = sp + 20|0;
 $p = sp + 16|0;
 $nName = sp + 12|0;
 $extraFlags = sp + 8|0;
 $rc = sp + 4|0;
 HEAP32[$1>>2] = $db;
 HEAP32[$2>>2] = $zFunctionName;
 HEAP32[$3>>2] = $nArg;
 HEAP32[$4>>2] = $enc;
 HEAP32[$5>>2] = $pUserData;
 HEAP32[$6>>2] = $xFunc;
 HEAP32[$7>>2] = $xStep;
 HEAP32[$8>>2] = $xFinal;
 HEAP32[$9>>2] = $pDestructor;
 $10 = HEAP32[$2>>2]|0;
 $11 = ($10|0)==(0|0);
 do {
  if (!($11)) {
   $12 = HEAP32[$6>>2]|0;
   $13 = ($12|0)!=(0|0);
   if ($13) {
    $14 = HEAP32[$8>>2]|0;
    $15 = ($14|0)!=(0|0);
    $16 = HEAP32[$7>>2]|0;
    $17 = ($16|0)!=(0|0);
    $or$cond = $15 | $17;
    if ($or$cond) {
     break;
    }
   }
   $18 = HEAP32[$6>>2]|0;
   $19 = ($18|0)==(0|0);
   $20 = HEAP32[$8>>2]|0;
   $21 = ($20|0)!=(0|0);
   $or$cond3 = $19 & $21;
   $or$cond3$not = $or$cond3 ^ 1;
   $22 = HEAP32[$7>>2]|0;
   $23 = ($22|0)!=(0|0);
   $or$cond5 = $or$cond3$not | $23;
   if ($or$cond5) {
    $24 = HEAP32[$6>>2]|0;
    $25 = ($24|0)!=(0|0);
    $26 = HEAP32[$8>>2]|0;
    $27 = ($26|0)!=(0|0);
    $or$cond7 = $25 | $27;
    $or$cond7$not = $or$cond7 ^ 1;
    $28 = HEAP32[$7>>2]|0;
    $29 = ($28|0)!=(0|0);
    $or$cond9 = $or$cond7$not & $29;
    $30 = HEAP32[$3>>2]|0;
    $31 = ($30|0)<(-1);
    $or$cond11 = $or$cond9 | $31;
    $32 = HEAP32[$3>>2]|0;
    $33 = ($32|0)>(127);
    $or$cond13 = $or$cond11 | $33;
    if (!($or$cond13)) {
     $34 = HEAP32[$2>>2]|0;
     $35 = (_sqlite3Strlen30($34)|0);
     HEAP32[$nName>>2] = $35;
     $36 = (255)<($35|0);
     if (!($36)) {
      $38 = HEAP32[$4>>2]|0;
      $39 = $38 & 2048;
      HEAP32[$extraFlags>>2] = $39;
      $40 = HEAP32[$4>>2]|0;
      $41 = $40 & 7;
      HEAP32[$4>>2] = $41;
      $42 = HEAP32[$4>>2]|0;
      $43 = ($42|0)==(4);
      do {
       if ($43) {
        $44 = HEAP8[1136>>0]|0;
        $45 = $44 << 24 >> 24;
        $46 = ($45|0)==(0);
        $47 = $46 ? 3 : 2;
        HEAP32[$4>>2] = $47;
       } else {
        $48 = HEAP32[$4>>2]|0;
        $49 = ($48|0)==(5);
        if ($49) {
         $50 = HEAP32[$1>>2]|0;
         $51 = HEAP32[$2>>2]|0;
         $52 = HEAP32[$3>>2]|0;
         $53 = HEAP32[$extraFlags>>2]|0;
         $54 = 1 | $53;
         $55 = HEAP32[$5>>2]|0;
         $56 = HEAP32[$6>>2]|0;
         $57 = HEAP32[$7>>2]|0;
         $58 = HEAP32[$8>>2]|0;
         $59 = HEAP32[$9>>2]|0;
         $60 = (_sqlite3CreateFunc($50,$51,$52,$54,$55,$56,$57,$58,$59)|0);
         HEAP32[$rc>>2] = $60;
         $61 = HEAP32[$rc>>2]|0;
         $62 = ($61|0)==(0);
         if ($62) {
          $63 = HEAP32[$1>>2]|0;
          $64 = HEAP32[$2>>2]|0;
          $65 = HEAP32[$3>>2]|0;
          $66 = HEAP32[$extraFlags>>2]|0;
          $67 = 2 | $66;
          $68 = HEAP32[$5>>2]|0;
          $69 = HEAP32[$6>>2]|0;
          $70 = HEAP32[$7>>2]|0;
          $71 = HEAP32[$8>>2]|0;
          $72 = HEAP32[$9>>2]|0;
          $73 = (_sqlite3CreateFunc($63,$64,$65,$67,$68,$69,$70,$71,$72)|0);
          HEAP32[$rc>>2] = $73;
         }
         $74 = HEAP32[$rc>>2]|0;
         $75 = ($74|0)!=(0);
         if (!($75)) {
          HEAP32[$4>>2] = 3;
          break;
         }
         $76 = HEAP32[$rc>>2]|0;
         HEAP32[$0>>2] = $76;
         $147 = HEAP32[$0>>2]|0;
         STACKTOP = sp;return ($147|0);
        }
       }
      } while(0);
      $77 = HEAP32[$1>>2]|0;
      $78 = HEAP32[$2>>2]|0;
      $79 = HEAP32[$nName>>2]|0;
      $80 = HEAP32[$3>>2]|0;
      $81 = HEAP32[$4>>2]|0;
      $82 = $81&255;
      $83 = (_sqlite3FindFunction($77,$78,$79,$80,$82,0)|0);
      HEAP32[$p>>2] = $83;
      $84 = HEAP32[$p>>2]|0;
      $85 = ($84|0)!=(0|0);
      do {
       if ($85) {
        $86 = HEAP32[$p>>2]|0;
        $87 = (($86) + 2|0);
        $88 = HEAP16[$87>>1]|0;
        $89 = $88&65535;
        $90 = $89 & 3;
        $91 = HEAP32[$4>>2]|0;
        $92 = ($90|0)==($91|0);
        if ($92) {
         $93 = HEAP32[$p>>2]|0;
         $94 = HEAP16[$93>>1]|0;
         $95 = $94 << 16 >> 16;
         $96 = HEAP32[$3>>2]|0;
         $97 = ($95|0)==($96|0);
         if ($97) {
          $98 = HEAP32[$1>>2]|0;
          $99 = (($98) + 148|0);
          $100 = HEAP32[$99>>2]|0;
          $101 = ($100|0)!=(0);
          $102 = HEAP32[$1>>2]|0;
          if (!($101)) {
           _sqlite3ExpirePreparedStatements($102);
           break;
          }
          _sqlite3ErrorWithMsg($102,5,5048,$vararg_buffer);
          HEAP32[$0>>2] = 5;
          $147 = HEAP32[$0>>2]|0;
          STACKTOP = sp;return ($147|0);
         }
        }
       }
      } while(0);
      $103 = HEAP32[$1>>2]|0;
      $104 = HEAP32[$2>>2]|0;
      $105 = HEAP32[$nName>>2]|0;
      $106 = HEAP32[$3>>2]|0;
      $107 = HEAP32[$4>>2]|0;
      $108 = $107&255;
      $109 = (_sqlite3FindFunction($103,$104,$105,$106,$108,1)|0);
      HEAP32[$p>>2] = $109;
      $110 = HEAP32[$p>>2]|0;
      $111 = ($110|0)!=(0|0);
      if (!($111)) {
       HEAP32[$0>>2] = 7;
       $147 = HEAP32[$0>>2]|0;
       STACKTOP = sp;return ($147|0);
      }
      $112 = HEAP32[$1>>2]|0;
      $113 = HEAP32[$p>>2]|0;
      _functionDestroy($112,$113);
      $114 = HEAP32[$9>>2]|0;
      $115 = ($114|0)!=(0|0);
      if ($115) {
       $116 = HEAP32[$9>>2]|0;
       $117 = HEAP32[$116>>2]|0;
       $118 = (($117) + 1)|0;
       HEAP32[$116>>2] = $118;
      }
      $119 = HEAP32[$9>>2]|0;
      $120 = HEAP32[$p>>2]|0;
      $121 = (($120) + 32|0);
      HEAP32[$121>>2] = $119;
      $122 = HEAP32[$p>>2]|0;
      $123 = (($122) + 2|0);
      $124 = HEAP16[$123>>1]|0;
      $125 = $124&65535;
      $126 = $125 & 3;
      $127 = HEAP32[$extraFlags>>2]|0;
      $128 = $126 | $127;
      $129 = $128&65535;
      $130 = HEAP32[$p>>2]|0;
      $131 = (($130) + 2|0);
      HEAP16[$131>>1] = $129;
      $132 = HEAP32[$6>>2]|0;
      $133 = HEAP32[$p>>2]|0;
      $134 = (($133) + 12|0);
      HEAP32[$134>>2] = $132;
      $135 = HEAP32[$7>>2]|0;
      $136 = HEAP32[$p>>2]|0;
      $137 = (($136) + 16|0);
      HEAP32[$137>>2] = $135;
      $138 = HEAP32[$8>>2]|0;
      $139 = HEAP32[$p>>2]|0;
      $140 = (($139) + 20|0);
      HEAP32[$140>>2] = $138;
      $141 = HEAP32[$5>>2]|0;
      $142 = HEAP32[$p>>2]|0;
      $143 = (($142) + 4|0);
      HEAP32[$143>>2] = $141;
      $144 = HEAP32[$3>>2]|0;
      $145 = $144&65535;
      $146 = HEAP32[$p>>2]|0;
      HEAP16[$146>>1] = $145;
      HEAP32[$0>>2] = 0;
      $147 = HEAP32[$0>>2]|0;
      STACKTOP = sp;return ($147|0);
     }
    }
   }
  }
 } while(0);
 $37 = (_sqlite3MisuseError(126362)|0);
 HEAP32[$0>>2] = $37;
 $147 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($147|0);
}
function _sqlite3_overload_function($db,$zName,$nArg) {
 $db = $db|0;
 $zName = $zName|0;
 $nArg = $nArg|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $nName = 0;
 var $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 16|0;
 $1 = sp + 12|0;
 $2 = sp + 8|0;
 $nName = sp + 4|0;
 $rc = sp;
 HEAP32[$0>>2] = $db;
 HEAP32[$1>>2] = $zName;
 HEAP32[$2>>2] = $nArg;
 $3 = HEAP32[$1>>2]|0;
 $4 = (_sqlite3Strlen30($3)|0);
 HEAP32[$nName>>2] = $4;
 HEAP32[$rc>>2] = 0;
 $5 = HEAP32[$0>>2]|0;
 $6 = HEAP32[$1>>2]|0;
 $7 = HEAP32[$nName>>2]|0;
 $8 = HEAP32[$2>>2]|0;
 $9 = (_sqlite3FindFunction($5,$6,$7,$8,1,0)|0);
 $10 = ($9|0)==(0|0);
 if ($10) {
  $11 = HEAP32[$0>>2]|0;
  $12 = HEAP32[$1>>2]|0;
  $13 = HEAP32[$2>>2]|0;
  $14 = (_sqlite3CreateFunc($11,$12,$13,1,0,51,0,0,0)|0);
  HEAP32[$rc>>2] = $14;
 }
 $15 = HEAP32[$0>>2]|0;
 $16 = HEAP32[$rc>>2]|0;
 $17 = (_sqlite3ApiExit($15,$16)|0);
 HEAP32[$rc>>2] = $17;
 $18 = HEAP32[$rc>>2]|0;
 STACKTOP = sp;return ($18|0);
}
function _sqlite3FindFunction($db,$zName,$nName,$nArg,$enc,$createFlag) {
 $db = $db|0;
 $zName = $zName|0;
 $nName = $nName|0;
 $nArg = $nArg|0;
 $enc = $enc|0;
 $createFlag = $createFlag|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $12 = 0, $13 = 0;
 var $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0;
 var $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0;
 var $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0;
 var $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0;
 var $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $bestScore = 0, $h = 0, $or$cond = 0, $p = 0, $pBest = 0, $pHash = 0;
 var $score = 0, $score1 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 64|0;
 $0 = sp + 44|0;
 $1 = sp + 40|0;
 $2 = sp + 36|0;
 $3 = sp + 32|0;
 $4 = sp + 28|0;
 $5 = sp + 49|0;
 $6 = sp + 48|0;
 $p = sp + 24|0;
 $pBest = sp + 20|0;
 $bestScore = sp + 16|0;
 $h = sp + 12|0;
 $score = sp + 8|0;
 $pHash = sp + 4|0;
 $score1 = sp;
 HEAP32[$1>>2] = $db;
 HEAP32[$2>>2] = $zName;
 HEAP32[$3>>2] = $nName;
 HEAP32[$4>>2] = $nArg;
 HEAP8[$5>>0] = $enc;
 HEAP8[$6>>0] = $createFlag;
 HEAP32[$pBest>>2] = 0;
 HEAP32[$bestScore>>2] = 0;
 $7 = HEAP32[$2>>2]|0;
 $8 = HEAP8[$7>>0]|0;
 $9 = $8&255;
 $10 = (440 + ($9)|0);
 $11 = HEAP8[$10>>0]|0;
 $12 = $11&255;
 $13 = HEAP32[$3>>2]|0;
 $14 = (($12) + ($13))|0;
 $15 = (($14|0) % 23)&-1;
 HEAP32[$h>>2] = $15;
 $16 = HEAP32[$1>>2]|0;
 $17 = (($16) + 336|0);
 $18 = HEAP32[$h>>2]|0;
 $19 = HEAP32[$2>>2]|0;
 $20 = HEAP32[$3>>2]|0;
 $21 = (_functionSearch($17,$18,$19,$20)|0);
 HEAP32[$p>>2] = $21;
 while(1) {
  $22 = HEAP32[$p>>2]|0;
  $23 = ($22|0)!=(0|0);
  if (!($23)) {
   break;
  }
  $24 = HEAP32[$p>>2]|0;
  $25 = HEAP32[$4>>2]|0;
  $26 = HEAP8[$5>>0]|0;
  $27 = (_matchQuality($24,$25,$26)|0);
  HEAP32[$score>>2] = $27;
  $28 = HEAP32[$score>>2]|0;
  $29 = HEAP32[$bestScore>>2]|0;
  $30 = ($28|0)>($29|0);
  if ($30) {
   $31 = HEAP32[$p>>2]|0;
   HEAP32[$pBest>>2] = $31;
   $32 = HEAP32[$score>>2]|0;
   HEAP32[$bestScore>>2] = $32;
  }
  $33 = HEAP32[$p>>2]|0;
  $34 = (($33) + 8|0);
  $35 = HEAP32[$34>>2]|0;
  HEAP32[$p>>2] = $35;
 }
 $36 = HEAP8[$6>>0]|0;
 $37 = ($36<<24>>24)!=(0);
 L8: do {
  if (!($37)) {
   $38 = HEAP32[$pBest>>2]|0;
   $39 = ($38|0)==(0|0);
   if (!($39)) {
    $40 = HEAP32[$1>>2]|0;
    $41 = (($40) + 24|0);
    $42 = HEAP32[$41>>2]|0;
    $43 = $42 & 2097152;
    $44 = ($43|0)!=(0);
    if (!($44)) {
     break;
    }
   }
   HEAP32[$pHash>>2] = 1576;
   HEAP32[$bestScore>>2] = 0;
   $45 = HEAP32[$pHash>>2]|0;
   $46 = HEAP32[$h>>2]|0;
   $47 = HEAP32[$2>>2]|0;
   $48 = HEAP32[$3>>2]|0;
   $49 = (_functionSearch($45,$46,$47,$48)|0);
   HEAP32[$p>>2] = $49;
   while(1) {
    $50 = HEAP32[$p>>2]|0;
    $51 = ($50|0)!=(0|0);
    if (!($51)) {
     break L8;
    }
    $52 = HEAP32[$p>>2]|0;
    $53 = HEAP32[$4>>2]|0;
    $54 = HEAP8[$5>>0]|0;
    $55 = (_matchQuality($52,$53,$54)|0);
    HEAP32[$score1>>2] = $55;
    $56 = HEAP32[$score1>>2]|0;
    $57 = HEAP32[$bestScore>>2]|0;
    $58 = ($56|0)>($57|0);
    if ($58) {
     $59 = HEAP32[$p>>2]|0;
     HEAP32[$pBest>>2] = $59;
     $60 = HEAP32[$score1>>2]|0;
     HEAP32[$bestScore>>2] = $60;
    }
    $61 = HEAP32[$p>>2]|0;
    $62 = (($61) + 8|0);
    $63 = HEAP32[$62>>2]|0;
    HEAP32[$p>>2] = $63;
   }
  }
 } while(0);
 $64 = HEAP8[$6>>0]|0;
 $65 = $64&255;
 $66 = ($65|0)!=(0);
 $67 = HEAP32[$bestScore>>2]|0;
 $68 = ($67|0)<(6);
 $or$cond = $66 & $68;
 if ($or$cond) {
  $69 = HEAP32[$1>>2]|0;
  $70 = HEAP32[$3>>2]|0;
  $71 = (36 + ($70))|0;
  $72 = (($71) + 1)|0;
  $73 = (_sqlite3DbMallocZero($69,$72,0)|0);
  HEAP32[$pBest>>2] = $73;
  $74 = ($73|0)!=(0|0);
  if ($74) {
   $75 = HEAP32[$pBest>>2]|0;
   $76 = (($75) + 36|0);
   $77 = HEAP32[$pBest>>2]|0;
   $78 = (($77) + 24|0);
   HEAP32[$78>>2] = $76;
   $79 = HEAP32[$4>>2]|0;
   $80 = $79&65535;
   $81 = HEAP32[$pBest>>2]|0;
   HEAP16[$81>>1] = $80;
   $82 = HEAP8[$5>>0]|0;
   $83 = $82&255;
   $84 = HEAP32[$pBest>>2]|0;
   $85 = (($84) + 2|0);
   HEAP16[$85>>1] = $83;
   $86 = HEAP32[$pBest>>2]|0;
   $87 = (($86) + 24|0);
   $88 = HEAP32[$87>>2]|0;
   $89 = HEAP32[$2>>2]|0;
   $90 = HEAP32[$3>>2]|0;
   _memcpy(($88|0),($89|0),($90|0))|0;
   $91 = HEAP32[$3>>2]|0;
   $92 = HEAP32[$pBest>>2]|0;
   $93 = (($92) + 24|0);
   $94 = HEAP32[$93>>2]|0;
   $95 = (($94) + ($91)|0);
   HEAP8[$95>>0] = 0;
   $96 = HEAP32[$1>>2]|0;
   $97 = (($96) + 336|0);
   $98 = HEAP32[$pBest>>2]|0;
   _sqlite3FuncDefInsert($97,$98);
  }
 }
 $99 = HEAP32[$pBest>>2]|0;
 $100 = ($99|0)!=(0|0);
 do {
  if ($100) {
   $101 = HEAP32[$pBest>>2]|0;
   $102 = (($101) + 16|0);
   $103 = HEAP32[$102>>2]|0;
   $104 = ($103|0)!=(0|0);
   if (!($104)) {
    $105 = HEAP32[$pBest>>2]|0;
    $106 = (($105) + 12|0);
    $107 = HEAP32[$106>>2]|0;
    $108 = ($107|0)!=(0|0);
    if (!($108)) {
     $109 = HEAP8[$6>>0]|0;
     $110 = $109&255;
     $111 = ($110|0)!=(0);
     if (!($111)) {
      break;
     }
    }
   }
   $112 = HEAP32[$pBest>>2]|0;
   HEAP32[$0>>2] = $112;
   $113 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($113|0);
  }
 } while(0);
 HEAP32[$0>>2] = 0;
 $113 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($113|0);
}
function _sqlite3InvalidFunction($context,$NotUsed,$NotUsed2) {
 $context = $context|0;
 $NotUsed = $NotUsed|0;
 $NotUsed2 = $NotUsed2|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $vararg_buffer = 0, $zErr = 0, $zName = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $vararg_buffer = sp;
 $0 = sp + 20|0;
 $1 = sp + 16|0;
 $2 = sp + 12|0;
 $zName = sp + 8|0;
 $zErr = sp + 4|0;
 HEAP32[$0>>2] = $context;
 HEAP32[$1>>2] = $NotUsed;
 HEAP32[$2>>2] = $NotUsed2;
 $3 = HEAP32[$0>>2]|0;
 $4 = (($3) + 4|0);
 $5 = HEAP32[$4>>2]|0;
 $6 = (($5) + 24|0);
 $7 = HEAP32[$6>>2]|0;
 HEAP32[$zName>>2] = $7;
 $8 = HEAP32[$zName>>2]|0;
 HEAP32[$vararg_buffer>>2] = $8;
 $9 = (_sqlite3_mprintf(4928,$vararg_buffer)|0);
 HEAP32[$zErr>>2] = $9;
 $10 = HEAP32[$0>>2]|0;
 $11 = HEAP32[$zErr>>2]|0;
 _sqlite3_result_error($10,$11,-1);
 $12 = HEAP32[$zErr>>2]|0;
 _sqlite3_free($12);
 STACKTOP = sp;return;
}
function _sqlite3_wal_autocheckpoint($db,$nFrame) {
 $db = $db|0;
 $nFrame = $nFrame|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $1 = sp;
 HEAP32[$0>>2] = $db;
 HEAP32[$1>>2] = $nFrame;
 $2 = HEAP32[$1>>2]|0;
 $3 = ($2|0)>(0);
 $4 = HEAP32[$0>>2]|0;
 if ($3) {
  $5 = HEAP32[$1>>2]|0;
  $6 = $5;
  (_sqlite3_wal_hook($4,5,$6)|0);
  STACKTOP = sp;return 0;
 } else {
  (_sqlite3_wal_hook($4,0,0)|0);
  STACKTOP = sp;return 0;
 }
 return 0|0;
}
function _sqlite3_wal_hook($db,$xCallback,$pArg) {
 $db = $db|0;
 $xCallback = $xCallback|0;
 $pArg = $pArg|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $pRet = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $1 = sp + 8|0;
 $2 = sp + 4|0;
 $pRet = sp;
 HEAP32[$0>>2] = $db;
 HEAP32[$1>>2] = $xCallback;
 HEAP32[$2>>2] = $pArg;
 $3 = HEAP32[$0>>2]|0;
 $4 = (($3) + 216|0);
 $5 = HEAP32[$4>>2]|0;
 HEAP32[$pRet>>2] = $5;
 $6 = HEAP32[$1>>2]|0;
 $7 = HEAP32[$0>>2]|0;
 $8 = (($7) + 212|0);
 HEAP32[$8>>2] = $6;
 $9 = HEAP32[$2>>2]|0;
 $10 = HEAP32[$0>>2]|0;
 $11 = (($10) + 216|0);
 HEAP32[$11>>2] = $9;
 $12 = HEAP32[$pRet>>2]|0;
 STACKTOP = sp;return ($12|0);
}
function _sqlite3WalDefaultHook($pClientData,$db,$zDb,$nFrame) {
 $pClientData = $pClientData|0;
 $db = $db|0;
 $zDb = $zDb|0;
 $nFrame = $nFrame|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $1 = sp + 8|0;
 $2 = sp + 4|0;
 $3 = sp;
 HEAP32[$0>>2] = $pClientData;
 HEAP32[$1>>2] = $db;
 HEAP32[$2>>2] = $zDb;
 HEAP32[$3>>2] = $nFrame;
 $4 = HEAP32[$3>>2]|0;
 $5 = HEAP32[$0>>2]|0;
 $6 = $5;
 $7 = ($4|0)>=($6|0);
 if (!($7)) {
  STACKTOP = sp;return 0;
 }
 _sqlite3BeginBenignMalloc();
 $8 = HEAP32[$1>>2]|0;
 $9 = HEAP32[$2>>2]|0;
 (_sqlite3_wal_checkpoint($8,$9)|0);
 _sqlite3EndBenignMalloc();
 STACKTOP = sp;return 0;
}
function _sqlite3_wal_checkpoint_v2($db,$zDb,$eMode,$pnLog,$pnCkpt) {
 $db = $db|0;
 $zDb = $zDb|0;
 $eMode = $eMode|0;
 $pnLog = $pnLog|0;
 $pnCkpt = $pnCkpt|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $5 = 0, $6 = 0, $7 = 0;
 var $8 = 0, $9 = 0, $iDb = 0, $or$cond = 0, $rc = 0, $vararg_buffer = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $vararg_buffer = sp;
 $0 = sp + 32|0;
 $1 = sp + 28|0;
 $2 = sp + 24|0;
 $3 = sp + 20|0;
 $4 = sp + 16|0;
 $5 = sp + 12|0;
 $rc = sp + 8|0;
 $iDb = sp + 4|0;
 HEAP32[$1>>2] = $db;
 HEAP32[$2>>2] = $zDb;
 HEAP32[$3>>2] = $eMode;
 HEAP32[$4>>2] = $pnLog;
 HEAP32[$5>>2] = $pnCkpt;
 HEAP32[$iDb>>2] = 10;
 $6 = HEAP32[$4>>2]|0;
 $7 = ($6|0)!=(0|0);
 if ($7) {
  $8 = HEAP32[$4>>2]|0;
  HEAP32[$8>>2] = -1;
 }
 $9 = HEAP32[$5>>2]|0;
 $10 = ($9|0)!=(0|0);
 if ($10) {
  $11 = HEAP32[$5>>2]|0;
  HEAP32[$11>>2] = -1;
 }
 $12 = HEAP32[$3>>2]|0;
 $13 = ($12|0)<(0);
 $14 = HEAP32[$3>>2]|0;
 $15 = ($14|0)>(2);
 $or$cond = $13 | $15;
 if ($or$cond) {
  HEAP32[$0>>2] = 21;
  $41 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($41|0);
 }
 $16 = HEAP32[$2>>2]|0;
 $17 = ($16|0)!=(0|0);
 if ($17) {
  $18 = HEAP32[$2>>2]|0;
  $19 = HEAP8[$18>>0]|0;
  $20 = $19 << 24 >> 24;
  $21 = ($20|0)!=(0);
  if ($21) {
   $22 = HEAP32[$1>>2]|0;
   $23 = HEAP32[$2>>2]|0;
   $24 = (_sqlite3FindDbName($22,$23)|0);
   HEAP32[$iDb>>2] = $24;
  }
 }
 $25 = HEAP32[$iDb>>2]|0;
 $26 = ($25|0)<(0);
 if ($26) {
  HEAP32[$rc>>2] = 1;
  $27 = HEAP32[$1>>2]|0;
  $28 = HEAP32[$2>>2]|0;
  HEAP32[$vararg_buffer>>2] = $28;
  _sqlite3ErrorWithMsg($27,1,1672,$vararg_buffer);
 } else {
  $29 = HEAP32[$1>>2]|0;
  $30 = HEAP32[$iDb>>2]|0;
  $31 = HEAP32[$3>>2]|0;
  $32 = HEAP32[$4>>2]|0;
  $33 = HEAP32[$5>>2]|0;
  $34 = (_sqlite3Checkpoint($29,$30,$31,$32,$33)|0);
  HEAP32[$rc>>2] = $34;
  $35 = HEAP32[$1>>2]|0;
  $36 = HEAP32[$rc>>2]|0;
  _sqlite3Error($35,$36);
 }
 $37 = HEAP32[$1>>2]|0;
 $38 = HEAP32[$rc>>2]|0;
 $39 = (_sqlite3ApiExit($37,$38)|0);
 HEAP32[$rc>>2] = $39;
 $40 = HEAP32[$rc>>2]|0;
 HEAP32[$0>>2] = $40;
 $41 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($41|0);
}
function _sqlite3FindDbName($db,$zName) {
 $db = $db|0;
 $zName = $zName|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $i = 0, $n = 0, $pDb = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 16|0;
 $1 = sp + 12|0;
 $i = sp + 8|0;
 $pDb = sp + 4|0;
 $n = sp;
 HEAP32[$0>>2] = $db;
 HEAP32[$1>>2] = $zName;
 HEAP32[$i>>2] = -1;
 $2 = HEAP32[$1>>2]|0;
 $3 = ($2|0)!=(0|0);
 if (!($3)) {
  $31 = HEAP32[$i>>2]|0;
  STACKTOP = sp;return ($31|0);
 }
 $4 = HEAP32[$1>>2]|0;
 $5 = (_sqlite3Strlen30($4)|0);
 HEAP32[$n>>2] = $5;
 $6 = HEAP32[$0>>2]|0;
 $7 = (($6) + 20|0);
 $8 = HEAP32[$7>>2]|0;
 $9 = (($8) - 1)|0;
 HEAP32[$i>>2] = $9;
 $10 = HEAP32[$i>>2]|0;
 $11 = HEAP32[$0>>2]|0;
 $12 = (($11) + 16|0);
 $13 = HEAP32[$12>>2]|0;
 $14 = (($13) + ($10<<4)|0);
 HEAP32[$pDb>>2] = $14;
 while(1) {
  $15 = HEAP32[$i>>2]|0;
  $16 = ($15|0)>=(0);
  if (!($16)) {
   label = 7;
   break;
  }
  $17 = HEAP32[$n>>2]|0;
  $18 = HEAP32[$pDb>>2]|0;
  $19 = HEAP32[$18>>2]|0;
  $20 = (_sqlite3Strlen30($19)|0);
  $21 = ($17|0)==($20|0);
  if ($21) {
   $22 = HEAP32[$pDb>>2]|0;
   $23 = HEAP32[$22>>2]|0;
   $24 = HEAP32[$1>>2]|0;
   $25 = (_sqlite3_stricmp($23,$24)|0);
   $26 = (0)==($25|0);
   if ($26) {
    label = 7;
    break;
   }
  }
  $27 = HEAP32[$i>>2]|0;
  $28 = (($27) + -1)|0;
  HEAP32[$i>>2] = $28;
  $29 = HEAP32[$pDb>>2]|0;
  $30 = (($29) + -16|0);
  HEAP32[$pDb>>2] = $30;
 }
 if ((label|0) == 7) {
  $31 = HEAP32[$i>>2]|0;
  STACKTOP = sp;return ($31|0);
 }
 return 0|0;
}
function _sqlite3Checkpoint($db,$iDb,$eMode,$pnLog,$pnCkpt) {
 $db = $db|0;
 $iDb = $iDb|0;
 $eMode = $eMode|0;
 $pnLog = $pnLog|0;
 $pnCkpt = $pnCkpt|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $bBusy = 0, $i = 0;
 var $or$cond = 0, $or$cond3 = 0, $or$cond5 = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 28|0;
 $1 = sp + 24|0;
 $2 = sp + 20|0;
 $3 = sp + 16|0;
 $4 = sp + 12|0;
 $rc = sp + 8|0;
 $i = sp + 4|0;
 $bBusy = sp;
 HEAP32[$0>>2] = $db;
 HEAP32[$1>>2] = $iDb;
 HEAP32[$2>>2] = $eMode;
 HEAP32[$3>>2] = $pnLog;
 HEAP32[$4>>2] = $pnCkpt;
 HEAP32[$rc>>2] = 0;
 HEAP32[$bBusy>>2] = 0;
 HEAP32[$i>>2] = 0;
 while(1) {
  $5 = HEAP32[$i>>2]|0;
  $6 = HEAP32[$0>>2]|0;
  $7 = (($6) + 20|0);
  $8 = HEAP32[$7>>2]|0;
  $9 = ($5|0)<($8|0);
  $10 = HEAP32[$rc>>2]|0;
  $11 = ($10|0)==(0);
  $or$cond = $9 & $11;
  if (!($or$cond)) {
   break;
  }
  $12 = HEAP32[$i>>2]|0;
  $13 = HEAP32[$1>>2]|0;
  $14 = ($12|0)==($13|0);
  $15 = HEAP32[$1>>2]|0;
  $16 = ($15|0)==(10);
  $or$cond3 = $14 | $16;
  if ($or$cond3) {
   $17 = HEAP32[$i>>2]|0;
   $18 = HEAP32[$0>>2]|0;
   $19 = (($18) + 16|0);
   $20 = HEAP32[$19>>2]|0;
   $21 = (($20) + ($17<<4)|0);
   $22 = (($21) + 4|0);
   $23 = HEAP32[$22>>2]|0;
   $24 = HEAP32[$2>>2]|0;
   $25 = HEAP32[$3>>2]|0;
   $26 = HEAP32[$4>>2]|0;
   $27 = (_sqlite3BtreeCheckpoint($23,$24,$25,$26)|0);
   HEAP32[$rc>>2] = $27;
   HEAP32[$3>>2] = 0;
   HEAP32[$4>>2] = 0;
   $28 = HEAP32[$rc>>2]|0;
   $29 = ($28|0)==(5);
   if ($29) {
    HEAP32[$bBusy>>2] = 1;
    HEAP32[$rc>>2] = 0;
   }
  }
  $30 = HEAP32[$i>>2]|0;
  $31 = (($30) + 1)|0;
  HEAP32[$i>>2] = $31;
 }
 $32 = HEAP32[$rc>>2]|0;
 $33 = ($32|0)==(0);
 $34 = HEAP32[$bBusy>>2]|0;
 $35 = ($34|0)!=(0);
 $or$cond5 = $33 & $35;
 $36 = HEAP32[$rc>>2]|0;
 $37 = $or$cond5 ? 5 : $36;
 STACKTOP = sp;return ($37|0);
}
function _sqlite3_wal_checkpoint($db,$zDb) {
 $db = $db|0;
 $zDb = $zDb|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $1 = sp;
 HEAP32[$0>>2] = $db;
 HEAP32[$1>>2] = $zDb;
 $2 = HEAP32[$0>>2]|0;
 $3 = HEAP32[$1>>2]|0;
 $4 = (_sqlite3_wal_checkpoint_v2($2,$3,0,0,0)|0);
 STACKTOP = sp;return ($4|0);
}
function _sqlite3SafetyCheckSickOrOk($db) {
 $db = $db|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $magic = 0, $or$cond = 0, $or$cond3 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $1 = sp + 4|0;
 $magic = sp;
 HEAP32[$1>>2] = $db;
 $2 = HEAP32[$1>>2]|0;
 $3 = (($2) + 76|0);
 $4 = HEAP32[$3>>2]|0;
 HEAP32[$magic>>2] = $4;
 $5 = HEAP32[$magic>>2]|0;
 $6 = ($5|0)!=(1266094736);
 $7 = HEAP32[$magic>>2]|0;
 $8 = ($7|0)!=(-1607883113);
 $or$cond = $6 & $8;
 $9 = HEAP32[$magic>>2]|0;
 $10 = ($9|0)!=(-264537850);
 $or$cond3 = $or$cond & $10;
 if ($or$cond3) {
  _logBadConnection(4872);
  HEAP32[$0>>2] = 0;
  $11 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($11|0);
 } else {
  HEAP32[$0>>2] = 1;
  $11 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($11|0);
 }
 return 0|0;
}
function _sqlite3_errcode($db) {
 $db = $db|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0;
 var $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $1 = sp;
 HEAP32[$1>>2] = $db;
 $2 = HEAP32[$1>>2]|0;
 $3 = ($2|0)!=(0|0);
 if ($3) {
  $4 = HEAP32[$1>>2]|0;
  $5 = (_sqlite3SafetyCheckSickOrOk($4)|0);
  $6 = ($5|0)!=(0);
  if (!($6)) {
   $7 = (_sqlite3MisuseError(126921)|0);
   HEAP32[$0>>2] = $7;
   $22 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($22|0);
  }
 }
 $8 = HEAP32[$1>>2]|0;
 $9 = ($8|0)!=(0|0);
 if ($9) {
  $10 = HEAP32[$1>>2]|0;
  $11 = (($10) + 64|0);
  $12 = HEAP8[$11>>0]|0;
  $13 = $12&255;
  $14 = ($13|0)!=(0);
  if (!($14)) {
   $15 = HEAP32[$1>>2]|0;
   $16 = (($15) + 52|0);
   $17 = HEAP32[$16>>2]|0;
   $18 = HEAP32[$1>>2]|0;
   $19 = (($18) + 56|0);
   $20 = HEAP32[$19>>2]|0;
   $21 = $17 & $20;
   HEAP32[$0>>2] = $21;
   $22 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($22|0);
  }
 }
 HEAP32[$0>>2] = 7;
 $22 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($22|0);
}
function _sqlite3_limit($db,$limitId,$newLimit) {
 $db = $db|0;
 $limitId = $limitId|0;
 $newLimit = $newLimit|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $oldLimit = 0, $or$cond = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 16|0;
 $1 = sp + 12|0;
 $2 = sp + 8|0;
 $3 = sp + 4|0;
 $oldLimit = sp;
 HEAP32[$1>>2] = $db;
 HEAP32[$2>>2] = $limitId;
 HEAP32[$3>>2] = $newLimit;
 $4 = HEAP32[$2>>2]|0;
 $5 = ($4|0)<(0);
 $6 = HEAP32[$2>>2]|0;
 $7 = ($6|0)>=(12);
 $or$cond = $5 | $7;
 if ($or$cond) {
  HEAP32[$0>>2] = -1;
  $29 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($29|0);
 }
 $8 = HEAP32[$2>>2]|0;
 $9 = HEAP32[$1>>2]|0;
 $10 = (($9) + 88|0);
 $11 = (($10) + ($8<<2)|0);
 $12 = HEAP32[$11>>2]|0;
 HEAP32[$oldLimit>>2] = $12;
 $13 = HEAP32[$3>>2]|0;
 $14 = ($13|0)>=(0);
 if ($14) {
  $15 = HEAP32[$3>>2]|0;
  $16 = HEAP32[$2>>2]|0;
  $17 = (1696 + ($16<<2)|0);
  $18 = HEAP32[$17>>2]|0;
  $19 = ($15|0)>($18|0);
  if ($19) {
   $20 = HEAP32[$2>>2]|0;
   $21 = (1696 + ($20<<2)|0);
   $22 = HEAP32[$21>>2]|0;
   HEAP32[$3>>2] = $22;
  }
  $23 = HEAP32[$3>>2]|0;
  $24 = HEAP32[$2>>2]|0;
  $25 = HEAP32[$1>>2]|0;
  $26 = (($25) + 88|0);
  $27 = (($26) + ($24<<2)|0);
  HEAP32[$27>>2] = $23;
 }
 $28 = HEAP32[$oldLimit>>2]|0;
 HEAP32[$0>>2] = $28;
 $29 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($29|0);
}
function _sqlite3_open($zFilename,$ppDb) {
 $zFilename = $zFilename|0;
 $ppDb = $ppDb|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $1 = sp;
 HEAP32[$0>>2] = $zFilename;
 HEAP32[$1>>2] = $ppDb;
 $2 = HEAP32[$0>>2]|0;
 $3 = HEAP32[$1>>2]|0;
 $4 = (_openDatabase($2,$3,6,0)|0);
 STACKTOP = sp;return ($4|0);
}
function _openDatabase($zFilename,$ppDb,$flags,$zVfs) {
 $zFilename = $zFilename|0;
 $ppDb = $ppDb|0;
 $flags = $flags|0;
 $zVfs = $zVfs|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0;
 var $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0, $188 = 0;
 var $189 = 0, $19 = 0, $190 = 0, $191 = 0, $192 = 0, $193 = 0, $194 = 0, $195 = 0, $196 = 0, $197 = 0, $198 = 0, $199 = 0, $2 = 0, $20 = 0, $200 = 0, $201 = 0, $202 = 0, $203 = 0, $204 = 0, $205 = 0;
 var $206 = 0, $207 = 0, $208 = 0, $209 = 0, $21 = 0, $210 = 0, $211 = 0, $212 = 0, $213 = 0, $214 = 0, $215 = 0, $216 = 0, $217 = 0, $218 = 0, $219 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0;
 var $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0;
 var $db = 0, $isThreadsafe = 0, $rc = 0, $vararg_buffer = 0, $zErrMsg = 0, $zOpen = 0, dest = 0, label = 0, sp = 0, src = 0, stop = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $vararg_buffer = sp;
 $0 = sp + 40|0;
 $1 = sp + 36|0;
 $2 = sp + 32|0;
 $3 = sp + 28|0;
 $4 = sp + 24|0;
 $db = sp + 20|0;
 $rc = sp + 16|0;
 $isThreadsafe = sp + 12|0;
 $zOpen = sp + 8|0;
 $zErrMsg = sp + 4|0;
 HEAP32[$1>>2] = $zFilename;
 HEAP32[$2>>2] = $ppDb;
 HEAP32[$3>>2] = $flags;
 HEAP32[$4>>2] = $zVfs;
 HEAP32[$zOpen>>2] = 0;
 HEAP32[$zErrMsg>>2] = 0;
 $5 = HEAP32[$2>>2]|0;
 HEAP32[$5>>2] = 0;
 $6 = (_sqlite3_initialize()|0);
 HEAP32[$rc>>2] = $6;
 $7 = HEAP32[$rc>>2]|0;
 $8 = ($7|0)!=(0);
 if ($8) {
  $9 = HEAP32[$rc>>2]|0;
  HEAP32[$0>>2] = $9;
  $219 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($219|0);
 }
 $10 = HEAP32[$3>>2]|0;
 $11 = $10 & 7;
 $12 = 1 << $11;
 $13 = $12 & 70;
 $14 = ($13|0)==(0);
 if ($14) {
  $15 = (_sqlite3MisuseError(127429)|0);
  HEAP32[$0>>2] = $15;
  $219 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($219|0);
 }
 $16 = HEAP32[((38640 + 4|0))>>2]|0;
 $17 = ($16|0)==(0);
 do {
  if ($17) {
   HEAP32[$isThreadsafe>>2] = 0;
  } else {
   $18 = HEAP32[$3>>2]|0;
   $19 = $18 & 32768;
   $20 = ($19|0)!=(0);
   if ($20) {
    HEAP32[$isThreadsafe>>2] = 0;
    break;
   }
   $21 = HEAP32[$3>>2]|0;
   $22 = $21 & 65536;
   $23 = ($22|0)!=(0);
   if ($23) {
    HEAP32[$isThreadsafe>>2] = 1;
    break;
   } else {
    $24 = HEAP32[((38640 + 8|0))>>2]|0;
    HEAP32[$isThreadsafe>>2] = $24;
    break;
   }
  }
 } while(0);
 $25 = HEAP32[$3>>2]|0;
 $26 = $25 & 262144;
 $27 = ($26|0)!=(0);
 if ($27) {
  $28 = HEAP32[$3>>2]|0;
  $29 = $28 & -131073;
  HEAP32[$3>>2] = $29;
 } else {
  $30 = HEAP32[((38640 + 220|0))>>2]|0;
  $31 = ($30|0)!=(0);
  if ($31) {
   $32 = HEAP32[$3>>2]|0;
   $33 = $32 | 131072;
   HEAP32[$3>>2] = $33;
  }
 }
 $34 = HEAP32[$3>>2]|0;
 $35 = $34 & -655129;
 HEAP32[$3>>2] = $35;
 $36 = (_sqlite3MallocZero(528,0)|0);
 HEAP32[$db>>2] = $36;
 $37 = HEAP32[$db>>2]|0;
 $38 = ($37|0)==(0|0);
 do {
  if (!($38)) {
   $39 = HEAP32[$isThreadsafe>>2]|0;
   $40 = ($39|0)!=(0);
   if ($40) {
    $41 = HEAP32[$db>>2]|0;
    $42 = (($41) + 12|0);
    HEAP32[$42>>2] = (8);
    $43 = HEAP32[$db>>2]|0;
    $44 = (($43) + 12|0);
    $45 = HEAP32[$44>>2]|0;
    $46 = ($45|0)==(0|0);
    if ($46) {
     $47 = HEAP32[$db>>2]|0;
     _sqlite3_free($47);
     HEAP32[$db>>2] = 0;
     break;
    }
   }
   $48 = HEAP32[$db>>2]|0;
   $49 = (($48) + 56|0);
   HEAP32[$49>>2] = 255;
   $50 = HEAP32[$db>>2]|0;
   $51 = (($50) + 20|0);
   HEAP32[$51>>2] = 2;
   $52 = HEAP32[$db>>2]|0;
   $53 = (($52) + 76|0);
   HEAP32[$53>>2] = -264537850;
   $54 = HEAP32[$db>>2]|0;
   $55 = (($54) + 456|0);
   $56 = HEAP32[$db>>2]|0;
   $57 = (($56) + 16|0);
   HEAP32[$57>>2] = $55;
   $58 = HEAP32[$db>>2]|0;
   $59 = (($58) + 88|0);
   dest=$59+0|0; src=1696+0|0; stop=dest+48|0; do { HEAP32[dest>>2]=HEAP32[src>>2]|0; dest=dest+4|0; src=src+4|0; } while ((dest|0) < (stop|0));
   $60 = HEAP32[$db>>2]|0;
   $61 = (($60) + 88|0);
   $62 = (($61) + 44|0);
   HEAP32[$62>>2] = 0;
   $63 = HEAP32[$db>>2]|0;
   $64 = (($63) + 62|0);
   HEAP8[$64>>0] = 1;
   $65 = HEAP32[$db>>2]|0;
   $66 = (($65) + 66|0);
   HEAP8[$66>>0] = -1;
   $67 = ((38640 + 176|0));
   $68 = $67;
   $69 = HEAP32[$68>>2]|0;
   $70 = (($67) + 4)|0;
   $71 = $70;
   $72 = HEAP32[$71>>2]|0;
   $73 = HEAP32[$db>>2]|0;
   $74 = (($73) + 40|0);
   $75 = $74;
   $76 = $75;
   HEAP32[$76>>2] = $69;
   $77 = (($75) + 4)|0;
   $78 = $77;
   HEAP32[$78>>2] = $72;
   $79 = HEAP32[$db>>2]|0;
   $80 = (($79) + 72|0);
   HEAP32[$80>>2] = 0;
   $81 = HEAP32[$db>>2]|0;
   $82 = (($81) + 136|0);
   HEAP32[$82>>2] = 2147483647;
   $83 = HEAP32[$db>>2]|0;
   $84 = (($83) + 24|0);
   $85 = HEAP32[$84>>2]|0;
   $86 = $85 | 9437264;
   HEAP32[$84>>2] = $86;
   $87 = HEAP32[$db>>2]|0;
   $88 = (($87) + 428|0);
   _sqlite3HashInit($88);
   $89 = HEAP32[$db>>2]|0;
   $90 = (($89) + 308|0);
   _sqlite3HashInit($90);
   $91 = HEAP32[$db>>2]|0;
   (_createCollation($91,3320,1,0,8,0)|0);
   $92 = HEAP32[$db>>2]|0;
   (_createCollation($92,3320,3,0,8,0)|0);
   $93 = HEAP32[$db>>2]|0;
   (_createCollation($93,3320,2,0,8,0)|0);
   $94 = HEAP32[$db>>2]|0;
   (_createCollation($94,3328,1,(1),8,0)|0);
   $95 = HEAP32[$db>>2]|0;
   $96 = (($95) + 64|0);
   $97 = HEAP8[$96>>0]|0;
   $98 = ($97<<24>>24)!=(0);
   if (!($98)) {
    $99 = HEAP32[$db>>2]|0;
    $100 = (_sqlite3FindCollSeq($99,1,3320,0)|0);
    $101 = HEAP32[$db>>2]|0;
    $102 = (($101) + 8|0);
    HEAP32[$102>>2] = $100;
    $103 = HEAP32[$db>>2]|0;
    (_createCollation($103,3336,1,0,9,0)|0);
    $104 = HEAP32[$3>>2]|0;
    $105 = HEAP32[$db>>2]|0;
    $106 = (($105) + 48|0);
    HEAP32[$106>>2] = $104;
    $107 = HEAP32[$4>>2]|0;
    $108 = HEAP32[$1>>2]|0;
    $109 = HEAP32[$db>>2]|0;
    $110 = (_sqlite3ParseUri($107,$108,$3,$109,$zOpen,$zErrMsg)|0);
    HEAP32[$rc>>2] = $110;
    $111 = HEAP32[$rc>>2]|0;
    $112 = ($111|0)!=(0);
    if ($112) {
     $113 = HEAP32[$rc>>2]|0;
     $114 = ($113|0)==(7);
     if ($114) {
      $115 = HEAP32[$db>>2]|0;
      $116 = (($115) + 64|0);
      HEAP8[$116>>0] = 1;
     }
     $117 = HEAP32[$db>>2]|0;
     $118 = HEAP32[$rc>>2]|0;
     $119 = HEAP32[$zErrMsg>>2]|0;
     $120 = ($119|0)!=(0|0);
     $121 = $120 ? 1200 : 0;
     $122 = HEAP32[$zErrMsg>>2]|0;
     HEAP32[$vararg_buffer>>2] = $122;
     _sqlite3ErrorWithMsg($117,$118,$121,$vararg_buffer);
     $123 = HEAP32[$zErrMsg>>2]|0;
     _sqlite3_free($123);
     break;
    }
    $124 = HEAP32[$db>>2]|0;
    $125 = HEAP32[$124>>2]|0;
    $126 = HEAP32[$zOpen>>2]|0;
    $127 = HEAP32[$db>>2]|0;
    $128 = HEAP32[$db>>2]|0;
    $129 = (($128) + 16|0);
    $130 = HEAP32[$129>>2]|0;
    $131 = (($130) + 4|0);
    $132 = HEAP32[$3>>2]|0;
    $133 = $132 | 256;
    $134 = (_sqlite3BtreeOpen($125,$126,$127,$131,0,$133)|0);
    HEAP32[$rc>>2] = $134;
    $135 = HEAP32[$rc>>2]|0;
    $136 = ($135|0)!=(0);
    if ($136) {
     $137 = HEAP32[$rc>>2]|0;
     $138 = ($137|0)==(3082);
     if ($138) {
      HEAP32[$rc>>2] = 7;
     }
     $139 = HEAP32[$db>>2]|0;
     $140 = HEAP32[$rc>>2]|0;
     _sqlite3Error($139,$140);
     break;
    }
    $141 = HEAP32[$db>>2]|0;
    $142 = (($141) + 16|0);
    $143 = HEAP32[$142>>2]|0;
    $144 = (($143) + 4|0);
    $145 = HEAP32[$144>>2]|0;
    _sqlite3BtreeEnter($145);
    $146 = HEAP32[$db>>2]|0;
    $147 = HEAP32[$db>>2]|0;
    $148 = (($147) + 16|0);
    $149 = HEAP32[$148>>2]|0;
    $150 = (($149) + 4|0);
    $151 = HEAP32[$150>>2]|0;
    $152 = (_sqlite3SchemaGet($146,$151)|0);
    $153 = HEAP32[$db>>2]|0;
    $154 = (($153) + 16|0);
    $155 = HEAP32[$154>>2]|0;
    $156 = (($155) + 12|0);
    HEAP32[$156>>2] = $152;
    $157 = HEAP32[$db>>2]|0;
    $158 = (_sqlite3SchemaGet($157,0)|0);
    $159 = HEAP32[$db>>2]|0;
    $160 = (($159) + 16|0);
    $161 = HEAP32[$160>>2]|0;
    $162 = (($161) + 16|0);
    $163 = (($162) + 12|0);
    HEAP32[$163>>2] = $158;
    $164 = HEAP32[$db>>2]|0;
    $165 = (($164) + 16|0);
    $166 = HEAP32[$165>>2]|0;
    HEAP32[$166>>2] = 3344;
    $167 = HEAP32[$db>>2]|0;
    $168 = (($167) + 16|0);
    $169 = HEAP32[$168>>2]|0;
    $170 = (($169) + 8|0);
    HEAP8[$170>>0] = 3;
    $171 = HEAP32[$db>>2]|0;
    $172 = (($171) + 16|0);
    $173 = HEAP32[$172>>2]|0;
    $174 = (($173) + 16|0);
    HEAP32[$174>>2] = 1496;
    $175 = HEAP32[$db>>2]|0;
    $176 = (($175) + 16|0);
    $177 = HEAP32[$176>>2]|0;
    $178 = (($177) + 16|0);
    $179 = (($178) + 8|0);
    HEAP8[$179>>0] = 1;
    $180 = HEAP32[$db>>2]|0;
    $181 = (($180) + 76|0);
    HEAP32[$181>>2] = -1607883113;
    $182 = HEAP32[$db>>2]|0;
    $183 = (($182) + 64|0);
    $184 = HEAP8[$183>>0]|0;
    $185 = ($184<<24>>24)!=(0);
    if (!($185)) {
     $186 = HEAP32[$db>>2]|0;
     _sqlite3Error($186,0);
     $187 = HEAP32[$db>>2]|0;
     _sqlite3RegisterBuiltinFunctions($187);
     $188 = HEAP32[$db>>2]|0;
     $189 = (_sqlite3_errcode($188)|0);
     HEAP32[$rc>>2] = $189;
     $190 = HEAP32[$rc>>2]|0;
     $191 = ($190|0)==(0);
     if ($191) {
      $192 = HEAP32[$db>>2]|0;
      _sqlite3AutoLoadExtensions($192);
      $193 = HEAP32[$db>>2]|0;
      $194 = (_sqlite3_errcode($193)|0);
      HEAP32[$rc>>2] = $194;
      $195 = HEAP32[$rc>>2]|0;
      $196 = ($195|0)!=(0);
      if ($196) {
       break;
      }
     }
     $197 = HEAP32[$rc>>2]|0;
     $198 = ($197|0)!=(0);
     if ($198) {
      $199 = HEAP32[$db>>2]|0;
      $200 = HEAP32[$rc>>2]|0;
      _sqlite3Error($199,$200);
     }
     $201 = HEAP32[$db>>2]|0;
     $202 = HEAP32[((38640 + 28|0))>>2]|0;
     $203 = HEAP32[((38640 + 32|0))>>2]|0;
     (_setupLookaside($201,0,$202,$203)|0);
     $204 = HEAP32[$db>>2]|0;
     (_sqlite3_wal_autocheckpoint($204,1000)|0);
    }
   }
  }
 } while(0);
 $205 = HEAP32[$zOpen>>2]|0;
 _sqlite3_free($205);
 $206 = HEAP32[$db>>2]|0;
 $207 = (_sqlite3_errcode($206)|0);
 HEAP32[$rc>>2] = $207;
 $208 = HEAP32[$rc>>2]|0;
 $209 = ($208|0)==(7);
 if ($209) {
  $210 = HEAP32[$db>>2]|0;
  (_sqlite3_close($210)|0);
  HEAP32[$db>>2] = 0;
 } else {
  $211 = HEAP32[$rc>>2]|0;
  $212 = ($211|0)!=(0);
  if ($212) {
   $213 = HEAP32[$db>>2]|0;
   $214 = (($213) + 76|0);
   HEAP32[$214>>2] = 1266094736;
  }
 }
 $215 = HEAP32[$db>>2]|0;
 $216 = HEAP32[$2>>2]|0;
 HEAP32[$216>>2] = $215;
 $217 = HEAP32[$rc>>2]|0;
 $218 = (_sqlite3ApiExit(0,$217)|0);
 HEAP32[$0>>2] = $218;
 $219 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($219|0);
}
function _createCollation($db,$zName,$enc,$pCtx,$xCompare,$xDel) {
 $db = $db|0;
 $zName = $zName|0;
 $enc = $enc|0;
 $pCtx = $pCtx|0;
 $xCompare = $xCompare|0;
 $xDel = $xDel|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0;
 var $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0;
 var $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0;
 var $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0;
 var $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0;
 var $96 = 0, $97 = 0, $98 = 0, $99 = 0, $aColl = 0, $enc2 = 0, $j = 0, $or$cond = 0, $or$cond3 = 0, $p = 0, $pColl = 0, $vararg_buffer = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 64|0;
 $vararg_buffer = sp;
 $0 = sp + 44|0;
 $1 = sp + 40|0;
 $2 = sp + 36|0;
 $3 = sp + 48|0;
 $4 = sp + 32|0;
 $5 = sp + 28|0;
 $6 = sp + 24|0;
 $pColl = sp + 20|0;
 $enc2 = sp + 16|0;
 $aColl = sp + 12|0;
 $j = sp + 8|0;
 $p = sp + 4|0;
 HEAP32[$1>>2] = $db;
 HEAP32[$2>>2] = $zName;
 HEAP8[$3>>0] = $enc;
 HEAP32[$4>>2] = $pCtx;
 HEAP32[$5>>2] = $xCompare;
 HEAP32[$6>>2] = $xDel;
 $7 = HEAP8[$3>>0]|0;
 $8 = $7&255;
 HEAP32[$enc2>>2] = $8;
 $9 = HEAP32[$enc2>>2]|0;
 $10 = ($9|0)==(4);
 $11 = HEAP32[$enc2>>2]|0;
 $12 = ($11|0)==(8);
 $or$cond = $10 | $12;
 if ($or$cond) {
  $13 = HEAP8[1136>>0]|0;
  $14 = $13 << 24 >> 24;
  $15 = ($14|0)==(0);
  $16 = $15 ? 3 : 2;
  HEAP32[$enc2>>2] = $16;
 }
 $17 = HEAP32[$enc2>>2]|0;
 $18 = ($17|0)<(1);
 $19 = HEAP32[$enc2>>2]|0;
 $20 = ($19|0)>(3);
 $or$cond3 = $18 | $20;
 if ($or$cond3) {
  $21 = (_sqlite3MisuseError(127001)|0);
  HEAP32[$0>>2] = $21;
  $103 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($103|0);
 }
 $22 = HEAP32[$1>>2]|0;
 $23 = HEAP32[$enc2>>2]|0;
 $24 = $23&255;
 $25 = HEAP32[$2>>2]|0;
 $26 = (_sqlite3FindCollSeq($22,$24,$25,0)|0);
 HEAP32[$pColl>>2] = $26;
 $27 = HEAP32[$pColl>>2]|0;
 $28 = ($27|0)!=(0|0);
 L8: do {
  if ($28) {
   $29 = HEAP32[$pColl>>2]|0;
   $30 = (($29) + 12|0);
   $31 = HEAP32[$30>>2]|0;
   $32 = ($31|0)!=(0|0);
   if ($32) {
    $33 = HEAP32[$1>>2]|0;
    $34 = (($33) + 148|0);
    $35 = HEAP32[$34>>2]|0;
    $36 = ($35|0)!=(0);
    $37 = HEAP32[$1>>2]|0;
    if ($36) {
     _sqlite3ErrorWithMsg($37,5,3248,$vararg_buffer);
     HEAP32[$0>>2] = 5;
     $103 = HEAP32[$0>>2]|0;
     STACKTOP = sp;return ($103|0);
    }
    _sqlite3ExpirePreparedStatements($37);
    $38 = HEAP32[$1>>2]|0;
    _invalidateCachedKeyInfo($38);
    $39 = HEAP32[$pColl>>2]|0;
    $40 = (($39) + 4|0);
    $41 = HEAP8[$40>>0]|0;
    $42 = $41&255;
    $43 = $42 & -9;
    $44 = HEAP32[$enc2>>2]|0;
    $45 = ($43|0)==($44|0);
    if ($45) {
     $46 = HEAP32[$1>>2]|0;
     $47 = (($46) + 428|0);
     $48 = HEAP32[$2>>2]|0;
     $49 = (_sqlite3HashFind($47,$48)|0);
     HEAP32[$aColl>>2] = $49;
     HEAP32[$j>>2] = 0;
     while(1) {
      $50 = HEAP32[$j>>2]|0;
      $51 = ($50|0)<(3);
      if (!($51)) {
       break L8;
      }
      $52 = HEAP32[$j>>2]|0;
      $53 = HEAP32[$aColl>>2]|0;
      $54 = (($53) + (($52*20)|0)|0);
      HEAP32[$p>>2] = $54;
      $55 = HEAP32[$p>>2]|0;
      $56 = (($55) + 4|0);
      $57 = HEAP8[$56>>0]|0;
      $58 = $57&255;
      $59 = HEAP32[$pColl>>2]|0;
      $60 = (($59) + 4|0);
      $61 = HEAP8[$60>>0]|0;
      $62 = $61&255;
      $63 = ($58|0)==($62|0);
      if ($63) {
       $64 = HEAP32[$p>>2]|0;
       $65 = (($64) + 16|0);
       $66 = HEAP32[$65>>2]|0;
       $67 = ($66|0)!=(0|0);
       if ($67) {
        $68 = HEAP32[$p>>2]|0;
        $69 = (($68) + 16|0);
        $70 = HEAP32[$69>>2]|0;
        $71 = HEAP32[$p>>2]|0;
        $72 = (($71) + 8|0);
        $73 = HEAP32[$72>>2]|0;
        FUNCTION_TABLE_vi[$70 & 31]($73);
       }
       $74 = HEAP32[$p>>2]|0;
       $75 = (($74) + 12|0);
       HEAP32[$75>>2] = 0;
      }
      $76 = HEAP32[$j>>2]|0;
      $77 = (($76) + 1)|0;
      HEAP32[$j>>2] = $77;
     }
    }
   }
  }
 } while(0);
 $78 = HEAP32[$1>>2]|0;
 $79 = HEAP32[$enc2>>2]|0;
 $80 = $79&255;
 $81 = HEAP32[$2>>2]|0;
 $82 = (_sqlite3FindCollSeq($78,$80,$81,1)|0);
 HEAP32[$pColl>>2] = $82;
 $83 = HEAP32[$pColl>>2]|0;
 $84 = ($83|0)==(0|0);
 if ($84) {
  HEAP32[$0>>2] = 7;
  $103 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($103|0);
 } else {
  $85 = HEAP32[$5>>2]|0;
  $86 = HEAP32[$pColl>>2]|0;
  $87 = (($86) + 12|0);
  HEAP32[$87>>2] = $85;
  $88 = HEAP32[$4>>2]|0;
  $89 = HEAP32[$pColl>>2]|0;
  $90 = (($89) + 8|0);
  HEAP32[$90>>2] = $88;
  $91 = HEAP32[$6>>2]|0;
  $92 = HEAP32[$pColl>>2]|0;
  $93 = (($92) + 16|0);
  HEAP32[$93>>2] = $91;
  $94 = HEAP32[$enc2>>2]|0;
  $95 = HEAP8[$3>>0]|0;
  $96 = $95&255;
  $97 = $96 & 8;
  $98 = $94 | $97;
  $99 = $98&255;
  $100 = HEAP32[$pColl>>2]|0;
  $101 = (($100) + 4|0);
  HEAP8[$101>>0] = $99;
  $102 = HEAP32[$1>>2]|0;
  _sqlite3Error($102,0);
  HEAP32[$0>>2] = 0;
  $103 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($103|0);
 }
 return 0|0;
}
function _sqlite3OsSleep($pVfs,$nMicro) {
 $pVfs = $pVfs|0;
 $nMicro = $nMicro|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $1 = sp;
 HEAP32[$0>>2] = $pVfs;
 HEAP32[$1>>2] = $nMicro;
 $2 = HEAP32[$0>>2]|0;
 $3 = (($2) + 60|0);
 $4 = HEAP32[$3>>2]|0;
 $5 = HEAP32[$0>>2]|0;
 $6 = HEAP32[$1>>2]|0;
 $7 = (FUNCTION_TABLE_iii[$4 & 63]($5,$6)|0);
 STACKTOP = sp;return ($7|0);
}
function _sqlite3_file_control($db,$zDbName,$op,$pArg) {
 $db = $db|0;
 $zDbName = $zDbName|0;
 $op = $op|0;
 $pArg = $pArg|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $3 = 0, $4 = 0;
 var $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $fd = 0, $pBtree = 0, $pPager = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 28|0;
 $1 = sp + 24|0;
 $2 = sp + 20|0;
 $3 = sp + 16|0;
 $rc = sp + 12|0;
 $pBtree = sp + 8|0;
 $pPager = sp + 4|0;
 $fd = sp;
 HEAP32[$0>>2] = $db;
 HEAP32[$1>>2] = $zDbName;
 HEAP32[$2>>2] = $op;
 HEAP32[$3>>2] = $pArg;
 HEAP32[$rc>>2] = 1;
 $4 = HEAP32[$0>>2]|0;
 $5 = HEAP32[$1>>2]|0;
 $6 = (_sqlite3DbNameToBtree($4,$5)|0);
 HEAP32[$pBtree>>2] = $6;
 $7 = HEAP32[$pBtree>>2]|0;
 $8 = ($7|0)!=(0|0);
 if (!($8)) {
  $24 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($24|0);
 }
 $9 = HEAP32[$pBtree>>2]|0;
 _sqlite3BtreeEnter($9);
 $10 = HEAP32[$pBtree>>2]|0;
 $11 = (_sqlite3BtreePager($10)|0);
 HEAP32[$pPager>>2] = $11;
 $12 = HEAP32[$pPager>>2]|0;
 $13 = (_sqlite3PagerFile($12)|0);
 HEAP32[$fd>>2] = $13;
 $14 = HEAP32[$2>>2]|0;
 $15 = ($14|0)==(7);
 $16 = HEAP32[$fd>>2]|0;
 if ($15) {
  $17 = HEAP32[$3>>2]|0;
  HEAP32[$17>>2] = $16;
  HEAP32[$rc>>2] = 0;
  $24 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($24|0);
 }
 $18 = HEAP32[$16>>2]|0;
 $19 = ($18|0)!=(0|0);
 if ($19) {
  $20 = HEAP32[$fd>>2]|0;
  $21 = HEAP32[$2>>2]|0;
  $22 = HEAP32[$3>>2]|0;
  $23 = (_sqlite3OsFileControl($20,$21,$22)|0);
  HEAP32[$rc>>2] = $23;
  $24 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($24|0);
 } else {
  HEAP32[$rc>>2] = 12;
  $24 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($24|0);
 }
 return 0|0;
}
function _sqlite3DbNameToBtree($db,$zDbName) {
 $db = $db|0;
 $zDbName = $zDbName|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $i = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $1 = sp + 8|0;
 $2 = sp + 4|0;
 $i = sp;
 HEAP32[$1>>2] = $db;
 HEAP32[$2>>2] = $zDbName;
 HEAP32[$i>>2] = 0;
 while(1) {
  $3 = HEAP32[$i>>2]|0;
  $4 = HEAP32[$1>>2]|0;
  $5 = (($4) + 20|0);
  $6 = HEAP32[$5>>2]|0;
  $7 = ($3|0)<($6|0);
  if (!($7)) {
   label = 8;
   break;
  }
  $8 = HEAP32[$i>>2]|0;
  $9 = HEAP32[$1>>2]|0;
  $10 = (($9) + 16|0);
  $11 = HEAP32[$10>>2]|0;
  $12 = (($11) + ($8<<4)|0);
  $13 = (($12) + 4|0);
  $14 = HEAP32[$13>>2]|0;
  $15 = ($14|0)!=(0|0);
  if ($15) {
   $16 = HEAP32[$2>>2]|0;
   $17 = ($16|0)==(0|0);
   if ($17) {
    label = 6;
    break;
   }
   $18 = HEAP32[$2>>2]|0;
   $19 = HEAP32[$i>>2]|0;
   $20 = HEAP32[$1>>2]|0;
   $21 = (($20) + 16|0);
   $22 = HEAP32[$21>>2]|0;
   $23 = (($22) + ($19<<4)|0);
   $24 = HEAP32[$23>>2]|0;
   $25 = (_sqlite3_stricmp($18,$24)|0);
   $26 = ($25|0)==(0);
   if ($26) {
    label = 6;
    break;
   }
  }
  $34 = HEAP32[$i>>2]|0;
  $35 = (($34) + 1)|0;
  HEAP32[$i>>2] = $35;
 }
 if ((label|0) == 6) {
  $27 = HEAP32[$i>>2]|0;
  $28 = HEAP32[$1>>2]|0;
  $29 = (($28) + 16|0);
  $30 = HEAP32[$29>>2]|0;
  $31 = (($30) + ($27<<4)|0);
  $32 = (($31) + 4|0);
  $33 = HEAP32[$32>>2]|0;
  HEAP32[$0>>2] = $33;
  $36 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($36|0);
 }
 else if ((label|0) == 8) {
  HEAP32[$0>>2] = 0;
  $36 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($36|0);
 }
 return 0|0;
}
function _sqlite3OsFileControl($id,$op,$pArg) {
 $id = $id|0;
 $op = $op|0;
 $pArg = $pArg|0;
 var $0 = 0, $1 = 0, $10 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $1 = sp + 4|0;
 $2 = sp;
 HEAP32[$0>>2] = $id;
 HEAP32[$1>>2] = $op;
 HEAP32[$2>>2] = $pArg;
 $3 = HEAP32[$0>>2]|0;
 $4 = HEAP32[$3>>2]|0;
 $5 = (($4) + 40|0);
 $6 = HEAP32[$5>>2]|0;
 $7 = HEAP32[$0>>2]|0;
 $8 = HEAP32[$1>>2]|0;
 $9 = HEAP32[$2>>2]|0;
 $10 = (FUNCTION_TABLE_iiii[$6 & 31]($7,$8,$9)|0);
 STACKTOP = sp;return ($10|0);
}
function _sqlite3FaultSim($iTest) {
 $iTest = $iTest|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $xCallback = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $xCallback = sp;
 HEAP32[$0>>2] = $iTest;
 $1 = HEAP32[((38640 + 260|0))>>2]|0;
 HEAP32[$xCallback>>2] = $1;
 $2 = HEAP32[$xCallback>>2]|0;
 $3 = ($2|0)!=(0|0);
 if (!($3)) {
  $7 = 0;
  STACKTOP = sp;return ($7|0);
 }
 $4 = HEAP32[$xCallback>>2]|0;
 $5 = HEAP32[$0>>2]|0;
 $6 = (FUNCTION_TABLE_ii[$4 & 31]($5)|0);
 $7 = $6;
 STACKTOP = sp;return ($7|0);
}
function _sqlite3BtreeSetPageSize($p,$pageSize,$nReserve,$iFix) {
 $p = $p|0;
 $pageSize = $pageSize|0;
 $nReserve = $nReserve|0;
 $iFix = $iFix|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $7 = 0;
 var $8 = 0, $9 = 0, $or$cond = 0, $pBt = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 24|0;
 $1 = sp + 20|0;
 $2 = sp + 16|0;
 $3 = sp + 12|0;
 $4 = sp + 8|0;
 $rc = sp + 4|0;
 $pBt = sp;
 HEAP32[$1>>2] = $p;
 HEAP32[$2>>2] = $pageSize;
 HEAP32[$3>>2] = $nReserve;
 HEAP32[$4>>2] = $iFix;
 HEAP32[$rc>>2] = 0;
 $5 = HEAP32[$1>>2]|0;
 $6 = (($5) + 4|0);
 $7 = HEAP32[$6>>2]|0;
 HEAP32[$pBt>>2] = $7;
 $8 = HEAP32[$1>>2]|0;
 _sqlite3BtreeEnter($8);
 $9 = HEAP32[$pBt>>2]|0;
 $10 = (($9) + 22|0);
 $11 = HEAP16[$10>>1]|0;
 $12 = $11&65535;
 $13 = $12 & 2;
 $14 = ($13|0)!=(0);
 if ($14) {
  HEAP32[$0>>2] = 8;
  $61 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($61|0);
 }
 $15 = HEAP32[$3>>2]|0;
 $16 = ($15|0)<(0);
 if ($16) {
  $17 = HEAP32[$pBt>>2]|0;
  $18 = (($17) + 32|0);
  $19 = HEAP32[$18>>2]|0;
  $20 = HEAP32[$pBt>>2]|0;
  $21 = (($20) + 36|0);
  $22 = HEAP32[$21>>2]|0;
  $23 = (($19) - ($22))|0;
  HEAP32[$3>>2] = $23;
 }
 $24 = HEAP32[$2>>2]|0;
 $25 = ($24|0)>=(512);
 $26 = HEAP32[$2>>2]|0;
 $27 = ($26|0)<=(65536);
 $or$cond = $25 & $27;
 if ($or$cond) {
  $28 = HEAP32[$2>>2]|0;
  $29 = (($28) - 1)|0;
  $30 = HEAP32[$2>>2]|0;
  $31 = $29 & $30;
  $32 = ($31|0)==(0);
  if ($32) {
   $33 = HEAP32[$2>>2]|0;
   $34 = HEAP32[$pBt>>2]|0;
   $35 = (($34) + 32|0);
   HEAP32[$35>>2] = $33;
   $36 = HEAP32[$pBt>>2]|0;
   _freeTempSpace($36);
  }
 }
 $37 = HEAP32[$pBt>>2]|0;
 $38 = HEAP32[$37>>2]|0;
 $39 = HEAP32[$pBt>>2]|0;
 $40 = (($39) + 32|0);
 $41 = HEAP32[$3>>2]|0;
 $42 = (_sqlite3PagerSetPagesize($38,$40,$41)|0);
 HEAP32[$rc>>2] = $42;
 $43 = HEAP32[$pBt>>2]|0;
 $44 = (($43) + 32|0);
 $45 = HEAP32[$44>>2]|0;
 $46 = HEAP32[$3>>2]|0;
 $47 = $46&65535;
 $48 = $47&65535;
 $49 = (($45) - ($48))|0;
 $50 = HEAP32[$pBt>>2]|0;
 $51 = (($50) + 36|0);
 HEAP32[$51>>2] = $49;
 $52 = HEAP32[$4>>2]|0;
 $53 = ($52|0)!=(0);
 if ($53) {
  $54 = HEAP32[$pBt>>2]|0;
  $55 = (($54) + 22|0);
  $56 = HEAP16[$55>>1]|0;
  $57 = $56&65535;
  $58 = $57 | 2;
  $59 = $58&65535;
  HEAP16[$55>>1] = $59;
 }
 $60 = HEAP32[$rc>>2]|0;
 HEAP32[$0>>2] = $60;
 $61 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($61|0);
}
function _sqlite3KeywordCode($z,$n) {
 $z = $z|0;
 $n = $n|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $1 = sp;
 HEAP32[$0>>2] = $z;
 HEAP32[$1>>2] = $n;
 $2 = HEAP32[$0>>2]|0;
 $3 = HEAP32[$1>>2]|0;
 $4 = (_keywordCode($2,$3)|0);
 STACKTOP = sp;return ($4|0);
}
function _sqlite3ScratchMalloc($n) {
 $n = $n|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0;
 var $7 = 0, $8 = 0, $9 = 0, $or$cond = 0, $p = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $p = sp;
 HEAP32[$0>>2] = $n;
 $1 = HEAP32[$0>>2]|0;
 _sqlite3StatusSet(8,$1);
 $2 = HEAP32[((136 + 32|0))>>2]|0;
 $3 = ($2|0)!=(0);
 if ($3) {
  $4 = HEAP32[((38640 + 196|0))>>2]|0;
  $5 = HEAP32[$0>>2]|0;
  $6 = ($4|0)>=($5|0);
  if ($6) {
   $7 = HEAP32[((136 + 28|0))>>2]|0;
   HEAP32[$p>>2] = $7;
   $8 = HEAP32[((136 + 28|0))>>2]|0;
   $9 = HEAP32[$8>>2]|0;
   HEAP32[((136 + 28|0))>>2] = $9;
   $10 = HEAP32[((136 + 32|0))>>2]|0;
   $11 = (($10) + -1)|0;
   HEAP32[((136 + 32|0))>>2] = $11;
   _sqlite3StatusAdd(3,1);
   $22 = HEAP32[$p>>2]|0;
   STACKTOP = sp;return ($22|0);
  }
 }
 $12 = HEAP32[$0>>2]|0;
 $13 = ($12|0)<(0);
 $14 = $13 << 31 >> 31;
 $15 = (_sqlite3Malloc($12,$14)|0);
 HEAP32[$p>>2] = $15;
 $16 = HEAP32[38640>>2]|0;
 $17 = ($16|0)!=(0);
 $18 = HEAP32[$p>>2]|0;
 $19 = ($18|0)!=(0|0);
 $or$cond = $17 & $19;
 if (!($or$cond)) {
  $22 = HEAP32[$p>>2]|0;
  STACKTOP = sp;return ($22|0);
 }
 $20 = HEAP32[$p>>2]|0;
 $21 = (_sqlite3MallocSize($20)|0);
 _sqlite3StatusAdd(4,$21);
 $22 = HEAP32[$p>>2]|0;
 STACKTOP = sp;return ($22|0);
}
function _sqlite3ScratchFree($p) {
 $p = $p|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $iSize = 0, $pSlot = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $pSlot = sp + 4|0;
 $iSize = sp;
 HEAP32[$0>>2] = $p;
 $1 = HEAP32[$0>>2]|0;
 $2 = ($1|0)!=(0|0);
 if (!($2)) {
  STACKTOP = sp;return;
 }
 $3 = HEAP32[$0>>2]|0;
 $4 = HEAP32[((38640 + 192|0))>>2]|0;
 $5 = ($3>>>0)>=($4>>>0);
 if ($5) {
  $6 = HEAP32[$0>>2]|0;
  $7 = HEAP32[((136 + 24|0))>>2]|0;
  $8 = ($6>>>0)<($7>>>0);
  if ($8) {
   $9 = HEAP32[$0>>2]|0;
   HEAP32[$pSlot>>2] = $9;
   $10 = HEAP32[((136 + 28|0))>>2]|0;
   $11 = HEAP32[$pSlot>>2]|0;
   HEAP32[$11>>2] = $10;
   $12 = HEAP32[$pSlot>>2]|0;
   HEAP32[((136 + 28|0))>>2] = $12;
   $13 = HEAP32[((136 + 32|0))>>2]|0;
   $14 = (($13) + 1)|0;
   HEAP32[((136 + 32|0))>>2] = $14;
   _sqlite3StatusAdd(3,-1);
   STACKTOP = sp;return;
  }
 }
 $15 = HEAP32[38640>>2]|0;
 $16 = ($15|0)!=(0);
 if ($16) {
  $17 = HEAP32[$0>>2]|0;
  $18 = (_sqlite3MallocSize($17)|0);
  HEAP32[$iSize>>2] = $18;
  $19 = HEAP32[$iSize>>2]|0;
  $20 = (0 - ($19))|0;
  _sqlite3StatusAdd(4,$20);
  $21 = HEAP32[$iSize>>2]|0;
  $22 = (0 - ($21))|0;
  _sqlite3StatusAdd(0,$22);
  _sqlite3StatusAdd(9,-1);
  $23 = HEAP32[((38640 + 40|0))>>2]|0;
  $24 = HEAP32[$0>>2]|0;
  FUNCTION_TABLE_vi[$23 & 31]($24);
  STACKTOP = sp;return;
 } else {
  $25 = HEAP32[((38640 + 40|0))>>2]|0;
  $26 = HEAP32[$0>>2]|0;
  FUNCTION_TABLE_vi[$25 & 31]($26);
  STACKTOP = sp;return;
 }
}
function _sqlite3_uri_parameter($zFilename,$zParam) {
 $zFilename = $zFilename|0;
 $zParam = $zParam|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $x = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $1 = sp + 8|0;
 $2 = sp + 4|0;
 $x = sp;
 HEAP32[$1>>2] = $zFilename;
 HEAP32[$2>>2] = $zParam;
 $3 = HEAP32[$1>>2]|0;
 $4 = ($3|0)==(0|0);
 if ($4) {
  HEAP32[$0>>2] = 0;
  $28 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($28|0);
 }
 $5 = HEAP32[$1>>2]|0;
 $6 = (_sqlite3Strlen30($5)|0);
 $7 = (($6) + 1)|0;
 $8 = HEAP32[$1>>2]|0;
 $9 = (($8) + ($7)|0);
 HEAP32[$1>>2] = $9;
 while(1) {
  $10 = HEAP32[$1>>2]|0;
  $11 = HEAP8[$10>>0]|0;
  $12 = ($11<<24>>24)!=(0);
  if (!($12)) {
   label = 8;
   break;
  }
  $13 = HEAP32[$1>>2]|0;
  $14 = HEAP32[$2>>2]|0;
  $15 = (_strcmp($13,$14)|0);
  HEAP32[$x>>2] = $15;
  $16 = HEAP32[$1>>2]|0;
  $17 = (_sqlite3Strlen30($16)|0);
  $18 = (($17) + 1)|0;
  $19 = HEAP32[$1>>2]|0;
  $20 = (($19) + ($18)|0);
  HEAP32[$1>>2] = $20;
  $21 = HEAP32[$x>>2]|0;
  $22 = ($21|0)==(0);
  $23 = HEAP32[$1>>2]|0;
  if ($22) {
   label = 6;
   break;
  }
  $24 = (_sqlite3Strlen30($23)|0);
  $25 = (($24) + 1)|0;
  $26 = HEAP32[$1>>2]|0;
  $27 = (($26) + ($25)|0);
  HEAP32[$1>>2] = $27;
 }
 if ((label|0) == 6) {
  HEAP32[$0>>2] = $23;
  $28 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($28|0);
 }
 else if ((label|0) == 8) {
  HEAP32[$0>>2] = 0;
  $28 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($28|0);
 }
 return 0|0;
}
function _sqlite3_uri_boolean($zFilename,$zParam,$bDflt) {
 $zFilename = $zFilename|0;
 $zParam = $zParam|0;
 $bDflt = $bDflt|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $z = 0, label = 0;
 var sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $1 = sp + 8|0;
 $2 = sp + 4|0;
 $z = sp;
 HEAP32[$0>>2] = $zFilename;
 HEAP32[$1>>2] = $zParam;
 HEAP32[$2>>2] = $bDflt;
 $3 = HEAP32[$0>>2]|0;
 $4 = HEAP32[$1>>2]|0;
 $5 = (_sqlite3_uri_parameter($3,$4)|0);
 HEAP32[$z>>2] = $5;
 $6 = HEAP32[$2>>2]|0;
 $7 = ($6|0)!=(0);
 $8 = $7&1;
 HEAP32[$2>>2] = $8;
 $9 = HEAP32[$z>>2]|0;
 $10 = ($9|0)!=(0|0);
 if ($10) {
  $11 = HEAP32[$z>>2]|0;
  $12 = HEAP32[$2>>2]|0;
  $13 = $12&255;
  $14 = (_sqlite3GetBoolean($11,$13)|0);
  $15 = $14&255;
  $17 = $15;
  STACKTOP = sp;return ($17|0);
 } else {
  $16 = HEAP32[$2>>2]|0;
  $17 = $16;
  STACKTOP = sp;return ($17|0);
 }
 return 0|0;
}
function _sqlite3GetBoolean($z,$dflt) {
 $z = $z|0;
 $dflt = $dflt|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 $1 = sp + 4|0;
 HEAP32[$0>>2] = $z;
 HEAP8[$1>>0] = $dflt;
 $2 = HEAP32[$0>>2]|0;
 $3 = HEAP8[$1>>0]|0;
 $4 = (_getSafetyLevel($2,1,$3)|0);
 $5 = $4&255;
 $6 = ($5|0)!=(0);
 $7 = $6&1;
 $8 = $7&255;
 STACKTOP = sp;return ($8|0);
}
function _sqlite3DecOrHexToI64($z,$pOut) {
 $z = $z|0;
 $pOut = $pOut|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0;
 var $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $i = 0, $k = 0, $u = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 24|0;
 $1 = sp + 20|0;
 $2 = sp + 16|0;
 $u = sp;
 $i = sp + 12|0;
 $k = sp + 8|0;
 HEAP32[$1>>2] = $z;
 HEAP32[$2>>2] = $pOut;
 $3 = HEAP32[$1>>2]|0;
 $4 = HEAP8[$3>>0]|0;
 $5 = $4 << 24 >> 24;
 $6 = ($5|0)==(48);
 do {
  if ($6) {
   $7 = HEAP32[$1>>2]|0;
   $8 = (($7) + 1|0);
   $9 = HEAP8[$8>>0]|0;
   $10 = $9 << 24 >> 24;
   $11 = ($10|0)==(120);
   if (!($11)) {
    $12 = HEAP32[$1>>2]|0;
    $13 = (($12) + 1|0);
    $14 = HEAP8[$13>>0]|0;
    $15 = $14 << 24 >> 24;
    $16 = ($15|0)==(88);
    if (!($16)) {
     break;
    }
   }
   $17 = HEAP32[$1>>2]|0;
   $18 = (($17) + 2|0);
   $19 = HEAP8[$18>>0]|0;
   $20 = $19&255;
   $21 = (1224 + ($20)|0);
   $22 = HEAP8[$21>>0]|0;
   $23 = $22&255;
   $24 = $23 & 8;
   $25 = ($24|0)!=(0);
   if ($25) {
    $26 = $u;
    $27 = $26;
    HEAP32[$27>>2] = 0;
    $28 = (($26) + 4)|0;
    $29 = $28;
    HEAP32[$29>>2] = 0;
    HEAP32[$i>>2] = 2;
    while(1) {
     $30 = HEAP32[$i>>2]|0;
     $31 = HEAP32[$1>>2]|0;
     $32 = (($31) + ($30)|0);
     $33 = HEAP8[$32>>0]|0;
     $34 = $33 << 24 >> 24;
     $35 = ($34|0)==(48);
     $36 = HEAP32[$i>>2]|0;
     if (!($35)) {
      break;
     }
     $37 = (($36) + 1)|0;
     HEAP32[$i>>2] = $37;
    }
    HEAP32[$k>>2] = $36;
    while(1) {
     $38 = HEAP32[$k>>2]|0;
     $39 = HEAP32[$1>>2]|0;
     $40 = (($39) + ($38)|0);
     $41 = HEAP8[$40>>0]|0;
     $42 = $41&255;
     $43 = (1224 + ($42)|0);
     $44 = HEAP8[$43>>0]|0;
     $45 = $44&255;
     $46 = $45 & 8;
     $47 = ($46|0)!=(0);
     if (!($47)) {
      break;
     }
     $48 = $u;
     $49 = $48;
     $50 = HEAP32[$49>>2]|0;
     $51 = (($48) + 4)|0;
     $52 = $51;
     $53 = HEAP32[$52>>2]|0;
     $54 = (___muldi3(($50|0),($53|0),16,0)|0);
     $55 = tempRet0;
     $56 = HEAP32[$k>>2]|0;
     $57 = HEAP32[$1>>2]|0;
     $58 = (($57) + ($56)|0);
     $59 = HEAP8[$58>>0]|0;
     $60 = $59 << 24 >> 24;
     $61 = (_sqlite3HexToInt($60)|0);
     $62 = $61&255;
     $63 = (_i64Add(($54|0),($55|0),($62|0),0)|0);
     $64 = tempRet0;
     $65 = $u;
     $66 = $65;
     HEAP32[$66>>2] = $63;
     $67 = (($65) + 4)|0;
     $68 = $67;
     HEAP32[$68>>2] = $64;
     $69 = HEAP32[$k>>2]|0;
     $70 = (($69) + 1)|0;
     HEAP32[$k>>2] = $70;
    }
    $71 = HEAP32[$2>>2]|0;
    ;HEAP32[$71+0>>2]=HEAP32[$u+0>>2]|0;HEAP32[$71+4>>2]=HEAP32[$u+4>>2]|0;
    $72 = HEAP32[$k>>2]|0;
    $73 = HEAP32[$1>>2]|0;
    $74 = (($73) + ($72)|0);
    $75 = HEAP8[$74>>0]|0;
    $76 = $75 << 24 >> 24;
    $77 = ($76|0)==(0);
    if ($77) {
     $78 = HEAP32[$k>>2]|0;
     $79 = HEAP32[$i>>2]|0;
     $80 = (($78) - ($79))|0;
     $81 = ($80|0)<=(16);
     $83 = $81;
    } else {
     $83 = 0;
    }
    $82 = $83 ? 0 : 1;
    HEAP32[$0>>2] = $82;
    $89 = HEAP32[$0>>2]|0;
    STACKTOP = sp;return ($89|0);
   }
  }
 } while(0);
 $84 = HEAP32[$1>>2]|0;
 $85 = HEAP32[$2>>2]|0;
 $86 = HEAP32[$1>>2]|0;
 $87 = (_sqlite3Strlen30($86)|0);
 $88 = (_sqlite3Atoi64($84,$85,$87,1)|0);
 HEAP32[$0>>2] = $88;
 $89 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($89|0);
}
function _sqlite3BtreeGetFilename($p) {
 $p = $p|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 HEAP32[$0>>2] = $p;
 $1 = HEAP32[$0>>2]|0;
 $2 = (($1) + 4|0);
 $3 = HEAP32[$2>>2]|0;
 $4 = HEAP32[$3>>2]|0;
 $5 = (_sqlite3PagerFilename($4,1)|0);
 STACKTOP = sp;return ($5|0);
}
function _sqlite3BtreeIsReadonly($p) {
 $p = $p|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 HEAP32[$0>>2] = $p;
 $1 = HEAP32[$0>>2]|0;
 $2 = (($1) + 4|0);
 $3 = HEAP32[$2>>2]|0;
 $4 = (($3) + 22|0);
 $5 = HEAP16[$4>>1]|0;
 $6 = $5&65535;
 $7 = $6 & 1;
 $8 = ($7|0)!=(0);
 $9 = $8&1;
 STACKTOP = sp;return ($9|0);
}
function _sqlite3PagerFilename($pPager,$nullIfMemDb) {
 $pPager = $pPager|0;
 $nullIfMemDb = $nullIfMemDb|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $1 = sp;
 HEAP32[$0>>2] = $pPager;
 HEAP32[$1>>2] = $nullIfMemDb;
 $2 = HEAP32[$1>>2]|0;
 $3 = ($2|0)!=(0);
 if ($3) {
  $4 = HEAP32[$0>>2]|0;
  $5 = (($4) + 15|0);
  $6 = HEAP8[$5>>0]|0;
  $7 = $6&255;
  $8 = ($7|0)!=(0);
  if ($8) {
   $12 = 1216;
   STACKTOP = sp;return ($12|0);
  }
 }
 $9 = HEAP32[$0>>2]|0;
 $10 = (($9) + 168|0);
 $11 = HEAP32[$10>>2]|0;
 $12 = $11;
 STACKTOP = sp;return ($12|0);
}
function _sqlite3HexToInt($h) {
 $h = $h|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 HEAP32[$0>>2] = $h;
 $1 = HEAP32[$0>>2]|0;
 $2 = $1 >> 6;
 $3 = 1 & $2;
 $4 = ($3*9)|0;
 $5 = HEAP32[$0>>2]|0;
 $6 = (($5) + ($4))|0;
 HEAP32[$0>>2] = $6;
 $7 = HEAP32[$0>>2]|0;
 $8 = $7 & 15;
 $9 = $8&255;
 STACKTOP = sp;return ($9|0);
}
function _sqlite3Atoi64($zNum,$pNum,$length,$enc) {
 $zNum = $zNum|0;
 $pNum = $pNum|0;
 $length = $length|0;
 $enc = $enc|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0;
 var $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0, $188 = 0;
 var $189 = 0, $19 = 0, $190 = 0, $191 = 0, $192 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0;
 var $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0;
 var $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0;
 var $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0;
 var $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $c = 0, $i = 0, $incr = 0, $neg = 0, $nonNum = 0, $or$cond = 0, $or$cond4 = 0;
 var $u = 0, $zEnd = 0, $zStart = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 64|0;
 $0 = sp + 48|0;
 $1 = sp + 44|0;
 $2 = sp + 40|0;
 $3 = sp + 36|0;
 $4 = sp + 52|0;
 $incr = sp + 32|0;
 $u = sp;
 $neg = sp + 28|0;
 $i = sp + 24|0;
 $c = sp + 20|0;
 $nonNum = sp + 16|0;
 $zStart = sp + 12|0;
 $zEnd = sp + 8|0;
 HEAP32[$1>>2] = $zNum;
 HEAP32[$2>>2] = $pNum;
 HEAP32[$3>>2] = $length;
 HEAP8[$4>>0] = $enc;
 $5 = $u;
 $6 = $5;
 HEAP32[$6>>2] = 0;
 $7 = (($5) + 4)|0;
 $8 = $7;
 HEAP32[$8>>2] = 0;
 HEAP32[$neg>>2] = 0;
 HEAP32[$c>>2] = 0;
 HEAP32[$nonNum>>2] = 0;
 $9 = HEAP32[$1>>2]|0;
 $10 = HEAP32[$3>>2]|0;
 $11 = (($9) + ($10)|0);
 HEAP32[$zEnd>>2] = $11;
 $12 = HEAP8[$4>>0]|0;
 $13 = $12&255;
 $14 = ($13|0)==(1);
 if ($14) {
  HEAP32[$incr>>2] = 1;
 } else {
  HEAP32[$incr>>2] = 2;
  $15 = HEAP8[$4>>0]|0;
  $16 = $15&255;
  $17 = (3 - ($16))|0;
  HEAP32[$i>>2] = $17;
  while(1) {
   $18 = HEAP32[$i>>2]|0;
   $19 = HEAP32[$3>>2]|0;
   $20 = ($18|0)<($19|0);
   if (!($20)) {
    break;
   }
   $21 = HEAP32[$i>>2]|0;
   $22 = HEAP32[$1>>2]|0;
   $23 = (($22) + ($21)|0);
   $24 = HEAP8[$23>>0]|0;
   $25 = $24 << 24 >> 24;
   $26 = ($25|0)==(0);
   if (!($26)) {
    break;
   }
   $27 = HEAP32[$i>>2]|0;
   $28 = (($27) + 2)|0;
   HEAP32[$i>>2] = $28;
  }
  $29 = HEAP32[$i>>2]|0;
  $30 = HEAP32[$3>>2]|0;
  $31 = ($29|0)<($30|0);
  $32 = $31&1;
  HEAP32[$nonNum>>2] = $32;
  $33 = HEAP32[$1>>2]|0;
  $34 = HEAP32[$i>>2]|0;
  $35 = (($33) + ($34)|0);
  $36 = HEAP8[$4>>0]|0;
  $37 = $36&255;
  $38 = (($35) + ($37)|0);
  $39 = (($38) + -3|0);
  HEAP32[$zEnd>>2] = $39;
  $40 = HEAP8[$4>>0]|0;
  $41 = $40&255;
  $42 = $41 & 1;
  $43 = HEAP32[$1>>2]|0;
  $44 = (($43) + ($42)|0);
  HEAP32[$1>>2] = $44;
 }
 while(1) {
  $45 = HEAP32[$1>>2]|0;
  $46 = HEAP32[$zEnd>>2]|0;
  $47 = ($45>>>0)<($46>>>0);
  if (!($47)) {
   break;
  }
  $48 = HEAP32[$1>>2]|0;
  $49 = HEAP8[$48>>0]|0;
  $50 = $49&255;
  $51 = (1224 + ($50)|0);
  $52 = HEAP8[$51>>0]|0;
  $53 = $52&255;
  $54 = $53 & 1;
  $55 = ($54|0)!=(0);
  if (!($55)) {
   break;
  }
  $56 = HEAP32[$incr>>2]|0;
  $57 = HEAP32[$1>>2]|0;
  $58 = (($57) + ($56)|0);
  HEAP32[$1>>2] = $58;
 }
 $59 = HEAP32[$1>>2]|0;
 $60 = HEAP32[$zEnd>>2]|0;
 $61 = ($59>>>0)<($60>>>0);
 do {
  if ($61) {
   $62 = HEAP32[$1>>2]|0;
   $63 = HEAP8[$62>>0]|0;
   $64 = $63 << 24 >> 24;
   $65 = ($64|0)==(45);
   if ($65) {
    HEAP32[$neg>>2] = 1;
    $66 = HEAP32[$incr>>2]|0;
    $67 = HEAP32[$1>>2]|0;
    $68 = (($67) + ($66)|0);
    HEAP32[$1>>2] = $68;
    break;
   }
   $69 = HEAP32[$1>>2]|0;
   $70 = HEAP8[$69>>0]|0;
   $71 = $70 << 24 >> 24;
   $72 = ($71|0)==(43);
   if ($72) {
    $73 = HEAP32[$incr>>2]|0;
    $74 = HEAP32[$1>>2]|0;
    $75 = (($74) + ($73)|0);
    HEAP32[$1>>2] = $75;
   }
  }
 } while(0);
 $76 = HEAP32[$1>>2]|0;
 HEAP32[$zStart>>2] = $76;
 while(1) {
  $77 = HEAP32[$1>>2]|0;
  $78 = HEAP32[$zEnd>>2]|0;
  $79 = ($77>>>0)<($78>>>0);
  if (!($79)) {
   break;
  }
  $80 = HEAP32[$1>>2]|0;
  $81 = HEAP8[$80>>0]|0;
  $82 = $81 << 24 >> 24;
  $83 = ($82|0)==(48);
  if (!($83)) {
   break;
  }
  $84 = HEAP32[$incr>>2]|0;
  $85 = HEAP32[$1>>2]|0;
  $86 = (($85) + ($84)|0);
  HEAP32[$1>>2] = $86;
 }
 HEAP32[$i>>2] = 0;
 while(1) {
  $87 = HEAP32[$i>>2]|0;
  $88 = HEAP32[$1>>2]|0;
  $89 = (($88) + ($87)|0);
  $90 = HEAP32[$zEnd>>2]|0;
  $91 = ($89>>>0)<($90>>>0);
  if (!($91)) {
   break;
  }
  $92 = HEAP32[$i>>2]|0;
  $93 = HEAP32[$1>>2]|0;
  $94 = (($93) + ($92)|0);
  $95 = HEAP8[$94>>0]|0;
  $96 = $95 << 24 >> 24;
  HEAP32[$c>>2] = $96;
  $97 = ($96|0)>=(48);
  $98 = HEAP32[$c>>2]|0;
  $99 = ($98|0)<=(57);
  $or$cond = $97 & $99;
  if (!($or$cond)) {
   break;
  }
  $100 = $u;
  $101 = $100;
  $102 = HEAP32[$101>>2]|0;
  $103 = (($100) + 4)|0;
  $104 = $103;
  $105 = HEAP32[$104>>2]|0;
  $106 = (___muldi3(($102|0),($105|0),10,0)|0);
  $107 = tempRet0;
  $108 = HEAP32[$c>>2]|0;
  $109 = ($108|0)<(0);
  $110 = $109 << 31 >> 31;
  $111 = (_i64Add(($106|0),($107|0),($108|0),($110|0))|0);
  $112 = tempRet0;
  $113 = (_i64Subtract(($111|0),($112|0),48,0)|0);
  $114 = tempRet0;
  $115 = $u;
  $116 = $115;
  HEAP32[$116>>2] = $113;
  $117 = (($115) + 4)|0;
  $118 = $117;
  HEAP32[$118>>2] = $114;
  $119 = HEAP32[$incr>>2]|0;
  $120 = HEAP32[$i>>2]|0;
  $121 = (($120) + ($119))|0;
  HEAP32[$i>>2] = $121;
 }
 $122 = $u;
 $123 = $122;
 $124 = HEAP32[$123>>2]|0;
 $125 = (($122) + 4)|0;
 $126 = $125;
 $127 = HEAP32[$126>>2]|0;
 $128 = ($127>>>0)>(2147483647);
 $129 = ($124>>>0)>(4294967295);
 $130 = ($127|0)==(2147483647);
 $131 = $130 & $129;
 $132 = $128 | $131;
 $133 = HEAP32[$neg>>2]|0;
 $134 = ($133|0)!=(0);
 do {
  if ($132) {
   $135 = $134 ? 0 : -1;
   $136 = $134 ? -2147483648 : 2147483647;
   $137 = HEAP32[$2>>2]|0;
   $138 = $137;
   $139 = $138;
   HEAP32[$139>>2] = $135;
   $140 = (($138) + 4)|0;
   $141 = $140;
   HEAP32[$141>>2] = $136;
  } else {
   $142 = $u;
   $143 = $142;
   $144 = HEAP32[$143>>2]|0;
   $145 = (($142) + 4)|0;
   $146 = $145;
   $147 = HEAP32[$146>>2]|0;
   if ($134) {
    $148 = (_i64Subtract(0,0,($144|0),($147|0))|0);
    $149 = tempRet0;
    $150 = HEAP32[$2>>2]|0;
    $151 = $150;
    $152 = $151;
    HEAP32[$152>>2] = $148;
    $153 = (($151) + 4)|0;
    $154 = $153;
    HEAP32[$154>>2] = $149;
    break;
   } else {
    $155 = HEAP32[$2>>2]|0;
    $156 = $155;
    $157 = $156;
    HEAP32[$157>>2] = $144;
    $158 = (($156) + 4)|0;
    $159 = $158;
    HEAP32[$159>>2] = $147;
    break;
   }
  }
 } while(0);
 $160 = HEAP32[$c>>2]|0;
 $161 = ($160|0)!=(0);
 if ($161) {
  $162 = HEAP32[$i>>2]|0;
  $163 = HEAP32[$1>>2]|0;
  $164 = (($163) + ($162)|0);
  $165 = HEAP32[$zEnd>>2]|0;
  $166 = ($164>>>0)<($165>>>0);
  if (!($166)) {
   label = 31;
  }
 } else {
  label = 31;
 }
 do {
  if ((label|0) == 31) {
   $167 = HEAP32[$i>>2]|0;
   $168 = ($167|0)==(0);
   if ($168) {
    $169 = HEAP32[$zStart>>2]|0;
    $170 = HEAP32[$1>>2]|0;
    $171 = ($169|0)==($170|0);
    if ($171) {
     break;
    }
   }
   $172 = HEAP32[$i>>2]|0;
   $173 = HEAP32[$incr>>2]|0;
   $174 = ($173*19)|0;
   $175 = ($172|0)>($174|0);
   $176 = HEAP32[$nonNum>>2]|0;
   $177 = ($176|0)!=(0);
   $or$cond4 = $175 | $177;
   if (!($or$cond4)) {
    $178 = HEAP32[$i>>2]|0;
    $179 = HEAP32[$incr>>2]|0;
    $180 = ($179*19)|0;
    $181 = ($178|0)<($180|0);
    if ($181) {
     HEAP32[$0>>2] = 0;
     $192 = HEAP32[$0>>2]|0;
     STACKTOP = sp;return ($192|0);
    }
    $182 = HEAP32[$1>>2]|0;
    $183 = HEAP32[$incr>>2]|0;
    $184 = (_compare2pow63($182,$183)|0);
    HEAP32[$c>>2] = $184;
    $185 = HEAP32[$c>>2]|0;
    $186 = ($185|0)<(0);
    if ($186) {
     HEAP32[$0>>2] = 0;
     $192 = HEAP32[$0>>2]|0;
     STACKTOP = sp;return ($192|0);
    }
    $187 = HEAP32[$c>>2]|0;
    $188 = ($187|0)>(0);
    if ($188) {
     HEAP32[$0>>2] = 1;
     $192 = HEAP32[$0>>2]|0;
     STACKTOP = sp;return ($192|0);
    } else {
     $189 = HEAP32[$neg>>2]|0;
     $190 = ($189|0)!=(0);
     $191 = $190 ? 0 : 2;
     HEAP32[$0>>2] = $191;
     $192 = HEAP32[$0>>2]|0;
     STACKTOP = sp;return ($192|0);
    }
   }
  }
 } while(0);
 HEAP32[$0>>2] = 1;
 $192 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($192|0);
}
function _compare2pow63($zNum,$incr) {
 $zNum = $zNum|0;
 $incr = $incr|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $c = 0, $i = 0, $or$cond = 0, $pow63 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 16|0;
 $1 = sp + 12|0;
 $c = sp + 8|0;
 $i = sp + 4|0;
 $pow63 = sp;
 HEAP32[$0>>2] = $zNum;
 HEAP32[$1>>2] = $incr;
 HEAP32[$c>>2] = 0;
 HEAP32[$pow63>>2] = 1744;
 HEAP32[$i>>2] = 0;
 while(1) {
  $2 = HEAP32[$c>>2]|0;
  $3 = ($2|0)==(0);
  $4 = HEAP32[$i>>2]|0;
  $5 = ($4|0)<(18);
  $or$cond = $3 & $5;
  if (!($or$cond)) {
   break;
  }
  $6 = HEAP32[$i>>2]|0;
  $7 = HEAP32[$1>>2]|0;
  $8 = Math_imul($6, $7)|0;
  $9 = HEAP32[$0>>2]|0;
  $10 = (($9) + ($8)|0);
  $11 = HEAP8[$10>>0]|0;
  $12 = $11 << 24 >> 24;
  $13 = HEAP32[$i>>2]|0;
  $14 = HEAP32[$pow63>>2]|0;
  $15 = (($14) + ($13)|0);
  $16 = HEAP8[$15>>0]|0;
  $17 = $16 << 24 >> 24;
  $18 = (($12) - ($17))|0;
  $19 = ($18*10)|0;
  HEAP32[$c>>2] = $19;
  $20 = HEAP32[$i>>2]|0;
  $21 = (($20) + 1)|0;
  HEAP32[$i>>2] = $21;
 }
 $22 = HEAP32[$c>>2]|0;
 $23 = ($22|0)==(0);
 if (!($23)) {
  $31 = HEAP32[$c>>2]|0;
  STACKTOP = sp;return ($31|0);
 }
 $24 = HEAP32[$1>>2]|0;
 $25 = ($24*18)|0;
 $26 = HEAP32[$0>>2]|0;
 $27 = (($26) + ($25)|0);
 $28 = HEAP8[$27>>0]|0;
 $29 = $28 << 24 >> 24;
 $30 = (($29) - 56)|0;
 HEAP32[$c>>2] = $30;
 $31 = HEAP32[$c>>2]|0;
 STACKTOP = sp;return ($31|0);
}
function _getSafetyLevel($z,$omitFull,$dflt) {
 $z = $z|0;
 $omitFull = $omitFull|0;
 $dflt = $dflt|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $5 = 0, $6 = 0, $7 = 0;
 var $8 = 0, $9 = 0, $i = 0, $n = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 17|0;
 $1 = sp + 12|0;
 $2 = sp + 8|0;
 $3 = sp + 16|0;
 $i = sp + 4|0;
 $n = sp;
 HEAP32[$1>>2] = $z;
 HEAP32[$2>>2] = $omitFull;
 HEAP8[$3>>0] = $dflt;
 $4 = HEAP32[$1>>2]|0;
 $5 = HEAP8[$4>>0]|0;
 $6 = $5&255;
 $7 = (1224 + ($6)|0);
 $8 = HEAP8[$7>>0]|0;
 $9 = $8&255;
 $10 = $9 & 4;
 $11 = ($10|0)!=(0);
 $12 = HEAP32[$1>>2]|0;
 if ($11) {
  $13 = (_sqlite3Atoi($12)|0);
  $14 = $13&255;
  HEAP8[$0>>0] = $14;
  $41 = HEAP8[$0>>0]|0;
  STACKTOP = sp;return ($41|0);
 }
 $15 = (_sqlite3Strlen30($12)|0);
 HEAP32[$n>>2] = $15;
 HEAP32[$i>>2] = 0;
 while(1) {
  $16 = HEAP32[$i>>2]|0;
  $17 = HEAP32[$2>>2]|0;
  $18 = (7 - ($17))|0;
  $19 = ($16|0)<($18|0);
  if (!($19)) {
   label = 9;
   break;
  }
  $20 = HEAP32[$i>>2]|0;
  $21 = (1800 + ($20)|0);
  $22 = HEAP8[$21>>0]|0;
  $23 = $22&255;
  $24 = HEAP32[$n>>2]|0;
  $25 = ($23|0)==($24|0);
  if ($25) {
   $26 = HEAP32[$i>>2]|0;
   $27 = (1792 + ($26)|0);
   $28 = HEAP8[$27>>0]|0;
   $29 = $28&255;
   $30 = (1768 + ($29)|0);
   $31 = HEAP32[$1>>2]|0;
   $32 = HEAP32[$n>>2]|0;
   $33 = (_sqlite3_strnicmp($30,$31,$32)|0);
   $34 = ($33|0)==(0);
   if ($34) {
    label = 7;
    break;
   }
  }
  $38 = HEAP32[$i>>2]|0;
  $39 = (($38) + 1)|0;
  HEAP32[$i>>2] = $39;
 }
 if ((label|0) == 7) {
  $35 = HEAP32[$i>>2]|0;
  $36 = (1808 + ($35)|0);
  $37 = HEAP8[$36>>0]|0;
  HEAP8[$0>>0] = $37;
  $41 = HEAP8[$0>>0]|0;
  STACKTOP = sp;return ($41|0);
 }
 else if ((label|0) == 9) {
  $40 = HEAP8[$3>>0]|0;
  HEAP8[$0>>0] = $40;
  $41 = HEAP8[$0>>0]|0;
  STACKTOP = sp;return ($41|0);
 }
 return 0|0;
}
function _sqlite3Atoi($z) {
 $z = $z|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $x = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $x = sp;
 HEAP32[$0>>2] = $z;
 HEAP32[$x>>2] = 0;
 $1 = HEAP32[$0>>2]|0;
 $2 = ($1|0)!=(0|0);
 if ($2) {
  $3 = HEAP32[$0>>2]|0;
  (_sqlite3GetInt32($3,$x)|0);
 }
 $4 = HEAP32[$x>>2]|0;
 STACKTOP = sp;return ($4|0);
}
function _sqlite3GetInt32($zNum,$pValue) {
 $zNum = $zNum|0;
 $pValue = $pValue|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0;
 var $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0;
 var $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0;
 var $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0;
 var $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $c = 0, $i = 0, $neg = 0, $or$cond = 0;
 var $or$cond3 = 0, $u = 0, $v = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $0 = sp + 32|0;
 $1 = sp + 28|0;
 $2 = sp + 24|0;
 $v = sp;
 $i = sp + 20|0;
 $c = sp + 16|0;
 $neg = sp + 12|0;
 $u = sp + 8|0;
 HEAP32[$1>>2] = $zNum;
 HEAP32[$2>>2] = $pValue;
 $3 = $v;
 $4 = $3;
 HEAP32[$4>>2] = 0;
 $5 = (($3) + 4)|0;
 $6 = $5;
 HEAP32[$6>>2] = 0;
 HEAP32[$neg>>2] = 0;
 $7 = HEAP32[$1>>2]|0;
 $8 = HEAP8[$7>>0]|0;
 $9 = $8 << 24 >> 24;
 $10 = ($9|0)==(45);
 do {
  if ($10) {
   HEAP32[$neg>>2] = 1;
   $11 = HEAP32[$1>>2]|0;
   $12 = (($11) + 1|0);
   HEAP32[$1>>2] = $12;
  } else {
   $13 = HEAP32[$1>>2]|0;
   $14 = HEAP8[$13>>0]|0;
   $15 = $14 << 24 >> 24;
   $16 = ($15|0)==(43);
   $17 = HEAP32[$1>>2]|0;
   if ($16) {
    $18 = (($17) + 1|0);
    HEAP32[$1>>2] = $18;
    break;
   }
   $19 = HEAP8[$17>>0]|0;
   $20 = $19 << 24 >> 24;
   $21 = ($20|0)==(48);
   if ($21) {
    $22 = HEAP32[$1>>2]|0;
    $23 = (($22) + 1|0);
    $24 = HEAP8[$23>>0]|0;
    $25 = $24 << 24 >> 24;
    $26 = ($25|0)==(120);
    if (!($26)) {
     $27 = HEAP32[$1>>2]|0;
     $28 = (($27) + 1|0);
     $29 = HEAP8[$28>>0]|0;
     $30 = $29 << 24 >> 24;
     $31 = ($30|0)==(88);
     if (!($31)) {
      break;
     }
    }
    $32 = HEAP32[$1>>2]|0;
    $33 = (($32) + 2|0);
    $34 = HEAP8[$33>>0]|0;
    $35 = $34&255;
    $36 = (1224 + ($35)|0);
    $37 = HEAP8[$36>>0]|0;
    $38 = $37&255;
    $39 = $38 & 8;
    $40 = ($39|0)!=(0);
    if ($40) {
     HEAP32[$u>>2] = 0;
     $41 = HEAP32[$1>>2]|0;
     $42 = (($41) + 2|0);
     HEAP32[$1>>2] = $42;
     while(1) {
      $43 = HEAP32[$1>>2]|0;
      $44 = HEAP8[$43>>0]|0;
      $45 = $44 << 24 >> 24;
      $46 = ($45|0)==(48);
      if (!($46)) {
       break;
      }
      $47 = HEAP32[$1>>2]|0;
      $48 = (($47) + 1|0);
      HEAP32[$1>>2] = $48;
     }
     HEAP32[$i>>2] = 0;
     while(1) {
      $49 = HEAP32[$i>>2]|0;
      $50 = HEAP32[$1>>2]|0;
      $51 = (($50) + ($49)|0);
      $52 = HEAP8[$51>>0]|0;
      $53 = $52&255;
      $54 = (1224 + ($53)|0);
      $55 = HEAP8[$54>>0]|0;
      $56 = $55&255;
      $57 = $56 & 8;
      $58 = ($57|0)!=(0);
      $59 = HEAP32[$i>>2]|0;
      $60 = ($59|0)<(8);
      $or$cond = $58 & $60;
      $61 = HEAP32[$u>>2]|0;
      if (!($or$cond)) {
       break;
      }
      $62 = $61<<4;
      $63 = HEAP32[$i>>2]|0;
      $64 = HEAP32[$1>>2]|0;
      $65 = (($64) + ($63)|0);
      $66 = HEAP8[$65>>0]|0;
      $67 = $66 << 24 >> 24;
      $68 = (_sqlite3HexToInt($67)|0);
      $69 = $68&255;
      $70 = (($62) + ($69))|0;
      HEAP32[$u>>2] = $70;
      $71 = HEAP32[$i>>2]|0;
      $72 = (($71) + 1)|0;
      HEAP32[$i>>2] = $72;
     }
     $73 = $61 & -2147483648;
     $74 = ($73|0)==(0);
     if ($74) {
      $75 = HEAP32[$i>>2]|0;
      $76 = HEAP32[$1>>2]|0;
      $77 = (($76) + ($75)|0);
      $78 = HEAP8[$77>>0]|0;
      $79 = $78&255;
      $80 = (1224 + ($79)|0);
      $81 = HEAP8[$80>>0]|0;
      $82 = $81&255;
      $83 = $82 & 8;
      $84 = ($83|0)==(0);
      if ($84) {
       $85 = HEAP32[$2>>2]|0;
       ;HEAP32[$85+0>>2]=HEAP32[$u+0>>2]|0;
       HEAP32[$0>>2] = 1;
       $155 = HEAP32[$0>>2]|0;
       STACKTOP = sp;return ($155|0);
      }
     }
     HEAP32[$0>>2] = 0;
     $155 = HEAP32[$0>>2]|0;
     STACKTOP = sp;return ($155|0);
    }
   }
  }
 } while(0);
 HEAP32[$i>>2] = 0;
 while(1) {
  $86 = HEAP32[$i>>2]|0;
  $87 = ($86|0)<(11);
  if (!($87)) {
   break;
  }
  $88 = HEAP32[$i>>2]|0;
  $89 = HEAP32[$1>>2]|0;
  $90 = (($89) + ($88)|0);
  $91 = HEAP8[$90>>0]|0;
  $92 = $91 << 24 >> 24;
  $93 = (($92) - 48)|0;
  HEAP32[$c>>2] = $93;
  $94 = ($93|0)>=(0);
  $95 = HEAP32[$c>>2]|0;
  $96 = ($95|0)<=(9);
  $or$cond3 = $94 & $96;
  if (!($or$cond3)) {
   break;
  }
  $97 = $v;
  $98 = $97;
  $99 = HEAP32[$98>>2]|0;
  $100 = (($97) + 4)|0;
  $101 = $100;
  $102 = HEAP32[$101>>2]|0;
  $103 = (___muldi3(($99|0),($102|0),10,0)|0);
  $104 = tempRet0;
  $105 = HEAP32[$c>>2]|0;
  $106 = ($105|0)<(0);
  $107 = $106 << 31 >> 31;
  $108 = (_i64Add(($103|0),($104|0),($105|0),($107|0))|0);
  $109 = tempRet0;
  $110 = $v;
  $111 = $110;
  HEAP32[$111>>2] = $108;
  $112 = (($110) + 4)|0;
  $113 = $112;
  HEAP32[$113>>2] = $109;
  $114 = HEAP32[$i>>2]|0;
  $115 = (($114) + 1)|0;
  HEAP32[$i>>2] = $115;
 }
 $116 = HEAP32[$i>>2]|0;
 $117 = ($116|0)>(10);
 if ($117) {
  HEAP32[$0>>2] = 0;
  $155 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($155|0);
 }
 $118 = $v;
 $119 = $118;
 $120 = HEAP32[$119>>2]|0;
 $121 = (($118) + 4)|0;
 $122 = $121;
 $123 = HEAP32[$122>>2]|0;
 $124 = HEAP32[$neg>>2]|0;
 $125 = ($124|0)<(0);
 $126 = $125 << 31 >> 31;
 $127 = (_i64Subtract(($120|0),($123|0),($124|0),($126|0))|0);
 $128 = tempRet0;
 $129 = ($128|0)>(0);
 $130 = ($127>>>0)>(2147483647);
 $131 = ($128|0)==(0);
 $132 = $131 & $130;
 $133 = $129 | $132;
 if ($133) {
  HEAP32[$0>>2] = 0;
  $155 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($155|0);
 }
 $134 = HEAP32[$neg>>2]|0;
 $135 = ($134|0)!=(0);
 if ($135) {
  $136 = $v;
  $137 = $136;
  $138 = HEAP32[$137>>2]|0;
  $139 = (($136) + 4)|0;
  $140 = $139;
  $141 = HEAP32[$140>>2]|0;
  $142 = (_i64Subtract(0,0,($138|0),($141|0))|0);
  $143 = tempRet0;
  $144 = $v;
  $145 = $144;
  HEAP32[$145>>2] = $142;
  $146 = (($144) + 4)|0;
  $147 = $146;
  HEAP32[$147>>2] = $143;
 }
 $148 = $v;
 $149 = $148;
 $150 = HEAP32[$149>>2]|0;
 $151 = (($148) + 4)|0;
 $152 = $151;
 $153 = HEAP32[$152>>2]|0;
 $154 = HEAP32[$2>>2]|0;
 HEAP32[$154>>2] = $150;
 HEAP32[$0>>2] = 1;
 $155 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($155|0);
}
function _sqlite3StatusSet($op,$X) {
 $op = $op|0;
 $X = $X|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $1 = sp;
 HEAP32[$0>>2] = $op;
 HEAP32[$1>>2] = $X;
 $2 = HEAP32[$1>>2]|0;
 $3 = HEAP32[$0>>2]|0;
 $4 = (48 + ($3<<2)|0);
 HEAP32[$4>>2] = $2;
 $5 = HEAP32[$0>>2]|0;
 $6 = (48 + ($5<<2)|0);
 $7 = HEAP32[$6>>2]|0;
 $8 = HEAP32[$0>>2]|0;
 $9 = (((48 + 40|0)) + ($8<<2)|0);
 $10 = HEAP32[$9>>2]|0;
 $11 = ($7|0)>($10|0);
 if (!($11)) {
  STACKTOP = sp;return;
 }
 $12 = HEAP32[$0>>2]|0;
 $13 = (48 + ($12<<2)|0);
 $14 = HEAP32[$13>>2]|0;
 $15 = HEAP32[$0>>2]|0;
 $16 = (((48 + 40|0)) + ($15<<2)|0);
 HEAP32[$16>>2] = $14;
 STACKTOP = sp;return;
}
function _keywordCode($z,$n) {
 $z = $z|0;
 $n = $n|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $h = 0, $i = 0;
 var label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 16|0;
 $1 = sp + 12|0;
 $2 = sp + 8|0;
 $h = sp + 4|0;
 $i = sp;
 HEAP32[$1>>2] = $z;
 HEAP32[$2>>2] = $n;
 $3 = HEAP32[$2>>2]|0;
 $4 = ($3|0)<(2);
 if ($4) {
  HEAP32[$0>>2] = 27;
  $57 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($57|0);
 }
 $5 = HEAP32[$1>>2]|0;
 $6 = HEAP8[$5>>0]|0;
 $7 = $6&255;
 $8 = (440 + ($7)|0);
 $9 = HEAP8[$8>>0]|0;
 $10 = $9&255;
 $11 = $10<<2;
 $12 = HEAP32[$2>>2]|0;
 $13 = (($12) - 1)|0;
 $14 = HEAP32[$1>>2]|0;
 $15 = (($14) + ($13)|0);
 $16 = HEAP8[$15>>0]|0;
 $17 = $16&255;
 $18 = (440 + ($17)|0);
 $19 = HEAP8[$18>>0]|0;
 $20 = $19&255;
 $21 = ($20*3)|0;
 $22 = $11 ^ $21;
 $23 = HEAP32[$2>>2]|0;
 $24 = $22 ^ $23;
 $25 = (($24|0) % 127)&-1;
 HEAP32[$h>>2] = $25;
 $26 = HEAP32[$h>>2]|0;
 $27 = (2376 + ($26)|0);
 $28 = HEAP8[$27>>0]|0;
 $29 = $28&255;
 $30 = (($29) - 1)|0;
 HEAP32[$i>>2] = $30;
 while(1) {
  $31 = HEAP32[$i>>2]|0;
  $32 = ($31|0)>=(0);
  if (!($32)) {
   label = 9;
   break;
  }
  $33 = HEAP32[$i>>2]|0;
  $34 = (2632 + ($33)|0);
  $35 = HEAP8[$34>>0]|0;
  $36 = $35&255;
  $37 = HEAP32[$2>>2]|0;
  $38 = ($36|0)==($37|0);
  if ($38) {
   $39 = HEAP32[$i>>2]|0;
   $40 = (2760 + ($39<<1)|0);
   $41 = HEAP16[$40>>1]|0;
   $42 = $41&65535;
   $43 = (1816 + ($42)|0);
   $44 = HEAP32[$1>>2]|0;
   $45 = HEAP32[$2>>2]|0;
   $46 = (_sqlite3_strnicmp($43,$44,$45)|0);
   $47 = ($46|0)==(0);
   if ($47) {
    label = 7;
    break;
   }
  }
  $52 = HEAP32[$i>>2]|0;
  $53 = (2504 + ($52)|0);
  $54 = HEAP8[$53>>0]|0;
  $55 = $54&255;
  $56 = (($55) - 1)|0;
  HEAP32[$i>>2] = $56;
 }
 if ((label|0) == 7) {
  $48 = HEAP32[$i>>2]|0;
  $49 = (3008 + ($48)|0);
  $50 = HEAP8[$49>>0]|0;
  $51 = $50&255;
  HEAP32[$0>>2] = $51;
  $57 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($57|0);
 }
 else if ((label|0) == 9) {
  HEAP32[$0>>2] = 27;
  $57 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($57|0);
 }
 return 0|0;
}
function _freeTempSpace($pBt) {
 $pBt = $pBt|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 HEAP32[$0>>2] = $pBt;
 $1 = HEAP32[$0>>2]|0;
 $2 = (($1) + 80|0);
 $3 = HEAP32[$2>>2]|0;
 $4 = ($3|0)!=(0|0);
 if (!($4)) {
  STACKTOP = sp;return;
 }
 $5 = HEAP32[$0>>2]|0;
 $6 = (($5) + 80|0);
 $7 = HEAP32[$6>>2]|0;
 $8 = (($7) + -4|0);
 HEAP32[$6>>2] = $8;
 $9 = HEAP32[$0>>2]|0;
 $10 = (($9) + 80|0);
 $11 = HEAP32[$10>>2]|0;
 _sqlite3PageFree($11);
 $12 = HEAP32[$0>>2]|0;
 $13 = (($12) + 80|0);
 HEAP32[$13>>2] = 0;
 STACKTOP = sp;return;
}
function _sqlite3PagerSetPagesize($pPager,$pPageSize,$nReserve) {
 $pPager = $pPager|0;
 $pPageSize = $pPageSize|0;
 $nReserve = $nReserve|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0;
 var $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0;
 var $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0;
 var $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0;
 var $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0;
 var $96 = 0, $97 = 0, $98 = 0, $99 = 0, $nByte = 0, $or$cond = 0, $pNew = 0, $pageSize = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 28|0;
 $1 = sp + 24|0;
 $2 = sp + 20|0;
 $rc = sp + 16|0;
 $pageSize = sp + 12|0;
 $pNew = sp + 8|0;
 $nByte = sp;
 HEAP32[$0>>2] = $pPager;
 HEAP32[$1>>2] = $pPageSize;
 HEAP32[$2>>2] = $nReserve;
 HEAP32[$rc>>2] = 0;
 $3 = HEAP32[$1>>2]|0;
 $4 = HEAP32[$3>>2]|0;
 HEAP32[$pageSize>>2] = $4;
 $5 = HEAP32[$0>>2]|0;
 $6 = (($5) + 15|0);
 $7 = HEAP8[$6>>0]|0;
 $8 = $7&255;
 $9 = ($8|0)==(0);
 if ($9) {
  label = 3;
 } else {
  $10 = HEAP32[$0>>2]|0;
  $11 = (($10) + 24|0);
  $12 = HEAP32[$11>>2]|0;
  $13 = ($12|0)==(0);
  if ($13) {
   label = 3;
  }
 }
 do {
  if ((label|0) == 3) {
   $14 = HEAP32[$0>>2]|0;
   $15 = (($14) + 204|0);
   $16 = HEAP32[$15>>2]|0;
   $17 = (_sqlite3PcacheRefCount($16)|0);
   $18 = ($17|0)==(0);
   $19 = HEAP32[$pageSize>>2]|0;
   $20 = ($19|0)!=(0);
   $or$cond = $18 & $20;
   if ($or$cond) {
    $21 = HEAP32[$pageSize>>2]|0;
    $22 = HEAP32[$0>>2]|0;
    $23 = (($22) + 152|0);
    $24 = HEAP32[$23>>2]|0;
    $25 = ($21|0)!=($24|0);
    if ($25) {
     HEAP32[$pNew>>2] = 0;
     $26 = $nByte;
     $27 = $26;
     HEAP32[$27>>2] = 0;
     $28 = (($26) + 4)|0;
     $29 = $28;
     HEAP32[$29>>2] = 0;
     $30 = HEAP32[$0>>2]|0;
     $31 = (($30) + 16|0);
     $32 = HEAP8[$31>>0]|0;
     $33 = $32&255;
     $34 = ($33|0)>(0);
     if ($34) {
      $35 = HEAP32[$0>>2]|0;
      $36 = (($35) + 60|0);
      $37 = HEAP32[$36>>2]|0;
      $38 = HEAP32[$37>>2]|0;
      $39 = ($38|0)!=(0|0);
      if ($39) {
       $40 = HEAP32[$0>>2]|0;
       $41 = (($40) + 60|0);
       $42 = HEAP32[$41>>2]|0;
       $43 = (_sqlite3OsFileSize($42,$nByte)|0);
       HEAP32[$rc>>2] = $43;
      }
     }
     $44 = HEAP32[$rc>>2]|0;
     $45 = ($44|0)==(0);
     if ($45) {
      $46 = HEAP32[$pageSize>>2]|0;
      $47 = (_sqlite3PageMalloc($46)|0);
      HEAP32[$pNew>>2] = $47;
      $48 = HEAP32[$pNew>>2]|0;
      $49 = ($48|0)!=(0|0);
      if (!($49)) {
       HEAP32[$rc>>2] = 7;
      }
     }
     $50 = HEAP32[$rc>>2]|0;
     $51 = ($50|0)==(0);
     if ($51) {
      $52 = HEAP32[$0>>2]|0;
      _pager_reset($52);
      $53 = HEAP32[$0>>2]|0;
      $54 = (($53) + 204|0);
      $55 = HEAP32[$54>>2]|0;
      $56 = HEAP32[$pageSize>>2]|0;
      $57 = (_sqlite3PcacheSetPageSize($55,$56)|0);
      HEAP32[$rc>>2] = $57;
     }
     $58 = HEAP32[$rc>>2]|0;
     $59 = ($58|0)==(0);
     if ($59) {
      $60 = HEAP32[$0>>2]|0;
      $61 = (($60) + 200|0);
      $62 = HEAP32[$61>>2]|0;
      _sqlite3PageFree($62);
      $63 = HEAP32[$pNew>>2]|0;
      $64 = HEAP32[$0>>2]|0;
      $65 = (($64) + 200|0);
      HEAP32[$65>>2] = $63;
      $66 = $nByte;
      $67 = $66;
      $68 = HEAP32[$67>>2]|0;
      $69 = (($66) + 4)|0;
      $70 = $69;
      $71 = HEAP32[$70>>2]|0;
      $72 = HEAP32[$pageSize>>2]|0;
      $73 = (_i64Add(($68|0),($71|0),($72|0),0)|0);
      $74 = tempRet0;
      $75 = (_i64Subtract(($73|0),($74|0),1,0)|0);
      $76 = tempRet0;
      $77 = HEAP32[$pageSize>>2]|0;
      $78 = (___divdi3(($75|0),($76|0),($77|0),0)|0);
      $79 = tempRet0;
      $80 = HEAP32[$0>>2]|0;
      $81 = (($80) + 24|0);
      HEAP32[$81>>2] = $78;
      $82 = HEAP32[$pageSize>>2]|0;
      $83 = HEAP32[$0>>2]|0;
      $84 = (($83) + 152|0);
      HEAP32[$84>>2] = $82;
      break;
     } else {
      $85 = HEAP32[$pNew>>2]|0;
      _sqlite3PageFree($85);
      break;
     }
    }
   }
  }
 } while(0);
 $86 = HEAP32[$0>>2]|0;
 $87 = (($86) + 152|0);
 $88 = HEAP32[$87>>2]|0;
 $89 = HEAP32[$1>>2]|0;
 HEAP32[$89>>2] = $88;
 $90 = HEAP32[$rc>>2]|0;
 $91 = ($90|0)==(0);
 if (!($91)) {
  $103 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($103|0);
 }
 $92 = HEAP32[$2>>2]|0;
 $93 = ($92|0)<(0);
 if ($93) {
  $94 = HEAP32[$0>>2]|0;
  $95 = (($94) + 142|0);
  $96 = HEAP16[$95>>1]|0;
  $97 = $96 << 16 >> 16;
  HEAP32[$2>>2] = $97;
 }
 $98 = HEAP32[$2>>2]|0;
 $99 = $98&65535;
 $100 = HEAP32[$0>>2]|0;
 $101 = (($100) + 142|0);
 HEAP16[$101>>1] = $99;
 $102 = HEAP32[$0>>2]|0;
 _pagerFixMaplimit($102);
 $103 = HEAP32[$rc>>2]|0;
 STACKTOP = sp;return ($103|0);
}
function _sqlite3PcacheRefCount($pCache) {
 $pCache = $pCache|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 HEAP32[$0>>2] = $pCache;
 $1 = HEAP32[$0>>2]|0;
 $2 = (($1) + 12|0);
 $3 = HEAP32[$2>>2]|0;
 STACKTOP = sp;return ($3|0);
}
function _sqlite3OsFileSize($id,$pSize) {
 $id = $id|0;
 $pSize = $pSize|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $1 = sp;
 HEAP32[$0>>2] = $id;
 HEAP32[$1>>2] = $pSize;
 $2 = HEAP32[$0>>2]|0;
 $3 = HEAP32[$2>>2]|0;
 $4 = (($3) + 24|0);
 $5 = HEAP32[$4>>2]|0;
 $6 = HEAP32[$0>>2]|0;
 $7 = HEAP32[$1>>2]|0;
 $8 = (FUNCTION_TABLE_iii[$5 & 63]($6,$7)|0);
 STACKTOP = sp;return ($8|0);
}
function _sqlite3PageMalloc($sz) {
 $sz = $sz|0;
 var $0 = 0, $1 = 0, $2 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 HEAP32[$0>>2] = $sz;
 $1 = HEAP32[$0>>2]|0;
 $2 = (_pcache1Alloc($1)|0);
 STACKTOP = sp;return ($2|0);
}
function _pager_reset($pPager) {
 $pPager = $pPager|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 HEAP32[$0>>2] = $pPager;
 $1 = HEAP32[$0>>2]|0;
 $2 = (($1) + 88|0);
 $3 = HEAP32[$2>>2]|0;
 _sqlite3BackupRestart($3);
 $4 = HEAP32[$0>>2]|0;
 $5 = (($4) + 204|0);
 $6 = HEAP32[$5>>2]|0;
 _sqlite3PcacheClear($6);
 STACKTOP = sp;return;
}
function _sqlite3PcacheSetPageSize($pCache,$szPage) {
 $pCache = $pCache|0;
 $szPage = $szPage|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0;
 var $9 = 0, $pNew = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $1 = sp + 8|0;
 $2 = sp + 4|0;
 $pNew = sp;
 HEAP32[$1>>2] = $pCache;
 HEAP32[$2>>2] = $szPage;
 $3 = HEAP32[$1>>2]|0;
 $4 = (($3) + 20|0);
 $5 = HEAP32[$4>>2]|0;
 $6 = ($5|0)!=(0);
 if ($6) {
  $7 = HEAP32[((38640 + 120|0))>>2]|0;
  $8 = HEAP32[$2>>2]|0;
  $9 = HEAP32[$1>>2]|0;
  $10 = (($9) + 24|0);
  $11 = HEAP32[$10>>2]|0;
  $12 = (($11) + 40)|0;
  $13 = HEAP32[$1>>2]|0;
  $14 = (($13) + 28|0);
  $15 = HEAP8[$14>>0]|0;
  $16 = $15&255;
  $17 = (FUNCTION_TABLE_iiii[$7 & 31]($8,$12,$16)|0);
  HEAP32[$pNew>>2] = $17;
  $18 = HEAP32[$pNew>>2]|0;
  $19 = ($18|0)==(0|0);
  if ($19) {
   HEAP32[$0>>2] = 7;
   $40 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($40|0);
  }
  $20 = HEAP32[((38640 + 124|0))>>2]|0;
  $21 = HEAP32[$pNew>>2]|0;
  $22 = HEAP32[$1>>2]|0;
  $23 = (_numberOfCachePages($22)|0);
  FUNCTION_TABLE_vii[$20 & 7]($21,$23);
  $24 = HEAP32[$1>>2]|0;
  $25 = (($24) + 40|0);
  $26 = HEAP32[$25>>2]|0;
  $27 = ($26|0)!=(0|0);
  if ($27) {
   $28 = HEAP32[((38640 + 148|0))>>2]|0;
   $29 = HEAP32[$1>>2]|0;
   $30 = (($29) + 40|0);
   $31 = HEAP32[$30>>2]|0;
   FUNCTION_TABLE_vi[$28 & 31]($31);
  }
  $32 = HEAP32[$pNew>>2]|0;
  $33 = HEAP32[$1>>2]|0;
  $34 = (($33) + 40|0);
  HEAP32[$34>>2] = $32;
  $35 = HEAP32[$1>>2]|0;
  $36 = (($35) + 44|0);
  HEAP32[$36>>2] = 0;
  $37 = HEAP32[$2>>2]|0;
  $38 = HEAP32[$1>>2]|0;
  $39 = (($38) + 20|0);
  HEAP32[$39>>2] = $37;
 }
 HEAP32[$0>>2] = 0;
 $40 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($40|0);
}
function _sqlite3PageFree($p) {
 $p = $p|0;
 var $0 = 0, $1 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 HEAP32[$0>>2] = $p;
 $1 = HEAP32[$0>>2]|0;
 (_pcache1Free($1)|0);
 STACKTOP = sp;return;
}
function _pagerFixMaplimit($pPager) {
 $pPager = $pPager|0;
 var $0 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 HEAP32[$0>>2] = $pPager;
 STACKTOP = sp;return;
}
function _pcache1Free($p) {
 $p = $p|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $nFreed = 0, $pSlot = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $1 = sp + 8|0;
 $nFreed = sp + 4|0;
 $pSlot = sp;
 HEAP32[$1>>2] = $p;
 HEAP32[$nFreed>>2] = 0;
 $2 = HEAP32[$1>>2]|0;
 $3 = ($2|0)==(0|0);
 if ($3) {
  HEAP32[$0>>2] = 0;
  $26 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($26|0);
 }
 $4 = HEAP32[$1>>2]|0;
 $5 = HEAP32[((3136 + 44|0))>>2]|0;
 $6 = ($4>>>0)>=($5>>>0);
 if ($6) {
  $7 = HEAP32[$1>>2]|0;
  $8 = HEAP32[((3136 + 48|0))>>2]|0;
  $9 = ($7>>>0)<($8>>>0);
  if ($9) {
   _sqlite3StatusAdd(1,-1);
   $10 = HEAP32[$1>>2]|0;
   HEAP32[$pSlot>>2] = $10;
   $11 = HEAP32[((3136 + 56|0))>>2]|0;
   $12 = HEAP32[$pSlot>>2]|0;
   HEAP32[$12>>2] = $11;
   $13 = HEAP32[$pSlot>>2]|0;
   HEAP32[((3136 + 56|0))>>2] = $13;
   $14 = HEAP32[((3136 + 60|0))>>2]|0;
   $15 = (($14) + 1)|0;
   HEAP32[((3136 + 60|0))>>2] = $15;
   $16 = HEAP32[((3136 + 60|0))>>2]|0;
   $17 = HEAP32[((3136 + 40|0))>>2]|0;
   $18 = ($16|0)<($17|0);
   $19 = $18&1;
   HEAP32[((3136 + 64|0))>>2] = $19;
  } else {
   label = 6;
  }
 } else {
  label = 6;
 }
 if ((label|0) == 6) {
  $20 = HEAP32[$1>>2]|0;
  $21 = (_sqlite3MallocSize($20)|0);
  HEAP32[$nFreed>>2] = $21;
  $22 = HEAP32[$nFreed>>2]|0;
  $23 = (0 - ($22))|0;
  _sqlite3StatusAdd(2,$23);
  $24 = HEAP32[$1>>2]|0;
  _sqlite3_free($24);
 }
 $25 = HEAP32[$nFreed>>2]|0;
 HEAP32[$0>>2] = $25;
 $26 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($26|0);
}
function _numberOfCachePages($p) {
 $p = $p|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $3 = 0, $4 = 0;
 var $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $1 = sp;
 HEAP32[$1>>2] = $p;
 $2 = HEAP32[$1>>2]|0;
 $3 = (($2) + 16|0);
 $4 = HEAP32[$3>>2]|0;
 $5 = ($4|0)>=(0);
 $6 = HEAP32[$1>>2]|0;
 $7 = (($6) + 16|0);
 $8 = HEAP32[$7>>2]|0;
 if ($5) {
  HEAP32[$0>>2] = $8;
  $24 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($24|0);
 } else {
  $9 = ($8|0)<(0);
  $10 = $9 << 31 >> 31;
  $11 = (___muldi3(-1024,-1,($8|0),($10|0))|0);
  $12 = tempRet0;
  $13 = HEAP32[$1>>2]|0;
  $14 = (($13) + 20|0);
  $15 = HEAP32[$14>>2]|0;
  $16 = HEAP32[$1>>2]|0;
  $17 = (($16) + 24|0);
  $18 = HEAP32[$17>>2]|0;
  $19 = (($15) + ($18))|0;
  $20 = ($19|0)<(0);
  $21 = $20 << 31 >> 31;
  $22 = (___divdi3(($11|0),($12|0),($19|0),($21|0))|0);
  $23 = tempRet0;
  HEAP32[$0>>2] = $22;
  $24 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($24|0);
 }
 return 0|0;
}
function _sqlite3BackupRestart($pBackup) {
 $pBackup = $pBackup|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $p = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $p = sp;
 HEAP32[$0>>2] = $pBackup;
 $1 = HEAP32[$0>>2]|0;
 HEAP32[$p>>2] = $1;
 while(1) {
  $2 = HEAP32[$p>>2]|0;
  $3 = ($2|0)!=(0|0);
  if (!($3)) {
   break;
  }
  $4 = HEAP32[$p>>2]|0;
  $5 = (($4) + 16|0);
  HEAP32[$5>>2] = 1;
  $6 = HEAP32[$p>>2]|0;
  $7 = (($6) + 44|0);
  $8 = HEAP32[$7>>2]|0;
  HEAP32[$p>>2] = $8;
 }
 STACKTOP = sp;return;
}
function _sqlite3PcacheClear($pCache) {
 $pCache = $pCache|0;
 var $0 = 0, $1 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 HEAP32[$0>>2] = $pCache;
 $1 = HEAP32[$0>>2]|0;
 _sqlite3PcacheTruncate($1,0);
 STACKTOP = sp;return;
}
function _sqlite3PcacheTruncate($pCache,$pgno) {
 $pCache = $pCache|0;
 $pgno = $pgno|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0;
 var $p = 0, $pNext = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $1 = sp + 8|0;
 $p = sp + 4|0;
 $pNext = sp;
 HEAP32[$0>>2] = $pCache;
 HEAP32[$1>>2] = $pgno;
 $2 = HEAP32[$0>>2]|0;
 $3 = (($2) + 40|0);
 $4 = HEAP32[$3>>2]|0;
 $5 = ($4|0)!=(0|0);
 if (!($5)) {
  STACKTOP = sp;return;
 }
 $6 = HEAP32[$0>>2]|0;
 $7 = HEAP32[$6>>2]|0;
 HEAP32[$p>>2] = $7;
 while(1) {
  $8 = HEAP32[$p>>2]|0;
  $9 = ($8|0)!=(0|0);
  if (!($9)) {
   break;
  }
  $10 = HEAP32[$p>>2]|0;
  $11 = (($10) + 32|0);
  $12 = HEAP32[$11>>2]|0;
  HEAP32[$pNext>>2] = $12;
  $13 = HEAP32[$p>>2]|0;
  $14 = (($13) + 20|0);
  $15 = HEAP32[$14>>2]|0;
  $16 = HEAP32[$1>>2]|0;
  $17 = ($15>>>0)>($16>>>0);
  if ($17) {
   $18 = HEAP32[$p>>2]|0;
   _sqlite3PcacheMakeClean($18);
  }
  $19 = HEAP32[$pNext>>2]|0;
  HEAP32[$p>>2] = $19;
 }
 $20 = HEAP32[$1>>2]|0;
 $21 = ($20|0)==(0);
 if ($21) {
  $22 = HEAP32[$0>>2]|0;
  $23 = (($22) + 44|0);
  $24 = HEAP32[$23>>2]|0;
  $25 = ($24|0)!=(0|0);
  if ($25) {
   $26 = HEAP32[$0>>2]|0;
   $27 = (($26) + 44|0);
   $28 = HEAP32[$27>>2]|0;
   $29 = (($28) + 4|0);
   $30 = HEAP32[$29>>2]|0;
   $31 = HEAP32[$0>>2]|0;
   $32 = (($31) + 20|0);
   $33 = HEAP32[$32>>2]|0;
   _memset(($30|0),0,($33|0))|0;
   HEAP32[$1>>2] = 1;
  }
 }
 $34 = HEAP32[((38640 + 144|0))>>2]|0;
 $35 = HEAP32[$0>>2]|0;
 $36 = (($35) + 40|0);
 $37 = HEAP32[$36>>2]|0;
 $38 = HEAP32[$1>>2]|0;
 $39 = (($38) + 1)|0;
 FUNCTION_TABLE_vii[$34 & 7]($37,$39);
 STACKTOP = sp;return;
}
function _sqlite3PcacheMakeClean($p) {
 $p = $p|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0;
 var label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 HEAP32[$0>>2] = $p;
 $1 = HEAP32[$0>>2]|0;
 $2 = (($1) + 24|0);
 $3 = HEAP16[$2>>1]|0;
 $4 = $3&65535;
 $5 = $4 & 2;
 $6 = ($5|0)!=(0);
 if (!($6)) {
  STACKTOP = sp;return;
 }
 $7 = HEAP32[$0>>2]|0;
 _pcacheManageDirtyList($7,1);
 $8 = HEAP32[$0>>2]|0;
 $9 = (($8) + 24|0);
 $10 = HEAP16[$9>>1]|0;
 $11 = $10&65535;
 $12 = $11 & -7;
 $13 = $12&65535;
 HEAP16[$9>>1] = $13;
 $14 = HEAP32[$0>>2]|0;
 $15 = (($14) + 26|0);
 $16 = HEAP16[$15>>1]|0;
 $17 = $16 << 16 >> 16;
 $18 = ($17|0)==(0);
 if (!($18)) {
  STACKTOP = sp;return;
 }
 $19 = HEAP32[$0>>2]|0;
 _pcacheUnpin($19);
 STACKTOP = sp;return;
}
function _pcacheManageDirtyList($pPage,$addRemove) {
 $pPage = $pPage|0;
 $addRemove = $addRemove|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0;
 var $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0;
 var $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0;
 var $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0;
 var $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0;
 var $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $p = 0, $pSynced = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $1 = sp + 12|0;
 $p = sp + 4|0;
 $pSynced = sp;
 HEAP32[$0>>2] = $pPage;
 HEAP8[$1>>0] = $addRemove;
 $2 = HEAP32[$0>>2]|0;
 $3 = (($2) + 28|0);
 $4 = HEAP32[$3>>2]|0;
 HEAP32[$p>>2] = $4;
 $5 = HEAP8[$1>>0]|0;
 $6 = $5&255;
 $7 = $6 & 1;
 $8 = ($7|0)!=(0);
 if ($8) {
  $9 = HEAP32[$p>>2]|0;
  $10 = (($9) + 8|0);
  $11 = HEAP32[$10>>2]|0;
  $12 = HEAP32[$0>>2]|0;
  $13 = ($11|0)==($12|0);
  if ($13) {
   $14 = HEAP32[$0>>2]|0;
   $15 = (($14) + 36|0);
   $16 = HEAP32[$15>>2]|0;
   HEAP32[$pSynced>>2] = $16;
   while(1) {
    $17 = HEAP32[$pSynced>>2]|0;
    $18 = ($17|0)!=(0|0);
    if ($18) {
     $19 = HEAP32[$pSynced>>2]|0;
     $20 = (($19) + 24|0);
     $21 = HEAP16[$20>>1]|0;
     $22 = $21&65535;
     $23 = $22 & 4;
     $24 = ($23|0)!=(0);
     $109 = $24;
    } else {
     $109 = 0;
    }
    $25 = HEAP32[$pSynced>>2]|0;
    if (!($109)) {
     break;
    }
    $26 = (($25) + 36|0);
    $27 = HEAP32[$26>>2]|0;
    HEAP32[$pSynced>>2] = $27;
   }
   $28 = HEAP32[$p>>2]|0;
   $29 = (($28) + 8|0);
   HEAP32[$29>>2] = $25;
  }
  $30 = HEAP32[$0>>2]|0;
  $31 = (($30) + 32|0);
  $32 = HEAP32[$31>>2]|0;
  $33 = ($32|0)!=(0|0);
  $34 = HEAP32[$0>>2]|0;
  $35 = (($34) + 36|0);
  $36 = HEAP32[$35>>2]|0;
  if ($33) {
   $37 = HEAP32[$0>>2]|0;
   $38 = (($37) + 32|0);
   $39 = HEAP32[$38>>2]|0;
   $40 = (($39) + 36|0);
   HEAP32[$40>>2] = $36;
  } else {
   $41 = HEAP32[$p>>2]|0;
   $42 = (($41) + 4|0);
   HEAP32[$42>>2] = $36;
  }
  $43 = HEAP32[$0>>2]|0;
  $44 = (($43) + 36|0);
  $45 = HEAP32[$44>>2]|0;
  $46 = ($45|0)!=(0|0);
  $47 = HEAP32[$0>>2]|0;
  $48 = (($47) + 32|0);
  $49 = HEAP32[$48>>2]|0;
  if ($46) {
   $50 = HEAP32[$0>>2]|0;
   $51 = (($50) + 36|0);
   $52 = HEAP32[$51>>2]|0;
   $53 = (($52) + 32|0);
   HEAP32[$53>>2] = $49;
  } else {
   $54 = HEAP32[$p>>2]|0;
   HEAP32[$54>>2] = $49;
   $55 = HEAP32[$p>>2]|0;
   $56 = HEAP32[$55>>2]|0;
   $57 = ($56|0)==(0|0);
   if ($57) {
    $58 = HEAP32[$p>>2]|0;
    $59 = (($58) + 28|0);
    $60 = HEAP8[$59>>0]|0;
    $61 = $60&255;
    $62 = ($61|0)!=(0);
    if ($62) {
     $63 = HEAP32[$p>>2]|0;
     $64 = (($63) + 29|0);
     HEAP8[$64>>0] = 2;
    }
   }
  }
  $65 = HEAP32[$0>>2]|0;
  $66 = (($65) + 32|0);
  HEAP32[$66>>2] = 0;
  $67 = HEAP32[$0>>2]|0;
  $68 = (($67) + 36|0);
  HEAP32[$68>>2] = 0;
 }
 $69 = HEAP8[$1>>0]|0;
 $70 = $69&255;
 $71 = $70 & 2;
 $72 = ($71|0)!=(0);
 if (!($72)) {
  STACKTOP = sp;return;
 }
 $73 = HEAP32[$p>>2]|0;
 $74 = HEAP32[$73>>2]|0;
 $75 = HEAP32[$0>>2]|0;
 $76 = (($75) + 32|0);
 HEAP32[$76>>2] = $74;
 $77 = HEAP32[$0>>2]|0;
 $78 = (($77) + 32|0);
 $79 = HEAP32[$78>>2]|0;
 $80 = ($79|0)!=(0|0);
 $81 = HEAP32[$0>>2]|0;
 if ($80) {
  $82 = HEAP32[$0>>2]|0;
  $83 = (($82) + 32|0);
  $84 = HEAP32[$83>>2]|0;
  $85 = (($84) + 36|0);
  HEAP32[$85>>2] = $81;
 } else {
  $86 = HEAP32[$p>>2]|0;
  $87 = (($86) + 4|0);
  HEAP32[$87>>2] = $81;
  $88 = HEAP32[$p>>2]|0;
  $89 = (($88) + 28|0);
  $90 = HEAP8[$89>>0]|0;
  $91 = ($90<<24>>24)!=(0);
  if ($91) {
   $92 = HEAP32[$p>>2]|0;
   $93 = (($92) + 29|0);
   HEAP8[$93>>0] = 1;
  }
 }
 $94 = HEAP32[$0>>2]|0;
 $95 = HEAP32[$p>>2]|0;
 HEAP32[$95>>2] = $94;
 $96 = HEAP32[$p>>2]|0;
 $97 = (($96) + 8|0);
 $98 = HEAP32[$97>>2]|0;
 $99 = ($98|0)!=(0|0);
 if ($99) {
  STACKTOP = sp;return;
 }
 $100 = HEAP32[$0>>2]|0;
 $101 = (($100) + 24|0);
 $102 = HEAP16[$101>>1]|0;
 $103 = $102&65535;
 $104 = $103 & 4;
 $105 = (0)==($104|0);
 if (!($105)) {
  STACKTOP = sp;return;
 }
 $106 = HEAP32[$0>>2]|0;
 $107 = HEAP32[$p>>2]|0;
 $108 = (($107) + 8|0);
 HEAP32[$108>>2] = $106;
 STACKTOP = sp;return;
}
function _pcacheUnpin($p) {
 $p = $p|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0;
 var $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 HEAP32[$0>>2] = $p;
 $1 = HEAP32[$0>>2]|0;
 $2 = (($1) + 28|0);
 $3 = HEAP32[$2>>2]|0;
 $4 = (($3) + 28|0);
 $5 = HEAP8[$4>>0]|0;
 $6 = ($5<<24>>24)!=(0);
 if (!($6)) {
  STACKTOP = sp;return;
 }
 $7 = HEAP32[$0>>2]|0;
 $8 = (($7) + 20|0);
 $9 = HEAP32[$8>>2]|0;
 $10 = ($9|0)==(1);
 if ($10) {
  $11 = HEAP32[$0>>2]|0;
  $12 = (($11) + 28|0);
  $13 = HEAP32[$12>>2]|0;
  $14 = (($13) + 44|0);
  HEAP32[$14>>2] = 0;
 }
 $15 = HEAP32[((38640 + 136|0))>>2]|0;
 $16 = HEAP32[$0>>2]|0;
 $17 = (($16) + 28|0);
 $18 = HEAP32[$17>>2]|0;
 $19 = (($18) + 40|0);
 $20 = HEAP32[$19>>2]|0;
 $21 = HEAP32[$0>>2]|0;
 $22 = HEAP32[$21>>2]|0;
 FUNCTION_TABLE_viii[$15 & 63]($20,$22,0);
 STACKTOP = sp;return;
}
function _pcache1Alloc($nByte) {
 $nByte = $nByte|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $p = 0, $sz = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $p = sp + 4|0;
 $sz = sp;
 HEAP32[$0>>2] = $nByte;
 HEAP32[$p>>2] = 0;
 $1 = HEAP32[$0>>2]|0;
 _sqlite3StatusSet(7,$1);
 $2 = HEAP32[$0>>2]|0;
 $3 = HEAP32[((3136 + 32|0))>>2]|0;
 $4 = ($2|0)<=($3|0);
 if ($4) {
  $5 = HEAP32[((3136 + 56|0))>>2]|0;
  HEAP32[$p>>2] = $5;
  $6 = HEAP32[$p>>2]|0;
  $7 = ($6|0)!=(0|0);
  if ($7) {
   $8 = HEAP32[((3136 + 56|0))>>2]|0;
   $9 = HEAP32[$8>>2]|0;
   HEAP32[((3136 + 56|0))>>2] = $9;
   $10 = HEAP32[((3136 + 60|0))>>2]|0;
   $11 = (($10) + -1)|0;
   HEAP32[((3136 + 60|0))>>2] = $11;
   $12 = HEAP32[((3136 + 60|0))>>2]|0;
   $13 = HEAP32[((3136 + 40|0))>>2]|0;
   $14 = ($12|0)<($13|0);
   $15 = $14&1;
   HEAP32[((3136 + 64|0))>>2] = $15;
   _sqlite3StatusAdd(1,1);
  }
 }
 $16 = HEAP32[$p>>2]|0;
 $17 = ($16|0)==(0|0);
 if (!($17)) {
  $27 = HEAP32[$p>>2]|0;
  STACKTOP = sp;return ($27|0);
 }
 $18 = HEAP32[$0>>2]|0;
 $19 = ($18|0)<(0);
 $20 = $19 << 31 >> 31;
 $21 = (_sqlite3Malloc($18,$20)|0);
 HEAP32[$p>>2] = $21;
 $22 = HEAP32[$p>>2]|0;
 $23 = ($22|0)!=(0|0);
 if (!($23)) {
  $27 = HEAP32[$p>>2]|0;
  STACKTOP = sp;return ($27|0);
 }
 $24 = HEAP32[$p>>2]|0;
 $25 = (_sqlite3MallocSize($24)|0);
 HEAP32[$sz>>2] = $25;
 $26 = HEAP32[$sz>>2]|0;
 _sqlite3StatusAdd(2,$26);
 $27 = HEAP32[$p>>2]|0;
 STACKTOP = sp;return ($27|0);
}
function _sqlite3BitvecCreate($iSize) {
 $iSize = $iSize|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $p = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $p = sp;
 HEAP32[$0>>2] = $iSize;
 $1 = (_sqlite3MallocZero(512,0)|0);
 HEAP32[$p>>2] = $1;
 $2 = HEAP32[$p>>2]|0;
 $3 = ($2|0)!=(0|0);
 if (!($3)) {
  $6 = HEAP32[$p>>2]|0;
  STACKTOP = sp;return ($6|0);
 }
 $4 = HEAP32[$0>>2]|0;
 $5 = HEAP32[$p>>2]|0;
 HEAP32[$5>>2] = $4;
 $6 = HEAP32[$p>>2]|0;
 STACKTOP = sp;return ($6|0);
}
function _sqlite3BitvecSet($p,$i) {
 $p = $p|0;
 $i = $i|0;
 var $$ = 0, $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0;
 var $115 = 0, $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0;
 var $133 = 0, $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0;
 var $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0;
 var $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0;
 var $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0;
 var $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0;
 var $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $aiValues = 0, $bin = 0, $h = 0, $j = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 28|0;
 $1 = sp + 24|0;
 $2 = sp + 20|0;
 $h = sp + 16|0;
 $bin = sp + 12|0;
 $j = sp + 8|0;
 $rc = sp + 4|0;
 $aiValues = sp;
 HEAP32[$1>>2] = $p;
 HEAP32[$2>>2] = $i;
 $3 = HEAP32[$1>>2]|0;
 $4 = ($3|0)==(0|0);
 if ($4) {
  HEAP32[$0>>2] = 0;
  $144 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($144|0);
 }
 $5 = HEAP32[$2>>2]|0;
 $6 = (($5) + -1)|0;
 HEAP32[$2>>2] = $6;
 while(1) {
  $7 = HEAP32[$1>>2]|0;
  $8 = HEAP32[$7>>2]|0;
  $9 = ($8>>>0)>(4000);
  if (!($9)) {
   break;
  }
  $10 = HEAP32[$1>>2]|0;
  $11 = (($10) + 8|0);
  $12 = HEAP32[$11>>2]|0;
  $13 = ($12|0)!=(0);
  if (!($13)) {
   break;
  }
  $14 = HEAP32[$2>>2]|0;
  $15 = HEAP32[$1>>2]|0;
  $16 = (($15) + 8|0);
  $17 = HEAP32[$16>>2]|0;
  $18 = (($14>>>0) / ($17>>>0))&-1;
  HEAP32[$bin>>2] = $18;
  $19 = HEAP32[$2>>2]|0;
  $20 = HEAP32[$1>>2]|0;
  $21 = (($20) + 8|0);
  $22 = HEAP32[$21>>2]|0;
  $23 = (($19>>>0) % ($22>>>0))&-1;
  HEAP32[$2>>2] = $23;
  $24 = HEAP32[$bin>>2]|0;
  $25 = HEAP32[$1>>2]|0;
  $26 = (($25) + 12|0);
  $27 = (($26) + ($24<<2)|0);
  $28 = HEAP32[$27>>2]|0;
  $29 = ($28|0)==(0|0);
  if ($29) {
   $30 = HEAP32[$1>>2]|0;
   $31 = (($30) + 8|0);
   $32 = HEAP32[$31>>2]|0;
   $33 = (_sqlite3BitvecCreate($32)|0);
   $34 = HEAP32[$bin>>2]|0;
   $35 = HEAP32[$1>>2]|0;
   $36 = (($35) + 12|0);
   $37 = (($36) + ($34<<2)|0);
   HEAP32[$37>>2] = $33;
   $38 = HEAP32[$bin>>2]|0;
   $39 = HEAP32[$1>>2]|0;
   $40 = (($39) + 12|0);
   $41 = (($40) + ($38<<2)|0);
   $42 = HEAP32[$41>>2]|0;
   $43 = ($42|0)==(0|0);
   if ($43) {
    label = 8;
    break;
   }
  }
  $44 = HEAP32[$bin>>2]|0;
  $45 = HEAP32[$1>>2]|0;
  $46 = (($45) + 12|0);
  $47 = (($46) + ($44<<2)|0);
  $48 = HEAP32[$47>>2]|0;
  HEAP32[$1>>2] = $48;
 }
 if ((label|0) == 8) {
  HEAP32[$0>>2] = 7;
  $144 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($144|0);
 }
 $49 = HEAP32[$1>>2]|0;
 $50 = HEAP32[$49>>2]|0;
 $51 = ($50>>>0)<=(4000);
 $52 = HEAP32[$2>>2]|0;
 if ($51) {
  $53 = $52 & 7;
  $54 = 1 << $53;
  $55 = HEAP32[$2>>2]|0;
  $56 = (($55>>>0) / 8)&-1;
  $57 = HEAP32[$1>>2]|0;
  $58 = (($57) + 12|0);
  $59 = (($58) + ($56)|0);
  $60 = HEAP8[$59>>0]|0;
  $61 = $60&255;
  $62 = $61 | $54;
  $63 = $62&255;
  HEAP8[$59>>0] = $63;
  HEAP32[$0>>2] = 0;
  $144 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($144|0);
 }
 $64 = (($52) + 1)|0;
 HEAP32[$2>>2] = $64;
 $65 = $52;
 $66 = (($65>>>0) % 125)&-1;
 HEAP32[$h>>2] = $66;
 $67 = HEAP32[$h>>2]|0;
 $68 = HEAP32[$1>>2]|0;
 $69 = (($68) + 12|0);
 $70 = (($69) + ($67<<2)|0);
 $71 = HEAP32[$70>>2]|0;
 $72 = ($71|0)!=(0);
 L20: do {
  if ($72) {
   while(1) {
    $77 = HEAP32[$h>>2]|0;
    $78 = HEAP32[$1>>2]|0;
    $79 = (($78) + 12|0);
    $80 = (($79) + ($77<<2)|0);
    $81 = HEAP32[$80>>2]|0;
    $82 = HEAP32[$2>>2]|0;
    $83 = ($81|0)==($82|0);
    if ($83) {
     break;
    }
    $84 = HEAP32[$h>>2]|0;
    $85 = (($84) + 1)|0;
    HEAP32[$h>>2] = $85;
    $86 = HEAP32[$h>>2]|0;
    $87 = ($86>>>0)>=(125);
    $$ = $87 ? 0 : $85;
    HEAP32[$h>>2] = $$;
    $88 = HEAP32[$h>>2]|0;
    $89 = HEAP32[$1>>2]|0;
    $90 = (($89) + 12|0);
    $91 = (($90) + ($88<<2)|0);
    $92 = HEAP32[$91>>2]|0;
    $93 = ($92|0)!=(0);
    if (!($93)) {
     label = 17;
     break L20;
    }
   }
   HEAP32[$0>>2] = 0;
   $144 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($144|0);
  } else {
   $73 = HEAP32[$1>>2]|0;
   $74 = (($73) + 4|0);
   $75 = HEAP32[$74>>2]|0;
   $76 = ($75>>>0)<(124);
   if (!($76)) {
    label = 17;
   }
  }
 } while(0);
 if ((label|0) == 17) {
  $94 = HEAP32[$1>>2]|0;
  $95 = (($94) + 4|0);
  $96 = HEAP32[$95>>2]|0;
  $97 = ($96>>>0)>=(62);
  if ($97) {
   $98 = (_sqlite3DbMallocRaw(0,500,0)|0);
   HEAP32[$aiValues>>2] = $98;
   $99 = HEAP32[$aiValues>>2]|0;
   $100 = ($99|0)==(0|0);
   if ($100) {
    HEAP32[$0>>2] = 7;
    $144 = HEAP32[$0>>2]|0;
    STACKTOP = sp;return ($144|0);
   }
   $101 = HEAP32[$aiValues>>2]|0;
   $102 = HEAP32[$1>>2]|0;
   $103 = (($102) + 12|0);
   _memcpy(($101|0),($103|0),500)|0;
   $104 = HEAP32[$1>>2]|0;
   $105 = (($104) + 12|0);
   _memset(($105|0),0,500)|0;
   $106 = HEAP32[$1>>2]|0;
   $107 = HEAP32[$106>>2]|0;
   $108 = (($107) + 125)|0;
   $109 = (($108) - 1)|0;
   $110 = (($109>>>0) / 125)&-1;
   $111 = HEAP32[$1>>2]|0;
   $112 = (($111) + 8|0);
   HEAP32[$112>>2] = $110;
   $113 = HEAP32[$1>>2]|0;
   $114 = HEAP32[$2>>2]|0;
   $115 = (_sqlite3BitvecSet($113,$114)|0);
   HEAP32[$rc>>2] = $115;
   HEAP32[$j>>2] = 0;
   while(1) {
    $116 = HEAP32[$j>>2]|0;
    $117 = ($116>>>0)<(125);
    if (!($117)) {
     break;
    }
    $118 = HEAP32[$j>>2]|0;
    $119 = HEAP32[$aiValues>>2]|0;
    $120 = (($119) + ($118<<2)|0);
    $121 = HEAP32[$120>>2]|0;
    $122 = ($121|0)!=(0);
    if ($122) {
     $123 = HEAP32[$1>>2]|0;
     $124 = HEAP32[$j>>2]|0;
     $125 = HEAP32[$aiValues>>2]|0;
     $126 = (($125) + ($124<<2)|0);
     $127 = HEAP32[$126>>2]|0;
     $128 = (_sqlite3BitvecSet($123,$127)|0);
     $129 = HEAP32[$rc>>2]|0;
     $130 = $129 | $128;
     HEAP32[$rc>>2] = $130;
    }
    $131 = HEAP32[$j>>2]|0;
    $132 = (($131) + 1)|0;
    HEAP32[$j>>2] = $132;
   }
   $133 = HEAP32[$aiValues>>2]|0;
   _sqlite3DbFree(0,$133);
   $134 = HEAP32[$rc>>2]|0;
   HEAP32[$0>>2] = $134;
   $144 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($144|0);
  }
 }
 $135 = HEAP32[$1>>2]|0;
 $136 = (($135) + 4|0);
 $137 = HEAP32[$136>>2]|0;
 $138 = (($137) + 1)|0;
 HEAP32[$136>>2] = $138;
 $139 = HEAP32[$2>>2]|0;
 $140 = HEAP32[$h>>2]|0;
 $141 = HEAP32[$1>>2]|0;
 $142 = (($141) + 12|0);
 $143 = (($142) + ($140<<2)|0);
 HEAP32[$143>>2] = $139;
 HEAP32[$0>>2] = 0;
 $144 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($144|0);
}
function _sqlite3BitvecClear($p,$i,$pBuf) {
 $p = $p|0;
 $i = $i|0;
 $pBuf = $pBuf|0;
 var $$ = 0, $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0;
 var $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0;
 var $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0;
 var $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0;
 var $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $aiValues = 0, $bin = 0;
 var $h = 0, $j = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 24|0;
 $1 = sp + 20|0;
 $2 = sp + 16|0;
 $bin = sp + 12|0;
 $j = sp + 8|0;
 $aiValues = sp + 4|0;
 $h = sp;
 HEAP32[$0>>2] = $p;
 HEAP32[$1>>2] = $i;
 HEAP32[$2>>2] = $pBuf;
 $3 = HEAP32[$0>>2]|0;
 $4 = ($3|0)==(0|0);
 if ($4) {
  STACKTOP = sp;return;
 }
 $5 = HEAP32[$1>>2]|0;
 $6 = (($5) + -1)|0;
 HEAP32[$1>>2] = $6;
 while(1) {
  $7 = HEAP32[$0>>2]|0;
  $8 = (($7) + 8|0);
  $9 = HEAP32[$8>>2]|0;
  $10 = ($9|0)!=(0);
  if (!($10)) {
   break;
  }
  $11 = HEAP32[$1>>2]|0;
  $12 = HEAP32[$0>>2]|0;
  $13 = (($12) + 8|0);
  $14 = HEAP32[$13>>2]|0;
  $15 = (($11>>>0) / ($14>>>0))&-1;
  HEAP32[$bin>>2] = $15;
  $16 = HEAP32[$1>>2]|0;
  $17 = HEAP32[$0>>2]|0;
  $18 = (($17) + 8|0);
  $19 = HEAP32[$18>>2]|0;
  $20 = (($16>>>0) % ($19>>>0))&-1;
  HEAP32[$1>>2] = $20;
  $21 = HEAP32[$bin>>2]|0;
  $22 = HEAP32[$0>>2]|0;
  $23 = (($22) + 12|0);
  $24 = (($23) + ($21<<2)|0);
  $25 = HEAP32[$24>>2]|0;
  HEAP32[$0>>2] = $25;
  $26 = HEAP32[$0>>2]|0;
  $27 = ($26|0)!=(0|0);
  if (!($27)) {
   label = 16;
   break;
  }
 }
 if ((label|0) == 16) {
  STACKTOP = sp;return;
 }
 $28 = HEAP32[$0>>2]|0;
 $29 = HEAP32[$28>>2]|0;
 $30 = ($29>>>0)<=(4000);
 if ($30) {
  $31 = HEAP32[$1>>2]|0;
  $32 = $31 & 7;
  $33 = 1 << $32;
  $34 = $33 ^ -1;
  $35 = HEAP32[$1>>2]|0;
  $36 = (($35>>>0) / 8)&-1;
  $37 = HEAP32[$0>>2]|0;
  $38 = (($37) + 12|0);
  $39 = (($38) + ($36)|0);
  $40 = HEAP8[$39>>0]|0;
  $41 = $40&255;
  $42 = $41 & $34;
  $43 = $42&255;
  HEAP8[$39>>0] = $43;
  STACKTOP = sp;return;
 }
 $44 = HEAP32[$2>>2]|0;
 HEAP32[$aiValues>>2] = $44;
 $45 = HEAP32[$aiValues>>2]|0;
 $46 = HEAP32[$0>>2]|0;
 $47 = (($46) + 12|0);
 _memcpy(($45|0),($47|0),500)|0;
 $48 = HEAP32[$0>>2]|0;
 $49 = (($48) + 12|0);
 _memset(($49|0),0,500)|0;
 $50 = HEAP32[$0>>2]|0;
 $51 = (($50) + 4|0);
 HEAP32[$51>>2] = 0;
 HEAP32[$j>>2] = 0;
 while(1) {
  $52 = HEAP32[$j>>2]|0;
  $53 = ($52>>>0)<(125);
  if (!($53)) {
   break;
  }
  $54 = HEAP32[$j>>2]|0;
  $55 = HEAP32[$aiValues>>2]|0;
  $56 = (($55) + ($54<<2)|0);
  $57 = HEAP32[$56>>2]|0;
  $58 = ($57|0)!=(0);
  if ($58) {
   $59 = HEAP32[$j>>2]|0;
   $60 = HEAP32[$aiValues>>2]|0;
   $61 = (($60) + ($59<<2)|0);
   $62 = HEAP32[$61>>2]|0;
   $63 = HEAP32[$1>>2]|0;
   $64 = (($63) + 1)|0;
   $65 = ($62|0)!=($64|0);
   if ($65) {
    $66 = HEAP32[$j>>2]|0;
    $67 = HEAP32[$aiValues>>2]|0;
    $68 = (($67) + ($66<<2)|0);
    $69 = HEAP32[$68>>2]|0;
    $70 = (($69) - 1)|0;
    $71 = $70;
    $72 = (($71>>>0) % 125)&-1;
    HEAP32[$h>>2] = $72;
    $73 = HEAP32[$0>>2]|0;
    $74 = (($73) + 4|0);
    $75 = HEAP32[$74>>2]|0;
    $76 = (($75) + 1)|0;
    HEAP32[$74>>2] = $76;
    while(1) {
     $77 = HEAP32[$h>>2]|0;
     $78 = HEAP32[$0>>2]|0;
     $79 = (($78) + 12|0);
     $80 = (($79) + ($77<<2)|0);
     $81 = HEAP32[$80>>2]|0;
     $82 = ($81|0)!=(0);
     if (!($82)) {
      break;
     }
     $83 = HEAP32[$h>>2]|0;
     $84 = (($83) + 1)|0;
     HEAP32[$h>>2] = $84;
     $85 = HEAP32[$h>>2]|0;
     $86 = ($85>>>0)>=(125);
     $$ = $86 ? 0 : $84;
     HEAP32[$h>>2] = $$;
    }
    $87 = HEAP32[$j>>2]|0;
    $88 = HEAP32[$aiValues>>2]|0;
    $89 = (($88) + ($87<<2)|0);
    $90 = HEAP32[$89>>2]|0;
    $91 = HEAP32[$h>>2]|0;
    $92 = HEAP32[$0>>2]|0;
    $93 = (($92) + 12|0);
    $94 = (($93) + ($91<<2)|0);
    HEAP32[$94>>2] = $90;
   }
  }
  $95 = HEAP32[$j>>2]|0;
  $96 = (($95) + 1)|0;
  HEAP32[$j>>2] = $96;
 }
 STACKTOP = sp;return;
}
function _sqlite3BitvecTest($p,$i) {
 $p = $p|0;
 $i = $i|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $8 = 0, $9 = 0, $bin = 0, $h = 0, $or$cond = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 16|0;
 $1 = sp + 12|0;
 $2 = sp + 8|0;
 $bin = sp + 4|0;
 $h = sp;
 HEAP32[$1>>2] = $p;
 HEAP32[$2>>2] = $i;
 $3 = HEAP32[$1>>2]|0;
 $4 = ($3|0)==(0|0);
 if ($4) {
  HEAP32[$0>>2] = 0;
  $69 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($69|0);
 }
 $5 = HEAP32[$2>>2]|0;
 $6 = HEAP32[$1>>2]|0;
 $7 = HEAP32[$6>>2]|0;
 $8 = ($5>>>0)>($7>>>0);
 $9 = HEAP32[$2>>2]|0;
 $10 = ($9|0)==(0);
 $or$cond = $8 | $10;
 if ($or$cond) {
  HEAP32[$0>>2] = 0;
  $69 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($69|0);
 }
 $11 = HEAP32[$2>>2]|0;
 $12 = (($11) + -1)|0;
 HEAP32[$2>>2] = $12;
 while(1) {
  $13 = HEAP32[$1>>2]|0;
  $14 = (($13) + 8|0);
  $15 = HEAP32[$14>>2]|0;
  $16 = ($15|0)!=(0);
  if (!($16)) {
   break;
  }
  $17 = HEAP32[$2>>2]|0;
  $18 = HEAP32[$1>>2]|0;
  $19 = (($18) + 8|0);
  $20 = HEAP32[$19>>2]|0;
  $21 = (($17>>>0) / ($20>>>0))&-1;
  HEAP32[$bin>>2] = $21;
  $22 = HEAP32[$2>>2]|0;
  $23 = HEAP32[$1>>2]|0;
  $24 = (($23) + 8|0);
  $25 = HEAP32[$24>>2]|0;
  $26 = (($22>>>0) % ($25>>>0))&-1;
  HEAP32[$2>>2] = $26;
  $27 = HEAP32[$bin>>2]|0;
  $28 = HEAP32[$1>>2]|0;
  $29 = (($28) + 12|0);
  $30 = (($29) + ($27<<2)|0);
  $31 = HEAP32[$30>>2]|0;
  HEAP32[$1>>2] = $31;
  $32 = HEAP32[$1>>2]|0;
  $33 = ($32|0)!=(0|0);
  if (!($33)) {
   label = 8;
   break;
  }
 }
 if ((label|0) == 8) {
  HEAP32[$0>>2] = 0;
  $69 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($69|0);
 }
 $34 = HEAP32[$1>>2]|0;
 $35 = HEAP32[$34>>2]|0;
 $36 = ($35>>>0)<=(4000);
 $37 = HEAP32[$2>>2]|0;
 if ($36) {
  $38 = (($37>>>0) / 8)&-1;
  $39 = HEAP32[$1>>2]|0;
  $40 = (($39) + 12|0);
  $41 = (($40) + ($38)|0);
  $42 = HEAP8[$41>>0]|0;
  $43 = $42&255;
  $44 = HEAP32[$2>>2]|0;
  $45 = $44 & 7;
  $46 = 1 << $45;
  $47 = $43 & $46;
  $48 = ($47|0)!=(0);
  $49 = $48&1;
  HEAP32[$0>>2] = $49;
  $69 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($69|0);
 }
 $50 = (($37) + 1)|0;
 HEAP32[$2>>2] = $50;
 $51 = $37;
 $52 = (($51>>>0) % 125)&-1;
 HEAP32[$h>>2] = $52;
 while(1) {
  $53 = HEAP32[$h>>2]|0;
  $54 = HEAP32[$1>>2]|0;
  $55 = (($54) + 12|0);
  $56 = (($55) + ($53<<2)|0);
  $57 = HEAP32[$56>>2]|0;
  $58 = ($57|0)!=(0);
  if (!($58)) {
   label = 16;
   break;
  }
  $59 = HEAP32[$h>>2]|0;
  $60 = HEAP32[$1>>2]|0;
  $61 = (($60) + 12|0);
  $62 = (($61) + ($59<<2)|0);
  $63 = HEAP32[$62>>2]|0;
  $64 = HEAP32[$2>>2]|0;
  $65 = ($63|0)==($64|0);
  if ($65) {
   label = 14;
   break;
  }
  $66 = HEAP32[$h>>2]|0;
  $67 = (($66) + 1)|0;
  $68 = (($67>>>0) % 125)&-1;
  HEAP32[$h>>2] = $68;
 }
 if ((label|0) == 14) {
  HEAP32[$0>>2] = 1;
  $69 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($69|0);
 }
 else if ((label|0) == 16) {
  HEAP32[$0>>2] = 0;
  $69 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($69|0);
 }
 return 0|0;
}
function _sqlite3BitvecSize($p) {
 $p = $p|0;
 var $0 = 0, $1 = 0, $2 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 HEAP32[$0>>2] = $p;
 $1 = HEAP32[$0>>2]|0;
 $2 = HEAP32[$1>>2]|0;
 STACKTOP = sp;return ($2|0);
}
function _sqlite3BitvecDestroy($p) {
 $p = $p|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $i = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $i = sp;
 HEAP32[$0>>2] = $p;
 $1 = HEAP32[$0>>2]|0;
 $2 = ($1|0)==(0|0);
 if ($2) {
  STACKTOP = sp;return;
 }
 $3 = HEAP32[$0>>2]|0;
 $4 = (($3) + 8|0);
 $5 = HEAP32[$4>>2]|0;
 $6 = ($5|0)!=(0);
 L4: do {
  if ($6) {
   HEAP32[$i>>2] = 0;
   while(1) {
    $7 = HEAP32[$i>>2]|0;
    $8 = ($7>>>0)<(125);
    if (!($8)) {
     break L4;
    }
    $9 = HEAP32[$i>>2]|0;
    $10 = HEAP32[$0>>2]|0;
    $11 = (($10) + 12|0);
    $12 = (($11) + ($9<<2)|0);
    $13 = HEAP32[$12>>2]|0;
    _sqlite3BitvecDestroy($13);
    $14 = HEAP32[$i>>2]|0;
    $15 = (($14) + 1)|0;
    HEAP32[$i>>2] = $15;
   }
  }
 } while(0);
 $16 = HEAP32[$0>>2]|0;
 _sqlite3_free($16);
 STACKTOP = sp;return;
}
function _sqlite3FindCollSeq($db,$enc,$zName,$create) {
 $db = $db|0;
 $enc = $enc|0;
 $zName = $zName|0;
 $create = $create|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0;
 var $pColl = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 12|0;
 $1 = sp + 16|0;
 $2 = sp + 8|0;
 $3 = sp + 4|0;
 $pColl = sp;
 HEAP32[$0>>2] = $db;
 HEAP8[$1>>0] = $enc;
 HEAP32[$2>>2] = $zName;
 HEAP32[$3>>2] = $create;
 $4 = HEAP32[$2>>2]|0;
 $5 = ($4|0)!=(0|0);
 $6 = HEAP32[$0>>2]|0;
 if ($5) {
  $7 = HEAP32[$2>>2]|0;
  $8 = HEAP32[$3>>2]|0;
  $9 = (_findCollSeqEntry($6,$7,$8)|0);
  HEAP32[$pColl>>2] = $9;
 } else {
  $10 = (($6) + 8|0);
  $11 = HEAP32[$10>>2]|0;
  HEAP32[$pColl>>2] = $11;
 }
 $12 = HEAP32[$pColl>>2]|0;
 $13 = ($12|0)!=(0|0);
 if (!($13)) {
  $19 = HEAP32[$pColl>>2]|0;
  STACKTOP = sp;return ($19|0);
 }
 $14 = HEAP8[$1>>0]|0;
 $15 = $14&255;
 $16 = (($15) - 1)|0;
 $17 = HEAP32[$pColl>>2]|0;
 $18 = (($17) + (($16*20)|0)|0);
 HEAP32[$pColl>>2] = $18;
 $19 = HEAP32[$pColl>>2]|0;
 STACKTOP = sp;return ($19|0);
}
function _invalidateCachedKeyInfo($db) {
 $db = $db|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $iDb = 0, $k = 0, $pDb = 0;
 var $pIdx = 0, $pTab = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 20|0;
 $pDb = sp + 16|0;
 $iDb = sp + 12|0;
 $k = sp + 8|0;
 $pTab = sp + 4|0;
 $pIdx = sp;
 HEAP32[$0>>2] = $db;
 HEAP32[$iDb>>2] = 0;
 $1 = HEAP32[$0>>2]|0;
 $2 = (($1) + 16|0);
 $3 = HEAP32[$2>>2]|0;
 HEAP32[$pDb>>2] = $3;
 while(1) {
  $4 = HEAP32[$iDb>>2]|0;
  $5 = HEAP32[$0>>2]|0;
  $6 = (($5) + 20|0);
  $7 = HEAP32[$6>>2]|0;
  $8 = ($4|0)<($7|0);
  if (!($8)) {
   break;
  }
  $9 = HEAP32[$pDb>>2]|0;
  $10 = (($9) + 4|0);
  $11 = HEAP32[$10>>2]|0;
  $12 = ($11|0)==(0|0);
  L4: do {
   if (!($12)) {
    $13 = HEAP32[$pDb>>2]|0;
    $14 = (($13) + 4|0);
    $15 = HEAP32[$14>>2]|0;
    _sqlite3BtreeEnter($15);
    $16 = HEAP32[$pDb>>2]|0;
    $17 = (($16) + 12|0);
    $18 = HEAP32[$17>>2]|0;
    $19 = (($18) + 8|0);
    $20 = (($19) + 8|0);
    $21 = HEAP32[$20>>2]|0;
    HEAP32[$k>>2] = $21;
    while(1) {
     $22 = HEAP32[$k>>2]|0;
     $23 = ($22|0)!=(0|0);
     if (!($23)) {
      break L4;
     }
     $24 = HEAP32[$k>>2]|0;
     $25 = (($24) + 8|0);
     $26 = HEAP32[$25>>2]|0;
     HEAP32[$pTab>>2] = $26;
     $27 = HEAP32[$pTab>>2]|0;
     $28 = (($27) + 8|0);
     $29 = HEAP32[$28>>2]|0;
     HEAP32[$pIdx>>2] = $29;
     while(1) {
      $30 = HEAP32[$pIdx>>2]|0;
      $31 = ($30|0)!=(0|0);
      if (!($31)) {
       break;
      }
      $32 = HEAP32[$pIdx>>2]|0;
      $33 = (($32) + 40|0);
      $34 = HEAP32[$33>>2]|0;
      $35 = ($34|0)!=(0|0);
      if ($35) {
       $36 = HEAP32[$pIdx>>2]|0;
       $37 = (($36) + 40|0);
       $38 = HEAP32[$37>>2]|0;
       $39 = (($38) + 12|0);
       $40 = HEAP32[$39>>2]|0;
       $41 = HEAP32[$0>>2]|0;
       $42 = ($40|0)==($41|0);
       if ($42) {
        $43 = HEAP32[$pIdx>>2]|0;
        $44 = (($43) + 40|0);
        $45 = HEAP32[$44>>2]|0;
        _sqlite3KeyInfoUnref($45);
        $46 = HEAP32[$pIdx>>2]|0;
        $47 = (($46) + 40|0);
        HEAP32[$47>>2] = 0;
       }
      }
      $48 = HEAP32[$pIdx>>2]|0;
      $49 = (($48) + 20|0);
      $50 = HEAP32[$49>>2]|0;
      HEAP32[$pIdx>>2] = $50;
     }
     $51 = HEAP32[$k>>2]|0;
     $52 = HEAP32[$51>>2]|0;
     HEAP32[$k>>2] = $52;
    }
   }
  } while(0);
  $53 = HEAP32[$iDb>>2]|0;
  $54 = (($53) + 1)|0;
  HEAP32[$iDb>>2] = $54;
  $55 = HEAP32[$pDb>>2]|0;
  $56 = (($55) + 16|0);
  HEAP32[$pDb>>2] = $56;
 }
 STACKTOP = sp;return;
}
function _sqlite3HashFind($pH,$pKey) {
 $pH = $pH|0;
 $pKey = $pKey|0;
 var $0 = 0, $1 = 0, $10 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $elem = 0, $h = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $1 = sp + 8|0;
 $elem = sp + 4|0;
 $h = sp;
 HEAP32[$0>>2] = $pH;
 HEAP32[$1>>2] = $pKey;
 $2 = HEAP32[$0>>2]|0;
 $3 = HEAP32[$1>>2]|0;
 $4 = (_findElementWithHash($2,$3,$h)|0);
 HEAP32[$elem>>2] = $4;
 $5 = HEAP32[$elem>>2]|0;
 $6 = ($5|0)!=(0|0);
 if (!($6)) {
  $10 = 0;
  STACKTOP = sp;return ($10|0);
 }
 $7 = HEAP32[$elem>>2]|0;
 $8 = (($7) + 8|0);
 $9 = HEAP32[$8>>2]|0;
 $10 = $9;
 STACKTOP = sp;return ($10|0);
}
function _findElementWithHash($pH,$pKey,$pHash) {
 $pH = $pH|0;
 $pKey = $pKey|0;
 $pHash = $pHash|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $5 = 0, $6 = 0;
 var $7 = 0, $8 = 0, $9 = 0, $count = 0, $elem = 0, $h = 0, $pEntry = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 28|0;
 $1 = sp + 24|0;
 $2 = sp + 20|0;
 $3 = sp + 16|0;
 $elem = sp + 12|0;
 $count = sp + 8|0;
 $h = sp + 4|0;
 $pEntry = sp;
 HEAP32[$1>>2] = $pH;
 HEAP32[$2>>2] = $pKey;
 HEAP32[$3>>2] = $pHash;
 $4 = HEAP32[$1>>2]|0;
 $5 = (($4) + 12|0);
 $6 = HEAP32[$5>>2]|0;
 $7 = ($6|0)!=(0|0);
 if ($7) {
  $8 = HEAP32[$2>>2]|0;
  $9 = (_strHash($8)|0);
  $10 = HEAP32[$1>>2]|0;
  $11 = HEAP32[$10>>2]|0;
  $12 = (($9>>>0) % ($11>>>0))&-1;
  HEAP32[$h>>2] = $12;
  $13 = HEAP32[$h>>2]|0;
  $14 = HEAP32[$1>>2]|0;
  $15 = (($14) + 12|0);
  $16 = HEAP32[$15>>2]|0;
  $17 = (($16) + ($13<<3)|0);
  HEAP32[$pEntry>>2] = $17;
  $18 = HEAP32[$pEntry>>2]|0;
  $19 = (($18) + 4|0);
  $20 = HEAP32[$19>>2]|0;
  HEAP32[$elem>>2] = $20;
  $21 = HEAP32[$pEntry>>2]|0;
  $22 = HEAP32[$21>>2]|0;
  HEAP32[$count>>2] = $22;
 } else {
  HEAP32[$h>>2] = 0;
  $23 = HEAP32[$1>>2]|0;
  $24 = (($23) + 8|0);
  $25 = HEAP32[$24>>2]|0;
  HEAP32[$elem>>2] = $25;
  $26 = HEAP32[$1>>2]|0;
  $27 = (($26) + 4|0);
  $28 = HEAP32[$27>>2]|0;
  HEAP32[$count>>2] = $28;
 }
 $29 = HEAP32[$h>>2]|0;
 $30 = HEAP32[$3>>2]|0;
 HEAP32[$30>>2] = $29;
 while(1) {
  $31 = HEAP32[$count>>2]|0;
  $32 = (($31) + -1)|0;
  HEAP32[$count>>2] = $32;
  $33 = ($31|0)!=(0);
  if (!($33)) {
   label = 9;
   break;
  }
  $34 = HEAP32[$elem>>2]|0;
  $35 = (($34) + 12|0);
  $36 = HEAP32[$35>>2]|0;
  $37 = HEAP32[$2>>2]|0;
  $38 = (_sqlite3_stricmp($36,$37)|0);
  $39 = ($38|0)==(0);
  $40 = HEAP32[$elem>>2]|0;
  if ($39) {
   label = 7;
   break;
  }
  $41 = HEAP32[$40>>2]|0;
  HEAP32[$elem>>2] = $41;
 }
 if ((label|0) == 7) {
  HEAP32[$0>>2] = $40;
  $42 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($42|0);
 }
 else if ((label|0) == 9) {
  HEAP32[$0>>2] = 0;
  $42 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($42|0);
 }
 return 0|0;
}
function _strHash($z) {
 $z = $z|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $c = 0, $h = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $h = sp;
 $c = sp + 8|0;
 HEAP32[$0>>2] = $z;
 HEAP32[$h>>2] = 0;
 while(1) {
  $1 = HEAP32[$0>>2]|0;
  $2 = (($1) + 1|0);
  HEAP32[$0>>2] = $2;
  $3 = HEAP8[$1>>0]|0;
  HEAP8[$c>>0] = $3;
  $4 = $3&255;
  $5 = ($4|0)!=(0);
  $6 = HEAP32[$h>>2]|0;
  if (!($5)) {
   break;
  }
  $7 = $6 << 3;
  $8 = HEAP32[$h>>2]|0;
  $9 = $7 ^ $8;
  $10 = HEAP8[$c>>0]|0;
  $11 = $10&255;
  $12 = (440 + ($11)|0);
  $13 = HEAP8[$12>>0]|0;
  $14 = $13&255;
  $15 = $9 ^ $14;
  HEAP32[$h>>2] = $15;
 }
 STACKTOP = sp;return ($6|0);
}
function _sqlite3KeyInfoUnref($p) {
 $p = $p|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 HEAP32[$0>>2] = $p;
 $1 = HEAP32[$0>>2]|0;
 $2 = ($1|0)!=(0|0);
 if (!($2)) {
  STACKTOP = sp;return;
 }
 $3 = HEAP32[$0>>2]|0;
 $4 = HEAP32[$3>>2]|0;
 $5 = (($4) + -1)|0;
 HEAP32[$3>>2] = $5;
 $6 = HEAP32[$0>>2]|0;
 $7 = HEAP32[$6>>2]|0;
 $8 = ($7|0)==(0);
 if (!($8)) {
  STACKTOP = sp;return;
 }
 $9 = HEAP32[$0>>2]|0;
 _sqlite3DbFree(0,$9);
 STACKTOP = sp;return;
}
function _findCollSeqEntry($db,$zName,$create) {
 $db = $db|0;
 $zName = $zName|0;
 $create = $create|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0;
 var $nName = 0, $or$cond = 0, $pColl = 0, $pDel = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 20|0;
 $1 = sp + 16|0;
 $2 = sp + 12|0;
 $pColl = sp + 8|0;
 $nName = sp + 4|0;
 $pDel = sp;
 HEAP32[$0>>2] = $db;
 HEAP32[$1>>2] = $zName;
 HEAP32[$2>>2] = $create;
 $3 = HEAP32[$0>>2]|0;
 $4 = (($3) + 428|0);
 $5 = HEAP32[$1>>2]|0;
 $6 = (_sqlite3HashFind($4,$5)|0);
 HEAP32[$pColl>>2] = $6;
 $7 = HEAP32[$pColl>>2]|0;
 $8 = (0|0)==($7|0);
 $9 = HEAP32[$2>>2]|0;
 $10 = ($9|0)!=(0);
 $or$cond = $8 & $10;
 if (!($or$cond)) {
  $59 = HEAP32[$pColl>>2]|0;
  STACKTOP = sp;return ($59|0);
 }
 $11 = HEAP32[$1>>2]|0;
 $12 = (_sqlite3Strlen30($11)|0);
 HEAP32[$nName>>2] = $12;
 $13 = HEAP32[$0>>2]|0;
 $14 = HEAP32[$nName>>2]|0;
 $15 = (60 + ($14))|0;
 $16 = (($15) + 1)|0;
 $17 = (_sqlite3DbMallocZero($13,$16,0)|0);
 HEAP32[$pColl>>2] = $17;
 $18 = HEAP32[$pColl>>2]|0;
 $19 = ($18|0)!=(0|0);
 if (!($19)) {
  $59 = HEAP32[$pColl>>2]|0;
  STACKTOP = sp;return ($59|0);
 }
 HEAP32[$pDel>>2] = 0;
 $20 = HEAP32[$pColl>>2]|0;
 $21 = (($20) + 60|0);
 $22 = HEAP32[$pColl>>2]|0;
 HEAP32[$22>>2] = $21;
 $23 = HEAP32[$pColl>>2]|0;
 $24 = (($23) + 4|0);
 HEAP8[$24>>0] = 1;
 $25 = HEAP32[$pColl>>2]|0;
 $26 = (($25) + 60|0);
 $27 = HEAP32[$pColl>>2]|0;
 $28 = (($27) + 20|0);
 HEAP32[$28>>2] = $26;
 $29 = HEAP32[$pColl>>2]|0;
 $30 = (($29) + 20|0);
 $31 = (($30) + 4|0);
 HEAP8[$31>>0] = 2;
 $32 = HEAP32[$pColl>>2]|0;
 $33 = (($32) + 60|0);
 $34 = HEAP32[$pColl>>2]|0;
 $35 = (($34) + 40|0);
 HEAP32[$35>>2] = $33;
 $36 = HEAP32[$pColl>>2]|0;
 $37 = (($36) + 40|0);
 $38 = (($37) + 4|0);
 HEAP8[$38>>0] = 3;
 $39 = HEAP32[$pColl>>2]|0;
 $40 = HEAP32[$39>>2]|0;
 $41 = HEAP32[$1>>2]|0;
 $42 = HEAP32[$nName>>2]|0;
 _memcpy(($40|0),($41|0),($42|0))|0;
 $43 = HEAP32[$nName>>2]|0;
 $44 = HEAP32[$pColl>>2]|0;
 $45 = HEAP32[$44>>2]|0;
 $46 = (($45) + ($43)|0);
 HEAP8[$46>>0] = 0;
 $47 = HEAP32[$0>>2]|0;
 $48 = (($47) + 428|0);
 $49 = HEAP32[$pColl>>2]|0;
 $50 = HEAP32[$49>>2]|0;
 $51 = HEAP32[$pColl>>2]|0;
 $52 = (_sqlite3HashInsert($48,$50,$51)|0);
 HEAP32[$pDel>>2] = $52;
 $53 = HEAP32[$pDel>>2]|0;
 $54 = ($53|0)!=(0|0);
 if (!($54)) {
  $59 = HEAP32[$pColl>>2]|0;
  STACKTOP = sp;return ($59|0);
 }
 $55 = HEAP32[$0>>2]|0;
 $56 = (($55) + 64|0);
 HEAP8[$56>>0] = 1;
 $57 = HEAP32[$0>>2]|0;
 $58 = HEAP32[$pDel>>2]|0;
 _sqlite3DbFree($57,$58);
 HEAP32[$pColl>>2] = 0;
 $59 = HEAP32[$pColl>>2]|0;
 STACKTOP = sp;return ($59|0);
}
function _sqlite3HashInsert($pH,$pKey,$data) {
 $pH = $pH|0;
 $pKey = $pKey|0;
 $data = $data|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $8 = 0, $9 = 0, $elem = 0, $h = 0, $new_elem = 0, $old_data = 0;
 var label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 28|0;
 $1 = sp + 24|0;
 $2 = sp + 20|0;
 $3 = sp + 16|0;
 $h = sp + 12|0;
 $elem = sp + 8|0;
 $new_elem = sp + 4|0;
 $old_data = sp;
 HEAP32[$1>>2] = $pH;
 HEAP32[$2>>2] = $pKey;
 HEAP32[$3>>2] = $data;
 $4 = HEAP32[$1>>2]|0;
 $5 = HEAP32[$2>>2]|0;
 $6 = (_findElementWithHash($4,$5,$h)|0);
 HEAP32[$elem>>2] = $6;
 $7 = HEAP32[$elem>>2]|0;
 $8 = ($7|0)!=(0|0);
 if ($8) {
  $9 = HEAP32[$elem>>2]|0;
  $10 = (($9) + 8|0);
  $11 = HEAP32[$10>>2]|0;
  HEAP32[$old_data>>2] = $11;
  $12 = HEAP32[$3>>2]|0;
  $13 = ($12|0)==(0|0);
  if ($13) {
   $14 = HEAP32[$1>>2]|0;
   $15 = HEAP32[$elem>>2]|0;
   $16 = HEAP32[$h>>2]|0;
   _removeElementGivenHash($14,$15,$16);
  } else {
   $17 = HEAP32[$3>>2]|0;
   $18 = HEAP32[$elem>>2]|0;
   $19 = (($18) + 8|0);
   HEAP32[$19>>2] = $17;
   $20 = HEAP32[$2>>2]|0;
   $21 = HEAP32[$elem>>2]|0;
   $22 = (($21) + 12|0);
   HEAP32[$22>>2] = $20;
  }
  $23 = HEAP32[$old_data>>2]|0;
  HEAP32[$0>>2] = $23;
  $75 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($75|0);
 }
 $24 = HEAP32[$3>>2]|0;
 $25 = ($24|0)==(0|0);
 if ($25) {
  HEAP32[$0>>2] = 0;
  $75 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($75|0);
 }
 $26 = (_sqlite3Malloc(16,0)|0);
 HEAP32[$new_elem>>2] = $26;
 $27 = HEAP32[$new_elem>>2]|0;
 $28 = ($27|0)==(0|0);
 if ($28) {
  $29 = HEAP32[$3>>2]|0;
  HEAP32[$0>>2] = $29;
  $75 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($75|0);
 }
 $30 = HEAP32[$2>>2]|0;
 $31 = HEAP32[$new_elem>>2]|0;
 $32 = (($31) + 12|0);
 HEAP32[$32>>2] = $30;
 $33 = HEAP32[$3>>2]|0;
 $34 = HEAP32[$new_elem>>2]|0;
 $35 = (($34) + 8|0);
 HEAP32[$35>>2] = $33;
 $36 = HEAP32[$1>>2]|0;
 $37 = (($36) + 4|0);
 $38 = HEAP32[$37>>2]|0;
 $39 = (($38) + 1)|0;
 HEAP32[$37>>2] = $39;
 $40 = HEAP32[$1>>2]|0;
 $41 = (($40) + 4|0);
 $42 = HEAP32[$41>>2]|0;
 $43 = ($42>>>0)>=(10);
 if ($43) {
  $44 = HEAP32[$1>>2]|0;
  $45 = (($44) + 4|0);
  $46 = HEAP32[$45>>2]|0;
  $47 = HEAP32[$1>>2]|0;
  $48 = HEAP32[$47>>2]|0;
  $49 = $48<<1;
  $50 = ($46>>>0)>($49>>>0);
  if ($50) {
   $51 = HEAP32[$1>>2]|0;
   $52 = HEAP32[$1>>2]|0;
   $53 = (($52) + 4|0);
   $54 = HEAP32[$53>>2]|0;
   $55 = $54<<1;
   $56 = (_rehash($51,$55)|0);
   $57 = ($56|0)!=(0);
   if ($57) {
    $58 = HEAP32[$2>>2]|0;
    $59 = (_strHash($58)|0);
    $60 = HEAP32[$1>>2]|0;
    $61 = HEAP32[$60>>2]|0;
    $62 = (($59>>>0) % ($61>>>0))&-1;
    HEAP32[$h>>2] = $62;
   }
  }
 }
 $63 = HEAP32[$1>>2]|0;
 $64 = HEAP32[$1>>2]|0;
 $65 = (($64) + 12|0);
 $66 = HEAP32[$65>>2]|0;
 $67 = ($66|0)!=(0|0);
 if ($67) {
  $68 = HEAP32[$h>>2]|0;
  $69 = HEAP32[$1>>2]|0;
  $70 = (($69) + 12|0);
  $71 = HEAP32[$70>>2]|0;
  $72 = (($71) + ($68<<3)|0);
  $74 = $72;
 } else {
  $74 = 0;
 }
 $73 = HEAP32[$new_elem>>2]|0;
 _insertElement($63,$74,$73);
 HEAP32[$0>>2] = 0;
 $75 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($75|0);
}
function _removeElementGivenHash($pH,$elem,$h) {
 $pH = $pH|0;
 $elem = $elem|0;
 $h = $h|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $pEntry = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $1 = sp + 8|0;
 $2 = sp + 4|0;
 $pEntry = sp;
 HEAP32[$0>>2] = $pH;
 HEAP32[$1>>2] = $elem;
 HEAP32[$2>>2] = $h;
 $3 = HEAP32[$1>>2]|0;
 $4 = (($3) + 4|0);
 $5 = HEAP32[$4>>2]|0;
 $6 = ($5|0)!=(0|0);
 $7 = HEAP32[$1>>2]|0;
 $8 = HEAP32[$7>>2]|0;
 if ($6) {
  $9 = HEAP32[$1>>2]|0;
  $10 = (($9) + 4|0);
  $11 = HEAP32[$10>>2]|0;
  HEAP32[$11>>2] = $8;
 } else {
  $12 = HEAP32[$0>>2]|0;
  $13 = (($12) + 8|0);
  HEAP32[$13>>2] = $8;
 }
 $14 = HEAP32[$1>>2]|0;
 $15 = HEAP32[$14>>2]|0;
 $16 = ($15|0)!=(0|0);
 if ($16) {
  $17 = HEAP32[$1>>2]|0;
  $18 = (($17) + 4|0);
  $19 = HEAP32[$18>>2]|0;
  $20 = HEAP32[$1>>2]|0;
  $21 = HEAP32[$20>>2]|0;
  $22 = (($21) + 4|0);
  HEAP32[$22>>2] = $19;
 }
 $23 = HEAP32[$0>>2]|0;
 $24 = (($23) + 12|0);
 $25 = HEAP32[$24>>2]|0;
 $26 = ($25|0)!=(0|0);
 if ($26) {
  $27 = HEAP32[$2>>2]|0;
  $28 = HEAP32[$0>>2]|0;
  $29 = (($28) + 12|0);
  $30 = HEAP32[$29>>2]|0;
  $31 = (($30) + ($27<<3)|0);
  HEAP32[$pEntry>>2] = $31;
  $32 = HEAP32[$pEntry>>2]|0;
  $33 = (($32) + 4|0);
  $34 = HEAP32[$33>>2]|0;
  $35 = HEAP32[$1>>2]|0;
  $36 = ($34|0)==($35|0);
  if ($36) {
   $37 = HEAP32[$1>>2]|0;
   $38 = HEAP32[$37>>2]|0;
   $39 = HEAP32[$pEntry>>2]|0;
   $40 = (($39) + 4|0);
   HEAP32[$40>>2] = $38;
  }
  $41 = HEAP32[$pEntry>>2]|0;
  $42 = HEAP32[$41>>2]|0;
  $43 = (($42) + -1)|0;
  HEAP32[$41>>2] = $43;
 }
 $44 = HEAP32[$1>>2]|0;
 _sqlite3_free($44);
 $45 = HEAP32[$0>>2]|0;
 $46 = (($45) + 4|0);
 $47 = HEAP32[$46>>2]|0;
 $48 = (($47) + -1)|0;
 HEAP32[$46>>2] = $48;
 $49 = HEAP32[$0>>2]|0;
 $50 = (($49) + 4|0);
 $51 = HEAP32[$50>>2]|0;
 $52 = ($51|0)==(0);
 if (!($52)) {
  STACKTOP = sp;return;
 }
 $53 = HEAP32[$0>>2]|0;
 _sqlite3HashClear($53);
 STACKTOP = sp;return;
}
function _rehash($pH,$new_size) {
 $pH = $pH|0;
 $new_size = $new_size|0;
 var $$new_size = 0, $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0;
 var $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0;
 var $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $elem = 0, $h = 0, $new_ht = 0, $next_elem = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 24|0;
 $1 = sp + 20|0;
 $2 = sp + 16|0;
 $new_ht = sp + 12|0;
 $elem = sp + 8|0;
 $next_elem = sp + 4|0;
 $h = sp;
 HEAP32[$1>>2] = $pH;
 HEAP32[$2>>2] = $new_size;
 $3 = HEAP32[$2>>2]|0;
 $4 = $3<<3;
 $5 = ($4>>>0)>(1024);
 $$new_size = $5 ? 128 : $new_size;
 HEAP32[$2>>2] = $$new_size;
 $6 = HEAP32[$2>>2]|0;
 $7 = HEAP32[$1>>2]|0;
 $8 = HEAP32[$7>>2]|0;
 $9 = ($6|0)==($8|0);
 if ($9) {
  HEAP32[$0>>2] = 0;
  $49 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($49|0);
 }
 _sqlite3BeginBenignMalloc();
 $10 = HEAP32[$2>>2]|0;
 $11 = $10<<3;
 $12 = (_sqlite3Malloc($11,0)|0);
 HEAP32[$new_ht>>2] = $12;
 _sqlite3EndBenignMalloc();
 $13 = HEAP32[$new_ht>>2]|0;
 $14 = ($13|0)==(0|0);
 if ($14) {
  HEAP32[$0>>2] = 0;
  $49 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($49|0);
 }
 $15 = HEAP32[$1>>2]|0;
 $16 = (($15) + 12|0);
 $17 = HEAP32[$16>>2]|0;
 _sqlite3_free($17);
 $18 = HEAP32[$new_ht>>2]|0;
 $19 = HEAP32[$1>>2]|0;
 $20 = (($19) + 12|0);
 HEAP32[$20>>2] = $18;
 $21 = HEAP32[$new_ht>>2]|0;
 $22 = (_sqlite3MallocSize($21)|0);
 $23 = (($22>>>0) / 8)&-1;
 HEAP32[$2>>2] = $23;
 $24 = HEAP32[$1>>2]|0;
 HEAP32[$24>>2] = $23;
 $25 = HEAP32[$new_ht>>2]|0;
 $26 = HEAP32[$2>>2]|0;
 $27 = $26<<3;
 _memset(($25|0),0,($27|0))|0;
 $28 = HEAP32[$1>>2]|0;
 $29 = (($28) + 8|0);
 $30 = HEAP32[$29>>2]|0;
 HEAP32[$elem>>2] = $30;
 $31 = HEAP32[$1>>2]|0;
 $32 = (($31) + 8|0);
 HEAP32[$32>>2] = 0;
 while(1) {
  $33 = HEAP32[$elem>>2]|0;
  $34 = ($33|0)!=(0|0);
  if (!($34)) {
   break;
  }
  $35 = HEAP32[$elem>>2]|0;
  $36 = (($35) + 12|0);
  $37 = HEAP32[$36>>2]|0;
  $38 = (_strHash($37)|0);
  $39 = HEAP32[$2>>2]|0;
  $40 = (($38>>>0) % ($39>>>0))&-1;
  HEAP32[$h>>2] = $40;
  $41 = HEAP32[$elem>>2]|0;
  $42 = HEAP32[$41>>2]|0;
  HEAP32[$next_elem>>2] = $42;
  $43 = HEAP32[$1>>2]|0;
  $44 = HEAP32[$h>>2]|0;
  $45 = HEAP32[$new_ht>>2]|0;
  $46 = (($45) + ($44<<3)|0);
  $47 = HEAP32[$elem>>2]|0;
  _insertElement($43,$46,$47);
  $48 = HEAP32[$next_elem>>2]|0;
  HEAP32[$elem>>2] = $48;
 }
 HEAP32[$0>>2] = 1;
 $49 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($49|0);
}
function _insertElement($pH,$pEntry,$pNew) {
 $pH = $pH|0;
 $pEntry = $pEntry|0;
 $pNew = $pNew|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $pHead = 0, label = 0;
 var sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $1 = sp + 8|0;
 $2 = sp + 4|0;
 $pHead = sp;
 HEAP32[$0>>2] = $pH;
 HEAP32[$1>>2] = $pEntry;
 HEAP32[$2>>2] = $pNew;
 $3 = HEAP32[$1>>2]|0;
 $4 = ($3|0)!=(0|0);
 if ($4) {
  $5 = HEAP32[$1>>2]|0;
  $6 = HEAP32[$5>>2]|0;
  $7 = ($6|0)!=(0);
  if ($7) {
   $8 = HEAP32[$1>>2]|0;
   $9 = (($8) + 4|0);
   $10 = HEAP32[$9>>2]|0;
   $11 = $10;
  } else {
   $11 = 0;
  }
  HEAP32[$pHead>>2] = $11;
  $12 = HEAP32[$1>>2]|0;
  $13 = HEAP32[$12>>2]|0;
  $14 = (($13) + 1)|0;
  HEAP32[$12>>2] = $14;
  $15 = HEAP32[$2>>2]|0;
  $16 = HEAP32[$1>>2]|0;
  $17 = (($16) + 4|0);
  HEAP32[$17>>2] = $15;
 } else {
  HEAP32[$pHead>>2] = 0;
 }
 $18 = HEAP32[$pHead>>2]|0;
 $19 = ($18|0)!=(0|0);
 if (!($19)) {
  $40 = HEAP32[$0>>2]|0;
  $41 = (($40) + 8|0);
  $42 = HEAP32[$41>>2]|0;
  $43 = HEAP32[$2>>2]|0;
  HEAP32[$43>>2] = $42;
  $44 = HEAP32[$0>>2]|0;
  $45 = (($44) + 8|0);
  $46 = HEAP32[$45>>2]|0;
  $47 = ($46|0)!=(0|0);
  if ($47) {
   $48 = HEAP32[$2>>2]|0;
   $49 = HEAP32[$0>>2]|0;
   $50 = (($49) + 8|0);
   $51 = HEAP32[$50>>2]|0;
   $52 = (($51) + 4|0);
   HEAP32[$52>>2] = $48;
  }
  $53 = HEAP32[$2>>2]|0;
  $54 = (($53) + 4|0);
  HEAP32[$54>>2] = 0;
  $55 = HEAP32[$2>>2]|0;
  $56 = HEAP32[$0>>2]|0;
  $57 = (($56) + 8|0);
  HEAP32[$57>>2] = $55;
  STACKTOP = sp;return;
 }
 $20 = HEAP32[$pHead>>2]|0;
 $21 = HEAP32[$2>>2]|0;
 HEAP32[$21>>2] = $20;
 $22 = HEAP32[$pHead>>2]|0;
 $23 = (($22) + 4|0);
 $24 = HEAP32[$23>>2]|0;
 $25 = HEAP32[$2>>2]|0;
 $26 = (($25) + 4|0);
 HEAP32[$26>>2] = $24;
 $27 = HEAP32[$pHead>>2]|0;
 $28 = (($27) + 4|0);
 $29 = HEAP32[$28>>2]|0;
 $30 = ($29|0)!=(0|0);
 $31 = HEAP32[$2>>2]|0;
 if ($30) {
  $32 = HEAP32[$pHead>>2]|0;
  $33 = (($32) + 4|0);
  $34 = HEAP32[$33>>2]|0;
  HEAP32[$34>>2] = $31;
 } else {
  $35 = HEAP32[$0>>2]|0;
  $36 = (($35) + 8|0);
  HEAP32[$36>>2] = $31;
 }
 $37 = HEAP32[$2>>2]|0;
 $38 = HEAP32[$pHead>>2]|0;
 $39 = (($38) + 4|0);
 HEAP32[$39>>2] = $37;
 STACKTOP = sp;return;
}
function _sqlite3BeginBenignMalloc() {
 var $0 = 0, $1 = 0, $2 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 $0 = HEAP32[3208>>2]|0;
 $1 = ($0|0)!=(0|0);
 if (!($1)) {
  STACKTOP = sp;return;
 }
 $2 = HEAP32[3208>>2]|0;
 FUNCTION_TABLE_v[$2 & 0]();
 STACKTOP = sp;return;
}
function _sqlite3EndBenignMalloc() {
 var $0 = 0, $1 = 0, $2 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 $0 = HEAP32[((3208 + 4|0))>>2]|0;
 $1 = ($0|0)!=(0|0);
 if (!($1)) {
  STACKTOP = sp;return;
 }
 $2 = HEAP32[((3208 + 4|0))>>2]|0;
 FUNCTION_TABLE_v[$2 & 0]();
 STACKTOP = sp;return;
}
function _sqlite3HashClear($pH) {
 $pH = $pH|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0;
 var $elem = 0, $next_elem = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $elem = sp + 4|0;
 $next_elem = sp;
 HEAP32[$0>>2] = $pH;
 $1 = HEAP32[$0>>2]|0;
 $2 = (($1) + 8|0);
 $3 = HEAP32[$2>>2]|0;
 HEAP32[$elem>>2] = $3;
 $4 = HEAP32[$0>>2]|0;
 $5 = (($4) + 8|0);
 HEAP32[$5>>2] = 0;
 $6 = HEAP32[$0>>2]|0;
 $7 = (($6) + 12|0);
 $8 = HEAP32[$7>>2]|0;
 _sqlite3_free($8);
 $9 = HEAP32[$0>>2]|0;
 $10 = (($9) + 12|0);
 HEAP32[$10>>2] = 0;
 $11 = HEAP32[$0>>2]|0;
 HEAP32[$11>>2] = 0;
 while(1) {
  $12 = HEAP32[$elem>>2]|0;
  $13 = ($12|0)!=(0|0);
  if (!($13)) {
   break;
  }
  $14 = HEAP32[$elem>>2]|0;
  $15 = HEAP32[$14>>2]|0;
  HEAP32[$next_elem>>2] = $15;
  $16 = HEAP32[$elem>>2]|0;
  _sqlite3_free($16);
  $17 = HEAP32[$next_elem>>2]|0;
  HEAP32[$elem>>2] = $17;
 }
 $18 = HEAP32[$0>>2]|0;
 $19 = (($18) + 4|0);
 HEAP32[$19>>2] = 0;
 STACKTOP = sp;return;
}
function _sqlite3HashInit($pNew) {
 $pNew = $pNew|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 HEAP32[$0>>2] = $pNew;
 $1 = HEAP32[$0>>2]|0;
 $2 = (($1) + 8|0);
 HEAP32[$2>>2] = 0;
 $3 = HEAP32[$0>>2]|0;
 $4 = (($3) + 4|0);
 HEAP32[$4>>2] = 0;
 $5 = HEAP32[$0>>2]|0;
 HEAP32[$5>>2] = 0;
 $6 = HEAP32[$0>>2]|0;
 $7 = (($6) + 12|0);
 HEAP32[$7>>2] = 0;
 STACKTOP = sp;return;
}
function _binCollFunc($padFlag,$nKey1,$pKey1,$nKey2,$pKey2) {
 $padFlag = $padFlag|0;
 $nKey1 = $nKey1|0;
 $pKey1 = $pKey1|0;
 $nKey2 = $nKey2|0;
 $pKey2 = $pKey2|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $n = 0;
 var $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 24|0;
 $1 = sp + 20|0;
 $2 = sp + 16|0;
 $3 = sp + 12|0;
 $4 = sp + 8|0;
 $rc = sp + 4|0;
 $n = sp;
 HEAP32[$0>>2] = $padFlag;
 HEAP32[$1>>2] = $nKey1;
 HEAP32[$2>>2] = $pKey1;
 HEAP32[$3>>2] = $nKey2;
 HEAP32[$4>>2] = $pKey2;
 $5 = HEAP32[$1>>2]|0;
 $6 = HEAP32[$3>>2]|0;
 $7 = ($5|0)<($6|0);
 $8 = HEAP32[$1>>2]|0;
 $9 = HEAP32[$3>>2]|0;
 $10 = $7 ? $8 : $9;
 HEAP32[$n>>2] = $10;
 $11 = HEAP32[$2>>2]|0;
 $12 = HEAP32[$4>>2]|0;
 $13 = HEAP32[$n>>2]|0;
 $14 = (_memcmp($11,$12,$13)|0);
 HEAP32[$rc>>2] = $14;
 $15 = HEAP32[$rc>>2]|0;
 $16 = ($15|0)==(0);
 if (!($16)) {
  $38 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($38|0);
 }
 $17 = HEAP32[$0>>2]|0;
 $18 = ($17|0)!=(0|0);
 if ($18) {
  $19 = HEAP32[$2>>2]|0;
  $20 = HEAP32[$n>>2]|0;
  $21 = (($19) + ($20)|0);
  $22 = HEAP32[$1>>2]|0;
  $23 = HEAP32[$n>>2]|0;
  $24 = (($22) - ($23))|0;
  $25 = (_allSpaces($21,$24)|0);
  $26 = ($25|0)!=(0);
  if ($26) {
   $27 = HEAP32[$4>>2]|0;
   $28 = HEAP32[$n>>2]|0;
   $29 = (($27) + ($28)|0);
   $30 = HEAP32[$3>>2]|0;
   $31 = HEAP32[$n>>2]|0;
   $32 = (($30) - ($31))|0;
   $33 = (_allSpaces($29,$32)|0);
   $34 = ($33|0)!=(0);
   if ($34) {
    $38 = HEAP32[$rc>>2]|0;
    STACKTOP = sp;return ($38|0);
   }
  }
 }
 $35 = HEAP32[$1>>2]|0;
 $36 = HEAP32[$3>>2]|0;
 $37 = (($35) - ($36))|0;
 HEAP32[$rc>>2] = $37;
 $38 = HEAP32[$rc>>2]|0;
 STACKTOP = sp;return ($38|0);
}
function _nocaseCollatingFunc($NotUsed,$nKey1,$pKey1,$nKey2,$pKey2) {
 $NotUsed = $NotUsed|0;
 $nKey1 = $nKey1|0;
 $pKey1 = $pKey1|0;
 $nKey2 = $nKey2|0;
 $pKey2 = $pKey2|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0;
 var $r = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 20|0;
 $1 = sp + 16|0;
 $2 = sp + 12|0;
 $3 = sp + 8|0;
 $4 = sp + 4|0;
 $r = sp;
 HEAP32[$0>>2] = $NotUsed;
 HEAP32[$1>>2] = $nKey1;
 HEAP32[$2>>2] = $pKey1;
 HEAP32[$3>>2] = $nKey2;
 HEAP32[$4>>2] = $pKey2;
 $5 = HEAP32[$2>>2]|0;
 $6 = HEAP32[$4>>2]|0;
 $7 = HEAP32[$1>>2]|0;
 $8 = HEAP32[$3>>2]|0;
 $9 = ($7|0)<($8|0);
 $10 = HEAP32[$1>>2]|0;
 $11 = HEAP32[$3>>2]|0;
 $12 = $9 ? $10 : $11;
 $13 = (_sqlite3_strnicmp($5,$6,$12)|0);
 HEAP32[$r>>2] = $13;
 $14 = HEAP32[$r>>2]|0;
 $15 = (0)==($14|0);
 if (!($15)) {
  $19 = HEAP32[$r>>2]|0;
  STACKTOP = sp;return ($19|0);
 }
 $16 = HEAP32[$1>>2]|0;
 $17 = HEAP32[$3>>2]|0;
 $18 = (($16) - ($17))|0;
 HEAP32[$r>>2] = $18;
 $19 = HEAP32[$r>>2]|0;
 STACKTOP = sp;return ($19|0);
}
function _sqlite3ParseUri($zDefaultVfs,$zUri,$pFlags,$ppVfs,$pzFile,$pzErrMsg) {
 $zDefaultVfs = $zDefaultVfs|0;
 $zUri = $zUri|0;
 $pFlags = $pFlags|0;
 $ppVfs = $ppVfs|0;
 $pzFile = $pzFile|0;
 $pzErrMsg = $pzErrMsg|0;
 var $$old = 0, $$old1 = 0, $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0;
 var $114 = 0, $115 = 0, $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0;
 var $132 = 0, $133 = 0, $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0;
 var $150 = 0, $151 = 0, $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0;
 var $169 = 0, $17 = 0, $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0;
 var $187 = 0, $188 = 0, $189 = 0, $19 = 0, $190 = 0, $191 = 0, $192 = 0, $193 = 0, $194 = 0, $195 = 0, $196 = 0, $197 = 0, $198 = 0, $199 = 0, $2 = 0, $20 = 0, $200 = 0, $201 = 0, $202 = 0, $203 = 0;
 var $204 = 0, $205 = 0, $206 = 0, $207 = 0, $208 = 0, $209 = 0, $21 = 0, $210 = 0, $211 = 0, $212 = 0, $213 = 0, $214 = 0, $215 = 0, $216 = 0, $217 = 0, $218 = 0, $219 = 0, $22 = 0, $220 = 0, $221 = 0;
 var $222 = 0, $223 = 0, $224 = 0, $225 = 0, $226 = 0, $227 = 0, $228 = 0, $229 = 0, $23 = 0, $230 = 0, $231 = 0, $232 = 0, $233 = 0, $234 = 0, $235 = 0, $236 = 0, $237 = 0, $238 = 0, $239 = 0, $24 = 0;
 var $240 = 0, $241 = 0, $242 = 0, $243 = 0, $244 = 0, $245 = 0, $246 = 0, $247 = 0, $248 = 0, $249 = 0, $25 = 0, $250 = 0, $251 = 0, $252 = 0, $253 = 0, $254 = 0, $255 = 0, $256 = 0, $257 = 0, $258 = 0;
 var $259 = 0, $26 = 0, $260 = 0, $261 = 0, $262 = 0, $263 = 0, $264 = 0, $265 = 0, $266 = 0, $267 = 0, $268 = 0, $269 = 0, $27 = 0, $270 = 0, $271 = 0, $272 = 0, $273 = 0, $274 = 0, $275 = 0, $276 = 0;
 var $277 = 0, $278 = 0, $279 = 0, $28 = 0, $280 = 0, $281 = 0, $282 = 0, $283 = 0, $284 = 0, $285 = 0, $286 = 0, $287 = 0, $288 = 0, $289 = 0, $29 = 0, $290 = 0, $291 = 0, $292 = 0, $293 = 0, $294 = 0;
 var $295 = 0, $296 = 0, $297 = 0, $298 = 0, $299 = 0, $3 = 0, $30 = 0, $300 = 0, $301 = 0, $302 = 0, $303 = 0, $304 = 0, $305 = 0, $306 = 0, $307 = 0, $308 = 0, $309 = 0, $31 = 0, $310 = 0, $311 = 0;
 var $312 = 0, $313 = 0, $314 = 0, $315 = 0, $316 = 0, $317 = 0, $318 = 0, $319 = 0, $32 = 0, $320 = 0, $321 = 0, $322 = 0, $323 = 0, $324 = 0, $325 = 0, $326 = 0, $327 = 0, $328 = 0, $329 = 0, $33 = 0;
 var $330 = 0, $331 = 0, $332 = 0, $333 = 0, $334 = 0, $335 = 0, $336 = 0, $337 = 0, $338 = 0, $339 = 0, $34 = 0, $340 = 0, $341 = 0, $342 = 0, $343 = 0, $344 = 0, $345 = 0, $346 = 0, $347 = 0, $348 = 0;
 var $349 = 0, $35 = 0, $350 = 0, $351 = 0, $352 = 0, $353 = 0, $354 = 0, $355 = 0, $356 = 0, $357 = 0, $358 = 0, $359 = 0, $36 = 0, $360 = 0, $361 = 0, $362 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0;
 var $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0;
 var $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0;
 var $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0;
 var $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $aMode = 0, $c = 0, $eState = 0, $flags = 0, $i = 0, $iIn = 0, $iOut = 0, $limit = 0, $mask = 0, $mode = 0, $nByte = 0, $nOpt = 0, $nUri = 0, $nVal = 0, $octet = 0;
 var $or$cond = 0, $rc = 0, $vararg_buffer19 = 0, $vararg_ptr10 = 0, $vararg_ptr14 = 0, $vararg_ptr18 = 0, $z = 0, $zFile = 0, $zModeType = 0, $zOpt = 0, $zVal = 0, $zVfs = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 128|0;
 $vararg_buffer19 = sp;
 $0 = sp + 116|0;
 $1 = sp + 112|0;
 $2 = sp + 108|0;
 $3 = sp + 104|0;
 $4 = sp + 100|0;
 $5 = sp + 96|0;
 $6 = sp + 92|0;
 $rc = sp + 88|0;
 $flags = sp + 84|0;
 $zVfs = sp + 80|0;
 $zFile = sp + 76|0;
 $c = sp + 120|0;
 $nUri = sp + 72|0;
 $zOpt = sp + 68|0;
 $eState = sp + 64|0;
 $iIn = sp + 60|0;
 $iOut = sp + 56|0;
 $nByte = sp + 52|0;
 $octet = sp + 48|0;
 $nOpt = sp + 44|0;
 $zVal = sp + 40|0;
 $nVal = sp + 36|0;
 $aMode = sp + 32|0;
 $zModeType = sp + 28|0;
 $mask = sp + 24|0;
 $limit = sp + 20|0;
 $i = sp + 16|0;
 $mode = sp + 12|0;
 $z = sp + 8|0;
 HEAP32[$1>>2] = $zDefaultVfs;
 HEAP32[$2>>2] = $zUri;
 HEAP32[$3>>2] = $pFlags;
 HEAP32[$4>>2] = $ppVfs;
 HEAP32[$5>>2] = $pzFile;
 HEAP32[$6>>2] = $pzErrMsg;
 HEAP32[$rc>>2] = 0;
 $7 = HEAP32[$3>>2]|0;
 $8 = HEAP32[$7>>2]|0;
 HEAP32[$flags>>2] = $8;
 $9 = HEAP32[$1>>2]|0;
 HEAP32[$zVfs>>2] = $9;
 $10 = HEAP32[$2>>2]|0;
 $11 = (_sqlite3Strlen30($10)|0);
 HEAP32[$nUri>>2] = $11;
 $12 = HEAP32[$flags>>2]|0;
 $13 = $12 & 64;
 $14 = ($13|0)!=(0);
 if ($14) {
  $$old = HEAP32[$nUri>>2]|0;
  $$old1 = ($$old|0)>=(5);
  if ($$old1) {
   label = 4;
  } else {
   label = 83;
  }
 } else {
  $15 = HEAP32[((38640 + 12|0))>>2]|0;
  $16 = ($15|0)!=(0);
  $17 = HEAP32[$nUri>>2]|0;
  $18 = ($17|0)>=(5);
  $or$cond = $16 & $18;
  if ($or$cond) {
   label = 4;
  } else {
   label = 83;
  }
 }
 L4: do {
  if ((label|0) == 4) {
   $19 = HEAP32[$2>>2]|0;
   $20 = (_memcmp($19,4608,5)|0);
   $21 = ($20|0)==(0);
   if ($21) {
    HEAP32[$iOut>>2] = 0;
    $22 = HEAP32[$nUri>>2]|0;
    $23 = (($22) + 2)|0;
    HEAP32[$nByte>>2] = $23;
    $24 = HEAP32[$flags>>2]|0;
    $25 = $24 | 64;
    HEAP32[$flags>>2] = $25;
    HEAP32[$iIn>>2] = 0;
    while(1) {
     $26 = HEAP32[$iIn>>2]|0;
     $27 = HEAP32[$nUri>>2]|0;
     $28 = ($26|0)<($27|0);
     if (!($28)) {
      break;
     }
     $29 = HEAP32[$iIn>>2]|0;
     $30 = HEAP32[$2>>2]|0;
     $31 = (($30) + ($29)|0);
     $32 = HEAP8[$31>>0]|0;
     $33 = $32 << 24 >> 24;
     $34 = ($33|0)==(38);
     $35 = $34&1;
     $36 = HEAP32[$nByte>>2]|0;
     $37 = (($36) + ($35))|0;
     HEAP32[$nByte>>2] = $37;
     $38 = HEAP32[$iIn>>2]|0;
     $39 = (($38) + 1)|0;
     HEAP32[$iIn>>2] = $39;
    }
    $40 = HEAP32[$nByte>>2]|0;
    $41 = (_sqlite3_malloc($40)|0);
    HEAP32[$zFile>>2] = $41;
    $42 = HEAP32[$zFile>>2]|0;
    $43 = ($42|0)!=(0|0);
    if (!($43)) {
     HEAP32[$0>>2] = 7;
     $361 = HEAP32[$0>>2]|0;
     STACKTOP = sp;return ($361|0);
    }
    HEAP32[$iIn>>2] = 5;
    $44 = HEAP32[$2>>2]|0;
    $45 = (($44) + 5|0);
    $46 = HEAP8[$45>>0]|0;
    $47 = $46 << 24 >> 24;
    $48 = ($47|0)==(47);
    do {
     if ($48) {
      $49 = HEAP32[$2>>2]|0;
      $50 = (($49) + 6|0);
      $51 = HEAP8[$50>>0]|0;
      $52 = $51 << 24 >> 24;
      $53 = ($52|0)==(47);
      if ($53) {
       HEAP32[$iIn>>2] = 7;
       while(1) {
        $54 = HEAP32[$iIn>>2]|0;
        $55 = HEAP32[$2>>2]|0;
        $56 = (($55) + ($54)|0);
        $57 = HEAP8[$56>>0]|0;
        $58 = $57 << 24 >> 24;
        $59 = ($58|0)!=(0);
        if ($59) {
         $60 = HEAP32[$iIn>>2]|0;
         $61 = HEAP32[$2>>2]|0;
         $62 = (($61) + ($60)|0);
         $63 = HEAP8[$62>>0]|0;
         $64 = $63 << 24 >> 24;
         $65 = ($64|0)!=(47);
         $362 = $65;
        } else {
         $362 = 0;
        }
        $66 = HEAP32[$iIn>>2]|0;
        if (!($362)) {
         break;
        }
        $67 = (($66) + 1)|0;
        HEAP32[$iIn>>2] = $67;
       }
       $68 = ($66|0)!=(7);
       if ($68) {
        $69 = HEAP32[$iIn>>2]|0;
        $70 = ($69|0)!=(16);
        if (!($70)) {
         $71 = HEAP32[$2>>2]|0;
         $72 = (($71) + 7|0);
         $73 = (_memcmp(4616,$72,9)|0);
         $74 = ($73|0)!=(0);
         if (!($74)) {
          break;
         }
        }
        $75 = HEAP32[$iIn>>2]|0;
        $76 = (($75) - 7)|0;
        $77 = HEAP32[$2>>2]|0;
        $78 = (($77) + 7|0);
        HEAP32[$vararg_buffer19>>2] = $76;
        $vararg_ptr10 = (($vararg_buffer19) + 4|0);
        HEAP32[$vararg_ptr10>>2] = $78;
        $79 = (_sqlite3_mprintf(4632,$vararg_buffer19)|0);
        $80 = HEAP32[$6>>2]|0;
        HEAP32[$80>>2] = $79;
        HEAP32[$rc>>2] = 1;
        break L4;
       }
      }
     }
    } while(0);
    HEAP32[$eState>>2] = 0;
    L30: while(1) {
     $81 = HEAP32[$iIn>>2]|0;
     $82 = HEAP32[$2>>2]|0;
     $83 = (($82) + ($81)|0);
     $84 = HEAP8[$83>>0]|0;
     HEAP8[$c>>0] = $84;
     $85 = $84 << 24 >> 24;
     $86 = ($85|0)!=(0);
     if (!($86)) {
      break;
     }
     $87 = HEAP8[$c>>0]|0;
     $88 = $87 << 24 >> 24;
     $89 = ($88|0)!=(35);
     if (!($89)) {
      break;
     }
     $90 = HEAP32[$iIn>>2]|0;
     $91 = (($90) + 1)|0;
     HEAP32[$iIn>>2] = $91;
     $92 = HEAP8[$c>>0]|0;
     $93 = $92 << 24 >> 24;
     $94 = ($93|0)==(37);
     do {
      if ($94) {
       $95 = HEAP32[$iIn>>2]|0;
       $96 = HEAP32[$2>>2]|0;
       $97 = (($96) + ($95)|0);
       $98 = HEAP8[$97>>0]|0;
       $99 = $98&255;
       $100 = (1224 + ($99)|0);
       $101 = HEAP8[$100>>0]|0;
       $102 = $101&255;
       $103 = $102 & 8;
       $104 = ($103|0)!=(0);
       if ($104) {
        $105 = HEAP32[$iIn>>2]|0;
        $106 = (($105) + 1)|0;
        $107 = HEAP32[$2>>2]|0;
        $108 = (($107) + ($106)|0);
        $109 = HEAP8[$108>>0]|0;
        $110 = $109&255;
        $111 = (1224 + ($110)|0);
        $112 = HEAP8[$111>>0]|0;
        $113 = $112&255;
        $114 = $113 & 8;
        $115 = ($114|0)!=(0);
        if ($115) {
         $116 = HEAP32[$iIn>>2]|0;
         $117 = (($116) + 1)|0;
         HEAP32[$iIn>>2] = $117;
         $118 = HEAP32[$2>>2]|0;
         $119 = (($118) + ($116)|0);
         $120 = HEAP8[$119>>0]|0;
         $121 = $120 << 24 >> 24;
         $122 = (_sqlite3HexToInt($121)|0);
         $123 = $122&255;
         $124 = $123 << 4;
         HEAP32[$octet>>2] = $124;
         $125 = HEAP32[$iIn>>2]|0;
         $126 = (($125) + 1)|0;
         HEAP32[$iIn>>2] = $126;
         $127 = HEAP32[$2>>2]|0;
         $128 = (($127) + ($125)|0);
         $129 = HEAP8[$128>>0]|0;
         $130 = $129 << 24 >> 24;
         $131 = (_sqlite3HexToInt($130)|0);
         $132 = $131&255;
         $133 = HEAP32[$octet>>2]|0;
         $134 = (($133) + ($132))|0;
         HEAP32[$octet>>2] = $134;
         $135 = HEAP32[$octet>>2]|0;
         $136 = ($135|0)==(0);
         if (!($136)) {
          $166 = HEAP32[$octet>>2]|0;
          $167 = $166&255;
          HEAP8[$c>>0] = $167;
          break;
         }
         while(1) {
          $137 = HEAP32[$iIn>>2]|0;
          $138 = HEAP32[$2>>2]|0;
          $139 = (($138) + ($137)|0);
          $140 = HEAP8[$139>>0]|0;
          HEAP8[$c>>0] = $140;
          $141 = $140 << 24 >> 24;
          $142 = ($141|0)!=(0);
          if (!($142)) {
           continue L30;
          }
          $143 = HEAP8[$c>>0]|0;
          $144 = $143 << 24 >> 24;
          $145 = ($144|0)!=(35);
          if (!($145)) {
           continue L30;
          }
          $146 = HEAP32[$eState>>2]|0;
          $147 = ($146|0)!=(0);
          if (!($147)) {
           $148 = HEAP8[$c>>0]|0;
           $149 = $148 << 24 >> 24;
           $150 = ($149|0)!=(63);
           if (!($150)) {
            continue L30;
           }
          }
          $151 = HEAP32[$eState>>2]|0;
          $152 = ($151|0)!=(1);
          if (!($152)) {
           $153 = HEAP8[$c>>0]|0;
           $154 = $153 << 24 >> 24;
           $155 = ($154|0)!=(61);
           if (!($155)) {
            continue L30;
           }
           $156 = HEAP8[$c>>0]|0;
           $157 = $156 << 24 >> 24;
           $158 = ($157|0)!=(38);
           if (!($158)) {
            continue L30;
           }
          }
          $159 = HEAP32[$eState>>2]|0;
          $160 = ($159|0)!=(2);
          if (!($160)) {
           $161 = HEAP8[$c>>0]|0;
           $162 = $161 << 24 >> 24;
           $163 = ($162|0)!=(38);
           if (!($163)) {
            continue L30;
           }
          }
          $164 = HEAP32[$iIn>>2]|0;
          $165 = (($164) + 1)|0;
          HEAP32[$iIn>>2] = $165;
         }
        } else {
         label = 39;
        }
       } else {
        label = 39;
       }
      } else {
       label = 39;
      }
     } while(0);
     L54: do {
      if ((label|0) == 39) {
       label = 0;
       $168 = HEAP32[$eState>>2]|0;
       $169 = ($168|0)==(1);
       do {
        if ($169) {
         $170 = HEAP8[$c>>0]|0;
         $171 = $170 << 24 >> 24;
         $172 = ($171|0)==(38);
         if (!($172)) {
          $173 = HEAP8[$c>>0]|0;
          $174 = $173 << 24 >> 24;
          $175 = ($174|0)==(61);
          if (!($175)) {
           break;
          }
         }
         $176 = HEAP32[$iOut>>2]|0;
         $177 = (($176) - 1)|0;
         $178 = HEAP32[$zFile>>2]|0;
         $179 = (($178) + ($177)|0);
         $180 = HEAP8[$179>>0]|0;
         $181 = $180 << 24 >> 24;
         $182 = ($181|0)==(0);
         if ($182) {
          while(1) {
           $183 = HEAP32[$iIn>>2]|0;
           $184 = HEAP32[$2>>2]|0;
           $185 = (($184) + ($183)|0);
           $186 = HEAP8[$185>>0]|0;
           $187 = $186 << 24 >> 24;
           $188 = ($187|0)!=(0);
           if (!($188)) {
            continue L30;
           }
           $189 = HEAP32[$iIn>>2]|0;
           $190 = HEAP32[$2>>2]|0;
           $191 = (($190) + ($189)|0);
           $192 = HEAP8[$191>>0]|0;
           $193 = $192 << 24 >> 24;
           $194 = ($193|0)!=(35);
           if (!($194)) {
            continue L30;
           }
           $195 = HEAP32[$iIn>>2]|0;
           $196 = (($195) - 1)|0;
           $197 = HEAP32[$2>>2]|0;
           $198 = (($197) + ($196)|0);
           $199 = HEAP8[$198>>0]|0;
           $200 = $199 << 24 >> 24;
           $201 = ($200|0)!=(38);
           if (!($201)) {
            continue L30;
           }
           $202 = HEAP32[$iIn>>2]|0;
           $203 = (($202) + 1)|0;
           HEAP32[$iIn>>2] = $203;
          }
         } else {
          $204 = HEAP8[$c>>0]|0;
          $205 = $204 << 24 >> 24;
          $206 = ($205|0)==(38);
          if ($206) {
           $207 = HEAP32[$iOut>>2]|0;
           $208 = (($207) + 1)|0;
           HEAP32[$iOut>>2] = $208;
           $209 = HEAP32[$zFile>>2]|0;
           $210 = (($209) + ($207)|0);
           HEAP8[$210>>0] = 0;
          } else {
           HEAP32[$eState>>2] = 2;
          }
          HEAP8[$c>>0] = 0;
          break L54;
         }
        }
       } while(0);
       $211 = HEAP32[$eState>>2]|0;
       $212 = ($211|0)==(0);
       if ($212) {
        $213 = HEAP8[$c>>0]|0;
        $214 = $213 << 24 >> 24;
        $215 = ($214|0)==(63);
        if (!($215)) {
         label = 53;
        }
       } else {
        label = 53;
       }
       if ((label|0) == 53) {
        label = 0;
        $216 = HEAP32[$eState>>2]|0;
        $217 = ($216|0)==(2);
        if (!($217)) {
         break;
        }
        $218 = HEAP8[$c>>0]|0;
        $219 = $218 << 24 >> 24;
        $220 = ($219|0)==(38);
        if (!($220)) {
         break;
        }
       }
       HEAP8[$c>>0] = 0;
       HEAP32[$eState>>2] = 1;
      }
     } while(0);
     $221 = HEAP8[$c>>0]|0;
     $222 = HEAP32[$iOut>>2]|0;
     $223 = (($222) + 1)|0;
     HEAP32[$iOut>>2] = $223;
     $224 = HEAP32[$zFile>>2]|0;
     $225 = (($224) + ($222)|0);
     HEAP8[$225>>0] = $221;
    }
    $226 = HEAP32[$eState>>2]|0;
    $227 = ($226|0)==(1);
    if ($227) {
     $228 = HEAP32[$iOut>>2]|0;
     $229 = (($228) + 1)|0;
     HEAP32[$iOut>>2] = $229;
     $230 = HEAP32[$zFile>>2]|0;
     $231 = (($230) + ($228)|0);
     HEAP8[$231>>0] = 0;
    }
    $232 = HEAP32[$iOut>>2]|0;
    $233 = (($232) + 1)|0;
    HEAP32[$iOut>>2] = $233;
    $234 = HEAP32[$zFile>>2]|0;
    $235 = (($234) + ($232)|0);
    HEAP8[$235>>0] = 0;
    $236 = HEAP32[$iOut>>2]|0;
    $237 = (($236) + 1)|0;
    HEAP32[$iOut>>2] = $237;
    $238 = HEAP32[$zFile>>2]|0;
    $239 = (($238) + ($236)|0);
    HEAP8[$239>>0] = 0;
    $240 = HEAP32[$zFile>>2]|0;
    $241 = (_sqlite3Strlen30($240)|0);
    $242 = (($241) + 1)|0;
    $243 = HEAP32[$zFile>>2]|0;
    $244 = (($243) + ($242)|0);
    HEAP32[$zOpt>>2] = $244;
    while(1) {
     $245 = HEAP32[$zOpt>>2]|0;
     $246 = HEAP8[$245>>0]|0;
     $247 = ($246<<24>>24)!=(0);
     if (!($247)) {
      label = 86;
      break L4;
     }
     $248 = HEAP32[$zOpt>>2]|0;
     $249 = (_sqlite3Strlen30($248)|0);
     HEAP32[$nOpt>>2] = $249;
     $250 = HEAP32[$nOpt>>2]|0;
     $251 = (($250) + 1)|0;
     $252 = HEAP32[$zOpt>>2]|0;
     $253 = (($252) + ($251)|0);
     HEAP32[$zVal>>2] = $253;
     $254 = HEAP32[$zVal>>2]|0;
     $255 = (_sqlite3Strlen30($254)|0);
     HEAP32[$nVal>>2] = $255;
     $256 = HEAP32[$nOpt>>2]|0;
     $257 = ($256|0)==(3);
     if ($257) {
      $258 = HEAP32[$zOpt>>2]|0;
      $259 = (_memcmp(4664,$258,3)|0);
      $260 = ($259|0)==(0);
      if ($260) {
       $261 = HEAP32[$zVal>>2]|0;
       HEAP32[$zVfs>>2] = $261;
      } else {
       label = 64;
      }
     } else {
      label = 64;
     }
     if ((label|0) == 64) {
      label = 0;
      HEAP32[$aMode>>2] = 0;
      HEAP32[$zModeType>>2] = 0;
      HEAP32[$mask>>2] = 0;
      HEAP32[$limit>>2] = 0;
      $262 = HEAP32[$nOpt>>2]|0;
      $263 = ($262|0)==(5);
      if ($263) {
       $264 = HEAP32[$zOpt>>2]|0;
       $265 = (_memcmp(4672,$264,5)|0);
       $266 = ($265|0)==(0);
       if ($266) {
        HEAP32[$mask>>2] = 393216;
        HEAP32[$aMode>>2] = 4680;
        $267 = HEAP32[$mask>>2]|0;
        HEAP32[$limit>>2] = $267;
        HEAP32[$zModeType>>2] = 4672;
       }
      }
      $268 = HEAP32[$nOpt>>2]|0;
      $269 = ($268|0)==(4);
      if ($269) {
       $270 = HEAP32[$zOpt>>2]|0;
       $271 = (_memcmp(4720,$270,4)|0);
       $272 = ($271|0)==(0);
       if ($272) {
        HEAP32[$mask>>2] = 135;
        HEAP32[$aMode>>2] = 4728;
        $273 = HEAP32[$mask>>2]|0;
        $274 = HEAP32[$flags>>2]|0;
        $275 = $273 & $274;
        HEAP32[$limit>>2] = $275;
        HEAP32[$zModeType>>2] = 4800;
       }
      }
      $276 = HEAP32[$aMode>>2]|0;
      $277 = ($276|0)!=(0|0);
      if ($277) {
       HEAP32[$mode>>2] = 0;
       HEAP32[$i>>2] = 0;
       while(1) {
        $278 = HEAP32[$i>>2]|0;
        $279 = HEAP32[$aMode>>2]|0;
        $280 = (($279) + ($278<<3)|0);
        $281 = HEAP32[$280>>2]|0;
        $282 = ($281|0)!=(0|0);
        if (!($282)) {
         break;
        }
        $283 = HEAP32[$i>>2]|0;
        $284 = HEAP32[$aMode>>2]|0;
        $285 = (($284) + ($283<<3)|0);
        $286 = HEAP32[$285>>2]|0;
        HEAP32[$z>>2] = $286;
        $287 = HEAP32[$nVal>>2]|0;
        $288 = HEAP32[$z>>2]|0;
        $289 = (_sqlite3Strlen30($288)|0);
        $290 = ($287|0)==($289|0);
        if ($290) {
         $291 = HEAP32[$zVal>>2]|0;
         $292 = HEAP32[$z>>2]|0;
         $293 = HEAP32[$nVal>>2]|0;
         $294 = (_memcmp($291,$292,$293)|0);
         $295 = (0)==($294|0);
         if ($295) {
          label = 75;
          break;
         }
        }
        $301 = HEAP32[$i>>2]|0;
        $302 = (($301) + 1)|0;
        HEAP32[$i>>2] = $302;
       }
       if ((label|0) == 75) {
        label = 0;
        $296 = HEAP32[$i>>2]|0;
        $297 = HEAP32[$aMode>>2]|0;
        $298 = (($297) + ($296<<3)|0);
        $299 = (($298) + 4|0);
        $300 = HEAP32[$299>>2]|0;
        HEAP32[$mode>>2] = $300;
       }
       $303 = HEAP32[$mode>>2]|0;
       $304 = ($303|0)==(0);
       if ($304) {
        label = 78;
        break;
       }
       $309 = HEAP32[$mode>>2]|0;
       $310 = $309 & -129;
       $311 = HEAP32[$limit>>2]|0;
       $312 = ($310|0)>($311|0);
       if ($312) {
        label = 80;
        break;
       }
       $317 = HEAP32[$flags>>2]|0;
       $318 = HEAP32[$mask>>2]|0;
       $319 = $318 ^ -1;
       $320 = $317 & $319;
       $321 = HEAP32[$mode>>2]|0;
       $322 = $320 | $321;
       HEAP32[$flags>>2] = $322;
      }
     }
     $323 = HEAP32[$nVal>>2]|0;
     $324 = (($323) + 1)|0;
     $325 = HEAP32[$zVal>>2]|0;
     $326 = (($325) + ($324)|0);
     HEAP32[$zOpt>>2] = $326;
    }
    if ((label|0) == 78) {
     $305 = HEAP32[$zModeType>>2]|0;
     $306 = HEAP32[$zVal>>2]|0;
     HEAP32[$vararg_buffer19>>2] = $305;
     $vararg_ptr14 = (($vararg_buffer19) + 4|0);
     HEAP32[$vararg_ptr14>>2] = $306;
     $307 = (_sqlite3_mprintf(4808,$vararg_buffer19)|0);
     $308 = HEAP32[$6>>2]|0;
     HEAP32[$308>>2] = $307;
     HEAP32[$rc>>2] = 1;
     break;
    }
    else if ((label|0) == 80) {
     $313 = HEAP32[$zModeType>>2]|0;
     $314 = HEAP32[$zVal>>2]|0;
     HEAP32[$vararg_buffer19>>2] = $313;
     $vararg_ptr18 = (($vararg_buffer19) + 4|0);
     HEAP32[$vararg_ptr18>>2] = $314;
     $315 = (_sqlite3_mprintf(4832,$vararg_buffer19)|0);
     $316 = HEAP32[$6>>2]|0;
     HEAP32[$316>>2] = $315;
     HEAP32[$rc>>2] = 3;
     break;
    }
   } else {
    label = 83;
   }
  }
 } while(0);
 do {
  if ((label|0) == 83) {
   $327 = HEAP32[$nUri>>2]|0;
   $328 = (($327) + 2)|0;
   $329 = (_sqlite3_malloc($328)|0);
   HEAP32[$zFile>>2] = $329;
   $330 = HEAP32[$zFile>>2]|0;
   $331 = ($330|0)!=(0|0);
   if ($331) {
    $332 = HEAP32[$zFile>>2]|0;
    $333 = HEAP32[$2>>2]|0;
    $334 = HEAP32[$nUri>>2]|0;
    _memcpy(($332|0),($333|0),($334|0))|0;
    $335 = HEAP32[$nUri>>2]|0;
    $336 = HEAP32[$zFile>>2]|0;
    $337 = (($336) + ($335)|0);
    HEAP8[$337>>0] = 0;
    $338 = HEAP32[$nUri>>2]|0;
    $339 = (($338) + 1)|0;
    $340 = HEAP32[$zFile>>2]|0;
    $341 = (($340) + ($339)|0);
    HEAP8[$341>>0] = 0;
    $342 = HEAP32[$flags>>2]|0;
    $343 = $342 & -65;
    HEAP32[$flags>>2] = $343;
    label = 86;
    break;
   }
   HEAP32[$0>>2] = 7;
   $361 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($361|0);
  }
 } while(0);
 if ((label|0) == 86) {
  $344 = HEAP32[$zVfs>>2]|0;
  $345 = (_sqlite3_vfs_find($344)|0);
  $346 = HEAP32[$4>>2]|0;
  HEAP32[$346>>2] = $345;
  $347 = HEAP32[$4>>2]|0;
  $348 = HEAP32[$347>>2]|0;
  $349 = ($348|0)==(0|0);
  if ($349) {
   $350 = HEAP32[$zVfs>>2]|0;
   HEAP32[$vararg_buffer19>>2] = $350;
   $351 = (_sqlite3_mprintf(4856,$vararg_buffer19)|0);
   $352 = HEAP32[$6>>2]|0;
   HEAP32[$352>>2] = $351;
   HEAP32[$rc>>2] = 1;
  }
 }
 $353 = HEAP32[$rc>>2]|0;
 $354 = ($353|0)!=(0);
 if ($354) {
  $355 = HEAP32[$zFile>>2]|0;
  _sqlite3_free($355);
  HEAP32[$zFile>>2] = 0;
 }
 $356 = HEAP32[$flags>>2]|0;
 $357 = HEAP32[$3>>2]|0;
 HEAP32[$357>>2] = $356;
 $358 = HEAP32[$zFile>>2]|0;
 $359 = HEAP32[$5>>2]|0;
 HEAP32[$359>>2] = $358;
 $360 = HEAP32[$rc>>2]|0;
 HEAP32[$0>>2] = $360;
 $361 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($361|0);
}
function _sqlite3BtreeOpen($pVfs,$zFilename,$db,$ppBtree,$flags,$vfsFlags) {
 $pVfs = $pVfs|0;
 $zFilename = $zFilename|0;
 $db = $db|0;
 $ppBtree = $ppBtree|0;
 $flags = $flags|0;
 $vfsFlags = $vfsFlags|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0;
 var $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0, $188 = 0;
 var $189 = 0, $19 = 0, $190 = 0, $191 = 0, $192 = 0, $193 = 0, $194 = 0, $195 = 0, $196 = 0, $197 = 0, $198 = 0, $199 = 0, $2 = 0, $20 = 0, $200 = 0, $201 = 0, $202 = 0, $203 = 0, $204 = 0, $205 = 0;
 var $206 = 0, $207 = 0, $208 = 0, $209 = 0, $21 = 0, $210 = 0, $211 = 0, $212 = 0, $213 = 0, $214 = 0, $215 = 0, $216 = 0, $217 = 0, $218 = 0, $219 = 0, $22 = 0, $220 = 0, $221 = 0, $222 = 0, $223 = 0;
 var $224 = 0, $225 = 0, $226 = 0, $227 = 0, $228 = 0, $229 = 0, $23 = 0, $230 = 0, $231 = 0, $232 = 0, $233 = 0, $234 = 0, $235 = 0, $236 = 0, $237 = 0, $238 = 0, $239 = 0, $24 = 0, $240 = 0, $241 = 0;
 var $242 = 0, $243 = 0, $244 = 0, $245 = 0, $246 = 0, $247 = 0, $248 = 0, $249 = 0, $25 = 0, $250 = 0, $251 = 0, $252 = 0, $253 = 0, $254 = 0, $255 = 0, $256 = 0, $257 = 0, $258 = 0, $259 = 0, $26 = 0;
 var $260 = 0, $261 = 0, $262 = 0, $263 = 0, $264 = 0, $265 = 0, $266 = 0, $267 = 0, $268 = 0, $269 = 0, $27 = 0, $270 = 0, $271 = 0, $272 = 0, $273 = 0, $274 = 0, $275 = 0, $276 = 0, $277 = 0, $278 = 0;
 var $279 = 0, $28 = 0, $280 = 0, $281 = 0, $282 = 0, $283 = 0, $284 = 0, $285 = 0, $286 = 0, $287 = 0, $288 = 0, $289 = 0, $29 = 0, $290 = 0, $291 = 0, $292 = 0, $293 = 0, $294 = 0, $295 = 0, $296 = 0;
 var $297 = 0, $298 = 0, $299 = 0, $3 = 0, $30 = 0, $300 = 0, $301 = 0, $302 = 0, $303 = 0, $304 = 0, $305 = 0, $306 = 0, $307 = 0, $308 = 0, $309 = 0, $31 = 0, $310 = 0, $311 = 0, $312 = 0, $313 = 0;
 var $314 = 0, $315 = 0, $316 = 0, $317 = 0, $318 = 0, $319 = 0, $32 = 0, $320 = 0, $321 = 0, $322 = 0, $323 = 0, $324 = 0, $325 = 0, $326 = 0, $327 = 0, $328 = 0, $329 = 0, $33 = 0, $330 = 0, $331 = 0;
 var $332 = 0, $333 = 0, $334 = 0, $335 = 0, $336 = 0, $337 = 0, $338 = 0, $339 = 0, $34 = 0, $340 = 0, $341 = 0, $342 = 0, $343 = 0, $344 = 0, $345 = 0, $346 = 0, $347 = 0, $348 = 0, $349 = 0, $35 = 0;
 var $350 = 0, $351 = 0, $352 = 0, $353 = 0, $354 = 0, $355 = 0, $356 = 0, $357 = 0, $358 = 0, $359 = 0, $36 = 0, $360 = 0, $361 = 0, $362 = 0, $363 = 0, $364 = 0, $365 = 0, $366 = 0, $367 = 0, $368 = 0;
 var $369 = 0, $37 = 0, $370 = 0, $371 = 0, $372 = 0, $373 = 0, $374 = 0, $375 = 0, $376 = 0, $377 = 0, $378 = 0, $379 = 0, $38 = 0, $380 = 0, $381 = 0, $382 = 0, $383 = 0, $384 = 0, $385 = 0, $386 = 0;
 var $387 = 0, $388 = 0, $389 = 0, $39 = 0, $390 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0;
 var $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0;
 var $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0;
 var $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $i = 0, $iDb = 0, $isMemdb = 0, $isTempDb = 0, $mutexOpen = 0, $nFullPathname = 0, $nReserve = 0, $or$cond = 0, $or$cond3 = 0;
 var $p = 0, $pBt = 0, $pExisting = 0, $pSib = 0, $rc = 0, $zDbHeader = 0, $zFullPathname = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 192|0;
 $0 = sp + 72|0;
 $1 = sp + 68|0;
 $2 = sp + 64|0;
 $3 = sp + 60|0;
 $4 = sp + 56|0;
 $5 = sp + 52|0;
 $6 = sp + 48|0;
 $pBt = sp + 44|0;
 $p = sp + 40|0;
 $mutexOpen = sp + 36|0;
 $rc = sp + 32|0;
 $nReserve = sp + 176|0;
 $zDbHeader = sp + 76|0;
 $isTempDb = sp + 28|0;
 $isMemdb = sp + 24|0;
 $nFullPathname = sp + 20|0;
 $zFullPathname = sp + 16|0;
 $iDb = sp + 12|0;
 $pExisting = sp + 8|0;
 $i = sp + 4|0;
 $pSib = sp;
 HEAP32[$1>>2] = $pVfs;
 HEAP32[$2>>2] = $zFilename;
 HEAP32[$3>>2] = $db;
 HEAP32[$4>>2] = $ppBtree;
 HEAP32[$5>>2] = $flags;
 HEAP32[$6>>2] = $vfsFlags;
 HEAP32[$pBt>>2] = 0;
 HEAP32[$mutexOpen>>2] = 0;
 HEAP32[$rc>>2] = 0;
 $7 = HEAP32[$2>>2]|0;
 $8 = ($7|0)==(0|0);
 if ($8) {
  $14 = 1;
 } else {
  $9 = HEAP32[$2>>2]|0;
  $10 = HEAP8[$9>>0]|0;
  $11 = $10 << 24 >> 24;
  $12 = ($11|0)==(0);
  $14 = $12;
 }
 $13 = $14&1;
 HEAP32[$isTempDb>>2] = $13;
 $15 = HEAP32[$2>>2]|0;
 $16 = ($15|0)!=(0|0);
 if ($16) {
  $17 = HEAP32[$2>>2]|0;
  $18 = (_strcmp($17,4216)|0);
  $19 = ($18|0)==(0);
  if ($19) {
   $29 = 1;
  } else {
   label = 5;
  }
 } else {
  label = 5;
 }
 do {
  if ((label|0) == 5) {
   $20 = HEAP32[$isTempDb>>2]|0;
   $21 = ($20|0)!=(0);
   if ($21) {
    $22 = HEAP32[$3>>2]|0;
    $23 = (_sqlite3TempInMemory($22)|0);
    $24 = ($23|0)!=(0);
    if ($24) {
     $29 = 1;
     break;
    }
   }
   $25 = HEAP32[$6>>2]|0;
   $26 = $25 & 128;
   $27 = ($26|0)!=(0);
   $29 = $27;
  }
 } while(0);
 $28 = $29&1;
 HEAP32[$isMemdb>>2] = $28;
 $30 = HEAP32[$isMemdb>>2]|0;
 $31 = ($30|0)!=(0);
 if ($31) {
  $32 = HEAP32[$5>>2]|0;
  $33 = $32 | 2;
  HEAP32[$5>>2] = $33;
 }
 $34 = HEAP32[$6>>2]|0;
 $35 = $34 & 256;
 $36 = ($35|0)!=(0);
 if ($36) {
  $37 = HEAP32[$isMemdb>>2]|0;
  $38 = ($37|0)!=(0);
  $39 = HEAP32[$isTempDb>>2]|0;
  $40 = ($39|0)!=(0);
  $or$cond = $38 | $40;
  if ($or$cond) {
   $41 = HEAP32[$6>>2]|0;
   $42 = $41 & -257;
   $43 = $42 | 512;
   HEAP32[$6>>2] = $43;
  }
 }
 $44 = (_sqlite3MallocZero(44,0)|0);
 HEAP32[$p>>2] = $44;
 $45 = HEAP32[$p>>2]|0;
 $46 = ($45|0)!=(0|0);
 if (!($46)) {
  HEAP32[$0>>2] = 7;
  $389 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($389|0);
 }
 $47 = HEAP32[$p>>2]|0;
 $48 = (($47) + 8|0);
 HEAP8[$48>>0] = 0;
 $49 = HEAP32[$3>>2]|0;
 $50 = HEAP32[$p>>2]|0;
 HEAP32[$50>>2] = $49;
 $51 = HEAP32[$p>>2]|0;
 $52 = HEAP32[$p>>2]|0;
 $53 = (($52) + 28|0);
 HEAP32[$53>>2] = $51;
 $54 = HEAP32[$p>>2]|0;
 $55 = (($54) + 28|0);
 $56 = (($55) + 4|0);
 HEAP32[$56>>2] = 1;
 $57 = HEAP32[$isTempDb>>2]|0;
 $58 = ($57|0)==(0);
 do {
  if ($58) {
   $59 = HEAP32[$isMemdb>>2]|0;
   $60 = ($59|0)==(0);
   if (!($60)) {
    $61 = HEAP32[$6>>2]|0;
    $62 = $61 & 64;
    $63 = ($62|0)!=(0);
    if (!($63)) {
     break;
    }
   }
   $64 = HEAP32[$6>>2]|0;
   $65 = $64 & 131072;
   $66 = ($65|0)!=(0);
   if ($66) {
    $67 = HEAP32[$1>>2]|0;
    $68 = (($67) + 8|0);
    $69 = HEAP32[$68>>2]|0;
    $70 = (($69) + 1)|0;
    HEAP32[$nFullPathname>>2] = $70;
    $71 = HEAP32[$nFullPathname>>2]|0;
    $72 = ($71|0)<(0);
    $73 = $72 << 31 >> 31;
    $74 = (_sqlite3Malloc($71,$73)|0);
    HEAP32[$zFullPathname>>2] = $74;
    $75 = HEAP32[$p>>2]|0;
    $76 = (($75) + 9|0);
    HEAP8[$76>>0] = 1;
    $77 = HEAP32[$zFullPathname>>2]|0;
    $78 = ($77|0)!=(0|0);
    if (!($78)) {
     $79 = HEAP32[$p>>2]|0;
     _sqlite3_free($79);
     HEAP32[$0>>2] = 7;
     $389 = HEAP32[$0>>2]|0;
     STACKTOP = sp;return ($389|0);
    }
    $80 = HEAP32[$isMemdb>>2]|0;
    $81 = ($80|0)!=(0);
    if ($81) {
     $82 = HEAP32[$zFullPathname>>2]|0;
     $83 = HEAP32[$2>>2]|0;
     $84 = HEAP32[$2>>2]|0;
     $85 = (_sqlite3Strlen30($84)|0);
     $86 = (($85) + 1)|0;
     _memcpy(($82|0),($83|0),($86|0))|0;
    } else {
     $87 = HEAP32[$1>>2]|0;
     $88 = HEAP32[$2>>2]|0;
     $89 = HEAP32[$nFullPathname>>2]|0;
     $90 = HEAP32[$zFullPathname>>2]|0;
     $91 = (_sqlite3OsFullPathname($87,$88,$89,$90)|0);
     HEAP32[$rc>>2] = $91;
     $92 = HEAP32[$rc>>2]|0;
     $93 = ($92|0)!=(0);
     if ($93) {
      $94 = HEAP32[$zFullPathname>>2]|0;
      _sqlite3_free($94);
      $95 = HEAP32[$p>>2]|0;
      _sqlite3_free($95);
      $96 = HEAP32[$rc>>2]|0;
      HEAP32[$0>>2] = $96;
      $389 = HEAP32[$0>>2]|0;
      STACKTOP = sp;return ($389|0);
     }
    }
    $97 = HEAP32[4232>>2]|0;
    HEAP32[$pBt>>2] = $97;
    while(1) {
     $98 = HEAP32[$pBt>>2]|0;
     $99 = ($98|0)!=(0|0);
     if (!($99)) {
      break;
     }
     $100 = HEAP32[$zFullPathname>>2]|0;
     $101 = HEAP32[$pBt>>2]|0;
     $102 = HEAP32[$101>>2]|0;
     $103 = (_sqlite3PagerFilename($102,0)|0);
     $104 = (_strcmp($100,$103)|0);
     $105 = (0)==($104|0);
     if ($105) {
      $106 = HEAP32[$pBt>>2]|0;
      $107 = HEAP32[$106>>2]|0;
      $108 = (_sqlite3PagerVfs($107)|0);
      $109 = HEAP32[$1>>2]|0;
      $110 = ($108|0)==($109|0);
      if ($110) {
       label = 29;
       break;
      }
     }
     $142 = HEAP32[$pBt>>2]|0;
     $143 = (($142) + 68|0);
     $144 = HEAP32[$143>>2]|0;
     HEAP32[$pBt>>2] = $144;
    }
    do {
     if ((label|0) == 29) {
      $111 = HEAP32[$3>>2]|0;
      $112 = (($111) + 20|0);
      $113 = HEAP32[$112>>2]|0;
      $114 = (($113) - 1)|0;
      HEAP32[$iDb>>2] = $114;
      while(1) {
       $115 = HEAP32[$iDb>>2]|0;
       $116 = ($115|0)>=(0);
       if (!($116)) {
        label = 35;
        break;
       }
       $117 = HEAP32[$iDb>>2]|0;
       $118 = HEAP32[$3>>2]|0;
       $119 = (($118) + 16|0);
       $120 = HEAP32[$119>>2]|0;
       $121 = (($120) + ($117<<4)|0);
       $122 = (($121) + 4|0);
       $123 = HEAP32[$122>>2]|0;
       HEAP32[$pExisting>>2] = $123;
       $124 = HEAP32[$pExisting>>2]|0;
       $125 = ($124|0)!=(0|0);
       if ($125) {
        $126 = HEAP32[$pExisting>>2]|0;
        $127 = (($126) + 4|0);
        $128 = HEAP32[$127>>2]|0;
        $129 = HEAP32[$pBt>>2]|0;
        $130 = ($128|0)==($129|0);
        if ($130) {
         break;
        }
       }
       $133 = HEAP32[$iDb>>2]|0;
       $134 = (($133) + -1)|0;
       HEAP32[$iDb>>2] = $134;
      }
      if ((label|0) == 35) {
       $135 = HEAP32[$pBt>>2]|0;
       $136 = HEAP32[$p>>2]|0;
       $137 = (($136) + 4|0);
       HEAP32[$137>>2] = $135;
       $138 = HEAP32[$pBt>>2]|0;
       $139 = (($138) + 64|0);
       $140 = HEAP32[$139>>2]|0;
       $141 = (($140) + 1)|0;
       HEAP32[$139>>2] = $141;
       break;
      }
      $131 = HEAP32[$zFullPathname>>2]|0;
      _sqlite3_free($131);
      $132 = HEAP32[$p>>2]|0;
      _sqlite3_free($132);
      HEAP32[$0>>2] = 19;
      $389 = HEAP32[$0>>2]|0;
      STACKTOP = sp;return ($389|0);
     }
    } while(0);
    $145 = HEAP32[$zFullPathname>>2]|0;
    _sqlite3_free($145);
   }
  }
 } while(0);
 $146 = HEAP32[$pBt>>2]|0;
 $147 = ($146|0)==(0|0);
 do {
  if ($147) {
   $148 = (_sqlite3MallocZero(84,0)|0);
   HEAP32[$pBt>>2] = $148;
   $149 = HEAP32[$pBt>>2]|0;
   $150 = ($149|0)==(0|0);
   if ($150) {
    HEAP32[$rc>>2] = 7;
    break;
   }
   $151 = HEAP32[$1>>2]|0;
   $152 = HEAP32[$pBt>>2]|0;
   $153 = HEAP32[$2>>2]|0;
   $154 = HEAP32[$5>>2]|0;
   $155 = HEAP32[$6>>2]|0;
   $156 = (_sqlite3PagerOpen($151,$152,$153,76,$154,$155,14)|0);
   HEAP32[$rc>>2] = $156;
   $157 = HEAP32[$rc>>2]|0;
   $158 = ($157|0)==(0);
   if ($158) {
    $159 = HEAP32[$pBt>>2]|0;
    $160 = HEAP32[$159>>2]|0;
    $161 = HEAP32[$3>>2]|0;
    $162 = (($161) + 40|0);
    $163 = $162;
    $164 = $163;
    $165 = HEAP32[$164>>2]|0;
    $166 = (($163) + 4)|0;
    $167 = $166;
    $168 = HEAP32[$167>>2]|0;
    _sqlite3PagerSetMmapLimit($160,$165,$168);
    $169 = HEAP32[$pBt>>2]|0;
    $170 = HEAP32[$169>>2]|0;
    $171 = (_sqlite3PagerReadFileheader($170,100,$zDbHeader)|0);
    HEAP32[$rc>>2] = $171;
   }
   $172 = HEAP32[$rc>>2]|0;
   $173 = ($172|0)!=(0);
   if (!($173)) {
    $174 = HEAP32[$5>>2]|0;
    $175 = $174&255;
    $176 = HEAP32[$pBt>>2]|0;
    $177 = (($176) + 16|0);
    HEAP8[$177>>0] = $175;
    $178 = HEAP32[$3>>2]|0;
    $179 = HEAP32[$pBt>>2]|0;
    $180 = (($179) + 4|0);
    HEAP32[$180>>2] = $178;
    $181 = HEAP32[$pBt>>2]|0;
    $182 = HEAP32[$181>>2]|0;
    $183 = HEAP32[$pBt>>2]|0;
    _sqlite3PagerSetBusyhandler($182,18,$183);
    $184 = HEAP32[$pBt>>2]|0;
    $185 = HEAP32[$p>>2]|0;
    $186 = (($185) + 4|0);
    HEAP32[$186>>2] = $184;
    $187 = HEAP32[$pBt>>2]|0;
    $188 = (($187) + 8|0);
    HEAP32[$188>>2] = 0;
    $189 = HEAP32[$pBt>>2]|0;
    $190 = (($189) + 12|0);
    HEAP32[$190>>2] = 0;
    $191 = HEAP32[$pBt>>2]|0;
    $192 = HEAP32[$191>>2]|0;
    $193 = (_sqlite3PagerIsreadonly($192)|0);
    $194 = ($193<<24>>24)!=(0);
    if ($194) {
     $195 = HEAP32[$pBt>>2]|0;
     $196 = (($195) + 22|0);
     $197 = HEAP16[$196>>1]|0;
     $198 = $197&65535;
     $199 = $198 | 1;
     $200 = $199&65535;
     HEAP16[$196>>1] = $200;
    }
    $201 = (($zDbHeader) + 16|0);
    $202 = HEAP8[$201>>0]|0;
    $203 = $202&255;
    $204 = $203 << 8;
    $205 = (($zDbHeader) + 17|0);
    $206 = HEAP8[$205>>0]|0;
    $207 = $206&255;
    $208 = $207 << 16;
    $209 = $204 | $208;
    $210 = HEAP32[$pBt>>2]|0;
    $211 = (($210) + 32|0);
    HEAP32[$211>>2] = $209;
    $212 = HEAP32[$pBt>>2]|0;
    $213 = (($212) + 32|0);
    $214 = HEAP32[$213>>2]|0;
    $215 = ($214>>>0)<(512);
    if ($215) {
     label = 49;
    } else {
     $216 = HEAP32[$pBt>>2]|0;
     $217 = (($216) + 32|0);
     $218 = HEAP32[$217>>2]|0;
     $219 = ($218>>>0)>(65536);
     if ($219) {
      label = 49;
     } else {
      $220 = HEAP32[$pBt>>2]|0;
      $221 = (($220) + 32|0);
      $222 = HEAP32[$221>>2]|0;
      $223 = (($222) - 1)|0;
      $224 = HEAP32[$pBt>>2]|0;
      $225 = (($224) + 32|0);
      $226 = HEAP32[$225>>2]|0;
      $227 = $223 & $226;
      $228 = ($227|0)!=(0);
      if ($228) {
       label = 49;
      } else {
       $239 = (($zDbHeader) + 20|0);
       $240 = HEAP8[$239>>0]|0;
       HEAP8[$nReserve>>0] = $240;
       $241 = HEAP32[$pBt>>2]|0;
       $242 = (($241) + 22|0);
       $243 = HEAP16[$242>>1]|0;
       $244 = $243&65535;
       $245 = $244 | 2;
       $246 = $245&65535;
       HEAP16[$242>>1] = $246;
       $247 = (($zDbHeader) + 52|0);
       $248 = (_sqlite3Get4byte($247)|0);
       $249 = ($248|0)!=(0);
       $250 = $249 ? 1 : 0;
       $251 = $250&255;
       $252 = HEAP32[$pBt>>2]|0;
       $253 = (($252) + 17|0);
       HEAP8[$253>>0] = $251;
       $254 = (($zDbHeader) + 64|0);
       $255 = (_sqlite3Get4byte($254)|0);
       $256 = ($255|0)!=(0);
       $257 = $256 ? 1 : 0;
       $258 = $257&255;
       $259 = HEAP32[$pBt>>2]|0;
       $260 = (($259) + 18|0);
       HEAP8[$260>>0] = $258;
      }
     }
    }
    if ((label|0) == 49) {
     $229 = HEAP32[$pBt>>2]|0;
     $230 = (($229) + 32|0);
     HEAP32[$230>>2] = 0;
     $231 = HEAP32[$2>>2]|0;
     $232 = ($231|0)==(0|0);
     $233 = HEAP32[$isMemdb>>2]|0;
     $234 = ($233|0)!=(0);
     $or$cond3 = $232 | $234;
     if (!($or$cond3)) {
      $235 = HEAP32[$pBt>>2]|0;
      $236 = (($235) + 17|0);
      HEAP8[$236>>0] = 0;
      $237 = HEAP32[$pBt>>2]|0;
      $238 = (($237) + 18|0);
      HEAP8[$238>>0] = 0;
     }
     HEAP8[$nReserve>>0] = 0;
    }
    $261 = HEAP32[$pBt>>2]|0;
    $262 = HEAP32[$261>>2]|0;
    $263 = HEAP32[$pBt>>2]|0;
    $264 = (($263) + 32|0);
    $265 = HEAP8[$nReserve>>0]|0;
    $266 = $265&255;
    $267 = (_sqlite3PagerSetPagesize($262,$264,$266)|0);
    HEAP32[$rc>>2] = $267;
    $268 = HEAP32[$rc>>2]|0;
    $269 = ($268|0)!=(0);
    if (!($269)) {
     $270 = HEAP32[$pBt>>2]|0;
     $271 = (($270) + 32|0);
     $272 = HEAP32[$271>>2]|0;
     $273 = HEAP8[$nReserve>>0]|0;
     $274 = $273&255;
     $275 = (($272) - ($274))|0;
     $276 = HEAP32[$pBt>>2]|0;
     $277 = (($276) + 36|0);
     HEAP32[$277>>2] = $275;
     $278 = HEAP32[$p>>2]|0;
     $279 = (($278) + 9|0);
     $280 = HEAP8[$279>>0]|0;
     $281 = ($280<<24>>24)!=(0);
     if ($281) {
      $282 = HEAP32[$pBt>>2]|0;
      $283 = (($282) + 64|0);
      HEAP32[$283>>2] = 1;
      $284 = HEAP32[4232>>2]|0;
      $285 = HEAP32[$pBt>>2]|0;
      $286 = (($285) + 68|0);
      HEAP32[$286>>2] = $284;
      $287 = HEAP32[$pBt>>2]|0;
      HEAP32[4232>>2] = $287;
      label = 56;
     } else {
      label = 56;
     }
    }
   }
  } else {
   label = 56;
  }
 } while(0);
 if ((label|0) == 56) {
  $288 = HEAP32[$p>>2]|0;
  $289 = (($288) + 9|0);
  $290 = HEAP8[$289>>0]|0;
  $291 = ($290<<24>>24)!=(0);
  L85: do {
   if ($291) {
    HEAP32[$i>>2] = 0;
    while(1) {
     $292 = HEAP32[$i>>2]|0;
     $293 = HEAP32[$3>>2]|0;
     $294 = (($293) + 20|0);
     $295 = HEAP32[$294>>2]|0;
     $296 = ($292|0)<($295|0);
     if (!($296)) {
      break L85;
     }
     $297 = HEAP32[$i>>2]|0;
     $298 = HEAP32[$3>>2]|0;
     $299 = (($298) + 16|0);
     $300 = HEAP32[$299>>2]|0;
     $301 = (($300) + ($297<<4)|0);
     $302 = (($301) + 4|0);
     $303 = HEAP32[$302>>2]|0;
     HEAP32[$pSib>>2] = $303;
     $304 = ($303|0)!=(0|0);
     if ($304) {
      $305 = HEAP32[$pSib>>2]|0;
      $306 = (($305) + 9|0);
      $307 = HEAP8[$306>>0]|0;
      $308 = $307&255;
      $309 = ($308|0)!=(0);
      if ($309) {
       break;
      }
     }
     $365 = HEAP32[$i>>2]|0;
     $366 = (($365) + 1)|0;
     HEAP32[$i>>2] = $366;
    }
    while(1) {
     $310 = HEAP32[$pSib>>2]|0;
     $311 = (($310) + 24|0);
     $312 = HEAP32[$311>>2]|0;
     $313 = ($312|0)!=(0|0);
     if (!($313)) {
      break;
     }
     $314 = HEAP32[$pSib>>2]|0;
     $315 = (($314) + 24|0);
     $316 = HEAP32[$315>>2]|0;
     HEAP32[$pSib>>2] = $316;
    }
    $317 = HEAP32[$p>>2]|0;
    $318 = (($317) + 4|0);
    $319 = HEAP32[$318>>2]|0;
    $320 = HEAP32[$pSib>>2]|0;
    $321 = (($320) + 4|0);
    $322 = HEAP32[$321>>2]|0;
    $323 = ($319>>>0)<($322>>>0);
    if ($323) {
     $324 = HEAP32[$pSib>>2]|0;
     $325 = HEAP32[$p>>2]|0;
     $326 = (($325) + 20|0);
     HEAP32[$326>>2] = $324;
     $327 = HEAP32[$p>>2]|0;
     $328 = (($327) + 24|0);
     HEAP32[$328>>2] = 0;
     $329 = HEAP32[$p>>2]|0;
     $330 = HEAP32[$pSib>>2]|0;
     $331 = (($330) + 24|0);
     HEAP32[$331>>2] = $329;
     break;
    }
    while(1) {
     $332 = HEAP32[$pSib>>2]|0;
     $333 = (($332) + 20|0);
     $334 = HEAP32[$333>>2]|0;
     $335 = ($334|0)!=(0|0);
     if ($335) {
      $336 = HEAP32[$pSib>>2]|0;
      $337 = (($336) + 20|0);
      $338 = HEAP32[$337>>2]|0;
      $339 = (($338) + 4|0);
      $340 = HEAP32[$339>>2]|0;
      $341 = HEAP32[$p>>2]|0;
      $342 = (($341) + 4|0);
      $343 = HEAP32[$342>>2]|0;
      $344 = ($340>>>0)<($343>>>0);
      $390 = $344;
     } else {
      $390 = 0;
     }
     $345 = HEAP32[$pSib>>2]|0;
     $346 = (($345) + 20|0);
     $347 = HEAP32[$346>>2]|0;
     if (!($390)) {
      break;
     }
     HEAP32[$pSib>>2] = $347;
    }
    $348 = HEAP32[$p>>2]|0;
    $349 = (($348) + 20|0);
    HEAP32[$349>>2] = $347;
    $350 = HEAP32[$pSib>>2]|0;
    $351 = HEAP32[$p>>2]|0;
    $352 = (($351) + 24|0);
    HEAP32[$352>>2] = $350;
    $353 = HEAP32[$p>>2]|0;
    $354 = (($353) + 20|0);
    $355 = HEAP32[$354>>2]|0;
    $356 = ($355|0)!=(0|0);
    if ($356) {
     $357 = HEAP32[$p>>2]|0;
     $358 = HEAP32[$p>>2]|0;
     $359 = (($358) + 20|0);
     $360 = HEAP32[$359>>2]|0;
     $361 = (($360) + 24|0);
     HEAP32[$361>>2] = $357;
    }
    $362 = HEAP32[$p>>2]|0;
    $363 = HEAP32[$pSib>>2]|0;
    $364 = (($363) + 20|0);
    HEAP32[$364>>2] = $362;
   }
  } while(0);
  $367 = HEAP32[$p>>2]|0;
  $368 = HEAP32[$4>>2]|0;
  HEAP32[$368>>2] = $367;
 }
 $369 = HEAP32[$rc>>2]|0;
 $370 = ($369|0)!=(0);
 if ($370) {
  $371 = HEAP32[$pBt>>2]|0;
  $372 = ($371|0)!=(0|0);
  if ($372) {
   $373 = HEAP32[$pBt>>2]|0;
   $374 = HEAP32[$373>>2]|0;
   $375 = ($374|0)!=(0|0);
   if ($375) {
    $376 = HEAP32[$pBt>>2]|0;
    $377 = HEAP32[$376>>2]|0;
    (_sqlite3PagerClose($377)|0);
   }
  }
  $378 = HEAP32[$pBt>>2]|0;
  _sqlite3_free($378);
  $379 = HEAP32[$p>>2]|0;
  _sqlite3_free($379);
  $380 = HEAP32[$4>>2]|0;
  HEAP32[$380>>2] = 0;
 } else {
  $381 = HEAP32[$p>>2]|0;
  $382 = (_sqlite3BtreeSchema($381,0,0)|0);
  $383 = ($382|0)==(0|0);
  if ($383) {
   $384 = HEAP32[$p>>2]|0;
   $385 = (($384) + 4|0);
   $386 = HEAP32[$385>>2]|0;
   $387 = HEAP32[$386>>2]|0;
   _sqlite3PagerSetCachesize($387,2000);
  }
 }
 $388 = HEAP32[$rc>>2]|0;
 HEAP32[$0>>2] = $388;
 $389 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($389|0);
}
function _sqlite3SchemaGet($db,$pBt) {
 $db = $db|0;
 $pBt = $pBt|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $p = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $1 = sp + 4|0;
 $p = sp;
 HEAP32[$0>>2] = $db;
 HEAP32[$1>>2] = $pBt;
 $2 = HEAP32[$1>>2]|0;
 $3 = ($2|0)!=(0|0);
 if ($3) {
  $4 = HEAP32[$1>>2]|0;
  $5 = (_sqlite3BtreeSchema($4,84,15)|0);
  HEAP32[$p>>2] = $5;
 } else {
  $6 = (_sqlite3DbMallocZero(0,84,0)|0);
  HEAP32[$p>>2] = $6;
 }
 $7 = HEAP32[$p>>2]|0;
 $8 = ($7|0)!=(0|0);
 if (!($8)) {
  $9 = HEAP32[$0>>2]|0;
  $10 = (($9) + 64|0);
  HEAP8[$10>>0] = 1;
  $26 = HEAP32[$p>>2]|0;
  STACKTOP = sp;return ($26|0);
 }
 $11 = HEAP32[$p>>2]|0;
 $12 = (($11) + 76|0);
 $13 = HEAP8[$12>>0]|0;
 $14 = $13&255;
 $15 = (0)==($14|0);
 if (!($15)) {
  $26 = HEAP32[$p>>2]|0;
  STACKTOP = sp;return ($26|0);
 }
 $16 = HEAP32[$p>>2]|0;
 $17 = (($16) + 8|0);
 _sqlite3HashInit($17);
 $18 = HEAP32[$p>>2]|0;
 $19 = (($18) + 24|0);
 _sqlite3HashInit($19);
 $20 = HEAP32[$p>>2]|0;
 $21 = (($20) + 40|0);
 _sqlite3HashInit($21);
 $22 = HEAP32[$p>>2]|0;
 $23 = (($22) + 56|0);
 _sqlite3HashInit($23);
 $24 = HEAP32[$p>>2]|0;
 $25 = (($24) + 77|0);
 HEAP8[$25>>0] = 1;
 $26 = HEAP32[$p>>2]|0;
 STACKTOP = sp;return ($26|0);
}
function _sqlite3RegisterBuiltinFunctions($db) {
 $db = $db|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $rc = sp;
 HEAP32[$0>>2] = $db;
 $1 = HEAP32[$0>>2]|0;
 $2 = (_sqlite3_overload_function($1,4208,2)|0);
 HEAP32[$rc>>2] = $2;
 $3 = HEAP32[$rc>>2]|0;
 $4 = ($3|0)==(7);
 if (!($4)) {
  STACKTOP = sp;return;
 }
 $5 = HEAP32[$0>>2]|0;
 $6 = (($5) + 64|0);
 HEAP8[$6>>0] = 1;
 STACKTOP = sp;return;
}
function _sqlite3AutoLoadExtensions($db) {
 $db = $db|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $3 = 0, $4 = 0, $5 = 0;
 var $6 = 0, $7 = 0, $8 = 0, $9 = 0, $go = 0, $i = 0, $rc = 0, $vararg_buffer = 0, $xInit = 0, $zErrmsg = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $vararg_buffer = sp;
 $0 = sp + 24|0;
 $i = sp + 20|0;
 $go = sp + 16|0;
 $rc = sp + 12|0;
 $xInit = sp + 8|0;
 $zErrmsg = sp + 4|0;
 HEAP32[$0>>2] = $db;
 HEAP32[$go>>2] = 1;
 $1 = HEAP32[1480>>2]|0;
 $2 = ($1|0)==(0);
 if ($2) {
  STACKTOP = sp;return;
 }
 HEAP32[$i>>2] = 0;
 while(1) {
  $3 = HEAP32[$go>>2]|0;
  $4 = ($3|0)!=(0);
  if (!($4)) {
   break;
  }
  $5 = HEAP32[$i>>2]|0;
  $6 = HEAP32[1480>>2]|0;
  $7 = ($5|0)>=($6|0);
  if ($7) {
   HEAP32[$xInit>>2] = 0;
   HEAP32[$go>>2] = 0;
  } else {
   $8 = HEAP32[$i>>2]|0;
   $9 = HEAP32[((1480 + 4|0))>>2]|0;
   $10 = (($9) + ($8<<2)|0);
   $11 = HEAP32[$10>>2]|0;
   HEAP32[$xInit>>2] = $11;
  }
  HEAP32[$zErrmsg>>2] = 0;
  $12 = HEAP32[$xInit>>2]|0;
  $13 = ($12|0)!=(0|0);
  if ($13) {
   $14 = HEAP32[$xInit>>2]|0;
   $15 = HEAP32[$0>>2]|0;
   $16 = (FUNCTION_TABLE_iiii[$14 & 31]($15,$zErrmsg,3352)|0);
   HEAP32[$rc>>2] = $16;
   $17 = ($16|0)!=(0);
   if ($17) {
    $18 = HEAP32[$0>>2]|0;
    $19 = HEAP32[$rc>>2]|0;
    $20 = HEAP32[$zErrmsg>>2]|0;
    HEAP32[$vararg_buffer>>2] = $20;
    _sqlite3ErrorWithMsg($18,$19,4168,$vararg_buffer);
    HEAP32[$go>>2] = 0;
   }
  }
  $21 = HEAP32[$zErrmsg>>2]|0;
  _sqlite3_free($21);
  $22 = HEAP32[$i>>2]|0;
  $23 = (($22) + 1)|0;
  HEAP32[$i>>2] = $23;
 }
 STACKTOP = sp;return;
}
function _sqlite3BtreeSchema($p,$nBytes,$xFree) {
 $p = $p|0;
 $nBytes = $nBytes|0;
 $xFree = $xFree|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $3 = 0, $4 = 0;
 var $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $or$cond = 0, $pBt = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $1 = sp + 8|0;
 $2 = sp + 4|0;
 $pBt = sp;
 HEAP32[$0>>2] = $p;
 HEAP32[$1>>2] = $nBytes;
 HEAP32[$2>>2] = $xFree;
 $3 = HEAP32[$0>>2]|0;
 $4 = (($3) + 4|0);
 $5 = HEAP32[$4>>2]|0;
 HEAP32[$pBt>>2] = $5;
 $6 = HEAP32[$0>>2]|0;
 _sqlite3BtreeEnter($6);
 $7 = HEAP32[$pBt>>2]|0;
 $8 = (($7) + 48|0);
 $9 = HEAP32[$8>>2]|0;
 $10 = ($9|0)==(0|0);
 $11 = HEAP32[$1>>2]|0;
 $12 = ($11|0)!=(0);
 $or$cond = $10 & $12;
 if (!($or$cond)) {
  $22 = HEAP32[$pBt>>2]|0;
  $23 = (($22) + 48|0);
  $24 = HEAP32[$23>>2]|0;
  STACKTOP = sp;return ($24|0);
 }
 $13 = HEAP32[$1>>2]|0;
 $14 = ($13|0)<(0);
 $15 = $14 << 31 >> 31;
 $16 = (_sqlite3DbMallocZero(0,$13,$15)|0);
 $17 = HEAP32[$pBt>>2]|0;
 $18 = (($17) + 48|0);
 HEAP32[$18>>2] = $16;
 $19 = HEAP32[$2>>2]|0;
 $20 = HEAP32[$pBt>>2]|0;
 $21 = (($20) + 52|0);
 HEAP32[$21>>2] = $19;
 $22 = HEAP32[$pBt>>2]|0;
 $23 = (($22) + 48|0);
 $24 = HEAP32[$23>>2]|0;
 STACKTOP = sp;return ($24|0);
}
function _sqlite3SchemaClear($p) {
 $p = $p|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $pElem = 0, $pSchema = 0, $pTab = 0, $temp1 = 0, $temp2 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 64|0;
 $0 = sp + 48|0;
 $temp1 = sp + 32|0;
 $temp2 = sp + 16|0;
 $pElem = sp + 8|0;
 $pSchema = sp + 4|0;
 $pTab = sp;
 HEAP32[$0>>2] = $p;
 $1 = HEAP32[$0>>2]|0;
 HEAP32[$pSchema>>2] = $1;
 $2 = HEAP32[$pSchema>>2]|0;
 $3 = (($2) + 8|0);
 ;HEAP32[$temp1+0>>2]=HEAP32[$3+0>>2]|0;HEAP32[$temp1+4>>2]=HEAP32[$3+4>>2]|0;HEAP32[$temp1+8>>2]=HEAP32[$3+8>>2]|0;HEAP32[$temp1+12>>2]=HEAP32[$3+12>>2]|0;
 $4 = HEAP32[$pSchema>>2]|0;
 $5 = (($4) + 40|0);
 ;HEAP32[$temp2+0>>2]=HEAP32[$5+0>>2]|0;HEAP32[$temp2+4>>2]=HEAP32[$5+4>>2]|0;HEAP32[$temp2+8>>2]=HEAP32[$5+8>>2]|0;HEAP32[$temp2+12>>2]=HEAP32[$5+12>>2]|0;
 $6 = HEAP32[$pSchema>>2]|0;
 $7 = (($6) + 40|0);
 _sqlite3HashInit($7);
 $8 = HEAP32[$pSchema>>2]|0;
 $9 = (($8) + 24|0);
 _sqlite3HashClear($9);
 $10 = (($temp2) + 8|0);
 $11 = HEAP32[$10>>2]|0;
 HEAP32[$pElem>>2] = $11;
 while(1) {
  $12 = HEAP32[$pElem>>2]|0;
  $13 = ($12|0)!=(0|0);
  if (!($13)) {
   break;
  }
  $14 = HEAP32[$pElem>>2]|0;
  $15 = (($14) + 8|0);
  $16 = HEAP32[$15>>2]|0;
  _sqlite3DeleteTrigger(0,$16);
  $17 = HEAP32[$pElem>>2]|0;
  $18 = HEAP32[$17>>2]|0;
  HEAP32[$pElem>>2] = $18;
 }
 _sqlite3HashClear($temp2);
 $19 = HEAP32[$pSchema>>2]|0;
 $20 = (($19) + 8|0);
 _sqlite3HashInit($20);
 $21 = (($temp1) + 8|0);
 $22 = HEAP32[$21>>2]|0;
 HEAP32[$pElem>>2] = $22;
 while(1) {
  $23 = HEAP32[$pElem>>2]|0;
  $24 = ($23|0)!=(0|0);
  if (!($24)) {
   break;
  }
  $25 = HEAP32[$pElem>>2]|0;
  $26 = (($25) + 8|0);
  $27 = HEAP32[$26>>2]|0;
  HEAP32[$pTab>>2] = $27;
  $28 = HEAP32[$pTab>>2]|0;
  _sqlite3DeleteTable(0,$28);
  $29 = HEAP32[$pElem>>2]|0;
  $30 = HEAP32[$29>>2]|0;
  HEAP32[$pElem>>2] = $30;
 }
 _sqlite3HashClear($temp1);
 $31 = HEAP32[$pSchema>>2]|0;
 $32 = (($31) + 56|0);
 _sqlite3HashClear($32);
 $33 = HEAP32[$pSchema>>2]|0;
 $34 = (($33) + 72|0);
 HEAP32[$34>>2] = 0;
 $35 = HEAP32[$pSchema>>2]|0;
 $36 = (($35) + 78|0);
 $37 = HEAP16[$36>>1]|0;
 $38 = $37&65535;
 $39 = $38 & 1;
 $40 = ($39|0)!=(0);
 if (!($40)) {
  STACKTOP = sp;return;
 }
 $41 = HEAP32[$pSchema>>2]|0;
 $42 = (($41) + 4|0);
 $43 = HEAP32[$42>>2]|0;
 $44 = (($43) + 1)|0;
 HEAP32[$42>>2] = $44;
 $45 = HEAP32[$pSchema>>2]|0;
 $46 = (($45) + 78|0);
 $47 = HEAP16[$46>>1]|0;
 $48 = $47&65535;
 $49 = $48 & -2;
 $50 = $49&65535;
 HEAP16[$46>>1] = $50;
 STACKTOP = sp;return;
}
function _sqlite3TempInMemory($db) {
 $db = $db|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 HEAP32[$0>>2] = $db;
 $1 = HEAP32[$0>>2]|0;
 $2 = (($1) + 63|0);
 $3 = HEAP8[$2>>0]|0;
 $4 = $3&255;
 $5 = ($4|0)==(2);
 $6 = $5&1;
 STACKTOP = sp;return ($6|0);
}
function _sqlite3OsFullPathname($pVfs,$zPath,$nPathOut,$zPathOut) {
 $pVfs = $pVfs|0;
 $zPath = $zPath|0;
 $nPathOut = $nPathOut|0;
 $zPathOut = $zPathOut|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $1 = sp + 8|0;
 $2 = sp + 4|0;
 $3 = sp;
 HEAP32[$0>>2] = $pVfs;
 HEAP32[$1>>2] = $zPath;
 HEAP32[$2>>2] = $nPathOut;
 HEAP32[$3>>2] = $zPathOut;
 $4 = HEAP32[$3>>2]|0;
 HEAP8[$4>>0] = 0;
 $5 = HEAP32[$0>>2]|0;
 $6 = (($5) + 36|0);
 $7 = HEAP32[$6>>2]|0;
 $8 = HEAP32[$0>>2]|0;
 $9 = HEAP32[$1>>2]|0;
 $10 = HEAP32[$2>>2]|0;
 $11 = HEAP32[$3>>2]|0;
 $12 = (FUNCTION_TABLE_iiiii[$7 & 7]($8,$9,$10,$11)|0);
 STACKTOP = sp;return ($12|0);
}
function _sqlite3PagerVfs($pPager) {
 $pPager = $pPager|0;
 var $0 = 0, $1 = 0, $2 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 HEAP32[$0>>2] = $pPager;
 $1 = HEAP32[$0>>2]|0;
 $2 = HEAP32[$1>>2]|0;
 STACKTOP = sp;return ($2|0);
}
function _sqlite3PagerOpen($pVfs,$ppPager,$zFilename,$nExtra,$flags,$vfsFlags,$xReinit) {
 $pVfs = $pVfs|0;
 $ppPager = $ppPager|0;
 $zFilename = $zFilename|0;
 $nExtra = $nExtra|0;
 $flags = $flags|0;
 $vfsFlags = $vfsFlags|0;
 $xReinit = $xReinit|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0;
 var $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0, $188 = 0;
 var $189 = 0, $19 = 0, $190 = 0, $191 = 0, $192 = 0, $193 = 0, $194 = 0, $195 = 0, $196 = 0, $197 = 0, $198 = 0, $199 = 0, $2 = 0, $20 = 0, $200 = 0, $201 = 0, $202 = 0, $203 = 0, $204 = 0, $205 = 0;
 var $206 = 0, $207 = 0, $208 = 0, $209 = 0, $21 = 0, $210 = 0, $211 = 0, $212 = 0, $213 = 0, $214 = 0, $215 = 0, $216 = 0, $217 = 0, $218 = 0, $219 = 0, $22 = 0, $220 = 0, $221 = 0, $222 = 0, $223 = 0;
 var $224 = 0, $225 = 0, $226 = 0, $227 = 0, $228 = 0, $229 = 0, $23 = 0, $230 = 0, $231 = 0, $232 = 0, $233 = 0, $234 = 0, $235 = 0, $236 = 0, $237 = 0, $238 = 0, $239 = 0, $24 = 0, $240 = 0, $241 = 0;
 var $242 = 0, $243 = 0, $244 = 0, $245 = 0, $246 = 0, $247 = 0, $248 = 0, $249 = 0, $25 = 0, $250 = 0, $251 = 0, $252 = 0, $253 = 0, $254 = 0, $255 = 0, $256 = 0, $257 = 0, $258 = 0, $259 = 0, $26 = 0;
 var $260 = 0, $261 = 0, $262 = 0, $263 = 0, $264 = 0, $265 = 0, $266 = 0, $267 = 0, $268 = 0, $269 = 0, $27 = 0, $270 = 0, $271 = 0, $272 = 0, $273 = 0, $274 = 0, $275 = 0, $276 = 0, $277 = 0, $278 = 0;
 var $279 = 0, $28 = 0, $280 = 0, $281 = 0, $282 = 0, $283 = 0, $284 = 0, $285 = 0, $286 = 0, $287 = 0, $288 = 0, $289 = 0, $29 = 0, $290 = 0, $291 = 0, $292 = 0, $293 = 0, $294 = 0, $295 = 0, $296 = 0;
 var $297 = 0, $298 = 0, $299 = 0, $3 = 0, $30 = 0, $300 = 0, $301 = 0, $302 = 0, $303 = 0, $304 = 0, $305 = 0, $306 = 0, $307 = 0, $308 = 0, $309 = 0, $31 = 0, $310 = 0, $311 = 0, $312 = 0, $313 = 0;
 var $314 = 0, $315 = 0, $316 = 0, $317 = 0, $318 = 0, $319 = 0, $32 = 0, $320 = 0, $321 = 0, $322 = 0, $323 = 0, $324 = 0, $325 = 0, $326 = 0, $327 = 0, $328 = 0, $329 = 0, $33 = 0, $330 = 0, $331 = 0;
 var $332 = 0, $333 = 0, $334 = 0, $335 = 0, $336 = 0, $337 = 0, $338 = 0, $339 = 0, $34 = 0, $340 = 0, $341 = 0, $342 = 0, $343 = 0, $344 = 0, $345 = 0, $346 = 0, $347 = 0, $348 = 0, $349 = 0, $35 = 0;
 var $350 = 0, $351 = 0, $352 = 0, $353 = 0, $354 = 0, $355 = 0, $356 = 0, $357 = 0, $358 = 0, $359 = 0, $36 = 0, $360 = 0, $361 = 0, $362 = 0, $363 = 0, $364 = 0, $365 = 0, $366 = 0, $367 = 0, $368 = 0;
 var $369 = 0, $37 = 0, $370 = 0, $371 = 0, $372 = 0, $373 = 0, $374 = 0, $375 = 0, $376 = 0, $377 = 0, $378 = 0, $379 = 0, $38 = 0, $380 = 0, $381 = 0, $382 = 0, $383 = 0, $384 = 0, $385 = 0, $39 = 0;
 var $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0;
 var $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0;
 var $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0;
 var $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $fout = 0, $iDc = 0, $journalFileSize = 0, $memDb = 0, $nPathname = 0, $nUri = 0, $pPager = 0, $pPtr = 0, $pcacheSize = 0, $rc = 0, $readOnly = 0, $szPageDflt = 0, $tempFile = 0, $useJournal = 0;
 var $z = 0, $zPathname = 0, $zUri = 0, dest = 0, label = 0, sp = 0, src = 0, stop = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 112|0;
 $0 = sp + 96|0;
 $1 = sp + 92|0;
 $2 = sp + 88|0;
 $3 = sp + 84|0;
 $4 = sp + 80|0;
 $5 = sp + 76|0;
 $6 = sp + 72|0;
 $7 = sp + 68|0;
 $pPtr = sp + 64|0;
 $pPager = sp + 60|0;
 $rc = sp + 56|0;
 $tempFile = sp + 52|0;
 $memDb = sp + 48|0;
 $readOnly = sp + 44|0;
 $journalFileSize = sp + 40|0;
 $zPathname = sp + 36|0;
 $nPathname = sp + 32|0;
 $useJournal = sp + 28|0;
 $pcacheSize = sp + 24|0;
 $szPageDflt = sp + 20|0;
 $zUri = sp + 16|0;
 $nUri = sp + 12|0;
 $z = sp + 8|0;
 $fout = sp + 4|0;
 $iDc = sp;
 HEAP32[$1>>2] = $pVfs;
 HEAP32[$2>>2] = $ppPager;
 HEAP32[$3>>2] = $zFilename;
 HEAP32[$4>>2] = $nExtra;
 HEAP32[$5>>2] = $flags;
 HEAP32[$6>>2] = $vfsFlags;
 HEAP32[$7>>2] = $xReinit;
 HEAP32[$pPager>>2] = 0;
 HEAP32[$rc>>2] = 0;
 HEAP32[$tempFile>>2] = 0;
 HEAP32[$memDb>>2] = 0;
 HEAP32[$readOnly>>2] = 0;
 HEAP32[$zPathname>>2] = 0;
 HEAP32[$nPathname>>2] = 0;
 $8 = HEAP32[$5>>2]|0;
 $9 = $8 & 1;
 $10 = ($9|0)==(0);
 $11 = $10&1;
 HEAP32[$useJournal>>2] = $11;
 $12 = (_sqlite3PcacheSize()|0);
 HEAP32[$pcacheSize>>2] = $12;
 HEAP32[$szPageDflt>>2] = 1024;
 HEAP32[$zUri>>2] = 0;
 HEAP32[$nUri>>2] = 0;
 $13 = HEAP32[$1>>2]|0;
 $14 = (($13) + 4|0);
 $15 = HEAP32[$14>>2]|0;
 $16 = (_sqlite3MemJournalSize()|0);
 $17 = ($15|0)>($16|0);
 if ($17) {
  $18 = HEAP32[$1>>2]|0;
  $19 = (($18) + 4|0);
  $20 = HEAP32[$19>>2]|0;
  $21 = (($20) + 7)|0;
  $22 = $21 & -8;
  HEAP32[$journalFileSize>>2] = $22;
 } else {
  $23 = (_sqlite3MemJournalSize()|0);
  $24 = (($23) + 7)|0;
  $25 = $24 & -8;
  HEAP32[$journalFileSize>>2] = $25;
 }
 $26 = HEAP32[$2>>2]|0;
 HEAP32[$26>>2] = 0;
 $27 = HEAP32[$5>>2]|0;
 $28 = $27 & 2;
 $29 = ($28|0)!=(0);
 do {
  if ($29) {
   HEAP32[$memDb>>2] = 1;
   $30 = HEAP32[$3>>2]|0;
   $31 = ($30|0)!=(0|0);
   if ($31) {
    $32 = HEAP32[$3>>2]|0;
    $33 = HEAP8[$32>>0]|0;
    $34 = $33 << 24 >> 24;
    $35 = ($34|0)!=(0);
    if ($35) {
     $36 = HEAP32[$3>>2]|0;
     $37 = (_sqlite3DbStrDup(0,$36)|0);
     HEAP32[$zPathname>>2] = $37;
     $38 = HEAP32[$zPathname>>2]|0;
     $39 = ($38|0)==(0|0);
     if (!($39)) {
      $40 = HEAP32[$zPathname>>2]|0;
      $41 = (_sqlite3Strlen30($40)|0);
      HEAP32[$nPathname>>2] = $41;
      HEAP32[$3>>2] = 0;
      break;
     }
     HEAP32[$0>>2] = 7;
     $385 = HEAP32[$0>>2]|0;
     STACKTOP = sp;return ($385|0);
    }
   }
  }
 } while(0);
 $42 = HEAP32[$3>>2]|0;
 $43 = ($42|0)!=(0|0);
 if ($43) {
  $44 = HEAP32[$3>>2]|0;
  $45 = HEAP8[$44>>0]|0;
  $46 = $45 << 24 >> 24;
  $47 = ($46|0)!=(0);
  if ($47) {
   $48 = HEAP32[$1>>2]|0;
   $49 = (($48) + 8|0);
   $50 = HEAP32[$49>>2]|0;
   $51 = (($50) + 1)|0;
   HEAP32[$nPathname>>2] = $51;
   $52 = HEAP32[$nPathname>>2]|0;
   $53 = $52<<1;
   $54 = ($53|0)<(0);
   $55 = $54 << 31 >> 31;
   $56 = (_sqlite3DbMallocRaw(0,$53,$55)|0);
   HEAP32[$zPathname>>2] = $56;
   $57 = HEAP32[$zPathname>>2]|0;
   $58 = ($57|0)==(0|0);
   if ($58) {
    HEAP32[$0>>2] = 7;
    $385 = HEAP32[$0>>2]|0;
    STACKTOP = sp;return ($385|0);
   }
   $59 = HEAP32[$zPathname>>2]|0;
   HEAP8[$59>>0] = 0;
   $60 = HEAP32[$1>>2]|0;
   $61 = HEAP32[$3>>2]|0;
   $62 = HEAP32[$nPathname>>2]|0;
   $63 = HEAP32[$zPathname>>2]|0;
   $64 = (_sqlite3OsFullPathname($60,$61,$62,$63)|0);
   HEAP32[$rc>>2] = $64;
   $65 = HEAP32[$zPathname>>2]|0;
   $66 = (_sqlite3Strlen30($65)|0);
   HEAP32[$nPathname>>2] = $66;
   $67 = HEAP32[$3>>2]|0;
   $68 = (_sqlite3Strlen30($67)|0);
   $69 = (($68) + 1)|0;
   $70 = HEAP32[$3>>2]|0;
   $71 = (($70) + ($69)|0);
   HEAP32[$zUri>>2] = $71;
   HEAP32[$z>>2] = $71;
   while(1) {
    $72 = HEAP32[$z>>2]|0;
    $73 = HEAP8[$72>>0]|0;
    $74 = ($73<<24>>24)!=(0);
    $75 = HEAP32[$z>>2]|0;
    if (!($74)) {
     break;
    }
    $76 = (_sqlite3Strlen30($75)|0);
    $77 = (($76) + 1)|0;
    $78 = HEAP32[$z>>2]|0;
    $79 = (($78) + ($77)|0);
    HEAP32[$z>>2] = $79;
    $80 = HEAP32[$z>>2]|0;
    $81 = (_sqlite3Strlen30($80)|0);
    $82 = (($81) + 1)|0;
    $83 = HEAP32[$z>>2]|0;
    $84 = (($83) + ($82)|0);
    HEAP32[$z>>2] = $84;
   }
   $85 = (($75) + 1|0);
   $86 = HEAP32[$zUri>>2]|0;
   $87 = $85;
   $88 = $86;
   $89 = (($87) - ($88))|0;
   HEAP32[$nUri>>2] = $89;
   $90 = HEAP32[$rc>>2]|0;
   $91 = ($90|0)==(0);
   if ($91) {
    $92 = HEAP32[$nPathname>>2]|0;
    $93 = (($92) + 8)|0;
    $94 = HEAP32[$1>>2]|0;
    $95 = (($94) + 8|0);
    $96 = HEAP32[$95>>2]|0;
    $97 = ($93|0)>($96|0);
    if ($97) {
     $98 = (_sqlite3CantopenError(45462)|0);
     HEAP32[$rc>>2] = $98;
    }
   }
   $99 = HEAP32[$rc>>2]|0;
   $100 = ($99|0)!=(0);
   if ($100) {
    $101 = HEAP32[$zPathname>>2]|0;
    _sqlite3DbFree(0,$101);
    $102 = HEAP32[$rc>>2]|0;
    HEAP32[$0>>2] = $102;
    $385 = HEAP32[$0>>2]|0;
    STACKTOP = sp;return ($385|0);
   }
  }
 }
 $103 = HEAP32[$pcacheSize>>2]|0;
 $104 = (($103) + 7)|0;
 $105 = $104 & -8;
 $106 = (216 + ($105))|0;
 $107 = HEAP32[$1>>2]|0;
 $108 = (($107) + 4|0);
 $109 = HEAP32[$108>>2]|0;
 $110 = (($109) + 7)|0;
 $111 = $110 & -8;
 $112 = (($106) + ($111))|0;
 $113 = HEAP32[$journalFileSize>>2]|0;
 $114 = $113<<1;
 $115 = (($112) + ($114))|0;
 $116 = HEAP32[$nPathname>>2]|0;
 $117 = (($115) + ($116))|0;
 $118 = (($117) + 1)|0;
 $119 = HEAP32[$nUri>>2]|0;
 $120 = (($118) + ($119))|0;
 $121 = HEAP32[$nPathname>>2]|0;
 $122 = (($120) + ($121))|0;
 $123 = (($122) + 8)|0;
 $124 = (($123) + 2)|0;
 $125 = HEAP32[$nPathname>>2]|0;
 $126 = (($124) + ($125))|0;
 $127 = (($126) + 4)|0;
 $128 = (($127) + 2)|0;
 $129 = (_sqlite3MallocZero($128,0)|0);
 HEAP32[$pPtr>>2] = $129;
 $130 = HEAP32[$pPtr>>2]|0;
 $131 = ($130|0)!=(0|0);
 if (!($131)) {
  $132 = HEAP32[$zPathname>>2]|0;
  _sqlite3DbFree(0,$132);
  HEAP32[$0>>2] = 7;
  $385 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($385|0);
 }
 $133 = HEAP32[$pPtr>>2]|0;
 HEAP32[$pPager>>2] = $133;
 $134 = HEAP32[$pPtr>>2]|0;
 $135 = (($134) + 216|0);
 HEAP32[$pPtr>>2] = $135;
 $136 = HEAP32[$pPager>>2]|0;
 $137 = (($136) + 204|0);
 HEAP32[$137>>2] = $135;
 $138 = HEAP32[$pcacheSize>>2]|0;
 $139 = (($138) + 7)|0;
 $140 = $139 & -8;
 $141 = HEAP32[$pPtr>>2]|0;
 $142 = (($141) + ($140)|0);
 HEAP32[$pPtr>>2] = $142;
 $143 = HEAP32[$pPager>>2]|0;
 $144 = (($143) + 60|0);
 HEAP32[$144>>2] = $142;
 $145 = HEAP32[$1>>2]|0;
 $146 = (($145) + 4|0);
 $147 = HEAP32[$146>>2]|0;
 $148 = (($147) + 7)|0;
 $149 = $148 & -8;
 $150 = HEAP32[$pPtr>>2]|0;
 $151 = (($150) + ($149)|0);
 HEAP32[$pPtr>>2] = $151;
 $152 = HEAP32[$pPager>>2]|0;
 $153 = (($152) + 68|0);
 HEAP32[$153>>2] = $151;
 $154 = HEAP32[$journalFileSize>>2]|0;
 $155 = HEAP32[$pPtr>>2]|0;
 $156 = (($155) + ($154)|0);
 HEAP32[$pPtr>>2] = $156;
 $157 = HEAP32[$pPager>>2]|0;
 $158 = (($157) + 64|0);
 HEAP32[$158>>2] = $156;
 $159 = HEAP32[$journalFileSize>>2]|0;
 $160 = HEAP32[$pPtr>>2]|0;
 $161 = (($160) + ($159)|0);
 HEAP32[$pPtr>>2] = $161;
 $162 = HEAP32[$pPager>>2]|0;
 $163 = (($162) + 168|0);
 HEAP32[$163>>2] = $161;
 $164 = HEAP32[$zPathname>>2]|0;
 $165 = ($164|0)!=(0|0);
 if ($165) {
  $166 = HEAP32[$nPathname>>2]|0;
  $167 = (($166) + 1)|0;
  $168 = HEAP32[$nUri>>2]|0;
  $169 = (($167) + ($168))|0;
  $170 = HEAP32[$pPtr>>2]|0;
  $171 = (($170) + ($169)|0);
  HEAP32[$pPtr>>2] = $171;
  $172 = HEAP32[$pPager>>2]|0;
  $173 = (($172) + 172|0);
  HEAP32[$173>>2] = $171;
  $174 = HEAP32[$pPager>>2]|0;
  $175 = (($174) + 168|0);
  $176 = HEAP32[$175>>2]|0;
  $177 = HEAP32[$zPathname>>2]|0;
  $178 = HEAP32[$nPathname>>2]|0;
  _memcpy(($176|0),($177|0),($178|0))|0;
  $179 = HEAP32[$nUri>>2]|0;
  $180 = ($179|0)!=(0);
  if ($180) {
   $181 = HEAP32[$nPathname>>2]|0;
   $182 = (($181) + 1)|0;
   $183 = HEAP32[$pPager>>2]|0;
   $184 = (($183) + 168|0);
   $185 = HEAP32[$184>>2]|0;
   $186 = (($185) + ($182)|0);
   $187 = HEAP32[$zUri>>2]|0;
   $188 = HEAP32[$nUri>>2]|0;
   _memcpy(($186|0),($187|0),($188|0))|0;
  }
  $189 = HEAP32[$pPager>>2]|0;
  $190 = (($189) + 172|0);
  $191 = HEAP32[$190>>2]|0;
  $192 = HEAP32[$zPathname>>2]|0;
  $193 = HEAP32[$nPathname>>2]|0;
  _memcpy(($191|0),($192|0),($193|0))|0;
  $194 = HEAP32[$nPathname>>2]|0;
  $195 = HEAP32[$pPager>>2]|0;
  $196 = (($195) + 172|0);
  $197 = HEAP32[$196>>2]|0;
  $198 = (($197) + ($194)|0);
  dest=$198+0|0; src=4552+0|0; stop=dest+10|0; do { HEAP8[dest>>0]=HEAP8[src>>0]|0; dest=dest+1|0; src=src+1|0; } while ((dest|0) < (stop|0));
  $199 = HEAP32[$nPathname>>2]|0;
  $200 = (($199) + 8)|0;
  $201 = (($200) + 1)|0;
  $202 = HEAP32[$pPager>>2]|0;
  $203 = (($202) + 172|0);
  $204 = HEAP32[$203>>2]|0;
  $205 = (($204) + ($201)|0);
  $206 = HEAP32[$pPager>>2]|0;
  $207 = (($206) + 212|0);
  HEAP32[$207>>2] = $205;
  $208 = HEAP32[$pPager>>2]|0;
  $209 = (($208) + 212|0);
  $210 = HEAP32[$209>>2]|0;
  $211 = HEAP32[$zPathname>>2]|0;
  $212 = HEAP32[$nPathname>>2]|0;
  _memcpy(($210|0),($211|0),($212|0))|0;
  $213 = HEAP32[$nPathname>>2]|0;
  $214 = HEAP32[$pPager>>2]|0;
  $215 = (($214) + 212|0);
  $216 = HEAP32[$215>>2]|0;
  $217 = (($216) + ($213)|0);
  ;HEAP8[$217+0>>0]=HEAP8[4568+0>>0]|0;HEAP8[$217+1>>0]=HEAP8[4568+1>>0]|0;HEAP8[$217+2>>0]=HEAP8[4568+2>>0]|0;HEAP8[$217+3>>0]=HEAP8[4568+3>>0]|0;HEAP8[$217+4>>0]=HEAP8[4568+4>>0]|0;
  $218 = HEAP32[$zPathname>>2]|0;
  _sqlite3DbFree(0,$218);
 }
 $219 = HEAP32[$1>>2]|0;
 $220 = HEAP32[$pPager>>2]|0;
 HEAP32[$220>>2] = $219;
 $221 = HEAP32[$6>>2]|0;
 $222 = HEAP32[$pPager>>2]|0;
 $223 = (($222) + 144|0);
 HEAP32[$223>>2] = $221;
 $224 = HEAP32[$3>>2]|0;
 $225 = ($224|0)!=(0|0);
 do {
  if ($225) {
   $226 = HEAP32[$3>>2]|0;
   $227 = HEAP8[$226>>0]|0;
   $228 = $227 << 24 >> 24;
   $229 = ($228|0)!=(0);
   if ($229) {
    HEAP32[$fout>>2] = 0;
    $230 = HEAP32[$1>>2]|0;
    $231 = HEAP32[$pPager>>2]|0;
    $232 = (($231) + 168|0);
    $233 = HEAP32[$232>>2]|0;
    $234 = HEAP32[$pPager>>2]|0;
    $235 = (($234) + 60|0);
    $236 = HEAP32[$235>>2]|0;
    $237 = HEAP32[$6>>2]|0;
    $238 = (_sqlite3OsOpen($230,$233,$236,$237,$fout)|0);
    HEAP32[$rc>>2] = $238;
    $239 = HEAP32[$fout>>2]|0;
    $240 = $239 & 1;
    HEAP32[$readOnly>>2] = $240;
    $241 = HEAP32[$rc>>2]|0;
    $242 = ($241|0)==(0);
    if ($242) {
     $243 = HEAP32[$pPager>>2]|0;
     $244 = (($243) + 60|0);
     $245 = HEAP32[$244>>2]|0;
     $246 = (_sqlite3OsDeviceCharacteristics($245)|0);
     HEAP32[$iDc>>2] = $246;
     $247 = HEAP32[$readOnly>>2]|0;
     $248 = ($247|0)!=(0);
     do {
      if (!($248)) {
       $249 = HEAP32[$pPager>>2]|0;
       _setSectorSize($249);
       $250 = HEAP32[$szPageDflt>>2]|0;
       $251 = HEAP32[$pPager>>2]|0;
       $252 = (($251) + 148|0);
       $253 = HEAP32[$252>>2]|0;
       $254 = ($250>>>0)<($253>>>0);
       if ($254) {
        $255 = HEAP32[$pPager>>2]|0;
        $256 = (($255) + 148|0);
        $257 = HEAP32[$256>>2]|0;
        $258 = ($257>>>0)>(8192);
        if ($258) {
         HEAP32[$szPageDflt>>2] = 8192;
         break;
        } else {
         $259 = HEAP32[$pPager>>2]|0;
         $260 = (($259) + 148|0);
         $261 = HEAP32[$260>>2]|0;
         HEAP32[$szPageDflt>>2] = $261;
         break;
        }
       }
      }
     } while(0);
     $262 = HEAP32[$3>>2]|0;
     $263 = (_sqlite3_uri_boolean($262,4576,0)|0);
     $264 = $263&255;
     $265 = HEAP32[$pPager>>2]|0;
     $266 = (($265) + 13|0);
     HEAP8[$266>>0] = $264;
     $267 = HEAP32[$iDc>>2]|0;
     $268 = $267 & 8192;
     $269 = ($268|0)!=(0);
     if (!($269)) {
      $270 = HEAP32[$3>>2]|0;
      $271 = (_sqlite3_uri_boolean($270,4584,0)|0);
      $272 = ($271|0)!=(0);
      if (!($272)) {
       break;
      }
     }
     $273 = HEAP32[$6>>2]|0;
     $274 = $273 | 1;
     HEAP32[$6>>2] = $274;
     label = 39;
    }
   } else {
    label = 39;
   }
  } else {
   label = 39;
  }
 } while(0);
 if ((label|0) == 39) {
  HEAP32[$tempFile>>2] = 1;
  $275 = HEAP32[$pPager>>2]|0;
  $276 = (($275) + 16|0);
  HEAP8[$276>>0] = 1;
  $277 = HEAP32[$pPager>>2]|0;
  $278 = (($277) + 17|0);
  HEAP8[$278>>0] = 4;
  $279 = HEAP32[$pPager>>2]|0;
  $280 = (($279) + 13|0);
  HEAP8[$280>>0] = 1;
  $281 = HEAP32[$6>>2]|0;
  $282 = $281 & 1;
  HEAP32[$readOnly>>2] = $282;
 }
 $283 = HEAP32[$rc>>2]|0;
 $284 = ($283|0)==(0);
 if ($284) {
  $285 = HEAP32[$pPager>>2]|0;
  $286 = (_sqlite3PagerSetPagesize($285,$szPageDflt,-1)|0);
  HEAP32[$rc>>2] = $286;
 }
 $287 = HEAP32[$rc>>2]|0;
 $288 = ($287|0)==(0);
 if ($288) {
  $289 = HEAP32[$4>>2]|0;
  $290 = (($289) + 7)|0;
  $291 = $290 & -8;
  HEAP32[$4>>2] = $291;
  $292 = HEAP32[$szPageDflt>>2]|0;
  $293 = HEAP32[$4>>2]|0;
  $294 = HEAP32[$memDb>>2]|0;
  $295 = ($294|0)!=(0);
  $296 = $295 ^ 1;
  $297 = $296&1;
  $298 = HEAP32[$memDb>>2]|0;
  $299 = ($298|0)!=(0);
  $300 = $299 ^ 1;
  $301 = $300 ? 35 : 0;
  $302 = HEAP32[$pPager>>2]|0;
  $303 = HEAP32[$pPager>>2]|0;
  $304 = (($303) + 204|0);
  $305 = HEAP32[$304>>2]|0;
  $306 = (_sqlite3PcacheOpen($292,$293,$297,$301,$302,$305)|0);
  HEAP32[$rc>>2] = $306;
 }
 $307 = HEAP32[$rc>>2]|0;
 $308 = ($307|0)!=(0);
 if ($308) {
  $309 = HEAP32[$pPager>>2]|0;
  $310 = (($309) + 60|0);
  $311 = HEAP32[$310>>2]|0;
  (_sqlite3OsClose($311)|0);
  $312 = HEAP32[$pPager>>2]|0;
  $313 = (($312) + 200|0);
  $314 = HEAP32[$313>>2]|0;
  _sqlite3PageFree($314);
  $315 = HEAP32[$pPager>>2]|0;
  _sqlite3_free($315);
  $316 = HEAP32[$rc>>2]|0;
  HEAP32[$0>>2] = $316;
  $385 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($385|0);
 }
 $317 = HEAP32[$useJournal>>2]|0;
 $318 = $317&255;
 $319 = HEAP32[$pPager>>2]|0;
 $320 = (($319) + 6|0);
 HEAP8[$320>>0] = $318;
 $321 = HEAP32[$pPager>>2]|0;
 $322 = (($321) + 156|0);
 HEAP32[$322>>2] = 1073741823;
 $323 = HEAP32[$tempFile>>2]|0;
 $324 = $323&255;
 $325 = HEAP32[$pPager>>2]|0;
 $326 = (($325) + 12|0);
 HEAP8[$326>>0] = $324;
 $327 = HEAP32[$tempFile>>2]|0;
 $328 = $327&255;
 $329 = HEAP32[$pPager>>2]|0;
 $330 = (($329) + 4|0);
 HEAP8[$330>>0] = $328;
 $331 = HEAP32[$pPager>>2]|0;
 $332 = (($331) + 12|0);
 $333 = HEAP8[$332>>0]|0;
 $334 = HEAP32[$pPager>>2]|0;
 $335 = (($334) + 18|0);
 HEAP8[$335>>0] = $333;
 $336 = HEAP32[$memDb>>2]|0;
 $337 = $336&255;
 $338 = HEAP32[$pPager>>2]|0;
 $339 = (($338) + 15|0);
 HEAP8[$339>>0] = $337;
 $340 = HEAP32[$readOnly>>2]|0;
 $341 = $340&255;
 $342 = HEAP32[$pPager>>2]|0;
 $343 = (($342) + 14|0);
 HEAP8[$343>>0] = $341;
 $344 = HEAP32[$pPager>>2]|0;
 $345 = (($344) + 12|0);
 $346 = HEAP8[$345>>0]|0;
 $347 = HEAP32[$pPager>>2]|0;
 $348 = (($347) + 7|0);
 HEAP8[$348>>0] = $346;
 $349 = HEAP32[$pPager>>2]|0;
 $350 = (($349) + 7|0);
 $351 = HEAP8[$350>>0]|0;
 $352 = ($351<<24>>24)!=(0);
 if (!($352)) {
  $353 = HEAP32[$pPager>>2]|0;
  $354 = (($353) + 8|0);
  HEAP8[$354>>0] = 1;
  $355 = HEAP32[$pPager>>2]|0;
  $356 = (($355) + 11|0);
  HEAP8[$356>>0] = 2;
  $357 = HEAP32[$pPager>>2]|0;
  $358 = (($357) + 10|0);
  HEAP8[$358>>0] = 34;
  $359 = HEAP32[$pPager>>2]|0;
  $360 = (($359) + 9|0);
  HEAP8[$360>>0] = 2;
 }
 $361 = HEAP32[$4>>2]|0;
 $362 = $361&65535;
 $363 = HEAP32[$pPager>>2]|0;
 $364 = (($363) + 140|0);
 HEAP16[$364>>1] = $362;
 $365 = HEAP32[$pPager>>2]|0;
 $366 = (($365) + 160|0);
 $367 = $366;
 $368 = $367;
 HEAP32[$368>>2] = -1;
 $369 = (($367) + 4)|0;
 $370 = $369;
 HEAP32[$370>>2] = -1;
 $371 = HEAP32[$pPager>>2]|0;
 _setSectorSize($371);
 $372 = HEAP32[$useJournal>>2]|0;
 $373 = ($372|0)!=(0);
 if ($373) {
  $376 = HEAP32[$memDb>>2]|0;
  $377 = ($376|0)!=(0);
  if ($377) {
   $378 = HEAP32[$pPager>>2]|0;
   $379 = (($378) + 5|0);
   HEAP8[$379>>0] = 4;
  }
 } else {
  $374 = HEAP32[$pPager>>2]|0;
  $375 = (($374) + 5|0);
  HEAP8[$375>>0] = 2;
 }
 $380 = HEAP32[$7>>2]|0;
 $381 = HEAP32[$pPager>>2]|0;
 $382 = (($381) + 196|0);
 HEAP32[$382>>2] = $380;
 $383 = HEAP32[$pPager>>2]|0;
 $384 = HEAP32[$2>>2]|0;
 HEAP32[$384>>2] = $383;
 HEAP32[$0>>2] = 0;
 $385 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($385|0);
}
function _pageReinit($pData) {
 $pData = $pData|0;
 var $0 = 0, $1 = 0, $10 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $pPage = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $pPage = sp;
 HEAP32[$0>>2] = $pData;
 $1 = HEAP32[$0>>2]|0;
 $2 = (_sqlite3PagerGetExtra($1)|0);
 HEAP32[$pPage>>2] = $2;
 $3 = HEAP32[$pPage>>2]|0;
 $4 = HEAP8[$3>>0]|0;
 $5 = ($4<<24>>24)!=(0);
 if (!($5)) {
  STACKTOP = sp;return;
 }
 $6 = HEAP32[$pPage>>2]|0;
 HEAP8[$6>>0] = 0;
 $7 = HEAP32[$0>>2]|0;
 $8 = (_sqlite3PagerPageRefcount($7)|0);
 $9 = ($8|0)>(1);
 if (!($9)) {
  STACKTOP = sp;return;
 }
 $10 = HEAP32[$pPage>>2]|0;
 (_btreeInitPage($10)|0);
 STACKTOP = sp;return;
}
function _sqlite3PagerSetMmapLimit($pPager,$0,$1) {
 $pPager = $pPager|0;
 $0 = $0|0;
 $1 = $1|0;
 var $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0;
 var sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $2 = sp + 8|0;
 $3 = sp;
 HEAP32[$2>>2] = $pPager;
 $4 = $3;
 $5 = $4;
 HEAP32[$5>>2] = $0;
 $6 = (($4) + 4)|0;
 $7 = $6;
 HEAP32[$7>>2] = $1;
 $8 = $3;
 $9 = $8;
 $10 = HEAP32[$9>>2]|0;
 $11 = (($8) + 4)|0;
 $12 = $11;
 $13 = HEAP32[$12>>2]|0;
 $14 = HEAP32[$2>>2]|0;
 $15 = (($14) + 128|0);
 $16 = $15;
 $17 = $16;
 HEAP32[$17>>2] = $10;
 $18 = (($16) + 4)|0;
 $19 = $18;
 HEAP32[$19>>2] = $13;
 $20 = HEAP32[$2>>2]|0;
 _pagerFixMaplimit($20);
 STACKTOP = sp;return;
}
function _sqlite3PagerReadFileheader($pPager,$N,$pDest) {
 $pPager = $pPager|0;
 $N = $N|0;
 $pDest = $pDest|0;
 var $$ = 0, $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0;
 var $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $1 = sp + 8|0;
 $2 = sp + 4|0;
 $rc = sp;
 HEAP32[$0>>2] = $pPager;
 HEAP32[$1>>2] = $N;
 HEAP32[$2>>2] = $pDest;
 HEAP32[$rc>>2] = 0;
 $3 = HEAP32[$2>>2]|0;
 $4 = HEAP32[$1>>2]|0;
 _memset(($3|0),0,($4|0))|0;
 $5 = HEAP32[$0>>2]|0;
 $6 = (($5) + 60|0);
 $7 = HEAP32[$6>>2]|0;
 $8 = HEAP32[$7>>2]|0;
 $9 = ($8|0)!=(0|0);
 if (!($9)) {
  $18 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($18|0);
 }
 $10 = HEAP32[$0>>2]|0;
 $11 = (($10) + 60|0);
 $12 = HEAP32[$11>>2]|0;
 $13 = HEAP32[$2>>2]|0;
 $14 = HEAP32[$1>>2]|0;
 $15 = (_sqlite3OsRead($12,$13,$14,0,0)|0);
 HEAP32[$rc>>2] = $15;
 $16 = HEAP32[$rc>>2]|0;
 $17 = ($16|0)==(522);
 $$ = $17 ? 0 : $15;
 HEAP32[$rc>>2] = $$;
 $18 = HEAP32[$rc>>2]|0;
 STACKTOP = sp;return ($18|0);
}
function _sqlite3PagerSetBusyhandler($pPager,$xBusyHandler,$pBusyHandlerArg) {
 $pPager = $pPager|0;
 $xBusyHandler = $xBusyHandler|0;
 $pBusyHandlerArg = $pBusyHandlerArg|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0;
 var $ap = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $1 = sp + 8|0;
 $2 = sp + 4|0;
 $ap = sp;
 HEAP32[$0>>2] = $pPager;
 HEAP32[$1>>2] = $xBusyHandler;
 HEAP32[$2>>2] = $pBusyHandlerArg;
 $3 = HEAP32[$1>>2]|0;
 $4 = HEAP32[$0>>2]|0;
 $5 = (($4) + 176|0);
 HEAP32[$5>>2] = $3;
 $6 = HEAP32[$2>>2]|0;
 $7 = HEAP32[$0>>2]|0;
 $8 = (($7) + 180|0);
 HEAP32[$8>>2] = $6;
 $9 = HEAP32[$0>>2]|0;
 $10 = (($9) + 60|0);
 $11 = HEAP32[$10>>2]|0;
 $12 = HEAP32[$11>>2]|0;
 $13 = ($12|0)!=(0|0);
 if (!($13)) {
  STACKTOP = sp;return;
 }
 $14 = HEAP32[$0>>2]|0;
 $15 = (($14) + 176|0);
 HEAP32[$ap>>2] = $15;
 $16 = HEAP32[$0>>2]|0;
 $17 = (($16) + 60|0);
 $18 = HEAP32[$17>>2]|0;
 $19 = HEAP32[$ap>>2]|0;
 _sqlite3OsFileControlHint($18,15,$19);
 STACKTOP = sp;return;
}
function _btreeInvokeBusyHandler($pArg) {
 $pArg = $pArg|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $pBt = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $pBt = sp;
 HEAP32[$0>>2] = $pArg;
 $1 = HEAP32[$0>>2]|0;
 HEAP32[$pBt>>2] = $1;
 $2 = HEAP32[$pBt>>2]|0;
 $3 = (($2) + 4|0);
 $4 = HEAP32[$3>>2]|0;
 $5 = (($4) + 444|0);
 $6 = (_sqlite3InvokeBusyHandler($5)|0);
 STACKTOP = sp;return ($6|0);
}
function _sqlite3PagerIsreadonly($pPager) {
 $pPager = $pPager|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 HEAP32[$0>>2] = $pPager;
 $1 = HEAP32[$0>>2]|0;
 $2 = (($1) + 14|0);
 $3 = HEAP8[$2>>0]|0;
 STACKTOP = sp;return ($3|0);
}
function _sqlite3Get4byte($p) {
 $p = $p|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0;
 var $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 HEAP32[$0>>2] = $p;
 $1 = HEAP32[$0>>2]|0;
 $2 = HEAP8[$1>>0]|0;
 $3 = $2&255;
 $4 = $3 << 24;
 $5 = HEAP32[$0>>2]|0;
 $6 = (($5) + 1|0);
 $7 = HEAP8[$6>>0]|0;
 $8 = $7&255;
 $9 = $8 << 16;
 $10 = $4 | $9;
 $11 = HEAP32[$0>>2]|0;
 $12 = (($11) + 2|0);
 $13 = HEAP8[$12>>0]|0;
 $14 = $13&255;
 $15 = $14 << 8;
 $16 = $10 | $15;
 $17 = HEAP32[$0>>2]|0;
 $18 = (($17) + 3|0);
 $19 = HEAP8[$18>>0]|0;
 $20 = $19&255;
 $21 = $16 | $20;
 STACKTOP = sp;return ($21|0);
}
function _sqlite3PagerClose($pPager) {
 $pPager = $pPager|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $pTmp = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $pTmp = sp;
 HEAP32[$0>>2] = $pPager;
 $1 = HEAP32[$0>>2]|0;
 $2 = (($1) + 200|0);
 $3 = HEAP32[$2>>2]|0;
 HEAP32[$pTmp>>2] = $3;
 _sqlite3BeginBenignMalloc();
 $4 = HEAP32[$0>>2]|0;
 _pagerFreeMapHdrs($4);
 $5 = HEAP32[$0>>2]|0;
 $6 = (($5) + 4|0);
 HEAP8[$6>>0] = 0;
 $7 = HEAP32[$0>>2]|0;
 $8 = (($7) + 208|0);
 $9 = HEAP32[$8>>2]|0;
 $10 = HEAP32[$0>>2]|0;
 $11 = (($10) + 9|0);
 $12 = HEAP8[$11>>0]|0;
 $13 = $12&255;
 $14 = HEAP32[$0>>2]|0;
 $15 = (($14) + 152|0);
 $16 = HEAP32[$15>>2]|0;
 $17 = HEAP32[$pTmp>>2]|0;
 (_sqlite3WalClose($9,$13,$16,$17)|0);
 $18 = HEAP32[$0>>2]|0;
 $19 = (($18) + 208|0);
 HEAP32[$19>>2] = 0;
 $20 = HEAP32[$0>>2]|0;
 _pager_reset($20);
 $21 = HEAP32[$0>>2]|0;
 $22 = (($21) + 15|0);
 $23 = HEAP8[$22>>0]|0;
 $24 = ($23<<24>>24)!=(0);
 $25 = HEAP32[$0>>2]|0;
 if ($24) {
  _pager_unlock($25);
 } else {
  $26 = (($25) + 64|0);
  $27 = HEAP32[$26>>2]|0;
  $28 = HEAP32[$27>>2]|0;
  $29 = ($28|0)!=(0|0);
  if ($29) {
   $30 = HEAP32[$0>>2]|0;
   $31 = HEAP32[$0>>2]|0;
   $32 = (_pagerSyncHotJournal($31)|0);
   (_pager_error($30,$32)|0);
  }
  $33 = HEAP32[$0>>2]|0;
  _pagerUnlockAndRollback($33);
 }
 _sqlite3EndBenignMalloc();
 $34 = HEAP32[$0>>2]|0;
 $35 = (($34) + 64|0);
 $36 = HEAP32[$35>>2]|0;
 (_sqlite3OsClose($36)|0);
 $37 = HEAP32[$0>>2]|0;
 $38 = (($37) + 60|0);
 $39 = HEAP32[$38>>2]|0;
 (_sqlite3OsClose($39)|0);
 $40 = HEAP32[$pTmp>>2]|0;
 _sqlite3PageFree($40);
 $41 = HEAP32[$0>>2]|0;
 $42 = (($41) + 204|0);
 $43 = HEAP32[$42>>2]|0;
 _sqlite3PcacheClose($43);
 $44 = HEAP32[$0>>2]|0;
 _sqlite3_free($44);
 STACKTOP = sp;return 0;
}
function _sqlite3PagerSetCachesize($pPager,$mxPage) {
 $pPager = $pPager|0;
 $mxPage = $mxPage|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $1 = sp;
 HEAP32[$0>>2] = $pPager;
 HEAP32[$1>>2] = $mxPage;
 $2 = HEAP32[$0>>2]|0;
 $3 = (($2) + 204|0);
 $4 = HEAP32[$3>>2]|0;
 $5 = HEAP32[$1>>2]|0;
 _sqlite3PcacheSetCachesize($4,$5);
 STACKTOP = sp;return;
}
function _sqlite3PcacheSetCachesize($pCache,$mxPage) {
 $pCache = $pCache|0;
 $mxPage = $mxPage|0;
 var $0 = 0, $1 = 0, $10 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $1 = sp;
 HEAP32[$0>>2] = $pCache;
 HEAP32[$1>>2] = $mxPage;
 $2 = HEAP32[$1>>2]|0;
 $3 = HEAP32[$0>>2]|0;
 $4 = (($3) + 16|0);
 HEAP32[$4>>2] = $2;
 $5 = HEAP32[((38640 + 124|0))>>2]|0;
 $6 = HEAP32[$0>>2]|0;
 $7 = (($6) + 40|0);
 $8 = HEAP32[$7>>2]|0;
 $9 = HEAP32[$0>>2]|0;
 $10 = (_numberOfCachePages($9)|0);
 FUNCTION_TABLE_vii[$5 & 7]($8,$10);
 STACKTOP = sp;return;
}
function _pagerFreeMapHdrs($pPager) {
 $pPager = $pPager|0;
 var $0 = 0, $1 = 0, $10 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $p = 0, $pNext = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $p = sp + 4|0;
 $pNext = sp;
 HEAP32[$0>>2] = $pPager;
 $1 = HEAP32[$0>>2]|0;
 $2 = (($1) + 136|0);
 $3 = HEAP32[$2>>2]|0;
 HEAP32[$p>>2] = $3;
 while(1) {
  $4 = HEAP32[$p>>2]|0;
  $5 = ($4|0)!=(0|0);
  if (!($5)) {
   break;
  }
  $6 = HEAP32[$p>>2]|0;
  $7 = (($6) + 12|0);
  $8 = HEAP32[$7>>2]|0;
  HEAP32[$pNext>>2] = $8;
  $9 = HEAP32[$p>>2]|0;
  _sqlite3_free($9);
  $10 = HEAP32[$pNext>>2]|0;
  HEAP32[$p>>2] = $10;
 }
 STACKTOP = sp;return;
}
function _sqlite3WalClose($pWal,$sync_flags,$nBuf,$zBuf) {
 $pWal = $pWal|0;
 $sync_flags = $sync_flags|0;
 $nBuf = $nBuf|0;
 $zBuf = $zBuf|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $7 = 0;
 var $8 = 0, $9 = 0, $bPersist = 0, $isDelete = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 24|0;
 $1 = sp + 20|0;
 $2 = sp + 16|0;
 $3 = sp + 12|0;
 $rc = sp + 8|0;
 $isDelete = sp + 4|0;
 $bPersist = sp;
 HEAP32[$0>>2] = $pWal;
 HEAP32[$1>>2] = $sync_flags;
 HEAP32[$2>>2] = $nBuf;
 HEAP32[$3>>2] = $zBuf;
 HEAP32[$rc>>2] = 0;
 $4 = HEAP32[$0>>2]|0;
 $5 = ($4|0)!=(0|0);
 if (!($5)) {
  $61 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($61|0);
 }
 HEAP32[$isDelete>>2] = 0;
 $6 = HEAP32[$0>>2]|0;
 $7 = (($6) + 4|0);
 $8 = HEAP32[$7>>2]|0;
 $9 = (_sqlite3OsLock($8,4)|0);
 HEAP32[$rc>>2] = $9;
 $10 = HEAP32[$rc>>2]|0;
 $11 = ($10|0)==(0);
 do {
  if ($11) {
   $12 = HEAP32[$0>>2]|0;
   $13 = (($12) + 43|0);
   $14 = HEAP8[$13>>0]|0;
   $15 = $14&255;
   $16 = ($15|0)==(0);
   if ($16) {
    $17 = HEAP32[$0>>2]|0;
    $18 = (($17) + 43|0);
    HEAP8[$18>>0] = 1;
   }
   $19 = HEAP32[$0>>2]|0;
   $20 = HEAP32[$1>>2]|0;
   $21 = HEAP32[$2>>2]|0;
   $22 = HEAP32[$3>>2]|0;
   $23 = (_sqlite3WalCheckpoint($19,0,0,0,$20,$21,$22,0,0)|0);
   HEAP32[$rc>>2] = $23;
   $24 = HEAP32[$rc>>2]|0;
   $25 = ($24|0)==(0);
   if ($25) {
    HEAP32[$bPersist>>2] = -1;
    $26 = HEAP32[$0>>2]|0;
    $27 = (($26) + 4|0);
    $28 = HEAP32[$27>>2]|0;
    _sqlite3OsFileControlHint($28,10,$bPersist);
    $29 = HEAP32[$bPersist>>2]|0;
    $30 = ($29|0)!=(1);
    if ($30) {
     HEAP32[$isDelete>>2] = 1;
     break;
    }
    $31 = HEAP32[$0>>2]|0;
    $32 = (($31) + 16|0);
    $33 = $32;
    $34 = $33;
    $35 = HEAP32[$34>>2]|0;
    $36 = (($33) + 4)|0;
    $37 = $36;
    $38 = HEAP32[$37>>2]|0;
    $39 = ($38|0)>(0);
    $40 = ($35>>>0)>=(0);
    $41 = ($38|0)==(0);
    $42 = $41 & $40;
    $43 = $39 | $42;
    if ($43) {
     $44 = HEAP32[$0>>2]|0;
     _walLimitSize($44,0,0);
    }
   }
  }
 } while(0);
 $45 = HEAP32[$0>>2]|0;
 $46 = HEAP32[$isDelete>>2]|0;
 _walIndexClose($45,$46);
 $47 = HEAP32[$0>>2]|0;
 $48 = (($47) + 8|0);
 $49 = HEAP32[$48>>2]|0;
 (_sqlite3OsClose($49)|0);
 $50 = HEAP32[$isDelete>>2]|0;
 $51 = ($50|0)!=(0);
 if ($51) {
  _sqlite3BeginBenignMalloc();
  $52 = HEAP32[$0>>2]|0;
  $53 = HEAP32[$52>>2]|0;
  $54 = HEAP32[$0>>2]|0;
  $55 = (($54) + 100|0);
  $56 = HEAP32[$55>>2]|0;
  (_sqlite3OsDelete($53,$56,0)|0);
  _sqlite3EndBenignMalloc();
 }
 $57 = HEAP32[$0>>2]|0;
 $58 = (($57) + 32|0);
 $59 = HEAP32[$58>>2]|0;
 _sqlite3_free($59);
 $60 = HEAP32[$0>>2]|0;
 _sqlite3_free($60);
 $61 = HEAP32[$rc>>2]|0;
 STACKTOP = sp;return ($61|0);
}
function _pager_unlock($pPager) {
 $pPager = $pPager|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0;
 var $81 = 0, $82 = 0, $9 = 0, $iDc = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $rc = sp + 4|0;
 $iDc = sp;
 HEAP32[$0>>2] = $pPager;
 $1 = HEAP32[$0>>2]|0;
 $2 = (($1) + 56|0);
 $3 = HEAP32[$2>>2]|0;
 _sqlite3BitvecDestroy($3);
 $4 = HEAP32[$0>>2]|0;
 $5 = (($4) + 56|0);
 HEAP32[$5>>2] = 0;
 $6 = HEAP32[$0>>2]|0;
 _releaseAllSavepoints($6);
 $7 = HEAP32[$0>>2]|0;
 $8 = (_pagerUseWal($7)|0);
 $9 = ($8|0)!=(0);
 $10 = HEAP32[$0>>2]|0;
 if ($9) {
  $11 = (($10) + 208|0);
  $12 = HEAP32[$11>>2]|0;
  _sqlite3WalEndReadTransaction($12);
  $13 = HEAP32[$0>>2]|0;
  $14 = (($13) + 16|0);
  HEAP8[$14>>0] = 0;
 } else {
  $15 = (($10) + 4|0);
  $16 = HEAP8[$15>>0]|0;
  $17 = ($16<<24>>24)!=(0);
  if (!($17)) {
   $18 = HEAP32[$0>>2]|0;
   $19 = (($18) + 60|0);
   $20 = HEAP32[$19>>2]|0;
   $21 = HEAP32[$20>>2]|0;
   $22 = ($21|0)!=(0|0);
   if ($22) {
    $23 = HEAP32[$0>>2]|0;
    $24 = (($23) + 60|0);
    $25 = HEAP32[$24>>2]|0;
    $26 = (_sqlite3OsDeviceCharacteristics($25)|0);
    $27 = $26;
   } else {
    $27 = 0;
   }
   HEAP32[$iDc>>2] = $27;
   $28 = HEAP32[$iDc>>2]|0;
   $29 = $28 & 2048;
   $30 = (0)==($29|0);
   if ($30) {
    label = 8;
   } else {
    $31 = HEAP32[$0>>2]|0;
    $32 = (($31) + 5|0);
    $33 = HEAP8[$32>>0]|0;
    $34 = $33&255;
    $35 = $34 & 5;
    $36 = (1)!=($35|0);
    if ($36) {
     label = 8;
    }
   }
   if ((label|0) == 8) {
    $37 = HEAP32[$0>>2]|0;
    $38 = (($37) + 64|0);
    $39 = HEAP32[$38>>2]|0;
    (_sqlite3OsClose($39)|0);
   }
   $40 = HEAP32[$0>>2]|0;
   $41 = (_pagerUnlockDb($40,0)|0);
   HEAP32[$rc>>2] = $41;
   $42 = HEAP32[$rc>>2]|0;
   $43 = ($42|0)!=(0);
   if ($43) {
    $44 = HEAP32[$0>>2]|0;
    $45 = (($44) + 16|0);
    $46 = HEAP8[$45>>0]|0;
    $47 = $46&255;
    $48 = ($47|0)==(6);
    if ($48) {
     $49 = HEAP32[$0>>2]|0;
     $50 = (($49) + 17|0);
     HEAP8[$50>>0] = 5;
    }
   }
   $51 = HEAP32[$0>>2]|0;
   $52 = (($51) + 18|0);
   HEAP8[$52>>0] = 0;
   $53 = HEAP32[$0>>2]|0;
   $54 = (($53) + 16|0);
   HEAP8[$54>>0] = 0;
  }
 }
 $55 = HEAP32[$0>>2]|0;
 $56 = (($55) + 40|0);
 $57 = HEAP32[$56>>2]|0;
 $58 = ($57|0)!=(0);
 if (!($58)) {
  $69 = HEAP32[$0>>2]|0;
  $70 = (($69) + 72|0);
  $71 = $70;
  $72 = $71;
  HEAP32[$72>>2] = 0;
  $73 = (($71) + 4)|0;
  $74 = $73;
  HEAP32[$74>>2] = 0;
  $75 = HEAP32[$0>>2]|0;
  $76 = (($75) + 80|0);
  $77 = $76;
  $78 = $77;
  HEAP32[$78>>2] = 0;
  $79 = (($77) + 4)|0;
  $80 = $79;
  HEAP32[$80>>2] = 0;
  $81 = HEAP32[$0>>2]|0;
  $82 = (($81) + 19|0);
  HEAP8[$82>>0] = 0;
  STACKTOP = sp;return;
 }
 $59 = HEAP32[$0>>2]|0;
 _pager_reset($59);
 $60 = HEAP32[$0>>2]|0;
 $61 = (($60) + 12|0);
 $62 = HEAP8[$61>>0]|0;
 $63 = HEAP32[$0>>2]|0;
 $64 = (($63) + 18|0);
 HEAP8[$64>>0] = $62;
 $65 = HEAP32[$0>>2]|0;
 $66 = (($65) + 16|0);
 HEAP8[$66>>0] = 0;
 $67 = HEAP32[$0>>2]|0;
 $68 = (($67) + 40|0);
 HEAP32[$68>>2] = 0;
 $69 = HEAP32[$0>>2]|0;
 $70 = (($69) + 72|0);
 $71 = $70;
 $72 = $71;
 HEAP32[$72>>2] = 0;
 $73 = (($71) + 4)|0;
 $74 = $73;
 HEAP32[$74>>2] = 0;
 $75 = HEAP32[$0>>2]|0;
 $76 = (($75) + 80|0);
 $77 = $76;
 $78 = $77;
 HEAP32[$78>>2] = 0;
 $79 = (($77) + 4)|0;
 $80 = $79;
 HEAP32[$80>>2] = 0;
 $81 = HEAP32[$0>>2]|0;
 $82 = (($81) + 19|0);
 HEAP8[$82>>0] = 0;
 STACKTOP = sp;return;
}
function _pager_error($pPager,$rc) {
 $pPager = $pPager|0;
 $rc = $rc|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $or$cond = 0, $rc2 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $1 = sp + 4|0;
 $rc2 = sp;
 HEAP32[$0>>2] = $pPager;
 HEAP32[$1>>2] = $rc;
 $2 = HEAP32[$1>>2]|0;
 $3 = $2 & 255;
 HEAP32[$rc2>>2] = $3;
 $4 = HEAP32[$rc2>>2]|0;
 $5 = ($4|0)==(13);
 $6 = HEAP32[$rc2>>2]|0;
 $7 = ($6|0)==(10);
 $or$cond = $5 | $7;
 if (!($or$cond)) {
  $13 = HEAP32[$1>>2]|0;
  STACKTOP = sp;return ($13|0);
 }
 $8 = HEAP32[$1>>2]|0;
 $9 = HEAP32[$0>>2]|0;
 $10 = (($9) + 40|0);
 HEAP32[$10>>2] = $8;
 $11 = HEAP32[$0>>2]|0;
 $12 = (($11) + 16|0);
 HEAP8[$12>>0] = 6;
 $13 = HEAP32[$1>>2]|0;
 STACKTOP = sp;return ($13|0);
}
function _pagerSyncHotJournal($pPager) {
 $pPager = $pPager|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $rc = 0, label = 0;
 var sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $rc = sp;
 HEAP32[$0>>2] = $pPager;
 HEAP32[$rc>>2] = 0;
 $1 = HEAP32[$0>>2]|0;
 $2 = (($1) + 7|0);
 $3 = HEAP8[$2>>0]|0;
 $4 = ($3<<24>>24)!=(0);
 if (!($4)) {
  $5 = HEAP32[$0>>2]|0;
  $6 = (($5) + 64|0);
  $7 = HEAP32[$6>>2]|0;
  $8 = (_sqlite3OsSync($7,2)|0);
  HEAP32[$rc>>2] = $8;
 }
 $9 = HEAP32[$rc>>2]|0;
 $10 = ($9|0)==(0);
 if (!($10)) {
  $17 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($17|0);
 }
 $11 = HEAP32[$0>>2]|0;
 $12 = (($11) + 64|0);
 $13 = HEAP32[$12>>2]|0;
 $14 = HEAP32[$0>>2]|0;
 $15 = (($14) + 80|0);
 $16 = (_sqlite3OsFileSize($13,$15)|0);
 HEAP32[$rc>>2] = $16;
 $17 = HEAP32[$rc>>2]|0;
 STACKTOP = sp;return ($17|0);
}
function _pagerUnlockAndRollback($pPager) {
 $pPager = $pPager|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0;
 var $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 HEAP32[$0>>2] = $pPager;
 $1 = HEAP32[$0>>2]|0;
 $2 = (($1) + 16|0);
 $3 = HEAP8[$2>>0]|0;
 $4 = $3&255;
 $5 = ($4|0)!=(6);
 do {
  if ($5) {
   $6 = HEAP32[$0>>2]|0;
   $7 = (($6) + 16|0);
   $8 = HEAP8[$7>>0]|0;
   $9 = $8&255;
   $10 = ($9|0)!=(0);
   if ($10) {
    $11 = HEAP32[$0>>2]|0;
    $12 = (($11) + 16|0);
    $13 = HEAP8[$12>>0]|0;
    $14 = $13&255;
    $15 = ($14|0)>=(2);
    if ($15) {
     _sqlite3BeginBenignMalloc();
     $16 = HEAP32[$0>>2]|0;
     (_sqlite3PagerRollback($16)|0);
     _sqlite3EndBenignMalloc();
     break;
    }
    $17 = HEAP32[$0>>2]|0;
    $18 = (($17) + 4|0);
    $19 = HEAP8[$18>>0]|0;
    $20 = ($19<<24>>24)!=(0);
    if (!($20)) {
     $21 = HEAP32[$0>>2]|0;
     (_pager_end_transaction($21,0,0)|0);
    }
   }
  }
 } while(0);
 $22 = HEAP32[$0>>2]|0;
 _pager_unlock($22);
 STACKTOP = sp;return;
}
function _sqlite3OsClose($pId) {
 $pId = $pId|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $rc = sp;
 HEAP32[$0>>2] = $pId;
 HEAP32[$rc>>2] = 0;
 $1 = HEAP32[$0>>2]|0;
 $2 = HEAP32[$1>>2]|0;
 $3 = ($2|0)!=(0|0);
 if (!($3)) {
  $11 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($11|0);
 }
 $4 = HEAP32[$0>>2]|0;
 $5 = HEAP32[$4>>2]|0;
 $6 = (($5) + 4|0);
 $7 = HEAP32[$6>>2]|0;
 $8 = HEAP32[$0>>2]|0;
 $9 = (FUNCTION_TABLE_ii[$7 & 31]($8)|0);
 HEAP32[$rc>>2] = $9;
 $10 = HEAP32[$0>>2]|0;
 HEAP32[$10>>2] = 0;
 $11 = HEAP32[$rc>>2]|0;
 STACKTOP = sp;return ($11|0);
}
function _sqlite3PcacheClose($pCache) {
 $pCache = $pCache|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 HEAP32[$0>>2] = $pCache;
 $1 = HEAP32[((38640 + 148|0))>>2]|0;
 $2 = HEAP32[$0>>2]|0;
 $3 = (($2) + 40|0);
 $4 = HEAP32[$3>>2]|0;
 FUNCTION_TABLE_vi[$1 & 31]($4);
 STACKTOP = sp;return;
}
function _sqlite3PagerRollback($pPager) {
 $pPager = $pPager|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0;
 var $eState = 0, $or$cond = 0, $rc = 0, $rc2 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 16|0;
 $1 = sp + 12|0;
 $rc = sp + 8|0;
 $rc2 = sp + 4|0;
 $eState = sp;
 HEAP32[$1>>2] = $pPager;
 HEAP32[$rc>>2] = 0;
 $2 = HEAP32[$1>>2]|0;
 $3 = (($2) + 16|0);
 $4 = HEAP8[$3>>0]|0;
 $5 = $4&255;
 $6 = ($5|0)==(6);
 $7 = HEAP32[$1>>2]|0;
 if ($6) {
  $8 = (($7) + 40|0);
  $9 = HEAP32[$8>>2]|0;
  HEAP32[$0>>2] = $9;
  $59 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($59|0);
 }
 $10 = (($7) + 16|0);
 $11 = HEAP8[$10>>0]|0;
 $12 = $11&255;
 $13 = ($12|0)<=(1);
 if ($13) {
  HEAP32[$0>>2] = 0;
  $59 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($59|0);
 }
 $14 = HEAP32[$1>>2]|0;
 $15 = (_pagerUseWal($14)|0);
 $16 = ($15|0)!=(0);
 $17 = HEAP32[$1>>2]|0;
 do {
  if ($16) {
   $18 = (_sqlite3PagerSavepoint($17,2,-1)|0);
   HEAP32[$rc>>2] = $18;
   $19 = HEAP32[$1>>2]|0;
   $20 = HEAP32[$1>>2]|0;
   $21 = (($20) + 19|0);
   $22 = HEAP8[$21>>0]|0;
   $23 = $22&255;
   $24 = (_pager_end_transaction($19,$23,0)|0);
   HEAP32[$rc2>>2] = $24;
   $25 = HEAP32[$rc>>2]|0;
   $26 = ($25|0)==(0);
   if ($26) {
    $27 = HEAP32[$rc2>>2]|0;
    HEAP32[$rc>>2] = $27;
   }
  } else {
   $28 = (($17) + 64|0);
   $29 = HEAP32[$28>>2]|0;
   $30 = HEAP32[$29>>2]|0;
   $31 = ($30|0)!=(0|0);
   if ($31) {
    $32 = HEAP32[$1>>2]|0;
    $33 = (($32) + 16|0);
    $34 = HEAP8[$33>>0]|0;
    $35 = $34&255;
    $36 = ($35|0)==(2);
    if (!($36)) {
     $54 = HEAP32[$1>>2]|0;
     $55 = (_pager_playback($54,0)|0);
     HEAP32[$rc>>2] = $55;
     break;
    }
   }
   $37 = HEAP32[$1>>2]|0;
   $38 = (($37) + 16|0);
   $39 = HEAP8[$38>>0]|0;
   $40 = $39&255;
   HEAP32[$eState>>2] = $40;
   $41 = HEAP32[$1>>2]|0;
   $42 = (_pager_end_transaction($41,0,0)|0);
   HEAP32[$rc>>2] = $42;
   $43 = HEAP32[$1>>2]|0;
   $44 = (($43) + 15|0);
   $45 = HEAP8[$44>>0]|0;
   $46 = ($45<<24>>24)==(0);
   $47 = HEAP32[$eState>>2]|0;
   $48 = ($47|0)>(2);
   $or$cond = $46 & $48;
   if ($or$cond) {
    $49 = HEAP32[$1>>2]|0;
    $50 = (($49) + 40|0);
    HEAP32[$50>>2] = 4;
    $51 = HEAP32[$1>>2]|0;
    $52 = (($51) + 16|0);
    HEAP8[$52>>0] = 6;
    $53 = HEAP32[$rc>>2]|0;
    HEAP32[$0>>2] = $53;
    $59 = HEAP32[$0>>2]|0;
    STACKTOP = sp;return ($59|0);
   }
  }
 } while(0);
 $56 = HEAP32[$1>>2]|0;
 $57 = HEAP32[$rc>>2]|0;
 $58 = (_pager_error($56,$57)|0);
 HEAP32[$0>>2] = $58;
 $59 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($59|0);
}
function _pager_end_transaction($pPager,$hasMaster,$bCommit) {
 $pPager = $pPager|0;
 $hasMaster = $hasMaster|0;
 $bCommit = $bCommit|0;
 var $$ = 0, $$4 = 0, $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0;
 var $114 = 0, $115 = 0, $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0;
 var $132 = 0, $133 = 0, $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0;
 var $150 = 0, $151 = 0, $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0;
 var $169 = 0, $17 = 0, $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0;
 var $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0;
 var $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0;
 var $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0;
 var $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0;
 var $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $bDelete = 0, $or$cond = 0, $or$cond3 = 0, $rc = 0, $rc2 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 24|0;
 $1 = sp + 20|0;
 $2 = sp + 16|0;
 $3 = sp + 12|0;
 $rc = sp + 8|0;
 $rc2 = sp + 4|0;
 $bDelete = sp;
 HEAP32[$1>>2] = $pPager;
 HEAP32[$2>>2] = $hasMaster;
 HEAP32[$3>>2] = $bCommit;
 HEAP32[$rc>>2] = 0;
 HEAP32[$rc2>>2] = 0;
 $4 = HEAP32[$1>>2]|0;
 $5 = (($4) + 16|0);
 $6 = HEAP8[$5>>0]|0;
 $7 = $6&255;
 $8 = ($7|0)<(2);
 if ($8) {
  $9 = HEAP32[$1>>2]|0;
  $10 = (($9) + 17|0);
  $11 = HEAP8[$10>>0]|0;
  $12 = $11&255;
  $13 = ($12|0)<(2);
  if ($13) {
   HEAP32[$0>>2] = 0;
   $186 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($186|0);
  }
 }
 $14 = HEAP32[$1>>2]|0;
 _releaseAllSavepoints($14);
 $15 = HEAP32[$1>>2]|0;
 $16 = (($15) + 64|0);
 $17 = HEAP32[$16>>2]|0;
 $18 = HEAP32[$17>>2]|0;
 $19 = ($18|0)!=(0|0);
 L6: do {
  if ($19) {
   $20 = HEAP32[$1>>2]|0;
   $21 = (($20) + 64|0);
   $22 = HEAP32[$21>>2]|0;
   $23 = (_sqlite3IsMemJournal($22)|0);
   $24 = ($23|0)!=(0);
   $25 = HEAP32[$1>>2]|0;
   if ($24) {
    $26 = (($25) + 64|0);
    $27 = HEAP32[$26>>2]|0;
    (_sqlite3OsClose($27)|0);
    break;
   }
   $28 = (($25) + 5|0);
   $29 = HEAP8[$28>>0]|0;
   $30 = $29&255;
   $31 = ($30|0)==(3);
   $32 = HEAP32[$1>>2]|0;
   if ($31) {
    $33 = (($32) + 72|0);
    $34 = $33;
    $35 = $34;
    $36 = HEAP32[$35>>2]|0;
    $37 = (($34) + 4)|0;
    $38 = $37;
    $39 = HEAP32[$38>>2]|0;
    $40 = ($36|0)==(0);
    $41 = ($39|0)==(0);
    $42 = $40 & $41;
    if ($42) {
     HEAP32[$rc>>2] = 0;
    } else {
     $43 = HEAP32[$1>>2]|0;
     $44 = (($43) + 64|0);
     $45 = HEAP32[$44>>2]|0;
     $46 = (_sqlite3OsTruncate($45,0,0)|0);
     HEAP32[$rc>>2] = $46;
     $47 = HEAP32[$rc>>2]|0;
     $48 = ($47|0)==(0);
     if ($48) {
      $49 = HEAP32[$1>>2]|0;
      $50 = (($49) + 8|0);
      $51 = HEAP8[$50>>0]|0;
      $52 = $51&255;
      $53 = ($52|0)!=(0);
      if ($53) {
       $54 = HEAP32[$1>>2]|0;
       $55 = (($54) + 64|0);
       $56 = HEAP32[$55>>2]|0;
       $57 = HEAP32[$1>>2]|0;
       $58 = (($57) + 11|0);
       $59 = HEAP8[$58>>0]|0;
       $60 = $59&255;
       $61 = (_sqlite3OsSync($56,$60)|0);
       HEAP32[$rc>>2] = $61;
      }
     }
    }
    $62 = HEAP32[$1>>2]|0;
    $63 = (($62) + 72|0);
    $64 = $63;
    $65 = $64;
    HEAP32[$65>>2] = 0;
    $66 = (($64) + 4)|0;
    $67 = $66;
    HEAP32[$67>>2] = 0;
    break;
   } else {
    $68 = (($32) + 5|0);
    $69 = HEAP8[$68>>0]|0;
    $70 = $69&255;
    $71 = ($70|0)==(1);
    do {
     if (!($71)) {
      $72 = HEAP32[$1>>2]|0;
      $73 = (($72) + 4|0);
      $74 = HEAP8[$73>>0]|0;
      $75 = $74&255;
      $76 = ($75|0)!=(0);
      if ($76) {
       $77 = HEAP32[$1>>2]|0;
       $78 = (($77) + 5|0);
       $79 = HEAP8[$78>>0]|0;
       $80 = $79&255;
       $81 = ($80|0)!=(5);
       if ($81) {
        break;
       }
      }
      $91 = HEAP32[$1>>2]|0;
      $92 = (($91) + 12|0);
      $93 = HEAP8[$92>>0]|0;
      $94 = ($93<<24>>24)!=(0);
      $$ = $94 ? 0 : 1;
      $95 = $$&1;
      HEAP32[$bDelete>>2] = $95;
      $96 = HEAP32[$1>>2]|0;
      $97 = (($96) + 64|0);
      $98 = HEAP32[$97>>2]|0;
      (_sqlite3OsClose($98)|0);
      $99 = HEAP32[$bDelete>>2]|0;
      $100 = ($99|0)!=(0);
      if (!($100)) {
       break L6;
      }
      $101 = HEAP32[$1>>2]|0;
      $102 = HEAP32[$101>>2]|0;
      $103 = HEAP32[$1>>2]|0;
      $104 = (($103) + 172|0);
      $105 = HEAP32[$104>>2]|0;
      $106 = (_sqlite3OsDelete($102,$105,0)|0);
      HEAP32[$rc>>2] = $106;
      break L6;
     }
    } while(0);
    $82 = HEAP32[$1>>2]|0;
    $83 = HEAP32[$2>>2]|0;
    $84 = (_zeroJournalHdr($82,$83)|0);
    HEAP32[$rc>>2] = $84;
    $85 = HEAP32[$1>>2]|0;
    $86 = (($85) + 72|0);
    $87 = $86;
    $88 = $87;
    HEAP32[$88>>2] = 0;
    $89 = (($87) + 4)|0;
    $90 = $89;
    HEAP32[$90>>2] = 0;
    break;
   }
  }
 } while(0);
 $107 = HEAP32[$1>>2]|0;
 $108 = (($107) + 56|0);
 $109 = HEAP32[$108>>2]|0;
 _sqlite3BitvecDestroy($109);
 $110 = HEAP32[$1>>2]|0;
 $111 = (($110) + 56|0);
 HEAP32[$111>>2] = 0;
 $112 = HEAP32[$1>>2]|0;
 $113 = (($112) + 44|0);
 HEAP32[$113>>2] = 0;
 $114 = HEAP32[$1>>2]|0;
 $115 = (($114) + 204|0);
 $116 = HEAP32[$115>>2]|0;
 _sqlite3PcacheCleanAll($116);
 $117 = HEAP32[$1>>2]|0;
 $118 = (($117) + 204|0);
 $119 = HEAP32[$118>>2]|0;
 $120 = HEAP32[$1>>2]|0;
 $121 = (($120) + 24|0);
 $122 = HEAP32[$121>>2]|0;
 _sqlite3PcacheTruncate($119,$122);
 $123 = HEAP32[$1>>2]|0;
 $124 = (_pagerUseWal($123)|0);
 $125 = ($124|0)!=(0);
 if ($125) {
  $126 = HEAP32[$1>>2]|0;
  $127 = (($126) + 208|0);
  $128 = HEAP32[$127>>2]|0;
  $129 = (_sqlite3WalEndWriteTransaction($128)|0);
  HEAP32[$rc2>>2] = $129;
 } else {
  $130 = HEAP32[$rc>>2]|0;
  $131 = ($130|0)==(0);
  $132 = HEAP32[$3>>2]|0;
  $133 = ($132|0)!=(0);
  $or$cond = $131 & $133;
  if ($or$cond) {
   $134 = HEAP32[$1>>2]|0;
   $135 = (($134) + 32|0);
   $136 = HEAP32[$135>>2]|0;
   $137 = HEAP32[$1>>2]|0;
   $138 = (($137) + 24|0);
   $139 = HEAP32[$138>>2]|0;
   $140 = ($136>>>0)>($139>>>0);
   if ($140) {
    $141 = HEAP32[$1>>2]|0;
    $142 = HEAP32[$1>>2]|0;
    $143 = (($142) + 24|0);
    $144 = HEAP32[$143>>2]|0;
    $145 = (_pager_truncate($141,$144)|0);
    HEAP32[$rc>>2] = $145;
   }
  }
 }
 $146 = HEAP32[$rc>>2]|0;
 $147 = ($146|0)==(0);
 $148 = HEAP32[$3>>2]|0;
 $149 = ($148|0)!=(0);
 $or$cond3 = $147 & $149;
 if ($or$cond3) {
  $150 = HEAP32[$1>>2]|0;
  $151 = (($150) + 60|0);
  $152 = HEAP32[$151>>2]|0;
  $153 = HEAP32[$152>>2]|0;
  $154 = ($153|0)!=(0|0);
  if ($154) {
   $155 = HEAP32[$1>>2]|0;
   $156 = (($155) + 60|0);
   $157 = HEAP32[$156>>2]|0;
   $158 = (_sqlite3OsFileControl($157,22,0)|0);
   HEAP32[$rc>>2] = $158;
   $159 = HEAP32[$rc>>2]|0;
   $160 = ($159|0)==(12);
   $$4 = $160 ? 0 : $158;
   HEAP32[$rc>>2] = $$4;
  }
 }
 $161 = HEAP32[$1>>2]|0;
 $162 = (($161) + 4|0);
 $163 = HEAP8[$162>>0]|0;
 $164 = ($163<<24>>24)!=(0);
 do {
  if (!($164)) {
   $165 = HEAP32[$1>>2]|0;
   $166 = (_pagerUseWal($165)|0);
   $167 = ($166|0)!=(0);
   if ($167) {
    $168 = HEAP32[$1>>2]|0;
    $169 = (($168) + 208|0);
    $170 = HEAP32[$169>>2]|0;
    $171 = (_sqlite3WalExclusiveMode($170,0)|0);
    $172 = ($171|0)!=(0);
    if (!($172)) {
     break;
    }
   }
   $173 = HEAP32[$1>>2]|0;
   $174 = (_pagerUnlockDb($173,1)|0);
   HEAP32[$rc2>>2] = $174;
   $175 = HEAP32[$1>>2]|0;
   $176 = (($175) + 18|0);
   HEAP8[$176>>0] = 0;
  }
 } while(0);
 $177 = HEAP32[$1>>2]|0;
 $178 = (($177) + 16|0);
 HEAP8[$178>>0] = 1;
 $179 = HEAP32[$1>>2]|0;
 $180 = (($179) + 19|0);
 HEAP8[$180>>0] = 0;
 $181 = HEAP32[$rc>>2]|0;
 $182 = ($181|0)==(0);
 $183 = HEAP32[$rc2>>2]|0;
 $184 = HEAP32[$rc>>2]|0;
 $185 = $182 ? $183 : $184;
 HEAP32[$0>>2] = $185;
 $186 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($186|0);
}
function _releaseAllSavepoints($pPager) {
 $pPager = $pPager|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $ii = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $ii = sp;
 HEAP32[$0>>2] = $pPager;
 HEAP32[$ii>>2] = 0;
 while(1) {
  $1 = HEAP32[$ii>>2]|0;
  $2 = HEAP32[$0>>2]|0;
  $3 = (($2) + 96|0);
  $4 = HEAP32[$3>>2]|0;
  $5 = ($1|0)<($4|0);
  if (!($5)) {
   break;
  }
  $6 = HEAP32[$ii>>2]|0;
  $7 = HEAP32[$0>>2]|0;
  $8 = (($7) + 92|0);
  $9 = HEAP32[$8>>2]|0;
  $10 = (($9) + (($6*48)|0)|0);
  $11 = (($10) + 16|0);
  $12 = HEAP32[$11>>2]|0;
  _sqlite3BitvecDestroy($12);
  $13 = HEAP32[$ii>>2]|0;
  $14 = (($13) + 1)|0;
  HEAP32[$ii>>2] = $14;
 }
 $15 = HEAP32[$0>>2]|0;
 $16 = (($15) + 4|0);
 $17 = HEAP8[$16>>0]|0;
 $18 = ($17<<24>>24)!=(0);
 if ($18) {
  $19 = HEAP32[$0>>2]|0;
  $20 = (($19) + 68|0);
  $21 = HEAP32[$20>>2]|0;
  $22 = (_sqlite3IsMemJournal($21)|0);
  $23 = ($22|0)!=(0);
  if ($23) {
   label = 6;
  }
 } else {
  label = 6;
 }
 if ((label|0) == 6) {
  $24 = HEAP32[$0>>2]|0;
  $25 = (($24) + 68|0);
  $26 = HEAP32[$25>>2]|0;
  (_sqlite3OsClose($26)|0);
 }
 $27 = HEAP32[$0>>2]|0;
 $28 = (($27) + 92|0);
 $29 = HEAP32[$28>>2]|0;
 _sqlite3_free($29);
 $30 = HEAP32[$0>>2]|0;
 $31 = (($30) + 92|0);
 HEAP32[$31>>2] = 0;
 $32 = HEAP32[$0>>2]|0;
 $33 = (($32) + 96|0);
 HEAP32[$33>>2] = 0;
 $34 = HEAP32[$0>>2]|0;
 $35 = (($34) + 52|0);
 HEAP32[$35>>2] = 0;
 STACKTOP = sp;return;
}
function _sqlite3IsMemJournal($pJfd) {
 $pJfd = $pJfd|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 HEAP32[$0>>2] = $pJfd;
 $1 = HEAP32[$0>>2]|0;
 $2 = HEAP32[$1>>2]|0;
 $3 = ($2|0)==(4272|0);
 $4 = $3&1;
 STACKTOP = sp;return ($4|0);
}
function _sqlite3OsTruncate($id,$0,$1) {
 $id = $id|0;
 $0 = $0|0;
 $1 = $1|0;
 var $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $2 = sp + 8|0;
 $3 = sp;
 HEAP32[$2>>2] = $id;
 $4 = $3;
 $5 = $4;
 HEAP32[$5>>2] = $0;
 $6 = (($4) + 4)|0;
 $7 = $6;
 HEAP32[$7>>2] = $1;
 $8 = HEAP32[$2>>2]|0;
 $9 = HEAP32[$8>>2]|0;
 $10 = (($9) + 16|0);
 $11 = HEAP32[$10>>2]|0;
 $12 = HEAP32[$2>>2]|0;
 $13 = $3;
 $14 = $13;
 $15 = HEAP32[$14>>2]|0;
 $16 = (($13) + 4)|0;
 $17 = $16;
 $18 = HEAP32[$17>>2]|0;
 $19 = (FUNCTION_TABLE_iiii[$11 & 31]($12,$15,$18)|0);
 STACKTOP = sp;return ($19|0);
}
function _sqlite3OsSync($id,$flags) {
 $id = $id|0;
 $flags = $flags|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $1 = sp;
 HEAP32[$0>>2] = $id;
 HEAP32[$1>>2] = $flags;
 $2 = HEAP32[$0>>2]|0;
 $3 = HEAP32[$2>>2]|0;
 $4 = (($3) + 20|0);
 $5 = HEAP32[$4>>2]|0;
 $6 = HEAP32[$0>>2]|0;
 $7 = HEAP32[$1>>2]|0;
 $8 = (FUNCTION_TABLE_iii[$5 & 63]($6,$7)|0);
 STACKTOP = sp;return ($8|0);
}
function _zeroJournalHdr($pPager,$doTruncate) {
 $pPager = $pPager|0;
 $doTruncate = $doTruncate|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0;
 var $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0;
 var $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0;
 var $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0;
 var $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0;
 var $97 = 0, $98 = 0, $99 = 0, $iLimit = 0, $or$cond = 0, $or$cond3 = 0, $rc = 0, $sz = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 24|0;
 $1 = sp + 20|0;
 $rc = sp + 16|0;
 $iLimit = sp + 8|0;
 $sz = sp;
 HEAP32[$0>>2] = $pPager;
 HEAP32[$1>>2] = $doTruncate;
 HEAP32[$rc>>2] = 0;
 $2 = HEAP32[$0>>2]|0;
 $3 = (($2) + 72|0);
 $4 = $3;
 $5 = $4;
 $6 = HEAP32[$5>>2]|0;
 $7 = (($4) + 4)|0;
 $8 = $7;
 $9 = HEAP32[$8>>2]|0;
 $10 = ($6|0)!=(0);
 $11 = ($9|0)!=(0);
 $12 = $10 | $11;
 if (!($12)) {
  $102 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($102|0);
 }
 $13 = HEAP32[$0>>2]|0;
 $14 = (($13) + 160|0);
 $15 = $14;
 $16 = $15;
 $17 = HEAP32[$16>>2]|0;
 $18 = (($15) + 4)|0;
 $19 = $18;
 $20 = HEAP32[$19>>2]|0;
 $21 = $iLimit;
 $22 = $21;
 HEAP32[$22>>2] = $17;
 $23 = (($21) + 4)|0;
 $24 = $23;
 HEAP32[$24>>2] = $20;
 $25 = HEAP32[$1>>2]|0;
 $26 = ($25|0)!=(0);
 $27 = $iLimit;
 $28 = $27;
 $29 = HEAP32[$28>>2]|0;
 $30 = (($27) + 4)|0;
 $31 = $30;
 $32 = HEAP32[$31>>2]|0;
 $33 = ($29|0)==(0);
 $34 = ($32|0)==(0);
 $35 = $33 & $34;
 $or$cond = $26 | $35;
 $36 = HEAP32[$0>>2]|0;
 $37 = (($36) + 64|0);
 $38 = HEAP32[$37>>2]|0;
 if ($or$cond) {
  $39 = (_sqlite3OsTruncate($38,0,0)|0);
  HEAP32[$rc>>2] = $39;
 } else {
  $40 = (_sqlite3OsWrite($38,4240,28,0,0)|0);
  HEAP32[$rc>>2] = $40;
 }
 $41 = HEAP32[$rc>>2]|0;
 $42 = ($41|0)==(0);
 if ($42) {
  $43 = HEAP32[$0>>2]|0;
  $44 = (($43) + 7|0);
  $45 = HEAP8[$44>>0]|0;
  $46 = ($45<<24>>24)!=(0);
  if (!($46)) {
   $47 = HEAP32[$0>>2]|0;
   $48 = (($47) + 64|0);
   $49 = HEAP32[$48>>2]|0;
   $50 = HEAP32[$0>>2]|0;
   $51 = (($50) + 11|0);
   $52 = HEAP8[$51>>0]|0;
   $53 = $52&255;
   $54 = 16 | $53;
   $55 = (_sqlite3OsSync($49,$54)|0);
   HEAP32[$rc>>2] = $55;
  }
 }
 $56 = HEAP32[$rc>>2]|0;
 $57 = ($56|0)==(0);
 $58 = $iLimit;
 $59 = $58;
 $60 = HEAP32[$59>>2]|0;
 $61 = (($58) + 4)|0;
 $62 = $61;
 $63 = HEAP32[$62>>2]|0;
 $64 = ($63|0)>(0);
 $65 = ($60>>>0)>(0);
 $66 = ($63|0)==(0);
 $67 = $66 & $65;
 $68 = $64 | $67;
 $or$cond3 = $57 & $68;
 if (!($or$cond3)) {
  $102 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($102|0);
 }
 $69 = HEAP32[$0>>2]|0;
 $70 = (($69) + 64|0);
 $71 = HEAP32[$70>>2]|0;
 $72 = (_sqlite3OsFileSize($71,$sz)|0);
 HEAP32[$rc>>2] = $72;
 $73 = HEAP32[$rc>>2]|0;
 $74 = ($73|0)==(0);
 if (!($74)) {
  $102 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($102|0);
 }
 $75 = $sz;
 $76 = $75;
 $77 = HEAP32[$76>>2]|0;
 $78 = (($75) + 4)|0;
 $79 = $78;
 $80 = HEAP32[$79>>2]|0;
 $81 = $iLimit;
 $82 = $81;
 $83 = HEAP32[$82>>2]|0;
 $84 = (($81) + 4)|0;
 $85 = $84;
 $86 = HEAP32[$85>>2]|0;
 $87 = ($80|0)>($86|0);
 $88 = ($77>>>0)>($83>>>0);
 $89 = ($80|0)==($86|0);
 $90 = $89 & $88;
 $91 = $87 | $90;
 if (!($91)) {
  $102 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($102|0);
 }
 $92 = HEAP32[$0>>2]|0;
 $93 = (($92) + 64|0);
 $94 = HEAP32[$93>>2]|0;
 $95 = $iLimit;
 $96 = $95;
 $97 = HEAP32[$96>>2]|0;
 $98 = (($95) + 4)|0;
 $99 = $98;
 $100 = HEAP32[$99>>2]|0;
 $101 = (_sqlite3OsTruncate($94,$97,$100)|0);
 HEAP32[$rc>>2] = $101;
 $102 = HEAP32[$rc>>2]|0;
 STACKTOP = sp;return ($102|0);
}
function _sqlite3OsDelete($pVfs,$zPath,$dirSync) {
 $pVfs = $pVfs|0;
 $zPath = $zPath|0;
 $dirSync = $dirSync|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $1 = sp + 4|0;
 $2 = sp;
 HEAP32[$0>>2] = $pVfs;
 HEAP32[$1>>2] = $zPath;
 HEAP32[$2>>2] = $dirSync;
 $3 = HEAP32[$0>>2]|0;
 $4 = (($3) + 28|0);
 $5 = HEAP32[$4>>2]|0;
 $6 = HEAP32[$0>>2]|0;
 $7 = HEAP32[$1>>2]|0;
 $8 = HEAP32[$2>>2]|0;
 $9 = (FUNCTION_TABLE_iiii[$5 & 31]($6,$7,$8)|0);
 STACKTOP = sp;return ($9|0);
}
function _sqlite3PcacheCleanAll($pCache) {
 $pCache = $pCache|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $p = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $p = sp;
 HEAP32[$0>>2] = $pCache;
 while(1) {
  $1 = HEAP32[$0>>2]|0;
  $2 = HEAP32[$1>>2]|0;
  HEAP32[$p>>2] = $2;
  $3 = ($2|0)!=(0|0);
  if (!($3)) {
   break;
  }
  $4 = HEAP32[$p>>2]|0;
  _sqlite3PcacheMakeClean($4);
 }
 STACKTOP = sp;return;
}
function _pagerUseWal($pPager) {
 $pPager = $pPager|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 HEAP32[$0>>2] = $pPager;
 $1 = HEAP32[$0>>2]|0;
 $2 = (($1) + 208|0);
 $3 = HEAP32[$2>>2]|0;
 $4 = ($3|0)!=(0|0);
 $5 = $4&1;
 STACKTOP = sp;return ($5|0);
}
function _sqlite3WalEndWriteTransaction($pWal) {
 $pWal = $pWal|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 HEAP32[$0>>2] = $pWal;
 $1 = HEAP32[$0>>2]|0;
 $2 = (($1) + 44|0);
 $3 = HEAP8[$2>>0]|0;
 $4 = ($3<<24>>24)!=(0);
 if (!($4)) {
  STACKTOP = sp;return 0;
 }
 $5 = HEAP32[$0>>2]|0;
 _walUnlockExclusive($5,0,1);
 $6 = HEAP32[$0>>2]|0;
 $7 = (($6) + 44|0);
 HEAP8[$7>>0] = 0;
 $8 = HEAP32[$0>>2]|0;
 $9 = (($8) + 47|0);
 HEAP8[$9>>0] = 0;
 STACKTOP = sp;return 0;
}
function _pager_truncate($pPager,$nPage) {
 $pPager = $pPager|0;
 $nPage = $nPage|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0;
 var $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0;
 var $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0;
 var $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0;
 var $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0;
 var $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $currentSize = 0, $newSize = 0, $pTmp = 0, $rc = 0, $szPage = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $0 = sp + 32|0;
 $1 = sp + 28|0;
 $rc = sp + 24|0;
 $currentSize = sp + 8|0;
 $newSize = sp;
 $szPage = sp + 20|0;
 $pTmp = sp + 16|0;
 HEAP32[$0>>2] = $pPager;
 HEAP32[$1>>2] = $nPage;
 HEAP32[$rc>>2] = 0;
 $2 = HEAP32[$0>>2]|0;
 $3 = (($2) + 60|0);
 $4 = HEAP32[$3>>2]|0;
 $5 = HEAP32[$4>>2]|0;
 $6 = ($5|0)!=(0|0);
 if (!($6)) {
  $127 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($127|0);
 }
 $7 = HEAP32[$0>>2]|0;
 $8 = (($7) + 16|0);
 $9 = HEAP8[$8>>0]|0;
 $10 = $9&255;
 $11 = ($10|0)>=(4);
 if (!($11)) {
  $12 = HEAP32[$0>>2]|0;
  $13 = (($12) + 16|0);
  $14 = HEAP8[$13>>0]|0;
  $15 = $14&255;
  $16 = ($15|0)==(0);
  if (!($16)) {
   $127 = HEAP32[$rc>>2]|0;
   STACKTOP = sp;return ($127|0);
  }
 }
 $17 = HEAP32[$0>>2]|0;
 $18 = (($17) + 152|0);
 $19 = HEAP32[$18>>2]|0;
 HEAP32[$szPage>>2] = $19;
 $20 = HEAP32[$0>>2]|0;
 $21 = (($20) + 60|0);
 $22 = HEAP32[$21>>2]|0;
 $23 = (_sqlite3OsFileSize($22,$currentSize)|0);
 HEAP32[$rc>>2] = $23;
 $24 = HEAP32[$szPage>>2]|0;
 $25 = ($24|0)<(0);
 $26 = $25 << 31 >> 31;
 $27 = HEAP32[$1>>2]|0;
 $28 = (___muldi3(($24|0),($26|0),($27|0),0)|0);
 $29 = tempRet0;
 $30 = $newSize;
 $31 = $30;
 HEAP32[$31>>2] = $28;
 $32 = (($30) + 4)|0;
 $33 = $32;
 HEAP32[$33>>2] = $29;
 $34 = HEAP32[$rc>>2]|0;
 $35 = ($34|0)==(0);
 if (!($35)) {
  $127 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($127|0);
 }
 $36 = $currentSize;
 $37 = $36;
 $38 = HEAP32[$37>>2]|0;
 $39 = (($36) + 4)|0;
 $40 = $39;
 $41 = HEAP32[$40>>2]|0;
 $42 = $newSize;
 $43 = $42;
 $44 = HEAP32[$43>>2]|0;
 $45 = (($42) + 4)|0;
 $46 = $45;
 $47 = HEAP32[$46>>2]|0;
 $48 = ($38|0)!=($44|0);
 $49 = ($41|0)!=($47|0);
 $50 = $48 | $49;
 if (!($50)) {
  $127 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($127|0);
 }
 $51 = $currentSize;
 $52 = $51;
 $53 = HEAP32[$52>>2]|0;
 $54 = (($51) + 4)|0;
 $55 = $54;
 $56 = HEAP32[$55>>2]|0;
 $57 = $newSize;
 $58 = $57;
 $59 = HEAP32[$58>>2]|0;
 $60 = (($57) + 4)|0;
 $61 = $60;
 $62 = HEAP32[$61>>2]|0;
 $63 = ($56|0)>($62|0);
 $64 = ($53>>>0)>($59>>>0);
 $65 = ($56|0)==($62|0);
 $66 = $65 & $64;
 $67 = $63 | $66;
 if ($67) {
  $68 = HEAP32[$0>>2]|0;
  $69 = (($68) + 60|0);
  $70 = HEAP32[$69>>2]|0;
  $71 = $newSize;
  $72 = $71;
  $73 = HEAP32[$72>>2]|0;
  $74 = (($71) + 4)|0;
  $75 = $74;
  $76 = HEAP32[$75>>2]|0;
  $77 = (_sqlite3OsTruncate($70,$73,$76)|0);
  HEAP32[$rc>>2] = $77;
 } else {
  $78 = $currentSize;
  $79 = $78;
  $80 = HEAP32[$79>>2]|0;
  $81 = (($78) + 4)|0;
  $82 = $81;
  $83 = HEAP32[$82>>2]|0;
  $84 = HEAP32[$szPage>>2]|0;
  $85 = ($84|0)<(0);
  $86 = $85 << 31 >> 31;
  $87 = (_i64Add(($80|0),($83|0),($84|0),($86|0))|0);
  $88 = tempRet0;
  $89 = $newSize;
  $90 = $89;
  $91 = HEAP32[$90>>2]|0;
  $92 = (($89) + 4)|0;
  $93 = $92;
  $94 = HEAP32[$93>>2]|0;
  $95 = ($88|0)<($94|0);
  $96 = ($87>>>0)<=($91>>>0);
  $97 = ($88|0)==($94|0);
  $98 = $97 & $96;
  $99 = $95 | $98;
  if ($99) {
   $100 = HEAP32[$0>>2]|0;
   $101 = (($100) + 200|0);
   $102 = HEAP32[$101>>2]|0;
   HEAP32[$pTmp>>2] = $102;
   $103 = HEAP32[$pTmp>>2]|0;
   $104 = HEAP32[$szPage>>2]|0;
   _memset(($103|0),0,($104|0))|0;
   $105 = HEAP32[$0>>2]|0;
   $106 = (($105) + 60|0);
   $107 = HEAP32[$106>>2]|0;
   $108 = HEAP32[$pTmp>>2]|0;
   $109 = HEAP32[$szPage>>2]|0;
   $110 = $newSize;
   $111 = $110;
   $112 = HEAP32[$111>>2]|0;
   $113 = (($110) + 4)|0;
   $114 = $113;
   $115 = HEAP32[$114>>2]|0;
   $116 = HEAP32[$szPage>>2]|0;
   $117 = ($116|0)<(0);
   $118 = $117 << 31 >> 31;
   $119 = (_i64Subtract(($112|0),($115|0),($116|0),($118|0))|0);
   $120 = tempRet0;
   $121 = (_sqlite3OsWrite($107,$108,$109,$119,$120)|0);
   HEAP32[$rc>>2] = $121;
  }
 }
 $122 = HEAP32[$rc>>2]|0;
 $123 = ($122|0)==(0);
 if (!($123)) {
  $127 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($127|0);
 }
 $124 = HEAP32[$1>>2]|0;
 $125 = HEAP32[$0>>2]|0;
 $126 = (($125) + 32|0);
 HEAP32[$126>>2] = $124;
 $127 = HEAP32[$rc>>2]|0;
 STACKTOP = sp;return ($127|0);
}
function _sqlite3WalExclusiveMode($pWal,$op) {
 $pWal = $pWal|0;
 $op = $op|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $5 = 0, $6 = 0, $7 = 0;
 var $8 = 0, $9 = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $1 = sp + 4|0;
 $rc = sp;
 HEAP32[$0>>2] = $pWal;
 HEAP32[$1>>2] = $op;
 $2 = HEAP32[$1>>2]|0;
 $3 = ($2|0)==(0);
 if (!($3)) {
  $26 = HEAP32[$1>>2]|0;
  $27 = ($26|0)>(0);
  $28 = HEAP32[$0>>2]|0;
  if ($27) {
   $29 = HEAP32[$0>>2]|0;
   $30 = (($29) + 40|0);
   $31 = HEAP16[$30>>1]|0;
   $32 = $31 << 16 >> 16;
   $33 = (3 + ($32))|0;
   _walUnlockShared($28,$33);
   $34 = HEAP32[$0>>2]|0;
   $35 = (($34) + 43|0);
   HEAP8[$35>>0] = 1;
   HEAP32[$rc>>2] = 1;
   $41 = HEAP32[$rc>>2]|0;
   STACKTOP = sp;return ($41|0);
  } else {
   $36 = (($28) + 43|0);
   $37 = HEAP8[$36>>0]|0;
   $38 = $37&255;
   $39 = ($38|0)==(0);
   $40 = $39&1;
   HEAP32[$rc>>2] = $40;
   $41 = HEAP32[$rc>>2]|0;
   STACKTOP = sp;return ($41|0);
  }
 }
 $4 = HEAP32[$0>>2]|0;
 $5 = (($4) + 43|0);
 $6 = HEAP8[$5>>0]|0;
 $7 = ($6<<24>>24)!=(0);
 if (!($7)) {
  HEAP32[$rc>>2] = 0;
  $41 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($41|0);
 }
 $8 = HEAP32[$0>>2]|0;
 $9 = (($8) + 43|0);
 HEAP8[$9>>0] = 0;
 $10 = HEAP32[$0>>2]|0;
 $11 = HEAP32[$0>>2]|0;
 $12 = (($11) + 40|0);
 $13 = HEAP16[$12>>1]|0;
 $14 = $13 << 16 >> 16;
 $15 = (3 + ($14))|0;
 $16 = (_walLockShared($10,$15)|0);
 $17 = ($16|0)!=(0);
 if ($17) {
  $18 = HEAP32[$0>>2]|0;
  $19 = (($18) + 43|0);
  HEAP8[$19>>0] = 1;
 }
 $20 = HEAP32[$0>>2]|0;
 $21 = (($20) + 43|0);
 $22 = HEAP8[$21>>0]|0;
 $23 = $22&255;
 $24 = ($23|0)==(0);
 $25 = $24&1;
 HEAP32[$rc>>2] = $25;
 $41 = HEAP32[$rc>>2]|0;
 STACKTOP = sp;return ($41|0);
}
function _pagerUnlockDb($pPager,$eLock) {
 $pPager = $pPager|0;
 $eLock = $eLock|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $1 = sp + 4|0;
 $rc = sp;
 HEAP32[$0>>2] = $pPager;
 HEAP32[$1>>2] = $eLock;
 HEAP32[$rc>>2] = 0;
 $2 = HEAP32[$0>>2]|0;
 $3 = (($2) + 60|0);
 $4 = HEAP32[$3>>2]|0;
 $5 = HEAP32[$4>>2]|0;
 $6 = ($5|0)!=(0|0);
 if (!($6)) {
  $27 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($27|0);
 }
 $7 = HEAP32[$0>>2]|0;
 $8 = (($7) + 13|0);
 $9 = HEAP8[$8>>0]|0;
 $10 = $9&255;
 $11 = ($10|0)!=(0);
 if ($11) {
  $17 = 0;
 } else {
  $12 = HEAP32[$0>>2]|0;
  $13 = (($12) + 60|0);
  $14 = HEAP32[$13>>2]|0;
  $15 = HEAP32[$1>>2]|0;
  $16 = (_sqlite3OsUnlock($14,$15)|0);
  $17 = $16;
 }
 HEAP32[$rc>>2] = $17;
 $18 = HEAP32[$0>>2]|0;
 $19 = (($18) + 17|0);
 $20 = HEAP8[$19>>0]|0;
 $21 = $20&255;
 $22 = ($21|0)!=(5);
 if (!($22)) {
  $27 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($27|0);
 }
 $23 = HEAP32[$1>>2]|0;
 $24 = $23&255;
 $25 = HEAP32[$0>>2]|0;
 $26 = (($25) + 17|0);
 HEAP8[$26>>0] = $24;
 $27 = HEAP32[$rc>>2]|0;
 STACKTOP = sp;return ($27|0);
}
function _sqlite3OsUnlock($id,$lockType) {
 $id = $id|0;
 $lockType = $lockType|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $1 = sp;
 HEAP32[$0>>2] = $id;
 HEAP32[$1>>2] = $lockType;
 $2 = HEAP32[$0>>2]|0;
 $3 = HEAP32[$2>>2]|0;
 $4 = (($3) + 32|0);
 $5 = HEAP32[$4>>2]|0;
 $6 = HEAP32[$0>>2]|0;
 $7 = HEAP32[$1>>2]|0;
 $8 = (FUNCTION_TABLE_iii[$5 & 63]($6,$7)|0);
 STACKTOP = sp;return ($8|0);
}
function _walLockShared($pWal,$lockIdx) {
 $pWal = $pWal|0;
 $lockIdx = $lockIdx|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $1 = sp + 8|0;
 $2 = sp + 4|0;
 $rc = sp;
 HEAP32[$1>>2] = $pWal;
 HEAP32[$2>>2] = $lockIdx;
 $3 = HEAP32[$1>>2]|0;
 $4 = (($3) + 43|0);
 $5 = HEAP8[$4>>0]|0;
 $6 = ($5<<24>>24)!=(0);
 if ($6) {
  HEAP32[$0>>2] = 0;
  $13 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($13|0);
 } else {
  $7 = HEAP32[$1>>2]|0;
  $8 = (($7) + 4|0);
  $9 = HEAP32[$8>>2]|0;
  $10 = HEAP32[$2>>2]|0;
  $11 = (_sqlite3OsShmLock($9,$10,1,6)|0);
  HEAP32[$rc>>2] = $11;
  $12 = HEAP32[$rc>>2]|0;
  HEAP32[$0>>2] = $12;
  $13 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($13|0);
 }
 return 0|0;
}
function _walUnlockShared($pWal,$lockIdx) {
 $pWal = $pWal|0;
 $lockIdx = $lockIdx|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $1 = sp;
 HEAP32[$0>>2] = $pWal;
 HEAP32[$1>>2] = $lockIdx;
 $2 = HEAP32[$0>>2]|0;
 $3 = (($2) + 43|0);
 $4 = HEAP8[$3>>0]|0;
 $5 = ($4<<24>>24)!=(0);
 if ($5) {
  STACKTOP = sp;return;
 }
 $6 = HEAP32[$0>>2]|0;
 $7 = (($6) + 4|0);
 $8 = HEAP32[$7>>2]|0;
 $9 = HEAP32[$1>>2]|0;
 (_sqlite3OsShmLock($8,$9,1,5)|0);
 STACKTOP = sp;return;
}
function _sqlite3OsShmLock($id,$offset,$n,$flags) {
 $id = $id|0;
 $offset = $offset|0;
 $n = $n|0;
 $flags = $flags|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $1 = sp + 8|0;
 $2 = sp + 4|0;
 $3 = sp;
 HEAP32[$0>>2] = $id;
 HEAP32[$1>>2] = $offset;
 HEAP32[$2>>2] = $n;
 HEAP32[$3>>2] = $flags;
 $4 = HEAP32[$0>>2]|0;
 $5 = HEAP32[$4>>2]|0;
 $6 = (($5) + 56|0);
 $7 = HEAP32[$6>>2]|0;
 $8 = HEAP32[$0>>2]|0;
 $9 = HEAP32[$1>>2]|0;
 $10 = HEAP32[$2>>2]|0;
 $11 = HEAP32[$3>>2]|0;
 $12 = (FUNCTION_TABLE_iiiii[$7 & 7]($8,$9,$10,$11)|0);
 STACKTOP = sp;return ($12|0);
}
function _walUnlockExclusive($pWal,$lockIdx,$n) {
 $pWal = $pWal|0;
 $lockIdx = $lockIdx|0;
 $n = $n|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $1 = sp + 4|0;
 $2 = sp;
 HEAP32[$0>>2] = $pWal;
 HEAP32[$1>>2] = $lockIdx;
 HEAP32[$2>>2] = $n;
 $3 = HEAP32[$0>>2]|0;
 $4 = (($3) + 43|0);
 $5 = HEAP8[$4>>0]|0;
 $6 = ($5<<24>>24)!=(0);
 if ($6) {
  STACKTOP = sp;return;
 }
 $7 = HEAP32[$0>>2]|0;
 $8 = (($7) + 4|0);
 $9 = HEAP32[$8>>2]|0;
 $10 = HEAP32[$1>>2]|0;
 $11 = HEAP32[$2>>2]|0;
 (_sqlite3OsShmLock($9,$10,$11,9)|0);
 STACKTOP = sp;return;
}
function _memjrnlClose($pJfd) {
 $pJfd = $pJfd|0;
 var $0 = 0, $1 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 HEAP32[$0>>2] = $pJfd;
 $1 = HEAP32[$0>>2]|0;
 (_memjrnlTruncate($1,0,0)|0);
 STACKTOP = sp;return 0;
}
function _memjrnlRead($pJfd,$zBuf,$iAmt,$0,$1) {
 $pJfd = $pJfd|0;
 $zBuf = $zBuf|0;
 $iAmt = $iAmt|0;
 $0 = $0|0;
 $1 = $1|0;
 var $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0, $116 = 0, $117 = 0;
 var $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0, $134 = 0, $135 = 0;
 var $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0;
 var $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0;
 var $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0;
 var $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0;
 var $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0;
 var $99 = 0, $iChunkOffset = 0, $iOff = 0, $iSpace = 0, $nCopy = 0, $nRead = 0, $or$cond = 0, $or$cond4 = 0, $p = 0, $pChunk = 0, $zOut = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 64|0;
 $2 = sp + 52|0;
 $3 = sp + 48|0;
 $4 = sp + 44|0;
 $5 = sp + 8|0;
 $p = sp + 40|0;
 $zOut = sp + 36|0;
 $nRead = sp + 32|0;
 $iChunkOffset = sp + 28|0;
 $pChunk = sp + 24|0;
 $iOff = sp;
 $iSpace = sp + 20|0;
 $nCopy = sp + 16|0;
 HEAP32[$2>>2] = $pJfd;
 HEAP32[$3>>2] = $zBuf;
 HEAP32[$4>>2] = $iAmt;
 $6 = $5;
 $7 = $6;
 HEAP32[$7>>2] = $0;
 $8 = (($6) + 4)|0;
 $9 = $8;
 HEAP32[$9>>2] = $1;
 $10 = HEAP32[$2>>2]|0;
 HEAP32[$p>>2] = $10;
 $11 = HEAP32[$3>>2]|0;
 HEAP32[$zOut>>2] = $11;
 $12 = HEAP32[$4>>2]|0;
 HEAP32[$nRead>>2] = $12;
 $13 = HEAP32[$p>>2]|0;
 $14 = (($13) + 24|0);
 $15 = $14;
 $16 = $15;
 $17 = HEAP32[$16>>2]|0;
 $18 = (($15) + 4)|0;
 $19 = $18;
 $20 = HEAP32[$19>>2]|0;
 $21 = $5;
 $22 = $21;
 $23 = HEAP32[$22>>2]|0;
 $24 = (($21) + 4)|0;
 $25 = $24;
 $26 = HEAP32[$25>>2]|0;
 $27 = ($17|0)!=($23|0);
 $28 = ($20|0)!=($26|0);
 $29 = $27 | $28;
 $30 = $5;
 $31 = $30;
 $32 = HEAP32[$31>>2]|0;
 $33 = (($30) + 4)|0;
 $34 = $33;
 $35 = HEAP32[$34>>2]|0;
 $36 = ($32|0)==(0);
 $37 = ($35|0)==(0);
 $38 = $36 & $37;
 $or$cond = $29 | $38;
 L1: do {
  if ($or$cond) {
   $39 = $iOff;
   $40 = $39;
   HEAP32[$40>>2] = 0;
   $41 = (($39) + 4)|0;
   $42 = $41;
   HEAP32[$42>>2] = 0;
   $43 = HEAP32[$p>>2]|0;
   $44 = (($43) + 4|0);
   $45 = HEAP32[$44>>2]|0;
   HEAP32[$pChunk>>2] = $45;
   while(1) {
    $46 = HEAP32[$pChunk>>2]|0;
    $47 = ($46|0)!=(0|0);
    if (!($47)) {
     break L1;
    }
    $48 = $iOff;
    $49 = $48;
    $50 = HEAP32[$49>>2]|0;
    $51 = (($48) + 4)|0;
    $52 = $51;
    $53 = HEAP32[$52>>2]|0;
    $54 = (_i64Add(($50|0),($53|0),1020,0)|0);
    $55 = tempRet0;
    $56 = $5;
    $57 = $56;
    $58 = HEAP32[$57>>2]|0;
    $59 = (($56) + 4)|0;
    $60 = $59;
    $61 = HEAP32[$60>>2]|0;
    $62 = ($55|0)<($61|0);
    $63 = ($54>>>0)<=($58>>>0);
    $64 = ($55|0)==($61|0);
    $65 = $64 & $63;
    $66 = $62 | $65;
    if (!($66)) {
     break L1;
    }
    $67 = $iOff;
    $68 = $67;
    $69 = HEAP32[$68>>2]|0;
    $70 = (($67) + 4)|0;
    $71 = $70;
    $72 = HEAP32[$71>>2]|0;
    $73 = (_i64Add(($69|0),($72|0),1020,0)|0);
    $74 = tempRet0;
    $75 = $iOff;
    $76 = $75;
    HEAP32[$76>>2] = $73;
    $77 = (($75) + 4)|0;
    $78 = $77;
    HEAP32[$78>>2] = $74;
    $79 = HEAP32[$pChunk>>2]|0;
    $80 = HEAP32[$79>>2]|0;
    HEAP32[$pChunk>>2] = $80;
   }
  } else {
   $81 = HEAP32[$p>>2]|0;
   $82 = (($81) + 24|0);
   $83 = (($82) + 8|0);
   $84 = HEAP32[$83>>2]|0;
   HEAP32[$pChunk>>2] = $84;
  }
 } while(0);
 $85 = $5;
 $86 = $85;
 $87 = HEAP32[$86>>2]|0;
 $88 = (($85) + 4)|0;
 $89 = $88;
 $90 = HEAP32[$89>>2]|0;
 $91 = (___remdi3(($87|0),($90|0),1020,0)|0);
 $92 = tempRet0;
 HEAP32[$iChunkOffset>>2] = $91;
 while(1) {
  $93 = HEAP32[$iChunkOffset>>2]|0;
  $94 = (1020 - ($93))|0;
  HEAP32[$iSpace>>2] = $94;
  $95 = HEAP32[$nRead>>2]|0;
  $96 = HEAP32[$iChunkOffset>>2]|0;
  $97 = (1020 - ($96))|0;
  $98 = ($95|0)<($97|0);
  if ($98) {
   $99 = HEAP32[$nRead>>2]|0;
   $102 = $99;
  } else {
   $100 = HEAP32[$iChunkOffset>>2]|0;
   $101 = (1020 - ($100))|0;
   $102 = $101;
  }
  HEAP32[$nCopy>>2] = $102;
  $103 = HEAP32[$zOut>>2]|0;
  $104 = HEAP32[$iChunkOffset>>2]|0;
  $105 = HEAP32[$pChunk>>2]|0;
  $106 = (($105) + 4|0);
  $107 = (($106) + ($104)|0);
  $108 = HEAP32[$nCopy>>2]|0;
  _memcpy(($103|0),($107|0),($108|0))|0;
  $109 = HEAP32[$nCopy>>2]|0;
  $110 = HEAP32[$zOut>>2]|0;
  $111 = (($110) + ($109)|0);
  HEAP32[$zOut>>2] = $111;
  $112 = HEAP32[$iSpace>>2]|0;
  $113 = HEAP32[$nRead>>2]|0;
  $114 = (($113) - ($112))|0;
  HEAP32[$nRead>>2] = $114;
  HEAP32[$iChunkOffset>>2] = 0;
  $115 = HEAP32[$nRead>>2]|0;
  $116 = ($115|0)>=(0);
  if (!($116)) {
   label = 13;
   break;
  }
  $117 = HEAP32[$pChunk>>2]|0;
  $118 = HEAP32[$117>>2]|0;
  HEAP32[$pChunk>>2] = $118;
  $119 = ($118|0)!=(0|0);
  $120 = HEAP32[$nRead>>2]|0;
  $121 = ($120|0)>(0);
  $or$cond4 = $119 & $121;
  if (!($or$cond4)) {
   label = 13;
   break;
  }
 }
 if ((label|0) == 13) {
  $122 = $5;
  $123 = $122;
  $124 = HEAP32[$123>>2]|0;
  $125 = (($122) + 4)|0;
  $126 = $125;
  $127 = HEAP32[$126>>2]|0;
  $128 = HEAP32[$4>>2]|0;
  $129 = ($128|0)<(0);
  $130 = $129 << 31 >> 31;
  $131 = (_i64Add(($124|0),($127|0),($128|0),($130|0))|0);
  $132 = tempRet0;
  $133 = HEAP32[$p>>2]|0;
  $134 = (($133) + 24|0);
  $135 = $134;
  $136 = $135;
  HEAP32[$136>>2] = $131;
  $137 = (($135) + 4)|0;
  $138 = $137;
  HEAP32[$138>>2] = $132;
  $139 = HEAP32[$pChunk>>2]|0;
  $140 = HEAP32[$p>>2]|0;
  $141 = (($140) + 24|0);
  $142 = (($141) + 8|0);
  HEAP32[$142>>2] = $139;
  STACKTOP = sp;return 0;
 }
 return 0|0;
}
function _memjrnlWrite($pJfd,$zBuf,$iAmt,$0,$1) {
 $pJfd = $pJfd|0;
 $zBuf = $zBuf|0;
 $iAmt = $iAmt|0;
 $0 = $0|0;
 $1 = $1|0;
 var $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0;
 var $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0;
 var $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0;
 var $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0;
 var $83 = 0, $84 = 0, $85 = 0, $86 = 0, $9 = 0, $iChunkOffset = 0, $iSpace = 0, $nWrite = 0, $p = 0, $pChunk = 0, $pNew = 0, $zWrite = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 64|0;
 $2 = sp + 48|0;
 $3 = sp + 44|0;
 $4 = sp + 40|0;
 $5 = sp + 36|0;
 $6 = sp;
 $p = sp + 32|0;
 $nWrite = sp + 28|0;
 $zWrite = sp + 24|0;
 $pChunk = sp + 20|0;
 $iChunkOffset = sp + 16|0;
 $iSpace = sp + 12|0;
 $pNew = sp + 8|0;
 HEAP32[$3>>2] = $pJfd;
 HEAP32[$4>>2] = $zBuf;
 HEAP32[$5>>2] = $iAmt;
 $7 = $6;
 $8 = $7;
 HEAP32[$8>>2] = $0;
 $9 = (($7) + 4)|0;
 $10 = $9;
 HEAP32[$10>>2] = $1;
 $11 = HEAP32[$3>>2]|0;
 HEAP32[$p>>2] = $11;
 $12 = HEAP32[$5>>2]|0;
 HEAP32[$nWrite>>2] = $12;
 $13 = HEAP32[$4>>2]|0;
 HEAP32[$zWrite>>2] = $13;
 while(1) {
  $14 = HEAP32[$nWrite>>2]|0;
  $15 = ($14|0)>(0);
  if (!($15)) {
   label = 14;
   break;
  }
  $16 = HEAP32[$p>>2]|0;
  $17 = (($16) + 8|0);
  $18 = (($17) + 8|0);
  $19 = HEAP32[$18>>2]|0;
  HEAP32[$pChunk>>2] = $19;
  $20 = HEAP32[$p>>2]|0;
  $21 = (($20) + 8|0);
  $22 = $21;
  $23 = $22;
  $24 = HEAP32[$23>>2]|0;
  $25 = (($22) + 4)|0;
  $26 = $25;
  $27 = HEAP32[$26>>2]|0;
  $28 = (___remdi3(($24|0),($27|0),1020,0)|0);
  $29 = tempRet0;
  HEAP32[$iChunkOffset>>2] = $28;
  $30 = HEAP32[$nWrite>>2]|0;
  $31 = HEAP32[$iChunkOffset>>2]|0;
  $32 = (1020 - ($31))|0;
  $33 = ($30|0)<($32|0);
  if ($33) {
   $34 = HEAP32[$nWrite>>2]|0;
   $37 = $34;
  } else {
   $35 = HEAP32[$iChunkOffset>>2]|0;
   $36 = (1020 - ($35))|0;
   $37 = $36;
  }
  HEAP32[$iSpace>>2] = $37;
  $38 = HEAP32[$iChunkOffset>>2]|0;
  $39 = ($38|0)==(0);
  if ($39) {
   $40 = (_sqlite3_malloc(1024)|0);
   HEAP32[$pNew>>2] = $40;
   $41 = HEAP32[$pNew>>2]|0;
   $42 = ($41|0)!=(0|0);
   if (!($42)) {
    label = 8;
    break;
   }
   $43 = HEAP32[$pNew>>2]|0;
   HEAP32[$43>>2] = 0;
   $44 = HEAP32[$pChunk>>2]|0;
   $45 = ($44|0)!=(0|0);
   $46 = HEAP32[$pNew>>2]|0;
   if ($45) {
    $47 = HEAP32[$pChunk>>2]|0;
    HEAP32[$47>>2] = $46;
   } else {
    $48 = HEAP32[$p>>2]|0;
    $49 = (($48) + 4|0);
    HEAP32[$49>>2] = $46;
   }
   $50 = HEAP32[$pNew>>2]|0;
   $51 = HEAP32[$p>>2]|0;
   $52 = (($51) + 8|0);
   $53 = (($52) + 8|0);
   HEAP32[$53>>2] = $50;
  }
  $54 = HEAP32[$iChunkOffset>>2]|0;
  $55 = HEAP32[$p>>2]|0;
  $56 = (($55) + 8|0);
  $57 = (($56) + 8|0);
  $58 = HEAP32[$57>>2]|0;
  $59 = (($58) + 4|0);
  $60 = (($59) + ($54)|0);
  $61 = HEAP32[$zWrite>>2]|0;
  $62 = HEAP32[$iSpace>>2]|0;
  _memcpy(($60|0),($61|0),($62|0))|0;
  $63 = HEAP32[$iSpace>>2]|0;
  $64 = HEAP32[$zWrite>>2]|0;
  $65 = (($64) + ($63)|0);
  HEAP32[$zWrite>>2] = $65;
  $66 = HEAP32[$iSpace>>2]|0;
  $67 = HEAP32[$nWrite>>2]|0;
  $68 = (($67) - ($66))|0;
  HEAP32[$nWrite>>2] = $68;
  $69 = HEAP32[$iSpace>>2]|0;
  $70 = ($69|0)<(0);
  $71 = $70 << 31 >> 31;
  $72 = HEAP32[$p>>2]|0;
  $73 = (($72) + 8|0);
  $74 = $73;
  $75 = $74;
  $76 = HEAP32[$75>>2]|0;
  $77 = (($74) + 4)|0;
  $78 = $77;
  $79 = HEAP32[$78>>2]|0;
  $80 = (_i64Add(($76|0),($79|0),($69|0),($71|0))|0);
  $81 = tempRet0;
  $82 = $73;
  $83 = $82;
  HEAP32[$83>>2] = $80;
  $84 = (($82) + 4)|0;
  $85 = $84;
  HEAP32[$85>>2] = $81;
 }
 if ((label|0) == 8) {
  HEAP32[$2>>2] = 3082;
  $86 = HEAP32[$2>>2]|0;
  STACKTOP = sp;return ($86|0);
 }
 else if ((label|0) == 14) {
  HEAP32[$2>>2] = 0;
  $86 = HEAP32[$2>>2]|0;
  STACKTOP = sp;return ($86|0);
 }
 return 0|0;
}
function _memjrnlTruncate($pJfd,$0,$1) {
 $pJfd = $pJfd|0;
 $0 = $0|0;
 $1 = $1|0;
 var $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $p = 0, $pChunk = 0, $pTmp = 0;
 var label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $2 = sp + 20|0;
 $3 = sp;
 $p = sp + 16|0;
 $pChunk = sp + 12|0;
 $pTmp = sp + 8|0;
 HEAP32[$2>>2] = $pJfd;
 $4 = $3;
 $5 = $4;
 HEAP32[$5>>2] = $0;
 $6 = (($4) + 4)|0;
 $7 = $6;
 HEAP32[$7>>2] = $1;
 $8 = HEAP32[$2>>2]|0;
 HEAP32[$p>>2] = $8;
 $9 = HEAP32[$p>>2]|0;
 $10 = (($9) + 4|0);
 $11 = HEAP32[$10>>2]|0;
 HEAP32[$pChunk>>2] = $11;
 while(1) {
  $12 = HEAP32[$pChunk>>2]|0;
  $13 = ($12|0)!=(0|0);
  if (!($13)) {
   break;
  }
  $14 = HEAP32[$pChunk>>2]|0;
  HEAP32[$pTmp>>2] = $14;
  $15 = HEAP32[$pChunk>>2]|0;
  $16 = HEAP32[$15>>2]|0;
  HEAP32[$pChunk>>2] = $16;
  $17 = HEAP32[$pTmp>>2]|0;
  _sqlite3_free($17);
 }
 $18 = HEAP32[$2>>2]|0;
 _sqlite3MemJournalOpen($18);
 STACKTOP = sp;return 0;
}
function _memjrnlSync($NotUsed,$NotUsed2) {
 $NotUsed = $NotUsed|0;
 $NotUsed2 = $NotUsed2|0;
 var $0 = 0, $1 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $1 = sp;
 HEAP32[$0>>2] = $NotUsed;
 HEAP32[$1>>2] = $NotUsed2;
 STACKTOP = sp;return 0;
}
function _memjrnlFileSize($pJfd,$pSize) {
 $pJfd = $pJfd|0;
 $pSize = $pSize|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $p = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $1 = sp + 4|0;
 $p = sp;
 HEAP32[$0>>2] = $pJfd;
 HEAP32[$1>>2] = $pSize;
 $2 = HEAP32[$0>>2]|0;
 HEAP32[$p>>2] = $2;
 $3 = HEAP32[$p>>2]|0;
 $4 = (($3) + 8|0);
 $5 = $4;
 $6 = $5;
 $7 = HEAP32[$6>>2]|0;
 $8 = (($5) + 4)|0;
 $9 = $8;
 $10 = HEAP32[$9>>2]|0;
 $11 = HEAP32[$1>>2]|0;
 $12 = $11;
 $13 = $12;
 HEAP32[$13>>2] = $7;
 $14 = (($12) + 4)|0;
 $15 = $14;
 HEAP32[$15>>2] = $10;
 STACKTOP = sp;return 0;
}
function _sqlite3MemJournalOpen($pJfd) {
 $pJfd = $pJfd|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $p = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $p = sp;
 HEAP32[$0>>2] = $pJfd;
 $1 = HEAP32[$0>>2]|0;
 HEAP32[$p>>2] = $1;
 $2 = HEAP32[$p>>2]|0;
 $3 = (_sqlite3MemJournalSize()|0);
 _memset(($2|0),0,($3|0))|0;
 $4 = HEAP32[$p>>2]|0;
 HEAP32[$4>>2] = 4272;
 STACKTOP = sp;return;
}
function _sqlite3MemJournalSize() {
 var label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = sp;return 40;
}
function _sqlite3PagerSavepoint($pPager,$op,$iSavepoint) {
 $pPager = $pPager|0;
 $op = $op|0;
 $iSavepoint = $iSavepoint|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $8 = 0, $9 = 0, $ii = 0, $nNew = 0, $pSavepoint = 0;
 var $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 24|0;
 $1 = sp + 20|0;
 $2 = sp + 16|0;
 $rc = sp + 12|0;
 $ii = sp + 8|0;
 $nNew = sp + 4|0;
 $pSavepoint = sp;
 HEAP32[$0>>2] = $pPager;
 HEAP32[$1>>2] = $op;
 HEAP32[$2>>2] = $iSavepoint;
 $3 = HEAP32[$0>>2]|0;
 $4 = (($3) + 40|0);
 $5 = HEAP32[$4>>2]|0;
 HEAP32[$rc>>2] = $5;
 $6 = HEAP32[$rc>>2]|0;
 $7 = ($6|0)==(0);
 if (!($7)) {
  $76 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($76|0);
 }
 $8 = HEAP32[$2>>2]|0;
 $9 = HEAP32[$0>>2]|0;
 $10 = (($9) + 96|0);
 $11 = HEAP32[$10>>2]|0;
 $12 = ($8|0)<($11|0);
 if (!($12)) {
  $76 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($76|0);
 }
 $13 = HEAP32[$2>>2]|0;
 $14 = HEAP32[$1>>2]|0;
 $15 = ($14|0)==(1);
 $16 = $15 ? 0 : 1;
 $17 = (($13) + ($16))|0;
 HEAP32[$nNew>>2] = $17;
 $18 = HEAP32[$nNew>>2]|0;
 HEAP32[$ii>>2] = $18;
 while(1) {
  $19 = HEAP32[$ii>>2]|0;
  $20 = HEAP32[$0>>2]|0;
  $21 = (($20) + 96|0);
  $22 = HEAP32[$21>>2]|0;
  $23 = ($19|0)<($22|0);
  if (!($23)) {
   break;
  }
  $24 = HEAP32[$ii>>2]|0;
  $25 = HEAP32[$0>>2]|0;
  $26 = (($25) + 92|0);
  $27 = HEAP32[$26>>2]|0;
  $28 = (($27) + (($24*48)|0)|0);
  $29 = (($28) + 16|0);
  $30 = HEAP32[$29>>2]|0;
  _sqlite3BitvecDestroy($30);
  $31 = HEAP32[$ii>>2]|0;
  $32 = (($31) + 1)|0;
  HEAP32[$ii>>2] = $32;
 }
 $33 = HEAP32[$nNew>>2]|0;
 $34 = HEAP32[$0>>2]|0;
 $35 = (($34) + 96|0);
 HEAP32[$35>>2] = $33;
 $36 = HEAP32[$1>>2]|0;
 $37 = ($36|0)==(1);
 if (!($37)) {
  $56 = HEAP32[$0>>2]|0;
  $57 = (_pagerUseWal($56)|0);
  $58 = ($57|0)!=(0);
  if (!($58)) {
   $59 = HEAP32[$0>>2]|0;
   $60 = (($59) + 64|0);
   $61 = HEAP32[$60>>2]|0;
   $62 = HEAP32[$61>>2]|0;
   $63 = ($62|0)!=(0|0);
   if (!($63)) {
    $76 = HEAP32[$rc>>2]|0;
    STACKTOP = sp;return ($76|0);
   }
  }
  $64 = HEAP32[$nNew>>2]|0;
  $65 = ($64|0)==(0);
  if ($65) {
   $72 = 0;
  } else {
   $66 = HEAP32[$nNew>>2]|0;
   $67 = (($66) - 1)|0;
   $68 = HEAP32[$0>>2]|0;
   $69 = (($68) + 92|0);
   $70 = HEAP32[$69>>2]|0;
   $71 = (($70) + (($67*48)|0)|0);
   $72 = $71;
  }
  HEAP32[$pSavepoint>>2] = $72;
  $73 = HEAP32[$0>>2]|0;
  $74 = HEAP32[$pSavepoint>>2]|0;
  $75 = (_pagerPlaybackSavepoint($73,$74)|0);
  HEAP32[$rc>>2] = $75;
  $76 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($76|0);
 }
 $38 = HEAP32[$nNew>>2]|0;
 $39 = ($38|0)==(0);
 if (!($39)) {
  $76 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($76|0);
 }
 $40 = HEAP32[$0>>2]|0;
 $41 = (($40) + 68|0);
 $42 = HEAP32[$41>>2]|0;
 $43 = HEAP32[$42>>2]|0;
 $44 = ($43|0)!=(0|0);
 if (!($44)) {
  $76 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($76|0);
 }
 $45 = HEAP32[$0>>2]|0;
 $46 = (($45) + 68|0);
 $47 = HEAP32[$46>>2]|0;
 $48 = (_sqlite3IsMemJournal($47)|0);
 $49 = ($48|0)!=(0);
 if ($49) {
  $50 = HEAP32[$0>>2]|0;
  $51 = (($50) + 68|0);
  $52 = HEAP32[$51>>2]|0;
  $53 = (_sqlite3OsTruncate($52,0,0)|0);
  HEAP32[$rc>>2] = $53;
 }
 $54 = HEAP32[$0>>2]|0;
 $55 = (($54) + 52|0);
 HEAP32[$55>>2] = 0;
 $76 = HEAP32[$rc>>2]|0;
 STACKTOP = sp;return ($76|0);
}
function _pager_playback($pPager,$isHot) {
 $pPager = $pPager|0;
 $isHot = $isHot|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0;
 var $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0, $188 = 0;
 var $189 = 0, $19 = 0, $190 = 0, $191 = 0, $192 = 0, $193 = 0, $194 = 0, $195 = 0, $196 = 0, $197 = 0, $198 = 0, $199 = 0, $2 = 0, $20 = 0, $200 = 0, $201 = 0, $202 = 0, $203 = 0, $204 = 0, $205 = 0;
 var $206 = 0, $207 = 0, $208 = 0, $209 = 0, $21 = 0, $210 = 0, $211 = 0, $212 = 0, $213 = 0, $214 = 0, $215 = 0, $216 = 0, $217 = 0, $218 = 0, $219 = 0, $22 = 0, $220 = 0, $221 = 0, $222 = 0, $223 = 0;
 var $224 = 0, $225 = 0, $226 = 0, $227 = 0, $228 = 0, $229 = 0, $23 = 0, $230 = 0, $231 = 0, $232 = 0, $233 = 0, $234 = 0, $235 = 0, $236 = 0, $237 = 0, $238 = 0, $239 = 0, $24 = 0, $240 = 0, $241 = 0;
 var $242 = 0, $243 = 0, $244 = 0, $245 = 0, $246 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0;
 var $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0;
 var $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0;
 var $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0;
 var $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $mxPg = 0, $nPlayback = 0, $nRec = 0, $needPagerReset = 0, $or$cond = 0, $or$cond3 = 0, $or$cond5 = 0, $or$cond7 = 0, $pVfs = 0, $rc = 0, $res = 0, $szJ = 0, $u = 0;
 var $vararg_buffer = 0, $vararg_ptr8 = 0, $zMaster = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 64|0;
 $vararg_buffer = sp + 8|0;
 $0 = sp + 56|0;
 $1 = sp + 52|0;
 $pVfs = sp + 48|0;
 $szJ = sp;
 $nRec = sp + 44|0;
 $u = sp + 40|0;
 $mxPg = sp + 36|0;
 $rc = sp + 32|0;
 $res = sp + 28|0;
 $zMaster = sp + 24|0;
 $needPagerReset = sp + 20|0;
 $nPlayback = sp + 16|0;
 HEAP32[$0>>2] = $pPager;
 HEAP32[$1>>2] = $isHot;
 $2 = HEAP32[$0>>2]|0;
 $3 = HEAP32[$2>>2]|0;
 HEAP32[$pVfs>>2] = $3;
 HEAP32[$mxPg>>2] = 0;
 HEAP32[$res>>2] = 1;
 HEAP32[$zMaster>>2] = 0;
 HEAP32[$nPlayback>>2] = 0;
 $4 = HEAP32[$0>>2]|0;
 $5 = (($4) + 64|0);
 $6 = HEAP32[$5>>2]|0;
 $7 = (_sqlite3OsFileSize($6,$szJ)|0);
 HEAP32[$rc>>2] = $7;
 $8 = HEAP32[$rc>>2]|0;
 $9 = ($8|0)!=(0);
 L1: do {
  if (!($9)) {
   $10 = HEAP32[$0>>2]|0;
   $11 = (($10) + 200|0);
   $12 = HEAP32[$11>>2]|0;
   HEAP32[$zMaster>>2] = $12;
   $13 = HEAP32[$0>>2]|0;
   $14 = (($13) + 64|0);
   $15 = HEAP32[$14>>2]|0;
   $16 = HEAP32[$zMaster>>2]|0;
   $17 = HEAP32[$0>>2]|0;
   $18 = HEAP32[$17>>2]|0;
   $19 = (($18) + 8|0);
   $20 = HEAP32[$19>>2]|0;
   $21 = (($20) + 1)|0;
   $22 = (_readMasterJournal($15,$16,$21)|0);
   HEAP32[$rc>>2] = $22;
   $23 = HEAP32[$rc>>2]|0;
   $24 = ($23|0)==(0);
   if ($24) {
    $25 = HEAP32[$zMaster>>2]|0;
    $26 = HEAP8[$25>>0]|0;
    $27 = $26 << 24 >> 24;
    $28 = ($27|0)!=(0);
    if ($28) {
     $29 = HEAP32[$pVfs>>2]|0;
     $30 = HEAP32[$zMaster>>2]|0;
     $31 = (_sqlite3OsAccess($29,$30,0,$res)|0);
     HEAP32[$rc>>2] = $31;
    }
   }
   HEAP32[$zMaster>>2] = 0;
   $32 = HEAP32[$rc>>2]|0;
   $33 = ($32|0)==(0);
   $34 = HEAP32[$res>>2]|0;
   $35 = ($34|0)!=(0);
   $or$cond = $33 & $35;
   if ($or$cond) {
    $36 = HEAP32[$0>>2]|0;
    $37 = (($36) + 72|0);
    $38 = $37;
    $39 = $38;
    HEAP32[$39>>2] = 0;
    $40 = (($38) + 4)|0;
    $41 = $40;
    HEAP32[$41>>2] = 0;
    $42 = HEAP32[$1>>2]|0;
    HEAP32[$needPagerReset>>2] = $42;
    L8: while(1) {
     $43 = HEAP32[$0>>2]|0;
     $44 = HEAP32[$1>>2]|0;
     $45 = $szJ;
     $46 = $45;
     $47 = HEAP32[$46>>2]|0;
     $48 = (($45) + 4)|0;
     $49 = $48;
     $50 = HEAP32[$49>>2]|0;
     $51 = (_readJournalHdr($43,$44,$47,$50,$nRec,$mxPg)|0);
     HEAP32[$rc>>2] = $51;
     $52 = HEAP32[$rc>>2]|0;
     $53 = ($52|0)!=(0);
     if ($53) {
      label = 8;
      break;
     }
     $56 = HEAP32[$nRec>>2]|0;
     $57 = ($56|0)==(-1);
     if ($57) {
      $58 = $szJ;
      $59 = $58;
      $60 = HEAP32[$59>>2]|0;
      $61 = (($58) + 4)|0;
      $62 = $61;
      $63 = HEAP32[$62>>2]|0;
      $64 = HEAP32[$0>>2]|0;
      $65 = (($64) + 148|0);
      $66 = HEAP32[$65>>2]|0;
      $67 = (_i64Subtract(($60|0),($63|0),($66|0),0)|0);
      $68 = tempRet0;
      $69 = HEAP32[$0>>2]|0;
      $70 = (($69) + 152|0);
      $71 = HEAP32[$70>>2]|0;
      $72 = (($71) + 8)|0;
      $73 = ($72|0)<(0);
      $74 = $73 << 31 >> 31;
      $75 = (___divdi3(($67|0),($68|0),($72|0),($74|0))|0);
      $76 = tempRet0;
      HEAP32[$nRec>>2] = $75;
     }
     $77 = HEAP32[$nRec>>2]|0;
     $78 = ($77|0)!=(0);
     $79 = HEAP32[$1>>2]|0;
     $80 = ($79|0)!=(0);
     $or$cond3 = $78 | $80;
     if (!($or$cond3)) {
      $81 = HEAP32[$0>>2]|0;
      $82 = (($81) + 80|0);
      $83 = $82;
      $84 = $83;
      $85 = HEAP32[$84>>2]|0;
      $86 = (($83) + 4)|0;
      $87 = $86;
      $88 = HEAP32[$87>>2]|0;
      $89 = HEAP32[$0>>2]|0;
      $90 = (($89) + 148|0);
      $91 = HEAP32[$90>>2]|0;
      $92 = (_i64Add(($85|0),($88|0),($91|0),0)|0);
      $93 = tempRet0;
      $94 = HEAP32[$0>>2]|0;
      $95 = (($94) + 72|0);
      $96 = $95;
      $97 = $96;
      $98 = HEAP32[$97>>2]|0;
      $99 = (($96) + 4)|0;
      $100 = $99;
      $101 = HEAP32[$100>>2]|0;
      $102 = ($92|0)==($98|0);
      $103 = ($93|0)==($101|0);
      $104 = $102 & $103;
      if ($104) {
       $105 = $szJ;
       $106 = $105;
       $107 = HEAP32[$106>>2]|0;
       $108 = (($105) + 4)|0;
       $109 = $108;
       $110 = HEAP32[$109>>2]|0;
       $111 = HEAP32[$0>>2]|0;
       $112 = (($111) + 72|0);
       $113 = $112;
       $114 = $113;
       $115 = HEAP32[$114>>2]|0;
       $116 = (($113) + 4)|0;
       $117 = $116;
       $118 = HEAP32[$117>>2]|0;
       $119 = (_i64Subtract(($107|0),($110|0),($115|0),($118|0))|0);
       $120 = tempRet0;
       $121 = HEAP32[$0>>2]|0;
       $122 = (($121) + 152|0);
       $123 = HEAP32[$122>>2]|0;
       $124 = (($123) + 8)|0;
       $125 = ($124|0)<(0);
       $126 = $125 << 31 >> 31;
       $127 = (___divdi3(($119|0),($120|0),($124|0),($126|0))|0);
       $128 = tempRet0;
       HEAP32[$nRec>>2] = $127;
      }
     }
     $129 = HEAP32[$0>>2]|0;
     $130 = (($129) + 72|0);
     $131 = $130;
     $132 = $131;
     $133 = HEAP32[$132>>2]|0;
     $134 = (($131) + 4)|0;
     $135 = $134;
     $136 = HEAP32[$135>>2]|0;
     $137 = HEAP32[$0>>2]|0;
     $138 = (($137) + 148|0);
     $139 = HEAP32[$138>>2]|0;
     $140 = ($133|0)==($139|0);
     $141 = ($136|0)==(0);
     $142 = $140 & $141;
     if ($142) {
      $143 = HEAP32[$0>>2]|0;
      $144 = HEAP32[$mxPg>>2]|0;
      $145 = (_pager_truncate($143,$144)|0);
      HEAP32[$rc>>2] = $145;
      $146 = HEAP32[$rc>>2]|0;
      $147 = ($146|0)!=(0);
      if ($147) {
       break L1;
      }
      $148 = HEAP32[$mxPg>>2]|0;
      $149 = HEAP32[$0>>2]|0;
      $150 = (($149) + 24|0);
      HEAP32[$150>>2] = $148;
     }
     HEAP32[$u>>2] = 0;
     while(1) {
      $151 = HEAP32[$u>>2]|0;
      $152 = HEAP32[$nRec>>2]|0;
      $153 = ($151>>>0)<($152>>>0);
      if (!($153)) {
       continue L8;
      }
      $154 = HEAP32[$needPagerReset>>2]|0;
      $155 = ($154|0)!=(0);
      if ($155) {
       $156 = HEAP32[$0>>2]|0;
       _pager_reset($156);
       HEAP32[$needPagerReset>>2] = 0;
      }
      $157 = HEAP32[$0>>2]|0;
      $158 = HEAP32[$0>>2]|0;
      $159 = (($158) + 72|0);
      $160 = (_pager_playback_one_page($157,$159,0,1,0)|0);
      HEAP32[$rc>>2] = $160;
      $161 = HEAP32[$rc>>2]|0;
      $162 = ($161|0)==(0);
      if (!($162)) {
       break;
      }
      $163 = HEAP32[$nPlayback>>2]|0;
      $164 = (($163) + 1)|0;
      HEAP32[$nPlayback>>2] = $164;
      $165 = HEAP32[$u>>2]|0;
      $166 = (($165) + 1)|0;
      HEAP32[$u>>2] = $166;
     }
     $167 = HEAP32[$rc>>2]|0;
     $168 = ($167|0)==(101);
     if (!($168)) {
      label = 26;
      break;
     }
     $169 = $szJ;
     $170 = $169;
     $171 = HEAP32[$170>>2]|0;
     $172 = (($169) + 4)|0;
     $173 = $172;
     $174 = HEAP32[$173>>2]|0;
     $175 = HEAP32[$0>>2]|0;
     $176 = (($175) + 72|0);
     $177 = $176;
     $178 = $177;
     HEAP32[$178>>2] = $171;
     $179 = (($177) + 4)|0;
     $180 = $179;
     HEAP32[$180>>2] = $174;
    }
    if ((label|0) == 8) {
     $54 = HEAP32[$rc>>2]|0;
     $55 = ($54|0)==(101);
     if (!($55)) {
      break;
     }
     HEAP32[$rc>>2] = 0;
     break;
    }
    else if ((label|0) == 26) {
     $181 = HEAP32[$rc>>2]|0;
     $182 = ($181|0)==(522);
     if (!($182)) {
      break;
     }
     HEAP32[$rc>>2] = 0;
     break;
    }
   }
  }
 } while(0);
 $183 = HEAP32[$0>>2]|0;
 $184 = (($183) + 12|0);
 $185 = HEAP8[$184>>0]|0;
 $186 = HEAP32[$0>>2]|0;
 $187 = (($186) + 18|0);
 HEAP8[$187>>0] = $185;
 $188 = HEAP32[$rc>>2]|0;
 $189 = ($188|0)==(0);
 if ($189) {
  $190 = HEAP32[$0>>2]|0;
  $191 = (($190) + 200|0);
  $192 = HEAP32[$191>>2]|0;
  HEAP32[$zMaster>>2] = $192;
  $193 = HEAP32[$0>>2]|0;
  $194 = (($193) + 64|0);
  $195 = HEAP32[$194>>2]|0;
  $196 = HEAP32[$zMaster>>2]|0;
  $197 = HEAP32[$0>>2]|0;
  $198 = HEAP32[$197>>2]|0;
  $199 = (($198) + 8|0);
  $200 = HEAP32[$199>>2]|0;
  $201 = (($200) + 1)|0;
  $202 = (_readMasterJournal($195,$196,$201)|0);
  HEAP32[$rc>>2] = $202;
 }
 $203 = HEAP32[$rc>>2]|0;
 $204 = ($203|0)==(0);
 do {
  if ($204) {
   $205 = HEAP32[$0>>2]|0;
   $206 = (($205) + 16|0);
   $207 = HEAP8[$206>>0]|0;
   $208 = $207&255;
   $209 = ($208|0)>=(4);
   if (!($209)) {
    $210 = HEAP32[$0>>2]|0;
    $211 = (($210) + 16|0);
    $212 = HEAP8[$211>>0]|0;
    $213 = $212&255;
    $214 = ($213|0)==(0);
    if (!($214)) {
     break;
    }
   }
   $215 = HEAP32[$0>>2]|0;
   $216 = (_sqlite3PagerSync($215,0)|0);
   HEAP32[$rc>>2] = $216;
  }
 } while(0);
 $217 = HEAP32[$rc>>2]|0;
 $218 = ($217|0)==(0);
 if ($218) {
  $219 = HEAP32[$0>>2]|0;
  $220 = HEAP32[$zMaster>>2]|0;
  $221 = HEAP8[$220>>0]|0;
  $222 = $221 << 24 >> 24;
  $223 = ($222|0)!=(0);
  $224 = $223&1;
  $225 = (_pager_end_transaction($219,$224,0)|0);
  HEAP32[$rc>>2] = $225;
 }
 $226 = HEAP32[$rc>>2]|0;
 $227 = ($226|0)==(0);
 if ($227) {
  $228 = HEAP32[$zMaster>>2]|0;
  $229 = HEAP8[$228>>0]|0;
  $230 = $229 << 24 >> 24;
  $231 = ($230|0)!=(0);
  $232 = HEAP32[$res>>2]|0;
  $233 = ($232|0)!=(0);
  $or$cond5 = $231 & $233;
  if ($or$cond5) {
   $234 = HEAP32[$0>>2]|0;
   $235 = HEAP32[$zMaster>>2]|0;
   $236 = (_pager_delmaster($234,$235)|0);
   HEAP32[$rc>>2] = $236;
  }
 }
 $237 = HEAP32[$1>>2]|0;
 $238 = ($237|0)!=(0);
 $239 = HEAP32[$nPlayback>>2]|0;
 $240 = ($239|0)!=(0);
 $or$cond7 = $238 & $240;
 if (!($or$cond7)) {
  $245 = HEAP32[$0>>2]|0;
  _setSectorSize($245);
  $246 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($246|0);
 }
 $241 = HEAP32[$nPlayback>>2]|0;
 $242 = HEAP32[$0>>2]|0;
 $243 = (($242) + 172|0);
 $244 = HEAP32[$243>>2]|0;
 HEAP32[$vararg_buffer>>2] = $241;
 $vararg_ptr8 = (($vararg_buffer) + 4|0);
 HEAP32[$vararg_ptr8>>2] = $244;
 _sqlite3_log(539,4352,$vararg_buffer);
 $245 = HEAP32[$0>>2]|0;
 _setSectorSize($245);
 $246 = HEAP32[$rc>>2]|0;
 STACKTOP = sp;return ($246|0);
}
function _readMasterJournal($pJrnl,$zMaster,$nMaster) {
 $pJrnl = $pJrnl|0;
 $zMaster = $zMaster|0;
 $nMaster = $nMaster|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0;
 var $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $aMagic = 0, $cksum = 0, $len = 0, $or$cond = 0, $or$cond3 = 0, $rc = 0;
 var $szJ = 0, $u = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $0 = sp + 36|0;
 $1 = sp + 32|0;
 $2 = sp + 28|0;
 $3 = sp + 24|0;
 $rc = sp + 20|0;
 $len = sp + 16|0;
 $szJ = sp;
 $cksum = sp + 12|0;
 $u = sp + 8|0;
 $aMagic = sp + 40|0;
 HEAP32[$1>>2] = $pJrnl;
 HEAP32[$2>>2] = $zMaster;
 HEAP32[$3>>2] = $nMaster;
 $4 = HEAP32[$2>>2]|0;
 HEAP8[$4>>0] = 0;
 $5 = HEAP32[$1>>2]|0;
 $6 = (_sqlite3OsFileSize($5,$szJ)|0);
 HEAP32[$rc>>2] = $6;
 $7 = (0)!=($6|0);
 $8 = $szJ;
 $9 = $8;
 $10 = HEAP32[$9>>2]|0;
 $11 = (($8) + 4)|0;
 $12 = $11;
 $13 = HEAP32[$12>>2]|0;
 $14 = ($13|0)<(0);
 $15 = ($10>>>0)<(16);
 $16 = ($13|0)==(0);
 $17 = $16 & $15;
 $18 = $14 | $17;
 $or$cond = $7 | $18;
 if (!($or$cond)) {
  $19 = HEAP32[$1>>2]|0;
  $20 = $szJ;
  $21 = $20;
  $22 = HEAP32[$21>>2]|0;
  $23 = (($20) + 4)|0;
  $24 = $23;
  $25 = HEAP32[$24>>2]|0;
  $26 = (_i64Subtract(($22|0),($25|0),16,0)|0);
  $27 = tempRet0;
  $28 = (_read32bits($19,$26,$27,$len)|0);
  HEAP32[$rc>>2] = $28;
  $29 = (0)!=($28|0);
  if (!($29)) {
   $30 = HEAP32[$len>>2]|0;
   $31 = HEAP32[$3>>2]|0;
   $32 = ($30>>>0)>=($31>>>0);
   $33 = HEAP32[$len>>2]|0;
   $34 = ($33|0)==(0);
   $or$cond3 = $32 | $34;
   if (!($or$cond3)) {
    $35 = HEAP32[$1>>2]|0;
    $36 = $szJ;
    $37 = $36;
    $38 = HEAP32[$37>>2]|0;
    $39 = (($36) + 4)|0;
    $40 = $39;
    $41 = HEAP32[$40>>2]|0;
    $42 = (_i64Subtract(($38|0),($41|0),12,0)|0);
    $43 = tempRet0;
    $44 = (_read32bits($35,$42,$43,$cksum)|0);
    HEAP32[$rc>>2] = $44;
    $45 = (0)!=($44|0);
    if (!($45)) {
     $46 = HEAP32[$1>>2]|0;
     $47 = $szJ;
     $48 = $47;
     $49 = HEAP32[$48>>2]|0;
     $50 = (($47) + 4)|0;
     $51 = $50;
     $52 = HEAP32[$51>>2]|0;
     $53 = (_i64Subtract(($49|0),($52|0),8,0)|0);
     $54 = tempRet0;
     $55 = (_sqlite3OsRead($46,$aMagic,8,$53,$54)|0);
     HEAP32[$rc>>2] = $55;
     $56 = (0)!=($55|0);
     if (!($56)) {
      $57 = (_memcmp($aMagic,4384,8)|0);
      $58 = ($57|0)!=(0);
      if (!($58)) {
       $59 = HEAP32[$1>>2]|0;
       $60 = HEAP32[$2>>2]|0;
       $61 = HEAP32[$len>>2]|0;
       $62 = $szJ;
       $63 = $62;
       $64 = HEAP32[$63>>2]|0;
       $65 = (($62) + 4)|0;
       $66 = $65;
       $67 = HEAP32[$66>>2]|0;
       $68 = (_i64Subtract(($64|0),($67|0),16,0)|0);
       $69 = tempRet0;
       $70 = HEAP32[$len>>2]|0;
       $71 = (_i64Subtract(($68|0),($69|0),($70|0),0)|0);
       $72 = tempRet0;
       $73 = (_sqlite3OsRead($59,$60,$61,$71,$72)|0);
       HEAP32[$rc>>2] = $73;
       $74 = (0)!=($73|0);
       if (!($74)) {
        HEAP32[$u>>2] = 0;
        while(1) {
         $76 = HEAP32[$u>>2]|0;
         $77 = HEAP32[$len>>2]|0;
         $78 = ($76>>>0)<($77>>>0);
         if (!($78)) {
          break;
         }
         $79 = HEAP32[$u>>2]|0;
         $80 = HEAP32[$2>>2]|0;
         $81 = (($80) + ($79)|0);
         $82 = HEAP8[$81>>0]|0;
         $83 = $82 << 24 >> 24;
         $84 = HEAP32[$cksum>>2]|0;
         $85 = (($84) - ($83))|0;
         HEAP32[$cksum>>2] = $85;
         $86 = HEAP32[$u>>2]|0;
         $87 = (($86) + 1)|0;
         HEAP32[$u>>2] = $87;
        }
        $88 = HEAP32[$cksum>>2]|0;
        $89 = ($88|0)!=(0);
        if ($89) {
         HEAP32[$len>>2] = 0;
        }
        $90 = HEAP32[$len>>2]|0;
        $91 = HEAP32[$2>>2]|0;
        $92 = (($91) + ($90)|0);
        HEAP8[$92>>0] = 0;
        HEAP32[$0>>2] = 0;
        $93 = HEAP32[$0>>2]|0;
        STACKTOP = sp;return ($93|0);
       }
      }
     }
    }
   }
  }
 }
 $75 = HEAP32[$rc>>2]|0;
 HEAP32[$0>>2] = $75;
 $93 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($93|0);
}
function _sqlite3OsAccess($pVfs,$zPath,$flags,$pResOut) {
 $pVfs = $pVfs|0;
 $zPath = $zPath|0;
 $flags = $flags|0;
 $pResOut = $pResOut|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $1 = sp + 8|0;
 $2 = sp + 4|0;
 $3 = sp;
 HEAP32[$0>>2] = $pVfs;
 HEAP32[$1>>2] = $zPath;
 HEAP32[$2>>2] = $flags;
 HEAP32[$3>>2] = $pResOut;
 $4 = HEAP32[$0>>2]|0;
 $5 = (($4) + 32|0);
 $6 = HEAP32[$5>>2]|0;
 $7 = HEAP32[$0>>2]|0;
 $8 = HEAP32[$1>>2]|0;
 $9 = HEAP32[$2>>2]|0;
 $10 = HEAP32[$3>>2]|0;
 $11 = (FUNCTION_TABLE_iiiii[$6 & 7]($7,$8,$9,$10)|0);
 STACKTOP = sp;return ($11|0);
}
function _readJournalHdr($pPager,$isHot,$0,$1,$pNRec,$pDbSize) {
 $pPager = $pPager|0;
 $isHot = $isHot|0;
 $0 = $0|0;
 $1 = $1|0;
 $pNRec = $pNRec|0;
 $pDbSize = $pDbSize|0;
 var $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0, $116 = 0, $117 = 0;
 var $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0, $134 = 0, $135 = 0;
 var $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0, $152 = 0, $153 = 0;
 var $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0, $170 = 0, $171 = 0;
 var $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0, $188 = 0, $189 = 0, $19 = 0;
 var $190 = 0, $191 = 0, $192 = 0, $193 = 0, $194 = 0, $195 = 0, $196 = 0, $197 = 0, $198 = 0, $199 = 0, $2 = 0, $20 = 0, $200 = 0, $201 = 0, $202 = 0, $203 = 0, $204 = 0, $205 = 0, $206 = 0, $207 = 0;
 var $208 = 0, $209 = 0, $21 = 0, $210 = 0, $211 = 0, $212 = 0, $213 = 0, $214 = 0, $215 = 0, $216 = 0, $217 = 0, $218 = 0, $219 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0;
 var $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0;
 var $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0;
 var $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0;
 var $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $aMagic = 0, $iHdrOff = 0;
 var $iPageSize = 0, $iSectorSize = 0, $or$cond = 0, $or$cond3 = 0, $or$cond5 = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 64|0;
 $2 = sp + 44|0;
 $3 = sp + 40|0;
 $4 = sp + 36|0;
 $5 = sp + 8|0;
 $6 = sp + 32|0;
 $7 = sp + 28|0;
 $rc = sp + 24|0;
 $aMagic = sp + 48|0;
 $iHdrOff = sp;
 $iPageSize = sp + 20|0;
 $iSectorSize = sp + 16|0;
 HEAP32[$3>>2] = $pPager;
 HEAP32[$4>>2] = $isHot;
 $8 = $5;
 $9 = $8;
 HEAP32[$9>>2] = $0;
 $10 = (($8) + 4)|0;
 $11 = $10;
 HEAP32[$11>>2] = $1;
 HEAP32[$6>>2] = $pNRec;
 HEAP32[$7>>2] = $pDbSize;
 $12 = HEAP32[$3>>2]|0;
 $13 = (_journalHdrOffset($12)|0);
 $14 = tempRet0;
 $15 = HEAP32[$3>>2]|0;
 $16 = (($15) + 72|0);
 $17 = $16;
 $18 = $17;
 HEAP32[$18>>2] = $13;
 $19 = (($17) + 4)|0;
 $20 = $19;
 HEAP32[$20>>2] = $14;
 $21 = HEAP32[$3>>2]|0;
 $22 = (($21) + 72|0);
 $23 = $22;
 $24 = $23;
 $25 = HEAP32[$24>>2]|0;
 $26 = (($23) + 4)|0;
 $27 = $26;
 $28 = HEAP32[$27>>2]|0;
 $29 = HEAP32[$3>>2]|0;
 $30 = (($29) + 148|0);
 $31 = HEAP32[$30>>2]|0;
 $32 = (_i64Add(($25|0),($28|0),($31|0),0)|0);
 $33 = tempRet0;
 $34 = $5;
 $35 = $34;
 $36 = HEAP32[$35>>2]|0;
 $37 = (($34) + 4)|0;
 $38 = $37;
 $39 = HEAP32[$38>>2]|0;
 $40 = ($33|0)>($39|0);
 $41 = ($32>>>0)>($36>>>0);
 $42 = ($33|0)==($39|0);
 $43 = $42 & $41;
 $44 = $40 | $43;
 if ($44) {
  HEAP32[$2>>2] = 101;
  $219 = HEAP32[$2>>2]|0;
  STACKTOP = sp;return ($219|0);
 }
 $45 = HEAP32[$3>>2]|0;
 $46 = (($45) + 72|0);
 $47 = $46;
 $48 = $47;
 $49 = HEAP32[$48>>2]|0;
 $50 = (($47) + 4)|0;
 $51 = $50;
 $52 = HEAP32[$51>>2]|0;
 $53 = $iHdrOff;
 $54 = $53;
 HEAP32[$54>>2] = $49;
 $55 = (($53) + 4)|0;
 $56 = $55;
 HEAP32[$56>>2] = $52;
 $57 = HEAP32[$4>>2]|0;
 $58 = ($57|0)!=(0);
 if ($58) {
  label = 5;
 } else {
  $59 = $iHdrOff;
  $60 = $59;
  $61 = HEAP32[$60>>2]|0;
  $62 = (($59) + 4)|0;
  $63 = $62;
  $64 = HEAP32[$63>>2]|0;
  $65 = HEAP32[$3>>2]|0;
  $66 = (($65) + 80|0);
  $67 = $66;
  $68 = $67;
  $69 = HEAP32[$68>>2]|0;
  $70 = (($67) + 4)|0;
  $71 = $70;
  $72 = HEAP32[$71>>2]|0;
  $73 = ($61|0)!=($69|0);
  $74 = ($64|0)!=($72|0);
  $75 = $73 | $74;
  if ($75) {
   label = 5;
  }
 }
 if ((label|0) == 5) {
  $76 = HEAP32[$3>>2]|0;
  $77 = (($76) + 64|0);
  $78 = HEAP32[$77>>2]|0;
  $79 = $iHdrOff;
  $80 = $79;
  $81 = HEAP32[$80>>2]|0;
  $82 = (($79) + 4)|0;
  $83 = $82;
  $84 = HEAP32[$83>>2]|0;
  $85 = (_sqlite3OsRead($78,$aMagic,8,$81,$84)|0);
  HEAP32[$rc>>2] = $85;
  $86 = HEAP32[$rc>>2]|0;
  $87 = ($86|0)!=(0);
  if ($87) {
   $88 = HEAP32[$rc>>2]|0;
   HEAP32[$2>>2] = $88;
   $219 = HEAP32[$2>>2]|0;
   STACKTOP = sp;return ($219|0);
  }
  $89 = (_memcmp($aMagic,4384,8)|0);
  $90 = ($89|0)!=(0);
  if ($90) {
   HEAP32[$2>>2] = 101;
   $219 = HEAP32[$2>>2]|0;
   STACKTOP = sp;return ($219|0);
  }
 }
 $91 = HEAP32[$3>>2]|0;
 $92 = (($91) + 64|0);
 $93 = HEAP32[$92>>2]|0;
 $94 = $iHdrOff;
 $95 = $94;
 $96 = HEAP32[$95>>2]|0;
 $97 = (($94) + 4)|0;
 $98 = $97;
 $99 = HEAP32[$98>>2]|0;
 $100 = (_i64Add(($96|0),($99|0),8,0)|0);
 $101 = tempRet0;
 $102 = HEAP32[$6>>2]|0;
 $103 = (_read32bits($93,$100,$101,$102)|0);
 HEAP32[$rc>>2] = $103;
 $104 = (0)!=($103|0);
 if (!($104)) {
  $105 = HEAP32[$3>>2]|0;
  $106 = (($105) + 64|0);
  $107 = HEAP32[$106>>2]|0;
  $108 = $iHdrOff;
  $109 = $108;
  $110 = HEAP32[$109>>2]|0;
  $111 = (($108) + 4)|0;
  $112 = $111;
  $113 = HEAP32[$112>>2]|0;
  $114 = (_i64Add(($110|0),($113|0),12,0)|0);
  $115 = tempRet0;
  $116 = HEAP32[$3>>2]|0;
  $117 = (($116) + 48|0);
  $118 = (_read32bits($107,$114,$115,$117)|0);
  HEAP32[$rc>>2] = $118;
  $119 = (0)!=($118|0);
  if (!($119)) {
   $120 = HEAP32[$3>>2]|0;
   $121 = (($120) + 64|0);
   $122 = HEAP32[$121>>2]|0;
   $123 = $iHdrOff;
   $124 = $123;
   $125 = HEAP32[$124>>2]|0;
   $126 = (($123) + 4)|0;
   $127 = $126;
   $128 = HEAP32[$127>>2]|0;
   $129 = (_i64Add(($125|0),($128|0),16,0)|0);
   $130 = tempRet0;
   $131 = HEAP32[$7>>2]|0;
   $132 = (_read32bits($122,$129,$130,$131)|0);
   HEAP32[$rc>>2] = $132;
   $133 = (0)!=($132|0);
   if (!($133)) {
    $135 = HEAP32[$3>>2]|0;
    $136 = (($135) + 72|0);
    $137 = $136;
    $138 = $137;
    $139 = HEAP32[$138>>2]|0;
    $140 = (($137) + 4)|0;
    $141 = $140;
    $142 = HEAP32[$141>>2]|0;
    $143 = ($139|0)==(0);
    $144 = ($142|0)==(0);
    $145 = $143 & $144;
    do {
     if ($145) {
      $146 = HEAP32[$3>>2]|0;
      $147 = (($146) + 64|0);
      $148 = HEAP32[$147>>2]|0;
      $149 = $iHdrOff;
      $150 = $149;
      $151 = HEAP32[$150>>2]|0;
      $152 = (($149) + 4)|0;
      $153 = $152;
      $154 = HEAP32[$153>>2]|0;
      $155 = (_i64Add(($151|0),($154|0),20,0)|0);
      $156 = tempRet0;
      $157 = (_read32bits($148,$155,$156,$iSectorSize)|0);
      HEAP32[$rc>>2] = $157;
      $158 = (0)!=($157|0);
      if (!($158)) {
       $159 = HEAP32[$3>>2]|0;
       $160 = (($159) + 64|0);
       $161 = HEAP32[$160>>2]|0;
       $162 = $iHdrOff;
       $163 = $162;
       $164 = HEAP32[$163>>2]|0;
       $165 = (($162) + 4)|0;
       $166 = $165;
       $167 = HEAP32[$166>>2]|0;
       $168 = (_i64Add(($164|0),($167|0),24,0)|0);
       $169 = tempRet0;
       $170 = (_read32bits($161,$168,$169,$iPageSize)|0);
       HEAP32[$rc>>2] = $170;
       $171 = (0)!=($170|0);
       if (!($171)) {
        $173 = HEAP32[$iPageSize>>2]|0;
        $174 = ($173|0)==(0);
        if ($174) {
         $175 = HEAP32[$3>>2]|0;
         $176 = (($175) + 152|0);
         $177 = HEAP32[$176>>2]|0;
         HEAP32[$iPageSize>>2] = $177;
        }
        $178 = HEAP32[$iPageSize>>2]|0;
        $179 = ($178>>>0)<(512);
        $180 = HEAP32[$iSectorSize>>2]|0;
        $181 = ($180>>>0)<(32);
        $or$cond = $179 | $181;
        $182 = HEAP32[$iPageSize>>2]|0;
        $183 = ($182>>>0)>(65536);
        $or$cond3 = $or$cond | $183;
        $184 = HEAP32[$iSectorSize>>2]|0;
        $185 = ($184>>>0)>(65536);
        $or$cond5 = $or$cond3 | $185;
        if (!($or$cond5)) {
         $186 = HEAP32[$iPageSize>>2]|0;
         $187 = (($186) - 1)|0;
         $188 = HEAP32[$iPageSize>>2]|0;
         $189 = $187 & $188;
         $190 = ($189|0)!=(0);
         if (!($190)) {
          $191 = HEAP32[$iSectorSize>>2]|0;
          $192 = (($191) - 1)|0;
          $193 = HEAP32[$iSectorSize>>2]|0;
          $194 = $192 & $193;
          $195 = ($194|0)!=(0);
          if (!($195)) {
           $196 = HEAP32[$3>>2]|0;
           $197 = (_sqlite3PagerSetPagesize($196,$iPageSize,-1)|0);
           HEAP32[$rc>>2] = $197;
           $198 = HEAP32[$iSectorSize>>2]|0;
           $199 = HEAP32[$3>>2]|0;
           $200 = (($199) + 148|0);
           HEAP32[$200>>2] = $198;
           break;
          }
         }
        }
        HEAP32[$2>>2] = 101;
        $219 = HEAP32[$2>>2]|0;
        STACKTOP = sp;return ($219|0);
       }
      }
      $172 = HEAP32[$rc>>2]|0;
      HEAP32[$2>>2] = $172;
      $219 = HEAP32[$2>>2]|0;
      STACKTOP = sp;return ($219|0);
     }
    } while(0);
    $201 = HEAP32[$3>>2]|0;
    $202 = (($201) + 148|0);
    $203 = HEAP32[$202>>2]|0;
    $204 = HEAP32[$3>>2]|0;
    $205 = (($204) + 72|0);
    $206 = $205;
    $207 = $206;
    $208 = HEAP32[$207>>2]|0;
    $209 = (($206) + 4)|0;
    $210 = $209;
    $211 = HEAP32[$210>>2]|0;
    $212 = (_i64Add(($208|0),($211|0),($203|0),0)|0);
    $213 = tempRet0;
    $214 = $205;
    $215 = $214;
    HEAP32[$215>>2] = $212;
    $216 = (($214) + 4)|0;
    $217 = $216;
    HEAP32[$217>>2] = $213;
    $218 = HEAP32[$rc>>2]|0;
    HEAP32[$2>>2] = $218;
    $219 = HEAP32[$2>>2]|0;
    STACKTOP = sp;return ($219|0);
   }
  }
 }
 $134 = HEAP32[$rc>>2]|0;
 HEAP32[$2>>2] = $134;
 $219 = HEAP32[$2>>2]|0;
 STACKTOP = sp;return ($219|0);
}
function _pager_playback_one_page($pPager,$pOffset,$pDone,$isMainJrnl,$isSavepnt) {
 $pPager = $pPager|0;
 $pOffset = $pOffset|0;
 $pDone = $pDone|0;
 $isMainJrnl = $isMainJrnl|0;
 $isSavepnt = $isSavepnt|0;
 var $$old = 0, $$old1 = 0, $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0;
 var $114 = 0, $115 = 0, $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0;
 var $132 = 0, $133 = 0, $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0;
 var $150 = 0, $151 = 0, $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0;
 var $169 = 0, $17 = 0, $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0;
 var $187 = 0, $188 = 0, $189 = 0, $19 = 0, $190 = 0, $191 = 0, $192 = 0, $193 = 0, $194 = 0, $195 = 0, $196 = 0, $197 = 0, $198 = 0, $199 = 0, $2 = 0, $20 = 0, $200 = 0, $201 = 0, $202 = 0, $203 = 0;
 var $204 = 0, $205 = 0, $206 = 0, $207 = 0, $208 = 0, $209 = 0, $21 = 0, $210 = 0, $211 = 0, $212 = 0, $213 = 0, $214 = 0, $215 = 0, $216 = 0, $217 = 0, $218 = 0, $219 = 0, $22 = 0, $220 = 0, $221 = 0;
 var $222 = 0, $223 = 0, $224 = 0, $225 = 0, $226 = 0, $227 = 0, $228 = 0, $229 = 0, $23 = 0, $230 = 0, $231 = 0, $232 = 0, $233 = 0, $234 = 0, $235 = 0, $236 = 0, $237 = 0, $238 = 0, $239 = 0, $24 = 0;
 var $240 = 0, $241 = 0, $242 = 0, $243 = 0, $244 = 0, $245 = 0, $246 = 0, $247 = 0, $248 = 0, $249 = 0, $25 = 0, $250 = 0, $251 = 0, $252 = 0, $253 = 0, $254 = 0, $255 = 0, $256 = 0, $257 = 0, $258 = 0;
 var $259 = 0, $26 = 0, $260 = 0, $261 = 0, $262 = 0, $263 = 0, $264 = 0, $265 = 0, $266 = 0, $267 = 0, $268 = 0, $269 = 0, $27 = 0, $270 = 0, $271 = 0, $272 = 0, $273 = 0, $274 = 0, $275 = 0, $276 = 0;
 var $277 = 0, $278 = 0, $279 = 0, $28 = 0, $280 = 0, $281 = 0, $282 = 0, $283 = 0, $284 = 0, $285 = 0, $286 = 0, $287 = 0, $288 = 0, $289 = 0, $29 = 0, $290 = 0, $291 = 0, $292 = 0, $293 = 0, $294 = 0;
 var $295 = 0, $296 = 0, $297 = 0, $298 = 0, $299 = 0, $3 = 0, $30 = 0, $300 = 0, $301 = 0, $302 = 0, $303 = 0, $304 = 0, $305 = 0, $306 = 0, $307 = 0, $308 = 0, $309 = 0, $31 = 0, $310 = 0, $311 = 0;
 var $312 = 0, $313 = 0, $314 = 0, $315 = 0, $316 = 0, $317 = 0, $318 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0;
 var $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0;
 var $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0;
 var $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0;
 var $99 = 0, $aData = 0, $cksum = 0, $isSynced = 0, $jfd = 0, $ofst = 0, $or$cond = 0, $or$cond4 = 0, $pData = 0, $pPg = 0, $pgno = 0, $rc = 0, dest = 0, label = 0, sp = 0, src = 0, stop = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 64|0;
 $0 = sp + 60|0;
 $1 = sp + 56|0;
 $2 = sp + 52|0;
 $3 = sp + 48|0;
 $4 = sp + 44|0;
 $5 = sp + 40|0;
 $rc = sp + 36|0;
 $pPg = sp + 32|0;
 $pgno = sp + 28|0;
 $cksum = sp + 24|0;
 $aData = sp + 20|0;
 $jfd = sp + 16|0;
 $isSynced = sp + 12|0;
 $ofst = sp;
 $pData = sp + 8|0;
 HEAP32[$1>>2] = $pPager;
 HEAP32[$2>>2] = $pOffset;
 HEAP32[$3>>2] = $pDone;
 HEAP32[$4>>2] = $isMainJrnl;
 HEAP32[$5>>2] = $isSavepnt;
 $6 = HEAP32[$1>>2]|0;
 $7 = (($6) + 200|0);
 $8 = HEAP32[$7>>2]|0;
 HEAP32[$aData>>2] = $8;
 $9 = HEAP32[$4>>2]|0;
 $10 = ($9|0)!=(0);
 $11 = HEAP32[$1>>2]|0;
 if ($10) {
  $12 = (($11) + 64|0);
  $13 = HEAP32[$12>>2]|0;
  $16 = $13;
 } else {
  $14 = (($11) + 68|0);
  $15 = HEAP32[$14>>2]|0;
  $16 = $15;
 }
 HEAP32[$jfd>>2] = $16;
 $17 = HEAP32[$jfd>>2]|0;
 $18 = HEAP32[$2>>2]|0;
 $19 = $18;
 $20 = $19;
 $21 = HEAP32[$20>>2]|0;
 $22 = (($19) + 4)|0;
 $23 = $22;
 $24 = HEAP32[$23>>2]|0;
 $25 = (_read32bits($17,$21,$24,$pgno)|0);
 HEAP32[$rc>>2] = $25;
 $26 = HEAP32[$rc>>2]|0;
 $27 = ($26|0)!=(0);
 if ($27) {
  $28 = HEAP32[$rc>>2]|0;
  HEAP32[$0>>2] = $28;
  $318 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($318|0);
 }
 $29 = HEAP32[$jfd>>2]|0;
 $30 = HEAP32[$aData>>2]|0;
 $31 = HEAP32[$1>>2]|0;
 $32 = (($31) + 152|0);
 $33 = HEAP32[$32>>2]|0;
 $34 = HEAP32[$2>>2]|0;
 $35 = $34;
 $36 = $35;
 $37 = HEAP32[$36>>2]|0;
 $38 = (($35) + 4)|0;
 $39 = $38;
 $40 = HEAP32[$39>>2]|0;
 $41 = (_i64Add(($37|0),($40|0),4,0)|0);
 $42 = tempRet0;
 $43 = (_sqlite3OsRead($29,$30,$33,$41,$42)|0);
 HEAP32[$rc>>2] = $43;
 $44 = HEAP32[$rc>>2]|0;
 $45 = ($44|0)!=(0);
 if ($45) {
  $46 = HEAP32[$rc>>2]|0;
  HEAP32[$0>>2] = $46;
  $318 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($318|0);
 }
 $47 = HEAP32[$1>>2]|0;
 $48 = (($47) + 152|0);
 $49 = HEAP32[$48>>2]|0;
 $50 = (($49) + 4)|0;
 $51 = HEAP32[$4>>2]|0;
 $52 = $51<<2;
 $53 = (($50) + ($52))|0;
 $54 = ($53|0)<(0);
 $55 = $54 << 31 >> 31;
 $56 = HEAP32[$2>>2]|0;
 $57 = $56;
 $58 = $57;
 $59 = HEAP32[$58>>2]|0;
 $60 = (($57) + 4)|0;
 $61 = $60;
 $62 = HEAP32[$61>>2]|0;
 $63 = (_i64Add(($59|0),($62|0),($53|0),($55|0))|0);
 $64 = tempRet0;
 $65 = $56;
 $66 = $65;
 HEAP32[$66>>2] = $63;
 $67 = (($65) + 4)|0;
 $68 = $67;
 HEAP32[$68>>2] = $64;
 $69 = HEAP32[$pgno>>2]|0;
 $70 = ($69|0)==(0);
 if (!($70)) {
  $71 = HEAP32[$pgno>>2]|0;
  $72 = HEAP32[1128>>2]|0;
  $73 = HEAP32[$1>>2]|0;
  $74 = (($73) + 152|0);
  $75 = HEAP32[$74>>2]|0;
  $76 = (($72|0) / ($75|0))&-1;
  $77 = (($76) + 1)|0;
  $78 = ($71|0)==($77|0);
  if (!($78)) {
   $79 = HEAP32[$pgno>>2]|0;
   $80 = HEAP32[$1>>2]|0;
   $81 = (($80) + 24|0);
   $82 = HEAP32[$81>>2]|0;
   $83 = ($79>>>0)>($82>>>0);
   if (!($83)) {
    $84 = HEAP32[$3>>2]|0;
    $85 = HEAP32[$pgno>>2]|0;
    $86 = (_sqlite3BitvecTest($84,$85)|0);
    $87 = ($86|0)!=(0);
    if (!($87)) {
     $88 = HEAP32[$4>>2]|0;
     $89 = ($88|0)!=(0);
     if ($89) {
      $90 = HEAP32[$jfd>>2]|0;
      $91 = HEAP32[$2>>2]|0;
      $92 = $91;
      $93 = $92;
      $94 = HEAP32[$93>>2]|0;
      $95 = (($92) + 4)|0;
      $96 = $95;
      $97 = HEAP32[$96>>2]|0;
      $98 = (_i64Subtract(($94|0),($97|0),4,0)|0);
      $99 = tempRet0;
      $100 = (_read32bits($90,$98,$99,$cksum)|0);
      HEAP32[$rc>>2] = $100;
      $101 = HEAP32[$rc>>2]|0;
      $102 = ($101|0)!=(0);
      if ($102) {
       $103 = HEAP32[$rc>>2]|0;
       HEAP32[$0>>2] = $103;
       $318 = HEAP32[$0>>2]|0;
       STACKTOP = sp;return ($318|0);
      }
      $104 = HEAP32[$5>>2]|0;
      $105 = ($104|0)!=(0);
      if (!($105)) {
       $106 = HEAP32[$1>>2]|0;
       $107 = HEAP32[$aData>>2]|0;
       $108 = (_pager_cksum($106,$107)|0);
       $109 = HEAP32[$cksum>>2]|0;
       $110 = ($108|0)!=($109|0);
       if ($110) {
        HEAP32[$0>>2] = 101;
        $318 = HEAP32[$0>>2]|0;
        STACKTOP = sp;return ($318|0);
       }
      }
     }
     $111 = HEAP32[$3>>2]|0;
     $112 = ($111|0)!=(0|0);
     if ($112) {
      $113 = HEAP32[$3>>2]|0;
      $114 = HEAP32[$pgno>>2]|0;
      $115 = (_sqlite3BitvecSet($113,$114)|0);
      HEAP32[$rc>>2] = $115;
      $116 = ($115|0)!=(0);
      if ($116) {
       $117 = HEAP32[$rc>>2]|0;
       HEAP32[$0>>2] = $117;
       $318 = HEAP32[$0>>2]|0;
       STACKTOP = sp;return ($318|0);
      }
     }
     $118 = HEAP32[$pgno>>2]|0;
     $119 = ($118|0)==(1);
     if ($119) {
      $120 = HEAP32[$1>>2]|0;
      $121 = (($120) + 142|0);
      $122 = HEAP16[$121>>1]|0;
      $123 = $122 << 16 >> 16;
      $124 = HEAP32[$aData>>2]|0;
      $125 = (($124) + 20|0);
      $126 = HEAP8[$125>>0]|0;
      $127 = $126&255;
      $128 = ($123|0)!=($127|0);
      if ($128) {
       $129 = HEAP32[$aData>>2]|0;
       $130 = (($129) + 20|0);
       $131 = HEAP8[$130>>0]|0;
       $132 = $131&255;
       $133 = HEAP32[$1>>2]|0;
       $134 = (($133) + 142|0);
       HEAP16[$134>>1] = $132;
      }
     }
     $135 = HEAP32[$1>>2]|0;
     $136 = (_pagerUseWal($135)|0);
     $137 = ($136|0)!=(0);
     if ($137) {
      HEAP32[$pPg>>2] = 0;
     } else {
      $138 = HEAP32[$1>>2]|0;
      $139 = HEAP32[$pgno>>2]|0;
      $140 = (_sqlite3PagerLookup($138,$139)|0);
      HEAP32[$pPg>>2] = $140;
     }
     $141 = HEAP32[$4>>2]|0;
     $142 = ($141|0)!=(0);
     if ($142) {
      $143 = HEAP32[$1>>2]|0;
      $144 = (($143) + 7|0);
      $145 = HEAP8[$144>>0]|0;
      $146 = $145&255;
      $147 = ($146|0)!=(0);
      if ($147) {
       $169 = 1;
      } else {
       $148 = HEAP32[$2>>2]|0;
       $149 = $148;
       $150 = $149;
       $151 = HEAP32[$150>>2]|0;
       $152 = (($149) + 4)|0;
       $153 = $152;
       $154 = HEAP32[$153>>2]|0;
       $155 = HEAP32[$1>>2]|0;
       $156 = (($155) + 80|0);
       $157 = $156;
       $158 = $157;
       $159 = HEAP32[$158>>2]|0;
       $160 = (($157) + 4)|0;
       $161 = $160;
       $162 = HEAP32[$161>>2]|0;
       $163 = ($154|0)<($162|0);
       $164 = ($151>>>0)<=($159>>>0);
       $165 = ($154|0)==($162|0);
       $166 = $165 & $164;
       $167 = $163 | $166;
       $169 = $167;
      }
      $168 = $169&1;
      HEAP32[$isSynced>>2] = $168;
     } else {
      $170 = HEAP32[$pPg>>2]|0;
      $171 = ($170|0)==(0|0);
      if ($171) {
       $179 = 1;
      } else {
       $172 = HEAP32[$pPg>>2]|0;
       $173 = (($172) + 24|0);
       $174 = HEAP16[$173>>1]|0;
       $175 = $174&65535;
       $176 = $175 & 4;
       $177 = (0)==($176|0);
       $179 = $177;
      }
      $178 = $179&1;
      HEAP32[$isSynced>>2] = $178;
     }
     $180 = HEAP32[$1>>2]|0;
     $181 = (($180) + 60|0);
     $182 = HEAP32[$181>>2]|0;
     $183 = HEAP32[$182>>2]|0;
     $184 = ($183|0)!=(0|0);
     do {
      if ($184) {
       $185 = HEAP32[$1>>2]|0;
       $186 = (($185) + 16|0);
       $187 = HEAP8[$186>>0]|0;
       $188 = $187&255;
       $189 = ($188|0)>=(4);
       if ($189) {
        $$old = HEAP32[$isSynced>>2]|0;
        $$old1 = ($$old|0)!=(0);
        if (!($$old1)) {
         label = 44;
         break;
        }
       } else {
        $190 = HEAP32[$1>>2]|0;
        $191 = (($190) + 16|0);
        $192 = HEAP8[$191>>0]|0;
        $193 = $192&255;
        $194 = ($193|0)==(0);
        $195 = HEAP32[$isSynced>>2]|0;
        $196 = ($195|0)!=(0);
        $or$cond = $194 & $196;
        if (!($or$cond)) {
         label = 44;
         break;
        }
       }
       $197 = HEAP32[$pgno>>2]|0;
       $198 = (($197) - 1)|0;
       $199 = HEAP32[$1>>2]|0;
       $200 = (($199) + 152|0);
       $201 = HEAP32[$200>>2]|0;
       $202 = ($201|0)<(0);
       $203 = $202 << 31 >> 31;
       $204 = (___muldi3(($198|0),0,($201|0),($203|0))|0);
       $205 = tempRet0;
       $206 = $ofst;
       $207 = $206;
       HEAP32[$207>>2] = $204;
       $208 = (($206) + 4)|0;
       $209 = $208;
       HEAP32[$209>>2] = $205;
       $210 = HEAP32[$1>>2]|0;
       $211 = (($210) + 60|0);
       $212 = HEAP32[$211>>2]|0;
       $213 = HEAP32[$aData>>2]|0;
       $214 = HEAP32[$1>>2]|0;
       $215 = (($214) + 152|0);
       $216 = HEAP32[$215>>2]|0;
       $217 = $ofst;
       $218 = $217;
       $219 = HEAP32[$218>>2]|0;
       $220 = (($217) + 4)|0;
       $221 = $220;
       $222 = HEAP32[$221>>2]|0;
       $223 = (_sqlite3OsWrite($212,$213,$216,$219,$222)|0);
       HEAP32[$rc>>2] = $223;
       $224 = HEAP32[$pgno>>2]|0;
       $225 = HEAP32[$1>>2]|0;
       $226 = (($225) + 32|0);
       $227 = HEAP32[$226>>2]|0;
       $228 = ($224>>>0)>($227>>>0);
       if ($228) {
        $229 = HEAP32[$pgno>>2]|0;
        $230 = HEAP32[$1>>2]|0;
        $231 = (($230) + 32|0);
        HEAP32[$231>>2] = $229;
       }
       $232 = HEAP32[$1>>2]|0;
       $233 = (($232) + 88|0);
       $234 = HEAP32[$233>>2]|0;
       $235 = ($234|0)!=(0|0);
       if ($235) {
        $236 = HEAP32[$1>>2]|0;
        $237 = (($236) + 88|0);
        $238 = HEAP32[$237>>2]|0;
        $239 = HEAP32[$pgno>>2]|0;
        $240 = HEAP32[$aData>>2]|0;
        _sqlite3BackupUpdate($238,$239,$240);
        $241 = HEAP32[$aData>>2]|0;
        HEAP32[$aData>>2] = $241;
       }
      } else {
       label = 44;
      }
     } while(0);
     do {
      if ((label|0) == 44) {
       $242 = HEAP32[$4>>2]|0;
       $243 = ($242|0)==(0);
       $244 = HEAP32[$pPg>>2]|0;
       $245 = ($244|0)==(0|0);
       $or$cond4 = $243 & $245;
       if ($or$cond4) {
        $246 = HEAP32[$1>>2]|0;
        $247 = (($246) + 20|0);
        $248 = HEAP8[$247>>0]|0;
        $249 = $248&255;
        $250 = $249 | 2;
        $251 = $250&255;
        HEAP8[$247>>0] = $251;
        $252 = HEAP32[$1>>2]|0;
        $253 = HEAP32[$pgno>>2]|0;
        $254 = (_sqlite3PagerAcquire($252,$253,$pPg,1)|0);
        HEAP32[$rc>>2] = $254;
        $255 = HEAP32[$1>>2]|0;
        $256 = (($255) + 20|0);
        $257 = HEAP8[$256>>0]|0;
        $258 = $257&255;
        $259 = $258 & -3;
        $260 = $259&255;
        HEAP8[$256>>0] = $260;
        $261 = HEAP32[$rc>>2]|0;
        $262 = ($261|0)!=(0);
        if (!($262)) {
         $264 = HEAP32[$pPg>>2]|0;
         $265 = (($264) + 24|0);
         $266 = HEAP16[$265>>1]|0;
         $267 = $266&65535;
         $268 = $267 & -9;
         $269 = $268&65535;
         HEAP16[$265>>1] = $269;
         $270 = HEAP32[$pPg>>2]|0;
         _sqlite3PcacheMakeDirty($270);
         break;
        }
        $263 = HEAP32[$rc>>2]|0;
        HEAP32[$0>>2] = $263;
        $318 = HEAP32[$0>>2]|0;
        STACKTOP = sp;return ($318|0);
       }
      }
     } while(0);
     $271 = HEAP32[$pPg>>2]|0;
     $272 = ($271|0)!=(0|0);
     if ($272) {
      $273 = HEAP32[$pPg>>2]|0;
      $274 = (($273) + 4|0);
      $275 = HEAP32[$274>>2]|0;
      HEAP32[$pData>>2] = $275;
      $276 = HEAP32[$pData>>2]|0;
      $277 = HEAP32[$aData>>2]|0;
      $278 = HEAP32[$1>>2]|0;
      $279 = (($278) + 152|0);
      $280 = HEAP32[$279>>2]|0;
      _memcpy(($276|0),($277|0),($280|0))|0;
      $281 = HEAP32[$1>>2]|0;
      $282 = (($281) + 196|0);
      $283 = HEAP32[$282>>2]|0;
      $284 = HEAP32[$pPg>>2]|0;
      FUNCTION_TABLE_vi[$283 & 31]($284);
      $285 = HEAP32[$4>>2]|0;
      $286 = ($285|0)!=(0);
      do {
       if ($286) {
        $287 = HEAP32[$5>>2]|0;
        $288 = ($287|0)!=(0);
        if ($288) {
         $289 = HEAP32[$2>>2]|0;
         $290 = $289;
         $291 = $290;
         $292 = HEAP32[$291>>2]|0;
         $293 = (($290) + 4)|0;
         $294 = $293;
         $295 = HEAP32[$294>>2]|0;
         $296 = HEAP32[$1>>2]|0;
         $297 = (($296) + 80|0);
         $298 = $297;
         $299 = $298;
         $300 = HEAP32[$299>>2]|0;
         $301 = (($298) + 4)|0;
         $302 = $301;
         $303 = HEAP32[$302>>2]|0;
         $304 = ($295|0)<($303|0);
         $305 = ($292>>>0)<=($300>>>0);
         $306 = ($295|0)==($303|0);
         $307 = $306 & $305;
         $308 = $304 | $307;
         if (!($308)) {
          break;
         }
        }
        $309 = HEAP32[$pPg>>2]|0;
        _sqlite3PcacheMakeClean($309);
       }
      } while(0);
      $310 = HEAP32[$pgno>>2]|0;
      $311 = ($310|0)==(1);
      if ($311) {
       $312 = HEAP32[$1>>2]|0;
       $313 = (($312) + 100|0);
       $314 = HEAP32[$pData>>2]|0;
       $315 = (($314) + 24|0);
       dest=$313+0|0; src=$315+0|0; stop=dest+16|0; do { HEAP8[dest>>0]=HEAP8[src>>0]|0; dest=dest+1|0; src=src+1|0; } while ((dest|0) < (stop|0));
      }
      $316 = HEAP32[$pPg>>2]|0;
      _sqlite3PcacheRelease($316);
     }
     $317 = HEAP32[$rc>>2]|0;
     HEAP32[$0>>2] = $317;
     $318 = HEAP32[$0>>2]|0;
     STACKTOP = sp;return ($318|0);
    }
   }
   HEAP32[$0>>2] = 0;
   $318 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($318|0);
  }
 }
 HEAP32[$0>>2] = 101;
 $318 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($318|0);
}
function _pager_delmaster($pPager,$zMaster) {
 $pPager = $pPager|0;
 $zMaster = $zMaster|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0;
 var $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0;
 var $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0;
 var $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0;
 var $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0;
 var $96 = 0, $97 = 0, $98 = 0, $99 = 0, $c = 0, $exists = 0, $flags = 0, $flags1 = 0, $nMasterJournal = 0, $nMasterPtr = 0, $pJournal = 0, $pMaster = 0, $pVfs = 0, $rc = 0, $zJournal = 0, $zMasterJournal = 0, $zMasterPtr = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 64|0;
 $0 = sp + 60|0;
 $1 = sp + 56|0;
 $pVfs = sp + 52|0;
 $rc = sp + 48|0;
 $pMaster = sp + 44|0;
 $pJournal = sp + 40|0;
 $zMasterJournal = sp + 36|0;
 $nMasterJournal = sp;
 $zJournal = sp + 32|0;
 $zMasterPtr = sp + 28|0;
 $nMasterPtr = sp + 24|0;
 $flags = sp + 20|0;
 $exists = sp + 16|0;
 $c = sp + 12|0;
 $flags1 = sp + 8|0;
 HEAP32[$0>>2] = $pPager;
 HEAP32[$1>>2] = $zMaster;
 $2 = HEAP32[$0>>2]|0;
 $3 = HEAP32[$2>>2]|0;
 HEAP32[$pVfs>>2] = $3;
 HEAP32[$zMasterJournal>>2] = 0;
 $4 = HEAP32[$pVfs>>2]|0;
 $5 = (($4) + 4|0);
 $6 = HEAP32[$5>>2]|0;
 $7 = $6<<1;
 $8 = ($7|0)<(0);
 $9 = $8 << 31 >> 31;
 $10 = (_sqlite3MallocZero($7,$9)|0);
 HEAP32[$pMaster>>2] = $10;
 $11 = HEAP32[$pMaster>>2]|0;
 $12 = HEAP32[$pVfs>>2]|0;
 $13 = (($12) + 4|0);
 $14 = HEAP32[$13>>2]|0;
 $15 = (($11) + ($14)|0);
 HEAP32[$pJournal>>2] = $15;
 $16 = HEAP32[$pMaster>>2]|0;
 $17 = ($16|0)!=(0|0);
 if ($17) {
  HEAP32[$flags>>2] = 16385;
  $18 = HEAP32[$pVfs>>2]|0;
  $19 = HEAP32[$1>>2]|0;
  $20 = HEAP32[$pMaster>>2]|0;
  $21 = (_sqlite3OsOpen($18,$19,$20,16385,0)|0);
  HEAP32[$rc>>2] = $21;
 } else {
  HEAP32[$rc>>2] = 7;
 }
 $22 = HEAP32[$rc>>2]|0;
 $23 = ($22|0)!=(0);
 L5: do {
  if (!($23)) {
   $24 = HEAP32[$pMaster>>2]|0;
   $25 = (_sqlite3OsFileSize($24,$nMasterJournal)|0);
   HEAP32[$rc>>2] = $25;
   $26 = HEAP32[$rc>>2]|0;
   $27 = ($26|0)!=(0);
   if (!($27)) {
    $28 = HEAP32[$pVfs>>2]|0;
    $29 = (($28) + 8|0);
    $30 = HEAP32[$29>>2]|0;
    $31 = (($30) + 1)|0;
    HEAP32[$nMasterPtr>>2] = $31;
    $32 = $nMasterJournal;
    $33 = $32;
    $34 = HEAP32[$33>>2]|0;
    $35 = (($32) + 4)|0;
    $36 = $35;
    $37 = HEAP32[$36>>2]|0;
    $38 = HEAP32[$nMasterPtr>>2]|0;
    $39 = ($38|0)<(0);
    $40 = $39 << 31 >> 31;
    $41 = (_i64Add(($34|0),($37|0),($38|0),($40|0))|0);
    $42 = tempRet0;
    $43 = (_i64Add(($41|0),($42|0),1,0)|0);
    $44 = tempRet0;
    $45 = (_sqlite3Malloc($43,$44)|0);
    HEAP32[$zMasterJournal>>2] = $45;
    $46 = HEAP32[$zMasterJournal>>2]|0;
    $47 = ($46|0)!=(0|0);
    if (!($47)) {
     HEAP32[$rc>>2] = 7;
     break;
    }
    $48 = $nMasterJournal;
    $49 = $48;
    $50 = HEAP32[$49>>2]|0;
    $51 = (($48) + 4)|0;
    $52 = $51;
    $53 = HEAP32[$52>>2]|0;
    $54 = (_i64Add(($50|0),($53|0),1,0)|0);
    $55 = tempRet0;
    $56 = HEAP32[$zMasterJournal>>2]|0;
    $57 = (($56) + ($54)|0);
    HEAP32[$zMasterPtr>>2] = $57;
    $58 = HEAP32[$pMaster>>2]|0;
    $59 = HEAP32[$zMasterJournal>>2]|0;
    $60 = $nMasterJournal;
    $61 = $60;
    $62 = HEAP32[$61>>2]|0;
    $63 = (($60) + 4)|0;
    $64 = $63;
    $65 = HEAP32[$64>>2]|0;
    $66 = (_sqlite3OsRead($58,$59,$62,0,0)|0);
    HEAP32[$rc>>2] = $66;
    $67 = HEAP32[$rc>>2]|0;
    $68 = ($67|0)!=(0);
    if (!($68)) {
     $69 = $nMasterJournal;
     $70 = $69;
     $71 = HEAP32[$70>>2]|0;
     $72 = (($69) + 4)|0;
     $73 = $72;
     $74 = HEAP32[$73>>2]|0;
     $75 = HEAP32[$zMasterJournal>>2]|0;
     $76 = (($75) + ($71)|0);
     HEAP8[$76>>0] = 0;
     $77 = HEAP32[$zMasterJournal>>2]|0;
     HEAP32[$zJournal>>2] = $77;
     while(1) {
      $78 = HEAP32[$zJournal>>2]|0;
      $79 = HEAP32[$zMasterJournal>>2]|0;
      $80 = $78;
      $81 = $79;
      $82 = (($80) - ($81))|0;
      $83 = ($82|0)<(0);
      $84 = $83 << 31 >> 31;
      $85 = $nMasterJournal;
      $86 = $85;
      $87 = HEAP32[$86>>2]|0;
      $88 = (($85) + 4)|0;
      $89 = $88;
      $90 = HEAP32[$89>>2]|0;
      $91 = ($84|0)<($90|0);
      $92 = ($82>>>0)<($87>>>0);
      $93 = ($84|0)==($90|0);
      $94 = $93 & $92;
      $95 = $91 | $94;
      if (!($95)) {
       break;
      }
      $96 = HEAP32[$pVfs>>2]|0;
      $97 = HEAP32[$zJournal>>2]|0;
      $98 = (_sqlite3OsAccess($96,$97,0,$exists)|0);
      HEAP32[$rc>>2] = $98;
      $99 = HEAP32[$rc>>2]|0;
      $100 = ($99|0)!=(0);
      if ($100) {
       break L5;
      }
      $101 = HEAP32[$exists>>2]|0;
      $102 = ($101|0)!=(0);
      if ($102) {
       HEAP32[$flags1>>2] = 2049;
       $103 = HEAP32[$pVfs>>2]|0;
       $104 = HEAP32[$zJournal>>2]|0;
       $105 = HEAP32[$pJournal>>2]|0;
       $106 = HEAP32[$flags1>>2]|0;
       $107 = (_sqlite3OsOpen($103,$104,$105,$106,0)|0);
       HEAP32[$rc>>2] = $107;
       $108 = HEAP32[$rc>>2]|0;
       $109 = ($108|0)!=(0);
       if ($109) {
        break L5;
       }
       $110 = HEAP32[$pJournal>>2]|0;
       $111 = HEAP32[$zMasterPtr>>2]|0;
       $112 = HEAP32[$nMasterPtr>>2]|0;
       $113 = (_readMasterJournal($110,$111,$112)|0);
       HEAP32[$rc>>2] = $113;
       $114 = HEAP32[$pJournal>>2]|0;
       (_sqlite3OsClose($114)|0);
       $115 = HEAP32[$rc>>2]|0;
       $116 = ($115|0)!=(0);
       if ($116) {
        break L5;
       }
       $117 = HEAP32[$zMasterPtr>>2]|0;
       $118 = HEAP8[$117>>0]|0;
       $119 = $118 << 24 >> 24;
       $120 = ($119|0)!=(0);
       if ($120) {
        $121 = HEAP32[$zMasterPtr>>2]|0;
        $122 = HEAP32[$1>>2]|0;
        $123 = (_strcmp($121,$122)|0);
        $124 = ($123|0)==(0);
        $126 = $124;
       } else {
        $126 = 0;
       }
       $125 = $126&1;
       HEAP32[$c>>2] = $125;
       $127 = HEAP32[$c>>2]|0;
       $128 = ($127|0)!=(0);
       if ($128) {
        break L5;
       }
      }
      $129 = HEAP32[$zJournal>>2]|0;
      $130 = (_sqlite3Strlen30($129)|0);
      $131 = (($130) + 1)|0;
      $132 = HEAP32[$zJournal>>2]|0;
      $133 = (($132) + ($131)|0);
      HEAP32[$zJournal>>2] = $133;
     }
     $134 = HEAP32[$pMaster>>2]|0;
     (_sqlite3OsClose($134)|0);
     $135 = HEAP32[$pVfs>>2]|0;
     $136 = HEAP32[$1>>2]|0;
     $137 = (_sqlite3OsDelete($135,$136,0)|0);
     HEAP32[$rc>>2] = $137;
    }
   }
  }
 } while(0);
 $138 = HEAP32[$zMasterJournal>>2]|0;
 _sqlite3_free($138);
 $139 = HEAP32[$pMaster>>2]|0;
 $140 = ($139|0)!=(0|0);
 if (!($140)) {
  $143 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($143|0);
 }
 $141 = HEAP32[$pMaster>>2]|0;
 (_sqlite3OsClose($141)|0);
 $142 = HEAP32[$pMaster>>2]|0;
 _sqlite3_free($142);
 $143 = HEAP32[$rc>>2]|0;
 STACKTOP = sp;return ($143|0);
}
function _setSectorSize($pPager) {
 $pPager = $pPager|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0;
 var label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 HEAP32[$0>>2] = $pPager;
 $1 = HEAP32[$0>>2]|0;
 $2 = (($1) + 12|0);
 $3 = HEAP8[$2>>0]|0;
 $4 = $3&255;
 $5 = ($4|0)!=(0);
 if (!($5)) {
  $6 = HEAP32[$0>>2]|0;
  $7 = (($6) + 60|0);
  $8 = HEAP32[$7>>2]|0;
  $9 = (_sqlite3OsDeviceCharacteristics($8)|0);
  $10 = $9 & 4096;
  $11 = ($10|0)!=(0);
  if (!($11)) {
   $14 = HEAP32[$0>>2]|0;
   $15 = (($14) + 60|0);
   $16 = HEAP32[$15>>2]|0;
   $17 = (_sqlite3SectorSize($16)|0);
   $18 = HEAP32[$0>>2]|0;
   $19 = (($18) + 148|0);
   HEAP32[$19>>2] = $17;
   STACKTOP = sp;return;
  }
 }
 $12 = HEAP32[$0>>2]|0;
 $13 = (($12) + 148|0);
 HEAP32[$13>>2] = 512;
 STACKTOP = sp;return;
}
function _sqlite3OsDeviceCharacteristics($id) {
 $id = $id|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 HEAP32[$0>>2] = $id;
 $1 = HEAP32[$0>>2]|0;
 $2 = HEAP32[$1>>2]|0;
 $3 = (($2) + 48|0);
 $4 = HEAP32[$3>>2]|0;
 $5 = HEAP32[$0>>2]|0;
 $6 = (FUNCTION_TABLE_ii[$4 & 31]($5)|0);
 STACKTOP = sp;return ($6|0);
}
function _sqlite3SectorSize($pFile) {
 $pFile = $pFile|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $iRet = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $iRet = sp;
 HEAP32[$0>>2] = $pFile;
 $1 = HEAP32[$0>>2]|0;
 $2 = (_sqlite3OsSectorSize($1)|0);
 HEAP32[$iRet>>2] = $2;
 $3 = HEAP32[$iRet>>2]|0;
 $4 = ($3|0)<(32);
 if ($4) {
  HEAP32[$iRet>>2] = 512;
 } else {
  $5 = HEAP32[$iRet>>2]|0;
  $6 = ($5|0)>(65536);
  if ($6) {
   HEAP32[$iRet>>2] = 65536;
  }
 }
 $7 = HEAP32[$iRet>>2]|0;
 STACKTOP = sp;return ($7|0);
}
function _sqlite3OsSectorSize($id) {
 $id = $id|0;
 var $0 = 0, $1 = 0, $10 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $xSectorSize = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $xSectorSize = sp;
 HEAP32[$0>>2] = $id;
 $1 = HEAP32[$0>>2]|0;
 $2 = HEAP32[$1>>2]|0;
 $3 = (($2) + 44|0);
 $4 = HEAP32[$3>>2]|0;
 HEAP32[$xSectorSize>>2] = $4;
 $5 = HEAP32[$xSectorSize>>2]|0;
 $6 = ($5|0)!=(0|0);
 if (!($6)) {
  $10 = 4096;
  STACKTOP = sp;return ($10|0);
 }
 $7 = HEAP32[$xSectorSize>>2]|0;
 $8 = HEAP32[$0>>2]|0;
 $9 = (FUNCTION_TABLE_ii[$7 & 31]($8)|0);
 $10 = $9;
 STACKTOP = sp;return ($10|0);
}
function _sqlite3OsOpen($pVfs,$zPath,$pFile,$flags,$pFlagsOut) {
 $pVfs = $pVfs|0;
 $zPath = $zPath|0;
 $pFile = $pFile|0;
 $flags = $flags|0;
 $pFlagsOut = $pFlagsOut|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 20|0;
 $1 = sp + 16|0;
 $2 = sp + 12|0;
 $3 = sp + 8|0;
 $4 = sp + 4|0;
 $rc = sp;
 HEAP32[$0>>2] = $pVfs;
 HEAP32[$1>>2] = $zPath;
 HEAP32[$2>>2] = $pFile;
 HEAP32[$3>>2] = $flags;
 HEAP32[$4>>2] = $pFlagsOut;
 $5 = HEAP32[$0>>2]|0;
 $6 = (($5) + 24|0);
 $7 = HEAP32[$6>>2]|0;
 $8 = HEAP32[$0>>2]|0;
 $9 = HEAP32[$1>>2]|0;
 $10 = HEAP32[$2>>2]|0;
 $11 = HEAP32[$3>>2]|0;
 $12 = $11 & 556927;
 $13 = HEAP32[$4>>2]|0;
 $14 = (FUNCTION_TABLE_iiiiii[$7 & 15]($8,$9,$10,$12,$13)|0);
 HEAP32[$rc>>2] = $14;
 $15 = HEAP32[$rc>>2]|0;
 STACKTOP = sp;return ($15|0);
}
function _sqlite3OsRead($id,$pBuf,$amt,$0,$1) {
 $id = $id|0;
 $pBuf = $pBuf|0;
 $amt = $amt|0;
 $0 = $0|0;
 $1 = $1|0;
 var $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0;
 var $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $2 = sp + 16|0;
 $3 = sp + 12|0;
 $4 = sp + 8|0;
 $5 = sp;
 HEAP32[$2>>2] = $id;
 HEAP32[$3>>2] = $pBuf;
 HEAP32[$4>>2] = $amt;
 $6 = $5;
 $7 = $6;
 HEAP32[$7>>2] = $0;
 $8 = (($6) + 4)|0;
 $9 = $8;
 HEAP32[$9>>2] = $1;
 $10 = HEAP32[$2>>2]|0;
 $11 = HEAP32[$10>>2]|0;
 $12 = (($11) + 8|0);
 $13 = HEAP32[$12>>2]|0;
 $14 = HEAP32[$2>>2]|0;
 $15 = HEAP32[$3>>2]|0;
 $16 = HEAP32[$4>>2]|0;
 $17 = $5;
 $18 = $17;
 $19 = HEAP32[$18>>2]|0;
 $20 = (($17) + 4)|0;
 $21 = $20;
 $22 = HEAP32[$21>>2]|0;
 $23 = (FUNCTION_TABLE_iiiiii[$13 & 15]($14,$15,$16,$19,$22)|0);
 STACKTOP = sp;return ($23|0);
}
function _read32bits($fd,$0,$1,$pRes) {
 $fd = $fd|0;
 $0 = $0|0;
 $1 = $1|0;
 $pRes = $pRes|0;
 var $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0;
 var $ac = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $2 = sp + 16|0;
 $3 = sp;
 $4 = sp + 12|0;
 $ac = sp + 20|0;
 $rc = sp + 8|0;
 HEAP32[$2>>2] = $fd;
 $5 = $3;
 $6 = $5;
 HEAP32[$6>>2] = $0;
 $7 = (($5) + 4)|0;
 $8 = $7;
 HEAP32[$8>>2] = $1;
 HEAP32[$4>>2] = $pRes;
 $9 = HEAP32[$2>>2]|0;
 $10 = $3;
 $11 = $10;
 $12 = HEAP32[$11>>2]|0;
 $13 = (($10) + 4)|0;
 $14 = $13;
 $15 = HEAP32[$14>>2]|0;
 $16 = (_sqlite3OsRead($9,$ac,4,$12,$15)|0);
 HEAP32[$rc>>2] = $16;
 $17 = HEAP32[$rc>>2]|0;
 $18 = ($17|0)==(0);
 if (!($18)) {
  $21 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($21|0);
 }
 $19 = (_sqlite3Get4byte($ac)|0);
 $20 = HEAP32[$4>>2]|0;
 HEAP32[$20>>2] = $19;
 $21 = HEAP32[$rc>>2]|0;
 STACKTOP = sp;return ($21|0);
}
function _pager_cksum($pPager,$aData) {
 $pPager = $pPager|0;
 $aData = $aData|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0;
 var $9 = 0, $cksum = 0, $i = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $1 = sp + 8|0;
 $cksum = sp + 4|0;
 $i = sp;
 HEAP32[$0>>2] = $pPager;
 HEAP32[$1>>2] = $aData;
 $2 = HEAP32[$0>>2]|0;
 $3 = (($2) + 48|0);
 $4 = HEAP32[$3>>2]|0;
 HEAP32[$cksum>>2] = $4;
 $5 = HEAP32[$0>>2]|0;
 $6 = (($5) + 152|0);
 $7 = HEAP32[$6>>2]|0;
 $8 = (($7) - 200)|0;
 HEAP32[$i>>2] = $8;
 while(1) {
  $9 = HEAP32[$i>>2]|0;
  $10 = ($9|0)>(0);
  if (!($10)) {
   break;
  }
  $11 = HEAP32[$i>>2]|0;
  $12 = HEAP32[$1>>2]|0;
  $13 = (($12) + ($11)|0);
  $14 = HEAP8[$13>>0]|0;
  $15 = $14&255;
  $16 = HEAP32[$cksum>>2]|0;
  $17 = (($16) + ($15))|0;
  HEAP32[$cksum>>2] = $17;
  $18 = HEAP32[$i>>2]|0;
  $19 = (($18) - 200)|0;
  HEAP32[$i>>2] = $19;
 }
 $20 = HEAP32[$cksum>>2]|0;
 STACKTOP = sp;return ($20|0);
}
function _sqlite3PagerLookup($pPager,$pgno) {
 $pPager = $pPager|0;
 $pgno = $pgno|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $pPage = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $1 = sp + 4|0;
 $pPage = sp;
 HEAP32[$0>>2] = $pPager;
 HEAP32[$1>>2] = $pgno;
 $2 = HEAP32[$0>>2]|0;
 $3 = (($2) + 204|0);
 $4 = HEAP32[$3>>2]|0;
 $5 = HEAP32[$1>>2]|0;
 $6 = (_sqlite3PcacheFetch($4,$5,0)|0);
 HEAP32[$pPage>>2] = $6;
 $7 = HEAP32[$0>>2]|0;
 $8 = (($7) + 204|0);
 $9 = HEAP32[$8>>2]|0;
 $10 = HEAP32[$1>>2]|0;
 $11 = HEAP32[$pPage>>2]|0;
 $12 = (_sqlite3PcacheFetchFinish($9,$10,$11)|0);
 STACKTOP = sp;return ($12|0);
}
function _sqlite3BackupUpdate($pBackup,$iPage,$aData) {
 $pBackup = $pBackup|0;
 $iPage = $iPage|0;
 $aData = $aData|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $p = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 16|0;
 $1 = sp + 12|0;
 $2 = sp + 8|0;
 $p = sp + 4|0;
 $rc = sp;
 HEAP32[$0>>2] = $pBackup;
 HEAP32[$1>>2] = $iPage;
 HEAP32[$2>>2] = $aData;
 $3 = HEAP32[$0>>2]|0;
 HEAP32[$p>>2] = $3;
 while(1) {
  $4 = HEAP32[$p>>2]|0;
  $5 = ($4|0)!=(0|0);
  if (!($5)) {
   break;
  }
  $6 = HEAP32[$p>>2]|0;
  $7 = (($6) + 28|0);
  $8 = HEAP32[$7>>2]|0;
  $9 = (_isFatalError($8)|0);
  $10 = ($9|0)!=(0);
  if (!($10)) {
   $11 = HEAP32[$1>>2]|0;
   $12 = HEAP32[$p>>2]|0;
   $13 = (($12) + 16|0);
   $14 = HEAP32[$13>>2]|0;
   $15 = ($11>>>0)<($14>>>0);
   if ($15) {
    $16 = HEAP32[$p>>2]|0;
    $17 = HEAP32[$1>>2]|0;
    $18 = HEAP32[$2>>2]|0;
    $19 = (_backupOnePage($16,$17,$18,1)|0);
    HEAP32[$rc>>2] = $19;
    $20 = HEAP32[$rc>>2]|0;
    $21 = ($20|0)!=(0);
    if ($21) {
     $22 = HEAP32[$rc>>2]|0;
     $23 = HEAP32[$p>>2]|0;
     $24 = (($23) + 28|0);
     HEAP32[$24>>2] = $22;
    }
   }
  }
  $25 = HEAP32[$p>>2]|0;
  $26 = (($25) + 44|0);
  $27 = HEAP32[$26>>2]|0;
  HEAP32[$p>>2] = $27;
 }
 STACKTOP = sp;return;
}
function _sqlite3PcacheMakeDirty($p) {
 $p = $p|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0;
 var label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 HEAP32[$0>>2] = $p;
 $1 = HEAP32[$0>>2]|0;
 $2 = (($1) + 24|0);
 $3 = HEAP16[$2>>1]|0;
 $4 = $3&65535;
 $5 = $4 & -33;
 $6 = $5&65535;
 HEAP16[$2>>1] = $6;
 $7 = HEAP32[$0>>2]|0;
 $8 = (($7) + 24|0);
 $9 = HEAP16[$8>>1]|0;
 $10 = $9&65535;
 $11 = $10 & 2;
 $12 = (0)==($11|0);
 if (!($12)) {
  STACKTOP = sp;return;
 }
 $13 = HEAP32[$0>>2]|0;
 $14 = (($13) + 24|0);
 $15 = HEAP16[$14>>1]|0;
 $16 = $15&65535;
 $17 = $16 | 2;
 $18 = $17&65535;
 HEAP16[$14>>1] = $18;
 $19 = HEAP32[$0>>2]|0;
 _pcacheManageDirtyList($19,2);
 STACKTOP = sp;return;
}
function _sqlite3PcacheRelease($p) {
 $p = $p|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 HEAP32[$0>>2] = $p;
 $1 = HEAP32[$0>>2]|0;
 $2 = (($1) + 26|0);
 $3 = HEAP16[$2>>1]|0;
 $4 = (($3) + -1)<<16>>16;
 HEAP16[$2>>1] = $4;
 $5 = HEAP32[$0>>2]|0;
 $6 = (($5) + 26|0);
 $7 = HEAP16[$6>>1]|0;
 $8 = $7 << 16 >> 16;
 $9 = ($8|0)==(0);
 if (!($9)) {
  STACKTOP = sp;return;
 }
 $10 = HEAP32[$0>>2]|0;
 $11 = (($10) + 28|0);
 $12 = HEAP32[$11>>2]|0;
 $13 = (($12) + 12|0);
 $14 = HEAP32[$13>>2]|0;
 $15 = (($14) + -1)|0;
 HEAP32[$13>>2] = $15;
 $16 = HEAP32[$0>>2]|0;
 $17 = (($16) + 24|0);
 $18 = HEAP16[$17>>1]|0;
 $19 = $18&65535;
 $20 = $19 & 2;
 $21 = ($20|0)==(0);
 $22 = HEAP32[$0>>2]|0;
 if ($21) {
  _pcacheUnpin($22);
  STACKTOP = sp;return;
 }
 $23 = (($22) + 36|0);
 $24 = HEAP32[$23>>2]|0;
 $25 = ($24|0)!=(0|0);
 if (!($25)) {
  STACKTOP = sp;return;
 }
 $26 = HEAP32[$0>>2]|0;
 _pcacheManageDirtyList($26,3);
 STACKTOP = sp;return;
}
function _sqlite3PcacheFetch($pCache,$pgno,$createFlag) {
 $pCache = $pCache|0;
 $pgno = $pgno|0;
 $createFlag = $createFlag|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $eCreate = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $1 = sp + 8|0;
 $2 = sp + 4|0;
 $eCreate = sp;
 HEAP32[$0>>2] = $pCache;
 HEAP32[$1>>2] = $pgno;
 HEAP32[$2>>2] = $createFlag;
 $3 = HEAP32[$2>>2]|0;
 $4 = HEAP32[$0>>2]|0;
 $5 = (($4) + 29|0);
 $6 = HEAP8[$5>>0]|0;
 $7 = $6&255;
 $8 = $3 & $7;
 HEAP32[$eCreate>>2] = $8;
 $9 = HEAP32[((38640 + 132|0))>>2]|0;
 $10 = HEAP32[$0>>2]|0;
 $11 = (($10) + 40|0);
 $12 = HEAP32[$11>>2]|0;
 $13 = HEAP32[$1>>2]|0;
 $14 = HEAP32[$eCreate>>2]|0;
 $15 = (FUNCTION_TABLE_iiii[$9 & 31]($12,$13,$14)|0);
 STACKTOP = sp;return ($15|0);
}
function _sqlite3PcacheFetchFinish($pCache,$pgno,$pPage) {
 $pCache = $pCache|0;
 $pgno = $pgno|0;
 $pPage = $pPage|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $pPgHdr = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 16|0;
 $1 = sp + 12|0;
 $2 = sp + 8|0;
 $3 = sp + 4|0;
 $pPgHdr = sp;
 HEAP32[$1>>2] = $pCache;
 HEAP32[$2>>2] = $pgno;
 HEAP32[$3>>2] = $pPage;
 $4 = HEAP32[$3>>2]|0;
 $5 = ($4|0)==(0|0);
 if ($5) {
  HEAP32[$0>>2] = 0;
  $35 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($35|0);
 }
 $6 = HEAP32[$3>>2]|0;
 $7 = (($6) + 4|0);
 $8 = HEAP32[$7>>2]|0;
 HEAP32[$pPgHdr>>2] = $8;
 $9 = HEAP32[$pPgHdr>>2]|0;
 $10 = HEAP32[$9>>2]|0;
 $11 = ($10|0)!=(0|0);
 if (!($11)) {
  $12 = HEAP32[$1>>2]|0;
  $13 = HEAP32[$2>>2]|0;
  $14 = HEAP32[$3>>2]|0;
  $15 = (_pcacheFetchFinishWithInit($12,$13,$14)|0);
  HEAP32[$0>>2] = $15;
  $35 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($35|0);
 }
 $16 = HEAP32[$pPgHdr>>2]|0;
 $17 = (($16) + 26|0);
 $18 = HEAP16[$17>>1]|0;
 $19 = $18 << 16 >> 16;
 $20 = (0)==($19|0);
 if ($20) {
  $21 = HEAP32[$1>>2]|0;
  $22 = (($21) + 12|0);
  $23 = HEAP32[$22>>2]|0;
  $24 = (($23) + 1)|0;
  HEAP32[$22>>2] = $24;
 }
 $25 = HEAP32[$pPgHdr>>2]|0;
 $26 = (($25) + 26|0);
 $27 = HEAP16[$26>>1]|0;
 $28 = (($27) + 1)<<16>>16;
 HEAP16[$26>>1] = $28;
 $29 = HEAP32[$2>>2]|0;
 $30 = ($29|0)==(1);
 if ($30) {
  $31 = HEAP32[$pPgHdr>>2]|0;
  $32 = HEAP32[$1>>2]|0;
  $33 = (($32) + 44|0);
  HEAP32[$33>>2] = $31;
 }
 $34 = HEAP32[$pPgHdr>>2]|0;
 HEAP32[$0>>2] = $34;
 $35 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($35|0);
}
function _pcacheFetchFinishWithInit($pCache,$pgno,$pPage) {
 $pCache = $pCache|0;
 $pgno = $pgno|0;
 $pPage = $pPage|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $pPgHdr = 0, dest = 0, label = 0, sp = 0, stop = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $1 = sp + 8|0;
 $2 = sp + 4|0;
 $pPgHdr = sp;
 HEAP32[$0>>2] = $pCache;
 HEAP32[$1>>2] = $pgno;
 HEAP32[$2>>2] = $pPage;
 $3 = HEAP32[$2>>2]|0;
 $4 = (($3) + 4|0);
 $5 = HEAP32[$4>>2]|0;
 HEAP32[$pPgHdr>>2] = $5;
 $6 = HEAP32[$pPgHdr>>2]|0;
 dest=$6+0|0; stop=dest+40|0; do { HEAP32[dest>>2]=0|0; dest=dest+4|0; } while ((dest|0) < (stop|0));
 $7 = HEAP32[$2>>2]|0;
 $8 = HEAP32[$pPgHdr>>2]|0;
 HEAP32[$8>>2] = $7;
 $9 = HEAP32[$2>>2]|0;
 $10 = HEAP32[$9>>2]|0;
 $11 = HEAP32[$pPgHdr>>2]|0;
 $12 = (($11) + 4|0);
 HEAP32[$12>>2] = $10;
 $13 = HEAP32[$pPgHdr>>2]|0;
 $14 = (($13) + 40|0);
 $15 = HEAP32[$pPgHdr>>2]|0;
 $16 = (($15) + 8|0);
 HEAP32[$16>>2] = $14;
 $17 = HEAP32[$pPgHdr>>2]|0;
 $18 = (($17) + 8|0);
 $19 = HEAP32[$18>>2]|0;
 $20 = HEAP32[$0>>2]|0;
 $21 = (($20) + 24|0);
 $22 = HEAP32[$21>>2]|0;
 _memset(($19|0),0,($22|0))|0;
 $23 = HEAP32[$0>>2]|0;
 $24 = HEAP32[$pPgHdr>>2]|0;
 $25 = (($24) + 28|0);
 HEAP32[$25>>2] = $23;
 $26 = HEAP32[$1>>2]|0;
 $27 = HEAP32[$pPgHdr>>2]|0;
 $28 = (($27) + 20|0);
 HEAP32[$28>>2] = $26;
 $29 = HEAP32[$0>>2]|0;
 $30 = HEAP32[$1>>2]|0;
 $31 = HEAP32[$2>>2]|0;
 $32 = (_sqlite3PcacheFetchFinish($29,$30,$31)|0);
 STACKTOP = sp;return ($32|0);
}
function _journalHdrOffset($pPager) {
 $pPager = $pPager|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $c = 0, $offset = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 16|0;
 $offset = sp + 8|0;
 $c = sp;
 HEAP32[$0>>2] = $pPager;
 $1 = $offset;
 $2 = $1;
 HEAP32[$2>>2] = 0;
 $3 = (($1) + 4)|0;
 $4 = $3;
 HEAP32[$4>>2] = 0;
 $5 = HEAP32[$0>>2]|0;
 $6 = (($5) + 72|0);
 $7 = $6;
 $8 = $7;
 $9 = HEAP32[$8>>2]|0;
 $10 = (($7) + 4)|0;
 $11 = $10;
 $12 = HEAP32[$11>>2]|0;
 $13 = $c;
 $14 = $13;
 HEAP32[$14>>2] = $9;
 $15 = (($13) + 4)|0;
 $16 = $15;
 HEAP32[$16>>2] = $12;
 $17 = $c;
 $18 = $17;
 $19 = HEAP32[$18>>2]|0;
 $20 = (($17) + 4)|0;
 $21 = $20;
 $22 = HEAP32[$21>>2]|0;
 $23 = ($19|0)!=(0);
 $24 = ($22|0)!=(0);
 $25 = $23 | $24;
 if (!($25)) {
  $50 = $offset;
  $51 = $50;
  $52 = HEAP32[$51>>2]|0;
  $53 = (($50) + 4)|0;
  $54 = $53;
  $55 = HEAP32[$54>>2]|0;
  tempRet0 = $55;
  STACKTOP = sp;return ($52|0);
 }
 $26 = $c;
 $27 = $26;
 $28 = HEAP32[$27>>2]|0;
 $29 = (($26) + 4)|0;
 $30 = $29;
 $31 = HEAP32[$30>>2]|0;
 $32 = (_i64Subtract(($28|0),($31|0),1,0)|0);
 $33 = tempRet0;
 $34 = HEAP32[$0>>2]|0;
 $35 = (($34) + 148|0);
 $36 = HEAP32[$35>>2]|0;
 $37 = (___divdi3(($32|0),($33|0),($36|0),0)|0);
 $38 = tempRet0;
 $39 = (_i64Add(($37|0),($38|0),1,0)|0);
 $40 = tempRet0;
 $41 = HEAP32[$0>>2]|0;
 $42 = (($41) + 148|0);
 $43 = HEAP32[$42>>2]|0;
 $44 = (___muldi3(($39|0),($40|0),($43|0),0)|0);
 $45 = tempRet0;
 $46 = $offset;
 $47 = $46;
 HEAP32[$47>>2] = $44;
 $48 = (($46) + 4)|0;
 $49 = $48;
 HEAP32[$49>>2] = $45;
 $50 = $offset;
 $51 = $50;
 $52 = HEAP32[$51>>2]|0;
 $53 = (($50) + 4)|0;
 $54 = $53;
 $55 = HEAP32[$54>>2]|0;
 tempRet0 = $55;
 STACKTOP = sp;return ($52|0);
}
function _pagerPlaybackSavepoint($pPager,$pSavepoint) {
 $pPager = $pPager|0;
 $pSavepoint = $pSavepoint|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0;
 var $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0, $188 = 0;
 var $189 = 0, $19 = 0, $190 = 0, $191 = 0, $192 = 0, $193 = 0, $194 = 0, $195 = 0, $196 = 0, $197 = 0, $198 = 0, $199 = 0, $2 = 0, $20 = 0, $200 = 0, $201 = 0, $202 = 0, $203 = 0, $204 = 0, $205 = 0;
 var $206 = 0, $207 = 0, $208 = 0, $209 = 0, $21 = 0, $210 = 0, $211 = 0, $212 = 0, $213 = 0, $214 = 0, $215 = 0, $216 = 0, $217 = 0, $218 = 0, $219 = 0, $22 = 0, $220 = 0, $221 = 0, $222 = 0, $223 = 0;
 var $224 = 0, $225 = 0, $226 = 0, $227 = 0, $228 = 0, $229 = 0, $23 = 0, $230 = 0, $231 = 0, $232 = 0, $233 = 0, $234 = 0, $235 = 0, $236 = 0, $237 = 0, $238 = 0, $239 = 0, $24 = 0, $240 = 0, $241 = 0;
 var $242 = 0, $243 = 0, $244 = 0, $245 = 0, $246 = 0, $247 = 0, $248 = 0, $249 = 0, $25 = 0, $250 = 0, $251 = 0, $252 = 0, $253 = 0, $254 = 0, $255 = 0, $256 = 0, $257 = 0, $258 = 0, $259 = 0, $26 = 0;
 var $260 = 0, $261 = 0, $262 = 0, $263 = 0, $264 = 0, $265 = 0, $266 = 0, $267 = 0, $268 = 0, $269 = 0, $27 = 0, $270 = 0, $271 = 0, $272 = 0, $273 = 0, $274 = 0, $275 = 0, $276 = 0, $277 = 0, $278 = 0;
 var $279 = 0, $28 = 0, $280 = 0, $281 = 0, $282 = 0, $283 = 0, $284 = 0, $285 = 0, $286 = 0, $287 = 0, $288 = 0, $289 = 0, $29 = 0, $290 = 0, $291 = 0, $292 = 0, $293 = 0, $294 = 0, $3 = 0, $30 = 0;
 var $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0;
 var $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0;
 var $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0;
 var $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $dummy = 0, $iHdrOff = 0, $ii = 0, $ii1 = 0, $nJRec = 0;
 var $offset = 0, $pDone = 0, $rc = 0, $szJ = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 64|0;
 $0 = sp + 56|0;
 $1 = sp + 52|0;
 $2 = sp + 48|0;
 $szJ = sp + 16|0;
 $iHdrOff = sp + 8|0;
 $rc = sp + 44|0;
 $pDone = sp + 40|0;
 $ii = sp + 36|0;
 $nJRec = sp + 32|0;
 $dummy = sp + 28|0;
 $ii1 = sp + 24|0;
 $offset = sp;
 HEAP32[$1>>2] = $pPager;
 HEAP32[$2>>2] = $pSavepoint;
 HEAP32[$rc>>2] = 0;
 HEAP32[$pDone>>2] = 0;
 $3 = HEAP32[$2>>2]|0;
 $4 = ($3|0)!=(0|0);
 if ($4) {
  $5 = HEAP32[$2>>2]|0;
  $6 = (($5) + 20|0);
  $7 = HEAP32[$6>>2]|0;
  $8 = (_sqlite3BitvecCreate($7)|0);
  HEAP32[$pDone>>2] = $8;
  $9 = HEAP32[$pDone>>2]|0;
  $10 = ($9|0)!=(0|0);
  if (!($10)) {
   HEAP32[$0>>2] = 7;
   $294 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($294|0);
  }
 }
 $11 = HEAP32[$2>>2]|0;
 $12 = ($11|0)!=(0|0);
 if ($12) {
  $13 = HEAP32[$2>>2]|0;
  $14 = (($13) + 20|0);
  $15 = HEAP32[$14>>2]|0;
  $21 = $15;
 } else {
  $16 = HEAP32[$1>>2]|0;
  $17 = (($16) + 28|0);
  $18 = HEAP32[$17>>2]|0;
  $21 = $18;
 }
 $19 = HEAP32[$1>>2]|0;
 $20 = (($19) + 24|0);
 HEAP32[$20>>2] = $21;
 $22 = HEAP32[$1>>2]|0;
 $23 = (($22) + 12|0);
 $24 = HEAP8[$23>>0]|0;
 $25 = HEAP32[$1>>2]|0;
 $26 = (($25) + 18|0);
 HEAP8[$26>>0] = $24;
 $27 = HEAP32[$2>>2]|0;
 $28 = ($27|0)!=(0|0);
 if (!($28)) {
  $29 = HEAP32[$1>>2]|0;
  $30 = (_pagerUseWal($29)|0);
  $31 = ($30|0)!=(0);
  if ($31) {
   $32 = HEAP32[$1>>2]|0;
   $33 = (_pagerRollbackWal($32)|0);
   HEAP32[$0>>2] = $33;
   $294 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($294|0);
  }
 }
 $34 = HEAP32[$1>>2]|0;
 $35 = (($34) + 72|0);
 $36 = $35;
 $37 = $36;
 $38 = HEAP32[$37>>2]|0;
 $39 = (($36) + 4)|0;
 $40 = $39;
 $41 = HEAP32[$40>>2]|0;
 $42 = $szJ;
 $43 = $42;
 HEAP32[$43>>2] = $38;
 $44 = (($42) + 4)|0;
 $45 = $44;
 HEAP32[$45>>2] = $41;
 $46 = HEAP32[$2>>2]|0;
 $47 = ($46|0)!=(0|0);
 L15: do {
  if ($47) {
   $48 = HEAP32[$1>>2]|0;
   $49 = (_pagerUseWal($48)|0);
   $50 = ($49|0)!=(0);
   if ($50) {
    label = 19;
   } else {
    $51 = HEAP32[$2>>2]|0;
    $52 = (($51) + 8|0);
    $53 = $52;
    $54 = $53;
    $55 = HEAP32[$54>>2]|0;
    $56 = (($53) + 4)|0;
    $57 = $56;
    $58 = HEAP32[$57>>2]|0;
    $59 = ($55|0)!=(0);
    $60 = ($58|0)!=(0);
    $61 = $59 | $60;
    if ($61) {
     $62 = HEAP32[$2>>2]|0;
     $63 = (($62) + 8|0);
     $64 = $63;
     $65 = $64;
     $66 = HEAP32[$65>>2]|0;
     $67 = (($64) + 4)|0;
     $68 = $67;
     $69 = HEAP32[$68>>2]|0;
     $78 = $66;$81 = $69;
    } else {
     $70 = $szJ;
     $71 = $70;
     $72 = HEAP32[$71>>2]|0;
     $73 = (($70) + 4)|0;
     $74 = $73;
     $75 = HEAP32[$74>>2]|0;
     $78 = $72;$81 = $75;
    }
    $76 = $iHdrOff;
    $77 = $76;
    HEAP32[$77>>2] = $78;
    $79 = (($76) + 4)|0;
    $80 = $79;
    HEAP32[$80>>2] = $81;
    $82 = HEAP32[$2>>2]|0;
    $83 = $82;
    $84 = $83;
    $85 = HEAP32[$84>>2]|0;
    $86 = (($83) + 4)|0;
    $87 = $86;
    $88 = HEAP32[$87>>2]|0;
    $89 = HEAP32[$1>>2]|0;
    $90 = (($89) + 72|0);
    $91 = $90;
    $92 = $91;
    HEAP32[$92>>2] = $85;
    $93 = (($91) + 4)|0;
    $94 = $93;
    HEAP32[$94>>2] = $88;
    while(1) {
     $95 = HEAP32[$rc>>2]|0;
     $96 = ($95|0)==(0);
     if (!($96)) {
      break L15;
     }
     $97 = HEAP32[$1>>2]|0;
     $98 = (($97) + 72|0);
     $99 = $98;
     $100 = $99;
     $101 = HEAP32[$100>>2]|0;
     $102 = (($99) + 4)|0;
     $103 = $102;
     $104 = HEAP32[$103>>2]|0;
     $105 = $iHdrOff;
     $106 = $105;
     $107 = HEAP32[$106>>2]|0;
     $108 = (($105) + 4)|0;
     $109 = $108;
     $110 = HEAP32[$109>>2]|0;
     $111 = ($104|0)<($110|0);
     $112 = ($101>>>0)<($107>>>0);
     $113 = ($104|0)==($110|0);
     $114 = $113 & $112;
     $115 = $111 | $114;
     if (!($115)) {
      break L15;
     }
     $116 = HEAP32[$1>>2]|0;
     $117 = HEAP32[$1>>2]|0;
     $118 = (($117) + 72|0);
     $119 = HEAP32[$pDone>>2]|0;
     $120 = (_pager_playback_one_page($116,$118,$119,1,1)|0);
     HEAP32[$rc>>2] = $120;
    }
   }
  } else {
   label = 19;
  }
 } while(0);
 if ((label|0) == 19) {
  $121 = HEAP32[$1>>2]|0;
  $122 = (($121) + 72|0);
  $123 = $122;
  $124 = $123;
  HEAP32[$124>>2] = 0;
  $125 = (($123) + 4)|0;
  $126 = $125;
  HEAP32[$126>>2] = 0;
 }
 L28: while(1) {
  $127 = HEAP32[$rc>>2]|0;
  $128 = ($127|0)==(0);
  if (!($128)) {
   break;
  }
  $129 = HEAP32[$1>>2]|0;
  $130 = (($129) + 72|0);
  $131 = $130;
  $132 = $131;
  $133 = HEAP32[$132>>2]|0;
  $134 = (($131) + 4)|0;
  $135 = $134;
  $136 = HEAP32[$135>>2]|0;
  $137 = $szJ;
  $138 = $137;
  $139 = HEAP32[$138>>2]|0;
  $140 = (($137) + 4)|0;
  $141 = $140;
  $142 = HEAP32[$141>>2]|0;
  $143 = ($136|0)<($142|0);
  $144 = ($133>>>0)<($139>>>0);
  $145 = ($136|0)==($142|0);
  $146 = $145 & $144;
  $147 = $143 | $146;
  if (!($147)) {
   break;
  }
  HEAP32[$nJRec>>2] = 0;
  $148 = HEAP32[$1>>2]|0;
  $149 = $szJ;
  $150 = $149;
  $151 = HEAP32[$150>>2]|0;
  $152 = (($149) + 4)|0;
  $153 = $152;
  $154 = HEAP32[$153>>2]|0;
  $155 = (_readJournalHdr($148,0,$151,$154,$nJRec,$dummy)|0);
  HEAP32[$rc>>2] = $155;
  $156 = HEAP32[$nJRec>>2]|0;
  $157 = ($156|0)==(0);
  if ($157) {
   $158 = HEAP32[$1>>2]|0;
   $159 = (($158) + 80|0);
   $160 = $159;
   $161 = $160;
   $162 = HEAP32[$161>>2]|0;
   $163 = (($160) + 4)|0;
   $164 = $163;
   $165 = HEAP32[$164>>2]|0;
   $166 = HEAP32[$1>>2]|0;
   $167 = (($166) + 148|0);
   $168 = HEAP32[$167>>2]|0;
   $169 = (_i64Add(($162|0),($165|0),($168|0),0)|0);
   $170 = tempRet0;
   $171 = HEAP32[$1>>2]|0;
   $172 = (($171) + 72|0);
   $173 = $172;
   $174 = $173;
   $175 = HEAP32[$174>>2]|0;
   $176 = (($173) + 4)|0;
   $177 = $176;
   $178 = HEAP32[$177>>2]|0;
   $179 = ($169|0)==($175|0);
   $180 = ($170|0)==($178|0);
   $181 = $179 & $180;
   if ($181) {
    $182 = $szJ;
    $183 = $182;
    $184 = HEAP32[$183>>2]|0;
    $185 = (($182) + 4)|0;
    $186 = $185;
    $187 = HEAP32[$186>>2]|0;
    $188 = HEAP32[$1>>2]|0;
    $189 = (($188) + 72|0);
    $190 = $189;
    $191 = $190;
    $192 = HEAP32[$191>>2]|0;
    $193 = (($190) + 4)|0;
    $194 = $193;
    $195 = HEAP32[$194>>2]|0;
    $196 = (_i64Subtract(($184|0),($187|0),($192|0),($195|0))|0);
    $197 = tempRet0;
    $198 = HEAP32[$1>>2]|0;
    $199 = (($198) + 152|0);
    $200 = HEAP32[$199>>2]|0;
    $201 = (($200) + 8)|0;
    $202 = ($201|0)<(0);
    $203 = $202 << 31 >> 31;
    $204 = (___divdi3(($196|0),($197|0),($201|0),($203|0))|0);
    $205 = tempRet0;
    HEAP32[$nJRec>>2] = $204;
   }
  }
  HEAP32[$ii>>2] = 0;
  while(1) {
   $206 = HEAP32[$rc>>2]|0;
   $207 = ($206|0)==(0);
   if (!($207)) {
    continue L28;
   }
   $208 = HEAP32[$ii>>2]|0;
   $209 = HEAP32[$nJRec>>2]|0;
   $210 = ($208>>>0)<($209>>>0);
   if (!($210)) {
    continue L28;
   }
   $211 = HEAP32[$1>>2]|0;
   $212 = (($211) + 72|0);
   $213 = $212;
   $214 = $213;
   $215 = HEAP32[$214>>2]|0;
   $216 = (($213) + 4)|0;
   $217 = $216;
   $218 = HEAP32[$217>>2]|0;
   $219 = $szJ;
   $220 = $219;
   $221 = HEAP32[$220>>2]|0;
   $222 = (($219) + 4)|0;
   $223 = $222;
   $224 = HEAP32[$223>>2]|0;
   $225 = ($218|0)<($224|0);
   $226 = ($215>>>0)<($221>>>0);
   $227 = ($218|0)==($224|0);
   $228 = $227 & $226;
   $229 = $225 | $228;
   if (!($229)) {
    continue L28;
   }
   $230 = HEAP32[$1>>2]|0;
   $231 = HEAP32[$1>>2]|0;
   $232 = (($231) + 72|0);
   $233 = HEAP32[$pDone>>2]|0;
   $234 = (_pager_playback_one_page($230,$232,$233,1,1)|0);
   HEAP32[$rc>>2] = $234;
   $235 = HEAP32[$ii>>2]|0;
   $236 = (($235) + 1)|0;
   HEAP32[$ii>>2] = $236;
  }
 }
 $237 = HEAP32[$2>>2]|0;
 $238 = ($237|0)!=(0|0);
 L42: do {
  if ($238) {
   $239 = HEAP32[$2>>2]|0;
   $240 = (($239) + 24|0);
   $241 = HEAP32[$240>>2]|0;
   $242 = HEAP32[$1>>2]|0;
   $243 = (($242) + 152|0);
   $244 = HEAP32[$243>>2]|0;
   $245 = (4 + ($244))|0;
   $246 = ($245|0)<(0);
   $247 = $246 << 31 >> 31;
   $248 = (___muldi3(($241|0),0,($245|0),($247|0))|0);
   $249 = tempRet0;
   $250 = $offset;
   $251 = $250;
   HEAP32[$251>>2] = $248;
   $252 = (($250) + 4)|0;
   $253 = $252;
   HEAP32[$253>>2] = $249;
   $254 = HEAP32[$1>>2]|0;
   $255 = (_pagerUseWal($254)|0);
   $256 = ($255|0)!=(0);
   if ($256) {
    $257 = HEAP32[$1>>2]|0;
    $258 = (($257) + 208|0);
    $259 = HEAP32[$258>>2]|0;
    $260 = HEAP32[$2>>2]|0;
    $261 = (($260) + 28|0);
    $262 = (_sqlite3WalSavepointUndo($259,$261)|0);
    HEAP32[$rc>>2] = $262;
   }
   $263 = HEAP32[$2>>2]|0;
   $264 = (($263) + 24|0);
   $265 = HEAP32[$264>>2]|0;
   HEAP32[$ii1>>2] = $265;
   while(1) {
    $266 = HEAP32[$rc>>2]|0;
    $267 = ($266|0)==(0);
    if (!($267)) {
     break L42;
    }
    $268 = HEAP32[$ii1>>2]|0;
    $269 = HEAP32[$1>>2]|0;
    $270 = (($269) + 52|0);
    $271 = HEAP32[$270>>2]|0;
    $272 = ($268>>>0)<($271>>>0);
    if (!($272)) {
     break L42;
    }
    $273 = HEAP32[$1>>2]|0;
    $274 = HEAP32[$pDone>>2]|0;
    $275 = (_pager_playback_one_page($273,$offset,$274,0,1)|0);
    HEAP32[$rc>>2] = $275;
    $276 = HEAP32[$ii1>>2]|0;
    $277 = (($276) + 1)|0;
    HEAP32[$ii1>>2] = $277;
   }
  }
 } while(0);
 $278 = HEAP32[$pDone>>2]|0;
 _sqlite3BitvecDestroy($278);
 $279 = HEAP32[$rc>>2]|0;
 $280 = ($279|0)==(0);
 if ($280) {
  $281 = $szJ;
  $282 = $281;
  $283 = HEAP32[$282>>2]|0;
  $284 = (($281) + 4)|0;
  $285 = $284;
  $286 = HEAP32[$285>>2]|0;
  $287 = HEAP32[$1>>2]|0;
  $288 = (($287) + 72|0);
  $289 = $288;
  $290 = $289;
  HEAP32[$290>>2] = $283;
  $291 = (($289) + 4)|0;
  $292 = $291;
  HEAP32[$292>>2] = $286;
 }
 $293 = HEAP32[$rc>>2]|0;
 HEAP32[$0>>2] = $293;
 $294 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($294|0);
}
function _pagerRollbackWal($pPager) {
 $pPager = $pPager|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $or$cond = 0, $pList = 0, $pNext = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $rc = sp + 8|0;
 $pList = sp + 4|0;
 $pNext = sp;
 HEAP32[$0>>2] = $pPager;
 $1 = HEAP32[$0>>2]|0;
 $2 = (($1) + 28|0);
 $3 = HEAP32[$2>>2]|0;
 $4 = HEAP32[$0>>2]|0;
 $5 = (($4) + 24|0);
 HEAP32[$5>>2] = $3;
 $6 = HEAP32[$0>>2]|0;
 $7 = (($6) + 208|0);
 $8 = HEAP32[$7>>2]|0;
 $9 = HEAP32[$0>>2]|0;
 $10 = (_sqlite3WalUndo($8,36,$9)|0);
 HEAP32[$rc>>2] = $10;
 $11 = HEAP32[$0>>2]|0;
 $12 = (($11) + 204|0);
 $13 = HEAP32[$12>>2]|0;
 $14 = (_sqlite3PcacheDirtyList($13)|0);
 HEAP32[$pList>>2] = $14;
 while(1) {
  $15 = HEAP32[$pList>>2]|0;
  $16 = ($15|0)!=(0|0);
  $17 = HEAP32[$rc>>2]|0;
  $18 = ($17|0)==(0);
  $or$cond = $16 & $18;
  if (!($or$cond)) {
   break;
  }
  $19 = HEAP32[$pList>>2]|0;
  $20 = (($19) + 12|0);
  $21 = HEAP32[$20>>2]|0;
  HEAP32[$pNext>>2] = $21;
  $22 = HEAP32[$0>>2]|0;
  $23 = HEAP32[$pList>>2]|0;
  $24 = (($23) + 20|0);
  $25 = HEAP32[$24>>2]|0;
  $26 = (_pagerUndoCallback($22,$25)|0);
  HEAP32[$rc>>2] = $26;
  $27 = HEAP32[$pNext>>2]|0;
  HEAP32[$pList>>2] = $27;
 }
 $28 = HEAP32[$rc>>2]|0;
 STACKTOP = sp;return ($28|0);
}
function _sqlite3WalSavepointUndo($pWal,$aWalData) {
 $pWal = $pWal|0;
 $aWalData = $aWalData|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $5 = 0, $6 = 0, $7 = 0;
 var $8 = 0, $9 = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $1 = sp + 4|0;
 $rc = sp;
 HEAP32[$0>>2] = $pWal;
 HEAP32[$1>>2] = $aWalData;
 HEAP32[$rc>>2] = 0;
 $2 = HEAP32[$1>>2]|0;
 $3 = (($2) + 12|0);
 $4 = HEAP32[$3>>2]|0;
 $5 = HEAP32[$0>>2]|0;
 $6 = (($5) + 104|0);
 $7 = HEAP32[$6>>2]|0;
 $8 = ($4|0)!=($7|0);
 if ($8) {
  $9 = HEAP32[$1>>2]|0;
  HEAP32[$9>>2] = 0;
  $10 = HEAP32[$0>>2]|0;
  $11 = (($10) + 104|0);
  $12 = HEAP32[$11>>2]|0;
  $13 = HEAP32[$1>>2]|0;
  $14 = (($13) + 12|0);
  HEAP32[$14>>2] = $12;
 }
 $15 = HEAP32[$1>>2]|0;
 $16 = HEAP32[$15>>2]|0;
 $17 = HEAP32[$0>>2]|0;
 $18 = (($17) + 52|0);
 $19 = (($18) + 16|0);
 $20 = HEAP32[$19>>2]|0;
 $21 = ($16>>>0)<($20>>>0);
 if (!($21)) {
  $41 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($41|0);
 }
 $22 = HEAP32[$1>>2]|0;
 $23 = HEAP32[$22>>2]|0;
 $24 = HEAP32[$0>>2]|0;
 $25 = (($24) + 52|0);
 $26 = (($25) + 16|0);
 HEAP32[$26>>2] = $23;
 $27 = HEAP32[$1>>2]|0;
 $28 = (($27) + 4|0);
 $29 = HEAP32[$28>>2]|0;
 $30 = HEAP32[$0>>2]|0;
 $31 = (($30) + 52|0);
 $32 = (($31) + 24|0);
 HEAP32[$32>>2] = $29;
 $33 = HEAP32[$1>>2]|0;
 $34 = (($33) + 8|0);
 $35 = HEAP32[$34>>2]|0;
 $36 = HEAP32[$0>>2]|0;
 $37 = (($36) + 52|0);
 $38 = (($37) + 24|0);
 $39 = (($38) + 4|0);
 HEAP32[$39>>2] = $35;
 $40 = HEAP32[$0>>2]|0;
 _walCleanupHash($40);
 $41 = HEAP32[$rc>>2]|0;
 STACKTOP = sp;return ($41|0);
}
function _walCleanupHash($pWal) {
 $pWal = $pWal|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $aHash = 0, $aPgno = 0, $i = 0, $iLimit = 0, $iZero = 0, $nByte = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 24|0;
 $aHash = sp + 20|0;
 $aPgno = sp + 16|0;
 $iZero = sp + 12|0;
 $iLimit = sp + 8|0;
 $nByte = sp + 4|0;
 $i = sp;
 HEAP32[$0>>2] = $pWal;
 HEAP32[$aHash>>2] = 0;
 HEAP32[$aPgno>>2] = 0;
 HEAP32[$iZero>>2] = 0;
 HEAP32[$iLimit>>2] = 0;
 $1 = HEAP32[$0>>2]|0;
 $2 = (($1) + 52|0);
 $3 = (($2) + 16|0);
 $4 = HEAP32[$3>>2]|0;
 $5 = ($4|0)==(0);
 if ($5) {
  STACKTOP = sp;return;
 }
 $6 = HEAP32[$0>>2]|0;
 $7 = HEAP32[$0>>2]|0;
 $8 = (($7) + 52|0);
 $9 = (($8) + 16|0);
 $10 = HEAP32[$9>>2]|0;
 $11 = (_walFramePage($10)|0);
 (_walHashGet($6,$11,$aHash,$aPgno,$iZero)|0);
 $12 = HEAP32[$0>>2]|0;
 $13 = (($12) + 52|0);
 $14 = (($13) + 16|0);
 $15 = HEAP32[$14>>2]|0;
 $16 = HEAP32[$iZero>>2]|0;
 $17 = (($15) - ($16))|0;
 HEAP32[$iLimit>>2] = $17;
 HEAP32[$i>>2] = 0;
 while(1) {
  $18 = HEAP32[$i>>2]|0;
  $19 = ($18|0)<(8192);
  if (!($19)) {
   break;
  }
  $20 = HEAP32[$i>>2]|0;
  $21 = HEAP32[$aHash>>2]|0;
  $22 = (($21) + ($20<<1)|0);
  $23 = HEAP16[$22>>1]|0;
  $24 = $23&65535;
  $25 = HEAP32[$iLimit>>2]|0;
  $26 = ($24|0)>($25|0);
  if ($26) {
   $27 = HEAP32[$i>>2]|0;
   $28 = HEAP32[$aHash>>2]|0;
   $29 = (($28) + ($27<<1)|0);
   HEAP16[$29>>1] = 0;
  }
  $30 = HEAP32[$i>>2]|0;
  $31 = (($30) + 1)|0;
  HEAP32[$i>>2] = $31;
 }
 $32 = HEAP32[$aHash>>2]|0;
 $33 = HEAP32[$iLimit>>2]|0;
 $34 = (($33) + 1)|0;
 $35 = HEAP32[$aPgno>>2]|0;
 $36 = (($35) + ($34<<2)|0);
 $37 = $32;
 $38 = $36;
 $39 = (($37) - ($38))|0;
 HEAP32[$nByte>>2] = $39;
 $40 = HEAP32[$iLimit>>2]|0;
 $41 = (($40) + 1)|0;
 $42 = HEAP32[$aPgno>>2]|0;
 $43 = (($42) + ($41<<2)|0);
 $44 = HEAP32[$nByte>>2]|0;
 _memset(($43|0),0,($44|0))|0;
 STACKTOP = sp;return;
}
function _walHashGet($pWal,$iHash,$paHash,$paPgno,$piZero) {
 $pWal = $pWal|0;
 $iHash = $iHash|0;
 $paHash = $paHash|0;
 $paPgno = $paPgno|0;
 $piZero = $piZero|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $aHash = 0, $aPgno = 0, $iZero = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $0 = sp + 32|0;
 $1 = sp + 28|0;
 $2 = sp + 24|0;
 $3 = sp + 20|0;
 $4 = sp + 16|0;
 $rc = sp + 12|0;
 $aPgno = sp + 8|0;
 $iZero = sp + 4|0;
 $aHash = sp;
 HEAP32[$0>>2] = $pWal;
 HEAP32[$1>>2] = $iHash;
 HEAP32[$2>>2] = $paHash;
 HEAP32[$3>>2] = $paPgno;
 HEAP32[$4>>2] = $piZero;
 $5 = HEAP32[$0>>2]|0;
 $6 = HEAP32[$1>>2]|0;
 $7 = (_walIndexPage($5,$6,$aPgno)|0);
 HEAP32[$rc>>2] = $7;
 $8 = HEAP32[$rc>>2]|0;
 $9 = ($8|0)==(0);
 if (!($9)) {
  $27 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($27|0);
 }
 $10 = HEAP32[$aPgno>>2]|0;
 $11 = (($10) + 16384|0);
 HEAP32[$aHash>>2] = $11;
 $12 = HEAP32[$1>>2]|0;
 $13 = ($12|0)==(0);
 if ($13) {
  $14 = HEAP32[$aPgno>>2]|0;
  $15 = (($14) + 136|0);
  HEAP32[$aPgno>>2] = $15;
  HEAP32[$iZero>>2] = 0;
 } else {
  $16 = HEAP32[$1>>2]|0;
  $17 = (($16) - 1)|0;
  $18 = $17<<12;
  $19 = (4062 + ($18))|0;
  HEAP32[$iZero>>2] = $19;
 }
 $20 = HEAP32[$aPgno>>2]|0;
 $21 = (($20) + -4|0);
 $22 = HEAP32[$3>>2]|0;
 HEAP32[$22>>2] = $21;
 $23 = HEAP32[$aHash>>2]|0;
 $24 = HEAP32[$2>>2]|0;
 HEAP32[$24>>2] = $23;
 $25 = HEAP32[$iZero>>2]|0;
 $26 = HEAP32[$4>>2]|0;
 HEAP32[$26>>2] = $25;
 $27 = HEAP32[$rc>>2]|0;
 STACKTOP = sp;return ($27|0);
}
function _walFramePage($iFrame) {
 $iFrame = $iFrame|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $iHash = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $iHash = sp;
 HEAP32[$0>>2] = $iFrame;
 $1 = HEAP32[$0>>2]|0;
 $2 = (($1) + 4096)|0;
 $3 = (($2) - 4062)|0;
 $4 = (($3) - 1)|0;
 $5 = (($4>>>0) / 4096)&-1;
 HEAP32[$iHash>>2] = $5;
 $6 = HEAP32[$iHash>>2]|0;
 STACKTOP = sp;return ($6|0);
}
function _walIndexPage($pWal,$iPage,$ppPage) {
 $pWal = $pWal|0;
 $iPage = $iPage|0;
 $ppPage = $ppPage|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0;
 var $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $apNew = 0, $nByte = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 24|0;
 $1 = sp + 20|0;
 $2 = sp + 16|0;
 $3 = sp + 12|0;
 $rc = sp + 8|0;
 $nByte = sp + 4|0;
 $apNew = sp;
 HEAP32[$1>>2] = $pWal;
 HEAP32[$2>>2] = $iPage;
 HEAP32[$3>>2] = $ppPage;
 HEAP32[$rc>>2] = 0;
 $4 = HEAP32[$1>>2]|0;
 $5 = (($4) + 24|0);
 $6 = HEAP32[$5>>2]|0;
 $7 = HEAP32[$2>>2]|0;
 $8 = ($6|0)<=($7|0);
 do {
  if ($8) {
   $9 = HEAP32[$2>>2]|0;
   $10 = (($9) + 1)|0;
   $11 = $10<<2;
   HEAP32[$nByte>>2] = $11;
   $12 = HEAP32[$1>>2]|0;
   $13 = (($12) + 32|0);
   $14 = HEAP32[$13>>2]|0;
   $15 = HEAP32[$nByte>>2]|0;
   $16 = (_sqlite3_realloc($14,$15)|0);
   HEAP32[$apNew>>2] = $16;
   $17 = HEAP32[$apNew>>2]|0;
   $18 = ($17|0)!=(0|0);
   if ($18) {
    $20 = HEAP32[$1>>2]|0;
    $21 = (($20) + 24|0);
    $22 = HEAP32[$21>>2]|0;
    $23 = HEAP32[$apNew>>2]|0;
    $24 = (($23) + ($22<<2)|0);
    $25 = HEAP32[$2>>2]|0;
    $26 = (($25) + 1)|0;
    $27 = HEAP32[$1>>2]|0;
    $28 = (($27) + 24|0);
    $29 = HEAP32[$28>>2]|0;
    $30 = (($26) - ($29))|0;
    $31 = $30<<2;
    _memset(($24|0),0,($31|0))|0;
    $32 = HEAP32[$apNew>>2]|0;
    $33 = HEAP32[$1>>2]|0;
    $34 = (($33) + 32|0);
    HEAP32[$34>>2] = $32;
    $35 = HEAP32[$2>>2]|0;
    $36 = (($35) + 1)|0;
    $37 = HEAP32[$1>>2]|0;
    $38 = (($37) + 24|0);
    HEAP32[$38>>2] = $36;
    break;
   }
   $19 = HEAP32[$3>>2]|0;
   HEAP32[$19>>2] = 0;
   HEAP32[$0>>2] = 7;
   $94 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($94|0);
  }
 } while(0);
 $39 = HEAP32[$2>>2]|0;
 $40 = HEAP32[$1>>2]|0;
 $41 = (($40) + 32|0);
 $42 = HEAP32[$41>>2]|0;
 $43 = (($42) + ($39<<2)|0);
 $44 = HEAP32[$43>>2]|0;
 $45 = ($44|0)==(0|0);
 do {
  if ($45) {
   $46 = HEAP32[$1>>2]|0;
   $47 = (($46) + 43|0);
   $48 = HEAP8[$47>>0]|0;
   $49 = $48&255;
   $50 = ($49|0)==(2);
   if ($50) {
    $51 = (_sqlite3MallocZero(32768,0)|0);
    $52 = HEAP32[$2>>2]|0;
    $53 = HEAP32[$1>>2]|0;
    $54 = (($53) + 32|0);
    $55 = HEAP32[$54>>2]|0;
    $56 = (($55) + ($52<<2)|0);
    HEAP32[$56>>2] = $51;
    $57 = HEAP32[$2>>2]|0;
    $58 = HEAP32[$1>>2]|0;
    $59 = (($58) + 32|0);
    $60 = HEAP32[$59>>2]|0;
    $61 = (($60) + ($57<<2)|0);
    $62 = HEAP32[$61>>2]|0;
    $63 = ($62|0)!=(0|0);
    if ($63) {
     break;
    }
    HEAP32[$rc>>2] = 7;
    break;
   } else {
    $64 = HEAP32[$1>>2]|0;
    $65 = (($64) + 4|0);
    $66 = HEAP32[$65>>2]|0;
    $67 = HEAP32[$2>>2]|0;
    $68 = HEAP32[$1>>2]|0;
    $69 = (($68) + 44|0);
    $70 = HEAP8[$69>>0]|0;
    $71 = $70&255;
    $72 = HEAP32[$2>>2]|0;
    $73 = HEAP32[$1>>2]|0;
    $74 = (($73) + 32|0);
    $75 = HEAP32[$74>>2]|0;
    $76 = (($75) + ($72<<2)|0);
    $77 = (_sqlite3OsShmMap($66,$67,32768,$71,$76)|0);
    HEAP32[$rc>>2] = $77;
    $78 = HEAP32[$rc>>2]|0;
    $79 = ($78|0)==(8);
    if (!($79)) {
     break;
    }
    $80 = HEAP32[$1>>2]|0;
    $81 = (($80) + 46|0);
    $82 = HEAP8[$81>>0]|0;
    $83 = $82&255;
    $84 = $83 | 2;
    $85 = $84&255;
    HEAP8[$81>>0] = $85;
    HEAP32[$rc>>2] = 0;
    break;
   }
  }
 } while(0);
 $86 = HEAP32[$2>>2]|0;
 $87 = HEAP32[$1>>2]|0;
 $88 = (($87) + 32|0);
 $89 = HEAP32[$88>>2]|0;
 $90 = (($89) + ($86<<2)|0);
 $91 = HEAP32[$90>>2]|0;
 $92 = HEAP32[$3>>2]|0;
 HEAP32[$92>>2] = $91;
 $93 = HEAP32[$rc>>2]|0;
 HEAP32[$0>>2] = $93;
 $94 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($94|0);
}
function _sqlite3OsShmMap($id,$iPage,$pgsz,$bExtend,$pp) {
 $id = $id|0;
 $iPage = $iPage|0;
 $pgsz = $pgsz|0;
 $bExtend = $bExtend|0;
 $pp = $pp|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 16|0;
 $1 = sp + 12|0;
 $2 = sp + 8|0;
 $3 = sp + 4|0;
 $4 = sp;
 HEAP32[$0>>2] = $id;
 HEAP32[$1>>2] = $iPage;
 HEAP32[$2>>2] = $pgsz;
 HEAP32[$3>>2] = $bExtend;
 HEAP32[$4>>2] = $pp;
 $5 = HEAP32[$0>>2]|0;
 $6 = HEAP32[$5>>2]|0;
 $7 = (($6) + 52|0);
 $8 = HEAP32[$7>>2]|0;
 $9 = HEAP32[$0>>2]|0;
 $10 = HEAP32[$1>>2]|0;
 $11 = HEAP32[$2>>2]|0;
 $12 = HEAP32[$3>>2]|0;
 $13 = HEAP32[$4>>2]|0;
 $14 = (FUNCTION_TABLE_iiiiii[$8 & 15]($9,$10,$11,$12,$13)|0);
 STACKTOP = sp;return ($14|0);
}
function _sqlite3WalUndo($pWal,$xUndo,$pUndoCtx) {
 $pWal = $pWal|0;
 $xUndo = $xUndo|0;
 $pUndoCtx = $pUndoCtx|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0;
 var $9 = 0, $iFrame = 0, $iMax = 0, $rc = 0, dest = 0, label = 0, sp = 0, src = 0, stop = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 20|0;
 $1 = sp + 16|0;
 $2 = sp + 12|0;
 $rc = sp + 8|0;
 $iMax = sp + 4|0;
 $iFrame = sp;
 HEAP32[$0>>2] = $pWal;
 HEAP32[$1>>2] = $xUndo;
 HEAP32[$2>>2] = $pUndoCtx;
 HEAP32[$rc>>2] = 0;
 $3 = HEAP32[$0>>2]|0;
 $4 = (($3) + 44|0);
 $5 = HEAP8[$4>>0]|0;
 $6 = ($5<<24>>24)!=(0);
 if (!($6)) {
  $40 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($40|0);
 }
 $7 = HEAP32[$0>>2]|0;
 $8 = (($7) + 52|0);
 $9 = (($8) + 16|0);
 $10 = HEAP32[$9>>2]|0;
 HEAP32[$iMax>>2] = $10;
 $11 = HEAP32[$0>>2]|0;
 $12 = (($11) + 52|0);
 $13 = HEAP32[$0>>2]|0;
 $14 = (_walIndexHdr($13)|0);
 dest=$12+0|0; src=$14+0|0; stop=dest+48|0; do { HEAP8[dest>>0]=HEAP8[src>>0]|0; dest=dest+1|0; src=src+1|0; } while ((dest|0) < (stop|0));
 $15 = HEAP32[$0>>2]|0;
 $16 = (($15) + 52|0);
 $17 = (($16) + 16|0);
 $18 = HEAP32[$17>>2]|0;
 $19 = (($18) + 1)|0;
 HEAP32[$iFrame>>2] = $19;
 while(1) {
  $20 = HEAP32[$rc>>2]|0;
  $21 = ($20|0)==(0);
  if (!($21)) {
   break;
  }
  $22 = HEAP32[$iFrame>>2]|0;
  $23 = HEAP32[$iMax>>2]|0;
  $24 = ($22>>>0)<=($23>>>0);
  if (!($24)) {
   break;
  }
  $25 = HEAP32[$1>>2]|0;
  $26 = HEAP32[$2>>2]|0;
  $27 = HEAP32[$0>>2]|0;
  $28 = HEAP32[$iFrame>>2]|0;
  $29 = (_walFramePgno($27,$28)|0);
  $30 = (FUNCTION_TABLE_iii[$25 & 63]($26,$29)|0);
  HEAP32[$rc>>2] = $30;
  $31 = HEAP32[$iFrame>>2]|0;
  $32 = (($31) + 1)|0;
  HEAP32[$iFrame>>2] = $32;
 }
 $33 = HEAP32[$iMax>>2]|0;
 $34 = HEAP32[$0>>2]|0;
 $35 = (($34) + 52|0);
 $36 = (($35) + 16|0);
 $37 = HEAP32[$36>>2]|0;
 $38 = ($33|0)!=($37|0);
 if (!($38)) {
  $40 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($40|0);
 }
 $39 = HEAP32[$0>>2]|0;
 _walCleanupHash($39);
 $40 = HEAP32[$rc>>2]|0;
 STACKTOP = sp;return ($40|0);
}
function _pagerUndoCallback($pCtx,$iPg) {
 $pCtx = $pCtx|0;
 $iPg = $iPg|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $iFrame = 0, $pPager = 0, $pPg = 0, $rc = 0, label = 0;
 var sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 20|0;
 $1 = sp + 16|0;
 $rc = sp + 12|0;
 $pPager = sp + 8|0;
 $pPg = sp + 4|0;
 $iFrame = sp;
 HEAP32[$0>>2] = $pCtx;
 HEAP32[$1>>2] = $iPg;
 HEAP32[$rc>>2] = 0;
 $2 = HEAP32[$0>>2]|0;
 HEAP32[$pPager>>2] = $2;
 $3 = HEAP32[$pPager>>2]|0;
 $4 = HEAP32[$1>>2]|0;
 $5 = (_sqlite3PagerLookup($3,$4)|0);
 HEAP32[$pPg>>2] = $5;
 $6 = HEAP32[$pPg>>2]|0;
 $7 = ($6|0)!=(0|0);
 do {
  if ($7) {
   $8 = HEAP32[$pPg>>2]|0;
   $9 = (_sqlite3PcachePageRefcount($8)|0);
   $10 = ($9|0)==(1);
   if ($10) {
    $11 = HEAP32[$pPg>>2]|0;
    _sqlite3PcacheDrop($11);
    break;
   }
   HEAP32[$iFrame>>2] = 0;
   $12 = HEAP32[$pPager>>2]|0;
   $13 = (($12) + 208|0);
   $14 = HEAP32[$13>>2]|0;
   $15 = HEAP32[$pPg>>2]|0;
   $16 = (($15) + 20|0);
   $17 = HEAP32[$16>>2]|0;
   $18 = (_sqlite3WalFindFrame($14,$17,$iFrame)|0);
   HEAP32[$rc>>2] = $18;
   $19 = HEAP32[$rc>>2]|0;
   $20 = ($19|0)==(0);
   if ($20) {
    $21 = HEAP32[$pPg>>2]|0;
    $22 = HEAP32[$iFrame>>2]|0;
    $23 = (_readDbPage($21,$22)|0);
    HEAP32[$rc>>2] = $23;
   }
   $24 = HEAP32[$rc>>2]|0;
   $25 = ($24|0)==(0);
   if ($25) {
    $26 = HEAP32[$pPager>>2]|0;
    $27 = (($26) + 196|0);
    $28 = HEAP32[$27>>2]|0;
    $29 = HEAP32[$pPg>>2]|0;
    FUNCTION_TABLE_vi[$28 & 31]($29);
   }
   $30 = HEAP32[$pPg>>2]|0;
   _sqlite3PagerUnrefNotNull($30);
  }
 } while(0);
 $31 = HEAP32[$pPager>>2]|0;
 $32 = (($31) + 88|0);
 $33 = HEAP32[$32>>2]|0;
 _sqlite3BackupRestart($33);
 $34 = HEAP32[$rc>>2]|0;
 STACKTOP = sp;return ($34|0);
}
function _sqlite3PcacheDirtyList($pCache) {
 $pCache = $pCache|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $p = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $p = sp;
 HEAP32[$0>>2] = $pCache;
 $1 = HEAP32[$0>>2]|0;
 $2 = HEAP32[$1>>2]|0;
 HEAP32[$p>>2] = $2;
 while(1) {
  $3 = HEAP32[$p>>2]|0;
  $4 = ($3|0)!=(0|0);
  if (!($4)) {
   break;
  }
  $5 = HEAP32[$p>>2]|0;
  $6 = (($5) + 32|0);
  $7 = HEAP32[$6>>2]|0;
  $8 = HEAP32[$p>>2]|0;
  $9 = (($8) + 12|0);
  HEAP32[$9>>2] = $7;
  $10 = HEAP32[$p>>2]|0;
  $11 = (($10) + 32|0);
  $12 = HEAP32[$11>>2]|0;
  HEAP32[$p>>2] = $12;
 }
 $13 = HEAP32[$0>>2]|0;
 $14 = HEAP32[$13>>2]|0;
 $15 = (_pcacheSortDirtyList($14)|0);
 STACKTOP = sp;return ($15|0);
}
function _pcacheSortDirtyList($pIn) {
 $pIn = $pIn|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $a = 0, $i = 0, $p = 0, dest = 0, label = 0, sp = 0, stop = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 144|0;
 $0 = sp + 136|0;
 $a = sp + 8|0;
 $p = sp + 4|0;
 $i = sp;
 HEAP32[$0>>2] = $pIn;
 dest=$a+0|0; stop=dest+128|0; do { HEAP32[dest>>2]=0|0; dest=dest+4|0; } while ((dest|0) < (stop|0));
 while(1) {
  $1 = HEAP32[$0>>2]|0;
  $2 = ($1|0)!=(0|0);
  if (!($2)) {
   break;
  }
  $3 = HEAP32[$0>>2]|0;
  HEAP32[$p>>2] = $3;
  $4 = HEAP32[$p>>2]|0;
  $5 = (($4) + 12|0);
  $6 = HEAP32[$5>>2]|0;
  HEAP32[$0>>2] = $6;
  $7 = HEAP32[$p>>2]|0;
  $8 = (($7) + 12|0);
  HEAP32[$8>>2] = 0;
  HEAP32[$i>>2] = 0;
  while(1) {
   $9 = HEAP32[$i>>2]|0;
   $10 = ($9|0)<(31);
   if (!($10)) {
    break;
   }
   $11 = HEAP32[$i>>2]|0;
   $12 = (($a) + ($11<<2)|0);
   $13 = HEAP32[$12>>2]|0;
   $14 = ($13|0)==(0|0);
   if ($14) {
    label = 6;
    break;
   }
   $18 = HEAP32[$i>>2]|0;
   $19 = (($a) + ($18<<2)|0);
   $20 = HEAP32[$19>>2]|0;
   $21 = HEAP32[$p>>2]|0;
   $22 = (_pcacheMergeDirtyList($20,$21)|0);
   HEAP32[$p>>2] = $22;
   $23 = HEAP32[$i>>2]|0;
   $24 = (($a) + ($23<<2)|0);
   HEAP32[$24>>2] = 0;
   $25 = HEAP32[$i>>2]|0;
   $26 = (($25) + 1)|0;
   HEAP32[$i>>2] = $26;
  }
  if ((label|0) == 6) {
   label = 0;
   $15 = HEAP32[$p>>2]|0;
   $16 = HEAP32[$i>>2]|0;
   $17 = (($a) + ($16<<2)|0);
   HEAP32[$17>>2] = $15;
  }
  $27 = HEAP32[$i>>2]|0;
  $28 = ($27|0)==(31);
  if (!($28)) {
   continue;
  }
  $29 = HEAP32[$i>>2]|0;
  $30 = (($a) + ($29<<2)|0);
  $31 = HEAP32[$30>>2]|0;
  $32 = HEAP32[$p>>2]|0;
  $33 = (_pcacheMergeDirtyList($31,$32)|0);
  $34 = HEAP32[$i>>2]|0;
  $35 = (($a) + ($34<<2)|0);
  HEAP32[$35>>2] = $33;
 }
 $36 = HEAP32[$a>>2]|0;
 HEAP32[$p>>2] = $36;
 HEAP32[$i>>2] = 1;
 while(1) {
  $37 = HEAP32[$i>>2]|0;
  $38 = ($37|0)<(32);
  $39 = HEAP32[$p>>2]|0;
  if (!($38)) {
   break;
  }
  $40 = HEAP32[$i>>2]|0;
  $41 = (($a) + ($40<<2)|0);
  $42 = HEAP32[$41>>2]|0;
  $43 = (_pcacheMergeDirtyList($39,$42)|0);
  HEAP32[$p>>2] = $43;
  $44 = HEAP32[$i>>2]|0;
  $45 = (($44) + 1)|0;
  HEAP32[$i>>2] = $45;
 }
 STACKTOP = sp;return ($39|0);
}
function _pcacheMergeDirtyList($pA,$pB) {
 $pA = $pA|0;
 $pB = $pB|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0;
 var $9 = 0, $pTail = 0, $result = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 64|0;
 $0 = sp + 52|0;
 $1 = sp + 48|0;
 $result = sp + 8|0;
 $pTail = sp;
 HEAP32[$0>>2] = $pA;
 HEAP32[$1>>2] = $pB;
 HEAP32[$pTail>>2] = $result;
 while(1) {
  $2 = HEAP32[$0>>2]|0;
  $3 = ($2|0)!=(0|0);
  if ($3) {
   $4 = HEAP32[$1>>2]|0;
   $5 = ($4|0)!=(0|0);
   $40 = $5;
  } else {
   $40 = 0;
  }
  $6 = HEAP32[$0>>2]|0;
  if (!($40)) {
   break;
  }
  $7 = (($6) + 20|0);
  $8 = HEAP32[$7>>2]|0;
  $9 = HEAP32[$1>>2]|0;
  $10 = (($9) + 20|0);
  $11 = HEAP32[$10>>2]|0;
  $12 = ($8>>>0)<($11>>>0);
  if ($12) {
   $13 = HEAP32[$0>>2]|0;
   $14 = HEAP32[$pTail>>2]|0;
   $15 = (($14) + 12|0);
   HEAP32[$15>>2] = $13;
   $16 = HEAP32[$0>>2]|0;
   HEAP32[$pTail>>2] = $16;
   $17 = HEAP32[$0>>2]|0;
   $18 = (($17) + 12|0);
   $19 = HEAP32[$18>>2]|0;
   HEAP32[$0>>2] = $19;
   continue;
  } else {
   $20 = HEAP32[$1>>2]|0;
   $21 = HEAP32[$pTail>>2]|0;
   $22 = (($21) + 12|0);
   HEAP32[$22>>2] = $20;
   $23 = HEAP32[$1>>2]|0;
   HEAP32[$pTail>>2] = $23;
   $24 = HEAP32[$1>>2]|0;
   $25 = (($24) + 12|0);
   $26 = HEAP32[$25>>2]|0;
   HEAP32[$1>>2] = $26;
   continue;
  }
 }
 $27 = ($6|0)!=(0|0);
 if ($27) {
  $28 = HEAP32[$0>>2]|0;
  $29 = HEAP32[$pTail>>2]|0;
  $30 = (($29) + 12|0);
  HEAP32[$30>>2] = $28;
  $38 = (($result) + 12|0);
  $39 = HEAP32[$38>>2]|0;
  STACKTOP = sp;return ($39|0);
 }
 $31 = HEAP32[$1>>2]|0;
 $32 = ($31|0)!=(0|0);
 if ($32) {
  $33 = HEAP32[$1>>2]|0;
  $34 = HEAP32[$pTail>>2]|0;
  $35 = (($34) + 12|0);
  HEAP32[$35>>2] = $33;
  $38 = (($result) + 12|0);
  $39 = HEAP32[$38>>2]|0;
  STACKTOP = sp;return ($39|0);
 } else {
  $36 = HEAP32[$pTail>>2]|0;
  $37 = (($36) + 12|0);
  HEAP32[$37>>2] = 0;
  $38 = (($result) + 12|0);
  $39 = HEAP32[$38>>2]|0;
  STACKTOP = sp;return ($39|0);
 }
 return 0|0;
}
function _sqlite3PcachePageRefcount($p) {
 $p = $p|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 HEAP32[$0>>2] = $p;
 $1 = HEAP32[$0>>2]|0;
 $2 = (($1) + 26|0);
 $3 = HEAP16[$2>>1]|0;
 $4 = $3 << 16 >> 16;
 STACKTOP = sp;return ($4|0);
}
function _sqlite3PcacheDrop($p) {
 $p = $p|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 HEAP32[$0>>2] = $p;
 $1 = HEAP32[$0>>2]|0;
 $2 = (($1) + 24|0);
 $3 = HEAP16[$2>>1]|0;
 $4 = $3&65535;
 $5 = $4 & 2;
 $6 = ($5|0)!=(0);
 if ($6) {
  $7 = HEAP32[$0>>2]|0;
  _pcacheManageDirtyList($7,1);
 }
 $8 = HEAP32[$0>>2]|0;
 $9 = (($8) + 28|0);
 $10 = HEAP32[$9>>2]|0;
 $11 = (($10) + 12|0);
 $12 = HEAP32[$11>>2]|0;
 $13 = (($12) + -1)|0;
 HEAP32[$11>>2] = $13;
 $14 = HEAP32[$0>>2]|0;
 $15 = (($14) + 20|0);
 $16 = HEAP32[$15>>2]|0;
 $17 = ($16|0)==(1);
 if ($17) {
  $18 = HEAP32[$0>>2]|0;
  $19 = (($18) + 28|0);
  $20 = HEAP32[$19>>2]|0;
  $21 = (($20) + 44|0);
  HEAP32[$21>>2] = 0;
 }
 $22 = HEAP32[((38640 + 136|0))>>2]|0;
 $23 = HEAP32[$0>>2]|0;
 $24 = (($23) + 28|0);
 $25 = HEAP32[$24>>2]|0;
 $26 = (($25) + 40|0);
 $27 = HEAP32[$26>>2]|0;
 $28 = HEAP32[$0>>2]|0;
 $29 = HEAP32[$28>>2]|0;
 FUNCTION_TABLE_viii[$22 & 63]($27,$29,1);
 STACKTOP = sp;return;
}
function _sqlite3WalFindFrame($pWal,$pgno,$piRead) {
 $pWal = $pWal|0;
 $pgno = $pgno|0;
 $piRead = $piRead|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $7 = 0, $8 = 0, $9 = 0, $aHash = 0, $aPgno = 0, $iFrame = 0, $iHash = 0, $iKey = 0, $iLast = 0, $iRead = 0, $iZero = 0, $nCollide = 0, $or$cond = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 64|0;
 $0 = sp + 52|0;
 $1 = sp + 48|0;
 $2 = sp + 44|0;
 $3 = sp + 40|0;
 $iRead = sp + 36|0;
 $iLast = sp + 32|0;
 $iHash = sp + 28|0;
 $aHash = sp + 24|0;
 $aPgno = sp + 20|0;
 $iZero = sp + 16|0;
 $iKey = sp + 12|0;
 $nCollide = sp + 8|0;
 $rc = sp + 4|0;
 $iFrame = sp;
 HEAP32[$1>>2] = $pWal;
 HEAP32[$2>>2] = $pgno;
 HEAP32[$3>>2] = $piRead;
 HEAP32[$iRead>>2] = 0;
 $4 = HEAP32[$1>>2]|0;
 $5 = (($4) + 52|0);
 $6 = (($5) + 16|0);
 $7 = HEAP32[$6>>2]|0;
 HEAP32[$iLast>>2] = $7;
 $8 = HEAP32[$iLast>>2]|0;
 $9 = ($8|0)==(0);
 if (!($9)) {
  $10 = HEAP32[$1>>2]|0;
  $11 = (($10) + 40|0);
  $12 = HEAP16[$11>>1]|0;
  $13 = $12 << 16 >> 16;
  $14 = ($13|0)==(0);
  if (!($14)) {
   $16 = HEAP32[$iLast>>2]|0;
   $17 = (_walFramePage($16)|0);
   HEAP32[$iHash>>2] = $17;
   L4: while(1) {
    $18 = HEAP32[$iHash>>2]|0;
    $19 = ($18|0)>=(0);
    $20 = HEAP32[$iRead>>2]|0;
    $21 = ($20|0)==(0);
    $or$cond = $19 & $21;
    if (!($or$cond)) {
     label = 17;
     break;
    }
    $22 = HEAP32[$1>>2]|0;
    $23 = HEAP32[$iHash>>2]|0;
    $24 = (_walHashGet($22,$23,$aHash,$aPgno,$iZero)|0);
    HEAP32[$rc>>2] = $24;
    $25 = HEAP32[$rc>>2]|0;
    $26 = ($25|0)!=(0);
    if ($26) {
     label = 7;
     break;
    }
    HEAP32[$nCollide>>2] = 8192;
    $28 = HEAP32[$2>>2]|0;
    $29 = (_walHash($28)|0);
    HEAP32[$iKey>>2] = $29;
    while(1) {
     $30 = HEAP32[$iKey>>2]|0;
     $31 = HEAP32[$aHash>>2]|0;
     $32 = (($31) + ($30<<1)|0);
     $33 = HEAP16[$32>>1]|0;
     $34 = ($33<<16>>16)!=(0);
     if (!($34)) {
      break;
     }
     $35 = HEAP32[$iKey>>2]|0;
     $36 = HEAP32[$aHash>>2]|0;
     $37 = (($36) + ($35<<1)|0);
     $38 = HEAP16[$37>>1]|0;
     $39 = $38&65535;
     $40 = HEAP32[$iZero>>2]|0;
     $41 = (($39) + ($40))|0;
     HEAP32[$iFrame>>2] = $41;
     $42 = HEAP32[$iFrame>>2]|0;
     $43 = HEAP32[$iLast>>2]|0;
     $44 = ($42>>>0)<=($43>>>0);
     if ($44) {
      $45 = HEAP32[$iKey>>2]|0;
      $46 = HEAP32[$aHash>>2]|0;
      $47 = (($46) + ($45<<1)|0);
      $48 = HEAP16[$47>>1]|0;
      $49 = $48&65535;
      $50 = HEAP32[$aPgno>>2]|0;
      $51 = (($50) + ($49<<2)|0);
      $52 = HEAP32[$51>>2]|0;
      $53 = HEAP32[$2>>2]|0;
      $54 = ($52|0)==($53|0);
      if ($54) {
       $55 = HEAP32[$iFrame>>2]|0;
       HEAP32[$iRead>>2] = $55;
      }
     }
     $56 = HEAP32[$nCollide>>2]|0;
     $57 = (($56) + -1)|0;
     HEAP32[$nCollide>>2] = $57;
     $58 = ($56|0)==(0);
     if ($58) {
      label = 14;
      break L4;
     }
     $60 = HEAP32[$iKey>>2]|0;
     $61 = (_walNextHash($60)|0);
     HEAP32[$iKey>>2] = $61;
    }
    $62 = HEAP32[$iHash>>2]|0;
    $63 = (($62) + -1)|0;
    HEAP32[$iHash>>2] = $63;
   }
   if ((label|0) == 7) {
    $27 = HEAP32[$rc>>2]|0;
    HEAP32[$0>>2] = $27;
    $66 = HEAP32[$0>>2]|0;
    STACKTOP = sp;return ($66|0);
   }
   else if ((label|0) == 14) {
    $59 = (_sqlite3CorruptError(50484)|0);
    HEAP32[$0>>2] = $59;
    $66 = HEAP32[$0>>2]|0;
    STACKTOP = sp;return ($66|0);
   }
   else if ((label|0) == 17) {
    $64 = HEAP32[$iRead>>2]|0;
    $65 = HEAP32[$3>>2]|0;
    HEAP32[$65>>2] = $64;
    HEAP32[$0>>2] = 0;
    $66 = HEAP32[$0>>2]|0;
    STACKTOP = sp;return ($66|0);
   }
  }
 }
 $15 = HEAP32[$3>>2]|0;
 HEAP32[$15>>2] = 0;
 HEAP32[$0>>2] = 0;
 $66 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($66|0);
}
function _readDbPage($pPg,$iFrame) {
 $pPg = $pPg|0;
 $iFrame = $iFrame|0;
 var $$ = 0, $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0;
 var $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0;
 var $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0;
 var $62 = 0, $63 = 0, $64 = 0, $7 = 0, $8 = 0, $9 = 0, $dbFileVers = 0, $iOffset = 0, $pPager = 0, $pgno = 0, $pgsz = 0, $rc = 0, dest = 0, label = 0, sp = 0, src = 0, stop = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $0 = sp + 32|0;
 $1 = sp + 28|0;
 $pPager = sp + 24|0;
 $pgno = sp + 20|0;
 $rc = sp + 16|0;
 $pgsz = sp + 12|0;
 $iOffset = sp;
 $dbFileVers = sp + 8|0;
 HEAP32[$0>>2] = $pPg;
 HEAP32[$1>>2] = $iFrame;
 $2 = HEAP32[$0>>2]|0;
 $3 = (($2) + 16|0);
 $4 = HEAP32[$3>>2]|0;
 HEAP32[$pPager>>2] = $4;
 $5 = HEAP32[$0>>2]|0;
 $6 = (($5) + 20|0);
 $7 = HEAP32[$6>>2]|0;
 HEAP32[$pgno>>2] = $7;
 HEAP32[$rc>>2] = 0;
 $8 = HEAP32[$pPager>>2]|0;
 $9 = (($8) + 152|0);
 $10 = HEAP32[$9>>2]|0;
 HEAP32[$pgsz>>2] = $10;
 $11 = HEAP32[$1>>2]|0;
 $12 = ($11|0)!=(0);
 if ($12) {
  $13 = HEAP32[$pPager>>2]|0;
  $14 = (($13) + 208|0);
  $15 = HEAP32[$14>>2]|0;
  $16 = HEAP32[$1>>2]|0;
  $17 = HEAP32[$pgsz>>2]|0;
  $18 = HEAP32[$0>>2]|0;
  $19 = (($18) + 4|0);
  $20 = HEAP32[$19>>2]|0;
  $21 = (_sqlite3WalReadFrame($15,$16,$17,$20)|0);
  HEAP32[$rc>>2] = $21;
 } else {
  $22 = HEAP32[$pgno>>2]|0;
  $23 = (($22) - 1)|0;
  $24 = HEAP32[$pPager>>2]|0;
  $25 = (($24) + 152|0);
  $26 = HEAP32[$25>>2]|0;
  $27 = ($26|0)<(0);
  $28 = $27 << 31 >> 31;
  $29 = (___muldi3(($23|0),0,($26|0),($28|0))|0);
  $30 = tempRet0;
  $31 = $iOffset;
  $32 = $31;
  HEAP32[$32>>2] = $29;
  $33 = (($31) + 4)|0;
  $34 = $33;
  HEAP32[$34>>2] = $30;
  $35 = HEAP32[$pPager>>2]|0;
  $36 = (($35) + 60|0);
  $37 = HEAP32[$36>>2]|0;
  $38 = HEAP32[$0>>2]|0;
  $39 = (($38) + 4|0);
  $40 = HEAP32[$39>>2]|0;
  $41 = HEAP32[$pgsz>>2]|0;
  $42 = $iOffset;
  $43 = $42;
  $44 = HEAP32[$43>>2]|0;
  $45 = (($42) + 4)|0;
  $46 = $45;
  $47 = HEAP32[$46>>2]|0;
  $48 = (_sqlite3OsRead($37,$40,$41,$44,$47)|0);
  HEAP32[$rc>>2] = $48;
  $49 = HEAP32[$rc>>2]|0;
  $50 = ($49|0)==(522);
  $$ = $50 ? 0 : $48;
  HEAP32[$rc>>2] = $$;
 }
 $51 = HEAP32[$pgno>>2]|0;
 $52 = ($51|0)==(1);
 if (!($52)) {
  $64 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($64|0);
 }
 $53 = HEAP32[$rc>>2]|0;
 $54 = ($53|0)!=(0);
 if ($54) {
  $55 = HEAP32[$pPager>>2]|0;
  $56 = (($55) + 100|0);
  dest=$56+0|0; stop=dest+16|0; do { HEAP8[dest>>0]=255|0; dest=dest+1|0; } while ((dest|0) < (stop|0));
  $64 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($64|0);
 } else {
  $57 = HEAP32[$0>>2]|0;
  $58 = (($57) + 4|0);
  $59 = HEAP32[$58>>2]|0;
  $60 = (($59) + 24|0);
  HEAP32[$dbFileVers>>2] = $60;
  $61 = HEAP32[$pPager>>2]|0;
  $62 = (($61) + 100|0);
  $63 = HEAP32[$dbFileVers>>2]|0;
  dest=$62+0|0; src=$63+0|0; stop=dest+16|0; do { HEAP8[dest>>0]=HEAP8[src>>0]|0; dest=dest+1|0; src=src+1|0; } while ((dest|0) < (stop|0));
  $64 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($64|0);
 }
 return 0|0;
}
function _sqlite3PagerUnrefNotNull($pPg) {
 $pPg = $pPg|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $pPager = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $pPager = sp;
 HEAP32[$0>>2] = $pPg;
 $1 = HEAP32[$0>>2]|0;
 $2 = (($1) + 16|0);
 $3 = HEAP32[$2>>2]|0;
 HEAP32[$pPager>>2] = $3;
 $4 = HEAP32[$0>>2]|0;
 $5 = (($4) + 24|0);
 $6 = HEAP16[$5>>1]|0;
 $7 = $6&65535;
 $8 = $7 & 64;
 $9 = ($8|0)!=(0);
 $10 = HEAP32[$0>>2]|0;
 if ($9) {
  _pagerReleaseMapPage($10);
  $11 = HEAP32[$pPager>>2]|0;
  _pagerUnlockIfUnused($11);
  STACKTOP = sp;return;
 } else {
  _sqlite3PcacheRelease($10);
  $11 = HEAP32[$pPager>>2]|0;
  _pagerUnlockIfUnused($11);
  STACKTOP = sp;return;
 }
}
function _pagerReleaseMapPage($pPg) {
 $pPg = $pPg|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $pPager = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $pPager = sp;
 HEAP32[$0>>2] = $pPg;
 $1 = HEAP32[$0>>2]|0;
 $2 = (($1) + 16|0);
 $3 = HEAP32[$2>>2]|0;
 HEAP32[$pPager>>2] = $3;
 $4 = HEAP32[$pPager>>2]|0;
 $5 = (($4) + 120|0);
 $6 = HEAP32[$5>>2]|0;
 $7 = (($6) + -1)|0;
 HEAP32[$5>>2] = $7;
 $8 = HEAP32[$pPager>>2]|0;
 $9 = (($8) + 136|0);
 $10 = HEAP32[$9>>2]|0;
 $11 = HEAP32[$0>>2]|0;
 $12 = (($11) + 12|0);
 HEAP32[$12>>2] = $10;
 $13 = HEAP32[$0>>2]|0;
 $14 = HEAP32[$pPager>>2]|0;
 $15 = (($14) + 136|0);
 HEAP32[$15>>2] = $13;
 $16 = HEAP32[$pPager>>2]|0;
 $17 = (($16) + 60|0);
 $18 = HEAP32[$17>>2]|0;
 $19 = HEAP32[$0>>2]|0;
 $20 = (($19) + 20|0);
 $21 = HEAP32[$20>>2]|0;
 $22 = (($21) - 1)|0;
 $23 = HEAP32[$pPager>>2]|0;
 $24 = (($23) + 152|0);
 $25 = HEAP32[$24>>2]|0;
 $26 = ($25|0)<(0);
 $27 = $26 << 31 >> 31;
 $28 = (___muldi3(($22|0),0,($25|0),($27|0))|0);
 $29 = tempRet0;
 $30 = HEAP32[$0>>2]|0;
 $31 = (($30) + 4|0);
 $32 = HEAP32[$31>>2]|0;
 (_sqlite3OsUnfetch($18,$28,$29,$32)|0);
 STACKTOP = sp;return;
}
function _pagerUnlockIfUnused($pPager) {
 $pPager = $pPager|0;
 var $0 = 0, $1 = 0, $10 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 HEAP32[$0>>2] = $pPager;
 $1 = HEAP32[$0>>2]|0;
 $2 = (($1) + 120|0);
 $3 = HEAP32[$2>>2]|0;
 $4 = ($3|0)==(0);
 if (!($4)) {
  STACKTOP = sp;return;
 }
 $5 = HEAP32[$0>>2]|0;
 $6 = (($5) + 204|0);
 $7 = HEAP32[$6>>2]|0;
 $8 = (_sqlite3PcacheRefCount($7)|0);
 $9 = ($8|0)==(0);
 if (!($9)) {
  STACKTOP = sp;return;
 }
 $10 = HEAP32[$0>>2]|0;
 _pagerUnlockAndRollback($10);
 STACKTOP = sp;return;
}
function _sqlite3OsUnfetch($id,$0,$1,$p) {
 $id = $id|0;
 $0 = $0|0;
 $1 = $1|0;
 $p = $p|0;
 var $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $2 = sp + 12|0;
 $3 = sp;
 $4 = sp + 8|0;
 HEAP32[$2>>2] = $id;
 $5 = $3;
 $6 = $5;
 HEAP32[$6>>2] = $0;
 $7 = (($5) + 4)|0;
 $8 = $7;
 HEAP32[$8>>2] = $1;
 HEAP32[$4>>2] = $p;
 STACKTOP = sp;return 0;
}
function _sqlite3WalReadFrame($pWal,$iRead,$nOut,$pOut) {
 $pWal = $pWal|0;
 $iRead = $iRead|0;
 $nOut = $nOut|0;
 $pOut = $pOut|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $iOffset = 0, $sz = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 24|0;
 $1 = sp + 20|0;
 $2 = sp + 16|0;
 $3 = sp + 12|0;
 $sz = sp + 8|0;
 $iOffset = sp;
 HEAP32[$0>>2] = $pWal;
 HEAP32[$1>>2] = $iRead;
 HEAP32[$2>>2] = $nOut;
 HEAP32[$3>>2] = $pOut;
 $4 = HEAP32[$0>>2]|0;
 $5 = (($4) + 52|0);
 $6 = (($5) + 14|0);
 $7 = HEAP16[$6>>1]|0;
 $8 = $7&65535;
 HEAP32[$sz>>2] = $8;
 $9 = HEAP32[$sz>>2]|0;
 $10 = $9 & 65024;
 $11 = HEAP32[$sz>>2]|0;
 $12 = $11 & 1;
 $13 = $12 << 16;
 $14 = (($10) + ($13))|0;
 HEAP32[$sz>>2] = $14;
 $15 = HEAP32[$1>>2]|0;
 $16 = (($15) - 1)|0;
 $17 = HEAP32[$sz>>2]|0;
 $18 = (($17) + 24)|0;
 $19 = ($18|0)<(0);
 $20 = $19 << 31 >> 31;
 $21 = (___muldi3(($16|0),0,($18|0),($20|0))|0);
 $22 = tempRet0;
 $23 = (_i64Add(32,0,($21|0),($22|0))|0);
 $24 = tempRet0;
 $25 = (_i64Add(($23|0),($24|0),24,0)|0);
 $26 = tempRet0;
 $27 = $iOffset;
 $28 = $27;
 HEAP32[$28>>2] = $25;
 $29 = (($27) + 4)|0;
 $30 = $29;
 HEAP32[$30>>2] = $26;
 $31 = HEAP32[$0>>2]|0;
 $32 = (($31) + 8|0);
 $33 = HEAP32[$32>>2]|0;
 $34 = HEAP32[$3>>2]|0;
 $35 = HEAP32[$2>>2]|0;
 $36 = HEAP32[$sz>>2]|0;
 $37 = ($35|0)>($36|0);
 $38 = HEAP32[$sz>>2]|0;
 $39 = HEAP32[$2>>2]|0;
 $40 = $37 ? $38 : $39;
 $41 = $iOffset;
 $42 = $41;
 $43 = HEAP32[$42>>2]|0;
 $44 = (($41) + 4)|0;
 $45 = $44;
 $46 = HEAP32[$45>>2]|0;
 $47 = (_sqlite3OsRead($33,$34,$40,$43,$46)|0);
 STACKTOP = sp;return ($47|0);
}
function _walHash($iPage) {
 $iPage = $iPage|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 HEAP32[$0>>2] = $iPage;
 $1 = HEAP32[$0>>2]|0;
 $2 = ($1*383)|0;
 $3 = $2 & 8191;
 STACKTOP = sp;return ($3|0);
}
function _sqlite3CorruptError($lineno) {
 $lineno = $lineno|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $vararg_buffer = 0, $vararg_ptr1 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $vararg_buffer = sp;
 $0 = sp + 8|0;
 HEAP32[$0>>2] = $lineno;
 $1 = HEAP32[$0>>2]|0;
 $2 = (_sqlite3_sourceid()|0);
 $3 = (($2) + 20|0);
 HEAP32[$vararg_buffer>>2] = $1;
 $vararg_ptr1 = (($vararg_buffer) + 4|0);
 HEAP32[$vararg_ptr1>>2] = $3;
 _sqlite3_log(11,4392,$vararg_buffer);
 STACKTOP = sp;return 11;
}
function _walNextHash($iPriorHash) {
 $iPriorHash = $iPriorHash|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 HEAP32[$0>>2] = $iPriorHash;
 $1 = HEAP32[$0>>2]|0;
 $2 = (($1) + 1)|0;
 $3 = $2 & 8191;
 STACKTOP = sp;return ($3|0);
}
function _walIndexHdr($pWal) {
 $pWal = $pWal|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 HEAP32[$0>>2] = $pWal;
 $1 = HEAP32[$0>>2]|0;
 $2 = (($1) + 32|0);
 $3 = HEAP32[$2>>2]|0;
 $4 = HEAP32[$3>>2]|0;
 STACKTOP = sp;return ($4|0);
}
function _walFramePgno($pWal,$iFrame) {
 $pWal = $pWal|0;
 $iFrame = $iFrame|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $iHash = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $1 = sp + 8|0;
 $2 = sp + 4|0;
 $iHash = sp;
 HEAP32[$1>>2] = $pWal;
 HEAP32[$2>>2] = $iFrame;
 $3 = HEAP32[$2>>2]|0;
 $4 = (_walFramePage($3)|0);
 HEAP32[$iHash>>2] = $4;
 $5 = HEAP32[$iHash>>2]|0;
 $6 = ($5|0)==(0);
 $7 = HEAP32[$2>>2]|0;
 if ($6) {
  $8 = (34 + ($7))|0;
  $9 = (($8) - 1)|0;
  $10 = HEAP32[$1>>2]|0;
  $11 = (($10) + 32|0);
  $12 = HEAP32[$11>>2]|0;
  $13 = HEAP32[$12>>2]|0;
  $14 = (($13) + ($9<<2)|0);
  $15 = HEAP32[$14>>2]|0;
  HEAP32[$0>>2] = $15;
  $27 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($27|0);
 } else {
  $16 = (($7) - 1)|0;
  $17 = (($16) - 4062)|0;
  $18 = (($17>>>0) % 4096)&-1;
  $19 = HEAP32[$iHash>>2]|0;
  $20 = HEAP32[$1>>2]|0;
  $21 = (($20) + 32|0);
  $22 = HEAP32[$21>>2]|0;
  $23 = (($22) + ($19<<2)|0);
  $24 = HEAP32[$23>>2]|0;
  $25 = (($24) + ($18<<2)|0);
  $26 = HEAP32[$25>>2]|0;
  HEAP32[$0>>2] = $26;
  $27 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($27|0);
 }
 return 0|0;
}
function _sqlite3WalEndReadTransaction($pWal) {
 $pWal = $pWal|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 HEAP32[$0>>2] = $pWal;
 $1 = HEAP32[$0>>2]|0;
 (_sqlite3WalEndWriteTransaction($1)|0);
 $2 = HEAP32[$0>>2]|0;
 $3 = (($2) + 40|0);
 $4 = HEAP16[$3>>1]|0;
 $5 = $4 << 16 >> 16;
 $6 = ($5|0)>=(0);
 if (!($6)) {
  STACKTOP = sp;return;
 }
 $7 = HEAP32[$0>>2]|0;
 $8 = HEAP32[$0>>2]|0;
 $9 = (($8) + 40|0);
 $10 = HEAP16[$9>>1]|0;
 $11 = $10 << 16 >> 16;
 $12 = (3 + ($11))|0;
 _walUnlockShared($7,$12);
 $13 = HEAP32[$0>>2]|0;
 $14 = (($13) + 40|0);
 HEAP16[$14>>1] = -1;
 STACKTOP = sp;return;
}
function _sqlite3OsLock($id,$lockType) {
 $id = $id|0;
 $lockType = $lockType|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $1 = sp;
 HEAP32[$0>>2] = $id;
 HEAP32[$1>>2] = $lockType;
 $2 = HEAP32[$0>>2]|0;
 $3 = HEAP32[$2>>2]|0;
 $4 = (($3) + 28|0);
 $5 = HEAP32[$4>>2]|0;
 $6 = HEAP32[$0>>2]|0;
 $7 = HEAP32[$1>>2]|0;
 $8 = (FUNCTION_TABLE_iii[$5 & 63]($6,$7)|0);
 STACKTOP = sp;return ($8|0);
}
function _sqlite3WalCheckpoint($pWal,$eMode,$xBusy,$pBusyArg,$sync_flags,$nBuf,$zBuf,$pnLog,$pnCkpt) {
 $pWal = $pWal|0;
 $eMode = $eMode|0;
 $xBusy = $xBusy|0;
 $pBusyArg = $pBusyArg|0;
 $sync_flags = $sync_flags|0;
 $nBuf = $nBuf|0;
 $zBuf = $zBuf|0;
 $pnLog = $pnLog|0;
 $pnCkpt = $pnCkpt|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0;
 var $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0;
 var $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0;
 var $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0;
 var $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0;
 var $99 = 0, $eMode2 = 0, $isChanged = 0, $or$cond = 0, $rc = 0, dest = 0, label = 0, sp = 0, stop = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 64|0;
 $0 = sp + 48|0;
 $1 = sp + 44|0;
 $2 = sp + 40|0;
 $3 = sp + 36|0;
 $4 = sp + 32|0;
 $5 = sp + 28|0;
 $6 = sp + 24|0;
 $7 = sp + 20|0;
 $8 = sp + 16|0;
 $9 = sp + 12|0;
 $rc = sp + 8|0;
 $isChanged = sp + 4|0;
 $eMode2 = sp;
 HEAP32[$1>>2] = $pWal;
 HEAP32[$2>>2] = $eMode;
 HEAP32[$3>>2] = $xBusy;
 HEAP32[$4>>2] = $pBusyArg;
 HEAP32[$5>>2] = $sync_flags;
 HEAP32[$6>>2] = $nBuf;
 HEAP32[$7>>2] = $zBuf;
 HEAP32[$8>>2] = $pnLog;
 HEAP32[$9>>2] = $pnCkpt;
 HEAP32[$isChanged>>2] = 0;
 $10 = HEAP32[$2>>2]|0;
 HEAP32[$eMode2>>2] = $10;
 $11 = HEAP32[$1>>2]|0;
 $12 = (($11) + 46|0);
 $13 = HEAP8[$12>>0]|0;
 $14 = ($13<<24>>24)!=(0);
 if ($14) {
  HEAP32[$0>>2] = 8;
  $100 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($100|0);
 }
 $15 = HEAP32[$1>>2]|0;
 $16 = (_walLockExclusive($15,1,1)|0);
 HEAP32[$rc>>2] = $16;
 $17 = HEAP32[$rc>>2]|0;
 $18 = ($17|0)!=(0);
 if ($18) {
  $19 = HEAP32[$rc>>2]|0;
  HEAP32[$0>>2] = $19;
  $100 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($100|0);
 }
 $20 = HEAP32[$1>>2]|0;
 $21 = (($20) + 45|0);
 HEAP8[$21>>0] = 1;
 $22 = HEAP32[$2>>2]|0;
 $23 = ($22|0)!=(0);
 do {
  if ($23) {
   $24 = HEAP32[$1>>2]|0;
   $25 = HEAP32[$3>>2]|0;
   $26 = HEAP32[$4>>2]|0;
   $27 = (_walBusyLock($24,$25,$26,0,1)|0);
   HEAP32[$rc>>2] = $27;
   $28 = HEAP32[$rc>>2]|0;
   $29 = ($28|0)==(0);
   if ($29) {
    $30 = HEAP32[$1>>2]|0;
    $31 = (($30) + 44|0);
    HEAP8[$31>>0] = 1;
    break;
   }
   $32 = HEAP32[$rc>>2]|0;
   $33 = ($32|0)==(5);
   if ($33) {
    HEAP32[$eMode2>>2] = 0;
    HEAP32[$rc>>2] = 0;
   }
  }
 } while(0);
 $34 = HEAP32[$rc>>2]|0;
 $35 = ($34|0)==(0);
 if ($35) {
  $36 = HEAP32[$1>>2]|0;
  $37 = (_walIndexReadHdr($36,$isChanged)|0);
  HEAP32[$rc>>2] = $37;
  $38 = HEAP32[$isChanged>>2]|0;
  $39 = ($38|0)!=(0);
  if ($39) {
   $40 = HEAP32[$1>>2]|0;
   $41 = (($40) + 4|0);
   $42 = HEAP32[$41>>2]|0;
   $43 = HEAP32[$42>>2]|0;
   $44 = HEAP32[$43>>2]|0;
   $45 = ($44|0)>=(3);
   if ($45) {
    $46 = HEAP32[$1>>2]|0;
    $47 = (($46) + 4|0);
    $48 = HEAP32[$47>>2]|0;
    (_sqlite3OsUnfetch($48,0,0,0)|0);
   }
  }
 }
 $49 = HEAP32[$rc>>2]|0;
 $50 = ($49|0)==(0);
 if ($50) {
  $51 = HEAP32[$1>>2]|0;
  $52 = (($51) + 52|0);
  $53 = (($52) + 16|0);
  $54 = HEAP32[$53>>2]|0;
  $55 = ($54|0)!=(0);
  if ($55) {
   $56 = HEAP32[$1>>2]|0;
   $57 = (_walPagesize($56)|0);
   $58 = HEAP32[$6>>2]|0;
   $59 = ($57|0)!=($58|0);
   if ($59) {
    $60 = (_sqlite3CorruptError(51084)|0);
    HEAP32[$rc>>2] = $60;
   } else {
    label = 18;
   }
  } else {
   label = 18;
  }
  if ((label|0) == 18) {
   $61 = HEAP32[$1>>2]|0;
   $62 = HEAP32[$eMode2>>2]|0;
   $63 = HEAP32[$3>>2]|0;
   $64 = HEAP32[$4>>2]|0;
   $65 = HEAP32[$5>>2]|0;
   $66 = HEAP32[$7>>2]|0;
   $67 = (_walCheckpoint($61,$62,$63,$64,$65,$66)|0);
   HEAP32[$rc>>2] = $67;
  }
  $68 = HEAP32[$rc>>2]|0;
  $69 = ($68|0)==(0);
  $70 = HEAP32[$rc>>2]|0;
  $71 = ($70|0)==(5);
  $or$cond = $69 | $71;
  if ($or$cond) {
   $72 = HEAP32[$8>>2]|0;
   $73 = ($72|0)!=(0|0);
   if ($73) {
    $74 = HEAP32[$1>>2]|0;
    $75 = (($74) + 52|0);
    $76 = (($75) + 16|0);
    $77 = HEAP32[$76>>2]|0;
    $78 = HEAP32[$8>>2]|0;
    HEAP32[$78>>2] = $77;
   }
   $79 = HEAP32[$9>>2]|0;
   $80 = ($79|0)!=(0|0);
   if ($80) {
    $81 = HEAP32[$1>>2]|0;
    $82 = (_walCkptInfo($81)|0);
    $83 = HEAP32[$82>>2]|0;
    $84 = HEAP32[$9>>2]|0;
    HEAP32[$84>>2] = $83;
   }
  }
 }
 $85 = HEAP32[$isChanged>>2]|0;
 $86 = ($85|0)!=(0);
 if ($86) {
  $87 = HEAP32[$1>>2]|0;
  $88 = (($87) + 52|0);
  dest=$88+0|0; stop=dest+48|0; do { HEAP32[dest>>2]=0|0; dest=dest+4|0; } while ((dest|0) < (stop|0));
 }
 $89 = HEAP32[$1>>2]|0;
 (_sqlite3WalEndWriteTransaction($89)|0);
 $90 = HEAP32[$1>>2]|0;
 _walUnlockExclusive($90,1,1);
 $91 = HEAP32[$1>>2]|0;
 $92 = (($91) + 45|0);
 HEAP8[$92>>0] = 0;
 $93 = HEAP32[$rc>>2]|0;
 $94 = ($93|0)==(0);
 if ($94) {
  $95 = HEAP32[$2>>2]|0;
  $96 = HEAP32[$eMode2>>2]|0;
  $97 = ($95|0)!=($96|0);
  if ($97) {
   $99 = 5;
  } else {
   label = 28;
  }
 } else {
  label = 28;
 }
 if ((label|0) == 28) {
  $98 = HEAP32[$rc>>2]|0;
  $99 = $98;
 }
 HEAP32[$0>>2] = $99;
 $100 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($100|0);
}
function _sqlite3OsFileControlHint($id,$op,$pArg) {
 $id = $id|0;
 $op = $op|0;
 $pArg = $pArg|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $1 = sp + 4|0;
 $2 = sp;
 HEAP32[$0>>2] = $id;
 HEAP32[$1>>2] = $op;
 HEAP32[$2>>2] = $pArg;
 $3 = HEAP32[$0>>2]|0;
 $4 = HEAP32[$3>>2]|0;
 $5 = (($4) + 40|0);
 $6 = HEAP32[$5>>2]|0;
 $7 = HEAP32[$0>>2]|0;
 $8 = HEAP32[$1>>2]|0;
 $9 = HEAP32[$2>>2]|0;
 (FUNCTION_TABLE_iiii[$6 & 31]($7,$8,$9)|0);
 STACKTOP = sp;return;
}
function _walLimitSize($pWal,$0,$1) {
 $pWal = $pWal|0;
 $0 = $0|0;
 $1 = $1|0;
 var $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0;
 var $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0;
 var $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $rx = 0, $sz = 0, $vararg_buffer = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $vararg_buffer = sp + 16|0;
 $2 = sp + 24|0;
 $3 = sp + 8|0;
 $sz = sp;
 $rx = sp + 20|0;
 HEAP32[$2>>2] = $pWal;
 $4 = $3;
 $5 = $4;
 HEAP32[$5>>2] = $0;
 $6 = (($4) + 4)|0;
 $7 = $6;
 HEAP32[$7>>2] = $1;
 _sqlite3BeginBenignMalloc();
 $8 = HEAP32[$2>>2]|0;
 $9 = (($8) + 8|0);
 $10 = HEAP32[$9>>2]|0;
 $11 = (_sqlite3OsFileSize($10,$sz)|0);
 HEAP32[$rx>>2] = $11;
 $12 = HEAP32[$rx>>2]|0;
 $13 = ($12|0)==(0);
 if ($13) {
  $14 = $sz;
  $15 = $14;
  $16 = HEAP32[$15>>2]|0;
  $17 = (($14) + 4)|0;
  $18 = $17;
  $19 = HEAP32[$18>>2]|0;
  $20 = $3;
  $21 = $20;
  $22 = HEAP32[$21>>2]|0;
  $23 = (($20) + 4)|0;
  $24 = $23;
  $25 = HEAP32[$24>>2]|0;
  $26 = ($19|0)>($25|0);
  $27 = ($16>>>0)>($22>>>0);
  $28 = ($19|0)==($25|0);
  $29 = $28 & $27;
  $30 = $26 | $29;
  if ($30) {
   $31 = HEAP32[$2>>2]|0;
   $32 = (($31) + 8|0);
   $33 = HEAP32[$32>>2]|0;
   $34 = $3;
   $35 = $34;
   $36 = HEAP32[$35>>2]|0;
   $37 = (($34) + 4)|0;
   $38 = $37;
   $39 = HEAP32[$38>>2]|0;
   $40 = (_sqlite3OsTruncate($33,$36,$39)|0);
   HEAP32[$rx>>2] = $40;
  }
 }
 _sqlite3EndBenignMalloc();
 $41 = HEAP32[$rx>>2]|0;
 $42 = ($41|0)!=(0);
 if (!($42)) {
  STACKTOP = sp;return;
 }
 $43 = HEAP32[$rx>>2]|0;
 $44 = HEAP32[$2>>2]|0;
 $45 = (($44) + 100|0);
 $46 = HEAP32[$45>>2]|0;
 HEAP32[$vararg_buffer>>2] = $46;
 _sqlite3_log($43,4440,$vararg_buffer);
 STACKTOP = sp;return;
}
function _walIndexClose($pWal,$isDelete) {
 $pWal = $pWal|0;
 $isDelete = $isDelete|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $i = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $1 = sp + 4|0;
 $i = sp;
 HEAP32[$0>>2] = $pWal;
 HEAP32[$1>>2] = $isDelete;
 $2 = HEAP32[$0>>2]|0;
 $3 = (($2) + 43|0);
 $4 = HEAP8[$3>>0]|0;
 $5 = $4&255;
 $6 = ($5|0)==(2);
 if (!($6)) {
  $25 = HEAP32[$0>>2]|0;
  $26 = (($25) + 4|0);
  $27 = HEAP32[$26>>2]|0;
  $28 = HEAP32[$1>>2]|0;
  (_sqlite3OsShmUnmap($27,$28)|0);
  STACKTOP = sp;return;
 }
 HEAP32[$i>>2] = 0;
 while(1) {
  $7 = HEAP32[$i>>2]|0;
  $8 = HEAP32[$0>>2]|0;
  $9 = (($8) + 24|0);
  $10 = HEAP32[$9>>2]|0;
  $11 = ($7|0)<($10|0);
  if (!($11)) {
   break;
  }
  $12 = HEAP32[$i>>2]|0;
  $13 = HEAP32[$0>>2]|0;
  $14 = (($13) + 32|0);
  $15 = HEAP32[$14>>2]|0;
  $16 = (($15) + ($12<<2)|0);
  $17 = HEAP32[$16>>2]|0;
  _sqlite3_free($17);
  $18 = HEAP32[$i>>2]|0;
  $19 = HEAP32[$0>>2]|0;
  $20 = (($19) + 32|0);
  $21 = HEAP32[$20>>2]|0;
  $22 = (($21) + ($18<<2)|0);
  HEAP32[$22>>2] = 0;
  $23 = HEAP32[$i>>2]|0;
  $24 = (($23) + 1)|0;
  HEAP32[$i>>2] = $24;
 }
 STACKTOP = sp;return;
}
function _sqlite3OsShmUnmap($id,$deleteFlag) {
 $id = $id|0;
 $deleteFlag = $deleteFlag|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $1 = sp;
 HEAP32[$0>>2] = $id;
 HEAP32[$1>>2] = $deleteFlag;
 $2 = HEAP32[$0>>2]|0;
 $3 = HEAP32[$2>>2]|0;
 $4 = (($3) + 64|0);
 $5 = HEAP32[$4>>2]|0;
 $6 = HEAP32[$0>>2]|0;
 $7 = HEAP32[$1>>2]|0;
 $8 = (FUNCTION_TABLE_iii[$5 & 63]($6,$7)|0);
 STACKTOP = sp;return ($8|0);
}
function _walLockExclusive($pWal,$lockIdx,$n) {
 $pWal = $pWal|0;
 $lockIdx = $lockIdx|0;
 $n = $n|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 16|0;
 $1 = sp + 12|0;
 $2 = sp + 8|0;
 $3 = sp + 4|0;
 $rc = sp;
 HEAP32[$1>>2] = $pWal;
 HEAP32[$2>>2] = $lockIdx;
 HEAP32[$3>>2] = $n;
 $4 = HEAP32[$1>>2]|0;
 $5 = (($4) + 43|0);
 $6 = HEAP8[$5>>0]|0;
 $7 = ($6<<24>>24)!=(0);
 if ($7) {
  HEAP32[$0>>2] = 0;
  $15 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($15|0);
 } else {
  $8 = HEAP32[$1>>2]|0;
  $9 = (($8) + 4|0);
  $10 = HEAP32[$9>>2]|0;
  $11 = HEAP32[$2>>2]|0;
  $12 = HEAP32[$3>>2]|0;
  $13 = (_sqlite3OsShmLock($10,$11,$12,10)|0);
  HEAP32[$rc>>2] = $13;
  $14 = HEAP32[$rc>>2]|0;
  HEAP32[$0>>2] = $14;
  $15 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($15|0);
 }
 return 0|0;
}
function _walBusyLock($pWal,$xBusy,$pBusyArg,$lockIdx,$n) {
 $pWal = $pWal|0;
 $xBusy = $xBusy|0;
 $pBusyArg = $pBusyArg|0;
 $lockIdx = $lockIdx|0;
 $n = $n|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $or$cond = 0, $rc = 0;
 var label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 20|0;
 $1 = sp + 16|0;
 $2 = sp + 12|0;
 $3 = sp + 8|0;
 $4 = sp + 4|0;
 $rc = sp;
 HEAP32[$0>>2] = $pWal;
 HEAP32[$1>>2] = $xBusy;
 HEAP32[$2>>2] = $pBusyArg;
 HEAP32[$3>>2] = $lockIdx;
 HEAP32[$4>>2] = $n;
 while(1) {
  $5 = HEAP32[$0>>2]|0;
  $6 = HEAP32[$3>>2]|0;
  $7 = HEAP32[$4>>2]|0;
  $8 = (_walLockExclusive($5,$6,$7)|0);
  HEAP32[$rc>>2] = $8;
  $9 = HEAP32[$1>>2]|0;
  $10 = ($9|0)!=(0|0);
  $11 = HEAP32[$rc>>2]|0;
  $12 = ($11|0)==(5);
  $or$cond = $10 & $12;
  if (!($or$cond)) {
   label = 4;
   break;
  }
  $13 = HEAP32[$1>>2]|0;
  $14 = HEAP32[$2>>2]|0;
  $15 = (FUNCTION_TABLE_ii[$13 & 31]($14)|0);
  $16 = ($15|0)!=(0);
  if (!($16)) {
   label = 4;
   break;
  }
 }
 if ((label|0) == 4) {
  $17 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($17|0);
 }
 return 0|0;
}
function _walIndexReadHdr($pWal,$pChanged) {
 $pWal = $pWal|0;
 $pChanged = $pChanged|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $badHdr = 0, $page0 = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 20|0;
 $1 = sp + 16|0;
 $2 = sp + 12|0;
 $rc = sp + 8|0;
 $badHdr = sp + 4|0;
 $page0 = sp;
 HEAP32[$1>>2] = $pWal;
 HEAP32[$2>>2] = $pChanged;
 $3 = HEAP32[$1>>2]|0;
 $4 = (_walIndexPage($3,0,$page0)|0);
 HEAP32[$rc>>2] = $4;
 $5 = HEAP32[$rc>>2]|0;
 $6 = ($5|0)!=(0);
 if ($6) {
  $7 = HEAP32[$rc>>2]|0;
  HEAP32[$0>>2] = $7;
  $52 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($52|0);
 }
 $8 = HEAP32[$page0>>2]|0;
 $9 = ($8|0)!=(0|0);
 if ($9) {
  $10 = HEAP32[$1>>2]|0;
  $11 = HEAP32[$2>>2]|0;
  $12 = (_walIndexTryHdr($10,$11)|0);
  $13 = $12;
 } else {
  $13 = 1;
 }
 HEAP32[$badHdr>>2] = $13;
 $14 = HEAP32[$badHdr>>2]|0;
 $15 = ($14|0)!=(0);
 do {
  if ($15) {
   $16 = HEAP32[$1>>2]|0;
   $17 = (($16) + 46|0);
   $18 = HEAP8[$17>>0]|0;
   $19 = $18&255;
   $20 = $19 & 2;
   $21 = ($20|0)!=(0);
   $22 = HEAP32[$1>>2]|0;
   if ($21) {
    $23 = (_walLockShared($22,0)|0);
    HEAP32[$rc>>2] = $23;
    $24 = (0)==($23|0);
    if (!($24)) {
     break;
    }
    $25 = HEAP32[$1>>2]|0;
    _walUnlockShared($25,0);
    HEAP32[$rc>>2] = 264;
    break;
   }
   $26 = (_walLockExclusive($22,0,1)|0);
   HEAP32[$rc>>2] = $26;
   $27 = (0)==($26|0);
   if ($27) {
    $28 = HEAP32[$1>>2]|0;
    $29 = (($28) + 44|0);
    HEAP8[$29>>0] = 1;
    $30 = HEAP32[$1>>2]|0;
    $31 = (_walIndexPage($30,0,$page0)|0);
    HEAP32[$rc>>2] = $31;
    $32 = (0)==($31|0);
    if ($32) {
     $33 = HEAP32[$1>>2]|0;
     $34 = HEAP32[$2>>2]|0;
     $35 = (_walIndexTryHdr($33,$34)|0);
     HEAP32[$badHdr>>2] = $35;
     $36 = HEAP32[$badHdr>>2]|0;
     $37 = ($36|0)!=(0);
     if ($37) {
      $38 = HEAP32[$1>>2]|0;
      $39 = (_walIndexRecover($38)|0);
      HEAP32[$rc>>2] = $39;
      $40 = HEAP32[$2>>2]|0;
      HEAP32[$40>>2] = 1;
     }
    }
    $41 = HEAP32[$1>>2]|0;
    $42 = (($41) + 44|0);
    HEAP8[$42>>0] = 0;
    $43 = HEAP32[$1>>2]|0;
    _walUnlockExclusive($43,0,1);
   }
  }
 } while(0);
 $44 = HEAP32[$badHdr>>2]|0;
 $45 = ($44|0)==(0);
 if ($45) {
  $46 = HEAP32[$1>>2]|0;
  $47 = (($46) + 52|0);
  $48 = HEAP32[$47>>2]|0;
  $49 = ($48|0)!=(3007000);
  if ($49) {
   $50 = (_sqlite3CantopenError(50131)|0);
   HEAP32[$rc>>2] = $50;
  }
 }
 $51 = HEAP32[$rc>>2]|0;
 HEAP32[$0>>2] = $51;
 $52 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($52|0);
}
function _walPagesize($pWal) {
 $pWal = $pWal|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 HEAP32[$0>>2] = $pWal;
 $1 = HEAP32[$0>>2]|0;
 $2 = (($1) + 52|0);
 $3 = (($2) + 14|0);
 $4 = HEAP16[$3>>1]|0;
 $5 = $4&65535;
 $6 = $5 & 65024;
 $7 = HEAP32[$0>>2]|0;
 $8 = (($7) + 52|0);
 $9 = (($8) + 14|0);
 $10 = HEAP16[$9>>1]|0;
 $11 = $10&65535;
 $12 = $11 & 1;
 $13 = $12 << 16;
 $14 = (($6) + ($13))|0;
 STACKTOP = sp;return ($14|0);
}
function _walCheckpoint($pWal,$eMode,$xBusyCall,$pBusyArg,$sync_flags,$zBuf) {
 $pWal = $pWal|0;
 $eMode = $eMode|0;
 $xBusyCall = $xBusyCall|0;
 $pBusyArg = $pBusyArg|0;
 $sync_flags = $sync_flags|0;
 $zBuf = $zBuf|0;
 var $$old = 0, $$old4 = 0, $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0;
 var $114 = 0, $115 = 0, $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0;
 var $132 = 0, $133 = 0, $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0;
 var $150 = 0, $151 = 0, $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0;
 var $169 = 0, $17 = 0, $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0;
 var $187 = 0, $188 = 0, $189 = 0, $19 = 0, $190 = 0, $191 = 0, $192 = 0, $193 = 0, $194 = 0, $195 = 0, $196 = 0, $197 = 0, $198 = 0, $199 = 0, $2 = 0, $20 = 0, $200 = 0, $201 = 0, $202 = 0, $203 = 0;
 var $204 = 0, $205 = 0, $206 = 0, $207 = 0, $208 = 0, $209 = 0, $21 = 0, $210 = 0, $211 = 0, $212 = 0, $213 = 0, $214 = 0, $215 = 0, $216 = 0, $217 = 0, $218 = 0, $219 = 0, $22 = 0, $220 = 0, $221 = 0;
 var $222 = 0, $223 = 0, $224 = 0, $225 = 0, $226 = 0, $227 = 0, $228 = 0, $229 = 0, $23 = 0, $230 = 0, $231 = 0, $232 = 0, $233 = 0, $234 = 0, $235 = 0, $236 = 0, $237 = 0, $238 = 0, $239 = 0, $24 = 0;
 var $240 = 0, $241 = 0, $242 = 0, $243 = 0, $244 = 0, $245 = 0, $246 = 0, $247 = 0, $248 = 0, $249 = 0, $25 = 0, $250 = 0, $251 = 0, $252 = 0, $253 = 0, $254 = 0, $255 = 0, $256 = 0, $257 = 0, $258 = 0;
 var $259 = 0, $26 = 0, $260 = 0, $261 = 0, $262 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0;
 var $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0;
 var $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0;
 var $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0;
 var $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $i = 0, $iDbpage = 0, $iFrame = 0, $iOffset = 0, $mxPage = 0, $mxSafeFrame = 0, $nBackfill = 0, $nReq = 0, $nSize = 0, $or$cond = 0, $or$cond3 = 0, $or$cond5 = 0, $pInfo = 0, $pIter = 0, $rc = 0;
 var $szDb = 0, $szPage = 0, $xBusy = 0, $y = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 112|0;
 $0 = sp + 104|0;
 $1 = sp + 100|0;
 $2 = sp + 96|0;
 $3 = sp + 92|0;
 $4 = sp + 88|0;
 $5 = sp + 84|0;
 $6 = sp + 80|0;
 $rc = sp + 76|0;
 $szPage = sp + 72|0;
 $pIter = sp + 68|0;
 $iDbpage = sp + 64|0;
 $iFrame = sp + 60|0;
 $mxSafeFrame = sp + 56|0;
 $mxPage = sp + 52|0;
 $i = sp + 48|0;
 $pInfo = sp + 44|0;
 $xBusy = sp + 40|0;
 $y = sp + 36|0;
 $nSize = sp + 24|0;
 $nBackfill = sp + 32|0;
 $nReq = sp + 16|0;
 $iOffset = sp + 8|0;
 $szDb = sp;
 HEAP32[$1>>2] = $pWal;
 HEAP32[$2>>2] = $eMode;
 HEAP32[$3>>2] = $xBusyCall;
 HEAP32[$4>>2] = $pBusyArg;
 HEAP32[$5>>2] = $sync_flags;
 HEAP32[$6>>2] = $zBuf;
 HEAP32[$pIter>>2] = 0;
 HEAP32[$iDbpage>>2] = 0;
 HEAP32[$iFrame>>2] = 0;
 HEAP32[$xBusy>>2] = 0;
 $7 = HEAP32[$1>>2]|0;
 $8 = (_walPagesize($7)|0);
 HEAP32[$szPage>>2] = $8;
 $9 = HEAP32[$1>>2]|0;
 $10 = (_walCkptInfo($9)|0);
 HEAP32[$pInfo>>2] = $10;
 $11 = HEAP32[$pInfo>>2]|0;
 $12 = HEAP32[$11>>2]|0;
 $13 = HEAP32[$1>>2]|0;
 $14 = (($13) + 52|0);
 $15 = (($14) + 16|0);
 $16 = HEAP32[$15>>2]|0;
 $17 = ($12>>>0)>=($16>>>0);
 if ($17) {
  HEAP32[$0>>2] = 0;
  $262 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($262|0);
 }
 $18 = HEAP32[$1>>2]|0;
 $19 = (_walIteratorInit($18,$pIter)|0);
 HEAP32[$rc>>2] = $19;
 $20 = HEAP32[$rc>>2]|0;
 $21 = ($20|0)!=(0);
 if ($21) {
  $22 = HEAP32[$rc>>2]|0;
  HEAP32[$0>>2] = $22;
  $262 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($262|0);
 }
 $23 = HEAP32[$2>>2]|0;
 $24 = ($23|0)!=(0);
 if ($24) {
  $25 = HEAP32[$3>>2]|0;
  HEAP32[$xBusy>>2] = $25;
 }
 $26 = HEAP32[$1>>2]|0;
 $27 = (($26) + 52|0);
 $28 = (($27) + 16|0);
 $29 = HEAP32[$28>>2]|0;
 HEAP32[$mxSafeFrame>>2] = $29;
 $30 = HEAP32[$1>>2]|0;
 $31 = (($30) + 52|0);
 $32 = (($31) + 20|0);
 $33 = HEAP32[$32>>2]|0;
 HEAP32[$mxPage>>2] = $33;
 HEAP32[$i>>2] = 1;
 L12: while(1) {
  $34 = HEAP32[$i>>2]|0;
  $35 = ($34|0)<(5);
  if (!($35)) {
   label = 15;
   break;
  }
  $36 = HEAP32[$i>>2]|0;
  $37 = HEAP32[$pInfo>>2]|0;
  $38 = (($37) + 4|0);
  $39 = (($38) + ($36<<2)|0);
  $40 = HEAP32[$39>>2]|0;
  HEAP32[$y>>2] = $40;
  $41 = HEAP32[$mxSafeFrame>>2]|0;
  $42 = HEAP32[$y>>2]|0;
  $43 = ($41>>>0)>($42>>>0);
  do {
   if ($43) {
    $44 = HEAP32[$1>>2]|0;
    $45 = HEAP32[$xBusy>>2]|0;
    $46 = HEAP32[$4>>2]|0;
    $47 = HEAP32[$i>>2]|0;
    $48 = (3 + ($47))|0;
    $49 = (_walBusyLock($44,$45,$46,$48,1)|0);
    HEAP32[$rc>>2] = $49;
    $50 = HEAP32[$rc>>2]|0;
    $51 = ($50|0)==(0);
    if ($51) {
     $52 = HEAP32[$i>>2]|0;
     $53 = ($52|0)==(1);
     $54 = HEAP32[$mxSafeFrame>>2]|0;
     $55 = $53 ? $54 : -1;
     $56 = HEAP32[$i>>2]|0;
     $57 = HEAP32[$pInfo>>2]|0;
     $58 = (($57) + 4|0);
     $59 = (($58) + ($56<<2)|0);
     HEAP32[$59>>2] = $55;
     $60 = HEAP32[$1>>2]|0;
     $61 = HEAP32[$i>>2]|0;
     $62 = (3 + ($61))|0;
     _walUnlockExclusive($60,$62,1);
     break;
    }
    $63 = HEAP32[$rc>>2]|0;
    $64 = ($63|0)==(5);
    if (!($64)) {
     break L12;
    }
    $65 = HEAP32[$y>>2]|0;
    HEAP32[$mxSafeFrame>>2] = $65;
    HEAP32[$xBusy>>2] = 0;
   }
  } while(0);
  $66 = HEAP32[$i>>2]|0;
  $67 = (($66) + 1)|0;
  HEAP32[$i>>2] = $67;
 }
 do {
  if ((label|0) == 15) {
   $68 = HEAP32[$pInfo>>2]|0;
   $69 = HEAP32[$68>>2]|0;
   $70 = HEAP32[$mxSafeFrame>>2]|0;
   $71 = ($69>>>0)<($70>>>0);
   if ($71) {
    $72 = HEAP32[$1>>2]|0;
    $73 = HEAP32[$xBusy>>2]|0;
    $74 = HEAP32[$4>>2]|0;
    $75 = (_walBusyLock($72,$73,$74,3,1)|0);
    HEAP32[$rc>>2] = $75;
    $76 = ($75|0)==(0);
    if ($76) {
     $77 = HEAP32[$pInfo>>2]|0;
     $78 = HEAP32[$77>>2]|0;
     HEAP32[$nBackfill>>2] = $78;
     $79 = HEAP32[$5>>2]|0;
     $80 = ($79|0)!=(0);
     if ($80) {
      $81 = HEAP32[$1>>2]|0;
      $82 = (($81) + 8|0);
      $83 = HEAP32[$82>>2]|0;
      $84 = HEAP32[$5>>2]|0;
      $85 = (_sqlite3OsSync($83,$84)|0);
      HEAP32[$rc>>2] = $85;
     }
     $86 = HEAP32[$rc>>2]|0;
     $87 = ($86|0)==(0);
     if ($87) {
      $88 = HEAP32[$mxPage>>2]|0;
      $89 = HEAP32[$szPage>>2]|0;
      $90 = ($89|0)<(0);
      $91 = $90 << 31 >> 31;
      $92 = (___muldi3(($88|0),0,($89|0),($91|0))|0);
      $93 = tempRet0;
      $94 = $nReq;
      $95 = $94;
      HEAP32[$95>>2] = $92;
      $96 = (($94) + 4)|0;
      $97 = $96;
      HEAP32[$97>>2] = $93;
      $98 = HEAP32[$1>>2]|0;
      $99 = (($98) + 4|0);
      $100 = HEAP32[$99>>2]|0;
      $101 = (_sqlite3OsFileSize($100,$nSize)|0);
      HEAP32[$rc>>2] = $101;
      $102 = HEAP32[$rc>>2]|0;
      $103 = ($102|0)==(0);
      if ($103) {
       $104 = $nSize;
       $105 = $104;
       $106 = HEAP32[$105>>2]|0;
       $107 = (($104) + 4)|0;
       $108 = $107;
       $109 = HEAP32[$108>>2]|0;
       $110 = $nReq;
       $111 = $110;
       $112 = HEAP32[$111>>2]|0;
       $113 = (($110) + 4)|0;
       $114 = $113;
       $115 = HEAP32[$114>>2]|0;
       $116 = ($109|0)<($115|0);
       $117 = ($106>>>0)<($112>>>0);
       $118 = ($109|0)==($115|0);
       $119 = $118 & $117;
       $120 = $116 | $119;
       if ($120) {
        $121 = HEAP32[$1>>2]|0;
        $122 = (($121) + 4|0);
        $123 = HEAP32[$122>>2]|0;
        _sqlite3OsFileControlHint($123,5,$nReq);
       }
      }
     }
     L34: while(1) {
      $$old = HEAP32[$rc>>2]|0;
      $$old4 = ($$old|0)==(0);
      if (!($$old4)) {
       break;
      }
      while(1) {
       $124 = HEAP32[$pIter>>2]|0;
       $125 = (_walIteratorNext($124,$iDbpage,$iFrame)|0);
       $126 = (0)==($125|0);
       if (!($126)) {
        break L34;
       }
       $127 = HEAP32[$iFrame>>2]|0;
       $128 = HEAP32[$nBackfill>>2]|0;
       $129 = ($127>>>0)<=($128>>>0);
       if ($129) {
        continue L34;
       }
       $130 = HEAP32[$iFrame>>2]|0;
       $131 = HEAP32[$mxSafeFrame>>2]|0;
       $132 = ($130>>>0)>($131>>>0);
       if ($132) {
        continue L34;
       }
       $133 = HEAP32[$iDbpage>>2]|0;
       $134 = HEAP32[$mxPage>>2]|0;
       $135 = ($133>>>0)>($134>>>0);
       if ($135) {
        continue L34;
       }
       $136 = HEAP32[$iFrame>>2]|0;
       $137 = (($136) - 1)|0;
       $138 = HEAP32[$szPage>>2]|0;
       $139 = (($138) + 24)|0;
       $140 = ($139|0)<(0);
       $141 = $140 << 31 >> 31;
       $142 = (___muldi3(($137|0),0,($139|0),($141|0))|0);
       $143 = tempRet0;
       $144 = (_i64Add(32,0,($142|0),($143|0))|0);
       $145 = tempRet0;
       $146 = (_i64Add(($144|0),($145|0),24,0)|0);
       $147 = tempRet0;
       $148 = $iOffset;
       $149 = $148;
       HEAP32[$149>>2] = $146;
       $150 = (($148) + 4)|0;
       $151 = $150;
       HEAP32[$151>>2] = $147;
       $152 = HEAP32[$1>>2]|0;
       $153 = (($152) + 8|0);
       $154 = HEAP32[$153>>2]|0;
       $155 = HEAP32[$6>>2]|0;
       $156 = HEAP32[$szPage>>2]|0;
       $157 = $iOffset;
       $158 = $157;
       $159 = HEAP32[$158>>2]|0;
       $160 = (($157) + 4)|0;
       $161 = $160;
       $162 = HEAP32[$161>>2]|0;
       $163 = (_sqlite3OsRead($154,$155,$156,$159,$162)|0);
       HEAP32[$rc>>2] = $163;
       $164 = HEAP32[$rc>>2]|0;
       $165 = ($164|0)!=(0);
       if ($165) {
        break L34;
       }
       $166 = HEAP32[$iDbpage>>2]|0;
       $167 = (($166) - 1)|0;
       $168 = HEAP32[$szPage>>2]|0;
       $169 = ($168|0)<(0);
       $170 = $169 << 31 >> 31;
       $171 = (___muldi3(($167|0),0,($168|0),($170|0))|0);
       $172 = tempRet0;
       $173 = $iOffset;
       $174 = $173;
       HEAP32[$174>>2] = $171;
       $175 = (($173) + 4)|0;
       $176 = $175;
       HEAP32[$176>>2] = $172;
       $177 = HEAP32[$1>>2]|0;
       $178 = (($177) + 4|0);
       $179 = HEAP32[$178>>2]|0;
       $180 = HEAP32[$6>>2]|0;
       $181 = HEAP32[$szPage>>2]|0;
       $182 = $iOffset;
       $183 = $182;
       $184 = HEAP32[$183>>2]|0;
       $185 = (($182) + 4)|0;
       $186 = $185;
       $187 = HEAP32[$186>>2]|0;
       $188 = (_sqlite3OsWrite($179,$180,$181,$184,$187)|0);
       HEAP32[$rc>>2] = $188;
       $189 = HEAP32[$rc>>2]|0;
       $190 = ($189|0)==(0);
       $191 = HEAP32[$rc>>2]|0;
       $192 = ($191|0)==(0);
       $or$cond5 = $190 & $192;
       if (!($or$cond5)) {
        break L34;
       }
      }
     }
     $193 = HEAP32[$rc>>2]|0;
     $194 = ($193|0)==(0);
     if ($194) {
      $195 = HEAP32[$mxSafeFrame>>2]|0;
      $196 = HEAP32[$1>>2]|0;
      $197 = (_walIndexHdr($196)|0);
      $198 = (($197) + 16|0);
      $199 = HEAP32[$198>>2]|0;
      $200 = ($195|0)==($199|0);
      if ($200) {
       $201 = HEAP32[$1>>2]|0;
       $202 = (($201) + 52|0);
       $203 = (($202) + 20|0);
       $204 = HEAP32[$203>>2]|0;
       $205 = HEAP32[$szPage>>2]|0;
       $206 = ($205|0)<(0);
       $207 = $206 << 31 >> 31;
       $208 = (___muldi3(($204|0),0,($205|0),($207|0))|0);
       $209 = tempRet0;
       $210 = $szDb;
       $211 = $210;
       HEAP32[$211>>2] = $208;
       $212 = (($210) + 4)|0;
       $213 = $212;
       HEAP32[$213>>2] = $209;
       $214 = HEAP32[$1>>2]|0;
       $215 = (($214) + 4|0);
       $216 = HEAP32[$215>>2]|0;
       $217 = $szDb;
       $218 = $217;
       $219 = HEAP32[$218>>2]|0;
       $220 = (($217) + 4)|0;
       $221 = $220;
       $222 = HEAP32[$221>>2]|0;
       $223 = (_sqlite3OsTruncate($216,$219,$222)|0);
       HEAP32[$rc>>2] = $223;
       $224 = HEAP32[$rc>>2]|0;
       $225 = ($224|0)==(0);
       $226 = HEAP32[$5>>2]|0;
       $227 = ($226|0)!=(0);
       $or$cond = $225 & $227;
       if ($or$cond) {
        $228 = HEAP32[$1>>2]|0;
        $229 = (($228) + 4|0);
        $230 = HEAP32[$229>>2]|0;
        $231 = HEAP32[$5>>2]|0;
        $232 = (_sqlite3OsSync($230,$231)|0);
        HEAP32[$rc>>2] = $232;
       }
      }
      $233 = HEAP32[$rc>>2]|0;
      $234 = ($233|0)==(0);
      if ($234) {
       $235 = HEAP32[$mxSafeFrame>>2]|0;
       $236 = HEAP32[$pInfo>>2]|0;
       HEAP32[$236>>2] = $235;
      }
     }
     $237 = HEAP32[$1>>2]|0;
     _walUnlockExclusive($237,3,1);
    }
   }
   $238 = HEAP32[$rc>>2]|0;
   $239 = ($238|0)==(5);
   if ($239) {
    HEAP32[$rc>>2] = 0;
   }
   $240 = HEAP32[$rc>>2]|0;
   $241 = ($240|0)==(0);
   $242 = HEAP32[$2>>2]|0;
   $243 = ($242|0)!=(0);
   $or$cond3 = $241 & $243;
   if ($or$cond3) {
    $244 = HEAP32[$pInfo>>2]|0;
    $245 = HEAP32[$244>>2]|0;
    $246 = HEAP32[$1>>2]|0;
    $247 = (($246) + 52|0);
    $248 = (($247) + 16|0);
    $249 = HEAP32[$248>>2]|0;
    $250 = ($245>>>0)<($249>>>0);
    if ($250) {
     HEAP32[$rc>>2] = 5;
     break;
    }
    $251 = HEAP32[$2>>2]|0;
    $252 = ($251|0)==(2);
    if ($252) {
     $253 = HEAP32[$1>>2]|0;
     $254 = HEAP32[$xBusy>>2]|0;
     $255 = HEAP32[$4>>2]|0;
     $256 = (_walBusyLock($253,$254,$255,4,4)|0);
     HEAP32[$rc>>2] = $256;
     $257 = HEAP32[$rc>>2]|0;
     $258 = ($257|0)==(0);
     if ($258) {
      $259 = HEAP32[$1>>2]|0;
      _walUnlockExclusive($259,4,4);
     }
    }
   }
  }
 } while(0);
 $260 = HEAP32[$pIter>>2]|0;
 _walIteratorFree($260);
 $261 = HEAP32[$rc>>2]|0;
 HEAP32[$0>>2] = $261;
 $262 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($262|0);
}
function _walCkptInfo($pWal) {
 $pWal = $pWal|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 HEAP32[$0>>2] = $pWal;
 $1 = HEAP32[$0>>2]|0;
 $2 = (($1) + 32|0);
 $3 = HEAP32[$2>>2]|0;
 $4 = HEAP32[$3>>2]|0;
 $5 = (($4) + 96|0);
 STACKTOP = sp;return ($5|0);
}
function _walIteratorInit($pWal,$pp) {
 $pWal = $pWal|0;
 $pp = $pp|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0;
 var $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0;
 var $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0;
 var $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0;
 var $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $aHash = 0, $aIndex = 0, $aPgno = 0, $aTmp = 0;
 var $i = 0, $iLast = 0, $iZero = 0, $j = 0, $nByte = 0, $nEntry = 0, $nSegment = 0, $p = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 64|0;
 $0 = sp + 60|0;
 $1 = sp + 56|0;
 $2 = sp + 52|0;
 $p = sp + 48|0;
 $nSegment = sp + 44|0;
 $iLast = sp + 40|0;
 $nByte = sp + 36|0;
 $i = sp + 32|0;
 $aTmp = sp + 28|0;
 $rc = sp + 24|0;
 $aHash = sp + 20|0;
 $iZero = sp + 16|0;
 $aPgno = sp + 12|0;
 $j = sp + 8|0;
 $nEntry = sp + 4|0;
 $aIndex = sp;
 HEAP32[$1>>2] = $pWal;
 HEAP32[$2>>2] = $pp;
 HEAP32[$rc>>2] = 0;
 $3 = HEAP32[$1>>2]|0;
 $4 = (($3) + 52|0);
 $5 = (($4) + 16|0);
 $6 = HEAP32[$5>>2]|0;
 HEAP32[$iLast>>2] = $6;
 $7 = HEAP32[$iLast>>2]|0;
 $8 = (_walFramePage($7)|0);
 $9 = (($8) + 1)|0;
 HEAP32[$nSegment>>2] = $9;
 $10 = HEAP32[$nSegment>>2]|0;
 $11 = (($10) - 1)|0;
 $12 = ($11*20)|0;
 $13 = (28 + ($12))|0;
 $14 = HEAP32[$iLast>>2]|0;
 $15 = $14<<1;
 $16 = (($13) + ($15))|0;
 HEAP32[$nByte>>2] = $16;
 $17 = HEAP32[$nByte>>2]|0;
 $18 = (_sqlite3ScratchMalloc($17)|0);
 HEAP32[$p>>2] = $18;
 $19 = HEAP32[$p>>2]|0;
 $20 = ($19|0)!=(0|0);
 if (!($20)) {
  HEAP32[$0>>2] = 7;
  $115 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($115|0);
 }
 $21 = HEAP32[$p>>2]|0;
 $22 = HEAP32[$nByte>>2]|0;
 _memset(($21|0),0,($22|0))|0;
 $23 = HEAP32[$nSegment>>2]|0;
 $24 = HEAP32[$p>>2]|0;
 $25 = (($24) + 4|0);
 HEAP32[$25>>2] = $23;
 $26 = HEAP32[$iLast>>2]|0;
 $27 = ($26>>>0)>(4096);
 $28 = HEAP32[$iLast>>2]|0;
 $29 = $27 ? 4096 : $28;
 $30 = $29<<1;
 $31 = (_sqlite3ScratchMalloc($30)|0);
 HEAP32[$aTmp>>2] = $31;
 $32 = HEAP32[$aTmp>>2]|0;
 $33 = ($32|0)!=(0|0);
 if (!($33)) {
  HEAP32[$rc>>2] = 7;
 }
 HEAP32[$i>>2] = 0;
 while(1) {
  $34 = HEAP32[$rc>>2]|0;
  $35 = ($34|0)==(0);
  if (!($35)) {
   break;
  }
  $36 = HEAP32[$i>>2]|0;
  $37 = HEAP32[$nSegment>>2]|0;
  $38 = ($36|0)<($37|0);
  if (!($38)) {
   break;
  }
  $39 = HEAP32[$1>>2]|0;
  $40 = HEAP32[$i>>2]|0;
  $41 = (_walHashGet($39,$40,$aHash,$aPgno,$iZero)|0);
  HEAP32[$rc>>2] = $41;
  $42 = HEAP32[$rc>>2]|0;
  $43 = ($42|0)==(0);
  if ($43) {
   $44 = HEAP32[$aPgno>>2]|0;
   $45 = (($44) + 4|0);
   HEAP32[$aPgno>>2] = $45;
   $46 = HEAP32[$i>>2]|0;
   $47 = (($46) + 1)|0;
   $48 = HEAP32[$nSegment>>2]|0;
   $49 = ($47|0)==($48|0);
   if ($49) {
    $50 = HEAP32[$iLast>>2]|0;
    $51 = HEAP32[$iZero>>2]|0;
    $52 = (($50) - ($51))|0;
    HEAP32[$nEntry>>2] = $52;
   } else {
    $53 = HEAP32[$aHash>>2]|0;
    $54 = HEAP32[$aPgno>>2]|0;
    $55 = $53;
    $56 = $54;
    $57 = (($55) - ($56))|0;
    $58 = (($57|0) / 4)&-1;
    HEAP32[$nEntry>>2] = $58;
   }
   $59 = HEAP32[$iZero>>2]|0;
   $60 = HEAP32[$p>>2]|0;
   $61 = (($60) + 4|0);
   $62 = HEAP32[$61>>2]|0;
   $63 = HEAP32[$p>>2]|0;
   $64 = (($63) + 8|0);
   $65 = (($64) + (($62*20)|0)|0);
   $66 = (($65) + ($59<<1)|0);
   HEAP32[$aIndex>>2] = $66;
   $67 = HEAP32[$iZero>>2]|0;
   $68 = (($67) + 1)|0;
   HEAP32[$iZero>>2] = $68;
   HEAP32[$j>>2] = 0;
   while(1) {
    $69 = HEAP32[$j>>2]|0;
    $70 = HEAP32[$nEntry>>2]|0;
    $71 = ($69|0)<($70|0);
    if (!($71)) {
     break;
    }
    $72 = HEAP32[$j>>2]|0;
    $73 = $72&65535;
    $74 = HEAP32[$j>>2]|0;
    $75 = HEAP32[$aIndex>>2]|0;
    $76 = (($75) + ($74<<1)|0);
    HEAP16[$76>>1] = $73;
    $77 = HEAP32[$j>>2]|0;
    $78 = (($77) + 1)|0;
    HEAP32[$j>>2] = $78;
   }
   $79 = HEAP32[$aPgno>>2]|0;
   $80 = HEAP32[$aTmp>>2]|0;
   $81 = HEAP32[$aIndex>>2]|0;
   _walMergesort($79,$80,$81,$nEntry);
   $82 = HEAP32[$iZero>>2]|0;
   $83 = HEAP32[$i>>2]|0;
   $84 = HEAP32[$p>>2]|0;
   $85 = (($84) + 8|0);
   $86 = (($85) + (($83*20)|0)|0);
   $87 = (($86) + 16|0);
   HEAP32[$87>>2] = $82;
   $88 = HEAP32[$nEntry>>2]|0;
   $89 = HEAP32[$i>>2]|0;
   $90 = HEAP32[$p>>2]|0;
   $91 = (($90) + 8|0);
   $92 = (($91) + (($89*20)|0)|0);
   $93 = (($92) + 12|0);
   HEAP32[$93>>2] = $88;
   $94 = HEAP32[$aIndex>>2]|0;
   $95 = HEAP32[$i>>2]|0;
   $96 = HEAP32[$p>>2]|0;
   $97 = (($96) + 8|0);
   $98 = (($97) + (($95*20)|0)|0);
   $99 = (($98) + 4|0);
   HEAP32[$99>>2] = $94;
   $100 = HEAP32[$aPgno>>2]|0;
   $101 = HEAP32[$i>>2]|0;
   $102 = HEAP32[$p>>2]|0;
   $103 = (($102) + 8|0);
   $104 = (($103) + (($101*20)|0)|0);
   $105 = (($104) + 8|0);
   HEAP32[$105>>2] = $100;
  }
  $106 = HEAP32[$i>>2]|0;
  $107 = (($106) + 1)|0;
  HEAP32[$i>>2] = $107;
 }
 $108 = HEAP32[$aTmp>>2]|0;
 _sqlite3ScratchFree($108);
 $109 = HEAP32[$rc>>2]|0;
 $110 = ($109|0)!=(0);
 if ($110) {
  $111 = HEAP32[$p>>2]|0;
  _walIteratorFree($111);
 }
 $112 = HEAP32[$p>>2]|0;
 $113 = HEAP32[$2>>2]|0;
 HEAP32[$113>>2] = $112;
 $114 = HEAP32[$rc>>2]|0;
 HEAP32[$0>>2] = $114;
 $115 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($115|0);
}
function _walIteratorNext($p,$piPage,$piFrame) {
 $p = $p|0;
 $piPage = $piPage|0;
 $piFrame = $piFrame|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $7 = 0, $8 = 0, $9 = 0, $i = 0, $iMin = 0, $iPg = 0, $iRet = 0, $pSegment = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 28|0;
 $1 = sp + 24|0;
 $2 = sp + 20|0;
 $iMin = sp + 16|0;
 $iRet = sp + 12|0;
 $i = sp + 8|0;
 $pSegment = sp + 4|0;
 $iPg = sp;
 HEAP32[$0>>2] = $p;
 HEAP32[$1>>2] = $piPage;
 HEAP32[$2>>2] = $piFrame;
 HEAP32[$iRet>>2] = -1;
 $3 = HEAP32[$0>>2]|0;
 $4 = HEAP32[$3>>2]|0;
 HEAP32[$iMin>>2] = $4;
 $5 = HEAP32[$0>>2]|0;
 $6 = (($5) + 4|0);
 $7 = HEAP32[$6>>2]|0;
 $8 = (($7) - 1)|0;
 HEAP32[$i>>2] = $8;
 while(1) {
  $9 = HEAP32[$i>>2]|0;
  $10 = ($9|0)>=(0);
  if (!($10)) {
   break;
  }
  $11 = HEAP32[$i>>2]|0;
  $12 = HEAP32[$0>>2]|0;
  $13 = (($12) + 8|0);
  $14 = (($13) + (($11*20)|0)|0);
  HEAP32[$pSegment>>2] = $14;
  while(1) {
   $15 = HEAP32[$pSegment>>2]|0;
   $16 = HEAP32[$15>>2]|0;
   $17 = HEAP32[$pSegment>>2]|0;
   $18 = (($17) + 12|0);
   $19 = HEAP32[$18>>2]|0;
   $20 = ($16|0)<($19|0);
   if (!($20)) {
    break;
   }
   $21 = HEAP32[$pSegment>>2]|0;
   $22 = HEAP32[$21>>2]|0;
   $23 = HEAP32[$pSegment>>2]|0;
   $24 = (($23) + 4|0);
   $25 = HEAP32[$24>>2]|0;
   $26 = (($25) + ($22<<1)|0);
   $27 = HEAP16[$26>>1]|0;
   $28 = $27&65535;
   $29 = HEAP32[$pSegment>>2]|0;
   $30 = (($29) + 8|0);
   $31 = HEAP32[$30>>2]|0;
   $32 = (($31) + ($28<<2)|0);
   $33 = HEAP32[$32>>2]|0;
   HEAP32[$iPg>>2] = $33;
   $34 = HEAP32[$iPg>>2]|0;
   $35 = HEAP32[$iMin>>2]|0;
   $36 = ($34>>>0)>($35>>>0);
   if ($36) {
    label = 6;
    break;
   }
   $54 = HEAP32[$pSegment>>2]|0;
   $55 = HEAP32[$54>>2]|0;
   $56 = (($55) + 1)|0;
   HEAP32[$54>>2] = $56;
  }
  if ((label|0) == 6) {
   label = 0;
   $37 = HEAP32[$iPg>>2]|0;
   $38 = HEAP32[$iRet>>2]|0;
   $39 = ($37>>>0)<($38>>>0);
   if ($39) {
    $40 = HEAP32[$iPg>>2]|0;
    HEAP32[$iRet>>2] = $40;
    $41 = HEAP32[$pSegment>>2]|0;
    $42 = (($41) + 16|0);
    $43 = HEAP32[$42>>2]|0;
    $44 = HEAP32[$pSegment>>2]|0;
    $45 = HEAP32[$44>>2]|0;
    $46 = HEAP32[$pSegment>>2]|0;
    $47 = (($46) + 4|0);
    $48 = HEAP32[$47>>2]|0;
    $49 = (($48) + ($45<<1)|0);
    $50 = HEAP16[$49>>1]|0;
    $51 = $50&65535;
    $52 = (($43) + ($51))|0;
    $53 = HEAP32[$2>>2]|0;
    HEAP32[$53>>2] = $52;
   }
  }
  $57 = HEAP32[$i>>2]|0;
  $58 = (($57) + -1)|0;
  HEAP32[$i>>2] = $58;
 }
 $59 = HEAP32[$iRet>>2]|0;
 $60 = HEAP32[$0>>2]|0;
 HEAP32[$60>>2] = $59;
 $61 = HEAP32[$1>>2]|0;
 HEAP32[$61>>2] = $59;
 $62 = HEAP32[$iRet>>2]|0;
 $63 = ($62|0)==(-1);
 $64 = $63&1;
 STACKTOP = sp;return ($64|0);
}
function _walIteratorFree($p) {
 $p = $p|0;
 var $0 = 0, $1 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 HEAP32[$0>>2] = $p;
 $1 = HEAP32[$0>>2]|0;
 _sqlite3ScratchFree($1);
 STACKTOP = sp;return;
}
function _walMergesort($aContent,$aBuffer,$aList,$pnList) {
 $aContent = $aContent|0;
 $aBuffer = $aBuffer|0;
 $aList = $aList|0;
 $pnList = $pnList|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $aMerge = 0;
 var $aSub = 0, $iList = 0, $iSub = 0, $nList = 0, $nMerge = 0, $p = 0, $p1 = 0, dest = 0, label = 0, sp = 0, stop = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 160|0;
 $0 = sp + 144|0;
 $1 = sp + 140|0;
 $2 = sp + 136|0;
 $3 = sp + 132|0;
 $nList = sp + 128|0;
 $nMerge = sp + 124|0;
 $aMerge = sp + 120|0;
 $iList = sp + 116|0;
 $iSub = sp + 112|0;
 $aSub = sp + 8|0;
 $p = sp + 4|0;
 $p1 = sp;
 HEAP32[$0>>2] = $aContent;
 HEAP32[$1>>2] = $aBuffer;
 HEAP32[$2>>2] = $aList;
 HEAP32[$3>>2] = $pnList;
 $4 = HEAP32[$3>>2]|0;
 $5 = HEAP32[$4>>2]|0;
 HEAP32[$nList>>2] = $5;
 HEAP32[$nMerge>>2] = 0;
 HEAP32[$aMerge>>2] = 0;
 HEAP32[$iSub>>2] = 0;
 dest=$aSub+0|0; stop=dest+104|0; do { HEAP32[dest>>2]=0|0; dest=dest+4|0; } while ((dest|0) < (stop|0));
 HEAP32[$iList>>2] = 0;
 while(1) {
  $6 = HEAP32[$iList>>2]|0;
  $7 = HEAP32[$nList>>2]|0;
  $8 = ($6|0)<($7|0);
  if (!($8)) {
   break;
  }
  HEAP32[$nMerge>>2] = 1;
  $9 = HEAP32[$iList>>2]|0;
  $10 = HEAP32[$2>>2]|0;
  $11 = (($10) + ($9<<1)|0);
  HEAP32[$aMerge>>2] = $11;
  HEAP32[$iSub>>2] = 0;
  while(1) {
   $12 = HEAP32[$iList>>2]|0;
   $13 = HEAP32[$iSub>>2]|0;
   $14 = 1 << $13;
   $15 = $12 & $14;
   $16 = ($15|0)!=(0);
   if (!($16)) {
    break;
   }
   $17 = HEAP32[$iSub>>2]|0;
   $18 = (($aSub) + ($17<<3)|0);
   HEAP32[$p>>2] = $18;
   $19 = HEAP32[$0>>2]|0;
   $20 = HEAP32[$p>>2]|0;
   $21 = (($20) + 4|0);
   $22 = HEAP32[$21>>2]|0;
   $23 = HEAP32[$p>>2]|0;
   $24 = HEAP32[$23>>2]|0;
   $25 = HEAP32[$1>>2]|0;
   _walMerge($19,$22,$24,$aMerge,$nMerge,$25);
   $26 = HEAP32[$iSub>>2]|0;
   $27 = (($26) + 1)|0;
   HEAP32[$iSub>>2] = $27;
  }
  $28 = HEAP32[$aMerge>>2]|0;
  $29 = HEAP32[$iSub>>2]|0;
  $30 = (($aSub) + ($29<<3)|0);
  $31 = (($30) + 4|0);
  HEAP32[$31>>2] = $28;
  $32 = HEAP32[$nMerge>>2]|0;
  $33 = HEAP32[$iSub>>2]|0;
  $34 = (($aSub) + ($33<<3)|0);
  HEAP32[$34>>2] = $32;
  $35 = HEAP32[$iList>>2]|0;
  $36 = (($35) + 1)|0;
  HEAP32[$iList>>2] = $36;
 }
 $37 = HEAP32[$iSub>>2]|0;
 $38 = (($37) + 1)|0;
 HEAP32[$iSub>>2] = $38;
 while(1) {
  $39 = HEAP32[$iSub>>2]|0;
  $40 = ($39|0)<(13);
  if (!($40)) {
   break;
  }
  $41 = HEAP32[$nList>>2]|0;
  $42 = HEAP32[$iSub>>2]|0;
  $43 = 1 << $42;
  $44 = $41 & $43;
  $45 = ($44|0)!=(0);
  if ($45) {
   $46 = HEAP32[$iSub>>2]|0;
   $47 = (($aSub) + ($46<<3)|0);
   HEAP32[$p1>>2] = $47;
   $48 = HEAP32[$0>>2]|0;
   $49 = HEAP32[$p1>>2]|0;
   $50 = (($49) + 4|0);
   $51 = HEAP32[$50>>2]|0;
   $52 = HEAP32[$p1>>2]|0;
   $53 = HEAP32[$52>>2]|0;
   $54 = HEAP32[$1>>2]|0;
   _walMerge($48,$51,$53,$aMerge,$nMerge,$54);
  }
  $55 = HEAP32[$iSub>>2]|0;
  $56 = (($55) + 1)|0;
  HEAP32[$iSub>>2] = $56;
 }
 $57 = HEAP32[$nMerge>>2]|0;
 $58 = HEAP32[$3>>2]|0;
 HEAP32[$58>>2] = $57;
 STACKTOP = sp;return;
}
function _walMerge($aContent,$aLeft,$nLeft,$paRight,$pnRight,$aTmp) {
 $aContent = $aContent|0;
 $aLeft = $aLeft|0;
 $nLeft = $nLeft|0;
 $paRight = $paRight|0;
 $pnRight = $pnRight|0;
 $aTmp = $aTmp|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0;
 var $81 = 0, $9 = 0, $aRight = 0, $dbpage = 0, $iLeft = 0, $iOut = 0, $iRight = 0, $logpage = 0, $nRight = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 64|0;
 $0 = sp + 44|0;
 $1 = sp + 40|0;
 $2 = sp + 36|0;
 $3 = sp + 32|0;
 $4 = sp + 28|0;
 $5 = sp + 24|0;
 $iLeft = sp + 20|0;
 $iRight = sp + 16|0;
 $iOut = sp + 12|0;
 $nRight = sp + 8|0;
 $aRight = sp + 4|0;
 $logpage = sp + 48|0;
 $dbpage = sp;
 HEAP32[$0>>2] = $aContent;
 HEAP32[$1>>2] = $aLeft;
 HEAP32[$2>>2] = $nLeft;
 HEAP32[$3>>2] = $paRight;
 HEAP32[$4>>2] = $pnRight;
 HEAP32[$5>>2] = $aTmp;
 HEAP32[$iLeft>>2] = 0;
 HEAP32[$iRight>>2] = 0;
 HEAP32[$iOut>>2] = 0;
 $6 = HEAP32[$4>>2]|0;
 $7 = HEAP32[$6>>2]|0;
 HEAP32[$nRight>>2] = $7;
 $8 = HEAP32[$3>>2]|0;
 $9 = HEAP32[$8>>2]|0;
 HEAP32[$aRight>>2] = $9;
 while(1) {
  $10 = HEAP32[$iRight>>2]|0;
  $11 = HEAP32[$nRight>>2]|0;
  $12 = ($10|0)<($11|0);
  if (!($12)) {
   $13 = HEAP32[$iLeft>>2]|0;
   $14 = HEAP32[$2>>2]|0;
   $15 = ($13|0)<($14|0);
   if (!($15)) {
    break;
   }
  }
  $16 = HEAP32[$iLeft>>2]|0;
  $17 = HEAP32[$2>>2]|0;
  $18 = ($16|0)<($17|0);
  do {
   if ($18) {
    $19 = HEAP32[$iRight>>2]|0;
    $20 = HEAP32[$nRight>>2]|0;
    $21 = ($19|0)>=($20|0);
    if (!($21)) {
     $22 = HEAP32[$iLeft>>2]|0;
     $23 = HEAP32[$1>>2]|0;
     $24 = (($23) + ($22<<1)|0);
     $25 = HEAP16[$24>>1]|0;
     $26 = $25&65535;
     $27 = HEAP32[$0>>2]|0;
     $28 = (($27) + ($26<<2)|0);
     $29 = HEAP32[$28>>2]|0;
     $30 = HEAP32[$iRight>>2]|0;
     $31 = HEAP32[$aRight>>2]|0;
     $32 = (($31) + ($30<<1)|0);
     $33 = HEAP16[$32>>1]|0;
     $34 = $33&65535;
     $35 = HEAP32[$0>>2]|0;
     $36 = (($35) + ($34<<2)|0);
     $37 = HEAP32[$36>>2]|0;
     $38 = ($29>>>0)<($37>>>0);
     if (!($38)) {
      label = 8;
      break;
     }
    }
    $39 = HEAP32[$iLeft>>2]|0;
    $40 = (($39) + 1)|0;
    HEAP32[$iLeft>>2] = $40;
    $41 = HEAP32[$1>>2]|0;
    $42 = (($41) + ($39<<1)|0);
    $43 = HEAP16[$42>>1]|0;
    HEAP16[$logpage>>1] = $43;
   } else {
    label = 8;
   }
  } while(0);
  if ((label|0) == 8) {
   label = 0;
   $44 = HEAP32[$iRight>>2]|0;
   $45 = (($44) + 1)|0;
   HEAP32[$iRight>>2] = $45;
   $46 = HEAP32[$aRight>>2]|0;
   $47 = (($46) + ($44<<1)|0);
   $48 = HEAP16[$47>>1]|0;
   HEAP16[$logpage>>1] = $48;
  }
  $49 = HEAP16[$logpage>>1]|0;
  $50 = $49&65535;
  $51 = HEAP32[$0>>2]|0;
  $52 = (($51) + ($50<<2)|0);
  $53 = HEAP32[$52>>2]|0;
  HEAP32[$dbpage>>2] = $53;
  $54 = HEAP16[$logpage>>1]|0;
  $55 = HEAP32[$iOut>>2]|0;
  $56 = (($55) + 1)|0;
  HEAP32[$iOut>>2] = $56;
  $57 = HEAP32[$5>>2]|0;
  $58 = (($57) + ($55<<1)|0);
  HEAP16[$58>>1] = $54;
  $59 = HEAP32[$iLeft>>2]|0;
  $60 = HEAP32[$2>>2]|0;
  $61 = ($59|0)<($60|0);
  if (!($61)) {
   continue;
  }
  $62 = HEAP32[$iLeft>>2]|0;
  $63 = HEAP32[$1>>2]|0;
  $64 = (($63) + ($62<<1)|0);
  $65 = HEAP16[$64>>1]|0;
  $66 = $65&65535;
  $67 = HEAP32[$0>>2]|0;
  $68 = (($67) + ($66<<2)|0);
  $69 = HEAP32[$68>>2]|0;
  $70 = HEAP32[$dbpage>>2]|0;
  $71 = ($69|0)==($70|0);
  if (!($71)) {
   continue;
  }
  $72 = HEAP32[$iLeft>>2]|0;
  $73 = (($72) + 1)|0;
  HEAP32[$iLeft>>2] = $73;
 }
 $74 = HEAP32[$1>>2]|0;
 $75 = HEAP32[$3>>2]|0;
 HEAP32[$75>>2] = $74;
 $76 = HEAP32[$iOut>>2]|0;
 $77 = HEAP32[$4>>2]|0;
 HEAP32[$77>>2] = $76;
 $78 = HEAP32[$1>>2]|0;
 $79 = HEAP32[$5>>2]|0;
 $80 = HEAP32[$iOut>>2]|0;
 $81 = $80<<1;
 _memcpy(($78|0),($79|0),($81|0))|0;
 STACKTOP = sp;return;
}
function _walIndexTryHdr($pWal,$pChanged) {
 $pWal = $pWal|0;
 $pChanged = $pChanged|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $aCksum = 0, $aHdr = 0, $h1 = 0, $h2 = 0, dest = 0, label = 0, sp = 0, src = 0, stop = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 128|0;
 $0 = sp + 120|0;
 $1 = sp + 116|0;
 $2 = sp + 112|0;
 $aCksum = sp + 104|0;
 $h1 = sp + 56|0;
 $h2 = sp + 8|0;
 $aHdr = sp;
 HEAP32[$1>>2] = $pWal;
 HEAP32[$2>>2] = $pChanged;
 $3 = HEAP32[$1>>2]|0;
 $4 = (_walIndexHdr($3)|0);
 HEAP32[$aHdr>>2] = $4;
 $5 = HEAP32[$aHdr>>2]|0;
 dest=$h1+0|0; src=$5+0|0; stop=dest+48|0; do { HEAP8[dest>>0]=HEAP8[src>>0]|0; dest=dest+1|0; src=src+1|0; } while ((dest|0) < (stop|0));
 $6 = HEAP32[$1>>2]|0;
 _walShmBarrier($6);
 $7 = HEAP32[$aHdr>>2]|0;
 $8 = (($7) + 48|0);
 dest=$h2+0|0; src=$8+0|0; stop=dest+48|0; do { HEAP8[dest>>0]=HEAP8[src>>0]|0; dest=dest+1|0; src=src+1|0; } while ((dest|0) < (stop|0));
 $9 = (_memcmp($h1,$h2,48)|0);
 $10 = ($9|0)!=(0);
 if ($10) {
  HEAP32[$0>>2] = 1;
  $48 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($48|0);
 }
 $11 = (($h1) + 12|0);
 $12 = HEAP8[$11>>0]|0;
 $13 = $12&255;
 $14 = ($13|0)==(0);
 if ($14) {
  HEAP32[$0>>2] = 1;
  $48 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($48|0);
 }
 _walChecksumBytes(1,$h1,40,0,$aCksum);
 $15 = HEAP32[$aCksum>>2]|0;
 $16 = (($h1) + 40|0);
 $17 = HEAP32[$16>>2]|0;
 $18 = ($15|0)!=($17|0);
 if (!($18)) {
  $19 = (($aCksum) + 4|0);
  $20 = HEAP32[$19>>2]|0;
  $21 = (($h1) + 40|0);
  $22 = (($21) + 4|0);
  $23 = HEAP32[$22>>2]|0;
  $24 = ($20|0)!=($23|0);
  if (!($24)) {
   $25 = HEAP32[$1>>2]|0;
   $26 = (($25) + 52|0);
   $27 = (_memcmp($26,$h1,48)|0);
   $28 = ($27|0)!=(0);
   if ($28) {
    $29 = HEAP32[$2>>2]|0;
    HEAP32[$29>>2] = 1;
    $30 = HEAP32[$1>>2]|0;
    $31 = (($30) + 52|0);
    dest=$31+0|0; src=$h1+0|0; stop=dest+48|0; do { HEAP32[dest>>2]=HEAP32[src>>2]|0; dest=dest+4|0; src=src+4|0; } while ((dest|0) < (stop|0));
    $32 = HEAP32[$1>>2]|0;
    $33 = (($32) + 52|0);
    $34 = (($33) + 14|0);
    $35 = HEAP16[$34>>1]|0;
    $36 = $35&65535;
    $37 = $36 & 65024;
    $38 = HEAP32[$1>>2]|0;
    $39 = (($38) + 52|0);
    $40 = (($39) + 14|0);
    $41 = HEAP16[$40>>1]|0;
    $42 = $41&65535;
    $43 = $42 & 1;
    $44 = $43 << 16;
    $45 = (($37) + ($44))|0;
    $46 = HEAP32[$1>>2]|0;
    $47 = (($46) + 36|0);
    HEAP32[$47>>2] = $45;
   }
   HEAP32[$0>>2] = 0;
   $48 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($48|0);
  }
 }
 HEAP32[$0>>2] = 1;
 $48 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($48|0);
}
function _walIndexRecover($pWal) {
 $pWal = $pWal|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0;
 var $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0, $188 = 0;
 var $189 = 0, $19 = 0, $190 = 0, $191 = 0, $192 = 0, $193 = 0, $194 = 0, $195 = 0, $196 = 0, $197 = 0, $198 = 0, $199 = 0, $2 = 0, $20 = 0, $200 = 0, $201 = 0, $202 = 0, $203 = 0, $204 = 0, $205 = 0;
 var $206 = 0, $207 = 0, $208 = 0, $209 = 0, $21 = 0, $210 = 0, $211 = 0, $212 = 0, $213 = 0, $214 = 0, $215 = 0, $216 = 0, $217 = 0, $218 = 0, $219 = 0, $22 = 0, $220 = 0, $221 = 0, $222 = 0, $223 = 0;
 var $224 = 0, $225 = 0, $226 = 0, $227 = 0, $228 = 0, $229 = 0, $23 = 0, $230 = 0, $231 = 0, $232 = 0, $233 = 0, $234 = 0, $235 = 0, $236 = 0, $237 = 0, $238 = 0, $239 = 0, $24 = 0, $240 = 0, $241 = 0;
 var $242 = 0, $243 = 0, $244 = 0, $245 = 0, $246 = 0, $247 = 0, $248 = 0, $249 = 0, $25 = 0, $250 = 0, $251 = 0, $252 = 0, $253 = 0, $254 = 0, $255 = 0, $256 = 0, $257 = 0, $258 = 0, $259 = 0, $26 = 0;
 var $260 = 0, $261 = 0, $262 = 0, $263 = 0, $264 = 0, $265 = 0, $266 = 0, $267 = 0, $268 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0;
 var $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0;
 var $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0;
 var $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0;
 var $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $aBuf = 0, $aData = 0, $aFrame = 0, $aFrameCksum = 0, $i = 0, $iFrame = 0, $iLock = 0, $iOffset = 0, $isValid = 0, $magic = 0, $nLock = 0;
 var $nSize = 0, $nTruncate = 0, $or$cond = 0, $or$cond3 = 0, $pInfo = 0, $pgno = 0, $rc = 0, $szFrame = 0, $szPage = 0, $vararg_buffer = 0, $vararg_ptr4 = 0, $version = 0, dest = 0, label = 0, sp = 0, stop = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 144|0;
 $vararg_buffer = sp + 16|0;
 $0 = sp + 96|0;
 $1 = sp + 92|0;
 $rc = sp + 88|0;
 $nSize = sp + 8|0;
 $aFrameCksum = sp + 80|0;
 $iLock = sp + 76|0;
 $nLock = sp + 72|0;
 $aBuf = sp + 104|0;
 $aFrame = sp + 68|0;
 $szFrame = sp + 64|0;
 $aData = sp + 60|0;
 $iFrame = sp + 56|0;
 $iOffset = sp;
 $szPage = sp + 52|0;
 $magic = sp + 48|0;
 $version = sp + 44|0;
 $isValid = sp + 40|0;
 $pgno = sp + 36|0;
 $nTruncate = sp + 32|0;
 $pInfo = sp + 28|0;
 $i = sp + 24|0;
 HEAP32[$1>>2] = $pWal;
 ;HEAP32[$aFrameCksum+0>>2]=0|0;HEAP32[$aFrameCksum+4>>2]=0|0;
 $2 = HEAP32[$1>>2]|0;
 $3 = (($2) + 45|0);
 $4 = HEAP8[$3>>0]|0;
 $5 = $4&255;
 $6 = (1 + ($5))|0;
 HEAP32[$iLock>>2] = $6;
 $7 = HEAP32[$iLock>>2]|0;
 $8 = (8 - ($7))|0;
 HEAP32[$nLock>>2] = $8;
 $9 = HEAP32[$1>>2]|0;
 $10 = HEAP32[$iLock>>2]|0;
 $11 = HEAP32[$nLock>>2]|0;
 $12 = (_walLockExclusive($9,$10,$11)|0);
 HEAP32[$rc>>2] = $12;
 $13 = HEAP32[$rc>>2]|0;
 $14 = ($13|0)!=(0);
 if ($14) {
  $15 = HEAP32[$rc>>2]|0;
  HEAP32[$0>>2] = $15;
  $268 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($268|0);
 }
 $16 = HEAP32[$1>>2]|0;
 $17 = (($16) + 52|0);
 dest=$17+0|0; stop=dest+48|0; do { HEAP32[dest>>2]=0|0; dest=dest+4|0; } while ((dest|0) < (stop|0));
 $18 = HEAP32[$1>>2]|0;
 $19 = (($18) + 8|0);
 $20 = HEAP32[$19>>2]|0;
 $21 = (_sqlite3OsFileSize($20,$nSize)|0);
 HEAP32[$rc>>2] = $21;
 $22 = HEAP32[$rc>>2]|0;
 $23 = ($22|0)!=(0);
 L5: do {
  if (!($23)) {
   $24 = $nSize;
   $25 = $24;
   $26 = HEAP32[$25>>2]|0;
   $27 = (($24) + 4)|0;
   $28 = $27;
   $29 = HEAP32[$28>>2]|0;
   $30 = ($29|0)>(0);
   $31 = ($26>>>0)>(32);
   $32 = ($29|0)==(0);
   $33 = $32 & $31;
   $34 = $30 | $33;
   do {
    if ($34) {
     HEAP32[$aFrame>>2] = 0;
     $35 = HEAP32[$1>>2]|0;
     $36 = (($35) + 8|0);
     $37 = HEAP32[$36>>2]|0;
     $38 = (_sqlite3OsRead($37,$aBuf,32,0,0)|0);
     HEAP32[$rc>>2] = $38;
     $39 = HEAP32[$rc>>2]|0;
     $40 = ($39|0)!=(0);
     if ($40) {
      break L5;
     }
     $41 = (_sqlite3Get4byte($aBuf)|0);
     HEAP32[$magic>>2] = $41;
     $42 = (($aBuf) + 8|0);
     $43 = (_sqlite3Get4byte($42)|0);
     HEAP32[$szPage>>2] = $43;
     $44 = HEAP32[$magic>>2]|0;
     $45 = $44 & -2;
     $46 = ($45|0)!=(931071618);
     if (!($46)) {
      $47 = HEAP32[$szPage>>2]|0;
      $48 = HEAP32[$szPage>>2]|0;
      $49 = (($48) - 1)|0;
      $50 = $47 & $49;
      $51 = ($50|0)!=(0);
      $52 = HEAP32[$szPage>>2]|0;
      $53 = ($52|0)>(65536);
      $or$cond = $51 | $53;
      $54 = HEAP32[$szPage>>2]|0;
      $55 = ($54|0)<(512);
      $or$cond3 = $or$cond | $55;
      if (!($or$cond3)) {
       $56 = HEAP32[$magic>>2]|0;
       $57 = $56 & 1;
       $58 = $57&255;
       $59 = HEAP32[$1>>2]|0;
       $60 = (($59) + 52|0);
       $61 = (($60) + 13|0);
       HEAP8[$61>>0] = $58;
       $62 = HEAP32[$szPage>>2]|0;
       $63 = HEAP32[$1>>2]|0;
       $64 = (($63) + 36|0);
       HEAP32[$64>>2] = $62;
       $65 = (($aBuf) + 12|0);
       $66 = (_sqlite3Get4byte($65)|0);
       $67 = HEAP32[$1>>2]|0;
       $68 = (($67) + 104|0);
       HEAP32[$68>>2] = $66;
       $69 = HEAP32[$1>>2]|0;
       $70 = (($69) + 52|0);
       $71 = (($70) + 32|0);
       $72 = (($aBuf) + 16|0);
       ;HEAP8[$71+0>>0]=HEAP8[$72+0>>0]|0;HEAP8[$71+1>>0]=HEAP8[$72+1>>0]|0;HEAP8[$71+2>>0]=HEAP8[$72+2>>0]|0;HEAP8[$71+3>>0]=HEAP8[$72+3>>0]|0;HEAP8[$71+4>>0]=HEAP8[$72+4>>0]|0;HEAP8[$71+5>>0]=HEAP8[$72+5>>0]|0;HEAP8[$71+6>>0]=HEAP8[$72+6>>0]|0;HEAP8[$71+7>>0]=HEAP8[$72+7>>0]|0;
       $73 = HEAP32[$1>>2]|0;
       $74 = (($73) + 52|0);
       $75 = (($74) + 13|0);
       $76 = HEAP8[$75>>0]|0;
       $77 = $76&255;
       $78 = HEAP8[1136>>0]|0;
       $79 = $78 << 24 >> 24;
       $80 = ($79|0)==(0);
       $81 = $80&1;
       $82 = ($77|0)==($81|0);
       $83 = $82&1;
       $84 = HEAP32[$1>>2]|0;
       $85 = (($84) + 52|0);
       $86 = (($85) + 24|0);
       _walChecksumBytes($83,$aBuf,24,0,$86);
       $87 = HEAP32[$1>>2]|0;
       $88 = (($87) + 52|0);
       $89 = (($88) + 24|0);
       $90 = HEAP32[$89>>2]|0;
       $91 = (($aBuf) + 24|0);
       $92 = (_sqlite3Get4byte($91)|0);
       $93 = ($90|0)!=($92|0);
       if (!($93)) {
        $94 = HEAP32[$1>>2]|0;
        $95 = (($94) + 52|0);
        $96 = (($95) + 24|0);
        $97 = (($96) + 4|0);
        $98 = HEAP32[$97>>2]|0;
        $99 = (($aBuf) + 28|0);
        $100 = (_sqlite3Get4byte($99)|0);
        $101 = ($98|0)!=($100|0);
        if (!($101)) {
         $102 = (($aBuf) + 4|0);
         $103 = (_sqlite3Get4byte($102)|0);
         HEAP32[$version>>2] = $103;
         $104 = HEAP32[$version>>2]|0;
         $105 = ($104|0)!=(3007000);
         if ($105) {
          $106 = (_sqlite3CantopenError(49260)|0);
          HEAP32[$rc>>2] = $106;
          break;
         }
         $107 = HEAP32[$szPage>>2]|0;
         $108 = (($107) + 24)|0;
         HEAP32[$szFrame>>2] = $108;
         $109 = HEAP32[$szFrame>>2]|0;
         $110 = (_sqlite3_malloc($109)|0);
         HEAP32[$aFrame>>2] = $110;
         $111 = HEAP32[$aFrame>>2]|0;
         $112 = ($111|0)!=(0|0);
         if (!($112)) {
          HEAP32[$rc>>2] = 7;
          break L5;
         }
         $113 = HEAP32[$aFrame>>2]|0;
         $114 = (($113) + 24|0);
         HEAP32[$aData>>2] = $114;
         HEAP32[$iFrame>>2] = 0;
         $115 = $iOffset;
         $116 = $115;
         HEAP32[$116>>2] = 32;
         $117 = (($115) + 4)|0;
         $118 = $117;
         HEAP32[$118>>2] = 0;
         while(1) {
          $119 = $iOffset;
          $120 = $119;
          $121 = HEAP32[$120>>2]|0;
          $122 = (($119) + 4)|0;
          $123 = $122;
          $124 = HEAP32[$123>>2]|0;
          $125 = HEAP32[$szFrame>>2]|0;
          $126 = ($125|0)<(0);
          $127 = $126 << 31 >> 31;
          $128 = (_i64Add(($121|0),($124|0),($125|0),($127|0))|0);
          $129 = tempRet0;
          $130 = $nSize;
          $131 = $130;
          $132 = HEAP32[$131>>2]|0;
          $133 = (($130) + 4)|0;
          $134 = $133;
          $135 = HEAP32[$134>>2]|0;
          $136 = ($129|0)<($135|0);
          $137 = ($128>>>0)<=($132>>>0);
          $138 = ($129|0)==($135|0);
          $139 = $138 & $137;
          $140 = $136 | $139;
          if (!($140)) {
           break;
          }
          $141 = HEAP32[$iFrame>>2]|0;
          $142 = (($141) + 1)|0;
          HEAP32[$iFrame>>2] = $142;
          $143 = HEAP32[$1>>2]|0;
          $144 = (($143) + 8|0);
          $145 = HEAP32[$144>>2]|0;
          $146 = HEAP32[$aFrame>>2]|0;
          $147 = HEAP32[$szFrame>>2]|0;
          $148 = $iOffset;
          $149 = $148;
          $150 = HEAP32[$149>>2]|0;
          $151 = (($148) + 4)|0;
          $152 = $151;
          $153 = HEAP32[$152>>2]|0;
          $154 = (_sqlite3OsRead($145,$146,$147,$150,$153)|0);
          HEAP32[$rc>>2] = $154;
          $155 = HEAP32[$rc>>2]|0;
          $156 = ($155|0)!=(0);
          if ($156) {
           break;
          }
          $157 = HEAP32[$1>>2]|0;
          $158 = HEAP32[$aData>>2]|0;
          $159 = HEAP32[$aFrame>>2]|0;
          $160 = (_walDecodeFrame($157,$pgno,$nTruncate,$158,$159)|0);
          HEAP32[$isValid>>2] = $160;
          $161 = HEAP32[$isValid>>2]|0;
          $162 = ($161|0)!=(0);
          if (!($162)) {
           break;
          }
          $163 = HEAP32[$1>>2]|0;
          $164 = HEAP32[$iFrame>>2]|0;
          $165 = HEAP32[$pgno>>2]|0;
          $166 = (_walIndexAppend($163,$164,$165)|0);
          HEAP32[$rc>>2] = $166;
          $167 = HEAP32[$rc>>2]|0;
          $168 = ($167|0)!=(0);
          if ($168) {
           break;
          }
          $169 = HEAP32[$nTruncate>>2]|0;
          $170 = ($169|0)!=(0);
          if ($170) {
           $171 = HEAP32[$iFrame>>2]|0;
           $172 = HEAP32[$1>>2]|0;
           $173 = (($172) + 52|0);
           $174 = (($173) + 16|0);
           HEAP32[$174>>2] = $171;
           $175 = HEAP32[$nTruncate>>2]|0;
           $176 = HEAP32[$1>>2]|0;
           $177 = (($176) + 52|0);
           $178 = (($177) + 20|0);
           HEAP32[$178>>2] = $175;
           $179 = HEAP32[$szPage>>2]|0;
           $180 = $179 & 65280;
           $181 = HEAP32[$szPage>>2]|0;
           $182 = $181 >> 16;
           $183 = $180 | $182;
           $184 = $183&65535;
           $185 = HEAP32[$1>>2]|0;
           $186 = (($185) + 52|0);
           $187 = (($186) + 14|0);
           HEAP16[$187>>1] = $184;
           $188 = HEAP32[$1>>2]|0;
           $189 = (($188) + 52|0);
           $190 = (($189) + 24|0);
           $191 = HEAP32[$190>>2]|0;
           HEAP32[$aFrameCksum>>2] = $191;
           $192 = HEAP32[$1>>2]|0;
           $193 = (($192) + 52|0);
           $194 = (($193) + 24|0);
           $195 = (($194) + 4|0);
           $196 = HEAP32[$195>>2]|0;
           $197 = (($aFrameCksum) + 4|0);
           HEAP32[$197>>2] = $196;
          }
          $198 = HEAP32[$szFrame>>2]|0;
          $199 = ($198|0)<(0);
          $200 = $199 << 31 >> 31;
          $201 = $iOffset;
          $202 = $201;
          $203 = HEAP32[$202>>2]|0;
          $204 = (($201) + 4)|0;
          $205 = $204;
          $206 = HEAP32[$205>>2]|0;
          $207 = (_i64Add(($203|0),($206|0),($198|0),($200|0))|0);
          $208 = tempRet0;
          $209 = $iOffset;
          $210 = $209;
          HEAP32[$210>>2] = $207;
          $211 = (($209) + 4)|0;
          $212 = $211;
          HEAP32[$212>>2] = $208;
         }
         $213 = HEAP32[$aFrame>>2]|0;
         _sqlite3_free($213);
        }
       }
      }
     }
    }
   } while(0);
   $214 = HEAP32[$rc>>2]|0;
   $215 = ($214|0)==(0);
   if ($215) {
    $216 = HEAP32[$aFrameCksum>>2]|0;
    $217 = HEAP32[$1>>2]|0;
    $218 = (($217) + 52|0);
    $219 = (($218) + 24|0);
    HEAP32[$219>>2] = $216;
    $220 = (($aFrameCksum) + 4|0);
    $221 = HEAP32[$220>>2]|0;
    $222 = HEAP32[$1>>2]|0;
    $223 = (($222) + 52|0);
    $224 = (($223) + 24|0);
    $225 = (($224) + 4|0);
    HEAP32[$225>>2] = $221;
    $226 = HEAP32[$1>>2]|0;
    _walIndexWriteHdr($226);
    $227 = HEAP32[$1>>2]|0;
    $228 = (_walCkptInfo($227)|0);
    HEAP32[$pInfo>>2] = $228;
    $229 = HEAP32[$pInfo>>2]|0;
    HEAP32[$229>>2] = 0;
    $230 = HEAP32[$pInfo>>2]|0;
    $231 = (($230) + 4|0);
    HEAP32[$231>>2] = 0;
    HEAP32[$i>>2] = 1;
    while(1) {
     $232 = HEAP32[$i>>2]|0;
     $233 = ($232|0)<(5);
     if (!($233)) {
      break;
     }
     $234 = HEAP32[$i>>2]|0;
     $235 = HEAP32[$pInfo>>2]|0;
     $236 = (($235) + 4|0);
     $237 = (($236) + ($234<<2)|0);
     HEAP32[$237>>2] = -1;
     $238 = HEAP32[$i>>2]|0;
     $239 = (($238) + 1)|0;
     HEAP32[$i>>2] = $239;
    }
    $240 = HEAP32[$1>>2]|0;
    $241 = (($240) + 52|0);
    $242 = (($241) + 16|0);
    $243 = HEAP32[$242>>2]|0;
    $244 = ($243|0)!=(0);
    if ($244) {
     $245 = HEAP32[$1>>2]|0;
     $246 = (($245) + 52|0);
     $247 = (($246) + 16|0);
     $248 = HEAP32[$247>>2]|0;
     $249 = HEAP32[$pInfo>>2]|0;
     $250 = (($249) + 4|0);
     $251 = (($250) + 4|0);
     HEAP32[$251>>2] = $248;
    }
    $252 = HEAP32[$1>>2]|0;
    $253 = (($252) + 52|0);
    $254 = (($253) + 20|0);
    $255 = HEAP32[$254>>2]|0;
    $256 = ($255|0)!=(0);
    if ($256) {
     $257 = HEAP32[$1>>2]|0;
     $258 = (($257) + 52|0);
     $259 = (($258) + 16|0);
     $260 = HEAP32[$259>>2]|0;
     $261 = HEAP32[$1>>2]|0;
     $262 = (($261) + 100|0);
     $263 = HEAP32[$262>>2]|0;
     HEAP32[$vararg_buffer>>2] = $260;
     $vararg_ptr4 = (($vararg_buffer) + 4|0);
     HEAP32[$vararg_ptr4>>2] = $263;
     _sqlite3_log(283,4512,$vararg_buffer);
    }
   }
  }
 } while(0);
 $264 = HEAP32[$1>>2]|0;
 $265 = HEAP32[$iLock>>2]|0;
 $266 = HEAP32[$nLock>>2]|0;
 _walUnlockExclusive($264,$265,$266);
 $267 = HEAP32[$rc>>2]|0;
 HEAP32[$0>>2] = $267;
 $268 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($268|0);
}
function _sqlite3CantopenError($lineno) {
 $lineno = $lineno|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $vararg_buffer = 0, $vararg_ptr1 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $vararg_buffer = sp;
 $0 = sp + 8|0;
 HEAP32[$0>>2] = $lineno;
 $1 = HEAP32[$0>>2]|0;
 $2 = (_sqlite3_sourceid()|0);
 $3 = (($2) + 20|0);
 HEAP32[$vararg_buffer>>2] = $1;
 $vararg_ptr1 = (($vararg_buffer) + 4|0);
 HEAP32[$vararg_ptr1>>2] = $3;
 _sqlite3_log(14,4472,$vararg_buffer);
 STACKTOP = sp;return 14;
}
function _walChecksumBytes($nativeCksum,$a,$nByte,$aIn,$aOut) {
 $nativeCksum = $nativeCksum|0;
 $a = $a|0;
 $nByte = $nByte|0;
 $aIn = $aIn|0;
 $aOut = $aOut|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0;
 var $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $aData = 0, $aEnd = 0, $s1 = 0, $s2 = 0, label = 0;
 var sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $0 = sp + 32|0;
 $1 = sp + 28|0;
 $2 = sp + 24|0;
 $3 = sp + 20|0;
 $4 = sp + 16|0;
 $s1 = sp + 12|0;
 $s2 = sp + 8|0;
 $aData = sp + 4|0;
 $aEnd = sp;
 HEAP32[$0>>2] = $nativeCksum;
 HEAP32[$1>>2] = $a;
 HEAP32[$2>>2] = $nByte;
 HEAP32[$3>>2] = $aIn;
 HEAP32[$4>>2] = $aOut;
 $5 = HEAP32[$1>>2]|0;
 HEAP32[$aData>>2] = $5;
 $6 = HEAP32[$2>>2]|0;
 $7 = HEAP32[$1>>2]|0;
 $8 = (($7) + ($6)|0);
 HEAP32[$aEnd>>2] = $8;
 $9 = HEAP32[$3>>2]|0;
 $10 = ($9|0)!=(0|0);
 if ($10) {
  $11 = HEAP32[$3>>2]|0;
  $12 = HEAP32[$11>>2]|0;
  HEAP32[$s1>>2] = $12;
  $13 = HEAP32[$3>>2]|0;
  $14 = (($13) + 4|0);
  $15 = HEAP32[$14>>2]|0;
  HEAP32[$s2>>2] = $15;
 } else {
  HEAP32[$s2>>2] = 0;
  HEAP32[$s1>>2] = 0;
 }
 $16 = HEAP32[$0>>2]|0;
 $17 = ($16|0)!=(0);
 if ($17) {
  while(1) {
   $18 = HEAP32[$aData>>2]|0;
   $19 = (($18) + 4|0);
   HEAP32[$aData>>2] = $19;
   $20 = HEAP32[$18>>2]|0;
   $21 = HEAP32[$s2>>2]|0;
   $22 = (($20) + ($21))|0;
   $23 = HEAP32[$s1>>2]|0;
   $24 = (($23) + ($22))|0;
   HEAP32[$s1>>2] = $24;
   $25 = HEAP32[$aData>>2]|0;
   $26 = (($25) + 4|0);
   HEAP32[$aData>>2] = $26;
   $27 = HEAP32[$25>>2]|0;
   $28 = HEAP32[$s1>>2]|0;
   $29 = (($27) + ($28))|0;
   $30 = HEAP32[$s2>>2]|0;
   $31 = (($30) + ($29))|0;
   HEAP32[$s2>>2] = $31;
   $32 = HEAP32[$aData>>2]|0;
   $33 = HEAP32[$aEnd>>2]|0;
   $34 = ($32>>>0)<($33>>>0);
   if (!($34)) {
    break;
   }
  }
  $90 = HEAP32[$s1>>2]|0;
  $91 = HEAP32[$4>>2]|0;
  HEAP32[$91>>2] = $90;
  $92 = HEAP32[$s2>>2]|0;
  $93 = HEAP32[$4>>2]|0;
  $94 = (($93) + 4|0);
  HEAP32[$94>>2] = $92;
  STACKTOP = sp;return;
 } else {
  while(1) {
   $35 = HEAP32[$aData>>2]|0;
   $36 = HEAP32[$35>>2]|0;
   $37 = $36 & 255;
   $38 = $37 << 24;
   $39 = HEAP32[$aData>>2]|0;
   $40 = HEAP32[$39>>2]|0;
   $41 = $40 & 65280;
   $42 = $41 << 8;
   $43 = (($38) + ($42))|0;
   $44 = HEAP32[$aData>>2]|0;
   $45 = HEAP32[$44>>2]|0;
   $46 = $45 & 16711680;
   $47 = $46 >>> 8;
   $48 = (($43) + ($47))|0;
   $49 = HEAP32[$aData>>2]|0;
   $50 = HEAP32[$49>>2]|0;
   $51 = $50 & -16777216;
   $52 = $51 >>> 24;
   $53 = (($48) + ($52))|0;
   $54 = HEAP32[$s2>>2]|0;
   $55 = (($53) + ($54))|0;
   $56 = HEAP32[$s1>>2]|0;
   $57 = (($56) + ($55))|0;
   HEAP32[$s1>>2] = $57;
   $58 = HEAP32[$aData>>2]|0;
   $59 = (($58) + 4|0);
   $60 = HEAP32[$59>>2]|0;
   $61 = $60 & 255;
   $62 = $61 << 24;
   $63 = HEAP32[$aData>>2]|0;
   $64 = (($63) + 4|0);
   $65 = HEAP32[$64>>2]|0;
   $66 = $65 & 65280;
   $67 = $66 << 8;
   $68 = (($62) + ($67))|0;
   $69 = HEAP32[$aData>>2]|0;
   $70 = (($69) + 4|0);
   $71 = HEAP32[$70>>2]|0;
   $72 = $71 & 16711680;
   $73 = $72 >>> 8;
   $74 = (($68) + ($73))|0;
   $75 = HEAP32[$aData>>2]|0;
   $76 = (($75) + 4|0);
   $77 = HEAP32[$76>>2]|0;
   $78 = $77 & -16777216;
   $79 = $78 >>> 24;
   $80 = (($74) + ($79))|0;
   $81 = HEAP32[$s1>>2]|0;
   $82 = (($80) + ($81))|0;
   $83 = HEAP32[$s2>>2]|0;
   $84 = (($83) + ($82))|0;
   HEAP32[$s2>>2] = $84;
   $85 = HEAP32[$aData>>2]|0;
   $86 = (($85) + 8|0);
   HEAP32[$aData>>2] = $86;
   $87 = HEAP32[$aData>>2]|0;
   $88 = HEAP32[$aEnd>>2]|0;
   $89 = ($87>>>0)<($88>>>0);
   if (!($89)) {
    break;
   }
  }
  $90 = HEAP32[$s1>>2]|0;
  $91 = HEAP32[$4>>2]|0;
  HEAP32[$91>>2] = $90;
  $92 = HEAP32[$s2>>2]|0;
  $93 = HEAP32[$4>>2]|0;
  $94 = (($93) + 4|0);
  HEAP32[$94>>2] = $92;
  STACKTOP = sp;return;
 }
}
function _walDecodeFrame($pWal,$piPage,$pnTruncate,$aData,$aFrame) {
 $pWal = $pWal|0;
 $piPage = $piPage|0;
 $pnTruncate = $pnTruncate|0;
 $aData = $aData|0;
 $aFrame = $aFrame|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $7 = 0;
 var $8 = 0, $9 = 0, $aCksum = 0, $nativeCksum = 0, $pgno = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $0 = sp + 32|0;
 $1 = sp + 28|0;
 $2 = sp + 24|0;
 $3 = sp + 20|0;
 $4 = sp + 16|0;
 $5 = sp + 12|0;
 $nativeCksum = sp + 8|0;
 $aCksum = sp + 4|0;
 $pgno = sp;
 HEAP32[$1>>2] = $pWal;
 HEAP32[$2>>2] = $piPage;
 HEAP32[$3>>2] = $pnTruncate;
 HEAP32[$4>>2] = $aData;
 HEAP32[$5>>2] = $aFrame;
 $6 = HEAP32[$1>>2]|0;
 $7 = (($6) + 52|0);
 $8 = (($7) + 24|0);
 HEAP32[$aCksum>>2] = $8;
 $9 = HEAP32[$1>>2]|0;
 $10 = (($9) + 52|0);
 $11 = (($10) + 32|0);
 $12 = HEAP32[$5>>2]|0;
 $13 = (($12) + 8|0);
 $14 = (_memcmp($11,$13,8)|0);
 $15 = ($14|0)!=(0);
 if ($15) {
  HEAP32[$0>>2] = 0;
  $61 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($61|0);
 }
 $16 = HEAP32[$5>>2]|0;
 $17 = (_sqlite3Get4byte($16)|0);
 HEAP32[$pgno>>2] = $17;
 $18 = HEAP32[$pgno>>2]|0;
 $19 = ($18|0)==(0);
 if ($19) {
  HEAP32[$0>>2] = 0;
  $61 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($61|0);
 }
 $20 = HEAP32[$1>>2]|0;
 $21 = (($20) + 52|0);
 $22 = (($21) + 13|0);
 $23 = HEAP8[$22>>0]|0;
 $24 = $23&255;
 $25 = HEAP8[1136>>0]|0;
 $26 = $25 << 24 >> 24;
 $27 = ($26|0)==(0);
 $28 = $27&1;
 $29 = ($24|0)==($28|0);
 $30 = $29&1;
 HEAP32[$nativeCksum>>2] = $30;
 $31 = HEAP32[$nativeCksum>>2]|0;
 $32 = HEAP32[$5>>2]|0;
 $33 = HEAP32[$aCksum>>2]|0;
 $34 = HEAP32[$aCksum>>2]|0;
 _walChecksumBytes($31,$32,8,$33,$34);
 $35 = HEAP32[$nativeCksum>>2]|0;
 $36 = HEAP32[$4>>2]|0;
 $37 = HEAP32[$1>>2]|0;
 $38 = (($37) + 36|0);
 $39 = HEAP32[$38>>2]|0;
 $40 = HEAP32[$aCksum>>2]|0;
 $41 = HEAP32[$aCksum>>2]|0;
 _walChecksumBytes($35,$36,$39,$40,$41);
 $42 = HEAP32[$aCksum>>2]|0;
 $43 = HEAP32[$42>>2]|0;
 $44 = HEAP32[$5>>2]|0;
 $45 = (($44) + 16|0);
 $46 = (_sqlite3Get4byte($45)|0);
 $47 = ($43|0)!=($46|0);
 if (!($47)) {
  $48 = HEAP32[$aCksum>>2]|0;
  $49 = (($48) + 4|0);
  $50 = HEAP32[$49>>2]|0;
  $51 = HEAP32[$5>>2]|0;
  $52 = (($51) + 20|0);
  $53 = (_sqlite3Get4byte($52)|0);
  $54 = ($50|0)!=($53|0);
  if (!($54)) {
   $55 = HEAP32[$pgno>>2]|0;
   $56 = HEAP32[$2>>2]|0;
   HEAP32[$56>>2] = $55;
   $57 = HEAP32[$5>>2]|0;
   $58 = (($57) + 4|0);
   $59 = (_sqlite3Get4byte($58)|0);
   $60 = HEAP32[$3>>2]|0;
   HEAP32[$60>>2] = $59;
   HEAP32[$0>>2] = 1;
   $61 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($61|0);
  }
 }
 HEAP32[$0>>2] = 0;
 $61 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($61|0);
}
function _walIndexAppend($pWal,$iFrame,$iPage) {
 $pWal = $pWal|0;
 $iFrame = $iFrame|0;
 $iPage = $iPage|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $aHash = 0, $aPgno = 0, $iKey = 0, $iZero = 0;
 var $idx = 0, $nByte = 0, $nCollide = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $0 = sp + 44|0;
 $1 = sp + 40|0;
 $2 = sp + 36|0;
 $3 = sp + 32|0;
 $rc = sp + 28|0;
 $iZero = sp + 24|0;
 $aPgno = sp + 20|0;
 $aHash = sp + 16|0;
 $iKey = sp + 12|0;
 $idx = sp + 8|0;
 $nCollide = sp + 4|0;
 $nByte = sp;
 HEAP32[$1>>2] = $pWal;
 HEAP32[$2>>2] = $iFrame;
 HEAP32[$3>>2] = $iPage;
 HEAP32[$iZero>>2] = 0;
 HEAP32[$aPgno>>2] = 0;
 HEAP32[$aHash>>2] = 0;
 $4 = HEAP32[$1>>2]|0;
 $5 = HEAP32[$2>>2]|0;
 $6 = (_walFramePage($5)|0);
 $7 = (_walHashGet($4,$6,$aHash,$aPgno,$iZero)|0);
 HEAP32[$rc>>2] = $7;
 $8 = HEAP32[$rc>>2]|0;
 $9 = ($8|0)==(0);
 do {
  if ($9) {
   $10 = HEAP32[$2>>2]|0;
   $11 = HEAP32[$iZero>>2]|0;
   $12 = (($10) - ($11))|0;
   HEAP32[$idx>>2] = $12;
   $13 = HEAP32[$idx>>2]|0;
   $14 = ($13|0)==(1);
   if ($14) {
    $15 = HEAP32[$aHash>>2]|0;
    $16 = (($15) + 16384|0);
    $17 = HEAP32[$aPgno>>2]|0;
    $18 = (($17) + 4|0);
    $19 = $16;
    $20 = $18;
    $21 = (($19) - ($20))|0;
    HEAP32[$nByte>>2] = $21;
    $22 = HEAP32[$aPgno>>2]|0;
    $23 = (($22) + 4|0);
    $24 = HEAP32[$nByte>>2]|0;
    _memset(($23|0),0,($24|0))|0;
   }
   $25 = HEAP32[$idx>>2]|0;
   $26 = HEAP32[$aPgno>>2]|0;
   $27 = (($26) + ($25<<2)|0);
   $28 = HEAP32[$27>>2]|0;
   $29 = ($28|0)!=(0);
   if ($29) {
    $30 = HEAP32[$1>>2]|0;
    _walCleanupHash($30);
   }
   $31 = HEAP32[$idx>>2]|0;
   HEAP32[$nCollide>>2] = $31;
   $32 = HEAP32[$3>>2]|0;
   $33 = (_walHash($32)|0);
   HEAP32[$iKey>>2] = $33;
   while(1) {
    $34 = HEAP32[$iKey>>2]|0;
    $35 = HEAP32[$aHash>>2]|0;
    $36 = (($35) + ($34<<1)|0);
    $37 = HEAP16[$36>>1]|0;
    $38 = ($37<<16>>16)!=(0);
    if (!($38)) {
     label = 11;
     break;
    }
    $39 = HEAP32[$nCollide>>2]|0;
    $40 = (($39) + -1)|0;
    HEAP32[$nCollide>>2] = $40;
    $41 = ($39|0)==(0);
    if ($41) {
     break;
    }
    $43 = HEAP32[$iKey>>2]|0;
    $44 = (_walNextHash($43)|0);
    HEAP32[$iKey>>2] = $44;
   }
   if ((label|0) == 11) {
    $45 = HEAP32[$3>>2]|0;
    $46 = HEAP32[$idx>>2]|0;
    $47 = HEAP32[$aPgno>>2]|0;
    $48 = (($47) + ($46<<2)|0);
    HEAP32[$48>>2] = $45;
    $49 = HEAP32[$idx>>2]|0;
    $50 = $49&65535;
    $51 = HEAP32[$iKey>>2]|0;
    $52 = HEAP32[$aHash>>2]|0;
    $53 = (($52) + ($51<<1)|0);
    HEAP16[$53>>1] = $50;
    break;
   }
   $42 = (_sqlite3CorruptError(49129)|0);
   HEAP32[$0>>2] = $42;
   $55 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($55|0);
  }
 } while(0);
 $54 = HEAP32[$rc>>2]|0;
 HEAP32[$0>>2] = $54;
 $55 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($55|0);
}
function _walIndexWriteHdr($pWal) {
 $pWal = $pWal|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0;
 var $9 = 0, $aHdr = 0, $nCksum = 0, dest = 0, label = 0, sp = 0, src = 0, stop = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $aHdr = sp + 4|0;
 $nCksum = sp;
 HEAP32[$0>>2] = $pWal;
 $1 = HEAP32[$0>>2]|0;
 $2 = (_walIndexHdr($1)|0);
 HEAP32[$aHdr>>2] = $2;
 HEAP32[$nCksum>>2] = 40;
 $3 = HEAP32[$0>>2]|0;
 $4 = (($3) + 52|0);
 $5 = (($4) + 12|0);
 HEAP8[$5>>0] = 1;
 $6 = HEAP32[$0>>2]|0;
 $7 = (($6) + 52|0);
 HEAP32[$7>>2] = 3007000;
 $8 = HEAP32[$0>>2]|0;
 $9 = (($8) + 52|0);
 $10 = HEAP32[$0>>2]|0;
 $11 = (($10) + 52|0);
 $12 = (($11) + 40|0);
 _walChecksumBytes(1,$9,40,0,$12);
 $13 = HEAP32[$aHdr>>2]|0;
 $14 = (($13) + 48|0);
 $15 = HEAP32[$0>>2]|0;
 $16 = (($15) + 52|0);
 dest=$14+0|0; src=$16+0|0; stop=dest+48|0; do { HEAP8[dest>>0]=HEAP8[src>>0]|0; dest=dest+1|0; src=src+1|0; } while ((dest|0) < (stop|0));
 $17 = HEAP32[$0>>2]|0;
 _walShmBarrier($17);
 $18 = HEAP32[$aHdr>>2]|0;
 $19 = HEAP32[$0>>2]|0;
 $20 = (($19) + 52|0);
 dest=$18+0|0; src=$20+0|0; stop=dest+48|0; do { HEAP8[dest>>0]=HEAP8[src>>0]|0; dest=dest+1|0; src=src+1|0; } while ((dest|0) < (stop|0));
 STACKTOP = sp;return;
}
function _walShmBarrier($pWal) {
 $pWal = $pWal|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 HEAP32[$0>>2] = $pWal;
 $1 = HEAP32[$0>>2]|0;
 $2 = (($1) + 43|0);
 $3 = HEAP8[$2>>0]|0;
 $4 = $3&255;
 $5 = ($4|0)!=(2);
 if (!($5)) {
  STACKTOP = sp;return;
 }
 $6 = HEAP32[$0>>2]|0;
 $7 = (($6) + 4|0);
 $8 = HEAP32[$7>>2]|0;
 _sqlite3OsShmBarrier($8);
 STACKTOP = sp;return;
}
function _sqlite3OsShmBarrier($id) {
 $id = $id|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 HEAP32[$0>>2] = $id;
 $1 = HEAP32[$0>>2]|0;
 $2 = HEAP32[$1>>2]|0;
 $3 = (($2) + 60|0);
 $4 = HEAP32[$3>>2]|0;
 $5 = HEAP32[$0>>2]|0;
 FUNCTION_TABLE_vi[$4 & 31]($5);
 STACKTOP = sp;return;
}
function _sqlite3InvokeBusyHandler($p) {
 $p = $p|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $1 = sp + 4|0;
 $rc = sp;
 HEAP32[$1>>2] = $p;
 $2 = HEAP32[$1>>2]|0;
 $3 = ($2|0)==(0|0);
 if (!($3)) {
  $4 = HEAP32[$1>>2]|0;
  $5 = HEAP32[$4>>2]|0;
  $6 = ($5|0)==(0|0);
  if (!($6)) {
   $7 = HEAP32[$1>>2]|0;
   $8 = (($7) + 8|0);
   $9 = HEAP32[$8>>2]|0;
   $10 = ($9|0)<(0);
   if (!($10)) {
    $11 = HEAP32[$1>>2]|0;
    $12 = HEAP32[$11>>2]|0;
    $13 = HEAP32[$1>>2]|0;
    $14 = (($13) + 4|0);
    $15 = HEAP32[$14>>2]|0;
    $16 = HEAP32[$1>>2]|0;
    $17 = (($16) + 8|0);
    $18 = HEAP32[$17>>2]|0;
    $19 = (FUNCTION_TABLE_iii[$12 & 63]($15,$18)|0);
    HEAP32[$rc>>2] = $19;
    $20 = HEAP32[$rc>>2]|0;
    $21 = ($20|0)==(0);
    $22 = HEAP32[$1>>2]|0;
    $23 = (($22) + 8|0);
    if ($21) {
     HEAP32[$23>>2] = -1;
    } else {
     $24 = HEAP32[$23>>2]|0;
     $25 = (($24) + 1)|0;
     HEAP32[$23>>2] = $25;
    }
    $26 = HEAP32[$rc>>2]|0;
    HEAP32[$0>>2] = $26;
    $27 = HEAP32[$0>>2]|0;
    STACKTOP = sp;return ($27|0);
   }
  }
 }
 HEAP32[$0>>2] = 0;
 $27 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($27|0);
}
function _sqlite3PagerGetExtra($pPg) {
 $pPg = $pPg|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 HEAP32[$0>>2] = $pPg;
 $1 = HEAP32[$0>>2]|0;
 $2 = (($1) + 8|0);
 $3 = HEAP32[$2>>2]|0;
 STACKTOP = sp;return ($3|0);
}
function _sqlite3PagerPageRefcount($pPage) {
 $pPage = $pPage|0;
 var $0 = 0, $1 = 0, $2 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 HEAP32[$0>>2] = $pPage;
 $1 = HEAP32[$0>>2]|0;
 $2 = (_sqlite3PcachePageRefcount($1)|0);
 STACKTOP = sp;return ($2|0);
}
function _btreeInitPage($pPage) {
 $pPage = $pPage|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0;
 var $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0, $188 = 0;
 var $189 = 0, $19 = 0, $190 = 0, $191 = 0, $192 = 0, $193 = 0, $194 = 0, $195 = 0, $196 = 0, $197 = 0, $198 = 0, $199 = 0, $2 = 0, $20 = 0, $200 = 0, $201 = 0, $202 = 0, $203 = 0, $204 = 0, $205 = 0;
 var $206 = 0, $207 = 0, $208 = 0, $209 = 0, $21 = 0, $210 = 0, $211 = 0, $212 = 0, $213 = 0, $214 = 0, $215 = 0, $216 = 0, $217 = 0, $218 = 0, $219 = 0, $22 = 0, $220 = 0, $221 = 0, $222 = 0, $223 = 0;
 var $224 = 0, $225 = 0, $226 = 0, $227 = 0, $228 = 0, $229 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0;
 var $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0;
 var $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0;
 var $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0;
 var $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $cellOffset = 0, $data = 0, $hdr = 0, $iCellFirst = 0, $iCellLast = 0, $nFree = 0, $next = 0, $pBt = 0, $pc = 0, $size = 0;
 var $top = 0, $usableSize = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $0 = sp + 32|0;
 $1 = sp + 28|0;
 $pc = sp + 42|0;
 $hdr = sp + 44|0;
 $data = sp + 24|0;
 $pBt = sp + 20|0;
 $usableSize = sp + 16|0;
 $cellOffset = sp + 40|0;
 $nFree = sp + 12|0;
 $top = sp + 8|0;
 $iCellFirst = sp + 4|0;
 $iCellLast = sp;
 $next = sp + 38|0;
 $size = sp + 36|0;
 HEAP32[$1>>2] = $pPage;
 $2 = HEAP32[$1>>2]|0;
 $3 = HEAP8[$2>>0]|0;
 $4 = ($3<<24>>24)!=(0);
 do {
  if (!($4)) {
   $5 = HEAP32[$1>>2]|0;
   $6 = (($5) + 52|0);
   $7 = HEAP32[$6>>2]|0;
   HEAP32[$pBt>>2] = $7;
   $8 = HEAP32[$1>>2]|0;
   $9 = (($8) + 6|0);
   $10 = HEAP8[$9>>0]|0;
   HEAP8[$hdr>>0] = $10;
   $11 = HEAP32[$1>>2]|0;
   $12 = (($11) + 56|0);
   $13 = HEAP32[$12>>2]|0;
   HEAP32[$data>>2] = $13;
   $14 = HEAP32[$1>>2]|0;
   $15 = HEAP8[$hdr>>0]|0;
   $16 = $15&255;
   $17 = HEAP32[$data>>2]|0;
   $18 = (($17) + ($16)|0);
   $19 = HEAP8[$18>>0]|0;
   $20 = $19&255;
   $21 = (_decodeFlags($14,$20)|0);
   $22 = ($21|0)!=(0);
   if ($22) {
    $23 = (_sqlite3CorruptError(53694)|0);
    HEAP32[$0>>2] = $23;
    $229 = HEAP32[$0>>2]|0;
    STACKTOP = sp;return ($229|0);
   }
   $24 = HEAP32[$pBt>>2]|0;
   $25 = (($24) + 32|0);
   $26 = HEAP32[$25>>2]|0;
   $27 = (($26) - 1)|0;
   $28 = $27&65535;
   $29 = HEAP32[$1>>2]|0;
   $30 = (($29) + 20|0);
   HEAP16[$30>>1] = $28;
   $31 = HEAP32[$1>>2]|0;
   $32 = (($31) + 1|0);
   HEAP8[$32>>0] = 0;
   $33 = HEAP32[$pBt>>2]|0;
   $34 = (($33) + 36|0);
   $35 = HEAP32[$34>>2]|0;
   HEAP32[$usableSize>>2] = $35;
   $36 = HEAP8[$hdr>>0]|0;
   $37 = $36&255;
   $38 = (($37) + 12)|0;
   $39 = HEAP32[$1>>2]|0;
   $40 = (($39) + 5|0);
   $41 = HEAP8[$40>>0]|0;
   $42 = $41&255;
   $43 = $42<<2;
   $44 = (($38) - ($43))|0;
   $45 = $44&65535;
   HEAP16[$cellOffset>>1] = $45;
   $46 = HEAP32[$1>>2]|0;
   $47 = (($46) + 14|0);
   HEAP16[$47>>1] = $45;
   $48 = HEAP32[$usableSize>>2]|0;
   $49 = HEAP32[$data>>2]|0;
   $50 = (($49) + ($48)|0);
   $51 = HEAP32[$1>>2]|0;
   $52 = (($51) + 60|0);
   HEAP32[$52>>2] = $50;
   $53 = HEAP16[$cellOffset>>1]|0;
   $54 = $53&65535;
   $55 = HEAP32[$data>>2]|0;
   $56 = (($55) + ($54)|0);
   $57 = HEAP32[$1>>2]|0;
   $58 = (($57) + 64|0);
   HEAP32[$58>>2] = $56;
   $59 = HEAP8[$hdr>>0]|0;
   $60 = $59&255;
   $61 = (($60) + 5)|0;
   $62 = HEAP32[$data>>2]|0;
   $63 = (($62) + ($61)|0);
   $64 = HEAP8[$63>>0]|0;
   $65 = $64&255;
   $66 = $65 << 8;
   $67 = HEAP8[$hdr>>0]|0;
   $68 = $67&255;
   $69 = (($68) + 5)|0;
   $70 = HEAP32[$data>>2]|0;
   $71 = (($70) + ($69)|0);
   $72 = (($71) + 1|0);
   $73 = HEAP8[$72>>0]|0;
   $74 = $73&255;
   $75 = $66 | $74;
   $76 = (($75) - 1)|0;
   $77 = $76 & 65535;
   $78 = (($77) + 1)|0;
   HEAP32[$top>>2] = $78;
   $79 = HEAP8[$hdr>>0]|0;
   $80 = $79&255;
   $81 = (($80) + 3)|0;
   $82 = HEAP32[$data>>2]|0;
   $83 = (($82) + ($81)|0);
   $84 = HEAP8[$83>>0]|0;
   $85 = $84&255;
   $86 = $85 << 8;
   $87 = HEAP8[$hdr>>0]|0;
   $88 = $87&255;
   $89 = (($88) + 3)|0;
   $90 = HEAP32[$data>>2]|0;
   $91 = (($90) + ($89)|0);
   $92 = (($91) + 1|0);
   $93 = HEAP8[$92>>0]|0;
   $94 = $93&255;
   $95 = $86 | $94;
   $96 = $95&65535;
   $97 = HEAP32[$1>>2]|0;
   $98 = (($97) + 18|0);
   HEAP16[$98>>1] = $96;
   $99 = HEAP32[$1>>2]|0;
   $100 = (($99) + 18|0);
   $101 = HEAP16[$100>>1]|0;
   $102 = $101&65535;
   $103 = HEAP32[$pBt>>2]|0;
   $104 = (($103) + 32|0);
   $105 = HEAP32[$104>>2]|0;
   $106 = (($105) - 8)|0;
   $107 = (($106>>>0) / 6)&-1;
   $108 = ($102>>>0)>($107>>>0);
   if ($108) {
    $109 = (_sqlite3CorruptError(53706)|0);
    HEAP32[$0>>2] = $109;
    $229 = HEAP32[$0>>2]|0;
    STACKTOP = sp;return ($229|0);
   }
   $110 = HEAP16[$cellOffset>>1]|0;
   $111 = $110&65535;
   $112 = HEAP32[$1>>2]|0;
   $113 = (($112) + 18|0);
   $114 = HEAP16[$113>>1]|0;
   $115 = $114&65535;
   $116 = $115<<1;
   $117 = (($111) + ($116))|0;
   HEAP32[$iCellFirst>>2] = $117;
   $118 = HEAP32[$usableSize>>2]|0;
   $119 = (($118) - 4)|0;
   HEAP32[$iCellLast>>2] = $119;
   $120 = HEAP8[$hdr>>0]|0;
   $121 = $120&255;
   $122 = (($121) + 1)|0;
   $123 = HEAP32[$data>>2]|0;
   $124 = (($123) + ($122)|0);
   $125 = HEAP8[$124>>0]|0;
   $126 = $125&255;
   $127 = $126 << 8;
   $128 = HEAP8[$hdr>>0]|0;
   $129 = $128&255;
   $130 = (($129) + 1)|0;
   $131 = HEAP32[$data>>2]|0;
   $132 = (($131) + ($130)|0);
   $133 = (($132) + 1|0);
   $134 = HEAP8[$133>>0]|0;
   $135 = $134&255;
   $136 = $127 | $135;
   $137 = $136&65535;
   HEAP16[$pc>>1] = $137;
   $138 = HEAP8[$hdr>>0]|0;
   $139 = $138&255;
   $140 = (($139) + 7)|0;
   $141 = HEAP32[$data>>2]|0;
   $142 = (($141) + ($140)|0);
   $143 = HEAP8[$142>>0]|0;
   $144 = $143&255;
   $145 = HEAP32[$top>>2]|0;
   $146 = (($144) + ($145))|0;
   HEAP32[$nFree>>2] = $146;
   while(1) {
    $147 = HEAP16[$pc>>1]|0;
    $148 = $147&65535;
    $149 = ($148|0)>(0);
    if (!($149)) {
     label = 16;
     break;
    }
    $150 = HEAP16[$pc>>1]|0;
    $151 = $150&65535;
    $152 = HEAP32[$iCellFirst>>2]|0;
    $153 = ($151|0)<($152|0);
    if ($153) {
     label = 10;
     break;
    }
    $154 = HEAP16[$pc>>1]|0;
    $155 = $154&65535;
    $156 = HEAP32[$iCellLast>>2]|0;
    $157 = ($155|0)>($156|0);
    if ($157) {
     label = 10;
     break;
    }
    $159 = HEAP16[$pc>>1]|0;
    $160 = $159&65535;
    $161 = HEAP32[$data>>2]|0;
    $162 = (($161) + ($160)|0);
    $163 = HEAP8[$162>>0]|0;
    $164 = $163&255;
    $165 = $164 << 8;
    $166 = HEAP16[$pc>>1]|0;
    $167 = $166&65535;
    $168 = HEAP32[$data>>2]|0;
    $169 = (($168) + ($167)|0);
    $170 = (($169) + 1|0);
    $171 = HEAP8[$170>>0]|0;
    $172 = $171&255;
    $173 = $165 | $172;
    $174 = $173&65535;
    HEAP16[$next>>1] = $174;
    $175 = HEAP16[$pc>>1]|0;
    $176 = $175&65535;
    $177 = (($176) + 2)|0;
    $178 = HEAP32[$data>>2]|0;
    $179 = (($178) + ($177)|0);
    $180 = HEAP8[$179>>0]|0;
    $181 = $180&255;
    $182 = $181 << 8;
    $183 = HEAP16[$pc>>1]|0;
    $184 = $183&65535;
    $185 = (($184) + 2)|0;
    $186 = HEAP32[$data>>2]|0;
    $187 = (($186) + ($185)|0);
    $188 = (($187) + 1|0);
    $189 = HEAP8[$188>>0]|0;
    $190 = $189&255;
    $191 = $182 | $190;
    $192 = $191&65535;
    HEAP16[$size>>1] = $192;
    $193 = HEAP16[$next>>1]|0;
    $194 = $193&65535;
    $195 = ($194|0)>(0);
    if ($195) {
     $196 = HEAP16[$next>>1]|0;
     $197 = $196&65535;
     $198 = HEAP16[$pc>>1]|0;
     $199 = $198&65535;
     $200 = HEAP16[$size>>1]|0;
     $201 = $200&65535;
     $202 = (($199) + ($201))|0;
     $203 = (($202) + 3)|0;
     $204 = ($197|0)<=($203|0);
     if ($204) {
      label = 14;
      break;
     }
    }
    $205 = HEAP16[$pc>>1]|0;
    $206 = $205&65535;
    $207 = HEAP16[$size>>1]|0;
    $208 = $207&65535;
    $209 = (($206) + ($208))|0;
    $210 = HEAP32[$usableSize>>2]|0;
    $211 = ($209|0)>($210|0);
    if ($211) {
     label = 14;
     break;
    }
    $213 = HEAP32[$nFree>>2]|0;
    $214 = HEAP16[$size>>1]|0;
    $215 = $214&65535;
    $216 = (($213) + ($215))|0;
    HEAP32[$nFree>>2] = $216;
    $217 = HEAP16[$next>>1]|0;
    HEAP16[$pc>>1] = $217;
   }
   if ((label|0) == 10) {
    $158 = (_sqlite3CorruptError(53749)|0);
    HEAP32[$0>>2] = $158;
    $229 = HEAP32[$0>>2]|0;
    STACKTOP = sp;return ($229|0);
   }
   else if ((label|0) == 14) {
    $212 = (_sqlite3CorruptError(53756)|0);
    HEAP32[$0>>2] = $212;
    $229 = HEAP32[$0>>2]|0;
    STACKTOP = sp;return ($229|0);
   }
   else if ((label|0) == 16) {
    $218 = HEAP32[$nFree>>2]|0;
    $219 = HEAP32[$usableSize>>2]|0;
    $220 = ($218|0)>($219|0);
    if (!($220)) {
     $222 = HEAP32[$nFree>>2]|0;
     $223 = HEAP32[$iCellFirst>>2]|0;
     $224 = (($222) - ($223))|0;
     $225 = $224&65535;
     $226 = HEAP32[$1>>2]|0;
     $227 = (($226) + 16|0);
     HEAP16[$227>>1] = $225;
     $228 = HEAP32[$1>>2]|0;
     HEAP8[$228>>0] = 1;
     break;
    }
    $221 = (_sqlite3CorruptError(53770)|0);
    HEAP32[$0>>2] = $221;
    $229 = HEAP32[$0>>2]|0;
    STACKTOP = sp;return ($229|0);
   }
  }
 } while(0);
 HEAP32[$0>>2] = 0;
 $229 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($229|0);
}
function _decodeFlags($pPage,$flagByte) {
 $pPage = $pPage|0;
 $flagByte = $flagByte|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $8 = 0, $9 = 0, $pBt = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $1 = sp + 8|0;
 $2 = sp + 4|0;
 $pBt = sp;
 HEAP32[$1>>2] = $pPage;
 HEAP32[$2>>2] = $flagByte;
 $3 = HEAP32[$2>>2]|0;
 $4 = $3 >> 3;
 $5 = $4&255;
 $6 = HEAP32[$1>>2]|0;
 $7 = (($6) + 5|0);
 HEAP8[$7>>0] = $5;
 $8 = HEAP32[$2>>2]|0;
 $9 = $8 & -9;
 HEAP32[$2>>2] = $9;
 $10 = HEAP32[$1>>2]|0;
 $11 = (($10) + 5|0);
 $12 = HEAP8[$11>>0]|0;
 $13 = $12&255;
 $14 = $13<<2;
 $15 = (4 - ($14))|0;
 $16 = $15&255;
 $17 = HEAP32[$1>>2]|0;
 $18 = (($17) + 7|0);
 HEAP8[$18>>0] = $16;
 $19 = HEAP32[$1>>2]|0;
 $20 = (($19) + 52|0);
 $21 = HEAP32[$20>>2]|0;
 HEAP32[$pBt>>2] = $21;
 $22 = HEAP32[$2>>2]|0;
 $23 = ($22|0)==(5);
 do {
  if ($23) {
   $24 = HEAP32[$1>>2]|0;
   $25 = (($24) + 2|0);
   HEAP8[$25>>0] = 1;
   $26 = HEAP32[$1>>2]|0;
   $27 = (($26) + 5|0);
   $28 = HEAP8[$27>>0]|0;
   $29 = HEAP32[$1>>2]|0;
   $30 = (($29) + 3|0);
   HEAP8[$30>>0] = $28;
   $31 = HEAP32[$1>>2]|0;
   $32 = (($31) + 5|0);
   $33 = HEAP8[$32>>0]|0;
   $34 = ($33<<24>>24)!=(0);
   $35 = $34 ^ 1;
   $36 = $35&1;
   $37 = $36&255;
   $38 = HEAP32[$1>>2]|0;
   $39 = (($38) + 4|0);
   HEAP8[$39>>0] = $37;
   $40 = HEAP32[$pBt>>2]|0;
   $41 = (($40) + 28|0);
   $42 = HEAP16[$41>>1]|0;
   $43 = HEAP32[$1>>2]|0;
   $44 = (($43) + 10|0);
   HEAP16[$44>>1] = $42;
   $45 = HEAP32[$pBt>>2]|0;
   $46 = (($45) + 30|0);
   $47 = HEAP16[$46>>1]|0;
   $48 = HEAP32[$1>>2]|0;
   $49 = (($48) + 12|0);
   HEAP16[$49>>1] = $47;
  } else {
   $50 = HEAP32[$2>>2]|0;
   $51 = ($50|0)==(2);
   if ($51) {
    $52 = HEAP32[$1>>2]|0;
    $53 = (($52) + 2|0);
    HEAP8[$53>>0] = 0;
    $54 = HEAP32[$1>>2]|0;
    $55 = (($54) + 3|0);
    HEAP8[$55>>0] = 0;
    $56 = HEAP32[$1>>2]|0;
    $57 = (($56) + 4|0);
    HEAP8[$57>>0] = 0;
    $58 = HEAP32[$pBt>>2]|0;
    $59 = (($58) + 24|0);
    $60 = HEAP16[$59>>1]|0;
    $61 = HEAP32[$1>>2]|0;
    $62 = (($61) + 10|0);
    HEAP16[$62>>1] = $60;
    $63 = HEAP32[$pBt>>2]|0;
    $64 = (($63) + 26|0);
    $65 = HEAP16[$64>>1]|0;
    $66 = HEAP32[$1>>2]|0;
    $67 = (($66) + 12|0);
    HEAP16[$67>>1] = $65;
    break;
   }
   $68 = (_sqlite3CorruptError(53655)|0);
   HEAP32[$0>>2] = $68;
   $74 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($74|0);
  }
 } while(0);
 $69 = HEAP32[$pBt>>2]|0;
 $70 = (($69) + 21|0);
 $71 = HEAP8[$70>>0]|0;
 $72 = HEAP32[$1>>2]|0;
 $73 = (($72) + 8|0);
 HEAP8[$73>>0] = $71;
 HEAP32[$0>>2] = 0;
 $74 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($74|0);
}
function _sqlite3PcacheSize() {
 var label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = sp;return 48;
}
function _sqlite3PcacheOpen($szPage,$szExtra,$bPurgeable,$xStress,$pStress,$p) {
 $szPage = $szPage|0;
 $szExtra = $szExtra|0;
 $bPurgeable = $bPurgeable|0;
 $xStress = $xStress|0;
 $pStress = $pStress|0;
 $p = $p|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, dest = 0, label = 0, sp = 0, stop = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 20|0;
 $1 = sp + 16|0;
 $2 = sp + 12|0;
 $3 = sp + 8|0;
 $4 = sp + 4|0;
 $5 = sp;
 HEAP32[$0>>2] = $szPage;
 HEAP32[$1>>2] = $szExtra;
 HEAP32[$2>>2] = $bPurgeable;
 HEAP32[$3>>2] = $xStress;
 HEAP32[$4>>2] = $pStress;
 HEAP32[$5>>2] = $p;
 $6 = HEAP32[$5>>2]|0;
 dest=$6+0|0; stop=dest+48|0; do { HEAP32[dest>>2]=0|0; dest=dest+4|0; } while ((dest|0) < (stop|0));
 $7 = HEAP32[$5>>2]|0;
 $8 = (($7) + 20|0);
 HEAP32[$8>>2] = 1;
 $9 = HEAP32[$1>>2]|0;
 $10 = HEAP32[$5>>2]|0;
 $11 = (($10) + 24|0);
 HEAP32[$11>>2] = $9;
 $12 = HEAP32[$2>>2]|0;
 $13 = $12&255;
 $14 = HEAP32[$5>>2]|0;
 $15 = (($14) + 28|0);
 HEAP8[$15>>0] = $13;
 $16 = HEAP32[$5>>2]|0;
 $17 = (($16) + 29|0);
 HEAP8[$17>>0] = 2;
 $18 = HEAP32[$3>>2]|0;
 $19 = HEAP32[$5>>2]|0;
 $20 = (($19) + 32|0);
 HEAP32[$20>>2] = $18;
 $21 = HEAP32[$4>>2]|0;
 $22 = HEAP32[$5>>2]|0;
 $23 = (($22) + 36|0);
 HEAP32[$23>>2] = $21;
 $24 = HEAP32[$5>>2]|0;
 $25 = (($24) + 16|0);
 HEAP32[$25>>2] = 100;
 $26 = HEAP32[$5>>2]|0;
 $27 = HEAP32[$0>>2]|0;
 $28 = (_sqlite3PcacheSetPageSize($26,$27)|0);
 STACKTOP = sp;return ($28|0);
}
function _pagerStress($p,$pPg) {
 $p = $p|0;
 $pPg = $pPg|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $8 = 0, $9 = 0, $pPager = 0, $rc = 0;
 var label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 16|0;
 $1 = sp + 12|0;
 $2 = sp + 8|0;
 $pPager = sp + 4|0;
 $rc = sp;
 HEAP32[$1>>2] = $p;
 HEAP32[$2>>2] = $pPg;
 $3 = HEAP32[$1>>2]|0;
 HEAP32[$pPager>>2] = $3;
 HEAP32[$rc>>2] = 0;
 $4 = HEAP32[$pPager>>2]|0;
 $5 = (($4) + 40|0);
 $6 = HEAP32[$5>>2]|0;
 $7 = ($6|0)!=(0);
 if ($7) {
  HEAP32[$0>>2] = 0;
  $77 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($77|0);
 }
 $8 = HEAP32[$pPager>>2]|0;
 $9 = (($8) + 20|0);
 $10 = HEAP8[$9>>0]|0;
 $11 = $10&255;
 $12 = ($11|0)!=(0);
 do {
  if ($12) {
   $13 = HEAP32[$pPager>>2]|0;
   $14 = (($13) + 20|0);
   $15 = HEAP8[$14>>0]|0;
   $16 = $15&255;
   $17 = $16 & 3;
   $18 = ($17|0)!=(0);
   if (!($18)) {
    $19 = HEAP32[$2>>2]|0;
    $20 = (($19) + 24|0);
    $21 = HEAP16[$20>>1]|0;
    $22 = $21&65535;
    $23 = $22 & 4;
    $24 = ($23|0)!=(0);
    if (!($24)) {
     break;
    }
   }
   HEAP32[$0>>2] = 0;
   $77 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($77|0);
  }
 } while(0);
 $25 = HEAP32[$2>>2]|0;
 $26 = (($25) + 12|0);
 HEAP32[$26>>2] = 0;
 $27 = HEAP32[$pPager>>2]|0;
 $28 = (_pagerUseWal($27)|0);
 $29 = ($28|0)!=(0);
 $30 = HEAP32[$2>>2]|0;
 if ($29) {
  $31 = (_subjRequiresPage($30)|0);
  $32 = ($31|0)!=(0);
  if ($32) {
   $33 = HEAP32[$2>>2]|0;
   $34 = (_subjournalPage($33)|0);
   HEAP32[$rc>>2] = $34;
  }
  $35 = HEAP32[$rc>>2]|0;
  $36 = ($35|0)==(0);
  if ($36) {
   $37 = HEAP32[$pPager>>2]|0;
   $38 = HEAP32[$2>>2]|0;
   $39 = (_pagerWalFrames($37,$38,0,0)|0);
   HEAP32[$rc>>2] = $39;
  }
 } else {
  $40 = (($30) + 24|0);
  $41 = HEAP16[$40>>1]|0;
  $42 = $41&65535;
  $43 = $42 & 4;
  $44 = ($43|0)!=(0);
  if ($44) {
   label = 14;
  } else {
   $45 = HEAP32[$pPager>>2]|0;
   $46 = (($45) + 16|0);
   $47 = HEAP8[$46>>0]|0;
   $48 = $47&255;
   $49 = ($48|0)==(3);
   if ($49) {
    label = 14;
   }
  }
  if ((label|0) == 14) {
   $50 = HEAP32[$pPager>>2]|0;
   $51 = (_syncJournal($50,1)|0);
   HEAP32[$rc>>2] = $51;
  }
  $52 = HEAP32[$rc>>2]|0;
  $53 = ($52|0)==(0);
  if ($53) {
   $54 = HEAP32[$2>>2]|0;
   $55 = (($54) + 20|0);
   $56 = HEAP32[$55>>2]|0;
   $57 = HEAP32[$pPager>>2]|0;
   $58 = (($57) + 24|0);
   $59 = HEAP32[$58>>2]|0;
   $60 = ($56>>>0)>($59>>>0);
   if ($60) {
    $61 = HEAP32[$2>>2]|0;
    $62 = (_subjRequiresPage($61)|0);
    $63 = ($62|0)!=(0);
    if ($63) {
     $64 = HEAP32[$2>>2]|0;
     $65 = (_subjournalPage($64)|0);
     HEAP32[$rc>>2] = $65;
    }
   }
  }
  $66 = HEAP32[$rc>>2]|0;
  $67 = ($66|0)==(0);
  if ($67) {
   $68 = HEAP32[$pPager>>2]|0;
   $69 = HEAP32[$2>>2]|0;
   $70 = (_pager_write_pagelist($68,$69)|0);
   HEAP32[$rc>>2] = $70;
  }
 }
 $71 = HEAP32[$rc>>2]|0;
 $72 = ($71|0)==(0);
 if ($72) {
  $73 = HEAP32[$2>>2]|0;
  _sqlite3PcacheMakeClean($73);
 }
 $74 = HEAP32[$pPager>>2]|0;
 $75 = HEAP32[$rc>>2]|0;
 $76 = (_pager_error($74,$75)|0);
 HEAP32[$0>>2] = $76;
 $77 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($77|0);
}
function _subjRequiresPage($pPg) {
 $pPg = $pPg|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $i = 0, $p = 0, $pPager = 0, $pgno = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 20|0;
 $1 = sp + 16|0;
 $pPager = sp + 12|0;
 $p = sp + 8|0;
 $pgno = sp + 4|0;
 $i = sp;
 HEAP32[$1>>2] = $pPg;
 $2 = HEAP32[$1>>2]|0;
 $3 = (($2) + 16|0);
 $4 = HEAP32[$3>>2]|0;
 HEAP32[$pPager>>2] = $4;
 $5 = HEAP32[$1>>2]|0;
 $6 = (($5) + 20|0);
 $7 = HEAP32[$6>>2]|0;
 HEAP32[$pgno>>2] = $7;
 HEAP32[$i>>2] = 0;
 while(1) {
  $8 = HEAP32[$i>>2]|0;
  $9 = HEAP32[$pPager>>2]|0;
  $10 = (($9) + 96|0);
  $11 = HEAP32[$10>>2]|0;
  $12 = ($8|0)<($11|0);
  if (!($12)) {
   label = 7;
   break;
  }
  $13 = HEAP32[$i>>2]|0;
  $14 = HEAP32[$pPager>>2]|0;
  $15 = (($14) + 92|0);
  $16 = HEAP32[$15>>2]|0;
  $17 = (($16) + (($13*48)|0)|0);
  HEAP32[$p>>2] = $17;
  $18 = HEAP32[$p>>2]|0;
  $19 = (($18) + 20|0);
  $20 = HEAP32[$19>>2]|0;
  $21 = HEAP32[$pgno>>2]|0;
  $22 = ($20>>>0)>=($21>>>0);
  if ($22) {
   $23 = HEAP32[$p>>2]|0;
   $24 = (($23) + 16|0);
   $25 = HEAP32[$24>>2]|0;
   $26 = HEAP32[$pgno>>2]|0;
   $27 = (_sqlite3BitvecTest($25,$26)|0);
   $28 = (0)==($27|0);
   if ($28) {
    label = 5;
    break;
   }
  }
  $29 = HEAP32[$i>>2]|0;
  $30 = (($29) + 1)|0;
  HEAP32[$i>>2] = $30;
 }
 if ((label|0) == 5) {
  HEAP32[$0>>2] = 1;
  $31 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($31|0);
 }
 else if ((label|0) == 7) {
  HEAP32[$0>>2] = 0;
  $31 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($31|0);
 }
 return 0|0;
}
function _subjournalPage($pPg) {
 $pPg = $pPg|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $8 = 0, $9 = 0, $offset = 0, $pData = 0, $pData2 = 0, $pPager = 0, $rc = 0;
 var label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 24|0;
 $rc = sp + 20|0;
 $pPager = sp + 16|0;
 $pData = sp + 12|0;
 $offset = sp;
 $pData2 = sp + 8|0;
 HEAP32[$0>>2] = $pPg;
 HEAP32[$rc>>2] = 0;
 $1 = HEAP32[$0>>2]|0;
 $2 = (($1) + 16|0);
 $3 = HEAP32[$2>>2]|0;
 HEAP32[$pPager>>2] = $3;
 $4 = HEAP32[$pPager>>2]|0;
 $5 = (($4) + 5|0);
 $6 = HEAP8[$5>>0]|0;
 $7 = $6&255;
 $8 = ($7|0)!=(2);
 if ($8) {
  $9 = HEAP32[$pPager>>2]|0;
  $10 = (_openSubJournal($9)|0);
  HEAP32[$rc>>2] = $10;
  $11 = HEAP32[$rc>>2]|0;
  $12 = ($11|0)==(0);
  if ($12) {
   $13 = HEAP32[$0>>2]|0;
   $14 = (($13) + 4|0);
   $15 = HEAP32[$14>>2]|0;
   HEAP32[$pData>>2] = $15;
   $16 = HEAP32[$pPager>>2]|0;
   $17 = (($16) + 52|0);
   $18 = HEAP32[$17>>2]|0;
   $19 = HEAP32[$pPager>>2]|0;
   $20 = (($19) + 152|0);
   $21 = HEAP32[$20>>2]|0;
   $22 = (4 + ($21))|0;
   $23 = ($22|0)<(0);
   $24 = $23 << 31 >> 31;
   $25 = (___muldi3(($18|0),0,($22|0),($24|0))|0);
   $26 = tempRet0;
   $27 = $offset;
   $28 = $27;
   HEAP32[$28>>2] = $25;
   $29 = (($27) + 4)|0;
   $30 = $29;
   HEAP32[$30>>2] = $26;
   $31 = HEAP32[$pData>>2]|0;
   HEAP32[$pData2>>2] = $31;
   $32 = HEAP32[$pPager>>2]|0;
   $33 = (($32) + 68|0);
   $34 = HEAP32[$33>>2]|0;
   $35 = $offset;
   $36 = $35;
   $37 = HEAP32[$36>>2]|0;
   $38 = (($35) + 4)|0;
   $39 = $38;
   $40 = HEAP32[$39>>2]|0;
   $41 = HEAP32[$0>>2]|0;
   $42 = (($41) + 20|0);
   $43 = HEAP32[$42>>2]|0;
   $44 = (_write32bits($34,$37,$40,$43)|0);
   HEAP32[$rc>>2] = $44;
   $45 = HEAP32[$rc>>2]|0;
   $46 = ($45|0)==(0);
   if ($46) {
    $47 = HEAP32[$pPager>>2]|0;
    $48 = (($47) + 68|0);
    $49 = HEAP32[$48>>2]|0;
    $50 = HEAP32[$pData2>>2]|0;
    $51 = HEAP32[$pPager>>2]|0;
    $52 = (($51) + 152|0);
    $53 = HEAP32[$52>>2]|0;
    $54 = $offset;
    $55 = $54;
    $56 = HEAP32[$55>>2]|0;
    $57 = (($54) + 4)|0;
    $58 = $57;
    $59 = HEAP32[$58>>2]|0;
    $60 = (_i64Add(($56|0),($59|0),4,0)|0);
    $61 = tempRet0;
    $62 = (_sqlite3OsWrite($49,$50,$53,$60,$61)|0);
    HEAP32[$rc>>2] = $62;
   }
  }
 }
 $63 = HEAP32[$rc>>2]|0;
 $64 = ($63|0)==(0);
 if (!($64)) {
  $74 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($74|0);
 }
 $65 = HEAP32[$pPager>>2]|0;
 $66 = (($65) + 52|0);
 $67 = HEAP32[$66>>2]|0;
 $68 = (($67) + 1)|0;
 HEAP32[$66>>2] = $68;
 $69 = HEAP32[$pPager>>2]|0;
 $70 = HEAP32[$0>>2]|0;
 $71 = (($70) + 20|0);
 $72 = HEAP32[$71>>2]|0;
 $73 = (_addToSavepointBitvecs($69,$72)|0);
 HEAP32[$rc>>2] = $73;
 $74 = HEAP32[$rc>>2]|0;
 STACKTOP = sp;return ($74|0);
}
function _pagerWalFrames($pPager,$pList,$nTruncate,$isCommit) {
 $pPager = $pPager|0;
 $pList = $pList|0;
 $nTruncate = $nTruncate|0;
 $isCommit = $isCommit|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $7 = 0, $8 = 0, $9 = 0, $nList = 0, $p = 0, $p1 = 0, $ppNext = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $0 = sp + 32|0;
 $1 = sp + 28|0;
 $2 = sp + 24|0;
 $3 = sp + 20|0;
 $rc = sp + 16|0;
 $nList = sp + 12|0;
 $p = sp + 8|0;
 $ppNext = sp + 4|0;
 $p1 = sp;
 HEAP32[$0>>2] = $pPager;
 HEAP32[$1>>2] = $pList;
 HEAP32[$2>>2] = $nTruncate;
 HEAP32[$3>>2] = $isCommit;
 $4 = HEAP32[$3>>2]|0;
 $5 = ($4|0)!=(0);
 L1: do {
  if ($5) {
   HEAP32[$ppNext>>2] = $1;
   HEAP32[$nList>>2] = 0;
   $6 = HEAP32[$1>>2]|0;
   HEAP32[$p>>2] = $6;
   while(1) {
    $7 = HEAP32[$p>>2]|0;
    $8 = HEAP32[$ppNext>>2]|0;
    HEAP32[$8>>2] = $7;
    $9 = ($7|0)!=(0|0);
    if (!($9)) {
     break L1;
    }
    $10 = HEAP32[$p>>2]|0;
    $11 = (($10) + 20|0);
    $12 = HEAP32[$11>>2]|0;
    $13 = HEAP32[$2>>2]|0;
    $14 = ($12>>>0)<=($13>>>0);
    if ($14) {
     $15 = HEAP32[$p>>2]|0;
     $16 = (($15) + 12|0);
     HEAP32[$ppNext>>2] = $16;
     $17 = HEAP32[$nList>>2]|0;
     $18 = (($17) + 1)|0;
     HEAP32[$nList>>2] = $18;
    }
    $19 = HEAP32[$p>>2]|0;
    $20 = (($19) + 12|0);
    $21 = HEAP32[$20>>2]|0;
    HEAP32[$p>>2] = $21;
   }
  } else {
   HEAP32[$nList>>2] = 1;
  }
 } while(0);
 $22 = HEAP32[$nList>>2]|0;
 $23 = HEAP32[$0>>2]|0;
 $24 = (($23) + 184|0);
 $25 = (($24) + 8|0);
 $26 = HEAP32[$25>>2]|0;
 $27 = (($26) + ($22))|0;
 HEAP32[$25>>2] = $27;
 $28 = HEAP32[$1>>2]|0;
 $29 = (($28) + 20|0);
 $30 = HEAP32[$29>>2]|0;
 $31 = ($30|0)==(1);
 if ($31) {
  $32 = HEAP32[$1>>2]|0;
  _pager_write_changecounter($32);
 }
 $33 = HEAP32[$0>>2]|0;
 $34 = (($33) + 208|0);
 $35 = HEAP32[$34>>2]|0;
 $36 = HEAP32[$0>>2]|0;
 $37 = (($36) + 152|0);
 $38 = HEAP32[$37>>2]|0;
 $39 = HEAP32[$1>>2]|0;
 $40 = HEAP32[$2>>2]|0;
 $41 = HEAP32[$3>>2]|0;
 $42 = HEAP32[$0>>2]|0;
 $43 = (($42) + 10|0);
 $44 = HEAP8[$43>>0]|0;
 $45 = $44&255;
 $46 = (_sqlite3WalFrames($35,$38,$39,$40,$41,$45)|0);
 HEAP32[$rc>>2] = $46;
 $47 = HEAP32[$rc>>2]|0;
 $48 = ($47|0)==(0);
 if (!($48)) {
  $68 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($68|0);
 }
 $49 = HEAP32[$0>>2]|0;
 $50 = (($49) + 88|0);
 $51 = HEAP32[$50>>2]|0;
 $52 = ($51|0)!=(0|0);
 if (!($52)) {
  $68 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($68|0);
 }
 $53 = HEAP32[$1>>2]|0;
 HEAP32[$p1>>2] = $53;
 while(1) {
  $54 = HEAP32[$p1>>2]|0;
  $55 = ($54|0)!=(0|0);
  if (!($55)) {
   break;
  }
  $56 = HEAP32[$0>>2]|0;
  $57 = (($56) + 88|0);
  $58 = HEAP32[$57>>2]|0;
  $59 = HEAP32[$p1>>2]|0;
  $60 = (($59) + 20|0);
  $61 = HEAP32[$60>>2]|0;
  $62 = HEAP32[$p1>>2]|0;
  $63 = (($62) + 4|0);
  $64 = HEAP32[$63>>2]|0;
  _sqlite3BackupUpdate($58,$61,$64);
  $65 = HEAP32[$p1>>2]|0;
  $66 = (($65) + 12|0);
  $67 = HEAP32[$66>>2]|0;
  HEAP32[$p1>>2] = $67;
 }
 $68 = HEAP32[$rc>>2]|0;
 STACKTOP = sp;return ($68|0);
}
function _syncJournal($pPager,$newHdr) {
 $pPager = $pPager|0;
 $newHdr = $newHdr|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0;
 var $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0;
 var $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0;
 var $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0;
 var $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0;
 var $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $aMagic = 0, $iDc = 0, $iNextHdrOffset = 0, $or$cond = 0, $rc = 0, $zHeader = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $0 = sp + 24|0;
 $1 = sp + 20|0;
 $2 = sp + 16|0;
 $rc = sp + 12|0;
 $iDc = sp + 8|0;
 $iNextHdrOffset = sp;
 $aMagic = sp + 40|0;
 $zHeader = sp + 28|0;
 HEAP32[$1>>2] = $pPager;
 HEAP32[$2>>2] = $newHdr;
 $3 = HEAP32[$1>>2]|0;
 $4 = (_sqlite3PagerExclusiveLock($3)|0);
 HEAP32[$rc>>2] = $4;
 $5 = HEAP32[$rc>>2]|0;
 $6 = ($5|0)!=(0);
 if ($6) {
  $7 = HEAP32[$rc>>2]|0;
  HEAP32[$0>>2] = $7;
  $169 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($169|0);
 }
 $8 = HEAP32[$1>>2]|0;
 $9 = (($8) + 7|0);
 $10 = HEAP8[$9>>0]|0;
 $11 = ($10<<24>>24)!=(0);
 do {
  if (!($11)) {
   $12 = HEAP32[$1>>2]|0;
   $13 = (($12) + 64|0);
   $14 = HEAP32[$13>>2]|0;
   $15 = HEAP32[$14>>2]|0;
   $16 = ($15|0)!=(0|0);
   if ($16) {
    $17 = HEAP32[$1>>2]|0;
    $18 = (($17) + 5|0);
    $19 = HEAP8[$18>>0]|0;
    $20 = $19&255;
    $21 = ($20|0)!=(4);
    if ($21) {
     $22 = HEAP32[$1>>2]|0;
     $23 = (($22) + 60|0);
     $24 = HEAP32[$23>>2]|0;
     $25 = (_sqlite3OsDeviceCharacteristics($24)|0);
     HEAP32[$iDc>>2] = $25;
     $26 = HEAP32[$iDc>>2]|0;
     $27 = $26 & 512;
     $28 = (0)==($27|0);
     if ($28) {
      ;HEAP8[$zHeader+0>>0]=HEAP8[4384+0>>0]|0;HEAP8[$zHeader+1>>0]=HEAP8[4384+1>>0]|0;HEAP8[$zHeader+2>>0]=HEAP8[4384+2>>0]|0;HEAP8[$zHeader+3>>0]=HEAP8[4384+3>>0]|0;HEAP8[$zHeader+4>>0]=HEAP8[4384+4>>0]|0;HEAP8[$zHeader+5>>0]=HEAP8[4384+5>>0]|0;HEAP8[$zHeader+6>>0]=HEAP8[4384+6>>0]|0;HEAP8[$zHeader+7>>0]=HEAP8[4384+7>>0]|0;
      $29 = (($zHeader) + 8|0);
      $30 = HEAP32[$1>>2]|0;
      $31 = (($30) + 44|0);
      $32 = HEAP32[$31>>2]|0;
      _sqlite3Put4byte($29,$32);
      $33 = HEAP32[$1>>2]|0;
      $34 = (_journalHdrOffset($33)|0);
      $35 = tempRet0;
      $36 = $iNextHdrOffset;
      $37 = $36;
      HEAP32[$37>>2] = $34;
      $38 = (($36) + 4)|0;
      $39 = $38;
      HEAP32[$39>>2] = $35;
      $40 = HEAP32[$1>>2]|0;
      $41 = (($40) + 64|0);
      $42 = HEAP32[$41>>2]|0;
      $43 = $iNextHdrOffset;
      $44 = $43;
      $45 = HEAP32[$44>>2]|0;
      $46 = (($43) + 4)|0;
      $47 = $46;
      $48 = HEAP32[$47>>2]|0;
      $49 = (_sqlite3OsRead($42,$aMagic,8,$45,$48)|0);
      HEAP32[$rc>>2] = $49;
      $50 = HEAP32[$rc>>2]|0;
      $51 = ($50|0)==(0);
      if ($51) {
       $52 = (_memcmp($aMagic,4384,8)|0);
       $53 = (0)==($52|0);
       if ($53) {
        $54 = HEAP32[$1>>2]|0;
        $55 = (($54) + 64|0);
        $56 = HEAP32[$55>>2]|0;
        $57 = $iNextHdrOffset;
        $58 = $57;
        $59 = HEAP32[$58>>2]|0;
        $60 = (($57) + 4)|0;
        $61 = $60;
        $62 = HEAP32[$61>>2]|0;
        $63 = (_sqlite3OsWrite($56,4600,1,$59,$62)|0);
        HEAP32[$rc>>2] = $63;
       }
      }
      $64 = HEAP32[$rc>>2]|0;
      $65 = ($64|0)!=(0);
      $66 = HEAP32[$rc>>2]|0;
      $67 = ($66|0)!=(522);
      $or$cond = $65 & $67;
      if ($or$cond) {
       $68 = HEAP32[$rc>>2]|0;
       HEAP32[$0>>2] = $68;
       $169 = HEAP32[$0>>2]|0;
       STACKTOP = sp;return ($169|0);
      }
      $69 = HEAP32[$1>>2]|0;
      $70 = (($69) + 8|0);
      $71 = HEAP8[$70>>0]|0;
      $72 = $71&255;
      $73 = ($72|0)!=(0);
      if ($73) {
       $74 = HEAP32[$iDc>>2]|0;
       $75 = $74 & 1024;
       $76 = (0)==($75|0);
       if ($76) {
        $77 = HEAP32[$1>>2]|0;
        $78 = (($77) + 64|0);
        $79 = HEAP32[$78>>2]|0;
        $80 = HEAP32[$1>>2]|0;
        $81 = (($80) + 11|0);
        $82 = HEAP8[$81>>0]|0;
        $83 = $82&255;
        $84 = (_sqlite3OsSync($79,$83)|0);
        HEAP32[$rc>>2] = $84;
        $85 = HEAP32[$rc>>2]|0;
        $86 = ($85|0)!=(0);
        if ($86) {
         $87 = HEAP32[$rc>>2]|0;
         HEAP32[$0>>2] = $87;
         $169 = HEAP32[$0>>2]|0;
         STACKTOP = sp;return ($169|0);
        }
       }
      }
      $88 = HEAP32[$1>>2]|0;
      $89 = (($88) + 64|0);
      $90 = HEAP32[$89>>2]|0;
      $91 = HEAP32[$1>>2]|0;
      $92 = (($91) + 80|0);
      $93 = $92;
      $94 = $93;
      $95 = HEAP32[$94>>2]|0;
      $96 = (($93) + 4)|0;
      $97 = $96;
      $98 = HEAP32[$97>>2]|0;
      $99 = (_sqlite3OsWrite($90,$zHeader,12,$95,$98)|0);
      HEAP32[$rc>>2] = $99;
      $100 = HEAP32[$rc>>2]|0;
      $101 = ($100|0)!=(0);
      if ($101) {
       $102 = HEAP32[$rc>>2]|0;
       HEAP32[$0>>2] = $102;
       $169 = HEAP32[$0>>2]|0;
       STACKTOP = sp;return ($169|0);
      }
     }
     $103 = HEAP32[$iDc>>2]|0;
     $104 = $103 & 1024;
     $105 = (0)==($104|0);
     if ($105) {
      $106 = HEAP32[$1>>2]|0;
      $107 = (($106) + 64|0);
      $108 = HEAP32[$107>>2]|0;
      $109 = HEAP32[$1>>2]|0;
      $110 = (($109) + 11|0);
      $111 = HEAP8[$110>>0]|0;
      $112 = $111&255;
      $113 = HEAP32[$1>>2]|0;
      $114 = (($113) + 11|0);
      $115 = HEAP8[$114>>0]|0;
      $116 = $115&255;
      $117 = ($116|0)==(3);
      $118 = $117 ? 16 : 0;
      $119 = $112 | $118;
      $120 = (_sqlite3OsSync($108,$119)|0);
      HEAP32[$rc>>2] = $120;
      $121 = HEAP32[$rc>>2]|0;
      $122 = ($121|0)!=(0);
      if ($122) {
       $123 = HEAP32[$rc>>2]|0;
       HEAP32[$0>>2] = $123;
       $169 = HEAP32[$0>>2]|0;
       STACKTOP = sp;return ($169|0);
      }
     }
     $124 = HEAP32[$1>>2]|0;
     $125 = (($124) + 72|0);
     $126 = $125;
     $127 = $126;
     $128 = HEAP32[$127>>2]|0;
     $129 = (($126) + 4)|0;
     $130 = $129;
     $131 = HEAP32[$130>>2]|0;
     $132 = HEAP32[$1>>2]|0;
     $133 = (($132) + 80|0);
     $134 = $133;
     $135 = $134;
     HEAP32[$135>>2] = $128;
     $136 = (($134) + 4)|0;
     $137 = $136;
     HEAP32[$137>>2] = $131;
     $138 = HEAP32[$2>>2]|0;
     $139 = ($138|0)!=(0);
     if (!($139)) {
      break;
     }
     $140 = HEAP32[$iDc>>2]|0;
     $141 = $140 & 512;
     $142 = (0)==($141|0);
     if (!($142)) {
      break;
     }
     $143 = HEAP32[$1>>2]|0;
     $144 = (($143) + 44|0);
     HEAP32[$144>>2] = 0;
     $145 = HEAP32[$1>>2]|0;
     $146 = (_writeJournalHdr($145)|0);
     HEAP32[$rc>>2] = $146;
     $147 = HEAP32[$rc>>2]|0;
     $148 = ($147|0)!=(0);
     if (!($148)) {
      break;
     }
     $149 = HEAP32[$rc>>2]|0;
     HEAP32[$0>>2] = $149;
     $169 = HEAP32[$0>>2]|0;
     STACKTOP = sp;return ($169|0);
    }
   }
   $150 = HEAP32[$1>>2]|0;
   $151 = (($150) + 72|0);
   $152 = $151;
   $153 = $152;
   $154 = HEAP32[$153>>2]|0;
   $155 = (($152) + 4)|0;
   $156 = $155;
   $157 = HEAP32[$156>>2]|0;
   $158 = HEAP32[$1>>2]|0;
   $159 = (($158) + 80|0);
   $160 = $159;
   $161 = $160;
   HEAP32[$161>>2] = $154;
   $162 = (($160) + 4)|0;
   $163 = $162;
   HEAP32[$163>>2] = $157;
  }
 } while(0);
 $164 = HEAP32[$1>>2]|0;
 $165 = (($164) + 204|0);
 $166 = HEAP32[$165>>2]|0;
 _sqlite3PcacheClearSyncFlags($166);
 $167 = HEAP32[$1>>2]|0;
 $168 = (($167) + 16|0);
 HEAP8[$168>>0] = 4;
 HEAP32[$0>>2] = 0;
 $169 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($169|0);
}
function _pager_write_pagelist($pPager,$pList) {
 $pPager = $pPager|0;
 $pList = $pList|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0;
 var $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0;
 var $offset = 0, $or$cond = 0, $pData = 0, $pgno = 0, $rc = 0, $szFile = 0, dest = 0, label = 0, sp = 0, src = 0, stop = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $0 = sp + 32|0;
 $1 = sp + 28|0;
 $rc = sp + 24|0;
 $szFile = sp + 8|0;
 $pgno = sp + 20|0;
 $offset = sp;
 $pData = sp + 16|0;
 HEAP32[$0>>2] = $pPager;
 HEAP32[$1>>2] = $pList;
 HEAP32[$rc>>2] = 0;
 $2 = HEAP32[$0>>2]|0;
 $3 = (($2) + 60|0);
 $4 = HEAP32[$3>>2]|0;
 $5 = HEAP32[$4>>2]|0;
 $6 = ($5|0)!=(0|0);
 if (!($6)) {
  $7 = HEAP32[$0>>2]|0;
  $8 = HEAP32[$0>>2]|0;
  $9 = (($8) + 60|0);
  $10 = HEAP32[$9>>2]|0;
  $11 = HEAP32[$0>>2]|0;
  $12 = (($11) + 144|0);
  $13 = HEAP32[$12>>2]|0;
  $14 = (_pagerOpentemp($7,$10,$13)|0);
  HEAP32[$rc>>2] = $14;
 }
 $15 = HEAP32[$rc>>2]|0;
 $16 = ($15|0)==(0);
 do {
  if ($16) {
   $17 = HEAP32[$0>>2]|0;
   $18 = (($17) + 36|0);
   $19 = HEAP32[$18>>2]|0;
   $20 = HEAP32[$0>>2]|0;
   $21 = (($20) + 24|0);
   $22 = HEAP32[$21>>2]|0;
   $23 = ($19>>>0)<($22>>>0);
   if ($23) {
    $24 = HEAP32[$1>>2]|0;
    $25 = (($24) + 12|0);
    $26 = HEAP32[$25>>2]|0;
    $27 = ($26|0)!=(0|0);
    if (!($27)) {
     $28 = HEAP32[$1>>2]|0;
     $29 = (($28) + 20|0);
     $30 = HEAP32[$29>>2]|0;
     $31 = HEAP32[$0>>2]|0;
     $32 = (($31) + 36|0);
     $33 = HEAP32[$32>>2]|0;
     $34 = ($30>>>0)>($33>>>0);
     if (!($34)) {
      break;
     }
    }
    $35 = HEAP32[$0>>2]|0;
    $36 = (($35) + 152|0);
    $37 = HEAP32[$36>>2]|0;
    $38 = ($37|0)<(0);
    $39 = $38 << 31 >> 31;
    $40 = HEAP32[$0>>2]|0;
    $41 = (($40) + 24|0);
    $42 = HEAP32[$41>>2]|0;
    $43 = (___muldi3(($37|0),($39|0),($42|0),0)|0);
    $44 = tempRet0;
    $45 = $szFile;
    $46 = $45;
    HEAP32[$46>>2] = $43;
    $47 = (($45) + 4)|0;
    $48 = $47;
    HEAP32[$48>>2] = $44;
    $49 = HEAP32[$0>>2]|0;
    $50 = (($49) + 60|0);
    $51 = HEAP32[$50>>2]|0;
    _sqlite3OsFileControlHint($51,5,$szFile);
    $52 = HEAP32[$0>>2]|0;
    $53 = (($52) + 24|0);
    $54 = HEAP32[$53>>2]|0;
    $55 = HEAP32[$0>>2]|0;
    $56 = (($55) + 36|0);
    HEAP32[$56>>2] = $54;
   }
  }
 } while(0);
 while(1) {
  $57 = HEAP32[$rc>>2]|0;
  $58 = ($57|0)==(0);
  $59 = HEAP32[$1>>2]|0;
  $60 = ($59|0)!=(0|0);
  $or$cond = $58 & $60;
  if (!($or$cond)) {
   break;
  }
  $61 = HEAP32[$1>>2]|0;
  $62 = (($61) + 20|0);
  $63 = HEAP32[$62>>2]|0;
  HEAP32[$pgno>>2] = $63;
  $64 = HEAP32[$pgno>>2]|0;
  $65 = HEAP32[$0>>2]|0;
  $66 = (($65) + 24|0);
  $67 = HEAP32[$66>>2]|0;
  $68 = ($64>>>0)<=($67>>>0);
  if ($68) {
   $69 = HEAP32[$1>>2]|0;
   $70 = (($69) + 24|0);
   $71 = HEAP16[$70>>1]|0;
   $72 = $71&65535;
   $73 = $72 & 32;
   $74 = (0)==($73|0);
   if ($74) {
    $75 = HEAP32[$pgno>>2]|0;
    $76 = (($75) - 1)|0;
    $77 = HEAP32[$0>>2]|0;
    $78 = (($77) + 152|0);
    $79 = HEAP32[$78>>2]|0;
    $80 = ($79|0)<(0);
    $81 = $80 << 31 >> 31;
    $82 = (___muldi3(($76|0),0,($79|0),($81|0))|0);
    $83 = tempRet0;
    $84 = $offset;
    $85 = $84;
    HEAP32[$85>>2] = $82;
    $86 = (($84) + 4)|0;
    $87 = $86;
    HEAP32[$87>>2] = $83;
    $88 = HEAP32[$1>>2]|0;
    $89 = (($88) + 20|0);
    $90 = HEAP32[$89>>2]|0;
    $91 = ($90|0)==(1);
    if ($91) {
     $92 = HEAP32[$1>>2]|0;
     _pager_write_changecounter($92);
    }
    $93 = HEAP32[$1>>2]|0;
    $94 = (($93) + 4|0);
    $95 = HEAP32[$94>>2]|0;
    HEAP32[$pData>>2] = $95;
    $96 = HEAP32[$0>>2]|0;
    $97 = (($96) + 60|0);
    $98 = HEAP32[$97>>2]|0;
    $99 = HEAP32[$pData>>2]|0;
    $100 = HEAP32[$0>>2]|0;
    $101 = (($100) + 152|0);
    $102 = HEAP32[$101>>2]|0;
    $103 = $offset;
    $104 = $103;
    $105 = HEAP32[$104>>2]|0;
    $106 = (($103) + 4)|0;
    $107 = $106;
    $108 = HEAP32[$107>>2]|0;
    $109 = (_sqlite3OsWrite($98,$99,$102,$105,$108)|0);
    HEAP32[$rc>>2] = $109;
    $110 = HEAP32[$pgno>>2]|0;
    $111 = ($110|0)==(1);
    if ($111) {
     $112 = HEAP32[$0>>2]|0;
     $113 = (($112) + 100|0);
     $114 = HEAP32[$pData>>2]|0;
     $115 = (($114) + 24|0);
     dest=$113+0|0; src=$115+0|0; stop=dest+16|0; do { HEAP8[dest>>0]=HEAP8[src>>0]|0; dest=dest+1|0; src=src+1|0; } while ((dest|0) < (stop|0));
    }
    $116 = HEAP32[$pgno>>2]|0;
    $117 = HEAP32[$0>>2]|0;
    $118 = (($117) + 32|0);
    $119 = HEAP32[$118>>2]|0;
    $120 = ($116>>>0)>($119>>>0);
    if ($120) {
     $121 = HEAP32[$pgno>>2]|0;
     $122 = HEAP32[$0>>2]|0;
     $123 = (($122) + 32|0);
     HEAP32[$123>>2] = $121;
    }
    $124 = HEAP32[$0>>2]|0;
    $125 = (($124) + 184|0);
    $126 = (($125) + 8|0);
    $127 = HEAP32[$126>>2]|0;
    $128 = (($127) + 1)|0;
    HEAP32[$126>>2] = $128;
    $129 = HEAP32[$0>>2]|0;
    $130 = (($129) + 88|0);
    $131 = HEAP32[$130>>2]|0;
    $132 = HEAP32[$pgno>>2]|0;
    $133 = HEAP32[$1>>2]|0;
    $134 = (($133) + 4|0);
    $135 = HEAP32[$134>>2]|0;
    _sqlite3BackupUpdate($131,$132,$135);
   }
  }
  $136 = HEAP32[$1>>2]|0;
  $137 = (($136) + 12|0);
  $138 = HEAP32[$137>>2]|0;
  HEAP32[$1>>2] = $138;
 }
 $139 = HEAP32[$rc>>2]|0;
 STACKTOP = sp;return ($139|0);
}
function _pagerOpentemp($pPager,$pFile,$vfsFlags) {
 $pPager = $pPager|0;
 $pFile = $pFile|0;
 $vfsFlags = $vfsFlags|0;
 var $0 = 0, $1 = 0, $10 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $1 = sp + 8|0;
 $2 = sp + 4|0;
 $rc = sp;
 HEAP32[$0>>2] = $pPager;
 HEAP32[$1>>2] = $pFile;
 HEAP32[$2>>2] = $vfsFlags;
 $3 = HEAP32[$2>>2]|0;
 $4 = $3 | 30;
 HEAP32[$2>>2] = $4;
 $5 = HEAP32[$0>>2]|0;
 $6 = HEAP32[$5>>2]|0;
 $7 = HEAP32[$1>>2]|0;
 $8 = HEAP32[$2>>2]|0;
 $9 = (_sqlite3OsOpen($6,0,$7,$8,0)|0);
 HEAP32[$rc>>2] = $9;
 $10 = HEAP32[$rc>>2]|0;
 STACKTOP = sp;return ($10|0);
}
function _pager_write_changecounter($pPg) {
 $pPg = $pPg|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0;
 var $9 = 0, $change_counter = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $change_counter = sp;
 HEAP32[$0>>2] = $pPg;
 $1 = HEAP32[$0>>2]|0;
 $2 = (($1) + 16|0);
 $3 = HEAP32[$2>>2]|0;
 $4 = (($3) + 100|0);
 $5 = (_sqlite3Get4byte($4)|0);
 $6 = (($5) + 1)|0;
 HEAP32[$change_counter>>2] = $6;
 $7 = HEAP32[$0>>2]|0;
 $8 = (($7) + 4|0);
 $9 = HEAP32[$8>>2]|0;
 $10 = (($9) + 24|0);
 $11 = HEAP32[$change_counter>>2]|0;
 _sqlite3Put4byte($10,$11);
 $12 = HEAP32[$0>>2]|0;
 $13 = (($12) + 4|0);
 $14 = HEAP32[$13>>2]|0;
 $15 = (($14) + 92|0);
 $16 = HEAP32[$change_counter>>2]|0;
 _sqlite3Put4byte($15,$16);
 $17 = HEAP32[$0>>2]|0;
 $18 = (($17) + 4|0);
 $19 = HEAP32[$18>>2]|0;
 $20 = (($19) + 96|0);
 _sqlite3Put4byte($20,3008007);
 STACKTOP = sp;return;
}
function _sqlite3Put4byte($p,$v) {
 $p = $p|0;
 $v = $v|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0;
 var label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $1 = sp;
 HEAP32[$0>>2] = $p;
 HEAP32[$1>>2] = $v;
 $2 = HEAP32[$1>>2]|0;
 $3 = $2 >>> 24;
 $4 = $3&255;
 $5 = HEAP32[$0>>2]|0;
 HEAP8[$5>>0] = $4;
 $6 = HEAP32[$1>>2]|0;
 $7 = $6 >>> 16;
 $8 = $7&255;
 $9 = HEAP32[$0>>2]|0;
 $10 = (($9) + 1|0);
 HEAP8[$10>>0] = $8;
 $11 = HEAP32[$1>>2]|0;
 $12 = $11 >>> 8;
 $13 = $12&255;
 $14 = HEAP32[$0>>2]|0;
 $15 = (($14) + 2|0);
 HEAP8[$15>>0] = $13;
 $16 = HEAP32[$1>>2]|0;
 $17 = $16&255;
 $18 = HEAP32[$0>>2]|0;
 $19 = (($18) + 3|0);
 HEAP8[$19>>0] = $17;
 STACKTOP = sp;return;
}
function _sqlite3PagerExclusiveLock($pPager) {
 $pPager = $pPager|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $rc = sp;
 HEAP32[$0>>2] = $pPager;
 HEAP32[$rc>>2] = 0;
 $1 = HEAP32[$0>>2]|0;
 $2 = (_pagerUseWal($1)|0);
 $3 = (0)==($2|0);
 if (!($3)) {
  $6 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($6|0);
 }
 $4 = HEAP32[$0>>2]|0;
 $5 = (_pager_wait_on_lock($4,4)|0);
 HEAP32[$rc>>2] = $5;
 $6 = HEAP32[$rc>>2]|0;
 STACKTOP = sp;return ($6|0);
}
function _writeJournalHdr($pPager) {
 $pPager = $pPager|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0;
 var $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0;
 var $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0;
 var $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0;
 var $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $ii = 0, $nHeader = 0, $nWrite = 0, $rc = 0;
 var $zHeader = 0, dest = 0, label = 0, sp = 0, stop = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 20|0;
 $rc = sp + 16|0;
 $zHeader = sp + 12|0;
 $nHeader = sp + 8|0;
 $nWrite = sp + 4|0;
 $ii = sp;
 HEAP32[$0>>2] = $pPager;
 HEAP32[$rc>>2] = 0;
 $1 = HEAP32[$0>>2]|0;
 $2 = (($1) + 200|0);
 $3 = HEAP32[$2>>2]|0;
 HEAP32[$zHeader>>2] = $3;
 $4 = HEAP32[$0>>2]|0;
 $5 = (($4) + 152|0);
 $6 = HEAP32[$5>>2]|0;
 HEAP32[$nHeader>>2] = $6;
 $7 = HEAP32[$nHeader>>2]|0;
 $8 = HEAP32[$0>>2]|0;
 $9 = (($8) + 148|0);
 $10 = HEAP32[$9>>2]|0;
 $11 = ($7>>>0)>($10>>>0);
 if ($11) {
  $12 = HEAP32[$0>>2]|0;
  $13 = (($12) + 148|0);
  $14 = HEAP32[$13>>2]|0;
  HEAP32[$nHeader>>2] = $14;
 }
 HEAP32[$ii>>2] = 0;
 while(1) {
  $15 = HEAP32[$ii>>2]|0;
  $16 = HEAP32[$0>>2]|0;
  $17 = (($16) + 96|0);
  $18 = HEAP32[$17>>2]|0;
  $19 = ($15|0)<($18|0);
  if (!($19)) {
   break;
  }
  $20 = HEAP32[$ii>>2]|0;
  $21 = HEAP32[$0>>2]|0;
  $22 = (($21) + 92|0);
  $23 = HEAP32[$22>>2]|0;
  $24 = (($23) + (($20*48)|0)|0);
  $25 = (($24) + 8|0);
  $26 = $25;
  $27 = $26;
  $28 = HEAP32[$27>>2]|0;
  $29 = (($26) + 4)|0;
  $30 = $29;
  $31 = HEAP32[$30>>2]|0;
  $32 = ($28|0)==(0);
  $33 = ($31|0)==(0);
  $34 = $32 & $33;
  if ($34) {
   $35 = HEAP32[$0>>2]|0;
   $36 = (($35) + 72|0);
   $37 = $36;
   $38 = $37;
   $39 = HEAP32[$38>>2]|0;
   $40 = (($37) + 4)|0;
   $41 = $40;
   $42 = HEAP32[$41>>2]|0;
   $43 = HEAP32[$ii>>2]|0;
   $44 = HEAP32[$0>>2]|0;
   $45 = (($44) + 92|0);
   $46 = HEAP32[$45>>2]|0;
   $47 = (($46) + (($43*48)|0)|0);
   $48 = (($47) + 8|0);
   $49 = $48;
   $50 = $49;
   HEAP32[$50>>2] = $39;
   $51 = (($49) + 4)|0;
   $52 = $51;
   HEAP32[$52>>2] = $42;
  }
  $53 = HEAP32[$ii>>2]|0;
  $54 = (($53) + 1)|0;
  HEAP32[$ii>>2] = $54;
 }
 $55 = HEAP32[$0>>2]|0;
 $56 = (_journalHdrOffset($55)|0);
 $57 = tempRet0;
 $58 = HEAP32[$0>>2]|0;
 $59 = (($58) + 72|0);
 $60 = $59;
 $61 = $60;
 HEAP32[$61>>2] = $56;
 $62 = (($60) + 4)|0;
 $63 = $62;
 HEAP32[$63>>2] = $57;
 $64 = HEAP32[$0>>2]|0;
 $65 = (($64) + 80|0);
 $66 = $65;
 $67 = $66;
 HEAP32[$67>>2] = $56;
 $68 = (($66) + 4)|0;
 $69 = $68;
 HEAP32[$69>>2] = $57;
 $70 = HEAP32[$0>>2]|0;
 $71 = (($70) + 7|0);
 $72 = HEAP8[$71>>0]|0;
 $73 = $72&255;
 $74 = ($73|0)!=(0);
 if ($74) {
  label = 11;
 } else {
  $75 = HEAP32[$0>>2]|0;
  $76 = (($75) + 5|0);
  $77 = HEAP8[$76>>0]|0;
  $78 = $77&255;
  $79 = ($78|0)==(4);
  if ($79) {
   label = 11;
  } else {
   $80 = HEAP32[$0>>2]|0;
   $81 = (($80) + 60|0);
   $82 = HEAP32[$81>>2]|0;
   $83 = (_sqlite3OsDeviceCharacteristics($82)|0);
   $84 = $83 & 512;
   $85 = ($84|0)!=(0);
   if ($85) {
    label = 11;
   } else {
    $89 = HEAP32[$zHeader>>2]|0;
    dest=$89+0|0; stop=dest+12|0; do { HEAP8[dest>>0]=0|0; dest=dest+1|0; } while ((dest|0) < (stop|0));
   }
  }
 }
 if ((label|0) == 11) {
  $86 = HEAP32[$zHeader>>2]|0;
  ;HEAP8[$86+0>>0]=HEAP8[4384+0>>0]|0;HEAP8[$86+1>>0]=HEAP8[4384+1>>0]|0;HEAP8[$86+2>>0]=HEAP8[4384+2>>0]|0;HEAP8[$86+3>>0]=HEAP8[4384+3>>0]|0;HEAP8[$86+4>>0]=HEAP8[4384+4>>0]|0;HEAP8[$86+5>>0]=HEAP8[4384+5>>0]|0;HEAP8[$86+6>>0]=HEAP8[4384+6>>0]|0;HEAP8[$86+7>>0]=HEAP8[4384+7>>0]|0;
  $87 = HEAP32[$zHeader>>2]|0;
  $88 = (($87) + 8|0);
  _sqlite3Put4byte($88,-1);
 }
 $90 = HEAP32[$0>>2]|0;
 $91 = (($90) + 48|0);
 _sqlite3_randomness(4,$91);
 $92 = HEAP32[$zHeader>>2]|0;
 $93 = (($92) + 12|0);
 $94 = HEAP32[$0>>2]|0;
 $95 = (($94) + 48|0);
 $96 = HEAP32[$95>>2]|0;
 _sqlite3Put4byte($93,$96);
 $97 = HEAP32[$zHeader>>2]|0;
 $98 = (($97) + 16|0);
 $99 = HEAP32[$0>>2]|0;
 $100 = (($99) + 28|0);
 $101 = HEAP32[$100>>2]|0;
 _sqlite3Put4byte($98,$101);
 $102 = HEAP32[$zHeader>>2]|0;
 $103 = (($102) + 20|0);
 $104 = HEAP32[$0>>2]|0;
 $105 = (($104) + 148|0);
 $106 = HEAP32[$105>>2]|0;
 _sqlite3Put4byte($103,$106);
 $107 = HEAP32[$zHeader>>2]|0;
 $108 = (($107) + 24|0);
 $109 = HEAP32[$0>>2]|0;
 $110 = (($109) + 152|0);
 $111 = HEAP32[$110>>2]|0;
 _sqlite3Put4byte($108,$111);
 $112 = HEAP32[$zHeader>>2]|0;
 $113 = (($112) + 28|0);
 $114 = HEAP32[$nHeader>>2]|0;
 $115 = (($114) - 28)|0;
 _memset(($113|0),0,($115|0))|0;
 HEAP32[$nWrite>>2] = 0;
 while(1) {
  $116 = HEAP32[$rc>>2]|0;
  $117 = ($116|0)==(0);
  if (!($117)) {
   label = 17;
   break;
  }
  $118 = HEAP32[$nWrite>>2]|0;
  $119 = HEAP32[$0>>2]|0;
  $120 = (($119) + 148|0);
  $121 = HEAP32[$120>>2]|0;
  $122 = ($118>>>0)<($121>>>0);
  if (!($122)) {
   label = 17;
   break;
  }
  $123 = HEAP32[$0>>2]|0;
  $124 = (($123) + 64|0);
  $125 = HEAP32[$124>>2]|0;
  $126 = HEAP32[$zHeader>>2]|0;
  $127 = HEAP32[$nHeader>>2]|0;
  $128 = HEAP32[$0>>2]|0;
  $129 = (($128) + 72|0);
  $130 = $129;
  $131 = $130;
  $132 = HEAP32[$131>>2]|0;
  $133 = (($130) + 4)|0;
  $134 = $133;
  $135 = HEAP32[$134>>2]|0;
  $136 = (_sqlite3OsWrite($125,$126,$127,$132,$135)|0);
  HEAP32[$rc>>2] = $136;
  $137 = HEAP32[$nHeader>>2]|0;
  $138 = HEAP32[$0>>2]|0;
  $139 = (($138) + 72|0);
  $140 = $139;
  $141 = $140;
  $142 = HEAP32[$141>>2]|0;
  $143 = (($140) + 4)|0;
  $144 = $143;
  $145 = HEAP32[$144>>2]|0;
  $146 = (_i64Add(($142|0),($145|0),($137|0),0)|0);
  $147 = tempRet0;
  $148 = $139;
  $149 = $148;
  HEAP32[$149>>2] = $146;
  $150 = (($148) + 4)|0;
  $151 = $150;
  HEAP32[$151>>2] = $147;
  $152 = HEAP32[$nHeader>>2]|0;
  $153 = HEAP32[$nWrite>>2]|0;
  $154 = (($153) + ($152))|0;
  HEAP32[$nWrite>>2] = $154;
 }
 if ((label|0) == 17) {
  $155 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($155|0);
 }
 return 0|0;
}
function _sqlite3PcacheClearSyncFlags($pCache) {
 $pCache = $pCache|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $p = 0;
 var label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $p = sp;
 HEAP32[$0>>2] = $pCache;
 $1 = HEAP32[$0>>2]|0;
 $2 = HEAP32[$1>>2]|0;
 HEAP32[$p>>2] = $2;
 while(1) {
  $3 = HEAP32[$p>>2]|0;
  $4 = ($3|0)!=(0|0);
  if (!($4)) {
   break;
  }
  $5 = HEAP32[$p>>2]|0;
  $6 = (($5) + 24|0);
  $7 = HEAP16[$6>>1]|0;
  $8 = $7&65535;
  $9 = $8 & -5;
  $10 = $9&65535;
  HEAP16[$6>>1] = $10;
  $11 = HEAP32[$p>>2]|0;
  $12 = (($11) + 32|0);
  $13 = HEAP32[$12>>2]|0;
  HEAP32[$p>>2] = $13;
 }
 $14 = HEAP32[$0>>2]|0;
 $15 = (($14) + 4|0);
 $16 = HEAP32[$15>>2]|0;
 $17 = HEAP32[$0>>2]|0;
 $18 = (($17) + 8|0);
 HEAP32[$18>>2] = $16;
 STACKTOP = sp;return;
}
function _pager_wait_on_lock($pPager,$locktype) {
 $pPager = $pPager|0;
 $locktype = $locktype|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $1 = sp + 4|0;
 $rc = sp;
 HEAP32[$0>>2] = $pPager;
 HEAP32[$1>>2] = $locktype;
 while(1) {
  $2 = HEAP32[$0>>2]|0;
  $3 = HEAP32[$1>>2]|0;
  $4 = (_pagerLockDb($2,$3)|0);
  HEAP32[$rc>>2] = $4;
  $5 = HEAP32[$rc>>2]|0;
  $6 = ($5|0)==(5);
  if (!($6)) {
   label = 4;
   break;
  }
  $7 = HEAP32[$0>>2]|0;
  $8 = (($7) + 176|0);
  $9 = HEAP32[$8>>2]|0;
  $10 = HEAP32[$0>>2]|0;
  $11 = (($10) + 180|0);
  $12 = HEAP32[$11>>2]|0;
  $13 = (FUNCTION_TABLE_ii[$9 & 31]($12)|0);
  $14 = ($13|0)!=(0);
  if (!($14)) {
   label = 4;
   break;
  }
 }
 if ((label|0) == 4) {
  $15 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($15|0);
 }
 return 0|0;
}
function _pagerLockDb($pPager,$eLock) {
 $pPager = $pPager|0;
 $eLock = $eLock|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $or$cond = 0, $rc = 0;
 var label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $1 = sp + 4|0;
 $rc = sp;
 HEAP32[$0>>2] = $pPager;
 HEAP32[$1>>2] = $eLock;
 HEAP32[$rc>>2] = 0;
 $2 = HEAP32[$0>>2]|0;
 $3 = (($2) + 17|0);
 $4 = HEAP8[$3>>0]|0;
 $5 = $4&255;
 $6 = HEAP32[$1>>2]|0;
 $7 = ($5|0)<($6|0);
 if (!($7)) {
  $8 = HEAP32[$0>>2]|0;
  $9 = (($8) + 17|0);
  $10 = HEAP8[$9>>0]|0;
  $11 = $10&255;
  $12 = ($11|0)==(5);
  if (!($12)) {
   $37 = HEAP32[$rc>>2]|0;
   STACKTOP = sp;return ($37|0);
  }
 }
 $13 = HEAP32[$0>>2]|0;
 $14 = (($13) + 13|0);
 $15 = HEAP8[$14>>0]|0;
 $16 = $15&255;
 $17 = ($16|0)!=(0);
 if ($17) {
  $23 = 0;
 } else {
  $18 = HEAP32[$0>>2]|0;
  $19 = (($18) + 60|0);
  $20 = HEAP32[$19>>2]|0;
  $21 = HEAP32[$1>>2]|0;
  $22 = (_sqlite3OsLock($20,$21)|0);
  $23 = $22;
 }
 HEAP32[$rc>>2] = $23;
 $24 = HEAP32[$rc>>2]|0;
 $25 = ($24|0)==(0);
 if (!($25)) {
  $37 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($37|0);
 }
 $26 = HEAP32[$0>>2]|0;
 $27 = (($26) + 17|0);
 $28 = HEAP8[$27>>0]|0;
 $29 = $28&255;
 $30 = ($29|0)!=(5);
 $31 = HEAP32[$1>>2]|0;
 $32 = ($31|0)==(4);
 $or$cond = $30 | $32;
 if (!($or$cond)) {
  $37 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($37|0);
 }
 $33 = HEAP32[$1>>2]|0;
 $34 = $33&255;
 $35 = HEAP32[$0>>2]|0;
 $36 = (($35) + 17|0);
 HEAP8[$36>>0] = $34;
 $37 = HEAP32[$rc>>2]|0;
 STACKTOP = sp;return ($37|0);
}
function _sqlite3WalFrames($pWal,$szPage,$pList,$nTruncate,$isCommit,$sync_flags) {
 $pWal = $pWal|0;
 $szPage = $szPage|0;
 $pList = $pList|0;
 $nTruncate = $nTruncate|0;
 $isCommit = $isCommit|0;
 $sync_flags = $sync_flags|0;
 var $$ = 0, $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0;
 var $115 = 0, $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0;
 var $133 = 0, $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0;
 var $151 = 0, $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0;
 var $17 = 0, $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0;
 var $188 = 0, $189 = 0, $19 = 0, $190 = 0, $191 = 0, $192 = 0, $193 = 0, $194 = 0, $195 = 0, $196 = 0, $197 = 0, $198 = 0, $199 = 0, $2 = 0, $20 = 0, $200 = 0, $201 = 0, $202 = 0, $203 = 0, $204 = 0;
 var $205 = 0, $206 = 0, $207 = 0, $208 = 0, $209 = 0, $21 = 0, $210 = 0, $211 = 0, $212 = 0, $213 = 0, $214 = 0, $215 = 0, $216 = 0, $217 = 0, $218 = 0, $219 = 0, $22 = 0, $220 = 0, $221 = 0, $222 = 0;
 var $223 = 0, $224 = 0, $225 = 0, $226 = 0, $227 = 0, $228 = 0, $229 = 0, $23 = 0, $230 = 0, $231 = 0, $232 = 0, $233 = 0, $234 = 0, $235 = 0, $236 = 0, $237 = 0, $238 = 0, $239 = 0, $24 = 0, $240 = 0;
 var $241 = 0, $242 = 0, $243 = 0, $244 = 0, $245 = 0, $246 = 0, $247 = 0, $248 = 0, $249 = 0, $25 = 0, $250 = 0, $251 = 0, $252 = 0, $253 = 0, $254 = 0, $255 = 0, $256 = 0, $257 = 0, $258 = 0, $259 = 0;
 var $26 = 0, $260 = 0, $261 = 0, $262 = 0, $263 = 0, $264 = 0, $265 = 0, $266 = 0, $267 = 0, $268 = 0, $269 = 0, $27 = 0, $270 = 0, $271 = 0, $272 = 0, $273 = 0, $274 = 0, $275 = 0, $276 = 0, $277 = 0;
 var $278 = 0, $279 = 0, $28 = 0, $280 = 0, $281 = 0, $282 = 0, $283 = 0, $284 = 0, $285 = 0, $286 = 0, $287 = 0, $288 = 0, $289 = 0, $29 = 0, $290 = 0, $291 = 0, $292 = 0, $293 = 0, $294 = 0, $295 = 0;
 var $296 = 0, $297 = 0, $298 = 0, $299 = 0, $3 = 0, $30 = 0, $300 = 0, $301 = 0, $302 = 0, $303 = 0, $304 = 0, $305 = 0, $306 = 0, $307 = 0, $308 = 0, $309 = 0, $31 = 0, $310 = 0, $311 = 0, $312 = 0;
 var $313 = 0, $314 = 0, $315 = 0, $316 = 0, $317 = 0, $318 = 0, $319 = 0, $32 = 0, $320 = 0, $321 = 0, $322 = 0, $323 = 0, $324 = 0, $325 = 0, $326 = 0, $327 = 0, $328 = 0, $329 = 0, $33 = 0, $330 = 0;
 var $331 = 0, $332 = 0, $333 = 0, $334 = 0, $335 = 0, $336 = 0, $337 = 0, $338 = 0, $339 = 0, $34 = 0, $340 = 0, $341 = 0, $342 = 0, $343 = 0, $344 = 0, $345 = 0, $346 = 0, $347 = 0, $348 = 0, $349 = 0;
 var $35 = 0, $350 = 0, $351 = 0, $352 = 0, $353 = 0, $354 = 0, $355 = 0, $356 = 0, $357 = 0, $358 = 0, $359 = 0, $36 = 0, $360 = 0, $361 = 0, $362 = 0, $363 = 0, $364 = 0, $365 = 0, $366 = 0, $367 = 0;
 var $368 = 0, $369 = 0, $37 = 0, $370 = 0, $371 = 0, $372 = 0, $373 = 0, $374 = 0, $375 = 0, $376 = 0, $377 = 0, $378 = 0, $379 = 0, $38 = 0, $380 = 0, $381 = 0, $382 = 0, $383 = 0, $384 = 0, $385 = 0;
 var $386 = 0, $387 = 0, $388 = 0, $389 = 0, $39 = 0, $390 = 0, $391 = 0, $392 = 0, $393 = 0, $394 = 0, $395 = 0, $396 = 0, $397 = 0, $398 = 0, $399 = 0, $4 = 0, $40 = 0, $400 = 0, $401 = 0, $402 = 0;
 var $403 = 0, $404 = 0, $405 = 0, $406 = 0, $407 = 0, $408 = 0, $409 = 0, $41 = 0, $410 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0;
 var $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0;
 var $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0;
 var $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $aCksum = 0, $aWalHdr = 0, $iFrame = 0, $iOffset = 0, $nDbSize = 0, $nExtra = 0, $or$cond = 0, $or$cond3 = 0;
 var $or$cond5 = 0, $p = 0, $pLast = 0, $rc = 0, $sectorSize = 0, $sz = 0, $szFrame = 0, $w = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 144|0;
 $0 = sp + 104|0;
 $1 = sp + 100|0;
 $2 = sp + 96|0;
 $3 = sp + 92|0;
 $4 = sp + 88|0;
 $5 = sp + 84|0;
 $6 = sp + 80|0;
 $rc = sp + 76|0;
 $iFrame = sp + 72|0;
 $p = sp + 68|0;
 $pLast = sp + 64|0;
 $nExtra = sp + 60|0;
 $szFrame = sp + 56|0;
 $iOffset = sp + 32|0;
 $w = sp + 8|0;
 $aWalHdr = sp + 112|0;
 $aCksum = sp + 48|0;
 $nDbSize = sp + 44|0;
 $sectorSize = sp + 40|0;
 $sz = sp;
 HEAP32[$1>>2] = $pWal;
 HEAP32[$2>>2] = $szPage;
 HEAP32[$3>>2] = $pList;
 HEAP32[$4>>2] = $nTruncate;
 HEAP32[$5>>2] = $isCommit;
 HEAP32[$6>>2] = $sync_flags;
 HEAP32[$pLast>>2] = 0;
 HEAP32[$nExtra>>2] = 0;
 $7 = HEAP32[$1>>2]|0;
 $8 = (_walRestartLog($7)|0);
 HEAP32[$rc>>2] = $8;
 $9 = (0)!=($8|0);
 if ($9) {
  $10 = HEAP32[$rc>>2]|0;
  HEAP32[$0>>2] = $10;
  $410 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($410|0);
 }
 $11 = HEAP32[$1>>2]|0;
 $12 = (($11) + 52|0);
 $13 = (($12) + 16|0);
 $14 = HEAP32[$13>>2]|0;
 HEAP32[$iFrame>>2] = $14;
 $15 = HEAP32[$iFrame>>2]|0;
 $16 = ($15|0)==(0);
 if ($16) {
  $17 = HEAP8[1136>>0]|0;
  $18 = $17 << 24 >> 24;
  $19 = ($18|0)==(0);
  $20 = $19&1;
  $21 = 931071618 | $20;
  _sqlite3Put4byte($aWalHdr,$21);
  $22 = (($aWalHdr) + 4|0);
  _sqlite3Put4byte($22,3007000);
  $23 = (($aWalHdr) + 8|0);
  $24 = HEAP32[$2>>2]|0;
  _sqlite3Put4byte($23,$24);
  $25 = (($aWalHdr) + 12|0);
  $26 = HEAP32[$1>>2]|0;
  $27 = (($26) + 104|0);
  $28 = HEAP32[$27>>2]|0;
  _sqlite3Put4byte($25,$28);
  $29 = HEAP32[$1>>2]|0;
  $30 = (($29) + 104|0);
  $31 = HEAP32[$30>>2]|0;
  $32 = ($31|0)==(0);
  if ($32) {
   $33 = HEAP32[$1>>2]|0;
   $34 = (($33) + 52|0);
   $35 = (($34) + 32|0);
   _sqlite3_randomness(8,$35);
  }
  $36 = (($aWalHdr) + 16|0);
  $37 = HEAP32[$1>>2]|0;
  $38 = (($37) + 52|0);
  $39 = (($38) + 32|0);
  ;HEAP8[$36+0>>0]=HEAP8[$39+0>>0]|0;HEAP8[$36+1>>0]=HEAP8[$39+1>>0]|0;HEAP8[$36+2>>0]=HEAP8[$39+2>>0]|0;HEAP8[$36+3>>0]=HEAP8[$39+3>>0]|0;HEAP8[$36+4>>0]=HEAP8[$39+4>>0]|0;HEAP8[$36+5>>0]=HEAP8[$39+5>>0]|0;HEAP8[$36+6>>0]=HEAP8[$39+6>>0]|0;HEAP8[$36+7>>0]=HEAP8[$39+7>>0]|0;
  _walChecksumBytes(1,$aWalHdr,24,0,$aCksum);
  $40 = (($aWalHdr) + 24|0);
  $41 = HEAP32[$aCksum>>2]|0;
  _sqlite3Put4byte($40,$41);
  $42 = (($aWalHdr) + 28|0);
  $43 = (($aCksum) + 4|0);
  $44 = HEAP32[$43>>2]|0;
  _sqlite3Put4byte($42,$44);
  $45 = HEAP32[$2>>2]|0;
  $46 = HEAP32[$1>>2]|0;
  $47 = (($46) + 36|0);
  HEAP32[$47>>2] = $45;
  $48 = HEAP8[1136>>0]|0;
  $49 = $48 << 24 >> 24;
  $50 = ($49|0)==(0);
  $51 = $50&1;
  $52 = $51&255;
  $53 = HEAP32[$1>>2]|0;
  $54 = (($53) + 52|0);
  $55 = (($54) + 13|0);
  HEAP8[$55>>0] = $52;
  $56 = HEAP32[$aCksum>>2]|0;
  $57 = HEAP32[$1>>2]|0;
  $58 = (($57) + 52|0);
  $59 = (($58) + 24|0);
  HEAP32[$59>>2] = $56;
  $60 = (($aCksum) + 4|0);
  $61 = HEAP32[$60>>2]|0;
  $62 = HEAP32[$1>>2]|0;
  $63 = (($62) + 52|0);
  $64 = (($63) + 24|0);
  $65 = (($64) + 4|0);
  HEAP32[$65>>2] = $61;
  $66 = HEAP32[$1>>2]|0;
  $67 = (($66) + 47|0);
  HEAP8[$67>>0] = 1;
  $68 = HEAP32[$1>>2]|0;
  $69 = (($68) + 8|0);
  $70 = HEAP32[$69>>2]|0;
  $71 = (_sqlite3OsWrite($70,$aWalHdr,32,0,0)|0);
  HEAP32[$rc>>2] = $71;
  $72 = HEAP32[$rc>>2]|0;
  $73 = ($72|0)!=(0);
  if ($73) {
   $74 = HEAP32[$rc>>2]|0;
   HEAP32[$0>>2] = $74;
   $410 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($410|0);
  }
  $75 = HEAP32[$1>>2]|0;
  $76 = (($75) + 48|0);
  $77 = HEAP8[$76>>0]|0;
  $78 = $77&255;
  $79 = ($78|0)!=(0);
  $80 = HEAP32[$6>>2]|0;
  $81 = ($80|0)!=(0);
  $or$cond = $79 & $81;
  if ($or$cond) {
   $82 = HEAP32[$1>>2]|0;
   $83 = (($82) + 8|0);
   $84 = HEAP32[$83>>2]|0;
   $85 = HEAP32[$6>>2]|0;
   $86 = $85 & 19;
   $87 = (_sqlite3OsSync($84,$86)|0);
   HEAP32[$rc>>2] = $87;
   $88 = HEAP32[$rc>>2]|0;
   $89 = ($88|0)!=(0);
   if ($89) {
    $90 = HEAP32[$rc>>2]|0;
    HEAP32[$0>>2] = $90;
    $410 = HEAP32[$0>>2]|0;
    STACKTOP = sp;return ($410|0);
   }
  }
 }
 $91 = HEAP32[$1>>2]|0;
 HEAP32[$w>>2] = $91;
 $92 = HEAP32[$1>>2]|0;
 $93 = (($92) + 8|0);
 $94 = HEAP32[$93>>2]|0;
 $95 = (($w) + 4|0);
 HEAP32[$95>>2] = $94;
 $96 = (($w) + 8|0);
 $97 = $96;
 $98 = $97;
 HEAP32[$98>>2] = 0;
 $99 = (($97) + 4)|0;
 $100 = $99;
 HEAP32[$100>>2] = 0;
 $101 = HEAP32[$6>>2]|0;
 $102 = (($w) + 16|0);
 HEAP32[$102>>2] = $101;
 $103 = HEAP32[$2>>2]|0;
 $104 = (($w) + 20|0);
 HEAP32[$104>>2] = $103;
 $105 = HEAP32[$iFrame>>2]|0;
 $106 = (($105) + 1)|0;
 $107 = (($106) - 1)|0;
 $108 = HEAP32[$2>>2]|0;
 $109 = (($108) + 24)|0;
 $110 = ($109|0)<(0);
 $111 = $110 << 31 >> 31;
 $112 = (___muldi3(($107|0),0,($109|0),($111|0))|0);
 $113 = tempRet0;
 $114 = (_i64Add(32,0,($112|0),($113|0))|0);
 $115 = tempRet0;
 $116 = $iOffset;
 $117 = $116;
 HEAP32[$117>>2] = $114;
 $118 = (($116) + 4)|0;
 $119 = $118;
 HEAP32[$119>>2] = $115;
 $120 = HEAP32[$2>>2]|0;
 $121 = (($120) + 24)|0;
 HEAP32[$szFrame>>2] = $121;
 $122 = HEAP32[$3>>2]|0;
 HEAP32[$p>>2] = $122;
 while(1) {
  $123 = HEAP32[$p>>2]|0;
  $124 = ($123|0)!=(0|0);
  if (!($124)) {
   break;
  }
  $125 = HEAP32[$iFrame>>2]|0;
  $126 = (($125) + 1)|0;
  HEAP32[$iFrame>>2] = $126;
  $127 = HEAP32[$5>>2]|0;
  $128 = ($127|0)!=(0);
  if ($128) {
   $129 = HEAP32[$p>>2]|0;
   $130 = (($129) + 12|0);
   $131 = HEAP32[$130>>2]|0;
   $132 = ($131|0)==(0|0);
   $133 = HEAP32[$4>>2]|0;
   $$ = $132 ? $133 : 0;
   $134 = $$;
  } else {
   $134 = 0;
  }
  HEAP32[$nDbSize>>2] = $134;
  $135 = HEAP32[$p>>2]|0;
  $136 = HEAP32[$nDbSize>>2]|0;
  $137 = $iOffset;
  $138 = $137;
  $139 = HEAP32[$138>>2]|0;
  $140 = (($137) + 4)|0;
  $141 = $140;
  $142 = HEAP32[$141>>2]|0;
  $143 = (_walWriteOneFrame($w,$135,$136,$139,$142)|0);
  HEAP32[$rc>>2] = $143;
  $144 = HEAP32[$rc>>2]|0;
  $145 = ($144|0)!=(0);
  if ($145) {
   label = 16;
   break;
  }
  $147 = HEAP32[$p>>2]|0;
  HEAP32[$pLast>>2] = $147;
  $148 = HEAP32[$szFrame>>2]|0;
  $149 = ($148|0)<(0);
  $150 = $149 << 31 >> 31;
  $151 = $iOffset;
  $152 = $151;
  $153 = HEAP32[$152>>2]|0;
  $154 = (($151) + 4)|0;
  $155 = $154;
  $156 = HEAP32[$155>>2]|0;
  $157 = (_i64Add(($153|0),($156|0),($148|0),($150|0))|0);
  $158 = tempRet0;
  $159 = $iOffset;
  $160 = $159;
  HEAP32[$160>>2] = $157;
  $161 = (($159) + 4)|0;
  $162 = $161;
  HEAP32[$162>>2] = $158;
  $163 = HEAP32[$p>>2]|0;
  $164 = (($163) + 12|0);
  $165 = HEAP32[$164>>2]|0;
  HEAP32[$p>>2] = $165;
 }
 if ((label|0) == 16) {
  $146 = HEAP32[$rc>>2]|0;
  HEAP32[$0>>2] = $146;
  $410 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($410|0);
 }
 $166 = HEAP32[$5>>2]|0;
 $167 = ($166|0)!=(0);
 L29: do {
  if ($167) {
   $168 = HEAP32[$6>>2]|0;
   $169 = $168 & 32;
   $170 = ($169|0)!=(0);
   if ($170) {
    $171 = HEAP32[$1>>2]|0;
    $172 = (($171) + 49|0);
    $173 = HEAP8[$172>>0]|0;
    $174 = ($173<<24>>24)!=(0);
    if (!($174)) {
     $254 = (($w) + 4|0);
     $255 = HEAP32[$254>>2]|0;
     $256 = HEAP32[$6>>2]|0;
     $257 = $256 & 19;
     $258 = (_sqlite3OsSync($255,$257)|0);
     HEAP32[$rc>>2] = $258;
     break;
    }
    $175 = HEAP32[$1>>2]|0;
    $176 = (($175) + 8|0);
    $177 = HEAP32[$176>>2]|0;
    $178 = (_sqlite3SectorSize($177)|0);
    HEAP32[$sectorSize>>2] = $178;
    $179 = $iOffset;
    $180 = $179;
    $181 = HEAP32[$180>>2]|0;
    $182 = (($179) + 4)|0;
    $183 = $182;
    $184 = HEAP32[$183>>2]|0;
    $185 = HEAP32[$sectorSize>>2]|0;
    $186 = ($185|0)<(0);
    $187 = $186 << 31 >> 31;
    $188 = (_i64Add(($181|0),($184|0),($185|0),($187|0))|0);
    $189 = tempRet0;
    $190 = (_i64Subtract(($188|0),($189|0),1,0)|0);
    $191 = tempRet0;
    $192 = HEAP32[$sectorSize>>2]|0;
    $193 = ($192|0)<(0);
    $194 = $193 << 31 >> 31;
    $195 = (___divdi3(($190|0),($191|0),($192|0),($194|0))|0);
    $196 = tempRet0;
    $197 = HEAP32[$sectorSize>>2]|0;
    $198 = ($197|0)<(0);
    $199 = $198 << 31 >> 31;
    $200 = (___muldi3(($195|0),($196|0),($197|0),($199|0))|0);
    $201 = tempRet0;
    $202 = (($w) + 8|0);
    $203 = $202;
    $204 = $203;
    HEAP32[$204>>2] = $200;
    $205 = (($203) + 4)|0;
    $206 = $205;
    HEAP32[$206>>2] = $201;
    while(1) {
     $207 = $iOffset;
     $208 = $207;
     $209 = HEAP32[$208>>2]|0;
     $210 = (($207) + 4)|0;
     $211 = $210;
     $212 = HEAP32[$211>>2]|0;
     $213 = (($w) + 8|0);
     $214 = $213;
     $215 = $214;
     $216 = HEAP32[$215>>2]|0;
     $217 = (($214) + 4)|0;
     $218 = $217;
     $219 = HEAP32[$218>>2]|0;
     $220 = ($212|0)<($219|0);
     $221 = ($209>>>0)<($216>>>0);
     $222 = ($212|0)==($219|0);
     $223 = $222 & $221;
     $224 = $220 | $223;
     if (!($224)) {
      break L29;
     }
     $225 = HEAP32[$pLast>>2]|0;
     $226 = HEAP32[$4>>2]|0;
     $227 = $iOffset;
     $228 = $227;
     $229 = HEAP32[$228>>2]|0;
     $230 = (($227) + 4)|0;
     $231 = $230;
     $232 = HEAP32[$231>>2]|0;
     $233 = (_walWriteOneFrame($w,$225,$226,$229,$232)|0);
     HEAP32[$rc>>2] = $233;
     $234 = HEAP32[$rc>>2]|0;
     $235 = ($234|0)!=(0);
     if ($235) {
      break;
     }
     $237 = HEAP32[$szFrame>>2]|0;
     $238 = ($237|0)<(0);
     $239 = $238 << 31 >> 31;
     $240 = $iOffset;
     $241 = $240;
     $242 = HEAP32[$241>>2]|0;
     $243 = (($240) + 4)|0;
     $244 = $243;
     $245 = HEAP32[$244>>2]|0;
     $246 = (_i64Add(($242|0),($245|0),($237|0),($239|0))|0);
     $247 = tempRet0;
     $248 = $iOffset;
     $249 = $248;
     HEAP32[$249>>2] = $246;
     $250 = (($248) + 4)|0;
     $251 = $250;
     HEAP32[$251>>2] = $247;
     $252 = HEAP32[$nExtra>>2]|0;
     $253 = (($252) + 1)|0;
     HEAP32[$nExtra>>2] = $253;
    }
    $236 = HEAP32[$rc>>2]|0;
    HEAP32[$0>>2] = $236;
    $410 = HEAP32[$0>>2]|0;
    STACKTOP = sp;return ($410|0);
   }
  }
 } while(0);
 $259 = HEAP32[$5>>2]|0;
 $260 = ($259|0)!=(0);
 if ($260) {
  $261 = HEAP32[$1>>2]|0;
  $262 = (($261) + 47|0);
  $263 = HEAP8[$262>>0]|0;
  $264 = $263&255;
  $265 = ($264|0)!=(0);
  if ($265) {
   $266 = HEAP32[$1>>2]|0;
   $267 = (($266) + 16|0);
   $268 = $267;
   $269 = $268;
   $270 = HEAP32[$269>>2]|0;
   $271 = (($268) + 4)|0;
   $272 = $271;
   $273 = HEAP32[$272>>2]|0;
   $274 = ($273|0)>(0);
   $275 = ($270>>>0)>=(0);
   $276 = ($273|0)==(0);
   $277 = $276 & $275;
   $278 = $274 | $277;
   if ($278) {
    $279 = HEAP32[$1>>2]|0;
    $280 = (($279) + 16|0);
    $281 = $280;
    $282 = $281;
    $283 = HEAP32[$282>>2]|0;
    $284 = (($281) + 4)|0;
    $285 = $284;
    $286 = HEAP32[$285>>2]|0;
    $287 = $sz;
    $288 = $287;
    HEAP32[$288>>2] = $283;
    $289 = (($287) + 4)|0;
    $290 = $289;
    HEAP32[$290>>2] = $286;
    $291 = HEAP32[$iFrame>>2]|0;
    $292 = HEAP32[$nExtra>>2]|0;
    $293 = (($291) + ($292))|0;
    $294 = (($293) + 1)|0;
    $295 = (($294) - 1)|0;
    $296 = HEAP32[$2>>2]|0;
    $297 = (($296) + 24)|0;
    $298 = ($297|0)<(0);
    $299 = $298 << 31 >> 31;
    $300 = (___muldi3(($295|0),0,($297|0),($299|0))|0);
    $301 = tempRet0;
    $302 = (_i64Add(32,0,($300|0),($301|0))|0);
    $303 = tempRet0;
    $304 = HEAP32[$1>>2]|0;
    $305 = (($304) + 16|0);
    $306 = $305;
    $307 = $306;
    $308 = HEAP32[$307>>2]|0;
    $309 = (($306) + 4)|0;
    $310 = $309;
    $311 = HEAP32[$310>>2]|0;
    $312 = ($303|0)>($311|0);
    $313 = ($302>>>0)>($308>>>0);
    $314 = ($303|0)==($311|0);
    $315 = $314 & $313;
    $316 = $312 | $315;
    if ($316) {
     $317 = HEAP32[$iFrame>>2]|0;
     $318 = HEAP32[$nExtra>>2]|0;
     $319 = (($317) + ($318))|0;
     $320 = (($319) + 1)|0;
     $321 = (($320) - 1)|0;
     $322 = HEAP32[$2>>2]|0;
     $323 = (($322) + 24)|0;
     $324 = ($323|0)<(0);
     $325 = $324 << 31 >> 31;
     $326 = (___muldi3(($321|0),0,($323|0),($325|0))|0);
     $327 = tempRet0;
     $328 = (_i64Add(32,0,($326|0),($327|0))|0);
     $329 = tempRet0;
     $330 = $sz;
     $331 = $330;
     HEAP32[$331>>2] = $328;
     $332 = (($330) + 4)|0;
     $333 = $332;
     HEAP32[$333>>2] = $329;
    }
    $334 = HEAP32[$1>>2]|0;
    $335 = $sz;
    $336 = $335;
    $337 = HEAP32[$336>>2]|0;
    $338 = (($335) + 4)|0;
    $339 = $338;
    $340 = HEAP32[$339>>2]|0;
    _walLimitSize($334,$337,$340);
    $341 = HEAP32[$1>>2]|0;
    $342 = (($341) + 47|0);
    HEAP8[$342>>0] = 0;
   }
  }
 }
 $343 = HEAP32[$1>>2]|0;
 $344 = (($343) + 52|0);
 $345 = (($344) + 16|0);
 $346 = HEAP32[$345>>2]|0;
 HEAP32[$iFrame>>2] = $346;
 $347 = HEAP32[$3>>2]|0;
 HEAP32[$p>>2] = $347;
 while(1) {
  $348 = HEAP32[$p>>2]|0;
  $349 = ($348|0)!=(0|0);
  $350 = HEAP32[$rc>>2]|0;
  $351 = ($350|0)==(0);
  $or$cond3 = $349 & $351;
  if (!($or$cond3)) {
   break;
  }
  $352 = HEAP32[$iFrame>>2]|0;
  $353 = (($352) + 1)|0;
  HEAP32[$iFrame>>2] = $353;
  $354 = HEAP32[$1>>2]|0;
  $355 = HEAP32[$iFrame>>2]|0;
  $356 = HEAP32[$p>>2]|0;
  $357 = (($356) + 20|0);
  $358 = HEAP32[$357>>2]|0;
  $359 = (_walIndexAppend($354,$355,$358)|0);
  HEAP32[$rc>>2] = $359;
  $360 = HEAP32[$p>>2]|0;
  $361 = (($360) + 12|0);
  $362 = HEAP32[$361>>2]|0;
  HEAP32[$p>>2] = $362;
 }
 while(1) {
  $363 = HEAP32[$rc>>2]|0;
  $364 = ($363|0)==(0);
  $365 = HEAP32[$nExtra>>2]|0;
  $366 = ($365|0)>(0);
  $or$cond5 = $364 & $366;
  if (!($or$cond5)) {
   break;
  }
  $367 = HEAP32[$iFrame>>2]|0;
  $368 = (($367) + 1)|0;
  HEAP32[$iFrame>>2] = $368;
  $369 = HEAP32[$nExtra>>2]|0;
  $370 = (($369) + -1)|0;
  HEAP32[$nExtra>>2] = $370;
  $371 = HEAP32[$1>>2]|0;
  $372 = HEAP32[$iFrame>>2]|0;
  $373 = HEAP32[$pLast>>2]|0;
  $374 = (($373) + 20|0);
  $375 = HEAP32[$374>>2]|0;
  $376 = (_walIndexAppend($371,$372,$375)|0);
  HEAP32[$rc>>2] = $376;
 }
 $377 = HEAP32[$rc>>2]|0;
 $378 = ($377|0)==(0);
 if ($378) {
  $379 = HEAP32[$2>>2]|0;
  $380 = $379 & 65280;
  $381 = HEAP32[$2>>2]|0;
  $382 = $381 >> 16;
  $383 = $380 | $382;
  $384 = $383&65535;
  $385 = HEAP32[$1>>2]|0;
  $386 = (($385) + 52|0);
  $387 = (($386) + 14|0);
  HEAP16[$387>>1] = $384;
  $388 = HEAP32[$iFrame>>2]|0;
  $389 = HEAP32[$1>>2]|0;
  $390 = (($389) + 52|0);
  $391 = (($390) + 16|0);
  HEAP32[$391>>2] = $388;
  $392 = HEAP32[$5>>2]|0;
  $393 = ($392|0)!=(0);
  if ($393) {
   $394 = HEAP32[$1>>2]|0;
   $395 = (($394) + 52|0);
   $396 = (($395) + 8|0);
   $397 = HEAP32[$396>>2]|0;
   $398 = (($397) + 1)|0;
   HEAP32[$396>>2] = $398;
   $399 = HEAP32[$4>>2]|0;
   $400 = HEAP32[$1>>2]|0;
   $401 = (($400) + 52|0);
   $402 = (($401) + 20|0);
   HEAP32[$402>>2] = $399;
  }
  $403 = HEAP32[$5>>2]|0;
  $404 = ($403|0)!=(0);
  if ($404) {
   $405 = HEAP32[$1>>2]|0;
   _walIndexWriteHdr($405);
   $406 = HEAP32[$iFrame>>2]|0;
   $407 = HEAP32[$1>>2]|0;
   $408 = (($407) + 12|0);
   HEAP32[$408>>2] = $406;
  }
 }
 $409 = HEAP32[$rc>>2]|0;
 HEAP32[$0>>2] = $409;
 $410 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($410|0);
}
function _walRestartLog($pWal) {
 $pWal = $pWal|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $7 = 0, $8 = 0;
 var $9 = 0, $aSalt = 0, $cnt = 0, $i = 0, $notUsed = 0, $pInfo = 0, $rc = 0, $salt1 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $0 = sp + 32|0;
 $1 = sp + 28|0;
 $rc = sp + 24|0;
 $cnt = sp + 20|0;
 $pInfo = sp + 16|0;
 $salt1 = sp + 12|0;
 $i = sp + 8|0;
 $aSalt = sp + 4|0;
 $notUsed = sp;
 HEAP32[$1>>2] = $pWal;
 HEAP32[$rc>>2] = 0;
 $2 = HEAP32[$1>>2]|0;
 $3 = (($2) + 40|0);
 $4 = HEAP16[$3>>1]|0;
 $5 = $4 << 16 >> 16;
 $6 = ($5|0)==(0);
 if ($6) {
  $7 = HEAP32[$1>>2]|0;
  $8 = (_walCkptInfo($7)|0);
  HEAP32[$pInfo>>2] = $8;
  $9 = HEAP32[$pInfo>>2]|0;
  $10 = HEAP32[$9>>2]|0;
  $11 = ($10>>>0)>(0);
  do {
   if ($11) {
    _sqlite3_randomness(4,$salt1);
    $12 = HEAP32[$1>>2]|0;
    $13 = (_walLockExclusive($12,4,4)|0);
    HEAP32[$rc>>2] = $13;
    $14 = HEAP32[$rc>>2]|0;
    $15 = ($14|0)==(0);
    if (!($15)) {
     $47 = HEAP32[$rc>>2]|0;
     $48 = ($47|0)!=(5);
     if (!($48)) {
      break;
     }
     $49 = HEAP32[$rc>>2]|0;
     HEAP32[$0>>2] = $49;
     $60 = HEAP32[$0>>2]|0;
     STACKTOP = sp;return ($60|0);
    }
    $16 = HEAP32[$1>>2]|0;
    $17 = (($16) + 52|0);
    $18 = (($17) + 32|0);
    HEAP32[$aSalt>>2] = $18;
    $19 = HEAP32[$1>>2]|0;
    $20 = (($19) + 104|0);
    $21 = HEAP32[$20>>2]|0;
    $22 = (($21) + 1)|0;
    HEAP32[$20>>2] = $22;
    $23 = HEAP32[$1>>2]|0;
    $24 = (($23) + 52|0);
    $25 = (($24) + 16|0);
    HEAP32[$25>>2] = 0;
    $26 = HEAP32[$aSalt>>2]|0;
    $27 = HEAP32[$aSalt>>2]|0;
    $28 = (_sqlite3Get4byte($27)|0);
    $29 = (1 + ($28))|0;
    _sqlite3Put4byte($26,$29);
    $30 = HEAP32[$salt1>>2]|0;
    $31 = HEAP32[$aSalt>>2]|0;
    $32 = (($31) + 4|0);
    HEAP32[$32>>2] = $30;
    $33 = HEAP32[$1>>2]|0;
    _walIndexWriteHdr($33);
    $34 = HEAP32[$pInfo>>2]|0;
    HEAP32[$34>>2] = 0;
    $35 = HEAP32[$pInfo>>2]|0;
    $36 = (($35) + 4|0);
    $37 = (($36) + 4|0);
    HEAP32[$37>>2] = 0;
    HEAP32[$i>>2] = 2;
    while(1) {
     $38 = HEAP32[$i>>2]|0;
     $39 = ($38|0)<(5);
     if (!($39)) {
      break;
     }
     $40 = HEAP32[$i>>2]|0;
     $41 = HEAP32[$pInfo>>2]|0;
     $42 = (($41) + 4|0);
     $43 = (($42) + ($40<<2)|0);
     HEAP32[$43>>2] = -1;
     $44 = HEAP32[$i>>2]|0;
     $45 = (($44) + 1)|0;
     HEAP32[$i>>2] = $45;
    }
    $46 = HEAP32[$1>>2]|0;
    _walUnlockExclusive($46,4,4);
   }
  } while(0);
  $50 = HEAP32[$1>>2]|0;
  _walUnlockShared($50,3);
  $51 = HEAP32[$1>>2]|0;
  $52 = (($51) + 40|0);
  HEAP16[$52>>1] = -1;
  HEAP32[$cnt>>2] = 0;
  while(1) {
   $53 = HEAP32[$1>>2]|0;
   $54 = HEAP32[$cnt>>2]|0;
   $55 = (($54) + 1)|0;
   HEAP32[$cnt>>2] = $55;
   $56 = (_walTryBeginRead($53,$notUsed,1,$55)|0);
   HEAP32[$rc>>2] = $56;
   $57 = HEAP32[$rc>>2]|0;
   $58 = ($57|0)==(-1);
   if (!($58)) {
    break;
   }
  }
 }
 $59 = HEAP32[$rc>>2]|0;
 HEAP32[$0>>2] = $59;
 $60 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($60|0);
}
function _walWriteOneFrame($p,$pPage,$nTruncate,$0,$1) {
 $p = $p|0;
 $pPage = $pPage|0;
 $nTruncate = $nTruncate|0;
 $0 = $0|0;
 $1 = $1|0;
 var $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0;
 var $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0;
 var $47 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $aFrame = 0, $pData = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 64|0;
 $2 = sp + 28|0;
 $3 = sp + 24|0;
 $4 = sp + 20|0;
 $5 = sp + 16|0;
 $6 = sp;
 $rc = sp + 12|0;
 $pData = sp + 8|0;
 $aFrame = sp + 32|0;
 HEAP32[$3>>2] = $p;
 HEAP32[$4>>2] = $pPage;
 HEAP32[$5>>2] = $nTruncate;
 $7 = $6;
 $8 = $7;
 HEAP32[$8>>2] = $0;
 $9 = (($7) + 4)|0;
 $10 = $9;
 HEAP32[$10>>2] = $1;
 $11 = HEAP32[$4>>2]|0;
 $12 = (($11) + 4|0);
 $13 = HEAP32[$12>>2]|0;
 HEAP32[$pData>>2] = $13;
 $14 = HEAP32[$3>>2]|0;
 $15 = HEAP32[$14>>2]|0;
 $16 = HEAP32[$4>>2]|0;
 $17 = (($16) + 20|0);
 $18 = HEAP32[$17>>2]|0;
 $19 = HEAP32[$5>>2]|0;
 $20 = HEAP32[$pData>>2]|0;
 _walEncodeFrame($15,$18,$19,$20,$aFrame);
 $21 = HEAP32[$3>>2]|0;
 $22 = $6;
 $23 = $22;
 $24 = HEAP32[$23>>2]|0;
 $25 = (($22) + 4)|0;
 $26 = $25;
 $27 = HEAP32[$26>>2]|0;
 $28 = (_walWriteToLog($21,$aFrame,24,$24,$27)|0);
 HEAP32[$rc>>2] = $28;
 $29 = HEAP32[$rc>>2]|0;
 $30 = ($29|0)!=(0);
 if ($30) {
  $31 = HEAP32[$rc>>2]|0;
  HEAP32[$2>>2] = $31;
  $47 = HEAP32[$2>>2]|0;
  STACKTOP = sp;return ($47|0);
 } else {
  $32 = HEAP32[$3>>2]|0;
  $33 = HEAP32[$pData>>2]|0;
  $34 = HEAP32[$3>>2]|0;
  $35 = (($34) + 20|0);
  $36 = HEAP32[$35>>2]|0;
  $37 = $6;
  $38 = $37;
  $39 = HEAP32[$38>>2]|0;
  $40 = (($37) + 4)|0;
  $41 = $40;
  $42 = HEAP32[$41>>2]|0;
  $43 = (_i64Add(($39|0),($42|0),24,0)|0);
  $44 = tempRet0;
  $45 = (_walWriteToLog($32,$33,$36,$43,$44)|0);
  HEAP32[$rc>>2] = $45;
  $46 = HEAP32[$rc>>2]|0;
  HEAP32[$2>>2] = $46;
  $47 = HEAP32[$2>>2]|0;
  STACKTOP = sp;return ($47|0);
 }
 return 0|0;
}
function _walEncodeFrame($pWal,$iPage,$nTruncate,$aData,$aFrame) {
 $pWal = $pWal|0;
 $iPage = $iPage|0;
 $nTruncate = $nTruncate|0;
 $aData = $aData|0;
 $aFrame = $aFrame|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $aCksum = 0, $nativeCksum = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 24|0;
 $1 = sp + 20|0;
 $2 = sp + 16|0;
 $3 = sp + 12|0;
 $4 = sp + 8|0;
 $nativeCksum = sp + 4|0;
 $aCksum = sp;
 HEAP32[$0>>2] = $pWal;
 HEAP32[$1>>2] = $iPage;
 HEAP32[$2>>2] = $nTruncate;
 HEAP32[$3>>2] = $aData;
 HEAP32[$4>>2] = $aFrame;
 $5 = HEAP32[$0>>2]|0;
 $6 = (($5) + 52|0);
 $7 = (($6) + 24|0);
 HEAP32[$aCksum>>2] = $7;
 $8 = HEAP32[$4>>2]|0;
 $9 = HEAP32[$1>>2]|0;
 _sqlite3Put4byte($8,$9);
 $10 = HEAP32[$4>>2]|0;
 $11 = (($10) + 4|0);
 $12 = HEAP32[$2>>2]|0;
 _sqlite3Put4byte($11,$12);
 $13 = HEAP32[$4>>2]|0;
 $14 = (($13) + 8|0);
 $15 = HEAP32[$0>>2]|0;
 $16 = (($15) + 52|0);
 $17 = (($16) + 32|0);
 ;HEAP8[$14+0>>0]=HEAP8[$17+0>>0]|0;HEAP8[$14+1>>0]=HEAP8[$17+1>>0]|0;HEAP8[$14+2>>0]=HEAP8[$17+2>>0]|0;HEAP8[$14+3>>0]=HEAP8[$17+3>>0]|0;HEAP8[$14+4>>0]=HEAP8[$17+4>>0]|0;HEAP8[$14+5>>0]=HEAP8[$17+5>>0]|0;HEAP8[$14+6>>0]=HEAP8[$17+6>>0]|0;HEAP8[$14+7>>0]=HEAP8[$17+7>>0]|0;
 $18 = HEAP32[$0>>2]|0;
 $19 = (($18) + 52|0);
 $20 = (($19) + 13|0);
 $21 = HEAP8[$20>>0]|0;
 $22 = $21&255;
 $23 = HEAP8[1136>>0]|0;
 $24 = $23 << 24 >> 24;
 $25 = ($24|0)==(0);
 $26 = $25&1;
 $27 = ($22|0)==($26|0);
 $28 = $27&1;
 HEAP32[$nativeCksum>>2] = $28;
 $29 = HEAP32[$nativeCksum>>2]|0;
 $30 = HEAP32[$4>>2]|0;
 $31 = HEAP32[$aCksum>>2]|0;
 $32 = HEAP32[$aCksum>>2]|0;
 _walChecksumBytes($29,$30,8,$31,$32);
 $33 = HEAP32[$nativeCksum>>2]|0;
 $34 = HEAP32[$3>>2]|0;
 $35 = HEAP32[$0>>2]|0;
 $36 = (($35) + 36|0);
 $37 = HEAP32[$36>>2]|0;
 $38 = HEAP32[$aCksum>>2]|0;
 $39 = HEAP32[$aCksum>>2]|0;
 _walChecksumBytes($33,$34,$37,$38,$39);
 $40 = HEAP32[$4>>2]|0;
 $41 = (($40) + 16|0);
 $42 = HEAP32[$aCksum>>2]|0;
 $43 = HEAP32[$42>>2]|0;
 _sqlite3Put4byte($41,$43);
 $44 = HEAP32[$4>>2]|0;
 $45 = (($44) + 20|0);
 $46 = HEAP32[$aCksum>>2]|0;
 $47 = (($46) + 4|0);
 $48 = HEAP32[$47>>2]|0;
 _sqlite3Put4byte($45,$48);
 STACKTOP = sp;return;
}
function _walWriteToLog($p,$pContent,$iAmt,$0,$1) {
 $p = $p|0;
 $pContent = $pContent|0;
 $iAmt = $iAmt|0;
 $0 = $0|0;
 $1 = $1|0;
 var $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0, $116 = 0, $117 = 0;
 var $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $14 = 0, $15 = 0, $16 = 0;
 var $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0;
 var $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0;
 var $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0;
 var $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0;
 var $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $iFirstAmt = 0, $or$cond = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $2 = sp + 28|0;
 $3 = sp + 24|0;
 $4 = sp + 20|0;
 $5 = sp + 16|0;
 $6 = sp;
 $rc = sp + 12|0;
 $iFirstAmt = sp + 8|0;
 HEAP32[$3>>2] = $p;
 HEAP32[$4>>2] = $pContent;
 HEAP32[$5>>2] = $iAmt;
 $7 = $6;
 $8 = $7;
 HEAP32[$8>>2] = $0;
 $9 = (($7) + 4)|0;
 $10 = $9;
 HEAP32[$10>>2] = $1;
 $11 = $6;
 $12 = $11;
 $13 = HEAP32[$12>>2]|0;
 $14 = (($11) + 4)|0;
 $15 = $14;
 $16 = HEAP32[$15>>2]|0;
 $17 = HEAP32[$3>>2]|0;
 $18 = (($17) + 8|0);
 $19 = $18;
 $20 = $19;
 $21 = HEAP32[$20>>2]|0;
 $22 = (($19) + 4)|0;
 $23 = $22;
 $24 = HEAP32[$23>>2]|0;
 $25 = ($16|0)<($24|0);
 $26 = ($13>>>0)<($21>>>0);
 $27 = ($16|0)==($24|0);
 $28 = $27 & $26;
 $29 = $25 | $28;
 if ($29) {
  $30 = $6;
  $31 = $30;
  $32 = HEAP32[$31>>2]|0;
  $33 = (($30) + 4)|0;
  $34 = $33;
  $35 = HEAP32[$34>>2]|0;
  $36 = HEAP32[$5>>2]|0;
  $37 = ($36|0)<(0);
  $38 = $37 << 31 >> 31;
  $39 = (_i64Add(($32|0),($35|0),($36|0),($38|0))|0);
  $40 = tempRet0;
  $41 = HEAP32[$3>>2]|0;
  $42 = (($41) + 8|0);
  $43 = $42;
  $44 = $43;
  $45 = HEAP32[$44>>2]|0;
  $46 = (($43) + 4)|0;
  $47 = $46;
  $48 = HEAP32[$47>>2]|0;
  $49 = ($40|0)>($48|0);
  $50 = ($39>>>0)>=($45>>>0);
  $51 = ($40|0)==($48|0);
  $52 = $51 & $50;
  $53 = $49 | $52;
  if ($53) {
   $54 = HEAP32[$3>>2]|0;
   $55 = (($54) + 8|0);
   $56 = $55;
   $57 = $56;
   $58 = HEAP32[$57>>2]|0;
   $59 = (($56) + 4)|0;
   $60 = $59;
   $61 = HEAP32[$60>>2]|0;
   $62 = $6;
   $63 = $62;
   $64 = HEAP32[$63>>2]|0;
   $65 = (($62) + 4)|0;
   $66 = $65;
   $67 = HEAP32[$66>>2]|0;
   $68 = (_i64Subtract(($58|0),($61|0),($64|0),($67|0))|0);
   $69 = tempRet0;
   HEAP32[$iFirstAmt>>2] = $68;
   $70 = HEAP32[$3>>2]|0;
   $71 = (($70) + 4|0);
   $72 = HEAP32[$71>>2]|0;
   $73 = HEAP32[$4>>2]|0;
   $74 = HEAP32[$iFirstAmt>>2]|0;
   $75 = $6;
   $76 = $75;
   $77 = HEAP32[$76>>2]|0;
   $78 = (($75) + 4)|0;
   $79 = $78;
   $80 = HEAP32[$79>>2]|0;
   $81 = (_sqlite3OsWrite($72,$73,$74,$77,$80)|0);
   HEAP32[$rc>>2] = $81;
   $82 = HEAP32[$rc>>2]|0;
   $83 = ($82|0)!=(0);
   if ($83) {
    $84 = HEAP32[$rc>>2]|0;
    HEAP32[$2>>2] = $84;
    $132 = HEAP32[$2>>2]|0;
    STACKTOP = sp;return ($132|0);
   }
   $85 = HEAP32[$iFirstAmt>>2]|0;
   $86 = ($85|0)<(0);
   $87 = $86 << 31 >> 31;
   $88 = $6;
   $89 = $88;
   $90 = HEAP32[$89>>2]|0;
   $91 = (($88) + 4)|0;
   $92 = $91;
   $93 = HEAP32[$92>>2]|0;
   $94 = (_i64Add(($90|0),($93|0),($85|0),($87|0))|0);
   $95 = tempRet0;
   $96 = $6;
   $97 = $96;
   HEAP32[$97>>2] = $94;
   $98 = (($96) + 4)|0;
   $99 = $98;
   HEAP32[$99>>2] = $95;
   $100 = HEAP32[$iFirstAmt>>2]|0;
   $101 = HEAP32[$5>>2]|0;
   $102 = (($101) - ($100))|0;
   HEAP32[$5>>2] = $102;
   $103 = HEAP32[$iFirstAmt>>2]|0;
   $104 = HEAP32[$4>>2]|0;
   $105 = (($104) + ($103)|0);
   HEAP32[$4>>2] = $105;
   $106 = HEAP32[$3>>2]|0;
   $107 = (($106) + 4|0);
   $108 = HEAP32[$107>>2]|0;
   $109 = HEAP32[$3>>2]|0;
   $110 = (($109) + 16|0);
   $111 = HEAP32[$110>>2]|0;
   $112 = $111 & 19;
   $113 = (_sqlite3OsSync($108,$112)|0);
   HEAP32[$rc>>2] = $113;
   $114 = HEAP32[$5>>2]|0;
   $115 = ($114|0)==(0);
   $116 = HEAP32[$rc>>2]|0;
   $117 = ($116|0)!=(0);
   $or$cond = $115 | $117;
   if ($or$cond) {
    $118 = HEAP32[$rc>>2]|0;
    HEAP32[$2>>2] = $118;
    $132 = HEAP32[$2>>2]|0;
    STACKTOP = sp;return ($132|0);
   }
  }
 }
 $119 = HEAP32[$3>>2]|0;
 $120 = (($119) + 4|0);
 $121 = HEAP32[$120>>2]|0;
 $122 = HEAP32[$4>>2]|0;
 $123 = HEAP32[$5>>2]|0;
 $124 = $6;
 $125 = $124;
 $126 = HEAP32[$125>>2]|0;
 $127 = (($124) + 4)|0;
 $128 = $127;
 $129 = HEAP32[$128>>2]|0;
 $130 = (_sqlite3OsWrite($121,$122,$123,$126,$129)|0);
 HEAP32[$rc>>2] = $130;
 $131 = HEAP32[$rc>>2]|0;
 HEAP32[$2>>2] = $131;
 $132 = HEAP32[$2>>2]|0;
 STACKTOP = sp;return ($132|0);
}
function _walTryBeginRead($pWal,$pChanged,$useWal,$cnt) {
 $pWal = $pWal|0;
 $pChanged = $pChanged|0;
 $useWal = $useWal|0;
 $cnt = $cnt|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0;
 var $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0;
 var $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0;
 var $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0;
 var $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0;
 var $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $i = 0, $mxI = 0, $mxReadMark = 0, $nDelay = 0, $or$cond = 0, $pInfo = 0, $rc = 0, $thisMark = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $0 = sp + 44|0;
 $1 = sp + 40|0;
 $2 = sp + 36|0;
 $3 = sp + 32|0;
 $4 = sp + 28|0;
 $pInfo = sp + 24|0;
 $mxReadMark = sp + 20|0;
 $mxI = sp + 16|0;
 $i = sp + 12|0;
 $rc = sp + 8|0;
 $nDelay = sp + 4|0;
 $thisMark = sp;
 HEAP32[$1>>2] = $pWal;
 HEAP32[$2>>2] = $pChanged;
 HEAP32[$3>>2] = $useWal;
 HEAP32[$4>>2] = $cnt;
 HEAP32[$rc>>2] = 0;
 $5 = HEAP32[$4>>2]|0;
 $6 = ($5|0)>(5);
 if ($6) {
  HEAP32[$nDelay>>2] = 1;
  $7 = HEAP32[$4>>2]|0;
  $8 = ($7|0)>(100);
  if ($8) {
   HEAP32[$0>>2] = 15;
   $164 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($164|0);
  }
  $9 = HEAP32[$4>>2]|0;
  $10 = ($9|0)>=(10);
  if ($10) {
   $11 = HEAP32[$4>>2]|0;
   $12 = (($11) - 9)|0;
   $13 = HEAP32[$4>>2]|0;
   $14 = (($13) - 9)|0;
   $15 = Math_imul($12, $14)|0;
   $16 = ($15*39)|0;
   HEAP32[$nDelay>>2] = $16;
  }
  $17 = HEAP32[$1>>2]|0;
  $18 = HEAP32[$17>>2]|0;
  $19 = HEAP32[$nDelay>>2]|0;
  (_sqlite3OsSleep($18,$19)|0);
 }
 $20 = HEAP32[$3>>2]|0;
 $21 = ($20|0)!=(0);
 if (!($21)) {
  $22 = HEAP32[$1>>2]|0;
  $23 = HEAP32[$2>>2]|0;
  $24 = (_walIndexReadHdr($22,$23)|0);
  HEAP32[$rc>>2] = $24;
  $25 = HEAP32[$rc>>2]|0;
  $26 = ($25|0)==(5);
  do {
   if ($26) {
    $27 = HEAP32[$1>>2]|0;
    $28 = (($27) + 32|0);
    $29 = HEAP32[$28>>2]|0;
    $30 = HEAP32[$29>>2]|0;
    $31 = ($30|0)==(0|0);
    if ($31) {
     HEAP32[$rc>>2] = -1;
     break;
    }
    $32 = HEAP32[$1>>2]|0;
    $33 = (_walLockShared($32,2)|0);
    HEAP32[$rc>>2] = $33;
    $34 = (0)==($33|0);
    if ($34) {
     $35 = HEAP32[$1>>2]|0;
     _walUnlockShared($35,2);
     HEAP32[$rc>>2] = -1;
     break;
    }
    $36 = HEAP32[$rc>>2]|0;
    $37 = ($36|0)==(5);
    if ($37) {
     HEAP32[$rc>>2] = 261;
    }
   }
  } while(0);
  $38 = HEAP32[$rc>>2]|0;
  $39 = ($38|0)!=(0);
  if ($39) {
   $40 = HEAP32[$rc>>2]|0;
   HEAP32[$0>>2] = $40;
   $164 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($164|0);
  }
 }
 $41 = HEAP32[$1>>2]|0;
 $42 = (_walCkptInfo($41)|0);
 HEAP32[$pInfo>>2] = $42;
 $43 = HEAP32[$3>>2]|0;
 $44 = ($43|0)!=(0);
 do {
  if (!($44)) {
   $45 = HEAP32[$pInfo>>2]|0;
   $46 = HEAP32[$45>>2]|0;
   $47 = HEAP32[$1>>2]|0;
   $48 = (($47) + 52|0);
   $49 = (($48) + 16|0);
   $50 = HEAP32[$49>>2]|0;
   $51 = ($46|0)==($50|0);
   if ($51) {
    $52 = HEAP32[$1>>2]|0;
    $53 = (_walLockShared($52,3)|0);
    HEAP32[$rc>>2] = $53;
    $54 = HEAP32[$1>>2]|0;
    _walShmBarrier($54);
    $55 = HEAP32[$rc>>2]|0;
    $56 = ($55|0)==(0);
    if (!($56)) {
     $65 = HEAP32[$rc>>2]|0;
     $66 = ($65|0)!=(5);
     if (!($66)) {
      break;
     }
     $67 = HEAP32[$rc>>2]|0;
     HEAP32[$0>>2] = $67;
     $164 = HEAP32[$0>>2]|0;
     STACKTOP = sp;return ($164|0);
    }
    $57 = HEAP32[$1>>2]|0;
    $58 = (_walIndexHdr($57)|0);
    $59 = HEAP32[$1>>2]|0;
    $60 = (($59) + 52|0);
    $61 = (_memcmp($58,$60,48)|0);
    $62 = ($61|0)!=(0);
    $63 = HEAP32[$1>>2]|0;
    if ($62) {
     _walUnlockShared($63,3);
     HEAP32[$0>>2] = -1;
     $164 = HEAP32[$0>>2]|0;
     STACKTOP = sp;return ($164|0);
    } else {
     $64 = (($63) + 40|0);
     HEAP16[$64>>1] = 0;
     HEAP32[$0>>2] = 0;
     $164 = HEAP32[$0>>2]|0;
     STACKTOP = sp;return ($164|0);
    }
   }
  }
 } while(0);
 HEAP32[$mxReadMark>>2] = 0;
 HEAP32[$mxI>>2] = 0;
 HEAP32[$i>>2] = 1;
 while(1) {
  $68 = HEAP32[$i>>2]|0;
  $69 = ($68|0)<(5);
  if (!($69)) {
   break;
  }
  $70 = HEAP32[$i>>2]|0;
  $71 = HEAP32[$pInfo>>2]|0;
  $72 = (($71) + 4|0);
  $73 = (($72) + ($70<<2)|0);
  $74 = HEAP32[$73>>2]|0;
  HEAP32[$thisMark>>2] = $74;
  $75 = HEAP32[$mxReadMark>>2]|0;
  $76 = HEAP32[$thisMark>>2]|0;
  $77 = ($75>>>0)<=($76>>>0);
  if ($77) {
   $78 = HEAP32[$thisMark>>2]|0;
   $79 = HEAP32[$1>>2]|0;
   $80 = (($79) + 52|0);
   $81 = (($80) + 16|0);
   $82 = HEAP32[$81>>2]|0;
   $83 = ($78>>>0)<=($82>>>0);
   if ($83) {
    $84 = HEAP32[$thisMark>>2]|0;
    HEAP32[$mxReadMark>>2] = $84;
    $85 = HEAP32[$i>>2]|0;
    HEAP32[$mxI>>2] = $85;
   }
  }
  $86 = HEAP32[$i>>2]|0;
  $87 = (($86) + 1)|0;
  HEAP32[$i>>2] = $87;
 }
 $88 = HEAP32[$1>>2]|0;
 $89 = (($88) + 46|0);
 $90 = HEAP8[$89>>0]|0;
 $91 = $90&255;
 $92 = $91 & 2;
 $93 = ($92|0)==(0);
 L48: do {
  if ($93) {
   $94 = HEAP32[$mxReadMark>>2]|0;
   $95 = HEAP32[$1>>2]|0;
   $96 = (($95) + 52|0);
   $97 = (($96) + 16|0);
   $98 = HEAP32[$97>>2]|0;
   $99 = ($94>>>0)<($98>>>0);
   $100 = HEAP32[$mxI>>2]|0;
   $101 = ($100|0)==(0);
   $or$cond = $99 | $101;
   if ($or$cond) {
    HEAP32[$i>>2] = 1;
    while(1) {
     $102 = HEAP32[$i>>2]|0;
     $103 = ($102|0)<(5);
     if (!($103)) {
      break L48;
     }
     $104 = HEAP32[$1>>2]|0;
     $105 = HEAP32[$i>>2]|0;
     $106 = (3 + ($105))|0;
     $107 = (_walLockExclusive($104,$106,1)|0);
     HEAP32[$rc>>2] = $107;
     $108 = HEAP32[$rc>>2]|0;
     $109 = ($108|0)==(0);
     if ($109) {
      label = 36;
      break;
     }
     $122 = HEAP32[$rc>>2]|0;
     $123 = ($122|0)!=(5);
     if ($123) {
      break;
     }
     $125 = HEAP32[$i>>2]|0;
     $126 = (($125) + 1)|0;
     HEAP32[$i>>2] = $126;
    }
    if ((label|0) == 36) {
     $110 = HEAP32[$1>>2]|0;
     $111 = (($110) + 52|0);
     $112 = (($111) + 16|0);
     $113 = HEAP32[$112>>2]|0;
     $114 = HEAP32[$i>>2]|0;
     $115 = HEAP32[$pInfo>>2]|0;
     $116 = (($115) + 4|0);
     $117 = (($116) + ($114<<2)|0);
     HEAP32[$117>>2] = $113;
     HEAP32[$mxReadMark>>2] = $113;
     $118 = HEAP32[$i>>2]|0;
     HEAP32[$mxI>>2] = $118;
     $119 = HEAP32[$1>>2]|0;
     $120 = HEAP32[$i>>2]|0;
     $121 = (3 + ($120))|0;
     _walUnlockExclusive($119,$121,1);
     break;
    }
    $124 = HEAP32[$rc>>2]|0;
    HEAP32[$0>>2] = $124;
    $164 = HEAP32[$0>>2]|0;
    STACKTOP = sp;return ($164|0);
   }
  }
 } while(0);
 $127 = HEAP32[$mxI>>2]|0;
 $128 = ($127|0)==(0);
 if ($128) {
  $129 = HEAP32[$rc>>2]|0;
  $130 = ($129|0)==(5);
  $131 = $130 ? -1 : 520;
  HEAP32[$0>>2] = $131;
  $164 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($164|0);
 }
 $132 = HEAP32[$1>>2]|0;
 $133 = HEAP32[$mxI>>2]|0;
 $134 = (3 + ($133))|0;
 $135 = (_walLockShared($132,$134)|0);
 HEAP32[$rc>>2] = $135;
 $136 = HEAP32[$rc>>2]|0;
 $137 = ($136|0)!=(0);
 if ($137) {
  $138 = HEAP32[$rc>>2]|0;
  $139 = ($138|0)==(5);
  $140 = HEAP32[$rc>>2]|0;
  $141 = $139 ? -1 : $140;
  HEAP32[$0>>2] = $141;
  $164 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($164|0);
 }
 $142 = HEAP32[$1>>2]|0;
 _walShmBarrier($142);
 $143 = HEAP32[$mxI>>2]|0;
 $144 = HEAP32[$pInfo>>2]|0;
 $145 = (($144) + 4|0);
 $146 = (($145) + ($143<<2)|0);
 $147 = HEAP32[$146>>2]|0;
 $148 = HEAP32[$mxReadMark>>2]|0;
 $149 = ($147|0)!=($148|0);
 if (!($149)) {
  $150 = HEAP32[$1>>2]|0;
  $151 = (_walIndexHdr($150)|0);
  $152 = HEAP32[$1>>2]|0;
  $153 = (($152) + 52|0);
  $154 = (_memcmp($151,$153,48)|0);
  $155 = ($154|0)!=(0);
  if (!($155)) {
   $159 = HEAP32[$mxI>>2]|0;
   $160 = $159&65535;
   $161 = HEAP32[$1>>2]|0;
   $162 = (($161) + 40|0);
   HEAP16[$162>>1] = $160;
   $163 = HEAP32[$rc>>2]|0;
   HEAP32[$0>>2] = $163;
   $164 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($164|0);
  }
 }
 $156 = HEAP32[$1>>2]|0;
 $157 = HEAP32[$mxI>>2]|0;
 $158 = (3 + ($157))|0;
 _walUnlockShared($156,$158);
 HEAP32[$0>>2] = -1;
 $164 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($164|0);
}
function _openSubJournal($pPager) {
 $pPager = $pPager|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $3 = 0, $4 = 0;
 var $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $rc = sp;
 HEAP32[$0>>2] = $pPager;
 HEAP32[$rc>>2] = 0;
 $1 = HEAP32[$0>>2]|0;
 $2 = (($1) + 68|0);
 $3 = HEAP32[$2>>2]|0;
 $4 = HEAP32[$3>>2]|0;
 $5 = ($4|0)!=(0|0);
 if ($5) {
  $24 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($24|0);
 }
 $6 = HEAP32[$0>>2]|0;
 $7 = (($6) + 5|0);
 $8 = HEAP8[$7>>0]|0;
 $9 = $8&255;
 $10 = ($9|0)==(4);
 if (!($10)) {
  $11 = HEAP32[$0>>2]|0;
  $12 = (($11) + 21|0);
  $13 = HEAP8[$12>>0]|0;
  $14 = $13&255;
  $15 = ($14|0)!=(0);
  if (!($15)) {
   $19 = HEAP32[$0>>2]|0;
   $20 = HEAP32[$0>>2]|0;
   $21 = (($20) + 68|0);
   $22 = HEAP32[$21>>2]|0;
   $23 = (_pagerOpentemp($19,$22,8192)|0);
   HEAP32[$rc>>2] = $23;
   $24 = HEAP32[$rc>>2]|0;
   STACKTOP = sp;return ($24|0);
  }
 }
 $16 = HEAP32[$0>>2]|0;
 $17 = (($16) + 68|0);
 $18 = HEAP32[$17>>2]|0;
 _sqlite3MemJournalOpen($18);
 $24 = HEAP32[$rc>>2]|0;
 STACKTOP = sp;return ($24|0);
}
function _write32bits($fd,$0,$1,$val) {
 $fd = $fd|0;
 $0 = $0|0;
 $1 = $1|0;
 $val = $val|0;
 var $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $ac = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $2 = sp + 12|0;
 $3 = sp;
 $4 = sp + 8|0;
 $ac = sp + 16|0;
 HEAP32[$2>>2] = $fd;
 $5 = $3;
 $6 = $5;
 HEAP32[$6>>2] = $0;
 $7 = (($5) + 4)|0;
 $8 = $7;
 HEAP32[$8>>2] = $1;
 HEAP32[$4>>2] = $val;
 $9 = HEAP32[$4>>2]|0;
 _sqlite3Put4byte($ac,$9);
 $10 = HEAP32[$2>>2]|0;
 $11 = $3;
 $12 = $11;
 $13 = HEAP32[$12>>2]|0;
 $14 = (($11) + 4)|0;
 $15 = $14;
 $16 = HEAP32[$15>>2]|0;
 $17 = (_sqlite3OsWrite($10,$ac,4,$13,$16)|0);
 STACKTOP = sp;return ($17|0);
}
function _addToSavepointBitvecs($pPager,$pgno) {
 $pPager = $pPager|0;
 $pgno = $pgno|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $ii = 0, $p = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 16|0;
 $1 = sp + 12|0;
 $ii = sp + 8|0;
 $rc = sp + 4|0;
 $p = sp;
 HEAP32[$0>>2] = $pPager;
 HEAP32[$1>>2] = $pgno;
 HEAP32[$rc>>2] = 0;
 HEAP32[$ii>>2] = 0;
 while(1) {
  $2 = HEAP32[$ii>>2]|0;
  $3 = HEAP32[$0>>2]|0;
  $4 = (($3) + 96|0);
  $5 = HEAP32[$4>>2]|0;
  $6 = ($2|0)<($5|0);
  if (!($6)) {
   break;
  }
  $7 = HEAP32[$ii>>2]|0;
  $8 = HEAP32[$0>>2]|0;
  $9 = (($8) + 92|0);
  $10 = HEAP32[$9>>2]|0;
  $11 = (($10) + (($7*48)|0)|0);
  HEAP32[$p>>2] = $11;
  $12 = HEAP32[$1>>2]|0;
  $13 = HEAP32[$p>>2]|0;
  $14 = (($13) + 20|0);
  $15 = HEAP32[$14>>2]|0;
  $16 = ($12>>>0)<=($15>>>0);
  if ($16) {
   $17 = HEAP32[$p>>2]|0;
   $18 = (($17) + 16|0);
   $19 = HEAP32[$18>>2]|0;
   $20 = HEAP32[$1>>2]|0;
   $21 = (_sqlite3BitvecSet($19,$20)|0);
   $22 = HEAP32[$rc>>2]|0;
   $23 = $22 | $21;
   HEAP32[$rc>>2] = $23;
  }
  $24 = HEAP32[$ii>>2]|0;
  $25 = (($24) + 1)|0;
  HEAP32[$ii>>2] = $25;
 }
 $26 = HEAP32[$rc>>2]|0;
 STACKTOP = sp;return ($26|0);
}
function _allSpaces($z,$n) {
 $z = $z|0;
 $n = $n|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $1 = sp;
 HEAP32[$0>>2] = $z;
 HEAP32[$1>>2] = $n;
 while(1) {
  $2 = HEAP32[$1>>2]|0;
  $3 = ($2|0)>(0);
  if ($3) {
   $4 = HEAP32[$1>>2]|0;
   $5 = (($4) - 1)|0;
   $6 = HEAP32[$0>>2]|0;
   $7 = (($6) + ($5)|0);
   $8 = HEAP8[$7>>0]|0;
   $9 = $8 << 24 >> 24;
   $10 = ($9|0)==(32);
   $15 = $10;
  } else {
   $15 = 0;
  }
  $11 = HEAP32[$1>>2]|0;
  if (!($15)) {
   break;
  }
  $12 = (($11) + -1)|0;
  HEAP32[$1>>2] = $12;
 }
 $13 = ($11|0)==(0);
 $14 = $13&1;
 STACKTOP = sp;return ($14|0);
}
function _logBadConnection($zType) {
 $zType = $zType|0;
 var $0 = 0, $1 = 0, $vararg_buffer = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $vararg_buffer = sp;
 $0 = sp + 4|0;
 HEAP32[$0>>2] = $zType;
 $1 = HEAP32[$0>>2]|0;
 HEAP32[$vararg_buffer>>2] = $1;
 _sqlite3_log(21,4880,$vararg_buffer);
 STACKTOP = sp;return;
}
function _sqlite3BtreeCheckpoint($p,$eMode,$pnLog,$pnCkpt) {
 $p = $p|0;
 $eMode = $eMode|0;
 $pnLog = $pnLog|0;
 $pnCkpt = $pnCkpt|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0;
 var $8 = 0, $9 = 0, $pBt = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 20|0;
 $1 = sp + 16|0;
 $2 = sp + 12|0;
 $3 = sp + 8|0;
 $rc = sp + 4|0;
 $pBt = sp;
 HEAP32[$0>>2] = $p;
 HEAP32[$1>>2] = $eMode;
 HEAP32[$2>>2] = $pnLog;
 HEAP32[$3>>2] = $pnCkpt;
 HEAP32[$rc>>2] = 0;
 $4 = HEAP32[$0>>2]|0;
 $5 = ($4|0)!=(0|0);
 if (!($5)) {
  $21 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($21|0);
 }
 $6 = HEAP32[$0>>2]|0;
 $7 = (($6) + 4|0);
 $8 = HEAP32[$7>>2]|0;
 HEAP32[$pBt>>2] = $8;
 $9 = HEAP32[$0>>2]|0;
 _sqlite3BtreeEnter($9);
 $10 = HEAP32[$pBt>>2]|0;
 $11 = (($10) + 20|0);
 $12 = HEAP8[$11>>0]|0;
 $13 = $12&255;
 $14 = ($13|0)!=(0);
 if ($14) {
  HEAP32[$rc>>2] = 6;
  $21 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($21|0);
 } else {
  $15 = HEAP32[$pBt>>2]|0;
  $16 = HEAP32[$15>>2]|0;
  $17 = HEAP32[$1>>2]|0;
  $18 = HEAP32[$2>>2]|0;
  $19 = HEAP32[$3>>2]|0;
  $20 = (_sqlite3PagerCheckpoint($16,$17,$18,$19)|0);
  HEAP32[$rc>>2] = $20;
  $21 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($21|0);
 }
 return 0|0;
}
function _sqlite3PagerCheckpoint($pPager,$eMode,$pnLog,$pnCkpt) {
 $pPager = $pPager|0;
 $eMode = $eMode|0;
 $pnLog = $pnLog|0;
 $pnCkpt = $pnCkpt|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 16|0;
 $1 = sp + 12|0;
 $2 = sp + 8|0;
 $3 = sp + 4|0;
 $rc = sp;
 HEAP32[$0>>2] = $pPager;
 HEAP32[$1>>2] = $eMode;
 HEAP32[$2>>2] = $pnLog;
 HEAP32[$3>>2] = $pnCkpt;
 HEAP32[$rc>>2] = 0;
 $4 = HEAP32[$0>>2]|0;
 $5 = (($4) + 208|0);
 $6 = HEAP32[$5>>2]|0;
 $7 = ($6|0)!=(0|0);
 if (!($7)) {
  $31 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($31|0);
 }
 $8 = HEAP32[$0>>2]|0;
 $9 = (($8) + 208|0);
 $10 = HEAP32[$9>>2]|0;
 $11 = HEAP32[$1>>2]|0;
 $12 = HEAP32[$0>>2]|0;
 $13 = (($12) + 176|0);
 $14 = HEAP32[$13>>2]|0;
 $15 = HEAP32[$0>>2]|0;
 $16 = (($15) + 180|0);
 $17 = HEAP32[$16>>2]|0;
 $18 = HEAP32[$0>>2]|0;
 $19 = (($18) + 9|0);
 $20 = HEAP8[$19>>0]|0;
 $21 = $20&255;
 $22 = HEAP32[$0>>2]|0;
 $23 = (($22) + 152|0);
 $24 = HEAP32[$23>>2]|0;
 $25 = HEAP32[$0>>2]|0;
 $26 = (($25) + 200|0);
 $27 = HEAP32[$26>>2]|0;
 $28 = HEAP32[$2>>2]|0;
 $29 = HEAP32[$3>>2]|0;
 $30 = (_sqlite3WalCheckpoint($10,$11,$14,$17,$21,$24,$27,$28,$29)|0);
 HEAP32[$rc>>2] = $30;
 $31 = HEAP32[$rc>>2]|0;
 STACKTOP = sp;return ($31|0);
}
function _functionSearch($pHash,$h,$zFunc,$nFunc) {
 $pHash = $pHash|0;
 $h = $h|0;
 $zFunc = $zFunc|0;
 $nFunc = $nFunc|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $p = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 20|0;
 $1 = sp + 16|0;
 $2 = sp + 12|0;
 $3 = sp + 8|0;
 $4 = sp + 4|0;
 $p = sp;
 HEAP32[$1>>2] = $pHash;
 HEAP32[$2>>2] = $h;
 HEAP32[$3>>2] = $zFunc;
 HEAP32[$4>>2] = $nFunc;
 $5 = HEAP32[$2>>2]|0;
 $6 = HEAP32[$1>>2]|0;
 $7 = (($6) + ($5<<2)|0);
 $8 = HEAP32[$7>>2]|0;
 HEAP32[$p>>2] = $8;
 while(1) {
  $9 = HEAP32[$p>>2]|0;
  $10 = ($9|0)!=(0|0);
  if (!($10)) {
   label = 7;
   break;
  }
  $11 = HEAP32[$p>>2]|0;
  $12 = (($11) + 24|0);
  $13 = HEAP32[$12>>2]|0;
  $14 = HEAP32[$3>>2]|0;
  $15 = HEAP32[$4>>2]|0;
  $16 = (_sqlite3_strnicmp($13,$14,$15)|0);
  $17 = ($16|0)==(0);
  if ($17) {
   $18 = HEAP32[$4>>2]|0;
   $19 = HEAP32[$p>>2]|0;
   $20 = (($19) + 24|0);
   $21 = HEAP32[$20>>2]|0;
   $22 = (($21) + ($18)|0);
   $23 = HEAP8[$22>>0]|0;
   $24 = $23 << 24 >> 24;
   $25 = ($24|0)==(0);
   if ($25) {
    label = 5;
    break;
   }
  }
  $27 = HEAP32[$p>>2]|0;
  $28 = (($27) + 28|0);
  $29 = HEAP32[$28>>2]|0;
  HEAP32[$p>>2] = $29;
 }
 if ((label|0) == 5) {
  $26 = HEAP32[$p>>2]|0;
  HEAP32[$0>>2] = $26;
  $30 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($30|0);
 }
 else if ((label|0) == 7) {
  HEAP32[$0>>2] = 0;
  $30 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($30|0);
 }
 return 0|0;
}
function _matchQuality($p,$nArg,$enc) {
 $p = $p|0;
 $nArg = $nArg|0;
 $enc = $enc|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $match = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 12|0;
 $1 = sp + 8|0;
 $2 = sp + 4|0;
 $3 = sp + 16|0;
 $match = sp;
 HEAP32[$1>>2] = $p;
 HEAP32[$2>>2] = $nArg;
 HEAP8[$3>>0] = $enc;
 $4 = HEAP32[$2>>2]|0;
 $5 = ($4|0)==(-2);
 $6 = HEAP32[$1>>2]|0;
 if ($5) {
  $7 = (($6) + 12|0);
  $8 = HEAP32[$7>>2]|0;
  $9 = ($8|0)==(0|0);
  if ($9) {
   $10 = HEAP32[$1>>2]|0;
   $11 = (($10) + 16|0);
   $12 = HEAP32[$11>>2]|0;
   $13 = ($12|0)==(0|0);
   $15 = $13;
  } else {
   $15 = 0;
  }
  $14 = $15 ? 0 : 6;
  HEAP32[$0>>2] = $14;
  $51 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($51|0);
 }
 $16 = HEAP16[$6>>1]|0;
 $17 = $16 << 16 >> 16;
 $18 = HEAP32[$2>>2]|0;
 $19 = ($17|0)!=($18|0);
 if ($19) {
  $20 = HEAP32[$1>>2]|0;
  $21 = HEAP16[$20>>1]|0;
  $22 = $21 << 16 >> 16;
  $23 = ($22|0)>=(0);
  if ($23) {
   HEAP32[$0>>2] = 0;
   $51 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($51|0);
  }
 }
 $24 = HEAP32[$1>>2]|0;
 $25 = HEAP16[$24>>1]|0;
 $26 = $25 << 16 >> 16;
 $27 = HEAP32[$2>>2]|0;
 $28 = ($26|0)==($27|0);
 if ($28) {
  HEAP32[$match>>2] = 4;
 } else {
  HEAP32[$match>>2] = 1;
 }
 $29 = HEAP8[$3>>0]|0;
 $30 = $29&255;
 $31 = HEAP32[$1>>2]|0;
 $32 = (($31) + 2|0);
 $33 = HEAP16[$32>>1]|0;
 $34 = $33&65535;
 $35 = $34 & 3;
 $36 = ($30|0)==($35|0);
 if ($36) {
  $37 = HEAP32[$match>>2]|0;
  $38 = (($37) + 2)|0;
  HEAP32[$match>>2] = $38;
 } else {
  $39 = HEAP8[$3>>0]|0;
  $40 = $39&255;
  $41 = HEAP32[$1>>2]|0;
  $42 = (($41) + 2|0);
  $43 = HEAP16[$42>>1]|0;
  $44 = $43&65535;
  $45 = $40 & $44;
  $46 = $45 & 2;
  $47 = ($46|0)!=(0);
  if ($47) {
   $48 = HEAP32[$match>>2]|0;
   $49 = (($48) + 1)|0;
   HEAP32[$match>>2] = $49;
  }
 }
 $50 = HEAP32[$match>>2]|0;
 HEAP32[$0>>2] = $50;
 $51 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($51|0);
}
function _sqlite3FuncDefInsert($pHash,$pDef) {
 $pHash = $pHash|0;
 $pDef = $pDef|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $c1 = 0, $h = 0, $nName = 0, $pOther = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 16|0;
 $1 = sp + 12|0;
 $pOther = sp + 8|0;
 $nName = sp + 4|0;
 $c1 = sp + 20|0;
 $h = sp;
 HEAP32[$0>>2] = $pHash;
 HEAP32[$1>>2] = $pDef;
 $2 = HEAP32[$1>>2]|0;
 $3 = (($2) + 24|0);
 $4 = HEAP32[$3>>2]|0;
 $5 = (_sqlite3Strlen30($4)|0);
 HEAP32[$nName>>2] = $5;
 $6 = HEAP32[$1>>2]|0;
 $7 = (($6) + 24|0);
 $8 = HEAP32[$7>>2]|0;
 $9 = HEAP8[$8>>0]|0;
 HEAP8[$c1>>0] = $9;
 $10 = HEAP8[$c1>>0]|0;
 $11 = $10&255;
 $12 = (440 + ($11)|0);
 $13 = HEAP8[$12>>0]|0;
 $14 = $13&255;
 $15 = HEAP32[$nName>>2]|0;
 $16 = (($14) + ($15))|0;
 $17 = (($16|0) % 23)&-1;
 HEAP32[$h>>2] = $17;
 $18 = HEAP32[$0>>2]|0;
 $19 = HEAP32[$h>>2]|0;
 $20 = HEAP32[$1>>2]|0;
 $21 = (($20) + 24|0);
 $22 = HEAP32[$21>>2]|0;
 $23 = HEAP32[$nName>>2]|0;
 $24 = (_functionSearch($18,$19,$22,$23)|0);
 HEAP32[$pOther>>2] = $24;
 $25 = HEAP32[$pOther>>2]|0;
 $26 = ($25|0)!=(0|0);
 if ($26) {
  $27 = HEAP32[$pOther>>2]|0;
  $28 = (($27) + 8|0);
  $29 = HEAP32[$28>>2]|0;
  $30 = HEAP32[$1>>2]|0;
  $31 = (($30) + 8|0);
  HEAP32[$31>>2] = $29;
  $32 = HEAP32[$1>>2]|0;
  $33 = HEAP32[$pOther>>2]|0;
  $34 = (($33) + 8|0);
  HEAP32[$34>>2] = $32;
  STACKTOP = sp;return;
 } else {
  $35 = HEAP32[$1>>2]|0;
  $36 = (($35) + 8|0);
  HEAP32[$36>>2] = 0;
  $37 = HEAP32[$h>>2]|0;
  $38 = HEAP32[$0>>2]|0;
  $39 = (($38) + ($37<<2)|0);
  $40 = HEAP32[$39>>2]|0;
  $41 = HEAP32[$1>>2]|0;
  $42 = (($41) + 28|0);
  HEAP32[$42>>2] = $40;
  $43 = HEAP32[$1>>2]|0;
  $44 = HEAP32[$h>>2]|0;
  $45 = HEAP32[$0>>2]|0;
  $46 = (($45) + ($44<<2)|0);
  HEAP32[$46>>2] = $43;
  STACKTOP = sp;return;
 }
}
function _sqlite3VdbeChangeEncoding($pMem,$desiredEnc) {
 $pMem = $pMem|0;
 $desiredEnc = $desiredEnc|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0;
 var $9 = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $1 = sp + 8|0;
 $2 = sp + 4|0;
 $rc = sp;
 HEAP32[$1>>2] = $pMem;
 HEAP32[$2>>2] = $desiredEnc;
 $3 = HEAP32[$1>>2]|0;
 $4 = (($3) + 8|0);
 $5 = HEAP16[$4>>1]|0;
 $6 = $5&65535;
 $7 = $6 & 2;
 $8 = ($7|0)!=(0);
 if ($8) {
  $9 = HEAP32[$1>>2]|0;
  $10 = (($9) + 10|0);
  $11 = HEAP8[$10>>0]|0;
  $12 = $11&255;
  $13 = HEAP32[$2>>2]|0;
  $14 = ($12|0)==($13|0);
  if (!($14)) {
   $15 = HEAP32[$1>>2]|0;
   $16 = HEAP32[$2>>2]|0;
   $17 = $16&255;
   $18 = (_sqlite3VdbeMemTranslate($15,$17)|0);
   HEAP32[$rc>>2] = $18;
   $19 = HEAP32[$rc>>2]|0;
   HEAP32[$0>>2] = $19;
   $20 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($20|0);
  }
 }
 HEAP32[$0>>2] = 0;
 $20 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($20|0);
}
function _sqlite3VdbeMemTranslate($pMem,$desiredEnc) {
 $pMem = $pMem|0;
 $desiredEnc = $desiredEnc|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0;
 var $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0, $188 = 0;
 var $189 = 0, $19 = 0, $190 = 0, $191 = 0, $192 = 0, $193 = 0, $194 = 0, $195 = 0, $196 = 0, $197 = 0, $198 = 0, $199 = 0, $2 = 0, $20 = 0, $200 = 0, $201 = 0, $202 = 0, $203 = 0, $204 = 0, $205 = 0;
 var $206 = 0, $207 = 0, $208 = 0, $209 = 0, $21 = 0, $210 = 0, $211 = 0, $212 = 0, $213 = 0, $214 = 0, $215 = 0, $216 = 0, $217 = 0, $218 = 0, $219 = 0, $22 = 0, $220 = 0, $221 = 0, $222 = 0, $223 = 0;
 var $224 = 0, $225 = 0, $226 = 0, $227 = 0, $228 = 0, $229 = 0, $23 = 0, $230 = 0, $231 = 0, $232 = 0, $233 = 0, $234 = 0, $235 = 0, $236 = 0, $237 = 0, $238 = 0, $239 = 0, $24 = 0, $240 = 0, $241 = 0;
 var $242 = 0, $243 = 0, $244 = 0, $245 = 0, $246 = 0, $247 = 0, $248 = 0, $249 = 0, $25 = 0, $250 = 0, $251 = 0, $252 = 0, $253 = 0, $254 = 0, $255 = 0, $256 = 0, $257 = 0, $258 = 0, $259 = 0, $26 = 0;
 var $260 = 0, $261 = 0, $262 = 0, $263 = 0, $264 = 0, $265 = 0, $266 = 0, $267 = 0, $268 = 0, $269 = 0, $27 = 0, $270 = 0, $271 = 0, $272 = 0, $273 = 0, $274 = 0, $275 = 0, $276 = 0, $277 = 0, $278 = 0;
 var $279 = 0, $28 = 0, $280 = 0, $281 = 0, $282 = 0, $283 = 0, $284 = 0, $285 = 0, $286 = 0, $287 = 0, $288 = 0, $289 = 0, $29 = 0, $290 = 0, $291 = 0, $292 = 0, $293 = 0, $294 = 0, $295 = 0, $296 = 0;
 var $297 = 0, $298 = 0, $299 = 0, $3 = 0, $30 = 0, $300 = 0, $301 = 0, $302 = 0, $303 = 0, $304 = 0, $305 = 0, $306 = 0, $307 = 0, $308 = 0, $309 = 0, $31 = 0, $310 = 0, $311 = 0, $312 = 0, $313 = 0;
 var $314 = 0, $315 = 0, $316 = 0, $317 = 0, $318 = 0, $319 = 0, $32 = 0, $320 = 0, $321 = 0, $322 = 0, $323 = 0, $324 = 0, $325 = 0, $326 = 0, $327 = 0, $328 = 0, $329 = 0, $33 = 0, $330 = 0, $331 = 0;
 var $332 = 0, $333 = 0, $334 = 0, $335 = 0, $336 = 0, $337 = 0, $338 = 0, $339 = 0, $34 = 0, $340 = 0, $341 = 0, $342 = 0, $343 = 0, $344 = 0, $345 = 0, $346 = 0, $347 = 0, $348 = 0, $349 = 0, $35 = 0;
 var $350 = 0, $351 = 0, $352 = 0, $353 = 0, $354 = 0, $355 = 0, $356 = 0, $357 = 0, $358 = 0, $359 = 0, $36 = 0, $360 = 0, $361 = 0, $362 = 0, $363 = 0, $364 = 0, $365 = 0, $366 = 0, $367 = 0, $368 = 0;
 var $369 = 0, $37 = 0, $370 = 0, $371 = 0, $372 = 0, $373 = 0, $374 = 0, $375 = 0, $376 = 0, $377 = 0, $378 = 0, $379 = 0, $38 = 0, $380 = 0, $381 = 0, $382 = 0, $383 = 0, $384 = 0, $385 = 0, $386 = 0;
 var $387 = 0, $388 = 0, $389 = 0, $39 = 0, $390 = 0, $391 = 0, $392 = 0, $393 = 0, $394 = 0, $395 = 0, $396 = 0, $397 = 0, $398 = 0, $399 = 0, $4 = 0, $40 = 0, $400 = 0, $401 = 0, $402 = 0, $403 = 0;
 var $404 = 0, $405 = 0, $406 = 0, $407 = 0, $408 = 0, $409 = 0, $41 = 0, $410 = 0, $411 = 0, $412 = 0, $413 = 0, $414 = 0, $415 = 0, $416 = 0, $417 = 0, $418 = 0, $419 = 0, $42 = 0, $420 = 0, $421 = 0;
 var $422 = 0, $423 = 0, $424 = 0, $425 = 0, $426 = 0, $427 = 0, $428 = 0, $429 = 0, $43 = 0, $430 = 0, $431 = 0, $432 = 0, $433 = 0, $434 = 0, $435 = 0, $436 = 0, $437 = 0, $438 = 0, $439 = 0, $44 = 0;
 var $440 = 0, $441 = 0, $442 = 0, $443 = 0, $444 = 0, $445 = 0, $446 = 0, $447 = 0, $448 = 0, $449 = 0, $45 = 0, $450 = 0, $451 = 0, $452 = 0, $453 = 0, $454 = 0, $455 = 0, $456 = 0, $457 = 0, $458 = 0;
 var $459 = 0, $46 = 0, $460 = 0, $461 = 0, $462 = 0, $463 = 0, $464 = 0, $465 = 0, $466 = 0, $467 = 0, $468 = 0, $469 = 0, $47 = 0, $470 = 0, $471 = 0, $472 = 0, $473 = 0, $474 = 0, $475 = 0, $476 = 0;
 var $477 = 0, $478 = 0, $479 = 0, $48 = 0, $480 = 0, $481 = 0, $482 = 0, $483 = 0, $484 = 0, $485 = 0, $486 = 0, $487 = 0, $488 = 0, $489 = 0, $49 = 0, $490 = 0, $491 = 0, $492 = 0, $493 = 0, $494 = 0;
 var $495 = 0, $496 = 0, $497 = 0, $498 = 0, $499 = 0, $5 = 0, $50 = 0, $500 = 0, $501 = 0, $502 = 0, $503 = 0, $504 = 0, $505 = 0, $506 = 0, $507 = 0, $508 = 0, $509 = 0, $51 = 0, $510 = 0, $511 = 0;
 var $512 = 0, $513 = 0, $514 = 0, $515 = 0, $516 = 0, $517 = 0, $518 = 0, $519 = 0, $52 = 0, $520 = 0, $521 = 0, $522 = 0, $523 = 0, $524 = 0, $525 = 0, $526 = 0, $527 = 0, $528 = 0, $529 = 0, $53 = 0;
 var $530 = 0, $531 = 0, $532 = 0, $533 = 0, $534 = 0, $535 = 0, $536 = 0, $537 = 0, $538 = 0, $539 = 0, $54 = 0, $540 = 0, $541 = 0, $542 = 0, $543 = 0, $544 = 0, $545 = 0, $546 = 0, $547 = 0, $548 = 0;
 var $549 = 0, $55 = 0, $550 = 0, $551 = 0, $552 = 0, $553 = 0, $554 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0;
 var $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0;
 var $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $c = 0, $c2 = 0, $c21 = 0, $len = 0, $or$cond = 0;
 var $or$cond3 = 0, $rc = 0, $temp = 0, $z = 0, $zIn = 0, $zOut = 0, $zTerm = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $0 = sp + 40|0;
 $1 = sp + 36|0;
 $2 = sp + 45|0;
 $len = sp + 32|0;
 $zOut = sp + 28|0;
 $zIn = sp + 24|0;
 $zTerm = sp + 20|0;
 $z = sp + 16|0;
 $c = sp + 12|0;
 $temp = sp + 44|0;
 $rc = sp + 8|0;
 $c2 = sp + 4|0;
 $c21 = sp;
 HEAP32[$1>>2] = $pMem;
 HEAP8[$2>>0] = $desiredEnc;
 $3 = HEAP32[$1>>2]|0;
 $4 = (($3) + 10|0);
 $5 = HEAP8[$4>>0]|0;
 $6 = $5&255;
 $7 = ($6|0)!=(1);
 if ($7) {
  $8 = HEAP8[$2>>0]|0;
  $9 = $8&255;
  $10 = ($9|0)!=(1);
  if ($10) {
   $11 = HEAP32[$1>>2]|0;
   $12 = (_sqlite3VdbeMemMakeWriteable($11)|0);
   HEAP32[$rc>>2] = $12;
   $13 = HEAP32[$rc>>2]|0;
   $14 = ($13|0)!=(0);
   if ($14) {
    HEAP32[$0>>2] = 7;
    $552 = HEAP32[$0>>2]|0;
    STACKTOP = sp;return ($552|0);
   }
   $15 = HEAP32[$1>>2]|0;
   $16 = (($15) + 16|0);
   $17 = HEAP32[$16>>2]|0;
   HEAP32[$zIn>>2] = $17;
   $18 = HEAP32[$1>>2]|0;
   $19 = (($18) + 12|0);
   $20 = HEAP32[$19>>2]|0;
   $21 = $20 & -2;
   $22 = HEAP32[$zIn>>2]|0;
   $23 = (($22) + ($21)|0);
   HEAP32[$zTerm>>2] = $23;
   while(1) {
    $24 = HEAP32[$zIn>>2]|0;
    $25 = HEAP32[$zTerm>>2]|0;
    $26 = ($24>>>0)<($25>>>0);
    if (!($26)) {
     break;
    }
    $27 = HEAP32[$zIn>>2]|0;
    $28 = HEAP8[$27>>0]|0;
    HEAP8[$temp>>0] = $28;
    $29 = HEAP32[$zIn>>2]|0;
    $30 = (($29) + 1|0);
    $31 = HEAP8[$30>>0]|0;
    $32 = HEAP32[$zIn>>2]|0;
    HEAP8[$32>>0] = $31;
    $33 = HEAP32[$zIn>>2]|0;
    $34 = (($33) + 1|0);
    HEAP32[$zIn>>2] = $34;
    $35 = HEAP8[$temp>>0]|0;
    $36 = HEAP32[$zIn>>2]|0;
    $37 = (($36) + 1|0);
    HEAP32[$zIn>>2] = $37;
    HEAP8[$36>>0] = $35;
   }
   $38 = HEAP8[$2>>0]|0;
   $39 = HEAP32[$1>>2]|0;
   $40 = (($39) + 10|0);
   HEAP8[$40>>0] = $38;
  } else {
   label = 9;
  }
 } else {
  label = 9;
 }
 if ((label|0) == 9) {
  $41 = HEAP8[$2>>0]|0;
  $42 = $41&255;
  $43 = ($42|0)==(1);
  $44 = HEAP32[$1>>2]|0;
  $45 = (($44) + 12|0);
  $46 = HEAP32[$45>>2]|0;
  if ($43) {
   $47 = $46 & -2;
   HEAP32[$45>>2] = $47;
   $48 = HEAP32[$1>>2]|0;
   $49 = (($48) + 12|0);
   $50 = HEAP32[$49>>2]|0;
   $51 = $50<<1;
   $52 = (($51) + 1)|0;
   HEAP32[$len>>2] = $52;
  } else {
   $53 = $46<<1;
   $54 = (($53) + 2)|0;
   HEAP32[$len>>2] = $54;
  }
  $55 = HEAP32[$1>>2]|0;
  $56 = (($55) + 16|0);
  $57 = HEAP32[$56>>2]|0;
  HEAP32[$zIn>>2] = $57;
  $58 = HEAP32[$1>>2]|0;
  $59 = (($58) + 12|0);
  $60 = HEAP32[$59>>2]|0;
  $61 = HEAP32[$zIn>>2]|0;
  $62 = (($61) + ($60)|0);
  HEAP32[$zTerm>>2] = $62;
  $63 = HEAP32[$1>>2]|0;
  $64 = (($63) + 32|0);
  $65 = HEAP32[$64>>2]|0;
  $66 = HEAP32[$len>>2]|0;
  $67 = ($66|0)<(0);
  $68 = $67 << 31 >> 31;
  $69 = (_sqlite3DbMallocRaw($65,$66,$68)|0);
  HEAP32[$zOut>>2] = $69;
  $70 = HEAP32[$zOut>>2]|0;
  $71 = ($70|0)!=(0|0);
  if (!($71)) {
   HEAP32[$0>>2] = 7;
   $552 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($552|0);
  }
  $72 = HEAP32[$zOut>>2]|0;
  HEAP32[$z>>2] = $72;
  $73 = HEAP32[$1>>2]|0;
  $74 = (($73) + 10|0);
  $75 = HEAP8[$74>>0]|0;
  $76 = $75&255;
  $77 = ($76|0)==(1);
  if ($77) {
   $78 = HEAP8[$2>>0]|0;
   $79 = $78&255;
   $80 = ($79|0)==(2);
   L59: do {
    if ($80) {
     while(1) {
      $81 = HEAP32[$zIn>>2]|0;
      $82 = HEAP32[$zTerm>>2]|0;
      $83 = ($81>>>0)<($82>>>0);
      if (!($83)) {
       break L59;
      }
      $84 = HEAP32[$zIn>>2]|0;
      $85 = (($84) + 1|0);
      HEAP32[$zIn>>2] = $85;
      $86 = HEAP8[$84>>0]|0;
      $87 = $86&255;
      HEAP32[$c>>2] = $87;
      $88 = HEAP32[$c>>2]|0;
      $89 = ($88>>>0)>=(192);
      do {
       if ($89) {
        $90 = HEAP32[$c>>2]|0;
        $91 = (($90) - 192)|0;
        $92 = (4984 + ($91)|0);
        $93 = HEAP8[$92>>0]|0;
        $94 = $93&255;
        HEAP32[$c>>2] = $94;
        while(1) {
         $95 = HEAP32[$zIn>>2]|0;
         $96 = HEAP32[$zTerm>>2]|0;
         $97 = ($95|0)!=($96|0);
         if ($97) {
          $98 = HEAP32[$zIn>>2]|0;
          $99 = HEAP8[$98>>0]|0;
          $100 = $99&255;
          $101 = $100 & 192;
          $102 = ($101|0)==(128);
          $553 = $102;
         } else {
          $553 = 0;
         }
         $103 = HEAP32[$c>>2]|0;
         if (!($553)) {
          break;
         }
         $104 = $103 << 6;
         $105 = HEAP32[$zIn>>2]|0;
         $106 = (($105) + 1|0);
         HEAP32[$zIn>>2] = $106;
         $107 = HEAP8[$105>>0]|0;
         $108 = $107&255;
         $109 = 63 & $108;
         $110 = (($104) + ($109))|0;
         HEAP32[$c>>2] = $110;
        }
        $111 = ($103>>>0)<(128);
        if (!($111)) {
         $112 = HEAP32[$c>>2]|0;
         $113 = $112 & -2048;
         $114 = ($113|0)==(55296);
         if (!($114)) {
          $115 = HEAP32[$c>>2]|0;
          $116 = $115 & -2;
          $117 = ($116|0)==(65534);
          if (!($117)) {
           break;
          }
         }
        }
        HEAP32[$c>>2] = 65533;
       }
      } while(0);
      $118 = HEAP32[$c>>2]|0;
      $119 = ($118>>>0)<=(65535);
      $120 = HEAP32[$c>>2]|0;
      if ($119) {
       $121 = $120 & 255;
       $122 = $121&255;
       $123 = HEAP32[$z>>2]|0;
       $124 = (($123) + 1|0);
       HEAP32[$z>>2] = $124;
       HEAP8[$123>>0] = $122;
       $125 = HEAP32[$c>>2]|0;
       $126 = $125 >>> 8;
       $127 = $126 & 255;
       $128 = $127&255;
       $129 = HEAP32[$z>>2]|0;
       $130 = (($129) + 1|0);
       HEAP32[$z>>2] = $130;
       HEAP8[$129>>0] = $128;
       continue;
      } else {
       $131 = $120 >>> 10;
       $132 = $131 & 63;
       $133 = HEAP32[$c>>2]|0;
       $134 = (($133) - 65536)|0;
       $135 = $134 >>> 10;
       $136 = $135 & 192;
       $137 = (($132) + ($136))|0;
       $138 = $137&255;
       $139 = HEAP32[$z>>2]|0;
       $140 = (($139) + 1|0);
       HEAP32[$z>>2] = $140;
       HEAP8[$139>>0] = $138;
       $141 = HEAP32[$c>>2]|0;
       $142 = (($141) - 65536)|0;
       $143 = $142 >>> 18;
       $144 = $143 & 3;
       $145 = (216 + ($144))|0;
       $146 = $145&255;
       $147 = HEAP32[$z>>2]|0;
       $148 = (($147) + 1|0);
       HEAP32[$z>>2] = $148;
       HEAP8[$147>>0] = $146;
       $149 = HEAP32[$c>>2]|0;
       $150 = $149 & 255;
       $151 = $150&255;
       $152 = HEAP32[$z>>2]|0;
       $153 = (($152) + 1|0);
       HEAP32[$z>>2] = $153;
       HEAP8[$152>>0] = $151;
       $154 = HEAP32[$c>>2]|0;
       $155 = $154 >>> 8;
       $156 = $155 & 3;
       $157 = (220 + ($156))|0;
       $158 = $157&255;
       $159 = HEAP32[$z>>2]|0;
       $160 = (($159) + 1|0);
       HEAP32[$z>>2] = $160;
       HEAP8[$159>>0] = $158;
       continue;
      }
     }
    } else {
     while(1) {
      $161 = HEAP32[$zIn>>2]|0;
      $162 = HEAP32[$zTerm>>2]|0;
      $163 = ($161>>>0)<($162>>>0);
      if (!($163)) {
       break L59;
      }
      $164 = HEAP32[$zIn>>2]|0;
      $165 = (($164) + 1|0);
      HEAP32[$zIn>>2] = $165;
      $166 = HEAP8[$164>>0]|0;
      $167 = $166&255;
      HEAP32[$c>>2] = $167;
      $168 = HEAP32[$c>>2]|0;
      $169 = ($168>>>0)>=(192);
      do {
       if ($169) {
        $170 = HEAP32[$c>>2]|0;
        $171 = (($170) - 192)|0;
        $172 = (4984 + ($171)|0);
        $173 = HEAP8[$172>>0]|0;
        $174 = $173&255;
        HEAP32[$c>>2] = $174;
        while(1) {
         $175 = HEAP32[$zIn>>2]|0;
         $176 = HEAP32[$zTerm>>2]|0;
         $177 = ($175|0)!=($176|0);
         if ($177) {
          $178 = HEAP32[$zIn>>2]|0;
          $179 = HEAP8[$178>>0]|0;
          $180 = $179&255;
          $181 = $180 & 192;
          $182 = ($181|0)==(128);
          $554 = $182;
         } else {
          $554 = 0;
         }
         $183 = HEAP32[$c>>2]|0;
         if (!($554)) {
          break;
         }
         $184 = $183 << 6;
         $185 = HEAP32[$zIn>>2]|0;
         $186 = (($185) + 1|0);
         HEAP32[$zIn>>2] = $186;
         $187 = HEAP8[$185>>0]|0;
         $188 = $187&255;
         $189 = 63 & $188;
         $190 = (($184) + ($189))|0;
         HEAP32[$c>>2] = $190;
        }
        $191 = ($183>>>0)<(128);
        if (!($191)) {
         $192 = HEAP32[$c>>2]|0;
         $193 = $192 & -2048;
         $194 = ($193|0)==(55296);
         if (!($194)) {
          $195 = HEAP32[$c>>2]|0;
          $196 = $195 & -2;
          $197 = ($196|0)==(65534);
          if (!($197)) {
           break;
          }
         }
        }
        HEAP32[$c>>2] = 65533;
       }
      } while(0);
      $198 = HEAP32[$c>>2]|0;
      $199 = ($198>>>0)<=(65535);
      $200 = HEAP32[$c>>2]|0;
      if ($199) {
       $201 = $200 >>> 8;
       $202 = $201 & 255;
       $203 = $202&255;
       $204 = HEAP32[$z>>2]|0;
       $205 = (($204) + 1|0);
       HEAP32[$z>>2] = $205;
       HEAP8[$204>>0] = $203;
       $206 = HEAP32[$c>>2]|0;
       $207 = $206 & 255;
       $208 = $207&255;
       $209 = HEAP32[$z>>2]|0;
       $210 = (($209) + 1|0);
       HEAP32[$z>>2] = $210;
       HEAP8[$209>>0] = $208;
       continue;
      } else {
       $211 = (($200) - 65536)|0;
       $212 = $211 >>> 18;
       $213 = $212 & 3;
       $214 = (216 + ($213))|0;
       $215 = $214&255;
       $216 = HEAP32[$z>>2]|0;
       $217 = (($216) + 1|0);
       HEAP32[$z>>2] = $217;
       HEAP8[$216>>0] = $215;
       $218 = HEAP32[$c>>2]|0;
       $219 = $218 >>> 10;
       $220 = $219 & 63;
       $221 = HEAP32[$c>>2]|0;
       $222 = (($221) - 65536)|0;
       $223 = $222 >>> 10;
       $224 = $223 & 192;
       $225 = (($220) + ($224))|0;
       $226 = $225&255;
       $227 = HEAP32[$z>>2]|0;
       $228 = (($227) + 1|0);
       HEAP32[$z>>2] = $228;
       HEAP8[$227>>0] = $226;
       $229 = HEAP32[$c>>2]|0;
       $230 = $229 >>> 8;
       $231 = $230 & 3;
       $232 = (220 + ($231))|0;
       $233 = $232&255;
       $234 = HEAP32[$z>>2]|0;
       $235 = (($234) + 1|0);
       HEAP32[$z>>2] = $235;
       HEAP8[$234>>0] = $233;
       $236 = HEAP32[$c>>2]|0;
       $237 = $236 & 255;
       $238 = $237&255;
       $239 = HEAP32[$z>>2]|0;
       $240 = (($239) + 1|0);
       HEAP32[$z>>2] = $240;
       HEAP8[$239>>0] = $238;
       continue;
      }
     }
    }
   } while(0);
   $241 = HEAP32[$z>>2]|0;
   $242 = HEAP32[$zOut>>2]|0;
   $243 = $241;
   $244 = $242;
   $245 = (($243) - ($244))|0;
   $246 = HEAP32[$1>>2]|0;
   $247 = (($246) + 12|0);
   HEAP32[$247>>2] = $245;
   $248 = HEAP32[$z>>2]|0;
   $249 = (($248) + 1|0);
   HEAP32[$z>>2] = $249;
   HEAP8[$248>>0] = 0;
  } else {
   $250 = HEAP32[$1>>2]|0;
   $251 = (($250) + 10|0);
   $252 = HEAP8[$251>>0]|0;
   $253 = $252&255;
   $254 = ($253|0)==(2);
   L24: do {
    if ($254) {
     while(1) {
      $255 = HEAP32[$zIn>>2]|0;
      $256 = HEAP32[$zTerm>>2]|0;
      $257 = ($255>>>0)<($256>>>0);
      if (!($257)) {
       break L24;
      }
      $258 = HEAP32[$zIn>>2]|0;
      $259 = (($258) + 1|0);
      HEAP32[$zIn>>2] = $259;
      $260 = HEAP8[$258>>0]|0;
      $261 = $260&255;
      HEAP32[$c>>2] = $261;
      $262 = HEAP32[$zIn>>2]|0;
      $263 = (($262) + 1|0);
      HEAP32[$zIn>>2] = $263;
      $264 = HEAP8[$262>>0]|0;
      $265 = $264&255;
      $266 = $265 << 8;
      $267 = HEAP32[$c>>2]|0;
      $268 = (($267) + ($266))|0;
      HEAP32[$c>>2] = $268;
      $269 = HEAP32[$c>>2]|0;
      $270 = ($269>>>0)>=(55296);
      $271 = HEAP32[$c>>2]|0;
      $272 = ($271>>>0)<(57344);
      $or$cond = $270 & $272;
      if ($or$cond) {
       $273 = HEAP32[$zIn>>2]|0;
       $274 = HEAP32[$zTerm>>2]|0;
       $275 = ($273>>>0)<($274>>>0);
       if ($275) {
        $276 = HEAP32[$zIn>>2]|0;
        $277 = (($276) + 1|0);
        HEAP32[$zIn>>2] = $277;
        $278 = HEAP8[$276>>0]|0;
        $279 = $278&255;
        HEAP32[$c2>>2] = $279;
        $280 = HEAP32[$zIn>>2]|0;
        $281 = (($280) + 1|0);
        HEAP32[$zIn>>2] = $281;
        $282 = HEAP8[$280>>0]|0;
        $283 = $282&255;
        $284 = $283 << 8;
        $285 = HEAP32[$c2>>2]|0;
        $286 = (($285) + ($284))|0;
        HEAP32[$c2>>2] = $286;
        $287 = HEAP32[$c2>>2]|0;
        $288 = $287 & 1023;
        $289 = HEAP32[$c>>2]|0;
        $290 = $289 & 63;
        $291 = $290 << 10;
        $292 = (($288) + ($291))|0;
        $293 = HEAP32[$c>>2]|0;
        $294 = $293 & 960;
        $295 = (($294) + 64)|0;
        $296 = $295 << 10;
        $297 = (($292) + ($296))|0;
        HEAP32[$c>>2] = $297;
       }
      }
      $298 = HEAP32[$c>>2]|0;
      $299 = ($298>>>0)<(128);
      $300 = HEAP32[$c>>2]|0;
      if ($299) {
       $301 = $300 & 255;
       $302 = $301&255;
       $303 = HEAP32[$z>>2]|0;
       $304 = (($303) + 1|0);
       HEAP32[$z>>2] = $304;
       HEAP8[$303>>0] = $302;
       continue;
      }
      $305 = ($300>>>0)<(2048);
      $306 = HEAP32[$c>>2]|0;
      if ($305) {
       $307 = $306 >>> 6;
       $308 = $307 & 31;
       $309 = $308&255;
       $310 = $309&255;
       $311 = (192 + ($310))|0;
       $312 = $311&255;
       $313 = HEAP32[$z>>2]|0;
       $314 = (($313) + 1|0);
       HEAP32[$z>>2] = $314;
       HEAP8[$313>>0] = $312;
       $315 = HEAP32[$c>>2]|0;
       $316 = $315 & 63;
       $317 = $316&255;
       $318 = $317&255;
       $319 = (128 + ($318))|0;
       $320 = $319&255;
       $321 = HEAP32[$z>>2]|0;
       $322 = (($321) + 1|0);
       HEAP32[$z>>2] = $322;
       HEAP8[$321>>0] = $320;
       continue;
      }
      $323 = ($306>>>0)<(65536);
      $324 = HEAP32[$c>>2]|0;
      if ($323) {
       $325 = $324 >>> 12;
       $326 = $325 & 15;
       $327 = $326&255;
       $328 = $327&255;
       $329 = (224 + ($328))|0;
       $330 = $329&255;
       $331 = HEAP32[$z>>2]|0;
       $332 = (($331) + 1|0);
       HEAP32[$z>>2] = $332;
       HEAP8[$331>>0] = $330;
       $333 = HEAP32[$c>>2]|0;
       $334 = $333 >>> 6;
       $335 = $334 & 63;
       $336 = $335&255;
       $337 = $336&255;
       $338 = (128 + ($337))|0;
       $339 = $338&255;
       $340 = HEAP32[$z>>2]|0;
       $341 = (($340) + 1|0);
       HEAP32[$z>>2] = $341;
       HEAP8[$340>>0] = $339;
       $342 = HEAP32[$c>>2]|0;
       $343 = $342 & 63;
       $344 = $343&255;
       $345 = $344&255;
       $346 = (128 + ($345))|0;
       $347 = $346&255;
       $348 = HEAP32[$z>>2]|0;
       $349 = (($348) + 1|0);
       HEAP32[$z>>2] = $349;
       HEAP8[$348>>0] = $347;
       continue;
      } else {
       $350 = $324 >>> 18;
       $351 = $350 & 7;
       $352 = $351&255;
       $353 = $352&255;
       $354 = (240 + ($353))|0;
       $355 = $354&255;
       $356 = HEAP32[$z>>2]|0;
       $357 = (($356) + 1|0);
       HEAP32[$z>>2] = $357;
       HEAP8[$356>>0] = $355;
       $358 = HEAP32[$c>>2]|0;
       $359 = $358 >>> 12;
       $360 = $359 & 63;
       $361 = $360&255;
       $362 = $361&255;
       $363 = (128 + ($362))|0;
       $364 = $363&255;
       $365 = HEAP32[$z>>2]|0;
       $366 = (($365) + 1|0);
       HEAP32[$z>>2] = $366;
       HEAP8[$365>>0] = $364;
       $367 = HEAP32[$c>>2]|0;
       $368 = $367 >>> 6;
       $369 = $368 & 63;
       $370 = $369&255;
       $371 = $370&255;
       $372 = (128 + ($371))|0;
       $373 = $372&255;
       $374 = HEAP32[$z>>2]|0;
       $375 = (($374) + 1|0);
       HEAP32[$z>>2] = $375;
       HEAP8[$374>>0] = $373;
       $376 = HEAP32[$c>>2]|0;
       $377 = $376 & 63;
       $378 = $377&255;
       $379 = $378&255;
       $380 = (128 + ($379))|0;
       $381 = $380&255;
       $382 = HEAP32[$z>>2]|0;
       $383 = (($382) + 1|0);
       HEAP32[$z>>2] = $383;
       HEAP8[$382>>0] = $381;
       continue;
      }
     }
    } else {
     while(1) {
      $384 = HEAP32[$zIn>>2]|0;
      $385 = HEAP32[$zTerm>>2]|0;
      $386 = ($384>>>0)<($385>>>0);
      if (!($386)) {
       break L24;
      }
      $387 = HEAP32[$zIn>>2]|0;
      $388 = (($387) + 1|0);
      HEAP32[$zIn>>2] = $388;
      $389 = HEAP8[$387>>0]|0;
      $390 = $389&255;
      $391 = $390 << 8;
      HEAP32[$c>>2] = $391;
      $392 = HEAP32[$zIn>>2]|0;
      $393 = (($392) + 1|0);
      HEAP32[$zIn>>2] = $393;
      $394 = HEAP8[$392>>0]|0;
      $395 = $394&255;
      $396 = HEAP32[$c>>2]|0;
      $397 = (($396) + ($395))|0;
      HEAP32[$c>>2] = $397;
      $398 = HEAP32[$c>>2]|0;
      $399 = ($398>>>0)>=(55296);
      $400 = HEAP32[$c>>2]|0;
      $401 = ($400>>>0)<(57344);
      $or$cond3 = $399 & $401;
      if ($or$cond3) {
       $402 = HEAP32[$zIn>>2]|0;
       $403 = HEAP32[$zTerm>>2]|0;
       $404 = ($402>>>0)<($403>>>0);
       if ($404) {
        $405 = HEAP32[$zIn>>2]|0;
        $406 = (($405) + 1|0);
        HEAP32[$zIn>>2] = $406;
        $407 = HEAP8[$405>>0]|0;
        $408 = $407&255;
        $409 = $408 << 8;
        HEAP32[$c21>>2] = $409;
        $410 = HEAP32[$zIn>>2]|0;
        $411 = (($410) + 1|0);
        HEAP32[$zIn>>2] = $411;
        $412 = HEAP8[$410>>0]|0;
        $413 = $412&255;
        $414 = HEAP32[$c21>>2]|0;
        $415 = (($414) + ($413))|0;
        HEAP32[$c21>>2] = $415;
        $416 = HEAP32[$c21>>2]|0;
        $417 = $416 & 1023;
        $418 = HEAP32[$c>>2]|0;
        $419 = $418 & 63;
        $420 = $419 << 10;
        $421 = (($417) + ($420))|0;
        $422 = HEAP32[$c>>2]|0;
        $423 = $422 & 960;
        $424 = (($423) + 64)|0;
        $425 = $424 << 10;
        $426 = (($421) + ($425))|0;
        HEAP32[$c>>2] = $426;
       }
      }
      $427 = HEAP32[$c>>2]|0;
      $428 = ($427>>>0)<(128);
      $429 = HEAP32[$c>>2]|0;
      if ($428) {
       $430 = $429 & 255;
       $431 = $430&255;
       $432 = HEAP32[$z>>2]|0;
       $433 = (($432) + 1|0);
       HEAP32[$z>>2] = $433;
       HEAP8[$432>>0] = $431;
       continue;
      }
      $434 = ($429>>>0)<(2048);
      $435 = HEAP32[$c>>2]|0;
      if ($434) {
       $436 = $435 >>> 6;
       $437 = $436 & 31;
       $438 = $437&255;
       $439 = $438&255;
       $440 = (192 + ($439))|0;
       $441 = $440&255;
       $442 = HEAP32[$z>>2]|0;
       $443 = (($442) + 1|0);
       HEAP32[$z>>2] = $443;
       HEAP8[$442>>0] = $441;
       $444 = HEAP32[$c>>2]|0;
       $445 = $444 & 63;
       $446 = $445&255;
       $447 = $446&255;
       $448 = (128 + ($447))|0;
       $449 = $448&255;
       $450 = HEAP32[$z>>2]|0;
       $451 = (($450) + 1|0);
       HEAP32[$z>>2] = $451;
       HEAP8[$450>>0] = $449;
       continue;
      }
      $452 = ($435>>>0)<(65536);
      $453 = HEAP32[$c>>2]|0;
      if ($452) {
       $454 = $453 >>> 12;
       $455 = $454 & 15;
       $456 = $455&255;
       $457 = $456&255;
       $458 = (224 + ($457))|0;
       $459 = $458&255;
       $460 = HEAP32[$z>>2]|0;
       $461 = (($460) + 1|0);
       HEAP32[$z>>2] = $461;
       HEAP8[$460>>0] = $459;
       $462 = HEAP32[$c>>2]|0;
       $463 = $462 >>> 6;
       $464 = $463 & 63;
       $465 = $464&255;
       $466 = $465&255;
       $467 = (128 + ($466))|0;
       $468 = $467&255;
       $469 = HEAP32[$z>>2]|0;
       $470 = (($469) + 1|0);
       HEAP32[$z>>2] = $470;
       HEAP8[$469>>0] = $468;
       $471 = HEAP32[$c>>2]|0;
       $472 = $471 & 63;
       $473 = $472&255;
       $474 = $473&255;
       $475 = (128 + ($474))|0;
       $476 = $475&255;
       $477 = HEAP32[$z>>2]|0;
       $478 = (($477) + 1|0);
       HEAP32[$z>>2] = $478;
       HEAP8[$477>>0] = $476;
       continue;
      } else {
       $479 = $453 >>> 18;
       $480 = $479 & 7;
       $481 = $480&255;
       $482 = $481&255;
       $483 = (240 + ($482))|0;
       $484 = $483&255;
       $485 = HEAP32[$z>>2]|0;
       $486 = (($485) + 1|0);
       HEAP32[$z>>2] = $486;
       HEAP8[$485>>0] = $484;
       $487 = HEAP32[$c>>2]|0;
       $488 = $487 >>> 12;
       $489 = $488 & 63;
       $490 = $489&255;
       $491 = $490&255;
       $492 = (128 + ($491))|0;
       $493 = $492&255;
       $494 = HEAP32[$z>>2]|0;
       $495 = (($494) + 1|0);
       HEAP32[$z>>2] = $495;
       HEAP8[$494>>0] = $493;
       $496 = HEAP32[$c>>2]|0;
       $497 = $496 >>> 6;
       $498 = $497 & 63;
       $499 = $498&255;
       $500 = $499&255;
       $501 = (128 + ($500))|0;
       $502 = $501&255;
       $503 = HEAP32[$z>>2]|0;
       $504 = (($503) + 1|0);
       HEAP32[$z>>2] = $504;
       HEAP8[$503>>0] = $502;
       $505 = HEAP32[$c>>2]|0;
       $506 = $505 & 63;
       $507 = $506&255;
       $508 = $507&255;
       $509 = (128 + ($508))|0;
       $510 = $509&255;
       $511 = HEAP32[$z>>2]|0;
       $512 = (($511) + 1|0);
       HEAP32[$z>>2] = $512;
       HEAP8[$511>>0] = $510;
       continue;
      }
     }
    }
   } while(0);
   $513 = HEAP32[$z>>2]|0;
   $514 = HEAP32[$zOut>>2]|0;
   $515 = $513;
   $516 = $514;
   $517 = (($515) - ($516))|0;
   $518 = HEAP32[$1>>2]|0;
   $519 = (($518) + 12|0);
   HEAP32[$519>>2] = $517;
  }
  $520 = HEAP32[$z>>2]|0;
  HEAP8[$520>>0] = 0;
  $521 = HEAP32[$1>>2]|0;
  $522 = (($521) + 8|0);
  $523 = HEAP16[$522>>1]|0;
  $524 = $523&65535;
  HEAP32[$c>>2] = $524;
  $525 = HEAP32[$1>>2]|0;
  _sqlite3VdbeMemRelease($525);
  $526 = HEAP32[$c>>2]|0;
  $527 = $526 & 31;
  $528 = 514 | $527;
  $529 = $528&65535;
  $530 = HEAP32[$1>>2]|0;
  $531 = (($530) + 8|0);
  HEAP16[$531>>1] = $529;
  $532 = HEAP8[$2>>0]|0;
  $533 = HEAP32[$1>>2]|0;
  $534 = (($533) + 10|0);
  HEAP8[$534>>0] = $532;
  $535 = HEAP32[$zOut>>2]|0;
  $536 = HEAP32[$1>>2]|0;
  $537 = (($536) + 16|0);
  HEAP32[$537>>2] = $535;
  $538 = HEAP32[$1>>2]|0;
  $539 = (($538) + 16|0);
  $540 = HEAP32[$539>>2]|0;
  $541 = HEAP32[$1>>2]|0;
  $542 = (($541) + 20|0);
  HEAP32[$542>>2] = $540;
  $543 = HEAP32[$1>>2]|0;
  $544 = (($543) + 32|0);
  $545 = HEAP32[$544>>2]|0;
  $546 = HEAP32[$1>>2]|0;
  $547 = (($546) + 16|0);
  $548 = HEAP32[$547>>2]|0;
  $549 = (_sqlite3DbMallocSize($545,$548)|0);
  $550 = HEAP32[$1>>2]|0;
  $551 = (($550) + 24|0);
  HEAP32[$551>>2] = $549;
 }
 HEAP32[$0>>2] = 0;
 $552 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($552|0);
}
function _sqlite3VdbeMemMakeWriteable($pMem) {
 $pMem = $pMem|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $f = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $1 = sp + 4|0;
 $f = sp;
 HEAP32[$1>>2] = $pMem;
 $2 = HEAP32[$1>>2]|0;
 $3 = (($2) + 8|0);
 $4 = HEAP16[$3>>1]|0;
 $5 = $4&65535;
 $6 = $5 & 16384;
 $7 = ($6|0)!=(0);
 if ($7) {
  $8 = HEAP32[$1>>2]|0;
  (_sqlite3VdbeMemExpandBlob($8)|0);
 }
 $9 = HEAP32[$1>>2]|0;
 $10 = (($9) + 8|0);
 $11 = HEAP16[$10>>1]|0;
 $12 = $11&65535;
 HEAP32[$f>>2] = $12;
 $13 = HEAP32[$f>>2]|0;
 $14 = $13 & 18;
 $15 = ($14|0)!=(0);
 do {
  if ($15) {
   $16 = HEAP32[$1>>2]|0;
   $17 = (($16) + 24|0);
   $18 = HEAP32[$17>>2]|0;
   $19 = ($18|0)==(0);
   if (!($19)) {
    $20 = HEAP32[$1>>2]|0;
    $21 = (($20) + 16|0);
    $22 = HEAP32[$21>>2]|0;
    $23 = HEAP32[$1>>2]|0;
    $24 = (($23) + 20|0);
    $25 = HEAP32[$24>>2]|0;
    $26 = ($22|0)!=($25|0);
    if (!($26)) {
     break;
    }
   }
   $27 = HEAP32[$1>>2]|0;
   $28 = HEAP32[$1>>2]|0;
   $29 = (($28) + 12|0);
   $30 = HEAP32[$29>>2]|0;
   $31 = (($30) + 2)|0;
   $32 = (_sqlite3VdbeMemGrow($27,$31,1)|0);
   $33 = ($32|0)!=(0);
   if (!($33)) {
    $34 = HEAP32[$1>>2]|0;
    $35 = (($34) + 12|0);
    $36 = HEAP32[$35>>2]|0;
    $37 = HEAP32[$1>>2]|0;
    $38 = (($37) + 16|0);
    $39 = HEAP32[$38>>2]|0;
    $40 = (($39) + ($36)|0);
    HEAP8[$40>>0] = 0;
    $41 = HEAP32[$1>>2]|0;
    $42 = (($41) + 12|0);
    $43 = HEAP32[$42>>2]|0;
    $44 = (($43) + 1)|0;
    $45 = HEAP32[$1>>2]|0;
    $46 = (($45) + 16|0);
    $47 = HEAP32[$46>>2]|0;
    $48 = (($47) + ($44)|0);
    HEAP8[$48>>0] = 0;
    $49 = HEAP32[$1>>2]|0;
    $50 = (($49) + 8|0);
    $51 = HEAP16[$50>>1]|0;
    $52 = $51&65535;
    $53 = $52 | 512;
    $54 = $53&65535;
    HEAP16[$50>>1] = $54;
    break;
   }
   HEAP32[$0>>2] = 7;
   $55 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($55|0);
  }
 } while(0);
 HEAP32[$0>>2] = 0;
 $55 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($55|0);
}
function _sqlite3DbMallocSize($db,$p) {
 $db = $db|0;
 $p = $p|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0;
 var sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $1 = sp + 4|0;
 $2 = sp;
 HEAP32[$1>>2] = $db;
 HEAP32[$2>>2] = $p;
 $3 = HEAP32[$1>>2]|0;
 $4 = ($3|0)==(0|0);
 if ($4) {
  $5 = HEAP32[$2>>2]|0;
  $6 = (_sqlite3MallocSize($5)|0);
  HEAP32[$0>>2] = $6;
  $18 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($18|0);
 }
 $7 = HEAP32[$1>>2]|0;
 $8 = HEAP32[$2>>2]|0;
 $9 = (_isLookaside($7,$8)|0);
 $10 = ($9|0)!=(0);
 if ($10) {
  $11 = HEAP32[$1>>2]|0;
  $12 = (($11) + 248|0);
  $13 = HEAP16[$12>>1]|0;
  $14 = $13&65535;
  HEAP32[$0>>2] = $14;
  $18 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($18|0);
 } else {
  $15 = HEAP32[((38640 + 48|0))>>2]|0;
  $16 = HEAP32[$2>>2]|0;
  $17 = (FUNCTION_TABLE_ii[$15 & 31]($16)|0);
  HEAP32[$0>>2] = $17;
  $18 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($18|0);
 }
 return 0|0;
}
function _isLookaside($db,$p) {
 $db = $db|0;
 $p = $p|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $1 = sp;
 HEAP32[$0>>2] = $db;
 HEAP32[$1>>2] = $p;
 $2 = HEAP32[$1>>2]|0;
 $3 = HEAP32[$0>>2]|0;
 $4 = (($3) + 248|0);
 $5 = (($4) + 28|0);
 $6 = HEAP32[$5>>2]|0;
 $7 = ($2>>>0)>=($6>>>0);
 if (!($7)) {
  $15 = 0;
  $14 = $15&1;
  STACKTOP = sp;return ($14|0);
 }
 $8 = HEAP32[$1>>2]|0;
 $9 = HEAP32[$0>>2]|0;
 $10 = (($9) + 248|0);
 $11 = (($10) + 32|0);
 $12 = HEAP32[$11>>2]|0;
 $13 = ($8>>>0)<($12>>>0);
 $15 = $13;
 $14 = $15&1;
 STACKTOP = sp;return ($14|0);
}
function _sqlite3VdbeMemGrow($pMem,$n,$bPreserve) {
 $pMem = $pMem|0;
 $n = $n|0;
 $bPreserve = $bPreserve|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0;
 var $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0;
 var label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $1 = sp + 8|0;
 $2 = sp + 4|0;
 $3 = sp;
 HEAP32[$1>>2] = $pMem;
 HEAP32[$2>>2] = $n;
 HEAP32[$3>>2] = $bPreserve;
 $4 = HEAP32[$1>>2]|0;
 $5 = (($4) + 24|0);
 $6 = HEAP32[$5>>2]|0;
 $7 = HEAP32[$2>>2]|0;
 $8 = ($6|0)<($7|0);
 do {
  if ($8) {
   $9 = HEAP32[$2>>2]|0;
   $10 = ($9|0)<(32);
   if ($10) {
    HEAP32[$2>>2] = 32;
   }
   $11 = HEAP32[$3>>2]|0;
   $12 = ($11|0)!=(0);
   if ($12) {
    $13 = HEAP32[$1>>2]|0;
    $14 = (($13) + 24|0);
    $15 = HEAP32[$14>>2]|0;
    $16 = ($15|0)>(0);
    if ($16) {
     $17 = HEAP32[$1>>2]|0;
     $18 = (($17) + 16|0);
     $19 = HEAP32[$18>>2]|0;
     $20 = HEAP32[$1>>2]|0;
     $21 = (($20) + 20|0);
     $22 = HEAP32[$21>>2]|0;
     $23 = ($19|0)==($22|0);
     if ($23) {
      $24 = HEAP32[$1>>2]|0;
      $25 = (($24) + 32|0);
      $26 = HEAP32[$25>>2]|0;
      $27 = HEAP32[$1>>2]|0;
      $28 = (($27) + 16|0);
      $29 = HEAP32[$28>>2]|0;
      $30 = HEAP32[$2>>2]|0;
      $31 = ($30|0)<(0);
      $32 = $31 << 31 >> 31;
      $33 = (_sqlite3DbReallocOrFree($26,$29,$30,$32)|0);
      $34 = HEAP32[$1>>2]|0;
      $35 = (($34) + 20|0);
      HEAP32[$35>>2] = $33;
      $36 = HEAP32[$1>>2]|0;
      $37 = (($36) + 16|0);
      HEAP32[$37>>2] = $33;
      HEAP32[$3>>2] = 0;
     } else {
      label = 8;
     }
    } else {
     label = 8;
    }
   } else {
    label = 8;
   }
   if ((label|0) == 8) {
    $38 = HEAP32[$1>>2]|0;
    $39 = (($38) + 24|0);
    $40 = HEAP32[$39>>2]|0;
    $41 = ($40|0)>(0);
    if ($41) {
     $42 = HEAP32[$1>>2]|0;
     $43 = (($42) + 32|0);
     $44 = HEAP32[$43>>2]|0;
     $45 = HEAP32[$1>>2]|0;
     $46 = (($45) + 20|0);
     $47 = HEAP32[$46>>2]|0;
     _sqlite3DbFree($44,$47);
    }
    $48 = HEAP32[$1>>2]|0;
    $49 = (($48) + 32|0);
    $50 = HEAP32[$49>>2]|0;
    $51 = HEAP32[$2>>2]|0;
    $52 = ($51|0)<(0);
    $53 = $52 << 31 >> 31;
    $54 = (_sqlite3DbMallocRaw($50,$51,$53)|0);
    $55 = HEAP32[$1>>2]|0;
    $56 = (($55) + 20|0);
    HEAP32[$56>>2] = $54;
   }
   $57 = HEAP32[$1>>2]|0;
   $58 = (($57) + 20|0);
   $59 = HEAP32[$58>>2]|0;
   $60 = ($59|0)==(0|0);
   $61 = HEAP32[$1>>2]|0;
   if (!($60)) {
    $66 = (($61) + 32|0);
    $67 = HEAP32[$66>>2]|0;
    $68 = HEAP32[$1>>2]|0;
    $69 = (($68) + 20|0);
    $70 = HEAP32[$69>>2]|0;
    $71 = (_sqlite3DbMallocSize($67,$70)|0);
    $72 = HEAP32[$1>>2]|0;
    $73 = (($72) + 24|0);
    HEAP32[$73>>2] = $71;
    break;
   }
   _sqlite3VdbeMemSetNull($61);
   $62 = HEAP32[$1>>2]|0;
   $63 = (($62) + 16|0);
   HEAP32[$63>>2] = 0;
   $64 = HEAP32[$1>>2]|0;
   $65 = (($64) + 24|0);
   HEAP32[$65>>2] = 0;
   HEAP32[$0>>2] = 7;
   $119 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($119|0);
  }
 } while(0);
 $74 = HEAP32[$3>>2]|0;
 $75 = ($74|0)!=(0);
 if ($75) {
  $76 = HEAP32[$1>>2]|0;
  $77 = (($76) + 16|0);
  $78 = HEAP32[$77>>2]|0;
  $79 = ($78|0)!=(0|0);
  if ($79) {
   $80 = HEAP32[$1>>2]|0;
   $81 = (($80) + 16|0);
   $82 = HEAP32[$81>>2]|0;
   $83 = HEAP32[$1>>2]|0;
   $84 = (($83) + 20|0);
   $85 = HEAP32[$84>>2]|0;
   $86 = ($82|0)!=($85|0);
   if ($86) {
    $87 = HEAP32[$1>>2]|0;
    $88 = (($87) + 20|0);
    $89 = HEAP32[$88>>2]|0;
    $90 = HEAP32[$1>>2]|0;
    $91 = (($90) + 16|0);
    $92 = HEAP32[$91>>2]|0;
    $93 = HEAP32[$1>>2]|0;
    $94 = (($93) + 12|0);
    $95 = HEAP32[$94>>2]|0;
    _memcpy(($89|0),($92|0),($95|0))|0;
   }
  }
 }
 $96 = HEAP32[$1>>2]|0;
 $97 = (($96) + 8|0);
 $98 = HEAP16[$97>>1]|0;
 $99 = $98&65535;
 $100 = $99 & 1024;
 $101 = ($100|0)!=(0);
 if ($101) {
  $102 = HEAP32[$1>>2]|0;
  $103 = (($102) + 36|0);
  $104 = HEAP32[$103>>2]|0;
  $105 = HEAP32[$1>>2]|0;
  $106 = (($105) + 16|0);
  $107 = HEAP32[$106>>2]|0;
  FUNCTION_TABLE_vi[$104 & 31]($107);
 }
 $108 = HEAP32[$1>>2]|0;
 $109 = (($108) + 20|0);
 $110 = HEAP32[$109>>2]|0;
 $111 = HEAP32[$1>>2]|0;
 $112 = (($111) + 16|0);
 HEAP32[$112>>2] = $110;
 $113 = HEAP32[$1>>2]|0;
 $114 = (($113) + 8|0);
 $115 = HEAP16[$114>>1]|0;
 $116 = $115&65535;
 $117 = $116 & -7169;
 $118 = $117&65535;
 HEAP16[$114>>1] = $118;
 HEAP32[$0>>2] = 0;
 $119 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($119|0);
}
function _sqlite3DbReallocOrFree($db,$p,$0,$1) {
 $db = $db|0;
 $p = $p|0;
 $0 = $0|0;
 $1 = $1|0;
 var $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0;
 var $9 = 0, $pNew = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $2 = sp + 16|0;
 $3 = sp + 12|0;
 $4 = sp;
 $pNew = sp + 8|0;
 HEAP32[$2>>2] = $db;
 HEAP32[$3>>2] = $p;
 $5 = $4;
 $6 = $5;
 HEAP32[$6>>2] = $0;
 $7 = (($5) + 4)|0;
 $8 = $7;
 HEAP32[$8>>2] = $1;
 $9 = HEAP32[$2>>2]|0;
 $10 = HEAP32[$3>>2]|0;
 $11 = $4;
 $12 = $11;
 $13 = HEAP32[$12>>2]|0;
 $14 = (($11) + 4)|0;
 $15 = $14;
 $16 = HEAP32[$15>>2]|0;
 $17 = (_sqlite3DbRealloc($9,$10,$13,$16)|0);
 HEAP32[$pNew>>2] = $17;
 $18 = HEAP32[$pNew>>2]|0;
 $19 = ($18|0)!=(0|0);
 if ($19) {
  $22 = HEAP32[$pNew>>2]|0;
  STACKTOP = sp;return ($22|0);
 }
 $20 = HEAP32[$2>>2]|0;
 $21 = HEAP32[$3>>2]|0;
 _sqlite3DbFree($20,$21);
 $22 = HEAP32[$pNew>>2]|0;
 STACKTOP = sp;return ($22|0);
}
function _sqlite3DbRealloc($db,$p,$0,$1) {
 $db = $db|0;
 $p = $p|0;
 $0 = $0|0;
 $1 = $1|0;
 var $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0;
 var $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0;
 var $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0;
 var $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $8 = 0, $9 = 0, $pNew = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $2 = sp + 20|0;
 $3 = sp + 16|0;
 $4 = sp + 12|0;
 $5 = sp;
 $pNew = sp + 8|0;
 HEAP32[$3>>2] = $db;
 HEAP32[$4>>2] = $p;
 $6 = $5;
 $7 = $6;
 HEAP32[$7>>2] = $0;
 $8 = (($6) + 4)|0;
 $9 = $8;
 HEAP32[$9>>2] = $1;
 HEAP32[$pNew>>2] = 0;
 $10 = HEAP32[$3>>2]|0;
 $11 = (($10) + 64|0);
 $12 = HEAP8[$11>>0]|0;
 $13 = $12&255;
 $14 = ($13|0)==(0);
 do {
  if ($14) {
   $15 = HEAP32[$4>>2]|0;
   $16 = ($15|0)==(0|0);
   $17 = HEAP32[$3>>2]|0;
   if ($16) {
    $18 = $5;
    $19 = $18;
    $20 = HEAP32[$19>>2]|0;
    $21 = (($18) + 4)|0;
    $22 = $21;
    $23 = HEAP32[$22>>2]|0;
    $24 = (_sqlite3DbMallocRaw($17,$20,$23)|0);
    HEAP32[$2>>2] = $24;
    $75 = HEAP32[$2>>2]|0;
    STACKTOP = sp;return ($75|0);
   }
   $25 = HEAP32[$4>>2]|0;
   $26 = (_isLookaside($17,$25)|0);
   $27 = ($26|0)!=(0);
   if (!($27)) {
    $62 = HEAP32[$4>>2]|0;
    $63 = $5;
    $64 = $63;
    $65 = HEAP32[$64>>2]|0;
    $66 = (($63) + 4)|0;
    $67 = $66;
    $68 = HEAP32[$67>>2]|0;
    $69 = (_sqlite3_realloc64($62,$65,$68)|0);
    HEAP32[$pNew>>2] = $69;
    $70 = HEAP32[$pNew>>2]|0;
    $71 = ($70|0)!=(0|0);
    if ($71) {
     break;
    }
    $72 = HEAP32[$3>>2]|0;
    $73 = (($72) + 64|0);
    HEAP8[$73>>0] = 1;
    break;
   }
   $28 = $5;
   $29 = $28;
   $30 = HEAP32[$29>>2]|0;
   $31 = (($28) + 4)|0;
   $32 = $31;
   $33 = HEAP32[$32>>2]|0;
   $34 = HEAP32[$3>>2]|0;
   $35 = (($34) + 248|0);
   $36 = HEAP16[$35>>1]|0;
   $37 = $36&65535;
   $38 = ($33>>>0)<(0);
   $39 = ($30>>>0)<=($37>>>0);
   $40 = ($33|0)==(0);
   $41 = $40 & $39;
   $42 = $38 | $41;
   if ($42) {
    $43 = HEAP32[$4>>2]|0;
    HEAP32[$2>>2] = $43;
    $75 = HEAP32[$2>>2]|0;
    STACKTOP = sp;return ($75|0);
   } else {
    $44 = HEAP32[$3>>2]|0;
    $45 = $5;
    $46 = $45;
    $47 = HEAP32[$46>>2]|0;
    $48 = (($45) + 4)|0;
    $49 = $48;
    $50 = HEAP32[$49>>2]|0;
    $51 = (_sqlite3DbMallocRaw($44,$47,$50)|0);
    HEAP32[$pNew>>2] = $51;
    $52 = HEAP32[$pNew>>2]|0;
    $53 = ($52|0)!=(0|0);
    if (!($53)) {
     break;
    }
    $54 = HEAP32[$pNew>>2]|0;
    $55 = HEAP32[$4>>2]|0;
    $56 = HEAP32[$3>>2]|0;
    $57 = (($56) + 248|0);
    $58 = HEAP16[$57>>1]|0;
    $59 = $58&65535;
    _memcpy(($54|0),($55|0),($59|0))|0;
    $60 = HEAP32[$3>>2]|0;
    $61 = HEAP32[$4>>2]|0;
    _sqlite3DbFree($60,$61);
    break;
   }
  }
 } while(0);
 $74 = HEAP32[$pNew>>2]|0;
 HEAP32[$2>>2] = $74;
 $75 = HEAP32[$2>>2]|0;
 STACKTOP = sp;return ($75|0);
}
function _functionDestroy($db,$p) {
 $db = $db|0;
 $p = $p|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0;
 var $9 = 0, $pDestructor = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $1 = sp + 4|0;
 $pDestructor = sp;
 HEAP32[$0>>2] = $db;
 HEAP32[$1>>2] = $p;
 $2 = HEAP32[$1>>2]|0;
 $3 = (($2) + 32|0);
 $4 = HEAP32[$3>>2]|0;
 HEAP32[$pDestructor>>2] = $4;
 $5 = HEAP32[$pDestructor>>2]|0;
 $6 = ($5|0)!=(0|0);
 if (!($6)) {
  STACKTOP = sp;return;
 }
 $7 = HEAP32[$pDestructor>>2]|0;
 $8 = HEAP32[$7>>2]|0;
 $9 = (($8) + -1)|0;
 HEAP32[$7>>2] = $9;
 $10 = HEAP32[$pDestructor>>2]|0;
 $11 = HEAP32[$10>>2]|0;
 $12 = ($11|0)==(0);
 if (!($12)) {
  STACKTOP = sp;return;
 }
 $13 = HEAP32[$pDestructor>>2]|0;
 $14 = (($13) + 4|0);
 $15 = HEAP32[$14>>2]|0;
 $16 = HEAP32[$pDestructor>>2]|0;
 $17 = (($16) + 8|0);
 $18 = HEAP32[$17>>2]|0;
 FUNCTION_TABLE_vi[$15 & 31]($18);
 $19 = HEAP32[$0>>2]|0;
 $20 = HEAP32[$pDestructor>>2]|0;
 _sqlite3DbFree($19,$20);
 STACKTOP = sp;return;
}
function _connectionIsBusy($db) {
 $db = $db|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $3 = 0;
 var $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $j = 0, $pBt = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $1 = sp + 8|0;
 $j = sp + 4|0;
 $pBt = sp;
 HEAP32[$1>>2] = $db;
 $2 = HEAP32[$1>>2]|0;
 $3 = (($2) + 4|0);
 $4 = HEAP32[$3>>2]|0;
 $5 = ($4|0)!=(0|0);
 if ($5) {
  HEAP32[$0>>2] = 1;
  $25 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($25|0);
 }
 HEAP32[$j>>2] = 0;
 while(1) {
  $6 = HEAP32[$j>>2]|0;
  $7 = HEAP32[$1>>2]|0;
  $8 = (($7) + 20|0);
  $9 = HEAP32[$8>>2]|0;
  $10 = ($6|0)<($9|0);
  if (!($10)) {
   label = 9;
   break;
  }
  $11 = HEAP32[$j>>2]|0;
  $12 = HEAP32[$1>>2]|0;
  $13 = (($12) + 16|0);
  $14 = HEAP32[$13>>2]|0;
  $15 = (($14) + ($11<<4)|0);
  $16 = (($15) + 4|0);
  $17 = HEAP32[$16>>2]|0;
  HEAP32[$pBt>>2] = $17;
  $18 = HEAP32[$pBt>>2]|0;
  $19 = ($18|0)!=(0|0);
  if ($19) {
   $20 = HEAP32[$pBt>>2]|0;
   $21 = (_sqlite3BtreeIsInBackup($20)|0);
   $22 = ($21|0)!=(0);
   if ($22) {
    label = 7;
    break;
   }
  }
  $23 = HEAP32[$j>>2]|0;
  $24 = (($23) + 1)|0;
  HEAP32[$j>>2] = $24;
 }
 if ((label|0) == 7) {
  HEAP32[$0>>2] = 1;
  $25 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($25|0);
 }
 else if ((label|0) == 9) {
  HEAP32[$0>>2] = 0;
  $25 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($25|0);
 }
 return 0|0;
}
function _sqlite3RollbackAll($db,$tripCode) {
 $db = $db|0;
 $tripCode = $tripCode|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0;
 var $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $9 = 0, $i = 0, $inTrans = 0, $p = 0, $schemaChange = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 20|0;
 $1 = sp + 16|0;
 $i = sp + 12|0;
 $inTrans = sp + 8|0;
 $schemaChange = sp + 4|0;
 $p = sp;
 HEAP32[$0>>2] = $db;
 HEAP32[$1>>2] = $tripCode;
 HEAP32[$inTrans>>2] = 0;
 _sqlite3BeginBenignMalloc();
 $2 = HEAP32[$0>>2]|0;
 _sqlite3BtreeEnterAll($2);
 $3 = HEAP32[$0>>2]|0;
 $4 = (($3) + 24|0);
 $5 = HEAP32[$4>>2]|0;
 $6 = $5 & 2;
 $7 = ($6|0)!=(0);
 if ($7) {
  $8 = HEAP32[$0>>2]|0;
  $9 = (($8) + 140|0);
  $10 = (($9) + 5|0);
  $11 = HEAP8[$10>>0]|0;
  $12 = $11&255;
  $13 = ($12|0)==(0);
  $15 = $13;
 } else {
  $15 = 0;
 }
 $14 = $15&1;
 HEAP32[$schemaChange>>2] = $14;
 HEAP32[$i>>2] = 0;
 while(1) {
  $16 = HEAP32[$i>>2]|0;
  $17 = HEAP32[$0>>2]|0;
  $18 = (($17) + 20|0);
  $19 = HEAP32[$18>>2]|0;
  $20 = ($16|0)<($19|0);
  if (!($20)) {
   break;
  }
  $21 = HEAP32[$i>>2]|0;
  $22 = HEAP32[$0>>2]|0;
  $23 = (($22) + 16|0);
  $24 = HEAP32[$23>>2]|0;
  $25 = (($24) + ($21<<4)|0);
  $26 = (($25) + 4|0);
  $27 = HEAP32[$26>>2]|0;
  HEAP32[$p>>2] = $27;
  $28 = HEAP32[$p>>2]|0;
  $29 = ($28|0)!=(0|0);
  if ($29) {
   $30 = HEAP32[$p>>2]|0;
   $31 = (_sqlite3BtreeIsInTrans($30)|0);
   $32 = ($31|0)!=(0);
   if ($32) {
    HEAP32[$inTrans>>2] = 1;
   }
   $33 = HEAP32[$p>>2]|0;
   $34 = HEAP32[$1>>2]|0;
   $35 = HEAP32[$schemaChange>>2]|0;
   $36 = ($35|0)!=(0);
   $37 = $36 ^ 1;
   $38 = $37&1;
   (_sqlite3BtreeRollback($33,$34,$38)|0);
  }
  $39 = HEAP32[$i>>2]|0;
  $40 = (($39) + 1)|0;
  HEAP32[$i>>2] = $40;
 }
 $41 = HEAP32[$0>>2]|0;
 (_sqlite3VtabRollback($41)|0);
 _sqlite3EndBenignMalloc();
 $42 = HEAP32[$0>>2]|0;
 $43 = (($42) + 24|0);
 $44 = HEAP32[$43>>2]|0;
 $45 = $44 & 2;
 $46 = ($45|0)!=(0);
 if ($46) {
  $47 = HEAP32[$0>>2]|0;
  $48 = (($47) + 140|0);
  $49 = (($48) + 5|0);
  $50 = HEAP8[$49>>0]|0;
  $51 = $50&255;
  $52 = ($51|0)==(0);
  if ($52) {
   $53 = HEAP32[$0>>2]|0;
   _sqlite3ExpirePreparedStatements($53);
   $54 = HEAP32[$0>>2]|0;
   _sqlite3ResetAllSchemasOfConnection($54);
  }
 }
 $55 = HEAP32[$0>>2]|0;
 $56 = (($55) + 504|0);
 $57 = $56;
 $58 = $57;
 HEAP32[$58>>2] = 0;
 $59 = (($57) + 4)|0;
 $60 = $59;
 HEAP32[$60>>2] = 0;
 $61 = HEAP32[$0>>2]|0;
 $62 = (($61) + 512|0);
 $63 = $62;
 $64 = $63;
 HEAP32[$64>>2] = 0;
 $65 = (($63) + 4)|0;
 $66 = $65;
 HEAP32[$66>>2] = 0;
 $67 = HEAP32[$0>>2]|0;
 $68 = (($67) + 24|0);
 $69 = HEAP32[$68>>2]|0;
 $70 = $69 & -16777217;
 HEAP32[$68>>2] = $70;
 $71 = HEAP32[$0>>2]|0;
 $72 = (($71) + 200|0);
 $73 = HEAP32[$72>>2]|0;
 $74 = ($73|0)!=(0|0);
 if (!($74)) {
  STACKTOP = sp;return;
 }
 $75 = HEAP32[$inTrans>>2]|0;
 $76 = ($75|0)!=(0);
 if (!($76)) {
  $77 = HEAP32[$0>>2]|0;
  $78 = (($77) + 62|0);
  $79 = HEAP8[$78>>0]|0;
  $80 = ($79<<24>>24)!=(0);
  if ($80) {
   STACKTOP = sp;return;
  }
 }
 $81 = HEAP32[$0>>2]|0;
 $82 = (($81) + 200|0);
 $83 = HEAP32[$82>>2]|0;
 $84 = HEAP32[$0>>2]|0;
 $85 = (($84) + 196|0);
 $86 = HEAP32[$85>>2]|0;
 FUNCTION_TABLE_vi[$83 & 31]($86);
 STACKTOP = sp;return;
}
function _sqlite3CloseSavepoints($db) {
 $db = $db|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0;
 var $pTmp = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $pTmp = sp;
 HEAP32[$0>>2] = $db;
 while(1) {
  $1 = HEAP32[$0>>2]|0;
  $2 = (($1) + 488|0);
  $3 = HEAP32[$2>>2]|0;
  $4 = ($3|0)!=(0|0);
  $5 = HEAP32[$0>>2]|0;
  if (!($4)) {
   break;
  }
  $6 = (($5) + 488|0);
  $7 = HEAP32[$6>>2]|0;
  HEAP32[$pTmp>>2] = $7;
  $8 = HEAP32[$pTmp>>2]|0;
  $9 = (($8) + 24|0);
  $10 = HEAP32[$9>>2]|0;
  $11 = HEAP32[$0>>2]|0;
  $12 = (($11) + 488|0);
  HEAP32[$12>>2] = $10;
  $13 = HEAP32[$0>>2]|0;
  $14 = HEAP32[$pTmp>>2]|0;
  _sqlite3DbFree($13,$14);
 }
 $15 = (($5) + 496|0);
 HEAP32[$15>>2] = 0;
 $16 = HEAP32[$0>>2]|0;
 $17 = (($16) + 500|0);
 HEAP32[$17>>2] = 0;
 $18 = HEAP32[$0>>2]|0;
 $19 = (($18) + 69|0);
 HEAP8[$19>>0] = 0;
 STACKTOP = sp;return;
}
function _sqlite3BtreeClose($p) {
 $p = $p|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $8 = 0, $9 = 0, $pBt = 0, $pCur = 0, $pTmp = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $pBt = sp + 8|0;
 $pCur = sp + 4|0;
 $pTmp = sp;
 HEAP32[$0>>2] = $p;
 $1 = HEAP32[$0>>2]|0;
 $2 = (($1) + 4|0);
 $3 = HEAP32[$2>>2]|0;
 HEAP32[$pBt>>2] = $3;
 $4 = HEAP32[$0>>2]|0;
 _sqlite3BtreeEnter($4);
 $5 = HEAP32[$pBt>>2]|0;
 $6 = (($5) + 8|0);
 $7 = HEAP32[$6>>2]|0;
 HEAP32[$pCur>>2] = $7;
 while(1) {
  $8 = HEAP32[$pCur>>2]|0;
  $9 = ($8|0)!=(0|0);
  if (!($9)) {
   break;
  }
  $10 = HEAP32[$pCur>>2]|0;
  HEAP32[$pTmp>>2] = $10;
  $11 = HEAP32[$pCur>>2]|0;
  $12 = (($11) + 8|0);
  $13 = HEAP32[$12>>2]|0;
  HEAP32[$pCur>>2] = $13;
  $14 = HEAP32[$pTmp>>2]|0;
  $15 = HEAP32[$14>>2]|0;
  $16 = HEAP32[$0>>2]|0;
  $17 = ($15|0)==($16|0);
  if (!($17)) {
   continue;
  }
  $18 = HEAP32[$pTmp>>2]|0;
  (_sqlite3BtreeCloseCursor($18)|0);
 }
 $19 = HEAP32[$0>>2]|0;
 (_sqlite3BtreeRollback($19,0,0)|0);
 $20 = HEAP32[$0>>2]|0;
 $21 = (($20) + 9|0);
 $22 = HEAP8[$21>>0]|0;
 $23 = ($22<<24>>24)!=(0);
 if ($23) {
  $24 = HEAP32[$pBt>>2]|0;
  $25 = (_removeFromSharingList($24)|0);
  $26 = ($25|0)!=(0);
  if ($26) {
   label = 7;
  }
 } else {
  label = 7;
 }
 if ((label|0) == 7) {
  $27 = HEAP32[$pBt>>2]|0;
  $28 = HEAP32[$27>>2]|0;
  (_sqlite3PagerClose($28)|0);
  $29 = HEAP32[$pBt>>2]|0;
  $30 = (($29) + 52|0);
  $31 = HEAP32[$30>>2]|0;
  $32 = ($31|0)!=(0|0);
  if ($32) {
   $33 = HEAP32[$pBt>>2]|0;
   $34 = (($33) + 48|0);
   $35 = HEAP32[$34>>2]|0;
   $36 = ($35|0)!=(0|0);
   if ($36) {
    $37 = HEAP32[$pBt>>2]|0;
    $38 = (($37) + 52|0);
    $39 = HEAP32[$38>>2]|0;
    $40 = HEAP32[$pBt>>2]|0;
    $41 = (($40) + 48|0);
    $42 = HEAP32[$41>>2]|0;
    FUNCTION_TABLE_vi[$39 & 31]($42);
   }
  }
  $43 = HEAP32[$pBt>>2]|0;
  $44 = (($43) + 48|0);
  $45 = HEAP32[$44>>2]|0;
  _sqlite3DbFree(0,$45);
  $46 = HEAP32[$pBt>>2]|0;
  _freeTempSpace($46);
  $47 = HEAP32[$pBt>>2]|0;
  _sqlite3_free($47);
 }
 $48 = HEAP32[$0>>2]|0;
 $49 = (($48) + 24|0);
 $50 = HEAP32[$49>>2]|0;
 $51 = ($50|0)!=(0|0);
 if ($51) {
  $52 = HEAP32[$0>>2]|0;
  $53 = (($52) + 20|0);
  $54 = HEAP32[$53>>2]|0;
  $55 = HEAP32[$0>>2]|0;
  $56 = (($55) + 24|0);
  $57 = HEAP32[$56>>2]|0;
  $58 = (($57) + 20|0);
  HEAP32[$58>>2] = $54;
 }
 $59 = HEAP32[$0>>2]|0;
 $60 = (($59) + 20|0);
 $61 = HEAP32[$60>>2]|0;
 $62 = ($61|0)!=(0|0);
 if (!($62)) {
  $70 = HEAP32[$0>>2]|0;
  _sqlite3_free($70);
  STACKTOP = sp;return 0;
 }
 $63 = HEAP32[$0>>2]|0;
 $64 = (($63) + 24|0);
 $65 = HEAP32[$64>>2]|0;
 $66 = HEAP32[$0>>2]|0;
 $67 = (($66) + 20|0);
 $68 = HEAP32[$67>>2]|0;
 $69 = (($68) + 24|0);
 HEAP32[$69>>2] = $65;
 $70 = HEAP32[$0>>2]|0;
 _sqlite3_free($70);
 STACKTOP = sp;return 0;
}
function _sqlite3VtabUnlockList($db) {
 $db = $db|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $p = 0, $pNext = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $p = sp + 4|0;
 $pNext = sp;
 HEAP32[$0>>2] = $db;
 $1 = HEAP32[$0>>2]|0;
 $2 = (($1) + 332|0);
 $3 = HEAP32[$2>>2]|0;
 HEAP32[$p>>2] = $3;
 $4 = HEAP32[$0>>2]|0;
 $5 = (($4) + 332|0);
 HEAP32[$5>>2] = 0;
 $6 = HEAP32[$p>>2]|0;
 $7 = ($6|0)!=(0|0);
 if (!($7)) {
  STACKTOP = sp;return;
 }
 $8 = HEAP32[$0>>2]|0;
 _sqlite3ExpirePreparedStatements($8);
 while(1) {
  $9 = HEAP32[$p>>2]|0;
  $10 = (($9) + 24|0);
  $11 = HEAP32[$10>>2]|0;
  HEAP32[$pNext>>2] = $11;
  $12 = HEAP32[$p>>2]|0;
  _sqlite3VtabUnlock($12);
  $13 = HEAP32[$pNext>>2]|0;
  HEAP32[$p>>2] = $13;
  $14 = HEAP32[$p>>2]|0;
  $15 = ($14|0)!=(0|0);
  if (!($15)) {
   break;
  }
 }
 STACKTOP = sp;return;
}
function _sqlite3CollapseDatabaseArray($db) {
 $db = $db|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $8 = 0, $9 = 0, $i = 0, $j = 0, $pDb = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $i = sp + 8|0;
 $j = sp + 4|0;
 $pDb = sp;
 HEAP32[$0>>2] = $db;
 HEAP32[$j>>2] = 2;
 HEAP32[$i>>2] = 2;
 while(1) {
  $1 = HEAP32[$i>>2]|0;
  $2 = HEAP32[$0>>2]|0;
  $3 = (($2) + 20|0);
  $4 = HEAP32[$3>>2]|0;
  $5 = ($1|0)<($4|0);
  if (!($5)) {
   break;
  }
  $6 = HEAP32[$i>>2]|0;
  $7 = HEAP32[$0>>2]|0;
  $8 = (($7) + 16|0);
  $9 = HEAP32[$8>>2]|0;
  $10 = (($9) + ($6<<4)|0);
  HEAP32[$pDb>>2] = $10;
  $11 = HEAP32[$pDb>>2]|0;
  $12 = (($11) + 4|0);
  $13 = HEAP32[$12>>2]|0;
  $14 = ($13|0)==(0|0);
  if ($14) {
   $15 = HEAP32[$0>>2]|0;
   $16 = HEAP32[$pDb>>2]|0;
   $17 = HEAP32[$16>>2]|0;
   _sqlite3DbFree($15,$17);
   $18 = HEAP32[$pDb>>2]|0;
   HEAP32[$18>>2] = 0;
  } else {
   $19 = HEAP32[$j>>2]|0;
   $20 = HEAP32[$i>>2]|0;
   $21 = ($19|0)<($20|0);
   if ($21) {
    $22 = HEAP32[$j>>2]|0;
    $23 = HEAP32[$0>>2]|0;
    $24 = (($23) + 16|0);
    $25 = HEAP32[$24>>2]|0;
    $26 = (($25) + ($22<<4)|0);
    $27 = HEAP32[$i>>2]|0;
    $28 = HEAP32[$0>>2]|0;
    $29 = (($28) + 16|0);
    $30 = HEAP32[$29>>2]|0;
    $31 = (($30) + ($27<<4)|0);
    ;HEAP32[$26+0>>2]=HEAP32[$31+0>>2]|0;HEAP32[$26+4>>2]=HEAP32[$31+4>>2]|0;HEAP32[$26+8>>2]=HEAP32[$31+8>>2]|0;HEAP32[$26+12>>2]=HEAP32[$31+12>>2]|0;
   }
   $32 = HEAP32[$j>>2]|0;
   $33 = (($32) + 1)|0;
   HEAP32[$j>>2] = $33;
  }
  $34 = HEAP32[$i>>2]|0;
  $35 = (($34) + 1)|0;
  HEAP32[$i>>2] = $35;
 }
 $36 = HEAP32[$j>>2]|0;
 $37 = HEAP32[$0>>2]|0;
 $38 = (($37) + 16|0);
 $39 = HEAP32[$38>>2]|0;
 $40 = (($39) + ($36<<4)|0);
 $41 = HEAP32[$0>>2]|0;
 $42 = (($41) + 20|0);
 $43 = HEAP32[$42>>2]|0;
 $44 = HEAP32[$j>>2]|0;
 $45 = (($43) - ($44))|0;
 $46 = $45<<4;
 _memset(($40|0),0,($46|0))|0;
 $47 = HEAP32[$j>>2]|0;
 $48 = HEAP32[$0>>2]|0;
 $49 = (($48) + 20|0);
 HEAP32[$49>>2] = $47;
 $50 = HEAP32[$0>>2]|0;
 $51 = (($50) + 20|0);
 $52 = HEAP32[$51>>2]|0;
 $53 = ($52|0)<=(2);
 if (!($53)) {
  STACKTOP = sp;return;
 }
 $54 = HEAP32[$0>>2]|0;
 $55 = (($54) + 16|0);
 $56 = HEAP32[$55>>2]|0;
 $57 = HEAP32[$0>>2]|0;
 $58 = (($57) + 456|0);
 $59 = ($56|0)!=($58|0);
 if (!($59)) {
  STACKTOP = sp;return;
 }
 $60 = HEAP32[$0>>2]|0;
 $61 = (($60) + 456|0);
 $62 = HEAP32[$0>>2]|0;
 $63 = (($62) + 16|0);
 $64 = HEAP32[$63>>2]|0;
 ;HEAP32[$61+0>>2]=HEAP32[$64+0>>2]|0;HEAP32[$61+4>>2]=HEAP32[$64+4>>2]|0;HEAP32[$61+8>>2]=HEAP32[$64+8>>2]|0;HEAP32[$61+12>>2]=HEAP32[$64+12>>2]|0;HEAP32[$61+16>>2]=HEAP32[$64+16>>2]|0;HEAP32[$61+20>>2]=HEAP32[$64+20>>2]|0;HEAP32[$61+24>>2]=HEAP32[$64+24>>2]|0;HEAP32[$61+28>>2]=HEAP32[$64+28>>2]|0;
 $65 = HEAP32[$0>>2]|0;
 $66 = HEAP32[$0>>2]|0;
 $67 = (($66) + 16|0);
 $68 = HEAP32[$67>>2]|0;
 _sqlite3DbFree($65,$68);
 $69 = HEAP32[$0>>2]|0;
 $70 = (($69) + 456|0);
 $71 = HEAP32[$0>>2]|0;
 $72 = (($71) + 16|0);
 HEAP32[$72>>2] = $70;
 STACKTOP = sp;return;
}
function _sqlite3VtabUnlock($pVTab) {
 $pVTab = $pVTab|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0;
 var $7 = 0, $8 = 0, $9 = 0, $db = 0, $p = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $db = sp + 4|0;
 $p = sp;
 HEAP32[$0>>2] = $pVTab;
 $1 = HEAP32[$0>>2]|0;
 $2 = HEAP32[$1>>2]|0;
 HEAP32[$db>>2] = $2;
 $3 = HEAP32[$0>>2]|0;
 $4 = (($3) + 12|0);
 $5 = HEAP32[$4>>2]|0;
 $6 = (($5) + -1)|0;
 HEAP32[$4>>2] = $6;
 $7 = HEAP32[$0>>2]|0;
 $8 = (($7) + 12|0);
 $9 = HEAP32[$8>>2]|0;
 $10 = ($9|0)==(0);
 if (!($10)) {
  STACKTOP = sp;return;
 }
 $11 = HEAP32[$0>>2]|0;
 $12 = (($11) + 8|0);
 $13 = HEAP32[$12>>2]|0;
 HEAP32[$p>>2] = $13;
 $14 = HEAP32[$p>>2]|0;
 $15 = ($14|0)!=(0|0);
 if ($15) {
  $16 = HEAP32[$p>>2]|0;
  $17 = HEAP32[$16>>2]|0;
  $18 = (($17) + 16|0);
  $19 = HEAP32[$18>>2]|0;
  $20 = HEAP32[$p>>2]|0;
  (FUNCTION_TABLE_ii[$19 & 31]($20)|0);
 }
 $21 = HEAP32[$db>>2]|0;
 $22 = HEAP32[$0>>2]|0;
 _sqlite3DbFree($21,$22);
 STACKTOP = sp;return;
}
function _sqlite3BtreeCloseCursor($pCur) {
 $pCur = $pCur|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $i = 0, $pBt = 0, $pBtree = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $pBtree = sp + 8|0;
 $i = sp + 4|0;
 $pBt = sp;
 HEAP32[$0>>2] = $pCur;
 $1 = HEAP32[$0>>2]|0;
 $2 = HEAP32[$1>>2]|0;
 HEAP32[$pBtree>>2] = $2;
 $3 = HEAP32[$pBtree>>2]|0;
 $4 = ($3|0)!=(0|0);
 if (!($4)) {
  STACKTOP = sp;return 0;
 }
 $5 = HEAP32[$0>>2]|0;
 $6 = (($5) + 4|0);
 $7 = HEAP32[$6>>2]|0;
 HEAP32[$pBt>>2] = $7;
 $8 = HEAP32[$pBtree>>2]|0;
 _sqlite3BtreeEnter($8);
 $9 = HEAP32[$0>>2]|0;
 _sqlite3BtreeClearCursor($9);
 $10 = HEAP32[$0>>2]|0;
 $11 = (($10) + 12|0);
 $12 = HEAP32[$11>>2]|0;
 $13 = ($12|0)!=(0|0);
 $14 = HEAP32[$0>>2]|0;
 $15 = (($14) + 8|0);
 $16 = HEAP32[$15>>2]|0;
 if ($13) {
  $17 = HEAP32[$0>>2]|0;
  $18 = (($17) + 12|0);
  $19 = HEAP32[$18>>2]|0;
  $20 = (($19) + 8|0);
  HEAP32[$20>>2] = $16;
 } else {
  $21 = HEAP32[$pBt>>2]|0;
  $22 = (($21) + 8|0);
  HEAP32[$22>>2] = $16;
 }
 $23 = HEAP32[$0>>2]|0;
 $24 = (($23) + 8|0);
 $25 = HEAP32[$24>>2]|0;
 $26 = ($25|0)!=(0|0);
 if ($26) {
  $27 = HEAP32[$0>>2]|0;
  $28 = (($27) + 12|0);
  $29 = HEAP32[$28>>2]|0;
  $30 = HEAP32[$0>>2]|0;
  $31 = (($30) + 8|0);
  $32 = HEAP32[$31>>2]|0;
  $33 = (($32) + 12|0);
  HEAP32[$33>>2] = $29;
 }
 HEAP32[$i>>2] = 0;
 while(1) {
  $34 = HEAP32[$i>>2]|0;
  $35 = HEAP32[$0>>2]|0;
  $36 = (($35) + 76|0);
  $37 = HEAP16[$36>>1]|0;
  $38 = $37 << 16 >> 16;
  $39 = ($34|0)<=($38|0);
  if (!($39)) {
   break;
  }
  $40 = HEAP32[$i>>2]|0;
  $41 = HEAP32[$0>>2]|0;
  $42 = (($41) + 120|0);
  $43 = (($42) + ($40<<2)|0);
  $44 = HEAP32[$43>>2]|0;
  _releasePage($44);
  $45 = HEAP32[$i>>2]|0;
  $46 = (($45) + 1)|0;
  HEAP32[$i>>2] = $46;
 }
 $47 = HEAP32[$pBt>>2]|0;
 _unlockBtreeIfUnused($47);
 $48 = HEAP32[$pBtree>>2]|0;
 $49 = HEAP32[$48>>2]|0;
 $50 = HEAP32[$0>>2]|0;
 $51 = (($50) + 20|0);
 $52 = HEAP32[$51>>2]|0;
 _sqlite3DbFree($49,$52);
 STACKTOP = sp;return 0;
}
function _removeFromSharingList($pBt) {
 $pBt = $pBt|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $pList = 0, $removed = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $pList = sp + 4|0;
 $removed = sp;
 HEAP32[$0>>2] = $pBt;
 HEAP32[$removed>>2] = 0;
 $1 = HEAP32[$0>>2]|0;
 $2 = (($1) + 64|0);
 $3 = HEAP32[$2>>2]|0;
 $4 = (($3) + -1)|0;
 HEAP32[$2>>2] = $4;
 $5 = HEAP32[$0>>2]|0;
 $6 = (($5) + 64|0);
 $7 = HEAP32[$6>>2]|0;
 $8 = ($7|0)<=(0);
 if (!($8)) {
  $32 = HEAP32[$removed>>2]|0;
  STACKTOP = sp;return ($32|0);
 }
 $9 = HEAP32[4232>>2]|0;
 $10 = HEAP32[$0>>2]|0;
 $11 = ($9|0)==($10|0);
 if ($11) {
  $12 = HEAP32[$0>>2]|0;
  $13 = (($12) + 68|0);
  $14 = HEAP32[$13>>2]|0;
  HEAP32[4232>>2] = $14;
 } else {
  $15 = HEAP32[4232>>2]|0;
  HEAP32[$pList>>2] = $15;
  while(1) {
   $16 = HEAP32[$pList>>2]|0;
   $17 = ($16|0)!=(0|0);
   if ($17) {
    $18 = HEAP32[$pList>>2]|0;
    $19 = (($18) + 68|0);
    $20 = HEAP32[$19>>2]|0;
    $21 = HEAP32[$0>>2]|0;
    $22 = ($20|0)!=($21|0);
    $33 = $22;
   } else {
    $33 = 0;
   }
   $23 = HEAP32[$pList>>2]|0;
   if (!($33)) {
    break;
   }
   $24 = (($23) + 68|0);
   $25 = HEAP32[$24>>2]|0;
   HEAP32[$pList>>2] = $25;
  }
  $26 = ($23|0)!=(0|0);
  if ($26) {
   $27 = HEAP32[$0>>2]|0;
   $28 = (($27) + 68|0);
   $29 = HEAP32[$28>>2]|0;
   $30 = HEAP32[$pList>>2]|0;
   $31 = (($30) + 68|0);
   HEAP32[$31>>2] = $29;
  }
 }
 HEAP32[$removed>>2] = 1;
 $32 = HEAP32[$removed>>2]|0;
 STACKTOP = sp;return ($32|0);
}
function _sqlite3BtreeClearCursor($pCur) {
 $pCur = $pCur|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 HEAP32[$0>>2] = $pCur;
 $1 = HEAP32[$0>>2]|0;
 $2 = (($1) + 56|0);
 $3 = HEAP32[$2>>2]|0;
 _sqlite3_free($3);
 $4 = HEAP32[$0>>2]|0;
 $5 = (($4) + 56|0);
 HEAP32[$5>>2] = 0;
 $6 = HEAP32[$0>>2]|0;
 $7 = (($6) + 73|0);
 HEAP8[$7>>0] = 0;
 STACKTOP = sp;return;
}
function _releasePage($pPage) {
 $pPage = $pPage|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 HEAP32[$0>>2] = $pPage;
 $1 = HEAP32[$0>>2]|0;
 $2 = ($1|0)!=(0|0);
 if (!($2)) {
  STACKTOP = sp;return;
 }
 $3 = HEAP32[$0>>2]|0;
 $4 = (($3) + 68|0);
 $5 = HEAP32[$4>>2]|0;
 _sqlite3PagerUnrefNotNull($5);
 STACKTOP = sp;return;
}
function _unlockBtreeIfUnused($pBt) {
 $pBt = $pBt|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $pPage1 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $pPage1 = sp;
 HEAP32[$0>>2] = $pBt;
 $1 = HEAP32[$0>>2]|0;
 $2 = (($1) + 20|0);
 $3 = HEAP8[$2>>0]|0;
 $4 = $3&255;
 $5 = ($4|0)==(0);
 if (!($5)) {
  STACKTOP = sp;return;
 }
 $6 = HEAP32[$0>>2]|0;
 $7 = (($6) + 12|0);
 $8 = HEAP32[$7>>2]|0;
 $9 = ($8|0)!=(0|0);
 if (!($9)) {
  STACKTOP = sp;return;
 }
 $10 = HEAP32[$0>>2]|0;
 $11 = (($10) + 12|0);
 $12 = HEAP32[$11>>2]|0;
 HEAP32[$pPage1>>2] = $12;
 $13 = HEAP32[$0>>2]|0;
 $14 = (($13) + 12|0);
 HEAP32[$14>>2] = 0;
 $15 = HEAP32[$pPage1>>2]|0;
 _releasePage($15);
 STACKTOP = sp;return;
}
function _sqlite3BtreeIsInTrans($p) {
 $p = $p|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 HEAP32[$0>>2] = $p;
 $1 = HEAP32[$0>>2]|0;
 $2 = ($1|0)!=(0|0);
 if (!($2)) {
  $9 = 0;
  $8 = $9&1;
  STACKTOP = sp;return ($8|0);
 }
 $3 = HEAP32[$0>>2]|0;
 $4 = (($3) + 8|0);
 $5 = HEAP8[$4>>0]|0;
 $6 = $5&255;
 $7 = ($6|0)==(2);
 $9 = $7;
 $8 = $9&1;
 STACKTOP = sp;return ($8|0);
}
function _sqlite3VtabRollback($db) {
 $db = $db|0;
 var $0 = 0, $1 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 HEAP32[$0>>2] = $db;
 $1 = HEAP32[$0>>2]|0;
 _callFinaliser($1,68);
 STACKTOP = sp;return 0;
}
function _callFinaliser($db,$offset) {
 $db = $db|0;
 $offset = $offset|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $5 = 0;
 var $6 = 0, $7 = 0, $8 = 0, $9 = 0, $i = 0, $p = 0, $pVTab = 0, $x = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 20|0;
 $1 = sp + 16|0;
 $i = sp + 12|0;
 $pVTab = sp + 8|0;
 $p = sp + 4|0;
 $x = sp;
 HEAP32[$0>>2] = $db;
 HEAP32[$1>>2] = $offset;
 $2 = HEAP32[$0>>2]|0;
 $3 = (($2) + 328|0);
 $4 = HEAP32[$3>>2]|0;
 $5 = ($4|0)!=(0|0);
 if (!($5)) {
  STACKTOP = sp;return;
 }
 HEAP32[$i>>2] = 0;
 while(1) {
  $6 = HEAP32[$i>>2]|0;
  $7 = HEAP32[$0>>2]|0;
  $8 = (($7) + 304|0);
  $9 = HEAP32[$8>>2]|0;
  $10 = ($6|0)<($9|0);
  if (!($10)) {
   break;
  }
  $11 = HEAP32[$i>>2]|0;
  $12 = HEAP32[$0>>2]|0;
  $13 = (($12) + 328|0);
  $14 = HEAP32[$13>>2]|0;
  $15 = (($14) + ($11<<2)|0);
  $16 = HEAP32[$15>>2]|0;
  HEAP32[$pVTab>>2] = $16;
  $17 = HEAP32[$pVTab>>2]|0;
  $18 = (($17) + 8|0);
  $19 = HEAP32[$18>>2]|0;
  HEAP32[$p>>2] = $19;
  $20 = HEAP32[$p>>2]|0;
  $21 = ($20|0)!=(0|0);
  if ($21) {
   $22 = HEAP32[$p>>2]|0;
   $23 = HEAP32[$22>>2]|0;
   $24 = HEAP32[$1>>2]|0;
   $25 = (($23) + ($24)|0);
   $26 = HEAP32[$25>>2]|0;
   HEAP32[$x>>2] = $26;
   $27 = HEAP32[$x>>2]|0;
   $28 = ($27|0)!=(0|0);
   if ($28) {
    $29 = HEAP32[$x>>2]|0;
    $30 = HEAP32[$p>>2]|0;
    (FUNCTION_TABLE_ii[$29 & 31]($30)|0);
   }
  }
  $31 = HEAP32[$pVTab>>2]|0;
  $32 = (($31) + 20|0);
  HEAP32[$32>>2] = 0;
  $33 = HEAP32[$pVTab>>2]|0;
  _sqlite3VtabUnlock($33);
  $34 = HEAP32[$i>>2]|0;
  $35 = (($34) + 1)|0;
  HEAP32[$i>>2] = $35;
 }
 $36 = HEAP32[$0>>2]|0;
 $37 = HEAP32[$0>>2]|0;
 $38 = (($37) + 328|0);
 $39 = HEAP32[$38>>2]|0;
 _sqlite3DbFree($36,$39);
 $40 = HEAP32[$0>>2]|0;
 $41 = (($40) + 304|0);
 HEAP32[$41>>2] = 0;
 $42 = HEAP32[$0>>2]|0;
 $43 = (($42) + 328|0);
 HEAP32[$43>>2] = 0;
 STACKTOP = sp;return;
}
function _sqlite3BtreeIsInBackup($p) {
 $p = $p|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 HEAP32[$0>>2] = $p;
 $1 = HEAP32[$0>>2]|0;
 $2 = (($1) + 16|0);
 $3 = HEAP32[$2>>2]|0;
 $4 = ($3|0)!=(0);
 $5 = $4&1;
 STACKTOP = sp;return ($5|0);
}
function _disconnectAllVtab($db) {
 $db = $db|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $5 = 0;
 var $6 = 0, $7 = 0, $8 = 0, $9 = 0, $i = 0, $p = 0, $pSchema = 0, $pTab = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 16|0;
 $i = sp + 12|0;
 $pSchema = sp + 8|0;
 $p = sp + 4|0;
 $pTab = sp;
 HEAP32[$0>>2] = $db;
 $1 = HEAP32[$0>>2]|0;
 _sqlite3BtreeEnterAll($1);
 HEAP32[$i>>2] = 0;
 while(1) {
  $2 = HEAP32[$i>>2]|0;
  $3 = HEAP32[$0>>2]|0;
  $4 = (($3) + 20|0);
  $5 = HEAP32[$4>>2]|0;
  $6 = ($2|0)<($5|0);
  if (!($6)) {
   break;
  }
  $7 = HEAP32[$i>>2]|0;
  $8 = HEAP32[$0>>2]|0;
  $9 = (($8) + 16|0);
  $10 = HEAP32[$9>>2]|0;
  $11 = (($10) + ($7<<4)|0);
  $12 = (($11) + 12|0);
  $13 = HEAP32[$12>>2]|0;
  HEAP32[$pSchema>>2] = $13;
  $14 = HEAP32[$i>>2]|0;
  $15 = HEAP32[$0>>2]|0;
  $16 = (($15) + 16|0);
  $17 = HEAP32[$16>>2]|0;
  $18 = (($17) + ($14<<4)|0);
  $19 = (($18) + 12|0);
  $20 = HEAP32[$19>>2]|0;
  $21 = ($20|0)!=(0|0);
  L4: do {
   if ($21) {
    $22 = HEAP32[$pSchema>>2]|0;
    $23 = (($22) + 8|0);
    $24 = (($23) + 8|0);
    $25 = HEAP32[$24>>2]|0;
    HEAP32[$p>>2] = $25;
    while(1) {
     $26 = HEAP32[$p>>2]|0;
     $27 = ($26|0)!=(0|0);
     if (!($27)) {
      break L4;
     }
     $28 = HEAP32[$p>>2]|0;
     $29 = (($28) + 8|0);
     $30 = HEAP32[$29>>2]|0;
     HEAP32[$pTab>>2] = $30;
     $31 = HEAP32[$pTab>>2]|0;
     $32 = (($31) + 44|0);
     $33 = HEAP8[$32>>0]|0;
     $34 = $33&255;
     $35 = $34 & 16;
     $36 = ($35|0)!=(0);
     if ($36) {
      $37 = HEAP32[$0>>2]|0;
      $38 = HEAP32[$pTab>>2]|0;
      _sqlite3VtabDisconnect($37,$38);
     }
     $39 = HEAP32[$p>>2]|0;
     $40 = HEAP32[$39>>2]|0;
     HEAP32[$p>>2] = $40;
    }
   }
  } while(0);
  $41 = HEAP32[$i>>2]|0;
  $42 = (($41) + 1)|0;
  HEAP32[$i>>2] = $42;
 }
 $43 = HEAP32[$0>>2]|0;
 _sqlite3VtabUnlockList($43);
 STACKTOP = sp;return;
}
function _sqlite3VtabDisconnect($db,$p) {
 $db = $db|0;
 $p = $p|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0;
 var $pVTab = 0, $ppVTab = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $1 = sp + 8|0;
 $ppVTab = sp + 4|0;
 $pVTab = sp;
 HEAP32[$0>>2] = $db;
 HEAP32[$1>>2] = $p;
 $2 = HEAP32[$1>>2]|0;
 $3 = (($2) + 60|0);
 HEAP32[$ppVTab>>2] = $3;
 while(1) {
  $4 = HEAP32[$ppVTab>>2]|0;
  $5 = HEAP32[$4>>2]|0;
  $6 = ($5|0)!=(0|0);
  if (!($6)) {
   label = 6;
   break;
  }
  $7 = HEAP32[$ppVTab>>2]|0;
  $8 = HEAP32[$7>>2]|0;
  $9 = HEAP32[$8>>2]|0;
  $10 = HEAP32[$0>>2]|0;
  $11 = ($9|0)==($10|0);
  $12 = HEAP32[$ppVTab>>2]|0;
  $13 = HEAP32[$12>>2]|0;
  if ($11) {
   break;
  }
  $19 = (($13) + 24|0);
  HEAP32[$ppVTab>>2] = $19;
 }
 if ((label|0) == 6) {
  STACKTOP = sp;return;
 }
 HEAP32[$pVTab>>2] = $13;
 $14 = HEAP32[$pVTab>>2]|0;
 $15 = (($14) + 24|0);
 $16 = HEAP32[$15>>2]|0;
 $17 = HEAP32[$ppVTab>>2]|0;
 HEAP32[$17>>2] = $16;
 $18 = HEAP32[$pVTab>>2]|0;
 _sqlite3VtabUnlock($18);
 STACKTOP = sp;return;
}
function _sqlite3PcacheShrink($pCache) {
 $pCache = $pCache|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 HEAP32[$0>>2] = $pCache;
 $1 = HEAP32[((38640 + 152|0))>>2]|0;
 $2 = HEAP32[$0>>2]|0;
 $3 = (($2) + 40|0);
 $4 = HEAP32[$3>>2]|0;
 FUNCTION_TABLE_vi[$1 & 31]($4);
 STACKTOP = sp;return;
}
function _pcache1Init($NotUsed) {
 $NotUsed = $NotUsed|0;
 var $0 = 0, $1 = 0, $2 = 0, dest = 0, label = 0, sp = 0, stop = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 HEAP32[$0>>2] = $NotUsed;
 dest=3136+0|0; stop=dest+68|0; do { HEAP32[dest>>2]=0|0; dest=dest+4|0; } while ((dest|0) < (stop|0));
 $1 = HEAP32[((38640 + 4|0))>>2]|0;
 $2 = ($1|0)!=(0);
 if ($2) {
  HEAP32[3136>>2] = (8);
  HEAP32[((3136 + 52|0))>>2] = (8);
 }
 HEAP32[((3136 + 12|0))>>2] = 10;
 HEAP32[((3136 + 28|0))>>2] = 1;
 STACKTOP = sp;return 0;
}
function _pcache1Shutdown($NotUsed) {
 $NotUsed = $NotUsed|0;
 var $0 = 0, dest = 0, label = 0, sp = 0, stop = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 HEAP32[$0>>2] = $NotUsed;
 dest=3136+0|0; stop=dest+68|0; do { HEAP32[dest>>2]=0|0; dest=dest+4|0; } while ((dest|0) < (stop|0));
 STACKTOP = sp;return;
}
function _pcache1Create($szPage,$szExtra,$bPurgeable) {
 $szPage = $szPage|0;
 $szExtra = $szExtra|0;
 $bPurgeable = $bPurgeable|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $pCache = 0, $pGroup = 0, $separateCache = 0, $sz = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 24|0;
 $1 = sp + 20|0;
 $2 = sp + 16|0;
 $pCache = sp + 12|0;
 $pGroup = sp + 8|0;
 $sz = sp + 4|0;
 $separateCache = sp;
 HEAP32[$0>>2] = $szPage;
 HEAP32[$1>>2] = $szExtra;
 HEAP32[$2>>2] = $bPurgeable;
 HEAP32[$separateCache>>2] = 0;
 HEAP32[$sz>>2] = 48;
 $3 = HEAP32[$sz>>2]|0;
 $4 = ($3|0)<(0);
 $5 = $4 << 31 >> 31;
 $6 = (_sqlite3MallocZero($3,$5)|0);
 HEAP32[$pCache>>2] = $6;
 $7 = HEAP32[$pCache>>2]|0;
 $8 = ($7|0)!=(0|0);
 if (!($8)) {
  $49 = HEAP32[$pCache>>2]|0;
  STACKTOP = sp;return ($49|0);
 }
 HEAP32[$pGroup>>2] = 3136;
 $9 = HEAP32[$pGroup>>2]|0;
 $10 = HEAP32[$pCache>>2]|0;
 HEAP32[$10>>2] = $9;
 $11 = HEAP32[$0>>2]|0;
 $12 = HEAP32[$pCache>>2]|0;
 $13 = (($12) + 4|0);
 HEAP32[$13>>2] = $11;
 $14 = HEAP32[$1>>2]|0;
 $15 = HEAP32[$pCache>>2]|0;
 $16 = (($15) + 8|0);
 HEAP32[$16>>2] = $14;
 $17 = HEAP32[$2>>2]|0;
 $18 = ($17|0)!=(0);
 $19 = $18 ? 1 : 0;
 $20 = HEAP32[$pCache>>2]|0;
 $21 = (($20) + 12|0);
 HEAP32[$21>>2] = $19;
 $22 = HEAP32[$pCache>>2]|0;
 _pcache1ResizeHash($22);
 $23 = HEAP32[$2>>2]|0;
 $24 = ($23|0)!=(0);
 if ($24) {
  $25 = HEAP32[$pCache>>2]|0;
  $26 = (($25) + 16|0);
  HEAP32[$26>>2] = 10;
  $27 = HEAP32[$pCache>>2]|0;
  $28 = (($27) + 16|0);
  $29 = HEAP32[$28>>2]|0;
  $30 = HEAP32[$pGroup>>2]|0;
  $31 = (($30) + 8|0);
  $32 = HEAP32[$31>>2]|0;
  $33 = (($32) + ($29))|0;
  HEAP32[$31>>2] = $33;
  $34 = HEAP32[$pGroup>>2]|0;
  $35 = (($34) + 4|0);
  $36 = HEAP32[$35>>2]|0;
  $37 = (($36) + 10)|0;
  $38 = HEAP32[$pGroup>>2]|0;
  $39 = (($38) + 8|0);
  $40 = HEAP32[$39>>2]|0;
  $41 = (($37) - ($40))|0;
  $42 = HEAP32[$pGroup>>2]|0;
  $43 = (($42) + 12|0);
  HEAP32[$43>>2] = $41;
 }
 $44 = HEAP32[$pCache>>2]|0;
 $45 = (($44) + 40|0);
 $46 = HEAP32[$45>>2]|0;
 $47 = ($46|0)==(0);
 if (!($47)) {
  $49 = HEAP32[$pCache>>2]|0;
  STACKTOP = sp;return ($49|0);
 }
 $48 = HEAP32[$pCache>>2]|0;
 _pcache1Destroy($48);
 HEAP32[$pCache>>2] = 0;
 $49 = HEAP32[$pCache>>2]|0;
 STACKTOP = sp;return ($49|0);
}
function _pcache1Cachesize($p,$nMax) {
 $p = $p|0;
 $nMax = $nMax|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $pCache = 0;
 var $pGroup = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $1 = sp + 8|0;
 $pCache = sp + 4|0;
 $pGroup = sp;
 HEAP32[$0>>2] = $p;
 HEAP32[$1>>2] = $nMax;
 $2 = HEAP32[$0>>2]|0;
 HEAP32[$pCache>>2] = $2;
 $3 = HEAP32[$pCache>>2]|0;
 $4 = (($3) + 12|0);
 $5 = HEAP32[$4>>2]|0;
 $6 = ($5|0)!=(0);
 if (!($6)) {
  STACKTOP = sp;return;
 }
 $7 = HEAP32[$pCache>>2]|0;
 $8 = HEAP32[$7>>2]|0;
 HEAP32[$pGroup>>2] = $8;
 $9 = HEAP32[$1>>2]|0;
 $10 = HEAP32[$pCache>>2]|0;
 $11 = (($10) + 20|0);
 $12 = HEAP32[$11>>2]|0;
 $13 = (($9) - ($12))|0;
 $14 = HEAP32[$pGroup>>2]|0;
 $15 = (($14) + 4|0);
 $16 = HEAP32[$15>>2]|0;
 $17 = (($16) + ($13))|0;
 HEAP32[$15>>2] = $17;
 $18 = HEAP32[$pGroup>>2]|0;
 $19 = (($18) + 4|0);
 $20 = HEAP32[$19>>2]|0;
 $21 = (($20) + 10)|0;
 $22 = HEAP32[$pGroup>>2]|0;
 $23 = (($22) + 8|0);
 $24 = HEAP32[$23>>2]|0;
 $25 = (($21) - ($24))|0;
 $26 = HEAP32[$pGroup>>2]|0;
 $27 = (($26) + 12|0);
 HEAP32[$27>>2] = $25;
 $28 = HEAP32[$1>>2]|0;
 $29 = HEAP32[$pCache>>2]|0;
 $30 = (($29) + 20|0);
 HEAP32[$30>>2] = $28;
 $31 = HEAP32[$pCache>>2]|0;
 $32 = (($31) + 20|0);
 $33 = HEAP32[$32>>2]|0;
 $34 = ($33*9)|0;
 $35 = (($34>>>0) / 10)&-1;
 $36 = HEAP32[$pCache>>2]|0;
 $37 = (($36) + 24|0);
 HEAP32[$37>>2] = $35;
 $38 = HEAP32[$pGroup>>2]|0;
 _pcache1EnforceMaxPage($38);
 STACKTOP = sp;return;
}
function _pcache1Pagecount($p) {
 $p = $p|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $n = 0, $pCache = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $n = sp + 4|0;
 $pCache = sp;
 HEAP32[$0>>2] = $p;
 $1 = HEAP32[$0>>2]|0;
 HEAP32[$pCache>>2] = $1;
 $2 = HEAP32[$pCache>>2]|0;
 $3 = (($2) + 36|0);
 $4 = HEAP32[$3>>2]|0;
 HEAP32[$n>>2] = $4;
 $5 = HEAP32[$n>>2]|0;
 STACKTOP = sp;return ($5|0);
}
function _pcache1Fetch($p,$iKey,$createFlag) {
 $p = $p|0;
 $iKey = $iKey|0;
 $createFlag = $createFlag|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $pCache = 0, $pPage = 0;
 var label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 16|0;
 $1 = sp + 12|0;
 $2 = sp + 8|0;
 $pCache = sp + 4|0;
 $pPage = sp;
 HEAP32[$0>>2] = $p;
 HEAP32[$1>>2] = $iKey;
 HEAP32[$2>>2] = $createFlag;
 $3 = HEAP32[$0>>2]|0;
 HEAP32[$pCache>>2] = $3;
 HEAP32[$pPage>>2] = 0;
 $4 = HEAP32[$1>>2]|0;
 $5 = HEAP32[$pCache>>2]|0;
 $6 = (($5) + 40|0);
 $7 = HEAP32[$6>>2]|0;
 $8 = (($4>>>0) % ($7>>>0))&-1;
 $9 = HEAP32[$pCache>>2]|0;
 $10 = (($9) + 44|0);
 $11 = HEAP32[$10>>2]|0;
 $12 = (($11) + ($8<<2)|0);
 $13 = HEAP32[$12>>2]|0;
 HEAP32[$pPage>>2] = $13;
 while(1) {
  $14 = HEAP32[$pPage>>2]|0;
  $15 = ($14|0)!=(0|0);
  if ($15) {
   $16 = HEAP32[$pPage>>2]|0;
   $17 = (($16) + 8|0);
   $18 = HEAP32[$17>>2]|0;
   $19 = HEAP32[$1>>2]|0;
   $20 = ($18|0)!=($19|0);
   $37 = $20;
  } else {
   $37 = 0;
  }
  $21 = HEAP32[$pPage>>2]|0;
  if (!($37)) {
   break;
  }
  $22 = (($21) + 16|0);
  $23 = HEAP32[$22>>2]|0;
  HEAP32[$pPage>>2] = $23;
 }
 $24 = ($21|0)!=(0|0);
 if ($24) {
  $25 = HEAP32[$pPage>>2]|0;
  $26 = (($25) + 12|0);
  $27 = HEAP8[$26>>0]|0;
  $28 = ($27<<24>>24)!=(0);
  if ($28) {
   $36 = HEAP32[$pPage>>2]|0;
   STACKTOP = sp;return ($36|0);
  }
  $29 = HEAP32[$pPage>>2]|0;
  _pcache1PinPage($29);
  $36 = HEAP32[$pPage>>2]|0;
  STACKTOP = sp;return ($36|0);
 } else {
  $30 = HEAP32[$2>>2]|0;
  $31 = ($30|0)!=(0);
  if (!($31)) {
   $36 = HEAP32[$pPage>>2]|0;
   STACKTOP = sp;return ($36|0);
  }
  $32 = HEAP32[$pCache>>2]|0;
  $33 = HEAP32[$1>>2]|0;
  $34 = HEAP32[$2>>2]|0;
  $35 = (_pcache1FetchStage2($32,$33,$34)|0);
  HEAP32[$pPage>>2] = $35;
  $36 = HEAP32[$pPage>>2]|0;
  STACKTOP = sp;return ($36|0);
 }
 return 0|0;
}
function _pcache1Unpin($p,$pPg,$reuseUnlikely) {
 $p = $p|0;
 $pPg = $pPg|0;
 $reuseUnlikely = $reuseUnlikely|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $pCache = 0, $pGroup = 0, $pPage = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 20|0;
 $1 = sp + 16|0;
 $2 = sp + 12|0;
 $pCache = sp + 8|0;
 $pPage = sp + 4|0;
 $pGroup = sp;
 HEAP32[$0>>2] = $p;
 HEAP32[$1>>2] = $pPg;
 HEAP32[$2>>2] = $reuseUnlikely;
 $3 = HEAP32[$0>>2]|0;
 HEAP32[$pCache>>2] = $3;
 $4 = HEAP32[$1>>2]|0;
 HEAP32[$pPage>>2] = $4;
 $5 = HEAP32[$pCache>>2]|0;
 $6 = HEAP32[$5>>2]|0;
 HEAP32[$pGroup>>2] = $6;
 $7 = HEAP32[$2>>2]|0;
 $8 = ($7|0)!=(0);
 if (!($8)) {
  $9 = HEAP32[$pGroup>>2]|0;
  $10 = (($9) + 16|0);
  $11 = HEAP32[$10>>2]|0;
  $12 = HEAP32[$pGroup>>2]|0;
  $13 = (($12) + 4|0);
  $14 = HEAP32[$13>>2]|0;
  $15 = ($11>>>0)>($14>>>0);
  if (!($15)) {
   $18 = HEAP32[$pGroup>>2]|0;
   $19 = (($18) + 20|0);
   $20 = HEAP32[$19>>2]|0;
   $21 = ($20|0)!=(0|0);
   $22 = HEAP32[$pPage>>2]|0;
   $23 = HEAP32[$pGroup>>2]|0;
   if ($21) {
    $24 = (($23) + 20|0);
    $25 = HEAP32[$24>>2]|0;
    $26 = (($25) + 28|0);
    HEAP32[$26>>2] = $22;
    $27 = HEAP32[$pGroup>>2]|0;
    $28 = (($27) + 20|0);
    $29 = HEAP32[$28>>2]|0;
    $30 = HEAP32[$pPage>>2]|0;
    $31 = (($30) + 24|0);
    HEAP32[$31>>2] = $29;
    $32 = HEAP32[$pPage>>2]|0;
    $33 = HEAP32[$pGroup>>2]|0;
    $34 = (($33) + 20|0);
    HEAP32[$34>>2] = $32;
   } else {
    $35 = (($23) + 24|0);
    HEAP32[$35>>2] = $22;
    $36 = HEAP32[$pPage>>2]|0;
    $37 = HEAP32[$pGroup>>2]|0;
    $38 = (($37) + 20|0);
    HEAP32[$38>>2] = $36;
   }
   $39 = HEAP32[$pCache>>2]|0;
   $40 = (($39) + 32|0);
   $41 = HEAP32[$40>>2]|0;
   $42 = (($41) + 1)|0;
   HEAP32[$40>>2] = $42;
   $43 = HEAP32[$pPage>>2]|0;
   $44 = (($43) + 12|0);
   HEAP8[$44>>0] = 0;
   STACKTOP = sp;return;
  }
 }
 $16 = HEAP32[$pPage>>2]|0;
 _pcache1RemoveFromHash($16);
 $17 = HEAP32[$pPage>>2]|0;
 _pcache1FreePage($17);
 STACKTOP = sp;return;
}
function _pcache1Rekey($p,$pPg,$iOld,$iNew) {
 $p = $p|0;
 $pPg = $pPg|0;
 $iOld = $iOld|0;
 $iNew = $iNew|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $h = 0, $pCache = 0, $pPage = 0;
 var $pp = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 28|0;
 $1 = sp + 24|0;
 $2 = sp + 20|0;
 $3 = sp + 16|0;
 $pCache = sp + 12|0;
 $pPage = sp + 8|0;
 $pp = sp + 4|0;
 $h = sp;
 HEAP32[$0>>2] = $p;
 HEAP32[$1>>2] = $pPg;
 HEAP32[$2>>2] = $iOld;
 HEAP32[$3>>2] = $iNew;
 $4 = HEAP32[$0>>2]|0;
 HEAP32[$pCache>>2] = $4;
 $5 = HEAP32[$1>>2]|0;
 HEAP32[$pPage>>2] = $5;
 $6 = HEAP32[$2>>2]|0;
 $7 = HEAP32[$pCache>>2]|0;
 $8 = (($7) + 40|0);
 $9 = HEAP32[$8>>2]|0;
 $10 = (($6>>>0) % ($9>>>0))&-1;
 HEAP32[$h>>2] = $10;
 $11 = HEAP32[$h>>2]|0;
 $12 = HEAP32[$pCache>>2]|0;
 $13 = (($12) + 44|0);
 $14 = HEAP32[$13>>2]|0;
 $15 = (($14) + ($11<<2)|0);
 HEAP32[$pp>>2] = $15;
 while(1) {
  $16 = HEAP32[$pp>>2]|0;
  $17 = HEAP32[$16>>2]|0;
  $18 = HEAP32[$pPage>>2]|0;
  $19 = ($17|0)!=($18|0);
  if (!($19)) {
   break;
  }
  $20 = HEAP32[$pp>>2]|0;
  $21 = HEAP32[$20>>2]|0;
  $22 = (($21) + 16|0);
  HEAP32[$pp>>2] = $22;
 }
 $23 = HEAP32[$pPage>>2]|0;
 $24 = (($23) + 16|0);
 $25 = HEAP32[$24>>2]|0;
 $26 = HEAP32[$pp>>2]|0;
 HEAP32[$26>>2] = $25;
 $27 = HEAP32[$3>>2]|0;
 $28 = HEAP32[$pCache>>2]|0;
 $29 = (($28) + 40|0);
 $30 = HEAP32[$29>>2]|0;
 $31 = (($27>>>0) % ($30>>>0))&-1;
 HEAP32[$h>>2] = $31;
 $32 = HEAP32[$3>>2]|0;
 $33 = HEAP32[$pPage>>2]|0;
 $34 = (($33) + 8|0);
 HEAP32[$34>>2] = $32;
 $35 = HEAP32[$h>>2]|0;
 $36 = HEAP32[$pCache>>2]|0;
 $37 = (($36) + 44|0);
 $38 = HEAP32[$37>>2]|0;
 $39 = (($38) + ($35<<2)|0);
 $40 = HEAP32[$39>>2]|0;
 $41 = HEAP32[$pPage>>2]|0;
 $42 = (($41) + 16|0);
 HEAP32[$42>>2] = $40;
 $43 = HEAP32[$pPage>>2]|0;
 $44 = HEAP32[$h>>2]|0;
 $45 = HEAP32[$pCache>>2]|0;
 $46 = (($45) + 44|0);
 $47 = HEAP32[$46>>2]|0;
 $48 = (($47) + ($44<<2)|0);
 HEAP32[$48>>2] = $43;
 $49 = HEAP32[$3>>2]|0;
 $50 = HEAP32[$pCache>>2]|0;
 $51 = (($50) + 28|0);
 $52 = HEAP32[$51>>2]|0;
 $53 = ($49>>>0)>($52>>>0);
 if (!($53)) {
  STACKTOP = sp;return;
 }
 $54 = HEAP32[$3>>2]|0;
 $55 = HEAP32[$pCache>>2]|0;
 $56 = (($55) + 28|0);
 HEAP32[$56>>2] = $54;
 STACKTOP = sp;return;
}
function _pcache1Truncate($p,$iLimit) {
 $p = $p|0;
 $iLimit = $iLimit|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $pCache = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $1 = sp + 4|0;
 $pCache = sp;
 HEAP32[$0>>2] = $p;
 HEAP32[$1>>2] = $iLimit;
 $2 = HEAP32[$0>>2]|0;
 HEAP32[$pCache>>2] = $2;
 $3 = HEAP32[$1>>2]|0;
 $4 = HEAP32[$pCache>>2]|0;
 $5 = (($4) + 28|0);
 $6 = HEAP32[$5>>2]|0;
 $7 = ($3>>>0)<=($6>>>0);
 if (!($7)) {
  STACKTOP = sp;return;
 }
 $8 = HEAP32[$pCache>>2]|0;
 $9 = HEAP32[$1>>2]|0;
 _pcache1TruncateUnsafe($8,$9);
 $10 = HEAP32[$1>>2]|0;
 $11 = (($10) - 1)|0;
 $12 = HEAP32[$pCache>>2]|0;
 $13 = (($12) + 28|0);
 HEAP32[$13>>2] = $11;
 STACKTOP = sp;return;
}
function _pcache1Destroy($p) {
 $p = $p|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $pCache = 0, $pGroup = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $pCache = sp + 4|0;
 $pGroup = sp;
 HEAP32[$0>>2] = $p;
 $1 = HEAP32[$0>>2]|0;
 HEAP32[$pCache>>2] = $1;
 $2 = HEAP32[$pCache>>2]|0;
 $3 = HEAP32[$2>>2]|0;
 HEAP32[$pGroup>>2] = $3;
 $4 = HEAP32[$pCache>>2]|0;
 _pcache1TruncateUnsafe($4,0);
 $5 = HEAP32[$pCache>>2]|0;
 $6 = (($5) + 20|0);
 $7 = HEAP32[$6>>2]|0;
 $8 = HEAP32[$pGroup>>2]|0;
 $9 = (($8) + 4|0);
 $10 = HEAP32[$9>>2]|0;
 $11 = (($10) - ($7))|0;
 HEAP32[$9>>2] = $11;
 $12 = HEAP32[$pCache>>2]|0;
 $13 = (($12) + 16|0);
 $14 = HEAP32[$13>>2]|0;
 $15 = HEAP32[$pGroup>>2]|0;
 $16 = (($15) + 8|0);
 $17 = HEAP32[$16>>2]|0;
 $18 = (($17) - ($14))|0;
 HEAP32[$16>>2] = $18;
 $19 = HEAP32[$pGroup>>2]|0;
 $20 = (($19) + 4|0);
 $21 = HEAP32[$20>>2]|0;
 $22 = (($21) + 10)|0;
 $23 = HEAP32[$pGroup>>2]|0;
 $24 = (($23) + 8|0);
 $25 = HEAP32[$24>>2]|0;
 $26 = (($22) - ($25))|0;
 $27 = HEAP32[$pGroup>>2]|0;
 $28 = (($27) + 12|0);
 HEAP32[$28>>2] = $26;
 $29 = HEAP32[$pGroup>>2]|0;
 _pcache1EnforceMaxPage($29);
 $30 = HEAP32[$pCache>>2]|0;
 $31 = (($30) + 44|0);
 $32 = HEAP32[$31>>2]|0;
 _sqlite3_free($32);
 $33 = HEAP32[$pCache>>2]|0;
 _sqlite3_free($33);
 STACKTOP = sp;return;
}
function _pcache1Shrink($p) {
 $p = $p|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $pCache = 0, $pGroup = 0, $savedMaxPage = 0;
 var label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $pCache = sp + 8|0;
 $pGroup = sp + 4|0;
 $savedMaxPage = sp;
 HEAP32[$0>>2] = $p;
 $1 = HEAP32[$0>>2]|0;
 HEAP32[$pCache>>2] = $1;
 $2 = HEAP32[$pCache>>2]|0;
 $3 = (($2) + 12|0);
 $4 = HEAP32[$3>>2]|0;
 $5 = ($4|0)!=(0);
 if (!($5)) {
  STACKTOP = sp;return;
 }
 $6 = HEAP32[$pCache>>2]|0;
 $7 = HEAP32[$6>>2]|0;
 HEAP32[$pGroup>>2] = $7;
 $8 = HEAP32[$pGroup>>2]|0;
 $9 = (($8) + 4|0);
 $10 = HEAP32[$9>>2]|0;
 HEAP32[$savedMaxPage>>2] = $10;
 $11 = HEAP32[$pGroup>>2]|0;
 $12 = (($11) + 4|0);
 HEAP32[$12>>2] = 0;
 $13 = HEAP32[$pGroup>>2]|0;
 _pcache1EnforceMaxPage($13);
 $14 = HEAP32[$savedMaxPage>>2]|0;
 $15 = HEAP32[$pGroup>>2]|0;
 $16 = (($15) + 4|0);
 HEAP32[$16>>2] = $14;
 STACKTOP = sp;return;
}
function _pcache1EnforceMaxPage($pGroup) {
 $pGroup = $pGroup|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $p = 0, label = 0;
 var sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $p = sp;
 HEAP32[$0>>2] = $pGroup;
 while(1) {
  $1 = HEAP32[$0>>2]|0;
  $2 = (($1) + 16|0);
  $3 = HEAP32[$2>>2]|0;
  $4 = HEAP32[$0>>2]|0;
  $5 = (($4) + 4|0);
  $6 = HEAP32[$5>>2]|0;
  $7 = ($3>>>0)>($6>>>0);
  if (!($7)) {
   label = 5;
   break;
  }
  $8 = HEAP32[$0>>2]|0;
  $9 = (($8) + 24|0);
  $10 = HEAP32[$9>>2]|0;
  $11 = ($10|0)!=(0|0);
  if (!($11)) {
   label = 5;
   break;
  }
  $12 = HEAP32[$0>>2]|0;
  $13 = (($12) + 24|0);
  $14 = HEAP32[$13>>2]|0;
  HEAP32[$p>>2] = $14;
  $15 = HEAP32[$p>>2]|0;
  _pcache1PinPage($15);
  $16 = HEAP32[$p>>2]|0;
  _pcache1RemoveFromHash($16);
  $17 = HEAP32[$p>>2]|0;
  _pcache1FreePage($17);
 }
 if ((label|0) == 5) {
  STACKTOP = sp;return;
 }
}
function _pcache1PinPage($pPage) {
 $pPage = $pPage|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $5 = 0, $6 = 0, $7 = 0;
 var $8 = 0, $9 = 0, $pCache = 0, $pGroup = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $pCache = sp + 4|0;
 $pGroup = sp;
 HEAP32[$0>>2] = $pPage;
 $1 = HEAP32[$0>>2]|0;
 $2 = (($1) + 20|0);
 $3 = HEAP32[$2>>2]|0;
 HEAP32[$pCache>>2] = $3;
 $4 = HEAP32[$pCache>>2]|0;
 $5 = HEAP32[$4>>2]|0;
 HEAP32[$pGroup>>2] = $5;
 $6 = HEAP32[$0>>2]|0;
 $7 = (($6) + 28|0);
 $8 = HEAP32[$7>>2]|0;
 $9 = ($8|0)!=(0|0);
 $10 = HEAP32[$0>>2]|0;
 $11 = (($10) + 24|0);
 $12 = HEAP32[$11>>2]|0;
 if ($9) {
  $13 = HEAP32[$0>>2]|0;
  $14 = (($13) + 28|0);
  $15 = HEAP32[$14>>2]|0;
  $16 = (($15) + 24|0);
  HEAP32[$16>>2] = $12;
 } else {
  $17 = HEAP32[$pGroup>>2]|0;
  $18 = (($17) + 20|0);
  HEAP32[$18>>2] = $12;
 }
 $19 = HEAP32[$0>>2]|0;
 $20 = (($19) + 24|0);
 $21 = HEAP32[$20>>2]|0;
 $22 = ($21|0)!=(0|0);
 $23 = HEAP32[$0>>2]|0;
 $24 = (($23) + 28|0);
 $25 = HEAP32[$24>>2]|0;
 if ($22) {
  $26 = HEAP32[$0>>2]|0;
  $27 = (($26) + 24|0);
  $28 = HEAP32[$27>>2]|0;
  $29 = (($28) + 28|0);
  HEAP32[$29>>2] = $25;
 } else {
  $30 = HEAP32[$pGroup>>2]|0;
  $31 = (($30) + 24|0);
  HEAP32[$31>>2] = $25;
 }
 $32 = HEAP32[$0>>2]|0;
 $33 = (($32) + 24|0);
 HEAP32[$33>>2] = 0;
 $34 = HEAP32[$0>>2]|0;
 $35 = (($34) + 28|0);
 HEAP32[$35>>2] = 0;
 $36 = HEAP32[$0>>2]|0;
 $37 = (($36) + 12|0);
 HEAP8[$37>>0] = 1;
 $38 = HEAP32[$pCache>>2]|0;
 $39 = (($38) + 32|0);
 $40 = HEAP32[$39>>2]|0;
 $41 = (($40) + -1)|0;
 HEAP32[$39>>2] = $41;
 STACKTOP = sp;return;
}
function _pcache1RemoveFromHash($pPage) {
 $pPage = $pPage|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $h = 0, $pCache = 0, $pp = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $h = sp + 8|0;
 $pCache = sp + 4|0;
 $pp = sp;
 HEAP32[$0>>2] = $pPage;
 $1 = HEAP32[$0>>2]|0;
 $2 = (($1) + 20|0);
 $3 = HEAP32[$2>>2]|0;
 HEAP32[$pCache>>2] = $3;
 $4 = HEAP32[$0>>2]|0;
 $5 = (($4) + 8|0);
 $6 = HEAP32[$5>>2]|0;
 $7 = HEAP32[$pCache>>2]|0;
 $8 = (($7) + 40|0);
 $9 = HEAP32[$8>>2]|0;
 $10 = (($6>>>0) % ($9>>>0))&-1;
 HEAP32[$h>>2] = $10;
 $11 = HEAP32[$h>>2]|0;
 $12 = HEAP32[$pCache>>2]|0;
 $13 = (($12) + 44|0);
 $14 = HEAP32[$13>>2]|0;
 $15 = (($14) + ($11<<2)|0);
 HEAP32[$pp>>2] = $15;
 while(1) {
  $16 = HEAP32[$pp>>2]|0;
  $17 = HEAP32[$16>>2]|0;
  $18 = HEAP32[$0>>2]|0;
  $19 = ($17|0)!=($18|0);
  $20 = HEAP32[$pp>>2]|0;
  $21 = HEAP32[$20>>2]|0;
  $22 = (($21) + 16|0);
  if (!($19)) {
   break;
  }
  HEAP32[$pp>>2] = $22;
 }
 $23 = HEAP32[$22>>2]|0;
 $24 = HEAP32[$pp>>2]|0;
 HEAP32[$24>>2] = $23;
 $25 = HEAP32[$pCache>>2]|0;
 $26 = (($25) + 36|0);
 $27 = HEAP32[$26>>2]|0;
 $28 = (($27) + -1)|0;
 HEAP32[$26>>2] = $28;
 STACKTOP = sp;return;
}
function _pcache1FreePage($p) {
 $p = $p|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $pCache = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $pCache = sp;
 HEAP32[$0>>2] = $p;
 $1 = HEAP32[$0>>2]|0;
 $2 = ($1|0)!=(0|0);
 if (!($2)) {
  STACKTOP = sp;return;
 }
 $3 = HEAP32[$0>>2]|0;
 $4 = (($3) + 20|0);
 $5 = HEAP32[$4>>2]|0;
 HEAP32[$pCache>>2] = $5;
 $6 = HEAP32[$0>>2]|0;
 $7 = HEAP32[$6>>2]|0;
 (_pcache1Free($7)|0);
 $8 = HEAP32[$pCache>>2]|0;
 $9 = (($8) + 12|0);
 $10 = HEAP32[$9>>2]|0;
 $11 = ($10|0)!=(0);
 if (!($11)) {
  STACKTOP = sp;return;
 }
 $12 = HEAP32[$pCache>>2]|0;
 $13 = HEAP32[$12>>2]|0;
 $14 = (($13) + 16|0);
 $15 = HEAP32[$14>>2]|0;
 $16 = (($15) + -1)|0;
 HEAP32[$14>>2] = $16;
 STACKTOP = sp;return;
}
function _pcache1TruncateUnsafe($pCache,$iLimit) {
 $pCache = $pCache|0;
 $iLimit = $iLimit|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $h = 0, $pPage = 0;
 var $pp = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 16|0;
 $1 = sp + 12|0;
 $h = sp + 8|0;
 $pp = sp + 4|0;
 $pPage = sp;
 HEAP32[$0>>2] = $pCache;
 HEAP32[$1>>2] = $iLimit;
 HEAP32[$h>>2] = 0;
 while(1) {
  $2 = HEAP32[$h>>2]|0;
  $3 = HEAP32[$0>>2]|0;
  $4 = (($3) + 40|0);
  $5 = HEAP32[$4>>2]|0;
  $6 = ($2>>>0)<($5>>>0);
  if (!($6)) {
   break;
  }
  $7 = HEAP32[$h>>2]|0;
  $8 = HEAP32[$0>>2]|0;
  $9 = (($8) + 44|0);
  $10 = HEAP32[$9>>2]|0;
  $11 = (($10) + ($7<<2)|0);
  HEAP32[$pp>>2] = $11;
  while(1) {
   $12 = HEAP32[$pp>>2]|0;
   $13 = HEAP32[$12>>2]|0;
   HEAP32[$pPage>>2] = $13;
   $14 = ($13|0)!=(0|0);
   if (!($14)) {
    break;
   }
   $15 = HEAP32[$pPage>>2]|0;
   $16 = (($15) + 8|0);
   $17 = HEAP32[$16>>2]|0;
   $18 = HEAP32[$1>>2]|0;
   $19 = ($17>>>0)>=($18>>>0);
   if (!($19)) {
    $34 = HEAP32[$pPage>>2]|0;
    $35 = (($34) + 16|0);
    HEAP32[$pp>>2] = $35;
    continue;
   }
   $20 = HEAP32[$0>>2]|0;
   $21 = (($20) + 36|0);
   $22 = HEAP32[$21>>2]|0;
   $23 = (($22) + -1)|0;
   HEAP32[$21>>2] = $23;
   $24 = HEAP32[$pPage>>2]|0;
   $25 = (($24) + 16|0);
   $26 = HEAP32[$25>>2]|0;
   $27 = HEAP32[$pp>>2]|0;
   HEAP32[$27>>2] = $26;
   $28 = HEAP32[$pPage>>2]|0;
   $29 = (($28) + 12|0);
   $30 = HEAP8[$29>>0]|0;
   $31 = ($30<<24>>24)!=(0);
   if (!($31)) {
    $32 = HEAP32[$pPage>>2]|0;
    _pcache1PinPage($32);
   }
   $33 = HEAP32[$pPage>>2]|0;
   _pcache1FreePage($33);
  }
  $36 = HEAP32[$h>>2]|0;
  $37 = (($36) + 1)|0;
  HEAP32[$h>>2] = $37;
 }
 STACKTOP = sp;return;
}
function _pcache1FetchStage2($pCache,$iKey,$createFlag) {
 $pCache = $pCache|0;
 $iKey = $iKey|0;
 $createFlag = $createFlag|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0;
 var $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0;
 var $h = 0, $nPinned = 0, $pGroup = 0, $pOther = 0, $pPage = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $0 = sp + 32|0;
 $1 = sp + 28|0;
 $2 = sp + 24|0;
 $3 = sp + 20|0;
 $nPinned = sp + 16|0;
 $pGroup = sp + 12|0;
 $pPage = sp + 8|0;
 $pOther = sp + 4|0;
 $h = sp;
 HEAP32[$1>>2] = $pCache;
 HEAP32[$2>>2] = $iKey;
 HEAP32[$3>>2] = $createFlag;
 $4 = HEAP32[$1>>2]|0;
 $5 = HEAP32[$4>>2]|0;
 HEAP32[$pGroup>>2] = $5;
 HEAP32[$pPage>>2] = 0;
 $6 = HEAP32[$1>>2]|0;
 $7 = (($6) + 36|0);
 $8 = HEAP32[$7>>2]|0;
 $9 = HEAP32[$1>>2]|0;
 $10 = (($9) + 32|0);
 $11 = HEAP32[$10>>2]|0;
 $12 = (($8) - ($11))|0;
 HEAP32[$nPinned>>2] = $12;
 $13 = HEAP32[$3>>2]|0;
 $14 = ($13|0)==(1);
 do {
  if ($14) {
   $15 = HEAP32[$nPinned>>2]|0;
   $16 = HEAP32[$pGroup>>2]|0;
   $17 = (($16) + 12|0);
   $18 = HEAP32[$17>>2]|0;
   $19 = ($15>>>0)>=($18>>>0);
   if (!($19)) {
    $20 = HEAP32[$nPinned>>2]|0;
    $21 = HEAP32[$1>>2]|0;
    $22 = (($21) + 24|0);
    $23 = HEAP32[$22>>2]|0;
    $24 = ($20>>>0)>=($23>>>0);
    if (!($24)) {
     $25 = HEAP32[$1>>2]|0;
     $26 = (_pcache1UnderMemoryPressure($25)|0);
     $27 = ($26|0)!=(0);
     if (!($27)) {
      break;
     }
     $28 = HEAP32[$1>>2]|0;
     $29 = (($28) + 32|0);
     $30 = HEAP32[$29>>2]|0;
     $31 = HEAP32[$nPinned>>2]|0;
     $32 = ($30>>>0)<($31>>>0);
     if (!($32)) {
      break;
     }
    }
   }
   HEAP32[$0>>2] = 0;
   $159 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($159|0);
  }
 } while(0);
 $33 = HEAP32[$1>>2]|0;
 $34 = (($33) + 36|0);
 $35 = HEAP32[$34>>2]|0;
 $36 = HEAP32[$1>>2]|0;
 $37 = (($36) + 40|0);
 $38 = HEAP32[$37>>2]|0;
 $39 = ($35>>>0)>=($38>>>0);
 if ($39) {
  $40 = HEAP32[$1>>2]|0;
  _pcache1ResizeHash($40);
 }
 $41 = HEAP32[$1>>2]|0;
 $42 = (($41) + 12|0);
 $43 = HEAP32[$42>>2]|0;
 $44 = ($43|0)!=(0);
 do {
  if ($44) {
   $45 = HEAP32[$pGroup>>2]|0;
   $46 = (($45) + 24|0);
   $47 = HEAP32[$46>>2]|0;
   $48 = ($47|0)!=(0|0);
   if ($48) {
    $49 = HEAP32[$1>>2]|0;
    $50 = (($49) + 36|0);
    $51 = HEAP32[$50>>2]|0;
    $52 = (($51) + 1)|0;
    $53 = HEAP32[$1>>2]|0;
    $54 = (($53) + 20|0);
    $55 = HEAP32[$54>>2]|0;
    $56 = ($52>>>0)>=($55>>>0);
    if (!($56)) {
     $57 = HEAP32[$pGroup>>2]|0;
     $58 = (($57) + 16|0);
     $59 = HEAP32[$58>>2]|0;
     $60 = HEAP32[$pGroup>>2]|0;
     $61 = (($60) + 4|0);
     $62 = HEAP32[$61>>2]|0;
     $63 = ($59>>>0)>=($62>>>0);
     if (!($63)) {
      $64 = HEAP32[$1>>2]|0;
      $65 = (_pcache1UnderMemoryPressure($64)|0);
      $66 = ($65|0)!=(0);
      if (!($66)) {
       break;
      }
     }
    }
    $67 = HEAP32[$pGroup>>2]|0;
    $68 = (($67) + 24|0);
    $69 = HEAP32[$68>>2]|0;
    HEAP32[$pPage>>2] = $69;
    $70 = HEAP32[$pPage>>2]|0;
    _pcache1RemoveFromHash($70);
    $71 = HEAP32[$pPage>>2]|0;
    _pcache1PinPage($71);
    $72 = HEAP32[$pPage>>2]|0;
    $73 = (($72) + 20|0);
    $74 = HEAP32[$73>>2]|0;
    HEAP32[$pOther>>2] = $74;
    $75 = HEAP32[$pOther>>2]|0;
    $76 = (($75) + 4|0);
    $77 = HEAP32[$76>>2]|0;
    $78 = HEAP32[$pOther>>2]|0;
    $79 = (($78) + 8|0);
    $80 = HEAP32[$79>>2]|0;
    $81 = (($77) + ($80))|0;
    $82 = HEAP32[$1>>2]|0;
    $83 = (($82) + 4|0);
    $84 = HEAP32[$83>>2]|0;
    $85 = HEAP32[$1>>2]|0;
    $86 = (($85) + 8|0);
    $87 = HEAP32[$86>>2]|0;
    $88 = (($84) + ($87))|0;
    $89 = ($81|0)!=($88|0);
    if ($89) {
     $90 = HEAP32[$pPage>>2]|0;
     _pcache1FreePage($90);
     HEAP32[$pPage>>2] = 0;
     break;
    } else {
     $91 = HEAP32[$pOther>>2]|0;
     $92 = (($91) + 12|0);
     $93 = HEAP32[$92>>2]|0;
     $94 = HEAP32[$1>>2]|0;
     $95 = (($94) + 12|0);
     $96 = HEAP32[$95>>2]|0;
     $97 = (($93) - ($96))|0;
     $98 = HEAP32[$pGroup>>2]|0;
     $99 = (($98) + 16|0);
     $100 = HEAP32[$99>>2]|0;
     $101 = (($100) - ($97))|0;
     HEAP32[$99>>2] = $101;
     break;
    }
   }
  }
 } while(0);
 $102 = HEAP32[$pPage>>2]|0;
 $103 = ($102|0)!=(0|0);
 if (!($103)) {
  $104 = HEAP32[$3>>2]|0;
  $105 = ($104|0)==(1);
  if ($105) {
   _sqlite3BeginBenignMalloc();
  }
  $106 = HEAP32[$1>>2]|0;
  $107 = (_pcache1AllocPage($106)|0);
  HEAP32[$pPage>>2] = $107;
  $108 = HEAP32[$3>>2]|0;
  $109 = ($108|0)==(1);
  if ($109) {
   _sqlite3EndBenignMalloc();
  }
 }
 $110 = HEAP32[$pPage>>2]|0;
 $111 = ($110|0)!=(0|0);
 if ($111) {
  $112 = HEAP32[$2>>2]|0;
  $113 = HEAP32[$1>>2]|0;
  $114 = (($113) + 40|0);
  $115 = HEAP32[$114>>2]|0;
  $116 = (($112>>>0) % ($115>>>0))&-1;
  HEAP32[$h>>2] = $116;
  $117 = HEAP32[$1>>2]|0;
  $118 = (($117) + 36|0);
  $119 = HEAP32[$118>>2]|0;
  $120 = (($119) + 1)|0;
  HEAP32[$118>>2] = $120;
  $121 = HEAP32[$2>>2]|0;
  $122 = HEAP32[$pPage>>2]|0;
  $123 = (($122) + 8|0);
  HEAP32[$123>>2] = $121;
  $124 = HEAP32[$h>>2]|0;
  $125 = HEAP32[$1>>2]|0;
  $126 = (($125) + 44|0);
  $127 = HEAP32[$126>>2]|0;
  $128 = (($127) + ($124<<2)|0);
  $129 = HEAP32[$128>>2]|0;
  $130 = HEAP32[$pPage>>2]|0;
  $131 = (($130) + 16|0);
  HEAP32[$131>>2] = $129;
  $132 = HEAP32[$1>>2]|0;
  $133 = HEAP32[$pPage>>2]|0;
  $134 = (($133) + 20|0);
  HEAP32[$134>>2] = $132;
  $135 = HEAP32[$pPage>>2]|0;
  $136 = (($135) + 28|0);
  HEAP32[$136>>2] = 0;
  $137 = HEAP32[$pPage>>2]|0;
  $138 = (($137) + 24|0);
  HEAP32[$138>>2] = 0;
  $139 = HEAP32[$pPage>>2]|0;
  $140 = (($139) + 12|0);
  HEAP8[$140>>0] = 1;
  $141 = HEAP32[$pPage>>2]|0;
  $142 = (($141) + 4|0);
  $143 = HEAP32[$142>>2]|0;
  HEAP32[$143>>2] = 0;
  $144 = HEAP32[$pPage>>2]|0;
  $145 = HEAP32[$h>>2]|0;
  $146 = HEAP32[$1>>2]|0;
  $147 = (($146) + 44|0);
  $148 = HEAP32[$147>>2]|0;
  $149 = (($148) + ($145<<2)|0);
  HEAP32[$149>>2] = $144;
  $150 = HEAP32[$2>>2]|0;
  $151 = HEAP32[$1>>2]|0;
  $152 = (($151) + 28|0);
  $153 = HEAP32[$152>>2]|0;
  $154 = ($150>>>0)>($153>>>0);
  if ($154) {
   $155 = HEAP32[$2>>2]|0;
   $156 = HEAP32[$1>>2]|0;
   $157 = (($156) + 28|0);
   HEAP32[$157>>2] = $155;
  }
 }
 $158 = HEAP32[$pPage>>2]|0;
 HEAP32[$0>>2] = $158;
 $159 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($159|0);
}
function _pcache1UnderMemoryPressure($pCache) {
 $pCache = $pCache|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $1 = sp;
 HEAP32[$1>>2] = $pCache;
 $2 = HEAP32[((3136 + 36|0))>>2]|0;
 $3 = ($2|0)!=(0);
 if ($3) {
  $4 = HEAP32[$1>>2]|0;
  $5 = (($4) + 4|0);
  $6 = HEAP32[$5>>2]|0;
  $7 = HEAP32[$1>>2]|0;
  $8 = (($7) + 8|0);
  $9 = HEAP32[$8>>2]|0;
  $10 = (($6) + ($9))|0;
  $11 = HEAP32[((3136 + 32|0))>>2]|0;
  $12 = ($10|0)<=($11|0);
  if ($12) {
   $13 = HEAP32[((3136 + 64|0))>>2]|0;
   HEAP32[$0>>2] = $13;
   $15 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($15|0);
  }
 }
 $14 = (_sqlite3HeapNearlyFull()|0);
 HEAP32[$0>>2] = $14;
 $15 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($15|0);
}
function _pcache1ResizeHash($p) {
 $p = $p|0;
 var $$ = 0, $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0;
 var $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0;
 var $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0;
 var $7 = 0, $8 = 0, $9 = 0, $apNew = 0, $h = 0, $i = 0, $nNew = 0, $pNext = 0, $pPage = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 24|0;
 $apNew = sp + 20|0;
 $nNew = sp + 16|0;
 $i = sp + 12|0;
 $pPage = sp + 8|0;
 $pNext = sp + 4|0;
 $h = sp;
 HEAP32[$0>>2] = $p;
 $1 = HEAP32[$0>>2]|0;
 $2 = (($1) + 40|0);
 $3 = HEAP32[$2>>2]|0;
 $4 = $3<<1;
 HEAP32[$nNew>>2] = $4;
 $5 = HEAP32[$nNew>>2]|0;
 $6 = ($5>>>0)<(256);
 $$ = $6 ? 256 : $4;
 HEAP32[$nNew>>2] = $$;
 $7 = HEAP32[$0>>2]|0;
 $8 = (($7) + 40|0);
 $9 = HEAP32[$8>>2]|0;
 $10 = ($9|0)!=(0);
 if ($10) {
  _sqlite3BeginBenignMalloc();
 }
 $11 = HEAP32[$nNew>>2]|0;
 $12 = $11<<2;
 $13 = (_sqlite3MallocZero($12,0)|0);
 HEAP32[$apNew>>2] = $13;
 $14 = HEAP32[$0>>2]|0;
 $15 = (($14) + 40|0);
 $16 = HEAP32[$15>>2]|0;
 $17 = ($16|0)!=(0);
 if ($17) {
  _sqlite3EndBenignMalloc();
 }
 $18 = HEAP32[$apNew>>2]|0;
 $19 = ($18|0)!=(0|0);
 if (!($19)) {
  STACKTOP = sp;return;
 }
 HEAP32[$i>>2] = 0;
 while(1) {
  $20 = HEAP32[$i>>2]|0;
  $21 = HEAP32[$0>>2]|0;
  $22 = (($21) + 40|0);
  $23 = HEAP32[$22>>2]|0;
  $24 = ($20>>>0)<($23>>>0);
  if (!($24)) {
   break;
  }
  $25 = HEAP32[$i>>2]|0;
  $26 = HEAP32[$0>>2]|0;
  $27 = (($26) + 44|0);
  $28 = HEAP32[$27>>2]|0;
  $29 = (($28) + ($25<<2)|0);
  $30 = HEAP32[$29>>2]|0;
  HEAP32[$pNext>>2] = $30;
  while(1) {
   $31 = HEAP32[$pNext>>2]|0;
   HEAP32[$pPage>>2] = $31;
   $32 = ($31|0)!=(0|0);
   if (!($32)) {
    break;
   }
   $33 = HEAP32[$pPage>>2]|0;
   $34 = (($33) + 8|0);
   $35 = HEAP32[$34>>2]|0;
   $36 = HEAP32[$nNew>>2]|0;
   $37 = (($35>>>0) % ($36>>>0))&-1;
   HEAP32[$h>>2] = $37;
   $38 = HEAP32[$pPage>>2]|0;
   $39 = (($38) + 16|0);
   $40 = HEAP32[$39>>2]|0;
   HEAP32[$pNext>>2] = $40;
   $41 = HEAP32[$h>>2]|0;
   $42 = HEAP32[$apNew>>2]|0;
   $43 = (($42) + ($41<<2)|0);
   $44 = HEAP32[$43>>2]|0;
   $45 = HEAP32[$pPage>>2]|0;
   $46 = (($45) + 16|0);
   HEAP32[$46>>2] = $44;
   $47 = HEAP32[$pPage>>2]|0;
   $48 = HEAP32[$h>>2]|0;
   $49 = HEAP32[$apNew>>2]|0;
   $50 = (($49) + ($48<<2)|0);
   HEAP32[$50>>2] = $47;
  }
  $51 = HEAP32[$i>>2]|0;
  $52 = (($51) + 1)|0;
  HEAP32[$i>>2] = $52;
 }
 $53 = HEAP32[$0>>2]|0;
 $54 = (($53) + 44|0);
 $55 = HEAP32[$54>>2]|0;
 _sqlite3_free($55);
 $56 = HEAP32[$apNew>>2]|0;
 $57 = HEAP32[$0>>2]|0;
 $58 = (($57) + 44|0);
 HEAP32[$58>>2] = $56;
 $59 = HEAP32[$nNew>>2]|0;
 $60 = HEAP32[$0>>2]|0;
 $61 = (($60) + 40|0);
 HEAP32[$61>>2] = $59;
 STACKTOP = sp;return;
}
function _pcache1AllocPage($pCache) {
 $pCache = $pCache|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $p = 0, $pPg = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $1 = sp + 8|0;
 $p = sp + 4|0;
 $pPg = sp;
 HEAP32[$1>>2] = $pCache;
 HEAP32[$p>>2] = 0;
 $2 = HEAP32[$1>>2]|0;
 $3 = (($2) + 4|0);
 $4 = HEAP32[$3>>2]|0;
 $5 = (32 + ($4))|0;
 $6 = HEAP32[$1>>2]|0;
 $7 = (($6) + 8|0);
 $8 = HEAP32[$7>>2]|0;
 $9 = (($5) + ($8))|0;
 $10 = (_pcache1Alloc($9)|0);
 HEAP32[$pPg>>2] = $10;
 $11 = HEAP32[$1>>2]|0;
 $12 = (($11) + 4|0);
 $13 = HEAP32[$12>>2]|0;
 $14 = HEAP32[$pPg>>2]|0;
 $15 = (($14) + ($13)|0);
 HEAP32[$p>>2] = $15;
 $16 = HEAP32[$pPg>>2]|0;
 $17 = ($16|0)!=(0|0);
 if (!($17)) {
  HEAP32[$0>>2] = 0;
  $34 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($34|0);
 }
 $18 = HEAP32[$pPg>>2]|0;
 $19 = HEAP32[$p>>2]|0;
 HEAP32[$19>>2] = $18;
 $20 = HEAP32[$p>>2]|0;
 $21 = (($20) + 32|0);
 $22 = HEAP32[$p>>2]|0;
 $23 = (($22) + 4|0);
 HEAP32[$23>>2] = $21;
 $24 = HEAP32[$1>>2]|0;
 $25 = (($24) + 12|0);
 $26 = HEAP32[$25>>2]|0;
 $27 = ($26|0)!=(0);
 if ($27) {
  $28 = HEAP32[$1>>2]|0;
  $29 = HEAP32[$28>>2]|0;
  $30 = (($29) + 16|0);
  $31 = HEAP32[$30>>2]|0;
  $32 = (($31) + 1)|0;
  HEAP32[$30>>2] = $32;
 }
 $33 = HEAP32[$p>>2]|0;
 HEAP32[$0>>2] = $33;
 $34 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($34|0);
}
function _sqlite3HeapNearlyFull() {
 var $0 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 $0 = HEAP32[((136 + 36|0))>>2]|0;
 STACKTOP = sp;return ($0|0);
}
function _sqlite3MemMalloc($nByte) {
 $nByte = $nByte|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0;
 var $p = 0, $vararg_buffer = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $vararg_buffer = sp;
 $0 = sp + 8|0;
 $p = sp + 4|0;
 HEAP32[$0>>2] = $nByte;
 $1 = HEAP32[$0>>2]|0;
 $2 = (($1) + 7)|0;
 $3 = $2 & -8;
 HEAP32[$0>>2] = $3;
 $4 = HEAP32[$0>>2]|0;
 $5 = (($4) + 8)|0;
 $6 = (_malloc($5)|0);
 HEAP32[$p>>2] = $6;
 $7 = HEAP32[$p>>2]|0;
 $8 = ($7|0)!=(0|0);
 $9 = HEAP32[$0>>2]|0;
 if ($8) {
  $10 = ($9|0)<(0);
  $11 = $10 << 31 >> 31;
  $12 = HEAP32[$p>>2]|0;
  $13 = $12;
  $14 = $13;
  HEAP32[$14>>2] = $9;
  $15 = (($13) + 4)|0;
  $16 = $15;
  HEAP32[$16>>2] = $11;
  $17 = HEAP32[$p>>2]|0;
  $18 = (($17) + 8|0);
  HEAP32[$p>>2] = $18;
  $19 = HEAP32[$p>>2]|0;
  STACKTOP = sp;return ($19|0);
 } else {
  HEAP32[$vararg_buffer>>2] = $9;
  _sqlite3_log(7,6192,$vararg_buffer);
  $19 = HEAP32[$p>>2]|0;
  STACKTOP = sp;return ($19|0);
 }
 return 0|0;
}
function _sqlite3MemFree($pPrior) {
 $pPrior = $pPrior|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $p = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $p = sp;
 HEAP32[$0>>2] = $pPrior;
 $1 = HEAP32[$0>>2]|0;
 HEAP32[$p>>2] = $1;
 $2 = HEAP32[$p>>2]|0;
 $3 = (($2) + -8|0);
 HEAP32[$p>>2] = $3;
 $4 = HEAP32[$p>>2]|0;
 _free($4);
 STACKTOP = sp;return;
}
function _sqlite3MemRealloc($pPrior,$nByte) {
 $pPrior = $pPrior|0;
 $nByte = $nByte|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $3 = 0, $4 = 0;
 var $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $p = 0, $vararg_buffer = 0, $vararg_ptr1 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $vararg_buffer = sp;
 $0 = sp + 16|0;
 $1 = sp + 12|0;
 $p = sp + 8|0;
 HEAP32[$0>>2] = $pPrior;
 HEAP32[$1>>2] = $nByte;
 $2 = HEAP32[$0>>2]|0;
 HEAP32[$p>>2] = $2;
 $3 = HEAP32[$p>>2]|0;
 $4 = (($3) + -8|0);
 HEAP32[$p>>2] = $4;
 $5 = HEAP32[$p>>2]|0;
 $6 = HEAP32[$1>>2]|0;
 $7 = (($6) + 8)|0;
 $8 = (_realloc($5,$7)|0);
 HEAP32[$p>>2] = $8;
 $9 = HEAP32[$p>>2]|0;
 $10 = ($9|0)!=(0|0);
 if ($10) {
  $11 = HEAP32[$1>>2]|0;
  $12 = ($11|0)<(0);
  $13 = $12 << 31 >> 31;
  $14 = HEAP32[$p>>2]|0;
  $15 = $14;
  $16 = $15;
  HEAP32[$16>>2] = $11;
  $17 = (($15) + 4)|0;
  $18 = $17;
  HEAP32[$18>>2] = $13;
  $19 = HEAP32[$p>>2]|0;
  $20 = (($19) + 8|0);
  HEAP32[$p>>2] = $20;
  $24 = HEAP32[$p>>2]|0;
  STACKTOP = sp;return ($24|0);
 } else {
  $21 = HEAP32[$0>>2]|0;
  $22 = (_sqlite3MemSize($21)|0);
  $23 = HEAP32[$1>>2]|0;
  HEAP32[$vararg_buffer>>2] = $22;
  $vararg_ptr1 = (($vararg_buffer) + 4|0);
  HEAP32[$vararg_ptr1>>2] = $23;
  _sqlite3_log(7,6152,$vararg_buffer);
  $24 = HEAP32[$p>>2]|0;
  STACKTOP = sp;return ($24|0);
 }
 return 0|0;
}
function _sqlite3MemSize($pPrior) {
 $pPrior = $pPrior|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $p = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $1 = sp + 4|0;
 $p = sp;
 HEAP32[$1>>2] = $pPrior;
 $2 = HEAP32[$1>>2]|0;
 $3 = ($2|0)==(0|0);
 if ($3) {
  HEAP32[$0>>2] = 0;
  $14 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($14|0);
 } else {
  $4 = HEAP32[$1>>2]|0;
  HEAP32[$p>>2] = $4;
  $5 = HEAP32[$p>>2]|0;
  $6 = (($5) + -8|0);
  HEAP32[$p>>2] = $6;
  $7 = HEAP32[$p>>2]|0;
  $8 = $7;
  $9 = $8;
  $10 = HEAP32[$9>>2]|0;
  $11 = (($8) + 4)|0;
  $12 = $11;
  $13 = HEAP32[$12>>2]|0;
  HEAP32[$0>>2] = $10;
  $14 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($14|0);
 }
 return 0|0;
}
function _sqlite3MemRoundup($n) {
 $n = $n|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 HEAP32[$0>>2] = $n;
 $1 = HEAP32[$0>>2]|0;
 $2 = (($1) + 7)|0;
 $3 = $2 & -8;
 STACKTOP = sp;return ($3|0);
}
function _sqlite3MemInit($NotUsed) {
 $NotUsed = $NotUsed|0;
 var $0 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 HEAP32[$0>>2] = $NotUsed;
 STACKTOP = sp;return 0;
}
function _sqlite3MemShutdown($NotUsed) {
 $NotUsed = $NotUsed|0;
 var $0 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 HEAP32[$0>>2] = $NotUsed;
 STACKTOP = sp;return;
}
function _trimFunc($context,$argc,$argv) {
 $context = $context|0;
 $argc = $argc|0;
 $argv = $argv|0;
 var $$old = 0, $$old1 = 0, $$old2 = 0, $$old3 = 0, $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0;
 var $112 = 0, $113 = 0, $114 = 0, $115 = 0, $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0;
 var $130 = 0, $131 = 0, $132 = 0, $133 = 0, $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0;
 var $149 = 0, $15 = 0, $150 = 0, $151 = 0, $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0;
 var $167 = 0, $168 = 0, $169 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0;
 var $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0;
 var $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0;
 var $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0;
 var $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $aLen = 0, $azChar = 0, $flags = 0, $i = 0, $len = 0, $len1 = 0;
 var $nChar = 0, $nIn = 0, $or$cond = 0, $or$cond4 = 0, $z = 0, $zCharSet = 0, $zIn = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 64|0;
 $0 = sp + 52|0;
 $1 = sp + 48|0;
 $2 = sp + 44|0;
 $zIn = sp + 40|0;
 $zCharSet = sp + 36|0;
 $nIn = sp + 32|0;
 $flags = sp + 28|0;
 $i = sp + 24|0;
 $aLen = sp + 20|0;
 $azChar = sp + 16|0;
 $nChar = sp + 12|0;
 $z = sp + 8|0;
 $len = sp + 4|0;
 $len1 = sp;
 HEAP32[$0>>2] = $context;
 HEAP32[$1>>2] = $argc;
 HEAP32[$2>>2] = $argv;
 HEAP32[$aLen>>2] = 0;
 HEAP32[$azChar>>2] = 0;
 $3 = HEAP32[$2>>2]|0;
 $4 = HEAP32[$3>>2]|0;
 $5 = (_sqlite3_value_type($4)|0);
 $6 = ($5|0)==(5);
 if ($6) {
  STACKTOP = sp;return;
 }
 $7 = HEAP32[$2>>2]|0;
 $8 = HEAP32[$7>>2]|0;
 $9 = (_sqlite3_value_text($8)|0);
 HEAP32[$zIn>>2] = $9;
 $10 = HEAP32[$zIn>>2]|0;
 $11 = ($10|0)==(0|0);
 if ($11) {
  STACKTOP = sp;return;
 }
 $12 = HEAP32[$2>>2]|0;
 $13 = HEAP32[$12>>2]|0;
 $14 = (_sqlite3_value_bytes($13)|0);
 HEAP32[$nIn>>2] = $14;
 $15 = HEAP32[$1>>2]|0;
 $16 = ($15|0)==(1);
 L7: do {
  if ($16) {
   HEAP32[$nChar>>2] = 1;
   HEAP32[$aLen>>2] = 9896;
   HEAP32[$azChar>>2] = 9904;
   HEAP32[$zCharSet>>2] = 0;
  } else {
   $17 = HEAP32[$2>>2]|0;
   $18 = (($17) + 4|0);
   $19 = HEAP32[$18>>2]|0;
   $20 = (_sqlite3_value_text($19)|0);
   HEAP32[$zCharSet>>2] = $20;
   $21 = ($20|0)==(0|0);
   if ($21) {
    STACKTOP = sp;return;
   }
   $22 = HEAP32[$zCharSet>>2]|0;
   HEAP32[$z>>2] = $22;
   HEAP32[$nChar>>2] = 0;
   while(1) {
    $23 = HEAP32[$z>>2]|0;
    $24 = HEAP8[$23>>0]|0;
    $25 = ($24<<24>>24)!=(0);
    if (!($25)) {
     break;
    }
    $26 = HEAP32[$z>>2]|0;
    $27 = (($26) + 1|0);
    HEAP32[$z>>2] = $27;
    $28 = HEAP8[$26>>0]|0;
    $29 = $28&255;
    $30 = ($29|0)>=(192);
    L16: do {
     if ($30) {
      while(1) {
       $31 = HEAP32[$z>>2]|0;
       $32 = HEAP8[$31>>0]|0;
       $33 = $32&255;
       $34 = $33 & 192;
       $35 = ($34|0)==(128);
       if (!($35)) {
        break L16;
       }
       $36 = HEAP32[$z>>2]|0;
       $37 = (($36) + 1|0);
       HEAP32[$z>>2] = $37;
      }
     }
    } while(0);
    $38 = HEAP32[$nChar>>2]|0;
    $39 = (($38) + 1)|0;
    HEAP32[$nChar>>2] = $39;
   }
   $40 = HEAP32[$nChar>>2]|0;
   $41 = ($40|0)>(0);
   if ($41) {
    $42 = HEAP32[$0>>2]|0;
    $43 = HEAP32[$nChar>>2]|0;
    $44 = ($43|0)<(0);
    $45 = $44 << 31 >> 31;
    $46 = (___muldi3(($43|0),($45|0),5,0)|0);
    $47 = tempRet0;
    $48 = (_contextMalloc($42,$46,$47)|0);
    HEAP32[$azChar>>2] = $48;
    $49 = HEAP32[$azChar>>2]|0;
    $50 = ($49|0)==(0|0);
    if ($50) {
     STACKTOP = sp;return;
    }
    $51 = HEAP32[$nChar>>2]|0;
    $52 = HEAP32[$azChar>>2]|0;
    $53 = (($52) + ($51<<2)|0);
    HEAP32[$aLen>>2] = $53;
    $54 = HEAP32[$zCharSet>>2]|0;
    HEAP32[$z>>2] = $54;
    HEAP32[$nChar>>2] = 0;
    while(1) {
     $55 = HEAP32[$z>>2]|0;
     $56 = HEAP8[$55>>0]|0;
     $57 = ($56<<24>>24)!=(0);
     if (!($57)) {
      break L7;
     }
     $58 = HEAP32[$z>>2]|0;
     $59 = HEAP32[$nChar>>2]|0;
     $60 = HEAP32[$azChar>>2]|0;
     $61 = (($60) + ($59<<2)|0);
     HEAP32[$61>>2] = $58;
     $62 = HEAP32[$z>>2]|0;
     $63 = (($62) + 1|0);
     HEAP32[$z>>2] = $63;
     $64 = HEAP8[$62>>0]|0;
     $65 = $64&255;
     $66 = ($65|0)>=(192);
     L29: do {
      if ($66) {
       while(1) {
        $67 = HEAP32[$z>>2]|0;
        $68 = HEAP8[$67>>0]|0;
        $69 = $68&255;
        $70 = $69 & 192;
        $71 = ($70|0)==(128);
        if (!($71)) {
         break L29;
        }
        $72 = HEAP32[$z>>2]|0;
        $73 = (($72) + 1|0);
        HEAP32[$z>>2] = $73;
       }
      }
     } while(0);
     $74 = HEAP32[$z>>2]|0;
     $75 = HEAP32[$nChar>>2]|0;
     $76 = HEAP32[$azChar>>2]|0;
     $77 = (($76) + ($75<<2)|0);
     $78 = HEAP32[$77>>2]|0;
     $79 = $74;
     $80 = $78;
     $81 = (($79) - ($80))|0;
     $82 = $81&255;
     $83 = HEAP32[$nChar>>2]|0;
     $84 = HEAP32[$aLen>>2]|0;
     $85 = (($84) + ($83)|0);
     HEAP8[$85>>0] = $82;
     $86 = HEAP32[$nChar>>2]|0;
     $87 = (($86) + 1)|0;
     HEAP32[$nChar>>2] = $87;
    }
   }
  }
 } while(0);
 $88 = HEAP32[$nChar>>2]|0;
 $89 = ($88|0)>(0);
 if ($89) {
  $90 = HEAP32[$0>>2]|0;
  $91 = (_sqlite3_user_data($90)|0);
  $92 = $91;
  HEAP32[$flags>>2] = $92;
  $93 = HEAP32[$flags>>2]|0;
  $94 = $93 & 1;
  $95 = ($94|0)!=(0);
  $96 = HEAP32[$nIn>>2]|0;
  $97 = ($96|0)>(0);
  $or$cond = $95 & $97;
  L37: do {
   if ($or$cond) {
    while(1) {
     HEAP32[$len>>2] = 0;
     HEAP32[$i>>2] = 0;
     while(1) {
      $98 = HEAP32[$i>>2]|0;
      $99 = HEAP32[$nChar>>2]|0;
      $100 = ($98|0)<($99|0);
      if (!($100)) {
       break;
      }
      $101 = HEAP32[$i>>2]|0;
      $102 = HEAP32[$aLen>>2]|0;
      $103 = (($102) + ($101)|0);
      $104 = HEAP8[$103>>0]|0;
      $105 = $104&255;
      HEAP32[$len>>2] = $105;
      $106 = HEAP32[$len>>2]|0;
      $107 = HEAP32[$nIn>>2]|0;
      $108 = ($106|0)<=($107|0);
      if ($108) {
       $109 = HEAP32[$zIn>>2]|0;
       $110 = HEAP32[$i>>2]|0;
       $111 = HEAP32[$azChar>>2]|0;
       $112 = (($111) + ($110<<2)|0);
       $113 = HEAP32[$112>>2]|0;
       $114 = HEAP32[$len>>2]|0;
       $115 = (_memcmp($109,$113,$114)|0);
       $116 = ($115|0)==(0);
       if ($116) {
        break;
       }
      }
      $117 = HEAP32[$i>>2]|0;
      $118 = (($117) + 1)|0;
      HEAP32[$i>>2] = $118;
     }
     $119 = HEAP32[$i>>2]|0;
     $120 = HEAP32[$nChar>>2]|0;
     $121 = ($119|0)>=($120|0);
     if ($121) {
      break L37;
     }
     $122 = HEAP32[$len>>2]|0;
     $123 = HEAP32[$zIn>>2]|0;
     $124 = (($123) + ($122)|0);
     HEAP32[$zIn>>2] = $124;
     $125 = HEAP32[$len>>2]|0;
     $126 = HEAP32[$nIn>>2]|0;
     $127 = (($126) - ($125))|0;
     HEAP32[$nIn>>2] = $127;
     $$old = HEAP32[$nIn>>2]|0;
     $$old1 = ($$old|0)>(0);
     if (!($$old1)) {
      break;
     }
    }
   }
  } while(0);
  $128 = HEAP32[$flags>>2]|0;
  $129 = $128 & 2;
  $130 = ($129|0)!=(0);
  $131 = HEAP32[$nIn>>2]|0;
  $132 = ($131|0)>(0);
  $or$cond4 = $130 & $132;
  L49: do {
   if ($or$cond4) {
    while(1) {
     HEAP32[$len1>>2] = 0;
     HEAP32[$i>>2] = 0;
     while(1) {
      $133 = HEAP32[$i>>2]|0;
      $134 = HEAP32[$nChar>>2]|0;
      $135 = ($133|0)<($134|0);
      if (!($135)) {
       break;
      }
      $136 = HEAP32[$i>>2]|0;
      $137 = HEAP32[$aLen>>2]|0;
      $138 = (($137) + ($136)|0);
      $139 = HEAP8[$138>>0]|0;
      $140 = $139&255;
      HEAP32[$len1>>2] = $140;
      $141 = HEAP32[$len1>>2]|0;
      $142 = HEAP32[$nIn>>2]|0;
      $143 = ($141|0)<=($142|0);
      if ($143) {
       $144 = HEAP32[$nIn>>2]|0;
       $145 = HEAP32[$len1>>2]|0;
       $146 = (($144) - ($145))|0;
       $147 = HEAP32[$zIn>>2]|0;
       $148 = (($147) + ($146)|0);
       $149 = HEAP32[$i>>2]|0;
       $150 = HEAP32[$azChar>>2]|0;
       $151 = (($150) + ($149<<2)|0);
       $152 = HEAP32[$151>>2]|0;
       $153 = HEAP32[$len1>>2]|0;
       $154 = (_memcmp($148,$152,$153)|0);
       $155 = ($154|0)==(0);
       if ($155) {
        break;
       }
      }
      $156 = HEAP32[$i>>2]|0;
      $157 = (($156) + 1)|0;
      HEAP32[$i>>2] = $157;
     }
     $158 = HEAP32[$i>>2]|0;
     $159 = HEAP32[$nChar>>2]|0;
     $160 = ($158|0)>=($159|0);
     if ($160) {
      break L49;
     }
     $161 = HEAP32[$len1>>2]|0;
     $162 = HEAP32[$nIn>>2]|0;
     $163 = (($162) - ($161))|0;
     HEAP32[$nIn>>2] = $163;
     $$old2 = HEAP32[$nIn>>2]|0;
     $$old3 = ($$old2|0)>(0);
     if (!($$old3)) {
      break;
     }
    }
   }
  } while(0);
  $164 = HEAP32[$zCharSet>>2]|0;
  $165 = ($164|0)!=(0|0);
  if ($165) {
   $166 = HEAP32[$azChar>>2]|0;
   _sqlite3_free($166);
  }
 }
 $167 = HEAP32[$0>>2]|0;
 $168 = HEAP32[$zIn>>2]|0;
 $169 = HEAP32[$nIn>>2]|0;
 _sqlite3_result_text($167,$168,$169,(-1));
 STACKTOP = sp;return;
}
function _minmaxFunc($context,$argc,$argv) {
 $context = $context|0;
 $argc = $argc|0;
 $argv = $argv|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $5 = 0, $6 = 0;
 var $7 = 0, $8 = 0, $9 = 0, $i = 0, $iBest = 0, $mask = 0, $pColl = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 24|0;
 $1 = sp + 20|0;
 $2 = sp + 16|0;
 $i = sp + 12|0;
 $mask = sp + 8|0;
 $iBest = sp + 4|0;
 $pColl = sp;
 HEAP32[$0>>2] = $context;
 HEAP32[$1>>2] = $argc;
 HEAP32[$2>>2] = $argv;
 $3 = HEAP32[$0>>2]|0;
 $4 = (_sqlite3_user_data($3)|0);
 $5 = ($4|0)==(0|0);
 $6 = $5 ? 0 : -1;
 HEAP32[$mask>>2] = $6;
 $7 = HEAP32[$0>>2]|0;
 $8 = (_sqlite3GetFuncCollSeq($7)|0);
 HEAP32[$pColl>>2] = $8;
 HEAP32[$iBest>>2] = 0;
 $9 = HEAP32[$2>>2]|0;
 $10 = HEAP32[$9>>2]|0;
 $11 = (_sqlite3_value_type($10)|0);
 $12 = ($11|0)==(5);
 if ($12) {
  STACKTOP = sp;return;
 }
 HEAP32[$i>>2] = 1;
 while(1) {
  $13 = HEAP32[$i>>2]|0;
  $14 = HEAP32[$1>>2]|0;
  $15 = ($13|0)<($14|0);
  if (!($15)) {
   break;
  }
  $16 = HEAP32[$i>>2]|0;
  $17 = HEAP32[$2>>2]|0;
  $18 = (($17) + ($16<<2)|0);
  $19 = HEAP32[$18>>2]|0;
  $20 = (_sqlite3_value_type($19)|0);
  $21 = ($20|0)==(5);
  if ($21) {
   label = 9;
   break;
  }
  $22 = HEAP32[$iBest>>2]|0;
  $23 = HEAP32[$2>>2]|0;
  $24 = (($23) + ($22<<2)|0);
  $25 = HEAP32[$24>>2]|0;
  $26 = HEAP32[$i>>2]|0;
  $27 = HEAP32[$2>>2]|0;
  $28 = (($27) + ($26<<2)|0);
  $29 = HEAP32[$28>>2]|0;
  $30 = HEAP32[$pColl>>2]|0;
  $31 = (_sqlite3MemCompare($25,$29,$30)|0);
  $32 = HEAP32[$mask>>2]|0;
  $33 = $31 ^ $32;
  $34 = ($33|0)>=(0);
  if ($34) {
   $35 = HEAP32[$i>>2]|0;
   HEAP32[$iBest>>2] = $35;
  }
  $36 = HEAP32[$i>>2]|0;
  $37 = (($36) + 1)|0;
  HEAP32[$i>>2] = $37;
 }
 if ((label|0) == 9) {
  STACKTOP = sp;return;
 }
 $38 = HEAP32[$0>>2]|0;
 $39 = HEAP32[$iBest>>2]|0;
 $40 = HEAP32[$2>>2]|0;
 $41 = (($40) + ($39<<2)|0);
 $42 = HEAP32[$41>>2]|0;
 _sqlite3_result_value($38,$42);
 STACKTOP = sp;return;
}
function _minmaxStep($context,$NotUsed,$argv) {
 $context = $context|0;
 $NotUsed = $NotUsed|0;
 $argv = $argv|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $5 = 0;
 var $6 = 0, $7 = 0, $8 = 0, $9 = 0, $cmp = 0, $max = 0, $or$cond = 0, $or$cond3 = 0, $pArg = 0, $pBest = 0, $pColl = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 28|0;
 $1 = sp + 24|0;
 $2 = sp + 20|0;
 $pArg = sp + 16|0;
 $pBest = sp + 12|0;
 $max = sp + 8|0;
 $cmp = sp + 4|0;
 $pColl = sp;
 HEAP32[$0>>2] = $context;
 HEAP32[$1>>2] = $NotUsed;
 HEAP32[$2>>2] = $argv;
 $3 = HEAP32[$2>>2]|0;
 $4 = HEAP32[$3>>2]|0;
 HEAP32[$pArg>>2] = $4;
 $5 = HEAP32[$0>>2]|0;
 $6 = (_sqlite3_aggregate_context($5,40)|0);
 HEAP32[$pBest>>2] = $6;
 $7 = HEAP32[$pBest>>2]|0;
 $8 = ($7|0)!=(0|0);
 if (!($8)) {
  STACKTOP = sp;return;
 }
 $9 = HEAP32[$2>>2]|0;
 $10 = HEAP32[$9>>2]|0;
 $11 = (_sqlite3_value_type($10)|0);
 $12 = ($11|0)==(5);
 $13 = HEAP32[$pBest>>2]|0;
 $14 = (($13) + 8|0);
 $15 = HEAP16[$14>>1]|0;
 $16 = ($15<<16>>16)!=(0);
 if ($12) {
  if (!($16)) {
   STACKTOP = sp;return;
  }
  $17 = HEAP32[$0>>2]|0;
  _sqlite3SkipAccumulatorLoad($17);
  STACKTOP = sp;return;
 }
 $18 = HEAP32[$0>>2]|0;
 if (!($16)) {
  $39 = (_sqlite3_context_db_handle($18)|0);
  $40 = HEAP32[$pBest>>2]|0;
  $41 = (($40) + 32|0);
  HEAP32[$41>>2] = $39;
  $42 = HEAP32[$pBest>>2]|0;
  $43 = HEAP32[$pArg>>2]|0;
  (_sqlite3VdbeMemCopy($42,$43)|0);
  STACKTOP = sp;return;
 }
 $19 = (_sqlite3GetFuncCollSeq($18)|0);
 HEAP32[$pColl>>2] = $19;
 $20 = HEAP32[$0>>2]|0;
 $21 = (_sqlite3_user_data($20)|0);
 $22 = ($21|0)!=(0|0);
 $23 = $22&1;
 HEAP32[$max>>2] = $23;
 $24 = HEAP32[$pBest>>2]|0;
 $25 = HEAP32[$pArg>>2]|0;
 $26 = HEAP32[$pColl>>2]|0;
 $27 = (_sqlite3MemCompare($24,$25,$26)|0);
 HEAP32[$cmp>>2] = $27;
 $28 = HEAP32[$max>>2]|0;
 $29 = ($28|0)!=(0);
 $30 = HEAP32[$cmp>>2]|0;
 $31 = ($30|0)<(0);
 $or$cond = $29 & $31;
 if (!($or$cond)) {
  $32 = HEAP32[$max>>2]|0;
  $33 = ($32|0)==(0);
  $34 = HEAP32[$cmp>>2]|0;
  $35 = ($34|0)>(0);
  $or$cond3 = $33 & $35;
  if (!($or$cond3)) {
   $38 = HEAP32[$0>>2]|0;
   _sqlite3SkipAccumulatorLoad($38);
   STACKTOP = sp;return;
  }
 }
 $36 = HEAP32[$pBest>>2]|0;
 $37 = HEAP32[$pArg>>2]|0;
 (_sqlite3VdbeMemCopy($36,$37)|0);
 STACKTOP = sp;return;
}
function _minMaxFinalize($context) {
 $context = $context|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $pRes = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $pRes = sp;
 HEAP32[$0>>2] = $context;
 $1 = HEAP32[$0>>2]|0;
 $2 = (_sqlite3_aggregate_context($1,0)|0);
 HEAP32[$pRes>>2] = $2;
 $3 = HEAP32[$pRes>>2]|0;
 $4 = ($3|0)!=(0|0);
 if (!($4)) {
  STACKTOP = sp;return;
 }
 $5 = HEAP32[$pRes>>2]|0;
 $6 = (($5) + 8|0);
 $7 = HEAP16[$6>>1]|0;
 $8 = ($7<<16>>16)!=(0);
 if ($8) {
  $9 = HEAP32[$0>>2]|0;
  $10 = HEAP32[$pRes>>2]|0;
  _sqlite3_result_value($9,$10);
 }
 $11 = HEAP32[$pRes>>2]|0;
 _sqlite3VdbeMemRelease($11);
 STACKTOP = sp;return;
}
function _typeofFunc($context,$NotUsed,$argv) {
 $context = $context|0;
 $NotUsed = $NotUsed|0;
 $argv = $argv|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $z = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $1 = sp + 8|0;
 $2 = sp + 4|0;
 $z = sp;
 HEAP32[$0>>2] = $context;
 HEAP32[$1>>2] = $NotUsed;
 HEAP32[$2>>2] = $argv;
 HEAP32[$z>>2] = 0;
 $3 = HEAP32[$2>>2]|0;
 $4 = HEAP32[$3>>2]|0;
 $5 = (_sqlite3_value_type($4)|0);
 if ((($5|0) == 4)) {
  HEAP32[$z>>2] = 9880;
 } else if ((($5|0) == 2)) {
  HEAP32[$z>>2] = 9872;
 } else if ((($5|0) == 1)) {
  HEAP32[$z>>2] = 9856;
 } else if ((($5|0) == 3)) {
  HEAP32[$z>>2] = 9864;
 } else {
  HEAP32[$z>>2] = 9888;
 }
 $6 = HEAP32[$0>>2]|0;
 $7 = HEAP32[$z>>2]|0;
 _sqlite3_result_text($6,$7,-1,0);
 STACKTOP = sp;return;
}
function _lengthFunc($context,$argc,$argv) {
 $context = $context|0;
 $argc = $argc|0;
 $argv = $argv|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $len = 0, $z = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 16|0;
 $1 = sp + 12|0;
 $2 = sp + 8|0;
 $len = sp + 4|0;
 $z = sp;
 HEAP32[$0>>2] = $context;
 HEAP32[$1>>2] = $argc;
 HEAP32[$2>>2] = $argv;
 $3 = HEAP32[$2>>2]|0;
 $4 = HEAP32[$3>>2]|0;
 $5 = (_sqlite3_value_type($4)|0);
 if ((($5|0) == 3)) {
  $10 = HEAP32[$2>>2]|0;
  $11 = HEAP32[$10>>2]|0;
  $12 = (_sqlite3_value_text($11)|0);
  HEAP32[$z>>2] = $12;
  $13 = HEAP32[$z>>2]|0;
  $14 = ($13|0)==(0|0);
  if ($14) {
   STACKTOP = sp;return;
  }
  HEAP32[$len>>2] = 0;
  L6: while(1) {
   $15 = HEAP32[$z>>2]|0;
   $16 = HEAP8[$15>>0]|0;
   $17 = ($16<<24>>24)!=(0);
   if (!($17)) {
    break;
   }
   $18 = HEAP32[$len>>2]|0;
   $19 = (($18) + 1)|0;
   HEAP32[$len>>2] = $19;
   $20 = HEAP32[$z>>2]|0;
   $21 = (($20) + 1|0);
   HEAP32[$z>>2] = $21;
   $22 = HEAP8[$20>>0]|0;
   $23 = $22&255;
   $24 = ($23|0)>=(192);
   if (!($24)) {
    continue;
   }
   while(1) {
    $25 = HEAP32[$z>>2]|0;
    $26 = HEAP8[$25>>0]|0;
    $27 = $26&255;
    $28 = $27 & 192;
    $29 = ($28|0)==(128);
    if (!($29)) {
     continue L6;
    }
    $30 = HEAP32[$z>>2]|0;
    $31 = (($30) + 1|0);
    HEAP32[$z>>2] = $31;
   }
  }
  $32 = HEAP32[$0>>2]|0;
  $33 = HEAP32[$len>>2]|0;
  _sqlite3_result_int($32,$33);
  STACKTOP = sp;return;
 } else if ((($5|0) == 2) | (($5|0) == 1) | (($5|0) == 4)) {
  $6 = HEAP32[$0>>2]|0;
  $7 = HEAP32[$2>>2]|0;
  $8 = HEAP32[$7>>2]|0;
  $9 = (_sqlite3_value_bytes($8)|0);
  _sqlite3_result_int($6,$9);
  STACKTOP = sp;return;
 } else {
  $34 = HEAP32[$0>>2]|0;
  _sqlite3_result_null($34);
  STACKTOP = sp;return;
 }
}
function _instrFunc($context,$argc,$argv) {
 $context = $context|0;
 $argc = $argc|0;
 $argv = $argv|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $7 = 0, $8 = 0, $9 = 0, $N = 0, $isText = 0, $nHaystack = 0, $nNeedle = 0, $or$cond = 0, $or$cond3 = 0, $typeHaystack = 0, $typeNeedle = 0, $zHaystack = 0, $zNeedle = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $0 = sp + 40|0;
 $1 = sp + 36|0;
 $2 = sp + 32|0;
 $zHaystack = sp + 28|0;
 $zNeedle = sp + 24|0;
 $nHaystack = sp + 20|0;
 $nNeedle = sp + 16|0;
 $typeHaystack = sp + 12|0;
 $typeNeedle = sp + 8|0;
 $N = sp + 4|0;
 $isText = sp;
 HEAP32[$0>>2] = $context;
 HEAP32[$1>>2] = $argc;
 HEAP32[$2>>2] = $argv;
 HEAP32[$N>>2] = 1;
 $3 = HEAP32[$2>>2]|0;
 $4 = HEAP32[$3>>2]|0;
 $5 = (_sqlite3_value_type($4)|0);
 HEAP32[$typeHaystack>>2] = $5;
 $6 = HEAP32[$2>>2]|0;
 $7 = (($6) + 4|0);
 $8 = HEAP32[$7>>2]|0;
 $9 = (_sqlite3_value_type($8)|0);
 HEAP32[$typeNeedle>>2] = $9;
 $10 = HEAP32[$typeHaystack>>2]|0;
 $11 = ($10|0)==(5);
 $12 = HEAP32[$typeNeedle>>2]|0;
 $13 = ($12|0)==(5);
 $or$cond = $11 | $13;
 if ($or$cond) {
  STACKTOP = sp;return;
 }
 $14 = HEAP32[$2>>2]|0;
 $15 = HEAP32[$14>>2]|0;
 $16 = (_sqlite3_value_bytes($15)|0);
 HEAP32[$nHaystack>>2] = $16;
 $17 = HEAP32[$2>>2]|0;
 $18 = (($17) + 4|0);
 $19 = HEAP32[$18>>2]|0;
 $20 = (_sqlite3_value_bytes($19)|0);
 HEAP32[$nNeedle>>2] = $20;
 $21 = HEAP32[$typeHaystack>>2]|0;
 $22 = ($21|0)==(4);
 $23 = HEAP32[$typeNeedle>>2]|0;
 $24 = ($23|0)==(4);
 $or$cond3 = $22 & $24;
 $25 = HEAP32[$2>>2]|0;
 $26 = HEAP32[$25>>2]|0;
 if ($or$cond3) {
  $27 = (_sqlite3_value_blob($26)|0);
  HEAP32[$zHaystack>>2] = $27;
  $28 = HEAP32[$2>>2]|0;
  $29 = (($28) + 4|0);
  $30 = HEAP32[$29>>2]|0;
  $31 = (_sqlite3_value_blob($30)|0);
  HEAP32[$zNeedle>>2] = $31;
  HEAP32[$isText>>2] = 0;
 } else {
  $32 = (_sqlite3_value_text($26)|0);
  HEAP32[$zHaystack>>2] = $32;
  $33 = HEAP32[$2>>2]|0;
  $34 = (($33) + 4|0);
  $35 = HEAP32[$34>>2]|0;
  $36 = (_sqlite3_value_text($35)|0);
  HEAP32[$zNeedle>>2] = $36;
  HEAP32[$isText>>2] = 1;
 }
 L7: while(1) {
  $37 = HEAP32[$nNeedle>>2]|0;
  $38 = HEAP32[$nHaystack>>2]|0;
  $39 = ($37|0)<=($38|0);
  if (!($39)) {
   break;
  }
  $40 = HEAP32[$zHaystack>>2]|0;
  $41 = HEAP32[$zNeedle>>2]|0;
  $42 = HEAP32[$nNeedle>>2]|0;
  $43 = (_memcmp($40,$41,$42)|0);
  $44 = ($43|0)!=(0);
  if (!($44)) {
   break;
  }
  $45 = HEAP32[$N>>2]|0;
  $46 = (($45) + 1)|0;
  HEAP32[$N>>2] = $46;
  while(1) {
   $47 = HEAP32[$nHaystack>>2]|0;
   $48 = (($47) + -1)|0;
   HEAP32[$nHaystack>>2] = $48;
   $49 = HEAP32[$zHaystack>>2]|0;
   $50 = (($49) + 1|0);
   HEAP32[$zHaystack>>2] = $50;
   $51 = HEAP32[$isText>>2]|0;
   $52 = ($51|0)!=(0);
   if (!($52)) {
    continue L7;
   }
   $53 = HEAP32[$zHaystack>>2]|0;
   $54 = HEAP8[$53>>0]|0;
   $55 = $54&255;
   $56 = $55 & 192;
   $57 = ($56|0)==(128);
   if (!($57)) {
    continue L7;
   }
  }
 }
 $58 = HEAP32[$nNeedle>>2]|0;
 $59 = HEAP32[$nHaystack>>2]|0;
 $60 = ($58|0)>($59|0);
 if ($60) {
  HEAP32[$N>>2] = 0;
 }
 $61 = HEAP32[$0>>2]|0;
 $62 = HEAP32[$N>>2]|0;
 _sqlite3_result_int($61,$62);
 STACKTOP = sp;return;
}
function _substrFunc($context,$argc,$argv) {
 $context = $context|0;
 $argc = $argc|0;
 $argv = $argv|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0;
 var $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0, $188 = 0;
 var $189 = 0, $19 = 0, $190 = 0, $191 = 0, $192 = 0, $193 = 0, $194 = 0, $195 = 0, $196 = 0, $197 = 0, $198 = 0, $199 = 0, $2 = 0, $20 = 0, $200 = 0, $201 = 0, $202 = 0, $203 = 0, $204 = 0, $205 = 0;
 var $206 = 0, $207 = 0, $208 = 0, $209 = 0, $21 = 0, $210 = 0, $211 = 0, $212 = 0, $213 = 0, $214 = 0, $215 = 0, $216 = 0, $217 = 0, $218 = 0, $219 = 0, $22 = 0, $220 = 0, $221 = 0, $222 = 0, $223 = 0;
 var $224 = 0, $225 = 0, $226 = 0, $227 = 0, $228 = 0, $229 = 0, $23 = 0, $230 = 0, $231 = 0, $232 = 0, $233 = 0, $234 = 0, $235 = 0, $236 = 0, $237 = 0, $238 = 0, $239 = 0, $24 = 0, $240 = 0, $241 = 0;
 var $242 = 0, $243 = 0, $244 = 0, $245 = 0, $246 = 0, $247 = 0, $248 = 0, $249 = 0, $25 = 0, $250 = 0, $251 = 0, $252 = 0, $253 = 0, $254 = 0, $255 = 0, $256 = 0, $257 = 0, $258 = 0, $259 = 0, $26 = 0;
 var $260 = 0, $261 = 0, $262 = 0, $263 = 0, $264 = 0, $265 = 0, $266 = 0, $267 = 0, $268 = 0, $269 = 0, $27 = 0, $270 = 0, $271 = 0, $272 = 0, $273 = 0, $274 = 0, $275 = 0, $276 = 0, $277 = 0, $278 = 0;
 var $279 = 0, $28 = 0, $280 = 0, $281 = 0, $282 = 0, $283 = 0, $284 = 0, $285 = 0, $286 = 0, $287 = 0, $288 = 0, $289 = 0, $29 = 0, $290 = 0, $291 = 0, $292 = 0, $293 = 0, $294 = 0, $295 = 0, $296 = 0;
 var $297 = 0, $298 = 0, $299 = 0, $3 = 0, $30 = 0, $300 = 0, $301 = 0, $302 = 0, $303 = 0, $304 = 0, $305 = 0, $306 = 0, $307 = 0, $308 = 0, $309 = 0, $31 = 0, $310 = 0, $311 = 0, $312 = 0, $313 = 0;
 var $314 = 0, $315 = 0, $316 = 0, $317 = 0, $318 = 0, $319 = 0, $32 = 0, $320 = 0, $321 = 0, $322 = 0, $323 = 0, $324 = 0, $325 = 0, $326 = 0, $327 = 0, $328 = 0, $329 = 0, $33 = 0, $330 = 0, $331 = 0;
 var $332 = 0, $333 = 0, $334 = 0, $335 = 0, $336 = 0, $337 = 0, $338 = 0, $339 = 0, $34 = 0, $340 = 0, $341 = 0, $342 = 0, $343 = 0, $344 = 0, $345 = 0, $346 = 0, $347 = 0, $348 = 0, $349 = 0, $35 = 0;
 var $350 = 0, $351 = 0, $352 = 0, $353 = 0, $354 = 0, $355 = 0, $356 = 0, $357 = 0, $358 = 0, $359 = 0, $36 = 0, $360 = 0, $361 = 0, $362 = 0, $363 = 0, $364 = 0, $365 = 0, $366 = 0, $367 = 0, $368 = 0;
 var $369 = 0, $37 = 0, $370 = 0, $371 = 0, $372 = 0, $373 = 0, $374 = 0, $375 = 0, $376 = 0, $377 = 0, $378 = 0, $379 = 0, $38 = 0, $380 = 0, $381 = 0, $382 = 0, $383 = 0, $384 = 0, $385 = 0, $386 = 0;
 var $387 = 0, $388 = 0, $389 = 0, $39 = 0, $390 = 0, $391 = 0, $392 = 0, $393 = 0, $394 = 0, $395 = 0, $396 = 0, $397 = 0, $398 = 0, $399 = 0, $4 = 0, $40 = 0, $400 = 0, $401 = 0, $402 = 0, $403 = 0;
 var $404 = 0, $405 = 0, $406 = 0, $407 = 0, $408 = 0, $409 = 0, $41 = 0, $410 = 0, $411 = 0, $412 = 0, $413 = 0, $414 = 0, $415 = 0, $416 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0;
 var $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0;
 var $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0;
 var $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $len = 0, $negP2 = 0, $or$cond = 0;
 var $p0type = 0, $p1 = 0, $p2 = 0, $z = 0, $z2 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $0 = sp + 44|0;
 $1 = sp + 40|0;
 $2 = sp + 36|0;
 $z = sp + 32|0;
 $z2 = sp + 28|0;
 $len = sp + 24|0;
 $p0type = sp + 20|0;
 $p1 = sp + 8|0;
 $p2 = sp;
 $negP2 = sp + 16|0;
 HEAP32[$0>>2] = $context;
 HEAP32[$1>>2] = $argc;
 HEAP32[$2>>2] = $argv;
 HEAP32[$negP2>>2] = 0;
 $3 = HEAP32[$2>>2]|0;
 $4 = (($3) + 4|0);
 $5 = HEAP32[$4>>2]|0;
 $6 = (_sqlite3_value_type($5)|0);
 $7 = ($6|0)==(5);
 if ($7) {
  STACKTOP = sp;return;
 }
 $8 = HEAP32[$1>>2]|0;
 $9 = ($8|0)==(3);
 if ($9) {
  $10 = HEAP32[$2>>2]|0;
  $11 = (($10) + 8|0);
  $12 = HEAP32[$11>>2]|0;
  $13 = (_sqlite3_value_type($12)|0);
  $14 = ($13|0)==(5);
  if ($14) {
   STACKTOP = sp;return;
  }
 }
 $15 = HEAP32[$2>>2]|0;
 $16 = HEAP32[$15>>2]|0;
 $17 = (_sqlite3_value_type($16)|0);
 HEAP32[$p0type>>2] = $17;
 $18 = HEAP32[$2>>2]|0;
 $19 = (($18) + 4|0);
 $20 = HEAP32[$19>>2]|0;
 $21 = (_sqlite3_value_int($20)|0);
 $22 = ($21|0)<(0);
 $23 = $22 << 31 >> 31;
 $24 = $p1;
 $25 = $24;
 HEAP32[$25>>2] = $21;
 $26 = (($24) + 4)|0;
 $27 = $26;
 HEAP32[$27>>2] = $23;
 $28 = HEAP32[$p0type>>2]|0;
 $29 = ($28|0)==(4);
 $30 = HEAP32[$2>>2]|0;
 $31 = HEAP32[$30>>2]|0;
 L8: do {
  if ($29) {
   $32 = (_sqlite3_value_bytes($31)|0);
   HEAP32[$len>>2] = $32;
   $33 = HEAP32[$2>>2]|0;
   $34 = HEAP32[$33>>2]|0;
   $35 = (_sqlite3_value_blob($34)|0);
   HEAP32[$z>>2] = $35;
   $36 = HEAP32[$z>>2]|0;
   $37 = ($36|0)==(0|0);
   if ($37) {
    STACKTOP = sp;return;
   }
  } else {
   $38 = (_sqlite3_value_text($31)|0);
   HEAP32[$z>>2] = $38;
   $39 = HEAP32[$z>>2]|0;
   $40 = ($39|0)==(0|0);
   if ($40) {
    STACKTOP = sp;return;
   }
   HEAP32[$len>>2] = 0;
   $41 = $p1;
   $42 = $41;
   $43 = HEAP32[$42>>2]|0;
   $44 = (($41) + 4)|0;
   $45 = $44;
   $46 = HEAP32[$45>>2]|0;
   $47 = ($46|0)<(0);
   if ($47) {
    $48 = HEAP32[$z>>2]|0;
    HEAP32[$z2>>2] = $48;
    while(1) {
     $49 = HEAP32[$z2>>2]|0;
     $50 = HEAP8[$49>>0]|0;
     $51 = ($50<<24>>24)!=(0);
     if (!($51)) {
      break L8;
     }
     $52 = HEAP32[$z2>>2]|0;
     $53 = (($52) + 1|0);
     HEAP32[$z2>>2] = $53;
     $54 = HEAP8[$52>>0]|0;
     $55 = $54&255;
     $56 = ($55|0)>=(192);
     L17: do {
      if ($56) {
       while(1) {
        $57 = HEAP32[$z2>>2]|0;
        $58 = HEAP8[$57>>0]|0;
        $59 = $58&255;
        $60 = $59 & 192;
        $61 = ($60|0)==(128);
        if (!($61)) {
         break L17;
        }
        $62 = HEAP32[$z2>>2]|0;
        $63 = (($62) + 1|0);
        HEAP32[$z2>>2] = $63;
       }
      }
     } while(0);
     $64 = HEAP32[$len>>2]|0;
     $65 = (($64) + 1)|0;
     HEAP32[$len>>2] = $65;
    }
   }
  }
 } while(0);
 $66 = HEAP32[$1>>2]|0;
 $67 = ($66|0)==(3);
 if ($67) {
  $68 = HEAP32[$2>>2]|0;
  $69 = (($68) + 8|0);
  $70 = HEAP32[$69>>2]|0;
  $71 = (_sqlite3_value_int($70)|0);
  $72 = ($71|0)<(0);
  $73 = $72 << 31 >> 31;
  $74 = $p2;
  $75 = $74;
  HEAP32[$75>>2] = $71;
  $76 = (($74) + 4)|0;
  $77 = $76;
  HEAP32[$77>>2] = $73;
  $78 = $p2;
  $79 = $78;
  $80 = HEAP32[$79>>2]|0;
  $81 = (($78) + 4)|0;
  $82 = $81;
  $83 = HEAP32[$82>>2]|0;
  $84 = ($83|0)<(0);
  if ($84) {
   $85 = $p2;
   $86 = $85;
   $87 = HEAP32[$86>>2]|0;
   $88 = (($85) + 4)|0;
   $89 = $88;
   $90 = HEAP32[$89>>2]|0;
   $91 = (_i64Subtract(0,0,($87|0),($90|0))|0);
   $92 = tempRet0;
   $93 = $p2;
   $94 = $93;
   HEAP32[$94>>2] = $91;
   $95 = (($93) + 4)|0;
   $96 = $95;
   HEAP32[$96>>2] = $92;
   HEAP32[$negP2>>2] = 1;
  }
 } else {
  $97 = HEAP32[$0>>2]|0;
  $98 = (_sqlite3_context_db_handle($97)|0);
  $99 = (($98) + 88|0);
  $100 = HEAP32[$99>>2]|0;
  $101 = ($100|0)<(0);
  $102 = $101 << 31 >> 31;
  $103 = $p2;
  $104 = $103;
  HEAP32[$104>>2] = $100;
  $105 = (($103) + 4)|0;
  $106 = $105;
  HEAP32[$106>>2] = $102;
 }
 $107 = $p1;
 $108 = $107;
 $109 = HEAP32[$108>>2]|0;
 $110 = (($107) + 4)|0;
 $111 = $110;
 $112 = HEAP32[$111>>2]|0;
 $113 = ($112|0)<(0);
 do {
  if ($113) {
   $114 = HEAP32[$len>>2]|0;
   $115 = ($114|0)<(0);
   $116 = $115 << 31 >> 31;
   $117 = $p1;
   $118 = $117;
   $119 = HEAP32[$118>>2]|0;
   $120 = (($117) + 4)|0;
   $121 = $120;
   $122 = HEAP32[$121>>2]|0;
   $123 = (_i64Add(($119|0),($122|0),($114|0),($116|0))|0);
   $124 = tempRet0;
   $125 = $p1;
   $126 = $125;
   HEAP32[$126>>2] = $123;
   $127 = (($125) + 4)|0;
   $128 = $127;
   HEAP32[$128>>2] = $124;
   $129 = $p1;
   $130 = $129;
   $131 = HEAP32[$130>>2]|0;
   $132 = (($129) + 4)|0;
   $133 = $132;
   $134 = HEAP32[$133>>2]|0;
   $135 = ($134|0)<(0);
   if ($135) {
    $136 = $p1;
    $137 = $136;
    $138 = HEAP32[$137>>2]|0;
    $139 = (($136) + 4)|0;
    $140 = $139;
    $141 = HEAP32[$140>>2]|0;
    $142 = $p2;
    $143 = $142;
    $144 = HEAP32[$143>>2]|0;
    $145 = (($142) + 4)|0;
    $146 = $145;
    $147 = HEAP32[$146>>2]|0;
    $148 = (_i64Add(($144|0),($147|0),($138|0),($141|0))|0);
    $149 = tempRet0;
    $150 = $p2;
    $151 = $150;
    HEAP32[$151>>2] = $148;
    $152 = (($150) + 4)|0;
    $153 = $152;
    HEAP32[$153>>2] = $149;
    $154 = $p2;
    $155 = $154;
    $156 = HEAP32[$155>>2]|0;
    $157 = (($154) + 4)|0;
    $158 = $157;
    $159 = HEAP32[$158>>2]|0;
    $160 = ($159|0)<(0);
    $161 = $160 ? 0 : $148;
    $162 = $160 ? 0 : $149;
    $163 = $p2;
    $164 = $163;
    HEAP32[$164>>2] = $161;
    $165 = (($163) + 4)|0;
    $166 = $165;
    HEAP32[$166>>2] = $162;
    $167 = $p1;
    $168 = $167;
    HEAP32[$168>>2] = 0;
    $169 = (($167) + 4)|0;
    $170 = $169;
    HEAP32[$170>>2] = 0;
   }
  } else {
   $171 = $p1;
   $172 = $171;
   $173 = HEAP32[$172>>2]|0;
   $174 = (($171) + 4)|0;
   $175 = $174;
   $176 = HEAP32[$175>>2]|0;
   $177 = ($176|0)>(0);
   $178 = ($173>>>0)>(0);
   $179 = ($176|0)==(0);
   $180 = $179 & $178;
   $181 = $177 | $180;
   if ($181) {
    $182 = $p1;
    $183 = $182;
    $184 = HEAP32[$183>>2]|0;
    $185 = (($182) + 4)|0;
    $186 = $185;
    $187 = HEAP32[$186>>2]|0;
    $188 = (_i64Add(($184|0),($187|0),-1,-1)|0);
    $189 = tempRet0;
    $190 = $p1;
    $191 = $190;
    HEAP32[$191>>2] = $188;
    $192 = (($190) + 4)|0;
    $193 = $192;
    HEAP32[$193>>2] = $189;
    break;
   }
   $194 = $p2;
   $195 = $194;
   $196 = HEAP32[$195>>2]|0;
   $197 = (($194) + 4)|0;
   $198 = $197;
   $199 = HEAP32[$198>>2]|0;
   $200 = ($199|0)>(0);
   $201 = ($196>>>0)>(0);
   $202 = ($199|0)==(0);
   $203 = $202 & $201;
   $204 = $200 | $203;
   if ($204) {
    $205 = $p2;
    $206 = $205;
    $207 = HEAP32[$206>>2]|0;
    $208 = (($205) + 4)|0;
    $209 = $208;
    $210 = HEAP32[$209>>2]|0;
    $211 = (_i64Add(($207|0),($210|0),-1,-1)|0);
    $212 = tempRet0;
    $213 = $p2;
    $214 = $213;
    HEAP32[$214>>2] = $211;
    $215 = (($213) + 4)|0;
    $216 = $215;
    HEAP32[$216>>2] = $212;
   }
  }
 } while(0);
 $217 = HEAP32[$negP2>>2]|0;
 $218 = ($217|0)!=(0);
 if ($218) {
  $219 = $p2;
  $220 = $219;
  $221 = HEAP32[$220>>2]|0;
  $222 = (($219) + 4)|0;
  $223 = $222;
  $224 = HEAP32[$223>>2]|0;
  $225 = $p1;
  $226 = $225;
  $227 = HEAP32[$226>>2]|0;
  $228 = (($225) + 4)|0;
  $229 = $228;
  $230 = HEAP32[$229>>2]|0;
  $231 = (_i64Subtract(($227|0),($230|0),($221|0),($224|0))|0);
  $232 = tempRet0;
  $233 = $p1;
  $234 = $233;
  HEAP32[$234>>2] = $231;
  $235 = (($233) + 4)|0;
  $236 = $235;
  HEAP32[$236>>2] = $232;
  $237 = $p1;
  $238 = $237;
  $239 = HEAP32[$238>>2]|0;
  $240 = (($237) + 4)|0;
  $241 = $240;
  $242 = HEAP32[$241>>2]|0;
  $243 = ($242|0)<(0);
  if ($243) {
   $244 = $p1;
   $245 = $244;
   $246 = HEAP32[$245>>2]|0;
   $247 = (($244) + 4)|0;
   $248 = $247;
   $249 = HEAP32[$248>>2]|0;
   $250 = $p2;
   $251 = $250;
   $252 = HEAP32[$251>>2]|0;
   $253 = (($250) + 4)|0;
   $254 = $253;
   $255 = HEAP32[$254>>2]|0;
   $256 = (_i64Add(($252|0),($255|0),($246|0),($249|0))|0);
   $257 = tempRet0;
   $258 = $p2;
   $259 = $258;
   HEAP32[$259>>2] = $256;
   $260 = (($258) + 4)|0;
   $261 = $260;
   HEAP32[$261>>2] = $257;
   $262 = $p1;
   $263 = $262;
   HEAP32[$263>>2] = 0;
   $264 = (($262) + 4)|0;
   $265 = $264;
   HEAP32[$265>>2] = 0;
  }
 }
 $266 = HEAP32[$p0type>>2]|0;
 $267 = ($266|0)!=(4);
 if (!($267)) {
  $351 = $p1;
  $352 = $351;
  $353 = HEAP32[$352>>2]|0;
  $354 = (($351) + 4)|0;
  $355 = $354;
  $356 = HEAP32[$355>>2]|0;
  $357 = $p2;
  $358 = $357;
  $359 = HEAP32[$358>>2]|0;
  $360 = (($357) + 4)|0;
  $361 = $360;
  $362 = HEAP32[$361>>2]|0;
  $363 = (_i64Add(($353|0),($356|0),($359|0),($362|0))|0);
  $364 = tempRet0;
  $365 = HEAP32[$len>>2]|0;
  $366 = ($365|0)<(0);
  $367 = $366 << 31 >> 31;
  $368 = ($364|0)>($367|0);
  $369 = ($363>>>0)>($365>>>0);
  $370 = ($364|0)==($367|0);
  $371 = $370 & $369;
  $372 = $368 | $371;
  if ($372) {
   $373 = HEAP32[$len>>2]|0;
   $374 = ($373|0)<(0);
   $375 = $374 << 31 >> 31;
   $376 = $p1;
   $377 = $376;
   $378 = HEAP32[$377>>2]|0;
   $379 = (($376) + 4)|0;
   $380 = $379;
   $381 = HEAP32[$380>>2]|0;
   $382 = (_i64Subtract(($373|0),($375|0),($378|0),($381|0))|0);
   $383 = tempRet0;
   $384 = $p2;
   $385 = $384;
   HEAP32[$385>>2] = $382;
   $386 = (($384) + 4)|0;
   $387 = $386;
   HEAP32[$387>>2] = $383;
   $388 = $p2;
   $389 = $388;
   $390 = HEAP32[$389>>2]|0;
   $391 = (($388) + 4)|0;
   $392 = $391;
   $393 = HEAP32[$392>>2]|0;
   $394 = ($393|0)<(0);
   $395 = $394 ? 0 : $382;
   $396 = $394 ? 0 : $383;
   $397 = $p2;
   $398 = $397;
   HEAP32[$398>>2] = $395;
   $399 = (($397) + 4)|0;
   $400 = $399;
   HEAP32[$400>>2] = $396;
  }
  $401 = HEAP32[$0>>2]|0;
  $402 = $p1;
  $403 = $402;
  $404 = HEAP32[$403>>2]|0;
  $405 = (($402) + 4)|0;
  $406 = $405;
  $407 = HEAP32[$406>>2]|0;
  $408 = HEAP32[$z>>2]|0;
  $409 = (($408) + ($404)|0);
  $410 = $p2;
  $411 = $410;
  $412 = HEAP32[$411>>2]|0;
  $413 = (($410) + 4)|0;
  $414 = $413;
  $415 = HEAP32[$414>>2]|0;
  _sqlite3_result_blob64($401,$409,$412,$415,(-1));
  STACKTOP = sp;return;
 }
 while(1) {
  $268 = HEAP32[$z>>2]|0;
  $269 = HEAP8[$268>>0]|0;
  $270 = $269&255;
  $271 = ($270|0)!=(0);
  if ($271) {
   $272 = $p1;
   $273 = $272;
   $274 = HEAP32[$273>>2]|0;
   $275 = (($272) + 4)|0;
   $276 = $275;
   $277 = HEAP32[$276>>2]|0;
   $278 = ($274|0)!=(0);
   $279 = ($277|0)!=(0);
   $280 = $278 | $279;
   $416 = $280;
  } else {
   $416 = 0;
  }
  $281 = HEAP32[$z>>2]|0;
  if (!($416)) {
   break;
  }
  $282 = (($281) + 1|0);
  HEAP32[$z>>2] = $282;
  $283 = HEAP8[$281>>0]|0;
  $284 = $283&255;
  $285 = ($284|0)>=(192);
  L55: do {
   if ($285) {
    while(1) {
     $286 = HEAP32[$z>>2]|0;
     $287 = HEAP8[$286>>0]|0;
     $288 = $287&255;
     $289 = $288 & 192;
     $290 = ($289|0)==(128);
     if (!($290)) {
      break L55;
     }
     $291 = HEAP32[$z>>2]|0;
     $292 = (($291) + 1|0);
     HEAP32[$z>>2] = $292;
    }
   }
  } while(0);
  $293 = $p1;
  $294 = $293;
  $295 = HEAP32[$294>>2]|0;
  $296 = (($293) + 4)|0;
  $297 = $296;
  $298 = HEAP32[$297>>2]|0;
  $299 = (_i64Add(($295|0),($298|0),-1,-1)|0);
  $300 = tempRet0;
  $301 = $p1;
  $302 = $301;
  HEAP32[$302>>2] = $299;
  $303 = (($301) + 4)|0;
  $304 = $303;
  HEAP32[$304>>2] = $300;
 }
 HEAP32[$z2>>2] = $281;
 while(1) {
  $305 = HEAP32[$z2>>2]|0;
  $306 = HEAP8[$305>>0]|0;
  $307 = $306&255;
  $308 = ($307|0)!=(0);
  $309 = $p2;
  $310 = $309;
  $311 = HEAP32[$310>>2]|0;
  $312 = (($309) + 4)|0;
  $313 = $312;
  $314 = HEAP32[$313>>2]|0;
  $315 = ($311|0)!=(0);
  $316 = ($314|0)!=(0);
  $317 = $315 | $316;
  $or$cond = $308 & $317;
  if (!($or$cond)) {
   break;
  }
  $318 = HEAP32[$z2>>2]|0;
  $319 = (($318) + 1|0);
  HEAP32[$z2>>2] = $319;
  $320 = HEAP8[$318>>0]|0;
  $321 = $320&255;
  $322 = ($321|0)>=(192);
  L64: do {
   if ($322) {
    while(1) {
     $323 = HEAP32[$z2>>2]|0;
     $324 = HEAP8[$323>>0]|0;
     $325 = $324&255;
     $326 = $325 & 192;
     $327 = ($326|0)==(128);
     if (!($327)) {
      break L64;
     }
     $328 = HEAP32[$z2>>2]|0;
     $329 = (($328) + 1|0);
     HEAP32[$z2>>2] = $329;
    }
   }
  } while(0);
  $330 = $p2;
  $331 = $330;
  $332 = HEAP32[$331>>2]|0;
  $333 = (($330) + 4)|0;
  $334 = $333;
  $335 = HEAP32[$334>>2]|0;
  $336 = (_i64Add(($332|0),($335|0),-1,-1)|0);
  $337 = tempRet0;
  $338 = $p2;
  $339 = $338;
  HEAP32[$339>>2] = $336;
  $340 = (($338) + 4)|0;
  $341 = $340;
  HEAP32[$341>>2] = $337;
 }
 $342 = HEAP32[$0>>2]|0;
 $343 = HEAP32[$z>>2]|0;
 $344 = HEAP32[$z2>>2]|0;
 $345 = HEAP32[$z>>2]|0;
 $346 = $344;
 $347 = $345;
 $348 = (($346) - ($347))|0;
 $349 = ($348|0)<(0);
 $350 = $349 << 31 >> 31;
 _sqlite3_result_text64($342,$343,$348,$350,(-1),1);
 STACKTOP = sp;return;
}
function _printfFunc($context,$argc,$argv) {
 $context = $context|0;
 $argc = $argc|0;
 $argv = $argv|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0;
 var $7 = 0, $8 = 0, $9 = 0, $n = 0, $str = 0, $vararg_buffer = 0, $x = 0, $zFormat = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 64|0;
 $vararg_buffer = sp;
 $0 = sp + 60|0;
 $1 = sp + 56|0;
 $2 = sp + 52|0;
 $x = sp + 40|0;
 $str = sp + 12|0;
 $zFormat = sp + 8|0;
 $n = sp + 4|0;
 HEAP32[$0>>2] = $context;
 HEAP32[$1>>2] = $argc;
 HEAP32[$2>>2] = $argv;
 $3 = HEAP32[$1>>2]|0;
 $4 = ($3|0)>=(1);
 if (!($4)) {
  STACKTOP = sp;return;
 }
 $5 = HEAP32[$2>>2]|0;
 $6 = HEAP32[$5>>2]|0;
 $7 = (_sqlite3_value_text($6)|0);
 HEAP32[$zFormat>>2] = $7;
 $8 = ($7|0)!=(0|0);
 if (!($8)) {
  STACKTOP = sp;return;
 }
 $9 = HEAP32[$1>>2]|0;
 $10 = (($9) - 1)|0;
 HEAP32[$x>>2] = $10;
 $11 = (($x) + 4|0);
 HEAP32[$11>>2] = 0;
 $12 = HEAP32[$2>>2]|0;
 $13 = (($12) + 4|0);
 $14 = (($x) + 8|0);
 HEAP32[$14>>2] = $13;
 _sqlite3StrAccumInit($str,0,0,1000000000);
 $15 = HEAP32[$0>>2]|0;
 $16 = (_sqlite3_context_db_handle($15)|0);
 HEAP32[$str>>2] = $16;
 $17 = HEAP32[$zFormat>>2]|0;
 HEAP32[$vararg_buffer>>2] = $x;
 _sqlite3XPrintf($str,2,$17,$vararg_buffer);
 $18 = (($str) + 12|0);
 $19 = HEAP32[$18>>2]|0;
 HEAP32[$n>>2] = $19;
 $20 = HEAP32[$0>>2]|0;
 $21 = (_sqlite3StrAccumFinish($str)|0);
 $22 = HEAP32[$n>>2]|0;
 _sqlite3_result_text($20,$21,$22,16);
 STACKTOP = sp;return;
}
function _unicodeFunc($context,$argc,$argv) {
 $context = $context|0;
 $argc = $argc|0;
 $argv = $argv|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $z = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $1 = sp + 8|0;
 $2 = sp + 4|0;
 $z = sp;
 HEAP32[$0>>2] = $context;
 HEAP32[$1>>2] = $argc;
 HEAP32[$2>>2] = $argv;
 $3 = HEAP32[$2>>2]|0;
 $4 = HEAP32[$3>>2]|0;
 $5 = (_sqlite3_value_text($4)|0);
 HEAP32[$z>>2] = $5;
 $6 = HEAP32[$z>>2]|0;
 $7 = ($6|0)!=(0|0);
 if (!($7)) {
  STACKTOP = sp;return;
 }
 $8 = HEAP32[$z>>2]|0;
 $9 = HEAP8[$8>>0]|0;
 $10 = $9&255;
 $11 = ($10|0)!=(0);
 if (!($11)) {
  STACKTOP = sp;return;
 }
 $12 = HEAP32[$0>>2]|0;
 $13 = (_sqlite3Utf8Read($z)|0);
 _sqlite3_result_int($12,$13);
 STACKTOP = sp;return;
}
function _charFunc($context,$argc,$argv) {
 $context = $context|0;
 $argc = $argc|0;
 $argv = $argv|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $16 = 0;
 var $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0;
 var $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0;
 var $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0;
 var $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0;
 var $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $c = 0, $i = 0, $or$cond = 0, $x = 0, $z = 0, $zOut = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $0 = sp + 32|0;
 $1 = sp + 28|0;
 $2 = sp + 24|0;
 $z = sp + 20|0;
 $zOut = sp + 16|0;
 $i = sp + 12|0;
 $x = sp;
 $c = sp + 8|0;
 HEAP32[$0>>2] = $context;
 HEAP32[$1>>2] = $argc;
 HEAP32[$2>>2] = $argv;
 $3 = HEAP32[$1>>2]|0;
 $4 = $3<<2;
 $5 = (($4) + 1)|0;
 $6 = (_sqlite3_malloc($5)|0);
 HEAP32[$z>>2] = $6;
 HEAP32[$zOut>>2] = $6;
 $7 = HEAP32[$z>>2]|0;
 $8 = ($7|0)==(0|0);
 if ($8) {
  $9 = HEAP32[$0>>2]|0;
  _sqlite3_result_error_nomem($9);
  STACKTOP = sp;return;
 }
 HEAP32[$i>>2] = 0;
 while(1) {
  $10 = HEAP32[$i>>2]|0;
  $11 = HEAP32[$1>>2]|0;
  $12 = ($10|0)<($11|0);
  if (!($12)) {
   break;
  }
  $13 = HEAP32[$i>>2]|0;
  $14 = HEAP32[$2>>2]|0;
  $15 = (($14) + ($13<<2)|0);
  $16 = HEAP32[$15>>2]|0;
  $17 = (_sqlite3_value_int64($16)|0);
  $18 = tempRet0;
  $19 = $x;
  $20 = $19;
  HEAP32[$20>>2] = $17;
  $21 = (($19) + 4)|0;
  $22 = $21;
  HEAP32[$22>>2] = $18;
  $23 = $x;
  $24 = $23;
  $25 = HEAP32[$24>>2]|0;
  $26 = (($23) + 4)|0;
  $27 = $26;
  $28 = HEAP32[$27>>2]|0;
  $29 = ($28|0)<(0);
  $30 = $x;
  $31 = $30;
  $32 = HEAP32[$31>>2]|0;
  $33 = (($30) + 4)|0;
  $34 = $33;
  $35 = HEAP32[$34>>2]|0;
  $36 = ($35|0)>(0);
  $37 = ($32>>>0)>(1114111);
  $38 = ($35|0)==(0);
  $39 = $38 & $37;
  $40 = $36 | $39;
  $or$cond = $29 | $40;
  $41 = $or$cond ? 65533 : $17;
  $42 = $or$cond ? 0 : $18;
  $43 = $x;
  $44 = $43;
  HEAP32[$44>>2] = $41;
  $45 = (($43) + 4)|0;
  $46 = $45;
  HEAP32[$46>>2] = $42;
  $47 = $x;
  $48 = $47;
  $49 = HEAP32[$48>>2]|0;
  $50 = (($47) + 4)|0;
  $51 = $50;
  $52 = HEAP32[$51>>2]|0;
  $53 = $49 & 2097151;
  HEAP32[$c>>2] = $53;
  $54 = HEAP32[$c>>2]|0;
  $55 = ($54>>>0)<(128);
  $56 = HEAP32[$c>>2]|0;
  do {
   if ($55) {
    $57 = $56 & 255;
    $58 = $57&255;
    $59 = HEAP32[$zOut>>2]|0;
    $60 = (($59) + 1|0);
    HEAP32[$zOut>>2] = $60;
    HEAP8[$59>>0] = $58;
   } else {
    $61 = ($56>>>0)<(2048);
    $62 = HEAP32[$c>>2]|0;
    if ($61) {
     $63 = $62 >>> 6;
     $64 = $63 & 31;
     $65 = $64&255;
     $66 = $65&255;
     $67 = (192 + ($66))|0;
     $68 = $67&255;
     $69 = HEAP32[$zOut>>2]|0;
     $70 = (($69) + 1|0);
     HEAP32[$zOut>>2] = $70;
     HEAP8[$69>>0] = $68;
     $71 = HEAP32[$c>>2]|0;
     $72 = $71 & 63;
     $73 = $72&255;
     $74 = $73&255;
     $75 = (128 + ($74))|0;
     $76 = $75&255;
     $77 = HEAP32[$zOut>>2]|0;
     $78 = (($77) + 1|0);
     HEAP32[$zOut>>2] = $78;
     HEAP8[$77>>0] = $76;
     break;
    }
    $79 = ($62>>>0)<(65536);
    $80 = HEAP32[$c>>2]|0;
    if ($79) {
     $81 = $80 >>> 12;
     $82 = $81 & 15;
     $83 = $82&255;
     $84 = $83&255;
     $85 = (224 + ($84))|0;
     $86 = $85&255;
     $87 = HEAP32[$zOut>>2]|0;
     $88 = (($87) + 1|0);
     HEAP32[$zOut>>2] = $88;
     HEAP8[$87>>0] = $86;
     $89 = HEAP32[$c>>2]|0;
     $90 = $89 >>> 6;
     $91 = $90 & 63;
     $92 = $91&255;
     $93 = $92&255;
     $94 = (128 + ($93))|0;
     $95 = $94&255;
     $96 = HEAP32[$zOut>>2]|0;
     $97 = (($96) + 1|0);
     HEAP32[$zOut>>2] = $97;
     HEAP8[$96>>0] = $95;
     $98 = HEAP32[$c>>2]|0;
     $99 = $98 & 63;
     $100 = $99&255;
     $101 = $100&255;
     $102 = (128 + ($101))|0;
     $103 = $102&255;
     $104 = HEAP32[$zOut>>2]|0;
     $105 = (($104) + 1|0);
     HEAP32[$zOut>>2] = $105;
     HEAP8[$104>>0] = $103;
     break;
    } else {
     $106 = $80 >>> 18;
     $107 = $106 & 7;
     $108 = $107&255;
     $109 = $108&255;
     $110 = (240 + ($109))|0;
     $111 = $110&255;
     $112 = HEAP32[$zOut>>2]|0;
     $113 = (($112) + 1|0);
     HEAP32[$zOut>>2] = $113;
     HEAP8[$112>>0] = $111;
     $114 = HEAP32[$c>>2]|0;
     $115 = $114 >>> 12;
     $116 = $115 & 63;
     $117 = $116&255;
     $118 = $117&255;
     $119 = (128 + ($118))|0;
     $120 = $119&255;
     $121 = HEAP32[$zOut>>2]|0;
     $122 = (($121) + 1|0);
     HEAP32[$zOut>>2] = $122;
     HEAP8[$121>>0] = $120;
     $123 = HEAP32[$c>>2]|0;
     $124 = $123 >>> 6;
     $125 = $124 & 63;
     $126 = $125&255;
     $127 = $126&255;
     $128 = (128 + ($127))|0;
     $129 = $128&255;
     $130 = HEAP32[$zOut>>2]|0;
     $131 = (($130) + 1|0);
     HEAP32[$zOut>>2] = $131;
     HEAP8[$130>>0] = $129;
     $132 = HEAP32[$c>>2]|0;
     $133 = $132 & 63;
     $134 = $133&255;
     $135 = $134&255;
     $136 = (128 + ($135))|0;
     $137 = $136&255;
     $138 = HEAP32[$zOut>>2]|0;
     $139 = (($138) + 1|0);
     HEAP32[$zOut>>2] = $139;
     HEAP8[$138>>0] = $137;
     break;
    }
   }
  } while(0);
  $140 = HEAP32[$i>>2]|0;
  $141 = (($140) + 1)|0;
  HEAP32[$i>>2] = $141;
 }
 $142 = HEAP32[$0>>2]|0;
 $143 = HEAP32[$z>>2]|0;
 $144 = HEAP32[$zOut>>2]|0;
 $145 = HEAP32[$z>>2]|0;
 $146 = $144;
 $147 = $145;
 $148 = (($146) - ($147))|0;
 $149 = ($148|0)<(0);
 $150 = $149 << 31 >> 31;
 _sqlite3_result_text64($142,$143,$148,$150,13,1);
 STACKTOP = sp;return;
}
function _absFunc($context,$argc,$argv) {
 $context = $context|0;
 $argc = $argc|0;
 $argv = $argv|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0.0, $54 = 0.0, $55 = 0, $56 = 0.0, $57 = 0.0, $58 = 0, $59 = 0.0, $6 = 0, $7 = 0, $8 = 0, $9 = 0;
 var $iVal = 0, $rVal = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 24|0;
 $1 = sp + 20|0;
 $2 = sp + 16|0;
 $iVal = sp + 8|0;
 $rVal = sp;
 HEAP32[$0>>2] = $context;
 HEAP32[$1>>2] = $argc;
 HEAP32[$2>>2] = $argv;
 $3 = HEAP32[$2>>2]|0;
 $4 = HEAP32[$3>>2]|0;
 $5 = (_sqlite3_value_type($4)|0);
 if ((($5|0) == 1)) {
  $6 = HEAP32[$2>>2]|0;
  $7 = HEAP32[$6>>2]|0;
  $8 = (_sqlite3_value_int64($7)|0);
  $9 = tempRet0;
  $10 = $iVal;
  $11 = $10;
  HEAP32[$11>>2] = $8;
  $12 = (($10) + 4)|0;
  $13 = $12;
  HEAP32[$13>>2] = $9;
  $14 = $iVal;
  $15 = $14;
  $16 = HEAP32[$15>>2]|0;
  $17 = (($14) + 4)|0;
  $18 = $17;
  $19 = HEAP32[$18>>2]|0;
  $20 = ($19|0)<(0);
  do {
   if ($20) {
    $21 = $iVal;
    $22 = $21;
    $23 = HEAP32[$22>>2]|0;
    $24 = (($21) + 4)|0;
    $25 = $24;
    $26 = HEAP32[$25>>2]|0;
    $27 = ($23|0)==(0);
    $28 = ($26|0)==(-2147483648);
    $29 = $27 & $28;
    if (!($29)) {
     $31 = $iVal;
     $32 = $31;
     $33 = HEAP32[$32>>2]|0;
     $34 = (($31) + 4)|0;
     $35 = $34;
     $36 = HEAP32[$35>>2]|0;
     $37 = (_i64Subtract(0,0,($33|0),($36|0))|0);
     $38 = tempRet0;
     $39 = $iVal;
     $40 = $39;
     HEAP32[$40>>2] = $37;
     $41 = (($39) + 4)|0;
     $42 = $41;
     HEAP32[$42>>2] = $38;
     break;
    }
    $30 = HEAP32[$0>>2]|0;
    _sqlite3_result_error($30,9784,-1);
    STACKTOP = sp;return;
   }
  } while(0);
  $43 = HEAP32[$0>>2]|0;
  $44 = $iVal;
  $45 = $44;
  $46 = HEAP32[$45>>2]|0;
  $47 = (($44) + 4)|0;
  $48 = $47;
  $49 = HEAP32[$48>>2]|0;
  _sqlite3_result_int64($43,$46,$49);
  STACKTOP = sp;return;
 } else if ((($5|0) == 5)) {
  $50 = HEAP32[$0>>2]|0;
  _sqlite3_result_null($50);
  STACKTOP = sp;return;
 } else {
  $51 = HEAP32[$2>>2]|0;
  $52 = HEAP32[$51>>2]|0;
  $53 = (+_sqlite3_value_double($52));
  HEAPF64[$rVal>>3] = $53;
  $54 = +HEAPF64[$rVal>>3];
  $55 = $54 < 0.0;
  if ($55) {
   $56 = +HEAPF64[$rVal>>3];
   $57 = -$56;
   HEAPF64[$rVal>>3] = $57;
  }
  $58 = HEAP32[$0>>2]|0;
  $59 = +HEAPF64[$rVal>>3];
  _sqlite3_result_double($58,$59);
  STACKTOP = sp;return;
 }
}
function _roundFunc($context,$argc,$argv) {
 $context = $context|0;
 $argc = $argc|0;
 $argv = $argv|0;
 var $$ = 0, $$$ = 0, $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0.0;
 var $25 = 0, $26 = 0, $27 = 0.0, $28 = 0, $29 = 0.0, $3 = 0, $30 = 0, $31 = 0.0, $32 = 0.0, $33 = 0, $34 = 0, $35 = 0.0, $36 = 0, $37 = 0, $38 = 0.0, $39 = 0, $4 = 0, $40 = 0.0, $41 = 0.0, $42 = 0;
 var $43 = 0.0, $44 = 0.0, $45 = 0.0, $46 = 0, $47 = 0, $48 = 0.0, $49 = 0.0, $5 = 0, $50 = 0, $51 = 0.0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0;
 var $61 = 0.0, $7 = 0, $8 = 0, $9 = 0, $n = 0, $or$cond = 0, $or$cond3 = 0, $or$cond5 = 0, $r = 0, $vararg_buffer = 0, $vararg_ptr6 = 0, $zBuf = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $vararg_buffer = sp + 8|0;
 $0 = sp + 36|0;
 $1 = sp + 32|0;
 $2 = sp + 28|0;
 $n = sp + 24|0;
 $r = sp;
 $zBuf = sp + 20|0;
 HEAP32[$0>>2] = $context;
 HEAP32[$1>>2] = $argc;
 HEAP32[$2>>2] = $argv;
 HEAP32[$n>>2] = 0;
 $3 = HEAP32[$1>>2]|0;
 $4 = ($3|0)==(2);
 do {
  if ($4) {
   $5 = HEAP32[$2>>2]|0;
   $6 = (($5) + 4|0);
   $7 = HEAP32[$6>>2]|0;
   $8 = (_sqlite3_value_type($7)|0);
   $9 = (5)==($8|0);
   if ($9) {
    STACKTOP = sp;return;
   } else {
    $10 = HEAP32[$2>>2]|0;
    $11 = (($10) + 4|0);
    $12 = HEAP32[$11>>2]|0;
    $13 = (_sqlite3_value_int($12)|0);
    HEAP32[$n>>2] = $13;
    $14 = HEAP32[$n>>2]|0;
    $15 = ($14|0)>(30);
    $$ = $15 ? 30 : $13;
    HEAP32[$n>>2] = $$;
    $16 = HEAP32[$n>>2]|0;
    $17 = ($16|0)<(0);
    $$$ = $17 ? 0 : $$;
    HEAP32[$n>>2] = $$$;
    break;
   }
  }
 } while(0);
 $18 = HEAP32[$2>>2]|0;
 $19 = HEAP32[$18>>2]|0;
 $20 = (_sqlite3_value_type($19)|0);
 $21 = ($20|0)==(5);
 if ($21) {
  STACKTOP = sp;return;
 }
 $22 = HEAP32[$2>>2]|0;
 $23 = HEAP32[$22>>2]|0;
 $24 = (+_sqlite3_value_double($23));
 HEAPF64[$r>>3] = $24;
 $25 = HEAP32[$n>>2]|0;
 $26 = ($25|0)==(0);
 $27 = +HEAPF64[$r>>3];
 $28 = $27 >= 0.0;
 $or$cond = $26 & $28;
 $29 = +HEAPF64[$r>>3];
 $30 = $29 < 9.2233720368547758E+18;
 $or$cond3 = $or$cond & $30;
 do {
  if ($or$cond3) {
   $31 = +HEAPF64[$r>>3];
   $32 = $31 + 0.5;
   $33 = (~~$32)>>>0;
   $34 = +Math_abs($32) >= 1.0 ? $32 > 0.0 ? (Math_min(+Math_floor($32 / 4294967296.0), 4294967295.0) | 0) >>> 0 : ~~+Math_ceil(($32 - +(~~$32 >>> 0)) / 4294967296.0) >>> 0 : 0;
   $35 = (+($33>>>0)) + (4294967296.0*(+($34|0)));
   HEAPF64[$r>>3] = $35;
  } else {
   $36 = HEAP32[$n>>2]|0;
   $37 = ($36|0)==(0);
   $38 = +HEAPF64[$r>>3];
   $39 = $38 < 0.0;
   $or$cond5 = $37 & $39;
   if ($or$cond5) {
    $40 = +HEAPF64[$r>>3];
    $41 = -$40;
    $42 = $41 < 9.2233720368547758E+18;
    if ($42) {
     $43 = +HEAPF64[$r>>3];
     $44 = -$43;
     $45 = $44 + 0.5;
     $46 = (~~$45)>>>0;
     $47 = +Math_abs($45) >= 1.0 ? $45 > 0.0 ? (Math_min(+Math_floor($45 / 4294967296.0), 4294967295.0) | 0) >>> 0 : ~~+Math_ceil(($45 - +(~~$45 >>> 0)) / 4294967296.0) >>> 0 : 0;
     $48 = (+($46>>>0)) + (4294967296.0*(+($47|0)));
     $49 = -$48;
     HEAPF64[$r>>3] = $49;
     break;
    }
   }
   $50 = HEAP32[$n>>2]|0;
   $51 = +HEAPF64[$r>>3];
   HEAP32[$vararg_buffer>>2] = $50;
   $vararg_ptr6 = (($vararg_buffer) + 4|0);
   HEAPF64[tempDoublePtr>>3]=$51;HEAP32[$vararg_ptr6>>2]=HEAP32[tempDoublePtr>>2];HEAP32[$vararg_ptr6+4>>2]=HEAP32[tempDoublePtr+4>>2];
   $52 = (_sqlite3_mprintf(9848,$vararg_buffer)|0);
   HEAP32[$zBuf>>2] = $52;
   $53 = HEAP32[$zBuf>>2]|0;
   $54 = ($53|0)==(0|0);
   if (!($54)) {
    $56 = HEAP32[$zBuf>>2]|0;
    $57 = HEAP32[$zBuf>>2]|0;
    $58 = (_sqlite3Strlen30($57)|0);
    (_sqlite3AtoF($56,$r,$58,1)|0);
    $59 = HEAP32[$zBuf>>2]|0;
    _sqlite3_free($59);
    break;
   }
   $55 = HEAP32[$0>>2]|0;
   _sqlite3_result_error_nomem($55);
   STACKTOP = sp;return;
  }
 } while(0);
 $60 = HEAP32[$0>>2]|0;
 $61 = +HEAPF64[$r>>3];
 _sqlite3_result_double($60,$61);
 STACKTOP = sp;return;
}
function _upperFunc($context,$argc,$argv) {
 $context = $context|0;
 $argc = $argc|0;
 $argv = $argv|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $i = 0, $n = 0, $z1 = 0, $z2 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 24|0;
 $1 = sp + 20|0;
 $2 = sp + 16|0;
 $z1 = sp + 12|0;
 $z2 = sp + 8|0;
 $i = sp + 4|0;
 $n = sp;
 HEAP32[$0>>2] = $context;
 HEAP32[$1>>2] = $argc;
 HEAP32[$2>>2] = $argv;
 $3 = HEAP32[$2>>2]|0;
 $4 = HEAP32[$3>>2]|0;
 $5 = (_sqlite3_value_text($4)|0);
 HEAP32[$z2>>2] = $5;
 $6 = HEAP32[$2>>2]|0;
 $7 = HEAP32[$6>>2]|0;
 $8 = (_sqlite3_value_bytes($7)|0);
 HEAP32[$n>>2] = $8;
 $9 = HEAP32[$z2>>2]|0;
 $10 = ($9|0)!=(0|0);
 if (!($10)) {
  STACKTOP = sp;return;
 }
 $11 = HEAP32[$0>>2]|0;
 $12 = HEAP32[$n>>2]|0;
 $13 = ($12|0)<(0);
 $14 = $13 << 31 >> 31;
 $15 = (_i64Add(($12|0),($14|0),1,0)|0);
 $16 = tempRet0;
 $17 = (_contextMalloc($11,$15,$16)|0);
 HEAP32[$z1>>2] = $17;
 $18 = HEAP32[$z1>>2]|0;
 $19 = ($18|0)!=(0|0);
 if (!($19)) {
  STACKTOP = sp;return;
 }
 HEAP32[$i>>2] = 0;
 while(1) {
  $20 = HEAP32[$i>>2]|0;
  $21 = HEAP32[$n>>2]|0;
  $22 = ($20|0)<($21|0);
  if (!($22)) {
   break;
  }
  $23 = HEAP32[$i>>2]|0;
  $24 = HEAP32[$z2>>2]|0;
  $25 = (($24) + ($23)|0);
  $26 = HEAP8[$25>>0]|0;
  $27 = $26 << 24 >> 24;
  $28 = HEAP32[$i>>2]|0;
  $29 = HEAP32[$z2>>2]|0;
  $30 = (($29) + ($28)|0);
  $31 = HEAP8[$30>>0]|0;
  $32 = $31&255;
  $33 = (1224 + ($32)|0);
  $34 = HEAP8[$33>>0]|0;
  $35 = $34&255;
  $36 = $35 & 32;
  $37 = $36 ^ -1;
  $38 = $27 & $37;
  $39 = $38&255;
  $40 = HEAP32[$i>>2]|0;
  $41 = HEAP32[$z1>>2]|0;
  $42 = (($41) + ($40)|0);
  HEAP8[$42>>0] = $39;
  $43 = HEAP32[$i>>2]|0;
  $44 = (($43) + 1)|0;
  HEAP32[$i>>2] = $44;
 }
 $45 = HEAP32[$0>>2]|0;
 $46 = HEAP32[$z1>>2]|0;
 $47 = HEAP32[$n>>2]|0;
 _sqlite3_result_text($45,$46,$47,13);
 STACKTOP = sp;return;
}
function _lowerFunc($context,$argc,$argv) {
 $context = $context|0;
 $argc = $argc|0;
 $argv = $argv|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $i = 0, $n = 0;
 var $z1 = 0, $z2 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 24|0;
 $1 = sp + 20|0;
 $2 = sp + 16|0;
 $z1 = sp + 12|0;
 $z2 = sp + 8|0;
 $i = sp + 4|0;
 $n = sp;
 HEAP32[$0>>2] = $context;
 HEAP32[$1>>2] = $argc;
 HEAP32[$2>>2] = $argv;
 $3 = HEAP32[$2>>2]|0;
 $4 = HEAP32[$3>>2]|0;
 $5 = (_sqlite3_value_text($4)|0);
 HEAP32[$z2>>2] = $5;
 $6 = HEAP32[$2>>2]|0;
 $7 = HEAP32[$6>>2]|0;
 $8 = (_sqlite3_value_bytes($7)|0);
 HEAP32[$n>>2] = $8;
 $9 = HEAP32[$z2>>2]|0;
 $10 = ($9|0)!=(0|0);
 if (!($10)) {
  STACKTOP = sp;return;
 }
 $11 = HEAP32[$0>>2]|0;
 $12 = HEAP32[$n>>2]|0;
 $13 = ($12|0)<(0);
 $14 = $13 << 31 >> 31;
 $15 = (_i64Add(($12|0),($14|0),1,0)|0);
 $16 = tempRet0;
 $17 = (_contextMalloc($11,$15,$16)|0);
 HEAP32[$z1>>2] = $17;
 $18 = HEAP32[$z1>>2]|0;
 $19 = ($18|0)!=(0|0);
 if (!($19)) {
  STACKTOP = sp;return;
 }
 HEAP32[$i>>2] = 0;
 while(1) {
  $20 = HEAP32[$i>>2]|0;
  $21 = HEAP32[$n>>2]|0;
  $22 = ($20|0)<($21|0);
  if (!($22)) {
   break;
  }
  $23 = HEAP32[$i>>2]|0;
  $24 = HEAP32[$z2>>2]|0;
  $25 = (($24) + ($23)|0);
  $26 = HEAP8[$25>>0]|0;
  $27 = $26&255;
  $28 = (440 + ($27)|0);
  $29 = HEAP8[$28>>0]|0;
  $30 = HEAP32[$i>>2]|0;
  $31 = HEAP32[$z1>>2]|0;
  $32 = (($31) + ($30)|0);
  HEAP8[$32>>0] = $29;
  $33 = HEAP32[$i>>2]|0;
  $34 = (($33) + 1)|0;
  HEAP32[$i>>2] = $34;
 }
 $35 = HEAP32[$0>>2]|0;
 $36 = HEAP32[$z1>>2]|0;
 $37 = HEAP32[$n>>2]|0;
 _sqlite3_result_text($35,$36,$37,13);
 STACKTOP = sp;return;
}
function _versionFunc($context,$NotUsed,$NotUsed2) {
 $context = $context|0;
 $NotUsed = $NotUsed|0;
 $NotUsed2 = $NotUsed2|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $1 = sp + 4|0;
 $2 = sp;
 HEAP32[$0>>2] = $context;
 HEAP32[$1>>2] = $NotUsed;
 HEAP32[$2>>2] = $NotUsed2;
 $3 = HEAP32[$0>>2]|0;
 $4 = (_sqlite3_libversion()|0);
 _sqlite3_result_text($3,$4,-1,0);
 STACKTOP = sp;return;
}
function _hexFunc($context,$argc,$argv) {
 $context = $context|0;
 $argc = $argc|0;
 $argv = $argv|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $c = 0, $i = 0, $n = 0, $pBlob = 0, $z = 0, $zHex = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $0 = sp + 28|0;
 $1 = sp + 24|0;
 $2 = sp + 20|0;
 $i = sp + 16|0;
 $n = sp + 12|0;
 $pBlob = sp + 8|0;
 $zHex = sp + 4|0;
 $z = sp;
 $c = sp + 32|0;
 HEAP32[$0>>2] = $context;
 HEAP32[$1>>2] = $argc;
 HEAP32[$2>>2] = $argv;
 $3 = HEAP32[$2>>2]|0;
 $4 = HEAP32[$3>>2]|0;
 $5 = (_sqlite3_value_blob($4)|0);
 HEAP32[$pBlob>>2] = $5;
 $6 = HEAP32[$2>>2]|0;
 $7 = HEAP32[$6>>2]|0;
 $8 = (_sqlite3_value_bytes($7)|0);
 HEAP32[$n>>2] = $8;
 $9 = HEAP32[$0>>2]|0;
 $10 = HEAP32[$n>>2]|0;
 $11 = ($10|0)<(0);
 $12 = $11 << 31 >> 31;
 $13 = (___muldi3(($10|0),($12|0),2,0)|0);
 $14 = tempRet0;
 $15 = (_i64Add(($13|0),($14|0),1,0)|0);
 $16 = tempRet0;
 $17 = (_contextMalloc($9,$15,$16)|0);
 HEAP32[$zHex>>2] = $17;
 HEAP32[$z>>2] = $17;
 $18 = HEAP32[$zHex>>2]|0;
 $19 = ($18|0)!=(0|0);
 if (!($19)) {
  STACKTOP = sp;return;
 }
 HEAP32[$i>>2] = 0;
 while(1) {
  $20 = HEAP32[$i>>2]|0;
  $21 = HEAP32[$n>>2]|0;
  $22 = ($20|0)<($21|0);
  if (!($22)) {
   break;
  }
  $23 = HEAP32[$pBlob>>2]|0;
  $24 = HEAP8[$23>>0]|0;
  HEAP8[$c>>0] = $24;
  $25 = HEAP8[$c>>0]|0;
  $26 = $25&255;
  $27 = $26 >> 4;
  $28 = $27 & 15;
  $29 = (9824 + ($28)|0);
  $30 = HEAP8[$29>>0]|0;
  $31 = HEAP32[$z>>2]|0;
  $32 = (($31) + 1|0);
  HEAP32[$z>>2] = $32;
  HEAP8[$31>>0] = $30;
  $33 = HEAP8[$c>>0]|0;
  $34 = $33&255;
  $35 = $34 & 15;
  $36 = (9824 + ($35)|0);
  $37 = HEAP8[$36>>0]|0;
  $38 = HEAP32[$z>>2]|0;
  $39 = (($38) + 1|0);
  HEAP32[$z>>2] = $39;
  HEAP8[$38>>0] = $37;
  $40 = HEAP32[$i>>2]|0;
  $41 = (($40) + 1)|0;
  HEAP32[$i>>2] = $41;
  $42 = HEAP32[$pBlob>>2]|0;
  $43 = (($42) + 1|0);
  HEAP32[$pBlob>>2] = $43;
 }
 $44 = HEAP32[$z>>2]|0;
 HEAP8[$44>>0] = 0;
 $45 = HEAP32[$0>>2]|0;
 $46 = HEAP32[$zHex>>2]|0;
 $47 = HEAP32[$n>>2]|0;
 $48 = $47<<1;
 _sqlite3_result_text($45,$46,$48,13);
 STACKTOP = sp;return;
}
function _randomFunc($context,$NotUsed,$NotUsed2) {
 $context = $context|0;
 $NotUsed = $NotUsed|0;
 $NotUsed2 = $NotUsed2|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $r = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 16|0;
 $1 = sp + 12|0;
 $2 = sp + 8|0;
 $r = sp;
 HEAP32[$0>>2] = $context;
 HEAP32[$1>>2] = $NotUsed;
 HEAP32[$2>>2] = $NotUsed2;
 _sqlite3_randomness(8,$r);
 $3 = $r;
 $4 = $3;
 $5 = HEAP32[$4>>2]|0;
 $6 = (($3) + 4)|0;
 $7 = $6;
 $8 = HEAP32[$7>>2]|0;
 $9 = ($8|0)<(0);
 if ($9) {
  $10 = $r;
  $11 = $10;
  $12 = HEAP32[$11>>2]|0;
  $13 = (($10) + 4)|0;
  $14 = $13;
  $15 = HEAP32[$14>>2]|0;
  $16 = $15 & 2147483647;
  $17 = (_i64Subtract(0,0,($12|0),($16|0))|0);
  $18 = tempRet0;
  $19 = $r;
  $20 = $19;
  HEAP32[$20>>2] = $17;
  $21 = (($19) + 4)|0;
  $22 = $21;
  HEAP32[$22>>2] = $18;
 }
 $23 = HEAP32[$0>>2]|0;
 $24 = $r;
 $25 = $24;
 $26 = HEAP32[$25>>2]|0;
 $27 = (($24) + 4)|0;
 $28 = $27;
 $29 = HEAP32[$28>>2]|0;
 _sqlite3_result_int64($23,$26,$29);
 STACKTOP = sp;return;
}
function _randomBlob($context,$argc,$argv) {
 $context = $context|0;
 $argc = $argc|0;
 $argv = $argv|0;
 var $$ = 0, $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0;
 var $9 = 0, $n = 0, $p = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 16|0;
 $1 = sp + 12|0;
 $2 = sp + 8|0;
 $n = sp + 4|0;
 $p = sp;
 HEAP32[$0>>2] = $context;
 HEAP32[$1>>2] = $argc;
 HEAP32[$2>>2] = $argv;
 $3 = HEAP32[$2>>2]|0;
 $4 = HEAP32[$3>>2]|0;
 $5 = (_sqlite3_value_int($4)|0);
 HEAP32[$n>>2] = $5;
 $6 = HEAP32[$n>>2]|0;
 $7 = ($6|0)<(1);
 $$ = $7 ? 1 : $5;
 HEAP32[$n>>2] = $$;
 $8 = HEAP32[$0>>2]|0;
 $9 = HEAP32[$n>>2]|0;
 $10 = ($9|0)<(0);
 $11 = $10 << 31 >> 31;
 $12 = (_contextMalloc($8,$9,$11)|0);
 HEAP32[$p>>2] = $12;
 $13 = HEAP32[$p>>2]|0;
 $14 = ($13|0)!=(0|0);
 if (!($14)) {
  STACKTOP = sp;return;
 }
 $15 = HEAP32[$n>>2]|0;
 $16 = HEAP32[$p>>2]|0;
 _sqlite3_randomness($15,$16);
 $17 = HEAP32[$0>>2]|0;
 $18 = HEAP32[$p>>2]|0;
 $19 = HEAP32[$n>>2]|0;
 _sqlite3_result_blob($17,$18,$19,13);
 STACKTOP = sp;return;
}
function _nullifFunc($context,$NotUsed,$argv) {
 $context = $context|0;
 $NotUsed = $NotUsed|0;
 $argv = $argv|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $pColl = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $1 = sp + 8|0;
 $2 = sp + 4|0;
 $pColl = sp;
 HEAP32[$0>>2] = $context;
 HEAP32[$1>>2] = $NotUsed;
 HEAP32[$2>>2] = $argv;
 $3 = HEAP32[$0>>2]|0;
 $4 = (_sqlite3GetFuncCollSeq($3)|0);
 HEAP32[$pColl>>2] = $4;
 $5 = HEAP32[$2>>2]|0;
 $6 = HEAP32[$5>>2]|0;
 $7 = HEAP32[$2>>2]|0;
 $8 = (($7) + 4|0);
 $9 = HEAP32[$8>>2]|0;
 $10 = HEAP32[$pColl>>2]|0;
 $11 = (_sqlite3MemCompare($6,$9,$10)|0);
 $12 = ($11|0)!=(0);
 if (!($12)) {
  STACKTOP = sp;return;
 }
 $13 = HEAP32[$0>>2]|0;
 $14 = HEAP32[$2>>2]|0;
 $15 = HEAP32[$14>>2]|0;
 _sqlite3_result_value($13,$15);
 STACKTOP = sp;return;
}
function _sourceidFunc($context,$NotUsed,$NotUsed2) {
 $context = $context|0;
 $NotUsed = $NotUsed|0;
 $NotUsed2 = $NotUsed2|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $1 = sp + 4|0;
 $2 = sp;
 HEAP32[$0>>2] = $context;
 HEAP32[$1>>2] = $NotUsed;
 HEAP32[$2>>2] = $NotUsed2;
 $3 = HEAP32[$0>>2]|0;
 $4 = (_sqlite3_sourceid()|0);
 _sqlite3_result_text($3,$4,-1,0);
 STACKTOP = sp;return;
}
function _errlogFunc($context,$argc,$argv) {
 $context = $context|0;
 $argc = $argc|0;
 $argv = $argv|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $vararg_buffer = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $vararg_buffer = sp;
 $0 = sp + 12|0;
 $1 = sp + 8|0;
 $2 = sp + 4|0;
 HEAP32[$0>>2] = $context;
 HEAP32[$1>>2] = $argc;
 HEAP32[$2>>2] = $argv;
 $3 = HEAP32[$2>>2]|0;
 $4 = HEAP32[$3>>2]|0;
 $5 = (_sqlite3_value_int($4)|0);
 $6 = HEAP32[$2>>2]|0;
 $7 = (($6) + 4|0);
 $8 = HEAP32[$7>>2]|0;
 $9 = (_sqlite3_value_text($8)|0);
 HEAP32[$vararg_buffer>>2] = $9;
 _sqlite3_log($5,1200,$vararg_buffer);
 STACKTOP = sp;return;
}
function _compileoptionusedFunc($context,$argc,$argv) {
 $context = $context|0;
 $argc = $argc|0;
 $argv = $argv|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $zOptName = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $1 = sp + 8|0;
 $2 = sp + 4|0;
 $zOptName = sp;
 HEAP32[$0>>2] = $context;
 HEAP32[$1>>2] = $argc;
 HEAP32[$2>>2] = $argv;
 $3 = HEAP32[$2>>2]|0;
 $4 = HEAP32[$3>>2]|0;
 $5 = (_sqlite3_value_text($4)|0);
 HEAP32[$zOptName>>2] = $5;
 $6 = ($5|0)!=(0|0);
 if (!($6)) {
  STACKTOP = sp;return;
 }
 $7 = HEAP32[$0>>2]|0;
 $8 = HEAP32[$zOptName>>2]|0;
 $9 = (_sqlite3_compileoption_used($8)|0);
 _sqlite3_result_int($7,$9);
 STACKTOP = sp;return;
}
function _compileoptiongetFunc($context,$argc,$argv) {
 $context = $context|0;
 $argc = $argc|0;
 $argv = $argv|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $n = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $1 = sp + 8|0;
 $2 = sp + 4|0;
 $n = sp;
 HEAP32[$0>>2] = $context;
 HEAP32[$1>>2] = $argc;
 HEAP32[$2>>2] = $argv;
 $3 = HEAP32[$2>>2]|0;
 $4 = HEAP32[$3>>2]|0;
 $5 = (_sqlite3_value_int($4)|0);
 HEAP32[$n>>2] = $5;
 $6 = HEAP32[$0>>2]|0;
 $7 = HEAP32[$n>>2]|0;
 $8 = (_sqlite3_compileoption_get($7)|0);
 _sqlite3_result_text($6,$8,-1,0);
 STACKTOP = sp;return;
}
function _quoteFunc($context,$argc,$argv) {
 $context = $context|0;
 $argc = $argc|0;
 $argv = $argv|0;
 var $0 = 0, $1 = 0, $10 = 0.0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0.0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0.0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0;
 var $170 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0;
 var $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0;
 var $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0;
 var $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0.0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0;
 var $9 = 0.0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $i = 0, $i1 = 0, $j = 0, $n = 0, $nBlob = 0, $r1 = 0, $r2 = 0, $vararg_buffer1 = 0, $z = 0;
 var $zArg = 0, $zBlob = 0, $zBuf = 0, $zText = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 128|0;
 $vararg_buffer1 = sp + 24|0;
 $0 = sp + 72|0;
 $1 = sp + 68|0;
 $2 = sp + 64|0;
 $r1 = sp + 16|0;
 $r2 = sp + 8|0;
 $zBuf = sp + 76|0;
 $zText = sp + 60|0;
 $zBlob = sp + 56|0;
 $nBlob = sp + 52|0;
 $i = sp + 48|0;
 $i1 = sp + 44|0;
 $j = sp + 40|0;
 $n = sp;
 $zArg = sp + 36|0;
 $z = sp + 32|0;
 HEAP32[$0>>2] = $context;
 HEAP32[$1>>2] = $argc;
 HEAP32[$2>>2] = $argv;
 $3 = HEAP32[$2>>2]|0;
 $4 = HEAP32[$3>>2]|0;
 $5 = (_sqlite3_value_type($4)|0);
 if ((($5|0) == 2)) {
  $6 = HEAP32[$2>>2]|0;
  $7 = HEAP32[$6>>2]|0;
  $8 = (+_sqlite3_value_double($7));
  HEAPF64[$r1>>3] = $8;
  $9 = +HEAPF64[$r1>>3];
  HEAPF64[tempDoublePtr>>3]=$9;HEAP32[$vararg_buffer1>>2]=HEAP32[tempDoublePtr>>2];HEAP32[$vararg_buffer1+4>>2]=HEAP32[tempDoublePtr+4>>2];
  (_sqlite3_snprintf(50,$zBuf,9808,$vararg_buffer1)|0);
  (_sqlite3AtoF($zBuf,$r2,20,1)|0);
  $10 = +HEAPF64[$r1>>3];
  $11 = +HEAPF64[$r2>>3];
  $12 = $10 != $11;
  if ($12) {
   $13 = +HEAPF64[$r1>>3];
   HEAPF64[tempDoublePtr>>3]=$13;HEAP32[$vararg_buffer1>>2]=HEAP32[tempDoublePtr>>2];HEAP32[$vararg_buffer1+4>>2]=HEAP32[tempDoublePtr+4>>2];
   (_sqlite3_snprintf(50,$zBuf,9816,$vararg_buffer1)|0);
  }
  $14 = HEAP32[$0>>2]|0;
  _sqlite3_result_text($14,$zBuf,-1,(-1));
  STACKTOP = sp;return;
 } else if ((($5|0) == 1)) {
  $15 = HEAP32[$0>>2]|0;
  $16 = HEAP32[$2>>2]|0;
  $17 = HEAP32[$16>>2]|0;
  _sqlite3_result_value($15,$17);
  STACKTOP = sp;return;
 } else if ((($5|0) == 4)) {
  HEAP32[$zText>>2] = 0;
  $18 = HEAP32[$2>>2]|0;
  $19 = HEAP32[$18>>2]|0;
  $20 = (_sqlite3_value_blob($19)|0);
  HEAP32[$zBlob>>2] = $20;
  $21 = HEAP32[$2>>2]|0;
  $22 = HEAP32[$21>>2]|0;
  $23 = (_sqlite3_value_bytes($22)|0);
  HEAP32[$nBlob>>2] = $23;
  $24 = HEAP32[$0>>2]|0;
  $25 = HEAP32[$nBlob>>2]|0;
  $26 = ($25|0)<(0);
  $27 = $26 << 31 >> 31;
  $28 = (___muldi3(2,0,($25|0),($27|0))|0);
  $29 = tempRet0;
  $30 = (_i64Add(($28|0),($29|0),4,0)|0);
  $31 = tempRet0;
  $32 = (_contextMalloc($24,$30,$31)|0);
  HEAP32[$zText>>2] = $32;
  $33 = HEAP32[$zText>>2]|0;
  $34 = ($33|0)!=(0|0);
  if (!($34)) {
   STACKTOP = sp;return;
  }
  HEAP32[$i>>2] = 0;
  while(1) {
   $35 = HEAP32[$i>>2]|0;
   $36 = HEAP32[$nBlob>>2]|0;
   $37 = ($35|0)<($36|0);
   if (!($37)) {
    break;
   }
   $38 = HEAP32[$i>>2]|0;
   $39 = HEAP32[$zBlob>>2]|0;
   $40 = (($39) + ($38)|0);
   $41 = HEAP8[$40>>0]|0;
   $42 = $41 << 24 >> 24;
   $43 = $42 >> 4;
   $44 = $43 & 15;
   $45 = (9824 + ($44)|0);
   $46 = HEAP8[$45>>0]|0;
   $47 = HEAP32[$i>>2]|0;
   $48 = $47<<1;
   $49 = (($48) + 2)|0;
   $50 = HEAP32[$zText>>2]|0;
   $51 = (($50) + ($49)|0);
   HEAP8[$51>>0] = $46;
   $52 = HEAP32[$i>>2]|0;
   $53 = HEAP32[$zBlob>>2]|0;
   $54 = (($53) + ($52)|0);
   $55 = HEAP8[$54>>0]|0;
   $56 = $55 << 24 >> 24;
   $57 = $56 & 15;
   $58 = (9824 + ($57)|0);
   $59 = HEAP8[$58>>0]|0;
   $60 = HEAP32[$i>>2]|0;
   $61 = $60<<1;
   $62 = (($61) + 3)|0;
   $63 = HEAP32[$zText>>2]|0;
   $64 = (($63) + ($62)|0);
   HEAP8[$64>>0] = $59;
   $65 = HEAP32[$i>>2]|0;
   $66 = (($65) + 1)|0;
   HEAP32[$i>>2] = $66;
  }
  $67 = HEAP32[$nBlob>>2]|0;
  $68 = $67<<1;
  $69 = (($68) + 2)|0;
  $70 = HEAP32[$zText>>2]|0;
  $71 = (($70) + ($69)|0);
  HEAP8[$71>>0] = 39;
  $72 = HEAP32[$nBlob>>2]|0;
  $73 = $72<<1;
  $74 = (($73) + 3)|0;
  $75 = HEAP32[$zText>>2]|0;
  $76 = (($75) + ($74)|0);
  HEAP8[$76>>0] = 0;
  $77 = HEAP32[$zText>>2]|0;
  HEAP8[$77>>0] = 88;
  $78 = HEAP32[$zText>>2]|0;
  $79 = (($78) + 1|0);
  HEAP8[$79>>0] = 39;
  $80 = HEAP32[$0>>2]|0;
  $81 = HEAP32[$zText>>2]|0;
  _sqlite3_result_text($80,$81,-1,(-1));
  $82 = HEAP32[$zText>>2]|0;
  _sqlite3_free($82);
  STACKTOP = sp;return;
 } else if ((($5|0) == 3)) {
  $83 = HEAP32[$2>>2]|0;
  $84 = HEAP32[$83>>2]|0;
  $85 = (_sqlite3_value_text($84)|0);
  HEAP32[$zArg>>2] = $85;
  $86 = HEAP32[$zArg>>2]|0;
  $87 = ($86|0)==(0|0);
  if ($87) {
   STACKTOP = sp;return;
  }
  HEAP32[$i1>>2] = 0;
  $88 = $n;
  $89 = $88;
  HEAP32[$89>>2] = 0;
  $90 = (($88) + 4)|0;
  $91 = $90;
  HEAP32[$91>>2] = 0;
  while(1) {
   $92 = HEAP32[$i1>>2]|0;
   $93 = HEAP32[$zArg>>2]|0;
   $94 = (($93) + ($92)|0);
   $95 = HEAP8[$94>>0]|0;
   $96 = ($95<<24>>24)!=(0);
   if (!($96)) {
    break;
   }
   $97 = HEAP32[$i1>>2]|0;
   $98 = HEAP32[$zArg>>2]|0;
   $99 = (($98) + ($97)|0);
   $100 = HEAP8[$99>>0]|0;
   $101 = $100&255;
   $102 = ($101|0)==(39);
   if ($102) {
    $103 = $n;
    $104 = $103;
    $105 = HEAP32[$104>>2]|0;
    $106 = (($103) + 4)|0;
    $107 = $106;
    $108 = HEAP32[$107>>2]|0;
    $109 = (_i64Add(($105|0),($108|0),1,0)|0);
    $110 = tempRet0;
    $111 = $n;
    $112 = $111;
    HEAP32[$112>>2] = $109;
    $113 = (($111) + 4)|0;
    $114 = $113;
    HEAP32[$114>>2] = $110;
   }
   $115 = HEAP32[$i1>>2]|0;
   $116 = (($115) + 1)|0;
   HEAP32[$i1>>2] = $116;
  }
  $117 = HEAP32[$0>>2]|0;
  $118 = HEAP32[$i1>>2]|0;
  $119 = ($118|0)<(0);
  $120 = $119 << 31 >> 31;
  $121 = $n;
  $122 = $121;
  $123 = HEAP32[$122>>2]|0;
  $124 = (($121) + 4)|0;
  $125 = $124;
  $126 = HEAP32[$125>>2]|0;
  $127 = (_i64Add(($118|0),($120|0),($123|0),($126|0))|0);
  $128 = tempRet0;
  $129 = (_i64Add(($127|0),($128|0),3,0)|0);
  $130 = tempRet0;
  $131 = (_contextMalloc($117,$129,$130)|0);
  HEAP32[$z>>2] = $131;
  $132 = HEAP32[$z>>2]|0;
  $133 = ($132|0)!=(0|0);
  if (!($133)) {
   STACKTOP = sp;return;
  }
  $134 = HEAP32[$z>>2]|0;
  HEAP8[$134>>0] = 39;
  HEAP32[$i1>>2] = 0;
  HEAP32[$j>>2] = 1;
  while(1) {
   $135 = HEAP32[$i1>>2]|0;
   $136 = HEAP32[$zArg>>2]|0;
   $137 = (($136) + ($135)|0);
   $138 = HEAP8[$137>>0]|0;
   $139 = ($138<<24>>24)!=(0);
   if (!($139)) {
    break;
   }
   $140 = HEAP32[$i1>>2]|0;
   $141 = HEAP32[$zArg>>2]|0;
   $142 = (($141) + ($140)|0);
   $143 = HEAP8[$142>>0]|0;
   $144 = HEAP32[$j>>2]|0;
   $145 = (($144) + 1)|0;
   HEAP32[$j>>2] = $145;
   $146 = HEAP32[$z>>2]|0;
   $147 = (($146) + ($144)|0);
   HEAP8[$147>>0] = $143;
   $148 = HEAP32[$i1>>2]|0;
   $149 = HEAP32[$zArg>>2]|0;
   $150 = (($149) + ($148)|0);
   $151 = HEAP8[$150>>0]|0;
   $152 = $151&255;
   $153 = ($152|0)==(39);
   if ($153) {
    $154 = HEAP32[$j>>2]|0;
    $155 = (($154) + 1)|0;
    HEAP32[$j>>2] = $155;
    $156 = HEAP32[$z>>2]|0;
    $157 = (($156) + ($154)|0);
    HEAP8[$157>>0] = 39;
   }
   $158 = HEAP32[$i1>>2]|0;
   $159 = (($158) + 1)|0;
   HEAP32[$i1>>2] = $159;
  }
  $160 = HEAP32[$j>>2]|0;
  $161 = (($160) + 1)|0;
  HEAP32[$j>>2] = $161;
  $162 = HEAP32[$z>>2]|0;
  $163 = (($162) + ($160)|0);
  HEAP8[$163>>0] = 39;
  $164 = HEAP32[$j>>2]|0;
  $165 = HEAP32[$z>>2]|0;
  $166 = (($165) + ($164)|0);
  HEAP8[$166>>0] = 0;
  $167 = HEAP32[$0>>2]|0;
  $168 = HEAP32[$z>>2]|0;
  $169 = HEAP32[$j>>2]|0;
  _sqlite3_result_text($167,$168,$169,13);
  STACKTOP = sp;return;
 } else {
  $170 = HEAP32[$0>>2]|0;
  _sqlite3_result_text($170,9840,4,0);
  STACKTOP = sp;return;
 }
}
function _last_insert_rowid($context,$NotUsed,$NotUsed2) {
 $context = $context|0;
 $NotUsed = $NotUsed|0;
 $NotUsed2 = $NotUsed2|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $db = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $1 = sp + 8|0;
 $2 = sp + 4|0;
 $db = sp;
 HEAP32[$0>>2] = $context;
 HEAP32[$1>>2] = $NotUsed;
 HEAP32[$2>>2] = $NotUsed2;
 $3 = HEAP32[$0>>2]|0;
 $4 = (_sqlite3_context_db_handle($3)|0);
 HEAP32[$db>>2] = $4;
 $5 = HEAP32[$0>>2]|0;
 $6 = HEAP32[$db>>2]|0;
 $7 = (_sqlite3_last_insert_rowid($6)|0);
 $8 = tempRet0;
 _sqlite3_result_int64($5,$7,$8);
 STACKTOP = sp;return;
}
function _changes($context,$NotUsed,$NotUsed2) {
 $context = $context|0;
 $NotUsed = $NotUsed|0;
 $NotUsed2 = $NotUsed2|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $db = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $1 = sp + 8|0;
 $2 = sp + 4|0;
 $db = sp;
 HEAP32[$0>>2] = $context;
 HEAP32[$1>>2] = $NotUsed;
 HEAP32[$2>>2] = $NotUsed2;
 $3 = HEAP32[$0>>2]|0;
 $4 = (_sqlite3_context_db_handle($3)|0);
 HEAP32[$db>>2] = $4;
 $5 = HEAP32[$0>>2]|0;
 $6 = HEAP32[$db>>2]|0;
 $7 = (_sqlite3_changes($6)|0);
 _sqlite3_result_int($5,$7);
 STACKTOP = sp;return;
}
function _total_changes($context,$NotUsed,$NotUsed2) {
 $context = $context|0;
 $NotUsed = $NotUsed|0;
 $NotUsed2 = $NotUsed2|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $db = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $1 = sp + 8|0;
 $2 = sp + 4|0;
 $db = sp;
 HEAP32[$0>>2] = $context;
 HEAP32[$1>>2] = $NotUsed;
 HEAP32[$2>>2] = $NotUsed2;
 $3 = HEAP32[$0>>2]|0;
 $4 = (_sqlite3_context_db_handle($3)|0);
 HEAP32[$db>>2] = $4;
 $5 = HEAP32[$0>>2]|0;
 $6 = HEAP32[$db>>2]|0;
 $7 = (_sqlite3_total_changes($6)|0);
 _sqlite3_result_int($5,$7);
 STACKTOP = sp;return;
}
function _replaceFunc($context,$argc,$argv) {
 $context = $context|0;
 $argc = $argc|0;
 $argv = $argv|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0;
 var $170 = 0, $171 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0;
 var $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0;
 var $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0;
 var $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0;
 var $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $db = 0, $i = 0, $j = 0, $loopLimit = 0, $nOut = 0, $nPattern = 0, $nRep = 0, $nStr = 0;
 var $zOld = 0, $zOut = 0, $zPattern = 0, $zRep = 0, $zStr = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 80|0;
 $0 = sp + 64|0;
 $1 = sp + 60|0;
 $2 = sp + 56|0;
 $zStr = sp + 52|0;
 $zPattern = sp + 48|0;
 $zRep = sp + 44|0;
 $zOut = sp + 40|0;
 $nStr = sp + 36|0;
 $nPattern = sp + 32|0;
 $nRep = sp + 28|0;
 $nOut = sp;
 $loopLimit = sp + 24|0;
 $i = sp + 20|0;
 $j = sp + 16|0;
 $zOld = sp + 12|0;
 $db = sp + 8|0;
 HEAP32[$0>>2] = $context;
 HEAP32[$1>>2] = $argc;
 HEAP32[$2>>2] = $argv;
 $3 = HEAP32[$2>>2]|0;
 $4 = HEAP32[$3>>2]|0;
 $5 = (_sqlite3_value_text($4)|0);
 HEAP32[$zStr>>2] = $5;
 $6 = HEAP32[$zStr>>2]|0;
 $7 = ($6|0)==(0|0);
 if ($7) {
  STACKTOP = sp;return;
 }
 $8 = HEAP32[$2>>2]|0;
 $9 = HEAP32[$8>>2]|0;
 $10 = (_sqlite3_value_bytes($9)|0);
 HEAP32[$nStr>>2] = $10;
 $11 = HEAP32[$2>>2]|0;
 $12 = (($11) + 4|0);
 $13 = HEAP32[$12>>2]|0;
 $14 = (_sqlite3_value_text($13)|0);
 HEAP32[$zPattern>>2] = $14;
 $15 = HEAP32[$zPattern>>2]|0;
 $16 = ($15|0)==(0|0);
 if ($16) {
  STACKTOP = sp;return;
 }
 $17 = HEAP32[$zPattern>>2]|0;
 $18 = HEAP8[$17>>0]|0;
 $19 = $18&255;
 $20 = ($19|0)==(0);
 if ($20) {
  $21 = HEAP32[$0>>2]|0;
  $22 = HEAP32[$2>>2]|0;
  $23 = HEAP32[$22>>2]|0;
  _sqlite3_result_value($21,$23);
  STACKTOP = sp;return;
 }
 $24 = HEAP32[$2>>2]|0;
 $25 = (($24) + 4|0);
 $26 = HEAP32[$25>>2]|0;
 $27 = (_sqlite3_value_bytes($26)|0);
 HEAP32[$nPattern>>2] = $27;
 $28 = HEAP32[$2>>2]|0;
 $29 = (($28) + 8|0);
 $30 = HEAP32[$29>>2]|0;
 $31 = (_sqlite3_value_text($30)|0);
 HEAP32[$zRep>>2] = $31;
 $32 = HEAP32[$zRep>>2]|0;
 $33 = ($32|0)==(0|0);
 if ($33) {
  STACKTOP = sp;return;
 }
 $34 = HEAP32[$2>>2]|0;
 $35 = (($34) + 8|0);
 $36 = HEAP32[$35>>2]|0;
 $37 = (_sqlite3_value_bytes($36)|0);
 HEAP32[$nRep>>2] = $37;
 $38 = HEAP32[$nStr>>2]|0;
 $39 = (($38) + 1)|0;
 $40 = ($39|0)<(0);
 $41 = $40 << 31 >> 31;
 $42 = $nOut;
 $43 = $42;
 HEAP32[$43>>2] = $39;
 $44 = (($42) + 4)|0;
 $45 = $44;
 HEAP32[$45>>2] = $41;
 $46 = HEAP32[$0>>2]|0;
 $47 = $nOut;
 $48 = $47;
 $49 = HEAP32[$48>>2]|0;
 $50 = (($47) + 4)|0;
 $51 = $50;
 $52 = HEAP32[$51>>2]|0;
 $53 = (_contextMalloc($46,$49,$52)|0);
 HEAP32[$zOut>>2] = $53;
 $54 = HEAP32[$zOut>>2]|0;
 $55 = ($54|0)==(0|0);
 if ($55) {
  STACKTOP = sp;return;
 }
 $56 = HEAP32[$nStr>>2]|0;
 $57 = HEAP32[$nPattern>>2]|0;
 $58 = (($56) - ($57))|0;
 HEAP32[$loopLimit>>2] = $58;
 HEAP32[$j>>2] = 0;
 HEAP32[$i>>2] = 0;
 while(1) {
  $59 = HEAP32[$i>>2]|0;
  $60 = HEAP32[$loopLimit>>2]|0;
  $61 = ($59|0)<=($60|0);
  if (!($61)) {
   label = 18;
   break;
  }
  $62 = HEAP32[$i>>2]|0;
  $63 = HEAP32[$zStr>>2]|0;
  $64 = (($63) + ($62)|0);
  $65 = HEAP8[$64>>0]|0;
  $66 = $65&255;
  $67 = HEAP32[$zPattern>>2]|0;
  $68 = HEAP8[$67>>0]|0;
  $69 = $68&255;
  $70 = ($66|0)!=($69|0);
  if ($70) {
   label = 11;
  } else {
   $71 = HEAP32[$i>>2]|0;
   $72 = HEAP32[$zStr>>2]|0;
   $73 = (($72) + ($71)|0);
   $74 = HEAP32[$zPattern>>2]|0;
   $75 = HEAP32[$nPattern>>2]|0;
   $76 = (_memcmp($73,$74,$75)|0);
   $77 = ($76|0)!=(0);
   if ($77) {
    label = 11;
   } else {
    $86 = HEAP32[$0>>2]|0;
    $87 = (_sqlite3_context_db_handle($86)|0);
    HEAP32[$db>>2] = $87;
    $88 = HEAP32[$nRep>>2]|0;
    $89 = HEAP32[$nPattern>>2]|0;
    $90 = (($88) - ($89))|0;
    $91 = ($90|0)<(0);
    $92 = $91 << 31 >> 31;
    $93 = $nOut;
    $94 = $93;
    $95 = HEAP32[$94>>2]|0;
    $96 = (($93) + 4)|0;
    $97 = $96;
    $98 = HEAP32[$97>>2]|0;
    $99 = (_i64Add(($95|0),($98|0),($90|0),($92|0))|0);
    $100 = tempRet0;
    $101 = $nOut;
    $102 = $101;
    HEAP32[$102>>2] = $99;
    $103 = (($101) + 4)|0;
    $104 = $103;
    HEAP32[$104>>2] = $100;
    $105 = $nOut;
    $106 = $105;
    $107 = HEAP32[$106>>2]|0;
    $108 = (($105) + 4)|0;
    $109 = $108;
    $110 = HEAP32[$109>>2]|0;
    $111 = (_i64Subtract(($107|0),($110|0),1,0)|0);
    $112 = tempRet0;
    $113 = HEAP32[$db>>2]|0;
    $114 = (($113) + 88|0);
    $115 = HEAP32[$114>>2]|0;
    $116 = ($115|0)<(0);
    $117 = $116 << 31 >> 31;
    $118 = ($112|0)>($117|0);
    $119 = ($111>>>0)>($115>>>0);
    $120 = ($112|0)==($117|0);
    $121 = $120 & $119;
    $122 = $118 | $121;
    if ($122) {
     label = 13;
     break;
    }
    $125 = HEAP32[$zOut>>2]|0;
    HEAP32[$zOld>>2] = $125;
    $126 = HEAP32[$zOut>>2]|0;
    $127 = $nOut;
    $128 = $127;
    $129 = HEAP32[$128>>2]|0;
    $130 = (($127) + 4)|0;
    $131 = $130;
    $132 = HEAP32[$131>>2]|0;
    $133 = (_sqlite3_realloc($126,$129)|0);
    HEAP32[$zOut>>2] = $133;
    $134 = HEAP32[$zOut>>2]|0;
    $135 = ($134|0)==(0|0);
    if ($135) {
     label = 15;
     break;
    }
    $138 = HEAP32[$j>>2]|0;
    $139 = HEAP32[$zOut>>2]|0;
    $140 = (($139) + ($138)|0);
    $141 = HEAP32[$zRep>>2]|0;
    $142 = HEAP32[$nRep>>2]|0;
    _memcpy(($140|0),($141|0),($142|0))|0;
    $143 = HEAP32[$nRep>>2]|0;
    $144 = HEAP32[$j>>2]|0;
    $145 = (($144) + ($143))|0;
    HEAP32[$j>>2] = $145;
    $146 = HEAP32[$nPattern>>2]|0;
    $147 = (($146) - 1)|0;
    $148 = HEAP32[$i>>2]|0;
    $149 = (($148) + ($147))|0;
    HEAP32[$i>>2] = $149;
   }
  }
  if ((label|0) == 11) {
   label = 0;
   $78 = HEAP32[$i>>2]|0;
   $79 = HEAP32[$zStr>>2]|0;
   $80 = (($79) + ($78)|0);
   $81 = HEAP8[$80>>0]|0;
   $82 = HEAP32[$j>>2]|0;
   $83 = (($82) + 1)|0;
   HEAP32[$j>>2] = $83;
   $84 = HEAP32[$zOut>>2]|0;
   $85 = (($84) + ($82)|0);
   HEAP8[$85>>0] = $81;
  }
  $150 = HEAP32[$i>>2]|0;
  $151 = (($150) + 1)|0;
  HEAP32[$i>>2] = $151;
 }
 if ((label|0) == 13) {
  $123 = HEAP32[$0>>2]|0;
  _sqlite3_result_error_toobig($123);
  $124 = HEAP32[$zOut>>2]|0;
  _sqlite3_free($124);
  STACKTOP = sp;return;
 }
 else if ((label|0) == 15) {
  $136 = HEAP32[$0>>2]|0;
  _sqlite3_result_error_nomem($136);
  $137 = HEAP32[$zOld>>2]|0;
  _sqlite3_free($137);
  STACKTOP = sp;return;
 }
 else if ((label|0) == 18) {
  $152 = HEAP32[$j>>2]|0;
  $153 = HEAP32[$zOut>>2]|0;
  $154 = (($153) + ($152)|0);
  $155 = HEAP32[$i>>2]|0;
  $156 = HEAP32[$zStr>>2]|0;
  $157 = (($156) + ($155)|0);
  $158 = HEAP32[$nStr>>2]|0;
  $159 = HEAP32[$i>>2]|0;
  $160 = (($158) - ($159))|0;
  _memcpy(($154|0),($157|0),($160|0))|0;
  $161 = HEAP32[$nStr>>2]|0;
  $162 = HEAP32[$i>>2]|0;
  $163 = (($161) - ($162))|0;
  $164 = HEAP32[$j>>2]|0;
  $165 = (($164) + ($163))|0;
  HEAP32[$j>>2] = $165;
  $166 = HEAP32[$j>>2]|0;
  $167 = HEAP32[$zOut>>2]|0;
  $168 = (($167) + ($166)|0);
  HEAP8[$168>>0] = 0;
  $169 = HEAP32[$0>>2]|0;
  $170 = HEAP32[$zOut>>2]|0;
  $171 = HEAP32[$j>>2]|0;
  _sqlite3_result_text($169,$170,$171,13);
  STACKTOP = sp;return;
 }
}
function _zeroblobFunc($context,$argc,$argv) {
 $context = $context|0;
 $argc = $argc|0;
 $argv = $argv|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $db = 0, $n = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 20|0;
 $1 = sp + 16|0;
 $2 = sp + 12|0;
 $n = sp;
 $db = sp + 8|0;
 HEAP32[$0>>2] = $context;
 HEAP32[$1>>2] = $argc;
 HEAP32[$2>>2] = $argv;
 $3 = HEAP32[$0>>2]|0;
 $4 = (_sqlite3_context_db_handle($3)|0);
 HEAP32[$db>>2] = $4;
 $5 = HEAP32[$2>>2]|0;
 $6 = HEAP32[$5>>2]|0;
 $7 = (_sqlite3_value_int64($6)|0);
 $8 = tempRet0;
 $9 = $n;
 $10 = $9;
 HEAP32[$10>>2] = $7;
 $11 = (($9) + 4)|0;
 $12 = $11;
 HEAP32[$12>>2] = $8;
 $13 = $n;
 $14 = $13;
 $15 = HEAP32[$14>>2]|0;
 $16 = (($13) + 4)|0;
 $17 = $16;
 $18 = HEAP32[$17>>2]|0;
 $19 = HEAP32[$db>>2]|0;
 $20 = (($19) + 88|0);
 $21 = HEAP32[$20>>2]|0;
 $22 = ($21|0)<(0);
 $23 = $22 << 31 >> 31;
 $24 = ($18|0)>($23|0);
 $25 = ($15>>>0)>($21>>>0);
 $26 = ($18|0)==($23|0);
 $27 = $26 & $25;
 $28 = $24 | $27;
 $29 = HEAP32[$0>>2]|0;
 if ($28) {
  _sqlite3_result_error_toobig($29);
  STACKTOP = sp;return;
 } else {
  $30 = $n;
  $31 = $30;
  $32 = HEAP32[$31>>2]|0;
  $33 = (($30) + 4)|0;
  $34 = $33;
  $35 = HEAP32[$34>>2]|0;
  _sqlite3_result_zeroblob($29,$32);
  STACKTOP = sp;return;
 }
}
function _sumStep($context,$argc,$argv) {
 $context = $context|0;
 $argc = $argc|0;
 $argv = $argv|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0.0, $43 = 0, $44 = 0.0;
 var $45 = 0.0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0.0, $69 = 0, $7 = 0, $70 = 0.0, $71 = 0.0, $72 = 0, $73 = 0, $8 = 0, $9 = 0, $or$cond = 0, $p = 0, $type = 0, $v = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 24|0;
 $1 = sp + 20|0;
 $2 = sp + 16|0;
 $p = sp + 12|0;
 $type = sp + 8|0;
 $v = sp;
 HEAP32[$0>>2] = $context;
 HEAP32[$1>>2] = $argc;
 HEAP32[$2>>2] = $argv;
 $3 = HEAP32[$0>>2]|0;
 $4 = (_sqlite3_aggregate_context($3,32)|0);
 HEAP32[$p>>2] = $4;
 $5 = HEAP32[$2>>2]|0;
 $6 = HEAP32[$5>>2]|0;
 $7 = (_sqlite3_value_numeric_type($6)|0);
 HEAP32[$type>>2] = $7;
 $8 = HEAP32[$p>>2]|0;
 $9 = ($8|0)!=(0|0);
 $10 = HEAP32[$type>>2]|0;
 $11 = ($10|0)!=(5);
 $or$cond = $9 & $11;
 if (!($or$cond)) {
  STACKTOP = sp;return;
 }
 $12 = HEAP32[$p>>2]|0;
 $13 = (($12) + 16|0);
 $14 = $13;
 $15 = $14;
 $16 = HEAP32[$15>>2]|0;
 $17 = (($14) + 4)|0;
 $18 = $17;
 $19 = HEAP32[$18>>2]|0;
 $20 = (_i64Add(($16|0),($19|0),1,0)|0);
 $21 = tempRet0;
 $22 = $13;
 $23 = $22;
 HEAP32[$23>>2] = $20;
 $24 = (($22) + 4)|0;
 $25 = $24;
 HEAP32[$25>>2] = $21;
 $26 = HEAP32[$type>>2]|0;
 $27 = ($26|0)==(1);
 $28 = HEAP32[$2>>2]|0;
 $29 = HEAP32[$28>>2]|0;
 if (!($27)) {
  $68 = (+_sqlite3_value_double($29));
  $69 = HEAP32[$p>>2]|0;
  $70 = +HEAPF64[$69>>3];
  $71 = $70 + $68;
  HEAPF64[$69>>3] = $71;
  $72 = HEAP32[$p>>2]|0;
  $73 = (($72) + 25|0);
  HEAP8[$73>>0] = 1;
  STACKTOP = sp;return;
 }
 $30 = (_sqlite3_value_int64($29)|0);
 $31 = tempRet0;
 $32 = $v;
 $33 = $32;
 HEAP32[$33>>2] = $30;
 $34 = (($32) + 4)|0;
 $35 = $34;
 HEAP32[$35>>2] = $31;
 $36 = $v;
 $37 = $36;
 $38 = HEAP32[$37>>2]|0;
 $39 = (($36) + 4)|0;
 $40 = $39;
 $41 = HEAP32[$40>>2]|0;
 $42 = (+($38>>>0)) + (4294967296.0*(+($41|0)));
 $43 = HEAP32[$p>>2]|0;
 $44 = +HEAPF64[$43>>3];
 $45 = $44 + $42;
 HEAPF64[$43>>3] = $45;
 $46 = HEAP32[$p>>2]|0;
 $47 = (($46) + 25|0);
 $48 = HEAP8[$47>>0]|0;
 $49 = $48&255;
 $50 = HEAP32[$p>>2]|0;
 $51 = (($50) + 24|0);
 $52 = HEAP8[$51>>0]|0;
 $53 = $52&255;
 $54 = $49 | $53;
 $55 = ($54|0)==(0);
 if (!($55)) {
  STACKTOP = sp;return;
 }
 $56 = HEAP32[$p>>2]|0;
 $57 = (($56) + 8|0);
 $58 = $v;
 $59 = $58;
 $60 = HEAP32[$59>>2]|0;
 $61 = (($58) + 4)|0;
 $62 = $61;
 $63 = HEAP32[$62>>2]|0;
 $64 = (_sqlite3AddInt64($57,$60,$63)|0);
 $65 = ($64|0)!=(0);
 if (!($65)) {
  STACKTOP = sp;return;
 }
 $66 = HEAP32[$p>>2]|0;
 $67 = (($66) + 24|0);
 HEAP8[$67>>0] = 1;
 STACKTOP = sp;return;
}
function _sumFinalize($context) {
 $context = $context|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0.0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $p = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $p = sp;
 HEAP32[$0>>2] = $context;
 $1 = HEAP32[$0>>2]|0;
 $2 = (_sqlite3_aggregate_context($1,0)|0);
 HEAP32[$p>>2] = $2;
 $3 = HEAP32[$p>>2]|0;
 $4 = ($3|0)!=(0|0);
 if (!($4)) {
  STACKTOP = sp;return;
 }
 $5 = HEAP32[$p>>2]|0;
 $6 = (($5) + 16|0);
 $7 = $6;
 $8 = $7;
 $9 = HEAP32[$8>>2]|0;
 $10 = (($7) + 4)|0;
 $11 = $10;
 $12 = HEAP32[$11>>2]|0;
 $13 = ($12|0)>(0);
 $14 = ($9>>>0)>(0);
 $15 = ($12|0)==(0);
 $16 = $15 & $14;
 $17 = $13 | $16;
 if (!($17)) {
  STACKTOP = sp;return;
 }
 $18 = HEAP32[$p>>2]|0;
 $19 = (($18) + 24|0);
 $20 = HEAP8[$19>>0]|0;
 $21 = ($20<<24>>24)!=(0);
 if ($21) {
  $22 = HEAP32[$0>>2]|0;
  _sqlite3_result_error($22,9784,-1);
  STACKTOP = sp;return;
 }
 $23 = HEAP32[$p>>2]|0;
 $24 = (($23) + 25|0);
 $25 = HEAP8[$24>>0]|0;
 $26 = ($25<<24>>24)!=(0);
 $27 = HEAP32[$0>>2]|0;
 $28 = HEAP32[$p>>2]|0;
 if ($26) {
  $29 = +HEAPF64[$28>>3];
  _sqlite3_result_double($27,$29);
  STACKTOP = sp;return;
 } else {
  $30 = (($28) + 8|0);
  $31 = $30;
  $32 = $31;
  $33 = HEAP32[$32>>2]|0;
  $34 = (($31) + 4)|0;
  $35 = $34;
  $36 = HEAP32[$35>>2]|0;
  _sqlite3_result_int64($27,$33,$36);
  STACKTOP = sp;return;
 }
}
function _totalFinalize($context) {
 $context = $context|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0.0, $8 = 0.0, $p = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $p = sp;
 HEAP32[$0>>2] = $context;
 $1 = HEAP32[$0>>2]|0;
 $2 = (_sqlite3_aggregate_context($1,0)|0);
 HEAP32[$p>>2] = $2;
 $3 = HEAP32[$0>>2]|0;
 $4 = HEAP32[$p>>2]|0;
 $5 = ($4|0)!=(0|0);
 if (!($5)) {
  $8 = 0.0;
  _sqlite3_result_double($3,$8);
  STACKTOP = sp;return;
 }
 $6 = HEAP32[$p>>2]|0;
 $7 = +HEAPF64[$6>>3];
 $8 = $7;
 _sqlite3_result_double($3,$8);
 STACKTOP = sp;return;
}
function _avgFinalize($context) {
 $context = $context|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0.0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0.0, $3 = 0, $30 = 0.0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $p = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $p = sp;
 HEAP32[$0>>2] = $context;
 $1 = HEAP32[$0>>2]|0;
 $2 = (_sqlite3_aggregate_context($1,0)|0);
 HEAP32[$p>>2] = $2;
 $3 = HEAP32[$p>>2]|0;
 $4 = ($3|0)!=(0|0);
 if (!($4)) {
  STACKTOP = sp;return;
 }
 $5 = HEAP32[$p>>2]|0;
 $6 = (($5) + 16|0);
 $7 = $6;
 $8 = $7;
 $9 = HEAP32[$8>>2]|0;
 $10 = (($7) + 4)|0;
 $11 = $10;
 $12 = HEAP32[$11>>2]|0;
 $13 = ($12|0)>(0);
 $14 = ($9>>>0)>(0);
 $15 = ($12|0)==(0);
 $16 = $15 & $14;
 $17 = $13 | $16;
 if (!($17)) {
  STACKTOP = sp;return;
 }
 $18 = HEAP32[$0>>2]|0;
 $19 = HEAP32[$p>>2]|0;
 $20 = +HEAPF64[$19>>3];
 $21 = HEAP32[$p>>2]|0;
 $22 = (($21) + 16|0);
 $23 = $22;
 $24 = $23;
 $25 = HEAP32[$24>>2]|0;
 $26 = (($23) + 4)|0;
 $27 = $26;
 $28 = HEAP32[$27>>2]|0;
 $29 = (+($25>>>0)) + (4294967296.0*(+($28|0)));
 $30 = $20 / $29;
 _sqlite3_result_double($18,$30);
 STACKTOP = sp;return;
}
function _countStep($context,$argc,$argv) {
 $context = $context|0;
 $argc = $argc|0;
 $argv = $argv|0;
 var $$old = 0, $$old1 = 0, $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0;
 var $25 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $or$cond = 0, $p = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $1 = sp + 8|0;
 $2 = sp + 4|0;
 $p = sp;
 HEAP32[$0>>2] = $context;
 HEAP32[$1>>2] = $argc;
 HEAP32[$2>>2] = $argv;
 $3 = HEAP32[$0>>2]|0;
 $4 = (_sqlite3_aggregate_context($3,8)|0);
 HEAP32[$p>>2] = $4;
 $5 = HEAP32[$1>>2]|0;
 $6 = ($5|0)==(0);
 if ($6) {
  $$old = HEAP32[$p>>2]|0;
  $$old1 = ($$old|0)!=(0|0);
  if (!($$old1)) {
   STACKTOP = sp;return;
  }
 } else {
  $7 = HEAP32[$2>>2]|0;
  $8 = HEAP32[$7>>2]|0;
  $9 = (_sqlite3_value_type($8)|0);
  $10 = (5)!=($9|0);
  $11 = HEAP32[$p>>2]|0;
  $12 = ($11|0)!=(0|0);
  $or$cond = $10 & $12;
  if (!($or$cond)) {
   STACKTOP = sp;return;
  }
 }
 $13 = HEAP32[$p>>2]|0;
 $14 = $13;
 $15 = $14;
 $16 = HEAP32[$15>>2]|0;
 $17 = (($14) + 4)|0;
 $18 = $17;
 $19 = HEAP32[$18>>2]|0;
 $20 = (_i64Add(($16|0),($19|0),1,0)|0);
 $21 = tempRet0;
 $22 = $13;
 $23 = $22;
 HEAP32[$23>>2] = $20;
 $24 = (($22) + 4)|0;
 $25 = $24;
 HEAP32[$25>>2] = $21;
 STACKTOP = sp;return;
}
function _countFinalize($context) {
 $context = $context|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $p = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $p = sp;
 HEAP32[$0>>2] = $context;
 $1 = HEAP32[$0>>2]|0;
 $2 = (_sqlite3_aggregate_context($1,0)|0);
 HEAP32[$p>>2] = $2;
 $3 = HEAP32[$0>>2]|0;
 $4 = HEAP32[$p>>2]|0;
 $5 = ($4|0)!=(0|0);
 if (!($5)) {
  $13 = 0;$14 = 0;
  _sqlite3_result_int64($3,$13,$14);
  STACKTOP = sp;return;
 }
 $6 = HEAP32[$p>>2]|0;
 $7 = $6;
 $8 = $7;
 $9 = HEAP32[$8>>2]|0;
 $10 = (($7) + 4)|0;
 $11 = $10;
 $12 = HEAP32[$11>>2]|0;
 $13 = $9;$14 = $12;
 _sqlite3_result_int64($3,$13,$14);
 STACKTOP = sp;return;
}
function _groupConcatStep($context,$argc,$argv) {
 $context = $context|0;
 $argc = $argc|0;
 $argv = $argv|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $db = 0, $firstTerm = 0, $nSep = 0, $nVal = 0, $pAccum = 0, $zSep = 0;
 var $zVal = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $0 = sp + 36|0;
 $1 = sp + 32|0;
 $2 = sp + 28|0;
 $zVal = sp + 24|0;
 $pAccum = sp + 20|0;
 $zSep = sp + 16|0;
 $nVal = sp + 12|0;
 $nSep = sp + 8|0;
 $db = sp + 4|0;
 $firstTerm = sp;
 HEAP32[$0>>2] = $context;
 HEAP32[$1>>2] = $argc;
 HEAP32[$2>>2] = $argv;
 $3 = HEAP32[$2>>2]|0;
 $4 = HEAP32[$3>>2]|0;
 $5 = (_sqlite3_value_type($4)|0);
 $6 = ($5|0)==(5);
 if ($6) {
  STACKTOP = sp;return;
 }
 $7 = HEAP32[$0>>2]|0;
 $8 = (_sqlite3_aggregate_context($7,28)|0);
 HEAP32[$pAccum>>2] = $8;
 $9 = HEAP32[$pAccum>>2]|0;
 $10 = ($9|0)!=(0|0);
 if (!($10)) {
  STACKTOP = sp;return;
 }
 $11 = HEAP32[$0>>2]|0;
 $12 = (_sqlite3_context_db_handle($11)|0);
 HEAP32[$db>>2] = $12;
 $13 = HEAP32[$pAccum>>2]|0;
 $14 = (($13) + 24|0);
 $15 = HEAP8[$14>>0]|0;
 $16 = $15&255;
 $17 = ($16|0)==(0);
 $18 = $17&1;
 HEAP32[$firstTerm>>2] = $18;
 $19 = HEAP32[$pAccum>>2]|0;
 $20 = (($19) + 24|0);
 HEAP8[$20>>0] = 2;
 $21 = HEAP32[$db>>2]|0;
 $22 = (($21) + 88|0);
 $23 = HEAP32[$22>>2]|0;
 $24 = HEAP32[$pAccum>>2]|0;
 $25 = (($24) + 20|0);
 HEAP32[$25>>2] = $23;
 $26 = HEAP32[$firstTerm>>2]|0;
 $27 = ($26|0)!=(0);
 if (!($27)) {
  $28 = HEAP32[$1>>2]|0;
  $29 = ($28|0)==(2);
  if ($29) {
   $30 = HEAP32[$2>>2]|0;
   $31 = (($30) + 4|0);
   $32 = HEAP32[$31>>2]|0;
   $33 = (_sqlite3_value_text($32)|0);
   HEAP32[$zSep>>2] = $33;
   $34 = HEAP32[$2>>2]|0;
   $35 = (($34) + 4|0);
   $36 = HEAP32[$35>>2]|0;
   $37 = (_sqlite3_value_bytes($36)|0);
   HEAP32[$nSep>>2] = $37;
  } else {
   HEAP32[$zSep>>2] = 9776;
   HEAP32[$nSep>>2] = 1;
  }
  $38 = HEAP32[$nSep>>2]|0;
  $39 = ($38|0)!=(0);
  if ($39) {
   $40 = HEAP32[$pAccum>>2]|0;
   $41 = HEAP32[$zSep>>2]|0;
   $42 = HEAP32[$nSep>>2]|0;
   _sqlite3StrAccumAppend($40,$41,$42);
  }
 }
 $43 = HEAP32[$2>>2]|0;
 $44 = HEAP32[$43>>2]|0;
 $45 = (_sqlite3_value_text($44)|0);
 HEAP32[$zVal>>2] = $45;
 $46 = HEAP32[$2>>2]|0;
 $47 = HEAP32[$46>>2]|0;
 $48 = (_sqlite3_value_bytes($47)|0);
 HEAP32[$nVal>>2] = $48;
 $49 = HEAP32[$zVal>>2]|0;
 $50 = ($49|0)!=(0|0);
 if (!($50)) {
  STACKTOP = sp;return;
 }
 $51 = HEAP32[$pAccum>>2]|0;
 $52 = HEAP32[$zVal>>2]|0;
 $53 = HEAP32[$nVal>>2]|0;
 _sqlite3StrAccumAppend($51,$52,$53);
 STACKTOP = sp;return;
}
function _groupConcatFinalize($context) {
 $context = $context|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $pAccum = 0;
 var label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $pAccum = sp;
 HEAP32[$0>>2] = $context;
 $1 = HEAP32[$0>>2]|0;
 $2 = (_sqlite3_aggregate_context($1,0)|0);
 HEAP32[$pAccum>>2] = $2;
 $3 = HEAP32[$pAccum>>2]|0;
 $4 = ($3|0)!=(0|0);
 if (!($4)) {
  STACKTOP = sp;return;
 }
 $5 = HEAP32[$pAccum>>2]|0;
 $6 = (($5) + 25|0);
 $7 = HEAP8[$6>>0]|0;
 $8 = $7&255;
 $9 = ($8|0)==(2);
 if ($9) {
  $10 = HEAP32[$0>>2]|0;
  _sqlite3_result_error_toobig($10);
  STACKTOP = sp;return;
 }
 $11 = HEAP32[$pAccum>>2]|0;
 $12 = (($11) + 25|0);
 $13 = HEAP8[$12>>0]|0;
 $14 = $13&255;
 $15 = ($14|0)==(1);
 $16 = HEAP32[$0>>2]|0;
 if ($15) {
  _sqlite3_result_error_nomem($16);
  STACKTOP = sp;return;
 } else {
  $17 = HEAP32[$pAccum>>2]|0;
  $18 = (_sqlite3StrAccumFinish($17)|0);
  _sqlite3_result_text($16,$18,-1,13);
  STACKTOP = sp;return;
 }
}
function _likeFunc($context,$argc,$argv) {
 $context = $context|0;
 $argc = $argc|0;
 $argv = $argv|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $db = 0, $escape = 0, $nPat = 0, $or$cond = 0, $pInfo = 0, $zA = 0, $zB = 0, $zEsc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $0 = sp + 36|0;
 $1 = sp + 32|0;
 $2 = sp + 28|0;
 $zA = sp + 24|0;
 $zB = sp + 20|0;
 $escape = sp + 16|0;
 $nPat = sp + 12|0;
 $db = sp + 8|0;
 $zEsc = sp + 4|0;
 $pInfo = sp;
 HEAP32[$0>>2] = $context;
 HEAP32[$1>>2] = $argc;
 HEAP32[$2>>2] = $argv;
 HEAP32[$escape>>2] = 0;
 $3 = HEAP32[$0>>2]|0;
 $4 = (_sqlite3_context_db_handle($3)|0);
 HEAP32[$db>>2] = $4;
 $5 = HEAP32[$2>>2]|0;
 $6 = HEAP32[$5>>2]|0;
 $7 = (_sqlite3_value_text($6)|0);
 HEAP32[$zB>>2] = $7;
 $8 = HEAP32[$2>>2]|0;
 $9 = (($8) + 4|0);
 $10 = HEAP32[$9>>2]|0;
 $11 = (_sqlite3_value_text($10)|0);
 HEAP32[$zA>>2] = $11;
 $12 = HEAP32[$2>>2]|0;
 $13 = HEAP32[$12>>2]|0;
 $14 = (_sqlite3_value_bytes($13)|0);
 HEAP32[$nPat>>2] = $14;
 $15 = HEAP32[$nPat>>2]|0;
 $16 = HEAP32[$db>>2]|0;
 $17 = (($16) + 88|0);
 $18 = (($17) + 32|0);
 $19 = HEAP32[$18>>2]|0;
 $20 = ($15|0)>($19|0);
 if ($20) {
  $21 = HEAP32[$0>>2]|0;
  _sqlite3_result_error($21,9688,-1);
  STACKTOP = sp;return;
 }
 $22 = HEAP32[$1>>2]|0;
 $23 = ($22|0)==(3);
 do {
  if ($23) {
   $24 = HEAP32[$2>>2]|0;
   $25 = (($24) + 8|0);
   $26 = HEAP32[$25>>2]|0;
   $27 = (_sqlite3_value_text($26)|0);
   HEAP32[$zEsc>>2] = $27;
   $28 = HEAP32[$zEsc>>2]|0;
   $29 = ($28|0)==(0|0);
   if ($29) {
    STACKTOP = sp;return;
   }
   $30 = HEAP32[$zEsc>>2]|0;
   $31 = (_sqlite3Utf8CharLen($30,-1)|0);
   $32 = ($31|0)!=(1);
   if (!($32)) {
    $34 = (_sqlite3Utf8Read($zEsc)|0);
    HEAP32[$escape>>2] = $34;
    break;
   }
   $33 = HEAP32[$0>>2]|0;
   _sqlite3_result_error($33,9728,-1);
   STACKTOP = sp;return;
  }
 } while(0);
 $35 = HEAP32[$zA>>2]|0;
 $36 = ($35|0)!=(0|0);
 $37 = HEAP32[$zB>>2]|0;
 $38 = ($37|0)!=(0|0);
 $or$cond = $36 & $38;
 if (!($or$cond)) {
  STACKTOP = sp;return;
 }
 $39 = HEAP32[$0>>2]|0;
 $40 = (_sqlite3_user_data($39)|0);
 HEAP32[$pInfo>>2] = $40;
 $41 = HEAP32[$0>>2]|0;
 $42 = HEAP32[$zB>>2]|0;
 $43 = HEAP32[$zA>>2]|0;
 $44 = HEAP32[$pInfo>>2]|0;
 $45 = HEAP32[$escape>>2]|0;
 $46 = (_patternCompare($42,$43,$44,$45)|0);
 _sqlite3_result_int($41,$46);
 STACKTOP = sp;return;
}
function _sqlite3RegisterDateTimeFunctions() {
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $aFunc = 0, $i = 0, $pHash = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $i = sp + 8|0;
 $pHash = sp + 4|0;
 $aFunc = sp;
 HEAP32[$pHash>>2] = 1576;
 HEAP32[$aFunc>>2] = 9016;
 HEAP32[$i>>2] = 0;
 while(1) {
  $0 = HEAP32[$i>>2]|0;
  $1 = ($0|0)<(8);
  if (!($1)) {
   break;
  }
  $2 = HEAP32[$pHash>>2]|0;
  $3 = HEAP32[$i>>2]|0;
  $4 = HEAP32[$aFunc>>2]|0;
  $5 = (($4) + (($3*36)|0)|0);
  _sqlite3FuncDefInsert($2,$5);
  $6 = HEAP32[$i>>2]|0;
  $7 = (($6) + 1)|0;
  HEAP32[$i>>2] = $7;
 }
 STACKTOP = sp;return;
}
function _sqlite3AlterFunctions() {
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $aFunc = 0, $i = 0, $pHash = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $i = sp + 8|0;
 $pHash = sp + 4|0;
 $aFunc = sp;
 HEAP32[$pHash>>2] = 1576;
 HEAP32[$aFunc>>2] = 8792;
 HEAP32[$i>>2] = 0;
 while(1) {
  $0 = HEAP32[$i>>2]|0;
  $1 = ($0|0)<(3);
  if (!($1)) {
   break;
  }
  $2 = HEAP32[$pHash>>2]|0;
  $3 = HEAP32[$i>>2]|0;
  $4 = HEAP32[$aFunc>>2]|0;
  $5 = (($4) + (($3*36)|0)|0);
  _sqlite3FuncDefInsert($2,$5);
  $6 = HEAP32[$i>>2]|0;
  $7 = (($6) + 1)|0;
  HEAP32[$i>>2] = $7;
 }
 STACKTOP = sp;return;
}
function _renameTableFunc($context,$NotUsed,$argv) {
 $context = $context|0;
 $NotUsed = $NotUsed|0;
 $argv = $argv|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $db = 0, $len = 0, $or$cond = 0, $tname = 0, $token = 0, $vararg_buffer = 0, $vararg_ptr2 = 0, $vararg_ptr3 = 0, $vararg_ptr4 = 0, $zCsr = 0, $zRet = 0, $zSql = 0, $zTableName = 0;
 var label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 64|0;
 $vararg_buffer = sp;
 $0 = sp + 60|0;
 $1 = sp + 56|0;
 $2 = sp + 52|0;
 $zSql = sp + 48|0;
 $zTableName = sp + 44|0;
 $token = sp + 40|0;
 $tname = sp + 32|0;
 $zCsr = sp + 28|0;
 $len = sp + 24|0;
 $zRet = sp + 20|0;
 $db = sp + 16|0;
 HEAP32[$0>>2] = $context;
 HEAP32[$1>>2] = $NotUsed;
 HEAP32[$2>>2] = $argv;
 $3 = HEAP32[$2>>2]|0;
 $4 = HEAP32[$3>>2]|0;
 $5 = (_sqlite3_value_text($4)|0);
 HEAP32[$zSql>>2] = $5;
 $6 = HEAP32[$2>>2]|0;
 $7 = (($6) + 4|0);
 $8 = HEAP32[$7>>2]|0;
 $9 = (_sqlite3_value_text($8)|0);
 HEAP32[$zTableName>>2] = $9;
 $10 = HEAP32[$zSql>>2]|0;
 HEAP32[$zCsr>>2] = $10;
 HEAP32[$len>>2] = 0;
 $11 = HEAP32[$0>>2]|0;
 $12 = (_sqlite3_context_db_handle($11)|0);
 HEAP32[$db>>2] = $12;
 $13 = HEAP32[$zSql>>2]|0;
 $14 = ($13|0)!=(0|0);
 if (!($14)) {
  STACKTOP = sp;return;
 }
 while(1) {
  $15 = HEAP32[$zCsr>>2]|0;
  $16 = HEAP8[$15>>0]|0;
  $17 = ($16<<24>>24)!=(0);
  if (!($17)) {
   label = 7;
   break;
  }
  $18 = HEAP32[$zCsr>>2]|0;
  HEAP32[$tname>>2] = $18;
  $19 = HEAP32[$len>>2]|0;
  $20 = (($tname) + 4|0);
  HEAP32[$20>>2] = $19;
  while(1) {
   $21 = HEAP32[$len>>2]|0;
   $22 = HEAP32[$zCsr>>2]|0;
   $23 = (($22) + ($21)|0);
   HEAP32[$zCsr>>2] = $23;
   $24 = HEAP32[$zCsr>>2]|0;
   $25 = (_sqlite3GetToken($24,$token)|0);
   HEAP32[$len>>2] = $25;
   $26 = HEAP32[$token>>2]|0;
   $27 = ($26|0)==(151);
   if (!($27)) {
    break;
   }
  }
  $28 = HEAP32[$token>>2]|0;
  $29 = ($28|0)!=(22);
  $30 = HEAP32[$token>>2]|0;
  $31 = ($30|0)!=(125);
  $or$cond = $29 & $31;
  if (!($or$cond)) {
   break;
  }
 }
 if ((label|0) == 7) {
  STACKTOP = sp;return;
 }
 $32 = HEAP32[$db>>2]|0;
 $33 = HEAP32[$tname>>2]|0;
 $34 = HEAP32[$zSql>>2]|0;
 $35 = $33;
 $36 = $34;
 $37 = (($35) - ($36))|0;
 $38 = HEAP32[$zSql>>2]|0;
 $39 = HEAP32[$zTableName>>2]|0;
 $40 = HEAP32[$tname>>2]|0;
 $41 = (($tname) + 4|0);
 $42 = HEAP32[$41>>2]|0;
 $43 = (($40) + ($42)|0);
 HEAP32[$vararg_buffer>>2] = $37;
 $vararg_ptr2 = (($vararg_buffer) + 4|0);
 HEAP32[$vararg_ptr2>>2] = $38;
 $vararg_ptr3 = (($vararg_buffer) + 8|0);
 HEAP32[$vararg_ptr3>>2] = $39;
 $vararg_ptr4 = (($vararg_buffer) + 12|0);
 HEAP32[$vararg_ptr4>>2] = $43;
 $44 = (_sqlite3MPrintf($32,9000,$vararg_buffer)|0);
 HEAP32[$zRet>>2] = $44;
 $45 = HEAP32[$0>>2]|0;
 $46 = HEAP32[$zRet>>2]|0;
 _sqlite3_result_text($45,$46,-1,16);
 STACKTOP = sp;return;
}
function _renameTriggerFunc($context,$NotUsed,$argv) {
 $context = $context|0;
 $NotUsed = $NotUsed|0;
 $argv = $argv|0;
 var $$ = 0, $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0;
 var $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0;
 var $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $db = 0, $dist = 0;
 var $len = 0, $or$cond = 0, $or$cond3 = 0, $or$cond6 = 0, $tname = 0, $token = 0, $vararg_buffer = 0, $vararg_ptr7 = 0, $vararg_ptr8 = 0, $vararg_ptr9 = 0, $zCsr = 0, $zRet = 0, $zSql = 0, $zTableName = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 80|0;
 $vararg_buffer = sp;
 $0 = sp + 68|0;
 $1 = sp + 64|0;
 $2 = sp + 60|0;
 $zSql = sp + 56|0;
 $zTableName = sp + 52|0;
 $token = sp + 48|0;
 $tname = sp + 40|0;
 $dist = sp + 32|0;
 $zCsr = sp + 28|0;
 $len = sp + 24|0;
 $zRet = sp + 20|0;
 $db = sp + 16|0;
 HEAP32[$0>>2] = $context;
 HEAP32[$1>>2] = $NotUsed;
 HEAP32[$2>>2] = $argv;
 $3 = HEAP32[$2>>2]|0;
 $4 = HEAP32[$3>>2]|0;
 $5 = (_sqlite3_value_text($4)|0);
 HEAP32[$zSql>>2] = $5;
 $6 = HEAP32[$2>>2]|0;
 $7 = (($6) + 4|0);
 $8 = HEAP32[$7>>2]|0;
 $9 = (_sqlite3_value_text($8)|0);
 HEAP32[$zTableName>>2] = $9;
 HEAP32[$dist>>2] = 3;
 $10 = HEAP32[$zSql>>2]|0;
 HEAP32[$zCsr>>2] = $10;
 HEAP32[$len>>2] = 0;
 $11 = HEAP32[$0>>2]|0;
 $12 = (_sqlite3_context_db_handle($11)|0);
 HEAP32[$db>>2] = $12;
 $13 = HEAP32[$zSql>>2]|0;
 $14 = ($13|0)!=(0|0);
 if (!($14)) {
  STACKTOP = sp;return;
 }
 while(1) {
  $15 = HEAP32[$zCsr>>2]|0;
  $16 = HEAP8[$15>>0]|0;
  $17 = ($16<<24>>24)!=(0);
  if (!($17)) {
   label = 8;
   break;
  }
  $18 = HEAP32[$zCsr>>2]|0;
  HEAP32[$tname>>2] = $18;
  $19 = HEAP32[$len>>2]|0;
  $20 = (($tname) + 4|0);
  HEAP32[$20>>2] = $19;
  while(1) {
   $21 = HEAP32[$len>>2]|0;
   $22 = HEAP32[$zCsr>>2]|0;
   $23 = (($22) + ($21)|0);
   HEAP32[$zCsr>>2] = $23;
   $24 = HEAP32[$zCsr>>2]|0;
   $25 = (_sqlite3GetToken($24,$token)|0);
   HEAP32[$len>>2] = $25;
   $26 = HEAP32[$token>>2]|0;
   $27 = ($26|0)==(151);
   if (!($27)) {
    break;
   }
  }
  $28 = HEAP32[$dist>>2]|0;
  $29 = (($28) + 1)|0;
  HEAP32[$dist>>2] = $29;
  $30 = HEAP32[$token>>2]|0;
  $31 = ($30|0)==(122);
  $32 = HEAP32[$token>>2]|0;
  $33 = ($32|0)==(107);
  $or$cond = $31 | $33;
  $$ = $or$cond ? 0 : $29;
  HEAP32[$dist>>2] = $$;
  $34 = HEAP32[$dist>>2]|0;
  $35 = ($34|0)!=(2);
  if ($35) {
   continue;
  }
  $36 = HEAP32[$token>>2]|0;
  $37 = ($36|0)!=(137);
  $38 = HEAP32[$token>>2]|0;
  $39 = ($38|0)!=(46);
  $or$cond3 = $37 & $39;
  $40 = HEAP32[$token>>2]|0;
  $41 = ($40|0)!=(5);
  $or$cond6 = $or$cond3 & $41;
  if (!($or$cond6)) {
   break;
  }
 }
 if ((label|0) == 8) {
  STACKTOP = sp;return;
 }
 $42 = HEAP32[$db>>2]|0;
 $43 = HEAP32[$tname>>2]|0;
 $44 = HEAP32[$zSql>>2]|0;
 $45 = $43;
 $46 = $44;
 $47 = (($45) - ($46))|0;
 $48 = HEAP32[$zSql>>2]|0;
 $49 = HEAP32[$zTableName>>2]|0;
 $50 = HEAP32[$tname>>2]|0;
 $51 = (($tname) + 4|0);
 $52 = HEAP32[$51>>2]|0;
 $53 = (($50) + ($52)|0);
 HEAP32[$vararg_buffer>>2] = $47;
 $vararg_ptr7 = (($vararg_buffer) + 4|0);
 HEAP32[$vararg_ptr7>>2] = $48;
 $vararg_ptr8 = (($vararg_buffer) + 8|0);
 HEAP32[$vararg_ptr8>>2] = $49;
 $vararg_ptr9 = (($vararg_buffer) + 12|0);
 HEAP32[$vararg_ptr9>>2] = $53;
 $54 = (_sqlite3MPrintf($42,9000,$vararg_buffer)|0);
 HEAP32[$zRet>>2] = $54;
 $55 = HEAP32[$0>>2]|0;
 $56 = HEAP32[$zRet>>2]|0;
 _sqlite3_result_text($55,$56,-1,16);
 STACKTOP = sp;return;
}
function _renameParentFunc($context,$NotUsed,$argv) {
 $context = $context|0;
 $NotUsed = $NotUsed|0;
 $argv = $argv|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0;
 var $81 = 0, $82 = 0, $9 = 0, $db = 0, $n = 0, $or$cond = 0, $token = 0, $vararg_buffer5 = 0, $vararg_ptr2 = 0, $vararg_ptr3 = 0, $vararg_ptr4 = 0, $vararg_ptr8 = 0, $z = 0, $zInput = 0, $zNew = 0, $zOld = 0, $zOut = 0, $zOutput = 0, $zParent = 0, $zResult = 0;
 var label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 80|0;
 $vararg_buffer5 = sp;
 $0 = sp + 68|0;
 $1 = sp + 64|0;
 $2 = sp + 60|0;
 $db = sp + 56|0;
 $zOutput = sp + 52|0;
 $zResult = sp + 48|0;
 $zInput = sp + 44|0;
 $zOld = sp + 40|0;
 $zNew = sp + 36|0;
 $z = sp + 32|0;
 $n = sp + 28|0;
 $token = sp + 24|0;
 $zParent = sp + 20|0;
 $zOut = sp + 16|0;
 HEAP32[$0>>2] = $context;
 HEAP32[$1>>2] = $NotUsed;
 HEAP32[$2>>2] = $argv;
 $3 = HEAP32[$0>>2]|0;
 $4 = (_sqlite3_context_db_handle($3)|0);
 HEAP32[$db>>2] = $4;
 HEAP32[$zOutput>>2] = 0;
 $5 = HEAP32[$2>>2]|0;
 $6 = HEAP32[$5>>2]|0;
 $7 = (_sqlite3_value_text($6)|0);
 HEAP32[$zInput>>2] = $7;
 $8 = HEAP32[$2>>2]|0;
 $9 = (($8) + 4|0);
 $10 = HEAP32[$9>>2]|0;
 $11 = (_sqlite3_value_text($10)|0);
 HEAP32[$zOld>>2] = $11;
 $12 = HEAP32[$2>>2]|0;
 $13 = (($12) + 8|0);
 $14 = HEAP32[$13>>2]|0;
 $15 = (_sqlite3_value_text($14)|0);
 HEAP32[$zNew>>2] = $15;
 $16 = HEAP32[$zInput>>2]|0;
 $17 = ($16|0)==(0|0);
 $18 = HEAP32[$zOld>>2]|0;
 $19 = ($18|0)==(0|0);
 $or$cond = $17 | $19;
 if ($or$cond) {
  STACKTOP = sp;return;
 }
 $20 = HEAP32[$zInput>>2]|0;
 HEAP32[$z>>2] = $20;
 while(1) {
  $21 = HEAP32[$z>>2]|0;
  $22 = HEAP8[$21>>0]|0;
  $23 = ($22<<24>>24)!=(0);
  if (!($23)) {
   break;
  }
  $24 = HEAP32[$z>>2]|0;
  $25 = (_sqlite3GetToken($24,$token)|0);
  HEAP32[$n>>2] = $25;
  $26 = HEAP32[$token>>2]|0;
  $27 = ($26|0)==(105);
  if ($27) {
   while(1) {
    $28 = HEAP32[$n>>2]|0;
    $29 = HEAP32[$z>>2]|0;
    $30 = (($29) + ($28)|0);
    HEAP32[$z>>2] = $30;
    $31 = HEAP32[$z>>2]|0;
    $32 = (_sqlite3GetToken($31,$token)|0);
    HEAP32[$n>>2] = $32;
    $33 = HEAP32[$token>>2]|0;
    $34 = ($33|0)==(151);
    if (!($34)) {
     break;
    }
   }
   $35 = HEAP32[$db>>2]|0;
   $36 = HEAP32[$z>>2]|0;
   $37 = HEAP32[$n>>2]|0;
   $38 = ($37|0)<(0);
   $39 = $38 << 31 >> 31;
   $40 = (_sqlite3DbStrNDup($35,$36,$37,$39)|0);
   HEAP32[$zParent>>2] = $40;
   $41 = HEAP32[$zParent>>2]|0;
   $42 = ($41|0)==(0|0);
   if ($42) {
    break;
   }
   $43 = HEAP32[$zParent>>2]|0;
   (_sqlite3Dequote($43)|0);
   $44 = HEAP32[$zOld>>2]|0;
   $45 = HEAP32[$zParent>>2]|0;
   $46 = (_sqlite3_stricmp($44,$45)|0);
   $47 = (0)==($46|0);
   if ($47) {
    $48 = HEAP32[$db>>2]|0;
    $49 = HEAP32[$zOutput>>2]|0;
    $50 = ($49|0)!=(0|0);
    $51 = HEAP32[$zOutput>>2]|0;
    $52 = $50 ? $51 : 1216;
    $53 = HEAP32[$z>>2]|0;
    $54 = HEAP32[$zInput>>2]|0;
    $55 = $53;
    $56 = $54;
    $57 = (($55) - ($56))|0;
    $58 = HEAP32[$zInput>>2]|0;
    $59 = HEAP32[$zNew>>2]|0;
    HEAP32[$vararg_buffer5>>2] = $52;
    $vararg_ptr2 = (($vararg_buffer5) + 4|0);
    HEAP32[$vararg_ptr2>>2] = $57;
    $vararg_ptr3 = (($vararg_buffer5) + 8|0);
    HEAP32[$vararg_ptr3>>2] = $58;
    $vararg_ptr4 = (($vararg_buffer5) + 12|0);
    HEAP32[$vararg_ptr4>>2] = $59;
    $60 = (_sqlite3MPrintf($48,8976,$vararg_buffer5)|0);
    HEAP32[$zOut>>2] = $60;
    $61 = HEAP32[$db>>2]|0;
    $62 = HEAP32[$zOutput>>2]|0;
    _sqlite3DbFree($61,$62);
    $63 = HEAP32[$zOut>>2]|0;
    HEAP32[$zOutput>>2] = $63;
    $64 = HEAP32[$n>>2]|0;
    $65 = HEAP32[$z>>2]|0;
    $66 = (($65) + ($64)|0);
    HEAP32[$zInput>>2] = $66;
   }
   $67 = HEAP32[$db>>2]|0;
   $68 = HEAP32[$zParent>>2]|0;
   _sqlite3DbFree($67,$68);
  }
  $69 = HEAP32[$z>>2]|0;
  $70 = HEAP32[$n>>2]|0;
  $71 = (($69) + ($70)|0);
  HEAP32[$z>>2] = $71;
 }
 $72 = HEAP32[$db>>2]|0;
 $73 = HEAP32[$zOutput>>2]|0;
 $74 = ($73|0)!=(0|0);
 $75 = HEAP32[$zOutput>>2]|0;
 $76 = $74 ? $75 : 1216;
 $77 = HEAP32[$zInput>>2]|0;
 HEAP32[$vararg_buffer5>>2] = $76;
 $vararg_ptr8 = (($vararg_buffer5) + 4|0);
 HEAP32[$vararg_ptr8>>2] = $77;
 $78 = (_sqlite3MPrintf($72,8992,$vararg_buffer5)|0);
 HEAP32[$zResult>>2] = $78;
 $79 = HEAP32[$0>>2]|0;
 $80 = HEAP32[$zResult>>2]|0;
 _sqlite3_result_text($79,$80,-1,16);
 $81 = HEAP32[$db>>2]|0;
 $82 = HEAP32[$zOutput>>2]|0;
 _sqlite3DbFree($81,$82);
 STACKTOP = sp;return;
}
function _sqlite3GetToken($z,$tokenType) {
 $z = $z|0;
 $tokenType = $tokenType|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0;
 var $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0, $188 = 0;
 var $189 = 0, $19 = 0, $190 = 0, $191 = 0, $192 = 0, $193 = 0, $194 = 0, $195 = 0, $196 = 0, $197 = 0, $198 = 0, $199 = 0, $2 = 0, $20 = 0, $200 = 0, $201 = 0, $202 = 0, $203 = 0, $204 = 0, $205 = 0;
 var $206 = 0, $207 = 0, $208 = 0, $209 = 0, $21 = 0, $210 = 0, $211 = 0, $212 = 0, $213 = 0, $214 = 0, $215 = 0, $216 = 0, $217 = 0, $218 = 0, $219 = 0, $22 = 0, $220 = 0, $221 = 0, $222 = 0, $223 = 0;
 var $224 = 0, $225 = 0, $226 = 0, $227 = 0, $228 = 0, $229 = 0, $23 = 0, $230 = 0, $231 = 0, $232 = 0, $233 = 0, $234 = 0, $235 = 0, $236 = 0, $237 = 0, $238 = 0, $239 = 0, $24 = 0, $240 = 0, $241 = 0;
 var $242 = 0, $243 = 0, $244 = 0, $245 = 0, $246 = 0, $247 = 0, $248 = 0, $249 = 0, $25 = 0, $250 = 0, $251 = 0, $252 = 0, $253 = 0, $254 = 0, $255 = 0, $256 = 0, $257 = 0, $258 = 0, $259 = 0, $26 = 0;
 var $260 = 0, $261 = 0, $262 = 0, $263 = 0, $264 = 0, $265 = 0, $266 = 0, $267 = 0, $268 = 0, $269 = 0, $27 = 0, $270 = 0, $271 = 0, $272 = 0, $273 = 0, $274 = 0, $275 = 0, $276 = 0, $277 = 0, $278 = 0;
 var $279 = 0, $28 = 0, $280 = 0, $281 = 0, $282 = 0, $283 = 0, $284 = 0, $285 = 0, $286 = 0, $287 = 0, $288 = 0, $289 = 0, $29 = 0, $290 = 0, $291 = 0, $292 = 0, $293 = 0, $294 = 0, $295 = 0, $296 = 0;
 var $297 = 0, $298 = 0, $299 = 0, $3 = 0, $30 = 0, $300 = 0, $301 = 0, $302 = 0, $303 = 0, $304 = 0, $305 = 0, $306 = 0, $307 = 0, $308 = 0, $309 = 0, $31 = 0, $310 = 0, $311 = 0, $312 = 0, $313 = 0;
 var $314 = 0, $315 = 0, $316 = 0, $317 = 0, $318 = 0, $319 = 0, $32 = 0, $320 = 0, $321 = 0, $322 = 0, $323 = 0, $324 = 0, $325 = 0, $326 = 0, $327 = 0, $328 = 0, $329 = 0, $33 = 0, $330 = 0, $331 = 0;
 var $332 = 0, $333 = 0, $334 = 0, $335 = 0, $336 = 0, $337 = 0, $338 = 0, $339 = 0, $34 = 0, $340 = 0, $341 = 0, $342 = 0, $343 = 0, $344 = 0, $345 = 0, $346 = 0, $347 = 0, $348 = 0, $349 = 0, $35 = 0;
 var $350 = 0, $351 = 0, $352 = 0, $353 = 0, $354 = 0, $355 = 0, $356 = 0, $357 = 0, $358 = 0, $359 = 0, $36 = 0, $360 = 0, $361 = 0, $362 = 0, $363 = 0, $364 = 0, $365 = 0, $366 = 0, $367 = 0, $368 = 0;
 var $369 = 0, $37 = 0, $370 = 0, $371 = 0, $372 = 0, $373 = 0, $374 = 0, $375 = 0, $376 = 0, $377 = 0, $378 = 0, $379 = 0, $38 = 0, $380 = 0, $381 = 0, $382 = 0, $383 = 0, $384 = 0, $385 = 0, $386 = 0;
 var $387 = 0, $388 = 0, $389 = 0, $39 = 0, $390 = 0, $391 = 0, $392 = 0, $393 = 0, $394 = 0, $395 = 0, $396 = 0, $397 = 0, $398 = 0, $399 = 0, $4 = 0, $40 = 0, $400 = 0, $401 = 0, $402 = 0, $403 = 0;
 var $404 = 0, $405 = 0, $406 = 0, $407 = 0, $408 = 0, $409 = 0, $41 = 0, $410 = 0, $411 = 0, $412 = 0, $413 = 0, $414 = 0, $415 = 0, $416 = 0, $417 = 0, $418 = 0, $419 = 0, $42 = 0, $420 = 0, $421 = 0;
 var $422 = 0, $423 = 0, $424 = 0, $425 = 0, $426 = 0, $427 = 0, $428 = 0, $429 = 0, $43 = 0, $430 = 0, $431 = 0, $432 = 0, $433 = 0, $434 = 0, $435 = 0, $436 = 0, $437 = 0, $438 = 0, $439 = 0, $44 = 0;
 var $440 = 0, $441 = 0, $442 = 0, $443 = 0, $444 = 0, $445 = 0, $446 = 0, $447 = 0, $448 = 0, $449 = 0, $45 = 0, $450 = 0, $451 = 0, $452 = 0, $453 = 0, $454 = 0, $455 = 0, $456 = 0, $457 = 0, $458 = 0;
 var $459 = 0, $46 = 0, $460 = 0, $461 = 0, $462 = 0, $463 = 0, $464 = 0, $465 = 0, $466 = 0, $467 = 0, $468 = 0, $469 = 0, $47 = 0, $470 = 0, $471 = 0, $472 = 0, $473 = 0, $474 = 0, $475 = 0, $476 = 0;
 var $477 = 0, $478 = 0, $479 = 0, $48 = 0, $480 = 0, $481 = 0, $482 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0;
 var $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0;
 var $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0;
 var $97 = 0, $98 = 0, $99 = 0, $c = 0, $delim = 0, $i = 0, $n = 0, $or$cond = 0, $or$cond3 = 0, $or$cond7 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 24|0;
 $1 = sp + 20|0;
 $2 = sp + 16|0;
 $i = sp + 12|0;
 $c = sp + 8|0;
 $delim = sp + 4|0;
 $n = sp;
 HEAP32[$1>>2] = $z;
 HEAP32[$2>>2] = $tokenType;
 $3 = HEAP32[$1>>2]|0;
 $4 = HEAP8[$3>>0]|0;
 $5 = $4&255;
 do {
  switch ($5|0) {
  case 126:  {
   $124 = HEAP32[$2>>2]|0;
   HEAP32[$124>>2] = 96;
   HEAP32[$0>>2] = 1;
   $482 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($482|0);
   break;
  }
  case 60:  {
   $89 = HEAP32[$1>>2]|0;
   $90 = (($89) + 1|0);
   $91 = HEAP8[$90>>0]|0;
   $92 = $91&255;
   HEAP32[$c>>2] = $92;
   $93 = ($92|0)==(61);
   if ($93) {
    $94 = HEAP32[$2>>2]|0;
    HEAP32[$94>>2] = 81;
    HEAP32[$0>>2] = 2;
    $482 = HEAP32[$0>>2]|0;
    STACKTOP = sp;return ($482|0);
   }
   $95 = HEAP32[$c>>2]|0;
   $96 = ($95|0)==(62);
   if ($96) {
    $97 = HEAP32[$2>>2]|0;
    HEAP32[$97>>2] = 78;
    HEAP32[$0>>2] = 2;
    $482 = HEAP32[$0>>2]|0;
    STACKTOP = sp;return ($482|0);
   }
   $98 = HEAP32[$c>>2]|0;
   $99 = ($98|0)==(60);
   $100 = HEAP32[$2>>2]|0;
   if ($99) {
    HEAP32[$100>>2] = 87;
    HEAP32[$0>>2] = 2;
    $482 = HEAP32[$0>>2]|0;
    STACKTOP = sp;return ($482|0);
   } else {
    HEAP32[$100>>2] = 82;
    HEAP32[$0>>2] = 1;
    $482 = HEAP32[$0>>2]|0;
    STACKTOP = sp;return ($482|0);
   }
   break;
  }
  case 45:  {
   $20 = HEAP32[$1>>2]|0;
   $21 = (($20) + 1|0);
   $22 = HEAP8[$21>>0]|0;
   $23 = $22&255;
   $24 = ($23|0)==(45);
   if (!($24)) {
    $37 = HEAP32[$2>>2]|0;
    HEAP32[$37>>2] = 90;
    HEAP32[$0>>2] = 1;
    $482 = HEAP32[$0>>2]|0;
    STACKTOP = sp;return ($482|0);
   }
   HEAP32[$i>>2] = 2;
   while(1) {
    $25 = HEAP32[$i>>2]|0;
    $26 = HEAP32[$1>>2]|0;
    $27 = (($26) + ($25)|0);
    $28 = HEAP8[$27>>0]|0;
    $29 = $28&255;
    HEAP32[$c>>2] = $29;
    $30 = ($29|0)!=(0);
    $31 = HEAP32[$c>>2]|0;
    $32 = ($31|0)!=(10);
    $or$cond = $30 & $32;
    if (!($or$cond)) {
     break;
    }
    $33 = HEAP32[$i>>2]|0;
    $34 = (($33) + 1)|0;
    HEAP32[$i>>2] = $34;
   }
   $35 = HEAP32[$2>>2]|0;
   HEAP32[$35>>2] = 151;
   $36 = HEAP32[$i>>2]|0;
   HEAP32[$0>>2] = $36;
   $482 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($482|0);
   break;
  }
  case 124:  {
   $116 = HEAP32[$1>>2]|0;
   $117 = (($116) + 1|0);
   $118 = HEAP8[$117>>0]|0;
   $119 = $118&255;
   $120 = ($119|0)!=(124);
   $121 = HEAP32[$2>>2]|0;
   if ($120) {
    HEAP32[$121>>2] = 86;
    HEAP32[$0>>2] = 1;
    $482 = HEAP32[$0>>2]|0;
    STACKTOP = sp;return ($482|0);
   } else {
    HEAP32[$121>>2] = 94;
    HEAP32[$0>>2] = 2;
    $482 = HEAP32[$0>>2]|0;
    STACKTOP = sp;return ($482|0);
   }
   break;
  }
  case 37:  {
   $80 = HEAP32[$2>>2]|0;
   HEAP32[$80>>2] = 93;
   HEAP32[$0>>2] = 1;
   $482 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($482|0);
   break;
  }
  case 46:  {
   $160 = HEAP32[$1>>2]|0;
   $161 = (($160) + 1|0);
   $162 = HEAP8[$161>>0]|0;
   $163 = $162&255;
   $164 = (1224 + ($163)|0);
   $165 = HEAP8[$164>>0]|0;
   $166 = $165&255;
   $167 = $166 & 4;
   $168 = ($167|0)!=(0);
   if (!($168)) {
    $169 = HEAP32[$2>>2]|0;
    HEAP32[$169>>2] = 122;
    HEAP32[$0>>2] = 1;
    $482 = HEAP32[$0>>2]|0;
    STACKTOP = sp;return ($482|0);
   }
   break;
  }
  case 91:  {
   HEAP32[$i>>2] = 1;
   $315 = HEAP32[$1>>2]|0;
   $316 = HEAP8[$315>>0]|0;
   $317 = $316&255;
   HEAP32[$c>>2] = $317;
   while(1) {
    $318 = HEAP32[$c>>2]|0;
    $319 = ($318|0)!=(93);
    if (!($319)) {
     break;
    }
    $320 = HEAP32[$i>>2]|0;
    $321 = HEAP32[$1>>2]|0;
    $322 = (($321) + ($320)|0);
    $323 = HEAP8[$322>>0]|0;
    $324 = $323&255;
    HEAP32[$c>>2] = $324;
    $325 = ($324|0)!=(0);
    if (!($325)) {
     break;
    }
    $326 = HEAP32[$i>>2]|0;
    $327 = (($326) + 1)|0;
    HEAP32[$i>>2] = $327;
   }
   $328 = HEAP32[$c>>2]|0;
   $329 = ($328|0)==(93);
   $330 = $329 ? 27 : 150;
   $331 = HEAP32[$2>>2]|0;
   HEAP32[$331>>2] = $330;
   $332 = HEAP32[$i>>2]|0;
   HEAP32[$0>>2] = $332;
   $482 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($482|0);
   break;
  }
  case 59:  {
   $40 = HEAP32[$2>>2]|0;
   HEAP32[$40>>2] = 1;
   HEAP32[$0>>2] = 1;
   $482 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($482|0);
   break;
  }
  case 57: case 56: case 55: case 54: case 53: case 52: case 51: case 50: case 49: case 48:  {
   break;
  }
  case 44:  {
   $122 = HEAP32[$2>>2]|0;
   HEAP32[$122>>2] = 26;
   HEAP32[$0>>2] = 1;
   $482 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($482|0);
   break;
  }
  case 38:  {
   $123 = HEAP32[$2>>2]|0;
   HEAP32[$123>>2] = 85;
   HEAP32[$0>>2] = 1;
   $482 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($482|0);
   break;
  }
  case 13: case 12: case 10: case 9: case 32:  {
   HEAP32[$i>>2] = 1;
   while(1) {
    $6 = HEAP32[$i>>2]|0;
    $7 = HEAP32[$1>>2]|0;
    $8 = (($7) + ($6)|0);
    $9 = HEAP8[$8>>0]|0;
    $10 = $9&255;
    $11 = (1224 + ($10)|0);
    $12 = HEAP8[$11>>0]|0;
    $13 = $12&255;
    $14 = $13 & 1;
    $15 = ($14|0)!=(0);
    if (!($15)) {
     break;
    }
    $16 = HEAP32[$i>>2]|0;
    $17 = (($16) + 1)|0;
    HEAP32[$i>>2] = $17;
   }
   $18 = HEAP32[$2>>2]|0;
   HEAP32[$18>>2] = 151;
   $19 = HEAP32[$i>>2]|0;
   HEAP32[$0>>2] = $19;
   $482 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($482|0);
   break;
  }
  case 58: case 35: case 64: case 36:  {
   HEAP32[$n>>2] = 0;
   $346 = HEAP32[$2>>2]|0;
   HEAP32[$346>>2] = 135;
   HEAP32[$i>>2] = 1;
   while(1) {
    $347 = HEAP32[$i>>2]|0;
    $348 = HEAP32[$1>>2]|0;
    $349 = (($348) + ($347)|0);
    $350 = HEAP8[$349>>0]|0;
    $351 = $350&255;
    HEAP32[$c>>2] = $351;
    $352 = ($351|0)!=(0);
    if (!($352)) {
     break;
    }
    $353 = HEAP32[$c>>2]|0;
    $354 = $353&255;
    $355 = $354&255;
    $356 = (1224 + ($355)|0);
    $357 = HEAP8[$356>>0]|0;
    $358 = $357&255;
    $359 = $358 & 70;
    $360 = ($359|0)!=(0);
    if ($360) {
     $361 = HEAP32[$n>>2]|0;
     $362 = (($361) + 1)|0;
     HEAP32[$n>>2] = $362;
    } else {
     $363 = HEAP32[$c>>2]|0;
     $364 = ($363|0)==(40);
     $365 = HEAP32[$n>>2]|0;
     $366 = ($365|0)>(0);
     $or$cond3 = $364 & $366;
     if ($or$cond3) {
      label = 107;
      break;
     }
     $390 = HEAP32[$c>>2]|0;
     $391 = ($390|0)==(58);
     if (!($391)) {
      break;
     }
     $392 = HEAP32[$i>>2]|0;
     $393 = (($392) + 1)|0;
     $394 = HEAP32[$1>>2]|0;
     $395 = (($394) + ($393)|0);
     $396 = HEAP8[$395>>0]|0;
     $397 = $396&255;
     $398 = ($397|0)==(58);
     if (!($398)) {
      break;
     }
     $399 = HEAP32[$i>>2]|0;
     $400 = (($399) + 1)|0;
     HEAP32[$i>>2] = $400;
    }
    $401 = HEAP32[$i>>2]|0;
    $402 = (($401) + 1)|0;
    HEAP32[$i>>2] = $402;
   }
   do {
    if ((label|0) == 107) {
     while(1) {
      label = 0;
      $367 = HEAP32[$i>>2]|0;
      $368 = (($367) + 1)|0;
      HEAP32[$i>>2] = $368;
      $369 = HEAP32[$i>>2]|0;
      $370 = HEAP32[$1>>2]|0;
      $371 = (($370) + ($369)|0);
      $372 = HEAP8[$371>>0]|0;
      $373 = $372&255;
      HEAP32[$c>>2] = $373;
      $374 = ($373|0)!=(0);
      if (!($374)) {
       break;
      }
      $375 = HEAP32[$c>>2]|0;
      $376 = $375&255;
      $377 = $376&255;
      $378 = (1224 + ($377)|0);
      $379 = HEAP8[$378>>0]|0;
      $380 = $379&255;
      $381 = $380 & 1;
      $382 = ($381|0)==(0);
      $383 = HEAP32[$c>>2]|0;
      $384 = ($383|0)!=(41);
      $or$cond7 = $382 & $384;
      if ($or$cond7) {
       label = 107;
      } else {
       break;
      }
     }
     $385 = HEAP32[$c>>2]|0;
     $386 = ($385|0)==(41);
     if ($386) {
      $387 = HEAP32[$i>>2]|0;
      $388 = (($387) + 1)|0;
      HEAP32[$i>>2] = $388;
      break;
     } else {
      $389 = HEAP32[$2>>2]|0;
      HEAP32[$389>>2] = 150;
      break;
     }
    }
   } while(0);
   $403 = HEAP32[$n>>2]|0;
   $404 = ($403|0)==(0);
   if ($404) {
    $405 = HEAP32[$2>>2]|0;
    HEAP32[$405>>2] = 150;
   }
   $406 = HEAP32[$i>>2]|0;
   HEAP32[$0>>2] = $406;
   $482 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($482|0);
   break;
  }
  case 34: case 39: case 96:  {
   $125 = HEAP32[$1>>2]|0;
   $126 = HEAP8[$125>>0]|0;
   $127 = $126&255;
   HEAP32[$delim>>2] = $127;
   HEAP32[$i>>2] = 1;
   while(1) {
    $128 = HEAP32[$i>>2]|0;
    $129 = HEAP32[$1>>2]|0;
    $130 = (($129) + ($128)|0);
    $131 = HEAP8[$130>>0]|0;
    $132 = $131&255;
    HEAP32[$c>>2] = $132;
    $133 = ($132|0)!=(0);
    if (!($133)) {
     break;
    }
    $134 = HEAP32[$c>>2]|0;
    $135 = HEAP32[$delim>>2]|0;
    $136 = ($134|0)==($135|0);
    if ($136) {
     $137 = HEAP32[$i>>2]|0;
     $138 = (($137) + 1)|0;
     $139 = HEAP32[$1>>2]|0;
     $140 = (($139) + ($138)|0);
     $141 = HEAP8[$140>>0]|0;
     $142 = $141&255;
     $143 = HEAP32[$delim>>2]|0;
     $144 = ($142|0)==($143|0);
     if (!($144)) {
      break;
     }
     $145 = HEAP32[$i>>2]|0;
     $146 = (($145) + 1)|0;
     HEAP32[$i>>2] = $146;
    }
    $147 = HEAP32[$i>>2]|0;
    $148 = (($147) + 1)|0;
    HEAP32[$i>>2] = $148;
   }
   $149 = HEAP32[$c>>2]|0;
   $150 = ($149|0)==(39);
   if ($150) {
    $151 = HEAP32[$2>>2]|0;
    HEAP32[$151>>2] = 97;
    $152 = HEAP32[$i>>2]|0;
    $153 = (($152) + 1)|0;
    HEAP32[$0>>2] = $153;
    $482 = HEAP32[$0>>2]|0;
    STACKTOP = sp;return ($482|0);
   }
   $154 = HEAP32[$c>>2]|0;
   $155 = ($154|0)!=(0);
   $156 = HEAP32[$2>>2]|0;
   if ($155) {
    HEAP32[$156>>2] = 27;
    $157 = HEAP32[$i>>2]|0;
    $158 = (($157) + 1)|0;
    HEAP32[$0>>2] = $158;
    $482 = HEAP32[$0>>2]|0;
    STACKTOP = sp;return ($482|0);
   } else {
    HEAP32[$156>>2] = 150;
    $159 = HEAP32[$i>>2]|0;
    HEAP32[$0>>2] = $159;
    $482 = HEAP32[$0>>2]|0;
    STACKTOP = sp;return ($482|0);
   }
   break;
  }
  case 88: case 120:  {
   $407 = HEAP32[$1>>2]|0;
   $408 = (($407) + 1|0);
   $409 = HEAP8[$408>>0]|0;
   $410 = $409&255;
   $411 = ($410|0)==(39);
   if ($411) {
    $412 = HEAP32[$2>>2]|0;
    HEAP32[$412>>2] = 134;
    HEAP32[$i>>2] = 2;
    while(1) {
     $413 = HEAP32[$i>>2]|0;
     $414 = HEAP32[$1>>2]|0;
     $415 = (($414) + ($413)|0);
     $416 = HEAP8[$415>>0]|0;
     $417 = $416&255;
     $418 = (1224 + ($417)|0);
     $419 = HEAP8[$418>>0]|0;
     $420 = $419&255;
     $421 = $420 & 8;
     $422 = ($421|0)!=(0);
     $423 = HEAP32[$i>>2]|0;
     if (!($422)) {
      break;
     }
     $424 = (($423) + 1)|0;
     HEAP32[$i>>2] = $424;
    }
    $425 = HEAP32[$1>>2]|0;
    $426 = (($425) + ($423)|0);
    $427 = HEAP8[$426>>0]|0;
    $428 = $427&255;
    $429 = ($428|0)!=(39);
    if ($429) {
     label = 125;
    } else {
     $430 = HEAP32[$i>>2]|0;
     $431 = (($430|0) % 2)&-1;
     $432 = ($431|0)!=(0);
     if ($432) {
      label = 125;
     }
    }
    L108: do {
     if ((label|0) == 125) {
      $433 = HEAP32[$2>>2]|0;
      HEAP32[$433>>2] = 150;
      while(1) {
       $434 = HEAP32[$i>>2]|0;
       $435 = HEAP32[$1>>2]|0;
       $436 = (($435) + ($434)|0);
       $437 = HEAP8[$436>>0]|0;
       $438 = $437&255;
       $439 = ($438|0)!=(0);
       if (!($439)) {
        break L108;
       }
       $440 = HEAP32[$i>>2]|0;
       $441 = HEAP32[$1>>2]|0;
       $442 = (($441) + ($440)|0);
       $443 = HEAP8[$442>>0]|0;
       $444 = $443&255;
       $445 = ($444|0)!=(39);
       if (!($445)) {
        break L108;
       }
       $446 = HEAP32[$i>>2]|0;
       $447 = (($446) + 1)|0;
       HEAP32[$i>>2] = $447;
      }
     }
    } while(0);
    $448 = HEAP32[$i>>2]|0;
    $449 = HEAP32[$1>>2]|0;
    $450 = (($449) + ($448)|0);
    $451 = HEAP8[$450>>0]|0;
    $452 = ($451<<24>>24)!=(0);
    if ($452) {
     $453 = HEAP32[$i>>2]|0;
     $454 = (($453) + 1)|0;
     HEAP32[$i>>2] = $454;
    }
    $455 = HEAP32[$i>>2]|0;
    HEAP32[$0>>2] = $455;
    $482 = HEAP32[$0>>2]|0;
    STACKTOP = sp;return ($482|0);
   } else {
    label = 132;
   }
   break;
  }
  case 61:  {
   $81 = HEAP32[$2>>2]|0;
   HEAP32[$81>>2] = 79;
   $82 = HEAP32[$1>>2]|0;
   $83 = (($82) + 1|0);
   $84 = HEAP8[$83>>0]|0;
   $85 = $84&255;
   $86 = ($85|0)==(61);
   $87 = $86&1;
   $88 = (1 + ($87))|0;
   HEAP32[$0>>2] = $88;
   $482 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($482|0);
   break;
  }
  case 41:  {
   $39 = HEAP32[$2>>2]|0;
   HEAP32[$39>>2] = 23;
   HEAP32[$0>>2] = 1;
   $482 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($482|0);
   break;
  }
  case 63:  {
   $333 = HEAP32[$2>>2]|0;
   HEAP32[$333>>2] = 135;
   HEAP32[$i>>2] = 1;
   while(1) {
    $334 = HEAP32[$i>>2]|0;
    $335 = HEAP32[$1>>2]|0;
    $336 = (($335) + ($334)|0);
    $337 = HEAP8[$336>>0]|0;
    $338 = $337&255;
    $339 = (1224 + ($338)|0);
    $340 = HEAP8[$339>>0]|0;
    $341 = $340&255;
    $342 = $341 & 4;
    $343 = ($342|0)!=(0);
    $344 = HEAP32[$i>>2]|0;
    if (!($343)) {
     break;
    }
    $345 = (($344) + 1)|0;
    HEAP32[$i>>2] = $345;
   }
   HEAP32[$0>>2] = $344;
   $482 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($482|0);
   break;
  }
  case 33:  {
   $110 = HEAP32[$1>>2]|0;
   $111 = (($110) + 1|0);
   $112 = HEAP8[$111>>0]|0;
   $113 = $112&255;
   $114 = ($113|0)!=(61);
   $115 = HEAP32[$2>>2]|0;
   if ($114) {
    HEAP32[$115>>2] = 150;
    HEAP32[$0>>2] = 2;
    $482 = HEAP32[$0>>2]|0;
    STACKTOP = sp;return ($482|0);
   } else {
    HEAP32[$115>>2] = 78;
    HEAP32[$0>>2] = 2;
    $482 = HEAP32[$0>>2]|0;
    STACKTOP = sp;return ($482|0);
   }
   break;
  }
  case 62:  {
   $101 = HEAP32[$1>>2]|0;
   $102 = (($101) + 1|0);
   $103 = HEAP8[$102>>0]|0;
   $104 = $103&255;
   HEAP32[$c>>2] = $104;
   $105 = ($104|0)==(61);
   if ($105) {
    $106 = HEAP32[$2>>2]|0;
    HEAP32[$106>>2] = 83;
    HEAP32[$0>>2] = 2;
    $482 = HEAP32[$0>>2]|0;
    STACKTOP = sp;return ($482|0);
   }
   $107 = HEAP32[$c>>2]|0;
   $108 = ($107|0)==(62);
   $109 = HEAP32[$2>>2]|0;
   if ($108) {
    HEAP32[$109>>2] = 88;
    HEAP32[$0>>2] = 2;
    $482 = HEAP32[$0>>2]|0;
    STACKTOP = sp;return ($482|0);
   } else {
    HEAP32[$109>>2] = 80;
    HEAP32[$0>>2] = 1;
    $482 = HEAP32[$0>>2]|0;
    STACKTOP = sp;return ($482|0);
   }
   break;
  }
  case 47:  {
   $43 = HEAP32[$1>>2]|0;
   $44 = (($43) + 1|0);
   $45 = HEAP8[$44>>0]|0;
   $46 = $45&255;
   $47 = ($46|0)!=(42);
   if (!($47)) {
    $48 = HEAP32[$1>>2]|0;
    $49 = (($48) + 2|0);
    $50 = HEAP8[$49>>0]|0;
    $51 = $50&255;
    $52 = ($51|0)==(0);
    if (!($52)) {
     HEAP32[$i>>2] = 3;
     $54 = HEAP32[$1>>2]|0;
     $55 = (($54) + 2|0);
     $56 = HEAP8[$55>>0]|0;
     $57 = $56&255;
     HEAP32[$c>>2] = $57;
     while(1) {
      $58 = HEAP32[$c>>2]|0;
      $59 = ($58|0)!=(42);
      if (!($59)) {
       $60 = HEAP32[$i>>2]|0;
       $61 = HEAP32[$1>>2]|0;
       $62 = (($61) + ($60)|0);
       $63 = HEAP8[$62>>0]|0;
       $64 = $63&255;
       $65 = ($64|0)!=(47);
       if (!($65)) {
        break;
       }
      }
      $66 = HEAP32[$i>>2]|0;
      $67 = HEAP32[$1>>2]|0;
      $68 = (($67) + ($66)|0);
      $69 = HEAP8[$68>>0]|0;
      $70 = $69&255;
      HEAP32[$c>>2] = $70;
      $71 = ($70|0)!=(0);
      if (!($71)) {
       break;
      }
      $72 = HEAP32[$i>>2]|0;
      $73 = (($72) + 1)|0;
      HEAP32[$i>>2] = $73;
     }
     $74 = HEAP32[$c>>2]|0;
     $75 = ($74|0)!=(0);
     if ($75) {
      $76 = HEAP32[$i>>2]|0;
      $77 = (($76) + 1)|0;
      HEAP32[$i>>2] = $77;
     }
     $78 = HEAP32[$2>>2]|0;
     HEAP32[$78>>2] = 151;
     $79 = HEAP32[$i>>2]|0;
     HEAP32[$0>>2] = $79;
     $482 = HEAP32[$0>>2]|0;
     STACKTOP = sp;return ($482|0);
    }
   }
   $53 = HEAP32[$2>>2]|0;
   HEAP32[$53>>2] = 92;
   HEAP32[$0>>2] = 1;
   $482 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($482|0);
   break;
  }
  case 43:  {
   $41 = HEAP32[$2>>2]|0;
   HEAP32[$41>>2] = 89;
   HEAP32[$0>>2] = 1;
   $482 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($482|0);
   break;
  }
  case 40:  {
   $38 = HEAP32[$2>>2]|0;
   HEAP32[$38>>2] = 22;
   HEAP32[$0>>2] = 1;
   $482 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($482|0);
   break;
  }
  case 42:  {
   $42 = HEAP32[$2>>2]|0;
   HEAP32[$42>>2] = 91;
   HEAP32[$0>>2] = 1;
   $482 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($482|0);
   break;
  }
  default: {
   label = 132;
  }
  }
 } while(0);
 if ((label|0) == 132) {
  $456 = HEAP32[$1>>2]|0;
  $457 = HEAP8[$456>>0]|0;
  $458 = $457&255;
  $459 = (1224 + ($458)|0);
  $460 = HEAP8[$459>>0]|0;
  $461 = $460&255;
  $462 = $461 & 70;
  $463 = ($462|0)!=(0);
  if (!($463)) {
   $481 = HEAP32[$2>>2]|0;
   HEAP32[$481>>2] = 150;
   HEAP32[$0>>2] = 1;
   $482 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($482|0);
  }
  HEAP32[$i>>2] = 1;
  while(1) {
   $464 = HEAP32[$i>>2]|0;
   $465 = HEAP32[$1>>2]|0;
   $466 = (($465) + ($464)|0);
   $467 = HEAP8[$466>>0]|0;
   $468 = $467&255;
   $469 = (1224 + ($468)|0);
   $470 = HEAP8[$469>>0]|0;
   $471 = $470&255;
   $472 = $471 & 70;
   $473 = ($472|0)!=(0);
   if (!($473)) {
    break;
   }
   $474 = HEAP32[$i>>2]|0;
   $475 = (($474) + 1)|0;
   HEAP32[$i>>2] = $475;
  }
  $476 = HEAP32[$1>>2]|0;
  $477 = HEAP32[$i>>2]|0;
  $478 = (_keywordCode($476,$477)|0);
  $479 = HEAP32[$2>>2]|0;
  HEAP32[$479>>2] = $478;
  $480 = HEAP32[$i>>2]|0;
  HEAP32[$0>>2] = $480;
  $482 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($482|0);
 }
 $170 = HEAP32[$2>>2]|0;
 HEAP32[$170>>2] = 132;
 $171 = HEAP32[$1>>2]|0;
 $172 = HEAP8[$171>>0]|0;
 $173 = $172&255;
 $174 = ($173|0)==(48);
 do {
  if ($174) {
   $175 = HEAP32[$1>>2]|0;
   $176 = (($175) + 1|0);
   $177 = HEAP8[$176>>0]|0;
   $178 = $177&255;
   $179 = ($178|0)==(120);
   if (!($179)) {
    $180 = HEAP32[$1>>2]|0;
    $181 = (($180) + 1|0);
    $182 = HEAP8[$181>>0]|0;
    $183 = $182&255;
    $184 = ($183|0)==(88);
    if (!($184)) {
     break;
    }
   }
   $185 = HEAP32[$1>>2]|0;
   $186 = (($185) + 2|0);
   $187 = HEAP8[$186>>0]|0;
   $188 = $187&255;
   $189 = (1224 + ($188)|0);
   $190 = HEAP8[$189>>0]|0;
   $191 = $190&255;
   $192 = $191 & 8;
   $193 = ($192|0)!=(0);
   if ($193) {
    HEAP32[$i>>2] = 3;
    while(1) {
     $194 = HEAP32[$i>>2]|0;
     $195 = HEAP32[$1>>2]|0;
     $196 = (($195) + ($194)|0);
     $197 = HEAP8[$196>>0]|0;
     $198 = $197&255;
     $199 = (1224 + ($198)|0);
     $200 = HEAP8[$199>>0]|0;
     $201 = $200&255;
     $202 = $201 & 8;
     $203 = ($202|0)!=(0);
     $204 = HEAP32[$i>>2]|0;
     if (!($203)) {
      break;
     }
     $205 = (($204) + 1)|0;
     HEAP32[$i>>2] = $205;
    }
    HEAP32[$0>>2] = $204;
    $482 = HEAP32[$0>>2]|0;
    STACKTOP = sp;return ($482|0);
   }
  }
 } while(0);
 HEAP32[$i>>2] = 0;
 while(1) {
  $206 = HEAP32[$i>>2]|0;
  $207 = HEAP32[$1>>2]|0;
  $208 = (($207) + ($206)|0);
  $209 = HEAP8[$208>>0]|0;
  $210 = $209&255;
  $211 = (1224 + ($210)|0);
  $212 = HEAP8[$211>>0]|0;
  $213 = $212&255;
  $214 = $213 & 4;
  $215 = ($214|0)!=(0);
  $216 = HEAP32[$i>>2]|0;
  if (!($215)) {
   break;
  }
  $217 = (($216) + 1)|0;
  HEAP32[$i>>2] = $217;
 }
 $218 = HEAP32[$1>>2]|0;
 $219 = (($218) + ($216)|0);
 $220 = HEAP8[$219>>0]|0;
 $221 = $220&255;
 $222 = ($221|0)==(46);
 if ($222) {
  $223 = HEAP32[$i>>2]|0;
  $224 = (($223) + 1)|0;
  HEAP32[$i>>2] = $224;
  while(1) {
   $225 = HEAP32[$i>>2]|0;
   $226 = HEAP32[$1>>2]|0;
   $227 = (($226) + ($225)|0);
   $228 = HEAP8[$227>>0]|0;
   $229 = $228&255;
   $230 = (1224 + ($229)|0);
   $231 = HEAP8[$230>>0]|0;
   $232 = $231&255;
   $233 = $232 & 4;
   $234 = ($233|0)!=(0);
   if (!($234)) {
    break;
   }
   $235 = HEAP32[$i>>2]|0;
   $236 = (($235) + 1)|0;
   HEAP32[$i>>2] = $236;
  }
  $237 = HEAP32[$2>>2]|0;
  HEAP32[$237>>2] = 133;
 }
 $238 = HEAP32[$i>>2]|0;
 $239 = HEAP32[$1>>2]|0;
 $240 = (($239) + ($238)|0);
 $241 = HEAP8[$240>>0]|0;
 $242 = $241&255;
 $243 = ($242|0)==(101);
 if ($243) {
  label = 82;
 } else {
  $244 = HEAP32[$i>>2]|0;
  $245 = HEAP32[$1>>2]|0;
  $246 = (($245) + ($244)|0);
  $247 = HEAP8[$246>>0]|0;
  $248 = $247&255;
  $249 = ($248|0)==(69);
  if ($249) {
   label = 82;
  }
 }
 do {
  if ((label|0) == 82) {
   $250 = HEAP32[$i>>2]|0;
   $251 = (($250) + 1)|0;
   $252 = HEAP32[$1>>2]|0;
   $253 = (($252) + ($251)|0);
   $254 = HEAP8[$253>>0]|0;
   $255 = $254&255;
   $256 = (1224 + ($255)|0);
   $257 = HEAP8[$256>>0]|0;
   $258 = $257&255;
   $259 = $258 & 4;
   $260 = ($259|0)!=(0);
   if (!($260)) {
    $261 = HEAP32[$i>>2]|0;
    $262 = (($261) + 1)|0;
    $263 = HEAP32[$1>>2]|0;
    $264 = (($263) + ($262)|0);
    $265 = HEAP8[$264>>0]|0;
    $266 = $265&255;
    $267 = ($266|0)==(43);
    if (!($267)) {
     $268 = HEAP32[$i>>2]|0;
     $269 = (($268) + 1)|0;
     $270 = HEAP32[$1>>2]|0;
     $271 = (($270) + ($269)|0);
     $272 = HEAP8[$271>>0]|0;
     $273 = $272&255;
     $274 = ($273|0)==(45);
     if (!($274)) {
      break;
     }
    }
    $275 = HEAP32[$i>>2]|0;
    $276 = (($275) + 2)|0;
    $277 = HEAP32[$1>>2]|0;
    $278 = (($277) + ($276)|0);
    $279 = HEAP8[$278>>0]|0;
    $280 = $279&255;
    $281 = (1224 + ($280)|0);
    $282 = HEAP8[$281>>0]|0;
    $283 = $282&255;
    $284 = $283 & 4;
    $285 = ($284|0)!=(0);
    if (!($285)) {
     break;
    }
   }
   $286 = HEAP32[$i>>2]|0;
   $287 = (($286) + 2)|0;
   HEAP32[$i>>2] = $287;
   while(1) {
    $288 = HEAP32[$i>>2]|0;
    $289 = HEAP32[$1>>2]|0;
    $290 = (($289) + ($288)|0);
    $291 = HEAP8[$290>>0]|0;
    $292 = $291&255;
    $293 = (1224 + ($292)|0);
    $294 = HEAP8[$293>>0]|0;
    $295 = $294&255;
    $296 = $295 & 4;
    $297 = ($296|0)!=(0);
    if (!($297)) {
     break;
    }
    $298 = HEAP32[$i>>2]|0;
    $299 = (($298) + 1)|0;
    HEAP32[$i>>2] = $299;
   }
   $300 = HEAP32[$2>>2]|0;
   HEAP32[$300>>2] = 133;
  }
 } while(0);
 while(1) {
  $301 = HEAP32[$i>>2]|0;
  $302 = HEAP32[$1>>2]|0;
  $303 = (($302) + ($301)|0);
  $304 = HEAP8[$303>>0]|0;
  $305 = $304&255;
  $306 = (1224 + ($305)|0);
  $307 = HEAP8[$306>>0]|0;
  $308 = $307&255;
  $309 = $308 & 70;
  $310 = ($309|0)!=(0);
  if (!($310)) {
   break;
  }
  $311 = HEAP32[$2>>2]|0;
  HEAP32[$311>>2] = 150;
  $312 = HEAP32[$i>>2]|0;
  $313 = (($312) + 1)|0;
  HEAP32[$i>>2] = $313;
 }
 $314 = HEAP32[$i>>2]|0;
 HEAP32[$0>>2] = $314;
 $482 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($482|0);
}
function _sqlite3DbStrNDup($db,$z,$0,$1) {
 $db = $db|0;
 $z = $z|0;
 $0 = $0|0;
 $1 = $1|0;
 var $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0;
 var $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0;
 var $zNew = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $2 = sp + 20|0;
 $3 = sp + 16|0;
 $4 = sp + 12|0;
 $5 = sp;
 $zNew = sp + 8|0;
 HEAP32[$3>>2] = $db;
 HEAP32[$4>>2] = $z;
 $6 = $5;
 $7 = $6;
 HEAP32[$7>>2] = $0;
 $8 = (($6) + 4)|0;
 $9 = $8;
 HEAP32[$9>>2] = $1;
 $10 = HEAP32[$4>>2]|0;
 $11 = ($10|0)==(0|0);
 if ($11) {
  HEAP32[$2>>2] = 0;
  $41 = HEAP32[$2>>2]|0;
  STACKTOP = sp;return ($41|0);
 }
 $12 = HEAP32[$3>>2]|0;
 $13 = $5;
 $14 = $13;
 $15 = HEAP32[$14>>2]|0;
 $16 = (($13) + 4)|0;
 $17 = $16;
 $18 = HEAP32[$17>>2]|0;
 $19 = (_i64Add(($15|0),($18|0),1,0)|0);
 $20 = tempRet0;
 $21 = (_sqlite3DbMallocRaw($12,$19,$20)|0);
 HEAP32[$zNew>>2] = $21;
 $22 = HEAP32[$zNew>>2]|0;
 $23 = ($22|0)!=(0|0);
 if ($23) {
  $24 = HEAP32[$zNew>>2]|0;
  $25 = HEAP32[$4>>2]|0;
  $26 = $5;
  $27 = $26;
  $28 = HEAP32[$27>>2]|0;
  $29 = (($26) + 4)|0;
  $30 = $29;
  $31 = HEAP32[$30>>2]|0;
  _memcpy(($24|0),($25|0),($28|0))|0;
  $32 = $5;
  $33 = $32;
  $34 = HEAP32[$33>>2]|0;
  $35 = (($32) + 4)|0;
  $36 = $35;
  $37 = HEAP32[$36>>2]|0;
  $38 = HEAP32[$zNew>>2]|0;
  $39 = (($38) + ($34)|0);
  HEAP8[$39>>0] = 0;
 }
 $40 = HEAP32[$zNew>>2]|0;
 HEAP32[$2>>2] = $40;
 $41 = HEAP32[$2>>2]|0;
 STACKTOP = sp;return ($41|0);
}
function _sqlite3Dequote($z) {
 $z = $z|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $i = 0, $j = 0, $quote = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 12|0;
 $1 = sp + 8|0;
 $quote = sp + 16|0;
 $i = sp + 4|0;
 $j = sp;
 HEAP32[$1>>2] = $z;
 $2 = HEAP32[$1>>2]|0;
 $3 = ($2|0)==(0|0);
 if ($3) {
  HEAP32[$0>>2] = -1;
  $45 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($45|0);
 }
 $4 = HEAP32[$1>>2]|0;
 $5 = HEAP8[$4>>0]|0;
 HEAP8[$quote>>0] = $5;
 $6 = HEAP8[$quote>>0]|0;
 $7 = $6 << 24 >> 24;
 if ((($7|0) == 91)) {
  HEAP8[$quote>>0] = 93;
 } else if (!((($7|0) == 96) | (($7|0) == 34) | (($7|0) == 39))) {
  HEAP32[$0>>2] = -1;
  $45 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($45|0);
 }
 HEAP32[$i>>2] = 1;
 HEAP32[$j>>2] = 0;
 while(1) {
  $8 = HEAP32[$i>>2]|0;
  $9 = HEAP32[$1>>2]|0;
  $10 = (($9) + ($8)|0);
  $11 = HEAP8[$10>>0]|0;
  $12 = $11 << 24 >> 24;
  $13 = HEAP8[$quote>>0]|0;
  $14 = $13 << 24 >> 24;
  $15 = ($12|0)==($14|0);
  $16 = HEAP32[$i>>2]|0;
  if ($15) {
   $17 = (($16) + 1)|0;
   $18 = HEAP32[$1>>2]|0;
   $19 = (($18) + ($17)|0);
   $20 = HEAP8[$19>>0]|0;
   $21 = $20 << 24 >> 24;
   $22 = HEAP8[$quote>>0]|0;
   $23 = $22 << 24 >> 24;
   $24 = ($21|0)==($23|0);
   if (!($24)) {
    break;
   }
   $25 = HEAP8[$quote>>0]|0;
   $26 = HEAP32[$j>>2]|0;
   $27 = (($26) + 1)|0;
   HEAP32[$j>>2] = $27;
   $28 = HEAP32[$1>>2]|0;
   $29 = (($28) + ($26)|0);
   HEAP8[$29>>0] = $25;
   $30 = HEAP32[$i>>2]|0;
   $31 = (($30) + 1)|0;
   HEAP32[$i>>2] = $31;
  } else {
   $32 = HEAP32[$1>>2]|0;
   $33 = (($32) + ($16)|0);
   $34 = HEAP8[$33>>0]|0;
   $35 = HEAP32[$j>>2]|0;
   $36 = (($35) + 1)|0;
   HEAP32[$j>>2] = $36;
   $37 = HEAP32[$1>>2]|0;
   $38 = (($37) + ($35)|0);
   HEAP8[$38>>0] = $34;
  }
  $39 = HEAP32[$i>>2]|0;
  $40 = (($39) + 1)|0;
  HEAP32[$i>>2] = $40;
 }
 $41 = HEAP32[$j>>2]|0;
 $42 = HEAP32[$1>>2]|0;
 $43 = (($42) + ($41)|0);
 HEAP8[$43>>0] = 0;
 $44 = HEAP32[$j>>2]|0;
 HEAP32[$0>>2] = $44;
 $45 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($45|0);
}
function _juliandayFunc($context,$argc,$argv) {
 $context = $context|0;
 $argc = $argc|0;
 $argv = $argv|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0.0, $16 = 0.0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $x = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 64|0;
 $0 = sp + 56|0;
 $1 = sp + 52|0;
 $2 = sp + 48|0;
 $x = sp;
 HEAP32[$0>>2] = $context;
 HEAP32[$1>>2] = $argc;
 HEAP32[$2>>2] = $argv;
 $3 = HEAP32[$0>>2]|0;
 $4 = HEAP32[$1>>2]|0;
 $5 = HEAP32[$2>>2]|0;
 $6 = (_isDate($3,$4,$5,$x)|0);
 $7 = ($6|0)==(0);
 if (!($7)) {
  STACKTOP = sp;return;
 }
 _computeJD($x);
 $8 = HEAP32[$0>>2]|0;
 $9 = $x;
 $10 = $9;
 $11 = HEAP32[$10>>2]|0;
 $12 = (($9) + 4)|0;
 $13 = $12;
 $14 = HEAP32[$13>>2]|0;
 $15 = (+($11>>>0)) + (4294967296.0*(+($14|0)));
 $16 = $15 / 8.64E+7;
 _sqlite3_result_double($8,$16);
 STACKTOP = sp;return;
}
function _dateFunc($context,$argc,$argv) {
 $context = $context|0;
 $argc = $argc|0;
 $argv = $argv|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $vararg_buffer = 0, $vararg_ptr1 = 0, $vararg_ptr2 = 0, $x = 0, $zBuf = 0;
 var label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 176|0;
 $vararg_buffer = sp + 48|0;
 $0 = sp + 68|0;
 $1 = sp + 64|0;
 $2 = sp + 60|0;
 $x = sp;
 $zBuf = sp + 72|0;
 HEAP32[$0>>2] = $context;
 HEAP32[$1>>2] = $argc;
 HEAP32[$2>>2] = $argv;
 $3 = HEAP32[$0>>2]|0;
 $4 = HEAP32[$1>>2]|0;
 $5 = HEAP32[$2>>2]|0;
 $6 = (_isDate($3,$4,$5,$x)|0);
 $7 = ($6|0)==(0);
 if (!($7)) {
  STACKTOP = sp;return;
 }
 _computeYMD($x);
 $8 = (($x) + 8|0);
 $9 = HEAP32[$8>>2]|0;
 $10 = (($x) + 12|0);
 $11 = HEAP32[$10>>2]|0;
 $12 = (($x) + 16|0);
 $13 = HEAP32[$12>>2]|0;
 HEAP32[$vararg_buffer>>2] = $9;
 $vararg_ptr1 = (($vararg_buffer) + 4|0);
 HEAP32[$vararg_ptr1>>2] = $11;
 $vararg_ptr2 = (($vararg_buffer) + 8|0);
 HEAP32[$vararg_ptr2>>2] = $13;
 (_sqlite3_snprintf(100,$zBuf,9672,$vararg_buffer)|0);
 $14 = HEAP32[$0>>2]|0;
 _sqlite3_result_text($14,$zBuf,-1,(-1));
 STACKTOP = sp;return;
}
function _timeFunc($context,$argc,$argv) {
 $context = $context|0;
 $argc = $argc|0;
 $argv = $argv|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0.0, $14 = 0, $15 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $vararg_buffer = 0, $vararg_ptr1 = 0, $vararg_ptr2 = 0, $x = 0;
 var $zBuf = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 176|0;
 $vararg_buffer = sp + 48|0;
 $0 = sp + 68|0;
 $1 = sp + 64|0;
 $2 = sp + 60|0;
 $x = sp;
 $zBuf = sp + 72|0;
 HEAP32[$0>>2] = $context;
 HEAP32[$1>>2] = $argc;
 HEAP32[$2>>2] = $argv;
 $3 = HEAP32[$0>>2]|0;
 $4 = HEAP32[$1>>2]|0;
 $5 = HEAP32[$2>>2]|0;
 $6 = (_isDate($3,$4,$5,$x)|0);
 $7 = ($6|0)==(0);
 if (!($7)) {
  STACKTOP = sp;return;
 }
 _computeHMS($x);
 $8 = (($x) + 20|0);
 $9 = HEAP32[$8>>2]|0;
 $10 = (($x) + 24|0);
 $11 = HEAP32[$10>>2]|0;
 $12 = (($x) + 32|0);
 $13 = +HEAPF64[$12>>3];
 $14 = (~~(($13)));
 HEAP32[$vararg_buffer>>2] = $9;
 $vararg_ptr1 = (($vararg_buffer) + 4|0);
 HEAP32[$vararg_ptr1>>2] = $11;
 $vararg_ptr2 = (($vararg_buffer) + 8|0);
 HEAP32[$vararg_ptr2>>2] = $14;
 (_sqlite3_snprintf(100,$zBuf,9656,$vararg_buffer)|0);
 $15 = HEAP32[$0>>2]|0;
 _sqlite3_result_text($15,$zBuf,-1,(-1));
 STACKTOP = sp;return;
}
function _datetimeFunc($context,$argc,$argv) {
 $context = $context|0;
 $argc = $argc|0;
 $argv = $argv|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0.0, $2 = 0, $20 = 0, $21 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0;
 var $8 = 0, $9 = 0, $vararg_buffer = 0, $vararg_ptr1 = 0, $vararg_ptr2 = 0, $vararg_ptr3 = 0, $vararg_ptr4 = 0, $vararg_ptr5 = 0, $x = 0, $zBuf = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 192|0;
 $vararg_buffer = sp + 48|0;
 $0 = sp + 80|0;
 $1 = sp + 76|0;
 $2 = sp + 72|0;
 $x = sp;
 $zBuf = sp + 84|0;
 HEAP32[$0>>2] = $context;
 HEAP32[$1>>2] = $argc;
 HEAP32[$2>>2] = $argv;
 $3 = HEAP32[$0>>2]|0;
 $4 = HEAP32[$1>>2]|0;
 $5 = HEAP32[$2>>2]|0;
 $6 = (_isDate($3,$4,$5,$x)|0);
 $7 = ($6|0)==(0);
 if (!($7)) {
  STACKTOP = sp;return;
 }
 _computeYMD_HMS($x);
 $8 = (($x) + 8|0);
 $9 = HEAP32[$8>>2]|0;
 $10 = (($x) + 12|0);
 $11 = HEAP32[$10>>2]|0;
 $12 = (($x) + 16|0);
 $13 = HEAP32[$12>>2]|0;
 $14 = (($x) + 20|0);
 $15 = HEAP32[$14>>2]|0;
 $16 = (($x) + 24|0);
 $17 = HEAP32[$16>>2]|0;
 $18 = (($x) + 32|0);
 $19 = +HEAPF64[$18>>3];
 $20 = (~~(($19)));
 HEAP32[$vararg_buffer>>2] = $9;
 $vararg_ptr1 = (($vararg_buffer) + 4|0);
 HEAP32[$vararg_ptr1>>2] = $11;
 $vararg_ptr2 = (($vararg_buffer) + 8|0);
 HEAP32[$vararg_ptr2>>2] = $13;
 $vararg_ptr3 = (($vararg_buffer) + 12|0);
 HEAP32[$vararg_ptr3>>2] = $15;
 $vararg_ptr4 = (($vararg_buffer) + 16|0);
 HEAP32[$vararg_ptr4>>2] = $17;
 $vararg_ptr5 = (($vararg_buffer) + 20|0);
 HEAP32[$vararg_ptr5>>2] = $20;
 (_sqlite3_snprintf(100,$zBuf,9624,$vararg_buffer)|0);
 $21 = HEAP32[$0>>2]|0;
 _sqlite3_result_text($21,$zBuf,-1,(-1));
 STACKTOP = sp;return;
}
function _strftimeFunc($context,$argc,$argv) {
 $context = $context|0;
 $argc = $argc|0;
 $argv = $argv|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0;
 var $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0, $188 = 0.0;
 var $189 = 0.0, $19 = 0, $190 = 0, $191 = 0, $192 = 0, $193 = 0, $194 = 0.0, $195 = 0, $196 = 0, $197 = 0, $198 = 0, $199 = 0, $2 = 0, $20 = 0, $200 = 0, $201 = 0, $202 = 0, $203 = 0, $204 = 0, $205 = 0;
 var $206 = 0, $207 = 0, $208 = 0, $209 = 0, $21 = 0, $210 = 0, $211 = 0, $212 = 0, $213 = 0, $214 = 0, $215 = 0, $216 = 0, $217 = 0, $218 = 0, $219 = 0, $22 = 0, $220 = 0, $221 = 0, $222 = 0, $223 = 0;
 var $224 = 0, $225 = 0, $226 = 0, $227 = 0, $228 = 0, $229 = 0, $23 = 0, $230 = 0, $231 = 0, $232 = 0, $233 = 0, $234 = 0, $235 = 0, $236 = 0, $237 = 0, $238 = 0, $239 = 0, $24 = 0, $240 = 0, $241 = 0;
 var $242 = 0, $243 = 0, $244 = 0, $245 = 0, $246 = 0, $247 = 0, $248 = 0, $249 = 0, $25 = 0, $250 = 0, $251 = 0, $252 = 0, $253 = 0, $254 = 0, $255 = 0, $256 = 0, $257 = 0, $258 = 0, $259 = 0, $26 = 0;
 var $260 = 0, $261 = 0, $262 = 0, $263 = 0, $264 = 0, $265 = 0, $266 = 0, $267 = 0, $268 = 0, $269 = 0, $27 = 0, $270 = 0, $271 = 0, $272 = 0, $273 = 0.0, $274 = 0.0, $275 = 0, $276 = 0, $277 = 0, $278 = 0;
 var $279 = 0, $28 = 0, $280 = 0, $281 = 0, $282 = 0, $283 = 0, $284 = 0, $285 = 0, $286 = 0, $287 = 0, $288 = 0, $289 = 0, $29 = 0, $290 = 0, $291 = 0, $292 = 0, $293 = 0, $294 = 0, $295 = 0, $296 = 0;
 var $297 = 0, $298 = 0, $299 = 0, $3 = 0, $30 = 0, $300 = 0, $301 = 0, $302 = 0, $303 = 0, $304 = 0, $305 = 0, $306 = 0, $307 = 0, $308 = 0, $309 = 0, $31 = 0, $310 = 0, $311 = 0, $312 = 0, $313 = 0;
 var $314 = 0, $315 = 0, $316 = 0, $317 = 0, $318 = 0, $319 = 0, $32 = 0, $320 = 0, $321 = 0, $322 = 0.0, $323 = 0, $324 = 0, $325 = 0, $326 = 0, $327 = 0, $328 = 0, $329 = 0, $33 = 0, $330 = 0, $331 = 0;
 var $332 = 0, $333 = 0, $334 = 0, $335 = 0, $336 = 0, $337 = 0, $338 = 0, $339 = 0, $34 = 0, $340 = 0, $341 = 0, $342 = 0, $343 = 0, $344 = 0, $345 = 0, $346 = 0, $347 = 0, $348 = 0, $349 = 0, $35 = 0;
 var $350 = 0, $351 = 0, $352 = 0, $353 = 0, $354 = 0, $355 = 0, $356 = 0, $357 = 0, $358 = 0, $359 = 0, $36 = 0, $360 = 0, $361 = 0, $362 = 0, $363 = 0, $364 = 0, $365 = 0, $366 = 0, $367 = 0, $368 = 0;
 var $369 = 0, $37 = 0, $370 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0;
 var $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0;
 var $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0;
 var $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $db = 0, $i = 0, $j = 0, $n = 0, $nDay = 0, $s = 0, $vararg_buffer28 = 0, $wd = 0, $x = 0;
 var $y = 0, $z = 0, $zBuf = 0, $zFmt = 0, dest = 0, label = 0, sp = 0, src = 0, stop = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 272|0;
 $vararg_buffer28 = sp + 112|0;
 $0 = sp + 156|0;
 $1 = sp + 152|0;
 $2 = sp + 148|0;
 $x = sp + 64|0;
 $n = sp + 56|0;
 $i = sp + 144|0;
 $j = sp + 140|0;
 $z = sp + 136|0;
 $db = sp + 132|0;
 $zFmt = sp + 128|0;
 $zBuf = sp + 160|0;
 $s = sp + 48|0;
 $nDay = sp + 124|0;
 $y = sp;
 $wd = sp + 120|0;
 HEAP32[$0>>2] = $context;
 HEAP32[$1>>2] = $argc;
 HEAP32[$2>>2] = $argv;
 $3 = HEAP32[$2>>2]|0;
 $4 = HEAP32[$3>>2]|0;
 $5 = (_sqlite3_value_text($4)|0);
 HEAP32[$zFmt>>2] = $5;
 $6 = HEAP32[$zFmt>>2]|0;
 $7 = ($6|0)==(0|0);
 if ($7) {
  STACKTOP = sp;return;
 }
 $8 = HEAP32[$0>>2]|0;
 $9 = HEAP32[$1>>2]|0;
 $10 = (($9) - 1)|0;
 $11 = HEAP32[$2>>2]|0;
 $12 = (($11) + 4|0);
 $13 = (_isDate($8,$10,$12,$x)|0);
 $14 = ($13|0)!=(0);
 if ($14) {
  STACKTOP = sp;return;
 }
 $15 = HEAP32[$0>>2]|0;
 $16 = (_sqlite3_context_db_handle($15)|0);
 HEAP32[$db>>2] = $16;
 HEAP32[$i>>2] = 0;
 $17 = $n;
 $18 = $17;
 HEAP32[$18>>2] = 1;
 $19 = (($17) + 4)|0;
 $20 = $19;
 HEAP32[$20>>2] = 0;
 L7: while(1) {
  $21 = HEAP32[$i>>2]|0;
  $22 = HEAP32[$zFmt>>2]|0;
  $23 = (($22) + ($21)|0);
  $24 = HEAP8[$23>>0]|0;
  $25 = ($24<<24>>24)!=(0);
  if (!($25)) {
   break;
  }
  $26 = HEAP32[$i>>2]|0;
  $27 = HEAP32[$zFmt>>2]|0;
  $28 = (($27) + ($26)|0);
  $29 = HEAP8[$28>>0]|0;
  $30 = $29 << 24 >> 24;
  $31 = ($30|0)==(37);
  if ($31) {
   $32 = HEAP32[$i>>2]|0;
   $33 = (($32) + 1)|0;
   $34 = HEAP32[$zFmt>>2]|0;
   $35 = (($34) + ($33)|0);
   $36 = HEAP8[$35>>0]|0;
   $37 = $36 << 24 >> 24;
   switch ($37|0) {
   case 87: case 83: case 77: case 109: case 72: case 100:  {
    $38 = $n;
    $39 = $38;
    $40 = HEAP32[$39>>2]|0;
    $41 = (($38) + 4)|0;
    $42 = $41;
    $43 = HEAP32[$42>>2]|0;
    $44 = (_i64Add(($40|0),($43|0),1,0)|0);
    $45 = tempRet0;
    $46 = $n;
    $47 = $46;
    HEAP32[$47>>2] = $44;
    $48 = (($46) + 4)|0;
    $49 = $48;
    HEAP32[$49>>2] = $45;
    break;
   }
   case 37: case 119:  {
    break;
   }
   case 106:  {
    $62 = $n;
    $63 = $62;
    $64 = HEAP32[$63>>2]|0;
    $65 = (($62) + 4)|0;
    $66 = $65;
    $67 = HEAP32[$66>>2]|0;
    $68 = (_i64Add(($64|0),($67|0),3,0)|0);
    $69 = tempRet0;
    $70 = $n;
    $71 = $70;
    HEAP32[$71>>2] = $68;
    $72 = (($70) + 4)|0;
    $73 = $72;
    HEAP32[$73>>2] = $69;
    break;
   }
   case 89:  {
    $74 = $n;
    $75 = $74;
    $76 = HEAP32[$75>>2]|0;
    $77 = (($74) + 4)|0;
    $78 = $77;
    $79 = HEAP32[$78>>2]|0;
    $80 = (_i64Add(($76|0),($79|0),8,0)|0);
    $81 = tempRet0;
    $82 = $n;
    $83 = $82;
    HEAP32[$83>>2] = $80;
    $84 = (($82) + 4)|0;
    $85 = $84;
    HEAP32[$85>>2] = $81;
    break;
   }
   case 102:  {
    $50 = $n;
    $51 = $50;
    $52 = HEAP32[$51>>2]|0;
    $53 = (($50) + 4)|0;
    $54 = $53;
    $55 = HEAP32[$54>>2]|0;
    $56 = (_i64Add(($52|0),($55|0),8,0)|0);
    $57 = tempRet0;
    $58 = $n;
    $59 = $58;
    HEAP32[$59>>2] = $56;
    $60 = (($58) + 4)|0;
    $61 = $60;
    HEAP32[$61>>2] = $57;
    break;
   }
   case 74: case 115:  {
    $86 = $n;
    $87 = $86;
    $88 = HEAP32[$87>>2]|0;
    $89 = (($86) + 4)|0;
    $90 = $89;
    $91 = HEAP32[$90>>2]|0;
    $92 = (_i64Add(($88|0),($91|0),50,0)|0);
    $93 = tempRet0;
    $94 = $n;
    $95 = $94;
    HEAP32[$95>>2] = $92;
    $96 = (($94) + 4)|0;
    $97 = $96;
    HEAP32[$97>>2] = $93;
    break;
   }
   default: {
    label = 43;
    break L7;
   }
   }
   $98 = HEAP32[$i>>2]|0;
   $99 = (($98) + 1)|0;
   HEAP32[$i>>2] = $99;
  }
  $100 = HEAP32[$i>>2]|0;
  $101 = (($100) + 1)|0;
  HEAP32[$i>>2] = $101;
  $102 = $n;
  $103 = $102;
  $104 = HEAP32[$103>>2]|0;
  $105 = (($102) + 4)|0;
  $106 = $105;
  $107 = HEAP32[$106>>2]|0;
  $108 = (_i64Add(($104|0),($107|0),1,0)|0);
  $109 = tempRet0;
  $110 = $n;
  $111 = $110;
  HEAP32[$111>>2] = $108;
  $112 = (($110) + 4)|0;
  $113 = $112;
  HEAP32[$113>>2] = $109;
 }
 if ((label|0) == 43) {
  STACKTOP = sp;return;
 }
 $114 = $n;
 $115 = $114;
 $116 = HEAP32[$115>>2]|0;
 $117 = (($114) + 4)|0;
 $118 = $117;
 $119 = HEAP32[$118>>2]|0;
 $120 = ($119>>>0)<(0);
 $121 = ($116>>>0)<(100);
 $122 = ($119|0)==(0);
 $123 = $122 & $121;
 $124 = $120 | $123;
 if ($124) {
  HEAP32[$z>>2] = $zBuf;
 } else {
  $125 = $n;
  $126 = $125;
  $127 = HEAP32[$126>>2]|0;
  $128 = (($125) + 4)|0;
  $129 = $128;
  $130 = HEAP32[$129>>2]|0;
  $131 = HEAP32[$db>>2]|0;
  $132 = (($131) + 88|0);
  $133 = HEAP32[$132>>2]|0;
  $134 = ($133|0)<(0);
  $135 = $134 << 31 >> 31;
  $136 = ($130>>>0)>($135>>>0);
  $137 = ($127>>>0)>($133>>>0);
  $138 = ($130|0)==($135|0);
  $139 = $138 & $137;
  $140 = $136 | $139;
  if ($140) {
   $141 = HEAP32[$0>>2]|0;
   _sqlite3_result_error_toobig($141);
   STACKTOP = sp;return;
  }
  $142 = HEAP32[$db>>2]|0;
  $143 = $n;
  $144 = $143;
  $145 = HEAP32[$144>>2]|0;
  $146 = (($143) + 4)|0;
  $147 = $146;
  $148 = HEAP32[$147>>2]|0;
  $149 = ($145|0)<(0);
  $150 = $149 << 31 >> 31;
  $151 = (_sqlite3DbMallocRaw($142,$145,$150)|0);
  HEAP32[$z>>2] = $151;
  $152 = HEAP32[$z>>2]|0;
  $153 = ($152|0)==(0|0);
  if ($153) {
   $154 = HEAP32[$0>>2]|0;
   _sqlite3_result_error_nomem($154);
   STACKTOP = sp;return;
  }
 }
 _computeJD($x);
 _computeYMD_HMS($x);
 HEAP32[$j>>2] = 0;
 HEAP32[$i>>2] = 0;
 while(1) {
  $155 = HEAP32[$i>>2]|0;
  $156 = HEAP32[$zFmt>>2]|0;
  $157 = (($156) + ($155)|0);
  $158 = HEAP8[$157>>0]|0;
  $159 = ($158<<24>>24)!=(0);
  if (!($159)) {
   break;
  }
  $160 = HEAP32[$i>>2]|0;
  $161 = HEAP32[$zFmt>>2]|0;
  $162 = (($161) + ($160)|0);
  $163 = HEAP8[$162>>0]|0;
  $164 = $163 << 24 >> 24;
  $165 = ($164|0)!=(37);
  $166 = HEAP32[$i>>2]|0;
  L36: do {
   if ($165) {
    $167 = HEAP32[$zFmt>>2]|0;
    $168 = (($167) + ($166)|0);
    $169 = HEAP8[$168>>0]|0;
    $170 = HEAP32[$j>>2]|0;
    $171 = (($170) + 1)|0;
    HEAP32[$j>>2] = $171;
    $172 = HEAP32[$z>>2]|0;
    $173 = (($172) + ($170)|0);
    HEAP8[$173>>0] = $169;
   } else {
    $174 = (($166) + 1)|0;
    HEAP32[$i>>2] = $174;
    $175 = HEAP32[$i>>2]|0;
    $176 = HEAP32[$zFmt>>2]|0;
    $177 = (($176) + ($175)|0);
    $178 = HEAP8[$177>>0]|0;
    $179 = $178 << 24 >> 24;
    do {
     switch ($179|0) {
     case 74:  {
      $264 = HEAP32[$j>>2]|0;
      $265 = HEAP32[$z>>2]|0;
      $266 = (($265) + ($264)|0);
      $267 = $x;
      $268 = $267;
      $269 = HEAP32[$268>>2]|0;
      $270 = (($267) + 4)|0;
      $271 = $270;
      $272 = HEAP32[$271>>2]|0;
      $273 = (+($269>>>0)) + (4294967296.0*(+($272|0)));
      $274 = $273 / 8.64E+7;
      HEAPF64[tempDoublePtr>>3]=$274;HEAP32[$vararg_buffer28>>2]=HEAP32[tempDoublePtr>>2];HEAP32[$vararg_buffer28+4>>2]=HEAP32[tempDoublePtr+4>>2];
      (_sqlite3_snprintf(20,$266,9448,$vararg_buffer28)|0);
      $275 = HEAP32[$j>>2]|0;
      $276 = HEAP32[$z>>2]|0;
      $277 = (($276) + ($275)|0);
      $278 = (_sqlite3Strlen30($277)|0);
      $279 = HEAP32[$j>>2]|0;
      $280 = (($279) + ($278))|0;
      HEAP32[$j>>2] = $280;
      break L36;
      break;
     }
     case 72:  {
      $201 = HEAP32[$j>>2]|0;
      $202 = HEAP32[$z>>2]|0;
      $203 = (($202) + ($201)|0);
      $204 = (($x) + 20|0);
      $205 = HEAP32[$204>>2]|0;
      HEAP32[$vararg_buffer28>>2] = $205;
      (_sqlite3_snprintf(3,$203,9424,$vararg_buffer28)|0);
      $206 = HEAP32[$j>>2]|0;
      $207 = (($206) + 2)|0;
      HEAP32[$j>>2] = $207;
      break L36;
      break;
     }
     case 106: case 87:  {
      dest=$y+0|0; src=$x+0|0; stop=dest+48|0; do { HEAP32[dest>>2]=HEAP32[src>>2]|0; dest=dest+4|0; src=src+4|0; } while ((dest|0) < (stop|0));
      $208 = (($y) + 42|0);
      HEAP8[$208>>0] = 0;
      $209 = (($y) + 12|0);
      HEAP32[$209>>2] = 1;
      $210 = (($y) + 16|0);
      HEAP32[$210>>2] = 1;
      _computeJD($y);
      $211 = $x;
      $212 = $211;
      $213 = HEAP32[$212>>2]|0;
      $214 = (($211) + 4)|0;
      $215 = $214;
      $216 = HEAP32[$215>>2]|0;
      $217 = $y;
      $218 = $217;
      $219 = HEAP32[$218>>2]|0;
      $220 = (($217) + 4)|0;
      $221 = $220;
      $222 = HEAP32[$221>>2]|0;
      $223 = (_i64Subtract(($213|0),($216|0),($219|0),($222|0))|0);
      $224 = tempRet0;
      $225 = (_i64Add(($223|0),($224|0),43200000,0)|0);
      $226 = tempRet0;
      $227 = (___divdi3(($225|0),($226|0),86400000,0)|0);
      $228 = tempRet0;
      HEAP32[$nDay>>2] = $227;
      $229 = HEAP32[$i>>2]|0;
      $230 = HEAP32[$zFmt>>2]|0;
      $231 = (($230) + ($229)|0);
      $232 = HEAP8[$231>>0]|0;
      $233 = $232 << 24 >> 24;
      $234 = ($233|0)==(87);
      if ($234) {
       $235 = $x;
       $236 = $235;
       $237 = HEAP32[$236>>2]|0;
       $238 = (($235) + 4)|0;
       $239 = $238;
       $240 = HEAP32[$239>>2]|0;
       $241 = (_i64Add(($237|0),($240|0),43200000,0)|0);
       $242 = tempRet0;
       $243 = (___divdi3(($241|0),($242|0),86400000,0)|0);
       $244 = tempRet0;
       $245 = (___remdi3(($243|0),($244|0),7,0)|0);
       $246 = tempRet0;
       HEAP32[$wd>>2] = $245;
       $247 = HEAP32[$j>>2]|0;
       $248 = HEAP32[$z>>2]|0;
       $249 = (($248) + ($247)|0);
       $250 = HEAP32[$nDay>>2]|0;
       $251 = (($250) + 7)|0;
       $252 = HEAP32[$wd>>2]|0;
       $253 = (($251) - ($252))|0;
       $254 = (($253|0) / 7)&-1;
       HEAP32[$vararg_buffer28>>2] = $254;
       (_sqlite3_snprintf(3,$249,9424,$vararg_buffer28)|0);
       $255 = HEAP32[$j>>2]|0;
       $256 = (($255) + 2)|0;
       HEAP32[$j>>2] = $256;
       break L36;
      } else {
       $257 = HEAP32[$j>>2]|0;
       $258 = HEAP32[$z>>2]|0;
       $259 = (($258) + ($257)|0);
       $260 = HEAP32[$nDay>>2]|0;
       $261 = (($260) + 1)|0;
       HEAP32[$vararg_buffer28>>2] = $261;
       (_sqlite3_snprintf(4,$259,9440,$vararg_buffer28)|0);
       $262 = HEAP32[$j>>2]|0;
       $263 = (($262) + 3)|0;
       HEAP32[$j>>2] = $263;
       break L36;
      }
      break;
     }
     case 109:  {
      $281 = HEAP32[$j>>2]|0;
      $282 = HEAP32[$z>>2]|0;
      $283 = (($282) + ($281)|0);
      $284 = (($x) + 12|0);
      $285 = HEAP32[$284>>2]|0;
      HEAP32[$vararg_buffer28>>2] = $285;
      (_sqlite3_snprintf(3,$283,9424,$vararg_buffer28)|0);
      $286 = HEAP32[$j>>2]|0;
      $287 = (($286) + 2)|0;
      HEAP32[$j>>2] = $287;
      break L36;
      break;
     }
     case 102:  {
      $187 = (($x) + 32|0);
      $188 = +HEAPF64[$187>>3];
      HEAPF64[$s>>3] = $188;
      $189 = +HEAPF64[$s>>3];
      $190 = $189 > 59.999000000000002;
      if ($190) {
       HEAPF64[$s>>3] = 59.999000000000002;
      }
      $191 = HEAP32[$j>>2]|0;
      $192 = HEAP32[$z>>2]|0;
      $193 = (($192) + ($191)|0);
      $194 = +HEAPF64[$s>>3];
      HEAPF64[tempDoublePtr>>3]=$194;HEAP32[$vararg_buffer28>>2]=HEAP32[tempDoublePtr>>2];HEAP32[$vararg_buffer28+4>>2]=HEAP32[tempDoublePtr+4>>2];
      (_sqlite3_snprintf(7,$193,9432,$vararg_buffer28)|0);
      $195 = HEAP32[$j>>2]|0;
      $196 = HEAP32[$z>>2]|0;
      $197 = (($196) + ($195)|0);
      $198 = (_sqlite3Strlen30($197)|0);
      $199 = HEAP32[$j>>2]|0;
      $200 = (($199) + ($198))|0;
      HEAP32[$j>>2] = $200;
      break L36;
      break;
     }
     case 115:  {
      $295 = HEAP32[$j>>2]|0;
      $296 = HEAP32[$z>>2]|0;
      $297 = (($296) + ($295)|0);
      $298 = $x;
      $299 = $298;
      $300 = HEAP32[$299>>2]|0;
      $301 = (($298) + 4)|0;
      $302 = $301;
      $303 = HEAP32[$302>>2]|0;
      $304 = (___divdi3(($300|0),($303|0),1000,0)|0);
      $305 = tempRet0;
      $306 = (_i64Subtract(($304|0),($305|0),413362496,49)|0);
      $307 = tempRet0;
      $308 = $vararg_buffer28;
      $309 = $308;
      HEAP32[$309>>2] = $306;
      $310 = (($308) + 4)|0;
      $311 = $310;
      HEAP32[$311>>2] = $307;
      (_sqlite3_snprintf(30,$297,9456,$vararg_buffer28)|0);
      $312 = HEAP32[$j>>2]|0;
      $313 = HEAP32[$z>>2]|0;
      $314 = (($313) + ($312)|0);
      $315 = (_sqlite3Strlen30($314)|0);
      $316 = HEAP32[$j>>2]|0;
      $317 = (($316) + ($315))|0;
      HEAP32[$j>>2] = $317;
      break L36;
      break;
     }
     case 100:  {
      $180 = HEAP32[$j>>2]|0;
      $181 = HEAP32[$z>>2]|0;
      $182 = (($181) + ($180)|0);
      $183 = (($x) + 16|0);
      $184 = HEAP32[$183>>2]|0;
      HEAP32[$vararg_buffer28>>2] = $184;
      (_sqlite3_snprintf(3,$182,9424,$vararg_buffer28)|0);
      $185 = HEAP32[$j>>2]|0;
      $186 = (($185) + 2)|0;
      HEAP32[$j>>2] = $186;
      break L36;
      break;
     }
     case 119:  {
      $326 = $x;
      $327 = $326;
      $328 = HEAP32[$327>>2]|0;
      $329 = (($326) + 4)|0;
      $330 = $329;
      $331 = HEAP32[$330>>2]|0;
      $332 = (_i64Add(($328|0),($331|0),129600000,0)|0);
      $333 = tempRet0;
      $334 = (___divdi3(($332|0),($333|0),86400000,0)|0);
      $335 = tempRet0;
      $336 = (___remdi3(($334|0),($335|0),7,0)|0);
      $337 = tempRet0;
      $338 = $336&255;
      $339 = $338 << 24 >> 24;
      $340 = (($339) + 48)|0;
      $341 = $340&255;
      $342 = HEAP32[$j>>2]|0;
      $343 = (($342) + 1)|0;
      HEAP32[$j>>2] = $343;
      $344 = HEAP32[$z>>2]|0;
      $345 = (($344) + ($342)|0);
      HEAP8[$345>>0] = $341;
      break L36;
      break;
     }
     case 89:  {
      $346 = HEAP32[$j>>2]|0;
      $347 = HEAP32[$z>>2]|0;
      $348 = (($347) + ($346)|0);
      $349 = (($x) + 8|0);
      $350 = HEAP32[$349>>2]|0;
      HEAP32[$vararg_buffer28>>2] = $350;
      (_sqlite3_snprintf(5,$348,9464,$vararg_buffer28)|0);
      $351 = HEAP32[$j>>2]|0;
      $352 = HEAP32[$z>>2]|0;
      $353 = (($352) + ($351)|0);
      $354 = (_sqlite3Strlen30($353)|0);
      $355 = HEAP32[$j>>2]|0;
      $356 = (($355) + ($354))|0;
      HEAP32[$j>>2] = $356;
      break L36;
      break;
     }
     case 77:  {
      $288 = HEAP32[$j>>2]|0;
      $289 = HEAP32[$z>>2]|0;
      $290 = (($289) + ($288)|0);
      $291 = (($x) + 24|0);
      $292 = HEAP32[$291>>2]|0;
      HEAP32[$vararg_buffer28>>2] = $292;
      (_sqlite3_snprintf(3,$290,9424,$vararg_buffer28)|0);
      $293 = HEAP32[$j>>2]|0;
      $294 = (($293) + 2)|0;
      HEAP32[$j>>2] = $294;
      break L36;
      break;
     }
     case 83:  {
      $318 = HEAP32[$j>>2]|0;
      $319 = HEAP32[$z>>2]|0;
      $320 = (($319) + ($318)|0);
      $321 = (($x) + 32|0);
      $322 = +HEAPF64[$321>>3];
      $323 = (~~(($322)));
      HEAP32[$vararg_buffer28>>2] = $323;
      (_sqlite3_snprintf(3,$320,9424,$vararg_buffer28)|0);
      $324 = HEAP32[$j>>2]|0;
      $325 = (($324) + 2)|0;
      HEAP32[$j>>2] = $325;
      break L36;
      break;
     }
     default: {
      $357 = HEAP32[$j>>2]|0;
      $358 = (($357) + 1)|0;
      HEAP32[$j>>2] = $358;
      $359 = HEAP32[$z>>2]|0;
      $360 = (($359) + ($357)|0);
      HEAP8[$360>>0] = 37;
      break L36;
     }
     }
    } while(0);
   }
  } while(0);
  $361 = HEAP32[$i>>2]|0;
  $362 = (($361) + 1)|0;
  HEAP32[$i>>2] = $362;
 }
 $363 = HEAP32[$j>>2]|0;
 $364 = HEAP32[$z>>2]|0;
 $365 = (($364) + ($363)|0);
 HEAP8[$365>>0] = 0;
 $366 = HEAP32[$0>>2]|0;
 $367 = HEAP32[$z>>2]|0;
 $368 = HEAP32[$z>>2]|0;
 $369 = ($368|0)==($zBuf|0);
 $370 = $369 ? (-1) : 16;
 _sqlite3_result_text($366,$367,-1,$370);
 STACKTOP = sp;return;
}
function _ctimeFunc($context,$NotUsed,$NotUsed2) {
 $context = $context|0;
 $NotUsed = $NotUsed|0;
 $NotUsed2 = $NotUsed2|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $1 = sp + 4|0;
 $2 = sp;
 HEAP32[$0>>2] = $context;
 HEAP32[$1>>2] = $NotUsed;
 HEAP32[$2>>2] = $NotUsed2;
 $3 = HEAP32[$0>>2]|0;
 _timeFunc($3,0,0);
 STACKTOP = sp;return;
}
function _ctimestampFunc($context,$NotUsed,$NotUsed2) {
 $context = $context|0;
 $NotUsed = $NotUsed|0;
 $NotUsed2 = $NotUsed2|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $1 = sp + 4|0;
 $2 = sp;
 HEAP32[$0>>2] = $context;
 HEAP32[$1>>2] = $NotUsed;
 HEAP32[$2>>2] = $NotUsed2;
 $3 = HEAP32[$0>>2]|0;
 _datetimeFunc($3,0,0);
 STACKTOP = sp;return;
}
function _cdateFunc($context,$NotUsed,$NotUsed2) {
 $context = $context|0;
 $NotUsed = $NotUsed|0;
 $NotUsed2 = $NotUsed2|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $1 = sp + 4|0;
 $2 = sp;
 HEAP32[$0>>2] = $context;
 HEAP32[$1>>2] = $NotUsed;
 HEAP32[$2>>2] = $NotUsed2;
 $3 = HEAP32[$0>>2]|0;
 _dateFunc($3,0,0);
 STACKTOP = sp;return;
}
function _isDate($context,$argc,$argv,$p) {
 $context = $context|0;
 $argc = $argc|0;
 $argv = $argv|0;
 $p = $p|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0.0, $2 = 0, $20 = 0.0, $21 = 0.0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $eType = 0, $i = 0, $or$cond = 0;
 var $z = 0, dest = 0, label = 0, sp = 0, stop = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 28|0;
 $1 = sp + 24|0;
 $2 = sp + 20|0;
 $3 = sp + 16|0;
 $4 = sp + 12|0;
 $i = sp + 8|0;
 $z = sp + 4|0;
 $eType = sp;
 HEAP32[$1>>2] = $context;
 HEAP32[$2>>2] = $argc;
 HEAP32[$3>>2] = $argv;
 HEAP32[$4>>2] = $p;
 $5 = HEAP32[$4>>2]|0;
 dest=$5+0|0; stop=dest+48|0; do { HEAP32[dest>>2]=0|0; dest=dest+4|0; } while ((dest|0) < (stop|0));
 $6 = HEAP32[$2>>2]|0;
 $7 = ($6|0)==(0);
 if ($7) {
  $8 = HEAP32[$1>>2]|0;
  $9 = HEAP32[$4>>2]|0;
  $10 = (_setDateTimeToCurrent($8,$9)|0);
  HEAP32[$0>>2] = $10;
  $56 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($56|0);
 }
 $11 = HEAP32[$3>>2]|0;
 $12 = HEAP32[$11>>2]|0;
 $13 = (_sqlite3_value_type($12)|0);
 HEAP32[$eType>>2] = $13;
 $14 = ($13|0)==(2);
 $15 = HEAP32[$eType>>2]|0;
 $16 = ($15|0)==(1);
 $or$cond = $14 | $16;
 $17 = HEAP32[$3>>2]|0;
 $18 = HEAP32[$17>>2]|0;
 do {
  if ($or$cond) {
   $19 = (+_sqlite3_value_double($18));
   $20 = $19 * 8.64E+7;
   $21 = $20 + 0.5;
   $22 = (~~$21)>>>0;
   $23 = +Math_abs($21) >= 1.0 ? $21 > 0.0 ? (Math_min(+Math_floor($21 / 4294967296.0), 4294967295.0) | 0) >>> 0 : ~~+Math_ceil(($21 - +(~~$21 >>> 0)) / 4294967296.0) >>> 0 : 0;
   $24 = HEAP32[$4>>2]|0;
   $25 = $24;
   $26 = $25;
   HEAP32[$26>>2] = $22;
   $27 = (($25) + 4)|0;
   $28 = $27;
   HEAP32[$28>>2] = $23;
   $29 = HEAP32[$4>>2]|0;
   $30 = (($29) + 42|0);
   HEAP8[$30>>0] = 1;
  } else {
   $31 = (_sqlite3_value_text($18)|0);
   HEAP32[$z>>2] = $31;
   $32 = HEAP32[$z>>2]|0;
   $33 = ($32|0)!=(0|0);
   if ($33) {
    $34 = HEAP32[$1>>2]|0;
    $35 = HEAP32[$z>>2]|0;
    $36 = HEAP32[$4>>2]|0;
    $37 = (_parseDateOrTime($34,$35,$36)|0);
    $38 = ($37|0)!=(0);
    if (!($38)) {
     break;
    }
   }
   HEAP32[$0>>2] = 1;
   $56 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($56|0);
  }
 } while(0);
 HEAP32[$i>>2] = 1;
 while(1) {
  $39 = HEAP32[$i>>2]|0;
  $40 = HEAP32[$2>>2]|0;
  $41 = ($39|0)<($40|0);
  if (!($41)) {
   label = 14;
   break;
  }
  $42 = HEAP32[$i>>2]|0;
  $43 = HEAP32[$3>>2]|0;
  $44 = (($43) + ($42<<2)|0);
  $45 = HEAP32[$44>>2]|0;
  $46 = (_sqlite3_value_text($45)|0);
  HEAP32[$z>>2] = $46;
  $47 = HEAP32[$z>>2]|0;
  $48 = ($47|0)==(0|0);
  if ($48) {
   label = 12;
   break;
  }
  $49 = HEAP32[$1>>2]|0;
  $50 = HEAP32[$z>>2]|0;
  $51 = HEAP32[$4>>2]|0;
  $52 = (_parseModifier($49,$50,$51)|0);
  $53 = ($52|0)!=(0);
  if ($53) {
   label = 12;
   break;
  }
  $54 = HEAP32[$i>>2]|0;
  $55 = (($54) + 1)|0;
  HEAP32[$i>>2] = $55;
 }
 if ((label|0) == 12) {
  HEAP32[$0>>2] = 1;
  $56 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($56|0);
 }
 else if ((label|0) == 14) {
  HEAP32[$0>>2] = 0;
  $56 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($56|0);
 }
 return 0|0;
}
function _computeJD($p) {
 $p = $p|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0;
 var $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0;
 var $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0.0, $47 = 0.0, $48 = 0.0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0;
 var $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0.0, $76 = 0.0, $77 = 0, $78 = 0;
 var $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0;
 var $97 = 0, $98 = 0, $99 = 0, $A = 0, $B = 0, $D = 0, $M = 0, $X1 = 0, $X2 = 0, $Y = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 28|0;
 $Y = sp + 24|0;
 $M = sp + 20|0;
 $D = sp + 16|0;
 $A = sp + 12|0;
 $B = sp + 8|0;
 $X1 = sp + 4|0;
 $X2 = sp;
 HEAP32[$0>>2] = $p;
 $1 = HEAP32[$0>>2]|0;
 $2 = (($1) + 42|0);
 $3 = HEAP8[$2>>0]|0;
 $4 = ($3<<24>>24)!=(0);
 if ($4) {
  STACKTOP = sp;return;
 }
 $5 = HEAP32[$0>>2]|0;
 $6 = (($5) + 40|0);
 $7 = HEAP8[$6>>0]|0;
 $8 = ($7<<24>>24)!=(0);
 if ($8) {
  $9 = HEAP32[$0>>2]|0;
  $10 = (($9) + 8|0);
  $11 = HEAP32[$10>>2]|0;
  HEAP32[$Y>>2] = $11;
  $12 = HEAP32[$0>>2]|0;
  $13 = (($12) + 12|0);
  $14 = HEAP32[$13>>2]|0;
  HEAP32[$M>>2] = $14;
  $15 = HEAP32[$0>>2]|0;
  $16 = (($15) + 16|0);
  $17 = HEAP32[$16>>2]|0;
  HEAP32[$D>>2] = $17;
 } else {
  HEAP32[$Y>>2] = 2000;
  HEAP32[$M>>2] = 1;
  HEAP32[$D>>2] = 1;
 }
 $18 = HEAP32[$M>>2]|0;
 $19 = ($18|0)<=(2);
 if ($19) {
  $20 = HEAP32[$Y>>2]|0;
  $21 = (($20) + -1)|0;
  HEAP32[$Y>>2] = $21;
  $22 = HEAP32[$M>>2]|0;
  $23 = (($22) + 12)|0;
  HEAP32[$M>>2] = $23;
 }
 $24 = HEAP32[$Y>>2]|0;
 $25 = (($24|0) / 100)&-1;
 HEAP32[$A>>2] = $25;
 $26 = HEAP32[$A>>2]|0;
 $27 = (2 - ($26))|0;
 $28 = HEAP32[$A>>2]|0;
 $29 = (($28|0) / 4)&-1;
 $30 = (($27) + ($29))|0;
 HEAP32[$B>>2] = $30;
 $31 = HEAP32[$Y>>2]|0;
 $32 = (($31) + 4716)|0;
 $33 = ($32*36525)|0;
 $34 = (($33|0) / 100)&-1;
 HEAP32[$X1>>2] = $34;
 $35 = HEAP32[$M>>2]|0;
 $36 = (($35) + 1)|0;
 $37 = ($36*306001)|0;
 $38 = (($37|0) / 10000)&-1;
 HEAP32[$X2>>2] = $38;
 $39 = HEAP32[$X1>>2]|0;
 $40 = HEAP32[$X2>>2]|0;
 $41 = (($39) + ($40))|0;
 $42 = HEAP32[$D>>2]|0;
 $43 = (($41) + ($42))|0;
 $44 = HEAP32[$B>>2]|0;
 $45 = (($43) + ($44))|0;
 $46 = (+($45|0));
 $47 = $46 - 1524.5;
 $48 = $47 * 8.64E+7;
 $49 = (~~$48)>>>0;
 $50 = +Math_abs($48) >= 1.0 ? $48 > 0.0 ? (Math_min(+Math_floor($48 / 4294967296.0), 4294967295.0) | 0) >>> 0 : ~~+Math_ceil(($48 - +(~~$48 >>> 0)) / 4294967296.0) >>> 0 : 0;
 $51 = HEAP32[$0>>2]|0;
 $52 = $51;
 $53 = $52;
 HEAP32[$53>>2] = $49;
 $54 = (($52) + 4)|0;
 $55 = $54;
 HEAP32[$55>>2] = $50;
 $56 = HEAP32[$0>>2]|0;
 $57 = (($56) + 42|0);
 HEAP8[$57>>0] = 1;
 $58 = HEAP32[$0>>2]|0;
 $59 = (($58) + 41|0);
 $60 = HEAP8[$59>>0]|0;
 $61 = ($60<<24>>24)!=(0);
 if (!($61)) {
  STACKTOP = sp;return;
 }
 $62 = HEAP32[$0>>2]|0;
 $63 = (($62) + 20|0);
 $64 = HEAP32[$63>>2]|0;
 $65 = Math_imul($64, 3600000)|0;
 $66 = HEAP32[$0>>2]|0;
 $67 = (($66) + 24|0);
 $68 = HEAP32[$67>>2]|0;
 $69 = ($68*60000)|0;
 $70 = (($65) + ($69))|0;
 $71 = ($70|0)<(0);
 $72 = $71 << 31 >> 31;
 $73 = HEAP32[$0>>2]|0;
 $74 = (($73) + 32|0);
 $75 = +HEAPF64[$74>>3];
 $76 = $75 * 1000.0;
 $77 = (~~$76)>>>0;
 $78 = +Math_abs($76) >= 1.0 ? $76 > 0.0 ? (Math_min(+Math_floor($76 / 4294967296.0), 4294967295.0) | 0) >>> 0 : ~~+Math_ceil(($76 - +(~~$76 >>> 0)) / 4294967296.0) >>> 0 : 0;
 $79 = (_i64Add(($70|0),($72|0),($77|0),($78|0))|0);
 $80 = tempRet0;
 $81 = HEAP32[$0>>2]|0;
 $82 = $81;
 $83 = $82;
 $84 = HEAP32[$83>>2]|0;
 $85 = (($82) + 4)|0;
 $86 = $85;
 $87 = HEAP32[$86>>2]|0;
 $88 = (_i64Add(($84|0),($87|0),($79|0),($80|0))|0);
 $89 = tempRet0;
 $90 = $81;
 $91 = $90;
 HEAP32[$91>>2] = $88;
 $92 = (($90) + 4)|0;
 $93 = $92;
 HEAP32[$93>>2] = $89;
 $94 = HEAP32[$0>>2]|0;
 $95 = (($94) + 43|0);
 $96 = HEAP8[$95>>0]|0;
 $97 = ($96<<24>>24)!=(0);
 if (!($97)) {
  STACKTOP = sp;return;
 }
 $98 = HEAP32[$0>>2]|0;
 $99 = (($98) + 28|0);
 $100 = HEAP32[$99>>2]|0;
 $101 = ($100*60000)|0;
 $102 = ($101|0)<(0);
 $103 = $102 << 31 >> 31;
 $104 = HEAP32[$0>>2]|0;
 $105 = $104;
 $106 = $105;
 $107 = HEAP32[$106>>2]|0;
 $108 = (($105) + 4)|0;
 $109 = $108;
 $110 = HEAP32[$109>>2]|0;
 $111 = (_i64Subtract(($107|0),($110|0),($101|0),($103|0))|0);
 $112 = tempRet0;
 $113 = $104;
 $114 = $113;
 HEAP32[$114>>2] = $111;
 $115 = (($113) + 4)|0;
 $116 = $115;
 HEAP32[$116>>2] = $112;
 $117 = HEAP32[$0>>2]|0;
 $118 = (($117) + 40|0);
 HEAP8[$118>>0] = 0;
 $119 = HEAP32[$0>>2]|0;
 $120 = (($119) + 41|0);
 HEAP8[$120>>0] = 0;
 $121 = HEAP32[$0>>2]|0;
 $122 = (($121) + 43|0);
 HEAP8[$122>>0] = 0;
 STACKTOP = sp;return;
}
function _computeYMD_HMS($p) {
 $p = $p|0;
 var $0 = 0, $1 = 0, $2 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 HEAP32[$0>>2] = $p;
 $1 = HEAP32[$0>>2]|0;
 _computeYMD($1);
 $2 = HEAP32[$0>>2]|0;
 _computeHMS($2);
 STACKTOP = sp;return;
}
function _computeYMD($p) {
 $p = $p|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0.0;
 var $27 = 0.0, $28 = 0.0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0.0, $41 = 0.0, $42 = 0.0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0.0, $51 = 0.0, $52 = 0, $53 = 0, $54 = 0.0, $55 = 0.0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0;
 var $81 = 0, $82 = 0, $83 = 0, $9 = 0, $A = 0, $B = 0, $C = 0, $D = 0, $E = 0, $X1 = 0, $Z = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 28|0;
 $Z = sp + 24|0;
 $A = sp + 20|0;
 $B = sp + 16|0;
 $C = sp + 12|0;
 $D = sp + 8|0;
 $E = sp + 4|0;
 $X1 = sp;
 HEAP32[$0>>2] = $p;
 $1 = HEAP32[$0>>2]|0;
 $2 = (($1) + 40|0);
 $3 = HEAP8[$2>>0]|0;
 $4 = ($3<<24>>24)!=(0);
 if ($4) {
  STACKTOP = sp;return;
 }
 $5 = HEAP32[$0>>2]|0;
 $6 = (($5) + 42|0);
 $7 = HEAP8[$6>>0]|0;
 $8 = ($7<<24>>24)!=(0);
 $9 = HEAP32[$0>>2]|0;
 if ($8) {
  $15 = $9;
  $16 = $15;
  $17 = HEAP32[$16>>2]|0;
  $18 = (($15) + 4)|0;
  $19 = $18;
  $20 = HEAP32[$19>>2]|0;
  $21 = (_i64Add(($17|0),($20|0),43200000,0)|0);
  $22 = tempRet0;
  $23 = (___divdi3(($21|0),($22|0),86400000,0)|0);
  $24 = tempRet0;
  HEAP32[$Z>>2] = $23;
  $25 = HEAP32[$Z>>2]|0;
  $26 = (+($25|0));
  $27 = $26 - 1867216.25;
  $28 = $27 / 36524.25;
  $29 = (~~(($28)));
  HEAP32[$A>>2] = $29;
  $30 = HEAP32[$Z>>2]|0;
  $31 = (($30) + 1)|0;
  $32 = HEAP32[$A>>2]|0;
  $33 = (($31) + ($32))|0;
  $34 = HEAP32[$A>>2]|0;
  $35 = (($34|0) / 4)&-1;
  $36 = (($33) - ($35))|0;
  HEAP32[$A>>2] = $36;
  $37 = HEAP32[$A>>2]|0;
  $38 = (($37) + 1524)|0;
  HEAP32[$B>>2] = $38;
  $39 = HEAP32[$B>>2]|0;
  $40 = (+($39|0));
  $41 = $40 - 122.09999999999999;
  $42 = $41 / 365.25;
  $43 = (~~(($42)));
  HEAP32[$C>>2] = $43;
  $44 = HEAP32[$C>>2]|0;
  $45 = ($44*36525)|0;
  $46 = (($45|0) / 100)&-1;
  HEAP32[$D>>2] = $46;
  $47 = HEAP32[$B>>2]|0;
  $48 = HEAP32[$D>>2]|0;
  $49 = (($47) - ($48))|0;
  $50 = (+($49|0));
  $51 = $50 / 30.600100000000001;
  $52 = (~~(($51)));
  HEAP32[$E>>2] = $52;
  $53 = HEAP32[$E>>2]|0;
  $54 = (+($53|0));
  $55 = 30.600100000000001 * $54;
  $56 = (~~(($55)));
  HEAP32[$X1>>2] = $56;
  $57 = HEAP32[$B>>2]|0;
  $58 = HEAP32[$D>>2]|0;
  $59 = (($57) - ($58))|0;
  $60 = HEAP32[$X1>>2]|0;
  $61 = (($59) - ($60))|0;
  $62 = HEAP32[$0>>2]|0;
  $63 = (($62) + 16|0);
  HEAP32[$63>>2] = $61;
  $64 = HEAP32[$E>>2]|0;
  $65 = ($64|0)<(14);
  $66 = HEAP32[$E>>2]|0;
  $67 = (($66) - 1)|0;
  $68 = (($66) - 13)|0;
  $69 = $65 ? $67 : $68;
  $70 = HEAP32[$0>>2]|0;
  $71 = (($70) + 12|0);
  HEAP32[$71>>2] = $69;
  $72 = HEAP32[$0>>2]|0;
  $73 = (($72) + 12|0);
  $74 = HEAP32[$73>>2]|0;
  $75 = ($74|0)>(2);
  $76 = HEAP32[$C>>2]|0;
  $77 = (($76) - 4716)|0;
  $78 = (($76) - 4715)|0;
  $79 = $75 ? $77 : $78;
  $80 = HEAP32[$0>>2]|0;
  $81 = (($80) + 8|0);
  HEAP32[$81>>2] = $79;
 } else {
  $10 = (($9) + 8|0);
  HEAP32[$10>>2] = 2000;
  $11 = HEAP32[$0>>2]|0;
  $12 = (($11) + 12|0);
  HEAP32[$12>>2] = 1;
  $13 = HEAP32[$0>>2]|0;
  $14 = (($13) + 16|0);
  HEAP32[$14>>2] = 1;
 }
 $82 = HEAP32[$0>>2]|0;
 $83 = (($82) + 40|0);
 HEAP8[$83>>0] = 1;
 STACKTOP = sp;return;
}
function _computeHMS($p) {
 $p = $p|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0.0, $19 = 0.0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0.0, $25 = 0, $26 = 0;
 var $27 = 0.0, $28 = 0, $29 = 0, $3 = 0, $30 = 0.0, $31 = 0.0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0.0, $53 = 0, $54 = 0, $55 = 0.0, $56 = 0.0, $57 = 0, $58 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $s = 0;
 var label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $s = sp;
 HEAP32[$0>>2] = $p;
 $1 = HEAP32[$0>>2]|0;
 $2 = (($1) + 41|0);
 $3 = HEAP8[$2>>0]|0;
 $4 = ($3<<24>>24)!=(0);
 if ($4) {
  STACKTOP = sp;return;
 }
 $5 = HEAP32[$0>>2]|0;
 _computeJD($5);
 $6 = HEAP32[$0>>2]|0;
 $7 = $6;
 $8 = $7;
 $9 = HEAP32[$8>>2]|0;
 $10 = (($7) + 4)|0;
 $11 = $10;
 $12 = HEAP32[$11>>2]|0;
 $13 = (_i64Add(($9|0),($12|0),43200000,0)|0);
 $14 = tempRet0;
 $15 = (___remdi3(($13|0),($14|0),86400000,0)|0);
 $16 = tempRet0;
 HEAP32[$s>>2] = $15;
 $17 = HEAP32[$s>>2]|0;
 $18 = (+($17|0));
 $19 = $18 / 1000.0;
 $20 = HEAP32[$0>>2]|0;
 $21 = (($20) + 32|0);
 HEAPF64[$21>>3] = $19;
 $22 = HEAP32[$0>>2]|0;
 $23 = (($22) + 32|0);
 $24 = +HEAPF64[$23>>3];
 $25 = (~~(($24)));
 HEAP32[$s>>2] = $25;
 $26 = HEAP32[$s>>2]|0;
 $27 = (+($26|0));
 $28 = HEAP32[$0>>2]|0;
 $29 = (($28) + 32|0);
 $30 = +HEAPF64[$29>>3];
 $31 = $30 - $27;
 HEAPF64[$29>>3] = $31;
 $32 = HEAP32[$s>>2]|0;
 $33 = (($32|0) / 3600)&-1;
 $34 = HEAP32[$0>>2]|0;
 $35 = (($34) + 20|0);
 HEAP32[$35>>2] = $33;
 $36 = HEAP32[$0>>2]|0;
 $37 = (($36) + 20|0);
 $38 = HEAP32[$37>>2]|0;
 $39 = ($38*3600)|0;
 $40 = HEAP32[$s>>2]|0;
 $41 = (($40) - ($39))|0;
 HEAP32[$s>>2] = $41;
 $42 = HEAP32[$s>>2]|0;
 $43 = (($42|0) / 60)&-1;
 $44 = HEAP32[$0>>2]|0;
 $45 = (($44) + 24|0);
 HEAP32[$45>>2] = $43;
 $46 = HEAP32[$s>>2]|0;
 $47 = HEAP32[$0>>2]|0;
 $48 = (($47) + 24|0);
 $49 = HEAP32[$48>>2]|0;
 $50 = ($49*60)|0;
 $51 = (($46) - ($50))|0;
 $52 = (+($51|0));
 $53 = HEAP32[$0>>2]|0;
 $54 = (($53) + 32|0);
 $55 = +HEAPF64[$54>>3];
 $56 = $55 + $52;
 HEAPF64[$54>>3] = $56;
 $57 = HEAP32[$0>>2]|0;
 $58 = (($57) + 41|0);
 HEAP8[$58>>0] = 1;
 STACKTOP = sp;return;
}
function _setDateTimeToCurrent($context,$p) {
 $context = $context|0;
 $p = $p|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $3 = 0;
 var $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $1 = sp + 4|0;
 $2 = sp;
 HEAP32[$1>>2] = $context;
 HEAP32[$2>>2] = $p;
 $3 = HEAP32[$1>>2]|0;
 $4 = (_sqlite3StmtCurrentTime($3)|0);
 $5 = tempRet0;
 $6 = HEAP32[$2>>2]|0;
 $7 = $6;
 $8 = $7;
 HEAP32[$8>>2] = $4;
 $9 = (($7) + 4)|0;
 $10 = $9;
 HEAP32[$10>>2] = $5;
 $11 = HEAP32[$2>>2]|0;
 $12 = $11;
 $13 = $12;
 $14 = HEAP32[$13>>2]|0;
 $15 = (($12) + 4)|0;
 $16 = $15;
 $17 = HEAP32[$16>>2]|0;
 $18 = ($17|0)>(0);
 $19 = ($14>>>0)>(0);
 $20 = ($17|0)==(0);
 $21 = $20 & $19;
 $22 = $18 | $21;
 if ($22) {
  $23 = HEAP32[$2>>2]|0;
  $24 = (($23) + 42|0);
  HEAP8[$24>>0] = 1;
  HEAP32[$0>>2] = 0;
  $25 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($25|0);
 } else {
  HEAP32[$0>>2] = 1;
  $25 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($25|0);
 }
 return 0|0;
}
function _parseDateOrTime($context,$zDate,$p) {
 $context = $context|0;
 $zDate = $zDate|0;
 $p = $p|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0.0, $24 = 0.0, $25 = 0.0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $r = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 20|0;
 $1 = sp + 16|0;
 $2 = sp + 12|0;
 $3 = sp + 8|0;
 $r = sp;
 HEAP32[$1>>2] = $context;
 HEAP32[$2>>2] = $zDate;
 HEAP32[$3>>2] = $p;
 $4 = HEAP32[$2>>2]|0;
 $5 = HEAP32[$3>>2]|0;
 $6 = (_parseYyyyMmDd($4,$5)|0);
 $7 = ($6|0)==(0);
 if ($7) {
  HEAP32[$0>>2] = 0;
  $35 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($35|0);
 }
 $8 = HEAP32[$2>>2]|0;
 $9 = HEAP32[$3>>2]|0;
 $10 = (_parseHhMmSs($8,$9)|0);
 $11 = ($10|0)==(0);
 if ($11) {
  HEAP32[$0>>2] = 0;
  $35 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($35|0);
 }
 $12 = HEAP32[$2>>2]|0;
 $13 = (_sqlite3_stricmp($12,9616)|0);
 $14 = ($13|0)==(0);
 if ($14) {
  $15 = HEAP32[$1>>2]|0;
  $16 = HEAP32[$3>>2]|0;
  $17 = (_setDateTimeToCurrent($15,$16)|0);
  HEAP32[$0>>2] = $17;
  $35 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($35|0);
 }
 $18 = HEAP32[$2>>2]|0;
 $19 = HEAP32[$2>>2]|0;
 $20 = (_sqlite3Strlen30($19)|0);
 $21 = (_sqlite3AtoF($18,$r,$20,1)|0);
 $22 = ($21|0)!=(0);
 if ($22) {
  $23 = +HEAPF64[$r>>3];
  $24 = $23 * 8.64E+7;
  $25 = $24 + 0.5;
  $26 = (~~$25)>>>0;
  $27 = +Math_abs($25) >= 1.0 ? $25 > 0.0 ? (Math_min(+Math_floor($25 / 4294967296.0), 4294967295.0) | 0) >>> 0 : ~~+Math_ceil(($25 - +(~~$25 >>> 0)) / 4294967296.0) >>> 0 : 0;
  $28 = HEAP32[$3>>2]|0;
  $29 = $28;
  $30 = $29;
  HEAP32[$30>>2] = $26;
  $31 = (($29) + 4)|0;
  $32 = $31;
  HEAP32[$32>>2] = $27;
  $33 = HEAP32[$3>>2]|0;
  $34 = (($33) + 42|0);
  HEAP8[$34>>0] = 1;
  HEAP32[$0>>2] = 0;
  $35 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($35|0);
 } else {
  HEAP32[$0>>2] = 1;
  $35 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($35|0);
 }
 return 0|0;
}
function _parseModifier($pCtx,$zMod,$p) {
 $pCtx = $pCtx|0;
 $zMod = $zMod|0;
 $p = $p|0;
 var $$sink = 0, $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0;
 var $115 = 0, $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0;
 var $133 = 0, $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0.0, $148 = 0, $149 = 0.0, $15 = 0, $150 = 0.0;
 var $151 = 0, $152 = 0, $153 = 0, $154 = 0.0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0;
 var $17 = 0, $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0;
 var $188 = 0, $189 = 0, $19 = 0, $190 = 0, $191 = 0, $192 = 0, $193 = 0, $194 = 0, $195 = 0, $196 = 0, $197 = 0, $198 = 0, $199 = 0, $2 = 0, $20 = 0, $200 = 0, $201 = 0, $202 = 0, $203 = 0, $204 = 0;
 var $205 = 0, $206 = 0, $207 = 0, $208 = 0, $209 = 0, $21 = 0, $210 = 0, $211 = 0, $212 = 0, $213 = 0, $214 = 0, $215 = 0, $216 = 0, $217 = 0, $218 = 0, $219 = 0, $22 = 0, $220 = 0, $221 = 0, $222 = 0;
 var $223 = 0, $224 = 0, $225 = 0, $226 = 0, $227 = 0, $228 = 0, $229 = 0, $23 = 0, $230 = 0, $231 = 0, $232 = 0, $233 = 0, $234 = 0, $235 = 0, $236 = 0, $237 = 0, $238 = 0, $239 = 0, $24 = 0, $240 = 0;
 var $241 = 0, $242 = 0, $243 = 0, $244 = 0, $245 = 0, $246 = 0, $247 = 0, $248 = 0, $249 = 0, $25 = 0, $250 = 0, $251 = 0, $252 = 0, $253 = 0, $254 = 0, $255 = 0, $256 = 0, $257 = 0, $258 = 0, $259 = 0;
 var $26 = 0, $260 = 0, $261 = 0, $262 = 0, $263 = 0, $264 = 0, $265 = 0, $266 = 0, $267 = 0, $268 = 0, $269 = 0, $27 = 0, $270 = 0, $271 = 0, $272 = 0, $273 = 0, $274 = 0, $275 = 0, $276 = 0, $277 = 0;
 var $278 = 0, $279 = 0, $28 = 0, $280 = 0, $281 = 0, $282 = 0, $283 = 0, $284 = 0, $285 = 0, $286 = 0, $287 = 0, $288 = 0, $289 = 0, $29 = 0, $290 = 0, $291 = 0, $292 = 0, $293 = 0, $294 = 0, $295 = 0;
 var $296 = 0, $297 = 0, $298 = 0, $299 = 0, $3 = 0, $30 = 0, $300 = 0, $301 = 0, $302 = 0, $303 = 0, $304 = 0, $305 = 0, $306 = 0, $307 = 0, $308 = 0, $309 = 0, $31 = 0, $310 = 0, $311 = 0, $312 = 0;
 var $313 = 0, $314 = 0, $315 = 0, $316 = 0, $317 = 0, $318 = 0, $319 = 0, $32 = 0, $320 = 0, $321 = 0, $322 = 0, $323 = 0, $324 = 0, $325 = 0, $326 = 0, $327 = 0, $328 = 0, $329 = 0, $33 = 0, $330 = 0;
 var $331 = 0, $332 = 0, $333 = 0, $334 = 0, $335 = 0, $336 = 0, $337 = 0, $338 = 0, $339 = 0, $34 = 0, $340 = 0, $341 = 0, $342 = 0, $343 = 0, $344 = 0, $345 = 0, $346 = 0, $347 = 0, $348 = 0, $349 = 0;
 var $35 = 0, $350 = 0, $351 = 0, $352 = 0, $353 = 0, $354 = 0, $355 = 0, $356 = 0, $357 = 0, $358 = 0, $359 = 0, $36 = 0, $360 = 0, $361 = 0, $362 = 0, $363 = 0, $364 = 0, $365 = 0, $366 = 0, $367 = 0;
 var $368 = 0, $369 = 0, $37 = 0, $370 = 0, $371 = 0, $372 = 0, $373 = 0, $374 = 0, $375 = 0, $376 = 0, $377 = 0, $378 = 0, $379 = 0, $38 = 0, $380 = 0, $381 = 0, $382 = 0, $383 = 0, $384 = 0, $385 = 0;
 var $386 = 0, $387 = 0, $388 = 0, $389 = 0, $39 = 0, $390 = 0, $391 = 0, $392 = 0, $393 = 0, $394 = 0, $395 = 0, $396 = 0, $397 = 0, $398 = 0, $399 = 0, $4 = 0, $40 = 0, $400 = 0, $401 = 0, $402 = 0;
 var $403 = 0, $404 = 0, $405 = 0, $406 = 0, $407 = 0, $408 = 0, $409 = 0, $41 = 0, $410 = 0, $411 = 0, $412 = 0, $413 = 0, $414 = 0, $415 = 0, $416 = 0, $417 = 0, $418 = 0, $419 = 0, $42 = 0, $420 = 0;
 var $421 = 0, $422 = 0, $423 = 0, $424 = 0, $425 = 0, $426 = 0, $427 = 0.0, $428 = 0, $429 = 0.0, $43 = 0, $430 = 0, $431 = 0, $432 = 0, $433 = 0, $434 = 0, $435 = 0.0, $436 = 0.0, $437 = 0.0, $438 = 0.0, $439 = 0;
 var $44 = 0, $440 = 0, $441 = 0, $442 = 0, $443 = 0, $444 = 0, $445 = 0, $446 = 0, $447 = 0, $448 = 0, $449 = 0, $45 = 0, $450 = 0, $451 = 0, $452 = 0, $453 = 0, $454 = 0, $455 = 0, $456 = 0, $457 = 0;
 var $458 = 0, $459 = 0.0, $46 = 0, $460 = 0.0, $461 = 0.0, $462 = 0.0, $463 = 0, $464 = 0, $465 = 0, $466 = 0, $467 = 0, $468 = 0, $469 = 0, $47 = 0, $470 = 0, $471 = 0, $472 = 0, $473 = 0, $474 = 0, $475 = 0;
 var $476 = 0, $477 = 0, $478 = 0, $479 = 0, $48 = 0, $480 = 0, $481 = 0, $482 = 0, $483 = 0.0, $484 = 0.0, $485 = 0.0, $486 = 0.0, $487 = 0, $488 = 0, $489 = 0, $49 = 0, $490 = 0, $491 = 0, $492 = 0, $493 = 0;
 var $494 = 0, $495 = 0, $496 = 0, $497 = 0, $498 = 0, $499 = 0, $5 = 0, $50 = 0, $500 = 0, $501 = 0, $502 = 0, $503 = 0, $504 = 0, $505 = 0, $506 = 0, $507 = 0.0, $508 = 0.0, $509 = 0.0, $51 = 0, $510 = 0.0;
 var $511 = 0, $512 = 0, $513 = 0, $514 = 0, $515 = 0, $516 = 0, $517 = 0, $518 = 0, $519 = 0, $52 = 0, $520 = 0, $521 = 0, $522 = 0, $523 = 0, $524 = 0, $525 = 0, $526 = 0, $527 = 0, $528 = 0, $529 = 0;
 var $53 = 0, $530 = 0, $531 = 0, $532 = 0.0, $533 = 0, $534 = 0, $535 = 0, $536 = 0, $537 = 0, $538 = 0, $539 = 0, $54 = 0, $540 = 0, $541 = 0, $542 = 0, $543 = 0, $544 = 0, $545 = 0, $546 = 0, $547 = 0;
 var $548 = 0, $549 = 0, $55 = 0, $550 = 0, $551 = 0, $552 = 0, $553 = 0, $554 = 0, $555 = 0, $556 = 0, $557 = 0, $558 = 0, $559 = 0, $56 = 0, $560 = 0, $561 = 0, $562 = 0.0, $563 = 0, $564 = 0, $565 = 0.0;
 var $566 = 0.0, $567 = 0, $568 = 0.0, $569 = 0, $57 = 0, $570 = 0.0, $571 = 0.0, $572 = 0.0, $573 = 0.0, $574 = 0.0, $575 = 0.0, $576 = 0, $577 = 0, $578 = 0, $579 = 0, $58 = 0, $580 = 0, $581 = 0, $582 = 0, $583 = 0;
 var $584 = 0, $585 = 0, $586 = 0, $587 = 0, $588 = 0, $589 = 0, $59 = 0, $590 = 0, $591 = 0, $592 = 0, $593 = 0, $594 = 0, $595 = 0, $596 = 0.0, $597 = 0, $598 = 0, $599 = 0, $6 = 0, $60 = 0, $600 = 0;
 var $601 = 0, $602 = 0, $603 = 0, $604 = 0, $605 = 0, $606 = 0, $607 = 0, $608 = 0.0, $609 = 0.0, $61 = 0, $610 = 0, $611 = 0.0, $612 = 0, $613 = 0.0, $614 = 0.0, $615 = 0.0, $616 = 0.0, $617 = 0.0, $618 = 0.0, $619 = 0;
 var $62 = 0, $620 = 0, $621 = 0, $622 = 0, $623 = 0, $624 = 0, $625 = 0, $626 = 0, $627 = 0, $628 = 0, $629 = 0, $63 = 0, $630 = 0, $631 = 0, $632 = 0, $633 = 0, $634 = 0, $635 = 0, $64 = 0, $65 = 0;
 var $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0;
 var $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $Z = 0, $c1 = 0, $day = 0;
 var $n = 0, $or$cond = 0, $or$cond3 = 0, $or$cond5 = 0, $r = 0, $rRounder = 0, $rc = 0, $tx = 0, $x = 0, $y = 0, $y1 = 0, $z = 0, $z2 = 0, $zBuf = 0, dest = 0, label = 0, sp = 0, stop = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 160|0;
 $0 = sp + 124|0;
 $1 = sp + 120|0;
 $2 = sp + 116|0;
 $rc = sp + 112|0;
 $n = sp + 108|0;
 $r = sp + 80|0;
 $z = sp + 104|0;
 $zBuf = sp + 128|0;
 $c1 = sp + 72|0;
 $Z = sp + 64|0;
 $rRounder = sp + 56|0;
 $z2 = sp + 100|0;
 $tx = sp + 8|0;
 $day = sp;
 $x = sp + 96|0;
 $y = sp + 92|0;
 $y1 = sp + 88|0;
 HEAP32[$0>>2] = $pCtx;
 HEAP32[$1>>2] = $zMod;
 HEAP32[$2>>2] = $p;
 HEAP32[$rc>>2] = 1;
 HEAP32[$z>>2] = $zBuf;
 HEAP32[$n>>2] = 0;
 while(1) {
  $3 = HEAP32[$n>>2]|0;
  $4 = ($3|0)<(29);
  if (!($4)) {
   break;
  }
  $5 = HEAP32[$n>>2]|0;
  $6 = HEAP32[$1>>2]|0;
  $7 = (($6) + ($5)|0);
  $8 = HEAP8[$7>>0]|0;
  $9 = $8 << 24 >> 24;
  $10 = ($9|0)!=(0);
  if (!($10)) {
   break;
  }
  $11 = HEAP32[$n>>2]|0;
  $12 = HEAP32[$1>>2]|0;
  $13 = (($12) + ($11)|0);
  $14 = HEAP8[$13>>0]|0;
  $15 = $14&255;
  $16 = (440 + ($15)|0);
  $17 = HEAP8[$16>>0]|0;
  $18 = HEAP32[$n>>2]|0;
  $19 = HEAP32[$z>>2]|0;
  $20 = (($19) + ($18)|0);
  HEAP8[$20>>0] = $17;
  $21 = HEAP32[$n>>2]|0;
  $22 = (($21) + 1)|0;
  HEAP32[$n>>2] = $22;
 }
 $23 = HEAP32[$n>>2]|0;
 $24 = HEAP32[$z>>2]|0;
 $25 = (($24) + ($23)|0);
 HEAP8[$25>>0] = 0;
 $26 = HEAP32[$z>>2]|0;
 $27 = HEAP8[$26>>0]|0;
 $28 = $27 << 24 >> 24;
 switch ($28|0) {
 case 57: case 56: case 55: case 54: case 53: case 52: case 51: case 50: case 49: case 48: case 45: case 43:  {
  HEAP32[$n>>2] = 1;
  while(1) {
   $266 = HEAP32[$n>>2]|0;
   $267 = HEAP32[$z>>2]|0;
   $268 = (($267) + ($266)|0);
   $269 = HEAP8[$268>>0]|0;
   $270 = $269 << 24 >> 24;
   $271 = ($270|0)!=(0);
   if (!($271)) {
    break;
   }
   $272 = HEAP32[$n>>2]|0;
   $273 = HEAP32[$z>>2]|0;
   $274 = (($273) + ($272)|0);
   $275 = HEAP8[$274>>0]|0;
   $276 = $275 << 24 >> 24;
   $277 = ($276|0)!=(58);
   if (!($277)) {
    break;
   }
   $278 = HEAP32[$n>>2]|0;
   $279 = HEAP32[$z>>2]|0;
   $280 = (($279) + ($278)|0);
   $281 = HEAP8[$280>>0]|0;
   $282 = $281&255;
   $283 = (1224 + ($282)|0);
   $284 = HEAP8[$283>>0]|0;
   $285 = $284&255;
   $286 = $285 & 1;
   $287 = ($286|0)!=(0);
   if ($287) {
    break;
   }
   $288 = HEAP32[$n>>2]|0;
   $289 = (($288) + 1)|0;
   HEAP32[$n>>2] = $289;
  }
  $290 = HEAP32[$z>>2]|0;
  $291 = HEAP32[$n>>2]|0;
  $292 = (_sqlite3AtoF($290,$r,$291,1)|0);
  $293 = ($292|0)!=(0);
  if (!($293)) {
   HEAP32[$rc>>2] = 1;
   $635 = HEAP32[$rc>>2]|0;
   STACKTOP = sp;return ($635|0);
  }
  $294 = HEAP32[$n>>2]|0;
  $295 = HEAP32[$z>>2]|0;
  $296 = (($295) + ($294)|0);
  $297 = HEAP8[$296>>0]|0;
  $298 = $297 << 24 >> 24;
  $299 = ($298|0)==(58);
  if ($299) {
   $300 = HEAP32[$z>>2]|0;
   HEAP32[$z2>>2] = $300;
   $301 = HEAP32[$z2>>2]|0;
   $302 = HEAP8[$301>>0]|0;
   $303 = $302&255;
   $304 = (1224 + ($303)|0);
   $305 = HEAP8[$304>>0]|0;
   $306 = $305&255;
   $307 = $306 & 4;
   $308 = ($307|0)!=(0);
   if (!($308)) {
    $309 = HEAP32[$z2>>2]|0;
    $310 = (($309) + 1|0);
    HEAP32[$z2>>2] = $310;
   }
   dest=$tx+0|0; stop=dest+48|0; do { HEAP32[dest>>2]=0|0; dest=dest+4|0; } while ((dest|0) < (stop|0));
   $311 = HEAP32[$z2>>2]|0;
   $312 = (_parseHhMmSs($311,$tx)|0);
   $313 = ($312|0)!=(0);
   if ($313) {
    $635 = HEAP32[$rc>>2]|0;
    STACKTOP = sp;return ($635|0);
   }
   _computeJD($tx);
   $314 = $tx;
   $315 = $314;
   $316 = HEAP32[$315>>2]|0;
   $317 = (($314) + 4)|0;
   $318 = $317;
   $319 = HEAP32[$318>>2]|0;
   $320 = (_i64Subtract(($316|0),($319|0),43200000,0)|0);
   $321 = tempRet0;
   $322 = $tx;
   $323 = $322;
   HEAP32[$323>>2] = $320;
   $324 = (($322) + 4)|0;
   $325 = $324;
   HEAP32[$325>>2] = $321;
   $326 = $tx;
   $327 = $326;
   $328 = HEAP32[$327>>2]|0;
   $329 = (($326) + 4)|0;
   $330 = $329;
   $331 = HEAP32[$330>>2]|0;
   $332 = (___divdi3(($328|0),($331|0),86400000,0)|0);
   $333 = tempRet0;
   $334 = $day;
   $335 = $334;
   HEAP32[$335>>2] = $332;
   $336 = (($334) + 4)|0;
   $337 = $336;
   HEAP32[$337>>2] = $333;
   $338 = $day;
   $339 = $338;
   $340 = HEAP32[$339>>2]|0;
   $341 = (($338) + 4)|0;
   $342 = $341;
   $343 = HEAP32[$342>>2]|0;
   $344 = (___muldi3(($340|0),($343|0),86400000,0)|0);
   $345 = tempRet0;
   $346 = $tx;
   $347 = $346;
   $348 = HEAP32[$347>>2]|0;
   $349 = (($346) + 4)|0;
   $350 = $349;
   $351 = HEAP32[$350>>2]|0;
   $352 = (_i64Subtract(($348|0),($351|0),($344|0),($345|0))|0);
   $353 = tempRet0;
   $354 = $tx;
   $355 = $354;
   HEAP32[$355>>2] = $352;
   $356 = (($354) + 4)|0;
   $357 = $356;
   HEAP32[$357>>2] = $353;
   $358 = HEAP32[$z>>2]|0;
   $359 = HEAP8[$358>>0]|0;
   $360 = $359 << 24 >> 24;
   $361 = ($360|0)==(45);
   if ($361) {
    $362 = $tx;
    $363 = $362;
    $364 = HEAP32[$363>>2]|0;
    $365 = (($362) + 4)|0;
    $366 = $365;
    $367 = HEAP32[$366>>2]|0;
    $368 = (_i64Subtract(0,0,($364|0),($367|0))|0);
    $369 = tempRet0;
    $370 = $tx;
    $371 = $370;
    HEAP32[$371>>2] = $368;
    $372 = (($370) + 4)|0;
    $373 = $372;
    HEAP32[$373>>2] = $369;
   }
   $374 = HEAP32[$2>>2]|0;
   _computeJD($374);
   $375 = HEAP32[$2>>2]|0;
   _clearYMD_HMS_TZ($375);
   $376 = $tx;
   $377 = $376;
   $378 = HEAP32[$377>>2]|0;
   $379 = (($376) + 4)|0;
   $380 = $379;
   $381 = HEAP32[$380>>2]|0;
   $382 = HEAP32[$2>>2]|0;
   $383 = $382;
   $384 = $383;
   $385 = HEAP32[$384>>2]|0;
   $386 = (($383) + 4)|0;
   $387 = $386;
   $388 = HEAP32[$387>>2]|0;
   $389 = (_i64Add(($385|0),($388|0),($378|0),($381|0))|0);
   $390 = tempRet0;
   $391 = $382;
   $392 = $391;
   HEAP32[$392>>2] = $389;
   $393 = (($391) + 4)|0;
   $394 = $393;
   HEAP32[$394>>2] = $390;
   HEAP32[$rc>>2] = 0;
   $635 = HEAP32[$rc>>2]|0;
   STACKTOP = sp;return ($635|0);
  }
  $395 = HEAP32[$n>>2]|0;
  $396 = HEAP32[$z>>2]|0;
  $397 = (($396) + ($395)|0);
  HEAP32[$z>>2] = $397;
  while(1) {
   $398 = HEAP32[$z>>2]|0;
   $399 = HEAP8[$398>>0]|0;
   $400 = $399&255;
   $401 = (1224 + ($400)|0);
   $402 = HEAP8[$401>>0]|0;
   $403 = $402&255;
   $404 = $403 & 1;
   $405 = ($404|0)!=(0);
   $406 = HEAP32[$z>>2]|0;
   if (!($405)) {
    break;
   }
   $407 = (($406) + 1|0);
   HEAP32[$z>>2] = $407;
  }
  $408 = (_sqlite3Strlen30($406)|0);
  HEAP32[$n>>2] = $408;
  $409 = HEAP32[$n>>2]|0;
  $410 = ($409|0)>(10);
  $411 = HEAP32[$n>>2]|0;
  $412 = ($411|0)<(3);
  $or$cond5 = $410 | $412;
  if ($or$cond5) {
   $635 = HEAP32[$rc>>2]|0;
   STACKTOP = sp;return ($635|0);
  }
  $413 = HEAP32[$n>>2]|0;
  $414 = (($413) - 1)|0;
  $415 = HEAP32[$z>>2]|0;
  $416 = (($415) + ($414)|0);
  $417 = HEAP8[$416>>0]|0;
  $418 = $417 << 24 >> 24;
  $419 = ($418|0)==(115);
  if ($419) {
   $420 = HEAP32[$n>>2]|0;
   $421 = (($420) - 1)|0;
   $422 = HEAP32[$z>>2]|0;
   $423 = (($422) + ($421)|0);
   HEAP8[$423>>0] = 0;
   $424 = HEAP32[$n>>2]|0;
   $425 = (($424) + -1)|0;
   HEAP32[$n>>2] = $425;
  }
  $426 = HEAP32[$2>>2]|0;
  _computeJD($426);
  HEAP32[$rc>>2] = 0;
  $427 = +HEAPF64[$r>>3];
  $428 = $427 < 0.0;
  $429 = $428 ? -0.5 : 0.5;
  HEAPF64[$rRounder>>3] = $429;
  $430 = HEAP32[$n>>2]|0;
  $431 = ($430|0)==(3);
  if ($431) {
   $432 = HEAP32[$z>>2]|0;
   $433 = (_strcmp($432,9560)|0);
   $434 = ($433|0)==(0);
   if ($434) {
    $435 = +HEAPF64[$r>>3];
    $436 = $435 * 8.64E+7;
    $437 = +HEAPF64[$rRounder>>3];
    $438 = $436 + $437;
    $439 = (~~$438)>>>0;
    $440 = +Math_abs($438) >= 1.0 ? $438 > 0.0 ? (Math_min(+Math_floor($438 / 4294967296.0), 4294967295.0) | 0) >>> 0 : ~~+Math_ceil(($438 - +(~~$438 >>> 0)) / 4294967296.0) >>> 0 : 0;
    $441 = HEAP32[$2>>2]|0;
    $442 = $441;
    $443 = $442;
    $444 = HEAP32[$443>>2]|0;
    $445 = (($442) + 4)|0;
    $446 = $445;
    $447 = HEAP32[$446>>2]|0;
    $448 = (_i64Add(($444|0),($447|0),($439|0),($440|0))|0);
    $449 = tempRet0;
    $450 = $441;
    $451 = $450;
    HEAP32[$451>>2] = $448;
    $452 = (($450) + 4)|0;
    $453 = $452;
    HEAP32[$453>>2] = $449;
   } else {
    label = 50;
   }
  } else {
   label = 50;
  }
  do {
   if ((label|0) == 50) {
    $454 = HEAP32[$n>>2]|0;
    $455 = ($454|0)==(4);
    if ($455) {
     $456 = HEAP32[$z>>2]|0;
     $457 = (_strcmp($456,9568)|0);
     $458 = ($457|0)==(0);
     if ($458) {
      $459 = +HEAPF64[$r>>3];
      $460 = $459 * 3.6E+6;
      $461 = +HEAPF64[$rRounder>>3];
      $462 = $460 + $461;
      $463 = (~~$462)>>>0;
      $464 = +Math_abs($462) >= 1.0 ? $462 > 0.0 ? (Math_min(+Math_floor($462 / 4294967296.0), 4294967295.0) | 0) >>> 0 : ~~+Math_ceil(($462 - +(~~$462 >>> 0)) / 4294967296.0) >>> 0 : 0;
      $465 = HEAP32[$2>>2]|0;
      $466 = $465;
      $467 = $466;
      $468 = HEAP32[$467>>2]|0;
      $469 = (($466) + 4)|0;
      $470 = $469;
      $471 = HEAP32[$470>>2]|0;
      $472 = (_i64Add(($468|0),($471|0),($463|0),($464|0))|0);
      $473 = tempRet0;
      $474 = $465;
      $475 = $474;
      HEAP32[$475>>2] = $472;
      $476 = (($474) + 4)|0;
      $477 = $476;
      HEAP32[$477>>2] = $473;
      break;
     }
    }
    $478 = HEAP32[$n>>2]|0;
    $479 = ($478|0)==(6);
    if ($479) {
     $480 = HEAP32[$z>>2]|0;
     $481 = (_strcmp($480,9576)|0);
     $482 = ($481|0)==(0);
     if ($482) {
      $483 = +HEAPF64[$r>>3];
      $484 = $483 * 6.0E+4;
      $485 = +HEAPF64[$rRounder>>3];
      $486 = $484 + $485;
      $487 = (~~$486)>>>0;
      $488 = +Math_abs($486) >= 1.0 ? $486 > 0.0 ? (Math_min(+Math_floor($486 / 4294967296.0), 4294967295.0) | 0) >>> 0 : ~~+Math_ceil(($486 - +(~~$486 >>> 0)) / 4294967296.0) >>> 0 : 0;
      $489 = HEAP32[$2>>2]|0;
      $490 = $489;
      $491 = $490;
      $492 = HEAP32[$491>>2]|0;
      $493 = (($490) + 4)|0;
      $494 = $493;
      $495 = HEAP32[$494>>2]|0;
      $496 = (_i64Add(($492|0),($495|0),($487|0),($488|0))|0);
      $497 = tempRet0;
      $498 = $489;
      $499 = $498;
      HEAP32[$499>>2] = $496;
      $500 = (($498) + 4)|0;
      $501 = $500;
      HEAP32[$501>>2] = $497;
      break;
     }
    }
    $502 = HEAP32[$n>>2]|0;
    $503 = ($502|0)==(6);
    if ($503) {
     $504 = HEAP32[$z>>2]|0;
     $505 = (_strcmp($504,9584)|0);
     $506 = ($505|0)==(0);
     if ($506) {
      $507 = +HEAPF64[$r>>3];
      $508 = $507 * 1000.0;
      $509 = +HEAPF64[$rRounder>>3];
      $510 = $508 + $509;
      $511 = (~~$510)>>>0;
      $512 = +Math_abs($510) >= 1.0 ? $510 > 0.0 ? (Math_min(+Math_floor($510 / 4294967296.0), 4294967295.0) | 0) >>> 0 : ~~+Math_ceil(($510 - +(~~$510 >>> 0)) / 4294967296.0) >>> 0 : 0;
      $513 = HEAP32[$2>>2]|0;
      $514 = $513;
      $515 = $514;
      $516 = HEAP32[$515>>2]|0;
      $517 = (($514) + 4)|0;
      $518 = $517;
      $519 = HEAP32[$518>>2]|0;
      $520 = (_i64Add(($516|0),($519|0),($511|0),($512|0))|0);
      $521 = tempRet0;
      $522 = $513;
      $523 = $522;
      HEAP32[$523>>2] = $520;
      $524 = (($522) + 4)|0;
      $525 = $524;
      HEAP32[$525>>2] = $521;
      break;
     }
    }
    $526 = HEAP32[$n>>2]|0;
    $527 = ($526|0)==(5);
    if ($527) {
     $528 = HEAP32[$z>>2]|0;
     $529 = (_strcmp($528,9544)|0);
     $530 = ($529|0)==(0);
     if ($530) {
      $531 = HEAP32[$2>>2]|0;
      _computeYMD_HMS($531);
      $532 = +HEAPF64[$r>>3];
      $533 = (~~(($532)));
      $534 = HEAP32[$2>>2]|0;
      $535 = (($534) + 12|0);
      $536 = HEAP32[$535>>2]|0;
      $537 = (($536) + ($533))|0;
      HEAP32[$535>>2] = $537;
      $538 = HEAP32[$2>>2]|0;
      $539 = (($538) + 12|0);
      $540 = HEAP32[$539>>2]|0;
      $541 = ($540|0)>(0);
      $542 = HEAP32[$2>>2]|0;
      $543 = (($542) + 12|0);
      $544 = HEAP32[$543>>2]|0;
      $545 = (($544) - 1)|0;
      $546 = (($544) - 12)|0;
      $$sink = $541 ? $545 : $546;
      $547 = (($$sink|0) / 12)&-1;
      HEAP32[$x>>2] = $547;
      $548 = HEAP32[$x>>2]|0;
      $549 = HEAP32[$2>>2]|0;
      $550 = (($549) + 8|0);
      $551 = HEAP32[$550>>2]|0;
      $552 = (($551) + ($548))|0;
      HEAP32[$550>>2] = $552;
      $553 = HEAP32[$x>>2]|0;
      $554 = ($553*12)|0;
      $555 = HEAP32[$2>>2]|0;
      $556 = (($555) + 12|0);
      $557 = HEAP32[$556>>2]|0;
      $558 = (($557) - ($554))|0;
      HEAP32[$556>>2] = $558;
      $559 = HEAP32[$2>>2]|0;
      $560 = (($559) + 42|0);
      HEAP8[$560>>0] = 0;
      $561 = HEAP32[$2>>2]|0;
      _computeJD($561);
      $562 = +HEAPF64[$r>>3];
      $563 = (~~(($562)));
      HEAP32[$y>>2] = $563;
      $564 = HEAP32[$y>>2]|0;
      $565 = (+($564|0));
      $566 = +HEAPF64[$r>>3];
      $567 = $565 != $566;
      if (!($567)) {
       break;
      }
      $568 = +HEAPF64[$r>>3];
      $569 = HEAP32[$y>>2]|0;
      $570 = (+($569|0));
      $571 = $568 - $570;
      $572 = $571 * 30.0;
      $573 = $572 * 8.64E+7;
      $574 = +HEAPF64[$rRounder>>3];
      $575 = $573 + $574;
      $576 = (~~$575)>>>0;
      $577 = +Math_abs($575) >= 1.0 ? $575 > 0.0 ? (Math_min(+Math_floor($575 / 4294967296.0), 4294967295.0) | 0) >>> 0 : ~~+Math_ceil(($575 - +(~~$575 >>> 0)) / 4294967296.0) >>> 0 : 0;
      $578 = HEAP32[$2>>2]|0;
      $579 = $578;
      $580 = $579;
      $581 = HEAP32[$580>>2]|0;
      $582 = (($579) + 4)|0;
      $583 = $582;
      $584 = HEAP32[$583>>2]|0;
      $585 = (_i64Add(($581|0),($584|0),($576|0),($577|0))|0);
      $586 = tempRet0;
      $587 = $578;
      $588 = $587;
      HEAP32[$588>>2] = $585;
      $589 = (($587) + 4)|0;
      $590 = $589;
      HEAP32[$590>>2] = $586;
      break;
     }
    }
    $591 = HEAP32[$n>>2]|0;
    $592 = ($591|0)==(4);
    if ($592) {
     $593 = HEAP32[$z>>2]|0;
     $594 = (_strcmp($593,9552)|0);
     $595 = ($594|0)==(0);
     if ($595) {
      $596 = +HEAPF64[$r>>3];
      $597 = (~~(($596)));
      HEAP32[$y1>>2] = $597;
      $598 = HEAP32[$2>>2]|0;
      _computeYMD_HMS($598);
      $599 = HEAP32[$y1>>2]|0;
      $600 = HEAP32[$2>>2]|0;
      $601 = (($600) + 8|0);
      $602 = HEAP32[$601>>2]|0;
      $603 = (($602) + ($599))|0;
      HEAP32[$601>>2] = $603;
      $604 = HEAP32[$2>>2]|0;
      $605 = (($604) + 42|0);
      HEAP8[$605>>0] = 0;
      $606 = HEAP32[$2>>2]|0;
      _computeJD($606);
      $607 = HEAP32[$y1>>2]|0;
      $608 = (+($607|0));
      $609 = +HEAPF64[$r>>3];
      $610 = $608 != $609;
      if (!($610)) {
       break;
      }
      $611 = +HEAPF64[$r>>3];
      $612 = HEAP32[$y1>>2]|0;
      $613 = (+($612|0));
      $614 = $611 - $613;
      $615 = $614 * 365.0;
      $616 = $615 * 8.64E+7;
      $617 = +HEAPF64[$rRounder>>3];
      $618 = $616 + $617;
      $619 = (~~$618)>>>0;
      $620 = +Math_abs($618) >= 1.0 ? $618 > 0.0 ? (Math_min(+Math_floor($618 / 4294967296.0), 4294967295.0) | 0) >>> 0 : ~~+Math_ceil(($618 - +(~~$618 >>> 0)) / 4294967296.0) >>> 0 : 0;
      $621 = HEAP32[$2>>2]|0;
      $622 = $621;
      $623 = $622;
      $624 = HEAP32[$623>>2]|0;
      $625 = (($622) + 4)|0;
      $626 = $625;
      $627 = HEAP32[$626>>2]|0;
      $628 = (_i64Add(($624|0),($627|0),($619|0),($620|0))|0);
      $629 = tempRet0;
      $630 = $621;
      $631 = $630;
      HEAP32[$631>>2] = $628;
      $632 = (($630) + 4)|0;
      $633 = $632;
      HEAP32[$633>>2] = $629;
      break;
     }
    }
    HEAP32[$rc>>2] = 1;
   }
  } while(0);
  $634 = HEAP32[$2>>2]|0;
  _clearYMD_HMS_TZ($634);
  $635 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($635|0);
  break;
 }
 case 115:  {
  $232 = HEAP32[$z>>2]|0;
  $233 = (_strncmp($232,9528,9)|0);
  $234 = ($233|0)!=(0);
  if ($234) {
   $635 = HEAP32[$rc>>2]|0;
   STACKTOP = sp;return ($635|0);
  }
  $235 = HEAP32[$z>>2]|0;
  $236 = (($235) + 9|0);
  HEAP32[$z>>2] = $236;
  $237 = HEAP32[$2>>2]|0;
  _computeYMD($237);
  $238 = HEAP32[$2>>2]|0;
  $239 = (($238) + 41|0);
  HEAP8[$239>>0] = 1;
  $240 = HEAP32[$2>>2]|0;
  $241 = (($240) + 24|0);
  HEAP32[$241>>2] = 0;
  $242 = HEAP32[$2>>2]|0;
  $243 = (($242) + 20|0);
  HEAP32[$243>>2] = 0;
  $244 = HEAP32[$2>>2]|0;
  $245 = (($244) + 32|0);
  HEAPF64[$245>>3] = 0.0;
  $246 = HEAP32[$2>>2]|0;
  $247 = (($246) + 43|0);
  HEAP8[$247>>0] = 0;
  $248 = HEAP32[$2>>2]|0;
  $249 = (($248) + 42|0);
  HEAP8[$249>>0] = 0;
  $250 = HEAP32[$z>>2]|0;
  $251 = (_strcmp($250,9544)|0);
  $252 = ($251|0)==(0);
  if ($252) {
   $253 = HEAP32[$2>>2]|0;
   $254 = (($253) + 16|0);
   HEAP32[$254>>2] = 1;
   HEAP32[$rc>>2] = 0;
   $635 = HEAP32[$rc>>2]|0;
   STACKTOP = sp;return ($635|0);
  }
  $255 = HEAP32[$z>>2]|0;
  $256 = (_strcmp($255,9552)|0);
  $257 = ($256|0)==(0);
  if ($257) {
   $258 = HEAP32[$2>>2]|0;
   _computeYMD($258);
   $259 = HEAP32[$2>>2]|0;
   $260 = (($259) + 12|0);
   HEAP32[$260>>2] = 1;
   $261 = HEAP32[$2>>2]|0;
   $262 = (($261) + 16|0);
   HEAP32[$262>>2] = 1;
   HEAP32[$rc>>2] = 0;
   $635 = HEAP32[$rc>>2]|0;
   STACKTOP = sp;return ($635|0);
  }
  $263 = HEAP32[$z>>2]|0;
  $264 = (_strcmp($263,9560)|0);
  $265 = ($264|0)==(0);
  if (!($265)) {
   $635 = HEAP32[$rc>>2]|0;
   STACKTOP = sp;return ($635|0);
  }
  HEAP32[$rc>>2] = 0;
  $635 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($635|0);
  break;
 }
 case 117:  {
  $51 = HEAP32[$z>>2]|0;
  $52 = (_strcmp($51,9488)|0);
  $53 = ($52|0)==(0);
  if ($53) {
   $54 = HEAP32[$2>>2]|0;
   $55 = (($54) + 42|0);
   $56 = HEAP8[$55>>0]|0;
   $57 = $56 << 24 >> 24;
   $58 = ($57|0)!=(0);
   if ($58) {
    $59 = HEAP32[$2>>2]|0;
    $60 = $59;
    $61 = $60;
    $62 = HEAP32[$61>>2]|0;
    $63 = (($60) + 4)|0;
    $64 = $63;
    $65 = HEAP32[$64>>2]|0;
    $66 = (_i64Add(($62|0),($65|0),43200,0)|0);
    $67 = tempRet0;
    $68 = (___divdi3(($66|0),($67|0),86400,0)|0);
    $69 = tempRet0;
    $70 = (_i64Add(($68|0),($69|0),1045635584,49096)|0);
    $71 = tempRet0;
    $72 = HEAP32[$2>>2]|0;
    $73 = $72;
    $74 = $73;
    HEAP32[$74>>2] = $70;
    $75 = (($73) + 4)|0;
    $76 = $75;
    HEAP32[$76>>2] = $71;
    $77 = HEAP32[$2>>2]|0;
    _clearYMD_HMS_TZ($77);
    HEAP32[$rc>>2] = 0;
    $635 = HEAP32[$rc>>2]|0;
    STACKTOP = sp;return ($635|0);
   }
  }
  $78 = HEAP32[$z>>2]|0;
  $79 = (_strcmp($78,9504)|0);
  $80 = ($79|0)==(0);
  if (!($80)) {
   $635 = HEAP32[$rc>>2]|0;
   STACKTOP = sp;return ($635|0);
  }
  $81 = HEAP32[$2>>2]|0;
  _computeJD($81);
  $82 = HEAP32[$2>>2]|0;
  $83 = HEAP32[$0>>2]|0;
  $84 = (_localtimeOffset($82,$83,$rc)|0);
  $85 = tempRet0;
  $86 = $c1;
  $87 = $86;
  HEAP32[$87>>2] = $84;
  $88 = (($86) + 4)|0;
  $89 = $88;
  HEAP32[$89>>2] = $85;
  $90 = HEAP32[$rc>>2]|0;
  $91 = ($90|0)==(0);
  if (!($91)) {
   $635 = HEAP32[$rc>>2]|0;
   STACKTOP = sp;return ($635|0);
  }
  $92 = $c1;
  $93 = $92;
  $94 = HEAP32[$93>>2]|0;
  $95 = (($92) + 4)|0;
  $96 = $95;
  $97 = HEAP32[$96>>2]|0;
  $98 = HEAP32[$2>>2]|0;
  $99 = $98;
  $100 = $99;
  $101 = HEAP32[$100>>2]|0;
  $102 = (($99) + 4)|0;
  $103 = $102;
  $104 = HEAP32[$103>>2]|0;
  $105 = (_i64Subtract(($101|0),($104|0),($94|0),($97|0))|0);
  $106 = tempRet0;
  $107 = $98;
  $108 = $107;
  HEAP32[$108>>2] = $105;
  $109 = (($107) + 4)|0;
  $110 = $109;
  HEAP32[$110>>2] = $106;
  $111 = HEAP32[$2>>2]|0;
  _clearYMD_HMS_TZ($111);
  $112 = $c1;
  $113 = $112;
  $114 = HEAP32[$113>>2]|0;
  $115 = (($112) + 4)|0;
  $116 = $115;
  $117 = HEAP32[$116>>2]|0;
  $118 = HEAP32[$2>>2]|0;
  $119 = HEAP32[$0>>2]|0;
  $120 = (_localtimeOffset($118,$119,$rc)|0);
  $121 = tempRet0;
  $122 = (_i64Subtract(($114|0),($117|0),($120|0),($121|0))|0);
  $123 = tempRet0;
  $124 = HEAP32[$2>>2]|0;
  $125 = $124;
  $126 = $125;
  $127 = HEAP32[$126>>2]|0;
  $128 = (($125) + 4)|0;
  $129 = $128;
  $130 = HEAP32[$129>>2]|0;
  $131 = (_i64Add(($127|0),($130|0),($122|0),($123|0))|0);
  $132 = tempRet0;
  $133 = $124;
  $134 = $133;
  HEAP32[$134>>2] = $131;
  $135 = (($133) + 4)|0;
  $136 = $135;
  HEAP32[$136>>2] = $132;
  $635 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($635|0);
  break;
 }
 case 119:  {
  $137 = HEAP32[$z>>2]|0;
  $138 = (_strncmp($137,9512,8)|0);
  $139 = ($138|0)==(0);
  if (!($139)) {
   $635 = HEAP32[$rc>>2]|0;
   STACKTOP = sp;return ($635|0);
  }
  $140 = HEAP32[$z>>2]|0;
  $141 = (($140) + 8|0);
  $142 = HEAP32[$z>>2]|0;
  $143 = (($142) + 8|0);
  $144 = (_sqlite3Strlen30($143)|0);
  $145 = (_sqlite3AtoF($141,$r,$144,1)|0);
  $146 = ($145|0)!=(0);
  if (!($146)) {
   $635 = HEAP32[$rc>>2]|0;
   STACKTOP = sp;return ($635|0);
  }
  $147 = +HEAPF64[$r>>3];
  $148 = (~~(($147)));
  HEAP32[$n>>2] = $148;
  $149 = (+($148|0));
  $150 = +HEAPF64[$r>>3];
  $151 = $149 == $150;
  $152 = HEAP32[$n>>2]|0;
  $153 = ($152|0)>=(0);
  $or$cond = $151 & $153;
  $154 = +HEAPF64[$r>>3];
  $155 = $154 < 7.0;
  $or$cond3 = $or$cond & $155;
  if (!($or$cond3)) {
   $635 = HEAP32[$rc>>2]|0;
   STACKTOP = sp;return ($635|0);
  }
  $156 = HEAP32[$2>>2]|0;
  _computeYMD_HMS($156);
  $157 = HEAP32[$2>>2]|0;
  $158 = (($157) + 43|0);
  HEAP8[$158>>0] = 0;
  $159 = HEAP32[$2>>2]|0;
  $160 = (($159) + 42|0);
  HEAP8[$160>>0] = 0;
  $161 = HEAP32[$2>>2]|0;
  _computeJD($161);
  $162 = HEAP32[$2>>2]|0;
  $163 = $162;
  $164 = $163;
  $165 = HEAP32[$164>>2]|0;
  $166 = (($163) + 4)|0;
  $167 = $166;
  $168 = HEAP32[$167>>2]|0;
  $169 = (_i64Add(($165|0),($168|0),129600000,0)|0);
  $170 = tempRet0;
  $171 = (___divdi3(($169|0),($170|0),86400000,0)|0);
  $172 = tempRet0;
  $173 = (___remdi3(($171|0),($172|0),7,0)|0);
  $174 = tempRet0;
  $175 = $Z;
  $176 = $175;
  HEAP32[$176>>2] = $173;
  $177 = (($175) + 4)|0;
  $178 = $177;
  HEAP32[$178>>2] = $174;
  $179 = $Z;
  $180 = $179;
  $181 = HEAP32[$180>>2]|0;
  $182 = (($179) + 4)|0;
  $183 = $182;
  $184 = HEAP32[$183>>2]|0;
  $185 = HEAP32[$n>>2]|0;
  $186 = ($185|0)<(0);
  $187 = $186 << 31 >> 31;
  $188 = ($184|0)>($187|0);
  $189 = ($181>>>0)>($185>>>0);
  $190 = ($184|0)==($187|0);
  $191 = $190 & $189;
  $192 = $188 | $191;
  if ($192) {
   $193 = $Z;
   $194 = $193;
   $195 = HEAP32[$194>>2]|0;
   $196 = (($193) + 4)|0;
   $197 = $196;
   $198 = HEAP32[$197>>2]|0;
   $199 = (_i64Subtract(($195|0),($198|0),7,0)|0);
   $200 = tempRet0;
   $201 = $Z;
   $202 = $201;
   HEAP32[$202>>2] = $199;
   $203 = (($201) + 4)|0;
   $204 = $203;
   HEAP32[$204>>2] = $200;
  }
  $205 = HEAP32[$n>>2]|0;
  $206 = ($205|0)<(0);
  $207 = $206 << 31 >> 31;
  $208 = $Z;
  $209 = $208;
  $210 = HEAP32[$209>>2]|0;
  $211 = (($208) + 4)|0;
  $212 = $211;
  $213 = HEAP32[$212>>2]|0;
  $214 = (_i64Subtract(($205|0),($207|0),($210|0),($213|0))|0);
  $215 = tempRet0;
  $216 = (___muldi3(($214|0),($215|0),86400000,0)|0);
  $217 = tempRet0;
  $218 = HEAP32[$2>>2]|0;
  $219 = $218;
  $220 = $219;
  $221 = HEAP32[$220>>2]|0;
  $222 = (($219) + 4)|0;
  $223 = $222;
  $224 = HEAP32[$223>>2]|0;
  $225 = (_i64Add(($221|0),($224|0),($216|0),($217|0))|0);
  $226 = tempRet0;
  $227 = $218;
  $228 = $227;
  HEAP32[$228>>2] = $225;
  $229 = (($227) + 4)|0;
  $230 = $229;
  HEAP32[$230>>2] = $226;
  $231 = HEAP32[$2>>2]|0;
  _clearYMD_HMS_TZ($231);
  HEAP32[$rc>>2] = 0;
  $635 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($635|0);
  break;
 }
 case 108:  {
  $29 = HEAP32[$z>>2]|0;
  $30 = (_strcmp($29,9472)|0);
  $31 = ($30|0)==(0);
  if (!($31)) {
   $635 = HEAP32[$rc>>2]|0;
   STACKTOP = sp;return ($635|0);
  }
  $32 = HEAP32[$2>>2]|0;
  _computeJD($32);
  $33 = HEAP32[$2>>2]|0;
  $34 = HEAP32[$0>>2]|0;
  $35 = (_localtimeOffset($33,$34,$rc)|0);
  $36 = tempRet0;
  $37 = HEAP32[$2>>2]|0;
  $38 = $37;
  $39 = $38;
  $40 = HEAP32[$39>>2]|0;
  $41 = (($38) + 4)|0;
  $42 = $41;
  $43 = HEAP32[$42>>2]|0;
  $44 = (_i64Add(($40|0),($43|0),($35|0),($36|0))|0);
  $45 = tempRet0;
  $46 = $37;
  $47 = $46;
  HEAP32[$47>>2] = $44;
  $48 = (($46) + 4)|0;
  $49 = $48;
  HEAP32[$49>>2] = $45;
  $50 = HEAP32[$2>>2]|0;
  _clearYMD_HMS_TZ($50);
  $635 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($635|0);
  break;
 }
 default: {
  $635 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($635|0);
 }
 }
 return 0|0;
}
function _localtimeOffset($p,$pCtx,$pRc) {
 $p = $p|0;
 $pCtx = $pCtx|0;
 $pRc = $pRc|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0.0, $19 = 0.0, $2 = 0, $20 = 0, $21 = 0, $22 = 0.0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0.0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0;
 var $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $s = 0, $sLocal = 0, $t = 0, $x = 0, $y = 0, dest = 0, label = 0;
 var sp = 0, src = 0, stop = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 176|0;
 $0 = sp + 96|0;
 $1 = sp + 164|0;
 $2 = sp + 160|0;
 $3 = sp + 156|0;
 $x = sp + 48|0;
 $y = sp;
 $t = sp + 152|0;
 $sLocal = sp + 108|0;
 $s = sp + 104|0;
 HEAP32[$1>>2] = $p;
 HEAP32[$2>>2] = $pCtx;
 HEAP32[$3>>2] = $pRc;
 dest=$sLocal+0|0; stop=dest+44|0; do { HEAP32[dest>>2]=0|0; dest=dest+4|0; } while ((dest|0) < (stop|0));
 $4 = HEAP32[$1>>2]|0;
 dest=$x+0|0; src=$4+0|0; stop=dest+48|0; do { HEAP32[dest>>2]=HEAP32[src>>2]|0; dest=dest+4|0; src=src+4|0; } while ((dest|0) < (stop|0));
 _computeYMD_HMS($x);
 $5 = (($x) + 8|0);
 $6 = HEAP32[$5>>2]|0;
 $7 = ($6|0)<(1971);
 if ($7) {
  label = 3;
 } else {
  $8 = (($x) + 8|0);
  $9 = HEAP32[$8>>2]|0;
  $10 = ($9|0)>=(2038);
  if ($10) {
   label = 3;
  } else {
   $17 = (($x) + 32|0);
   $18 = +HEAPF64[$17>>3];
   $19 = $18 + 0.5;
   $20 = (~~(($19)));
   HEAP32[$s>>2] = $20;
   $21 = HEAP32[$s>>2]|0;
   $22 = (+($21|0));
   $23 = (($x) + 32|0);
   HEAPF64[$23>>3] = $22;
  }
 }
 if ((label|0) == 3) {
  $11 = (($x) + 8|0);
  HEAP32[$11>>2] = 2000;
  $12 = (($x) + 12|0);
  HEAP32[$12>>2] = 1;
  $13 = (($x) + 16|0);
  HEAP32[$13>>2] = 1;
  $14 = (($x) + 20|0);
  HEAP32[$14>>2] = 0;
  $15 = (($x) + 24|0);
  HEAP32[$15>>2] = 0;
  $16 = (($x) + 32|0);
  HEAPF64[$16>>3] = 0.0;
 }
 $24 = (($x) + 28|0);
 HEAP32[$24>>2] = 0;
 $25 = (($x) + 42|0);
 HEAP8[$25>>0] = 0;
 _computeJD($x);
 $26 = $x;
 $27 = $26;
 $28 = HEAP32[$27>>2]|0;
 $29 = (($26) + 4)|0;
 $30 = $29;
 $31 = HEAP32[$30>>2]|0;
 $32 = (___divdi3(($28|0),($31|0),1000,0)|0);
 $33 = tempRet0;
 $34 = (_i64Subtract(($32|0),($33|0),413362496,49)|0);
 $35 = tempRet0;
 HEAP32[$t>>2] = $34;
 $36 = (_osLocaltime($t,$sLocal)|0);
 $37 = ($36|0)!=(0);
 if ($37) {
  $38 = HEAP32[$2>>2]|0;
  _sqlite3_result_error($38,9592,-1);
  $39 = HEAP32[$3>>2]|0;
  HEAP32[$39>>2] = 1;
  $40 = $0;
  $41 = $40;
  HEAP32[$41>>2] = 0;
  $42 = (($40) + 4)|0;
  $43 = $42;
  HEAP32[$43>>2] = 0;
  $87 = $0;
  $88 = $87;
  $89 = HEAP32[$88>>2]|0;
  $90 = (($87) + 4)|0;
  $91 = $90;
  $92 = HEAP32[$91>>2]|0;
  tempRet0 = $92;
  STACKTOP = sp;return ($89|0);
 } else {
  $44 = (($sLocal) + 20|0);
  $45 = HEAP32[$44>>2]|0;
  $46 = (($45) + 1900)|0;
  $47 = (($y) + 8|0);
  HEAP32[$47>>2] = $46;
  $48 = (($sLocal) + 16|0);
  $49 = HEAP32[$48>>2]|0;
  $50 = (($49) + 1)|0;
  $51 = (($y) + 12|0);
  HEAP32[$51>>2] = $50;
  $52 = (($sLocal) + 12|0);
  $53 = HEAP32[$52>>2]|0;
  $54 = (($y) + 16|0);
  HEAP32[$54>>2] = $53;
  $55 = (($sLocal) + 8|0);
  $56 = HEAP32[$55>>2]|0;
  $57 = (($y) + 20|0);
  HEAP32[$57>>2] = $56;
  $58 = (($sLocal) + 4|0);
  $59 = HEAP32[$58>>2]|0;
  $60 = (($y) + 24|0);
  HEAP32[$60>>2] = $59;
  $61 = HEAP32[$sLocal>>2]|0;
  $62 = (+($61|0));
  $63 = (($y) + 32|0);
  HEAPF64[$63>>3] = $62;
  $64 = (($y) + 40|0);
  HEAP8[$64>>0] = 1;
  $65 = (($y) + 41|0);
  HEAP8[$65>>0] = 1;
  $66 = (($y) + 42|0);
  HEAP8[$66>>0] = 0;
  $67 = (($y) + 43|0);
  HEAP8[$67>>0] = 0;
  _computeJD($y);
  $68 = HEAP32[$3>>2]|0;
  HEAP32[$68>>2] = 0;
  $69 = $y;
  $70 = $69;
  $71 = HEAP32[$70>>2]|0;
  $72 = (($69) + 4)|0;
  $73 = $72;
  $74 = HEAP32[$73>>2]|0;
  $75 = $x;
  $76 = $75;
  $77 = HEAP32[$76>>2]|0;
  $78 = (($75) + 4)|0;
  $79 = $78;
  $80 = HEAP32[$79>>2]|0;
  $81 = (_i64Subtract(($71|0),($74|0),($77|0),($80|0))|0);
  $82 = tempRet0;
  $83 = $0;
  $84 = $83;
  HEAP32[$84>>2] = $81;
  $85 = (($83) + 4)|0;
  $86 = $85;
  HEAP32[$86>>2] = $82;
  $87 = $0;
  $88 = $87;
  $89 = HEAP32[$88>>2]|0;
  $90 = (($87) + 4)|0;
  $91 = $90;
  $92 = HEAP32[$91>>2]|0;
  tempRet0 = $92;
  STACKTOP = sp;return ($89|0);
 }
 return 0|0;
}
function _clearYMD_HMS_TZ($p) {
 $p = $p|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 HEAP32[$0>>2] = $p;
 $1 = HEAP32[$0>>2]|0;
 $2 = (($1) + 40|0);
 HEAP8[$2>>0] = 0;
 $3 = HEAP32[$0>>2]|0;
 $4 = (($3) + 41|0);
 HEAP8[$4>>0] = 0;
 $5 = HEAP32[$0>>2]|0;
 $6 = (($5) + 43|0);
 HEAP8[$6>>0] = 0;
 STACKTOP = sp;return;
}
function _sqlite3AtoF($z,$pResult,$length,$enc) {
 $z = $z|0;
 $pResult = $pResult|0;
 $length = $length|0;
 $enc = $enc|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0;
 var $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0, $188 = 0;
 var $189 = 0, $19 = 0, $190 = 0, $191 = 0, $192 = 0, $193 = 0, $194 = 0, $195 = 0, $196 = 0, $197 = 0, $198 = 0, $199 = 0, $2 = 0, $20 = 0, $200 = 0, $201 = 0, $202 = 0, $203 = 0, $204 = 0, $205 = 0;
 var $206 = 0, $207 = 0, $208 = 0, $209 = 0, $21 = 0, $210 = 0, $211 = 0, $212 = 0, $213 = 0, $214 = 0, $215 = 0, $216 = 0, $217 = 0, $218 = 0, $219 = 0, $22 = 0, $220 = 0, $221 = 0, $222 = 0, $223 = 0;
 var $224 = 0, $225 = 0, $226 = 0, $227 = 0, $228 = 0, $229 = 0, $23 = 0, $230 = 0, $231 = 0, $232 = 0, $233 = 0, $234 = 0, $235 = 0, $236 = 0, $237 = 0, $238 = 0, $239 = 0, $24 = 0, $240 = 0, $241 = 0;
 var $242 = 0, $243 = 0, $244 = 0, $245 = 0, $246 = 0, $247 = 0, $248 = 0, $249 = 0, $25 = 0, $250 = 0, $251 = 0, $252 = 0, $253 = 0, $254 = 0, $255 = 0, $256 = 0, $257 = 0, $258 = 0, $259 = 0, $26 = 0;
 var $260 = 0, $261 = 0, $262 = 0, $263 = 0, $264 = 0, $265 = 0, $266 = 0, $267 = 0, $268 = 0, $269 = 0, $27 = 0, $270 = 0, $271 = 0, $272 = 0, $273 = 0, $274 = 0, $275 = 0, $276 = 0, $277 = 0, $278 = 0;
 var $279 = 0, $28 = 0, $280 = 0, $281 = 0, $282 = 0, $283 = 0, $284 = 0, $285 = 0, $286 = 0, $287 = 0, $288 = 0, $289 = 0, $29 = 0, $290 = 0, $291 = 0, $292 = 0, $293 = 0, $294 = 0, $295 = 0, $296 = 0;
 var $297 = 0, $298 = 0, $299 = 0, $3 = 0, $30 = 0, $300 = 0, $301 = 0, $302 = 0, $303 = 0, $304 = 0, $305 = 0, $306 = 0, $307 = 0, $308 = 0, $309 = 0, $31 = 0, $310 = 0, $311 = 0, $312 = 0, $313 = 0;
 var $314 = 0, $315 = 0, $316 = 0, $317 = 0, $318 = 0, $319 = 0, $32 = 0, $320 = 0, $321 = 0, $322 = 0, $323 = 0, $324 = 0.0, $325 = 0, $326 = 0, $327 = 0, $328 = 0, $329 = 0, $33 = 0, $330 = 0, $331 = 0;
 var $332 = 0, $333 = 0, $334 = 0, $335 = 0, $336 = 0, $337 = 0, $338 = 0, $339 = 0, $34 = 0, $340 = 0, $341 = 0, $342 = 0, $343 = 0, $344 = 0, $345 = 0, $346 = 0, $347 = 0, $348 = 0, $349 = 0, $35 = 0;
 var $350 = 0, $351 = 0, $352 = 0, $353 = 0, $354 = 0, $355 = 0, $356 = 0, $357 = 0, $358 = 0, $359 = 0, $36 = 0, $360 = 0, $361 = 0, $362 = 0, $363 = 0, $364 = 0, $365 = 0, $366 = 0, $367 = 0, $368 = 0;
 var $369 = 0, $37 = 0, $370 = 0, $371 = 0, $372 = 0, $373 = 0, $374 = 0, $375 = 0, $376 = 0, $377 = 0, $378 = 0, $379 = 0, $38 = 0, $380 = 0, $381 = 0, $382 = 0, $383 = 0, $384 = 0, $385 = 0, $386 = 0;
 var $387 = 0, $388 = 0, $389 = 0, $39 = 0, $390 = 0, $391 = 0, $392 = 0, $393 = 0, $394 = 0, $395 = 0, $396 = 0, $397 = 0, $398 = 0, $399 = 0, $4 = 0, $40 = 0, $400 = 0, $401 = 0, $402 = 0, $403 = 0;
 var $404 = 0, $405 = 0, $406 = 0, $407 = 0.0, $408 = 0.0, $409 = 0, $41 = 0, $410 = 0, $411 = 0, $412 = 0, $413 = 0, $414 = 0, $415 = 0, $416 = 0, $417 = 0, $418 = 0, $419 = 0.0, $42 = 0, $420 = 0.0, $421 = 0.0;
 var $422 = 0.0, $423 = 0.0, $424 = 0.0, $425 = 0.0, $426 = 0.0, $427 = 0, $428 = 0, $429 = 0, $43 = 0, $430 = 0, $431 = 0, $432 = 0, $433 = 0, $434 = 0, $435 = 0, $436 = 0, $437 = 0.0, $438 = 0.0, $439 = 0.0, $44 = 0;
 var $440 = 0, $441 = 0, $442 = 0, $443 = 0.0, $444 = 0.0, $445 = 0, $446 = 0, $447 = 0, $448 = 0, $449 = 0.0, $45 = 0, $450 = 0.0, $451 = 0, $452 = 0, $453 = 0, $454 = 0, $455 = 0, $456 = 0, $457 = 0, $458 = 0;
 var $459 = 0, $46 = 0, $460 = 0, $461 = 0.0, $462 = 0.0, $463 = 0.0, $464 = 0.0, $465 = 0, $466 = 0, $467 = 0, $468 = 0, $469 = 0, $47 = 0, $470 = 0, $471 = 0.0, $472 = 0.0, $473 = 0, $474 = 0, $475 = 0, $476 = 0;
 var $477 = 0, $478 = 0, $479 = 0, $48 = 0, $480 = 0, $481 = 0, $482 = 0, $483 = 0, $484 = 0, $485 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0;
 var $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0;
 var $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0;
 var $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $d = 0, $e = 0, $eValid = 0, $esign = 0, $i = 0, $incr = 0, $nDigits = 0, $nonNum = 0, $or$cond = 0, $or$cond14 = 0, $or$cond16 = 0, $or$cond18 = 0, $or$cond20 = 0, $or$cond22 = 0;
 var $or$cond24 = 0, $or$cond26 = 0, $result = 0, $s = 0, $scale = 0, $sign = 0, $zEnd = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 96|0;
 $0 = sp + 76|0;
 $1 = sp + 72|0;
 $2 = sp + 68|0;
 $3 = sp + 64|0;
 $4 = sp + 80|0;
 $incr = sp + 60|0;
 $zEnd = sp + 56|0;
 $sign = sp + 52|0;
 $s = sp + 16|0;
 $d = sp + 48|0;
 $esign = sp + 44|0;
 $e = sp + 40|0;
 $eValid = sp + 36|0;
 $result = sp + 8|0;
 $nDigits = sp + 32|0;
 $nonNum = sp + 28|0;
 $i = sp + 24|0;
 $scale = sp;
 HEAP32[$1>>2] = $z;
 HEAP32[$2>>2] = $pResult;
 HEAP32[$3>>2] = $length;
 HEAP8[$4>>0] = $enc;
 $5 = HEAP32[$1>>2]|0;
 $6 = HEAP32[$3>>2]|0;
 $7 = (($5) + ($6)|0);
 HEAP32[$zEnd>>2] = $7;
 HEAP32[$sign>>2] = 1;
 $8 = $s;
 $9 = $8;
 HEAP32[$9>>2] = 0;
 $10 = (($8) + 4)|0;
 $11 = $10;
 HEAP32[$11>>2] = 0;
 HEAP32[$d>>2] = 0;
 HEAP32[$esign>>2] = 1;
 HEAP32[$e>>2] = 0;
 HEAP32[$eValid>>2] = 1;
 HEAP32[$nDigits>>2] = 0;
 HEAP32[$nonNum>>2] = 0;
 $12 = HEAP32[$2>>2]|0;
 HEAPF64[$12>>3] = 0.0;
 $13 = HEAP8[$4>>0]|0;
 $14 = $13&255;
 $15 = ($14|0)==(1);
 if ($15) {
  HEAP32[$incr>>2] = 1;
 } else {
  HEAP32[$incr>>2] = 2;
  $16 = HEAP8[$4>>0]|0;
  $17 = $16&255;
  $18 = (3 - ($17))|0;
  HEAP32[$i>>2] = $18;
  while(1) {
   $19 = HEAP32[$i>>2]|0;
   $20 = HEAP32[$3>>2]|0;
   $21 = ($19|0)<($20|0);
   if (!($21)) {
    break;
   }
   $22 = HEAP32[$i>>2]|0;
   $23 = HEAP32[$1>>2]|0;
   $24 = (($23) + ($22)|0);
   $25 = HEAP8[$24>>0]|0;
   $26 = $25 << 24 >> 24;
   $27 = ($26|0)==(0);
   if (!($27)) {
    break;
   }
   $28 = HEAP32[$i>>2]|0;
   $29 = (($28) + 2)|0;
   HEAP32[$i>>2] = $29;
  }
  $30 = HEAP32[$i>>2]|0;
  $31 = HEAP32[$3>>2]|0;
  $32 = ($30|0)<($31|0);
  $33 = $32&1;
  HEAP32[$nonNum>>2] = $33;
  $34 = HEAP32[$1>>2]|0;
  $35 = HEAP32[$i>>2]|0;
  $36 = (($34) + ($35)|0);
  $37 = HEAP8[$4>>0]|0;
  $38 = $37&255;
  $39 = (($36) + ($38)|0);
  $40 = (($39) + -3|0);
  HEAP32[$zEnd>>2] = $40;
  $41 = HEAP8[$4>>0]|0;
  $42 = $41&255;
  $43 = $42 & 1;
  $44 = HEAP32[$1>>2]|0;
  $45 = (($44) + ($43)|0);
  HEAP32[$1>>2] = $45;
 }
 while(1) {
  $46 = HEAP32[$1>>2]|0;
  $47 = HEAP32[$zEnd>>2]|0;
  $48 = ($46>>>0)<($47>>>0);
  if (!($48)) {
   break;
  }
  $49 = HEAP32[$1>>2]|0;
  $50 = HEAP8[$49>>0]|0;
  $51 = $50&255;
  $52 = (1224 + ($51)|0);
  $53 = HEAP8[$52>>0]|0;
  $54 = $53&255;
  $55 = $54 & 1;
  $56 = ($55|0)!=(0);
  if (!($56)) {
   break;
  }
  $57 = HEAP32[$incr>>2]|0;
  $58 = HEAP32[$1>>2]|0;
  $59 = (($58) + ($57)|0);
  HEAP32[$1>>2] = $59;
 }
 $60 = HEAP32[$1>>2]|0;
 $61 = HEAP32[$zEnd>>2]|0;
 $62 = ($60>>>0)>=($61>>>0);
 if ($62) {
  HEAP32[$0>>2] = 0;
  $485 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($485|0);
 }
 $63 = HEAP32[$1>>2]|0;
 $64 = HEAP8[$63>>0]|0;
 $65 = $64 << 24 >> 24;
 $66 = ($65|0)==(45);
 if ($66) {
  HEAP32[$sign>>2] = -1;
  $67 = HEAP32[$incr>>2]|0;
  $68 = HEAP32[$1>>2]|0;
  $69 = (($68) + ($67)|0);
  HEAP32[$1>>2] = $69;
 } else {
  $70 = HEAP32[$1>>2]|0;
  $71 = HEAP8[$70>>0]|0;
  $72 = $71 << 24 >> 24;
  $73 = ($72|0)==(43);
  if ($73) {
   $74 = HEAP32[$incr>>2]|0;
   $75 = HEAP32[$1>>2]|0;
   $76 = (($75) + ($74)|0);
   HEAP32[$1>>2] = $76;
  }
 }
 while(1) {
  $77 = HEAP32[$1>>2]|0;
  $78 = HEAP32[$zEnd>>2]|0;
  $79 = ($77>>>0)<($78>>>0);
  if (!($79)) {
   break;
  }
  $80 = HEAP32[$1>>2]|0;
  $81 = HEAP8[$80>>0]|0;
  $82 = $81 << 24 >> 24;
  $83 = ($82|0)==(48);
  if (!($83)) {
   break;
  }
  $84 = HEAP32[$incr>>2]|0;
  $85 = HEAP32[$1>>2]|0;
  $86 = (($85) + ($84)|0);
  HEAP32[$1>>2] = $86;
  $87 = HEAP32[$nDigits>>2]|0;
  $88 = (($87) + 1)|0;
  HEAP32[$nDigits>>2] = $88;
 }
 while(1) {
  $89 = HEAP32[$1>>2]|0;
  $90 = HEAP32[$zEnd>>2]|0;
  $91 = ($89>>>0)<($90>>>0);
  if (!($91)) {
   break;
  }
  $92 = HEAP32[$1>>2]|0;
  $93 = HEAP8[$92>>0]|0;
  $94 = $93&255;
  $95 = (1224 + ($94)|0);
  $96 = HEAP8[$95>>0]|0;
  $97 = $96&255;
  $98 = $97 & 4;
  $99 = ($98|0)!=(0);
  $100 = $s;
  $101 = $100;
  $102 = HEAP32[$101>>2]|0;
  $103 = (($100) + 4)|0;
  $104 = $103;
  $105 = HEAP32[$104>>2]|0;
  $106 = ($105|0)<(214748364);
  $107 = ($102>>>0)<(3435973835);
  $108 = ($105|0)==(214748364);
  $109 = $108 & $107;
  $110 = $106 | $109;
  $or$cond20 = $99 & $110;
  if (!($or$cond20)) {
   break;
  }
  $111 = $s;
  $112 = $111;
  $113 = HEAP32[$112>>2]|0;
  $114 = (($111) + 4)|0;
  $115 = $114;
  $116 = HEAP32[$115>>2]|0;
  $117 = (___muldi3(($113|0),($116|0),10,0)|0);
  $118 = tempRet0;
  $119 = HEAP32[$1>>2]|0;
  $120 = HEAP8[$119>>0]|0;
  $121 = $120 << 24 >> 24;
  $122 = (($121) - 48)|0;
  $123 = ($122|0)<(0);
  $124 = $123 << 31 >> 31;
  $125 = (_i64Add(($117|0),($118|0),($122|0),($124|0))|0);
  $126 = tempRet0;
  $127 = $s;
  $128 = $127;
  HEAP32[$128>>2] = $125;
  $129 = (($127) + 4)|0;
  $130 = $129;
  HEAP32[$130>>2] = $126;
  $131 = HEAP32[$incr>>2]|0;
  $132 = HEAP32[$1>>2]|0;
  $133 = (($132) + ($131)|0);
  HEAP32[$1>>2] = $133;
  $134 = HEAP32[$nDigits>>2]|0;
  $135 = (($134) + 1)|0;
  HEAP32[$nDigits>>2] = $135;
 }
 while(1) {
  $136 = HEAP32[$1>>2]|0;
  $137 = HEAP32[$zEnd>>2]|0;
  $138 = ($136>>>0)<($137>>>0);
  if (!($138)) {
   break;
  }
  $139 = HEAP32[$1>>2]|0;
  $140 = HEAP8[$139>>0]|0;
  $141 = $140&255;
  $142 = (1224 + ($141)|0);
  $143 = HEAP8[$142>>0]|0;
  $144 = $143&255;
  $145 = $144 & 4;
  $146 = ($145|0)!=(0);
  if (!($146)) {
   break;
  }
  $147 = HEAP32[$incr>>2]|0;
  $148 = HEAP32[$1>>2]|0;
  $149 = (($148) + ($147)|0);
  HEAP32[$1>>2] = $149;
  $150 = HEAP32[$nDigits>>2]|0;
  $151 = (($150) + 1)|0;
  HEAP32[$nDigits>>2] = $151;
  $152 = HEAP32[$d>>2]|0;
  $153 = (($152) + 1)|0;
  HEAP32[$d>>2] = $153;
 }
 $154 = HEAP32[$1>>2]|0;
 $155 = HEAP32[$zEnd>>2]|0;
 $156 = ($154>>>0)>=($155>>>0);
 L35: do {
  if (!($156)) {
   $157 = HEAP32[$1>>2]|0;
   $158 = HEAP8[$157>>0]|0;
   $159 = $158 << 24 >> 24;
   $160 = ($159|0)==(46);
   L37: do {
    if ($160) {
     $161 = HEAP32[$incr>>2]|0;
     $162 = HEAP32[$1>>2]|0;
     $163 = (($162) + ($161)|0);
     HEAP32[$1>>2] = $163;
     while(1) {
      $164 = HEAP32[$1>>2]|0;
      $165 = HEAP32[$zEnd>>2]|0;
      $166 = ($164>>>0)<($165>>>0);
      if (!($166)) {
       break;
      }
      $167 = HEAP32[$1>>2]|0;
      $168 = HEAP8[$167>>0]|0;
      $169 = $168&255;
      $170 = (1224 + ($169)|0);
      $171 = HEAP8[$170>>0]|0;
      $172 = $171&255;
      $173 = $172 & 4;
      $174 = ($173|0)!=(0);
      $175 = $s;
      $176 = $175;
      $177 = HEAP32[$176>>2]|0;
      $178 = (($175) + 4)|0;
      $179 = $178;
      $180 = HEAP32[$179>>2]|0;
      $181 = ($180|0)<(214748364);
      $182 = ($177>>>0)<(3435973835);
      $183 = ($180|0)==(214748364);
      $184 = $183 & $182;
      $185 = $181 | $184;
      $or$cond22 = $174 & $185;
      if (!($or$cond22)) {
       break;
      }
      $186 = $s;
      $187 = $186;
      $188 = HEAP32[$187>>2]|0;
      $189 = (($186) + 4)|0;
      $190 = $189;
      $191 = HEAP32[$190>>2]|0;
      $192 = (___muldi3(($188|0),($191|0),10,0)|0);
      $193 = tempRet0;
      $194 = HEAP32[$1>>2]|0;
      $195 = HEAP8[$194>>0]|0;
      $196 = $195 << 24 >> 24;
      $197 = (($196) - 48)|0;
      $198 = ($197|0)<(0);
      $199 = $198 << 31 >> 31;
      $200 = (_i64Add(($192|0),($193|0),($197|0),($199|0))|0);
      $201 = tempRet0;
      $202 = $s;
      $203 = $202;
      HEAP32[$203>>2] = $200;
      $204 = (($202) + 4)|0;
      $205 = $204;
      HEAP32[$205>>2] = $201;
      $206 = HEAP32[$incr>>2]|0;
      $207 = HEAP32[$1>>2]|0;
      $208 = (($207) + ($206)|0);
      HEAP32[$1>>2] = $208;
      $209 = HEAP32[$nDigits>>2]|0;
      $210 = (($209) + 1)|0;
      HEAP32[$nDigits>>2] = $210;
      $211 = HEAP32[$d>>2]|0;
      $212 = (($211) + -1)|0;
      HEAP32[$d>>2] = $212;
     }
     while(1) {
      $213 = HEAP32[$1>>2]|0;
      $214 = HEAP32[$zEnd>>2]|0;
      $215 = ($213>>>0)<($214>>>0);
      if (!($215)) {
       break L37;
      }
      $216 = HEAP32[$1>>2]|0;
      $217 = HEAP8[$216>>0]|0;
      $218 = $217&255;
      $219 = (1224 + ($218)|0);
      $220 = HEAP8[$219>>0]|0;
      $221 = $220&255;
      $222 = $221 & 4;
      $223 = ($222|0)!=(0);
      if (!($223)) {
       break L37;
      }
      $224 = HEAP32[$incr>>2]|0;
      $225 = HEAP32[$1>>2]|0;
      $226 = (($225) + ($224)|0);
      HEAP32[$1>>2] = $226;
      $227 = HEAP32[$nDigits>>2]|0;
      $228 = (($227) + 1)|0;
      HEAP32[$nDigits>>2] = $228;
     }
    }
   } while(0);
   $229 = HEAP32[$1>>2]|0;
   $230 = HEAP32[$zEnd>>2]|0;
   $231 = ($229>>>0)>=($230>>>0);
   if (!($231)) {
    $232 = HEAP32[$1>>2]|0;
    $233 = HEAP8[$232>>0]|0;
    $234 = $233 << 24 >> 24;
    $235 = ($234|0)==(101);
    if ($235) {
     label = 38;
    } else {
     $236 = HEAP32[$1>>2]|0;
     $237 = HEAP8[$236>>0]|0;
     $238 = $237 << 24 >> 24;
     $239 = ($238|0)==(69);
     if ($239) {
      label = 38;
     }
    }
    L51: do {
     if ((label|0) == 38) {
      $240 = HEAP32[$incr>>2]|0;
      $241 = HEAP32[$1>>2]|0;
      $242 = (($241) + ($240)|0);
      HEAP32[$1>>2] = $242;
      HEAP32[$eValid>>2] = 0;
      $243 = HEAP32[$1>>2]|0;
      $244 = HEAP32[$zEnd>>2]|0;
      $245 = ($243>>>0)>=($244>>>0);
      if ($245) {
       break L35;
      }
      $246 = HEAP32[$1>>2]|0;
      $247 = HEAP8[$246>>0]|0;
      $248 = $247 << 24 >> 24;
      $249 = ($248|0)==(45);
      if ($249) {
       HEAP32[$esign>>2] = -1;
       $250 = HEAP32[$incr>>2]|0;
       $251 = HEAP32[$1>>2]|0;
       $252 = (($251) + ($250)|0);
       HEAP32[$1>>2] = $252;
      } else {
       $253 = HEAP32[$1>>2]|0;
       $254 = HEAP8[$253>>0]|0;
       $255 = $254 << 24 >> 24;
       $256 = ($255|0)==(43);
       if ($256) {
        $257 = HEAP32[$incr>>2]|0;
        $258 = HEAP32[$1>>2]|0;
        $259 = (($258) + ($257)|0);
        HEAP32[$1>>2] = $259;
       }
      }
      while(1) {
       $260 = HEAP32[$1>>2]|0;
       $261 = HEAP32[$zEnd>>2]|0;
       $262 = ($260>>>0)<($261>>>0);
       if (!($262)) {
        break L51;
       }
       $263 = HEAP32[$1>>2]|0;
       $264 = HEAP8[$263>>0]|0;
       $265 = $264&255;
       $266 = (1224 + ($265)|0);
       $267 = HEAP8[$266>>0]|0;
       $268 = $267&255;
       $269 = $268 & 4;
       $270 = ($269|0)!=(0);
       if (!($270)) {
        break L51;
       }
       $271 = HEAP32[$e>>2]|0;
       $272 = ($271|0)<(10000);
       if ($272) {
        $273 = HEAP32[$e>>2]|0;
        $274 = ($273*10)|0;
        $275 = HEAP32[$1>>2]|0;
        $276 = HEAP8[$275>>0]|0;
        $277 = $276 << 24 >> 24;
        $278 = (($277) - 48)|0;
        $279 = (($274) + ($278))|0;
        $280 = $279;
       } else {
        $280 = 10000;
       }
       HEAP32[$e>>2] = $280;
       $281 = HEAP32[$incr>>2]|0;
       $282 = HEAP32[$1>>2]|0;
       $283 = (($282) + ($281)|0);
       HEAP32[$1>>2] = $283;
       HEAP32[$eValid>>2] = 1;
      }
     }
    } while(0);
    $284 = HEAP32[$nDigits>>2]|0;
    $285 = ($284|0)!=(0);
    $286 = HEAP32[$eValid>>2]|0;
    $287 = ($286|0)!=(0);
    $or$cond = $285 & $287;
    if ($or$cond) {
     while(1) {
      $288 = HEAP32[$1>>2]|0;
      $289 = HEAP32[$zEnd>>2]|0;
      $290 = ($288>>>0)<($289>>>0);
      if (!($290)) {
       break L35;
      }
      $291 = HEAP32[$1>>2]|0;
      $292 = HEAP8[$291>>0]|0;
      $293 = $292&255;
      $294 = (1224 + ($293)|0);
      $295 = HEAP8[$294>>0]|0;
      $296 = $295&255;
      $297 = $296 & 1;
      $298 = ($297|0)!=(0);
      if (!($298)) {
       break L35;
      }
      $299 = HEAP32[$incr>>2]|0;
      $300 = HEAP32[$1>>2]|0;
      $301 = (($300) + ($299)|0);
      HEAP32[$1>>2] = $301;
     }
    }
   }
  }
 } while(0);
 $302 = HEAP32[$e>>2]|0;
 $303 = HEAP32[$esign>>2]|0;
 $304 = Math_imul($302, $303)|0;
 $305 = HEAP32[$d>>2]|0;
 $306 = (($304) + ($305))|0;
 HEAP32[$e>>2] = $306;
 $307 = HEAP32[$e>>2]|0;
 $308 = ($307|0)<(0);
 if ($308) {
  HEAP32[$esign>>2] = -1;
  $309 = HEAP32[$e>>2]|0;
  $310 = Math_imul($309, -1)|0;
  HEAP32[$e>>2] = $310;
 } else {
  HEAP32[$esign>>2] = 1;
 }
 $311 = $s;
 $312 = $311;
 $313 = HEAP32[$312>>2]|0;
 $314 = (($311) + 4)|0;
 $315 = $314;
 $316 = HEAP32[$315>>2]|0;
 $317 = ($313|0)!=(0);
 $318 = ($316|0)!=(0);
 $319 = $317 | $318;
 do {
  if ($319) {
   $326 = HEAP32[$esign>>2]|0;
   $327 = ($326|0)>(0);
   L77: do {
    if ($327) {
     while(1) {
      $328 = $s;
      $329 = $328;
      $330 = HEAP32[$329>>2]|0;
      $331 = (($328) + 4)|0;
      $332 = $331;
      $333 = HEAP32[$332>>2]|0;
      $334 = ($333|0)<(214748364);
      $335 = ($330>>>0)<(3435973836);
      $336 = ($333|0)==(214748364);
      $337 = $336 & $335;
      $338 = $334 | $337;
      $339 = HEAP32[$e>>2]|0;
      $340 = ($339|0)>(0);
      $or$cond24 = $338 & $340;
      if (!($or$cond24)) {
       break L77;
      }
      $341 = HEAP32[$e>>2]|0;
      $342 = (($341) + -1)|0;
      HEAP32[$e>>2] = $342;
      $343 = $s;
      $344 = $343;
      $345 = HEAP32[$344>>2]|0;
      $346 = (($343) + 4)|0;
      $347 = $346;
      $348 = HEAP32[$347>>2]|0;
      $349 = (___muldi3(($345|0),($348|0),10,0)|0);
      $350 = tempRet0;
      $351 = $s;
      $352 = $351;
      HEAP32[$352>>2] = $349;
      $353 = (($351) + 4)|0;
      $354 = $353;
      HEAP32[$354>>2] = $350;
     }
    } else {
     while(1) {
      $355 = $s;
      $356 = $355;
      $357 = HEAP32[$356>>2]|0;
      $358 = (($355) + 4)|0;
      $359 = $358;
      $360 = HEAP32[$359>>2]|0;
      $361 = (___remdi3(($357|0),($360|0),10,0)|0);
      $362 = tempRet0;
      $363 = ($361|0)==(0);
      $364 = ($362|0)==(0);
      $365 = $363 & $364;
      $366 = HEAP32[$e>>2]|0;
      $367 = ($366|0)>(0);
      $or$cond26 = $365 & $367;
      if (!($or$cond26)) {
       break L77;
      }
      $368 = HEAP32[$e>>2]|0;
      $369 = (($368) + -1)|0;
      HEAP32[$e>>2] = $369;
      $370 = $s;
      $371 = $370;
      $372 = HEAP32[$371>>2]|0;
      $373 = (($370) + 4)|0;
      $374 = $373;
      $375 = HEAP32[$374>>2]|0;
      $376 = (___divdi3(($372|0),($375|0),10,0)|0);
      $377 = tempRet0;
      $378 = $s;
      $379 = $378;
      HEAP32[$379>>2] = $376;
      $380 = (($378) + 4)|0;
      $381 = $380;
      HEAP32[$381>>2] = $377;
     }
    }
   } while(0);
   $382 = HEAP32[$sign>>2]|0;
   $383 = ($382|0)<(0);
   $384 = $s;
   $385 = $384;
   $386 = HEAP32[$385>>2]|0;
   $387 = (($384) + 4)|0;
   $388 = $387;
   $389 = HEAP32[$388>>2]|0;
   $390 = (_i64Subtract(0,0,($386|0),($389|0))|0);
   $391 = tempRet0;
   $392 = $383 ? $390 : $386;
   $393 = $383 ? $391 : $389;
   $394 = $s;
   $395 = $394;
   HEAP32[$395>>2] = $392;
   $396 = (($394) + 4)|0;
   $397 = $396;
   HEAP32[$397>>2] = $393;
   $398 = HEAP32[$e>>2]|0;
   $399 = ($398|0)!=(0);
   if (!($399)) {
    $465 = $s;
    $466 = $465;
    $467 = HEAP32[$466>>2]|0;
    $468 = (($465) + 4)|0;
    $469 = $468;
    $470 = HEAP32[$469>>2]|0;
    $471 = (+($467>>>0)) + (4294967296.0*(+($470|0)));
    HEAPF64[$result>>3] = $471;
    break;
   }
   HEAPF64[$scale>>3] = 1.0;
   $400 = HEAP32[$e>>2]|0;
   $401 = ($400|0)>(307);
   $402 = HEAP32[$e>>2]|0;
   $403 = ($402|0)<(342);
   $or$cond14 = $401 & $403;
   if ($or$cond14) {
    while(1) {
     $404 = HEAP32[$e>>2]|0;
     $405 = (($404|0) % 308)&-1;
     $406 = ($405|0)!=(0);
     if (!($406)) {
      break;
     }
     $407 = +HEAPF64[$scale>>3];
     $408 = $407 * 10.0;
     HEAPF64[$scale>>3] = $408;
     $409 = HEAP32[$e>>2]|0;
     $410 = (($409) - 1)|0;
     HEAP32[$e>>2] = $410;
    }
    $411 = HEAP32[$esign>>2]|0;
    $412 = ($411|0)<(0);
    $413 = $s;
    $414 = $413;
    $415 = HEAP32[$414>>2]|0;
    $416 = (($413) + 4)|0;
    $417 = $416;
    $418 = HEAP32[$417>>2]|0;
    $419 = (+($415>>>0)) + (4294967296.0*(+($418|0)));
    $420 = +HEAPF64[$scale>>3];
    if ($412) {
     $421 = $419 / $420;
     HEAPF64[$result>>3] = $421;
     $422 = +HEAPF64[$result>>3];
     $423 = $422 / 1.0E+308;
     HEAPF64[$result>>3] = $423;
     break;
    } else {
     $424 = $419 * $420;
     HEAPF64[$result>>3] = $424;
     $425 = +HEAPF64[$result>>3];
     $426 = $425 * 1.0E+308;
     HEAPF64[$result>>3] = $426;
     break;
    }
   }
   $427 = HEAP32[$e>>2]|0;
   $428 = ($427|0)>=(342);
   if ($428) {
    $429 = HEAP32[$esign>>2]|0;
    $430 = ($429|0)<(0);
    $431 = $s;
    $432 = $431;
    $433 = HEAP32[$432>>2]|0;
    $434 = (($431) + 4)|0;
    $435 = $434;
    $436 = HEAP32[$435>>2]|0;
    $437 = (+($433>>>0)) + (4294967296.0*(+($436|0)));
    if ($430) {
     $438 = 0.0 * $437;
     HEAPF64[$result>>3] = $438;
     break;
    } else {
     $439 = inf * $437;
     HEAPF64[$result>>3] = $439;
     break;
    }
   }
   while(1) {
    $440 = HEAP32[$e>>2]|0;
    $441 = (($440|0) % 22)&-1;
    $442 = ($441|0)!=(0);
    if (!($442)) {
     break;
    }
    $443 = +HEAPF64[$scale>>3];
    $444 = $443 * 10.0;
    HEAPF64[$scale>>3] = $444;
    $445 = HEAP32[$e>>2]|0;
    $446 = (($445) - 1)|0;
    HEAP32[$e>>2] = $446;
   }
   while(1) {
    $447 = HEAP32[$e>>2]|0;
    $448 = ($447|0)>(0);
    if (!($448)) {
     break;
    }
    $449 = +HEAPF64[$scale>>3];
    $450 = $449 * 1.0E+22;
    HEAPF64[$scale>>3] = $450;
    $451 = HEAP32[$e>>2]|0;
    $452 = (($451) - 22)|0;
    HEAP32[$e>>2] = $452;
   }
   $453 = HEAP32[$esign>>2]|0;
   $454 = ($453|0)<(0);
   $455 = $s;
   $456 = $455;
   $457 = HEAP32[$456>>2]|0;
   $458 = (($455) + 4)|0;
   $459 = $458;
   $460 = HEAP32[$459>>2]|0;
   $461 = (+($457>>>0)) + (4294967296.0*(+($460|0)));
   $462 = +HEAPF64[$scale>>3];
   if ($454) {
    $463 = $461 / $462;
    HEAPF64[$result>>3] = $463;
    break;
   } else {
    $464 = $461 * $462;
    HEAPF64[$result>>3] = $464;
    break;
   }
  } else {
   $320 = HEAP32[$sign>>2]|0;
   $321 = ($320|0)<(0);
   if ($321) {
    $322 = HEAP32[$nDigits>>2]|0;
    $323 = ($322|0)!=(0);
    $325 = $323;
   } else {
    $325 = 0;
   }
   $324 = $325 ? -0.0 : 0.0;
   HEAPF64[$result>>3] = $324;
  }
 } while(0);
 $472 = +HEAPF64[$result>>3];
 $473 = HEAP32[$2>>2]|0;
 HEAPF64[$473>>3] = $472;
 $474 = HEAP32[$1>>2]|0;
 $475 = HEAP32[$zEnd>>2]|0;
 $476 = ($474>>>0)>=($475>>>0);
 $477 = HEAP32[$nDigits>>2]|0;
 $478 = ($477|0)>(0);
 $or$cond16 = $476 & $478;
 $479 = HEAP32[$eValid>>2]|0;
 $480 = ($479|0)!=(0);
 $or$cond18 = $or$cond16 & $480;
 if ($or$cond18) {
  $481 = HEAP32[$nonNum>>2]|0;
  $482 = ($481|0)==(0);
  $484 = $482;
 } else {
  $484 = 0;
 }
 $483 = $484&1;
 HEAP32[$0>>2] = $483;
 $485 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($485|0);
}
function _parseHhMmSs($zDate,$p) {
 $zDate = $zDate|0;
 $p = $p|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0.0, $43 = 0.0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0.0, $48 = 0.0, $49 = 0.0, $5 = 0, $50 = 0.0, $51 = 0.0, $52 = 0, $53 = 0, $54 = 0.0, $55 = 0.0, $56 = 0.0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0.0, $69 = 0.0, $7 = 0, $70 = 0.0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0;
 var $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $9 = 0, $h = 0, $m = 0, $ms = 0, $rScale = 0, $s = 0, $vararg_buffer10 = 0, $vararg_ptr1 = 0, $vararg_ptr13 = 0, $vararg_ptr14 = 0, $vararg_ptr15 = 0, $vararg_ptr16 = 0, $vararg_ptr2 = 0, $vararg_ptr3 = 0, $vararg_ptr4 = 0;
 var $vararg_ptr5 = 0, $vararg_ptr6 = 0, $vararg_ptr7 = 0, $vararg_ptr8 = 0, $vararg_ptr9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 80|0;
 $vararg_buffer10 = sp + 16|0;
 $0 = sp + 76|0;
 $1 = sp + 72|0;
 $2 = sp + 68|0;
 $h = sp + 64|0;
 $m = sp + 60|0;
 $s = sp + 56|0;
 $ms = sp + 8|0;
 $rScale = sp;
 HEAP32[$1>>2] = $zDate;
 HEAP32[$2>>2] = $p;
 HEAPF64[$ms>>3] = 0.0;
 $3 = HEAP32[$1>>2]|0;
 HEAP32[$vararg_buffer10>>2] = 2;
 $vararg_ptr1 = (($vararg_buffer10) + 4|0);
 HEAP32[$vararg_ptr1>>2] = 0;
 $vararg_ptr2 = (($vararg_buffer10) + 8|0);
 HEAP32[$vararg_ptr2>>2] = 24;
 $vararg_ptr3 = (($vararg_buffer10) + 12|0);
 HEAP32[$vararg_ptr3>>2] = 58;
 $vararg_ptr4 = (($vararg_buffer10) + 16|0);
 HEAP32[$vararg_ptr4>>2] = $h;
 $vararg_ptr5 = (($vararg_buffer10) + 20|0);
 HEAP32[$vararg_ptr5>>2] = 2;
 $vararg_ptr6 = (($vararg_buffer10) + 24|0);
 HEAP32[$vararg_ptr6>>2] = 0;
 $vararg_ptr7 = (($vararg_buffer10) + 28|0);
 HEAP32[$vararg_ptr7>>2] = 59;
 $vararg_ptr8 = (($vararg_buffer10) + 32|0);
 HEAP32[$vararg_ptr8>>2] = 0;
 $vararg_ptr9 = (($vararg_buffer10) + 36|0);
 HEAP32[$vararg_ptr9>>2] = $m;
 $4 = (_getDigits($3,$vararg_buffer10)|0);
 $5 = ($4|0)!=(2);
 if ($5) {
  HEAP32[$0>>2] = 1;
  $85 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($85|0);
 }
 $6 = HEAP32[$1>>2]|0;
 $7 = (($6) + 5|0);
 HEAP32[$1>>2] = $7;
 $8 = HEAP32[$1>>2]|0;
 $9 = HEAP8[$8>>0]|0;
 $10 = $9 << 24 >> 24;
 $11 = ($10|0)==(58);
 if ($11) {
  $12 = HEAP32[$1>>2]|0;
  $13 = (($12) + 1|0);
  HEAP32[$1>>2] = $13;
  $14 = HEAP32[$1>>2]|0;
  HEAP32[$vararg_buffer10>>2] = 2;
  $vararg_ptr13 = (($vararg_buffer10) + 4|0);
  HEAP32[$vararg_ptr13>>2] = 0;
  $vararg_ptr14 = (($vararg_buffer10) + 8|0);
  HEAP32[$vararg_ptr14>>2] = 59;
  $vararg_ptr15 = (($vararg_buffer10) + 12|0);
  HEAP32[$vararg_ptr15>>2] = 0;
  $vararg_ptr16 = (($vararg_buffer10) + 16|0);
  HEAP32[$vararg_ptr16>>2] = $s;
  $15 = (_getDigits($14,$vararg_buffer10)|0);
  $16 = ($15|0)!=(1);
  if ($16) {
   HEAP32[$0>>2] = 1;
   $85 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($85|0);
  }
  $17 = HEAP32[$1>>2]|0;
  $18 = (($17) + 2|0);
  HEAP32[$1>>2] = $18;
  $19 = HEAP32[$1>>2]|0;
  $20 = HEAP8[$19>>0]|0;
  $21 = $20 << 24 >> 24;
  $22 = ($21|0)==(46);
  if ($22) {
   $23 = HEAP32[$1>>2]|0;
   $24 = (($23) + 1|0);
   $25 = HEAP8[$24>>0]|0;
   $26 = $25&255;
   $27 = (1224 + ($26)|0);
   $28 = HEAP8[$27>>0]|0;
   $29 = $28&255;
   $30 = $29 & 4;
   $31 = ($30|0)!=(0);
   if ($31) {
    HEAPF64[$rScale>>3] = 1.0;
    $32 = HEAP32[$1>>2]|0;
    $33 = (($32) + 1|0);
    HEAP32[$1>>2] = $33;
    while(1) {
     $34 = HEAP32[$1>>2]|0;
     $35 = HEAP8[$34>>0]|0;
     $36 = $35&255;
     $37 = (1224 + ($36)|0);
     $38 = HEAP8[$37>>0]|0;
     $39 = $38&255;
     $40 = $39 & 4;
     $41 = ($40|0)!=(0);
     if (!($41)) {
      break;
     }
     $42 = +HEAPF64[$ms>>3];
     $43 = $42 * 10.0;
     $44 = HEAP32[$1>>2]|0;
     $45 = HEAP8[$44>>0]|0;
     $46 = $45 << 24 >> 24;
     $47 = (+($46|0));
     $48 = $43 + $47;
     $49 = $48 - 48.0;
     HEAPF64[$ms>>3] = $49;
     $50 = +HEAPF64[$rScale>>3];
     $51 = $50 * 10.0;
     HEAPF64[$rScale>>3] = $51;
     $52 = HEAP32[$1>>2]|0;
     $53 = (($52) + 1|0);
     HEAP32[$1>>2] = $53;
    }
    $54 = +HEAPF64[$rScale>>3];
    $55 = +HEAPF64[$ms>>3];
    $56 = $55 / $54;
    HEAPF64[$ms>>3] = $56;
   }
  }
 } else {
  HEAP32[$s>>2] = 0;
 }
 $57 = HEAP32[$2>>2]|0;
 $58 = (($57) + 42|0);
 HEAP8[$58>>0] = 0;
 $59 = HEAP32[$2>>2]|0;
 $60 = (($59) + 41|0);
 HEAP8[$60>>0] = 1;
 $61 = HEAP32[$h>>2]|0;
 $62 = HEAP32[$2>>2]|0;
 $63 = (($62) + 20|0);
 HEAP32[$63>>2] = $61;
 $64 = HEAP32[$m>>2]|0;
 $65 = HEAP32[$2>>2]|0;
 $66 = (($65) + 24|0);
 HEAP32[$66>>2] = $64;
 $67 = HEAP32[$s>>2]|0;
 $68 = (+($67|0));
 $69 = +HEAPF64[$ms>>3];
 $70 = $68 + $69;
 $71 = HEAP32[$2>>2]|0;
 $72 = (($71) + 32|0);
 HEAPF64[$72>>3] = $70;
 $73 = HEAP32[$1>>2]|0;
 $74 = HEAP32[$2>>2]|0;
 $75 = (_parseTimezone($73,$74)|0);
 $76 = ($75|0)!=(0);
 if ($76) {
  HEAP32[$0>>2] = 1;
  $85 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($85|0);
 } else {
  $77 = HEAP32[$2>>2]|0;
  $78 = (($77) + 28|0);
  $79 = HEAP32[$78>>2]|0;
  $80 = ($79|0)!=(0);
  $81 = $80 ? 1 : 0;
  $82 = $81&255;
  $83 = HEAP32[$2>>2]|0;
  $84 = (($83) + 43|0);
  HEAP8[$84>>0] = $82;
  HEAP32[$0>>2] = 0;
  $85 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($85|0);
 }
 return 0|0;
}
function _getDigits($zDate,$varargs) {
 $zDate = $zDate|0;
 $varargs = $varargs|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $N = 0, $ap = 0, $arglist_current = 0, $arglist_current11 = 0, $arglist_current2 = 0, $arglist_current5 = 0, $arglist_current8 = 0, $arglist_next = 0, $arglist_next12 = 0, $arglist_next3 = 0, $arglist_next6 = 0, $arglist_next9 = 0;
 var $cnt = 0, $max = 0, $min = 0, $nextC = 0, $pVal = 0, $val = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 64|0;
 $0 = sp + 48|0;
 $ap = sp + 32|0;
 $val = sp + 24|0;
 $N = sp + 20|0;
 $min = sp + 16|0;
 $max = sp + 12|0;
 $nextC = sp + 8|0;
 $pVal = sp + 4|0;
 $cnt = sp;
 HEAP32[$0>>2] = $zDate;
 HEAP32[$cnt>>2] = 0;
 HEAP32[$ap>>2] = $varargs;
 L1: while(1) {
  $arglist_current = HEAP32[$ap>>2]|0;
  $1 = HEAP32[$arglist_current>>2]|0;
  $arglist_next = (($arglist_current) + 4|0);
  HEAP32[$ap>>2] = $arglist_next;
  HEAP32[$N>>2] = $1;
  $arglist_current2 = HEAP32[$ap>>2]|0;
  $2 = HEAP32[$arglist_current2>>2]|0;
  $arglist_next3 = (($arglist_current2) + 4|0);
  HEAP32[$ap>>2] = $arglist_next3;
  HEAP32[$min>>2] = $2;
  $arglist_current5 = HEAP32[$ap>>2]|0;
  $3 = HEAP32[$arglist_current5>>2]|0;
  $arglist_next6 = (($arglist_current5) + 4|0);
  HEAP32[$ap>>2] = $arglist_next6;
  HEAP32[$max>>2] = $3;
  $arglist_current8 = HEAP32[$ap>>2]|0;
  $4 = HEAP32[$arglist_current8>>2]|0;
  $arglist_next9 = (($arglist_current8) + 4|0);
  HEAP32[$ap>>2] = $arglist_next9;
  HEAP32[$nextC>>2] = $4;
  $arglist_current11 = HEAP32[$ap>>2]|0;
  $5 = HEAP32[$arglist_current11>>2]|0;
  $arglist_next12 = (($arglist_current11) + 4|0);
  HEAP32[$ap>>2] = $arglist_next12;
  HEAP32[$pVal>>2] = $5;
  HEAP32[$val>>2] = 0;
  while(1) {
   $6 = HEAP32[$N>>2]|0;
   $7 = (($6) + -1)|0;
   HEAP32[$N>>2] = $7;
   $8 = ($6|0)!=(0);
   if (!($8)) {
    break;
   }
   $9 = HEAP32[$0>>2]|0;
   $10 = HEAP8[$9>>0]|0;
   $11 = $10&255;
   $12 = (1224 + ($11)|0);
   $13 = HEAP8[$12>>0]|0;
   $14 = $13&255;
   $15 = $14 & 4;
   $16 = ($15|0)!=(0);
   if (!($16)) {
    label = 11;
    break L1;
   }
   $17 = HEAP32[$val>>2]|0;
   $18 = ($17*10)|0;
   $19 = HEAP32[$0>>2]|0;
   $20 = HEAP8[$19>>0]|0;
   $21 = $20 << 24 >> 24;
   $22 = (($18) + ($21))|0;
   $23 = (($22) - 48)|0;
   HEAP32[$val>>2] = $23;
   $24 = HEAP32[$0>>2]|0;
   $25 = (($24) + 1|0);
   HEAP32[$0>>2] = $25;
  }
  $26 = HEAP32[$val>>2]|0;
  $27 = HEAP32[$min>>2]|0;
  $28 = ($26|0)<($27|0);
  if ($28) {
   label = 11;
   break;
  }
  $29 = HEAP32[$val>>2]|0;
  $30 = HEAP32[$max>>2]|0;
  $31 = ($29|0)>($30|0);
  if ($31) {
   label = 11;
   break;
  }
  $32 = HEAP32[$nextC>>2]|0;
  $33 = ($32|0)!=(0);
  if ($33) {
   $34 = HEAP32[$nextC>>2]|0;
   $35 = HEAP32[$0>>2]|0;
   $36 = HEAP8[$35>>0]|0;
   $37 = $36 << 24 >> 24;
   $38 = ($34|0)!=($37|0);
   if ($38) {
    label = 11;
    break;
   }
  }
  $39 = HEAP32[$val>>2]|0;
  $40 = HEAP32[$pVal>>2]|0;
  HEAP32[$40>>2] = $39;
  $41 = HEAP32[$0>>2]|0;
  $42 = (($41) + 1|0);
  HEAP32[$0>>2] = $42;
  $43 = HEAP32[$cnt>>2]|0;
  $44 = (($43) + 1)|0;
  HEAP32[$cnt>>2] = $44;
  $45 = HEAP32[$nextC>>2]|0;
  $46 = ($45|0)!=(0);
  if (!($46)) {
   label = 11;
   break;
  }
 }
 if ((label|0) == 11) {
  $47 = HEAP32[$cnt>>2]|0;
  STACKTOP = sp;return ($47|0);
 }
 return 0|0;
}
function _parseTimezone($zDate,$p) {
 $zDate = $zDate|0;
 $p = $p|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $7 = 0, $8 = 0;
 var $9 = 0, $c = 0, $nHr = 0, $nMn = 0, $or$cond = 0, $sgn = 0, $vararg_buffer = 0, $vararg_ptr10 = 0, $vararg_ptr2 = 0, $vararg_ptr3 = 0, $vararg_ptr4 = 0, $vararg_ptr5 = 0, $vararg_ptr6 = 0, $vararg_ptr7 = 0, $vararg_ptr8 = 0, $vararg_ptr9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 80|0;
 $vararg_buffer = sp;
 $0 = sp + 64|0;
 $1 = sp + 60|0;
 $2 = sp + 56|0;
 $sgn = sp + 52|0;
 $nHr = sp + 48|0;
 $nMn = sp + 44|0;
 $c = sp + 40|0;
 HEAP32[$1>>2] = $zDate;
 HEAP32[$2>>2] = $p;
 HEAP32[$sgn>>2] = 0;
 while(1) {
  $3 = HEAP32[$1>>2]|0;
  $4 = HEAP8[$3>>0]|0;
  $5 = $4&255;
  $6 = (1224 + ($5)|0);
  $7 = HEAP8[$6>>0]|0;
  $8 = $7&255;
  $9 = $8 & 1;
  $10 = ($9|0)!=(0);
  if (!($10)) {
   break;
  }
  $11 = HEAP32[$1>>2]|0;
  $12 = (($11) + 1|0);
  HEAP32[$1>>2] = $12;
 }
 $13 = HEAP32[$2>>2]|0;
 $14 = (($13) + 28|0);
 HEAP32[$14>>2] = 0;
 $15 = HEAP32[$1>>2]|0;
 $16 = HEAP8[$15>>0]|0;
 $17 = $16 << 24 >> 24;
 HEAP32[$c>>2] = $17;
 $18 = HEAP32[$c>>2]|0;
 $19 = ($18|0)==(45);
 do {
  if ($19) {
   HEAP32[$sgn>>2] = -1;
   label = 11;
  } else {
   $20 = HEAP32[$c>>2]|0;
   $21 = ($20|0)==(43);
   if ($21) {
    HEAP32[$sgn>>2] = 1;
    label = 11;
    break;
   }
   $22 = HEAP32[$c>>2]|0;
   $23 = ($22|0)==(90);
   $24 = HEAP32[$c>>2]|0;
   $25 = ($24|0)==(122);
   $or$cond = $23 | $25;
   if ($or$cond) {
    $26 = HEAP32[$1>>2]|0;
    $27 = (($26) + 1|0);
    HEAP32[$1>>2] = $27;
    break;
   }
   $28 = HEAP32[$c>>2]|0;
   $29 = ($28|0)!=(0);
   $30 = $29&1;
   HEAP32[$0>>2] = $30;
   $60 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($60|0);
  }
 } while(0);
 do {
  if ((label|0) == 11) {
   $31 = HEAP32[$1>>2]|0;
   $32 = (($31) + 1|0);
   HEAP32[$1>>2] = $32;
   $33 = HEAP32[$1>>2]|0;
   HEAP32[$vararg_buffer>>2] = 2;
   $vararg_ptr2 = (($vararg_buffer) + 4|0);
   HEAP32[$vararg_ptr2>>2] = 0;
   $vararg_ptr3 = (($vararg_buffer) + 8|0);
   HEAP32[$vararg_ptr3>>2] = 14;
   $vararg_ptr4 = (($vararg_buffer) + 12|0);
   HEAP32[$vararg_ptr4>>2] = 58;
   $vararg_ptr5 = (($vararg_buffer) + 16|0);
   HEAP32[$vararg_ptr5>>2] = $nHr;
   $vararg_ptr6 = (($vararg_buffer) + 20|0);
   HEAP32[$vararg_ptr6>>2] = 2;
   $vararg_ptr7 = (($vararg_buffer) + 24|0);
   HEAP32[$vararg_ptr7>>2] = 0;
   $vararg_ptr8 = (($vararg_buffer) + 28|0);
   HEAP32[$vararg_ptr8>>2] = 59;
   $vararg_ptr9 = (($vararg_buffer) + 32|0);
   HEAP32[$vararg_ptr9>>2] = 0;
   $vararg_ptr10 = (($vararg_buffer) + 36|0);
   HEAP32[$vararg_ptr10>>2] = $nMn;
   $34 = (_getDigits($33,$vararg_buffer)|0);
   $35 = ($34|0)!=(2);
   if (!($35)) {
    $36 = HEAP32[$1>>2]|0;
    $37 = (($36) + 5|0);
    HEAP32[$1>>2] = $37;
    $38 = HEAP32[$sgn>>2]|0;
    $39 = HEAP32[$nMn>>2]|0;
    $40 = HEAP32[$nHr>>2]|0;
    $41 = ($40*60)|0;
    $42 = (($39) + ($41))|0;
    $43 = Math_imul($38, $42)|0;
    $44 = HEAP32[$2>>2]|0;
    $45 = (($44) + 28|0);
    HEAP32[$45>>2] = $43;
    break;
   }
   HEAP32[$0>>2] = 1;
   $60 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($60|0);
  }
 } while(0);
 while(1) {
  $46 = HEAP32[$1>>2]|0;
  $47 = HEAP8[$46>>0]|0;
  $48 = $47&255;
  $49 = (1224 + ($48)|0);
  $50 = HEAP8[$49>>0]|0;
  $51 = $50&255;
  $52 = $51 & 1;
  $53 = ($52|0)!=(0);
  $54 = HEAP32[$1>>2]|0;
  if (!($53)) {
   break;
  }
  $55 = (($54) + 1|0);
  HEAP32[$1>>2] = $55;
 }
 $56 = HEAP8[$54>>0]|0;
 $57 = $56 << 24 >> 24;
 $58 = ($57|0)!=(0);
 $59 = $58&1;
 HEAP32[$0>>2] = $59;
 $60 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($60|0);
}
function _osLocaltime($t,$pTm) {
 $t = $t|0;
 $pTm = $pTm|0;
 var $$ = 0, $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $pX = 0, $rc = 0, dest = 0, label = 0, sp = 0;
 var src = 0, stop = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $1 = sp + 8|0;
 $rc = sp + 4|0;
 $pX = sp;
 HEAP32[$0>>2] = $t;
 HEAP32[$1>>2] = $pTm;
 $2 = HEAP32[$0>>2]|0;
 $3 = (_localtime(($2|0))|0);
 HEAP32[$pX>>2] = $3;
 $4 = HEAP32[((38640 + 264|0))>>2]|0;
 $5 = ($4|0)!=(0);
 $$ = $5 ? 0 : $3;
 HEAP32[$pX>>2] = $$;
 $6 = HEAP32[$pX>>2]|0;
 $7 = ($6|0)!=(0|0);
 if ($7) {
  $8 = HEAP32[$1>>2]|0;
  $9 = HEAP32[$pX>>2]|0;
  dest=$8+0|0; src=$9+0|0; stop=dest+44|0; do { HEAP32[dest>>2]=HEAP32[src>>2]|0; dest=dest+4|0; src=src+4|0; } while ((dest|0) < (stop|0));
 }
 $10 = HEAP32[$pX>>2]|0;
 $11 = ($10|0)==(0|0);
 $12 = $11&1;
 HEAP32[$rc>>2] = $12;
 $13 = HEAP32[$rc>>2]|0;
 STACKTOP = sp;return ($13|0);
}
function _parseYyyyMmDd($zDate,$p) {
 $zDate = $zDate|0;
 $p = $p|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $7 = 0, $8 = 0;
 var $9 = 0, $D = 0, $M = 0, $Y = 0, $neg = 0, $vararg_buffer = 0, $vararg_ptr1 = 0, $vararg_ptr10 = 0, $vararg_ptr11 = 0, $vararg_ptr12 = 0, $vararg_ptr13 = 0, $vararg_ptr14 = 0, $vararg_ptr2 = 0, $vararg_ptr3 = 0, $vararg_ptr4 = 0, $vararg_ptr5 = 0, $vararg_ptr6 = 0, $vararg_ptr7 = 0, $vararg_ptr8 = 0, $vararg_ptr9 = 0;
 var label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 96|0;
 $vararg_buffer = sp;
 $0 = sp + 84|0;
 $1 = sp + 80|0;
 $2 = sp + 76|0;
 $Y = sp + 72|0;
 $M = sp + 68|0;
 $D = sp + 64|0;
 $neg = sp + 60|0;
 HEAP32[$1>>2] = $zDate;
 HEAP32[$2>>2] = $p;
 $3 = HEAP32[$1>>2]|0;
 $4 = HEAP8[$3>>0]|0;
 $5 = $4 << 24 >> 24;
 $6 = ($5|0)==(45);
 if ($6) {
  $7 = HEAP32[$1>>2]|0;
  $8 = (($7) + 1|0);
  HEAP32[$1>>2] = $8;
  HEAP32[$neg>>2] = 1;
 } else {
  HEAP32[$neg>>2] = 0;
 }
 $9 = HEAP32[$1>>2]|0;
 HEAP32[$vararg_buffer>>2] = 4;
 $vararg_ptr1 = (($vararg_buffer) + 4|0);
 HEAP32[$vararg_ptr1>>2] = 0;
 $vararg_ptr2 = (($vararg_buffer) + 8|0);
 HEAP32[$vararg_ptr2>>2] = 9999;
 $vararg_ptr3 = (($vararg_buffer) + 12|0);
 HEAP32[$vararg_ptr3>>2] = 45;
 $vararg_ptr4 = (($vararg_buffer) + 16|0);
 HEAP32[$vararg_ptr4>>2] = $Y;
 $vararg_ptr5 = (($vararg_buffer) + 20|0);
 HEAP32[$vararg_ptr5>>2] = 2;
 $vararg_ptr6 = (($vararg_buffer) + 24|0);
 HEAP32[$vararg_ptr6>>2] = 1;
 $vararg_ptr7 = (($vararg_buffer) + 28|0);
 HEAP32[$vararg_ptr7>>2] = 12;
 $vararg_ptr8 = (($vararg_buffer) + 32|0);
 HEAP32[$vararg_ptr8>>2] = 45;
 $vararg_ptr9 = (($vararg_buffer) + 36|0);
 HEAP32[$vararg_ptr9>>2] = $M;
 $vararg_ptr10 = (($vararg_buffer) + 40|0);
 HEAP32[$vararg_ptr10>>2] = 2;
 $vararg_ptr11 = (($vararg_buffer) + 44|0);
 HEAP32[$vararg_ptr11>>2] = 1;
 $vararg_ptr12 = (($vararg_buffer) + 48|0);
 HEAP32[$vararg_ptr12>>2] = 31;
 $vararg_ptr13 = (($vararg_buffer) + 52|0);
 HEAP32[$vararg_ptr13>>2] = 0;
 $vararg_ptr14 = (($vararg_buffer) + 56|0);
 HEAP32[$vararg_ptr14>>2] = $D;
 $10 = (_getDigits($9,$vararg_buffer)|0);
 $11 = ($10|0)!=(3);
 if ($11) {
  HEAP32[$0>>2] = 1;
  $59 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($59|0);
 }
 $12 = HEAP32[$1>>2]|0;
 $13 = (($12) + 10|0);
 HEAP32[$1>>2] = $13;
 while(1) {
  $14 = HEAP32[$1>>2]|0;
  $15 = HEAP8[$14>>0]|0;
  $16 = $15&255;
  $17 = (1224 + ($16)|0);
  $18 = HEAP8[$17>>0]|0;
  $19 = $18&255;
  $20 = $19 & 1;
  $21 = ($20|0)!=(0);
  if ($21) {
   $60 = 1;
  } else {
   $22 = HEAP32[$1>>2]|0;
   $23 = HEAP8[$22>>0]|0;
   $24 = $23&255;
   $25 = (84)==($24|0);
   $60 = $25;
  }
  $26 = HEAP32[$1>>2]|0;
  if (!($60)) {
   break;
  }
  $27 = (($26) + 1|0);
  HEAP32[$1>>2] = $27;
 }
 $28 = HEAP32[$2>>2]|0;
 $29 = (_parseHhMmSs($26,$28)|0);
 $30 = ($29|0)==(0);
 do {
  if (!($30)) {
   $31 = HEAP32[$1>>2]|0;
   $32 = HEAP8[$31>>0]|0;
   $33 = $32 << 24 >> 24;
   $34 = ($33|0)==(0);
   if ($34) {
    $35 = HEAP32[$2>>2]|0;
    $36 = (($35) + 41|0);
    HEAP8[$36>>0] = 0;
    break;
   }
   HEAP32[$0>>2] = 1;
   $59 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($59|0);
  }
 } while(0);
 $37 = HEAP32[$2>>2]|0;
 $38 = (($37) + 42|0);
 HEAP8[$38>>0] = 0;
 $39 = HEAP32[$2>>2]|0;
 $40 = (($39) + 40|0);
 HEAP8[$40>>0] = 1;
 $41 = HEAP32[$neg>>2]|0;
 $42 = ($41|0)!=(0);
 $43 = HEAP32[$Y>>2]|0;
 $44 = (0 - ($43))|0;
 $45 = $42 ? $44 : $43;
 $46 = HEAP32[$2>>2]|0;
 $47 = (($46) + 8|0);
 HEAP32[$47>>2] = $45;
 $48 = HEAP32[$M>>2]|0;
 $49 = HEAP32[$2>>2]|0;
 $50 = (($49) + 12|0);
 HEAP32[$50>>2] = $48;
 $51 = HEAP32[$D>>2]|0;
 $52 = HEAP32[$2>>2]|0;
 $53 = (($52) + 16|0);
 HEAP32[$53>>2] = $51;
 $54 = HEAP32[$2>>2]|0;
 $55 = (($54) + 43|0);
 $56 = HEAP8[$55>>0]|0;
 $57 = ($56<<24>>24)!=(0);
 if ($57) {
  $58 = HEAP32[$2>>2]|0;
  _computeJD($58);
 }
 HEAP32[$0>>2] = 0;
 $59 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($59|0);
}
function _sqlite3StmtCurrentTime($p) {
 $p = $p|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $rc = 0;
 var $v = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $v = sp + 4|0;
 $rc = sp;
 HEAP32[$0>>2] = $p;
 $1 = HEAP32[$0>>2]|0;
 $2 = (($1) + 12|0);
 $3 = HEAP32[$2>>2]|0;
 HEAP32[$v>>2] = $3;
 $4 = HEAP32[$v>>2]|0;
 $5 = (($4) + 136|0);
 $6 = $5;
 $7 = $6;
 $8 = HEAP32[$7>>2]|0;
 $9 = (($6) + 4)|0;
 $10 = $9;
 $11 = HEAP32[$10>>2]|0;
 $12 = ($8|0)==(0);
 $13 = ($11|0)==(0);
 $14 = $12 & $13;
 if ($14) {
  $15 = HEAP32[$0>>2]|0;
  $16 = HEAP32[$15>>2]|0;
  $17 = (($16) + 32|0);
  $18 = HEAP32[$17>>2]|0;
  $19 = HEAP32[$18>>2]|0;
  $20 = HEAP32[$v>>2]|0;
  $21 = (($20) + 136|0);
  $22 = (_sqlite3OsCurrentTimeInt64($19,$21)|0);
  HEAP32[$rc>>2] = $22;
  $23 = HEAP32[$rc>>2]|0;
  $24 = ($23|0)!=(0);
  if ($24) {
   $25 = HEAP32[$v>>2]|0;
   $26 = (($25) + 136|0);
   $27 = $26;
   $28 = $27;
   HEAP32[$28>>2] = 0;
   $29 = (($27) + 4)|0;
   $30 = $29;
   HEAP32[$30>>2] = 0;
  }
 }
 $31 = HEAP32[$v>>2]|0;
 $32 = (($31) + 136|0);
 $33 = $32;
 $34 = $33;
 $35 = HEAP32[$34>>2]|0;
 $36 = (($33) + 4)|0;
 $37 = $36;
 $38 = HEAP32[$37>>2]|0;
 tempRet0 = $38;
 STACKTOP = sp;return ($35|0);
}
function _sqlite3OsCurrentTimeInt64($pVfs,$pTimeOut) {
 $pVfs = $pVfs|0;
 $pTimeOut = $pTimeOut|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0.0, $21 = 0.0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $r = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 16|0;
 $1 = sp + 12|0;
 $rc = sp + 8|0;
 $r = sp;
 HEAP32[$0>>2] = $pVfs;
 HEAP32[$1>>2] = $pTimeOut;
 $2 = HEAP32[$0>>2]|0;
 $3 = HEAP32[$2>>2]|0;
 $4 = ($3|0)>=(2);
 if ($4) {
  $5 = HEAP32[$0>>2]|0;
  $6 = (($5) + 72|0);
  $7 = HEAP32[$6>>2]|0;
  $8 = ($7|0)!=(0|0);
  if ($8) {
   $9 = HEAP32[$0>>2]|0;
   $10 = (($9) + 72|0);
   $11 = HEAP32[$10>>2]|0;
   $12 = HEAP32[$0>>2]|0;
   $13 = HEAP32[$1>>2]|0;
   $14 = (FUNCTION_TABLE_iii[$11 & 63]($12,$13)|0);
   HEAP32[$rc>>2] = $14;
   $29 = HEAP32[$rc>>2]|0;
   STACKTOP = sp;return ($29|0);
  }
 }
 $15 = HEAP32[$0>>2]|0;
 $16 = (($15) + 64|0);
 $17 = HEAP32[$16>>2]|0;
 $18 = HEAP32[$0>>2]|0;
 $19 = (FUNCTION_TABLE_iii[$17 & 63]($18,$r)|0);
 HEAP32[$rc>>2] = $19;
 $20 = +HEAPF64[$r>>3];
 $21 = $20 * 8.64E+7;
 $22 = (~~$21)>>>0;
 $23 = +Math_abs($21) >= 1.0 ? $21 > 0.0 ? (Math_min(+Math_floor($21 / 4294967296.0), 4294967295.0) | 0) >>> 0 : ~~+Math_ceil(($21 - +(~~$21 >>> 0)) / 4294967296.0) >>> 0 : 0;
 $24 = HEAP32[$1>>2]|0;
 $25 = $24;
 $26 = $25;
 HEAP32[$26>>2] = $22;
 $27 = (($25) + 4)|0;
 $28 = $27;
 HEAP32[$28>>2] = $23;
 $29 = HEAP32[$rc>>2]|0;
 STACKTOP = sp;return ($29|0);
}
function _sqlite3Utf8CharLen($zIn,$nByte) {
 $zIn = $zIn|0;
 $nByte = $nByte|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $r = 0, $z = 0, $zTerm = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 16|0;
 $1 = sp + 12|0;
 $r = sp + 8|0;
 $z = sp + 4|0;
 $zTerm = sp;
 HEAP32[$0>>2] = $zIn;
 HEAP32[$1>>2] = $nByte;
 HEAP32[$r>>2] = 0;
 $2 = HEAP32[$0>>2]|0;
 HEAP32[$z>>2] = $2;
 $3 = HEAP32[$1>>2]|0;
 $4 = ($3|0)>=(0);
 if ($4) {
  $5 = HEAP32[$1>>2]|0;
  $6 = HEAP32[$z>>2]|0;
  $7 = (($6) + ($5)|0);
  HEAP32[$zTerm>>2] = $7;
 } else {
  HEAP32[$zTerm>>2] = (-1);
 }
 while(1) {
  $8 = HEAP32[$z>>2]|0;
  $9 = HEAP8[$8>>0]|0;
  $10 = $9&255;
  $11 = ($10|0)!=(0);
  if (!($11)) {
   label = 10;
   break;
  }
  $12 = HEAP32[$z>>2]|0;
  $13 = HEAP32[$zTerm>>2]|0;
  $14 = ($12>>>0)<($13>>>0);
  if (!($14)) {
   label = 10;
   break;
  }
  $15 = HEAP32[$z>>2]|0;
  $16 = (($15) + 1|0);
  HEAP32[$z>>2] = $16;
  $17 = HEAP8[$15>>0]|0;
  $18 = $17&255;
  $19 = ($18|0)>=(192);
  L8: do {
   if ($19) {
    while(1) {
     $20 = HEAP32[$z>>2]|0;
     $21 = HEAP8[$20>>0]|0;
     $22 = $21&255;
     $23 = $22 & 192;
     $24 = ($23|0)==(128);
     if (!($24)) {
      break L8;
     }
     $25 = HEAP32[$z>>2]|0;
     $26 = (($25) + 1|0);
     HEAP32[$z>>2] = $26;
    }
   }
  } while(0);
  $27 = HEAP32[$r>>2]|0;
  $28 = (($27) + 1)|0;
  HEAP32[$r>>2] = $28;
 }
 if ((label|0) == 10) {
  $29 = HEAP32[$r>>2]|0;
  STACKTOP = sp;return ($29|0);
 }
 return 0|0;
}
function _sqlite3Utf8Read($pz) {
 $pz = $pz|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $c = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $c = sp;
 HEAP32[$0>>2] = $pz;
 $1 = HEAP32[$0>>2]|0;
 $2 = HEAP32[$1>>2]|0;
 $3 = (($2) + 1|0);
 HEAP32[$1>>2] = $3;
 $4 = HEAP8[$2>>0]|0;
 $5 = $4&255;
 HEAP32[$c>>2] = $5;
 $6 = HEAP32[$c>>2]|0;
 $7 = ($6>>>0)>=(192);
 if (!($7)) {
  $35 = HEAP32[$c>>2]|0;
  STACKTOP = sp;return ($35|0);
 }
 $8 = HEAP32[$c>>2]|0;
 $9 = (($8) - 192)|0;
 $10 = (4984 + ($9)|0);
 $11 = HEAP8[$10>>0]|0;
 $12 = $11&255;
 HEAP32[$c>>2] = $12;
 while(1) {
  $13 = HEAP32[$0>>2]|0;
  $14 = HEAP32[$13>>2]|0;
  $15 = HEAP8[$14>>0]|0;
  $16 = $15&255;
  $17 = $16 & 192;
  $18 = ($17|0)==(128);
  $19 = HEAP32[$c>>2]|0;
  if (!($18)) {
   break;
  }
  $20 = $19 << 6;
  $21 = HEAP32[$0>>2]|0;
  $22 = HEAP32[$21>>2]|0;
  $23 = (($22) + 1|0);
  HEAP32[$21>>2] = $23;
  $24 = HEAP8[$22>>0]|0;
  $25 = $24&255;
  $26 = 63 & $25;
  $27 = (($20) + ($26))|0;
  HEAP32[$c>>2] = $27;
 }
 $28 = ($19>>>0)<(128);
 if (!($28)) {
  $29 = HEAP32[$c>>2]|0;
  $30 = $29 & -2048;
  $31 = ($30|0)==(55296);
  if (!($31)) {
   $32 = HEAP32[$c>>2]|0;
   $33 = $32 & -2;
   $34 = ($33|0)==(65534);
   if (!($34)) {
    $35 = HEAP32[$c>>2]|0;
    STACKTOP = sp;return ($35|0);
   }
  }
 }
 HEAP32[$c>>2] = 65533;
 $35 = HEAP32[$c>>2]|0;
 STACKTOP = sp;return ($35|0);
}
function _sqlite3StrAccumAppend($p,$z,$N) {
 $p = $p|0;
 $z = $z|0;
 $N = $N|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $1 = sp + 4|0;
 $2 = sp;
 HEAP32[$0>>2] = $p;
 HEAP32[$1>>2] = $z;
 HEAP32[$2>>2] = $N;
 $3 = HEAP32[$0>>2]|0;
 $4 = (($3) + 12|0);
 $5 = HEAP32[$4>>2]|0;
 $6 = HEAP32[$2>>2]|0;
 $7 = (($5) + ($6))|0;
 $8 = HEAP32[$0>>2]|0;
 $9 = (($8) + 16|0);
 $10 = HEAP32[$9>>2]|0;
 $11 = ($7|0)>=($10|0);
 if ($11) {
  $12 = HEAP32[$0>>2]|0;
  $13 = HEAP32[$1>>2]|0;
  $14 = HEAP32[$2>>2]|0;
  _enlargeAndAppend($12,$13,$14);
  STACKTOP = sp;return;
 } else {
  $15 = HEAP32[$2>>2]|0;
  $16 = HEAP32[$0>>2]|0;
  $17 = (($16) + 12|0);
  $18 = HEAP32[$17>>2]|0;
  $19 = (($18) + ($15))|0;
  HEAP32[$17>>2] = $19;
  $20 = HEAP32[$0>>2]|0;
  $21 = (($20) + 12|0);
  $22 = HEAP32[$21>>2]|0;
  $23 = HEAP32[$2>>2]|0;
  $24 = (($22) - ($23))|0;
  $25 = HEAP32[$0>>2]|0;
  $26 = (($25) + 8|0);
  $27 = HEAP32[$26>>2]|0;
  $28 = (($27) + ($24)|0);
  $29 = HEAP32[$1>>2]|0;
  $30 = HEAP32[$2>>2]|0;
  _memcpy(($28|0),($29|0),($30|0))|0;
  STACKTOP = sp;return;
 }
}
function _enlargeAndAppend($p,$z,$N) {
 $p = $p|0;
 $z = $z|0;
 $N = $N|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0;
 var $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $1 = sp + 4|0;
 $2 = sp;
 HEAP32[$0>>2] = $p;
 HEAP32[$1>>2] = $z;
 HEAP32[$2>>2] = $N;
 $3 = HEAP32[$0>>2]|0;
 $4 = HEAP32[$2>>2]|0;
 $5 = (_sqlite3StrAccumEnlarge($3,$4)|0);
 HEAP32[$2>>2] = $5;
 $6 = HEAP32[$2>>2]|0;
 $7 = ($6|0)>(0);
 if (!($7)) {
  STACKTOP = sp;return;
 }
 $8 = HEAP32[$0>>2]|0;
 $9 = (($8) + 12|0);
 $10 = HEAP32[$9>>2]|0;
 $11 = HEAP32[$0>>2]|0;
 $12 = (($11) + 8|0);
 $13 = HEAP32[$12>>2]|0;
 $14 = (($13) + ($10)|0);
 $15 = HEAP32[$1>>2]|0;
 $16 = HEAP32[$2>>2]|0;
 _memcpy(($14|0),($15|0),($16|0))|0;
 $17 = HEAP32[$2>>2]|0;
 $18 = HEAP32[$0>>2]|0;
 $19 = (($18) + 12|0);
 $20 = HEAP32[$19>>2]|0;
 $21 = (($20) + ($17))|0;
 HEAP32[$19>>2] = $21;
 STACKTOP = sp;return;
}
function _sqlite3StrAccumEnlarge($p,$N) {
 $p = $p|0;
 $N = $N|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0;
 var $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0;
 var $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0;
 var $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0;
 var $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0;
 var $97 = 0, $98 = 0, $99 = 0, $szNew = 0, $zNew = 0, $zOld = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 24|0;
 $1 = sp + 20|0;
 $2 = sp + 16|0;
 $zNew = sp + 12|0;
 $zOld = sp + 8|0;
 $szNew = sp;
 HEAP32[$1>>2] = $p;
 HEAP32[$2>>2] = $N;
 $3 = HEAP32[$1>>2]|0;
 $4 = (($3) + 25|0);
 $5 = HEAP8[$4>>0]|0;
 $6 = ($5<<24>>24)!=(0);
 if ($6) {
  HEAP32[$0>>2] = 0;
  $122 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($122|0);
 }
 $7 = HEAP32[$1>>2]|0;
 $8 = (($7) + 24|0);
 $9 = HEAP8[$8>>0]|0;
 $10 = ($9<<24>>24)!=(0);
 $11 = HEAP32[$1>>2]|0;
 if (!($10)) {
  $12 = (($11) + 16|0);
  $13 = HEAP32[$12>>2]|0;
  $14 = HEAP32[$1>>2]|0;
  $15 = (($14) + 12|0);
  $16 = HEAP32[$15>>2]|0;
  $17 = (($13) - ($16))|0;
  $18 = (($17) - 1)|0;
  HEAP32[$2>>2] = $18;
  $19 = HEAP32[$1>>2]|0;
  _setStrAccumError($19,2);
  $20 = HEAP32[$2>>2]|0;
  HEAP32[$0>>2] = $20;
  $122 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($122|0);
 }
 $21 = (($11) + 8|0);
 $22 = HEAP32[$21>>2]|0;
 $23 = HEAP32[$1>>2]|0;
 $24 = (($23) + 4|0);
 $25 = HEAP32[$24>>2]|0;
 $26 = ($22|0)==($25|0);
 if ($26) {
  $30 = 0;
 } else {
  $27 = HEAP32[$1>>2]|0;
  $28 = (($27) + 8|0);
  $29 = HEAP32[$28>>2]|0;
  $30 = $29;
 }
 HEAP32[$zOld>>2] = $30;
 $31 = HEAP32[$1>>2]|0;
 $32 = (($31) + 12|0);
 $33 = HEAP32[$32>>2]|0;
 $34 = ($33|0)<(0);
 $35 = $34 << 31 >> 31;
 $36 = $szNew;
 $37 = $36;
 HEAP32[$37>>2] = $33;
 $38 = (($36) + 4)|0;
 $39 = $38;
 HEAP32[$39>>2] = $35;
 $40 = HEAP32[$2>>2]|0;
 $41 = (($40) + 1)|0;
 $42 = ($41|0)<(0);
 $43 = $42 << 31 >> 31;
 $44 = $szNew;
 $45 = $44;
 $46 = HEAP32[$45>>2]|0;
 $47 = (($44) + 4)|0;
 $48 = $47;
 $49 = HEAP32[$48>>2]|0;
 $50 = (_i64Add(($46|0),($49|0),($41|0),($43|0))|0);
 $51 = tempRet0;
 $52 = $szNew;
 $53 = $52;
 HEAP32[$53>>2] = $50;
 $54 = (($52) + 4)|0;
 $55 = $54;
 HEAP32[$55>>2] = $51;
 $56 = $szNew;
 $57 = $56;
 $58 = HEAP32[$57>>2]|0;
 $59 = (($56) + 4)|0;
 $60 = $59;
 $61 = HEAP32[$60>>2]|0;
 $62 = HEAP32[$1>>2]|0;
 $63 = (($62) + 20|0);
 $64 = HEAP32[$63>>2]|0;
 $65 = ($64|0)<(0);
 $66 = $65 << 31 >> 31;
 $67 = ($61|0)>($66|0);
 $68 = ($58>>>0)>($64>>>0);
 $69 = ($61|0)==($66|0);
 $70 = $69 & $68;
 $71 = $67 | $70;
 if ($71) {
  $72 = HEAP32[$1>>2]|0;
  _sqlite3StrAccumReset($72);
  $73 = HEAP32[$1>>2]|0;
  _setStrAccumError($73,2);
  HEAP32[$0>>2] = 0;
  $122 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($122|0);
 }
 $74 = $szNew;
 $75 = $74;
 $76 = HEAP32[$75>>2]|0;
 $77 = (($74) + 4)|0;
 $78 = $77;
 $79 = HEAP32[$78>>2]|0;
 $80 = HEAP32[$1>>2]|0;
 $81 = (($80) + 16|0);
 HEAP32[$81>>2] = $76;
 $82 = HEAP32[$1>>2]|0;
 $83 = (($82) + 24|0);
 $84 = HEAP8[$83>>0]|0;
 $85 = $84&255;
 $86 = ($85|0)==(1);
 if ($86) {
  $87 = HEAP32[$1>>2]|0;
  $88 = HEAP32[$87>>2]|0;
  $89 = HEAP32[$zOld>>2]|0;
  $90 = HEAP32[$1>>2]|0;
  $91 = (($90) + 16|0);
  $92 = HEAP32[$91>>2]|0;
  $93 = ($92|0)<(0);
  $94 = $93 << 31 >> 31;
  $95 = (_sqlite3DbRealloc($88,$89,$92,$94)|0);
  HEAP32[$zNew>>2] = $95;
 } else {
  $96 = HEAP32[$zOld>>2]|0;
  $97 = HEAP32[$1>>2]|0;
  $98 = (($97) + 16|0);
  $99 = HEAP32[$98>>2]|0;
  $100 = (_sqlite3_realloc($96,$99)|0);
  HEAP32[$zNew>>2] = $100;
 }
 $101 = HEAP32[$zNew>>2]|0;
 $102 = ($101|0)!=(0|0);
 if (!($102)) {
  $120 = HEAP32[$1>>2]|0;
  _sqlite3StrAccumReset($120);
  $121 = HEAP32[$1>>2]|0;
  _setStrAccumError($121,1);
  HEAP32[$0>>2] = 0;
  $122 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($122|0);
 }
 $103 = HEAP32[$zOld>>2]|0;
 $104 = ($103|0)==(0|0);
 if ($104) {
  $105 = HEAP32[$1>>2]|0;
  $106 = (($105) + 12|0);
  $107 = HEAP32[$106>>2]|0;
  $108 = ($107|0)>(0);
  if ($108) {
   $109 = HEAP32[$zNew>>2]|0;
   $110 = HEAP32[$1>>2]|0;
   $111 = (($110) + 8|0);
   $112 = HEAP32[$111>>2]|0;
   $113 = HEAP32[$1>>2]|0;
   $114 = (($113) + 12|0);
   $115 = HEAP32[$114>>2]|0;
   _memcpy(($109|0),($112|0),($115|0))|0;
  }
 }
 $116 = HEAP32[$zNew>>2]|0;
 $117 = HEAP32[$1>>2]|0;
 $118 = (($117) + 8|0);
 HEAP32[$118>>2] = $116;
 $119 = HEAP32[$2>>2]|0;
 HEAP32[$0>>2] = $119;
 $122 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($122|0);
}
function _setStrAccumError($p,$eError) {
 $p = $p|0;
 $eError = $eError|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 $1 = sp + 4|0;
 HEAP32[$0>>2] = $p;
 HEAP8[$1>>0] = $eError;
 $2 = HEAP8[$1>>0]|0;
 $3 = HEAP32[$0>>2]|0;
 $4 = (($3) + 25|0);
 HEAP8[$4>>0] = $2;
 $5 = HEAP32[$0>>2]|0;
 $6 = (($5) + 16|0);
 HEAP32[$6>>2] = 0;
 STACKTOP = sp;return;
}
function _sqlite3StrAccumReset($p) {
 $p = $p|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0;
 var $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 HEAP32[$0>>2] = $p;
 $1 = HEAP32[$0>>2]|0;
 $2 = (($1) + 8|0);
 $3 = HEAP32[$2>>2]|0;
 $4 = HEAP32[$0>>2]|0;
 $5 = (($4) + 4|0);
 $6 = HEAP32[$5>>2]|0;
 $7 = ($3|0)!=($6|0);
 do {
  if ($7) {
   $8 = HEAP32[$0>>2]|0;
   $9 = (($8) + 24|0);
   $10 = HEAP8[$9>>0]|0;
   $11 = $10&255;
   $12 = ($11|0)==(1);
   $13 = HEAP32[$0>>2]|0;
   if ($12) {
    $14 = HEAP32[$13>>2]|0;
    $15 = HEAP32[$0>>2]|0;
    $16 = (($15) + 8|0);
    $17 = HEAP32[$16>>2]|0;
    _sqlite3DbFree($14,$17);
    break;
   } else {
    $18 = (($13) + 8|0);
    $19 = HEAP32[$18>>2]|0;
    _sqlite3_free($19);
    break;
   }
  }
 } while(0);
 $20 = HEAP32[$0>>2]|0;
 $21 = (($20) + 8|0);
 HEAP32[$21>>2] = 0;
 STACKTOP = sp;return;
}
function _sqlite3AddInt64($pA,$0,$1) {
 $pA = $pA|0;
 $0 = $0|0;
 $1 = $1|0;
 var $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0;
 var $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0;
 var $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0;
 var $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0;
 var $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0;
 var $97 = 0, $98 = 0, $99 = 0, $iA = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $2 = sp + 20|0;
 $3 = sp + 16|0;
 $4 = sp + 8|0;
 $iA = sp;
 HEAP32[$3>>2] = $pA;
 $5 = $4;
 $6 = $5;
 HEAP32[$6>>2] = $0;
 $7 = (($5) + 4)|0;
 $8 = $7;
 HEAP32[$8>>2] = $1;
 $9 = HEAP32[$3>>2]|0;
 $10 = $9;
 $11 = $10;
 $12 = HEAP32[$11>>2]|0;
 $13 = (($10) + 4)|0;
 $14 = $13;
 $15 = HEAP32[$14>>2]|0;
 $16 = $iA;
 $17 = $16;
 HEAP32[$17>>2] = $12;
 $18 = (($16) + 4)|0;
 $19 = $18;
 HEAP32[$19>>2] = $15;
 $20 = $4;
 $21 = $20;
 $22 = HEAP32[$21>>2]|0;
 $23 = (($20) + 4)|0;
 $24 = $23;
 $25 = HEAP32[$24>>2]|0;
 $26 = ($25|0)>(0);
 $27 = ($22>>>0)>=(0);
 $28 = ($25|0)==(0);
 $29 = $28 & $27;
 $30 = $26 | $29;
 $31 = $iA;
 $32 = $31;
 $33 = HEAP32[$32>>2]|0;
 $34 = (($31) + 4)|0;
 $35 = $34;
 $36 = HEAP32[$35>>2]|0;
 if ($30) {
  $37 = ($36|0)>(0);
  $38 = ($33>>>0)>(0);
  $39 = ($36|0)==(0);
  $40 = $39 & $38;
  $41 = $37 | $40;
  if ($41) {
   $42 = $iA;
   $43 = $42;
   $44 = HEAP32[$43>>2]|0;
   $45 = (($42) + 4)|0;
   $46 = $45;
   $47 = HEAP32[$46>>2]|0;
   $48 = (_i64Subtract(-1,2147483647,($44|0),($47|0))|0);
   $49 = tempRet0;
   $50 = $4;
   $51 = $50;
   $52 = HEAP32[$51>>2]|0;
   $53 = (($50) + 4)|0;
   $54 = $53;
   $55 = HEAP32[$54>>2]|0;
   $56 = ($49|0)<($55|0);
   $57 = ($48>>>0)<($52>>>0);
   $58 = ($49|0)==($55|0);
   $59 = $58 & $57;
   $60 = $56 | $59;
   if ($60) {
    HEAP32[$2>>2] = 1;
    $104 = HEAP32[$2>>2]|0;
    STACKTOP = sp;return ($104|0);
   }
  }
 } else {
  $61 = ($36|0)<(0);
  if ($61) {
   $62 = $iA;
   $63 = $62;
   $64 = HEAP32[$63>>2]|0;
   $65 = (($62) + 4)|0;
   $66 = $65;
   $67 = HEAP32[$66>>2]|0;
   $68 = (_i64Add(($64|0),($67|0),-1,2147483647)|0);
   $69 = tempRet0;
   $70 = (_i64Subtract(0,0,($68|0),($69|0))|0);
   $71 = tempRet0;
   $72 = $4;
   $73 = $72;
   $74 = HEAP32[$73>>2]|0;
   $75 = (($72) + 4)|0;
   $76 = $75;
   $77 = HEAP32[$76>>2]|0;
   $78 = (_i64Add(($74|0),($77|0),1,0)|0);
   $79 = tempRet0;
   $80 = ($71|0)>($79|0);
   $81 = ($70>>>0)>($78>>>0);
   $82 = ($71|0)==($79|0);
   $83 = $82 & $81;
   $84 = $80 | $83;
   if ($84) {
    HEAP32[$2>>2] = 1;
    $104 = HEAP32[$2>>2]|0;
    STACKTOP = sp;return ($104|0);
   }
  }
 }
 $85 = $4;
 $86 = $85;
 $87 = HEAP32[$86>>2]|0;
 $88 = (($85) + 4)|0;
 $89 = $88;
 $90 = HEAP32[$89>>2]|0;
 $91 = HEAP32[$3>>2]|0;
 $92 = $91;
 $93 = $92;
 $94 = HEAP32[$93>>2]|0;
 $95 = (($92) + 4)|0;
 $96 = $95;
 $97 = HEAP32[$96>>2]|0;
 $98 = (_i64Add(($94|0),($97|0),($87|0),($90|0))|0);
 $99 = tempRet0;
 $100 = $91;
 $101 = $100;
 HEAP32[$101>>2] = $98;
 $102 = (($100) + 4)|0;
 $103 = $102;
 HEAP32[$103>>2] = $99;
 HEAP32[$2>>2] = 0;
 $104 = HEAP32[$2>>2]|0;
 STACKTOP = sp;return ($104|0);
}
function _contextMalloc($context,$0,$1) {
 $context = $context|0;
 $0 = $0|0;
 $1 = $1|0;
 var $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0;
 var $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $db = 0, $z = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $2 = sp + 16|0;
 $3 = sp;
 $z = sp + 12|0;
 $db = sp + 8|0;
 HEAP32[$2>>2] = $context;
 $4 = $3;
 $5 = $4;
 HEAP32[$5>>2] = $0;
 $6 = (($4) + 4)|0;
 $7 = $6;
 HEAP32[$7>>2] = $1;
 $8 = HEAP32[$2>>2]|0;
 $9 = (_sqlite3_context_db_handle($8)|0);
 HEAP32[$db>>2] = $9;
 $10 = $3;
 $11 = $10;
 $12 = HEAP32[$11>>2]|0;
 $13 = (($10) + 4)|0;
 $14 = $13;
 $15 = HEAP32[$14>>2]|0;
 $16 = HEAP32[$db>>2]|0;
 $17 = (($16) + 88|0);
 $18 = HEAP32[$17>>2]|0;
 $19 = ($18|0)<(0);
 $20 = $19 << 31 >> 31;
 $21 = ($15|0)>($20|0);
 $22 = ($12>>>0)>($18>>>0);
 $23 = ($15|0)==($20|0);
 $24 = $23 & $22;
 $25 = $21 | $24;
 if ($25) {
  $26 = HEAP32[$2>>2]|0;
  _sqlite3_result_error_toobig($26);
  HEAP32[$z>>2] = 0;
  $37 = HEAP32[$z>>2]|0;
  STACKTOP = sp;return ($37|0);
 }
 $27 = $3;
 $28 = $27;
 $29 = HEAP32[$28>>2]|0;
 $30 = (($27) + 4)|0;
 $31 = $30;
 $32 = HEAP32[$31>>2]|0;
 $33 = (_sqlite3Malloc($29,$32)|0);
 HEAP32[$z>>2] = $33;
 $34 = HEAP32[$z>>2]|0;
 $35 = ($34|0)!=(0|0);
 if ($35) {
  $37 = HEAP32[$z>>2]|0;
  STACKTOP = sp;return ($37|0);
 }
 $36 = HEAP32[$2>>2]|0;
 _sqlite3_result_error_nomem($36);
 $37 = HEAP32[$z>>2]|0;
 STACKTOP = sp;return ($37|0);
}
function _sqlite3GetFuncCollSeq($context) {
 $context = $context|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $pOp = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $pOp = sp;
 HEAP32[$0>>2] = $context;
 $1 = HEAP32[$0>>2]|0;
 $2 = (($1) + 16|0);
 $3 = HEAP32[$2>>2]|0;
 $4 = (($3) - 1)|0;
 $5 = HEAP32[$0>>2]|0;
 $6 = (($5) + 12|0);
 $7 = HEAP32[$6>>2]|0;
 $8 = (($7) + 4|0);
 $9 = HEAP32[$8>>2]|0;
 $10 = (($9) + (($4*20)|0)|0);
 HEAP32[$pOp>>2] = $10;
 $11 = HEAP32[$pOp>>2]|0;
 $12 = (($11) + 16|0);
 $13 = HEAP32[$12>>2]|0;
 STACKTOP = sp;return ($13|0);
}
function _sqlite3MemCompare($pMem1,$pMem2,$pColl) {
 $pMem1 = $pMem1|0;
 $pMem2 = $pMem2|0;
 $pColl = $pColl|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0.0, $101 = 0.0, $102 = 0.0, $103 = 0, $104 = 0.0, $105 = 0.0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0;
 var $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0;
 var $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0;
 var $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0.0, $74 = 0, $75 = 0;
 var $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0.0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0.0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0;
 var $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $combined_flags = 0, $f1 = 0, $f2 = 0, $r1 = 0, $r2 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $0 = sp + 40|0;
 $1 = sp + 36|0;
 $2 = sp + 32|0;
 $3 = sp + 28|0;
 $f1 = sp + 24|0;
 $f2 = sp + 20|0;
 $combined_flags = sp + 16|0;
 $r1 = sp + 8|0;
 $r2 = sp;
 HEAP32[$1>>2] = $pMem1;
 HEAP32[$2>>2] = $pMem2;
 HEAP32[$3>>2] = $pColl;
 $4 = HEAP32[$1>>2]|0;
 $5 = (($4) + 8|0);
 $6 = HEAP16[$5>>1]|0;
 $7 = $6&65535;
 HEAP32[$f1>>2] = $7;
 $8 = HEAP32[$2>>2]|0;
 $9 = (($8) + 8|0);
 $10 = HEAP16[$9>>1]|0;
 $11 = $10&65535;
 HEAP32[$f2>>2] = $11;
 $12 = HEAP32[$f1>>2]|0;
 $13 = HEAP32[$f2>>2]|0;
 $14 = $12 | $13;
 HEAP32[$combined_flags>>2] = $14;
 $15 = HEAP32[$combined_flags>>2]|0;
 $16 = $15 & 1;
 $17 = ($16|0)!=(0);
 L1: do {
  if ($17) {
   $18 = HEAP32[$f2>>2]|0;
   $19 = $18 & 1;
   $20 = HEAP32[$f1>>2]|0;
   $21 = $20 & 1;
   $22 = (($19) - ($21))|0;
   HEAP32[$0>>2] = $22;
  } else {
   $23 = HEAP32[$combined_flags>>2]|0;
   $24 = $23 & 12;
   $25 = ($24|0)!=(0);
   if (!($25)) {
    $107 = HEAP32[$combined_flags>>2]|0;
    $108 = $107 & 2;
    $109 = ($108|0)!=(0);
    if ($109) {
     $110 = HEAP32[$f1>>2]|0;
     $111 = $110 & 2;
     $112 = ($111|0)==(0);
     if ($112) {
      HEAP32[$0>>2] = 1;
      break;
     }
     $113 = HEAP32[$f2>>2]|0;
     $114 = $113 & 2;
     $115 = ($114|0)==(0);
     if ($115) {
      HEAP32[$0>>2] = -1;
      break;
     }
     $116 = HEAP32[$3>>2]|0;
     $117 = ($116|0)!=(0|0);
     if ($117) {
      $118 = HEAP32[$1>>2]|0;
      $119 = HEAP32[$2>>2]|0;
      $120 = HEAP32[$3>>2]|0;
      $121 = (_vdbeCompareMemString($118,$119,$120,0)|0);
      HEAP32[$0>>2] = $121;
      break;
     }
    }
    $122 = HEAP32[$1>>2]|0;
    $123 = HEAP32[$2>>2]|0;
    $124 = (_sqlite3BlobCompare($122,$123)|0);
    HEAP32[$0>>2] = $124;
    break;
   }
   $26 = HEAP32[$f1>>2]|0;
   $27 = HEAP32[$f2>>2]|0;
   $28 = $26 & $27;
   $29 = $28 & 4;
   $30 = ($29|0)!=(0);
   if ($30) {
    $31 = HEAP32[$1>>2]|0;
    $32 = $31;
    $33 = $32;
    $34 = HEAP32[$33>>2]|0;
    $35 = (($32) + 4)|0;
    $36 = $35;
    $37 = HEAP32[$36>>2]|0;
    $38 = HEAP32[$2>>2]|0;
    $39 = $38;
    $40 = $39;
    $41 = HEAP32[$40>>2]|0;
    $42 = (($39) + 4)|0;
    $43 = $42;
    $44 = HEAP32[$43>>2]|0;
    $45 = ($37|0)<($44|0);
    $46 = ($34>>>0)<($41>>>0);
    $47 = ($37|0)==($44|0);
    $48 = $47 & $46;
    $49 = $45 | $48;
    if ($49) {
     HEAP32[$0>>2] = -1;
     break;
    }
    $50 = HEAP32[$1>>2]|0;
    $51 = $50;
    $52 = $51;
    $53 = HEAP32[$52>>2]|0;
    $54 = (($51) + 4)|0;
    $55 = $54;
    $56 = HEAP32[$55>>2]|0;
    $57 = HEAP32[$2>>2]|0;
    $58 = $57;
    $59 = $58;
    $60 = HEAP32[$59>>2]|0;
    $61 = (($58) + 4)|0;
    $62 = $61;
    $63 = HEAP32[$62>>2]|0;
    $64 = ($56|0)>($63|0);
    $65 = ($53>>>0)>($60>>>0);
    $66 = ($56|0)==($63|0);
    $67 = $66 & $65;
    $68 = $64 | $67;
    if ($68) {
     HEAP32[$0>>2] = 1;
     break;
    } else {
     HEAP32[$0>>2] = 0;
     break;
    }
   }
   $69 = HEAP32[$f1>>2]|0;
   $70 = $69 & 8;
   $71 = ($70|0)!=(0);
   do {
    if ($71) {
     $72 = HEAP32[$1>>2]|0;
     $73 = +HEAPF64[$72>>3];
     HEAPF64[$r1>>3] = $73;
    } else {
     $74 = HEAP32[$f1>>2]|0;
     $75 = $74 & 4;
     $76 = ($75|0)!=(0);
     if ($76) {
      $77 = HEAP32[$1>>2]|0;
      $78 = $77;
      $79 = $78;
      $80 = HEAP32[$79>>2]|0;
      $81 = (($78) + 4)|0;
      $82 = $81;
      $83 = HEAP32[$82>>2]|0;
      $84 = (+($80>>>0)) + (4294967296.0*(+($83|0)));
      HEAPF64[$r1>>3] = $84;
      break;
     } else {
      HEAP32[$0>>2] = 1;
      break L1;
     }
    }
   } while(0);
   $85 = HEAP32[$f2>>2]|0;
   $86 = $85 & 8;
   $87 = ($86|0)!=(0);
   do {
    if ($87) {
     $88 = HEAP32[$2>>2]|0;
     $89 = +HEAPF64[$88>>3];
     HEAPF64[$r2>>3] = $89;
    } else {
     $90 = HEAP32[$f2>>2]|0;
     $91 = $90 & 4;
     $92 = ($91|0)!=(0);
     if ($92) {
      $93 = HEAP32[$2>>2]|0;
      $94 = $93;
      $95 = $94;
      $96 = HEAP32[$95>>2]|0;
      $97 = (($94) + 4)|0;
      $98 = $97;
      $99 = HEAP32[$98>>2]|0;
      $100 = (+($96>>>0)) + (4294967296.0*(+($99|0)));
      HEAPF64[$r2>>3] = $100;
      break;
     } else {
      HEAP32[$0>>2] = -1;
      break L1;
     }
    }
   } while(0);
   $101 = +HEAPF64[$r1>>3];
   $102 = +HEAPF64[$r2>>3];
   $103 = $101 < $102;
   if ($103) {
    HEAP32[$0>>2] = -1;
    break;
   }
   $104 = +HEAPF64[$r1>>3];
   $105 = +HEAPF64[$r2>>3];
   $106 = $104 > $105;
   if ($106) {
    HEAP32[$0>>2] = 1;
    break;
   } else {
    HEAP32[$0>>2] = 0;
    break;
   }
  }
 } while(0);
 $125 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($125|0);
}
function _vdbeCompareMemString($pMem1,$pMem2,$pColl,$prcErr) {
 $pMem1 = $pMem1|0;
 $pMem2 = $pMem2|0;
 $pColl = $pColl|0;
 $prcErr = $prcErr|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $8 = 0, $9 = 0, $c1 = 0;
 var $c2 = 0, $n1 = 0, $n2 = 0, $or$cond = 0, $or$cond3 = 0, $rc = 0, $v1 = 0, $v2 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 128|0;
 $0 = sp + 116|0;
 $1 = sp + 112|0;
 $2 = sp + 108|0;
 $3 = sp + 104|0;
 $4 = sp + 100|0;
 $rc = sp + 96|0;
 $v1 = sp + 92|0;
 $v2 = sp + 88|0;
 $n1 = sp + 84|0;
 $n2 = sp + 80|0;
 $c1 = sp + 40|0;
 $c2 = sp;
 HEAP32[$1>>2] = $pMem1;
 HEAP32[$2>>2] = $pMem2;
 HEAP32[$3>>2] = $pColl;
 HEAP32[$4>>2] = $prcErr;
 $5 = HEAP32[$1>>2]|0;
 $6 = (($5) + 10|0);
 $7 = HEAP8[$6>>0]|0;
 $8 = $7&255;
 $9 = HEAP32[$3>>2]|0;
 $10 = (($9) + 4|0);
 $11 = HEAP8[$10>>0]|0;
 $12 = $11&255;
 $13 = ($8|0)==($12|0);
 if ($13) {
  $14 = HEAP32[$3>>2]|0;
  $15 = (($14) + 12|0);
  $16 = HEAP32[$15>>2]|0;
  $17 = HEAP32[$3>>2]|0;
  $18 = (($17) + 8|0);
  $19 = HEAP32[$18>>2]|0;
  $20 = HEAP32[$1>>2]|0;
  $21 = (($20) + 12|0);
  $22 = HEAP32[$21>>2]|0;
  $23 = HEAP32[$1>>2]|0;
  $24 = (($23) + 16|0);
  $25 = HEAP32[$24>>2]|0;
  $26 = HEAP32[$2>>2]|0;
  $27 = (($26) + 12|0);
  $28 = HEAP32[$27>>2]|0;
  $29 = HEAP32[$2>>2]|0;
  $30 = (($29) + 16|0);
  $31 = HEAP32[$30>>2]|0;
  $32 = (FUNCTION_TABLE_iiiiii[$16 & 15]($19,$22,$25,$28,$31)|0);
  HEAP32[$0>>2] = $32;
  $78 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($78|0);
 }
 $33 = HEAP32[$1>>2]|0;
 $34 = (($33) + 32|0);
 $35 = HEAP32[$34>>2]|0;
 _sqlite3VdbeMemInit($c1,$35,1);
 $36 = HEAP32[$1>>2]|0;
 $37 = (($36) + 32|0);
 $38 = HEAP32[$37>>2]|0;
 _sqlite3VdbeMemInit($c2,$38,1);
 $39 = HEAP32[$1>>2]|0;
 _sqlite3VdbeMemShallowCopy($c1,$39,4096);
 $40 = HEAP32[$2>>2]|0;
 _sqlite3VdbeMemShallowCopy($c2,$40,4096);
 $41 = HEAP32[$3>>2]|0;
 $42 = (($41) + 4|0);
 $43 = HEAP8[$42>>0]|0;
 $44 = (_sqlite3ValueText($c1,$43)|0);
 HEAP32[$v1>>2] = $44;
 $45 = HEAP32[$v1>>2]|0;
 $46 = ($45|0)==(0|0);
 if ($46) {
  $49 = 0;
 } else {
  $47 = (($c1) + 12|0);
  $48 = HEAP32[$47>>2]|0;
  $49 = $48;
 }
 HEAP32[$n1>>2] = $49;
 $50 = HEAP32[$3>>2]|0;
 $51 = (($50) + 4|0);
 $52 = HEAP8[$51>>0]|0;
 $53 = (_sqlite3ValueText($c2,$52)|0);
 HEAP32[$v2>>2] = $53;
 $54 = HEAP32[$v2>>2]|0;
 $55 = ($54|0)==(0|0);
 if ($55) {
  $58 = 0;
 } else {
  $56 = (($c2) + 12|0);
  $57 = HEAP32[$56>>2]|0;
  $58 = $57;
 }
 HEAP32[$n2>>2] = $58;
 $59 = HEAP32[$3>>2]|0;
 $60 = (($59) + 12|0);
 $61 = HEAP32[$60>>2]|0;
 $62 = HEAP32[$3>>2]|0;
 $63 = (($62) + 8|0);
 $64 = HEAP32[$63>>2]|0;
 $65 = HEAP32[$n1>>2]|0;
 $66 = HEAP32[$v1>>2]|0;
 $67 = HEAP32[$n2>>2]|0;
 $68 = HEAP32[$v2>>2]|0;
 $69 = (FUNCTION_TABLE_iiiiii[$61 & 15]($64,$65,$66,$67,$68)|0);
 HEAP32[$rc>>2] = $69;
 _sqlite3VdbeMemRelease($c1);
 _sqlite3VdbeMemRelease($c2);
 $70 = HEAP32[$v1>>2]|0;
 $71 = ($70|0)==(0|0);
 $72 = HEAP32[$v2>>2]|0;
 $73 = ($72|0)==(0|0);
 $or$cond = $71 | $73;
 $74 = HEAP32[$4>>2]|0;
 $75 = ($74|0)!=(0|0);
 $or$cond3 = $or$cond & $75;
 if ($or$cond3) {
  $76 = HEAP32[$4>>2]|0;
  HEAP8[$76>>0] = 7;
 }
 $77 = HEAP32[$rc>>2]|0;
 HEAP32[$0>>2] = $77;
 $78 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($78|0);
}
function _sqlite3BlobCompare($pB1,$pB2) {
 $pB1 = $pB1|0;
 $pB2 = $pB2|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $c = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $1 = sp + 8|0;
 $2 = sp + 4|0;
 $c = sp;
 HEAP32[$1>>2] = $pB1;
 HEAP32[$2>>2] = $pB2;
 $3 = HEAP32[$1>>2]|0;
 $4 = (($3) + 16|0);
 $5 = HEAP32[$4>>2]|0;
 $6 = HEAP32[$2>>2]|0;
 $7 = (($6) + 16|0);
 $8 = HEAP32[$7>>2]|0;
 $9 = HEAP32[$1>>2]|0;
 $10 = (($9) + 12|0);
 $11 = HEAP32[$10>>2]|0;
 $12 = HEAP32[$2>>2]|0;
 $13 = (($12) + 12|0);
 $14 = HEAP32[$13>>2]|0;
 $15 = ($11|0)>($14|0);
 if ($15) {
  $16 = HEAP32[$2>>2]|0;
  $17 = (($16) + 12|0);
  $18 = HEAP32[$17>>2]|0;
  $22 = $18;
 } else {
  $19 = HEAP32[$1>>2]|0;
  $20 = (($19) + 12|0);
  $21 = HEAP32[$20>>2]|0;
  $22 = $21;
 }
 $23 = (_memcmp($5,$8,$22)|0);
 HEAP32[$c>>2] = $23;
 $24 = HEAP32[$c>>2]|0;
 $25 = ($24|0)!=(0);
 if ($25) {
  $26 = HEAP32[$c>>2]|0;
  HEAP32[$0>>2] = $26;
  $34 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($34|0);
 } else {
  $27 = HEAP32[$1>>2]|0;
  $28 = (($27) + 12|0);
  $29 = HEAP32[$28>>2]|0;
  $30 = HEAP32[$2>>2]|0;
  $31 = (($30) + 12|0);
  $32 = HEAP32[$31>>2]|0;
  $33 = (($29) - ($32))|0;
  HEAP32[$0>>2] = $33;
  $34 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($34|0);
 }
 return 0|0;
}
function _sqlite3VdbeMemInit($pMem,$db,$flags) {
 $pMem = $pMem|0;
 $db = $db|0;
 $flags = $flags|0;
 var $0 = 0, $1 = 0, $10 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $1 = sp;
 $2 = sp + 8|0;
 HEAP32[$0>>2] = $pMem;
 HEAP32[$1>>2] = $db;
 HEAP16[$2>>1] = $flags;
 $3 = HEAP16[$2>>1]|0;
 $4 = HEAP32[$0>>2]|0;
 $5 = (($4) + 8|0);
 HEAP16[$5>>1] = $3;
 $6 = HEAP32[$1>>2]|0;
 $7 = HEAP32[$0>>2]|0;
 $8 = (($7) + 32|0);
 HEAP32[$8>>2] = $6;
 $9 = HEAP32[$0>>2]|0;
 $10 = (($9) + 24|0);
 HEAP32[$10>>2] = 0;
 STACKTOP = sp;return;
}
function _sqlite3VdbeMemShallowCopy($pTo,$pFrom,$srcType) {
 $pTo = $pTo|0;
 $pFrom = $pFrom|0;
 $srcType = $srcType|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $1 = sp + 4|0;
 $2 = sp;
 HEAP32[$0>>2] = $pTo;
 HEAP32[$1>>2] = $pFrom;
 HEAP32[$2>>2] = $srcType;
 $3 = HEAP32[$0>>2]|0;
 $4 = (($3) + 8|0);
 $5 = HEAP16[$4>>1]|0;
 $6 = $5&65535;
 $7 = $6 & 9312;
 $8 = ($7|0)!=(0);
 if ($8) {
  $9 = HEAP32[$0>>2]|0;
  _vdbeMemClearExternAndSetNull($9);
 }
 $10 = HEAP32[$0>>2]|0;
 $11 = HEAP32[$1>>2]|0;
 ;HEAP32[$10+0>>2]=HEAP32[$11+0>>2]|0;HEAP32[$10+4>>2]=HEAP32[$11+4>>2]|0;HEAP32[$10+8>>2]=HEAP32[$11+8>>2]|0;HEAP32[$10+12>>2]=HEAP32[$11+12>>2]|0;HEAP32[$10+16>>2]=HEAP32[$11+16>>2]|0;
 $12 = HEAP32[$1>>2]|0;
 $13 = (($12) + 8|0);
 $14 = HEAP16[$13>>1]|0;
 $15 = $14&65535;
 $16 = $15 & 2048;
 $17 = ($16|0)==(0);
 if (!($17)) {
  STACKTOP = sp;return;
 }
 $18 = HEAP32[$0>>2]|0;
 $19 = (($18) + 8|0);
 $20 = HEAP16[$19>>1]|0;
 $21 = $20&65535;
 $22 = $21 & -7169;
 $23 = $22&65535;
 HEAP16[$19>>1] = $23;
 $24 = HEAP32[$2>>2]|0;
 $25 = HEAP32[$0>>2]|0;
 $26 = (($25) + 8|0);
 $27 = HEAP16[$26>>1]|0;
 $28 = $27&65535;
 $29 = $28 | $24;
 $30 = $29&65535;
 HEAP16[$26>>1] = $30;
 STACKTOP = sp;return;
}
function _vdbeMemClearExternAndSetNull($p) {
 $p = $p|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $pFrame = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $pFrame = sp;
 HEAP32[$0>>2] = $p;
 $1 = HEAP32[$0>>2]|0;
 $2 = (($1) + 8|0);
 $3 = HEAP16[$2>>1]|0;
 $4 = $3&65535;
 $5 = $4 & 8192;
 $6 = ($5|0)!=(0);
 if ($6) {
  $7 = HEAP32[$0>>2]|0;
  $8 = HEAP32[$0>>2]|0;
  $9 = HEAP32[$8>>2]|0;
  (_sqlite3VdbeMemFinalize($7,$9)|0);
 }
 $10 = HEAP32[$0>>2]|0;
 $11 = (($10) + 8|0);
 $12 = HEAP16[$11>>1]|0;
 $13 = $12&65535;
 $14 = $13 & 1024;
 $15 = ($14|0)!=(0);
 $16 = HEAP32[$0>>2]|0;
 do {
  if ($15) {
   $17 = (($16) + 36|0);
   $18 = HEAP32[$17>>2]|0;
   $19 = HEAP32[$0>>2]|0;
   $20 = (($19) + 16|0);
   $21 = HEAP32[$20>>2]|0;
   FUNCTION_TABLE_vi[$18 & 31]($21);
  } else {
   $22 = (($16) + 8|0);
   $23 = HEAP16[$22>>1]|0;
   $24 = $23&65535;
   $25 = $24 & 32;
   $26 = ($25|0)!=(0);
   $27 = HEAP32[$0>>2]|0;
   if ($26) {
    $28 = HEAP32[$27>>2]|0;
    _sqlite3RowSetClear($28);
    break;
   }
   $29 = (($27) + 8|0);
   $30 = HEAP16[$29>>1]|0;
   $31 = $30&65535;
   $32 = $31 & 64;
   $33 = ($32|0)!=(0);
   if ($33) {
    $34 = HEAP32[$0>>2]|0;
    $35 = HEAP32[$34>>2]|0;
    HEAP32[$pFrame>>2] = $35;
    $36 = HEAP32[$pFrame>>2]|0;
    $37 = HEAP32[$36>>2]|0;
    $38 = (($37) + 180|0);
    $39 = HEAP32[$38>>2]|0;
    $40 = HEAP32[$pFrame>>2]|0;
    $41 = (($40) + 4|0);
    HEAP32[$41>>2] = $39;
    $42 = HEAP32[$pFrame>>2]|0;
    $43 = HEAP32[$pFrame>>2]|0;
    $44 = HEAP32[$43>>2]|0;
    $45 = (($44) + 180|0);
    HEAP32[$45>>2] = $42;
   }
  }
 } while(0);
 $46 = HEAP32[$0>>2]|0;
 $47 = (($46) + 8|0);
 HEAP16[$47>>1] = 1;
 STACKTOP = sp;return;
}
function _sqlite3VdbeMemFinalize($pMem,$pFunc) {
 $pMem = $pMem|0;
 $pFunc = $pFunc|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $ctx = 0, $rc = 0, $t = 0, dest = 0, label = 0, sp = 0;
 var src = 0, stop = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 80|0;
 $0 = sp + 76|0;
 $1 = sp + 72|0;
 $rc = sp + 68|0;
 $ctx = sp + 40|0;
 $t = sp;
 HEAP32[$0>>2] = $pMem;
 HEAP32[$1>>2] = $pFunc;
 HEAP32[$rc>>2] = 0;
 $2 = HEAP32[$1>>2]|0;
 $3 = ($2|0)!=(0|0);
 if (!($3)) {
  $33 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($33|0);
 }
 $4 = HEAP32[$1>>2]|0;
 $5 = (($4) + 20|0);
 $6 = HEAP32[$5>>2]|0;
 $7 = ($6|0)!=(0|0);
 if (!($7)) {
  $33 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($33|0);
 }
 ;HEAP32[$ctx+0>>2]=0|0;HEAP32[$ctx+4>>2]=0|0;HEAP32[$ctx+8>>2]=0|0;HEAP32[$ctx+12>>2]=0|0;HEAP32[$ctx+16>>2]=0|0;HEAP32[$ctx+20>>2]=0|0;HEAP32[$ctx+24>>2]=0|0;
 dest=$t+0|0; stop=dest+40|0; do { HEAP32[dest>>2]=0|0; dest=dest+4|0; } while ((dest|0) < (stop|0));
 $8 = (($t) + 8|0);
 HEAP16[$8>>1] = 1;
 $9 = HEAP32[$0>>2]|0;
 $10 = (($9) + 32|0);
 $11 = HEAP32[$10>>2]|0;
 $12 = (($t) + 32|0);
 HEAP32[$12>>2] = $11;
 HEAP32[$ctx>>2] = $t;
 $13 = HEAP32[$0>>2]|0;
 $14 = (($ctx) + 8|0);
 HEAP32[$14>>2] = $13;
 $15 = HEAP32[$1>>2]|0;
 $16 = (($ctx) + 4|0);
 HEAP32[$16>>2] = $15;
 $17 = HEAP32[$1>>2]|0;
 $18 = (($17) + 20|0);
 $19 = HEAP32[$18>>2]|0;
 FUNCTION_TABLE_vi[$19 & 31]($ctx);
 $20 = HEAP32[$0>>2]|0;
 $21 = (($20) + 24|0);
 $22 = HEAP32[$21>>2]|0;
 $23 = ($22|0)>(0);
 if ($23) {
  $24 = HEAP32[$0>>2]|0;
  $25 = (($24) + 32|0);
  $26 = HEAP32[$25>>2]|0;
  $27 = HEAP32[$0>>2]|0;
  $28 = (($27) + 20|0);
  $29 = HEAP32[$28>>2]|0;
  _sqlite3DbFree($26,$29);
 }
 $30 = HEAP32[$0>>2]|0;
 dest=$30+0|0; src=$t+0|0; stop=dest+40|0; do { HEAP32[dest>>2]=HEAP32[src>>2]|0; dest=dest+4|0; src=src+4|0; } while ((dest|0) < (stop|0));
 $31 = (($ctx) + 20|0);
 $32 = HEAP32[$31>>2]|0;
 HEAP32[$rc>>2] = $32;
 $33 = HEAP32[$rc>>2]|0;
 STACKTOP = sp;return ($33|0);
}
function _sqlite3RowSetClear($p) {
 $p = $p|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0;
 var $7 = 0, $8 = 0, $9 = 0, $pChunk = 0, $pNextChunk = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $pChunk = sp + 4|0;
 $pNextChunk = sp;
 HEAP32[$0>>2] = $p;
 $1 = HEAP32[$0>>2]|0;
 $2 = HEAP32[$1>>2]|0;
 HEAP32[$pChunk>>2] = $2;
 while(1) {
  $3 = HEAP32[$pChunk>>2]|0;
  $4 = ($3|0)!=(0|0);
  if (!($4)) {
   break;
  }
  $5 = HEAP32[$pChunk>>2]|0;
  $6 = HEAP32[$5>>2]|0;
  HEAP32[$pNextChunk>>2] = $6;
  $7 = HEAP32[$0>>2]|0;
  $8 = (($7) + 4|0);
  $9 = HEAP32[$8>>2]|0;
  $10 = HEAP32[$pChunk>>2]|0;
  _sqlite3DbFree($9,$10);
  $11 = HEAP32[$pNextChunk>>2]|0;
  HEAP32[$pChunk>>2] = $11;
 }
 $12 = HEAP32[$0>>2]|0;
 HEAP32[$12>>2] = 0;
 $13 = HEAP32[$0>>2]|0;
 $14 = (($13) + 24|0);
 HEAP16[$14>>1] = 0;
 $15 = HEAP32[$0>>2]|0;
 $16 = (($15) + 8|0);
 HEAP32[$16>>2] = 0;
 $17 = HEAP32[$0>>2]|0;
 $18 = (($17) + 12|0);
 HEAP32[$18>>2] = 0;
 $19 = HEAP32[$0>>2]|0;
 $20 = (($19) + 20|0);
 HEAP32[$20>>2] = 0;
 $21 = HEAP32[$0>>2]|0;
 $22 = (($21) + 26|0);
 HEAP16[$22>>1] = 1;
 STACKTOP = sp;return;
}
function _sqlite3XPrintf($p,$bFlags,$zFormat,$varargs) {
 $p = $p|0;
 $bFlags = $bFlags|0;
 $zFormat = $zFormat|0;
 $varargs = $varargs|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $ap = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 24|0;
 $1 = sp + 20|0;
 $2 = sp + 16|0;
 $ap = sp;
 HEAP32[$0>>2] = $p;
 HEAP32[$1>>2] = $bFlags;
 HEAP32[$2>>2] = $zFormat;
 HEAP32[$ap>>2] = $varargs;
 $3 = HEAP32[$0>>2]|0;
 $4 = HEAP32[$1>>2]|0;
 $5 = HEAP32[$2>>2]|0;
 _sqlite3VXPrintf($3,$4,$5,$ap);
 STACKTOP = sp;return;
}
function _sqlite3SkipAccumulatorLoad($context) {
 $context = $context|0;
 var $0 = 0, $1 = 0, $2 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 HEAP32[$0>>2] = $context;
 $1 = HEAP32[$0>>2]|0;
 $2 = (($1) + 24|0);
 HEAP8[$2>>0] = 1;
 STACKTOP = sp;return;
}
function _sqlite3ParserAlloc($mallocProc) {
 $mallocProc = $mallocProc|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $pParser = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $pParser = sp;
 HEAP32[$0>>2] = $mallocProc;
 $1 = HEAP32[$0>>2]|0;
 $2 = (FUNCTION_TABLE_iii[$1 & 63](1612,0)|0);
 HEAP32[$pParser>>2] = $2;
 $3 = HEAP32[$pParser>>2]|0;
 $4 = ($3|0)!=(0|0);
 if (!($4)) {
  $6 = HEAP32[$pParser>>2]|0;
  STACKTOP = sp;return ($6|0);
 }
 $5 = HEAP32[$pParser>>2]|0;
 HEAP32[$5>>2] = -1;
 $6 = HEAP32[$pParser>>2]|0;
 STACKTOP = sp;return ($6|0);
}
function _sqlite3Parser($yyp,$yymajor,$yyminor,$pParse) {
 $yyp = $yyp|0;
 $yymajor = $yymajor|0;
 $yyminor = $yyminor|0;
 $pParse = $pParse|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $yyact = 0, $yyminorunion = 0, $yyminorunion$byval_copy = 0, $yypParser = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $yyminorunion$byval_copy = sp + 32|0;
 $0 = sp + 28|0;
 $1 = sp + 24|0;
 $2 = sp + 20|0;
 $yyminorunion = sp + 8|0;
 $yyact = sp + 4|0;
 $yypParser = sp;
 HEAP32[$0>>2] = $yyp;
 HEAP32[$1>>2] = $yymajor;
 HEAP32[$2>>2] = $pParse;
 $3 = HEAP32[$0>>2]|0;
 HEAP32[$yypParser>>2] = $3;
 $4 = HEAP32[$yypParser>>2]|0;
 $5 = HEAP32[$4>>2]|0;
 $6 = ($5|0)<(0);
 if ($6) {
  $7 = HEAP32[$yypParser>>2]|0;
  HEAP32[$7>>2] = 0;
  $8 = HEAP32[$yypParser>>2]|0;
  $9 = (($8) + 4|0);
  HEAP32[$9>>2] = -1;
  $10 = HEAP32[$yypParser>>2]|0;
  $11 = (($10) + 12|0);
  HEAP16[$11>>1] = 0;
  $12 = HEAP32[$yypParser>>2]|0;
  $13 = (($12) + 12|0);
  $14 = (($13) + 2|0);
  HEAP8[$14>>0] = 0;
 }
 ;HEAP32[$yyminorunion+0>>2]=HEAP32[$yyminor+0>>2]|0;HEAP32[$yyminorunion+4>>2]=HEAP32[$yyminor+4>>2]|0;
 $15 = HEAP32[$2>>2]|0;
 $16 = HEAP32[$yypParser>>2]|0;
 $17 = (($16) + 8|0);
 HEAP32[$17>>2] = $15;
 while(1) {
  $18 = HEAP32[$yypParser>>2]|0;
  $19 = HEAP32[$1>>2]|0;
  $20 = $19&255;
  $21 = (_yy_find_shift_action($18,$20)|0);
  HEAP32[$yyact>>2] = $21;
  $22 = HEAP32[$yyact>>2]|0;
  $23 = ($22|0)<(642);
  do {
   if ($23) {
    $24 = HEAP32[$yypParser>>2]|0;
    $25 = HEAP32[$yyact>>2]|0;
    $26 = HEAP32[$1>>2]|0;
    _yy_shift($24,$25,$26,$yyminorunion);
    $27 = HEAP32[$yypParser>>2]|0;
    $28 = (($27) + 4|0);
    $29 = HEAP32[$28>>2]|0;
    $30 = (($29) + -1)|0;
    HEAP32[$28>>2] = $30;
    HEAP32[$1>>2] = 254;
   } else {
    $31 = HEAP32[$yyact>>2]|0;
    $32 = ($31|0)<(969);
    $33 = HEAP32[$yypParser>>2]|0;
    if ($32) {
     $34 = HEAP32[$yyact>>2]|0;
     $35 = (($34) - 642)|0;
     _yy_reduce($33,$35);
     break;
    } else {
     $36 = HEAP32[$1>>2]|0;
     ;HEAP32[$yyminorunion$byval_copy+0>>2]=HEAP32[$yyminorunion+0>>2]|0;HEAP32[$yyminorunion$byval_copy+4>>2]=HEAP32[$yyminorunion+4>>2]|0;HEAP32[$yyminorunion$byval_copy+8>>2]=HEAP32[$yyminorunion+8>>2]|0;
     _yy_syntax_error($33,$36,$yyminorunion$byval_copy);
     $37 = HEAP32[$yypParser>>2]|0;
     $38 = HEAP32[$1>>2]|0;
     $39 = $38&255;
     _yy_destructor($37,$39,$yyminorunion);
     HEAP32[$1>>2] = 254;
     break;
    }
   }
  } while(0);
  $40 = HEAP32[$1>>2]|0;
  $41 = ($40|0)!=(254);
  if (!($41)) {
   label = 11;
   break;
  }
  $42 = HEAP32[$yypParser>>2]|0;
  $43 = HEAP32[$42>>2]|0;
  $44 = ($43|0)>=(0);
  if (!($44)) {
   label = 11;
   break;
  }
 }
 if ((label|0) == 11) {
  STACKTOP = sp;return;
 }
}
function _sqlite3ParserFree($p,$freeProc) {
 $p = $p|0;
 $freeProc = $freeProc|0;
 var $0 = 0, $1 = 0, $10 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $pParser = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $1 = sp + 4|0;
 $pParser = sp;
 HEAP32[$0>>2] = $p;
 HEAP32[$1>>2] = $freeProc;
 $2 = HEAP32[$0>>2]|0;
 HEAP32[$pParser>>2] = $2;
 $3 = HEAP32[$pParser>>2]|0;
 $4 = ($3|0)==(0|0);
 if ($4) {
  STACKTOP = sp;return;
 }
 while(1) {
  $5 = HEAP32[$pParser>>2]|0;
  $6 = HEAP32[$5>>2]|0;
  $7 = ($6|0)>=(0);
  if (!($7)) {
   break;
  }
  $8 = HEAP32[$pParser>>2]|0;
  (_yy_pop_parser_stack($8)|0);
 }
 $9 = HEAP32[$1>>2]|0;
 $10 = HEAP32[$pParser>>2]|0;
 FUNCTION_TABLE_vi[$9 & 31]($10);
 STACKTOP = sp;return;
}
function _sqlite3SetString($pz,$db,$zFormat,$varargs) {
 $pz = $pz|0;
 $db = $db|0;
 $zFormat = $zFormat|0;
 $varargs = $varargs|0;
 var $0 = 0, $1 = 0, $10 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $ap = 0, $z = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $0 = sp + 32|0;
 $1 = sp + 28|0;
 $2 = sp + 24|0;
 $ap = sp + 8|0;
 $z = sp;
 HEAP32[$0>>2] = $pz;
 HEAP32[$1>>2] = $db;
 HEAP32[$2>>2] = $zFormat;
 HEAP32[$ap>>2] = $varargs;
 $3 = HEAP32[$1>>2]|0;
 $4 = HEAP32[$2>>2]|0;
 $5 = (_sqlite3VMPrintf($3,$4,$ap)|0);
 HEAP32[$z>>2] = $5;
 $6 = HEAP32[$1>>2]|0;
 $7 = HEAP32[$0>>2]|0;
 $8 = HEAP32[$7>>2]|0;
 _sqlite3DbFree($6,$8);
 $9 = HEAP32[$z>>2]|0;
 $10 = HEAP32[$0>>2]|0;
 HEAP32[$10>>2] = $9;
 STACKTOP = sp;return;
}
function _sqlite3VdbeDelete($p) {
 $p = $p|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $db = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $db = sp;
 HEAP32[$0>>2] = $p;
 $1 = HEAP32[$0>>2]|0;
 $2 = ($1|0)==(0|0);
 if ($2) {
  STACKTOP = sp;return;
 }
 $3 = HEAP32[$0>>2]|0;
 $4 = HEAP32[$3>>2]|0;
 HEAP32[$db>>2] = $4;
 $5 = HEAP32[$db>>2]|0;
 $6 = HEAP32[$0>>2]|0;
 _sqlite3VdbeClearObject($5,$6);
 $7 = HEAP32[$0>>2]|0;
 $8 = (($7) + 48|0);
 $9 = HEAP32[$8>>2]|0;
 $10 = ($9|0)!=(0|0);
 $11 = HEAP32[$0>>2]|0;
 $12 = (($11) + 52|0);
 $13 = HEAP32[$12>>2]|0;
 if ($10) {
  $14 = HEAP32[$0>>2]|0;
  $15 = (($14) + 48|0);
  $16 = HEAP32[$15>>2]|0;
  $17 = (($16) + 52|0);
  HEAP32[$17>>2] = $13;
 } else {
  $18 = HEAP32[$db>>2]|0;
  $19 = (($18) + 4|0);
  HEAP32[$19>>2] = $13;
 }
 $20 = HEAP32[$0>>2]|0;
 $21 = (($20) + 52|0);
 $22 = HEAP32[$21>>2]|0;
 $23 = ($22|0)!=(0|0);
 if ($23) {
  $24 = HEAP32[$0>>2]|0;
  $25 = (($24) + 48|0);
  $26 = HEAP32[$25>>2]|0;
  $27 = HEAP32[$0>>2]|0;
  $28 = (($27) + 52|0);
  $29 = HEAP32[$28>>2]|0;
  $30 = (($29) + 48|0);
  HEAP32[$30>>2] = $26;
 }
 $31 = HEAP32[$0>>2]|0;
 $32 = (($31) + 40|0);
 HEAP32[$32>>2] = -1241070648;
 $33 = HEAP32[$0>>2]|0;
 HEAP32[$33>>2] = 0;
 $34 = HEAP32[$db>>2]|0;
 $35 = HEAP32[$0>>2]|0;
 _sqlite3DbFree($34,$35);
 STACKTOP = sp;return;
}
function _sqlite3WithDelete($db,$pWith) {
 $db = $db|0;
 $pWith = $pWith|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $i = 0, $pCte = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $1 = sp + 8|0;
 $i = sp + 4|0;
 $pCte = sp;
 HEAP32[$0>>2] = $db;
 HEAP32[$1>>2] = $pWith;
 $2 = HEAP32[$1>>2]|0;
 $3 = ($2|0)!=(0|0);
 if (!($3)) {
  STACKTOP = sp;return;
 }
 HEAP32[$i>>2] = 0;
 while(1) {
  $4 = HEAP32[$i>>2]|0;
  $5 = HEAP32[$1>>2]|0;
  $6 = HEAP32[$5>>2]|0;
  $7 = ($4|0)<($6|0);
  if (!($7)) {
   break;
  }
  $8 = HEAP32[$i>>2]|0;
  $9 = HEAP32[$1>>2]|0;
  $10 = (($9) + 8|0);
  $11 = (($10) + ($8<<4)|0);
  HEAP32[$pCte>>2] = $11;
  $12 = HEAP32[$0>>2]|0;
  $13 = HEAP32[$pCte>>2]|0;
  $14 = (($13) + 4|0);
  $15 = HEAP32[$14>>2]|0;
  _sqlite3ExprListDelete($12,$15);
  $16 = HEAP32[$0>>2]|0;
  $17 = HEAP32[$pCte>>2]|0;
  $18 = (($17) + 8|0);
  $19 = HEAP32[$18>>2]|0;
  _sqlite3SelectDelete($16,$19);
  $20 = HEAP32[$0>>2]|0;
  $21 = HEAP32[$pCte>>2]|0;
  $22 = HEAP32[$21>>2]|0;
  _sqlite3DbFree($20,$22);
  $23 = HEAP32[$i>>2]|0;
  $24 = (($23) + 1)|0;
  HEAP32[$i>>2] = $24;
 }
 $25 = HEAP32[$0>>2]|0;
 $26 = HEAP32[$1>>2]|0;
 _sqlite3DbFree($25,$26);
 STACKTOP = sp;return;
}
function _sqlite3ExprListDelete($db,$pList) {
 $db = $db|0;
 $pList = $pList|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $i = 0, $pItem = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $1 = sp + 8|0;
 $i = sp + 4|0;
 $pItem = sp;
 HEAP32[$0>>2] = $db;
 HEAP32[$1>>2] = $pList;
 $2 = HEAP32[$1>>2]|0;
 $3 = ($2|0)==(0|0);
 if ($3) {
  STACKTOP = sp;return;
 }
 $4 = HEAP32[$1>>2]|0;
 $5 = (($4) + 4|0);
 $6 = HEAP32[$5>>2]|0;
 HEAP32[$pItem>>2] = $6;
 HEAP32[$i>>2] = 0;
 while(1) {
  $7 = HEAP32[$i>>2]|0;
  $8 = HEAP32[$1>>2]|0;
  $9 = HEAP32[$8>>2]|0;
  $10 = ($7|0)<($9|0);
  $11 = HEAP32[$0>>2]|0;
  if (!($10)) {
   break;
  }
  $12 = HEAP32[$pItem>>2]|0;
  $13 = HEAP32[$12>>2]|0;
  _sqlite3ExprDelete($11,$13);
  $14 = HEAP32[$0>>2]|0;
  $15 = HEAP32[$pItem>>2]|0;
  $16 = (($15) + 4|0);
  $17 = HEAP32[$16>>2]|0;
  _sqlite3DbFree($14,$17);
  $18 = HEAP32[$0>>2]|0;
  $19 = HEAP32[$pItem>>2]|0;
  $20 = (($19) + 8|0);
  $21 = HEAP32[$20>>2]|0;
  _sqlite3DbFree($18,$21);
  $22 = HEAP32[$i>>2]|0;
  $23 = (($22) + 1)|0;
  HEAP32[$i>>2] = $23;
  $24 = HEAP32[$pItem>>2]|0;
  $25 = (($24) + 20|0);
  HEAP32[$pItem>>2] = $25;
 }
 $26 = HEAP32[$1>>2]|0;
 $27 = (($26) + 4|0);
 $28 = HEAP32[$27>>2]|0;
 _sqlite3DbFree($11,$28);
 $29 = HEAP32[$0>>2]|0;
 $30 = HEAP32[$1>>2]|0;
 _sqlite3DbFree($29,$30);
 STACKTOP = sp;return;
}
function _sqlite3SelectDelete($db,$p) {
 $db = $db|0;
 $p = $p|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $1 = sp;
 HEAP32[$0>>2] = $db;
 HEAP32[$1>>2] = $p;
 $2 = HEAP32[$1>>2]|0;
 $3 = ($2|0)!=(0|0);
 if (!($3)) {
  STACKTOP = sp;return;
 }
 $4 = HEAP32[$0>>2]|0;
 $5 = HEAP32[$1>>2]|0;
 _clearSelect($4,$5);
 $6 = HEAP32[$0>>2]|0;
 $7 = HEAP32[$1>>2]|0;
 _sqlite3DbFree($6,$7);
 STACKTOP = sp;return;
}
function _clearSelect($db,$p) {
 $db = $db|0;
 $p = $p|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0;
 var $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $1 = sp;
 HEAP32[$0>>2] = $db;
 HEAP32[$1>>2] = $p;
 $2 = HEAP32[$0>>2]|0;
 $3 = HEAP32[$1>>2]|0;
 $4 = HEAP32[$3>>2]|0;
 _sqlite3ExprListDelete($2,$4);
 $5 = HEAP32[$0>>2]|0;
 $6 = HEAP32[$1>>2]|0;
 $7 = (($6) + 32|0);
 $8 = HEAP32[$7>>2]|0;
 _sqlite3SrcListDelete($5,$8);
 $9 = HEAP32[$0>>2]|0;
 $10 = HEAP32[$1>>2]|0;
 $11 = (($10) + 36|0);
 $12 = HEAP32[$11>>2]|0;
 _sqlite3ExprDelete($9,$12);
 $13 = HEAP32[$0>>2]|0;
 $14 = HEAP32[$1>>2]|0;
 $15 = (($14) + 40|0);
 $16 = HEAP32[$15>>2]|0;
 _sqlite3ExprListDelete($13,$16);
 $17 = HEAP32[$0>>2]|0;
 $18 = HEAP32[$1>>2]|0;
 $19 = (($18) + 44|0);
 $20 = HEAP32[$19>>2]|0;
 _sqlite3ExprDelete($17,$20);
 $21 = HEAP32[$0>>2]|0;
 $22 = HEAP32[$1>>2]|0;
 $23 = (($22) + 48|0);
 $24 = HEAP32[$23>>2]|0;
 _sqlite3ExprListDelete($21,$24);
 $25 = HEAP32[$0>>2]|0;
 $26 = HEAP32[$1>>2]|0;
 $27 = (($26) + 52|0);
 $28 = HEAP32[$27>>2]|0;
 _sqlite3SelectDelete($25,$28);
 $29 = HEAP32[$0>>2]|0;
 $30 = HEAP32[$1>>2]|0;
 $31 = (($30) + 60|0);
 $32 = HEAP32[$31>>2]|0;
 _sqlite3ExprDelete($29,$32);
 $33 = HEAP32[$0>>2]|0;
 $34 = HEAP32[$1>>2]|0;
 $35 = (($34) + 64|0);
 $36 = HEAP32[$35>>2]|0;
 _sqlite3ExprDelete($33,$36);
 $37 = HEAP32[$0>>2]|0;
 $38 = HEAP32[$1>>2]|0;
 $39 = (($38) + 68|0);
 $40 = HEAP32[$39>>2]|0;
 _sqlite3WithDelete($37,$40);
 STACKTOP = sp;return;
}
function _sqlite3SrcListDelete($db,$pList) {
 $db = $db|0;
 $pList = $pList|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $i = 0, $pItem = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $1 = sp + 8|0;
 $i = sp + 4|0;
 $pItem = sp;
 HEAP32[$0>>2] = $db;
 HEAP32[$1>>2] = $pList;
 $2 = HEAP32[$1>>2]|0;
 $3 = ($2|0)==(0|0);
 if ($3) {
  STACKTOP = sp;return;
 }
 $4 = HEAP32[$1>>2]|0;
 $5 = (($4) + 8|0);
 HEAP32[$pItem>>2] = $5;
 HEAP32[$i>>2] = 0;
 while(1) {
  $6 = HEAP32[$i>>2]|0;
  $7 = HEAP32[$1>>2]|0;
  $8 = HEAP32[$7>>2]|0;
  $9 = ($6|0)<($8|0);
  $10 = HEAP32[$0>>2]|0;
  if (!($9)) {
   break;
  }
  $11 = HEAP32[$pItem>>2]|0;
  $12 = (($11) + 4|0);
  $13 = HEAP32[$12>>2]|0;
  _sqlite3DbFree($10,$13);
  $14 = HEAP32[$0>>2]|0;
  $15 = HEAP32[$pItem>>2]|0;
  $16 = (($15) + 8|0);
  $17 = HEAP32[$16>>2]|0;
  _sqlite3DbFree($14,$17);
  $18 = HEAP32[$0>>2]|0;
  $19 = HEAP32[$pItem>>2]|0;
  $20 = (($19) + 12|0);
  $21 = HEAP32[$20>>2]|0;
  _sqlite3DbFree($18,$21);
  $22 = HEAP32[$0>>2]|0;
  $23 = HEAP32[$pItem>>2]|0;
  $24 = (($23) + 64|0);
  $25 = HEAP32[$24>>2]|0;
  _sqlite3DbFree($22,$25);
  $26 = HEAP32[$0>>2]|0;
  $27 = HEAP32[$pItem>>2]|0;
  $28 = (($27) + 16|0);
  $29 = HEAP32[$28>>2]|0;
  _sqlite3DeleteTable($26,$29);
  $30 = HEAP32[$0>>2]|0;
  $31 = HEAP32[$pItem>>2]|0;
  $32 = (($31) + 20|0);
  $33 = HEAP32[$32>>2]|0;
  _sqlite3SelectDelete($30,$33);
  $34 = HEAP32[$0>>2]|0;
  $35 = HEAP32[$pItem>>2]|0;
  $36 = (($35) + 44|0);
  $37 = HEAP32[$36>>2]|0;
  _sqlite3ExprDelete($34,$37);
  $38 = HEAP32[$0>>2]|0;
  $39 = HEAP32[$pItem>>2]|0;
  $40 = (($39) + 48|0);
  $41 = HEAP32[$40>>2]|0;
  _sqlite3IdListDelete($38,$41);
  $42 = HEAP32[$i>>2]|0;
  $43 = (($42) + 1)|0;
  HEAP32[$i>>2] = $43;
  $44 = HEAP32[$pItem>>2]|0;
  $45 = (($44) + 72|0);
  HEAP32[$pItem>>2] = $45;
 }
 $46 = HEAP32[$1>>2]|0;
 _sqlite3DbFree($10,$46);
 STACKTOP = sp;return;
}
function _sqlite3ExprDelete($db,$p) {
 $db = $db|0;
 $p = $p|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $5 = 0, $6 = 0;
 var $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $1 = sp;
 HEAP32[$0>>2] = $db;
 HEAP32[$1>>2] = $p;
 $2 = HEAP32[$1>>2]|0;
 $3 = ($2|0)==(0|0);
 if ($3) {
  STACKTOP = sp;return;
 }
 $4 = HEAP32[$1>>2]|0;
 $5 = (($4) + 4|0);
 $6 = HEAP32[$5>>2]|0;
 $7 = $6 & 16384;
 $8 = ($7|0)!=(0);
 do {
  if (!($8)) {
   $9 = HEAP32[$0>>2]|0;
   $10 = HEAP32[$1>>2]|0;
   $11 = (($10) + 12|0);
   $12 = HEAP32[$11>>2]|0;
   _sqlite3ExprDelete($9,$12);
   $13 = HEAP32[$0>>2]|0;
   $14 = HEAP32[$1>>2]|0;
   $15 = (($14) + 16|0);
   $16 = HEAP32[$15>>2]|0;
   _sqlite3ExprDelete($13,$16);
   $17 = HEAP32[$1>>2]|0;
   $18 = (($17) + 4|0);
   $19 = HEAP32[$18>>2]|0;
   $20 = $19 & 65536;
   $21 = ($20|0)!=(0);
   if ($21) {
    $22 = HEAP32[$0>>2]|0;
    $23 = HEAP32[$1>>2]|0;
    $24 = (($23) + 8|0);
    $25 = HEAP32[$24>>2]|0;
    _sqlite3DbFree($22,$25);
   }
   $26 = HEAP32[$1>>2]|0;
   $27 = (($26) + 4|0);
   $28 = HEAP32[$27>>2]|0;
   $29 = $28 & 2048;
   $30 = ($29|0)!=(0);
   $31 = HEAP32[$0>>2]|0;
   $32 = HEAP32[$1>>2]|0;
   $33 = (($32) + 20|0);
   if ($30) {
    $34 = HEAP32[$33>>2]|0;
    _sqlite3SelectDelete($31,$34);
    break;
   } else {
    $35 = HEAP32[$33>>2]|0;
    _sqlite3ExprListDelete($31,$35);
    break;
   }
  }
 } while(0);
 $36 = HEAP32[$1>>2]|0;
 $37 = (($36) + 4|0);
 $38 = HEAP32[$37>>2]|0;
 $39 = $38 & 32768;
 $40 = ($39|0)!=(0);
 if ($40) {
  STACKTOP = sp;return;
 }
 $41 = HEAP32[$0>>2]|0;
 $42 = HEAP32[$1>>2]|0;
 _sqlite3DbFree($41,$42);
 STACKTOP = sp;return;
}
function _sqlite3IdListDelete($db,$pList) {
 $db = $db|0;
 $pList = $pList|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0;
 var $9 = 0, $i = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $1 = sp + 4|0;
 $i = sp;
 HEAP32[$0>>2] = $db;
 HEAP32[$1>>2] = $pList;
 $2 = HEAP32[$1>>2]|0;
 $3 = ($2|0)==(0|0);
 if ($3) {
  STACKTOP = sp;return;
 }
 HEAP32[$i>>2] = 0;
 while(1) {
  $4 = HEAP32[$i>>2]|0;
  $5 = HEAP32[$1>>2]|0;
  $6 = (($5) + 4|0);
  $7 = HEAP32[$6>>2]|0;
  $8 = ($4|0)<($7|0);
  $9 = HEAP32[$0>>2]|0;
  if (!($8)) {
   break;
  }
  $10 = HEAP32[$i>>2]|0;
  $11 = HEAP32[$1>>2]|0;
  $12 = HEAP32[$11>>2]|0;
  $13 = (($12) + ($10<<3)|0);
  $14 = HEAP32[$13>>2]|0;
  _sqlite3DbFree($9,$14);
  $15 = HEAP32[$i>>2]|0;
  $16 = (($15) + 1)|0;
  HEAP32[$i>>2] = $16;
 }
 $17 = HEAP32[$1>>2]|0;
 $18 = HEAP32[$17>>2]|0;
 _sqlite3DbFree($9,$18);
 $19 = HEAP32[$0>>2]|0;
 $20 = HEAP32[$1>>2]|0;
 _sqlite3DbFree($19,$20);
 STACKTOP = sp;return;
}
function _sqlite3VMPrintf($db,$zFormat,$ap) {
 $db = $db|0;
 $zFormat = $zFormat|0;
 $ap = $ap|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $acc = 0, $z = 0, $zBase = 0;
 var label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 128|0;
 $0 = sp + 40|0;
 $1 = sp + 36|0;
 $2 = sp + 32|0;
 $z = sp + 28|0;
 $zBase = sp + 44|0;
 $acc = sp;
 HEAP32[$0>>2] = $db;
 HEAP32[$1>>2] = $zFormat;
 HEAP32[$2>>2] = $ap;
 $3 = HEAP32[$0>>2]|0;
 $4 = (($3) + 88|0);
 $5 = HEAP32[$4>>2]|0;
 _sqlite3StrAccumInit($acc,$zBase,70,$5);
 $6 = HEAP32[$0>>2]|0;
 HEAP32[$acc>>2] = $6;
 $7 = HEAP32[$1>>2]|0;
 $8 = HEAP32[$2>>2]|0;
 _sqlite3VXPrintf($acc,1,$7,$8);
 $9 = (_sqlite3StrAccumFinish($acc)|0);
 HEAP32[$z>>2] = $9;
 $10 = (($acc) + 25|0);
 $11 = HEAP8[$10>>0]|0;
 $12 = $11&255;
 $13 = ($12|0)==(1);
 if (!($13)) {
  $16 = HEAP32[$z>>2]|0;
  STACKTOP = sp;return ($16|0);
 }
 $14 = HEAP32[$0>>2]|0;
 $15 = (($14) + 64|0);
 HEAP8[$15>>0] = 1;
 $16 = HEAP32[$z>>2]|0;
 STACKTOP = sp;return ($16|0);
}
function _yy_pop_parser_stack($pParser) {
 $pParser = $pParser|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0;
 var $7 = 0, $8 = 0, $9 = 0, $yymajor = 0, $yytos = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $1 = sp + 4|0;
 $yymajor = sp + 12|0;
 $yytos = sp;
 HEAP32[$1>>2] = $pParser;
 $2 = HEAP32[$1>>2]|0;
 $3 = HEAP32[$2>>2]|0;
 $4 = HEAP32[$1>>2]|0;
 $5 = (($4) + 12|0);
 $6 = (($5) + ($3<<4)|0);
 HEAP32[$yytos>>2] = $6;
 $7 = HEAP32[$1>>2]|0;
 $8 = HEAP32[$7>>2]|0;
 $9 = ($8|0)<(0);
 if ($9) {
  HEAP32[$0>>2] = 0;
  $22 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($22|0);
 } else {
  $10 = HEAP32[$yytos>>2]|0;
  $11 = (($10) + 2|0);
  $12 = HEAP8[$11>>0]|0;
  HEAP8[$yymajor>>0] = $12;
  $13 = HEAP32[$1>>2]|0;
  $14 = HEAP8[$yymajor>>0]|0;
  $15 = HEAP32[$yytos>>2]|0;
  $16 = (($15) + 4|0);
  _yy_destructor($13,$14,$16);
  $17 = HEAP32[$1>>2]|0;
  $18 = HEAP32[$17>>2]|0;
  $19 = (($18) + -1)|0;
  HEAP32[$17>>2] = $19;
  $20 = HEAP8[$yymajor>>0]|0;
  $21 = $20&255;
  HEAP32[$0>>2] = $21;
  $22 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($22|0);
 }
 return 0|0;
}
function _yy_destructor($yypParser,$yymajor,$yypminor) {
 $yypParser = $yypParser|0;
 $yymajor = $yymajor|0;
 $yypminor = $yypminor|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $pParse = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $1 = sp + 12|0;
 $2 = sp + 4|0;
 $pParse = sp;
 HEAP32[$0>>2] = $yypParser;
 HEAP8[$1>>0] = $yymajor;
 HEAP32[$2>>2] = $yypminor;
 $3 = HEAP32[$0>>2]|0;
 $4 = (($3) + 8|0);
 $5 = HEAP32[$4>>2]|0;
 HEAP32[$pParse>>2] = $5;
 $6 = HEAP8[$1>>0]|0;
 $7 = $6&255;
 do {
  switch ($7|0) {
  case 252: case 197:  {
   $24 = HEAP32[$pParse>>2]|0;
   $25 = HEAP32[$24>>2]|0;
   $26 = HEAP32[$2>>2]|0;
   $27 = HEAP32[$26>>2]|0;
   _sqlite3WithDelete($25,$27);
   STACKTOP = sp;return;
   break;
  }
  case 207: case 196: case 195: case 163:  {
   $8 = HEAP32[$pParse>>2]|0;
   $9 = HEAP32[$8>>2]|0;
   $10 = HEAP32[$2>>2]|0;
   $11 = HEAP32[$10>>2]|0;
   _sqlite3SelectDelete($9,$11);
   STACKTOP = sp;return;
   break;
  }
  case 228: case 221: case 220: case 210: case 209: case 208: case 205: case 203: case 200: case 188: case 179:  {
   $16 = HEAP32[$pParse>>2]|0;
   $17 = HEAP32[$16>>2]|0;
   $18 = HEAP32[$2>>2]|0;
   $19 = HEAP32[$18>>2]|0;
   _sqlite3ExprListDelete($17,$19);
   STACKTOP = sp;return;
   break;
  }
  case 239: case 234:  {
   $36 = HEAP32[$pParse>>2]|0;
   $37 = HEAP32[$36>>2]|0;
   $38 = HEAP32[$2>>2]|0;
   $39 = HEAP32[$38>>2]|0;
   _sqlite3DeleteTriggerStep($37,$39);
   STACKTOP = sp;return;
   break;
  }
  case 223: case 219: case 217:  {
   $32 = HEAP32[$pParse>>2]|0;
   $33 = HEAP32[$32>>2]|0;
   $34 = HEAP32[$2>>2]|0;
   $35 = HEAP32[$34>>2]|0;
   _sqlite3IdListDelete($33,$35);
   STACKTOP = sp;return;
   break;
  }
  case 236:  {
   $40 = HEAP32[$pParse>>2]|0;
   $41 = HEAP32[$40>>2]|0;
   $42 = HEAP32[$2>>2]|0;
   $43 = (($42) + 4|0);
   $44 = HEAP32[$43>>2]|0;
   _sqlite3IdListDelete($41,$44);
   STACKTOP = sp;return;
   break;
  }
  case 175: case 174:  {
   $12 = HEAP32[$pParse>>2]|0;
   $13 = HEAP32[$12>>2]|0;
   $14 = HEAP32[$2>>2]|0;
   $15 = HEAP32[$14>>2]|0;
   _sqlite3ExprDelete($13,$15);
   STACKTOP = sp;return;
   break;
  }
  case 213: case 212: case 201: case 194:  {
   $20 = HEAP32[$pParse>>2]|0;
   $21 = HEAP32[$20>>2]|0;
   $22 = HEAP32[$2>>2]|0;
   $23 = HEAP32[$22>>2]|0;
   _sqlite3SrcListDelete($21,$23);
   STACKTOP = sp;return;
   break;
  }
  case 243: case 238: case 229: case 227: case 216: case 204: case 202:  {
   $28 = HEAP32[$pParse>>2]|0;
   $29 = HEAP32[$28>>2]|0;
   $30 = HEAP32[$2>>2]|0;
   $31 = HEAP32[$30>>2]|0;
   _sqlite3ExprDelete($29,$31);
   STACKTOP = sp;return;
   break;
  }
  default: {
   STACKTOP = sp;return;
  }
  }
 } while(0);
}
function _sqlite3DeleteTriggerStep($db,$pTriggerStep) {
 $db = $db|0;
 $pTriggerStep = $pTriggerStep|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $3 = 0;
 var $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $pTmp = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $1 = sp + 4|0;
 $pTmp = sp;
 HEAP32[$0>>2] = $db;
 HEAP32[$1>>2] = $pTriggerStep;
 while(1) {
  $2 = HEAP32[$1>>2]|0;
  $3 = ($2|0)!=(0|0);
  if (!($3)) {
   break;
  }
  $4 = HEAP32[$1>>2]|0;
  HEAP32[$pTmp>>2] = $4;
  $5 = HEAP32[$1>>2]|0;
  $6 = (($5) + 32|0);
  $7 = HEAP32[$6>>2]|0;
  HEAP32[$1>>2] = $7;
  $8 = HEAP32[$0>>2]|0;
  $9 = HEAP32[$pTmp>>2]|0;
  $10 = (($9) + 20|0);
  $11 = HEAP32[$10>>2]|0;
  _sqlite3ExprDelete($8,$11);
  $12 = HEAP32[$0>>2]|0;
  $13 = HEAP32[$pTmp>>2]|0;
  $14 = (($13) + 24|0);
  $15 = HEAP32[$14>>2]|0;
  _sqlite3ExprListDelete($12,$15);
  $16 = HEAP32[$0>>2]|0;
  $17 = HEAP32[$pTmp>>2]|0;
  $18 = (($17) + 8|0);
  $19 = HEAP32[$18>>2]|0;
  _sqlite3SelectDelete($16,$19);
  $20 = HEAP32[$0>>2]|0;
  $21 = HEAP32[$pTmp>>2]|0;
  $22 = (($21) + 28|0);
  $23 = HEAP32[$22>>2]|0;
  _sqlite3IdListDelete($20,$23);
  $24 = HEAP32[$0>>2]|0;
  $25 = HEAP32[$pTmp>>2]|0;
  _sqlite3DbFree($24,$25);
 }
 STACKTOP = sp;return;
}
function _yy_find_shift_action($pParser,$iLookAhead) {
 $pParser = $pParser|0;
 $iLookAhead = $iLookAhead|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $8 = 0, $9 = 0, $i = 0, $iFallback = 0, $j = 0, $or$cond = 0;
 var $stateno = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 16|0;
 $1 = sp + 12|0;
 $2 = sp + 21|0;
 $i = sp + 8|0;
 $stateno = sp + 4|0;
 $iFallback = sp + 20|0;
 $j = sp;
 HEAP32[$1>>2] = $pParser;
 HEAP8[$2>>0] = $iLookAhead;
 $3 = HEAP32[$1>>2]|0;
 $4 = HEAP32[$3>>2]|0;
 $5 = HEAP32[$1>>2]|0;
 $6 = (($5) + 12|0);
 $7 = (($6) + ($4<<4)|0);
 $8 = HEAP16[$7>>1]|0;
 $9 = $8&65535;
 HEAP32[$stateno>>2] = $9;
 $10 = HEAP32[$stateno>>2]|0;
 $11 = ($10|0)>(429);
 if (!($11)) {
  $12 = HEAP32[$stateno>>2]|0;
  $13 = (27208 + ($12<<1)|0);
  $14 = HEAP16[$13>>1]|0;
  $15 = $14 << 16 >> 16;
  HEAP32[$i>>2] = $15;
  $16 = ($15|0)==(-86);
  if (!($16)) {
   $21 = HEAP8[$2>>0]|0;
   $22 = $21&255;
   $23 = HEAP32[$i>>2]|0;
   $24 = (($23) + ($22))|0;
   HEAP32[$i>>2] = $24;
   $25 = HEAP32[$i>>2]|0;
   $26 = ($25|0)<(0);
   $27 = HEAP32[$i>>2]|0;
   $28 = ($27|0)>=(1497);
   $or$cond = $26 | $28;
   if (!($or$cond)) {
    $29 = HEAP32[$i>>2]|0;
    $30 = (29360 + ($29)|0);
    $31 = HEAP8[$30>>0]|0;
    $32 = $31&255;
    $33 = HEAP8[$2>>0]|0;
    $34 = $33&255;
    $35 = ($32|0)!=($34|0);
    if (!($35)) {
     $71 = HEAP32[$i>>2]|0;
     $72 = (11720 + ($71<<1)|0);
     $73 = HEAP16[$72>>1]|0;
     $74 = $73&65535;
     HEAP32[$0>>2] = $74;
     $75 = HEAP32[$0>>2]|0;
     STACKTOP = sp;return ($75|0);
    }
   }
   $36 = HEAP8[$2>>0]|0;
   $37 = $36&255;
   $38 = ($37|0)>(0);
   if ($38) {
    $39 = HEAP8[$2>>0]|0;
    $40 = $39&255;
    $41 = ($40>>>0)<(70);
    if ($41) {
     $42 = HEAP8[$2>>0]|0;
     $43 = $42&255;
     $44 = (30864 + ($43)|0);
     $45 = HEAP8[$44>>0]|0;
     HEAP8[$iFallback>>0] = $45;
     $46 = $45&255;
     $47 = ($46|0)!=(0);
     if ($47) {
      $48 = HEAP32[$1>>2]|0;
      $49 = HEAP8[$iFallback>>0]|0;
      $50 = (_yy_find_shift_action($48,$49)|0);
      HEAP32[$0>>2] = $50;
      $75 = HEAP32[$0>>2]|0;
      STACKTOP = sp;return ($75|0);
     }
    }
    $51 = HEAP32[$i>>2]|0;
    $52 = HEAP8[$2>>0]|0;
    $53 = $52&255;
    $54 = (($51) - ($53))|0;
    $55 = (($54) + 70)|0;
    HEAP32[$j>>2] = $55;
    $56 = HEAP32[$j>>2]|0;
    $57 = ($56|0)>=(0);
    if ($57) {
     $58 = HEAP32[$j>>2]|0;
     $59 = (29360 + ($58)|0);
     $60 = HEAP8[$59>>0]|0;
     $61 = $60&255;
     $62 = ($61|0)==(70);
     if ($62) {
      $63 = HEAP32[$j>>2]|0;
      $64 = (11720 + ($63<<1)|0);
      $65 = HEAP16[$64>>1]|0;
      $66 = $65&65535;
      HEAP32[$0>>2] = $66;
      $75 = HEAP32[$0>>2]|0;
      STACKTOP = sp;return ($75|0);
     }
    }
   }
   $67 = HEAP32[$stateno>>2]|0;
   $68 = (28072 + ($67<<1)|0);
   $69 = HEAP16[$68>>1]|0;
   $70 = $69&65535;
   HEAP32[$0>>2] = $70;
   $75 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($75|0);
  }
 }
 $17 = HEAP32[$stateno>>2]|0;
 $18 = (28072 + ($17<<1)|0);
 $19 = HEAP16[$18>>1]|0;
 $20 = $19&65535;
 HEAP32[$0>>2] = $20;
 $75 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($75|0);
}
function _yy_shift($yypParser,$yyNewState,$yyMajor,$yypMinor) {
 $yypParser = $yypParser|0;
 $yyNewState = $yyNewState|0;
 $yyMajor = $yyMajor|0;
 $yypMinor = $yypMinor|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $3 = 0;
 var $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $yytos = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 16|0;
 $1 = sp + 12|0;
 $2 = sp + 8|0;
 $3 = sp + 4|0;
 $yytos = sp;
 HEAP32[$0>>2] = $yypParser;
 HEAP32[$1>>2] = $yyNewState;
 HEAP32[$2>>2] = $yyMajor;
 HEAP32[$3>>2] = $yypMinor;
 $4 = HEAP32[$0>>2]|0;
 $5 = HEAP32[$4>>2]|0;
 $6 = (($5) + 1)|0;
 HEAP32[$4>>2] = $6;
 $7 = HEAP32[$0>>2]|0;
 $8 = HEAP32[$7>>2]|0;
 $9 = ($8|0)>=(100);
 $10 = HEAP32[$0>>2]|0;
 if ($9) {
  $11 = HEAP32[$3>>2]|0;
  _yyStackOverflow($10,$11);
  STACKTOP = sp;return;
 } else {
  $12 = HEAP32[$10>>2]|0;
  $13 = HEAP32[$0>>2]|0;
  $14 = (($13) + 12|0);
  $15 = (($14) + ($12<<4)|0);
  HEAP32[$yytos>>2] = $15;
  $16 = HEAP32[$1>>2]|0;
  $17 = $16&65535;
  $18 = HEAP32[$yytos>>2]|0;
  HEAP16[$18>>1] = $17;
  $19 = HEAP32[$2>>2]|0;
  $20 = $19&255;
  $21 = HEAP32[$yytos>>2]|0;
  $22 = (($21) + 2|0);
  HEAP8[$22>>0] = $20;
  $23 = HEAP32[$yytos>>2]|0;
  $24 = (($23) + 4|0);
  $25 = HEAP32[$3>>2]|0;
  ;HEAP32[$24+0>>2]=HEAP32[$25+0>>2]|0;HEAP32[$24+4>>2]=HEAP32[$25+4>>2]|0;HEAP32[$24+8>>2]=HEAP32[$25+8>>2]|0;
  STACKTOP = sp;return;
 }
}
function _yy_reduce($yypParser,$yyruleno) {
 $yypParser = $yypParser|0;
 $yyruleno = $yyruleno|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $1000 = 0, $1001 = 0, $1002 = 0, $1003 = 0, $1004 = 0, $1005 = 0, $1006 = 0, $1007 = 0, $1008 = 0, $1009 = 0, $101 = 0, $1010 = 0, $1011 = 0, $1012 = 0, $1013 = 0, $1014 = 0;
 var $1015 = 0, $1016 = 0, $1017 = 0, $1018 = 0, $1019 = 0, $102 = 0, $1020 = 0, $1021 = 0, $1022 = 0, $1023 = 0, $1024 = 0, $1025 = 0, $1026 = 0, $1027 = 0, $1028 = 0, $1029 = 0, $103 = 0, $1030 = 0, $1031 = 0, $1032 = 0;
 var $1033 = 0, $1034 = 0, $1035 = 0, $1036 = 0, $1037 = 0, $1038 = 0, $1039 = 0, $104 = 0, $1040 = 0, $1041 = 0, $1042 = 0, $1043 = 0, $1044 = 0, $1045 = 0, $1046 = 0, $1047 = 0, $1048 = 0, $1049 = 0, $105 = 0, $1050 = 0;
 var $1051 = 0, $1052 = 0, $1053 = 0, $1054 = 0, $1055 = 0, $1056 = 0, $1057 = 0, $1058 = 0, $1059 = 0, $106 = 0, $1060 = 0, $1061 = 0, $1062 = 0, $1063 = 0, $1064 = 0, $1065 = 0, $1066 = 0, $1067 = 0, $1068 = 0, $1069 = 0;
 var $107 = 0, $1070 = 0, $1071 = 0, $1072 = 0, $1073 = 0, $1074 = 0, $1075 = 0, $1076 = 0, $1077 = 0, $1078 = 0, $1079 = 0, $108 = 0, $1080 = 0, $1081 = 0, $1082 = 0, $1083 = 0, $1084 = 0, $1085 = 0, $1086 = 0, $1087 = 0;
 var $1088 = 0, $1089 = 0, $109 = 0, $1090 = 0, $1091 = 0, $1092 = 0, $1093 = 0, $1094 = 0, $1095 = 0, $1096 = 0, $1097 = 0, $1098 = 0, $1099 = 0, $11 = 0, $110 = 0, $1100 = 0, $1101 = 0, $1102 = 0, $1103 = 0, $1104 = 0;
 var $1105 = 0, $1106 = 0, $1107 = 0, $1108 = 0, $1109 = 0, $111 = 0, $1110 = 0, $1111 = 0, $1112 = 0, $1113 = 0, $1114 = 0, $1115 = 0, $1116 = 0, $1117 = 0, $1118 = 0, $1119 = 0, $112 = 0, $1120 = 0, $1121 = 0, $1122 = 0;
 var $1123 = 0, $1124 = 0, $1125 = 0, $1126 = 0, $1127 = 0, $1128 = 0, $1129 = 0, $113 = 0, $1130 = 0, $1131 = 0, $1132 = 0, $1133 = 0, $1134 = 0, $1135 = 0, $1136 = 0, $1137 = 0, $1138 = 0, $1139 = 0, $114 = 0, $1140 = 0;
 var $1141 = 0, $1142 = 0, $1143 = 0, $1144 = 0, $1145 = 0, $1146 = 0, $1147 = 0, $1148 = 0, $1149 = 0, $115 = 0, $1150 = 0, $1151 = 0, $1152 = 0, $1153 = 0, $1154 = 0, $1155 = 0, $1156 = 0, $1157 = 0, $1158 = 0, $1159 = 0;
 var $116 = 0, $1160 = 0, $1161 = 0, $1162 = 0, $1163 = 0, $1164 = 0, $1165 = 0, $1166 = 0, $1167 = 0, $1168 = 0, $1169 = 0, $117 = 0, $1170 = 0, $1171 = 0, $1172 = 0, $1173 = 0, $1174 = 0, $1175 = 0, $1176 = 0, $1177 = 0;
 var $1178 = 0, $1179 = 0, $118 = 0, $1180 = 0, $1181 = 0, $1182 = 0, $1183 = 0, $1184 = 0, $1185 = 0, $1186 = 0, $1187 = 0, $1188 = 0, $1189 = 0, $119 = 0, $1190 = 0, $1191 = 0, $1192 = 0, $1193 = 0, $1194 = 0, $1195 = 0;
 var $1196 = 0, $1197 = 0, $1198 = 0, $1199 = 0, $12 = 0, $120 = 0, $1200 = 0, $1201 = 0, $1202 = 0, $1203 = 0, $1204 = 0, $1205 = 0, $1206 = 0, $1207 = 0, $1208 = 0, $1209 = 0, $121 = 0, $1210 = 0, $1211 = 0, $1212 = 0;
 var $1213 = 0, $1214 = 0, $1215 = 0, $1216 = 0, $1217 = 0, $1218 = 0, $1219 = 0, $122 = 0, $1220 = 0, $1221 = 0, $1222 = 0, $1223 = 0, $1224 = 0, $1225 = 0, $1226 = 0, $1227 = 0, $1228 = 0, $1229 = 0, $123 = 0, $1230 = 0;
 var $1231 = 0, $1232 = 0, $1233 = 0, $1234 = 0, $1235 = 0, $1236 = 0, $1237 = 0, $1238 = 0, $1239 = 0, $124 = 0, $1240 = 0, $1241 = 0, $1242 = 0, $1243 = 0, $1244 = 0, $1245 = 0, $1246 = 0, $1247 = 0, $1248 = 0, $1249 = 0;
 var $125 = 0, $1250 = 0, $1251 = 0, $1252 = 0, $1253 = 0, $1254 = 0, $1255 = 0, $1256 = 0, $1257 = 0, $1258 = 0, $1259 = 0, $126 = 0, $1260 = 0, $1261 = 0, $1262 = 0, $1263 = 0, $1264 = 0, $1265 = 0, $1266 = 0, $1267 = 0;
 var $1268 = 0, $1269 = 0, $127 = 0, $1270 = 0, $1271 = 0, $1272 = 0, $1273 = 0, $1274 = 0, $1275 = 0, $1276 = 0, $1277 = 0, $1278 = 0, $1279 = 0, $128 = 0, $1280 = 0, $1281 = 0, $1282 = 0, $1283 = 0, $1284 = 0, $1285 = 0;
 var $1286 = 0, $1287 = 0, $1288 = 0, $1289 = 0, $129 = 0, $1290 = 0, $1291 = 0, $1292 = 0, $1293 = 0, $1294 = 0, $1295 = 0, $1296 = 0, $1297 = 0, $1298 = 0, $1299 = 0, $13 = 0, $130 = 0, $1300 = 0, $1301 = 0, $1302 = 0;
 var $1303 = 0, $1304 = 0, $1305 = 0, $1306 = 0, $1307 = 0, $1308 = 0, $1309 = 0, $131 = 0, $1310 = 0, $1311 = 0, $1312 = 0, $1313 = 0, $1314 = 0, $1315 = 0, $1316 = 0, $1317 = 0, $1318 = 0, $1319 = 0, $132 = 0, $1320 = 0;
 var $1321 = 0, $1322 = 0, $1323 = 0, $1324 = 0, $1325 = 0, $1326 = 0, $1327 = 0, $1328 = 0, $1329 = 0, $133 = 0, $1330 = 0, $1331 = 0, $1332 = 0, $1333 = 0, $1334 = 0, $1335 = 0, $1336 = 0, $1337 = 0, $1338 = 0, $1339 = 0;
 var $134 = 0, $1340 = 0, $1341 = 0, $1342 = 0, $1343 = 0, $1344 = 0, $1345 = 0, $1346 = 0, $1347 = 0, $1348 = 0, $1349 = 0, $135 = 0, $1350 = 0, $1351 = 0, $1352 = 0, $1353 = 0, $1354 = 0, $1355 = 0, $1356 = 0, $1357 = 0;
 var $1358 = 0, $1359 = 0, $136 = 0, $1360 = 0, $1361 = 0, $1362 = 0, $1363 = 0, $1364 = 0, $1365 = 0, $1366 = 0, $1367 = 0, $1368 = 0, $1369 = 0, $137 = 0, $1370 = 0, $1371 = 0, $1372 = 0, $1373 = 0, $1374 = 0, $1375 = 0;
 var $1376 = 0, $1377 = 0, $1378 = 0, $1379 = 0, $138 = 0, $1380 = 0, $1381 = 0, $1382 = 0, $1383 = 0, $1384 = 0, $1385 = 0, $1386 = 0, $1387 = 0, $1388 = 0, $1389 = 0, $139 = 0, $1390 = 0, $1391 = 0, $1392 = 0, $1393 = 0;
 var $1394 = 0, $1395 = 0, $1396 = 0, $1397 = 0, $1398 = 0, $1399 = 0, $14 = 0, $140 = 0, $1400 = 0, $1401 = 0, $1402 = 0, $1403 = 0, $1404 = 0, $1405 = 0, $1406 = 0, $1407 = 0, $1408 = 0, $1409 = 0, $141 = 0, $1410 = 0;
 var $1411 = 0, $1412 = 0, $1413 = 0, $1414 = 0, $1415 = 0, $1416 = 0, $1417 = 0, $1418 = 0, $1419 = 0, $142 = 0, $1420 = 0, $1421 = 0, $1422 = 0, $1423 = 0, $1424 = 0, $1425 = 0, $1426 = 0, $1427 = 0, $1428 = 0, $1429 = 0;
 var $143 = 0, $1430 = 0, $1431 = 0, $1432 = 0, $1433 = 0, $1434 = 0, $1435 = 0, $1436 = 0, $1437 = 0, $1438 = 0, $1439 = 0, $144 = 0, $1440 = 0, $1441 = 0, $1442 = 0, $1443 = 0, $1444 = 0, $1445 = 0, $1446 = 0, $1447 = 0;
 var $1448 = 0, $1449 = 0, $145 = 0, $1450 = 0, $1451 = 0, $1452 = 0, $1453 = 0, $1454 = 0, $1455 = 0, $1456 = 0, $1457 = 0, $1458 = 0, $1459 = 0, $146 = 0, $1460 = 0, $1461 = 0, $1462 = 0, $1463 = 0, $1464 = 0, $1465 = 0;
 var $1466 = 0, $1467 = 0, $1468 = 0, $1469 = 0, $147 = 0, $1470 = 0, $1471 = 0, $1472 = 0, $1473 = 0, $1474 = 0, $1475 = 0, $1476 = 0, $1477 = 0, $1478 = 0, $1479 = 0, $148 = 0, $1480 = 0, $1481 = 0, $1482 = 0, $1483 = 0;
 var $1484 = 0, $1485 = 0, $1486 = 0, $1487 = 0, $1488 = 0, $1489 = 0, $149 = 0, $1490 = 0, $1491 = 0, $1492 = 0, $1493 = 0, $1494 = 0, $1495 = 0, $1496 = 0, $1497 = 0, $1498 = 0, $1499 = 0, $15 = 0, $150 = 0, $1500 = 0;
 var $1501 = 0, $1502 = 0, $1503 = 0, $1504 = 0, $1505 = 0, $1506 = 0, $1507 = 0, $1508 = 0, $1509 = 0, $151 = 0, $1510 = 0, $1511 = 0, $1512 = 0, $1513 = 0, $1514 = 0, $1515 = 0, $1516 = 0, $1517 = 0, $1518 = 0, $1519 = 0;
 var $152 = 0, $1520 = 0, $1521 = 0, $1522 = 0, $1523 = 0, $1524 = 0, $1525 = 0, $1526 = 0, $1527 = 0, $1528 = 0, $1529 = 0, $153 = 0, $1530 = 0, $1531 = 0, $1532 = 0, $1533 = 0, $1534 = 0, $1535 = 0, $1536 = 0, $1537 = 0;
 var $1538 = 0, $1539 = 0, $154 = 0, $1540 = 0, $1541 = 0, $1542 = 0, $1543 = 0, $1544 = 0, $1545 = 0, $1546 = 0, $1547 = 0, $1548 = 0, $1549 = 0, $155 = 0, $1550 = 0, $1551 = 0, $1552 = 0, $1553 = 0, $1554 = 0, $1555 = 0;
 var $1556 = 0, $1557 = 0, $1558 = 0, $1559 = 0, $156 = 0, $1560 = 0, $1561 = 0, $1562 = 0, $1563 = 0, $1564 = 0, $1565 = 0, $1566 = 0, $1567 = 0, $1568 = 0, $1569 = 0, $157 = 0, $1570 = 0, $1571 = 0, $1572 = 0, $1573 = 0;
 var $1574 = 0, $1575 = 0, $1576 = 0, $1577 = 0, $1578 = 0, $1579 = 0, $158 = 0, $1580 = 0, $1581 = 0, $1582 = 0, $1583 = 0, $1584 = 0, $1585 = 0, $1586 = 0, $1587 = 0, $1588 = 0, $1589 = 0, $159 = 0, $1590 = 0, $1591 = 0;
 var $1592 = 0, $1593 = 0, $1594 = 0, $1595 = 0, $1596 = 0, $1597 = 0, $1598 = 0, $1599 = 0, $16 = 0, $160 = 0, $1600 = 0, $1601 = 0, $1602 = 0, $1603 = 0, $1604 = 0, $1605 = 0, $1606 = 0, $1607 = 0, $1608 = 0, $1609 = 0;
 var $161 = 0, $1610 = 0, $1611 = 0, $1612 = 0, $1613 = 0, $1614 = 0, $1615 = 0, $1616 = 0, $1617 = 0, $1618 = 0, $1619 = 0, $162 = 0, $1620 = 0, $1621 = 0, $1622 = 0, $1623 = 0, $1624 = 0, $1625 = 0, $1626 = 0, $1627 = 0;
 var $1628 = 0, $1629 = 0, $163 = 0, $1630 = 0, $1631 = 0, $1632 = 0, $1633 = 0, $1634 = 0, $1635 = 0, $1636 = 0, $1637 = 0, $1638 = 0, $1639 = 0, $164 = 0, $1640 = 0, $1641 = 0, $1642 = 0, $1643 = 0, $1644 = 0, $1645 = 0;
 var $1646 = 0, $1647 = 0, $1648 = 0, $1649 = 0, $165 = 0, $1650 = 0, $1651 = 0, $1652 = 0, $1653 = 0, $1654 = 0, $1655 = 0, $1656 = 0, $1657 = 0, $1658 = 0, $1659 = 0, $166 = 0, $1660 = 0, $1661 = 0, $1662 = 0, $1663 = 0;
 var $1664 = 0, $1665 = 0, $1666 = 0, $1667 = 0, $1668 = 0, $1669 = 0, $167 = 0, $1670 = 0, $1671 = 0, $1672 = 0, $1673 = 0, $1674 = 0, $1675 = 0, $1676 = 0, $1677 = 0, $1678 = 0, $1679 = 0, $168 = 0, $1680 = 0, $1681 = 0;
 var $1682 = 0, $1683 = 0, $1684 = 0, $1685 = 0, $1686 = 0, $1687 = 0, $1688 = 0, $1689 = 0, $169 = 0, $1690 = 0, $1691 = 0, $1692 = 0, $1693 = 0, $1694 = 0, $1695 = 0, $1696 = 0, $1697 = 0, $1698 = 0, $1699 = 0, $17 = 0;
 var $170 = 0, $1700 = 0, $1701 = 0, $1702 = 0, $1703 = 0, $1704 = 0, $1705 = 0, $1706 = 0, $1707 = 0, $1708 = 0, $1709 = 0, $171 = 0, $1710 = 0, $1711 = 0, $1712 = 0, $1713 = 0, $1714 = 0, $1715 = 0, $1716 = 0, $1717 = 0;
 var $1718 = 0, $1719 = 0, $172 = 0, $1720 = 0, $1721 = 0, $1722 = 0, $1723 = 0, $1724 = 0, $1725 = 0, $1726 = 0, $1727 = 0, $1728 = 0, $1729 = 0, $173 = 0, $1730 = 0, $1731 = 0, $1732 = 0, $1733 = 0, $1734 = 0, $1735 = 0;
 var $1736 = 0, $1737 = 0, $1738 = 0, $1739 = 0, $174 = 0, $1740 = 0, $1741 = 0, $1742 = 0, $1743 = 0, $1744 = 0, $1745 = 0, $1746 = 0, $1747 = 0, $1748 = 0, $1749 = 0, $175 = 0, $1750 = 0, $1751 = 0, $1752 = 0, $1753 = 0;
 var $1754 = 0, $1755 = 0, $1756 = 0, $1757 = 0, $1758 = 0, $1759 = 0, $176 = 0, $1760 = 0, $1761 = 0, $1762 = 0, $1763 = 0, $1764 = 0, $1765 = 0, $1766 = 0, $1767 = 0, $1768 = 0, $1769 = 0, $177 = 0, $1770 = 0, $1771 = 0;
 var $1772 = 0, $1773 = 0, $1774 = 0, $1775 = 0, $1776 = 0, $1777 = 0, $1778 = 0, $1779 = 0, $178 = 0, $1780 = 0, $1781 = 0, $1782 = 0, $1783 = 0, $1784 = 0, $1785 = 0, $1786 = 0, $1787 = 0, $1788 = 0, $1789 = 0, $179 = 0;
 var $1790 = 0, $1791 = 0, $1792 = 0, $1793 = 0, $1794 = 0, $1795 = 0, $1796 = 0, $1797 = 0, $1798 = 0, $1799 = 0, $18 = 0, $180 = 0, $1800 = 0, $1801 = 0, $1802 = 0, $1803 = 0, $1804 = 0, $1805 = 0, $1806 = 0, $1807 = 0;
 var $1808 = 0, $1809 = 0, $181 = 0, $1810 = 0, $1811 = 0, $1812 = 0, $1813 = 0, $1814 = 0, $1815 = 0, $1816 = 0, $1817 = 0, $1818 = 0, $1819 = 0, $182 = 0, $1820 = 0, $1821 = 0, $1822 = 0, $1823 = 0, $1824 = 0, $1825 = 0;
 var $1826 = 0, $1827 = 0, $1828 = 0, $1829 = 0, $183 = 0, $1830 = 0, $1831 = 0, $1832 = 0, $1833 = 0, $1834 = 0, $1835 = 0, $1836 = 0, $1837 = 0, $1838 = 0, $1839 = 0, $184 = 0, $1840 = 0, $1841 = 0, $1842 = 0, $1843 = 0;
 var $1844 = 0, $1845 = 0, $1846 = 0, $1847 = 0, $1848 = 0, $1849 = 0, $185 = 0, $1850 = 0, $1851 = 0, $1852 = 0, $1853 = 0, $1854 = 0, $1855 = 0, $1856 = 0, $1857 = 0, $1858 = 0, $1859 = 0, $186 = 0, $1860 = 0, $1861 = 0;
 var $1862 = 0, $1863 = 0, $1864 = 0, $1865 = 0, $1866 = 0, $1867 = 0, $1868 = 0, $1869 = 0, $187 = 0, $1870 = 0, $1871 = 0, $1872 = 0, $1873 = 0, $1874 = 0, $1875 = 0, $1876 = 0, $1877 = 0, $1878 = 0, $1879 = 0, $188 = 0;
 var $1880 = 0, $1881 = 0, $1882 = 0, $1883 = 0, $1884 = 0, $1885 = 0, $1886 = 0, $1887 = 0, $1888 = 0, $1889 = 0, $189 = 0, $1890 = 0, $1891 = 0, $1892 = 0, $1893 = 0, $1894 = 0, $1895 = 0, $1896 = 0, $1897 = 0, $1898 = 0;
 var $1899 = 0, $19 = 0, $190 = 0, $1900 = 0, $1901 = 0, $1902 = 0, $1903 = 0, $1904 = 0, $1905 = 0, $1906 = 0, $1907 = 0, $1908 = 0, $1909 = 0, $191 = 0, $1910 = 0, $1911 = 0, $1912 = 0, $1913 = 0, $1914 = 0, $1915 = 0;
 var $1916 = 0, $1917 = 0, $1918 = 0, $1919 = 0, $192 = 0, $1920 = 0, $1921 = 0, $1922 = 0, $1923 = 0, $1924 = 0, $1925 = 0, $1926 = 0, $1927 = 0, $1928 = 0, $1929 = 0, $193 = 0, $1930 = 0, $1931 = 0, $1932 = 0, $1933 = 0;
 var $1934 = 0, $1935 = 0, $1936 = 0, $1937 = 0, $1938 = 0, $1939 = 0, $194 = 0, $1940 = 0, $1941 = 0, $1942 = 0, $1943 = 0, $1944 = 0, $1945 = 0, $1946 = 0, $1947 = 0, $1948 = 0, $1949 = 0, $195 = 0, $1950 = 0, $1951 = 0;
 var $1952 = 0, $1953 = 0, $1954 = 0, $1955 = 0, $1956 = 0, $1957 = 0, $1958 = 0, $1959 = 0, $196 = 0, $1960 = 0, $1961 = 0, $1962 = 0, $1963 = 0, $1964 = 0, $1965 = 0, $1966 = 0, $1967 = 0, $1968 = 0, $1969 = 0, $197 = 0;
 var $1970 = 0, $1971 = 0, $1972 = 0, $1973 = 0, $1974 = 0, $1975 = 0, $1976 = 0, $1977 = 0, $1978 = 0, $1979 = 0, $198 = 0, $1980 = 0, $1981 = 0, $1982 = 0, $1983 = 0, $1984 = 0, $1985 = 0, $1986 = 0, $1987 = 0, $1988 = 0;
 var $1989 = 0, $199 = 0, $1990 = 0, $1991 = 0, $1992 = 0, $1993 = 0, $1994 = 0, $1995 = 0, $1996 = 0, $1997 = 0, $1998 = 0, $1999 = 0, $2 = 0, $20 = 0, $200 = 0, $2000 = 0, $2001 = 0, $2002 = 0, $2003 = 0, $2004 = 0;
 var $2005 = 0, $2006 = 0, $2007 = 0, $2008 = 0, $2009 = 0, $201 = 0, $2010 = 0, $2011 = 0, $2012 = 0, $2013 = 0, $2014 = 0, $2015 = 0, $2016 = 0, $2017 = 0, $2018 = 0, $2019 = 0, $202 = 0, $2020 = 0, $2021 = 0, $2022 = 0;
 var $2023 = 0, $2024 = 0, $2025 = 0, $2026 = 0, $2027 = 0, $2028 = 0, $2029 = 0, $203 = 0, $2030 = 0, $2031 = 0, $2032 = 0, $2033 = 0, $2034 = 0, $2035 = 0, $2036 = 0, $2037 = 0, $2038 = 0, $2039 = 0, $204 = 0, $2040 = 0;
 var $2041 = 0, $2042 = 0, $2043 = 0, $2044 = 0, $2045 = 0, $2046 = 0, $2047 = 0, $2048 = 0, $2049 = 0, $205 = 0, $2050 = 0, $2051 = 0, $2052 = 0, $2053 = 0, $2054 = 0, $2055 = 0, $2056 = 0, $2057 = 0, $2058 = 0, $2059 = 0;
 var $206 = 0, $2060 = 0, $2061 = 0, $2062 = 0, $2063 = 0, $2064 = 0, $2065 = 0, $2066 = 0, $2067 = 0, $2068 = 0, $2069 = 0, $207 = 0, $2070 = 0, $2071 = 0, $2072 = 0, $2073 = 0, $2074 = 0, $2075 = 0, $2076 = 0, $2077 = 0;
 var $2078 = 0, $2079 = 0, $208 = 0, $2080 = 0, $2081 = 0, $2082 = 0, $2083 = 0, $2084 = 0, $2085 = 0, $2086 = 0, $2087 = 0, $2088 = 0, $2089 = 0, $209 = 0, $2090 = 0, $2091 = 0, $2092 = 0, $2093 = 0, $2094 = 0, $2095 = 0;
 var $2096 = 0, $2097 = 0, $2098 = 0, $2099 = 0, $21 = 0, $210 = 0, $2100 = 0, $2101 = 0, $2102 = 0, $2103 = 0, $2104 = 0, $2105 = 0, $2106 = 0, $2107 = 0, $2108 = 0, $2109 = 0, $211 = 0, $2110 = 0, $2111 = 0, $2112 = 0;
 var $2113 = 0, $2114 = 0, $2115 = 0, $2116 = 0, $2117 = 0, $2118 = 0, $2119 = 0, $212 = 0, $2120 = 0, $2121 = 0, $2122 = 0, $2123 = 0, $2124 = 0, $2125 = 0, $2126 = 0, $2127 = 0, $2128 = 0, $2129 = 0, $213 = 0, $2130 = 0;
 var $2131 = 0, $2132 = 0, $2133 = 0, $2134 = 0, $2135 = 0, $2136 = 0, $2137 = 0, $2138 = 0, $2139 = 0, $214 = 0, $2140 = 0, $2141 = 0, $2142 = 0, $2143 = 0, $2144 = 0, $2145 = 0, $2146 = 0, $2147 = 0, $2148 = 0, $2149 = 0;
 var $215 = 0, $2150 = 0, $2151 = 0, $2152 = 0, $2153 = 0, $2154 = 0, $2155 = 0, $2156 = 0, $2157 = 0, $2158 = 0, $2159 = 0, $216 = 0, $2160 = 0, $2161 = 0, $2162 = 0, $2163 = 0, $2164 = 0, $2165 = 0, $2166 = 0, $2167 = 0;
 var $2168 = 0, $2169 = 0, $217 = 0, $2170 = 0, $2171 = 0, $2172 = 0, $2173 = 0, $2174 = 0, $2175 = 0, $2176 = 0, $2177 = 0, $2178 = 0, $2179 = 0, $218 = 0, $2180 = 0, $2181 = 0, $2182 = 0, $2183 = 0, $2184 = 0, $2185 = 0;
 var $2186 = 0, $2187 = 0, $2188 = 0, $2189 = 0, $219 = 0, $2190 = 0, $2191 = 0, $2192 = 0, $2193 = 0, $2194 = 0, $2195 = 0, $2196 = 0, $2197 = 0, $2198 = 0, $2199 = 0, $22 = 0, $220 = 0, $2200 = 0, $2201 = 0, $2202 = 0;
 var $2203 = 0, $2204 = 0, $2205 = 0, $2206 = 0, $2207 = 0, $2208 = 0, $2209 = 0, $221 = 0, $2210 = 0, $2211 = 0, $2212 = 0, $2213 = 0, $2214 = 0, $2215 = 0, $2216 = 0, $2217 = 0, $2218 = 0, $2219 = 0, $222 = 0, $2220 = 0;
 var $2221 = 0, $2222 = 0, $2223 = 0, $2224 = 0, $2225 = 0, $2226 = 0, $2227 = 0, $2228 = 0, $2229 = 0, $223 = 0, $2230 = 0, $2231 = 0, $2232 = 0, $2233 = 0, $2234 = 0, $2235 = 0, $2236 = 0, $2237 = 0, $2238 = 0, $2239 = 0;
 var $224 = 0, $2240 = 0, $2241 = 0, $2242 = 0, $2243 = 0, $2244 = 0, $2245 = 0, $2246 = 0, $2247 = 0, $2248 = 0, $2249 = 0, $225 = 0, $2250 = 0, $2251 = 0, $2252 = 0, $2253 = 0, $2254 = 0, $2255 = 0, $2256 = 0, $2257 = 0;
 var $2258 = 0, $2259 = 0, $226 = 0, $2260 = 0, $2261 = 0, $2262 = 0, $2263 = 0, $2264 = 0, $2265 = 0, $2266 = 0, $2267 = 0, $2268 = 0, $2269 = 0, $227 = 0, $2270 = 0, $2271 = 0, $2272 = 0, $2273 = 0, $2274 = 0, $2275 = 0;
 var $2276 = 0, $2277 = 0, $2278 = 0, $2279 = 0, $228 = 0, $2280 = 0, $2281 = 0, $2282 = 0, $2283 = 0, $2284 = 0, $2285 = 0, $2286 = 0, $2287 = 0, $2288 = 0, $2289 = 0, $229 = 0, $2290 = 0, $2291 = 0, $2292 = 0, $2293 = 0;
 var $2294 = 0, $2295 = 0, $2296 = 0, $2297 = 0, $2298 = 0, $2299 = 0, $23 = 0, $230 = 0, $2300 = 0, $2301 = 0, $2302 = 0, $2303 = 0, $2304 = 0, $2305 = 0, $2306 = 0, $2307 = 0, $2308 = 0, $2309 = 0, $231 = 0, $2310 = 0;
 var $2311 = 0, $2312 = 0, $2313 = 0, $2314 = 0, $2315 = 0, $2316 = 0, $2317 = 0, $2318 = 0, $2319 = 0, $232 = 0, $2320 = 0, $2321 = 0, $2322 = 0, $2323 = 0, $2324 = 0, $2325 = 0, $2326 = 0, $2327 = 0, $2328 = 0, $2329 = 0;
 var $233 = 0, $2330 = 0, $2331 = 0, $2332 = 0, $2333 = 0, $2334 = 0, $2335 = 0, $2336 = 0, $2337 = 0, $2338 = 0, $2339 = 0, $234 = 0, $2340 = 0, $2341 = 0, $2342 = 0, $2343 = 0, $2344 = 0, $2345 = 0, $2346 = 0, $2347 = 0;
 var $2348 = 0, $2349 = 0, $235 = 0, $2350 = 0, $2351 = 0, $2352 = 0, $2353 = 0, $2354 = 0, $2355 = 0, $2356 = 0, $2357 = 0, $2358 = 0, $2359 = 0, $236 = 0, $2360 = 0, $2361 = 0, $2362 = 0, $2363 = 0, $2364 = 0, $2365 = 0;
 var $2366 = 0, $2367 = 0, $2368 = 0, $2369 = 0, $237 = 0, $2370 = 0, $2371 = 0, $2372 = 0, $2373 = 0, $2374 = 0, $2375 = 0, $2376 = 0, $2377 = 0, $2378 = 0, $2379 = 0, $238 = 0, $2380 = 0, $2381 = 0, $2382 = 0, $2383 = 0;
 var $2384 = 0, $2385 = 0, $2386 = 0, $2387 = 0, $2388 = 0, $2389 = 0, $239 = 0, $2390 = 0, $2391 = 0, $2392 = 0, $2393 = 0, $2394 = 0, $2395 = 0, $2396 = 0, $2397 = 0, $2398 = 0, $2399 = 0, $24 = 0, $240 = 0, $241 = 0;
 var $242 = 0, $243 = 0, $244 = 0, $245 = 0, $246 = 0, $247 = 0, $248 = 0, $249 = 0, $25 = 0, $250 = 0, $251 = 0, $252 = 0, $253 = 0, $254 = 0, $255 = 0, $256 = 0, $257 = 0, $258 = 0, $259 = 0, $26 = 0;
 var $260 = 0, $261 = 0, $262 = 0, $263 = 0, $264 = 0, $265 = 0, $266 = 0, $267 = 0, $268 = 0, $269 = 0, $27 = 0, $270 = 0, $271 = 0, $272 = 0, $273 = 0, $274 = 0, $275 = 0, $276 = 0, $277 = 0, $278 = 0;
 var $279 = 0, $28 = 0, $280 = 0, $281 = 0, $282 = 0, $283 = 0, $284 = 0, $285 = 0, $286 = 0, $287 = 0, $288 = 0, $289 = 0, $29 = 0, $290 = 0, $291 = 0, $292 = 0, $293 = 0, $294 = 0, $295 = 0, $296 = 0;
 var $297 = 0, $298 = 0, $299 = 0, $3 = 0, $30 = 0, $300 = 0, $301 = 0, $302 = 0, $303 = 0, $304 = 0, $305 = 0, $306 = 0, $307 = 0, $308 = 0, $309 = 0, $31 = 0, $310 = 0, $311 = 0, $312 = 0, $313 = 0;
 var $314 = 0, $315 = 0, $316 = 0, $317 = 0, $318 = 0, $319 = 0, $32 = 0, $320 = 0, $321 = 0, $322 = 0, $323 = 0, $324 = 0, $325 = 0, $326 = 0, $327 = 0, $328 = 0, $329 = 0, $33 = 0, $330 = 0, $331 = 0;
 var $332 = 0, $333 = 0, $334 = 0, $335 = 0, $336 = 0, $337 = 0, $338 = 0, $339 = 0, $34 = 0, $340 = 0, $341 = 0, $342 = 0, $343 = 0, $344 = 0, $345 = 0, $346 = 0, $347 = 0, $348 = 0, $349 = 0, $35 = 0;
 var $350 = 0, $351 = 0, $352 = 0, $353 = 0, $354 = 0, $355 = 0, $356 = 0, $357 = 0, $358 = 0, $359 = 0, $36 = 0, $360 = 0, $361 = 0, $362 = 0, $363 = 0, $364 = 0, $365 = 0, $366 = 0, $367 = 0, $368 = 0;
 var $369 = 0, $37 = 0, $370 = 0, $371 = 0, $372 = 0, $373 = 0, $374 = 0, $375 = 0, $376 = 0, $377 = 0, $378 = 0, $379 = 0, $38 = 0, $380 = 0, $381 = 0, $382 = 0, $383 = 0, $384 = 0, $385 = 0, $386 = 0;
 var $387 = 0, $388 = 0, $389 = 0, $39 = 0, $390 = 0, $391 = 0, $392 = 0, $393 = 0, $394 = 0, $395 = 0, $396 = 0, $397 = 0, $398 = 0, $399 = 0, $4 = 0, $40 = 0, $400 = 0, $401 = 0, $402 = 0, $403 = 0;
 var $404 = 0, $405 = 0, $406 = 0, $407 = 0, $408 = 0, $409 = 0, $41 = 0, $410 = 0, $411 = 0, $412 = 0, $413 = 0, $414 = 0, $415 = 0, $416 = 0, $417 = 0, $418 = 0, $419 = 0, $42 = 0, $420 = 0, $421 = 0;
 var $422 = 0, $423 = 0, $424 = 0, $425 = 0, $426 = 0, $427 = 0, $428 = 0, $429 = 0, $43 = 0, $430 = 0, $431 = 0, $432 = 0, $433 = 0, $434 = 0, $435 = 0, $436 = 0, $437 = 0, $438 = 0, $439 = 0, $44 = 0;
 var $440 = 0, $441 = 0, $442 = 0, $443 = 0, $444 = 0, $445 = 0, $446 = 0, $447 = 0, $448 = 0, $449 = 0, $45 = 0, $450 = 0, $451 = 0, $452 = 0, $453 = 0, $454 = 0, $455 = 0, $456 = 0, $457 = 0, $458 = 0;
 var $459 = 0, $46 = 0, $460 = 0, $461 = 0, $462 = 0, $463 = 0, $464 = 0, $465 = 0, $466 = 0, $467 = 0, $468 = 0, $469 = 0, $47 = 0, $470 = 0, $471 = 0, $472 = 0, $473 = 0, $474 = 0, $475 = 0, $476 = 0;
 var $477 = 0, $478 = 0, $479 = 0, $48 = 0, $480 = 0, $481 = 0, $482 = 0, $483 = 0, $484 = 0, $485 = 0, $486 = 0, $487 = 0, $488 = 0, $489 = 0, $49 = 0, $490 = 0, $491 = 0, $492 = 0, $493 = 0, $494 = 0;
 var $495 = 0, $496 = 0, $497 = 0, $498 = 0, $499 = 0, $5 = 0, $50 = 0, $500 = 0, $501 = 0, $502 = 0, $503 = 0, $504 = 0, $505 = 0, $506 = 0, $507 = 0, $508 = 0, $509 = 0, $51 = 0, $510 = 0, $511 = 0;
 var $512 = 0, $513 = 0, $514 = 0, $515 = 0, $516 = 0, $517 = 0, $518 = 0, $519 = 0, $52 = 0, $520 = 0, $521 = 0, $522 = 0, $523 = 0, $524 = 0, $525 = 0, $526 = 0, $527 = 0, $528 = 0, $529 = 0, $53 = 0;
 var $530 = 0, $531 = 0, $532 = 0, $533 = 0, $534 = 0, $535 = 0, $536 = 0, $537 = 0, $538 = 0, $539 = 0, $54 = 0, $540 = 0, $541 = 0, $542 = 0, $543 = 0, $544 = 0, $545 = 0, $546 = 0, $547 = 0, $548 = 0;
 var $549 = 0, $55 = 0, $550 = 0, $551 = 0, $552 = 0, $553 = 0, $554 = 0, $555 = 0, $556 = 0, $557 = 0, $558 = 0, $559 = 0, $56 = 0, $560 = 0, $561 = 0, $562 = 0, $563 = 0, $564 = 0, $565 = 0, $566 = 0;
 var $567 = 0, $568 = 0, $569 = 0, $57 = 0, $570 = 0, $571 = 0, $572 = 0, $573 = 0, $574 = 0, $575 = 0, $576 = 0, $577 = 0, $578 = 0, $579 = 0, $58 = 0, $580 = 0, $581 = 0, $582 = 0, $583 = 0, $584 = 0;
 var $585 = 0, $586 = 0, $587 = 0, $588 = 0, $589 = 0, $59 = 0, $590 = 0, $591 = 0, $592 = 0, $593 = 0, $594 = 0, $595 = 0, $596 = 0, $597 = 0, $598 = 0, $599 = 0, $6 = 0, $60 = 0, $600 = 0, $601 = 0;
 var $602 = 0, $603 = 0, $604 = 0, $605 = 0, $606 = 0, $607 = 0, $608 = 0, $609 = 0, $61 = 0, $610 = 0, $611 = 0, $612 = 0, $613 = 0, $614 = 0, $615 = 0, $616 = 0, $617 = 0, $618 = 0, $619 = 0, $62 = 0;
 var $620 = 0, $621 = 0, $622 = 0, $623 = 0, $624 = 0, $625 = 0, $626 = 0, $627 = 0, $628 = 0, $629 = 0, $63 = 0, $630 = 0, $631 = 0, $632 = 0, $633 = 0, $634 = 0, $635 = 0, $636 = 0, $637 = 0, $638 = 0;
 var $639 = 0, $64 = 0, $640 = 0, $641 = 0, $642 = 0, $643 = 0, $644 = 0, $645 = 0, $646 = 0, $647 = 0, $648 = 0, $649 = 0, $65 = 0, $650 = 0, $651 = 0, $652 = 0, $653 = 0, $654 = 0, $655 = 0, $656 = 0;
 var $657 = 0, $658 = 0, $659 = 0, $66 = 0, $660 = 0, $661 = 0, $662 = 0, $663 = 0, $664 = 0, $665 = 0, $666 = 0, $667 = 0, $668 = 0, $669 = 0, $67 = 0, $670 = 0, $671 = 0, $672 = 0, $673 = 0, $674 = 0;
 var $675 = 0, $676 = 0, $677 = 0, $678 = 0, $679 = 0, $68 = 0, $680 = 0, $681 = 0, $682 = 0, $683 = 0, $684 = 0, $685 = 0, $686 = 0, $687 = 0, $688 = 0, $689 = 0, $69 = 0, $690 = 0, $691 = 0, $692 = 0;
 var $693 = 0, $694 = 0, $695 = 0, $696 = 0, $697 = 0, $698 = 0, $699 = 0, $7 = 0, $70 = 0, $700 = 0, $701 = 0, $702 = 0, $703 = 0, $704 = 0, $705 = 0, $706 = 0, $707 = 0, $708 = 0, $709 = 0, $71 = 0;
 var $710 = 0, $711 = 0, $712 = 0, $713 = 0, $714 = 0, $715 = 0, $716 = 0, $717 = 0, $718 = 0, $719 = 0, $72 = 0, $720 = 0, $721 = 0, $722 = 0, $723 = 0, $724 = 0, $725 = 0, $726 = 0, $727 = 0, $728 = 0;
 var $729 = 0, $73 = 0, $730 = 0, $731 = 0, $732 = 0, $733 = 0, $734 = 0, $735 = 0, $736 = 0, $737 = 0, $738 = 0, $739 = 0, $74 = 0, $740 = 0, $741 = 0, $742 = 0, $743 = 0, $744 = 0, $745 = 0, $746 = 0;
 var $747 = 0, $748 = 0, $749 = 0, $75 = 0, $750 = 0, $751 = 0, $752 = 0, $753 = 0, $754 = 0, $755 = 0, $756 = 0, $757 = 0, $758 = 0, $759 = 0, $76 = 0, $760 = 0, $761 = 0, $762 = 0, $763 = 0, $764 = 0;
 var $765 = 0, $766 = 0, $767 = 0, $768 = 0, $769 = 0, $77 = 0, $770 = 0, $771 = 0, $772 = 0, $773 = 0, $774 = 0, $775 = 0, $776 = 0, $777 = 0, $778 = 0, $779 = 0, $78 = 0, $780 = 0, $781 = 0, $782 = 0;
 var $783 = 0, $784 = 0, $785 = 0, $786 = 0, $787 = 0, $788 = 0, $789 = 0, $79 = 0, $790 = 0, $791 = 0, $792 = 0, $793 = 0, $794 = 0, $795 = 0, $796 = 0, $797 = 0, $798 = 0, $799 = 0, $8 = 0, $80 = 0;
 var $800 = 0, $801 = 0, $802 = 0, $803 = 0, $804 = 0, $805 = 0, $806 = 0, $807 = 0, $808 = 0, $809 = 0, $81 = 0, $810 = 0, $811 = 0, $812 = 0, $813 = 0, $814 = 0, $815 = 0, $816 = 0, $817 = 0, $818 = 0;
 var $819 = 0, $82 = 0, $820 = 0, $821 = 0, $822 = 0, $823 = 0, $824 = 0, $825 = 0, $826 = 0, $827 = 0, $828 = 0, $829 = 0, $83 = 0, $830 = 0, $831 = 0, $832 = 0, $833 = 0, $834 = 0, $835 = 0, $836 = 0;
 var $837 = 0, $838 = 0, $839 = 0, $84 = 0, $840 = 0, $841 = 0, $842 = 0, $843 = 0, $844 = 0, $845 = 0, $846 = 0, $847 = 0, $848 = 0, $849 = 0, $85 = 0, $850 = 0, $851 = 0, $852 = 0, $853 = 0, $854 = 0;
 var $855 = 0, $856 = 0, $857 = 0, $858 = 0, $859 = 0, $86 = 0, $860 = 0, $861 = 0, $862 = 0, $863 = 0, $864 = 0, $865 = 0, $866 = 0, $867 = 0, $868 = 0, $869 = 0, $87 = 0, $870 = 0, $871 = 0, $872 = 0;
 var $873 = 0, $874 = 0, $875 = 0, $876 = 0, $877 = 0, $878 = 0, $879 = 0, $88 = 0, $880 = 0, $881 = 0, $882 = 0, $883 = 0, $884 = 0, $885 = 0, $886 = 0, $887 = 0, $888 = 0, $889 = 0, $89 = 0, $890 = 0;
 var $891 = 0, $892 = 0, $893 = 0, $894 = 0, $895 = 0, $896 = 0, $897 = 0, $898 = 0, $899 = 0, $9 = 0, $90 = 0, $900 = 0, $901 = 0, $902 = 0, $903 = 0, $904 = 0, $905 = 0, $906 = 0, $907 = 0, $908 = 0;
 var $909 = 0, $91 = 0, $910 = 0, $911 = 0, $912 = 0, $913 = 0, $914 = 0, $915 = 0, $916 = 0, $917 = 0, $918 = 0, $919 = 0, $92 = 0, $920 = 0, $921 = 0, $922 = 0, $923 = 0, $924 = 0, $925 = 0, $926 = 0;
 var $927 = 0, $928 = 0, $929 = 0, $93 = 0, $930 = 0, $931 = 0, $932 = 0, $933 = 0, $934 = 0, $935 = 0, $936 = 0, $937 = 0, $938 = 0, $939 = 0, $94 = 0, $940 = 0, $941 = 0, $942 = 0, $943 = 0, $944 = 0;
 var $945 = 0, $946 = 0, $947 = 0, $948 = 0, $949 = 0, $95 = 0, $950 = 0, $951 = 0, $952 = 0, $953 = 0, $954 = 0, $955 = 0, $956 = 0, $957 = 0, $958 = 0, $959 = 0, $96 = 0, $960 = 0, $961 = 0, $962 = 0;
 var $963 = 0, $964 = 0, $965 = 0, $966 = 0, $967 = 0, $968 = 0, $969 = 0, $97 = 0, $970 = 0, $971 = 0, $972 = 0, $973 = 0, $974 = 0, $975 = 0, $976 = 0, $977 = 0, $978 = 0, $979 = 0, $98 = 0, $980 = 0;
 var $981 = 0, $982 = 0, $983 = 0, $984 = 0, $985 = 0, $986 = 0, $987 = 0, $988 = 0, $989 = 0, $99 = 0, $990 = 0, $991 = 0, $992 = 0, $993 = 0, $994 = 0, $995 = 0, $996 = 0, $997 = 0, $998 = 0, $999 = 0;
 var $all = 0, $cnt = 0, $dest = 0, $mxSelect = 0, $p = 0, $p10 = 0, $p2 = 0, $p8 = 0, $p9 = 0, $pDot = 0, $pFrom = 0, $pLeft = 0, $pList = 0, $pList6 = 0, $pList7 = 0, $pLoop = 0, $pNew = 0, $pNext = 0, $pOld = 0, $pParse = 0;
 var $pRHS = 0, $pRhs = 0, $pRight = 0, $pRight3 = 0, $pSrc = 0, $pSubquery = 0, $temp1 = 0, $temp14 = 0, $temp2 = 0, $temp25 = 0, $temp3 = 0, $temp4 = 0, $v = 0, $v1 = 0, $vararg_buffer14 = 0, $vararg_ptr1 = 0, $x = 0, $yyact = 0, $yygoto = 0, $yygotominor = 0;
 var $yymsp = 0, $yysize = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 224|0;
 $vararg_buffer14 = sp;
 $0 = sp + 220|0;
 $1 = sp + 216|0;
 $yygoto = sp + 212|0;
 $yyact = sp + 208|0;
 $yygotominor = sp + 196|0;
 $yymsp = sp + 192|0;
 $yysize = sp + 188|0;
 $pParse = sp + 184|0;
 $v = sp + 172|0;
 $v1 = sp + 160|0;
 $dest = sp + 140|0;
 $p = sp + 136|0;
 $pNext = sp + 132|0;
 $pLoop = sp + 128|0;
 $cnt = sp + 124|0;
 $mxSelect = sp + 120|0;
 $pRhs = sp + 116|0;
 $pFrom = sp + 112|0;
 $x = sp + 104|0;
 $pRight = sp + 100|0;
 $p2 = sp + 96|0;
 $pRight3 = sp + 92|0;
 $pLeft = sp + 88|0;
 $pDot = sp + 84|0;
 $pNew = sp + 80|0;
 $pOld = sp + 76|0;
 $pSubquery = sp + 72|0;
 $temp1 = sp + 68|0;
 $temp2 = sp + 64|0;
 $temp14 = sp + 60|0;
 $temp25 = sp + 56|0;
 $temp3 = sp + 52|0;
 $temp4 = sp + 48|0;
 $pList = sp + 44|0;
 $pList6 = sp + 40|0;
 $pList7 = sp + 36|0;
 $pRHS = sp + 32|0;
 $pSrc = sp + 28|0;
 $p8 = sp + 24|0;
 $p9 = sp + 20|0;
 $p10 = sp + 16|0;
 $all = sp + 8|0;
 HEAP32[$0>>2] = $yypParser;
 HEAP32[$1>>2] = $yyruleno;
 $2 = HEAP32[$0>>2]|0;
 $3 = (($2) + 8|0);
 $4 = HEAP32[$3>>2]|0;
 HEAP32[$pParse>>2] = $4;
 $5 = HEAP32[$0>>2]|0;
 $6 = HEAP32[$5>>2]|0;
 $7 = HEAP32[$0>>2]|0;
 $8 = (($7) + 12|0);
 $9 = (($8) + ($6<<4)|0);
 HEAP32[$yymsp>>2] = $9;
 ;HEAP32[$yygotominor+0>>2]=HEAP32[27168+0>>2]|0;HEAP32[$yygotominor+4>>2]=HEAP32[27168+4>>2]|0;HEAP32[$yygotominor+8>>2]=HEAP32[27168+8>>2]|0;
 $10 = HEAP32[$1>>2]|0;
 L1: do {
  switch ($10|0) {
  case 56:  {
   $192 = HEAP32[$pParse>>2]|0;
   $193 = HEAP32[$yymsp>>2]|0;
   $194 = (($193) + -16|0);
   $195 = (($194) + 4|0);
   _sqlite3AddDefaultValue($192,$195);
   break;
  }
  case 62:  {
   $220 = HEAP32[$pParse>>2]|0;
   $221 = HEAP32[$yymsp>>2]|0;
   $222 = (($221) + -16|0);
   $223 = (($222) + 4|0);
   $224 = HEAP32[$223>>2]|0;
   $225 = HEAP32[$yymsp>>2]|0;
   $226 = (($225) + 4|0);
   $227 = HEAP32[$226>>2]|0;
   $228 = HEAP32[$yymsp>>2]|0;
   $229 = (($228) + -32|0);
   $230 = (($229) + 4|0);
   $231 = HEAP32[$230>>2]|0;
   _sqlite3AddPrimaryKey($220,0,$224,$227,$231);
   break;
  }
  case 111:  {
   ;HEAP32[$dest+0>>2]=HEAP32[10032+0>>2]|0;HEAP32[$dest+4>>2]=HEAP32[10032+4>>2]|0;HEAP32[$dest+8>>2]=HEAP32[10032+8>>2]|0;HEAP32[$dest+12>>2]=HEAP32[10032+12>>2]|0;HEAP32[$dest+16>>2]=HEAP32[10032+16>>2]|0;
   $379 = HEAP32[$pParse>>2]|0;
   $380 = HEAP32[$yymsp>>2]|0;
   $381 = (($380) + 4|0);
   $382 = HEAP32[$381>>2]|0;
   (_sqlite3Select($379,$382,$dest)|0);
   $383 = HEAP32[$pParse>>2]|0;
   $384 = HEAP32[$383>>2]|0;
   $385 = HEAP32[$yymsp>>2]|0;
   $386 = (($385) + 4|0);
   $387 = HEAP32[$386>>2]|0;
   _sqlite3SelectDelete($384,$387);
   break;
  }
  case 112:  {
   $388 = HEAP32[$yymsp>>2]|0;
   $389 = (($388) + 4|0);
   $390 = HEAP32[$389>>2]|0;
   HEAP32[$p>>2] = $390;
   $391 = HEAP32[$p>>2]|0;
   $392 = ($391|0)!=(0|0);
   if ($392) {
    HEAP32[$cnt>>2] = 0;
    $393 = HEAP32[$yymsp>>2]|0;
    $394 = (($393) + -16|0);
    $395 = (($394) + 4|0);
    $396 = HEAP32[$395>>2]|0;
    $397 = HEAP32[$p>>2]|0;
    $398 = (($397) + 68|0);
    HEAP32[$398>>2] = $396;
    $399 = HEAP32[$p>>2]|0;
    $400 = (($399) + 52|0);
    $401 = HEAP32[$400>>2]|0;
    $402 = ($401|0)!=(0|0);
    if ($402) {
     HEAP32[$pNext>>2] = 0;
     $403 = HEAP32[$p>>2]|0;
     HEAP32[$pLoop>>2] = $403;
     while(1) {
      $404 = HEAP32[$pLoop>>2]|0;
      $405 = ($404|0)!=(0|0);
      if (!($405)) {
       break;
      }
      $406 = HEAP32[$pNext>>2]|0;
      $407 = HEAP32[$pLoop>>2]|0;
      $408 = (($407) + 56|0);
      HEAP32[$408>>2] = $406;
      $409 = HEAP32[$pLoop>>2]|0;
      $410 = (($409) + 6|0);
      $411 = HEAP16[$410>>1]|0;
      $412 = $411&65535;
      $413 = $412 | 64;
      $414 = $413&65535;
      HEAP16[$410>>1] = $414;
      $415 = HEAP32[$pLoop>>2]|0;
      HEAP32[$pNext>>2] = $415;
      $416 = HEAP32[$pLoop>>2]|0;
      $417 = (($416) + 52|0);
      $418 = HEAP32[$417>>2]|0;
      HEAP32[$pLoop>>2] = $418;
      $419 = HEAP32[$cnt>>2]|0;
      $420 = (($419) + 1)|0;
      HEAP32[$cnt>>2] = $420;
     }
     $421 = HEAP32[$pParse>>2]|0;
     $422 = HEAP32[$421>>2]|0;
     $423 = (($422) + 88|0);
     $424 = (($423) + 16|0);
     $425 = HEAP32[$424>>2]|0;
     HEAP32[$mxSelect>>2] = $425;
     $426 = HEAP32[$mxSelect>>2]|0;
     $427 = ($426|0)!=(0);
     if ($427) {
      $428 = HEAP32[$cnt>>2]|0;
      $429 = HEAP32[$mxSelect>>2]|0;
      $430 = ($428|0)>($429|0);
      if ($430) {
       $431 = HEAP32[$pParse>>2]|0;
       _sqlite3ErrorMsg($431,10056,$vararg_buffer14);
      }
     }
    }
   } else {
    $432 = HEAP32[$pParse>>2]|0;
    $433 = HEAP32[$432>>2]|0;
    $434 = HEAP32[$yymsp>>2]|0;
    $435 = (($434) + -16|0);
    $436 = (($435) + 4|0);
    $437 = HEAP32[$436>>2]|0;
    _sqlite3WithDelete($433,$437);
   }
   $438 = HEAP32[$p>>2]|0;
   HEAP32[$yygotominor>>2] = $438;
   break;
  }
  case 24:  {
   $32 = HEAP32[$pParse>>2]|0;
   $33 = HEAP32[$yymsp>>2]|0;
   $34 = (($33) + 4|0);
   _sqlite3Savepoint($32,2,$34);
   break;
  }
  case 7:  {
   $13 = HEAP32[$pParse>>2]|0;
   _sqlite3BeginParse($13,2);
   break;
  }
  case 5:  {
   $11 = HEAP32[$pParse>>2]|0;
   _sqlite3BeginParse($11,0);
   break;
  }
  case 19:  {
   $25 = HEAP32[$pParse>>2]|0;
   _sqlite3RollbackTransaction($25);
   break;
  }
  case 279: case 263: case 262: case 261: case 260: case 259: case 258: case 257: case 256: case 247: case 150: case 141: case 131: case 130: case 48: case 45: case 42: case 41: case 40:  {
   $120 = HEAP32[$yymsp>>2]|0;
   $121 = (($120) + 4|0);
   ;HEAP32[$yygotominor+0>>2]=HEAP32[$121+0>>2]|0;HEAP32[$yygotominor+4>>2]=HEAP32[$121+4>>2]|0;
   break;
  }
  case 44:  {
   $122 = HEAP32[$pParse>>2]|0;
   $123 = HEAP32[$yymsp>>2]|0;
   $124 = (($123) + 4|0);
   _sqlite3AddColumnType($122,$124);
   break;
  }
  case 119: case 113:  {
   $439 = HEAP32[$yymsp>>2]|0;
   $440 = (($439) + 4|0);
   $441 = HEAP32[$440>>2]|0;
   HEAP32[$yygotominor>>2] = $441;
   break;
  }
  case 114:  {
   $442 = HEAP32[$yymsp>>2]|0;
   $443 = (($442) + 4|0);
   $444 = HEAP32[$443>>2]|0;
   HEAP32[$pRhs>>2] = $444;
   $445 = HEAP32[$pRhs>>2]|0;
   $446 = ($445|0)!=(0|0);
   if ($446) {
    $447 = HEAP32[$pRhs>>2]|0;
    $448 = (($447) + 52|0);
    $449 = HEAP32[$448>>2]|0;
    $450 = ($449|0)!=(0|0);
    if ($450) {
     $451 = (($x) + 4|0);
     HEAP32[$451>>2] = 0;
     $452 = HEAP32[$pParse>>2]|0;
     $453 = HEAP32[$pRhs>>2]|0;
     $454 = (_sqlite3SrcListAppendFromTerm($452,0,0,0,$x,$453,0,0)|0);
     HEAP32[$pFrom>>2] = $454;
     $455 = HEAP32[$pParse>>2]|0;
     $456 = HEAP32[$pFrom>>2]|0;
     $457 = (_sqlite3SelectNew($455,0,$456,0,0,0,0,0,0,0)|0);
     HEAP32[$pRhs>>2] = $457;
    }
   }
   $458 = HEAP32[$pRhs>>2]|0;
   $459 = ($458|0)!=(0|0);
   if ($459) {
    $460 = HEAP32[$yymsp>>2]|0;
    $461 = (($460) + -16|0);
    $462 = (($461) + 4|0);
    $463 = HEAP32[$462>>2]|0;
    $464 = $463&255;
    $465 = HEAP32[$pRhs>>2]|0;
    $466 = (($465) + 4|0);
    HEAP8[$466>>0] = $464;
    $467 = HEAP32[$yymsp>>2]|0;
    $468 = (($467) + -32|0);
    $469 = (($468) + 4|0);
    $470 = HEAP32[$469>>2]|0;
    $471 = HEAP32[$pRhs>>2]|0;
    $472 = (($471) + 52|0);
    HEAP32[$472>>2] = $470;
    $473 = HEAP32[$yymsp>>2]|0;
    $474 = (($473) + -16|0);
    $475 = (($474) + 4|0);
    $476 = HEAP32[$475>>2]|0;
    $477 = ($476|0)!=(116);
    if ($477) {
     $478 = HEAP32[$pParse>>2]|0;
     $479 = (($478) + 22|0);
     HEAP8[$479>>0] = 1;
    }
   } else {
    $480 = HEAP32[$pParse>>2]|0;
    $481 = HEAP32[$480>>2]|0;
    $482 = HEAP32[$yymsp>>2]|0;
    $483 = (($482) + -32|0);
    $484 = (($483) + 4|0);
    $485 = HEAP32[$484>>2]|0;
    _sqlite3SelectDelete($481,$485);
   }
   $486 = HEAP32[$pRhs>>2]|0;
   HEAP32[$yygotominor>>2] = $486;
   break;
  }
  case 22:  {
   $26 = HEAP32[$pParse>>2]|0;
   $27 = HEAP32[$yymsp>>2]|0;
   $28 = (($27) + 4|0);
   _sqlite3Savepoint($26,0,$28);
   break;
  }
  case 58:  {
   $196 = HEAP32[$pParse>>2]|0;
   $197 = HEAP32[$yymsp>>2]|0;
   $198 = (($197) + 4|0);
   $199 = HEAP32[$198>>2]|0;
   $200 = (_sqlite3PExpr($196,157,$199,0,0)|0);
   HEAP32[$v>>2] = $200;
   $201 = HEAP32[$yymsp>>2]|0;
   $202 = (($201) + -16|0);
   $203 = (($202) + 4|0);
   $204 = HEAP32[$203>>2]|0;
   $205 = (($v) + 4|0);
   HEAP32[$205>>2] = $204;
   $206 = HEAP32[$yymsp>>2]|0;
   $207 = (($206) + 4|0);
   $208 = (($207) + 8|0);
   $209 = HEAP32[$208>>2]|0;
   $210 = (($v) + 8|0);
   HEAP32[$210>>2] = $209;
   $211 = HEAP32[$pParse>>2]|0;
   _sqlite3AddDefaultValue($211,$v);
   break;
  }
  case 59:  {
   $212 = HEAP32[$pParse>>2]|0;
   $213 = HEAP32[$yymsp>>2]|0;
   $214 = (($213) + 4|0);
   _spanExpr($v1,$212,97,$214);
   $215 = HEAP32[$pParse>>2]|0;
   _sqlite3AddDefaultValue($215,$v1);
   break;
  }
  case 221: case 218: case 108: case 97: case 85: case 83: case 81: case 68: case 31: case 28:  {
   HEAP32[$yygotominor>>2] = 0;
   break;
  }
  case 23:  {
   $29 = HEAP32[$pParse>>2]|0;
   $30 = HEAP32[$yymsp>>2]|0;
   $31 = (($30) + 4|0);
   _sqlite3Savepoint($29,1,$31);
   break;
  }
  case 66:  {
   $252 = HEAP32[$pParse>>2]|0;
   $253 = HEAP32[$yymsp>>2]|0;
   $254 = (($253) + 4|0);
   $255 = HEAP32[$254>>2]|0;
   _sqlite3DeferForeignKey($252,$255);
   break;
  }
  case 67:  {
   $256 = HEAP32[$pParse>>2]|0;
   $257 = HEAP32[$yymsp>>2]|0;
   $258 = (($257) + 4|0);
   _sqlite3AddCollateType($256,$258);
   break;
  }
  case 70:  {
   HEAP32[$yygotominor>>2] = 0;
   break;
  }
  case 71:  {
   $259 = HEAP32[$yymsp>>2]|0;
   $260 = (($259) + -16|0);
   $261 = (($260) + 4|0);
   $262 = HEAP32[$261>>2]|0;
   $263 = HEAP32[$yymsp>>2]|0;
   $264 = (($263) + 4|0);
   $265 = (($264) + 4|0);
   $266 = HEAP32[$265>>2]|0;
   $267 = $266 ^ -1;
   $268 = $262 & $267;
   $269 = HEAP32[$yymsp>>2]|0;
   $270 = (($269) + 4|0);
   $271 = HEAP32[$270>>2]|0;
   $272 = $268 | $271;
   HEAP32[$yygotominor>>2] = $272;
   break;
  }
  case 73: case 72:  {
   HEAP32[$yygotominor>>2] = 0;
   $273 = (($yygotominor) + 4|0);
   HEAP32[$273>>2] = 0;
   break;
  }
  case 8:  {
   $14 = HEAP32[$pParse>>2]|0;
   _sqlite3FinishCoding($14);
   break;
  }
  case 33:  {
   $65 = HEAP32[$pParse>>2]|0;
   $66 = HEAP32[$yymsp>>2]|0;
   $67 = (($66) + 4|0);
   $68 = HEAP32[$67>>2]|0;
   _sqlite3EndTable($65,0,0,0,$68);
   $69 = HEAP32[$pParse>>2]|0;
   $70 = HEAP32[$69>>2]|0;
   $71 = HEAP32[$yymsp>>2]|0;
   $72 = (($71) + 4|0);
   $73 = HEAP32[$72>>2]|0;
   _sqlite3SelectDelete($70,$73);
   break;
  }
  case 13:  {
   HEAP32[$yygotominor>>2] = 7;
   break;
  }
  case 103: case 100: case 98: case 82:  {
   $283 = HEAP32[$yymsp>>2]|0;
   $284 = (($283) + 4|0);
   $285 = HEAP32[$284>>2]|0;
   HEAP32[$yygotominor>>2] = $285;
   break;
  }
  case 86:  {
   $286 = (($yygotominor) + 4|0);
   HEAP32[$286>>2] = 0;
   HEAP32[$yygotominor>>2] = 0;
   break;
  }
  case 47:  {
   $145 = HEAP32[$yymsp>>2]|0;
   $146 = (($145) + -80|0);
   $147 = (($146) + 4|0);
   $148 = HEAP32[$147>>2]|0;
   HEAP32[$yygotominor>>2] = $148;
   $149 = HEAP32[$yymsp>>2]|0;
   $150 = (($149) + 4|0);
   $151 = (($150) + 4|0);
   $152 = HEAP32[$151>>2]|0;
   $153 = HEAP32[$yymsp>>2]|0;
   $154 = (($153) + 4|0);
   $155 = HEAP32[$154>>2]|0;
   $156 = (($155) + ($152)|0);
   $157 = HEAP32[$yymsp>>2]|0;
   $158 = (($157) + -80|0);
   $159 = (($158) + 4|0);
   $160 = HEAP32[$159>>2]|0;
   $161 = $156;
   $162 = $160;
   $163 = (($161) - ($162))|0;
   $164 = (($yygotominor) + 4|0);
   HEAP32[$164>>2] = $163;
   break;
  }
  case 116:  {
   HEAP32[$yygotominor>>2] = 116;
   break;
  }
  case 109:  {
   $350 = HEAP32[$pParse>>2]|0;
   $351 = HEAP32[$yymsp>>2]|0;
   $352 = (($351) + -112|0);
   $353 = (($352) + 4|0);
   $354 = HEAP32[$yymsp>>2]|0;
   $355 = (($354) + -48|0);
   $356 = (($355) + 4|0);
   $357 = HEAP32[$yymsp>>2]|0;
   $358 = (($357) + -32|0);
   $359 = (($358) + 4|0);
   $360 = HEAP32[$yymsp>>2]|0;
   $361 = (($360) + 4|0);
   $362 = HEAP32[$361>>2]|0;
   $363 = HEAP32[$yymsp>>2]|0;
   $364 = (($363) + -96|0);
   $365 = (($364) + 4|0);
   $366 = HEAP32[$365>>2]|0;
   $367 = HEAP32[$yymsp>>2]|0;
   $368 = (($367) + -64|0);
   $369 = (($368) + 4|0);
   $370 = HEAP32[$369>>2]|0;
   _sqlite3CreateView($350,$353,$356,$359,$362,$366,$370);
   break;
  }
  case 63:  {
   $232 = HEAP32[$pParse>>2]|0;
   $233 = HEAP32[$yymsp>>2]|0;
   $234 = (($233) + 4|0);
   $235 = HEAP32[$234>>2]|0;
   (_sqlite3CreateIndex($232,0,0,0,0,$235,0,0,0,0)|0);
   break;
  }
  case 121:  {
   $530 = HEAP32[$pParse>>2]|0;
   $531 = HEAP32[$yymsp>>2]|0;
   $532 = (($531) + -16|0);
   $533 = (($532) + 4|0);
   $534 = HEAP32[$533>>2]|0;
   $535 = (_sqlite3SelectNew($530,$534,0,0,0,0,0,128,0,0)|0);
   HEAP32[$pRight>>2] = $535;
   $536 = HEAP32[$pRight>>2]|0;
   $537 = ($536|0)!=(0|0);
   if ($537) {
    $538 = HEAP32[$pRight>>2]|0;
    $539 = (($538) + 4|0);
    HEAP8[$539>>0] = 116;
    $540 = HEAP32[$yymsp>>2]|0;
    $541 = (($540) + -64|0);
    $542 = (($541) + 4|0);
    $543 = HEAP32[$542>>2]|0;
    $544 = HEAP32[$pRight>>2]|0;
    $545 = (($544) + 52|0);
    HEAP32[$545>>2] = $543;
    $546 = HEAP32[$pRight>>2]|0;
    HEAP32[$yygotominor>>2] = $546;
    break L1;
   } else {
    $547 = HEAP32[$yymsp>>2]|0;
    $548 = (($547) + -64|0);
    $549 = (($548) + 4|0);
    $550 = HEAP32[$549>>2]|0;
    HEAP32[$yygotominor>>2] = $550;
    break L1;
   }
   break;
  }
  case 122:  {
   HEAP16[$yygotominor>>1] = 1;
   break;
  }
  case 124: case 123:  {
   HEAP16[$yygotominor>>1] = 0;
   break;
  }
  case 243: case 125:  {
   $551 = HEAP32[$yymsp>>2]|0;
   $552 = (($551) + -16|0);
   $553 = (($552) + 4|0);
   $554 = HEAP32[$553>>2]|0;
   HEAP32[$yygotominor>>2] = $554;
   break;
  }
  case 93:  {
   $293 = HEAP32[$pParse>>2]|0;
   $294 = HEAP32[$yymsp>>2]|0;
   $295 = (($294) + -48|0);
   $296 = (($295) + 4|0);
   $297 = HEAP32[$296>>2]|0;
   $298 = HEAP32[$yymsp>>2]|0;
   $299 = (($298) + 4|0);
   $300 = HEAP32[$299>>2]|0;
   $301 = HEAP32[$yymsp>>2]|0;
   $302 = (($301) + -32|0);
   $303 = (($302) + 4|0);
   $304 = HEAP32[$303>>2]|0;
   _sqlite3AddPrimaryKey($293,$297,$300,$304,0);
   break;
  }
  case 26:  {
   $35 = HEAP32[$pParse>>2]|0;
   $36 = HEAP32[$yymsp>>2]|0;
   $37 = (($36) + -16|0);
   $38 = (($37) + 4|0);
   $39 = HEAP32[$yymsp>>2]|0;
   $40 = (($39) + 4|0);
   $41 = HEAP32[$yymsp>>2]|0;
   $42 = (($41) + -64|0);
   $43 = (($42) + 4|0);
   $44 = HEAP32[$43>>2]|0;
   $45 = HEAP32[$yymsp>>2]|0;
   $46 = (($45) + -32|0);
   $47 = (($46) + 4|0);
   $48 = HEAP32[$47>>2]|0;
   _sqlite3StartTable($35,$38,$40,$44,0,0,$48);
   break;
  }
  case 27:  {
   $49 = HEAP32[$pParse>>2]|0;
   $50 = HEAP32[$49>>2]|0;
   $51 = (($50) + 248|0);
   $52 = (($51) + 2|0);
   HEAP8[$52>>0] = 0;
   $53 = HEAP32[$yymsp>>2]|0;
   $54 = (($53) + 4|0);
   ;HEAP32[$yygotominor+0>>2]=HEAP32[$54+0>>2]|0;HEAP32[$yygotominor+4>>2]=HEAP32[$54+4>>2]|0;
   break;
  }
  case 118:  {
   $487 = HEAP32[$pParse>>2]|0;
   $488 = HEAP32[$yymsp>>2]|0;
   $489 = (($488) + -96|0);
   $490 = (($489) + 4|0);
   $491 = HEAP32[$490>>2]|0;
   $492 = HEAP32[$yymsp>>2]|0;
   $493 = (($492) + -80|0);
   $494 = (($493) + 4|0);
   $495 = HEAP32[$494>>2]|0;
   $496 = HEAP32[$yymsp>>2]|0;
   $497 = (($496) + -64|0);
   $498 = (($497) + 4|0);
   $499 = HEAP32[$498>>2]|0;
   $500 = HEAP32[$yymsp>>2]|0;
   $501 = (($500) + -48|0);
   $502 = (($501) + 4|0);
   $503 = HEAP32[$502>>2]|0;
   $504 = HEAP32[$yymsp>>2]|0;
   $505 = (($504) + -32|0);
   $506 = (($505) + 4|0);
   $507 = HEAP32[$506>>2]|0;
   $508 = HEAP32[$yymsp>>2]|0;
   $509 = (($508) + -16|0);
   $510 = (($509) + 4|0);
   $511 = HEAP32[$510>>2]|0;
   $512 = HEAP32[$yymsp>>2]|0;
   $513 = (($512) + -112|0);
   $514 = (($513) + 4|0);
   $515 = HEAP16[$514>>1]|0;
   $516 = HEAP32[$yymsp>>2]|0;
   $517 = (($516) + 4|0);
   $518 = HEAP32[$517>>2]|0;
   $519 = HEAP32[$yymsp>>2]|0;
   $520 = (($519) + 4|0);
   $521 = (($520) + 4|0);
   $522 = HEAP32[$521>>2]|0;
   $523 = (_sqlite3SelectNew($487,$491,$495,$499,$503,$507,$511,$515,$518,$522)|0);
   HEAP32[$yygotominor>>2] = $523;
   break;
  }
  case 120:  {
   $524 = HEAP32[$pParse>>2]|0;
   $525 = HEAP32[$yymsp>>2]|0;
   $526 = (($525) + -16|0);
   $527 = (($526) + 4|0);
   $528 = HEAP32[$527>>2]|0;
   $529 = (_sqlite3SelectNew($524,$528,0,0,0,0,0,128,0,0)|0);
   HEAP32[$yygotominor>>2] = $529;
   break;
  }
  case 134:  {
   $613 = HEAP32[$yymsp>>2]|0;
   $614 = (($613) + 4|0);
   $615 = HEAP32[$614>>2]|0;
   HEAP32[$yygotominor>>2] = $615;
   $616 = HEAP32[$yygotominor>>2]|0;
   _sqlite3SrcListShiftJoinType($616);
   break;
  }
  case 135:  {
   $617 = HEAP32[$yymsp>>2]|0;
   $618 = (($617) + -16|0);
   $619 = (($618) + 4|0);
   $620 = HEAP32[$619>>2]|0;
   HEAP32[$yygotominor>>2] = $620;
   $621 = HEAP32[$yygotominor>>2]|0;
   $622 = ($621|0)!=(0|0);
   if ($622) {
    $623 = HEAP32[$yygotominor>>2]|0;
    $624 = HEAP32[$623>>2]|0;
    $625 = ($624|0)>(0);
    if ($625) {
     $626 = HEAP32[$yymsp>>2]|0;
     $627 = (($626) + 4|0);
     $628 = HEAP32[$627>>2]|0;
     $629 = $628&255;
     $630 = HEAP32[$yygotominor>>2]|0;
     $631 = HEAP32[$630>>2]|0;
     $632 = (($631) - 1)|0;
     $633 = HEAP32[$yygotominor>>2]|0;
     $634 = (($633) + 8|0);
     $635 = (($634) + (($632*72)|0)|0);
     $636 = (($635) + 36|0);
     HEAP8[$636>>0] = $629;
    }
   }
   break;
  }
  case 136:  {
   HEAP32[$yygotominor>>2] = 0;
   break;
  }
  case 39:  {
   $112 = HEAP32[$pParse>>2]|0;
   $113 = HEAP32[$yymsp>>2]|0;
   $114 = (($113) + 4|0);
   _sqlite3AddColumn($112,$114);
   $115 = HEAP32[$yymsp>>2]|0;
   $116 = (($115) + 4|0);
   ;HEAP32[$yygotominor+0>>2]=HEAP32[$116+0>>2]|0;HEAP32[$yygotominor+4>>2]=HEAP32[$116+4>>2]|0;
   $117 = HEAP32[$pParse>>2]|0;
   $118 = (($117) + 320|0);
   $119 = (($118) + 4|0);
   HEAP32[$119>>2] = 0;
   break;
  }
  case 106:  {
   $342 = HEAP32[$pParse>>2]|0;
   $343 = HEAP32[$yymsp>>2]|0;
   $344 = (($343) + 4|0);
   $345 = HEAP32[$344>>2]|0;
   $346 = HEAP32[$yymsp>>2]|0;
   $347 = (($346) + -16|0);
   $348 = (($347) + 4|0);
   $349 = HEAP32[$348>>2]|0;
   _sqlite3DropTable($342,$345,0,$349);
   break;
  }
  case 61:  {
   $216 = HEAP32[$pParse>>2]|0;
   $217 = HEAP32[$yymsp>>2]|0;
   $218 = (($217) + 4|0);
   $219 = HEAP32[$218>>2]|0;
   _sqlite3AddNotNull($216,$219);
   break;
  }
  case 38:  {
   $92 = HEAP32[$yymsp>>2]|0;
   $93 = (($92) + -32|0);
   $94 = (($93) + 4|0);
   $95 = HEAP32[$94>>2]|0;
   HEAP32[$yygotominor>>2] = $95;
   $96 = HEAP32[$pParse>>2]|0;
   $97 = (($96) + 504|0);
   $98 = HEAP32[$97>>2]|0;
   $99 = HEAP32[$yymsp>>2]|0;
   $100 = (($99) + -32|0);
   $101 = (($100) + 4|0);
   $102 = HEAP32[$101>>2]|0;
   $103 = $98;
   $104 = $102;
   $105 = (($103) - ($104))|0;
   $106 = HEAP32[$pParse>>2]|0;
   $107 = (($106) + 504|0);
   $108 = (($107) + 4|0);
   $109 = HEAP32[$108>>2]|0;
   $110 = (($105) + ($109))|0;
   $111 = (($yygotominor) + 4|0);
   HEAP32[$111>>2] = $110;
   break;
  }
  case 99:  {
   HEAP32[$yygotominor>>2] = 10;
   break;
  }
  case 101:  {
   HEAP8[$yygotominor>>0] = 10;
   break;
  }
  case 96:  {
   $318 = HEAP32[$pParse>>2]|0;
   $319 = HEAP32[$yymsp>>2]|0;
   $320 = (($319) + -96|0);
   $321 = (($320) + 4|0);
   $322 = HEAP32[$321>>2]|0;
   $323 = HEAP32[$yymsp>>2]|0;
   $324 = (($323) + -48|0);
   $325 = (($324) + 4|0);
   $326 = HEAP32[$yymsp>>2]|0;
   $327 = (($326) + -32|0);
   $328 = (($327) + 4|0);
   $329 = HEAP32[$328>>2]|0;
   $330 = HEAP32[$yymsp>>2]|0;
   $331 = (($330) + -16|0);
   $332 = (($331) + 4|0);
   $333 = HEAP32[$332>>2]|0;
   _sqlite3CreateForeignKey($318,$322,$325,$329,$333);
   $334 = HEAP32[$pParse>>2]|0;
   $335 = HEAP32[$yymsp>>2]|0;
   $336 = (($335) + 4|0);
   $337 = HEAP32[$336>>2]|0;
   _sqlite3DeferForeignKey($334,$337);
   break;
  }
  case 65:  {
   $241 = HEAP32[$pParse>>2]|0;
   $242 = HEAP32[$yymsp>>2]|0;
   $243 = (($242) + -32|0);
   $244 = (($243) + 4|0);
   $245 = HEAP32[$yymsp>>2]|0;
   $246 = (($245) + -16|0);
   $247 = (($246) + 4|0);
   $248 = HEAP32[$247>>2]|0;
   $249 = HEAP32[$yymsp>>2]|0;
   $250 = (($249) + 4|0);
   $251 = HEAP32[$250>>2]|0;
   _sqlite3CreateForeignKey($241,0,$244,$248,$251);
   break;
  }
  case 49:  {
   $165 = HEAP32[$yymsp>>2]|0;
   $166 = (($165) + -16|0);
   $167 = (($166) + 4|0);
   $168 = HEAP32[$167>>2]|0;
   HEAP32[$yygotominor>>2] = $168;
   $169 = HEAP32[$yymsp>>2]|0;
   $170 = (($169) + 4|0);
   $171 = (($170) + 4|0);
   $172 = HEAP32[$171>>2]|0;
   $173 = HEAP32[$yymsp>>2]|0;
   $174 = (($173) + 4|0);
   $175 = HEAP32[$174>>2]|0;
   $176 = HEAP32[$yymsp>>2]|0;
   $177 = (($176) + -16|0);
   $178 = (($177) + 4|0);
   $179 = HEAP32[$178>>2]|0;
   $180 = $175;
   $181 = $179;
   $182 = (($180) - ($181))|0;
   $183 = (($172) + ($182))|0;
   $184 = (($yygotominor) + 4|0);
   HEAP32[$184>>2] = $183;
   break;
  }
  case 57: case 55:  {
   $189 = HEAP32[$pParse>>2]|0;
   $190 = HEAP32[$yymsp>>2]|0;
   $191 = (($190) + 4|0);
   _sqlite3AddDefaultValue($189,$191);
   break;
  }
  case 102:  {
   $338 = HEAP32[$yymsp>>2]|0;
   $339 = (($338) + 4|0);
   $340 = HEAP32[$339>>2]|0;
   $341 = $340&255;
   HEAP8[$yygotominor>>0] = $341;
   break;
  }
  case 104:  {
   HEAP32[$yygotominor>>2] = 4;
   break;
  }
  case 105:  {
   HEAP32[$yygotominor>>2] = 5;
   break;
  }
  case 95:  {
   $313 = HEAP32[$pParse>>2]|0;
   $314 = HEAP32[$yymsp>>2]|0;
   $315 = (($314) + -32|0);
   $316 = (($315) + 4|0);
   $317 = HEAP32[$316>>2]|0;
   _sqlite3AddCheckConstraint($313,$317);
   break;
  }
  case 64:  {
   $236 = HEAP32[$pParse>>2]|0;
   $237 = HEAP32[$yymsp>>2]|0;
   $238 = (($237) + -16|0);
   $239 = (($238) + 4|0);
   $240 = HEAP32[$239>>2]|0;
   _sqlite3AddCheckConstraint($236,$240);
   break;
  }
  case 87:  {
   $287 = HEAP32[$yymsp>>2]|0;
   $288 = (($287) + -16|0);
   $289 = (($288) + 4|0);
   ;HEAP32[$yygotominor+0>>2]=HEAP32[$289+0>>2]|0;HEAP32[$yygotominor+4>>2]=HEAP32[$289+4>>2]|0;
   break;
  }
  case 90:  {
   $290 = HEAP32[$pParse>>2]|0;
   $291 = (($290) + 320|0);
   $292 = (($291) + 4|0);
   HEAP32[$292>>2] = 0;
   break;
  }
  case 110:  {
   $371 = HEAP32[$pParse>>2]|0;
   $372 = HEAP32[$yymsp>>2]|0;
   $373 = (($372) + 4|0);
   $374 = HEAP32[$373>>2]|0;
   $375 = HEAP32[$yymsp>>2]|0;
   $376 = (($375) + -16|0);
   $377 = (($376) + 4|0);
   $378 = HEAP32[$377>>2]|0;
   _sqlite3DropTable($371,$374,1,$378);
   break;
  }
  case 242: case 236: case 161: case 154: case 126:  {
   HEAP32[$yygotominor>>2] = 0;
   break;
  }
  case 127:  {
   $555 = HEAP32[$pParse>>2]|0;
   $556 = HEAP32[$yymsp>>2]|0;
   $557 = (($556) + -32|0);
   $558 = (($557) + 4|0);
   $559 = HEAP32[$558>>2]|0;
   $560 = HEAP32[$yymsp>>2]|0;
   $561 = (($560) + -16|0);
   $562 = (($561) + 4|0);
   $563 = HEAP32[$562>>2]|0;
   $564 = (_sqlite3ExprListAppend($555,$559,$563)|0);
   HEAP32[$yygotominor>>2] = $564;
   $565 = HEAP32[$yymsp>>2]|0;
   $566 = (($565) + 4|0);
   $567 = (($566) + 4|0);
   $568 = HEAP32[$567>>2]|0;
   $569 = ($568>>>0)>(0);
   if ($569) {
    $570 = HEAP32[$pParse>>2]|0;
    $571 = HEAP32[$yygotominor>>2]|0;
    $572 = HEAP32[$yymsp>>2]|0;
    $573 = (($572) + 4|0);
    _sqlite3ExprListSetName($570,$571,$573,1);
   }
   $574 = HEAP32[$pParse>>2]|0;
   $575 = HEAP32[$yygotominor>>2]|0;
   $576 = HEAP32[$yymsp>>2]|0;
   $577 = (($576) + -16|0);
   $578 = (($577) + 4|0);
   _sqlite3ExprListSetSpan($574,$575,$578);
   break;
  }
  case 128:  {
   $579 = HEAP32[$pParse>>2]|0;
   $580 = HEAP32[$579>>2]|0;
   $581 = (_sqlite3Expr($580,116,0)|0);
   HEAP32[$p2>>2] = $581;
   $582 = HEAP32[$pParse>>2]|0;
   $583 = HEAP32[$yymsp>>2]|0;
   $584 = (($583) + -16|0);
   $585 = (($584) + 4|0);
   $586 = HEAP32[$585>>2]|0;
   $587 = HEAP32[$p2>>2]|0;
   $588 = (_sqlite3ExprListAppend($582,$586,$587)|0);
   HEAP32[$yygotominor>>2] = $588;
   break;
  }
  case 132:  {
   $609 = (($yygotominor) + 4|0);
   HEAP32[$609>>2] = 0;
   break;
  }
  case 129:  {
   $589 = HEAP32[$pParse>>2]|0;
   $590 = HEAP32[$yymsp>>2]|0;
   $591 = (($590) + 4|0);
   $592 = (_sqlite3PExpr($589,116,0,0,$591)|0);
   HEAP32[$pRight3>>2] = $592;
   $593 = HEAP32[$pParse>>2]|0;
   $594 = HEAP32[$yymsp>>2]|0;
   $595 = (($594) + -32|0);
   $596 = (($595) + 4|0);
   $597 = (_sqlite3PExpr($593,27,0,0,$596)|0);
   HEAP32[$pLeft>>2] = $597;
   $598 = HEAP32[$pParse>>2]|0;
   $599 = HEAP32[$pLeft>>2]|0;
   $600 = HEAP32[$pRight3>>2]|0;
   $601 = (_sqlite3PExpr($598,122,$599,$600,0)|0);
   HEAP32[$pDot>>2] = $601;
   $602 = HEAP32[$pParse>>2]|0;
   $603 = HEAP32[$yymsp>>2]|0;
   $604 = (($603) + -48|0);
   $605 = (($604) + 4|0);
   $606 = HEAP32[$605>>2]|0;
   $607 = HEAP32[$pDot>>2]|0;
   $608 = (_sqlite3ExprListAppend($602,$606,$607)|0);
   HEAP32[$yygotominor>>2] = $608;
   break;
  }
  case 133:  {
   $610 = HEAP32[$pParse>>2]|0;
   $611 = HEAP32[$610>>2]|0;
   $612 = (_sqlite3DbMallocZero($611,80,0)|0);
   HEAP32[$yygotominor>>2] = $612;
   break;
  }
  case 137:  {
   $637 = HEAP32[$pParse>>2]|0;
   $638 = HEAP32[$yymsp>>2]|0;
   $639 = (($638) + -96|0);
   $640 = (($639) + 4|0);
   $641 = HEAP32[$640>>2]|0;
   $642 = HEAP32[$yymsp>>2]|0;
   $643 = (($642) + -80|0);
   $644 = (($643) + 4|0);
   $645 = HEAP32[$yymsp>>2]|0;
   $646 = (($645) + -64|0);
   $647 = (($646) + 4|0);
   $648 = HEAP32[$yymsp>>2]|0;
   $649 = (($648) + -48|0);
   $650 = (($649) + 4|0);
   $651 = HEAP32[$yymsp>>2]|0;
   $652 = (($651) + -16|0);
   $653 = (($652) + 4|0);
   $654 = HEAP32[$653>>2]|0;
   $655 = HEAP32[$yymsp>>2]|0;
   $656 = (($655) + 4|0);
   $657 = HEAP32[$656>>2]|0;
   $658 = (_sqlite3SrcListAppendFromTerm($637,$641,$644,$647,$650,0,$654,$657)|0);
   HEAP32[$yygotominor>>2] = $658;
   $659 = HEAP32[$pParse>>2]|0;
   $660 = HEAP32[$yygotominor>>2]|0;
   $661 = HEAP32[$yymsp>>2]|0;
   $662 = (($661) + -32|0);
   $663 = (($662) + 4|0);
   _sqlite3SrcListIndexedBy($659,$660,$663);
   break;
  }
  case 138:  {
   $664 = HEAP32[$pParse>>2]|0;
   $665 = HEAP32[$yymsp>>2]|0;
   $666 = (($665) + -96|0);
   $667 = (($666) + 4|0);
   $668 = HEAP32[$667>>2]|0;
   $669 = HEAP32[$yymsp>>2]|0;
   $670 = (($669) + -32|0);
   $671 = (($670) + 4|0);
   $672 = HEAP32[$yymsp>>2]|0;
   $673 = (($672) + -64|0);
   $674 = (($673) + 4|0);
   $675 = HEAP32[$674>>2]|0;
   $676 = HEAP32[$yymsp>>2]|0;
   $677 = (($676) + -16|0);
   $678 = (($677) + 4|0);
   $679 = HEAP32[$678>>2]|0;
   $680 = HEAP32[$yymsp>>2]|0;
   $681 = (($680) + 4|0);
   $682 = HEAP32[$681>>2]|0;
   $683 = (_sqlite3SrcListAppendFromTerm($664,$668,0,0,$671,$675,$679,$682)|0);
   HEAP32[$yygotominor>>2] = $683;
   break;
  }
  case 139:  {
   $684 = HEAP32[$yymsp>>2]|0;
   $685 = (($684) + -96|0);
   $686 = (($685) + 4|0);
   $687 = HEAP32[$686>>2]|0;
   $688 = ($687|0)==(0|0);
   if ($688) {
    $689 = HEAP32[$yymsp>>2]|0;
    $690 = (($689) + -32|0);
    $691 = (($690) + 4|0);
    $692 = (($691) + 4|0);
    $693 = HEAP32[$692>>2]|0;
    $694 = ($693|0)==(0);
    if ($694) {
     $695 = HEAP32[$yymsp>>2]|0;
     $696 = (($695) + -16|0);
     $697 = (($696) + 4|0);
     $698 = HEAP32[$697>>2]|0;
     $699 = ($698|0)==(0|0);
     if ($699) {
      $700 = HEAP32[$yymsp>>2]|0;
      $701 = (($700) + 4|0);
      $702 = HEAP32[$701>>2]|0;
      $703 = ($702|0)==(0|0);
      if ($703) {
       $704 = HEAP32[$yymsp>>2]|0;
       $705 = (($704) + -64|0);
       $706 = (($705) + 4|0);
       $707 = HEAP32[$706>>2]|0;
       HEAP32[$yygotominor>>2] = $707;
       break L1;
      }
     }
    }
   }
   $708 = HEAP32[$yymsp>>2]|0;
   $709 = (($708) + -64|0);
   $710 = (($709) + 4|0);
   $711 = HEAP32[$710>>2]|0;
   $712 = HEAP32[$711>>2]|0;
   $713 = ($712|0)==(1);
   if (!($713)) {
    $770 = HEAP32[$yymsp>>2]|0;
    $771 = (($770) + -64|0);
    $772 = (($771) + 4|0);
    $773 = HEAP32[$772>>2]|0;
    _sqlite3SrcListShiftJoinType($773);
    $774 = HEAP32[$pParse>>2]|0;
    $775 = HEAP32[$yymsp>>2]|0;
    $776 = (($775) + -64|0);
    $777 = (($776) + 4|0);
    $778 = HEAP32[$777>>2]|0;
    $779 = (_sqlite3SelectNew($774,0,$778,0,0,0,0,512,0,0)|0);
    HEAP32[$pSubquery>>2] = $779;
    $780 = HEAP32[$pParse>>2]|0;
    $781 = HEAP32[$yymsp>>2]|0;
    $782 = (($781) + -96|0);
    $783 = (($782) + 4|0);
    $784 = HEAP32[$783>>2]|0;
    $785 = HEAP32[$yymsp>>2]|0;
    $786 = (($785) + -32|0);
    $787 = (($786) + 4|0);
    $788 = HEAP32[$pSubquery>>2]|0;
    $789 = HEAP32[$yymsp>>2]|0;
    $790 = (($789) + -16|0);
    $791 = (($790) + 4|0);
    $792 = HEAP32[$791>>2]|0;
    $793 = HEAP32[$yymsp>>2]|0;
    $794 = (($793) + 4|0);
    $795 = HEAP32[$794>>2]|0;
    $796 = (_sqlite3SrcListAppendFromTerm($780,$784,0,0,$787,$788,$792,$795)|0);
    HEAP32[$yygotominor>>2] = $796;
    break L1;
   }
   $714 = HEAP32[$pParse>>2]|0;
   $715 = HEAP32[$yymsp>>2]|0;
   $716 = (($715) + -96|0);
   $717 = (($716) + 4|0);
   $718 = HEAP32[$717>>2]|0;
   $719 = HEAP32[$yymsp>>2]|0;
   $720 = (($719) + -32|0);
   $721 = (($720) + 4|0);
   $722 = HEAP32[$yymsp>>2]|0;
   $723 = (($722) + -16|0);
   $724 = (($723) + 4|0);
   $725 = HEAP32[$724>>2]|0;
   $726 = HEAP32[$yymsp>>2]|0;
   $727 = (($726) + 4|0);
   $728 = HEAP32[$727>>2]|0;
   $729 = (_sqlite3SrcListAppendFromTerm($714,$718,0,0,$721,0,$725,$728)|0);
   HEAP32[$yygotominor>>2] = $729;
   $730 = HEAP32[$yygotominor>>2]|0;
   $731 = ($730|0)!=(0|0);
   if ($731) {
    $732 = HEAP32[$yygotominor>>2]|0;
    $733 = HEAP32[$732>>2]|0;
    $734 = (($733) - 1)|0;
    $735 = HEAP32[$yygotominor>>2]|0;
    $736 = (($735) + 8|0);
    $737 = (($736) + (($734*72)|0)|0);
    HEAP32[$pNew>>2] = $737;
    $738 = HEAP32[$yymsp>>2]|0;
    $739 = (($738) + -64|0);
    $740 = (($739) + 4|0);
    $741 = HEAP32[$740>>2]|0;
    $742 = (($741) + 8|0);
    HEAP32[$pOld>>2] = $742;
    $743 = HEAP32[$pOld>>2]|0;
    $744 = (($743) + 8|0);
    $745 = HEAP32[$744>>2]|0;
    $746 = HEAP32[$pNew>>2]|0;
    $747 = (($746) + 8|0);
    HEAP32[$747>>2] = $745;
    $748 = HEAP32[$pOld>>2]|0;
    $749 = (($748) + 4|0);
    $750 = HEAP32[$749>>2]|0;
    $751 = HEAP32[$pNew>>2]|0;
    $752 = (($751) + 4|0);
    HEAP32[$752>>2] = $750;
    $753 = HEAP32[$pOld>>2]|0;
    $754 = (($753) + 20|0);
    $755 = HEAP32[$754>>2]|0;
    $756 = HEAP32[$pNew>>2]|0;
    $757 = (($756) + 20|0);
    HEAP32[$757>>2] = $755;
    $758 = HEAP32[$pOld>>2]|0;
    $759 = (($758) + 4|0);
    HEAP32[$759>>2] = 0;
    $760 = HEAP32[$pOld>>2]|0;
    $761 = (($760) + 8|0);
    HEAP32[$761>>2] = 0;
    $762 = HEAP32[$pOld>>2]|0;
    $763 = (($762) + 20|0);
    HEAP32[$763>>2] = 0;
   }
   $764 = HEAP32[$pParse>>2]|0;
   $765 = HEAP32[$764>>2]|0;
   $766 = HEAP32[$yymsp>>2]|0;
   $767 = (($766) + -64|0);
   $768 = (($767) + 4|0);
   $769 = HEAP32[$768>>2]|0;
   _sqlite3SrcListDelete($765,$769);
   break;
  }
  case 149: case 140:  {
   HEAP32[$yygotominor>>2] = 0;
   $797 = (($yygotominor) + 4|0);
   HEAP32[$797>>2] = 0;
   break;
  }
  case 144:  {
   $806 = HEAP32[$pParse>>2]|0;
   $807 = HEAP32[$yymsp>>2]|0;
   $808 = (($807) + -16|0);
   $809 = (($808) + 4|0);
   $810 = (_sqlite3JoinType($806,$809,0,0)|0);
   HEAP32[$yygotominor>>2] = $810;
   break;
  }
  case 145:  {
   $811 = HEAP32[$pParse>>2]|0;
   $812 = HEAP32[$yymsp>>2]|0;
   $813 = (($812) + -32|0);
   $814 = (($813) + 4|0);
   $815 = HEAP32[$yymsp>>2]|0;
   $816 = (($815) + -16|0);
   $817 = (($816) + 4|0);
   $818 = (_sqlite3JoinType($811,$814,$817,0)|0);
   HEAP32[$yygotominor>>2] = $818;
   break;
  }
  case 146:  {
   $819 = HEAP32[$pParse>>2]|0;
   $820 = HEAP32[$yymsp>>2]|0;
   $821 = (($820) + -48|0);
   $822 = (($821) + 4|0);
   $823 = HEAP32[$yymsp>>2]|0;
   $824 = (($823) + -32|0);
   $825 = (($824) + 4|0);
   $826 = HEAP32[$yymsp>>2]|0;
   $827 = (($826) + -16|0);
   $828 = (($827) + 4|0);
   $829 = (_sqlite3JoinType($819,$822,$825,$828)|0);
   HEAP32[$yygotominor>>2] = $829;
   break;
  }
  case 143:  {
   HEAP32[$yygotominor>>2] = 1;
   break;
  }
  case 142:  {
   $798 = HEAP32[$pParse>>2]|0;
   $799 = HEAP32[$798>>2]|0;
   $800 = HEAP32[$yymsp>>2]|0;
   $801 = (($800) + -16|0);
   $802 = (($801) + 4|0);
   $803 = HEAP32[$yymsp>>2]|0;
   $804 = (($803) + 4|0);
   $805 = (_sqlite3SrcListAppend($799,0,$802,$804)|0);
   HEAP32[$yygotominor>>2] = $805;
   break;
  }
  case 180: case 152:  {
   $834 = HEAP32[$yymsp>>2]|0;
   $835 = (($834) + -16|0);
   $836 = (($835) + 4|0);
   $837 = HEAP32[$836>>2]|0;
   HEAP32[$yygotominor>>2] = $837;
   break;
  }
  case 179: case 153:  {
   HEAP32[$yygotominor>>2] = 0;
   break;
  }
  case 235: case 162: case 155:  {
   $838 = HEAP32[$yymsp>>2]|0;
   $839 = (($838) + 4|0);
   $840 = HEAP32[$839>>2]|0;
   HEAP32[$yygotominor>>2] = $840;
   break;
  }
  case 156:  {
   $841 = HEAP32[$pParse>>2]|0;
   $842 = HEAP32[$yymsp>>2]|0;
   $843 = (($842) + -48|0);
   $844 = (($843) + 4|0);
   $845 = HEAP32[$844>>2]|0;
   $846 = HEAP32[$yymsp>>2]|0;
   $847 = (($846) + -16|0);
   $848 = (($847) + 4|0);
   $849 = HEAP32[$848>>2]|0;
   $850 = (_sqlite3ExprListAppend($841,$845,$849)|0);
   HEAP32[$yygotominor>>2] = $850;
   $851 = HEAP32[$yygotominor>>2]|0;
   $852 = ($851|0)!=(0|0);
   if ($852) {
    $853 = HEAP32[$yymsp>>2]|0;
    $854 = (($853) + 4|0);
    $855 = HEAP32[$854>>2]|0;
    $856 = $855&255;
    $857 = HEAP32[$yygotominor>>2]|0;
    $858 = HEAP32[$857>>2]|0;
    $859 = (($858) - 1)|0;
    $860 = HEAP32[$yygotominor>>2]|0;
    $861 = (($860) + 4|0);
    $862 = HEAP32[$861>>2]|0;
    $863 = (($862) + (($859*20)|0)|0);
    $864 = (($863) + 12|0);
    HEAP8[$864>>0] = $856;
   }
   break;
  }
  case 233: case 231: case 171: case 164: case 147:  {
   $830 = HEAP32[$yymsp>>2]|0;
   $831 = (($830) + 4|0);
   $832 = HEAP32[$831>>2]|0;
   HEAP32[$yygotominor>>2] = $832;
   break;
  }
  case 234: case 232: case 170: case 163: case 148:  {
   HEAP32[$yygotominor>>2] = 0;
   break;
  }
  case 151:  {
   HEAP32[$yygotominor>>2] = 0;
   $833 = (($yygotominor) + 4|0);
   HEAP32[$833>>2] = 1;
   break;
  }
  case 160: case 158:  {
   HEAP32[$yygotominor>>2] = 0;
   break;
  }
  case 159:  {
   HEAP32[$yygotominor>>2] = 1;
   break;
  }
  case 165:  {
   HEAP32[$yygotominor>>2] = 0;
   $885 = (($yygotominor) + 4|0);
   HEAP32[$885>>2] = 0;
   break;
  }
  case 166:  {
   $886 = HEAP32[$yymsp>>2]|0;
   $887 = (($886) + 4|0);
   $888 = HEAP32[$887>>2]|0;
   HEAP32[$yygotominor>>2] = $888;
   $889 = (($yygotominor) + 4|0);
   HEAP32[$889>>2] = 0;
   break;
  }
  case 167:  {
   $890 = HEAP32[$yymsp>>2]|0;
   $891 = (($890) + -32|0);
   $892 = (($891) + 4|0);
   $893 = HEAP32[$892>>2]|0;
   HEAP32[$yygotominor>>2] = $893;
   $894 = HEAP32[$yymsp>>2]|0;
   $895 = (($894) + 4|0);
   $896 = HEAP32[$895>>2]|0;
   $897 = (($yygotominor) + 4|0);
   HEAP32[$897>>2] = $896;
   break;
  }
  case 157:  {
   $865 = HEAP32[$pParse>>2]|0;
   $866 = HEAP32[$yymsp>>2]|0;
   $867 = (($866) + -16|0);
   $868 = (($867) + 4|0);
   $869 = HEAP32[$868>>2]|0;
   $870 = (_sqlite3ExprListAppend($865,0,$869)|0);
   HEAP32[$yygotominor>>2] = $870;
   $871 = HEAP32[$yygotominor>>2]|0;
   $872 = ($871|0)!=(0|0);
   if ($872) {
    $873 = HEAP32[$yygotominor>>2]|0;
    $874 = (($873) + 4|0);
    $875 = HEAP32[$874>>2]|0;
    $876 = ($875|0)!=(0|0);
    if ($876) {
     $877 = HEAP32[$yymsp>>2]|0;
     $878 = (($877) + 4|0);
     $879 = HEAP32[$878>>2]|0;
     $880 = $879&255;
     $881 = HEAP32[$yygotominor>>2]|0;
     $882 = (($881) + 4|0);
     $883 = HEAP32[$882>>2]|0;
     $884 = (($883) + 12|0);
     HEAP8[$884>>0] = $880;
    }
   }
   break;
  }
  case 181:  {
   $1030 = HEAP32[$pParse>>2]|0;
   $1031 = HEAP32[$1030>>2]|0;
   $1032 = HEAP32[$yymsp>>2]|0;
   $1033 = (($1032) + -32|0);
   $1034 = (($1033) + 4|0);
   $1035 = HEAP32[$1034>>2]|0;
   $1036 = HEAP32[$yymsp>>2]|0;
   $1037 = (($1036) + 4|0);
   $1038 = (_sqlite3IdListAppend($1031,$1035,$1037)|0);
   HEAP32[$yygotominor>>2] = $1038;
   break;
  }
  case 182:  {
   $1039 = HEAP32[$pParse>>2]|0;
   $1040 = HEAP32[$1039>>2]|0;
   $1041 = HEAP32[$yymsp>>2]|0;
   $1042 = (($1041) + 4|0);
   $1043 = (_sqlite3IdListAppend($1040,0,$1042)|0);
   HEAP32[$yygotominor>>2] = $1043;
   break;
  }
  case 183:  {
   $1044 = HEAP32[$yymsp>>2]|0;
   $1045 = (($1044) + 4|0);
   ;HEAP32[$yygotominor+0>>2]=HEAP32[$1045+0>>2]|0;HEAP32[$yygotominor+4>>2]=HEAP32[$1045+4>>2]|0;HEAP32[$yygotominor+8>>2]=HEAP32[$1045+8>>2]|0;
   break;
  }
  case 168:  {
   $898 = HEAP32[$yymsp>>2]|0;
   $899 = (($898) + -32|0);
   $900 = (($899) + 4|0);
   $901 = HEAP32[$900>>2]|0;
   $902 = (($yygotominor) + 4|0);
   HEAP32[$902>>2] = $901;
   $903 = HEAP32[$yymsp>>2]|0;
   $904 = (($903) + 4|0);
   $905 = HEAP32[$904>>2]|0;
   HEAP32[$yygotominor>>2] = $905;
   break;
  }
  case 169:  {
   $906 = HEAP32[$pParse>>2]|0;
   $907 = HEAP32[$yymsp>>2]|0;
   $908 = (($907) + -80|0);
   $909 = (($908) + 4|0);
   $910 = HEAP32[$909>>2]|0;
   _sqlite3WithPush($906,$910,1);
   $911 = HEAP32[$pParse>>2]|0;
   $912 = HEAP32[$yymsp>>2]|0;
   $913 = (($912) + -32|0);
   $914 = (($913) + 4|0);
   $915 = HEAP32[$914>>2]|0;
   $916 = HEAP32[$yymsp>>2]|0;
   $917 = (($916) + -16|0);
   $918 = (($917) + 4|0);
   _sqlite3SrcListIndexedBy($911,$915,$918);
   $919 = HEAP32[$pParse>>2]|0;
   $920 = HEAP32[$yymsp>>2]|0;
   $921 = (($920) + -32|0);
   $922 = (($921) + 4|0);
   $923 = HEAP32[$922>>2]|0;
   $924 = HEAP32[$yymsp>>2]|0;
   $925 = (($924) + 4|0);
   $926 = HEAP32[$925>>2]|0;
   _sqlite3DeleteFrom($919,$923,$926);
   break;
  }
  case 35:  {
   $74 = HEAP32[$yymsp>>2]|0;
   $75 = (($74) + 4|0);
   $76 = (($75) + 4|0);
   $77 = HEAP32[$76>>2]|0;
   $78 = ($77|0)==(5);
   if ($78) {
    $79 = HEAP32[$yymsp>>2]|0;
    $80 = (($79) + 4|0);
    $81 = HEAP32[$80>>2]|0;
    $82 = (_sqlite3_strnicmp($81,9992,5)|0);
    $83 = ($82|0)==(0);
    if ($83) {
     HEAP8[$yygotominor>>0] = 32;
     break L1;
    }
   }
   HEAP8[$yygotominor>>0] = 0;
   $84 = HEAP32[$pParse>>2]|0;
   $85 = HEAP32[$yymsp>>2]|0;
   $86 = (($85) + 4|0);
   $87 = (($86) + 4|0);
   $88 = HEAP32[$87>>2]|0;
   $89 = HEAP32[$yymsp>>2]|0;
   $90 = (($89) + 4|0);
   $91 = HEAP32[$90>>2]|0;
   HEAP32[$vararg_buffer14>>2] = $88;
   $vararg_ptr1 = (($vararg_buffer14) + 4|0);
   HEAP32[$vararg_ptr1>>2] = $91;
   _sqlite3ErrorMsg($84,10000,$vararg_buffer14);
   break;
  }
  case 117: case 115: case 16: case 15: case 14:  {
   $20 = HEAP32[$yymsp>>2]|0;
   $21 = (($20) + 2|0);
   $22 = HEAP8[$21>>0]|0;
   $23 = $22&255;
   HEAP32[$yygotominor>>2] = $23;
   break;
  }
  case 32:  {
   $55 = HEAP32[$pParse>>2]|0;
   $56 = HEAP32[$yymsp>>2]|0;
   $57 = (($56) + -32|0);
   $58 = (($57) + 4|0);
   $59 = HEAP32[$yymsp>>2]|0;
   $60 = (($59) + -16|0);
   $61 = (($60) + 4|0);
   $62 = HEAP32[$yymsp>>2]|0;
   $63 = (($62) + 4|0);
   $64 = HEAP8[$63>>0]|0;
   _sqlite3EndTable($55,$58,$61,$64,0);
   break;
  }
  case 222: case 219: case 107: case 84: case 69: case 30: case 29:  {
   HEAP32[$yygotominor>>2] = 1;
   break;
  }
  case 18: case 17:  {
   $24 = HEAP32[$pParse>>2]|0;
   _sqlite3CommitTransaction($24);
   break;
  }
  case 75:  {
   $278 = HEAP32[$yymsp>>2]|0;
   $279 = (($278) + 4|0);
   $280 = HEAP32[$279>>2]|0;
   $281 = $280 << 8;
   HEAP32[$yygotominor>>2] = $281;
   $282 = (($yygotominor) + 4|0);
   HEAP32[$282>>2] = 65280;
   break;
  }
  case 76:  {
   HEAP32[$yygotominor>>2] = 7;
   break;
  }
  case 77:  {
   HEAP32[$yygotominor>>2] = 8;
   break;
  }
  case 78:  {
   HEAP32[$yygotominor>>2] = 9;
   break;
  }
  case 79:  {
   HEAP32[$yygotominor>>2] = 6;
   break;
  }
  case 80:  {
   HEAP32[$yygotominor>>2] = 0;
   break;
  }
  case 6:  {
   $12 = HEAP32[$pParse>>2]|0;
   _sqlite3BeginParse($12,1);
   break;
  }
  case 74:  {
   $274 = HEAP32[$yymsp>>2]|0;
   $275 = (($274) + 4|0);
   $276 = HEAP32[$275>>2]|0;
   HEAP32[$yygotominor>>2] = $276;
   $277 = (($yygotominor) + 4|0);
   HEAP32[$277>>2] = 255;
   break;
  }
  case 9:  {
   $15 = HEAP32[$pParse>>2]|0;
   $16 = HEAP32[$yymsp>>2]|0;
   $17 = (($16) + -16|0);
   $18 = (($17) + 4|0);
   $19 = HEAP32[$18>>2]|0;
   _sqlite3BeginTransaction($15,$19);
   break;
  }
  case 46:  {
   $125 = HEAP32[$yymsp>>2]|0;
   $126 = (($125) + -48|0);
   $127 = (($126) + 4|0);
   $128 = HEAP32[$127>>2]|0;
   HEAP32[$yygotominor>>2] = $128;
   $129 = HEAP32[$yymsp>>2]|0;
   $130 = (($129) + 4|0);
   $131 = (($130) + 4|0);
   $132 = HEAP32[$131>>2]|0;
   $133 = HEAP32[$yymsp>>2]|0;
   $134 = (($133) + 4|0);
   $135 = HEAP32[$134>>2]|0;
   $136 = (($135) + ($132)|0);
   $137 = HEAP32[$yymsp>>2]|0;
   $138 = (($137) + -48|0);
   $139 = (($138) + 4|0);
   $140 = HEAP32[$139>>2]|0;
   $141 = $136;
   $142 = $140;
   $143 = (($141) - ($142))|0;
   $144 = (($yygotominor) + 4|0);
   HEAP32[$144>>2] = $143;
   break;
  }
  case 34:  {
   HEAP8[$yygotominor>>0] = 0;
   break;
  }
  case 92: case 54:  {
   $185 = HEAP32[$pParse>>2]|0;
   $186 = (($185) + 320|0);
   $187 = HEAP32[$yymsp>>2]|0;
   $188 = (($187) + 4|0);
   ;HEAP32[$186+0>>2]=HEAP32[$188+0>>2]|0;HEAP32[$186+4>>2]=HEAP32[$188+4>>2]|0;
   break;
  }
  case 94:  {
   $305 = HEAP32[$pParse>>2]|0;
   $306 = HEAP32[$yymsp>>2]|0;
   $307 = (($306) + -32|0);
   $308 = (($307) + 4|0);
   $309 = HEAP32[$308>>2]|0;
   $310 = HEAP32[$yymsp>>2]|0;
   $311 = (($310) + 4|0);
   $312 = HEAP32[$311>>2]|0;
   (_sqlite3CreateIndex($305,0,0,0,$309,$312,0,0,0,0)|0);
   break;
  }
  case 172:  {
   $927 = HEAP32[$pParse>>2]|0;
   $928 = HEAP32[$yymsp>>2]|0;
   $929 = (($928) + -112|0);
   $930 = (($929) + 4|0);
   $931 = HEAP32[$930>>2]|0;
   _sqlite3WithPush($927,$931,1);
   $932 = HEAP32[$pParse>>2]|0;
   $933 = HEAP32[$yymsp>>2]|0;
   $934 = (($933) + -64|0);
   $935 = (($934) + 4|0);
   $936 = HEAP32[$935>>2]|0;
   $937 = HEAP32[$yymsp>>2]|0;
   $938 = (($937) + -48|0);
   $939 = (($938) + 4|0);
   _sqlite3SrcListIndexedBy($932,$936,$939);
   $940 = HEAP32[$pParse>>2]|0;
   $941 = HEAP32[$yymsp>>2]|0;
   $942 = (($941) + -16|0);
   $943 = (($942) + 4|0);
   $944 = HEAP32[$943>>2]|0;
   _sqlite3ExprListCheckLength($940,$944,10096);
   $945 = HEAP32[$pParse>>2]|0;
   $946 = HEAP32[$yymsp>>2]|0;
   $947 = (($946) + -64|0);
   $948 = (($947) + 4|0);
   $949 = HEAP32[$948>>2]|0;
   $950 = HEAP32[$yymsp>>2]|0;
   $951 = (($950) + -16|0);
   $952 = (($951) + 4|0);
   $953 = HEAP32[$952>>2]|0;
   $954 = HEAP32[$yymsp>>2]|0;
   $955 = (($954) + 4|0);
   $956 = HEAP32[$955>>2]|0;
   $957 = HEAP32[$yymsp>>2]|0;
   $958 = (($957) + -80|0);
   $959 = (($958) + 4|0);
   $960 = HEAP8[$959>>0]|0;
   $961 = $960&255;
   _sqlite3Update($945,$949,$953,$956,$961);
   break;
  }
  case 173:  {
   $962 = HEAP32[$pParse>>2]|0;
   $963 = HEAP32[$yymsp>>2]|0;
   $964 = (($963) + -64|0);
   $965 = (($964) + 4|0);
   $966 = HEAP32[$965>>2]|0;
   $967 = HEAP32[$yymsp>>2]|0;
   $968 = (($967) + 4|0);
   $969 = HEAP32[$968>>2]|0;
   $970 = (_sqlite3ExprListAppend($962,$966,$969)|0);
   HEAP32[$yygotominor>>2] = $970;
   $971 = HEAP32[$pParse>>2]|0;
   $972 = HEAP32[$yygotominor>>2]|0;
   $973 = HEAP32[$yymsp>>2]|0;
   $974 = (($973) + -32|0);
   $975 = (($974) + 4|0);
   _sqlite3ExprListSetName($971,$972,$975,1);
   break;
  }
  case 174:  {
   $976 = HEAP32[$pParse>>2]|0;
   $977 = HEAP32[$yymsp>>2]|0;
   $978 = (($977) + 4|0);
   $979 = HEAP32[$978>>2]|0;
   $980 = (_sqlite3ExprListAppend($976,0,$979)|0);
   HEAP32[$yygotominor>>2] = $980;
   $981 = HEAP32[$pParse>>2]|0;
   $982 = HEAP32[$yygotominor>>2]|0;
   $983 = HEAP32[$yymsp>>2]|0;
   $984 = (($983) + -32|0);
   $985 = (($984) + 4|0);
   _sqlite3ExprListSetName($981,$982,$985,1);
   break;
  }
  case 175:  {
   $986 = HEAP32[$pParse>>2]|0;
   $987 = HEAP32[$yymsp>>2]|0;
   $988 = (($987) + -80|0);
   $989 = (($988) + 4|0);
   $990 = HEAP32[$989>>2]|0;
   _sqlite3WithPush($986,$990,1);
   $991 = HEAP32[$pParse>>2]|0;
   $992 = HEAP32[$yymsp>>2]|0;
   $993 = (($992) + -32|0);
   $994 = (($993) + 4|0);
   $995 = HEAP32[$994>>2]|0;
   $996 = HEAP32[$yymsp>>2]|0;
   $997 = (($996) + 4|0);
   $998 = HEAP32[$997>>2]|0;
   $999 = HEAP32[$yymsp>>2]|0;
   $1000 = (($999) + -16|0);
   $1001 = (($1000) + 4|0);
   $1002 = HEAP32[$1001>>2]|0;
   $1003 = HEAP32[$yymsp>>2]|0;
   $1004 = (($1003) + -64|0);
   $1005 = (($1004) + 4|0);
   $1006 = HEAP8[$1005>>0]|0;
   $1007 = $1006&255;
   _sqlite3Insert($991,$995,$998,$1002,$1007);
   break;
  }
  case 176:  {
   $1008 = HEAP32[$pParse>>2]|0;
   $1009 = HEAP32[$yymsp>>2]|0;
   $1010 = (($1009) + -96|0);
   $1011 = (($1010) + 4|0);
   $1012 = HEAP32[$1011>>2]|0;
   _sqlite3WithPush($1008,$1012,1);
   $1013 = HEAP32[$pParse>>2]|0;
   $1014 = HEAP32[$yymsp>>2]|0;
   $1015 = (($1014) + -48|0);
   $1016 = (($1015) + 4|0);
   $1017 = HEAP32[$1016>>2]|0;
   $1018 = HEAP32[$yymsp>>2]|0;
   $1019 = (($1018) + -32|0);
   $1020 = (($1019) + 4|0);
   $1021 = HEAP32[$1020>>2]|0;
   $1022 = HEAP32[$yymsp>>2]|0;
   $1023 = (($1022) + -80|0);
   $1024 = (($1023) + 4|0);
   $1025 = HEAP8[$1024>>0]|0;
   $1026 = $1025&255;
   _sqlite3Insert($1013,$1017,0,$1021,$1026);
   break;
  }
  case 177:  {
   $1027 = HEAP32[$yymsp>>2]|0;
   $1028 = (($1027) + 4|0);
   $1029 = HEAP8[$1028>>0]|0;
   HEAP8[$yygotominor>>0] = $1029;
   break;
  }
  case 178:  {
   HEAP8[$yygotominor>>0] = 5;
   break;
  }
  case 184:  {
   $1046 = HEAP32[$yymsp>>2]|0;
   $1047 = (($1046) + -16|0);
   $1048 = (($1047) + 4|0);
   $1049 = HEAP32[$1048>>2]|0;
   HEAP32[$yygotominor>>2] = $1049;
   $1050 = HEAP32[$yymsp>>2]|0;
   $1051 = (($1050) + -32|0);
   $1052 = (($1051) + 4|0);
   $1053 = HEAP32[$yymsp>>2]|0;
   $1054 = (($1053) + 4|0);
   _spanSet($yygotominor,$1052,$1054);
   break;
  }
  case 191: case 190: case 185:  {
   $1055 = HEAP32[$pParse>>2]|0;
   $1056 = HEAP32[$yymsp>>2]|0;
   $1057 = (($1056) + 2|0);
   $1058 = HEAP8[$1057>>0]|0;
   $1059 = $1058&255;
   $1060 = HEAP32[$yymsp>>2]|0;
   $1061 = (($1060) + 4|0);
   _spanExpr($yygotominor,$1055,$1059,$1061);
   break;
  }
  case 187: case 186:  {
   $1062 = HEAP32[$pParse>>2]|0;
   $1063 = HEAP32[$yymsp>>2]|0;
   $1064 = (($1063) + 4|0);
   _spanExpr($yygotominor,$1062,27,$1064);
   break;
  }
  case 188:  {
   $1065 = HEAP32[$pParse>>2]|0;
   $1066 = HEAP32[$yymsp>>2]|0;
   $1067 = (($1066) + -32|0);
   $1068 = (($1067) + 4|0);
   $1069 = (_sqlite3PExpr($1065,27,0,0,$1068)|0);
   HEAP32[$temp1>>2] = $1069;
   $1070 = HEAP32[$pParse>>2]|0;
   $1071 = HEAP32[$yymsp>>2]|0;
   $1072 = (($1071) + 4|0);
   $1073 = (_sqlite3PExpr($1070,27,0,0,$1072)|0);
   HEAP32[$temp2>>2] = $1073;
   $1074 = HEAP32[$pParse>>2]|0;
   $1075 = HEAP32[$temp1>>2]|0;
   $1076 = HEAP32[$temp2>>2]|0;
   $1077 = (_sqlite3PExpr($1074,122,$1075,$1076,0)|0);
   HEAP32[$yygotominor>>2] = $1077;
   $1078 = HEAP32[$yymsp>>2]|0;
   $1079 = (($1078) + -32|0);
   $1080 = (($1079) + 4|0);
   $1081 = HEAP32[$yymsp>>2]|0;
   $1082 = (($1081) + 4|0);
   _spanSet($yygotominor,$1080,$1082);
   break;
  }
  case 189:  {
   $1083 = HEAP32[$pParse>>2]|0;
   $1084 = HEAP32[$yymsp>>2]|0;
   $1085 = (($1084) + -64|0);
   $1086 = (($1085) + 4|0);
   $1087 = (_sqlite3PExpr($1083,27,0,0,$1086)|0);
   HEAP32[$temp14>>2] = $1087;
   $1088 = HEAP32[$pParse>>2]|0;
   $1089 = HEAP32[$yymsp>>2]|0;
   $1090 = (($1089) + -32|0);
   $1091 = (($1090) + 4|0);
   $1092 = (_sqlite3PExpr($1088,27,0,0,$1091)|0);
   HEAP32[$temp25>>2] = $1092;
   $1093 = HEAP32[$pParse>>2]|0;
   $1094 = HEAP32[$yymsp>>2]|0;
   $1095 = (($1094) + 4|0);
   $1096 = (_sqlite3PExpr($1093,27,0,0,$1095)|0);
   HEAP32[$temp3>>2] = $1096;
   $1097 = HEAP32[$pParse>>2]|0;
   $1098 = HEAP32[$temp25>>2]|0;
   $1099 = HEAP32[$temp3>>2]|0;
   $1100 = (_sqlite3PExpr($1097,122,$1098,$1099,0)|0);
   HEAP32[$temp4>>2] = $1100;
   $1101 = HEAP32[$pParse>>2]|0;
   $1102 = HEAP32[$temp14>>2]|0;
   $1103 = HEAP32[$temp4>>2]|0;
   $1104 = (_sqlite3PExpr($1101,122,$1102,$1103,0)|0);
   HEAP32[$yygotominor>>2] = $1104;
   $1105 = HEAP32[$yymsp>>2]|0;
   $1106 = (($1105) + -64|0);
   $1107 = (($1106) + 4|0);
   $1108 = HEAP32[$yymsp>>2]|0;
   $1109 = (($1108) + 4|0);
   _spanSet($yygotominor,$1107,$1109);
   break;
  }
  case 192:  {
   $1110 = HEAP32[$yymsp>>2]|0;
   $1111 = (($1110) + 4|0);
   $1112 = (($1111) + 4|0);
   $1113 = HEAP32[$1112>>2]|0;
   $1114 = ($1113>>>0)>=(2);
   do {
    if ($1114) {
     $1115 = HEAP32[$yymsp>>2]|0;
     $1116 = (($1115) + 4|0);
     $1117 = HEAP32[$1116>>2]|0;
     $1118 = HEAP8[$1117>>0]|0;
     $1119 = $1118 << 24 >> 24;
     $1120 = ($1119|0)==(35);
     if ($1120) {
      $1121 = HEAP32[$yymsp>>2]|0;
      $1122 = (($1121) + 4|0);
      $1123 = HEAP32[$1122>>2]|0;
      $1124 = (($1123) + 1|0);
      $1125 = HEAP8[$1124>>0]|0;
      $1126 = $1125&255;
      $1127 = (1224 + ($1126)|0);
      $1128 = HEAP8[$1127>>0]|0;
      $1129 = $1128&255;
      $1130 = $1129 & 4;
      $1131 = ($1130|0)!=(0);
      if ($1131) {
       $1132 = HEAP32[$pParse>>2]|0;
       $1133 = (($1132) + 18|0);
       $1134 = HEAP8[$1133>>0]|0;
       $1135 = $1134&255;
       $1136 = ($1135|0)==(0);
       $1137 = HEAP32[$pParse>>2]|0;
       $1138 = HEAP32[$yymsp>>2]|0;
       $1139 = (($1138) + 4|0);
       if ($1136) {
        HEAP32[$vararg_buffer14>>2] = $1139;
        _sqlite3ErrorMsg($1137,9968,$vararg_buffer14);
        HEAP32[$yygotominor>>2] = 0;
        break;
       }
       $1140 = (_sqlite3PExpr($1137,159,0,0,$1139)|0);
       HEAP32[$yygotominor>>2] = $1140;
       $1141 = HEAP32[$yygotominor>>2]|0;
       $1142 = ($1141|0)!=(0|0);
       if ($1142) {
        $1143 = HEAP32[$yymsp>>2]|0;
        $1144 = (($1143) + 4|0);
        $1145 = HEAP32[$1144>>2]|0;
        $1146 = (($1145) + 1|0);
        $1147 = HEAP32[$yygotominor>>2]|0;
        $1148 = (($1147) + 28|0);
        (_sqlite3GetInt32($1146,$1148)|0);
       }
      } else {
       label = 170;
      }
     } else {
      label = 170;
     }
    } else {
     label = 170;
    }
   } while(0);
   if ((label|0) == 170) {
    $1149 = HEAP32[$pParse>>2]|0;
    $1150 = HEAP32[$yymsp>>2]|0;
    $1151 = (($1150) + 4|0);
    _spanExpr($yygotominor,$1149,135,$1151);
    $1152 = HEAP32[$pParse>>2]|0;
    $1153 = HEAP32[$yygotominor>>2]|0;
    _sqlite3ExprAssignVarNumber($1152,$1153);
   }
   $1154 = HEAP32[$yymsp>>2]|0;
   $1155 = (($1154) + 4|0);
   $1156 = HEAP32[$yymsp>>2]|0;
   $1157 = (($1156) + 4|0);
   _spanSet($yygotominor,$1155,$1157);
   break;
  }
  case 193:  {
   $1158 = HEAP32[$pParse>>2]|0;
   $1159 = HEAP32[$yymsp>>2]|0;
   $1160 = (($1159) + -32|0);
   $1161 = (($1160) + 4|0);
   $1162 = HEAP32[$1161>>2]|0;
   $1163 = HEAP32[$yymsp>>2]|0;
   $1164 = (($1163) + 4|0);
   $1165 = (_sqlite3ExprAddCollateToken($1158,$1162,$1164)|0);
   HEAP32[$yygotominor>>2] = $1165;
   $1166 = HEAP32[$yymsp>>2]|0;
   $1167 = (($1166) + -32|0);
   $1168 = (($1167) + 4|0);
   $1169 = (($1168) + 4|0);
   $1170 = HEAP32[$1169>>2]|0;
   $1171 = (($yygotominor) + 4|0);
   HEAP32[$1171>>2] = $1170;
   $1172 = HEAP32[$yymsp>>2]|0;
   $1173 = (($1172) + 4|0);
   $1174 = (($1173) + 4|0);
   $1175 = HEAP32[$1174>>2]|0;
   $1176 = HEAP32[$yymsp>>2]|0;
   $1177 = (($1176) + 4|0);
   $1178 = HEAP32[$1177>>2]|0;
   $1179 = (($1178) + ($1175)|0);
   $1180 = (($yygotominor) + 8|0);
   HEAP32[$1180>>2] = $1179;
   break;
  }
  case 194:  {
   $1181 = HEAP32[$pParse>>2]|0;
   $1182 = HEAP32[$yymsp>>2]|0;
   $1183 = (($1182) + -48|0);
   $1184 = (($1183) + 4|0);
   $1185 = HEAP32[$1184>>2]|0;
   $1186 = HEAP32[$yymsp>>2]|0;
   $1187 = (($1186) + -16|0);
   $1188 = (($1187) + 4|0);
   $1189 = (_sqlite3PExpr($1181,38,$1185,0,$1188)|0);
   HEAP32[$yygotominor>>2] = $1189;
   $1190 = HEAP32[$yymsp>>2]|0;
   $1191 = (($1190) + -80|0);
   $1192 = (($1191) + 4|0);
   $1193 = HEAP32[$yymsp>>2]|0;
   $1194 = (($1193) + 4|0);
   _spanSet($yygotominor,$1192,$1194);
   break;
  }
  case 195:  {
   $1195 = HEAP32[$yymsp>>2]|0;
   $1196 = (($1195) + -16|0);
   $1197 = (($1196) + 4|0);
   $1198 = HEAP32[$1197>>2]|0;
   $1199 = ($1198|0)!=(0|0);
   if ($1199) {
    $1200 = HEAP32[$yymsp>>2]|0;
    $1201 = (($1200) + -16|0);
    $1202 = (($1201) + 4|0);
    $1203 = HEAP32[$1202>>2]|0;
    $1204 = HEAP32[$1203>>2]|0;
    $1205 = HEAP32[$pParse>>2]|0;
    $1206 = HEAP32[$1205>>2]|0;
    $1207 = (($1206) + 88|0);
    $1208 = (($1207) + 24|0);
    $1209 = HEAP32[$1208>>2]|0;
    $1210 = ($1204|0)>($1209|0);
    if ($1210) {
     $1211 = HEAP32[$pParse>>2]|0;
     $1212 = HEAP32[$yymsp>>2]|0;
     $1213 = (($1212) + -64|0);
     $1214 = (($1213) + 4|0);
     HEAP32[$vararg_buffer14>>2] = $1214;
     _sqlite3ErrorMsg($1211,10112,$vararg_buffer14);
    }
   }
   $1215 = HEAP32[$pParse>>2]|0;
   $1216 = HEAP32[$yymsp>>2]|0;
   $1217 = (($1216) + -16|0);
   $1218 = (($1217) + 4|0);
   $1219 = HEAP32[$1218>>2]|0;
   $1220 = HEAP32[$yymsp>>2]|0;
   $1221 = (($1220) + -64|0);
   $1222 = (($1221) + 4|0);
   $1223 = (_sqlite3ExprFunction($1215,$1219,$1222)|0);
   HEAP32[$yygotominor>>2] = $1223;
   $1224 = HEAP32[$yymsp>>2]|0;
   $1225 = (($1224) + -64|0);
   $1226 = (($1225) + 4|0);
   $1227 = HEAP32[$yymsp>>2]|0;
   $1228 = (($1227) + 4|0);
   _spanSet($yygotominor,$1226,$1228);
   $1229 = HEAP32[$yymsp>>2]|0;
   $1230 = (($1229) + -32|0);
   $1231 = (($1230) + 4|0);
   $1232 = HEAP16[$1231>>1]|0;
   $1233 = $1232&65535;
   $1234 = ($1233|0)!=(0);
   if ($1234) {
    $1235 = HEAP32[$yygotominor>>2]|0;
    $1236 = ($1235|0)!=(0|0);
    if ($1236) {
     $1237 = HEAP32[$yygotominor>>2]|0;
     $1238 = (($1237) + 4|0);
     $1239 = HEAP32[$1238>>2]|0;
     $1240 = $1239 | 16;
     HEAP32[$1238>>2] = $1240;
    }
   }
   break;
  }
  case 196:  {
   $1241 = HEAP32[$pParse>>2]|0;
   $1242 = HEAP32[$yymsp>>2]|0;
   $1243 = (($1242) + -48|0);
   $1244 = (($1243) + 4|0);
   $1245 = (_sqlite3ExprFunction($1241,0,$1244)|0);
   HEAP32[$yygotominor>>2] = $1245;
   $1246 = HEAP32[$yymsp>>2]|0;
   $1247 = (($1246) + -48|0);
   $1248 = (($1247) + 4|0);
   $1249 = HEAP32[$yymsp>>2]|0;
   $1250 = (($1249) + 4|0);
   _spanSet($yygotominor,$1248,$1250);
   break;
  }
  case 197:  {
   $1251 = HEAP32[$pParse>>2]|0;
   $1252 = HEAP32[$yymsp>>2]|0;
   $1253 = (($1252) + 4|0);
   $1254 = (_sqlite3ExprFunction($1251,0,$1253)|0);
   HEAP32[$yygotominor>>2] = $1254;
   $1255 = HEAP32[$yymsp>>2]|0;
   $1256 = (($1255) + 4|0);
   $1257 = HEAP32[$yymsp>>2]|0;
   $1258 = (($1257) + 4|0);
   _spanSet($yygotominor,$1256,$1258);
   break;
  }
  case 205: case 204: case 203: case 202: case 201: case 200: case 199: case 198:  {
   $1259 = HEAP32[$pParse>>2]|0;
   $1260 = HEAP32[$yymsp>>2]|0;
   $1261 = (($1260) + -16|0);
   $1262 = (($1261) + 2|0);
   $1263 = HEAP8[$1262>>0]|0;
   $1264 = $1263&255;
   $1265 = HEAP32[$yymsp>>2]|0;
   $1266 = (($1265) + -32|0);
   $1267 = (($1266) + 4|0);
   $1268 = HEAP32[$yymsp>>2]|0;
   $1269 = (($1268) + 4|0);
   _spanBinaryExpr($yygotominor,$1259,$1264,$1267,$1269);
   break;
  }
  case 206:  {
   $1270 = HEAP32[$yymsp>>2]|0;
   $1271 = (($1270) + 4|0);
   ;HEAP32[$yygotominor+0>>2]=HEAP32[$1271+0>>2]|0;HEAP32[$yygotominor+4>>2]=HEAP32[$1271+4>>2]|0;
   $1272 = (($yygotominor) + 8|0);
   HEAP32[$1272>>2] = 0;
   break;
  }
  case 207:  {
   $1273 = HEAP32[$yymsp>>2]|0;
   $1274 = (($1273) + 4|0);
   ;HEAP32[$yygotominor+0>>2]=HEAP32[$1274+0>>2]|0;HEAP32[$yygotominor+4>>2]=HEAP32[$1274+4>>2]|0;
   $1275 = (($yygotominor) + 8|0);
   HEAP32[$1275>>2] = 1;
   break;
  }
  case 208:  {
   $1276 = HEAP32[$pParse>>2]|0;
   $1277 = HEAP32[$yymsp>>2]|0;
   $1278 = (($1277) + 4|0);
   $1279 = HEAP32[$1278>>2]|0;
   $1280 = (_sqlite3ExprListAppend($1276,0,$1279)|0);
   HEAP32[$pList>>2] = $1280;
   $1281 = HEAP32[$pParse>>2]|0;
   $1282 = HEAP32[$pList>>2]|0;
   $1283 = HEAP32[$yymsp>>2]|0;
   $1284 = (($1283) + -32|0);
   $1285 = (($1284) + 4|0);
   $1286 = HEAP32[$1285>>2]|0;
   $1287 = (_sqlite3ExprListAppend($1281,$1282,$1286)|0);
   HEAP32[$pList>>2] = $1287;
   $1288 = HEAP32[$pParse>>2]|0;
   $1289 = HEAP32[$pList>>2]|0;
   $1290 = HEAP32[$yymsp>>2]|0;
   $1291 = (($1290) + -16|0);
   $1292 = (($1291) + 4|0);
   $1293 = (_sqlite3ExprFunction($1288,$1289,$1292)|0);
   HEAP32[$yygotominor>>2] = $1293;
   $1294 = HEAP32[$yymsp>>2]|0;
   $1295 = (($1294) + -16|0);
   $1296 = (($1295) + 4|0);
   $1297 = (($1296) + 8|0);
   $1298 = HEAP32[$1297>>2]|0;
   $1299 = ($1298|0)!=(0);
   if ($1299) {
    $1300 = HEAP32[$pParse>>2]|0;
    $1301 = HEAP32[$yygotominor>>2]|0;
    $1302 = (_sqlite3PExpr($1300,19,$1301,0,0)|0);
    HEAP32[$yygotominor>>2] = $1302;
   }
   $1303 = HEAP32[$yymsp>>2]|0;
   $1304 = (($1303) + -32|0);
   $1305 = (($1304) + 4|0);
   $1306 = (($1305) + 4|0);
   $1307 = HEAP32[$1306>>2]|0;
   $1308 = (($yygotominor) + 4|0);
   HEAP32[$1308>>2] = $1307;
   $1309 = HEAP32[$yymsp>>2]|0;
   $1310 = (($1309) + 4|0);
   $1311 = (($1310) + 8|0);
   $1312 = HEAP32[$1311>>2]|0;
   $1313 = (($yygotominor) + 8|0);
   HEAP32[$1313>>2] = $1312;
   $1314 = HEAP32[$yygotominor>>2]|0;
   $1315 = ($1314|0)!=(0|0);
   if ($1315) {
    $1316 = HEAP32[$yygotominor>>2]|0;
    $1317 = (($1316) + 4|0);
    $1318 = HEAP32[$1317>>2]|0;
    $1319 = $1318 | 128;
    HEAP32[$1317>>2] = $1319;
   }
   break;
  }
  case 209:  {
   $1320 = HEAP32[$pParse>>2]|0;
   $1321 = HEAP32[$yymsp>>2]|0;
   $1322 = (($1321) + -32|0);
   $1323 = (($1322) + 4|0);
   $1324 = HEAP32[$1323>>2]|0;
   $1325 = (_sqlite3ExprListAppend($1320,0,$1324)|0);
   HEAP32[$pList6>>2] = $1325;
   $1326 = HEAP32[$pParse>>2]|0;
   $1327 = HEAP32[$pList6>>2]|0;
   $1328 = HEAP32[$yymsp>>2]|0;
   $1329 = (($1328) + -64|0);
   $1330 = (($1329) + 4|0);
   $1331 = HEAP32[$1330>>2]|0;
   $1332 = (_sqlite3ExprListAppend($1326,$1327,$1331)|0);
   HEAP32[$pList6>>2] = $1332;
   $1333 = HEAP32[$pParse>>2]|0;
   $1334 = HEAP32[$pList6>>2]|0;
   $1335 = HEAP32[$yymsp>>2]|0;
   $1336 = (($1335) + 4|0);
   $1337 = HEAP32[$1336>>2]|0;
   $1338 = (_sqlite3ExprListAppend($1333,$1334,$1337)|0);
   HEAP32[$pList6>>2] = $1338;
   $1339 = HEAP32[$pParse>>2]|0;
   $1340 = HEAP32[$pList6>>2]|0;
   $1341 = HEAP32[$yymsp>>2]|0;
   $1342 = (($1341) + -48|0);
   $1343 = (($1342) + 4|0);
   $1344 = (_sqlite3ExprFunction($1339,$1340,$1343)|0);
   HEAP32[$yygotominor>>2] = $1344;
   $1345 = HEAP32[$yymsp>>2]|0;
   $1346 = (($1345) + -48|0);
   $1347 = (($1346) + 4|0);
   $1348 = (($1347) + 8|0);
   $1349 = HEAP32[$1348>>2]|0;
   $1350 = ($1349|0)!=(0);
   if ($1350) {
    $1351 = HEAP32[$pParse>>2]|0;
    $1352 = HEAP32[$yygotominor>>2]|0;
    $1353 = (_sqlite3PExpr($1351,19,$1352,0,0)|0);
    HEAP32[$yygotominor>>2] = $1353;
   }
   $1354 = HEAP32[$yymsp>>2]|0;
   $1355 = (($1354) + -64|0);
   $1356 = (($1355) + 4|0);
   $1357 = (($1356) + 4|0);
   $1358 = HEAP32[$1357>>2]|0;
   $1359 = (($yygotominor) + 4|0);
   HEAP32[$1359>>2] = $1358;
   $1360 = HEAP32[$yymsp>>2]|0;
   $1361 = (($1360) + 4|0);
   $1362 = (($1361) + 8|0);
   $1363 = HEAP32[$1362>>2]|0;
   $1364 = (($yygotominor) + 8|0);
   HEAP32[$1364>>2] = $1363;
   $1365 = HEAP32[$yygotominor>>2]|0;
   $1366 = ($1365|0)!=(0|0);
   if ($1366) {
    $1367 = HEAP32[$yygotominor>>2]|0;
    $1368 = (($1367) + 4|0);
    $1369 = HEAP32[$1368>>2]|0;
    $1370 = $1369 | 128;
    HEAP32[$1368>>2] = $1370;
   }
   break;
  }
  case 223:  {
   $1477 = HEAP32[$yymsp>>2]|0;
   $1478 = (($1477) + -16|0);
   $1479 = (($1478) + 4|0);
   $1480 = HEAP32[$1479>>2]|0;
   $1481 = ($1480|0)==(0|0);
   do {
    if ($1481) {
     $1482 = HEAP32[$pParse>>2]|0;
     $1483 = HEAP32[$yymsp>>2]|0;
     $1484 = (($1483) + -48|0);
     $1485 = (($1484) + 4|0);
     $1486 = HEAP32[$1485>>2]|0;
     $1487 = (10152 + ($1486<<3)|0);
     $1488 = (_sqlite3PExpr($1482,132,0,0,$1487)|0);
     HEAP32[$yygotominor>>2] = $1488;
     $1489 = HEAP32[$pParse>>2]|0;
     $1490 = HEAP32[$1489>>2]|0;
     $1491 = HEAP32[$yymsp>>2]|0;
     $1492 = (($1491) + -64|0);
     $1493 = (($1492) + 4|0);
     $1494 = HEAP32[$1493>>2]|0;
     _sqlite3ExprDelete($1490,$1494);
    } else {
     $1495 = HEAP32[$yymsp>>2]|0;
     $1496 = (($1495) + -16|0);
     $1497 = (($1496) + 4|0);
     $1498 = HEAP32[$1497>>2]|0;
     $1499 = HEAP32[$1498>>2]|0;
     $1500 = ($1499|0)==(1);
     if ($1500) {
      $1501 = HEAP32[$yymsp>>2]|0;
      $1502 = (($1501) + -16|0);
      $1503 = (($1502) + 4|0);
      $1504 = HEAP32[$1503>>2]|0;
      $1505 = (($1504) + 4|0);
      $1506 = HEAP32[$1505>>2]|0;
      $1507 = HEAP32[$1506>>2]|0;
      HEAP32[$pRHS>>2] = $1507;
      $1508 = HEAP32[$yymsp>>2]|0;
      $1509 = (($1508) + -16|0);
      $1510 = (($1509) + 4|0);
      $1511 = HEAP32[$1510>>2]|0;
      $1512 = (($1511) + 4|0);
      $1513 = HEAP32[$1512>>2]|0;
      HEAP32[$1513>>2] = 0;
      $1514 = HEAP32[$pParse>>2]|0;
      $1515 = HEAP32[$1514>>2]|0;
      $1516 = HEAP32[$yymsp>>2]|0;
      $1517 = (($1516) + -16|0);
      $1518 = (($1517) + 4|0);
      $1519 = HEAP32[$1518>>2]|0;
      _sqlite3ExprListDelete($1515,$1519);
      $1520 = HEAP32[$pRHS>>2]|0;
      $1521 = ($1520|0)!=(0|0);
      if ($1521) {
       $1522 = HEAP32[$pRHS>>2]|0;
       $1523 = (($1522) + 4|0);
       $1524 = HEAP32[$1523>>2]|0;
       $1525 = $1524 & -257;
       HEAP32[$1523>>2] = $1525;
       $1526 = HEAP32[$pRHS>>2]|0;
       $1527 = (($1526) + 4|0);
       $1528 = HEAP32[$1527>>2]|0;
       $1529 = $1528 | 512;
       HEAP32[$1527>>2] = $1529;
      }
      $1530 = HEAP32[$pParse>>2]|0;
      $1531 = HEAP32[$yymsp>>2]|0;
      $1532 = (($1531) + -48|0);
      $1533 = (($1532) + 4|0);
      $1534 = HEAP32[$1533>>2]|0;
      $1535 = ($1534|0)!=(0);
      $1536 = $1535 ? 78 : 79;
      $1537 = HEAP32[$yymsp>>2]|0;
      $1538 = (($1537) + -64|0);
      $1539 = (($1538) + 4|0);
      $1540 = HEAP32[$1539>>2]|0;
      $1541 = HEAP32[$pRHS>>2]|0;
      $1542 = (_sqlite3PExpr($1530,$1536,$1540,$1541,0)|0);
      HEAP32[$yygotominor>>2] = $1542;
      break;
     }
     $1543 = HEAP32[$pParse>>2]|0;
     $1544 = HEAP32[$yymsp>>2]|0;
     $1545 = (($1544) + -64|0);
     $1546 = (($1545) + 4|0);
     $1547 = HEAP32[$1546>>2]|0;
     $1548 = (_sqlite3PExpr($1543,75,$1547,0,0)|0);
     HEAP32[$yygotominor>>2] = $1548;
     $1549 = HEAP32[$yygotominor>>2]|0;
     $1550 = ($1549|0)!=(0|0);
     if ($1550) {
      $1551 = HEAP32[$yymsp>>2]|0;
      $1552 = (($1551) + -16|0);
      $1553 = (($1552) + 4|0);
      $1554 = HEAP32[$1553>>2]|0;
      $1555 = HEAP32[$yygotominor>>2]|0;
      $1556 = (($1555) + 20|0);
      HEAP32[$1556>>2] = $1554;
      $1557 = HEAP32[$pParse>>2]|0;
      $1558 = HEAP32[$yygotominor>>2]|0;
      _sqlite3ExprSetHeight($1557,$1558);
     } else {
      $1559 = HEAP32[$pParse>>2]|0;
      $1560 = HEAP32[$1559>>2]|0;
      $1561 = HEAP32[$yymsp>>2]|0;
      $1562 = (($1561) + -16|0);
      $1563 = (($1562) + 4|0);
      $1564 = HEAP32[$1563>>2]|0;
      _sqlite3ExprListDelete($1560,$1564);
     }
     $1565 = HEAP32[$yymsp>>2]|0;
     $1566 = (($1565) + -48|0);
     $1567 = (($1566) + 4|0);
     $1568 = HEAP32[$1567>>2]|0;
     $1569 = ($1568|0)!=(0);
     if ($1569) {
      $1570 = HEAP32[$pParse>>2]|0;
      $1571 = HEAP32[$yygotominor>>2]|0;
      $1572 = (_sqlite3PExpr($1570,19,$1571,0,0)|0);
      HEAP32[$yygotominor>>2] = $1572;
     }
    }
   } while(0);
   $1573 = HEAP32[$yymsp>>2]|0;
   $1574 = (($1573) + -64|0);
   $1575 = (($1574) + 4|0);
   $1576 = (($1575) + 4|0);
   $1577 = HEAP32[$1576>>2]|0;
   $1578 = (($yygotominor) + 4|0);
   HEAP32[$1578>>2] = $1577;
   $1579 = HEAP32[$yymsp>>2]|0;
   $1580 = (($1579) + 4|0);
   $1581 = (($1580) + 4|0);
   $1582 = HEAP32[$1581>>2]|0;
   $1583 = HEAP32[$yymsp>>2]|0;
   $1584 = (($1583) + 4|0);
   $1585 = HEAP32[$1584>>2]|0;
   $1586 = (($1585) + ($1582)|0);
   $1587 = (($yygotominor) + 8|0);
   HEAP32[$1587>>2] = $1586;
   break;
  }
  case 210:  {
   $1371 = HEAP32[$pParse>>2]|0;
   $1372 = HEAP32[$yymsp>>2]|0;
   $1373 = (($1372) + 2|0);
   $1374 = HEAP8[$1373>>0]|0;
   $1375 = $1374&255;
   $1376 = HEAP32[$yymsp>>2]|0;
   $1377 = (($1376) + -16|0);
   $1378 = (($1377) + 4|0);
   $1379 = HEAP32[$yymsp>>2]|0;
   $1380 = (($1379) + 4|0);
   _spanUnaryPostfix($yygotominor,$1371,$1375,$1378,$1380);
   break;
  }
  case 211:  {
   $1381 = HEAP32[$pParse>>2]|0;
   $1382 = HEAP32[$yymsp>>2]|0;
   $1383 = (($1382) + -32|0);
   $1384 = (($1383) + 4|0);
   $1385 = HEAP32[$yymsp>>2]|0;
   $1386 = (($1385) + 4|0);
   _spanUnaryPostfix($yygotominor,$1381,77,$1384,$1386);
   break;
  }
  case 212:  {
   $1387 = HEAP32[$pParse>>2]|0;
   $1388 = HEAP32[$yymsp>>2]|0;
   $1389 = (($1388) + -32|0);
   $1390 = (($1389) + 4|0);
   $1391 = HEAP32[$yymsp>>2]|0;
   $1392 = (($1391) + 4|0);
   _spanBinaryExpr($yygotominor,$1387,73,$1390,$1392);
   $1393 = HEAP32[$pParse>>2]|0;
   $1394 = HEAP32[$yymsp>>2]|0;
   $1395 = (($1394) + 4|0);
   $1396 = HEAP32[$1395>>2]|0;
   $1397 = HEAP32[$yygotominor>>2]|0;
   _binaryToUnaryIfNull($1393,$1396,$1397,76);
   break;
  }
  case 213:  {
   $1398 = HEAP32[$pParse>>2]|0;
   $1399 = HEAP32[$yymsp>>2]|0;
   $1400 = (($1399) + -48|0);
   $1401 = (($1400) + 4|0);
   $1402 = HEAP32[$yymsp>>2]|0;
   $1403 = (($1402) + 4|0);
   _spanBinaryExpr($yygotominor,$1398,148,$1401,$1403);
   $1404 = HEAP32[$pParse>>2]|0;
   $1405 = HEAP32[$yymsp>>2]|0;
   $1406 = (($1405) + 4|0);
   $1407 = HEAP32[$1406>>2]|0;
   $1408 = HEAP32[$yygotominor>>2]|0;
   _binaryToUnaryIfNull($1404,$1407,$1408,77);
   break;
  }
  case 215: case 214:  {
   $1409 = HEAP32[$pParse>>2]|0;
   $1410 = HEAP32[$yymsp>>2]|0;
   $1411 = (($1410) + -16|0);
   $1412 = (($1411) + 2|0);
   $1413 = HEAP8[$1412>>0]|0;
   $1414 = $1413&255;
   $1415 = HEAP32[$yymsp>>2]|0;
   $1416 = (($1415) + 4|0);
   $1417 = HEAP32[$yymsp>>2]|0;
   $1418 = (($1417) + -16|0);
   $1419 = (($1418) + 4|0);
   _spanUnaryPrefix($yygotominor,$1409,$1414,$1416,$1419);
   break;
  }
  case 216:  {
   $1420 = HEAP32[$pParse>>2]|0;
   $1421 = HEAP32[$yymsp>>2]|0;
   $1422 = (($1421) + 4|0);
   $1423 = HEAP32[$yymsp>>2]|0;
   $1424 = (($1423) + -16|0);
   $1425 = (($1424) + 4|0);
   _spanUnaryPrefix($yygotominor,$1420,157,$1422,$1425);
   break;
  }
  case 217:  {
   $1426 = HEAP32[$pParse>>2]|0;
   $1427 = HEAP32[$yymsp>>2]|0;
   $1428 = (($1427) + 4|0);
   $1429 = HEAP32[$yymsp>>2]|0;
   $1430 = (($1429) + -16|0);
   $1431 = (($1430) + 4|0);
   _spanUnaryPrefix($yygotominor,$1426,158,$1428,$1431);
   break;
  }
  case 220:  {
   $1432 = HEAP32[$pParse>>2]|0;
   $1433 = HEAP32[$yymsp>>2]|0;
   $1434 = (($1433) + -32|0);
   $1435 = (($1434) + 4|0);
   $1436 = HEAP32[$1435>>2]|0;
   $1437 = (_sqlite3ExprListAppend($1432,0,$1436)|0);
   HEAP32[$pList7>>2] = $1437;
   $1438 = HEAP32[$pParse>>2]|0;
   $1439 = HEAP32[$pList7>>2]|0;
   $1440 = HEAP32[$yymsp>>2]|0;
   $1441 = (($1440) + 4|0);
   $1442 = HEAP32[$1441>>2]|0;
   $1443 = (_sqlite3ExprListAppend($1438,$1439,$1442)|0);
   HEAP32[$pList7>>2] = $1443;
   $1444 = HEAP32[$pParse>>2]|0;
   $1445 = HEAP32[$yymsp>>2]|0;
   $1446 = (($1445) + -64|0);
   $1447 = (($1446) + 4|0);
   $1448 = HEAP32[$1447>>2]|0;
   $1449 = (_sqlite3PExpr($1444,74,$1448,0,0)|0);
   HEAP32[$yygotominor>>2] = $1449;
   $1450 = HEAP32[$yygotominor>>2]|0;
   $1451 = ($1450|0)!=(0|0);
   if ($1451) {
    $1452 = HEAP32[$pList7>>2]|0;
    $1453 = HEAP32[$yygotominor>>2]|0;
    $1454 = (($1453) + 20|0);
    HEAP32[$1454>>2] = $1452;
   } else {
    $1455 = HEAP32[$pParse>>2]|0;
    $1456 = HEAP32[$1455>>2]|0;
    $1457 = HEAP32[$pList7>>2]|0;
    _sqlite3ExprListDelete($1456,$1457);
   }
   $1458 = HEAP32[$yymsp>>2]|0;
   $1459 = (($1458) + -48|0);
   $1460 = (($1459) + 4|0);
   $1461 = HEAP32[$1460>>2]|0;
   $1462 = ($1461|0)!=(0);
   if ($1462) {
    $1463 = HEAP32[$pParse>>2]|0;
    $1464 = HEAP32[$yygotominor>>2]|0;
    $1465 = (_sqlite3PExpr($1463,19,$1464,0,0)|0);
    HEAP32[$yygotominor>>2] = $1465;
   }
   $1466 = HEAP32[$yymsp>>2]|0;
   $1467 = (($1466) + -64|0);
   $1468 = (($1467) + 4|0);
   $1469 = (($1468) + 4|0);
   $1470 = HEAP32[$1469>>2]|0;
   $1471 = (($yygotominor) + 4|0);
   HEAP32[$1471>>2] = $1470;
   $1472 = HEAP32[$yymsp>>2]|0;
   $1473 = (($1472) + 4|0);
   $1474 = (($1473) + 8|0);
   $1475 = HEAP32[$1474>>2]|0;
   $1476 = (($yygotominor) + 8|0);
   HEAP32[$1476>>2] = $1475;
   break;
  }
  case 224:  {
   $1588 = HEAP32[$pParse>>2]|0;
   $1589 = (_sqlite3PExpr($1588,119,0,0,0)|0);
   HEAP32[$yygotominor>>2] = $1589;
   $1590 = HEAP32[$yygotominor>>2]|0;
   $1591 = ($1590|0)!=(0|0);
   if ($1591) {
    $1592 = HEAP32[$yymsp>>2]|0;
    $1593 = (($1592) + -16|0);
    $1594 = (($1593) + 4|0);
    $1595 = HEAP32[$1594>>2]|0;
    $1596 = HEAP32[$yygotominor>>2]|0;
    $1597 = (($1596) + 20|0);
    HEAP32[$1597>>2] = $1595;
    $1598 = HEAP32[$yygotominor>>2]|0;
    $1599 = (($1598) + 4|0);
    $1600 = HEAP32[$1599>>2]|0;
    $1601 = $1600 | 2048;
    HEAP32[$1599>>2] = $1601;
    $1602 = HEAP32[$pParse>>2]|0;
    $1603 = HEAP32[$yygotominor>>2]|0;
    _sqlite3ExprSetHeight($1602,$1603);
   } else {
    $1604 = HEAP32[$pParse>>2]|0;
    $1605 = HEAP32[$1604>>2]|0;
    $1606 = HEAP32[$yymsp>>2]|0;
    $1607 = (($1606) + -16|0);
    $1608 = (($1607) + 4|0);
    $1609 = HEAP32[$1608>>2]|0;
    _sqlite3SelectDelete($1605,$1609);
   }
   $1610 = HEAP32[$yymsp>>2]|0;
   $1611 = (($1610) + -32|0);
   $1612 = (($1611) + 4|0);
   $1613 = HEAP32[$1612>>2]|0;
   $1614 = (($yygotominor) + 4|0);
   HEAP32[$1614>>2] = $1613;
   $1615 = HEAP32[$yymsp>>2]|0;
   $1616 = (($1615) + 4|0);
   $1617 = (($1616) + 4|0);
   $1618 = HEAP32[$1617>>2]|0;
   $1619 = HEAP32[$yymsp>>2]|0;
   $1620 = (($1619) + 4|0);
   $1621 = HEAP32[$1620>>2]|0;
   $1622 = (($1621) + ($1618)|0);
   $1623 = (($yygotominor) + 8|0);
   HEAP32[$1623>>2] = $1622;
   break;
  }
  case 225:  {
   $1624 = HEAP32[$pParse>>2]|0;
   $1625 = HEAP32[$yymsp>>2]|0;
   $1626 = (($1625) + -64|0);
   $1627 = (($1626) + 4|0);
   $1628 = HEAP32[$1627>>2]|0;
   $1629 = (_sqlite3PExpr($1624,75,$1628,0,0)|0);
   HEAP32[$yygotominor>>2] = $1629;
   $1630 = HEAP32[$yygotominor>>2]|0;
   $1631 = ($1630|0)!=(0|0);
   if ($1631) {
    $1632 = HEAP32[$yymsp>>2]|0;
    $1633 = (($1632) + -16|0);
    $1634 = (($1633) + 4|0);
    $1635 = HEAP32[$1634>>2]|0;
    $1636 = HEAP32[$yygotominor>>2]|0;
    $1637 = (($1636) + 20|0);
    HEAP32[$1637>>2] = $1635;
    $1638 = HEAP32[$yygotominor>>2]|0;
    $1639 = (($1638) + 4|0);
    $1640 = HEAP32[$1639>>2]|0;
    $1641 = $1640 | 2048;
    HEAP32[$1639>>2] = $1641;
    $1642 = HEAP32[$pParse>>2]|0;
    $1643 = HEAP32[$yygotominor>>2]|0;
    _sqlite3ExprSetHeight($1642,$1643);
   } else {
    $1644 = HEAP32[$pParse>>2]|0;
    $1645 = HEAP32[$1644>>2]|0;
    $1646 = HEAP32[$yymsp>>2]|0;
    $1647 = (($1646) + -16|0);
    $1648 = (($1647) + 4|0);
    $1649 = HEAP32[$1648>>2]|0;
    _sqlite3SelectDelete($1645,$1649);
   }
   $1650 = HEAP32[$yymsp>>2]|0;
   $1651 = (($1650) + -48|0);
   $1652 = (($1651) + 4|0);
   $1653 = HEAP32[$1652>>2]|0;
   $1654 = ($1653|0)!=(0);
   if ($1654) {
    $1655 = HEAP32[$pParse>>2]|0;
    $1656 = HEAP32[$yygotominor>>2]|0;
    $1657 = (_sqlite3PExpr($1655,19,$1656,0,0)|0);
    HEAP32[$yygotominor>>2] = $1657;
   }
   $1658 = HEAP32[$yymsp>>2]|0;
   $1659 = (($1658) + -64|0);
   $1660 = (($1659) + 4|0);
   $1661 = (($1660) + 4|0);
   $1662 = HEAP32[$1661>>2]|0;
   $1663 = (($yygotominor) + 4|0);
   HEAP32[$1663>>2] = $1662;
   $1664 = HEAP32[$yymsp>>2]|0;
   $1665 = (($1664) + 4|0);
   $1666 = (($1665) + 4|0);
   $1667 = HEAP32[$1666>>2]|0;
   $1668 = HEAP32[$yymsp>>2]|0;
   $1669 = (($1668) + 4|0);
   $1670 = HEAP32[$1669>>2]|0;
   $1671 = (($1670) + ($1667)|0);
   $1672 = (($yygotominor) + 8|0);
   HEAP32[$1672>>2] = $1671;
   break;
  }
  case 226:  {
   $1673 = HEAP32[$pParse>>2]|0;
   $1674 = HEAP32[$1673>>2]|0;
   $1675 = HEAP32[$yymsp>>2]|0;
   $1676 = (($1675) + -16|0);
   $1677 = (($1676) + 4|0);
   $1678 = HEAP32[$yymsp>>2]|0;
   $1679 = (($1678) + 4|0);
   $1680 = (_sqlite3SrcListAppend($1674,0,$1677,$1679)|0);
   HEAP32[$pSrc>>2] = $1680;
   $1681 = HEAP32[$pParse>>2]|0;
   $1682 = HEAP32[$yymsp>>2]|0;
   $1683 = (($1682) + -48|0);
   $1684 = (($1683) + 4|0);
   $1685 = HEAP32[$1684>>2]|0;
   $1686 = (_sqlite3PExpr($1681,75,$1685,0,0)|0);
   HEAP32[$yygotominor>>2] = $1686;
   $1687 = HEAP32[$yygotominor>>2]|0;
   $1688 = ($1687|0)!=(0|0);
   $1689 = HEAP32[$pParse>>2]|0;
   if ($1688) {
    $1690 = HEAP32[$pSrc>>2]|0;
    $1691 = (_sqlite3SelectNew($1689,0,$1690,0,0,0,0,0,0,0)|0);
    $1692 = HEAP32[$yygotominor>>2]|0;
    $1693 = (($1692) + 20|0);
    HEAP32[$1693>>2] = $1691;
    $1694 = HEAP32[$yygotominor>>2]|0;
    $1695 = (($1694) + 4|0);
    $1696 = HEAP32[$1695>>2]|0;
    $1697 = $1696 | 2048;
    HEAP32[$1695>>2] = $1697;
    $1698 = HEAP32[$pParse>>2]|0;
    $1699 = HEAP32[$yygotominor>>2]|0;
    _sqlite3ExprSetHeight($1698,$1699);
   } else {
    $1700 = HEAP32[$1689>>2]|0;
    $1701 = HEAP32[$pSrc>>2]|0;
    _sqlite3SrcListDelete($1700,$1701);
   }
   $1702 = HEAP32[$yymsp>>2]|0;
   $1703 = (($1702) + -32|0);
   $1704 = (($1703) + 4|0);
   $1705 = HEAP32[$1704>>2]|0;
   $1706 = ($1705|0)!=(0);
   if ($1706) {
    $1707 = HEAP32[$pParse>>2]|0;
    $1708 = HEAP32[$yygotominor>>2]|0;
    $1709 = (_sqlite3PExpr($1707,19,$1708,0,0)|0);
    HEAP32[$yygotominor>>2] = $1709;
   }
   $1710 = HEAP32[$yymsp>>2]|0;
   $1711 = (($1710) + -48|0);
   $1712 = (($1711) + 4|0);
   $1713 = (($1712) + 4|0);
   $1714 = HEAP32[$1713>>2]|0;
   $1715 = (($yygotominor) + 4|0);
   HEAP32[$1715>>2] = $1714;
   $1716 = HEAP32[$yymsp>>2]|0;
   $1717 = (($1716) + 4|0);
   $1718 = HEAP32[$1717>>2]|0;
   $1719 = ($1718|0)!=(0|0);
   $1720 = HEAP32[$yymsp>>2]|0;
   if ($1719) {
    $1721 = (($1720) + 4|0);
    $1722 = (($1721) + 4|0);
    $1723 = HEAP32[$1722>>2]|0;
    $1724 = HEAP32[$yymsp>>2]|0;
    $1725 = (($1724) + 4|0);
    $1726 = HEAP32[$1725>>2]|0;
    $1727 = (($1726) + ($1723)|0);
    $1738 = $1727;
   } else {
    $1728 = (($1720) + -16|0);
    $1729 = (($1728) + 4|0);
    $1730 = (($1729) + 4|0);
    $1731 = HEAP32[$1730>>2]|0;
    $1732 = HEAP32[$yymsp>>2]|0;
    $1733 = (($1732) + -16|0);
    $1734 = (($1733) + 4|0);
    $1735 = HEAP32[$1734>>2]|0;
    $1736 = (($1735) + ($1731)|0);
    $1738 = $1736;
   }
   $1737 = (($yygotominor) + 8|0);
   HEAP32[$1737>>2] = $1738;
   break;
  }
  case 227:  {
   $1739 = HEAP32[$pParse>>2]|0;
   $1740 = (_sqlite3PExpr($1739,20,0,0,0)|0);
   HEAP32[$yygotominor>>2] = $1740;
   HEAP32[$p8>>2] = $1740;
   $1741 = HEAP32[$p8>>2]|0;
   $1742 = ($1741|0)!=(0|0);
   if ($1742) {
    $1743 = HEAP32[$yymsp>>2]|0;
    $1744 = (($1743) + -16|0);
    $1745 = (($1744) + 4|0);
    $1746 = HEAP32[$1745>>2]|0;
    $1747 = HEAP32[$p8>>2]|0;
    $1748 = (($1747) + 20|0);
    HEAP32[$1748>>2] = $1746;
    $1749 = HEAP32[$p8>>2]|0;
    $1750 = (($1749) + 4|0);
    $1751 = HEAP32[$1750>>2]|0;
    $1752 = $1751 | 2048;
    HEAP32[$1750>>2] = $1752;
    $1753 = HEAP32[$pParse>>2]|0;
    $1754 = HEAP32[$p8>>2]|0;
    _sqlite3ExprSetHeight($1753,$1754);
   } else {
    $1755 = HEAP32[$pParse>>2]|0;
    $1756 = HEAP32[$1755>>2]|0;
    $1757 = HEAP32[$yymsp>>2]|0;
    $1758 = (($1757) + -16|0);
    $1759 = (($1758) + 4|0);
    $1760 = HEAP32[$1759>>2]|0;
    _sqlite3SelectDelete($1756,$1760);
   }
   $1761 = HEAP32[$yymsp>>2]|0;
   $1762 = (($1761) + -48|0);
   $1763 = (($1762) + 4|0);
   $1764 = HEAP32[$1763>>2]|0;
   $1765 = (($yygotominor) + 4|0);
   HEAP32[$1765>>2] = $1764;
   $1766 = HEAP32[$yymsp>>2]|0;
   $1767 = (($1766) + 4|0);
   $1768 = (($1767) + 4|0);
   $1769 = HEAP32[$1768>>2]|0;
   $1770 = HEAP32[$yymsp>>2]|0;
   $1771 = (($1770) + 4|0);
   $1772 = HEAP32[$1771>>2]|0;
   $1773 = (($1772) + ($1769)|0);
   $1774 = (($yygotominor) + 8|0);
   HEAP32[$1774>>2] = $1773;
   break;
  }
  case 228:  {
   $1775 = HEAP32[$pParse>>2]|0;
   $1776 = HEAP32[$yymsp>>2]|0;
   $1777 = (($1776) + -48|0);
   $1778 = (($1777) + 4|0);
   $1779 = HEAP32[$1778>>2]|0;
   $1780 = (_sqlite3PExpr($1775,136,$1779,0,0)|0);
   HEAP32[$yygotominor>>2] = $1780;
   $1781 = HEAP32[$yygotominor>>2]|0;
   $1782 = ($1781|0)!=(0|0);
   if ($1782) {
    $1783 = HEAP32[$yymsp>>2]|0;
    $1784 = (($1783) + -16|0);
    $1785 = (($1784) + 4|0);
    $1786 = HEAP32[$1785>>2]|0;
    $1787 = ($1786|0)!=(0|0);
    if ($1787) {
     $1788 = HEAP32[$pParse>>2]|0;
     $1789 = HEAP32[$yymsp>>2]|0;
     $1790 = (($1789) + -32|0);
     $1791 = (($1790) + 4|0);
     $1792 = HEAP32[$1791>>2]|0;
     $1793 = HEAP32[$yymsp>>2]|0;
     $1794 = (($1793) + -16|0);
     $1795 = (($1794) + 4|0);
     $1796 = HEAP32[$1795>>2]|0;
     $1797 = (_sqlite3ExprListAppend($1788,$1792,$1796)|0);
     $1804 = $1797;
    } else {
     $1798 = HEAP32[$yymsp>>2]|0;
     $1799 = (($1798) + -32|0);
     $1800 = (($1799) + 4|0);
     $1801 = HEAP32[$1800>>2]|0;
     $1804 = $1801;
    }
    $1802 = HEAP32[$yygotominor>>2]|0;
    $1803 = (($1802) + 20|0);
    HEAP32[$1803>>2] = $1804;
    $1805 = HEAP32[$pParse>>2]|0;
    $1806 = HEAP32[$yygotominor>>2]|0;
    _sqlite3ExprSetHeight($1805,$1806);
   } else {
    $1807 = HEAP32[$pParse>>2]|0;
    $1808 = HEAP32[$1807>>2]|0;
    $1809 = HEAP32[$yymsp>>2]|0;
    $1810 = (($1809) + -32|0);
    $1811 = (($1810) + 4|0);
    $1812 = HEAP32[$1811>>2]|0;
    _sqlite3ExprListDelete($1808,$1812);
    $1813 = HEAP32[$pParse>>2]|0;
    $1814 = HEAP32[$1813>>2]|0;
    $1815 = HEAP32[$yymsp>>2]|0;
    $1816 = (($1815) + -16|0);
    $1817 = (($1816) + 4|0);
    $1818 = HEAP32[$1817>>2]|0;
    _sqlite3ExprDelete($1814,$1818);
   }
   $1819 = HEAP32[$yymsp>>2]|0;
   $1820 = (($1819) + -64|0);
   $1821 = (($1820) + 4|0);
   $1822 = HEAP32[$1821>>2]|0;
   $1823 = (($yygotominor) + 4|0);
   HEAP32[$1823>>2] = $1822;
   $1824 = HEAP32[$yymsp>>2]|0;
   $1825 = (($1824) + 4|0);
   $1826 = (($1825) + 4|0);
   $1827 = HEAP32[$1826>>2]|0;
   $1828 = HEAP32[$yymsp>>2]|0;
   $1829 = (($1828) + 4|0);
   $1830 = HEAP32[$1829>>2]|0;
   $1831 = (($1830) + ($1827)|0);
   $1832 = (($yygotominor) + 8|0);
   HEAP32[$1832>>2] = $1831;
   break;
  }
  case 238:  {
   $1870 = HEAP32[$pParse>>2]|0;
   $1871 = HEAP32[$yymsp>>2]|0;
   $1872 = (($1871) + 4|0);
   $1873 = HEAP32[$1872>>2]|0;
   $1874 = (_sqlite3ExprListAppend($1870,0,$1873)|0);
   HEAP32[$yygotominor>>2] = $1874;
   break;
  }
  case 229:  {
   $1833 = HEAP32[$pParse>>2]|0;
   $1834 = HEAP32[$yymsp>>2]|0;
   $1835 = (($1834) + -64|0);
   $1836 = (($1835) + 4|0);
   $1837 = HEAP32[$1836>>2]|0;
   $1838 = HEAP32[$yymsp>>2]|0;
   $1839 = (($1838) + -32|0);
   $1840 = (($1839) + 4|0);
   $1841 = HEAP32[$1840>>2]|0;
   $1842 = (_sqlite3ExprListAppend($1833,$1837,$1841)|0);
   HEAP32[$yygotominor>>2] = $1842;
   $1843 = HEAP32[$pParse>>2]|0;
   $1844 = HEAP32[$yygotominor>>2]|0;
   $1845 = HEAP32[$yymsp>>2]|0;
   $1846 = (($1845) + 4|0);
   $1847 = HEAP32[$1846>>2]|0;
   $1848 = (_sqlite3ExprListAppend($1843,$1844,$1847)|0);
   HEAP32[$yygotominor>>2] = $1848;
   break;
  }
  case 230:  {
   $1849 = HEAP32[$pParse>>2]|0;
   $1850 = HEAP32[$yymsp>>2]|0;
   $1851 = (($1850) + -32|0);
   $1852 = (($1851) + 4|0);
   $1853 = HEAP32[$1852>>2]|0;
   $1854 = (_sqlite3ExprListAppend($1849,0,$1853)|0);
   HEAP32[$yygotominor>>2] = $1854;
   $1855 = HEAP32[$pParse>>2]|0;
   $1856 = HEAP32[$yygotominor>>2]|0;
   $1857 = HEAP32[$yymsp>>2]|0;
   $1858 = (($1857) + 4|0);
   $1859 = HEAP32[$1858>>2]|0;
   $1860 = (_sqlite3ExprListAppend($1855,$1856,$1859)|0);
   HEAP32[$yygotominor>>2] = $1860;
   break;
  }
  case 237:  {
   $1861 = HEAP32[$pParse>>2]|0;
   $1862 = HEAP32[$yymsp>>2]|0;
   $1863 = (($1862) + -32|0);
   $1864 = (($1863) + 4|0);
   $1865 = HEAP32[$1864>>2]|0;
   $1866 = HEAP32[$yymsp>>2]|0;
   $1867 = (($1866) + 4|0);
   $1868 = HEAP32[$1867>>2]|0;
   $1869 = (_sqlite3ExprListAppend($1861,$1865,$1868)|0);
   HEAP32[$yygotominor>>2] = $1869;
   break;
  }
  case 248:  {
   $1969 = HEAP32[$pParse>>2]|0;
   $1970 = HEAP32[$yymsp>>2]|0;
   $1971 = (($1970) + 4|0);
   $1972 = HEAP32[$1971>>2]|0;
   $1973 = HEAP32[$yymsp>>2]|0;
   $1974 = (($1973) + -16|0);
   $1975 = (($1974) + 4|0);
   $1976 = HEAP32[$1975>>2]|0;
   _sqlite3DropIndex($1969,$1972,$1976);
   break;
  }
  case 250: case 249:  {
   $1977 = HEAP32[$pParse>>2]|0;
   _sqlite3Vacuum($1977);
   break;
  }
  case 251:  {
   $1978 = HEAP32[$pParse>>2]|0;
   $1979 = HEAP32[$yymsp>>2]|0;
   $1980 = (($1979) + -16|0);
   $1981 = (($1980) + 4|0);
   $1982 = HEAP32[$yymsp>>2]|0;
   $1983 = (($1982) + 4|0);
   _sqlite3Pragma($1978,$1981,$1983,0,0);
   break;
  }
  case 252:  {
   $1984 = HEAP32[$pParse>>2]|0;
   $1985 = HEAP32[$yymsp>>2]|0;
   $1986 = (($1985) + -48|0);
   $1987 = (($1986) + 4|0);
   $1988 = HEAP32[$yymsp>>2]|0;
   $1989 = (($1988) + -32|0);
   $1990 = (($1989) + 4|0);
   $1991 = HEAP32[$yymsp>>2]|0;
   $1992 = (($1991) + 4|0);
   _sqlite3Pragma($1984,$1987,$1990,$1992,0);
   break;
  }
  case 291: case 240:  {
   HEAP32[$yygotominor>>2] = 2;
   break;
  }
  case 241:  {
   HEAP32[$yygotominor>>2] = 0;
   break;
  }
  case 239:  {
   $1875 = HEAP32[$pParse>>2]|0;
   $1876 = HEAP32[$yymsp>>2]|0;
   $1877 = (($1876) + -112|0);
   $1878 = (($1877) + 4|0);
   $1879 = HEAP32[$yymsp>>2]|0;
   $1880 = (($1879) + -96|0);
   $1881 = (($1880) + 4|0);
   $1882 = HEAP32[$pParse>>2]|0;
   $1883 = HEAP32[$1882>>2]|0;
   $1884 = HEAP32[$yymsp>>2]|0;
   $1885 = (($1884) + -64|0);
   $1886 = (($1885) + 4|0);
   $1887 = (_sqlite3SrcListAppend($1883,0,$1886,0)|0);
   $1888 = HEAP32[$yymsp>>2]|0;
   $1889 = (($1888) + -32|0);
   $1890 = (($1889) + 4|0);
   $1891 = HEAP32[$1890>>2]|0;
   $1892 = HEAP32[$yymsp>>2]|0;
   $1893 = (($1892) + -160|0);
   $1894 = (($1893) + 4|0);
   $1895 = HEAP32[$1894>>2]|0;
   $1896 = HEAP32[$yymsp>>2]|0;
   $1897 = (($1896) + -176|0);
   $1898 = (($1897) + 4|0);
   $1899 = HEAP32[$yymsp>>2]|0;
   $1900 = (($1899) + 4|0);
   $1901 = HEAP32[$1900>>2]|0;
   $1902 = HEAP32[$yymsp>>2]|0;
   $1903 = (($1902) + -128|0);
   $1904 = (($1903) + 4|0);
   $1905 = HEAP32[$1904>>2]|0;
   (_sqlite3CreateIndex($1875,$1878,$1881,$1887,$1891,$1895,$1898,$1901,0,$1905)|0);
   break;
  }
  case 244:  {
   $1906 = HEAP32[$pParse>>2]|0;
   $1907 = HEAP32[$yymsp>>2]|0;
   $1908 = (($1907) + -16|0);
   $1909 = (($1908) + 4|0);
   $1910 = (_sqlite3ExprAddCollateToken($1906,0,$1909)|0);
   HEAP32[$p9>>2] = $1910;
   $1911 = HEAP32[$pParse>>2]|0;
   $1912 = HEAP32[$yymsp>>2]|0;
   $1913 = (($1912) + -64|0);
   $1914 = (($1913) + 4|0);
   $1915 = HEAP32[$1914>>2]|0;
   $1916 = HEAP32[$p9>>2]|0;
   $1917 = (_sqlite3ExprListAppend($1911,$1915,$1916)|0);
   HEAP32[$yygotominor>>2] = $1917;
   $1918 = HEAP32[$pParse>>2]|0;
   $1919 = HEAP32[$yygotominor>>2]|0;
   $1920 = HEAP32[$yymsp>>2]|0;
   $1921 = (($1920) + -32|0);
   $1922 = (($1921) + 4|0);
   _sqlite3ExprListSetName($1918,$1919,$1922,1);
   $1923 = HEAP32[$pParse>>2]|0;
   $1924 = HEAP32[$yygotominor>>2]|0;
   _sqlite3ExprListCheckLength($1923,$1924,10168);
   $1925 = HEAP32[$yygotominor>>2]|0;
   $1926 = ($1925|0)!=(0|0);
   if ($1926) {
    $1927 = HEAP32[$yymsp>>2]|0;
    $1928 = (($1927) + 4|0);
    $1929 = HEAP32[$1928>>2]|0;
    $1930 = $1929&255;
    $1931 = HEAP32[$yygotominor>>2]|0;
    $1932 = HEAP32[$1931>>2]|0;
    $1933 = (($1932) - 1)|0;
    $1934 = HEAP32[$yygotominor>>2]|0;
    $1935 = (($1934) + 4|0);
    $1936 = HEAP32[$1935>>2]|0;
    $1937 = (($1936) + (($1933*20)|0)|0);
    $1938 = (($1937) + 12|0);
    HEAP8[$1938>>0] = $1930;
   }
   break;
  }
  case 245:  {
   $1939 = HEAP32[$pParse>>2]|0;
   $1940 = HEAP32[$yymsp>>2]|0;
   $1941 = (($1940) + -16|0);
   $1942 = (($1941) + 4|0);
   $1943 = (_sqlite3ExprAddCollateToken($1939,0,$1942)|0);
   HEAP32[$p10>>2] = $1943;
   $1944 = HEAP32[$pParse>>2]|0;
   $1945 = HEAP32[$p10>>2]|0;
   $1946 = (_sqlite3ExprListAppend($1944,0,$1945)|0);
   HEAP32[$yygotominor>>2] = $1946;
   $1947 = HEAP32[$pParse>>2]|0;
   $1948 = HEAP32[$yygotominor>>2]|0;
   $1949 = HEAP32[$yymsp>>2]|0;
   $1950 = (($1949) + -32|0);
   $1951 = (($1950) + 4|0);
   _sqlite3ExprListSetName($1947,$1948,$1951,1);
   $1952 = HEAP32[$pParse>>2]|0;
   $1953 = HEAP32[$yygotominor>>2]|0;
   _sqlite3ExprListCheckLength($1952,$1953,10168);
   $1954 = HEAP32[$yygotominor>>2]|0;
   $1955 = ($1954|0)!=(0|0);
   if ($1955) {
    $1956 = HEAP32[$yymsp>>2]|0;
    $1957 = (($1956) + 4|0);
    $1958 = HEAP32[$1957>>2]|0;
    $1959 = $1958&255;
    $1960 = HEAP32[$yygotominor>>2]|0;
    $1961 = HEAP32[$1960>>2]|0;
    $1962 = (($1961) - 1)|0;
    $1963 = HEAP32[$yygotominor>>2]|0;
    $1964 = (($1963) + 4|0);
    $1965 = HEAP32[$1964>>2]|0;
    $1966 = (($1965) + (($1962*20)|0)|0);
    $1967 = (($1966) + 12|0);
    HEAP8[$1967>>0] = $1959;
   }
   break;
  }
  case 246:  {
   HEAP32[$yygotominor>>2] = 0;
   $1968 = (($yygotominor) + 4|0);
   HEAP32[$1968>>2] = 0;
   break;
  }
  case 271: case 270:  {
   $2093 = HEAP32[$yymsp>>2]|0;
   $2094 = (($2093) + 2|0);
   $2095 = HEAP8[$2094>>0]|0;
   $2096 = $2095&255;
   HEAP32[$yygotominor>>2] = $2096;
   $2097 = (($yygotominor) + 4|0);
   HEAP32[$2097>>2] = 0;
   break;
  }
  case 296: case 275:  {
   HEAP32[$yygotominor>>2] = 0;
   break;
  }
  case 272:  {
   HEAP32[$yygotominor>>2] = 110;
   $2098 = HEAP32[$yymsp>>2]|0;
   $2099 = (($2098) + 4|0);
   $2100 = HEAP32[$2099>>2]|0;
   $2101 = (($yygotominor) + 4|0);
   HEAP32[$2101>>2] = $2100;
   break;
  }
  case 297: case 276:  {
   $2102 = HEAP32[$yymsp>>2]|0;
   $2103 = (($2102) + 4|0);
   $2104 = HEAP32[$2103>>2]|0;
   HEAP32[$yygotominor>>2] = $2104;
   break;
  }
  case 253:  {
   $1993 = HEAP32[$pParse>>2]|0;
   $1994 = HEAP32[$yymsp>>2]|0;
   $1995 = (($1994) + -64|0);
   $1996 = (($1995) + 4|0);
   $1997 = HEAP32[$yymsp>>2]|0;
   $1998 = (($1997) + -48|0);
   $1999 = (($1998) + 4|0);
   $2000 = HEAP32[$yymsp>>2]|0;
   $2001 = (($2000) + -16|0);
   $2002 = (($2001) + 4|0);
   _sqlite3Pragma($1993,$1996,$1999,$2002,0);
   break;
  }
  case 254:  {
   $2003 = HEAP32[$pParse>>2]|0;
   $2004 = HEAP32[$yymsp>>2]|0;
   $2005 = (($2004) + -48|0);
   $2006 = (($2005) + 4|0);
   $2007 = HEAP32[$yymsp>>2]|0;
   $2008 = (($2007) + -32|0);
   $2009 = (($2008) + 4|0);
   $2010 = HEAP32[$yymsp>>2]|0;
   $2011 = (($2010) + 4|0);
   _sqlite3Pragma($2003,$2006,$2009,$2011,1);
   break;
  }
  case 255:  {
   $2012 = HEAP32[$pParse>>2]|0;
   $2013 = HEAP32[$yymsp>>2]|0;
   $2014 = (($2013) + -64|0);
   $2015 = (($2014) + 4|0);
   $2016 = HEAP32[$yymsp>>2]|0;
   $2017 = (($2016) + -48|0);
   $2018 = (($2017) + 4|0);
   $2019 = HEAP32[$yymsp>>2]|0;
   $2020 = (($2019) + -16|0);
   $2021 = (($2020) + 4|0);
   _sqlite3Pragma($2012,$2015,$2018,$2021,1);
   break;
  }
  case 264:  {
   $2022 = HEAP32[$yymsp>>2]|0;
   $2023 = (($2022) + -48|0);
   $2024 = (($2023) + 4|0);
   $2025 = HEAP32[$2024>>2]|0;
   HEAP32[$all>>2] = $2025;
   $2026 = HEAP32[$yymsp>>2]|0;
   $2027 = (($2026) + 4|0);
   $2028 = HEAP32[$2027>>2]|0;
   $2029 = HEAP32[$yymsp>>2]|0;
   $2030 = (($2029) + -48|0);
   $2031 = (($2030) + 4|0);
   $2032 = HEAP32[$2031>>2]|0;
   $2033 = $2028;
   $2034 = $2032;
   $2035 = (($2033) - ($2034))|0;
   $2036 = HEAP32[$yymsp>>2]|0;
   $2037 = (($2036) + 4|0);
   $2038 = (($2037) + 4|0);
   $2039 = HEAP32[$2038>>2]|0;
   $2040 = (($2035) + ($2039))|0;
   $2041 = (($all) + 4|0);
   HEAP32[$2041>>2] = $2040;
   $2042 = HEAP32[$pParse>>2]|0;
   $2043 = HEAP32[$yymsp>>2]|0;
   $2044 = (($2043) + -16|0);
   $2045 = (($2044) + 4|0);
   $2046 = HEAP32[$2045>>2]|0;
   _sqlite3FinishTrigger($2042,$2046,$all);
   break;
  }
  case 265:  {
   $2047 = HEAP32[$pParse>>2]|0;
   $2048 = HEAP32[$yymsp>>2]|0;
   $2049 = (($2048) + -112|0);
   $2050 = (($2049) + 4|0);
   $2051 = HEAP32[$yymsp>>2]|0;
   $2052 = (($2051) + -96|0);
   $2053 = (($2052) + 4|0);
   $2054 = HEAP32[$yymsp>>2]|0;
   $2055 = (($2054) + -80|0);
   $2056 = (($2055) + 4|0);
   $2057 = HEAP32[$2056>>2]|0;
   $2058 = HEAP32[$yymsp>>2]|0;
   $2059 = (($2058) + -64|0);
   $2060 = (($2059) + 4|0);
   $2061 = HEAP32[$2060>>2]|0;
   $2062 = HEAP32[$yymsp>>2]|0;
   $2063 = (($2062) + -64|0);
   $2064 = (($2063) + 4|0);
   $2065 = (($2064) + 4|0);
   $2066 = HEAP32[$2065>>2]|0;
   $2067 = HEAP32[$yymsp>>2]|0;
   $2068 = (($2067) + -32|0);
   $2069 = (($2068) + 4|0);
   $2070 = HEAP32[$2069>>2]|0;
   $2071 = HEAP32[$yymsp>>2]|0;
   $2072 = (($2071) + 4|0);
   $2073 = HEAP32[$2072>>2]|0;
   $2074 = HEAP32[$yymsp>>2]|0;
   $2075 = (($2074) + -160|0);
   $2076 = (($2075) + 4|0);
   $2077 = HEAP32[$2076>>2]|0;
   $2078 = HEAP32[$yymsp>>2]|0;
   $2079 = (($2078) + -128|0);
   $2080 = (($2079) + 4|0);
   $2081 = HEAP32[$2080>>2]|0;
   _sqlite3BeginTrigger($2047,$2050,$2053,$2057,$2061,$2066,$2070,$2073,$2077,$2081);
   $2082 = HEAP32[$yymsp>>2]|0;
   $2083 = (($2082) + -96|0);
   $2084 = (($2083) + 4|0);
   $2085 = (($2084) + 4|0);
   $2086 = HEAP32[$2085>>2]|0;
   $2087 = ($2086|0)==(0);
   $2088 = HEAP32[$yymsp>>2]|0;
   if ($2087) {
    $2089 = (($2088) + -112|0);
    $2090 = (($2089) + 4|0);
    ;HEAP32[$yygotominor+0>>2]=HEAP32[$2090+0>>2]|0;HEAP32[$yygotominor+4>>2]=HEAP32[$2090+4>>2]|0;
    break L1;
   } else {
    $2091 = (($2088) + -96|0);
    $2092 = (($2091) + 4|0);
    ;HEAP32[$yygotominor+0>>2]=HEAP32[$2092+0>>2]|0;HEAP32[$yygotominor+4>>2]=HEAP32[$2092+4>>2]|0;
    break L1;
   }
   break;
  }
  case 269: case 266:  {
   HEAP32[$yygotominor>>2] = 35;
   break;
  }
  case 267:  {
   HEAP32[$yygotominor>>2] = 31;
   break;
  }
  case 268:  {
   HEAP32[$yygotominor>>2] = 49;
   break;
  }
  case 283:  {
   $2146 = HEAP32[$pParse>>2]|0;
   _sqlite3ErrorMsg($2146,10360,$vararg_buffer14);
   break;
  }
  case 284:  {
   $2147 = HEAP32[$pParse>>2]|0;
   $2148 = HEAP32[$2147>>2]|0;
   $2149 = HEAP32[$yymsp>>2]|0;
   $2150 = (($2149) + -64|0);
   $2151 = (($2150) + 4|0);
   $2152 = HEAP32[$yymsp>>2]|0;
   $2153 = (($2152) + -16|0);
   $2154 = (($2153) + 4|0);
   $2155 = HEAP32[$2154>>2]|0;
   $2156 = HEAP32[$yymsp>>2]|0;
   $2157 = (($2156) + 4|0);
   $2158 = HEAP32[$2157>>2]|0;
   $2159 = HEAP32[$yymsp>>2]|0;
   $2160 = (($2159) + -80|0);
   $2161 = (($2160) + 4|0);
   $2162 = HEAP8[$2161>>0]|0;
   $2163 = (_sqlite3TriggerUpdateStep($2148,$2151,$2155,$2158,$2162)|0);
   HEAP32[$yygotominor>>2] = $2163;
   break;
  }
  case 277:  {
   $2105 = HEAP32[$yymsp>>2]|0;
   $2106 = (($2105) + -16|0);
   $2107 = (($2106) + 4|0);
   $2108 = HEAP32[$2107>>2]|0;
   $2109 = HEAP32[$yymsp>>2]|0;
   $2110 = (($2109) + -32|0);
   $2111 = (($2110) + 4|0);
   $2112 = HEAP32[$2111>>2]|0;
   $2113 = (($2112) + 36|0);
   $2114 = HEAP32[$2113>>2]|0;
   $2115 = (($2114) + 32|0);
   HEAP32[$2115>>2] = $2108;
   $2116 = HEAP32[$yymsp>>2]|0;
   $2117 = (($2116) + -16|0);
   $2118 = (($2117) + 4|0);
   $2119 = HEAP32[$2118>>2]|0;
   $2120 = HEAP32[$yymsp>>2]|0;
   $2121 = (($2120) + -32|0);
   $2122 = (($2121) + 4|0);
   $2123 = HEAP32[$2122>>2]|0;
   $2124 = (($2123) + 36|0);
   HEAP32[$2124>>2] = $2119;
   $2125 = HEAP32[$yymsp>>2]|0;
   $2126 = (($2125) + -32|0);
   $2127 = (($2126) + 4|0);
   $2128 = HEAP32[$2127>>2]|0;
   HEAP32[$yygotominor>>2] = $2128;
   break;
  }
  case 278:  {
   $2129 = HEAP32[$yymsp>>2]|0;
   $2130 = (($2129) + -16|0);
   $2131 = (($2130) + 4|0);
   $2132 = HEAP32[$2131>>2]|0;
   $2133 = HEAP32[$yymsp>>2]|0;
   $2134 = (($2133) + -16|0);
   $2135 = (($2134) + 4|0);
   $2136 = HEAP32[$2135>>2]|0;
   $2137 = (($2136) + 36|0);
   HEAP32[$2137>>2] = $2132;
   $2138 = HEAP32[$yymsp>>2]|0;
   $2139 = (($2138) + -16|0);
   $2140 = (($2139) + 4|0);
   $2141 = HEAP32[$2140>>2]|0;
   HEAP32[$yygotominor>>2] = $2141;
   break;
  }
  case 282:  {
   $2145 = HEAP32[$pParse>>2]|0;
   _sqlite3ErrorMsg($2145,10272,$vararg_buffer14);
   break;
  }
  case 280:  {
   $2142 = HEAP32[$yymsp>>2]|0;
   $2143 = (($2142) + 4|0);
   ;HEAP32[$yygotominor+0>>2]=HEAP32[$2143+0>>2]|0;HEAP32[$yygotominor+4>>2]=HEAP32[$2143+4>>2]|0;
   $2144 = HEAP32[$pParse>>2]|0;
   _sqlite3ErrorMsg($2144,10176,$vararg_buffer14);
   break;
  }
  case 287:  {
   $2190 = HEAP32[$pParse>>2]|0;
   $2191 = HEAP32[$2190>>2]|0;
   $2192 = HEAP32[$yymsp>>2]|0;
   $2193 = (($2192) + 4|0);
   $2194 = HEAP32[$2193>>2]|0;
   $2195 = (_sqlite3TriggerSelectStep($2191,$2194)|0);
   HEAP32[$yygotominor>>2] = $2195;
   break;
  }
  case 288:  {
   $2196 = HEAP32[$pParse>>2]|0;
   $2197 = (_sqlite3PExpr($2196,57,0,0,0)|0);
   HEAP32[$yygotominor>>2] = $2197;
   $2198 = HEAP32[$yygotominor>>2]|0;
   $2199 = ($2198|0)!=(0|0);
   if ($2199) {
    $2200 = HEAP32[$yygotominor>>2]|0;
    $2201 = (($2200) + 1|0);
    HEAP8[$2201>>0] = 4;
   }
   $2202 = HEAP32[$yymsp>>2]|0;
   $2203 = (($2202) + -48|0);
   $2204 = (($2203) + 4|0);
   $2205 = HEAP32[$2204>>2]|0;
   $2206 = (($yygotominor) + 4|0);
   HEAP32[$2206>>2] = $2205;
   $2207 = HEAP32[$yymsp>>2]|0;
   $2208 = (($2207) + 4|0);
   $2209 = (($2208) + 4|0);
   $2210 = HEAP32[$2209>>2]|0;
   $2211 = HEAP32[$yymsp>>2]|0;
   $2212 = (($2211) + 4|0);
   $2213 = HEAP32[$2212>>2]|0;
   $2214 = (($2213) + ($2210)|0);
   $2215 = (($yygotominor) + 8|0);
   HEAP32[$2215>>2] = $2214;
   break;
  }
  case 285:  {
   $2164 = HEAP32[$pParse>>2]|0;
   $2165 = HEAP32[$2164>>2]|0;
   $2166 = HEAP32[$yymsp>>2]|0;
   $2167 = (($2166) + -32|0);
   $2168 = (($2167) + 4|0);
   $2169 = HEAP32[$yymsp>>2]|0;
   $2170 = (($2169) + -16|0);
   $2171 = (($2170) + 4|0);
   $2172 = HEAP32[$2171>>2]|0;
   $2173 = HEAP32[$yymsp>>2]|0;
   $2174 = (($2173) + 4|0);
   $2175 = HEAP32[$2174>>2]|0;
   $2176 = HEAP32[$yymsp>>2]|0;
   $2177 = (($2176) + -64|0);
   $2178 = (($2177) + 4|0);
   $2179 = HEAP8[$2178>>0]|0;
   $2180 = (_sqlite3TriggerInsertStep($2165,$2168,$2172,$2175,$2179)|0);
   HEAP32[$yygotominor>>2] = $2180;
   break;
  }
  case 286:  {
   $2181 = HEAP32[$pParse>>2]|0;
   $2182 = HEAP32[$2181>>2]|0;
   $2183 = HEAP32[$yymsp>>2]|0;
   $2184 = (($2183) + -32|0);
   $2185 = (($2184) + 4|0);
   $2186 = HEAP32[$yymsp>>2]|0;
   $2187 = (($2186) + 4|0);
   $2188 = HEAP32[$2187>>2]|0;
   $2189 = (_sqlite3TriggerDeleteStep($2182,$2185,$2188)|0);
   HEAP32[$yygotominor>>2] = $2189;
   break;
  }
  case 293:  {
   $2244 = HEAP32[$pParse>>2]|0;
   $2245 = HEAP32[$yymsp>>2]|0;
   $2246 = (($2245) + 4|0);
   $2247 = HEAP32[$2246>>2]|0;
   $2248 = HEAP32[$yymsp>>2]|0;
   $2249 = (($2248) + -16|0);
   $2250 = (($2249) + 4|0);
   $2251 = HEAP32[$2250>>2]|0;
   _sqlite3DropTrigger($2244,$2247,$2251);
   break;
  }
  case 294:  {
   $2252 = HEAP32[$pParse>>2]|0;
   $2253 = HEAP32[$yymsp>>2]|0;
   $2254 = (($2253) + -48|0);
   $2255 = (($2254) + 4|0);
   $2256 = HEAP32[$2255>>2]|0;
   $2257 = HEAP32[$yymsp>>2]|0;
   $2258 = (($2257) + -16|0);
   $2259 = (($2258) + 4|0);
   $2260 = HEAP32[$2259>>2]|0;
   $2261 = HEAP32[$yymsp>>2]|0;
   $2262 = (($2261) + 4|0);
   $2263 = HEAP32[$2262>>2]|0;
   _sqlite3Attach($2252,$2256,$2260,$2263);
   break;
  }
  case 295:  {
   $2264 = HEAP32[$pParse>>2]|0;
   $2265 = HEAP32[$yymsp>>2]|0;
   $2266 = (($2265) + 4|0);
   $2267 = HEAP32[$2266>>2]|0;
   _sqlite3Detach($2264,$2267);
   break;
  }
  case 300:  {
   $2268 = HEAP32[$pParse>>2]|0;
   _sqlite3Reindex($2268,0,0);
   break;
  }
  case 289:  {
   $2216 = HEAP32[$pParse>>2]|0;
   $2217 = HEAP32[$yymsp>>2]|0;
   $2218 = (($2217) + -16|0);
   $2219 = (($2218) + 4|0);
   $2220 = (_sqlite3PExpr($2216,57,0,0,$2219)|0);
   HEAP32[$yygotominor>>2] = $2220;
   $2221 = HEAP32[$yygotominor>>2]|0;
   $2222 = ($2221|0)!=(0|0);
   if ($2222) {
    $2223 = HEAP32[$yymsp>>2]|0;
    $2224 = (($2223) + -48|0);
    $2225 = (($2224) + 4|0);
    $2226 = HEAP32[$2225>>2]|0;
    $2227 = $2226&255;
    $2228 = HEAP32[$yygotominor>>2]|0;
    $2229 = (($2228) + 1|0);
    HEAP8[$2229>>0] = $2227;
   }
   $2230 = HEAP32[$yymsp>>2]|0;
   $2231 = (($2230) + -80|0);
   $2232 = (($2231) + 4|0);
   $2233 = HEAP32[$2232>>2]|0;
   $2234 = (($yygotominor) + 4|0);
   HEAP32[$2234>>2] = $2233;
   $2235 = HEAP32[$yymsp>>2]|0;
   $2236 = (($2235) + 4|0);
   $2237 = (($2236) + 4|0);
   $2238 = HEAP32[$2237>>2]|0;
   $2239 = HEAP32[$yymsp>>2]|0;
   $2240 = (($2239) + 4|0);
   $2241 = HEAP32[$2240>>2]|0;
   $2242 = (($2241) + ($2238)|0);
   $2243 = (($yygotominor) + 8|0);
   HEAP32[$2243>>2] = $2242;
   break;
  }
  case 290:  {
   HEAP32[$yygotominor>>2] = 1;
   break;
  }
  case 292:  {
   HEAP32[$yygotominor>>2] = 3;
   break;
  }
  case 303:  {
   $2276 = HEAP32[$pParse>>2]|0;
   $2277 = HEAP32[$yymsp>>2]|0;
   $2278 = (($2277) + -16|0);
   $2279 = (($2278) + 4|0);
   $2280 = HEAP32[$yymsp>>2]|0;
   $2281 = (($2280) + 4|0);
   _sqlite3Analyze($2276,$2279,$2281);
   break;
  }
  case 304:  {
   $2282 = HEAP32[$pParse>>2]|0;
   $2283 = HEAP32[$yymsp>>2]|0;
   $2284 = (($2283) + -48|0);
   $2285 = (($2284) + 4|0);
   $2286 = HEAP32[$2285>>2]|0;
   $2287 = HEAP32[$yymsp>>2]|0;
   $2288 = (($2287) + 4|0);
   _sqlite3AlterRenameTable($2282,$2286,$2288);
   break;
  }
  case 305:  {
   $2289 = HEAP32[$pParse>>2]|0;
   $2290 = HEAP32[$yymsp>>2]|0;
   $2291 = (($2290) + 4|0);
   _sqlite3AlterFinishAddColumn($2289,$2291);
   break;
  }
  case 306:  {
   $2292 = HEAP32[$pParse>>2]|0;
   $2293 = HEAP32[$2292>>2]|0;
   $2294 = (($2293) + 248|0);
   $2295 = (($2294) + 2|0);
   HEAP8[$2295>>0] = 0;
   $2296 = HEAP32[$pParse>>2]|0;
   $2297 = HEAP32[$yymsp>>2]|0;
   $2298 = (($2297) + 4|0);
   $2299 = HEAP32[$2298>>2]|0;
   _sqlite3AlterBeginAddColumn($2296,$2299);
   break;
  }
  case 301:  {
   $2269 = HEAP32[$pParse>>2]|0;
   $2270 = HEAP32[$yymsp>>2]|0;
   $2271 = (($2270) + -16|0);
   $2272 = (($2271) + 4|0);
   $2273 = HEAP32[$yymsp>>2]|0;
   $2274 = (($2273) + 4|0);
   _sqlite3Reindex($2269,$2272,$2274);
   break;
  }
  case 302:  {
   $2275 = HEAP32[$pParse>>2]|0;
   _sqlite3Analyze($2275,0,0);
   break;
  }
  case 310:  {
   $2301 = HEAP32[$pParse>>2]|0;
   $2302 = HEAP32[$yymsp>>2]|0;
   $2303 = (($2302) + 4|0);
   _sqlite3VtabFinishParse($2301,$2303);
   break;
  }
  case 311:  {
   $2304 = HEAP32[$pParse>>2]|0;
   $2305 = HEAP32[$yymsp>>2]|0;
   $2306 = (($2305) + -48|0);
   $2307 = (($2306) + 4|0);
   $2308 = HEAP32[$yymsp>>2]|0;
   $2309 = (($2308) + -32|0);
   $2310 = (($2309) + 4|0);
   $2311 = HEAP32[$yymsp>>2]|0;
   $2312 = (($2311) + 4|0);
   $2313 = HEAP32[$yymsp>>2]|0;
   $2314 = (($2313) + -64|0);
   $2315 = (($2314) + 4|0);
   $2316 = HEAP32[$2315>>2]|0;
   _sqlite3VtabBeginParse($2304,$2307,$2310,$2312,$2316);
   break;
  }
  case 314:  {
   $2317 = HEAP32[$pParse>>2]|0;
   _sqlite3VtabArgInit($2317);
   break;
  }
  case 318: case 317: case 316:  {
   $2318 = HEAP32[$pParse>>2]|0;
   $2319 = HEAP32[$yymsp>>2]|0;
   $2320 = (($2319) + 4|0);
   _sqlite3VtabArgExtend($2318,$2320);
   break;
  }
  case 322:  {
   HEAP32[$yygotominor>>2] = 0;
   break;
  }
  case 309:  {
   $2300 = HEAP32[$pParse>>2]|0;
   _sqlite3VtabFinishParse($2300,0);
   break;
  }
  case 324: case 323:  {
   $2321 = HEAP32[$yymsp>>2]|0;
   $2322 = (($2321) + 4|0);
   $2323 = HEAP32[$2322>>2]|0;
   HEAP32[$yygotominor>>2] = $2323;
   break;
  }
  case 325:  {
   $2324 = HEAP32[$pParse>>2]|0;
   $2325 = HEAP32[$yymsp>>2]|0;
   $2326 = (($2325) + -80|0);
   $2327 = (($2326) + 4|0);
   $2328 = HEAP32[$yymsp>>2]|0;
   $2329 = (($2328) + -64|0);
   $2330 = (($2329) + 4|0);
   $2331 = HEAP32[$2330>>2]|0;
   $2332 = HEAP32[$yymsp>>2]|0;
   $2333 = (($2332) + -16|0);
   $2334 = (($2333) + 4|0);
   $2335 = HEAP32[$2334>>2]|0;
   $2336 = (_sqlite3WithAdd($2324,0,$2327,$2331,$2335)|0);
   HEAP32[$yygotominor>>2] = $2336;
   break;
  }
  case 326:  {
   $2337 = HEAP32[$pParse>>2]|0;
   $2338 = HEAP32[$yymsp>>2]|0;
   $2339 = (($2338) + -112|0);
   $2340 = (($2339) + 4|0);
   $2341 = HEAP32[$2340>>2]|0;
   $2342 = HEAP32[$yymsp>>2]|0;
   $2343 = (($2342) + -80|0);
   $2344 = (($2343) + 4|0);
   $2345 = HEAP32[$yymsp>>2]|0;
   $2346 = (($2345) + -64|0);
   $2347 = (($2346) + 4|0);
   $2348 = HEAP32[$2347>>2]|0;
   $2349 = HEAP32[$yymsp>>2]|0;
   $2350 = (($2349) + -16|0);
   $2351 = (($2350) + 4|0);
   $2352 = HEAP32[$2351>>2]|0;
   $2353 = (_sqlite3WithAdd($2337,$2341,$2344,$2348,$2352)|0);
   HEAP32[$yygotominor>>2] = $2353;
   break;
  }
  default: {
  }
  }
 } while(0);
 $2354 = HEAP32[$1>>2]|0;
 $2355 = (10448 + ($2354<<1)|0);
 $2356 = HEAP8[$2355>>0]|0;
 $2357 = $2356&255;
 HEAP32[$yygoto>>2] = $2357;
 $2358 = HEAP32[$1>>2]|0;
 $2359 = (10448 + ($2358<<1)|0);
 $2360 = (($2359) + 1|0);
 $2361 = HEAP8[$2360>>0]|0;
 $2362 = $2361&255;
 HEAP32[$yysize>>2] = $2362;
 $2363 = HEAP32[$yysize>>2]|0;
 $2364 = HEAP32[$0>>2]|0;
 $2365 = HEAP32[$2364>>2]|0;
 $2366 = (($2365) - ($2363))|0;
 HEAP32[$2364>>2] = $2366;
 $2367 = HEAP32[$yysize>>2]|0;
 $2368 = (0 - ($2367))|0;
 $2369 = HEAP32[$yymsp>>2]|0;
 $2370 = (($2369) + ($2368<<4)|0);
 $2371 = HEAP16[$2370>>1]|0;
 $2372 = $2371&65535;
 $2373 = HEAP32[$yygoto>>2]|0;
 $2374 = $2373&255;
 $2375 = (_yy_find_reduce_action($2372,$2374)|0);
 HEAP32[$yyact>>2] = $2375;
 $2376 = HEAP32[$yyact>>2]|0;
 $2377 = ($2376|0)<(642);
 if (!($2377)) {
  $2399 = HEAP32[$0>>2]|0;
  _yy_accept($2399);
  STACKTOP = sp;return;
 }
 $2378 = HEAP32[$yysize>>2]|0;
 $2379 = ($2378|0)!=(0);
 $2380 = HEAP32[$0>>2]|0;
 if ($2379) {
  $2381 = HEAP32[$2380>>2]|0;
  $2382 = (($2381) + 1)|0;
  HEAP32[$2380>>2] = $2382;
  $2383 = HEAP32[$yysize>>2]|0;
  $2384 = (($2383) - 1)|0;
  $2385 = HEAP32[$yymsp>>2]|0;
  $2386 = (0 - ($2384))|0;
  $2387 = (($2385) + ($2386<<4)|0);
  HEAP32[$yymsp>>2] = $2387;
  $2388 = HEAP32[$yyact>>2]|0;
  $2389 = $2388&65535;
  $2390 = HEAP32[$yymsp>>2]|0;
  HEAP16[$2390>>1] = $2389;
  $2391 = HEAP32[$yygoto>>2]|0;
  $2392 = $2391&255;
  $2393 = HEAP32[$yymsp>>2]|0;
  $2394 = (($2393) + 2|0);
  HEAP8[$2394>>0] = $2392;
  $2395 = HEAP32[$yymsp>>2]|0;
  $2396 = (($2395) + 4|0);
  ;HEAP32[$2396+0>>2]=HEAP32[$yygotominor+0>>2]|0;HEAP32[$2396+4>>2]=HEAP32[$yygotominor+4>>2]|0;HEAP32[$2396+8>>2]=HEAP32[$yygotominor+8>>2]|0;
  STACKTOP = sp;return;
 } else {
  $2397 = HEAP32[$yyact>>2]|0;
  $2398 = HEAP32[$yygoto>>2]|0;
  _yy_shift($2380,$2397,$2398,$yygotominor);
  STACKTOP = sp;return;
 }
}
function _yy_syntax_error($yypParser,$yymajor,$yyminor) {
 $yypParser = $yypParser|0;
 $yymajor = $yymajor|0;
 $yyminor = $yyminor|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $pParse = 0, $vararg_buffer = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $vararg_buffer = sp;
 $0 = sp + 12|0;
 $1 = sp + 8|0;
 $pParse = sp + 4|0;
 HEAP32[$0>>2] = $yypParser;
 HEAP32[$1>>2] = $yymajor;
 $2 = HEAP32[$0>>2]|0;
 $3 = (($2) + 8|0);
 $4 = HEAP32[$3>>2]|0;
 HEAP32[$pParse>>2] = $4;
 $5 = HEAP32[$pParse>>2]|0;
 HEAP32[$vararg_buffer>>2] = $yyminor;
 _sqlite3ErrorMsg($5,9968,$vararg_buffer);
 $6 = HEAP32[$pParse>>2]|0;
 $7 = HEAP32[$0>>2]|0;
 $8 = (($7) + 8|0);
 HEAP32[$8>>2] = $6;
 STACKTOP = sp;return;
}
function _sqlite3BeginParse($pParse,$explainFlag) {
 $pParse = $pParse|0;
 $explainFlag = $explainFlag|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $1 = sp;
 HEAP32[$0>>2] = $pParse;
 HEAP32[$1>>2] = $explainFlag;
 $2 = HEAP32[$1>>2]|0;
 $3 = $2&255;
 $4 = HEAP32[$0>>2]|0;
 $5 = (($4) + 450|0);
 HEAP8[$5>>0] = $3;
 $6 = HEAP32[$0>>2]|0;
 $7 = (($6) + 440|0);
 HEAP32[$7>>2] = 0;
 STACKTOP = sp;return;
}
function _sqlite3FinishCoding($pParse) {
 $pParse = $pParse|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0;
 var $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0;
 var $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0;
 var $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0;
 var $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0;
 var $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $db = 0, $i = 0, $iDb = 0, $pEL = 0, $v = 0;
 var $vtab = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 24|0;
 $db = sp + 20|0;
 $v = sp + 16|0;
 $iDb = sp + 12|0;
 $i = sp + 8|0;
 $vtab = sp + 4|0;
 $pEL = sp;
 HEAP32[$0>>2] = $pParse;
 $1 = HEAP32[$0>>2]|0;
 $2 = HEAP32[$1>>2]|0;
 HEAP32[$db>>2] = $2;
 $3 = HEAP32[$db>>2]|0;
 $4 = (($3) + 64|0);
 $5 = HEAP8[$4>>0]|0;
 $6 = ($5<<24>>24)!=(0);
 if ($6) {
  STACKTOP = sp;return;
 }
 $7 = HEAP32[$0>>2]|0;
 $8 = (($7) + 18|0);
 $9 = HEAP8[$8>>0]|0;
 $10 = ($9<<24>>24)!=(0);
 if ($10) {
  STACKTOP = sp;return;
 }
 $11 = HEAP32[$0>>2]|0;
 $12 = (($11) + 64|0);
 $13 = HEAP32[$12>>2]|0;
 $14 = ($13|0)!=(0);
 if ($14) {
  STACKTOP = sp;return;
 }
 $15 = HEAP32[$0>>2]|0;
 $16 = (_sqlite3GetVdbe($15)|0);
 HEAP32[$v>>2] = $16;
 $17 = HEAP32[$v>>2]|0;
 $18 = ($17|0)!=(0|0);
 do {
  if ($18) {
   while(1) {
    $19 = HEAP32[$v>>2]|0;
    $20 = (_sqlite3VdbeDeletePriorOpcode($19,61)|0);
    $21 = ($20|0)!=(0);
    if (!($21)) {
     break;
    }
   }
   $22 = HEAP32[$v>>2]|0;
   (_sqlite3VdbeAddOp0($22,24)|0);
   $23 = HEAP32[$db>>2]|0;
   $24 = (($23) + 64|0);
   $25 = HEAP8[$24>>0]|0;
   $26 = $25&255;
   $27 = ($26|0)==(0);
   if ($27) {
    $28 = HEAP32[$0>>2]|0;
    $29 = (($28) + 332|0);
    $30 = HEAP32[$29>>2]|0;
    $31 = ($30|0)!=(0);
    if (!($31)) {
     $32 = HEAP32[$0>>2]|0;
     $33 = (($32) + 316|0);
     $34 = HEAP32[$33>>2]|0;
     $35 = ($34|0)!=(0|0);
     if (!($35)) {
      break;
     }
    }
    $36 = HEAP32[$v>>2]|0;
    _sqlite3VdbeJumpHere($36,0);
    HEAP32[$iDb>>2] = 0;
    while(1) {
     $37 = HEAP32[$iDb>>2]|0;
     $38 = HEAP32[$db>>2]|0;
     $39 = (($38) + 20|0);
     $40 = HEAP32[$39>>2]|0;
     $41 = ($37|0)<($40|0);
     if (!($41)) {
      break;
     }
     $42 = HEAP32[$0>>2]|0;
     $43 = (($42) + 332|0);
     $44 = HEAP32[$43>>2]|0;
     $45 = HEAP32[$iDb>>2]|0;
     $46 = 1 << $45;
     $47 = $44 & $46;
     $48 = ($47|0)!=(0);
     $49 = $48&1;
     $50 = ($49|0)==(0);
     if (!($50)) {
      $51 = HEAP32[$v>>2]|0;
      $52 = HEAP32[$iDb>>2]|0;
      _sqlite3VdbeUsesBtree($51,$52);
      $53 = HEAP32[$v>>2]|0;
      $54 = HEAP32[$iDb>>2]|0;
      $55 = HEAP32[$0>>2]|0;
      $56 = (($55) + 328|0);
      $57 = HEAP32[$56>>2]|0;
      $58 = HEAP32[$iDb>>2]|0;
      $59 = 1 << $58;
      $60 = $57 & $59;
      $61 = ($60|0)!=(0);
      $62 = $61&1;
      $63 = HEAP32[$iDb>>2]|0;
      $64 = HEAP32[$0>>2]|0;
      $65 = (($64) + 336|0);
      $66 = (($65) + ($63<<2)|0);
      $67 = HEAP32[$66>>2]|0;
      $68 = HEAP32[$iDb>>2]|0;
      $69 = HEAP32[$db>>2]|0;
      $70 = (($69) + 16|0);
      $71 = HEAP32[$70>>2]|0;
      $72 = (($71) + ($68<<4)|0);
      $73 = (($72) + 12|0);
      $74 = HEAP32[$73>>2]|0;
      $75 = (($74) + 4|0);
      $76 = HEAP32[$75>>2]|0;
      (_sqlite3VdbeAddOp4Int($53,4,$54,$62,$67,$76)|0);
      $77 = HEAP32[$db>>2]|0;
      $78 = (($77) + 140|0);
      $79 = (($78) + 5|0);
      $80 = HEAP8[$79>>0]|0;
      $81 = $80&255;
      $82 = ($81|0)==(0);
      if ($82) {
       $83 = HEAP32[$v>>2]|0;
       _sqlite3VdbeChangeP5($83,1);
      }
     }
     $84 = HEAP32[$iDb>>2]|0;
     $85 = (($84) + 1)|0;
     HEAP32[$iDb>>2] = $85;
    }
    HEAP32[$i>>2] = 0;
    while(1) {
     $86 = HEAP32[$i>>2]|0;
     $87 = HEAP32[$0>>2]|0;
     $88 = (($87) + 452|0);
     $89 = HEAP32[$88>>2]|0;
     $90 = ($86|0)<($89|0);
     if (!($90)) {
      break;
     }
     $91 = HEAP32[$db>>2]|0;
     $92 = HEAP32[$i>>2]|0;
     $93 = HEAP32[$0>>2]|0;
     $94 = (($93) + 520|0);
     $95 = HEAP32[$94>>2]|0;
     $96 = (($95) + ($92<<2)|0);
     $97 = HEAP32[$96>>2]|0;
     $98 = (_sqlite3GetVTable($91,$97)|0);
     HEAP32[$vtab>>2] = $98;
     $99 = HEAP32[$v>>2]|0;
     $100 = HEAP32[$vtab>>2]|0;
     (_sqlite3VdbeAddOp4($99,144,0,0,0,$100,-10)|0);
     $101 = HEAP32[$i>>2]|0;
     $102 = (($101) + 1)|0;
     HEAP32[$i>>2] = $102;
    }
    $103 = HEAP32[$0>>2]|0;
    $104 = (($103) + 452|0);
    HEAP32[$104>>2] = 0;
    $105 = HEAP32[$0>>2]|0;
    _codeTableLocks($105);
    $106 = HEAP32[$0>>2]|0;
    _sqlite3AutoincrementBegin($106);
    $107 = HEAP32[$0>>2]|0;
    $108 = (($107) + 316|0);
    $109 = HEAP32[$108>>2]|0;
    $110 = ($109|0)!=(0|0);
    L30: do {
     if ($110) {
      $111 = HEAP32[$0>>2]|0;
      $112 = (($111) + 316|0);
      $113 = HEAP32[$112>>2]|0;
      HEAP32[$pEL>>2] = $113;
      $114 = HEAP32[$0>>2]|0;
      $115 = (($114) + 23|0);
      HEAP8[$115>>0] = 0;
      HEAP32[$i>>2] = 0;
      while(1) {
       $116 = HEAP32[$i>>2]|0;
       $117 = HEAP32[$pEL>>2]|0;
       $118 = HEAP32[$117>>2]|0;
       $119 = ($116|0)<($118|0);
       if (!($119)) {
        break L30;
       }
       $120 = HEAP32[$0>>2]|0;
       $121 = HEAP32[$i>>2]|0;
       $122 = HEAP32[$pEL>>2]|0;
       $123 = (($122) + 4|0);
       $124 = HEAP32[$123>>2]|0;
       $125 = (($124) + (($121*20)|0)|0);
       $126 = HEAP32[$125>>2]|0;
       $127 = HEAP32[$i>>2]|0;
       $128 = HEAP32[$pEL>>2]|0;
       $129 = (($128) + 4|0);
       $130 = HEAP32[$129>>2]|0;
       $131 = (($130) + (($127*20)|0)|0);
       $132 = (($131) + 16|0);
       $133 = HEAP32[$132>>2]|0;
       _sqlite3ExprCode($120,$126,$133);
       $134 = HEAP32[$i>>2]|0;
       $135 = (($134) + 1)|0;
       HEAP32[$i>>2] = $135;
      }
     }
    } while(0);
    $136 = HEAP32[$v>>2]|0;
    (_sqlite3VdbeAddOp2($136,16,0,1)|0);
   }
  }
 } while(0);
 $137 = HEAP32[$v>>2]|0;
 $138 = ($137|0)!=(0|0);
 if ($138) {
  $139 = HEAP32[$0>>2]|0;
  $140 = (($139) + 64|0);
  $141 = HEAP32[$140>>2]|0;
  $142 = ($141|0)==(0);
  if ($142) {
   $143 = HEAP32[$db>>2]|0;
   $144 = (($143) + 64|0);
   $145 = HEAP8[$144>>0]|0;
   $146 = ($145<<24>>24)!=(0);
   if ($146) {
    label = 30;
   } else {
    $147 = HEAP32[$0>>2]|0;
    $148 = (($147) + 404|0);
    $149 = HEAP32[$148>>2]|0;
    $150 = ($149|0)!=(0|0);
    if ($150) {
     $151 = HEAP32[$0>>2]|0;
     $152 = (($151) + 68|0);
     $153 = HEAP32[$152>>2]|0;
     $154 = ($153|0)==(0);
     if ($154) {
      $155 = HEAP32[$0>>2]|0;
      $156 = (($155) + 68|0);
      HEAP32[$156>>2] = 1;
     }
    }
    $157 = HEAP32[$v>>2]|0;
    $158 = HEAP32[$0>>2]|0;
    _sqlite3VdbeMakeReady($157,$158);
    $159 = HEAP32[$0>>2]|0;
    $160 = (($159) + 12|0);
    HEAP32[$160>>2] = 101;
    $161 = HEAP32[$0>>2]|0;
    $162 = (($161) + 16|0);
    HEAP8[$162>>0] = 0;
   }
  } else {
   label = 30;
  }
 } else {
  label = 30;
 }
 if ((label|0) == 30) {
  $163 = HEAP32[$0>>2]|0;
  $164 = (($163) + 12|0);
  HEAP32[$164>>2] = 1;
 }
 $165 = HEAP32[$0>>2]|0;
 $166 = (($165) + 68|0);
 HEAP32[$166>>2] = 0;
 $167 = HEAP32[$0>>2]|0;
 $168 = (($167) + 72|0);
 HEAP32[$168>>2] = 0;
 $169 = HEAP32[$0>>2]|0;
 $170 = (($169) + 76|0);
 HEAP32[$170>>2] = 0;
 $171 = HEAP32[$0>>2]|0;
 $172 = (($171) + 440|0);
 HEAP32[$172>>2] = 0;
 $173 = HEAP32[$0>>2]|0;
 $174 = (($173) + 332|0);
 HEAP32[$174>>2] = 0;
 STACKTOP = sp;return;
}
function _sqlite3BeginTransaction($pParse,$type) {
 $pParse = $pParse|0;
 $type = $type|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $db = 0, $i = 0, $v = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 16|0;
 $1 = sp + 12|0;
 $db = sp + 8|0;
 $v = sp + 4|0;
 $i = sp;
 HEAP32[$0>>2] = $pParse;
 HEAP32[$1>>2] = $type;
 $2 = HEAP32[$0>>2]|0;
 $3 = HEAP32[$2>>2]|0;
 HEAP32[$db>>2] = $3;
 $4 = HEAP32[$0>>2]|0;
 $5 = (_sqlite3AuthCheck($4,22,27128,0,0)|0);
 $6 = ($5|0)!=(0);
 if ($6) {
  STACKTOP = sp;return;
 }
 $7 = HEAP32[$0>>2]|0;
 $8 = (_sqlite3GetVdbe($7)|0);
 HEAP32[$v>>2] = $8;
 $9 = HEAP32[$v>>2]|0;
 $10 = ($9|0)!=(0|0);
 if (!($10)) {
  STACKTOP = sp;return;
 }
 $11 = HEAP32[$1>>2]|0;
 $12 = ($11|0)!=(7);
 L7: do {
  if ($12) {
   HEAP32[$i>>2] = 0;
   while(1) {
    $13 = HEAP32[$i>>2]|0;
    $14 = HEAP32[$db>>2]|0;
    $15 = (($14) + 20|0);
    $16 = HEAP32[$15>>2]|0;
    $17 = ($13|0)<($16|0);
    if (!($17)) {
     break L7;
    }
    $18 = HEAP32[$v>>2]|0;
    $19 = HEAP32[$i>>2]|0;
    $20 = HEAP32[$1>>2]|0;
    $21 = ($20|0)==(9);
    $22 = $21&1;
    $23 = (($22) + 1)|0;
    (_sqlite3VdbeAddOp2($18,4,$19,$23)|0);
    $24 = HEAP32[$v>>2]|0;
    $25 = HEAP32[$i>>2]|0;
    _sqlite3VdbeUsesBtree($24,$25);
    $26 = HEAP32[$i>>2]|0;
    $27 = (($26) + 1)|0;
    HEAP32[$i>>2] = $27;
   }
  }
 } while(0);
 $28 = HEAP32[$v>>2]|0;
 (_sqlite3VdbeAddOp2($28,3,0,0)|0);
 STACKTOP = sp;return;
}
function _sqlite3CommitTransaction($pParse) {
 $pParse = $pParse|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $v = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $v = sp;
 HEAP32[$0>>2] = $pParse;
 $1 = HEAP32[$0>>2]|0;
 $2 = (_sqlite3AuthCheck($1,22,27160,0,0)|0);
 $3 = ($2|0)!=(0);
 if ($3) {
  STACKTOP = sp;return;
 }
 $4 = HEAP32[$0>>2]|0;
 $5 = (_sqlite3GetVdbe($4)|0);
 HEAP32[$v>>2] = $5;
 $6 = HEAP32[$v>>2]|0;
 $7 = ($6|0)!=(0|0);
 if (!($7)) {
  STACKTOP = sp;return;
 }
 $8 = HEAP32[$v>>2]|0;
 (_sqlite3VdbeAddOp2($8,3,1,0)|0);
 STACKTOP = sp;return;
}
function _sqlite3RollbackTransaction($pParse) {
 $pParse = $pParse|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $v = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $v = sp;
 HEAP32[$0>>2] = $pParse;
 $1 = HEAP32[$0>>2]|0;
 $2 = (_sqlite3AuthCheck($1,22,27144,0,0)|0);
 $3 = ($2|0)!=(0);
 if ($3) {
  STACKTOP = sp;return;
 }
 $4 = HEAP32[$0>>2]|0;
 $5 = (_sqlite3GetVdbe($4)|0);
 HEAP32[$v>>2] = $5;
 $6 = HEAP32[$v>>2]|0;
 $7 = ($6|0)!=(0|0);
 if (!($7)) {
  STACKTOP = sp;return;
 }
 $8 = HEAP32[$v>>2]|0;
 (_sqlite3VdbeAddOp2($8,3,1,1)|0);
 STACKTOP = sp;return;
}
function _sqlite3Savepoint($pParse,$op,$pName) {
 $pParse = $pParse|0;
 $op = $op|0;
 $pName = $pName|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $3 = 0;
 var $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $v = 0, $zName = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 16|0;
 $1 = sp + 12|0;
 $2 = sp + 8|0;
 $zName = sp + 4|0;
 $v = sp;
 HEAP32[$0>>2] = $pParse;
 HEAP32[$1>>2] = $op;
 HEAP32[$2>>2] = $pName;
 $3 = HEAP32[$0>>2]|0;
 $4 = HEAP32[$3>>2]|0;
 $5 = HEAP32[$2>>2]|0;
 $6 = (_sqlite3NameFromToken($4,$5)|0);
 HEAP32[$zName>>2] = $6;
 $7 = HEAP32[$zName>>2]|0;
 $8 = ($7|0)!=(0|0);
 if (!($8)) {
  STACKTOP = sp;return;
 }
 $9 = HEAP32[$0>>2]|0;
 $10 = (_sqlite3GetVdbe($9)|0);
 HEAP32[$v>>2] = $10;
 $11 = HEAP32[$v>>2]|0;
 $12 = ($11|0)!=(0|0);
 if ($12) {
  $13 = HEAP32[$0>>2]|0;
  $14 = HEAP32[$1>>2]|0;
  $15 = (27112 + ($14<<2)|0);
  $16 = HEAP32[$15>>2]|0;
  $17 = HEAP32[$zName>>2]|0;
  $18 = (_sqlite3AuthCheck($13,32,$16,$17,0)|0);
  $19 = ($18|0)!=(0);
  if (!($19)) {
   $23 = HEAP32[$v>>2]|0;
   $24 = HEAP32[$1>>2]|0;
   $25 = HEAP32[$zName>>2]|0;
   (_sqlite3VdbeAddOp4($23,2,$24,0,0,$25,-1)|0);
   STACKTOP = sp;return;
  }
 }
 $20 = HEAP32[$0>>2]|0;
 $21 = HEAP32[$20>>2]|0;
 $22 = HEAP32[$zName>>2]|0;
 _sqlite3DbFree($21,$22);
 STACKTOP = sp;return;
}
function _sqlite3StartTable($pParse,$pName1,$pName2,$isTemp,$isView,$isVirtual,$noErr) {
 $pParse = $pParse|0;
 $pName1 = $pName1|0;
 $pName2 = $pName2|0;
 $isTemp = $isTemp|0;
 $isView = $isView|0;
 $isVirtual = $isVirtual|0;
 $noErr = $noErr|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0;
 var $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0, $188 = 0;
 var $189 = 0, $19 = 0, $190 = 0, $191 = 0, $192 = 0, $193 = 0, $194 = 0, $195 = 0, $196 = 0, $197 = 0, $198 = 0, $199 = 0, $2 = 0, $20 = 0, $200 = 0, $201 = 0, $202 = 0, $203 = 0, $204 = 0, $205 = 0;
 var $206 = 0, $207 = 0, $208 = 0, $209 = 0, $21 = 0, $210 = 0, $211 = 0, $212 = 0, $213 = 0, $214 = 0, $215 = 0, $216 = 0, $217 = 0, $218 = 0, $219 = 0, $22 = 0, $220 = 0, $221 = 0, $222 = 0, $223 = 0;
 var $224 = 0, $225 = 0, $226 = 0, $227 = 0, $228 = 0, $229 = 0, $23 = 0, $230 = 0, $231 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0;
 var $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0;
 var $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0;
 var $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0;
 var $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $code = 0, $db = 0, $fileFormat = 0, $iDb = 0, $j1 = 0, $or$cond = 0, $or$cond3 = 0, $pName = 0;
 var $pTable = 0, $reg1 = 0, $reg2 = 0, $reg3 = 0, $v = 0, $vararg_buffer6 = 0, $zDb = 0, $zDb1 = 0, $zName = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 96|0;
 $vararg_buffer6 = sp;
 $0 = sp + 84|0;
 $1 = sp + 80|0;
 $2 = sp + 76|0;
 $3 = sp + 72|0;
 $4 = sp + 68|0;
 $5 = sp + 64|0;
 $6 = sp + 60|0;
 $pTable = sp + 56|0;
 $zName = sp + 52|0;
 $db = sp + 48|0;
 $v = sp + 44|0;
 $iDb = sp + 40|0;
 $pName = sp + 36|0;
 $code = sp + 32|0;
 $zDb = sp + 28|0;
 $zDb1 = sp + 24|0;
 $j1 = sp + 20|0;
 $fileFormat = sp + 16|0;
 $reg1 = sp + 12|0;
 $reg2 = sp + 8|0;
 $reg3 = sp + 4|0;
 HEAP32[$0>>2] = $pParse;
 HEAP32[$1>>2] = $pName1;
 HEAP32[$2>>2] = $pName2;
 HEAP32[$3>>2] = $isTemp;
 HEAP32[$4>>2] = $isView;
 HEAP32[$5>>2] = $isVirtual;
 HEAP32[$6>>2] = $noErr;
 HEAP32[$zName>>2] = 0;
 $7 = HEAP32[$0>>2]|0;
 $8 = HEAP32[$7>>2]|0;
 HEAP32[$db>>2] = $8;
 $9 = HEAP32[$0>>2]|0;
 $10 = HEAP32[$1>>2]|0;
 $11 = HEAP32[$2>>2]|0;
 $12 = (_sqlite3TwoPartName($9,$10,$11,$pName)|0);
 HEAP32[$iDb>>2] = $12;
 $13 = HEAP32[$iDb>>2]|0;
 $14 = ($13|0)<(0);
 if ($14) {
  STACKTOP = sp;return;
 }
 $15 = HEAP32[$3>>2]|0;
 $16 = ($15|0)!=(0);
 if ($16) {
  $17 = HEAP32[$2>>2]|0;
  $18 = (($17) + 4|0);
  $19 = HEAP32[$18>>2]|0;
  $20 = ($19>>>0)>(0);
  $21 = HEAP32[$iDb>>2]|0;
  $22 = ($21|0)!=(1);
  $or$cond = $20 & $22;
  if ($or$cond) {
   $23 = HEAP32[$0>>2]|0;
   _sqlite3ErrorMsg($23,27000,$vararg_buffer6);
   STACKTOP = sp;return;
  }
 }
 $24 = HEAP32[$3>>2]|0;
 $25 = ($24|0)!=(0);
 if ($25) {
  HEAP32[$iDb>>2] = 1;
 }
 $26 = HEAP32[$0>>2]|0;
 $27 = (($26) + 496|0);
 $28 = HEAP32[$pName>>2]|0;
 ;HEAP32[$27+0>>2]=HEAP32[$28+0>>2]|0;HEAP32[$27+4>>2]=HEAP32[$28+4>>2]|0;
 $29 = HEAP32[$db>>2]|0;
 $30 = HEAP32[$pName>>2]|0;
 $31 = (_sqlite3NameFromToken($29,$30)|0);
 HEAP32[$zName>>2] = $31;
 $32 = HEAP32[$zName>>2]|0;
 $33 = ($32|0)==(0|0);
 if ($33) {
  STACKTOP = sp;return;
 }
 $34 = HEAP32[$0>>2]|0;
 $35 = HEAP32[$zName>>2]|0;
 $36 = (_sqlite3CheckObjectName($34,$35)|0);
 $37 = (0)!=($36|0);
 L15: do {
  if (!($37)) {
   $38 = HEAP32[$db>>2]|0;
   $39 = (($38) + 140|0);
   $40 = (($39) + 4|0);
   $41 = HEAP8[$40>>0]|0;
   $42 = $41&255;
   $43 = ($42|0)==(1);
   if ($43) {
    HEAP32[$3>>2] = 1;
   }
   $44 = HEAP32[$iDb>>2]|0;
   $45 = HEAP32[$db>>2]|0;
   $46 = (($45) + 16|0);
   $47 = HEAP32[$46>>2]|0;
   $48 = (($47) + ($44<<4)|0);
   $49 = HEAP32[$48>>2]|0;
   HEAP32[$zDb>>2] = $49;
   $50 = HEAP32[$0>>2]|0;
   $51 = HEAP32[$3>>2]|0;
   $52 = ($51|0)==(1);
   $53 = $52 ? 14904 : 14928;
   $54 = HEAP32[$zDb>>2]|0;
   $55 = (_sqlite3AuthCheck($50,18,$53,0,$54)|0);
   $56 = ($55|0)!=(0);
   if (!($56)) {
    $57 = HEAP32[$4>>2]|0;
    $58 = ($57|0)!=(0);
    $59 = HEAP32[$3>>2]|0;
    $60 = ($59|0)!=(0);
    do {
     if ($58) {
      if ($60) {
       HEAP32[$code>>2] = 6;
       break;
      } else {
       HEAP32[$code>>2] = 8;
       break;
      }
     } else {
      if ($60) {
       HEAP32[$code>>2] = 4;
       break;
      } else {
       HEAP32[$code>>2] = 2;
       break;
      }
     }
    } while(0);
    $61 = HEAP32[$5>>2]|0;
    $62 = ($61|0)!=(0);
    if (!($62)) {
     $63 = HEAP32[$0>>2]|0;
     $64 = HEAP32[$code>>2]|0;
     $65 = HEAP32[$zName>>2]|0;
     $66 = HEAP32[$zDb>>2]|0;
     $67 = (_sqlite3AuthCheck($63,$64,$65,0,$66)|0);
     $68 = ($67|0)!=(0);
     if ($68) {
      break;
     }
    }
    $69 = HEAP32[$0>>2]|0;
    $70 = (($69) + 451|0);
    $71 = HEAP8[$70>>0]|0;
    $72 = ($71<<24>>24)!=(0);
    do {
     if (!($72)) {
      $73 = HEAP32[$iDb>>2]|0;
      $74 = HEAP32[$db>>2]|0;
      $75 = (($74) + 16|0);
      $76 = HEAP32[$75>>2]|0;
      $77 = (($76) + ($73<<4)|0);
      $78 = HEAP32[$77>>2]|0;
      HEAP32[$zDb1>>2] = $78;
      $79 = HEAP32[$0>>2]|0;
      $80 = (_sqlite3ReadSchema($79)|0);
      $81 = (0)!=($80|0);
      if ($81) {
       break L15;
      }
      $82 = HEAP32[$db>>2]|0;
      $83 = HEAP32[$zName>>2]|0;
      $84 = HEAP32[$zDb1>>2]|0;
      $85 = (_sqlite3FindTable($82,$83,$84)|0);
      HEAP32[$pTable>>2] = $85;
      $86 = HEAP32[$pTable>>2]|0;
      $87 = ($86|0)!=(0|0);
      if (!($87)) {
       $93 = HEAP32[$db>>2]|0;
       $94 = HEAP32[$zName>>2]|0;
       $95 = HEAP32[$zDb1>>2]|0;
       $96 = (_sqlite3FindIndex($93,$94,$95)|0);
       $97 = ($96|0)!=(0|0);
       if (!($97)) {
        break;
       }
       $98 = HEAP32[$0>>2]|0;
       $99 = HEAP32[$zName>>2]|0;
       HEAP32[$vararg_buffer6>>2] = $99;
       _sqlite3ErrorMsg($98,27072,$vararg_buffer6);
       break L15;
      }
      $88 = HEAP32[$6>>2]|0;
      $89 = ($88|0)!=(0);
      $90 = HEAP32[$0>>2]|0;
      if ($89) {
       $92 = HEAP32[$iDb>>2]|0;
       _sqlite3CodeVerifySchema($90,$92);
       break L15;
      } else {
       $91 = HEAP32[$pName>>2]|0;
       HEAP32[$vararg_buffer6>>2] = $91;
       _sqlite3ErrorMsg($90,27048,$vararg_buffer6);
       break L15;
      }
     }
    } while(0);
    $100 = HEAP32[$db>>2]|0;
    $101 = (_sqlite3DbMallocZero($100,76,0)|0);
    HEAP32[$pTable>>2] = $101;
    $102 = HEAP32[$pTable>>2]|0;
    $103 = ($102|0)==(0|0);
    if ($103) {
     $104 = HEAP32[$db>>2]|0;
     $105 = (($104) + 64|0);
     HEAP8[$105>>0] = 1;
     $106 = HEAP32[$0>>2]|0;
     $107 = (($106) + 12|0);
     HEAP32[$107>>2] = 7;
     $108 = HEAP32[$0>>2]|0;
     $109 = (($108) + 64|0);
     $110 = HEAP32[$109>>2]|0;
     $111 = (($110) + 1)|0;
     HEAP32[$109>>2] = $111;
     break;
    }
    $112 = HEAP32[$zName>>2]|0;
    $113 = HEAP32[$pTable>>2]|0;
    HEAP32[$113>>2] = $112;
    $114 = HEAP32[$pTable>>2]|0;
    $115 = (($114) + 36|0);
    HEAP16[$115>>1] = -1;
    $116 = HEAP32[$iDb>>2]|0;
    $117 = HEAP32[$db>>2]|0;
    $118 = (($117) + 16|0);
    $119 = HEAP32[$118>>2]|0;
    $120 = (($119) + ($116<<4)|0);
    $121 = (($120) + 12|0);
    $122 = HEAP32[$121>>2]|0;
    $123 = HEAP32[$pTable>>2]|0;
    $124 = (($123) + 68|0);
    HEAP32[$124>>2] = $122;
    $125 = HEAP32[$pTable>>2]|0;
    $126 = (($125) + 40|0);
    HEAP16[$126>>1] = 1;
    $127 = HEAP32[$pTable>>2]|0;
    $128 = (($127) + 28|0);
    HEAP16[$128>>1] = 200;
    $129 = HEAP32[$pTable>>2]|0;
    $130 = HEAP32[$0>>2]|0;
    $131 = (($130) + 484|0);
    HEAP32[$131>>2] = $129;
    $132 = HEAP32[$0>>2]|0;
    $133 = (($132) + 18|0);
    $134 = HEAP8[$133>>0]|0;
    $135 = ($134<<24>>24)!=(0);
    if (!($135)) {
     $136 = HEAP32[$zName>>2]|0;
     $137 = (_strcmp($136,16152)|0);
     $138 = ($137|0)==(0);
     if ($138) {
      $139 = HEAP32[$pTable>>2]|0;
      $140 = HEAP32[$pTable>>2]|0;
      $141 = (($140) + 68|0);
      $142 = HEAP32[$141>>2]|0;
      $143 = (($142) + 72|0);
      HEAP32[$143>>2] = $139;
     }
    }
    $144 = HEAP32[$db>>2]|0;
    $145 = (($144) + 140|0);
    $146 = (($145) + 5|0);
    $147 = HEAP8[$146>>0]|0;
    $148 = ($147<<24>>24)!=(0);
    if ($148) {
     STACKTOP = sp;return;
    }
    $149 = HEAP32[$0>>2]|0;
    $150 = (_sqlite3GetVdbe($149)|0);
    HEAP32[$v>>2] = $150;
    $151 = ($150|0)!=(0|0);
    if (!($151)) {
     STACKTOP = sp;return;
    }
    $152 = HEAP32[$0>>2]|0;
    $153 = HEAP32[$iDb>>2]|0;
    _sqlite3BeginWriteOperation($152,0,$153);
    $154 = HEAP32[$5>>2]|0;
    $155 = ($154|0)!=(0);
    if ($155) {
     $156 = HEAP32[$v>>2]|0;
     (_sqlite3VdbeAddOp0($156,144)|0);
    }
    $157 = HEAP32[$0>>2]|0;
    $158 = (($157) + 72|0);
    $159 = HEAP32[$158>>2]|0;
    $160 = (($159) + 1)|0;
    HEAP32[$158>>2] = $160;
    $161 = HEAP32[$0>>2]|0;
    $162 = (($161) + 384|0);
    HEAP32[$162>>2] = $160;
    HEAP32[$reg1>>2] = $160;
    $163 = HEAP32[$0>>2]|0;
    $164 = (($163) + 72|0);
    $165 = HEAP32[$164>>2]|0;
    $166 = (($165) + 1)|0;
    HEAP32[$164>>2] = $166;
    $167 = HEAP32[$0>>2]|0;
    $168 = (($167) + 388|0);
    HEAP32[$168>>2] = $166;
    HEAP32[$reg2>>2] = $166;
    $169 = HEAP32[$0>>2]|0;
    $170 = (($169) + 72|0);
    $171 = HEAP32[$170>>2]|0;
    $172 = (($171) + 1)|0;
    HEAP32[$170>>2] = $172;
    HEAP32[$reg3>>2] = $172;
    $173 = HEAP32[$v>>2]|0;
    $174 = HEAP32[$iDb>>2]|0;
    $175 = HEAP32[$reg3>>2]|0;
    (_sqlite3VdbeAddOp3($173,51,$174,$175,2)|0);
    $176 = HEAP32[$v>>2]|0;
    $177 = HEAP32[$iDb>>2]|0;
    _sqlite3VdbeUsesBtree($176,$177);
    $178 = HEAP32[$v>>2]|0;
    $179 = HEAP32[$reg3>>2]|0;
    $180 = (_sqlite3VdbeAddOp1($178,45,$179)|0);
    HEAP32[$j1>>2] = $180;
    $181 = HEAP32[$db>>2]|0;
    $182 = (($181) + 24|0);
    $183 = HEAP32[$182>>2]|0;
    $184 = $183 & 32768;
    $185 = ($184|0)!=(0);
    $186 = $185 ? 1 : 4;
    HEAP32[$fileFormat>>2] = $186;
    $187 = HEAP32[$v>>2]|0;
    $188 = HEAP32[$fileFormat>>2]|0;
    $189 = HEAP32[$reg3>>2]|0;
    (_sqlite3VdbeAddOp2($187,25,$188,$189)|0);
    $190 = HEAP32[$v>>2]|0;
    $191 = HEAP32[$iDb>>2]|0;
    $192 = HEAP32[$reg3>>2]|0;
    (_sqlite3VdbeAddOp3($190,52,$191,2,$192)|0);
    $193 = HEAP32[$v>>2]|0;
    $194 = HEAP32[$db>>2]|0;
    $195 = (($194) + 16|0);
    $196 = HEAP32[$195>>2]|0;
    $197 = (($196) + 12|0);
    $198 = HEAP32[$197>>2]|0;
    $199 = (($198) + 77|0);
    $200 = HEAP8[$199>>0]|0;
    $201 = $200&255;
    $202 = HEAP32[$reg3>>2]|0;
    (_sqlite3VdbeAddOp2($193,25,$201,$202)|0);
    $203 = HEAP32[$v>>2]|0;
    $204 = HEAP32[$iDb>>2]|0;
    $205 = HEAP32[$reg3>>2]|0;
    (_sqlite3VdbeAddOp3($203,52,$204,5,$205)|0);
    $206 = HEAP32[$v>>2]|0;
    $207 = HEAP32[$j1>>2]|0;
    _sqlite3VdbeJumpHere($206,$207);
    $208 = HEAP32[$4>>2]|0;
    $209 = ($208|0)!=(0);
    $210 = HEAP32[$5>>2]|0;
    $211 = ($210|0)!=(0);
    $or$cond3 = $209 | $211;
    $212 = HEAP32[$v>>2]|0;
    if ($or$cond3) {
     $213 = HEAP32[$reg2>>2]|0;
     (_sqlite3VdbeAddOp2($212,25,0,$213)|0);
    } else {
     $214 = HEAP32[$iDb>>2]|0;
     $215 = HEAP32[$reg2>>2]|0;
     $216 = (_sqlite3VdbeAddOp2($212,121,$214,$215)|0);
     $217 = HEAP32[$0>>2]|0;
     $218 = (($217) + 416|0);
     HEAP32[$218>>2] = $216;
    }
    $219 = HEAP32[$0>>2]|0;
    $220 = HEAP32[$iDb>>2]|0;
    _sqlite3OpenMasterTable($219,$220);
    $221 = HEAP32[$v>>2]|0;
    $222 = HEAP32[$reg1>>2]|0;
    (_sqlite3VdbeAddOp2($221,74,0,$222)|0);
    $223 = HEAP32[$v>>2]|0;
    $224 = HEAP32[$reg3>>2]|0;
    (_sqlite3VdbeAddOp2($223,28,0,$224)|0);
    $225 = HEAP32[$v>>2]|0;
    $226 = HEAP32[$reg3>>2]|0;
    $227 = HEAP32[$reg1>>2]|0;
    (_sqlite3VdbeAddOp3($225,75,0,$226,$227)|0);
    $228 = HEAP32[$v>>2]|0;
    _sqlite3VdbeChangeP5($228,8);
    $229 = HEAP32[$v>>2]|0;
    (_sqlite3VdbeAddOp0($229,61)|0);
    STACKTOP = sp;return;
   }
  }
 } while(0);
 $230 = HEAP32[$db>>2]|0;
 $231 = HEAP32[$zName>>2]|0;
 _sqlite3DbFree($230,$231);
 STACKTOP = sp;return;
}
function _sqlite3EndTable($pParse,$pCons,$pEnd,$tabOpts,$pSelect) {
 $pParse = $pParse|0;
 $pCons = $pCons|0;
 $pEnd = $pEnd|0;
 $tabOpts = $tabOpts|0;
 $pSelect = $pSelect|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0;
 var $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0, $188 = 0;
 var $189 = 0, $19 = 0, $190 = 0, $191 = 0, $192 = 0, $193 = 0, $194 = 0, $195 = 0, $196 = 0, $197 = 0, $198 = 0, $199 = 0, $2 = 0, $20 = 0, $200 = 0, $201 = 0, $202 = 0, $203 = 0, $204 = 0, $205 = 0;
 var $206 = 0, $207 = 0, $208 = 0, $209 = 0, $21 = 0, $210 = 0, $211 = 0, $212 = 0, $213 = 0, $214 = 0, $215 = 0, $216 = 0, $217 = 0, $218 = 0, $219 = 0, $22 = 0, $220 = 0, $221 = 0, $222 = 0, $223 = 0;
 var $224 = 0, $225 = 0, $226 = 0, $227 = 0, $228 = 0, $229 = 0, $23 = 0, $230 = 0, $231 = 0, $232 = 0, $233 = 0, $234 = 0, $235 = 0, $236 = 0, $237 = 0, $238 = 0, $239 = 0, $24 = 0, $240 = 0, $241 = 0;
 var $242 = 0, $243 = 0, $244 = 0, $245 = 0, $246 = 0, $247 = 0, $248 = 0, $249 = 0, $25 = 0, $250 = 0, $251 = 0, $252 = 0, $253 = 0, $254 = 0, $255 = 0, $256 = 0, $257 = 0, $258 = 0, $259 = 0, $26 = 0;
 var $260 = 0, $261 = 0, $262 = 0, $263 = 0, $264 = 0, $265 = 0, $266 = 0, $267 = 0, $268 = 0, $269 = 0, $27 = 0, $270 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0;
 var $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0;
 var $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0;
 var $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0;
 var $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $db = 0, $dest = 0, $iDb = 0, $n = 0, $nName = 0, $or$cond = 0, $p = 0, $pDb = 0, $pEnd2 = 0;
 var $pIdx = 0, $pOld = 0, $pSchema = 0, $pSelTab = 0, $v = 0, $vararg_buffer22 = 0, $vararg_ptr12 = 0, $vararg_ptr13 = 0, $vararg_ptr14 = 0, $vararg_ptr15 = 0, $vararg_ptr16 = 0, $vararg_ptr17 = 0, $vararg_ptr18 = 0, $vararg_ptr7 = 0, $vararg_ptr8 = 0, $zName = 0, $zStmt = 0, $zType = 0, $zType2 = 0, label = 0;
 var sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 144|0;
 $vararg_buffer22 = sp;
 $0 = sp + 128|0;
 $1 = sp + 124|0;
 $2 = sp + 120|0;
 $3 = sp + 132|0;
 $4 = sp + 116|0;
 $p = sp + 112|0;
 $db = sp + 108|0;
 $iDb = sp + 104|0;
 $pIdx = sp + 100|0;
 $n = sp + 96|0;
 $v = sp + 92|0;
 $zType = sp + 88|0;
 $zType2 = sp + 84|0;
 $zStmt = sp + 80|0;
 $dest = sp + 60|0;
 $pSelTab = sp + 56|0;
 $pEnd2 = sp + 52|0;
 $pDb = sp + 48|0;
 $pOld = sp + 44|0;
 $pSchema = sp + 40|0;
 $zName = sp + 36|0;
 $nName = sp + 32|0;
 HEAP32[$0>>2] = $pParse;
 HEAP32[$1>>2] = $pCons;
 HEAP32[$2>>2] = $pEnd;
 HEAP8[$3>>0] = $tabOpts;
 HEAP32[$4>>2] = $pSelect;
 $5 = HEAP32[$0>>2]|0;
 $6 = HEAP32[$5>>2]|0;
 HEAP32[$db>>2] = $6;
 $7 = HEAP32[$2>>2]|0;
 $8 = ($7|0)==(0|0);
 $9 = HEAP32[$4>>2]|0;
 $10 = ($9|0)==(0|0);
 $or$cond = $8 & $10;
 if ($or$cond) {
  STACKTOP = sp;return;
 }
 $11 = HEAP32[$db>>2]|0;
 $12 = (($11) + 64|0);
 $13 = HEAP8[$12>>0]|0;
 $14 = $13&255;
 $15 = ($14|0)!=(0);
 if ($15) {
  STACKTOP = sp;return;
 }
 $16 = HEAP32[$0>>2]|0;
 $17 = (($16) + 484|0);
 $18 = HEAP32[$17>>2]|0;
 HEAP32[$p>>2] = $18;
 $19 = HEAP32[$p>>2]|0;
 $20 = ($19|0)==(0|0);
 if ($20) {
  STACKTOP = sp;return;
 }
 $21 = HEAP32[$db>>2]|0;
 $22 = (($21) + 140|0);
 $23 = (($22) + 5|0);
 $24 = HEAP8[$23>>0]|0;
 $25 = ($24<<24>>24)!=(0);
 if ($25) {
  $26 = HEAP32[$db>>2]|0;
  $27 = (($26) + 140|0);
  $28 = HEAP32[$27>>2]|0;
  $29 = HEAP32[$p>>2]|0;
  $30 = (($29) + 32|0);
  HEAP32[$30>>2] = $28;
 }
 $31 = HEAP8[$3>>0]|0;
 $32 = $31&255;
 $33 = $32 & 32;
 $34 = ($33|0)!=(0);
 do {
  if ($34) {
   $35 = HEAP32[$p>>2]|0;
   $36 = (($35) + 44|0);
   $37 = HEAP8[$36>>0]|0;
   $38 = $37&255;
   $39 = $38 & 8;
   $40 = ($39|0)!=(0);
   if ($40) {
    $41 = HEAP32[$0>>2]|0;
    _sqlite3ErrorMsg($41,26608,$vararg_buffer22);
    STACKTOP = sp;return;
   }
   $42 = HEAP32[$p>>2]|0;
   $43 = (($42) + 44|0);
   $44 = HEAP8[$43>>0]|0;
   $45 = $44&255;
   $46 = $45 & 4;
   $47 = ($46|0)==(0);
   if ($47) {
    $48 = HEAP32[$0>>2]|0;
    $49 = HEAP32[$p>>2]|0;
    $50 = HEAP32[$49>>2]|0;
    HEAP32[$vararg_buffer22>>2] = $50;
    _sqlite3ErrorMsg($48,26664,$vararg_buffer22);
    break;
   } else {
    $51 = HEAP32[$p>>2]|0;
    $52 = (($51) + 44|0);
    $53 = HEAP8[$52>>0]|0;
    $54 = $53&255;
    $55 = $54 | 32;
    $56 = $55&255;
    HEAP8[$52>>0] = $56;
    $57 = HEAP32[$0>>2]|0;
    $58 = HEAP32[$p>>2]|0;
    _convertToWithoutRowidTable($57,$58);
    break;
   }
  }
 } while(0);
 $59 = HEAP32[$db>>2]|0;
 $60 = HEAP32[$p>>2]|0;
 $61 = (($60) + 68|0);
 $62 = HEAP32[$61>>2]|0;
 $63 = (_sqlite3SchemaToIndex($59,$62)|0);
 HEAP32[$iDb>>2] = $63;
 $64 = HEAP32[$p>>2]|0;
 $65 = (($64) + 24|0);
 $66 = HEAP32[$65>>2]|0;
 $67 = ($66|0)!=(0|0);
 if ($67) {
  $68 = HEAP32[$0>>2]|0;
  $69 = HEAP32[$p>>2]|0;
  $70 = HEAP32[$p>>2]|0;
  $71 = (($70) + 24|0);
  $72 = HEAP32[$71>>2]|0;
  _sqlite3ResolveSelfReference($68,$69,4,0,$72);
 }
 $73 = HEAP32[$p>>2]|0;
 _estimateTableWidth($73);
 $74 = HEAP32[$p>>2]|0;
 $75 = (($74) + 8|0);
 $76 = HEAP32[$75>>2]|0;
 HEAP32[$pIdx>>2] = $76;
 while(1) {
  $77 = HEAP32[$pIdx>>2]|0;
  $78 = ($77|0)!=(0|0);
  if (!($78)) {
   break;
  }
  $79 = HEAP32[$pIdx>>2]|0;
  _estimateIndexWidth($79);
  $80 = HEAP32[$pIdx>>2]|0;
  $81 = (($80) + 20|0);
  $82 = HEAP32[$81>>2]|0;
  HEAP32[$pIdx>>2] = $82;
 }
 $83 = HEAP32[$db>>2]|0;
 $84 = (($83) + 140|0);
 $85 = (($84) + 5|0);
 $86 = HEAP8[$85>>0]|0;
 $87 = ($86<<24>>24)!=(0);
 if (!($87)) {
  $88 = HEAP32[$0>>2]|0;
  $89 = (_sqlite3GetVdbe($88)|0);
  HEAP32[$v>>2] = $89;
  $90 = HEAP32[$v>>2]|0;
  $91 = ($90|0)==(0|0);
  if ($91) {
   STACKTOP = sp;return;
  }
  $92 = HEAP32[$v>>2]|0;
  (_sqlite3VdbeAddOp1($92,61,0)|0);
  $93 = HEAP32[$p>>2]|0;
  $94 = (($93) + 12|0);
  $95 = HEAP32[$94>>2]|0;
  $96 = ($95|0)==(0|0);
  if ($96) {
   HEAP32[$zType>>2] = 21400;
   HEAP32[$zType2>>2] = 26696;
  } else {
   HEAP32[$zType>>2] = 25504;
   HEAP32[$zType2>>2] = 26704;
  }
  $97 = HEAP32[$4>>2]|0;
  $98 = ($97|0)!=(0|0);
  do {
   if ($98) {
    $99 = HEAP32[$v>>2]|0;
    $100 = HEAP32[$0>>2]|0;
    $101 = (($100) + 388|0);
    $102 = HEAP32[$101>>2]|0;
    $103 = HEAP32[$iDb>>2]|0;
    (_sqlite3VdbeAddOp3($99,55,1,$102,$103)|0);
    $104 = HEAP32[$v>>2]|0;
    _sqlite3VdbeChangeP5($104,2);
    $105 = HEAP32[$0>>2]|0;
    $106 = (($105) + 68|0);
    HEAP32[$106>>2] = 2;
    _sqlite3SelectDestInit($dest,14,1);
    $107 = HEAP32[$0>>2]|0;
    $108 = HEAP32[$4>>2]|0;
    (_sqlite3Select($107,$108,$dest)|0);
    $109 = HEAP32[$v>>2]|0;
    (_sqlite3VdbeAddOp1($109,61,1)|0);
    $110 = HEAP32[$0>>2]|0;
    $111 = (($110) + 64|0);
    $112 = HEAP32[$111>>2]|0;
    $113 = ($112|0)==(0);
    if ($113) {
     $114 = HEAP32[$0>>2]|0;
     $115 = HEAP32[$4>>2]|0;
     $116 = (_sqlite3ResultSetOfSelect($114,$115)|0);
     HEAP32[$pSelTab>>2] = $116;
     $117 = HEAP32[$pSelTab>>2]|0;
     $118 = ($117|0)==(0|0);
     if ($118) {
      STACKTOP = sp;return;
     } else {
      $119 = HEAP32[$pSelTab>>2]|0;
      $120 = (($119) + 38|0);
      $121 = HEAP16[$120>>1]|0;
      $122 = HEAP32[$p>>2]|0;
      $123 = (($122) + 38|0);
      HEAP16[$123>>1] = $121;
      $124 = HEAP32[$pSelTab>>2]|0;
      $125 = (($124) + 4|0);
      $126 = HEAP32[$125>>2]|0;
      $127 = HEAP32[$p>>2]|0;
      $128 = (($127) + 4|0);
      HEAP32[$128>>2] = $126;
      $129 = HEAP32[$pSelTab>>2]|0;
      $130 = (($129) + 38|0);
      HEAP16[$130>>1] = 0;
      $131 = HEAP32[$pSelTab>>2]|0;
      $132 = (($131) + 4|0);
      HEAP32[$132>>2] = 0;
      $133 = HEAP32[$db>>2]|0;
      $134 = HEAP32[$pSelTab>>2]|0;
      _sqlite3DeleteTable($133,$134);
      break;
     }
    }
   }
  } while(0);
  $135 = HEAP32[$4>>2]|0;
  $136 = ($135|0)!=(0|0);
  if ($136) {
   $137 = HEAP32[$db>>2]|0;
   $138 = HEAP32[$p>>2]|0;
   $139 = (_createTableStmt($137,$138)|0);
   HEAP32[$zStmt>>2] = $139;
  } else {
   $140 = HEAP8[$3>>0]|0;
   $141 = $140&255;
   $142 = ($141|0)!=(0);
   if ($142) {
    $143 = HEAP32[$0>>2]|0;
    $144 = (($143) + 504|0);
    $146 = $144;
   } else {
    $145 = HEAP32[$2>>2]|0;
    $146 = $145;
   }
   HEAP32[$pEnd2>>2] = $146;
   $147 = HEAP32[$pEnd2>>2]|0;
   $148 = HEAP32[$147>>2]|0;
   $149 = HEAP32[$0>>2]|0;
   $150 = (($149) + 496|0);
   $151 = HEAP32[$150>>2]|0;
   $152 = $148;
   $153 = $151;
   $154 = (($152) - ($153))|0;
   HEAP32[$n>>2] = $154;
   $155 = HEAP32[$pEnd2>>2]|0;
   $156 = HEAP32[$155>>2]|0;
   $157 = HEAP8[$156>>0]|0;
   $158 = $157 << 24 >> 24;
   $159 = ($158|0)!=(59);
   if ($159) {
    $160 = HEAP32[$pEnd2>>2]|0;
    $161 = (($160) + 4|0);
    $162 = HEAP32[$161>>2]|0;
    $163 = HEAP32[$n>>2]|0;
    $164 = (($163) + ($162))|0;
    HEAP32[$n>>2] = $164;
   }
   $165 = HEAP32[$db>>2]|0;
   $166 = HEAP32[$zType2>>2]|0;
   $167 = HEAP32[$n>>2]|0;
   $168 = HEAP32[$0>>2]|0;
   $169 = (($168) + 496|0);
   $170 = HEAP32[$169>>2]|0;
   HEAP32[$vararg_buffer22>>2] = $166;
   $vararg_ptr7 = (($vararg_buffer22) + 4|0);
   HEAP32[$vararg_ptr7>>2] = $167;
   $vararg_ptr8 = (($vararg_buffer22) + 8|0);
   HEAP32[$vararg_ptr8>>2] = $170;
   $171 = (_sqlite3MPrintf($165,26712,$vararg_buffer22)|0);
   HEAP32[$zStmt>>2] = $171;
  }
  $172 = HEAP32[$0>>2]|0;
  $173 = HEAP32[$iDb>>2]|0;
  $174 = HEAP32[$db>>2]|0;
  $175 = (($174) + 16|0);
  $176 = HEAP32[$175>>2]|0;
  $177 = (($176) + ($173<<4)|0);
  $178 = HEAP32[$177>>2]|0;
  $179 = HEAP32[$iDb>>2]|0;
  $180 = ($179|0)==(1);
  $181 = $180 ? 14904 : 14928;
  $182 = HEAP32[$zType>>2]|0;
  $183 = HEAP32[$p>>2]|0;
  $184 = HEAP32[$183>>2]|0;
  $185 = HEAP32[$p>>2]|0;
  $186 = HEAP32[$185>>2]|0;
  $187 = HEAP32[$0>>2]|0;
  $188 = (($187) + 388|0);
  $189 = HEAP32[$188>>2]|0;
  $190 = HEAP32[$zStmt>>2]|0;
  $191 = HEAP32[$0>>2]|0;
  $192 = (($191) + 384|0);
  $193 = HEAP32[$192>>2]|0;
  HEAP32[$vararg_buffer22>>2] = $178;
  $vararg_ptr12 = (($vararg_buffer22) + 4|0);
  HEAP32[$vararg_ptr12>>2] = $181;
  $vararg_ptr13 = (($vararg_buffer22) + 8|0);
  HEAP32[$vararg_ptr13>>2] = $182;
  $vararg_ptr14 = (($vararg_buffer22) + 12|0);
  HEAP32[$vararg_ptr14>>2] = $184;
  $vararg_ptr15 = (($vararg_buffer22) + 16|0);
  HEAP32[$vararg_ptr15>>2] = $186;
  $vararg_ptr16 = (($vararg_buffer22) + 20|0);
  HEAP32[$vararg_ptr16>>2] = $189;
  $vararg_ptr17 = (($vararg_buffer22) + 24|0);
  HEAP32[$vararg_ptr17>>2] = $190;
  $vararg_ptr18 = (($vararg_buffer22) + 28|0);
  HEAP32[$vararg_ptr18>>2] = $193;
  _sqlite3NestedParse($172,26728,$vararg_buffer22);
  $194 = HEAP32[$db>>2]|0;
  $195 = HEAP32[$zStmt>>2]|0;
  _sqlite3DbFree($194,$195);
  $196 = HEAP32[$0>>2]|0;
  $197 = HEAP32[$iDb>>2]|0;
  _sqlite3ChangeCookie($196,$197);
  $198 = HEAP32[$p>>2]|0;
  $199 = (($198) + 44|0);
  $200 = HEAP8[$199>>0]|0;
  $201 = $200&255;
  $202 = $201 & 8;
  $203 = ($202|0)!=(0);
  if ($203) {
   $204 = HEAP32[$iDb>>2]|0;
   $205 = HEAP32[$db>>2]|0;
   $206 = (($205) + 16|0);
   $207 = HEAP32[$206>>2]|0;
   $208 = (($207) + ($204<<4)|0);
   HEAP32[$pDb>>2] = $208;
   $209 = HEAP32[$pDb>>2]|0;
   $210 = (($209) + 12|0);
   $211 = HEAP32[$210>>2]|0;
   $212 = (($211) + 72|0);
   $213 = HEAP32[$212>>2]|0;
   $214 = ($213|0)==(0|0);
   if ($214) {
    $215 = HEAP32[$0>>2]|0;
    $216 = HEAP32[$pDb>>2]|0;
    $217 = HEAP32[$216>>2]|0;
    HEAP32[$vararg_buffer22>>2] = $217;
    _sqlite3NestedParse($215,26816,$vararg_buffer22);
   }
  }
  $218 = HEAP32[$v>>2]|0;
  $219 = HEAP32[$iDb>>2]|0;
  $220 = HEAP32[$db>>2]|0;
  $221 = HEAP32[$p>>2]|0;
  $222 = HEAP32[$221>>2]|0;
  HEAP32[$vararg_buffer22>>2] = $222;
  $223 = (_sqlite3MPrintf($220,26864,$vararg_buffer22)|0);
  _sqlite3VdbeAddParseSchemaOp($218,$219,$223);
 }
 $224 = HEAP32[$db>>2]|0;
 $225 = (($224) + 140|0);
 $226 = (($225) + 5|0);
 $227 = HEAP8[$226>>0]|0;
 $228 = ($227<<24>>24)!=(0);
 if (!($228)) {
  STACKTOP = sp;return;
 }
 $229 = HEAP32[$p>>2]|0;
 $230 = (($229) + 68|0);
 $231 = HEAP32[$230>>2]|0;
 HEAP32[$pSchema>>2] = $231;
 $232 = HEAP32[$pSchema>>2]|0;
 $233 = (($232) + 8|0);
 $234 = HEAP32[$p>>2]|0;
 $235 = HEAP32[$234>>2]|0;
 $236 = HEAP32[$p>>2]|0;
 $237 = (_sqlite3HashInsert($233,$235,$236)|0);
 HEAP32[$pOld>>2] = $237;
 $238 = HEAP32[$pOld>>2]|0;
 $239 = ($238|0)!=(0|0);
 if ($239) {
  $240 = HEAP32[$db>>2]|0;
  $241 = (($240) + 64|0);
  HEAP8[$241>>0] = 1;
  STACKTOP = sp;return;
 }
 $242 = HEAP32[$0>>2]|0;
 $243 = (($242) + 484|0);
 HEAP32[$243>>2] = 0;
 $244 = HEAP32[$db>>2]|0;
 $245 = (($244) + 24|0);
 $246 = HEAP32[$245>>2]|0;
 $247 = $246 | 2;
 HEAP32[$245>>2] = $247;
 $248 = HEAP32[$p>>2]|0;
 $249 = (($248) + 12|0);
 $250 = HEAP32[$249>>2]|0;
 $251 = ($250|0)!=(0|0);
 if ($251) {
  STACKTOP = sp;return;
 }
 $252 = HEAP32[$0>>2]|0;
 $253 = (($252) + 496|0);
 $254 = HEAP32[$253>>2]|0;
 HEAP32[$zName>>2] = $254;
 $255 = HEAP32[$1>>2]|0;
 $256 = HEAP32[$255>>2]|0;
 $257 = ($256|0)==(0|0);
 if ($257) {
  $258 = HEAP32[$2>>2]|0;
  HEAP32[$1>>2] = $258;
 }
 $259 = HEAP32[$1>>2]|0;
 $260 = HEAP32[$259>>2]|0;
 $261 = HEAP32[$zName>>2]|0;
 $262 = $260;
 $263 = $261;
 $264 = (($262) - ($263))|0;
 HEAP32[$nName>>2] = $264;
 $265 = HEAP32[$zName>>2]|0;
 $266 = HEAP32[$nName>>2]|0;
 $267 = (_sqlite3Utf8CharLen($265,$266)|0);
 $268 = (13 + ($267))|0;
 $269 = HEAP32[$p>>2]|0;
 $270 = (($269) + 48|0);
 HEAP32[$270>>2] = $268;
 STACKTOP = sp;return;
}
function _sqlite3AddColumn($pParse,$pName) {
 $pParse = $pParse|0;
 $pName = $pName|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0;
 var $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0;
 var $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0;
 var $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0;
 var $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0;
 var $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $aNew = 0, $db = 0, $i = 0, $p = 0, $pCol = 0, $vararg_buffer1 = 0, $z = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $vararg_buffer1 = sp;
 $0 = sp + 32|0;
 $1 = sp + 28|0;
 $p = sp + 24|0;
 $i = sp + 20|0;
 $z = sp + 16|0;
 $pCol = sp + 12|0;
 $db = sp + 8|0;
 $aNew = sp + 4|0;
 HEAP32[$0>>2] = $pParse;
 HEAP32[$1>>2] = $pName;
 $2 = HEAP32[$0>>2]|0;
 $3 = HEAP32[$2>>2]|0;
 HEAP32[$db>>2] = $3;
 $4 = HEAP32[$0>>2]|0;
 $5 = (($4) + 484|0);
 $6 = HEAP32[$5>>2]|0;
 HEAP32[$p>>2] = $6;
 $7 = ($6|0)==(0|0);
 if ($7) {
  STACKTOP = sp;return;
 }
 $8 = HEAP32[$p>>2]|0;
 $9 = (($8) + 38|0);
 $10 = HEAP16[$9>>1]|0;
 $11 = $10 << 16 >> 16;
 $12 = (($11) + 1)|0;
 $13 = HEAP32[$db>>2]|0;
 $14 = (($13) + 88|0);
 $15 = (($14) + 8|0);
 $16 = HEAP32[$15>>2]|0;
 $17 = ($12|0)>($16|0);
 if ($17) {
  $18 = HEAP32[$0>>2]|0;
  $19 = HEAP32[$p>>2]|0;
  $20 = HEAP32[$19>>2]|0;
  HEAP32[$vararg_buffer1>>2] = $20;
  _sqlite3ErrorMsg($18,26552,$vararg_buffer1);
  STACKTOP = sp;return;
 }
 $21 = HEAP32[$db>>2]|0;
 $22 = HEAP32[$1>>2]|0;
 $23 = (_sqlite3NameFromToken($21,$22)|0);
 HEAP32[$z>>2] = $23;
 $24 = HEAP32[$z>>2]|0;
 $25 = ($24|0)==(0|0);
 if ($25) {
  STACKTOP = sp;return;
 }
 HEAP32[$i>>2] = 0;
 while(1) {
  $26 = HEAP32[$i>>2]|0;
  $27 = HEAP32[$p>>2]|0;
  $28 = (($27) + 38|0);
  $29 = HEAP16[$28>>1]|0;
  $30 = $29 << 16 >> 16;
  $31 = ($26|0)<($30|0);
  if (!($31)) {
   break;
  }
  $32 = HEAP32[$z>>2]|0;
  $33 = HEAP8[$32>>0]|0;
  $34 = $33&255;
  $35 = (440 + ($34)|0);
  $36 = HEAP8[$35>>0]|0;
  $37 = $36&255;
  $38 = HEAP32[$i>>2]|0;
  $39 = HEAP32[$p>>2]|0;
  $40 = (($39) + 4|0);
  $41 = HEAP32[$40>>2]|0;
  $42 = (($41) + (($38*24)|0)|0);
  $43 = HEAP32[$42>>2]|0;
  $44 = HEAP8[$43>>0]|0;
  $45 = $44&255;
  $46 = (440 + ($45)|0);
  $47 = HEAP8[$46>>0]|0;
  $48 = $47&255;
  $49 = ($37|0)==($48|0);
  if ($49) {
   $50 = HEAP32[$z>>2]|0;
   $51 = (($50) + 1|0);
   $52 = HEAP32[$i>>2]|0;
   $53 = HEAP32[$p>>2]|0;
   $54 = (($53) + 4|0);
   $55 = HEAP32[$54>>2]|0;
   $56 = (($55) + (($52*24)|0)|0);
   $57 = HEAP32[$56>>2]|0;
   $58 = (($57) + 1|0);
   $59 = (_sqlite3_stricmp($51,$58)|0);
   $60 = ($59|0)==(0);
   if ($60) {
    label = 9;
    break;
   }
  }
  $65 = HEAP32[$i>>2]|0;
  $66 = (($65) + 1)|0;
  HEAP32[$i>>2] = $66;
 }
 if ((label|0) == 9) {
  $61 = HEAP32[$0>>2]|0;
  $62 = HEAP32[$z>>2]|0;
  HEAP32[$vararg_buffer1>>2] = $62;
  _sqlite3ErrorMsg($61,26576,$vararg_buffer1);
  $63 = HEAP32[$db>>2]|0;
  $64 = HEAP32[$z>>2]|0;
  _sqlite3DbFree($63,$64);
  STACKTOP = sp;return;
 }
 $67 = HEAP32[$p>>2]|0;
 $68 = (($67) + 38|0);
 $69 = HEAP16[$68>>1]|0;
 $70 = $69 << 16 >> 16;
 $71 = $70 & 7;
 $72 = ($71|0)==(0);
 do {
  if ($72) {
   $73 = HEAP32[$db>>2]|0;
   $74 = HEAP32[$p>>2]|0;
   $75 = (($74) + 4|0);
   $76 = HEAP32[$75>>2]|0;
   $77 = HEAP32[$p>>2]|0;
   $78 = (($77) + 38|0);
   $79 = HEAP16[$78>>1]|0;
   $80 = $79 << 16 >> 16;
   $81 = (($80) + 8)|0;
   $82 = ($81*24)|0;
   $83 = (_sqlite3DbRealloc($73,$76,$82,0)|0);
   HEAP32[$aNew>>2] = $83;
   $84 = HEAP32[$aNew>>2]|0;
   $85 = ($84|0)==(0|0);
   if (!($85)) {
    $88 = HEAP32[$aNew>>2]|0;
    $89 = HEAP32[$p>>2]|0;
    $90 = (($89) + 4|0);
    HEAP32[$90>>2] = $88;
    break;
   }
   $86 = HEAP32[$db>>2]|0;
   $87 = HEAP32[$z>>2]|0;
   _sqlite3DbFree($86,$87);
   STACKTOP = sp;return;
  }
 } while(0);
 $91 = HEAP32[$p>>2]|0;
 $92 = (($91) + 38|0);
 $93 = HEAP16[$92>>1]|0;
 $94 = $93 << 16 >> 16;
 $95 = HEAP32[$p>>2]|0;
 $96 = (($95) + 4|0);
 $97 = HEAP32[$96>>2]|0;
 $98 = (($97) + (($94*24)|0)|0);
 HEAP32[$pCol>>2] = $98;
 $99 = HEAP32[$pCol>>2]|0;
 ;HEAP32[$99+0>>2]=0|0;HEAP32[$99+4>>2]=0|0;HEAP32[$99+8>>2]=0|0;HEAP32[$99+12>>2]=0|0;HEAP32[$99+16>>2]=0|0;HEAP32[$99+20>>2]=0|0;
 $100 = HEAP32[$z>>2]|0;
 $101 = HEAP32[$pCol>>2]|0;
 HEAP32[$101>>2] = $100;
 $102 = HEAP32[$pCol>>2]|0;
 $103 = (($102) + 21|0);
 HEAP8[$103>>0] = 65;
 $104 = HEAP32[$pCol>>2]|0;
 $105 = (($104) + 22|0);
 HEAP8[$105>>0] = 1;
 $106 = HEAP32[$p>>2]|0;
 $107 = (($106) + 38|0);
 $108 = HEAP16[$107>>1]|0;
 $109 = (($108) + 1)<<16>>16;
 HEAP16[$107>>1] = $109;
 STACKTOP = sp;return;
}
function _sqlite3AddColumnType($pParse,$pType) {
 $pParse = $pParse|0;
 $pType = $pType|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $p = 0, $pCol = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $1 = sp + 8|0;
 $p = sp + 4|0;
 $pCol = sp;
 HEAP32[$0>>2] = $pParse;
 HEAP32[$1>>2] = $pType;
 $2 = HEAP32[$0>>2]|0;
 $3 = (($2) + 484|0);
 $4 = HEAP32[$3>>2]|0;
 HEAP32[$p>>2] = $4;
 $5 = HEAP32[$p>>2]|0;
 $6 = ($5|0)==(0|0);
 if ($6) {
  STACKTOP = sp;return;
 }
 $7 = HEAP32[$p>>2]|0;
 $8 = (($7) + 38|0);
 $9 = HEAP16[$8>>1]|0;
 $10 = $9 << 16 >> 16;
 $11 = ($10|0)<(1);
 if ($11) {
  STACKTOP = sp;return;
 }
 $12 = HEAP32[$p>>2]|0;
 $13 = (($12) + 38|0);
 $14 = HEAP16[$13>>1]|0;
 $15 = $14 << 16 >> 16;
 $16 = (($15) - 1)|0;
 $17 = HEAP32[$p>>2]|0;
 $18 = (($17) + 4|0);
 $19 = HEAP32[$18>>2]|0;
 $20 = (($19) + (($16*24)|0)|0);
 HEAP32[$pCol>>2] = $20;
 $21 = HEAP32[$0>>2]|0;
 $22 = HEAP32[$21>>2]|0;
 $23 = HEAP32[$1>>2]|0;
 $24 = (_sqlite3NameFromToken($22,$23)|0);
 $25 = HEAP32[$pCol>>2]|0;
 $26 = (($25) + 12|0);
 HEAP32[$26>>2] = $24;
 $27 = HEAP32[$pCol>>2]|0;
 $28 = (($27) + 12|0);
 $29 = HEAP32[$28>>2]|0;
 $30 = HEAP32[$pCol>>2]|0;
 $31 = (($30) + 22|0);
 $32 = (_sqlite3AffinityType($29,$31)|0);
 $33 = HEAP32[$pCol>>2]|0;
 $34 = (($33) + 21|0);
 HEAP8[$34>>0] = $32;
 STACKTOP = sp;return;
}
function _sqlite3AddDefaultValue($pParse,$pSpan) {
 $pParse = $pParse|0;
 $pSpan = $pSpan|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $7 = 0, $8 = 0, $9 = 0, $db = 0, $p = 0, $pCol = 0, $vararg_buffer = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $vararg_buffer = sp;
 $0 = sp + 20|0;
 $1 = sp + 16|0;
 $p = sp + 12|0;
 $pCol = sp + 8|0;
 $db = sp + 4|0;
 HEAP32[$0>>2] = $pParse;
 HEAP32[$1>>2] = $pSpan;
 $2 = HEAP32[$0>>2]|0;
 $3 = HEAP32[$2>>2]|0;
 HEAP32[$db>>2] = $3;
 $4 = HEAP32[$0>>2]|0;
 $5 = (($4) + 484|0);
 $6 = HEAP32[$5>>2]|0;
 HEAP32[$p>>2] = $6;
 $7 = HEAP32[$p>>2]|0;
 $8 = ($7|0)!=(0|0);
 if (!($8)) {
  $61 = HEAP32[$db>>2]|0;
  $62 = HEAP32[$1>>2]|0;
  $63 = HEAP32[$62>>2]|0;
  _sqlite3ExprDelete($61,$63);
  STACKTOP = sp;return;
 }
 $9 = HEAP32[$p>>2]|0;
 $10 = (($9) + 38|0);
 $11 = HEAP16[$10>>1]|0;
 $12 = $11 << 16 >> 16;
 $13 = (($12) - 1)|0;
 $14 = HEAP32[$p>>2]|0;
 $15 = (($14) + 4|0);
 $16 = HEAP32[$15>>2]|0;
 $17 = (($16) + (($13*24)|0)|0);
 HEAP32[$pCol>>2] = $17;
 $18 = HEAP32[$1>>2]|0;
 $19 = HEAP32[$18>>2]|0;
 $20 = HEAP32[$db>>2]|0;
 $21 = (($20) + 140|0);
 $22 = (($21) + 5|0);
 $23 = HEAP8[$22>>0]|0;
 $24 = (_sqlite3ExprIsConstantOrFunction($19,$23)|0);
 $25 = ($24|0)!=(0);
 if ($25) {
  $29 = HEAP32[$db>>2]|0;
  $30 = HEAP32[$pCol>>2]|0;
  $31 = (($30) + 4|0);
  $32 = HEAP32[$31>>2]|0;
  _sqlite3ExprDelete($29,$32);
  $33 = HEAP32[$db>>2]|0;
  $34 = HEAP32[$1>>2]|0;
  $35 = HEAP32[$34>>2]|0;
  $36 = (_sqlite3ExprDup($33,$35,1)|0);
  $37 = HEAP32[$pCol>>2]|0;
  $38 = (($37) + 4|0);
  HEAP32[$38>>2] = $36;
  $39 = HEAP32[$db>>2]|0;
  $40 = HEAP32[$pCol>>2]|0;
  $41 = (($40) + 8|0);
  $42 = HEAP32[$41>>2]|0;
  _sqlite3DbFree($39,$42);
  $43 = HEAP32[$db>>2]|0;
  $44 = HEAP32[$1>>2]|0;
  $45 = (($44) + 4|0);
  $46 = HEAP32[$45>>2]|0;
  $47 = HEAP32[$1>>2]|0;
  $48 = (($47) + 8|0);
  $49 = HEAP32[$48>>2]|0;
  $50 = HEAP32[$1>>2]|0;
  $51 = (($50) + 4|0);
  $52 = HEAP32[$51>>2]|0;
  $53 = $49;
  $54 = $52;
  $55 = (($53) - ($54))|0;
  $56 = ($55|0)<(0);
  $57 = $56 << 31 >> 31;
  $58 = (_sqlite3DbStrNDup($43,$46,$55,$57)|0);
  $59 = HEAP32[$pCol>>2]|0;
  $60 = (($59) + 8|0);
  HEAP32[$60>>2] = $58;
  $61 = HEAP32[$db>>2]|0;
  $62 = HEAP32[$1>>2]|0;
  $63 = HEAP32[$62>>2]|0;
  _sqlite3ExprDelete($61,$63);
  STACKTOP = sp;return;
 } else {
  $26 = HEAP32[$0>>2]|0;
  $27 = HEAP32[$pCol>>2]|0;
  $28 = HEAP32[$27>>2]|0;
  HEAP32[$vararg_buffer>>2] = $28;
  _sqlite3ErrorMsg($26,26504,$vararg_buffer);
  $61 = HEAP32[$db>>2]|0;
  $62 = HEAP32[$1>>2]|0;
  $63 = HEAP32[$62>>2]|0;
  _sqlite3ExprDelete($61,$63);
  STACKTOP = sp;return;
 }
}
function _sqlite3PExpr($pParse,$op,$pLeft,$pRight,$pToken) {
 $pParse = $pParse|0;
 $op = $op|0;
 $pLeft = $pLeft|0;
 $pRight = $pRight|0;
 $pToken = $pToken|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $or$cond = 0, $or$cond3 = 0, $p = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 20|0;
 $1 = sp + 16|0;
 $2 = sp + 12|0;
 $3 = sp + 8|0;
 $4 = sp + 4|0;
 $p = sp;
 HEAP32[$0>>2] = $pParse;
 HEAP32[$1>>2] = $op;
 HEAP32[$2>>2] = $pLeft;
 HEAP32[$3>>2] = $pRight;
 HEAP32[$4>>2] = $pToken;
 $5 = HEAP32[$1>>2]|0;
 $6 = ($5|0)==(72);
 $7 = HEAP32[$2>>2]|0;
 $8 = ($7|0)!=(0|0);
 $or$cond = $6 & $8;
 $9 = HEAP32[$3>>2]|0;
 $10 = ($9|0)!=(0|0);
 $or$cond3 = $or$cond & $10;
 $11 = HEAP32[$0>>2]|0;
 $12 = HEAP32[$11>>2]|0;
 if ($or$cond3) {
  $13 = HEAP32[$2>>2]|0;
  $14 = HEAP32[$3>>2]|0;
  $15 = (_sqlite3ExprAnd($12,$13,$14)|0);
  HEAP32[$p>>2] = $15;
 } else {
  $16 = HEAP32[$1>>2]|0;
  $17 = HEAP32[$4>>2]|0;
  $18 = (_sqlite3ExprAlloc($12,$16,$17,1)|0);
  HEAP32[$p>>2] = $18;
  $19 = HEAP32[$0>>2]|0;
  $20 = HEAP32[$19>>2]|0;
  $21 = HEAP32[$p>>2]|0;
  $22 = HEAP32[$2>>2]|0;
  $23 = HEAP32[$3>>2]|0;
  _sqlite3ExprAttachSubtrees($20,$21,$22,$23);
 }
 $24 = HEAP32[$p>>2]|0;
 $25 = ($24|0)!=(0|0);
 if (!($25)) {
  $30 = HEAP32[$p>>2]|0;
  STACKTOP = sp;return ($30|0);
 }
 $26 = HEAP32[$0>>2]|0;
 $27 = HEAP32[$p>>2]|0;
 $28 = (($27) + 24|0);
 $29 = HEAP32[$28>>2]|0;
 (_sqlite3ExprCheckHeight($26,$29)|0);
 $30 = HEAP32[$p>>2]|0;
 STACKTOP = sp;return ($30|0);
}
function _spanExpr($pOut,$pParse,$op,$pValue) {
 $pOut = $pOut|0;
 $pParse = $pParse|0;
 $op = $op|0;
 $pValue = $pValue|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0;
 var $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $1 = sp + 8|0;
 $2 = sp + 4|0;
 $3 = sp;
 HEAP32[$0>>2] = $pOut;
 HEAP32[$1>>2] = $pParse;
 HEAP32[$2>>2] = $op;
 HEAP32[$3>>2] = $pValue;
 $4 = HEAP32[$1>>2]|0;
 $5 = HEAP32[$2>>2]|0;
 $6 = HEAP32[$3>>2]|0;
 $7 = (_sqlite3PExpr($4,$5,0,0,$6)|0);
 $8 = HEAP32[$0>>2]|0;
 HEAP32[$8>>2] = $7;
 $9 = HEAP32[$3>>2]|0;
 $10 = HEAP32[$9>>2]|0;
 $11 = HEAP32[$0>>2]|0;
 $12 = (($11) + 4|0);
 HEAP32[$12>>2] = $10;
 $13 = HEAP32[$3>>2]|0;
 $14 = (($13) + 4|0);
 $15 = HEAP32[$14>>2]|0;
 $16 = HEAP32[$3>>2]|0;
 $17 = HEAP32[$16>>2]|0;
 $18 = (($17) + ($15)|0);
 $19 = HEAP32[$0>>2]|0;
 $20 = (($19) + 8|0);
 HEAP32[$20>>2] = $18;
 STACKTOP = sp;return;
}
function _sqlite3AddNotNull($pParse,$onError) {
 $pParse = $pParse|0;
 $onError = $onError|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $3 = 0, $4 = 0, $5 = 0;
 var $6 = 0, $7 = 0, $8 = 0, $9 = 0, $p = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $1 = sp + 4|0;
 $p = sp;
 HEAP32[$0>>2] = $pParse;
 HEAP32[$1>>2] = $onError;
 $2 = HEAP32[$0>>2]|0;
 $3 = (($2) + 484|0);
 $4 = HEAP32[$3>>2]|0;
 HEAP32[$p>>2] = $4;
 $5 = HEAP32[$p>>2]|0;
 $6 = ($5|0)==(0|0);
 if ($6) {
  STACKTOP = sp;return;
 }
 $7 = HEAP32[$p>>2]|0;
 $8 = (($7) + 38|0);
 $9 = HEAP16[$8>>1]|0;
 $10 = $9 << 16 >> 16;
 $11 = ($10|0)<(1);
 if ($11) {
  STACKTOP = sp;return;
 }
 $12 = HEAP32[$1>>2]|0;
 $13 = $12&255;
 $14 = HEAP32[$p>>2]|0;
 $15 = (($14) + 38|0);
 $16 = HEAP16[$15>>1]|0;
 $17 = $16 << 16 >> 16;
 $18 = (($17) - 1)|0;
 $19 = HEAP32[$p>>2]|0;
 $20 = (($19) + 4|0);
 $21 = HEAP32[$20>>2]|0;
 $22 = (($21) + (($18*24)|0)|0);
 $23 = (($22) + 20|0);
 HEAP8[$23>>0] = $13;
 STACKTOP = sp;return;
}
function _sqlite3AddPrimaryKey($pParse,$pList,$onError,$autoInc,$sortOrder) {
 $pParse = $pParse|0;
 $pList = $pList|0;
 $onError = $onError|0;
 $autoInc = $autoInc|0;
 $sortOrder = $sortOrder|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0;
 var $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0;
 var $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0;
 var $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0;
 var $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0;
 var $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $i = 0, $iCol = 0, $nTerm = 0, $or$cond = 0, $or$cond3 = 0, $p = 0, $pTab = 0, $v = 0, $vararg_buffer4 = 0, $zType = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 64|0;
 $vararg_buffer4 = sp;
 $0 = sp + 48|0;
 $1 = sp + 44|0;
 $2 = sp + 40|0;
 $3 = sp + 36|0;
 $4 = sp + 32|0;
 $pTab = sp + 28|0;
 $zType = sp + 24|0;
 $iCol = sp + 20|0;
 $i = sp + 16|0;
 $nTerm = sp + 12|0;
 $v = sp + 8|0;
 $p = sp + 4|0;
 HEAP32[$0>>2] = $pParse;
 HEAP32[$1>>2] = $pList;
 HEAP32[$2>>2] = $onError;
 HEAP32[$3>>2] = $autoInc;
 HEAP32[$4>>2] = $sortOrder;
 $5 = HEAP32[$0>>2]|0;
 $6 = (($5) + 484|0);
 $7 = HEAP32[$6>>2]|0;
 HEAP32[$pTab>>2] = $7;
 HEAP32[$zType>>2] = 0;
 HEAP32[$iCol>>2] = -1;
 $8 = HEAP32[$pTab>>2]|0;
 $9 = ($8|0)==(0|0);
 if ($9) {
  $163 = HEAP32[$0>>2]|0;
  $164 = HEAP32[$163>>2]|0;
  $165 = HEAP32[$1>>2]|0;
  _sqlite3ExprListDelete($164,$165);
  STACKTOP = sp;return;
 }
 $10 = HEAP32[$0>>2]|0;
 $11 = (($10) + 451|0);
 $12 = HEAP8[$11>>0]|0;
 $13 = $12&255;
 $14 = ($13|0)!=(0);
 if ($14) {
  $163 = HEAP32[$0>>2]|0;
  $164 = HEAP32[$163>>2]|0;
  $165 = HEAP32[$1>>2]|0;
  _sqlite3ExprListDelete($164,$165);
  STACKTOP = sp;return;
 }
 $15 = HEAP32[$pTab>>2]|0;
 $16 = (($15) + 44|0);
 $17 = HEAP8[$16>>0]|0;
 $18 = $17&255;
 $19 = $18 & 4;
 $20 = ($19|0)!=(0);
 if ($20) {
  $21 = HEAP32[$0>>2]|0;
  $22 = HEAP32[$pTab>>2]|0;
  $23 = HEAP32[$22>>2]|0;
  HEAP32[$vararg_buffer4>>2] = $23;
  _sqlite3ErrorMsg($21,26400,$vararg_buffer4);
  $163 = HEAP32[$0>>2]|0;
  $164 = HEAP32[$163>>2]|0;
  $165 = HEAP32[$1>>2]|0;
  _sqlite3ExprListDelete($164,$165);
  STACKTOP = sp;return;
 }
 $24 = HEAP32[$pTab>>2]|0;
 $25 = (($24) + 44|0);
 $26 = HEAP8[$25>>0]|0;
 $27 = $26&255;
 $28 = $27 | 4;
 $29 = $28&255;
 HEAP8[$25>>0] = $29;
 $30 = HEAP32[$1>>2]|0;
 $31 = ($30|0)==(0|0);
 L11: do {
  if ($31) {
   $32 = HEAP32[$pTab>>2]|0;
   $33 = (($32) + 38|0);
   $34 = HEAP16[$33>>1]|0;
   $35 = $34 << 16 >> 16;
   $36 = (($35) - 1)|0;
   HEAP32[$iCol>>2] = $36;
   $37 = HEAP32[$iCol>>2]|0;
   $38 = HEAP32[$pTab>>2]|0;
   $39 = (($38) + 4|0);
   $40 = HEAP32[$39>>2]|0;
   $41 = (($40) + (($37*24)|0)|0);
   $42 = (($41) + 23|0);
   $43 = HEAP8[$42>>0]|0;
   $44 = $43&255;
   $45 = $44 | 1;
   $46 = $45&255;
   HEAP8[$42>>0] = $46;
   $47 = HEAP32[$iCol>>2]|0;
   $48 = HEAP32[$pTab>>2]|0;
   $49 = (($48) + 4|0);
   $50 = HEAP32[$49>>2]|0;
   $51 = (($50) + (($47*24)|0)|0);
   $52 = (($51) + 12|0);
   $53 = HEAP32[$52>>2]|0;
   HEAP32[$zType>>2] = $53;
   HEAP32[$nTerm>>2] = 1;
  } else {
   $54 = HEAP32[$1>>2]|0;
   $55 = HEAP32[$54>>2]|0;
   HEAP32[$nTerm>>2] = $55;
   HEAP32[$i>>2] = 0;
   while(1) {
    $56 = HEAP32[$i>>2]|0;
    $57 = HEAP32[$nTerm>>2]|0;
    $58 = ($56|0)<($57|0);
    if (!($58)) {
     break L11;
    }
    HEAP32[$iCol>>2] = 0;
    while(1) {
     $59 = HEAP32[$iCol>>2]|0;
     $60 = HEAP32[$pTab>>2]|0;
     $61 = (($60) + 38|0);
     $62 = HEAP16[$61>>1]|0;
     $63 = $62 << 16 >> 16;
     $64 = ($59|0)<($63|0);
     if (!($64)) {
      break;
     }
     $65 = HEAP32[$i>>2]|0;
     $66 = HEAP32[$1>>2]|0;
     $67 = (($66) + 4|0);
     $68 = HEAP32[$67>>2]|0;
     $69 = (($68) + (($65*20)|0)|0);
     $70 = (($69) + 4|0);
     $71 = HEAP32[$70>>2]|0;
     $72 = HEAP32[$iCol>>2]|0;
     $73 = HEAP32[$pTab>>2]|0;
     $74 = (($73) + 4|0);
     $75 = HEAP32[$74>>2]|0;
     $76 = (($75) + (($72*24)|0)|0);
     $77 = HEAP32[$76>>2]|0;
     $78 = (_sqlite3_stricmp($71,$77)|0);
     $79 = ($78|0)==(0);
     $80 = HEAP32[$iCol>>2]|0;
     if ($79) {
      label = 12;
      break;
     }
     $97 = (($80) + 1)|0;
     HEAP32[$iCol>>2] = $97;
    }
    if ((label|0) == 12) {
     label = 0;
     $81 = HEAP32[$pTab>>2]|0;
     $82 = (($81) + 4|0);
     $83 = HEAP32[$82>>2]|0;
     $84 = (($83) + (($80*24)|0)|0);
     $85 = (($84) + 23|0);
     $86 = HEAP8[$85>>0]|0;
     $87 = $86&255;
     $88 = $87 | 1;
     $89 = $88&255;
     HEAP8[$85>>0] = $89;
     $90 = HEAP32[$iCol>>2]|0;
     $91 = HEAP32[$pTab>>2]|0;
     $92 = (($91) + 4|0);
     $93 = HEAP32[$92>>2]|0;
     $94 = (($93) + (($90*24)|0)|0);
     $95 = (($94) + 12|0);
     $96 = HEAP32[$95>>2]|0;
     HEAP32[$zType>>2] = $96;
    }
    $98 = HEAP32[$i>>2]|0;
    $99 = (($98) + 1)|0;
    HEAP32[$i>>2] = $99;
   }
  }
 } while(0);
 $100 = HEAP32[$nTerm>>2]|0;
 $101 = ($100|0)==(1);
 $102 = HEAP32[$zType>>2]|0;
 $103 = ($102|0)!=(0|0);
 $or$cond = $101 & $103;
 if ($or$cond) {
  $104 = HEAP32[$zType>>2]|0;
  $105 = (_sqlite3_stricmp($104,16416)|0);
  $106 = ($105|0)==(0);
  $107 = HEAP32[$4>>2]|0;
  $108 = ($107|0)==(0);
  $or$cond3 = $106 & $108;
  if ($or$cond3) {
   $109 = HEAP32[$iCol>>2]|0;
   $110 = $109&65535;
   $111 = HEAP32[$pTab>>2]|0;
   $112 = (($111) + 36|0);
   HEAP16[$112>>1] = $110;
   $113 = HEAP32[$2>>2]|0;
   $114 = $113&255;
   $115 = HEAP32[$pTab>>2]|0;
   $116 = (($115) + 45|0);
   HEAP8[$116>>0] = $114;
   $117 = HEAP32[$3>>2]|0;
   $118 = $117<<3;
   $119 = HEAP32[$pTab>>2]|0;
   $120 = (($119) + 44|0);
   $121 = HEAP8[$120>>0]|0;
   $122 = $121&255;
   $123 = $122 | $118;
   $124 = $123&255;
   HEAP8[$120>>0] = $124;
   $125 = HEAP32[$1>>2]|0;
   $126 = ($125|0)!=(0|0);
   if (!($126)) {
    $163 = HEAP32[$0>>2]|0;
    $164 = HEAP32[$163>>2]|0;
    $165 = HEAP32[$1>>2]|0;
    _sqlite3ExprListDelete($164,$165);
    STACKTOP = sp;return;
   }
   $127 = HEAP32[$1>>2]|0;
   $128 = (($127) + 4|0);
   $129 = HEAP32[$128>>2]|0;
   $130 = (($129) + 12|0);
   $131 = HEAP8[$130>>0]|0;
   $132 = HEAP32[$0>>2]|0;
   $133 = (($132) + 448|0);
   HEAP8[$133>>0] = $131;
   $163 = HEAP32[$0>>2]|0;
   $164 = HEAP32[$163>>2]|0;
   $165 = HEAP32[$1>>2]|0;
   _sqlite3ExprListDelete($164,$165);
   STACKTOP = sp;return;
  }
 }
 $134 = HEAP32[$3>>2]|0;
 $135 = ($134|0)!=(0);
 $136 = HEAP32[$0>>2]|0;
 if ($135) {
  _sqlite3ErrorMsg($136,26448,$vararg_buffer4);
  $163 = HEAP32[$0>>2]|0;
  $164 = HEAP32[$163>>2]|0;
  $165 = HEAP32[$1>>2]|0;
  _sqlite3ExprListDelete($164,$165);
  STACKTOP = sp;return;
 }
 $137 = (($136) + 8|0);
 $138 = HEAP32[$137>>2]|0;
 HEAP32[$v>>2] = $138;
 $139 = HEAP32[$v>>2]|0;
 $140 = ($139|0)!=(0|0);
 if ($140) {
  $141 = HEAP32[$v>>2]|0;
  $142 = (_sqlite3VdbeAddOp0($141,154)|0);
  $143 = HEAP32[$0>>2]|0;
  $144 = (($143) + 420|0);
  HEAP32[$144>>2] = $142;
 }
 $145 = HEAP32[$0>>2]|0;
 $146 = HEAP32[$1>>2]|0;
 $147 = HEAP32[$2>>2]|0;
 $148 = HEAP32[$4>>2]|0;
 $149 = (_sqlite3CreateIndex($145,0,0,0,$146,$147,0,0,$148,0)|0);
 HEAP32[$p>>2] = $149;
 $150 = HEAP32[$p>>2]|0;
 $151 = ($150|0)!=(0|0);
 if ($151) {
  $152 = HEAP32[$p>>2]|0;
  $153 = (($152) + 55|0);
  $154 = HEAP8[$153>>0]|0;
  $155 = $154 & -4;
  $156 = $155 | 2;
  HEAP8[$153>>0] = $156;
  $157 = HEAP32[$v>>2]|0;
  $158 = ($157|0)!=(0|0);
  if ($158) {
   $159 = HEAP32[$v>>2]|0;
   $160 = HEAP32[$0>>2]|0;
   $161 = (($160) + 420|0);
   $162 = HEAP32[$161>>2]|0;
   _sqlite3VdbeJumpHere($159,$162);
  }
 }
 HEAP32[$1>>2] = 0;
 $163 = HEAP32[$0>>2]|0;
 $164 = HEAP32[$163>>2]|0;
 $165 = HEAP32[$1>>2]|0;
 _sqlite3ExprListDelete($164,$165);
 STACKTOP = sp;return;
}
function _sqlite3CreateIndex($pParse,$pName1,$pName2,$pTblName,$pList,$onError,$pStart,$pPIWhere,$sortOrder,$ifNotExist) {
 $pParse = $pParse|0;
 $pName1 = $pName1|0;
 $pName2 = $pName2|0;
 $pTblName = $pTblName|0;
 $pList = $pList|0;
 $onError = $onError|0;
 $pStart = $pStart|0;
 $pPIWhere = $pPIWhere|0;
 $sortOrder = $sortOrder|0;
 $ifNotExist = $ifNotExist|0;
 var $$ = 0, $$2 = 0, $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0;
 var $114 = 0, $115 = 0, $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0;
 var $132 = 0, $133 = 0, $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0;
 var $150 = 0, $151 = 0, $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0;
 var $169 = 0, $17 = 0, $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0;
 var $187 = 0, $188 = 0, $189 = 0, $19 = 0, $190 = 0, $191 = 0, $192 = 0, $193 = 0, $194 = 0, $195 = 0, $196 = 0, $197 = 0, $198 = 0, $199 = 0, $2 = 0, $20 = 0, $200 = 0, $201 = 0, $202 = 0, $203 = 0;
 var $204 = 0, $205 = 0, $206 = 0, $207 = 0, $208 = 0, $209 = 0, $21 = 0, $210 = 0, $211 = 0, $212 = 0, $213 = 0, $214 = 0, $215 = 0, $216 = 0, $217 = 0, $218 = 0, $219 = 0, $22 = 0, $220 = 0, $221 = 0;
 var $222 = 0, $223 = 0, $224 = 0, $225 = 0, $226 = 0, $227 = 0, $228 = 0, $229 = 0, $23 = 0, $230 = 0, $231 = 0, $232 = 0, $233 = 0, $234 = 0, $235 = 0, $236 = 0, $237 = 0, $238 = 0, $239 = 0, $24 = 0;
 var $240 = 0, $241 = 0, $242 = 0, $243 = 0, $244 = 0, $245 = 0, $246 = 0, $247 = 0, $248 = 0, $249 = 0, $25 = 0, $250 = 0, $251 = 0, $252 = 0, $253 = 0, $254 = 0, $255 = 0, $256 = 0, $257 = 0, $258 = 0;
 var $259 = 0, $26 = 0, $260 = 0, $261 = 0, $262 = 0, $263 = 0, $264 = 0, $265 = 0, $266 = 0, $267 = 0, $268 = 0, $269 = 0, $27 = 0, $270 = 0, $271 = 0, $272 = 0, $273 = 0, $274 = 0, $275 = 0, $276 = 0;
 var $277 = 0, $278 = 0, $279 = 0, $28 = 0, $280 = 0, $281 = 0, $282 = 0, $283 = 0, $284 = 0, $285 = 0, $286 = 0, $287 = 0, $288 = 0, $289 = 0, $29 = 0, $290 = 0, $291 = 0, $292 = 0, $293 = 0, $294 = 0;
 var $295 = 0, $296 = 0, $297 = 0, $298 = 0, $299 = 0, $3 = 0, $30 = 0, $300 = 0, $301 = 0, $302 = 0, $303 = 0, $304 = 0, $305 = 0, $306 = 0, $307 = 0, $308 = 0, $309 = 0, $31 = 0, $310 = 0, $311 = 0;
 var $312 = 0, $313 = 0, $314 = 0, $315 = 0, $316 = 0, $317 = 0, $318 = 0, $319 = 0, $32 = 0, $320 = 0, $321 = 0, $322 = 0, $323 = 0, $324 = 0, $325 = 0, $326 = 0, $327 = 0, $328 = 0, $329 = 0, $33 = 0;
 var $330 = 0, $331 = 0, $332 = 0, $333 = 0, $334 = 0, $335 = 0, $336 = 0, $337 = 0, $338 = 0, $339 = 0, $34 = 0, $340 = 0, $341 = 0, $342 = 0, $343 = 0, $344 = 0, $345 = 0, $346 = 0, $347 = 0, $348 = 0;
 var $349 = 0, $35 = 0, $350 = 0, $351 = 0, $352 = 0, $353 = 0, $354 = 0, $355 = 0, $356 = 0, $357 = 0, $358 = 0, $359 = 0, $36 = 0, $360 = 0, $361 = 0, $362 = 0, $363 = 0, $364 = 0, $365 = 0, $366 = 0;
 var $367 = 0, $368 = 0, $369 = 0, $37 = 0, $370 = 0, $371 = 0, $372 = 0, $373 = 0, $374 = 0, $375 = 0, $376 = 0, $377 = 0, $378 = 0, $379 = 0, $38 = 0, $380 = 0, $381 = 0, $382 = 0, $383 = 0, $384 = 0;
 var $385 = 0, $386 = 0, $387 = 0, $388 = 0, $389 = 0, $39 = 0, $390 = 0, $391 = 0, $392 = 0, $393 = 0, $394 = 0, $395 = 0, $396 = 0, $397 = 0, $398 = 0, $399 = 0, $4 = 0, $40 = 0, $400 = 0, $401 = 0;
 var $402 = 0, $403 = 0, $404 = 0, $405 = 0, $406 = 0, $407 = 0, $408 = 0, $409 = 0, $41 = 0, $410 = 0, $411 = 0, $412 = 0, $413 = 0, $414 = 0, $415 = 0, $416 = 0, $417 = 0, $418 = 0, $419 = 0, $42 = 0;
 var $420 = 0, $421 = 0, $422 = 0, $423 = 0, $424 = 0, $425 = 0, $426 = 0, $427 = 0, $428 = 0, $429 = 0, $43 = 0, $430 = 0, $431 = 0, $432 = 0, $433 = 0, $434 = 0, $435 = 0, $436 = 0, $437 = 0, $438 = 0;
 var $439 = 0, $44 = 0, $440 = 0, $441 = 0, $442 = 0, $443 = 0, $444 = 0, $445 = 0, $446 = 0, $447 = 0, $448 = 0, $449 = 0, $45 = 0, $450 = 0, $451 = 0, $452 = 0, $453 = 0, $454 = 0, $455 = 0, $456 = 0;
 var $457 = 0, $458 = 0, $459 = 0, $46 = 0, $460 = 0, $461 = 0, $462 = 0, $463 = 0, $464 = 0, $465 = 0, $466 = 0, $467 = 0, $468 = 0, $469 = 0, $47 = 0, $470 = 0, $471 = 0, $472 = 0, $473 = 0, $474 = 0;
 var $475 = 0, $476 = 0, $477 = 0, $478 = 0, $479 = 0, $48 = 0, $480 = 0, $481 = 0, $482 = 0, $483 = 0, $484 = 0, $485 = 0, $486 = 0, $487 = 0, $488 = 0, $489 = 0, $49 = 0, $490 = 0, $491 = 0, $492 = 0;
 var $493 = 0, $494 = 0, $495 = 0, $496 = 0, $497 = 0, $498 = 0, $499 = 0, $5 = 0, $50 = 0, $500 = 0, $501 = 0, $502 = 0, $503 = 0, $504 = 0, $505 = 0, $506 = 0, $507 = 0, $508 = 0, $509 = 0, $51 = 0;
 var $510 = 0, $511 = 0, $512 = 0, $513 = 0, $514 = 0, $515 = 0, $516 = 0, $517 = 0, $518 = 0, $519 = 0, $52 = 0, $520 = 0, $521 = 0, $522 = 0, $523 = 0, $524 = 0, $525 = 0, $526 = 0, $527 = 0, $528 = 0;
 var $529 = 0, $53 = 0, $530 = 0, $531 = 0, $532 = 0, $533 = 0, $534 = 0, $535 = 0, $536 = 0, $537 = 0, $538 = 0, $539 = 0, $54 = 0, $540 = 0, $541 = 0, $542 = 0, $543 = 0, $544 = 0, $545 = 0, $546 = 0;
 var $547 = 0, $548 = 0, $549 = 0, $55 = 0, $550 = 0, $551 = 0, $552 = 0, $553 = 0, $554 = 0, $555 = 0, $556 = 0, $557 = 0, $558 = 0, $559 = 0, $56 = 0, $560 = 0, $561 = 0, $562 = 0, $563 = 0, $564 = 0;
 var $565 = 0, $566 = 0, $567 = 0, $568 = 0, $569 = 0, $57 = 0, $570 = 0, $571 = 0, $572 = 0, $573 = 0, $574 = 0, $575 = 0, $576 = 0, $577 = 0, $578 = 0, $579 = 0, $58 = 0, $580 = 0, $581 = 0, $582 = 0;
 var $583 = 0, $584 = 0, $585 = 0, $586 = 0, $587 = 0, $588 = 0, $589 = 0, $59 = 0, $590 = 0, $591 = 0, $592 = 0, $593 = 0, $594 = 0, $595 = 0, $596 = 0, $597 = 0, $598 = 0, $599 = 0, $6 = 0, $60 = 0;
 var $600 = 0, $601 = 0, $602 = 0, $603 = 0, $604 = 0, $605 = 0, $606 = 0, $607 = 0, $608 = 0, $609 = 0, $61 = 0, $610 = 0, $611 = 0, $612 = 0, $613 = 0, $614 = 0, $615 = 0, $616 = 0, $617 = 0, $618 = 0;
 var $619 = 0, $62 = 0, $620 = 0, $621 = 0, $622 = 0, $623 = 0, $624 = 0, $625 = 0, $626 = 0, $627 = 0, $628 = 0, $629 = 0, $63 = 0, $630 = 0, $631 = 0, $632 = 0, $633 = 0, $634 = 0, $635 = 0, $636 = 0;
 var $637 = 0, $638 = 0, $639 = 0, $64 = 0, $640 = 0, $641 = 0, $642 = 0, $643 = 0, $644 = 0, $645 = 0, $646 = 0, $647 = 0, $648 = 0, $649 = 0, $65 = 0, $650 = 0, $651 = 0, $652 = 0, $653 = 0, $654 = 0;
 var $655 = 0, $656 = 0, $657 = 0, $658 = 0, $659 = 0, $66 = 0, $660 = 0, $661 = 0, $662 = 0, $663 = 0, $664 = 0, $665 = 0, $666 = 0, $667 = 0, $668 = 0, $669 = 0, $67 = 0, $670 = 0, $671 = 0, $672 = 0;
 var $673 = 0, $674 = 0, $675 = 0, $676 = 0, $677 = 0, $678 = 0, $679 = 0, $68 = 0, $680 = 0, $681 = 0, $682 = 0, $683 = 0, $684 = 0, $685 = 0, $686 = 0, $687 = 0, $688 = 0, $689 = 0, $69 = 0, $690 = 0;
 var $691 = 0, $692 = 0, $693 = 0, $694 = 0, $695 = 0, $696 = 0, $697 = 0, $698 = 0, $699 = 0, $7 = 0, $70 = 0, $700 = 0, $701 = 0, $702 = 0, $703 = 0, $704 = 0, $705 = 0, $706 = 0, $707 = 0, $708 = 0;
 var $709 = 0, $71 = 0, $710 = 0, $711 = 0, $712 = 0, $713 = 0, $714 = 0, $715 = 0, $716 = 0, $717 = 0, $718 = 0, $719 = 0, $72 = 0, $720 = 0, $721 = 0, $722 = 0, $723 = 0, $724 = 0, $725 = 0, $726 = 0;
 var $727 = 0, $728 = 0, $729 = 0, $73 = 0, $730 = 0, $731 = 0, $732 = 0, $733 = 0, $734 = 0, $735 = 0, $736 = 0, $737 = 0, $738 = 0, $739 = 0, $74 = 0, $740 = 0, $741 = 0, $742 = 0, $743 = 0, $744 = 0;
 var $745 = 0, $746 = 0, $747 = 0, $748 = 0, $749 = 0, $75 = 0, $750 = 0, $751 = 0, $752 = 0, $753 = 0, $754 = 0, $755 = 0, $756 = 0, $757 = 0, $758 = 0, $759 = 0, $76 = 0, $760 = 0, $761 = 0, $762 = 0;
 var $763 = 0, $764 = 0, $765 = 0, $766 = 0, $767 = 0, $768 = 0, $769 = 0, $77 = 0, $770 = 0, $771 = 0, $772 = 0, $773 = 0, $774 = 0, $775 = 0, $776 = 0, $777 = 0, $778 = 0, $779 = 0, $78 = 0, $780 = 0;
 var $781 = 0, $782 = 0, $783 = 0, $784 = 0, $785 = 0, $786 = 0, $787 = 0, $788 = 0, $789 = 0, $79 = 0, $790 = 0, $791 = 0, $792 = 0, $793 = 0, $794 = 0, $795 = 0, $796 = 0, $797 = 0, $798 = 0, $799 = 0;
 var $8 = 0, $80 = 0, $800 = 0, $801 = 0, $802 = 0, $803 = 0, $804 = 0, $805 = 0, $806 = 0, $807 = 0, $808 = 0, $809 = 0, $81 = 0, $810 = 0, $811 = 0, $812 = 0, $813 = 0, $814 = 0, $815 = 0, $816 = 0;
 var $817 = 0, $818 = 0, $819 = 0, $82 = 0, $820 = 0, $821 = 0, $822 = 0, $823 = 0, $824 = 0, $825 = 0, $826 = 0, $827 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0;
 var $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $db = 0, $i = 0, $iDb = 0, $iMem = 0, $j = 0, $k = 0, $n = 0, $n1 = 0, $nColl = 0, $nExtra = 0;
 var $nExtraCol = 0, $nName = 0, $or$cond = 0, $or$cond4 = 0, $or$cond6 = 0, $p = 0, $pDb = 0, $pExpr = 0, $pIdx = 0, $pIndex = 0, $pListItem = 0, $pLoop = 0, $pName = 0, $pOther = 0, $pPk = 0, $pRet = 0, $pTab = 0, $pTabCol = 0, $requestedSortOrder = 0, $sFix = 0;
 var $sortOrderMask = 0, $v = 0, $vararg_buffer44 = 0, $vararg_ptr23 = 0, $vararg_ptr27 = 0, $vararg_ptr34 = 0, $vararg_ptr35 = 0, $vararg_ptr39 = 0, $vararg_ptr40 = 0, $vararg_ptr41 = 0, $vararg_ptr42 = 0, $vararg_ptr43 = 0, $x = 0, $z1 = 0, $z2 = 0, $zColName = 0, $zColl = 0, $zDb = 0, $zExtra = 0, $zName = 0;
 var $zStmt = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 240|0;
 $vararg_buffer44 = sp;
 $0 = sp + 232|0;
 $1 = sp + 228|0;
 $2 = sp + 224|0;
 $3 = sp + 220|0;
 $4 = sp + 216|0;
 $5 = sp + 212|0;
 $6 = sp + 208|0;
 $7 = sp + 204|0;
 $8 = sp + 200|0;
 $9 = sp + 196|0;
 $pRet = sp + 192|0;
 $pTab = sp + 188|0;
 $pIndex = sp + 184|0;
 $zName = sp + 180|0;
 $nName = sp + 176|0;
 $i = sp + 172|0;
 $j = sp + 168|0;
 $sFix = sp + 144|0;
 $sortOrderMask = sp + 140|0;
 $db = sp + 136|0;
 $pDb = sp + 132|0;
 $iDb = sp + 128|0;
 $pName = sp + 124|0;
 $pListItem = sp + 120|0;
 $pTabCol = sp + 116|0;
 $nExtra = sp + 112|0;
 $nExtraCol = sp + 108|0;
 $zExtra = sp + 104|0;
 $pPk = sp + 100|0;
 $n = sp + 96|0;
 $pLoop = sp + 92|0;
 $zDb = sp + 88|0;
 $pExpr = sp + 84|0;
 $zColName = sp + 80|0;
 $requestedSortOrder = sp + 76|0;
 $zColl = sp + 72|0;
 $nColl = sp + 68|0;
 $x = sp + 64|0;
 $pIdx = sp + 60|0;
 $k = sp + 56|0;
 $z1 = sp + 52|0;
 $z2 = sp + 48|0;
 $p = sp + 44|0;
 $v = sp + 40|0;
 $zStmt = sp + 36|0;
 $iMem = sp + 32|0;
 $n1 = sp + 28|0;
 $pOther = sp + 24|0;
 HEAP32[$0>>2] = $pParse;
 HEAP32[$1>>2] = $pName1;
 HEAP32[$2>>2] = $pName2;
 HEAP32[$3>>2] = $pTblName;
 HEAP32[$4>>2] = $pList;
 HEAP32[$5>>2] = $onError;
 HEAP32[$6>>2] = $pStart;
 HEAP32[$7>>2] = $pPIWhere;
 HEAP32[$8>>2] = $sortOrder;
 HEAP32[$9>>2] = $ifNotExist;
 HEAP32[$pRet>>2] = 0;
 HEAP32[$pTab>>2] = 0;
 HEAP32[$pIndex>>2] = 0;
 HEAP32[$zName>>2] = 0;
 $10 = HEAP32[$0>>2]|0;
 $11 = HEAP32[$10>>2]|0;
 HEAP32[$db>>2] = $11;
 HEAP32[$pName>>2] = 0;
 HEAP32[$nExtra>>2] = 0;
 HEAP32[$zExtra>>2] = 0;
 HEAP32[$pPk>>2] = 0;
 $12 = HEAP32[$db>>2]|0;
 $13 = (($12) + 64|0);
 $14 = HEAP8[$13>>0]|0;
 $15 = $14&255;
 $16 = ($15|0)!=(0);
 L1: do {
  if (!($16)) {
   $17 = HEAP32[$0>>2]|0;
   $18 = (($17) + 451|0);
   $19 = HEAP8[$18>>0]|0;
   $20 = $19&255;
   $21 = ($20|0)!=(0);
   if (!($21)) {
    $22 = HEAP32[$0>>2]|0;
    $23 = (_sqlite3ReadSchema($22)|0);
    $24 = (0)!=($23|0);
    if (!($24)) {
     $25 = HEAP32[$3>>2]|0;
     $26 = ($25|0)!=(0|0);
     $27 = HEAP32[$0>>2]|0;
     if ($26) {
      $28 = HEAP32[$1>>2]|0;
      $29 = HEAP32[$2>>2]|0;
      $30 = (_sqlite3TwoPartName($27,$28,$29,$pName)|0);
      HEAP32[$iDb>>2] = $30;
      $31 = HEAP32[$iDb>>2]|0;
      $32 = ($31|0)<(0);
      if ($32) {
       break;
      }
      $33 = HEAP32[$db>>2]|0;
      $34 = (($33) + 140|0);
      $35 = (($34) + 5|0);
      $36 = HEAP8[$35>>0]|0;
      $37 = ($36<<24>>24)!=(0);
      if (!($37)) {
       $38 = HEAP32[$0>>2]|0;
       $39 = HEAP32[$3>>2]|0;
       $40 = (_sqlite3SrcListLookup($38,$39)|0);
       HEAP32[$pTab>>2] = $40;
       $41 = HEAP32[$2>>2]|0;
       $42 = (($41) + 4|0);
       $43 = HEAP32[$42>>2]|0;
       $44 = ($43|0)==(0);
       $45 = HEAP32[$pTab>>2]|0;
       $46 = ($45|0)!=(0|0);
       $or$cond = $44 & $46;
       if ($or$cond) {
        $47 = HEAP32[$pTab>>2]|0;
        $48 = (($47) + 68|0);
        $49 = HEAP32[$48>>2]|0;
        $50 = HEAP32[$db>>2]|0;
        $51 = (($50) + 16|0);
        $52 = HEAP32[$51>>2]|0;
        $53 = (($52) + 16|0);
        $54 = (($53) + 12|0);
        $55 = HEAP32[$54>>2]|0;
        $56 = ($49|0)==($55|0);
        if ($56) {
         HEAP32[$iDb>>2] = 1;
        }
       }
      }
      $57 = HEAP32[$0>>2]|0;
      $58 = HEAP32[$iDb>>2]|0;
      $59 = HEAP32[$pName>>2]|0;
      _sqlite3FixInit($sFix,$57,$58,10168,$59);
      $60 = HEAP32[$3>>2]|0;
      (_sqlite3FixSrcList($sFix,$60)|0);
      $61 = HEAP32[$0>>2]|0;
      $62 = HEAP32[$3>>2]|0;
      $63 = (($62) + 8|0);
      $64 = (_sqlite3LocateTableItem($61,0,$63)|0);
      HEAP32[$pTab>>2] = $64;
      $65 = HEAP32[$pTab>>2]|0;
      $66 = ($65|0)==(0|0);
      if ($66) {
       break;
      }
      $67 = HEAP32[$iDb>>2]|0;
      $68 = ($67|0)==(1);
      if ($68) {
       $69 = HEAP32[$iDb>>2]|0;
       $70 = HEAP32[$db>>2]|0;
       $71 = (($70) + 16|0);
       $72 = HEAP32[$71>>2]|0;
       $73 = (($72) + ($69<<4)|0);
       $74 = (($73) + 12|0);
       $75 = HEAP32[$74>>2]|0;
       $76 = HEAP32[$pTab>>2]|0;
       $77 = (($76) + 68|0);
       $78 = HEAP32[$77>>2]|0;
       $79 = ($75|0)!=($78|0);
       if ($79) {
        $80 = HEAP32[$0>>2]|0;
        $81 = HEAP32[$pTab>>2]|0;
        $82 = HEAP32[$81>>2]|0;
        HEAP32[$vararg_buffer44>>2] = $82;
        _sqlite3ErrorMsg($80,25944,$vararg_buffer44);
        break;
       }
      }
      $83 = HEAP32[$pTab>>2]|0;
      $84 = (($83) + 44|0);
      $85 = HEAP8[$84>>0]|0;
      $86 = $85&255;
      $87 = $86 & 32;
      $88 = ($87|0)==(0);
      if (!($88)) {
       $89 = HEAP32[$pTab>>2]|0;
       $90 = (_sqlite3PrimaryKeyIndex($89)|0);
       HEAP32[$pPk>>2] = $90;
      }
     } else {
      $91 = (($27) + 484|0);
      $92 = HEAP32[$91>>2]|0;
      HEAP32[$pTab>>2] = $92;
      $93 = HEAP32[$pTab>>2]|0;
      $94 = ($93|0)!=(0|0);
      if (!($94)) {
       break;
      }
      $95 = HEAP32[$db>>2]|0;
      $96 = HEAP32[$pTab>>2]|0;
      $97 = (($96) + 68|0);
      $98 = HEAP32[$97>>2]|0;
      $99 = (_sqlite3SchemaToIndex($95,$98)|0);
      HEAP32[$iDb>>2] = $99;
     }
     $100 = HEAP32[$iDb>>2]|0;
     $101 = HEAP32[$db>>2]|0;
     $102 = (($101) + 16|0);
     $103 = HEAP32[$102>>2]|0;
     $104 = (($103) + ($100<<4)|0);
     HEAP32[$pDb>>2] = $104;
     $105 = HEAP32[$pTab>>2]|0;
     $106 = HEAP32[$105>>2]|0;
     $107 = (_sqlite3_strnicmp($106,15144,7)|0);
     $108 = ($107|0)==(0);
     if ($108) {
      $109 = HEAP32[$db>>2]|0;
      $110 = (($109) + 140|0);
      $111 = (($110) + 5|0);
      $112 = HEAP8[$111>>0]|0;
      $113 = $112&255;
      $114 = ($113|0)==(0);
      if ($114) {
       $115 = HEAP32[$pTab>>2]|0;
       $116 = HEAP32[$115>>2]|0;
       $117 = (($116) + 7|0);
       $118 = (_sqlite3_strnicmp($117,26000,9)|0);
       $119 = ($118|0)!=(0);
       if ($119) {
        $120 = HEAP32[$0>>2]|0;
        $121 = HEAP32[$pTab>>2]|0;
        $122 = HEAP32[$121>>2]|0;
        HEAP32[$vararg_buffer44>>2] = $122;
        _sqlite3ErrorMsg($120,26016,$vararg_buffer44);
        break;
       }
      }
     }
     $123 = HEAP32[$pTab>>2]|0;
     $124 = (($123) + 12|0);
     $125 = HEAP32[$124>>2]|0;
     $126 = ($125|0)!=(0|0);
     if ($126) {
      $127 = HEAP32[$0>>2]|0;
      _sqlite3ErrorMsg($127,26048,$vararg_buffer44);
      break;
     }
     $128 = HEAP32[$pTab>>2]|0;
     $129 = (($128) + 44|0);
     $130 = HEAP8[$129>>0]|0;
     $131 = $130&255;
     $132 = $131 & 16;
     $133 = ($132|0)!=(0);
     if ($133) {
      $134 = HEAP32[$0>>2]|0;
      _sqlite3ErrorMsg($134,26080,$vararg_buffer44);
      break;
     }
     $135 = HEAP32[$pName>>2]|0;
     $136 = ($135|0)!=(0|0);
     if ($136) {
      $137 = HEAP32[$db>>2]|0;
      $138 = HEAP32[$pName>>2]|0;
      $139 = (_sqlite3NameFromToken($137,$138)|0);
      HEAP32[$zName>>2] = $139;
      $140 = HEAP32[$zName>>2]|0;
      $141 = ($140|0)==(0|0);
      if ($141) {
       break;
      }
      $142 = HEAP32[$0>>2]|0;
      $143 = HEAP32[$zName>>2]|0;
      $144 = (_sqlite3CheckObjectName($142,$143)|0);
      $145 = (0)!=($144|0);
      if ($145) {
       break;
      }
      $146 = HEAP32[$db>>2]|0;
      $147 = (($146) + 140|0);
      $148 = (($147) + 5|0);
      $149 = HEAP8[$148>>0]|0;
      $150 = ($149<<24>>24)!=(0);
      if (!($150)) {
       $151 = HEAP32[$db>>2]|0;
       $152 = HEAP32[$zName>>2]|0;
       $153 = (_sqlite3FindTable($151,$152,0)|0);
       $154 = ($153|0)!=(0|0);
       if ($154) {
        $155 = HEAP32[$0>>2]|0;
        $156 = HEAP32[$zName>>2]|0;
        HEAP32[$vararg_buffer44>>2] = $156;
        _sqlite3ErrorMsg($155,26120,$vararg_buffer44);
        break;
       }
      }
      $157 = HEAP32[$db>>2]|0;
      $158 = HEAP32[$zName>>2]|0;
      $159 = HEAP32[$pDb>>2]|0;
      $160 = HEAP32[$159>>2]|0;
      $161 = (_sqlite3FindIndex($157,$158,$160)|0);
      $162 = ($161|0)!=(0|0);
      if ($162) {
       $163 = HEAP32[$9>>2]|0;
       $164 = ($163|0)!=(0);
       $165 = HEAP32[$0>>2]|0;
       if ($164) {
        $167 = HEAP32[$iDb>>2]|0;
        _sqlite3CodeVerifySchema($165,$167);
        break;
       } else {
        $166 = HEAP32[$zName>>2]|0;
        HEAP32[$vararg_buffer44>>2] = $166;
        _sqlite3ErrorMsg($165,26160,$vararg_buffer44);
        break;
       }
      }
     } else {
      $168 = HEAP32[$pTab>>2]|0;
      $169 = (($168) + 8|0);
      $170 = HEAP32[$169>>2]|0;
      HEAP32[$pLoop>>2] = $170;
      HEAP32[$n>>2] = 1;
      while(1) {
       $171 = HEAP32[$pLoop>>2]|0;
       $172 = ($171|0)!=(0|0);
       if (!($172)) {
        break;
       }
       $173 = HEAP32[$pLoop>>2]|0;
       $174 = (($173) + 20|0);
       $175 = HEAP32[$174>>2]|0;
       HEAP32[$pLoop>>2] = $175;
       $176 = HEAP32[$n>>2]|0;
       $177 = (($176) + 1)|0;
       HEAP32[$n>>2] = $177;
      }
      $178 = HEAP32[$db>>2]|0;
      $179 = HEAP32[$pTab>>2]|0;
      $180 = HEAP32[$179>>2]|0;
      $181 = HEAP32[$n>>2]|0;
      HEAP32[$vararg_buffer44>>2] = $180;
      $vararg_ptr23 = (($vararg_buffer44) + 4|0);
      HEAP32[$vararg_ptr23>>2] = $181;
      $182 = (_sqlite3MPrintf($178,26184,$vararg_buffer44)|0);
      HEAP32[$zName>>2] = $182;
      $183 = HEAP32[$zName>>2]|0;
      $184 = ($183|0)==(0|0);
      if ($184) {
       break;
      }
     }
     $185 = HEAP32[$pDb>>2]|0;
     $186 = HEAP32[$185>>2]|0;
     HEAP32[$zDb>>2] = $186;
     $187 = HEAP32[$0>>2]|0;
     $188 = HEAP32[$iDb>>2]|0;
     $189 = ($188|0)==(1);
     $190 = $189 ? 14904 : 14928;
     $191 = HEAP32[$zDb>>2]|0;
     $192 = (_sqlite3AuthCheck($187,18,$190,0,$191)|0);
     $193 = ($192|0)!=(0);
     if (!($193)) {
      HEAP32[$i>>2] = 1;
      $194 = HEAP32[$iDb>>2]|0;
      $195 = ($194|0)==(1);
      $$ = $195 ? 3 : 1;
      HEAP32[$i>>2] = $$;
      $196 = HEAP32[$0>>2]|0;
      $197 = HEAP32[$i>>2]|0;
      $198 = HEAP32[$zName>>2]|0;
      $199 = HEAP32[$pTab>>2]|0;
      $200 = HEAP32[$199>>2]|0;
      $201 = HEAP32[$zDb>>2]|0;
      $202 = (_sqlite3AuthCheck($196,$197,$198,$200,$201)|0);
      $203 = ($202|0)!=(0);
      if (!($203)) {
       $204 = HEAP32[$4>>2]|0;
       $205 = ($204|0)==(0|0);
       if ($205) {
        $206 = HEAP32[$0>>2]|0;
        $207 = (_sqlite3ExprListAppend($206,0,0)|0);
        HEAP32[$4>>2] = $207;
        $208 = HEAP32[$4>>2]|0;
        $209 = ($208|0)==(0|0);
        if ($209) {
         break;
        }
        $210 = HEAP32[$0>>2]|0;
        $211 = HEAP32[$210>>2]|0;
        $212 = HEAP32[$pTab>>2]|0;
        $213 = (($212) + 38|0);
        $214 = HEAP16[$213>>1]|0;
        $215 = $214 << 16 >> 16;
        $216 = (($215) - 1)|0;
        $217 = HEAP32[$pTab>>2]|0;
        $218 = (($217) + 4|0);
        $219 = HEAP32[$218>>2]|0;
        $220 = (($219) + (($216*24)|0)|0);
        $221 = HEAP32[$220>>2]|0;
        $222 = (_sqlite3DbStrDup($211,$221)|0);
        $223 = HEAP32[$4>>2]|0;
        $224 = (($223) + 4|0);
        $225 = HEAP32[$224>>2]|0;
        $226 = (($225) + 4|0);
        HEAP32[$226>>2] = $222;
        $227 = HEAP32[$8>>2]|0;
        $228 = $227&255;
        $229 = HEAP32[$4>>2]|0;
        $230 = (($229) + 4|0);
        $231 = HEAP32[$230>>2]|0;
        $232 = (($231) + 12|0);
        HEAP8[$232>>0] = $228;
       }
       HEAP32[$i>>2] = 0;
       while(1) {
        $233 = HEAP32[$i>>2]|0;
        $234 = HEAP32[$4>>2]|0;
        $235 = HEAP32[$234>>2]|0;
        $236 = ($233|0)<($235|0);
        if (!($236)) {
         break;
        }
        $237 = HEAP32[$i>>2]|0;
        $238 = HEAP32[$4>>2]|0;
        $239 = (($238) + 4|0);
        $240 = HEAP32[$239>>2]|0;
        $241 = (($240) + (($237*20)|0)|0);
        $242 = HEAP32[$241>>2]|0;
        HEAP32[$pExpr>>2] = $242;
        $243 = HEAP32[$pExpr>>2]|0;
        $244 = ($243|0)!=(0|0);
        if ($244) {
         $245 = HEAP32[$pExpr>>2]|0;
         $246 = (($245) + 8|0);
         $247 = HEAP32[$246>>2]|0;
         $248 = (_sqlite3Strlen30($247)|0);
         $249 = (1 + ($248))|0;
         $250 = HEAP32[$nExtra>>2]|0;
         $251 = (($250) + ($249))|0;
         HEAP32[$nExtra>>2] = $251;
        }
        $252 = HEAP32[$i>>2]|0;
        $253 = (($252) + 1)|0;
        HEAP32[$i>>2] = $253;
       }
       $254 = HEAP32[$zName>>2]|0;
       $255 = (_sqlite3Strlen30($254)|0);
       HEAP32[$nName>>2] = $255;
       $256 = HEAP32[$pPk>>2]|0;
       $257 = ($256|0)!=(0|0);
       if ($257) {
        $258 = HEAP32[$pPk>>2]|0;
        $259 = (($258) + 50|0);
        $260 = HEAP16[$259>>1]|0;
        $261 = $260&65535;
        $262 = $261;
       } else {
        $262 = 1;
       }
       HEAP32[$nExtraCol>>2] = $262;
       $263 = HEAP32[$db>>2]|0;
       $264 = HEAP32[$4>>2]|0;
       $265 = HEAP32[$264>>2]|0;
       $266 = HEAP32[$nExtraCol>>2]|0;
       $267 = (($265) + ($266))|0;
       $268 = $267&65535;
       $269 = HEAP32[$nName>>2]|0;
       $270 = HEAP32[$nExtra>>2]|0;
       $271 = (($269) + ($270))|0;
       $272 = (($271) + 1)|0;
       $273 = (_sqlite3AllocateIndexObject($263,$268,$272,$zExtra)|0);
       HEAP32[$pIndex>>2] = $273;
       $274 = HEAP32[$db>>2]|0;
       $275 = (($274) + 64|0);
       $276 = HEAP8[$275>>0]|0;
       $277 = ($276<<24>>24)!=(0);
       if (!($277)) {
        $278 = HEAP32[$zExtra>>2]|0;
        $279 = HEAP32[$pIndex>>2]|0;
        HEAP32[$279>>2] = $278;
        $280 = HEAP32[$nName>>2]|0;
        $281 = (($280) + 1)|0;
        $282 = HEAP32[$zExtra>>2]|0;
        $283 = (($282) + ($281)|0);
        HEAP32[$zExtra>>2] = $283;
        $284 = HEAP32[$pIndex>>2]|0;
        $285 = HEAP32[$284>>2]|0;
        $286 = HEAP32[$zName>>2]|0;
        $287 = HEAP32[$nName>>2]|0;
        $288 = (($287) + 1)|0;
        _memcpy(($285|0),($286|0),($288|0))|0;
        $289 = HEAP32[$pTab>>2]|0;
        $290 = HEAP32[$pIndex>>2]|0;
        $291 = (($290) + 12|0);
        HEAP32[$291>>2] = $289;
        $292 = HEAP32[$5>>2]|0;
        $293 = $292&255;
        $294 = HEAP32[$pIndex>>2]|0;
        $295 = (($294) + 54|0);
        HEAP8[$295>>0] = $293;
        $296 = HEAP32[$5>>2]|0;
        $297 = ($296|0)!=(0);
        $298 = $297&1;
        $299 = HEAP32[$pIndex>>2]|0;
        $300 = (($299) + 55|0);
        $301 = $298&255;
        $302 = HEAP8[$300>>0]|0;
        $303 = $301 & 1;
        $304 = ($303 << 3)&255;
        $305 = $302 & -9;
        $306 = $305 | $304;
        HEAP8[$300>>0] = $306;
        $307 = HEAP32[$pName>>2]|0;
        $308 = ($307|0)!=(0|0);
        $309 = $308 ? 0 : 1;
        $310 = HEAP32[$pIndex>>2]|0;
        $311 = (($310) + 55|0);
        $312 = $309&255;
        $313 = HEAP8[$311>>0]|0;
        $314 = $312 & 3;
        $315 = $313 & -4;
        $316 = $315 | $314;
        HEAP8[$311>>0] = $316;
        $317 = HEAP32[$iDb>>2]|0;
        $318 = HEAP32[$db>>2]|0;
        $319 = (($318) + 16|0);
        $320 = HEAP32[$319>>2]|0;
        $321 = (($320) + ($317<<4)|0);
        $322 = (($321) + 12|0);
        $323 = HEAP32[$322>>2]|0;
        $324 = HEAP32[$pIndex>>2]|0;
        $325 = (($324) + 24|0);
        HEAP32[$325>>2] = $323;
        $326 = HEAP32[$4>>2]|0;
        $327 = HEAP32[$326>>2]|0;
        $328 = $327&65535;
        $329 = HEAP32[$pIndex>>2]|0;
        $330 = (($329) + 50|0);
        HEAP16[$330>>1] = $328;
        $331 = HEAP32[$7>>2]|0;
        $332 = ($331|0)!=(0|0);
        if ($332) {
         $333 = HEAP32[$0>>2]|0;
         $334 = HEAP32[$pTab>>2]|0;
         $335 = HEAP32[$7>>2]|0;
         _sqlite3ResolveSelfReference($333,$334,16,$335,0);
         $336 = HEAP32[$7>>2]|0;
         $337 = HEAP32[$pIndex>>2]|0;
         $338 = (($337) + 36|0);
         HEAP32[$338>>2] = $336;
         HEAP32[$7>>2] = 0;
        }
        $339 = HEAP32[$pDb>>2]|0;
        $340 = (($339) + 12|0);
        $341 = HEAP32[$340>>2]|0;
        $342 = (($341) + 76|0);
        $343 = HEAP8[$342>>0]|0;
        $344 = $343&255;
        $345 = ($344|0)>=(4);
        if ($345) {
         HEAP32[$sortOrderMask>>2] = -1;
        } else {
         HEAP32[$sortOrderMask>>2] = 0;
        }
        HEAP32[$i>>2] = 0;
        $346 = HEAP32[$4>>2]|0;
        $347 = (($346) + 4|0);
        $348 = HEAP32[$347>>2]|0;
        HEAP32[$pListItem>>2] = $348;
        while(1) {
         $349 = HEAP32[$i>>2]|0;
         $350 = HEAP32[$4>>2]|0;
         $351 = HEAP32[$350>>2]|0;
         $352 = ($349|0)<($351|0);
         if (!($352)) {
          break;
         }
         $353 = HEAP32[$pListItem>>2]|0;
         $354 = (($353) + 4|0);
         $355 = HEAP32[$354>>2]|0;
         HEAP32[$zColName>>2] = $355;
         HEAP32[$j>>2] = 0;
         $356 = HEAP32[$pTab>>2]|0;
         $357 = (($356) + 4|0);
         $358 = HEAP32[$357>>2]|0;
         HEAP32[$pTabCol>>2] = $358;
         while(1) {
          $359 = HEAP32[$j>>2]|0;
          $360 = HEAP32[$pTab>>2]|0;
          $361 = (($360) + 38|0);
          $362 = HEAP16[$361>>1]|0;
          $363 = $362 << 16 >> 16;
          $364 = ($359|0)<($363|0);
          if (!($364)) {
           break;
          }
          $365 = HEAP32[$zColName>>2]|0;
          $366 = HEAP32[$pTabCol>>2]|0;
          $367 = HEAP32[$366>>2]|0;
          $368 = (_sqlite3_stricmp($365,$367)|0);
          $369 = ($368|0)==(0);
          if ($369) {
           break;
          }
          $370 = HEAP32[$j>>2]|0;
          $371 = (($370) + 1)|0;
          HEAP32[$j>>2] = $371;
          $372 = HEAP32[$pTabCol>>2]|0;
          $373 = (($372) + 24|0);
          HEAP32[$pTabCol>>2] = $373;
         }
         $374 = HEAP32[$j>>2]|0;
         $375 = HEAP32[$pTab>>2]|0;
         $376 = (($375) + 38|0);
         $377 = HEAP16[$376>>1]|0;
         $378 = $377 << 16 >> 16;
         $379 = ($374|0)>=($378|0);
         if ($379) {
          label = 65;
          break;
         }
         $386 = HEAP32[$j>>2]|0;
         $387 = $386&65535;
         $388 = HEAP32[$i>>2]|0;
         $389 = HEAP32[$pIndex>>2]|0;
         $390 = (($389) + 4|0);
         $391 = HEAP32[$390>>2]|0;
         $392 = (($391) + ($388<<1)|0);
         HEAP16[$392>>1] = $387;
         $393 = HEAP32[$pListItem>>2]|0;
         $394 = HEAP32[$393>>2]|0;
         $395 = ($394|0)!=(0|0);
         if ($395) {
          $396 = HEAP32[$pListItem>>2]|0;
          $397 = HEAP32[$396>>2]|0;
          $398 = (($397) + 8|0);
          $399 = HEAP32[$398>>2]|0;
          HEAP32[$zColl>>2] = $399;
          $400 = HEAP32[$zColl>>2]|0;
          $401 = (_sqlite3Strlen30($400)|0);
          $402 = (($401) + 1)|0;
          HEAP32[$nColl>>2] = $402;
          $403 = HEAP32[$zExtra>>2]|0;
          $404 = HEAP32[$zColl>>2]|0;
          $405 = HEAP32[$nColl>>2]|0;
          _memcpy(($403|0),($404|0),($405|0))|0;
          $406 = HEAP32[$zExtra>>2]|0;
          HEAP32[$zColl>>2] = $406;
          $407 = HEAP32[$nColl>>2]|0;
          $408 = HEAP32[$zExtra>>2]|0;
          $409 = (($408) + ($407)|0);
          HEAP32[$zExtra>>2] = $409;
          $410 = HEAP32[$nColl>>2]|0;
          $411 = HEAP32[$nExtra>>2]|0;
          $412 = (($411) - ($410))|0;
          HEAP32[$nExtra>>2] = $412;
         } else {
          $413 = HEAP32[$j>>2]|0;
          $414 = HEAP32[$pTab>>2]|0;
          $415 = (($414) + 4|0);
          $416 = HEAP32[$415>>2]|0;
          $417 = (($416) + (($413*24)|0)|0);
          $418 = (($417) + 16|0);
          $419 = HEAP32[$418>>2]|0;
          HEAP32[$zColl>>2] = $419;
          $420 = HEAP32[$zColl>>2]|0;
          $421 = ($420|0)!=(0|0);
          $$2 = $421 ? $419 : 3320;
          HEAP32[$zColl>>2] = $$2;
         }
         $422 = HEAP32[$db>>2]|0;
         $423 = (($422) + 140|0);
         $424 = (($423) + 5|0);
         $425 = HEAP8[$424>>0]|0;
         $426 = ($425<<24>>24)!=(0);
         if (!($426)) {
          $427 = HEAP32[$0>>2]|0;
          $428 = HEAP32[$zColl>>2]|0;
          $429 = (_sqlite3LocateCollSeq($427,$428)|0);
          $430 = ($429|0)!=(0|0);
          if (!($430)) {
           break L1;
          }
         }
         $431 = HEAP32[$zColl>>2]|0;
         $432 = HEAP32[$i>>2]|0;
         $433 = HEAP32[$pIndex>>2]|0;
         $434 = (($433) + 32|0);
         $435 = HEAP32[$434>>2]|0;
         $436 = (($435) + ($432<<2)|0);
         HEAP32[$436>>2] = $431;
         $437 = HEAP32[$pListItem>>2]|0;
         $438 = (($437) + 12|0);
         $439 = HEAP8[$438>>0]|0;
         $440 = $439&255;
         $441 = HEAP32[$sortOrderMask>>2]|0;
         $442 = $440 & $441;
         HEAP32[$requestedSortOrder>>2] = $442;
         $443 = HEAP32[$requestedSortOrder>>2]|0;
         $444 = $443&255;
         $445 = HEAP32[$i>>2]|0;
         $446 = HEAP32[$pIndex>>2]|0;
         $447 = (($446) + 28|0);
         $448 = HEAP32[$447>>2]|0;
         $449 = (($448) + ($445)|0);
         HEAP8[$449>>0] = $444;
         $450 = HEAP32[$j>>2]|0;
         $451 = HEAP32[$pTab>>2]|0;
         $452 = (($451) + 4|0);
         $453 = HEAP32[$452>>2]|0;
         $454 = (($453) + (($450*24)|0)|0);
         $455 = (($454) + 20|0);
         $456 = HEAP8[$455>>0]|0;
         $457 = $456&255;
         $458 = ($457|0)==(0);
         if ($458) {
          $459 = HEAP32[$pIndex>>2]|0;
          $460 = (($459) + 55|0);
          $461 = HEAP8[$460>>0]|0;
          $462 = $461 & -9;
          HEAP8[$460>>0] = $462;
         }
         $463 = HEAP32[$i>>2]|0;
         $464 = (($463) + 1)|0;
         HEAP32[$i>>2] = $464;
         $465 = HEAP32[$pListItem>>2]|0;
         $466 = (($465) + 20|0);
         HEAP32[$pListItem>>2] = $466;
        }
        if ((label|0) == 65) {
         $380 = HEAP32[$0>>2]|0;
         $381 = HEAP32[$pTab>>2]|0;
         $382 = HEAP32[$381>>2]|0;
         $383 = HEAP32[$zColName>>2]|0;
         HEAP32[$vararg_buffer44>>2] = $382;
         $vararg_ptr27 = (($vararg_buffer44) + 4|0);
         HEAP32[$vararg_ptr27>>2] = $383;
         _sqlite3ErrorMsg($380,26208,$vararg_buffer44);
         $384 = HEAP32[$0>>2]|0;
         $385 = (($384) + 17|0);
         HEAP8[$385>>0] = 1;
         break;
        }
        $467 = HEAP32[$pPk>>2]|0;
        $468 = ($467|0)!=(0|0);
        L97: do {
         if ($468) {
          HEAP32[$j>>2] = 0;
          while(1) {
           $469 = HEAP32[$j>>2]|0;
           $470 = HEAP32[$pPk>>2]|0;
           $471 = (($470) + 50|0);
           $472 = HEAP16[$471>>1]|0;
           $473 = $472&65535;
           $474 = ($469|0)<($473|0);
           if (!($474)) {
            break L97;
           }
           $475 = HEAP32[$j>>2]|0;
           $476 = HEAP32[$pPk>>2]|0;
           $477 = (($476) + 4|0);
           $478 = HEAP32[$477>>2]|0;
           $479 = (($478) + ($475<<1)|0);
           $480 = HEAP16[$479>>1]|0;
           $481 = $480 << 16 >> 16;
           HEAP32[$x>>2] = $481;
           $482 = HEAP32[$pIndex>>2]|0;
           $483 = (($482) + 4|0);
           $484 = HEAP32[$483>>2]|0;
           $485 = HEAP32[$pIndex>>2]|0;
           $486 = (($485) + 50|0);
           $487 = HEAP16[$486>>1]|0;
           $488 = $487&65535;
           $489 = HEAP32[$x>>2]|0;
           $490 = (_hasColumn($484,$488,$489)|0);
           $491 = ($490|0)!=(0);
           if ($491) {
            $492 = HEAP32[$pIndex>>2]|0;
            $493 = (($492) + 52|0);
            $494 = HEAP16[$493>>1]|0;
            $495 = (($494) + -1)<<16>>16;
            HEAP16[$493>>1] = $495;
           } else {
            $496 = HEAP32[$x>>2]|0;
            $497 = $496&65535;
            $498 = HEAP32[$i>>2]|0;
            $499 = HEAP32[$pIndex>>2]|0;
            $500 = (($499) + 4|0);
            $501 = HEAP32[$500>>2]|0;
            $502 = (($501) + ($498<<1)|0);
            HEAP16[$502>>1] = $497;
            $503 = HEAP32[$j>>2]|0;
            $504 = HEAP32[$pPk>>2]|0;
            $505 = (($504) + 32|0);
            $506 = HEAP32[$505>>2]|0;
            $507 = (($506) + ($503<<2)|0);
            $508 = HEAP32[$507>>2]|0;
            $509 = HEAP32[$i>>2]|0;
            $510 = HEAP32[$pIndex>>2]|0;
            $511 = (($510) + 32|0);
            $512 = HEAP32[$511>>2]|0;
            $513 = (($512) + ($509<<2)|0);
            HEAP32[$513>>2] = $508;
            $514 = HEAP32[$j>>2]|0;
            $515 = HEAP32[$pPk>>2]|0;
            $516 = (($515) + 28|0);
            $517 = HEAP32[$516>>2]|0;
            $518 = (($517) + ($514)|0);
            $519 = HEAP8[$518>>0]|0;
            $520 = HEAP32[$i>>2]|0;
            $521 = HEAP32[$pIndex>>2]|0;
            $522 = (($521) + 28|0);
            $523 = HEAP32[$522>>2]|0;
            $524 = (($523) + ($520)|0);
            HEAP8[$524>>0] = $519;
            $525 = HEAP32[$i>>2]|0;
            $526 = (($525) + 1)|0;
            HEAP32[$i>>2] = $526;
           }
           $527 = HEAP32[$j>>2]|0;
           $528 = (($527) + 1)|0;
           HEAP32[$j>>2] = $528;
          }
         } else {
          $529 = HEAP32[$i>>2]|0;
          $530 = HEAP32[$pIndex>>2]|0;
          $531 = (($530) + 4|0);
          $532 = HEAP32[$531>>2]|0;
          $533 = (($532) + ($529<<1)|0);
          HEAP16[$533>>1] = -1;
          $534 = HEAP32[$i>>2]|0;
          $535 = HEAP32[$pIndex>>2]|0;
          $536 = (($535) + 32|0);
          $537 = HEAP32[$536>>2]|0;
          $538 = (($537) + ($534<<2)|0);
          HEAP32[$538>>2] = 3320;
         }
        } while(0);
        $539 = HEAP32[$pIndex>>2]|0;
        _sqlite3DefaultRowEst($539);
        $540 = HEAP32[$0>>2]|0;
        $541 = (($540) + 484|0);
        $542 = HEAP32[$541>>2]|0;
        $543 = ($542|0)==(0|0);
        if ($543) {
         $544 = HEAP32[$pIndex>>2]|0;
         _estimateIndexWidth($544);
        }
        $545 = HEAP32[$pTab>>2]|0;
        $546 = HEAP32[$0>>2]|0;
        $547 = (($546) + 484|0);
        $548 = HEAP32[$547>>2]|0;
        $549 = ($545|0)==($548|0);
        L111: do {
         if ($549) {
          $550 = HEAP32[$pTab>>2]|0;
          $551 = (($550) + 8|0);
          $552 = HEAP32[$551>>2]|0;
          HEAP32[$pIdx>>2] = $552;
          while(1) {
           $553 = HEAP32[$pIdx>>2]|0;
           $554 = ($553|0)!=(0|0);
           if (!($554)) {
            break L111;
           }
           $555 = HEAP32[$pIdx>>2]|0;
           $556 = (($555) + 50|0);
           $557 = HEAP16[$556>>1]|0;
           $558 = $557&65535;
           $559 = HEAP32[$pIndex>>2]|0;
           $560 = (($559) + 50|0);
           $561 = HEAP16[$560>>1]|0;
           $562 = $561&65535;
           $563 = ($558|0)!=($562|0);
           if (!($563)) {
            HEAP32[$k>>2] = 0;
            while(1) {
             $564 = HEAP32[$k>>2]|0;
             $565 = HEAP32[$pIdx>>2]|0;
             $566 = (($565) + 50|0);
             $567 = HEAP16[$566>>1]|0;
             $568 = $567&65535;
             $569 = ($564|0)<($568|0);
             if (!($569)) {
              break;
             }
             $570 = HEAP32[$k>>2]|0;
             $571 = HEAP32[$pIdx>>2]|0;
             $572 = (($571) + 4|0);
             $573 = HEAP32[$572>>2]|0;
             $574 = (($573) + ($570<<1)|0);
             $575 = HEAP16[$574>>1]|0;
             $576 = $575 << 16 >> 16;
             $577 = HEAP32[$k>>2]|0;
             $578 = HEAP32[$pIndex>>2]|0;
             $579 = (($578) + 4|0);
             $580 = HEAP32[$579>>2]|0;
             $581 = (($580) + ($577<<1)|0);
             $582 = HEAP16[$581>>1]|0;
             $583 = $582 << 16 >> 16;
             $584 = ($576|0)!=($583|0);
             if ($584) {
              break;
             }
             $585 = HEAP32[$k>>2]|0;
             $586 = HEAP32[$pIdx>>2]|0;
             $587 = (($586) + 32|0);
             $588 = HEAP32[$587>>2]|0;
             $589 = (($588) + ($585<<2)|0);
             $590 = HEAP32[$589>>2]|0;
             HEAP32[$z1>>2] = $590;
             $591 = HEAP32[$k>>2]|0;
             $592 = HEAP32[$pIndex>>2]|0;
             $593 = (($592) + 32|0);
             $594 = HEAP32[$593>>2]|0;
             $595 = (($594) + ($591<<2)|0);
             $596 = HEAP32[$595>>2]|0;
             HEAP32[$z2>>2] = $596;
             $597 = HEAP32[$z1>>2]|0;
             $598 = HEAP32[$z2>>2]|0;
             $599 = ($597|0)!=($598|0);
             if ($599) {
              $600 = HEAP32[$z1>>2]|0;
              $601 = HEAP32[$z2>>2]|0;
              $602 = (_sqlite3_stricmp($600,$601)|0);
              $603 = ($602|0)!=(0);
              if ($603) {
               break;
              }
             }
             $604 = HEAP32[$k>>2]|0;
             $605 = (($604) + 1)|0;
             HEAP32[$k>>2] = $605;
            }
            $606 = HEAP32[$k>>2]|0;
            $607 = HEAP32[$pIdx>>2]|0;
            $608 = (($607) + 50|0);
            $609 = HEAP16[$608>>1]|0;
            $610 = $609&65535;
            $611 = ($606|0)==($610|0);
            if ($611) {
             break;
            }
           }
           $642 = HEAP32[$pIdx>>2]|0;
           $643 = (($642) + 20|0);
           $644 = HEAP32[$643>>2]|0;
           HEAP32[$pIdx>>2] = $644;
          }
          $612 = HEAP32[$pIdx>>2]|0;
          $613 = (($612) + 54|0);
          $614 = HEAP8[$613>>0]|0;
          $615 = $614&255;
          $616 = HEAP32[$pIndex>>2]|0;
          $617 = (($616) + 54|0);
          $618 = HEAP8[$617>>0]|0;
          $619 = $618&255;
          $620 = ($615|0)!=($619|0);
          if (!($620)) {
           break L1;
          }
          $621 = HEAP32[$pIdx>>2]|0;
          $622 = (($621) + 54|0);
          $623 = HEAP8[$622>>0]|0;
          $624 = $623&255;
          $625 = ($624|0)==(10);
          do {
           if (!($625)) {
            $626 = HEAP32[$pIndex>>2]|0;
            $627 = (($626) + 54|0);
            $628 = HEAP8[$627>>0]|0;
            $629 = $628&255;
            $630 = ($629|0)==(10);
            if ($630) {
             break;
            }
            $631 = HEAP32[$0>>2]|0;
            HEAP32[$vararg_buffer44>>2] = 0;
            _sqlite3ErrorMsg($631,26240,$vararg_buffer44);
           }
          } while(0);
          $632 = HEAP32[$pIdx>>2]|0;
          $633 = (($632) + 54|0);
          $634 = HEAP8[$633>>0]|0;
          $635 = $634&255;
          $636 = ($635|0)==(10);
          if (!($636)) {
           break L1;
          }
          $637 = HEAP32[$pIndex>>2]|0;
          $638 = (($637) + 54|0);
          $639 = HEAP8[$638>>0]|0;
          $640 = HEAP32[$pIdx>>2]|0;
          $641 = (($640) + 54|0);
          HEAP8[$641>>0] = $639;
          break L1;
         }
        } while(0);
        $645 = HEAP32[$db>>2]|0;
        $646 = (($645) + 140|0);
        $647 = (($646) + 5|0);
        $648 = HEAP8[$647>>0]|0;
        $649 = ($648<<24>>24)!=(0);
        do {
         if ($649) {
          $650 = HEAP32[$pIndex>>2]|0;
          $651 = (($650) + 24|0);
          $652 = HEAP32[$651>>2]|0;
          $653 = (($652) + 24|0);
          $654 = HEAP32[$pIndex>>2]|0;
          $655 = HEAP32[$654>>2]|0;
          $656 = HEAP32[$pIndex>>2]|0;
          $657 = (_sqlite3HashInsert($653,$655,$656)|0);
          HEAP32[$p>>2] = $657;
          $658 = HEAP32[$p>>2]|0;
          $659 = ($658|0)!=(0|0);
          $660 = HEAP32[$db>>2]|0;
          if ($659) {
           $661 = (($660) + 64|0);
           HEAP8[$661>>0] = 1;
           break L1;
          }
          $662 = (($660) + 24|0);
          $663 = HEAP32[$662>>2]|0;
          $664 = $663 | 2;
          HEAP32[$662>>2] = $664;
          $665 = HEAP32[$3>>2]|0;
          $666 = ($665|0)!=(0|0);
          if (!($666)) {
           break;
          }
          $667 = HEAP32[$db>>2]|0;
          $668 = (($667) + 140|0);
          $669 = HEAP32[$668>>2]|0;
          $670 = HEAP32[$pIndex>>2]|0;
          $671 = (($670) + 44|0);
          HEAP32[$671>>2] = $669;
         } else {
          $672 = HEAP32[$0>>2]|0;
          $673 = (($672) + 64|0);
          $674 = HEAP32[$673>>2]|0;
          $675 = ($674|0)==(0);
          if (!($675)) {
           break;
          }
          $676 = HEAP32[$pTab>>2]|0;
          $677 = (($676) + 44|0);
          $678 = HEAP8[$677>>0]|0;
          $679 = $678&255;
          $680 = $679 & 32;
          $681 = ($680|0)==(0);
          $682 = HEAP32[$3>>2]|0;
          $683 = ($682|0)!=(0|0);
          $or$cond4 = $681 | $683;
          if (!($or$cond4)) {
           break;
          }
          $684 = HEAP32[$0>>2]|0;
          $685 = (($684) + 72|0);
          $686 = HEAP32[$685>>2]|0;
          $687 = (($686) + 1)|0;
          HEAP32[$685>>2] = $687;
          HEAP32[$iMem>>2] = $687;
          $688 = HEAP32[$0>>2]|0;
          $689 = (_sqlite3GetVdbe($688)|0);
          HEAP32[$v>>2] = $689;
          $690 = HEAP32[$v>>2]|0;
          $691 = ($690|0)==(0|0);
          if ($691) {
           break L1;
          }
          $692 = HEAP32[$0>>2]|0;
          $693 = HEAP32[$iDb>>2]|0;
          _sqlite3BeginWriteOperation($692,1,$693);
          $694 = HEAP32[$v>>2]|0;
          $695 = HEAP32[$iDb>>2]|0;
          $696 = HEAP32[$iMem>>2]|0;
          (_sqlite3VdbeAddOp2($694,120,$695,$696)|0);
          $697 = HEAP32[$6>>2]|0;
          $698 = ($697|0)!=(0|0);
          if ($698) {
           $699 = HEAP32[$0>>2]|0;
           $700 = (($699) + 504|0);
           $701 = HEAP32[$700>>2]|0;
           $702 = HEAP32[$pName>>2]|0;
           $703 = HEAP32[$702>>2]|0;
           $704 = $701;
           $705 = $703;
           $706 = (($704) - ($705))|0;
           $707 = HEAP32[$0>>2]|0;
           $708 = (($707) + 504|0);
           $709 = (($708) + 4|0);
           $710 = HEAP32[$709>>2]|0;
           $711 = (($706) + ($710))|0;
           HEAP32[$n1>>2] = $711;
           $712 = HEAP32[$n1>>2]|0;
           $713 = (($712) - 1)|0;
           $714 = HEAP32[$pName>>2]|0;
           $715 = HEAP32[$714>>2]|0;
           $716 = (($715) + ($713)|0);
           $717 = HEAP8[$716>>0]|0;
           $718 = $717 << 24 >> 24;
           $719 = ($718|0)==(59);
           if ($719) {
            $720 = HEAP32[$n1>>2]|0;
            $721 = (($720) + -1)|0;
            HEAP32[$n1>>2] = $721;
           }
           $722 = HEAP32[$db>>2]|0;
           $723 = HEAP32[$5>>2]|0;
           $724 = ($723|0)==(0);
           $725 = $724 ? 1216 : 26312;
           $726 = HEAP32[$n1>>2]|0;
           $727 = HEAP32[$pName>>2]|0;
           $728 = HEAP32[$727>>2]|0;
           HEAP32[$vararg_buffer44>>2] = $725;
           $vararg_ptr34 = (($vararg_buffer44) + 4|0);
           HEAP32[$vararg_ptr34>>2] = $726;
           $vararg_ptr35 = (($vararg_buffer44) + 8|0);
           HEAP32[$vararg_ptr35>>2] = $728;
           $729 = (_sqlite3MPrintf($722,26288,$vararg_buffer44)|0);
           HEAP32[$zStmt>>2] = $729;
          } else {
           HEAP32[$zStmt>>2] = 0;
          }
          $730 = HEAP32[$0>>2]|0;
          $731 = HEAP32[$iDb>>2]|0;
          $732 = HEAP32[$db>>2]|0;
          $733 = (($732) + 16|0);
          $734 = HEAP32[$733>>2]|0;
          $735 = (($734) + ($731<<4)|0);
          $736 = HEAP32[$735>>2]|0;
          $737 = HEAP32[$iDb>>2]|0;
          $738 = ($737|0)==(1);
          $739 = $738 ? 14904 : 14928;
          $740 = HEAP32[$pIndex>>2]|0;
          $741 = HEAP32[$740>>2]|0;
          $742 = HEAP32[$pTab>>2]|0;
          $743 = HEAP32[$742>>2]|0;
          $744 = HEAP32[$iMem>>2]|0;
          $745 = HEAP32[$zStmt>>2]|0;
          HEAP32[$vararg_buffer44>>2] = $736;
          $vararg_ptr39 = (($vararg_buffer44) + 4|0);
          HEAP32[$vararg_ptr39>>2] = $739;
          $vararg_ptr40 = (($vararg_buffer44) + 8|0);
          HEAP32[$vararg_ptr40>>2] = $741;
          $vararg_ptr41 = (($vararg_buffer44) + 12|0);
          HEAP32[$vararg_ptr41>>2] = $743;
          $vararg_ptr42 = (($vararg_buffer44) + 16|0);
          HEAP32[$vararg_ptr42>>2] = $744;
          $vararg_ptr43 = (($vararg_buffer44) + 20|0);
          HEAP32[$vararg_ptr43>>2] = $745;
          _sqlite3NestedParse($730,26320,$vararg_buffer44);
          $746 = HEAP32[$db>>2]|0;
          $747 = HEAP32[$zStmt>>2]|0;
          _sqlite3DbFree($746,$747);
          $748 = HEAP32[$3>>2]|0;
          $749 = ($748|0)!=(0|0);
          if (!($749)) {
           break;
          }
          $750 = HEAP32[$0>>2]|0;
          $751 = HEAP32[$pIndex>>2]|0;
          $752 = HEAP32[$iMem>>2]|0;
          _sqlite3RefillIndex($750,$751,$752);
          $753 = HEAP32[$0>>2]|0;
          $754 = HEAP32[$iDb>>2]|0;
          _sqlite3ChangeCookie($753,$754);
          $755 = HEAP32[$v>>2]|0;
          $756 = HEAP32[$iDb>>2]|0;
          $757 = HEAP32[$db>>2]|0;
          $758 = HEAP32[$pIndex>>2]|0;
          $759 = HEAP32[$758>>2]|0;
          HEAP32[$vararg_buffer44>>2] = $759;
          $760 = (_sqlite3MPrintf($757,26368,$vararg_buffer44)|0);
          _sqlite3VdbeAddParseSchemaOp($755,$756,$760);
          $761 = HEAP32[$v>>2]|0;
          (_sqlite3VdbeAddOp1($761,142,0)|0);
         }
        } while(0);
        $762 = HEAP32[$db>>2]|0;
        $763 = (($762) + 140|0);
        $764 = (($763) + 5|0);
        $765 = HEAP8[$764>>0]|0;
        $766 = $765&255;
        $767 = ($766|0)!=(0);
        $768 = HEAP32[$3>>2]|0;
        $769 = ($768|0)==(0|0);
        $or$cond6 = $767 | $769;
        if (!($or$cond6)) {
         break;
        }
        $770 = HEAP32[$5>>2]|0;
        $771 = ($770|0)!=(5);
        do {
         if ($771) {
          label = 121;
         } else {
          $772 = HEAP32[$pTab>>2]|0;
          $773 = (($772) + 8|0);
          $774 = HEAP32[$773>>2]|0;
          $775 = ($774|0)==(0|0);
          if ($775) {
           label = 121;
           break;
          }
          $776 = HEAP32[$pTab>>2]|0;
          $777 = (($776) + 8|0);
          $778 = HEAP32[$777>>2]|0;
          $779 = (($778) + 54|0);
          $780 = HEAP8[$779>>0]|0;
          $781 = $780&255;
          $782 = ($781|0)==(5);
          if ($782) {
           label = 121;
           break;
          }
          $791 = HEAP32[$pTab>>2]|0;
          $792 = (($791) + 8|0);
          $793 = HEAP32[$792>>2]|0;
          HEAP32[$pOther>>2] = $793;
          while(1) {
           $794 = HEAP32[$pOther>>2]|0;
           $795 = (($794) + 20|0);
           $796 = HEAP32[$795>>2]|0;
           $797 = ($796|0)!=(0|0);
           if ($797) {
            $798 = HEAP32[$pOther>>2]|0;
            $799 = (($798) + 20|0);
            $800 = HEAP32[$799>>2]|0;
            $801 = (($800) + 54|0);
            $802 = HEAP8[$801>>0]|0;
            $803 = $802&255;
            $804 = ($803|0)!=(5);
            $827 = $804;
           } else {
            $827 = 0;
           }
           $805 = HEAP32[$pOther>>2]|0;
           $806 = (($805) + 20|0);
           $807 = HEAP32[$806>>2]|0;
           if (!($827)) {
            break;
           }
           HEAP32[$pOther>>2] = $807;
          }
          $808 = HEAP32[$pIndex>>2]|0;
          $809 = (($808) + 20|0);
          HEAP32[$809>>2] = $807;
          $810 = HEAP32[$pIndex>>2]|0;
          $811 = HEAP32[$pOther>>2]|0;
          $812 = (($811) + 20|0);
          HEAP32[$812>>2] = $810;
         }
        } while(0);
        if ((label|0) == 121) {
         $783 = HEAP32[$pTab>>2]|0;
         $784 = (($783) + 8|0);
         $785 = HEAP32[$784>>2]|0;
         $786 = HEAP32[$pIndex>>2]|0;
         $787 = (($786) + 20|0);
         HEAP32[$787>>2] = $785;
         $788 = HEAP32[$pIndex>>2]|0;
         $789 = HEAP32[$pTab>>2]|0;
         $790 = (($789) + 8|0);
         HEAP32[$790>>2] = $788;
        }
        $813 = HEAP32[$pIndex>>2]|0;
        HEAP32[$pRet>>2] = $813;
        HEAP32[$pIndex>>2] = 0;
       }
      }
     }
    }
   }
  }
 } while(0);
 $814 = HEAP32[$pIndex>>2]|0;
 $815 = ($814|0)!=(0|0);
 if (!($815)) {
  $818 = HEAP32[$db>>2]|0;
  $819 = HEAP32[$7>>2]|0;
  _sqlite3ExprDelete($818,$819);
  $820 = HEAP32[$db>>2]|0;
  $821 = HEAP32[$4>>2]|0;
  _sqlite3ExprListDelete($820,$821);
  $822 = HEAP32[$db>>2]|0;
  $823 = HEAP32[$3>>2]|0;
  _sqlite3SrcListDelete($822,$823);
  $824 = HEAP32[$db>>2]|0;
  $825 = HEAP32[$zName>>2]|0;
  _sqlite3DbFree($824,$825);
  $826 = HEAP32[$pRet>>2]|0;
  STACKTOP = sp;return ($826|0);
 }
 $816 = HEAP32[$db>>2]|0;
 $817 = HEAP32[$pIndex>>2]|0;
 _freeIndex($816,$817);
 $818 = HEAP32[$db>>2]|0;
 $819 = HEAP32[$7>>2]|0;
 _sqlite3ExprDelete($818,$819);
 $820 = HEAP32[$db>>2]|0;
 $821 = HEAP32[$4>>2]|0;
 _sqlite3ExprListDelete($820,$821);
 $822 = HEAP32[$db>>2]|0;
 $823 = HEAP32[$3>>2]|0;
 _sqlite3SrcListDelete($822,$823);
 $824 = HEAP32[$db>>2]|0;
 $825 = HEAP32[$zName>>2]|0;
 _sqlite3DbFree($824,$825);
 $826 = HEAP32[$pRet>>2]|0;
 STACKTOP = sp;return ($826|0);
}
function _sqlite3AddCheckConstraint($pParse,$pCheckExpr) {
 $pParse = $pParse|0;
 $pCheckExpr = $pCheckExpr|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $db = 0, $pTab = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $1 = sp + 8|0;
 $pTab = sp + 4|0;
 $db = sp;
 HEAP32[$0>>2] = $pParse;
 HEAP32[$1>>2] = $pCheckExpr;
 $2 = HEAP32[$0>>2]|0;
 $3 = (($2) + 484|0);
 $4 = HEAP32[$3>>2]|0;
 HEAP32[$pTab>>2] = $4;
 $5 = HEAP32[$0>>2]|0;
 $6 = HEAP32[$5>>2]|0;
 HEAP32[$db>>2] = $6;
 $7 = HEAP32[$pTab>>2]|0;
 $8 = ($7|0)!=(0|0);
 if ($8) {
  $9 = HEAP32[$0>>2]|0;
  $10 = (($9) + 451|0);
  $11 = HEAP8[$10>>0]|0;
  $12 = ($11<<24>>24)!=(0);
  if (!($12)) {
   $13 = HEAP32[$db>>2]|0;
   $14 = (($13) + 140|0);
   $15 = (($14) + 4|0);
   $16 = HEAP8[$15>>0]|0;
   $17 = $16&255;
   $18 = HEAP32[$db>>2]|0;
   $19 = (($18) + 16|0);
   $20 = HEAP32[$19>>2]|0;
   $21 = (($20) + ($17<<4)|0);
   $22 = (($21) + 4|0);
   $23 = HEAP32[$22>>2]|0;
   $24 = (_sqlite3BtreeIsReadonly($23)|0);
   $25 = ($24|0)!=(0);
   if (!($25)) {
    $26 = HEAP32[$0>>2]|0;
    $27 = HEAP32[$pTab>>2]|0;
    $28 = (($27) + 24|0);
    $29 = HEAP32[$28>>2]|0;
    $30 = HEAP32[$1>>2]|0;
    $31 = (_sqlite3ExprListAppend($26,$29,$30)|0);
    $32 = HEAP32[$pTab>>2]|0;
    $33 = (($32) + 24|0);
    HEAP32[$33>>2] = $31;
    $34 = HEAP32[$0>>2]|0;
    $35 = (($34) + 320|0);
    $36 = (($35) + 4|0);
    $37 = HEAP32[$36>>2]|0;
    $38 = ($37|0)!=(0);
    if (!($38)) {
     STACKTOP = sp;return;
    }
    $39 = HEAP32[$0>>2]|0;
    $40 = HEAP32[$pTab>>2]|0;
    $41 = (($40) + 24|0);
    $42 = HEAP32[$41>>2]|0;
    $43 = HEAP32[$0>>2]|0;
    $44 = (($43) + 320|0);
    _sqlite3ExprListSetName($39,$42,$44,1);
    STACKTOP = sp;return;
   }
  }
 }
 $45 = HEAP32[$0>>2]|0;
 $46 = HEAP32[$45>>2]|0;
 $47 = HEAP32[$1>>2]|0;
 _sqlite3ExprDelete($46,$47);
 STACKTOP = sp;return;
}
function _sqlite3CreateForeignKey($pParse,$pFromCol,$pTo,$pToCol,$flags) {
 $pParse = $pParse|0;
 $pFromCol = $pFromCol|0;
 $pTo = $pTo|0;
 $pToCol = $pToCol|0;
 $flags = $flags|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0;
 var $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0, $188 = 0;
 var $189 = 0, $19 = 0, $190 = 0, $191 = 0, $192 = 0, $193 = 0, $194 = 0, $195 = 0, $196 = 0, $197 = 0, $198 = 0, $199 = 0, $2 = 0, $20 = 0, $200 = 0, $201 = 0, $202 = 0, $203 = 0, $204 = 0, $205 = 0;
 var $206 = 0, $207 = 0, $208 = 0, $209 = 0, $21 = 0, $210 = 0, $211 = 0, $212 = 0, $213 = 0, $214 = 0, $215 = 0, $216 = 0, $217 = 0, $218 = 0, $219 = 0, $22 = 0, $220 = 0, $221 = 0, $222 = 0, $223 = 0;
 var $224 = 0, $225 = 0, $226 = 0, $227 = 0, $228 = 0, $229 = 0, $23 = 0, $230 = 0, $231 = 0, $232 = 0, $233 = 0, $234 = 0, $235 = 0, $236 = 0, $237 = 0, $238 = 0, $239 = 0, $24 = 0, $240 = 0, $241 = 0;
 var $242 = 0, $243 = 0, $244 = 0, $245 = 0, $246 = 0, $247 = 0, $248 = 0, $249 = 0, $25 = 0, $250 = 0, $251 = 0, $252 = 0, $253 = 0, $254 = 0, $255 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0;
 var $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0;
 var $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0;
 var $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0;
 var $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $db = 0, $i = 0, $iCol = 0, $j = 0;
 var $n = 0, $nByte = 0, $nCol = 0, $p = 0, $pFKey = 0, $pNextTo = 0, $vararg_buffer4 = 0, $vararg_ptr1 = 0, $z = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 80|0;
 $vararg_buffer4 = sp;
 $0 = sp + 68|0;
 $1 = sp + 64|0;
 $2 = sp + 60|0;
 $3 = sp + 56|0;
 $4 = sp + 52|0;
 $db = sp + 48|0;
 $pFKey = sp + 44|0;
 $pNextTo = sp + 40|0;
 $p = sp + 36|0;
 $nByte = sp + 32|0;
 $i = sp + 28|0;
 $nCol = sp + 24|0;
 $z = sp + 20|0;
 $iCol = sp + 16|0;
 $j = sp + 12|0;
 $n = sp + 8|0;
 HEAP32[$0>>2] = $pParse;
 HEAP32[$1>>2] = $pFromCol;
 HEAP32[$2>>2] = $pTo;
 HEAP32[$3>>2] = $pToCol;
 HEAP32[$4>>2] = $flags;
 $5 = HEAP32[$0>>2]|0;
 $6 = HEAP32[$5>>2]|0;
 HEAP32[$db>>2] = $6;
 HEAP32[$pFKey>>2] = 0;
 $7 = HEAP32[$0>>2]|0;
 $8 = (($7) + 484|0);
 $9 = HEAP32[$8>>2]|0;
 HEAP32[$p>>2] = $9;
 $10 = HEAP32[$p>>2]|0;
 $11 = ($10|0)==(0|0);
 L1: do {
  if (!($11)) {
   $12 = HEAP32[$0>>2]|0;
   $13 = (($12) + 451|0);
   $14 = HEAP8[$13>>0]|0;
   $15 = $14&255;
   $16 = ($15|0)!=(0);
   if (!($16)) {
    $17 = HEAP32[$1>>2]|0;
    $18 = ($17|0)==(0|0);
    if ($18) {
     $19 = HEAP32[$p>>2]|0;
     $20 = (($19) + 38|0);
     $21 = HEAP16[$20>>1]|0;
     $22 = $21 << 16 >> 16;
     $23 = (($22) - 1)|0;
     HEAP32[$iCol>>2] = $23;
     $24 = HEAP32[$iCol>>2]|0;
     $25 = ($24|0)<(0);
     if ($25) {
      break;
     }
     $26 = HEAP32[$3>>2]|0;
     $27 = ($26|0)!=(0|0);
     if ($27) {
      $28 = HEAP32[$3>>2]|0;
      $29 = HEAP32[$28>>2]|0;
      $30 = ($29|0)!=(1);
      if ($30) {
       $31 = HEAP32[$0>>2]|0;
       $32 = HEAP32[$iCol>>2]|0;
       $33 = HEAP32[$p>>2]|0;
       $34 = (($33) + 4|0);
       $35 = HEAP32[$34>>2]|0;
       $36 = (($35) + (($32*24)|0)|0);
       $37 = HEAP32[$36>>2]|0;
       $38 = HEAP32[$2>>2]|0;
       HEAP32[$vararg_buffer4>>2] = $37;
       $vararg_ptr1 = (($vararg_buffer4) + 4|0);
       HEAP32[$vararg_ptr1>>2] = $38;
       _sqlite3ErrorMsg($31,25736,$vararg_buffer4);
       break;
      }
     }
     HEAP32[$nCol>>2] = 1;
    } else {
     $39 = HEAP32[$3>>2]|0;
     $40 = ($39|0)!=(0|0);
     if ($40) {
      $41 = HEAP32[$3>>2]|0;
      $42 = HEAP32[$41>>2]|0;
      $43 = HEAP32[$1>>2]|0;
      $44 = HEAP32[$43>>2]|0;
      $45 = ($42|0)!=($44|0);
      if ($45) {
       $46 = HEAP32[$0>>2]|0;
       _sqlite3ErrorMsg($46,25800,$vararg_buffer4);
       break;
      }
     }
     $47 = HEAP32[$1>>2]|0;
     $48 = HEAP32[$47>>2]|0;
     HEAP32[$nCol>>2] = $48;
    }
    $49 = HEAP32[$nCol>>2]|0;
    $50 = (($49) - 1)|0;
    $51 = $50<<3;
    $52 = (44 + ($51))|0;
    $53 = HEAP32[$2>>2]|0;
    $54 = (($53) + 4|0);
    $55 = HEAP32[$54>>2]|0;
    $56 = (($52) + ($55))|0;
    $57 = (($56) + 1)|0;
    HEAP32[$nByte>>2] = $57;
    $58 = HEAP32[$3>>2]|0;
    $59 = ($58|0)!=(0|0);
    L17: do {
     if ($59) {
      HEAP32[$i>>2] = 0;
      while(1) {
       $60 = HEAP32[$i>>2]|0;
       $61 = HEAP32[$3>>2]|0;
       $62 = HEAP32[$61>>2]|0;
       $63 = ($60|0)<($62|0);
       if (!($63)) {
        break L17;
       }
       $64 = HEAP32[$i>>2]|0;
       $65 = HEAP32[$3>>2]|0;
       $66 = (($65) + 4|0);
       $67 = HEAP32[$66>>2]|0;
       $68 = (($67) + (($64*20)|0)|0);
       $69 = (($68) + 4|0);
       $70 = HEAP32[$69>>2]|0;
       $71 = (_sqlite3Strlen30($70)|0);
       $72 = (($71) + 1)|0;
       $73 = HEAP32[$nByte>>2]|0;
       $74 = (($73) + ($72))|0;
       HEAP32[$nByte>>2] = $74;
       $75 = HEAP32[$i>>2]|0;
       $76 = (($75) + 1)|0;
       HEAP32[$i>>2] = $76;
      }
     }
    } while(0);
    $77 = HEAP32[$db>>2]|0;
    $78 = HEAP32[$nByte>>2]|0;
    $79 = ($78|0)<(0);
    $80 = $79 << 31 >> 31;
    $81 = (_sqlite3DbMallocZero($77,$78,$80)|0);
    HEAP32[$pFKey>>2] = $81;
    $82 = HEAP32[$pFKey>>2]|0;
    $83 = ($82|0)==(0|0);
    if (!($83)) {
     $84 = HEAP32[$p>>2]|0;
     $85 = HEAP32[$pFKey>>2]|0;
     HEAP32[$85>>2] = $84;
     $86 = HEAP32[$p>>2]|0;
     $87 = (($86) + 16|0);
     $88 = HEAP32[$87>>2]|0;
     $89 = HEAP32[$pFKey>>2]|0;
     $90 = (($89) + 4|0);
     HEAP32[$90>>2] = $88;
     $91 = HEAP32[$nCol>>2]|0;
     $92 = HEAP32[$pFKey>>2]|0;
     $93 = (($92) + 36|0);
     $94 = (($93) + ($91<<3)|0);
     HEAP32[$z>>2] = $94;
     $95 = HEAP32[$z>>2]|0;
     $96 = HEAP32[$pFKey>>2]|0;
     $97 = (($96) + 8|0);
     HEAP32[$97>>2] = $95;
     $98 = HEAP32[$z>>2]|0;
     $99 = HEAP32[$2>>2]|0;
     $100 = HEAP32[$99>>2]|0;
     $101 = HEAP32[$2>>2]|0;
     $102 = (($101) + 4|0);
     $103 = HEAP32[$102>>2]|0;
     _memcpy(($98|0),($100|0),($103|0))|0;
     $104 = HEAP32[$2>>2]|0;
     $105 = (($104) + 4|0);
     $106 = HEAP32[$105>>2]|0;
     $107 = HEAP32[$z>>2]|0;
     $108 = (($107) + ($106)|0);
     HEAP8[$108>>0] = 0;
     $109 = HEAP32[$z>>2]|0;
     (_sqlite3Dequote($109)|0);
     $110 = HEAP32[$2>>2]|0;
     $111 = (($110) + 4|0);
     $112 = HEAP32[$111>>2]|0;
     $113 = (($112) + 1)|0;
     $114 = HEAP32[$z>>2]|0;
     $115 = (($114) + ($113)|0);
     HEAP32[$z>>2] = $115;
     $116 = HEAP32[$nCol>>2]|0;
     $117 = HEAP32[$pFKey>>2]|0;
     $118 = (($117) + 20|0);
     HEAP32[$118>>2] = $116;
     $119 = HEAP32[$1>>2]|0;
     $120 = ($119|0)==(0|0);
     L24: do {
      if ($120) {
       $121 = HEAP32[$p>>2]|0;
       $122 = (($121) + 38|0);
       $123 = HEAP16[$122>>1]|0;
       $124 = $123 << 16 >> 16;
       $125 = (($124) - 1)|0;
       $126 = HEAP32[$pFKey>>2]|0;
       $127 = (($126) + 36|0);
       HEAP32[$127>>2] = $125;
      } else {
       HEAP32[$i>>2] = 0;
       while(1) {
        $128 = HEAP32[$i>>2]|0;
        $129 = HEAP32[$nCol>>2]|0;
        $130 = ($128|0)<($129|0);
        if (!($130)) {
         break L24;
        }
        HEAP32[$j>>2] = 0;
        while(1) {
         $131 = HEAP32[$j>>2]|0;
         $132 = HEAP32[$p>>2]|0;
         $133 = (($132) + 38|0);
         $134 = HEAP16[$133>>1]|0;
         $135 = $134 << 16 >> 16;
         $136 = ($131|0)<($135|0);
         if (!($136)) {
          break;
         }
         $137 = HEAP32[$j>>2]|0;
         $138 = HEAP32[$p>>2]|0;
         $139 = (($138) + 4|0);
         $140 = HEAP32[$139>>2]|0;
         $141 = (($140) + (($137*24)|0)|0);
         $142 = HEAP32[$141>>2]|0;
         $143 = HEAP32[$i>>2]|0;
         $144 = HEAP32[$1>>2]|0;
         $145 = (($144) + 4|0);
         $146 = HEAP32[$145>>2]|0;
         $147 = (($146) + (($143*20)|0)|0);
         $148 = (($147) + 4|0);
         $149 = HEAP32[$148>>2]|0;
         $150 = (_sqlite3_stricmp($142,$149)|0);
         $151 = ($150|0)==(0);
         $152 = HEAP32[$j>>2]|0;
         if ($151) {
          label = 25;
          break;
         }
         $157 = (($152) + 1)|0;
         HEAP32[$j>>2] = $157;
        }
        if ((label|0) == 25) {
         label = 0;
         $153 = HEAP32[$i>>2]|0;
         $154 = HEAP32[$pFKey>>2]|0;
         $155 = (($154) + 36|0);
         $156 = (($155) + ($153<<3)|0);
         HEAP32[$156>>2] = $152;
        }
        $158 = HEAP32[$j>>2]|0;
        $159 = HEAP32[$p>>2]|0;
        $160 = (($159) + 38|0);
        $161 = HEAP16[$160>>1]|0;
        $162 = $161 << 16 >> 16;
        $163 = ($158|0)>=($162|0);
        if ($163) {
         break;
        }
        $172 = HEAP32[$i>>2]|0;
        $173 = (($172) + 1)|0;
        HEAP32[$i>>2] = $173;
       }
       $164 = HEAP32[$0>>2]|0;
       $165 = HEAP32[$i>>2]|0;
       $166 = HEAP32[$1>>2]|0;
       $167 = (($166) + 4|0);
       $168 = HEAP32[$167>>2]|0;
       $169 = (($168) + (($165*20)|0)|0);
       $170 = (($169) + 4|0);
       $171 = HEAP32[$170>>2]|0;
       HEAP32[$vararg_buffer4>>2] = $171;
       _sqlite3ErrorMsg($164,25896,$vararg_buffer4);
       break L1;
      }
     } while(0);
     $174 = HEAP32[$3>>2]|0;
     $175 = ($174|0)!=(0|0);
     L40: do {
      if ($175) {
       HEAP32[$i>>2] = 0;
       while(1) {
        $176 = HEAP32[$i>>2]|0;
        $177 = HEAP32[$nCol>>2]|0;
        $178 = ($176|0)<($177|0);
        if (!($178)) {
         break L40;
        }
        $179 = HEAP32[$i>>2]|0;
        $180 = HEAP32[$3>>2]|0;
        $181 = (($180) + 4|0);
        $182 = HEAP32[$181>>2]|0;
        $183 = (($182) + (($179*20)|0)|0);
        $184 = (($183) + 4|0);
        $185 = HEAP32[$184>>2]|0;
        $186 = (_sqlite3Strlen30($185)|0);
        HEAP32[$n>>2] = $186;
        $187 = HEAP32[$z>>2]|0;
        $188 = HEAP32[$i>>2]|0;
        $189 = HEAP32[$pFKey>>2]|0;
        $190 = (($189) + 36|0);
        $191 = (($190) + ($188<<3)|0);
        $192 = (($191) + 4|0);
        HEAP32[$192>>2] = $187;
        $193 = HEAP32[$z>>2]|0;
        $194 = HEAP32[$i>>2]|0;
        $195 = HEAP32[$3>>2]|0;
        $196 = (($195) + 4|0);
        $197 = HEAP32[$196>>2]|0;
        $198 = (($197) + (($194*20)|0)|0);
        $199 = (($198) + 4|0);
        $200 = HEAP32[$199>>2]|0;
        $201 = HEAP32[$n>>2]|0;
        _memcpy(($193|0),($200|0),($201|0))|0;
        $202 = HEAP32[$n>>2]|0;
        $203 = HEAP32[$z>>2]|0;
        $204 = (($203) + ($202)|0);
        HEAP8[$204>>0] = 0;
        $205 = HEAP32[$n>>2]|0;
        $206 = (($205) + 1)|0;
        $207 = HEAP32[$z>>2]|0;
        $208 = (($207) + ($206)|0);
        HEAP32[$z>>2] = $208;
        $209 = HEAP32[$i>>2]|0;
        $210 = (($209) + 1)|0;
        HEAP32[$i>>2] = $210;
       }
      }
     } while(0);
     $211 = HEAP32[$pFKey>>2]|0;
     $212 = (($211) + 24|0);
     HEAP8[$212>>0] = 0;
     $213 = HEAP32[$4>>2]|0;
     $214 = $213 & 255;
     $215 = $214&255;
     $216 = HEAP32[$pFKey>>2]|0;
     $217 = (($216) + 25|0);
     HEAP8[$217>>0] = $215;
     $218 = HEAP32[$4>>2]|0;
     $219 = $218 >> 8;
     $220 = $219 & 255;
     $221 = $220&255;
     $222 = HEAP32[$pFKey>>2]|0;
     $223 = (($222) + 25|0);
     $224 = (($223) + 1|0);
     HEAP8[$224>>0] = $221;
     $225 = HEAP32[$p>>2]|0;
     $226 = (($225) + 68|0);
     $227 = HEAP32[$226>>2]|0;
     $228 = (($227) + 56|0);
     $229 = HEAP32[$pFKey>>2]|0;
     $230 = (($229) + 8|0);
     $231 = HEAP32[$230>>2]|0;
     $232 = HEAP32[$pFKey>>2]|0;
     $233 = (_sqlite3HashInsert($228,$231,$232)|0);
     HEAP32[$pNextTo>>2] = $233;
     $234 = HEAP32[$pNextTo>>2]|0;
     $235 = HEAP32[$pFKey>>2]|0;
     $236 = ($234|0)==($235|0);
     if ($236) {
      $237 = HEAP32[$db>>2]|0;
      $238 = (($237) + 64|0);
      HEAP8[$238>>0] = 1;
      break;
     }
     $239 = HEAP32[$pNextTo>>2]|0;
     $240 = ($239|0)!=(0|0);
     if ($240) {
      $241 = HEAP32[$pNextTo>>2]|0;
      $242 = HEAP32[$pFKey>>2]|0;
      $243 = (($242) + 12|0);
      HEAP32[$243>>2] = $241;
      $244 = HEAP32[$pFKey>>2]|0;
      $245 = HEAP32[$pNextTo>>2]|0;
      $246 = (($245) + 16|0);
      HEAP32[$246>>2] = $244;
     }
     $247 = HEAP32[$pFKey>>2]|0;
     $248 = HEAP32[$p>>2]|0;
     $249 = (($248) + 16|0);
     HEAP32[$249>>2] = $247;
     HEAP32[$pFKey>>2] = 0;
    }
   }
  }
 } while(0);
 $250 = HEAP32[$db>>2]|0;
 $251 = HEAP32[$pFKey>>2]|0;
 _sqlite3DbFree($250,$251);
 $252 = HEAP32[$db>>2]|0;
 $253 = HEAP32[$1>>2]|0;
 _sqlite3ExprListDelete($252,$253);
 $254 = HEAP32[$db>>2]|0;
 $255 = HEAP32[$3>>2]|0;
 _sqlite3ExprListDelete($254,$255);
 STACKTOP = sp;return;
}
function _sqlite3DeferForeignKey($pParse,$isDeferred) {
 $pParse = $pParse|0;
 $isDeferred = $isDeferred|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $pFKey = 0, $pTab = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $1 = sp + 8|0;
 $pTab = sp + 4|0;
 $pFKey = sp;
 HEAP32[$0>>2] = $pParse;
 HEAP32[$1>>2] = $isDeferred;
 $2 = HEAP32[$0>>2]|0;
 $3 = (($2) + 484|0);
 $4 = HEAP32[$3>>2]|0;
 HEAP32[$pTab>>2] = $4;
 $5 = ($4|0)==(0|0);
 if ($5) {
  STACKTOP = sp;return;
 }
 $6 = HEAP32[$pTab>>2]|0;
 $7 = (($6) + 16|0);
 $8 = HEAP32[$7>>2]|0;
 HEAP32[$pFKey>>2] = $8;
 $9 = ($8|0)==(0|0);
 if ($9) {
  STACKTOP = sp;return;
 }
 $10 = HEAP32[$1>>2]|0;
 $11 = $10&255;
 $12 = HEAP32[$pFKey>>2]|0;
 $13 = (($12) + 24|0);
 HEAP8[$13>>0] = $11;
 STACKTOP = sp;return;
}
function _sqlite3AddCollateType($pParse,$pToken) {
 $pParse = $pParse|0;
 $pToken = $pToken|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $7 = 0, $8 = 0, $9 = 0, $db = 0, $i = 0, $p = 0, $pIdx = 0, $zColl = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 24|0;
 $1 = sp + 20|0;
 $p = sp + 16|0;
 $i = sp + 12|0;
 $zColl = sp + 8|0;
 $db = sp + 4|0;
 $pIdx = sp;
 HEAP32[$0>>2] = $pParse;
 HEAP32[$1>>2] = $pToken;
 $2 = HEAP32[$0>>2]|0;
 $3 = (($2) + 484|0);
 $4 = HEAP32[$3>>2]|0;
 HEAP32[$p>>2] = $4;
 $5 = ($4|0)==(0|0);
 if ($5) {
  STACKTOP = sp;return;
 }
 $6 = HEAP32[$p>>2]|0;
 $7 = (($6) + 38|0);
 $8 = HEAP16[$7>>1]|0;
 $9 = $8 << 16 >> 16;
 $10 = (($9) - 1)|0;
 HEAP32[$i>>2] = $10;
 $11 = HEAP32[$0>>2]|0;
 $12 = HEAP32[$11>>2]|0;
 HEAP32[$db>>2] = $12;
 $13 = HEAP32[$db>>2]|0;
 $14 = HEAP32[$1>>2]|0;
 $15 = (_sqlite3NameFromToken($13,$14)|0);
 HEAP32[$zColl>>2] = $15;
 $16 = HEAP32[$zColl>>2]|0;
 $17 = ($16|0)!=(0|0);
 if (!($17)) {
  STACKTOP = sp;return;
 }
 $18 = HEAP32[$0>>2]|0;
 $19 = HEAP32[$zColl>>2]|0;
 $20 = (_sqlite3LocateCollSeq($18,$19)|0);
 $21 = ($20|0)!=(0|0);
 $22 = HEAP32[$db>>2]|0;
 if (!($21)) {
  $62 = HEAP32[$zColl>>2]|0;
  _sqlite3DbFree($22,$62);
  STACKTOP = sp;return;
 }
 $23 = HEAP32[$i>>2]|0;
 $24 = HEAP32[$p>>2]|0;
 $25 = (($24) + 4|0);
 $26 = HEAP32[$25>>2]|0;
 $27 = (($26) + (($23*24)|0)|0);
 $28 = (($27) + 16|0);
 $29 = HEAP32[$28>>2]|0;
 _sqlite3DbFree($22,$29);
 $30 = HEAP32[$zColl>>2]|0;
 $31 = HEAP32[$i>>2]|0;
 $32 = HEAP32[$p>>2]|0;
 $33 = (($32) + 4|0);
 $34 = HEAP32[$33>>2]|0;
 $35 = (($34) + (($31*24)|0)|0);
 $36 = (($35) + 16|0);
 HEAP32[$36>>2] = $30;
 $37 = HEAP32[$p>>2]|0;
 $38 = (($37) + 8|0);
 $39 = HEAP32[$38>>2]|0;
 HEAP32[$pIdx>>2] = $39;
 while(1) {
  $40 = HEAP32[$pIdx>>2]|0;
  $41 = ($40|0)!=(0|0);
  if (!($41)) {
   break;
  }
  $42 = HEAP32[$pIdx>>2]|0;
  $43 = (($42) + 4|0);
  $44 = HEAP32[$43>>2]|0;
  $45 = HEAP16[$44>>1]|0;
  $46 = $45 << 16 >> 16;
  $47 = HEAP32[$i>>2]|0;
  $48 = ($46|0)==($47|0);
  if ($48) {
   $49 = HEAP32[$i>>2]|0;
   $50 = HEAP32[$p>>2]|0;
   $51 = (($50) + 4|0);
   $52 = HEAP32[$51>>2]|0;
   $53 = (($52) + (($49*24)|0)|0);
   $54 = (($53) + 16|0);
   $55 = HEAP32[$54>>2]|0;
   $56 = HEAP32[$pIdx>>2]|0;
   $57 = (($56) + 32|0);
   $58 = HEAP32[$57>>2]|0;
   HEAP32[$58>>2] = $55;
  }
  $59 = HEAP32[$pIdx>>2]|0;
  $60 = (($59) + 20|0);
  $61 = HEAP32[$60>>2]|0;
  HEAP32[$pIdx>>2] = $61;
 }
 STACKTOP = sp;return;
}
function _sqlite3DropTable($pParse,$pName,$isView,$noErr) {
 $pParse = $pParse|0;
 $pName = $pName|0;
 $isView = $isView|0;
 $noErr = $noErr|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0;
 var $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0;
 var $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0;
 var $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0;
 var $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0;
 var $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $code = 0, $db = 0, $iDb = 0, $pTab = 0, $v = 0, $vararg_buffer4 = 0, $zArg2 = 0, $zDb = 0, $zTab = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 64|0;
 $vararg_buffer4 = sp;
 $0 = sp + 48|0;
 $1 = sp + 44|0;
 $2 = sp + 40|0;
 $3 = sp + 36|0;
 $pTab = sp + 32|0;
 $v = sp + 28|0;
 $db = sp + 24|0;
 $iDb = sp + 20|0;
 $code = sp + 16|0;
 $zTab = sp + 12|0;
 $zDb = sp + 8|0;
 $zArg2 = sp + 4|0;
 HEAP32[$0>>2] = $pParse;
 HEAP32[$1>>2] = $pName;
 HEAP32[$2>>2] = $isView;
 HEAP32[$3>>2] = $noErr;
 $4 = HEAP32[$0>>2]|0;
 $5 = HEAP32[$4>>2]|0;
 HEAP32[$db>>2] = $5;
 $6 = HEAP32[$db>>2]|0;
 $7 = (($6) + 64|0);
 $8 = HEAP8[$7>>0]|0;
 $9 = ($8<<24>>24)!=(0);
 do {
  if (!($9)) {
   $10 = HEAP32[$3>>2]|0;
   $11 = ($10|0)!=(0);
   if ($11) {
    $12 = HEAP32[$db>>2]|0;
    $13 = (($12) + 67|0);
    $14 = HEAP8[$13>>0]|0;
    $15 = (($14) + 1)<<24>>24;
    HEAP8[$13>>0] = $15;
   }
   $16 = HEAP32[$0>>2]|0;
   $17 = HEAP32[$2>>2]|0;
   $18 = HEAP32[$1>>2]|0;
   $19 = (($18) + 8|0);
   $20 = (_sqlite3LocateTableItem($16,$17,$19)|0);
   HEAP32[$pTab>>2] = $20;
   $21 = HEAP32[$3>>2]|0;
   $22 = ($21|0)!=(0);
   if ($22) {
    $23 = HEAP32[$db>>2]|0;
    $24 = (($23) + 67|0);
    $25 = HEAP8[$24>>0]|0;
    $26 = (($25) + -1)<<24>>24;
    HEAP8[$24>>0] = $26;
   }
   $27 = HEAP32[$pTab>>2]|0;
   $28 = ($27|0)==(0|0);
   if ($28) {
    $29 = HEAP32[$3>>2]|0;
    $30 = ($29|0)!=(0);
    if (!($30)) {
     break;
    }
    $31 = HEAP32[$0>>2]|0;
    $32 = HEAP32[$1>>2]|0;
    $33 = (($32) + 8|0);
    $34 = (($33) + 4|0);
    $35 = HEAP32[$34>>2]|0;
    _sqlite3CodeVerifyNamedSchema($31,$35);
    break;
   }
   $36 = HEAP32[$db>>2]|0;
   $37 = HEAP32[$pTab>>2]|0;
   $38 = (($37) + 68|0);
   $39 = HEAP32[$38>>2]|0;
   $40 = (_sqlite3SchemaToIndex($36,$39)|0);
   HEAP32[$iDb>>2] = $40;
   $41 = HEAP32[$pTab>>2]|0;
   $42 = (($41) + 44|0);
   $43 = HEAP8[$42>>0]|0;
   $44 = $43&255;
   $45 = $44 & 16;
   $46 = ($45|0)!=(0);
   if ($46) {
    $47 = HEAP32[$0>>2]|0;
    $48 = HEAP32[$pTab>>2]|0;
    $49 = (_sqlite3ViewGetColumnNames($47,$48)|0);
    $50 = ($49|0)!=(0);
    if ($50) {
     break;
    }
   }
   $51 = HEAP32[$iDb>>2]|0;
   $52 = ($51|0)==(1);
   $53 = $52 ? 14904 : 14928;
   HEAP32[$zTab>>2] = $53;
   $54 = HEAP32[$iDb>>2]|0;
   $55 = HEAP32[$db>>2]|0;
   $56 = (($55) + 16|0);
   $57 = HEAP32[$56>>2]|0;
   $58 = (($57) + ($54<<4)|0);
   $59 = HEAP32[$58>>2]|0;
   HEAP32[$zDb>>2] = $59;
   HEAP32[$zArg2>>2] = 0;
   $60 = HEAP32[$0>>2]|0;
   $61 = HEAP32[$zTab>>2]|0;
   $62 = HEAP32[$zDb>>2]|0;
   $63 = (_sqlite3AuthCheck($60,9,$61,0,$62)|0);
   $64 = ($63|0)!=(0);
   if (!($64)) {
    $65 = HEAP32[$2>>2]|0;
    $66 = ($65|0)!=(0);
    do {
     if ($66) {
      $67 = HEAP32[$iDb>>2]|0;
      $68 = ($67|0)==(1);
      if ($68) {
       HEAP32[$code>>2] = 15;
       break;
      } else {
       HEAP32[$code>>2] = 17;
       break;
      }
     } else {
      $69 = HEAP32[$pTab>>2]|0;
      $70 = (($69) + 44|0);
      $71 = HEAP8[$70>>0]|0;
      $72 = $71&255;
      $73 = $72 & 16;
      $74 = ($73|0)!=(0);
      if ($74) {
       HEAP32[$code>>2] = 30;
       $75 = HEAP32[$db>>2]|0;
       $76 = HEAP32[$pTab>>2]|0;
       $77 = (_sqlite3GetVTable($75,$76)|0);
       $78 = (($77) + 4|0);
       $79 = HEAP32[$78>>2]|0;
       $80 = (($79) + 4|0);
       $81 = HEAP32[$80>>2]|0;
       HEAP32[$zArg2>>2] = $81;
       break;
      }
      $82 = HEAP32[$iDb>>2]|0;
      $83 = ($82|0)==(1);
      if ($83) {
       HEAP32[$code>>2] = 13;
       break;
      } else {
       HEAP32[$code>>2] = 11;
       break;
      }
     }
    } while(0);
    $84 = HEAP32[$0>>2]|0;
    $85 = HEAP32[$code>>2]|0;
    $86 = HEAP32[$pTab>>2]|0;
    $87 = HEAP32[$86>>2]|0;
    $88 = HEAP32[$zArg2>>2]|0;
    $89 = HEAP32[$zDb>>2]|0;
    $90 = (_sqlite3AuthCheck($84,$85,$87,$88,$89)|0);
    $91 = ($90|0)!=(0);
    if (!($91)) {
     $92 = HEAP32[$0>>2]|0;
     $93 = HEAP32[$pTab>>2]|0;
     $94 = HEAP32[$93>>2]|0;
     $95 = HEAP32[$zDb>>2]|0;
     $96 = (_sqlite3AuthCheck($92,9,$94,0,$95)|0);
     $97 = ($96|0)!=(0);
     if (!($97)) {
      $98 = HEAP32[$pTab>>2]|0;
      $99 = HEAP32[$98>>2]|0;
      $100 = (_sqlite3_strnicmp($99,15144,7)|0);
      $101 = ($100|0)==(0);
      if ($101) {
       $102 = HEAP32[$pTab>>2]|0;
       $103 = HEAP32[$102>>2]|0;
       $104 = (_sqlite3_strnicmp($103,25512,11)|0);
       $105 = ($104|0)!=(0);
       if ($105) {
        $106 = HEAP32[$0>>2]|0;
        $107 = HEAP32[$pTab>>2]|0;
        $108 = HEAP32[$107>>2]|0;
        HEAP32[$vararg_buffer4>>2] = $108;
        _sqlite3ErrorMsg($106,25528,$vararg_buffer4);
        break;
       }
      }
      $109 = HEAP32[$2>>2]|0;
      $110 = ($109|0)!=(0);
      if ($110) {
       $111 = HEAP32[$pTab>>2]|0;
       $112 = (($111) + 12|0);
       $113 = HEAP32[$112>>2]|0;
       $114 = ($113|0)==(0|0);
       if ($114) {
        $115 = HEAP32[$0>>2]|0;
        $116 = HEAP32[$pTab>>2]|0;
        $117 = HEAP32[$116>>2]|0;
        HEAP32[$vararg_buffer4>>2] = $117;
        _sqlite3ErrorMsg($115,25560,$vararg_buffer4);
        break;
       }
      }
      $118 = HEAP32[$2>>2]|0;
      $119 = ($118|0)!=(0);
      if (!($119)) {
       $120 = HEAP32[$pTab>>2]|0;
       $121 = (($120) + 12|0);
       $122 = HEAP32[$121>>2]|0;
       $123 = ($122|0)!=(0|0);
       if ($123) {
        $124 = HEAP32[$0>>2]|0;
        $125 = HEAP32[$pTab>>2]|0;
        $126 = HEAP32[$125>>2]|0;
        HEAP32[$vararg_buffer4>>2] = $126;
        _sqlite3ErrorMsg($124,25600,$vararg_buffer4);
        break;
       }
      }
      $127 = HEAP32[$0>>2]|0;
      $128 = (_sqlite3GetVdbe($127)|0);
      HEAP32[$v>>2] = $128;
      $129 = HEAP32[$v>>2]|0;
      $130 = ($129|0)!=(0|0);
      if ($130) {
       $131 = HEAP32[$0>>2]|0;
       $132 = HEAP32[$iDb>>2]|0;
       _sqlite3BeginWriteOperation($131,1,$132);
       $133 = HEAP32[$0>>2]|0;
       $134 = HEAP32[$iDb>>2]|0;
       $135 = HEAP32[$pTab>>2]|0;
       $136 = HEAP32[$135>>2]|0;
       _sqlite3ClearStatTables($133,$134,18168,$136);
       $137 = HEAP32[$0>>2]|0;
       $138 = HEAP32[$1>>2]|0;
       $139 = HEAP32[$pTab>>2]|0;
       _sqlite3FkDropTable($137,$138,$139);
       $140 = HEAP32[$0>>2]|0;
       $141 = HEAP32[$pTab>>2]|0;
       $142 = HEAP32[$iDb>>2]|0;
       $143 = HEAP32[$2>>2]|0;
       _sqlite3CodeDropTable($140,$141,$142,$143);
      }
     }
    }
   }
  }
 } while(0);
 $144 = HEAP32[$db>>2]|0;
 $145 = HEAP32[$1>>2]|0;
 _sqlite3SrcListDelete($144,$145);
 STACKTOP = sp;return;
}
function _sqlite3CreateView($pParse,$pBegin,$pName1,$pName2,$pSelect,$isTemp,$noErr) {
 $pParse = $pParse|0;
 $pBegin = $pBegin|0;
 $pName1 = $pName1|0;
 $pName2 = $pName2|0;
 $pSelect = $pSelect|0;
 $isTemp = $isTemp|0;
 $noErr = $noErr|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0;
 var $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0;
 var $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0;
 var $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0;
 var $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0;
 var $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $db = 0, $iDb = 0, $n = 0, $p = 0, $pName = 0, $sEnd = 0, $sFix = 0, $vararg_buffer = 0, $z = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 96|0;
 $vararg_buffer = sp;
 $0 = sp + 84|0;
 $1 = sp + 80|0;
 $2 = sp + 76|0;
 $3 = sp + 72|0;
 $4 = sp + 68|0;
 $5 = sp + 64|0;
 $6 = sp + 60|0;
 $p = sp + 56|0;
 $n = sp + 52|0;
 $z = sp + 48|0;
 $sEnd = sp + 40|0;
 $sFix = sp + 16|0;
 $pName = sp + 12|0;
 $iDb = sp + 8|0;
 $db = sp + 4|0;
 HEAP32[$0>>2] = $pParse;
 HEAP32[$1>>2] = $pBegin;
 HEAP32[$2>>2] = $pName1;
 HEAP32[$3>>2] = $pName2;
 HEAP32[$4>>2] = $pSelect;
 HEAP32[$5>>2] = $isTemp;
 HEAP32[$6>>2] = $noErr;
 HEAP32[$pName>>2] = 0;
 $7 = HEAP32[$0>>2]|0;
 $8 = HEAP32[$7>>2]|0;
 HEAP32[$db>>2] = $8;
 $9 = HEAP32[$0>>2]|0;
 $10 = (($9) + 440|0);
 $11 = HEAP32[$10>>2]|0;
 $12 = ($11|0)>(0);
 $13 = HEAP32[$0>>2]|0;
 if ($12) {
  _sqlite3ErrorMsg($13,25464,$vararg_buffer);
  $14 = HEAP32[$db>>2]|0;
  $15 = HEAP32[$4>>2]|0;
  _sqlite3SelectDelete($14,$15);
  STACKTOP = sp;return;
 }
 $16 = HEAP32[$2>>2]|0;
 $17 = HEAP32[$3>>2]|0;
 $18 = HEAP32[$5>>2]|0;
 $19 = HEAP32[$6>>2]|0;
 _sqlite3StartTable($13,$16,$17,$18,1,0,$19);
 $20 = HEAP32[$0>>2]|0;
 $21 = (($20) + 484|0);
 $22 = HEAP32[$21>>2]|0;
 HEAP32[$p>>2] = $22;
 $23 = HEAP32[$p>>2]|0;
 $24 = ($23|0)==(0|0);
 if (!($24)) {
  $25 = HEAP32[$0>>2]|0;
  $26 = (($25) + 64|0);
  $27 = HEAP32[$26>>2]|0;
  $28 = ($27|0)!=(0);
  if (!($28)) {
   $31 = HEAP32[$0>>2]|0;
   $32 = HEAP32[$2>>2]|0;
   $33 = HEAP32[$3>>2]|0;
   (_sqlite3TwoPartName($31,$32,$33,$pName)|0);
   $34 = HEAP32[$db>>2]|0;
   $35 = HEAP32[$p>>2]|0;
   $36 = (($35) + 68|0);
   $37 = HEAP32[$36>>2]|0;
   $38 = (_sqlite3SchemaToIndex($34,$37)|0);
   HEAP32[$iDb>>2] = $38;
   $39 = HEAP32[$0>>2]|0;
   $40 = HEAP32[$iDb>>2]|0;
   $41 = HEAP32[$pName>>2]|0;
   _sqlite3FixInit($sFix,$39,$40,25504,$41);
   $42 = HEAP32[$4>>2]|0;
   $43 = (_sqlite3FixSelect($sFix,$42)|0);
   $44 = ($43|0)!=(0);
   $45 = HEAP32[$db>>2]|0;
   $46 = HEAP32[$4>>2]|0;
   if ($44) {
    _sqlite3SelectDelete($45,$46);
    STACKTOP = sp;return;
   }
   $47 = (_sqlite3SelectDup($45,$46,1)|0);
   $48 = HEAP32[$p>>2]|0;
   $49 = (($48) + 12|0);
   HEAP32[$49>>2] = $47;
   $50 = HEAP32[$db>>2]|0;
   $51 = HEAP32[$4>>2]|0;
   _sqlite3SelectDelete($50,$51);
   $52 = HEAP32[$db>>2]|0;
   $53 = (($52) + 64|0);
   $54 = HEAP8[$53>>0]|0;
   $55 = ($54<<24>>24)!=(0);
   if ($55) {
    STACKTOP = sp;return;
   }
   $56 = HEAP32[$db>>2]|0;
   $57 = (($56) + 140|0);
   $58 = (($57) + 5|0);
   $59 = HEAP8[$58>>0]|0;
   $60 = ($59<<24>>24)!=(0);
   if (!($60)) {
    $61 = HEAP32[$0>>2]|0;
    $62 = HEAP32[$p>>2]|0;
    (_sqlite3ViewGetColumnNames($61,$62)|0);
   }
   $63 = HEAP32[$0>>2]|0;
   $64 = (($63) + 504|0);
   ;HEAP32[$sEnd+0>>2]=HEAP32[$64+0>>2]|0;HEAP32[$sEnd+4>>2]=HEAP32[$64+4>>2]|0;
   $65 = HEAP32[$sEnd>>2]|0;
   $66 = HEAP8[$65>>0]|0;
   $67 = $66 << 24 >> 24;
   $68 = ($67|0)!=(0);
   if ($68) {
    $69 = HEAP32[$sEnd>>2]|0;
    $70 = HEAP8[$69>>0]|0;
    $71 = $70 << 24 >> 24;
    $72 = ($71|0)!=(59);
    if ($72) {
     $73 = (($sEnd) + 4|0);
     $74 = HEAP32[$73>>2]|0;
     $75 = HEAP32[$sEnd>>2]|0;
     $76 = (($75) + ($74)|0);
     HEAP32[$sEnd>>2] = $76;
    }
   }
   $77 = (($sEnd) + 4|0);
   HEAP32[$77>>2] = 0;
   $78 = HEAP32[$sEnd>>2]|0;
   $79 = HEAP32[$1>>2]|0;
   $80 = HEAP32[$79>>2]|0;
   $81 = $78;
   $82 = $80;
   $83 = (($81) - ($82))|0;
   HEAP32[$n>>2] = $83;
   $84 = HEAP32[$1>>2]|0;
   $85 = HEAP32[$84>>2]|0;
   HEAP32[$z>>2] = $85;
   while(1) {
    $86 = HEAP32[$n>>2]|0;
    $87 = ($86|0)>(0);
    if ($87) {
     $88 = HEAP32[$n>>2]|0;
     $89 = (($88) - 1)|0;
     $90 = HEAP32[$z>>2]|0;
     $91 = (($90) + ($89)|0);
     $92 = HEAP8[$91>>0]|0;
     $93 = $92&255;
     $94 = (1224 + ($93)|0);
     $95 = HEAP8[$94>>0]|0;
     $96 = $95&255;
     $97 = $96 & 1;
     $98 = ($97|0)!=(0);
     $106 = $98;
    } else {
     $106 = 0;
    }
    $99 = HEAP32[$n>>2]|0;
    if (!($106)) {
     break;
    }
    $100 = (($99) + -1)|0;
    HEAP32[$n>>2] = $100;
   }
   $101 = (($99) - 1)|0;
   $102 = HEAP32[$z>>2]|0;
   $103 = (($102) + ($101)|0);
   HEAP32[$sEnd>>2] = $103;
   $104 = (($sEnd) + 4|0);
   HEAP32[$104>>2] = 1;
   $105 = HEAP32[$0>>2]|0;
   _sqlite3EndTable($105,0,$sEnd,0,0);
   STACKTOP = sp;return;
  }
 }
 $29 = HEAP32[$db>>2]|0;
 $30 = HEAP32[$4>>2]|0;
 _sqlite3SelectDelete($29,$30);
 STACKTOP = sp;return;
}
function _sqlite3Select($pParse,$p,$pDest) {
 $pParse = $pParse|0;
 $p = $p|0;
 $pDest = $pDest|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $1000 = 0, $1001 = 0, $1002 = 0, $1003 = 0, $1004 = 0, $1005 = 0, $1006 = 0, $1007 = 0, $1008 = 0, $1009 = 0, $101 = 0, $1010 = 0, $1011 = 0, $1012 = 0, $1013 = 0, $1014 = 0;
 var $1015 = 0, $1016 = 0, $1017 = 0, $1018 = 0, $1019 = 0, $102 = 0, $1020 = 0, $1021 = 0, $1022 = 0, $1023 = 0, $1024 = 0, $1025 = 0, $1026 = 0, $1027 = 0, $1028 = 0, $1029 = 0, $103 = 0, $1030 = 0, $1031 = 0, $1032 = 0;
 var $1033 = 0, $1034 = 0, $1035 = 0, $1036 = 0, $1037 = 0, $1038 = 0, $1039 = 0, $104 = 0, $1040 = 0, $1041 = 0, $1042 = 0, $1043 = 0, $1044 = 0, $1045 = 0, $1046 = 0, $1047 = 0, $1048 = 0, $1049 = 0, $105 = 0, $1050 = 0;
 var $1051 = 0, $1052 = 0, $1053 = 0, $1054 = 0, $1055 = 0, $1056 = 0, $1057 = 0, $1058 = 0, $1059 = 0, $106 = 0, $1060 = 0, $1061 = 0, $1062 = 0, $1063 = 0, $1064 = 0, $1065 = 0, $1066 = 0, $1067 = 0, $1068 = 0, $1069 = 0;
 var $107 = 0, $1070 = 0, $1071 = 0, $1072 = 0, $1073 = 0, $1074 = 0, $1075 = 0, $1076 = 0, $1077 = 0, $1078 = 0, $1079 = 0, $108 = 0, $1080 = 0, $1081 = 0, $1082 = 0, $1083 = 0, $1084 = 0, $1085 = 0, $1086 = 0, $1087 = 0;
 var $1088 = 0, $1089 = 0, $109 = 0, $1090 = 0, $1091 = 0, $1092 = 0, $1093 = 0, $1094 = 0, $1095 = 0, $1096 = 0, $1097 = 0, $1098 = 0, $1099 = 0, $11 = 0, $110 = 0, $1100 = 0, $1101 = 0, $1102 = 0, $1103 = 0, $1104 = 0;
 var $1105 = 0, $1106 = 0, $1107 = 0, $1108 = 0, $1109 = 0, $111 = 0, $1110 = 0, $1111 = 0, $1112 = 0, $1113 = 0, $1114 = 0, $1115 = 0, $1116 = 0, $1117 = 0, $1118 = 0, $1119 = 0, $112 = 0, $1120 = 0, $1121 = 0, $1122 = 0;
 var $1123 = 0, $1124 = 0, $1125 = 0, $1126 = 0, $1127 = 0, $1128 = 0, $1129 = 0, $113 = 0, $1130 = 0, $1131 = 0, $1132 = 0, $1133 = 0, $1134 = 0, $1135 = 0, $1136 = 0, $1137 = 0, $1138 = 0, $1139 = 0, $114 = 0, $1140 = 0;
 var $1141 = 0, $1142 = 0, $1143 = 0, $1144 = 0, $1145 = 0, $1146 = 0, $1147 = 0, $1148 = 0, $1149 = 0, $115 = 0, $1150 = 0, $1151 = 0, $1152 = 0, $1153 = 0, $1154 = 0, $1155 = 0, $1156 = 0, $1157 = 0, $1158 = 0, $1159 = 0;
 var $116 = 0, $1160 = 0, $1161 = 0, $1162 = 0, $1163 = 0, $1164 = 0, $1165 = 0, $1166 = 0, $1167 = 0, $1168 = 0, $1169 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0;
 var $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0, $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0;
 var $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0, $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0;
 var $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0, $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0;
 var $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0, $188 = 0, $189 = 0, $19 = 0, $190 = 0, $191 = 0, $192 = 0, $193 = 0, $194 = 0, $195 = 0, $196 = 0, $197 = 0;
 var $198 = 0, $199 = 0, $2 = 0, $20 = 0, $200 = 0, $201 = 0, $202 = 0, $203 = 0, $204 = 0, $205 = 0, $206 = 0, $207 = 0, $208 = 0, $209 = 0, $21 = 0, $210 = 0, $211 = 0, $212 = 0, $213 = 0, $214 = 0;
 var $215 = 0, $216 = 0, $217 = 0, $218 = 0, $219 = 0, $22 = 0, $220 = 0, $221 = 0, $222 = 0, $223 = 0, $224 = 0, $225 = 0, $226 = 0, $227 = 0, $228 = 0, $229 = 0, $23 = 0, $230 = 0, $231 = 0, $232 = 0;
 var $233 = 0, $234 = 0, $235 = 0, $236 = 0, $237 = 0, $238 = 0, $239 = 0, $24 = 0, $240 = 0, $241 = 0, $242 = 0, $243 = 0, $244 = 0, $245 = 0, $246 = 0, $247 = 0, $248 = 0, $249 = 0, $25 = 0, $250 = 0;
 var $251 = 0, $252 = 0, $253 = 0, $254 = 0, $255 = 0, $256 = 0, $257 = 0, $258 = 0, $259 = 0, $26 = 0, $260 = 0, $261 = 0, $262 = 0, $263 = 0, $264 = 0, $265 = 0, $266 = 0, $267 = 0, $268 = 0, $269 = 0;
 var $27 = 0, $270 = 0, $271 = 0, $272 = 0, $273 = 0, $274 = 0, $275 = 0, $276 = 0, $277 = 0, $278 = 0, $279 = 0, $28 = 0, $280 = 0, $281 = 0, $282 = 0, $283 = 0, $284 = 0, $285 = 0, $286 = 0, $287 = 0;
 var $288 = 0, $289 = 0, $29 = 0, $290 = 0, $291 = 0, $292 = 0, $293 = 0, $294 = 0, $295 = 0, $296 = 0, $297 = 0, $298 = 0, $299 = 0, $3 = 0, $30 = 0, $300 = 0, $301 = 0, $302 = 0, $303 = 0, $304 = 0;
 var $305 = 0, $306 = 0, $307 = 0, $308 = 0, $309 = 0, $31 = 0, $310 = 0, $311 = 0, $312 = 0, $313 = 0, $314 = 0, $315 = 0, $316 = 0, $317 = 0, $318 = 0, $319 = 0, $32 = 0, $320 = 0, $321 = 0, $322 = 0;
 var $323 = 0, $324 = 0, $325 = 0, $326 = 0, $327 = 0, $328 = 0, $329 = 0, $33 = 0, $330 = 0, $331 = 0, $332 = 0, $333 = 0, $334 = 0, $335 = 0, $336 = 0, $337 = 0, $338 = 0, $339 = 0, $34 = 0, $340 = 0;
 var $341 = 0, $342 = 0, $343 = 0, $344 = 0, $345 = 0, $346 = 0, $347 = 0, $348 = 0, $349 = 0, $35 = 0, $350 = 0, $351 = 0, $352 = 0, $353 = 0, $354 = 0, $355 = 0, $356 = 0, $357 = 0, $358 = 0, $359 = 0;
 var $36 = 0, $360 = 0, $361 = 0, $362 = 0, $363 = 0, $364 = 0, $365 = 0, $366 = 0, $367 = 0, $368 = 0, $369 = 0, $37 = 0, $370 = 0, $371 = 0, $372 = 0, $373 = 0, $374 = 0, $375 = 0, $376 = 0, $377 = 0;
 var $378 = 0, $379 = 0, $38 = 0, $380 = 0, $381 = 0, $382 = 0, $383 = 0, $384 = 0, $385 = 0, $386 = 0, $387 = 0, $388 = 0, $389 = 0, $39 = 0, $390 = 0, $391 = 0, $392 = 0, $393 = 0, $394 = 0, $395 = 0;
 var $396 = 0, $397 = 0, $398 = 0, $399 = 0, $4 = 0, $40 = 0, $400 = 0, $401 = 0, $402 = 0, $403 = 0, $404 = 0, $405 = 0, $406 = 0, $407 = 0, $408 = 0, $409 = 0, $41 = 0, $410 = 0, $411 = 0, $412 = 0;
 var $413 = 0, $414 = 0, $415 = 0, $416 = 0, $417 = 0, $418 = 0, $419 = 0, $42 = 0, $420 = 0, $421 = 0, $422 = 0, $423 = 0, $424 = 0, $425 = 0, $426 = 0, $427 = 0, $428 = 0, $429 = 0, $43 = 0, $430 = 0;
 var $431 = 0, $432 = 0, $433 = 0, $434 = 0, $435 = 0, $436 = 0, $437 = 0, $438 = 0, $439 = 0, $44 = 0, $440 = 0, $441 = 0, $442 = 0, $443 = 0, $444 = 0, $445 = 0, $446 = 0, $447 = 0, $448 = 0, $449 = 0;
 var $45 = 0, $450 = 0, $451 = 0, $452 = 0, $453 = 0, $454 = 0, $455 = 0, $456 = 0, $457 = 0, $458 = 0, $459 = 0, $46 = 0, $460 = 0, $461 = 0, $462 = 0, $463 = 0, $464 = 0, $465 = 0, $466 = 0, $467 = 0;
 var $468 = 0, $469 = 0, $47 = 0, $470 = 0, $471 = 0, $472 = 0, $473 = 0, $474 = 0, $475 = 0, $476 = 0, $477 = 0, $478 = 0, $479 = 0, $48 = 0, $480 = 0, $481 = 0, $482 = 0, $483 = 0, $484 = 0, $485 = 0;
 var $486 = 0, $487 = 0, $488 = 0, $489 = 0, $49 = 0, $490 = 0, $491 = 0, $492 = 0, $493 = 0, $494 = 0, $495 = 0, $496 = 0, $497 = 0, $498 = 0, $499 = 0, $5 = 0, $50 = 0, $500 = 0, $501 = 0, $502 = 0;
 var $503 = 0, $504 = 0, $505 = 0, $506 = 0, $507 = 0, $508 = 0, $509 = 0, $51 = 0, $510 = 0, $511 = 0, $512 = 0, $513 = 0, $514 = 0, $515 = 0, $516 = 0, $517 = 0, $518 = 0, $519 = 0, $52 = 0, $520 = 0;
 var $521 = 0, $522 = 0, $523 = 0, $524 = 0, $525 = 0, $526 = 0, $527 = 0, $528 = 0, $529 = 0, $53 = 0, $530 = 0, $531 = 0, $532 = 0, $533 = 0, $534 = 0, $535 = 0, $536 = 0, $537 = 0, $538 = 0, $539 = 0;
 var $54 = 0, $540 = 0, $541 = 0, $542 = 0, $543 = 0, $544 = 0, $545 = 0, $546 = 0, $547 = 0, $548 = 0, $549 = 0, $55 = 0, $550 = 0, $551 = 0, $552 = 0, $553 = 0, $554 = 0, $555 = 0, $556 = 0, $557 = 0;
 var $558 = 0, $559 = 0, $56 = 0, $560 = 0, $561 = 0, $562 = 0, $563 = 0, $564 = 0, $565 = 0, $566 = 0, $567 = 0, $568 = 0, $569 = 0, $57 = 0, $570 = 0, $571 = 0, $572 = 0, $573 = 0, $574 = 0, $575 = 0;
 var $576 = 0, $577 = 0, $578 = 0, $579 = 0, $58 = 0, $580 = 0, $581 = 0, $582 = 0, $583 = 0, $584 = 0, $585 = 0, $586 = 0, $587 = 0, $588 = 0, $589 = 0, $59 = 0, $590 = 0, $591 = 0, $592 = 0, $593 = 0;
 var $594 = 0, $595 = 0, $596 = 0, $597 = 0, $598 = 0, $599 = 0, $6 = 0, $60 = 0, $600 = 0, $601 = 0, $602 = 0, $603 = 0, $604 = 0, $605 = 0, $606 = 0, $607 = 0, $608 = 0, $609 = 0, $61 = 0, $610 = 0;
 var $611 = 0, $612 = 0, $613 = 0, $614 = 0, $615 = 0, $616 = 0, $617 = 0, $618 = 0, $619 = 0, $62 = 0, $620 = 0, $621 = 0, $622 = 0, $623 = 0, $624 = 0, $625 = 0, $626 = 0, $627 = 0, $628 = 0, $629 = 0;
 var $63 = 0, $630 = 0, $631 = 0, $632 = 0, $633 = 0, $634 = 0, $635 = 0, $636 = 0, $637 = 0, $638 = 0, $639 = 0, $64 = 0, $640 = 0, $641 = 0, $642 = 0, $643 = 0, $644 = 0, $645 = 0, $646 = 0, $647 = 0;
 var $648 = 0, $649 = 0, $65 = 0, $650 = 0, $651 = 0, $652 = 0, $653 = 0, $654 = 0, $655 = 0, $656 = 0, $657 = 0, $658 = 0, $659 = 0, $66 = 0, $660 = 0, $661 = 0, $662 = 0, $663 = 0, $664 = 0, $665 = 0;
 var $666 = 0, $667 = 0, $668 = 0, $669 = 0, $67 = 0, $670 = 0, $671 = 0, $672 = 0, $673 = 0, $674 = 0, $675 = 0, $676 = 0, $677 = 0, $678 = 0, $679 = 0, $68 = 0, $680 = 0, $681 = 0, $682 = 0, $683 = 0;
 var $684 = 0, $685 = 0, $686 = 0, $687 = 0, $688 = 0, $689 = 0, $69 = 0, $690 = 0, $691 = 0, $692 = 0, $693 = 0, $694 = 0, $695 = 0, $696 = 0, $697 = 0, $698 = 0, $699 = 0, $7 = 0, $70 = 0, $700 = 0;
 var $701 = 0, $702 = 0, $703 = 0, $704 = 0, $705 = 0, $706 = 0, $707 = 0, $708 = 0, $709 = 0, $71 = 0, $710 = 0, $711 = 0, $712 = 0, $713 = 0, $714 = 0, $715 = 0, $716 = 0, $717 = 0, $718 = 0, $719 = 0;
 var $72 = 0, $720 = 0, $721 = 0, $722 = 0, $723 = 0, $724 = 0, $725 = 0, $726 = 0, $727 = 0, $728 = 0, $729 = 0, $73 = 0, $730 = 0, $731 = 0, $732 = 0, $733 = 0, $734 = 0, $735 = 0, $736 = 0, $737 = 0;
 var $738 = 0, $739 = 0, $74 = 0, $740 = 0, $741 = 0, $742 = 0, $743 = 0, $744 = 0, $745 = 0, $746 = 0, $747 = 0, $748 = 0, $749 = 0, $75 = 0, $750 = 0, $751 = 0, $752 = 0, $753 = 0, $754 = 0, $755 = 0;
 var $756 = 0, $757 = 0, $758 = 0, $759 = 0, $76 = 0, $760 = 0, $761 = 0, $762 = 0, $763 = 0, $764 = 0, $765 = 0, $766 = 0, $767 = 0, $768 = 0, $769 = 0, $77 = 0, $770 = 0, $771 = 0, $772 = 0, $773 = 0;
 var $774 = 0, $775 = 0, $776 = 0, $777 = 0, $778 = 0, $779 = 0, $78 = 0, $780 = 0, $781 = 0, $782 = 0, $783 = 0, $784 = 0, $785 = 0, $786 = 0, $787 = 0, $788 = 0, $789 = 0, $79 = 0, $790 = 0, $791 = 0;
 var $792 = 0, $793 = 0, $794 = 0, $795 = 0, $796 = 0, $797 = 0, $798 = 0, $799 = 0, $8 = 0, $80 = 0, $800 = 0, $801 = 0, $802 = 0, $803 = 0, $804 = 0, $805 = 0, $806 = 0, $807 = 0, $808 = 0, $809 = 0;
 var $81 = 0, $810 = 0, $811 = 0, $812 = 0, $813 = 0, $814 = 0, $815 = 0, $816 = 0, $817 = 0, $818 = 0, $819 = 0, $82 = 0, $820 = 0, $821 = 0, $822 = 0, $823 = 0, $824 = 0, $825 = 0, $826 = 0, $827 = 0;
 var $828 = 0, $829 = 0, $83 = 0, $830 = 0, $831 = 0, $832 = 0, $833 = 0, $834 = 0, $835 = 0, $836 = 0, $837 = 0, $838 = 0, $839 = 0, $84 = 0, $840 = 0, $841 = 0, $842 = 0, $843 = 0, $844 = 0, $845 = 0;
 var $846 = 0, $847 = 0, $848 = 0, $849 = 0, $85 = 0, $850 = 0, $851 = 0, $852 = 0, $853 = 0, $854 = 0, $855 = 0, $856 = 0, $857 = 0, $858 = 0, $859 = 0, $86 = 0, $860 = 0, $861 = 0, $862 = 0, $863 = 0;
 var $864 = 0, $865 = 0, $866 = 0, $867 = 0, $868 = 0, $869 = 0, $87 = 0, $870 = 0, $871 = 0, $872 = 0, $873 = 0, $874 = 0, $875 = 0, $876 = 0, $877 = 0, $878 = 0, $879 = 0, $88 = 0, $880 = 0, $881 = 0;
 var $882 = 0, $883 = 0, $884 = 0, $885 = 0, $886 = 0, $887 = 0, $888 = 0, $889 = 0, $89 = 0, $890 = 0, $891 = 0, $892 = 0, $893 = 0, $894 = 0, $895 = 0, $896 = 0, $897 = 0, $898 = 0, $899 = 0, $9 = 0;
 var $90 = 0, $900 = 0, $901 = 0, $902 = 0, $903 = 0, $904 = 0, $905 = 0, $906 = 0, $907 = 0, $908 = 0, $909 = 0, $91 = 0, $910 = 0, $911 = 0, $912 = 0, $913 = 0, $914 = 0, $915 = 0, $916 = 0, $917 = 0;
 var $918 = 0, $919 = 0, $92 = 0, $920 = 0, $921 = 0, $922 = 0, $923 = 0, $924 = 0, $925 = 0, $926 = 0, $927 = 0, $928 = 0, $929 = 0, $93 = 0, $930 = 0, $931 = 0, $932 = 0, $933 = 0, $934 = 0, $935 = 0;
 var $936 = 0, $937 = 0, $938 = 0, $939 = 0, $94 = 0, $940 = 0, $941 = 0, $942 = 0, $943 = 0, $944 = 0, $945 = 0, $946 = 0, $947 = 0, $948 = 0, $949 = 0, $95 = 0, $950 = 0, $951 = 0, $952 = 0, $953 = 0;
 var $954 = 0, $955 = 0, $956 = 0, $957 = 0, $958 = 0, $959 = 0, $96 = 0, $960 = 0, $961 = 0, $962 = 0, $963 = 0, $964 = 0, $965 = 0, $966 = 0, $967 = 0, $968 = 0, $969 = 0, $97 = 0, $970 = 0, $971 = 0;
 var $972 = 0, $973 = 0, $974 = 0, $975 = 0, $976 = 0, $977 = 0, $978 = 0, $979 = 0, $98 = 0, $980 = 0, $981 = 0, $982 = 0, $983 = 0, $984 = 0, $985 = 0, $986 = 0, $987 = 0, $988 = 0, $989 = 0, $99 = 0;
 var $990 = 0, $991 = 0, $992 = 0, $993 = 0, $994 = 0, $995 = 0, $996 = 0, $997 = 0, $998 = 0, $999 = 0, $addrEnd = 0, $addrOutputRow = 0, $addrReset = 0, $addrSetAbort = 0, $addrSortingIdx = 0, $addrTop = 0, $addrTopOfLoop = 0, $db = 0, $dest = 0, $flag = 0;
 var $groupBySort = 0, $i = 0, $iAMem = 0, $iAbortFlag = 0, $iBMem = 0, $iCsr = 0, $iDb = 0, $iEnd = 0, $iRestoreSelectId = 0, $iRoot = 0, $iUseFlag = 0, $isAgg = 0, $isAggSub = 0, $j = 0, $j1 = 0, $k = 0, $nCol = 0, $nGroupBy = 0, $onceAddr = 0, $or$cond = 0;
 var $orderByGrp = 0, $pBest = 0, $pCol = 0, $pDel = 0, $pEList = 0, $pGroupBy = 0, $pHaving = 0, $pIdx = 0, $pItem = 0, $pItem1 = 0, $pKeyInfo = 0, $pKeyInfo2 = 0, $pKeyInfo3 = 0, $pMinMax = 0, $pSub = 0, $pTab = 0, $pTabList = 0, $pWInfo = 0, $pWhere = 0, $r1 = 0;
 var $r2 = 0, $rc = 0, $regBase = 0, $regOutputRow = 0, $regRecord = 0, $regReset = 0, $retAddr = 0, $sAggInfo = 0, $sDistinct = 0, $sNC = 0, $sSort = 0, $sortOut = 0, $sortPTab = 0, $topAddr = 0, $v = 0, $wctrlFlags = 0, dest = 0, label = 0, sp = 0, stop = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 400|0;
 $0 = sp + 384|0;
 $1 = sp + 380|0;
 $2 = sp + 376|0;
 $3 = sp + 372|0;
 $i = sp + 368|0;
 $j = sp + 364|0;
 $pWInfo = sp + 360|0;
 $v = sp + 356|0;
 $isAgg = sp + 352|0;
 $pEList = sp + 348|0;
 $pTabList = sp + 344|0;
 $pWhere = sp + 340|0;
 $pGroupBy = sp + 336|0;
 $pHaving = sp + 332|0;
 $rc = sp + 328|0;
 $sDistinct = sp + 316|0;
 $sSort = sp + 288|0;
 $sAggInfo = sp + 240|0;
 $iEnd = sp + 236|0;
 $db = sp + 232|0;
 $iRestoreSelectId = sp + 228|0;
 $pItem = sp + 224|0;
 $dest = sp + 204|0;
 $pSub = sp + 200|0;
 $isAggSub = sp + 196|0;
 $addrTop = sp + 192|0;
 $topAddr = sp + 188|0;
 $onceAddr = sp + 184|0;
 $retAddr = sp + 180|0;
 $pKeyInfo = sp + 176|0;
 $wctrlFlags = sp + 388|0;
 $sNC = sp + 144|0;
 $iAMem = sp + 140|0;
 $iBMem = sp + 136|0;
 $iUseFlag = sp + 132|0;
 $iAbortFlag = sp + 128|0;
 $groupBySort = sp + 124|0;
 $addrEnd = sp + 120|0;
 $sortPTab = sp + 116|0;
 $sortOut = sp + 112|0;
 $orderByGrp = sp + 108|0;
 $k = sp + 104|0;
 $pItem1 = sp + 100|0;
 $pKeyInfo2 = sp + 96|0;
 $j1 = sp + 92|0;
 $addrOutputRow = sp + 88|0;
 $regOutputRow = sp + 84|0;
 $addrSetAbort = sp + 80|0;
 $addrTopOfLoop = sp + 76|0;
 $addrSortingIdx = sp + 72|0;
 $addrReset = sp + 68|0;
 $regReset = sp + 64|0;
 $regBase = sp + 60|0;
 $regRecord = sp + 56|0;
 $nCol = sp + 52|0;
 $nGroupBy = sp + 48|0;
 $pCol = sp + 44|0;
 $r1 = sp + 40|0;
 $r2 = sp + 36|0;
 $pDel = sp + 32|0;
 $pTab = sp + 28|0;
 $iDb = sp + 24|0;
 $iCsr = sp + 20|0;
 $pIdx = sp + 16|0;
 $pKeyInfo3 = sp + 12|0;
 $pBest = sp + 8|0;
 $iRoot = sp + 4|0;
 $pMinMax = sp;
 $flag = sp + 390|0;
 HEAP32[$1>>2] = $pParse;
 HEAP32[$2>>2] = $p;
 HEAP32[$3>>2] = $pDest;
 HEAP32[$rc>>2] = 1;
 $4 = HEAP32[$1>>2]|0;
 $5 = (($4) + 464|0);
 $6 = HEAP32[$5>>2]|0;
 HEAP32[$iRestoreSelectId>>2] = $6;
 $7 = HEAP32[$1>>2]|0;
 $8 = (($7) + 468|0);
 $9 = HEAP32[$8>>2]|0;
 $10 = (($9) + 1)|0;
 HEAP32[$8>>2] = $10;
 $11 = HEAP32[$1>>2]|0;
 $12 = (($11) + 464|0);
 HEAP32[$12>>2] = $9;
 $13 = HEAP32[$1>>2]|0;
 $14 = HEAP32[$13>>2]|0;
 HEAP32[$db>>2] = $14;
 $15 = HEAP32[$2>>2]|0;
 $16 = ($15|0)==(0|0);
 if (!($16)) {
  $17 = HEAP32[$db>>2]|0;
  $18 = (($17) + 64|0);
  $19 = HEAP8[$18>>0]|0;
  $20 = $19&255;
  $21 = ($20|0)!=(0);
  if (!($21)) {
   $22 = HEAP32[$1>>2]|0;
   $23 = (($22) + 64|0);
   $24 = HEAP32[$23>>2]|0;
   $25 = ($24|0)!=(0);
   if (!($25)) {
    $26 = HEAP32[$1>>2]|0;
    $27 = (_sqlite3AuthCheck($26,21,0,0,0)|0);
    $28 = ($27|0)!=(0);
    if ($28) {
     HEAP32[$0>>2] = 1;
     $1169 = HEAP32[$0>>2]|0;
     STACKTOP = sp;return ($1169|0);
    }
    dest=$sAggInfo+0|0; stop=dest+48|0; do { HEAP32[dest>>2]=0|0; dest=dest+4|0; } while ((dest|0) < (stop|0));
    $29 = HEAP32[$3>>2]|0;
    $30 = HEAP8[$29>>0]|0;
    $31 = $30&255;
    $32 = ($31|0)<=(8);
    if ($32) {
     $33 = HEAP32[$db>>2]|0;
     $34 = HEAP32[$2>>2]|0;
     $35 = (($34) + 48|0);
     $36 = HEAP32[$35>>2]|0;
     _sqlite3ExprListDelete($33,$36);
     $37 = HEAP32[$2>>2]|0;
     $38 = (($37) + 48|0);
     HEAP32[$38>>2] = 0;
     $39 = HEAP32[$2>>2]|0;
     $40 = (($39) + 6|0);
     $41 = HEAP16[$40>>1]|0;
     $42 = $41&65535;
     $43 = $42 & -2;
     $44 = $43&65535;
     HEAP16[$40>>1] = $44;
    }
    $45 = HEAP32[$1>>2]|0;
    $46 = HEAP32[$2>>2]|0;
    _sqlite3SelectPrep($45,$46,0);
    ;HEAP32[$sSort+0>>2]=0|0;HEAP32[$sSort+4>>2]=0|0;HEAP32[$sSort+8>>2]=0|0;HEAP32[$sSort+12>>2]=0|0;HEAP32[$sSort+16>>2]=0|0;HEAP32[$sSort+20>>2]=0|0;HEAP32[$sSort+24>>2]=0|0;
    $47 = HEAP32[$2>>2]|0;
    $48 = (($47) + 48|0);
    $49 = HEAP32[$48>>2]|0;
    HEAP32[$sSort>>2] = $49;
    $50 = HEAP32[$2>>2]|0;
    $51 = (($50) + 32|0);
    $52 = HEAP32[$51>>2]|0;
    HEAP32[$pTabList>>2] = $52;
    $53 = HEAP32[$2>>2]|0;
    $54 = HEAP32[$53>>2]|0;
    HEAP32[$pEList>>2] = $54;
    $55 = HEAP32[$1>>2]|0;
    $56 = (($55) + 64|0);
    $57 = HEAP32[$56>>2]|0;
    $58 = ($57|0)!=(0);
    L12: do {
     if (!($58)) {
      $59 = HEAP32[$db>>2]|0;
      $60 = (($59) + 64|0);
      $61 = HEAP8[$60>>0]|0;
      $62 = $61&255;
      $63 = ($62|0)!=(0);
      if (!($63)) {
       $64 = HEAP32[$2>>2]|0;
       $65 = (($64) + 6|0);
       $66 = HEAP16[$65>>1]|0;
       $67 = $66&65535;
       $68 = $67 & 4;
       $69 = ($68|0)!=(0);
       $70 = $69&1;
       HEAP32[$isAgg>>2] = $70;
       $71 = HEAP32[$1>>2]|0;
       $72 = (_sqlite3GetVdbe($71)|0);
       HEAP32[$v>>2] = $72;
       $73 = HEAP32[$v>>2]|0;
       $74 = ($73|0)==(0|0);
       if (!($74)) {
        $75 = HEAP32[$1>>2]|0;
        $76 = HEAP32[$3>>2]|0;
        $77 = HEAP32[$pEList>>2]|0;
        $78 = HEAP32[$77>>2]|0;
        $79 = (_checkForMultiColumnSelectError($75,$76,$78)|0);
        $80 = ($79|0)!=(0);
        if (!($80)) {
         HEAP32[$i>>2] = 0;
         while(1) {
          $81 = HEAP32[$2>>2]|0;
          $82 = (($81) + 52|0);
          $83 = HEAP32[$82>>2]|0;
          $84 = ($83|0)!=(0|0);
          if ($84) {
           break;
          }
          $85 = HEAP32[$i>>2]|0;
          $86 = HEAP32[$pTabList>>2]|0;
          $87 = HEAP32[$86>>2]|0;
          $88 = ($85|0)<($87|0);
          if (!($88)) {
           break;
          }
          $89 = HEAP32[$i>>2]|0;
          $90 = HEAP32[$pTabList>>2]|0;
          $91 = (($90) + 8|0);
          $92 = (($91) + (($89*72)|0)|0);
          HEAP32[$pItem>>2] = $92;
          $93 = HEAP32[$pItem>>2]|0;
          $94 = (($93) + 20|0);
          $95 = HEAP32[$94>>2]|0;
          HEAP32[$pSub>>2] = $95;
          $96 = HEAP32[$pSub>>2]|0;
          $97 = ($96|0)==(0|0);
          do {
           if (!($97)) {
            $98 = HEAP32[$pItem>>2]|0;
            $99 = (($98) + 24|0);
            $100 = HEAP32[$99>>2]|0;
            $101 = ($100|0)!=(0);
            if ($101) {
             $102 = HEAP32[$pItem>>2]|0;
             $103 = (($102) + 37|0);
             $104 = HEAP8[$103>>0]|0;
             $105 = ($104&255) >>> 2;
             $106 = $105 & 1;
             $107 = $106&255;
             $108 = ($107|0)==(0);
             if (!($108)) {
              break;
             }
             $109 = HEAP32[$v>>2]|0;
             $110 = HEAP32[$pItem>>2]|0;
             $111 = (($110) + 28|0);
             $112 = HEAP32[$111>>2]|0;
             $113 = HEAP32[$pItem>>2]|0;
             $114 = (($113) + 24|0);
             $115 = HEAP32[$114>>2]|0;
             (_sqlite3VdbeAddOp2($109,17,$112,$115)|0);
             break;
            }
            $116 = HEAP32[$2>>2]|0;
            $117 = (_sqlite3SelectExprHeight($116)|0);
            $118 = HEAP32[$1>>2]|0;
            $119 = (($118) + 460|0);
            $120 = HEAP32[$119>>2]|0;
            $121 = (($120) + ($117))|0;
            HEAP32[$119>>2] = $121;
            $122 = HEAP32[$pSub>>2]|0;
            $123 = (($122) + 6|0);
            $124 = HEAP16[$123>>1]|0;
            $125 = $124&65535;
            $126 = $125 & 4;
            $127 = ($126|0)!=(0);
            $128 = $127&1;
            HEAP32[$isAggSub>>2] = $128;
            $129 = HEAP32[$1>>2]|0;
            $130 = HEAP32[$2>>2]|0;
            $131 = HEAP32[$i>>2]|0;
            $132 = HEAP32[$isAgg>>2]|0;
            $133 = HEAP32[$isAggSub>>2]|0;
            $134 = (_flattenSubquery($129,$130,$131,$132,$133)|0);
            $135 = ($134|0)!=(0);
            L27: do {
             if ($135) {
              $136 = HEAP32[$isAggSub>>2]|0;
              $137 = ($136|0)!=(0);
              if ($137) {
               HEAP32[$isAgg>>2] = 1;
               $138 = HEAP32[$2>>2]|0;
               $139 = (($138) + 6|0);
               $140 = HEAP16[$139>>1]|0;
               $141 = $140&65535;
               $142 = $141 | 4;
               $143 = $142&65535;
               HEAP16[$139>>1] = $143;
              }
              HEAP32[$i>>2] = -1;
             } else {
              $144 = HEAP32[$pTabList>>2]|0;
              $145 = HEAP32[$144>>2]|0;
              $146 = ($145|0)==(1);
              do {
               if ($146) {
                $147 = HEAP32[$db>>2]|0;
                $148 = (($147) + 60|0);
                $149 = HEAP16[$148>>1]|0;
                $150 = $149&65535;
                $151 = $150 & 256;
                $152 = ($151|0)==(0);
                if (!($152)) {
                 break;
                }
                $153 = HEAP32[$v>>2]|0;
                $154 = (_sqlite3VdbeCurrentAddr($153)|0);
                $155 = (($154) + 1)|0;
                HEAP32[$addrTop>>2] = $155;
                $156 = HEAP32[$1>>2]|0;
                $157 = (($156) + 72|0);
                $158 = HEAP32[$157>>2]|0;
                $159 = (($158) + 1)|0;
                HEAP32[$157>>2] = $159;
                $160 = HEAP32[$pItem>>2]|0;
                $161 = (($160) + 28|0);
                HEAP32[$161>>2] = $159;
                $162 = HEAP32[$v>>2]|0;
                $163 = HEAP32[$pItem>>2]|0;
                $164 = (($163) + 28|0);
                $165 = HEAP32[$164>>2]|0;
                $166 = HEAP32[$addrTop>>2]|0;
                (_sqlite3VdbeAddOp3($162,20,$165,0,$166)|0);
                $167 = HEAP32[$addrTop>>2]|0;
                $168 = HEAP32[$pItem>>2]|0;
                $169 = (($168) + 24|0);
                HEAP32[$169>>2] = $167;
                $170 = HEAP32[$pItem>>2]|0;
                $171 = (($170) + 28|0);
                $172 = HEAP32[$171>>2]|0;
                _sqlite3SelectDestInit($dest,13,$172);
                $173 = HEAP32[$1>>2]|0;
                $174 = (($173) + 468|0);
                $175 = HEAP32[$174>>2]|0;
                $176 = $175&255;
                $177 = HEAP32[$pItem>>2]|0;
                $178 = (($177) + 38|0);
                HEAP8[$178>>0] = $176;
                $179 = HEAP32[$1>>2]|0;
                $180 = HEAP32[$pSub>>2]|0;
                (_sqlite3Select($179,$180,$dest)|0);
                $181 = HEAP32[$pSub>>2]|0;
                $182 = (($181) + 24|0);
                $183 = $182;
                $184 = $183;
                $185 = HEAP32[$184>>2]|0;
                $186 = (($183) + 4)|0;
                $187 = $186;
                $188 = HEAP32[$187>>2]|0;
                $189 = (_sqlite3LogEst($185,$188)|0);
                $190 = HEAP32[$pItem>>2]|0;
                $191 = (($190) + 16|0);
                $192 = HEAP32[$191>>2]|0;
                $193 = (($192) + 28|0);
                HEAP16[$193>>1] = $189;
                $194 = HEAP32[$pItem>>2]|0;
                $195 = (($194) + 37|0);
                $196 = HEAP8[$195>>0]|0;
                $197 = $196 & -5;
                $198 = $197 | 4;
                HEAP8[$195>>0] = $198;
                $199 = (($dest) + 8|0);
                $200 = HEAP32[$199>>2]|0;
                $201 = HEAP32[$pItem>>2]|0;
                $202 = (($201) + 32|0);
                HEAP32[$202>>2] = $200;
                $203 = HEAP32[$v>>2]|0;
                $204 = HEAP32[$pItem>>2]|0;
                $205 = (($204) + 28|0);
                $206 = HEAP32[$205>>2]|0;
                (_sqlite3VdbeAddOp1($203,21,$206)|0);
                $207 = HEAP32[$v>>2]|0;
                $208 = HEAP32[$addrTop>>2]|0;
                $209 = (($208) - 1)|0;
                _sqlite3VdbeJumpHere($207,$209);
                $210 = HEAP32[$1>>2]|0;
                _sqlite3ClearTempRegCache($210);
                break L27;
               }
              } while(0);
              HEAP32[$onceAddr>>2] = 0;
              $211 = HEAP32[$1>>2]|0;
              $212 = (($211) + 72|0);
              $213 = HEAP32[$212>>2]|0;
              $214 = (($213) + 1)|0;
              HEAP32[$212>>2] = $214;
              $215 = HEAP32[$pItem>>2]|0;
              $216 = (($215) + 28|0);
              HEAP32[$216>>2] = $214;
              $217 = HEAP32[$v>>2]|0;
              $218 = HEAP32[$pItem>>2]|0;
              $219 = (($218) + 28|0);
              $220 = HEAP32[$219>>2]|0;
              $221 = (_sqlite3VdbeAddOp2($217,25,0,$220)|0);
              HEAP32[$topAddr>>2] = $221;
              $222 = HEAP32[$topAddr>>2]|0;
              $223 = (($222) + 1)|0;
              $224 = HEAP32[$pItem>>2]|0;
              $225 = (($224) + 24|0);
              HEAP32[$225>>2] = $223;
              $226 = HEAP32[$pItem>>2]|0;
              $227 = (($226) + 37|0);
              $228 = HEAP8[$227>>0]|0;
              $229 = ($228&255) >>> 1;
              $230 = $229 & 1;
              $231 = $230&255;
              $232 = ($231|0)==(0);
              if ($232) {
               $233 = HEAP32[$1>>2]|0;
               $234 = (_sqlite3CodeOnce($233)|0);
               HEAP32[$onceAddr>>2] = $234;
              }
              $235 = HEAP32[$pItem>>2]|0;
              $236 = (($235) + 40|0);
              $237 = HEAP32[$236>>2]|0;
              _sqlite3SelectDestInit($dest,12,$237);
              $238 = HEAP32[$1>>2]|0;
              $239 = (($238) + 468|0);
              $240 = HEAP32[$239>>2]|0;
              $241 = $240&255;
              $242 = HEAP32[$pItem>>2]|0;
              $243 = (($242) + 38|0);
              HEAP8[$243>>0] = $241;
              $244 = HEAP32[$1>>2]|0;
              $245 = HEAP32[$pSub>>2]|0;
              (_sqlite3Select($244,$245,$dest)|0);
              $246 = HEAP32[$pSub>>2]|0;
              $247 = (($246) + 24|0);
              $248 = $247;
              $249 = $248;
              $250 = HEAP32[$249>>2]|0;
              $251 = (($248) + 4)|0;
              $252 = $251;
              $253 = HEAP32[$252>>2]|0;
              $254 = (_sqlite3LogEst($250,$253)|0);
              $255 = HEAP32[$pItem>>2]|0;
              $256 = (($255) + 16|0);
              $257 = HEAP32[$256>>2]|0;
              $258 = (($257) + 28|0);
              HEAP16[$258>>1] = $254;
              $259 = HEAP32[$onceAddr>>2]|0;
              $260 = ($259|0)!=(0);
              if ($260) {
               $261 = HEAP32[$v>>2]|0;
               $262 = HEAP32[$onceAddr>>2]|0;
               _sqlite3VdbeJumpHere($261,$262);
              }
              $263 = HEAP32[$v>>2]|0;
              $264 = HEAP32[$pItem>>2]|0;
              $265 = (($264) + 28|0);
              $266 = HEAP32[$265>>2]|0;
              $267 = (_sqlite3VdbeAddOp1($263,18,$266)|0);
              HEAP32[$retAddr>>2] = $267;
              $268 = HEAP32[$v>>2]|0;
              $269 = HEAP32[$topAddr>>2]|0;
              $270 = HEAP32[$retAddr>>2]|0;
              _sqlite3VdbeChangeP1($268,$269,$270);
              $271 = HEAP32[$1>>2]|0;
              _sqlite3ClearTempRegCache($271);
             }
            } while(0);
            $272 = HEAP32[$db>>2]|0;
            $273 = (($272) + 64|0);
            $274 = HEAP8[$273>>0]|0;
            $275 = ($274<<24>>24)!=(0);
            if ($275) {
             break L12;
            }
            $276 = HEAP32[$2>>2]|0;
            $277 = (_sqlite3SelectExprHeight($276)|0);
            $278 = HEAP32[$1>>2]|0;
            $279 = (($278) + 460|0);
            $280 = HEAP32[$279>>2]|0;
            $281 = (($280) - ($277))|0;
            HEAP32[$279>>2] = $281;
            $282 = HEAP32[$2>>2]|0;
            $283 = (($282) + 32|0);
            $284 = HEAP32[$283>>2]|0;
            HEAP32[$pTabList>>2] = $284;
            $285 = HEAP32[$3>>2]|0;
            $286 = HEAP8[$285>>0]|0;
            $287 = $286&255;
            $288 = ($287|0)<=(8);
            if (!($288)) {
             $289 = HEAP32[$2>>2]|0;
             $290 = (($289) + 48|0);
             $291 = HEAP32[$290>>2]|0;
             HEAP32[$sSort>>2] = $291;
            }
           }
          } while(0);
          $292 = HEAP32[$i>>2]|0;
          $293 = (($292) + 1)|0;
          HEAP32[$i>>2] = $293;
         }
         $294 = HEAP32[$2>>2]|0;
         $295 = HEAP32[$294>>2]|0;
         HEAP32[$pEList>>2] = $295;
         $296 = HEAP32[$2>>2]|0;
         $297 = (($296) + 36|0);
         $298 = HEAP32[$297>>2]|0;
         HEAP32[$pWhere>>2] = $298;
         $299 = HEAP32[$2>>2]|0;
         $300 = (($299) + 40|0);
         $301 = HEAP32[$300>>2]|0;
         HEAP32[$pGroupBy>>2] = $301;
         $302 = HEAP32[$2>>2]|0;
         $303 = (($302) + 44|0);
         $304 = HEAP32[$303>>2]|0;
         HEAP32[$pHaving>>2] = $304;
         $305 = HEAP32[$2>>2]|0;
         $306 = (($305) + 6|0);
         $307 = HEAP16[$306>>1]|0;
         $308 = $307&65535;
         $309 = $308 & 1;
         $310 = ($309|0)!=(0);
         $311 = $310&1;
         $312 = $311&255;
         HEAP8[$sDistinct>>0] = $312;
         $313 = HEAP32[$2>>2]|0;
         $314 = (($313) + 52|0);
         $315 = HEAP32[$314>>2]|0;
         $316 = ($315|0)!=(0|0);
         if ($316) {
          $317 = HEAP32[$1>>2]|0;
          $318 = HEAP32[$2>>2]|0;
          $319 = HEAP32[$3>>2]|0;
          $320 = (_multiSelect($317,$318,$319)|0);
          HEAP32[$rc>>2] = $320;
          $321 = HEAP32[$iRestoreSelectId>>2]|0;
          $322 = HEAP32[$1>>2]|0;
          $323 = (($322) + 464|0);
          HEAP32[$323>>2] = $321;
          $324 = HEAP32[$rc>>2]|0;
          HEAP32[$0>>2] = $324;
          $1169 = HEAP32[$0>>2]|0;
          STACKTOP = sp;return ($1169|0);
         }
         $325 = HEAP32[$2>>2]|0;
         $326 = (($325) + 6|0);
         $327 = HEAP16[$326>>1]|0;
         $328 = $327&65535;
         $329 = $328 & 5;
         $330 = ($329|0)==(1);
         if ($330) {
          $331 = HEAP32[$sSort>>2]|0;
          $332 = HEAP32[$2>>2]|0;
          $333 = HEAP32[$332>>2]|0;
          $334 = (_sqlite3ExprListCompare($331,$333,-1)|0);
          $335 = ($334|0)==(0);
          if ($335) {
           $336 = HEAP32[$2>>2]|0;
           $337 = (($336) + 6|0);
           $338 = HEAP16[$337>>1]|0;
           $339 = $338&65535;
           $340 = $339 & -2;
           $341 = $340&65535;
           HEAP16[$337>>1] = $341;
           $342 = HEAP32[$db>>2]|0;
           $343 = HEAP32[$2>>2]|0;
           $344 = HEAP32[$343>>2]|0;
           $345 = (_sqlite3ExprListDup($342,$344,0)|0);
           $346 = HEAP32[$2>>2]|0;
           $347 = (($346) + 40|0);
           HEAP32[$347>>2] = $345;
           $348 = HEAP32[$2>>2]|0;
           $349 = (($348) + 40|0);
           $350 = HEAP32[$349>>2]|0;
           HEAP32[$pGroupBy>>2] = $350;
           HEAP32[$sSort>>2] = 0;
          }
         }
         $351 = HEAP32[$sSort>>2]|0;
         $352 = ($351|0)!=(0|0);
         if ($352) {
          $353 = HEAP32[$1>>2]|0;
          $354 = HEAP32[$sSort>>2]|0;
          $355 = (_keyInfoFromExprList($353,$354,0,0)|0);
          HEAP32[$pKeyInfo>>2] = $355;
          $356 = HEAP32[$1>>2]|0;
          $357 = (($356) + 68|0);
          $358 = HEAP32[$357>>2]|0;
          $359 = (($358) + 1)|0;
          HEAP32[$357>>2] = $359;
          $360 = (($sSort) + 8|0);
          HEAP32[$360>>2] = $358;
          $361 = HEAP32[$v>>2]|0;
          $362 = (($sSort) + 8|0);
          $363 = HEAP32[$362>>2]|0;
          $364 = HEAP32[$sSort>>2]|0;
          $365 = HEAP32[$364>>2]|0;
          $366 = (($365) + 1)|0;
          $367 = HEAP32[$pEList>>2]|0;
          $368 = HEAP32[$367>>2]|0;
          $369 = (($366) + ($368))|0;
          $370 = HEAP32[$pKeyInfo>>2]|0;
          $371 = (_sqlite3VdbeAddOp4($361,57,$363,$369,0,$370,-6)|0);
          $372 = (($sSort) + 20|0);
          HEAP32[$372>>2] = $371;
         } else {
          $373 = (($sSort) + 20|0);
          HEAP32[$373>>2] = -1;
         }
         $374 = HEAP32[$3>>2]|0;
         $375 = HEAP8[$374>>0]|0;
         $376 = $375&255;
         $377 = ($376|0)==(12);
         if ($377) {
          $378 = HEAP32[$v>>2]|0;
          $379 = HEAP32[$3>>2]|0;
          $380 = (($379) + 4|0);
          $381 = HEAP32[$380>>2]|0;
          $382 = HEAP32[$pEList>>2]|0;
          $383 = HEAP32[$382>>2]|0;
          (_sqlite3VdbeAddOp2($378,57,$381,$383)|0);
         }
         $384 = HEAP32[$v>>2]|0;
         $385 = (_sqlite3VdbeMakeLabel($384)|0);
         HEAP32[$iEnd>>2] = $385;
         $386 = HEAP32[$2>>2]|0;
         $387 = (($386) + 24|0);
         $388 = $387;
         $389 = $388;
         HEAP32[$389>>2] = -1;
         $390 = (($388) + 4)|0;
         $391 = $390;
         HEAP32[$391>>2] = 2147483647;
         $392 = HEAP32[$1>>2]|0;
         $393 = HEAP32[$2>>2]|0;
         $394 = HEAP32[$iEnd>>2]|0;
         _computeLimitRegisters($392,$393,$394);
         $395 = HEAP32[$2>>2]|0;
         $396 = (($395) + 8|0);
         $397 = HEAP32[$396>>2]|0;
         $398 = ($397|0)==(0);
         if ($398) {
          $399 = (($sSort) + 20|0);
          $400 = HEAP32[$399>>2]|0;
          $401 = ($400|0)>=(0);
          if ($401) {
           $402 = HEAP32[$v>>2]|0;
           $403 = (($sSort) + 20|0);
           $404 = HEAP32[$403>>2]|0;
           $405 = (_sqlite3VdbeGetOp($402,$404)|0);
           HEAP8[$405>>0] = 58;
           $406 = (($sSort) + 24|0);
           $407 = HEAP8[$406>>0]|0;
           $408 = $407&255;
           $409 = $408 | 1;
           $410 = $409&255;
           HEAP8[$406>>0] = $410;
          }
         }
         $411 = HEAP32[$2>>2]|0;
         $412 = (($411) + 6|0);
         $413 = HEAP16[$412>>1]|0;
         $414 = $413&65535;
         $415 = $414 & 1;
         $416 = ($415|0)!=(0);
         if ($416) {
          $417 = HEAP32[$1>>2]|0;
          $418 = (($417) + 68|0);
          $419 = HEAP32[$418>>2]|0;
          $420 = (($419) + 1)|0;
          HEAP32[$418>>2] = $420;
          $421 = (($sDistinct) + 4|0);
          HEAP32[$421>>2] = $419;
          $422 = HEAP32[$v>>2]|0;
          $423 = (($sDistinct) + 4|0);
          $424 = HEAP32[$423>>2]|0;
          $425 = HEAP32[$1>>2]|0;
          $426 = HEAP32[$2>>2]|0;
          $427 = HEAP32[$426>>2]|0;
          $428 = (_keyInfoFromExprList($425,$427,0,0)|0);
          $429 = (_sqlite3VdbeAddOp4($422,57,$424,0,0,$428,-6)|0);
          $430 = (($sDistinct) + 8|0);
          HEAP32[$430>>2] = $429;
          $431 = HEAP32[$v>>2]|0;
          _sqlite3VdbeChangeP5($431,8);
          $432 = (($sDistinct) + 1|0);
          HEAP8[$432>>0] = 3;
         } else {
          $433 = (($sDistinct) + 1|0);
          HEAP8[$433>>0] = 0;
         }
         $434 = HEAP32[$isAgg>>2]|0;
         $435 = ($434|0)==(0);
         $436 = HEAP32[$pGroupBy>>2]|0;
         $437 = ($436|0)==(0|0);
         $or$cond = $435 & $437;
         if ($or$cond) {
          $438 = HEAP8[$sDistinct>>0]|0;
          $439 = $438&255;
          $440 = ($439|0)!=(0);
          $441 = $440 ? 1024 : 0;
          $442 = $441&65535;
          HEAP16[$wctrlFlags>>1] = $442;
          $443 = HEAP32[$1>>2]|0;
          $444 = HEAP32[$pTabList>>2]|0;
          $445 = HEAP32[$pWhere>>2]|0;
          $446 = HEAP32[$sSort>>2]|0;
          $447 = HEAP32[$2>>2]|0;
          $448 = HEAP32[$447>>2]|0;
          $449 = HEAP16[$wctrlFlags>>1]|0;
          $450 = (_sqlite3WhereBegin($443,$444,$445,$446,$448,$449,0)|0);
          HEAP32[$pWInfo>>2] = $450;
          $451 = HEAP32[$pWInfo>>2]|0;
          $452 = ($451|0)==(0|0);
          if ($452) {
           break;
          }
          $453 = HEAP32[$pWInfo>>2]|0;
          $454 = (_sqlite3WhereOutputRowCount($453)|0);
          $455 = tempRet0;
          $456 = HEAP32[$2>>2]|0;
          $457 = (($456) + 24|0);
          $458 = $457;
          $459 = $458;
          $460 = HEAP32[$459>>2]|0;
          $461 = (($458) + 4)|0;
          $462 = $461;
          $463 = HEAP32[$462>>2]|0;
          $464 = ($455>>>0)<($463>>>0);
          $465 = ($454>>>0)<($460>>>0);
          $466 = ($455|0)==($463|0);
          $467 = $466 & $465;
          $468 = $464 | $467;
          if ($468) {
           $469 = HEAP32[$pWInfo>>2]|0;
           $470 = (_sqlite3WhereOutputRowCount($469)|0);
           $471 = tempRet0;
           $472 = HEAP32[$2>>2]|0;
           $473 = (($472) + 24|0);
           $474 = $473;
           $475 = $474;
           HEAP32[$475>>2] = $470;
           $476 = (($474) + 4)|0;
           $477 = $476;
           HEAP32[$477>>2] = $471;
          }
          $478 = HEAP8[$sDistinct>>0]|0;
          $479 = $478&255;
          $480 = ($479|0)!=(0);
          do {
           if ($480) {
            $481 = HEAP32[$pWInfo>>2]|0;
            $482 = (_sqlite3WhereIsDistinct($481)|0);
            $483 = ($482|0)!=(0);
            if (!($483)) {
             break;
            }
            $484 = HEAP32[$pWInfo>>2]|0;
            $485 = (_sqlite3WhereIsDistinct($484)|0);
            $486 = $485&255;
            $487 = (($sDistinct) + 1|0);
            HEAP8[$487>>0] = $486;
           }
          } while(0);
          $488 = HEAP32[$sSort>>2]|0;
          $489 = ($488|0)!=(0|0);
          do {
           if ($489) {
            $490 = HEAP32[$pWInfo>>2]|0;
            $491 = (_sqlite3WhereIsOrdered($490)|0);
            $492 = (($sSort) + 4|0);
            HEAP32[$492>>2] = $491;
            $493 = (($sSort) + 4|0);
            $494 = HEAP32[$493>>2]|0;
            $495 = HEAP32[$sSort>>2]|0;
            $496 = HEAP32[$495>>2]|0;
            $497 = ($494|0)==($496|0);
            if (!($497)) {
             break;
            }
            HEAP32[$sSort>>2] = 0;
           }
          } while(0);
          $498 = (($sSort) + 20|0);
          $499 = HEAP32[$498>>2]|0;
          $500 = ($499|0)>=(0);
          do {
           if ($500) {
            $501 = HEAP32[$sSort>>2]|0;
            $502 = ($501|0)==(0|0);
            if (!($502)) {
             break;
            }
            $503 = HEAP32[$v>>2]|0;
            $504 = (($sSort) + 20|0);
            $505 = HEAP32[$504>>2]|0;
            _sqlite3VdbeChangeToNoop($503,$505);
           }
          } while(0);
          $506 = HEAP32[$1>>2]|0;
          $507 = HEAP32[$2>>2]|0;
          $508 = HEAP32[$pEList>>2]|0;
          $509 = HEAP32[$3>>2]|0;
          $510 = HEAP32[$pWInfo>>2]|0;
          $511 = (_sqlite3WhereContinueLabel($510)|0);
          $512 = HEAP32[$pWInfo>>2]|0;
          $513 = (_sqlite3WhereBreakLabel($512)|0);
          _selectInnerLoop($506,$507,$508,-1,$sSort,$sDistinct,$509,$511,$513);
          $514 = HEAP32[$pWInfo>>2]|0;
          _sqlite3WhereEnd($514);
         } else {
          HEAP32[$sortPTab>>2] = 0;
          HEAP32[$sortOut>>2] = 0;
          HEAP32[$orderByGrp>>2] = 0;
          $515 = HEAP32[$pGroupBy>>2]|0;
          $516 = ($515|0)!=(0|0);
          $517 = HEAP32[$2>>2]|0;
          do {
           if ($516) {
            $518 = HEAP32[$517>>2]|0;
            $519 = HEAP32[$518>>2]|0;
            HEAP32[$k>>2] = $519;
            $520 = HEAP32[$2>>2]|0;
            $521 = HEAP32[$520>>2]|0;
            $522 = (($521) + 4|0);
            $523 = HEAP32[$522>>2]|0;
            HEAP32[$pItem1>>2] = $523;
            while(1) {
             $524 = HEAP32[$k>>2]|0;
             $525 = ($524|0)>(0);
             if (!($525)) {
              break;
             }
             $526 = HEAP32[$pItem1>>2]|0;
             $527 = (($526) + 16|0);
             $528 = (($527) + 2|0);
             HEAP16[$528>>1] = 0;
             $529 = HEAP32[$k>>2]|0;
             $530 = (($529) + -1)|0;
             HEAP32[$k>>2] = $530;
             $531 = HEAP32[$pItem1>>2]|0;
             $532 = (($531) + 20|0);
             HEAP32[$pItem1>>2] = $532;
            }
            $533 = HEAP32[$pGroupBy>>2]|0;
            $534 = HEAP32[$533>>2]|0;
            HEAP32[$k>>2] = $534;
            $535 = HEAP32[$pGroupBy>>2]|0;
            $536 = (($535) + 4|0);
            $537 = HEAP32[$536>>2]|0;
            HEAP32[$pItem1>>2] = $537;
            while(1) {
             $538 = HEAP32[$k>>2]|0;
             $539 = ($538|0)>(0);
             if (!($539)) {
              break;
             }
             $540 = HEAP32[$pItem1>>2]|0;
             $541 = (($540) + 16|0);
             $542 = (($541) + 2|0);
             HEAP16[$542>>1] = 0;
             $543 = HEAP32[$k>>2]|0;
             $544 = (($543) + -1)|0;
             HEAP32[$k>>2] = $544;
             $545 = HEAP32[$pItem1>>2]|0;
             $546 = (($545) + 20|0);
             HEAP32[$pItem1>>2] = $546;
            }
            $547 = HEAP32[$2>>2]|0;
            $548 = (($547) + 24|0);
            $549 = $548;
            $550 = $549;
            $551 = HEAP32[$550>>2]|0;
            $552 = (($549) + 4)|0;
            $553 = $552;
            $554 = HEAP32[$553>>2]|0;
            $555 = ($554>>>0)>(0);
            $556 = ($551>>>0)>(100);
            $557 = ($554|0)==(0);
            $558 = $557 & $556;
            $559 = $555 | $558;
            if (!($559)) {
             break;
            }
            $560 = HEAP32[$2>>2]|0;
            $561 = (($560) + 24|0);
            $562 = $561;
            $563 = $562;
            HEAP32[$563>>2] = 100;
            $564 = (($562) + 4)|0;
            $565 = $564;
            HEAP32[$565>>2] = 0;
           } else {
            $566 = (($517) + 24|0);
            $567 = $566;
            $568 = $567;
            HEAP32[$568>>2] = 1;
            $569 = (($567) + 4)|0;
            $570 = $569;
            HEAP32[$570>>2] = 0;
           }
          } while(0);
          $571 = HEAP32[$pGroupBy>>2]|0;
          $572 = HEAP32[$sSort>>2]|0;
          $573 = (_sqlite3ExprListCompare($571,$572,-1)|0);
          $574 = ($573|0)==(0);
          if ($574) {
           HEAP32[$orderByGrp>>2] = 1;
          }
          $575 = HEAP32[$v>>2]|0;
          $576 = (_sqlite3VdbeMakeLabel($575)|0);
          HEAP32[$addrEnd>>2] = $576;
          ;HEAP32[$sNC+0>>2]=0|0;HEAP32[$sNC+4>>2]=0|0;HEAP32[$sNC+8>>2]=0|0;HEAP32[$sNC+12>>2]=0|0;HEAP32[$sNC+16>>2]=0|0;HEAP32[$sNC+20>>2]=0|0;HEAP32[$sNC+24>>2]=0|0;HEAP32[$sNC+28>>2]=0|0;
          $577 = HEAP32[$1>>2]|0;
          HEAP32[$sNC>>2] = $577;
          $578 = HEAP32[$pTabList>>2]|0;
          $579 = (($sNC) + 4|0);
          HEAP32[$579>>2] = $578;
          $580 = (($sNC) + 12|0);
          HEAP32[$580>>2] = $sAggInfo;
          $581 = HEAP32[$1>>2]|0;
          $582 = (($581) + 72|0);
          $583 = HEAP32[$582>>2]|0;
          $584 = (($583) + 1)|0;
          $585 = (($sAggInfo) + 16|0);
          HEAP32[$585>>2] = $584;
          $586 = HEAP32[$pGroupBy>>2]|0;
          $587 = ($586|0)!=(0|0);
          if ($587) {
           $588 = HEAP32[$pGroupBy>>2]|0;
           $589 = HEAP32[$588>>2]|0;
           $591 = $589;
          } else {
           $591 = 0;
          }
          $590 = (($sAggInfo) + 12|0);
          HEAP32[$590>>2] = $591;
          $592 = HEAP32[$pGroupBy>>2]|0;
          $593 = (($sAggInfo) + 24|0);
          HEAP32[$593>>2] = $592;
          $594 = HEAP32[$pEList>>2]|0;
          _sqlite3ExprAnalyzeAggList($sNC,$594);
          $595 = HEAP32[$sSort>>2]|0;
          _sqlite3ExprAnalyzeAggList($sNC,$595);
          $596 = HEAP32[$pHaving>>2]|0;
          $597 = ($596|0)!=(0|0);
          if ($597) {
           $598 = HEAP32[$pHaving>>2]|0;
           _sqlite3ExprAnalyzeAggregates($sNC,$598);
          }
          $599 = (($sAggInfo) + 32|0);
          $600 = HEAP32[$599>>2]|0;
          $601 = (($sAggInfo) + 36|0);
          HEAP32[$601>>2] = $600;
          HEAP32[$i>>2] = 0;
          while(1) {
           $602 = HEAP32[$i>>2]|0;
           $603 = (($sAggInfo) + 44|0);
           $604 = HEAP32[$603>>2]|0;
           $605 = ($602|0)<($604|0);
           if (!($605)) {
            break;
           }
           $606 = (($sNC) + 28|0);
           $607 = HEAP16[$606>>1]|0;
           $608 = $607&65535;
           $609 = $608 | 8;
           $610 = $609&65535;
           HEAP16[$606>>1] = $610;
           $611 = HEAP32[$i>>2]|0;
           $612 = (($sAggInfo) + 40|0);
           $613 = HEAP32[$612>>2]|0;
           $614 = (($613) + ($611<<4)|0);
           $615 = HEAP32[$614>>2]|0;
           $616 = (($615) + 20|0);
           $617 = HEAP32[$616>>2]|0;
           _sqlite3ExprAnalyzeAggList($sNC,$617);
           $618 = (($sNC) + 28|0);
           $619 = HEAP16[$618>>1]|0;
           $620 = $619&65535;
           $621 = $620 & -9;
           $622 = $621&65535;
           HEAP16[$618>>1] = $622;
           $623 = HEAP32[$i>>2]|0;
           $624 = (($623) + 1)|0;
           HEAP32[$i>>2] = $624;
          }
          $625 = HEAP32[$1>>2]|0;
          $626 = (($625) + 72|0);
          $627 = HEAP32[$626>>2]|0;
          $628 = (($sAggInfo) + 20|0);
          HEAP32[$628>>2] = $627;
          $629 = HEAP32[$db>>2]|0;
          $630 = (($629) + 64|0);
          $631 = HEAP8[$630>>0]|0;
          $632 = ($631<<24>>24)!=(0);
          if ($632) {
           break;
          }
          $633 = HEAP32[$pGroupBy>>2]|0;
          $634 = ($633|0)!=(0|0);
          if ($634) {
           $635 = HEAP32[$1>>2]|0;
           $636 = (($635) + 68|0);
           $637 = HEAP32[$636>>2]|0;
           $638 = (($637) + 1)|0;
           HEAP32[$636>>2] = $638;
           $639 = (($sAggInfo) + 4|0);
           HEAP32[$639>>2] = $637;
           $640 = HEAP32[$1>>2]|0;
           $641 = HEAP32[$pGroupBy>>2]|0;
           $642 = (_keyInfoFromExprList($640,$641,0,0)|0);
           HEAP32[$pKeyInfo2>>2] = $642;
           $643 = HEAP32[$v>>2]|0;
           $644 = (($sAggInfo) + 4|0);
           $645 = HEAP32[$644>>2]|0;
           $646 = (($sAggInfo) + 12|0);
           $647 = HEAP32[$646>>2]|0;
           $648 = HEAP32[$pKeyInfo2>>2]|0;
           $649 = (_sqlite3VdbeAddOp4($643,58,$645,$647,0,$648,-6)|0);
           HEAP32[$addrSortingIdx>>2] = $649;
           $650 = HEAP32[$1>>2]|0;
           $651 = (($650) + 72|0);
           $652 = HEAP32[$651>>2]|0;
           $653 = (($652) + 1)|0;
           HEAP32[$651>>2] = $653;
           HEAP32[$iUseFlag>>2] = $653;
           $654 = HEAP32[$1>>2]|0;
           $655 = (($654) + 72|0);
           $656 = HEAP32[$655>>2]|0;
           $657 = (($656) + 1)|0;
           HEAP32[$655>>2] = $657;
           HEAP32[$iAbortFlag>>2] = $657;
           $658 = HEAP32[$1>>2]|0;
           $659 = (($658) + 72|0);
           $660 = HEAP32[$659>>2]|0;
           $661 = (($660) + 1)|0;
           HEAP32[$659>>2] = $661;
           HEAP32[$regOutputRow>>2] = $661;
           $662 = HEAP32[$v>>2]|0;
           $663 = (_sqlite3VdbeMakeLabel($662)|0);
           HEAP32[$addrOutputRow>>2] = $663;
           $664 = HEAP32[$1>>2]|0;
           $665 = (($664) + 72|0);
           $666 = HEAP32[$665>>2]|0;
           $667 = (($666) + 1)|0;
           HEAP32[$665>>2] = $667;
           HEAP32[$regReset>>2] = $667;
           $668 = HEAP32[$v>>2]|0;
           $669 = (_sqlite3VdbeMakeLabel($668)|0);
           HEAP32[$addrReset>>2] = $669;
           $670 = HEAP32[$1>>2]|0;
           $671 = (($670) + 72|0);
           $672 = HEAP32[$671>>2]|0;
           $673 = (($672) + 1)|0;
           HEAP32[$iAMem>>2] = $673;
           $674 = HEAP32[$pGroupBy>>2]|0;
           $675 = HEAP32[$674>>2]|0;
           $676 = HEAP32[$1>>2]|0;
           $677 = (($676) + 72|0);
           $678 = HEAP32[$677>>2]|0;
           $679 = (($678) + ($675))|0;
           HEAP32[$677>>2] = $679;
           $680 = HEAP32[$1>>2]|0;
           $681 = (($680) + 72|0);
           $682 = HEAP32[$681>>2]|0;
           $683 = (($682) + 1)|0;
           HEAP32[$iBMem>>2] = $683;
           $684 = HEAP32[$pGroupBy>>2]|0;
           $685 = HEAP32[$684>>2]|0;
           $686 = HEAP32[$1>>2]|0;
           $687 = (($686) + 72|0);
           $688 = HEAP32[$687>>2]|0;
           $689 = (($688) + ($685))|0;
           HEAP32[$687>>2] = $689;
           $690 = HEAP32[$v>>2]|0;
           $691 = HEAP32[$iAbortFlag>>2]|0;
           (_sqlite3VdbeAddOp2($690,25,0,$691)|0);
           $692 = HEAP32[$v>>2]|0;
           $693 = HEAP32[$iUseFlag>>2]|0;
           (_sqlite3VdbeAddOp2($692,25,0,$693)|0);
           $694 = HEAP32[$v>>2]|0;
           $695 = HEAP32[$iAMem>>2]|0;
           $696 = HEAP32[$iAMem>>2]|0;
           $697 = HEAP32[$pGroupBy>>2]|0;
           $698 = HEAP32[$697>>2]|0;
           $699 = (($696) + ($698))|0;
           $700 = (($699) - 1)|0;
           (_sqlite3VdbeAddOp3($694,28,0,$695,$700)|0);
           $701 = HEAP32[$v>>2]|0;
           $702 = HEAP32[$regReset>>2]|0;
           $703 = HEAP32[$addrReset>>2]|0;
           (_sqlite3VdbeAddOp2($701,17,$702,$703)|0);
           $704 = HEAP32[$1>>2]|0;
           $705 = HEAP32[$pTabList>>2]|0;
           $706 = HEAP32[$pWhere>>2]|0;
           $707 = HEAP32[$pGroupBy>>2]|0;
           $708 = HEAP32[$orderByGrp>>2]|0;
           $709 = ($708|0)!=(0);
           $710 = $709 ? 2048 : 0;
           $711 = 256 | $710;
           $712 = $711&65535;
           $713 = (_sqlite3WhereBegin($704,$705,$706,$707,0,$712,0)|0);
           HEAP32[$pWInfo>>2] = $713;
           $714 = HEAP32[$pWInfo>>2]|0;
           $715 = ($714|0)==(0|0);
           if ($715) {
            break;
           }
           $716 = HEAP32[$pWInfo>>2]|0;
           $717 = (_sqlite3WhereIsOrdered($716)|0);
           $718 = HEAP32[$pGroupBy>>2]|0;
           $719 = HEAP32[$718>>2]|0;
           $720 = ($717|0)==($719|0);
           if ($720) {
            HEAP32[$groupBySort>>2] = 0;
           } else {
            $721 = HEAP32[$1>>2]|0;
            $722 = HEAP8[$sDistinct>>0]|0;
            $723 = $722&255;
            $724 = ($723|0)!=(0);
            if ($724) {
             $725 = HEAP32[$2>>2]|0;
             $726 = (($725) + 6|0);
             $727 = HEAP16[$726>>1]|0;
             $728 = $727&65535;
             $729 = $728 & 1;
             $730 = ($729|0)==(0);
             $732 = $730;
            } else {
             $732 = 0;
            }
            $731 = $732 ? 24832 : 24848;
            _explainTempTable($721,$731);
            HEAP32[$groupBySort>>2] = 1;
            $733 = HEAP32[$pGroupBy>>2]|0;
            $734 = HEAP32[$733>>2]|0;
            HEAP32[$nGroupBy>>2] = $734;
            $735 = HEAP32[$nGroupBy>>2]|0;
            HEAP32[$nCol>>2] = $735;
            $736 = HEAP32[$nGroupBy>>2]|0;
            HEAP32[$j>>2] = $736;
            HEAP32[$i>>2] = 0;
            while(1) {
             $737 = HEAP32[$i>>2]|0;
             $738 = (($sAggInfo) + 32|0);
             $739 = HEAP32[$738>>2]|0;
             $740 = ($737|0)<($739|0);
             if (!($740)) {
              break;
             }
             $741 = HEAP32[$i>>2]|0;
             $742 = (($sAggInfo) + 28|0);
             $743 = HEAP32[$742>>2]|0;
             $744 = (($743) + (($741*24)|0)|0);
             $745 = (($744) + 12|0);
             $746 = HEAP32[$745>>2]|0;
             $747 = HEAP32[$j>>2]|0;
             $748 = ($746|0)>=($747|0);
             if ($748) {
              $749 = HEAP32[$nCol>>2]|0;
              $750 = (($749) + 1)|0;
              HEAP32[$nCol>>2] = $750;
              $751 = HEAP32[$j>>2]|0;
              $752 = (($751) + 1)|0;
              HEAP32[$j>>2] = $752;
             }
             $753 = HEAP32[$i>>2]|0;
             $754 = (($753) + 1)|0;
             HEAP32[$i>>2] = $754;
            }
            $755 = HEAP32[$1>>2]|0;
            $756 = HEAP32[$nCol>>2]|0;
            $757 = (_sqlite3GetTempRange($755,$756)|0);
            HEAP32[$regBase>>2] = $757;
            $758 = HEAP32[$1>>2]|0;
            _sqlite3ExprCacheClear($758);
            $759 = HEAP32[$1>>2]|0;
            $760 = HEAP32[$pGroupBy>>2]|0;
            $761 = HEAP32[$regBase>>2]|0;
            (_sqlite3ExprCodeExprList($759,$760,$761,0)|0);
            $762 = HEAP32[$nGroupBy>>2]|0;
            HEAP32[$j>>2] = $762;
            HEAP32[$i>>2] = 0;
            while(1) {
             $763 = HEAP32[$i>>2]|0;
             $764 = (($sAggInfo) + 32|0);
             $765 = HEAP32[$764>>2]|0;
             $766 = ($763|0)<($765|0);
             if (!($766)) {
              break;
             }
             $767 = HEAP32[$i>>2]|0;
             $768 = (($sAggInfo) + 28|0);
             $769 = HEAP32[$768>>2]|0;
             $770 = (($769) + (($767*24)|0)|0);
             HEAP32[$pCol>>2] = $770;
             $771 = HEAP32[$pCol>>2]|0;
             $772 = (($771) + 12|0);
             $773 = HEAP32[$772>>2]|0;
             $774 = HEAP32[$j>>2]|0;
             $775 = ($773|0)>=($774|0);
             if ($775) {
              $776 = HEAP32[$j>>2]|0;
              $777 = HEAP32[$regBase>>2]|0;
              $778 = (($776) + ($777))|0;
              HEAP32[$r1>>2] = $778;
              $779 = HEAP32[$1>>2]|0;
              $780 = HEAP32[$pCol>>2]|0;
              $781 = HEAP32[$780>>2]|0;
              $782 = HEAP32[$pCol>>2]|0;
              $783 = (($782) + 8|0);
              $784 = HEAP32[$783>>2]|0;
              $785 = HEAP32[$pCol>>2]|0;
              $786 = (($785) + 4|0);
              $787 = HEAP32[$786>>2]|0;
              $788 = HEAP32[$r1>>2]|0;
              $789 = (_sqlite3ExprCodeGetColumn($779,$781,$784,$787,$788,0)|0);
              HEAP32[$r2>>2] = $789;
              $790 = HEAP32[$r1>>2]|0;
              $791 = HEAP32[$r2>>2]|0;
              $792 = ($790|0)!=($791|0);
              if ($792) {
               $793 = HEAP32[$v>>2]|0;
               $794 = HEAP32[$r2>>2]|0;
               $795 = HEAP32[$r1>>2]|0;
               (_sqlite3VdbeAddOp2($793,34,$794,$795)|0);
              }
              $796 = HEAP32[$j>>2]|0;
              $797 = (($796) + 1)|0;
              HEAP32[$j>>2] = $797;
             }
             $798 = HEAP32[$i>>2]|0;
             $799 = (($798) + 1)|0;
             HEAP32[$i>>2] = $799;
            }
            $800 = HEAP32[$1>>2]|0;
            $801 = (_sqlite3GetTempReg($800)|0);
            HEAP32[$regRecord>>2] = $801;
            $802 = HEAP32[$v>>2]|0;
            $803 = HEAP32[$regBase>>2]|0;
            $804 = HEAP32[$nCol>>2]|0;
            $805 = HEAP32[$regRecord>>2]|0;
            (_sqlite3VdbeAddOp3($802,49,$803,$804,$805)|0);
            $806 = HEAP32[$v>>2]|0;
            $807 = (($sAggInfo) + 4|0);
            $808 = HEAP32[$807>>2]|0;
            $809 = HEAP32[$regRecord>>2]|0;
            (_sqlite3VdbeAddOp2($806,109,$808,$809)|0);
            $810 = HEAP32[$1>>2]|0;
            $811 = HEAP32[$regRecord>>2]|0;
            _sqlite3ReleaseTempReg($810,$811);
            $812 = HEAP32[$1>>2]|0;
            $813 = HEAP32[$regBase>>2]|0;
            $814 = HEAP32[$nCol>>2]|0;
            _sqlite3ReleaseTempRange($812,$813,$814);
            $815 = HEAP32[$pWInfo>>2]|0;
            _sqlite3WhereEnd($815);
            $816 = HEAP32[$1>>2]|0;
            $817 = (($816) + 68|0);
            $818 = HEAP32[$817>>2]|0;
            $819 = (($818) + 1)|0;
            HEAP32[$817>>2] = $819;
            HEAP32[$sortPTab>>2] = $818;
            $820 = (($sAggInfo) + 8|0);
            HEAP32[$820>>2] = $818;
            $821 = HEAP32[$1>>2]|0;
            $822 = (_sqlite3GetTempReg($821)|0);
            HEAP32[$sortOut>>2] = $822;
            $823 = HEAP32[$v>>2]|0;
            $824 = HEAP32[$sortPTab>>2]|0;
            $825 = HEAP32[$sortOut>>2]|0;
            $826 = HEAP32[$nCol>>2]|0;
            (_sqlite3VdbeAddOp3($823,60,$824,$825,$826)|0);
            $827 = HEAP32[$v>>2]|0;
            $828 = (($sAggInfo) + 4|0);
            $829 = HEAP32[$828>>2]|0;
            $830 = HEAP32[$addrEnd>>2]|0;
            (_sqlite3VdbeAddOp2($827,106,$829,$830)|0);
            $831 = (($sAggInfo) + 1|0);
            HEAP8[$831>>0] = 1;
            $832 = HEAP32[$1>>2]|0;
            _sqlite3ExprCacheClear($832);
           }
           $833 = HEAP32[$orderByGrp>>2]|0;
           $834 = ($833|0)!=(0);
           do {
            if ($834) {
             $835 = HEAP32[$db>>2]|0;
             $836 = (($835) + 60|0);
             $837 = HEAP16[$836>>1]|0;
             $838 = $837&65535;
             $839 = $838 & 4;
             $840 = ($839|0)==(0);
             if (!($840)) {
              break;
             }
             $841 = HEAP32[$groupBySort>>2]|0;
             $842 = ($841|0)!=(0);
             if (!($842)) {
              $843 = HEAP32[$pWInfo>>2]|0;
              $844 = (_sqlite3WhereIsSorted($843)|0);
              $845 = ($844|0)!=(0);
              if (!($845)) {
               break;
              }
             }
             HEAP32[$sSort>>2] = 0;
             $846 = HEAP32[$v>>2]|0;
             $847 = (($sSort) + 20|0);
             $848 = HEAP32[$847>>2]|0;
             _sqlite3VdbeChangeToNoop($846,$848);
            }
           } while(0);
           $849 = HEAP32[$v>>2]|0;
           $850 = (_sqlite3VdbeCurrentAddr($849)|0);
           HEAP32[$addrTopOfLoop>>2] = $850;
           $851 = HEAP32[$1>>2]|0;
           _sqlite3ExprCacheClear($851);
           $852 = HEAP32[$groupBySort>>2]|0;
           $853 = ($852|0)!=(0);
           if ($853) {
            $854 = HEAP32[$v>>2]|0;
            $855 = (($sAggInfo) + 4|0);
            $856 = HEAP32[$855>>2]|0;
            $857 = HEAP32[$sortOut>>2]|0;
            $858 = HEAP32[$sortPTab>>2]|0;
            (_sqlite3VdbeAddOp3($854,100,$856,$857,$858)|0);
           }
           HEAP32[$j>>2] = 0;
           while(1) {
            $859 = HEAP32[$j>>2]|0;
            $860 = HEAP32[$pGroupBy>>2]|0;
            $861 = HEAP32[$860>>2]|0;
            $862 = ($859|0)<($861|0);
            if (!($862)) {
             break;
            }
            $863 = HEAP32[$groupBySort>>2]|0;
            $864 = ($863|0)!=(0);
            if ($864) {
             $865 = HEAP32[$v>>2]|0;
             $866 = HEAP32[$sortPTab>>2]|0;
             $867 = HEAP32[$j>>2]|0;
             $868 = HEAP32[$iBMem>>2]|0;
             $869 = HEAP32[$j>>2]|0;
             $870 = (($868) + ($869))|0;
             (_sqlite3VdbeAddOp3($865,47,$866,$867,$870)|0);
            } else {
             HEAP8[$sAggInfo>>0] = 1;
             $871 = HEAP32[$1>>2]|0;
             $872 = HEAP32[$j>>2]|0;
             $873 = HEAP32[$pGroupBy>>2]|0;
             $874 = (($873) + 4|0);
             $875 = HEAP32[$874>>2]|0;
             $876 = (($875) + (($872*20)|0)|0);
             $877 = HEAP32[$876>>2]|0;
             $878 = HEAP32[$iBMem>>2]|0;
             $879 = HEAP32[$j>>2]|0;
             $880 = (($878) + ($879))|0;
             _sqlite3ExprCode($871,$877,$880);
            }
            $881 = HEAP32[$j>>2]|0;
            $882 = (($881) + 1)|0;
            HEAP32[$j>>2] = $882;
           }
           $883 = HEAP32[$v>>2]|0;
           $884 = HEAP32[$iAMem>>2]|0;
           $885 = HEAP32[$iBMem>>2]|0;
           $886 = HEAP32[$pGroupBy>>2]|0;
           $887 = HEAP32[$886>>2]|0;
           $888 = HEAP32[$pKeyInfo2>>2]|0;
           $889 = (_sqlite3KeyInfoRef($888)|0);
           (_sqlite3VdbeAddOp4($883,42,$884,$885,$887,$889,-6)|0);
           $890 = HEAP32[$v>>2]|0;
           $891 = (_sqlite3VdbeCurrentAddr($890)|0);
           HEAP32[$j1>>2] = $891;
           $892 = HEAP32[$v>>2]|0;
           $893 = HEAP32[$j1>>2]|0;
           $894 = (($893) + 1)|0;
           $895 = HEAP32[$j1>>2]|0;
           $896 = (($895) + 1)|0;
           (_sqlite3VdbeAddOp3($892,43,$894,0,$896)|0);
           $897 = HEAP32[$1>>2]|0;
           $898 = HEAP32[$iBMem>>2]|0;
           $899 = HEAP32[$iAMem>>2]|0;
           $900 = HEAP32[$pGroupBy>>2]|0;
           $901 = HEAP32[$900>>2]|0;
           _sqlite3ExprCodeMove($897,$898,$899,$901);
           $902 = HEAP32[$v>>2]|0;
           $903 = HEAP32[$regOutputRow>>2]|0;
           $904 = HEAP32[$addrOutputRow>>2]|0;
           (_sqlite3VdbeAddOp2($902,17,$903,$904)|0);
           $905 = HEAP32[$v>>2]|0;
           $906 = HEAP32[$iAbortFlag>>2]|0;
           $907 = HEAP32[$addrEnd>>2]|0;
           (_sqlite3VdbeAddOp2($905,137,$906,$907)|0);
           $908 = HEAP32[$v>>2]|0;
           $909 = HEAP32[$regReset>>2]|0;
           $910 = HEAP32[$addrReset>>2]|0;
           (_sqlite3VdbeAddOp2($908,17,$909,$910)|0);
           $911 = HEAP32[$v>>2]|0;
           $912 = HEAP32[$j1>>2]|0;
           _sqlite3VdbeJumpHere($911,$912);
           $913 = HEAP32[$1>>2]|0;
           _updateAccumulator($913,$sAggInfo);
           $914 = HEAP32[$v>>2]|0;
           $915 = HEAP32[$iUseFlag>>2]|0;
           (_sqlite3VdbeAddOp2($914,25,1,$915)|0);
           $916 = HEAP32[$groupBySort>>2]|0;
           $917 = ($916|0)!=(0);
           if ($917) {
            $918 = HEAP32[$v>>2]|0;
            $919 = (($sAggInfo) + 4|0);
            $920 = HEAP32[$919>>2]|0;
            $921 = HEAP32[$addrTopOfLoop>>2]|0;
            (_sqlite3VdbeAddOp2($918,5,$920,$921)|0);
           } else {
            $922 = HEAP32[$pWInfo>>2]|0;
            _sqlite3WhereEnd($922);
            $923 = HEAP32[$v>>2]|0;
            $924 = HEAP32[$addrSortingIdx>>2]|0;
            _sqlite3VdbeChangeToNoop($923,$924);
           }
           $925 = HEAP32[$v>>2]|0;
           $926 = HEAP32[$regOutputRow>>2]|0;
           $927 = HEAP32[$addrOutputRow>>2]|0;
           (_sqlite3VdbeAddOp2($925,17,$926,$927)|0);
           $928 = HEAP32[$v>>2]|0;
           $929 = HEAP32[$addrEnd>>2]|0;
           (_sqlite3VdbeAddOp2($928,16,0,$929)|0);
           $930 = HEAP32[$v>>2]|0;
           $931 = (_sqlite3VdbeCurrentAddr($930)|0);
           HEAP32[$addrSetAbort>>2] = $931;
           $932 = HEAP32[$v>>2]|0;
           $933 = HEAP32[$iAbortFlag>>2]|0;
           (_sqlite3VdbeAddOp2($932,25,1,$933)|0);
           $934 = HEAP32[$v>>2]|0;
           $935 = HEAP32[$regOutputRow>>2]|0;
           (_sqlite3VdbeAddOp1($934,18,$935)|0);
           $936 = HEAP32[$v>>2]|0;
           $937 = HEAP32[$addrOutputRow>>2]|0;
           _sqlite3VdbeResolveLabel($936,$937);
           $938 = HEAP32[$v>>2]|0;
           $939 = (_sqlite3VdbeCurrentAddr($938)|0);
           HEAP32[$addrOutputRow>>2] = $939;
           $940 = HEAP32[$v>>2]|0;
           $941 = HEAP32[$iUseFlag>>2]|0;
           $942 = HEAP32[$addrOutputRow>>2]|0;
           $943 = (($942) + 2)|0;
           (_sqlite3VdbeAddOp2($940,137,$941,$943)|0);
           $944 = HEAP32[$v>>2]|0;
           $945 = HEAP32[$regOutputRow>>2]|0;
           (_sqlite3VdbeAddOp1($944,18,$945)|0);
           $946 = HEAP32[$1>>2]|0;
           _finalizeAggFunctions($946,$sAggInfo);
           $947 = HEAP32[$1>>2]|0;
           $948 = HEAP32[$pHaving>>2]|0;
           $949 = HEAP32[$addrOutputRow>>2]|0;
           $950 = (($949) + 1)|0;
           _sqlite3ExprIfFalse($947,$948,$950,16);
           $951 = HEAP32[$1>>2]|0;
           $952 = HEAP32[$2>>2]|0;
           $953 = HEAP32[$2>>2]|0;
           $954 = HEAP32[$953>>2]|0;
           $955 = HEAP32[$3>>2]|0;
           $956 = HEAP32[$addrOutputRow>>2]|0;
           $957 = (($956) + 1)|0;
           $958 = HEAP32[$addrSetAbort>>2]|0;
           _selectInnerLoop($951,$952,$954,-1,$sSort,$sDistinct,$955,$957,$958);
           $959 = HEAP32[$v>>2]|0;
           $960 = HEAP32[$regOutputRow>>2]|0;
           (_sqlite3VdbeAddOp1($959,18,$960)|0);
           $961 = HEAP32[$v>>2]|0;
           $962 = HEAP32[$addrReset>>2]|0;
           _sqlite3VdbeResolveLabel($961,$962);
           $963 = HEAP32[$1>>2]|0;
           _resetAccumulator($963,$sAggInfo);
           $964 = HEAP32[$v>>2]|0;
           $965 = HEAP32[$regReset>>2]|0;
           (_sqlite3VdbeAddOp1($964,18,$965)|0);
          } else {
           HEAP32[$pDel>>2] = 0;
           $966 = HEAP32[$2>>2]|0;
           $967 = (_isSimpleCount($966,$sAggInfo)|0);
           HEAP32[$pTab>>2] = $967;
           $968 = ($967|0)!=(0|0);
           if ($968) {
            $969 = HEAP32[$1>>2]|0;
            $970 = HEAP32[$969>>2]|0;
            $971 = HEAP32[$pTab>>2]|0;
            $972 = (($971) + 68|0);
            $973 = HEAP32[$972>>2]|0;
            $974 = (_sqlite3SchemaToIndex($970,$973)|0);
            HEAP32[$iDb>>2] = $974;
            $975 = HEAP32[$1>>2]|0;
            $976 = (($975) + 68|0);
            $977 = HEAP32[$976>>2]|0;
            $978 = (($977) + 1)|0;
            HEAP32[$976>>2] = $978;
            HEAP32[$iCsr>>2] = $977;
            HEAP32[$pKeyInfo3>>2] = 0;
            HEAP32[$pBest>>2] = 0;
            $979 = HEAP32[$pTab>>2]|0;
            $980 = (($979) + 32|0);
            $981 = HEAP32[$980>>2]|0;
            HEAP32[$iRoot>>2] = $981;
            $982 = HEAP32[$1>>2]|0;
            $983 = HEAP32[$iDb>>2]|0;
            _sqlite3CodeVerifySchema($982,$983);
            $984 = HEAP32[$1>>2]|0;
            $985 = HEAP32[$iDb>>2]|0;
            $986 = HEAP32[$pTab>>2]|0;
            $987 = (($986) + 32|0);
            $988 = HEAP32[$987>>2]|0;
            $989 = HEAP32[$pTab>>2]|0;
            $990 = HEAP32[$989>>2]|0;
            _sqlite3TableLock($984,$985,$988,0,$990);
            $991 = HEAP32[$pTab>>2]|0;
            $992 = (($991) + 44|0);
            $993 = HEAP8[$992>>0]|0;
            $994 = $993&255;
            $995 = $994 & 32;
            $996 = ($995|0)==(0);
            if (!($996)) {
             $997 = HEAP32[$pTab>>2]|0;
             $998 = (_sqlite3PrimaryKeyIndex($997)|0);
             HEAP32[$pBest>>2] = $998;
            }
            $999 = HEAP32[$pTab>>2]|0;
            $1000 = (($999) + 8|0);
            $1001 = HEAP32[$1000>>2]|0;
            HEAP32[$pIdx>>2] = $1001;
            while(1) {
             $1002 = HEAP32[$pIdx>>2]|0;
             $1003 = ($1002|0)!=(0|0);
             if (!($1003)) {
              break;
             }
             $1004 = HEAP32[$pIdx>>2]|0;
             $1005 = (($1004) + 55|0);
             $1006 = HEAP8[$1005>>0]|0;
             $1007 = ($1006&255) >>> 2;
             $1008 = $1007 & 1;
             $1009 = $1008&255;
             $1010 = ($1009|0)==(0);
             do {
              if ($1010) {
               $1011 = HEAP32[$pIdx>>2]|0;
               $1012 = (($1011) + 48|0);
               $1013 = HEAP16[$1012>>1]|0;
               $1014 = $1013 << 16 >> 16;
               $1015 = HEAP32[$pTab>>2]|0;
               $1016 = (($1015) + 42|0);
               $1017 = HEAP16[$1016>>1]|0;
               $1018 = $1017 << 16 >> 16;
               $1019 = ($1014|0)<($1018|0);
               if (!($1019)) {
                break;
               }
               $1020 = HEAP32[$pIdx>>2]|0;
               $1021 = (($1020) + 36|0);
               $1022 = HEAP32[$1021>>2]|0;
               $1023 = ($1022|0)==(0|0);
               if (!($1023)) {
                break;
               }
               $1024 = HEAP32[$pBest>>2]|0;
               $1025 = ($1024|0)!=(0|0);
               if ($1025) {
                $1026 = HEAP32[$pIdx>>2]|0;
                $1027 = (($1026) + 48|0);
                $1028 = HEAP16[$1027>>1]|0;
                $1029 = $1028 << 16 >> 16;
                $1030 = HEAP32[$pBest>>2]|0;
                $1031 = (($1030) + 48|0);
                $1032 = HEAP16[$1031>>1]|0;
                $1033 = $1032 << 16 >> 16;
                $1034 = ($1029|0)<($1033|0);
                if (!($1034)) {
                 break;
                }
               }
               $1035 = HEAP32[$pIdx>>2]|0;
               HEAP32[$pBest>>2] = $1035;
              }
             } while(0);
             $1036 = HEAP32[$pIdx>>2]|0;
             $1037 = (($1036) + 20|0);
             $1038 = HEAP32[$1037>>2]|0;
             HEAP32[$pIdx>>2] = $1038;
            }
            $1039 = HEAP32[$pBest>>2]|0;
            $1040 = ($1039|0)!=(0|0);
            if ($1040) {
             $1041 = HEAP32[$pBest>>2]|0;
             $1042 = (($1041) + 44|0);
             $1043 = HEAP32[$1042>>2]|0;
             HEAP32[$iRoot>>2] = $1043;
             $1044 = HEAP32[$1>>2]|0;
             $1045 = HEAP32[$pBest>>2]|0;
             $1046 = (_sqlite3KeyInfoOfIndex($1044,$1045)|0);
             HEAP32[$pKeyInfo3>>2] = $1046;
            }
            $1047 = HEAP32[$v>>2]|0;
            $1048 = HEAP32[$iCsr>>2]|0;
            $1049 = HEAP32[$iRoot>>2]|0;
            $1050 = HEAP32[$iDb>>2]|0;
            (_sqlite3VdbeAddOp4Int($1047,54,$1048,$1049,$1050,1)|0);
            $1051 = HEAP32[$pKeyInfo3>>2]|0;
            $1052 = ($1051|0)!=(0|0);
            if ($1052) {
             $1053 = HEAP32[$v>>2]|0;
             $1054 = HEAP32[$pKeyInfo3>>2]|0;
             _sqlite3VdbeChangeP4($1053,-1,$1054,-6);
            }
            $1055 = HEAP32[$v>>2]|0;
            $1056 = HEAP32[$iCsr>>2]|0;
            $1057 = (($sAggInfo) + 40|0);
            $1058 = HEAP32[$1057>>2]|0;
            $1059 = (($1058) + 8|0);
            $1060 = HEAP32[$1059>>2]|0;
            (_sqlite3VdbeAddOp2($1055,50,$1056,$1060)|0);
            $1061 = HEAP32[$v>>2]|0;
            $1062 = HEAP32[$iCsr>>2]|0;
            (_sqlite3VdbeAddOp1($1061,61,$1062)|0);
            $1063 = HEAP32[$1>>2]|0;
            $1064 = HEAP32[$pTab>>2]|0;
            $1065 = HEAP32[$pBest>>2]|0;
            _explainSimpleCount($1063,$1064,$1065);
           } else {
            HEAP32[$pMinMax>>2] = 0;
            HEAP8[$flag>>0] = 0;
            $1066 = HEAP32[$2>>2]|0;
            $1067 = (($1066) + 44|0);
            $1068 = HEAP32[$1067>>2]|0;
            $1069 = ($1068|0)==(0|0);
            if ($1069) {
             $1070 = (_minMaxQuery($sAggInfo,$pMinMax)|0);
             HEAP8[$flag>>0] = $1070;
            }
            $1071 = HEAP8[$flag>>0]|0;
            $1072 = ($1071<<24>>24)!=(0);
            do {
             if ($1072) {
              $1073 = HEAP32[$db>>2]|0;
              $1074 = HEAP32[$pMinMax>>2]|0;
              $1075 = (_sqlite3ExprListDup($1073,$1074,0)|0);
              HEAP32[$pMinMax>>2] = $1075;
              $1076 = HEAP32[$pMinMax>>2]|0;
              HEAP32[$pDel>>2] = $1076;
              $1077 = HEAP32[$pMinMax>>2]|0;
              $1078 = ($1077|0)!=(0|0);
              if (!($1078)) {
               break;
              }
              $1079 = HEAP32[$db>>2]|0;
              $1080 = (($1079) + 64|0);
              $1081 = HEAP8[$1080>>0]|0;
              $1082 = ($1081<<24>>24)!=(0);
              if ($1082) {
               break;
              }
              $1083 = HEAP8[$flag>>0]|0;
              $1084 = $1083&255;
              $1085 = ($1084|0)!=(1);
              $1086 = $1085 ? 1 : 0;
              $1087 = $1086&255;
              $1088 = HEAP32[$pMinMax>>2]|0;
              $1089 = (($1088) + 4|0);
              $1090 = HEAP32[$1089>>2]|0;
              $1091 = (($1090) + 12|0);
              HEAP8[$1091>>0] = $1087;
              $1092 = HEAP32[$pMinMax>>2]|0;
              $1093 = (($1092) + 4|0);
              $1094 = HEAP32[$1093>>2]|0;
              $1095 = HEAP32[$1094>>2]|0;
              HEAP8[$1095>>0] = -102;
             }
            } while(0);
            $1096 = HEAP32[$1>>2]|0;
            _resetAccumulator($1096,$sAggInfo);
            $1097 = HEAP32[$1>>2]|0;
            $1098 = HEAP32[$pTabList>>2]|0;
            $1099 = HEAP32[$pWhere>>2]|0;
            $1100 = HEAP32[$pMinMax>>2]|0;
            $1101 = HEAP8[$flag>>0]|0;
            $1102 = $1101&255;
            $1103 = (_sqlite3WhereBegin($1097,$1098,$1099,$1100,0,$1102,0)|0);
            HEAP32[$pWInfo>>2] = $1103;
            $1104 = HEAP32[$pWInfo>>2]|0;
            $1105 = ($1104|0)==(0|0);
            if ($1105) {
             $1106 = HEAP32[$db>>2]|0;
             $1107 = HEAP32[$pDel>>2]|0;
             _sqlite3ExprListDelete($1106,$1107);
             break;
            }
            $1108 = HEAP32[$1>>2]|0;
            _updateAccumulator($1108,$sAggInfo);
            $1109 = HEAP32[$pWInfo>>2]|0;
            $1110 = (_sqlite3WhereIsOrdered($1109)|0);
            $1111 = ($1110|0)>(0);
            if ($1111) {
             $1112 = HEAP32[$v>>2]|0;
             $1113 = HEAP32[$pWInfo>>2]|0;
             $1114 = (_sqlite3WhereBreakLabel($1113)|0);
             (_sqlite3VdbeAddOp2($1112,16,0,$1114)|0);
            }
            $1115 = HEAP32[$pWInfo>>2]|0;
            _sqlite3WhereEnd($1115);
            $1116 = HEAP32[$1>>2]|0;
            _finalizeAggFunctions($1116,$sAggInfo);
           }
           HEAP32[$sSort>>2] = 0;
           $1117 = HEAP32[$1>>2]|0;
           $1118 = HEAP32[$pHaving>>2]|0;
           $1119 = HEAP32[$addrEnd>>2]|0;
           _sqlite3ExprIfFalse($1117,$1118,$1119,16);
           $1120 = HEAP32[$1>>2]|0;
           $1121 = HEAP32[$2>>2]|0;
           $1122 = HEAP32[$2>>2]|0;
           $1123 = HEAP32[$1122>>2]|0;
           $1124 = HEAP32[$3>>2]|0;
           $1125 = HEAP32[$addrEnd>>2]|0;
           $1126 = HEAP32[$addrEnd>>2]|0;
           _selectInnerLoop($1120,$1121,$1123,-1,0,0,$1124,$1125,$1126);
           $1127 = HEAP32[$db>>2]|0;
           $1128 = HEAP32[$pDel>>2]|0;
           _sqlite3ExprListDelete($1127,$1128);
          }
          $1129 = HEAP32[$v>>2]|0;
          $1130 = HEAP32[$addrEnd>>2]|0;
          _sqlite3VdbeResolveLabel($1129,$1130);
         }
         $1131 = (($sDistinct) + 1|0);
         $1132 = HEAP8[$1131>>0]|0;
         $1133 = $1132&255;
         $1134 = ($1133|0)==(3);
         if ($1134) {
          $1135 = HEAP32[$1>>2]|0;
          _explainTempTable($1135,24832);
         }
         $1136 = HEAP32[$sSort>>2]|0;
         $1137 = ($1136|0)!=(0|0);
         if ($1137) {
          $1138 = HEAP32[$1>>2]|0;
          $1139 = (($sSort) + 4|0);
          $1140 = HEAP32[$1139>>2]|0;
          $1141 = ($1140|0)>(0);
          $1142 = $1141 ? 24864 : 24888;
          _explainTempTable($1138,$1142);
          $1143 = HEAP32[$1>>2]|0;
          $1144 = HEAP32[$2>>2]|0;
          $1145 = HEAP32[$pEList>>2]|0;
          $1146 = HEAP32[$1145>>2]|0;
          $1147 = HEAP32[$3>>2]|0;
          _generateSortTail($1143,$1144,$sSort,$1146,$1147);
         }
         $1148 = HEAP32[$v>>2]|0;
         $1149 = HEAP32[$iEnd>>2]|0;
         _sqlite3VdbeResolveLabel($1148,$1149);
         HEAP32[$rc>>2] = 0;
        }
       }
      }
     }
    } while(0);
    $1150 = HEAP32[$iRestoreSelectId>>2]|0;
    $1151 = HEAP32[$1>>2]|0;
    $1152 = (($1151) + 464|0);
    HEAP32[$1152>>2] = $1150;
    $1153 = HEAP32[$rc>>2]|0;
    $1154 = ($1153|0)==(0);
    if ($1154) {
     $1155 = HEAP32[$3>>2]|0;
     $1156 = HEAP8[$1155>>0]|0;
     $1157 = $1156&255;
     $1158 = ($1157|0)==(9);
     if ($1158) {
      $1159 = HEAP32[$1>>2]|0;
      $1160 = HEAP32[$pTabList>>2]|0;
      $1161 = HEAP32[$pEList>>2]|0;
      _generateColumnNames($1159,$1160,$1161);
     }
    }
    $1162 = HEAP32[$db>>2]|0;
    $1163 = (($sAggInfo) + 28|0);
    $1164 = HEAP32[$1163>>2]|0;
    _sqlite3DbFree($1162,$1164);
    $1165 = HEAP32[$db>>2]|0;
    $1166 = (($sAggInfo) + 40|0);
    $1167 = HEAP32[$1166>>2]|0;
    _sqlite3DbFree($1165,$1167);
    $1168 = HEAP32[$rc>>2]|0;
    HEAP32[$0>>2] = $1168;
    $1169 = HEAP32[$0>>2]|0;
    STACKTOP = sp;return ($1169|0);
   }
  }
 }
 HEAP32[$0>>2] = 1;
 $1169 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($1169|0);
}
function _sqlite3SrcListAppendFromTerm($pParse,$p,$pTable,$pDatabase,$pAlias,$pSubquery,$pOn,$pUsing) {
 $pParse = $pParse|0;
 $p = $p|0;
 $pTable = $pTable|0;
 $pDatabase = $pDatabase|0;
 $pAlias = $pAlias|0;
 $pSubquery = $pSubquery|0;
 $pOn = $pOn|0;
 $pUsing = $pUsing|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $7 = 0, $8 = 0, $9 = 0, $db = 0, $or$cond = 0, $pItem = 0, $vararg_buffer = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $vararg_buffer = sp;
 $0 = sp + 44|0;
 $1 = sp + 40|0;
 $2 = sp + 36|0;
 $3 = sp + 32|0;
 $4 = sp + 28|0;
 $5 = sp + 24|0;
 $6 = sp + 20|0;
 $7 = sp + 16|0;
 $8 = sp + 12|0;
 $pItem = sp + 8|0;
 $db = sp + 4|0;
 HEAP32[$1>>2] = $pParse;
 HEAP32[$2>>2] = $p;
 HEAP32[$3>>2] = $pTable;
 HEAP32[$4>>2] = $pDatabase;
 HEAP32[$5>>2] = $pAlias;
 HEAP32[$6>>2] = $pSubquery;
 HEAP32[$7>>2] = $pOn;
 HEAP32[$8>>2] = $pUsing;
 $9 = HEAP32[$1>>2]|0;
 $10 = HEAP32[$9>>2]|0;
 HEAP32[$db>>2] = $10;
 $11 = HEAP32[$2>>2]|0;
 $12 = ($11|0)!=(0|0);
 if ($12) {
  label = 4;
 } else {
  $13 = HEAP32[$7>>2]|0;
  $14 = ($13|0)!=(0|0);
  $15 = HEAP32[$8>>2]|0;
  $16 = ($15|0)!=(0|0);
  $or$cond = $14 | $16;
  if ($or$cond) {
   $17 = HEAP32[$1>>2]|0;
   $18 = HEAP32[$7>>2]|0;
   $19 = ($18|0)!=(0|0);
   $20 = $19 ? 24816 : 24824;
   HEAP32[$vararg_buffer>>2] = $20;
   _sqlite3ErrorMsg($17,24776,$vararg_buffer);
  } else {
   label = 4;
  }
 }
 if ((label|0) == 4) {
  $21 = HEAP32[$db>>2]|0;
  $22 = HEAP32[$2>>2]|0;
  $23 = HEAP32[$3>>2]|0;
  $24 = HEAP32[$4>>2]|0;
  $25 = (_sqlite3SrcListAppend($21,$22,$23,$24)|0);
  HEAP32[$2>>2] = $25;
  $26 = HEAP32[$2>>2]|0;
  $27 = ($26|0)==(0|0);
  if (!($27)) {
   $28 = HEAP32[$2>>2]|0;
   $29 = HEAP32[$28>>2]|0;
   $30 = ($29|0)==(0);
   if (!($30)) {
    $31 = HEAP32[$2>>2]|0;
    $32 = HEAP32[$31>>2]|0;
    $33 = (($32) - 1)|0;
    $34 = HEAP32[$2>>2]|0;
    $35 = (($34) + 8|0);
    $36 = (($35) + (($33*72)|0)|0);
    HEAP32[$pItem>>2] = $36;
    $37 = HEAP32[$5>>2]|0;
    $38 = (($37) + 4|0);
    $39 = HEAP32[$38>>2]|0;
    $40 = ($39|0)!=(0);
    if ($40) {
     $41 = HEAP32[$db>>2]|0;
     $42 = HEAP32[$5>>2]|0;
     $43 = (_sqlite3NameFromToken($41,$42)|0);
     $44 = HEAP32[$pItem>>2]|0;
     $45 = (($44) + 12|0);
     HEAP32[$45>>2] = $43;
    }
    $46 = HEAP32[$6>>2]|0;
    $47 = HEAP32[$pItem>>2]|0;
    $48 = (($47) + 20|0);
    HEAP32[$48>>2] = $46;
    $49 = HEAP32[$7>>2]|0;
    $50 = HEAP32[$pItem>>2]|0;
    $51 = (($50) + 44|0);
    HEAP32[$51>>2] = $49;
    $52 = HEAP32[$8>>2]|0;
    $53 = HEAP32[$pItem>>2]|0;
    $54 = (($53) + 48|0);
    HEAP32[$54>>2] = $52;
    $55 = HEAP32[$2>>2]|0;
    HEAP32[$0>>2] = $55;
    $62 = HEAP32[$0>>2]|0;
    STACKTOP = sp;return ($62|0);
   }
  }
 }
 $56 = HEAP32[$db>>2]|0;
 $57 = HEAP32[$7>>2]|0;
 _sqlite3ExprDelete($56,$57);
 $58 = HEAP32[$db>>2]|0;
 $59 = HEAP32[$8>>2]|0;
 _sqlite3IdListDelete($58,$59);
 $60 = HEAP32[$db>>2]|0;
 $61 = HEAP32[$6>>2]|0;
 _sqlite3SelectDelete($60,$61);
 HEAP32[$0>>2] = 0;
 $62 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($62|0);
}
function _sqlite3SelectNew($pParse,$pEList,$pSrc,$pWhere,$pGroupBy,$pHaving,$pOrderBy,$selFlags,$pLimit,$pOffset) {
 $pParse = $pParse|0;
 $pEList = $pEList|0;
 $pSrc = $pSrc|0;
 $pWhere = $pWhere|0;
 $pGroupBy = $pGroupBy|0;
 $pHaving = $pHaving|0;
 $pOrderBy = $pOrderBy|0;
 $selFlags = $selFlags|0;
 $pLimit = $pLimit|0;
 $pOffset = $pOffset|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $8 = 0, $9 = 0, $db = 0, $pNew = 0, $standin = 0, dest = 0, label = 0, sp = 0, stop = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 128|0;
 $0 = sp + 112|0;
 $1 = sp + 108|0;
 $2 = sp + 104|0;
 $3 = sp + 100|0;
 $4 = sp + 96|0;
 $5 = sp + 92|0;
 $6 = sp + 88|0;
 $7 = sp + 116|0;
 $8 = sp + 84|0;
 $9 = sp + 80|0;
 $pNew = sp + 76|0;
 $standin = sp;
 $db = sp + 72|0;
 HEAP32[$0>>2] = $pParse;
 HEAP32[$1>>2] = $pEList;
 HEAP32[$2>>2] = $pSrc;
 HEAP32[$3>>2] = $pWhere;
 HEAP32[$4>>2] = $pGroupBy;
 HEAP32[$5>>2] = $pHaving;
 HEAP32[$6>>2] = $pOrderBy;
 HEAP16[$7>>1] = $selFlags;
 HEAP32[$8>>2] = $pLimit;
 HEAP32[$9>>2] = $pOffset;
 $10 = HEAP32[$0>>2]|0;
 $11 = HEAP32[$10>>2]|0;
 HEAP32[$db>>2] = $11;
 $12 = HEAP32[$db>>2]|0;
 $13 = (_sqlite3DbMallocZero($12,72,0)|0);
 HEAP32[$pNew>>2] = $13;
 $14 = HEAP32[$pNew>>2]|0;
 $15 = ($14|0)==(0|0);
 if ($15) {
  HEAP32[$pNew>>2] = $standin;
  $16 = HEAP32[$pNew>>2]|0;
  dest=$16+0|0; stop=dest+72|0; do { HEAP32[dest>>2]=0|0; dest=dest+4|0; } while ((dest|0) < (stop|0));
 }
 $17 = HEAP32[$1>>2]|0;
 $18 = ($17|0)==(0|0);
 if ($18) {
  $19 = HEAP32[$0>>2]|0;
  $20 = HEAP32[$db>>2]|0;
  $21 = (_sqlite3Expr($20,116,0)|0);
  $22 = (_sqlite3ExprListAppend($19,0,$21)|0);
  HEAP32[$1>>2] = $22;
 }
 $23 = HEAP32[$1>>2]|0;
 $24 = HEAP32[$pNew>>2]|0;
 HEAP32[$24>>2] = $23;
 $25 = HEAP32[$2>>2]|0;
 $26 = ($25|0)==(0|0);
 if ($26) {
  $27 = HEAP32[$db>>2]|0;
  $28 = (_sqlite3DbMallocZero($27,80,0)|0);
  HEAP32[$2>>2] = $28;
 }
 $29 = HEAP32[$2>>2]|0;
 $30 = HEAP32[$pNew>>2]|0;
 $31 = (($30) + 32|0);
 HEAP32[$31>>2] = $29;
 $32 = HEAP32[$3>>2]|0;
 $33 = HEAP32[$pNew>>2]|0;
 $34 = (($33) + 36|0);
 HEAP32[$34>>2] = $32;
 $35 = HEAP32[$4>>2]|0;
 $36 = HEAP32[$pNew>>2]|0;
 $37 = (($36) + 40|0);
 HEAP32[$37>>2] = $35;
 $38 = HEAP32[$5>>2]|0;
 $39 = HEAP32[$pNew>>2]|0;
 $40 = (($39) + 44|0);
 HEAP32[$40>>2] = $38;
 $41 = HEAP32[$6>>2]|0;
 $42 = HEAP32[$pNew>>2]|0;
 $43 = (($42) + 48|0);
 HEAP32[$43>>2] = $41;
 $44 = HEAP16[$7>>1]|0;
 $45 = HEAP32[$pNew>>2]|0;
 $46 = (($45) + 6|0);
 HEAP16[$46>>1] = $44;
 $47 = HEAP32[$pNew>>2]|0;
 $48 = (($47) + 4|0);
 HEAP8[$48>>0] = 119;
 $49 = HEAP32[$8>>2]|0;
 $50 = HEAP32[$pNew>>2]|0;
 $51 = (($50) + 60|0);
 HEAP32[$51>>2] = $49;
 $52 = HEAP32[$9>>2]|0;
 $53 = HEAP32[$pNew>>2]|0;
 $54 = (($53) + 64|0);
 HEAP32[$54>>2] = $52;
 $55 = HEAP32[$pNew>>2]|0;
 $56 = (($55) + 16|0);
 HEAP32[$56>>2] = -1;
 $57 = HEAP32[$pNew>>2]|0;
 $58 = (($57) + 16|0);
 $59 = (($58) + 4|0);
 HEAP32[$59>>2] = -1;
 $60 = HEAP32[$db>>2]|0;
 $61 = (($60) + 64|0);
 $62 = HEAP8[$61>>0]|0;
 $63 = ($62<<24>>24)!=(0);
 if (!($63)) {
  $70 = HEAP32[$pNew>>2]|0;
  STACKTOP = sp;return ($70|0);
 }
 $64 = HEAP32[$db>>2]|0;
 $65 = HEAP32[$pNew>>2]|0;
 _clearSelect($64,$65);
 $66 = HEAP32[$pNew>>2]|0;
 $67 = ($66|0)!=($standin|0);
 if ($67) {
  $68 = HEAP32[$db>>2]|0;
  $69 = HEAP32[$pNew>>2]|0;
  _sqlite3DbFree($68,$69);
 }
 HEAP32[$pNew>>2] = 0;
 $70 = HEAP32[$pNew>>2]|0;
 STACKTOP = sp;return ($70|0);
}
function _sqlite3ExprListAppend($pParse,$pList,$pExpr) {
 $pParse = $pParse|0;
 $pList = $pList|0;
 $pExpr = $pExpr|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $a = 0, $db = 0, $pItem = 0;
 var label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 24|0;
 $1 = sp + 20|0;
 $2 = sp + 16|0;
 $3 = sp + 12|0;
 $db = sp + 8|0;
 $a = sp + 4|0;
 $pItem = sp;
 HEAP32[$1>>2] = $pParse;
 HEAP32[$2>>2] = $pList;
 HEAP32[$3>>2] = $pExpr;
 $4 = HEAP32[$1>>2]|0;
 $5 = HEAP32[$4>>2]|0;
 HEAP32[$db>>2] = $5;
 $6 = HEAP32[$2>>2]|0;
 $7 = ($6|0)==(0|0);
 if ($7) {
  $8 = HEAP32[$db>>2]|0;
  $9 = (_sqlite3DbMallocZero($8,8,0)|0);
  HEAP32[$2>>2] = $9;
  $10 = HEAP32[$2>>2]|0;
  $11 = ($10|0)==(0|0);
  if ($11) {
   label = 8;
  } else {
   $12 = HEAP32[$db>>2]|0;
   $13 = (_sqlite3DbMallocRaw($12,20,0)|0);
   $14 = HEAP32[$2>>2]|0;
   $15 = (($14) + 4|0);
   HEAP32[$15>>2] = $13;
   $16 = HEAP32[$2>>2]|0;
   $17 = (($16) + 4|0);
   $18 = HEAP32[$17>>2]|0;
   $19 = ($18|0)==(0|0);
   if ($19) {
    label = 8;
   } else {
    label = 7;
   }
  }
 } else {
  $20 = HEAP32[$2>>2]|0;
  $21 = HEAP32[$20>>2]|0;
  $22 = HEAP32[$2>>2]|0;
  $23 = HEAP32[$22>>2]|0;
  $24 = (($23) - 1)|0;
  $25 = $21 & $24;
  $26 = ($25|0)==(0);
  if ($26) {
   $27 = HEAP32[$db>>2]|0;
   $28 = HEAP32[$2>>2]|0;
   $29 = (($28) + 4|0);
   $30 = HEAP32[$29>>2]|0;
   $31 = HEAP32[$2>>2]|0;
   $32 = HEAP32[$31>>2]|0;
   $33 = $32<<1;
   $34 = ($33*20)|0;
   $35 = (_sqlite3DbRealloc($27,$30,$34,0)|0);
   HEAP32[$a>>2] = $35;
   $36 = HEAP32[$a>>2]|0;
   $37 = ($36|0)==(0|0);
   if ($37) {
    label = 8;
   } else {
    $38 = HEAP32[$a>>2]|0;
    $39 = HEAP32[$2>>2]|0;
    $40 = (($39) + 4|0);
    HEAP32[$40>>2] = $38;
    label = 7;
   }
  } else {
   label = 7;
  }
 }
 if ((label|0) == 7) {
  $41 = HEAP32[$2>>2]|0;
  $42 = HEAP32[$41>>2]|0;
  $43 = (($42) + 1)|0;
  HEAP32[$41>>2] = $43;
  $44 = HEAP32[$2>>2]|0;
  $45 = (($44) + 4|0);
  $46 = HEAP32[$45>>2]|0;
  $47 = (($46) + (($42*20)|0)|0);
  HEAP32[$pItem>>2] = $47;
  $48 = HEAP32[$pItem>>2]|0;
  ;HEAP32[$48+0>>2]=0|0;HEAP32[$48+4>>2]=0|0;HEAP32[$48+8>>2]=0|0;HEAP32[$48+12>>2]=0|0;HEAP32[$48+16>>2]=0|0;
  $49 = HEAP32[$3>>2]|0;
  $50 = HEAP32[$pItem>>2]|0;
  HEAP32[$50>>2] = $49;
  $51 = HEAP32[$2>>2]|0;
  HEAP32[$0>>2] = $51;
  $56 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($56|0);
 }
 else if ((label|0) == 8) {
  $52 = HEAP32[$db>>2]|0;
  $53 = HEAP32[$3>>2]|0;
  _sqlite3ExprDelete($52,$53);
  $54 = HEAP32[$db>>2]|0;
  $55 = HEAP32[$2>>2]|0;
  _sqlite3ExprListDelete($54,$55);
  HEAP32[$0>>2] = 0;
  $56 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($56|0);
 }
 return 0|0;
}
function _sqlite3ExprListSetName($pParse,$pList,$pName,$dequote) {
 $pParse = $pParse|0;
 $pList = $pList|0;
 $pName = $pName|0;
 $dequote = $dequote|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $pItem = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 16|0;
 $1 = sp + 12|0;
 $2 = sp + 8|0;
 $3 = sp + 4|0;
 $pItem = sp;
 HEAP32[$0>>2] = $pParse;
 HEAP32[$1>>2] = $pList;
 HEAP32[$2>>2] = $pName;
 HEAP32[$3>>2] = $dequote;
 $4 = HEAP32[$1>>2]|0;
 $5 = ($4|0)!=(0|0);
 if (!($5)) {
  STACKTOP = sp;return;
 }
 $6 = HEAP32[$1>>2]|0;
 $7 = HEAP32[$6>>2]|0;
 $8 = (($7) - 1)|0;
 $9 = HEAP32[$1>>2]|0;
 $10 = (($9) + 4|0);
 $11 = HEAP32[$10>>2]|0;
 $12 = (($11) + (($8*20)|0)|0);
 HEAP32[$pItem>>2] = $12;
 $13 = HEAP32[$0>>2]|0;
 $14 = HEAP32[$13>>2]|0;
 $15 = HEAP32[$2>>2]|0;
 $16 = HEAP32[$15>>2]|0;
 $17 = HEAP32[$2>>2]|0;
 $18 = (($17) + 4|0);
 $19 = HEAP32[$18>>2]|0;
 $20 = (_sqlite3DbStrNDup($14,$16,$19,0)|0);
 $21 = HEAP32[$pItem>>2]|0;
 $22 = (($21) + 4|0);
 HEAP32[$22>>2] = $20;
 $23 = HEAP32[$3>>2]|0;
 $24 = ($23|0)!=(0);
 if (!($24)) {
  STACKTOP = sp;return;
 }
 $25 = HEAP32[$pItem>>2]|0;
 $26 = (($25) + 4|0);
 $27 = HEAP32[$26>>2]|0;
 $28 = ($27|0)!=(0|0);
 if (!($28)) {
  STACKTOP = sp;return;
 }
 $29 = HEAP32[$pItem>>2]|0;
 $30 = (($29) + 4|0);
 $31 = HEAP32[$30>>2]|0;
 (_sqlite3Dequote($31)|0);
 STACKTOP = sp;return;
}
function _sqlite3ExprListSetSpan($pParse,$pList,$pSpan) {
 $pParse = $pParse|0;
 $pList = $pList|0;
 $pSpan = $pSpan|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $db = 0, $pItem = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 16|0;
 $1 = sp + 12|0;
 $2 = sp + 8|0;
 $db = sp + 4|0;
 $pItem = sp;
 HEAP32[$0>>2] = $pParse;
 HEAP32[$1>>2] = $pList;
 HEAP32[$2>>2] = $pSpan;
 $3 = HEAP32[$0>>2]|0;
 $4 = HEAP32[$3>>2]|0;
 HEAP32[$db>>2] = $4;
 $5 = HEAP32[$1>>2]|0;
 $6 = ($5|0)!=(0|0);
 if (!($6)) {
  STACKTOP = sp;return;
 }
 $7 = HEAP32[$1>>2]|0;
 $8 = HEAP32[$7>>2]|0;
 $9 = (($8) - 1)|0;
 $10 = HEAP32[$1>>2]|0;
 $11 = (($10) + 4|0);
 $12 = HEAP32[$11>>2]|0;
 $13 = (($12) + (($9*20)|0)|0);
 HEAP32[$pItem>>2] = $13;
 $14 = HEAP32[$db>>2]|0;
 $15 = HEAP32[$pItem>>2]|0;
 $16 = (($15) + 8|0);
 $17 = HEAP32[$16>>2]|0;
 _sqlite3DbFree($14,$17);
 $18 = HEAP32[$db>>2]|0;
 $19 = HEAP32[$2>>2]|0;
 $20 = (($19) + 4|0);
 $21 = HEAP32[$20>>2]|0;
 $22 = HEAP32[$2>>2]|0;
 $23 = (($22) + 8|0);
 $24 = HEAP32[$23>>2]|0;
 $25 = HEAP32[$2>>2]|0;
 $26 = (($25) + 4|0);
 $27 = HEAP32[$26>>2]|0;
 $28 = $24;
 $29 = $27;
 $30 = (($28) - ($29))|0;
 $31 = ($30|0)<(0);
 $32 = $31 << 31 >> 31;
 $33 = (_sqlite3DbStrNDup($18,$21,$30,$32)|0);
 $34 = HEAP32[$pItem>>2]|0;
 $35 = (($34) + 8|0);
 HEAP32[$35>>2] = $33;
 STACKTOP = sp;return;
}
function _sqlite3Expr($db,$op,$zToken) {
 $db = $db|0;
 $op = $op|0;
 $zToken = $zToken|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $x = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 16|0;
 $1 = sp + 12|0;
 $2 = sp + 8|0;
 $x = sp;
 HEAP32[$0>>2] = $db;
 HEAP32[$1>>2] = $op;
 HEAP32[$2>>2] = $zToken;
 $3 = HEAP32[$2>>2]|0;
 HEAP32[$x>>2] = $3;
 $4 = HEAP32[$2>>2]|0;
 $5 = ($4|0)!=(0|0);
 if ($5) {
  $6 = HEAP32[$2>>2]|0;
  $7 = (_sqlite3Strlen30($6)|0);
  $9 = $7;
 } else {
  $9 = 0;
 }
 $8 = (($x) + 4|0);
 HEAP32[$8>>2] = $9;
 $10 = HEAP32[$0>>2]|0;
 $11 = HEAP32[$1>>2]|0;
 $12 = (_sqlite3ExprAlloc($10,$11,$x,0)|0);
 STACKTOP = sp;return ($12|0);
}
function _sqlite3SrcListShiftJoinType($p) {
 $p = $p|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $3 = 0, $4 = 0;
 var $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $i = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $i = sp;
 HEAP32[$0>>2] = $p;
 $1 = HEAP32[$0>>2]|0;
 $2 = ($1|0)!=(0|0);
 if (!($2)) {
  STACKTOP = sp;return;
 }
 $3 = HEAP32[$0>>2]|0;
 $4 = HEAP32[$3>>2]|0;
 $5 = (($4) - 1)|0;
 HEAP32[$i>>2] = $5;
 while(1) {
  $6 = HEAP32[$i>>2]|0;
  $7 = ($6|0)>(0);
  if (!($7)) {
   break;
  }
  $8 = HEAP32[$i>>2]|0;
  $9 = (($8) - 1)|0;
  $10 = HEAP32[$0>>2]|0;
  $11 = (($10) + 8|0);
  $12 = (($11) + (($9*72)|0)|0);
  $13 = (($12) + 36|0);
  $14 = HEAP8[$13>>0]|0;
  $15 = HEAP32[$i>>2]|0;
  $16 = HEAP32[$0>>2]|0;
  $17 = (($16) + 8|0);
  $18 = (($17) + (($15*72)|0)|0);
  $19 = (($18) + 36|0);
  HEAP8[$19>>0] = $14;
  $20 = HEAP32[$i>>2]|0;
  $21 = (($20) + -1)|0;
  HEAP32[$i>>2] = $21;
 }
 $22 = HEAP32[$0>>2]|0;
 $23 = (($22) + 8|0);
 $24 = (($23) + 36|0);
 HEAP8[$24>>0] = 0;
 STACKTOP = sp;return;
}
function _sqlite3SrcListIndexedBy($pParse,$p,$pIndexedBy) {
 $pParse = $pParse|0;
 $p = $p|0;
 $pIndexedBy = $pIndexedBy|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $pItem = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $1 = sp + 8|0;
 $2 = sp + 4|0;
 $pItem = sp;
 HEAP32[$0>>2] = $pParse;
 HEAP32[$1>>2] = $p;
 HEAP32[$2>>2] = $pIndexedBy;
 $3 = HEAP32[$1>>2]|0;
 $4 = ($3|0)!=(0|0);
 if (!($4)) {
  STACKTOP = sp;return;
 }
 $5 = HEAP32[$1>>2]|0;
 $6 = HEAP32[$5>>2]|0;
 $7 = ($6|0)>(0);
 if (!($7)) {
  STACKTOP = sp;return;
 }
 $8 = HEAP32[$1>>2]|0;
 $9 = HEAP32[$8>>2]|0;
 $10 = (($9) - 1)|0;
 $11 = HEAP32[$1>>2]|0;
 $12 = (($11) + 8|0);
 $13 = (($12) + (($10*72)|0)|0);
 HEAP32[$pItem>>2] = $13;
 $14 = HEAP32[$2>>2]|0;
 $15 = (($14) + 4|0);
 $16 = HEAP32[$15>>2]|0;
 $17 = ($16|0)==(1);
 if ($17) {
  $18 = HEAP32[$2>>2]|0;
  $19 = HEAP32[$18>>2]|0;
  $20 = ($19|0)!=(0|0);
  if (!($20)) {
   $21 = HEAP32[$pItem>>2]|0;
   $22 = (($21) + 37|0);
   $23 = HEAP8[$22>>0]|0;
   $24 = $23 & -2;
   $25 = $24 | 1;
   HEAP8[$22>>0] = $25;
   STACKTOP = sp;return;
  }
 }
 $26 = HEAP32[$0>>2]|0;
 $27 = HEAP32[$26>>2]|0;
 $28 = HEAP32[$2>>2]|0;
 $29 = (_sqlite3NameFromToken($27,$28)|0);
 $30 = HEAP32[$pItem>>2]|0;
 $31 = (($30) + 64|0);
 HEAP32[$31>>2] = $29;
 STACKTOP = sp;return;
}
function _sqlite3SrcListAppend($db,$pList,$pTable,$pDatabase) {
 $db = $db|0;
 $pList = $pList|0;
 $pTable = $pTable|0;
 $pDatabase = $pDatabase|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $pItem = 0, $pTemp = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 24|0;
 $1 = sp + 20|0;
 $2 = sp + 16|0;
 $3 = sp + 12|0;
 $4 = sp + 8|0;
 $pItem = sp + 4|0;
 $pTemp = sp;
 HEAP32[$1>>2] = $db;
 HEAP32[$2>>2] = $pList;
 HEAP32[$3>>2] = $pTable;
 HEAP32[$4>>2] = $pDatabase;
 $5 = HEAP32[$2>>2]|0;
 $6 = ($5|0)==(0|0);
 do {
  if ($6) {
   $7 = HEAP32[$1>>2]|0;
   $8 = (_sqlite3DbMallocZero($7,80,0)|0);
   HEAP32[$2>>2] = $8;
   $9 = HEAP32[$2>>2]|0;
   $10 = ($9|0)==(0|0);
   if (!($10)) {
    $11 = HEAP32[$2>>2]|0;
    $12 = (($11) + 4|0);
    HEAP32[$12>>2] = 1;
    break;
   }
   HEAP32[$0>>2] = 0;
   $51 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($51|0);
  }
 } while(0);
 $13 = HEAP32[$1>>2]|0;
 $14 = HEAP32[$2>>2]|0;
 $15 = HEAP32[$2>>2]|0;
 $16 = HEAP32[$15>>2]|0;
 $17 = (_sqlite3SrcListEnlarge($13,$14,1,$16)|0);
 HEAP32[$2>>2] = $17;
 $18 = HEAP32[$1>>2]|0;
 $19 = (($18) + 64|0);
 $20 = HEAP8[$19>>0]|0;
 $21 = ($20<<24>>24)!=(0);
 if ($21) {
  $22 = HEAP32[$1>>2]|0;
  $23 = HEAP32[$2>>2]|0;
  _sqlite3SrcListDelete($22,$23);
  HEAP32[$0>>2] = 0;
  $51 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($51|0);
 }
 $24 = HEAP32[$2>>2]|0;
 $25 = HEAP32[$24>>2]|0;
 $26 = (($25) - 1)|0;
 $27 = HEAP32[$2>>2]|0;
 $28 = (($27) + 8|0);
 $29 = (($28) + (($26*72)|0)|0);
 HEAP32[$pItem>>2] = $29;
 $30 = HEAP32[$4>>2]|0;
 $31 = ($30|0)!=(0|0);
 if ($31) {
  $32 = HEAP32[$4>>2]|0;
  $33 = HEAP32[$32>>2]|0;
  $34 = ($33|0)==(0|0);
  if ($34) {
   HEAP32[$4>>2] = 0;
  }
 }
 $35 = HEAP32[$4>>2]|0;
 $36 = ($35|0)!=(0|0);
 if ($36) {
  $37 = HEAP32[$4>>2]|0;
  HEAP32[$pTemp>>2] = $37;
  $38 = HEAP32[$3>>2]|0;
  HEAP32[$4>>2] = $38;
  $39 = HEAP32[$pTemp>>2]|0;
  HEAP32[$3>>2] = $39;
 }
 $40 = HEAP32[$1>>2]|0;
 $41 = HEAP32[$3>>2]|0;
 $42 = (_sqlite3NameFromToken($40,$41)|0);
 $43 = HEAP32[$pItem>>2]|0;
 $44 = (($43) + 8|0);
 HEAP32[$44>>2] = $42;
 $45 = HEAP32[$1>>2]|0;
 $46 = HEAP32[$4>>2]|0;
 $47 = (_sqlite3NameFromToken($45,$46)|0);
 $48 = HEAP32[$pItem>>2]|0;
 $49 = (($48) + 4|0);
 HEAP32[$49>>2] = $47;
 $50 = HEAP32[$2>>2]|0;
 HEAP32[$0>>2] = $50;
 $51 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($51|0);
}
function _sqlite3JoinType($pParse,$pA,$pB,$pC) {
 $pParse = $pParse|0;
 $pA = $pA|0;
 $pB = $pB|0;
 $pC = $pC|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $8 = 0, $9 = 0, $apAll = 0;
 var $i = 0, $j = 0, $jointype = 0, $p = 0, $vararg_buffer4 = 0, $vararg_ptr1 = 0, $vararg_ptr2 = 0, $vararg_ptr3 = 0, $zSp = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 64|0;
 $vararg_buffer4 = sp;
 $0 = sp + 60|0;
 $1 = sp + 56|0;
 $2 = sp + 52|0;
 $3 = sp + 48|0;
 $jointype = sp + 44|0;
 $apAll = sp + 32|0;
 $p = sp + 28|0;
 $i = sp + 24|0;
 $j = sp + 20|0;
 $zSp = sp + 16|0;
 HEAP32[$0>>2] = $pParse;
 HEAP32[$1>>2] = $pA;
 HEAP32[$2>>2] = $pB;
 HEAP32[$3>>2] = $pC;
 HEAP32[$jointype>>2] = 0;
 $4 = HEAP32[$1>>2]|0;
 HEAP32[$apAll>>2] = $4;
 $5 = HEAP32[$2>>2]|0;
 $6 = (($apAll) + 4|0);
 HEAP32[$6>>2] = $5;
 $7 = HEAP32[$3>>2]|0;
 $8 = (($apAll) + 8|0);
 HEAP32[$8>>2] = $7;
 HEAP32[$i>>2] = 0;
 while(1) {
  $9 = HEAP32[$i>>2]|0;
  $10 = ($9|0)<(3);
  if (!($10)) {
   break;
  }
  $11 = HEAP32[$i>>2]|0;
  $12 = (($apAll) + ($11<<2)|0);
  $13 = HEAP32[$12>>2]|0;
  $14 = ($13|0)!=(0|0);
  if (!($14)) {
   break;
  }
  $15 = HEAP32[$i>>2]|0;
  $16 = (($apAll) + ($15<<2)|0);
  $17 = HEAP32[$16>>2]|0;
  HEAP32[$p>>2] = $17;
  HEAP32[$j>>2] = 0;
  while(1) {
   $18 = HEAP32[$j>>2]|0;
   $19 = ($18|0)<(7);
   if (!($19)) {
    break;
   }
   $20 = HEAP32[$p>>2]|0;
   $21 = (($20) + 4|0);
   $22 = HEAP32[$21>>2]|0;
   $23 = HEAP32[$j>>2]|0;
   $24 = (24648 + (($23*3)|0)|0);
   $25 = (($24) + 1|0);
   $26 = HEAP8[$25>>0]|0;
   $27 = $26&255;
   $28 = ($22|0)==($27|0);
   if ($28) {
    $29 = HEAP32[$p>>2]|0;
    $30 = HEAP32[$29>>2]|0;
    $31 = HEAP32[$j>>2]|0;
    $32 = (24648 + (($31*3)|0)|0);
    $33 = HEAP8[$32>>0]|0;
    $34 = $33&255;
    $35 = (24608 + ($34)|0);
    $36 = HEAP32[$p>>2]|0;
    $37 = (($36) + 4|0);
    $38 = HEAP32[$37>>2]|0;
    $39 = (_sqlite3_strnicmp($30,$35,$38)|0);
    $40 = ($39|0)==(0);
    if ($40) {
     label = 8;
     break;
    }
   }
   $48 = HEAP32[$j>>2]|0;
   $49 = (($48) + 1)|0;
   HEAP32[$j>>2] = $49;
  }
  if ((label|0) == 8) {
   label = 0;
   $41 = HEAP32[$j>>2]|0;
   $42 = (24648 + (($41*3)|0)|0);
   $43 = (($42) + 2|0);
   $44 = HEAP8[$43>>0]|0;
   $45 = $44&255;
   $46 = HEAP32[$jointype>>2]|0;
   $47 = $46 | $45;
   HEAP32[$jointype>>2] = $47;
  }
  $50 = HEAP32[$j>>2]|0;
  $51 = ($50|0)>=(7);
  if ($51) {
   label = 11;
   break;
  }
  $54 = HEAP32[$i>>2]|0;
  $55 = (($54) + 1)|0;
  HEAP32[$i>>2] = $55;
 }
 if ((label|0) == 11) {
  $52 = HEAP32[$jointype>>2]|0;
  $53 = $52 | 64;
  HEAP32[$jointype>>2] = $53;
 }
 $56 = HEAP32[$jointype>>2]|0;
 $57 = $56 & 33;
 $58 = ($57|0)==(33);
 if (!($58)) {
  $59 = HEAP32[$jointype>>2]|0;
  $60 = $59 & 64;
  $61 = ($60|0)!=(0);
  if (!($61)) {
   $71 = HEAP32[$jointype>>2]|0;
   $72 = $71 & 32;
   $73 = ($72|0)!=(0);
   if (!($73)) {
    $78 = HEAP32[$jointype>>2]|0;
    STACKTOP = sp;return ($78|0);
   }
   $74 = HEAP32[$jointype>>2]|0;
   $75 = $74 & 24;
   $76 = ($75|0)!=(8);
   if (!($76)) {
    $78 = HEAP32[$jointype>>2]|0;
    STACKTOP = sp;return ($78|0);
   }
   $77 = HEAP32[$0>>2]|0;
   _sqlite3ErrorMsg($77,24720,$vararg_buffer4);
   HEAP32[$jointype>>2] = 1;
   $78 = HEAP32[$jointype>>2]|0;
   STACKTOP = sp;return ($78|0);
  }
 }
 HEAP32[$zSp>>2] = 9912;
 $62 = HEAP32[$3>>2]|0;
 $63 = ($62|0)==(0|0);
 if ($63) {
  $64 = HEAP32[$zSp>>2]|0;
  $65 = (($64) + 1|0);
  HEAP32[$zSp>>2] = $65;
 }
 $66 = HEAP32[$0>>2]|0;
 $67 = HEAP32[$1>>2]|0;
 $68 = HEAP32[$2>>2]|0;
 $69 = HEAP32[$zSp>>2]|0;
 $70 = HEAP32[$3>>2]|0;
 HEAP32[$vararg_buffer4>>2] = $67;
 $vararg_ptr1 = (($vararg_buffer4) + 4|0);
 HEAP32[$vararg_ptr1>>2] = $68;
 $vararg_ptr2 = (($vararg_buffer4) + 8|0);
 HEAP32[$vararg_ptr2>>2] = $69;
 $vararg_ptr3 = (($vararg_buffer4) + 12|0);
 HEAP32[$vararg_ptr3>>2] = $70;
 _sqlite3ErrorMsg($66,24672,$vararg_buffer4);
 HEAP32[$jointype>>2] = 1;
 $78 = HEAP32[$jointype>>2]|0;
 STACKTOP = sp;return ($78|0);
}
function _sqlite3WithPush($pParse,$pWith,$bFree) {
 $pParse = $pParse|0;
 $pWith = $pWith|0;
 $bFree = $bFree|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $1 = sp;
 $2 = sp + 8|0;
 HEAP32[$0>>2] = $pParse;
 HEAP32[$1>>2] = $pWith;
 HEAP8[$2>>0] = $bFree;
 $3 = HEAP32[$1>>2]|0;
 $4 = ($3|0)!=(0|0);
 if (!($4)) {
  STACKTOP = sp;return;
 }
 $5 = HEAP32[$0>>2]|0;
 $6 = (($5) + 532|0);
 $7 = HEAP32[$6>>2]|0;
 $8 = HEAP32[$1>>2]|0;
 $9 = (($8) + 4|0);
 HEAP32[$9>>2] = $7;
 $10 = HEAP32[$1>>2]|0;
 $11 = HEAP32[$0>>2]|0;
 $12 = (($11) + 532|0);
 HEAP32[$12>>2] = $10;
 $13 = HEAP8[$2>>0]|0;
 $14 = HEAP32[$0>>2]|0;
 $15 = (($14) + 449|0);
 HEAP8[$15>>0] = $13;
 STACKTOP = sp;return;
}
function _sqlite3DeleteFrom($pParse,$pTabList,$pWhere) {
 $pParse = $pParse|0;
 $pTabList = $pTabList|0;
 $pWhere = $pWhere|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0;
 var $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0, $188 = 0;
 var $189 = 0, $19 = 0, $190 = 0, $191 = 0, $192 = 0, $193 = 0, $194 = 0, $195 = 0, $196 = 0, $197 = 0, $198 = 0, $199 = 0, $2 = 0, $20 = 0, $200 = 0, $201 = 0, $202 = 0, $203 = 0, $204 = 0, $205 = 0;
 var $206 = 0, $207 = 0, $208 = 0, $209 = 0, $21 = 0, $210 = 0, $211 = 0, $212 = 0, $213 = 0, $214 = 0, $215 = 0, $216 = 0, $217 = 0, $218 = 0, $219 = 0, $22 = 0, $220 = 0, $221 = 0, $222 = 0, $223 = 0;
 var $224 = 0, $225 = 0, $226 = 0, $227 = 0, $228 = 0, $229 = 0, $23 = 0, $230 = 0, $231 = 0, $232 = 0, $233 = 0, $234 = 0, $235 = 0, $236 = 0, $237 = 0, $238 = 0, $239 = 0, $24 = 0, $240 = 0, $241 = 0;
 var $242 = 0, $243 = 0, $244 = 0, $245 = 0, $246 = 0, $247 = 0, $248 = 0, $249 = 0, $25 = 0, $250 = 0, $251 = 0, $252 = 0, $253 = 0, $254 = 0, $255 = 0, $256 = 0, $257 = 0, $258 = 0, $259 = 0, $26 = 0;
 var $260 = 0, $261 = 0, $262 = 0, $263 = 0, $264 = 0, $265 = 0, $266 = 0, $267 = 0, $268 = 0, $269 = 0, $27 = 0, $270 = 0, $271 = 0, $272 = 0, $273 = 0, $274 = 0, $275 = 0, $276 = 0, $277 = 0, $278 = 0;
 var $279 = 0, $28 = 0, $280 = 0, $281 = 0, $282 = 0, $283 = 0, $284 = 0, $285 = 0, $286 = 0, $287 = 0, $288 = 0, $289 = 0, $29 = 0, $290 = 0, $291 = 0, $292 = 0, $293 = 0, $294 = 0, $295 = 0, $296 = 0;
 var $297 = 0, $298 = 0, $299 = 0, $3 = 0, $30 = 0, $300 = 0, $301 = 0, $302 = 0, $303 = 0, $304 = 0, $305 = 0, $306 = 0, $307 = 0, $308 = 0, $309 = 0, $31 = 0, $310 = 0, $311 = 0, $312 = 0, $313 = 0;
 var $314 = 0, $315 = 0, $316 = 0, $317 = 0, $318 = 0, $319 = 0, $32 = 0, $320 = 0, $321 = 0, $322 = 0, $323 = 0, $324 = 0, $325 = 0, $326 = 0, $327 = 0, $328 = 0, $329 = 0, $33 = 0, $330 = 0, $331 = 0;
 var $332 = 0, $333 = 0, $334 = 0, $335 = 0, $336 = 0, $337 = 0, $338 = 0, $339 = 0, $34 = 0, $340 = 0, $341 = 0, $342 = 0, $343 = 0, $344 = 0, $345 = 0, $346 = 0, $347 = 0, $348 = 0, $349 = 0, $35 = 0;
 var $350 = 0, $351 = 0, $352 = 0, $353 = 0, $354 = 0, $355 = 0, $356 = 0, $357 = 0, $358 = 0, $359 = 0, $36 = 0, $360 = 0, $361 = 0, $362 = 0, $363 = 0, $364 = 0, $365 = 0, $366 = 0, $367 = 0, $368 = 0;
 var $369 = 0, $37 = 0, $370 = 0, $371 = 0, $372 = 0, $373 = 0, $374 = 0, $375 = 0, $376 = 0, $377 = 0, $378 = 0, $379 = 0, $38 = 0, $380 = 0, $381 = 0, $382 = 0, $383 = 0, $384 = 0, $385 = 0, $386 = 0;
 var $387 = 0, $388 = 0, $389 = 0, $39 = 0, $390 = 0, $391 = 0, $392 = 0, $393 = 0, $394 = 0, $395 = 0, $396 = 0, $397 = 0, $398 = 0, $399 = 0, $4 = 0, $40 = 0, $400 = 0, $401 = 0, $402 = 0, $403 = 0;
 var $404 = 0, $405 = 0, $406 = 0, $407 = 0, $408 = 0, $409 = 0, $41 = 0, $410 = 0, $411 = 0, $412 = 0, $413 = 0, $414 = 0, $415 = 0, $416 = 0, $417 = 0, $418 = 0, $419 = 0, $42 = 0, $420 = 0, $421 = 0;
 var $422 = 0, $423 = 0, $424 = 0, $425 = 0, $426 = 0, $427 = 0, $428 = 0, $429 = 0, $43 = 0, $430 = 0, $431 = 0, $432 = 0, $433 = 0, $434 = 0, $435 = 0, $436 = 0, $437 = 0, $438 = 0, $439 = 0, $44 = 0;
 var $440 = 0, $441 = 0, $442 = 0, $443 = 0, $444 = 0, $445 = 0, $446 = 0, $447 = 0, $448 = 0, $449 = 0, $45 = 0, $450 = 0, $451 = 0, $452 = 0, $453 = 0, $454 = 0, $455 = 0, $456 = 0, $457 = 0, $458 = 0;
 var $459 = 0, $46 = 0, $460 = 0, $461 = 0, $462 = 0, $463 = 0, $464 = 0, $465 = 0, $466 = 0, $467 = 0, $468 = 0, $469 = 0, $47 = 0, $470 = 0, $471 = 0, $472 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0;
 var $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0;
 var $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0;
 var $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $aToOpen = 0, $addrBypass = 0, $addrDelete = 0, $addrEphOpen = 0, $addrLoop = 0, $aiCurOnePass = 0, $count = 0;
 var $db = 0, $i = 0, $iDataCur = 0, $iDb = 0, $iEphCur = 0, $iIdxCur = 0, $iKey = 0, $iPk = 0, $iRowSet = 0, $iTabCur = 0, $isView = 0, $memCnt = 0, $nIdx = 0, $nKey = 0, $nPk = 0, $okOnePass = 0, $or$cond = 0, $or$cond$not = 0, $or$cond3 = 0, $pIdx = 0;
 var $pPk = 0, $pTab = 0, $pTrigger = 0, $pVTab = 0, $pWInfo = 0, $rcauth = 0, $sContext = 0, $sNC = 0, $v = 0, $zDb = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 192|0;
 $0 = sp + 172|0;
 $1 = sp + 168|0;
 $2 = sp + 164|0;
 $v = sp + 160|0;
 $pTab = sp + 156|0;
 $zDb = sp + 152|0;
 $i = sp + 148|0;
 $pWInfo = sp + 144|0;
 $pIdx = sp + 140|0;
 $iTabCur = sp + 136|0;
 $iDataCur = sp + 132|0;
 $iIdxCur = sp + 128|0;
 $nIdx = sp + 124|0;
 $db = sp + 120|0;
 $sContext = sp + 112|0;
 $sNC = sp + 80|0;
 $iDb = sp + 76|0;
 $memCnt = sp + 72|0;
 $rcauth = sp + 68|0;
 $okOnePass = sp + 64|0;
 $aiCurOnePass = sp + 56|0;
 $aToOpen = sp + 52|0;
 $pPk = sp + 48|0;
 $iPk = sp + 44|0;
 $nPk = sp + 178|0;
 $iKey = sp + 40|0;
 $nKey = sp + 176|0;
 $iEphCur = sp + 36|0;
 $iRowSet = sp + 32|0;
 $addrBypass = sp + 28|0;
 $addrLoop = sp + 24|0;
 $addrDelete = sp + 20|0;
 $addrEphOpen = sp + 16|0;
 $isView = sp + 12|0;
 $pTrigger = sp + 8|0;
 $pVTab = sp + 4|0;
 $count = sp;
 HEAP32[$0>>2] = $pParse;
 HEAP32[$1>>2] = $pTabList;
 HEAP32[$2>>2] = $pWhere;
 HEAP32[$memCnt>>2] = -1;
 HEAP32[$aToOpen>>2] = 0;
 HEAP32[$iPk>>2] = 0;
 HEAP16[$nPk>>1] = 1;
 HEAP32[$iEphCur>>2] = 0;
 HEAP32[$iRowSet>>2] = 0;
 HEAP32[$addrBypass>>2] = 0;
 HEAP32[$addrLoop>>2] = 0;
 HEAP32[$addrDelete>>2] = 0;
 HEAP32[$addrEphOpen>>2] = 0;
 ;HEAP32[$sContext+0>>2]=0|0;HEAP32[$sContext+4>>2]=0|0;
 $3 = HEAP32[$0>>2]|0;
 $4 = HEAP32[$3>>2]|0;
 HEAP32[$db>>2] = $4;
 $5 = HEAP32[$0>>2]|0;
 $6 = (($5) + 64|0);
 $7 = HEAP32[$6>>2]|0;
 $8 = ($7|0)!=(0);
 L1: do {
  if (!($8)) {
   $9 = HEAP32[$db>>2]|0;
   $10 = (($9) + 64|0);
   $11 = HEAP8[$10>>0]|0;
   $12 = $11&255;
   $13 = ($12|0)!=(0);
   if (!($13)) {
    $14 = HEAP32[$0>>2]|0;
    $15 = HEAP32[$1>>2]|0;
    $16 = (_sqlite3SrcListLookup($14,$15)|0);
    HEAP32[$pTab>>2] = $16;
    $17 = HEAP32[$pTab>>2]|0;
    $18 = ($17|0)==(0|0);
    if (!($18)) {
     $19 = HEAP32[$0>>2]|0;
     $20 = HEAP32[$pTab>>2]|0;
     $21 = (_sqlite3TriggersExist($19,$20,109,0,0)|0);
     HEAP32[$pTrigger>>2] = $21;
     $22 = HEAP32[$pTab>>2]|0;
     $23 = (($22) + 12|0);
     $24 = HEAP32[$23>>2]|0;
     $25 = ($24|0)!=(0|0);
     $26 = $25&1;
     HEAP32[$isView>>2] = $26;
     $27 = HEAP32[$0>>2]|0;
     $28 = HEAP32[$pTab>>2]|0;
     $29 = (_sqlite3ViewGetColumnNames($27,$28)|0);
     $30 = ($29|0)!=(0);
     if (!($30)) {
      $31 = HEAP32[$0>>2]|0;
      $32 = HEAP32[$pTab>>2]|0;
      $33 = HEAP32[$pTrigger>>2]|0;
      $34 = ($33|0)!=(0|0);
      $35 = $34 ? 1 : 0;
      $36 = (_sqlite3IsReadOnly($31,$32,$35)|0);
      $37 = ($36|0)!=(0);
      if (!($37)) {
       $38 = HEAP32[$db>>2]|0;
       $39 = HEAP32[$pTab>>2]|0;
       $40 = (($39) + 68|0);
       $41 = HEAP32[$40>>2]|0;
       $42 = (_sqlite3SchemaToIndex($38,$41)|0);
       HEAP32[$iDb>>2] = $42;
       $43 = HEAP32[$iDb>>2]|0;
       $44 = HEAP32[$db>>2]|0;
       $45 = (($44) + 16|0);
       $46 = HEAP32[$45>>2]|0;
       $47 = (($46) + ($43<<4)|0);
       $48 = HEAP32[$47>>2]|0;
       HEAP32[$zDb>>2] = $48;
       $49 = HEAP32[$0>>2]|0;
       $50 = HEAP32[$pTab>>2]|0;
       $51 = HEAP32[$50>>2]|0;
       $52 = HEAP32[$zDb>>2]|0;
       $53 = (_sqlite3AuthCheck($49,9,$51,0,$52)|0);
       HEAP32[$rcauth>>2] = $53;
       $54 = HEAP32[$rcauth>>2]|0;
       $55 = ($54|0)==(1);
       if (!($55)) {
        $56 = HEAP32[$0>>2]|0;
        $57 = (($56) + 68|0);
        $58 = HEAP32[$57>>2]|0;
        $59 = (($58) + 1)|0;
        HEAP32[$57>>2] = $59;
        $60 = HEAP32[$1>>2]|0;
        $61 = (($60) + 8|0);
        $62 = (($61) + 40|0);
        HEAP32[$62>>2] = $58;
        HEAP32[$iTabCur>>2] = $58;
        HEAP32[$nIdx>>2] = 0;
        $63 = HEAP32[$pTab>>2]|0;
        $64 = (($63) + 8|0);
        $65 = HEAP32[$64>>2]|0;
        HEAP32[$pIdx>>2] = $65;
        while(1) {
         $66 = HEAP32[$pIdx>>2]|0;
         $67 = ($66|0)!=(0|0);
         if (!($67)) {
          break;
         }
         $68 = HEAP32[$0>>2]|0;
         $69 = (($68) + 68|0);
         $70 = HEAP32[$69>>2]|0;
         $71 = (($70) + 1)|0;
         HEAP32[$69>>2] = $71;
         $72 = HEAP32[$pIdx>>2]|0;
         $73 = (($72) + 20|0);
         $74 = HEAP32[$73>>2]|0;
         HEAP32[$pIdx>>2] = $74;
         $75 = HEAP32[$nIdx>>2]|0;
         $76 = (($75) + 1)|0;
         HEAP32[$nIdx>>2] = $76;
        }
        $77 = HEAP32[$isView>>2]|0;
        $78 = ($77|0)!=(0);
        if ($78) {
         $79 = HEAP32[$0>>2]|0;
         $80 = HEAP32[$pTab>>2]|0;
         $81 = HEAP32[$80>>2]|0;
         _sqlite3AuthContextPush($79,$sContext,$81);
        }
        $82 = HEAP32[$0>>2]|0;
        $83 = (_sqlite3GetVdbe($82)|0);
        HEAP32[$v>>2] = $83;
        $84 = HEAP32[$v>>2]|0;
        $85 = ($84|0)==(0|0);
        if (!($85)) {
         $86 = HEAP32[$0>>2]|0;
         $87 = (($86) + 18|0);
         $88 = HEAP8[$87>>0]|0;
         $89 = $88&255;
         $90 = ($89|0)==(0);
         if ($90) {
          $91 = HEAP32[$v>>2]|0;
          _sqlite3VdbeCountChanges($91);
         }
         $92 = HEAP32[$0>>2]|0;
         $93 = HEAP32[$iDb>>2]|0;
         _sqlite3BeginWriteOperation($92,1,$93);
         $94 = HEAP32[$isView>>2]|0;
         $95 = ($94|0)!=(0);
         if ($95) {
          $96 = HEAP32[$0>>2]|0;
          $97 = HEAP32[$pTab>>2]|0;
          $98 = HEAP32[$2>>2]|0;
          $99 = HEAP32[$iTabCur>>2]|0;
          _sqlite3MaterializeView($96,$97,$98,$99);
          $100 = HEAP32[$iTabCur>>2]|0;
          HEAP32[$iIdxCur>>2] = $100;
          HEAP32[$iDataCur>>2] = $100;
         }
         ;HEAP32[$sNC+0>>2]=0|0;HEAP32[$sNC+4>>2]=0|0;HEAP32[$sNC+8>>2]=0|0;HEAP32[$sNC+12>>2]=0|0;HEAP32[$sNC+16>>2]=0|0;HEAP32[$sNC+20>>2]=0|0;HEAP32[$sNC+24>>2]=0|0;HEAP32[$sNC+28>>2]=0|0;
         $101 = HEAP32[$0>>2]|0;
         HEAP32[$sNC>>2] = $101;
         $102 = HEAP32[$1>>2]|0;
         $103 = (($sNC) + 4|0);
         HEAP32[$103>>2] = $102;
         $104 = HEAP32[$2>>2]|0;
         $105 = (_sqlite3ResolveExprNames($sNC,$104)|0);
         $106 = ($105|0)!=(0);
         if (!($106)) {
          $107 = HEAP32[$db>>2]|0;
          $108 = (($107) + 24|0);
          $109 = HEAP32[$108>>2]|0;
          $110 = $109 & 128;
          $111 = ($110|0)!=(0);
          if ($111) {
           $112 = HEAP32[$0>>2]|0;
           $113 = (($112) + 72|0);
           $114 = HEAP32[$113>>2]|0;
           $115 = (($114) + 1)|0;
           HEAP32[$113>>2] = $115;
           HEAP32[$memCnt>>2] = $115;
           $116 = HEAP32[$v>>2]|0;
           $117 = HEAP32[$memCnt>>2]|0;
           (_sqlite3VdbeAddOp2($116,25,0,$117)|0);
          }
          $118 = HEAP32[$rcauth>>2]|0;
          $119 = ($118|0)==(0);
          $120 = HEAP32[$2>>2]|0;
          $121 = ($120|0)==(0|0);
          $or$cond = $119 & $121;
          $or$cond$not = $or$cond ^ 1;
          $122 = HEAP32[$pTrigger>>2]|0;
          $123 = ($122|0)!=(0|0);
          $or$cond3 = $or$cond$not | $123;
          L26: do {
           if ($or$cond3) {
            label = 28;
           } else {
            $124 = HEAP32[$pTab>>2]|0;
            $125 = (($124) + 44|0);
            $126 = HEAP8[$125>>0]|0;
            $127 = $126&255;
            $128 = $127 & 16;
            $129 = ($128|0)!=(0);
            if ($129) {
             label = 28;
            } else {
             $130 = HEAP32[$0>>2]|0;
             $131 = HEAP32[$pTab>>2]|0;
             $132 = (_sqlite3FkRequired($130,$131,0,0)|0);
             $133 = (0)==($132|0);
             if ($133) {
              $134 = HEAP32[$0>>2]|0;
              $135 = HEAP32[$iDb>>2]|0;
              $136 = HEAP32[$pTab>>2]|0;
              $137 = (($136) + 32|0);
              $138 = HEAP32[$137>>2]|0;
              $139 = HEAP32[$pTab>>2]|0;
              $140 = HEAP32[$139>>2]|0;
              _sqlite3TableLock($134,$135,$138,1,$140);
              $141 = HEAP32[$pTab>>2]|0;
              $142 = (($141) + 44|0);
              $143 = HEAP8[$142>>0]|0;
              $144 = $143&255;
              $145 = $144 & 32;
              $146 = ($145|0)==(0);
              if ($146) {
               $147 = HEAP32[$v>>2]|0;
               $148 = HEAP32[$pTab>>2]|0;
               $149 = (($148) + 32|0);
               $150 = HEAP32[$149>>2]|0;
               $151 = HEAP32[$iDb>>2]|0;
               $152 = HEAP32[$memCnt>>2]|0;
               $153 = HEAP32[$pTab>>2]|0;
               $154 = HEAP32[$153>>2]|0;
               (_sqlite3VdbeAddOp4($147,118,$150,$151,$152,$154,-2)|0);
              }
              $155 = HEAP32[$pTab>>2]|0;
              $156 = (($155) + 8|0);
              $157 = HEAP32[$156>>2]|0;
              HEAP32[$pIdx>>2] = $157;
              while(1) {
               $158 = HEAP32[$pIdx>>2]|0;
               $159 = ($158|0)!=(0|0);
               if (!($159)) {
                break L26;
               }
               $160 = HEAP32[$v>>2]|0;
               $161 = HEAP32[$pIdx>>2]|0;
               $162 = (($161) + 44|0);
               $163 = HEAP32[$162>>2]|0;
               $164 = HEAP32[$iDb>>2]|0;
               (_sqlite3VdbeAddOp2($160,118,$163,$164)|0);
               $165 = HEAP32[$pIdx>>2]|0;
               $166 = (($165) + 20|0);
               $167 = HEAP32[$166>>2]|0;
               HEAP32[$pIdx>>2] = $167;
              }
             } else {
              label = 28;
             }
            }
           }
          } while(0);
          L36: do {
           if ((label|0) == 28) {
            $168 = HEAP32[$pTab>>2]|0;
            $169 = (($168) + 44|0);
            $170 = HEAP8[$169>>0]|0;
            $171 = $170&255;
            $172 = $171 & 32;
            $173 = ($172|0)==(0);
            if ($173) {
             HEAP32[$pPk>>2] = 0;
             HEAP16[$nPk>>1] = 1;
             $174 = HEAP32[$0>>2]|0;
             $175 = (($174) + 72|0);
             $176 = HEAP32[$175>>2]|0;
             $177 = (($176) + 1)|0;
             HEAP32[$175>>2] = $177;
             HEAP32[$iRowSet>>2] = $177;
             $178 = HEAP32[$v>>2]|0;
             $179 = HEAP32[$iRowSet>>2]|0;
             (_sqlite3VdbeAddOp2($178,28,0,$179)|0);
            } else {
             $180 = HEAP32[$pTab>>2]|0;
             $181 = (_sqlite3PrimaryKeyIndex($180)|0);
             HEAP32[$pPk>>2] = $181;
             $182 = HEAP32[$pPk>>2]|0;
             $183 = (($182) + 50|0);
             $184 = HEAP16[$183>>1]|0;
             HEAP16[$nPk>>1] = $184;
             $185 = HEAP32[$0>>2]|0;
             $186 = (($185) + 72|0);
             $187 = HEAP32[$186>>2]|0;
             $188 = (($187) + 1)|0;
             HEAP32[$iPk>>2] = $188;
             $189 = HEAP16[$nPk>>1]|0;
             $190 = $189 << 16 >> 16;
             $191 = HEAP32[$0>>2]|0;
             $192 = (($191) + 72|0);
             $193 = HEAP32[$192>>2]|0;
             $194 = (($193) + ($190))|0;
             HEAP32[$192>>2] = $194;
             $195 = HEAP32[$0>>2]|0;
             $196 = (($195) + 68|0);
             $197 = HEAP32[$196>>2]|0;
             $198 = (($197) + 1)|0;
             HEAP32[$196>>2] = $198;
             HEAP32[$iEphCur>>2] = $197;
             $199 = HEAP32[$v>>2]|0;
             $200 = HEAP32[$iEphCur>>2]|0;
             $201 = HEAP16[$nPk>>1]|0;
             $202 = $201 << 16 >> 16;
             $203 = (_sqlite3VdbeAddOp2($199,57,$200,$202)|0);
             HEAP32[$addrEphOpen>>2] = $203;
             $204 = HEAP32[$0>>2]|0;
             $205 = HEAP32[$pPk>>2]|0;
             _sqlite3VdbeSetP4KeyInfo($204,$205);
            }
            $206 = HEAP32[$0>>2]|0;
            $207 = HEAP32[$1>>2]|0;
            $208 = HEAP32[$2>>2]|0;
            $209 = HEAP32[$iTabCur>>2]|0;
            $210 = (($209) + 1)|0;
            $211 = (_sqlite3WhereBegin($206,$207,$208,0,0,12,$210)|0);
            HEAP32[$pWInfo>>2] = $211;
            $212 = HEAP32[$pWInfo>>2]|0;
            $213 = ($212|0)==(0|0);
            if ($213) {
             break L1;
            }
            $214 = HEAP32[$pWInfo>>2]|0;
            $215 = (_sqlite3WhereOkOnePass($214,$aiCurOnePass)|0);
            HEAP32[$okOnePass>>2] = $215;
            $216 = HEAP32[$db>>2]|0;
            $217 = (($216) + 24|0);
            $218 = HEAP32[$217>>2]|0;
            $219 = $218 & 128;
            $220 = ($219|0)!=(0);
            if ($220) {
             $221 = HEAP32[$v>>2]|0;
             $222 = HEAP32[$memCnt>>2]|0;
             (_sqlite3VdbeAddOp2($221,37,$222,1)|0);
            }
            $223 = HEAP32[$pPk>>2]|0;
            $224 = ($223|0)!=(0|0);
            do {
             if ($224) {
              HEAP32[$i>>2] = 0;
              while(1) {
               $225 = HEAP32[$i>>2]|0;
               $226 = HEAP16[$nPk>>1]|0;
               $227 = $226 << 16 >> 16;
               $228 = ($225|0)<($227|0);
               if (!($228)) {
                break;
               }
               $229 = HEAP32[$v>>2]|0;
               $230 = HEAP32[$pTab>>2]|0;
               $231 = HEAP32[$iTabCur>>2]|0;
               $232 = HEAP32[$i>>2]|0;
               $233 = HEAP32[$pPk>>2]|0;
               $234 = (($233) + 4|0);
               $235 = HEAP32[$234>>2]|0;
               $236 = (($235) + ($232<<1)|0);
               $237 = HEAP16[$236>>1]|0;
               $238 = $237 << 16 >> 16;
               $239 = HEAP32[$iPk>>2]|0;
               $240 = HEAP32[$i>>2]|0;
               $241 = (($239) + ($240))|0;
               _sqlite3ExprCodeGetColumnOfTable($229,$230,$231,$238,$241);
               $242 = HEAP32[$i>>2]|0;
               $243 = (($242) + 1)|0;
               HEAP32[$i>>2] = $243;
              }
              $244 = HEAP32[$iPk>>2]|0;
              HEAP32[$iKey>>2] = $244;
             } else {
              $245 = HEAP32[$0>>2]|0;
              $246 = (($245) + 72|0);
              $247 = HEAP32[$246>>2]|0;
              $248 = (($247) + 1)|0;
              HEAP32[$iKey>>2] = $248;
              $249 = HEAP32[$0>>2]|0;
              $250 = HEAP32[$pTab>>2]|0;
              $251 = HEAP32[$iTabCur>>2]|0;
              $252 = HEAP32[$iKey>>2]|0;
              $253 = (_sqlite3ExprCodeGetColumn($249,$250,-1,$251,$252,0)|0);
              HEAP32[$iKey>>2] = $253;
              $254 = HEAP32[$iKey>>2]|0;
              $255 = HEAP32[$0>>2]|0;
              $256 = (($255) + 72|0);
              $257 = HEAP32[$256>>2]|0;
              $258 = ($254|0)>($257|0);
              if (!($258)) {
               break;
              }
              $259 = HEAP32[$iKey>>2]|0;
              $260 = HEAP32[$0>>2]|0;
              $261 = (($260) + 72|0);
              HEAP32[$261>>2] = $259;
             }
            } while(0);
            $262 = HEAP32[$okOnePass>>2]|0;
            $263 = ($262|0)!=(0);
            do {
             if ($263) {
              $264 = HEAP16[$nPk>>1]|0;
              HEAP16[$nKey>>1] = $264;
              $265 = HEAP32[$db>>2]|0;
              $266 = HEAP32[$nIdx>>2]|0;
              $267 = (($266) + 2)|0;
              $268 = ($267|0)<(0);
              $269 = $268 << 31 >> 31;
              $270 = (_sqlite3DbMallocRaw($265,$267,$269)|0);
              HEAP32[$aToOpen>>2] = $270;
              $271 = HEAP32[$aToOpen>>2]|0;
              $272 = ($271|0)==(0|0);
              if ($272) {
               $273 = HEAP32[$pWInfo>>2]|0;
               _sqlite3WhereEnd($273);
               break L1;
              }
              $274 = HEAP32[$aToOpen>>2]|0;
              $275 = HEAP32[$nIdx>>2]|0;
              $276 = (($275) + 1)|0;
              _memset(($274|0),1,($276|0))|0;
              $277 = HEAP32[$nIdx>>2]|0;
              $278 = (($277) + 1)|0;
              $279 = HEAP32[$aToOpen>>2]|0;
              $280 = (($279) + ($278)|0);
              HEAP8[$280>>0] = 0;
              $281 = HEAP32[$aiCurOnePass>>2]|0;
              $282 = ($281|0)>=(0);
              if ($282) {
               $283 = HEAP32[$aiCurOnePass>>2]|0;
               $284 = HEAP32[$iTabCur>>2]|0;
               $285 = (($283) - ($284))|0;
               $286 = HEAP32[$aToOpen>>2]|0;
               $287 = (($286) + ($285)|0);
               HEAP8[$287>>0] = 0;
              }
              $288 = (($aiCurOnePass) + 4|0);
              $289 = HEAP32[$288>>2]|0;
              $290 = ($289|0)>=(0);
              if ($290) {
               $291 = (($aiCurOnePass) + 4|0);
               $292 = HEAP32[$291>>2]|0;
               $293 = HEAP32[$iTabCur>>2]|0;
               $294 = (($292) - ($293))|0;
               $295 = HEAP32[$aToOpen>>2]|0;
               $296 = (($295) + ($294)|0);
               HEAP8[$296>>0] = 0;
              }
              $297 = HEAP32[$addrEphOpen>>2]|0;
              $298 = ($297|0)!=(0);
              if ($298) {
               $299 = HEAP32[$v>>2]|0;
               $300 = HEAP32[$addrEphOpen>>2]|0;
               _sqlite3VdbeChangeToNoop($299,$300);
              }
              $301 = HEAP32[$v>>2]|0;
              $302 = (_sqlite3VdbeAddOp0($301,16)|0);
              HEAP32[$addrDelete>>2] = $302;
             } else {
              $303 = HEAP32[$pPk>>2]|0;
              $304 = ($303|0)!=(0|0);
              if ($304) {
               $305 = HEAP32[$0>>2]|0;
               $306 = (($305) + 72|0);
               $307 = HEAP32[$306>>2]|0;
               $308 = (($307) + 1)|0;
               HEAP32[$306>>2] = $308;
               HEAP32[$iKey>>2] = $308;
               HEAP16[$nKey>>1] = 0;
               $309 = HEAP32[$v>>2]|0;
               $310 = HEAP32[$iPk>>2]|0;
               $311 = HEAP16[$nPk>>1]|0;
               $312 = $311 << 16 >> 16;
               $313 = HEAP32[$iKey>>2]|0;
               $314 = HEAP32[$v>>2]|0;
               $315 = HEAP32[$pPk>>2]|0;
               $316 = (_sqlite3IndexAffinityStr($314,$315)|0);
               $317 = HEAP16[$nPk>>1]|0;
               $318 = $317 << 16 >> 16;
               (_sqlite3VdbeAddOp4($309,49,$310,$312,$313,$316,$318)|0);
               $319 = HEAP32[$v>>2]|0;
               $320 = HEAP32[$iEphCur>>2]|0;
               $321 = HEAP32[$iKey>>2]|0;
               (_sqlite3VdbeAddOp2($319,110,$320,$321)|0);
               break;
              } else {
               HEAP16[$nKey>>1] = 1;
               $322 = HEAP32[$v>>2]|0;
               $323 = HEAP32[$iRowSet>>2]|0;
               $324 = HEAP32[$iKey>>2]|0;
               (_sqlite3VdbeAddOp2($322,128,$323,$324)|0);
               break;
              }
             }
            } while(0);
            $325 = HEAP32[$pWInfo>>2]|0;
            _sqlite3WhereEnd($325);
            $326 = HEAP32[$okOnePass>>2]|0;
            $327 = ($326|0)!=(0);
            if ($327) {
             $328 = HEAP32[$v>>2]|0;
             $329 = (_sqlite3VdbeMakeLabel($328)|0);
             HEAP32[$addrBypass>>2] = $329;
             $330 = HEAP32[$v>>2]|0;
             $331 = HEAP32[$addrBypass>>2]|0;
             (_sqlite3VdbeAddOp2($330,16,0,$331)|0);
             $332 = HEAP32[$v>>2]|0;
             $333 = HEAP32[$addrDelete>>2]|0;
             _sqlite3VdbeJumpHere($332,$333);
            }
            $334 = HEAP32[$isView>>2]|0;
            $335 = ($334|0)!=(0);
            if (!($335)) {
             $336 = HEAP32[$0>>2]|0;
             $337 = HEAP32[$pTab>>2]|0;
             $338 = HEAP32[$iTabCur>>2]|0;
             $339 = HEAP32[$aToOpen>>2]|0;
             (_sqlite3OpenTableAndIndices($336,$337,55,$338,$339,$iDataCur,$iIdxCur)|0);
            }
            $340 = HEAP32[$okOnePass>>2]|0;
            $341 = ($340|0)!=(0);
            do {
             if ($341) {
              $342 = HEAP32[$iDataCur>>2]|0;
              $343 = HEAP32[$iTabCur>>2]|0;
              $344 = (($342) - ($343))|0;
              $345 = HEAP32[$aToOpen>>2]|0;
              $346 = (($345) + ($344)|0);
              $347 = HEAP8[$346>>0]|0;
              $348 = ($347<<24>>24)!=(0);
              if (!($348)) {
               break;
              }
              $349 = HEAP32[$v>>2]|0;
              $350 = HEAP32[$iDataCur>>2]|0;
              $351 = HEAP32[$addrBypass>>2]|0;
              $352 = HEAP32[$iKey>>2]|0;
              $353 = HEAP16[$nKey>>1]|0;
              $354 = $353 << 16 >> 16;
              (_sqlite3VdbeAddOp4Int($349,68,$350,$351,$352,$354)|0);
             } else {
              $355 = HEAP32[$pPk>>2]|0;
              $356 = ($355|0)!=(0|0);
              $357 = HEAP32[$v>>2]|0;
              if ($356) {
               $358 = HEAP32[$iEphCur>>2]|0;
               $359 = (_sqlite3VdbeAddOp1($357,108,$358)|0);
               HEAP32[$addrLoop>>2] = $359;
               $360 = HEAP32[$v>>2]|0;
               $361 = HEAP32[$iEphCur>>2]|0;
               $362 = HEAP32[$iKey>>2]|0;
               (_sqlite3VdbeAddOp2($360,101,$361,$362)|0);
               break;
              } else {
               $363 = HEAP32[$iRowSet>>2]|0;
               $364 = HEAP32[$iKey>>2]|0;
               $365 = (_sqlite3VdbeAddOp3($357,129,$363,0,$364)|0);
               HEAP32[$addrLoop>>2] = $365;
               break;
              }
             }
            } while(0);
            $366 = HEAP32[$pTab>>2]|0;
            $367 = (($366) + 44|0);
            $368 = HEAP8[$367>>0]|0;
            $369 = $368&255;
            $370 = $369 & 16;
            $371 = ($370|0)!=(0);
            if ($371) {
             $372 = HEAP32[$db>>2]|0;
             $373 = HEAP32[$pTab>>2]|0;
             $374 = (_sqlite3GetVTable($372,$373)|0);
             HEAP32[$pVTab>>2] = $374;
             $375 = HEAP32[$0>>2]|0;
             $376 = HEAP32[$pTab>>2]|0;
             _sqlite3VtabMakeWritable($375,$376);
             $377 = HEAP32[$v>>2]|0;
             $378 = HEAP32[$iKey>>2]|0;
             $379 = HEAP32[$pVTab>>2]|0;
             (_sqlite3VdbeAddOp4($377,15,0,1,$378,$379,-10)|0);
             $380 = HEAP32[$v>>2]|0;
             _sqlite3VdbeChangeP5($380,2);
             $381 = HEAP32[$0>>2]|0;
             _sqlite3MayAbort($381);
            } else {
             $382 = HEAP32[$0>>2]|0;
             $383 = (($382) + 18|0);
             $384 = HEAP8[$383>>0]|0;
             $385 = $384&255;
             $386 = ($385|0)==(0);
             $387 = $386&1;
             HEAP32[$count>>2] = $387;
             $388 = HEAP32[$0>>2]|0;
             $389 = HEAP32[$pTab>>2]|0;
             $390 = HEAP32[$pTrigger>>2]|0;
             $391 = HEAP32[$iDataCur>>2]|0;
             $392 = HEAP32[$iIdxCur>>2]|0;
             $393 = HEAP32[$iKey>>2]|0;
             $394 = HEAP16[$nKey>>1]|0;
             $395 = HEAP32[$count>>2]|0;
             $396 = $395&255;
             $397 = HEAP32[$okOnePass>>2]|0;
             $398 = $397&255;
             _sqlite3GenerateRowDelete($388,$389,$390,$391,$392,$393,$394,$396,10,$398);
            }
            $399 = HEAP32[$okOnePass>>2]|0;
            $400 = ($399|0)!=(0);
            do {
             if ($400) {
              $401 = HEAP32[$v>>2]|0;
              $402 = HEAP32[$addrBypass>>2]|0;
              _sqlite3VdbeResolveLabel($401,$402);
             } else {
              $403 = HEAP32[$pPk>>2]|0;
              $404 = ($403|0)!=(0|0);
              $405 = HEAP32[$v>>2]|0;
              if ($404) {
               $406 = HEAP32[$iEphCur>>2]|0;
               $407 = HEAP32[$addrLoop>>2]|0;
               $408 = (($407) + 1)|0;
               (_sqlite3VdbeAddOp2($405,9,$406,$408)|0);
               $409 = HEAP32[$v>>2]|0;
               $410 = HEAP32[$addrLoop>>2]|0;
               _sqlite3VdbeJumpHere($409,$410);
               break;
              } else {
               $411 = HEAP32[$addrLoop>>2]|0;
               (_sqlite3VdbeAddOp2($405,16,0,$411)|0);
               $412 = HEAP32[$v>>2]|0;
               $413 = HEAP32[$addrLoop>>2]|0;
               _sqlite3VdbeJumpHere($412,$413);
               break;
              }
             }
            } while(0);
            $414 = HEAP32[$isView>>2]|0;
            $415 = ($414|0)!=(0);
            if ($415) {
             break;
            }
            $416 = HEAP32[$pTab>>2]|0;
            $417 = (($416) + 44|0);
            $418 = HEAP8[$417>>0]|0;
            $419 = $418&255;
            $420 = $419 & 16;
            $421 = ($420|0)!=(0);
            if ($421) {
             break;
            }
            $422 = HEAP32[$pPk>>2]|0;
            $423 = ($422|0)!=(0|0);
            if (!($423)) {
             $424 = HEAP32[$v>>2]|0;
             $425 = HEAP32[$iDataCur>>2]|0;
             (_sqlite3VdbeAddOp1($424,61,$425)|0);
            }
            HEAP32[$i>>2] = 0;
            $426 = HEAP32[$pTab>>2]|0;
            $427 = (($426) + 8|0);
            $428 = HEAP32[$427>>2]|0;
            HEAP32[$pIdx>>2] = $428;
            while(1) {
             $429 = HEAP32[$pIdx>>2]|0;
             $430 = ($429|0)!=(0|0);
             if (!($430)) {
              break L36;
             }
             $431 = HEAP32[$v>>2]|0;
             $432 = HEAP32[$iIdxCur>>2]|0;
             $433 = HEAP32[$i>>2]|0;
             $434 = (($432) + ($433))|0;
             (_sqlite3VdbeAddOp1($431,61,$434)|0);
             $435 = HEAP32[$i>>2]|0;
             $436 = (($435) + 1)|0;
             HEAP32[$i>>2] = $436;
             $437 = HEAP32[$pIdx>>2]|0;
             $438 = (($437) + 20|0);
             $439 = HEAP32[$438>>2]|0;
             HEAP32[$pIdx>>2] = $439;
            }
           }
          } while(0);
          $440 = HEAP32[$0>>2]|0;
          $441 = (($440) + 18|0);
          $442 = HEAP8[$441>>0]|0;
          $443 = $442&255;
          $444 = ($443|0)==(0);
          if ($444) {
           $445 = HEAP32[$0>>2]|0;
           $446 = (($445) + 412|0);
           $447 = HEAP32[$446>>2]|0;
           $448 = ($447|0)==(0|0);
           if ($448) {
            $449 = HEAP32[$0>>2]|0;
            _sqlite3AutoincrementEnd($449);
           }
          }
          $450 = HEAP32[$db>>2]|0;
          $451 = (($450) + 24|0);
          $452 = HEAP32[$451>>2]|0;
          $453 = $452 & 128;
          $454 = ($453|0)!=(0);
          if ($454) {
           $455 = HEAP32[$0>>2]|0;
           $456 = (($455) + 18|0);
           $457 = HEAP8[$456>>0]|0;
           $458 = ($457<<24>>24)!=(0);
           if (!($458)) {
            $459 = HEAP32[$0>>2]|0;
            $460 = (($459) + 412|0);
            $461 = HEAP32[$460>>2]|0;
            $462 = ($461|0)!=(0|0);
            if (!($462)) {
             $463 = HEAP32[$v>>2]|0;
             $464 = HEAP32[$memCnt>>2]|0;
             (_sqlite3VdbeAddOp2($463,35,$464,1)|0);
             $465 = HEAP32[$v>>2]|0;
             _sqlite3VdbeSetNumCols($465,1);
             $466 = HEAP32[$v>>2]|0;
             (_sqlite3VdbeSetColName($466,0,0,24592,0)|0);
            }
           }
          }
         }
        }
       }
      }
     }
    }
   }
  }
 } while(0);
 _sqlite3AuthContextPop($sContext);
 $467 = HEAP32[$db>>2]|0;
 $468 = HEAP32[$1>>2]|0;
 _sqlite3SrcListDelete($467,$468);
 $469 = HEAP32[$db>>2]|0;
 $470 = HEAP32[$2>>2]|0;
 _sqlite3ExprDelete($469,$470);
 $471 = HEAP32[$db>>2]|0;
 $472 = HEAP32[$aToOpen>>2]|0;
 _sqlite3DbFree($471,$472);
 STACKTOP = sp;return;
}
function _sqlite3ExprListCheckLength($pParse,$pEList,$zObject) {
 $pParse = $pParse|0;
 $pEList = $pEList|0;
 $zObject = $zObject|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $mx = 0, $vararg_buffer = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $vararg_buffer = sp;
 $0 = sp + 16|0;
 $1 = sp + 12|0;
 $2 = sp + 8|0;
 $mx = sp + 4|0;
 HEAP32[$0>>2] = $pParse;
 HEAP32[$1>>2] = $pEList;
 HEAP32[$2>>2] = $zObject;
 $3 = HEAP32[$0>>2]|0;
 $4 = HEAP32[$3>>2]|0;
 $5 = (($4) + 88|0);
 $6 = (($5) + 8|0);
 $7 = HEAP32[$6>>2]|0;
 HEAP32[$mx>>2] = $7;
 $8 = HEAP32[$1>>2]|0;
 $9 = ($8|0)!=(0|0);
 if (!($9)) {
  STACKTOP = sp;return;
 }
 $10 = HEAP32[$1>>2]|0;
 $11 = HEAP32[$10>>2]|0;
 $12 = HEAP32[$mx>>2]|0;
 $13 = ($11|0)>($12|0);
 if (!($13)) {
  STACKTOP = sp;return;
 }
 $14 = HEAP32[$0>>2]|0;
 $15 = HEAP32[$2>>2]|0;
 HEAP32[$vararg_buffer>>2] = $15;
 _sqlite3ErrorMsg($14,24568,$vararg_buffer);
 STACKTOP = sp;return;
}
function _sqlite3Update($pParse,$pTabList,$pChanges,$pWhere,$onError) {
 $pParse = $pParse|0;
 $pTabList = $pTabList|0;
 $pChanges = $pChanges|0;
 $pWhere = $pWhere|0;
 $onError = $onError|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0;
 var $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0, $188 = 0;
 var $189 = 0, $19 = 0, $190 = 0, $191 = 0, $192 = 0, $193 = 0, $194 = 0, $195 = 0, $196 = 0, $197 = 0, $198 = 0, $199 = 0, $2 = 0, $20 = 0, $200 = 0, $201 = 0, $202 = 0, $203 = 0, $204 = 0, $205 = 0;
 var $206 = 0, $207 = 0, $208 = 0, $209 = 0, $21 = 0, $210 = 0, $211 = 0, $212 = 0, $213 = 0, $214 = 0, $215 = 0, $216 = 0, $217 = 0, $218 = 0, $219 = 0, $22 = 0, $220 = 0, $221 = 0, $222 = 0, $223 = 0;
 var $224 = 0, $225 = 0, $226 = 0, $227 = 0, $228 = 0, $229 = 0, $23 = 0, $230 = 0, $231 = 0, $232 = 0, $233 = 0, $234 = 0, $235 = 0, $236 = 0, $237 = 0, $238 = 0, $239 = 0, $24 = 0, $240 = 0, $241 = 0;
 var $242 = 0, $243 = 0, $244 = 0, $245 = 0, $246 = 0, $247 = 0, $248 = 0, $249 = 0, $25 = 0, $250 = 0, $251 = 0, $252 = 0, $253 = 0, $254 = 0, $255 = 0, $256 = 0, $257 = 0, $258 = 0, $259 = 0, $26 = 0;
 var $260 = 0, $261 = 0, $262 = 0, $263 = 0, $264 = 0, $265 = 0, $266 = 0, $267 = 0, $268 = 0, $269 = 0, $27 = 0, $270 = 0, $271 = 0, $272 = 0, $273 = 0, $274 = 0, $275 = 0, $276 = 0, $277 = 0, $278 = 0;
 var $279 = 0, $28 = 0, $280 = 0, $281 = 0, $282 = 0, $283 = 0, $284 = 0, $285 = 0, $286 = 0, $287 = 0, $288 = 0, $289 = 0, $29 = 0, $290 = 0, $291 = 0, $292 = 0, $293 = 0, $294 = 0, $295 = 0, $296 = 0;
 var $297 = 0, $298 = 0, $299 = 0, $3 = 0, $30 = 0, $300 = 0, $301 = 0, $302 = 0, $303 = 0, $304 = 0, $305 = 0, $306 = 0, $307 = 0, $308 = 0, $309 = 0, $31 = 0, $310 = 0, $311 = 0, $312 = 0, $313 = 0;
 var $314 = 0, $315 = 0, $316 = 0, $317 = 0, $318 = 0, $319 = 0, $32 = 0, $320 = 0, $321 = 0, $322 = 0, $323 = 0, $324 = 0, $325 = 0, $326 = 0, $327 = 0, $328 = 0, $329 = 0, $33 = 0, $330 = 0, $331 = 0;
 var $332 = 0, $333 = 0, $334 = 0, $335 = 0, $336 = 0, $337 = 0, $338 = 0, $339 = 0, $34 = 0, $340 = 0, $341 = 0, $342 = 0, $343 = 0, $344 = 0, $345 = 0, $346 = 0, $347 = 0, $348 = 0, $349 = 0, $35 = 0;
 var $350 = 0, $351 = 0, $352 = 0, $353 = 0, $354 = 0, $355 = 0, $356 = 0, $357 = 0, $358 = 0, $359 = 0, $36 = 0, $360 = 0, $361 = 0, $362 = 0, $363 = 0, $364 = 0, $365 = 0, $366 = 0, $367 = 0, $368 = 0;
 var $369 = 0, $37 = 0, $370 = 0, $371 = 0, $372 = 0, $373 = 0, $374 = 0, $375 = 0, $376 = 0, $377 = 0, $378 = 0, $379 = 0, $38 = 0, $380 = 0, $381 = 0, $382 = 0, $383 = 0, $384 = 0, $385 = 0, $386 = 0;
 var $387 = 0, $388 = 0, $389 = 0, $39 = 0, $390 = 0, $391 = 0, $392 = 0, $393 = 0, $394 = 0, $395 = 0, $396 = 0, $397 = 0, $398 = 0, $399 = 0, $4 = 0, $40 = 0, $400 = 0, $401 = 0, $402 = 0, $403 = 0;
 var $404 = 0, $405 = 0, $406 = 0, $407 = 0, $408 = 0, $409 = 0, $41 = 0, $410 = 0, $411 = 0, $412 = 0, $413 = 0, $414 = 0, $415 = 0, $416 = 0, $417 = 0, $418 = 0, $419 = 0, $42 = 0, $420 = 0, $421 = 0;
 var $422 = 0, $423 = 0, $424 = 0, $425 = 0, $426 = 0, $427 = 0, $428 = 0, $429 = 0, $43 = 0, $430 = 0, $431 = 0, $432 = 0, $433 = 0, $434 = 0, $435 = 0, $436 = 0, $437 = 0, $438 = 0, $439 = 0, $44 = 0;
 var $440 = 0, $441 = 0, $442 = 0, $443 = 0, $444 = 0, $445 = 0, $446 = 0, $447 = 0, $448 = 0, $449 = 0, $45 = 0, $450 = 0, $451 = 0, $452 = 0, $453 = 0, $454 = 0, $455 = 0, $456 = 0, $457 = 0, $458 = 0;
 var $459 = 0, $46 = 0, $460 = 0, $461 = 0, $462 = 0, $463 = 0, $464 = 0, $465 = 0, $466 = 0, $467 = 0, $468 = 0, $469 = 0, $47 = 0, $470 = 0, $471 = 0, $472 = 0, $473 = 0, $474 = 0, $475 = 0, $476 = 0;
 var $477 = 0, $478 = 0, $479 = 0, $48 = 0, $480 = 0, $481 = 0, $482 = 0, $483 = 0, $484 = 0, $485 = 0, $486 = 0, $487 = 0, $488 = 0, $489 = 0, $49 = 0, $490 = 0, $491 = 0, $492 = 0, $493 = 0, $494 = 0;
 var $495 = 0, $496 = 0, $497 = 0, $498 = 0, $499 = 0, $5 = 0, $50 = 0, $500 = 0, $501 = 0, $502 = 0, $503 = 0, $504 = 0, $505 = 0, $506 = 0, $507 = 0, $508 = 0, $509 = 0, $51 = 0, $510 = 0, $511 = 0;
 var $512 = 0, $513 = 0, $514 = 0, $515 = 0, $516 = 0, $517 = 0, $518 = 0, $519 = 0, $52 = 0, $520 = 0, $521 = 0, $522 = 0, $523 = 0, $524 = 0, $525 = 0, $526 = 0, $527 = 0, $528 = 0, $529 = 0, $53 = 0;
 var $530 = 0, $531 = 0, $532 = 0, $533 = 0, $534 = 0, $535 = 0, $536 = 0, $537 = 0, $538 = 0, $539 = 0, $54 = 0, $540 = 0, $541 = 0, $542 = 0, $543 = 0, $544 = 0, $545 = 0, $546 = 0, $547 = 0, $548 = 0;
 var $549 = 0, $55 = 0, $550 = 0, $551 = 0, $552 = 0, $553 = 0, $554 = 0, $555 = 0, $556 = 0, $557 = 0, $558 = 0, $559 = 0, $56 = 0, $560 = 0, $561 = 0, $562 = 0, $563 = 0, $564 = 0, $565 = 0, $566 = 0;
 var $567 = 0, $568 = 0, $569 = 0, $57 = 0, $570 = 0, $571 = 0, $572 = 0, $573 = 0, $574 = 0, $575 = 0, $576 = 0, $577 = 0, $578 = 0, $579 = 0, $58 = 0, $580 = 0, $581 = 0, $582 = 0, $583 = 0, $584 = 0;
 var $585 = 0, $586 = 0, $587 = 0, $588 = 0, $589 = 0, $59 = 0, $590 = 0, $591 = 0, $592 = 0, $593 = 0, $594 = 0, $595 = 0, $596 = 0, $597 = 0, $598 = 0, $599 = 0, $6 = 0, $60 = 0, $600 = 0, $601 = 0;
 var $602 = 0, $603 = 0, $604 = 0, $605 = 0, $606 = 0, $607 = 0, $608 = 0, $609 = 0, $61 = 0, $610 = 0, $611 = 0, $612 = 0, $613 = 0, $614 = 0, $615 = 0, $616 = 0, $617 = 0, $618 = 0, $619 = 0, $62 = 0;
 var $620 = 0, $621 = 0, $622 = 0, $623 = 0, $624 = 0, $625 = 0, $626 = 0, $627 = 0, $628 = 0, $629 = 0, $63 = 0, $630 = 0, $631 = 0, $632 = 0, $633 = 0, $634 = 0, $635 = 0, $636 = 0, $637 = 0, $638 = 0;
 var $639 = 0, $64 = 0, $640 = 0, $641 = 0, $642 = 0, $643 = 0, $644 = 0, $645 = 0, $646 = 0, $647 = 0, $648 = 0, $649 = 0, $65 = 0, $650 = 0, $651 = 0, $652 = 0, $653 = 0, $654 = 0, $655 = 0, $656 = 0;
 var $657 = 0, $658 = 0, $659 = 0, $66 = 0, $660 = 0, $661 = 0, $662 = 0, $663 = 0, $664 = 0, $665 = 0, $666 = 0, $667 = 0, $668 = 0, $669 = 0, $67 = 0, $670 = 0, $671 = 0, $672 = 0, $673 = 0, $674 = 0;
 var $675 = 0, $676 = 0, $677 = 0, $678 = 0, $679 = 0, $68 = 0, $680 = 0, $681 = 0, $682 = 0, $683 = 0, $684 = 0, $685 = 0, $686 = 0, $687 = 0, $688 = 0, $689 = 0, $69 = 0, $690 = 0, $691 = 0, $692 = 0;
 var $693 = 0, $694 = 0, $695 = 0, $696 = 0, $697 = 0, $698 = 0, $699 = 0, $7 = 0, $70 = 0, $700 = 0, $701 = 0, $702 = 0, $703 = 0, $704 = 0, $705 = 0, $706 = 0, $707 = 0, $708 = 0, $709 = 0, $71 = 0;
 var $710 = 0, $711 = 0, $712 = 0, $713 = 0, $714 = 0, $715 = 0, $716 = 0, $717 = 0, $718 = 0, $719 = 0, $72 = 0, $720 = 0, $721 = 0, $722 = 0, $723 = 0, $724 = 0, $725 = 0, $726 = 0, $727 = 0, $728 = 0;
 var $729 = 0, $73 = 0, $730 = 0, $731 = 0, $732 = 0, $733 = 0, $734 = 0, $735 = 0, $736 = 0, $737 = 0, $738 = 0, $739 = 0, $74 = 0, $740 = 0, $741 = 0, $742 = 0, $743 = 0, $744 = 0, $745 = 0, $746 = 0;
 var $747 = 0, $748 = 0, $749 = 0, $75 = 0, $750 = 0, $751 = 0, $752 = 0, $753 = 0, $754 = 0, $755 = 0, $756 = 0, $757 = 0, $758 = 0, $759 = 0, $76 = 0, $760 = 0, $761 = 0, $762 = 0, $763 = 0, $764 = 0;
 var $765 = 0, $766 = 0, $767 = 0, $768 = 0, $769 = 0, $77 = 0, $770 = 0, $771 = 0, $772 = 0, $773 = 0, $774 = 0, $775 = 0, $776 = 0, $777 = 0, $778 = 0, $779 = 0, $78 = 0, $780 = 0, $781 = 0, $782 = 0;
 var $783 = 0, $784 = 0, $785 = 0, $786 = 0, $787 = 0, $788 = 0, $789 = 0, $79 = 0, $790 = 0, $791 = 0, $792 = 0, $793 = 0, $794 = 0, $795 = 0, $796 = 0, $797 = 0, $798 = 0, $799 = 0, $8 = 0, $80 = 0;
 var $800 = 0, $801 = 0, $802 = 0, $803 = 0, $804 = 0, $805 = 0, $806 = 0, $807 = 0, $808 = 0, $809 = 0, $81 = 0, $810 = 0, $811 = 0, $812 = 0, $813 = 0, $814 = 0, $815 = 0, $816 = 0, $817 = 0, $818 = 0;
 var $819 = 0, $82 = 0, $820 = 0, $821 = 0, $822 = 0, $823 = 0, $824 = 0, $825 = 0, $826 = 0, $827 = 0, $828 = 0, $829 = 0, $83 = 0, $830 = 0, $831 = 0, $832 = 0, $833 = 0, $834 = 0, $835 = 0, $836 = 0;
 var $837 = 0, $838 = 0, $839 = 0, $84 = 0, $840 = 0, $841 = 0, $842 = 0, $843 = 0, $844 = 0, $845 = 0, $846 = 0, $847 = 0, $848 = 0, $849 = 0, $85 = 0, $850 = 0, $851 = 0, $852 = 0, $853 = 0, $854 = 0;
 var $855 = 0, $856 = 0, $857 = 0, $858 = 0, $859 = 0, $86 = 0, $860 = 0, $861 = 0, $862 = 0, $863 = 0, $864 = 0, $865 = 0, $866 = 0, $867 = 0, $868 = 0, $869 = 0, $87 = 0, $870 = 0, $871 = 0, $872 = 0;
 var $873 = 0, $874 = 0, $875 = 0, $876 = 0, $877 = 0, $878 = 0, $879 = 0, $88 = 0, $880 = 0, $881 = 0, $882 = 0, $883 = 0, $884 = 0, $885 = 0, $886 = 0, $887 = 0, $888 = 0, $889 = 0, $89 = 0, $890 = 0;
 var $891 = 0, $892 = 0, $893 = 0, $894 = 0, $895 = 0, $896 = 0, $897 = 0, $898 = 0, $899 = 0, $9 = 0, $90 = 0, $900 = 0, $901 = 0, $902 = 0, $903 = 0, $904 = 0, $905 = 0, $906 = 0, $907 = 0, $908 = 0;
 var $909 = 0, $91 = 0, $910 = 0, $911 = 0, $912 = 0, $913 = 0, $914 = 0, $915 = 0, $916 = 0, $917 = 0, $918 = 0, $919 = 0, $92 = 0, $920 = 0, $921 = 0, $922 = 0, $923 = 0, $924 = 0, $925 = 0, $926 = 0;
 var $927 = 0, $928 = 0, $929 = 0, $93 = 0, $930 = 0, $931 = 0, $932 = 0, $933 = 0, $934 = 0, $935 = 0, $936 = 0, $937 = 0, $938 = 0, $939 = 0, $94 = 0, $940 = 0, $941 = 0, $942 = 0, $943 = 0, $944 = 0;
 var $945 = 0, $946 = 0, $947 = 0, $948 = 0, $949 = 0, $95 = 0, $950 = 0, $951 = 0, $952 = 0, $953 = 0, $954 = 0, $955 = 0, $956 = 0, $957 = 0, $958 = 0, $959 = 0, $96 = 0, $960 = 0, $961 = 0, $962 = 0;
 var $963 = 0, $964 = 0, $965 = 0, $966 = 0, $967 = 0, $968 = 0, $969 = 0, $97 = 0, $970 = 0, $971 = 0, $972 = 0, $973 = 0, $974 = 0, $975 = 0, $976 = 0, $977 = 0, $978 = 0, $979 = 0, $98 = 0, $980 = 0;
 var $981 = 0, $982 = 0, $983 = 0, $984 = 0, $985 = 0, $986 = 0, $987 = 0, $988 = 0, $989 = 0, $99 = 0, $990 = 0, $aRegIdx = 0, $aToOpen = 0, $aXRef = 0, $addrOpen = 0, $addrTop = 0, $aiCurOnePass = 0, $bReplace = 0, $chngKey = 0, $chngPk = 0;
 var $chngRowid = 0, $db = 0, $hasFK = 0, $i = 0, $iBaseCur = 0, $iDataCur = 0, $iDb = 0, $iEph = 0, $iIdxCur = 0, $iPk = 0, $isView = 0, $j = 0, $j1 = 0, $labelBreak = 0, $labelContinue = 0, $nIdx = 0, $nKey = 0, $nPk = 0, $newmask = 0, $okOnePass = 0;
 var $oldmask = 0, $or$cond = 0, $or$cond11 = 0, $or$cond13 = 0, $or$cond15 = 0, $or$cond17 = 0, $or$cond19 = 0, $or$cond21 = 0, $or$cond23 = 0, $or$cond3 = 0, $or$cond5 = 0, $or$cond7 = 0, $or$cond9 = 0, $pIdx = 0, $pPk = 0, $pRowidExpr = 0, $pTab = 0, $pTrigger = 0, $pWInfo = 0, $rc = 0;
 var $reg = 0, $regKey = 0, $regNew = 0, $regNewRowid = 0, $regOld = 0, $regOldRowid = 0, $regRowCount = 0, $regRowSet = 0, $sContext = 0, $sNC = 0, $tmask = 0, $v = 0, $vararg_buffer = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 256|0;
 $vararg_buffer = sp;
 $0 = sp + 244|0;
 $1 = sp + 240|0;
 $2 = sp + 236|0;
 $3 = sp + 232|0;
 $4 = sp + 228|0;
 $i = sp + 224|0;
 $j = sp + 220|0;
 $pTab = sp + 216|0;
 $addrTop = sp + 212|0;
 $pWInfo = sp + 208|0;
 $v = sp + 204|0;
 $pIdx = sp + 200|0;
 $pPk = sp + 196|0;
 $nIdx = sp + 192|0;
 $iBaseCur = sp + 188|0;
 $iDataCur = sp + 184|0;
 $iIdxCur = sp + 180|0;
 $db = sp + 176|0;
 $aRegIdx = sp + 172|0;
 $aXRef = sp + 168|0;
 $aToOpen = sp + 164|0;
 $chngPk = sp + 252|0;
 $chngRowid = sp + 251|0;
 $chngKey = sp + 250|0;
 $pRowidExpr = sp + 160|0;
 $sContext = sp + 152|0;
 $sNC = sp + 120|0;
 $iDb = sp + 112|0;
 $okOnePass = sp + 108|0;
 $hasFK = sp + 104|0;
 $labelBreak = sp + 100|0;
 $labelContinue = sp + 96|0;
 $isView = sp + 92|0;
 $pTrigger = sp + 88|0;
 $tmask = sp + 84|0;
 $newmask = sp + 80|0;
 $iEph = sp + 76|0;
 $nKey = sp + 72|0;
 $aiCurOnePass = sp + 64|0;
 $regRowCount = sp + 56|0;
 $regOldRowid = sp + 52|0;
 $regNewRowid = sp + 48|0;
 $regNew = sp + 44|0;
 $regOld = sp + 40|0;
 $regRowSet = sp + 36|0;
 $regKey = sp + 32|0;
 $rc = sp + 28|0;
 $reg = sp + 24|0;
 $iPk = sp + 20|0;
 $nPk = sp + 248|0;
 $addrOpen = sp + 16|0;
 $oldmask = sp + 12|0;
 $j1 = sp + 8|0;
 $bReplace = sp + 4|0;
 HEAP32[$0>>2] = $pParse;
 HEAP32[$1>>2] = $pTabList;
 HEAP32[$2>>2] = $pChanges;
 HEAP32[$3>>2] = $pWhere;
 HEAP32[$4>>2] = $onError;
 HEAP32[$addrTop>>2] = 0;
 HEAP32[$aRegIdx>>2] = 0;
 HEAP32[$aXRef>>2] = 0;
 HEAP32[$pRowidExpr>>2] = 0;
 HEAP32[$iEph>>2] = 0;
 HEAP32[$nKey>>2] = 0;
 HEAP32[$regRowCount>>2] = 0;
 HEAP32[$regOld>>2] = 0;
 HEAP32[$regRowSet>>2] = 0;
 HEAP32[$regKey>>2] = 0;
 ;HEAP32[$sContext+0>>2]=0|0;HEAP32[$sContext+4>>2]=0|0;
 $5 = HEAP32[$0>>2]|0;
 $6 = HEAP32[$5>>2]|0;
 HEAP32[$db>>2] = $6;
 $7 = HEAP32[$0>>2]|0;
 $8 = (($7) + 64|0);
 $9 = HEAP32[$8>>2]|0;
 $10 = ($9|0)!=(0);
 if ($10) {
  _sqlite3AuthContextPop($sContext);
  $983 = HEAP32[$db>>2]|0;
  $984 = HEAP32[$aXRef>>2]|0;
  _sqlite3DbFree($983,$984);
  $985 = HEAP32[$db>>2]|0;
  $986 = HEAP32[$1>>2]|0;
  _sqlite3SrcListDelete($985,$986);
  $987 = HEAP32[$db>>2]|0;
  $988 = HEAP32[$2>>2]|0;
  _sqlite3ExprListDelete($987,$988);
  $989 = HEAP32[$db>>2]|0;
  $990 = HEAP32[$3>>2]|0;
  _sqlite3ExprDelete($989,$990);
  STACKTOP = sp;return;
 }
 $11 = HEAP32[$db>>2]|0;
 $12 = (($11) + 64|0);
 $13 = HEAP8[$12>>0]|0;
 $14 = $13&255;
 $15 = ($14|0)!=(0);
 if ($15) {
  _sqlite3AuthContextPop($sContext);
  $983 = HEAP32[$db>>2]|0;
  $984 = HEAP32[$aXRef>>2]|0;
  _sqlite3DbFree($983,$984);
  $985 = HEAP32[$db>>2]|0;
  $986 = HEAP32[$1>>2]|0;
  _sqlite3SrcListDelete($985,$986);
  $987 = HEAP32[$db>>2]|0;
  $988 = HEAP32[$2>>2]|0;
  _sqlite3ExprListDelete($987,$988);
  $989 = HEAP32[$db>>2]|0;
  $990 = HEAP32[$3>>2]|0;
  _sqlite3ExprDelete($989,$990);
  STACKTOP = sp;return;
 }
 $16 = HEAP32[$0>>2]|0;
 $17 = HEAP32[$1>>2]|0;
 $18 = (_sqlite3SrcListLookup($16,$17)|0);
 HEAP32[$pTab>>2] = $18;
 $19 = HEAP32[$pTab>>2]|0;
 $20 = ($19|0)==(0|0);
 if ($20) {
  _sqlite3AuthContextPop($sContext);
  $983 = HEAP32[$db>>2]|0;
  $984 = HEAP32[$aXRef>>2]|0;
  _sqlite3DbFree($983,$984);
  $985 = HEAP32[$db>>2]|0;
  $986 = HEAP32[$1>>2]|0;
  _sqlite3SrcListDelete($985,$986);
  $987 = HEAP32[$db>>2]|0;
  $988 = HEAP32[$2>>2]|0;
  _sqlite3ExprListDelete($987,$988);
  $989 = HEAP32[$db>>2]|0;
  $990 = HEAP32[$3>>2]|0;
  _sqlite3ExprDelete($989,$990);
  STACKTOP = sp;return;
 }
 $21 = HEAP32[$0>>2]|0;
 $22 = HEAP32[$21>>2]|0;
 $23 = HEAP32[$pTab>>2]|0;
 $24 = (($23) + 68|0);
 $25 = HEAP32[$24>>2]|0;
 $26 = (_sqlite3SchemaToIndex($22,$25)|0);
 HEAP32[$iDb>>2] = $26;
 $27 = HEAP32[$0>>2]|0;
 $28 = HEAP32[$pTab>>2]|0;
 $29 = HEAP32[$2>>2]|0;
 $30 = (_sqlite3TriggersExist($27,$28,110,$29,$tmask)|0);
 HEAP32[$pTrigger>>2] = $30;
 $31 = HEAP32[$pTab>>2]|0;
 $32 = (($31) + 12|0);
 $33 = HEAP32[$32>>2]|0;
 $34 = ($33|0)!=(0|0);
 $35 = $34&1;
 HEAP32[$isView>>2] = $35;
 $36 = HEAP32[$0>>2]|0;
 $37 = HEAP32[$pTab>>2]|0;
 $38 = (_sqlite3ViewGetColumnNames($36,$37)|0);
 $39 = ($38|0)!=(0);
 if ($39) {
  _sqlite3AuthContextPop($sContext);
  $983 = HEAP32[$db>>2]|0;
  $984 = HEAP32[$aXRef>>2]|0;
  _sqlite3DbFree($983,$984);
  $985 = HEAP32[$db>>2]|0;
  $986 = HEAP32[$1>>2]|0;
  _sqlite3SrcListDelete($985,$986);
  $987 = HEAP32[$db>>2]|0;
  $988 = HEAP32[$2>>2]|0;
  _sqlite3ExprListDelete($987,$988);
  $989 = HEAP32[$db>>2]|0;
  $990 = HEAP32[$3>>2]|0;
  _sqlite3ExprDelete($989,$990);
  STACKTOP = sp;return;
 }
 $40 = HEAP32[$0>>2]|0;
 $41 = HEAP32[$pTab>>2]|0;
 $42 = HEAP32[$tmask>>2]|0;
 $43 = (_sqlite3IsReadOnly($40,$41,$42)|0);
 $44 = ($43|0)!=(0);
 if ($44) {
  _sqlite3AuthContextPop($sContext);
  $983 = HEAP32[$db>>2]|0;
  $984 = HEAP32[$aXRef>>2]|0;
  _sqlite3DbFree($983,$984);
  $985 = HEAP32[$db>>2]|0;
  $986 = HEAP32[$1>>2]|0;
  _sqlite3SrcListDelete($985,$986);
  $987 = HEAP32[$db>>2]|0;
  $988 = HEAP32[$2>>2]|0;
  _sqlite3ExprListDelete($987,$988);
  $989 = HEAP32[$db>>2]|0;
  $990 = HEAP32[$3>>2]|0;
  _sqlite3ExprDelete($989,$990);
  STACKTOP = sp;return;
 }
 $45 = HEAP32[$0>>2]|0;
 $46 = (($45) + 68|0);
 $47 = HEAP32[$46>>2]|0;
 $48 = (($47) + 1)|0;
 HEAP32[$46>>2] = $48;
 HEAP32[$iDataCur>>2] = $47;
 HEAP32[$iBaseCur>>2] = $47;
 $49 = HEAP32[$1>>2]|0;
 $50 = (($49) + 8|0);
 $51 = (($50) + 40|0);
 HEAP32[$51>>2] = $47;
 $52 = HEAP32[$iDataCur>>2]|0;
 $53 = (($52) + 1)|0;
 HEAP32[$iIdxCur>>2] = $53;
 $54 = HEAP32[$pTab>>2]|0;
 $55 = (($54) + 44|0);
 $56 = HEAP8[$55>>0]|0;
 $57 = $56&255;
 $58 = $57 & 32;
 $59 = ($58|0)==(0);
 if ($59) {
  $62 = 0;
 } else {
  $60 = HEAP32[$pTab>>2]|0;
  $61 = (_sqlite3PrimaryKeyIndex($60)|0);
  $62 = $61;
 }
 HEAP32[$pPk>>2] = $62;
 HEAP32[$nIdx>>2] = 0;
 $63 = HEAP32[$pTab>>2]|0;
 $64 = (($63) + 8|0);
 $65 = HEAP32[$64>>2]|0;
 HEAP32[$pIdx>>2] = $65;
 while(1) {
  $66 = HEAP32[$pIdx>>2]|0;
  $67 = ($66|0)!=(0|0);
  if (!($67)) {
   break;
  }
  $68 = HEAP32[$pIdx>>2]|0;
  $69 = (($68) + 55|0);
  $70 = HEAP8[$69>>0]|0;
  $71 = $70 & 3;
  $72 = $71&255;
  $73 = ($72|0)==(2);
  $74 = HEAP32[$pPk>>2]|0;
  $75 = ($74|0)!=(0|0);
  $or$cond = $73 & $75;
  if ($or$cond) {
   $76 = HEAP32[$0>>2]|0;
   $77 = (($76) + 68|0);
   $78 = HEAP32[$77>>2]|0;
   HEAP32[$iDataCur>>2] = $78;
   $79 = HEAP32[$iDataCur>>2]|0;
   $80 = HEAP32[$1>>2]|0;
   $81 = (($80) + 8|0);
   $82 = (($81) + 40|0);
   HEAP32[$82>>2] = $79;
  }
  $83 = HEAP32[$0>>2]|0;
  $84 = (($83) + 68|0);
  $85 = HEAP32[$84>>2]|0;
  $86 = (($85) + 1)|0;
  HEAP32[$84>>2] = $86;
  $87 = HEAP32[$pIdx>>2]|0;
  $88 = (($87) + 20|0);
  $89 = HEAP32[$88>>2]|0;
  HEAP32[$pIdx>>2] = $89;
  $90 = HEAP32[$nIdx>>2]|0;
  $91 = (($90) + 1)|0;
  HEAP32[$nIdx>>2] = $91;
 }
 $92 = HEAP32[$db>>2]|0;
 $93 = HEAP32[$pTab>>2]|0;
 $94 = (($93) + 38|0);
 $95 = HEAP16[$94>>1]|0;
 $96 = $95 << 16 >> 16;
 $97 = HEAP32[$nIdx>>2]|0;
 $98 = (($96) + ($97))|0;
 $99 = $98<<2;
 $100 = HEAP32[$nIdx>>2]|0;
 $101 = (($99) + ($100))|0;
 $102 = (($101) + 2)|0;
 $103 = (_sqlite3DbMallocRaw($92,$102,0)|0);
 HEAP32[$aXRef>>2] = $103;
 $104 = HEAP32[$aXRef>>2]|0;
 $105 = ($104|0)==(0|0);
 if ($105) {
  _sqlite3AuthContextPop($sContext);
  $983 = HEAP32[$db>>2]|0;
  $984 = HEAP32[$aXRef>>2]|0;
  _sqlite3DbFree($983,$984);
  $985 = HEAP32[$db>>2]|0;
  $986 = HEAP32[$1>>2]|0;
  _sqlite3SrcListDelete($985,$986);
  $987 = HEAP32[$db>>2]|0;
  $988 = HEAP32[$2>>2]|0;
  _sqlite3ExprListDelete($987,$988);
  $989 = HEAP32[$db>>2]|0;
  $990 = HEAP32[$3>>2]|0;
  _sqlite3ExprDelete($989,$990);
  STACKTOP = sp;return;
 }
 $106 = HEAP32[$aXRef>>2]|0;
 $107 = HEAP32[$pTab>>2]|0;
 $108 = (($107) + 38|0);
 $109 = HEAP16[$108>>1]|0;
 $110 = $109 << 16 >> 16;
 $111 = (($106) + ($110<<2)|0);
 HEAP32[$aRegIdx>>2] = $111;
 $112 = HEAP32[$aRegIdx>>2]|0;
 $113 = HEAP32[$nIdx>>2]|0;
 $114 = (($112) + ($113<<2)|0);
 HEAP32[$aToOpen>>2] = $114;
 $115 = HEAP32[$aToOpen>>2]|0;
 $116 = HEAP32[$nIdx>>2]|0;
 $117 = (($116) + 1)|0;
 _memset(($115|0),1,($117|0))|0;
 $118 = HEAP32[$nIdx>>2]|0;
 $119 = (($118) + 1)|0;
 $120 = HEAP32[$aToOpen>>2]|0;
 $121 = (($120) + ($119)|0);
 HEAP8[$121>>0] = 0;
 HEAP32[$i>>2] = 0;
 while(1) {
  $122 = HEAP32[$i>>2]|0;
  $123 = HEAP32[$pTab>>2]|0;
  $124 = (($123) + 38|0);
  $125 = HEAP16[$124>>1]|0;
  $126 = $125 << 16 >> 16;
  $127 = ($122|0)<($126|0);
  if (!($127)) {
   break;
  }
  $128 = HEAP32[$i>>2]|0;
  $129 = HEAP32[$aXRef>>2]|0;
  $130 = (($129) + ($128<<2)|0);
  HEAP32[$130>>2] = -1;
  $131 = HEAP32[$i>>2]|0;
  $132 = (($131) + 1)|0;
  HEAP32[$i>>2] = $132;
 }
 ;HEAP32[$sNC+0>>2]=0|0;HEAP32[$sNC+4>>2]=0|0;HEAP32[$sNC+8>>2]=0|0;HEAP32[$sNC+12>>2]=0|0;HEAP32[$sNC+16>>2]=0|0;HEAP32[$sNC+20>>2]=0|0;HEAP32[$sNC+24>>2]=0|0;HEAP32[$sNC+28>>2]=0|0;
 $133 = HEAP32[$0>>2]|0;
 HEAP32[$sNC>>2] = $133;
 $134 = HEAP32[$1>>2]|0;
 $135 = (($sNC) + 4|0);
 HEAP32[$135>>2] = $134;
 HEAP8[$chngPk>>0] = 0;
 HEAP8[$chngRowid>>0] = 0;
 HEAP32[$i>>2] = 0;
 while(1) {
  $136 = HEAP32[$i>>2]|0;
  $137 = HEAP32[$2>>2]|0;
  $138 = HEAP32[$137>>2]|0;
  $139 = ($136|0)<($138|0);
  if (!($139)) {
   label = 41;
   break;
  }
  $140 = HEAP32[$i>>2]|0;
  $141 = HEAP32[$2>>2]|0;
  $142 = (($141) + 4|0);
  $143 = HEAP32[$142>>2]|0;
  $144 = (($143) + (($140*20)|0)|0);
  $145 = HEAP32[$144>>2]|0;
  $146 = (_sqlite3ResolveExprNames($sNC,$145)|0);
  $147 = ($146|0)!=(0);
  if ($147) {
   label = 182;
   break;
  }
  HEAP32[$j>>2] = 0;
  while(1) {
   $148 = HEAP32[$j>>2]|0;
   $149 = HEAP32[$pTab>>2]|0;
   $150 = (($149) + 38|0);
   $151 = HEAP16[$150>>1]|0;
   $152 = $151 << 16 >> 16;
   $153 = ($148|0)<($152|0);
   if (!($153)) {
    break;
   }
   $154 = HEAP32[$j>>2]|0;
   $155 = HEAP32[$pTab>>2]|0;
   $156 = (($155) + 4|0);
   $157 = HEAP32[$156>>2]|0;
   $158 = (($157) + (($154*24)|0)|0);
   $159 = HEAP32[$158>>2]|0;
   $160 = HEAP32[$i>>2]|0;
   $161 = HEAP32[$2>>2]|0;
   $162 = (($161) + 4|0);
   $163 = HEAP32[$162>>2]|0;
   $164 = (($163) + (($160*20)|0)|0);
   $165 = (($164) + 4|0);
   $166 = HEAP32[$165>>2]|0;
   $167 = (_sqlite3_stricmp($159,$166)|0);
   $168 = ($167|0)==(0);
   $169 = HEAP32[$j>>2]|0;
   if ($168) {
    label = 23;
    break;
   }
   $197 = (($169) + 1)|0;
   HEAP32[$j>>2] = $197;
  }
  if ((label|0) == 23) {
   label = 0;
   $170 = HEAP32[$pTab>>2]|0;
   $171 = (($170) + 36|0);
   $172 = HEAP16[$171>>1]|0;
   $173 = $172 << 16 >> 16;
   $174 = ($169|0)==($173|0);
   if ($174) {
    HEAP8[$chngRowid>>0] = 1;
    $175 = HEAP32[$i>>2]|0;
    $176 = HEAP32[$2>>2]|0;
    $177 = (($176) + 4|0);
    $178 = HEAP32[$177>>2]|0;
    $179 = (($178) + (($175*20)|0)|0);
    $180 = HEAP32[$179>>2]|0;
    HEAP32[$pRowidExpr>>2] = $180;
   } else {
    $181 = HEAP32[$pPk>>2]|0;
    $182 = ($181|0)!=(0|0);
    if ($182) {
     $183 = HEAP32[$j>>2]|0;
     $184 = HEAP32[$pTab>>2]|0;
     $185 = (($184) + 4|0);
     $186 = HEAP32[$185>>2]|0;
     $187 = (($186) + (($183*24)|0)|0);
     $188 = (($187) + 23|0);
     $189 = HEAP8[$188>>0]|0;
     $190 = $189&255;
     $191 = $190 & 1;
     $192 = ($191|0)!=(0);
     if ($192) {
      HEAP8[$chngPk>>0] = 1;
     }
    }
   }
   $193 = HEAP32[$i>>2]|0;
   $194 = HEAP32[$j>>2]|0;
   $195 = HEAP32[$aXRef>>2]|0;
   $196 = (($195) + ($194<<2)|0);
   HEAP32[$196>>2] = $193;
  }
  $198 = HEAP32[$j>>2]|0;
  $199 = HEAP32[$pTab>>2]|0;
  $200 = (($199) + 38|0);
  $201 = HEAP16[$200>>1]|0;
  $202 = $201 << 16 >> 16;
  $203 = ($198|0)>=($202|0);
  if ($203) {
   $204 = HEAP32[$pPk>>2]|0;
   $205 = ($204|0)==(0|0);
   if (!($205)) {
    label = 34;
    break;
   }
   $206 = HEAP32[$i>>2]|0;
   $207 = HEAP32[$2>>2]|0;
   $208 = (($207) + 4|0);
   $209 = HEAP32[$208>>2]|0;
   $210 = (($209) + (($206*20)|0)|0);
   $211 = (($210) + 4|0);
   $212 = HEAP32[$211>>2]|0;
   $213 = (_sqlite3IsRowid($212)|0);
   $214 = ($213|0)!=(0);
   if (!($214)) {
    label = 34;
    break;
   }
   HEAP32[$j>>2] = -1;
   HEAP8[$chngRowid>>0] = 1;
   $215 = HEAP32[$i>>2]|0;
   $216 = HEAP32[$2>>2]|0;
   $217 = (($216) + 4|0);
   $218 = HEAP32[$217>>2]|0;
   $219 = (($218) + (($215*20)|0)|0);
   $220 = HEAP32[$219>>2]|0;
   HEAP32[$pRowidExpr>>2] = $220;
  }
  $231 = HEAP32[$0>>2]|0;
  $232 = HEAP32[$pTab>>2]|0;
  $233 = HEAP32[$232>>2]|0;
  $234 = HEAP32[$j>>2]|0;
  $235 = ($234|0)<(0);
  if ($235) {
   $248 = 17288;
  } else {
   $236 = HEAP32[$j>>2]|0;
   $237 = HEAP32[$pTab>>2]|0;
   $238 = (($237) + 4|0);
   $239 = HEAP32[$238>>2]|0;
   $240 = (($239) + (($236*24)|0)|0);
   $241 = HEAP32[$240>>2]|0;
   $248 = $241;
  }
  $242 = HEAP32[$iDb>>2]|0;
  $243 = HEAP32[$db>>2]|0;
  $244 = (($243) + 16|0);
  $245 = HEAP32[$244>>2]|0;
  $246 = (($245) + ($242<<4)|0);
  $247 = HEAP32[$246>>2]|0;
  $249 = (_sqlite3AuthCheck($231,23,$233,$248,$247)|0);
  HEAP32[$rc>>2] = $249;
  $250 = HEAP32[$rc>>2]|0;
  $251 = ($250|0)==(1);
  if ($251) {
   label = 182;
   break;
  }
  $252 = HEAP32[$rc>>2]|0;
  $253 = ($252|0)==(2);
  if ($253) {
   $254 = HEAP32[$j>>2]|0;
   $255 = HEAP32[$aXRef>>2]|0;
   $256 = (($255) + ($254<<2)|0);
   HEAP32[$256>>2] = -1;
  }
  $257 = HEAP32[$i>>2]|0;
  $258 = (($257) + 1)|0;
  HEAP32[$i>>2] = $258;
 }
 if ((label|0) == 34) {
  $221 = HEAP32[$0>>2]|0;
  $222 = HEAP32[$i>>2]|0;
  $223 = HEAP32[$2>>2]|0;
  $224 = (($223) + 4|0);
  $225 = HEAP32[$224>>2]|0;
  $226 = (($225) + (($222*20)|0)|0);
  $227 = (($226) + 4|0);
  $228 = HEAP32[$227>>2]|0;
  HEAP32[$vararg_buffer>>2] = $228;
  _sqlite3ErrorMsg($221,24520,$vararg_buffer);
  $229 = HEAP32[$0>>2]|0;
  $230 = (($229) + 17|0);
  HEAP8[$230>>0] = 1;
  _sqlite3AuthContextPop($sContext);
  $983 = HEAP32[$db>>2]|0;
  $984 = HEAP32[$aXRef>>2]|0;
  _sqlite3DbFree($983,$984);
  $985 = HEAP32[$db>>2]|0;
  $986 = HEAP32[$1>>2]|0;
  _sqlite3SrcListDelete($985,$986);
  $987 = HEAP32[$db>>2]|0;
  $988 = HEAP32[$2>>2]|0;
  _sqlite3ExprListDelete($987,$988);
  $989 = HEAP32[$db>>2]|0;
  $990 = HEAP32[$3>>2]|0;
  _sqlite3ExprDelete($989,$990);
  STACKTOP = sp;return;
 }
 else if ((label|0) == 41) {
  $259 = HEAP8[$chngRowid>>0]|0;
  $260 = $259&255;
  $261 = HEAP8[$chngPk>>0]|0;
  $262 = $261&255;
  $263 = (($260) + ($262))|0;
  $264 = $263&255;
  HEAP8[$chngKey>>0] = $264;
  $265 = HEAP32[$1>>2]|0;
  $266 = (($265) + 8|0);
  $267 = (($266) + 56|0);
  $268 = $267;
  $269 = $268;
  HEAP32[$269>>2] = 0;
  $270 = (($268) + 4)|0;
  $271 = $270;
  HEAP32[$271>>2] = 0;
  $272 = HEAP32[$0>>2]|0;
  $273 = HEAP32[$pTab>>2]|0;
  $274 = HEAP32[$aXRef>>2]|0;
  $275 = HEAP8[$chngKey>>0]|0;
  $276 = $275&255;
  $277 = (_sqlite3FkRequired($272,$273,$274,$276)|0);
  HEAP32[$hasFK>>2] = $277;
  HEAP32[$j>>2] = 0;
  $278 = HEAP32[$pTab>>2]|0;
  $279 = (($278) + 8|0);
  $280 = HEAP32[$279>>2]|0;
  HEAP32[$pIdx>>2] = $280;
  while(1) {
   $281 = HEAP32[$pIdx>>2]|0;
   $282 = ($281|0)!=(0|0);
   if (!($282)) {
    break;
   }
   $283 = HEAP8[$chngKey>>0]|0;
   $284 = $283&255;
   $285 = ($284|0)!=(0);
   $286 = HEAP32[$hasFK>>2]|0;
   $287 = ($286|0)!=(0);
   $or$cond3 = $285 | $287;
   L69: do {
    if ($or$cond3) {
     label = 46;
    } else {
     $288 = HEAP32[$pIdx>>2]|0;
     $289 = (($288) + 36|0);
     $290 = HEAP32[$289>>2]|0;
     $291 = ($290|0)!=(0|0);
     if ($291) {
      label = 46;
     } else {
      $292 = HEAP32[$pIdx>>2]|0;
      $293 = HEAP32[$pPk>>2]|0;
      $294 = ($292|0)==($293|0);
      if ($294) {
       label = 46;
      } else {
       HEAP32[$reg>>2] = 0;
       HEAP32[$i>>2] = 0;
       while(1) {
        $299 = HEAP32[$i>>2]|0;
        $300 = HEAP32[$pIdx>>2]|0;
        $301 = (($300) + 50|0);
        $302 = HEAP16[$301>>1]|0;
        $303 = $302&65535;
        $304 = ($299|0)<($303|0);
        if (!($304)) {
         break L69;
        }
        $305 = HEAP32[$i>>2]|0;
        $306 = HEAP32[$pIdx>>2]|0;
        $307 = (($306) + 4|0);
        $308 = HEAP32[$307>>2]|0;
        $309 = (($308) + ($305<<1)|0);
        $310 = HEAP16[$309>>1]|0;
        $311 = $310 << 16 >> 16;
        $312 = HEAP32[$aXRef>>2]|0;
        $313 = (($312) + ($311<<2)|0);
        $314 = HEAP32[$313>>2]|0;
        $315 = ($314|0)>=(0);
        if ($315) {
         break;
        }
        $320 = HEAP32[$i>>2]|0;
        $321 = (($320) + 1)|0;
        HEAP32[$i>>2] = $321;
       }
       $316 = HEAP32[$0>>2]|0;
       $317 = (($316) + 72|0);
       $318 = HEAP32[$317>>2]|0;
       $319 = (($318) + 1)|0;
       HEAP32[$317>>2] = $319;
       HEAP32[$reg>>2] = $319;
      }
     }
    }
   } while(0);
   if ((label|0) == 46) {
    label = 0;
    $295 = HEAP32[$0>>2]|0;
    $296 = (($295) + 72|0);
    $297 = HEAP32[$296>>2]|0;
    $298 = (($297) + 1)|0;
    HEAP32[$296>>2] = $298;
    HEAP32[$reg>>2] = $298;
   }
   $322 = HEAP32[$reg>>2]|0;
   $323 = ($322|0)==(0);
   if ($323) {
    $324 = HEAP32[$j>>2]|0;
    $325 = (($324) + 1)|0;
    $326 = HEAP32[$aToOpen>>2]|0;
    $327 = (($326) + ($325)|0);
    HEAP8[$327>>0] = 0;
   }
   $328 = HEAP32[$reg>>2]|0;
   $329 = HEAP32[$j>>2]|0;
   $330 = HEAP32[$aRegIdx>>2]|0;
   $331 = (($330) + ($329<<2)|0);
   HEAP32[$331>>2] = $328;
   $332 = HEAP32[$pIdx>>2]|0;
   $333 = (($332) + 20|0);
   $334 = HEAP32[$333>>2]|0;
   HEAP32[$pIdx>>2] = $334;
   $335 = HEAP32[$j>>2]|0;
   $336 = (($335) + 1)|0;
   HEAP32[$j>>2] = $336;
  }
  $337 = HEAP32[$0>>2]|0;
  $338 = (_sqlite3GetVdbe($337)|0);
  HEAP32[$v>>2] = $338;
  $339 = HEAP32[$v>>2]|0;
  $340 = ($339|0)==(0|0);
  if ($340) {
   _sqlite3AuthContextPop($sContext);
   $983 = HEAP32[$db>>2]|0;
   $984 = HEAP32[$aXRef>>2]|0;
   _sqlite3DbFree($983,$984);
   $985 = HEAP32[$db>>2]|0;
   $986 = HEAP32[$1>>2]|0;
   _sqlite3SrcListDelete($985,$986);
   $987 = HEAP32[$db>>2]|0;
   $988 = HEAP32[$2>>2]|0;
   _sqlite3ExprListDelete($987,$988);
   $989 = HEAP32[$db>>2]|0;
   $990 = HEAP32[$3>>2]|0;
   _sqlite3ExprDelete($989,$990);
   STACKTOP = sp;return;
  }
  $341 = HEAP32[$0>>2]|0;
  $342 = (($341) + 18|0);
  $343 = HEAP8[$342>>0]|0;
  $344 = $343&255;
  $345 = ($344|0)==(0);
  if ($345) {
   $346 = HEAP32[$v>>2]|0;
   _sqlite3VdbeCountChanges($346);
  }
  $347 = HEAP32[$0>>2]|0;
  $348 = HEAP32[$iDb>>2]|0;
  _sqlite3BeginWriteOperation($347,1,$348);
  $349 = HEAP32[$pTab>>2]|0;
  $350 = (($349) + 44|0);
  $351 = HEAP8[$350>>0]|0;
  $352 = $351&255;
  $353 = $352 & 16;
  $354 = ($353|0)!=(0);
  $355 = HEAP32[$0>>2]|0;
  if ($354) {
   $356 = HEAP32[$1>>2]|0;
   $357 = HEAP32[$pTab>>2]|0;
   $358 = HEAP32[$2>>2]|0;
   $359 = HEAP32[$pRowidExpr>>2]|0;
   $360 = HEAP32[$aXRef>>2]|0;
   $361 = HEAP32[$3>>2]|0;
   $362 = HEAP32[$4>>2]|0;
   _updateVirtualTable($355,$356,$357,$358,$359,$360,$361,$362);
   HEAP32[$3>>2] = 0;
   HEAP32[$1>>2] = 0;
   _sqlite3AuthContextPop($sContext);
   $983 = HEAP32[$db>>2]|0;
   $984 = HEAP32[$aXRef>>2]|0;
   _sqlite3DbFree($983,$984);
   $985 = HEAP32[$db>>2]|0;
   $986 = HEAP32[$1>>2]|0;
   _sqlite3SrcListDelete($985,$986);
   $987 = HEAP32[$db>>2]|0;
   $988 = HEAP32[$2>>2]|0;
   _sqlite3ExprListDelete($987,$988);
   $989 = HEAP32[$db>>2]|0;
   $990 = HEAP32[$3>>2]|0;
   _sqlite3ExprDelete($989,$990);
   STACKTOP = sp;return;
  }
  $363 = (($355) + 72|0);
  $364 = HEAP32[$363>>2]|0;
  $365 = (($364) + 1)|0;
  HEAP32[$363>>2] = $365;
  HEAP32[$regRowSet>>2] = $365;
  $366 = HEAP32[$0>>2]|0;
  $367 = (($366) + 72|0);
  $368 = HEAP32[$367>>2]|0;
  $369 = (($368) + 1)|0;
  HEAP32[$367>>2] = $369;
  HEAP32[$regNewRowid>>2] = $369;
  HEAP32[$regOldRowid>>2] = $369;
  $370 = HEAP8[$chngPk>>0]|0;
  $371 = $370&255;
  $372 = ($371|0)!=(0);
  $373 = HEAP32[$pTrigger>>2]|0;
  $374 = ($373|0)!=(0|0);
  $or$cond5 = $372 | $374;
  $375 = HEAP32[$hasFK>>2]|0;
  $376 = ($375|0)!=(0);
  $or$cond7 = $or$cond5 | $376;
  if ($or$cond7) {
   $377 = HEAP32[$0>>2]|0;
   $378 = (($377) + 72|0);
   $379 = HEAP32[$378>>2]|0;
   $380 = (($379) + 1)|0;
   HEAP32[$regOld>>2] = $380;
   $381 = HEAP32[$pTab>>2]|0;
   $382 = (($381) + 38|0);
   $383 = HEAP16[$382>>1]|0;
   $384 = $383 << 16 >> 16;
   $385 = HEAP32[$0>>2]|0;
   $386 = (($385) + 72|0);
   $387 = HEAP32[$386>>2]|0;
   $388 = (($387) + ($384))|0;
   HEAP32[$386>>2] = $388;
  }
  $389 = HEAP8[$chngKey>>0]|0;
  $390 = $389&255;
  $391 = ($390|0)!=(0);
  $392 = HEAP32[$pTrigger>>2]|0;
  $393 = ($392|0)!=(0|0);
  $or$cond9 = $391 | $393;
  $394 = HEAP32[$hasFK>>2]|0;
  $395 = ($394|0)!=(0);
  $or$cond11 = $or$cond9 | $395;
  if ($or$cond11) {
   $396 = HEAP32[$0>>2]|0;
   $397 = (($396) + 72|0);
   $398 = HEAP32[$397>>2]|0;
   $399 = (($398) + 1)|0;
   HEAP32[$397>>2] = $399;
   HEAP32[$regNewRowid>>2] = $399;
  }
  $400 = HEAP32[$0>>2]|0;
  $401 = (($400) + 72|0);
  $402 = HEAP32[$401>>2]|0;
  $403 = (($402) + 1)|0;
  HEAP32[$regNew>>2] = $403;
  $404 = HEAP32[$pTab>>2]|0;
  $405 = (($404) + 38|0);
  $406 = HEAP16[$405>>1]|0;
  $407 = $406 << 16 >> 16;
  $408 = HEAP32[$0>>2]|0;
  $409 = (($408) + 72|0);
  $410 = HEAP32[$409>>2]|0;
  $411 = (($410) + ($407))|0;
  HEAP32[$409>>2] = $411;
  $412 = HEAP32[$isView>>2]|0;
  $413 = ($412|0)!=(0);
  if ($413) {
   $414 = HEAP32[$0>>2]|0;
   $415 = HEAP32[$pTab>>2]|0;
   $416 = HEAP32[$415>>2]|0;
   _sqlite3AuthContextPush($414,$sContext,$416);
  }
  $417 = HEAP32[$isView>>2]|0;
  $418 = ($417|0)!=(0);
  if ($418) {
   $419 = HEAP32[$0>>2]|0;
   $420 = HEAP32[$pTab>>2]|0;
   $421 = HEAP32[$3>>2]|0;
   $422 = HEAP32[$iDataCur>>2]|0;
   _sqlite3MaterializeView($419,$420,$421,$422);
  }
  $423 = HEAP32[$3>>2]|0;
  $424 = (_sqlite3ResolveExprNames($sNC,$423)|0);
  $425 = ($424|0)!=(0);
  if ($425) {
   _sqlite3AuthContextPop($sContext);
   $983 = HEAP32[$db>>2]|0;
   $984 = HEAP32[$aXRef>>2]|0;
   _sqlite3DbFree($983,$984);
   $985 = HEAP32[$db>>2]|0;
   $986 = HEAP32[$1>>2]|0;
   _sqlite3SrcListDelete($985,$986);
   $987 = HEAP32[$db>>2]|0;
   $988 = HEAP32[$2>>2]|0;
   _sqlite3ExprListDelete($987,$988);
   $989 = HEAP32[$db>>2]|0;
   $990 = HEAP32[$3>>2]|0;
   _sqlite3ExprDelete($989,$990);
   STACKTOP = sp;return;
  }
  $426 = HEAP32[$pTab>>2]|0;
  $427 = (($426) + 44|0);
  $428 = HEAP8[$427>>0]|0;
  $429 = $428&255;
  $430 = $429 & 32;
  $431 = ($430|0)==(0);
  if ($431) {
   $432 = HEAP32[$v>>2]|0;
   $433 = HEAP32[$regRowSet>>2]|0;
   $434 = HEAP32[$regOldRowid>>2]|0;
   (_sqlite3VdbeAddOp3($432,28,0,$433,$434)|0);
   $435 = HEAP32[$0>>2]|0;
   $436 = HEAP32[$1>>2]|0;
   $437 = HEAP32[$3>>2]|0;
   $438 = HEAP32[$iIdxCur>>2]|0;
   $439 = (_sqlite3WhereBegin($435,$436,$437,0,0,4,$438)|0);
   HEAP32[$pWInfo>>2] = $439;
   $440 = HEAP32[$pWInfo>>2]|0;
   $441 = ($440|0)==(0|0);
   if ($441) {
    _sqlite3AuthContextPop($sContext);
    $983 = HEAP32[$db>>2]|0;
    $984 = HEAP32[$aXRef>>2]|0;
    _sqlite3DbFree($983,$984);
    $985 = HEAP32[$db>>2]|0;
    $986 = HEAP32[$1>>2]|0;
    _sqlite3SrcListDelete($985,$986);
    $987 = HEAP32[$db>>2]|0;
    $988 = HEAP32[$2>>2]|0;
    _sqlite3ExprListDelete($987,$988);
    $989 = HEAP32[$db>>2]|0;
    $990 = HEAP32[$3>>2]|0;
    _sqlite3ExprDelete($989,$990);
    STACKTOP = sp;return;
   }
   $442 = HEAP32[$pWInfo>>2]|0;
   $443 = (_sqlite3WhereOkOnePass($442,$aiCurOnePass)|0);
   HEAP32[$okOnePass>>2] = $443;
   $444 = HEAP32[$v>>2]|0;
   $445 = HEAP32[$iDataCur>>2]|0;
   $446 = HEAP32[$regOldRowid>>2]|0;
   (_sqlite3VdbeAddOp2($444,103,$445,$446)|0);
   $447 = HEAP32[$okOnePass>>2]|0;
   $448 = ($447|0)!=(0);
   if (!($448)) {
    $449 = HEAP32[$v>>2]|0;
    $450 = HEAP32[$regRowSet>>2]|0;
    $451 = HEAP32[$regOldRowid>>2]|0;
    (_sqlite3VdbeAddOp2($449,128,$450,$451)|0);
   }
   $452 = HEAP32[$pWInfo>>2]|0;
   _sqlite3WhereEnd($452);
  } else {
   $453 = HEAP32[$pPk>>2]|0;
   $454 = (($453) + 50|0);
   $455 = HEAP16[$454>>1]|0;
   HEAP16[$nPk>>1] = $455;
   $456 = HEAP32[$0>>2]|0;
   $457 = (($456) + 72|0);
   $458 = HEAP32[$457>>2]|0;
   $459 = (($458) + 1)|0;
   HEAP32[$iPk>>2] = $459;
   $460 = HEAP16[$nPk>>1]|0;
   $461 = $460 << 16 >> 16;
   $462 = HEAP32[$0>>2]|0;
   $463 = (($462) + 72|0);
   $464 = HEAP32[$463>>2]|0;
   $465 = (($464) + ($461))|0;
   HEAP32[$463>>2] = $465;
   $466 = HEAP32[$0>>2]|0;
   $467 = (($466) + 72|0);
   $468 = HEAP32[$467>>2]|0;
   $469 = (($468) + 1)|0;
   HEAP32[$467>>2] = $469;
   HEAP32[$regKey>>2] = $469;
   $470 = HEAP32[$0>>2]|0;
   $471 = (($470) + 68|0);
   $472 = HEAP32[$471>>2]|0;
   $473 = (($472) + 1)|0;
   HEAP32[$471>>2] = $473;
   HEAP32[$iEph>>2] = $472;
   $474 = HEAP32[$v>>2]|0;
   $475 = HEAP32[$iPk>>2]|0;
   (_sqlite3VdbeAddOp2($474,28,0,$475)|0);
   $476 = HEAP32[$v>>2]|0;
   $477 = HEAP32[$iEph>>2]|0;
   $478 = HEAP16[$nPk>>1]|0;
   $479 = $478 << 16 >> 16;
   $480 = (_sqlite3VdbeAddOp2($476,57,$477,$479)|0);
   HEAP32[$addrOpen>>2] = $480;
   $481 = HEAP32[$0>>2]|0;
   $482 = HEAP32[$pPk>>2]|0;
   _sqlite3VdbeSetP4KeyInfo($481,$482);
   $483 = HEAP32[$0>>2]|0;
   $484 = HEAP32[$1>>2]|0;
   $485 = HEAP32[$3>>2]|0;
   $486 = HEAP32[$iIdxCur>>2]|0;
   $487 = (_sqlite3WhereBegin($483,$484,$485,0,0,4,$486)|0);
   HEAP32[$pWInfo>>2] = $487;
   $488 = HEAP32[$pWInfo>>2]|0;
   $489 = ($488|0)==(0|0);
   if ($489) {
    _sqlite3AuthContextPop($sContext);
    $983 = HEAP32[$db>>2]|0;
    $984 = HEAP32[$aXRef>>2]|0;
    _sqlite3DbFree($983,$984);
    $985 = HEAP32[$db>>2]|0;
    $986 = HEAP32[$1>>2]|0;
    _sqlite3SrcListDelete($985,$986);
    $987 = HEAP32[$db>>2]|0;
    $988 = HEAP32[$2>>2]|0;
    _sqlite3ExprListDelete($987,$988);
    $989 = HEAP32[$db>>2]|0;
    $990 = HEAP32[$3>>2]|0;
    _sqlite3ExprDelete($989,$990);
    STACKTOP = sp;return;
   }
   $490 = HEAP32[$pWInfo>>2]|0;
   $491 = (_sqlite3WhereOkOnePass($490,$aiCurOnePass)|0);
   HEAP32[$okOnePass>>2] = $491;
   HEAP32[$i>>2] = 0;
   while(1) {
    $492 = HEAP32[$i>>2]|0;
    $493 = HEAP16[$nPk>>1]|0;
    $494 = $493 << 16 >> 16;
    $495 = ($492|0)<($494|0);
    if (!($495)) {
     break;
    }
    $496 = HEAP32[$v>>2]|0;
    $497 = HEAP32[$pTab>>2]|0;
    $498 = HEAP32[$iDataCur>>2]|0;
    $499 = HEAP32[$i>>2]|0;
    $500 = HEAP32[$pPk>>2]|0;
    $501 = (($500) + 4|0);
    $502 = HEAP32[$501>>2]|0;
    $503 = (($502) + ($499<<1)|0);
    $504 = HEAP16[$503>>1]|0;
    $505 = $504 << 16 >> 16;
    $506 = HEAP32[$iPk>>2]|0;
    $507 = HEAP32[$i>>2]|0;
    $508 = (($506) + ($507))|0;
    _sqlite3ExprCodeGetColumnOfTable($496,$497,$498,$505,$508);
    $509 = HEAP32[$i>>2]|0;
    $510 = (($509) + 1)|0;
    HEAP32[$i>>2] = $510;
   }
   $511 = HEAP32[$okOnePass>>2]|0;
   $512 = ($511|0)!=(0);
   $513 = HEAP32[$v>>2]|0;
   if ($512) {
    $514 = HEAP32[$addrOpen>>2]|0;
    _sqlite3VdbeChangeToNoop($513,$514);
    $515 = HEAP16[$nPk>>1]|0;
    $516 = $515 << 16 >> 16;
    HEAP32[$nKey>>2] = $516;
    $517 = HEAP32[$iPk>>2]|0;
    HEAP32[$regKey>>2] = $517;
   } else {
    $518 = HEAP32[$iPk>>2]|0;
    $519 = HEAP16[$nPk>>1]|0;
    $520 = $519 << 16 >> 16;
    $521 = HEAP32[$regKey>>2]|0;
    $522 = HEAP32[$v>>2]|0;
    $523 = HEAP32[$pPk>>2]|0;
    $524 = (_sqlite3IndexAffinityStr($522,$523)|0);
    $525 = HEAP16[$nPk>>1]|0;
    $526 = $525 << 16 >> 16;
    (_sqlite3VdbeAddOp4($513,49,$518,$520,$521,$524,$526)|0);
    $527 = HEAP32[$v>>2]|0;
    $528 = HEAP32[$iEph>>2]|0;
    $529 = HEAP32[$regKey>>2]|0;
    (_sqlite3VdbeAddOp2($527,110,$528,$529)|0);
   }
   $530 = HEAP32[$pWInfo>>2]|0;
   _sqlite3WhereEnd($530);
  }
  $531 = HEAP32[$db>>2]|0;
  $532 = (($531) + 24|0);
  $533 = HEAP32[$532>>2]|0;
  $534 = $533 & 128;
  $535 = ($534|0)!=(0);
  if ($535) {
   $536 = HEAP32[$0>>2]|0;
   $537 = (($536) + 412|0);
   $538 = HEAP32[$537>>2]|0;
   $539 = ($538|0)!=(0|0);
   if (!($539)) {
    $540 = HEAP32[$0>>2]|0;
    $541 = (($540) + 72|0);
    $542 = HEAP32[$541>>2]|0;
    $543 = (($542) + 1)|0;
    HEAP32[$541>>2] = $543;
    HEAP32[$regRowCount>>2] = $543;
    $544 = HEAP32[$v>>2]|0;
    $545 = HEAP32[$regRowCount>>2]|0;
    (_sqlite3VdbeAddOp2($544,25,0,$545)|0);
   }
  }
  $546 = HEAP32[$v>>2]|0;
  $547 = (_sqlite3VdbeMakeLabel($546)|0);
  HEAP32[$labelBreak>>2] = $547;
  $548 = HEAP32[$isView>>2]|0;
  $549 = ($548|0)!=(0);
  if (!($549)) {
   $550 = HEAP32[$4>>2]|0;
   $551 = ($550|0)==(5);
   L137: do {
    if ($551) {
     $552 = HEAP32[$aToOpen>>2]|0;
     $553 = HEAP32[$nIdx>>2]|0;
     $554 = (($553) + 1)|0;
     _memset(($552|0),1,($554|0))|0;
    } else {
     $555 = HEAP32[$pTab>>2]|0;
     $556 = (($555) + 8|0);
     $557 = HEAP32[$556>>2]|0;
     HEAP32[$pIdx>>2] = $557;
     while(1) {
      $558 = HEAP32[$pIdx>>2]|0;
      $559 = ($558|0)!=(0|0);
      if (!($559)) {
       break L137;
      }
      $560 = HEAP32[$pIdx>>2]|0;
      $561 = (($560) + 54|0);
      $562 = HEAP8[$561>>0]|0;
      $563 = $562&255;
      $564 = ($563|0)==(5);
      if ($564) {
       break;
      }
      $568 = HEAP32[$pIdx>>2]|0;
      $569 = (($568) + 20|0);
      $570 = HEAP32[$569>>2]|0;
      HEAP32[$pIdx>>2] = $570;
     }
     $565 = HEAP32[$aToOpen>>2]|0;
     $566 = HEAP32[$nIdx>>2]|0;
     $567 = (($566) + 1)|0;
     _memset(($565|0),1,($567|0))|0;
    }
   } while(0);
   $571 = HEAP32[$okOnePass>>2]|0;
   $572 = ($571|0)!=(0);
   do {
    if ($572) {
     $573 = HEAP32[$aiCurOnePass>>2]|0;
     $574 = ($573|0)>=(0);
     if ($574) {
      $575 = HEAP32[$aiCurOnePass>>2]|0;
      $576 = HEAP32[$iBaseCur>>2]|0;
      $577 = (($575) - ($576))|0;
      $578 = HEAP32[$aToOpen>>2]|0;
      $579 = (($578) + ($577)|0);
      HEAP8[$579>>0] = 0;
     }
     $580 = (($aiCurOnePass) + 4|0);
     $581 = HEAP32[$580>>2]|0;
     $582 = ($581|0)>=(0);
     if (!($582)) {
      break;
     }
     $583 = (($aiCurOnePass) + 4|0);
     $584 = HEAP32[$583>>2]|0;
     $585 = HEAP32[$iBaseCur>>2]|0;
     $586 = (($584) - ($585))|0;
     $587 = HEAP32[$aToOpen>>2]|0;
     $588 = (($587) + ($586)|0);
     HEAP8[$588>>0] = 0;
    }
   } while(0);
   $589 = HEAP32[$0>>2]|0;
   $590 = HEAP32[$pTab>>2]|0;
   $591 = HEAP32[$iBaseCur>>2]|0;
   $592 = HEAP32[$aToOpen>>2]|0;
   (_sqlite3OpenTableAndIndices($589,$590,55,$591,$592,0,0)|0);
  }
  $593 = HEAP32[$okOnePass>>2]|0;
  $594 = ($593|0)!=(0);
  do {
   if ($594) {
    $595 = HEAP32[$iDataCur>>2]|0;
    $596 = HEAP32[$iBaseCur>>2]|0;
    $597 = (($595) - ($596))|0;
    $598 = HEAP32[$aToOpen>>2]|0;
    $599 = (($598) + ($597)|0);
    $600 = HEAP8[$599>>0]|0;
    $601 = $600&255;
    $602 = ($601|0)==(0);
    $603 = HEAP32[$isView>>2]|0;
    $604 = ($603|0)!=(0);
    $or$cond13 = $602 | $604;
    if (!($or$cond13)) {
     $605 = HEAP32[$v>>2]|0;
     $606 = HEAP32[$iDataCur>>2]|0;
     $607 = HEAP32[$labelBreak>>2]|0;
     $608 = HEAP32[$regKey>>2]|0;
     $609 = HEAP32[$nKey>>2]|0;
     (_sqlite3VdbeAddOp4Int($605,68,$606,$607,$608,$609)|0);
    }
    $610 = HEAP32[$labelBreak>>2]|0;
    HEAP32[$labelContinue>>2] = $610;
    $611 = HEAP32[$v>>2]|0;
    $612 = HEAP32[$pPk>>2]|0;
    $613 = ($612|0)!=(0|0);
    $614 = HEAP32[$regKey>>2]|0;
    $615 = HEAP32[$regOldRowid>>2]|0;
    $616 = $613 ? $614 : $615;
    $617 = HEAP32[$labelBreak>>2]|0;
    (_sqlite3VdbeAddOp2($611,76,$616,$617)|0);
   } else {
    $618 = HEAP32[$pPk>>2]|0;
    $619 = ($618|0)!=(0|0);
    $620 = HEAP32[$v>>2]|0;
    if ($619) {
     $621 = (_sqlite3VdbeMakeLabel($620)|0);
     HEAP32[$labelContinue>>2] = $621;
     $622 = HEAP32[$v>>2]|0;
     $623 = HEAP32[$iEph>>2]|0;
     $624 = HEAP32[$labelBreak>>2]|0;
     (_sqlite3VdbeAddOp2($622,108,$623,$624)|0);
     $625 = HEAP32[$v>>2]|0;
     $626 = HEAP32[$iEph>>2]|0;
     $627 = HEAP32[$regKey>>2]|0;
     $628 = (_sqlite3VdbeAddOp2($625,101,$626,$627)|0);
     HEAP32[$addrTop>>2] = $628;
     $629 = HEAP32[$v>>2]|0;
     $630 = HEAP32[$iDataCur>>2]|0;
     $631 = HEAP32[$labelContinue>>2]|0;
     $632 = HEAP32[$regKey>>2]|0;
     (_sqlite3VdbeAddOp4Int($629,68,$630,$631,$632,0)|0);
     break;
    } else {
     $633 = HEAP32[$regRowSet>>2]|0;
     $634 = HEAP32[$labelBreak>>2]|0;
     $635 = HEAP32[$regOldRowid>>2]|0;
     $636 = (_sqlite3VdbeAddOp3($620,129,$633,$634,$635)|0);
     HEAP32[$labelContinue>>2] = $636;
     $637 = HEAP32[$v>>2]|0;
     $638 = HEAP32[$iDataCur>>2]|0;
     $639 = HEAP32[$labelContinue>>2]|0;
     $640 = HEAP32[$regOldRowid>>2]|0;
     (_sqlite3VdbeAddOp3($637,70,$638,$639,$640)|0);
     break;
    }
   }
  } while(0);
  $641 = HEAP8[$chngRowid>>0]|0;
  $642 = ($641<<24>>24)!=(0);
  if ($642) {
   $643 = HEAP32[$0>>2]|0;
   $644 = HEAP32[$pRowidExpr>>2]|0;
   $645 = HEAP32[$regNewRowid>>2]|0;
   _sqlite3ExprCode($643,$644,$645);
   $646 = HEAP32[$v>>2]|0;
   $647 = HEAP32[$regNewRowid>>2]|0;
   (_sqlite3VdbeAddOp1($646,38,$647)|0);
  }
  $648 = HEAP8[$chngPk>>0]|0;
  $649 = $648&255;
  $650 = ($649|0)!=(0);
  $651 = HEAP32[$hasFK>>2]|0;
  $652 = ($651|0)!=(0);
  $or$cond15 = $650 | $652;
  $653 = HEAP32[$pTrigger>>2]|0;
  $654 = ($653|0)!=(0|0);
  $or$cond17 = $or$cond15 | $654;
  do {
   if ($or$cond17) {
    $655 = HEAP32[$hasFK>>2]|0;
    $656 = ($655|0)!=(0);
    if ($656) {
     $657 = HEAP32[$0>>2]|0;
     $658 = HEAP32[$pTab>>2]|0;
     $659 = (_sqlite3FkOldmask($657,$658)|0);
     $660 = $659;
    } else {
     $660 = 0;
    }
    HEAP32[$oldmask>>2] = $660;
    $661 = HEAP32[$0>>2]|0;
    $662 = HEAP32[$pTrigger>>2]|0;
    $663 = HEAP32[$2>>2]|0;
    $664 = HEAP32[$pTab>>2]|0;
    $665 = HEAP32[$4>>2]|0;
    $666 = (_sqlite3TriggerColmask($661,$662,$663,0,3,$664,$665)|0);
    $667 = HEAP32[$oldmask>>2]|0;
    $668 = $667 | $666;
    HEAP32[$oldmask>>2] = $668;
    HEAP32[$i>>2] = 0;
    while(1) {
     $669 = HEAP32[$i>>2]|0;
     $670 = HEAP32[$pTab>>2]|0;
     $671 = (($670) + 38|0);
     $672 = HEAP16[$671>>1]|0;
     $673 = $672 << 16 >> 16;
     $674 = ($669|0)<($673|0);
     if (!($674)) {
      break;
     }
     $675 = HEAP32[$oldmask>>2]|0;
     $676 = ($675|0)==(-1);
     do {
      if ($676) {
       label = 117;
      } else {
       $677 = HEAP32[$i>>2]|0;
       $678 = ($677|0)<(32);
       if ($678) {
        $679 = HEAP32[$oldmask>>2]|0;
        $680 = HEAP32[$i>>2]|0;
        $681 = 1 << $680;
        $682 = $679 & $681;
        $683 = ($682|0)!=(0);
        if ($683) {
         label = 117;
         break;
        }
       }
       $684 = HEAP32[$i>>2]|0;
       $685 = HEAP32[$pTab>>2]|0;
       $686 = (($685) + 4|0);
       $687 = HEAP32[$686>>2]|0;
       $688 = (($687) + (($684*24)|0)|0);
       $689 = (($688) + 23|0);
       $690 = HEAP8[$689>>0]|0;
       $691 = $690&255;
       $692 = $691 & 1;
       $693 = ($692|0)!=(0);
       if ($693) {
        label = 117;
        break;
       }
       $701 = HEAP32[$v>>2]|0;
       $702 = HEAP32[$regOld>>2]|0;
       $703 = HEAP32[$i>>2]|0;
       $704 = (($702) + ($703))|0;
       (_sqlite3VdbeAddOp2($701,28,0,$704)|0);
      }
     } while(0);
     if ((label|0) == 117) {
      label = 0;
      $694 = HEAP32[$v>>2]|0;
      $695 = HEAP32[$pTab>>2]|0;
      $696 = HEAP32[$iDataCur>>2]|0;
      $697 = HEAP32[$i>>2]|0;
      $698 = HEAP32[$regOld>>2]|0;
      $699 = HEAP32[$i>>2]|0;
      $700 = (($698) + ($699))|0;
      _sqlite3ExprCodeGetColumnOfTable($694,$695,$696,$697,$700);
     }
     $705 = HEAP32[$i>>2]|0;
     $706 = (($705) + 1)|0;
     HEAP32[$i>>2] = $706;
    }
    $707 = HEAP8[$chngRowid>>0]|0;
    $708 = $707&255;
    $709 = ($708|0)==(0);
    $710 = HEAP32[$pPk>>2]|0;
    $711 = ($710|0)==(0|0);
    $or$cond19 = $709 & $711;
    if (!($or$cond19)) {
     break;
    }
    $712 = HEAP32[$v>>2]|0;
    $713 = HEAP32[$regOldRowid>>2]|0;
    $714 = HEAP32[$regNewRowid>>2]|0;
    (_sqlite3VdbeAddOp2($712,33,$713,$714)|0);
   }
  } while(0);
  $715 = HEAP32[$0>>2]|0;
  $716 = HEAP32[$pTrigger>>2]|0;
  $717 = HEAP32[$2>>2]|0;
  $718 = HEAP32[$pTab>>2]|0;
  $719 = HEAP32[$4>>2]|0;
  $720 = (_sqlite3TriggerColmask($715,$716,$717,1,1,$718,$719)|0);
  HEAP32[$newmask>>2] = $720;
  HEAP32[$i>>2] = 0;
  while(1) {
   $721 = HEAP32[$i>>2]|0;
   $722 = HEAP32[$pTab>>2]|0;
   $723 = (($722) + 38|0);
   $724 = HEAP16[$723>>1]|0;
   $725 = $724 << 16 >> 16;
   $726 = ($721|0)<($725|0);
   if (!($726)) {
    break;
   }
   $727 = HEAP32[$i>>2]|0;
   $728 = HEAP32[$pTab>>2]|0;
   $729 = (($728) + 36|0);
   $730 = HEAP16[$729>>1]|0;
   $731 = $730 << 16 >> 16;
   $732 = ($727|0)==($731|0);
   L190: do {
    if ($732) {
     $733 = HEAP32[$v>>2]|0;
     $734 = HEAP32[$regNew>>2]|0;
     $735 = HEAP32[$i>>2]|0;
     $736 = (($734) + ($735))|0;
     (_sqlite3VdbeAddOp2($733,28,0,$736)|0);
    } else {
     $737 = HEAP32[$i>>2]|0;
     $738 = HEAP32[$aXRef>>2]|0;
     $739 = (($738) + ($737<<2)|0);
     $740 = HEAP32[$739>>2]|0;
     HEAP32[$j>>2] = $740;
     $741 = HEAP32[$j>>2]|0;
     $742 = ($741|0)>=(0);
     if ($742) {
      $743 = HEAP32[$0>>2]|0;
      $744 = HEAP32[$j>>2]|0;
      $745 = HEAP32[$2>>2]|0;
      $746 = (($745) + 4|0);
      $747 = HEAP32[$746>>2]|0;
      $748 = (($747) + (($744*20)|0)|0);
      $749 = HEAP32[$748>>2]|0;
      $750 = HEAP32[$regNew>>2]|0;
      $751 = HEAP32[$i>>2]|0;
      $752 = (($750) + ($751))|0;
      _sqlite3ExprCode($743,$749,$752);
      break;
     }
     $753 = HEAP32[$tmask>>2]|0;
     $754 = $753 & 1;
     $755 = (0)==($754|0);
     $756 = HEAP32[$i>>2]|0;
     $757 = ($756|0)>(31);
     $or$cond21 = $755 | $757;
     do {
      if (!($or$cond21)) {
       $758 = HEAP32[$newmask>>2]|0;
       $759 = HEAP32[$i>>2]|0;
       $760 = 1 << $759;
       $761 = $758 & $760;
       $762 = ($761|0)!=(0);
       if ($762) {
        break;
       }
       $770 = HEAP32[$v>>2]|0;
       $771 = HEAP32[$regNew>>2]|0;
       $772 = HEAP32[$i>>2]|0;
       $773 = (($771) + ($772))|0;
       (_sqlite3VdbeAddOp2($770,28,0,$773)|0);
       break L190;
      }
     } while(0);
     $763 = HEAP32[$v>>2]|0;
     $764 = HEAP32[$pTab>>2]|0;
     $765 = HEAP32[$iDataCur>>2]|0;
     $766 = HEAP32[$i>>2]|0;
     $767 = HEAP32[$regNew>>2]|0;
     $768 = HEAP32[$i>>2]|0;
     $769 = (($767) + ($768))|0;
     _sqlite3ExprCodeGetColumnOfTable($763,$764,$765,$766,$769);
    }
   } while(0);
   $774 = HEAP32[$i>>2]|0;
   $775 = (($774) + 1)|0;
   HEAP32[$i>>2] = $775;
  }
  $776 = HEAP32[$tmask>>2]|0;
  $777 = $776 & 1;
  $778 = ($777|0)!=(0);
  L202: do {
   if ($778) {
    $779 = HEAP32[$v>>2]|0;
    $780 = HEAP32[$pTab>>2]|0;
    $781 = HEAP32[$regNew>>2]|0;
    _sqlite3TableAffinity($779,$780,$781);
    $782 = HEAP32[$0>>2]|0;
    $783 = HEAP32[$pTrigger>>2]|0;
    $784 = HEAP32[$2>>2]|0;
    $785 = HEAP32[$pTab>>2]|0;
    $786 = HEAP32[$regOldRowid>>2]|0;
    $787 = HEAP32[$4>>2]|0;
    $788 = HEAP32[$labelContinue>>2]|0;
    _sqlite3CodeRowTrigger($782,$783,110,$784,1,$785,$786,$787,$788);
    $789 = HEAP32[$pPk>>2]|0;
    $790 = ($789|0)!=(0|0);
    $791 = HEAP32[$v>>2]|0;
    $792 = HEAP32[$iDataCur>>2]|0;
    $793 = HEAP32[$labelContinue>>2]|0;
    if ($790) {
     $794 = HEAP32[$regKey>>2]|0;
     $795 = HEAP32[$nKey>>2]|0;
     (_sqlite3VdbeAddOp4Int($791,68,$792,$793,$794,$795)|0);
    } else {
     $796 = HEAP32[$regOldRowid>>2]|0;
     (_sqlite3VdbeAddOp3($791,70,$792,$793,$796)|0);
    }
    HEAP32[$i>>2] = 0;
    while(1) {
     $797 = HEAP32[$i>>2]|0;
     $798 = HEAP32[$pTab>>2]|0;
     $799 = (($798) + 38|0);
     $800 = HEAP16[$799>>1]|0;
     $801 = $800 << 16 >> 16;
     $802 = ($797|0)<($801|0);
     if (!($802)) {
      break L202;
     }
     $803 = HEAP32[$i>>2]|0;
     $804 = HEAP32[$aXRef>>2]|0;
     $805 = (($804) + ($803<<2)|0);
     $806 = HEAP32[$805>>2]|0;
     $807 = ($806|0)<(0);
     do {
      if ($807) {
       $808 = HEAP32[$i>>2]|0;
       $809 = HEAP32[$pTab>>2]|0;
       $810 = (($809) + 36|0);
       $811 = HEAP16[$810>>1]|0;
       $812 = $811 << 16 >> 16;
       $813 = ($808|0)!=($812|0);
       if (!($813)) {
        break;
       }
       $814 = HEAP32[$v>>2]|0;
       $815 = HEAP32[$pTab>>2]|0;
       $816 = HEAP32[$iDataCur>>2]|0;
       $817 = HEAP32[$i>>2]|0;
       $818 = HEAP32[$regNew>>2]|0;
       $819 = HEAP32[$i>>2]|0;
       $820 = (($818) + ($819))|0;
       _sqlite3ExprCodeGetColumnOfTable($814,$815,$816,$817,$820);
      }
     } while(0);
     $821 = HEAP32[$i>>2]|0;
     $822 = (($821) + 1)|0;
     HEAP32[$i>>2] = $822;
    }
   }
  } while(0);
  $823 = HEAP32[$isView>>2]|0;
  $824 = ($823|0)!=(0);
  do {
   if (!($824)) {
    HEAP32[$j1>>2] = 0;
    HEAP32[$bReplace>>2] = 0;
    $825 = HEAP32[$0>>2]|0;
    $826 = HEAP32[$pTab>>2]|0;
    $827 = HEAP32[$aRegIdx>>2]|0;
    $828 = HEAP32[$iDataCur>>2]|0;
    $829 = HEAP32[$iIdxCur>>2]|0;
    $830 = HEAP32[$regNewRowid>>2]|0;
    $831 = HEAP32[$regOldRowid>>2]|0;
    $832 = HEAP8[$chngKey>>0]|0;
    $833 = HEAP32[$4>>2]|0;
    $834 = $833&255;
    $835 = HEAP32[$labelContinue>>2]|0;
    _sqlite3GenerateConstraintChecks($825,$826,$827,$828,$829,$830,$831,$832,$834,$835,$bReplace);
    $836 = HEAP32[$hasFK>>2]|0;
    $837 = ($836|0)!=(0);
    if ($837) {
     $838 = HEAP32[$0>>2]|0;
     $839 = HEAP32[$pTab>>2]|0;
     $840 = HEAP32[$regOldRowid>>2]|0;
     $841 = HEAP32[$aXRef>>2]|0;
     $842 = HEAP8[$chngKey>>0]|0;
     $843 = $842&255;
     _sqlite3FkCheck($838,$839,$840,0,$841,$843);
    }
    $844 = HEAP32[$bReplace>>2]|0;
    $845 = ($844|0)!=(0);
    if ($845) {
     label = 148;
    } else {
     $846 = HEAP8[$chngKey>>0]|0;
     $847 = $846&255;
     $848 = ($847|0)!=(0);
     if ($848) {
      label = 148;
     }
    }
    do {
     if ((label|0) == 148) {
      $849 = HEAP32[$pPk>>2]|0;
      $850 = ($849|0)!=(0|0);
      $851 = HEAP32[$v>>2]|0;
      $852 = HEAP32[$iDataCur>>2]|0;
      if ($850) {
       $853 = HEAP32[$regKey>>2]|0;
       $854 = HEAP32[$nKey>>2]|0;
       $855 = (_sqlite3VdbeAddOp4Int($851,68,$852,0,$853,$854)|0);
       HEAP32[$j1>>2] = $855;
       break;
      } else {
       $856 = HEAP32[$regOldRowid>>2]|0;
       $857 = (_sqlite3VdbeAddOp3($851,70,$852,0,$856)|0);
       HEAP32[$j1>>2] = $857;
       break;
      }
     }
    } while(0);
    $858 = HEAP32[$0>>2]|0;
    $859 = HEAP32[$pTab>>2]|0;
    $860 = HEAP32[$iDataCur>>2]|0;
    $861 = HEAP32[$iIdxCur>>2]|0;
    $862 = HEAP32[$aRegIdx>>2]|0;
    _sqlite3GenerateRowIndexDelete($858,$859,$860,$861,$862);
    $863 = HEAP32[$hasFK>>2]|0;
    $864 = ($863|0)!=(0);
    if ($864) {
     label = 153;
    } else {
     $865 = HEAP8[$chngKey>>0]|0;
     $866 = $865&255;
     $867 = ($866|0)!=(0);
     $868 = HEAP32[$pPk>>2]|0;
     $869 = ($868|0)!=(0|0);
     $or$cond23 = $867 | $869;
     if ($or$cond23) {
      label = 153;
     }
    }
    if ((label|0) == 153) {
     $870 = HEAP32[$v>>2]|0;
     $871 = HEAP32[$iDataCur>>2]|0;
     (_sqlite3VdbeAddOp2($870,95,$871,0)|0);
    }
    $872 = HEAP32[$bReplace>>2]|0;
    $873 = ($872|0)!=(0);
    if ($873) {
     label = 156;
    } else {
     $874 = HEAP8[$chngKey>>0]|0;
     $875 = $874&255;
     $876 = ($875|0)!=(0);
     if ($876) {
      label = 156;
     }
    }
    if ((label|0) == 156) {
     $877 = HEAP32[$v>>2]|0;
     $878 = HEAP32[$j1>>2]|0;
     _sqlite3VdbeJumpHere($877,$878);
    }
    $879 = HEAP32[$hasFK>>2]|0;
    $880 = ($879|0)!=(0);
    if ($880) {
     $881 = HEAP32[$0>>2]|0;
     $882 = HEAP32[$pTab>>2]|0;
     $883 = HEAP32[$regNewRowid>>2]|0;
     $884 = HEAP32[$aXRef>>2]|0;
     $885 = HEAP8[$chngKey>>0]|0;
     $886 = $885&255;
     _sqlite3FkCheck($881,$882,0,$883,$884,$886);
    }
    $887 = HEAP32[$0>>2]|0;
    $888 = HEAP32[$pTab>>2]|0;
    $889 = HEAP32[$iDataCur>>2]|0;
    $890 = HEAP32[$iIdxCur>>2]|0;
    $891 = HEAP32[$regNewRowid>>2]|0;
    $892 = HEAP32[$aRegIdx>>2]|0;
    _sqlite3CompleteInsertion($887,$888,$889,$890,$891,$892,1,0,0);
    $893 = HEAP32[$hasFK>>2]|0;
    $894 = ($893|0)!=(0);
    if (!($894)) {
     break;
    }
    $895 = HEAP32[$0>>2]|0;
    $896 = HEAP32[$pTab>>2]|0;
    $897 = HEAP32[$2>>2]|0;
    $898 = HEAP32[$regOldRowid>>2]|0;
    $899 = HEAP32[$aXRef>>2]|0;
    $900 = HEAP8[$chngKey>>0]|0;
    $901 = $900&255;
    _sqlite3FkActions($895,$896,$897,$898,$899,$901);
   }
  } while(0);
  $902 = HEAP32[$db>>2]|0;
  $903 = (($902) + 24|0);
  $904 = HEAP32[$903>>2]|0;
  $905 = $904 & 128;
  $906 = ($905|0)!=(0);
  do {
   if ($906) {
    $907 = HEAP32[$0>>2]|0;
    $908 = (($907) + 412|0);
    $909 = HEAP32[$908>>2]|0;
    $910 = ($909|0)!=(0|0);
    if ($910) {
     break;
    }
    $911 = HEAP32[$v>>2]|0;
    $912 = HEAP32[$regRowCount>>2]|0;
    (_sqlite3VdbeAddOp2($911,37,$912,1)|0);
   }
  } while(0);
  $913 = HEAP32[$0>>2]|0;
  $914 = HEAP32[$pTrigger>>2]|0;
  $915 = HEAP32[$2>>2]|0;
  $916 = HEAP32[$pTab>>2]|0;
  $917 = HEAP32[$regOldRowid>>2]|0;
  $918 = HEAP32[$4>>2]|0;
  $919 = HEAP32[$labelContinue>>2]|0;
  _sqlite3CodeRowTrigger($913,$914,110,$915,2,$916,$917,$918,$919);
  $920 = HEAP32[$okOnePass>>2]|0;
  $921 = ($920|0)!=(0);
  do {
   if (!($921)) {
    $922 = HEAP32[$pPk>>2]|0;
    $923 = ($922|0)!=(0|0);
    $924 = HEAP32[$v>>2]|0;
    $925 = HEAP32[$labelContinue>>2]|0;
    if ($923) {
     _sqlite3VdbeResolveLabel($924,$925);
     $926 = HEAP32[$v>>2]|0;
     $927 = HEAP32[$iEph>>2]|0;
     $928 = HEAP32[$addrTop>>2]|0;
     (_sqlite3VdbeAddOp2($926,9,$927,$928)|0);
     break;
    } else {
     (_sqlite3VdbeAddOp2($924,16,0,$925)|0);
     break;
    }
   }
  } while(0);
  $929 = HEAP32[$v>>2]|0;
  $930 = HEAP32[$labelBreak>>2]|0;
  _sqlite3VdbeResolveLabel($929,$930);
  HEAP32[$i>>2] = 0;
  $931 = HEAP32[$pTab>>2]|0;
  $932 = (($931) + 8|0);
  $933 = HEAP32[$932>>2]|0;
  HEAP32[$pIdx>>2] = $933;
  while(1) {
   $934 = HEAP32[$pIdx>>2]|0;
   $935 = ($934|0)!=(0|0);
   if (!($935)) {
    break;
   }
   $936 = HEAP32[$i>>2]|0;
   $937 = (($936) + 1)|0;
   $938 = HEAP32[$aToOpen>>2]|0;
   $939 = (($938) + ($937)|0);
   $940 = HEAP8[$939>>0]|0;
   $941 = ($940<<24>>24)!=(0);
   if ($941) {
    $942 = HEAP32[$v>>2]|0;
    $943 = HEAP32[$iIdxCur>>2]|0;
    $944 = HEAP32[$i>>2]|0;
    $945 = (($943) + ($944))|0;
    (_sqlite3VdbeAddOp2($942,61,$945,0)|0);
   }
   $946 = HEAP32[$pIdx>>2]|0;
   $947 = (($946) + 20|0);
   $948 = HEAP32[$947>>2]|0;
   HEAP32[$pIdx>>2] = $948;
   $949 = HEAP32[$i>>2]|0;
   $950 = (($949) + 1)|0;
   HEAP32[$i>>2] = $950;
  }
  $951 = HEAP32[$iDataCur>>2]|0;
  $952 = HEAP32[$iIdxCur>>2]|0;
  $953 = ($951|0)<($952|0);
  if ($953) {
   $954 = HEAP32[$v>>2]|0;
   $955 = HEAP32[$iDataCur>>2]|0;
   (_sqlite3VdbeAddOp2($954,61,$955,0)|0);
  }
  $956 = HEAP32[$0>>2]|0;
  $957 = (($956) + 18|0);
  $958 = HEAP8[$957>>0]|0;
  $959 = $958&255;
  $960 = ($959|0)==(0);
  do {
   if ($960) {
    $961 = HEAP32[$0>>2]|0;
    $962 = (($961) + 412|0);
    $963 = HEAP32[$962>>2]|0;
    $964 = ($963|0)==(0|0);
    if (!($964)) {
     break;
    }
    $965 = HEAP32[$0>>2]|0;
    _sqlite3AutoincrementEnd($965);
   }
  } while(0);
  $966 = HEAP32[$db>>2]|0;
  $967 = (($966) + 24|0);
  $968 = HEAP32[$967>>2]|0;
  $969 = $968 & 128;
  $970 = ($969|0)!=(0);
  if (!($970)) {
   _sqlite3AuthContextPop($sContext);
   $983 = HEAP32[$db>>2]|0;
   $984 = HEAP32[$aXRef>>2]|0;
   _sqlite3DbFree($983,$984);
   $985 = HEAP32[$db>>2]|0;
   $986 = HEAP32[$1>>2]|0;
   _sqlite3SrcListDelete($985,$986);
   $987 = HEAP32[$db>>2]|0;
   $988 = HEAP32[$2>>2]|0;
   _sqlite3ExprListDelete($987,$988);
   $989 = HEAP32[$db>>2]|0;
   $990 = HEAP32[$3>>2]|0;
   _sqlite3ExprDelete($989,$990);
   STACKTOP = sp;return;
  }
  $971 = HEAP32[$0>>2]|0;
  $972 = (($971) + 412|0);
  $973 = HEAP32[$972>>2]|0;
  $974 = ($973|0)!=(0|0);
  if ($974) {
   _sqlite3AuthContextPop($sContext);
   $983 = HEAP32[$db>>2]|0;
   $984 = HEAP32[$aXRef>>2]|0;
   _sqlite3DbFree($983,$984);
   $985 = HEAP32[$db>>2]|0;
   $986 = HEAP32[$1>>2]|0;
   _sqlite3SrcListDelete($985,$986);
   $987 = HEAP32[$db>>2]|0;
   $988 = HEAP32[$2>>2]|0;
   _sqlite3ExprListDelete($987,$988);
   $989 = HEAP32[$db>>2]|0;
   $990 = HEAP32[$3>>2]|0;
   _sqlite3ExprDelete($989,$990);
   STACKTOP = sp;return;
  }
  $975 = HEAP32[$0>>2]|0;
  $976 = (($975) + 18|0);
  $977 = HEAP8[$976>>0]|0;
  $978 = ($977<<24>>24)!=(0);
  if ($978) {
   _sqlite3AuthContextPop($sContext);
   $983 = HEAP32[$db>>2]|0;
   $984 = HEAP32[$aXRef>>2]|0;
   _sqlite3DbFree($983,$984);
   $985 = HEAP32[$db>>2]|0;
   $986 = HEAP32[$1>>2]|0;
   _sqlite3SrcListDelete($985,$986);
   $987 = HEAP32[$db>>2]|0;
   $988 = HEAP32[$2>>2]|0;
   _sqlite3ExprListDelete($987,$988);
   $989 = HEAP32[$db>>2]|0;
   $990 = HEAP32[$3>>2]|0;
   _sqlite3ExprDelete($989,$990);
   STACKTOP = sp;return;
  }
  $979 = HEAP32[$v>>2]|0;
  $980 = HEAP32[$regRowCount>>2]|0;
  (_sqlite3VdbeAddOp2($979,35,$980,1)|0);
  $981 = HEAP32[$v>>2]|0;
  _sqlite3VdbeSetNumCols($981,1);
  $982 = HEAP32[$v>>2]|0;
  (_sqlite3VdbeSetColName($982,0,0,24544,0)|0);
  _sqlite3AuthContextPop($sContext);
  $983 = HEAP32[$db>>2]|0;
  $984 = HEAP32[$aXRef>>2]|0;
  _sqlite3DbFree($983,$984);
  $985 = HEAP32[$db>>2]|0;
  $986 = HEAP32[$1>>2]|0;
  _sqlite3SrcListDelete($985,$986);
  $987 = HEAP32[$db>>2]|0;
  $988 = HEAP32[$2>>2]|0;
  _sqlite3ExprListDelete($987,$988);
  $989 = HEAP32[$db>>2]|0;
  $990 = HEAP32[$3>>2]|0;
  _sqlite3ExprDelete($989,$990);
  STACKTOP = sp;return;
 }
 else if ((label|0) == 182) {
  _sqlite3AuthContextPop($sContext);
  $983 = HEAP32[$db>>2]|0;
  $984 = HEAP32[$aXRef>>2]|0;
  _sqlite3DbFree($983,$984);
  $985 = HEAP32[$db>>2]|0;
  $986 = HEAP32[$1>>2]|0;
  _sqlite3SrcListDelete($985,$986);
  $987 = HEAP32[$db>>2]|0;
  $988 = HEAP32[$2>>2]|0;
  _sqlite3ExprListDelete($987,$988);
  $989 = HEAP32[$db>>2]|0;
  $990 = HEAP32[$3>>2]|0;
  _sqlite3ExprDelete($989,$990);
  STACKTOP = sp;return;
 }
}
function _sqlite3Insert($pParse,$pTabList,$pSelect,$pColumn,$onError) {
 $pParse = $pParse|0;
 $pTabList = $pTabList|0;
 $pSelect = $pSelect|0;
 $pColumn = $pColumn|0;
 $onError = $onError|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0;
 var $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0, $188 = 0;
 var $189 = 0, $19 = 0, $190 = 0, $191 = 0, $192 = 0, $193 = 0, $194 = 0, $195 = 0, $196 = 0, $197 = 0, $198 = 0, $199 = 0, $2 = 0, $20 = 0, $200 = 0, $201 = 0, $202 = 0, $203 = 0, $204 = 0, $205 = 0;
 var $206 = 0, $207 = 0, $208 = 0, $209 = 0, $21 = 0, $210 = 0, $211 = 0, $212 = 0, $213 = 0, $214 = 0, $215 = 0, $216 = 0, $217 = 0, $218 = 0, $219 = 0, $22 = 0, $220 = 0, $221 = 0, $222 = 0, $223 = 0;
 var $224 = 0, $225 = 0, $226 = 0, $227 = 0, $228 = 0, $229 = 0, $23 = 0, $230 = 0, $231 = 0, $232 = 0, $233 = 0, $234 = 0, $235 = 0, $236 = 0, $237 = 0, $238 = 0, $239 = 0, $24 = 0, $240 = 0, $241 = 0;
 var $242 = 0, $243 = 0, $244 = 0, $245 = 0, $246 = 0, $247 = 0, $248 = 0, $249 = 0, $25 = 0, $250 = 0, $251 = 0, $252 = 0, $253 = 0, $254 = 0, $255 = 0, $256 = 0, $257 = 0, $258 = 0, $259 = 0, $26 = 0;
 var $260 = 0, $261 = 0, $262 = 0, $263 = 0, $264 = 0, $265 = 0, $266 = 0, $267 = 0, $268 = 0, $269 = 0, $27 = 0, $270 = 0, $271 = 0, $272 = 0, $273 = 0, $274 = 0, $275 = 0, $276 = 0, $277 = 0, $278 = 0;
 var $279 = 0, $28 = 0, $280 = 0, $281 = 0, $282 = 0, $283 = 0, $284 = 0, $285 = 0, $286 = 0, $287 = 0, $288 = 0, $289 = 0, $29 = 0, $290 = 0, $291 = 0, $292 = 0, $293 = 0, $294 = 0, $295 = 0, $296 = 0;
 var $297 = 0, $298 = 0, $299 = 0, $3 = 0, $30 = 0, $300 = 0, $301 = 0, $302 = 0, $303 = 0, $304 = 0, $305 = 0, $306 = 0, $307 = 0, $308 = 0, $309 = 0, $31 = 0, $310 = 0, $311 = 0, $312 = 0, $313 = 0;
 var $314 = 0, $315 = 0, $316 = 0, $317 = 0, $318 = 0, $319 = 0, $32 = 0, $320 = 0, $321 = 0, $322 = 0, $323 = 0, $324 = 0, $325 = 0, $326 = 0, $327 = 0, $328 = 0, $329 = 0, $33 = 0, $330 = 0, $331 = 0;
 var $332 = 0, $333 = 0, $334 = 0, $335 = 0, $336 = 0, $337 = 0, $338 = 0, $339 = 0, $34 = 0, $340 = 0, $341 = 0, $342 = 0, $343 = 0, $344 = 0, $345 = 0, $346 = 0, $347 = 0, $348 = 0, $349 = 0, $35 = 0;
 var $350 = 0, $351 = 0, $352 = 0, $353 = 0, $354 = 0, $355 = 0, $356 = 0, $357 = 0, $358 = 0, $359 = 0, $36 = 0, $360 = 0, $361 = 0, $362 = 0, $363 = 0, $364 = 0, $365 = 0, $366 = 0, $367 = 0, $368 = 0;
 var $369 = 0, $37 = 0, $370 = 0, $371 = 0, $372 = 0, $373 = 0, $374 = 0, $375 = 0, $376 = 0, $377 = 0, $378 = 0, $379 = 0, $38 = 0, $380 = 0, $381 = 0, $382 = 0, $383 = 0, $384 = 0, $385 = 0, $386 = 0;
 var $387 = 0, $388 = 0, $389 = 0, $39 = 0, $390 = 0, $391 = 0, $392 = 0, $393 = 0, $394 = 0, $395 = 0, $396 = 0, $397 = 0, $398 = 0, $399 = 0, $4 = 0, $40 = 0, $400 = 0, $401 = 0, $402 = 0, $403 = 0;
 var $404 = 0, $405 = 0, $406 = 0, $407 = 0, $408 = 0, $409 = 0, $41 = 0, $410 = 0, $411 = 0, $412 = 0, $413 = 0, $414 = 0, $415 = 0, $416 = 0, $417 = 0, $418 = 0, $419 = 0, $42 = 0, $420 = 0, $421 = 0;
 var $422 = 0, $423 = 0, $424 = 0, $425 = 0, $426 = 0, $427 = 0, $428 = 0, $429 = 0, $43 = 0, $430 = 0, $431 = 0, $432 = 0, $433 = 0, $434 = 0, $435 = 0, $436 = 0, $437 = 0, $438 = 0, $439 = 0, $44 = 0;
 var $440 = 0, $441 = 0, $442 = 0, $443 = 0, $444 = 0, $445 = 0, $446 = 0, $447 = 0, $448 = 0, $449 = 0, $45 = 0, $450 = 0, $451 = 0, $452 = 0, $453 = 0, $454 = 0, $455 = 0, $456 = 0, $457 = 0, $458 = 0;
 var $459 = 0, $46 = 0, $460 = 0, $461 = 0, $462 = 0, $463 = 0, $464 = 0, $465 = 0, $466 = 0, $467 = 0, $468 = 0, $469 = 0, $47 = 0, $470 = 0, $471 = 0, $472 = 0, $473 = 0, $474 = 0, $475 = 0, $476 = 0;
 var $477 = 0, $478 = 0, $479 = 0, $48 = 0, $480 = 0, $481 = 0, $482 = 0, $483 = 0, $484 = 0, $485 = 0, $486 = 0, $487 = 0, $488 = 0, $489 = 0, $49 = 0, $490 = 0, $491 = 0, $492 = 0, $493 = 0, $494 = 0;
 var $495 = 0, $496 = 0, $497 = 0, $498 = 0, $499 = 0, $5 = 0, $50 = 0, $500 = 0, $501 = 0, $502 = 0, $503 = 0, $504 = 0, $505 = 0, $506 = 0, $507 = 0, $508 = 0, $509 = 0, $51 = 0, $510 = 0, $511 = 0;
 var $512 = 0, $513 = 0, $514 = 0, $515 = 0, $516 = 0, $517 = 0, $518 = 0, $519 = 0, $52 = 0, $520 = 0, $521 = 0, $522 = 0, $523 = 0, $524 = 0, $525 = 0, $526 = 0, $527 = 0, $528 = 0, $529 = 0, $53 = 0;
 var $530 = 0, $531 = 0, $532 = 0, $533 = 0, $534 = 0, $535 = 0, $536 = 0, $537 = 0, $538 = 0, $539 = 0, $54 = 0, $540 = 0, $541 = 0, $542 = 0, $543 = 0, $544 = 0, $545 = 0, $546 = 0, $547 = 0, $548 = 0;
 var $549 = 0, $55 = 0, $550 = 0, $551 = 0, $552 = 0, $553 = 0, $554 = 0, $555 = 0, $556 = 0, $557 = 0, $558 = 0, $559 = 0, $56 = 0, $560 = 0, $561 = 0, $562 = 0, $563 = 0, $564 = 0, $565 = 0, $566 = 0;
 var $567 = 0, $568 = 0, $569 = 0, $57 = 0, $570 = 0, $571 = 0, $572 = 0, $573 = 0, $574 = 0, $575 = 0, $576 = 0, $577 = 0, $578 = 0, $579 = 0, $58 = 0, $580 = 0, $581 = 0, $582 = 0, $583 = 0, $584 = 0;
 var $585 = 0, $586 = 0, $587 = 0, $588 = 0, $589 = 0, $59 = 0, $590 = 0, $591 = 0, $592 = 0, $593 = 0, $594 = 0, $595 = 0, $596 = 0, $597 = 0, $598 = 0, $599 = 0, $6 = 0, $60 = 0, $600 = 0, $601 = 0;
 var $602 = 0, $603 = 0, $604 = 0, $605 = 0, $606 = 0, $607 = 0, $608 = 0, $609 = 0, $61 = 0, $610 = 0, $611 = 0, $612 = 0, $613 = 0, $614 = 0, $615 = 0, $616 = 0, $617 = 0, $618 = 0, $619 = 0, $62 = 0;
 var $620 = 0, $621 = 0, $622 = 0, $623 = 0, $624 = 0, $625 = 0, $626 = 0, $627 = 0, $628 = 0, $629 = 0, $63 = 0, $630 = 0, $631 = 0, $632 = 0, $633 = 0, $634 = 0, $635 = 0, $636 = 0, $637 = 0, $638 = 0;
 var $639 = 0, $64 = 0, $640 = 0, $641 = 0, $642 = 0, $643 = 0, $644 = 0, $645 = 0, $646 = 0, $647 = 0, $648 = 0, $649 = 0, $65 = 0, $650 = 0, $651 = 0, $652 = 0, $653 = 0, $654 = 0, $655 = 0, $656 = 0;
 var $657 = 0, $658 = 0, $659 = 0, $66 = 0, $660 = 0, $661 = 0, $662 = 0, $663 = 0, $664 = 0, $665 = 0, $666 = 0, $667 = 0, $668 = 0, $669 = 0, $67 = 0, $670 = 0, $671 = 0, $672 = 0, $673 = 0, $674 = 0;
 var $675 = 0, $676 = 0, $677 = 0, $678 = 0, $679 = 0, $68 = 0, $680 = 0, $681 = 0, $682 = 0, $683 = 0, $684 = 0, $685 = 0, $686 = 0, $687 = 0, $688 = 0, $689 = 0, $69 = 0, $690 = 0, $691 = 0, $692 = 0;
 var $693 = 0, $694 = 0, $695 = 0, $696 = 0, $697 = 0, $698 = 0, $699 = 0, $7 = 0, $70 = 0, $700 = 0, $701 = 0, $702 = 0, $703 = 0, $704 = 0, $705 = 0, $706 = 0, $707 = 0, $708 = 0, $709 = 0, $71 = 0;
 var $710 = 0, $711 = 0, $712 = 0, $713 = 0, $714 = 0, $715 = 0, $716 = 0, $717 = 0, $718 = 0, $719 = 0, $72 = 0, $720 = 0, $721 = 0, $722 = 0, $723 = 0, $724 = 0, $725 = 0, $726 = 0, $727 = 0, $728 = 0;
 var $729 = 0, $73 = 0, $730 = 0, $731 = 0, $732 = 0, $733 = 0, $734 = 0, $735 = 0, $736 = 0, $737 = 0, $738 = 0, $739 = 0, $74 = 0, $740 = 0, $741 = 0, $742 = 0, $743 = 0, $744 = 0, $745 = 0, $746 = 0;
 var $747 = 0, $748 = 0, $749 = 0, $75 = 0, $750 = 0, $751 = 0, $752 = 0, $753 = 0, $754 = 0, $755 = 0, $756 = 0, $757 = 0, $758 = 0, $759 = 0, $76 = 0, $760 = 0, $761 = 0, $762 = 0, $763 = 0, $764 = 0;
 var $765 = 0, $766 = 0, $767 = 0, $768 = 0, $769 = 0, $77 = 0, $770 = 0, $771 = 0, $772 = 0, $773 = 0, $774 = 0, $775 = 0, $776 = 0, $777 = 0, $778 = 0, $779 = 0, $78 = 0, $780 = 0, $781 = 0, $782 = 0;
 var $783 = 0, $784 = 0, $785 = 0, $786 = 0, $787 = 0, $788 = 0, $789 = 0, $79 = 0, $790 = 0, $791 = 0, $792 = 0, $793 = 0, $794 = 0, $795 = 0, $796 = 0, $797 = 0, $798 = 0, $799 = 0, $8 = 0, $80 = 0;
 var $800 = 0, $801 = 0, $802 = 0, $803 = 0, $804 = 0, $805 = 0, $806 = 0, $807 = 0, $808 = 0, $809 = 0, $81 = 0, $810 = 0, $811 = 0, $812 = 0, $813 = 0, $814 = 0, $815 = 0, $816 = 0, $817 = 0, $818 = 0;
 var $819 = 0, $82 = 0, $820 = 0, $821 = 0, $822 = 0, $823 = 0, $824 = 0, $825 = 0, $826 = 0, $827 = 0, $828 = 0, $829 = 0, $83 = 0, $830 = 0, $831 = 0, $832 = 0, $833 = 0, $834 = 0, $835 = 0, $836 = 0;
 var $837 = 0, $838 = 0, $839 = 0, $84 = 0, $840 = 0, $841 = 0, $842 = 0, $843 = 0, $844 = 0, $845 = 0, $846 = 0, $847 = 0, $848 = 0, $849 = 0, $85 = 0, $850 = 0, $851 = 0, $852 = 0, $853 = 0, $854 = 0;
 var $855 = 0, $856 = 0, $857 = 0, $858 = 0, $859 = 0, $86 = 0, $860 = 0, $861 = 0, $862 = 0, $863 = 0, $864 = 0, $865 = 0, $866 = 0, $867 = 0, $868 = 0, $869 = 0, $87 = 0, $870 = 0, $871 = 0, $872 = 0;
 var $873 = 0, $874 = 0, $875 = 0, $876 = 0, $877 = 0, $878 = 0, $879 = 0, $88 = 0, $880 = 0, $881 = 0, $882 = 0, $883 = 0, $884 = 0, $885 = 0, $886 = 0, $887 = 0, $888 = 0, $889 = 0, $89 = 0, $890 = 0;
 var $891 = 0, $892 = 0, $893 = 0, $894 = 0, $895 = 0, $896 = 0, $897 = 0, $898 = 0, $899 = 0, $9 = 0, $90 = 0, $900 = 0, $901 = 0, $902 = 0, $903 = 0, $904 = 0, $905 = 0, $906 = 0, $907 = 0, $908 = 0;
 var $909 = 0, $91 = 0, $910 = 0, $911 = 0, $912 = 0, $913 = 0, $914 = 0, $915 = 0, $916 = 0, $917 = 0, $918 = 0, $919 = 0, $92 = 0, $920 = 0, $921 = 0, $922 = 0, $923 = 0, $924 = 0, $925 = 0, $926 = 0;
 var $927 = 0, $928 = 0, $929 = 0, $93 = 0, $930 = 0, $931 = 0, $932 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $aRegIdx = 0, $addrCont = 0, $addrInsTop = 0, $addrL = 0, $addrTop = 0, $appendFlag = 0, $bIdListInOrder = 0;
 var $db = 0, $dest = 0, $endOfLoop = 0, $i = 0, $iDataCur = 0, $iDb = 0, $iIdxCur = 0, $iRegStore = 0, $idx = 0, $ipkColumn = 0, $isReplace = 0, $isView = 0, $j = 0, $j1 = 0, $j11 = 0, $nColumn = 0, $nHidden = 0, $nIdx = 0, $or$cond = 0, $or$cond11 = 0;
 var $or$cond3 = 0, $or$cond5 = 0, $or$cond7 = 0, $or$cond9 = 0, $pDb = 0, $pIdx = 0, $pList = 0, $pOp = 0, $pTab = 0, $pTrigger = 0, $pVTab = 0, $rc = 0, $regAutoinc = 0, $regCols = 0, $regData = 0, $regFromSelect = 0, $regIns = 0, $regRec = 0, $regRowCount = 0, $regRowid = 0;
 var $regTempRowid = 0, $regYield = 0, $sNC = 0, $srcTab = 0, $tmask = 0, $useTempTable = 0, $v = 0, $vararg_buffer20 = 0, $vararg_ptr12 = 0, $vararg_ptr13 = 0, $vararg_ptr17 = 0, $vararg_ptr18 = 0, $vararg_ptr19 = 0, $vararg_ptr23 = 0, $withoutRowid = 0, $zDb = 0, $zTab = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 272|0;
 $vararg_buffer20 = sp;
 $0 = sp + 264|0;
 $1 = sp + 260|0;
 $2 = sp + 256|0;
 $3 = sp + 252|0;
 $4 = sp + 248|0;
 $db = sp + 244|0;
 $pTab = sp + 240|0;
 $zTab = sp + 236|0;
 $zDb = sp + 232|0;
 $i = sp + 228|0;
 $j = sp + 224|0;
 $idx = sp + 220|0;
 $v = sp + 216|0;
 $pIdx = sp + 212|0;
 $nColumn = sp + 208|0;
 $nHidden = sp + 204|0;
 $iDataCur = sp + 200|0;
 $iIdxCur = sp + 196|0;
 $ipkColumn = sp + 192|0;
 $endOfLoop = sp + 188|0;
 $srcTab = sp + 184|0;
 $addrInsTop = sp + 180|0;
 $addrCont = sp + 176|0;
 $dest = sp + 156|0;
 $iDb = sp + 152|0;
 $pDb = sp + 148|0;
 $useTempTable = sp + 271|0;
 $appendFlag = sp + 270|0;
 $withoutRowid = sp + 269|0;
 $bIdListInOrder = sp + 268|0;
 $pList = sp + 144|0;
 $regFromSelect = sp + 140|0;
 $regAutoinc = sp + 136|0;
 $regRowCount = sp + 132|0;
 $regIns = sp + 128|0;
 $regRowid = sp + 124|0;
 $regData = sp + 120|0;
 $aRegIdx = sp + 116|0;
 $isView = sp + 112|0;
 $pTrigger = sp + 108|0;
 $tmask = sp + 104|0;
 $regYield = sp + 100|0;
 $addrTop = sp + 96|0;
 $rc = sp + 92|0;
 $regRec = sp + 88|0;
 $regTempRowid = sp + 84|0;
 $addrL = sp + 80|0;
 $sNC = sp + 48|0;
 $nIdx = sp + 44|0;
 $regCols = sp + 40|0;
 $j1 = sp + 36|0;
 $pOp = sp + 32|0;
 $j11 = sp + 28|0;
 $iRegStore = sp + 24|0;
 $pVTab = sp + 20|0;
 $isReplace = sp + 16|0;
 HEAP32[$0>>2] = $pParse;
 HEAP32[$1>>2] = $pTabList;
 HEAP32[$2>>2] = $pSelect;
 HEAP32[$3>>2] = $pColumn;
 HEAP32[$4>>2] = $onError;
 HEAP32[$nHidden>>2] = 0;
 HEAP32[$iDataCur>>2] = 0;
 HEAP32[$iIdxCur>>2] = 0;
 HEAP32[$ipkColumn>>2] = -1;
 HEAP32[$srcTab>>2] = 0;
 HEAP32[$addrInsTop>>2] = 0;
 HEAP32[$addrCont>>2] = 0;
 HEAP8[$useTempTable>>0] = 0;
 HEAP8[$appendFlag>>0] = 0;
 HEAP8[$bIdListInOrder>>0] = 1;
 HEAP32[$pList>>2] = 0;
 HEAP32[$regFromSelect>>2] = 0;
 HEAP32[$regAutoinc>>2] = 0;
 HEAP32[$regRowCount>>2] = 0;
 HEAP32[$aRegIdx>>2] = 0;
 $5 = HEAP32[$0>>2]|0;
 $6 = HEAP32[$5>>2]|0;
 HEAP32[$db>>2] = $6;
 ;HEAP32[$dest+0>>2]=0|0;HEAP32[$dest+4>>2]=0|0;HEAP32[$dest+8>>2]=0|0;HEAP32[$dest+12>>2]=0|0;HEAP32[$dest+16>>2]=0|0;
 $7 = HEAP32[$0>>2]|0;
 $8 = (($7) + 64|0);
 $9 = HEAP32[$8>>2]|0;
 $10 = ($9|0)!=(0);
 L1: do {
  if (!($10)) {
   $11 = HEAP32[$db>>2]|0;
   $12 = (($11) + 64|0);
   $13 = HEAP8[$12>>0]|0;
   $14 = $13&255;
   $15 = ($14|0)!=(0);
   if (!($15)) {
    $16 = HEAP32[$2>>2]|0;
    $17 = ($16|0)!=(0|0);
    if ($17) {
     $18 = HEAP32[$2>>2]|0;
     $19 = (($18) + 6|0);
     $20 = HEAP16[$19>>1]|0;
     $21 = $20&65535;
     $22 = $21 & 128;
     $23 = ($22|0)!=(0);
     if ($23) {
      $24 = HEAP32[$2>>2]|0;
      $25 = (($24) + 52|0);
      $26 = HEAP32[$25>>2]|0;
      $27 = ($26|0)==(0|0);
      if ($27) {
       $28 = HEAP32[$2>>2]|0;
       $29 = HEAP32[$28>>2]|0;
       HEAP32[$pList>>2] = $29;
       $30 = HEAP32[$2>>2]|0;
       HEAP32[$30>>2] = 0;
       $31 = HEAP32[$db>>2]|0;
       $32 = HEAP32[$2>>2]|0;
       _sqlite3SelectDelete($31,$32);
       HEAP32[$2>>2] = 0;
      }
     }
    }
    $33 = HEAP32[$1>>2]|0;
    $34 = (($33) + 8|0);
    $35 = (($34) + 8|0);
    $36 = HEAP32[$35>>2]|0;
    HEAP32[$zTab>>2] = $36;
    $37 = HEAP32[$zTab>>2]|0;
    $38 = ($37|0)==(0|0);
    if (!($38)) {
     $39 = HEAP32[$0>>2]|0;
     $40 = HEAP32[$1>>2]|0;
     $41 = (_sqlite3SrcListLookup($39,$40)|0);
     HEAP32[$pTab>>2] = $41;
     $42 = HEAP32[$pTab>>2]|0;
     $43 = ($42|0)==(0|0);
     if (!($43)) {
      $44 = HEAP32[$db>>2]|0;
      $45 = HEAP32[$pTab>>2]|0;
      $46 = (($45) + 68|0);
      $47 = HEAP32[$46>>2]|0;
      $48 = (_sqlite3SchemaToIndex($44,$47)|0);
      HEAP32[$iDb>>2] = $48;
      $49 = HEAP32[$iDb>>2]|0;
      $50 = HEAP32[$db>>2]|0;
      $51 = (($50) + 16|0);
      $52 = HEAP32[$51>>2]|0;
      $53 = (($52) + ($49<<4)|0);
      HEAP32[$pDb>>2] = $53;
      $54 = HEAP32[$pDb>>2]|0;
      $55 = HEAP32[$54>>2]|0;
      HEAP32[$zDb>>2] = $55;
      $56 = HEAP32[$0>>2]|0;
      $57 = HEAP32[$pTab>>2]|0;
      $58 = HEAP32[$57>>2]|0;
      $59 = HEAP32[$zDb>>2]|0;
      $60 = (_sqlite3AuthCheck($56,18,$58,0,$59)|0);
      $61 = ($60|0)!=(0);
      if (!($61)) {
       $62 = HEAP32[$pTab>>2]|0;
       $63 = (($62) + 44|0);
       $64 = HEAP8[$63>>0]|0;
       $65 = $64&255;
       $66 = $65 & 32;
       $67 = ($66|0)==(0);
       $68 = $67 ^ 1;
       $69 = $68&1;
       $70 = $69&255;
       HEAP8[$withoutRowid>>0] = $70;
       $71 = HEAP32[$0>>2]|0;
       $72 = HEAP32[$pTab>>2]|0;
       $73 = (_sqlite3TriggersExist($71,$72,108,0,$tmask)|0);
       HEAP32[$pTrigger>>2] = $73;
       $74 = HEAP32[$pTab>>2]|0;
       $75 = (($74) + 12|0);
       $76 = HEAP32[$75>>2]|0;
       $77 = ($76|0)!=(0|0);
       $78 = $77&1;
       HEAP32[$isView>>2] = $78;
       $79 = HEAP32[$0>>2]|0;
       $80 = HEAP32[$pTab>>2]|0;
       $81 = (_sqlite3ViewGetColumnNames($79,$80)|0);
       $82 = ($81|0)!=(0);
       if (!($82)) {
        $83 = HEAP32[$0>>2]|0;
        $84 = HEAP32[$pTab>>2]|0;
        $85 = HEAP32[$tmask>>2]|0;
        $86 = (_sqlite3IsReadOnly($83,$84,$85)|0);
        $87 = ($86|0)!=(0);
        if (!($87)) {
         $88 = HEAP32[$0>>2]|0;
         $89 = (_sqlite3GetVdbe($88)|0);
         HEAP32[$v>>2] = $89;
         $90 = HEAP32[$v>>2]|0;
         $91 = ($90|0)==(0|0);
         if (!($91)) {
          $92 = HEAP32[$0>>2]|0;
          $93 = (($92) + 18|0);
          $94 = HEAP8[$93>>0]|0;
          $95 = $94&255;
          $96 = ($95|0)==(0);
          if ($96) {
           $97 = HEAP32[$v>>2]|0;
           _sqlite3VdbeCountChanges($97);
          }
          $98 = HEAP32[$0>>2]|0;
          $99 = HEAP32[$2>>2]|0;
          $100 = ($99|0)!=(0|0);
          if ($100) {
           $104 = 1;
          } else {
           $101 = HEAP32[$pTrigger>>2]|0;
           $102 = ($101|0)!=(0|0);
           $104 = $102;
          }
          $103 = $104&1;
          $105 = HEAP32[$iDb>>2]|0;
          _sqlite3BeginWriteOperation($98,$103,$105);
          $106 = HEAP32[$3>>2]|0;
          $107 = ($106|0)==(0|0);
          if ($107) {
           $108 = HEAP32[$0>>2]|0;
           $109 = HEAP32[$pTab>>2]|0;
           $110 = HEAP32[$2>>2]|0;
           $111 = HEAP32[$4>>2]|0;
           $112 = HEAP32[$iDb>>2]|0;
           $113 = (_xferOptimization($108,$109,$110,$111,$112)|0);
           $114 = ($113|0)!=(0);
           if (!($114)) {
            label = 19;
           }
          } else {
           label = 19;
          }
          L23: do {
           if ((label|0) == 19) {
            $115 = HEAP32[$0>>2]|0;
            $116 = HEAP32[$iDb>>2]|0;
            $117 = HEAP32[$pTab>>2]|0;
            $118 = (_autoIncBegin($115,$116,$117)|0);
            HEAP32[$regAutoinc>>2] = $118;
            $119 = HEAP32[$0>>2]|0;
            $120 = (($119) + 72|0);
            $121 = HEAP32[$120>>2]|0;
            $122 = (($121) + 1)|0;
            HEAP32[$regIns>>2] = $122;
            HEAP32[$regRowid>>2] = $122;
            $123 = HEAP32[$pTab>>2]|0;
            $124 = (($123) + 38|0);
            $125 = HEAP16[$124>>1]|0;
            $126 = $125 << 16 >> 16;
            $127 = (($126) + 1)|0;
            $128 = HEAP32[$0>>2]|0;
            $129 = (($128) + 72|0);
            $130 = HEAP32[$129>>2]|0;
            $131 = (($130) + ($127))|0;
            HEAP32[$129>>2] = $131;
            $132 = HEAP32[$pTab>>2]|0;
            $133 = (($132) + 44|0);
            $134 = HEAP8[$133>>0]|0;
            $135 = $134&255;
            $136 = $135 & 16;
            $137 = ($136|0)!=(0);
            if ($137) {
             $138 = HEAP32[$regRowid>>2]|0;
             $139 = (($138) + 1)|0;
             HEAP32[$regRowid>>2] = $139;
             $140 = HEAP32[$0>>2]|0;
             $141 = (($140) + 72|0);
             $142 = HEAP32[$141>>2]|0;
             $143 = (($142) + 1)|0;
             HEAP32[$141>>2] = $143;
            }
            $144 = HEAP32[$regRowid>>2]|0;
            $145 = (($144) + 1)|0;
            HEAP32[$regData>>2] = $145;
            $146 = HEAP32[$3>>2]|0;
            $147 = ($146|0)!=(0|0);
            L28: do {
             if ($147) {
              HEAP32[$i>>2] = 0;
              while(1) {
               $148 = HEAP32[$i>>2]|0;
               $149 = HEAP32[$3>>2]|0;
               $150 = (($149) + 4|0);
               $151 = HEAP32[$150>>2]|0;
               $152 = ($148|0)<($151|0);
               if (!($152)) {
                break;
               }
               $153 = HEAP32[$i>>2]|0;
               $154 = HEAP32[$3>>2]|0;
               $155 = HEAP32[$154>>2]|0;
               $156 = (($155) + ($153<<3)|0);
               $157 = (($156) + 4|0);
               HEAP32[$157>>2] = -1;
               $158 = HEAP32[$i>>2]|0;
               $159 = (($158) + 1)|0;
               HEAP32[$i>>2] = $159;
              }
              HEAP32[$i>>2] = 0;
              while(1) {
               $160 = HEAP32[$i>>2]|0;
               $161 = HEAP32[$3>>2]|0;
               $162 = (($161) + 4|0);
               $163 = HEAP32[$162>>2]|0;
               $164 = ($160|0)<($163|0);
               if (!($164)) {
                break L28;
               }
               HEAP32[$j>>2] = 0;
               while(1) {
                $165 = HEAP32[$j>>2]|0;
                $166 = HEAP32[$pTab>>2]|0;
                $167 = (($166) + 38|0);
                $168 = HEAP16[$167>>1]|0;
                $169 = $168 << 16 >> 16;
                $170 = ($165|0)<($169|0);
                if (!($170)) {
                 break;
                }
                $171 = HEAP32[$i>>2]|0;
                $172 = HEAP32[$3>>2]|0;
                $173 = HEAP32[$172>>2]|0;
                $174 = (($173) + ($171<<3)|0);
                $175 = HEAP32[$174>>2]|0;
                $176 = HEAP32[$j>>2]|0;
                $177 = HEAP32[$pTab>>2]|0;
                $178 = (($177) + 4|0);
                $179 = HEAP32[$178>>2]|0;
                $180 = (($179) + (($176*24)|0)|0);
                $181 = HEAP32[$180>>2]|0;
                $182 = (_sqlite3_stricmp($175,$181)|0);
                $183 = ($182|0)==(0);
                $184 = HEAP32[$j>>2]|0;
                if ($183) {
                 label = 30;
                 break;
                }
                $200 = (($184) + 1)|0;
                HEAP32[$j>>2] = $200;
               }
               do {
                if ((label|0) == 30) {
                 label = 0;
                 $185 = HEAP32[$i>>2]|0;
                 $186 = HEAP32[$3>>2]|0;
                 $187 = HEAP32[$186>>2]|0;
                 $188 = (($187) + ($185<<3)|0);
                 $189 = (($188) + 4|0);
                 HEAP32[$189>>2] = $184;
                 $190 = HEAP32[$i>>2]|0;
                 $191 = HEAP32[$j>>2]|0;
                 $192 = ($190|0)!=($191|0);
                 if ($192) {
                  HEAP8[$bIdListInOrder>>0] = 0;
                 }
                 $193 = HEAP32[$j>>2]|0;
                 $194 = HEAP32[$pTab>>2]|0;
                 $195 = (($194) + 36|0);
                 $196 = HEAP16[$195>>1]|0;
                 $197 = $196 << 16 >> 16;
                 $198 = ($193|0)==($197|0);
                 if (!($198)) {
                  break;
                 }
                 $199 = HEAP32[$i>>2]|0;
                 HEAP32[$ipkColumn>>2] = $199;
                }
               } while(0);
               $201 = HEAP32[$j>>2]|0;
               $202 = HEAP32[$pTab>>2]|0;
               $203 = (($202) + 38|0);
               $204 = HEAP16[$203>>1]|0;
               $205 = $204 << 16 >> 16;
               $206 = ($201|0)>=($205|0);
               if ($206) {
                $207 = HEAP32[$i>>2]|0;
                $208 = HEAP32[$3>>2]|0;
                $209 = HEAP32[$208>>2]|0;
                $210 = (($209) + ($207<<3)|0);
                $211 = HEAP32[$210>>2]|0;
                $212 = (_sqlite3IsRowid($211)|0);
                $213 = ($212|0)==(0);
                $214 = HEAP8[$withoutRowid>>0]|0;
                $215 = ($214<<24>>24)!=(0);
                $or$cond = $213 | $215;
                if ($or$cond) {
                 break;
                }
                $216 = HEAP32[$i>>2]|0;
                HEAP32[$ipkColumn>>2] = $216;
                HEAP8[$bIdListInOrder>>0] = 0;
               }
               $226 = HEAP32[$i>>2]|0;
               $227 = (($226) + 1)|0;
               HEAP32[$i>>2] = $227;
              }
              $217 = HEAP32[$0>>2]|0;
              $218 = HEAP32[$1>>2]|0;
              $219 = HEAP32[$i>>2]|0;
              $220 = HEAP32[$3>>2]|0;
              $221 = HEAP32[$220>>2]|0;
              $222 = (($221) + ($219<<3)|0);
              $223 = HEAP32[$222>>2]|0;
              HEAP32[$vararg_buffer20>>2] = $218;
              $vararg_ptr12 = (($vararg_buffer20) + 4|0);
              HEAP32[$vararg_ptr12>>2] = 0;
              $vararg_ptr13 = (($vararg_buffer20) + 8|0);
              HEAP32[$vararg_ptr13>>2] = $223;
              _sqlite3ErrorMsg($217,23432,$vararg_buffer20);
              $224 = HEAP32[$0>>2]|0;
              $225 = (($224) + 17|0);
              HEAP8[$225>>0] = 1;
              break L1;
             }
            } while(0);
            $228 = HEAP32[$2>>2]|0;
            $229 = ($228|0)!=(0|0);
            L54: do {
             if ($229) {
              $230 = HEAP32[$0>>2]|0;
              $231 = (($230) + 72|0);
              $232 = HEAP32[$231>>2]|0;
              $233 = (($232) + 1)|0;
              HEAP32[$231>>2] = $233;
              HEAP32[$regYield>>2] = $233;
              $234 = HEAP32[$v>>2]|0;
              $235 = (_sqlite3VdbeCurrentAddr($234)|0);
              $236 = (($235) + 1)|0;
              HEAP32[$addrTop>>2] = $236;
              $237 = HEAP32[$v>>2]|0;
              $238 = HEAP32[$regYield>>2]|0;
              $239 = HEAP32[$addrTop>>2]|0;
              (_sqlite3VdbeAddOp3($237,20,$238,0,$239)|0);
              $240 = HEAP32[$regYield>>2]|0;
              _sqlite3SelectDestInit($dest,13,$240);
              $241 = HEAP8[$bIdListInOrder>>0]|0;
              $242 = $241&255;
              $243 = ($242|0)!=(0);
              $244 = HEAP32[$regData>>2]|0;
              $245 = $243 ? $244 : 0;
              $246 = (($dest) + 8|0);
              HEAP32[$246>>2] = $245;
              $247 = HEAP32[$pTab>>2]|0;
              $248 = (($247) + 38|0);
              $249 = HEAP16[$248>>1]|0;
              $250 = $249 << 16 >> 16;
              $251 = (($dest) + 12|0);
              HEAP32[$251>>2] = $250;
              $252 = HEAP32[$0>>2]|0;
              $253 = HEAP32[$2>>2]|0;
              $254 = (_sqlite3Select($252,$253,$dest)|0);
              HEAP32[$rc>>2] = $254;
              $255 = (($dest) + 8|0);
              $256 = HEAP32[$255>>2]|0;
              HEAP32[$regFromSelect>>2] = $256;
              $257 = HEAP32[$rc>>2]|0;
              $258 = ($257|0)!=(0);
              if ($258) {
               break L1;
              }
              $259 = HEAP32[$db>>2]|0;
              $260 = (($259) + 64|0);
              $261 = HEAP8[$260>>0]|0;
              $262 = $261&255;
              $263 = ($262|0)!=(0);
              if ($263) {
               break L1;
              }
              $264 = HEAP32[$v>>2]|0;
              $265 = HEAP32[$regYield>>2]|0;
              (_sqlite3VdbeAddOp1($264,21,$265)|0);
              $266 = HEAP32[$v>>2]|0;
              $267 = HEAP32[$addrTop>>2]|0;
              $268 = (($267) - 1)|0;
              _sqlite3VdbeJumpHere($266,$268);
              $269 = HEAP32[$2>>2]|0;
              $270 = HEAP32[$269>>2]|0;
              $271 = HEAP32[$270>>2]|0;
              HEAP32[$nColumn>>2] = $271;
              $272 = HEAP32[$pTrigger>>2]|0;
              $273 = ($272|0)!=(0|0);
              if ($273) {
               label = 45;
              } else {
               $274 = HEAP32[$0>>2]|0;
               $275 = HEAP32[$iDb>>2]|0;
               $276 = HEAP32[$pTab>>2]|0;
               $277 = (_readsTable($274,$275,$276)|0);
               $278 = ($277|0)!=(0);
               if ($278) {
                label = 45;
               }
              }
              if ((label|0) == 45) {
               HEAP8[$useTempTable>>0] = 1;
              }
              $279 = HEAP8[$useTempTable>>0]|0;
              $280 = ($279<<24>>24)!=(0);
              if (!($280)) {
               break;
              }
              $281 = HEAP32[$0>>2]|0;
              $282 = (($281) + 68|0);
              $283 = HEAP32[$282>>2]|0;
              $284 = (($283) + 1)|0;
              HEAP32[$282>>2] = $284;
              HEAP32[$srcTab>>2] = $283;
              $285 = HEAP32[$0>>2]|0;
              $286 = (_sqlite3GetTempReg($285)|0);
              HEAP32[$regRec>>2] = $286;
              $287 = HEAP32[$0>>2]|0;
              $288 = (_sqlite3GetTempReg($287)|0);
              HEAP32[$regTempRowid>>2] = $288;
              $289 = HEAP32[$v>>2]|0;
              $290 = HEAP32[$srcTab>>2]|0;
              $291 = HEAP32[$nColumn>>2]|0;
              (_sqlite3VdbeAddOp2($289,57,$290,$291)|0);
              $292 = HEAP32[$v>>2]|0;
              $293 = (($dest) + 4|0);
              $294 = HEAP32[$293>>2]|0;
              $295 = (_sqlite3VdbeAddOp1($292,22,$294)|0);
              HEAP32[$addrL>>2] = $295;
              $296 = HEAP32[$v>>2]|0;
              $297 = HEAP32[$regFromSelect>>2]|0;
              $298 = HEAP32[$nColumn>>2]|0;
              $299 = HEAP32[$regRec>>2]|0;
              (_sqlite3VdbeAddOp3($296,49,$297,$298,$299)|0);
              $300 = HEAP32[$v>>2]|0;
              $301 = HEAP32[$srcTab>>2]|0;
              $302 = HEAP32[$regTempRowid>>2]|0;
              (_sqlite3VdbeAddOp2($300,74,$301,$302)|0);
              $303 = HEAP32[$v>>2]|0;
              $304 = HEAP32[$srcTab>>2]|0;
              $305 = HEAP32[$regRec>>2]|0;
              $306 = HEAP32[$regTempRowid>>2]|0;
              (_sqlite3VdbeAddOp3($303,75,$304,$305,$306)|0);
              $307 = HEAP32[$v>>2]|0;
              $308 = HEAP32[$addrL>>2]|0;
              (_sqlite3VdbeAddOp2($307,16,0,$308)|0);
              $309 = HEAP32[$v>>2]|0;
              $310 = HEAP32[$addrL>>2]|0;
              _sqlite3VdbeJumpHere($309,$310);
              $311 = HEAP32[$0>>2]|0;
              $312 = HEAP32[$regRec>>2]|0;
              _sqlite3ReleaseTempReg($311,$312);
              $313 = HEAP32[$0>>2]|0;
              $314 = HEAP32[$regTempRowid>>2]|0;
              _sqlite3ReleaseTempReg($313,$314);
             } else {
              ;HEAP32[$sNC+0>>2]=0|0;HEAP32[$sNC+4>>2]=0|0;HEAP32[$sNC+8>>2]=0|0;HEAP32[$sNC+12>>2]=0|0;HEAP32[$sNC+16>>2]=0|0;HEAP32[$sNC+20>>2]=0|0;HEAP32[$sNC+24>>2]=0|0;HEAP32[$sNC+28>>2]=0|0;
              $315 = HEAP32[$0>>2]|0;
              HEAP32[$sNC>>2] = $315;
              HEAP32[$srcTab>>2] = -1;
              $316 = HEAP32[$pList>>2]|0;
              $317 = ($316|0)!=(0|0);
              if ($317) {
               $318 = HEAP32[$pList>>2]|0;
               $319 = HEAP32[$318>>2]|0;
               $320 = $319;
              } else {
               $320 = 0;
              }
              HEAP32[$nColumn>>2] = $320;
              HEAP32[$i>>2] = 0;
              while(1) {
               $321 = HEAP32[$i>>2]|0;
               $322 = HEAP32[$nColumn>>2]|0;
               $323 = ($321|0)<($322|0);
               if (!($323)) {
                break L54;
               }
               $324 = HEAP32[$i>>2]|0;
               $325 = HEAP32[$pList>>2]|0;
               $326 = (($325) + 4|0);
               $327 = HEAP32[$326>>2]|0;
               $328 = (($327) + (($324*20)|0)|0);
               $329 = HEAP32[$328>>2]|0;
               $330 = (_sqlite3ResolveExprNames($sNC,$329)|0);
               $331 = ($330|0)!=(0);
               if ($331) {
                break L1;
               }
               $332 = HEAP32[$i>>2]|0;
               $333 = (($332) + 1)|0;
               HEAP32[$i>>2] = $333;
              }
             }
            } while(0);
            $334 = HEAP32[$3>>2]|0;
            $335 = ($334|0)==(0|0);
            $336 = HEAP32[$nColumn>>2]|0;
            $337 = ($336|0)>(0);
            $or$cond3 = $335 & $337;
            if ($or$cond3) {
             $338 = HEAP32[$pTab>>2]|0;
             $339 = (($338) + 36|0);
             $340 = HEAP16[$339>>1]|0;
             $341 = $340 << 16 >> 16;
             HEAP32[$ipkColumn>>2] = $341;
            }
            $342 = HEAP32[$pTab>>2]|0;
            $343 = (($342) + 44|0);
            $344 = HEAP8[$343>>0]|0;
            $345 = $344&255;
            $346 = $345 & 16;
            $347 = ($346|0)!=(0);
            L76: do {
             if ($347) {
              HEAP32[$i>>2] = 0;
              while(1) {
               $348 = HEAP32[$i>>2]|0;
               $349 = HEAP32[$pTab>>2]|0;
               $350 = (($349) + 38|0);
               $351 = HEAP16[$350>>1]|0;
               $352 = $351 << 16 >> 16;
               $353 = ($348|0)<($352|0);
               if (!($353)) {
                break L76;
               }
               $354 = HEAP32[$i>>2]|0;
               $355 = HEAP32[$pTab>>2]|0;
               $356 = (($355) + 4|0);
               $357 = HEAP32[$356>>2]|0;
               $358 = (($357) + (($354*24)|0)|0);
               $359 = (($358) + 23|0);
               $360 = HEAP8[$359>>0]|0;
               $361 = $360&255;
               $362 = $361 & 2;
               $363 = ($362|0)!=(0);
               $364 = $363 ? 1 : 0;
               $365 = HEAP32[$nHidden>>2]|0;
               $366 = (($365) + ($364))|0;
               HEAP32[$nHidden>>2] = $366;
               $367 = HEAP32[$i>>2]|0;
               $368 = (($367) + 1)|0;
               HEAP32[$i>>2] = $368;
              }
             }
            } while(0);
            $369 = HEAP32[$3>>2]|0;
            $370 = ($369|0)==(0|0);
            $371 = HEAP32[$nColumn>>2]|0;
            $372 = ($371|0)!=(0);
            $or$cond5 = $370 & $372;
            do {
             if ($or$cond5) {
              $373 = HEAP32[$nColumn>>2]|0;
              $374 = HEAP32[$pTab>>2]|0;
              $375 = (($374) + 38|0);
              $376 = HEAP16[$375>>1]|0;
              $377 = $376 << 16 >> 16;
              $378 = HEAP32[$nHidden>>2]|0;
              $379 = (($377) - ($378))|0;
              $380 = ($373|0)!=($379|0);
              if (!($380)) {
               break;
              }
              $381 = HEAP32[$0>>2]|0;
              $382 = HEAP32[$1>>2]|0;
              $383 = HEAP32[$pTab>>2]|0;
              $384 = (($383) + 38|0);
              $385 = HEAP16[$384>>1]|0;
              $386 = $385 << 16 >> 16;
              $387 = HEAP32[$nHidden>>2]|0;
              $388 = (($386) - ($387))|0;
              $389 = HEAP32[$nColumn>>2]|0;
              HEAP32[$vararg_buffer20>>2] = $382;
              $vararg_ptr17 = (($vararg_buffer20) + 4|0);
              HEAP32[$vararg_ptr17>>2] = 0;
              $vararg_ptr18 = (($vararg_buffer20) + 8|0);
              HEAP32[$vararg_ptr18>>2] = $388;
              $vararg_ptr19 = (($vararg_buffer20) + 12|0);
              HEAP32[$vararg_ptr19>>2] = $389;
              _sqlite3ErrorMsg($381,23464,$vararg_buffer20);
              break L1;
             }
            } while(0);
            $390 = HEAP32[$3>>2]|0;
            $391 = ($390|0)!=(0|0);
            do {
             if ($391) {
              $392 = HEAP32[$nColumn>>2]|0;
              $393 = HEAP32[$3>>2]|0;
              $394 = (($393) + 4|0);
              $395 = HEAP32[$394>>2]|0;
              $396 = ($392|0)!=($395|0);
              if (!($396)) {
               break;
              }
              $397 = HEAP32[$0>>2]|0;
              $398 = HEAP32[$nColumn>>2]|0;
              $399 = HEAP32[$3>>2]|0;
              $400 = (($399) + 4|0);
              $401 = HEAP32[$400>>2]|0;
              HEAP32[$vararg_buffer20>>2] = $398;
              $vararg_ptr23 = (($vararg_buffer20) + 4|0);
              HEAP32[$vararg_ptr23>>2] = $401;
              _sqlite3ErrorMsg($397,23520,$vararg_buffer20);
              break L1;
             }
            } while(0);
            $402 = HEAP32[$db>>2]|0;
            $403 = (($402) + 24|0);
            $404 = HEAP32[$403>>2]|0;
            $405 = $404 & 128;
            $406 = ($405|0)!=(0);
            if ($406) {
             $407 = HEAP32[$0>>2]|0;
             $408 = (($407) + 72|0);
             $409 = HEAP32[$408>>2]|0;
             $410 = (($409) + 1)|0;
             HEAP32[$408>>2] = $410;
             HEAP32[$regRowCount>>2] = $410;
             $411 = HEAP32[$v>>2]|0;
             $412 = HEAP32[$regRowCount>>2]|0;
             (_sqlite3VdbeAddOp2($411,25,0,$412)|0);
            }
            $413 = HEAP32[$isView>>2]|0;
            $414 = ($413|0)!=(0);
            L93: do {
             if (!($414)) {
              $415 = HEAP32[$0>>2]|0;
              $416 = HEAP32[$pTab>>2]|0;
              $417 = (_sqlite3OpenTableAndIndices($415,$416,55,-1,0,$iDataCur,$iIdxCur)|0);
              HEAP32[$nIdx>>2] = $417;
              $418 = HEAP32[$db>>2]|0;
              $419 = HEAP32[$nIdx>>2]|0;
              $420 = (($419) + 1)|0;
              $421 = $420<<2;
              $422 = (_sqlite3DbMallocRaw($418,$421,0)|0);
              HEAP32[$aRegIdx>>2] = $422;
              $423 = HEAP32[$aRegIdx>>2]|0;
              $424 = ($423|0)==(0|0);
              if ($424) {
               break L1;
              }
              HEAP32[$i>>2] = 0;
              while(1) {
               $425 = HEAP32[$i>>2]|0;
               $426 = HEAP32[$nIdx>>2]|0;
               $427 = ($425|0)<($426|0);
               if (!($427)) {
                break L93;
               }
               $428 = HEAP32[$0>>2]|0;
               $429 = (($428) + 72|0);
               $430 = HEAP32[$429>>2]|0;
               $431 = (($430) + 1)|0;
               HEAP32[$429>>2] = $431;
               $432 = HEAP32[$i>>2]|0;
               $433 = HEAP32[$aRegIdx>>2]|0;
               $434 = (($433) + ($432<<2)|0);
               HEAP32[$434>>2] = $431;
               $435 = HEAP32[$i>>2]|0;
               $436 = (($435) + 1)|0;
               HEAP32[$i>>2] = $436;
              }
             }
            } while(0);
            $437 = HEAP8[$useTempTable>>0]|0;
            $438 = ($437<<24>>24)!=(0);
            do {
             if ($438) {
              $439 = HEAP32[$v>>2]|0;
              $440 = HEAP32[$srcTab>>2]|0;
              $441 = (_sqlite3VdbeAddOp1($439,108,$440)|0);
              HEAP32[$addrInsTop>>2] = $441;
              $442 = HEAP32[$v>>2]|0;
              $443 = (_sqlite3VdbeCurrentAddr($442)|0);
              HEAP32[$addrCont>>2] = $443;
             } else {
              $444 = HEAP32[$2>>2]|0;
              $445 = ($444|0)!=(0|0);
              if (!($445)) {
               break;
              }
              $446 = HEAP32[$v>>2]|0;
              $447 = (($dest) + 4|0);
              $448 = HEAP32[$447>>2]|0;
              $449 = (_sqlite3VdbeAddOp1($446,22,$448)|0);
              HEAP32[$addrCont>>2] = $449;
              HEAP32[$addrInsTop>>2] = $449;
             }
            } while(0);
            $450 = HEAP32[$v>>2]|0;
            $451 = (_sqlite3VdbeMakeLabel($450)|0);
            HEAP32[$endOfLoop>>2] = $451;
            $452 = HEAP32[$tmask>>2]|0;
            $453 = $452 & 1;
            $454 = ($453|0)!=(0);
            if ($454) {
             $455 = HEAP32[$0>>2]|0;
             $456 = HEAP32[$pTab>>2]|0;
             $457 = (($456) + 38|0);
             $458 = HEAP16[$457>>1]|0;
             $459 = $458 << 16 >> 16;
             $460 = (($459) + 1)|0;
             $461 = (_sqlite3GetTempRange($455,$460)|0);
             HEAP32[$regCols>>2] = $461;
             $462 = HEAP32[$ipkColumn>>2]|0;
             $463 = ($462|0)<(0);
             if ($463) {
              $464 = HEAP32[$v>>2]|0;
              $465 = HEAP32[$regCols>>2]|0;
              (_sqlite3VdbeAddOp2($464,25,-1,$465)|0);
             } else {
              $466 = HEAP8[$useTempTable>>0]|0;
              $467 = ($466<<24>>24)!=(0);
              if ($467) {
               $468 = HEAP32[$v>>2]|0;
               $469 = HEAP32[$srcTab>>2]|0;
               $470 = HEAP32[$ipkColumn>>2]|0;
               $471 = HEAP32[$regCols>>2]|0;
               (_sqlite3VdbeAddOp3($468,47,$469,$470,$471)|0);
              } else {
               $472 = HEAP32[$0>>2]|0;
               $473 = HEAP32[$ipkColumn>>2]|0;
               $474 = HEAP32[$pList>>2]|0;
               $475 = (($474) + 4|0);
               $476 = HEAP32[$475>>2]|0;
               $477 = (($476) + (($473*20)|0)|0);
               $478 = HEAP32[$477>>2]|0;
               $479 = HEAP32[$regCols>>2]|0;
               _sqlite3ExprCode($472,$478,$479);
              }
              $480 = HEAP32[$v>>2]|0;
              $481 = HEAP32[$regCols>>2]|0;
              $482 = (_sqlite3VdbeAddOp1($480,77,$481)|0);
              HEAP32[$j1>>2] = $482;
              $483 = HEAP32[$v>>2]|0;
              $484 = HEAP32[$regCols>>2]|0;
              (_sqlite3VdbeAddOp2($483,25,-1,$484)|0);
              $485 = HEAP32[$v>>2]|0;
              $486 = HEAP32[$j1>>2]|0;
              _sqlite3VdbeJumpHere($485,$486);
              $487 = HEAP32[$v>>2]|0;
              $488 = HEAP32[$regCols>>2]|0;
              (_sqlite3VdbeAddOp1($487,38,$488)|0);
             }
             HEAP32[$i>>2] = 0;
             while(1) {
              $489 = HEAP32[$i>>2]|0;
              $490 = HEAP32[$pTab>>2]|0;
              $491 = (($490) + 38|0);
              $492 = HEAP16[$491>>1]|0;
              $493 = $492 << 16 >> 16;
              $494 = ($489|0)<($493|0);
              if (!($494)) {
               break;
              }
              $495 = HEAP32[$3>>2]|0;
              $496 = ($495|0)==(0|0);
              L118: do {
               if ($496) {
                $497 = HEAP32[$i>>2]|0;
                HEAP32[$j>>2] = $497;
               } else {
                HEAP32[$j>>2] = 0;
                while(1) {
                 $498 = HEAP32[$j>>2]|0;
                 $499 = HEAP32[$3>>2]|0;
                 $500 = (($499) + 4|0);
                 $501 = HEAP32[$500>>2]|0;
                 $502 = ($498|0)<($501|0);
                 if (!($502)) {
                  break L118;
                 }
                 $503 = HEAP32[$j>>2]|0;
                 $504 = HEAP32[$3>>2]|0;
                 $505 = HEAP32[$504>>2]|0;
                 $506 = (($505) + ($503<<3)|0);
                 $507 = (($506) + 4|0);
                 $508 = HEAP32[$507>>2]|0;
                 $509 = HEAP32[$i>>2]|0;
                 $510 = ($508|0)==($509|0);
                 if ($510) {
                  break L118;
                 }
                 $511 = HEAP32[$j>>2]|0;
                 $512 = (($511) + 1)|0;
                 HEAP32[$j>>2] = $512;
                }
               }
              } while(0);
              $513 = HEAP8[$useTempTable>>0]|0;
              $514 = ($513<<24>>24)!=(0);
              $515 = HEAP32[$pList>>2]|0;
              $516 = ($515|0)!=(0|0);
              $or$cond7 = $514 | $516;
              do {
               if ($or$cond7) {
                $517 = HEAP32[$3>>2]|0;
                $518 = ($517|0)!=(0|0);
                if ($518) {
                 $519 = HEAP32[$j>>2]|0;
                 $520 = HEAP32[$3>>2]|0;
                 $521 = (($520) + 4|0);
                 $522 = HEAP32[$521>>2]|0;
                 $523 = ($519|0)>=($522|0);
                 if ($523) {
                  label = 95;
                  break;
                 }
                }
                $536 = HEAP8[$useTempTable>>0]|0;
                $537 = ($536<<24>>24)!=(0);
                if ($537) {
                 $538 = HEAP32[$v>>2]|0;
                 $539 = HEAP32[$srcTab>>2]|0;
                 $540 = HEAP32[$j>>2]|0;
                 $541 = HEAP32[$regCols>>2]|0;
                 $542 = HEAP32[$i>>2]|0;
                 $543 = (($541) + ($542))|0;
                 $544 = (($543) + 1)|0;
                 (_sqlite3VdbeAddOp3($538,47,$539,$540,$544)|0);
                 break;
                } else {
                 $545 = HEAP32[$0>>2]|0;
                 $546 = HEAP32[$j>>2]|0;
                 $547 = HEAP32[$pList>>2]|0;
                 $548 = (($547) + 4|0);
                 $549 = HEAP32[$548>>2]|0;
                 $550 = (($549) + (($546*20)|0)|0);
                 $551 = HEAP32[$550>>2]|0;
                 $552 = HEAP32[$regCols>>2]|0;
                 $553 = HEAP32[$i>>2]|0;
                 $554 = (($552) + ($553))|0;
                 $555 = (($554) + 1)|0;
                 _sqlite3ExprCodeAndCache($545,$551,$555);
                 break;
                }
               } else {
                label = 95;
               }
              } while(0);
              if ((label|0) == 95) {
               label = 0;
               $524 = HEAP32[$0>>2]|0;
               $525 = HEAP32[$i>>2]|0;
               $526 = HEAP32[$pTab>>2]|0;
               $527 = (($526) + 4|0);
               $528 = HEAP32[$527>>2]|0;
               $529 = (($528) + (($525*24)|0)|0);
               $530 = (($529) + 4|0);
               $531 = HEAP32[$530>>2]|0;
               $532 = HEAP32[$regCols>>2]|0;
               $533 = HEAP32[$i>>2]|0;
               $534 = (($532) + ($533))|0;
               $535 = (($534) + 1)|0;
               _sqlite3ExprCode($524,$531,$535);
              }
              $556 = HEAP32[$i>>2]|0;
              $557 = (($556) + 1)|0;
              HEAP32[$i>>2] = $557;
             }
             $558 = HEAP32[$isView>>2]|0;
             $559 = ($558|0)!=(0);
             if (!($559)) {
              $560 = HEAP32[$v>>2]|0;
              $561 = HEAP32[$pTab>>2]|0;
              $562 = HEAP32[$regCols>>2]|0;
              $563 = (($562) + 1)|0;
              _sqlite3TableAffinity($560,$561,$563);
             }
             $564 = HEAP32[$0>>2]|0;
             $565 = HEAP32[$pTrigger>>2]|0;
             $566 = HEAP32[$pTab>>2]|0;
             $567 = HEAP32[$regCols>>2]|0;
             $568 = HEAP32[$pTab>>2]|0;
             $569 = (($568) + 38|0);
             $570 = HEAP16[$569>>1]|0;
             $571 = $570 << 16 >> 16;
             $572 = (($567) - ($571))|0;
             $573 = (($572) - 1)|0;
             $574 = HEAP32[$4>>2]|0;
             $575 = HEAP32[$endOfLoop>>2]|0;
             _sqlite3CodeRowTrigger($564,$565,108,0,1,$566,$573,$574,$575);
             $576 = HEAP32[$0>>2]|0;
             $577 = HEAP32[$regCols>>2]|0;
             $578 = HEAP32[$pTab>>2]|0;
             $579 = (($578) + 38|0);
             $580 = HEAP16[$579>>1]|0;
             $581 = $580 << 16 >> 16;
             $582 = (($581) + 1)|0;
             _sqlite3ReleaseTempRange($576,$577,$582);
            }
            $583 = HEAP32[$isView>>2]|0;
            $584 = ($583|0)!=(0);
            do {
             if (!($584)) {
              $585 = HEAP32[$pTab>>2]|0;
              $586 = (($585) + 44|0);
              $587 = HEAP8[$586>>0]|0;
              $588 = $587&255;
              $589 = $588 & 16;
              $590 = ($589|0)!=(0);
              if ($590) {
               $591 = HEAP32[$v>>2]|0;
               $592 = HEAP32[$regIns>>2]|0;
               (_sqlite3VdbeAddOp2($591,28,0,$592)|0);
              }
              $593 = HEAP32[$ipkColumn>>2]|0;
              $594 = ($593|0)>=(0);
              L147: do {
               if ($594) {
                $595 = HEAP8[$useTempTable>>0]|0;
                $596 = ($595<<24>>24)!=(0);
                do {
                 if ($596) {
                  $597 = HEAP32[$v>>2]|0;
                  $598 = HEAP32[$srcTab>>2]|0;
                  $599 = HEAP32[$ipkColumn>>2]|0;
                  $600 = HEAP32[$regRowid>>2]|0;
                  (_sqlite3VdbeAddOp3($597,47,$598,$599,$600)|0);
                 } else {
                  $601 = HEAP32[$2>>2]|0;
                  $602 = ($601|0)!=(0|0);
                  if ($602) {
                   $603 = HEAP32[$v>>2]|0;
                   $604 = HEAP32[$regFromSelect>>2]|0;
                   $605 = HEAP32[$ipkColumn>>2]|0;
                   $606 = (($604) + ($605))|0;
                   $607 = HEAP32[$regRowid>>2]|0;
                   (_sqlite3VdbeAddOp2($603,33,$606,$607)|0);
                   break;
                  }
                  $608 = HEAP32[$0>>2]|0;
                  $609 = HEAP32[$ipkColumn>>2]|0;
                  $610 = HEAP32[$pList>>2]|0;
                  $611 = (($610) + 4|0);
                  $612 = HEAP32[$611>>2]|0;
                  $613 = (($612) + (($609*20)|0)|0);
                  $614 = HEAP32[$613>>2]|0;
                  $615 = HEAP32[$regRowid>>2]|0;
                  _sqlite3ExprCode($608,$614,$615);
                  $616 = HEAP32[$v>>2]|0;
                  $617 = (_sqlite3VdbeGetOp($616,-1)|0);
                  HEAP32[$pOp>>2] = $617;
                  $618 = HEAP32[$pOp>>2]|0;
                  $619 = ($618|0)!=(0|0);
                  if (!($619)) {
                   break;
                  }
                  $620 = HEAP32[$pOp>>2]|0;
                  $621 = HEAP8[$620>>0]|0;
                  $622 = $621&255;
                  $623 = ($622|0)==(28);
                  if (!($623)) {
                   break;
                  }
                  $624 = HEAP32[$pTab>>2]|0;
                  $625 = (($624) + 44|0);
                  $626 = HEAP8[$625>>0]|0;
                  $627 = $626&255;
                  $628 = $627 & 16;
                  $629 = ($628|0)!=(0);
                  if ($629) {
                   break;
                  }
                  HEAP8[$appendFlag>>0] = 1;
                  $630 = HEAP32[$pOp>>2]|0;
                  HEAP8[$630>>0] = 74;
                  $631 = HEAP32[$iDataCur>>2]|0;
                  $632 = HEAP32[$pOp>>2]|0;
                  $633 = (($632) + 4|0);
                  HEAP32[$633>>2] = $631;
                  $634 = HEAP32[$regRowid>>2]|0;
                  $635 = HEAP32[$pOp>>2]|0;
                  $636 = (($635) + 8|0);
                  HEAP32[$636>>2] = $634;
                  $637 = HEAP32[$regAutoinc>>2]|0;
                  $638 = HEAP32[$pOp>>2]|0;
                  $639 = (($638) + 12|0);
                  HEAP32[$639>>2] = $637;
                 }
                } while(0);
                $640 = HEAP8[$appendFlag>>0]|0;
                $641 = ($640<<24>>24)!=(0);
                if ($641) {
                 break;
                }
                $642 = HEAP32[$pTab>>2]|0;
                $643 = (($642) + 44|0);
                $644 = HEAP8[$643>>0]|0;
                $645 = $644&255;
                $646 = $645 & 16;
                $647 = ($646|0)!=(0);
                $648 = HEAP32[$v>>2]|0;
                if ($647) {
                 $657 = (_sqlite3VdbeCurrentAddr($648)|0);
                 HEAP32[$j11>>2] = $657;
                 $658 = HEAP32[$v>>2]|0;
                 $659 = HEAP32[$regRowid>>2]|0;
                 $660 = HEAP32[$j11>>2]|0;
                 $661 = (($660) + 2)|0;
                 (_sqlite3VdbeAddOp2($658,76,$659,$661)|0);
                } else {
                 $649 = HEAP32[$regRowid>>2]|0;
                 $650 = (_sqlite3VdbeAddOp1($648,77,$649)|0);
                 HEAP32[$j11>>2] = $650;
                 $651 = HEAP32[$v>>2]|0;
                 $652 = HEAP32[$iDataCur>>2]|0;
                 $653 = HEAP32[$regRowid>>2]|0;
                 $654 = HEAP32[$regAutoinc>>2]|0;
                 (_sqlite3VdbeAddOp3($651,74,$652,$653,$654)|0);
                 $655 = HEAP32[$v>>2]|0;
                 $656 = HEAP32[$j11>>2]|0;
                 _sqlite3VdbeJumpHere($655,$656);
                }
                $662 = HEAP32[$v>>2]|0;
                $663 = HEAP32[$regRowid>>2]|0;
                (_sqlite3VdbeAddOp1($662,38,$663)|0);
               } else {
                $664 = HEAP32[$pTab>>2]|0;
                $665 = (($664) + 44|0);
                $666 = HEAP8[$665>>0]|0;
                $667 = $666&255;
                $668 = $667 & 16;
                $669 = ($668|0)!=(0);
                do {
                 if (!($669)) {
                  $670 = HEAP8[$withoutRowid>>0]|0;
                  $671 = $670&255;
                  $672 = ($671|0)!=(0);
                  if ($672) {
                   break;
                  }
                  $675 = HEAP32[$v>>2]|0;
                  $676 = HEAP32[$iDataCur>>2]|0;
                  $677 = HEAP32[$regRowid>>2]|0;
                  $678 = HEAP32[$regAutoinc>>2]|0;
                  (_sqlite3VdbeAddOp3($675,74,$676,$677,$678)|0);
                  HEAP8[$appendFlag>>0] = 1;
                  break L147;
                 }
                } while(0);
                $673 = HEAP32[$v>>2]|0;
                $674 = HEAP32[$regRowid>>2]|0;
                (_sqlite3VdbeAddOp2($673,28,0,$674)|0);
               }
              } while(0);
              $679 = HEAP32[$0>>2]|0;
              $680 = HEAP32[$regAutoinc>>2]|0;
              $681 = HEAP32[$regRowid>>2]|0;
              _autoIncStep($679,$680,$681);
              HEAP32[$nHidden>>2] = 0;
              HEAP32[$i>>2] = 0;
              while(1) {
               $682 = HEAP32[$i>>2]|0;
               $683 = HEAP32[$pTab>>2]|0;
               $684 = (($683) + 38|0);
               $685 = HEAP16[$684>>1]|0;
               $686 = $685 << 16 >> 16;
               $687 = ($682|0)<($686|0);
               if (!($687)) {
                break;
               }
               $688 = HEAP32[$regRowid>>2]|0;
               $689 = (($688) + 1)|0;
               $690 = HEAP32[$i>>2]|0;
               $691 = (($689) + ($690))|0;
               HEAP32[$iRegStore>>2] = $691;
               $692 = HEAP32[$i>>2]|0;
               $693 = HEAP32[$pTab>>2]|0;
               $694 = (($693) + 36|0);
               $695 = HEAP16[$694>>1]|0;
               $696 = $695 << 16 >> 16;
               $697 = ($692|0)==($696|0);
               L173: do {
                if ($697) {
                 $698 = HEAP32[$v>>2]|0;
                 $699 = HEAP32[$iRegStore>>2]|0;
                 (_sqlite3VdbeAddOp1($698,29,$699)|0);
                } else {
                 $700 = HEAP32[$3>>2]|0;
                 $701 = ($700|0)==(0|0);
                 L176: do {
                  if ($701) {
                   $702 = HEAP32[$i>>2]|0;
                   $703 = HEAP32[$pTab>>2]|0;
                   $704 = (($703) + 4|0);
                   $705 = HEAP32[$704>>2]|0;
                   $706 = (($705) + (($702*24)|0)|0);
                   $707 = (($706) + 23|0);
                   $708 = HEAP8[$707>>0]|0;
                   $709 = $708&255;
                   $710 = $709 & 2;
                   $711 = ($710|0)!=(0);
                   if ($711) {
                    HEAP32[$j>>2] = -1;
                    $712 = HEAP32[$nHidden>>2]|0;
                    $713 = (($712) + 1)|0;
                    HEAP32[$nHidden>>2] = $713;
                    break;
                   } else {
                    $714 = HEAP32[$i>>2]|0;
                    $715 = HEAP32[$nHidden>>2]|0;
                    $716 = (($714) - ($715))|0;
                    HEAP32[$j>>2] = $716;
                    break;
                   }
                  } else {
                   HEAP32[$j>>2] = 0;
                   while(1) {
                    $717 = HEAP32[$j>>2]|0;
                    $718 = HEAP32[$3>>2]|0;
                    $719 = (($718) + 4|0);
                    $720 = HEAP32[$719>>2]|0;
                    $721 = ($717|0)<($720|0);
                    if (!($721)) {
                     break L176;
                    }
                    $722 = HEAP32[$j>>2]|0;
                    $723 = HEAP32[$3>>2]|0;
                    $724 = HEAP32[$723>>2]|0;
                    $725 = (($724) + ($722<<3)|0);
                    $726 = (($725) + 4|0);
                    $727 = HEAP32[$726>>2]|0;
                    $728 = HEAP32[$i>>2]|0;
                    $729 = ($727|0)==($728|0);
                    if ($729) {
                     break L176;
                    }
                    $730 = HEAP32[$j>>2]|0;
                    $731 = (($730) + 1)|0;
                    HEAP32[$j>>2] = $731;
                   }
                  }
                 } while(0);
                 $732 = HEAP32[$j>>2]|0;
                 $733 = ($732|0)<(0);
                 $734 = HEAP32[$nColumn>>2]|0;
                 $735 = ($734|0)==(0);
                 $or$cond9 = $733 | $735;
                 do {
                  if (!($or$cond9)) {
                   $736 = HEAP32[$3>>2]|0;
                   $737 = ($736|0)!=(0|0);
                   if ($737) {
                    $738 = HEAP32[$j>>2]|0;
                    $739 = HEAP32[$3>>2]|0;
                    $740 = (($739) + 4|0);
                    $741 = HEAP32[$740>>2]|0;
                    $742 = ($738|0)>=($741|0);
                    if ($742) {
                     break;
                    }
                   }
                   $752 = HEAP8[$useTempTable>>0]|0;
                   $753 = ($752<<24>>24)!=(0);
                   if ($753) {
                    $754 = HEAP32[$v>>2]|0;
                    $755 = HEAP32[$srcTab>>2]|0;
                    $756 = HEAP32[$j>>2]|0;
                    $757 = HEAP32[$iRegStore>>2]|0;
                    (_sqlite3VdbeAddOp3($754,47,$755,$756,$757)|0);
                    break L173;
                   }
                   $758 = HEAP32[$2>>2]|0;
                   $759 = ($758|0)!=(0|0);
                   if (!($759)) {
                    $768 = HEAP32[$0>>2]|0;
                    $769 = HEAP32[$j>>2]|0;
                    $770 = HEAP32[$pList>>2]|0;
                    $771 = (($770) + 4|0);
                    $772 = HEAP32[$771>>2]|0;
                    $773 = (($772) + (($769*20)|0)|0);
                    $774 = HEAP32[$773>>2]|0;
                    $775 = HEAP32[$iRegStore>>2]|0;
                    _sqlite3ExprCode($768,$774,$775);
                    break L173;
                   }
                   $760 = HEAP32[$regFromSelect>>2]|0;
                   $761 = HEAP32[$regData>>2]|0;
                   $762 = ($760|0)!=($761|0);
                   if (!($762)) {
                    break L173;
                   }
                   $763 = HEAP32[$v>>2]|0;
                   $764 = HEAP32[$regFromSelect>>2]|0;
                   $765 = HEAP32[$j>>2]|0;
                   $766 = (($764) + ($765))|0;
                   $767 = HEAP32[$iRegStore>>2]|0;
                   (_sqlite3VdbeAddOp2($763,34,$766,$767)|0);
                   break L173;
                  }
                 } while(0);
                 $743 = HEAP32[$0>>2]|0;
                 $744 = HEAP32[$i>>2]|0;
                 $745 = HEAP32[$pTab>>2]|0;
                 $746 = (($745) + 4|0);
                 $747 = HEAP32[$746>>2]|0;
                 $748 = (($747) + (($744*24)|0)|0);
                 $749 = (($748) + 4|0);
                 $750 = HEAP32[$749>>2]|0;
                 $751 = HEAP32[$iRegStore>>2]|0;
                 _sqlite3ExprCodeFactorable($743,$750,$751);
                }
               } while(0);
               $776 = HEAP32[$i>>2]|0;
               $777 = (($776) + 1)|0;
               HEAP32[$i>>2] = $777;
              }
              $778 = HEAP32[$pTab>>2]|0;
              $779 = (($778) + 44|0);
              $780 = HEAP8[$779>>0]|0;
              $781 = $780&255;
              $782 = $781 & 16;
              $783 = ($782|0)!=(0);
              if ($783) {
               $784 = HEAP32[$db>>2]|0;
               $785 = HEAP32[$pTab>>2]|0;
               $786 = (_sqlite3GetVTable($784,$785)|0);
               HEAP32[$pVTab>>2] = $786;
               $787 = HEAP32[$0>>2]|0;
               $788 = HEAP32[$pTab>>2]|0;
               _sqlite3VtabMakeWritable($787,$788);
               $789 = HEAP32[$v>>2]|0;
               $790 = HEAP32[$pTab>>2]|0;
               $791 = (($790) + 38|0);
               $792 = HEAP16[$791>>1]|0;
               $793 = $792 << 16 >> 16;
               $794 = (($793) + 2)|0;
               $795 = HEAP32[$regIns>>2]|0;
               $796 = HEAP32[$pVTab>>2]|0;
               (_sqlite3VdbeAddOp4($789,15,1,$794,$795,$796,-10)|0);
               $797 = HEAP32[$v>>2]|0;
               $798 = HEAP32[$4>>2]|0;
               $799 = ($798|0)==(10);
               $800 = HEAP32[$4>>2]|0;
               $801 = $799 ? 2 : $800;
               $802 = $801&255;
               _sqlite3VdbeChangeP5($797,$802);
               $803 = HEAP32[$0>>2]|0;
               _sqlite3MayAbort($803);
               break;
              } else {
               $804 = HEAP32[$0>>2]|0;
               $805 = HEAP32[$pTab>>2]|0;
               $806 = HEAP32[$aRegIdx>>2]|0;
               $807 = HEAP32[$iDataCur>>2]|0;
               $808 = HEAP32[$iIdxCur>>2]|0;
               $809 = HEAP32[$regIns>>2]|0;
               $810 = HEAP32[$ipkColumn>>2]|0;
               $811 = ($810|0)>=(0);
               $812 = $811&1;
               $813 = $812&255;
               $814 = HEAP32[$4>>2]|0;
               $815 = $814&255;
               $816 = HEAP32[$endOfLoop>>2]|0;
               _sqlite3GenerateConstraintChecks($804,$805,$806,$807,$808,$809,0,$813,$815,$816,$isReplace);
               $817 = HEAP32[$0>>2]|0;
               $818 = HEAP32[$pTab>>2]|0;
               $819 = HEAP32[$regIns>>2]|0;
               _sqlite3FkCheck($817,$818,0,$819,0,0);
               $820 = HEAP32[$0>>2]|0;
               $821 = HEAP32[$pTab>>2]|0;
               $822 = HEAP32[$iDataCur>>2]|0;
               $823 = HEAP32[$iIdxCur>>2]|0;
               $824 = HEAP32[$regIns>>2]|0;
               $825 = HEAP32[$aRegIdx>>2]|0;
               $826 = HEAP8[$appendFlag>>0]|0;
               $827 = $826&255;
               $828 = HEAP32[$isReplace>>2]|0;
               $829 = ($828|0)==(0);
               $830 = $829&1;
               _sqlite3CompleteInsertion($820,$821,$822,$823,$824,$825,0,$827,$830);
               break;
              }
             }
            } while(0);
            $831 = HEAP32[$db>>2]|0;
            $832 = (($831) + 24|0);
            $833 = HEAP32[$832>>2]|0;
            $834 = $833 & 128;
            $835 = ($834|0)!=(0);
            if ($835) {
             $836 = HEAP32[$v>>2]|0;
             $837 = HEAP32[$regRowCount>>2]|0;
             (_sqlite3VdbeAddOp2($836,37,$837,1)|0);
            }
            $838 = HEAP32[$pTrigger>>2]|0;
            $839 = ($838|0)!=(0|0);
            if ($839) {
             $840 = HEAP32[$0>>2]|0;
             $841 = HEAP32[$pTrigger>>2]|0;
             $842 = HEAP32[$pTab>>2]|0;
             $843 = HEAP32[$regData>>2]|0;
             $844 = (($843) - 2)|0;
             $845 = HEAP32[$pTab>>2]|0;
             $846 = (($845) + 38|0);
             $847 = HEAP16[$846>>1]|0;
             $848 = $847 << 16 >> 16;
             $849 = (($844) - ($848))|0;
             $850 = HEAP32[$4>>2]|0;
             $851 = HEAP32[$endOfLoop>>2]|0;
             _sqlite3CodeRowTrigger($840,$841,108,0,2,$842,$849,$850,$851);
            }
            $852 = HEAP32[$v>>2]|0;
            $853 = HEAP32[$endOfLoop>>2]|0;
            _sqlite3VdbeResolveLabel($852,$853);
            $854 = HEAP8[$useTempTable>>0]|0;
            $855 = ($854<<24>>24)!=(0);
            do {
             if ($855) {
              $856 = HEAP32[$v>>2]|0;
              $857 = HEAP32[$srcTab>>2]|0;
              $858 = HEAP32[$addrCont>>2]|0;
              (_sqlite3VdbeAddOp2($856,9,$857,$858)|0);
              $859 = HEAP32[$v>>2]|0;
              $860 = HEAP32[$addrInsTop>>2]|0;
              _sqlite3VdbeJumpHere($859,$860);
              $861 = HEAP32[$v>>2]|0;
              $862 = HEAP32[$srcTab>>2]|0;
              (_sqlite3VdbeAddOp1($861,61,$862)|0);
             } else {
              $863 = HEAP32[$2>>2]|0;
              $864 = ($863|0)!=(0|0);
              if (!($864)) {
               break;
              }
              $865 = HEAP32[$v>>2]|0;
              $866 = HEAP32[$addrCont>>2]|0;
              (_sqlite3VdbeAddOp2($865,16,0,$866)|0);
              $867 = HEAP32[$v>>2]|0;
              $868 = HEAP32[$addrInsTop>>2]|0;
              _sqlite3VdbeJumpHere($867,$868);
             }
            } while(0);
            $869 = HEAP32[$pTab>>2]|0;
            $870 = (($869) + 44|0);
            $871 = HEAP8[$870>>0]|0;
            $872 = $871&255;
            $873 = $872 & 16;
            $874 = ($873|0)!=(0);
            $875 = HEAP32[$isView>>2]|0;
            $876 = ($875|0)!=(0);
            $or$cond11 = $874 | $876;
            if ($or$cond11) {
             break;
            }
            $877 = HEAP32[$iDataCur>>2]|0;
            $878 = HEAP32[$iIdxCur>>2]|0;
            $879 = ($877|0)<($878|0);
            if ($879) {
             $880 = HEAP32[$v>>2]|0;
             $881 = HEAP32[$iDataCur>>2]|0;
             (_sqlite3VdbeAddOp1($880,61,$881)|0);
            }
            HEAP32[$idx>>2] = 0;
            $882 = HEAP32[$pTab>>2]|0;
            $883 = (($882) + 8|0);
            $884 = HEAP32[$883>>2]|0;
            HEAP32[$pIdx>>2] = $884;
            while(1) {
             $885 = HEAP32[$pIdx>>2]|0;
             $886 = ($885|0)!=(0|0);
             if (!($886)) {
              break L23;
             }
             $887 = HEAP32[$v>>2]|0;
             $888 = HEAP32[$idx>>2]|0;
             $889 = HEAP32[$iIdxCur>>2]|0;
             $890 = (($888) + ($889))|0;
             (_sqlite3VdbeAddOp1($887,61,$890)|0);
             $891 = HEAP32[$pIdx>>2]|0;
             $892 = (($891) + 20|0);
             $893 = HEAP32[$892>>2]|0;
             HEAP32[$pIdx>>2] = $893;
             $894 = HEAP32[$idx>>2]|0;
             $895 = (($894) + 1)|0;
             HEAP32[$idx>>2] = $895;
            }
           }
          } while(0);
          $896 = HEAP32[$0>>2]|0;
          $897 = (($896) + 18|0);
          $898 = HEAP8[$897>>0]|0;
          $899 = $898&255;
          $900 = ($899|0)==(0);
          if ($900) {
           $901 = HEAP32[$0>>2]|0;
           $902 = (($901) + 412|0);
           $903 = HEAP32[$902>>2]|0;
           $904 = ($903|0)==(0|0);
           if ($904) {
            $905 = HEAP32[$0>>2]|0;
            _sqlite3AutoincrementEnd($905);
           }
          }
          $906 = HEAP32[$db>>2]|0;
          $907 = (($906) + 24|0);
          $908 = HEAP32[$907>>2]|0;
          $909 = $908 & 128;
          $910 = ($909|0)!=(0);
          if ($910) {
           $911 = HEAP32[$0>>2]|0;
           $912 = (($911) + 18|0);
           $913 = HEAP8[$912>>0]|0;
           $914 = ($913<<24>>24)!=(0);
           if (!($914)) {
            $915 = HEAP32[$0>>2]|0;
            $916 = (($915) + 412|0);
            $917 = HEAP32[$916>>2]|0;
            $918 = ($917|0)!=(0|0);
            if (!($918)) {
             $919 = HEAP32[$v>>2]|0;
             $920 = HEAP32[$regRowCount>>2]|0;
             (_sqlite3VdbeAddOp2($919,35,$920,1)|0);
             $921 = HEAP32[$v>>2]|0;
             _sqlite3VdbeSetNumCols($921,1);
             $922 = HEAP32[$v>>2]|0;
             (_sqlite3VdbeSetColName($922,0,0,23552,0)|0);
            }
           }
          }
         }
        }
       }
      }
     }
    }
   }
  }
 } while(0);
 $923 = HEAP32[$db>>2]|0;
 $924 = HEAP32[$1>>2]|0;
 _sqlite3SrcListDelete($923,$924);
 $925 = HEAP32[$db>>2]|0;
 $926 = HEAP32[$pList>>2]|0;
 _sqlite3ExprListDelete($925,$926);
 $927 = HEAP32[$db>>2]|0;
 $928 = HEAP32[$2>>2]|0;
 _sqlite3SelectDelete($927,$928);
 $929 = HEAP32[$db>>2]|0;
 $930 = HEAP32[$3>>2]|0;
 _sqlite3IdListDelete($929,$930);
 $931 = HEAP32[$db>>2]|0;
 $932 = HEAP32[$aRegIdx>>2]|0;
 _sqlite3DbFree($931,$932);
 STACKTOP = sp;return;
}
function _sqlite3IdListAppend($db,$pList,$pToken) {
 $db = $db|0;
 $pList = $pList|0;
 $pToken = $pToken|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $i = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 16|0;
 $1 = sp + 12|0;
 $2 = sp + 8|0;
 $3 = sp + 4|0;
 $i = sp;
 HEAP32[$1>>2] = $db;
 HEAP32[$2>>2] = $pList;
 HEAP32[$3>>2] = $pToken;
 $4 = HEAP32[$2>>2]|0;
 $5 = ($4|0)==(0|0);
 if ($5) {
  $6 = HEAP32[$1>>2]|0;
  $7 = (_sqlite3DbMallocZero($6,8,0)|0);
  HEAP32[$2>>2] = $7;
  $8 = HEAP32[$2>>2]|0;
  $9 = ($8|0)==(0|0);
  if ($9) {
   HEAP32[$0>>2] = 0;
   $28 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($28|0);
  }
 }
 $10 = HEAP32[$1>>2]|0;
 $11 = HEAP32[$2>>2]|0;
 $12 = HEAP32[$11>>2]|0;
 $13 = HEAP32[$2>>2]|0;
 $14 = (($13) + 4|0);
 $15 = (_sqlite3ArrayAllocate($10,$12,8,$14,$i)|0);
 $16 = HEAP32[$2>>2]|0;
 HEAP32[$16>>2] = $15;
 $17 = HEAP32[$i>>2]|0;
 $18 = ($17|0)<(0);
 $19 = HEAP32[$1>>2]|0;
 if ($18) {
  $20 = HEAP32[$2>>2]|0;
  _sqlite3IdListDelete($19,$20);
  HEAP32[$0>>2] = 0;
  $28 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($28|0);
 } else {
  $21 = HEAP32[$3>>2]|0;
  $22 = (_sqlite3NameFromToken($19,$21)|0);
  $23 = HEAP32[$i>>2]|0;
  $24 = HEAP32[$2>>2]|0;
  $25 = HEAP32[$24>>2]|0;
  $26 = (($25) + ($23<<3)|0);
  HEAP32[$26>>2] = $22;
  $27 = HEAP32[$2>>2]|0;
  HEAP32[$0>>2] = $27;
  $28 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($28|0);
 }
 return 0|0;
}
function _spanSet($pOut,$pStart,$pEnd) {
 $pOut = $pOut|0;
 $pStart = $pStart|0;
 $pEnd = $pEnd|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $1 = sp + 4|0;
 $2 = sp;
 HEAP32[$0>>2] = $pOut;
 HEAP32[$1>>2] = $pStart;
 HEAP32[$2>>2] = $pEnd;
 $3 = HEAP32[$1>>2]|0;
 $4 = HEAP32[$3>>2]|0;
 $5 = HEAP32[$0>>2]|0;
 $6 = (($5) + 4|0);
 HEAP32[$6>>2] = $4;
 $7 = HEAP32[$2>>2]|0;
 $8 = (($7) + 4|0);
 $9 = HEAP32[$8>>2]|0;
 $10 = HEAP32[$2>>2]|0;
 $11 = HEAP32[$10>>2]|0;
 $12 = (($11) + ($9)|0);
 $13 = HEAP32[$0>>2]|0;
 $14 = (($13) + 8|0);
 HEAP32[$14>>2] = $12;
 STACKTOP = sp;return;
}
function _sqlite3ExprAssignVarNumber($pParse,$pExpr) {
 $pParse = $pParse|0;
 $pExpr = $pExpr|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0;
 var $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0, $188 = 0;
 var $189 = 0, $19 = 0, $190 = 0, $191 = 0, $192 = 0, $193 = 0, $194 = 0, $195 = 0, $196 = 0, $197 = 0, $198 = 0, $199 = 0, $2 = 0, $20 = 0, $200 = 0, $201 = 0, $202 = 0, $203 = 0, $204 = 0, $205 = 0;
 var $206 = 0, $207 = 0, $208 = 0, $209 = 0, $21 = 0, $210 = 0, $211 = 0, $212 = 0, $213 = 0, $214 = 0, $215 = 0, $216 = 0, $217 = 0, $218 = 0, $219 = 0, $22 = 0, $220 = 0, $221 = 0, $222 = 0, $223 = 0;
 var $224 = 0, $225 = 0, $226 = 0, $227 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0;
 var $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0;
 var $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0;
 var $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0;
 var $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $a = 0, $bOk = 0, $db = 0, $i = 0, $i1 = 0, $n = 0, $or$cond = 0, $vararg_buffer2 = 0, $x = 0, $z = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $vararg_buffer2 = sp + 8|0;
 $0 = sp + 36|0;
 $1 = sp + 32|0;
 $db = sp + 28|0;
 $z = sp + 24|0;
 $x = sp + 42|0;
 $n = sp + 20|0;
 $i = sp;
 $bOk = sp + 16|0;
 $i1 = sp + 40|0;
 $a = sp + 12|0;
 HEAP32[$0>>2] = $pParse;
 HEAP32[$1>>2] = $pExpr;
 $2 = HEAP32[$0>>2]|0;
 $3 = HEAP32[$2>>2]|0;
 HEAP32[$db>>2] = $3;
 $4 = HEAP32[$1>>2]|0;
 $5 = ($4|0)==(0|0);
 if ($5) {
  STACKTOP = sp;return;
 }
 $6 = HEAP32[$1>>2]|0;
 $7 = (($6) + 8|0);
 $8 = HEAP32[$7>>2]|0;
 HEAP32[$z>>2] = $8;
 $9 = HEAP32[$z>>2]|0;
 $10 = (($9) + 1|0);
 $11 = HEAP8[$10>>0]|0;
 $12 = $11 << 24 >> 24;
 $13 = ($12|0)==(0);
 do {
  if ($13) {
   $14 = HEAP32[$0>>2]|0;
   $15 = (($14) + 440|0);
   $16 = HEAP32[$15>>2]|0;
   $17 = (($16) + 1)|0;
   HEAP32[$15>>2] = $17;
   $18 = $17&65535;
   $19 = HEAP32[$1>>2]|0;
   $20 = (($19) + 32|0);
   HEAP16[$20>>1] = $18;
  } else {
   HEAP16[$x>>1] = 0;
   $21 = HEAP32[$z>>2]|0;
   $22 = (_sqlite3Strlen30($21)|0);
   HEAP32[$n>>2] = $22;
   $23 = HEAP32[$z>>2]|0;
   $24 = HEAP8[$23>>0]|0;
   $25 = $24 << 24 >> 24;
   $26 = ($25|0)==(63);
   if ($26) {
    $27 = HEAP32[$z>>2]|0;
    $28 = (($27) + 1|0);
    $29 = HEAP32[$n>>2]|0;
    $30 = (($29) - 1)|0;
    $31 = (_sqlite3Atoi64($28,$i,$30,1)|0);
    $32 = (0)==($31|0);
    $33 = $32&1;
    HEAP32[$bOk>>2] = $33;
    $34 = $i;
    $35 = $34;
    $36 = HEAP32[$35>>2]|0;
    $37 = (($34) + 4)|0;
    $38 = $37;
    $39 = HEAP32[$38>>2]|0;
    $40 = $36&65535;
    HEAP16[$x>>1] = $40;
    $41 = HEAP32[$1>>2]|0;
    $42 = (($41) + 32|0);
    HEAP16[$42>>1] = $40;
    $43 = HEAP32[$bOk>>2]|0;
    $44 = ($43|0)==(0);
    $45 = $i;
    $46 = $45;
    $47 = HEAP32[$46>>2]|0;
    $48 = (($45) + 4)|0;
    $49 = $48;
    $50 = HEAP32[$49>>2]|0;
    $51 = ($50|0)<(0);
    $52 = ($47>>>0)<(1);
    $53 = ($50|0)==(0);
    $54 = $53 & $52;
    $55 = $51 | $54;
    $or$cond = $44 | $55;
    if ($or$cond) {
     label = 7;
    } else {
     $56 = $i;
     $57 = $56;
     $58 = HEAP32[$57>>2]|0;
     $59 = (($56) + 4)|0;
     $60 = $59;
     $61 = HEAP32[$60>>2]|0;
     $62 = HEAP32[$db>>2]|0;
     $63 = (($62) + 88|0);
     $64 = (($63) + 36|0);
     $65 = HEAP32[$64>>2]|0;
     $66 = ($65|0)<(0);
     $67 = $66 << 31 >> 31;
     $68 = ($61|0)>($67|0);
     $69 = ($58>>>0)>($65>>>0);
     $70 = ($61|0)==($67|0);
     $71 = $70 & $69;
     $72 = $68 | $71;
     if ($72) {
      label = 7;
     }
    }
    if ((label|0) == 7) {
     $73 = HEAP32[$0>>2]|0;
     $74 = HEAP32[$db>>2]|0;
     $75 = (($74) + 88|0);
     $76 = (($75) + 36|0);
     $77 = HEAP32[$76>>2]|0;
     HEAP32[$vararg_buffer2>>2] = $77;
     _sqlite3ErrorMsg($73,23360,$vararg_buffer2);
     HEAP16[$x>>1] = 0;
    }
    $78 = $i;
    $79 = $78;
    $80 = HEAP32[$79>>2]|0;
    $81 = (($78) + 4)|0;
    $82 = $81;
    $83 = HEAP32[$82>>2]|0;
    $84 = HEAP32[$0>>2]|0;
    $85 = (($84) + 440|0);
    $86 = HEAP32[$85>>2]|0;
    $87 = ($86|0)<(0);
    $88 = $87 << 31 >> 31;
    $89 = ($83|0)>($88|0);
    $90 = ($80>>>0)>($86>>>0);
    $91 = ($83|0)==($88|0);
    $92 = $91 & $90;
    $93 = $89 | $92;
    if ($93) {
     $94 = $i;
     $95 = $94;
     $96 = HEAP32[$95>>2]|0;
     $97 = (($94) + 4)|0;
     $98 = $97;
     $99 = HEAP32[$98>>2]|0;
     $100 = HEAP32[$0>>2]|0;
     $101 = (($100) + 440|0);
     HEAP32[$101>>2] = $96;
    }
   } else {
    HEAP16[$i1>>1] = 0;
    while(1) {
     $102 = HEAP16[$i1>>1]|0;
     $103 = $102 << 16 >> 16;
     $104 = HEAP32[$0>>2]|0;
     $105 = (($104) + 444|0);
     $106 = HEAP32[$105>>2]|0;
     $107 = ($103|0)<($106|0);
     if (!($107)) {
      break;
     }
     $108 = HEAP16[$i1>>1]|0;
     $109 = $108 << 16 >> 16;
     $110 = HEAP32[$0>>2]|0;
     $111 = (($110) + 472|0);
     $112 = HEAP32[$111>>2]|0;
     $113 = (($112) + ($109<<2)|0);
     $114 = HEAP32[$113>>2]|0;
     $115 = ($114|0)!=(0|0);
     if ($115) {
      $116 = HEAP16[$i1>>1]|0;
      $117 = $116 << 16 >> 16;
      $118 = HEAP32[$0>>2]|0;
      $119 = (($118) + 472|0);
      $120 = HEAP32[$119>>2]|0;
      $121 = (($120) + ($117<<2)|0);
      $122 = HEAP32[$121>>2]|0;
      $123 = HEAP32[$z>>2]|0;
      $124 = (_strcmp($122,$123)|0);
      $125 = ($124|0)==(0);
      if ($125) {
       label = 14;
       break;
      }
     }
     $132 = HEAP16[$i1>>1]|0;
     $133 = (($132) + 1)<<16>>16;
     HEAP16[$i1>>1] = $133;
    }
    if ((label|0) == 14) {
     $126 = HEAP16[$i1>>1]|0;
     $127 = $126 << 16 >> 16;
     $128 = (($127) + 1)|0;
     $129 = $128&65535;
     HEAP16[$x>>1] = $129;
     $130 = HEAP32[$1>>2]|0;
     $131 = (($130) + 32|0);
     HEAP16[$131>>1] = $129;
    }
    $134 = HEAP16[$x>>1]|0;
    $135 = $134 << 16 >> 16;
    $136 = ($135|0)==(0);
    if ($136) {
     $137 = HEAP32[$0>>2]|0;
     $138 = (($137) + 440|0);
     $139 = HEAP32[$138>>2]|0;
     $140 = (($139) + 1)|0;
     HEAP32[$138>>2] = $140;
     $141 = $140&65535;
     $142 = HEAP32[$1>>2]|0;
     $143 = (($142) + 32|0);
     HEAP16[$143>>1] = $141;
     HEAP16[$x>>1] = $141;
    }
   }
   $144 = HEAP16[$x>>1]|0;
   $145 = $144 << 16 >> 16;
   $146 = ($145|0)>(0);
   if ($146) {
    $147 = HEAP16[$x>>1]|0;
    $148 = $147 << 16 >> 16;
    $149 = HEAP32[$0>>2]|0;
    $150 = (($149) + 444|0);
    $151 = HEAP32[$150>>2]|0;
    $152 = ($148|0)>($151|0);
    do {
     if ($152) {
      $153 = HEAP32[$db>>2]|0;
      $154 = HEAP32[$0>>2]|0;
      $155 = (($154) + 472|0);
      $156 = HEAP32[$155>>2]|0;
      $157 = HEAP16[$x>>1]|0;
      $158 = $157 << 16 >> 16;
      $159 = $158<<2;
      $160 = (_sqlite3DbRealloc($153,$156,$159,0)|0);
      HEAP32[$a>>2] = $160;
      $161 = HEAP32[$a>>2]|0;
      $162 = ($161|0)==(0|0);
      if ($162) {
       STACKTOP = sp;return;
      } else {
       $163 = HEAP32[$a>>2]|0;
       $164 = HEAP32[$0>>2]|0;
       $165 = (($164) + 472|0);
       HEAP32[$165>>2] = $163;
       $166 = HEAP32[$0>>2]|0;
       $167 = (($166) + 444|0);
       $168 = HEAP32[$167>>2]|0;
       $169 = HEAP32[$a>>2]|0;
       $170 = (($169) + ($168<<2)|0);
       $171 = HEAP16[$x>>1]|0;
       $172 = $171 << 16 >> 16;
       $173 = HEAP32[$0>>2]|0;
       $174 = (($173) + 444|0);
       $175 = HEAP32[$174>>2]|0;
       $176 = (($172) - ($175))|0;
       $177 = $176<<2;
       _memset(($170|0),0,($177|0))|0;
       $178 = HEAP16[$x>>1]|0;
       $179 = $178 << 16 >> 16;
       $180 = HEAP32[$0>>2]|0;
       $181 = (($180) + 444|0);
       HEAP32[$181>>2] = $179;
       break;
      }
     }
    } while(0);
    $182 = HEAP32[$z>>2]|0;
    $183 = HEAP8[$182>>0]|0;
    $184 = $183 << 24 >> 24;
    $185 = ($184|0)!=(63);
    if (!($185)) {
     $186 = HEAP16[$x>>1]|0;
     $187 = $186 << 16 >> 16;
     $188 = (($187) - 1)|0;
     $189 = HEAP32[$0>>2]|0;
     $190 = (($189) + 472|0);
     $191 = HEAP32[$190>>2]|0;
     $192 = (($191) + ($188<<2)|0);
     $193 = HEAP32[$192>>2]|0;
     $194 = ($193|0)==(0|0);
     if (!($194)) {
      break;
     }
    }
    $195 = HEAP32[$db>>2]|0;
    $196 = HEAP16[$x>>1]|0;
    $197 = $196 << 16 >> 16;
    $198 = (($197) - 1)|0;
    $199 = HEAP32[$0>>2]|0;
    $200 = (($199) + 472|0);
    $201 = HEAP32[$200>>2]|0;
    $202 = (($201) + ($198<<2)|0);
    $203 = HEAP32[$202>>2]|0;
    _sqlite3DbFree($195,$203);
    $204 = HEAP32[$db>>2]|0;
    $205 = HEAP32[$z>>2]|0;
    $206 = HEAP32[$n>>2]|0;
    $207 = (_sqlite3DbStrNDup($204,$205,$206,0)|0);
    $208 = HEAP16[$x>>1]|0;
    $209 = $208 << 16 >> 16;
    $210 = (($209) - 1)|0;
    $211 = HEAP32[$0>>2]|0;
    $212 = (($211) + 472|0);
    $213 = HEAP32[$212>>2]|0;
    $214 = (($213) + ($210<<2)|0);
    HEAP32[$214>>2] = $207;
   }
  }
 } while(0);
 $215 = HEAP32[$0>>2]|0;
 $216 = (($215) + 64|0);
 $217 = HEAP32[$216>>2]|0;
 $218 = ($217|0)!=(0);
 if ($218) {
  STACKTOP = sp;return;
 }
 $219 = HEAP32[$0>>2]|0;
 $220 = (($219) + 440|0);
 $221 = HEAP32[$220>>2]|0;
 $222 = HEAP32[$db>>2]|0;
 $223 = (($222) + 88|0);
 $224 = (($223) + 36|0);
 $225 = HEAP32[$224>>2]|0;
 $226 = ($221|0)>($225|0);
 if (!($226)) {
  STACKTOP = sp;return;
 }
 $227 = HEAP32[$0>>2]|0;
 _sqlite3ErrorMsg($227,23408,$vararg_buffer2);
 STACKTOP = sp;return;
}
function _sqlite3ExprAddCollateToken($pParse,$pExpr,$pCollName) {
 $pParse = $pParse|0;
 $pExpr = $pExpr|0;
 $pCollName = $pCollName|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0;
 var $8 = 0, $9 = 0, $pNew = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $1 = sp + 8|0;
 $2 = sp + 4|0;
 $pNew = sp;
 HEAP32[$0>>2] = $pParse;
 HEAP32[$1>>2] = $pExpr;
 HEAP32[$2>>2] = $pCollName;
 $3 = HEAP32[$2>>2]|0;
 $4 = (($3) + 4|0);
 $5 = HEAP32[$4>>2]|0;
 $6 = ($5>>>0)>(0);
 if (!($6)) {
  $21 = HEAP32[$1>>2]|0;
  STACKTOP = sp;return ($21|0);
 }
 $7 = HEAP32[$0>>2]|0;
 $8 = HEAP32[$7>>2]|0;
 $9 = HEAP32[$2>>2]|0;
 $10 = (_sqlite3ExprAlloc($8,95,$9,1)|0);
 HEAP32[$pNew>>2] = $10;
 $11 = HEAP32[$pNew>>2]|0;
 $12 = ($11|0)!=(0|0);
 if (!($12)) {
  $21 = HEAP32[$1>>2]|0;
  STACKTOP = sp;return ($21|0);
 }
 $13 = HEAP32[$1>>2]|0;
 $14 = HEAP32[$pNew>>2]|0;
 $15 = (($14) + 12|0);
 HEAP32[$15>>2] = $13;
 $16 = HEAP32[$pNew>>2]|0;
 $17 = (($16) + 4|0);
 $18 = HEAP32[$17>>2]|0;
 $19 = $18 | 4352;
 HEAP32[$17>>2] = $19;
 $20 = HEAP32[$pNew>>2]|0;
 HEAP32[$1>>2] = $20;
 $21 = HEAP32[$1>>2]|0;
 STACKTOP = sp;return ($21|0);
}
function _sqlite3ExprFunction($pParse,$pList,$pToken) {
 $pParse = $pParse|0;
 $pList = $pList|0;
 $pToken = $pToken|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0;
 var $db = 0, $pNew = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 20|0;
 $1 = sp + 16|0;
 $2 = sp + 12|0;
 $3 = sp + 8|0;
 $pNew = sp + 4|0;
 $db = sp;
 HEAP32[$1>>2] = $pParse;
 HEAP32[$2>>2] = $pList;
 HEAP32[$3>>2] = $pToken;
 $4 = HEAP32[$1>>2]|0;
 $5 = HEAP32[$4>>2]|0;
 HEAP32[$db>>2] = $5;
 $6 = HEAP32[$db>>2]|0;
 $7 = HEAP32[$3>>2]|0;
 $8 = (_sqlite3ExprAlloc($6,153,$7,1)|0);
 HEAP32[$pNew>>2] = $8;
 $9 = HEAP32[$pNew>>2]|0;
 $10 = ($9|0)==(0|0);
 if ($10) {
  $11 = HEAP32[$db>>2]|0;
  $12 = HEAP32[$2>>2]|0;
  _sqlite3ExprListDelete($11,$12);
  HEAP32[$0>>2] = 0;
  $19 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($19|0);
 } else {
  $13 = HEAP32[$2>>2]|0;
  $14 = HEAP32[$pNew>>2]|0;
  $15 = (($14) + 20|0);
  HEAP32[$15>>2] = $13;
  $16 = HEAP32[$1>>2]|0;
  $17 = HEAP32[$pNew>>2]|0;
  _sqlite3ExprSetHeight($16,$17);
  $18 = HEAP32[$pNew>>2]|0;
  HEAP32[$0>>2] = $18;
  $19 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($19|0);
 }
 return 0|0;
}
function _spanBinaryExpr($pOut,$pParse,$op,$pLeft,$pRight) {
 $pOut = $pOut|0;
 $pParse = $pParse|0;
 $op = $op|0;
 $pLeft = $pLeft|0;
 $pRight = $pRight|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0;
 var $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 16|0;
 $1 = sp + 12|0;
 $2 = sp + 8|0;
 $3 = sp + 4|0;
 $4 = sp;
 HEAP32[$0>>2] = $pOut;
 HEAP32[$1>>2] = $pParse;
 HEAP32[$2>>2] = $op;
 HEAP32[$3>>2] = $pLeft;
 HEAP32[$4>>2] = $pRight;
 $5 = HEAP32[$1>>2]|0;
 $6 = HEAP32[$2>>2]|0;
 $7 = HEAP32[$3>>2]|0;
 $8 = HEAP32[$7>>2]|0;
 $9 = HEAP32[$4>>2]|0;
 $10 = HEAP32[$9>>2]|0;
 $11 = (_sqlite3PExpr($5,$6,$8,$10,0)|0);
 $12 = HEAP32[$0>>2]|0;
 HEAP32[$12>>2] = $11;
 $13 = HEAP32[$3>>2]|0;
 $14 = (($13) + 4|0);
 $15 = HEAP32[$14>>2]|0;
 $16 = HEAP32[$0>>2]|0;
 $17 = (($16) + 4|0);
 HEAP32[$17>>2] = $15;
 $18 = HEAP32[$4>>2]|0;
 $19 = (($18) + 8|0);
 $20 = HEAP32[$19>>2]|0;
 $21 = HEAP32[$0>>2]|0;
 $22 = (($21) + 8|0);
 HEAP32[$22>>2] = $20;
 STACKTOP = sp;return;
}
function _spanUnaryPostfix($pOut,$pParse,$op,$pOperand,$pPostOp) {
 $pOut = $pOut|0;
 $pParse = $pParse|0;
 $op = $op|0;
 $pOperand = $pOperand|0;
 $pPostOp = $pPostOp|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $3 = 0, $4 = 0, $5 = 0;
 var $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 16|0;
 $1 = sp + 12|0;
 $2 = sp + 8|0;
 $3 = sp + 4|0;
 $4 = sp;
 HEAP32[$0>>2] = $pOut;
 HEAP32[$1>>2] = $pParse;
 HEAP32[$2>>2] = $op;
 HEAP32[$3>>2] = $pOperand;
 HEAP32[$4>>2] = $pPostOp;
 $5 = HEAP32[$1>>2]|0;
 $6 = HEAP32[$2>>2]|0;
 $7 = HEAP32[$3>>2]|0;
 $8 = HEAP32[$7>>2]|0;
 $9 = (_sqlite3PExpr($5,$6,$8,0,0)|0);
 $10 = HEAP32[$0>>2]|0;
 HEAP32[$10>>2] = $9;
 $11 = HEAP32[$3>>2]|0;
 $12 = (($11) + 4|0);
 $13 = HEAP32[$12>>2]|0;
 $14 = HEAP32[$0>>2]|0;
 $15 = (($14) + 4|0);
 HEAP32[$15>>2] = $13;
 $16 = HEAP32[$4>>2]|0;
 $17 = (($16) + 4|0);
 $18 = HEAP32[$17>>2]|0;
 $19 = HEAP32[$4>>2]|0;
 $20 = HEAP32[$19>>2]|0;
 $21 = (($20) + ($18)|0);
 $22 = HEAP32[$0>>2]|0;
 $23 = (($22) + 8|0);
 HEAP32[$23>>2] = $21;
 STACKTOP = sp;return;
}
function _binaryToUnaryIfNull($pParse,$pY,$pA,$op) {
 $pParse = $pParse|0;
 $pY = $pY|0;
 $pA = $pA|0;
 $op = $op|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0;
 var $7 = 0, $8 = 0, $9 = 0, $db = 0, $or$cond = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 16|0;
 $1 = sp + 12|0;
 $2 = sp + 8|0;
 $3 = sp + 4|0;
 $db = sp;
 HEAP32[$0>>2] = $pParse;
 HEAP32[$1>>2] = $pY;
 HEAP32[$2>>2] = $pA;
 HEAP32[$3>>2] = $op;
 $4 = HEAP32[$0>>2]|0;
 $5 = HEAP32[$4>>2]|0;
 HEAP32[$db>>2] = $5;
 $6 = HEAP32[$1>>2]|0;
 $7 = ($6|0)!=(0|0);
 $8 = HEAP32[$2>>2]|0;
 $9 = ($8|0)!=(0|0);
 $or$cond = $7 & $9;
 if (!($or$cond)) {
  STACKTOP = sp;return;
 }
 $10 = HEAP32[$1>>2]|0;
 $11 = HEAP8[$10>>0]|0;
 $12 = $11&255;
 $13 = ($12|0)==(101);
 if (!($13)) {
  STACKTOP = sp;return;
 }
 $14 = HEAP32[$3>>2]|0;
 $15 = $14&255;
 $16 = HEAP32[$2>>2]|0;
 HEAP8[$16>>0] = $15;
 $17 = HEAP32[$db>>2]|0;
 $18 = HEAP32[$2>>2]|0;
 $19 = (($18) + 16|0);
 $20 = HEAP32[$19>>2]|0;
 _sqlite3ExprDelete($17,$20);
 $21 = HEAP32[$2>>2]|0;
 $22 = (($21) + 16|0);
 HEAP32[$22>>2] = 0;
 STACKTOP = sp;return;
}
function _spanUnaryPrefix($pOut,$pParse,$op,$pOperand,$pPreOp) {
 $pOut = $pOut|0;
 $pParse = $pParse|0;
 $op = $op|0;
 $pOperand = $pOperand|0;
 $pPreOp = $pPreOp|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0;
 var label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 16|0;
 $1 = sp + 12|0;
 $2 = sp + 8|0;
 $3 = sp + 4|0;
 $4 = sp;
 HEAP32[$0>>2] = $pOut;
 HEAP32[$1>>2] = $pParse;
 HEAP32[$2>>2] = $op;
 HEAP32[$3>>2] = $pOperand;
 HEAP32[$4>>2] = $pPreOp;
 $5 = HEAP32[$1>>2]|0;
 $6 = HEAP32[$2>>2]|0;
 $7 = HEAP32[$3>>2]|0;
 $8 = HEAP32[$7>>2]|0;
 $9 = (_sqlite3PExpr($5,$6,$8,0,0)|0);
 $10 = HEAP32[$0>>2]|0;
 HEAP32[$10>>2] = $9;
 $11 = HEAP32[$4>>2]|0;
 $12 = HEAP32[$11>>2]|0;
 $13 = HEAP32[$0>>2]|0;
 $14 = (($13) + 4|0);
 HEAP32[$14>>2] = $12;
 $15 = HEAP32[$3>>2]|0;
 $16 = (($15) + 8|0);
 $17 = HEAP32[$16>>2]|0;
 $18 = HEAP32[$0>>2]|0;
 $19 = (($18) + 8|0);
 HEAP32[$19>>2] = $17;
 STACKTOP = sp;return;
}
function _sqlite3ExprSetHeight($pParse,$p) {
 $pParse = $pParse|0;
 $p = $p|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $1 = sp;
 HEAP32[$0>>2] = $pParse;
 HEAP32[$1>>2] = $p;
 $2 = HEAP32[$1>>2]|0;
 _exprSetHeight($2);
 $3 = HEAP32[$0>>2]|0;
 $4 = HEAP32[$1>>2]|0;
 $5 = (($4) + 24|0);
 $6 = HEAP32[$5>>2]|0;
 (_sqlite3ExprCheckHeight($3,$6)|0);
 STACKTOP = sp;return;
}
function _sqlite3DropIndex($pParse,$pName,$ifExists) {
 $pParse = $pParse|0;
 $pName = $pName|0;
 $ifExists = $ifExists|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0;
 var $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0;
 var $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0;
 var $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0;
 var $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0;
 var $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $code = 0, $db = 0, $iDb = 0, $pIndex = 0, $pTab = 0, $v = 0, $vararg_buffer5 = 0, $vararg_ptr1 = 0, $vararg_ptr8 = 0, $vararg_ptr9 = 0, $zDb = 0, $zTab = 0;
 var label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 64|0;
 $vararg_buffer5 = sp;
 $0 = sp + 52|0;
 $1 = sp + 48|0;
 $2 = sp + 44|0;
 $pIndex = sp + 40|0;
 $v = sp + 36|0;
 $db = sp + 32|0;
 $iDb = sp + 28|0;
 $code = sp + 24|0;
 $pTab = sp + 20|0;
 $zDb = sp + 16|0;
 $zTab = sp + 12|0;
 HEAP32[$0>>2] = $pParse;
 HEAP32[$1>>2] = $pName;
 HEAP32[$2>>2] = $ifExists;
 $3 = HEAP32[$0>>2]|0;
 $4 = HEAP32[$3>>2]|0;
 HEAP32[$db>>2] = $4;
 $5 = HEAP32[$db>>2]|0;
 $6 = (($5) + 64|0);
 $7 = HEAP8[$6>>0]|0;
 $8 = ($7<<24>>24)!=(0);
 if ($8) {
  $106 = HEAP32[$db>>2]|0;
  $107 = HEAP32[$1>>2]|0;
  _sqlite3SrcListDelete($106,$107);
  STACKTOP = sp;return;
 }
 $9 = HEAP32[$0>>2]|0;
 $10 = (_sqlite3ReadSchema($9)|0);
 $11 = (0)!=($10|0);
 if ($11) {
  $106 = HEAP32[$db>>2]|0;
  $107 = HEAP32[$1>>2]|0;
  _sqlite3SrcListDelete($106,$107);
  STACKTOP = sp;return;
 }
 $12 = HEAP32[$db>>2]|0;
 $13 = HEAP32[$1>>2]|0;
 $14 = (($13) + 8|0);
 $15 = (($14) + 8|0);
 $16 = HEAP32[$15>>2]|0;
 $17 = HEAP32[$1>>2]|0;
 $18 = (($17) + 8|0);
 $19 = (($18) + 4|0);
 $20 = HEAP32[$19>>2]|0;
 $21 = (_sqlite3FindIndex($12,$16,$20)|0);
 HEAP32[$pIndex>>2] = $21;
 $22 = HEAP32[$pIndex>>2]|0;
 $23 = ($22|0)==(0|0);
 if ($23) {
  $24 = HEAP32[$2>>2]|0;
  $25 = ($24|0)!=(0);
  $26 = HEAP32[$0>>2]|0;
  $27 = HEAP32[$1>>2]|0;
  if ($25) {
   $28 = (($27) + 8|0);
   $29 = (($28) + 4|0);
   $30 = HEAP32[$29>>2]|0;
   _sqlite3CodeVerifyNamedSchema($26,$30);
  } else {
   HEAP32[$vararg_buffer5>>2] = $27;
   $vararg_ptr1 = (($vararg_buffer5) + 4|0);
   HEAP32[$vararg_ptr1>>2] = 0;
   _sqlite3ErrorMsg($26,23120,$vararg_buffer5);
  }
  $31 = HEAP32[$0>>2]|0;
  $32 = (($31) + 17|0);
  HEAP8[$32>>0] = 1;
  $106 = HEAP32[$db>>2]|0;
  $107 = HEAP32[$1>>2]|0;
  _sqlite3SrcListDelete($106,$107);
  STACKTOP = sp;return;
 }
 $33 = HEAP32[$pIndex>>2]|0;
 $34 = (($33) + 55|0);
 $35 = HEAP8[$34>>0]|0;
 $36 = $35 & 3;
 $37 = $36&255;
 $38 = ($37|0)!=(0);
 if ($38) {
  $39 = HEAP32[$0>>2]|0;
  HEAP32[$vararg_buffer5>>2] = 0;
  _sqlite3ErrorMsg($39,23144,$vararg_buffer5);
  $106 = HEAP32[$db>>2]|0;
  $107 = HEAP32[$1>>2]|0;
  _sqlite3SrcListDelete($106,$107);
  STACKTOP = sp;return;
 }
 $40 = HEAP32[$db>>2]|0;
 $41 = HEAP32[$pIndex>>2]|0;
 $42 = (($41) + 24|0);
 $43 = HEAP32[$42>>2]|0;
 $44 = (_sqlite3SchemaToIndex($40,$43)|0);
 HEAP32[$iDb>>2] = $44;
 HEAP32[$code>>2] = 10;
 $45 = HEAP32[$pIndex>>2]|0;
 $46 = (($45) + 12|0);
 $47 = HEAP32[$46>>2]|0;
 HEAP32[$pTab>>2] = $47;
 $48 = HEAP32[$iDb>>2]|0;
 $49 = HEAP32[$db>>2]|0;
 $50 = (($49) + 16|0);
 $51 = HEAP32[$50>>2]|0;
 $52 = (($51) + ($48<<4)|0);
 $53 = HEAP32[$52>>2]|0;
 HEAP32[$zDb>>2] = $53;
 $54 = HEAP32[$iDb>>2]|0;
 $55 = ($54|0)==(1);
 $56 = $55 ? 14904 : 14928;
 HEAP32[$zTab>>2] = $56;
 $57 = HEAP32[$0>>2]|0;
 $58 = HEAP32[$zTab>>2]|0;
 $59 = HEAP32[$zDb>>2]|0;
 $60 = (_sqlite3AuthCheck($57,9,$58,0,$59)|0);
 $61 = ($60|0)!=(0);
 if ($61) {
  $106 = HEAP32[$db>>2]|0;
  $107 = HEAP32[$1>>2]|0;
  _sqlite3SrcListDelete($106,$107);
  STACKTOP = sp;return;
 }
 $62 = HEAP32[$iDb>>2]|0;
 $63 = ($62|0)!=(0);
 if ($63) {
  HEAP32[$code>>2] = 12;
 }
 $64 = HEAP32[$0>>2]|0;
 $65 = HEAP32[$code>>2]|0;
 $66 = HEAP32[$pIndex>>2]|0;
 $67 = HEAP32[$66>>2]|0;
 $68 = HEAP32[$pTab>>2]|0;
 $69 = HEAP32[$68>>2]|0;
 $70 = HEAP32[$zDb>>2]|0;
 $71 = (_sqlite3AuthCheck($64,$65,$67,$69,$70)|0);
 $72 = ($71|0)!=(0);
 if ($72) {
  $106 = HEAP32[$db>>2]|0;
  $107 = HEAP32[$1>>2]|0;
  _sqlite3SrcListDelete($106,$107);
  STACKTOP = sp;return;
 }
 $73 = HEAP32[$0>>2]|0;
 $74 = (_sqlite3GetVdbe($73)|0);
 HEAP32[$v>>2] = $74;
 $75 = HEAP32[$v>>2]|0;
 $76 = ($75|0)!=(0|0);
 if (!($76)) {
  $106 = HEAP32[$db>>2]|0;
  $107 = HEAP32[$1>>2]|0;
  _sqlite3SrcListDelete($106,$107);
  STACKTOP = sp;return;
 }
 $77 = HEAP32[$0>>2]|0;
 $78 = HEAP32[$iDb>>2]|0;
 _sqlite3BeginWriteOperation($77,1,$78);
 $79 = HEAP32[$0>>2]|0;
 $80 = HEAP32[$iDb>>2]|0;
 $81 = HEAP32[$db>>2]|0;
 $82 = (($81) + 16|0);
 $83 = HEAP32[$82>>2]|0;
 $84 = (($83) + ($80<<4)|0);
 $85 = HEAP32[$84>>2]|0;
 $86 = HEAP32[$iDb>>2]|0;
 $87 = ($86|0)==(1);
 $88 = $87 ? 14904 : 14928;
 $89 = HEAP32[$pIndex>>2]|0;
 $90 = HEAP32[$89>>2]|0;
 HEAP32[$vararg_buffer5>>2] = $85;
 $vararg_ptr8 = (($vararg_buffer5) + 4|0);
 HEAP32[$vararg_ptr8>>2] = $88;
 $vararg_ptr9 = (($vararg_buffer5) + 8|0);
 HEAP32[$vararg_ptr9>>2] = $90;
 _sqlite3NestedParse($79,23224,$vararg_buffer5);
 $91 = HEAP32[$0>>2]|0;
 $92 = HEAP32[$iDb>>2]|0;
 $93 = HEAP32[$pIndex>>2]|0;
 $94 = HEAP32[$93>>2]|0;
 _sqlite3ClearStatTables($91,$92,18160,$94);
 $95 = HEAP32[$0>>2]|0;
 $96 = HEAP32[$iDb>>2]|0;
 _sqlite3ChangeCookie($95,$96);
 $97 = HEAP32[$0>>2]|0;
 $98 = HEAP32[$pIndex>>2]|0;
 $99 = (($98) + 44|0);
 $100 = HEAP32[$99>>2]|0;
 $101 = HEAP32[$iDb>>2]|0;
 _destroyRootPage($97,$100,$101);
 $102 = HEAP32[$v>>2]|0;
 $103 = HEAP32[$iDb>>2]|0;
 $104 = HEAP32[$pIndex>>2]|0;
 $105 = HEAP32[$104>>2]|0;
 (_sqlite3VdbeAddOp4($102,125,$103,0,0,$105,0)|0);
 $106 = HEAP32[$db>>2]|0;
 $107 = HEAP32[$1>>2]|0;
 _sqlite3SrcListDelete($106,$107);
 STACKTOP = sp;return;
}
function _sqlite3Vacuum($pParse) {
 $pParse = $pParse|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $v = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $v = sp;
 HEAP32[$0>>2] = $pParse;
 $1 = HEAP32[$0>>2]|0;
 $2 = (_sqlite3GetVdbe($1)|0);
 HEAP32[$v>>2] = $2;
 $3 = HEAP32[$v>>2]|0;
 $4 = ($3|0)!=(0|0);
 if (!($4)) {
  STACKTOP = sp;return;
 }
 $5 = HEAP32[$v>>2]|0;
 (_sqlite3VdbeAddOp2($5,13,0,0)|0);
 $6 = HEAP32[$v>>2]|0;
 _sqlite3VdbeUsesBtree($6,0);
 STACKTOP = sp;return;
}
function _sqlite3Pragma($pParse,$pId1,$pId2,$pValue,$minusFlag) {
 $pParse = $pParse|0;
 $pId1 = $pId1|0;
 $pId2 = $pId2|0;
 $pValue = $pValue|0;
 $minusFlag = $minusFlag|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $1000 = 0, $1001 = 0, $1002 = 0, $1003 = 0, $1004 = 0, $1005 = 0, $1006 = 0, $1007 = 0, $1008 = 0, $1009 = 0, $101 = 0, $1010 = 0, $1011 = 0, $1012 = 0, $1013 = 0, $1014 = 0;
 var $1015 = 0, $1016 = 0, $1017 = 0, $1018 = 0, $1019 = 0, $102 = 0, $1020 = 0, $1021 = 0, $1022 = 0, $1023 = 0, $1024 = 0, $1025 = 0, $1026 = 0, $1027 = 0, $1028 = 0, $1029 = 0, $103 = 0, $1030 = 0, $1031 = 0, $1032 = 0;
 var $1033 = 0, $1034 = 0, $1035 = 0, $1036 = 0, $1037 = 0, $1038 = 0, $1039 = 0, $104 = 0, $1040 = 0, $1041 = 0, $1042 = 0, $1043 = 0, $1044 = 0, $1045 = 0, $1046 = 0, $1047 = 0, $1048 = 0, $1049 = 0, $105 = 0, $1050 = 0;
 var $1051 = 0, $1052 = 0, $1053 = 0, $1054 = 0, $1055 = 0, $1056 = 0, $1057 = 0, $1058 = 0, $1059 = 0, $106 = 0, $1060 = 0, $1061 = 0, $1062 = 0, $1063 = 0, $1064 = 0, $1065 = 0, $1066 = 0, $1067 = 0, $1068 = 0, $1069 = 0;
 var $107 = 0, $1070 = 0, $1071 = 0, $1072 = 0, $1073 = 0, $1074 = 0, $1075 = 0, $1076 = 0, $1077 = 0, $1078 = 0, $1079 = 0, $108 = 0, $1080 = 0, $1081 = 0, $1082 = 0, $1083 = 0, $1084 = 0, $1085 = 0, $1086 = 0, $1087 = 0;
 var $1088 = 0, $1089 = 0, $109 = 0, $1090 = 0, $1091 = 0, $1092 = 0, $1093 = 0, $1094 = 0, $1095 = 0, $1096 = 0, $1097 = 0, $1098 = 0, $1099 = 0, $11 = 0, $110 = 0, $1100 = 0, $1101 = 0, $1102 = 0, $1103 = 0, $1104 = 0;
 var $1105 = 0, $1106 = 0, $1107 = 0, $1108 = 0, $1109 = 0, $111 = 0, $1110 = 0, $1111 = 0, $1112 = 0, $1113 = 0, $1114 = 0, $1115 = 0, $1116 = 0, $1117 = 0, $1118 = 0, $1119 = 0, $112 = 0, $1120 = 0, $1121 = 0, $1122 = 0;
 var $1123 = 0, $1124 = 0, $1125 = 0, $1126 = 0, $1127 = 0, $1128 = 0, $1129 = 0, $113 = 0, $1130 = 0, $1131 = 0, $1132 = 0, $1133 = 0, $1134 = 0, $1135 = 0, $1136 = 0, $1137 = 0, $1138 = 0, $1139 = 0, $114 = 0, $1140 = 0;
 var $1141 = 0, $1142 = 0, $1143 = 0, $1144 = 0, $1145 = 0, $1146 = 0, $1147 = 0, $1148 = 0, $1149 = 0, $115 = 0, $1150 = 0, $1151 = 0, $1152 = 0, $1153 = 0, $1154 = 0, $1155 = 0, $1156 = 0, $1157 = 0, $1158 = 0, $1159 = 0;
 var $116 = 0, $1160 = 0, $1161 = 0, $1162 = 0, $1163 = 0, $1164 = 0, $1165 = 0, $1166 = 0, $1167 = 0, $1168 = 0, $1169 = 0, $117 = 0, $1170 = 0, $1171 = 0, $1172 = 0, $1173 = 0, $1174 = 0, $1175 = 0, $1176 = 0, $1177 = 0;
 var $1178 = 0, $1179 = 0, $118 = 0, $1180 = 0, $1181 = 0, $1182 = 0, $1183 = 0, $1184 = 0, $1185 = 0, $1186 = 0, $1187 = 0, $1188 = 0, $1189 = 0, $119 = 0, $1190 = 0, $1191 = 0, $1192 = 0, $1193 = 0, $1194 = 0, $1195 = 0;
 var $1196 = 0, $1197 = 0, $1198 = 0, $1199 = 0, $12 = 0, $120 = 0, $1200 = 0, $1201 = 0, $1202 = 0, $1203 = 0, $1204 = 0, $1205 = 0, $1206 = 0, $1207 = 0, $1208 = 0, $1209 = 0, $121 = 0, $1210 = 0, $1211 = 0, $1212 = 0;
 var $1213 = 0, $1214 = 0, $1215 = 0, $1216 = 0, $1217 = 0, $1218 = 0, $1219 = 0, $122 = 0, $1220 = 0, $1221 = 0, $1222 = 0, $1223 = 0, $1224 = 0, $1225 = 0, $1226 = 0, $1227 = 0, $1228 = 0, $1229 = 0, $123 = 0, $1230 = 0;
 var $1231 = 0, $1232 = 0, $1233 = 0, $1234 = 0, $1235 = 0, $1236 = 0, $1237 = 0, $1238 = 0, $1239 = 0, $124 = 0, $1240 = 0, $1241 = 0, $1242 = 0, $1243 = 0, $1244 = 0, $1245 = 0, $1246 = 0, $1247 = 0, $1248 = 0, $1249 = 0;
 var $125 = 0, $1250 = 0, $1251 = 0, $1252 = 0, $1253 = 0, $1254 = 0, $1255 = 0, $1256 = 0, $1257 = 0, $1258 = 0, $1259 = 0, $126 = 0, $1260 = 0, $1261 = 0, $1262 = 0, $1263 = 0, $1264 = 0, $1265 = 0, $1266 = 0, $1267 = 0;
 var $1268 = 0, $1269 = 0, $127 = 0, $1270 = 0, $1271 = 0, $1272 = 0, $1273 = 0, $1274 = 0, $1275 = 0, $1276 = 0, $1277 = 0, $1278 = 0, $1279 = 0, $128 = 0, $1280 = 0, $1281 = 0, $1282 = 0, $1283 = 0, $1284 = 0, $1285 = 0;
 var $1286 = 0, $1287 = 0, $1288 = 0, $1289 = 0, $129 = 0, $1290 = 0, $1291 = 0, $1292 = 0, $1293 = 0, $1294 = 0, $1295 = 0, $1296 = 0, $1297 = 0, $1298 = 0, $1299 = 0, $13 = 0, $130 = 0, $1300 = 0, $1301 = 0, $1302 = 0;
 var $1303 = 0, $1304 = 0, $1305 = 0, $1306 = 0, $1307 = 0, $1308 = 0, $1309 = 0, $131 = 0, $1310 = 0, $1311 = 0, $1312 = 0, $1313 = 0, $1314 = 0, $1315 = 0, $1316 = 0, $1317 = 0, $1318 = 0, $1319 = 0, $132 = 0, $1320 = 0;
 var $1321 = 0, $1322 = 0, $1323 = 0, $1324 = 0, $1325 = 0, $1326 = 0, $1327 = 0, $1328 = 0, $1329 = 0, $133 = 0, $1330 = 0, $1331 = 0, $1332 = 0, $1333 = 0, $1334 = 0, $1335 = 0, $1336 = 0, $1337 = 0, $1338 = 0, $1339 = 0;
 var $134 = 0, $1340 = 0, $1341 = 0, $1342 = 0, $1343 = 0, $1344 = 0, $1345 = 0, $1346 = 0, $1347 = 0, $1348 = 0, $1349 = 0, $135 = 0, $1350 = 0, $1351 = 0, $1352 = 0, $1353 = 0, $1354 = 0, $1355 = 0, $1356 = 0, $1357 = 0;
 var $1358 = 0, $1359 = 0, $136 = 0, $1360 = 0, $1361 = 0, $1362 = 0, $1363 = 0, $1364 = 0, $1365 = 0, $1366 = 0, $1367 = 0, $1368 = 0, $1369 = 0, $137 = 0, $1370 = 0, $1371 = 0, $1372 = 0, $1373 = 0, $1374 = 0, $1375 = 0;
 var $1376 = 0, $1377 = 0, $1378 = 0, $1379 = 0, $138 = 0, $1380 = 0, $1381 = 0, $1382 = 0, $1383 = 0, $1384 = 0, $1385 = 0, $1386 = 0, $1387 = 0, $1388 = 0, $1389 = 0, $139 = 0, $1390 = 0, $1391 = 0, $1392 = 0, $1393 = 0;
 var $1394 = 0, $1395 = 0, $1396 = 0, $1397 = 0, $1398 = 0, $1399 = 0, $14 = 0, $140 = 0, $1400 = 0, $1401 = 0, $1402 = 0, $1403 = 0, $1404 = 0, $1405 = 0, $1406 = 0, $1407 = 0, $1408 = 0, $1409 = 0, $141 = 0, $1410 = 0;
 var $1411 = 0, $1412 = 0, $1413 = 0, $1414 = 0, $1415 = 0, $1416 = 0, $1417 = 0, $1418 = 0, $1419 = 0, $142 = 0, $1420 = 0, $1421 = 0, $1422 = 0, $1423 = 0, $1424 = 0, $1425 = 0, $1426 = 0, $1427 = 0, $1428 = 0, $1429 = 0;
 var $143 = 0, $1430 = 0, $1431 = 0, $1432 = 0, $1433 = 0, $1434 = 0, $1435 = 0, $1436 = 0, $1437 = 0, $1438 = 0, $1439 = 0, $144 = 0, $1440 = 0, $1441 = 0, $1442 = 0, $1443 = 0, $1444 = 0, $1445 = 0, $1446 = 0, $1447 = 0;
 var $1448 = 0, $1449 = 0, $145 = 0, $1450 = 0, $1451 = 0, $1452 = 0, $1453 = 0, $1454 = 0, $1455 = 0, $1456 = 0, $1457 = 0, $1458 = 0, $1459 = 0, $146 = 0, $1460 = 0, $1461 = 0, $1462 = 0, $1463 = 0, $1464 = 0, $1465 = 0;
 var $1466 = 0, $1467 = 0, $1468 = 0, $1469 = 0, $147 = 0, $1470 = 0, $1471 = 0, $1472 = 0, $1473 = 0, $1474 = 0, $1475 = 0, $1476 = 0, $1477 = 0, $1478 = 0, $1479 = 0, $148 = 0, $1480 = 0, $1481 = 0, $1482 = 0, $1483 = 0;
 var $1484 = 0, $1485 = 0, $1486 = 0, $1487 = 0, $1488 = 0, $1489 = 0, $149 = 0, $1490 = 0, $1491 = 0, $1492 = 0, $1493 = 0, $1494 = 0, $1495 = 0, $1496 = 0, $1497 = 0, $1498 = 0, $1499 = 0, $15 = 0, $150 = 0, $1500 = 0;
 var $1501 = 0, $1502 = 0, $1503 = 0, $1504 = 0, $1505 = 0, $1506 = 0, $1507 = 0, $1508 = 0, $1509 = 0, $151 = 0, $1510 = 0, $1511 = 0, $1512 = 0, $1513 = 0, $1514 = 0, $1515 = 0, $1516 = 0, $1517 = 0, $1518 = 0, $1519 = 0;
 var $152 = 0, $1520 = 0, $1521 = 0, $1522 = 0, $1523 = 0, $1524 = 0, $1525 = 0, $1526 = 0, $1527 = 0, $1528 = 0, $1529 = 0, $153 = 0, $1530 = 0, $1531 = 0, $1532 = 0, $1533 = 0, $1534 = 0, $1535 = 0, $1536 = 0, $1537 = 0;
 var $1538 = 0, $1539 = 0, $154 = 0, $1540 = 0, $1541 = 0, $1542 = 0, $1543 = 0, $1544 = 0, $1545 = 0, $1546 = 0, $1547 = 0, $1548 = 0, $1549 = 0, $155 = 0, $1550 = 0, $1551 = 0, $1552 = 0, $1553 = 0, $1554 = 0, $1555 = 0;
 var $1556 = 0, $1557 = 0, $1558 = 0, $1559 = 0, $156 = 0, $1560 = 0, $1561 = 0, $1562 = 0, $1563 = 0, $1564 = 0, $1565 = 0, $1566 = 0, $1567 = 0, $1568 = 0, $1569 = 0, $157 = 0, $1570 = 0, $1571 = 0, $1572 = 0, $1573 = 0;
 var $1574 = 0, $1575 = 0, $1576 = 0, $1577 = 0, $1578 = 0, $1579 = 0, $158 = 0, $1580 = 0, $1581 = 0, $1582 = 0, $1583 = 0, $1584 = 0, $1585 = 0, $1586 = 0, $1587 = 0, $1588 = 0, $1589 = 0, $159 = 0, $1590 = 0, $1591 = 0;
 var $1592 = 0, $1593 = 0, $1594 = 0, $1595 = 0, $1596 = 0, $1597 = 0, $1598 = 0, $1599 = 0, $16 = 0, $160 = 0, $1600 = 0, $1601 = 0, $1602 = 0, $1603 = 0, $1604 = 0, $1605 = 0, $1606 = 0, $1607 = 0, $1608 = 0, $1609 = 0;
 var $161 = 0, $1610 = 0, $1611 = 0, $1612 = 0, $1613 = 0, $1614 = 0, $1615 = 0, $1616 = 0, $1617 = 0, $1618 = 0, $1619 = 0, $162 = 0, $1620 = 0, $1621 = 0, $1622 = 0, $1623 = 0, $1624 = 0, $1625 = 0, $1626 = 0, $1627 = 0;
 var $1628 = 0, $1629 = 0, $163 = 0, $1630 = 0, $1631 = 0, $1632 = 0, $1633 = 0, $1634 = 0, $1635 = 0, $1636 = 0, $1637 = 0, $1638 = 0, $1639 = 0, $164 = 0, $1640 = 0, $1641 = 0, $1642 = 0, $1643 = 0, $1644 = 0, $1645 = 0;
 var $1646 = 0, $1647 = 0, $1648 = 0, $1649 = 0, $165 = 0, $1650 = 0, $1651 = 0, $1652 = 0, $1653 = 0, $1654 = 0, $1655 = 0, $1656 = 0, $1657 = 0, $1658 = 0, $1659 = 0, $166 = 0, $1660 = 0, $1661 = 0, $1662 = 0, $1663 = 0;
 var $1664 = 0, $1665 = 0, $1666 = 0, $1667 = 0, $1668 = 0, $1669 = 0, $167 = 0, $1670 = 0, $1671 = 0, $1672 = 0, $1673 = 0, $1674 = 0, $1675 = 0, $1676 = 0, $1677 = 0, $1678 = 0, $1679 = 0, $168 = 0, $1680 = 0, $1681 = 0;
 var $1682 = 0, $1683 = 0, $1684 = 0, $1685 = 0, $1686 = 0, $1687 = 0, $1688 = 0, $1689 = 0, $169 = 0, $1690 = 0, $1691 = 0, $1692 = 0, $1693 = 0, $1694 = 0, $1695 = 0, $1696 = 0, $1697 = 0, $1698 = 0, $1699 = 0, $17 = 0;
 var $170 = 0, $1700 = 0, $1701 = 0, $1702 = 0, $1703 = 0, $1704 = 0, $1705 = 0, $1706 = 0, $1707 = 0, $1708 = 0, $1709 = 0, $171 = 0, $1710 = 0, $1711 = 0, $1712 = 0, $1713 = 0, $1714 = 0, $1715 = 0, $1716 = 0, $1717 = 0;
 var $1718 = 0, $1719 = 0, $172 = 0, $1720 = 0, $1721 = 0, $1722 = 0, $1723 = 0, $1724 = 0, $1725 = 0, $1726 = 0, $1727 = 0, $1728 = 0, $1729 = 0, $173 = 0, $1730 = 0, $1731 = 0, $1732 = 0, $1733 = 0, $1734 = 0, $1735 = 0;
 var $1736 = 0, $1737 = 0, $1738 = 0, $1739 = 0, $174 = 0, $1740 = 0, $1741 = 0, $1742 = 0, $1743 = 0, $1744 = 0, $1745 = 0, $1746 = 0, $1747 = 0, $1748 = 0, $1749 = 0, $175 = 0, $1750 = 0, $1751 = 0, $1752 = 0, $1753 = 0;
 var $1754 = 0, $1755 = 0, $1756 = 0, $1757 = 0, $1758 = 0, $1759 = 0, $176 = 0, $1760 = 0, $1761 = 0, $1762 = 0, $1763 = 0, $1764 = 0, $1765 = 0, $1766 = 0, $1767 = 0, $1768 = 0, $1769 = 0, $177 = 0, $1770 = 0, $1771 = 0;
 var $1772 = 0, $1773 = 0, $1774 = 0, $1775 = 0, $1776 = 0, $1777 = 0, $1778 = 0, $1779 = 0, $178 = 0, $1780 = 0, $1781 = 0, $1782 = 0, $1783 = 0, $1784 = 0, $1785 = 0, $1786 = 0, $1787 = 0, $1788 = 0, $1789 = 0, $179 = 0;
 var $1790 = 0, $1791 = 0, $1792 = 0, $1793 = 0, $1794 = 0, $1795 = 0, $1796 = 0, $1797 = 0, $1798 = 0, $1799 = 0, $18 = 0, $180 = 0, $1800 = 0, $1801 = 0, $1802 = 0, $1803 = 0, $1804 = 0, $1805 = 0, $1806 = 0, $1807 = 0;
 var $1808 = 0, $1809 = 0, $181 = 0, $1810 = 0, $1811 = 0, $1812 = 0, $1813 = 0, $1814 = 0, $1815 = 0, $1816 = 0, $1817 = 0, $1818 = 0, $1819 = 0, $182 = 0, $1820 = 0, $1821 = 0, $1822 = 0, $1823 = 0, $1824 = 0, $1825 = 0;
 var $1826 = 0, $1827 = 0, $1828 = 0, $1829 = 0, $183 = 0, $1830 = 0, $1831 = 0, $1832 = 0, $1833 = 0, $1834 = 0, $1835 = 0, $1836 = 0, $1837 = 0, $1838 = 0, $1839 = 0, $184 = 0, $1840 = 0, $1841 = 0, $1842 = 0, $1843 = 0;
 var $1844 = 0, $1845 = 0, $1846 = 0, $1847 = 0, $1848 = 0, $1849 = 0, $185 = 0, $1850 = 0, $1851 = 0, $1852 = 0, $1853 = 0, $1854 = 0, $1855 = 0, $1856 = 0, $1857 = 0, $1858 = 0, $1859 = 0, $186 = 0, $1860 = 0, $1861 = 0;
 var $1862 = 0, $1863 = 0, $1864 = 0, $1865 = 0, $1866 = 0, $1867 = 0, $1868 = 0, $1869 = 0, $187 = 0, $1870 = 0, $1871 = 0, $1872 = 0, $1873 = 0, $1874 = 0, $1875 = 0, $1876 = 0, $1877 = 0, $1878 = 0, $1879 = 0, $188 = 0;
 var $1880 = 0, $1881 = 0, $1882 = 0, $1883 = 0, $1884 = 0, $1885 = 0, $1886 = 0, $1887 = 0, $1888 = 0, $1889 = 0, $189 = 0, $1890 = 0, $1891 = 0, $1892 = 0, $1893 = 0, $1894 = 0, $1895 = 0, $1896 = 0, $1897 = 0, $1898 = 0;
 var $1899 = 0, $19 = 0, $190 = 0, $1900 = 0, $1901 = 0, $1902 = 0, $1903 = 0, $1904 = 0, $1905 = 0, $1906 = 0, $1907 = 0, $1908 = 0, $1909 = 0, $191 = 0, $1910 = 0, $1911 = 0, $1912 = 0, $1913 = 0, $1914 = 0, $1915 = 0;
 var $1916 = 0, $1917 = 0, $1918 = 0, $1919 = 0, $192 = 0, $1920 = 0, $1921 = 0, $1922 = 0, $1923 = 0, $1924 = 0, $1925 = 0, $1926 = 0, $1927 = 0, $1928 = 0, $1929 = 0, $193 = 0, $1930 = 0, $1931 = 0, $1932 = 0, $1933 = 0;
 var $1934 = 0, $1935 = 0, $1936 = 0, $1937 = 0, $1938 = 0, $1939 = 0, $194 = 0, $1940 = 0, $1941 = 0, $1942 = 0, $1943 = 0, $1944 = 0, $1945 = 0, $1946 = 0, $1947 = 0, $1948 = 0, $1949 = 0, $195 = 0, $1950 = 0, $1951 = 0;
 var $1952 = 0, $1953 = 0, $1954 = 0, $1955 = 0, $1956 = 0, $1957 = 0, $1958 = 0, $1959 = 0, $196 = 0, $1960 = 0, $1961 = 0, $1962 = 0, $1963 = 0, $1964 = 0, $1965 = 0, $1966 = 0, $1967 = 0, $1968 = 0, $1969 = 0, $197 = 0;
 var $1970 = 0, $1971 = 0, $1972 = 0, $1973 = 0, $1974 = 0, $198 = 0, $199 = 0, $2 = 0, $20 = 0, $200 = 0, $201 = 0, $202 = 0, $203 = 0, $204 = 0, $205 = 0, $206 = 0, $207 = 0, $208 = 0, $209 = 0, $21 = 0;
 var $210 = 0, $211 = 0, $212 = 0, $213 = 0, $214 = 0, $215 = 0, $216 = 0, $217 = 0, $218 = 0, $219 = 0, $22 = 0, $220 = 0, $221 = 0, $222 = 0, $223 = 0, $224 = 0, $225 = 0, $226 = 0, $227 = 0, $228 = 0;
 var $229 = 0, $23 = 0, $230 = 0, $231 = 0, $232 = 0, $233 = 0, $234 = 0, $235 = 0, $236 = 0, $237 = 0, $238 = 0, $239 = 0, $24 = 0, $240 = 0, $241 = 0, $242 = 0, $243 = 0, $244 = 0, $245 = 0, $246 = 0;
 var $247 = 0, $248 = 0, $249 = 0, $25 = 0, $250 = 0, $251 = 0, $252 = 0, $253 = 0, $254 = 0, $255 = 0, $256 = 0, $257 = 0, $258 = 0, $259 = 0, $26 = 0, $260 = 0, $261 = 0, $262 = 0, $263 = 0, $264 = 0;
 var $265 = 0, $266 = 0, $267 = 0, $268 = 0, $269 = 0, $27 = 0, $270 = 0, $271 = 0, $272 = 0, $273 = 0, $274 = 0, $275 = 0, $276 = 0, $277 = 0, $278 = 0, $279 = 0, $28 = 0, $280 = 0, $281 = 0, $282 = 0;
 var $283 = 0, $284 = 0, $285 = 0, $286 = 0, $287 = 0, $288 = 0, $289 = 0, $29 = 0, $290 = 0, $291 = 0, $292 = 0, $293 = 0, $294 = 0, $295 = 0, $296 = 0, $297 = 0, $298 = 0, $299 = 0, $3 = 0, $30 = 0;
 var $300 = 0, $301 = 0, $302 = 0, $303 = 0, $304 = 0, $305 = 0, $306 = 0, $307 = 0, $308 = 0, $309 = 0, $31 = 0, $310 = 0, $311 = 0, $312 = 0, $313 = 0, $314 = 0, $315 = 0, $316 = 0, $317 = 0, $318 = 0;
 var $319 = 0, $32 = 0, $320 = 0, $321 = 0, $322 = 0, $323 = 0, $324 = 0, $325 = 0, $326 = 0, $327 = 0, $328 = 0, $329 = 0, $33 = 0, $330 = 0, $331 = 0, $332 = 0, $333 = 0, $334 = 0, $335 = 0, $336 = 0;
 var $337 = 0, $338 = 0, $339 = 0, $34 = 0, $340 = 0, $341 = 0, $342 = 0, $343 = 0, $344 = 0, $345 = 0, $346 = 0, $347 = 0, $348 = 0, $349 = 0, $35 = 0, $350 = 0, $351 = 0, $352 = 0, $353 = 0, $354 = 0;
 var $355 = 0, $356 = 0, $357 = 0, $358 = 0, $359 = 0, $36 = 0, $360 = 0, $361 = 0, $362 = 0, $363 = 0, $364 = 0, $365 = 0, $366 = 0, $367 = 0, $368 = 0, $369 = 0, $37 = 0, $370 = 0, $371 = 0, $372 = 0;
 var $373 = 0, $374 = 0, $375 = 0, $376 = 0, $377 = 0, $378 = 0, $379 = 0, $38 = 0, $380 = 0, $381 = 0, $382 = 0, $383 = 0, $384 = 0, $385 = 0, $386 = 0, $387 = 0, $388 = 0, $389 = 0, $39 = 0, $390 = 0;
 var $391 = 0, $392 = 0, $393 = 0, $394 = 0, $395 = 0, $396 = 0, $397 = 0, $398 = 0, $399 = 0, $4 = 0, $40 = 0, $400 = 0, $401 = 0, $402 = 0, $403 = 0, $404 = 0, $405 = 0, $406 = 0, $407 = 0, $408 = 0;
 var $409 = 0, $41 = 0, $410 = 0, $411 = 0, $412 = 0, $413 = 0, $414 = 0, $415 = 0, $416 = 0, $417 = 0, $418 = 0, $419 = 0, $42 = 0, $420 = 0, $421 = 0, $422 = 0, $423 = 0, $424 = 0, $425 = 0, $426 = 0;
 var $427 = 0, $428 = 0, $429 = 0, $43 = 0, $430 = 0, $431 = 0, $432 = 0, $433 = 0, $434 = 0, $435 = 0, $436 = 0, $437 = 0, $438 = 0, $439 = 0, $44 = 0, $440 = 0, $441 = 0, $442 = 0, $443 = 0, $444 = 0;
 var $445 = 0, $446 = 0, $447 = 0, $448 = 0, $449 = 0, $45 = 0, $450 = 0, $451 = 0, $452 = 0, $453 = 0, $454 = 0, $455 = 0, $456 = 0, $457 = 0, $458 = 0, $459 = 0, $46 = 0, $460 = 0, $461 = 0, $462 = 0;
 var $463 = 0, $464 = 0, $465 = 0, $466 = 0, $467 = 0, $468 = 0, $469 = 0, $47 = 0, $470 = 0, $471 = 0, $472 = 0, $473 = 0, $474 = 0, $475 = 0, $476 = 0, $477 = 0, $478 = 0, $479 = 0, $48 = 0, $480 = 0;
 var $481 = 0, $482 = 0, $483 = 0, $484 = 0, $485 = 0, $486 = 0, $487 = 0, $488 = 0, $489 = 0, $49 = 0, $490 = 0, $491 = 0, $492 = 0, $493 = 0, $494 = 0, $495 = 0, $496 = 0, $497 = 0, $498 = 0, $499 = 0;
 var $5 = 0, $50 = 0, $500 = 0, $501 = 0, $502 = 0, $503 = 0, $504 = 0, $505 = 0, $506 = 0, $507 = 0, $508 = 0, $509 = 0, $51 = 0, $510 = 0, $511 = 0, $512 = 0, $513 = 0, $514 = 0, $515 = 0, $516 = 0;
 var $517 = 0, $518 = 0, $519 = 0, $52 = 0, $520 = 0, $521 = 0, $522 = 0, $523 = 0, $524 = 0, $525 = 0, $526 = 0, $527 = 0, $528 = 0, $529 = 0, $53 = 0, $530 = 0, $531 = 0, $532 = 0, $533 = 0, $534 = 0;
 var $535 = 0, $536 = 0, $537 = 0, $538 = 0, $539 = 0, $54 = 0, $540 = 0, $541 = 0, $542 = 0, $543 = 0, $544 = 0, $545 = 0, $546 = 0, $547 = 0, $548 = 0, $549 = 0, $55 = 0, $550 = 0, $551 = 0, $552 = 0;
 var $553 = 0, $554 = 0, $555 = 0, $556 = 0, $557 = 0, $558 = 0, $559 = 0, $56 = 0, $560 = 0, $561 = 0, $562 = 0, $563 = 0, $564 = 0, $565 = 0, $566 = 0, $567 = 0, $568 = 0, $569 = 0, $57 = 0, $570 = 0;
 var $571 = 0, $572 = 0, $573 = 0, $574 = 0, $575 = 0, $576 = 0, $577 = 0, $578 = 0, $579 = 0, $58 = 0, $580 = 0, $581 = 0, $582 = 0, $583 = 0, $584 = 0, $585 = 0, $586 = 0, $587 = 0, $588 = 0, $589 = 0;
 var $59 = 0, $590 = 0, $591 = 0, $592 = 0, $593 = 0, $594 = 0, $595 = 0, $596 = 0, $597 = 0, $598 = 0, $599 = 0, $6 = 0, $60 = 0, $600 = 0, $601 = 0, $602 = 0, $603 = 0, $604 = 0, $605 = 0, $606 = 0;
 var $607 = 0, $608 = 0, $609 = 0, $61 = 0, $610 = 0, $611 = 0, $612 = 0, $613 = 0, $614 = 0, $615 = 0, $616 = 0, $617 = 0, $618 = 0, $619 = 0, $62 = 0, $620 = 0, $621 = 0, $622 = 0, $623 = 0, $624 = 0;
 var $625 = 0, $626 = 0, $627 = 0, $628 = 0, $629 = 0, $63 = 0, $630 = 0, $631 = 0, $632 = 0, $633 = 0, $634 = 0, $635 = 0, $636 = 0, $637 = 0, $638 = 0, $639 = 0, $64 = 0, $640 = 0, $641 = 0, $642 = 0;
 var $643 = 0, $644 = 0, $645 = 0, $646 = 0, $647 = 0, $648 = 0, $649 = 0, $65 = 0, $650 = 0, $651 = 0, $652 = 0, $653 = 0, $654 = 0, $655 = 0, $656 = 0, $657 = 0, $658 = 0, $659 = 0, $66 = 0, $660 = 0;
 var $661 = 0, $662 = 0, $663 = 0, $664 = 0, $665 = 0, $666 = 0, $667 = 0, $668 = 0, $669 = 0, $67 = 0, $670 = 0, $671 = 0, $672 = 0, $673 = 0, $674 = 0, $675 = 0, $676 = 0, $677 = 0, $678 = 0, $679 = 0;
 var $68 = 0, $680 = 0, $681 = 0, $682 = 0, $683 = 0, $684 = 0, $685 = 0, $686 = 0, $687 = 0, $688 = 0, $689 = 0, $69 = 0, $690 = 0, $691 = 0, $692 = 0, $693 = 0, $694 = 0, $695 = 0, $696 = 0, $697 = 0;
 var $698 = 0, $699 = 0, $7 = 0, $70 = 0, $700 = 0, $701 = 0, $702 = 0, $703 = 0, $704 = 0, $705 = 0, $706 = 0, $707 = 0, $708 = 0, $709 = 0, $71 = 0, $710 = 0, $711 = 0, $712 = 0, $713 = 0, $714 = 0;
 var $715 = 0, $716 = 0, $717 = 0, $718 = 0, $719 = 0, $72 = 0, $720 = 0, $721 = 0, $722 = 0, $723 = 0, $724 = 0, $725 = 0, $726 = 0, $727 = 0, $728 = 0, $729 = 0, $73 = 0, $730 = 0, $731 = 0, $732 = 0;
 var $733 = 0, $734 = 0, $735 = 0, $736 = 0, $737 = 0, $738 = 0, $739 = 0, $74 = 0, $740 = 0, $741 = 0, $742 = 0, $743 = 0, $744 = 0, $745 = 0, $746 = 0, $747 = 0, $748 = 0, $749 = 0, $75 = 0, $750 = 0;
 var $751 = 0, $752 = 0, $753 = 0, $754 = 0, $755 = 0, $756 = 0, $757 = 0, $758 = 0, $759 = 0, $76 = 0, $760 = 0, $761 = 0, $762 = 0, $763 = 0, $764 = 0, $765 = 0, $766 = 0, $767 = 0, $768 = 0, $769 = 0;
 var $77 = 0, $770 = 0, $771 = 0, $772 = 0, $773 = 0, $774 = 0, $775 = 0, $776 = 0, $777 = 0, $778 = 0, $779 = 0, $78 = 0, $780 = 0, $781 = 0, $782 = 0, $783 = 0, $784 = 0, $785 = 0, $786 = 0, $787 = 0;
 var $788 = 0, $789 = 0, $79 = 0, $790 = 0, $791 = 0, $792 = 0, $793 = 0, $794 = 0, $795 = 0, $796 = 0, $797 = 0, $798 = 0, $799 = 0, $8 = 0, $80 = 0, $800 = 0, $801 = 0, $802 = 0, $803 = 0, $804 = 0;
 var $805 = 0, $806 = 0, $807 = 0, $808 = 0, $809 = 0, $81 = 0, $810 = 0, $811 = 0, $812 = 0, $813 = 0, $814 = 0, $815 = 0, $816 = 0, $817 = 0, $818 = 0, $819 = 0, $82 = 0, $820 = 0, $821 = 0, $822 = 0;
 var $823 = 0, $824 = 0, $825 = 0, $826 = 0, $827 = 0, $828 = 0, $829 = 0, $83 = 0, $830 = 0, $831 = 0, $832 = 0, $833 = 0, $834 = 0, $835 = 0, $836 = 0, $837 = 0, $838 = 0, $839 = 0, $84 = 0, $840 = 0;
 var $841 = 0, $842 = 0, $843 = 0, $844 = 0, $845 = 0, $846 = 0, $847 = 0, $848 = 0, $849 = 0, $85 = 0, $850 = 0, $851 = 0, $852 = 0, $853 = 0, $854 = 0, $855 = 0, $856 = 0, $857 = 0, $858 = 0, $859 = 0;
 var $86 = 0, $860 = 0, $861 = 0, $862 = 0, $863 = 0, $864 = 0, $865 = 0, $866 = 0, $867 = 0, $868 = 0, $869 = 0, $87 = 0, $870 = 0, $871 = 0, $872 = 0, $873 = 0, $874 = 0, $875 = 0, $876 = 0, $877 = 0;
 var $878 = 0, $879 = 0, $88 = 0, $880 = 0, $881 = 0, $882 = 0, $883 = 0, $884 = 0, $885 = 0, $886 = 0, $887 = 0, $888 = 0, $889 = 0, $89 = 0, $890 = 0, $891 = 0, $892 = 0, $893 = 0, $894 = 0, $895 = 0;
 var $896 = 0, $897 = 0, $898 = 0, $899 = 0, $9 = 0, $90 = 0, $900 = 0, $901 = 0, $902 = 0, $903 = 0, $904 = 0, $905 = 0, $906 = 0, $907 = 0, $908 = 0, $909 = 0, $91 = 0, $910 = 0, $911 = 0, $912 = 0;
 var $913 = 0, $914 = 0, $915 = 0, $916 = 0, $917 = 0, $918 = 0, $919 = 0, $92 = 0, $920 = 0, $921 = 0, $922 = 0, $923 = 0, $924 = 0, $925 = 0, $926 = 0, $927 = 0, $928 = 0, $929 = 0, $93 = 0, $930 = 0;
 var $931 = 0, $932 = 0, $933 = 0, $934 = 0, $935 = 0, $936 = 0, $937 = 0, $938 = 0, $939 = 0, $94 = 0, $940 = 0, $941 = 0, $942 = 0, $943 = 0, $944 = 0, $945 = 0, $946 = 0, $947 = 0, $948 = 0, $949 = 0;
 var $95 = 0, $950 = 0, $951 = 0, $952 = 0, $953 = 0, $954 = 0, $955 = 0, $956 = 0, $957 = 0, $958 = 0, $959 = 0, $96 = 0, $960 = 0, $961 = 0, $962 = 0, $963 = 0, $964 = 0, $965 = 0, $966 = 0, $967 = 0;
 var $968 = 0, $969 = 0, $97 = 0, $970 = 0, $971 = 0, $972 = 0, $973 = 0, $974 = 0, $975 = 0, $976 = 0, $977 = 0, $978 = 0, $979 = 0, $98 = 0, $980 = 0, $981 = 0, $982 = 0, $983 = 0, $984 = 0, $985 = 0;
 var $986 = 0, $987 = 0, $988 = 0, $989 = 0, $99 = 0, $990 = 0, $991 = 0, $992 = 0, $993 = 0, $994 = 0, $995 = 0, $996 = 0, $997 = 0, $998 = 0, $999 = 0, $N = 0, $N44 = 0, $aFcntl = 0, $addr = 0, $addr31 = 0;
 var $addr40 = 0, $addr41 = 0, $addr9 = 0, $addrOk = 0, $addrTop = 0, $aiCols = 0, $b = 0, $ckUniq = 0, $cnt = 0, $cnum = 0, $db = 0, $eAuto = 0, $eMode = 0, $eMode4 = 0, $eMode43 = 0, $i = 0, $i11 = 0, $i15 = 0, $i18 = 0, $i19 = 0;
 var $i20 = 0, $i22 = 0, $i26 = 0, $i29 = 0, $i42 = 0, $iAddr = 0, $iBt = 0, $iCol = 0, $iCookie = 0, $iDataCur = 0, $iDb = 0, $iIdxCur = 0, $iKey = 0, $iLimit = 0, $iLimit8 = 0, $iReg = 0, $ii = 0, $ii3 = 0, $ii5 = 0, $isQuick = 0;
 var $j = 0, $j27 = 0, $j30 = 0, $jmp2 = 0, $jmp238 = 0, $jmp3 = 0, $jmp339 = 0, $jmp4 = 0, $jmp5 = 0, $jmp6 = 0, $k = 0, $k28 = 0, $kk = 0, $loopTop = 0, $lwr = 0, $mask = 0, $mem = 0, $mid = 0, $mxErr = 0, $n = 0;
 var $nHidden = 0, $or$cond = 0, $or$cond11 = 0, $or$cond13 = 0, $or$cond15 = 0, $or$cond17 = 0, $or$cond3 = 0, $or$cond5 = 0, $or$cond7 = 0, $or$cond9 = 0, $p = 0, $pBt = 0, $pBt2 = 0, $pBt7 = 0, $pCol = 0, $pColl = 0, $pDb = 0, $pEnc = 0, $pFK = 0, $pFK23 = 0;
 var $pId = 0, $pIdx = 0, $pIdx13 = 0, $pIdx16 = 0, $pIdx25 = 0, $pIdx34 = 0, $pIdx36 = 0, $pPager = 0, $pPager6 = 0, $pParent = 0, $pPk = 0, $pPk37 = 0, $pPrior = 0, $pTab = 0, $pTab12 = 0, $pTab14 = 0, $pTab17 = 0, $pTab21 = 0, $pTab24 = 0, $pTab33 = 0;
 var $pTab35 = 0, $pTbls = 0, $r1 = 0, $rc = 0, $regKey = 0, $regResult = 0, $regRow = 0, $res = 0, $size = 0, $size1 = 0, $size10 = 0, $sz = 0, $uniqOk = 0, $upr = 0, $v = 0, $vararg_buffer35 = 0, $vararg_ptr34 = 0, $x = 0, $x32 = 0, $zCol = 0;
 var $zDb = 0, $zErr = 0, $zLeft = 0, $zMode = 0, $zOnDelete = 0, $zOnUpdate = 0, $zOpt = 0, $zRet = 0, $zRight = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 560|0;
 $vararg_buffer35 = sp + 32|0;
 $0 = sp + 544|0;
 $1 = sp + 540|0;
 $2 = sp + 536|0;
 $3 = sp + 532|0;
 $4 = sp + 528|0;
 $zLeft = sp + 524|0;
 $zRight = sp + 520|0;
 $zDb = sp + 516|0;
 $pId = sp + 512|0;
 $aFcntl = sp + 496|0;
 $iDb = sp + 488|0;
 $lwr = sp + 484|0;
 $upr = sp + 480|0;
 $mid = sp + 476|0;
 $rc = sp + 472|0;
 $db = sp + 468|0;
 $pDb = sp + 464|0;
 $v = sp + 460|0;
 $mem = sp + 456|0;
 $addr = sp + 452|0;
 $size = sp + 448|0;
 $pBt = sp + 444|0;
 $size1 = sp + 440|0;
 $pBt2 = sp + 436|0;
 $b = sp + 432|0;
 $ii = sp + 428|0;
 $iReg = sp + 424|0;
 $zRet = sp + 420|0;
 $eMode = sp + 416|0;
 $pPager = sp + 412|0;
 $ii3 = sp + 408|0;
 $eMode4 = sp + 404|0;
 $ii5 = sp + 400|0;
 $zMode = sp + 396|0;
 $n = sp + 392|0;
 $pPager6 = sp + 388|0;
 $iLimit = sp + 24|0;
 $pBt7 = sp + 384|0;
 $eAuto = sp + 380|0;
 $iAddr = sp + 376|0;
 $iLimit8 = sp + 372|0;
 $addr9 = sp + 368|0;
 $size10 = sp + 364|0;
 $sz = sp + 16|0;
 $res = sp + 360|0;
 $mask = sp + 356|0;
 $pTab = sp + 352|0;
 $i = sp + 348|0;
 $k = sp + 344|0;
 $nHidden = sp + 340|0;
 $pCol = sp + 336|0;
 $pPk = sp + 332|0;
 $pIdx = sp + 328|0;
 $i11 = sp + 324|0;
 $pTab12 = sp + 320|0;
 $pIdx13 = sp + 316|0;
 $pTab14 = sp + 312|0;
 $i15 = sp + 308|0;
 $cnum = sp + 548|0;
 $pIdx16 = sp + 304|0;
 $pTab17 = sp + 300|0;
 $i18 = sp + 296|0;
 $i19 = sp + 292|0;
 $i20 = sp + 288|0;
 $p = sp + 284|0;
 $pColl = sp + 280|0;
 $pFK = sp + 276|0;
 $pTab21 = sp + 272|0;
 $i22 = sp + 268|0;
 $j = sp + 264|0;
 $zCol = sp + 260|0;
 $zOnDelete = sp + 256|0;
 $zOnUpdate = sp + 252|0;
 $pFK23 = sp + 248|0;
 $pTab24 = sp + 244|0;
 $pParent = sp + 240|0;
 $pIdx25 = sp + 236|0;
 $i26 = sp + 232|0;
 $j27 = sp + 228|0;
 $k28 = sp + 224|0;
 $x = sp + 220|0;
 $regResult = sp + 216|0;
 $regKey = sp + 212|0;
 $regRow = sp + 208|0;
 $addrTop = sp + 204|0;
 $addrOk = sp + 200|0;
 $aiCols = sp + 196|0;
 $iKey = sp + 192|0;
 $i29 = sp + 188|0;
 $j30 = sp + 184|0;
 $addr31 = sp + 180|0;
 $mxErr = sp + 176|0;
 $isQuick = sp + 172|0;
 $x32 = sp + 168|0;
 $pTbls = sp + 164|0;
 $cnt = sp + 160|0;
 $pTab33 = sp + 156|0;
 $pIdx34 = sp + 152|0;
 $pTab35 = sp + 148|0;
 $pIdx36 = sp + 144|0;
 $pPk37 = sp + 140|0;
 $pPrior = sp + 136|0;
 $loopTop = sp + 132|0;
 $iDataCur = sp + 128|0;
 $iIdxCur = sp + 124|0;
 $r1 = sp + 120|0;
 $zErr = sp + 116|0;
 $jmp2 = sp + 112|0;
 $jmp3 = sp + 108|0;
 $jmp238 = sp + 104|0;
 $jmp339 = sp + 100|0;
 $jmp4 = sp + 96|0;
 $jmp5 = sp + 92|0;
 $ckUniq = sp + 88|0;
 $uniqOk = sp + 84|0;
 $jmp6 = sp + 80|0;
 $kk = sp + 76|0;
 $iCol = sp + 72|0;
 $pEnc = sp + 68|0;
 $iCookie = sp + 64|0;
 $addr40 = sp + 60|0;
 $addr41 = sp + 56|0;
 $i42 = sp + 52|0;
 $zOpt = sp + 48|0;
 $iBt = sp + 44|0;
 $eMode43 = sp + 40|0;
 $N = sp + 8|0;
 $N44 = sp;
 HEAP32[$0>>2] = $pParse;
 HEAP32[$1>>2] = $pId1;
 HEAP32[$2>>2] = $pId2;
 HEAP32[$3>>2] = $pValue;
 HEAP32[$4>>2] = $minusFlag;
 HEAP32[$zLeft>>2] = 0;
 HEAP32[$zRight>>2] = 0;
 HEAP32[$zDb>>2] = 0;
 $5 = HEAP32[$0>>2]|0;
 $6 = HEAP32[$5>>2]|0;
 HEAP32[$db>>2] = $6;
 $7 = HEAP32[$0>>2]|0;
 $8 = (_sqlite3GetVdbe($7)|0);
 HEAP32[$v>>2] = $8;
 $9 = HEAP32[$v>>2]|0;
 $10 = ($9|0)==(0|0);
 if ($10) {
  STACKTOP = sp;return;
 }
 $11 = HEAP32[$v>>2]|0;
 _sqlite3VdbeRunOnlyOnce($11);
 $12 = HEAP32[$0>>2]|0;
 $13 = (($12) + 72|0);
 HEAP32[$13>>2] = 2;
 $14 = HEAP32[$0>>2]|0;
 $15 = HEAP32[$1>>2]|0;
 $16 = HEAP32[$2>>2]|0;
 $17 = (_sqlite3TwoPartName($14,$15,$16,$pId)|0);
 HEAP32[$iDb>>2] = $17;
 $18 = HEAP32[$iDb>>2]|0;
 $19 = ($18|0)<(0);
 if ($19) {
  STACKTOP = sp;return;
 }
 $20 = HEAP32[$iDb>>2]|0;
 $21 = HEAP32[$db>>2]|0;
 $22 = (($21) + 16|0);
 $23 = HEAP32[$22>>2]|0;
 $24 = (($23) + ($20<<4)|0);
 HEAP32[$pDb>>2] = $24;
 $25 = HEAP32[$iDb>>2]|0;
 $26 = ($25|0)==(1);
 if ($26) {
  $27 = HEAP32[$0>>2]|0;
  $28 = (_sqlite3OpenTempDatabase($27)|0);
  $29 = ($28|0)!=(0);
  if ($29) {
   STACKTOP = sp;return;
  }
 }
 $30 = HEAP32[$db>>2]|0;
 $31 = HEAP32[$pId>>2]|0;
 $32 = (_sqlite3NameFromToken($30,$31)|0);
 HEAP32[$zLeft>>2] = $32;
 $33 = HEAP32[$zLeft>>2]|0;
 $34 = ($33|0)!=(0|0);
 if (!($34)) {
  STACKTOP = sp;return;
 }
 $35 = HEAP32[$4>>2]|0;
 $36 = ($35|0)!=(0);
 $37 = HEAP32[$db>>2]|0;
 $38 = HEAP32[$3>>2]|0;
 if ($36) {
  HEAP32[$vararg_buffer35>>2] = $38;
  $39 = (_sqlite3MPrintf($37,20296,$vararg_buffer35)|0);
  HEAP32[$zRight>>2] = $39;
 } else {
  $40 = (_sqlite3NameFromToken($37,$38)|0);
  HEAP32[$zRight>>2] = $40;
 }
 $41 = HEAP32[$2>>2]|0;
 $42 = (($41) + 4|0);
 $43 = HEAP32[$42>>2]|0;
 $44 = ($43>>>0)>(0);
 if ($44) {
  $45 = HEAP32[$pDb>>2]|0;
  $46 = HEAP32[$45>>2]|0;
  $47 = $46;
 } else {
  $47 = 0;
 }
 HEAP32[$zDb>>2] = $47;
 $48 = HEAP32[$0>>2]|0;
 $49 = HEAP32[$zLeft>>2]|0;
 $50 = HEAP32[$zRight>>2]|0;
 $51 = HEAP32[$zDb>>2]|0;
 $52 = (_sqlite3AuthCheck($48,19,$49,$50,$51)|0);
 $53 = ($52|0)!=(0);
 L21: do {
  if (!($53)) {
   HEAP32[$aFcntl>>2] = 0;
   $54 = HEAP32[$zLeft>>2]|0;
   $55 = (($aFcntl) + 4|0);
   HEAP32[$55>>2] = $54;
   $56 = HEAP32[$zRight>>2]|0;
   $57 = (($aFcntl) + 8|0);
   HEAP32[$57>>2] = $56;
   $58 = (($aFcntl) + 12|0);
   HEAP32[$58>>2] = 0;
   $59 = HEAP32[$db>>2]|0;
   $60 = (($59) + 444|0);
   $61 = (($60) + 8|0);
   HEAP32[$61>>2] = 0;
   $62 = HEAP32[$db>>2]|0;
   $63 = HEAP32[$zDb>>2]|0;
   $64 = (_sqlite3_file_control($62,$63,14,$aFcntl)|0);
   HEAP32[$rc>>2] = $64;
   $65 = HEAP32[$rc>>2]|0;
   $66 = ($65|0)==(0);
   if ($66) {
    $67 = HEAP32[$aFcntl>>2]|0;
    $68 = ($67|0)!=(0|0);
    if (!($68)) {
     break;
    }
    $69 = HEAP32[$0>>2]|0;
    $70 = (($69) + 72|0);
    $71 = HEAP32[$70>>2]|0;
    $72 = (($71) + 1)|0;
    HEAP32[$70>>2] = $72;
    HEAP32[$mem>>2] = $72;
    $73 = HEAP32[$v>>2]|0;
    $74 = HEAP32[$mem>>2]|0;
    $75 = HEAP32[$aFcntl>>2]|0;
    (_sqlite3VdbeAddOp4($73,97,0,$74,0,$75,0)|0);
    $76 = HEAP32[$v>>2]|0;
    _sqlite3VdbeSetNumCols($76,1);
    $77 = HEAP32[$v>>2]|0;
    (_sqlite3VdbeSetColName($77,0,0,20304,0)|0);
    $78 = HEAP32[$v>>2]|0;
    $79 = HEAP32[$mem>>2]|0;
    (_sqlite3VdbeAddOp2($78,35,$79,1)|0);
    $80 = HEAP32[$aFcntl>>2]|0;
    _sqlite3_free($80);
    break;
   }
   $81 = HEAP32[$rc>>2]|0;
   $82 = ($81|0)!=(12);
   if ($82) {
    $83 = HEAP32[$aFcntl>>2]|0;
    $84 = ($83|0)!=(0|0);
    if ($84) {
     $85 = HEAP32[$0>>2]|0;
     $86 = HEAP32[$aFcntl>>2]|0;
     HEAP32[$vararg_buffer35>>2] = $86;
     _sqlite3ErrorMsg($85,1200,$vararg_buffer35);
     $87 = HEAP32[$aFcntl>>2]|0;
     _sqlite3_free($87);
    }
    $88 = HEAP32[$0>>2]|0;
    $89 = (($88) + 64|0);
    $90 = HEAP32[$89>>2]|0;
    $91 = (($90) + 1)|0;
    HEAP32[$89>>2] = $91;
    $92 = HEAP32[$rc>>2]|0;
    $93 = HEAP32[$0>>2]|0;
    $94 = (($93) + 12|0);
    HEAP32[$94>>2] = $92;
    break;
   }
   HEAP32[$lwr>>2] = 0;
   HEAP32[$upr>>2] = 54;
   while(1) {
    $95 = HEAP32[$lwr>>2]|0;
    $96 = HEAP32[$upr>>2]|0;
    $97 = ($95|0)<=($96|0);
    if (!($97)) {
     break;
    }
    $98 = HEAP32[$lwr>>2]|0;
    $99 = HEAP32[$upr>>2]|0;
    $100 = (($98) + ($99))|0;
    $101 = (($100|0) / 2)&-1;
    HEAP32[$mid>>2] = $101;
    $102 = HEAP32[$zLeft>>2]|0;
    $103 = HEAP32[$mid>>2]|0;
    $104 = (20312 + (($103*12)|0)|0);
    $105 = HEAP32[$104>>2]|0;
    $106 = (_sqlite3_stricmp($102,$105)|0);
    HEAP32[$rc>>2] = $106;
    $107 = HEAP32[$rc>>2]|0;
    $108 = ($107|0)==(0);
    if ($108) {
     break;
    }
    $109 = HEAP32[$rc>>2]|0;
    $110 = ($109|0)<(0);
    $111 = HEAP32[$mid>>2]|0;
    if ($110) {
     $112 = (($111) - 1)|0;
     HEAP32[$upr>>2] = $112;
     continue;
    } else {
     $113 = (($111) + 1)|0;
     HEAP32[$lwr>>2] = $113;
     continue;
    }
   }
   $114 = HEAP32[$lwr>>2]|0;
   $115 = HEAP32[$upr>>2]|0;
   $116 = ($114|0)>($115|0);
   if (!($116)) {
    $117 = HEAP32[$mid>>2]|0;
    $118 = (20312 + (($117*12)|0)|0);
    $119 = (($118) + 5|0);
    $120 = HEAP8[$119>>0]|0;
    $121 = $120&255;
    $122 = $121 & 1;
    $123 = ($122|0)!=(0);
    if ($123) {
     $124 = HEAP32[$0>>2]|0;
     $125 = (_sqlite3ReadSchema($124)|0);
     $126 = ($125|0)!=(0);
     if ($126) {
      break;
     }
    }
    $127 = HEAP32[$mid>>2]|0;
    $128 = (20312 + (($127*12)|0)|0);
    $129 = (($128) + 4|0);
    $130 = HEAP8[$129>>0]|0;
    $131 = $130&255;
    do {
     switch ($131|0) {
     case 1:  {
      $414 = HEAP32[$pDb>>2]|0;
      $415 = (($414) + 4|0);
      $416 = HEAP32[$415>>2]|0;
      HEAP32[$pBt7>>2] = $416;
      $417 = HEAP32[$zRight>>2]|0;
      $418 = ($417|0)!=(0|0);
      if (!($418)) {
       $419 = HEAP32[$0>>2]|0;
       $420 = HEAP32[$pBt7>>2]|0;
       $421 = (_sqlite3BtreeGetAutoVacuum($420)|0);
       $422 = ($421|0)<(0);
       $423 = $422 << 31 >> 31;
       _returnSingleInt($419,21144,$421,$423);
       break L21;
      }
      $424 = HEAP32[$zRight>>2]|0;
      $425 = (_getAutoVacuum($424)|0);
      HEAP32[$eAuto>>2] = $425;
      $426 = HEAP32[$eAuto>>2]|0;
      $427 = $426&255;
      $428 = HEAP32[$db>>2]|0;
      $429 = (($428) + 66|0);
      HEAP8[$429>>0] = $427;
      $430 = HEAP32[$pBt7>>2]|0;
      $431 = HEAP32[$eAuto>>2]|0;
      $432 = (_sqlite3BtreeSetAutoVacuum($430,$431)|0);
      HEAP32[$rc>>2] = $432;
      $433 = HEAP32[$rc>>2]|0;
      $434 = ($433|0)==(0);
      if (!($434)) {
       break L21;
      }
      $435 = HEAP32[$eAuto>>2]|0;
      $436 = ($435|0)==(1);
      $437 = HEAP32[$eAuto>>2]|0;
      $438 = ($437|0)==(2);
      $or$cond5 = $436 | $438;
      if (!($or$cond5)) {
       break L21;
      }
      $439 = HEAP32[$v>>2]|0;
      $440 = (_sqlite3VdbeAddOpList($439,6,21160,0)|0);
      HEAP32[$iAddr>>2] = $440;
      $441 = HEAP32[$v>>2]|0;
      $442 = HEAP32[$iAddr>>2]|0;
      $443 = HEAP32[$iDb>>2]|0;
      _sqlite3VdbeChangeP1($441,$442,$443);
      $444 = HEAP32[$v>>2]|0;
      $445 = HEAP32[$iAddr>>2]|0;
      $446 = (($445) + 1)|0;
      $447 = HEAP32[$iDb>>2]|0;
      _sqlite3VdbeChangeP1($444,$446,$447);
      $448 = HEAP32[$v>>2]|0;
      $449 = HEAP32[$iAddr>>2]|0;
      $450 = (($449) + 2)|0;
      $451 = HEAP32[$iAddr>>2]|0;
      $452 = (($451) + 4)|0;
      _sqlite3VdbeChangeP2($448,$450,$452);
      $453 = HEAP32[$v>>2]|0;
      $454 = HEAP32[$iAddr>>2]|0;
      $455 = (($454) + 4)|0;
      $456 = HEAP32[$eAuto>>2]|0;
      $457 = (($456) - 1)|0;
      _sqlite3VdbeChangeP1($453,$455,$457);
      $458 = HEAP32[$v>>2]|0;
      $459 = HEAP32[$iAddr>>2]|0;
      $460 = (($459) + 5)|0;
      $461 = HEAP32[$iDb>>2]|0;
      _sqlite3VdbeChangeP1($458,$460,$461);
      $462 = HEAP32[$v>>2]|0;
      $463 = HEAP32[$iDb>>2]|0;
      _sqlite3VdbeUsesBtree($462,$463);
      break L21;
      break;
     }
     case 19:  {
      $368 = HEAP32[$pDb>>2]|0;
      $369 = (($368) + 4|0);
      $370 = HEAP32[$369>>2]|0;
      $371 = (_sqlite3BtreePager($370)|0);
      HEAP32[$pPager6>>2] = $371;
      $372 = $iLimit;
      $373 = $372;
      HEAP32[$373>>2] = -2;
      $374 = (($372) + 4)|0;
      $375 = $374;
      HEAP32[$375>>2] = -1;
      $376 = HEAP32[$zRight>>2]|0;
      $377 = ($376|0)!=(0|0);
      if ($377) {
       $378 = HEAP32[$zRight>>2]|0;
       (_sqlite3DecOrHexToI64($378,$iLimit)|0);
       $379 = $iLimit;
       $380 = $379;
       $381 = HEAP32[$380>>2]|0;
       $382 = (($379) + 4)|0;
       $383 = $382;
       $384 = HEAP32[$383>>2]|0;
       $385 = ($384|0)<(-1);
       $386 = ($381>>>0)<(4294967295);
       $387 = ($384|0)==(-1);
       $388 = $387 & $386;
       $389 = $385 | $388;
       if ($389) {
        $390 = $iLimit;
        $391 = $390;
        HEAP32[$391>>2] = -1;
        $392 = (($390) + 4)|0;
        $393 = $392;
        HEAP32[$393>>2] = -1;
       }
      }
      $394 = HEAP32[$pPager6>>2]|0;
      $395 = $iLimit;
      $396 = $395;
      $397 = HEAP32[$396>>2]|0;
      $398 = (($395) + 4)|0;
      $399 = $398;
      $400 = HEAP32[$399>>2]|0;
      $401 = (_sqlite3PagerJournalSizeLimit($394,$397,$400)|0);
      $402 = tempRet0;
      $403 = $iLimit;
      $404 = $403;
      HEAP32[$404>>2] = $401;
      $405 = (($403) + 4)|0;
      $406 = $405;
      HEAP32[$406>>2] = $402;
      $407 = HEAP32[$0>>2]|0;
      $408 = $iLimit;
      $409 = $408;
      $410 = HEAP32[$409>>2]|0;
      $411 = (($408) + 4)|0;
      $412 = $411;
      $413 = HEAP32[$412>>2]|0;
      _returnSingleInt($407,21120,$410,$413);
      break L21;
      break;
     }
     case 18:  {
      $313 = HEAP32[$v>>2]|0;
      _sqlite3VdbeSetNumCols($313,1);
      $314 = HEAP32[$v>>2]|0;
      (_sqlite3VdbeSetColName($314,0,0,21104,0)|0);
      $315 = HEAP32[$zRight>>2]|0;
      $316 = ($315|0)==(0|0);
      if ($316) {
       HEAP32[$eMode4>>2] = -1;
      } else {
       $317 = HEAP32[$zRight>>2]|0;
       $318 = (_sqlite3Strlen30($317)|0);
       HEAP32[$n>>2] = $318;
       HEAP32[$eMode4>>2] = 0;
       while(1) {
        $319 = HEAP32[$eMode4>>2]|0;
        $320 = (_sqlite3JournalModename($319)|0);
        HEAP32[$zMode>>2] = $320;
        $321 = ($320|0)!=(0|0);
        if (!($321)) {
         break;
        }
        $322 = HEAP32[$zRight>>2]|0;
        $323 = HEAP32[$zMode>>2]|0;
        $324 = HEAP32[$n>>2]|0;
        $325 = (_sqlite3_strnicmp($322,$323,$324)|0);
        $326 = ($325|0)==(0);
        if ($326) {
         break;
        }
        $327 = HEAP32[$eMode4>>2]|0;
        $328 = (($327) + 1)|0;
        HEAP32[$eMode4>>2] = $328;
       }
       $329 = HEAP32[$zMode>>2]|0;
       $330 = ($329|0)!=(0|0);
       if (!($330)) {
        HEAP32[$eMode4>>2] = -1;
       }
      }
      $331 = HEAP32[$eMode4>>2]|0;
      $332 = ($331|0)==(-1);
      if ($332) {
       $333 = HEAP32[$2>>2]|0;
       $334 = (($333) + 4|0);
       $335 = HEAP32[$334>>2]|0;
       $336 = ($335|0)==(0);
       if ($336) {
        HEAP32[$iDb>>2] = 0;
        $337 = HEAP32[$2>>2]|0;
        $338 = (($337) + 4|0);
        HEAP32[$338>>2] = 1;
       }
      }
      $339 = HEAP32[$db>>2]|0;
      $340 = (($339) + 20|0);
      $341 = HEAP32[$340>>2]|0;
      $342 = (($341) - 1)|0;
      HEAP32[$ii5>>2] = $342;
      while(1) {
       $343 = HEAP32[$ii5>>2]|0;
       $344 = ($343|0)>=(0);
       if (!($344)) {
        break;
       }
       $345 = HEAP32[$ii5>>2]|0;
       $346 = HEAP32[$db>>2]|0;
       $347 = (($346) + 16|0);
       $348 = HEAP32[$347>>2]|0;
       $349 = (($348) + ($345<<4)|0);
       $350 = (($349) + 4|0);
       $351 = HEAP32[$350>>2]|0;
       $352 = ($351|0)!=(0|0);
       do {
        if ($352) {
         $353 = HEAP32[$ii5>>2]|0;
         $354 = HEAP32[$iDb>>2]|0;
         $355 = ($353|0)==($354|0);
         if (!($355)) {
          $356 = HEAP32[$2>>2]|0;
          $357 = (($356) + 4|0);
          $358 = HEAP32[$357>>2]|0;
          $359 = ($358|0)==(0);
          if (!($359)) {
           break;
          }
         }
         $360 = HEAP32[$v>>2]|0;
         $361 = HEAP32[$ii5>>2]|0;
         _sqlite3VdbeUsesBtree($360,$361);
         $362 = HEAP32[$v>>2]|0;
         $363 = HEAP32[$ii5>>2]|0;
         $364 = HEAP32[$eMode4>>2]|0;
         (_sqlite3VdbeAddOp3($362,12,$363,1,$364)|0);
        }
       } while(0);
       $365 = HEAP32[$ii5>>2]|0;
       $366 = (($365) + -1)|0;
       HEAP32[$ii5>>2] = $366;
      }
      $367 = HEAP32[$v>>2]|0;
      (_sqlite3VdbeAddOp2($367,35,1,1)|0);
      break L21;
      break;
     }
     case 23:  {
      $509 = $sz;
      $510 = $509;
      HEAP32[$510>>2] = 0;
      $511 = (($509) + 4)|0;
      $512 = $511;
      HEAP32[$512>>2] = 0;
      HEAP32[$rc>>2] = 0;
      $513 = HEAP32[$rc>>2]|0;
      $514 = ($513|0)==(0);
      if ($514) {
       $515 = HEAP32[$0>>2]|0;
       $516 = $sz;
       $517 = $516;
       $518 = HEAP32[$517>>2]|0;
       $519 = (($516) + 4)|0;
       $520 = $519;
       $521 = HEAP32[$520>>2]|0;
       _returnSingleInt($515,21184,$518,$521);
       break L21;
      }
      $522 = HEAP32[$rc>>2]|0;
      $523 = ($522|0)!=(12);
      if (!($523)) {
       break L21;
      }
      $524 = HEAP32[$0>>2]|0;
      $525 = (($524) + 64|0);
      $526 = HEAP32[$525>>2]|0;
      $527 = (($526) + 1)|0;
      HEAP32[$525>>2] = $527;
      $528 = HEAP32[$rc>>2]|0;
      $529 = HEAP32[$0>>2]|0;
      $530 = (($529) + 12|0);
      HEAP32[$530>>2] = $528;
      break L21;
      break;
     }
     case 24:  {
      $176 = HEAP32[$pDb>>2]|0;
      $177 = (($176) + 4|0);
      $178 = HEAP32[$177>>2]|0;
      HEAP32[$pBt>>2] = $178;
      $179 = HEAP32[$zRight>>2]|0;
      $180 = ($179|0)!=(0|0);
      if ($180) {
       $190 = HEAP32[$zRight>>2]|0;
       $191 = (_sqlite3Atoi($190)|0);
       $192 = HEAP32[$db>>2]|0;
       $193 = (($192) + 72|0);
       HEAP32[$193>>2] = $191;
       $194 = HEAP32[$pBt>>2]|0;
       $195 = HEAP32[$db>>2]|0;
       $196 = (($195) + 72|0);
       $197 = HEAP32[$196>>2]|0;
       $198 = (_sqlite3BtreeSetPageSize($194,$197,-1,0)|0);
       $199 = (7)==($198|0);
       if (!($199)) {
        break L21;
       }
       $200 = HEAP32[$db>>2]|0;
       $201 = (($200) + 64|0);
       HEAP8[$201>>0] = 1;
       break L21;
      }
      $181 = HEAP32[$pBt>>2]|0;
      $182 = ($181|0)!=(0|0);
      if ($182) {
       $183 = HEAP32[$pBt>>2]|0;
       $184 = (_sqlite3BtreeGetPageSize($183)|0);
       $185 = $184;
      } else {
       $185 = 0;
      }
      HEAP32[$size1>>2] = $185;
      $186 = HEAP32[$0>>2]|0;
      $187 = HEAP32[$size1>>2]|0;
      $188 = ($187|0)<(0);
      $189 = $188 << 31 >> 31;
      _returnSingleInt($186,21032,$187,$189);
      break L21;
      break;
     }
     case 2:  {
      $595 = HEAP32[$zRight>>2]|0;
      $596 = ($595|0)==(0|0);
      if ($596) {
       $597 = HEAP32[$0>>2]|0;
       $598 = HEAP32[$mid>>2]|0;
       $599 = (20312 + (($598*12)|0)|0);
       $600 = HEAP32[$599>>2]|0;
       $601 = HEAP32[$db>>2]|0;
       $602 = (($601) + 24|0);
       $603 = HEAP32[$602>>2]|0;
       $604 = HEAP32[$mid>>2]|0;
       $605 = (20312 + (($604*12)|0)|0);
       $606 = (($605) + 8|0);
       $607 = HEAP32[$606>>2]|0;
       $608 = $603 & $607;
       $609 = ($608|0)!=(0);
       $610 = $609&1;
       $611 = ($610|0)<(0);
       $612 = $611 << 31 >> 31;
       _returnSingleInt($597,$600,$610,$612);
       break L21;
      }
      $613 = HEAP32[$mid>>2]|0;
      $614 = (20312 + (($613*12)|0)|0);
      $615 = (($614) + 8|0);
      $616 = HEAP32[$615>>2]|0;
      HEAP32[$mask>>2] = $616;
      $617 = HEAP32[$db>>2]|0;
      $618 = (($617) + 62|0);
      $619 = HEAP8[$618>>0]|0;
      $620 = $619&255;
      $621 = ($620|0)==(0);
      if ($621) {
       $622 = HEAP32[$mask>>2]|0;
       $623 = $622 & -524289;
       HEAP32[$mask>>2] = $623;
      }
      $624 = HEAP32[$zRight>>2]|0;
      $625 = (_sqlite3GetBoolean($624,0)|0);
      $626 = ($625<<24>>24)!=(0);
      $627 = HEAP32[$mask>>2]|0;
      if ($626) {
       $628 = HEAP32[$db>>2]|0;
       $629 = (($628) + 24|0);
       $630 = HEAP32[$629>>2]|0;
       $631 = $630 | $627;
       HEAP32[$629>>2] = $631;
      } else {
       $632 = $627 ^ -1;
       $633 = HEAP32[$db>>2]|0;
       $634 = (($633) + 24|0);
       $635 = HEAP32[$634>>2]|0;
       $636 = $635 & $632;
       HEAP32[$634>>2] = $636;
       $637 = HEAP32[$mask>>2]|0;
       $638 = ($637|0)==(16777216);
       if ($638) {
        $639 = HEAP32[$db>>2]|0;
        $640 = (($639) + 512|0);
        $641 = $640;
        $642 = $641;
        HEAP32[$642>>2] = 0;
        $643 = (($641) + 4)|0;
        $644 = $643;
        HEAP32[$644>>2] = 0;
       }
      }
      $645 = HEAP32[$v>>2]|0;
      (_sqlite3VdbeAddOp2($645,142,0,0)|0);
      $646 = HEAP32[$db>>2]|0;
      _setAllPagerFlags($646);
      break L21;
      break;
     }
     case 10:  {
      $132 = HEAP32[$v>>2]|0;
      $133 = HEAP32[$iDb>>2]|0;
      _sqlite3VdbeUsesBtree($132,$133);
      $134 = HEAP32[$zRight>>2]|0;
      $135 = ($134|0)!=(0|0);
      if ($135) {
       $154 = HEAP32[$zRight>>2]|0;
       $155 = (_sqlite3Atoi($154)|0);
       $156 = (_sqlite3AbsInt32($155)|0);
       HEAP32[$size>>2] = $156;
       $157 = HEAP32[$0>>2]|0;
       $158 = HEAP32[$iDb>>2]|0;
       _sqlite3BeginWriteOperation($157,0,$158);
       $159 = HEAP32[$v>>2]|0;
       $160 = HEAP32[$size>>2]|0;
       (_sqlite3VdbeAddOp2($159,25,$160,1)|0);
       $161 = HEAP32[$v>>2]|0;
       $162 = HEAP32[$iDb>>2]|0;
       (_sqlite3VdbeAddOp3($161,52,$162,3,1)|0);
       $163 = HEAP32[$size>>2]|0;
       $164 = HEAP32[$pDb>>2]|0;
       $165 = (($164) + 12|0);
       $166 = HEAP32[$165>>2]|0;
       $167 = (($166) + 80|0);
       HEAP32[$167>>2] = $163;
       $168 = HEAP32[$pDb>>2]|0;
       $169 = (($168) + 4|0);
       $170 = HEAP32[$169>>2]|0;
       $171 = HEAP32[$pDb>>2]|0;
       $172 = (($171) + 12|0);
       $173 = HEAP32[$172>>2]|0;
       $174 = (($173) + 80|0);
       $175 = HEAP32[$174>>2]|0;
       (_sqlite3BtreeSetCacheSize($170,$175)|0);
       break L21;
      } else {
       $136 = HEAP32[$v>>2]|0;
       _sqlite3VdbeSetNumCols($136,1);
       $137 = HEAP32[$v>>2]|0;
       (_sqlite3VdbeSetColName($137,0,0,21016,0)|0);
       $138 = HEAP32[$0>>2]|0;
       $139 = (($138) + 72|0);
       $140 = HEAP32[$139>>2]|0;
       $141 = (($140) + 2)|0;
       HEAP32[$139>>2] = $141;
       $142 = HEAP32[$v>>2]|0;
       $143 = (_sqlite3VdbeAddOpList($142,9,20976,0)|0);
       HEAP32[$addr>>2] = $143;
       $144 = HEAP32[$v>>2]|0;
       $145 = HEAP32[$addr>>2]|0;
       $146 = HEAP32[$iDb>>2]|0;
       _sqlite3VdbeChangeP1($144,$145,$146);
       $147 = HEAP32[$v>>2]|0;
       $148 = HEAP32[$addr>>2]|0;
       $149 = (($148) + 1)|0;
       $150 = HEAP32[$iDb>>2]|0;
       _sqlite3VdbeChangeP1($147,$149,$150);
       $151 = HEAP32[$v>>2]|0;
       $152 = HEAP32[$addr>>2]|0;
       $153 = (($152) + 6)|0;
       _sqlite3VdbeChangeP1($151,$153,2000);
       break L21;
      }
      break;
     }
     case 21:  {
      HEAP32[$zRet>>2] = 21064;
      $262 = HEAP32[$zRight>>2]|0;
      $263 = (_getLockingMode($262)|0);
      HEAP32[$eMode>>2] = $263;
      $264 = HEAP32[$2>>2]|0;
      $265 = (($264) + 4|0);
      $266 = HEAP32[$265>>2]|0;
      $267 = ($266|0)==(0);
      $268 = HEAP32[$eMode>>2]|0;
      $269 = ($268|0)==(-1);
      $or$cond3 = $267 & $269;
      if ($or$cond3) {
       $270 = HEAP32[$db>>2]|0;
       $271 = (($270) + 65|0);
       $272 = HEAP8[$271>>0]|0;
       $273 = $272&255;
       HEAP32[$eMode>>2] = $273;
      } else {
       $274 = HEAP32[$2>>2]|0;
       $275 = (($274) + 4|0);
       $276 = HEAP32[$275>>2]|0;
       $277 = ($276|0)==(0);
       if ($277) {
        HEAP32[$ii3>>2] = 2;
        while(1) {
         $278 = HEAP32[$ii3>>2]|0;
         $279 = HEAP32[$db>>2]|0;
         $280 = (($279) + 20|0);
         $281 = HEAP32[$280>>2]|0;
         $282 = ($278|0)<($281|0);
         if (!($282)) {
          break;
         }
         $283 = HEAP32[$ii3>>2]|0;
         $284 = HEAP32[$db>>2]|0;
         $285 = (($284) + 16|0);
         $286 = HEAP32[$285>>2]|0;
         $287 = (($286) + ($283<<4)|0);
         $288 = (($287) + 4|0);
         $289 = HEAP32[$288>>2]|0;
         $290 = (_sqlite3BtreePager($289)|0);
         HEAP32[$pPager>>2] = $290;
         $291 = HEAP32[$pPager>>2]|0;
         $292 = HEAP32[$eMode>>2]|0;
         (_sqlite3PagerLockingMode($291,$292)|0);
         $293 = HEAP32[$ii3>>2]|0;
         $294 = (($293) + 1)|0;
         HEAP32[$ii3>>2] = $294;
        }
        $295 = HEAP32[$eMode>>2]|0;
        $296 = $295&255;
        $297 = HEAP32[$db>>2]|0;
        $298 = (($297) + 65|0);
        HEAP8[$298>>0] = $296;
       }
       $299 = HEAP32[$pDb>>2]|0;
       $300 = (($299) + 4|0);
       $301 = HEAP32[$300>>2]|0;
       $302 = (_sqlite3BtreePager($301)|0);
       HEAP32[$pPager>>2] = $302;
       $303 = HEAP32[$pPager>>2]|0;
       $304 = HEAP32[$eMode>>2]|0;
       $305 = (_sqlite3PagerLockingMode($303,$304)|0);
       HEAP32[$eMode>>2] = $305;
      }
      $306 = HEAP32[$eMode>>2]|0;
      $307 = ($306|0)==(1);
      if ($307) {
       HEAP32[$zRet>>2] = 21072;
      }
      $308 = HEAP32[$v>>2]|0;
      _sqlite3VdbeSetNumCols($308,1);
      $309 = HEAP32[$v>>2]|0;
      (_sqlite3VdbeSetColName($309,0,0,21088,0)|0);
      $310 = HEAP32[$v>>2]|0;
      $311 = HEAP32[$zRet>>2]|0;
      (_sqlite3VdbeAddOp4($310,97,0,1,0,$311,0)|0);
      $312 = HEAP32[$v>>2]|0;
      (_sqlite3VdbeAddOp2($312,35,1,1)|0);
      break L21;
      break;
     }
     case 22:  {
      $238 = HEAP32[$0>>2]|0;
      $239 = HEAP32[$iDb>>2]|0;
      _sqlite3CodeVerifySchema($238,$239);
      $240 = HEAP32[$0>>2]|0;
      $241 = (($240) + 72|0);
      $242 = HEAP32[$241>>2]|0;
      $243 = (($242) + 1)|0;
      HEAP32[$241>>2] = $243;
      HEAP32[$iReg>>2] = $243;
      $244 = HEAP32[$zLeft>>2]|0;
      $245 = HEAP8[$244>>0]|0;
      $246 = $245&255;
      $247 = (440 + ($246)|0);
      $248 = HEAP8[$247>>0]|0;
      $249 = $248&255;
      $250 = ($249|0)==(112);
      $251 = HEAP32[$v>>2]|0;
      $252 = HEAP32[$iDb>>2]|0;
      $253 = HEAP32[$iReg>>2]|0;
      if ($250) {
       (_sqlite3VdbeAddOp2($251,151,$252,$253)|0);
      } else {
       $254 = HEAP32[$zRight>>2]|0;
       $255 = (_sqlite3Atoi($254)|0);
       $256 = (_sqlite3AbsInt32($255)|0);
       (_sqlite3VdbeAddOp3($251,152,$252,$253,$256)|0);
      }
      $257 = HEAP32[$v>>2]|0;
      $258 = HEAP32[$iReg>>2]|0;
      (_sqlite3VdbeAddOp2($257,35,$258,1)|0);
      $259 = HEAP32[$v>>2]|0;
      _sqlite3VdbeSetNumCols($259,1);
      $260 = HEAP32[$v>>2]|0;
      $261 = HEAP32[$zLeft>>2]|0;
      (_sqlite3VdbeSetColName($260,0,0,$261,(-1))|0);
      break L21;
      break;
     }
     case 14:  {
      $464 = HEAP32[$zRight>>2]|0;
      $465 = ($464|0)==(0|0);
      if ($465) {
       label = 90;
      } else {
       $466 = HEAP32[$zRight>>2]|0;
       $467 = (_sqlite3GetInt32($466,$iLimit8)|0);
       $468 = ($467|0)==(0);
       $469 = HEAP32[$iLimit8>>2]|0;
       $470 = ($469|0)<=(0);
       $or$cond7 = $468 | $470;
       if ($or$cond7) {
        label = 90;
       }
      }
      if ((label|0) == 90) {
       HEAP32[$iLimit8>>2] = 2147483647;
      }
      $471 = HEAP32[$0>>2]|0;
      $472 = HEAP32[$iDb>>2]|0;
      _sqlite3BeginWriteOperation($471,0,$472);
      $473 = HEAP32[$v>>2]|0;
      $474 = HEAP32[$iLimit8>>2]|0;
      (_sqlite3VdbeAddOp2($473,25,$474,1)|0);
      $475 = HEAP32[$v>>2]|0;
      $476 = HEAP32[$iDb>>2]|0;
      $477 = (_sqlite3VdbeAddOp1($475,141,$476)|0);
      HEAP32[$addr9>>2] = $477;
      $478 = HEAP32[$v>>2]|0;
      (_sqlite3VdbeAddOp1($478,35,1)|0);
      $479 = HEAP32[$v>>2]|0;
      (_sqlite3VdbeAddOp2($479,37,1,-1)|0);
      $480 = HEAP32[$v>>2]|0;
      $481 = HEAP32[$addr9>>2]|0;
      (_sqlite3VdbeAddOp2($480,137,1,$481)|0);
      $482 = HEAP32[$v>>2]|0;
      $483 = HEAP32[$addr9>>2]|0;
      _sqlite3VdbeJumpHere($482,$483);
      break L21;
      break;
     }
     case 15:  {
      $817 = HEAP32[$zRight>>2]|0;
      $818 = ($817|0)!=(0|0);
      if (!($818)) {
       break L21;
      }
      $819 = HEAP32[$db>>2]|0;
      $820 = HEAP32[$zRight>>2]|0;
      $821 = HEAP32[$zDb>>2]|0;
      $822 = (_sqlite3FindIndex($819,$820,$821)|0);
      HEAP32[$pIdx13>>2] = $822;
      $823 = HEAP32[$pIdx13>>2]|0;
      $824 = ($823|0)!=(0|0);
      if (!($824)) {
       break L21;
      }
      $825 = HEAP32[$pIdx13>>2]|0;
      $826 = (($825) + 12|0);
      $827 = HEAP32[$826>>2]|0;
      HEAP32[$pTab14>>2] = $827;
      $828 = HEAP32[$v>>2]|0;
      _sqlite3VdbeSetNumCols($828,3);
      $829 = HEAP32[$0>>2]|0;
      $830 = (($829) + 72|0);
      HEAP32[$830>>2] = 3;
      $831 = HEAP32[$0>>2]|0;
      $832 = HEAP32[$iDb>>2]|0;
      _sqlite3CodeVerifySchema($831,$832);
      $833 = HEAP32[$v>>2]|0;
      (_sqlite3VdbeSetColName($833,0,0,21424,0)|0);
      $834 = HEAP32[$v>>2]|0;
      (_sqlite3VdbeSetColName($834,1,0,21344,0)|0);
      $835 = HEAP32[$v>>2]|0;
      (_sqlite3VdbeSetColName($835,2,0,21352,0)|0);
      HEAP32[$i15>>2] = 0;
      while(1) {
       $836 = HEAP32[$i15>>2]|0;
       $837 = HEAP32[$pIdx13>>2]|0;
       $838 = (($837) + 50|0);
       $839 = HEAP16[$838>>1]|0;
       $840 = $839&65535;
       $841 = ($836|0)<($840|0);
       if (!($841)) {
        break L21;
       }
       $842 = HEAP32[$i15>>2]|0;
       $843 = HEAP32[$pIdx13>>2]|0;
       $844 = (($843) + 4|0);
       $845 = HEAP32[$844>>2]|0;
       $846 = (($845) + ($842<<1)|0);
       $847 = HEAP16[$846>>1]|0;
       HEAP16[$cnum>>1] = $847;
       $848 = HEAP32[$v>>2]|0;
       $849 = HEAP32[$i15>>2]|0;
       (_sqlite3VdbeAddOp2($848,25,$849,1)|0);
       $850 = HEAP32[$v>>2]|0;
       $851 = HEAP16[$cnum>>1]|0;
       $852 = $851 << 16 >> 16;
       (_sqlite3VdbeAddOp2($850,25,$852,2)|0);
       $853 = HEAP32[$v>>2]|0;
       $854 = HEAP16[$cnum>>1]|0;
       $855 = $854 << 16 >> 16;
       $856 = HEAP32[$pTab14>>2]|0;
       $857 = (($856) + 4|0);
       $858 = HEAP32[$857>>2]|0;
       $859 = (($858) + (($855*24)|0)|0);
       $860 = HEAP32[$859>>2]|0;
       (_sqlite3VdbeAddOp4($853,97,0,3,0,$860,0)|0);
       $861 = HEAP32[$v>>2]|0;
       (_sqlite3VdbeAddOp2($861,35,1,3)|0);
       $862 = HEAP32[$i15>>2]|0;
       $863 = (($862) + 1)|0;
       HEAP32[$i15>>2] = $863;
      }
      break;
     }
     case 31:  {
      $531 = HEAP32[$zRight>>2]|0;
      $532 = ($531|0)!=(0|0);
      $533 = HEAP32[$0>>2]|0;
      if ($532) {
       $538 = HEAP32[$zRight>>2]|0;
       (_changeTempStorage($533,$538)|0);
       break L21;
      } else {
       $534 = HEAP32[$db>>2]|0;
       $535 = (($534) + 63|0);
       $536 = HEAP8[$535>>0]|0;
       $537 = $536&255;
       _returnSingleInt($533,21200,$537,0);
       break L21;
      }
      break;
     }
     case 32:  {
      $539 = HEAP32[$zRight>>2]|0;
      $540 = ($539|0)!=(0|0);
      if (!($540)) {
       $541 = HEAP32[1568>>2]|0;
       $542 = ($541|0)!=(0|0);
       if (!($542)) {
        break L21;
       }
       $543 = HEAP32[$v>>2]|0;
       _sqlite3VdbeSetNumCols($543,1);
       $544 = HEAP32[$v>>2]|0;
       (_sqlite3VdbeSetColName($544,0,0,21216,0)|0);
       $545 = HEAP32[$v>>2]|0;
       $546 = HEAP32[1568>>2]|0;
       (_sqlite3VdbeAddOp4($545,97,0,1,0,$546,0)|0);
       $547 = HEAP32[$v>>2]|0;
       (_sqlite3VdbeAddOp2($547,35,1,1)|0);
       break L21;
      }
      $548 = HEAP32[$zRight>>2]|0;
      $549 = HEAP8[$548>>0]|0;
      $550 = ($549<<24>>24)!=(0);
      if ($550) {
       $551 = HEAP32[$db>>2]|0;
       $552 = HEAP32[$551>>2]|0;
       $553 = HEAP32[$zRight>>2]|0;
       $554 = (_sqlite3OsAccess($552,$553,1,$res)|0);
       HEAP32[$rc>>2] = $554;
       $555 = HEAP32[$rc>>2]|0;
       $556 = ($555|0)!=(0);
       $557 = HEAP32[$res>>2]|0;
       $558 = ($557|0)==(0);
       $or$cond9 = $556 | $558;
       if ($or$cond9) {
        $559 = HEAP32[$0>>2]|0;
        _sqlite3ErrorMsg($559,21240,$vararg_buffer35);
        break L21;
       }
      }
      $560 = HEAP32[$db>>2]|0;
      $561 = (($560) + 63|0);
      $562 = HEAP8[$561>>0]|0;
      $563 = $562&255;
      $564 = ($563|0)<=(1);
      if ($564) {
       $565 = HEAP32[$0>>2]|0;
       (_invalidateTempStorage($565)|0);
      }
      $566 = HEAP32[1568>>2]|0;
      _sqlite3_free($566);
      $567 = HEAP32[$zRight>>2]|0;
      $568 = HEAP8[$567>>0]|0;
      $569 = ($568<<24>>24)!=(0);
      if ($569) {
       $570 = HEAP32[$zRight>>2]|0;
       HEAP32[$vararg_buffer35>>2] = $570;
       $571 = (_sqlite3_mprintf(1200,$vararg_buffer35)|0);
       HEAP32[1568>>2] = $571;
       break L21;
      } else {
       HEAP32[1568>>2] = 0;
       break L21;
      }
      break;
     }
     case 9:  {
      $905 = HEAP32[$v>>2]|0;
      _sqlite3VdbeSetNumCols($905,3);
      $906 = HEAP32[$0>>2]|0;
      $907 = (($906) + 72|0);
      HEAP32[$907>>2] = 3;
      $908 = HEAP32[$v>>2]|0;
      (_sqlite3VdbeSetColName($908,0,0,21432,0)|0);
      $909 = HEAP32[$v>>2]|0;
      (_sqlite3VdbeSetColName($909,1,0,21352,0)|0);
      $910 = HEAP32[$v>>2]|0;
      (_sqlite3VdbeSetColName($910,2,0,21448,0)|0);
      HEAP32[$i19>>2] = 0;
      while(1) {
       $911 = HEAP32[$i19>>2]|0;
       $912 = HEAP32[$db>>2]|0;
       $913 = (($912) + 20|0);
       $914 = HEAP32[$913>>2]|0;
       $915 = ($911|0)<($914|0);
       if (!($915)) {
        break L21;
       }
       $916 = HEAP32[$i19>>2]|0;
       $917 = HEAP32[$db>>2]|0;
       $918 = (($917) + 16|0);
       $919 = HEAP32[$918>>2]|0;
       $920 = (($919) + ($916<<4)|0);
       $921 = (($920) + 4|0);
       $922 = HEAP32[$921>>2]|0;
       $923 = ($922|0)==(0|0);
       if (!($923)) {
        $924 = HEAP32[$v>>2]|0;
        $925 = HEAP32[$i19>>2]|0;
        (_sqlite3VdbeAddOp2($924,25,$925,1)|0);
        $926 = HEAP32[$v>>2]|0;
        $927 = HEAP32[$i19>>2]|0;
        $928 = HEAP32[$db>>2]|0;
        $929 = (($928) + 16|0);
        $930 = HEAP32[$929>>2]|0;
        $931 = (($930) + ($927<<4)|0);
        $932 = HEAP32[$931>>2]|0;
        (_sqlite3VdbeAddOp4($926,97,0,2,0,$932,0)|0);
        $933 = HEAP32[$v>>2]|0;
        $934 = HEAP32[$i19>>2]|0;
        $935 = HEAP32[$db>>2]|0;
        $936 = (($935) + 16|0);
        $937 = HEAP32[$936>>2]|0;
        $938 = (($937) + ($934<<4)|0);
        $939 = (($938) + 4|0);
        $940 = HEAP32[$939>>2]|0;
        $941 = (_sqlite3BtreeGetFilename($940)|0);
        (_sqlite3VdbeAddOp4($933,97,0,3,0,$941,0)|0);
        $942 = HEAP32[$v>>2]|0;
        (_sqlite3VdbeAddOp2($942,35,1,3)|0);
       }
       $943 = HEAP32[$i19>>2]|0;
       $944 = (($943) + 1)|0;
       HEAP32[$i19>>2] = $944;
      }
      break;
     }
     case 13:  {
      $968 = HEAP32[$zRight>>2]|0;
      $969 = ($968|0)!=(0|0);
      if (!($969)) {
       break L21;
      }
      $970 = HEAP32[$db>>2]|0;
      $971 = HEAP32[$zRight>>2]|0;
      $972 = HEAP32[$zDb>>2]|0;
      $973 = (_sqlite3FindTable($970,$971,$972)|0);
      HEAP32[$pTab21>>2] = $973;
      $974 = HEAP32[$pTab21>>2]|0;
      $975 = ($974|0)!=(0|0);
      if (!($975)) {
       break L21;
      }
      $976 = HEAP32[$0>>2]|0;
      $977 = (_sqlite3GetVdbe($976)|0);
      HEAP32[$v>>2] = $977;
      $978 = HEAP32[$pTab21>>2]|0;
      $979 = (($978) + 16|0);
      $980 = HEAP32[$979>>2]|0;
      HEAP32[$pFK>>2] = $980;
      $981 = HEAP32[$pFK>>2]|0;
      $982 = ($981|0)!=(0|0);
      if (!($982)) {
       break L21;
      }
      HEAP32[$i22>>2] = 0;
      $983 = HEAP32[$v>>2]|0;
      _sqlite3VdbeSetNumCols($983,8);
      $984 = HEAP32[$0>>2]|0;
      $985 = (($984) + 72|0);
      HEAP32[$985>>2] = 8;
      $986 = HEAP32[$0>>2]|0;
      $987 = HEAP32[$iDb>>2]|0;
      _sqlite3CodeVerifySchema($986,$987);
      $988 = HEAP32[$v>>2]|0;
      (_sqlite3VdbeSetColName($988,0,0,21456,0)|0);
      $989 = HEAP32[$v>>2]|0;
      (_sqlite3VdbeSetColName($989,1,0,21432,0)|0);
      $990 = HEAP32[$v>>2]|0;
      (_sqlite3VdbeSetColName($990,2,0,21400,0)|0);
      $991 = HEAP32[$v>>2]|0;
      (_sqlite3VdbeSetColName($991,3,0,21464,0)|0);
      $992 = HEAP32[$v>>2]|0;
      (_sqlite3VdbeSetColName($992,4,0,21472,0)|0);
      $993 = HEAP32[$v>>2]|0;
      (_sqlite3VdbeSetColName($993,5,0,21480,0)|0);
      $994 = HEAP32[$v>>2]|0;
      (_sqlite3VdbeSetColName($994,6,0,21496,0)|0);
      $995 = HEAP32[$v>>2]|0;
      (_sqlite3VdbeSetColName($995,7,0,21512,0)|0);
      while(1) {
       $996 = HEAP32[$pFK>>2]|0;
       $997 = ($996|0)!=(0|0);
       if (!($997)) {
        break L21;
       }
       HEAP32[$j>>2] = 0;
       while(1) {
        $998 = HEAP32[$j>>2]|0;
        $999 = HEAP32[$pFK>>2]|0;
        $1000 = (($999) + 20|0);
        $1001 = HEAP32[$1000>>2]|0;
        $1002 = ($998|0)<($1001|0);
        if (!($1002)) {
         break;
        }
        $1003 = HEAP32[$j>>2]|0;
        $1004 = HEAP32[$pFK>>2]|0;
        $1005 = (($1004) + 36|0);
        $1006 = (($1005) + ($1003<<3)|0);
        $1007 = (($1006) + 4|0);
        $1008 = HEAP32[$1007>>2]|0;
        HEAP32[$zCol>>2] = $1008;
        $1009 = HEAP32[$pFK>>2]|0;
        $1010 = (($1009) + 25|0);
        $1011 = HEAP8[$1010>>0]|0;
        $1012 = (_actionName($1011)|0);
        HEAP32[$zOnDelete>>2] = $1012;
        $1013 = HEAP32[$pFK>>2]|0;
        $1014 = (($1013) + 25|0);
        $1015 = (($1014) + 1|0);
        $1016 = HEAP8[$1015>>0]|0;
        $1017 = (_actionName($1016)|0);
        HEAP32[$zOnUpdate>>2] = $1017;
        $1018 = HEAP32[$v>>2]|0;
        $1019 = HEAP32[$i22>>2]|0;
        (_sqlite3VdbeAddOp2($1018,25,$1019,1)|0);
        $1020 = HEAP32[$v>>2]|0;
        $1021 = HEAP32[$j>>2]|0;
        (_sqlite3VdbeAddOp2($1020,25,$1021,2)|0);
        $1022 = HEAP32[$v>>2]|0;
        $1023 = HEAP32[$pFK>>2]|0;
        $1024 = (($1023) + 8|0);
        $1025 = HEAP32[$1024>>2]|0;
        (_sqlite3VdbeAddOp4($1022,97,0,3,0,$1025,0)|0);
        $1026 = HEAP32[$v>>2]|0;
        $1027 = HEAP32[$j>>2]|0;
        $1028 = HEAP32[$pFK>>2]|0;
        $1029 = (($1028) + 36|0);
        $1030 = (($1029) + ($1027<<3)|0);
        $1031 = HEAP32[$1030>>2]|0;
        $1032 = HEAP32[$pTab21>>2]|0;
        $1033 = (($1032) + 4|0);
        $1034 = HEAP32[$1033>>2]|0;
        $1035 = (($1034) + (($1031*24)|0)|0);
        $1036 = HEAP32[$1035>>2]|0;
        (_sqlite3VdbeAddOp4($1026,97,0,4,0,$1036,0)|0);
        $1037 = HEAP32[$v>>2]|0;
        $1038 = HEAP32[$zCol>>2]|0;
        $1039 = ($1038|0)!=(0|0);
        $1040 = $1039 ? 97 : 28;
        $1041 = HEAP32[$zCol>>2]|0;
        (_sqlite3VdbeAddOp4($1037,$1040,0,5,0,$1041,0)|0);
        $1042 = HEAP32[$v>>2]|0;
        $1043 = HEAP32[$zOnUpdate>>2]|0;
        (_sqlite3VdbeAddOp4($1042,97,0,6,0,$1043,0)|0);
        $1044 = HEAP32[$v>>2]|0;
        $1045 = HEAP32[$zOnDelete>>2]|0;
        (_sqlite3VdbeAddOp4($1044,97,0,7,0,$1045,0)|0);
        $1046 = HEAP32[$v>>2]|0;
        (_sqlite3VdbeAddOp4($1046,97,0,8,0,21520,0)|0);
        $1047 = HEAP32[$v>>2]|0;
        (_sqlite3VdbeAddOp2($1047,35,1,8)|0);
        $1048 = HEAP32[$j>>2]|0;
        $1049 = (($1048) + 1)|0;
        HEAP32[$j>>2] = $1049;
       }
       $1050 = HEAP32[$i22>>2]|0;
       $1051 = (($1050) + 1)|0;
       HEAP32[$i22>>2] = $1051;
       $1052 = HEAP32[$pFK>>2]|0;
       $1053 = (($1052) + 4|0);
       $1054 = HEAP32[$1053>>2]|0;
       HEAP32[$pFK>>2] = $1054;
      }
      break;
     }
     case 6:  {
      HEAP32[$i20>>2] = 0;
      $945 = HEAP32[$v>>2]|0;
      _sqlite3VdbeSetNumCols($945,2);
      $946 = HEAP32[$0>>2]|0;
      $947 = (($946) + 72|0);
      HEAP32[$947>>2] = 2;
      $948 = HEAP32[$v>>2]|0;
      (_sqlite3VdbeSetColName($948,0,0,21432,0)|0);
      $949 = HEAP32[$v>>2]|0;
      (_sqlite3VdbeSetColName($949,1,0,21352,0)|0);
      $950 = HEAP32[$db>>2]|0;
      $951 = (($950) + 428|0);
      $952 = (($951) + 8|0);
      $953 = HEAP32[$952>>2]|0;
      HEAP32[$p>>2] = $953;
      while(1) {
       $954 = HEAP32[$p>>2]|0;
       $955 = ($954|0)!=(0|0);
       if (!($955)) {
        break L21;
       }
       $956 = HEAP32[$p>>2]|0;
       $957 = (($956) + 8|0);
       $958 = HEAP32[$957>>2]|0;
       HEAP32[$pColl>>2] = $958;
       $959 = HEAP32[$v>>2]|0;
       $960 = HEAP32[$i20>>2]|0;
       $961 = (($960) + 1)|0;
       HEAP32[$i20>>2] = $961;
       (_sqlite3VdbeAddOp2($959,25,$960,1)|0);
       $962 = HEAP32[$v>>2]|0;
       $963 = HEAP32[$pColl>>2]|0;
       $964 = HEAP32[$963>>2]|0;
       (_sqlite3VdbeAddOp4($962,97,0,2,0,$964,0)|0);
       $965 = HEAP32[$v>>2]|0;
       (_sqlite3VdbeAddOp2($965,35,1,2)|0);
       $966 = HEAP32[$p>>2]|0;
       $967 = HEAP32[$966>>2]|0;
       HEAP32[$p>>2] = $967;
      }
      break;
     }
     case 28:  {
      $751 = HEAP32[$0>>2]|0;
      $752 = (_sqlite3GetVdbe($751)|0);
      HEAP32[$v>>2] = $752;
      $753 = HEAP32[$v>>2]|0;
      _sqlite3VdbeSetNumCols($753,4);
      $754 = HEAP32[$0>>2]|0;
      $755 = (($754) + 72|0);
      HEAP32[$755>>2] = 4;
      $756 = HEAP32[$0>>2]|0;
      $757 = HEAP32[$iDb>>2]|0;
      _sqlite3CodeVerifySchema($756,$757);
      $758 = HEAP32[$v>>2]|0;
      (_sqlite3VdbeSetColName($758,0,0,21400,0)|0);
      $759 = HEAP32[$v>>2]|0;
      (_sqlite3VdbeSetColName($759,1,0,10168,0)|0);
      $760 = HEAP32[$v>>2]|0;
      (_sqlite3VdbeSetColName($760,2,0,21408,0)|0);
      $761 = HEAP32[$v>>2]|0;
      (_sqlite3VdbeSetColName($761,3,0,21416,0)|0);
      $762 = HEAP32[$pDb>>2]|0;
      $763 = (($762) + 12|0);
      $764 = HEAP32[$763>>2]|0;
      $765 = (($764) + 8|0);
      $766 = (($765) + 8|0);
      $767 = HEAP32[$766>>2]|0;
      HEAP32[$i11>>2] = $767;
      while(1) {
       $768 = HEAP32[$i11>>2]|0;
       $769 = ($768|0)!=(0|0);
       if (!($769)) {
        break L21;
       }
       $770 = HEAP32[$i11>>2]|0;
       $771 = (($770) + 8|0);
       $772 = HEAP32[$771>>2]|0;
       HEAP32[$pTab12>>2] = $772;
       $773 = HEAP32[$v>>2]|0;
       $774 = HEAP32[$pTab12>>2]|0;
       $775 = HEAP32[$774>>2]|0;
       (_sqlite3VdbeAddOp4($773,97,0,1,0,$775,0)|0);
       $776 = HEAP32[$v>>2]|0;
       (_sqlite3VdbeAddOp2($776,28,0,2)|0);
       $777 = HEAP32[$v>>2]|0;
       $778 = HEAP32[$pTab12>>2]|0;
       $779 = (($778) + 42|0);
       $780 = HEAP16[$779>>1]|0;
       $781 = (_sqlite3LogEstToInt($780)|0);
       $782 = tempRet0;
       (_sqlite3VdbeAddOp2($777,25,$781,3)|0);
       $783 = HEAP32[$v>>2]|0;
       $784 = HEAP32[$pTab12>>2]|0;
       $785 = (($784) + 28|0);
       $786 = HEAP16[$785>>1]|0;
       $787 = (_sqlite3LogEstToInt($786)|0);
       $788 = tempRet0;
       (_sqlite3VdbeAddOp2($783,25,$787,4)|0);
       $789 = HEAP32[$v>>2]|0;
       (_sqlite3VdbeAddOp2($789,35,1,4)|0);
       $790 = HEAP32[$pTab12>>2]|0;
       $791 = (($790) + 8|0);
       $792 = HEAP32[$791>>2]|0;
       HEAP32[$pIdx>>2] = $792;
       while(1) {
        $793 = HEAP32[$pIdx>>2]|0;
        $794 = ($793|0)!=(0|0);
        if (!($794)) {
         break;
        }
        $795 = HEAP32[$v>>2]|0;
        $796 = HEAP32[$pIdx>>2]|0;
        $797 = HEAP32[$796>>2]|0;
        (_sqlite3VdbeAddOp4($795,97,0,2,0,$797,0)|0);
        $798 = HEAP32[$v>>2]|0;
        $799 = HEAP32[$pIdx>>2]|0;
        $800 = (($799) + 48|0);
        $801 = HEAP16[$800>>1]|0;
        $802 = (_sqlite3LogEstToInt($801)|0);
        $803 = tempRet0;
        (_sqlite3VdbeAddOp2($798,25,$802,3)|0);
        $804 = HEAP32[$v>>2]|0;
        $805 = HEAP32[$pIdx>>2]|0;
        $806 = (($805) + 8|0);
        $807 = HEAP32[$806>>2]|0;
        $808 = HEAP16[$807>>1]|0;
        $809 = (_sqlite3LogEstToInt($808)|0);
        $810 = tempRet0;
        (_sqlite3VdbeAddOp2($804,25,$809,4)|0);
        $811 = HEAP32[$v>>2]|0;
        (_sqlite3VdbeAddOp2($811,35,1,4)|0);
        $812 = HEAP32[$pIdx>>2]|0;
        $813 = (($812) + 20|0);
        $814 = HEAP32[$813>>2]|0;
        HEAP32[$pIdx>>2] = $814;
       }
       $815 = HEAP32[$i11>>2]|0;
       $816 = HEAP32[$815>>2]|0;
       HEAP32[$i11>>2] = $816;
      }
      break;
     }
     case 4:  {
      $484 = HEAP32[$zRight>>2]|0;
      $485 = ($484|0)!=(0|0);
      if ($485) {
       $494 = HEAP32[$zRight>>2]|0;
       $495 = (_sqlite3Atoi($494)|0);
       HEAP32[$size10>>2] = $495;
       $496 = HEAP32[$size10>>2]|0;
       $497 = HEAP32[$pDb>>2]|0;
       $498 = (($497) + 12|0);
       $499 = HEAP32[$498>>2]|0;
       $500 = (($499) + 80|0);
       HEAP32[$500>>2] = $496;
       $501 = HEAP32[$pDb>>2]|0;
       $502 = (($501) + 4|0);
       $503 = HEAP32[$502>>2]|0;
       $504 = HEAP32[$pDb>>2]|0;
       $505 = (($504) + 12|0);
       $506 = HEAP32[$505>>2]|0;
       $507 = (($506) + 80|0);
       $508 = HEAP32[$507>>2]|0;
       (_sqlite3BtreeSetCacheSize($503,$508)|0);
       break L21;
      } else {
       $486 = HEAP32[$0>>2]|0;
       $487 = HEAP32[$pDb>>2]|0;
       $488 = (($487) + 12|0);
       $489 = HEAP32[$488>>2]|0;
       $490 = (($489) + 80|0);
       $491 = HEAP32[$490>>2]|0;
       $492 = ($491|0)<(0);
       $493 = $492 << 31 >> 31;
       _returnSingleInt($486,21016,$491,$493);
       break L21;
      }
      break;
     }
     case 25:  {
      $202 = HEAP32[$pDb>>2]|0;
      $203 = (($202) + 4|0);
      $204 = HEAP32[$203>>2]|0;
      HEAP32[$pBt2>>2] = $204;
      HEAP32[$b>>2] = -1;
      $205 = HEAP32[$zRight>>2]|0;
      $206 = ($205|0)!=(0|0);
      if ($206) {
       $207 = HEAP32[$zRight>>2]|0;
       $208 = (_sqlite3GetBoolean($207,0)|0);
       $209 = $208&255;
       HEAP32[$b>>2] = $209;
      }
      $210 = HEAP32[$2>>2]|0;
      $211 = (($210) + 4|0);
      $212 = HEAP32[$211>>2]|0;
      $213 = ($212|0)==(0);
      $214 = HEAP32[$b>>2]|0;
      $215 = ($214|0)>=(0);
      $or$cond = $213 & $215;
      L200: do {
       if ($or$cond) {
        HEAP32[$ii>>2] = 0;
        while(1) {
         $216 = HEAP32[$ii>>2]|0;
         $217 = HEAP32[$db>>2]|0;
         $218 = (($217) + 20|0);
         $219 = HEAP32[$218>>2]|0;
         $220 = ($216|0)<($219|0);
         if (!($220)) {
          break L200;
         }
         $221 = HEAP32[$ii>>2]|0;
         $222 = HEAP32[$db>>2]|0;
         $223 = (($222) + 16|0);
         $224 = HEAP32[$223>>2]|0;
         $225 = (($224) + ($221<<4)|0);
         $226 = (($225) + 4|0);
         $227 = HEAP32[$226>>2]|0;
         $228 = HEAP32[$b>>2]|0;
         (_sqlite3BtreeSecureDelete($227,$228)|0);
         $229 = HEAP32[$ii>>2]|0;
         $230 = (($229) + 1)|0;
         HEAP32[$ii>>2] = $230;
        }
       }
      } while(0);
      $231 = HEAP32[$pBt2>>2]|0;
      $232 = HEAP32[$b>>2]|0;
      $233 = (_sqlite3BtreeSecureDelete($231,$232)|0);
      HEAP32[$b>>2] = $233;
      $234 = HEAP32[$0>>2]|0;
      $235 = HEAP32[$b>>2]|0;
      $236 = ($235|0)<(0);
      $237 = $236 << 31 >> 31;
      _returnSingleInt($234,21048,$235,$237);
      break L21;
      break;
     }
     case 12:  {
      $1055 = HEAP32[$0>>2]|0;
      $1056 = (($1055) + 72|0);
      $1057 = HEAP32[$1056>>2]|0;
      $1058 = (($1057) + 1)|0;
      HEAP32[$regResult>>2] = $1058;
      $1059 = HEAP32[$0>>2]|0;
      $1060 = (($1059) + 72|0);
      $1061 = HEAP32[$1060>>2]|0;
      $1062 = (($1061) + 4)|0;
      HEAP32[$1060>>2] = $1062;
      $1063 = HEAP32[$0>>2]|0;
      $1064 = (($1063) + 72|0);
      $1065 = HEAP32[$1064>>2]|0;
      $1066 = (($1065) + 1)|0;
      HEAP32[$1064>>2] = $1066;
      HEAP32[$regKey>>2] = $1066;
      $1067 = HEAP32[$0>>2]|0;
      $1068 = (($1067) + 72|0);
      $1069 = HEAP32[$1068>>2]|0;
      $1070 = (($1069) + 1)|0;
      HEAP32[$1068>>2] = $1070;
      HEAP32[$regRow>>2] = $1070;
      $1071 = HEAP32[$0>>2]|0;
      $1072 = (_sqlite3GetVdbe($1071)|0);
      HEAP32[$v>>2] = $1072;
      $1073 = HEAP32[$v>>2]|0;
      _sqlite3VdbeSetNumCols($1073,4);
      $1074 = HEAP32[$v>>2]|0;
      (_sqlite3VdbeSetColName($1074,0,0,21400,0)|0);
      $1075 = HEAP32[$v>>2]|0;
      (_sqlite3VdbeSetColName($1075,1,0,9992,0)|0);
      $1076 = HEAP32[$v>>2]|0;
      (_sqlite3VdbeSetColName($1076,2,0,21528,0)|0);
      $1077 = HEAP32[$v>>2]|0;
      (_sqlite3VdbeSetColName($1077,3,0,21536,0)|0);
      $1078 = HEAP32[$0>>2]|0;
      $1079 = HEAP32[$iDb>>2]|0;
      _sqlite3CodeVerifySchema($1078,$1079);
      $1080 = HEAP32[$iDb>>2]|0;
      $1081 = HEAP32[$db>>2]|0;
      $1082 = (($1081) + 16|0);
      $1083 = HEAP32[$1082>>2]|0;
      $1084 = (($1083) + ($1080<<4)|0);
      $1085 = (($1084) + 12|0);
      $1086 = HEAP32[$1085>>2]|0;
      $1087 = (($1086) + 8|0);
      $1088 = (($1087) + 8|0);
      $1089 = HEAP32[$1088>>2]|0;
      HEAP32[$k28>>2] = $1089;
      while(1) {
       $1090 = HEAP32[$k28>>2]|0;
       $1091 = ($1090|0)!=(0|0);
       if (!($1091)) {
        break L21;
       }
       $1092 = HEAP32[$zRight>>2]|0;
       $1093 = ($1092|0)!=(0|0);
       if ($1093) {
        $1094 = HEAP32[$0>>2]|0;
        $1095 = HEAP32[$zRight>>2]|0;
        $1096 = HEAP32[$zDb>>2]|0;
        $1097 = (_sqlite3LocateTable($1094,0,$1095,$1096)|0);
        HEAP32[$pTab24>>2] = $1097;
        HEAP32[$k28>>2] = 0;
       } else {
        $1098 = HEAP32[$k28>>2]|0;
        $1099 = (($1098) + 8|0);
        $1100 = HEAP32[$1099>>2]|0;
        HEAP32[$pTab24>>2] = $1100;
        $1101 = HEAP32[$k28>>2]|0;
        $1102 = HEAP32[$1101>>2]|0;
        HEAP32[$k28>>2] = $1102;
       }
       $1103 = HEAP32[$pTab24>>2]|0;
       $1104 = ($1103|0)==(0|0);
       if ($1104) {
        continue;
       }
       $1105 = HEAP32[$pTab24>>2]|0;
       $1106 = (($1105) + 16|0);
       $1107 = HEAP32[$1106>>2]|0;
       $1108 = ($1107|0)==(0|0);
       if ($1108) {
        continue;
       }
       $1109 = HEAP32[$0>>2]|0;
       $1110 = HEAP32[$iDb>>2]|0;
       $1111 = HEAP32[$pTab24>>2]|0;
       $1112 = (($1111) + 32|0);
       $1113 = HEAP32[$1112>>2]|0;
       $1114 = HEAP32[$pTab24>>2]|0;
       $1115 = HEAP32[$1114>>2]|0;
       _sqlite3TableLock($1109,$1110,$1113,0,$1115);
       $1116 = HEAP32[$pTab24>>2]|0;
       $1117 = (($1116) + 38|0);
       $1118 = HEAP16[$1117>>1]|0;
       $1119 = $1118 << 16 >> 16;
       $1120 = HEAP32[$regRow>>2]|0;
       $1121 = (($1119) + ($1120))|0;
       $1122 = HEAP32[$0>>2]|0;
       $1123 = (($1122) + 72|0);
       $1124 = HEAP32[$1123>>2]|0;
       $1125 = ($1121|0)>($1124|0);
       if ($1125) {
        $1126 = HEAP32[$pTab24>>2]|0;
        $1127 = (($1126) + 38|0);
        $1128 = HEAP16[$1127>>1]|0;
        $1129 = $1128 << 16 >> 16;
        $1130 = HEAP32[$regRow>>2]|0;
        $1131 = (($1129) + ($1130))|0;
        $1132 = HEAP32[$0>>2]|0;
        $1133 = (($1132) + 72|0);
        HEAP32[$1133>>2] = $1131;
       }
       $1134 = HEAP32[$0>>2]|0;
       $1135 = HEAP32[$iDb>>2]|0;
       $1136 = HEAP32[$pTab24>>2]|0;
       _sqlite3OpenTable($1134,0,$1135,$1136,54);
       $1137 = HEAP32[$v>>2]|0;
       $1138 = HEAP32[$regResult>>2]|0;
       $1139 = HEAP32[$pTab24>>2]|0;
       $1140 = HEAP32[$1139>>2]|0;
       (_sqlite3VdbeAddOp4($1137,97,0,$1138,0,$1140,0)|0);
       HEAP32[$i26>>2] = 1;
       $1141 = HEAP32[$pTab24>>2]|0;
       $1142 = (($1141) + 16|0);
       $1143 = HEAP32[$1142>>2]|0;
       HEAP32[$pFK23>>2] = $1143;
       L219: while(1) {
        $1144 = HEAP32[$pFK23>>2]|0;
        $1145 = ($1144|0)!=(0|0);
        if (!($1145)) {
         break;
        }
        $1146 = HEAP32[$db>>2]|0;
        $1147 = HEAP32[$pFK23>>2]|0;
        $1148 = (($1147) + 8|0);
        $1149 = HEAP32[$1148>>2]|0;
        $1150 = HEAP32[$zDb>>2]|0;
        $1151 = (_sqlite3FindTable($1146,$1149,$1150)|0);
        HEAP32[$pParent>>2] = $1151;
        $1152 = HEAP32[$pParent>>2]|0;
        $1153 = ($1152|0)==(0|0);
        do {
         if (!($1153)) {
          HEAP32[$pIdx25>>2] = 0;
          $1154 = HEAP32[$0>>2]|0;
          $1155 = HEAP32[$iDb>>2]|0;
          $1156 = HEAP32[$pParent>>2]|0;
          $1157 = (($1156) + 32|0);
          $1158 = HEAP32[$1157>>2]|0;
          $1159 = HEAP32[$pParent>>2]|0;
          $1160 = HEAP32[$1159>>2]|0;
          _sqlite3TableLock($1154,$1155,$1158,0,$1160);
          $1161 = HEAP32[$0>>2]|0;
          $1162 = HEAP32[$pParent>>2]|0;
          $1163 = HEAP32[$pFK23>>2]|0;
          $1164 = (_sqlite3FkLocateIndex($1161,$1162,$1163,$pIdx25,0)|0);
          HEAP32[$x>>2] = $1164;
          $1165 = HEAP32[$x>>2]|0;
          $1166 = ($1165|0)==(0);
          if (!($1166)) {
           label = 195;
           break L219;
          }
          $1167 = HEAP32[$pIdx25>>2]|0;
          $1168 = ($1167|0)==(0|0);
          if ($1168) {
           $1169 = HEAP32[$0>>2]|0;
           $1170 = HEAP32[$i26>>2]|0;
           $1171 = HEAP32[$iDb>>2]|0;
           $1172 = HEAP32[$pParent>>2]|0;
           _sqlite3OpenTable($1169,$1170,$1171,$1172,54);
           break;
          } else {
           $1173 = HEAP32[$v>>2]|0;
           $1174 = HEAP32[$i26>>2]|0;
           $1175 = HEAP32[$pIdx25>>2]|0;
           $1176 = (($1175) + 44|0);
           $1177 = HEAP32[$1176>>2]|0;
           $1178 = HEAP32[$iDb>>2]|0;
           (_sqlite3VdbeAddOp3($1173,54,$1174,$1177,$1178)|0);
           $1179 = HEAP32[$0>>2]|0;
           $1180 = HEAP32[$pIdx25>>2]|0;
           _sqlite3VdbeSetP4KeyInfo($1179,$1180);
           break;
          }
         }
        } while(0);
        $1181 = HEAP32[$i26>>2]|0;
        $1182 = (($1181) + 1)|0;
        HEAP32[$i26>>2] = $1182;
        $1183 = HEAP32[$pFK23>>2]|0;
        $1184 = (($1183) + 4|0);
        $1185 = HEAP32[$1184>>2]|0;
        HEAP32[$pFK23>>2] = $1185;
       }
       if ((label|0) == 195) {
        label = 0;
        HEAP32[$k28>>2] = 0;
       }
       $1186 = HEAP32[$pFK23>>2]|0;
       $1187 = ($1186|0)!=(0|0);
       if ($1187) {
        break L21;
       }
       $1188 = HEAP32[$0>>2]|0;
       $1189 = (($1188) + 68|0);
       $1190 = HEAP32[$1189>>2]|0;
       $1191 = HEAP32[$i26>>2]|0;
       $1192 = ($1190|0)<($1191|0);
       if ($1192) {
        $1193 = HEAP32[$i26>>2]|0;
        $1194 = HEAP32[$0>>2]|0;
        $1195 = (($1194) + 68|0);
        HEAP32[$1195>>2] = $1193;
       }
       $1196 = HEAP32[$v>>2]|0;
       $1197 = (_sqlite3VdbeAddOp1($1196,108,0)|0);
       HEAP32[$addrTop>>2] = $1197;
       HEAP32[$i26>>2] = 1;
       $1198 = HEAP32[$pTab24>>2]|0;
       $1199 = (($1198) + 16|0);
       $1200 = HEAP32[$1199>>2]|0;
       HEAP32[$pFK23>>2] = $1200;
       while(1) {
        $1201 = HEAP32[$pFK23>>2]|0;
        $1202 = ($1201|0)!=(0|0);
        if (!($1202)) {
         break;
        }
        $1203 = HEAP32[$db>>2]|0;
        $1204 = HEAP32[$pFK23>>2]|0;
        $1205 = (($1204) + 8|0);
        $1206 = HEAP32[$1205>>2]|0;
        $1207 = HEAP32[$zDb>>2]|0;
        $1208 = (_sqlite3FindTable($1203,$1206,$1207)|0);
        HEAP32[$pParent>>2] = $1208;
        HEAP32[$pIdx25>>2] = 0;
        HEAP32[$aiCols>>2] = 0;
        $1209 = HEAP32[$pParent>>2]|0;
        $1210 = ($1209|0)!=(0|0);
        if ($1210) {
         $1211 = HEAP32[$0>>2]|0;
         $1212 = HEAP32[$pParent>>2]|0;
         $1213 = HEAP32[$pFK23>>2]|0;
         $1214 = (_sqlite3FkLocateIndex($1211,$1212,$1213,$pIdx25,$aiCols)|0);
         HEAP32[$x>>2] = $1214;
        }
        $1215 = HEAP32[$v>>2]|0;
        $1216 = (_sqlite3VdbeMakeLabel($1215)|0);
        HEAP32[$addrOk>>2] = $1216;
        $1217 = HEAP32[$pParent>>2]|0;
        $1218 = ($1217|0)!=(0|0);
        $1219 = HEAP32[$pIdx25>>2]|0;
        $1220 = ($1219|0)==(0|0);
        $or$cond11 = $1218 & $1220;
        do {
         if ($or$cond11) {
          $1221 = HEAP32[$pFK23>>2]|0;
          $1222 = (($1221) + 36|0);
          $1223 = HEAP32[$1222>>2]|0;
          HEAP32[$iKey>>2] = $1223;
          $1224 = HEAP32[$iKey>>2]|0;
          $1225 = HEAP32[$pTab24>>2]|0;
          $1226 = (($1225) + 36|0);
          $1227 = HEAP16[$1226>>1]|0;
          $1228 = $1227 << 16 >> 16;
          $1229 = ($1224|0)!=($1228|0);
          $1230 = HEAP32[$v>>2]|0;
          if ($1229) {
           $1231 = HEAP32[$iKey>>2]|0;
           $1232 = HEAP32[$regRow>>2]|0;
           (_sqlite3VdbeAddOp3($1230,47,0,$1231,$1232)|0);
           $1233 = HEAP32[$v>>2]|0;
           $1234 = HEAP32[$pTab24>>2]|0;
           $1235 = HEAP32[$iKey>>2]|0;
           $1236 = HEAP32[$regRow>>2]|0;
           _sqlite3ColumnDefault($1233,$1234,$1235,$1236);
           $1237 = HEAP32[$v>>2]|0;
           $1238 = HEAP32[$regRow>>2]|0;
           $1239 = HEAP32[$addrOk>>2]|0;
           (_sqlite3VdbeAddOp2($1237,76,$1238,$1239)|0);
           $1240 = HEAP32[$v>>2]|0;
           $1241 = HEAP32[$regRow>>2]|0;
           $1242 = HEAP32[$v>>2]|0;
           $1243 = (_sqlite3VdbeCurrentAddr($1242)|0);
           $1244 = (($1243) + 3)|0;
           (_sqlite3VdbeAddOp2($1240,38,$1241,$1244)|0);
          } else {
           $1245 = HEAP32[$regRow>>2]|0;
           (_sqlite3VdbeAddOp2($1230,103,0,$1245)|0);
          }
          $1246 = HEAP32[$v>>2]|0;
          $1247 = HEAP32[$i26>>2]|0;
          $1248 = HEAP32[$regRow>>2]|0;
          (_sqlite3VdbeAddOp3($1246,70,$1247,0,$1248)|0);
          $1249 = HEAP32[$v>>2]|0;
          $1250 = HEAP32[$addrOk>>2]|0;
          (_sqlite3VdbeAddOp2($1249,16,0,$1250)|0);
          $1251 = HEAP32[$v>>2]|0;
          $1252 = HEAP32[$v>>2]|0;
          $1253 = (_sqlite3VdbeCurrentAddr($1252)|0);
          $1254 = (($1253) - 2)|0;
          _sqlite3VdbeJumpHere($1251,$1254);
         } else {
          HEAP32[$j27>>2] = 0;
          while(1) {
           $1255 = HEAP32[$j27>>2]|0;
           $1256 = HEAP32[$pFK23>>2]|0;
           $1257 = (($1256) + 20|0);
           $1258 = HEAP32[$1257>>2]|0;
           $1259 = ($1255|0)<($1258|0);
           if (!($1259)) {
            break;
           }
           $1260 = HEAP32[$v>>2]|0;
           $1261 = HEAP32[$pTab24>>2]|0;
           $1262 = HEAP32[$aiCols>>2]|0;
           $1263 = ($1262|0)!=(0|0);
           $1264 = HEAP32[$j27>>2]|0;
           if ($1263) {
            $1265 = HEAP32[$aiCols>>2]|0;
            $1266 = (($1265) + ($1264<<2)|0);
            $1267 = HEAP32[$1266>>2]|0;
            $1275 = $1267;
           } else {
            $1268 = HEAP32[$pFK23>>2]|0;
            $1269 = (($1268) + 36|0);
            $1270 = (($1269) + ($1264<<3)|0);
            $1271 = HEAP32[$1270>>2]|0;
            $1275 = $1271;
           }
           $1272 = HEAP32[$regRow>>2]|0;
           $1273 = HEAP32[$j27>>2]|0;
           $1274 = (($1272) + ($1273))|0;
           _sqlite3ExprCodeGetColumnOfTable($1260,$1261,0,$1275,$1274);
           $1276 = HEAP32[$v>>2]|0;
           $1277 = HEAP32[$regRow>>2]|0;
           $1278 = HEAP32[$j27>>2]|0;
           $1279 = (($1277) + ($1278))|0;
           $1280 = HEAP32[$addrOk>>2]|0;
           (_sqlite3VdbeAddOp2($1276,76,$1279,$1280)|0);
           $1281 = HEAP32[$j27>>2]|0;
           $1282 = (($1281) + 1)|0;
           HEAP32[$j27>>2] = $1282;
          }
          $1283 = HEAP32[$pParent>>2]|0;
          $1284 = ($1283|0)!=(0|0);
          if (!($1284)) {
           break;
          }
          $1285 = HEAP32[$v>>2]|0;
          $1286 = HEAP32[$regRow>>2]|0;
          $1287 = HEAP32[$pFK23>>2]|0;
          $1288 = (($1287) + 20|0);
          $1289 = HEAP32[$1288>>2]|0;
          $1290 = HEAP32[$regKey>>2]|0;
          $1291 = HEAP32[$v>>2]|0;
          $1292 = HEAP32[$pIdx25>>2]|0;
          $1293 = (_sqlite3IndexAffinityStr($1291,$1292)|0);
          $1294 = HEAP32[$pFK23>>2]|0;
          $1295 = (($1294) + 20|0);
          $1296 = HEAP32[$1295>>2]|0;
          (_sqlite3VdbeAddOp4($1285,49,$1286,$1289,$1290,$1293,$1296)|0);
          $1297 = HEAP32[$v>>2]|0;
          $1298 = HEAP32[$i26>>2]|0;
          $1299 = HEAP32[$addrOk>>2]|0;
          $1300 = HEAP32[$regKey>>2]|0;
          (_sqlite3VdbeAddOp4Int($1297,69,$1298,$1299,$1300,0)|0);
         }
        } while(0);
        $1301 = HEAP32[$v>>2]|0;
        $1302 = HEAP32[$regResult>>2]|0;
        $1303 = (($1302) + 1)|0;
        (_sqlite3VdbeAddOp2($1301,103,0,$1303)|0);
        $1304 = HEAP32[$v>>2]|0;
        $1305 = HEAP32[$regResult>>2]|0;
        $1306 = (($1305) + 2)|0;
        $1307 = HEAP32[$pFK23>>2]|0;
        $1308 = (($1307) + 8|0);
        $1309 = HEAP32[$1308>>2]|0;
        (_sqlite3VdbeAddOp4($1304,97,0,$1306,0,$1309,0)|0);
        $1310 = HEAP32[$v>>2]|0;
        $1311 = HEAP32[$i26>>2]|0;
        $1312 = (($1311) - 1)|0;
        $1313 = HEAP32[$regResult>>2]|0;
        $1314 = (($1313) + 3)|0;
        (_sqlite3VdbeAddOp2($1310,25,$1312,$1314)|0);
        $1315 = HEAP32[$v>>2]|0;
        $1316 = HEAP32[$regResult>>2]|0;
        (_sqlite3VdbeAddOp2($1315,35,$1316,4)|0);
        $1317 = HEAP32[$v>>2]|0;
        $1318 = HEAP32[$addrOk>>2]|0;
        _sqlite3VdbeResolveLabel($1317,$1318);
        $1319 = HEAP32[$db>>2]|0;
        $1320 = HEAP32[$aiCols>>2]|0;
        _sqlite3DbFree($1319,$1320);
        $1321 = HEAP32[$i26>>2]|0;
        $1322 = (($1321) + 1)|0;
        HEAP32[$i26>>2] = $1322;
        $1323 = HEAP32[$pFK23>>2]|0;
        $1324 = (($1323) + 4|0);
        $1325 = HEAP32[$1324>>2]|0;
        HEAP32[$pFK23>>2] = $1325;
       }
       $1326 = HEAP32[$v>>2]|0;
       $1327 = HEAP32[$addrTop>>2]|0;
       $1328 = (($1327) + 1)|0;
       (_sqlite3VdbeAddOp2($1326,9,0,$1328)|0);
       $1329 = HEAP32[$v>>2]|0;
       $1330 = HEAP32[$addrTop>>2]|0;
       _sqlite3VdbeJumpHere($1329,$1330);
      }
      break;
     }
     case 17:  {
      $1337 = HEAP32[$zLeft>>2]|0;
      $1338 = HEAP8[$1337>>0]|0;
      $1339 = $1338&255;
      $1340 = (440 + ($1339)|0);
      $1341 = HEAP8[$1340>>0]|0;
      $1342 = $1341&255;
      $1343 = ($1342|0)==(113);
      $1344 = $1343&1;
      HEAP32[$isQuick>>2] = $1344;
      $1345 = HEAP32[$2>>2]|0;
      $1346 = HEAP32[$1345>>2]|0;
      $1347 = ($1346|0)==(0|0);
      if ($1347) {
       HEAP32[$iDb>>2] = -1;
      }
      $1348 = HEAP32[$0>>2]|0;
      $1349 = (($1348) + 72|0);
      HEAP32[$1349>>2] = 6;
      $1350 = HEAP32[$v>>2]|0;
      _sqlite3VdbeSetNumCols($1350,1);
      $1351 = HEAP32[$v>>2]|0;
      (_sqlite3VdbeSetColName($1351,0,0,21560,0)|0);
      HEAP32[$mxErr>>2] = 100;
      $1352 = HEAP32[$zRight>>2]|0;
      $1353 = ($1352|0)!=(0|0);
      if ($1353) {
       $1354 = HEAP32[$zRight>>2]|0;
       (_sqlite3GetInt32($1354,$mxErr)|0);
       $1355 = HEAP32[$mxErr>>2]|0;
       $1356 = ($1355|0)<=(0);
       if ($1356) {
        HEAP32[$mxErr>>2] = 100;
       }
      }
      $1357 = HEAP32[$v>>2]|0;
      $1358 = HEAP32[$mxErr>>2]|0;
      (_sqlite3VdbeAddOp2($1357,25,$1358,1)|0);
      HEAP32[$i29>>2] = 0;
      while(1) {
       $1359 = HEAP32[$i29>>2]|0;
       $1360 = HEAP32[$db>>2]|0;
       $1361 = (($1360) + 20|0);
       $1362 = HEAP32[$1361>>2]|0;
       $1363 = ($1359|0)<($1362|0);
       if (!($1363)) {
        break;
       }
       HEAP32[$cnt>>2] = 0;
       $1364 = HEAP32[$iDb>>2]|0;
       $1365 = ($1364|0)>=(0);
       if ($1365) {
        $1366 = HEAP32[$i29>>2]|0;
        $1367 = HEAP32[$iDb>>2]|0;
        $1368 = ($1366|0)!=($1367|0);
        if (!($1368)) {
         label = 230;
        }
       } else {
        label = 230;
       }
       L273: do {
        if ((label|0) == 230) {
         label = 0;
         $1369 = HEAP32[$0>>2]|0;
         $1370 = HEAP32[$i29>>2]|0;
         _sqlite3CodeVerifySchema($1369,$1370);
         $1371 = HEAP32[$v>>2]|0;
         $1372 = (_sqlite3VdbeAddOp1($1371,137,1)|0);
         HEAP32[$addr31>>2] = $1372;
         $1373 = HEAP32[$v>>2]|0;
         (_sqlite3VdbeAddOp2($1373,24,0,0)|0);
         $1374 = HEAP32[$v>>2]|0;
         $1375 = HEAP32[$addr31>>2]|0;
         _sqlite3VdbeJumpHere($1374,$1375);
         $1376 = HEAP32[$i29>>2]|0;
         $1377 = HEAP32[$db>>2]|0;
         $1378 = (($1377) + 16|0);
         $1379 = HEAP32[$1378>>2]|0;
         $1380 = (($1379) + ($1376<<4)|0);
         $1381 = (($1380) + 12|0);
         $1382 = HEAP32[$1381>>2]|0;
         $1383 = (($1382) + 8|0);
         HEAP32[$pTbls>>2] = $1383;
         $1384 = HEAP32[$pTbls>>2]|0;
         $1385 = (($1384) + 8|0);
         $1386 = HEAP32[$1385>>2]|0;
         HEAP32[$x32>>2] = $1386;
         while(1) {
          $1387 = HEAP32[$x32>>2]|0;
          $1388 = ($1387|0)!=(0|0);
          if (!($1388)) {
           break;
          }
          $1389 = HEAP32[$x32>>2]|0;
          $1390 = (($1389) + 8|0);
          $1391 = HEAP32[$1390>>2]|0;
          HEAP32[$pTab33>>2] = $1391;
          $1392 = HEAP32[$pTab33>>2]|0;
          $1393 = (($1392) + 44|0);
          $1394 = HEAP8[$1393>>0]|0;
          $1395 = $1394&255;
          $1396 = $1395 & 32;
          $1397 = ($1396|0)==(0);
          if ($1397) {
           $1398 = HEAP32[$v>>2]|0;
           $1399 = HEAP32[$pTab33>>2]|0;
           $1400 = (($1399) + 32|0);
           $1401 = HEAP32[$1400>>2]|0;
           $1402 = HEAP32[$cnt>>2]|0;
           $1403 = (2 + ($1402))|0;
           (_sqlite3VdbeAddOp2($1398,25,$1401,$1403)|0);
           $1404 = HEAP32[$cnt>>2]|0;
           $1405 = (($1404) + 1)|0;
           HEAP32[$cnt>>2] = $1405;
          }
          $1406 = HEAP32[$pTab33>>2]|0;
          $1407 = (($1406) + 8|0);
          $1408 = HEAP32[$1407>>2]|0;
          HEAP32[$pIdx34>>2] = $1408;
          while(1) {
           $1409 = HEAP32[$pIdx34>>2]|0;
           $1410 = ($1409|0)!=(0|0);
           if (!($1410)) {
            break;
           }
           $1411 = HEAP32[$v>>2]|0;
           $1412 = HEAP32[$pIdx34>>2]|0;
           $1413 = (($1412) + 44|0);
           $1414 = HEAP32[$1413>>2]|0;
           $1415 = HEAP32[$cnt>>2]|0;
           $1416 = (2 + ($1415))|0;
           (_sqlite3VdbeAddOp2($1411,25,$1414,$1416)|0);
           $1417 = HEAP32[$cnt>>2]|0;
           $1418 = (($1417) + 1)|0;
           HEAP32[$cnt>>2] = $1418;
           $1419 = HEAP32[$pIdx34>>2]|0;
           $1420 = (($1419) + 20|0);
           $1421 = HEAP32[$1420>>2]|0;
           HEAP32[$pIdx34>>2] = $1421;
          }
          $1422 = HEAP32[$x32>>2]|0;
          $1423 = HEAP32[$1422>>2]|0;
          HEAP32[$x32>>2] = $1423;
         }
         $1424 = HEAP32[$0>>2]|0;
         $1425 = (($1424) + 72|0);
         $1426 = HEAP32[$1425>>2]|0;
         $1427 = HEAP32[$cnt>>2]|0;
         $1428 = (($1427) + 8)|0;
         $1429 = ($1426|0)>($1428|0);
         if ($1429) {
          $1430 = HEAP32[$0>>2]|0;
          $1431 = (($1430) + 72|0);
          $1432 = HEAP32[$1431>>2]|0;
          $1437 = $1432;
         } else {
          $1433 = HEAP32[$cnt>>2]|0;
          $1434 = (($1433) + 8)|0;
          $1437 = $1434;
         }
         $1435 = HEAP32[$0>>2]|0;
         $1436 = (($1435) + 72|0);
         HEAP32[$1436>>2] = $1437;
         $1438 = HEAP32[$v>>2]|0;
         $1439 = HEAP32[$cnt>>2]|0;
         (_sqlite3VdbeAddOp3($1438,127,2,$1439,1)|0);
         $1440 = HEAP32[$v>>2]|0;
         $1441 = HEAP32[$i29>>2]|0;
         $1442 = $1441&255;
         _sqlite3VdbeChangeP5($1440,$1442);
         $1443 = HEAP32[$v>>2]|0;
         $1444 = (_sqlite3VdbeAddOp1($1443,76,2)|0);
         HEAP32[$addr31>>2] = $1444;
         $1445 = HEAP32[$v>>2]|0;
         $1446 = HEAP32[$db>>2]|0;
         $1447 = HEAP32[$i29>>2]|0;
         $1448 = HEAP32[$db>>2]|0;
         $1449 = (($1448) + 16|0);
         $1450 = HEAP32[$1449>>2]|0;
         $1451 = (($1450) + ($1447<<4)|0);
         $1452 = HEAP32[$1451>>2]|0;
         HEAP32[$vararg_buffer35>>2] = $1452;
         $1453 = (_sqlite3MPrintf($1446,21576,$vararg_buffer35)|0);
         (_sqlite3VdbeAddOp4($1445,97,0,3,0,$1453,-1)|0);
         $1454 = HEAP32[$v>>2]|0;
         (_sqlite3VdbeAddOp3($1454,32,2,4,1)|0);
         $1455 = HEAP32[$v>>2]|0;
         (_sqlite3VdbeAddOp3($1455,94,4,3,2)|0);
         $1456 = HEAP32[$v>>2]|0;
         (_sqlite3VdbeAddOp2($1456,35,2,1)|0);
         $1457 = HEAP32[$v>>2]|0;
         $1458 = HEAP32[$addr31>>2]|0;
         _sqlite3VdbeJumpHere($1457,$1458);
         $1459 = HEAP32[$pTbls>>2]|0;
         $1460 = (($1459) + 8|0);
         $1461 = HEAP32[$1460>>2]|0;
         HEAP32[$x32>>2] = $1461;
         while(1) {
          $1462 = HEAP32[$x32>>2]|0;
          $1463 = ($1462|0)==(0|0);
          $1464 = HEAP32[$isQuick>>2]|0;
          $1465 = ($1464|0)!=(0);
          $or$cond13 = $1463 | $1465;
          if ($or$cond13) {
           break L273;
          }
          $1466 = HEAP32[$x32>>2]|0;
          $1467 = (($1466) + 8|0);
          $1468 = HEAP32[$1467>>2]|0;
          HEAP32[$pTab35>>2] = $1468;
          HEAP32[$pPrior>>2] = 0;
          HEAP32[$r1>>2] = -1;
          $1469 = HEAP32[$pTab35>>2]|0;
          $1470 = (($1469) + 8|0);
          $1471 = HEAP32[$1470>>2]|0;
          $1472 = ($1471|0)==(0|0);
          L293: do {
           if (!($1472)) {
            $1473 = HEAP32[$pTab35>>2]|0;
            $1474 = (($1473) + 44|0);
            $1475 = HEAP8[$1474>>0]|0;
            $1476 = $1475&255;
            $1477 = $1476 & 32;
            $1478 = ($1477|0)==(0);
            if ($1478) {
             $1481 = 0;
            } else {
             $1479 = HEAP32[$pTab35>>2]|0;
             $1480 = (_sqlite3PrimaryKeyIndex($1479)|0);
             $1481 = $1480;
            }
            HEAP32[$pPk37>>2] = $1481;
            $1482 = HEAP32[$v>>2]|0;
            $1483 = (_sqlite3VdbeAddOp1($1482,137,1)|0);
            HEAP32[$addr31>>2] = $1483;
            $1484 = HEAP32[$v>>2]|0;
            (_sqlite3VdbeAddOp2($1484,24,0,0)|0);
            $1485 = HEAP32[$v>>2]|0;
            $1486 = HEAP32[$addr31>>2]|0;
            _sqlite3VdbeJumpHere($1485,$1486);
            $1487 = HEAP32[$0>>2]|0;
            _sqlite3ExprCacheClear($1487);
            $1488 = HEAP32[$0>>2]|0;
            $1489 = HEAP32[$pTab35>>2]|0;
            (_sqlite3OpenTableAndIndices($1488,$1489,54,1,0,$iDataCur,$iIdxCur)|0);
            $1490 = HEAP32[$v>>2]|0;
            (_sqlite3VdbeAddOp2($1490,25,0,7)|0);
            HEAP32[$j30>>2] = 0;
            $1491 = HEAP32[$pTab35>>2]|0;
            $1492 = (($1491) + 8|0);
            $1493 = HEAP32[$1492>>2]|0;
            HEAP32[$pIdx36>>2] = $1493;
            while(1) {
             $1494 = HEAP32[$pIdx36>>2]|0;
             $1495 = ($1494|0)!=(0|0);
             if (!($1495)) {
              break;
             }
             $1496 = HEAP32[$v>>2]|0;
             $1497 = HEAP32[$j30>>2]|0;
             $1498 = (8 + ($1497))|0;
             (_sqlite3VdbeAddOp2($1496,25,0,$1498)|0);
             $1499 = HEAP32[$pIdx36>>2]|0;
             $1500 = (($1499) + 20|0);
             $1501 = HEAP32[$1500>>2]|0;
             HEAP32[$pIdx36>>2] = $1501;
             $1502 = HEAP32[$j30>>2]|0;
             $1503 = (($1502) + 1)|0;
             HEAP32[$j30>>2] = $1503;
            }
            $1504 = HEAP32[$0>>2]|0;
            $1505 = (($1504) + 72|0);
            $1506 = HEAP32[$1505>>2]|0;
            $1507 = HEAP32[$j30>>2]|0;
            $1508 = (8 + ($1507))|0;
            $1509 = ($1506|0)>($1508|0);
            if ($1509) {
             $1510 = HEAP32[$0>>2]|0;
             $1511 = (($1510) + 72|0);
             $1512 = HEAP32[$1511>>2]|0;
             $1517 = $1512;
            } else {
             $1513 = HEAP32[$j30>>2]|0;
             $1514 = (8 + ($1513))|0;
             $1517 = $1514;
            }
            $1515 = HEAP32[$0>>2]|0;
            $1516 = (($1515) + 72|0);
            HEAP32[$1516>>2] = $1517;
            $1518 = HEAP32[$v>>2]|0;
            $1519 = HEAP32[$iDataCur>>2]|0;
            (_sqlite3VdbeAddOp2($1518,108,$1519,0)|0);
            $1520 = HEAP32[$v>>2]|0;
            $1521 = (_sqlite3VdbeAddOp2($1520,37,7,1)|0);
            HEAP32[$loopTop>>2] = $1521;
            HEAP32[$j30>>2] = 0;
            while(1) {
             $1522 = HEAP32[$j30>>2]|0;
             $1523 = HEAP32[$pTab35>>2]|0;
             $1524 = (($1523) + 38|0);
             $1525 = HEAP16[$1524>>1]|0;
             $1526 = $1525 << 16 >> 16;
             $1527 = ($1522|0)<($1526|0);
             if (!($1527)) {
              break;
             }
             $1528 = HEAP32[$j30>>2]|0;
             $1529 = HEAP32[$pTab35>>2]|0;
             $1530 = (($1529) + 36|0);
             $1531 = HEAP16[$1530>>1]|0;
             $1532 = $1531 << 16 >> 16;
             $1533 = ($1528|0)==($1532|0);
             do {
              if (!($1533)) {
               $1534 = HEAP32[$j30>>2]|0;
               $1535 = HEAP32[$pTab35>>2]|0;
               $1536 = (($1535) + 4|0);
               $1537 = HEAP32[$1536>>2]|0;
               $1538 = (($1537) + (($1534*24)|0)|0);
               $1539 = (($1538) + 20|0);
               $1540 = HEAP8[$1539>>0]|0;
               $1541 = $1540&255;
               $1542 = ($1541|0)==(0);
               if ($1542) {
                break;
               }
               $1543 = HEAP32[$v>>2]|0;
               $1544 = HEAP32[$pTab35>>2]|0;
               $1545 = HEAP32[$iDataCur>>2]|0;
               $1546 = HEAP32[$j30>>2]|0;
               _sqlite3ExprCodeGetColumnOfTable($1543,$1544,$1545,$1546,3);
               $1547 = HEAP32[$v>>2]|0;
               _sqlite3VdbeChangeP5($1547,-128);
               $1548 = HEAP32[$v>>2]|0;
               $1549 = (_sqlite3VdbeAddOp1($1548,77,3)|0);
               HEAP32[$jmp2>>2] = $1549;
               $1550 = HEAP32[$v>>2]|0;
               (_sqlite3VdbeAddOp2($1550,37,1,-1)|0);
               $1551 = HEAP32[$db>>2]|0;
               $1552 = HEAP32[$pTab35>>2]|0;
               $1553 = HEAP32[$1552>>2]|0;
               $1554 = HEAP32[$j30>>2]|0;
               $1555 = HEAP32[$pTab35>>2]|0;
               $1556 = (($1555) + 4|0);
               $1557 = HEAP32[$1556>>2]|0;
               $1558 = (($1557) + (($1554*24)|0)|0);
               $1559 = HEAP32[$1558>>2]|0;
               HEAP32[$vararg_buffer35>>2] = $1553;
               $vararg_ptr34 = (($vararg_buffer35) + 4|0);
               HEAP32[$vararg_ptr34>>2] = $1559;
               $1560 = (_sqlite3MPrintf($1551,21600,$vararg_buffer35)|0);
               HEAP32[$zErr>>2] = $1560;
               $1561 = HEAP32[$v>>2]|0;
               $1562 = HEAP32[$zErr>>2]|0;
               (_sqlite3VdbeAddOp4($1561,97,0,3,0,$1562,-1)|0);
               $1563 = HEAP32[$v>>2]|0;
               (_sqlite3VdbeAddOp2($1563,35,3,1)|0);
               $1564 = HEAP32[$v>>2]|0;
               $1565 = (_sqlite3VdbeAddOp1($1564,137,1)|0);
               HEAP32[$jmp3>>2] = $1565;
               $1566 = HEAP32[$v>>2]|0;
               (_sqlite3VdbeAddOp0($1566,24)|0);
               $1567 = HEAP32[$v>>2]|0;
               $1568 = HEAP32[$jmp2>>2]|0;
               _sqlite3VdbeJumpHere($1567,$1568);
               $1569 = HEAP32[$v>>2]|0;
               $1570 = HEAP32[$jmp3>>2]|0;
               _sqlite3VdbeJumpHere($1569,$1570);
              }
             } while(0);
             $1571 = HEAP32[$j30>>2]|0;
             $1572 = (($1571) + 1)|0;
             HEAP32[$j30>>2] = $1572;
            }
            HEAP32[$j30>>2] = 0;
            $1573 = HEAP32[$pTab35>>2]|0;
            $1574 = (($1573) + 8|0);
            $1575 = HEAP32[$1574>>2]|0;
            HEAP32[$pIdx36>>2] = $1575;
            while(1) {
             $1576 = HEAP32[$pIdx36>>2]|0;
             $1577 = ($1576|0)!=(0|0);
             $1578 = HEAP32[$v>>2]|0;
             if (!($1577)) {
              break;
             }
             $1579 = (_sqlite3VdbeMakeLabel($1578)|0);
             HEAP32[$ckUniq>>2] = $1579;
             $1580 = HEAP32[$pPk37>>2]|0;
             $1581 = HEAP32[$pIdx36>>2]|0;
             $1582 = ($1580|0)==($1581|0);
             if (!($1582)) {
              $1583 = HEAP32[$0>>2]|0;
              $1584 = HEAP32[$pIdx36>>2]|0;
              $1585 = HEAP32[$iDataCur>>2]|0;
              $1586 = HEAP32[$pPrior>>2]|0;
              $1587 = HEAP32[$r1>>2]|0;
              $1588 = (_sqlite3GenerateIndexKey($1583,$1584,$1585,0,0,$jmp339,$1586,$1587)|0);
              HEAP32[$r1>>2] = $1588;
              $1589 = HEAP32[$pIdx36>>2]|0;
              HEAP32[$pPrior>>2] = $1589;
              $1590 = HEAP32[$v>>2]|0;
              $1591 = HEAP32[$j30>>2]|0;
              $1592 = (8 + ($1591))|0;
              (_sqlite3VdbeAddOp2($1590,37,$1592,1)|0);
              $1593 = HEAP32[$v>>2]|0;
              $1594 = HEAP32[$iIdxCur>>2]|0;
              $1595 = HEAP32[$j30>>2]|0;
              $1596 = (($1594) + ($1595))|0;
              $1597 = HEAP32[$ckUniq>>2]|0;
              $1598 = HEAP32[$r1>>2]|0;
              $1599 = HEAP32[$pIdx36>>2]|0;
              $1600 = (($1599) + 52|0);
              $1601 = HEAP16[$1600>>1]|0;
              $1602 = $1601&65535;
              $1603 = (_sqlite3VdbeAddOp4Int($1593,69,$1596,$1597,$1598,$1602)|0);
              HEAP32[$jmp238>>2] = $1603;
              $1604 = HEAP32[$v>>2]|0;
              (_sqlite3VdbeAddOp2($1604,37,1,-1)|0);
              $1605 = HEAP32[$v>>2]|0;
              (_sqlite3VdbeAddOp4($1605,97,0,3,0,21624,-2)|0);
              $1606 = HEAP32[$v>>2]|0;
              (_sqlite3VdbeAddOp3($1606,94,7,3,3)|0);
              $1607 = HEAP32[$v>>2]|0;
              (_sqlite3VdbeAddOp4($1607,97,0,4,0,21632,-2)|0);
              $1608 = HEAP32[$v>>2]|0;
              (_sqlite3VdbeAddOp3($1608,94,4,3,3)|0);
              $1609 = HEAP32[$v>>2]|0;
              $1610 = HEAP32[$pIdx36>>2]|0;
              $1611 = HEAP32[$1610>>2]|0;
              $1612 = (_sqlite3VdbeAddOp4($1609,97,0,4,0,$1611,0)|0);
              HEAP32[$jmp5>>2] = $1612;
              $1613 = HEAP32[$v>>2]|0;
              (_sqlite3VdbeAddOp3($1613,94,4,3,3)|0);
              $1614 = HEAP32[$v>>2]|0;
              (_sqlite3VdbeAddOp2($1614,35,3,1)|0);
              $1615 = HEAP32[$v>>2]|0;
              $1616 = (_sqlite3VdbeAddOp1($1615,137,1)|0);
              HEAP32[$jmp4>>2] = $1616;
              $1617 = HEAP32[$v>>2]|0;
              (_sqlite3VdbeAddOp0($1617,24)|0);
              $1618 = HEAP32[$v>>2]|0;
              $1619 = HEAP32[$jmp238>>2]|0;
              _sqlite3VdbeJumpHere($1618,$1619);
              $1620 = HEAP32[$pIdx36>>2]|0;
              $1621 = (($1620) + 54|0);
              $1622 = HEAP8[$1621>>0]|0;
              $1623 = $1622&255;
              $1624 = ($1623|0)!=(0);
              if ($1624) {
               $1625 = HEAP32[$v>>2]|0;
               $1626 = (_sqlite3VdbeMakeLabel($1625)|0);
               HEAP32[$uniqOk>>2] = $1626;
               HEAP32[$kk>>2] = 0;
               while(1) {
                $1627 = HEAP32[$kk>>2]|0;
                $1628 = HEAP32[$pIdx36>>2]|0;
                $1629 = (($1628) + 50|0);
                $1630 = HEAP16[$1629>>1]|0;
                $1631 = $1630&65535;
                $1632 = ($1627|0)<($1631|0);
                if (!($1632)) {
                 break;
                }
                $1633 = HEAP32[$kk>>2]|0;
                $1634 = HEAP32[$pIdx36>>2]|0;
                $1635 = (($1634) + 4|0);
                $1636 = HEAP32[$1635>>2]|0;
                $1637 = (($1636) + ($1633<<1)|0);
                $1638 = HEAP16[$1637>>1]|0;
                $1639 = $1638 << 16 >> 16;
                HEAP32[$iCol>>2] = $1639;
                $1640 = HEAP32[$iCol>>2]|0;
                $1641 = HEAP32[$pTab35>>2]|0;
                $1642 = (($1641) + 4|0);
                $1643 = HEAP32[$1642>>2]|0;
                $1644 = (($1643) + (($1640*24)|0)|0);
                $1645 = (($1644) + 20|0);
                $1646 = HEAP8[$1645>>0]|0;
                $1647 = ($1646<<24>>24)!=(0);
                if (!($1647)) {
                 $1648 = HEAP32[$v>>2]|0;
                 $1649 = HEAP32[$r1>>2]|0;
                 $1650 = HEAP32[$kk>>2]|0;
                 $1651 = (($1649) + ($1650))|0;
                 $1652 = HEAP32[$uniqOk>>2]|0;
                 (_sqlite3VdbeAddOp2($1648,76,$1651,$1652)|0);
                }
                $1653 = HEAP32[$kk>>2]|0;
                $1654 = (($1653) + 1)|0;
                HEAP32[$kk>>2] = $1654;
               }
               $1655 = HEAP32[$v>>2]|0;
               $1656 = HEAP32[$iIdxCur>>2]|0;
               $1657 = HEAP32[$j30>>2]|0;
               $1658 = (($1656) + ($1657))|0;
               $1659 = (_sqlite3VdbeAddOp1($1655,9,$1658)|0);
               HEAP32[$jmp6>>2] = $1659;
               $1660 = HEAP32[$v>>2]|0;
               $1661 = HEAP32[$uniqOk>>2]|0;
               (_sqlite3VdbeAddOp2($1660,16,0,$1661)|0);
               $1662 = HEAP32[$v>>2]|0;
               $1663 = HEAP32[$jmp6>>2]|0;
               _sqlite3VdbeJumpHere($1662,$1663);
               $1664 = HEAP32[$v>>2]|0;
               $1665 = HEAP32[$iIdxCur>>2]|0;
               $1666 = HEAP32[$j30>>2]|0;
               $1667 = (($1665) + ($1666))|0;
               $1668 = HEAP32[$uniqOk>>2]|0;
               $1669 = HEAP32[$r1>>2]|0;
               $1670 = HEAP32[$pIdx36>>2]|0;
               $1671 = (($1670) + 50|0);
               $1672 = HEAP16[$1671>>1]|0;
               $1673 = $1672&65535;
               (_sqlite3VdbeAddOp4Int($1664,114,$1667,$1668,$1669,$1673)|0);
               $1674 = HEAP32[$v>>2]|0;
               (_sqlite3VdbeAddOp2($1674,37,1,-1)|0);
               $1675 = HEAP32[$v>>2]|0;
               (_sqlite3VdbeAddOp4($1675,97,0,3,0,21656,-2)|0);
               $1676 = HEAP32[$v>>2]|0;
               $1677 = HEAP32[$jmp5>>2]|0;
               (_sqlite3VdbeAddOp2($1676,16,0,$1677)|0);
               $1678 = HEAP32[$v>>2]|0;
               $1679 = HEAP32[$uniqOk>>2]|0;
               _sqlite3VdbeResolveLabel($1678,$1679);
              }
              $1680 = HEAP32[$v>>2]|0;
              $1681 = HEAP32[$jmp4>>2]|0;
              _sqlite3VdbeJumpHere($1680,$1681);
              $1682 = HEAP32[$0>>2]|0;
              $1683 = HEAP32[$jmp339>>2]|0;
              _sqlite3ResolvePartIdxLabel($1682,$1683);
             }
             $1684 = HEAP32[$pIdx36>>2]|0;
             $1685 = (($1684) + 20|0);
             $1686 = HEAP32[$1685>>2]|0;
             HEAP32[$pIdx36>>2] = $1686;
             $1687 = HEAP32[$j30>>2]|0;
             $1688 = (($1687) + 1)|0;
             HEAP32[$j30>>2] = $1688;
            }
            $1689 = HEAP32[$iDataCur>>2]|0;
            $1690 = HEAP32[$loopTop>>2]|0;
            (_sqlite3VdbeAddOp2($1578,9,$1689,$1690)|0);
            $1691 = HEAP32[$v>>2]|0;
            $1692 = HEAP32[$loopTop>>2]|0;
            $1693 = (($1692) - 1)|0;
            _sqlite3VdbeJumpHere($1691,$1693);
            $1694 = HEAP32[$v>>2]|0;
            (_sqlite3VdbeAddOp4($1694,97,0,2,0,21688,-2)|0);
            HEAP32[$j30>>2] = 0;
            $1695 = HEAP32[$pTab35>>2]|0;
            $1696 = (($1695) + 8|0);
            $1697 = HEAP32[$1696>>2]|0;
            HEAP32[$pIdx36>>2] = $1697;
            while(1) {
             $1698 = HEAP32[$pIdx36>>2]|0;
             $1699 = ($1698|0)!=(0|0);
             if (!($1699)) {
              break L293;
             }
             $1700 = HEAP32[$pPk37>>2]|0;
             $1701 = HEAP32[$pIdx36>>2]|0;
             $1702 = ($1700|0)==($1701|0);
             if (!($1702)) {
              $1703 = HEAP32[$v>>2]|0;
              $1704 = (_sqlite3VdbeCurrentAddr($1703)|0);
              HEAP32[$addr31>>2] = $1704;
              $1705 = HEAP32[$v>>2]|0;
              $1706 = HEAP32[$addr31>>2]|0;
              $1707 = (($1706) + 2)|0;
              (_sqlite3VdbeAddOp2($1705,137,1,$1707)|0);
              $1708 = HEAP32[$v>>2]|0;
              (_sqlite3VdbeAddOp2($1708,24,0,0)|0);
              $1709 = HEAP32[$v>>2]|0;
              $1710 = HEAP32[$iIdxCur>>2]|0;
              $1711 = HEAP32[$j30>>2]|0;
              $1712 = (($1710) + ($1711))|0;
              (_sqlite3VdbeAddOp2($1709,50,$1712,3)|0);
              $1713 = HEAP32[$v>>2]|0;
              $1714 = HEAP32[$j30>>2]|0;
              $1715 = (8 + ($1714))|0;
              $1716 = HEAP32[$addr31>>2]|0;
              $1717 = (($1716) + 8)|0;
              (_sqlite3VdbeAddOp3($1713,79,$1715,$1717,3)|0);
              $1718 = HEAP32[$v>>2]|0;
              _sqlite3VdbeChangeP5($1718,-112);
              $1719 = HEAP32[$v>>2]|0;
              (_sqlite3VdbeAddOp2($1719,37,1,-1)|0);
              $1720 = HEAP32[$v>>2]|0;
              $1721 = HEAP32[$pIdx36>>2]|0;
              $1722 = HEAP32[$1721>>2]|0;
              (_sqlite3VdbeAddOp4($1720,97,0,3,0,$1722,0)|0);
              $1723 = HEAP32[$v>>2]|0;
              (_sqlite3VdbeAddOp3($1723,94,3,2,7)|0);
              $1724 = HEAP32[$v>>2]|0;
              (_sqlite3VdbeAddOp2($1724,35,7,1)|0);
             }
             $1725 = HEAP32[$pIdx36>>2]|0;
             $1726 = (($1725) + 20|0);
             $1727 = HEAP32[$1726>>2]|0;
             HEAP32[$pIdx36>>2] = $1727;
             $1728 = HEAP32[$j30>>2]|0;
             $1729 = (($1728) + 1)|0;
             HEAP32[$j30>>2] = $1729;
            }
           }
          } while(0);
          $1730 = HEAP32[$x32>>2]|0;
          $1731 = HEAP32[$1730>>2]|0;
          HEAP32[$x32>>2] = $1731;
         }
        }
       } while(0);
       $1732 = HEAP32[$i29>>2]|0;
       $1733 = (($1732) + 1)|0;
       HEAP32[$i29>>2] = $1733;
      }
      $1734 = HEAP32[$v>>2]|0;
      $1735 = (_sqlite3VdbeAddOpList($1734,3,21544,0)|0);
      HEAP32[$addr31>>2] = $1735;
      $1736 = HEAP32[$v>>2]|0;
      $1737 = HEAP32[$addr31>>2]|0;
      $1738 = HEAP32[$mxErr>>2]|0;
      $1739 = (0 - ($1738))|0;
      _sqlite3VdbeChangeP3($1736,$1737,$1739);
      $1740 = HEAP32[$v>>2]|0;
      $1741 = HEAP32[$addr31>>2]|0;
      _sqlite3VdbeJumpHere($1740,$1741);
      $1742 = HEAP32[$v>>2]|0;
      $1743 = HEAP32[$addr31>>2]|0;
      $1744 = (($1743) + 1)|0;
      _sqlite3VdbeChangeP4($1742,$1744,21720,-2);
      break L21;
      break;
     }
     case 5:  {
      $1331 = HEAP32[$zRight>>2]|0;
      $1332 = ($1331|0)!=(0|0);
      if (!($1332)) {
       break L21;
      }
      $1333 = HEAP32[$db>>2]|0;
      $1334 = HEAP32[$zRight>>2]|0;
      $1335 = (_sqlite3GetBoolean($1334,0)|0);
      $1336 = $1335&255;
      _sqlite3RegisterLikeFunctions($1333,$1336);
      break L21;
      break;
     }
     case 11:  {
      $1745 = HEAP32[$zRight>>2]|0;
      $1746 = ($1745|0)!=(0|0);
      if (!($1746)) {
       $1747 = HEAP32[$0>>2]|0;
       $1748 = (_sqlite3ReadSchema($1747)|0);
       $1749 = ($1748|0)!=(0);
       if ($1749) {
        break L21;
       }
       $1750 = HEAP32[$v>>2]|0;
       _sqlite3VdbeSetNumCols($1750,1);
       $1751 = HEAP32[$v>>2]|0;
       (_sqlite3VdbeSetColName($1751,0,0,21880,0)|0);
       $1752 = HEAP32[$v>>2]|0;
       (_sqlite3VdbeAddOp2($1752,97,0,1)|0);
       $1753 = HEAP32[$v>>2]|0;
       $1754 = HEAP32[$0>>2]|0;
       $1755 = HEAP32[$1754>>2]|0;
       $1756 = (($1755) + 16|0);
       $1757 = HEAP32[$1756>>2]|0;
       $1758 = (($1757) + 12|0);
       $1759 = HEAP32[$1758>>2]|0;
       $1760 = (($1759) + 77|0);
       $1761 = HEAP8[$1760>>0]|0;
       $1762 = $1761&255;
       $1763 = (21728 + ($1762<<3)|0);
       $1764 = HEAP32[$1763>>2]|0;
       _sqlite3VdbeChangeP4($1753,-1,$1764,-2);
       $1765 = HEAP32[$v>>2]|0;
       (_sqlite3VdbeAddOp2($1765,35,1,1)|0);
       break L21;
      }
      $1766 = HEAP32[$db>>2]|0;
      $1767 = (($1766) + 16|0);
      $1768 = HEAP32[$1767>>2]|0;
      $1769 = (($1768) + 12|0);
      $1770 = HEAP32[$1769>>2]|0;
      $1771 = (($1770) + 78|0);
      $1772 = HEAP16[$1771>>1]|0;
      $1773 = $1772&65535;
      $1774 = $1773 & 1;
      $1775 = ($1774|0)==(1);
      if ($1775) {
       $1776 = HEAP32[$db>>2]|0;
       $1777 = (($1776) + 16|0);
       $1778 = HEAP32[$1777>>2]|0;
       $1779 = (($1778) + 12|0);
       $1780 = HEAP32[$1779>>2]|0;
       $1781 = (($1780) + 78|0);
       $1782 = HEAP16[$1781>>1]|0;
       $1783 = $1782&65535;
       $1784 = $1783 & 4;
       $1785 = ($1784|0)==(4);
       if (!($1785)) {
        break L21;
       }
      }
      HEAP32[$pEnc>>2] = 21728;
      while(1) {
       $1786 = HEAP32[$pEnc>>2]|0;
       $1787 = HEAP32[$1786>>2]|0;
       $1788 = ($1787|0)!=(0|0);
       if (!($1788)) {
        break;
       }
       $1789 = HEAP32[$zRight>>2]|0;
       $1790 = HEAP32[$pEnc>>2]|0;
       $1791 = HEAP32[$1790>>2]|0;
       $1792 = (_sqlite3_stricmp($1789,$1791)|0);
       $1793 = (0)==($1792|0);
       $1794 = HEAP32[$pEnc>>2]|0;
       if ($1793) {
        label = 286;
        break;
       }
       $1816 = (($1794) + 8|0);
       HEAP32[$pEnc>>2] = $1816;
      }
      if ((label|0) == 286) {
       $1795 = (($1794) + 4|0);
       $1796 = HEAP8[$1795>>0]|0;
       $1797 = $1796&255;
       $1798 = ($1797|0)!=(0);
       if ($1798) {
        $1799 = HEAP32[$pEnc>>2]|0;
        $1800 = (($1799) + 4|0);
        $1801 = HEAP8[$1800>>0]|0;
        $1802 = $1801&255;
        $1808 = $1802;
       } else {
        $1803 = HEAP8[1136>>0]|0;
        $1804 = $1803 << 24 >> 24;
        $1805 = ($1804|0)==(0);
        $1806 = $1805 ? 3 : 2;
        $1808 = $1806;
       }
       $1807 = $1808&255;
       $1809 = HEAP32[$0>>2]|0;
       $1810 = HEAP32[$1809>>2]|0;
       $1811 = (($1810) + 16|0);
       $1812 = HEAP32[$1811>>2]|0;
       $1813 = (($1812) + 12|0);
       $1814 = HEAP32[$1813>>2]|0;
       $1815 = (($1814) + 77|0);
       HEAP8[$1815>>0] = $1807;
      }
      $1817 = HEAP32[$pEnc>>2]|0;
      $1818 = HEAP32[$1817>>2]|0;
      $1819 = ($1818|0)!=(0|0);
      if ($1819) {
       break L21;
      }
      $1820 = HEAP32[$0>>2]|0;
      $1821 = HEAP32[$zRight>>2]|0;
      HEAP32[$vararg_buffer35>>2] = $1821;
      _sqlite3ErrorMsg($1820,21896,$vararg_buffer35);
      break L21;
      break;
     }
     case 0:  {
      $1822 = HEAP32[$v>>2]|0;
      $1823 = HEAP32[$iDb>>2]|0;
      _sqlite3VdbeUsesBtree($1822,$1823);
      $1824 = HEAP32[$zLeft>>2]|0;
      $1825 = HEAP8[$1824>>0]|0;
      $1826 = $1825 << 24 >> 24;
      switch ($1826|0) {
      case 65: case 97:  {
       HEAP32[$iCookie>>2] = 8;
       break;
      }
      case 70: case 102:  {
       HEAP32[$iCookie>>2] = 0;
       break;
      }
      case 83: case 115:  {
       HEAP32[$iCookie>>2] = 1;
       break;
      }
      default: {
       HEAP32[$iCookie>>2] = 6;
      }
      }
      $1827 = HEAP32[$zRight>>2]|0;
      $1828 = ($1827|0)!=(0|0);
      $1829 = HEAP32[$iCookie>>2]|0;
      $1830 = ($1829|0)!=(0);
      $or$cond15 = $1828 & $1830;
      $1831 = HEAP32[$v>>2]|0;
      if ($or$cond15) {
       $1832 = (_sqlite3VdbeAddOpList($1831,3,21928,0)|0);
       HEAP32[$addr40>>2] = $1832;
       $1833 = HEAP32[$v>>2]|0;
       $1834 = HEAP32[$addr40>>2]|0;
       $1835 = HEAP32[$iDb>>2]|0;
       _sqlite3VdbeChangeP1($1833,$1834,$1835);
       $1836 = HEAP32[$v>>2]|0;
       $1837 = HEAP32[$addr40>>2]|0;
       $1838 = (($1837) + 1)|0;
       $1839 = HEAP32[$zRight>>2]|0;
       $1840 = (_sqlite3Atoi($1839)|0);
       _sqlite3VdbeChangeP1($1836,$1838,$1840);
       $1841 = HEAP32[$v>>2]|0;
       $1842 = HEAP32[$addr40>>2]|0;
       $1843 = (($1842) + 2)|0;
       $1844 = HEAP32[$iDb>>2]|0;
       _sqlite3VdbeChangeP1($1841,$1843,$1844);
       $1845 = HEAP32[$v>>2]|0;
       $1846 = HEAP32[$addr40>>2]|0;
       $1847 = (($1846) + 2)|0;
       $1848 = HEAP32[$iCookie>>2]|0;
       _sqlite3VdbeChangeP2($1845,$1847,$1848);
       break L21;
      } else {
       $1849 = (_sqlite3VdbeAddOpList($1831,3,21944,0)|0);
       HEAP32[$addr41>>2] = $1849;
       $1850 = HEAP32[$v>>2]|0;
       $1851 = HEAP32[$addr41>>2]|0;
       $1852 = HEAP32[$iDb>>2]|0;
       _sqlite3VdbeChangeP1($1850,$1851,$1852);
       $1853 = HEAP32[$v>>2]|0;
       $1854 = HEAP32[$addr41>>2]|0;
       $1855 = (($1854) + 1)|0;
       $1856 = HEAP32[$iDb>>2]|0;
       _sqlite3VdbeChangeP1($1853,$1855,$1856);
       $1857 = HEAP32[$v>>2]|0;
       $1858 = HEAP32[$addr41>>2]|0;
       $1859 = (($1858) + 1)|0;
       $1860 = HEAP32[$iCookie>>2]|0;
       _sqlite3VdbeChangeP3($1857,$1859,$1860);
       $1861 = HEAP32[$v>>2]|0;
       _sqlite3VdbeSetNumCols($1861,1);
       $1862 = HEAP32[$v>>2]|0;
       $1863 = HEAP32[$zLeft>>2]|0;
       (_sqlite3VdbeSetColName($1862,0,0,$1863,(-1))|0);
       break L21;
      }
      break;
     }
     case 35:  {
      $1875 = HEAP32[$2>>2]|0;
      $1876 = HEAP32[$1875>>2]|0;
      $1877 = ($1876|0)!=(0|0);
      $1878 = HEAP32[$iDb>>2]|0;
      $1879 = $1877 ? $1878 : 10;
      HEAP32[$iBt>>2] = $1879;
      HEAP32[$eMode43>>2] = 0;
      $1880 = HEAP32[$zRight>>2]|0;
      $1881 = ($1880|0)!=(0|0);
      do {
       if ($1881) {
        $1882 = HEAP32[$zRight>>2]|0;
        $1883 = (_sqlite3_stricmp($1882,21976)|0);
        $1884 = ($1883|0)==(0);
        if ($1884) {
         HEAP32[$eMode43>>2] = 1;
         break;
        }
        $1885 = HEAP32[$zRight>>2]|0;
        $1886 = (_sqlite3_stricmp($1885,21984)|0);
        $1887 = ($1886|0)==(0);
        if ($1887) {
         HEAP32[$eMode43>>2] = 2;
        }
       }
      } while(0);
      $1888 = HEAP32[$v>>2]|0;
      _sqlite3VdbeSetNumCols($1888,3);
      $1889 = HEAP32[$0>>2]|0;
      $1890 = (($1889) + 72|0);
      HEAP32[$1890>>2] = 3;
      $1891 = HEAP32[$v>>2]|0;
      (_sqlite3VdbeSetColName($1891,0,0,21992,0)|0);
      $1892 = HEAP32[$v>>2]|0;
      (_sqlite3VdbeSetColName($1892,1,0,22000,0)|0);
      $1893 = HEAP32[$v>>2]|0;
      (_sqlite3VdbeSetColName($1893,2,0,22008,0)|0);
      $1894 = HEAP32[$v>>2]|0;
      $1895 = HEAP32[$iBt>>2]|0;
      $1896 = HEAP32[$eMode43>>2]|0;
      (_sqlite3VdbeAddOp3($1894,11,$1895,$1896,1)|0);
      $1897 = HEAP32[$v>>2]|0;
      (_sqlite3VdbeAddOp2($1897,35,1,3)|0);
      break L21;
      break;
     }
     case 7:  {
      HEAP32[$i42>>2] = 0;
      $1864 = HEAP32[$v>>2]|0;
      _sqlite3VdbeSetNumCols($1864,1);
      $1865 = HEAP32[$0>>2]|0;
      $1866 = (($1865) + 72|0);
      HEAP32[$1866>>2] = 1;
      $1867 = HEAP32[$v>>2]|0;
      (_sqlite3VdbeSetColName($1867,0,0,21960,0)|0);
      while(1) {
       $1868 = HEAP32[$i42>>2]|0;
       $1869 = (($1868) + 1)|0;
       HEAP32[$i42>>2] = $1869;
       $1870 = (_sqlite3_compileoption_get($1868)|0);
       HEAP32[$zOpt>>2] = $1870;
       $1871 = ($1870|0)!=(0|0);
       if (!($1871)) {
        break L21;
       }
       $1872 = HEAP32[$v>>2]|0;
       $1873 = HEAP32[$zOpt>>2]|0;
       (_sqlite3VdbeAddOp4($1872,97,0,1,0,$1873,0)|0);
       $1874 = HEAP32[$v>>2]|0;
       (_sqlite3VdbeAddOp2($1874,35,1,1)|0);
      }
      break;
     }
     case 26:  {
      $1915 = HEAP32[$db>>2]|0;
      (_sqlite3_db_release_memory($1915)|0);
      break L21;
      break;
     }
     case 34:  {
      $1898 = HEAP32[$zRight>>2]|0;
      $1899 = ($1898|0)!=(0|0);
      if ($1899) {
       $1900 = HEAP32[$db>>2]|0;
       $1901 = HEAP32[$zRight>>2]|0;
       $1902 = (_sqlite3Atoi($1901)|0);
       (_sqlite3_wal_autocheckpoint($1900,$1902)|0);
      }
      $1903 = HEAP32[$0>>2]|0;
      $1904 = HEAP32[$db>>2]|0;
      $1905 = (($1904) + 212|0);
      $1906 = HEAP32[$1905>>2]|0;
      $1907 = ($1906|0)==(5|0);
      if ($1907) {
       $1908 = HEAP32[$db>>2]|0;
       $1909 = (($1908) + 216|0);
       $1910 = HEAP32[$1909>>2]|0;
       $1911 = $1910;
       $1913 = $1911;
      } else {
       $1913 = 0;
      }
      $1912 = ($1913|0)<(0);
      $1914 = $1912 << 31 >> 31;
      _returnSingleInt($1903,22024,$1913,$1914);
      break L21;
      break;
     }
     case 27:  {
      $1927 = HEAP32[$zRight>>2]|0;
      $1928 = ($1927|0)!=(0|0);
      if ($1928) {
       $1929 = HEAP32[$zRight>>2]|0;
       $1930 = (_sqlite3DecOrHexToI64($1929,$N)|0);
       $1931 = ($1930|0)==(0);
       if ($1931) {
        $1932 = $N;
        $1933 = $1932;
        $1934 = HEAP32[$1933>>2]|0;
        $1935 = (($1932) + 4)|0;
        $1936 = $1935;
        $1937 = HEAP32[$1936>>2]|0;
        (_sqlite3_soft_heap_limit64($1934,$1937)|0);
        $1938 = tempRet0;
       }
      }
      $1939 = HEAP32[$0>>2]|0;
      $1940 = (_sqlite3_soft_heap_limit64(-1,-1)|0);
      $1941 = tempRet0;
      _returnSingleInt($1939,22056,$1940,$1941);
      break L21;
      break;
     }
     case 33:  {
      $1942 = HEAP32[$zRight>>2]|0;
      $1943 = ($1942|0)!=(0|0);
      if ($1943) {
       $1944 = HEAP32[$zRight>>2]|0;
       $1945 = (_sqlite3DecOrHexToI64($1944,$N44)|0);
       $1946 = ($1945|0)==(0);
       $1947 = $N44;
       $1948 = $1947;
       $1949 = HEAP32[$1948>>2]|0;
       $1950 = (($1947) + 4)|0;
       $1951 = $1950;
       $1952 = HEAP32[$1951>>2]|0;
       $1953 = ($1952|0)>(0);
       $1954 = ($1949>>>0)>=(0);
       $1955 = ($1952|0)==(0);
       $1956 = $1955 & $1954;
       $1957 = $1953 | $1956;
       $or$cond17 = $1946 & $1957;
       if ($or$cond17) {
        $1958 = HEAP32[$db>>2]|0;
        $1959 = $N44;
        $1960 = $1959;
        $1961 = HEAP32[$1960>>2]|0;
        $1962 = (($1959) + 4)|0;
        $1963 = $1962;
        $1964 = HEAP32[$1963>>2]|0;
        $1965 = $1961 & 2147483647;
        (_sqlite3_limit($1958,11,$1965)|0);
       }
      }
      $1966 = HEAP32[$0>>2]|0;
      $1967 = HEAP32[$db>>2]|0;
      $1968 = (_sqlite3_limit($1967,11,-1)|0);
      $1969 = ($1968|0)<(0);
      $1970 = $1969 << 31 >> 31;
      _returnSingleInt($1966,22072,$1968,$1970);
      break L21;
      break;
     }
     case 29:  {
      $572 = HEAP32[$zRight>>2]|0;
      $573 = ($572|0)!=(0|0);
      if (!($573)) {
       $574 = HEAP32[$0>>2]|0;
       $575 = HEAP32[$pDb>>2]|0;
       $576 = (($575) + 8|0);
       $577 = HEAP8[$576>>0]|0;
       $578 = $577&255;
       $579 = (($578) - 1)|0;
       $580 = ($579|0)<(0);
       $581 = $580 << 31 >> 31;
       _returnSingleInt($574,21272,$579,$581);
       break L21;
      }
      $582 = HEAP32[$db>>2]|0;
      $583 = (($582) + 62|0);
      $584 = HEAP8[$583>>0]|0;
      $585 = ($584<<24>>24)!=(0);
      if ($585) {
       $587 = HEAP32[$zRight>>2]|0;
       $588 = (_getSafetyLevel($587,0,1)|0);
       $589 = $588&255;
       $590 = (($589) + 1)|0;
       $591 = $590&255;
       $592 = HEAP32[$pDb>>2]|0;
       $593 = (($592) + 8|0);
       HEAP8[$593>>0] = $591;
       $594 = HEAP32[$db>>2]|0;
       _setAllPagerFlags($594);
       break L21;
      } else {
       $586 = HEAP32[$0>>2]|0;
       _sqlite3ErrorMsg($586,21288,$vararg_buffer35);
       break L21;
      }
      break;
     }
     case 30:  {
      $647 = HEAP32[$zRight>>2]|0;
      $648 = ($647|0)!=(0|0);
      if (!($648)) {
       break L21;
      }
      $649 = HEAP32[$db>>2]|0;
      $650 = HEAP32[$zRight>>2]|0;
      $651 = HEAP32[$zDb>>2]|0;
      $652 = (_sqlite3FindTable($649,$650,$651)|0);
      HEAP32[$pTab>>2] = $652;
      $653 = HEAP32[$pTab>>2]|0;
      $654 = ($653|0)!=(0|0);
      if (!($654)) {
       break L21;
      }
      HEAP32[$nHidden>>2] = 0;
      $655 = HEAP32[$pTab>>2]|0;
      $656 = (_sqlite3PrimaryKeyIndex($655)|0);
      HEAP32[$pPk>>2] = $656;
      $657 = HEAP32[$v>>2]|0;
      _sqlite3VdbeSetNumCols($657,6);
      $658 = HEAP32[$0>>2]|0;
      $659 = (($658) + 72|0);
      HEAP32[$659>>2] = 6;
      $660 = HEAP32[$0>>2]|0;
      $661 = HEAP32[$iDb>>2]|0;
      _sqlite3CodeVerifySchema($660,$661);
      $662 = HEAP32[$v>>2]|0;
      (_sqlite3VdbeSetColName($662,0,0,21344,0)|0);
      $663 = HEAP32[$v>>2]|0;
      (_sqlite3VdbeSetColName($663,1,0,21352,0)|0);
      $664 = HEAP32[$v>>2]|0;
      (_sqlite3VdbeSetColName($664,2,0,21360,0)|0);
      $665 = HEAP32[$v>>2]|0;
      (_sqlite3VdbeSetColName($665,3,0,21368,0)|0);
      $666 = HEAP32[$v>>2]|0;
      (_sqlite3VdbeSetColName($666,4,0,21376,0)|0);
      $667 = HEAP32[$v>>2]|0;
      (_sqlite3VdbeSetColName($667,5,0,21392,0)|0);
      $668 = HEAP32[$0>>2]|0;
      $669 = HEAP32[$pTab>>2]|0;
      (_sqlite3ViewGetColumnNames($668,$669)|0);
      HEAP32[$i>>2] = 0;
      $670 = HEAP32[$pTab>>2]|0;
      $671 = (($670) + 4|0);
      $672 = HEAP32[$671>>2]|0;
      HEAP32[$pCol>>2] = $672;
      while(1) {
       $673 = HEAP32[$i>>2]|0;
       $674 = HEAP32[$pTab>>2]|0;
       $675 = (($674) + 38|0);
       $676 = HEAP16[$675>>1]|0;
       $677 = $676 << 16 >> 16;
       $678 = ($673|0)<($677|0);
       if (!($678)) {
        break L21;
       }
       $679 = HEAP32[$pCol>>2]|0;
       $680 = (($679) + 23|0);
       $681 = HEAP8[$680>>0]|0;
       $682 = $681&255;
       $683 = $682 & 2;
       $684 = ($683|0)!=(0);
       if ($684) {
        $685 = HEAP32[$nHidden>>2]|0;
        $686 = (($685) + 1)|0;
        HEAP32[$nHidden>>2] = $686;
       } else {
        $687 = HEAP32[$v>>2]|0;
        $688 = HEAP32[$i>>2]|0;
        $689 = HEAP32[$nHidden>>2]|0;
        $690 = (($688) - ($689))|0;
        (_sqlite3VdbeAddOp2($687,25,$690,1)|0);
        $691 = HEAP32[$v>>2]|0;
        $692 = HEAP32[$pCol>>2]|0;
        $693 = HEAP32[$692>>2]|0;
        (_sqlite3VdbeAddOp4($691,97,0,2,0,$693,0)|0);
        $694 = HEAP32[$v>>2]|0;
        $695 = HEAP32[$pCol>>2]|0;
        $696 = (($695) + 12|0);
        $697 = HEAP32[$696>>2]|0;
        $698 = ($697|0)!=(0|0);
        if ($698) {
         $699 = HEAP32[$pCol>>2]|0;
         $700 = (($699) + 12|0);
         $701 = HEAP32[$700>>2]|0;
         $702 = $701;
        } else {
         $702 = 1216;
        }
        (_sqlite3VdbeAddOp4($694,97,0,3,0,$702,0)|0);
        $703 = HEAP32[$v>>2]|0;
        $704 = HEAP32[$pCol>>2]|0;
        $705 = (($704) + 20|0);
        $706 = HEAP8[$705>>0]|0;
        $707 = $706&255;
        $708 = ($707|0)!=(0);
        $709 = $708 ? 1 : 0;
        (_sqlite3VdbeAddOp2($703,25,$709,4)|0);
        $710 = HEAP32[$pCol>>2]|0;
        $711 = (($710) + 8|0);
        $712 = HEAP32[$711>>2]|0;
        $713 = ($712|0)!=(0|0);
        $714 = HEAP32[$v>>2]|0;
        if ($713) {
         $715 = HEAP32[$pCol>>2]|0;
         $716 = (($715) + 8|0);
         $717 = HEAP32[$716>>2]|0;
         (_sqlite3VdbeAddOp4($714,97,0,5,0,$717,0)|0);
        } else {
         (_sqlite3VdbeAddOp2($714,28,0,5)|0);
        }
        $718 = HEAP32[$pCol>>2]|0;
        $719 = (($718) + 23|0);
        $720 = HEAP8[$719>>0]|0;
        $721 = $720&255;
        $722 = $721 & 1;
        $723 = ($722|0)==(0);
        L429: do {
         if ($723) {
          HEAP32[$k>>2] = 0;
         } else {
          $724 = HEAP32[$pPk>>2]|0;
          $725 = ($724|0)==(0|0);
          HEAP32[$k>>2] = 1;
          if ($725) {
           break;
          }
          while(1) {
           $726 = HEAP32[$k>>2]|0;
           $727 = HEAP32[$pTab>>2]|0;
           $728 = (($727) + 38|0);
           $729 = HEAP16[$728>>1]|0;
           $730 = $729 << 16 >> 16;
           $731 = ($726|0)<=($730|0);
           if (!($731)) {
            break L429;
           }
           $732 = HEAP32[$k>>2]|0;
           $733 = (($732) - 1)|0;
           $734 = HEAP32[$pPk>>2]|0;
           $735 = (($734) + 4|0);
           $736 = HEAP32[$735>>2]|0;
           $737 = (($736) + ($733<<1)|0);
           $738 = HEAP16[$737>>1]|0;
           $739 = $738 << 16 >> 16;
           $740 = HEAP32[$i>>2]|0;
           $741 = ($739|0)!=($740|0);
           if (!($741)) {
            break L429;
           }
           $742 = HEAP32[$k>>2]|0;
           $743 = (($742) + 1)|0;
           HEAP32[$k>>2] = $743;
          }
         }
        } while(0);
        $744 = HEAP32[$v>>2]|0;
        $745 = HEAP32[$k>>2]|0;
        (_sqlite3VdbeAddOp2($744,25,$745,6)|0);
        $746 = HEAP32[$v>>2]|0;
        (_sqlite3VdbeAddOp2($746,35,1,6)|0);
       }
       $747 = HEAP32[$i>>2]|0;
       $748 = (($747) + 1)|0;
       HEAP32[$i>>2] = $748;
       $749 = HEAP32[$pCol>>2]|0;
       $750 = (($749) + 24|0);
       HEAP32[$pCol>>2] = $750;
      }
      break;
     }
     case 16:  {
      $864 = HEAP32[$zRight>>2]|0;
      $865 = ($864|0)!=(0|0);
      if (!($865)) {
       break L21;
      }
      $866 = HEAP32[$db>>2]|0;
      $867 = HEAP32[$zRight>>2]|0;
      $868 = HEAP32[$zDb>>2]|0;
      $869 = (_sqlite3FindTable($866,$867,$868)|0);
      HEAP32[$pTab17>>2] = $869;
      $870 = HEAP32[$pTab17>>2]|0;
      $871 = ($870|0)!=(0|0);
      if (!($871)) {
       break L21;
      }
      $872 = HEAP32[$0>>2]|0;
      $873 = (_sqlite3GetVdbe($872)|0);
      HEAP32[$v>>2] = $873;
      $874 = HEAP32[$v>>2]|0;
      _sqlite3VdbeSetNumCols($874,3);
      $875 = HEAP32[$0>>2]|0;
      $876 = (($875) + 72|0);
      HEAP32[$876>>2] = 3;
      $877 = HEAP32[$0>>2]|0;
      $878 = HEAP32[$iDb>>2]|0;
      _sqlite3CodeVerifySchema($877,$878);
      $879 = HEAP32[$v>>2]|0;
      (_sqlite3VdbeSetColName($879,0,0,21432,0)|0);
      $880 = HEAP32[$v>>2]|0;
      (_sqlite3VdbeSetColName($880,1,0,21352,0)|0);
      $881 = HEAP32[$v>>2]|0;
      (_sqlite3VdbeSetColName($881,2,0,21440,0)|0);
      $882 = HEAP32[$pTab17>>2]|0;
      $883 = (($882) + 8|0);
      $884 = HEAP32[$883>>2]|0;
      HEAP32[$pIdx16>>2] = $884;
      HEAP32[$i18>>2] = 0;
      while(1) {
       $885 = HEAP32[$pIdx16>>2]|0;
       $886 = ($885|0)!=(0|0);
       if (!($886)) {
        break L21;
       }
       $887 = HEAP32[$v>>2]|0;
       $888 = HEAP32[$i18>>2]|0;
       (_sqlite3VdbeAddOp2($887,25,$888,1)|0);
       $889 = HEAP32[$v>>2]|0;
       $890 = HEAP32[$pIdx16>>2]|0;
       $891 = HEAP32[$890>>2]|0;
       (_sqlite3VdbeAddOp4($889,97,0,2,0,$891,0)|0);
       $892 = HEAP32[$v>>2]|0;
       $893 = HEAP32[$pIdx16>>2]|0;
       $894 = (($893) + 54|0);
       $895 = HEAP8[$894>>0]|0;
       $896 = $895&255;
       $897 = ($896|0)!=(0);
       $898 = $897&1;
       (_sqlite3VdbeAddOp2($892,25,$898,3)|0);
       $899 = HEAP32[$v>>2]|0;
       (_sqlite3VdbeAddOp2($899,35,1,3)|0);
       $900 = HEAP32[$pIdx16>>2]|0;
       $901 = (($900) + 20|0);
       $902 = HEAP32[$901>>2]|0;
       HEAP32[$pIdx16>>2] = $902;
       $903 = HEAP32[$i18>>2]|0;
       $904 = (($903) + 1)|0;
       HEAP32[$i18>>2] = $904;
      }
      break;
     }
     default: {
      $1916 = HEAP32[$zRight>>2]|0;
      $1917 = ($1916|0)!=(0|0);
      if ($1917) {
       $1918 = HEAP32[$db>>2]|0;
       $1919 = HEAP32[$zRight>>2]|0;
       $1920 = (_sqlite3Atoi($1919)|0);
       (_sqlite3_busy_timeout($1918,$1920)|0);
      }
      $1921 = HEAP32[$0>>2]|0;
      $1922 = HEAP32[$db>>2]|0;
      $1923 = (($1922) + 492|0);
      $1924 = HEAP32[$1923>>2]|0;
      $1925 = ($1924|0)<(0);
      $1926 = $1925 << 31 >> 31;
      _returnSingleInt($1921,22048,$1924,$1926);
      break L21;
     }
     }
    } while(0);
   }
  }
 } while(0);
 $1971 = HEAP32[$db>>2]|0;
 $1972 = HEAP32[$zLeft>>2]|0;
 _sqlite3DbFree($1971,$1972);
 $1973 = HEAP32[$db>>2]|0;
 $1974 = HEAP32[$zRight>>2]|0;
 _sqlite3DbFree($1973,$1974);
 STACKTOP = sp;return;
}
function _sqlite3FinishTrigger($pParse,$pStepList,$pAll) {
 $pParse = $pParse|0;
 $pStepList = $pStepList|0;
 $pAll = $pAll|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0;
 var $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0;
 var $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0;
 var $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0;
 var $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0;
 var $98 = 0, $99 = 0, $db = 0, $iDb = 0, $nameToken = 0, $or$cond = 0, $pHash = 0, $pLink = 0, $pTab = 0, $pTrig = 0, $sFix = 0, $v = 0, $vararg_buffer6 = 0, $vararg_ptr2 = 0, $vararg_ptr3 = 0, $vararg_ptr4 = 0, $vararg_ptr5 = 0, $z = 0, $zName = 0, label = 0;
 var sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 112|0;
 $vararg_buffer6 = sp;
 $0 = sp + 100|0;
 $1 = sp + 96|0;
 $2 = sp + 92|0;
 $pTrig = sp + 88|0;
 $zName = sp + 84|0;
 $db = sp + 80|0;
 $sFix = sp + 56|0;
 $iDb = sp + 48|0;
 $nameToken = sp + 40|0;
 $v = sp + 36|0;
 $z = sp + 32|0;
 $pLink = sp + 28|0;
 $pHash = sp + 24|0;
 $pTab = sp + 20|0;
 HEAP32[$0>>2] = $pParse;
 HEAP32[$1>>2] = $pStepList;
 HEAP32[$2>>2] = $pAll;
 $3 = HEAP32[$0>>2]|0;
 $4 = (($3) + 488|0);
 $5 = HEAP32[$4>>2]|0;
 HEAP32[$pTrig>>2] = $5;
 $6 = HEAP32[$0>>2]|0;
 $7 = HEAP32[$6>>2]|0;
 HEAP32[$db>>2] = $7;
 $8 = HEAP32[$0>>2]|0;
 $9 = (($8) + 488|0);
 HEAP32[$9>>2] = 0;
 $10 = HEAP32[$0>>2]|0;
 $11 = (($10) + 64|0);
 $12 = HEAP32[$11>>2]|0;
 $13 = ($12|0)==(0);
 $14 = HEAP32[$pTrig>>2]|0;
 $15 = ($14|0)!=(0|0);
 $or$cond = $13 & $15;
 do {
  if ($or$cond) {
   $16 = HEAP32[$pTrig>>2]|0;
   $17 = HEAP32[$16>>2]|0;
   HEAP32[$zName>>2] = $17;
   $18 = HEAP32[$0>>2]|0;
   $19 = HEAP32[$18>>2]|0;
   $20 = HEAP32[$pTrig>>2]|0;
   $21 = (($20) + 20|0);
   $22 = HEAP32[$21>>2]|0;
   $23 = (_sqlite3SchemaToIndex($19,$22)|0);
   HEAP32[$iDb>>2] = $23;
   $24 = HEAP32[$1>>2]|0;
   $25 = HEAP32[$pTrig>>2]|0;
   $26 = (($25) + 28|0);
   HEAP32[$26>>2] = $24;
   while(1) {
    $27 = HEAP32[$1>>2]|0;
    $28 = ($27|0)!=(0|0);
    $29 = HEAP32[$pTrig>>2]|0;
    if (!($28)) {
     break;
    }
    $30 = HEAP32[$1>>2]|0;
    $31 = (($30) + 4|0);
    HEAP32[$31>>2] = $29;
    $32 = HEAP32[$1>>2]|0;
    $33 = (($32) + 32|0);
    $34 = HEAP32[$33>>2]|0;
    HEAP32[$1>>2] = $34;
   }
   $35 = HEAP32[$29>>2]|0;
   HEAP32[$nameToken>>2] = $35;
   $36 = HEAP32[$nameToken>>2]|0;
   $37 = (_sqlite3Strlen30($36)|0);
   $38 = (($nameToken) + 4|0);
   HEAP32[$38>>2] = $37;
   $39 = HEAP32[$0>>2]|0;
   $40 = HEAP32[$iDb>>2]|0;
   _sqlite3FixInit($sFix,$39,$40,1488,$nameToken);
   $41 = HEAP32[$pTrig>>2]|0;
   $42 = (($41) + 28|0);
   $43 = HEAP32[$42>>2]|0;
   $44 = (_sqlite3FixTriggerStep($sFix,$43)|0);
   $45 = ($44|0)!=(0);
   if (!($45)) {
    $46 = HEAP32[$pTrig>>2]|0;
    $47 = (($46) + 12|0);
    $48 = HEAP32[$47>>2]|0;
    $49 = (_sqlite3FixExpr($sFix,$48)|0);
    $50 = ($49|0)!=(0);
    if (!($50)) {
     $51 = HEAP32[$db>>2]|0;
     $52 = (($51) + 140|0);
     $53 = (($52) + 5|0);
     $54 = HEAP8[$53>>0]|0;
     $55 = ($54<<24>>24)!=(0);
     if (!($55)) {
      $56 = HEAP32[$0>>2]|0;
      $57 = (_sqlite3GetVdbe($56)|0);
      HEAP32[$v>>2] = $57;
      $58 = HEAP32[$v>>2]|0;
      $59 = ($58|0)==(0|0);
      if ($59) {
       break;
      }
      $60 = HEAP32[$0>>2]|0;
      $61 = HEAP32[$iDb>>2]|0;
      _sqlite3BeginWriteOperation($60,0,$61);
      $62 = HEAP32[$db>>2]|0;
      $63 = HEAP32[$2>>2]|0;
      $64 = HEAP32[$63>>2]|0;
      $65 = HEAP32[$2>>2]|0;
      $66 = (($65) + 4|0);
      $67 = HEAP32[$66>>2]|0;
      $68 = (_sqlite3DbStrNDup($62,$64,$67,0)|0);
      HEAP32[$z>>2] = $68;
      $69 = HEAP32[$0>>2]|0;
      $70 = HEAP32[$iDb>>2]|0;
      $71 = HEAP32[$db>>2]|0;
      $72 = (($71) + 16|0);
      $73 = HEAP32[$72>>2]|0;
      $74 = (($73) + ($70<<4)|0);
      $75 = HEAP32[$74>>2]|0;
      $76 = HEAP32[$iDb>>2]|0;
      $77 = ($76|0)==(1);
      $78 = $77 ? 14904 : 14928;
      $79 = HEAP32[$zName>>2]|0;
      $80 = HEAP32[$pTrig>>2]|0;
      $81 = (($80) + 4|0);
      $82 = HEAP32[$81>>2]|0;
      $83 = HEAP32[$z>>2]|0;
      HEAP32[$vararg_buffer6>>2] = $75;
      $vararg_ptr2 = (($vararg_buffer6) + 4|0);
      HEAP32[$vararg_ptr2>>2] = $78;
      $vararg_ptr3 = (($vararg_buffer6) + 8|0);
      HEAP32[$vararg_ptr3>>2] = $79;
      $vararg_ptr4 = (($vararg_buffer6) + 12|0);
      HEAP32[$vararg_ptr4>>2] = $82;
      $vararg_ptr5 = (($vararg_buffer6) + 16|0);
      HEAP32[$vararg_ptr5>>2] = $83;
      _sqlite3NestedParse($69,20200,$vararg_buffer6);
      $84 = HEAP32[$db>>2]|0;
      $85 = HEAP32[$z>>2]|0;
      _sqlite3DbFree($84,$85);
      $86 = HEAP32[$0>>2]|0;
      $87 = HEAP32[$iDb>>2]|0;
      _sqlite3ChangeCookie($86,$87);
      $88 = HEAP32[$v>>2]|0;
      $89 = HEAP32[$iDb>>2]|0;
      $90 = HEAP32[$db>>2]|0;
      $91 = HEAP32[$zName>>2]|0;
      HEAP32[$vararg_buffer6>>2] = $91;
      $92 = (_sqlite3MPrintf($90,20264,$vararg_buffer6)|0);
      _sqlite3VdbeAddParseSchemaOp($88,$89,$92);
     }
     $93 = HEAP32[$db>>2]|0;
     $94 = (($93) + 140|0);
     $95 = (($94) + 5|0);
     $96 = HEAP8[$95>>0]|0;
     $97 = ($96<<24>>24)!=(0);
     if ($97) {
      $98 = HEAP32[$pTrig>>2]|0;
      HEAP32[$pLink>>2] = $98;
      $99 = HEAP32[$iDb>>2]|0;
      $100 = HEAP32[$db>>2]|0;
      $101 = (($100) + 16|0);
      $102 = HEAP32[$101>>2]|0;
      $103 = (($102) + ($99<<4)|0);
      $104 = (($103) + 12|0);
      $105 = HEAP32[$104>>2]|0;
      $106 = (($105) + 40|0);
      HEAP32[$pHash>>2] = $106;
      $107 = HEAP32[$pHash>>2]|0;
      $108 = HEAP32[$zName>>2]|0;
      $109 = HEAP32[$pTrig>>2]|0;
      $110 = (_sqlite3HashInsert($107,$108,$109)|0);
      HEAP32[$pTrig>>2] = $110;
      $111 = HEAP32[$pTrig>>2]|0;
      $112 = ($111|0)!=(0|0);
      if ($112) {
       $113 = HEAP32[$db>>2]|0;
       $114 = (($113) + 64|0);
       HEAP8[$114>>0] = 1;
       break;
      }
      $115 = HEAP32[$pLink>>2]|0;
      $116 = (($115) + 20|0);
      $117 = HEAP32[$116>>2]|0;
      $118 = HEAP32[$pLink>>2]|0;
      $119 = (($118) + 24|0);
      $120 = HEAP32[$119>>2]|0;
      $121 = ($117|0)==($120|0);
      if ($121) {
       $122 = HEAP32[$pLink>>2]|0;
       $123 = (($122) + 24|0);
       $124 = HEAP32[$123>>2]|0;
       $125 = (($124) + 8|0);
       $126 = HEAP32[$pLink>>2]|0;
       $127 = (($126) + 4|0);
       $128 = HEAP32[$127>>2]|0;
       $129 = (_sqlite3HashFind($125,$128)|0);
       HEAP32[$pTab>>2] = $129;
       $130 = HEAP32[$pTab>>2]|0;
       $131 = (($130) + 64|0);
       $132 = HEAP32[$131>>2]|0;
       $133 = HEAP32[$pLink>>2]|0;
       $134 = (($133) + 32|0);
       HEAP32[$134>>2] = $132;
       $135 = HEAP32[$pLink>>2]|0;
       $136 = HEAP32[$pTab>>2]|0;
       $137 = (($136) + 64|0);
       HEAP32[$137>>2] = $135;
      }
     }
    }
   }
  }
 } while(0);
 $138 = HEAP32[$db>>2]|0;
 $139 = HEAP32[$pTrig>>2]|0;
 _sqlite3DeleteTrigger($138,$139);
 $140 = HEAP32[$db>>2]|0;
 $141 = HEAP32[$1>>2]|0;
 _sqlite3DeleteTriggerStep($140,$141);
 STACKTOP = sp;return;
}
function _sqlite3BeginTrigger($pParse,$pName1,$pName2,$tr_tm,$op,$pColumns,$pTableName,$pWhen,$isTemp,$noErr) {
 $pParse = $pParse|0;
 $pName1 = $pName1|0;
 $pName2 = $pName2|0;
 $tr_tm = $tr_tm|0;
 $op = $op|0;
 $pColumns = $pColumns|0;
 $pTableName = $pTableName|0;
 $pWhen = $pWhen|0;
 $isTemp = $isTemp|0;
 $noErr = $noErr|0;
 var $$old = 0, $$old10 = 0, $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0;
 var $114 = 0, $115 = 0, $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0;
 var $132 = 0, $133 = 0, $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0;
 var $150 = 0, $151 = 0, $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0;
 var $169 = 0, $17 = 0, $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0;
 var $187 = 0, $188 = 0, $189 = 0, $19 = 0, $190 = 0, $191 = 0, $192 = 0, $193 = 0, $194 = 0, $195 = 0, $196 = 0, $197 = 0, $198 = 0, $199 = 0, $2 = 0, $20 = 0, $200 = 0, $201 = 0, $202 = 0, $203 = 0;
 var $204 = 0, $205 = 0, $206 = 0, $207 = 0, $208 = 0, $209 = 0, $21 = 0, $210 = 0, $211 = 0, $212 = 0, $213 = 0, $214 = 0, $215 = 0, $216 = 0, $217 = 0, $218 = 0, $219 = 0, $22 = 0, $220 = 0, $221 = 0;
 var $222 = 0, $223 = 0, $224 = 0, $225 = 0, $226 = 0, $227 = 0, $228 = 0, $229 = 0, $23 = 0, $230 = 0, $231 = 0, $232 = 0, $233 = 0, $234 = 0, $235 = 0, $236 = 0, $237 = 0, $238 = 0, $239 = 0, $24 = 0;
 var $240 = 0, $241 = 0, $242 = 0, $243 = 0, $244 = 0, $245 = 0, $246 = 0, $247 = 0, $248 = 0, $249 = 0, $25 = 0, $250 = 0, $251 = 0, $252 = 0, $253 = 0, $254 = 0, $255 = 0, $256 = 0, $257 = 0, $258 = 0;
 var $259 = 0, $26 = 0, $260 = 0, $261 = 0, $262 = 0, $263 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0;
 var $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0;
 var $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0;
 var $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0;
 var $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $code = 0, $db = 0, $iDb = 0, $iTabDb = 0, $or$cond = 0, $or$cond11 = 0, $or$cond3 = 0, $or$cond5 = 0, $or$cond7 = 0, $or$cond9 = 0, $pName = 0, $pTab = 0, $pTrigger = 0, $sFix = 0;
 var $vararg_buffer23 = 0, $vararg_ptr21 = 0, $vararg_ptr22 = 0, $vararg_ptr26 = 0, $zDb = 0, $zDbTrig = 0, $zName = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 128|0;
 $vararg_buffer23 = sp;
 $0 = sp + 116|0;
 $1 = sp + 112|0;
 $2 = sp + 108|0;
 $3 = sp + 104|0;
 $4 = sp + 100|0;
 $5 = sp + 96|0;
 $6 = sp + 92|0;
 $7 = sp + 88|0;
 $8 = sp + 84|0;
 $9 = sp + 80|0;
 $pTrigger = sp + 76|0;
 $pTab = sp + 72|0;
 $zName = sp + 68|0;
 $db = sp + 64|0;
 $iDb = sp + 60|0;
 $pName = sp + 56|0;
 $sFix = sp + 32|0;
 $iTabDb = sp + 24|0;
 $code = sp + 20|0;
 $zDb = sp + 16|0;
 $zDbTrig = sp + 12|0;
 HEAP32[$0>>2] = $pParse;
 HEAP32[$1>>2] = $pName1;
 HEAP32[$2>>2] = $pName2;
 HEAP32[$3>>2] = $tr_tm;
 HEAP32[$4>>2] = $op;
 HEAP32[$5>>2] = $pColumns;
 HEAP32[$6>>2] = $pTableName;
 HEAP32[$7>>2] = $pWhen;
 HEAP32[$8>>2] = $isTemp;
 HEAP32[$9>>2] = $noErr;
 HEAP32[$pTrigger>>2] = 0;
 HEAP32[$zName>>2] = 0;
 $10 = HEAP32[$0>>2]|0;
 $11 = HEAP32[$10>>2]|0;
 HEAP32[$db>>2] = $11;
 $12 = HEAP32[$8>>2]|0;
 $13 = ($12|0)!=(0);
 do {
  if ($13) {
   $14 = HEAP32[$2>>2]|0;
   $15 = (($14) + 4|0);
   $16 = HEAP32[$15>>2]|0;
   $17 = ($16>>>0)>(0);
   if ($17) {
    $18 = HEAP32[$0>>2]|0;
    _sqlite3ErrorMsg($18,19856,$vararg_buffer23);
    break;
   } else {
    HEAP32[$iDb>>2] = 1;
    $19 = HEAP32[$1>>2]|0;
    HEAP32[$pName>>2] = $19;
    $$old = HEAP32[$6>>2]|0;
    $$old10 = ($$old|0)!=(0|0);
    if ($$old10) {
     label = 6;
     break;
    } else {
     break;
    }
   }
  } else {
   $20 = HEAP32[$0>>2]|0;
   $21 = HEAP32[$1>>2]|0;
   $22 = HEAP32[$2>>2]|0;
   $23 = (_sqlite3TwoPartName($20,$21,$22,$pName)|0);
   HEAP32[$iDb>>2] = $23;
   $24 = HEAP32[$iDb>>2]|0;
   $25 = ($24|0)>=(0);
   $26 = HEAP32[$6>>2]|0;
   $27 = ($26|0)!=(0|0);
   $or$cond11 = $25 & $27;
   if ($or$cond11) {
    label = 6;
   }
  }
 } while(0);
 do {
  if ((label|0) == 6) {
   $28 = HEAP32[$db>>2]|0;
   $29 = (($28) + 64|0);
   $30 = HEAP8[$29>>0]|0;
   $31 = $30&255;
   $32 = ($31|0)!=(0);
   if (!($32)) {
    $33 = HEAP32[$db>>2]|0;
    $34 = (($33) + 140|0);
    $35 = (($34) + 5|0);
    $36 = HEAP8[$35>>0]|0;
    $37 = $36&255;
    $38 = ($37|0)!=(0);
    $39 = HEAP32[$iDb>>2]|0;
    $40 = ($39|0)!=(1);
    $or$cond = $38 & $40;
    if ($or$cond) {
     $41 = HEAP32[$db>>2]|0;
     $42 = HEAP32[$6>>2]|0;
     $43 = (($42) + 8|0);
     $44 = (($43) + 4|0);
     $45 = HEAP32[$44>>2]|0;
     _sqlite3DbFree($41,$45);
     $46 = HEAP32[$6>>2]|0;
     $47 = (($46) + 8|0);
     $48 = (($47) + 4|0);
     HEAP32[$48>>2] = 0;
    }
    $49 = HEAP32[$0>>2]|0;
    $50 = HEAP32[$6>>2]|0;
    $51 = (_sqlite3SrcListLookup($49,$50)|0);
    HEAP32[$pTab>>2] = $51;
    $52 = HEAP32[$db>>2]|0;
    $53 = (($52) + 140|0);
    $54 = (($53) + 5|0);
    $55 = HEAP8[$54>>0]|0;
    $56 = $55&255;
    $57 = ($56|0)==(0);
    if ($57) {
     $58 = HEAP32[$2>>2]|0;
     $59 = (($58) + 4|0);
     $60 = HEAP32[$59>>2]|0;
     $61 = ($60|0)==(0);
     $62 = HEAP32[$pTab>>2]|0;
     $63 = ($62|0)!=(0|0);
     $or$cond3 = $61 & $63;
     if ($or$cond3) {
      $64 = HEAP32[$pTab>>2]|0;
      $65 = (($64) + 68|0);
      $66 = HEAP32[$65>>2]|0;
      $67 = HEAP32[$db>>2]|0;
      $68 = (($67) + 16|0);
      $69 = HEAP32[$68>>2]|0;
      $70 = (($69) + 16|0);
      $71 = (($70) + 12|0);
      $72 = HEAP32[$71>>2]|0;
      $73 = ($66|0)==($72|0);
      if ($73) {
       HEAP32[$iDb>>2] = 1;
      }
     }
    }
    $74 = HEAP32[$db>>2]|0;
    $75 = (($74) + 64|0);
    $76 = HEAP8[$75>>0]|0;
    $77 = ($76<<24>>24)!=(0);
    if (!($77)) {
     $78 = HEAP32[$0>>2]|0;
     $79 = HEAP32[$iDb>>2]|0;
     $80 = HEAP32[$pName>>2]|0;
     _sqlite3FixInit($sFix,$78,$79,1488,$80);
     $81 = HEAP32[$6>>2]|0;
     $82 = (_sqlite3FixSrcList($sFix,$81)|0);
     $83 = ($82|0)!=(0);
     if (!($83)) {
      $84 = HEAP32[$0>>2]|0;
      $85 = HEAP32[$6>>2]|0;
      $86 = (_sqlite3SrcListLookup($84,$85)|0);
      HEAP32[$pTab>>2] = $86;
      $87 = HEAP32[$pTab>>2]|0;
      $88 = ($87|0)!=(0|0);
      if (!($88)) {
       $89 = HEAP32[$db>>2]|0;
       $90 = (($89) + 140|0);
       $91 = (($90) + 4|0);
       $92 = HEAP8[$91>>0]|0;
       $93 = $92&255;
       $94 = ($93|0)==(1);
       if (!($94)) {
        break;
       }
       $95 = HEAP32[$db>>2]|0;
       $96 = (($95) + 140|0);
       $97 = (($96) + 6|0);
       HEAP8[$97>>0] = 1;
       break;
      }
      $98 = HEAP32[$pTab>>2]|0;
      $99 = (($98) + 44|0);
      $100 = HEAP8[$99>>0]|0;
      $101 = $100&255;
      $102 = $101 & 16;
      $103 = ($102|0)!=(0);
      if ($103) {
       $104 = HEAP32[$0>>2]|0;
       _sqlite3ErrorMsg($104,19904,$vararg_buffer23);
       break;
      }
      $105 = HEAP32[$db>>2]|0;
      $106 = HEAP32[$pName>>2]|0;
      $107 = (_sqlite3NameFromToken($105,$106)|0);
      HEAP32[$zName>>2] = $107;
      $108 = HEAP32[$zName>>2]|0;
      $109 = ($108|0)!=(0|0);
      if ($109) {
       $110 = HEAP32[$0>>2]|0;
       $111 = HEAP32[$zName>>2]|0;
       $112 = (_sqlite3CheckObjectName($110,$111)|0);
       $113 = (0)!=($112|0);
       if (!($113)) {
        $114 = HEAP32[$iDb>>2]|0;
        $115 = HEAP32[$db>>2]|0;
        $116 = (($115) + 16|0);
        $117 = HEAP32[$116>>2]|0;
        $118 = (($117) + ($114<<4)|0);
        $119 = (($118) + 12|0);
        $120 = HEAP32[$119>>2]|0;
        $121 = (($120) + 40|0);
        $122 = HEAP32[$zName>>2]|0;
        $123 = (_sqlite3HashFind($121,$122)|0);
        $124 = ($123|0)!=(0|0);
        if ($124) {
         $125 = HEAP32[$9>>2]|0;
         $126 = ($125|0)!=(0);
         $127 = HEAP32[$0>>2]|0;
         if ($126) {
          $129 = HEAP32[$iDb>>2]|0;
          _sqlite3CodeVerifySchema($127,$129);
          break;
         } else {
          $128 = HEAP32[$pName>>2]|0;
          HEAP32[$vararg_buffer23>>2] = $128;
          _sqlite3ErrorMsg($127,19952,$vararg_buffer23);
          break;
         }
        }
        $130 = HEAP32[$pTab>>2]|0;
        $131 = HEAP32[$130>>2]|0;
        $132 = (_sqlite3_strnicmp($131,15144,7)|0);
        $133 = ($132|0)==(0);
        if ($133) {
         $134 = HEAP32[$0>>2]|0;
         _sqlite3ErrorMsg($134,19984,$vararg_buffer23);
         $135 = HEAP32[$0>>2]|0;
         $136 = (($135) + 64|0);
         $137 = HEAP32[$136>>2]|0;
         $138 = (($137) + 1)|0;
         HEAP32[$136>>2] = $138;
         break;
        }
        $139 = HEAP32[$pTab>>2]|0;
        $140 = (($139) + 12|0);
        $141 = HEAP32[$140>>2]|0;
        $142 = ($141|0)!=(0|0);
        $143 = HEAP32[$3>>2]|0;
        $144 = ($143|0)!=(49);
        $or$cond5 = $142 & $144;
        if ($or$cond5) {
         $145 = HEAP32[$0>>2]|0;
         $146 = HEAP32[$3>>2]|0;
         $147 = ($146|0)==(35);
         $148 = $147 ? 20064 : 20072;
         $149 = HEAP32[$6>>2]|0;
         HEAP32[$vararg_buffer23>>2] = $148;
         $vararg_ptr21 = (($vararg_buffer23) + 4|0);
         HEAP32[$vararg_ptr21>>2] = $149;
         $vararg_ptr22 = (($vararg_buffer23) + 8|0);
         HEAP32[$vararg_ptr22>>2] = 0;
         _sqlite3ErrorMsg($145,20024,$vararg_buffer23);
         break;
        }
        $150 = HEAP32[$pTab>>2]|0;
        $151 = (($150) + 12|0);
        $152 = HEAP32[$151>>2]|0;
        $153 = ($152|0)==(0|0);
        $154 = HEAP32[$3>>2]|0;
        $155 = ($154|0)==(49);
        $or$cond7 = $153 & $155;
        if ($or$cond7) {
         $156 = HEAP32[$0>>2]|0;
         $157 = HEAP32[$6>>2]|0;
         HEAP32[$vararg_buffer23>>2] = $157;
         $vararg_ptr26 = (($vararg_buffer23) + 4|0);
         HEAP32[$vararg_ptr26>>2] = 0;
         _sqlite3ErrorMsg($156,20080,$vararg_buffer23);
         break;
        }
        $158 = HEAP32[$db>>2]|0;
        $159 = HEAP32[$pTab>>2]|0;
        $160 = (($159) + 68|0);
        $161 = HEAP32[$160>>2]|0;
        $162 = (_sqlite3SchemaToIndex($158,$161)|0);
        HEAP32[$iTabDb>>2] = $162;
        HEAP32[$code>>2] = 7;
        $163 = HEAP32[$iTabDb>>2]|0;
        $164 = HEAP32[$db>>2]|0;
        $165 = (($164) + 16|0);
        $166 = HEAP32[$165>>2]|0;
        $167 = (($166) + ($163<<4)|0);
        $168 = HEAP32[$167>>2]|0;
        HEAP32[$zDb>>2] = $168;
        $169 = HEAP32[$8>>2]|0;
        $170 = ($169|0)!=(0);
        if ($170) {
         $171 = HEAP32[$db>>2]|0;
         $172 = (($171) + 16|0);
         $173 = HEAP32[$172>>2]|0;
         $174 = (($173) + 16|0);
         $175 = HEAP32[$174>>2]|0;
         $177 = $175;
        } else {
         $176 = HEAP32[$zDb>>2]|0;
         $177 = $176;
        }
        HEAP32[$zDbTrig>>2] = $177;
        $178 = HEAP32[$iTabDb>>2]|0;
        $179 = ($178|0)==(1);
        $180 = HEAP32[$8>>2]|0;
        $181 = ($180|0)!=(0);
        $or$cond9 = $179 | $181;
        if ($or$cond9) {
         HEAP32[$code>>2] = 5;
        }
        $182 = HEAP32[$0>>2]|0;
        $183 = HEAP32[$code>>2]|0;
        $184 = HEAP32[$zName>>2]|0;
        $185 = HEAP32[$pTab>>2]|0;
        $186 = HEAP32[$185>>2]|0;
        $187 = HEAP32[$zDbTrig>>2]|0;
        $188 = (_sqlite3AuthCheck($182,$183,$184,$186,$187)|0);
        $189 = ($188|0)!=(0);
        if (!($189)) {
         $190 = HEAP32[$0>>2]|0;
         $191 = HEAP32[$iTabDb>>2]|0;
         $192 = ($191|0)==(1);
         $193 = $192 ? 14904 : 14928;
         $194 = HEAP32[$zDb>>2]|0;
         $195 = (_sqlite3AuthCheck($190,18,$193,0,$194)|0);
         $196 = ($195|0)!=(0);
         if (!($196)) {
          $197 = HEAP32[$3>>2]|0;
          $198 = ($197|0)==(49);
          if ($198) {
           HEAP32[$3>>2] = 35;
          }
          $199 = HEAP32[$db>>2]|0;
          $200 = (_sqlite3DbMallocZero($199,36,0)|0);
          HEAP32[$pTrigger>>2] = $200;
          $201 = HEAP32[$pTrigger>>2]|0;
          $202 = ($201|0)==(0|0);
          if ($202) {
           break;
          }
          $203 = HEAP32[$zName>>2]|0;
          $204 = HEAP32[$pTrigger>>2]|0;
          HEAP32[$204>>2] = $203;
          HEAP32[$zName>>2] = 0;
          $205 = HEAP32[$db>>2]|0;
          $206 = HEAP32[$6>>2]|0;
          $207 = (($206) + 8|0);
          $208 = (($207) + 8|0);
          $209 = HEAP32[$208>>2]|0;
          $210 = (_sqlite3DbStrDup($205,$209)|0);
          $211 = HEAP32[$pTrigger>>2]|0;
          $212 = (($211) + 4|0);
          HEAP32[$212>>2] = $210;
          $213 = HEAP32[$iDb>>2]|0;
          $214 = HEAP32[$db>>2]|0;
          $215 = (($214) + 16|0);
          $216 = HEAP32[$215>>2]|0;
          $217 = (($216) + ($213<<4)|0);
          $218 = (($217) + 12|0);
          $219 = HEAP32[$218>>2]|0;
          $220 = HEAP32[$pTrigger>>2]|0;
          $221 = (($220) + 20|0);
          HEAP32[$221>>2] = $219;
          $222 = HEAP32[$pTab>>2]|0;
          $223 = (($222) + 68|0);
          $224 = HEAP32[$223>>2]|0;
          $225 = HEAP32[$pTrigger>>2]|0;
          $226 = (($225) + 24|0);
          HEAP32[$226>>2] = $224;
          $227 = HEAP32[$4>>2]|0;
          $228 = $227&255;
          $229 = HEAP32[$pTrigger>>2]|0;
          $230 = (($229) + 8|0);
          HEAP8[$230>>0] = $228;
          $231 = HEAP32[$3>>2]|0;
          $232 = ($231|0)==(35);
          $233 = $232 ? 1 : 2;
          $234 = $233&255;
          $235 = HEAP32[$pTrigger>>2]|0;
          $236 = (($235) + 9|0);
          HEAP8[$236>>0] = $234;
          $237 = HEAP32[$db>>2]|0;
          $238 = HEAP32[$7>>2]|0;
          $239 = (_sqlite3ExprDup($237,$238,1)|0);
          $240 = HEAP32[$pTrigger>>2]|0;
          $241 = (($240) + 12|0);
          HEAP32[$241>>2] = $239;
          $242 = HEAP32[$db>>2]|0;
          $243 = HEAP32[$5>>2]|0;
          $244 = (_sqlite3IdListDup($242,$243)|0);
          $245 = HEAP32[$pTrigger>>2]|0;
          $246 = (($245) + 16|0);
          HEAP32[$246>>2] = $244;
          $247 = HEAP32[$pTrigger>>2]|0;
          $248 = HEAP32[$0>>2]|0;
          $249 = (($248) + 488|0);
          HEAP32[$249>>2] = $247;
         }
        }
       }
      }
     }
    }
   }
  }
 } while(0);
 $250 = HEAP32[$db>>2]|0;
 $251 = HEAP32[$zName>>2]|0;
 _sqlite3DbFree($250,$251);
 $252 = HEAP32[$db>>2]|0;
 $253 = HEAP32[$6>>2]|0;
 _sqlite3SrcListDelete($252,$253);
 $254 = HEAP32[$db>>2]|0;
 $255 = HEAP32[$5>>2]|0;
 _sqlite3IdListDelete($254,$255);
 $256 = HEAP32[$db>>2]|0;
 $257 = HEAP32[$7>>2]|0;
 _sqlite3ExprDelete($256,$257);
 $258 = HEAP32[$0>>2]|0;
 $259 = (($258) + 488|0);
 $260 = HEAP32[$259>>2]|0;
 $261 = ($260|0)!=(0|0);
 if ($261) {
  STACKTOP = sp;return;
 }
 $262 = HEAP32[$db>>2]|0;
 $263 = HEAP32[$pTrigger>>2]|0;
 _sqlite3DeleteTrigger($262,$263);
 STACKTOP = sp;return;
}
function _sqlite3TriggerUpdateStep($db,$pTableName,$pEList,$pWhere,$orconf) {
 $db = $db|0;
 $pTableName = $pTableName|0;
 $pEList = $pEList|0;
 $pWhere = $pWhere|0;
 $orconf = $orconf|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $pTriggerStep = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 16|0;
 $1 = sp + 12|0;
 $2 = sp + 8|0;
 $3 = sp + 4|0;
 $4 = sp + 20|0;
 $pTriggerStep = sp;
 HEAP32[$0>>2] = $db;
 HEAP32[$1>>2] = $pTableName;
 HEAP32[$2>>2] = $pEList;
 HEAP32[$3>>2] = $pWhere;
 HEAP8[$4>>0] = $orconf;
 $5 = HEAP32[$0>>2]|0;
 $6 = HEAP32[$1>>2]|0;
 $7 = (_triggerStepAllocate($5,110,$6)|0);
 HEAP32[$pTriggerStep>>2] = $7;
 $8 = HEAP32[$pTriggerStep>>2]|0;
 $9 = ($8|0)!=(0|0);
 if ($9) {
  $10 = HEAP32[$0>>2]|0;
  $11 = HEAP32[$2>>2]|0;
  $12 = (_sqlite3ExprListDup($10,$11,1)|0);
  $13 = HEAP32[$pTriggerStep>>2]|0;
  $14 = (($13) + 24|0);
  HEAP32[$14>>2] = $12;
  $15 = HEAP32[$0>>2]|0;
  $16 = HEAP32[$3>>2]|0;
  $17 = (_sqlite3ExprDup($15,$16,1)|0);
  $18 = HEAP32[$pTriggerStep>>2]|0;
  $19 = (($18) + 20|0);
  HEAP32[$19>>2] = $17;
  $20 = HEAP8[$4>>0]|0;
  $21 = HEAP32[$pTriggerStep>>2]|0;
  $22 = (($21) + 1|0);
  HEAP8[$22>>0] = $20;
 }
 $23 = HEAP32[$0>>2]|0;
 $24 = HEAP32[$2>>2]|0;
 _sqlite3ExprListDelete($23,$24);
 $25 = HEAP32[$0>>2]|0;
 $26 = HEAP32[$3>>2]|0;
 _sqlite3ExprDelete($25,$26);
 $27 = HEAP32[$pTriggerStep>>2]|0;
 STACKTOP = sp;return ($27|0);
}
function _sqlite3TriggerInsertStep($db,$pTableName,$pColumn,$pSelect,$orconf) {
 $db = $db|0;
 $pTableName = $pTableName|0;
 $pColumn = $pColumn|0;
 $pSelect = $pSelect|0;
 $orconf = $orconf|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $3 = 0, $4 = 0;
 var $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $pTriggerStep = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 16|0;
 $1 = sp + 12|0;
 $2 = sp + 8|0;
 $3 = sp + 4|0;
 $4 = sp + 20|0;
 $pTriggerStep = sp;
 HEAP32[$0>>2] = $db;
 HEAP32[$1>>2] = $pTableName;
 HEAP32[$2>>2] = $pColumn;
 HEAP32[$3>>2] = $pSelect;
 HEAP8[$4>>0] = $orconf;
 $5 = HEAP32[$0>>2]|0;
 $6 = HEAP32[$1>>2]|0;
 $7 = (_triggerStepAllocate($5,108,$6)|0);
 HEAP32[$pTriggerStep>>2] = $7;
 $8 = HEAP32[$pTriggerStep>>2]|0;
 $9 = ($8|0)!=(0|0);
 $10 = HEAP32[$0>>2]|0;
 if ($9) {
  $11 = HEAP32[$3>>2]|0;
  $12 = (_sqlite3SelectDup($10,$11,1)|0);
  $13 = HEAP32[$pTriggerStep>>2]|0;
  $14 = (($13) + 8|0);
  HEAP32[$14>>2] = $12;
  $15 = HEAP32[$2>>2]|0;
  $16 = HEAP32[$pTriggerStep>>2]|0;
  $17 = (($16) + 28|0);
  HEAP32[$17>>2] = $15;
  $18 = HEAP8[$4>>0]|0;
  $19 = HEAP32[$pTriggerStep>>2]|0;
  $20 = (($19) + 1|0);
  HEAP8[$20>>0] = $18;
  $22 = HEAP32[$0>>2]|0;
  $23 = HEAP32[$3>>2]|0;
  _sqlite3SelectDelete($22,$23);
  $24 = HEAP32[$pTriggerStep>>2]|0;
  STACKTOP = sp;return ($24|0);
 } else {
  $21 = HEAP32[$2>>2]|0;
  _sqlite3IdListDelete($10,$21);
  $22 = HEAP32[$0>>2]|0;
  $23 = HEAP32[$3>>2]|0;
  _sqlite3SelectDelete($22,$23);
  $24 = HEAP32[$pTriggerStep>>2]|0;
  STACKTOP = sp;return ($24|0);
 }
 return 0|0;
}
function _sqlite3TriggerDeleteStep($db,$pTableName,$pWhere) {
 $db = $db|0;
 $pTableName = $pTableName|0;
 $pWhere = $pWhere|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $pTriggerStep = 0, label = 0;
 var sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $1 = sp + 8|0;
 $2 = sp + 4|0;
 $pTriggerStep = sp;
 HEAP32[$0>>2] = $db;
 HEAP32[$1>>2] = $pTableName;
 HEAP32[$2>>2] = $pWhere;
 $3 = HEAP32[$0>>2]|0;
 $4 = HEAP32[$1>>2]|0;
 $5 = (_triggerStepAllocate($3,109,$4)|0);
 HEAP32[$pTriggerStep>>2] = $5;
 $6 = HEAP32[$pTriggerStep>>2]|0;
 $7 = ($6|0)!=(0|0);
 if ($7) {
  $8 = HEAP32[$0>>2]|0;
  $9 = HEAP32[$2>>2]|0;
  $10 = (_sqlite3ExprDup($8,$9,1)|0);
  $11 = HEAP32[$pTriggerStep>>2]|0;
  $12 = (($11) + 20|0);
  HEAP32[$12>>2] = $10;
  $13 = HEAP32[$pTriggerStep>>2]|0;
  $14 = (($13) + 1|0);
  HEAP8[$14>>0] = 10;
 }
 $15 = HEAP32[$0>>2]|0;
 $16 = HEAP32[$2>>2]|0;
 _sqlite3ExprDelete($15,$16);
 $17 = HEAP32[$pTriggerStep>>2]|0;
 STACKTOP = sp;return ($17|0);
}
function _sqlite3TriggerSelectStep($db,$pSelect) {
 $db = $db|0;
 $pSelect = $pSelect|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $pTriggerStep = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $1 = sp + 8|0;
 $2 = sp + 4|0;
 $pTriggerStep = sp;
 HEAP32[$1>>2] = $db;
 HEAP32[$2>>2] = $pSelect;
 $3 = HEAP32[$1>>2]|0;
 $4 = (_sqlite3DbMallocZero($3,40,0)|0);
 HEAP32[$pTriggerStep>>2] = $4;
 $5 = HEAP32[$pTriggerStep>>2]|0;
 $6 = ($5|0)==(0|0);
 if ($6) {
  $7 = HEAP32[$1>>2]|0;
  $8 = HEAP32[$2>>2]|0;
  _sqlite3SelectDelete($7,$8);
  HEAP32[$0>>2] = 0;
  $16 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($16|0);
 } else {
  $9 = HEAP32[$pTriggerStep>>2]|0;
  HEAP8[$9>>0] = 119;
  $10 = HEAP32[$2>>2]|0;
  $11 = HEAP32[$pTriggerStep>>2]|0;
  $12 = (($11) + 8|0);
  HEAP32[$12>>2] = $10;
  $13 = HEAP32[$pTriggerStep>>2]|0;
  $14 = (($13) + 1|0);
  HEAP8[$14>>0] = 10;
  $15 = HEAP32[$pTriggerStep>>2]|0;
  HEAP32[$0>>2] = $15;
  $16 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($16|0);
 }
 return 0|0;
}
function _sqlite3DropTrigger($pParse,$pName,$noErr) {
 $pParse = $pParse|0;
 $pName = $pName|0;
 $noErr = $noErr|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $7 = 0, $8 = 0, $9 = 0, $db = 0, $i = 0, $j = 0, $pTrigger = 0, $vararg_buffer = 0, $vararg_ptr1 = 0, $zDb = 0, $zName = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $vararg_buffer = sp;
 $0 = sp + 40|0;
 $1 = sp + 36|0;
 $2 = sp + 32|0;
 $pTrigger = sp + 28|0;
 $i = sp + 24|0;
 $zDb = sp + 20|0;
 $zName = sp + 16|0;
 $db = sp + 12|0;
 $j = sp + 8|0;
 HEAP32[$0>>2] = $pParse;
 HEAP32[$1>>2] = $pName;
 HEAP32[$2>>2] = $noErr;
 HEAP32[$pTrigger>>2] = 0;
 $3 = HEAP32[$0>>2]|0;
 $4 = HEAP32[$3>>2]|0;
 HEAP32[$db>>2] = $4;
 $5 = HEAP32[$db>>2]|0;
 $6 = (($5) + 64|0);
 $7 = HEAP8[$6>>0]|0;
 $8 = ($7<<24>>24)!=(0);
 if ($8) {
  $66 = HEAP32[$db>>2]|0;
  $67 = HEAP32[$1>>2]|0;
  _sqlite3SrcListDelete($66,$67);
  STACKTOP = sp;return;
 }
 $9 = HEAP32[$0>>2]|0;
 $10 = (_sqlite3ReadSchema($9)|0);
 $11 = (0)!=($10|0);
 if ($11) {
  $66 = HEAP32[$db>>2]|0;
  $67 = HEAP32[$1>>2]|0;
  _sqlite3SrcListDelete($66,$67);
  STACKTOP = sp;return;
 }
 $12 = HEAP32[$1>>2]|0;
 $13 = (($12) + 8|0);
 $14 = (($13) + 4|0);
 $15 = HEAP32[$14>>2]|0;
 HEAP32[$zDb>>2] = $15;
 $16 = HEAP32[$1>>2]|0;
 $17 = (($16) + 8|0);
 $18 = (($17) + 8|0);
 $19 = HEAP32[$18>>2]|0;
 HEAP32[$zName>>2] = $19;
 HEAP32[$i>>2] = 0;
 while(1) {
  $20 = HEAP32[$i>>2]|0;
  $21 = HEAP32[$db>>2]|0;
  $22 = (($21) + 20|0);
  $23 = HEAP32[$22>>2]|0;
  $24 = ($20|0)<($23|0);
  if (!($24)) {
   break;
  }
  $25 = HEAP32[$i>>2]|0;
  $26 = ($25|0)<(2);
  $27 = HEAP32[$i>>2]|0;
  $28 = $27 ^ 1;
  $29 = $26 ? $28 : $27;
  HEAP32[$j>>2] = $29;
  $30 = HEAP32[$zDb>>2]|0;
  $31 = ($30|0)!=(0|0);
  if ($31) {
   $32 = HEAP32[$j>>2]|0;
   $33 = HEAP32[$db>>2]|0;
   $34 = (($33) + 16|0);
   $35 = HEAP32[$34>>2]|0;
   $36 = (($35) + ($32<<4)|0);
   $37 = HEAP32[$36>>2]|0;
   $38 = HEAP32[$zDb>>2]|0;
   $39 = (_sqlite3_stricmp($37,$38)|0);
   $40 = ($39|0)!=(0);
   if (!($40)) {
    label = 7;
   }
  } else {
   label = 7;
  }
  if ((label|0) == 7) {
   label = 0;
   $41 = HEAP32[$j>>2]|0;
   $42 = HEAP32[$db>>2]|0;
   $43 = (($42) + 16|0);
   $44 = HEAP32[$43>>2]|0;
   $45 = (($44) + ($41<<4)|0);
   $46 = (($45) + 12|0);
   $47 = HEAP32[$46>>2]|0;
   $48 = (($47) + 40|0);
   $49 = HEAP32[$zName>>2]|0;
   $50 = (_sqlite3HashFind($48,$49)|0);
   HEAP32[$pTrigger>>2] = $50;
   $51 = HEAP32[$pTrigger>>2]|0;
   $52 = ($51|0)!=(0|0);
   if ($52) {
    break;
   }
  }
  $53 = HEAP32[$i>>2]|0;
  $54 = (($53) + 1)|0;
  HEAP32[$i>>2] = $54;
 }
 $55 = HEAP32[$pTrigger>>2]|0;
 $56 = ($55|0)!=(0|0);
 if ($56) {
  $64 = HEAP32[$0>>2]|0;
  $65 = HEAP32[$pTrigger>>2]|0;
  _sqlite3DropTriggerPtr($64,$65);
  $66 = HEAP32[$db>>2]|0;
  $67 = HEAP32[$1>>2]|0;
  _sqlite3SrcListDelete($66,$67);
  STACKTOP = sp;return;
 }
 $57 = HEAP32[$2>>2]|0;
 $58 = ($57|0)!=(0);
 $59 = HEAP32[$0>>2]|0;
 if ($58) {
  $61 = HEAP32[$zDb>>2]|0;
  _sqlite3CodeVerifyNamedSchema($59,$61);
 } else {
  $60 = HEAP32[$1>>2]|0;
  HEAP32[$vararg_buffer>>2] = $60;
  $vararg_ptr1 = (($vararg_buffer) + 4|0);
  HEAP32[$vararg_ptr1>>2] = 0;
  _sqlite3ErrorMsg($59,19792,$vararg_buffer);
 }
 $62 = HEAP32[$0>>2]|0;
 $63 = (($62) + 17|0);
 HEAP8[$63>>0] = 1;
 $66 = HEAP32[$db>>2]|0;
 $67 = HEAP32[$1>>2]|0;
 _sqlite3SrcListDelete($66,$67);
 STACKTOP = sp;return;
}
function _sqlite3Attach($pParse,$p,$pDbname,$pKey) {
 $pParse = $pParse|0;
 $p = $p|0;
 $pDbname = $pDbname|0;
 $pKey = $pKey|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $1 = sp + 8|0;
 $2 = sp + 4|0;
 $3 = sp;
 HEAP32[$0>>2] = $pParse;
 HEAP32[$1>>2] = $p;
 HEAP32[$2>>2] = $pDbname;
 HEAP32[$3>>2] = $pKey;
 $4 = HEAP32[$0>>2]|0;
 $5 = HEAP32[$1>>2]|0;
 $6 = HEAP32[$1>>2]|0;
 $7 = HEAP32[$2>>2]|0;
 $8 = HEAP32[$3>>2]|0;
 _codeAttach($4,24,19552,$5,$6,$7,$8);
 STACKTOP = sp;return;
}
function _sqlite3Detach($pParse,$pDbname) {
 $pParse = $pParse|0;
 $pDbname = $pDbname|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $1 = sp;
 HEAP32[$0>>2] = $pParse;
 HEAP32[$1>>2] = $pDbname;
 $2 = HEAP32[$0>>2]|0;
 $3 = HEAP32[$1>>2]|0;
 $4 = HEAP32[$1>>2]|0;
 _codeAttach($2,25,19368,$3,0,0,$4);
 STACKTOP = sp;return;
}
function _sqlite3Reindex($pParse,$pName1,$pName2) {
 $pParse = $pParse|0;
 $pName1 = $pName1|0;
 $pName2 = $pName2|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $8 = 0, $9 = 0, $db = 0;
 var $iDb = 0, $pColl = 0, $pIndex = 0, $pObjName = 0, $pTab = 0, $vararg_buffer = 0, $z = 0, $zColl = 0, $zDb = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 64|0;
 $vararg_buffer = sp;
 $0 = sp + 48|0;
 $1 = sp + 44|0;
 $2 = sp + 40|0;
 $pColl = sp + 36|0;
 $z = sp + 32|0;
 $zDb = sp + 28|0;
 $pTab = sp + 24|0;
 $pIndex = sp + 20|0;
 $iDb = sp + 16|0;
 $db = sp + 12|0;
 $pObjName = sp + 8|0;
 $zColl = sp + 4|0;
 HEAP32[$0>>2] = $pParse;
 HEAP32[$1>>2] = $pName1;
 HEAP32[$2>>2] = $pName2;
 $3 = HEAP32[$0>>2]|0;
 $4 = HEAP32[$3>>2]|0;
 HEAP32[$db>>2] = $4;
 $5 = HEAP32[$0>>2]|0;
 $6 = (_sqlite3ReadSchema($5)|0);
 $7 = (0)!=($6|0);
 if ($7) {
  STACKTOP = sp;return;
 }
 $8 = HEAP32[$1>>2]|0;
 $9 = ($8|0)==(0|0);
 if ($9) {
  $10 = HEAP32[$0>>2]|0;
  _reindexDatabases($10,0);
  STACKTOP = sp;return;
 }
 $11 = HEAP32[$2>>2]|0;
 $12 = ($11|0)==(0|0);
 if ($12) {
  label = 6;
 } else {
  $13 = HEAP32[$2>>2]|0;
  $14 = HEAP32[$13>>2]|0;
  $15 = ($14|0)==(0|0);
  if ($15) {
   label = 6;
  }
 }
 do {
  if ((label|0) == 6) {
   $16 = HEAP32[$0>>2]|0;
   $17 = HEAP32[$16>>2]|0;
   $18 = HEAP32[$1>>2]|0;
   $19 = (_sqlite3NameFromToken($17,$18)|0);
   HEAP32[$zColl>>2] = $19;
   $20 = HEAP32[$zColl>>2]|0;
   $21 = ($20|0)!=(0|0);
   if (!($21)) {
    STACKTOP = sp;return;
   }
   $22 = HEAP32[$db>>2]|0;
   $23 = HEAP32[$db>>2]|0;
   $24 = (($23) + 16|0);
   $25 = HEAP32[$24>>2]|0;
   $26 = (($25) + 12|0);
   $27 = HEAP32[$26>>2]|0;
   $28 = (($27) + 77|0);
   $29 = HEAP8[$28>>0]|0;
   $30 = HEAP32[$zColl>>2]|0;
   $31 = (_sqlite3FindCollSeq($22,$29,$30,0)|0);
   HEAP32[$pColl>>2] = $31;
   $32 = HEAP32[$pColl>>2]|0;
   $33 = ($32|0)!=(0|0);
   if (!($33)) {
    $38 = HEAP32[$db>>2]|0;
    $39 = HEAP32[$zColl>>2]|0;
    _sqlite3DbFree($38,$39);
    break;
   }
   $34 = HEAP32[$0>>2]|0;
   $35 = HEAP32[$zColl>>2]|0;
   _reindexDatabases($34,$35);
   $36 = HEAP32[$db>>2]|0;
   $37 = HEAP32[$zColl>>2]|0;
   _sqlite3DbFree($36,$37);
   STACKTOP = sp;return;
  }
 } while(0);
 $40 = HEAP32[$0>>2]|0;
 $41 = HEAP32[$1>>2]|0;
 $42 = HEAP32[$2>>2]|0;
 $43 = (_sqlite3TwoPartName($40,$41,$42,$pObjName)|0);
 HEAP32[$iDb>>2] = $43;
 $44 = HEAP32[$iDb>>2]|0;
 $45 = ($44|0)<(0);
 if ($45) {
  STACKTOP = sp;return;
 }
 $46 = HEAP32[$db>>2]|0;
 $47 = HEAP32[$pObjName>>2]|0;
 $48 = (_sqlite3NameFromToken($46,$47)|0);
 HEAP32[$z>>2] = $48;
 $49 = HEAP32[$z>>2]|0;
 $50 = ($49|0)==(0|0);
 if ($50) {
  STACKTOP = sp;return;
 }
 $51 = HEAP32[$iDb>>2]|0;
 $52 = HEAP32[$db>>2]|0;
 $53 = (($52) + 16|0);
 $54 = HEAP32[$53>>2]|0;
 $55 = (($54) + ($51<<4)|0);
 $56 = HEAP32[$55>>2]|0;
 HEAP32[$zDb>>2] = $56;
 $57 = HEAP32[$db>>2]|0;
 $58 = HEAP32[$z>>2]|0;
 $59 = HEAP32[$zDb>>2]|0;
 $60 = (_sqlite3FindTable($57,$58,$59)|0);
 HEAP32[$pTab>>2] = $60;
 $61 = HEAP32[$pTab>>2]|0;
 $62 = ($61|0)!=(0|0);
 if ($62) {
  $63 = HEAP32[$0>>2]|0;
  $64 = HEAP32[$pTab>>2]|0;
  _reindexTable($63,$64,0);
  $65 = HEAP32[$db>>2]|0;
  $66 = HEAP32[$z>>2]|0;
  _sqlite3DbFree($65,$66);
  STACKTOP = sp;return;
 }
 $67 = HEAP32[$db>>2]|0;
 $68 = HEAP32[$z>>2]|0;
 $69 = HEAP32[$zDb>>2]|0;
 $70 = (_sqlite3FindIndex($67,$68,$69)|0);
 HEAP32[$pIndex>>2] = $70;
 $71 = HEAP32[$db>>2]|0;
 $72 = HEAP32[$z>>2]|0;
 _sqlite3DbFree($71,$72);
 $73 = HEAP32[$pIndex>>2]|0;
 $74 = ($73|0)!=(0|0);
 $75 = HEAP32[$0>>2]|0;
 if ($74) {
  $76 = HEAP32[$iDb>>2]|0;
  _sqlite3BeginWriteOperation($75,0,$76);
  $77 = HEAP32[$0>>2]|0;
  $78 = HEAP32[$pIndex>>2]|0;
  _sqlite3RefillIndex($77,$78,-1);
  STACKTOP = sp;return;
 } else {
  _sqlite3ErrorMsg($75,19064,$vararg_buffer);
  STACKTOP = sp;return;
 }
}
function _sqlite3Analyze($pParse,$pName1,$pName2) {
 $pParse = $pParse|0;
 $pName1 = $pName1|0;
 $pName2 = $pName2|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0;
 var $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $db = 0, $i = 0, $iDb = 0, $pIdx = 0, $pTab = 0, $pTableName = 0, $v = 0;
 var $z = 0, $zDb = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $0 = sp + 44|0;
 $1 = sp + 40|0;
 $2 = sp + 36|0;
 $db = sp + 32|0;
 $iDb = sp + 28|0;
 $i = sp + 24|0;
 $z = sp + 20|0;
 $zDb = sp + 16|0;
 $pTab = sp + 12|0;
 $pIdx = sp + 8|0;
 $pTableName = sp + 4|0;
 $v = sp;
 HEAP32[$0>>2] = $pParse;
 HEAP32[$1>>2] = $pName1;
 HEAP32[$2>>2] = $pName2;
 $3 = HEAP32[$0>>2]|0;
 $4 = HEAP32[$3>>2]|0;
 HEAP32[$db>>2] = $4;
 $5 = HEAP32[$0>>2]|0;
 $6 = (_sqlite3ReadSchema($5)|0);
 $7 = (0)!=($6|0);
 if ($7) {
  STACKTOP = sp;return;
 }
 $8 = HEAP32[$1>>2]|0;
 $9 = ($8|0)==(0|0);
 L4: do {
  if ($9) {
   HEAP32[$i>>2] = 0;
   while(1) {
    $10 = HEAP32[$i>>2]|0;
    $11 = HEAP32[$db>>2]|0;
    $12 = (($11) + 20|0);
    $13 = HEAP32[$12>>2]|0;
    $14 = ($10|0)<($13|0);
    if (!($14)) {
     break L4;
    }
    $15 = HEAP32[$i>>2]|0;
    $16 = ($15|0)==(1);
    if (!($16)) {
     $17 = HEAP32[$0>>2]|0;
     $18 = HEAP32[$i>>2]|0;
     _analyzeDatabase($17,$18);
    }
    $19 = HEAP32[$i>>2]|0;
    $20 = (($19) + 1)|0;
    HEAP32[$i>>2] = $20;
   }
  } else {
   $21 = HEAP32[$2>>2]|0;
   $22 = (($21) + 4|0);
   $23 = HEAP32[$22>>2]|0;
   $24 = ($23|0)==(0);
   if (!($24)) {
    $53 = HEAP32[$0>>2]|0;
    $54 = HEAP32[$1>>2]|0;
    $55 = HEAP32[$2>>2]|0;
    $56 = (_sqlite3TwoPartName($53,$54,$55,$pTableName)|0);
    HEAP32[$iDb>>2] = $56;
    $57 = HEAP32[$iDb>>2]|0;
    $58 = ($57|0)>=(0);
    if (!($58)) {
     break;
    }
    $59 = HEAP32[$iDb>>2]|0;
    $60 = HEAP32[$db>>2]|0;
    $61 = (($60) + 16|0);
    $62 = HEAP32[$61>>2]|0;
    $63 = (($62) + ($59<<4)|0);
    $64 = HEAP32[$63>>2]|0;
    HEAP32[$zDb>>2] = $64;
    $65 = HEAP32[$db>>2]|0;
    $66 = HEAP32[$pTableName>>2]|0;
    $67 = (_sqlite3NameFromToken($65,$66)|0);
    HEAP32[$z>>2] = $67;
    $68 = HEAP32[$z>>2]|0;
    $69 = ($68|0)!=(0|0);
    if (!($69)) {
     break;
    }
    $70 = HEAP32[$db>>2]|0;
    $71 = HEAP32[$z>>2]|0;
    $72 = HEAP32[$zDb>>2]|0;
    $73 = (_sqlite3FindIndex($70,$71,$72)|0);
    HEAP32[$pIdx>>2] = $73;
    $74 = ($73|0)!=(0|0);
    $75 = HEAP32[$0>>2]|0;
    if ($74) {
     $76 = HEAP32[$pIdx>>2]|0;
     $77 = (($76) + 12|0);
     $78 = HEAP32[$77>>2]|0;
     $79 = HEAP32[$pIdx>>2]|0;
     _analyzeTable($75,$78,$79);
    } else {
     $80 = HEAP32[$z>>2]|0;
     $81 = HEAP32[$zDb>>2]|0;
     $82 = (_sqlite3LocateTable($75,0,$80,$81)|0);
     HEAP32[$pTab>>2] = $82;
     $83 = ($82|0)!=(0|0);
     if ($83) {
      $84 = HEAP32[$0>>2]|0;
      $85 = HEAP32[$pTab>>2]|0;
      _analyzeTable($84,$85,0);
     }
    }
    $86 = HEAP32[$db>>2]|0;
    $87 = HEAP32[$z>>2]|0;
    _sqlite3DbFree($86,$87);
    break;
   }
   $25 = HEAP32[$db>>2]|0;
   $26 = HEAP32[$1>>2]|0;
   $27 = (_sqlite3FindDb($25,$26)|0);
   HEAP32[$iDb>>2] = $27;
   $28 = HEAP32[$iDb>>2]|0;
   $29 = ($28|0)>=(0);
   if ($29) {
    $30 = HEAP32[$0>>2]|0;
    $31 = HEAP32[$iDb>>2]|0;
    _analyzeDatabase($30,$31);
    break;
   }
   $32 = HEAP32[$db>>2]|0;
   $33 = HEAP32[$1>>2]|0;
   $34 = (_sqlite3NameFromToken($32,$33)|0);
   HEAP32[$z>>2] = $34;
   $35 = HEAP32[$z>>2]|0;
   $36 = ($35|0)!=(0|0);
   if ($36) {
    $37 = HEAP32[$db>>2]|0;
    $38 = HEAP32[$z>>2]|0;
    $39 = (_sqlite3FindIndex($37,$38,0)|0);
    HEAP32[$pIdx>>2] = $39;
    $40 = ($39|0)!=(0|0);
    $41 = HEAP32[$0>>2]|0;
    if ($40) {
     $42 = HEAP32[$pIdx>>2]|0;
     $43 = (($42) + 12|0);
     $44 = HEAP32[$43>>2]|0;
     $45 = HEAP32[$pIdx>>2]|0;
     _analyzeTable($41,$44,$45);
    } else {
     $46 = HEAP32[$z>>2]|0;
     $47 = (_sqlite3LocateTable($41,0,$46,0)|0);
     HEAP32[$pTab>>2] = $47;
     $48 = ($47|0)!=(0|0);
     if ($48) {
      $49 = HEAP32[$0>>2]|0;
      $50 = HEAP32[$pTab>>2]|0;
      _analyzeTable($49,$50,0);
     }
    }
    $51 = HEAP32[$db>>2]|0;
    $52 = HEAP32[$z>>2]|0;
    _sqlite3DbFree($51,$52);
   }
  }
 } while(0);
 $88 = HEAP32[$0>>2]|0;
 $89 = (_sqlite3GetVdbe($88)|0);
 HEAP32[$v>>2] = $89;
 $90 = HEAP32[$v>>2]|0;
 $91 = ($90|0)!=(0|0);
 if (!($91)) {
  STACKTOP = sp;return;
 }
 $92 = HEAP32[$v>>2]|0;
 (_sqlite3VdbeAddOp0($92,142)|0);
 STACKTOP = sp;return;
}
function _sqlite3AlterRenameTable($pParse,$pSrc,$pName) {
 $pParse = $pParse|0;
 $pSrc = $pSrc|0;
 $pName = $pName|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0;
 var $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0, $188 = 0;
 var $189 = 0, $19 = 0, $190 = 0, $191 = 0, $192 = 0, $193 = 0, $194 = 0, $195 = 0, $196 = 0, $197 = 0, $198 = 0, $199 = 0, $2 = 0, $20 = 0, $200 = 0, $201 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0;
 var $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0;
 var $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0;
 var $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0;
 var $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0;
 var $98 = 0, $99 = 0, $db = 0, $i = 0, $iDb = 0, $nTabName = 0, $p = 0, $pFrom = 0, $pTab = 0, $pVTab = 0, $savedDbFlags = 0, $v = 0, $vararg_buffer27 = 0, $vararg_ptr10 = 0, $vararg_ptr14 = 0, $vararg_ptr15 = 0, $vararg_ptr16 = 0, $vararg_ptr17 = 0, $vararg_ptr18 = 0, $vararg_ptr19 = 0;
 var $vararg_ptr20 = 0, $vararg_ptr21 = 0, $vararg_ptr25 = 0, $vararg_ptr26 = 0, $vararg_ptr30 = 0, $vararg_ptr31 = 0, $vararg_ptr7 = 0, $vararg_ptr8 = 0, $vararg_ptr9 = 0, $zDb = 0, $zName = 0, $zTabName = 0, $zWhere = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 112|0;
 $vararg_buffer27 = sp;
 $0 = sp + 100|0;
 $1 = sp + 96|0;
 $2 = sp + 92|0;
 $iDb = sp + 88|0;
 $zDb = sp + 84|0;
 $pTab = sp + 80|0;
 $zName = sp + 76|0;
 $db = sp + 72|0;
 $nTabName = sp + 68|0;
 $zTabName = sp + 64|0;
 $v = sp + 60|0;
 $zWhere = sp + 56|0;
 $pVTab = sp + 52|0;
 $savedDbFlags = sp + 48|0;
 $i = sp + 44|0;
 $p = sp + 40|0;
 $pFrom = sp + 36|0;
 HEAP32[$0>>2] = $pParse;
 HEAP32[$1>>2] = $pSrc;
 HEAP32[$2>>2] = $pName;
 HEAP32[$zName>>2] = 0;
 $3 = HEAP32[$0>>2]|0;
 $4 = HEAP32[$3>>2]|0;
 HEAP32[$db>>2] = $4;
 HEAP32[$zWhere>>2] = 0;
 HEAP32[$pVTab>>2] = 0;
 $5 = HEAP32[$db>>2]|0;
 $6 = (($5) + 24|0);
 $7 = HEAP32[$6>>2]|0;
 HEAP32[$savedDbFlags>>2] = $7;
 $8 = HEAP32[$db>>2]|0;
 $9 = (($8) + 64|0);
 $10 = HEAP8[$9>>0]|0;
 $11 = ($10<<24>>24)!=(0);
 do {
  if (!($11)) {
   $12 = HEAP32[$0>>2]|0;
   $13 = HEAP32[$1>>2]|0;
   $14 = (($13) + 8|0);
   $15 = (_sqlite3LocateTableItem($12,0,$14)|0);
   HEAP32[$pTab>>2] = $15;
   $16 = HEAP32[$pTab>>2]|0;
   $17 = ($16|0)!=(0|0);
   if ($17) {
    $18 = HEAP32[$0>>2]|0;
    $19 = HEAP32[$18>>2]|0;
    $20 = HEAP32[$pTab>>2]|0;
    $21 = (($20) + 68|0);
    $22 = HEAP32[$21>>2]|0;
    $23 = (_sqlite3SchemaToIndex($19,$22)|0);
    HEAP32[$iDb>>2] = $23;
    $24 = HEAP32[$iDb>>2]|0;
    $25 = HEAP32[$db>>2]|0;
    $26 = (($25) + 16|0);
    $27 = HEAP32[$26>>2]|0;
    $28 = (($27) + ($24<<4)|0);
    $29 = HEAP32[$28>>2]|0;
    HEAP32[$zDb>>2] = $29;
    $30 = HEAP32[$db>>2]|0;
    $31 = (($30) + 24|0);
    $32 = HEAP32[$31>>2]|0;
    $33 = $32 | 2097152;
    HEAP32[$31>>2] = $33;
    $34 = HEAP32[$db>>2]|0;
    $35 = HEAP32[$2>>2]|0;
    $36 = (_sqlite3NameFromToken($34,$35)|0);
    HEAP32[$zName>>2] = $36;
    $37 = HEAP32[$zName>>2]|0;
    $38 = ($37|0)!=(0|0);
    if ($38) {
     $39 = HEAP32[$db>>2]|0;
     $40 = HEAP32[$zName>>2]|0;
     $41 = HEAP32[$zDb>>2]|0;
     $42 = (_sqlite3FindTable($39,$40,$41)|0);
     $43 = ($42|0)!=(0|0);
     if (!($43)) {
      $44 = HEAP32[$db>>2]|0;
      $45 = HEAP32[$zName>>2]|0;
      $46 = HEAP32[$zDb>>2]|0;
      $47 = (_sqlite3FindIndex($44,$45,$46)|0);
      $48 = ($47|0)!=(0|0);
      if (!($48)) {
       $51 = HEAP32[$0>>2]|0;
       $52 = HEAP32[$pTab>>2]|0;
       $53 = HEAP32[$52>>2]|0;
       $54 = (_isSystemTable($51,$53)|0);
       $55 = (0)!=($54|0);
       if ($55) {
        break;
       }
       $56 = HEAP32[$0>>2]|0;
       $57 = HEAP32[$zName>>2]|0;
       $58 = (_sqlite3CheckObjectName($56,$57)|0);
       $59 = (0)!=($58|0);
       if ($59) {
        break;
       }
       $60 = HEAP32[$pTab>>2]|0;
       $61 = (($60) + 12|0);
       $62 = HEAP32[$61>>2]|0;
       $63 = ($62|0)!=(0|0);
       $64 = HEAP32[$0>>2]|0;
       if ($63) {
        $65 = HEAP32[$pTab>>2]|0;
        $66 = HEAP32[$65>>2]|0;
        HEAP32[$vararg_buffer27>>2] = $66;
        _sqlite3ErrorMsg($64,15664,$vararg_buffer27);
        break;
       }
       $67 = HEAP32[$zDb>>2]|0;
       $68 = HEAP32[$pTab>>2]|0;
       $69 = HEAP32[$68>>2]|0;
       $70 = (_sqlite3AuthCheck($64,26,$67,$69,0)|0);
       $71 = ($70|0)!=(0);
       if ($71) {
        break;
       }
       $72 = HEAP32[$0>>2]|0;
       $73 = HEAP32[$pTab>>2]|0;
       $74 = (_sqlite3ViewGetColumnNames($72,$73)|0);
       $75 = ($74|0)!=(0);
       if ($75) {
        break;
       }
       $76 = HEAP32[$pTab>>2]|0;
       $77 = (($76) + 44|0);
       $78 = HEAP8[$77>>0]|0;
       $79 = $78&255;
       $80 = $79 & 16;
       $81 = ($80|0)!=(0);
       if ($81) {
        $82 = HEAP32[$db>>2]|0;
        $83 = HEAP32[$pTab>>2]|0;
        $84 = (_sqlite3GetVTable($82,$83)|0);
        HEAP32[$pVTab>>2] = $84;
        $85 = HEAP32[$pVTab>>2]|0;
        $86 = (($85) + 8|0);
        $87 = HEAP32[$86>>2]|0;
        $88 = HEAP32[$87>>2]|0;
        $89 = (($88) + 76|0);
        $90 = HEAP32[$89>>2]|0;
        $91 = ($90|0)==(0|0);
        if ($91) {
         HEAP32[$pVTab>>2] = 0;
        }
       }
       $92 = HEAP32[$0>>2]|0;
       $93 = (_sqlite3GetVdbe($92)|0);
       HEAP32[$v>>2] = $93;
       $94 = HEAP32[$v>>2]|0;
       $95 = ($94|0)==(0|0);
       if ($95) {
        break;
       }
       $96 = HEAP32[$0>>2]|0;
       $97 = HEAP32[$pVTab>>2]|0;
       $98 = ($97|0)!=(0|0);
       $99 = $98&1;
       $100 = HEAP32[$iDb>>2]|0;
       _sqlite3BeginWriteOperation($96,$99,$100);
       $101 = HEAP32[$0>>2]|0;
       $102 = HEAP32[$iDb>>2]|0;
       _sqlite3ChangeCookie($101,$102);
       $103 = HEAP32[$pVTab>>2]|0;
       $104 = ($103|0)!=(0|0);
       if ($104) {
        $105 = HEAP32[$0>>2]|0;
        $106 = (($105) + 72|0);
        $107 = HEAP32[$106>>2]|0;
        $108 = (($107) + 1)|0;
        HEAP32[$106>>2] = $108;
        HEAP32[$i>>2] = $108;
        $109 = HEAP32[$v>>2]|0;
        $110 = HEAP32[$i>>2]|0;
        $111 = HEAP32[$zName>>2]|0;
        (_sqlite3VdbeAddOp4($109,97,0,$110,0,$111,0)|0);
        $112 = HEAP32[$v>>2]|0;
        $113 = HEAP32[$i>>2]|0;
        $114 = HEAP32[$pVTab>>2]|0;
        (_sqlite3VdbeAddOp4($112,150,$113,0,0,$114,-10)|0);
        $115 = HEAP32[$0>>2]|0;
        _sqlite3MayAbort($115);
       }
       $116 = HEAP32[$pTab>>2]|0;
       $117 = HEAP32[$116>>2]|0;
       HEAP32[$zTabName>>2] = $117;
       $118 = HEAP32[$zTabName>>2]|0;
       $119 = (_sqlite3Utf8CharLen($118,-1)|0);
       HEAP32[$nTabName>>2] = $119;
       $120 = HEAP32[$db>>2]|0;
       $121 = (($120) + 24|0);
       $122 = HEAP32[$121>>2]|0;
       $123 = $122 & 524288;
       $124 = ($123|0)!=(0);
       if ($124) {
        $125 = HEAP32[$0>>2]|0;
        $126 = HEAP32[$pTab>>2]|0;
        $127 = (_whereForeignKeys($125,$126)|0);
        HEAP32[$zWhere>>2] = $127;
        $128 = ($127|0)!=(0|0);
        if ($128) {
         $129 = HEAP32[$0>>2]|0;
         $130 = HEAP32[$zDb>>2]|0;
         $131 = HEAP32[$iDb>>2]|0;
         $132 = ($131|0)==(1);
         $133 = $132 ? 14904 : 14928;
         $134 = HEAP32[$zTabName>>2]|0;
         $135 = HEAP32[$zName>>2]|0;
         $136 = HEAP32[$zWhere>>2]|0;
         HEAP32[$vararg_buffer27>>2] = $130;
         $vararg_ptr7 = (($vararg_buffer27) + 4|0);
         HEAP32[$vararg_ptr7>>2] = $133;
         $vararg_ptr8 = (($vararg_buffer27) + 8|0);
         HEAP32[$vararg_ptr8>>2] = $134;
         $vararg_ptr9 = (($vararg_buffer27) + 12|0);
         HEAP32[$vararg_ptr9>>2] = $135;
         $vararg_ptr10 = (($vararg_buffer27) + 16|0);
         HEAP32[$vararg_ptr10>>2] = $136;
         _sqlite3NestedParse($129,15696,$vararg_buffer27);
         $137 = HEAP32[$db>>2]|0;
         $138 = HEAP32[$zWhere>>2]|0;
         _sqlite3DbFree($137,$138);
        }
       }
       $139 = HEAP32[$0>>2]|0;
       $140 = HEAP32[$zDb>>2]|0;
       $141 = HEAP32[$iDb>>2]|0;
       $142 = ($141|0)==(1);
       $143 = $142 ? 14904 : 14928;
       $144 = HEAP32[$zName>>2]|0;
       $145 = HEAP32[$zName>>2]|0;
       $146 = HEAP32[$zName>>2]|0;
       $147 = HEAP32[$zName>>2]|0;
       $148 = HEAP32[$zName>>2]|0;
       $149 = HEAP32[$nTabName>>2]|0;
       $150 = HEAP32[$zTabName>>2]|0;
       HEAP32[$vararg_buffer27>>2] = $140;
       $vararg_ptr14 = (($vararg_buffer27) + 4|0);
       HEAP32[$vararg_ptr14>>2] = $143;
       $vararg_ptr15 = (($vararg_buffer27) + 8|0);
       HEAP32[$vararg_ptr15>>2] = $144;
       $vararg_ptr16 = (($vararg_buffer27) + 12|0);
       HEAP32[$vararg_ptr16>>2] = $145;
       $vararg_ptr17 = (($vararg_buffer27) + 16|0);
       HEAP32[$vararg_ptr17>>2] = $146;
       $vararg_ptr18 = (($vararg_buffer27) + 20|0);
       HEAP32[$vararg_ptr18>>2] = $147;
       $vararg_ptr19 = (($vararg_buffer27) + 24|0);
       HEAP32[$vararg_ptr19>>2] = $148;
       $vararg_ptr20 = (($vararg_buffer27) + 28|0);
       HEAP32[$vararg_ptr20>>2] = $149;
       $vararg_ptr21 = (($vararg_buffer27) + 32|0);
       HEAP32[$vararg_ptr21>>2] = $150;
       _sqlite3NestedParse($139,15768,$vararg_buffer27);
       $151 = HEAP32[$db>>2]|0;
       $152 = HEAP32[$zDb>>2]|0;
       $153 = (_sqlite3FindTable($151,16152,$152)|0);
       $154 = ($153|0)!=(0|0);
       if ($154) {
        $155 = HEAP32[$0>>2]|0;
        $156 = HEAP32[$zDb>>2]|0;
        $157 = HEAP32[$zName>>2]|0;
        $158 = HEAP32[$pTab>>2]|0;
        $159 = HEAP32[$158>>2]|0;
        HEAP32[$vararg_buffer27>>2] = $156;
        $vararg_ptr25 = (($vararg_buffer27) + 4|0);
        HEAP32[$vararg_ptr25>>2] = $157;
        $vararg_ptr26 = (($vararg_buffer27) + 8|0);
        HEAP32[$vararg_ptr26>>2] = $159;
        _sqlite3NestedParse($155,16168,$vararg_buffer27);
       }
       $160 = HEAP32[$0>>2]|0;
       $161 = HEAP32[$pTab>>2]|0;
       $162 = (_whereTempTriggers($160,$161)|0);
       HEAP32[$zWhere>>2] = $162;
       $163 = ($162|0)!=(0|0);
       if ($163) {
        $164 = HEAP32[$0>>2]|0;
        $165 = HEAP32[$zName>>2]|0;
        $166 = HEAP32[$zName>>2]|0;
        $167 = HEAP32[$zWhere>>2]|0;
        HEAP32[$vararg_buffer27>>2] = $165;
        $vararg_ptr30 = (($vararg_buffer27) + 4|0);
        HEAP32[$vararg_ptr30>>2] = $166;
        $vararg_ptr31 = (($vararg_buffer27) + 8|0);
        HEAP32[$vararg_ptr31>>2] = $167;
        _sqlite3NestedParse($164,16232,$vararg_buffer27);
        $168 = HEAP32[$db>>2]|0;
        $169 = HEAP32[$zWhere>>2]|0;
        _sqlite3DbFree($168,$169);
       }
       $170 = HEAP32[$db>>2]|0;
       $171 = (($170) + 24|0);
       $172 = HEAP32[$171>>2]|0;
       $173 = $172 & 524288;
       $174 = ($173|0)!=(0);
       L33: do {
        if ($174) {
         $175 = HEAP32[$pTab>>2]|0;
         $176 = (_sqlite3FkReferences($175)|0);
         HEAP32[$p>>2] = $176;
         while(1) {
          $177 = HEAP32[$p>>2]|0;
          $178 = ($177|0)!=(0|0);
          if (!($178)) {
           break L33;
          }
          $179 = HEAP32[$p>>2]|0;
          $180 = HEAP32[$179>>2]|0;
          HEAP32[$pFrom>>2] = $180;
          $181 = HEAP32[$pFrom>>2]|0;
          $182 = HEAP32[$pTab>>2]|0;
          $183 = ($181|0)!=($182|0);
          if ($183) {
           $184 = HEAP32[$0>>2]|0;
           $185 = HEAP32[$p>>2]|0;
           $186 = HEAP32[$185>>2]|0;
           $187 = HEAP32[$pFrom>>2]|0;
           $188 = HEAP32[$187>>2]|0;
           _reloadTableSchema($184,$186,$188);
          }
          $189 = HEAP32[$p>>2]|0;
          $190 = (($189) + 12|0);
          $191 = HEAP32[$190>>2]|0;
          HEAP32[$p>>2] = $191;
         }
        }
       } while(0);
       $192 = HEAP32[$0>>2]|0;
       $193 = HEAP32[$pTab>>2]|0;
       $194 = HEAP32[$zName>>2]|0;
       _reloadTableSchema($192,$193,$194);
       break;
      }
     }
     $49 = HEAP32[$0>>2]|0;
     $50 = HEAP32[$zName>>2]|0;
     HEAP32[$vararg_buffer27>>2] = $50;
     _sqlite3ErrorMsg($49,15600,$vararg_buffer27);
    }
   }
  }
 } while(0);
 $195 = HEAP32[$db>>2]|0;
 $196 = HEAP32[$1>>2]|0;
 _sqlite3SrcListDelete($195,$196);
 $197 = HEAP32[$db>>2]|0;
 $198 = HEAP32[$zName>>2]|0;
 _sqlite3DbFree($197,$198);
 $199 = HEAP32[$savedDbFlags>>2]|0;
 $200 = HEAP32[$db>>2]|0;
 $201 = (($200) + 24|0);
 HEAP32[$201>>2] = $199;
 STACKTOP = sp;return;
}
function _sqlite3AlterFinishAddColumn($pParse,$pColDef) {
 $pParse = $pParse|0;
 $pColDef = $pColDef|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $17 = 0, $18 = 0;
 var $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0;
 var $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0;
 var $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0;
 var $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0;
 var $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $db = 0, $iDb = 0, $or$cond = 0, $or$cond3 = 0, $pCol = 0, $pDflt = 0, $pNew = 0, $pTab = 0, $pVal = 0, $savedDbFlags = 0, $vararg_buffer13 = 0;
 var $vararg_ptr15 = 0, $vararg_ptr16 = 0, $vararg_ptr17 = 0, $vararg_ptr18 = 0, $vararg_ptr19 = 0, $zCol = 0, $zDb = 0, $zEnd = 0, $zTab = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 80|0;
 $vararg_buffer13 = sp;
 $0 = sp + 76|0;
 $1 = sp + 72|0;
 $pNew = sp + 68|0;
 $pTab = sp + 64|0;
 $iDb = sp + 60|0;
 $zDb = sp + 56|0;
 $zTab = sp + 52|0;
 $zCol = sp + 48|0;
 $pCol = sp + 44|0;
 $pDflt = sp + 40|0;
 $db = sp + 36|0;
 $pVal = sp + 32|0;
 $zEnd = sp + 28|0;
 $savedDbFlags = sp + 24|0;
 HEAP32[$0>>2] = $pParse;
 HEAP32[$1>>2] = $pColDef;
 $2 = HEAP32[$0>>2]|0;
 $3 = HEAP32[$2>>2]|0;
 HEAP32[$db>>2] = $3;
 $4 = HEAP32[$0>>2]|0;
 $5 = (($4) + 64|0);
 $6 = HEAP32[$5>>2]|0;
 $7 = ($6|0)!=(0);
 if ($7) {
  STACKTOP = sp;return;
 }
 $8 = HEAP32[$db>>2]|0;
 $9 = (($8) + 64|0);
 $10 = HEAP8[$9>>0]|0;
 $11 = $10&255;
 $12 = ($11|0)!=(0);
 if ($12) {
  STACKTOP = sp;return;
 }
 $13 = HEAP32[$0>>2]|0;
 $14 = (($13) + 484|0);
 $15 = HEAP32[$14>>2]|0;
 HEAP32[$pNew>>2] = $15;
 $16 = HEAP32[$db>>2]|0;
 $17 = HEAP32[$pNew>>2]|0;
 $18 = (($17) + 68|0);
 $19 = HEAP32[$18>>2]|0;
 $20 = (_sqlite3SchemaToIndex($16,$19)|0);
 HEAP32[$iDb>>2] = $20;
 $21 = HEAP32[$iDb>>2]|0;
 $22 = HEAP32[$db>>2]|0;
 $23 = (($22) + 16|0);
 $24 = HEAP32[$23>>2]|0;
 $25 = (($24) + ($21<<4)|0);
 $26 = HEAP32[$25>>2]|0;
 HEAP32[$zDb>>2] = $26;
 $27 = HEAP32[$pNew>>2]|0;
 $28 = HEAP32[$27>>2]|0;
 $29 = (($28) + 16|0);
 HEAP32[$zTab>>2] = $29;
 $30 = HEAP32[$pNew>>2]|0;
 $31 = (($30) + 38|0);
 $32 = HEAP16[$31>>1]|0;
 $33 = $32 << 16 >> 16;
 $34 = (($33) - 1)|0;
 $35 = HEAP32[$pNew>>2]|0;
 $36 = (($35) + 4|0);
 $37 = HEAP32[$36>>2]|0;
 $38 = (($37) + (($34*24)|0)|0);
 HEAP32[$pCol>>2] = $38;
 $39 = HEAP32[$pCol>>2]|0;
 $40 = (($39) + 4|0);
 $41 = HEAP32[$40>>2]|0;
 HEAP32[$pDflt>>2] = $41;
 $42 = HEAP32[$db>>2]|0;
 $43 = HEAP32[$zTab>>2]|0;
 $44 = HEAP32[$zDb>>2]|0;
 $45 = (_sqlite3FindTable($42,$43,$44)|0);
 HEAP32[$pTab>>2] = $45;
 $46 = HEAP32[$0>>2]|0;
 $47 = HEAP32[$zDb>>2]|0;
 $48 = HEAP32[$pTab>>2]|0;
 $49 = HEAP32[$48>>2]|0;
 $50 = (_sqlite3AuthCheck($46,26,$47,$49,0)|0);
 $51 = ($50|0)!=(0);
 if ($51) {
  STACKTOP = sp;return;
 }
 $52 = HEAP32[$pDflt>>2]|0;
 $53 = ($52|0)!=(0|0);
 if ($53) {
  $54 = HEAP32[$pDflt>>2]|0;
  $55 = HEAP8[$54>>0]|0;
  $56 = $55&255;
  $57 = ($56|0)==(101);
  if ($57) {
   HEAP32[$pDflt>>2] = 0;
  }
 }
 $58 = HEAP32[$pCol>>2]|0;
 $59 = (($58) + 23|0);
 $60 = HEAP8[$59>>0]|0;
 $61 = $60&255;
 $62 = $61 & 1;
 $63 = ($62|0)!=(0);
 if ($63) {
  $64 = HEAP32[$0>>2]|0;
  _sqlite3ErrorMsg($64,15184,$vararg_buffer13);
  STACKTOP = sp;return;
 }
 $65 = HEAP32[$pNew>>2]|0;
 $66 = (($65) + 8|0);
 $67 = HEAP32[$66>>2]|0;
 $68 = ($67|0)!=(0|0);
 if ($68) {
  $69 = HEAP32[$0>>2]|0;
  _sqlite3ErrorMsg($69,15216,$vararg_buffer13);
  STACKTOP = sp;return;
 }
 $70 = HEAP32[$db>>2]|0;
 $71 = (($70) + 24|0);
 $72 = HEAP32[$71>>2]|0;
 $73 = $72 & 524288;
 $74 = ($73|0)!=(0);
 if ($74) {
  $75 = HEAP32[$pNew>>2]|0;
  $76 = (($75) + 16|0);
  $77 = HEAP32[$76>>2]|0;
  $78 = ($77|0)!=(0|0);
  $79 = HEAP32[$pDflt>>2]|0;
  $80 = ($79|0)!=(0|0);
  $or$cond = $78 & $80;
  if ($or$cond) {
   $81 = HEAP32[$0>>2]|0;
   _sqlite3ErrorMsg($81,15248,$vararg_buffer13);
   STACKTOP = sp;return;
  }
 }
 $82 = HEAP32[$pCol>>2]|0;
 $83 = (($82) + 20|0);
 $84 = HEAP8[$83>>0]|0;
 $85 = $84&255;
 $86 = ($85|0)==(0);
 $87 = HEAP32[$pDflt>>2]|0;
 $88 = ($87|0)!=(0|0);
 $or$cond3 = $86 | $88;
 if (!($or$cond3)) {
  $89 = HEAP32[$0>>2]|0;
  _sqlite3ErrorMsg($89,15312,$vararg_buffer13);
  STACKTOP = sp;return;
 }
 $90 = HEAP32[$pDflt>>2]|0;
 $91 = ($90|0)!=(0|0);
 do {
  if ($91) {
   HEAP32[$pVal>>2] = 0;
   $92 = HEAP32[$db>>2]|0;
   $93 = HEAP32[$pDflt>>2]|0;
   $94 = (_sqlite3ValueFromExpr($92,$93,1,65,$pVal)|0);
   $95 = ($94|0)!=(0);
   if ($95) {
    $96 = HEAP32[$db>>2]|0;
    $97 = (($96) + 64|0);
    HEAP8[$97>>0] = 1;
    STACKTOP = sp;return;
   }
   $98 = HEAP32[$pVal>>2]|0;
   $99 = ($98|0)!=(0|0);
   if ($99) {
    $101 = HEAP32[$pVal>>2]|0;
    _sqlite3ValueFree($101);
    break;
   }
   $100 = HEAP32[$0>>2]|0;
   _sqlite3ErrorMsg($100,15368,$vararg_buffer13);
   STACKTOP = sp;return;
  }
 } while(0);
 $102 = HEAP32[$db>>2]|0;
 $103 = HEAP32[$1>>2]|0;
 $104 = HEAP32[$103>>2]|0;
 $105 = HEAP32[$1>>2]|0;
 $106 = (($105) + 4|0);
 $107 = HEAP32[$106>>2]|0;
 $108 = (_sqlite3DbStrNDup($102,$104,$107,0)|0);
 HEAP32[$zCol>>2] = $108;
 $109 = HEAP32[$zCol>>2]|0;
 $110 = ($109|0)!=(0|0);
 if ($110) {
  $111 = HEAP32[$1>>2]|0;
  $112 = (($111) + 4|0);
  $113 = HEAP32[$112>>2]|0;
  $114 = (($113) - 1)|0;
  $115 = HEAP32[$zCol>>2]|0;
  $116 = (($115) + ($114)|0);
  HEAP32[$zEnd>>2] = $116;
  $117 = HEAP32[$db>>2]|0;
  $118 = (($117) + 24|0);
  $119 = HEAP32[$118>>2]|0;
  HEAP32[$savedDbFlags>>2] = $119;
  while(1) {
   $120 = HEAP32[$zEnd>>2]|0;
   $121 = HEAP32[$zCol>>2]|0;
   $122 = ($120>>>0)>($121>>>0);
   if (!($122)) {
    break;
   }
   $123 = HEAP32[$zEnd>>2]|0;
   $124 = HEAP8[$123>>0]|0;
   $125 = $124 << 24 >> 24;
   $126 = ($125|0)==(59);
   if (!($126)) {
    $127 = HEAP32[$zEnd>>2]|0;
    $128 = HEAP8[$127>>0]|0;
    $129 = $128&255;
    $130 = (1224 + ($129)|0);
    $131 = HEAP8[$130>>0]|0;
    $132 = $131&255;
    $133 = $132 & 1;
    $134 = ($133|0)!=(0);
    if (!($134)) {
     break;
    }
   }
   $135 = HEAP32[$zEnd>>2]|0;
   $136 = (($135) + -1|0);
   HEAP32[$zEnd>>2] = $136;
   HEAP8[$135>>0] = 0;
  }
  $137 = HEAP32[$db>>2]|0;
  $138 = (($137) + 24|0);
  $139 = HEAP32[$138>>2]|0;
  $140 = $139 | 2097152;
  HEAP32[$138>>2] = $140;
  $141 = HEAP32[$0>>2]|0;
  $142 = HEAP32[$zDb>>2]|0;
  $143 = HEAP32[$iDb>>2]|0;
  $144 = ($143|0)==(1);
  $145 = $144 ? 14904 : 14928;
  $146 = HEAP32[$pNew>>2]|0;
  $147 = (($146) + 48|0);
  $148 = HEAP32[$147>>2]|0;
  $149 = HEAP32[$zCol>>2]|0;
  $150 = HEAP32[$pNew>>2]|0;
  $151 = (($150) + 48|0);
  $152 = HEAP32[$151>>2]|0;
  $153 = (($152) + 1)|0;
  $154 = HEAP32[$zTab>>2]|0;
  HEAP32[$vararg_buffer13>>2] = $142;
  $vararg_ptr15 = (($vararg_buffer13) + 4|0);
  HEAP32[$vararg_ptr15>>2] = $145;
  $vararg_ptr16 = (($vararg_buffer13) + 8|0);
  HEAP32[$vararg_ptr16>>2] = $148;
  $vararg_ptr17 = (($vararg_buffer13) + 12|0);
  HEAP32[$vararg_ptr17>>2] = $149;
  $vararg_ptr18 = (($vararg_buffer13) + 16|0);
  HEAP32[$vararg_ptr18>>2] = $153;
  $vararg_ptr19 = (($vararg_buffer13) + 20|0);
  HEAP32[$vararg_ptr19>>2] = $154;
  _sqlite3NestedParse($141,15416,$vararg_buffer13);
  $155 = HEAP32[$db>>2]|0;
  $156 = HEAP32[$zCol>>2]|0;
  _sqlite3DbFree($155,$156);
  $157 = HEAP32[$savedDbFlags>>2]|0;
  $158 = HEAP32[$db>>2]|0;
  $159 = (($158) + 24|0);
  HEAP32[$159>>2] = $157;
 }
 $160 = HEAP32[$0>>2]|0;
 $161 = HEAP32[$iDb>>2]|0;
 $162 = HEAP32[$pDflt>>2]|0;
 $163 = ($162|0)!=(0|0);
 $164 = $163 ? 3 : 2;
 _sqlite3MinimumFileFormat($160,$161,$164);
 $165 = HEAP32[$0>>2]|0;
 $166 = HEAP32[$pTab>>2]|0;
 $167 = HEAP32[$pTab>>2]|0;
 $168 = HEAP32[$167>>2]|0;
 _reloadTableSchema($165,$166,$168);
 STACKTOP = sp;return;
}
function _sqlite3AlterBeginAddColumn($pParse,$pSrc) {
 $pParse = $pParse|0;
 $pSrc = $pSrc|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0;
 var $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0;
 var $db = 0, $i = 0, $iDb = 0, $nAlloc = 0, $pCol = 0, $pNew = 0, $pTab = 0, $v = 0, $vararg_buffer3 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $vararg_buffer3 = sp;
 $0 = sp + 40|0;
 $1 = sp + 36|0;
 $pNew = sp + 32|0;
 $pTab = sp + 28|0;
 $v = sp + 24|0;
 $iDb = sp + 20|0;
 $i = sp + 16|0;
 $nAlloc = sp + 12|0;
 $db = sp + 8|0;
 $pCol = sp + 4|0;
 HEAP32[$0>>2] = $pParse;
 HEAP32[$1>>2] = $pSrc;
 $2 = HEAP32[$0>>2]|0;
 $3 = HEAP32[$2>>2]|0;
 HEAP32[$db>>2] = $3;
 $4 = HEAP32[$db>>2]|0;
 $5 = (($4) + 64|0);
 $6 = HEAP8[$5>>0]|0;
 $7 = ($6<<24>>24)!=(0);
 if ($7) {
  $138 = HEAP32[$db>>2]|0;
  $139 = HEAP32[$1>>2]|0;
  _sqlite3SrcListDelete($138,$139);
  STACKTOP = sp;return;
 }
 $8 = HEAP32[$0>>2]|0;
 $9 = HEAP32[$1>>2]|0;
 $10 = (($9) + 8|0);
 $11 = (_sqlite3LocateTableItem($8,0,$10)|0);
 HEAP32[$pTab>>2] = $11;
 $12 = HEAP32[$pTab>>2]|0;
 $13 = ($12|0)!=(0|0);
 if (!($13)) {
  $138 = HEAP32[$db>>2]|0;
  $139 = HEAP32[$1>>2]|0;
  _sqlite3SrcListDelete($138,$139);
  STACKTOP = sp;return;
 }
 $14 = HEAP32[$pTab>>2]|0;
 $15 = (($14) + 44|0);
 $16 = HEAP8[$15>>0]|0;
 $17 = $16&255;
 $18 = $17 & 16;
 $19 = ($18|0)!=(0);
 if ($19) {
  $20 = HEAP32[$0>>2]|0;
  _sqlite3ErrorMsg($20,14976,$vararg_buffer3);
  $138 = HEAP32[$db>>2]|0;
  $139 = HEAP32[$1>>2]|0;
  _sqlite3SrcListDelete($138,$139);
  STACKTOP = sp;return;
 }
 $21 = HEAP32[$pTab>>2]|0;
 $22 = (($21) + 12|0);
 $23 = HEAP32[$22>>2]|0;
 $24 = ($23|0)!=(0|0);
 $25 = HEAP32[$0>>2]|0;
 if ($24) {
  _sqlite3ErrorMsg($25,15016,$vararg_buffer3);
  $138 = HEAP32[$db>>2]|0;
  $139 = HEAP32[$1>>2]|0;
  _sqlite3SrcListDelete($138,$139);
  STACKTOP = sp;return;
 }
 $26 = HEAP32[$pTab>>2]|0;
 $27 = HEAP32[$26>>2]|0;
 $28 = (_isSystemTable($25,$27)|0);
 $29 = (0)!=($28|0);
 if ($29) {
  $138 = HEAP32[$db>>2]|0;
  $139 = HEAP32[$1>>2]|0;
  _sqlite3SrcListDelete($138,$139);
  STACKTOP = sp;return;
 }
 $30 = HEAP32[$db>>2]|0;
 $31 = HEAP32[$pTab>>2]|0;
 $32 = (($31) + 68|0);
 $33 = HEAP32[$32>>2]|0;
 $34 = (_sqlite3SchemaToIndex($30,$33)|0);
 HEAP32[$iDb>>2] = $34;
 $35 = HEAP32[$db>>2]|0;
 $36 = (_sqlite3DbMallocZero($35,76,0)|0);
 HEAP32[$pNew>>2] = $36;
 $37 = HEAP32[$pNew>>2]|0;
 $38 = ($37|0)!=(0|0);
 if (!($38)) {
  $138 = HEAP32[$db>>2]|0;
  $139 = HEAP32[$1>>2]|0;
  _sqlite3SrcListDelete($138,$139);
  STACKTOP = sp;return;
 }
 $39 = HEAP32[$pNew>>2]|0;
 $40 = HEAP32[$0>>2]|0;
 $41 = (($40) + 484|0);
 HEAP32[$41>>2] = $39;
 $42 = HEAP32[$pNew>>2]|0;
 $43 = (($42) + 40|0);
 HEAP16[$43>>1] = 1;
 $44 = HEAP32[$pTab>>2]|0;
 $45 = (($44) + 38|0);
 $46 = HEAP16[$45>>1]|0;
 $47 = HEAP32[$pNew>>2]|0;
 $48 = (($47) + 38|0);
 HEAP16[$48>>1] = $46;
 $49 = HEAP32[$pNew>>2]|0;
 $50 = (($49) + 38|0);
 $51 = HEAP16[$50>>1]|0;
 $52 = $51 << 16 >> 16;
 $53 = (($52) - 1)|0;
 $54 = (($53|0) / 8)&-1;
 $55 = $54<<3;
 $56 = (($55) + 8)|0;
 HEAP32[$nAlloc>>2] = $56;
 $57 = HEAP32[$db>>2]|0;
 $58 = HEAP32[$nAlloc>>2]|0;
 $59 = ($58*24)|0;
 $60 = (_sqlite3DbMallocZero($57,$59,0)|0);
 $61 = HEAP32[$pNew>>2]|0;
 $62 = (($61) + 4|0);
 HEAP32[$62>>2] = $60;
 $63 = HEAP32[$db>>2]|0;
 $64 = HEAP32[$pTab>>2]|0;
 $65 = HEAP32[$64>>2]|0;
 HEAP32[$vararg_buffer3>>2] = $65;
 $66 = (_sqlite3MPrintf($63,15048,$vararg_buffer3)|0);
 $67 = HEAP32[$pNew>>2]|0;
 HEAP32[$67>>2] = $66;
 $68 = HEAP32[$pNew>>2]|0;
 $69 = (($68) + 4|0);
 $70 = HEAP32[$69>>2]|0;
 $71 = ($70|0)!=(0|0);
 if ($71) {
  $72 = HEAP32[$pNew>>2]|0;
  $73 = HEAP32[$72>>2]|0;
  $74 = ($73|0)!=(0|0);
  if ($74) {
   $77 = HEAP32[$pNew>>2]|0;
   $78 = (($77) + 4|0);
   $79 = HEAP32[$78>>2]|0;
   $80 = HEAP32[$pTab>>2]|0;
   $81 = (($80) + 4|0);
   $82 = HEAP32[$81>>2]|0;
   $83 = HEAP32[$pNew>>2]|0;
   $84 = (($83) + 38|0);
   $85 = HEAP16[$84>>1]|0;
   $86 = $85 << 16 >> 16;
   $87 = ($86*24)|0;
   _memcpy(($79|0),($82|0),($87|0))|0;
   HEAP32[$i>>2] = 0;
   while(1) {
    $88 = HEAP32[$i>>2]|0;
    $89 = HEAP32[$pNew>>2]|0;
    $90 = (($89) + 38|0);
    $91 = HEAP16[$90>>1]|0;
    $92 = $91 << 16 >> 16;
    $93 = ($88|0)<($92|0);
    if (!($93)) {
     break;
    }
    $94 = HEAP32[$i>>2]|0;
    $95 = HEAP32[$pNew>>2]|0;
    $96 = (($95) + 4|0);
    $97 = HEAP32[$96>>2]|0;
    $98 = (($97) + (($94*24)|0)|0);
    HEAP32[$pCol>>2] = $98;
    $99 = HEAP32[$db>>2]|0;
    $100 = HEAP32[$pCol>>2]|0;
    $101 = HEAP32[$100>>2]|0;
    $102 = (_sqlite3DbStrDup($99,$101)|0);
    $103 = HEAP32[$pCol>>2]|0;
    HEAP32[$103>>2] = $102;
    $104 = HEAP32[$pCol>>2]|0;
    $105 = (($104) + 16|0);
    HEAP32[$105>>2] = 0;
    $106 = HEAP32[$pCol>>2]|0;
    $107 = (($106) + 12|0);
    HEAP32[$107>>2] = 0;
    $108 = HEAP32[$pCol>>2]|0;
    $109 = (($108) + 4|0);
    HEAP32[$109>>2] = 0;
    $110 = HEAP32[$pCol>>2]|0;
    $111 = (($110) + 8|0);
    HEAP32[$111>>2] = 0;
    $112 = HEAP32[$i>>2]|0;
    $113 = (($112) + 1)|0;
    HEAP32[$i>>2] = $113;
   }
   $114 = HEAP32[$iDb>>2]|0;
   $115 = HEAP32[$db>>2]|0;
   $116 = (($115) + 16|0);
   $117 = HEAP32[$116>>2]|0;
   $118 = (($117) + ($114<<4)|0);
   $119 = (($118) + 12|0);
   $120 = HEAP32[$119>>2]|0;
   $121 = HEAP32[$pNew>>2]|0;
   $122 = (($121) + 68|0);
   HEAP32[$122>>2] = $120;
   $123 = HEAP32[$pTab>>2]|0;
   $124 = (($123) + 48|0);
   $125 = HEAP32[$124>>2]|0;
   $126 = HEAP32[$pNew>>2]|0;
   $127 = (($126) + 48|0);
   HEAP32[$127>>2] = $125;
   $128 = HEAP32[$pNew>>2]|0;
   $129 = (($128) + 40|0);
   HEAP16[$129>>1] = 1;
   $130 = HEAP32[$0>>2]|0;
   $131 = HEAP32[$iDb>>2]|0;
   _sqlite3BeginWriteOperation($130,0,$131);
   $132 = HEAP32[$0>>2]|0;
   $133 = (_sqlite3GetVdbe($132)|0);
   HEAP32[$v>>2] = $133;
   $134 = HEAP32[$v>>2]|0;
   $135 = ($134|0)!=(0|0);
   if (!($135)) {
    $138 = HEAP32[$db>>2]|0;
    $139 = HEAP32[$1>>2]|0;
    _sqlite3SrcListDelete($138,$139);
    STACKTOP = sp;return;
   }
   $136 = HEAP32[$0>>2]|0;
   $137 = HEAP32[$iDb>>2]|0;
   _sqlite3ChangeCookie($136,$137);
   $138 = HEAP32[$db>>2]|0;
   $139 = HEAP32[$1>>2]|0;
   _sqlite3SrcListDelete($138,$139);
   STACKTOP = sp;return;
  }
 }
 $75 = HEAP32[$db>>2]|0;
 $76 = (($75) + 64|0);
 HEAP8[$76>>0] = 1;
 $138 = HEAP32[$db>>2]|0;
 $139 = HEAP32[$1>>2]|0;
 _sqlite3SrcListDelete($138,$139);
 STACKTOP = sp;return;
}
function _sqlite3VtabFinishParse($pParse,$pEnd) {
 $pParse = $pParse|0;
 $pEnd = $pEnd|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0;
 var $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0;
 var $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0;
 var $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0;
 var $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0;
 var $97 = 0, $98 = 0, $99 = 0, $db = 0, $iDb = 0, $pOld = 0, $pSchema = 0, $pTab = 0, $v = 0, $vararg_buffer9 = 0, $vararg_ptr4 = 0, $vararg_ptr5 = 0, $vararg_ptr6 = 0, $vararg_ptr7 = 0, $vararg_ptr8 = 0, $zName = 0, $zStmt = 0, $zWhere = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 80|0;
 $vararg_buffer9 = sp;
 $0 = sp + 64|0;
 $1 = sp + 60|0;
 $pTab = sp + 56|0;
 $db = sp + 52|0;
 $zStmt = sp + 48|0;
 $zWhere = sp + 44|0;
 $iDb = sp + 40|0;
 $v = sp + 36|0;
 $pOld = sp + 32|0;
 $pSchema = sp + 28|0;
 $zName = sp + 24|0;
 HEAP32[$0>>2] = $pParse;
 HEAP32[$1>>2] = $pEnd;
 $2 = HEAP32[$0>>2]|0;
 $3 = (($2) + 484|0);
 $4 = HEAP32[$3>>2]|0;
 HEAP32[$pTab>>2] = $4;
 $5 = HEAP32[$0>>2]|0;
 $6 = HEAP32[$5>>2]|0;
 HEAP32[$db>>2] = $6;
 $7 = HEAP32[$pTab>>2]|0;
 $8 = ($7|0)==(0|0);
 if ($8) {
  STACKTOP = sp;return;
 }
 $9 = HEAP32[$0>>2]|0;
 _addArgumentToVtab($9);
 $10 = HEAP32[$0>>2]|0;
 $11 = (($10) + 512|0);
 HEAP32[$11>>2] = 0;
 $12 = HEAP32[$pTab>>2]|0;
 $13 = (($12) + 52|0);
 $14 = HEAP32[$13>>2]|0;
 $15 = ($14|0)<(1);
 if ($15) {
  STACKTOP = sp;return;
 }
 $16 = HEAP32[$db>>2]|0;
 $17 = (($16) + 140|0);
 $18 = (($17) + 5|0);
 $19 = HEAP8[$18>>0]|0;
 $20 = ($19<<24>>24)!=(0);
 if (!($20)) {
  $21 = HEAP32[$1>>2]|0;
  $22 = ($21|0)!=(0|0);
  if ($22) {
   $23 = HEAP32[$1>>2]|0;
   $24 = HEAP32[$23>>2]|0;
   $25 = HEAP32[$0>>2]|0;
   $26 = (($25) + 496|0);
   $27 = HEAP32[$26>>2]|0;
   $28 = $24;
   $29 = $27;
   $30 = (($28) - ($29))|0;
   $31 = HEAP32[$1>>2]|0;
   $32 = (($31) + 4|0);
   $33 = HEAP32[$32>>2]|0;
   $34 = (($30) + ($33))|0;
   $35 = HEAP32[$0>>2]|0;
   $36 = (($35) + 496|0);
   $37 = (($36) + 4|0);
   HEAP32[$37>>2] = $34;
  }
  $38 = HEAP32[$db>>2]|0;
  $39 = HEAP32[$0>>2]|0;
  $40 = (($39) + 496|0);
  HEAP32[$vararg_buffer9>>2] = $40;
  $41 = (_sqlite3MPrintf($38,14792,$vararg_buffer9)|0);
  HEAP32[$zStmt>>2] = $41;
  $42 = HEAP32[$db>>2]|0;
  $43 = HEAP32[$pTab>>2]|0;
  $44 = (($43) + 68|0);
  $45 = HEAP32[$44>>2]|0;
  $46 = (_sqlite3SchemaToIndex($42,$45)|0);
  HEAP32[$iDb>>2] = $46;
  $47 = HEAP32[$0>>2]|0;
  $48 = HEAP32[$iDb>>2]|0;
  $49 = HEAP32[$db>>2]|0;
  $50 = (($49) + 16|0);
  $51 = HEAP32[$50>>2]|0;
  $52 = (($51) + ($48<<4)|0);
  $53 = HEAP32[$52>>2]|0;
  $54 = HEAP32[$iDb>>2]|0;
  $55 = ($54|0)==(1);
  $56 = $55 ? 14904 : 14928;
  $57 = HEAP32[$pTab>>2]|0;
  $58 = HEAP32[$57>>2]|0;
  $59 = HEAP32[$pTab>>2]|0;
  $60 = HEAP32[$59>>2]|0;
  $61 = HEAP32[$zStmt>>2]|0;
  $62 = HEAP32[$0>>2]|0;
  $63 = (($62) + 384|0);
  $64 = HEAP32[$63>>2]|0;
  HEAP32[$vararg_buffer9>>2] = $53;
  $vararg_ptr4 = (($vararg_buffer9) + 4|0);
  HEAP32[$vararg_ptr4>>2] = $56;
  $vararg_ptr5 = (($vararg_buffer9) + 8|0);
  HEAP32[$vararg_ptr5>>2] = $58;
  $vararg_ptr6 = (($vararg_buffer9) + 12|0);
  HEAP32[$vararg_ptr6>>2] = $60;
  $vararg_ptr7 = (($vararg_buffer9) + 16|0);
  HEAP32[$vararg_ptr7>>2] = $61;
  $vararg_ptr8 = (($vararg_buffer9) + 20|0);
  HEAP32[$vararg_ptr8>>2] = $64;
  _sqlite3NestedParse($47,14816,$vararg_buffer9);
  $65 = HEAP32[$db>>2]|0;
  $66 = HEAP32[$zStmt>>2]|0;
  _sqlite3DbFree($65,$66);
  $67 = HEAP32[$0>>2]|0;
  $68 = (_sqlite3GetVdbe($67)|0);
  HEAP32[$v>>2] = $68;
  $69 = HEAP32[$0>>2]|0;
  $70 = HEAP32[$iDb>>2]|0;
  _sqlite3ChangeCookie($69,$70);
  $71 = HEAP32[$v>>2]|0;
  (_sqlite3VdbeAddOp2($71,142,0,0)|0);
  $72 = HEAP32[$db>>2]|0;
  $73 = HEAP32[$pTab>>2]|0;
  $74 = HEAP32[$73>>2]|0;
  HEAP32[$vararg_buffer9>>2] = $74;
  $75 = (_sqlite3MPrintf($72,14944,$vararg_buffer9)|0);
  HEAP32[$zWhere>>2] = $75;
  $76 = HEAP32[$v>>2]|0;
  $77 = HEAP32[$iDb>>2]|0;
  $78 = HEAP32[$zWhere>>2]|0;
  _sqlite3VdbeAddParseSchemaOp($76,$77,$78);
  $79 = HEAP32[$v>>2]|0;
  $80 = HEAP32[$iDb>>2]|0;
  $81 = HEAP32[$pTab>>2]|0;
  $82 = HEAP32[$81>>2]|0;
  $83 = HEAP32[$pTab>>2]|0;
  $84 = HEAP32[$83>>2]|0;
  $85 = (_sqlite3Strlen30($84)|0);
  $86 = (($85) + 1)|0;
  (_sqlite3VdbeAddOp4($79,145,$80,0,0,$82,$86)|0);
  STACKTOP = sp;return;
 }
 $87 = HEAP32[$pTab>>2]|0;
 $88 = (($87) + 68|0);
 $89 = HEAP32[$88>>2]|0;
 HEAP32[$pSchema>>2] = $89;
 $90 = HEAP32[$pTab>>2]|0;
 $91 = HEAP32[$90>>2]|0;
 HEAP32[$zName>>2] = $91;
 $92 = HEAP32[$pSchema>>2]|0;
 $93 = (($92) + 8|0);
 $94 = HEAP32[$zName>>2]|0;
 $95 = HEAP32[$pTab>>2]|0;
 $96 = (_sqlite3HashInsert($93,$94,$95)|0);
 HEAP32[$pOld>>2] = $96;
 $97 = HEAP32[$pOld>>2]|0;
 $98 = ($97|0)!=(0|0);
 if ($98) {
  $99 = HEAP32[$db>>2]|0;
  $100 = (($99) + 64|0);
  HEAP8[$100>>0] = 1;
  STACKTOP = sp;return;
 } else {
  $101 = HEAP32[$0>>2]|0;
  $102 = (($101) + 484|0);
  HEAP32[$102>>2] = 0;
  STACKTOP = sp;return;
 }
}
function _sqlite3VtabBeginParse($pParse,$pName1,$pName2,$pModuleName,$ifNotExists) {
 $pParse = $pParse|0;
 $pName1 = $pName1|0;
 $pName2 = $pName2|0;
 $pModuleName = $pModuleName|0;
 $ifNotExists = $ifNotExists|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $8 = 0, $9 = 0, $db = 0, $iDb = 0, $pTable = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 28|0;
 $1 = sp + 24|0;
 $2 = sp + 20|0;
 $3 = sp + 16|0;
 $4 = sp + 12|0;
 $iDb = sp + 8|0;
 $pTable = sp + 4|0;
 $db = sp;
 HEAP32[$0>>2] = $pParse;
 HEAP32[$1>>2] = $pName1;
 HEAP32[$2>>2] = $pName2;
 HEAP32[$3>>2] = $pModuleName;
 HEAP32[$4>>2] = $ifNotExists;
 $5 = HEAP32[$0>>2]|0;
 $6 = HEAP32[$1>>2]|0;
 $7 = HEAP32[$2>>2]|0;
 $8 = HEAP32[$4>>2]|0;
 _sqlite3StartTable($5,$6,$7,0,0,1,$8);
 $9 = HEAP32[$0>>2]|0;
 $10 = (($9) + 484|0);
 $11 = HEAP32[$10>>2]|0;
 HEAP32[$pTable>>2] = $11;
 $12 = HEAP32[$pTable>>2]|0;
 $13 = ($12|0)==(0|0);
 if ($13) {
  STACKTOP = sp;return;
 }
 $14 = HEAP32[$0>>2]|0;
 $15 = HEAP32[$14>>2]|0;
 HEAP32[$db>>2] = $15;
 $16 = HEAP32[$db>>2]|0;
 $17 = HEAP32[$pTable>>2]|0;
 $18 = (($17) + 68|0);
 $19 = HEAP32[$18>>2]|0;
 $20 = (_sqlite3SchemaToIndex($16,$19)|0);
 HEAP32[$iDb>>2] = $20;
 $21 = HEAP32[$pTable>>2]|0;
 $22 = (($21) + 44|0);
 $23 = HEAP8[$22>>0]|0;
 $24 = $23&255;
 $25 = $24 | 16;
 $26 = $25&255;
 HEAP8[$22>>0] = $26;
 $27 = HEAP32[$pTable>>2]|0;
 $28 = (($27) + 52|0);
 HEAP32[$28>>2] = 0;
 $29 = HEAP32[$db>>2]|0;
 $30 = HEAP32[$pTable>>2]|0;
 $31 = HEAP32[$db>>2]|0;
 $32 = HEAP32[$3>>2]|0;
 $33 = (_sqlite3NameFromToken($31,$32)|0);
 _addModuleArgument($29,$30,$33);
 $34 = HEAP32[$db>>2]|0;
 $35 = HEAP32[$pTable>>2]|0;
 _addModuleArgument($34,$35,0);
 $36 = HEAP32[$db>>2]|0;
 $37 = HEAP32[$pTable>>2]|0;
 $38 = HEAP32[$db>>2]|0;
 $39 = HEAP32[$pTable>>2]|0;
 $40 = HEAP32[$39>>2]|0;
 $41 = (_sqlite3DbStrDup($38,$40)|0);
 _addModuleArgument($36,$37,$41);
 $42 = HEAP32[$3>>2]|0;
 $43 = (($42) + 4|0);
 $44 = HEAP32[$43>>2]|0;
 $45 = HEAP32[$3>>2]|0;
 $46 = HEAP32[$45>>2]|0;
 $47 = (($46) + ($44)|0);
 $48 = HEAP32[$1>>2]|0;
 $49 = HEAP32[$48>>2]|0;
 $50 = $47;
 $51 = $49;
 $52 = (($50) - ($51))|0;
 $53 = HEAP32[$0>>2]|0;
 $54 = (($53) + 496|0);
 $55 = (($54) + 4|0);
 HEAP32[$55>>2] = $52;
 $56 = HEAP32[$pTable>>2]|0;
 $57 = (($56) + 56|0);
 $58 = HEAP32[$57>>2]|0;
 $59 = ($58|0)!=(0|0);
 if (!($59)) {
  STACKTOP = sp;return;
 }
 $60 = HEAP32[$0>>2]|0;
 $61 = HEAP32[$pTable>>2]|0;
 $62 = HEAP32[$61>>2]|0;
 $63 = HEAP32[$pTable>>2]|0;
 $64 = (($63) + 56|0);
 $65 = HEAP32[$64>>2]|0;
 $66 = HEAP32[$65>>2]|0;
 $67 = HEAP32[$iDb>>2]|0;
 $68 = HEAP32[$0>>2]|0;
 $69 = HEAP32[$68>>2]|0;
 $70 = (($69) + 16|0);
 $71 = HEAP32[$70>>2]|0;
 $72 = (($71) + ($67<<4)|0);
 $73 = HEAP32[$72>>2]|0;
 (_sqlite3AuthCheck($60,29,$62,$66,$73)|0);
 STACKTOP = sp;return;
}
function _sqlite3VtabArgInit($pParse) {
 $pParse = $pParse|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 HEAP32[$0>>2] = $pParse;
 $1 = HEAP32[$0>>2]|0;
 _addArgumentToVtab($1);
 $2 = HEAP32[$0>>2]|0;
 $3 = (($2) + 512|0);
 HEAP32[$3>>2] = 0;
 $4 = HEAP32[$0>>2]|0;
 $5 = (($4) + 512|0);
 $6 = (($5) + 4|0);
 HEAP32[$6>>2] = 0;
 STACKTOP = sp;return;
}
function _sqlite3VtabArgExtend($pParse,$p) {
 $pParse = $pParse|0;
 $p = $p|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $pArg = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $1 = sp + 4|0;
 $pArg = sp;
 HEAP32[$0>>2] = $pParse;
 HEAP32[$1>>2] = $p;
 $2 = HEAP32[$0>>2]|0;
 $3 = (($2) + 512|0);
 HEAP32[$pArg>>2] = $3;
 $4 = HEAP32[$pArg>>2]|0;
 $5 = HEAP32[$4>>2]|0;
 $6 = ($5|0)==(0|0);
 $7 = HEAP32[$1>>2]|0;
 if ($6) {
  $8 = HEAP32[$7>>2]|0;
  $9 = HEAP32[$pArg>>2]|0;
  HEAP32[$9>>2] = $8;
  $10 = HEAP32[$1>>2]|0;
  $11 = (($10) + 4|0);
  $12 = HEAP32[$11>>2]|0;
  $13 = HEAP32[$pArg>>2]|0;
  $14 = (($13) + 4|0);
  HEAP32[$14>>2] = $12;
  STACKTOP = sp;return;
 } else {
  $15 = (($7) + 4|0);
  $16 = HEAP32[$15>>2]|0;
  $17 = HEAP32[$1>>2]|0;
  $18 = HEAP32[$17>>2]|0;
  $19 = (($18) + ($16)|0);
  $20 = HEAP32[$pArg>>2]|0;
  $21 = HEAP32[$20>>2]|0;
  $22 = $19;
  $23 = $21;
  $24 = (($22) - ($23))|0;
  $25 = HEAP32[$pArg>>2]|0;
  $26 = (($25) + 4|0);
  HEAP32[$26>>2] = $24;
  STACKTOP = sp;return;
 }
}
function _sqlite3WithAdd($pParse,$pWith,$pName,$pArglist,$pQuery) {
 $pParse = $pParse|0;
 $pWith = $pWith|0;
 $pName = $pName|0;
 $pArglist = $pArglist|0;
 $pQuery = $pQuery|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0;
 var $81 = 0, $82 = 0, $83 = 0, $9 = 0, $db = 0, $i = 0, $nByte = 0, $or$cond = 0, $pNew = 0, $vararg_buffer = 0, $zName = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $vararg_buffer = sp;
 $0 = sp + 40|0;
 $1 = sp + 36|0;
 $2 = sp + 32|0;
 $3 = sp + 28|0;
 $4 = sp + 24|0;
 $db = sp + 20|0;
 $pNew = sp + 16|0;
 $zName = sp + 12|0;
 $i = sp + 8|0;
 $nByte = sp + 4|0;
 HEAP32[$0>>2] = $pParse;
 HEAP32[$1>>2] = $pWith;
 HEAP32[$2>>2] = $pName;
 HEAP32[$3>>2] = $pArglist;
 HEAP32[$4>>2] = $pQuery;
 $5 = HEAP32[$0>>2]|0;
 $6 = HEAP32[$5>>2]|0;
 HEAP32[$db>>2] = $6;
 $7 = HEAP32[$0>>2]|0;
 $8 = HEAP32[$7>>2]|0;
 $9 = HEAP32[$2>>2]|0;
 $10 = (_sqlite3NameFromToken($8,$9)|0);
 HEAP32[$zName>>2] = $10;
 $11 = HEAP32[$zName>>2]|0;
 $12 = ($11|0)!=(0|0);
 $13 = HEAP32[$1>>2]|0;
 $14 = ($13|0)!=(0|0);
 $or$cond = $12 & $14;
 L1: do {
  if ($or$cond) {
   HEAP32[$i>>2] = 0;
   while(1) {
    $15 = HEAP32[$i>>2]|0;
    $16 = HEAP32[$1>>2]|0;
    $17 = HEAP32[$16>>2]|0;
    $18 = ($15|0)<($17|0);
    if (!($18)) {
     break L1;
    }
    $19 = HEAP32[$zName>>2]|0;
    $20 = HEAP32[$i>>2]|0;
    $21 = HEAP32[$1>>2]|0;
    $22 = (($21) + 8|0);
    $23 = (($22) + ($20<<4)|0);
    $24 = HEAP32[$23>>2]|0;
    $25 = (_sqlite3_stricmp($19,$24)|0);
    $26 = ($25|0)==(0);
    if ($26) {
     $27 = HEAP32[$0>>2]|0;
     $28 = HEAP32[$zName>>2]|0;
     HEAP32[$vararg_buffer>>2] = $28;
     _sqlite3ErrorMsg($27,14720,$vararg_buffer);
    }
    $29 = HEAP32[$i>>2]|0;
    $30 = (($29) + 1)|0;
    HEAP32[$i>>2] = $30;
   }
  }
 } while(0);
 $31 = HEAP32[$1>>2]|0;
 $32 = ($31|0)!=(0|0);
 if ($32) {
  $33 = HEAP32[$1>>2]|0;
  $34 = HEAP32[$33>>2]|0;
  $35 = $34<<4;
  $36 = (24 + ($35))|0;
  HEAP32[$nByte>>2] = $36;
  $37 = HEAP32[$db>>2]|0;
  $38 = HEAP32[$1>>2]|0;
  $39 = HEAP32[$nByte>>2]|0;
  $40 = ($39|0)<(0);
  $41 = $40 << 31 >> 31;
  $42 = (_sqlite3DbRealloc($37,$38,$39,$41)|0);
  HEAP32[$pNew>>2] = $42;
 } else {
  $43 = HEAP32[$db>>2]|0;
  $44 = (_sqlite3DbMallocZero($43,24,0)|0);
  HEAP32[$pNew>>2] = $44;
 }
 $45 = HEAP32[$pNew>>2]|0;
 $46 = ($45|0)==(0|0);
 if ($46) {
  $47 = HEAP32[$db>>2]|0;
  $48 = HEAP32[$3>>2]|0;
  _sqlite3ExprListDelete($47,$48);
  $49 = HEAP32[$db>>2]|0;
  $50 = HEAP32[$4>>2]|0;
  _sqlite3SelectDelete($49,$50);
  $51 = HEAP32[$db>>2]|0;
  $52 = HEAP32[$zName>>2]|0;
  _sqlite3DbFree($51,$52);
  $53 = HEAP32[$1>>2]|0;
  HEAP32[$pNew>>2] = $53;
  $83 = HEAP32[$pNew>>2]|0;
  STACKTOP = sp;return ($83|0);
 } else {
  $54 = HEAP32[$4>>2]|0;
  $55 = HEAP32[$pNew>>2]|0;
  $56 = HEAP32[$55>>2]|0;
  $57 = HEAP32[$pNew>>2]|0;
  $58 = (($57) + 8|0);
  $59 = (($58) + ($56<<4)|0);
  $60 = (($59) + 8|0);
  HEAP32[$60>>2] = $54;
  $61 = HEAP32[$3>>2]|0;
  $62 = HEAP32[$pNew>>2]|0;
  $63 = HEAP32[$62>>2]|0;
  $64 = HEAP32[$pNew>>2]|0;
  $65 = (($64) + 8|0);
  $66 = (($65) + ($63<<4)|0);
  $67 = (($66) + 4|0);
  HEAP32[$67>>2] = $61;
  $68 = HEAP32[$zName>>2]|0;
  $69 = HEAP32[$pNew>>2]|0;
  $70 = HEAP32[$69>>2]|0;
  $71 = HEAP32[$pNew>>2]|0;
  $72 = (($71) + 8|0);
  $73 = (($72) + ($70<<4)|0);
  HEAP32[$73>>2] = $68;
  $74 = HEAP32[$pNew>>2]|0;
  $75 = HEAP32[$74>>2]|0;
  $76 = HEAP32[$pNew>>2]|0;
  $77 = (($76) + 8|0);
  $78 = (($77) + ($75<<4)|0);
  $79 = (($78) + 12|0);
  HEAP32[$79>>2] = 0;
  $80 = HEAP32[$pNew>>2]|0;
  $81 = HEAP32[$80>>2]|0;
  $82 = (($81) + 1)|0;
  HEAP32[$80>>2] = $82;
  $83 = HEAP32[$pNew>>2]|0;
  STACKTOP = sp;return ($83|0);
 }
 return 0|0;
}
function _yy_find_reduce_action($stateno,$iLookAhead) {
 $stateno = $stateno|0;
 $iLookAhead = $iLookAhead|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $i = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $1 = sp + 8|0;
 $i = sp;
 HEAP32[$0>>2] = $stateno;
 HEAP8[$1>>0] = $iLookAhead;
 $2 = HEAP32[$0>>2]|0;
 $3 = (11104 + ($2<<1)|0);
 $4 = HEAP16[$3>>1]|0;
 $5 = $4 << 16 >> 16;
 HEAP32[$i>>2] = $5;
 $6 = HEAP8[$1>>0]|0;
 $7 = $6&255;
 $8 = HEAP32[$i>>2]|0;
 $9 = (($8) + ($7))|0;
 HEAP32[$i>>2] = $9;
 $10 = HEAP32[$i>>2]|0;
 $11 = (11720 + ($10<<1)|0);
 $12 = HEAP16[$11>>1]|0;
 $13 = $12&65535;
 STACKTOP = sp;return ($13|0);
}
function _yy_accept($yypParser) {
 $yypParser = $yypParser|0;
 var $0 = 0, $1 = 0, $10 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $pParse = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $pParse = sp;
 HEAP32[$0>>2] = $yypParser;
 $1 = HEAP32[$0>>2]|0;
 $2 = (($1) + 8|0);
 $3 = HEAP32[$2>>2]|0;
 HEAP32[$pParse>>2] = $3;
 while(1) {
  $4 = HEAP32[$0>>2]|0;
  $5 = HEAP32[$4>>2]|0;
  $6 = ($5|0)>=(0);
  if (!($6)) {
   break;
  }
  $7 = HEAP32[$0>>2]|0;
  (_yy_pop_parser_stack($7)|0);
 }
 $8 = HEAP32[$pParse>>2]|0;
 $9 = HEAP32[$0>>2]|0;
 $10 = (($9) + 8|0);
 HEAP32[$10>>2] = $8;
 STACKTOP = sp;return;
}
function _sqlite3NameFromToken($db,$pName) {
 $db = $db|0;
 $pName = $pName|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $zName = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $1 = sp + 4|0;
 $zName = sp;
 HEAP32[$0>>2] = $db;
 HEAP32[$1>>2] = $pName;
 $2 = HEAP32[$1>>2]|0;
 $3 = ($2|0)!=(0|0);
 if ($3) {
  $4 = HEAP32[$0>>2]|0;
  $5 = HEAP32[$1>>2]|0;
  $6 = HEAP32[$5>>2]|0;
  $7 = HEAP32[$1>>2]|0;
  $8 = (($7) + 4|0);
  $9 = HEAP32[$8>>2]|0;
  $10 = (_sqlite3DbStrNDup($4,$6,$9,0)|0);
  HEAP32[$zName>>2] = $10;
  $11 = HEAP32[$zName>>2]|0;
  (_sqlite3Dequote($11)|0);
  $12 = HEAP32[$zName>>2]|0;
  STACKTOP = sp;return ($12|0);
 } else {
  HEAP32[$zName>>2] = 0;
  $12 = HEAP32[$zName>>2]|0;
  STACKTOP = sp;return ($12|0);
 }
 return 0|0;
}
function _addArgumentToVtab($pParse) {
 $pParse = $pParse|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $db = 0, $n = 0, $z = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $z = sp + 8|0;
 $n = sp + 4|0;
 $db = sp;
 HEAP32[$0>>2] = $pParse;
 $1 = HEAP32[$0>>2]|0;
 $2 = (($1) + 512|0);
 $3 = HEAP32[$2>>2]|0;
 $4 = ($3|0)!=(0|0);
 if (!($4)) {
  STACKTOP = sp;return;
 }
 $5 = HEAP32[$0>>2]|0;
 $6 = (($5) + 484|0);
 $7 = HEAP32[$6>>2]|0;
 $8 = ($7|0)!=(0|0);
 if (!($8)) {
  STACKTOP = sp;return;
 }
 $9 = HEAP32[$0>>2]|0;
 $10 = (($9) + 512|0);
 $11 = HEAP32[$10>>2]|0;
 HEAP32[$z>>2] = $11;
 $12 = HEAP32[$0>>2]|0;
 $13 = (($12) + 512|0);
 $14 = (($13) + 4|0);
 $15 = HEAP32[$14>>2]|0;
 HEAP32[$n>>2] = $15;
 $16 = HEAP32[$0>>2]|0;
 $17 = HEAP32[$16>>2]|0;
 HEAP32[$db>>2] = $17;
 $18 = HEAP32[$db>>2]|0;
 $19 = HEAP32[$0>>2]|0;
 $20 = (($19) + 484|0);
 $21 = HEAP32[$20>>2]|0;
 $22 = HEAP32[$db>>2]|0;
 $23 = HEAP32[$z>>2]|0;
 $24 = HEAP32[$n>>2]|0;
 $25 = ($24|0)<(0);
 $26 = $25 << 31 >> 31;
 $27 = (_sqlite3DbStrNDup($22,$23,$24,$26)|0);
 _addModuleArgument($18,$21,$27);
 STACKTOP = sp;return;
}
function _addModuleArgument($db,$pTable,$zArg) {
 $db = $db|0;
 $pTable = $pTable|0;
 $zArg = $zArg|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $azModuleArg = 0, $i = 0, $j = 0, $nBytes = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 24|0;
 $1 = sp + 20|0;
 $2 = sp + 16|0;
 $i = sp + 12|0;
 $nBytes = sp + 8|0;
 $azModuleArg = sp + 4|0;
 $j = sp;
 HEAP32[$0>>2] = $db;
 HEAP32[$1>>2] = $pTable;
 HEAP32[$2>>2] = $zArg;
 $3 = HEAP32[$1>>2]|0;
 $4 = (($3) + 52|0);
 $5 = HEAP32[$4>>2]|0;
 $6 = (($5) + 1)|0;
 HEAP32[$4>>2] = $6;
 HEAP32[$i>>2] = $5;
 $7 = HEAP32[$1>>2]|0;
 $8 = (($7) + 52|0);
 $9 = HEAP32[$8>>2]|0;
 $10 = (1 + ($9))|0;
 $11 = $10<<2;
 HEAP32[$nBytes>>2] = $11;
 $12 = HEAP32[$0>>2]|0;
 $13 = HEAP32[$1>>2]|0;
 $14 = (($13) + 56|0);
 $15 = HEAP32[$14>>2]|0;
 $16 = HEAP32[$nBytes>>2]|0;
 $17 = ($16|0)<(0);
 $18 = $17 << 31 >> 31;
 $19 = (_sqlite3DbRealloc($12,$15,$16,$18)|0);
 HEAP32[$azModuleArg>>2] = $19;
 $20 = HEAP32[$azModuleArg>>2]|0;
 $21 = ($20|0)==(0|0);
 if (!($21)) {
  $41 = HEAP32[$2>>2]|0;
  $42 = HEAP32[$i>>2]|0;
  $43 = HEAP32[$azModuleArg>>2]|0;
  $44 = (($43) + ($42<<2)|0);
  HEAP32[$44>>2] = $41;
  $45 = HEAP32[$i>>2]|0;
  $46 = (($45) + 1)|0;
  $47 = HEAP32[$azModuleArg>>2]|0;
  $48 = (($47) + ($46<<2)|0);
  HEAP32[$48>>2] = 0;
  $49 = HEAP32[$azModuleArg>>2]|0;
  $50 = HEAP32[$1>>2]|0;
  $51 = (($50) + 56|0);
  HEAP32[$51>>2] = $49;
  STACKTOP = sp;return;
 }
 HEAP32[$j>>2] = 0;
 while(1) {
  $22 = HEAP32[$j>>2]|0;
  $23 = HEAP32[$i>>2]|0;
  $24 = ($22|0)<($23|0);
  $25 = HEAP32[$0>>2]|0;
  if (!($24)) {
   break;
  }
  $26 = HEAP32[$j>>2]|0;
  $27 = HEAP32[$1>>2]|0;
  $28 = (($27) + 56|0);
  $29 = HEAP32[$28>>2]|0;
  $30 = (($29) + ($26<<2)|0);
  $31 = HEAP32[$30>>2]|0;
  _sqlite3DbFree($25,$31);
  $32 = HEAP32[$j>>2]|0;
  $33 = (($32) + 1)|0;
  HEAP32[$j>>2] = $33;
 }
 $34 = HEAP32[$2>>2]|0;
 _sqlite3DbFree($25,$34);
 $35 = HEAP32[$0>>2]|0;
 $36 = HEAP32[$1>>2]|0;
 $37 = (($36) + 56|0);
 $38 = HEAP32[$37>>2]|0;
 _sqlite3DbFree($35,$38);
 $39 = HEAP32[$1>>2]|0;
 $40 = (($39) + 52|0);
 HEAP32[$40>>2] = 0;
 $49 = HEAP32[$azModuleArg>>2]|0;
 $50 = HEAP32[$1>>2]|0;
 $51 = (($50) + 56|0);
 HEAP32[$51>>2] = $49;
 STACKTOP = sp;return;
}
function _sqlite3AuthCheck($pParse,$code,$zArg1,$zArg2,$zArg3) {
 $pParse = $pParse|0;
 $code = $code|0;
 $zArg1 = $zArg1|0;
 $zArg2 = $zArg2|0;
 $zArg3 = $zArg3|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $db = 0, $or$cond = 0, $rc = 0, $vararg_buffer = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $vararg_buffer = sp;
 $0 = sp + 32|0;
 $1 = sp + 28|0;
 $2 = sp + 24|0;
 $3 = sp + 20|0;
 $4 = sp + 16|0;
 $5 = sp + 12|0;
 $db = sp + 8|0;
 $rc = sp + 4|0;
 HEAP32[$1>>2] = $pParse;
 HEAP32[$2>>2] = $code;
 HEAP32[$3>>2] = $zArg1;
 HEAP32[$4>>2] = $zArg2;
 HEAP32[$5>>2] = $zArg3;
 $6 = HEAP32[$1>>2]|0;
 $7 = HEAP32[$6>>2]|0;
 HEAP32[$db>>2] = $7;
 $8 = HEAP32[$db>>2]|0;
 $9 = (($8) + 140|0);
 $10 = (($9) + 5|0);
 $11 = HEAP8[$10>>0]|0;
 $12 = $11&255;
 $13 = ($12|0)!=(0);
 if (!($13)) {
  $14 = HEAP32[$1>>2]|0;
  $15 = (($14) + 451|0);
  $16 = HEAP8[$15>>0]|0;
  $17 = $16&255;
  $18 = ($17|0)!=(0);
  if (!($18)) {
   $19 = HEAP32[$db>>2]|0;
   $20 = (($19) + 284|0);
   $21 = HEAP32[$20>>2]|0;
   $22 = ($21|0)==(0|0);
   if ($22) {
    HEAP32[$0>>2] = 0;
    $48 = HEAP32[$0>>2]|0;
    STACKTOP = sp;return ($48|0);
   }
   $23 = HEAP32[$db>>2]|0;
   $24 = (($23) + 284|0);
   $25 = HEAP32[$24>>2]|0;
   $26 = HEAP32[$db>>2]|0;
   $27 = (($26) + 288|0);
   $28 = HEAP32[$27>>2]|0;
   $29 = HEAP32[$2>>2]|0;
   $30 = HEAP32[$3>>2]|0;
   $31 = HEAP32[$4>>2]|0;
   $32 = HEAP32[$5>>2]|0;
   $33 = HEAP32[$1>>2]|0;
   $34 = (($33) + 492|0);
   $35 = HEAP32[$34>>2]|0;
   $36 = (FUNCTION_TABLE_iiiiiii[$25 & 1]($28,$29,$30,$31,$32,$35)|0);
   HEAP32[$rc>>2] = $36;
   $37 = HEAP32[$rc>>2]|0;
   $38 = ($37|0)==(1);
   if ($38) {
    $39 = HEAP32[$1>>2]|0;
    _sqlite3ErrorMsg($39,14752,$vararg_buffer);
    $40 = HEAP32[$1>>2]|0;
    $41 = (($40) + 12|0);
    HEAP32[$41>>2] = 23;
   } else {
    $42 = HEAP32[$rc>>2]|0;
    $43 = ($42|0)!=(0);
    $44 = HEAP32[$rc>>2]|0;
    $45 = ($44|0)!=(2);
    $or$cond = $43 & $45;
    if ($or$cond) {
     HEAP32[$rc>>2] = 1;
     $46 = HEAP32[$1>>2]|0;
     _sqliteAuthBadReturnCode($46);
    }
   }
   $47 = HEAP32[$rc>>2]|0;
   HEAP32[$0>>2] = $47;
   $48 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($48|0);
  }
 }
 HEAP32[$0>>2] = 0;
 $48 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($48|0);
}
function _sqliteAuthBadReturnCode($pParse) {
 $pParse = $pParse|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $vararg_buffer = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $vararg_buffer = sp;
 $0 = sp + 4|0;
 HEAP32[$0>>2] = $pParse;
 $1 = HEAP32[$0>>2]|0;
 _sqlite3ErrorMsg($1,14768,$vararg_buffer);
 $2 = HEAP32[$0>>2]|0;
 $3 = (($2) + 12|0);
 HEAP32[$3>>2] = 1;
 STACKTOP = sp;return;
}
function _sqlite3NestedParse($pParse,$zFormat,$varargs) {
 $pParse = $pParse|0;
 $zFormat = $zFormat|0;
 $varargs = $varargs|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $ap = 0, $db = 0, $saveBuf = 0, $zErrMsg = 0, $zSql = 0, dest = 0, label = 0;
 var sp = 0, src = 0, stop = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 144|0;
 $0 = sp + 36|0;
 $1 = sp + 32|0;
 $ap = sp + 16|0;
 $zSql = sp + 8|0;
 $zErrMsg = sp + 4|0;
 $db = sp;
 $saveBuf = sp + 40|0;
 HEAP32[$0>>2] = $pParse;
 HEAP32[$1>>2] = $zFormat;
 HEAP32[$zErrMsg>>2] = 0;
 $2 = HEAP32[$0>>2]|0;
 $3 = HEAP32[$2>>2]|0;
 HEAP32[$db>>2] = $3;
 $4 = HEAP32[$0>>2]|0;
 $5 = (($4) + 64|0);
 $6 = HEAP32[$5>>2]|0;
 $7 = ($6|0)!=(0);
 if ($7) {
  STACKTOP = sp;return;
 }
 HEAP32[$ap>>2] = $varargs;
 $8 = HEAP32[$db>>2]|0;
 $9 = HEAP32[$1>>2]|0;
 $10 = (_sqlite3VMPrintf($8,$9,$ap)|0);
 HEAP32[$zSql>>2] = $10;
 $11 = HEAP32[$zSql>>2]|0;
 $12 = ($11|0)==(0|0);
 if ($12) {
  STACKTOP = sp;return;
 }
 $13 = HEAP32[$0>>2]|0;
 $14 = (($13) + 18|0);
 $15 = HEAP8[$14>>0]|0;
 $16 = (($15) + 1)<<24>>24;
 HEAP8[$14>>0] = $16;
 $17 = HEAP32[$0>>2]|0;
 $18 = (($17) + 440|0);
 dest=$saveBuf+0|0; src=$18+0|0; stop=dest+96|0; do { HEAP8[dest>>0]=HEAP8[src>>0]|0; dest=dest+1|0; src=src+1|0; } while ((dest|0) < (stop|0));
 $19 = HEAP32[$0>>2]|0;
 $20 = (($19) + 440|0);
 dest=$20+0|0; stop=dest+96|0; do { HEAP32[dest>>2]=0|0; dest=dest+4|0; } while ((dest|0) < (stop|0));
 $21 = HEAP32[$0>>2]|0;
 $22 = HEAP32[$zSql>>2]|0;
 (_sqlite3RunParser($21,$22,$zErrMsg)|0);
 $23 = HEAP32[$db>>2]|0;
 $24 = HEAP32[$zErrMsg>>2]|0;
 _sqlite3DbFree($23,$24);
 $25 = HEAP32[$db>>2]|0;
 $26 = HEAP32[$zSql>>2]|0;
 _sqlite3DbFree($25,$26);
 $27 = HEAP32[$0>>2]|0;
 $28 = (($27) + 440|0);
 dest=$28+0|0; src=$saveBuf+0|0; stop=dest+96|0; do { HEAP8[dest>>0]=HEAP8[src>>0]|0; dest=dest+1|0; src=src+1|0; } while ((dest|0) < (stop|0));
 $29 = HEAP32[$0>>2]|0;
 $30 = (($29) + 18|0);
 $31 = HEAP8[$30>>0]|0;
 $32 = (($31) + -1)<<24>>24;
 HEAP8[$30>>0] = $32;
 STACKTOP = sp;return;
}
function _sqlite3GetVdbe($pParse) {
 $pParse = $pParse|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $v = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $v = sp;
 HEAP32[$0>>2] = $pParse;
 $1 = HEAP32[$0>>2]|0;
 $2 = (($1) + 8|0);
 $3 = HEAP32[$2>>2]|0;
 HEAP32[$v>>2] = $3;
 $4 = HEAP32[$v>>2]|0;
 $5 = ($4|0)==(0|0);
 if ($5) {
  $6 = HEAP32[$0>>2]|0;
  $7 = (_sqlite3VdbeCreate($6)|0);
  $8 = HEAP32[$0>>2]|0;
  $9 = (($8) + 8|0);
  HEAP32[$9>>2] = $7;
  HEAP32[$v>>2] = $7;
  $10 = HEAP32[$v>>2]|0;
  $11 = ($10|0)!=(0|0);
  if ($11) {
   $12 = HEAP32[$v>>2]|0;
   (_sqlite3VdbeAddOp0($12,153)|0);
  }
  $13 = HEAP32[$0>>2]|0;
  $14 = (($13) + 408|0);
  $15 = HEAP32[$14>>2]|0;
  $16 = ($15|0)==(0|0);
  if ($16) {
   $17 = HEAP32[$0>>2]|0;
   $18 = HEAP32[$17>>2]|0;
   $19 = (($18) + 60|0);
   $20 = HEAP16[$19>>1]|0;
   $21 = $20&65535;
   $22 = $21 & 8;
   $23 = ($22|0)==(0);
   if ($23) {
    $24 = HEAP32[$0>>2]|0;
    $25 = (($24) + 23|0);
    HEAP8[$25>>0] = 1;
   }
  }
 }
 $26 = HEAP32[$v>>2]|0;
 STACKTOP = sp;return ($26|0);
}
function _sqlite3ChangeCookie($pParse,$iDb) {
 $pParse = $pParse|0;
 $iDb = $iDb|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $3 = 0, $4 = 0;
 var $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $db = 0, $r1 = 0, $v = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 16|0;
 $1 = sp + 12|0;
 $r1 = sp + 8|0;
 $db = sp + 4|0;
 $v = sp;
 HEAP32[$0>>2] = $pParse;
 HEAP32[$1>>2] = $iDb;
 $2 = HEAP32[$0>>2]|0;
 $3 = (_sqlite3GetTempReg($2)|0);
 HEAP32[$r1>>2] = $3;
 $4 = HEAP32[$0>>2]|0;
 $5 = HEAP32[$4>>2]|0;
 HEAP32[$db>>2] = $5;
 $6 = HEAP32[$0>>2]|0;
 $7 = (($6) + 8|0);
 $8 = HEAP32[$7>>2]|0;
 HEAP32[$v>>2] = $8;
 $9 = HEAP32[$v>>2]|0;
 $10 = HEAP32[$1>>2]|0;
 $11 = HEAP32[$db>>2]|0;
 $12 = (($11) + 16|0);
 $13 = HEAP32[$12>>2]|0;
 $14 = (($13) + ($10<<4)|0);
 $15 = (($14) + 12|0);
 $16 = HEAP32[$15>>2]|0;
 $17 = HEAP32[$16>>2]|0;
 $18 = (($17) + 1)|0;
 $19 = HEAP32[$r1>>2]|0;
 (_sqlite3VdbeAddOp2($9,25,$18,$19)|0);
 $20 = HEAP32[$v>>2]|0;
 $21 = HEAP32[$1>>2]|0;
 $22 = HEAP32[$r1>>2]|0;
 (_sqlite3VdbeAddOp3($20,52,$21,1,$22)|0);
 $23 = HEAP32[$0>>2]|0;
 $24 = HEAP32[$r1>>2]|0;
 _sqlite3ReleaseTempReg($23,$24);
 STACKTOP = sp;return;
}
function _sqlite3VdbeAddOp2($p,$op,$p1,$p2) {
 $p = $p|0;
 $op = $op|0;
 $p1 = $p1|0;
 $p2 = $p2|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $1 = sp + 8|0;
 $2 = sp + 4|0;
 $3 = sp;
 HEAP32[$0>>2] = $p;
 HEAP32[$1>>2] = $op;
 HEAP32[$2>>2] = $p1;
 HEAP32[$3>>2] = $p2;
 $4 = HEAP32[$0>>2]|0;
 $5 = HEAP32[$1>>2]|0;
 $6 = HEAP32[$2>>2]|0;
 $7 = HEAP32[$3>>2]|0;
 $8 = (_sqlite3VdbeAddOp3($4,$5,$6,$7,0)|0);
 STACKTOP = sp;return ($8|0);
}
function _sqlite3VdbeAddParseSchemaOp($p,$iDb,$zWhere) {
 $p = $p|0;
 $iDb = $iDb|0;
 $zWhere = $zWhere|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $addr = 0;
 var $j = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 16|0;
 $1 = sp + 12|0;
 $2 = sp + 8|0;
 $j = sp + 4|0;
 $addr = sp;
 HEAP32[$0>>2] = $p;
 HEAP32[$1>>2] = $iDb;
 HEAP32[$2>>2] = $zWhere;
 $3 = HEAP32[$0>>2]|0;
 $4 = HEAP32[$1>>2]|0;
 $5 = (_sqlite3VdbeAddOp3($3,122,$4,0,0)|0);
 HEAP32[$addr>>2] = $5;
 $6 = HEAP32[$0>>2]|0;
 $7 = HEAP32[$addr>>2]|0;
 $8 = HEAP32[$2>>2]|0;
 _sqlite3VdbeChangeP4($6,$7,$8,-1);
 HEAP32[$j>>2] = 0;
 while(1) {
  $9 = HEAP32[$j>>2]|0;
  $10 = HEAP32[$0>>2]|0;
  $11 = HEAP32[$10>>2]|0;
  $12 = (($11) + 20|0);
  $13 = HEAP32[$12>>2]|0;
  $14 = ($9|0)<($13|0);
  if (!($14)) {
   break;
  }
  $15 = HEAP32[$0>>2]|0;
  $16 = HEAP32[$j>>2]|0;
  _sqlite3VdbeUsesBtree($15,$16);
  $17 = HEAP32[$j>>2]|0;
  $18 = (($17) + 1)|0;
  HEAP32[$j>>2] = $18;
 }
 STACKTOP = sp;return;
}
function _sqlite3VdbeAddOp4($p,$op,$p1,$p2,$p3,$zP4,$p4type) {
 $p = $p|0;
 $op = $op|0;
 $p1 = $p1|0;
 $p2 = $p2|0;
 $p3 = $p3|0;
 $zP4 = $zP4|0;
 $p4type = $p4type|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $addr = 0, label = 0;
 var sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 28|0;
 $1 = sp + 24|0;
 $2 = sp + 20|0;
 $3 = sp + 16|0;
 $4 = sp + 12|0;
 $5 = sp + 8|0;
 $6 = sp + 4|0;
 $addr = sp;
 HEAP32[$0>>2] = $p;
 HEAP32[$1>>2] = $op;
 HEAP32[$2>>2] = $p1;
 HEAP32[$3>>2] = $p2;
 HEAP32[$4>>2] = $p3;
 HEAP32[$5>>2] = $zP4;
 HEAP32[$6>>2] = $p4type;
 $7 = HEAP32[$0>>2]|0;
 $8 = HEAP32[$1>>2]|0;
 $9 = HEAP32[$2>>2]|0;
 $10 = HEAP32[$3>>2]|0;
 $11 = HEAP32[$4>>2]|0;
 $12 = (_sqlite3VdbeAddOp3($7,$8,$9,$10,$11)|0);
 HEAP32[$addr>>2] = $12;
 $13 = HEAP32[$0>>2]|0;
 $14 = HEAP32[$addr>>2]|0;
 $15 = HEAP32[$5>>2]|0;
 $16 = HEAP32[$6>>2]|0;
 _sqlite3VdbeChangeP4($13,$14,$15,$16);
 $17 = HEAP32[$addr>>2]|0;
 STACKTOP = sp;return ($17|0);
}
function _sqlite3VdbeAddOp3($p,$op,$p1,$p2,$p3) {
 $p = $p|0;
 $op = $op|0;
 $p1 = $p1|0;
 $p2 = $p2|0;
 $p3 = $p3|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $i = 0, $pOp = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 28|0;
 $1 = sp + 24|0;
 $2 = sp + 20|0;
 $3 = sp + 16|0;
 $4 = sp + 12|0;
 $5 = sp + 8|0;
 $i = sp + 4|0;
 $pOp = sp;
 HEAP32[$1>>2] = $p;
 HEAP32[$2>>2] = $op;
 HEAP32[$3>>2] = $p1;
 HEAP32[$4>>2] = $p2;
 HEAP32[$5>>2] = $p3;
 $6 = HEAP32[$1>>2]|0;
 $7 = (($6) + 32|0);
 $8 = HEAP32[$7>>2]|0;
 HEAP32[$i>>2] = $8;
 $9 = HEAP32[$1>>2]|0;
 $10 = (($9) + 24|0);
 $11 = HEAP32[$10>>2]|0;
 $12 = (($11) + 84|0);
 $13 = HEAP32[$12>>2]|0;
 $14 = HEAP32[$i>>2]|0;
 $15 = ($13|0)<=($14|0);
 if ($15) {
  $16 = HEAP32[$1>>2]|0;
  $17 = (_growOpArray($16,1)|0);
  $18 = ($17|0)!=(0);
  if ($18) {
   HEAP32[$0>>2] = 1;
   $47 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($47|0);
  }
 }
 $19 = HEAP32[$1>>2]|0;
 $20 = (($19) + 32|0);
 $21 = HEAP32[$20>>2]|0;
 $22 = (($21) + 1)|0;
 HEAP32[$20>>2] = $22;
 $23 = HEAP32[$i>>2]|0;
 $24 = HEAP32[$1>>2]|0;
 $25 = (($24) + 4|0);
 $26 = HEAP32[$25>>2]|0;
 $27 = (($26) + (($23*20)|0)|0);
 HEAP32[$pOp>>2] = $27;
 $28 = HEAP32[$2>>2]|0;
 $29 = $28&255;
 $30 = HEAP32[$pOp>>2]|0;
 HEAP8[$30>>0] = $29;
 $31 = HEAP32[$pOp>>2]|0;
 $32 = (($31) + 3|0);
 HEAP8[$32>>0] = 0;
 $33 = HEAP32[$3>>2]|0;
 $34 = HEAP32[$pOp>>2]|0;
 $35 = (($34) + 4|0);
 HEAP32[$35>>2] = $33;
 $36 = HEAP32[$4>>2]|0;
 $37 = HEAP32[$pOp>>2]|0;
 $38 = (($37) + 8|0);
 HEAP32[$38>>2] = $36;
 $39 = HEAP32[$5>>2]|0;
 $40 = HEAP32[$pOp>>2]|0;
 $41 = (($40) + 12|0);
 HEAP32[$41>>2] = $39;
 $42 = HEAP32[$pOp>>2]|0;
 $43 = (($42) + 16|0);
 HEAP32[$43>>2] = 0;
 $44 = HEAP32[$pOp>>2]|0;
 $45 = (($44) + 1|0);
 HEAP8[$45>>0] = 0;
 $46 = HEAP32[$i>>2]|0;
 HEAP32[$0>>2] = $46;
 $47 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($47|0);
}
function _growOpArray($v,$nOp) {
 $v = $v|0;
 $nOp = $nOp|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $nNew = 0, $p = 0, $pNew = 0;
 var label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 16|0;
 $1 = sp + 12|0;
 $pNew = sp + 8|0;
 $p = sp + 4|0;
 $nNew = sp;
 HEAP32[$0>>2] = $v;
 HEAP32[$1>>2] = $nOp;
 $2 = HEAP32[$0>>2]|0;
 $3 = (($2) + 24|0);
 $4 = HEAP32[$3>>2]|0;
 HEAP32[$p>>2] = $4;
 $5 = HEAP32[$p>>2]|0;
 $6 = (($5) + 84|0);
 $7 = HEAP32[$6>>2]|0;
 $8 = ($7|0)!=(0);
 if ($8) {
  $9 = HEAP32[$p>>2]|0;
  $10 = (($9) + 84|0);
  $11 = HEAP32[$10>>2]|0;
  $12 = $11<<1;
  $13 = $12;
 } else {
  $13 = 51;
 }
 HEAP32[$nNew>>2] = $13;
 $14 = HEAP32[$p>>2]|0;
 $15 = HEAP32[$14>>2]|0;
 $16 = HEAP32[$0>>2]|0;
 $17 = (($16) + 4|0);
 $18 = HEAP32[$17>>2]|0;
 $19 = HEAP32[$nNew>>2]|0;
 $20 = ($19*20)|0;
 $21 = (_sqlite3DbRealloc($15,$18,$20,0)|0);
 HEAP32[$pNew>>2] = $21;
 $22 = HEAP32[$pNew>>2]|0;
 $23 = ($22|0)!=(0|0);
 if (!($23)) {
  $34 = HEAP32[$pNew>>2]|0;
  $35 = ($34|0)!=(0|0);
  $36 = $35 ? 0 : 7;
  STACKTOP = sp;return ($36|0);
 }
 $24 = HEAP32[$p>>2]|0;
 $25 = HEAP32[$24>>2]|0;
 $26 = HEAP32[$pNew>>2]|0;
 $27 = (_sqlite3DbMallocSize($25,$26)|0);
 $28 = (($27>>>0) / 20)&-1;
 $29 = HEAP32[$p>>2]|0;
 $30 = (($29) + 84|0);
 HEAP32[$30>>2] = $28;
 $31 = HEAP32[$pNew>>2]|0;
 $32 = HEAP32[$0>>2]|0;
 $33 = (($32) + 4|0);
 HEAP32[$33>>2] = $31;
 $34 = HEAP32[$pNew>>2]|0;
 $35 = ($34|0)!=(0|0);
 $36 = $35 ? 0 : 7;
 STACKTOP = sp;return ($36|0);
}
function _sqlite3GetTempReg($pParse) {
 $pParse = $pParse|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0;
 var label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $1 = sp;
 HEAP32[$1>>2] = $pParse;
 $2 = HEAP32[$1>>2]|0;
 $3 = (($2) + 19|0);
 $4 = HEAP8[$3>>0]|0;
 $5 = $4&255;
 $6 = ($5|0)==(0);
 $7 = HEAP32[$1>>2]|0;
 if ($6) {
  $8 = (($7) + 72|0);
  $9 = HEAP32[$8>>2]|0;
  $10 = (($9) + 1)|0;
  HEAP32[$8>>2] = $10;
  HEAP32[$0>>2] = $10;
  $19 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($19|0);
 } else {
  $11 = (($7) + 19|0);
  $12 = HEAP8[$11>>0]|0;
  $13 = (($12) + -1)<<24>>24;
  HEAP8[$11>>0] = $13;
  $14 = $13&255;
  $15 = HEAP32[$1>>2]|0;
  $16 = (($15) + 24|0);
  $17 = (($16) + ($14<<2)|0);
  $18 = HEAP32[$17>>2]|0;
  HEAP32[$0>>2] = $18;
  $19 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($19|0);
 }
 return 0|0;
}
function _sqlite3ReleaseTempReg($pParse,$iReg) {
 $pParse = $pParse|0;
 $iReg = $iReg|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $i = 0, $p = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $1 = sp + 8|0;
 $i = sp + 4|0;
 $p = sp;
 HEAP32[$0>>2] = $pParse;
 HEAP32[$1>>2] = $iReg;
 $2 = HEAP32[$1>>2]|0;
 $3 = ($2|0)!=(0);
 if (!($3)) {
  STACKTOP = sp;return;
 }
 $4 = HEAP32[$0>>2]|0;
 $5 = (($4) + 19|0);
 $6 = HEAP8[$5>>0]|0;
 $7 = $6&255;
 $8 = ($7|0)<(8);
 if (!($8)) {
  STACKTOP = sp;return;
 }
 HEAP32[$i>>2] = 0;
 $9 = HEAP32[$0>>2]|0;
 $10 = (($9) + 116|0);
 HEAP32[$p>>2] = $10;
 while(1) {
  $11 = HEAP32[$i>>2]|0;
  $12 = ($11|0)<(10);
  if (!($12)) {
   label = 8;
   break;
  }
  $13 = HEAP32[$p>>2]|0;
  $14 = (($13) + 12|0);
  $15 = HEAP32[$14>>2]|0;
  $16 = HEAP32[$1>>2]|0;
  $17 = ($15|0)==($16|0);
  if ($17) {
   label = 6;
   break;
  }
  $20 = HEAP32[$i>>2]|0;
  $21 = (($20) + 1)|0;
  HEAP32[$i>>2] = $21;
  $22 = HEAP32[$p>>2]|0;
  $23 = (($22) + 20|0);
  HEAP32[$p>>2] = $23;
 }
 if ((label|0) == 6) {
  $18 = HEAP32[$p>>2]|0;
  $19 = (($18) + 6|0);
  HEAP8[$19>>0] = 1;
  STACKTOP = sp;return;
 }
 else if ((label|0) == 8) {
  $24 = HEAP32[$1>>2]|0;
  $25 = HEAP32[$0>>2]|0;
  $26 = (($25) + 19|0);
  $27 = HEAP8[$26>>0]|0;
  $28 = (($27) + 1)<<24>>24;
  HEAP8[$26>>0] = $28;
  $29 = $27&255;
  $30 = HEAP32[$0>>2]|0;
  $31 = (($30) + 24|0);
  $32 = (($31) + ($29<<2)|0);
  HEAP32[$32>>2] = $24;
  STACKTOP = sp;return;
 }
}
function _sqlite3VdbeAddOp0($p,$op) {
 $p = $p|0;
 $op = $op|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $1 = sp;
 HEAP32[$0>>2] = $p;
 HEAP32[$1>>2] = $op;
 $2 = HEAP32[$0>>2]|0;
 $3 = HEAP32[$1>>2]|0;
 $4 = (_sqlite3VdbeAddOp3($2,$3,0,0,0)|0);
 STACKTOP = sp;return ($4|0);
}
function _sqlite3LocateTableItem($pParse,$isView,$p) {
 $pParse = $pParse|0;
 $isView = $isView|0;
 $p = $p|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $iDb = 0, $zDb = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 16|0;
 $1 = sp + 12|0;
 $2 = sp + 8|0;
 $zDb = sp + 4|0;
 $iDb = sp;
 HEAP32[$0>>2] = $pParse;
 HEAP32[$1>>2] = $isView;
 HEAP32[$2>>2] = $p;
 $3 = HEAP32[$2>>2]|0;
 $4 = HEAP32[$3>>2]|0;
 $5 = ($4|0)!=(0|0);
 if ($5) {
  $6 = HEAP32[$0>>2]|0;
  $7 = HEAP32[$6>>2]|0;
  $8 = HEAP32[$2>>2]|0;
  $9 = HEAP32[$8>>2]|0;
  $10 = (_sqlite3SchemaToIndex($7,$9)|0);
  HEAP32[$iDb>>2] = $10;
  $11 = HEAP32[$iDb>>2]|0;
  $12 = HEAP32[$0>>2]|0;
  $13 = HEAP32[$12>>2]|0;
  $14 = (($13) + 16|0);
  $15 = HEAP32[$14>>2]|0;
  $16 = (($15) + ($11<<4)|0);
  $17 = HEAP32[$16>>2]|0;
  HEAP32[$zDb>>2] = $17;
 } else {
  $18 = HEAP32[$2>>2]|0;
  $19 = (($18) + 4|0);
  $20 = HEAP32[$19>>2]|0;
  HEAP32[$zDb>>2] = $20;
 }
 $21 = HEAP32[$0>>2]|0;
 $22 = HEAP32[$1>>2]|0;
 $23 = HEAP32[$2>>2]|0;
 $24 = (($23) + 8|0);
 $25 = HEAP32[$24>>2]|0;
 $26 = HEAP32[$zDb>>2]|0;
 $27 = (_sqlite3LocateTable($21,$22,$25,$26)|0);
 STACKTOP = sp;return ($27|0);
}
function _isSystemTable($pParse,$zName) {
 $pParse = $pParse|0;
 $zName = $zName|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $vararg_buffer = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $vararg_buffer = sp;
 $0 = sp + 12|0;
 $1 = sp + 8|0;
 $2 = sp + 4|0;
 HEAP32[$1>>2] = $pParse;
 HEAP32[$2>>2] = $zName;
 $3 = HEAP32[$2>>2]|0;
 $4 = (_sqlite3Strlen30($3)|0);
 $5 = ($4|0)>(6);
 if ($5) {
  $6 = HEAP32[$2>>2]|0;
  $7 = (_sqlite3_strnicmp($6,15144,7)|0);
  $8 = (0)==($7|0);
  if ($8) {
   $9 = HEAP32[$1>>2]|0;
   $10 = HEAP32[$2>>2]|0;
   HEAP32[$vararg_buffer>>2] = $10;
   _sqlite3ErrorMsg($9,15152,$vararg_buffer);
   HEAP32[$0>>2] = 1;
   $11 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($11|0);
  }
 }
 HEAP32[$0>>2] = 0;
 $11 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($11|0);
}
function _sqlite3BeginWriteOperation($pParse,$setStatement,$iDb) {
 $pParse = $pParse|0;
 $setStatement = $setStatement|0;
 $iDb = $iDb|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $3 = 0;
 var $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $pToplevel = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $1 = sp + 8|0;
 $2 = sp + 4|0;
 $pToplevel = sp;
 HEAP32[$0>>2] = $pParse;
 HEAP32[$1>>2] = $setStatement;
 HEAP32[$2>>2] = $iDb;
 $3 = HEAP32[$0>>2]|0;
 $4 = (($3) + 408|0);
 $5 = HEAP32[$4>>2]|0;
 $6 = ($5|0)!=(0|0);
 $7 = HEAP32[$0>>2]|0;
 if ($6) {
  $8 = (($7) + 408|0);
  $9 = HEAP32[$8>>2]|0;
  $10 = $9;
 } else {
  $10 = $7;
 }
 HEAP32[$pToplevel>>2] = $10;
 $11 = HEAP32[$0>>2]|0;
 $12 = HEAP32[$2>>2]|0;
 _sqlite3CodeVerifySchema($11,$12);
 $13 = HEAP32[$2>>2]|0;
 $14 = 1 << $13;
 $15 = HEAP32[$pToplevel>>2]|0;
 $16 = (($15) + 328|0);
 $17 = HEAP32[$16>>2]|0;
 $18 = $17 | $14;
 HEAP32[$16>>2] = $18;
 $19 = HEAP32[$1>>2]|0;
 $20 = HEAP32[$pToplevel>>2]|0;
 $21 = (($20) + 20|0);
 $22 = HEAP8[$21>>0]|0;
 $23 = $22&255;
 $24 = $23 | $19;
 $25 = $24&255;
 HEAP8[$21>>0] = $25;
 STACKTOP = sp;return;
}
function _sqlite3CodeVerifySchema($pParse,$iDb) {
 $pParse = $pParse|0;
 $iDb = $iDb|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $5 = 0, $6 = 0, $7 = 0;
 var $8 = 0, $9 = 0, $db = 0, $pToplevel = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $1 = sp + 8|0;
 $pToplevel = sp + 4|0;
 $db = sp;
 HEAP32[$0>>2] = $pParse;
 HEAP32[$1>>2] = $iDb;
 $2 = HEAP32[$0>>2]|0;
 $3 = (($2) + 408|0);
 $4 = HEAP32[$3>>2]|0;
 $5 = ($4|0)!=(0|0);
 $6 = HEAP32[$0>>2]|0;
 if ($5) {
  $7 = (($6) + 408|0);
  $8 = HEAP32[$7>>2]|0;
  $9 = $8;
 } else {
  $9 = $6;
 }
 HEAP32[$pToplevel>>2] = $9;
 $10 = HEAP32[$pToplevel>>2]|0;
 $11 = HEAP32[$10>>2]|0;
 HEAP32[$db>>2] = $11;
 $12 = HEAP32[$pToplevel>>2]|0;
 $13 = (($12) + 332|0);
 $14 = HEAP32[$13>>2]|0;
 $15 = HEAP32[$1>>2]|0;
 $16 = 1 << $15;
 $17 = $14 & $16;
 $18 = ($17|0)!=(0);
 $19 = $18&1;
 $20 = ($19|0)==(0);
 if (!($20)) {
  STACKTOP = sp;return;
 }
 $21 = HEAP32[$1>>2]|0;
 $22 = 1 << $21;
 $23 = HEAP32[$pToplevel>>2]|0;
 $24 = (($23) + 332|0);
 $25 = HEAP32[$24>>2]|0;
 $26 = $25 | $22;
 HEAP32[$24>>2] = $26;
 $27 = HEAP32[$1>>2]|0;
 $28 = HEAP32[$db>>2]|0;
 $29 = (($28) + 16|0);
 $30 = HEAP32[$29>>2]|0;
 $31 = (($30) + ($27<<4)|0);
 $32 = (($31) + 12|0);
 $33 = HEAP32[$32>>2]|0;
 $34 = HEAP32[$33>>2]|0;
 $35 = HEAP32[$1>>2]|0;
 $36 = HEAP32[$pToplevel>>2]|0;
 $37 = (($36) + 336|0);
 $38 = (($37) + ($35<<2)|0);
 HEAP32[$38>>2] = $34;
 $39 = HEAP32[$1>>2]|0;
 $40 = ($39|0)==(1);
 if (!($40)) {
  STACKTOP = sp;return;
 }
 $41 = HEAP32[$pToplevel>>2]|0;
 (_sqlite3OpenTempDatabase($41)|0);
 STACKTOP = sp;return;
}
function _sqlite3OpenTempDatabase($pParse) {
 $pParse = $pParse|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0;
 var $db = 0, $pBt = 0, $rc = 0, $vararg_buffer = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $vararg_buffer = sp;
 $0 = sp + 20|0;
 $1 = sp + 16|0;
 $db = sp + 12|0;
 $rc = sp + 8|0;
 $pBt = sp + 4|0;
 HEAP32[$1>>2] = $pParse;
 $2 = HEAP32[$1>>2]|0;
 $3 = HEAP32[$2>>2]|0;
 HEAP32[$db>>2] = $3;
 $4 = HEAP32[$db>>2]|0;
 $5 = (($4) + 16|0);
 $6 = HEAP32[$5>>2]|0;
 $7 = (($6) + 16|0);
 $8 = (($7) + 4|0);
 $9 = HEAP32[$8>>2]|0;
 $10 = ($9|0)==(0|0);
 if ($10) {
  $11 = HEAP32[$1>>2]|0;
  $12 = (($11) + 450|0);
  $13 = HEAP8[$12>>0]|0;
  $14 = ($13<<24>>24)!=(0);
  if (!($14)) {
   $15 = HEAP32[$db>>2]|0;
   $16 = HEAP32[$15>>2]|0;
   $17 = HEAP32[$db>>2]|0;
   $18 = (_sqlite3BtreeOpen($16,0,$17,$pBt,0,542)|0);
   HEAP32[$rc>>2] = $18;
   $19 = HEAP32[$rc>>2]|0;
   $20 = ($19|0)!=(0);
   if ($20) {
    $21 = HEAP32[$1>>2]|0;
    _sqlite3ErrorMsg($21,15072,$vararg_buffer);
    $22 = HEAP32[$rc>>2]|0;
    $23 = HEAP32[$1>>2]|0;
    $24 = (($23) + 12|0);
    HEAP32[$24>>2] = $22;
    HEAP32[$0>>2] = 1;
    $39 = HEAP32[$0>>2]|0;
    STACKTOP = sp;return ($39|0);
   }
   $25 = HEAP32[$pBt>>2]|0;
   $26 = HEAP32[$db>>2]|0;
   $27 = (($26) + 16|0);
   $28 = HEAP32[$27>>2]|0;
   $29 = (($28) + 16|0);
   $30 = (($29) + 4|0);
   HEAP32[$30>>2] = $25;
   $31 = HEAP32[$pBt>>2]|0;
   $32 = HEAP32[$db>>2]|0;
   $33 = (($32) + 72|0);
   $34 = HEAP32[$33>>2]|0;
   $35 = (_sqlite3BtreeSetPageSize($31,$34,-1,0)|0);
   $36 = (7)==($35|0);
   if ($36) {
    $37 = HEAP32[$db>>2]|0;
    $38 = (($37) + 64|0);
    HEAP8[$38>>0] = 1;
    HEAP32[$0>>2] = 1;
    $39 = HEAP32[$0>>2]|0;
    STACKTOP = sp;return ($39|0);
   }
  }
 }
 HEAP32[$0>>2] = 0;
 $39 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($39|0);
}
function _sqlite3FindTable($db,$zName,$zDatabase) {
 $db = $db|0;
 $zName = $zName|0;
 $zDatabase = $zDatabase|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $i = 0;
 var $j = 0, $p = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 20|0;
 $1 = sp + 16|0;
 $2 = sp + 12|0;
 $p = sp + 8|0;
 $i = sp + 4|0;
 $j = sp;
 HEAP32[$0>>2] = $db;
 HEAP32[$1>>2] = $zName;
 HEAP32[$2>>2] = $zDatabase;
 HEAP32[$p>>2] = 0;
 HEAP32[$i>>2] = 0;
 while(1) {
  $3 = HEAP32[$i>>2]|0;
  $4 = HEAP32[$0>>2]|0;
  $5 = (($4) + 20|0);
  $6 = HEAP32[$5>>2]|0;
  $7 = ($3|0)<($6|0);
  if (!($7)) {
   label = 7;
   break;
  }
  $8 = HEAP32[$i>>2]|0;
  $9 = ($8|0)<(2);
  $10 = HEAP32[$i>>2]|0;
  $11 = $10 ^ 1;
  $12 = $9 ? $11 : $10;
  HEAP32[$j>>2] = $12;
  $13 = HEAP32[$2>>2]|0;
  $14 = ($13|0)!=(0|0);
  if ($14) {
   $15 = HEAP32[$2>>2]|0;
   $16 = HEAP32[$j>>2]|0;
   $17 = HEAP32[$0>>2]|0;
   $18 = (($17) + 16|0);
   $19 = HEAP32[$18>>2]|0;
   $20 = (($19) + ($16<<4)|0);
   $21 = HEAP32[$20>>2]|0;
   $22 = (_sqlite3_stricmp($15,$21)|0);
   $23 = ($22|0)!=(0);
   if (!($23)) {
    label = 5;
   }
  } else {
   label = 5;
  }
  if ((label|0) == 5) {
   label = 0;
   $24 = HEAP32[$j>>2]|0;
   $25 = HEAP32[$0>>2]|0;
   $26 = (($25) + 16|0);
   $27 = HEAP32[$26>>2]|0;
   $28 = (($27) + ($24<<4)|0);
   $29 = (($28) + 12|0);
   $30 = HEAP32[$29>>2]|0;
   $31 = (($30) + 8|0);
   $32 = HEAP32[$1>>2]|0;
   $33 = (_sqlite3HashFind($31,$32)|0);
   HEAP32[$p>>2] = $33;
   $34 = HEAP32[$p>>2]|0;
   $35 = ($34|0)!=(0|0);
   if ($35) {
    label = 7;
    break;
   }
  }
  $36 = HEAP32[$i>>2]|0;
  $37 = (($36) + 1)|0;
  HEAP32[$i>>2] = $37;
 }
 if ((label|0) == 7) {
  $38 = HEAP32[$p>>2]|0;
  STACKTOP = sp;return ($38|0);
 }
 return 0|0;
}
function _sqlite3ValueFromExpr($db,$pExpr,$enc,$affinity,$ppVal) {
 $db = $db|0;
 $pExpr = $pExpr|0;
 $enc = $enc|0;
 $affinity = $affinity|0;
 $ppVal = $ppVal|0;
 var $0 = 0, $1 = 0, $10 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $1 = sp + 4|0;
 $2 = sp + 13|0;
 $3 = sp + 12|0;
 $4 = sp;
 HEAP32[$0>>2] = $db;
 HEAP32[$1>>2] = $pExpr;
 HEAP8[$2>>0] = $enc;
 HEAP8[$3>>0] = $affinity;
 HEAP32[$4>>2] = $ppVal;
 $5 = HEAP32[$0>>2]|0;
 $6 = HEAP32[$1>>2]|0;
 $7 = HEAP8[$2>>0]|0;
 $8 = HEAP8[$3>>0]|0;
 $9 = HEAP32[$4>>2]|0;
 $10 = (_valueFromExpr($5,$6,$7,$8,$9,0)|0);
 STACKTOP = sp;return ($10|0);
}
function _sqlite3MinimumFileFormat($pParse,$iDb,$minFormat) {
 $pParse = $pParse|0;
 $iDb = $iDb|0;
 $minFormat = $minFormat|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $j1 = 0, $r1 = 0, $r2 = 0, $v = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 24|0;
 $1 = sp + 20|0;
 $2 = sp + 16|0;
 $v = sp + 12|0;
 $r1 = sp + 8|0;
 $r2 = sp + 4|0;
 $j1 = sp;
 HEAP32[$0>>2] = $pParse;
 HEAP32[$1>>2] = $iDb;
 HEAP32[$2>>2] = $minFormat;
 $3 = HEAP32[$0>>2]|0;
 $4 = (_sqlite3GetVdbe($3)|0);
 HEAP32[$v>>2] = $4;
 $5 = HEAP32[$v>>2]|0;
 $6 = ($5|0)!=(0|0);
 if (!($6)) {
  STACKTOP = sp;return;
 }
 $7 = HEAP32[$0>>2]|0;
 $8 = (_sqlite3GetTempReg($7)|0);
 HEAP32[$r1>>2] = $8;
 $9 = HEAP32[$0>>2]|0;
 $10 = (_sqlite3GetTempReg($9)|0);
 HEAP32[$r2>>2] = $10;
 $11 = HEAP32[$v>>2]|0;
 $12 = HEAP32[$1>>2]|0;
 $13 = HEAP32[$r1>>2]|0;
 (_sqlite3VdbeAddOp3($11,51,$12,$13,2)|0);
 $14 = HEAP32[$v>>2]|0;
 $15 = HEAP32[$1>>2]|0;
 _sqlite3VdbeUsesBtree($14,$15);
 $16 = HEAP32[$v>>2]|0;
 $17 = HEAP32[$2>>2]|0;
 $18 = HEAP32[$r2>>2]|0;
 (_sqlite3VdbeAddOp2($16,25,$17,$18)|0);
 $19 = HEAP32[$v>>2]|0;
 $20 = HEAP32[$r2>>2]|0;
 $21 = HEAP32[$r1>>2]|0;
 $22 = (_sqlite3VdbeAddOp3($19,83,$20,0,$21)|0);
 HEAP32[$j1>>2] = $22;
 $23 = HEAP32[$v>>2]|0;
 _sqlite3VdbeChangeP5($23,-112);
 $24 = HEAP32[$v>>2]|0;
 $25 = HEAP32[$1>>2]|0;
 $26 = HEAP32[$r2>>2]|0;
 (_sqlite3VdbeAddOp3($24,52,$25,2,$26)|0);
 $27 = HEAP32[$v>>2]|0;
 $28 = HEAP32[$j1>>2]|0;
 _sqlite3VdbeJumpHere($27,$28);
 $29 = HEAP32[$0>>2]|0;
 $30 = HEAP32[$r1>>2]|0;
 _sqlite3ReleaseTempReg($29,$30);
 $31 = HEAP32[$0>>2]|0;
 $32 = HEAP32[$r2>>2]|0;
 _sqlite3ReleaseTempReg($31,$32);
 STACKTOP = sp;return;
}
function _reloadTableSchema($pParse,$pTab,$zName) {
 $pParse = $pParse|0;
 $pTab = $pTab|0;
 $zName = $zName|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $iDb = 0, $iTrigDb = 0, $pTrig = 0, $v = 0, $vararg_buffer = 0, $zWhere = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $vararg_buffer = sp;
 $0 = sp + 32|0;
 $1 = sp + 28|0;
 $2 = sp + 24|0;
 $v = sp + 20|0;
 $zWhere = sp + 16|0;
 $iDb = sp + 12|0;
 $pTrig = sp + 8|0;
 $iTrigDb = sp + 4|0;
 HEAP32[$0>>2] = $pParse;
 HEAP32[$1>>2] = $pTab;
 HEAP32[$2>>2] = $zName;
 $3 = HEAP32[$0>>2]|0;
 $4 = (_sqlite3GetVdbe($3)|0);
 HEAP32[$v>>2] = $4;
 $5 = HEAP32[$v>>2]|0;
 $6 = ($5|0)==(0|0);
 if ($6) {
  STACKTOP = sp;return;
 }
 $7 = HEAP32[$0>>2]|0;
 $8 = HEAP32[$7>>2]|0;
 $9 = HEAP32[$1>>2]|0;
 $10 = (($9) + 68|0);
 $11 = HEAP32[$10>>2]|0;
 $12 = (_sqlite3SchemaToIndex($8,$11)|0);
 HEAP32[$iDb>>2] = $12;
 $13 = HEAP32[$0>>2]|0;
 $14 = HEAP32[$1>>2]|0;
 $15 = (_sqlite3TriggerList($13,$14)|0);
 HEAP32[$pTrig>>2] = $15;
 while(1) {
  $16 = HEAP32[$pTrig>>2]|0;
  $17 = ($16|0)!=(0|0);
  if (!($17)) {
   break;
  }
  $18 = HEAP32[$0>>2]|0;
  $19 = HEAP32[$18>>2]|0;
  $20 = HEAP32[$pTrig>>2]|0;
  $21 = (($20) + 20|0);
  $22 = HEAP32[$21>>2]|0;
  $23 = (_sqlite3SchemaToIndex($19,$22)|0);
  HEAP32[$iTrigDb>>2] = $23;
  $24 = HEAP32[$v>>2]|0;
  $25 = HEAP32[$iTrigDb>>2]|0;
  $26 = HEAP32[$pTrig>>2]|0;
  $27 = HEAP32[$26>>2]|0;
  (_sqlite3VdbeAddOp4($24,126,$25,0,0,$27,0)|0);
  $28 = HEAP32[$pTrig>>2]|0;
  $29 = (($28) + 32|0);
  $30 = HEAP32[$29>>2]|0;
  HEAP32[$pTrig>>2] = $30;
 }
 $31 = HEAP32[$v>>2]|0;
 $32 = HEAP32[$iDb>>2]|0;
 $33 = HEAP32[$1>>2]|0;
 $34 = HEAP32[$33>>2]|0;
 (_sqlite3VdbeAddOp4($31,124,$32,0,0,$34,0)|0);
 $35 = HEAP32[$0>>2]|0;
 $36 = HEAP32[$35>>2]|0;
 $37 = HEAP32[$2>>2]|0;
 HEAP32[$vararg_buffer>>2] = $37;
 $38 = (_sqlite3MPrintf($36,15528,$vararg_buffer)|0);
 HEAP32[$zWhere>>2] = $38;
 $39 = HEAP32[$zWhere>>2]|0;
 $40 = ($39|0)!=(0|0);
 if (!($40)) {
  STACKTOP = sp;return;
 }
 $41 = HEAP32[$v>>2]|0;
 $42 = HEAP32[$iDb>>2]|0;
 $43 = HEAP32[$zWhere>>2]|0;
 _sqlite3VdbeAddParseSchemaOp($41,$42,$43);
 $44 = HEAP32[$0>>2]|0;
 $45 = HEAP32[$1>>2]|0;
 $46 = (_whereTempTriggers($44,$45)|0);
 HEAP32[$zWhere>>2] = $46;
 $47 = ($46|0)!=(0|0);
 if (!($47)) {
  STACKTOP = sp;return;
 }
 $48 = HEAP32[$v>>2]|0;
 $49 = HEAP32[$zWhere>>2]|0;
 _sqlite3VdbeAddParseSchemaOp($48,1,$49);
 STACKTOP = sp;return;
}
function _sqlite3TriggerList($pParse,$pTab) {
 $pParse = $pParse|0;
 $pTab = $pTab|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $7 = 0;
 var $8 = 0, $9 = 0, $p = 0, $pList = 0, $pTmpSchema = 0, $pTrig = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 24|0;
 $1 = sp + 20|0;
 $2 = sp + 16|0;
 $pTmpSchema = sp + 12|0;
 $pList = sp + 8|0;
 $p = sp + 4|0;
 $pTrig = sp;
 HEAP32[$1>>2] = $pParse;
 HEAP32[$2>>2] = $pTab;
 $3 = HEAP32[$1>>2]|0;
 $4 = HEAP32[$3>>2]|0;
 $5 = (($4) + 16|0);
 $6 = HEAP32[$5>>2]|0;
 $7 = (($6) + 16|0);
 $8 = (($7) + 12|0);
 $9 = HEAP32[$8>>2]|0;
 HEAP32[$pTmpSchema>>2] = $9;
 HEAP32[$pList>>2] = 0;
 $10 = HEAP32[$1>>2]|0;
 $11 = (($10) + 438|0);
 $12 = HEAP8[$11>>0]|0;
 $13 = ($12<<24>>24)!=(0);
 if ($13) {
  HEAP32[$0>>2] = 0;
  $61 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($61|0);
 }
 $14 = HEAP32[$pTmpSchema>>2]|0;
 $15 = HEAP32[$2>>2]|0;
 $16 = (($15) + 68|0);
 $17 = HEAP32[$16>>2]|0;
 $18 = ($14|0)!=($17|0);
 L5: do {
  if ($18) {
   $19 = HEAP32[$pTmpSchema>>2]|0;
   $20 = (($19) + 40|0);
   $21 = (($20) + 8|0);
   $22 = HEAP32[$21>>2]|0;
   HEAP32[$p>>2] = $22;
   while(1) {
    $23 = HEAP32[$p>>2]|0;
    $24 = ($23|0)!=(0|0);
    if (!($24)) {
     break L5;
    }
    $25 = HEAP32[$p>>2]|0;
    $26 = (($25) + 8|0);
    $27 = HEAP32[$26>>2]|0;
    HEAP32[$pTrig>>2] = $27;
    $28 = HEAP32[$pTrig>>2]|0;
    $29 = (($28) + 24|0);
    $30 = HEAP32[$29>>2]|0;
    $31 = HEAP32[$2>>2]|0;
    $32 = (($31) + 68|0);
    $33 = HEAP32[$32>>2]|0;
    $34 = ($30|0)==($33|0);
    if ($34) {
     $35 = HEAP32[$pTrig>>2]|0;
     $36 = (($35) + 4|0);
     $37 = HEAP32[$36>>2]|0;
     $38 = HEAP32[$2>>2]|0;
     $39 = HEAP32[$38>>2]|0;
     $40 = (_sqlite3_stricmp($37,$39)|0);
     $41 = (0)==($40|0);
     if ($41) {
      $42 = HEAP32[$pList>>2]|0;
      $43 = ($42|0)!=(0|0);
      if ($43) {
       $44 = HEAP32[$pList>>2]|0;
       $50 = $44;
      } else {
       $45 = HEAP32[$2>>2]|0;
       $46 = (($45) + 64|0);
       $47 = HEAP32[$46>>2]|0;
       $50 = $47;
      }
      $48 = HEAP32[$pTrig>>2]|0;
      $49 = (($48) + 32|0);
      HEAP32[$49>>2] = $50;
      $51 = HEAP32[$pTrig>>2]|0;
      HEAP32[$pList>>2] = $51;
     }
    }
    $52 = HEAP32[$p>>2]|0;
    $53 = HEAP32[$52>>2]|0;
    HEAP32[$p>>2] = $53;
   }
  }
 } while(0);
 $54 = HEAP32[$pList>>2]|0;
 $55 = ($54|0)!=(0|0);
 if ($55) {
  $56 = HEAP32[$pList>>2]|0;
  $60 = $56;
 } else {
  $57 = HEAP32[$2>>2]|0;
  $58 = (($57) + 64|0);
  $59 = HEAP32[$58>>2]|0;
  $60 = $59;
 }
 HEAP32[$0>>2] = $60;
 $61 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($61|0);
}
function _whereTempTriggers($pParse,$pTab) {
 $pParse = $pParse|0;
 $pTab = $pTab|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $db = 0, $pTempSchema = 0, $pTrig = 0, $vararg_buffer = 0, $zNew = 0, $zWhere = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $vararg_buffer = sp;
 $0 = sp + 28|0;
 $1 = sp + 24|0;
 $pTrig = sp + 20|0;
 $zWhere = sp + 16|0;
 $pTempSchema = sp + 12|0;
 $db = sp + 8|0;
 $zNew = sp + 4|0;
 HEAP32[$0>>2] = $pParse;
 HEAP32[$1>>2] = $pTab;
 HEAP32[$zWhere>>2] = 0;
 $2 = HEAP32[$0>>2]|0;
 $3 = HEAP32[$2>>2]|0;
 $4 = (($3) + 16|0);
 $5 = HEAP32[$4>>2]|0;
 $6 = (($5) + 16|0);
 $7 = (($6) + 12|0);
 $8 = HEAP32[$7>>2]|0;
 HEAP32[$pTempSchema>>2] = $8;
 $9 = HEAP32[$1>>2]|0;
 $10 = (($9) + 68|0);
 $11 = HEAP32[$10>>2]|0;
 $12 = HEAP32[$pTempSchema>>2]|0;
 $13 = ($11|0)!=($12|0);
 L1: do {
  if ($13) {
   $14 = HEAP32[$0>>2]|0;
   $15 = HEAP32[$14>>2]|0;
   HEAP32[$db>>2] = $15;
   $16 = HEAP32[$0>>2]|0;
   $17 = HEAP32[$1>>2]|0;
   $18 = (_sqlite3TriggerList($16,$17)|0);
   HEAP32[$pTrig>>2] = $18;
   while(1) {
    $19 = HEAP32[$pTrig>>2]|0;
    $20 = ($19|0)!=(0|0);
    if (!($20)) {
     break L1;
    }
    $21 = HEAP32[$pTrig>>2]|0;
    $22 = (($21) + 20|0);
    $23 = HEAP32[$22>>2]|0;
    $24 = HEAP32[$pTempSchema>>2]|0;
    $25 = ($23|0)==($24|0);
    if ($25) {
     $26 = HEAP32[$db>>2]|0;
     $27 = HEAP32[$zWhere>>2]|0;
     $28 = HEAP32[$pTrig>>2]|0;
     $29 = HEAP32[$28>>2]|0;
     $30 = (_whereOrName($26,$27,$29)|0);
     HEAP32[$zWhere>>2] = $30;
    }
    $31 = HEAP32[$pTrig>>2]|0;
    $32 = (($31) + 32|0);
    $33 = HEAP32[$32>>2]|0;
    HEAP32[$pTrig>>2] = $33;
   }
  }
 } while(0);
 $34 = HEAP32[$zWhere>>2]|0;
 $35 = ($34|0)!=(0|0);
 if (!($35)) {
  $44 = HEAP32[$zWhere>>2]|0;
  STACKTOP = sp;return ($44|0);
 }
 $36 = HEAP32[$0>>2]|0;
 $37 = HEAP32[$36>>2]|0;
 $38 = HEAP32[$zWhere>>2]|0;
 HEAP32[$vararg_buffer>>2] = $38;
 $39 = (_sqlite3MPrintf($37,15544,$vararg_buffer)|0);
 HEAP32[$zNew>>2] = $39;
 $40 = HEAP32[$0>>2]|0;
 $41 = HEAP32[$40>>2]|0;
 $42 = HEAP32[$zWhere>>2]|0;
 _sqlite3DbFree($41,$42);
 $43 = HEAP32[$zNew>>2]|0;
 HEAP32[$zWhere>>2] = $43;
 $44 = HEAP32[$zWhere>>2]|0;
 STACKTOP = sp;return ($44|0);
}
function _whereOrName($db,$zWhere,$zConstant) {
 $db = $db|0;
 $zWhere = $zWhere|0;
 $zConstant = $zConstant|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $vararg_buffer1 = 0, $vararg_ptr4 = 0, $zNew = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $vararg_buffer1 = sp;
 $0 = sp + 20|0;
 $1 = sp + 16|0;
 $2 = sp + 12|0;
 $zNew = sp + 8|0;
 HEAP32[$0>>2] = $db;
 HEAP32[$1>>2] = $zWhere;
 HEAP32[$2>>2] = $zConstant;
 $3 = HEAP32[$1>>2]|0;
 $4 = ($3|0)!=(0|0);
 $5 = HEAP32[$0>>2]|0;
 if ($4) {
  $8 = HEAP32[$1>>2]|0;
  $9 = HEAP32[$2>>2]|0;
  HEAP32[$vararg_buffer1>>2] = $8;
  $vararg_ptr4 = (($vararg_buffer1) + 4|0);
  HEAP32[$vararg_ptr4>>2] = $9;
  $10 = (_sqlite3MPrintf($5,15576,$vararg_buffer1)|0);
  HEAP32[$zNew>>2] = $10;
  $11 = HEAP32[$0>>2]|0;
  $12 = HEAP32[$1>>2]|0;
  _sqlite3DbFree($11,$12);
  $13 = HEAP32[$zNew>>2]|0;
  STACKTOP = sp;return ($13|0);
 } else {
  $6 = HEAP32[$2>>2]|0;
  HEAP32[$vararg_buffer1>>2] = $6;
  $7 = (_sqlite3MPrintf($5,15568,$vararg_buffer1)|0);
  HEAP32[$zNew>>2] = $7;
  $13 = HEAP32[$zNew>>2]|0;
  STACKTOP = sp;return ($13|0);
 }
 return 0|0;
}
function _sqlite3VdbeJumpHere($p,$addr) {
 $p = $p|0;
 $addr = $addr|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $1 = sp;
 HEAP32[$0>>2] = $p;
 HEAP32[$1>>2] = $addr;
 $2 = HEAP32[$0>>2]|0;
 $3 = HEAP32[$1>>2]|0;
 $4 = HEAP32[$0>>2]|0;
 $5 = (($4) + 32|0);
 $6 = HEAP32[$5>>2]|0;
 _sqlite3VdbeChangeP2($2,$3,$6);
 $7 = HEAP32[$0>>2]|0;
 $8 = (($7) + 32|0);
 $9 = HEAP32[$8>>2]|0;
 $10 = (($9) - 1)|0;
 $11 = HEAP32[$0>>2]|0;
 $12 = (($11) + 24|0);
 $13 = HEAP32[$12>>2]|0;
 $14 = (($13) + 88|0);
 HEAP32[$14>>2] = $10;
 STACKTOP = sp;return;
}
function _valueFromExpr($db,$pExpr,$enc,$affinity,$ppVal,$pCtx) {
 $db = $db|0;
 $pExpr = $pExpr|0;
 $enc = $enc|0;
 $affinity = $affinity|0;
 $ppVal = $ppVal|0;
 $pCtx = $pCtx|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0.0, $155 = 0.0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0;
 var $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0, $188 = 0;
 var $189 = 0, $19 = 0, $190 = 0, $191 = 0, $192 = 0, $193 = 0, $194 = 0, $195 = 0, $196 = 0, $197 = 0, $198 = 0, $199 = 0, $2 = 0, $20 = 0, $200 = 0, $201 = 0, $202 = 0, $203 = 0, $204 = 0, $205 = 0;
 var $206 = 0, $207 = 0, $208 = 0, $209 = 0, $21 = 0, $210 = 0, $211 = 0, $212 = 0, $213 = 0, $214 = 0, $215 = 0, $216 = 0, $217 = 0, $218 = 0, $219 = 0, $22 = 0, $220 = 0, $221 = 0, $222 = 0, $223 = 0;
 var $224 = 0, $225 = 0, $226 = 0, $227 = 0, $228 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0;
 var $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0;
 var $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0;
 var $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0;
 var $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $aff = 0, $nVal = 0, $negInt = 0, $op = 0, $or$cond = 0, $or$cond3 = 0, $or$cond5 = 0, $or$cond7 = 0, $pVal = 0, $rc = 0, $vararg_buffer = 0;
 var $vararg_ptr8 = 0, $zNeg = 0, $zVal = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 64|0;
 $vararg_buffer = sp;
 $0 = sp + 52|0;
 $1 = sp + 48|0;
 $2 = sp + 44|0;
 $3 = sp + 58|0;
 $4 = sp + 57|0;
 $5 = sp + 40|0;
 $6 = sp + 36|0;
 $op = sp + 32|0;
 $zVal = sp + 28|0;
 $pVal = sp + 24|0;
 $negInt = sp + 20|0;
 $zNeg = sp + 16|0;
 $rc = sp + 12|0;
 $aff = sp + 56|0;
 $nVal = sp + 8|0;
 HEAP32[$1>>2] = $db;
 HEAP32[$2>>2] = $pExpr;
 HEAP8[$3>>0] = $enc;
 HEAP8[$4>>0] = $affinity;
 HEAP32[$5>>2] = $ppVal;
 HEAP32[$6>>2] = $pCtx;
 HEAP32[$zVal>>2] = 0;
 HEAP32[$pVal>>2] = 0;
 HEAP32[$negInt>>2] = 1;
 HEAP32[$zNeg>>2] = 1216;
 HEAP32[$rc>>2] = 0;
 $7 = HEAP32[$2>>2]|0;
 $8 = ($7|0)!=(0|0);
 if (!($8)) {
  $9 = HEAP32[$5>>2]|0;
  HEAP32[$9>>2] = 0;
  HEAP32[$0>>2] = 0;
  $228 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($228|0);
 }
 while(1) {
  $10 = HEAP32[$2>>2]|0;
  $11 = HEAP8[$10>>0]|0;
  $12 = $11&255;
  HEAP32[$op>>2] = $12;
  $13 = ($12|0)==(158);
  if (!($13)) {
   break;
  }
  $14 = HEAP32[$2>>2]|0;
  $15 = (($14) + 12|0);
  $16 = HEAP32[$15>>2]|0;
  HEAP32[$2>>2] = $16;
 }
 $17 = HEAP32[$op>>2]|0;
 $18 = ($17|0)==(159);
 if ($18) {
  $19 = HEAP32[$2>>2]|0;
  $20 = (($19) + 38|0);
  $21 = HEAP8[$20>>0]|0;
  $22 = $21&255;
  HEAP32[$op>>2] = $22;
 }
 $23 = HEAP32[$op>>2]|0;
 $24 = ($23|0)==(38);
 if ($24) {
  $25 = HEAP32[$2>>2]|0;
  $26 = (($25) + 8|0);
  $27 = HEAP32[$26>>2]|0;
  $28 = (_sqlite3AffinityType($27,0)|0);
  HEAP8[$aff>>0] = $28;
  $29 = HEAP32[$1>>2]|0;
  $30 = HEAP32[$2>>2]|0;
  $31 = (($30) + 12|0);
  $32 = HEAP32[$31>>2]|0;
  $33 = HEAP8[$3>>0]|0;
  $34 = HEAP8[$aff>>0]|0;
  $35 = HEAP32[$5>>2]|0;
  $36 = HEAP32[$6>>2]|0;
  $37 = (_valueFromExpr($29,$32,$33,$34,$35,$36)|0);
  HEAP32[$rc>>2] = $37;
  $38 = HEAP32[$5>>2]|0;
  $39 = HEAP32[$38>>2]|0;
  $40 = ($39|0)!=(0|0);
  if ($40) {
   $41 = HEAP32[$5>>2]|0;
   $42 = HEAP32[$41>>2]|0;
   $43 = HEAP8[$aff>>0]|0;
   _sqlite3VdbeMemCast($42,$43,1);
   $44 = HEAP32[$5>>2]|0;
   $45 = HEAP32[$44>>2]|0;
   $46 = HEAP8[$4>>0]|0;
   _sqlite3ValueApplyAffinity($45,$46,1);
  }
  $47 = HEAP32[$rc>>2]|0;
  HEAP32[$0>>2] = $47;
  $228 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($228|0);
 }
 $48 = HEAP32[$op>>2]|0;
 $49 = ($48|0)==(157);
 do {
  if ($49) {
   $50 = HEAP32[$2>>2]|0;
   $51 = (($50) + 12|0);
   $52 = HEAP32[$51>>2]|0;
   $53 = HEAP8[$52>>0]|0;
   $54 = $53&255;
   $55 = ($54|0)==(132);
   if (!($55)) {
    $56 = HEAP32[$2>>2]|0;
    $57 = (($56) + 12|0);
    $58 = HEAP32[$57>>2]|0;
    $59 = HEAP8[$58>>0]|0;
    $60 = $59&255;
    $61 = ($60|0)==(133);
    if (!($61)) {
     break;
    }
   }
   $62 = HEAP32[$2>>2]|0;
   $63 = (($62) + 12|0);
   $64 = HEAP32[$63>>2]|0;
   HEAP32[$2>>2] = $64;
   $65 = HEAP32[$2>>2]|0;
   $66 = HEAP8[$65>>0]|0;
   $67 = $66&255;
   HEAP32[$op>>2] = $67;
   HEAP32[$negInt>>2] = -1;
   HEAP32[$zNeg>>2] = 15592;
  }
 } while(0);
 $68 = HEAP32[$op>>2]|0;
 $69 = ($68|0)==(97);
 $70 = HEAP32[$op>>2]|0;
 $71 = ($70|0)==(133);
 $or$cond = $69 | $71;
 $72 = HEAP32[$op>>2]|0;
 $73 = ($72|0)==(132);
 $or$cond3 = $or$cond | $73;
 do {
  if ($or$cond3) {
   $74 = HEAP32[$1>>2]|0;
   $75 = HEAP32[$6>>2]|0;
   $76 = (_valueNew($74,$75)|0);
   HEAP32[$pVal>>2] = $76;
   $77 = HEAP32[$pVal>>2]|0;
   $78 = ($77|0)==(0|0);
   if ($78) {
    label = 43;
   } else {
    $79 = HEAP32[$2>>2]|0;
    $80 = (($79) + 4|0);
    $81 = HEAP32[$80>>2]|0;
    $82 = $81 & 1024;
    $83 = ($82|0)!=(0);
    if ($83) {
     $84 = HEAP32[$pVal>>2]|0;
     $85 = HEAP32[$2>>2]|0;
     $86 = (($85) + 8|0);
     $87 = HEAP32[$86>>2]|0;
     $88 = ($87|0)<(0);
     $89 = $88 << 31 >> 31;
     $90 = HEAP32[$negInt>>2]|0;
     $91 = ($90|0)<(0);
     $92 = $91 << 31 >> 31;
     $93 = (___muldi3(($87|0),($89|0),($90|0),($92|0))|0);
     $94 = tempRet0;
     _sqlite3VdbeMemSetInt64($84,$93,$94);
    } else {
     $95 = HEAP32[$1>>2]|0;
     $96 = HEAP32[$zNeg>>2]|0;
     $97 = HEAP32[$2>>2]|0;
     $98 = (($97) + 8|0);
     $99 = HEAP32[$98>>2]|0;
     HEAP32[$vararg_buffer>>2] = $96;
     $vararg_ptr8 = (($vararg_buffer) + 4|0);
     HEAP32[$vararg_ptr8>>2] = $99;
     $100 = (_sqlite3MPrintf($95,8992,$vararg_buffer)|0);
     HEAP32[$zVal>>2] = $100;
     $101 = HEAP32[$zVal>>2]|0;
     $102 = ($101|0)==(0|0);
     if ($102) {
      label = 43;
      break;
     }
     $103 = HEAP32[$pVal>>2]|0;
     $104 = HEAP32[$zVal>>2]|0;
     _sqlite3ValueSetStr($103,-1,$104,1,16);
    }
    $105 = HEAP32[$op>>2]|0;
    $106 = ($105|0)==(132);
    $107 = HEAP32[$op>>2]|0;
    $108 = ($107|0)==(133);
    $or$cond5 = $106 | $108;
    if ($or$cond5) {
     $109 = HEAP8[$4>>0]|0;
     $110 = $109&255;
     $111 = ($110|0)==(65);
     if ($111) {
      $112 = HEAP32[$pVal>>2]|0;
      _sqlite3ValueApplyAffinity($112,67,1);
     } else {
      label = 24;
     }
    } else {
     label = 24;
    }
    if ((label|0) == 24) {
     $113 = HEAP32[$pVal>>2]|0;
     $114 = HEAP8[$4>>0]|0;
     _sqlite3ValueApplyAffinity($113,$114,1);
    }
    $115 = HEAP32[$pVal>>2]|0;
    $116 = (($115) + 8|0);
    $117 = HEAP16[$116>>1]|0;
    $118 = $117&65535;
    $119 = $118 & 12;
    $120 = ($119|0)!=(0);
    if ($120) {
     $121 = HEAP32[$pVal>>2]|0;
     $122 = (($121) + 8|0);
     $123 = HEAP16[$122>>1]|0;
     $124 = $123&65535;
     $125 = $124 & -3;
     $126 = $125&65535;
     HEAP16[$122>>1] = $126;
    }
    $127 = HEAP8[$3>>0]|0;
    $128 = $127&255;
    $129 = ($128|0)!=(1);
    if ($129) {
     $130 = HEAP32[$pVal>>2]|0;
     $131 = HEAP8[$3>>0]|0;
     $132 = $131&255;
     $133 = (_sqlite3VdbeChangeEncoding($130,$132)|0);
     HEAP32[$rc>>2] = $133;
     label = 42;
    } else {
     label = 42;
    }
   }
  } else {
   $134 = HEAP32[$op>>2]|0;
   $135 = ($134|0)==(157);
   if (!($135)) {
    $192 = HEAP32[$op>>2]|0;
    $193 = ($192|0)==(101);
    if ($193) {
     $194 = HEAP32[$1>>2]|0;
     $195 = HEAP32[$6>>2]|0;
     $196 = (_valueNew($194,$195)|0);
     HEAP32[$pVal>>2] = $196;
     $197 = HEAP32[$pVal>>2]|0;
     $198 = ($197|0)==(0|0);
     if ($198) {
      label = 43;
      break;
     } else {
      label = 42;
      break;
     }
    }
    $199 = HEAP32[$op>>2]|0;
    $200 = ($199|0)==(134);
    if (!($200)) {
     label = 42;
     break;
    }
    $201 = HEAP32[$1>>2]|0;
    $202 = HEAP32[$6>>2]|0;
    $203 = (_valueNew($201,$202)|0);
    HEAP32[$pVal>>2] = $203;
    $204 = HEAP32[$pVal>>2]|0;
    $205 = ($204|0)!=(0|0);
    if (!($205)) {
     label = 43;
     break;
    }
    $206 = HEAP32[$2>>2]|0;
    $207 = (($206) + 8|0);
    $208 = HEAP32[$207>>2]|0;
    $209 = (($208) + 2|0);
    HEAP32[$zVal>>2] = $209;
    $210 = HEAP32[$zVal>>2]|0;
    $211 = (_sqlite3Strlen30($210)|0);
    $212 = (($211) - 1)|0;
    HEAP32[$nVal>>2] = $212;
    $213 = HEAP32[$pVal>>2]|0;
    $214 = HEAP32[$1>>2]|0;
    $215 = HEAP32[$zVal>>2]|0;
    $216 = HEAP32[$nVal>>2]|0;
    $217 = (_sqlite3HexToBlob($214,$215,$216)|0);
    $218 = HEAP32[$nVal>>2]|0;
    $219 = (($218|0) / 2)&-1;
    (_sqlite3VdbeMemSetStr($213,$217,$219,0,16)|0);
    label = 42;
    break;
   }
   $136 = HEAP32[$1>>2]|0;
   $137 = HEAP32[$2>>2]|0;
   $138 = (($137) + 12|0);
   $139 = HEAP32[$138>>2]|0;
   $140 = HEAP8[$3>>0]|0;
   $141 = HEAP8[$4>>0]|0;
   $142 = (_sqlite3ValueFromExpr($136,$139,$140,$141,$pVal)|0);
   $143 = (0)==($142|0);
   $144 = HEAP32[$pVal>>2]|0;
   $145 = ($144|0)!=(0|0);
   $or$cond7 = $143 & $145;
   if ($or$cond7) {
    $146 = HEAP32[$pVal>>2]|0;
    (_sqlite3VdbeMemNumerify($146)|0);
    $147 = HEAP32[$pVal>>2]|0;
    $148 = (($147) + 8|0);
    $149 = HEAP16[$148>>1]|0;
    $150 = $149&65535;
    $151 = $150 & 8;
    $152 = ($151|0)!=(0);
    $153 = HEAP32[$pVal>>2]|0;
    do {
     if ($152) {
      $154 = +HEAPF64[$153>>3];
      $155 = -$154;
      $156 = HEAP32[$pVal>>2]|0;
      HEAPF64[$156>>3] = $155;
     } else {
      $157 = $153;
      $158 = $157;
      $159 = HEAP32[$158>>2]|0;
      $160 = (($157) + 4)|0;
      $161 = $160;
      $162 = HEAP32[$161>>2]|0;
      $163 = ($159|0)==(0);
      $164 = ($162|0)==(-2147483648);
      $165 = $163 & $164;
      $166 = HEAP32[$pVal>>2]|0;
      if ($165) {
       HEAPF64[$166>>3] = 9.2233720368547758E+18;
       $167 = HEAP32[$pVal>>2]|0;
       $168 = (($167) + 8|0);
       $169 = HEAP16[$168>>1]|0;
       $170 = $169&65535;
       $171 = $170 & -16896;
       $172 = $171 | 8;
       $173 = $172&65535;
       $174 = HEAP32[$pVal>>2]|0;
       $175 = (($174) + 8|0);
       HEAP16[$175>>1] = $173;
       break;
      } else {
       $176 = $166;
       $177 = $176;
       $178 = HEAP32[$177>>2]|0;
       $179 = (($176) + 4)|0;
       $180 = $179;
       $181 = HEAP32[$180>>2]|0;
       $182 = (_i64Subtract(0,0,($178|0),($181|0))|0);
       $183 = tempRet0;
       $184 = HEAP32[$pVal>>2]|0;
       $185 = $184;
       $186 = $185;
       HEAP32[$186>>2] = $182;
       $187 = (($185) + 4)|0;
       $188 = $187;
       HEAP32[$188>>2] = $183;
       break;
      }
     }
    } while(0);
    $189 = HEAP32[$pVal>>2]|0;
    $190 = HEAP8[$4>>0]|0;
    $191 = HEAP8[$3>>0]|0;
    _sqlite3ValueApplyAffinity($189,$190,$191);
    label = 42;
   } else {
    label = 42;
   }
  }
 } while(0);
 if ((label|0) == 42) {
  $220 = HEAP32[$pVal>>2]|0;
  $221 = HEAP32[$5>>2]|0;
  HEAP32[$221>>2] = $220;
  $222 = HEAP32[$rc>>2]|0;
  HEAP32[$0>>2] = $222;
  $228 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($228|0);
 }
 else if ((label|0) == 43) {
  $223 = HEAP32[$1>>2]|0;
  $224 = (($223) + 64|0);
  HEAP8[$224>>0] = 1;
  $225 = HEAP32[$1>>2]|0;
  $226 = HEAP32[$zVal>>2]|0;
  _sqlite3DbFree($225,$226);
  $227 = HEAP32[$pVal>>2]|0;
  _sqlite3ValueFree($227);
  HEAP32[$0>>2] = 7;
  $228 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($228|0);
 }
 return 0|0;
}
function _sqlite3AffinityType($zIn,$pszEst) {
 $zIn = $zIn|0;
 $pszEst = $pszEst|0;
 var $$ = 0, $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0;
 var $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0;
 var $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0;
 var $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0;
 var $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $aff = 0, $h = 0, $v = 0, $zChar = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 21|0;
 $1 = sp + 16|0;
 $2 = sp + 12|0;
 $h = sp + 8|0;
 $aff = sp + 20|0;
 $zChar = sp + 4|0;
 $v = sp;
 HEAP32[$1>>2] = $zIn;
 HEAP32[$2>>2] = $pszEst;
 HEAP32[$h>>2] = 0;
 HEAP8[$aff>>0] = 67;
 HEAP32[$zChar>>2] = 0;
 $3 = HEAP32[$1>>2]|0;
 $4 = ($3|0)==(0|0);
 if ($4) {
  $5 = HEAP8[$aff>>0]|0;
  HEAP8[$0>>0] = $5;
  $91 = HEAP8[$0>>0]|0;
  STACKTOP = sp;return ($91|0);
 }
 L4: while(1) {
  $6 = HEAP32[$1>>2]|0;
  $7 = HEAP8[$6>>0]|0;
  $8 = ($7<<24>>24)!=(0);
  if (!($8)) {
   break;
  }
  $9 = HEAP32[$h>>2]|0;
  $10 = $9 << 8;
  $11 = HEAP32[$1>>2]|0;
  $12 = HEAP8[$11>>0]|0;
  $13 = $12 << 24 >> 24;
  $14 = $13 & 255;
  $15 = (440 + ($14)|0);
  $16 = HEAP8[$15>>0]|0;
  $17 = $16&255;
  $18 = (($10) + ($17))|0;
  HEAP32[$h>>2] = $18;
  $19 = HEAP32[$1>>2]|0;
  $20 = (($19) + 1|0);
  HEAP32[$1>>2] = $20;
  $21 = HEAP32[$h>>2]|0;
  $22 = ($21|0)==(1667785074);
  if ($22) {
   HEAP8[$aff>>0] = 66;
   $23 = HEAP32[$1>>2]|0;
   HEAP32[$zChar>>2] = $23;
   continue;
  }
  $24 = HEAP32[$h>>2]|0;
  $25 = ($24|0)==(1668050786);
  if ($25) {
   HEAP8[$aff>>0] = 66;
   continue;
  }
  $26 = HEAP32[$h>>2]|0;
  $27 = ($26|0)==(1952807028);
  if ($27) {
   HEAP8[$aff>>0] = 66;
   continue;
  }
  $28 = HEAP32[$h>>2]|0;
  $29 = ($28|0)==(1651273570);
  do {
   if ($29) {
    $30 = HEAP8[$aff>>0]|0;
    $31 = $30 << 24 >> 24;
    $32 = ($31|0)==(67);
    if (!($32)) {
     $33 = HEAP8[$aff>>0]|0;
     $34 = $33 << 24 >> 24;
     $35 = ($34|0)==(69);
     if (!($35)) {
      break;
     }
    }
    HEAP8[$aff>>0] = 65;
    $36 = HEAP32[$1>>2]|0;
    $37 = HEAP8[$36>>0]|0;
    $38 = $37 << 24 >> 24;
    $39 = ($38|0)==(40);
    if (!($39)) {
     continue L4;
    }
    $40 = HEAP32[$1>>2]|0;
    HEAP32[$zChar>>2] = $40;
    continue L4;
   }
  } while(0);
  $41 = HEAP32[$h>>2]|0;
  $42 = ($41|0)==(1919246700);
  if ($42) {
   $43 = HEAP8[$aff>>0]|0;
   $44 = $43 << 24 >> 24;
   $45 = ($44|0)==(67);
   if ($45) {
    HEAP8[$aff>>0] = 69;
    continue;
   }
  }
  $46 = HEAP32[$h>>2]|0;
  $47 = ($46|0)==(1718382433);
  if ($47) {
   $48 = HEAP8[$aff>>0]|0;
   $49 = $48 << 24 >> 24;
   $50 = ($49|0)==(67);
   if ($50) {
    HEAP8[$aff>>0] = 69;
    continue;
   }
  }
  $51 = HEAP32[$h>>2]|0;
  $52 = ($51|0)==(1685026146);
  if ($52) {
   $53 = HEAP8[$aff>>0]|0;
   $54 = $53 << 24 >> 24;
   $55 = ($54|0)==(67);
   if ($55) {
    HEAP8[$aff>>0] = 69;
    continue;
   }
  }
  $56 = HEAP32[$h>>2]|0;
  $57 = $56 & 16777215;
  $58 = ($57|0)==(6909556);
  if ($58) {
   label = 25;
   break;
  }
 }
 if ((label|0) == 25) {
  HEAP8[$aff>>0] = 68;
 }
 $59 = HEAP32[$2>>2]|0;
 $60 = ($59|0)!=(0|0);
 L38: do {
  if ($60) {
   $61 = HEAP32[$2>>2]|0;
   HEAP8[$61>>0] = 1;
   $62 = HEAP8[$aff>>0]|0;
   $63 = $62 << 24 >> 24;
   $64 = ($63|0)<(67);
   if ($64) {
    $65 = HEAP32[$zChar>>2]|0;
    $66 = ($65|0)!=(0|0);
    if (!($66)) {
     $89 = HEAP32[$2>>2]|0;
     HEAP8[$89>>0] = 5;
     break;
    }
    while(1) {
     $67 = HEAP32[$zChar>>2]|0;
     $68 = HEAP8[$67>>0]|0;
     $69 = ($68<<24>>24)!=(0);
     if (!($69)) {
      break L38;
     }
     $70 = HEAP32[$zChar>>2]|0;
     $71 = HEAP8[$70>>0]|0;
     $72 = $71&255;
     $73 = (1224 + ($72)|0);
     $74 = HEAP8[$73>>0]|0;
     $75 = $74&255;
     $76 = $75 & 4;
     $77 = ($76|0)!=(0);
     if ($77) {
      break;
     }
     $87 = HEAP32[$zChar>>2]|0;
     $88 = (($87) + 1|0);
     HEAP32[$zChar>>2] = $88;
    }
    HEAP32[$v>>2] = 0;
    $78 = HEAP32[$zChar>>2]|0;
    (_sqlite3GetInt32($78,$v)|0);
    $79 = HEAP32[$v>>2]|0;
    $80 = (($79|0) / 4)&-1;
    $81 = (($80) + 1)|0;
    HEAP32[$v>>2] = $81;
    $82 = HEAP32[$v>>2]|0;
    $83 = ($82|0)>(255);
    $$ = $83 ? 255 : $81;
    HEAP32[$v>>2] = $$;
    $84 = HEAP32[$v>>2]|0;
    $85 = $84&255;
    $86 = HEAP32[$2>>2]|0;
    HEAP8[$86>>0] = $85;
   }
  }
 } while(0);
 $90 = HEAP8[$aff>>0]|0;
 HEAP8[$0>>0] = $90;
 $91 = HEAP8[$0>>0]|0;
 STACKTOP = sp;return ($91|0);
}
function _sqlite3VdbeMemCast($pMem,$aff,$encoding) {
 $pMem = $pMem|0;
 $aff = $aff|0;
 $encoding = $encoding|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 $1 = sp + 5|0;
 $2 = sp + 4|0;
 HEAP32[$0>>2] = $pMem;
 HEAP8[$1>>0] = $aff;
 HEAP8[$2>>0] = $encoding;
 $3 = HEAP32[$0>>2]|0;
 $4 = (($3) + 8|0);
 $5 = HEAP16[$4>>1]|0;
 $6 = $5&65535;
 $7 = $6 & 1;
 $8 = ($7|0)!=(0);
 if ($8) {
  STACKTOP = sp;return;
 }
 $9 = HEAP8[$1>>0]|0;
 $10 = $9&255;
 if ((($10|0) == 65)) {
  $11 = HEAP32[$0>>2]|0;
  $12 = (($11) + 8|0);
  $13 = HEAP16[$12>>1]|0;
  $14 = $13&65535;
  $15 = $14 & 16;
  $16 = ($15|0)==(0);
  $17 = HEAP32[$0>>2]|0;
  if ($16) {
   $18 = HEAP8[$2>>0]|0;
   _sqlite3ValueApplyAffinity($17,66,$18);
   $19 = HEAP32[$0>>2]|0;
   $20 = (($19) + 8|0);
   $21 = HEAP16[$20>>1]|0;
   $22 = $21&65535;
   $23 = $22 & -16896;
   $24 = $23 | 16;
   $25 = $24&65535;
   $26 = HEAP32[$0>>2]|0;
   $27 = (($26) + 8|0);
   HEAP16[$27>>1] = $25;
   STACKTOP = sp;return;
  } else {
   $28 = (($17) + 8|0);
   $29 = HEAP16[$28>>1]|0;
   $30 = $29&65535;
   $31 = $30 & -496;
   $32 = $31&65535;
   HEAP16[$28>>1] = $32;
   STACKTOP = sp;return;
  }
 } else if ((($10|0) == 69)) {
  $35 = HEAP32[$0>>2]|0;
  (_sqlite3VdbeMemRealify($35)|0);
  STACKTOP = sp;return;
 } else if ((($10|0) == 68)) {
  $34 = HEAP32[$0>>2]|0;
  (_sqlite3VdbeMemIntegerify($34)|0);
  STACKTOP = sp;return;
 } else if ((($10|0) == 67)) {
  $33 = HEAP32[$0>>2]|0;
  (_sqlite3VdbeMemNumerify($33)|0);
  STACKTOP = sp;return;
 } else {
  $36 = HEAP32[$0>>2]|0;
  $37 = (($36) + 8|0);
  $38 = HEAP16[$37>>1]|0;
  $39 = $38&65535;
  $40 = $39 & 16;
  $41 = $40 >> 3;
  $42 = HEAP32[$0>>2]|0;
  $43 = (($42) + 8|0);
  $44 = HEAP16[$43>>1]|0;
  $45 = $44&65535;
  $46 = $45 | $41;
  $47 = $46&65535;
  HEAP16[$43>>1] = $47;
  $48 = HEAP32[$0>>2]|0;
  $49 = HEAP8[$2>>0]|0;
  _sqlite3ValueApplyAffinity($48,66,$49);
  $50 = HEAP32[$0>>2]|0;
  $51 = (($50) + 8|0);
  $52 = HEAP16[$51>>1]|0;
  $53 = $52&65535;
  $54 = $53 & -16413;
  $55 = $54&65535;
  HEAP16[$51>>1] = $55;
  STACKTOP = sp;return;
 }
}
function _sqlite3ValueApplyAffinity($pVal,$affinity,$enc) {
 $pVal = $pVal|0;
 $affinity = $affinity|0;
 $enc = $enc|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 $1 = sp + 5|0;
 $2 = sp + 4|0;
 HEAP32[$0>>2] = $pVal;
 HEAP8[$1>>0] = $affinity;
 HEAP8[$2>>0] = $enc;
 $3 = HEAP32[$0>>2]|0;
 $4 = HEAP8[$1>>0]|0;
 $5 = HEAP8[$2>>0]|0;
 _applyAffinity($3,$4,$5);
 STACKTOP = sp;return;
}
function _valueNew($db,$p) {
 $db = $db|0;
 $p = $p|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $1 = sp;
 HEAP32[$0>>2] = $db;
 HEAP32[$1>>2] = $p;
 $2 = HEAP32[$0>>2]|0;
 $3 = (_sqlite3ValueNew($2)|0);
 STACKTOP = sp;return ($3|0);
}
function _sqlite3VdbeMemNumerify($pMem) {
 $pMem = $pMem|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0.0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 HEAP32[$0>>2] = $pMem;
 $1 = HEAP32[$0>>2]|0;
 $2 = (($1) + 8|0);
 $3 = HEAP16[$2>>1]|0;
 $4 = $3&65535;
 $5 = $4 & 13;
 $6 = ($5|0)==(0);
 do {
  if ($6) {
   $7 = HEAP32[$0>>2]|0;
   $8 = (($7) + 16|0);
   $9 = HEAP32[$8>>2]|0;
   $10 = HEAP32[$0>>2]|0;
   $11 = HEAP32[$0>>2]|0;
   $12 = (($11) + 12|0);
   $13 = HEAP32[$12>>2]|0;
   $14 = HEAP32[$0>>2]|0;
   $15 = (($14) + 10|0);
   $16 = HEAP8[$15>>0]|0;
   $17 = (_sqlite3Atoi64($9,$10,$13,$16)|0);
   $18 = (0)==($17|0);
   $19 = HEAP32[$0>>2]|0;
   if ($18) {
    $20 = (($19) + 8|0);
    $21 = HEAP16[$20>>1]|0;
    $22 = $21&65535;
    $23 = $22 & -16896;
    $24 = $23 | 4;
    $25 = $24&65535;
    $26 = HEAP32[$0>>2]|0;
    $27 = (($26) + 8|0);
    HEAP16[$27>>1] = $25;
    break;
   } else {
    $28 = (+_sqlite3VdbeRealValue($19));
    $29 = HEAP32[$0>>2]|0;
    HEAPF64[$29>>3] = $28;
    $30 = HEAP32[$0>>2]|0;
    $31 = (($30) + 8|0);
    $32 = HEAP16[$31>>1]|0;
    $33 = $32&65535;
    $34 = $33 & -16896;
    $35 = $34 | 8;
    $36 = $35&65535;
    $37 = HEAP32[$0>>2]|0;
    $38 = (($37) + 8|0);
    HEAP16[$38>>1] = $36;
    $39 = HEAP32[$0>>2]|0;
    _sqlite3VdbeIntegerAffinity($39);
    break;
   }
  }
 } while(0);
 $40 = HEAP32[$0>>2]|0;
 $41 = (($40) + 8|0);
 $42 = HEAP16[$41>>1]|0;
 $43 = $42&65535;
 $44 = $43 & -19;
 $45 = $44&65535;
 HEAP16[$41>>1] = $45;
 STACKTOP = sp;return 0;
}
function _sqlite3HexToBlob($db,$z,$n) {
 $db = $db|0;
 $z = $z|0;
 $n = $n|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $i = 0, $zBlob = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 16|0;
 $1 = sp + 12|0;
 $2 = sp + 8|0;
 $zBlob = sp + 4|0;
 $i = sp;
 HEAP32[$0>>2] = $db;
 HEAP32[$1>>2] = $z;
 HEAP32[$2>>2] = $n;
 $3 = HEAP32[$0>>2]|0;
 $4 = HEAP32[$2>>2]|0;
 $5 = (($4|0) / 2)&-1;
 $6 = (($5) + 1)|0;
 $7 = ($6|0)<(0);
 $8 = $7 << 31 >> 31;
 $9 = (_sqlite3DbMallocRaw($3,$6,$8)|0);
 HEAP32[$zBlob>>2] = $9;
 $10 = HEAP32[$2>>2]|0;
 $11 = (($10) + -1)|0;
 HEAP32[$2>>2] = $11;
 $12 = HEAP32[$zBlob>>2]|0;
 $13 = ($12|0)!=(0|0);
 if (!($13)) {
  $44 = HEAP32[$zBlob>>2]|0;
  STACKTOP = sp;return ($44|0);
 }
 HEAP32[$i>>2] = 0;
 while(1) {
  $14 = HEAP32[$i>>2]|0;
  $15 = HEAP32[$2>>2]|0;
  $16 = ($14|0)<($15|0);
  $17 = HEAP32[$i>>2]|0;
  if (!($16)) {
   break;
  }
  $18 = HEAP32[$1>>2]|0;
  $19 = (($18) + ($17)|0);
  $20 = HEAP8[$19>>0]|0;
  $21 = $20 << 24 >> 24;
  $22 = (_sqlite3HexToInt($21)|0);
  $23 = $22&255;
  $24 = $23 << 4;
  $25 = HEAP32[$i>>2]|0;
  $26 = (($25) + 1)|0;
  $27 = HEAP32[$1>>2]|0;
  $28 = (($27) + ($26)|0);
  $29 = HEAP8[$28>>0]|0;
  $30 = $29 << 24 >> 24;
  $31 = (_sqlite3HexToInt($30)|0);
  $32 = $31&255;
  $33 = $24 | $32;
  $34 = $33&255;
  $35 = HEAP32[$i>>2]|0;
  $36 = (($35|0) / 2)&-1;
  $37 = HEAP32[$zBlob>>2]|0;
  $38 = (($37) + ($36)|0);
  HEAP8[$38>>0] = $34;
  $39 = HEAP32[$i>>2]|0;
  $40 = (($39) + 2)|0;
  HEAP32[$i>>2] = $40;
 }
 $41 = (($17|0) / 2)&-1;
 $42 = HEAP32[$zBlob>>2]|0;
 $43 = (($42) + ($41)|0);
 HEAP8[$43>>0] = 0;
 $44 = HEAP32[$zBlob>>2]|0;
 STACKTOP = sp;return ($44|0);
}
function _sqlite3VdbeIntegerAffinity($pMem) {
 $pMem = $pMem|0;
 var $0 = 0, $1 = 0, $10 = 0.0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0.0, $18 = 0, $19 = 0, $2 = 0.0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $7 = 0, $8 = 0;
 var $9 = 0, $ix = 0, $or$cond = 0, $or$cond3 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $ix = sp;
 HEAP32[$0>>2] = $pMem;
 $1 = HEAP32[$0>>2]|0;
 $2 = +HEAPF64[$1>>3];
 $3 = (_doubleToInt64($2)|0);
 $4 = tempRet0;
 $5 = $ix;
 $6 = $5;
 HEAP32[$6>>2] = $3;
 $7 = (($5) + 4)|0;
 $8 = $7;
 HEAP32[$8>>2] = $4;
 $9 = HEAP32[$0>>2]|0;
 $10 = +HEAPF64[$9>>3];
 $11 = $ix;
 $12 = $11;
 $13 = HEAP32[$12>>2]|0;
 $14 = (($11) + 4)|0;
 $15 = $14;
 $16 = HEAP32[$15>>2]|0;
 $17 = (+($13>>>0)) + (4294967296.0*(+($16|0)));
 $18 = $10 == $17;
 $19 = $ix;
 $20 = $19;
 $21 = HEAP32[$20>>2]|0;
 $22 = (($19) + 4)|0;
 $23 = $22;
 $24 = HEAP32[$23>>2]|0;
 $25 = ($24|0)>(-2147483648);
 $26 = ($21>>>0)>(0);
 $27 = ($24|0)==(-2147483648);
 $28 = $27 & $26;
 $29 = $25 | $28;
 $or$cond = $18 & $29;
 $30 = $ix;
 $31 = $30;
 $32 = HEAP32[$31>>2]|0;
 $33 = (($30) + 4)|0;
 $34 = $33;
 $35 = HEAP32[$34>>2]|0;
 $36 = ($35|0)<(2147483647);
 $37 = ($32>>>0)<(4294967295);
 $38 = ($35|0)==(2147483647);
 $39 = $38 & $37;
 $40 = $36 | $39;
 $or$cond3 = $or$cond & $40;
 if (!($or$cond3)) {
  STACKTOP = sp;return;
 }
 $41 = $ix;
 $42 = $41;
 $43 = HEAP32[$42>>2]|0;
 $44 = (($41) + 4)|0;
 $45 = $44;
 $46 = HEAP32[$45>>2]|0;
 $47 = HEAP32[$0>>2]|0;
 $48 = $47;
 $49 = $48;
 HEAP32[$49>>2] = $43;
 $50 = (($48) + 4)|0;
 $51 = $50;
 HEAP32[$51>>2] = $46;
 $52 = HEAP32[$0>>2]|0;
 $53 = (($52) + 8|0);
 $54 = HEAP16[$53>>1]|0;
 $55 = $54&65535;
 $56 = $55 & -16896;
 $57 = $56 | 4;
 $58 = $57&65535;
 $59 = HEAP32[$0>>2]|0;
 $60 = (($59) + 8|0);
 HEAP16[$60>>1] = $58;
 STACKTOP = sp;return;
}
function _doubleToInt64($r) {
 $r = +$r;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0.0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0.0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0.0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $1 = sp;
 HEAPF64[$1>>3] = $r;
 $2 = +HEAPF64[$1>>3];
 $3 = $2 <= -9.2233720368547758E+18;
 do {
  if ($3) {
   $4 = $0;
   $5 = $4;
   HEAP32[$5>>2] = 0;
   $6 = (($4) + 4)|0;
   $7 = $6;
   HEAP32[$7>>2] = -2147483648;
  } else {
   $8 = +HEAPF64[$1>>3];
   $9 = $8 >= 9.2233720368547758E+18;
   if ($9) {
    $10 = $0;
    $11 = $10;
    HEAP32[$11>>2] = -1;
    $12 = (($10) + 4)|0;
    $13 = $12;
    HEAP32[$13>>2] = 2147483647;
    break;
   } else {
    $14 = +HEAPF64[$1>>3];
    $15 = (~~$14)>>>0;
    $16 = +Math_abs($14) >= 1.0 ? $14 > 0.0 ? (Math_min(+Math_floor($14 / 4294967296.0), 4294967295.0) | 0) >>> 0 : ~~+Math_ceil(($14 - +(~~$14 >>> 0)) / 4294967296.0) >>> 0 : 0;
    $17 = $0;
    $18 = $17;
    HEAP32[$18>>2] = $15;
    $19 = (($17) + 4)|0;
    $20 = $19;
    HEAP32[$20>>2] = $16;
    break;
   }
  }
 } while(0);
 $21 = $0;
 $22 = $21;
 $23 = HEAP32[$22>>2]|0;
 $24 = (($21) + 4)|0;
 $25 = $24;
 $26 = HEAP32[$25>>2]|0;
 tempRet0 = $26;
 STACKTOP = sp;return ($23|0);
}
function _applyAffinity($pRec,$affinity,$enc) {
 $pRec = $pRec|0;
 $affinity = $affinity|0;
 $enc = $enc|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $5 = 0, $6 = 0, $7 = 0;
 var $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 $1 = sp + 5|0;
 $2 = sp + 4|0;
 HEAP32[$0>>2] = $pRec;
 HEAP8[$1>>0] = $affinity;
 HEAP8[$2>>0] = $enc;
 $3 = HEAP8[$1>>0]|0;
 $4 = $3 << 24 >> 24;
 $5 = ($4|0)>=(67);
 if (!($5)) {
  $25 = HEAP8[$1>>0]|0;
  $26 = $25 << 24 >> 24;
  $27 = ($26|0)==(66);
  if (!($27)) {
   STACKTOP = sp;return;
  }
  $28 = HEAP32[$0>>2]|0;
  $29 = (($28) + 8|0);
  $30 = HEAP16[$29>>1]|0;
  $31 = $30&65535;
  $32 = $31 & 2;
  $33 = (0)==($32|0);
  if (!($33)) {
   STACKTOP = sp;return;
  }
  $34 = HEAP32[$0>>2]|0;
  $35 = (($34) + 8|0);
  $36 = HEAP16[$35>>1]|0;
  $37 = $36&65535;
  $38 = $37 & 12;
  $39 = ($38|0)!=(0);
  if (!($39)) {
   STACKTOP = sp;return;
  }
  $40 = HEAP32[$0>>2]|0;
  $41 = HEAP8[$2>>0]|0;
  (_sqlite3VdbeMemStringify($40,$41,1)|0);
  STACKTOP = sp;return;
 }
 $6 = HEAP32[$0>>2]|0;
 $7 = (($6) + 8|0);
 $8 = HEAP16[$7>>1]|0;
 $9 = $8&65535;
 $10 = $9 & 4;
 $11 = ($10|0)==(0);
 if (!($11)) {
  STACKTOP = sp;return;
 }
 $12 = HEAP32[$0>>2]|0;
 $13 = (($12) + 8|0);
 $14 = HEAP16[$13>>1]|0;
 $15 = $14&65535;
 $16 = $15 & 8;
 $17 = ($16|0)==(0);
 $18 = HEAP32[$0>>2]|0;
 if (!($17)) {
  _sqlite3VdbeIntegerAffinity($18);
  STACKTOP = sp;return;
 }
 $19 = (($18) + 8|0);
 $20 = HEAP16[$19>>1]|0;
 $21 = $20&65535;
 $22 = $21 & 2;
 $23 = ($22|0)!=(0);
 if (!($23)) {
  STACKTOP = sp;return;
 }
 $24 = HEAP32[$0>>2]|0;
 _applyNumericAffinity($24,1);
 STACKTOP = sp;return;
}
function _sqlite3VdbeMemStringify($pMem,$enc,$bForce) {
 $pMem = $pMem|0;
 $enc = $enc|0;
 $bForce = $bForce|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0.0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $fg = 0, $nByte = 0, $vararg_buffer1 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $vararg_buffer1 = sp;
 $0 = sp + 20|0;
 $1 = sp + 16|0;
 $2 = sp + 25|0;
 $3 = sp + 24|0;
 $fg = sp + 12|0;
 $nByte = sp + 8|0;
 HEAP32[$1>>2] = $pMem;
 HEAP8[$2>>0] = $enc;
 HEAP8[$3>>0] = $bForce;
 $4 = HEAP32[$1>>2]|0;
 $5 = (($4) + 8|0);
 $6 = HEAP16[$5>>1]|0;
 $7 = $6&65535;
 HEAP32[$fg>>2] = $7;
 HEAP32[$nByte>>2] = 32;
 $8 = HEAP32[$1>>2]|0;
 $9 = (_sqlite3VdbeMemClearAndResize($8,32)|0);
 $10 = ($9|0)!=(0);
 if ($10) {
  HEAP32[$0>>2] = 7;
  $54 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($54|0);
 }
 $11 = HEAP32[$fg>>2]|0;
 $12 = $11 & 4;
 $13 = ($12|0)!=(0);
 $14 = HEAP32[$1>>2]|0;
 $15 = (($14) + 16|0);
 $16 = HEAP32[$15>>2]|0;
 $17 = HEAP32[$1>>2]|0;
 if ($13) {
  $18 = $17;
  $19 = $18;
  $20 = HEAP32[$19>>2]|0;
  $21 = (($18) + 4)|0;
  $22 = $21;
  $23 = HEAP32[$22>>2]|0;
  $24 = $vararg_buffer1;
  $25 = $24;
  HEAP32[$25>>2] = $20;
  $26 = (($24) + 4)|0;
  $27 = $26;
  HEAP32[$27>>2] = $23;
  (_sqlite3_snprintf(32,$16,9456,$vararg_buffer1)|0);
 } else {
  $28 = +HEAPF64[$17>>3];
  HEAPF64[tempDoublePtr>>3]=$28;HEAP32[$vararg_buffer1>>2]=HEAP32[tempDoublePtr>>2];HEAP32[$vararg_buffer1+4>>2]=HEAP32[tempDoublePtr+4>>2];
  (_sqlite3_snprintf(32,$16,9808,$vararg_buffer1)|0);
 }
 $29 = HEAP32[$1>>2]|0;
 $30 = (($29) + 16|0);
 $31 = HEAP32[$30>>2]|0;
 $32 = (_sqlite3Strlen30($31)|0);
 $33 = HEAP32[$1>>2]|0;
 $34 = (($33) + 12|0);
 HEAP32[$34>>2] = $32;
 $35 = HEAP32[$1>>2]|0;
 $36 = (($35) + 10|0);
 HEAP8[$36>>0] = 1;
 $37 = HEAP32[$1>>2]|0;
 $38 = (($37) + 8|0);
 $39 = HEAP16[$38>>1]|0;
 $40 = $39&65535;
 $41 = $40 | 514;
 $42 = $41&65535;
 HEAP16[$38>>1] = $42;
 $43 = HEAP8[$3>>0]|0;
 $44 = ($43<<24>>24)!=(0);
 if ($44) {
  $45 = HEAP32[$1>>2]|0;
  $46 = (($45) + 8|0);
  $47 = HEAP16[$46>>1]|0;
  $48 = $47&65535;
  $49 = $48 & -13;
  $50 = $49&65535;
  HEAP16[$46>>1] = $50;
 }
 $51 = HEAP32[$1>>2]|0;
 $52 = HEAP8[$2>>0]|0;
 $53 = $52&255;
 (_sqlite3VdbeChangeEncoding($51,$53)|0);
 HEAP32[$0>>2] = 0;
 $54 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($54|0);
}
function _sqlite3VdbeMemClearAndResize($pMem,$szNew) {
 $pMem = $pMem|0;
 $szNew = $szNew|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0;
 var $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $1 = sp + 4|0;
 $2 = sp;
 HEAP32[$1>>2] = $pMem;
 HEAP32[$2>>2] = $szNew;
 $3 = HEAP32[$1>>2]|0;
 $4 = (($3) + 24|0);
 $5 = HEAP32[$4>>2]|0;
 $6 = HEAP32[$2>>2]|0;
 $7 = ($5|0)<($6|0);
 $8 = HEAP32[$1>>2]|0;
 if ($7) {
  $9 = HEAP32[$2>>2]|0;
  $10 = (_sqlite3VdbeMemGrow($8,$9,0)|0);
  HEAP32[$0>>2] = $10;
  $21 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($21|0);
 } else {
  $11 = (($8) + 20|0);
  $12 = HEAP32[$11>>2]|0;
  $13 = HEAP32[$1>>2]|0;
  $14 = (($13) + 16|0);
  HEAP32[$14>>2] = $12;
  $15 = HEAP32[$1>>2]|0;
  $16 = (($15) + 8|0);
  $17 = HEAP16[$16>>1]|0;
  $18 = $17&65535;
  $19 = $18 & 13;
  $20 = $19&65535;
  HEAP16[$16>>1] = $20;
  HEAP32[$0>>2] = 0;
  $21 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($21|0);
 }
 return 0|0;
}
function _sqlite3VdbeMemIntegerify($pMem) {
 $pMem = $pMem|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 HEAP32[$0>>2] = $pMem;
 $1 = HEAP32[$0>>2]|0;
 $2 = (_sqlite3VdbeIntValue($1)|0);
 $3 = tempRet0;
 $4 = HEAP32[$0>>2]|0;
 $5 = $4;
 $6 = $5;
 HEAP32[$6>>2] = $2;
 $7 = (($5) + 4)|0;
 $8 = $7;
 HEAP32[$8>>2] = $3;
 $9 = HEAP32[$0>>2]|0;
 $10 = (($9) + 8|0);
 $11 = HEAP16[$10>>1]|0;
 $12 = $11&65535;
 $13 = $12 & -16896;
 $14 = $13 | 4;
 $15 = $14&65535;
 $16 = HEAP32[$0>>2]|0;
 $17 = (($16) + 8|0);
 HEAP16[$17>>1] = $15;
 STACKTOP = sp;return 0;
}
function _sqlite3VdbeMemRealify($pMem) {
 $pMem = $pMem|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $2 = 0.0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 HEAP32[$0>>2] = $pMem;
 $1 = HEAP32[$0>>2]|0;
 $2 = (+_sqlite3VdbeRealValue($1));
 $3 = HEAP32[$0>>2]|0;
 HEAPF64[$3>>3] = $2;
 $4 = HEAP32[$0>>2]|0;
 $5 = (($4) + 8|0);
 $6 = HEAP16[$5>>1]|0;
 $7 = $6&65535;
 $8 = $7 & -16896;
 $9 = $8 | 8;
 $10 = $9&65535;
 $11 = HEAP32[$0>>2]|0;
 $12 = (($11) + 8|0);
 HEAP16[$12>>1] = $10;
 STACKTOP = sp;return 0;
}
function _sqlite3FindIndex($db,$zName,$zDb) {
 $db = $db|0;
 $zName = $zName|0;
 $zDb = $zDb|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0;
 var $i = 0, $j = 0, $p = 0, $pSchema = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 24|0;
 $1 = sp + 20|0;
 $2 = sp + 16|0;
 $p = sp + 12|0;
 $i = sp + 8|0;
 $j = sp + 4|0;
 $pSchema = sp;
 HEAP32[$0>>2] = $db;
 HEAP32[$1>>2] = $zName;
 HEAP32[$2>>2] = $zDb;
 HEAP32[$p>>2] = 0;
 HEAP32[$i>>2] = 0;
 while(1) {
  $3 = HEAP32[$i>>2]|0;
  $4 = HEAP32[$0>>2]|0;
  $5 = (($4) + 20|0);
  $6 = HEAP32[$5>>2]|0;
  $7 = ($3|0)<($6|0);
  if (!($7)) {
   label = 7;
   break;
  }
  $8 = HEAP32[$i>>2]|0;
  $9 = ($8|0)<(2);
  $10 = HEAP32[$i>>2]|0;
  $11 = $10 ^ 1;
  $12 = $9 ? $11 : $10;
  HEAP32[$j>>2] = $12;
  $13 = HEAP32[$j>>2]|0;
  $14 = HEAP32[$0>>2]|0;
  $15 = (($14) + 16|0);
  $16 = HEAP32[$15>>2]|0;
  $17 = (($16) + ($13<<4)|0);
  $18 = (($17) + 12|0);
  $19 = HEAP32[$18>>2]|0;
  HEAP32[$pSchema>>2] = $19;
  $20 = HEAP32[$2>>2]|0;
  $21 = ($20|0)!=(0|0);
  if ($21) {
   $22 = HEAP32[$2>>2]|0;
   $23 = HEAP32[$j>>2]|0;
   $24 = HEAP32[$0>>2]|0;
   $25 = (($24) + 16|0);
   $26 = HEAP32[$25>>2]|0;
   $27 = (($26) + ($23<<4)|0);
   $28 = HEAP32[$27>>2]|0;
   $29 = (_sqlite3_stricmp($22,$28)|0);
   $30 = ($29|0)!=(0);
   if (!($30)) {
    label = 5;
   }
  } else {
   label = 5;
  }
  if ((label|0) == 5) {
   label = 0;
   $31 = HEAP32[$pSchema>>2]|0;
   $32 = (($31) + 24|0);
   $33 = HEAP32[$1>>2]|0;
   $34 = (_sqlite3HashFind($32,$33)|0);
   HEAP32[$p>>2] = $34;
   $35 = HEAP32[$p>>2]|0;
   $36 = ($35|0)!=(0|0);
   if ($36) {
    label = 7;
    break;
   }
  }
  $37 = HEAP32[$i>>2]|0;
  $38 = (($37) + 1)|0;
  HEAP32[$i>>2] = $38;
 }
 if ((label|0) == 7) {
  $39 = HEAP32[$p>>2]|0;
  STACKTOP = sp;return ($39|0);
 }
 return 0|0;
}
function _sqlite3CheckObjectName($pParse,$zName) {
 $pParse = $pParse|0;
 $zName = $zName|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $3 = 0;
 var $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $vararg_buffer = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $vararg_buffer = sp;
 $0 = sp + 12|0;
 $1 = sp + 8|0;
 $2 = sp + 4|0;
 HEAP32[$1>>2] = $pParse;
 HEAP32[$2>>2] = $zName;
 $3 = HEAP32[$1>>2]|0;
 $4 = HEAP32[$3>>2]|0;
 $5 = (($4) + 140|0);
 $6 = (($5) + 5|0);
 $7 = HEAP8[$6>>0]|0;
 $8 = ($7<<24>>24)!=(0);
 if (!($8)) {
  $9 = HEAP32[$1>>2]|0;
  $10 = (($9) + 18|0);
  $11 = HEAP8[$10>>0]|0;
  $12 = $11&255;
  $13 = ($12|0)==(0);
  if ($13) {
   $14 = HEAP32[$1>>2]|0;
   $15 = HEAP32[$14>>2]|0;
   $16 = (($15) + 24|0);
   $17 = HEAP32[$16>>2]|0;
   $18 = $17 & 2048;
   $19 = ($18|0)==(0);
   if ($19) {
    $20 = HEAP32[$2>>2]|0;
    $21 = (_sqlite3_strnicmp($20,15144,7)|0);
    $22 = (0)==($21|0);
    if ($22) {
     $23 = HEAP32[$1>>2]|0;
     $24 = HEAP32[$2>>2]|0;
     HEAP32[$vararg_buffer>>2] = $24;
     _sqlite3ErrorMsg($23,18064,$vararg_buffer);
     HEAP32[$0>>2] = 1;
     $25 = HEAP32[$0>>2]|0;
     STACKTOP = sp;return ($25|0);
    }
   }
  }
 }
 HEAP32[$0>>2] = 0;
 $25 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($25|0);
}
function _sqlite3ViewGetColumnNames($pParse,$pTable) {
 $pParse = $pParse|0;
 $pTable = $pTable|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0;
 var $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0;
 var $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0;
 var $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0;
 var $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0;
 var $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $db = 0, $enableLookaside = 0, $n = 0, $nErr = 0, $pSel = 0, $pSelTab = 0, $vararg_buffer = 0, $xAuth = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $vararg_buffer = sp;
 $0 = sp + 36|0;
 $1 = sp + 32|0;
 $2 = sp + 28|0;
 $pSelTab = sp + 24|0;
 $pSel = sp + 20|0;
 $nErr = sp + 16|0;
 $n = sp + 12|0;
 $db = sp + 8|0;
 $xAuth = sp + 4|0;
 $enableLookaside = sp + 40|0;
 HEAP32[$1>>2] = $pParse;
 HEAP32[$2>>2] = $pTable;
 HEAP32[$nErr>>2] = 0;
 $3 = HEAP32[$1>>2]|0;
 $4 = HEAP32[$3>>2]|0;
 HEAP32[$db>>2] = $4;
 $5 = HEAP32[$1>>2]|0;
 $6 = HEAP32[$2>>2]|0;
 $7 = (_sqlite3VtabCallConnect($5,$6)|0);
 $8 = ($7|0)!=(0);
 if ($8) {
  HEAP32[$0>>2] = 1;
  $104 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($104|0);
 }
 $9 = HEAP32[$2>>2]|0;
 $10 = (($9) + 44|0);
 $11 = HEAP8[$10>>0]|0;
 $12 = $11&255;
 $13 = $12 & 16;
 $14 = ($13|0)!=(0);
 if ($14) {
  HEAP32[$0>>2] = 0;
  $104 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($104|0);
 }
 $15 = HEAP32[$2>>2]|0;
 $16 = (($15) + 38|0);
 $17 = HEAP16[$16>>1]|0;
 $18 = $17 << 16 >> 16;
 $19 = ($18|0)>(0);
 if ($19) {
  HEAP32[$0>>2] = 0;
  $104 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($104|0);
 }
 $20 = HEAP32[$2>>2]|0;
 $21 = (($20) + 38|0);
 $22 = HEAP16[$21>>1]|0;
 $23 = $22 << 16 >> 16;
 $24 = ($23|0)<(0);
 if ($24) {
  $25 = HEAP32[$1>>2]|0;
  $26 = HEAP32[$2>>2]|0;
  $27 = HEAP32[$26>>2]|0;
  HEAP32[$vararg_buffer>>2] = $27;
  _sqlite3ErrorMsg($25,16328,$vararg_buffer);
  HEAP32[$0>>2] = 1;
  $104 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($104|0);
 }
 $28 = HEAP32[$db>>2]|0;
 $29 = HEAP32[$2>>2]|0;
 $30 = (($29) + 12|0);
 $31 = HEAP32[$30>>2]|0;
 $32 = (_sqlite3SelectDup($28,$31,0)|0);
 HEAP32[$pSel>>2] = $32;
 $33 = HEAP32[$pSel>>2]|0;
 $34 = ($33|0)!=(0|0);
 if ($34) {
  $35 = HEAP32[$db>>2]|0;
  $36 = (($35) + 248|0);
  $37 = (($36) + 2|0);
  $38 = HEAP8[$37>>0]|0;
  HEAP8[$enableLookaside>>0] = $38;
  $39 = HEAP32[$1>>2]|0;
  $40 = (($39) + 68|0);
  $41 = HEAP32[$40>>2]|0;
  HEAP32[$n>>2] = $41;
  $42 = HEAP32[$1>>2]|0;
  $43 = HEAP32[$pSel>>2]|0;
  $44 = (($43) + 32|0);
  $45 = HEAP32[$44>>2]|0;
  _sqlite3SrcListAssignCursors($42,$45);
  $46 = HEAP32[$2>>2]|0;
  $47 = (($46) + 38|0);
  HEAP16[$47>>1] = -1;
  $48 = HEAP32[$db>>2]|0;
  $49 = (($48) + 248|0);
  $50 = (($49) + 2|0);
  HEAP8[$50>>0] = 0;
  $51 = HEAP32[$db>>2]|0;
  $52 = (($51) + 284|0);
  $53 = HEAP32[$52>>2]|0;
  HEAP32[$xAuth>>2] = $53;
  $54 = HEAP32[$db>>2]|0;
  $55 = (($54) + 284|0);
  HEAP32[$55>>2] = 0;
  $56 = HEAP32[$1>>2]|0;
  $57 = HEAP32[$pSel>>2]|0;
  $58 = (_sqlite3ResultSetOfSelect($56,$57)|0);
  HEAP32[$pSelTab>>2] = $58;
  $59 = HEAP32[$xAuth>>2]|0;
  $60 = HEAP32[$db>>2]|0;
  $61 = (($60) + 284|0);
  HEAP32[$61>>2] = $59;
  $62 = HEAP8[$enableLookaside>>0]|0;
  $63 = HEAP32[$db>>2]|0;
  $64 = (($63) + 248|0);
  $65 = (($64) + 2|0);
  HEAP8[$65>>0] = $62;
  $66 = HEAP32[$n>>2]|0;
  $67 = HEAP32[$1>>2]|0;
  $68 = (($67) + 68|0);
  HEAP32[$68>>2] = $66;
  $69 = HEAP32[$pSelTab>>2]|0;
  $70 = ($69|0)!=(0|0);
  if ($70) {
   $71 = HEAP32[$pSelTab>>2]|0;
   $72 = (($71) + 38|0);
   $73 = HEAP16[$72>>1]|0;
   $74 = HEAP32[$2>>2]|0;
   $75 = (($74) + 38|0);
   HEAP16[$75>>1] = $73;
   $76 = HEAP32[$pSelTab>>2]|0;
   $77 = (($76) + 4|0);
   $78 = HEAP32[$77>>2]|0;
   $79 = HEAP32[$2>>2]|0;
   $80 = (($79) + 4|0);
   HEAP32[$80>>2] = $78;
   $81 = HEAP32[$pSelTab>>2]|0;
   $82 = (($81) + 38|0);
   HEAP16[$82>>1] = 0;
   $83 = HEAP32[$pSelTab>>2]|0;
   $84 = (($83) + 4|0);
   HEAP32[$84>>2] = 0;
   $85 = HEAP32[$db>>2]|0;
   $86 = HEAP32[$pSelTab>>2]|0;
   _sqlite3DeleteTable($85,$86);
   $87 = HEAP32[$2>>2]|0;
   $88 = (($87) + 68|0);
   $89 = HEAP32[$88>>2]|0;
   $90 = (($89) + 78|0);
   $91 = HEAP16[$90>>1]|0;
   $92 = $91&65535;
   $93 = $92 | 2;
   $94 = $93&65535;
   HEAP16[$90>>1] = $94;
  } else {
   $95 = HEAP32[$2>>2]|0;
   $96 = (($95) + 38|0);
   HEAP16[$96>>1] = 0;
   $97 = HEAP32[$nErr>>2]|0;
   $98 = (($97) + 1)|0;
   HEAP32[$nErr>>2] = $98;
  }
  $99 = HEAP32[$db>>2]|0;
  $100 = HEAP32[$pSel>>2]|0;
  _sqlite3SelectDelete($99,$100);
 } else {
  $101 = HEAP32[$nErr>>2]|0;
  $102 = (($101) + 1)|0;
  HEAP32[$nErr>>2] = $102;
 }
 $103 = HEAP32[$nErr>>2]|0;
 HEAP32[$0>>2] = $103;
 $104 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($104|0);
}
function _sqlite3GetVTable($db,$pTab) {
 $db = $db|0;
 $pTab = $pTab|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $pVtab = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $1 = sp + 4|0;
 $pVtab = sp;
 HEAP32[$0>>2] = $db;
 HEAP32[$1>>2] = $pTab;
 $2 = HEAP32[$1>>2]|0;
 $3 = (($2) + 60|0);
 $4 = HEAP32[$3>>2]|0;
 HEAP32[$pVtab>>2] = $4;
 while(1) {
  $5 = HEAP32[$pVtab>>2]|0;
  $6 = ($5|0)!=(0|0);
  if (!($6)) {
   label = 5;
   break;
  }
  $7 = HEAP32[$pVtab>>2]|0;
  $8 = HEAP32[$7>>2]|0;
  $9 = HEAP32[$0>>2]|0;
  $10 = ($8|0)!=($9|0);
  if (!($10)) {
   label = 5;
   break;
  }
  $11 = HEAP32[$pVtab>>2]|0;
  $12 = (($11) + 24|0);
  $13 = HEAP32[$12>>2]|0;
  HEAP32[$pVtab>>2] = $13;
 }
 if ((label|0) == 5) {
  $14 = HEAP32[$pVtab>>2]|0;
  STACKTOP = sp;return ($14|0);
 }
 return 0|0;
}
function _sqlite3MayAbort($pParse) {
 $pParse = $pParse|0;
 var $0 = 0, $1 = 0, $10 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $pToplevel = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $pToplevel = sp;
 HEAP32[$0>>2] = $pParse;
 $1 = HEAP32[$0>>2]|0;
 $2 = (($1) + 408|0);
 $3 = HEAP32[$2>>2]|0;
 $4 = ($3|0)!=(0|0);
 $5 = HEAP32[$0>>2]|0;
 if ($4) {
  $6 = (($5) + 408|0);
  $7 = HEAP32[$6>>2]|0;
  $8 = $7;
 } else {
  $8 = $5;
 }
 HEAP32[$pToplevel>>2] = $8;
 $9 = HEAP32[$pToplevel>>2]|0;
 $10 = (($9) + 21|0);
 HEAP8[$10>>0] = 1;
 STACKTOP = sp;return;
}
function _whereForeignKeys($pParse,$pTab) {
 $pParse = $pParse|0;
 $pTab = $pTab|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $p = 0, $zWhere = 0, label = 0;
 var sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $1 = sp + 8|0;
 $p = sp + 4|0;
 $zWhere = sp;
 HEAP32[$0>>2] = $pParse;
 HEAP32[$1>>2] = $pTab;
 HEAP32[$zWhere>>2] = 0;
 $2 = HEAP32[$1>>2]|0;
 $3 = (_sqlite3FkReferences($2)|0);
 HEAP32[$p>>2] = $3;
 while(1) {
  $4 = HEAP32[$p>>2]|0;
  $5 = ($4|0)!=(0|0);
  if (!($5)) {
   break;
  }
  $6 = HEAP32[$0>>2]|0;
  $7 = HEAP32[$6>>2]|0;
  $8 = HEAP32[$zWhere>>2]|0;
  $9 = HEAP32[$p>>2]|0;
  $10 = HEAP32[$9>>2]|0;
  $11 = HEAP32[$10>>2]|0;
  $12 = (_whereOrName($7,$8,$11)|0);
  HEAP32[$zWhere>>2] = $12;
  $13 = HEAP32[$p>>2]|0;
  $14 = (($13) + 12|0);
  $15 = HEAP32[$14>>2]|0;
  HEAP32[$p>>2] = $15;
 }
 $16 = HEAP32[$zWhere>>2]|0;
 STACKTOP = sp;return ($16|0);
}
function _sqlite3FkReferences($pTab) {
 $pTab = $pTab|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 HEAP32[$0>>2] = $pTab;
 $1 = HEAP32[$0>>2]|0;
 $2 = (($1) + 68|0);
 $3 = HEAP32[$2>>2]|0;
 $4 = (($3) + 56|0);
 $5 = HEAP32[$0>>2]|0;
 $6 = HEAP32[$5>>2]|0;
 $7 = (_sqlite3HashFind($4,$6)|0);
 STACKTOP = sp;return ($7|0);
}
function _sqlite3VtabCallConnect($pParse,$pTab) {
 $pParse = $pParse|0;
 $pTab = $pTab|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $db = 0, $pMod = 0, $rc = 0, $vararg_buffer1 = 0, $zErr = 0, $zMod = 0, $zModule = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $vararg_buffer1 = sp;
 $0 = sp + 36|0;
 $1 = sp + 32|0;
 $2 = sp + 28|0;
 $db = sp + 24|0;
 $zMod = sp + 20|0;
 $pMod = sp + 16|0;
 $rc = sp + 12|0;
 $zModule = sp + 8|0;
 $zErr = sp + 4|0;
 HEAP32[$1>>2] = $pParse;
 HEAP32[$2>>2] = $pTab;
 $3 = HEAP32[$1>>2]|0;
 $4 = HEAP32[$3>>2]|0;
 HEAP32[$db>>2] = $4;
 $5 = HEAP32[$2>>2]|0;
 $6 = (($5) + 44|0);
 $7 = HEAP8[$6>>0]|0;
 $8 = $7&255;
 $9 = $8 & 16;
 $10 = ($9|0)==(0);
 if (!($10)) {
  $11 = HEAP32[$db>>2]|0;
  $12 = HEAP32[$2>>2]|0;
  $13 = (_sqlite3GetVTable($11,$12)|0);
  $14 = ($13|0)!=(0|0);
  if (!($14)) {
   $15 = HEAP32[$2>>2]|0;
   $16 = (($15) + 56|0);
   $17 = HEAP32[$16>>2]|0;
   $18 = HEAP32[$17>>2]|0;
   HEAP32[$zMod>>2] = $18;
   $19 = HEAP32[$db>>2]|0;
   $20 = (($19) + 308|0);
   $21 = HEAP32[$zMod>>2]|0;
   $22 = (_sqlite3HashFind($20,$21)|0);
   HEAP32[$pMod>>2] = $22;
   $23 = HEAP32[$pMod>>2]|0;
   $24 = ($23|0)!=(0|0);
   if ($24) {
    HEAP32[$zErr>>2] = 0;
    $31 = HEAP32[$db>>2]|0;
    $32 = HEAP32[$2>>2]|0;
    $33 = HEAP32[$pMod>>2]|0;
    $34 = HEAP32[$pMod>>2]|0;
    $35 = HEAP32[$34>>2]|0;
    $36 = (($35) + 8|0);
    $37 = HEAP32[$36>>2]|0;
    $38 = (_vtabCallConstructor($31,$32,$33,$37,$zErr)|0);
    HEAP32[$rc>>2] = $38;
    $39 = HEAP32[$rc>>2]|0;
    $40 = ($39|0)!=(0);
    if ($40) {
     $41 = HEAP32[$1>>2]|0;
     $42 = HEAP32[$zErr>>2]|0;
     HEAP32[$vararg_buffer1>>2] = $42;
     _sqlite3ErrorMsg($41,1200,$vararg_buffer1);
    }
    $43 = HEAP32[$db>>2]|0;
    $44 = HEAP32[$zErr>>2]|0;
    _sqlite3DbFree($43,$44);
   } else {
    $25 = HEAP32[$2>>2]|0;
    $26 = (($25) + 56|0);
    $27 = HEAP32[$26>>2]|0;
    $28 = HEAP32[$27>>2]|0;
    HEAP32[$zModule>>2] = $28;
    $29 = HEAP32[$1>>2]|0;
    $30 = HEAP32[$zModule>>2]|0;
    HEAP32[$vararg_buffer1>>2] = $30;
    _sqlite3ErrorMsg($29,17944,$vararg_buffer1);
    HEAP32[$rc>>2] = 1;
   }
   $45 = HEAP32[$rc>>2]|0;
   HEAP32[$0>>2] = $45;
   $46 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($46|0);
  }
 }
 HEAP32[$0>>2] = 0;
 $46 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($46|0);
}
function _sqlite3SelectDup($db,$p,$flags) {
 $db = $db|0;
 $p = $p|0;
 $flags = $flags|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $14 = 0, $15 = 0;
 var $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0;
 var $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0;
 var $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0;
 var $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0;
 var $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $pNew = 0, $pPrior = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 20|0;
 $1 = sp + 16|0;
 $2 = sp + 12|0;
 $3 = sp + 8|0;
 $pNew = sp + 4|0;
 $pPrior = sp;
 HEAP32[$1>>2] = $db;
 HEAP32[$2>>2] = $p;
 HEAP32[$3>>2] = $flags;
 $4 = HEAP32[$2>>2]|0;
 $5 = ($4|0)==(0|0);
 if ($5) {
  HEAP32[$0>>2] = 0;
  $131 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($131|0);
 }
 $6 = HEAP32[$1>>2]|0;
 $7 = (_sqlite3DbMallocRaw($6,72,0)|0);
 HEAP32[$pNew>>2] = $7;
 $8 = HEAP32[$pNew>>2]|0;
 $9 = ($8|0)==(0|0);
 if ($9) {
  HEAP32[$0>>2] = 0;
  $131 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($131|0);
 }
 $10 = HEAP32[$1>>2]|0;
 $11 = HEAP32[$2>>2]|0;
 $12 = HEAP32[$11>>2]|0;
 $13 = HEAP32[$3>>2]|0;
 $14 = (_sqlite3ExprListDup($10,$12,$13)|0);
 $15 = HEAP32[$pNew>>2]|0;
 HEAP32[$15>>2] = $14;
 $16 = HEAP32[$1>>2]|0;
 $17 = HEAP32[$2>>2]|0;
 $18 = (($17) + 32|0);
 $19 = HEAP32[$18>>2]|0;
 $20 = HEAP32[$3>>2]|0;
 $21 = (_sqlite3SrcListDup($16,$19,$20)|0);
 $22 = HEAP32[$pNew>>2]|0;
 $23 = (($22) + 32|0);
 HEAP32[$23>>2] = $21;
 $24 = HEAP32[$1>>2]|0;
 $25 = HEAP32[$2>>2]|0;
 $26 = (($25) + 36|0);
 $27 = HEAP32[$26>>2]|0;
 $28 = HEAP32[$3>>2]|0;
 $29 = (_sqlite3ExprDup($24,$27,$28)|0);
 $30 = HEAP32[$pNew>>2]|0;
 $31 = (($30) + 36|0);
 HEAP32[$31>>2] = $29;
 $32 = HEAP32[$1>>2]|0;
 $33 = HEAP32[$2>>2]|0;
 $34 = (($33) + 40|0);
 $35 = HEAP32[$34>>2]|0;
 $36 = HEAP32[$3>>2]|0;
 $37 = (_sqlite3ExprListDup($32,$35,$36)|0);
 $38 = HEAP32[$pNew>>2]|0;
 $39 = (($38) + 40|0);
 HEAP32[$39>>2] = $37;
 $40 = HEAP32[$1>>2]|0;
 $41 = HEAP32[$2>>2]|0;
 $42 = (($41) + 44|0);
 $43 = HEAP32[$42>>2]|0;
 $44 = HEAP32[$3>>2]|0;
 $45 = (_sqlite3ExprDup($40,$43,$44)|0);
 $46 = HEAP32[$pNew>>2]|0;
 $47 = (($46) + 44|0);
 HEAP32[$47>>2] = $45;
 $48 = HEAP32[$1>>2]|0;
 $49 = HEAP32[$2>>2]|0;
 $50 = (($49) + 48|0);
 $51 = HEAP32[$50>>2]|0;
 $52 = HEAP32[$3>>2]|0;
 $53 = (_sqlite3ExprListDup($48,$51,$52)|0);
 $54 = HEAP32[$pNew>>2]|0;
 $55 = (($54) + 48|0);
 HEAP32[$55>>2] = $53;
 $56 = HEAP32[$2>>2]|0;
 $57 = (($56) + 4|0);
 $58 = HEAP8[$57>>0]|0;
 $59 = HEAP32[$pNew>>2]|0;
 $60 = (($59) + 4|0);
 HEAP8[$60>>0] = $58;
 $61 = HEAP32[$1>>2]|0;
 $62 = HEAP32[$2>>2]|0;
 $63 = (($62) + 52|0);
 $64 = HEAP32[$63>>2]|0;
 $65 = HEAP32[$3>>2]|0;
 $66 = (_sqlite3SelectDup($61,$64,$65)|0);
 HEAP32[$pPrior>>2] = $66;
 $67 = HEAP32[$pNew>>2]|0;
 $68 = (($67) + 52|0);
 HEAP32[$68>>2] = $66;
 $69 = HEAP32[$pPrior>>2]|0;
 $70 = ($69|0)!=(0|0);
 if ($70) {
  $71 = HEAP32[$pNew>>2]|0;
  $72 = HEAP32[$pPrior>>2]|0;
  $73 = (($72) + 56|0);
  HEAP32[$73>>2] = $71;
 }
 $74 = HEAP32[$pNew>>2]|0;
 $75 = (($74) + 56|0);
 HEAP32[$75>>2] = 0;
 $76 = HEAP32[$1>>2]|0;
 $77 = HEAP32[$2>>2]|0;
 $78 = (($77) + 60|0);
 $79 = HEAP32[$78>>2]|0;
 $80 = HEAP32[$3>>2]|0;
 $81 = (_sqlite3ExprDup($76,$79,$80)|0);
 $82 = HEAP32[$pNew>>2]|0;
 $83 = (($82) + 60|0);
 HEAP32[$83>>2] = $81;
 $84 = HEAP32[$1>>2]|0;
 $85 = HEAP32[$2>>2]|0;
 $86 = (($85) + 64|0);
 $87 = HEAP32[$86>>2]|0;
 $88 = HEAP32[$3>>2]|0;
 $89 = (_sqlite3ExprDup($84,$87,$88)|0);
 $90 = HEAP32[$pNew>>2]|0;
 $91 = (($90) + 64|0);
 HEAP32[$91>>2] = $89;
 $92 = HEAP32[$pNew>>2]|0;
 $93 = (($92) + 8|0);
 HEAP32[$93>>2] = 0;
 $94 = HEAP32[$pNew>>2]|0;
 $95 = (($94) + 12|0);
 HEAP32[$95>>2] = 0;
 $96 = HEAP32[$2>>2]|0;
 $97 = (($96) + 6|0);
 $98 = HEAP16[$97>>1]|0;
 $99 = $98&65535;
 $100 = $99 & -9;
 $101 = $100&65535;
 $102 = HEAP32[$pNew>>2]|0;
 $103 = (($102) + 6|0);
 HEAP16[$103>>1] = $101;
 $104 = HEAP32[$pNew>>2]|0;
 $105 = (($104) + 16|0);
 HEAP32[$105>>2] = -1;
 $106 = HEAP32[$pNew>>2]|0;
 $107 = (($106) + 16|0);
 $108 = (($107) + 4|0);
 HEAP32[$108>>2] = -1;
 $109 = HEAP32[$2>>2]|0;
 $110 = (($109) + 24|0);
 $111 = $110;
 $112 = $111;
 $113 = HEAP32[$112>>2]|0;
 $114 = (($111) + 4)|0;
 $115 = $114;
 $116 = HEAP32[$115>>2]|0;
 $117 = HEAP32[$pNew>>2]|0;
 $118 = (($117) + 24|0);
 $119 = $118;
 $120 = $119;
 HEAP32[$120>>2] = $113;
 $121 = (($119) + 4)|0;
 $122 = $121;
 HEAP32[$122>>2] = $116;
 $123 = HEAP32[$1>>2]|0;
 $124 = HEAP32[$2>>2]|0;
 $125 = (($124) + 68|0);
 $126 = HEAP32[$125>>2]|0;
 $127 = (_withDup($123,$126)|0);
 $128 = HEAP32[$pNew>>2]|0;
 $129 = (($128) + 68|0);
 HEAP32[$129>>2] = $127;
 $130 = HEAP32[$pNew>>2]|0;
 HEAP32[$0>>2] = $130;
 $131 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($131|0);
}
function _sqlite3SrcListAssignCursors($pParse,$pList) {
 $pParse = $pParse|0;
 $pList = $pList|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $i = 0, $pItem = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $1 = sp + 8|0;
 $i = sp + 4|0;
 $pItem = sp;
 HEAP32[$0>>2] = $pParse;
 HEAP32[$1>>2] = $pList;
 $2 = HEAP32[$1>>2]|0;
 $3 = ($2|0)!=(0|0);
 if (!($3)) {
  STACKTOP = sp;return;
 }
 HEAP32[$i>>2] = 0;
 $4 = HEAP32[$1>>2]|0;
 $5 = (($4) + 8|0);
 HEAP32[$pItem>>2] = $5;
 while(1) {
  $6 = HEAP32[$i>>2]|0;
  $7 = HEAP32[$1>>2]|0;
  $8 = HEAP32[$7>>2]|0;
  $9 = ($6|0)<($8|0);
  if (!($9)) {
   label = 8;
   break;
  }
  $10 = HEAP32[$pItem>>2]|0;
  $11 = (($10) + 40|0);
  $12 = HEAP32[$11>>2]|0;
  $13 = ($12|0)>=(0);
  if ($13) {
   label = 8;
   break;
  }
  $14 = HEAP32[$0>>2]|0;
  $15 = (($14) + 68|0);
  $16 = HEAP32[$15>>2]|0;
  $17 = (($16) + 1)|0;
  HEAP32[$15>>2] = $17;
  $18 = HEAP32[$pItem>>2]|0;
  $19 = (($18) + 40|0);
  HEAP32[$19>>2] = $16;
  $20 = HEAP32[$pItem>>2]|0;
  $21 = (($20) + 20|0);
  $22 = HEAP32[$21>>2]|0;
  $23 = ($22|0)!=(0|0);
  if ($23) {
   $24 = HEAP32[$0>>2]|0;
   $25 = HEAP32[$pItem>>2]|0;
   $26 = (($25) + 20|0);
   $27 = HEAP32[$26>>2]|0;
   $28 = (($27) + 32|0);
   $29 = HEAP32[$28>>2]|0;
   _sqlite3SrcListAssignCursors($24,$29);
  }
  $30 = HEAP32[$i>>2]|0;
  $31 = (($30) + 1)|0;
  HEAP32[$i>>2] = $31;
  $32 = HEAP32[$pItem>>2]|0;
  $33 = (($32) + 72|0);
  HEAP32[$pItem>>2] = $33;
 }
 if ((label|0) == 8) {
  STACKTOP = sp;return;
 }
}
function _sqlite3ResultSetOfSelect($pParse,$pSelect) {
 $pParse = $pParse|0;
 $pSelect = $pSelect|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $7 = 0, $8 = 0;
 var $9 = 0, $db = 0, $pTab = 0, $savedFlags = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 20|0;
 $1 = sp + 16|0;
 $2 = sp + 12|0;
 $pTab = sp + 8|0;
 $db = sp + 4|0;
 $savedFlags = sp;
 HEAP32[$1>>2] = $pParse;
 HEAP32[$2>>2] = $pSelect;
 $3 = HEAP32[$1>>2]|0;
 $4 = HEAP32[$3>>2]|0;
 HEAP32[$db>>2] = $4;
 $5 = HEAP32[$db>>2]|0;
 $6 = (($5) + 24|0);
 $7 = HEAP32[$6>>2]|0;
 HEAP32[$savedFlags>>2] = $7;
 $8 = HEAP32[$db>>2]|0;
 $9 = (($8) + 24|0);
 $10 = HEAP32[$9>>2]|0;
 $11 = $10 & -33;
 HEAP32[$9>>2] = $11;
 $12 = HEAP32[$db>>2]|0;
 $13 = (($12) + 24|0);
 $14 = HEAP32[$13>>2]|0;
 $15 = $14 | 64;
 HEAP32[$13>>2] = $15;
 $16 = HEAP32[$1>>2]|0;
 $17 = HEAP32[$2>>2]|0;
 _sqlite3SelectPrep($16,$17,0);
 $18 = HEAP32[$1>>2]|0;
 $19 = (($18) + 64|0);
 $20 = HEAP32[$19>>2]|0;
 $21 = ($20|0)!=(0);
 if ($21) {
  HEAP32[$0>>2] = 0;
  $60 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($60|0);
 }
 while(1) {
  $22 = HEAP32[$2>>2]|0;
  $23 = (($22) + 52|0);
  $24 = HEAP32[$23>>2]|0;
  $25 = ($24|0)!=(0|0);
  if (!($25)) {
   break;
  }
  $26 = HEAP32[$2>>2]|0;
  $27 = (($26) + 52|0);
  $28 = HEAP32[$27>>2]|0;
  HEAP32[$2>>2] = $28;
 }
 $29 = HEAP32[$savedFlags>>2]|0;
 $30 = HEAP32[$db>>2]|0;
 $31 = (($30) + 24|0);
 HEAP32[$31>>2] = $29;
 $32 = HEAP32[$db>>2]|0;
 $33 = (_sqlite3DbMallocZero($32,76,0)|0);
 HEAP32[$pTab>>2] = $33;
 $34 = HEAP32[$pTab>>2]|0;
 $35 = ($34|0)==(0|0);
 if ($35) {
  HEAP32[$0>>2] = 0;
  $60 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($60|0);
 }
 $36 = HEAP32[$pTab>>2]|0;
 $37 = (($36) + 40|0);
 HEAP16[$37>>1] = 1;
 $38 = HEAP32[$pTab>>2]|0;
 HEAP32[$38>>2] = 0;
 $39 = HEAP32[$pTab>>2]|0;
 $40 = (($39) + 28|0);
 HEAP16[$40>>1] = 200;
 $41 = HEAP32[$1>>2]|0;
 $42 = HEAP32[$2>>2]|0;
 $43 = HEAP32[$42>>2]|0;
 $44 = HEAP32[$pTab>>2]|0;
 $45 = (($44) + 38|0);
 $46 = HEAP32[$pTab>>2]|0;
 $47 = (($46) + 4|0);
 (_selectColumnsFromExprList($41,$43,$45,$47)|0);
 $48 = HEAP32[$1>>2]|0;
 $49 = HEAP32[$pTab>>2]|0;
 $50 = HEAP32[$2>>2]|0;
 _selectAddColumnTypeAndCollation($48,$49,$50);
 $51 = HEAP32[$pTab>>2]|0;
 $52 = (($51) + 36|0);
 HEAP16[$52>>1] = -1;
 $53 = HEAP32[$db>>2]|0;
 $54 = (($53) + 64|0);
 $55 = HEAP8[$54>>0]|0;
 $56 = ($55<<24>>24)!=(0);
 if ($56) {
  $57 = HEAP32[$db>>2]|0;
  $58 = HEAP32[$pTab>>2]|0;
  _sqlite3DeleteTable($57,$58);
  HEAP32[$0>>2] = 0;
  $60 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($60|0);
 } else {
  $59 = HEAP32[$pTab>>2]|0;
  HEAP32[$0>>2] = $59;
  $60 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($60|0);
 }
 return 0|0;
}
function _sqlite3SelectPrep($pParse,$p,$pOuterNC) {
 $pParse = $pParse|0;
 $p = $p|0;
 $pOuterNC = $pOuterNC|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $5 = 0, $6 = 0, $7 = 0;
 var $8 = 0, $9 = 0, $db = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $1 = sp + 8|0;
 $2 = sp + 4|0;
 $db = sp;
 HEAP32[$0>>2] = $pParse;
 HEAP32[$1>>2] = $p;
 HEAP32[$2>>2] = $pOuterNC;
 $3 = HEAP32[$1>>2]|0;
 $4 = ($3|0)==(0|0);
 if ($4) {
  STACKTOP = sp;return;
 }
 $5 = HEAP32[$0>>2]|0;
 $6 = HEAP32[$5>>2]|0;
 HEAP32[$db>>2] = $6;
 $7 = HEAP32[$db>>2]|0;
 $8 = (($7) + 64|0);
 $9 = HEAP8[$8>>0]|0;
 $10 = ($9<<24>>24)!=(0);
 if ($10) {
  STACKTOP = sp;return;
 }
 $11 = HEAP32[$1>>2]|0;
 $12 = (($11) + 6|0);
 $13 = HEAP16[$12>>1]|0;
 $14 = $13&65535;
 $15 = $14 & 32;
 $16 = ($15|0)!=(0);
 if ($16) {
  STACKTOP = sp;return;
 }
 $17 = HEAP32[$0>>2]|0;
 $18 = HEAP32[$1>>2]|0;
 _sqlite3SelectExpand($17,$18);
 $19 = HEAP32[$0>>2]|0;
 $20 = (($19) + 64|0);
 $21 = HEAP32[$20>>2]|0;
 $22 = ($21|0)!=(0);
 if ($22) {
  STACKTOP = sp;return;
 }
 $23 = HEAP32[$db>>2]|0;
 $24 = (($23) + 64|0);
 $25 = HEAP8[$24>>0]|0;
 $26 = $25&255;
 $27 = ($26|0)!=(0);
 if ($27) {
  STACKTOP = sp;return;
 }
 $28 = HEAP32[$0>>2]|0;
 $29 = HEAP32[$1>>2]|0;
 $30 = HEAP32[$2>>2]|0;
 _sqlite3ResolveSelectNames($28,$29,$30);
 $31 = HEAP32[$0>>2]|0;
 $32 = (($31) + 64|0);
 $33 = HEAP32[$32>>2]|0;
 $34 = ($33|0)!=(0);
 if ($34) {
  STACKTOP = sp;return;
 }
 $35 = HEAP32[$db>>2]|0;
 $36 = (($35) + 64|0);
 $37 = HEAP8[$36>>0]|0;
 $38 = $37&255;
 $39 = ($38|0)!=(0);
 if ($39) {
  STACKTOP = sp;return;
 }
 $40 = HEAP32[$0>>2]|0;
 $41 = HEAP32[$1>>2]|0;
 _sqlite3SelectAddTypeInfo($40,$41);
 STACKTOP = sp;return;
}
function _selectColumnsFromExprList($pParse,$pEList,$pnCol,$paCol) {
 $pParse = $pParse|0;
 $pEList = $pEList|0;
 $pnCol = $pnCol|0;
 $paCol = $paCol|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0;
 var $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0;
 var $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0;
 var $aCol = 0, $cnt = 0, $db = 0, $i = 0, $iCol = 0, $j = 0, $k = 0, $nCol = 0, $nName = 0, $p = 0, $pCol = 0, $pColExpr = 0, $pTab = 0, $vararg_buffer7 = 0, $vararg_ptr10 = 0, $zName = 0, $zNewName = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 96|0;
 $vararg_buffer7 = sp;
 $0 = sp + 84|0;
 $1 = sp + 80|0;
 $2 = sp + 76|0;
 $3 = sp + 72|0;
 $4 = sp + 68|0;
 $db = sp + 64|0;
 $i = sp + 60|0;
 $j = sp + 56|0;
 $cnt = sp + 52|0;
 $aCol = sp + 48|0;
 $pCol = sp + 44|0;
 $nCol = sp + 40|0;
 $p = sp + 36|0;
 $zName = sp + 32|0;
 $nName = sp + 28|0;
 $pColExpr = sp + 24|0;
 $pTab = sp + 20|0;
 $iCol = sp + 16|0;
 $zNewName = sp + 12|0;
 $k = sp + 8|0;
 HEAP32[$1>>2] = $pParse;
 HEAP32[$2>>2] = $pEList;
 HEAP32[$3>>2] = $pnCol;
 HEAP32[$4>>2] = $paCol;
 $5 = HEAP32[$1>>2]|0;
 $6 = HEAP32[$5>>2]|0;
 HEAP32[$db>>2] = $6;
 $7 = HEAP32[$2>>2]|0;
 $8 = ($7|0)!=(0|0);
 if ($8) {
  $9 = HEAP32[$2>>2]|0;
  $10 = HEAP32[$9>>2]|0;
  HEAP32[$nCol>>2] = $10;
  $11 = HEAP32[$db>>2]|0;
  $12 = HEAP32[$nCol>>2]|0;
  $13 = ($12*24)|0;
  $14 = (_sqlite3DbMallocZero($11,$13,0)|0);
  HEAP32[$aCol>>2] = $14;
 } else {
  HEAP32[$nCol>>2] = 0;
  HEAP32[$aCol>>2] = 0;
 }
 $15 = HEAP32[$nCol>>2]|0;
 $16 = $15&65535;
 $17 = HEAP32[$3>>2]|0;
 HEAP16[$17>>1] = $16;
 $18 = HEAP32[$aCol>>2]|0;
 $19 = HEAP32[$4>>2]|0;
 HEAP32[$19>>2] = $18;
 HEAP32[$i>>2] = 0;
 $20 = HEAP32[$aCol>>2]|0;
 HEAP32[$pCol>>2] = $20;
 while(1) {
  $21 = HEAP32[$i>>2]|0;
  $22 = HEAP32[$nCol>>2]|0;
  $23 = ($21|0)<($22|0);
  if (!($23)) {
   break;
  }
  $24 = HEAP32[$i>>2]|0;
  $25 = HEAP32[$2>>2]|0;
  $26 = (($25) + 4|0);
  $27 = HEAP32[$26>>2]|0;
  $28 = (($27) + (($24*20)|0)|0);
  $29 = HEAP32[$28>>2]|0;
  $30 = (_sqlite3ExprSkipCollate($29)|0);
  HEAP32[$p>>2] = $30;
  $31 = HEAP32[$i>>2]|0;
  $32 = HEAP32[$2>>2]|0;
  $33 = (($32) + 4|0);
  $34 = HEAP32[$33>>2]|0;
  $35 = (($34) + (($31*20)|0)|0);
  $36 = (($35) + 4|0);
  $37 = HEAP32[$36>>2]|0;
  HEAP32[$zName>>2] = $37;
  $38 = ($37|0)!=(0|0);
  do {
   if ($38) {
    $39 = HEAP32[$db>>2]|0;
    $40 = HEAP32[$zName>>2]|0;
    $41 = (_sqlite3DbStrDup($39,$40)|0);
    HEAP32[$zName>>2] = $41;
   } else {
    $42 = HEAP32[$p>>2]|0;
    HEAP32[$pColExpr>>2] = $42;
    while(1) {
     $43 = HEAP32[$pColExpr>>2]|0;
     $44 = HEAP8[$43>>0]|0;
     $45 = $44&255;
     $46 = ($45|0)==(122);
     $47 = HEAP32[$pColExpr>>2]|0;
     if (!($46)) {
      break;
     }
     $48 = (($47) + 16|0);
     $49 = HEAP32[$48>>2]|0;
     HEAP32[$pColExpr>>2] = $49;
    }
    $50 = HEAP8[$47>>0]|0;
    $51 = $50&255;
    $52 = ($51|0)==(154);
    if ($52) {
     $53 = HEAP32[$pColExpr>>2]|0;
     $54 = (($53) + 44|0);
     $55 = HEAP32[$54>>2]|0;
     $56 = ($55|0)!=(0|0);
     if ($56) {
      $57 = HEAP32[$pColExpr>>2]|0;
      $58 = (($57) + 32|0);
      $59 = HEAP16[$58>>1]|0;
      $60 = $59 << 16 >> 16;
      HEAP32[$iCol>>2] = $60;
      $61 = HEAP32[$pColExpr>>2]|0;
      $62 = (($61) + 44|0);
      $63 = HEAP32[$62>>2]|0;
      HEAP32[$pTab>>2] = $63;
      $64 = HEAP32[$iCol>>2]|0;
      $65 = ($64|0)<(0);
      if ($65) {
       $66 = HEAP32[$pTab>>2]|0;
       $67 = (($66) + 36|0);
       $68 = HEAP16[$67>>1]|0;
       $69 = $68 << 16 >> 16;
       HEAP32[$iCol>>2] = $69;
      }
      $70 = HEAP32[$db>>2]|0;
      $71 = HEAP32[$iCol>>2]|0;
      $72 = ($71|0)>=(0);
      if ($72) {
       $73 = HEAP32[$iCol>>2]|0;
       $74 = HEAP32[$pTab>>2]|0;
       $75 = (($74) + 4|0);
       $76 = HEAP32[$75>>2]|0;
       $77 = (($76) + (($73*24)|0)|0);
       $78 = HEAP32[$77>>2]|0;
       $79 = $78;
      } else {
       $79 = 9992;
      }
      HEAP32[$vararg_buffer7>>2] = $79;
      $80 = (_sqlite3MPrintf($70,1200,$vararg_buffer7)|0);
      HEAP32[$zName>>2] = $80;
      break;
     }
    }
    $81 = HEAP32[$pColExpr>>2]|0;
    $82 = HEAP8[$81>>0]|0;
    $83 = $82&255;
    $84 = ($83|0)==(27);
    $85 = HEAP32[$db>>2]|0;
    if ($84) {
     $86 = HEAP32[$pColExpr>>2]|0;
     $87 = (($86) + 8|0);
     $88 = HEAP32[$87>>2]|0;
     HEAP32[$vararg_buffer7>>2] = $88;
     $89 = (_sqlite3MPrintf($85,1200,$vararg_buffer7)|0);
     HEAP32[$zName>>2] = $89;
     break;
    } else {
     $90 = HEAP32[$i>>2]|0;
     $91 = HEAP32[$2>>2]|0;
     $92 = (($91) + 4|0);
     $93 = HEAP32[$92>>2]|0;
     $94 = (($93) + (($90*20)|0)|0);
     $95 = (($94) + 8|0);
     $96 = HEAP32[$95>>2]|0;
     HEAP32[$vararg_buffer7>>2] = $96;
     $97 = (_sqlite3MPrintf($85,1200,$vararg_buffer7)|0);
     HEAP32[$zName>>2] = $97;
     break;
    }
   }
  } while(0);
  $98 = HEAP32[$db>>2]|0;
  $99 = (($98) + 64|0);
  $100 = HEAP8[$99>>0]|0;
  $101 = ($100<<24>>24)!=(0);
  if ($101) {
   label = 22;
   break;
  }
  $104 = HEAP32[$zName>>2]|0;
  $105 = (_sqlite3Strlen30($104)|0);
  HEAP32[$nName>>2] = $105;
  HEAP32[$cnt>>2] = 0;
  HEAP32[$j>>2] = 0;
  while(1) {
   $106 = HEAP32[$j>>2]|0;
   $107 = HEAP32[$i>>2]|0;
   $108 = ($106|0)<($107|0);
   if (!($108)) {
    break;
   }
   $109 = HEAP32[$j>>2]|0;
   $110 = HEAP32[$aCol>>2]|0;
   $111 = (($110) + (($109*24)|0)|0);
   $112 = HEAP32[$111>>2]|0;
   $113 = HEAP32[$zName>>2]|0;
   $114 = (_sqlite3_stricmp($112,$113)|0);
   $115 = ($114|0)==(0);
   if ($115) {
    $116 = HEAP32[$nName>>2]|0;
    $117 = (($116) - 1)|0;
    HEAP32[$k>>2] = $117;
    while(1) {
     $118 = HEAP32[$k>>2]|0;
     $119 = ($118|0)>(1);
     if (!($119)) {
      break;
     }
     $120 = HEAP32[$k>>2]|0;
     $121 = HEAP32[$zName>>2]|0;
     $122 = (($121) + ($120)|0);
     $123 = HEAP8[$122>>0]|0;
     $124 = $123&255;
     $125 = (1224 + ($124)|0);
     $126 = HEAP8[$125>>0]|0;
     $127 = $126&255;
     $128 = $127 & 4;
     $129 = ($128|0)!=(0);
     if (!($129)) {
      break;
     }
     $130 = HEAP32[$k>>2]|0;
     $131 = (($130) + -1)|0;
     HEAP32[$k>>2] = $131;
    }
    $132 = HEAP32[$k>>2]|0;
    $133 = ($132|0)>=(0);
    if ($133) {
     $134 = HEAP32[$k>>2]|0;
     $135 = HEAP32[$zName>>2]|0;
     $136 = (($135) + ($134)|0);
     $137 = HEAP8[$136>>0]|0;
     $138 = $137 << 24 >> 24;
     $139 = ($138|0)==(58);
     if ($139) {
      $140 = HEAP32[$k>>2]|0;
      HEAP32[$nName>>2] = $140;
     }
    }
    $141 = HEAP32[$nName>>2]|0;
    $142 = HEAP32[$zName>>2]|0;
    $143 = (($142) + ($141)|0);
    HEAP8[$143>>0] = 0;
    $144 = HEAP32[$db>>2]|0;
    $145 = HEAP32[$zName>>2]|0;
    $146 = HEAP32[$cnt>>2]|0;
    $147 = (($146) + 1)|0;
    HEAP32[$cnt>>2] = $147;
    HEAP32[$vararg_buffer7>>2] = $145;
    $vararg_ptr10 = (($vararg_buffer7) + 4|0);
    HEAP32[$vararg_ptr10>>2] = $147;
    $148 = (_sqlite3MPrintf($144,16424,$vararg_buffer7)|0);
    HEAP32[$zNewName>>2] = $148;
    $149 = HEAP32[$db>>2]|0;
    $150 = HEAP32[$zName>>2]|0;
    _sqlite3DbFree($149,$150);
    $151 = HEAP32[$zNewName>>2]|0;
    HEAP32[$zName>>2] = $151;
    HEAP32[$j>>2] = -1;
    $152 = HEAP32[$zName>>2]|0;
    $153 = ($152|0)==(0|0);
    if ($153) {
     break;
    }
   }
   $154 = HEAP32[$j>>2]|0;
   $155 = (($154) + 1)|0;
   HEAP32[$j>>2] = $155;
  }
  $156 = HEAP32[$zName>>2]|0;
  $157 = HEAP32[$pCol>>2]|0;
  HEAP32[$157>>2] = $156;
  $158 = HEAP32[$i>>2]|0;
  $159 = (($158) + 1)|0;
  HEAP32[$i>>2] = $159;
  $160 = HEAP32[$pCol>>2]|0;
  $161 = (($160) + 24|0);
  HEAP32[$pCol>>2] = $161;
 }
 if ((label|0) == 22) {
  $102 = HEAP32[$db>>2]|0;
  $103 = HEAP32[$zName>>2]|0;
  _sqlite3DbFree($102,$103);
 }
 $162 = HEAP32[$db>>2]|0;
 $163 = (($162) + 64|0);
 $164 = HEAP8[$163>>0]|0;
 $165 = ($164<<24>>24)!=(0);
 if (!($165)) {
  HEAP32[$0>>2] = 0;
  $179 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($179|0);
 }
 HEAP32[$j>>2] = 0;
 while(1) {
  $166 = HEAP32[$j>>2]|0;
  $167 = HEAP32[$i>>2]|0;
  $168 = ($166|0)<($167|0);
  $169 = HEAP32[$db>>2]|0;
  if (!($168)) {
   break;
  }
  $170 = HEAP32[$j>>2]|0;
  $171 = HEAP32[$aCol>>2]|0;
  $172 = (($171) + (($170*24)|0)|0);
  $173 = HEAP32[$172>>2]|0;
  _sqlite3DbFree($169,$173);
  $174 = HEAP32[$j>>2]|0;
  $175 = (($174) + 1)|0;
  HEAP32[$j>>2] = $175;
 }
 $176 = HEAP32[$aCol>>2]|0;
 _sqlite3DbFree($169,$176);
 $177 = HEAP32[$4>>2]|0;
 HEAP32[$177>>2] = 0;
 $178 = HEAP32[$3>>2]|0;
 HEAP16[$178>>1] = 0;
 HEAP32[$0>>2] = 7;
 $179 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($179|0);
}
function _selectAddColumnTypeAndCollation($pParse,$pTab,$pSelect) {
 $pParse = $pParse|0;
 $pTab = $pTab|0;
 $pSelect = $pSelect|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0;
 var $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $a = 0, $db = 0, $i = 0, $p = 0, $pCol = 0;
 var $pColl = 0, $sNC = 0, $szAll = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 80|0;
 $0 = sp + 76|0;
 $1 = sp + 72|0;
 $2 = sp + 68|0;
 $db = sp + 64|0;
 $sNC = sp + 32|0;
 $pCol = sp + 24|0;
 $pColl = sp + 20|0;
 $i = sp + 16|0;
 $p = sp + 12|0;
 $a = sp + 8|0;
 $szAll = sp;
 HEAP32[$0>>2] = $pParse;
 HEAP32[$1>>2] = $pTab;
 HEAP32[$2>>2] = $pSelect;
 $3 = HEAP32[$0>>2]|0;
 $4 = HEAP32[$3>>2]|0;
 HEAP32[$db>>2] = $4;
 $5 = $szAll;
 $6 = $5;
 HEAP32[$6>>2] = 0;
 $7 = (($5) + 4)|0;
 $8 = $7;
 HEAP32[$8>>2] = 0;
 $9 = HEAP32[$db>>2]|0;
 $10 = (($9) + 64|0);
 $11 = HEAP8[$10>>0]|0;
 $12 = ($11<<24>>24)!=(0);
 if ($12) {
  STACKTOP = sp;return;
 }
 ;HEAP32[$sNC+0>>2]=0|0;HEAP32[$sNC+4>>2]=0|0;HEAP32[$sNC+8>>2]=0|0;HEAP32[$sNC+12>>2]=0|0;HEAP32[$sNC+16>>2]=0|0;HEAP32[$sNC+20>>2]=0|0;HEAP32[$sNC+24>>2]=0|0;HEAP32[$sNC+28>>2]=0|0;
 $13 = HEAP32[$2>>2]|0;
 $14 = (($13) + 32|0);
 $15 = HEAP32[$14>>2]|0;
 $16 = (($sNC) + 4|0);
 HEAP32[$16>>2] = $15;
 $17 = HEAP32[$2>>2]|0;
 $18 = HEAP32[$17>>2]|0;
 $19 = (($18) + 4|0);
 $20 = HEAP32[$19>>2]|0;
 HEAP32[$a>>2] = $20;
 HEAP32[$i>>2] = 0;
 $21 = HEAP32[$1>>2]|0;
 $22 = (($21) + 4|0);
 $23 = HEAP32[$22>>2]|0;
 HEAP32[$pCol>>2] = $23;
 while(1) {
  $24 = HEAP32[$i>>2]|0;
  $25 = HEAP32[$1>>2]|0;
  $26 = (($25) + 38|0);
  $27 = HEAP16[$26>>1]|0;
  $28 = $27 << 16 >> 16;
  $29 = ($24|0)<($28|0);
  if (!($29)) {
   break;
  }
  $30 = HEAP32[$i>>2]|0;
  $31 = HEAP32[$a>>2]|0;
  $32 = (($31) + (($30*20)|0)|0);
  $33 = HEAP32[$32>>2]|0;
  HEAP32[$p>>2] = $33;
  $34 = HEAP32[$db>>2]|0;
  $35 = HEAP32[$p>>2]|0;
  $36 = HEAP32[$pCol>>2]|0;
  $37 = (($36) + 22|0);
  $38 = (_columnTypeImpl($sNC,$35,$37)|0);
  $39 = (_sqlite3DbStrDup($34,$38)|0);
  $40 = HEAP32[$pCol>>2]|0;
  $41 = (($40) + 12|0);
  HEAP32[$41>>2] = $39;
  $42 = HEAP32[$pCol>>2]|0;
  $43 = (($42) + 22|0);
  $44 = HEAP8[$43>>0]|0;
  $45 = $44&255;
  $46 = $szAll;
  $47 = $46;
  $48 = HEAP32[$47>>2]|0;
  $49 = (($46) + 4)|0;
  $50 = $49;
  $51 = HEAP32[$50>>2]|0;
  $52 = (_i64Add(($48|0),($51|0),($45|0),0)|0);
  $53 = tempRet0;
  $54 = $szAll;
  $55 = $54;
  HEAP32[$55>>2] = $52;
  $56 = (($54) + 4)|0;
  $57 = $56;
  HEAP32[$57>>2] = $53;
  $58 = HEAP32[$p>>2]|0;
  $59 = (_sqlite3ExprAffinity($58)|0);
  $60 = HEAP32[$pCol>>2]|0;
  $61 = (($60) + 21|0);
  HEAP8[$61>>0] = $59;
  $62 = HEAP32[$pCol>>2]|0;
  $63 = (($62) + 21|0);
  $64 = HEAP8[$63>>0]|0;
  $65 = $64 << 24 >> 24;
  $66 = ($65|0)==(0);
  if ($66) {
   $67 = HEAP32[$pCol>>2]|0;
   $68 = (($67) + 21|0);
   HEAP8[$68>>0] = 65;
  }
  $69 = HEAP32[$0>>2]|0;
  $70 = HEAP32[$p>>2]|0;
  $71 = (_sqlite3ExprCollSeq($69,$70)|0);
  HEAP32[$pColl>>2] = $71;
  $72 = HEAP32[$pColl>>2]|0;
  $73 = ($72|0)!=(0|0);
  if ($73) {
   $74 = HEAP32[$db>>2]|0;
   $75 = HEAP32[$pColl>>2]|0;
   $76 = HEAP32[$75>>2]|0;
   $77 = (_sqlite3DbStrDup($74,$76)|0);
   $78 = HEAP32[$pCol>>2]|0;
   $79 = (($78) + 16|0);
   HEAP32[$79>>2] = $77;
  }
  $80 = HEAP32[$i>>2]|0;
  $81 = (($80) + 1)|0;
  HEAP32[$i>>2] = $81;
  $82 = HEAP32[$pCol>>2]|0;
  $83 = (($82) + 24|0);
  HEAP32[$pCol>>2] = $83;
 }
 $84 = $szAll;
 $85 = $84;
 $86 = HEAP32[$85>>2]|0;
 $87 = (($84) + 4)|0;
 $88 = $87;
 $89 = HEAP32[$88>>2]|0;
 $90 = (___muldi3(($86|0),($89|0),4,0)|0);
 $91 = tempRet0;
 $92 = (_sqlite3LogEst($90,$91)|0);
 $93 = HEAP32[$1>>2]|0;
 $94 = (($93) + 42|0);
 HEAP16[$94>>1] = $92;
 STACKTOP = sp;return;
}
function _columnTypeImpl($pNC,$pExpr,$pEstWidth) {
 $pNC = $pNC|0;
 $pExpr = $pExpr|0;
 $pEstWidth = $pEstWidth|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0;
 var $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0;
 var $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0;
 var $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0;
 var $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $estWidth = 0, $iCol = 0;
 var $j = 0, $p = 0, $p3 = 0, $pS = 0, $pS2 = 0, $pTab = 0, $pTabList = 0, $sNC = 0, $sNC1 = 0, $zType = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 128|0;
 $0 = sp + 116|0;
 $1 = sp + 112|0;
 $2 = sp + 108|0;
 $3 = sp + 104|0;
 $zType = sp + 100|0;
 $j = sp + 96|0;
 $estWidth = sp + 120|0;
 $pTab = sp + 92|0;
 $pS = sp + 88|0;
 $iCol = sp + 84|0;
 $pTabList = sp + 80|0;
 $sNC = sp + 48|0;
 $p = sp + 40|0;
 $sNC1 = sp + 8|0;
 $pS2 = sp + 4|0;
 $p3 = sp;
 HEAP32[$1>>2] = $pNC;
 HEAP32[$2>>2] = $pExpr;
 HEAP32[$3>>2] = $pEstWidth;
 HEAP32[$zType>>2] = 0;
 HEAP8[$estWidth>>0] = 1;
 $4 = HEAP32[$2>>2]|0;
 $5 = ($4|0)==(0|0);
 if (!($5)) {
  $6 = HEAP32[$1>>2]|0;
  $7 = (($6) + 4|0);
  $8 = HEAP32[$7>>2]|0;
  $9 = ($8|0)==(0|0);
  if (!($9)) {
   $10 = HEAP32[$2>>2]|0;
   $11 = HEAP8[$10>>0]|0;
   $12 = $11&255;
   do {
    if ((($12|0) == 154) | (($12|0) == 156)) {
     HEAP32[$pTab>>2] = 0;
     HEAP32[$pS>>2] = 0;
     $13 = HEAP32[$2>>2]|0;
     $14 = (($13) + 32|0);
     $15 = HEAP16[$14>>1]|0;
     $16 = $15 << 16 >> 16;
     HEAP32[$iCol>>2] = $16;
     while(1) {
      $17 = HEAP32[$1>>2]|0;
      $18 = ($17|0)!=(0|0);
      if (!($18)) {
       break;
      }
      $19 = HEAP32[$pTab>>2]|0;
      $20 = ($19|0)!=(0|0);
      $21 = $20 ^ 1;
      if (!($21)) {
       break;
      }
      $22 = HEAP32[$1>>2]|0;
      $23 = (($22) + 4|0);
      $24 = HEAP32[$23>>2]|0;
      HEAP32[$pTabList>>2] = $24;
      HEAP32[$j>>2] = 0;
      while(1) {
       $25 = HEAP32[$j>>2]|0;
       $26 = HEAP32[$pTabList>>2]|0;
       $27 = HEAP32[$26>>2]|0;
       $28 = ($25|0)<($27|0);
       if (!($28)) {
        break;
       }
       $29 = HEAP32[$j>>2]|0;
       $30 = HEAP32[$pTabList>>2]|0;
       $31 = (($30) + 8|0);
       $32 = (($31) + (($29*72)|0)|0);
       $33 = (($32) + 40|0);
       $34 = HEAP32[$33>>2]|0;
       $35 = HEAP32[$2>>2]|0;
       $36 = (($35) + 28|0);
       $37 = HEAP32[$36>>2]|0;
       $38 = ($34|0)!=($37|0);
       if (!($38)) {
        break;
       }
       $39 = HEAP32[$j>>2]|0;
       $40 = (($39) + 1)|0;
       HEAP32[$j>>2] = $40;
      }
      $41 = HEAP32[$j>>2]|0;
      $42 = HEAP32[$pTabList>>2]|0;
      $43 = HEAP32[$42>>2]|0;
      $44 = ($41|0)<($43|0);
      if ($44) {
       $45 = HEAP32[$j>>2]|0;
       $46 = HEAP32[$pTabList>>2]|0;
       $47 = (($46) + 8|0);
       $48 = (($47) + (($45*72)|0)|0);
       $49 = (($48) + 16|0);
       $50 = HEAP32[$49>>2]|0;
       HEAP32[$pTab>>2] = $50;
       $51 = HEAP32[$j>>2]|0;
       $52 = HEAP32[$pTabList>>2]|0;
       $53 = (($52) + 8|0);
       $54 = (($53) + (($51*72)|0)|0);
       $55 = (($54) + 20|0);
       $56 = HEAP32[$55>>2]|0;
       HEAP32[$pS>>2] = $56;
       continue;
      } else {
       $57 = HEAP32[$1>>2]|0;
       $58 = (($57) + 16|0);
       $59 = HEAP32[$58>>2]|0;
       HEAP32[$1>>2] = $59;
       continue;
      }
     }
     $60 = HEAP32[$pTab>>2]|0;
     $61 = ($60|0)==(0|0);
     if (!($61)) {
      $62 = HEAP32[$pS>>2]|0;
      $63 = ($62|0)!=(0|0);
      if ($63) {
       $64 = HEAP32[$iCol>>2]|0;
       $65 = ($64|0)>=(0);
       if (!($65)) {
        break;
       }
       $66 = HEAP32[$iCol>>2]|0;
       $67 = HEAP32[$pS>>2]|0;
       $68 = HEAP32[$67>>2]|0;
       $69 = HEAP32[$68>>2]|0;
       $70 = ($66|0)<($69|0);
       if (!($70)) {
        break;
       }
       $71 = HEAP32[$iCol>>2]|0;
       $72 = HEAP32[$pS>>2]|0;
       $73 = HEAP32[$72>>2]|0;
       $74 = (($73) + 4|0);
       $75 = HEAP32[$74>>2]|0;
       $76 = (($75) + (($71*20)|0)|0);
       $77 = HEAP32[$76>>2]|0;
       HEAP32[$p>>2] = $77;
       $78 = HEAP32[$pS>>2]|0;
       $79 = (($78) + 32|0);
       $80 = HEAP32[$79>>2]|0;
       $81 = (($sNC) + 4|0);
       HEAP32[$81>>2] = $80;
       $82 = HEAP32[$1>>2]|0;
       $83 = (($sNC) + 16|0);
       HEAP32[$83>>2] = $82;
       $84 = HEAP32[$1>>2]|0;
       $85 = HEAP32[$84>>2]|0;
       HEAP32[$sNC>>2] = $85;
       $86 = HEAP32[$p>>2]|0;
       $87 = (_columnTypeImpl($sNC,$86,$estWidth)|0);
       HEAP32[$zType>>2] = $87;
       break;
      }
      $88 = HEAP32[$pTab>>2]|0;
      $89 = (($88) + 68|0);
      $90 = HEAP32[$89>>2]|0;
      $91 = ($90|0)!=(0|0);
      if ($91) {
       $92 = HEAP32[$iCol>>2]|0;
       $93 = ($92|0)<(0);
       if ($93) {
        $94 = HEAP32[$pTab>>2]|0;
        $95 = (($94) + 36|0);
        $96 = HEAP16[$95>>1]|0;
        $97 = $96 << 16 >> 16;
        HEAP32[$iCol>>2] = $97;
       }
       $98 = HEAP32[$iCol>>2]|0;
       $99 = ($98|0)<(0);
       if ($99) {
        HEAP32[$zType>>2] = 16416;
        break;
       } else {
        $100 = HEAP32[$iCol>>2]|0;
        $101 = HEAP32[$pTab>>2]|0;
        $102 = (($101) + 4|0);
        $103 = HEAP32[$102>>2]|0;
        $104 = (($103) + (($100*24)|0)|0);
        $105 = (($104) + 12|0);
        $106 = HEAP32[$105>>2]|0;
        HEAP32[$zType>>2] = $106;
        $107 = HEAP32[$iCol>>2]|0;
        $108 = HEAP32[$pTab>>2]|0;
        $109 = (($108) + 4|0);
        $110 = HEAP32[$109>>2]|0;
        $111 = (($110) + (($107*24)|0)|0);
        $112 = (($111) + 22|0);
        $113 = HEAP8[$112>>0]|0;
        HEAP8[$estWidth>>0] = $113;
        break;
       }
      }
     }
    } else if ((($12|0) == 119)) {
     $114 = HEAP32[$2>>2]|0;
     $115 = (($114) + 20|0);
     $116 = HEAP32[$115>>2]|0;
     HEAP32[$pS2>>2] = $116;
     $117 = HEAP32[$pS2>>2]|0;
     $118 = HEAP32[$117>>2]|0;
     $119 = (($118) + 4|0);
     $120 = HEAP32[$119>>2]|0;
     $121 = HEAP32[$120>>2]|0;
     HEAP32[$p3>>2] = $121;
     $122 = HEAP32[$pS2>>2]|0;
     $123 = (($122) + 32|0);
     $124 = HEAP32[$123>>2]|0;
     $125 = (($sNC1) + 4|0);
     HEAP32[$125>>2] = $124;
     $126 = HEAP32[$1>>2]|0;
     $127 = (($sNC1) + 16|0);
     HEAP32[$127>>2] = $126;
     $128 = HEAP32[$1>>2]|0;
     $129 = HEAP32[$128>>2]|0;
     HEAP32[$sNC1>>2] = $129;
     $130 = HEAP32[$p3>>2]|0;
     $131 = (_columnTypeImpl($sNC1,$130,$estWidth)|0);
     HEAP32[$zType>>2] = $131;
    }
   } while(0);
   $132 = HEAP32[$3>>2]|0;
   $133 = ($132|0)!=(0|0);
   if ($133) {
    $134 = HEAP8[$estWidth>>0]|0;
    $135 = HEAP32[$3>>2]|0;
    HEAP8[$135>>0] = $134;
   }
   $136 = HEAP32[$zType>>2]|0;
   HEAP32[$0>>2] = $136;
   $137 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($137|0);
  }
 }
 HEAP32[$0>>2] = 0;
 $137 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($137|0);
}
function _sqlite3ExprAffinity($pExpr) {
 $pExpr = $pExpr|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $j = 0, $op = 0, $or$cond = 0;
 var $or$cond3 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $1 = sp + 8|0;
 $op = sp + 4|0;
 $j = sp;
 HEAP32[$1>>2] = $pExpr;
 $2 = HEAP32[$1>>2]|0;
 $3 = (_sqlite3ExprSkipCollate($2)|0);
 HEAP32[$1>>2] = $3;
 $4 = HEAP32[$1>>2]|0;
 $5 = (($4) + 4|0);
 $6 = HEAP32[$5>>2]|0;
 $7 = $6 & 512;
 $8 = ($7|0)!=(0);
 if ($8) {
  HEAP8[$0>>0] = 0;
  $56 = HEAP8[$0>>0]|0;
  STACKTOP = sp;return ($56|0);
 }
 $9 = HEAP32[$1>>2]|0;
 $10 = HEAP8[$9>>0]|0;
 $11 = $10&255;
 HEAP32[$op>>2] = $11;
 $12 = HEAP32[$op>>2]|0;
 $13 = ($12|0)==(119);
 if ($13) {
  $14 = HEAP32[$1>>2]|0;
  $15 = (($14) + 20|0);
  $16 = HEAP32[$15>>2]|0;
  $17 = HEAP32[$16>>2]|0;
  $18 = (($17) + 4|0);
  $19 = HEAP32[$18>>2]|0;
  $20 = HEAP32[$19>>2]|0;
  $21 = (_sqlite3ExprAffinity($20)|0);
  HEAP8[$0>>0] = $21;
  $56 = HEAP8[$0>>0]|0;
  STACKTOP = sp;return ($56|0);
 }
 $22 = HEAP32[$op>>2]|0;
 $23 = ($22|0)==(38);
 if ($23) {
  $24 = HEAP32[$1>>2]|0;
  $25 = (($24) + 8|0);
  $26 = HEAP32[$25>>2]|0;
  $27 = (_sqlite3AffinityType($26,0)|0);
  HEAP8[$0>>0] = $27;
  $56 = HEAP8[$0>>0]|0;
  STACKTOP = sp;return ($56|0);
 }
 $28 = HEAP32[$op>>2]|0;
 $29 = ($28|0)==(156);
 $30 = HEAP32[$op>>2]|0;
 $31 = ($30|0)==(154);
 $or$cond = $29 | $31;
 $32 = HEAP32[$op>>2]|0;
 $33 = ($32|0)==(159);
 $or$cond3 = $or$cond | $33;
 if ($or$cond3) {
  $34 = HEAP32[$1>>2]|0;
  $35 = (($34) + 44|0);
  $36 = HEAP32[$35>>2]|0;
  $37 = ($36|0)!=(0|0);
  if ($37) {
   $38 = HEAP32[$1>>2]|0;
   $39 = (($38) + 32|0);
   $40 = HEAP16[$39>>1]|0;
   $41 = $40 << 16 >> 16;
   HEAP32[$j>>2] = $41;
   $42 = HEAP32[$j>>2]|0;
   $43 = ($42|0)<(0);
   if ($43) {
    HEAP8[$0>>0] = 68;
    $56 = HEAP8[$0>>0]|0;
    STACKTOP = sp;return ($56|0);
   } else {
    $44 = HEAP32[$j>>2]|0;
    $45 = HEAP32[$1>>2]|0;
    $46 = (($45) + 44|0);
    $47 = HEAP32[$46>>2]|0;
    $48 = (($47) + 4|0);
    $49 = HEAP32[$48>>2]|0;
    $50 = (($49) + (($44*24)|0)|0);
    $51 = (($50) + 21|0);
    $52 = HEAP8[$51>>0]|0;
    HEAP8[$0>>0] = $52;
    $56 = HEAP8[$0>>0]|0;
    STACKTOP = sp;return ($56|0);
   }
  }
 }
 $53 = HEAP32[$1>>2]|0;
 $54 = (($53) + 1|0);
 $55 = HEAP8[$54>>0]|0;
 HEAP8[$0>>0] = $55;
 $56 = HEAP8[$0>>0]|0;
 STACKTOP = sp;return ($56|0);
}
function _sqlite3ExprCollSeq($pParse,$pExpr) {
 $pParse = $pParse|0;
 $pExpr = $pExpr|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0;
 var $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0;
 var $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0;
 var $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0;
 var $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0;
 var $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $db = 0, $j = 0, $op = 0, $or$cond = 0, $or$cond3 = 0, $or$cond5 = 0, $or$cond7 = 0, $p = 0, $pColl = 0, $zColl = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 28|0;
 $1 = sp + 24|0;
 $db = sp + 20|0;
 $pColl = sp + 16|0;
 $p = sp + 12|0;
 $op = sp + 8|0;
 $j = sp + 4|0;
 $zColl = sp;
 HEAP32[$0>>2] = $pParse;
 HEAP32[$1>>2] = $pExpr;
 $2 = HEAP32[$0>>2]|0;
 $3 = HEAP32[$2>>2]|0;
 HEAP32[$db>>2] = $3;
 HEAP32[$pColl>>2] = 0;
 $4 = HEAP32[$1>>2]|0;
 HEAP32[$p>>2] = $4;
 while(1) {
  $5 = HEAP32[$p>>2]|0;
  $6 = ($5|0)!=(0|0);
  if (!($6)) {
   break;
  }
  $7 = HEAP32[$p>>2]|0;
  $8 = HEAP8[$7>>0]|0;
  $9 = $8&255;
  HEAP32[$op>>2] = $9;
  $10 = HEAP32[$p>>2]|0;
  $11 = (($10) + 4|0);
  $12 = HEAP32[$11>>2]|0;
  $13 = $12 & 512;
  $14 = ($13|0)!=(0);
  if ($14) {
   break;
  }
  $15 = HEAP32[$op>>2]|0;
  $16 = ($15|0)==(38);
  $17 = HEAP32[$op>>2]|0;
  $18 = ($17|0)==(158);
  $or$cond = $16 | $18;
  if ($or$cond) {
   $19 = HEAP32[$p>>2]|0;
   $20 = (($19) + 12|0);
   $21 = HEAP32[$20>>2]|0;
   HEAP32[$p>>2] = $21;
   continue;
  }
  $22 = HEAP32[$op>>2]|0;
  $23 = ($22|0)==(95);
  if ($23) {
   label = 9;
   break;
  }
  $24 = HEAP32[$op>>2]|0;
  $25 = ($24|0)==(159);
  if ($25) {
   $26 = HEAP32[$p>>2]|0;
   $27 = (($26) + 38|0);
   $28 = HEAP8[$27>>0]|0;
   $29 = $28&255;
   $30 = ($29|0)==(95);
   if ($30) {
    label = 9;
    break;
   }
  }
  $43 = HEAP32[$p>>2]|0;
  $44 = (($43) + 44|0);
  $45 = HEAP32[$44>>2]|0;
  $46 = ($45|0)!=(0|0);
  if ($46) {
   $47 = HEAP32[$op>>2]|0;
   $48 = ($47|0)==(156);
   $49 = HEAP32[$op>>2]|0;
   $50 = ($49|0)==(154);
   $or$cond3 = $48 | $50;
   $51 = HEAP32[$op>>2]|0;
   $52 = ($51|0)==(159);
   $or$cond5 = $or$cond3 | $52;
   $53 = HEAP32[$op>>2]|0;
   $54 = ($53|0)==(62);
   $or$cond7 = $or$cond5 | $54;
   if ($or$cond7) {
    label = 12;
    break;
   }
  }
  $80 = HEAP32[$p>>2]|0;
  $81 = (($80) + 4|0);
  $82 = HEAP32[$81>>2]|0;
  $83 = $82 & 256;
  $84 = ($83|0)!=(0);
  if (!($84)) {
   break;
  }
  $85 = HEAP32[$p>>2]|0;
  $86 = (($85) + 12|0);
  $87 = HEAP32[$86>>2]|0;
  $88 = ($87|0)!=(0|0);
  if ($88) {
   $89 = HEAP32[$p>>2]|0;
   $90 = (($89) + 12|0);
   $91 = HEAP32[$90>>2]|0;
   $92 = (($91) + 4|0);
   $93 = HEAP32[$92>>2]|0;
   $94 = $93 & 256;
   $95 = ($94|0)!=(0);
   if ($95) {
    $96 = HEAP32[$p>>2]|0;
    $97 = (($96) + 12|0);
    $98 = HEAP32[$97>>2]|0;
    HEAP32[$p>>2] = $98;
    continue;
   }
  }
  $99 = HEAP32[$p>>2]|0;
  $100 = (($99) + 16|0);
  $101 = HEAP32[$100>>2]|0;
  HEAP32[$p>>2] = $101;
 }
 if ((label|0) == 9) {
  $31 = HEAP32[$0>>2]|0;
  $32 = HEAP32[$db>>2]|0;
  $33 = (($32) + 16|0);
  $34 = HEAP32[$33>>2]|0;
  $35 = (($34) + 12|0);
  $36 = HEAP32[$35>>2]|0;
  $37 = (($36) + 77|0);
  $38 = HEAP8[$37>>0]|0;
  $39 = HEAP32[$p>>2]|0;
  $40 = (($39) + 8|0);
  $41 = HEAP32[$40>>2]|0;
  $42 = (_sqlite3GetCollSeq($31,$38,0,$41)|0);
  HEAP32[$pColl>>2] = $42;
 }
 else if ((label|0) == 12) {
  $55 = HEAP32[$p>>2]|0;
  $56 = (($55) + 32|0);
  $57 = HEAP16[$56>>1]|0;
  $58 = $57 << 16 >> 16;
  HEAP32[$j>>2] = $58;
  $59 = HEAP32[$j>>2]|0;
  $60 = ($59|0)>=(0);
  if ($60) {
   $61 = HEAP32[$j>>2]|0;
   $62 = HEAP32[$p>>2]|0;
   $63 = (($62) + 44|0);
   $64 = HEAP32[$63>>2]|0;
   $65 = (($64) + 4|0);
   $66 = HEAP32[$65>>2]|0;
   $67 = (($66) + (($61*24)|0)|0);
   $68 = (($67) + 16|0);
   $69 = HEAP32[$68>>2]|0;
   HEAP32[$zColl>>2] = $69;
   $70 = HEAP32[$db>>2]|0;
   $71 = HEAP32[$db>>2]|0;
   $72 = (($71) + 16|0);
   $73 = HEAP32[$72>>2]|0;
   $74 = (($73) + 12|0);
   $75 = HEAP32[$74>>2]|0;
   $76 = (($75) + 77|0);
   $77 = HEAP8[$76>>0]|0;
   $78 = HEAP32[$zColl>>2]|0;
   $79 = (_sqlite3FindCollSeq($70,$77,$78,0)|0);
   HEAP32[$pColl>>2] = $79;
  }
 }
 $102 = HEAP32[$0>>2]|0;
 $103 = HEAP32[$pColl>>2]|0;
 $104 = (_sqlite3CheckCollSeq($102,$103)|0);
 $105 = ($104|0)!=(0);
 if (!($105)) {
  $106 = HEAP32[$pColl>>2]|0;
  STACKTOP = sp;return ($106|0);
 }
 HEAP32[$pColl>>2] = 0;
 $106 = HEAP32[$pColl>>2]|0;
 STACKTOP = sp;return ($106|0);
}
function _sqlite3LogEst($0,$1) {
 $0 = $0|0;
 $1 = $1|0;
 var $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0, $116 = 0, $117 = 0;
 var $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0;
 var $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0;
 var $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0;
 var $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0;
 var $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0;
 var $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $y = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $2 = sp + 10|0;
 $3 = sp;
 $y = sp + 8|0;
 $4 = $3;
 $5 = $4;
 HEAP32[$5>>2] = $0;
 $6 = (($4) + 4)|0;
 $7 = $6;
 HEAP32[$7>>2] = $1;
 HEAP16[$y>>1] = 40;
 $8 = $3;
 $9 = $8;
 $10 = HEAP32[$9>>2]|0;
 $11 = (($8) + 4)|0;
 $12 = $11;
 $13 = HEAP32[$12>>2]|0;
 $14 = ($13>>>0)<(0);
 $15 = ($10>>>0)<(8);
 $16 = ($13|0)==(0);
 $17 = $16 & $15;
 $18 = $14 | $17;
 L1: do {
  if ($18) {
   $19 = $3;
   $20 = $19;
   $21 = HEAP32[$20>>2]|0;
   $22 = (($19) + 4)|0;
   $23 = $22;
   $24 = HEAP32[$23>>2]|0;
   $25 = ($24>>>0)<(0);
   $26 = ($21>>>0)<(2);
   $27 = ($24|0)==(0);
   $28 = $27 & $26;
   $29 = $25 | $28;
   if ($29) {
    HEAP16[$2>>1] = 0;
    $126 = HEAP16[$2>>1]|0;
    STACKTOP = sp;return ($126|0);
   } else {
    while(1) {
     $30 = $3;
     $31 = $30;
     $32 = HEAP32[$31>>2]|0;
     $33 = (($30) + 4)|0;
     $34 = $33;
     $35 = HEAP32[$34>>2]|0;
     $36 = ($35>>>0)<(0);
     $37 = ($32>>>0)<(8);
     $38 = ($35|0)==(0);
     $39 = $38 & $37;
     $40 = $36 | $39;
     if (!($40)) {
      break L1;
     }
     $41 = HEAP16[$y>>1]|0;
     $42 = $41 << 16 >> 16;
     $43 = (($42) - 10)|0;
     $44 = $43&65535;
     HEAP16[$y>>1] = $44;
     $45 = $3;
     $46 = $45;
     $47 = HEAP32[$46>>2]|0;
     $48 = (($45) + 4)|0;
     $49 = $48;
     $50 = HEAP32[$49>>2]|0;
     $51 = (_bitshift64Shl(($47|0),($50|0),1)|0);
     $52 = tempRet0;
     $53 = $3;
     $54 = $53;
     HEAP32[$54>>2] = $51;
     $55 = (($53) + 4)|0;
     $56 = $55;
     HEAP32[$56>>2] = $52;
    }
   }
  } else {
   while(1) {
    $57 = $3;
    $58 = $57;
    $59 = HEAP32[$58>>2]|0;
    $60 = (($57) + 4)|0;
    $61 = $60;
    $62 = HEAP32[$61>>2]|0;
    $63 = ($62>>>0)>(0);
    $64 = ($59>>>0)>(255);
    $65 = ($62|0)==(0);
    $66 = $65 & $64;
    $67 = $63 | $66;
    if (!($67)) {
     break;
    }
    $68 = HEAP16[$y>>1]|0;
    $69 = $68 << 16 >> 16;
    $70 = (($69) + 40)|0;
    $71 = $70&65535;
    HEAP16[$y>>1] = $71;
    $72 = $3;
    $73 = $72;
    $74 = HEAP32[$73>>2]|0;
    $75 = (($72) + 4)|0;
    $76 = $75;
    $77 = HEAP32[$76>>2]|0;
    $78 = (_bitshift64Lshr(($74|0),($77|0),4)|0);
    $79 = tempRet0;
    $80 = $3;
    $81 = $80;
    HEAP32[$81>>2] = $78;
    $82 = (($80) + 4)|0;
    $83 = $82;
    HEAP32[$83>>2] = $79;
   }
   while(1) {
    $84 = $3;
    $85 = $84;
    $86 = HEAP32[$85>>2]|0;
    $87 = (($84) + 4)|0;
    $88 = $87;
    $89 = HEAP32[$88>>2]|0;
    $90 = ($89>>>0)>(0);
    $91 = ($86>>>0)>(15);
    $92 = ($89|0)==(0);
    $93 = $92 & $91;
    $94 = $90 | $93;
    if (!($94)) {
     break L1;
    }
    $95 = HEAP16[$y>>1]|0;
    $96 = $95 << 16 >> 16;
    $97 = (($96) + 10)|0;
    $98 = $97&65535;
    HEAP16[$y>>1] = $98;
    $99 = $3;
    $100 = $99;
    $101 = HEAP32[$100>>2]|0;
    $102 = (($99) + 4)|0;
    $103 = $102;
    $104 = HEAP32[$103>>2]|0;
    $105 = (_bitshift64Lshr(($101|0),($104|0),1)|0);
    $106 = tempRet0;
    $107 = $3;
    $108 = $107;
    HEAP32[$108>>2] = $105;
    $109 = (($107) + 4)|0;
    $110 = $109;
    HEAP32[$110>>2] = $106;
   }
  }
 } while(0);
 $111 = $3;
 $112 = $111;
 $113 = HEAP32[$112>>2]|0;
 $114 = (($111) + 4)|0;
 $115 = $114;
 $116 = HEAP32[$115>>2]|0;
 $117 = $113 & 7;
 $118 = (16360 + ($117<<1)|0);
 $119 = HEAP16[$118>>1]|0;
 $120 = $119 << 16 >> 16;
 $121 = HEAP16[$y>>1]|0;
 $122 = $121 << 16 >> 16;
 $123 = (($120) + ($122))|0;
 $124 = (($123) - 10)|0;
 $125 = $124&65535;
 HEAP16[$2>>1] = $125;
 $126 = HEAP16[$2>>1]|0;
 STACKTOP = sp;return ($126|0);
}
function _sqlite3GetCollSeq($pParse,$enc,$pColl,$zName) {
 $pParse = $pParse|0;
 $enc = $enc|0;
 $pColl = $pColl|0;
 $zName = $zName|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $5 = 0, $6 = 0, $7 = 0;
 var $8 = 0, $9 = 0, $db = 0, $p = 0, $vararg_buffer = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $vararg_buffer = sp;
 $0 = sp + 20|0;
 $1 = sp + 24|0;
 $2 = sp + 16|0;
 $3 = sp + 12|0;
 $p = sp + 8|0;
 $db = sp + 4|0;
 HEAP32[$0>>2] = $pParse;
 HEAP8[$1>>0] = $enc;
 HEAP32[$2>>2] = $pColl;
 HEAP32[$3>>2] = $zName;
 $4 = HEAP32[$0>>2]|0;
 $5 = HEAP32[$4>>2]|0;
 HEAP32[$db>>2] = $5;
 $6 = HEAP32[$2>>2]|0;
 HEAP32[$p>>2] = $6;
 $7 = HEAP32[$p>>2]|0;
 $8 = ($7|0)!=(0|0);
 if (!($8)) {
  $9 = HEAP32[$db>>2]|0;
  $10 = HEAP8[$1>>0]|0;
  $11 = HEAP32[$3>>2]|0;
  $12 = (_sqlite3FindCollSeq($9,$10,$11,0)|0);
  HEAP32[$p>>2] = $12;
 }
 $13 = HEAP32[$p>>2]|0;
 $14 = ($13|0)!=(0|0);
 if ($14) {
  $15 = HEAP32[$p>>2]|0;
  $16 = (($15) + 12|0);
  $17 = HEAP32[$16>>2]|0;
  $18 = ($17|0)!=(0|0);
  if (!($18)) {
   label = 5;
  }
 } else {
  label = 5;
 }
 if ((label|0) == 5) {
  $19 = HEAP32[$db>>2]|0;
  $20 = HEAP8[$1>>0]|0;
  $21 = $20&255;
  $22 = HEAP32[$3>>2]|0;
  _callCollNeeded($19,$21,$22);
  $23 = HEAP32[$db>>2]|0;
  $24 = HEAP8[$1>>0]|0;
  $25 = HEAP32[$3>>2]|0;
  $26 = (_sqlite3FindCollSeq($23,$24,$25,0)|0);
  HEAP32[$p>>2] = $26;
 }
 $27 = HEAP32[$p>>2]|0;
 $28 = ($27|0)!=(0|0);
 if ($28) {
  $29 = HEAP32[$p>>2]|0;
  $30 = (($29) + 12|0);
  $31 = HEAP32[$30>>2]|0;
  $32 = ($31|0)!=(0|0);
  if (!($32)) {
   $33 = HEAP32[$db>>2]|0;
   $34 = HEAP32[$p>>2]|0;
   $35 = (_synthCollSeq($33,$34)|0);
   $36 = ($35|0)!=(0);
   if ($36) {
    HEAP32[$p>>2] = 0;
   }
  }
 }
 $37 = HEAP32[$p>>2]|0;
 $38 = ($37|0)==(0|0);
 if (!($38)) {
  $41 = HEAP32[$p>>2]|0;
  STACKTOP = sp;return ($41|0);
 }
 $39 = HEAP32[$0>>2]|0;
 $40 = HEAP32[$3>>2]|0;
 HEAP32[$vararg_buffer>>2] = $40;
 _sqlite3ErrorMsg($39,16376,$vararg_buffer);
 $41 = HEAP32[$p>>2]|0;
 STACKTOP = sp;return ($41|0);
}
function _sqlite3CheckCollSeq($pParse,$pColl) {
 $pParse = $pParse|0;
 $pColl = $pColl|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0;
 var $7 = 0, $8 = 0, $9 = 0, $db = 0, $p = 0, $zName = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 20|0;
 $1 = sp + 16|0;
 $2 = sp + 12|0;
 $zName = sp + 8|0;
 $db = sp + 4|0;
 $p = sp;
 HEAP32[$1>>2] = $pParse;
 HEAP32[$2>>2] = $pColl;
 $3 = HEAP32[$2>>2]|0;
 $4 = ($3|0)!=(0|0);
 if ($4) {
  $5 = HEAP32[$2>>2]|0;
  $6 = HEAP32[$5>>2]|0;
  HEAP32[$zName>>2] = $6;
  $7 = HEAP32[$1>>2]|0;
  $8 = HEAP32[$7>>2]|0;
  HEAP32[$db>>2] = $8;
  $9 = HEAP32[$1>>2]|0;
  $10 = HEAP32[$db>>2]|0;
  $11 = (($10) + 16|0);
  $12 = HEAP32[$11>>2]|0;
  $13 = (($12) + 12|0);
  $14 = HEAP32[$13>>2]|0;
  $15 = (($14) + 77|0);
  $16 = HEAP8[$15>>0]|0;
  $17 = HEAP32[$2>>2]|0;
  $18 = HEAP32[$zName>>2]|0;
  $19 = (_sqlite3GetCollSeq($9,$16,$17,$18)|0);
  HEAP32[$p>>2] = $19;
  $20 = HEAP32[$p>>2]|0;
  $21 = ($20|0)!=(0|0);
  if (!($21)) {
   HEAP32[$0>>2] = 1;
   $22 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($22|0);
  }
 }
 HEAP32[$0>>2] = 0;
 $22 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($22|0);
}
function _callCollNeeded($db,$enc,$zName) {
 $db = $db|0;
 $enc = $enc|0;
 $zName = $zName|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $pTmp = 0, $zExternal = 0, $zExternal1 = 0;
 var label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 20|0;
 $1 = sp + 16|0;
 $2 = sp + 12|0;
 $zExternal = sp + 8|0;
 $zExternal1 = sp + 4|0;
 $pTmp = sp;
 HEAP32[$0>>2] = $db;
 HEAP32[$1>>2] = $enc;
 HEAP32[$2>>2] = $zName;
 $3 = HEAP32[$0>>2]|0;
 $4 = (($3) + 220|0);
 $5 = HEAP32[$4>>2]|0;
 $6 = ($5|0)!=(0|0);
 do {
  if ($6) {
   $7 = HEAP32[$0>>2]|0;
   $8 = HEAP32[$2>>2]|0;
   $9 = (_sqlite3DbStrDup($7,$8)|0);
   HEAP32[$zExternal>>2] = $9;
   $10 = HEAP32[$zExternal>>2]|0;
   $11 = ($10|0)!=(0|0);
   if ($11) {
    $12 = HEAP32[$0>>2]|0;
    $13 = (($12) + 220|0);
    $14 = HEAP32[$13>>2]|0;
    $15 = HEAP32[$0>>2]|0;
    $16 = (($15) + 228|0);
    $17 = HEAP32[$16>>2]|0;
    $18 = HEAP32[$0>>2]|0;
    $19 = HEAP32[$1>>2]|0;
    $20 = HEAP32[$zExternal>>2]|0;
    FUNCTION_TABLE_viiii[$14 & 3]($17,$18,$19,$20);
    $21 = HEAP32[$0>>2]|0;
    $22 = HEAP32[$zExternal>>2]|0;
    _sqlite3DbFree($21,$22);
    break;
   } else {
    STACKTOP = sp;return;
   }
  }
 } while(0);
 $23 = HEAP32[$0>>2]|0;
 $24 = (($23) + 224|0);
 $25 = HEAP32[$24>>2]|0;
 $26 = ($25|0)!=(0|0);
 if (!($26)) {
  STACKTOP = sp;return;
 }
 $27 = HEAP32[$0>>2]|0;
 $28 = (_sqlite3ValueNew($27)|0);
 HEAP32[$pTmp>>2] = $28;
 $29 = HEAP32[$pTmp>>2]|0;
 $30 = HEAP32[$2>>2]|0;
 _sqlite3ValueSetStr($29,-1,$30,1,0);
 $31 = HEAP32[$pTmp>>2]|0;
 $32 = HEAP8[1136>>0]|0;
 $33 = $32 << 24 >> 24;
 $34 = ($33|0)==(0);
 $35 = $34 ? 3 : 2;
 $36 = $35&255;
 $37 = (_sqlite3ValueText($31,$36)|0);
 HEAP32[$zExternal1>>2] = $37;
 $38 = HEAP32[$zExternal1>>2]|0;
 $39 = ($38|0)!=(0|0);
 if ($39) {
  $40 = HEAP32[$0>>2]|0;
  $41 = (($40) + 224|0);
  $42 = HEAP32[$41>>2]|0;
  $43 = HEAP32[$0>>2]|0;
  $44 = (($43) + 228|0);
  $45 = HEAP32[$44>>2]|0;
  $46 = HEAP32[$0>>2]|0;
  $47 = HEAP32[$0>>2]|0;
  $48 = (($47) + 16|0);
  $49 = HEAP32[$48>>2]|0;
  $50 = (($49) + 12|0);
  $51 = HEAP32[$50>>2]|0;
  $52 = (($51) + 77|0);
  $53 = HEAP8[$52>>0]|0;
  $54 = $53&255;
  $55 = HEAP32[$zExternal1>>2]|0;
  FUNCTION_TABLE_viiii[$42 & 3]($45,$46,$54,$55);
 }
 $56 = HEAP32[$pTmp>>2]|0;
 _sqlite3ValueFree($56);
 STACKTOP = sp;return;
}
function _synthCollSeq($db,$pColl) {
 $db = $db|0;
 $pColl = $pColl|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $3 = 0, $4 = 0, $5 = 0;
 var $6 = 0, $7 = 0, $8 = 0, $9 = 0, $i = 0, $pColl2 = 0, $z = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 20|0;
 $1 = sp + 16|0;
 $2 = sp + 12|0;
 $pColl2 = sp + 8|0;
 $z = sp + 4|0;
 $i = sp;
 HEAP32[$1>>2] = $db;
 HEAP32[$2>>2] = $pColl;
 $3 = HEAP32[$2>>2]|0;
 $4 = HEAP32[$3>>2]|0;
 HEAP32[$z>>2] = $4;
 HEAP32[$i>>2] = 0;
 while(1) {
  $5 = HEAP32[$i>>2]|0;
  $6 = ($5|0)<(3);
  if (!($6)) {
   label = 6;
   break;
  }
  $7 = HEAP32[$1>>2]|0;
  $8 = HEAP32[$i>>2]|0;
  $9 = (16408 + ($8)|0);
  $10 = HEAP8[$9>>0]|0;
  $11 = HEAP32[$z>>2]|0;
  $12 = (_sqlite3FindCollSeq($7,$10,$11,0)|0);
  HEAP32[$pColl2>>2] = $12;
  $13 = HEAP32[$pColl2>>2]|0;
  $14 = (($13) + 12|0);
  $15 = HEAP32[$14>>2]|0;
  $16 = ($15|0)!=(0|0);
  if ($16) {
   label = 4;
   break;
  }
  $21 = HEAP32[$i>>2]|0;
  $22 = (($21) + 1)|0;
  HEAP32[$i>>2] = $22;
 }
 if ((label|0) == 4) {
  $17 = HEAP32[$2>>2]|0;
  $18 = HEAP32[$pColl2>>2]|0;
  ;HEAP32[$17+0>>2]=HEAP32[$18+0>>2]|0;HEAP32[$17+4>>2]=HEAP32[$18+4>>2]|0;HEAP32[$17+8>>2]=HEAP32[$18+8>>2]|0;HEAP32[$17+12>>2]=HEAP32[$18+12>>2]|0;HEAP32[$17+16>>2]=HEAP32[$18+16>>2]|0;
  $19 = HEAP32[$2>>2]|0;
  $20 = (($19) + 16|0);
  HEAP32[$20>>2] = 0;
  HEAP32[$0>>2] = 0;
  $23 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($23|0);
 }
 else if ((label|0) == 6) {
  HEAP32[$0>>2] = 1;
  $23 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($23|0);
 }
 return 0|0;
}
function _sqlite3ExprSkipCollate($pExpr) {
 $pExpr = $pExpr|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0;
 var $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 HEAP32[$0>>2] = $pExpr;
 while(1) {
  $1 = HEAP32[$0>>2]|0;
  $2 = ($1|0)!=(0|0);
  if ($2) {
   $3 = HEAP32[$0>>2]|0;
   $4 = (($3) + 4|0);
   $5 = HEAP32[$4>>2]|0;
   $6 = $5 & 4096;
   $7 = ($6|0)!=(0);
   $21 = $7;
  } else {
   $21 = 0;
  }
  $8 = HEAP32[$0>>2]|0;
  if (!($21)) {
   break;
  }
  $9 = (($8) + 4|0);
  $10 = HEAP32[$9>>2]|0;
  $11 = $10 & 262144;
  $12 = ($11|0)!=(0);
  $13 = HEAP32[$0>>2]|0;
  if ($12) {
   $14 = (($13) + 20|0);
   $15 = HEAP32[$14>>2]|0;
   $16 = (($15) + 4|0);
   $17 = HEAP32[$16>>2]|0;
   $18 = HEAP32[$17>>2]|0;
   HEAP32[$0>>2] = $18;
   continue;
  } else {
   $19 = (($13) + 12|0);
   $20 = HEAP32[$19>>2]|0;
   HEAP32[$0>>2] = $20;
   continue;
  }
 }
 STACKTOP = sp;return ($8|0);
}
function _sqlite3SelectExpand($pParse,$pSelect) {
 $pParse = $pParse|0;
 $pSelect = $pSelect|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $w = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 28|0;
 $1 = sp + 24|0;
 $w = sp;
 HEAP32[$0>>2] = $pParse;
 HEAP32[$1>>2] = $pSelect;
 ;HEAP32[$w+0>>2]=0|0;HEAP32[$w+4>>2]=0|0;HEAP32[$w+8>>2]=0|0;HEAP32[$w+12>>2]=0|0;HEAP32[$w+16>>2]=0|0;HEAP32[$w+20>>2]=0|0;
 HEAP32[$w>>2] = 37;
 $2 = HEAP32[$0>>2]|0;
 $3 = (($w) + 12|0);
 HEAP32[$3>>2] = $2;
 $4 = HEAP32[$0>>2]|0;
 $5 = (($4) + 22|0);
 $6 = HEAP8[$5>>0]|0;
 $7 = ($6<<24>>24)!=(0);
 if ($7) {
  $8 = (($w) + 4|0);
  HEAP32[$8>>2] = 38;
  $9 = HEAP32[$1>>2]|0;
  (_sqlite3WalkSelect($w,$9)|0);
 }
 $10 = (($w) + 4|0);
 HEAP32[$10>>2] = 39;
 $11 = (($w) + 8|0);
 HEAP32[$11>>2] = 3;
 $12 = HEAP32[$1>>2]|0;
 (_sqlite3WalkSelect($w,$12)|0);
 STACKTOP = sp;return;
}
function _sqlite3ResolveSelectNames($pParse,$p,$pOuterNC) {
 $pParse = $pParse|0;
 $p = $p|0;
 $pOuterNC = $pOuterNC|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $w = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $0 = sp + 32|0;
 $1 = sp + 28|0;
 $2 = sp + 24|0;
 $w = sp;
 HEAP32[$0>>2] = $pParse;
 HEAP32[$1>>2] = $p;
 HEAP32[$2>>2] = $pOuterNC;
 ;HEAP32[$w+0>>2]=0|0;HEAP32[$w+4>>2]=0|0;HEAP32[$w+8>>2]=0|0;HEAP32[$w+12>>2]=0|0;HEAP32[$w+16>>2]=0|0;HEAP32[$w+20>>2]=0|0;
 HEAP32[$w>>2] = 40;
 $3 = (($w) + 4|0);
 HEAP32[$3>>2] = 41;
 $4 = HEAP32[$0>>2]|0;
 $5 = (($w) + 12|0);
 HEAP32[$5>>2] = $4;
 $6 = HEAP32[$2>>2]|0;
 $7 = (($w) + 20|0);
 HEAP32[$7>>2] = $6;
 $8 = HEAP32[$1>>2]|0;
 (_sqlite3WalkSelect($w,$8)|0);
 STACKTOP = sp;return;
}
function _sqlite3SelectAddTypeInfo($pParse,$pSelect) {
 $pParse = $pParse|0;
 $pSelect = $pSelect|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $w = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 28|0;
 $1 = sp + 24|0;
 $w = sp;
 HEAP32[$0>>2] = $pParse;
 HEAP32[$1>>2] = $pSelect;
 ;HEAP32[$w+0>>2]=0|0;HEAP32[$w+4>>2]=0|0;HEAP32[$w+8>>2]=0|0;HEAP32[$w+12>>2]=0|0;HEAP32[$w+16>>2]=0|0;HEAP32[$w+20>>2]=0|0;
 $2 = (($w) + 8|0);
 HEAP32[$2>>2] = 4;
 HEAP32[$w>>2] = 37;
 $3 = HEAP32[$0>>2]|0;
 $4 = (($w) + 12|0);
 HEAP32[$4>>2] = $3;
 $5 = HEAP32[$1>>2]|0;
 (_sqlite3WalkSelect($w,$5)|0);
 STACKTOP = sp;return;
}
function _selectAddSubqueryTypeInfo($pWalker,$p) {
 $pWalker = $pWalker|0;
 $p = $p|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $i = 0, $pFrom = 0, $pParse = 0, $pSel = 0;
 var $pTab = 0, $pTabList = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 28|0;
 $1 = sp + 24|0;
 $pParse = sp + 20|0;
 $i = sp + 16|0;
 $pTabList = sp + 12|0;
 $pFrom = sp + 8|0;
 $pTab = sp + 4|0;
 $pSel = sp;
 HEAP32[$0>>2] = $pWalker;
 HEAP32[$1>>2] = $p;
 $2 = HEAP32[$1>>2]|0;
 $3 = (($2) + 6|0);
 $4 = HEAP16[$3>>1]|0;
 $5 = $4&65535;
 $6 = $5 & 32;
 $7 = ($6|0)==(0);
 if (!($7)) {
  STACKTOP = sp;return;
 }
 $8 = HEAP32[$1>>2]|0;
 $9 = (($8) + 6|0);
 $10 = HEAP16[$9>>1]|0;
 $11 = $10&65535;
 $12 = $11 | 32;
 $13 = $12&65535;
 HEAP16[$9>>1] = $13;
 $14 = HEAP32[$0>>2]|0;
 $15 = (($14) + 12|0);
 $16 = HEAP32[$15>>2]|0;
 HEAP32[$pParse>>2] = $16;
 $17 = HEAP32[$1>>2]|0;
 $18 = (($17) + 32|0);
 $19 = HEAP32[$18>>2]|0;
 HEAP32[$pTabList>>2] = $19;
 HEAP32[$i>>2] = 0;
 $20 = HEAP32[$pTabList>>2]|0;
 $21 = (($20) + 8|0);
 HEAP32[$pFrom>>2] = $21;
 while(1) {
  $22 = HEAP32[$i>>2]|0;
  $23 = HEAP32[$pTabList>>2]|0;
  $24 = HEAP32[$23>>2]|0;
  $25 = ($22|0)<($24|0);
  if (!($25)) {
   break;
  }
  $26 = HEAP32[$pFrom>>2]|0;
  $27 = (($26) + 16|0);
  $28 = HEAP32[$27>>2]|0;
  HEAP32[$pTab>>2] = $28;
  $29 = HEAP32[$pTab>>2]|0;
  $30 = ($29|0)!=(0|0);
  if ($30) {
   $31 = HEAP32[$pTab>>2]|0;
   $32 = (($31) + 44|0);
   $33 = HEAP8[$32>>0]|0;
   $34 = $33&255;
   $35 = $34 & 2;
   $36 = ($35|0)!=(0);
   if ($36) {
    $37 = HEAP32[$pFrom>>2]|0;
    $38 = (($37) + 20|0);
    $39 = HEAP32[$38>>2]|0;
    HEAP32[$pSel>>2] = $39;
    $40 = HEAP32[$pSel>>2]|0;
    $41 = ($40|0)!=(0|0);
    if ($41) {
     while(1) {
      $42 = HEAP32[$pSel>>2]|0;
      $43 = (($42) + 52|0);
      $44 = HEAP32[$43>>2]|0;
      $45 = ($44|0)!=(0|0);
      if (!($45)) {
       break;
      }
      $46 = HEAP32[$pSel>>2]|0;
      $47 = (($46) + 52|0);
      $48 = HEAP32[$47>>2]|0;
      HEAP32[$pSel>>2] = $48;
     }
     $49 = HEAP32[$pParse>>2]|0;
     $50 = HEAP32[$pTab>>2]|0;
     $51 = HEAP32[$pSel>>2]|0;
     _selectAddColumnTypeAndCollation($49,$50,$51);
    }
   }
  }
  $52 = HEAP32[$i>>2]|0;
  $53 = (($52) + 1)|0;
  HEAP32[$i>>2] = $53;
  $54 = HEAP32[$pFrom>>2]|0;
  $55 = (($54) + 72|0);
  HEAP32[$pFrom>>2] = $55;
 }
 STACKTOP = sp;return;
}
function _exprWalkNoop($NotUsed,$NotUsed2) {
 $NotUsed = $NotUsed|0;
 $NotUsed2 = $NotUsed2|0;
 var $0 = 0, $1 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $1 = sp;
 HEAP32[$0>>2] = $NotUsed;
 HEAP32[$1>>2] = $NotUsed2;
 STACKTOP = sp;return 0;
}
function _sqlite3WalkSelect($pWalker,$p) {
 $pWalker = $pWalker|0;
 $p = $p|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $7 = 0;
 var $8 = 0, $9 = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $1 = sp + 8|0;
 $2 = sp + 4|0;
 $rc = sp;
 HEAP32[$1>>2] = $pWalker;
 HEAP32[$2>>2] = $p;
 $3 = HEAP32[$2>>2]|0;
 $4 = ($3|0)==(0|0);
 do {
  if (!($4)) {
   $5 = HEAP32[$1>>2]|0;
   $6 = (($5) + 4|0);
   $7 = HEAP32[$6>>2]|0;
   $8 = ($7|0)==(0|0);
   if ($8) {
    $9 = HEAP32[$1>>2]|0;
    $10 = (($9) + 8|0);
    $11 = HEAP32[$10>>2]|0;
    $12 = ($11|0)==(0|0);
    if ($12) {
     break;
    }
   }
   HEAP32[$rc>>2] = 0;
   $13 = HEAP32[$1>>2]|0;
   $14 = (($13) + 16|0);
   $15 = HEAP32[$14>>2]|0;
   $16 = (($15) + 1)|0;
   HEAP32[$14>>2] = $16;
   while(1) {
    $17 = HEAP32[$2>>2]|0;
    $18 = ($17|0)!=(0|0);
    if (!($18)) {
     label = 15;
     break;
    }
    $19 = HEAP32[$1>>2]|0;
    $20 = (($19) + 4|0);
    $21 = HEAP32[$20>>2]|0;
    $22 = ($21|0)!=(0|0);
    if ($22) {
     $23 = HEAP32[$1>>2]|0;
     $24 = (($23) + 4|0);
     $25 = HEAP32[$24>>2]|0;
     $26 = HEAP32[$1>>2]|0;
     $27 = HEAP32[$2>>2]|0;
     $28 = (FUNCTION_TABLE_iii[$25 & 63]($26,$27)|0);
     HEAP32[$rc>>2] = $28;
     $29 = HEAP32[$rc>>2]|0;
     $30 = ($29|0)!=(0);
     if ($30) {
      label = 15;
      break;
     }
    }
    $31 = HEAP32[$1>>2]|0;
    $32 = HEAP32[$2>>2]|0;
    $33 = (_sqlite3WalkSelectExpr($31,$32)|0);
    $34 = ($33|0)!=(0);
    if ($34) {
     label = 11;
     break;
    }
    $35 = HEAP32[$1>>2]|0;
    $36 = HEAP32[$2>>2]|0;
    $37 = (_sqlite3WalkSelectFrom($35,$36)|0);
    $38 = ($37|0)!=(0);
    if ($38) {
     label = 11;
     break;
    }
    $43 = HEAP32[$1>>2]|0;
    $44 = (($43) + 8|0);
    $45 = HEAP32[$44>>2]|0;
    $46 = ($45|0)!=(0|0);
    if ($46) {
     $47 = HEAP32[$1>>2]|0;
     $48 = (($47) + 8|0);
     $49 = HEAP32[$48>>2]|0;
     $50 = HEAP32[$1>>2]|0;
     $51 = HEAP32[$2>>2]|0;
     FUNCTION_TABLE_vii[$49 & 7]($50,$51);
    }
    $52 = HEAP32[$2>>2]|0;
    $53 = (($52) + 52|0);
    $54 = HEAP32[$53>>2]|0;
    HEAP32[$2>>2] = $54;
   }
   if ((label|0) == 11) {
    $39 = HEAP32[$1>>2]|0;
    $40 = (($39) + 16|0);
    $41 = HEAP32[$40>>2]|0;
    $42 = (($41) + -1)|0;
    HEAP32[$40>>2] = $42;
    HEAP32[$0>>2] = 2;
    $61 = HEAP32[$0>>2]|0;
    STACKTOP = sp;return ($61|0);
   }
   else if ((label|0) == 15) {
    $55 = HEAP32[$1>>2]|0;
    $56 = (($55) + 16|0);
    $57 = HEAP32[$56>>2]|0;
    $58 = (($57) + -1)|0;
    HEAP32[$56>>2] = $58;
    $59 = HEAP32[$rc>>2]|0;
    $60 = $59 & 2;
    HEAP32[$0>>2] = $60;
    $61 = HEAP32[$0>>2]|0;
    STACKTOP = sp;return ($61|0);
   }
  }
 } while(0);
 HEAP32[$0>>2] = 0;
 $61 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($61|0);
}
function _sqlite3WalkSelectExpr($pWalker,$p) {
 $pWalker = $pWalker|0;
 $p = $p|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $1 = sp + 4|0;
 $2 = sp;
 HEAP32[$1>>2] = $pWalker;
 HEAP32[$2>>2] = $p;
 $3 = HEAP32[$1>>2]|0;
 $4 = HEAP32[$2>>2]|0;
 $5 = HEAP32[$4>>2]|0;
 $6 = (_sqlite3WalkExprList($3,$5)|0);
 $7 = ($6|0)!=(0);
 do {
  if ($7) {
   HEAP32[$0>>2] = 2;
  } else {
   $8 = HEAP32[$1>>2]|0;
   $9 = HEAP32[$2>>2]|0;
   $10 = (($9) + 36|0);
   $11 = HEAP32[$10>>2]|0;
   $12 = (_sqlite3WalkExpr($8,$11)|0);
   $13 = ($12|0)!=(0);
   if ($13) {
    HEAP32[$0>>2] = 2;
    break;
   }
   $14 = HEAP32[$1>>2]|0;
   $15 = HEAP32[$2>>2]|0;
   $16 = (($15) + 40|0);
   $17 = HEAP32[$16>>2]|0;
   $18 = (_sqlite3WalkExprList($14,$17)|0);
   $19 = ($18|0)!=(0);
   if ($19) {
    HEAP32[$0>>2] = 2;
    break;
   }
   $20 = HEAP32[$1>>2]|0;
   $21 = HEAP32[$2>>2]|0;
   $22 = (($21) + 44|0);
   $23 = HEAP32[$22>>2]|0;
   $24 = (_sqlite3WalkExpr($20,$23)|0);
   $25 = ($24|0)!=(0);
   if ($25) {
    HEAP32[$0>>2] = 2;
    break;
   }
   $26 = HEAP32[$1>>2]|0;
   $27 = HEAP32[$2>>2]|0;
   $28 = (($27) + 48|0);
   $29 = HEAP32[$28>>2]|0;
   $30 = (_sqlite3WalkExprList($26,$29)|0);
   $31 = ($30|0)!=(0);
   if ($31) {
    HEAP32[$0>>2] = 2;
    break;
   }
   $32 = HEAP32[$1>>2]|0;
   $33 = HEAP32[$2>>2]|0;
   $34 = (($33) + 60|0);
   $35 = HEAP32[$34>>2]|0;
   $36 = (_sqlite3WalkExpr($32,$35)|0);
   $37 = ($36|0)!=(0);
   if ($37) {
    HEAP32[$0>>2] = 2;
    break;
   }
   $38 = HEAP32[$1>>2]|0;
   $39 = HEAP32[$2>>2]|0;
   $40 = (($39) + 64|0);
   $41 = HEAP32[$40>>2]|0;
   $42 = (_sqlite3WalkExpr($38,$41)|0);
   $43 = ($42|0)!=(0);
   if ($43) {
    HEAP32[$0>>2] = 2;
    break;
   } else {
    HEAP32[$0>>2] = 0;
    break;
   }
  }
 } while(0);
 $44 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($44|0);
}
function _sqlite3WalkSelectFrom($pWalker,$p) {
 $pWalker = $pWalker|0;
 $p = $p|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $3 = 0, $4 = 0;
 var $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $i = 0, $pItem = 0, $pSrc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 20|0;
 $1 = sp + 16|0;
 $2 = sp + 12|0;
 $pSrc = sp + 8|0;
 $i = sp + 4|0;
 $pItem = sp;
 HEAP32[$1>>2] = $pWalker;
 HEAP32[$2>>2] = $p;
 $3 = HEAP32[$2>>2]|0;
 $4 = (($3) + 32|0);
 $5 = HEAP32[$4>>2]|0;
 HEAP32[$pSrc>>2] = $5;
 $6 = HEAP32[$pSrc>>2]|0;
 $7 = ($6|0)!=(0|0);
 L1: do {
  if ($7) {
   $8 = HEAP32[$pSrc>>2]|0;
   $9 = HEAP32[$8>>2]|0;
   HEAP32[$i>>2] = $9;
   $10 = HEAP32[$pSrc>>2]|0;
   $11 = (($10) + 8|0);
   HEAP32[$pItem>>2] = $11;
   while(1) {
    $12 = HEAP32[$i>>2]|0;
    $13 = ($12|0)>(0);
    if (!($13)) {
     break L1;
    }
    $14 = HEAP32[$1>>2]|0;
    $15 = HEAP32[$pItem>>2]|0;
    $16 = (($15) + 20|0);
    $17 = HEAP32[$16>>2]|0;
    $18 = (_sqlite3WalkSelect($14,$17)|0);
    $19 = ($18|0)!=(0);
    if ($19) {
     break;
    }
    $20 = HEAP32[$i>>2]|0;
    $21 = (($20) + -1)|0;
    HEAP32[$i>>2] = $21;
    $22 = HEAP32[$pItem>>2]|0;
    $23 = (($22) + 72|0);
    HEAP32[$pItem>>2] = $23;
   }
   HEAP32[$0>>2] = 2;
   $24 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($24|0);
  }
 } while(0);
 HEAP32[$0>>2] = 0;
 $24 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($24|0);
}
function _sqlite3WalkExprList($pWalker,$p) {
 $pWalker = $pWalker|0;
 $p = $p|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0;
 var $8 = 0, $9 = 0, $i = 0, $pItem = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 16|0;
 $1 = sp + 12|0;
 $2 = sp + 8|0;
 $i = sp + 4|0;
 $pItem = sp;
 HEAP32[$1>>2] = $pWalker;
 HEAP32[$2>>2] = $p;
 $3 = HEAP32[$2>>2]|0;
 $4 = ($3|0)!=(0|0);
 L1: do {
  if ($4) {
   $5 = HEAP32[$2>>2]|0;
   $6 = HEAP32[$5>>2]|0;
   HEAP32[$i>>2] = $6;
   $7 = HEAP32[$2>>2]|0;
   $8 = (($7) + 4|0);
   $9 = HEAP32[$8>>2]|0;
   HEAP32[$pItem>>2] = $9;
   while(1) {
    $10 = HEAP32[$i>>2]|0;
    $11 = ($10|0)>(0);
    if (!($11)) {
     break L1;
    }
    $12 = HEAP32[$1>>2]|0;
    $13 = HEAP32[$pItem>>2]|0;
    $14 = HEAP32[$13>>2]|0;
    $15 = (_sqlite3WalkExpr($12,$14)|0);
    $16 = ($15|0)!=(0);
    if ($16) {
     break;
    }
    $17 = HEAP32[$i>>2]|0;
    $18 = (($17) + -1)|0;
    HEAP32[$i>>2] = $18;
    $19 = HEAP32[$pItem>>2]|0;
    $20 = (($19) + 20|0);
    HEAP32[$pItem>>2] = $20;
   }
   HEAP32[$0>>2] = 2;
   $21 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($21|0);
  }
 } while(0);
 HEAP32[$0>>2] = 0;
 $21 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($21|0);
}
function _sqlite3WalkExpr($pWalker,$pExpr) {
 $pWalker = $pWalker|0;
 $pExpr = $pExpr|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $1 = sp + 8|0;
 $2 = sp + 4|0;
 $rc = sp;
 HEAP32[$1>>2] = $pWalker;
 HEAP32[$2>>2] = $pExpr;
 $3 = HEAP32[$2>>2]|0;
 $4 = ($3|0)==(0|0);
 L1: do {
  if ($4) {
   HEAP32[$0>>2] = 0;
  } else {
   $5 = HEAP32[$1>>2]|0;
   $6 = HEAP32[$5>>2]|0;
   $7 = HEAP32[$1>>2]|0;
   $8 = HEAP32[$2>>2]|0;
   $9 = (FUNCTION_TABLE_iii[$6 & 63]($7,$8)|0);
   HEAP32[$rc>>2] = $9;
   $10 = HEAP32[$rc>>2]|0;
   $11 = ($10|0)==(0);
   do {
    if ($11) {
     $12 = HEAP32[$2>>2]|0;
     $13 = (($12) + 4|0);
     $14 = HEAP32[$13>>2]|0;
     $15 = $14 & 16384;
     $16 = ($15|0)!=(0);
     if (!($16)) {
      $17 = HEAP32[$1>>2]|0;
      $18 = HEAP32[$2>>2]|0;
      $19 = (($18) + 12|0);
      $20 = HEAP32[$19>>2]|0;
      $21 = (_sqlite3WalkExpr($17,$20)|0);
      $22 = ($21|0)!=(0);
      if ($22) {
       HEAP32[$0>>2] = 2;
       break L1;
      }
      $23 = HEAP32[$1>>2]|0;
      $24 = HEAP32[$2>>2]|0;
      $25 = (($24) + 16|0);
      $26 = HEAP32[$25>>2]|0;
      $27 = (_sqlite3WalkExpr($23,$26)|0);
      $28 = ($27|0)!=(0);
      if ($28) {
       HEAP32[$0>>2] = 2;
       break L1;
      }
      $29 = HEAP32[$2>>2]|0;
      $30 = (($29) + 4|0);
      $31 = HEAP32[$30>>2]|0;
      $32 = $31 & 2048;
      $33 = ($32|0)!=(0);
      $34 = HEAP32[$1>>2]|0;
      $35 = HEAP32[$2>>2]|0;
      $36 = (($35) + 20|0);
      if ($33) {
       $37 = HEAP32[$36>>2]|0;
       $38 = (_sqlite3WalkSelect($34,$37)|0);
       $39 = ($38|0)!=(0);
       if (!($39)) {
        break;
       }
       HEAP32[$0>>2] = 2;
       break L1;
      } else {
       $40 = HEAP32[$36>>2]|0;
       $41 = (_sqlite3WalkExprList($34,$40)|0);
       $42 = ($41|0)!=(0);
       if (!($42)) {
        break;
       }
       HEAP32[$0>>2] = 2;
       break L1;
      }
     }
    }
   } while(0);
   $43 = HEAP32[$rc>>2]|0;
   $44 = $43 & 2;
   HEAP32[$0>>2] = $44;
  }
 } while(0);
 $45 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($45|0);
}
function _resolveExprStep($pWalker,$pExpr) {
 $pWalker = $pWalker|0;
 $pExpr = $pExpr|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0;
 var $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0, $188 = 0;
 var $189 = 0, $19 = 0, $190 = 0, $191 = 0, $192 = 0, $193 = 0, $194 = 0, $195 = 0, $196 = 0, $197 = 0, $198 = 0, $199 = 0, $2 = 0, $20 = 0, $200 = 0, $201 = 0, $202 = 0, $203 = 0, $204 = 0, $205 = 0;
 var $206 = 0, $207 = 0, $208 = 0, $209 = 0, $21 = 0, $210 = 0, $211 = 0, $212 = 0, $213 = 0, $214 = 0, $215 = 0, $216 = 0, $217 = 0, $218 = 0, $219 = 0, $22 = 0, $220 = 0, $221 = 0, $222 = 0, $223 = 0;
 var $224 = 0, $225 = 0, $226 = 0, $227 = 0, $228 = 0, $229 = 0, $23 = 0, $230 = 0, $231 = 0, $232 = 0, $233 = 0, $234 = 0, $235 = 0, $236 = 0, $237 = 0, $238 = 0, $239 = 0, $24 = 0, $240 = 0, $241 = 0;
 var $242 = 0, $243 = 0, $244 = 0, $245 = 0, $246 = 0, $247 = 0, $248 = 0, $249 = 0, $25 = 0, $250 = 0, $251 = 0, $252 = 0, $253 = 0, $254 = 0, $255 = 0, $256 = 0, $257 = 0, $258 = 0, $259 = 0, $26 = 0;
 var $260 = 0, $261 = 0, $262 = 0, $263 = 0, $264 = 0, $265 = 0, $266 = 0, $267 = 0, $268 = 0, $269 = 0, $27 = 0, $270 = 0, $271 = 0, $272 = 0, $273 = 0, $274 = 0, $275 = 0, $276 = 0, $277 = 0, $278 = 0;
 var $279 = 0, $28 = 0, $280 = 0, $281 = 0, $282 = 0, $283 = 0, $284 = 0, $285 = 0, $286 = 0, $287 = 0, $288 = 0, $289 = 0, $29 = 0, $290 = 0, $291 = 0, $292 = 0, $293 = 0, $294 = 0, $295 = 0, $296 = 0;
 var $297 = 0, $298 = 0, $299 = 0, $3 = 0, $30 = 0, $300 = 0, $301 = 0, $302 = 0, $303 = 0, $304 = 0, $305 = 0, $306 = 0, $307 = 0, $308 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0;
 var $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0;
 var $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0;
 var $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0;
 var $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $auth = 0, $enc = 0, $is_agg = 0, $n = 0, $nId = 0, $nRef = 0, $no_such_func = 0, $pDef = 0, $pList = 0, $pNC = 0, $pNC2 = 0;
 var $pParse = 0, $pRight = 0, $vararg_buffer11 = 0, $vararg_ptr10 = 0, $vararg_ptr14 = 0, $vararg_ptr6 = 0, $wrong_num_args = 0, $zColumn = 0, $zDb = 0, $zId = 0, $zTable = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 96|0;
 $vararg_buffer11 = sp;
 $0 = sp + 84|0;
 $1 = sp + 80|0;
 $2 = sp + 76|0;
 $pNC = sp + 72|0;
 $pParse = sp + 68|0;
 $zColumn = sp + 64|0;
 $zTable = sp + 60|0;
 $zDb = sp + 56|0;
 $pRight = sp + 52|0;
 $pList = sp + 48|0;
 $n = sp + 44|0;
 $no_such_func = sp + 40|0;
 $wrong_num_args = sp + 36|0;
 $is_agg = sp + 32|0;
 $auth = sp + 28|0;
 $nId = sp + 24|0;
 $zId = sp + 20|0;
 $pDef = sp + 16|0;
 $enc = sp + 88|0;
 $pNC2 = sp + 12|0;
 $nRef = sp + 8|0;
 HEAP32[$1>>2] = $pWalker;
 HEAP32[$2>>2] = $pExpr;
 $3 = HEAP32[$1>>2]|0;
 $4 = (($3) + 20|0);
 $5 = HEAP32[$4>>2]|0;
 HEAP32[$pNC>>2] = $5;
 $6 = HEAP32[$pNC>>2]|0;
 $7 = HEAP32[$6>>2]|0;
 HEAP32[$pParse>>2] = $7;
 $8 = HEAP32[$2>>2]|0;
 $9 = (($8) + 4|0);
 $10 = HEAP32[$9>>2]|0;
 $11 = $10 & 4;
 $12 = ($11|0)!=(0);
 if ($12) {
  HEAP32[$0>>2] = 1;
  $308 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($308|0);
 }
 $13 = HEAP32[$2>>2]|0;
 $14 = (($13) + 4|0);
 $15 = HEAP32[$14>>2]|0;
 $16 = $15 | 4;
 HEAP32[$14>>2] = $16;
 $17 = HEAP32[$2>>2]|0;
 $18 = HEAP8[$17>>0]|0;
 $19 = $18&255;
 switch ($19|0) {
 case 135:  {
  $292 = HEAP32[$pParse>>2]|0;
  $293 = HEAP32[$pNC>>2]|0;
  _notValidCheckConstraint($292,$293,17056);
  $294 = HEAP32[$pParse>>2]|0;
  $295 = HEAP32[$pNC>>2]|0;
  _notValidPartIdxWhere($294,$295,17056);
  break;
 }
 case 75: case 20: case 119:  {
  $267 = HEAP32[$2>>2]|0;
  $268 = (($267) + 4|0);
  $269 = HEAP32[$268>>2]|0;
  $270 = $269 & 2048;
  $271 = ($270|0)!=(0);
  if ($271) {
   $272 = HEAP32[$pNC>>2]|0;
   $273 = (($272) + 20|0);
   $274 = HEAP32[$273>>2]|0;
   HEAP32[$nRef>>2] = $274;
   $275 = HEAP32[$pParse>>2]|0;
   $276 = HEAP32[$pNC>>2]|0;
   _notValidCheckConstraint($275,$276,17040);
   $277 = HEAP32[$pParse>>2]|0;
   $278 = HEAP32[$pNC>>2]|0;
   _notValidPartIdxWhere($277,$278,17040);
   $279 = HEAP32[$1>>2]|0;
   $280 = HEAP32[$2>>2]|0;
   $281 = (($280) + 20|0);
   $282 = HEAP32[$281>>2]|0;
   (_sqlite3WalkSelect($279,$282)|0);
   $283 = HEAP32[$nRef>>2]|0;
   $284 = HEAP32[$pNC>>2]|0;
   $285 = (($284) + 20|0);
   $286 = HEAP32[$285>>2]|0;
   $287 = ($283|0)!=($286|0);
   if ($287) {
    $288 = HEAP32[$2>>2]|0;
    $289 = (($288) + 4|0);
    $290 = HEAP32[$289>>2]|0;
    $291 = $290 | 32;
    HEAP32[$289>>2] = $291;
   }
  }
  break;
 }
 case 27:  {
  $20 = HEAP32[$pParse>>2]|0;
  $21 = HEAP32[$2>>2]|0;
  $22 = (($21) + 8|0);
  $23 = HEAP32[$22>>2]|0;
  $24 = HEAP32[$pNC>>2]|0;
  $25 = HEAP32[$2>>2]|0;
  $26 = (_lookupName($20,0,0,$23,$24,$25)|0);
  HEAP32[$0>>2] = $26;
  $308 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($308|0);
  break;
 }
 case 122:  {
  $27 = HEAP32[$2>>2]|0;
  $28 = (($27) + 16|0);
  $29 = HEAP32[$28>>2]|0;
  HEAP32[$pRight>>2] = $29;
  $30 = HEAP32[$pRight>>2]|0;
  $31 = HEAP8[$30>>0]|0;
  $32 = $31&255;
  $33 = ($32|0)==(27);
  if ($33) {
   HEAP32[$zDb>>2] = 0;
   $34 = HEAP32[$2>>2]|0;
   $35 = (($34) + 12|0);
   $36 = HEAP32[$35>>2]|0;
   $37 = (($36) + 8|0);
   $38 = HEAP32[$37>>2]|0;
   HEAP32[$zTable>>2] = $38;
   $39 = HEAP32[$pRight>>2]|0;
   $40 = (($39) + 8|0);
   $41 = HEAP32[$40>>2]|0;
   HEAP32[$zColumn>>2] = $41;
  } else {
   $42 = HEAP32[$2>>2]|0;
   $43 = (($42) + 12|0);
   $44 = HEAP32[$43>>2]|0;
   $45 = (($44) + 8|0);
   $46 = HEAP32[$45>>2]|0;
   HEAP32[$zDb>>2] = $46;
   $47 = HEAP32[$pRight>>2]|0;
   $48 = (($47) + 12|0);
   $49 = HEAP32[$48>>2]|0;
   $50 = (($49) + 8|0);
   $51 = HEAP32[$50>>2]|0;
   HEAP32[$zTable>>2] = $51;
   $52 = HEAP32[$pRight>>2]|0;
   $53 = (($52) + 16|0);
   $54 = HEAP32[$53>>2]|0;
   $55 = (($54) + 8|0);
   $56 = HEAP32[$55>>2]|0;
   HEAP32[$zColumn>>2] = $56;
  }
  $57 = HEAP32[$pParse>>2]|0;
  $58 = HEAP32[$zDb>>2]|0;
  $59 = HEAP32[$zTable>>2]|0;
  $60 = HEAP32[$zColumn>>2]|0;
  $61 = HEAP32[$pNC>>2]|0;
  $62 = HEAP32[$2>>2]|0;
  $63 = (_lookupName($57,$58,$59,$60,$61,$62)|0);
  HEAP32[$0>>2] = $63;
  $308 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($308|0);
  break;
 }
 case 153:  {
  $64 = HEAP32[$2>>2]|0;
  $65 = (($64) + 20|0);
  $66 = HEAP32[$65>>2]|0;
  HEAP32[$pList>>2] = $66;
  $67 = HEAP32[$pList>>2]|0;
  $68 = ($67|0)!=(0|0);
  if ($68) {
   $69 = HEAP32[$pList>>2]|0;
   $70 = HEAP32[$69>>2]|0;
   $71 = $70;
  } else {
   $71 = 0;
  }
  HEAP32[$n>>2] = $71;
  HEAP32[$no_such_func>>2] = 0;
  HEAP32[$wrong_num_args>>2] = 0;
  HEAP32[$is_agg>>2] = 0;
  $72 = HEAP32[$pParse>>2]|0;
  $73 = HEAP32[$72>>2]|0;
  $74 = (($73) + 16|0);
  $75 = HEAP32[$74>>2]|0;
  $76 = (($75) + 12|0);
  $77 = HEAP32[$76>>2]|0;
  $78 = (($77) + 77|0);
  $79 = HEAP8[$78>>0]|0;
  HEAP8[$enc>>0] = $79;
  $80 = HEAP32[$pParse>>2]|0;
  $81 = HEAP32[$pNC>>2]|0;
  _notValidPartIdxWhere($80,$81,16800);
  $82 = HEAP32[$2>>2]|0;
  $83 = (($82) + 8|0);
  $84 = HEAP32[$83>>2]|0;
  HEAP32[$zId>>2] = $84;
  $85 = HEAP32[$zId>>2]|0;
  $86 = (_sqlite3Strlen30($85)|0);
  HEAP32[$nId>>2] = $86;
  $87 = HEAP32[$pParse>>2]|0;
  $88 = HEAP32[$87>>2]|0;
  $89 = HEAP32[$zId>>2]|0;
  $90 = HEAP32[$nId>>2]|0;
  $91 = HEAP32[$n>>2]|0;
  $92 = HEAP8[$enc>>0]|0;
  $93 = (_sqlite3FindFunction($88,$89,$90,$91,$92,0)|0);
  HEAP32[$pDef>>2] = $93;
  $94 = HEAP32[$pDef>>2]|0;
  $95 = ($94|0)==(0|0);
  do {
   if ($95) {
    $96 = HEAP32[$pParse>>2]|0;
    $97 = HEAP32[$96>>2]|0;
    $98 = HEAP32[$zId>>2]|0;
    $99 = HEAP32[$nId>>2]|0;
    $100 = HEAP8[$enc>>0]|0;
    $101 = (_sqlite3FindFunction($97,$98,$99,-2,$100,0)|0);
    HEAP32[$pDef>>2] = $101;
    $102 = HEAP32[$pDef>>2]|0;
    $103 = ($102|0)==(0|0);
    if ($103) {
     HEAP32[$no_such_func>>2] = 1;
     break;
    } else {
     HEAP32[$wrong_num_args>>2] = 1;
     break;
    }
   } else {
    $104 = HEAP32[$pDef>>2]|0;
    $105 = (($104) + 12|0);
    $106 = HEAP32[$105>>2]|0;
    $107 = ($106|0)==(0|0);
    $108 = $107&1;
    HEAP32[$is_agg>>2] = $108;
    $109 = HEAP32[$pDef>>2]|0;
    $110 = (($109) + 2|0);
    $111 = HEAP16[$110>>1]|0;
    $112 = $111&65535;
    $113 = $112 & 1024;
    $114 = ($113|0)!=(0);
    do {
     if ($114) {
      $115 = HEAP32[$2>>2]|0;
      $116 = (($115) + 4|0);
      $117 = HEAP32[$116>>2]|0;
      $118 = $117 | 266240;
      HEAP32[$116>>2] = $118;
      $119 = HEAP32[$n>>2]|0;
      $120 = ($119|0)==(2);
      if (!($120)) {
       $138 = HEAP32[$pDef>>2]|0;
       $139 = (($138) + 24|0);
       $140 = HEAP32[$139>>2]|0;
       $141 = HEAP8[$140>>0]|0;
       $142 = $141 << 24 >> 24;
       $143 = ($142|0)==(117);
       $144 = $143 ? 62 : 938;
       $145 = HEAP32[$2>>2]|0;
       $146 = (($145) + 28|0);
       HEAP32[$146>>2] = $144;
       break;
      }
      $121 = HEAP32[$pList>>2]|0;
      $122 = (($121) + 4|0);
      $123 = HEAP32[$122>>2]|0;
      $124 = (($123) + 20|0);
      $125 = HEAP32[$124>>2]|0;
      $126 = (_exprProbability($125)|0);
      $127 = HEAP32[$2>>2]|0;
      $128 = (($127) + 28|0);
      HEAP32[$128>>2] = $126;
      $129 = HEAP32[$2>>2]|0;
      $130 = (($129) + 28|0);
      $131 = HEAP32[$130>>2]|0;
      $132 = ($131|0)<(0);
      if ($132) {
       $133 = HEAP32[$pParse>>2]|0;
       _sqlite3ErrorMsg($133,16816,$vararg_buffer11);
       $134 = HEAP32[$pNC>>2]|0;
       $135 = (($134) + 24|0);
       $136 = HEAP32[$135>>2]|0;
       $137 = (($136) + 1)|0;
       HEAP32[$135>>2] = $137;
      }
     }
    } while(0);
    $147 = HEAP32[$pParse>>2]|0;
    $148 = HEAP32[$pDef>>2]|0;
    $149 = (($148) + 24|0);
    $150 = HEAP32[$149>>2]|0;
    $151 = (_sqlite3AuthCheck($147,31,0,$150,0)|0);
    HEAP32[$auth>>2] = $151;
    $152 = HEAP32[$auth>>2]|0;
    $153 = ($152|0)!=(0);
    if (!($153)) {
     $165 = HEAP32[$pDef>>2]|0;
     $166 = (($165) + 2|0);
     $167 = HEAP16[$166>>1]|0;
     $168 = $167&65535;
     $169 = $168 & 2048;
     $170 = ($169|0)!=(0);
     if (!($170)) {
      break;
     }
     $171 = HEAP32[$2>>2]|0;
     $172 = (($171) + 4|0);
     $173 = HEAP32[$172>>2]|0;
     $174 = $173 | 524288;
     HEAP32[$172>>2] = $174;
     break;
    }
    $154 = HEAP32[$auth>>2]|0;
    $155 = ($154|0)==(1);
    if ($155) {
     $156 = HEAP32[$pParse>>2]|0;
     $157 = HEAP32[$pDef>>2]|0;
     $158 = (($157) + 24|0);
     $159 = HEAP32[$158>>2]|0;
     HEAP32[$vararg_buffer11>>2] = $159;
     _sqlite3ErrorMsg($156,16888,$vararg_buffer11);
     $160 = HEAP32[$pNC>>2]|0;
     $161 = (($160) + 24|0);
     $162 = HEAP32[$161>>2]|0;
     $163 = (($162) + 1)|0;
     HEAP32[$161>>2] = $163;
    }
    $164 = HEAP32[$2>>2]|0;
    HEAP8[$164>>0] = 101;
    HEAP32[$0>>2] = 1;
    $308 = HEAP32[$0>>2]|0;
    STACKTOP = sp;return ($308|0);
   }
  } while(0);
  $175 = HEAP32[$is_agg>>2]|0;
  $176 = ($175|0)!=(0);
  if ($176) {
   $177 = HEAP32[$pNC>>2]|0;
   $178 = (($177) + 28|0);
   $179 = HEAP16[$178>>1]|0;
   $180 = $179&65535;
   $181 = $180 & 1;
   $182 = ($181|0)==(0);
   if ($182) {
    $183 = HEAP32[$pParse>>2]|0;
    $184 = HEAP32[$nId>>2]|0;
    $185 = HEAP32[$zId>>2]|0;
    HEAP32[$vararg_buffer11>>2] = $184;
    $vararg_ptr6 = (($vararg_buffer11) + 4|0);
    HEAP32[$vararg_ptr6>>2] = $185;
    _sqlite3ErrorMsg($183,16928,$vararg_buffer11);
    $186 = HEAP32[$pNC>>2]|0;
    $187 = (($186) + 24|0);
    $188 = HEAP32[$187>>2]|0;
    $189 = (($188) + 1)|0;
    HEAP32[$187>>2] = $189;
    HEAP32[$is_agg>>2] = 0;
   } else {
    label = 29;
   }
  } else {
   label = 29;
  }
  do {
   if ((label|0) == 29) {
    $190 = HEAP32[$no_such_func>>2]|0;
    $191 = ($190|0)!=(0);
    if ($191) {
     $192 = HEAP32[$pParse>>2]|0;
     $193 = HEAP32[$192>>2]|0;
     $194 = (($193) + 140|0);
     $195 = (($194) + 5|0);
     $196 = HEAP8[$195>>0]|0;
     $197 = $196&255;
     $198 = ($197|0)==(0);
     if ($198) {
      $199 = HEAP32[$pParse>>2]|0;
      $200 = HEAP32[$nId>>2]|0;
      $201 = HEAP32[$zId>>2]|0;
      HEAP32[$vararg_buffer11>>2] = $200;
      $vararg_ptr10 = (($vararg_buffer11) + 4|0);
      HEAP32[$vararg_ptr10>>2] = $201;
      _sqlite3ErrorMsg($199,16968,$vararg_buffer11);
      $202 = HEAP32[$pNC>>2]|0;
      $203 = (($202) + 24|0);
      $204 = HEAP32[$203>>2]|0;
      $205 = (($204) + 1)|0;
      HEAP32[$203>>2] = $205;
      break;
     }
    }
    $206 = HEAP32[$wrong_num_args>>2]|0;
    $207 = ($206|0)!=(0);
    if ($207) {
     $208 = HEAP32[$pParse>>2]|0;
     $209 = HEAP32[$nId>>2]|0;
     $210 = HEAP32[$zId>>2]|0;
     HEAP32[$vararg_buffer11>>2] = $209;
     $vararg_ptr14 = (($vararg_buffer11) + 4|0);
     HEAP32[$vararg_ptr14>>2] = $210;
     _sqlite3ErrorMsg($208,16992,$vararg_buffer11);
     $211 = HEAP32[$pNC>>2]|0;
     $212 = (($211) + 24|0);
     $213 = HEAP32[$212>>2]|0;
     $214 = (($213) + 1)|0;
     HEAP32[$212>>2] = $214;
    }
   }
  } while(0);
  $215 = HEAP32[$is_agg>>2]|0;
  $216 = ($215|0)!=(0);
  if ($216) {
   $217 = HEAP32[$pNC>>2]|0;
   $218 = (($217) + 28|0);
   $219 = HEAP16[$218>>1]|0;
   $220 = $219&65535;
   $221 = $220 & -2;
   $222 = $221&65535;
   HEAP16[$218>>1] = $222;
  }
  $223 = HEAP32[$1>>2]|0;
  $224 = HEAP32[$pList>>2]|0;
  (_sqlite3WalkExprList($223,$224)|0);
  $225 = HEAP32[$is_agg>>2]|0;
  $226 = ($225|0)!=(0);
  if ($226) {
   $227 = HEAP32[$pNC>>2]|0;
   HEAP32[$pNC2>>2] = $227;
   $228 = HEAP32[$2>>2]|0;
   HEAP8[$228>>0] = -101;
   $229 = HEAP32[$2>>2]|0;
   $230 = (($229) + 38|0);
   HEAP8[$230>>0] = 0;
   while(1) {
    $231 = HEAP32[$pNC2>>2]|0;
    $232 = ($231|0)!=(0|0);
    if (!($232)) {
     break;
    }
    $233 = HEAP32[$2>>2]|0;
    $234 = HEAP32[$pNC2>>2]|0;
    $235 = (($234) + 4|0);
    $236 = HEAP32[$235>>2]|0;
    $237 = (_sqlite3FunctionUsesThisSrc($233,$236)|0);
    $238 = ($237|0)!=(0);
    $239 = $238 ^ 1;
    if (!($239)) {
     break;
    }
    $240 = HEAP32[$2>>2]|0;
    $241 = (($240) + 38|0);
    $242 = HEAP8[$241>>0]|0;
    $243 = (($242) + 1)<<24>>24;
    HEAP8[$241>>0] = $243;
    $244 = HEAP32[$pNC2>>2]|0;
    $245 = (($244) + 16|0);
    $246 = HEAP32[$245>>2]|0;
    HEAP32[$pNC2>>2] = $246;
   }
   $247 = HEAP32[$pNC2>>2]|0;
   $248 = ($247|0)!=(0|0);
   if ($248) {
    $249 = HEAP32[$pDef>>2]|0;
    $250 = (($249) + 2|0);
    $251 = HEAP16[$250>>1]|0;
    $252 = $251&65535;
    $253 = $252 & 4096;
    $254 = 2 | $253;
    $255 = HEAP32[$pNC2>>2]|0;
    $256 = (($255) + 28|0);
    $257 = HEAP16[$256>>1]|0;
    $258 = $257&65535;
    $259 = $258 | $254;
    $260 = $259&65535;
    HEAP16[$256>>1] = $260;
   }
   $261 = HEAP32[$pNC>>2]|0;
   $262 = (($261) + 28|0);
   $263 = HEAP16[$262>>1]|0;
   $264 = $263&65535;
   $265 = $264 | 1;
   $266 = $265&65535;
   HEAP16[$262>>1] = $266;
  }
  HEAP32[$0>>2] = 1;
  $308 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($308|0);
  break;
 }
 default: {
 }
 }
 $296 = HEAP32[$pParse>>2]|0;
 $297 = (($296) + 64|0);
 $298 = HEAP32[$297>>2]|0;
 $299 = ($298|0)!=(0);
 if ($299) {
  $307 = 1;
 } else {
  $300 = HEAP32[$pParse>>2]|0;
  $301 = HEAP32[$300>>2]|0;
  $302 = (($301) + 64|0);
  $303 = HEAP8[$302>>0]|0;
  $304 = $303&255;
  $305 = ($304|0)!=(0);
  $307 = $305;
 }
 $306 = $307 ? 2 : 0;
 HEAP32[$0>>2] = $306;
 $308 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($308|0);
}
function _resolveSelectStep($pWalker,$p) {
 $pWalker = $pWalker|0;
 $p = $p|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0;
 var $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0, $188 = 0;
 var $189 = 0, $19 = 0, $190 = 0, $191 = 0, $192 = 0, $193 = 0, $194 = 0, $195 = 0, $196 = 0, $197 = 0, $198 = 0, $199 = 0, $2 = 0, $20 = 0, $200 = 0, $201 = 0, $202 = 0, $203 = 0, $204 = 0, $205 = 0;
 var $206 = 0, $207 = 0, $208 = 0, $209 = 0, $21 = 0, $210 = 0, $211 = 0, $212 = 0, $213 = 0, $214 = 0, $215 = 0, $216 = 0, $217 = 0, $218 = 0, $219 = 0, $22 = 0, $220 = 0, $221 = 0, $222 = 0, $223 = 0;
 var $224 = 0, $225 = 0, $226 = 0, $227 = 0, $228 = 0, $229 = 0, $23 = 0, $230 = 0, $231 = 0, $232 = 0, $233 = 0, $234 = 0, $235 = 0, $236 = 0, $237 = 0, $238 = 0, $239 = 0, $24 = 0, $240 = 0, $241 = 0;
 var $242 = 0, $243 = 0, $244 = 0, $245 = 0, $246 = 0, $247 = 0, $248 = 0, $249 = 0, $25 = 0, $250 = 0, $251 = 0, $252 = 0, $253 = 0, $254 = 0, $255 = 0, $256 = 0, $257 = 0, $258 = 0, $259 = 0, $26 = 0;
 var $260 = 0, $261 = 0, $262 = 0, $263 = 0, $264 = 0, $265 = 0, $266 = 0, $267 = 0, $268 = 0, $269 = 0, $27 = 0, $270 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0;
 var $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0;
 var $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0;
 var $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0;
 var $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $db = 0, $i = 0, $isCompound = 0, $nCompound = 0, $nRef = 0, $or$cond = 0, $pEList = 0, $pGroupBy = 0, $pItem = 0;
 var $pItem1 = 0, $pLeftmost = 0, $pNC = 0, $pOuterNC = 0, $pParse = 0, $pX = 0, $sNC = 0, $vararg_buffer2 = 0, $zSavedContext = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 112|0;
 $vararg_buffer2 = sp;
 $0 = sp + 108|0;
 $1 = sp + 104|0;
 $2 = sp + 100|0;
 $pOuterNC = sp + 96|0;
 $sNC = sp + 64|0;
 $isCompound = sp + 56|0;
 $nCompound = sp + 52|0;
 $pParse = sp + 48|0;
 $pEList = sp + 44|0;
 $i = sp + 40|0;
 $pGroupBy = sp + 36|0;
 $pLeftmost = sp + 32|0;
 $db = sp + 28|0;
 $pItem = sp + 24|0;
 $pNC = sp + 20|0;
 $nRef = sp + 16|0;
 $zSavedContext = sp + 12|0;
 $pX = sp + 8|0;
 $pItem1 = sp + 4|0;
 HEAP32[$1>>2] = $pWalker;
 HEAP32[$2>>2] = $p;
 $3 = HEAP32[$2>>2]|0;
 $4 = (($3) + 6|0);
 $5 = HEAP16[$4>>1]|0;
 $6 = $5&65535;
 $7 = $6 & 2;
 $8 = ($7|0)!=(0);
 if ($8) {
  HEAP32[$0>>2] = 1;
  $270 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($270|0);
 }
 $9 = HEAP32[$1>>2]|0;
 $10 = (($9) + 20|0);
 $11 = HEAP32[$10>>2]|0;
 HEAP32[$pOuterNC>>2] = $11;
 $12 = HEAP32[$1>>2]|0;
 $13 = (($12) + 12|0);
 $14 = HEAP32[$13>>2]|0;
 HEAP32[$pParse>>2] = $14;
 $15 = HEAP32[$pParse>>2]|0;
 $16 = HEAP32[$15>>2]|0;
 HEAP32[$db>>2] = $16;
 $17 = HEAP32[$2>>2]|0;
 $18 = (($17) + 6|0);
 $19 = HEAP16[$18>>1]|0;
 $20 = $19&65535;
 $21 = $20 & 16;
 $22 = ($21|0)==(0);
 if ($22) {
  $23 = HEAP32[$pParse>>2]|0;
  $24 = HEAP32[$2>>2]|0;
  $25 = HEAP32[$pOuterNC>>2]|0;
  _sqlite3SelectPrep($23,$24,$25);
  $26 = HEAP32[$pParse>>2]|0;
  $27 = (($26) + 64|0);
  $28 = HEAP32[$27>>2]|0;
  $29 = ($28|0)!=(0);
  if ($29) {
   $36 = 1;
  } else {
   $30 = HEAP32[$db>>2]|0;
   $31 = (($30) + 64|0);
   $32 = HEAP8[$31>>0]|0;
   $33 = $32&255;
   $34 = ($33|0)!=(0);
   $36 = $34;
  }
  $35 = $36 ? 2 : 1;
  HEAP32[$0>>2] = $35;
  $270 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($270|0);
 }
 $37 = HEAP32[$2>>2]|0;
 $38 = (($37) + 52|0);
 $39 = HEAP32[$38>>2]|0;
 $40 = ($39|0)!=(0|0);
 $41 = $40&1;
 HEAP32[$isCompound>>2] = $41;
 HEAP32[$nCompound>>2] = 0;
 $42 = HEAP32[$2>>2]|0;
 HEAP32[$pLeftmost>>2] = $42;
 L12: while(1) {
  $43 = HEAP32[$2>>2]|0;
  $44 = ($43|0)!=(0|0);
  if (!($44)) {
   label = 58;
   break;
  }
  $45 = HEAP32[$2>>2]|0;
  $46 = (($45) + 6|0);
  $47 = HEAP16[$46>>1]|0;
  $48 = $47&65535;
  $49 = $48 | 2;
  $50 = $49&65535;
  HEAP16[$46>>1] = $50;
  ;HEAP32[$sNC+0>>2]=0|0;HEAP32[$sNC+4>>2]=0|0;HEAP32[$sNC+8>>2]=0|0;HEAP32[$sNC+12>>2]=0|0;HEAP32[$sNC+16>>2]=0|0;HEAP32[$sNC+20>>2]=0|0;HEAP32[$sNC+24>>2]=0|0;HEAP32[$sNC+28>>2]=0|0;
  $51 = HEAP32[$pParse>>2]|0;
  HEAP32[$sNC>>2] = $51;
  $52 = HEAP32[$2>>2]|0;
  $53 = (($52) + 60|0);
  $54 = HEAP32[$53>>2]|0;
  $55 = (_sqlite3ResolveExprNames($sNC,$54)|0);
  $56 = ($55|0)!=(0);
  if ($56) {
   label = 11;
   break;
  }
  $57 = HEAP32[$2>>2]|0;
  $58 = (($57) + 64|0);
  $59 = HEAP32[$58>>2]|0;
  $60 = (_sqlite3ResolveExprNames($sNC,$59)|0);
  $61 = ($60|0)!=(0);
  if ($61) {
   label = 11;
   break;
  }
  HEAP32[$i>>2] = 0;
  while(1) {
   $62 = HEAP32[$i>>2]|0;
   $63 = HEAP32[$2>>2]|0;
   $64 = (($63) + 32|0);
   $65 = HEAP32[$64>>2]|0;
   $66 = HEAP32[$65>>2]|0;
   $67 = ($62|0)<($66|0);
   if (!($67)) {
    break;
   }
   $68 = HEAP32[$i>>2]|0;
   $69 = HEAP32[$2>>2]|0;
   $70 = (($69) + 32|0);
   $71 = HEAP32[$70>>2]|0;
   $72 = (($71) + 8|0);
   $73 = (($72) + (($68*72)|0)|0);
   HEAP32[$pItem>>2] = $73;
   $74 = HEAP32[$pItem>>2]|0;
   $75 = (($74) + 20|0);
   $76 = HEAP32[$75>>2]|0;
   $77 = ($76|0)!=(0|0);
   if ($77) {
    HEAP32[$nRef>>2] = 0;
    $78 = HEAP32[$pParse>>2]|0;
    $79 = (($78) + 492|0);
    $80 = HEAP32[$79>>2]|0;
    HEAP32[$zSavedContext>>2] = $80;
    $81 = HEAP32[$pOuterNC>>2]|0;
    HEAP32[$pNC>>2] = $81;
    while(1) {
     $82 = HEAP32[$pNC>>2]|0;
     $83 = ($82|0)!=(0|0);
     if (!($83)) {
      break;
     }
     $84 = HEAP32[$pNC>>2]|0;
     $85 = (($84) + 20|0);
     $86 = HEAP32[$85>>2]|0;
     $87 = HEAP32[$nRef>>2]|0;
     $88 = (($87) + ($86))|0;
     HEAP32[$nRef>>2] = $88;
     $89 = HEAP32[$pNC>>2]|0;
     $90 = (($89) + 16|0);
     $91 = HEAP32[$90>>2]|0;
     HEAP32[$pNC>>2] = $91;
    }
    $92 = HEAP32[$pItem>>2]|0;
    $93 = (($92) + 8|0);
    $94 = HEAP32[$93>>2]|0;
    $95 = ($94|0)!=(0|0);
    if ($95) {
     $96 = HEAP32[$pItem>>2]|0;
     $97 = (($96) + 8|0);
     $98 = HEAP32[$97>>2]|0;
     $99 = HEAP32[$pParse>>2]|0;
     $100 = (($99) + 492|0);
     HEAP32[$100>>2] = $98;
    }
    $101 = HEAP32[$pParse>>2]|0;
    $102 = HEAP32[$pItem>>2]|0;
    $103 = (($102) + 20|0);
    $104 = HEAP32[$103>>2]|0;
    $105 = HEAP32[$pOuterNC>>2]|0;
    _sqlite3ResolveSelectNames($101,$104,$105);
    $106 = HEAP32[$zSavedContext>>2]|0;
    $107 = HEAP32[$pParse>>2]|0;
    $108 = (($107) + 492|0);
    HEAP32[$108>>2] = $106;
    $109 = HEAP32[$pParse>>2]|0;
    $110 = (($109) + 64|0);
    $111 = HEAP32[$110>>2]|0;
    $112 = ($111|0)!=(0);
    if ($112) {
     label = 22;
     break L12;
    }
    $113 = HEAP32[$db>>2]|0;
    $114 = (($113) + 64|0);
    $115 = HEAP8[$114>>0]|0;
    $116 = $115&255;
    $117 = ($116|0)!=(0);
    if ($117) {
     label = 22;
     break L12;
    }
    $118 = HEAP32[$pOuterNC>>2]|0;
    HEAP32[$pNC>>2] = $118;
    while(1) {
     $119 = HEAP32[$pNC>>2]|0;
     $120 = ($119|0)!=(0|0);
     if (!($120)) {
      break;
     }
     $121 = HEAP32[$pNC>>2]|0;
     $122 = (($121) + 20|0);
     $123 = HEAP32[$122>>2]|0;
     $124 = HEAP32[$nRef>>2]|0;
     $125 = (($124) - ($123))|0;
     HEAP32[$nRef>>2] = $125;
     $126 = HEAP32[$pNC>>2]|0;
     $127 = (($126) + 16|0);
     $128 = HEAP32[$127>>2]|0;
     HEAP32[$pNC>>2] = $128;
    }
    $129 = HEAP32[$nRef>>2]|0;
    $130 = ($129|0)!=(0);
    $131 = $130&1;
    $132 = HEAP32[$pItem>>2]|0;
    $133 = (($132) + 37|0);
    $134 = $131&255;
    $135 = HEAP8[$133>>0]|0;
    $136 = $134 & 1;
    $137 = ($136 << 1)&255;
    $138 = $135 & -3;
    $139 = $138 | $137;
    HEAP8[$133>>0] = $139;
   }
   $140 = HEAP32[$i>>2]|0;
   $141 = (($140) + 1)|0;
   HEAP32[$i>>2] = $141;
  }
  $142 = (($sNC) + 28|0);
  HEAP16[$142>>1] = 1;
  $143 = HEAP32[$2>>2]|0;
  $144 = (($143) + 32|0);
  $145 = HEAP32[$144>>2]|0;
  $146 = (($sNC) + 4|0);
  HEAP32[$146>>2] = $145;
  $147 = HEAP32[$pOuterNC>>2]|0;
  $148 = (($sNC) + 16|0);
  HEAP32[$148>>2] = $147;
  $149 = HEAP32[$2>>2]|0;
  $150 = HEAP32[$149>>2]|0;
  HEAP32[$pEList>>2] = $150;
  HEAP32[$i>>2] = 0;
  while(1) {
   $151 = HEAP32[$i>>2]|0;
   $152 = HEAP32[$pEList>>2]|0;
   $153 = HEAP32[$152>>2]|0;
   $154 = ($151|0)<($153|0);
   if (!($154)) {
    break;
   }
   $155 = HEAP32[$i>>2]|0;
   $156 = HEAP32[$pEList>>2]|0;
   $157 = (($156) + 4|0);
   $158 = HEAP32[$157>>2]|0;
   $159 = (($158) + (($155*20)|0)|0);
   $160 = HEAP32[$159>>2]|0;
   HEAP32[$pX>>2] = $160;
   $161 = HEAP32[$pX>>2]|0;
   $162 = (_sqlite3ResolveExprNames($sNC,$161)|0);
   $163 = ($162|0)!=(0);
   if ($163) {
    label = 31;
    break L12;
   }
   $164 = HEAP32[$i>>2]|0;
   $165 = (($164) + 1)|0;
   HEAP32[$i>>2] = $165;
  }
  $166 = HEAP32[$2>>2]|0;
  $167 = (($166) + 40|0);
  $168 = HEAP32[$167>>2]|0;
  HEAP32[$pGroupBy>>2] = $168;
  $169 = HEAP32[$pGroupBy>>2]|0;
  $170 = ($169|0)!=(0|0);
  if ($170) {
   label = 35;
  } else {
   $171 = (($sNC) + 28|0);
   $172 = HEAP16[$171>>1]|0;
   $173 = $172&65535;
   $174 = $173 & 2;
   $175 = ($174|0)!=(0);
   if ($175) {
    label = 35;
   } else {
    $187 = (($sNC) + 28|0);
    $188 = HEAP16[$187>>1]|0;
    $189 = $188&65535;
    $190 = $189 & -2;
    $191 = $190&65535;
    HEAP16[$187>>1] = $191;
   }
  }
  if ((label|0) == 35) {
   label = 0;
   $176 = (($sNC) + 28|0);
   $177 = HEAP16[$176>>1]|0;
   $178 = $177&65535;
   $179 = $178 & 4096;
   $180 = 4 | $179;
   $181 = HEAP32[$2>>2]|0;
   $182 = (($181) + 6|0);
   $183 = HEAP16[$182>>1]|0;
   $184 = $183&65535;
   $185 = $184 | $180;
   $186 = $185&65535;
   HEAP16[$182>>1] = $186;
  }
  $192 = HEAP32[$2>>2]|0;
  $193 = (($192) + 44|0);
  $194 = HEAP32[$193>>2]|0;
  $195 = ($194|0)==(0|0);
  $196 = HEAP32[$pGroupBy>>2]|0;
  $197 = ($196|0)!=(0|0);
  $or$cond = $195 | $197;
  if (!($or$cond)) {
   label = 38;
   break;
  }
  $199 = HEAP32[$2>>2]|0;
  $200 = HEAP32[$199>>2]|0;
  $201 = (($sNC) + 8|0);
  HEAP32[$201>>2] = $200;
  $202 = HEAP32[$2>>2]|0;
  $203 = (($202) + 44|0);
  $204 = HEAP32[$203>>2]|0;
  $205 = (_sqlite3ResolveExprNames($sNC,$204)|0);
  $206 = ($205|0)!=(0);
  if ($206) {
   label = 40;
   break;
  }
  $207 = HEAP32[$2>>2]|0;
  $208 = (($207) + 36|0);
  $209 = HEAP32[$208>>2]|0;
  $210 = (_sqlite3ResolveExprNames($sNC,$209)|0);
  $211 = ($210|0)!=(0);
  if ($211) {
   label = 42;
   break;
  }
  $212 = (($sNC) + 16|0);
  HEAP32[$212>>2] = 0;
  $213 = (($sNC) + 28|0);
  $214 = HEAP16[$213>>1]|0;
  $215 = $214&65535;
  $216 = $215 | 1;
  $217 = $216&65535;
  HEAP16[$213>>1] = $217;
  $218 = HEAP32[$isCompound>>2]|0;
  $219 = ($218|0)!=(0);
  if (!($219)) {
   $220 = HEAP32[$2>>2]|0;
   $221 = HEAP32[$2>>2]|0;
   $222 = (($221) + 48|0);
   $223 = HEAP32[$222>>2]|0;
   $224 = (_resolveOrderGroupBy($sNC,$220,$223,16480)|0);
   $225 = ($224|0)!=(0);
   if ($225) {
    label = 45;
    break;
   }
  }
  $226 = HEAP32[$db>>2]|0;
  $227 = (($226) + 64|0);
  $228 = HEAP8[$227>>0]|0;
  $229 = ($228<<24>>24)!=(0);
  if ($229) {
   label = 47;
   break;
  }
  $230 = HEAP32[$pGroupBy>>2]|0;
  $231 = ($230|0)!=(0|0);
  L55: do {
   if ($231) {
    $232 = HEAP32[$2>>2]|0;
    $233 = HEAP32[$pGroupBy>>2]|0;
    $234 = (_resolveOrderGroupBy($sNC,$232,$233,16488)|0);
    $235 = ($234|0)!=(0);
    if ($235) {
     label = 51;
     break L12;
    }
    $236 = HEAP32[$db>>2]|0;
    $237 = (($236) + 64|0);
    $238 = HEAP8[$237>>0]|0;
    $239 = $238&255;
    $240 = ($239|0)!=(0);
    if ($240) {
     label = 51;
     break L12;
    }
    HEAP32[$i>>2] = 0;
    $241 = HEAP32[$pGroupBy>>2]|0;
    $242 = (($241) + 4|0);
    $243 = HEAP32[$242>>2]|0;
    HEAP32[$pItem1>>2] = $243;
    while(1) {
     $244 = HEAP32[$i>>2]|0;
     $245 = HEAP32[$pGroupBy>>2]|0;
     $246 = HEAP32[$245>>2]|0;
     $247 = ($244|0)<($246|0);
     if (!($247)) {
      break L55;
     }
     $248 = HEAP32[$pItem1>>2]|0;
     $249 = HEAP32[$248>>2]|0;
     $250 = (($249) + 4|0);
     $251 = HEAP32[$250>>2]|0;
     $252 = $251 & 2;
     $253 = ($252|0)!=(0);
     if ($253) {
      label = 55;
      break L12;
     }
     $255 = HEAP32[$i>>2]|0;
     $256 = (($255) + 1)|0;
     HEAP32[$i>>2] = $256;
     $257 = HEAP32[$pItem1>>2]|0;
     $258 = (($257) + 20|0);
     HEAP32[$pItem1>>2] = $258;
    }
   }
  } while(0);
  $259 = HEAP32[$2>>2]|0;
  $260 = (($259) + 52|0);
  $261 = HEAP32[$260>>2]|0;
  HEAP32[$2>>2] = $261;
  $262 = HEAP32[$nCompound>>2]|0;
  $263 = (($262) + 1)|0;
  HEAP32[$nCompound>>2] = $263;
 }
 switch (label|0) {
  case 11: {
   HEAP32[$0>>2] = 2;
   $270 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($270|0);
   break;
  }
  case 22: {
   HEAP32[$0>>2] = 2;
   $270 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($270|0);
   break;
  }
  case 31: {
   HEAP32[$0>>2] = 2;
   $270 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($270|0);
   break;
  }
  case 38: {
   $198 = HEAP32[$pParse>>2]|0;
   _sqlite3ErrorMsg($198,16432,$vararg_buffer2);
   HEAP32[$0>>2] = 2;
   $270 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($270|0);
   break;
  }
  case 40: {
   HEAP32[$0>>2] = 2;
   $270 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($270|0);
   break;
  }
  case 42: {
   HEAP32[$0>>2] = 2;
   $270 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($270|0);
   break;
  }
  case 45: {
   HEAP32[$0>>2] = 2;
   $270 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($270|0);
   break;
  }
  case 47: {
   HEAP32[$0>>2] = 2;
   $270 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($270|0);
   break;
  }
  case 51: {
   HEAP32[$0>>2] = 2;
   $270 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($270|0);
   break;
  }
  case 55: {
   $254 = HEAP32[$pParse>>2]|0;
   _sqlite3ErrorMsg($254,16496,$vararg_buffer2);
   HEAP32[$0>>2] = 2;
   $270 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($270|0);
   break;
  }
  case 58: {
   $264 = HEAP32[$isCompound>>2]|0;
   $265 = ($264|0)!=(0);
   if ($265) {
    $266 = HEAP32[$pParse>>2]|0;
    $267 = HEAP32[$pLeftmost>>2]|0;
    $268 = (_resolveCompoundOrderBy($266,$267)|0);
    $269 = ($268|0)!=(0);
    if ($269) {
     HEAP32[$0>>2] = 2;
     $270 = HEAP32[$0>>2]|0;
     STACKTOP = sp;return ($270|0);
    }
   }
   HEAP32[$0>>2] = 1;
   $270 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($270|0);
   break;
  }
 }
 return 0|0;
}
function _sqlite3ResolveExprNames($pNC,$pExpr) {
 $pNC = $pNC|0;
 $pExpr = $pExpr|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0;
 var $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $pParse = 0, $savedHasAgg = 0, $w = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $0 = sp + 40|0;
 $1 = sp + 36|0;
 $2 = sp + 32|0;
 $savedHasAgg = sp + 44|0;
 $w = sp + 8|0;
 $pParse = sp;
 HEAP32[$1>>2] = $pNC;
 HEAP32[$2>>2] = $pExpr;
 $3 = HEAP32[$2>>2]|0;
 $4 = ($3|0)==(0|0);
 if ($4) {
  HEAP32[$0>>2] = 0;
  $89 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($89|0);
 }
 $5 = HEAP32[$1>>2]|0;
 $6 = HEAP32[$5>>2]|0;
 HEAP32[$pParse>>2] = $6;
 $7 = HEAP32[$pParse>>2]|0;
 $8 = HEAP32[$2>>2]|0;
 $9 = (($8) + 24|0);
 $10 = HEAP32[$9>>2]|0;
 $11 = HEAP32[$1>>2]|0;
 $12 = HEAP32[$11>>2]|0;
 $13 = (($12) + 460|0);
 $14 = HEAP32[$13>>2]|0;
 $15 = (($10) + ($14))|0;
 $16 = (_sqlite3ExprCheckHeight($7,$15)|0);
 $17 = ($16|0)!=(0);
 if ($17) {
  HEAP32[$0>>2] = 1;
  $89 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($89|0);
 }
 $18 = HEAP32[$2>>2]|0;
 $19 = (($18) + 24|0);
 $20 = HEAP32[$19>>2]|0;
 $21 = HEAP32[$pParse>>2]|0;
 $22 = (($21) + 460|0);
 $23 = HEAP32[$22>>2]|0;
 $24 = (($23) + ($20))|0;
 HEAP32[$22>>2] = $24;
 $25 = HEAP32[$1>>2]|0;
 $26 = (($25) + 28|0);
 $27 = HEAP16[$26>>1]|0;
 $28 = $27&65535;
 $29 = $28 & 4098;
 $30 = $29&65535;
 HEAP16[$savedHasAgg>>1] = $30;
 $31 = HEAP32[$1>>2]|0;
 $32 = (($31) + 28|0);
 $33 = HEAP16[$32>>1]|0;
 $34 = $33&65535;
 $35 = $34 & -4099;
 $36 = $35&65535;
 HEAP16[$32>>1] = $36;
 ;HEAP32[$w+0>>2]=0|0;HEAP32[$w+4>>2]=0|0;HEAP32[$w+8>>2]=0|0;HEAP32[$w+12>>2]=0|0;HEAP32[$w+16>>2]=0|0;HEAP32[$w+20>>2]=0|0;
 HEAP32[$w>>2] = 40;
 $37 = (($w) + 4|0);
 HEAP32[$37>>2] = 41;
 $38 = HEAP32[$1>>2]|0;
 $39 = HEAP32[$38>>2]|0;
 $40 = (($w) + 12|0);
 HEAP32[$40>>2] = $39;
 $41 = HEAP32[$1>>2]|0;
 $42 = (($w) + 20|0);
 HEAP32[$42>>2] = $41;
 $43 = HEAP32[$2>>2]|0;
 (_sqlite3WalkExpr($w,$43)|0);
 $44 = HEAP32[$2>>2]|0;
 $45 = (($44) + 24|0);
 $46 = HEAP32[$45>>2]|0;
 $47 = HEAP32[$1>>2]|0;
 $48 = HEAP32[$47>>2]|0;
 $49 = (($48) + 460|0);
 $50 = HEAP32[$49>>2]|0;
 $51 = (($50) - ($46))|0;
 HEAP32[$49>>2] = $51;
 $52 = HEAP32[$1>>2]|0;
 $53 = (($52) + 24|0);
 $54 = HEAP32[$53>>2]|0;
 $55 = ($54|0)>(0);
 if ($55) {
  label = 7;
 } else {
  $56 = (($w) + 12|0);
  $57 = HEAP32[$56>>2]|0;
  $58 = (($57) + 64|0);
  $59 = HEAP32[$58>>2]|0;
  $60 = ($59|0)>(0);
  if ($60) {
   label = 7;
  }
 }
 if ((label|0) == 7) {
  $61 = HEAP32[$2>>2]|0;
  $62 = (($61) + 4|0);
  $63 = HEAP32[$62>>2]|0;
  $64 = $63 | 8;
  HEAP32[$62>>2] = $64;
 }
 $65 = HEAP32[$1>>2]|0;
 $66 = (($65) + 28|0);
 $67 = HEAP16[$66>>1]|0;
 $68 = $67&65535;
 $69 = $68 & 2;
 $70 = ($69|0)!=(0);
 if ($70) {
  $71 = HEAP32[$2>>2]|0;
  $72 = (($71) + 4|0);
  $73 = HEAP32[$72>>2]|0;
  $74 = $73 | 2;
  HEAP32[$72>>2] = $74;
 }
 $75 = HEAP16[$savedHasAgg>>1]|0;
 $76 = $75&65535;
 $77 = HEAP32[$1>>2]|0;
 $78 = (($77) + 28|0);
 $79 = HEAP16[$78>>1]|0;
 $80 = $79&65535;
 $81 = $80 | $76;
 $82 = $81&65535;
 HEAP16[$78>>1] = $82;
 $83 = HEAP32[$2>>2]|0;
 $84 = (($83) + 4|0);
 $85 = HEAP32[$84>>2]|0;
 $86 = $85 & 8;
 $87 = ($86|0)!=(0);
 $88 = $87&1;
 HEAP32[$0>>2] = $88;
 $89 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($89|0);
}
function _resolveOrderGroupBy($pNC,$pSelect,$pOrderBy,$zType) {
 $pNC = $pNC|0;
 $pSelect = $pSelect|0;
 $pOrderBy = $pOrderBy|0;
 $zType = $zType|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0;
 var $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $i = 0, $iCol = 0, $j = 0, $nResult = 0, $or$cond = 0, $pE = 0, $pE2 = 0, $pItem = 0;
 var $pParse = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 64|0;
 $0 = sp + 48|0;
 $1 = sp + 44|0;
 $2 = sp + 40|0;
 $3 = sp + 36|0;
 $4 = sp + 32|0;
 $i = sp + 28|0;
 $j = sp + 24|0;
 $iCol = sp + 20|0;
 $pItem = sp + 16|0;
 $pParse = sp + 12|0;
 $nResult = sp + 8|0;
 $pE = sp + 4|0;
 $pE2 = sp;
 HEAP32[$1>>2] = $pNC;
 HEAP32[$2>>2] = $pSelect;
 HEAP32[$3>>2] = $pOrderBy;
 HEAP32[$4>>2] = $zType;
 $5 = HEAP32[$3>>2]|0;
 $6 = ($5|0)==(0|0);
 if ($6) {
  HEAP32[$0>>2] = 0;
  $91 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($91|0);
 }
 $7 = HEAP32[$2>>2]|0;
 $8 = HEAP32[$7>>2]|0;
 $9 = HEAP32[$8>>2]|0;
 HEAP32[$nResult>>2] = $9;
 $10 = HEAP32[$1>>2]|0;
 $11 = HEAP32[$10>>2]|0;
 HEAP32[$pParse>>2] = $11;
 HEAP32[$i>>2] = 0;
 $12 = HEAP32[$3>>2]|0;
 $13 = (($12) + 4|0);
 $14 = HEAP32[$13>>2]|0;
 HEAP32[$pItem>>2] = $14;
 L5: while(1) {
  $15 = HEAP32[$i>>2]|0;
  $16 = HEAP32[$3>>2]|0;
  $17 = HEAP32[$16>>2]|0;
  $18 = ($15|0)<($17|0);
  if (!($18)) {
   label = 20;
   break;
  }
  $19 = HEAP32[$pItem>>2]|0;
  $20 = HEAP32[$19>>2]|0;
  HEAP32[$pE>>2] = $20;
  $21 = HEAP32[$pE>>2]|0;
  $22 = (_sqlite3ExprSkipCollate($21)|0);
  HEAP32[$pE2>>2] = $22;
  $23 = HEAP32[$4>>2]|0;
  $24 = HEAP8[$23>>0]|0;
  $25 = $24 << 24 >> 24;
  $26 = ($25|0)!=(71);
  if ($26) {
   $27 = HEAP32[$pParse>>2]|0;
   $28 = HEAP32[$2>>2]|0;
   $29 = HEAP32[$28>>2]|0;
   $30 = HEAP32[$pE2>>2]|0;
   $31 = (_resolveAsName($27,$29,$30)|0);
   HEAP32[$iCol>>2] = $31;
   $32 = HEAP32[$iCol>>2]|0;
   $33 = ($32|0)>(0);
   if ($33) {
    $34 = HEAP32[$iCol>>2]|0;
    $35 = $34&65535;
    $36 = HEAP32[$pItem>>2]|0;
    $37 = (($36) + 16|0);
    HEAP16[$37>>1] = $35;
   } else {
    label = 8;
   }
  } else {
   label = 8;
  }
  L11: do {
   if ((label|0) == 8) {
    label = 0;
    $38 = HEAP32[$pE2>>2]|0;
    $39 = (_sqlite3ExprIsInteger($38,$iCol)|0);
    $40 = ($39|0)!=(0);
    if ($40) {
     $41 = HEAP32[$iCol>>2]|0;
     $42 = ($41|0)<(1);
     $43 = HEAP32[$iCol>>2]|0;
     $44 = ($43|0)>(65535);
     $or$cond = $42 | $44;
     if ($or$cond) {
      label = 10;
      break L5;
     }
     $50 = HEAP32[$iCol>>2]|0;
     $51 = $50&65535;
     $52 = HEAP32[$pItem>>2]|0;
     $53 = (($52) + 16|0);
     HEAP16[$53>>1] = $51;
     break;
    }
    $54 = HEAP32[$pItem>>2]|0;
    $55 = (($54) + 16|0);
    HEAP16[$55>>1] = 0;
    $56 = HEAP32[$1>>2]|0;
    $57 = HEAP32[$pE>>2]|0;
    $58 = (_sqlite3ResolveExprNames($56,$57)|0);
    $59 = ($58|0)!=(0);
    if ($59) {
     label = 13;
     break L5;
    }
    HEAP32[$j>>2] = 0;
    while(1) {
     $60 = HEAP32[$j>>2]|0;
     $61 = HEAP32[$2>>2]|0;
     $62 = HEAP32[$61>>2]|0;
     $63 = HEAP32[$62>>2]|0;
     $64 = ($60|0)<($63|0);
     if (!($64)) {
      break L11;
     }
     $65 = HEAP32[$pE>>2]|0;
     $66 = HEAP32[$j>>2]|0;
     $67 = HEAP32[$2>>2]|0;
     $68 = HEAP32[$67>>2]|0;
     $69 = (($68) + 4|0);
     $70 = HEAP32[$69>>2]|0;
     $71 = (($70) + (($66*20)|0)|0);
     $72 = HEAP32[$71>>2]|0;
     $73 = (_sqlite3ExprCompare($65,$72,-1)|0);
     $74 = ($73|0)==(0);
     if ($74) {
      $75 = HEAP32[$j>>2]|0;
      $76 = (($75) + 1)|0;
      $77 = $76&65535;
      $78 = HEAP32[$pItem>>2]|0;
      $79 = (($78) + 16|0);
      HEAP16[$79>>1] = $77;
     }
     $80 = HEAP32[$j>>2]|0;
     $81 = (($80) + 1)|0;
     HEAP32[$j>>2] = $81;
    }
   }
  } while(0);
  $82 = HEAP32[$i>>2]|0;
  $83 = (($82) + 1)|0;
  HEAP32[$i>>2] = $83;
  $84 = HEAP32[$pItem>>2]|0;
  $85 = (($84) + 20|0);
  HEAP32[$pItem>>2] = $85;
 }
 if ((label|0) == 10) {
  $45 = HEAP32[$pParse>>2]|0;
  $46 = HEAP32[$4>>2]|0;
  $47 = HEAP32[$i>>2]|0;
  $48 = (($47) + 1)|0;
  $49 = HEAP32[$nResult>>2]|0;
  _resolveOutOfRangeError($45,$46,$48,$49);
  HEAP32[$0>>2] = 1;
  $91 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($91|0);
 }
 else if ((label|0) == 13) {
  HEAP32[$0>>2] = 1;
  $91 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($91|0);
 }
 else if ((label|0) == 20) {
  $86 = HEAP32[$pParse>>2]|0;
  $87 = HEAP32[$2>>2]|0;
  $88 = HEAP32[$3>>2]|0;
  $89 = HEAP32[$4>>2]|0;
  $90 = (_sqlite3ResolveOrderGroupBy($86,$87,$88,$89)|0);
  HEAP32[$0>>2] = $90;
  $91 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($91|0);
 }
 return 0|0;
}
function _resolveCompoundOrderBy($pParse,$pSelect) {
 $pParse = $pParse|0;
 $pSelect = $pSelect|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0;
 var $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0;
 var $db = 0, $i = 0, $iCol = 0, $moreToDo = 0, $or$cond = 0, $pDup = 0, $pE = 0, $pEList = 0, $pItem = 0, $pNew = 0, $pOrderBy = 0, $vararg_buffer2 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 64|0;
 $vararg_buffer2 = sp;
 $0 = sp + 52|0;
 $1 = sp + 48|0;
 $2 = sp + 44|0;
 $i = sp + 40|0;
 $pOrderBy = sp + 36|0;
 $pEList = sp + 32|0;
 $db = sp + 28|0;
 $moreToDo = sp + 24|0;
 $pItem = sp + 20|0;
 $iCol = sp + 16|0;
 $pE = sp + 12|0;
 $pDup = sp + 8|0;
 $pNew = sp + 4|0;
 HEAP32[$1>>2] = $pParse;
 HEAP32[$2>>2] = $pSelect;
 HEAP32[$moreToDo>>2] = 1;
 $3 = HEAP32[$2>>2]|0;
 $4 = (($3) + 48|0);
 $5 = HEAP32[$4>>2]|0;
 HEAP32[$pOrderBy>>2] = $5;
 $6 = HEAP32[$pOrderBy>>2]|0;
 $7 = ($6|0)==(0|0);
 if ($7) {
  HEAP32[$0>>2] = 0;
  $159 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($159|0);
 }
 $8 = HEAP32[$1>>2]|0;
 $9 = HEAP32[$8>>2]|0;
 HEAP32[$db>>2] = $9;
 $10 = HEAP32[$pOrderBy>>2]|0;
 $11 = HEAP32[$10>>2]|0;
 $12 = HEAP32[$db>>2]|0;
 $13 = (($12) + 88|0);
 $14 = (($13) + 8|0);
 $15 = HEAP32[$14>>2]|0;
 $16 = ($11|0)>($15|0);
 if ($16) {
  $17 = HEAP32[$1>>2]|0;
  _sqlite3ErrorMsg($17,16560,$vararg_buffer2);
  HEAP32[$0>>2] = 1;
  $159 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($159|0);
 }
 HEAP32[$i>>2] = 0;
 while(1) {
  $18 = HEAP32[$i>>2]|0;
  $19 = HEAP32[$pOrderBy>>2]|0;
  $20 = HEAP32[$19>>2]|0;
  $21 = ($18|0)<($20|0);
  if (!($21)) {
   break;
  }
  $22 = HEAP32[$i>>2]|0;
  $23 = HEAP32[$pOrderBy>>2]|0;
  $24 = (($23) + 4|0);
  $25 = HEAP32[$24>>2]|0;
  $26 = (($25) + (($22*20)|0)|0);
  $27 = (($26) + 13|0);
  $28 = HEAP8[$27>>0]|0;
  $29 = $28 & -2;
  HEAP8[$27>>0] = $29;
  $30 = HEAP32[$i>>2]|0;
  $31 = (($30) + 1)|0;
  HEAP32[$i>>2] = $31;
 }
 $32 = HEAP32[$2>>2]|0;
 $33 = (($32) + 56|0);
 HEAP32[$33>>2] = 0;
 while(1) {
  $34 = HEAP32[$2>>2]|0;
  $35 = (($34) + 52|0);
  $36 = HEAP32[$35>>2]|0;
  $37 = ($36|0)!=(0|0);
  if (!($37)) {
   break;
  }
  $38 = HEAP32[$2>>2]|0;
  $39 = HEAP32[$2>>2]|0;
  $40 = (($39) + 52|0);
  $41 = HEAP32[$40>>2]|0;
  $42 = (($41) + 56|0);
  HEAP32[$42>>2] = $38;
  $43 = HEAP32[$2>>2]|0;
  $44 = (($43) + 52|0);
  $45 = HEAP32[$44>>2]|0;
  HEAP32[$2>>2] = $45;
 }
 L16: while(1) {
  $46 = HEAP32[$2>>2]|0;
  $47 = ($46|0)!=(0|0);
  $48 = HEAP32[$moreToDo>>2]|0;
  $49 = ($48|0)!=(0);
  $or$cond = $47 & $49;
  if (!($or$cond)) {
   label = 33;
   break;
  }
  HEAP32[$moreToDo>>2] = 0;
  $50 = HEAP32[$2>>2]|0;
  $51 = HEAP32[$50>>2]|0;
  HEAP32[$pEList>>2] = $51;
  HEAP32[$i>>2] = 0;
  $52 = HEAP32[$pOrderBy>>2]|0;
  $53 = (($52) + 4|0);
  $54 = HEAP32[$53>>2]|0;
  HEAP32[$pItem>>2] = $54;
  while(1) {
   $55 = HEAP32[$i>>2]|0;
   $56 = HEAP32[$pOrderBy>>2]|0;
   $57 = HEAP32[$56>>2]|0;
   $58 = ($55|0)<($57|0);
   if (!($58)) {
    break;
   }
   HEAP32[$iCol>>2] = -1;
   $59 = HEAP32[$pItem>>2]|0;
   $60 = (($59) + 13|0);
   $61 = HEAP8[$60>>0]|0;
   $62 = $61 & 1;
   $63 = $62&255;
   $64 = ($63|0)!=(0);
   do {
    if (!($64)) {
     $65 = HEAP32[$pItem>>2]|0;
     $66 = HEAP32[$65>>2]|0;
     $67 = (_sqlite3ExprSkipCollate($66)|0);
     HEAP32[$pE>>2] = $67;
     $68 = HEAP32[$pE>>2]|0;
     $69 = (_sqlite3ExprIsInteger($68,$iCol)|0);
     $70 = ($69|0)!=(0);
     if ($70) {
      $71 = HEAP32[$iCol>>2]|0;
      $72 = ($71|0)<=(0);
      if ($72) {
       label = 18;
       break L16;
      }
      $73 = HEAP32[$iCol>>2]|0;
      $74 = HEAP32[$pEList>>2]|0;
      $75 = HEAP32[$74>>2]|0;
      $76 = ($73|0)>($75|0);
      if ($76) {
       label = 18;
       break L16;
      }
     } else {
      $82 = HEAP32[$1>>2]|0;
      $83 = HEAP32[$pEList>>2]|0;
      $84 = HEAP32[$pE>>2]|0;
      $85 = (_resolveAsName($82,$83,$84)|0);
      HEAP32[$iCol>>2] = $85;
      $86 = HEAP32[$iCol>>2]|0;
      $87 = ($86|0)==(0);
      if ($87) {
       $88 = HEAP32[$db>>2]|0;
       $89 = HEAP32[$pE>>2]|0;
       $90 = (_sqlite3ExprDup($88,$89,0)|0);
       HEAP32[$pDup>>2] = $90;
       $91 = HEAP32[$db>>2]|0;
       $92 = (($91) + 64|0);
       $93 = HEAP8[$92>>0]|0;
       $94 = ($93<<24>>24)!=(0);
       if (!($94)) {
        $95 = HEAP32[$1>>2]|0;
        $96 = HEAP32[$2>>2]|0;
        $97 = HEAP32[$pDup>>2]|0;
        $98 = (_resolveOrderByTermToExprList($95,$96,$97)|0);
        HEAP32[$iCol>>2] = $98;
       }
       $99 = HEAP32[$db>>2]|0;
       $100 = HEAP32[$pDup>>2]|0;
       _sqlite3ExprDelete($99,$100);
      }
     }
     $101 = HEAP32[$iCol>>2]|0;
     $102 = ($101|0)>(0);
     if (!($102)) {
      HEAP32[$moreToDo>>2] = 1;
      break;
     }
     $103 = HEAP32[$db>>2]|0;
     $104 = (_sqlite3Expr($103,132,0)|0);
     HEAP32[$pNew>>2] = $104;
     $105 = HEAP32[$pNew>>2]|0;
     $106 = ($105|0)==(0|0);
     if ($106) {
      label = 25;
      break L16;
     }
     $107 = HEAP32[$pNew>>2]|0;
     $108 = (($107) + 4|0);
     $109 = HEAP32[$108>>2]|0;
     $110 = $109 | 1024;
     HEAP32[$108>>2] = $110;
     $111 = HEAP32[$iCol>>2]|0;
     $112 = HEAP32[$pNew>>2]|0;
     $113 = (($112) + 8|0);
     HEAP32[$113>>2] = $111;
     $114 = HEAP32[$pItem>>2]|0;
     $115 = HEAP32[$114>>2]|0;
     $116 = HEAP32[$pE>>2]|0;
     $117 = ($115|0)==($116|0);
     $118 = HEAP32[$pNew>>2]|0;
     $119 = HEAP32[$pItem>>2]|0;
     if ($117) {
      HEAP32[$119>>2] = $118;
     } else {
      $120 = HEAP32[$119>>2]|0;
      $121 = (($120) + 12|0);
      HEAP32[$121>>2] = $118;
     }
     $122 = HEAP32[$db>>2]|0;
     $123 = HEAP32[$pE>>2]|0;
     _sqlite3ExprDelete($122,$123);
     $124 = HEAP32[$iCol>>2]|0;
     $125 = $124&65535;
     $126 = HEAP32[$pItem>>2]|0;
     $127 = (($126) + 16|0);
     HEAP16[$127>>1] = $125;
     $128 = HEAP32[$pItem>>2]|0;
     $129 = (($128) + 13|0);
     $130 = HEAP8[$129>>0]|0;
     $131 = $130 & -2;
     $132 = $131 | 1;
     HEAP8[$129>>0] = $132;
    }
   } while(0);
   $133 = HEAP32[$i>>2]|0;
   $134 = (($133) + 1)|0;
   HEAP32[$i>>2] = $134;
   $135 = HEAP32[$pItem>>2]|0;
   $136 = (($135) + 20|0);
   HEAP32[$pItem>>2] = $136;
  }
  $137 = HEAP32[$2>>2]|0;
  $138 = (($137) + 56|0);
  $139 = HEAP32[$138>>2]|0;
  HEAP32[$2>>2] = $139;
 }
 if ((label|0) == 18) {
  $77 = HEAP32[$1>>2]|0;
  $78 = HEAP32[$i>>2]|0;
  $79 = (($78) + 1)|0;
  $80 = HEAP32[$pEList>>2]|0;
  $81 = HEAP32[$80>>2]|0;
  _resolveOutOfRangeError($77,16480,$79,$81);
  HEAP32[$0>>2] = 1;
  $159 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($159|0);
 }
 else if ((label|0) == 25) {
  HEAP32[$0>>2] = 1;
  $159 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($159|0);
 }
 else if ((label|0) == 33) {
  HEAP32[$i>>2] = 0;
  while(1) {
   $140 = HEAP32[$i>>2]|0;
   $141 = HEAP32[$pOrderBy>>2]|0;
   $142 = HEAP32[$141>>2]|0;
   $143 = ($140|0)<($142|0);
   if (!($143)) {
    label = 38;
    break;
   }
   $144 = HEAP32[$i>>2]|0;
   $145 = HEAP32[$pOrderBy>>2]|0;
   $146 = (($145) + 4|0);
   $147 = HEAP32[$146>>2]|0;
   $148 = (($147) + (($144*20)|0)|0);
   $149 = (($148) + 13|0);
   $150 = HEAP8[$149>>0]|0;
   $151 = $150 & 1;
   $152 = $151&255;
   $153 = ($152|0)==(0);
   if ($153) {
    label = 36;
    break;
   }
   $157 = HEAP32[$i>>2]|0;
   $158 = (($157) + 1)|0;
   HEAP32[$i>>2] = $158;
  }
  if ((label|0) == 36) {
   $154 = HEAP32[$1>>2]|0;
   $155 = HEAP32[$i>>2]|0;
   $156 = (($155) + 1)|0;
   HEAP32[$vararg_buffer2>>2] = $156;
   _sqlite3ErrorMsg($154,16600,$vararg_buffer2);
   HEAP32[$0>>2] = 1;
   $159 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($159|0);
  }
  else if ((label|0) == 38) {
   HEAP32[$0>>2] = 0;
   $159 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($159|0);
  }
 }
 return 0|0;
}
function _sqlite3ExprIsInteger($p,$pValue) {
 $p = $p|0;
 $pValue = $pValue|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $rc = 0, $v = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 16|0;
 $1 = sp + 12|0;
 $2 = sp + 8|0;
 $rc = sp + 4|0;
 $v = sp;
 HEAP32[$1>>2] = $p;
 HEAP32[$2>>2] = $pValue;
 HEAP32[$rc>>2] = 0;
 $3 = HEAP32[$1>>2]|0;
 $4 = (($3) + 4|0);
 $5 = HEAP32[$4>>2]|0;
 $6 = $5 & 1024;
 $7 = ($6|0)!=(0);
 $8 = HEAP32[$1>>2]|0;
 if ($7) {
  $9 = (($8) + 8|0);
  $10 = HEAP32[$9>>2]|0;
  $11 = HEAP32[$2>>2]|0;
  HEAP32[$11>>2] = $10;
  HEAP32[$0>>2] = 1;
  $28 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($28|0);
 }
 $12 = HEAP8[$8>>0]|0;
 $13 = $12&255;
 if ((($13|0) == 158)) {
  $14 = HEAP32[$1>>2]|0;
  $15 = (($14) + 12|0);
  $16 = HEAP32[$15>>2]|0;
  $17 = HEAP32[$2>>2]|0;
  $18 = (_sqlite3ExprIsInteger($16,$17)|0);
  HEAP32[$rc>>2] = $18;
 } else if ((($13|0) == 157)) {
  $19 = HEAP32[$1>>2]|0;
  $20 = (($19) + 12|0);
  $21 = HEAP32[$20>>2]|0;
  $22 = (_sqlite3ExprIsInteger($21,$v)|0);
  $23 = ($22|0)!=(0);
  if ($23) {
   $24 = HEAP32[$v>>2]|0;
   $25 = (0 - ($24))|0;
   $26 = HEAP32[$2>>2]|0;
   HEAP32[$26>>2] = $25;
   HEAP32[$rc>>2] = 1;
  }
 }
 $27 = HEAP32[$rc>>2]|0;
 HEAP32[$0>>2] = $27;
 $28 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($28|0);
}
function _resolveOutOfRangeError($pParse,$zType,$i,$mx) {
 $pParse = $pParse|0;
 $zType = $zType|0;
 $i = $i|0;
 $mx = $mx|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $vararg_buffer = 0, $vararg_ptr1 = 0, $vararg_ptr2 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $vararg_buffer = sp;
 $0 = sp + 24|0;
 $1 = sp + 20|0;
 $2 = sp + 16|0;
 $3 = sp + 12|0;
 HEAP32[$0>>2] = $pParse;
 HEAP32[$1>>2] = $zType;
 HEAP32[$2>>2] = $i;
 HEAP32[$3>>2] = $mx;
 $4 = HEAP32[$0>>2]|0;
 $5 = HEAP32[$2>>2]|0;
 $6 = HEAP32[$1>>2]|0;
 $7 = HEAP32[$3>>2]|0;
 HEAP32[$vararg_buffer>>2] = $5;
 $vararg_ptr1 = (($vararg_buffer) + 4|0);
 HEAP32[$vararg_ptr1>>2] = $6;
 $vararg_ptr2 = (($vararg_buffer) + 8|0);
 HEAP32[$vararg_ptr2>>2] = $7;
 _sqlite3ErrorMsg($4,16664,$vararg_buffer);
 STACKTOP = sp;return;
}
function _resolveAsName($pParse,$pEList,$pE) {
 $pParse = $pParse|0;
 $pEList = $pEList|0;
 $pE = $pE|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $i = 0, $zAs = 0, $zCol = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 24|0;
 $1 = sp + 20|0;
 $2 = sp + 16|0;
 $3 = sp + 12|0;
 $i = sp + 8|0;
 $zCol = sp + 4|0;
 $zAs = sp;
 HEAP32[$1>>2] = $pParse;
 HEAP32[$2>>2] = $pEList;
 HEAP32[$3>>2] = $pE;
 $4 = HEAP32[$3>>2]|0;
 $5 = HEAP8[$4>>0]|0;
 $6 = $5&255;
 $7 = ($6|0)==(27);
 L1: do {
  if ($7) {
   $8 = HEAP32[$3>>2]|0;
   $9 = (($8) + 8|0);
   $10 = HEAP32[$9>>2]|0;
   HEAP32[$zCol>>2] = $10;
   HEAP32[$i>>2] = 0;
   while(1) {
    $11 = HEAP32[$i>>2]|0;
    $12 = HEAP32[$2>>2]|0;
    $13 = HEAP32[$12>>2]|0;
    $14 = ($11|0)<($13|0);
    if (!($14)) {
     break L1;
    }
    $15 = HEAP32[$i>>2]|0;
    $16 = HEAP32[$2>>2]|0;
    $17 = (($16) + 4|0);
    $18 = HEAP32[$17>>2]|0;
    $19 = (($18) + (($15*20)|0)|0);
    $20 = (($19) + 4|0);
    $21 = HEAP32[$20>>2]|0;
    HEAP32[$zAs>>2] = $21;
    $22 = HEAP32[$zAs>>2]|0;
    $23 = ($22|0)!=(0|0);
    if ($23) {
     $24 = HEAP32[$zAs>>2]|0;
     $25 = HEAP32[$zCol>>2]|0;
     $26 = (_sqlite3_stricmp($24,$25)|0);
     $27 = ($26|0)==(0);
     if ($27) {
      break;
     }
    }
    $30 = HEAP32[$i>>2]|0;
    $31 = (($30) + 1)|0;
    HEAP32[$i>>2] = $31;
   }
   $28 = HEAP32[$i>>2]|0;
   $29 = (($28) + 1)|0;
   HEAP32[$0>>2] = $29;
   $32 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($32|0);
  }
 } while(0);
 HEAP32[$0>>2] = 0;
 $32 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($32|0);
}
function _sqlite3ExprDup($db,$p,$flags) {
 $db = $db|0;
 $p = $p|0;
 $flags = $flags|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $1 = sp + 4|0;
 $2 = sp;
 HEAP32[$0>>2] = $db;
 HEAP32[$1>>2] = $p;
 HEAP32[$2>>2] = $flags;
 $3 = HEAP32[$0>>2]|0;
 $4 = HEAP32[$1>>2]|0;
 $5 = HEAP32[$2>>2]|0;
 $6 = (_exprDup($3,$4,$5,0)|0);
 STACKTOP = sp;return ($6|0);
}
function _resolveOrderByTermToExprList($pParse,$pSelect,$pE) {
 $pParse = $pParse|0;
 $pSelect = $pSelect|0;
 $pE = $pE|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $db = 0, $i = 0, $nc = 0, $pEList = 0, $rc = 0, $savedSuppErr = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 80|0;
 $0 = sp + 60|0;
 $1 = sp + 56|0;
 $2 = sp + 52|0;
 $3 = sp + 48|0;
 $i = sp + 44|0;
 $pEList = sp + 40|0;
 $nc = sp + 8|0;
 $db = sp + 4|0;
 $rc = sp;
 $savedSuppErr = sp + 64|0;
 HEAP32[$1>>2] = $pParse;
 HEAP32[$2>>2] = $pSelect;
 HEAP32[$3>>2] = $pE;
 $4 = HEAP32[$2>>2]|0;
 $5 = HEAP32[$4>>2]|0;
 HEAP32[$pEList>>2] = $5;
 ;HEAP32[$nc+0>>2]=0|0;HEAP32[$nc+4>>2]=0|0;HEAP32[$nc+8>>2]=0|0;HEAP32[$nc+12>>2]=0|0;HEAP32[$nc+16>>2]=0|0;HEAP32[$nc+20>>2]=0|0;HEAP32[$nc+24>>2]=0|0;HEAP32[$nc+28>>2]=0|0;
 $6 = HEAP32[$1>>2]|0;
 HEAP32[$nc>>2] = $6;
 $7 = HEAP32[$2>>2]|0;
 $8 = (($7) + 32|0);
 $9 = HEAP32[$8>>2]|0;
 $10 = (($nc) + 4|0);
 HEAP32[$10>>2] = $9;
 $11 = HEAP32[$pEList>>2]|0;
 $12 = (($nc) + 8|0);
 HEAP32[$12>>2] = $11;
 $13 = (($nc) + 28|0);
 HEAP16[$13>>1] = 1;
 $14 = (($nc) + 24|0);
 HEAP32[$14>>2] = 0;
 $15 = HEAP32[$1>>2]|0;
 $16 = HEAP32[$15>>2]|0;
 HEAP32[$db>>2] = $16;
 $17 = HEAP32[$db>>2]|0;
 $18 = (($17) + 67|0);
 $19 = HEAP8[$18>>0]|0;
 HEAP8[$savedSuppErr>>0] = $19;
 $20 = HEAP32[$db>>2]|0;
 $21 = (($20) + 67|0);
 HEAP8[$21>>0] = 1;
 $22 = HEAP32[$3>>2]|0;
 $23 = (_sqlite3ResolveExprNames($nc,$22)|0);
 HEAP32[$rc>>2] = $23;
 $24 = HEAP8[$savedSuppErr>>0]|0;
 $25 = HEAP32[$db>>2]|0;
 $26 = (($25) + 67|0);
 HEAP8[$26>>0] = $24;
 $27 = HEAP32[$rc>>2]|0;
 $28 = ($27|0)!=(0);
 if ($28) {
  HEAP32[$0>>2] = 0;
  $44 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($44|0);
 }
 HEAP32[$i>>2] = 0;
 while(1) {
  $29 = HEAP32[$i>>2]|0;
  $30 = HEAP32[$pEList>>2]|0;
  $31 = HEAP32[$30>>2]|0;
  $32 = ($29|0)<($31|0);
  if (!($32)) {
   label = 8;
   break;
  }
  $33 = HEAP32[$i>>2]|0;
  $34 = HEAP32[$pEList>>2]|0;
  $35 = (($34) + 4|0);
  $36 = HEAP32[$35>>2]|0;
  $37 = (($36) + (($33*20)|0)|0);
  $38 = HEAP32[$37>>2]|0;
  $39 = HEAP32[$3>>2]|0;
  $40 = (_sqlite3ExprCompare($38,$39,-1)|0);
  $41 = ($40|0)<(2);
  $42 = HEAP32[$i>>2]|0;
  $43 = (($42) + 1)|0;
  if ($41) {
   label = 6;
   break;
  }
  HEAP32[$i>>2] = $43;
 }
 if ((label|0) == 6) {
  HEAP32[$0>>2] = $43;
  $44 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($44|0);
 }
 else if ((label|0) == 8) {
  HEAP32[$0>>2] = 0;
  $44 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($44|0);
 }
 return 0|0;
}
function _sqlite3ExprCompare($pA,$pB,$iTab) {
 $pA = $pA|0;
 $pB = $pB|0;
 $iTab = $iTab|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0;
 var $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0;
 var $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0;
 var $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0;
 var $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $combinedFlags = 0;
 var $or$cond = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 16|0;
 $1 = sp + 12|0;
 $2 = sp + 8|0;
 $3 = sp + 4|0;
 $combinedFlags = sp;
 HEAP32[$1>>2] = $pA;
 HEAP32[$2>>2] = $pB;
 HEAP32[$3>>2] = $iTab;
 $4 = HEAP32[$1>>2]|0;
 $5 = ($4|0)==(0|0);
 $6 = HEAP32[$2>>2]|0;
 $7 = ($6|0)==(0|0);
 $or$cond = $5 | $7;
 if ($or$cond) {
  $8 = HEAP32[$2>>2]|0;
  $9 = HEAP32[$1>>2]|0;
  $10 = ($8|0)==($9|0);
  $11 = $10 ? 0 : 2;
  HEAP32[$0>>2] = $11;
  $158 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($158|0);
 }
 $12 = HEAP32[$1>>2]|0;
 $13 = (($12) + 4|0);
 $14 = HEAP32[$13>>2]|0;
 $15 = HEAP32[$2>>2]|0;
 $16 = (($15) + 4|0);
 $17 = HEAP32[$16>>2]|0;
 $18 = $14 | $17;
 HEAP32[$combinedFlags>>2] = $18;
 $19 = HEAP32[$combinedFlags>>2]|0;
 $20 = $19 & 1024;
 $21 = ($20|0)!=(0);
 $22 = HEAP32[$1>>2]|0;
 if ($21) {
  $23 = (($22) + 4|0);
  $24 = HEAP32[$23>>2]|0;
  $25 = HEAP32[$2>>2]|0;
  $26 = (($25) + 4|0);
  $27 = HEAP32[$26>>2]|0;
  $28 = $24 & $27;
  $29 = $28 & 1024;
  $30 = ($29|0)!=(0);
  if ($30) {
   $31 = HEAP32[$1>>2]|0;
   $32 = (($31) + 8|0);
   $33 = HEAP32[$32>>2]|0;
   $34 = HEAP32[$2>>2]|0;
   $35 = (($34) + 8|0);
   $36 = HEAP32[$35>>2]|0;
   $37 = ($33|0)==($36|0);
   if ($37) {
    HEAP32[$0>>2] = 0;
    $158 = HEAP32[$0>>2]|0;
    STACKTOP = sp;return ($158|0);
   }
  }
  HEAP32[$0>>2] = 2;
  $158 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($158|0);
 }
 $38 = HEAP8[$22>>0]|0;
 $39 = $38&255;
 $40 = HEAP32[$2>>2]|0;
 $41 = HEAP8[$40>>0]|0;
 $42 = $41&255;
 $43 = ($39|0)!=($42|0);
 $44 = HEAP32[$1>>2]|0;
 $45 = HEAP8[$44>>0]|0;
 $46 = $45&255;
 if ($43) {
  $47 = ($46|0)==(95);
  if ($47) {
   $48 = HEAP32[$1>>2]|0;
   $49 = (($48) + 12|0);
   $50 = HEAP32[$49>>2]|0;
   $51 = HEAP32[$2>>2]|0;
   $52 = HEAP32[$3>>2]|0;
   $53 = (_sqlite3ExprCompare($50,$51,$52)|0);
   $54 = ($53|0)<(2);
   if ($54) {
    HEAP32[$0>>2] = 1;
    $158 = HEAP32[$0>>2]|0;
    STACKTOP = sp;return ($158|0);
   }
  }
  $55 = HEAP32[$2>>2]|0;
  $56 = HEAP8[$55>>0]|0;
  $57 = $56&255;
  $58 = ($57|0)==(95);
  if ($58) {
   $59 = HEAP32[$1>>2]|0;
   $60 = HEAP32[$2>>2]|0;
   $61 = (($60) + 12|0);
   $62 = HEAP32[$61>>2]|0;
   $63 = HEAP32[$3>>2]|0;
   $64 = (_sqlite3ExprCompare($59,$62,$63)|0);
   $65 = ($64|0)<(2);
   if ($65) {
    HEAP32[$0>>2] = 1;
    $158 = HEAP32[$0>>2]|0;
    STACKTOP = sp;return ($158|0);
   }
  }
  HEAP32[$0>>2] = 2;
  $158 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($158|0);
 }
 $66 = ($46|0)!=(154);
 if ($66) {
  $67 = HEAP32[$1>>2]|0;
  $68 = HEAP8[$67>>0]|0;
  $69 = $68&255;
  $70 = ($69|0)!=(156);
  if ($70) {
   $71 = HEAP32[$1>>2]|0;
   $72 = (($71) + 8|0);
   $73 = HEAP32[$72>>2]|0;
   $74 = ($73|0)!=(0|0);
   if ($74) {
    $75 = HEAP32[$1>>2]|0;
    $76 = (($75) + 8|0);
    $77 = HEAP32[$76>>2]|0;
    $78 = HEAP32[$2>>2]|0;
    $79 = (($78) + 8|0);
    $80 = HEAP32[$79>>2]|0;
    $81 = (_strcmp($77,$80)|0);
    $82 = ($81|0)!=(0);
    if ($82) {
     $83 = HEAP32[$1>>2]|0;
     $84 = HEAP8[$83>>0]|0;
     $85 = $84&255;
     $86 = ($85|0)==(95);
     $87 = $86 ? 1 : 2;
     HEAP32[$0>>2] = $87;
     $158 = HEAP32[$0>>2]|0;
     STACKTOP = sp;return ($158|0);
    }
   }
  }
 }
 $88 = HEAP32[$1>>2]|0;
 $89 = (($88) + 4|0);
 $90 = HEAP32[$89>>2]|0;
 $91 = $90 & 16;
 $92 = HEAP32[$2>>2]|0;
 $93 = (($92) + 4|0);
 $94 = HEAP32[$93>>2]|0;
 $95 = $94 & 16;
 $96 = ($91|0)!=($95|0);
 if ($96) {
  HEAP32[$0>>2] = 2;
  $158 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($158|0);
 }
 $97 = HEAP32[$combinedFlags>>2]|0;
 $98 = $97 & 16384;
 $99 = ($98|0)==(0);
 do {
  if ($99) {
   $100 = HEAP32[$combinedFlags>>2]|0;
   $101 = $100 & 2048;
   $102 = ($101|0)!=(0);
   if ($102) {
    HEAP32[$0>>2] = 2;
    $158 = HEAP32[$0>>2]|0;
    STACKTOP = sp;return ($158|0);
   }
   $103 = HEAP32[$1>>2]|0;
   $104 = (($103) + 12|0);
   $105 = HEAP32[$104>>2]|0;
   $106 = HEAP32[$2>>2]|0;
   $107 = (($106) + 12|0);
   $108 = HEAP32[$107>>2]|0;
   $109 = HEAP32[$3>>2]|0;
   $110 = (_sqlite3ExprCompare($105,$108,$109)|0);
   $111 = ($110|0)!=(0);
   if ($111) {
    HEAP32[$0>>2] = 2;
    $158 = HEAP32[$0>>2]|0;
    STACKTOP = sp;return ($158|0);
   }
   $112 = HEAP32[$1>>2]|0;
   $113 = (($112) + 16|0);
   $114 = HEAP32[$113>>2]|0;
   $115 = HEAP32[$2>>2]|0;
   $116 = (($115) + 16|0);
   $117 = HEAP32[$116>>2]|0;
   $118 = HEAP32[$3>>2]|0;
   $119 = (_sqlite3ExprCompare($114,$117,$118)|0);
   $120 = ($119|0)!=(0);
   if ($120) {
    HEAP32[$0>>2] = 2;
    $158 = HEAP32[$0>>2]|0;
    STACKTOP = sp;return ($158|0);
   }
   $121 = HEAP32[$1>>2]|0;
   $122 = (($121) + 20|0);
   $123 = HEAP32[$122>>2]|0;
   $124 = HEAP32[$2>>2]|0;
   $125 = (($124) + 20|0);
   $126 = HEAP32[$125>>2]|0;
   $127 = HEAP32[$3>>2]|0;
   $128 = (_sqlite3ExprListCompare($123,$126,$127)|0);
   $129 = ($128|0)!=(0);
   if ($129) {
    HEAP32[$0>>2] = 2;
    $158 = HEAP32[$0>>2]|0;
    STACKTOP = sp;return ($158|0);
   }
   $130 = HEAP32[$combinedFlags>>2]|0;
   $131 = $130 & 8192;
   $132 = ($131|0)==(0);
   if ($132) {
    $133 = HEAP32[$1>>2]|0;
    $134 = (($133) + 32|0);
    $135 = HEAP16[$134>>1]|0;
    $136 = $135 << 16 >> 16;
    $137 = HEAP32[$2>>2]|0;
    $138 = (($137) + 32|0);
    $139 = HEAP16[$138>>1]|0;
    $140 = $139 << 16 >> 16;
    $141 = ($136|0)!=($140|0);
    if ($141) {
     HEAP32[$0>>2] = 2;
     $158 = HEAP32[$0>>2]|0;
     STACKTOP = sp;return ($158|0);
    }
    $142 = HEAP32[$1>>2]|0;
    $143 = (($142) + 28|0);
    $144 = HEAP32[$143>>2]|0;
    $145 = HEAP32[$2>>2]|0;
    $146 = (($145) + 28|0);
    $147 = HEAP32[$146>>2]|0;
    $148 = ($144|0)!=($147|0);
    if ($148) {
     $149 = HEAP32[$1>>2]|0;
     $150 = (($149) + 28|0);
     $151 = HEAP32[$150>>2]|0;
     $152 = HEAP32[$3>>2]|0;
     $153 = ($151|0)!=($152|0);
     if (!($153)) {
      $154 = HEAP32[$2>>2]|0;
      $155 = (($154) + 28|0);
      $156 = HEAP32[$155>>2]|0;
      $157 = ($156|0)>=(0);
      if (!($157)) {
       break;
      }
     }
     HEAP32[$0>>2] = 2;
     $158 = HEAP32[$0>>2]|0;
     STACKTOP = sp;return ($158|0);
    }
   }
  }
 } while(0);
 HEAP32[$0>>2] = 0;
 $158 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($158|0);
}
function _sqlite3ExprListCompare($pA,$pB,$iTab) {
 $pA = $pA|0;
 $pB = $pB|0;
 $iTab = $iTab|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $i = 0, $or$cond = 0;
 var $or$cond3 = 0, $pExprA = 0, $pExprB = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 24|0;
 $1 = sp + 20|0;
 $2 = sp + 16|0;
 $3 = sp + 12|0;
 $i = sp + 8|0;
 $pExprA = sp + 4|0;
 $pExprB = sp;
 HEAP32[$1>>2] = $pA;
 HEAP32[$2>>2] = $pB;
 HEAP32[$3>>2] = $iTab;
 $4 = HEAP32[$1>>2]|0;
 $5 = ($4|0)==(0|0);
 $6 = HEAP32[$2>>2]|0;
 $7 = ($6|0)==(0|0);
 $or$cond = $5 & $7;
 if ($or$cond) {
  HEAP32[$0>>2] = 0;
  $57 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($57|0);
 }
 $8 = HEAP32[$1>>2]|0;
 $9 = ($8|0)==(0|0);
 $10 = HEAP32[$2>>2]|0;
 $11 = ($10|0)==(0|0);
 $or$cond3 = $9 | $11;
 if ($or$cond3) {
  HEAP32[$0>>2] = 1;
  $57 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($57|0);
 }
 $12 = HEAP32[$1>>2]|0;
 $13 = HEAP32[$12>>2]|0;
 $14 = HEAP32[$2>>2]|0;
 $15 = HEAP32[$14>>2]|0;
 $16 = ($13|0)!=($15|0);
 if ($16) {
  HEAP32[$0>>2] = 1;
  $57 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($57|0);
 }
 HEAP32[$i>>2] = 0;
 while(1) {
  $17 = HEAP32[$i>>2]|0;
  $18 = HEAP32[$1>>2]|0;
  $19 = HEAP32[$18>>2]|0;
  $20 = ($17|0)<($19|0);
  if (!($20)) {
   label = 14;
   break;
  }
  $21 = HEAP32[$i>>2]|0;
  $22 = HEAP32[$1>>2]|0;
  $23 = (($22) + 4|0);
  $24 = HEAP32[$23>>2]|0;
  $25 = (($24) + (($21*20)|0)|0);
  $26 = HEAP32[$25>>2]|0;
  HEAP32[$pExprA>>2] = $26;
  $27 = HEAP32[$i>>2]|0;
  $28 = HEAP32[$2>>2]|0;
  $29 = (($28) + 4|0);
  $30 = HEAP32[$29>>2]|0;
  $31 = (($30) + (($27*20)|0)|0);
  $32 = HEAP32[$31>>2]|0;
  HEAP32[$pExprB>>2] = $32;
  $33 = HEAP32[$i>>2]|0;
  $34 = HEAP32[$1>>2]|0;
  $35 = (($34) + 4|0);
  $36 = HEAP32[$35>>2]|0;
  $37 = (($36) + (($33*20)|0)|0);
  $38 = (($37) + 12|0);
  $39 = HEAP8[$38>>0]|0;
  $40 = $39&255;
  $41 = HEAP32[$i>>2]|0;
  $42 = HEAP32[$2>>2]|0;
  $43 = (($42) + 4|0);
  $44 = HEAP32[$43>>2]|0;
  $45 = (($44) + (($41*20)|0)|0);
  $46 = (($45) + 12|0);
  $47 = HEAP8[$46>>0]|0;
  $48 = $47&255;
  $49 = ($40|0)!=($48|0);
  if ($49) {
   label = 10;
   break;
  }
  $50 = HEAP32[$pExprA>>2]|0;
  $51 = HEAP32[$pExprB>>2]|0;
  $52 = HEAP32[$3>>2]|0;
  $53 = (_sqlite3ExprCompare($50,$51,$52)|0);
  $54 = ($53|0)!=(0);
  if ($54) {
   label = 12;
   break;
  }
  $55 = HEAP32[$i>>2]|0;
  $56 = (($55) + 1)|0;
  HEAP32[$i>>2] = $56;
 }
 if ((label|0) == 10) {
  HEAP32[$0>>2] = 1;
  $57 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($57|0);
 }
 else if ((label|0) == 12) {
  HEAP32[$0>>2] = 1;
  $57 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($57|0);
 }
 else if ((label|0) == 14) {
  HEAP32[$0>>2] = 0;
  $57 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($57|0);
 }
 return 0|0;
}
function _exprDup($db,$p,$flags,$pzBuffer) {
 $db = $db|0;
 $p = $p|0;
 $flags = $flags|0;
 $pzBuffer = $pzBuffer|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0;
 var $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0;
 var $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0;
 var $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0;
 var $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0;
 var $98 = 0, $99 = 0, $isReduced = 0, $nNewSize = 0, $nSize = 0, $nStructSize = 0, $nToken = 0, $pNew = 0, $staticFlag = 0, $zAlloc = 0, $zToken = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 64|0;
 $0 = sp + 48|0;
 $1 = sp + 44|0;
 $2 = sp + 40|0;
 $3 = sp + 36|0;
 $pNew = sp + 32|0;
 $isReduced = sp + 28|0;
 $zAlloc = sp + 24|0;
 $staticFlag = sp + 20|0;
 $nStructSize = sp + 16|0;
 $nNewSize = sp + 12|0;
 $nToken = sp + 8|0;
 $nSize = sp + 4|0;
 $zToken = sp;
 HEAP32[$0>>2] = $db;
 HEAP32[$1>>2] = $p;
 HEAP32[$2>>2] = $flags;
 HEAP32[$3>>2] = $pzBuffer;
 HEAP32[$pNew>>2] = 0;
 $4 = HEAP32[$1>>2]|0;
 $5 = ($4|0)!=(0|0);
 if (!($5)) {
  $161 = HEAP32[$pNew>>2]|0;
  STACKTOP = sp;return ($161|0);
 }
 $6 = HEAP32[$2>>2]|0;
 $7 = $6 & 1;
 HEAP32[$isReduced>>2] = $7;
 HEAP32[$staticFlag>>2] = 0;
 $8 = HEAP32[$3>>2]|0;
 $9 = ($8|0)!=(0|0);
 if ($9) {
  $10 = HEAP32[$3>>2]|0;
  $11 = HEAP32[$10>>2]|0;
  HEAP32[$zAlloc>>2] = $11;
  HEAP32[$staticFlag>>2] = 32768;
 } else {
  $12 = HEAP32[$0>>2]|0;
  $13 = HEAP32[$1>>2]|0;
  $14 = HEAP32[$2>>2]|0;
  $15 = (_dupedExprSize($13,$14)|0);
  $16 = ($15|0)<(0);
  $17 = $16 << 31 >> 31;
  $18 = (_sqlite3DbMallocRaw($12,$15,$17)|0);
  HEAP32[$zAlloc>>2] = $18;
 }
 $19 = HEAP32[$zAlloc>>2]|0;
 HEAP32[$pNew>>2] = $19;
 $20 = HEAP32[$pNew>>2]|0;
 $21 = ($20|0)!=(0|0);
 if (!($21)) {
  $161 = HEAP32[$pNew>>2]|0;
  STACKTOP = sp;return ($161|0);
 }
 $22 = HEAP32[$1>>2]|0;
 $23 = HEAP32[$2>>2]|0;
 $24 = (_dupedExprStructSize($22,$23)|0);
 HEAP32[$nStructSize>>2] = $24;
 $25 = HEAP32[$nStructSize>>2]|0;
 $26 = $25 & 4095;
 HEAP32[$nNewSize>>2] = $26;
 $27 = HEAP32[$1>>2]|0;
 $28 = (($27) + 4|0);
 $29 = HEAP32[$28>>2]|0;
 $30 = $29 & 1024;
 $31 = ($30|0)!=(0);
 if ($31) {
  label = 9;
 } else {
  $32 = HEAP32[$1>>2]|0;
  $33 = (($32) + 8|0);
  $34 = HEAP32[$33>>2]|0;
  $35 = ($34|0)!=(0|0);
  if ($35) {
   $36 = HEAP32[$1>>2]|0;
   $37 = (($36) + 8|0);
   $38 = HEAP32[$37>>2]|0;
   $39 = (_sqlite3Strlen30($38)|0);
   $40 = (($39) + 1)|0;
   HEAP32[$nToken>>2] = $40;
  } else {
   label = 9;
  }
 }
 if ((label|0) == 9) {
  HEAP32[$nToken>>2] = 0;
 }
 $41 = HEAP32[$isReduced>>2]|0;
 $42 = ($41|0)!=(0);
 if ($42) {
  $43 = HEAP32[$zAlloc>>2]|0;
  $44 = HEAP32[$1>>2]|0;
  $45 = HEAP32[$nNewSize>>2]|0;
  _memcpy(($43|0),($44|0),($45|0))|0;
 } else {
  $46 = HEAP32[$1>>2]|0;
  $47 = (_exprStructSize($46)|0);
  HEAP32[$nSize>>2] = $47;
  $48 = HEAP32[$zAlloc>>2]|0;
  $49 = HEAP32[$1>>2]|0;
  $50 = HEAP32[$nSize>>2]|0;
  _memcpy(($48|0),($49|0),($50|0))|0;
  $51 = HEAP32[$nSize>>2]|0;
  $52 = HEAP32[$zAlloc>>2]|0;
  $53 = (($52) + ($51)|0);
  $54 = HEAP32[$nSize>>2]|0;
  $55 = (48 - ($54))|0;
  _memset(($53|0),0,($55|0))|0;
 }
 $56 = HEAP32[$pNew>>2]|0;
 $57 = (($56) + 4|0);
 $58 = HEAP32[$57>>2]|0;
 $59 = $58 & -122881;
 HEAP32[$57>>2] = $59;
 $60 = HEAP32[$nStructSize>>2]|0;
 $61 = $60 & 24576;
 $62 = HEAP32[$pNew>>2]|0;
 $63 = (($62) + 4|0);
 $64 = HEAP32[$63>>2]|0;
 $65 = $64 | $61;
 HEAP32[$63>>2] = $65;
 $66 = HEAP32[$staticFlag>>2]|0;
 $67 = HEAP32[$pNew>>2]|0;
 $68 = (($67) + 4|0);
 $69 = HEAP32[$68>>2]|0;
 $70 = $69 | $66;
 HEAP32[$68>>2] = $70;
 $71 = HEAP32[$nToken>>2]|0;
 $72 = ($71|0)!=(0);
 if ($72) {
  $73 = HEAP32[$nNewSize>>2]|0;
  $74 = HEAP32[$zAlloc>>2]|0;
  $75 = (($74) + ($73)|0);
  $76 = HEAP32[$pNew>>2]|0;
  $77 = (($76) + 8|0);
  HEAP32[$77>>2] = $75;
  HEAP32[$zToken>>2] = $75;
  $78 = HEAP32[$zToken>>2]|0;
  $79 = HEAP32[$1>>2]|0;
  $80 = (($79) + 8|0);
  $81 = HEAP32[$80>>2]|0;
  $82 = HEAP32[$nToken>>2]|0;
  _memcpy(($78|0),($81|0),($82|0))|0;
 }
 $83 = HEAP32[$1>>2]|0;
 $84 = (($83) + 4|0);
 $85 = HEAP32[$84>>2]|0;
 $86 = HEAP32[$pNew>>2]|0;
 $87 = (($86) + 4|0);
 $88 = HEAP32[$87>>2]|0;
 $89 = $85 | $88;
 $90 = $89 & 16384;
 $91 = (0)==($90|0);
 do {
  if ($91) {
   $92 = HEAP32[$1>>2]|0;
   $93 = (($92) + 4|0);
   $94 = HEAP32[$93>>2]|0;
   $95 = $94 & 2048;
   $96 = ($95|0)!=(0);
   $97 = HEAP32[$0>>2]|0;
   $98 = HEAP32[$1>>2]|0;
   $99 = (($98) + 20|0);
   if ($96) {
    $100 = HEAP32[$99>>2]|0;
    $101 = HEAP32[$isReduced>>2]|0;
    $102 = (_sqlite3SelectDup($97,$100,$101)|0);
    $103 = HEAP32[$pNew>>2]|0;
    $104 = (($103) + 20|0);
    HEAP32[$104>>2] = $102;
    break;
   } else {
    $105 = HEAP32[$99>>2]|0;
    $106 = HEAP32[$isReduced>>2]|0;
    $107 = (_sqlite3ExprListDup($97,$105,$106)|0);
    $108 = HEAP32[$pNew>>2]|0;
    $109 = (($108) + 20|0);
    HEAP32[$109>>2] = $107;
    break;
   }
  }
 } while(0);
 $110 = HEAP32[$pNew>>2]|0;
 $111 = (($110) + 4|0);
 $112 = HEAP32[$111>>2]|0;
 $113 = $112 & 24576;
 $114 = ($113|0)!=(0);
 $115 = HEAP32[$1>>2]|0;
 if (!($114)) {
  $143 = (($115) + 4|0);
  $144 = HEAP32[$143>>2]|0;
  $145 = $144 & 16384;
  $146 = ($145|0)!=(0);
  if ($146) {
   $161 = HEAP32[$pNew>>2]|0;
   STACKTOP = sp;return ($161|0);
  }
  $147 = HEAP32[$0>>2]|0;
  $148 = HEAP32[$1>>2]|0;
  $149 = (($148) + 12|0);
  $150 = HEAP32[$149>>2]|0;
  $151 = (_sqlite3ExprDup($147,$150,0)|0);
  $152 = HEAP32[$pNew>>2]|0;
  $153 = (($152) + 12|0);
  HEAP32[$153>>2] = $151;
  $154 = HEAP32[$0>>2]|0;
  $155 = HEAP32[$1>>2]|0;
  $156 = (($155) + 16|0);
  $157 = HEAP32[$156>>2]|0;
  $158 = (_sqlite3ExprDup($154,$157,0)|0);
  $159 = HEAP32[$pNew>>2]|0;
  $160 = (($159) + 16|0);
  HEAP32[$160>>2] = $158;
  $161 = HEAP32[$pNew>>2]|0;
  STACKTOP = sp;return ($161|0);
 }
 $116 = HEAP32[$2>>2]|0;
 $117 = (_dupedExprNodeSize($115,$116)|0);
 $118 = HEAP32[$zAlloc>>2]|0;
 $119 = (($118) + ($117)|0);
 HEAP32[$zAlloc>>2] = $119;
 $120 = HEAP32[$pNew>>2]|0;
 $121 = (($120) + 4|0);
 $122 = HEAP32[$121>>2]|0;
 $123 = $122 & 8192;
 $124 = ($123|0)!=(0);
 if ($124) {
  $125 = HEAP32[$0>>2]|0;
  $126 = HEAP32[$1>>2]|0;
  $127 = (($126) + 12|0);
  $128 = HEAP32[$127>>2]|0;
  $129 = (_exprDup($125,$128,1,$zAlloc)|0);
  $130 = HEAP32[$pNew>>2]|0;
  $131 = (($130) + 12|0);
  HEAP32[$131>>2] = $129;
  $132 = HEAP32[$0>>2]|0;
  $133 = HEAP32[$1>>2]|0;
  $134 = (($133) + 16|0);
  $135 = HEAP32[$134>>2]|0;
  $136 = (_exprDup($132,$135,1,$zAlloc)|0);
  $137 = HEAP32[$pNew>>2]|0;
  $138 = (($137) + 16|0);
  HEAP32[$138>>2] = $136;
 }
 $139 = HEAP32[$3>>2]|0;
 $140 = ($139|0)!=(0|0);
 if (!($140)) {
  $161 = HEAP32[$pNew>>2]|0;
  STACKTOP = sp;return ($161|0);
 }
 $141 = HEAP32[$zAlloc>>2]|0;
 $142 = HEAP32[$3>>2]|0;
 HEAP32[$142>>2] = $141;
 $161 = HEAP32[$pNew>>2]|0;
 STACKTOP = sp;return ($161|0);
}
function _dupedExprSize($p,$flags) {
 $p = $p|0;
 $flags = $flags|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $3 = 0, $4 = 0, $5 = 0;
 var $6 = 0, $7 = 0, $8 = 0, $9 = 0, $nByte = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $1 = sp + 4|0;
 $nByte = sp;
 HEAP32[$0>>2] = $p;
 HEAP32[$1>>2] = $flags;
 HEAP32[$nByte>>2] = 0;
 $2 = HEAP32[$0>>2]|0;
 $3 = ($2|0)!=(0|0);
 if (!($3)) {
  $23 = HEAP32[$nByte>>2]|0;
  STACKTOP = sp;return ($23|0);
 }
 $4 = HEAP32[$0>>2]|0;
 $5 = HEAP32[$1>>2]|0;
 $6 = (_dupedExprNodeSize($4,$5)|0);
 HEAP32[$nByte>>2] = $6;
 $7 = HEAP32[$1>>2]|0;
 $8 = $7 & 1;
 $9 = ($8|0)!=(0);
 if (!($9)) {
  $23 = HEAP32[$nByte>>2]|0;
  STACKTOP = sp;return ($23|0);
 }
 $10 = HEAP32[$0>>2]|0;
 $11 = (($10) + 12|0);
 $12 = HEAP32[$11>>2]|0;
 $13 = HEAP32[$1>>2]|0;
 $14 = (_dupedExprSize($12,$13)|0);
 $15 = HEAP32[$0>>2]|0;
 $16 = (($15) + 16|0);
 $17 = HEAP32[$16>>2]|0;
 $18 = HEAP32[$1>>2]|0;
 $19 = (_dupedExprSize($17,$18)|0);
 $20 = (($14) + ($19))|0;
 $21 = HEAP32[$nByte>>2]|0;
 $22 = (($21) + ($20))|0;
 HEAP32[$nByte>>2] = $22;
 $23 = HEAP32[$nByte>>2]|0;
 STACKTOP = sp;return ($23|0);
}
function _dupedExprStructSize($p,$flags) {
 $p = $p|0;
 $flags = $flags|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $nSize = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $1 = sp + 4|0;
 $nSize = sp;
 HEAP32[$0>>2] = $p;
 HEAP32[$1>>2] = $flags;
 $2 = HEAP32[$1>>2]|0;
 $3 = $2 & 1;
 $4 = (0)==($3|0);
 do {
  if ($4) {
   HEAP32[$nSize>>2] = 48;
  } else {
   $5 = HEAP32[$0>>2]|0;
   $6 = (($5) + 12|0);
   $7 = HEAP32[$6>>2]|0;
   $8 = ($7|0)!=(0|0);
   if (!($8)) {
    $9 = HEAP32[$0>>2]|0;
    $10 = (($9) + 20|0);
    $11 = HEAP32[$10>>2]|0;
    $12 = ($11|0)!=(0|0);
    if (!($12)) {
     HEAP32[$nSize>>2] = 16396;
     break;
    }
   }
   HEAP32[$nSize>>2] = 8220;
  }
 } while(0);
 $13 = HEAP32[$nSize>>2]|0;
 STACKTOP = sp;return ($13|0);
}
function _exprStructSize($p) {
 $p = $p|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $1 = sp;
 HEAP32[$1>>2] = $p;
 $2 = HEAP32[$1>>2]|0;
 $3 = (($2) + 4|0);
 $4 = HEAP32[$3>>2]|0;
 $5 = $4 & 16384;
 $6 = ($5|0)!=(0);
 do {
  if ($6) {
   HEAP32[$0>>2] = 12;
  } else {
   $7 = HEAP32[$1>>2]|0;
   $8 = (($7) + 4|0);
   $9 = HEAP32[$8>>2]|0;
   $10 = $9 & 8192;
   $11 = ($10|0)!=(0);
   if ($11) {
    HEAP32[$0>>2] = 28;
    break;
   } else {
    HEAP32[$0>>2] = 48;
    break;
   }
  }
 } while(0);
 $12 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($12|0);
}
function _sqlite3ExprListDup($db,$p,$flags) {
 $db = $db|0;
 $p = $p|0;
 $flags = $flags|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0;
 var $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $i = 0, $pItem = 0, $pNew = 0, $pOldExpr = 0;
 var $pOldItem = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $0 = sp + 32|0;
 $1 = sp + 28|0;
 $2 = sp + 24|0;
 $3 = sp + 20|0;
 $pNew = sp + 16|0;
 $pItem = sp + 12|0;
 $pOldItem = sp + 8|0;
 $i = sp + 4|0;
 $pOldExpr = sp;
 HEAP32[$1>>2] = $db;
 HEAP32[$2>>2] = $p;
 HEAP32[$3>>2] = $flags;
 $4 = HEAP32[$2>>2]|0;
 $5 = ($4|0)==(0|0);
 if ($5) {
  HEAP32[$0>>2] = 0;
  $95 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($95|0);
 }
 $6 = HEAP32[$1>>2]|0;
 $7 = (_sqlite3DbMallocRaw($6,8,0)|0);
 HEAP32[$pNew>>2] = $7;
 $8 = HEAP32[$pNew>>2]|0;
 $9 = ($8|0)==(0|0);
 if ($9) {
  HEAP32[$0>>2] = 0;
  $95 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($95|0);
 }
 $10 = HEAP32[$2>>2]|0;
 $11 = HEAP32[$10>>2]|0;
 HEAP32[$i>>2] = $11;
 $12 = HEAP32[$pNew>>2]|0;
 HEAP32[$12>>2] = $11;
 $13 = HEAP32[$3>>2]|0;
 $14 = $13 & 1;
 $15 = ($14|0)==(0);
 L9: do {
  if ($15) {
   HEAP32[$i>>2] = 1;
   while(1) {
    $16 = HEAP32[$i>>2]|0;
    $17 = HEAP32[$2>>2]|0;
    $18 = HEAP32[$17>>2]|0;
    $19 = ($16|0)<($18|0);
    if (!($19)) {
     break L9;
    }
    $20 = HEAP32[$i>>2]|0;
    $21 = HEAP32[$i>>2]|0;
    $22 = (($21) + ($20))|0;
    HEAP32[$i>>2] = $22;
   }
  }
 } while(0);
 $23 = HEAP32[$1>>2]|0;
 $24 = HEAP32[$i>>2]|0;
 $25 = ($24*20)|0;
 $26 = (_sqlite3DbMallocRaw($23,$25,0)|0);
 HEAP32[$pItem>>2] = $26;
 $27 = HEAP32[$pNew>>2]|0;
 $28 = (($27) + 4|0);
 HEAP32[$28>>2] = $26;
 $29 = HEAP32[$pItem>>2]|0;
 $30 = ($29|0)==(0|0);
 if ($30) {
  $31 = HEAP32[$1>>2]|0;
  $32 = HEAP32[$pNew>>2]|0;
  _sqlite3DbFree($31,$32);
  HEAP32[$0>>2] = 0;
  $95 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($95|0);
 }
 $33 = HEAP32[$2>>2]|0;
 $34 = (($33) + 4|0);
 $35 = HEAP32[$34>>2]|0;
 HEAP32[$pOldItem>>2] = $35;
 HEAP32[$i>>2] = 0;
 while(1) {
  $36 = HEAP32[$i>>2]|0;
  $37 = HEAP32[$2>>2]|0;
  $38 = HEAP32[$37>>2]|0;
  $39 = ($36|0)<($38|0);
  if (!($39)) {
   break;
  }
  $40 = HEAP32[$pOldItem>>2]|0;
  $41 = HEAP32[$40>>2]|0;
  HEAP32[$pOldExpr>>2] = $41;
  $42 = HEAP32[$1>>2]|0;
  $43 = HEAP32[$pOldExpr>>2]|0;
  $44 = HEAP32[$3>>2]|0;
  $45 = (_sqlite3ExprDup($42,$43,$44)|0);
  $46 = HEAP32[$pItem>>2]|0;
  HEAP32[$46>>2] = $45;
  $47 = HEAP32[$1>>2]|0;
  $48 = HEAP32[$pOldItem>>2]|0;
  $49 = (($48) + 4|0);
  $50 = HEAP32[$49>>2]|0;
  $51 = (_sqlite3DbStrDup($47,$50)|0);
  $52 = HEAP32[$pItem>>2]|0;
  $53 = (($52) + 4|0);
  HEAP32[$53>>2] = $51;
  $54 = HEAP32[$1>>2]|0;
  $55 = HEAP32[$pOldItem>>2]|0;
  $56 = (($55) + 8|0);
  $57 = HEAP32[$56>>2]|0;
  $58 = (_sqlite3DbStrDup($54,$57)|0);
  $59 = HEAP32[$pItem>>2]|0;
  $60 = (($59) + 8|0);
  HEAP32[$60>>2] = $58;
  $61 = HEAP32[$pOldItem>>2]|0;
  $62 = (($61) + 12|0);
  $63 = HEAP8[$62>>0]|0;
  $64 = HEAP32[$pItem>>2]|0;
  $65 = (($64) + 12|0);
  HEAP8[$65>>0] = $63;
  $66 = HEAP32[$pItem>>2]|0;
  $67 = (($66) + 13|0);
  $68 = HEAP8[$67>>0]|0;
  $69 = $68 & -2;
  HEAP8[$67>>0] = $69;
  $70 = HEAP32[$pOldItem>>2]|0;
  $71 = (($70) + 13|0);
  $72 = HEAP8[$71>>0]|0;
  $73 = ($72&255) >>> 1;
  $74 = $73 & 1;
  $75 = $74&255;
  $76 = HEAP32[$pItem>>2]|0;
  $77 = (($76) + 13|0);
  $78 = $75&255;
  $79 = HEAP8[$77>>0]|0;
  $80 = $78 & 1;
  $81 = ($80 << 1)&255;
  $82 = $79 & -3;
  $83 = $82 | $81;
  HEAP8[$77>>0] = $83;
  $84 = HEAP32[$pItem>>2]|0;
  $85 = (($84) + 16|0);
  $86 = HEAP32[$pOldItem>>2]|0;
  $87 = (($86) + 16|0);
  ;HEAP32[$85+0>>2]=HEAP32[$87+0>>2]|0;
  $88 = HEAP32[$i>>2]|0;
  $89 = (($88) + 1)|0;
  HEAP32[$i>>2] = $89;
  $90 = HEAP32[$pItem>>2]|0;
  $91 = (($90) + 20|0);
  HEAP32[$pItem>>2] = $91;
  $92 = HEAP32[$pOldItem>>2]|0;
  $93 = (($92) + 20|0);
  HEAP32[$pOldItem>>2] = $93;
 }
 $94 = HEAP32[$pNew>>2]|0;
 HEAP32[$0>>2] = $94;
 $95 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($95|0);
}
function _dupedExprNodeSize($p,$flags) {
 $p = $p|0;
 $flags = $flags|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $3 = 0, $4 = 0;
 var $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $nByte = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $1 = sp + 4|0;
 $nByte = sp;
 HEAP32[$0>>2] = $p;
 HEAP32[$1>>2] = $flags;
 $2 = HEAP32[$0>>2]|0;
 $3 = HEAP32[$1>>2]|0;
 $4 = (_dupedExprStructSize($2,$3)|0);
 $5 = $4 & 4095;
 HEAP32[$nByte>>2] = $5;
 $6 = HEAP32[$0>>2]|0;
 $7 = (($6) + 4|0);
 $8 = HEAP32[$7>>2]|0;
 $9 = $8 & 1024;
 $10 = ($9|0)!=(0);
 if (!($10)) {
  $11 = HEAP32[$0>>2]|0;
  $12 = (($11) + 8|0);
  $13 = HEAP32[$12>>2]|0;
  $14 = ($13|0)!=(0|0);
  if ($14) {
   $15 = HEAP32[$0>>2]|0;
   $16 = (($15) + 8|0);
   $17 = HEAP32[$16>>2]|0;
   $18 = (_sqlite3Strlen30($17)|0);
   $19 = (($18) + 1)|0;
   $20 = HEAP32[$nByte>>2]|0;
   $21 = (($20) + ($19))|0;
   HEAP32[$nByte>>2] = $21;
  }
 }
 $22 = HEAP32[$nByte>>2]|0;
 $23 = (($22) + 7)|0;
 $24 = $23 & -8;
 STACKTOP = sp;return ($24|0);
}
function _sqlite3ResolveOrderGroupBy($pParse,$pSelect,$pOrderBy,$zType) {
 $pParse = $pParse|0;
 $pSelect = $pSelect|0;
 $pOrderBy = $pOrderBy|0;
 $zType = $zType|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $7 = 0, $8 = 0, $9 = 0, $db = 0, $i = 0, $pEList = 0, $pItem = 0, $vararg_buffer = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $vararg_buffer = sp;
 $0 = sp + 36|0;
 $1 = sp + 32|0;
 $2 = sp + 28|0;
 $3 = sp + 24|0;
 $4 = sp + 20|0;
 $i = sp + 16|0;
 $db = sp + 12|0;
 $pEList = sp + 8|0;
 $pItem = sp + 4|0;
 HEAP32[$1>>2] = $pParse;
 HEAP32[$2>>2] = $pSelect;
 HEAP32[$3>>2] = $pOrderBy;
 HEAP32[$4>>2] = $zType;
 $5 = HEAP32[$1>>2]|0;
 $6 = HEAP32[$5>>2]|0;
 HEAP32[$db>>2] = $6;
 $7 = HEAP32[$3>>2]|0;
 $8 = ($7|0)==(0|0);
 if (!($8)) {
  $9 = HEAP32[$1>>2]|0;
  $10 = HEAP32[$9>>2]|0;
  $11 = (($10) + 64|0);
  $12 = HEAP8[$11>>0]|0;
  $13 = $12&255;
  $14 = ($13|0)!=(0);
  if (!($14)) {
   $15 = HEAP32[$3>>2]|0;
   $16 = HEAP32[$15>>2]|0;
   $17 = HEAP32[$db>>2]|0;
   $18 = (($17) + 88|0);
   $19 = (($18) + 8|0);
   $20 = HEAP32[$19>>2]|0;
   $21 = ($16|0)>($20|0);
   if ($21) {
    $22 = HEAP32[$1>>2]|0;
    $23 = HEAP32[$4>>2]|0;
    HEAP32[$vararg_buffer>>2] = $23;
    _sqlite3ErrorMsg($22,16720,$vararg_buffer);
    HEAP32[$0>>2] = 1;
    $63 = HEAP32[$0>>2]|0;
    STACKTOP = sp;return ($63|0);
   }
   $24 = HEAP32[$2>>2]|0;
   $25 = HEAP32[$24>>2]|0;
   HEAP32[$pEList>>2] = $25;
   HEAP32[$i>>2] = 0;
   $26 = HEAP32[$3>>2]|0;
   $27 = (($26) + 4|0);
   $28 = HEAP32[$27>>2]|0;
   HEAP32[$pItem>>2] = $28;
   while(1) {
    $29 = HEAP32[$i>>2]|0;
    $30 = HEAP32[$3>>2]|0;
    $31 = HEAP32[$30>>2]|0;
    $32 = ($29|0)<($31|0);
    if (!($32)) {
     label = 13;
     break;
    }
    $33 = HEAP32[$pItem>>2]|0;
    $34 = (($33) + 16|0);
    $35 = HEAP16[$34>>1]|0;
    $36 = ($35<<16>>16)!=(0);
    if ($36) {
     $37 = HEAP32[$pItem>>2]|0;
     $38 = (($37) + 16|0);
     $39 = HEAP16[$38>>1]|0;
     $40 = $39&65535;
     $41 = HEAP32[$pEList>>2]|0;
     $42 = HEAP32[$41>>2]|0;
     $43 = ($40|0)>($42|0);
     $44 = HEAP32[$1>>2]|0;
     if ($43) {
      label = 10;
      break;
     }
     $50 = HEAP32[$pEList>>2]|0;
     $51 = HEAP32[$pItem>>2]|0;
     $52 = (($51) + 16|0);
     $53 = HEAP16[$52>>1]|0;
     $54 = $53&65535;
     $55 = (($54) - 1)|0;
     $56 = HEAP32[$pItem>>2]|0;
     $57 = HEAP32[$56>>2]|0;
     $58 = HEAP32[$4>>2]|0;
     _resolveAlias($44,$50,$55,$57,$58,0);
    }
    $59 = HEAP32[$i>>2]|0;
    $60 = (($59) + 1)|0;
    HEAP32[$i>>2] = $60;
    $61 = HEAP32[$pItem>>2]|0;
    $62 = (($61) + 20|0);
    HEAP32[$pItem>>2] = $62;
   }
   if ((label|0) == 10) {
    $45 = HEAP32[$4>>2]|0;
    $46 = HEAP32[$i>>2]|0;
    $47 = (($46) + 1)|0;
    $48 = HEAP32[$pEList>>2]|0;
    $49 = HEAP32[$48>>2]|0;
    _resolveOutOfRangeError($44,$45,$47,$49);
    HEAP32[$0>>2] = 1;
    $63 = HEAP32[$0>>2]|0;
    STACKTOP = sp;return ($63|0);
   }
   else if ((label|0) == 13) {
    HEAP32[$0>>2] = 0;
    $63 = HEAP32[$0>>2]|0;
    STACKTOP = sp;return ($63|0);
   }
  }
 }
 HEAP32[$0>>2] = 0;
 $63 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($63|0);
}
function _resolveAlias($pParse,$pEList,$iCol,$pExpr,$zType,$nSubquery) {
 $pParse = $pParse|0;
 $pEList = $pEList|0;
 $iCol = $iCol|0;
 $pExpr = $pExpr|0;
 $zType = $zType|0;
 $nSubquery = $nSubquery|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0;
 var $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0;
 var $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0;
 var $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0;
 var $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0;
 var $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $db = 0, $pDup = 0, $pOrig = 0, dest = 0, label = 0, sp = 0, src = 0, stop = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $0 = sp + 32|0;
 $1 = sp + 28|0;
 $2 = sp + 24|0;
 $3 = sp + 20|0;
 $4 = sp + 16|0;
 $5 = sp + 12|0;
 $pOrig = sp + 8|0;
 $pDup = sp + 4|0;
 $db = sp;
 HEAP32[$0>>2] = $pParse;
 HEAP32[$1>>2] = $pEList;
 HEAP32[$2>>2] = $iCol;
 HEAP32[$3>>2] = $pExpr;
 HEAP32[$4>>2] = $zType;
 HEAP32[$5>>2] = $nSubquery;
 $6 = HEAP32[$2>>2]|0;
 $7 = HEAP32[$1>>2]|0;
 $8 = (($7) + 4|0);
 $9 = HEAP32[$8>>2]|0;
 $10 = (($9) + (($6*20)|0)|0);
 $11 = HEAP32[$10>>2]|0;
 HEAP32[$pOrig>>2] = $11;
 $12 = HEAP32[$0>>2]|0;
 $13 = HEAP32[$12>>2]|0;
 HEAP32[$db>>2] = $13;
 $14 = HEAP32[$db>>2]|0;
 $15 = HEAP32[$pOrig>>2]|0;
 $16 = (_sqlite3ExprDup($14,$15,0)|0);
 HEAP32[$pDup>>2] = $16;
 $17 = HEAP32[$pDup>>2]|0;
 $18 = ($17|0)==(0|0);
 if ($18) {
  STACKTOP = sp;return;
 }
 $19 = HEAP32[$pOrig>>2]|0;
 $20 = HEAP8[$19>>0]|0;
 $21 = $20&255;
 $22 = ($21|0)!=(154);
 if ($22) {
  $23 = HEAP32[$4>>2]|0;
  $24 = HEAP8[$23>>0]|0;
  $25 = $24 << 24 >> 24;
  $26 = ($25|0)!=(71);
  if ($26) {
   $27 = HEAP32[$pDup>>2]|0;
   $28 = HEAP32[$5>>2]|0;
   _incrAggFunctionDepth($27,$28);
   $29 = HEAP32[$0>>2]|0;
   $30 = HEAP32[$pDup>>2]|0;
   $31 = (_sqlite3PExpr($29,24,$30,0,0)|0);
   HEAP32[$pDup>>2] = $31;
   $32 = HEAP32[$pDup>>2]|0;
   $33 = ($32|0)==(0|0);
   if ($33) {
    STACKTOP = sp;return;
   }
   $34 = HEAP32[$pDup>>2]|0;
   $35 = (($34) + 4|0);
   $36 = HEAP32[$35>>2]|0;
   $37 = $36 | 4096;
   HEAP32[$35>>2] = $37;
   $38 = HEAP32[$2>>2]|0;
   $39 = HEAP32[$1>>2]|0;
   $40 = (($39) + 4|0);
   $41 = HEAP32[$40>>2]|0;
   $42 = (($41) + (($38*20)|0)|0);
   $43 = (($42) + 16|0);
   $44 = (($43) + 2|0);
   $45 = HEAP16[$44>>1]|0;
   $46 = $45&65535;
   $47 = ($46|0)==(0);
   if ($47) {
    $48 = HEAP32[$0>>2]|0;
    $49 = (($48) + 456|0);
    $50 = HEAP32[$49>>2]|0;
    $51 = (($50) + 1)|0;
    HEAP32[$49>>2] = $51;
    $52 = $51&65535;
    $53 = HEAP32[$2>>2]|0;
    $54 = HEAP32[$1>>2]|0;
    $55 = (($54) + 4|0);
    $56 = HEAP32[$55>>2]|0;
    $57 = (($56) + (($53*20)|0)|0);
    $58 = (($57) + 16|0);
    $59 = (($58) + 2|0);
    HEAP16[$59>>1] = $52;
   }
   $60 = HEAP32[$2>>2]|0;
   $61 = HEAP32[$1>>2]|0;
   $62 = (($61) + 4|0);
   $63 = HEAP32[$62>>2]|0;
   $64 = (($63) + (($60*20)|0)|0);
   $65 = (($64) + 16|0);
   $66 = (($65) + 2|0);
   $67 = HEAP16[$66>>1]|0;
   $68 = $67&65535;
   $69 = HEAP32[$pDup>>2]|0;
   $70 = (($69) + 28|0);
   HEAP32[$70>>2] = $68;
  }
 }
 $71 = HEAP32[$3>>2]|0;
 $72 = HEAP8[$71>>0]|0;
 $73 = $72&255;
 $74 = ($73|0)==(95);
 if ($74) {
  $75 = HEAP32[$0>>2]|0;
  $76 = HEAP32[$pDup>>2]|0;
  $77 = HEAP32[$3>>2]|0;
  $78 = (($77) + 8|0);
  $79 = HEAP32[$78>>2]|0;
  $80 = (_sqlite3ExprAddCollateString($75,$76,$79)|0);
  HEAP32[$pDup>>2] = $80;
 }
 $81 = HEAP32[$3>>2]|0;
 $82 = (($81) + 4|0);
 $83 = HEAP32[$82>>2]|0;
 $84 = $83 | 32768;
 HEAP32[$82>>2] = $84;
 $85 = HEAP32[$db>>2]|0;
 $86 = HEAP32[$3>>2]|0;
 _sqlite3ExprDelete($85,$86);
 $87 = HEAP32[$3>>2]|0;
 $88 = HEAP32[$pDup>>2]|0;
 dest=$87+0|0; src=$88+0|0; stop=dest+48|0; do { HEAP32[dest>>2]=HEAP32[src>>2]|0; dest=dest+4|0; src=src+4|0; } while ((dest|0) < (stop|0));
 $89 = HEAP32[$3>>2]|0;
 $90 = (($89) + 4|0);
 $91 = HEAP32[$90>>2]|0;
 $92 = $91 & 1024;
 $93 = ($92|0)!=(0);
 if (!($93)) {
  $94 = HEAP32[$3>>2]|0;
  $95 = (($94) + 8|0);
  $96 = HEAP32[$95>>2]|0;
  $97 = ($96|0)!=(0|0);
  if ($97) {
   $98 = HEAP32[$db>>2]|0;
   $99 = HEAP32[$3>>2]|0;
   $100 = (($99) + 8|0);
   $101 = HEAP32[$100>>2]|0;
   $102 = (_sqlite3DbStrDup($98,$101)|0);
   $103 = HEAP32[$3>>2]|0;
   $104 = (($103) + 8|0);
   HEAP32[$104>>2] = $102;
   $105 = HEAP32[$3>>2]|0;
   $106 = (($105) + 4|0);
   $107 = HEAP32[$106>>2]|0;
   $108 = $107 | 65536;
   HEAP32[$106>>2] = $108;
  }
 }
 $109 = HEAP32[$db>>2]|0;
 $110 = HEAP32[$pDup>>2]|0;
 _sqlite3DbFree($109,$110);
 STACKTOP = sp;return;
}
function _incrAggFunctionDepth($pExpr,$N) {
 $pExpr = $pExpr|0;
 $N = $N|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $w = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 28|0;
 $1 = sp + 24|0;
 $w = sp;
 HEAP32[$0>>2] = $pExpr;
 HEAP32[$1>>2] = $N;
 $2 = HEAP32[$1>>2]|0;
 $3 = ($2|0)>(0);
 if (!($3)) {
  STACKTOP = sp;return;
 }
 ;HEAP32[$w+0>>2]=0|0;HEAP32[$w+4>>2]=0|0;HEAP32[$w+8>>2]=0|0;HEAP32[$w+12>>2]=0|0;HEAP32[$w+16>>2]=0|0;HEAP32[$w+20>>2]=0|0;
 HEAP32[$w>>2] = 42;
 $4 = HEAP32[$1>>2]|0;
 $5 = (($w) + 20|0);
 HEAP32[$5>>2] = $4;
 $6 = HEAP32[$0>>2]|0;
 (_sqlite3WalkExpr($w,$6)|0);
 STACKTOP = sp;return;
}
function _sqlite3ExprAddCollateString($pParse,$pExpr,$zC) {
 $pParse = $pParse|0;
 $pExpr = $pExpr|0;
 $zC = $zC|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $s = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 16|0;
 $1 = sp + 12|0;
 $2 = sp + 8|0;
 $s = sp;
 HEAP32[$0>>2] = $pParse;
 HEAP32[$1>>2] = $pExpr;
 HEAP32[$2>>2] = $zC;
 $3 = HEAP32[$2>>2]|0;
 HEAP32[$s>>2] = $3;
 $4 = HEAP32[$s>>2]|0;
 $5 = (_sqlite3Strlen30($4)|0);
 $6 = (($s) + 4|0);
 HEAP32[$6>>2] = $5;
 $7 = HEAP32[$0>>2]|0;
 $8 = HEAP32[$1>>2]|0;
 $9 = (_sqlite3ExprAddCollateToken($7,$8,$s)|0);
 STACKTOP = sp;return ($9|0);
}
function _incrAggDepth($pWalker,$pExpr) {
 $pWalker = $pWalker|0;
 $pExpr = $pExpr|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $1 = sp;
 HEAP32[$0>>2] = $pWalker;
 HEAP32[$1>>2] = $pExpr;
 $2 = HEAP32[$1>>2]|0;
 $3 = HEAP8[$2>>0]|0;
 $4 = $3&255;
 $5 = ($4|0)==(155);
 if (!($5)) {
  STACKTOP = sp;return 0;
 }
 $6 = HEAP32[$0>>2]|0;
 $7 = (($6) + 20|0);
 $8 = HEAP32[$7>>2]|0;
 $9 = HEAP32[$1>>2]|0;
 $10 = (($9) + 38|0);
 $11 = HEAP8[$10>>0]|0;
 $12 = $11&255;
 $13 = (($12) + ($8))|0;
 $14 = $13&255;
 HEAP8[$10>>0] = $14;
 STACKTOP = sp;return 0;
}
function _sqlite3ExprCheckHeight($pParse,$nHeight) {
 $pParse = $pParse|0;
 $nHeight = $nHeight|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $mxHeight = 0, $rc = 0, $vararg_buffer = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $vararg_buffer = sp;
 $0 = sp + 16|0;
 $1 = sp + 12|0;
 $rc = sp + 8|0;
 $mxHeight = sp + 4|0;
 HEAP32[$0>>2] = $pParse;
 HEAP32[$1>>2] = $nHeight;
 HEAP32[$rc>>2] = 0;
 $2 = HEAP32[$0>>2]|0;
 $3 = HEAP32[$2>>2]|0;
 $4 = (($3) + 88|0);
 $5 = (($4) + 12|0);
 $6 = HEAP32[$5>>2]|0;
 HEAP32[$mxHeight>>2] = $6;
 $7 = HEAP32[$1>>2]|0;
 $8 = HEAP32[$mxHeight>>2]|0;
 $9 = ($7|0)>($8|0);
 if (!($9)) {
  $12 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($12|0);
 }
 $10 = HEAP32[$0>>2]|0;
 $11 = HEAP32[$mxHeight>>2]|0;
 HEAP32[$vararg_buffer>>2] = $11;
 _sqlite3ErrorMsg($10,16752,$vararg_buffer);
 HEAP32[$rc>>2] = 1;
 $12 = HEAP32[$rc>>2]|0;
 STACKTOP = sp;return ($12|0);
}
function _lookupName($pParse,$zDb,$zTab,$zCol,$pNC,$pExpr) {
 $pParse = $pParse|0;
 $zDb = $zDb|0;
 $zTab = $zTab|0;
 $zCol = $zCol|0;
 $pNC = $pNC|0;
 $pExpr = $pExpr|0;
 var $$ = 0, $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0;
 var $115 = 0, $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0;
 var $133 = 0, $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0;
 var $151 = 0, $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0;
 var $17 = 0, $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0;
 var $188 = 0, $189 = 0, $19 = 0, $190 = 0, $191 = 0, $192 = 0, $193 = 0, $194 = 0, $195 = 0, $196 = 0, $197 = 0, $198 = 0, $199 = 0, $2 = 0, $20 = 0, $200 = 0, $201 = 0, $202 = 0, $203 = 0, $204 = 0;
 var $205 = 0, $206 = 0, $207 = 0, $208 = 0, $209 = 0, $21 = 0, $210 = 0, $211 = 0, $212 = 0, $213 = 0, $214 = 0, $215 = 0, $216 = 0, $217 = 0, $218 = 0, $219 = 0, $22 = 0, $220 = 0, $221 = 0, $222 = 0;
 var $223 = 0, $224 = 0, $225 = 0, $226 = 0, $227 = 0, $228 = 0, $229 = 0, $23 = 0, $230 = 0, $231 = 0, $232 = 0, $233 = 0, $234 = 0, $235 = 0, $236 = 0, $237 = 0, $238 = 0, $239 = 0, $24 = 0, $240 = 0;
 var $241 = 0, $242 = 0, $243 = 0, $244 = 0, $245 = 0, $246 = 0, $247 = 0, $248 = 0, $249 = 0, $25 = 0, $250 = 0, $251 = 0, $252 = 0, $253 = 0, $254 = 0, $255 = 0, $256 = 0, $257 = 0, $258 = 0, $259 = 0;
 var $26 = 0, $260 = 0, $261 = 0, $262 = 0, $263 = 0, $264 = 0, $265 = 0, $266 = 0, $267 = 0, $268 = 0, $269 = 0, $27 = 0, $270 = 0, $271 = 0, $272 = 0, $273 = 0, $274 = 0, $275 = 0, $276 = 0, $277 = 0;
 var $278 = 0, $279 = 0, $28 = 0, $280 = 0, $281 = 0, $282 = 0, $283 = 0, $284 = 0, $285 = 0, $286 = 0, $287 = 0, $288 = 0, $289 = 0, $29 = 0, $290 = 0, $291 = 0, $292 = 0, $293 = 0, $294 = 0, $295 = 0;
 var $296 = 0, $297 = 0, $298 = 0, $299 = 0, $3 = 0, $30 = 0, $300 = 0, $301 = 0, $302 = 0, $303 = 0, $304 = 0, $305 = 0, $306 = 0, $307 = 0, $308 = 0, $309 = 0, $31 = 0, $310 = 0, $311 = 0, $312 = 0;
 var $313 = 0, $314 = 0, $315 = 0, $316 = 0, $317 = 0, $318 = 0, $319 = 0, $32 = 0, $320 = 0, $321 = 0, $322 = 0, $323 = 0, $324 = 0, $325 = 0, $326 = 0, $327 = 0, $328 = 0, $329 = 0, $33 = 0, $330 = 0;
 var $331 = 0, $332 = 0, $333 = 0, $334 = 0, $335 = 0, $336 = 0, $337 = 0, $338 = 0, $339 = 0, $34 = 0, $340 = 0, $341 = 0, $342 = 0, $343 = 0, $344 = 0, $345 = 0, $346 = 0, $347 = 0, $348 = 0, $349 = 0;
 var $35 = 0, $350 = 0, $351 = 0, $352 = 0, $353 = 0, $354 = 0, $355 = 0, $356 = 0, $357 = 0, $358 = 0, $359 = 0, $36 = 0, $360 = 0, $361 = 0, $362 = 0, $363 = 0, $364 = 0, $365 = 0, $366 = 0, $367 = 0;
 var $368 = 0, $369 = 0, $37 = 0, $370 = 0, $371 = 0, $372 = 0, $373 = 0, $374 = 0, $375 = 0, $376 = 0, $377 = 0, $378 = 0, $379 = 0, $38 = 0, $380 = 0, $381 = 0, $382 = 0, $383 = 0, $384 = 0, $385 = 0;
 var $386 = 0, $387 = 0, $388 = 0, $389 = 0, $39 = 0, $390 = 0, $391 = 0, $392 = 0, $393 = 0, $394 = 0, $395 = 0, $396 = 0, $397 = 0, $398 = 0, $399 = 0, $4 = 0, $40 = 0, $400 = 0, $401 = 0, $402 = 0;
 var $403 = 0, $404 = 0, $405 = 0, $406 = 0, $407 = 0, $408 = 0, $409 = 0, $41 = 0, $410 = 0, $411 = 0, $412 = 0, $413 = 0, $414 = 0, $415 = 0, $416 = 0, $417 = 0, $418 = 0, $419 = 0, $42 = 0, $420 = 0;
 var $421 = 0, $422 = 0, $423 = 0, $424 = 0, $425 = 0, $426 = 0, $427 = 0, $428 = 0, $429 = 0, $43 = 0, $430 = 0, $431 = 0, $432 = 0, $433 = 0, $434 = 0, $435 = 0, $436 = 0, $437 = 0, $438 = 0, $439 = 0;
 var $44 = 0, $440 = 0, $441 = 0, $442 = 0, $443 = 0, $444 = 0, $445 = 0, $446 = 0, $447 = 0, $448 = 0, $449 = 0, $45 = 0, $450 = 0, $451 = 0, $452 = 0, $453 = 0, $454 = 0, $455 = 0, $456 = 0, $457 = 0;
 var $458 = 0, $459 = 0, $46 = 0, $460 = 0, $461 = 0, $462 = 0, $463 = 0, $464 = 0, $465 = 0, $466 = 0, $467 = 0, $468 = 0, $469 = 0, $47 = 0, $470 = 0, $471 = 0, $472 = 0, $473 = 0, $474 = 0, $475 = 0;
 var $476 = 0, $477 = 0, $478 = 0, $479 = 0, $48 = 0, $480 = 0, $481 = 0, $482 = 0, $483 = 0, $484 = 0, $485 = 0, $486 = 0, $487 = 0, $488 = 0, $489 = 0, $49 = 0, $490 = 0, $491 = 0, $492 = 0, $493 = 0;
 var $494 = 0, $495 = 0, $496 = 0, $497 = 0, $498 = 0, $499 = 0, $5 = 0, $50 = 0, $500 = 0, $501 = 0, $502 = 0, $503 = 0, $504 = 0, $505 = 0, $506 = 0, $507 = 0, $508 = 0, $509 = 0, $51 = 0, $510 = 0;
 var $511 = 0, $512 = 0, $513 = 0, $514 = 0, $515 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0;
 var $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0;
 var $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $cnt = 0, $cntTab = 0, $db = 0;
 var $hit = 0, $i = 0, $iCol = 0, $isTrigger = 0, $j = 0, $n = 0, $nSubquery = 0, $op = 0, $or$cond = 0, $or$cond11 = 0, $or$cond13 = 0, $or$cond15 = 0, $or$cond17 = 0, $or$cond19 = 0, $or$cond3 = 0, $or$cond5 = 0, $or$cond7 = 0, $or$cond9 = 0, $pCol = 0, $pEList = 0;
 var $pItem = 0, $pMatch = 0, $pOrig = 0, $pSchema = 0, $pSrcList = 0, $pTab = 0, $pTopNC = 0, $vararg_buffer31 = 0, $vararg_ptr23 = 0, $vararg_ptr24 = 0, $vararg_ptr25 = 0, $vararg_ptr29 = 0, $vararg_ptr30 = 0, $vararg_ptr34 = 0, $zAs = 0, $zErr = 0, $zTabName = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 144|0;
 $vararg_buffer31 = sp;
 $0 = sp + 132|0;
 $1 = sp + 128|0;
 $2 = sp + 124|0;
 $3 = sp + 120|0;
 $4 = sp + 116|0;
 $5 = sp + 112|0;
 $6 = sp + 108|0;
 $i = sp + 104|0;
 $j = sp + 100|0;
 $cnt = sp + 96|0;
 $cntTab = sp + 92|0;
 $nSubquery = sp + 88|0;
 $db = sp + 84|0;
 $pItem = sp + 80|0;
 $pMatch = sp + 76|0;
 $pTopNC = sp + 72|0;
 $pSchema = sp + 68|0;
 $isTrigger = sp + 64|0;
 $pTab = sp + 60|0;
 $pCol = sp + 56|0;
 $pEList = sp + 52|0;
 $pSrcList = sp + 48|0;
 $hit = sp + 44|0;
 $zTabName = sp + 40|0;
 $op = sp + 36|0;
 $iCol = sp + 32|0;
 $zAs = sp + 28|0;
 $pOrig = sp + 24|0;
 $zErr = sp + 20|0;
 $n = sp + 16|0;
 HEAP32[$1>>2] = $pParse;
 HEAP32[$2>>2] = $zDb;
 HEAP32[$3>>2] = $zTab;
 HEAP32[$4>>2] = $zCol;
 HEAP32[$5>>2] = $pNC;
 HEAP32[$6>>2] = $pExpr;
 HEAP32[$cnt>>2] = 0;
 HEAP32[$cntTab>>2] = 0;
 HEAP32[$nSubquery>>2] = 0;
 $7 = HEAP32[$1>>2]|0;
 $8 = HEAP32[$7>>2]|0;
 HEAP32[$db>>2] = $8;
 HEAP32[$pMatch>>2] = 0;
 $9 = HEAP32[$5>>2]|0;
 HEAP32[$pTopNC>>2] = $9;
 HEAP32[$pSchema>>2] = 0;
 HEAP32[$isTrigger>>2] = 0;
 HEAP32[$pTab>>2] = 0;
 $10 = HEAP32[$6>>2]|0;
 $11 = (($10) + 28|0);
 HEAP32[$11>>2] = -1;
 $12 = HEAP32[$6>>2]|0;
 $13 = (($12) + 44|0);
 HEAP32[$13>>2] = 0;
 $14 = HEAP32[$2>>2]|0;
 $15 = ($14|0)!=(0|0);
 L1: do {
  if ($15) {
   $16 = HEAP32[$5>>2]|0;
   $17 = (($16) + 28|0);
   $18 = HEAP16[$17>>1]|0;
   $19 = $18&65535;
   $20 = $19 & 20;
   $21 = ($20|0)!=(0);
   if ($21) {
    HEAP32[$2>>2] = 0;
    break;
   }
   HEAP32[$i>>2] = 0;
   while(1) {
    $22 = HEAP32[$i>>2]|0;
    $23 = HEAP32[$db>>2]|0;
    $24 = (($23) + 20|0);
    $25 = HEAP32[$24>>2]|0;
    $26 = ($22|0)<($25|0);
    if (!($26)) {
     break L1;
    }
    $27 = HEAP32[$i>>2]|0;
    $28 = HEAP32[$db>>2]|0;
    $29 = (($28) + 16|0);
    $30 = HEAP32[$29>>2]|0;
    $31 = (($30) + ($27<<4)|0);
    $32 = HEAP32[$31>>2]|0;
    $33 = HEAP32[$2>>2]|0;
    $34 = (_sqlite3_stricmp($32,$33)|0);
    $35 = ($34|0)==(0);
    $36 = HEAP32[$i>>2]|0;
    if ($35) {
     break;
    }
    $43 = (($36) + 1)|0;
    HEAP32[$i>>2] = $43;
   }
   $37 = HEAP32[$db>>2]|0;
   $38 = (($37) + 16|0);
   $39 = HEAP32[$38>>2]|0;
   $40 = (($39) + ($36<<4)|0);
   $41 = (($40) + 12|0);
   $42 = HEAP32[$41>>2]|0;
   HEAP32[$pSchema>>2] = $42;
  }
 } while(0);
 L11: while(1) {
  $44 = HEAP32[$5>>2]|0;
  $45 = ($44|0)!=(0|0);
  $46 = HEAP32[$cnt>>2]|0;
  $47 = ($46|0)==(0);
  $or$cond19 = $45 & $47;
  if (!($or$cond19)) {
   label = 92;
   break;
  }
  $48 = HEAP32[$5>>2]|0;
  $49 = (($48) + 4|0);
  $50 = HEAP32[$49>>2]|0;
  HEAP32[$pSrcList>>2] = $50;
  $51 = HEAP32[$pSrcList>>2]|0;
  $52 = ($51|0)!=(0|0);
  if ($52) {
   HEAP32[$i>>2] = 0;
   $53 = HEAP32[$pSrcList>>2]|0;
   $54 = (($53) + 8|0);
   HEAP32[$pItem>>2] = $54;
   while(1) {
    $55 = HEAP32[$i>>2]|0;
    $56 = HEAP32[$pSrcList>>2]|0;
    $57 = HEAP32[$56>>2]|0;
    $58 = ($55|0)<($57|0);
    if (!($58)) {
     break;
    }
    $59 = HEAP32[$pItem>>2]|0;
    $60 = (($59) + 16|0);
    $61 = HEAP32[$60>>2]|0;
    HEAP32[$pTab>>2] = $61;
    $62 = HEAP32[$pItem>>2]|0;
    $63 = (($62) + 20|0);
    $64 = HEAP32[$63>>2]|0;
    $65 = ($64|0)!=(0|0);
    if ($65) {
     $66 = HEAP32[$pItem>>2]|0;
     $67 = (($66) + 20|0);
     $68 = HEAP32[$67>>2]|0;
     $69 = (($68) + 6|0);
     $70 = HEAP16[$69>>1]|0;
     $71 = $70&65535;
     $72 = $71 & 512;
     $73 = ($72|0)!=(0);
     if ($73) {
      HEAP32[$hit>>2] = 0;
      $74 = HEAP32[$pItem>>2]|0;
      $75 = (($74) + 20|0);
      $76 = HEAP32[$75>>2]|0;
      $77 = HEAP32[$76>>2]|0;
      HEAP32[$pEList>>2] = $77;
      HEAP32[$j>>2] = 0;
      while(1) {
       $78 = HEAP32[$j>>2]|0;
       $79 = HEAP32[$pEList>>2]|0;
       $80 = HEAP32[$79>>2]|0;
       $81 = ($78|0)<($80|0);
       if (!($81)) {
        break;
       }
       $82 = HEAP32[$j>>2]|0;
       $83 = HEAP32[$pEList>>2]|0;
       $84 = (($83) + 4|0);
       $85 = HEAP32[$84>>2]|0;
       $86 = (($85) + (($82*20)|0)|0);
       $87 = (($86) + 8|0);
       $88 = HEAP32[$87>>2]|0;
       $89 = HEAP32[$4>>2]|0;
       $90 = HEAP32[$3>>2]|0;
       $91 = HEAP32[$2>>2]|0;
       $92 = (_sqlite3MatchSpanName($88,$89,$90,$91)|0);
       $93 = ($92|0)!=(0);
       if ($93) {
        $94 = HEAP32[$cnt>>2]|0;
        $95 = (($94) + 1)|0;
        HEAP32[$cnt>>2] = $95;
        HEAP32[$cntTab>>2] = 2;
        $96 = HEAP32[$pItem>>2]|0;
        HEAP32[$pMatch>>2] = $96;
        $97 = HEAP32[$j>>2]|0;
        $98 = $97&65535;
        $99 = HEAP32[$6>>2]|0;
        $100 = (($99) + 32|0);
        HEAP16[$100>>1] = $98;
        HEAP32[$hit>>2] = 1;
       }
       $101 = HEAP32[$j>>2]|0;
       $102 = (($101) + 1)|0;
       HEAP32[$j>>2] = $102;
      }
      $103 = HEAP32[$hit>>2]|0;
      $104 = ($103|0)!=(0);
      $105 = HEAP32[$3>>2]|0;
      $106 = ($105|0)==(0|0);
      $or$cond = $104 | $106;
      if (!($or$cond)) {
       label = 21;
      }
     } else {
      label = 21;
     }
    } else {
     label = 21;
    }
    L29: do {
     if ((label|0) == 21) {
      label = 0;
      $107 = HEAP32[$2>>2]|0;
      $108 = ($107|0)!=(0|0);
      if ($108) {
       $109 = HEAP32[$pTab>>2]|0;
       $110 = (($109) + 68|0);
       $111 = HEAP32[$110>>2]|0;
       $112 = HEAP32[$pSchema>>2]|0;
       $113 = ($111|0)!=($112|0);
       if ($113) {
        break;
       }
      }
      $114 = HEAP32[$3>>2]|0;
      $115 = ($114|0)!=(0|0);
      if ($115) {
       $116 = HEAP32[$pItem>>2]|0;
       $117 = (($116) + 12|0);
       $118 = HEAP32[$117>>2]|0;
       $119 = ($118|0)!=(0|0);
       if ($119) {
        $120 = HEAP32[$pItem>>2]|0;
        $121 = (($120) + 12|0);
        $122 = HEAP32[$121>>2]|0;
        $125 = $122;
       } else {
        $123 = HEAP32[$pTab>>2]|0;
        $124 = HEAP32[$123>>2]|0;
        $125 = $124;
       }
       HEAP32[$zTabName>>2] = $125;
       $126 = HEAP32[$zTabName>>2]|0;
       $127 = HEAP32[$3>>2]|0;
       $128 = (_sqlite3_stricmp($126,$127)|0);
       $129 = ($128|0)!=(0);
       if ($129) {
        break;
       }
      }
      $130 = HEAP32[$cntTab>>2]|0;
      $131 = (($130) + 1)|0;
      HEAP32[$cntTab>>2] = $131;
      $132 = (0)==($130|0);
      if ($132) {
       $133 = HEAP32[$pItem>>2]|0;
       HEAP32[$pMatch>>2] = $133;
      }
      HEAP32[$j>>2] = 0;
      $134 = HEAP32[$pTab>>2]|0;
      $135 = (($134) + 4|0);
      $136 = HEAP32[$135>>2]|0;
      HEAP32[$pCol>>2] = $136;
      while(1) {
       $137 = HEAP32[$j>>2]|0;
       $138 = HEAP32[$pTab>>2]|0;
       $139 = (($138) + 38|0);
       $140 = HEAP16[$139>>1]|0;
       $141 = $140 << 16 >> 16;
       $142 = ($137|0)<($141|0);
       if (!($142)) {
        break L29;
       }
       $143 = HEAP32[$pCol>>2]|0;
       $144 = HEAP32[$143>>2]|0;
       $145 = HEAP32[$4>>2]|0;
       $146 = (_sqlite3_stricmp($144,$145)|0);
       $147 = ($146|0)==(0);
       if ($147) {
        $148 = HEAP32[$cnt>>2]|0;
        $149 = ($148|0)==(1);
        if (!($149)) {
         break;
        }
        $150 = HEAP32[$pItem>>2]|0;
        $151 = (($150) + 36|0);
        $152 = HEAP8[$151>>0]|0;
        $153 = $152&255;
        $154 = $153 & 4;
        $155 = ($154|0)!=(0);
        if (!($155)) {
         $156 = HEAP32[$pItem>>2]|0;
         $157 = (($156) + 48|0);
         $158 = HEAP32[$157>>2]|0;
         $159 = HEAP32[$4>>2]|0;
         $160 = (_nameInUsingClause($158,$159)|0);
         $161 = ($160|0)!=(0);
         if (!($161)) {
          break;
         }
        }
       }
       $178 = HEAP32[$j>>2]|0;
       $179 = (($178) + 1)|0;
       HEAP32[$j>>2] = $179;
       $180 = HEAP32[$pCol>>2]|0;
       $181 = (($180) + 24|0);
       HEAP32[$pCol>>2] = $181;
      }
      $162 = HEAP32[$cnt>>2]|0;
      $163 = (($162) + 1)|0;
      HEAP32[$cnt>>2] = $163;
      $164 = HEAP32[$pItem>>2]|0;
      HEAP32[$pMatch>>2] = $164;
      $165 = HEAP32[$j>>2]|0;
      $166 = HEAP32[$pTab>>2]|0;
      $167 = (($166) + 36|0);
      $168 = HEAP16[$167>>1]|0;
      $169 = $168 << 16 >> 16;
      $170 = ($165|0)==($169|0);
      if ($170) {
       $175 = -1;
      } else {
       $171 = HEAP32[$j>>2]|0;
       $172 = $171&65535;
       $173 = $172 << 16 >> 16;
       $175 = $173;
      }
      $174 = $175&65535;
      $176 = HEAP32[$6>>2]|0;
      $177 = (($176) + 32|0);
      HEAP16[$177>>1] = $174;
     }
    } while(0);
    $182 = HEAP32[$i>>2]|0;
    $183 = (($182) + 1)|0;
    HEAP32[$i>>2] = $183;
    $184 = HEAP32[$pItem>>2]|0;
    $185 = (($184) + 72|0);
    HEAP32[$pItem>>2] = $185;
   }
   $186 = HEAP32[$pMatch>>2]|0;
   $187 = ($186|0)!=(0|0);
   if ($187) {
    $188 = HEAP32[$pMatch>>2]|0;
    $189 = (($188) + 40|0);
    $190 = HEAP32[$189>>2]|0;
    $191 = HEAP32[$6>>2]|0;
    $192 = (($191) + 28|0);
    HEAP32[$192>>2] = $190;
    $193 = HEAP32[$pMatch>>2]|0;
    $194 = (($193) + 16|0);
    $195 = HEAP32[$194>>2]|0;
    $196 = HEAP32[$6>>2]|0;
    $197 = (($196) + 44|0);
    HEAP32[$197>>2] = $195;
    $198 = HEAP32[$pMatch>>2]|0;
    $199 = (($198) + 36|0);
    $200 = HEAP8[$199>>0]|0;
    $201 = $200&255;
    $202 = $201 & 8;
    $203 = ($202|0)!=(0);
    if ($203) {
     $204 = HEAP32[$6>>2]|0;
     $205 = (($204) + 4|0);
     $206 = HEAP32[$205>>2]|0;
     $207 = $206 | 1048576;
     HEAP32[$205>>2] = $207;
    }
    $208 = HEAP32[$6>>2]|0;
    $209 = (($208) + 44|0);
    $210 = HEAP32[$209>>2]|0;
    $211 = (($210) + 68|0);
    $212 = HEAP32[$211>>2]|0;
    HEAP32[$pSchema>>2] = $212;
   }
  }
  $213 = HEAP32[$2>>2]|0;
  $214 = ($213|0)==(0|0);
  $215 = HEAP32[$3>>2]|0;
  $216 = ($215|0)!=(0|0);
  $or$cond3 = $214 & $216;
  $217 = HEAP32[$cntTab>>2]|0;
  $218 = ($217|0)==(0);
  $or$cond5 = $or$cond3 & $218;
  if ($or$cond5) {
   $219 = HEAP32[$1>>2]|0;
   $220 = (($219) + 412|0);
   $221 = HEAP32[$220>>2]|0;
   $222 = ($221|0)!=(0|0);
   if ($222) {
    $223 = HEAP32[$1>>2]|0;
    $224 = (($223) + 436|0);
    $225 = HEAP8[$224>>0]|0;
    $226 = $225&255;
    HEAP32[$op>>2] = $226;
    $227 = HEAP32[$op>>2]|0;
    $228 = ($227|0)!=(109);
    if ($228) {
     $229 = HEAP32[$3>>2]|0;
     $230 = (_sqlite3_stricmp(17160,$229)|0);
     $231 = ($230|0)==(0);
     if ($231) {
      $232 = HEAP32[$6>>2]|0;
      $233 = (($232) + 28|0);
      HEAP32[$233>>2] = 1;
      $234 = HEAP32[$1>>2]|0;
      $235 = (($234) + 412|0);
      $236 = HEAP32[$235>>2]|0;
      HEAP32[$pTab>>2] = $236;
     } else {
      label = 50;
     }
    } else {
     label = 50;
    }
    do {
     if ((label|0) == 50) {
      label = 0;
      $237 = HEAP32[$op>>2]|0;
      $238 = ($237|0)!=(108);
      if ($238) {
       $239 = HEAP32[$3>>2]|0;
       $240 = (_sqlite3_stricmp(17168,$239)|0);
       $241 = ($240|0)==(0);
       if ($241) {
        $242 = HEAP32[$6>>2]|0;
        $243 = (($242) + 28|0);
        HEAP32[$243>>2] = 0;
        $244 = HEAP32[$1>>2]|0;
        $245 = (($244) + 412|0);
        $246 = HEAP32[$245>>2]|0;
        HEAP32[$pTab>>2] = $246;
        break;
       }
      }
      HEAP32[$pTab>>2] = 0;
     }
    } while(0);
    $247 = HEAP32[$pTab>>2]|0;
    $248 = ($247|0)!=(0|0);
    if ($248) {
     $249 = HEAP32[$pTab>>2]|0;
     $250 = (($249) + 68|0);
     $251 = HEAP32[$250>>2]|0;
     HEAP32[$pSchema>>2] = $251;
     $252 = HEAP32[$cntTab>>2]|0;
     $253 = (($252) + 1)|0;
     HEAP32[$cntTab>>2] = $253;
     HEAP32[$iCol>>2] = 0;
     $254 = HEAP32[$pTab>>2]|0;
     $255 = (($254) + 4|0);
     $256 = HEAP32[$255>>2]|0;
     HEAP32[$pCol>>2] = $256;
     while(1) {
      $257 = HEAP32[$iCol>>2]|0;
      $258 = HEAP32[$pTab>>2]|0;
      $259 = (($258) + 38|0);
      $260 = HEAP16[$259>>1]|0;
      $261 = $260 << 16 >> 16;
      $262 = ($257|0)<($261|0);
      if (!($262)) {
       break;
      }
      $263 = HEAP32[$pCol>>2]|0;
      $264 = HEAP32[$263>>2]|0;
      $265 = HEAP32[$4>>2]|0;
      $266 = (_sqlite3_stricmp($264,$265)|0);
      $267 = ($266|0)==(0);
      $268 = HEAP32[$iCol>>2]|0;
      if ($267) {
       label = 58;
       break;
      }
      $274 = (($268) + 1)|0;
      HEAP32[$iCol>>2] = $274;
      $275 = HEAP32[$pCol>>2]|0;
      $276 = (($275) + 24|0);
      HEAP32[$pCol>>2] = $276;
     }
     if ((label|0) == 58) {
      label = 0;
      $269 = HEAP32[$pTab>>2]|0;
      $270 = (($269) + 36|0);
      $271 = HEAP16[$270>>1]|0;
      $272 = $271 << 16 >> 16;
      $273 = ($268|0)==($272|0);
      if ($273) {
       HEAP32[$iCol>>2] = -1;
      }
     }
     $277 = HEAP32[$iCol>>2]|0;
     $278 = HEAP32[$pTab>>2]|0;
     $279 = (($278) + 38|0);
     $280 = HEAP16[$279>>1]|0;
     $281 = $280 << 16 >> 16;
     $282 = ($277|0)>=($281|0);
     if ($282) {
      $283 = HEAP32[$4>>2]|0;
      $284 = (_sqlite3IsRowid($283)|0);
      $285 = ($284|0)!=(0);
      if ($285) {
       $286 = HEAP32[$pTab>>2]|0;
       $287 = (($286) + 44|0);
       $288 = HEAP8[$287>>0]|0;
       $289 = $288&255;
       $290 = $289 & 32;
       $291 = ($290|0)==(0);
       if ($291) {
        HEAP32[$iCol>>2] = -1;
       }
      }
     }
     $292 = HEAP32[$iCol>>2]|0;
     $293 = HEAP32[$pTab>>2]|0;
     $294 = (($293) + 38|0);
     $295 = HEAP16[$294>>1]|0;
     $296 = $295 << 16 >> 16;
     $297 = ($292|0)<($296|0);
     if ($297) {
      $298 = HEAP32[$cnt>>2]|0;
      $299 = (($298) + 1)|0;
      HEAP32[$cnt>>2] = $299;
      $300 = HEAP32[$iCol>>2]|0;
      $301 = ($300|0)<(0);
      $302 = HEAP32[$6>>2]|0;
      do {
       if ($301) {
        $303 = (($302) + 1|0);
        HEAP8[$303>>0] = 68;
       } else {
        $304 = (($302) + 28|0);
        $305 = HEAP32[$304>>2]|0;
        $306 = ($305|0)==(0);
        $307 = HEAP32[$iCol>>2]|0;
        $308 = ($307|0)>=(32);
        if ($306) {
         if ($308) {
          $315 = -1;
         } else {
          $309 = HEAP32[$iCol>>2]|0;
          $310 = 1 << $309;
          $315 = $310;
         }
         $311 = HEAP32[$1>>2]|0;
         $312 = (($311) + 428|0);
         $313 = HEAP32[$312>>2]|0;
         $314 = $313 | $315;
         HEAP32[$312>>2] = $314;
         break;
        } else {
         if ($308) {
          $322 = -1;
         } else {
          $316 = HEAP32[$iCol>>2]|0;
          $317 = 1 << $316;
          $322 = $317;
         }
         $318 = HEAP32[$1>>2]|0;
         $319 = (($318) + 432|0);
         $320 = HEAP32[$319>>2]|0;
         $321 = $320 | $322;
         HEAP32[$319>>2] = $321;
         break;
        }
       }
      } while(0);
      $323 = HEAP32[$iCol>>2]|0;
      $324 = $323&65535;
      $325 = HEAP32[$6>>2]|0;
      $326 = (($325) + 32|0);
      HEAP16[$326>>1] = $324;
      $327 = HEAP32[$pTab>>2]|0;
      $328 = HEAP32[$6>>2]|0;
      $329 = (($328) + 44|0);
      HEAP32[$329>>2] = $327;
      HEAP32[$isTrigger>>2] = 1;
     }
    }
   }
  }
  $330 = HEAP32[$cnt>>2]|0;
  $331 = ($330|0)==(0);
  $332 = HEAP32[$cntTab>>2]|0;
  $333 = ($332|0)==(1);
  $or$cond7 = $331 & $333;
  $334 = HEAP32[$pMatch>>2]|0;
  $335 = ($334|0)!=(0|0);
  $or$cond9 = $or$cond7 & $335;
  if ($or$cond9) {
   $336 = HEAP32[$4>>2]|0;
   $337 = (_sqlite3IsRowid($336)|0);
   $338 = ($337|0)!=(0);
   if ($338) {
    $339 = HEAP32[$pMatch>>2]|0;
    $340 = (($339) + 16|0);
    $341 = HEAP32[$340>>2]|0;
    $342 = (($341) + 44|0);
    $343 = HEAP8[$342>>0]|0;
    $344 = $343&255;
    $345 = $344 & 32;
    $346 = ($345|0)==(0);
    if ($346) {
     HEAP32[$cnt>>2] = 1;
     $347 = HEAP32[$6>>2]|0;
     $348 = (($347) + 32|0);
     HEAP16[$348>>1] = -1;
     $349 = HEAP32[$6>>2]|0;
     $350 = (($349) + 1|0);
     HEAP8[$350>>0] = 68;
    }
   }
  }
  $351 = HEAP32[$5>>2]|0;
  $352 = (($351) + 8|0);
  $353 = HEAP32[$352>>2]|0;
  HEAP32[$pEList>>2] = $353;
  $354 = ($353|0)!=(0|0);
  $355 = HEAP32[$3>>2]|0;
  $356 = ($355|0)==(0|0);
  $or$cond11 = $354 & $356;
  $357 = HEAP32[$cnt>>2]|0;
  $358 = ($357|0)==(0);
  $or$cond13 = $or$cond11 & $358;
  L110: do {
   if ($or$cond13) {
    HEAP32[$j>>2] = 0;
    while(1) {
     $359 = HEAP32[$j>>2]|0;
     $360 = HEAP32[$pEList>>2]|0;
     $361 = HEAP32[$360>>2]|0;
     $362 = ($359|0)<($361|0);
     if (!($362)) {
      break L110;
     }
     $363 = HEAP32[$j>>2]|0;
     $364 = HEAP32[$pEList>>2]|0;
     $365 = (($364) + 4|0);
     $366 = HEAP32[$365>>2]|0;
     $367 = (($366) + (($363*20)|0)|0);
     $368 = (($367) + 4|0);
     $369 = HEAP32[$368>>2]|0;
     HEAP32[$zAs>>2] = $369;
     $370 = HEAP32[$zAs>>2]|0;
     $371 = ($370|0)!=(0|0);
     if ($371) {
      $372 = HEAP32[$zAs>>2]|0;
      $373 = HEAP32[$4>>2]|0;
      $374 = (_sqlite3_stricmp($372,$373)|0);
      $375 = ($374|0)==(0);
      if ($375) {
       label = 85;
       break L11;
      }
     }
     $400 = HEAP32[$j>>2]|0;
     $401 = (($400) + 1)|0;
     HEAP32[$j>>2] = $401;
    }
   }
  } while(0);
  $402 = HEAP32[$cnt>>2]|0;
  $403 = ($402|0)==(0);
  if (!($403)) {
   continue;
  }
  $404 = HEAP32[$5>>2]|0;
  $405 = (($404) + 16|0);
  $406 = HEAP32[$405>>2]|0;
  HEAP32[$5>>2] = $406;
  $407 = HEAP32[$nSubquery>>2]|0;
  $408 = (($407) + 1)|0;
  HEAP32[$nSubquery>>2] = $408;
 }
 if ((label|0) == 85) {
  $376 = HEAP32[$j>>2]|0;
  $377 = HEAP32[$pEList>>2]|0;
  $378 = (($377) + 4|0);
  $379 = HEAP32[$378>>2]|0;
  $380 = (($379) + (($376*20)|0)|0);
  $381 = HEAP32[$380>>2]|0;
  HEAP32[$pOrig>>2] = $381;
  $382 = HEAP32[$5>>2]|0;
  $383 = (($382) + 28|0);
  $384 = HEAP16[$383>>1]|0;
  $385 = $384&65535;
  $386 = $385 & 1;
  $387 = ($386|0)==(0);
  if ($387) {
   $388 = HEAP32[$pOrig>>2]|0;
   $389 = (($388) + 4|0);
   $390 = HEAP32[$389>>2]|0;
   $391 = $390 & 2;
   $392 = ($391|0)!=(0);
   if ($392) {
    $393 = HEAP32[$1>>2]|0;
    $394 = HEAP32[$zAs>>2]|0;
    HEAP32[$vararg_buffer31>>2] = $394;
    _sqlite3ErrorMsg($393,17176,$vararg_buffer31);
    HEAP32[$0>>2] = 2;
    $515 = HEAP32[$0>>2]|0;
    STACKTOP = sp;return ($515|0);
   }
  }
  $395 = HEAP32[$1>>2]|0;
  $396 = HEAP32[$pEList>>2]|0;
  $397 = HEAP32[$j>>2]|0;
  $398 = HEAP32[$6>>2]|0;
  $399 = HEAP32[$nSubquery>>2]|0;
  _resolveAlias($395,$396,$397,$398,1216,$399);
  HEAP32[$cnt>>2] = 1;
  HEAP32[$pMatch>>2] = 0;
 }
 else if ((label|0) == 92) {
  $409 = HEAP32[$cnt>>2]|0;
  $410 = ($409|0)==(0);
  $411 = HEAP32[$3>>2]|0;
  $412 = ($411|0)==(0|0);
  $or$cond15 = $410 & $412;
  if ($or$cond15) {
   $413 = HEAP32[$6>>2]|0;
   $414 = (($413) + 4|0);
   $415 = HEAP32[$414>>2]|0;
   $416 = $415 & 64;
   $417 = ($416|0)!=(0);
   if ($417) {
    $418 = HEAP32[$6>>2]|0;
    HEAP8[$418>>0] = 97;
    $419 = HEAP32[$6>>2]|0;
    $420 = (($419) + 44|0);
    HEAP32[$420>>2] = 0;
    HEAP32[$0>>2] = 1;
    $515 = HEAP32[$0>>2]|0;
    STACKTOP = sp;return ($515|0);
   }
  }
  $421 = HEAP32[$cnt>>2]|0;
  $422 = ($421|0)!=(1);
  if ($422) {
   $423 = HEAP32[$cnt>>2]|0;
   $424 = ($423|0)==(0);
   $425 = $424 ? 17208 : 17224;
   HEAP32[$zErr>>2] = $425;
   $426 = HEAP32[$2>>2]|0;
   $427 = ($426|0)!=(0|0);
   do {
    if ($427) {
     $428 = HEAP32[$1>>2]|0;
     $429 = HEAP32[$zErr>>2]|0;
     $430 = HEAP32[$2>>2]|0;
     $431 = HEAP32[$3>>2]|0;
     $432 = HEAP32[$4>>2]|0;
     HEAP32[$vararg_buffer31>>2] = $429;
     $vararg_ptr23 = (($vararg_buffer31) + 4|0);
     HEAP32[$vararg_ptr23>>2] = $430;
     $vararg_ptr24 = (($vararg_buffer31) + 8|0);
     HEAP32[$vararg_ptr24>>2] = $431;
     $vararg_ptr25 = (($vararg_buffer31) + 12|0);
     HEAP32[$vararg_ptr25>>2] = $432;
     _sqlite3ErrorMsg($428,17248,$vararg_buffer31);
    } else {
     $433 = HEAP32[$3>>2]|0;
     $434 = ($433|0)!=(0|0);
     $435 = HEAP32[$1>>2]|0;
     $436 = HEAP32[$zErr>>2]|0;
     if ($434) {
      $437 = HEAP32[$3>>2]|0;
      $438 = HEAP32[$4>>2]|0;
      HEAP32[$vararg_buffer31>>2] = $436;
      $vararg_ptr29 = (($vararg_buffer31) + 4|0);
      HEAP32[$vararg_ptr29>>2] = $437;
      $vararg_ptr30 = (($vararg_buffer31) + 8|0);
      HEAP32[$vararg_ptr30>>2] = $438;
      _sqlite3ErrorMsg($435,17264,$vararg_buffer31);
      break;
     } else {
      $439 = HEAP32[$4>>2]|0;
      HEAP32[$vararg_buffer31>>2] = $436;
      $vararg_ptr34 = (($vararg_buffer31) + 4|0);
      HEAP32[$vararg_ptr34>>2] = $439;
      _sqlite3ErrorMsg($435,17280,$vararg_buffer31);
      break;
     }
    }
   } while(0);
   $440 = HEAP32[$1>>2]|0;
   $441 = (($440) + 17|0);
   HEAP8[$441>>0] = 1;
   $442 = HEAP32[$pTopNC>>2]|0;
   $443 = (($442) + 24|0);
   $444 = HEAP32[$443>>2]|0;
   $445 = (($444) + 1)|0;
   HEAP32[$443>>2] = $445;
  }
  $446 = HEAP32[$6>>2]|0;
  $447 = (($446) + 32|0);
  $448 = HEAP16[$447>>1]|0;
  $449 = $448 << 16 >> 16;
  $450 = ($449|0)>=(0);
  $451 = HEAP32[$pMatch>>2]|0;
  $452 = ($451|0)!=(0|0);
  $or$cond17 = $450 & $452;
  if ($or$cond17) {
   $453 = HEAP32[$6>>2]|0;
   $454 = (($453) + 32|0);
   $455 = HEAP16[$454>>1]|0;
   $456 = $455 << 16 >> 16;
   HEAP32[$n>>2] = $456;
   $457 = HEAP32[$n>>2]|0;
   $458 = ($457|0)>=(64);
   $$ = $458 ? 63 : $456;
   HEAP32[$n>>2] = $$;
   $459 = HEAP32[$n>>2]|0;
   $460 = (_bitshift64Shl(1,0,($459|0))|0);
   $461 = tempRet0;
   $462 = HEAP32[$pMatch>>2]|0;
   $463 = (($462) + 56|0);
   $464 = $463;
   $465 = $464;
   $466 = HEAP32[$465>>2]|0;
   $467 = (($464) + 4)|0;
   $468 = $467;
   $469 = HEAP32[$468>>2]|0;
   $470 = $466 | $460;
   $471 = $469 | $461;
   $472 = $463;
   $473 = $472;
   HEAP32[$473>>2] = $470;
   $474 = (($472) + 4)|0;
   $475 = $474;
   HEAP32[$475>>2] = $471;
  }
  $476 = HEAP32[$db>>2]|0;
  $477 = HEAP32[$6>>2]|0;
  $478 = (($477) + 12|0);
  $479 = HEAP32[$478>>2]|0;
  _sqlite3ExprDelete($476,$479);
  $480 = HEAP32[$6>>2]|0;
  $481 = (($480) + 12|0);
  HEAP32[$481>>2] = 0;
  $482 = HEAP32[$db>>2]|0;
  $483 = HEAP32[$6>>2]|0;
  $484 = (($483) + 16|0);
  $485 = HEAP32[$484>>2]|0;
  _sqlite3ExprDelete($482,$485);
  $486 = HEAP32[$6>>2]|0;
  $487 = (($486) + 16|0);
  HEAP32[$487>>2] = 0;
  $488 = HEAP32[$isTrigger>>2]|0;
  $489 = ($488|0)!=(0);
  $490 = $489 ? 62 : 154;
  $491 = $490&255;
  $492 = HEAP32[$6>>2]|0;
  HEAP8[$492>>0] = $491;
 }
 $493 = HEAP32[$cnt>>2]|0;
 $494 = ($493|0)==(1);
 if (!($494)) {
  HEAP32[$0>>2] = 2;
  $515 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($515|0);
 }
 $495 = HEAP32[$6>>2]|0;
 $496 = HEAP8[$495>>0]|0;
 $497 = $496&255;
 $498 = ($497|0)!=(24);
 if ($498) {
  $499 = HEAP32[$1>>2]|0;
  $500 = HEAP32[$6>>2]|0;
  $501 = HEAP32[$pSchema>>2]|0;
  $502 = HEAP32[$5>>2]|0;
  $503 = (($502) + 4|0);
  $504 = HEAP32[$503>>2]|0;
  _sqlite3AuthRead($499,$500,$501,$504);
 }
 while(1) {
  $505 = HEAP32[$pTopNC>>2]|0;
  $506 = (($505) + 20|0);
  $507 = HEAP32[$506>>2]|0;
  $508 = (($507) + 1)|0;
  HEAP32[$506>>2] = $508;
  $509 = HEAP32[$pTopNC>>2]|0;
  $510 = HEAP32[$5>>2]|0;
  $511 = ($509|0)==($510|0);
  if ($511) {
   break;
  }
  $512 = HEAP32[$pTopNC>>2]|0;
  $513 = (($512) + 16|0);
  $514 = HEAP32[$513>>2]|0;
  HEAP32[$pTopNC>>2] = $514;
 }
 HEAP32[$0>>2] = 1;
 $515 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($515|0);
}
function _notValidPartIdxWhere($pParse,$pNC,$zMsg) {
 $pParse = $pParse|0;
 $pNC = $pNC|0;
 $zMsg = $zMsg|0;
 var $0 = 0, $1 = 0, $10 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $vararg_buffer = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $vararg_buffer = sp;
 $0 = sp + 12|0;
 $1 = sp + 8|0;
 $2 = sp + 4|0;
 HEAP32[$0>>2] = $pParse;
 HEAP32[$1>>2] = $pNC;
 HEAP32[$2>>2] = $zMsg;
 $3 = HEAP32[$1>>2]|0;
 $4 = (($3) + 28|0);
 $5 = HEAP16[$4>>1]|0;
 $6 = $5&65535;
 $7 = $6 & 16;
 $8 = ($7|0)!=(0);
 if (!($8)) {
  STACKTOP = sp;return;
 }
 $9 = HEAP32[$0>>2]|0;
 $10 = HEAP32[$2>>2]|0;
 HEAP32[$vararg_buffer>>2] = $10;
 _sqlite3ErrorMsg($9,17112,$vararg_buffer);
 STACKTOP = sp;return;
}
function _exprProbability($p) {
 $p = $p|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0.0, $14 = 0, $15 = 0.0, $16 = 0.0, $17 = 0, $18 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $r = 0;
 var label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $1 = sp + 8|0;
 $r = sp;
 HEAP32[$1>>2] = $p;
 HEAPF64[$r>>3] = -1.0;
 $2 = HEAP32[$1>>2]|0;
 $3 = HEAP8[$2>>0]|0;
 $4 = $3&255;
 $5 = ($4|0)!=(133);
 if ($5) {
  HEAP32[$0>>2] = -1;
  $18 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($18|0);
 }
 $6 = HEAP32[$1>>2]|0;
 $7 = (($6) + 8|0);
 $8 = HEAP32[$7>>2]|0;
 $9 = HEAP32[$1>>2]|0;
 $10 = (($9) + 8|0);
 $11 = HEAP32[$10>>2]|0;
 $12 = (_sqlite3Strlen30($11)|0);
 (_sqlite3AtoF($8,$r,$12,1)|0);
 $13 = +HEAPF64[$r>>3];
 $14 = $13 > 1.0;
 if ($14) {
  HEAP32[$0>>2] = -1;
  $18 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($18|0);
 } else {
  $15 = +HEAPF64[$r>>3];
  $16 = $15 * 1000.0;
  $17 = (~~(($16)));
  HEAP32[$0>>2] = $17;
  $18 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($18|0);
 }
 return 0|0;
}
function _sqlite3FunctionUsesThisSrc($pExpr,$pSrcList) {
 $pExpr = $pExpr|0;
 $pSrcList = $pSrcList|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $cnt = 0, $w = 0, label = 0;
 var sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $0 = sp + 44|0;
 $1 = sp + 40|0;
 $w = sp + 16|0;
 $cnt = sp;
 HEAP32[$0>>2] = $pExpr;
 HEAP32[$1>>2] = $pSrcList;
 ;HEAP32[$w+0>>2]=0|0;HEAP32[$w+4>>2]=0|0;HEAP32[$w+8>>2]=0|0;HEAP32[$w+12>>2]=0|0;HEAP32[$w+16>>2]=0|0;HEAP32[$w+20>>2]=0|0;
 HEAP32[$w>>2] = 43;
 $2 = (($w) + 20|0);
 HEAP32[$2>>2] = $cnt;
 $3 = HEAP32[$1>>2]|0;
 HEAP32[$cnt>>2] = $3;
 $4 = (($cnt) + 4|0);
 HEAP32[$4>>2] = 0;
 $5 = (($cnt) + 8|0);
 HEAP32[$5>>2] = 0;
 $6 = HEAP32[$0>>2]|0;
 $7 = (($6) + 20|0);
 $8 = HEAP32[$7>>2]|0;
 (_sqlite3WalkExprList($w,$8)|0);
 $9 = (($cnt) + 4|0);
 $10 = HEAP32[$9>>2]|0;
 $11 = ($10|0)>(0);
 if ($11) {
  $16 = 1;
  $15 = $16&1;
  STACKTOP = sp;return ($15|0);
 }
 $12 = (($cnt) + 8|0);
 $13 = HEAP32[$12>>2]|0;
 $14 = ($13|0)==(0);
 $16 = $14;
 $15 = $16&1;
 STACKTOP = sp;return ($15|0);
}
function _notValidCheckConstraint($pParse,$pNC,$zMsg) {
 $pParse = $pParse|0;
 $pNC = $pNC|0;
 $zMsg = $zMsg|0;
 var $0 = 0, $1 = 0, $10 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $vararg_buffer = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $vararg_buffer = sp;
 $0 = sp + 12|0;
 $1 = sp + 8|0;
 $2 = sp + 4|0;
 HEAP32[$0>>2] = $pParse;
 HEAP32[$1>>2] = $pNC;
 HEAP32[$2>>2] = $zMsg;
 $3 = HEAP32[$1>>2]|0;
 $4 = (($3) + 28|0);
 $5 = HEAP16[$4>>1]|0;
 $6 = $5&65535;
 $7 = $6 & 4;
 $8 = ($7|0)!=(0);
 if (!($8)) {
  STACKTOP = sp;return;
 }
 $9 = HEAP32[$0>>2]|0;
 $10 = HEAP32[$2>>2]|0;
 HEAP32[$vararg_buffer>>2] = $10;
 _sqlite3ErrorMsg($9,17072,$vararg_buffer);
 STACKTOP = sp;return;
}
function _exprSrcCount($pWalker,$pExpr) {
 $pWalker = $pWalker|0;
 $pExpr = $pExpr|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $5 = 0, $6 = 0, $7 = 0;
 var $8 = 0, $9 = 0, $i = 0, $p = 0, $pSrc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 16|0;
 $1 = sp + 12|0;
 $i = sp + 8|0;
 $p = sp + 4|0;
 $pSrc = sp;
 HEAP32[$0>>2] = $pWalker;
 HEAP32[$1>>2] = $pExpr;
 $2 = HEAP32[$1>>2]|0;
 $3 = HEAP8[$2>>0]|0;
 $4 = $3&255;
 $5 = ($4|0)==(154);
 if (!($5)) {
  $6 = HEAP32[$1>>2]|0;
  $7 = HEAP8[$6>>0]|0;
  $8 = $7&255;
  $9 = ($8|0)==(156);
  if (!($9)) {
   STACKTOP = sp;return 0;
  }
 }
 $10 = HEAP32[$0>>2]|0;
 $11 = (($10) + 20|0);
 $12 = HEAP32[$11>>2]|0;
 HEAP32[$p>>2] = $12;
 $13 = HEAP32[$p>>2]|0;
 $14 = HEAP32[$13>>2]|0;
 HEAP32[$pSrc>>2] = $14;
 HEAP32[$i>>2] = 0;
 while(1) {
  $15 = HEAP32[$i>>2]|0;
  $16 = HEAP32[$pSrc>>2]|0;
  $17 = HEAP32[$16>>2]|0;
  $18 = ($15|0)<($17|0);
  if (!($18)) {
   break;
  }
  $19 = HEAP32[$1>>2]|0;
  $20 = (($19) + 28|0);
  $21 = HEAP32[$20>>2]|0;
  $22 = HEAP32[$i>>2]|0;
  $23 = HEAP32[$pSrc>>2]|0;
  $24 = (($23) + 8|0);
  $25 = (($24) + (($22*72)|0)|0);
  $26 = (($25) + 40|0);
  $27 = HEAP32[$26>>2]|0;
  $28 = ($21|0)==($27|0);
  if ($28) {
   break;
  }
  $29 = HEAP32[$i>>2]|0;
  $30 = (($29) + 1)|0;
  HEAP32[$i>>2] = $30;
 }
 $31 = HEAP32[$i>>2]|0;
 $32 = HEAP32[$pSrc>>2]|0;
 $33 = HEAP32[$32>>2]|0;
 $34 = ($31|0)<($33|0);
 $35 = HEAP32[$p>>2]|0;
 if ($34) {
  $36 = (($35) + 4|0);
  $37 = HEAP32[$36>>2]|0;
  $38 = (($37) + 1)|0;
  HEAP32[$36>>2] = $38;
  STACKTOP = sp;return 0;
 } else {
  $39 = (($35) + 8|0);
  $40 = HEAP32[$39>>2]|0;
  $41 = (($40) + 1)|0;
  HEAP32[$39>>2] = $41;
  STACKTOP = sp;return 0;
 }
 return 0|0;
}
function _sqlite3MatchSpanName($zSpan,$zCol,$zTab,$zDb) {
 $zSpan = $zSpan|0;
 $zCol = $zCol|0;
 $zTab = $zTab|0;
 $zDb = $zDb|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $8 = 0, $9 = 0, $n = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 20|0;
 $1 = sp + 16|0;
 $2 = sp + 12|0;
 $3 = sp + 8|0;
 $4 = sp + 4|0;
 $n = sp;
 HEAP32[$1>>2] = $zSpan;
 HEAP32[$2>>2] = $zCol;
 HEAP32[$3>>2] = $zTab;
 HEAP32[$4>>2] = $zDb;
 HEAP32[$n>>2] = 0;
 while(1) {
  $5 = HEAP32[$n>>2]|0;
  $6 = HEAP32[$1>>2]|0;
  $7 = (($6) + ($5)|0);
  $8 = HEAP8[$7>>0]|0;
  $9 = $8 << 24 >> 24;
  $10 = ($9|0)!=(0);
  if (!($10)) {
   break;
  }
  $11 = HEAP32[$n>>2]|0;
  $12 = HEAP32[$1>>2]|0;
  $13 = (($12) + ($11)|0);
  $14 = HEAP8[$13>>0]|0;
  $15 = $14 << 24 >> 24;
  $16 = ($15|0)!=(46);
  if (!($16)) {
   break;
  }
  $17 = HEAP32[$n>>2]|0;
  $18 = (($17) + 1)|0;
  HEAP32[$n>>2] = $18;
 }
 $19 = HEAP32[$4>>2]|0;
 $20 = ($19|0)!=(0|0);
 do {
  if ($20) {
   $21 = HEAP32[$1>>2]|0;
   $22 = HEAP32[$4>>2]|0;
   $23 = HEAP32[$n>>2]|0;
   $24 = (_sqlite3_strnicmp($21,$22,$23)|0);
   $25 = ($24|0)!=(0);
   if (!($25)) {
    $26 = HEAP32[$n>>2]|0;
    $27 = HEAP32[$4>>2]|0;
    $28 = (($27) + ($26)|0);
    $29 = HEAP8[$28>>0]|0;
    $30 = $29 << 24 >> 24;
    $31 = ($30|0)!=(0);
    if (!($31)) {
     break;
    }
   }
   HEAP32[$0>>2] = 0;
   $73 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($73|0);
  }
 } while(0);
 $32 = HEAP32[$n>>2]|0;
 $33 = (($32) + 1)|0;
 $34 = HEAP32[$1>>2]|0;
 $35 = (($34) + ($33)|0);
 HEAP32[$1>>2] = $35;
 HEAP32[$n>>2] = 0;
 while(1) {
  $36 = HEAP32[$n>>2]|0;
  $37 = HEAP32[$1>>2]|0;
  $38 = (($37) + ($36)|0);
  $39 = HEAP8[$38>>0]|0;
  $40 = $39 << 24 >> 24;
  $41 = ($40|0)!=(0);
  if (!($41)) {
   break;
  }
  $42 = HEAP32[$n>>2]|0;
  $43 = HEAP32[$1>>2]|0;
  $44 = (($43) + ($42)|0);
  $45 = HEAP8[$44>>0]|0;
  $46 = $45 << 24 >> 24;
  $47 = ($46|0)!=(46);
  if (!($47)) {
   break;
  }
  $48 = HEAP32[$n>>2]|0;
  $49 = (($48) + 1)|0;
  HEAP32[$n>>2] = $49;
 }
 $50 = HEAP32[$3>>2]|0;
 $51 = ($50|0)!=(0|0);
 do {
  if ($51) {
   $52 = HEAP32[$1>>2]|0;
   $53 = HEAP32[$3>>2]|0;
   $54 = HEAP32[$n>>2]|0;
   $55 = (_sqlite3_strnicmp($52,$53,$54)|0);
   $56 = ($55|0)!=(0);
   if (!($56)) {
    $57 = HEAP32[$n>>2]|0;
    $58 = HEAP32[$3>>2]|0;
    $59 = (($58) + ($57)|0);
    $60 = HEAP8[$59>>0]|0;
    $61 = $60 << 24 >> 24;
    $62 = ($61|0)!=(0);
    if (!($62)) {
     break;
    }
   }
   HEAP32[$0>>2] = 0;
   $73 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($73|0);
  }
 } while(0);
 $63 = HEAP32[$n>>2]|0;
 $64 = (($63) + 1)|0;
 $65 = HEAP32[$1>>2]|0;
 $66 = (($65) + ($64)|0);
 HEAP32[$1>>2] = $66;
 $67 = HEAP32[$2>>2]|0;
 $68 = ($67|0)!=(0|0);
 if ($68) {
  $69 = HEAP32[$1>>2]|0;
  $70 = HEAP32[$2>>2]|0;
  $71 = (_sqlite3_stricmp($69,$70)|0);
  $72 = ($71|0)!=(0);
  if ($72) {
   HEAP32[$0>>2] = 0;
   $73 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($73|0);
  }
 }
 HEAP32[$0>>2] = 1;
 $73 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($73|0);
}
function _nameInUsingClause($pUsing,$zCol) {
 $pUsing = $pUsing|0;
 $zCol = $zCol|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0;
 var $9 = 0, $k = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $1 = sp + 8|0;
 $2 = sp + 4|0;
 $k = sp;
 HEAP32[$1>>2] = $pUsing;
 HEAP32[$2>>2] = $zCol;
 $3 = HEAP32[$1>>2]|0;
 $4 = ($3|0)!=(0|0);
 L1: do {
  if ($4) {
   HEAP32[$k>>2] = 0;
   while(1) {
    $5 = HEAP32[$k>>2]|0;
    $6 = HEAP32[$1>>2]|0;
    $7 = (($6) + 4|0);
    $8 = HEAP32[$7>>2]|0;
    $9 = ($5|0)<($8|0);
    if (!($9)) {
     break L1;
    }
    $10 = HEAP32[$k>>2]|0;
    $11 = HEAP32[$1>>2]|0;
    $12 = HEAP32[$11>>2]|0;
    $13 = (($12) + ($10<<3)|0);
    $14 = HEAP32[$13>>2]|0;
    $15 = HEAP32[$2>>2]|0;
    $16 = (_sqlite3_stricmp($14,$15)|0);
    $17 = ($16|0)==(0);
    if ($17) {
     break;
    }
    $18 = HEAP32[$k>>2]|0;
    $19 = (($18) + 1)|0;
    HEAP32[$k>>2] = $19;
   }
   HEAP32[$0>>2] = 1;
   $20 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($20|0);
  }
 } while(0);
 HEAP32[$0>>2] = 0;
 $20 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($20|0);
}
function _sqlite3IsRowid($z) {
 $z = $z|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $1 = sp;
 HEAP32[$1>>2] = $z;
 $2 = HEAP32[$1>>2]|0;
 $3 = (_sqlite3_stricmp($2,17368)|0);
 $4 = ($3|0)==(0);
 do {
  if ($4) {
   HEAP32[$0>>2] = 1;
  } else {
   $5 = HEAP32[$1>>2]|0;
   $6 = (_sqlite3_stricmp($5,17288)|0);
   $7 = ($6|0)==(0);
   if ($7) {
    HEAP32[$0>>2] = 1;
    break;
   }
   $8 = HEAP32[$1>>2]|0;
   $9 = (_sqlite3_stricmp($8,17376)|0);
   $10 = ($9|0)==(0);
   if ($10) {
    HEAP32[$0>>2] = 1;
    break;
   } else {
    HEAP32[$0>>2] = 0;
    break;
   }
  }
 } while(0);
 $11 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($11|0);
}
function _sqlite3AuthRead($pParse,$pExpr,$pSchema,$pTabList) {
 $pParse = $pParse|0;
 $pExpr = $pExpr|0;
 $pSchema = $pSchema|0;
 $pTabList = $pTabList|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $9 = 0;
 var $db = 0, $iCol = 0, $iDb = 0, $iSrc = 0, $pTab = 0, $zCol = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $0 = sp + 36|0;
 $1 = sp + 32|0;
 $2 = sp + 28|0;
 $3 = sp + 24|0;
 $db = sp + 20|0;
 $pTab = sp + 16|0;
 $zCol = sp + 12|0;
 $iSrc = sp + 8|0;
 $iDb = sp + 4|0;
 $iCol = sp;
 HEAP32[$0>>2] = $pParse;
 HEAP32[$1>>2] = $pExpr;
 HEAP32[$2>>2] = $pSchema;
 HEAP32[$3>>2] = $pTabList;
 $4 = HEAP32[$0>>2]|0;
 $5 = HEAP32[$4>>2]|0;
 HEAP32[$db>>2] = $5;
 HEAP32[$pTab>>2] = 0;
 $6 = HEAP32[$db>>2]|0;
 $7 = (($6) + 284|0);
 $8 = HEAP32[$7>>2]|0;
 $9 = ($8|0)==(0|0);
 if ($9) {
  STACKTOP = sp;return;
 }
 $10 = HEAP32[$0>>2]|0;
 $11 = HEAP32[$10>>2]|0;
 $12 = HEAP32[$2>>2]|0;
 $13 = (_sqlite3SchemaToIndex($11,$12)|0);
 HEAP32[$iDb>>2] = $13;
 $14 = HEAP32[$iDb>>2]|0;
 $15 = ($14|0)<(0);
 if ($15) {
  STACKTOP = sp;return;
 }
 $16 = HEAP32[$1>>2]|0;
 $17 = HEAP8[$16>>0]|0;
 $18 = $17&255;
 $19 = ($18|0)==(62);
 L7: do {
  if ($19) {
   $20 = HEAP32[$0>>2]|0;
   $21 = (($20) + 412|0);
   $22 = HEAP32[$21>>2]|0;
   HEAP32[$pTab>>2] = $22;
  } else {
   HEAP32[$iSrc>>2] = 0;
   while(1) {
    $23 = HEAP32[$iSrc>>2]|0;
    $24 = HEAP32[$3>>2]|0;
    $25 = HEAP32[$24>>2]|0;
    $26 = ($23|0)<($25|0);
    if (!($26)) {
     break L7;
    }
    $27 = HEAP32[$1>>2]|0;
    $28 = (($27) + 28|0);
    $29 = HEAP32[$28>>2]|0;
    $30 = HEAP32[$iSrc>>2]|0;
    $31 = HEAP32[$3>>2]|0;
    $32 = (($31) + 8|0);
    $33 = (($32) + (($30*72)|0)|0);
    $34 = (($33) + 40|0);
    $35 = HEAP32[$34>>2]|0;
    $36 = ($29|0)==($35|0);
    $37 = HEAP32[$iSrc>>2]|0;
    if ($36) {
     break;
    }
    $43 = (($37) + 1)|0;
    HEAP32[$iSrc>>2] = $43;
   }
   $38 = HEAP32[$3>>2]|0;
   $39 = (($38) + 8|0);
   $40 = (($39) + (($37*72)|0)|0);
   $41 = (($40) + 16|0);
   $42 = HEAP32[$41>>2]|0;
   HEAP32[$pTab>>2] = $42;
  }
 } while(0);
 $44 = HEAP32[$1>>2]|0;
 $45 = (($44) + 32|0);
 $46 = HEAP16[$45>>1]|0;
 $47 = $46 << 16 >> 16;
 HEAP32[$iCol>>2] = $47;
 $48 = HEAP32[$pTab>>2]|0;
 $49 = ($48|0)==(0|0);
 if ($49) {
  STACKTOP = sp;return;
 }
 $50 = HEAP32[$iCol>>2]|0;
 $51 = ($50|0)>=(0);
 do {
  if ($51) {
   $52 = HEAP32[$iCol>>2]|0;
   $53 = HEAP32[$pTab>>2]|0;
   $54 = (($53) + 4|0);
   $55 = HEAP32[$54>>2]|0;
   $56 = (($55) + (($52*24)|0)|0);
   $57 = HEAP32[$56>>2]|0;
   HEAP32[$zCol>>2] = $57;
  } else {
   $58 = HEAP32[$pTab>>2]|0;
   $59 = (($58) + 36|0);
   $60 = HEAP16[$59>>1]|0;
   $61 = $60 << 16 >> 16;
   $62 = ($61|0)>=(0);
   if ($62) {
    $63 = HEAP32[$pTab>>2]|0;
    $64 = (($63) + 36|0);
    $65 = HEAP16[$64>>1]|0;
    $66 = $65 << 16 >> 16;
    $67 = HEAP32[$pTab>>2]|0;
    $68 = (($67) + 4|0);
    $69 = HEAP32[$68>>2]|0;
    $70 = (($69) + (($66*24)|0)|0);
    $71 = HEAP32[$70>>2]|0;
    HEAP32[$zCol>>2] = $71;
    break;
   } else {
    HEAP32[$zCol>>2] = 17288;
    break;
   }
  }
 } while(0);
 $72 = HEAP32[$0>>2]|0;
 $73 = HEAP32[$pTab>>2]|0;
 $74 = HEAP32[$73>>2]|0;
 $75 = HEAP32[$zCol>>2]|0;
 $76 = HEAP32[$iDb>>2]|0;
 $77 = (_sqlite3AuthReadCol($72,$74,$75,$76)|0);
 $78 = (2)==($77|0);
 if (!($78)) {
  STACKTOP = sp;return;
 }
 $79 = HEAP32[$1>>2]|0;
 HEAP8[$79>>0] = 101;
 STACKTOP = sp;return;
}
function _sqlite3AuthReadCol($pParse,$zTab,$zCol,$iDb) {
 $pParse = $pParse|0;
 $zTab = $zTab|0;
 $zCol = $zCol|0;
 $iDb = $iDb|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $db = 0, $or$cond = 0, $or$cond3 = 0, $rc = 0, $vararg_buffer6 = 0, $vararg_ptr4 = 0, $vararg_ptr5 = 0, $vararg_ptr9 = 0, $zDb = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $vararg_buffer6 = sp;
 $0 = sp + 36|0;
 $1 = sp + 32|0;
 $2 = sp + 28|0;
 $3 = sp + 24|0;
 $db = sp + 20|0;
 $zDb = sp + 16|0;
 $rc = sp + 12|0;
 HEAP32[$0>>2] = $pParse;
 HEAP32[$1>>2] = $zTab;
 HEAP32[$2>>2] = $zCol;
 HEAP32[$3>>2] = $iDb;
 $4 = HEAP32[$0>>2]|0;
 $5 = HEAP32[$4>>2]|0;
 HEAP32[$db>>2] = $5;
 $6 = HEAP32[$3>>2]|0;
 $7 = HEAP32[$db>>2]|0;
 $8 = (($7) + 16|0);
 $9 = HEAP32[$8>>2]|0;
 $10 = (($9) + ($6<<4)|0);
 $11 = HEAP32[$10>>2]|0;
 HEAP32[$zDb>>2] = $11;
 $12 = HEAP32[$db>>2]|0;
 $13 = (($12) + 284|0);
 $14 = HEAP32[$13>>2]|0;
 $15 = HEAP32[$db>>2]|0;
 $16 = (($15) + 288|0);
 $17 = HEAP32[$16>>2]|0;
 $18 = HEAP32[$1>>2]|0;
 $19 = HEAP32[$2>>2]|0;
 $20 = HEAP32[$zDb>>2]|0;
 $21 = HEAP32[$0>>2]|0;
 $22 = (($21) + 492|0);
 $23 = HEAP32[$22>>2]|0;
 $24 = (FUNCTION_TABLE_iiiiiii[$14 & 1]($17,20,$18,$19,$20,$23)|0);
 HEAP32[$rc>>2] = $24;
 $25 = HEAP32[$rc>>2]|0;
 $26 = ($25|0)==(1);
 if (!($26)) {
  $41 = HEAP32[$rc>>2]|0;
  $42 = ($41|0)!=(2);
  $43 = HEAP32[$rc>>2]|0;
  $44 = ($43|0)!=(0);
  $or$cond3 = $42 & $44;
  if (!($or$cond3)) {
   $46 = HEAP32[$rc>>2]|0;
   STACKTOP = sp;return ($46|0);
  }
  $45 = HEAP32[$0>>2]|0;
  _sqliteAuthBadReturnCode($45);
  $46 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($46|0);
 }
 $27 = HEAP32[$db>>2]|0;
 $28 = (($27) + 20|0);
 $29 = HEAP32[$28>>2]|0;
 $30 = ($29|0)>(2);
 $31 = HEAP32[$3>>2]|0;
 $32 = ($31|0)!=(0);
 $or$cond = $30 | $32;
 $33 = HEAP32[$0>>2]|0;
 if ($or$cond) {
  $34 = HEAP32[$zDb>>2]|0;
  $35 = HEAP32[$1>>2]|0;
  $36 = HEAP32[$2>>2]|0;
  HEAP32[$vararg_buffer6>>2] = $34;
  $vararg_ptr4 = (($vararg_buffer6) + 4|0);
  HEAP32[$vararg_ptr4>>2] = $35;
  $vararg_ptr5 = (($vararg_buffer6) + 8|0);
  HEAP32[$vararg_ptr5>>2] = $36;
  _sqlite3ErrorMsg($33,17296,$vararg_buffer6);
 } else {
  $37 = HEAP32[$1>>2]|0;
  $38 = HEAP32[$2>>2]|0;
  HEAP32[$vararg_buffer6>>2] = $37;
  $vararg_ptr9 = (($vararg_buffer6) + 4|0);
  HEAP32[$vararg_ptr9>>2] = $38;
  _sqlite3ErrorMsg($33,17336,$vararg_buffer6);
 }
 $39 = HEAP32[$0>>2]|0;
 $40 = (($39) + 12|0);
 HEAP32[$40>>2] = 23;
 $46 = HEAP32[$rc>>2]|0;
 STACKTOP = sp;return ($46|0);
}
function _convertCompoundSelectToSubquery($pWalker,$p) {
 $pWalker = $pWalker|0;
 $p = $p|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0;
 var $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0;
 var $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0;
 var $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0;
 var $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0;
 var $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $a = 0, $db = 0, $dummy = 0, $i = 0, $pNew = 0, $pNewSrc = 0, $pParse = 0, $pX = 0, dest = 0, label = 0, sp = 0, src = 0, stop = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $0 = sp + 44|0;
 $1 = sp + 40|0;
 $2 = sp + 36|0;
 $i = sp + 32|0;
 $pNew = sp + 28|0;
 $pX = sp + 24|0;
 $db = sp + 20|0;
 $a = sp + 16|0;
 $pNewSrc = sp + 12|0;
 $pParse = sp + 8|0;
 $dummy = sp;
 HEAP32[$1>>2] = $pWalker;
 HEAP32[$2>>2] = $p;
 $3 = HEAP32[$2>>2]|0;
 $4 = (($3) + 52|0);
 $5 = HEAP32[$4>>2]|0;
 $6 = ($5|0)==(0|0);
 if ($6) {
  HEAP32[$0>>2] = 0;
  $106 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($106|0);
 }
 $7 = HEAP32[$2>>2]|0;
 $8 = (($7) + 48|0);
 $9 = HEAP32[$8>>2]|0;
 $10 = ($9|0)==(0|0);
 if ($10) {
  HEAP32[$0>>2] = 0;
  $106 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($106|0);
 }
 $11 = HEAP32[$2>>2]|0;
 HEAP32[$pX>>2] = $11;
 while(1) {
  $12 = HEAP32[$pX>>2]|0;
  $13 = ($12|0)!=(0|0);
  if (!($13)) {
   break;
  }
  $14 = HEAP32[$pX>>2]|0;
  $15 = (($14) + 4|0);
  $16 = HEAP8[$15>>0]|0;
  $17 = $16&255;
  $18 = ($17|0)==(116);
  if (!($18)) {
   $19 = HEAP32[$pX>>2]|0;
   $20 = (($19) + 4|0);
   $21 = HEAP8[$20>>0]|0;
   $22 = $21&255;
   $23 = ($22|0)==(119);
   if (!($23)) {
    break;
   }
  }
  $24 = HEAP32[$pX>>2]|0;
  $25 = (($24) + 52|0);
  $26 = HEAP32[$25>>2]|0;
  HEAP32[$pX>>2] = $26;
 }
 $27 = HEAP32[$pX>>2]|0;
 $28 = ($27|0)==(0|0);
 if ($28) {
  HEAP32[$0>>2] = 0;
  $106 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($106|0);
 }
 $29 = HEAP32[$2>>2]|0;
 $30 = (($29) + 48|0);
 $31 = HEAP32[$30>>2]|0;
 $32 = (($31) + 4|0);
 $33 = HEAP32[$32>>2]|0;
 HEAP32[$a>>2] = $33;
 $34 = HEAP32[$2>>2]|0;
 $35 = (($34) + 48|0);
 $36 = HEAP32[$35>>2]|0;
 $37 = HEAP32[$36>>2]|0;
 $38 = (($37) - 1)|0;
 HEAP32[$i>>2] = $38;
 while(1) {
  $39 = HEAP32[$i>>2]|0;
  $40 = ($39|0)>=(0);
  if (!($40)) {
   break;
  }
  $41 = HEAP32[$i>>2]|0;
  $42 = HEAP32[$a>>2]|0;
  $43 = (($42) + (($41*20)|0)|0);
  $44 = HEAP32[$43>>2]|0;
  $45 = (($44) + 4|0);
  $46 = HEAP32[$45>>2]|0;
  $47 = $46 & 256;
  $48 = ($47|0)!=(0);
  if ($48) {
   break;
  }
  $49 = HEAP32[$i>>2]|0;
  $50 = (($49) + -1)|0;
  HEAP32[$i>>2] = $50;
 }
 $51 = HEAP32[$i>>2]|0;
 $52 = ($51|0)<(0);
 if ($52) {
  HEAP32[$0>>2] = 0;
  $106 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($106|0);
 }
 $53 = HEAP32[$1>>2]|0;
 $54 = (($53) + 12|0);
 $55 = HEAP32[$54>>2]|0;
 HEAP32[$pParse>>2] = $55;
 $56 = HEAP32[$pParse>>2]|0;
 $57 = HEAP32[$56>>2]|0;
 HEAP32[$db>>2] = $57;
 $58 = HEAP32[$db>>2]|0;
 $59 = (_sqlite3DbMallocZero($58,72,0)|0);
 HEAP32[$pNew>>2] = $59;
 $60 = HEAP32[$pNew>>2]|0;
 $61 = ($60|0)==(0|0);
 if ($61) {
  HEAP32[$0>>2] = 2;
  $106 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($106|0);
 }
 ;HEAP32[$dummy+0>>2]=0|0;HEAP32[$dummy+4>>2]=0|0;
 $62 = HEAP32[$pParse>>2]|0;
 $63 = HEAP32[$pNew>>2]|0;
 $64 = (_sqlite3SrcListAppendFromTerm($62,0,0,0,$dummy,$63,0,0)|0);
 HEAP32[$pNewSrc>>2] = $64;
 $65 = HEAP32[$pNewSrc>>2]|0;
 $66 = ($65|0)==(0|0);
 if ($66) {
  HEAP32[$0>>2] = 2;
  $106 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($106|0);
 } else {
  $67 = HEAP32[$pNew>>2]|0;
  $68 = HEAP32[$2>>2]|0;
  dest=$67+0|0; src=$68+0|0; stop=dest+72|0; do { HEAP32[dest>>2]=HEAP32[src>>2]|0; dest=dest+4|0; src=src+4|0; } while ((dest|0) < (stop|0));
  $69 = HEAP32[$pNewSrc>>2]|0;
  $70 = HEAP32[$2>>2]|0;
  $71 = (($70) + 32|0);
  HEAP32[$71>>2] = $69;
  $72 = HEAP32[$pParse>>2]|0;
  $73 = HEAP32[$db>>2]|0;
  $74 = (_sqlite3Expr($73,116,0)|0);
  $75 = (_sqlite3ExprListAppend($72,0,$74)|0);
  $76 = HEAP32[$2>>2]|0;
  HEAP32[$76>>2] = $75;
  $77 = HEAP32[$2>>2]|0;
  $78 = (($77) + 4|0);
  HEAP8[$78>>0] = 119;
  $79 = HEAP32[$2>>2]|0;
  $80 = (($79) + 36|0);
  HEAP32[$80>>2] = 0;
  $81 = HEAP32[$pNew>>2]|0;
  $82 = (($81) + 40|0);
  HEAP32[$82>>2] = 0;
  $83 = HEAP32[$pNew>>2]|0;
  $84 = (($83) + 44|0);
  HEAP32[$84>>2] = 0;
  $85 = HEAP32[$pNew>>2]|0;
  $86 = (($85) + 48|0);
  HEAP32[$86>>2] = 0;
  $87 = HEAP32[$2>>2]|0;
  $88 = (($87) + 52|0);
  HEAP32[$88>>2] = 0;
  $89 = HEAP32[$2>>2]|0;
  $90 = (($89) + 56|0);
  HEAP32[$90>>2] = 0;
  $91 = HEAP32[$2>>2]|0;
  $92 = (($91) + 6|0);
  $93 = HEAP16[$92>>1]|0;
  $94 = $93&65535;
  $95 = $94 & -65;
  $96 = $95&65535;
  HEAP16[$92>>1] = $96;
  $97 = HEAP32[$pNew>>2]|0;
  $98 = HEAP32[$pNew>>2]|0;
  $99 = (($98) + 52|0);
  $100 = HEAP32[$99>>2]|0;
  $101 = (($100) + 56|0);
  HEAP32[$101>>2] = $97;
  $102 = HEAP32[$pNew>>2]|0;
  $103 = (($102) + 60|0);
  HEAP32[$103>>2] = 0;
  $104 = HEAP32[$pNew>>2]|0;
  $105 = (($104) + 64|0);
  HEAP32[$105>>2] = 0;
  HEAP32[$0>>2] = 0;
  $106 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($106|0);
 }
 return 0|0;
}
function _selectExpander($pWalker,$p) {
 $pWalker = $pWalker|0;
 $p = $p|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0;
 var $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0, $188 = 0;
 var $189 = 0, $19 = 0, $190 = 0, $191 = 0, $192 = 0, $193 = 0, $194 = 0, $195 = 0, $196 = 0, $197 = 0, $198 = 0, $199 = 0, $2 = 0, $20 = 0, $200 = 0, $201 = 0, $202 = 0, $203 = 0, $204 = 0, $205 = 0;
 var $206 = 0, $207 = 0, $208 = 0, $209 = 0, $21 = 0, $210 = 0, $211 = 0, $212 = 0, $213 = 0, $214 = 0, $215 = 0, $216 = 0, $217 = 0, $218 = 0, $219 = 0, $22 = 0, $220 = 0, $221 = 0, $222 = 0, $223 = 0;
 var $224 = 0, $225 = 0, $226 = 0, $227 = 0, $228 = 0, $229 = 0, $23 = 0, $230 = 0, $231 = 0, $232 = 0, $233 = 0, $234 = 0, $235 = 0, $236 = 0, $237 = 0, $238 = 0, $239 = 0, $24 = 0, $240 = 0, $241 = 0;
 var $242 = 0, $243 = 0, $244 = 0, $245 = 0, $246 = 0, $247 = 0, $248 = 0, $249 = 0, $25 = 0, $250 = 0, $251 = 0, $252 = 0, $253 = 0, $254 = 0, $255 = 0, $256 = 0, $257 = 0, $258 = 0, $259 = 0, $26 = 0;
 var $260 = 0, $261 = 0, $262 = 0, $263 = 0, $264 = 0, $265 = 0, $266 = 0, $267 = 0, $268 = 0, $269 = 0, $27 = 0, $270 = 0, $271 = 0, $272 = 0, $273 = 0, $274 = 0, $275 = 0, $276 = 0, $277 = 0, $278 = 0;
 var $279 = 0, $28 = 0, $280 = 0, $281 = 0, $282 = 0, $283 = 0, $284 = 0, $285 = 0, $286 = 0, $287 = 0, $288 = 0, $289 = 0, $29 = 0, $290 = 0, $291 = 0, $292 = 0, $293 = 0, $294 = 0, $295 = 0, $296 = 0;
 var $297 = 0, $298 = 0, $299 = 0, $3 = 0, $30 = 0, $300 = 0, $301 = 0, $302 = 0, $303 = 0, $304 = 0, $305 = 0, $306 = 0, $307 = 0, $308 = 0, $309 = 0, $31 = 0, $310 = 0, $311 = 0, $312 = 0, $313 = 0;
 var $314 = 0, $315 = 0, $316 = 0, $317 = 0, $318 = 0, $319 = 0, $32 = 0, $320 = 0, $321 = 0, $322 = 0, $323 = 0, $324 = 0, $325 = 0, $326 = 0, $327 = 0, $328 = 0, $329 = 0, $33 = 0, $330 = 0, $331 = 0;
 var $332 = 0, $333 = 0, $334 = 0, $335 = 0, $336 = 0, $337 = 0, $338 = 0, $339 = 0, $34 = 0, $340 = 0, $341 = 0, $342 = 0, $343 = 0, $344 = 0, $345 = 0, $346 = 0, $347 = 0, $348 = 0, $349 = 0, $35 = 0;
 var $350 = 0, $351 = 0, $352 = 0, $353 = 0, $354 = 0, $355 = 0, $356 = 0, $357 = 0, $358 = 0, $359 = 0, $36 = 0, $360 = 0, $361 = 0, $362 = 0, $363 = 0, $364 = 0, $365 = 0, $366 = 0, $367 = 0, $368 = 0;
 var $369 = 0, $37 = 0, $370 = 0, $371 = 0, $372 = 0, $373 = 0, $374 = 0, $375 = 0, $376 = 0, $377 = 0, $378 = 0, $379 = 0, $38 = 0, $380 = 0, $381 = 0, $382 = 0, $383 = 0, $384 = 0, $385 = 0, $386 = 0;
 var $387 = 0, $388 = 0, $389 = 0, $39 = 0, $390 = 0, $391 = 0, $392 = 0, $393 = 0, $394 = 0, $395 = 0, $396 = 0, $397 = 0, $398 = 0, $399 = 0, $4 = 0, $40 = 0, $400 = 0, $401 = 0, $402 = 0, $403 = 0;
 var $404 = 0, $405 = 0, $406 = 0, $407 = 0, $408 = 0, $409 = 0, $41 = 0, $410 = 0, $411 = 0, $412 = 0, $413 = 0, $414 = 0, $415 = 0, $416 = 0, $417 = 0, $418 = 0, $419 = 0, $42 = 0, $420 = 0, $421 = 0;
 var $422 = 0, $423 = 0, $424 = 0, $425 = 0, $426 = 0, $427 = 0, $428 = 0, $429 = 0, $43 = 0, $430 = 0, $431 = 0, $432 = 0, $433 = 0, $434 = 0, $435 = 0, $436 = 0, $437 = 0, $438 = 0, $439 = 0, $44 = 0;
 var $440 = 0, $441 = 0, $442 = 0, $443 = 0, $444 = 0, $445 = 0, $446 = 0, $447 = 0, $448 = 0, $449 = 0, $45 = 0, $450 = 0, $451 = 0, $452 = 0, $453 = 0, $454 = 0, $455 = 0, $456 = 0, $457 = 0, $458 = 0;
 var $459 = 0, $46 = 0, $460 = 0, $461 = 0, $462 = 0, $463 = 0, $464 = 0, $465 = 0, $466 = 0, $467 = 0, $468 = 0, $469 = 0, $47 = 0, $470 = 0, $471 = 0, $472 = 0, $473 = 0, $474 = 0, $475 = 0, $476 = 0;
 var $477 = 0, $478 = 0, $479 = 0, $48 = 0, $480 = 0, $481 = 0, $482 = 0, $483 = 0, $484 = 0, $485 = 0, $486 = 0, $487 = 0, $488 = 0, $489 = 0, $49 = 0, $490 = 0, $491 = 0, $492 = 0, $493 = 0, $494 = 0;
 var $495 = 0, $496 = 0, $497 = 0, $498 = 0, $499 = 0, $5 = 0, $50 = 0, $500 = 0, $501 = 0, $502 = 0, $503 = 0, $504 = 0, $505 = 0, $506 = 0, $507 = 0, $508 = 0, $509 = 0, $51 = 0, $510 = 0, $511 = 0;
 var $512 = 0, $513 = 0, $514 = 0, $515 = 0, $516 = 0, $517 = 0, $518 = 0, $519 = 0, $52 = 0, $520 = 0, $521 = 0, $522 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0;
 var $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0;
 var $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0;
 var $97 = 0, $98 = 0, $99 = 0, $a = 0, $db = 0, $flags = 0, $i = 0, $iDb = 0, $j = 0, $k = 0, $longNames = 0, $or$cond = 0, $or$cond3 = 0, $pE = 0, $pEList = 0, $pExpr = 0, $pFrom = 0, $pLeft = 0, $pNew = 0, $pParse = 0;
 var $pRight = 0, $pSel = 0, $pSub = 0, $pTab = 0, $pTab1 = 0, $pTabList = 0, $pX = 0, $sColname = 0, $selFlags = 0, $tableSeen = 0, $vararg_buffer21 = 0, $vararg_ptr10 = 0, $vararg_ptr14 = 0, $vararg_ptr15 = 0, $zColname = 0, $zName = 0, $zSchemaName = 0, $zTName = 0, $zTabName = 0, $zToFree = 0;
 var label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 160|0;
 $vararg_buffer21 = sp;
 $0 = sp + 148|0;
 $1 = sp + 144|0;
 $2 = sp + 140|0;
 $pParse = sp + 136|0;
 $i = sp + 132|0;
 $j = sp + 128|0;
 $k = sp + 124|0;
 $pTabList = sp + 120|0;
 $pEList = sp + 116|0;
 $pFrom = sp + 112|0;
 $db = sp + 108|0;
 $pE = sp + 104|0;
 $pRight = sp + 100|0;
 $pExpr = sp + 96|0;
 $selFlags = sp + 152|0;
 $pTab = sp + 92|0;
 $pSel = sp + 88|0;
 $a = sp + 84|0;
 $pNew = sp + 80|0;
 $flags = sp + 76|0;
 $longNames = sp + 72|0;
 $tableSeen = sp + 68|0;
 $zTName = sp + 64|0;
 $pTab1 = sp + 60|0;
 $pSub = sp + 56|0;
 $zTabName = sp + 52|0;
 $zSchemaName = sp + 48|0;
 $iDb = sp + 44|0;
 $zName = sp + 40|0;
 $zColname = sp + 36|0;
 $zToFree = sp + 32|0;
 $sColname = sp + 24|0;
 $pLeft = sp + 16|0;
 $pX = sp + 12|0;
 HEAP32[$1>>2] = $pWalker;
 HEAP32[$2>>2] = $p;
 $3 = HEAP32[$1>>2]|0;
 $4 = (($3) + 12|0);
 $5 = HEAP32[$4>>2]|0;
 HEAP32[$pParse>>2] = $5;
 $6 = HEAP32[$pParse>>2]|0;
 $7 = HEAP32[$6>>2]|0;
 HEAP32[$db>>2] = $7;
 $8 = HEAP32[$2>>2]|0;
 $9 = (($8) + 6|0);
 $10 = HEAP16[$9>>1]|0;
 HEAP16[$selFlags>>1] = $10;
 $11 = HEAP32[$2>>2]|0;
 $12 = (($11) + 6|0);
 $13 = HEAP16[$12>>1]|0;
 $14 = $13&65535;
 $15 = $14 | 16;
 $16 = $15&65535;
 HEAP16[$12>>1] = $16;
 $17 = HEAP32[$db>>2]|0;
 $18 = (($17) + 64|0);
 $19 = HEAP8[$18>>0]|0;
 $20 = ($19<<24>>24)!=(0);
 if ($20) {
  HEAP32[$0>>2] = 2;
  $522 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($522|0);
 }
 $21 = HEAP32[$2>>2]|0;
 $22 = (($21) + 32|0);
 $23 = HEAP32[$22>>2]|0;
 $24 = ($23|0)==(0|0);
 if (!($24)) {
  $25 = HEAP16[$selFlags>>1]|0;
  $26 = $25&65535;
  $27 = $26 & 16;
  $28 = ($27|0)!=(0);
  if (!($28)) {
   $29 = HEAP32[$2>>2]|0;
   $30 = (($29) + 32|0);
   $31 = HEAP32[$30>>2]|0;
   HEAP32[$pTabList>>2] = $31;
   $32 = HEAP32[$2>>2]|0;
   $33 = HEAP32[$32>>2]|0;
   HEAP32[$pEList>>2] = $33;
   $34 = HEAP32[$pParse>>2]|0;
   $35 = HEAP32[$2>>2]|0;
   $36 = (_findRightmost($35)|0);
   $37 = (($36) + 68|0);
   $38 = HEAP32[$37>>2]|0;
   _sqlite3WithPush($34,$38,0);
   $39 = HEAP32[$pParse>>2]|0;
   $40 = HEAP32[$pTabList>>2]|0;
   _sqlite3SrcListAssignCursors($39,$40);
   HEAP32[$i>>2] = 0;
   $41 = HEAP32[$pTabList>>2]|0;
   $42 = (($41) + 8|0);
   HEAP32[$pFrom>>2] = $42;
   L8: while(1) {
    $43 = HEAP32[$i>>2]|0;
    $44 = HEAP32[$pTabList>>2]|0;
    $45 = HEAP32[$44>>2]|0;
    $46 = ($43|0)<($45|0);
    if (!($46)) {
     label = 33;
     break;
    }
    $47 = HEAP32[$pFrom>>2]|0;
    $48 = (($47) + 37|0);
    $49 = HEAP8[$48>>0]|0;
    $50 = ($49&255) >>> 3;
    $51 = $50 & 1;
    $52 = $51&255;
    $53 = ($52|0)!=(0);
    if (!($53)) {
     $54 = HEAP32[$pFrom>>2]|0;
     $55 = (($54) + 16|0);
     $56 = HEAP32[$55>>2]|0;
     $57 = ($56|0)!=(0|0);
     $58 = HEAP32[$1>>2]|0;
     if ($57) {
      label = 10;
      break;
     }
     $60 = HEAP32[$pFrom>>2]|0;
     $61 = (_withExpand($58,$60)|0);
     $62 = ($61|0)!=(0);
     if ($62) {
      label = 12;
      break;
     }
     $63 = HEAP32[$pFrom>>2]|0;
     $64 = (($63) + 16|0);
     $65 = HEAP32[$64>>2]|0;
     $66 = ($65|0)!=(0|0);
     do {
      if (!($66)) {
       $67 = HEAP32[$pFrom>>2]|0;
       $68 = (($67) + 8|0);
       $69 = HEAP32[$68>>2]|0;
       $70 = ($69|0)==(0|0);
       if ($70) {
        $71 = HEAP32[$pFrom>>2]|0;
        $72 = (($71) + 20|0);
        $73 = HEAP32[$72>>2]|0;
        HEAP32[$pSel>>2] = $73;
        $74 = HEAP32[$1>>2]|0;
        $75 = HEAP32[$pSel>>2]|0;
        (_sqlite3WalkSelect($74,$75)|0);
        $76 = HEAP32[$db>>2]|0;
        $77 = (_sqlite3DbMallocZero($76,76,0)|0);
        HEAP32[$pTab>>2] = $77;
        $78 = HEAP32[$pFrom>>2]|0;
        $79 = (($78) + 16|0);
        HEAP32[$79>>2] = $77;
        $80 = HEAP32[$pTab>>2]|0;
        $81 = ($80|0)==(0|0);
        if ($81) {
         label = 16;
         break L8;
        }
        $82 = HEAP32[$pTab>>2]|0;
        $83 = (($82) + 40|0);
        HEAP16[$83>>1] = 1;
        $84 = HEAP32[$db>>2]|0;
        $85 = HEAP32[$pTab>>2]|0;
        HEAP32[$vararg_buffer21>>2] = $85;
        $86 = (_sqlite3MPrintf($84,17384,$vararg_buffer21)|0);
        $87 = HEAP32[$pTab>>2]|0;
        HEAP32[$87>>2] = $86;
        while(1) {
         $88 = HEAP32[$pSel>>2]|0;
         $89 = (($88) + 52|0);
         $90 = HEAP32[$89>>2]|0;
         $91 = ($90|0)!=(0|0);
         if (!($91)) {
          break;
         }
         $92 = HEAP32[$pSel>>2]|0;
         $93 = (($92) + 52|0);
         $94 = HEAP32[$93>>2]|0;
         HEAP32[$pSel>>2] = $94;
        }
        $95 = HEAP32[$pParse>>2]|0;
        $96 = HEAP32[$pSel>>2]|0;
        $97 = HEAP32[$96>>2]|0;
        $98 = HEAP32[$pTab>>2]|0;
        $99 = (($98) + 38|0);
        $100 = HEAP32[$pTab>>2]|0;
        $101 = (($100) + 4|0);
        (_selectColumnsFromExprList($95,$97,$99,$101)|0);
        $102 = HEAP32[$pTab>>2]|0;
        $103 = (($102) + 36|0);
        HEAP16[$103>>1] = -1;
        $104 = HEAP32[$pTab>>2]|0;
        $105 = (($104) + 28|0);
        HEAP16[$105>>1] = 200;
        $106 = HEAP32[$pTab>>2]|0;
        $107 = (($106) + 44|0);
        $108 = HEAP8[$107>>0]|0;
        $109 = $108&255;
        $110 = $109 | 2;
        $111 = $110&255;
        HEAP8[$107>>0] = $111;
        break;
       }
       $112 = HEAP32[$pParse>>2]|0;
       $113 = HEAP32[$pFrom>>2]|0;
       $114 = (_sqlite3LocateTableItem($112,0,$113)|0);
       HEAP32[$pTab>>2] = $114;
       $115 = HEAP32[$pFrom>>2]|0;
       $116 = (($115) + 16|0);
       HEAP32[$116>>2] = $114;
       $117 = HEAP32[$pTab>>2]|0;
       $118 = ($117|0)==(0|0);
       if ($118) {
        label = 22;
        break L8;
       }
       $119 = HEAP32[$pTab>>2]|0;
       $120 = (($119) + 40|0);
       $121 = HEAP16[$120>>1]|0;
       $122 = $121&65535;
       $123 = ($122|0)==(65535);
       if ($123) {
        label = 24;
        break L8;
       }
       $129 = HEAP32[$pTab>>2]|0;
       $130 = (($129) + 40|0);
       $131 = HEAP16[$130>>1]|0;
       $132 = (($131) + 1)<<16>>16;
       HEAP16[$130>>1] = $132;
       $133 = HEAP32[$pTab>>2]|0;
       $134 = (($133) + 12|0);
       $135 = HEAP32[$134>>2]|0;
       $136 = ($135|0)!=(0|0);
       if (!($136)) {
        $137 = HEAP32[$pTab>>2]|0;
        $138 = (($137) + 44|0);
        $139 = HEAP8[$138>>0]|0;
        $140 = $139&255;
        $141 = $140 & 16;
        $142 = ($141|0)!=(0);
        if (!($142)) {
         break;
        }
       }
       $143 = HEAP32[$pParse>>2]|0;
       $144 = HEAP32[$pTab>>2]|0;
       $145 = (_sqlite3ViewGetColumnNames($143,$144)|0);
       $146 = ($145|0)!=(0);
       if ($146) {
        label = 28;
        break L8;
       }
       $147 = HEAP32[$db>>2]|0;
       $148 = HEAP32[$pTab>>2]|0;
       $149 = (($148) + 12|0);
       $150 = HEAP32[$149>>2]|0;
       $151 = (_sqlite3SelectDup($147,$150,0)|0);
       $152 = HEAP32[$pFrom>>2]|0;
       $153 = (($152) + 20|0);
       HEAP32[$153>>2] = $151;
       $154 = HEAP32[$1>>2]|0;
       $155 = HEAP32[$pFrom>>2]|0;
       $156 = (($155) + 20|0);
       $157 = HEAP32[$156>>2]|0;
       (_sqlite3WalkSelect($154,$157)|0);
      }
     } while(0);
     $158 = HEAP32[$pParse>>2]|0;
     $159 = HEAP32[$pFrom>>2]|0;
     $160 = (_sqlite3IndexedByLookup($158,$159)|0);
     $161 = ($160|0)!=(0);
     if ($161) {
      label = 31;
      break;
     }
    }
    $162 = HEAP32[$i>>2]|0;
    $163 = (($162) + 1)|0;
    HEAP32[$i>>2] = $163;
    $164 = HEAP32[$pFrom>>2]|0;
    $165 = (($164) + 72|0);
    HEAP32[$pFrom>>2] = $165;
   }
   if ((label|0) == 10) {
    $59 = HEAP32[$2>>2]|0;
    _selectPopWith($58,$59);
    HEAP32[$0>>2] = 1;
    $522 = HEAP32[$0>>2]|0;
    STACKTOP = sp;return ($522|0);
   }
   else if ((label|0) == 12) {
    HEAP32[$0>>2] = 2;
    $522 = HEAP32[$0>>2]|0;
    STACKTOP = sp;return ($522|0);
   }
   else if ((label|0) == 16) {
    HEAP32[$0>>2] = 2;
    $522 = HEAP32[$0>>2]|0;
    STACKTOP = sp;return ($522|0);
   }
   else if ((label|0) == 22) {
    HEAP32[$0>>2] = 2;
    $522 = HEAP32[$0>>2]|0;
    STACKTOP = sp;return ($522|0);
   }
   else if ((label|0) == 24) {
    $124 = HEAP32[$pParse>>2]|0;
    $125 = HEAP32[$pTab>>2]|0;
    $126 = HEAP32[$125>>2]|0;
    HEAP32[$vararg_buffer21>>2] = $126;
    _sqlite3ErrorMsg($124,17400,$vararg_buffer21);
    $127 = HEAP32[$pFrom>>2]|0;
    $128 = (($127) + 16|0);
    HEAP32[$128>>2] = 0;
    HEAP32[$0>>2] = 2;
    $522 = HEAP32[$0>>2]|0;
    STACKTOP = sp;return ($522|0);
   }
   else if ((label|0) == 28) {
    HEAP32[$0>>2] = 2;
    $522 = HEAP32[$0>>2]|0;
    STACKTOP = sp;return ($522|0);
   }
   else if ((label|0) == 31) {
    HEAP32[$0>>2] = 2;
    $522 = HEAP32[$0>>2]|0;
    STACKTOP = sp;return ($522|0);
   }
   else if ((label|0) == 33) {
    $166 = HEAP32[$db>>2]|0;
    $167 = (($166) + 64|0);
    $168 = HEAP8[$167>>0]|0;
    $169 = $168&255;
    $170 = ($169|0)!=(0);
    if (!($170)) {
     $171 = HEAP32[$pParse>>2]|0;
     $172 = HEAP32[$2>>2]|0;
     $173 = (_sqliteProcessJoin($171,$172)|0);
     $174 = ($173|0)!=(0);
     if (!($174)) {
      HEAP32[$k>>2] = 0;
      while(1) {
       $175 = HEAP32[$k>>2]|0;
       $176 = HEAP32[$pEList>>2]|0;
       $177 = HEAP32[$176>>2]|0;
       $178 = ($175|0)<($177|0);
       if (!($178)) {
        break;
       }
       $179 = HEAP32[$k>>2]|0;
       $180 = HEAP32[$pEList>>2]|0;
       $181 = (($180) + 4|0);
       $182 = HEAP32[$181>>2]|0;
       $183 = (($182) + (($179*20)|0)|0);
       $184 = HEAP32[$183>>2]|0;
       HEAP32[$pE>>2] = $184;
       $185 = HEAP32[$pE>>2]|0;
       $186 = HEAP8[$185>>0]|0;
       $187 = $186&255;
       $188 = ($187|0)==(116);
       if ($188) {
        break;
       }
       $189 = HEAP32[$pE>>2]|0;
       $190 = HEAP8[$189>>0]|0;
       $191 = $190&255;
       $192 = ($191|0)==(122);
       if ($192) {
        $193 = HEAP32[$pE>>2]|0;
        $194 = (($193) + 16|0);
        $195 = HEAP32[$194>>2]|0;
        $196 = HEAP8[$195>>0]|0;
        $197 = $196&255;
        $198 = ($197|0)==(116);
        if ($198) {
         break;
        }
       }
       $199 = HEAP32[$k>>2]|0;
       $200 = (($199) + 1)|0;
       HEAP32[$k>>2] = $200;
      }
      $201 = HEAP32[$k>>2]|0;
      $202 = HEAP32[$pEList>>2]|0;
      $203 = HEAP32[$202>>2]|0;
      $204 = ($201|0)<($203|0);
      if ($204) {
       $205 = HEAP32[$pEList>>2]|0;
       $206 = (($205) + 4|0);
       $207 = HEAP32[$206>>2]|0;
       HEAP32[$a>>2] = $207;
       HEAP32[$pNew>>2] = 0;
       $208 = HEAP32[$pParse>>2]|0;
       $209 = HEAP32[$208>>2]|0;
       $210 = (($209) + 24|0);
       $211 = HEAP32[$210>>2]|0;
       HEAP32[$flags>>2] = $211;
       $212 = HEAP32[$flags>>2]|0;
       $213 = $212 & 32;
       $214 = ($213|0)!=(0);
       if ($214) {
        $215 = HEAP32[$flags>>2]|0;
        $216 = $215 & 64;
        $217 = ($216|0)==(0);
        $219 = $217;
       } else {
        $219 = 0;
       }
       $218 = $219&1;
       HEAP32[$longNames>>2] = $218;
       HEAP32[$k>>2] = 0;
       while(1) {
        $220 = HEAP32[$k>>2]|0;
        $221 = HEAP32[$pEList>>2]|0;
        $222 = HEAP32[$221>>2]|0;
        $223 = ($220|0)<($222|0);
        if (!($223)) {
         break;
        }
        $224 = HEAP32[$k>>2]|0;
        $225 = HEAP32[$a>>2]|0;
        $226 = (($225) + (($224*20)|0)|0);
        $227 = HEAP32[$226>>2]|0;
        HEAP32[$pE>>2] = $227;
        $228 = HEAP32[$pE>>2]|0;
        $229 = (($228) + 16|0);
        $230 = HEAP32[$229>>2]|0;
        HEAP32[$pRight>>2] = $230;
        $231 = HEAP32[$pE>>2]|0;
        $232 = HEAP8[$231>>0]|0;
        $233 = $232&255;
        $234 = ($233|0)!=(116);
        do {
         if ($234) {
          $235 = HEAP32[$pE>>2]|0;
          $236 = HEAP8[$235>>0]|0;
          $237 = $236&255;
          $238 = ($237|0)!=(122);
          if (!($238)) {
           $239 = HEAP32[$pRight>>2]|0;
           $240 = HEAP8[$239>>0]|0;
           $241 = $240&255;
           $242 = ($241|0)!=(116);
           if (!($242)) {
            label = 53;
            break;
           }
          }
          $243 = HEAP32[$pParse>>2]|0;
          $244 = HEAP32[$pNew>>2]|0;
          $245 = HEAP32[$k>>2]|0;
          $246 = HEAP32[$a>>2]|0;
          $247 = (($246) + (($245*20)|0)|0);
          $248 = HEAP32[$247>>2]|0;
          $249 = (_sqlite3ExprListAppend($243,$244,$248)|0);
          HEAP32[$pNew>>2] = $249;
          $250 = HEAP32[$pNew>>2]|0;
          $251 = ($250|0)!=(0|0);
          if ($251) {
           $252 = HEAP32[$k>>2]|0;
           $253 = HEAP32[$a>>2]|0;
           $254 = (($253) + (($252*20)|0)|0);
           $255 = (($254) + 4|0);
           $256 = HEAP32[$255>>2]|0;
           $257 = HEAP32[$pNew>>2]|0;
           $258 = HEAP32[$257>>2]|0;
           $259 = (($258) - 1)|0;
           $260 = HEAP32[$pNew>>2]|0;
           $261 = (($260) + 4|0);
           $262 = HEAP32[$261>>2]|0;
           $263 = (($262) + (($259*20)|0)|0);
           $264 = (($263) + 4|0);
           HEAP32[$264>>2] = $256;
           $265 = HEAP32[$k>>2]|0;
           $266 = HEAP32[$a>>2]|0;
           $267 = (($266) + (($265*20)|0)|0);
           $268 = (($267) + 8|0);
           $269 = HEAP32[$268>>2]|0;
           $270 = HEAP32[$pNew>>2]|0;
           $271 = HEAP32[$270>>2]|0;
           $272 = (($271) - 1)|0;
           $273 = HEAP32[$pNew>>2]|0;
           $274 = (($273) + 4|0);
           $275 = HEAP32[$274>>2]|0;
           $276 = (($275) + (($272*20)|0)|0);
           $277 = (($276) + 8|0);
           HEAP32[$277>>2] = $269;
           $278 = HEAP32[$k>>2]|0;
           $279 = HEAP32[$a>>2]|0;
           $280 = (($279) + (($278*20)|0)|0);
           $281 = (($280) + 4|0);
           HEAP32[$281>>2] = 0;
           $282 = HEAP32[$k>>2]|0;
           $283 = HEAP32[$a>>2]|0;
           $284 = (($283) + (($282*20)|0)|0);
           $285 = (($284) + 8|0);
           HEAP32[$285>>2] = 0;
          }
          $286 = HEAP32[$k>>2]|0;
          $287 = HEAP32[$a>>2]|0;
          $288 = (($287) + (($286*20)|0)|0);
          HEAP32[$288>>2] = 0;
         } else {
          label = 53;
         }
        } while(0);
        do {
         if ((label|0) == 53) {
          label = 0;
          HEAP32[$tableSeen>>2] = 0;
          HEAP32[$zTName>>2] = 0;
          $289 = HEAP32[$pE>>2]|0;
          $290 = HEAP8[$289>>0]|0;
          $291 = $290&255;
          $292 = ($291|0)==(122);
          if ($292) {
           $293 = HEAP32[$pE>>2]|0;
           $294 = (($293) + 12|0);
           $295 = HEAP32[$294>>2]|0;
           $296 = (($295) + 8|0);
           $297 = HEAP32[$296>>2]|0;
           HEAP32[$zTName>>2] = $297;
          }
          HEAP32[$i>>2] = 0;
          $298 = HEAP32[$pTabList>>2]|0;
          $299 = (($298) + 8|0);
          HEAP32[$pFrom>>2] = $299;
          while(1) {
           $300 = HEAP32[$i>>2]|0;
           $301 = HEAP32[$pTabList>>2]|0;
           $302 = HEAP32[$301>>2]|0;
           $303 = ($300|0)<($302|0);
           if (!($303)) {
            break;
           }
           $304 = HEAP32[$pFrom>>2]|0;
           $305 = (($304) + 16|0);
           $306 = HEAP32[$305>>2]|0;
           HEAP32[$pTab1>>2] = $306;
           $307 = HEAP32[$pFrom>>2]|0;
           $308 = (($307) + 20|0);
           $309 = HEAP32[$308>>2]|0;
           HEAP32[$pSub>>2] = $309;
           $310 = HEAP32[$pFrom>>2]|0;
           $311 = (($310) + 12|0);
           $312 = HEAP32[$311>>2]|0;
           HEAP32[$zTabName>>2] = $312;
           HEAP32[$zSchemaName>>2] = 0;
           $313 = HEAP32[$zTabName>>2]|0;
           $314 = ($313|0)==(0|0);
           if ($314) {
            $315 = HEAP32[$pTab1>>2]|0;
            $316 = HEAP32[$315>>2]|0;
            HEAP32[$zTabName>>2] = $316;
           }
           $317 = HEAP32[$db>>2]|0;
           $318 = (($317) + 64|0);
           $319 = HEAP8[$318>>0]|0;
           $320 = ($319<<24>>24)!=(0);
           if ($320) {
            break;
           }
           $321 = HEAP32[$pSub>>2]|0;
           $322 = ($321|0)==(0|0);
           if ($322) {
            label = 62;
           } else {
            $323 = HEAP32[$pSub>>2]|0;
            $324 = (($323) + 6|0);
            $325 = HEAP16[$324>>1]|0;
            $326 = $325&65535;
            $327 = $326 & 512;
            $328 = ($327|0)==(0);
            if ($328) {
             label = 62;
            } else {
             label = 67;
            }
           }
           do {
            if ((label|0) == 62) {
             label = 0;
             HEAP32[$pSub>>2] = 0;
             $329 = HEAP32[$zTName>>2]|0;
             $330 = ($329|0)!=(0|0);
             if ($330) {
              $331 = HEAP32[$zTName>>2]|0;
              $332 = HEAP32[$zTabName>>2]|0;
              $333 = (_sqlite3_stricmp($331,$332)|0);
              $334 = ($333|0)!=(0);
              if ($334) {
               break;
              }
             }
             $335 = HEAP32[$db>>2]|0;
             $336 = HEAP32[$pTab1>>2]|0;
             $337 = (($336) + 68|0);
             $338 = HEAP32[$337>>2]|0;
             $339 = (_sqlite3SchemaToIndex($335,$338)|0);
             HEAP32[$iDb>>2] = $339;
             $340 = HEAP32[$iDb>>2]|0;
             $341 = ($340|0)>=(0);
             if ($341) {
              $342 = HEAP32[$iDb>>2]|0;
              $343 = HEAP32[$db>>2]|0;
              $344 = (($343) + 16|0);
              $345 = HEAP32[$344>>2]|0;
              $346 = (($345) + ($342<<4)|0);
              $347 = HEAP32[$346>>2]|0;
              $348 = $347;
             } else {
              $348 = 17440;
             }
             HEAP32[$zSchemaName>>2] = $348;
             label = 67;
            }
           } while(0);
           L98: do {
            if ((label|0) == 67) {
             label = 0;
             HEAP32[$j>>2] = 0;
             while(1) {
              $349 = HEAP32[$j>>2]|0;
              $350 = HEAP32[$pTab1>>2]|0;
              $351 = (($350) + 38|0);
              $352 = HEAP16[$351>>1]|0;
              $353 = $352 << 16 >> 16;
              $354 = ($349|0)<($353|0);
              if (!($354)) {
               break L98;
              }
              $355 = HEAP32[$j>>2]|0;
              $356 = HEAP32[$pTab1>>2]|0;
              $357 = (($356) + 4|0);
              $358 = HEAP32[$357>>2]|0;
              $359 = (($358) + (($355*24)|0)|0);
              $360 = HEAP32[$359>>2]|0;
              HEAP32[$zName>>2] = $360;
              $361 = HEAP32[$zTName>>2]|0;
              $362 = ($361|0)!=(0|0);
              $363 = HEAP32[$pSub>>2]|0;
              $364 = ($363|0)!=(0|0);
              $or$cond = $362 & $364;
              if ($or$cond) {
               $365 = HEAP32[$j>>2]|0;
               $366 = HEAP32[$pSub>>2]|0;
               $367 = HEAP32[$366>>2]|0;
               $368 = (($367) + 4|0);
               $369 = HEAP32[$368>>2]|0;
               $370 = (($369) + (($365*20)|0)|0);
               $371 = (($370) + 8|0);
               $372 = HEAP32[$371>>2]|0;
               $373 = HEAP32[$zTName>>2]|0;
               $374 = (_sqlite3MatchSpanName($372,0,$373,0)|0);
               $375 = ($374|0)==(0);
               if (!($375)) {
                label = 71;
               }
              } else {
               label = 71;
              }
              do {
               if ((label|0) == 71) {
                label = 0;
                $376 = HEAP32[$j>>2]|0;
                $377 = HEAP32[$pTab1>>2]|0;
                $378 = (($377) + 4|0);
                $379 = HEAP32[$378>>2]|0;
                $380 = (($379) + (($376*24)|0)|0);
                $381 = (($380) + 23|0);
                $382 = HEAP8[$381>>0]|0;
                $383 = $382&255;
                $384 = $383 & 2;
                $385 = ($384|0)!=(0);
                if ($385) {
                 break;
                }
                HEAP32[$tableSeen>>2] = 1;
                $386 = HEAP32[$i>>2]|0;
                $387 = ($386|0)>(0);
                $388 = HEAP32[$zTName>>2]|0;
                $389 = ($388|0)==(0|0);
                $or$cond3 = $387 & $389;
                if ($or$cond3) {
                 $390 = HEAP32[$pFrom>>2]|0;
                 $391 = (($390) + 36|0);
                 $392 = HEAP8[$391>>0]|0;
                 $393 = $392&255;
                 $394 = $393 & 4;
                 $395 = ($394|0)!=(0);
                 if ($395) {
                  $396 = HEAP32[$pTabList>>2]|0;
                  $397 = HEAP32[$i>>2]|0;
                  $398 = HEAP32[$zName>>2]|0;
                  $399 = (_tableAndColumnIndex($396,$397,$398,0,0)|0);
                  $400 = ($399|0)!=(0);
                  if ($400) {
                   break;
                  }
                 }
                 $401 = HEAP32[$pFrom>>2]|0;
                 $402 = (($401) + 48|0);
                 $403 = HEAP32[$402>>2]|0;
                 $404 = HEAP32[$zName>>2]|0;
                 $405 = (_sqlite3IdListIndex($403,$404)|0);
                 $406 = ($405|0)>=(0);
                 if ($406) {
                  break;
                 }
                }
                $407 = HEAP32[$db>>2]|0;
                $408 = HEAP32[$zName>>2]|0;
                $409 = (_sqlite3Expr($407,27,$408)|0);
                HEAP32[$pRight>>2] = $409;
                $410 = HEAP32[$zName>>2]|0;
                HEAP32[$zColname>>2] = $410;
                HEAP32[$zToFree>>2] = 0;
                $411 = HEAP32[$longNames>>2]|0;
                $412 = ($411|0)!=(0);
                do {
                 if ($412) {
                  label = 78;
                 } else {
                  $413 = HEAP32[$pTabList>>2]|0;
                  $414 = HEAP32[$413>>2]|0;
                  $415 = ($414|0)>(1);
                  if ($415) {
                   label = 78;
                   break;
                  }
                  $439 = HEAP32[$pRight>>2]|0;
                  HEAP32[$pExpr>>2] = $439;
                 }
                } while(0);
                do {
                 if ((label|0) == 78) {
                  label = 0;
                  $416 = HEAP32[$db>>2]|0;
                  $417 = HEAP32[$zTabName>>2]|0;
                  $418 = (_sqlite3Expr($416,27,$417)|0);
                  HEAP32[$pLeft>>2] = $418;
                  $419 = HEAP32[$pParse>>2]|0;
                  $420 = HEAP32[$pLeft>>2]|0;
                  $421 = HEAP32[$pRight>>2]|0;
                  $422 = (_sqlite3PExpr($419,122,$420,$421,0)|0);
                  HEAP32[$pExpr>>2] = $422;
                  $423 = HEAP32[$zSchemaName>>2]|0;
                  $424 = ($423|0)!=(0|0);
                  if ($424) {
                   $425 = HEAP32[$db>>2]|0;
                   $426 = HEAP32[$zSchemaName>>2]|0;
                   $427 = (_sqlite3Expr($425,27,$426)|0);
                   HEAP32[$pLeft>>2] = $427;
                   $428 = HEAP32[$pParse>>2]|0;
                   $429 = HEAP32[$pLeft>>2]|0;
                   $430 = HEAP32[$pExpr>>2]|0;
                   $431 = (_sqlite3PExpr($428,122,$429,$430,0)|0);
                   HEAP32[$pExpr>>2] = $431;
                  }
                  $432 = HEAP32[$longNames>>2]|0;
                  $433 = ($432|0)!=(0);
                  if (!($433)) {
                   break;
                  }
                  $434 = HEAP32[$db>>2]|0;
                  $435 = HEAP32[$zTabName>>2]|0;
                  $436 = HEAP32[$zName>>2]|0;
                  HEAP32[$vararg_buffer21>>2] = $435;
                  $vararg_ptr10 = (($vararg_buffer21) + 4|0);
                  HEAP32[$vararg_ptr10>>2] = $436;
                  $437 = (_sqlite3MPrintf($434,17448,$vararg_buffer21)|0);
                  HEAP32[$zColname>>2] = $437;
                  $438 = HEAP32[$zColname>>2]|0;
                  HEAP32[$zToFree>>2] = $438;
                 }
                } while(0);
                $440 = HEAP32[$pParse>>2]|0;
                $441 = HEAP32[$pNew>>2]|0;
                $442 = HEAP32[$pExpr>>2]|0;
                $443 = (_sqlite3ExprListAppend($440,$441,$442)|0);
                HEAP32[$pNew>>2] = $443;
                $444 = HEAP32[$zColname>>2]|0;
                HEAP32[$sColname>>2] = $444;
                $445 = HEAP32[$zColname>>2]|0;
                $446 = (_sqlite3Strlen30($445)|0);
                $447 = (($sColname) + 4|0);
                HEAP32[$447>>2] = $446;
                $448 = HEAP32[$pParse>>2]|0;
                $449 = HEAP32[$pNew>>2]|0;
                _sqlite3ExprListSetName($448,$449,$sColname,0);
                $450 = HEAP32[$pNew>>2]|0;
                $451 = ($450|0)!=(0|0);
                do {
                 if ($451) {
                  $452 = HEAP32[$2>>2]|0;
                  $453 = (($452) + 6|0);
                  $454 = HEAP16[$453>>1]|0;
                  $455 = $454&65535;
                  $456 = $455 & 512;
                  $457 = ($456|0)!=(0);
                  if (!($457)) {
                   break;
                  }
                  $458 = HEAP32[$pNew>>2]|0;
                  $459 = HEAP32[$458>>2]|0;
                  $460 = (($459) - 1)|0;
                  $461 = HEAP32[$pNew>>2]|0;
                  $462 = (($461) + 4|0);
                  $463 = HEAP32[$462>>2]|0;
                  $464 = (($463) + (($460*20)|0)|0);
                  HEAP32[$pX>>2] = $464;
                  $465 = HEAP32[$pSub>>2]|0;
                  $466 = ($465|0)!=(0|0);
                  $467 = HEAP32[$db>>2]|0;
                  if ($466) {
                   $468 = HEAP32[$j>>2]|0;
                   $469 = HEAP32[$pSub>>2]|0;
                   $470 = HEAP32[$469>>2]|0;
                   $471 = (($470) + 4|0);
                   $472 = HEAP32[$471>>2]|0;
                   $473 = (($472) + (($468*20)|0)|0);
                   $474 = (($473) + 8|0);
                   $475 = HEAP32[$474>>2]|0;
                   $476 = (_sqlite3DbStrDup($467,$475)|0);
                   $477 = HEAP32[$pX>>2]|0;
                   $478 = (($477) + 8|0);
                   HEAP32[$478>>2] = $476;
                  } else {
                   $479 = HEAP32[$zSchemaName>>2]|0;
                   $480 = HEAP32[$zTabName>>2]|0;
                   $481 = HEAP32[$zColname>>2]|0;
                   HEAP32[$vararg_buffer21>>2] = $479;
                   $vararg_ptr14 = (($vararg_buffer21) + 4|0);
                   HEAP32[$vararg_ptr14>>2] = $480;
                   $vararg_ptr15 = (($vararg_buffer21) + 8|0);
                   HEAP32[$vararg_ptr15>>2] = $481;
                   $482 = (_sqlite3MPrintf($467,17456,$vararg_buffer21)|0);
                   $483 = HEAP32[$pX>>2]|0;
                   $484 = (($483) + 8|0);
                   HEAP32[$484>>2] = $482;
                  }
                  $485 = HEAP32[$pX>>2]|0;
                  $486 = (($485) + 13|0);
                  $487 = HEAP8[$486>>0]|0;
                  $488 = $487 & -3;
                  $489 = $488 | 2;
                  HEAP8[$486>>0] = $489;
                 }
                } while(0);
                $490 = HEAP32[$db>>2]|0;
                $491 = HEAP32[$zToFree>>2]|0;
                _sqlite3DbFree($490,$491);
               }
              } while(0);
              $492 = HEAP32[$j>>2]|0;
              $493 = (($492) + 1)|0;
              HEAP32[$j>>2] = $493;
             }
            }
           } while(0);
           $494 = HEAP32[$i>>2]|0;
           $495 = (($494) + 1)|0;
           HEAP32[$i>>2] = $495;
           $496 = HEAP32[$pFrom>>2]|0;
           $497 = (($496) + 72|0);
           HEAP32[$pFrom>>2] = $497;
          }
          $498 = HEAP32[$tableSeen>>2]|0;
          $499 = ($498|0)!=(0);
          if (!($499)) {
           $500 = HEAP32[$zTName>>2]|0;
           $501 = ($500|0)!=(0|0);
           $502 = HEAP32[$pParse>>2]|0;
           if ($501) {
            $503 = HEAP32[$zTName>>2]|0;
            HEAP32[$vararg_buffer21>>2] = $503;
            _sqlite3ErrorMsg($502,17472,$vararg_buffer21);
            break;
           } else {
            _sqlite3ErrorMsg($502,17496,$vararg_buffer21);
            break;
           }
          }
         }
        } while(0);
        $504 = HEAP32[$k>>2]|0;
        $505 = (($504) + 1)|0;
        HEAP32[$k>>2] = $505;
       }
       $506 = HEAP32[$db>>2]|0;
       $507 = HEAP32[$pEList>>2]|0;
       _sqlite3ExprListDelete($506,$507);
       $508 = HEAP32[$pNew>>2]|0;
       $509 = HEAP32[$2>>2]|0;
       HEAP32[$509>>2] = $508;
      }
      $510 = HEAP32[$2>>2]|0;
      $511 = HEAP32[$510>>2]|0;
      $512 = ($511|0)!=(0|0);
      if ($512) {
       $513 = HEAP32[$2>>2]|0;
       $514 = HEAP32[$513>>2]|0;
       $515 = HEAP32[$514>>2]|0;
       $516 = HEAP32[$db>>2]|0;
       $517 = (($516) + 88|0);
       $518 = (($517) + 8|0);
       $519 = HEAP32[$518>>2]|0;
       $520 = ($515|0)>($519|0);
       if ($520) {
        $521 = HEAP32[$pParse>>2]|0;
        _sqlite3ErrorMsg($521,17520,$vararg_buffer21);
       }
      }
      HEAP32[$0>>2] = 0;
      $522 = HEAP32[$0>>2]|0;
      STACKTOP = sp;return ($522|0);
     }
    }
    HEAP32[$0>>2] = 2;
    $522 = HEAP32[$0>>2]|0;
    STACKTOP = sp;return ($522|0);
   }
  }
 }
 HEAP32[$0>>2] = 1;
 $522 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($522|0);
}
function _selectPopWith($pWalker,$p) {
 $pWalker = $pWalker|0;
 $p = $p|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $pParse = 0, $pWith = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $1 = sp + 8|0;
 $pParse = sp + 4|0;
 $pWith = sp;
 HEAP32[$0>>2] = $pWalker;
 HEAP32[$1>>2] = $p;
 $2 = HEAP32[$0>>2]|0;
 $3 = (($2) + 12|0);
 $4 = HEAP32[$3>>2]|0;
 HEAP32[$pParse>>2] = $4;
 $5 = HEAP32[$1>>2]|0;
 $6 = (_findRightmost($5)|0);
 $7 = (($6) + 68|0);
 $8 = HEAP32[$7>>2]|0;
 HEAP32[$pWith>>2] = $8;
 $9 = HEAP32[$pWith>>2]|0;
 $10 = ($9|0)!=(0|0);
 if (!($10)) {
  STACKTOP = sp;return;
 }
 $11 = HEAP32[$pWith>>2]|0;
 $12 = (($11) + 4|0);
 $13 = HEAP32[$12>>2]|0;
 $14 = HEAP32[$pParse>>2]|0;
 $15 = (($14) + 532|0);
 HEAP32[$15>>2] = $13;
 STACKTOP = sp;return;
}
function _findRightmost($p) {
 $p = $p|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 HEAP32[$0>>2] = $p;
 while(1) {
  $1 = HEAP32[$0>>2]|0;
  $2 = (($1) + 56|0);
  $3 = HEAP32[$2>>2]|0;
  $4 = ($3|0)!=(0|0);
  $5 = HEAP32[$0>>2]|0;
  if (!($4)) {
   break;
  }
  $6 = (($5) + 56|0);
  $7 = HEAP32[$6>>2]|0;
  HEAP32[$0>>2] = $7;
 }
 STACKTOP = sp;return ($5|0);
}
function _withExpand($pWalker,$pFrom) {
 $pWalker = $pWalker|0;
 $pFrom = $pFrom|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0;
 var $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0, $188 = 0;
 var $189 = 0, $19 = 0, $190 = 0, $191 = 0, $192 = 0, $193 = 0, $194 = 0, $195 = 0, $196 = 0, $197 = 0, $198 = 0, $199 = 0, $2 = 0, $20 = 0, $200 = 0, $201 = 0, $202 = 0, $203 = 0, $204 = 0, $205 = 0;
 var $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0;
 var $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0;
 var $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0;
 var $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0;
 var $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $bMayRecursive = 0, $db = 0, $i = 0, $pCte = 0, $pEList = 0, $pItem = 0, $pLeft = 0, $pParse = 0, $pSavedWith = 0, $pSel = 0, $pSrc = 0, $pTab = 0, $pWith = 0, $vararg_buffer4 = 0;
 var $vararg_ptr7 = 0, $vararg_ptr8 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 80|0;
 $vararg_buffer4 = sp;
 $0 = sp + 72|0;
 $1 = sp + 68|0;
 $2 = sp + 64|0;
 $pParse = sp + 60|0;
 $db = sp + 56|0;
 $pCte = sp + 52|0;
 $pWith = sp + 48|0;
 $pTab = sp + 44|0;
 $pEList = sp + 40|0;
 $pSel = sp + 36|0;
 $pLeft = sp + 32|0;
 $bMayRecursive = sp + 28|0;
 $pSavedWith = sp + 24|0;
 $i = sp + 20|0;
 $pSrc = sp + 16|0;
 $pItem = sp + 12|0;
 HEAP32[$1>>2] = $pWalker;
 HEAP32[$2>>2] = $pFrom;
 $3 = HEAP32[$1>>2]|0;
 $4 = (($3) + 12|0);
 $5 = HEAP32[$4>>2]|0;
 HEAP32[$pParse>>2] = $5;
 $6 = HEAP32[$pParse>>2]|0;
 $7 = HEAP32[$6>>2]|0;
 HEAP32[$db>>2] = $7;
 $8 = HEAP32[$pParse>>2]|0;
 $9 = (($8) + 532|0);
 $10 = HEAP32[$9>>2]|0;
 $11 = HEAP32[$2>>2]|0;
 $12 = (_searchWith($10,$11,$pWith)|0);
 HEAP32[$pCte>>2] = $12;
 $13 = HEAP32[$pCte>>2]|0;
 $14 = ($13|0)!=(0|0);
 if ($14) {
  $15 = HEAP32[$pCte>>2]|0;
  $16 = (($15) + 12|0);
  $17 = HEAP32[$16>>2]|0;
  $18 = ($17|0)!=(0|0);
  if ($18) {
   $19 = HEAP32[$pParse>>2]|0;
   $20 = HEAP32[$pCte>>2]|0;
   $21 = (($20) + 12|0);
   $22 = HEAP32[$21>>2]|0;
   $23 = HEAP32[$pCte>>2]|0;
   $24 = HEAP32[$23>>2]|0;
   HEAP32[$vararg_buffer4>>2] = $24;
   _sqlite3ErrorMsg($19,$22,$vararg_buffer4);
   HEAP32[$0>>2] = 1;
   $205 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($205|0);
  }
  $25 = HEAP32[$db>>2]|0;
  $26 = (_sqlite3DbMallocZero($25,76,0)|0);
  HEAP32[$pTab>>2] = $26;
  $27 = HEAP32[$2>>2]|0;
  $28 = (($27) + 16|0);
  HEAP32[$28>>2] = $26;
  $29 = HEAP32[$pTab>>2]|0;
  $30 = ($29|0)==(0|0);
  if ($30) {
   HEAP32[$0>>2] = 2;
   $205 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($205|0);
  }
  $31 = HEAP32[$pTab>>2]|0;
  $32 = (($31) + 40|0);
  HEAP16[$32>>1] = 1;
  $33 = HEAP32[$db>>2]|0;
  $34 = HEAP32[$pCte>>2]|0;
  $35 = HEAP32[$34>>2]|0;
  $36 = (_sqlite3DbStrDup($33,$35)|0);
  $37 = HEAP32[$pTab>>2]|0;
  HEAP32[$37>>2] = $36;
  $38 = HEAP32[$pTab>>2]|0;
  $39 = (($38) + 36|0);
  HEAP16[$39>>1] = -1;
  $40 = HEAP32[$pTab>>2]|0;
  $41 = (($40) + 28|0);
  HEAP16[$41>>1] = 200;
  $42 = HEAP32[$pTab>>2]|0;
  $43 = (($42) + 44|0);
  $44 = HEAP8[$43>>0]|0;
  $45 = $44&255;
  $46 = $45 | 2;
  $47 = $46&255;
  HEAP8[$43>>0] = $47;
  $48 = HEAP32[$db>>2]|0;
  $49 = HEAP32[$pCte>>2]|0;
  $50 = (($49) + 8|0);
  $51 = HEAP32[$50>>2]|0;
  $52 = (_sqlite3SelectDup($48,$51,0)|0);
  $53 = HEAP32[$2>>2]|0;
  $54 = (($53) + 20|0);
  HEAP32[$54>>2] = $52;
  $55 = HEAP32[$db>>2]|0;
  $56 = (($55) + 64|0);
  $57 = HEAP8[$56>>0]|0;
  $58 = ($57<<24>>24)!=(0);
  if ($58) {
   HEAP32[$0>>2] = 7;
   $205 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($205|0);
  }
  $59 = HEAP32[$2>>2]|0;
  $60 = (($59) + 20|0);
  $61 = HEAP32[$60>>2]|0;
  HEAP32[$pSel>>2] = $61;
  $62 = HEAP32[$pSel>>2]|0;
  $63 = (($62) + 4|0);
  $64 = HEAP8[$63>>0]|0;
  $65 = $64&255;
  $66 = ($65|0)==(116);
  if ($66) {
   $73 = 1;
  } else {
   $67 = HEAP32[$pSel>>2]|0;
   $68 = (($67) + 4|0);
   $69 = HEAP8[$68>>0]|0;
   $70 = $69&255;
   $71 = ($70|0)==(115);
   $73 = $71;
  }
  $72 = $73&1;
  HEAP32[$bMayRecursive>>2] = $72;
  $74 = HEAP32[$bMayRecursive>>2]|0;
  $75 = ($74|0)!=(0);
  L18: do {
   if ($75) {
    $76 = HEAP32[$2>>2]|0;
    $77 = (($76) + 20|0);
    $78 = HEAP32[$77>>2]|0;
    $79 = (($78) + 32|0);
    $80 = HEAP32[$79>>2]|0;
    HEAP32[$pSrc>>2] = $80;
    HEAP32[$i>>2] = 0;
    while(1) {
     $81 = HEAP32[$i>>2]|0;
     $82 = HEAP32[$pSrc>>2]|0;
     $83 = HEAP32[$82>>2]|0;
     $84 = ($81|0)<($83|0);
     if (!($84)) {
      break L18;
     }
     $85 = HEAP32[$i>>2]|0;
     $86 = HEAP32[$pSrc>>2]|0;
     $87 = (($86) + 8|0);
     $88 = (($87) + (($85*72)|0)|0);
     HEAP32[$pItem>>2] = $88;
     $89 = HEAP32[$pItem>>2]|0;
     $90 = (($89) + 4|0);
     $91 = HEAP32[$90>>2]|0;
     $92 = ($91|0)==(0|0);
     if ($92) {
      $93 = HEAP32[$pItem>>2]|0;
      $94 = (($93) + 8|0);
      $95 = HEAP32[$94>>2]|0;
      $96 = ($95|0)!=(0|0);
      if ($96) {
       $97 = HEAP32[$pItem>>2]|0;
       $98 = (($97) + 8|0);
       $99 = HEAP32[$98>>2]|0;
       $100 = HEAP32[$pCte>>2]|0;
       $101 = HEAP32[$100>>2]|0;
       $102 = (_sqlite3_stricmp($99,$101)|0);
       $103 = (0)==($102|0);
       if ($103) {
        $104 = HEAP32[$pTab>>2]|0;
        $105 = HEAP32[$pItem>>2]|0;
        $106 = (($105) + 16|0);
        HEAP32[$106>>2] = $104;
        $107 = HEAP32[$pItem>>2]|0;
        $108 = (($107) + 37|0);
        $109 = HEAP8[$108>>0]|0;
        $110 = $109 & -9;
        $111 = $110 | 8;
        HEAP8[$108>>0] = $111;
        $112 = HEAP32[$pTab>>2]|0;
        $113 = (($112) + 40|0);
        $114 = HEAP16[$113>>1]|0;
        $115 = (($114) + 1)<<16>>16;
        HEAP16[$113>>1] = $115;
        $116 = HEAP32[$pSel>>2]|0;
        $117 = (($116) + 6|0);
        $118 = HEAP16[$117>>1]|0;
        $119 = $118&65535;
        $120 = $119 | 2048;
        $121 = $120&65535;
        HEAP16[$117>>1] = $121;
       }
      }
     }
     $122 = HEAP32[$i>>2]|0;
     $123 = (($122) + 1)|0;
     HEAP32[$i>>2] = $123;
    }
   }
  } while(0);
  $124 = HEAP32[$pTab>>2]|0;
  $125 = (($124) + 40|0);
  $126 = HEAP16[$125>>1]|0;
  $127 = $126&65535;
  $128 = ($127|0)>(2);
  if ($128) {
   $129 = HEAP32[$pParse>>2]|0;
   $130 = HEAP32[$pCte>>2]|0;
   $131 = HEAP32[$130>>2]|0;
   HEAP32[$vararg_buffer4>>2] = $131;
   _sqlite3ErrorMsg($129,17752,$vararg_buffer4);
   HEAP32[$0>>2] = 1;
   $205 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($205|0);
  }
  $132 = HEAP32[$pCte>>2]|0;
  $133 = (($132) + 12|0);
  HEAP32[$133>>2] = 17800;
  $134 = HEAP32[$pParse>>2]|0;
  $135 = (($134) + 532|0);
  $136 = HEAP32[$135>>2]|0;
  HEAP32[$pSavedWith>>2] = $136;
  $137 = HEAP32[$pWith>>2]|0;
  $138 = HEAP32[$pParse>>2]|0;
  $139 = (($138) + 532|0);
  HEAP32[$139>>2] = $137;
  $140 = HEAP32[$1>>2]|0;
  $141 = HEAP32[$bMayRecursive>>2]|0;
  $142 = ($141|0)!=(0);
  $143 = HEAP32[$pSel>>2]|0;
  if ($142) {
   $144 = (($143) + 52|0);
   $145 = HEAP32[$144>>2]|0;
   $146 = $145;
  } else {
   $146 = $143;
  }
  (_sqlite3WalkSelect($140,$146)|0);
  $147 = HEAP32[$pSel>>2]|0;
  HEAP32[$pLeft>>2] = $147;
  while(1) {
   $148 = HEAP32[$pLeft>>2]|0;
   $149 = (($148) + 52|0);
   $150 = HEAP32[$149>>2]|0;
   $151 = ($150|0)!=(0|0);
   $152 = HEAP32[$pLeft>>2]|0;
   if (!($151)) {
    break;
   }
   $153 = (($152) + 52|0);
   $154 = HEAP32[$153>>2]|0;
   HEAP32[$pLeft>>2] = $154;
  }
  $155 = HEAP32[$152>>2]|0;
  HEAP32[$pEList>>2] = $155;
  $156 = HEAP32[$pCte>>2]|0;
  $157 = (($156) + 4|0);
  $158 = HEAP32[$157>>2]|0;
  $159 = ($158|0)!=(0|0);
  do {
   if ($159) {
    $160 = HEAP32[$pEList>>2]|0;
    $161 = HEAP32[$160>>2]|0;
    $162 = HEAP32[$pCte>>2]|0;
    $163 = (($162) + 4|0);
    $164 = HEAP32[$163>>2]|0;
    $165 = HEAP32[$164>>2]|0;
    $166 = ($161|0)!=($165|0);
    if (!($166)) {
     $179 = HEAP32[$pCte>>2]|0;
     $180 = (($179) + 4|0);
     $181 = HEAP32[$180>>2]|0;
     HEAP32[$pEList>>2] = $181;
     break;
    }
    $167 = HEAP32[$pParse>>2]|0;
    $168 = HEAP32[$pCte>>2]|0;
    $169 = HEAP32[$168>>2]|0;
    $170 = HEAP32[$pEList>>2]|0;
    $171 = HEAP32[$170>>2]|0;
    $172 = HEAP32[$pCte>>2]|0;
    $173 = (($172) + 4|0);
    $174 = HEAP32[$173>>2]|0;
    $175 = HEAP32[$174>>2]|0;
    HEAP32[$vararg_buffer4>>2] = $169;
    $vararg_ptr7 = (($vararg_buffer4) + 4|0);
    HEAP32[$vararg_ptr7>>2] = $171;
    $vararg_ptr8 = (($vararg_buffer4) + 8|0);
    HEAP32[$vararg_ptr8>>2] = $175;
    _sqlite3ErrorMsg($167,17824,$vararg_buffer4);
    $176 = HEAP32[$pSavedWith>>2]|0;
    $177 = HEAP32[$pParse>>2]|0;
    $178 = (($177) + 532|0);
    HEAP32[$178>>2] = $176;
    HEAP32[$0>>2] = 1;
    $205 = HEAP32[$0>>2]|0;
    STACKTOP = sp;return ($205|0);
   }
  } while(0);
  $182 = HEAP32[$pParse>>2]|0;
  $183 = HEAP32[$pEList>>2]|0;
  $184 = HEAP32[$pTab>>2]|0;
  $185 = (($184) + 38|0);
  $186 = HEAP32[$pTab>>2]|0;
  $187 = (($186) + 4|0);
  (_selectColumnsFromExprList($182,$183,$185,$187)|0);
  $188 = HEAP32[$bMayRecursive>>2]|0;
  $189 = ($188|0)!=(0);
  if ($189) {
   $190 = HEAP32[$pSel>>2]|0;
   $191 = (($190) + 6|0);
   $192 = HEAP16[$191>>1]|0;
   $193 = $192&65535;
   $194 = $193 & 2048;
   $195 = ($194|0)!=(0);
   $196 = HEAP32[$pCte>>2]|0;
   $197 = (($196) + 12|0);
   if ($195) {
    HEAP32[$197>>2] = 17864;
   } else {
    HEAP32[$197>>2] = 17904;
   }
   $198 = HEAP32[$1>>2]|0;
   $199 = HEAP32[$pSel>>2]|0;
   (_sqlite3WalkSelect($198,$199)|0);
  }
  $200 = HEAP32[$pCte>>2]|0;
  $201 = (($200) + 12|0);
  HEAP32[$201>>2] = 0;
  $202 = HEAP32[$pSavedWith>>2]|0;
  $203 = HEAP32[$pParse>>2]|0;
  $204 = (($203) + 532|0);
  HEAP32[$204>>2] = $202;
 }
 HEAP32[$0>>2] = 0;
 $205 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($205|0);
}
function _sqlite3IndexedByLookup($pParse,$pFrom) {
 $pParse = $pParse|0;
 $pFrom = $pFrom|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0;
 var $pIdx = 0, $pTab = 0, $vararg_buffer = 0, $vararg_ptr1 = 0, $zIndex = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $vararg_buffer = sp;
 $0 = sp + 28|0;
 $1 = sp + 24|0;
 $2 = sp + 20|0;
 $pTab = sp + 16|0;
 $zIndex = sp + 12|0;
 $pIdx = sp + 8|0;
 HEAP32[$1>>2] = $pParse;
 HEAP32[$2>>2] = $pFrom;
 $3 = HEAP32[$2>>2]|0;
 $4 = (($3) + 16|0);
 $5 = HEAP32[$4>>2]|0;
 $6 = ($5|0)!=(0|0);
 do {
  if ($6) {
   $7 = HEAP32[$2>>2]|0;
   $8 = (($7) + 64|0);
   $9 = HEAP32[$8>>2]|0;
   $10 = ($9|0)!=(0|0);
   if ($10) {
    $11 = HEAP32[$2>>2]|0;
    $12 = (($11) + 16|0);
    $13 = HEAP32[$12>>2]|0;
    HEAP32[$pTab>>2] = $13;
    $14 = HEAP32[$2>>2]|0;
    $15 = (($14) + 64|0);
    $16 = HEAP32[$15>>2]|0;
    HEAP32[$zIndex>>2] = $16;
    $17 = HEAP32[$pTab>>2]|0;
    $18 = (($17) + 8|0);
    $19 = HEAP32[$18>>2]|0;
    HEAP32[$pIdx>>2] = $19;
    while(1) {
     $20 = HEAP32[$pIdx>>2]|0;
     $21 = ($20|0)!=(0|0);
     if (!($21)) {
      break;
     }
     $22 = HEAP32[$pIdx>>2]|0;
     $23 = HEAP32[$22>>2]|0;
     $24 = HEAP32[$zIndex>>2]|0;
     $25 = (_sqlite3_stricmp($23,$24)|0);
     $26 = ($25|0)!=(0);
     if (!($26)) {
      break;
     }
     $27 = HEAP32[$pIdx>>2]|0;
     $28 = (($27) + 20|0);
     $29 = HEAP32[$28>>2]|0;
     HEAP32[$pIdx>>2] = $29;
    }
    $30 = HEAP32[$pIdx>>2]|0;
    $31 = ($30|0)!=(0|0);
    if ($31) {
     $36 = HEAP32[$pIdx>>2]|0;
     $37 = HEAP32[$2>>2]|0;
     $38 = (($37) + 68|0);
     HEAP32[$38>>2] = $36;
     break;
    }
    $32 = HEAP32[$1>>2]|0;
    $33 = HEAP32[$zIndex>>2]|0;
    HEAP32[$vararg_buffer>>2] = $33;
    $vararg_ptr1 = (($vararg_buffer) + 4|0);
    HEAP32[$vararg_ptr1>>2] = 0;
    _sqlite3ErrorMsg($32,17728,$vararg_buffer);
    $34 = HEAP32[$1>>2]|0;
    $35 = (($34) + 17|0);
    HEAP8[$35>>0] = 1;
    HEAP32[$0>>2] = 1;
    $39 = HEAP32[$0>>2]|0;
    STACKTOP = sp;return ($39|0);
   }
  }
 } while(0);
 HEAP32[$0>>2] = 0;
 $39 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($39|0);
}
function _sqliteProcessJoin($pParse,$p) {
 $pParse = $pParse|0;
 $p = $p|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0;
 var $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0;
 var $i = 0, $iLeft = 0, $iLeft2 = 0, $iLeftCol = 0, $iLeftCol3 = 0, $iRightCol = 0, $isOuter = 0, $j = 0, $or$cond = 0, $pLeft = 0, $pLeftTab = 0, $pList = 0, $pRight = 0, $pRightTab = 0, $pSrc = 0, $vararg_buffer4 = 0, $zName = 0, $zName1 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 80|0;
 $vararg_buffer4 = sp;
 $0 = sp + 76|0;
 $1 = sp + 72|0;
 $2 = sp + 68|0;
 $pSrc = sp + 64|0;
 $i = sp + 60|0;
 $j = sp + 56|0;
 $pLeft = sp + 52|0;
 $pRight = sp + 48|0;
 $pLeftTab = sp + 44|0;
 $pRightTab = sp + 40|0;
 $isOuter = sp + 36|0;
 $zName = sp + 32|0;
 $iLeft = sp + 28|0;
 $iLeftCol = sp + 24|0;
 $pList = sp + 20|0;
 $zName1 = sp + 16|0;
 $iLeft2 = sp + 12|0;
 $iLeftCol3 = sp + 8|0;
 $iRightCol = sp + 4|0;
 HEAP32[$1>>2] = $pParse;
 HEAP32[$2>>2] = $p;
 $3 = HEAP32[$2>>2]|0;
 $4 = (($3) + 32|0);
 $5 = HEAP32[$4>>2]|0;
 HEAP32[$pSrc>>2] = $5;
 $6 = HEAP32[$pSrc>>2]|0;
 $7 = (($6) + 8|0);
 HEAP32[$pLeft>>2] = $7;
 $8 = HEAP32[$pLeft>>2]|0;
 $9 = (($8) + 72|0);
 HEAP32[$pRight>>2] = $9;
 HEAP32[$i>>2] = 0;
 L1: while(1) {
  $10 = HEAP32[$i>>2]|0;
  $11 = HEAP32[$pSrc>>2]|0;
  $12 = HEAP32[$11>>2]|0;
  $13 = (($12) - 1)|0;
  $14 = ($10|0)<($13|0);
  if (!($14)) {
   label = 28;
   break;
  }
  $15 = HEAP32[$pLeft>>2]|0;
  $16 = (($15) + 16|0);
  $17 = HEAP32[$16>>2]|0;
  HEAP32[$pLeftTab>>2] = $17;
  $18 = HEAP32[$pRight>>2]|0;
  $19 = (($18) + 16|0);
  $20 = HEAP32[$19>>2]|0;
  HEAP32[$pRightTab>>2] = $20;
  $21 = HEAP32[$pLeftTab>>2]|0;
  $22 = ($21|0)==(0|0);
  $23 = HEAP32[$pRightTab>>2]|0;
  $24 = ($23|0)==(0|0);
  $or$cond = $22 | $24;
  L4: do {
   if (!($or$cond)) {
    $25 = HEAP32[$pRight>>2]|0;
    $26 = (($25) + 36|0);
    $27 = HEAP8[$26>>0]|0;
    $28 = $27&255;
    $29 = $28 & 32;
    $30 = ($29|0)!=(0);
    $31 = $30&1;
    HEAP32[$isOuter>>2] = $31;
    $32 = HEAP32[$pRight>>2]|0;
    $33 = (($32) + 36|0);
    $34 = HEAP8[$33>>0]|0;
    $35 = $34&255;
    $36 = $35 & 4;
    $37 = ($36|0)!=(0);
    L6: do {
     if ($37) {
      $38 = HEAP32[$pRight>>2]|0;
      $39 = (($38) + 44|0);
      $40 = HEAP32[$39>>2]|0;
      $41 = ($40|0)!=(0|0);
      if ($41) {
       label = 7;
       break L1;
      }
      $42 = HEAP32[$pRight>>2]|0;
      $43 = (($42) + 48|0);
      $44 = HEAP32[$43>>2]|0;
      $45 = ($44|0)!=(0|0);
      if ($45) {
       label = 7;
       break L1;
      }
      HEAP32[$j>>2] = 0;
      while(1) {
       $47 = HEAP32[$j>>2]|0;
       $48 = HEAP32[$pRightTab>>2]|0;
       $49 = (($48) + 38|0);
       $50 = HEAP16[$49>>1]|0;
       $51 = $50 << 16 >> 16;
       $52 = ($47|0)<($51|0);
       if (!($52)) {
        break L6;
       }
       $53 = HEAP32[$j>>2]|0;
       $54 = HEAP32[$pRightTab>>2]|0;
       $55 = (($54) + 4|0);
       $56 = HEAP32[$55>>2]|0;
       $57 = (($56) + (($53*24)|0)|0);
       $58 = HEAP32[$57>>2]|0;
       HEAP32[$zName>>2] = $58;
       $59 = HEAP32[$pSrc>>2]|0;
       $60 = HEAP32[$i>>2]|0;
       $61 = (($60) + 1)|0;
       $62 = HEAP32[$zName>>2]|0;
       $63 = (_tableAndColumnIndex($59,$61,$62,$iLeft,$iLeftCol)|0);
       $64 = ($63|0)!=(0);
       if ($64) {
        $65 = HEAP32[$1>>2]|0;
        $66 = HEAP32[$pSrc>>2]|0;
        $67 = HEAP32[$iLeft>>2]|0;
        $68 = HEAP32[$iLeftCol>>2]|0;
        $69 = HEAP32[$i>>2]|0;
        $70 = (($69) + 1)|0;
        $71 = HEAP32[$j>>2]|0;
        $72 = HEAP32[$isOuter>>2]|0;
        $73 = HEAP32[$2>>2]|0;
        $74 = (($73) + 36|0);
        _addWhereTerm($65,$66,$67,$68,$70,$71,$72,$74);
       }
       $75 = HEAP32[$j>>2]|0;
       $76 = (($75) + 1)|0;
       HEAP32[$j>>2] = $76;
      }
     }
    } while(0);
    $77 = HEAP32[$pRight>>2]|0;
    $78 = (($77) + 44|0);
    $79 = HEAP32[$78>>2]|0;
    $80 = ($79|0)!=(0|0);
    if ($80) {
     $81 = HEAP32[$pRight>>2]|0;
     $82 = (($81) + 48|0);
     $83 = HEAP32[$82>>2]|0;
     $84 = ($83|0)!=(0|0);
     if ($84) {
      label = 15;
      break L1;
     }
    }
    $86 = HEAP32[$pRight>>2]|0;
    $87 = (($86) + 44|0);
    $88 = HEAP32[$87>>2]|0;
    $89 = ($88|0)!=(0|0);
    if ($89) {
     $90 = HEAP32[$isOuter>>2]|0;
     $91 = ($90|0)!=(0);
     if ($91) {
      $92 = HEAP32[$pRight>>2]|0;
      $93 = (($92) + 44|0);
      $94 = HEAP32[$93>>2]|0;
      $95 = HEAP32[$pRight>>2]|0;
      $96 = (($95) + 40|0);
      $97 = HEAP32[$96>>2]|0;
      _setJoinExpr($94,$97);
     }
     $98 = HEAP32[$1>>2]|0;
     $99 = HEAP32[$98>>2]|0;
     $100 = HEAP32[$2>>2]|0;
     $101 = (($100) + 36|0);
     $102 = HEAP32[$101>>2]|0;
     $103 = HEAP32[$pRight>>2]|0;
     $104 = (($103) + 44|0);
     $105 = HEAP32[$104>>2]|0;
     $106 = (_sqlite3ExprAnd($99,$102,$105)|0);
     $107 = HEAP32[$2>>2]|0;
     $108 = (($107) + 36|0);
     HEAP32[$108>>2] = $106;
     $109 = HEAP32[$pRight>>2]|0;
     $110 = (($109) + 44|0);
     HEAP32[$110>>2] = 0;
    }
    $111 = HEAP32[$pRight>>2]|0;
    $112 = (($111) + 48|0);
    $113 = HEAP32[$112>>2]|0;
    $114 = ($113|0)!=(0|0);
    if ($114) {
     $115 = HEAP32[$pRight>>2]|0;
     $116 = (($115) + 48|0);
     $117 = HEAP32[$116>>2]|0;
     HEAP32[$pList>>2] = $117;
     HEAP32[$j>>2] = 0;
     while(1) {
      $118 = HEAP32[$j>>2]|0;
      $119 = HEAP32[$pList>>2]|0;
      $120 = (($119) + 4|0);
      $121 = HEAP32[$120>>2]|0;
      $122 = ($118|0)<($121|0);
      if (!($122)) {
       break L4;
      }
      $123 = HEAP32[$j>>2]|0;
      $124 = HEAP32[$pList>>2]|0;
      $125 = HEAP32[$124>>2]|0;
      $126 = (($125) + ($123<<3)|0);
      $127 = HEAP32[$126>>2]|0;
      HEAP32[$zName1>>2] = $127;
      $128 = HEAP32[$pRightTab>>2]|0;
      $129 = HEAP32[$zName1>>2]|0;
      $130 = (_columnIndex($128,$129)|0);
      HEAP32[$iRightCol>>2] = $130;
      $131 = HEAP32[$iRightCol>>2]|0;
      $132 = ($131|0)<(0);
      if ($132) {
       label = 25;
       break L1;
      }
      $133 = HEAP32[$pSrc>>2]|0;
      $134 = HEAP32[$i>>2]|0;
      $135 = (($134) + 1)|0;
      $136 = HEAP32[$zName1>>2]|0;
      $137 = (_tableAndColumnIndex($133,$135,$136,$iLeft2,$iLeftCol3)|0);
      $138 = ($137|0)!=(0);
      if (!($138)) {
       label = 25;
       break L1;
      }
      $141 = HEAP32[$1>>2]|0;
      $142 = HEAP32[$pSrc>>2]|0;
      $143 = HEAP32[$iLeft2>>2]|0;
      $144 = HEAP32[$iLeftCol3>>2]|0;
      $145 = HEAP32[$i>>2]|0;
      $146 = (($145) + 1)|0;
      $147 = HEAP32[$iRightCol>>2]|0;
      $148 = HEAP32[$isOuter>>2]|0;
      $149 = HEAP32[$2>>2]|0;
      $150 = (($149) + 36|0);
      _addWhereTerm($141,$142,$143,$144,$146,$147,$148,$150);
      $151 = HEAP32[$j>>2]|0;
      $152 = (($151) + 1)|0;
      HEAP32[$j>>2] = $152;
     }
    }
   }
  } while(0);
  $153 = HEAP32[$i>>2]|0;
  $154 = (($153) + 1)|0;
  HEAP32[$i>>2] = $154;
  $155 = HEAP32[$pRight>>2]|0;
  $156 = (($155) + 72|0);
  HEAP32[$pRight>>2] = $156;
  $157 = HEAP32[$pLeft>>2]|0;
  $158 = (($157) + 72|0);
  HEAP32[$pLeft>>2] = $158;
 }
 if ((label|0) == 7) {
  $46 = HEAP32[$1>>2]|0;
  HEAP32[$vararg_buffer4>>2] = 0;
  _sqlite3ErrorMsg($46,17552,$vararg_buffer4);
  HEAP32[$0>>2] = 1;
  $159 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($159|0);
 }
 else if ((label|0) == 15) {
  $85 = HEAP32[$1>>2]|0;
  _sqlite3ErrorMsg($85,17608,$vararg_buffer4);
  HEAP32[$0>>2] = 1;
  $159 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($159|0);
 }
 else if ((label|0) == 25) {
  $139 = HEAP32[$1>>2]|0;
  $140 = HEAP32[$zName1>>2]|0;
  HEAP32[$vararg_buffer4>>2] = $140;
  _sqlite3ErrorMsg($139,17664,$vararg_buffer4);
  HEAP32[$0>>2] = 1;
  $159 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($159|0);
 }
 else if ((label|0) == 28) {
  HEAP32[$0>>2] = 0;
  $159 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($159|0);
 }
 return 0|0;
}
function _tableAndColumnIndex($pSrc,$N,$zCol,$piTab,$piCol) {
 $pSrc = $pSrc|0;
 $N = $N|0;
 $zCol = $zCol|0;
 $piTab = $piTab|0;
 $piCol = $piCol|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $i = 0, $iCol = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 28|0;
 $1 = sp + 24|0;
 $2 = sp + 20|0;
 $3 = sp + 16|0;
 $4 = sp + 12|0;
 $5 = sp + 8|0;
 $i = sp + 4|0;
 $iCol = sp;
 HEAP32[$1>>2] = $pSrc;
 HEAP32[$2>>2] = $N;
 HEAP32[$3>>2] = $zCol;
 HEAP32[$4>>2] = $piTab;
 HEAP32[$5>>2] = $piCol;
 HEAP32[$i>>2] = 0;
 while(1) {
  $6 = HEAP32[$i>>2]|0;
  $7 = HEAP32[$2>>2]|0;
  $8 = ($6|0)<($7|0);
  if (!($8)) {
   label = 8;
   break;
  }
  $9 = HEAP32[$i>>2]|0;
  $10 = HEAP32[$1>>2]|0;
  $11 = (($10) + 8|0);
  $12 = (($11) + (($9*72)|0)|0);
  $13 = (($12) + 16|0);
  $14 = HEAP32[$13>>2]|0;
  $15 = HEAP32[$3>>2]|0;
  $16 = (_columnIndex($14,$15)|0);
  HEAP32[$iCol>>2] = $16;
  $17 = HEAP32[$iCol>>2]|0;
  $18 = ($17|0)>=(0);
  if ($18) {
   break;
  }
  $25 = HEAP32[$i>>2]|0;
  $26 = (($25) + 1)|0;
  HEAP32[$i>>2] = $26;
 }
 if ((label|0) == 8) {
  HEAP32[$0>>2] = 0;
  $27 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($27|0);
 }
 $19 = HEAP32[$4>>2]|0;
 $20 = ($19|0)!=(0|0);
 if ($20) {
  $21 = HEAP32[$i>>2]|0;
  $22 = HEAP32[$4>>2]|0;
  HEAP32[$22>>2] = $21;
  $23 = HEAP32[$iCol>>2]|0;
  $24 = HEAP32[$5>>2]|0;
  HEAP32[$24>>2] = $23;
 }
 HEAP32[$0>>2] = 1;
 $27 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($27|0);
}
function _sqlite3IdListIndex($pList,$zName) {
 $pList = $pList|0;
 $zName = $zName|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0;
 var $9 = 0, $i = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $1 = sp + 8|0;
 $2 = sp + 4|0;
 $i = sp;
 HEAP32[$1>>2] = $pList;
 HEAP32[$2>>2] = $zName;
 $3 = HEAP32[$1>>2]|0;
 $4 = ($3|0)==(0|0);
 if ($4) {
  HEAP32[$0>>2] = -1;
  $20 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($20|0);
 }
 HEAP32[$i>>2] = 0;
 while(1) {
  $5 = HEAP32[$i>>2]|0;
  $6 = HEAP32[$1>>2]|0;
  $7 = (($6) + 4|0);
  $8 = HEAP32[$7>>2]|0;
  $9 = ($5|0)<($8|0);
  if (!($9)) {
   label = 8;
   break;
  }
  $10 = HEAP32[$i>>2]|0;
  $11 = HEAP32[$1>>2]|0;
  $12 = HEAP32[$11>>2]|0;
  $13 = (($12) + ($10<<3)|0);
  $14 = HEAP32[$13>>2]|0;
  $15 = HEAP32[$2>>2]|0;
  $16 = (_sqlite3_stricmp($14,$15)|0);
  $17 = ($16|0)==(0);
  $18 = HEAP32[$i>>2]|0;
  if ($17) {
   label = 6;
   break;
  }
  $19 = (($18) + 1)|0;
  HEAP32[$i>>2] = $19;
 }
 if ((label|0) == 6) {
  HEAP32[$0>>2] = $18;
  $20 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($20|0);
 }
 else if ((label|0) == 8) {
  HEAP32[$0>>2] = -1;
  $20 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($20|0);
 }
 return 0|0;
}
function _columnIndex($pTab,$zCol) {
 $pTab = $pTab|0;
 $zCol = $zCol|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0;
 var $9 = 0, $i = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $1 = sp + 8|0;
 $2 = sp + 4|0;
 $i = sp;
 HEAP32[$1>>2] = $pTab;
 HEAP32[$2>>2] = $zCol;
 HEAP32[$i>>2] = 0;
 while(1) {
  $3 = HEAP32[$i>>2]|0;
  $4 = HEAP32[$1>>2]|0;
  $5 = (($4) + 38|0);
  $6 = HEAP16[$5>>1]|0;
  $7 = $6 << 16 >> 16;
  $8 = ($3|0)<($7|0);
  if (!($8)) {
   label = 6;
   break;
  }
  $9 = HEAP32[$i>>2]|0;
  $10 = HEAP32[$1>>2]|0;
  $11 = (($10) + 4|0);
  $12 = HEAP32[$11>>2]|0;
  $13 = (($12) + (($9*24)|0)|0);
  $14 = HEAP32[$13>>2]|0;
  $15 = HEAP32[$2>>2]|0;
  $16 = (_sqlite3_stricmp($14,$15)|0);
  $17 = ($16|0)==(0);
  $18 = HEAP32[$i>>2]|0;
  if ($17) {
   label = 4;
   break;
  }
  $19 = (($18) + 1)|0;
  HEAP32[$i>>2] = $19;
 }
 if ((label|0) == 4) {
  HEAP32[$0>>2] = $18;
  $20 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($20|0);
 }
 else if ((label|0) == 6) {
  HEAP32[$0>>2] = -1;
  $20 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($20|0);
 }
 return 0|0;
}
function _addWhereTerm($pParse,$pSrc,$iLeft,$iColLeft,$iRight,$iColRight,$isOuterJoin,$ppWhere) {
 $pParse = $pParse|0;
 $pSrc = $pSrc|0;
 $iLeft = $iLeft|0;
 $iColLeft = $iColLeft|0;
 $iRight = $iRight|0;
 $iColRight = $iColRight|0;
 $isOuterJoin = $isOuterJoin|0;
 $ppWhere = $ppWhere|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $5 = 0;
 var $6 = 0, $7 = 0, $8 = 0, $9 = 0, $db = 0, $or$cond = 0, $pE1 = 0, $pE2 = 0, $pEq = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $0 = sp + 44|0;
 $1 = sp + 40|0;
 $2 = sp + 36|0;
 $3 = sp + 32|0;
 $4 = sp + 28|0;
 $5 = sp + 24|0;
 $6 = sp + 20|0;
 $7 = sp + 16|0;
 $db = sp + 12|0;
 $pE1 = sp + 8|0;
 $pE2 = sp + 4|0;
 $pEq = sp;
 HEAP32[$0>>2] = $pParse;
 HEAP32[$1>>2] = $pSrc;
 HEAP32[$2>>2] = $iLeft;
 HEAP32[$3>>2] = $iColLeft;
 HEAP32[$4>>2] = $iRight;
 HEAP32[$5>>2] = $iColRight;
 HEAP32[$6>>2] = $isOuterJoin;
 HEAP32[$7>>2] = $ppWhere;
 $8 = HEAP32[$0>>2]|0;
 $9 = HEAP32[$8>>2]|0;
 HEAP32[$db>>2] = $9;
 $10 = HEAP32[$db>>2]|0;
 $11 = HEAP32[$1>>2]|0;
 $12 = HEAP32[$2>>2]|0;
 $13 = HEAP32[$3>>2]|0;
 $14 = (_sqlite3CreateColumnExpr($10,$11,$12,$13)|0);
 HEAP32[$pE1>>2] = $14;
 $15 = HEAP32[$db>>2]|0;
 $16 = HEAP32[$1>>2]|0;
 $17 = HEAP32[$4>>2]|0;
 $18 = HEAP32[$5>>2]|0;
 $19 = (_sqlite3CreateColumnExpr($15,$16,$17,$18)|0);
 HEAP32[$pE2>>2] = $19;
 $20 = HEAP32[$0>>2]|0;
 $21 = HEAP32[$pE1>>2]|0;
 $22 = HEAP32[$pE2>>2]|0;
 $23 = (_sqlite3PExpr($20,79,$21,$22,0)|0);
 HEAP32[$pEq>>2] = $23;
 $24 = HEAP32[$pEq>>2]|0;
 $25 = ($24|0)!=(0|0);
 $26 = HEAP32[$6>>2]|0;
 $27 = ($26|0)!=(0);
 $or$cond = $25 & $27;
 if (!($or$cond)) {
  $38 = HEAP32[$db>>2]|0;
  $39 = HEAP32[$7>>2]|0;
  $40 = HEAP32[$39>>2]|0;
  $41 = HEAP32[$pEq>>2]|0;
  $42 = (_sqlite3ExprAnd($38,$40,$41)|0);
  $43 = HEAP32[$7>>2]|0;
  HEAP32[$43>>2] = $42;
  STACKTOP = sp;return;
 }
 $28 = HEAP32[$pEq>>2]|0;
 $29 = (($28) + 4|0);
 $30 = HEAP32[$29>>2]|0;
 $31 = $30 | 1;
 HEAP32[$29>>2] = $31;
 $32 = HEAP32[$pE2>>2]|0;
 $33 = (($32) + 28|0);
 $34 = HEAP32[$33>>2]|0;
 $35 = $34&65535;
 $36 = HEAP32[$pEq>>2]|0;
 $37 = (($36) + 36|0);
 HEAP16[$37>>1] = $35;
 $38 = HEAP32[$db>>2]|0;
 $39 = HEAP32[$7>>2]|0;
 $40 = HEAP32[$39>>2]|0;
 $41 = HEAP32[$pEq>>2]|0;
 $42 = (_sqlite3ExprAnd($38,$40,$41)|0);
 $43 = HEAP32[$7>>2]|0;
 HEAP32[$43>>2] = $42;
 STACKTOP = sp;return;
}
function _setJoinExpr($p,$iTable) {
 $p = $p|0;
 $iTable = $iTable|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0;
 var sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $1 = sp;
 HEAP32[$0>>2] = $p;
 HEAP32[$1>>2] = $iTable;
 while(1) {
  $2 = HEAP32[$0>>2]|0;
  $3 = ($2|0)!=(0|0);
  if (!($3)) {
   break;
  }
  $4 = HEAP32[$0>>2]|0;
  $5 = (($4) + 4|0);
  $6 = HEAP32[$5>>2]|0;
  $7 = $6 | 1;
  HEAP32[$5>>2] = $7;
  $8 = HEAP32[$1>>2]|0;
  $9 = $8&65535;
  $10 = HEAP32[$0>>2]|0;
  $11 = (($10) + 36|0);
  HEAP16[$11>>1] = $9;
  $12 = HEAP32[$0>>2]|0;
  $13 = (($12) + 12|0);
  $14 = HEAP32[$13>>2]|0;
  $15 = HEAP32[$1>>2]|0;
  _setJoinExpr($14,$15);
  $16 = HEAP32[$0>>2]|0;
  $17 = (($16) + 16|0);
  $18 = HEAP32[$17>>2]|0;
  HEAP32[$0>>2] = $18;
 }
 STACKTOP = sp;return;
}
function _sqlite3ExprAnd($db,$pLeft,$pRight) {
 $db = $db|0;
 $pLeft = $pLeft|0;
 $pRight = $pRight|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $pNew = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 16|0;
 $1 = sp + 12|0;
 $2 = sp + 8|0;
 $3 = sp + 4|0;
 $pNew = sp;
 HEAP32[$1>>2] = $db;
 HEAP32[$2>>2] = $pLeft;
 HEAP32[$3>>2] = $pRight;
 $4 = HEAP32[$2>>2]|0;
 $5 = ($4|0)==(0|0);
 $6 = HEAP32[$3>>2]|0;
 if ($5) {
  HEAP32[$0>>2] = $6;
  $27 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($27|0);
 }
 $7 = ($6|0)==(0|0);
 $8 = HEAP32[$2>>2]|0;
 if ($7) {
  HEAP32[$0>>2] = $8;
  $27 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($27|0);
 }
 $9 = (_exprAlwaysFalse($8)|0);
 $10 = ($9|0)!=(0);
 if (!($10)) {
  $11 = HEAP32[$3>>2]|0;
  $12 = (_exprAlwaysFalse($11)|0);
  $13 = ($12|0)!=(0);
  if (!($13)) {
   $20 = HEAP32[$1>>2]|0;
   $21 = (_sqlite3ExprAlloc($20,72,0,0)|0);
   HEAP32[$pNew>>2] = $21;
   $22 = HEAP32[$1>>2]|0;
   $23 = HEAP32[$pNew>>2]|0;
   $24 = HEAP32[$2>>2]|0;
   $25 = HEAP32[$3>>2]|0;
   _sqlite3ExprAttachSubtrees($22,$23,$24,$25);
   $26 = HEAP32[$pNew>>2]|0;
   HEAP32[$0>>2] = $26;
   $27 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($27|0);
  }
 }
 $14 = HEAP32[$1>>2]|0;
 $15 = HEAP32[$2>>2]|0;
 _sqlite3ExprDelete($14,$15);
 $16 = HEAP32[$1>>2]|0;
 $17 = HEAP32[$3>>2]|0;
 _sqlite3ExprDelete($16,$17);
 $18 = HEAP32[$1>>2]|0;
 $19 = (_sqlite3ExprAlloc($18,132,10152,0)|0);
 HEAP32[$0>>2] = $19;
 $27 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($27|0);
}
function _exprAlwaysFalse($p) {
 $p = $p|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $v = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $1 = sp + 4|0;
 $v = sp;
 HEAP32[$1>>2] = $p;
 HEAP32[$v>>2] = 0;
 $2 = HEAP32[$1>>2]|0;
 $3 = (($2) + 4|0);
 $4 = HEAP32[$3>>2]|0;
 $5 = $4 & 1;
 $6 = ($5|0)!=(0);
 do {
  if ($6) {
   HEAP32[$0>>2] = 0;
  } else {
   $7 = HEAP32[$1>>2]|0;
   $8 = (_sqlite3ExprIsInteger($7,$v)|0);
   $9 = ($8|0)!=(0);
   if ($9) {
    $10 = HEAP32[$v>>2]|0;
    $11 = ($10|0)==(0);
    $12 = $11&1;
    HEAP32[$0>>2] = $12;
    break;
   } else {
    HEAP32[$0>>2] = 0;
    break;
   }
  }
 } while(0);
 $13 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($13|0);
}
function _sqlite3ExprAlloc($db,$op,$pToken,$dequote) {
 $db = $db|0;
 $op = $op|0;
 $pToken = $pToken|0;
 $dequote = $dequote|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0;
 var $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $c = 0, $iValue = 0, $nExtra = 0, $or$cond = 0, $or$cond3 = 0, $or$cond5 = 0, $or$cond7 = 0, $pNew = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 28|0;
 $1 = sp + 24|0;
 $2 = sp + 20|0;
 $3 = sp + 16|0;
 $pNew = sp + 12|0;
 $nExtra = sp + 8|0;
 $iValue = sp + 4|0;
 $c = sp;
 HEAP32[$0>>2] = $db;
 HEAP32[$1>>2] = $op;
 HEAP32[$2>>2] = $pToken;
 HEAP32[$3>>2] = $dequote;
 HEAP32[$nExtra>>2] = 0;
 HEAP32[$iValue>>2] = 0;
 $4 = HEAP32[$2>>2]|0;
 $5 = ($4|0)!=(0|0);
 do {
  if ($5) {
   $6 = HEAP32[$1>>2]|0;
   $7 = ($6|0)!=(132);
   if (!($7)) {
    $8 = HEAP32[$2>>2]|0;
    $9 = HEAP32[$8>>2]|0;
    $10 = ($9|0)==(0|0);
    if (!($10)) {
     $11 = HEAP32[$2>>2]|0;
     $12 = HEAP32[$11>>2]|0;
     $13 = (_sqlite3GetInt32($12,$iValue)|0);
     $14 = ($13|0)==(0);
     if (!($14)) {
      break;
     }
    }
   }
   $15 = HEAP32[$2>>2]|0;
   $16 = (($15) + 4|0);
   $17 = HEAP32[$16>>2]|0;
   $18 = (($17) + 1)|0;
   HEAP32[$nExtra>>2] = $18;
  }
 } while(0);
 $19 = HEAP32[$0>>2]|0;
 $20 = HEAP32[$nExtra>>2]|0;
 $21 = (48 + ($20))|0;
 $22 = (_sqlite3DbMallocZero($19,$21,0)|0);
 HEAP32[$pNew>>2] = $22;
 $23 = HEAP32[$pNew>>2]|0;
 $24 = ($23|0)!=(0|0);
 if (!($24)) {
  $89 = HEAP32[$pNew>>2]|0;
  STACKTOP = sp;return ($89|0);
 }
 $25 = HEAP32[$1>>2]|0;
 $26 = $25&255;
 $27 = HEAP32[$pNew>>2]|0;
 HEAP8[$27>>0] = $26;
 $28 = HEAP32[$pNew>>2]|0;
 $29 = (($28) + 34|0);
 HEAP16[$29>>1] = -1;
 $30 = HEAP32[$2>>2]|0;
 $31 = ($30|0)!=(0|0);
 do {
  if ($31) {
   $32 = HEAP32[$nExtra>>2]|0;
   $33 = ($32|0)==(0);
   $34 = HEAP32[$pNew>>2]|0;
   if ($33) {
    $35 = (($34) + 4|0);
    $36 = HEAP32[$35>>2]|0;
    $37 = $36 | 1024;
    HEAP32[$35>>2] = $37;
    $38 = HEAP32[$iValue>>2]|0;
    $39 = HEAP32[$pNew>>2]|0;
    $40 = (($39) + 8|0);
    HEAP32[$40>>2] = $38;
    break;
   }
   $41 = (($34) + 48|0);
   $42 = HEAP32[$pNew>>2]|0;
   $43 = (($42) + 8|0);
   HEAP32[$43>>2] = $41;
   $44 = HEAP32[$2>>2]|0;
   $45 = (($44) + 4|0);
   $46 = HEAP32[$45>>2]|0;
   $47 = ($46|0)!=(0);
   if ($47) {
    $48 = HEAP32[$pNew>>2]|0;
    $49 = (($48) + 8|0);
    $50 = HEAP32[$49>>2]|0;
    $51 = HEAP32[$2>>2]|0;
    $52 = HEAP32[$51>>2]|0;
    $53 = HEAP32[$2>>2]|0;
    $54 = (($53) + 4|0);
    $55 = HEAP32[$54>>2]|0;
    _memcpy(($50|0),($52|0),($55|0))|0;
   }
   $56 = HEAP32[$2>>2]|0;
   $57 = (($56) + 4|0);
   $58 = HEAP32[$57>>2]|0;
   $59 = HEAP32[$pNew>>2]|0;
   $60 = (($59) + 8|0);
   $61 = HEAP32[$60>>2]|0;
   $62 = (($61) + ($58)|0);
   HEAP8[$62>>0] = 0;
   $63 = HEAP32[$3>>2]|0;
   $64 = ($63|0)!=(0);
   $65 = HEAP32[$nExtra>>2]|0;
   $66 = ($65|0)>=(3);
   $or$cond = $64 & $66;
   if ($or$cond) {
    $67 = HEAP32[$2>>2]|0;
    $68 = HEAP32[$67>>2]|0;
    $69 = HEAP8[$68>>0]|0;
    $70 = $69 << 24 >> 24;
    HEAP32[$c>>2] = $70;
    $71 = ($70|0)==(39);
    $72 = HEAP32[$c>>2]|0;
    $73 = ($72|0)==(34);
    $or$cond3 = $71 | $73;
    $74 = HEAP32[$c>>2]|0;
    $75 = ($74|0)==(91);
    $or$cond5 = $or$cond3 | $75;
    $76 = HEAP32[$c>>2]|0;
    $77 = ($76|0)==(96);
    $or$cond7 = $or$cond5 | $77;
    if ($or$cond7) {
     $78 = HEAP32[$pNew>>2]|0;
     $79 = (($78) + 8|0);
     $80 = HEAP32[$79>>2]|0;
     (_sqlite3Dequote($80)|0);
     $81 = HEAP32[$c>>2]|0;
     $82 = ($81|0)==(34);
     if ($82) {
      $83 = HEAP32[$pNew>>2]|0;
      $84 = (($83) + 4|0);
      $85 = HEAP32[$84>>2]|0;
      $86 = $85 | 64;
      HEAP32[$84>>2] = $86;
     }
    }
   }
  }
 } while(0);
 $87 = HEAP32[$pNew>>2]|0;
 $88 = (($87) + 24|0);
 HEAP32[$88>>2] = 1;
 $89 = HEAP32[$pNew>>2]|0;
 STACKTOP = sp;return ($89|0);
}
function _sqlite3ExprAttachSubtrees($db,$pRoot,$pLeft,$pRight) {
 $db = $db|0;
 $pRoot = $pRoot|0;
 $pLeft = $pLeft|0;
 $pRight = $pRight|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $1 = sp + 8|0;
 $2 = sp + 4|0;
 $3 = sp;
 HEAP32[$0>>2] = $db;
 HEAP32[$1>>2] = $pRoot;
 HEAP32[$2>>2] = $pLeft;
 HEAP32[$3>>2] = $pRight;
 $4 = HEAP32[$1>>2]|0;
 $5 = ($4|0)==(0|0);
 if ($5) {
  $6 = HEAP32[$0>>2]|0;
  $7 = HEAP32[$2>>2]|0;
  _sqlite3ExprDelete($6,$7);
  $8 = HEAP32[$0>>2]|0;
  $9 = HEAP32[$3>>2]|0;
  _sqlite3ExprDelete($8,$9);
  STACKTOP = sp;return;
 }
 $10 = HEAP32[$3>>2]|0;
 $11 = ($10|0)!=(0|0);
 if ($11) {
  $12 = HEAP32[$3>>2]|0;
  $13 = HEAP32[$1>>2]|0;
  $14 = (($13) + 16|0);
  HEAP32[$14>>2] = $12;
  $15 = HEAP32[$3>>2]|0;
  $16 = (($15) + 4|0);
  $17 = HEAP32[$16>>2]|0;
  $18 = 256 & $17;
  $19 = HEAP32[$1>>2]|0;
  $20 = (($19) + 4|0);
  $21 = HEAP32[$20>>2]|0;
  $22 = $21 | $18;
  HEAP32[$20>>2] = $22;
 }
 $23 = HEAP32[$2>>2]|0;
 $24 = ($23|0)!=(0|0);
 if ($24) {
  $25 = HEAP32[$2>>2]|0;
  $26 = HEAP32[$1>>2]|0;
  $27 = (($26) + 12|0);
  HEAP32[$27>>2] = $25;
  $28 = HEAP32[$2>>2]|0;
  $29 = (($28) + 4|0);
  $30 = HEAP32[$29>>2]|0;
  $31 = 256 & $30;
  $32 = HEAP32[$1>>2]|0;
  $33 = (($32) + 4|0);
  $34 = HEAP32[$33>>2]|0;
  $35 = $34 | $31;
  HEAP32[$33>>2] = $35;
 }
 $36 = HEAP32[$1>>2]|0;
 _exprSetHeight($36);
 STACKTOP = sp;return;
}
function _exprSetHeight($p) {
 $p = $p|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0;
 var $nHeight = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $nHeight = sp;
 HEAP32[$0>>2] = $p;
 HEAP32[$nHeight>>2] = 0;
 $1 = HEAP32[$0>>2]|0;
 $2 = (($1) + 12|0);
 $3 = HEAP32[$2>>2]|0;
 _heightOfExpr($3,$nHeight);
 $4 = HEAP32[$0>>2]|0;
 $5 = (($4) + 16|0);
 $6 = HEAP32[$5>>2]|0;
 _heightOfExpr($6,$nHeight);
 $7 = HEAP32[$0>>2]|0;
 $8 = (($7) + 4|0);
 $9 = HEAP32[$8>>2]|0;
 $10 = $9 & 2048;
 $11 = ($10|0)!=(0);
 $12 = HEAP32[$0>>2]|0;
 $13 = (($12) + 20|0);
 if ($11) {
  $14 = HEAP32[$13>>2]|0;
  _heightOfSelect($14,$nHeight);
 } else {
  $15 = HEAP32[$13>>2]|0;
  _heightOfExprList($15,$nHeight);
 }
 $16 = HEAP32[$nHeight>>2]|0;
 $17 = (($16) + 1)|0;
 $18 = HEAP32[$0>>2]|0;
 $19 = (($18) + 24|0);
 HEAP32[$19>>2] = $17;
 STACKTOP = sp;return;
}
function _heightOfExpr($p,$pnHeight) {
 $p = $p|0;
 $pnHeight = $pnHeight|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $1 = sp;
 HEAP32[$0>>2] = $p;
 HEAP32[$1>>2] = $pnHeight;
 $2 = HEAP32[$0>>2]|0;
 $3 = ($2|0)!=(0|0);
 if (!($3)) {
  STACKTOP = sp;return;
 }
 $4 = HEAP32[$0>>2]|0;
 $5 = (($4) + 24|0);
 $6 = HEAP32[$5>>2]|0;
 $7 = HEAP32[$1>>2]|0;
 $8 = HEAP32[$7>>2]|0;
 $9 = ($6|0)>($8|0);
 if (!($9)) {
  STACKTOP = sp;return;
 }
 $10 = HEAP32[$0>>2]|0;
 $11 = (($10) + 24|0);
 $12 = HEAP32[$11>>2]|0;
 $13 = HEAP32[$1>>2]|0;
 HEAP32[$13>>2] = $12;
 STACKTOP = sp;return;
}
function _heightOfSelect($p,$pnHeight) {
 $p = $p|0;
 $pnHeight = $pnHeight|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $1 = sp;
 HEAP32[$0>>2] = $p;
 HEAP32[$1>>2] = $pnHeight;
 $2 = HEAP32[$0>>2]|0;
 $3 = ($2|0)!=(0|0);
 if (!($3)) {
  STACKTOP = sp;return;
 }
 $4 = HEAP32[$0>>2]|0;
 $5 = (($4) + 36|0);
 $6 = HEAP32[$5>>2]|0;
 $7 = HEAP32[$1>>2]|0;
 _heightOfExpr($6,$7);
 $8 = HEAP32[$0>>2]|0;
 $9 = (($8) + 44|0);
 $10 = HEAP32[$9>>2]|0;
 $11 = HEAP32[$1>>2]|0;
 _heightOfExpr($10,$11);
 $12 = HEAP32[$0>>2]|0;
 $13 = (($12) + 60|0);
 $14 = HEAP32[$13>>2]|0;
 $15 = HEAP32[$1>>2]|0;
 _heightOfExpr($14,$15);
 $16 = HEAP32[$0>>2]|0;
 $17 = (($16) + 64|0);
 $18 = HEAP32[$17>>2]|0;
 $19 = HEAP32[$1>>2]|0;
 _heightOfExpr($18,$19);
 $20 = HEAP32[$0>>2]|0;
 $21 = HEAP32[$20>>2]|0;
 $22 = HEAP32[$1>>2]|0;
 _heightOfExprList($21,$22);
 $23 = HEAP32[$0>>2]|0;
 $24 = (($23) + 40|0);
 $25 = HEAP32[$24>>2]|0;
 $26 = HEAP32[$1>>2]|0;
 _heightOfExprList($25,$26);
 $27 = HEAP32[$0>>2]|0;
 $28 = (($27) + 48|0);
 $29 = HEAP32[$28>>2]|0;
 $30 = HEAP32[$1>>2]|0;
 _heightOfExprList($29,$30);
 $31 = HEAP32[$0>>2]|0;
 $32 = (($31) + 52|0);
 $33 = HEAP32[$32>>2]|0;
 $34 = HEAP32[$1>>2]|0;
 _heightOfSelect($33,$34);
 STACKTOP = sp;return;
}
function _heightOfExprList($p,$pnHeight) {
 $p = $p|0;
 $pnHeight = $pnHeight|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $i = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $1 = sp + 4|0;
 $i = sp;
 HEAP32[$0>>2] = $p;
 HEAP32[$1>>2] = $pnHeight;
 $2 = HEAP32[$0>>2]|0;
 $3 = ($2|0)!=(0|0);
 if (!($3)) {
  STACKTOP = sp;return;
 }
 HEAP32[$i>>2] = 0;
 while(1) {
  $4 = HEAP32[$i>>2]|0;
  $5 = HEAP32[$0>>2]|0;
  $6 = HEAP32[$5>>2]|0;
  $7 = ($4|0)<($6|0);
  if (!($7)) {
   break;
  }
  $8 = HEAP32[$i>>2]|0;
  $9 = HEAP32[$0>>2]|0;
  $10 = (($9) + 4|0);
  $11 = HEAP32[$10>>2]|0;
  $12 = (($11) + (($8*20)|0)|0);
  $13 = HEAP32[$12>>2]|0;
  $14 = HEAP32[$1>>2]|0;
  _heightOfExpr($13,$14);
  $15 = HEAP32[$i>>2]|0;
  $16 = (($15) + 1)|0;
  HEAP32[$i>>2] = $16;
 }
 STACKTOP = sp;return;
}
function _sqlite3CreateColumnExpr($db,$pSrc,$iSrc,$iCol) {
 $db = $db|0;
 $pSrc = $pSrc|0;
 $iSrc = $iSrc|0;
 $iCol = $iCol|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $7 = 0, $8 = 0;
 var $9 = 0, $p = 0, $pItem = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 20|0;
 $1 = sp + 16|0;
 $2 = sp + 12|0;
 $3 = sp + 8|0;
 $p = sp + 4|0;
 $pItem = sp;
 HEAP32[$0>>2] = $db;
 HEAP32[$1>>2] = $pSrc;
 HEAP32[$2>>2] = $iSrc;
 HEAP32[$3>>2] = $iCol;
 $4 = HEAP32[$0>>2]|0;
 $5 = (_sqlite3ExprAlloc($4,154,0,0)|0);
 HEAP32[$p>>2] = $5;
 $6 = HEAP32[$p>>2]|0;
 $7 = ($6|0)!=(0|0);
 if (!($7)) {
  $60 = HEAP32[$p>>2]|0;
  STACKTOP = sp;return ($60|0);
 }
 $8 = HEAP32[$2>>2]|0;
 $9 = HEAP32[$1>>2]|0;
 $10 = (($9) + 8|0);
 $11 = (($10) + (($8*72)|0)|0);
 HEAP32[$pItem>>2] = $11;
 $12 = HEAP32[$pItem>>2]|0;
 $13 = (($12) + 16|0);
 $14 = HEAP32[$13>>2]|0;
 $15 = HEAP32[$p>>2]|0;
 $16 = (($15) + 44|0);
 HEAP32[$16>>2] = $14;
 $17 = HEAP32[$pItem>>2]|0;
 $18 = (($17) + 40|0);
 $19 = HEAP32[$18>>2]|0;
 $20 = HEAP32[$p>>2]|0;
 $21 = (($20) + 28|0);
 HEAP32[$21>>2] = $19;
 $22 = HEAP32[$p>>2]|0;
 $23 = (($22) + 44|0);
 $24 = HEAP32[$23>>2]|0;
 $25 = (($24) + 36|0);
 $26 = HEAP16[$25>>1]|0;
 $27 = $26 << 16 >> 16;
 $28 = HEAP32[$3>>2]|0;
 $29 = ($27|0)==($28|0);
 if ($29) {
  $30 = HEAP32[$p>>2]|0;
  $31 = (($30) + 32|0);
  HEAP16[$31>>1] = -1;
 } else {
  $32 = HEAP32[$3>>2]|0;
  $33 = $32&65535;
  $34 = HEAP32[$p>>2]|0;
  $35 = (($34) + 32|0);
  HEAP16[$35>>1] = $33;
  $36 = HEAP32[$3>>2]|0;
  $37 = ($36|0)>=(64);
  $38 = HEAP32[$3>>2]|0;
  $39 = $37 ? 63 : $38;
  $40 = (_bitshift64Shl(1,0,($39|0))|0);
  $41 = tempRet0;
  $42 = HEAP32[$pItem>>2]|0;
  $43 = (($42) + 56|0);
  $44 = $43;
  $45 = $44;
  $46 = HEAP32[$45>>2]|0;
  $47 = (($44) + 4)|0;
  $48 = $47;
  $49 = HEAP32[$48>>2]|0;
  $50 = $46 | $40;
  $51 = $49 | $41;
  $52 = $43;
  $53 = $52;
  HEAP32[$53>>2] = $50;
  $54 = (($52) + 4)|0;
  $55 = $54;
  HEAP32[$55>>2] = $51;
 }
 $56 = HEAP32[$p>>2]|0;
 $57 = (($56) + 4|0);
 $58 = HEAP32[$57>>2]|0;
 $59 = $58 | 4;
 HEAP32[$57>>2] = $59;
 $60 = HEAP32[$p>>2]|0;
 STACKTOP = sp;return ($60|0);
}
function _searchWith($pWith,$pItem,$ppContext) {
 $pWith = $pWith|0;
 $pItem = $pItem|0;
 $ppContext = $ppContext|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $i = 0;
 var $p = 0, $zName = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 24|0;
 $1 = sp + 20|0;
 $2 = sp + 16|0;
 $3 = sp + 12|0;
 $zName = sp + 8|0;
 $p = sp + 4|0;
 $i = sp;
 HEAP32[$1>>2] = $pWith;
 HEAP32[$2>>2] = $pItem;
 HEAP32[$3>>2] = $ppContext;
 $4 = HEAP32[$2>>2]|0;
 $5 = (($4) + 4|0);
 $6 = HEAP32[$5>>2]|0;
 $7 = ($6|0)==(0|0);
 L1: do {
  if ($7) {
   $8 = HEAP32[$2>>2]|0;
   $9 = (($8) + 8|0);
   $10 = HEAP32[$9>>2]|0;
   HEAP32[$zName>>2] = $10;
   $11 = ($10|0)!=(0|0);
   if ($11) {
    $12 = HEAP32[$1>>2]|0;
    HEAP32[$p>>2] = $12;
    L4: while(1) {
     $13 = HEAP32[$p>>2]|0;
     $14 = ($13|0)!=(0|0);
     if (!($14)) {
      break L1;
     }
     HEAP32[$i>>2] = 0;
     while(1) {
      $15 = HEAP32[$i>>2]|0;
      $16 = HEAP32[$p>>2]|0;
      $17 = HEAP32[$16>>2]|0;
      $18 = ($15|0)<($17|0);
      if (!($18)) {
       break;
      }
      $19 = HEAP32[$zName>>2]|0;
      $20 = HEAP32[$i>>2]|0;
      $21 = HEAP32[$p>>2]|0;
      $22 = (($21) + 8|0);
      $23 = (($22) + ($20<<4)|0);
      $24 = HEAP32[$23>>2]|0;
      $25 = (_sqlite3_stricmp($19,$24)|0);
      $26 = ($25|0)==(0);
      if ($26) {
       break L4;
      }
      $33 = HEAP32[$i>>2]|0;
      $34 = (($33) + 1)|0;
      HEAP32[$i>>2] = $34;
     }
     $35 = HEAP32[$p>>2]|0;
     $36 = (($35) + 4|0);
     $37 = HEAP32[$36>>2]|0;
     HEAP32[$p>>2] = $37;
    }
    $27 = HEAP32[$p>>2]|0;
    $28 = HEAP32[$3>>2]|0;
    HEAP32[$28>>2] = $27;
    $29 = HEAP32[$i>>2]|0;
    $30 = HEAP32[$p>>2]|0;
    $31 = (($30) + 8|0);
    $32 = (($31) + ($29<<4)|0);
    HEAP32[$0>>2] = $32;
    $38 = HEAP32[$0>>2]|0;
    STACKTOP = sp;return ($38|0);
   }
  }
 } while(0);
 HEAP32[$0>>2] = 0;
 $38 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($38|0);
}
function _sqlite3SrcListDup($db,$p,$flags) {
 $db = $db|0;
 $p = $p|0;
 $flags = $flags|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0;
 var $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0, $188 = 0;
 var $189 = 0, $19 = 0, $190 = 0, $191 = 0, $192 = 0, $193 = 0, $194 = 0, $195 = 0, $196 = 0, $197 = 0, $198 = 0, $199 = 0, $2 = 0, $20 = 0, $200 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0;
 var $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0;
 var $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0;
 var $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0;
 var $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0;
 var $99 = 0, $i = 0, $nByte = 0, $pNew = 0, $pNewItem = 0, $pOldItem = 0, $pTab = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $0 = sp + 36|0;
 $1 = sp + 32|0;
 $2 = sp + 28|0;
 $3 = sp + 24|0;
 $pNew = sp + 20|0;
 $i = sp + 16|0;
 $nByte = sp + 12|0;
 $pNewItem = sp + 8|0;
 $pOldItem = sp + 4|0;
 $pTab = sp;
 HEAP32[$1>>2] = $db;
 HEAP32[$2>>2] = $p;
 HEAP32[$3>>2] = $flags;
 $4 = HEAP32[$2>>2]|0;
 $5 = ($4|0)==(0|0);
 if ($5) {
  HEAP32[$0>>2] = 0;
  $200 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($200|0);
 }
 $6 = HEAP32[$2>>2]|0;
 $7 = HEAP32[$6>>2]|0;
 $8 = ($7|0)>(0);
 if ($8) {
  $9 = HEAP32[$2>>2]|0;
  $10 = HEAP32[$9>>2]|0;
  $11 = (($10) - 1)|0;
  $12 = ($11*72)|0;
  $14 = $12;
 } else {
  $14 = 0;
 }
 $13 = (80 + ($14))|0;
 HEAP32[$nByte>>2] = $13;
 $15 = HEAP32[$1>>2]|0;
 $16 = HEAP32[$nByte>>2]|0;
 $17 = ($16|0)<(0);
 $18 = $17 << 31 >> 31;
 $19 = (_sqlite3DbMallocRaw($15,$16,$18)|0);
 HEAP32[$pNew>>2] = $19;
 $20 = HEAP32[$pNew>>2]|0;
 $21 = ($20|0)==(0|0);
 if ($21) {
  HEAP32[$0>>2] = 0;
  $200 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($200|0);
 }
 $22 = HEAP32[$2>>2]|0;
 $23 = HEAP32[$22>>2]|0;
 $24 = HEAP32[$pNew>>2]|0;
 $25 = (($24) + 4|0);
 HEAP32[$25>>2] = $23;
 $26 = HEAP32[$pNew>>2]|0;
 HEAP32[$26>>2] = $23;
 HEAP32[$i>>2] = 0;
 while(1) {
  $27 = HEAP32[$i>>2]|0;
  $28 = HEAP32[$2>>2]|0;
  $29 = HEAP32[$28>>2]|0;
  $30 = ($27|0)<($29|0);
  if (!($30)) {
   break;
  }
  $31 = HEAP32[$i>>2]|0;
  $32 = HEAP32[$pNew>>2]|0;
  $33 = (($32) + 8|0);
  $34 = (($33) + (($31*72)|0)|0);
  HEAP32[$pNewItem>>2] = $34;
  $35 = HEAP32[$i>>2]|0;
  $36 = HEAP32[$2>>2]|0;
  $37 = (($36) + 8|0);
  $38 = (($37) + (($35*72)|0)|0);
  HEAP32[$pOldItem>>2] = $38;
  $39 = HEAP32[$pOldItem>>2]|0;
  $40 = HEAP32[$39>>2]|0;
  $41 = HEAP32[$pNewItem>>2]|0;
  HEAP32[$41>>2] = $40;
  $42 = HEAP32[$1>>2]|0;
  $43 = HEAP32[$pOldItem>>2]|0;
  $44 = (($43) + 4|0);
  $45 = HEAP32[$44>>2]|0;
  $46 = (_sqlite3DbStrDup($42,$45)|0);
  $47 = HEAP32[$pNewItem>>2]|0;
  $48 = (($47) + 4|0);
  HEAP32[$48>>2] = $46;
  $49 = HEAP32[$1>>2]|0;
  $50 = HEAP32[$pOldItem>>2]|0;
  $51 = (($50) + 8|0);
  $52 = HEAP32[$51>>2]|0;
  $53 = (_sqlite3DbStrDup($49,$52)|0);
  $54 = HEAP32[$pNewItem>>2]|0;
  $55 = (($54) + 8|0);
  HEAP32[$55>>2] = $53;
  $56 = HEAP32[$1>>2]|0;
  $57 = HEAP32[$pOldItem>>2]|0;
  $58 = (($57) + 12|0);
  $59 = HEAP32[$58>>2]|0;
  $60 = (_sqlite3DbStrDup($56,$59)|0);
  $61 = HEAP32[$pNewItem>>2]|0;
  $62 = (($61) + 12|0);
  HEAP32[$62>>2] = $60;
  $63 = HEAP32[$pOldItem>>2]|0;
  $64 = (($63) + 36|0);
  $65 = HEAP8[$64>>0]|0;
  $66 = HEAP32[$pNewItem>>2]|0;
  $67 = (($66) + 36|0);
  HEAP8[$67>>0] = $65;
  $68 = HEAP32[$pOldItem>>2]|0;
  $69 = (($68) + 40|0);
  $70 = HEAP32[$69>>2]|0;
  $71 = HEAP32[$pNewItem>>2]|0;
  $72 = (($71) + 40|0);
  HEAP32[$72>>2] = $70;
  $73 = HEAP32[$pOldItem>>2]|0;
  $74 = (($73) + 24|0);
  $75 = HEAP32[$74>>2]|0;
  $76 = HEAP32[$pNewItem>>2]|0;
  $77 = (($76) + 24|0);
  HEAP32[$77>>2] = $75;
  $78 = HEAP32[$pOldItem>>2]|0;
  $79 = (($78) + 28|0);
  $80 = HEAP32[$79>>2]|0;
  $81 = HEAP32[$pNewItem>>2]|0;
  $82 = (($81) + 28|0);
  HEAP32[$82>>2] = $80;
  $83 = HEAP32[$pOldItem>>2]|0;
  $84 = (($83) + 37|0);
  $85 = HEAP8[$84>>0]|0;
  $86 = ($85&255) >>> 1;
  $87 = $86 & 1;
  $88 = $87&255;
  $89 = HEAP32[$pNewItem>>2]|0;
  $90 = (($89) + 37|0);
  $91 = $88&255;
  $92 = HEAP8[$90>>0]|0;
  $93 = $91 & 1;
  $94 = ($93 << 1)&255;
  $95 = $92 & -3;
  $96 = $95 | $94;
  HEAP8[$90>>0] = $96;
  $97 = HEAP32[$pOldItem>>2]|0;
  $98 = (($97) + 37|0);
  $99 = HEAP8[$98>>0]|0;
  $100 = ($99&255) >>> 2;
  $101 = $100 & 1;
  $102 = $101&255;
  $103 = HEAP32[$pNewItem>>2]|0;
  $104 = (($103) + 37|0);
  $105 = $102&255;
  $106 = HEAP8[$104>>0]|0;
  $107 = $105 & 1;
  $108 = ($107 << 2)&255;
  $109 = $106 & -5;
  $110 = $109 | $108;
  HEAP8[$104>>0] = $110;
  $111 = HEAP32[$pOldItem>>2]|0;
  $112 = (($111) + 37|0);
  $113 = HEAP8[$112>>0]|0;
  $114 = ($113&255) >>> 3;
  $115 = $114 & 1;
  $116 = $115&255;
  $117 = HEAP32[$pNewItem>>2]|0;
  $118 = (($117) + 37|0);
  $119 = $116&255;
  $120 = HEAP8[$118>>0]|0;
  $121 = $119 & 1;
  $122 = ($121 << 3)&255;
  $123 = $120 & -9;
  $124 = $123 | $122;
  HEAP8[$118>>0] = $124;
  $125 = HEAP32[$1>>2]|0;
  $126 = HEAP32[$pOldItem>>2]|0;
  $127 = (($126) + 64|0);
  $128 = HEAP32[$127>>2]|0;
  $129 = (_sqlite3DbStrDup($125,$128)|0);
  $130 = HEAP32[$pNewItem>>2]|0;
  $131 = (($130) + 64|0);
  HEAP32[$131>>2] = $129;
  $132 = HEAP32[$pOldItem>>2]|0;
  $133 = (($132) + 37|0);
  $134 = HEAP8[$133>>0]|0;
  $135 = $134 & 1;
  $136 = $135&255;
  $137 = HEAP32[$pNewItem>>2]|0;
  $138 = (($137) + 37|0);
  $139 = $136&255;
  $140 = HEAP8[$138>>0]|0;
  $141 = $139 & 1;
  $142 = $140 & -2;
  $143 = $142 | $141;
  HEAP8[$138>>0] = $143;
  $144 = HEAP32[$pOldItem>>2]|0;
  $145 = (($144) + 68|0);
  $146 = HEAP32[$145>>2]|0;
  $147 = HEAP32[$pNewItem>>2]|0;
  $148 = (($147) + 68|0);
  HEAP32[$148>>2] = $146;
  $149 = HEAP32[$pOldItem>>2]|0;
  $150 = (($149) + 16|0);
  $151 = HEAP32[$150>>2]|0;
  $152 = HEAP32[$pNewItem>>2]|0;
  $153 = (($152) + 16|0);
  HEAP32[$153>>2] = $151;
  HEAP32[$pTab>>2] = $151;
  $154 = HEAP32[$pTab>>2]|0;
  $155 = ($154|0)!=(0|0);
  if ($155) {
   $156 = HEAP32[$pTab>>2]|0;
   $157 = (($156) + 40|0);
   $158 = HEAP16[$157>>1]|0;
   $159 = (($158) + 1)<<16>>16;
   HEAP16[$157>>1] = $159;
  }
  $160 = HEAP32[$1>>2]|0;
  $161 = HEAP32[$pOldItem>>2]|0;
  $162 = (($161) + 20|0);
  $163 = HEAP32[$162>>2]|0;
  $164 = HEAP32[$3>>2]|0;
  $165 = (_sqlite3SelectDup($160,$163,$164)|0);
  $166 = HEAP32[$pNewItem>>2]|0;
  $167 = (($166) + 20|0);
  HEAP32[$167>>2] = $165;
  $168 = HEAP32[$1>>2]|0;
  $169 = HEAP32[$pOldItem>>2]|0;
  $170 = (($169) + 44|0);
  $171 = HEAP32[$170>>2]|0;
  $172 = HEAP32[$3>>2]|0;
  $173 = (_sqlite3ExprDup($168,$171,$172)|0);
  $174 = HEAP32[$pNewItem>>2]|0;
  $175 = (($174) + 44|0);
  HEAP32[$175>>2] = $173;
  $176 = HEAP32[$1>>2]|0;
  $177 = HEAP32[$pOldItem>>2]|0;
  $178 = (($177) + 48|0);
  $179 = HEAP32[$178>>2]|0;
  $180 = (_sqlite3IdListDup($176,$179)|0);
  $181 = HEAP32[$pNewItem>>2]|0;
  $182 = (($181) + 48|0);
  HEAP32[$182>>2] = $180;
  $183 = HEAP32[$pOldItem>>2]|0;
  $184 = (($183) + 56|0);
  $185 = $184;
  $186 = $185;
  $187 = HEAP32[$186>>2]|0;
  $188 = (($185) + 4)|0;
  $189 = $188;
  $190 = HEAP32[$189>>2]|0;
  $191 = HEAP32[$pNewItem>>2]|0;
  $192 = (($191) + 56|0);
  $193 = $192;
  $194 = $193;
  HEAP32[$194>>2] = $187;
  $195 = (($193) + 4)|0;
  $196 = $195;
  HEAP32[$196>>2] = $190;
  $197 = HEAP32[$i>>2]|0;
  $198 = (($197) + 1)|0;
  HEAP32[$i>>2] = $198;
 }
 $199 = HEAP32[$pNew>>2]|0;
 HEAP32[$0>>2] = $199;
 $200 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($200|0);
}
function _withDup($db,$p) {
 $db = $db|0;
 $p = $p|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $7 = 0, $8 = 0, $9 = 0, $i = 0, $nByte = 0, $pRet = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 16|0;
 $1 = sp + 12|0;
 $pRet = sp + 8|0;
 $nByte = sp + 4|0;
 $i = sp;
 HEAP32[$0>>2] = $db;
 HEAP32[$1>>2] = $p;
 HEAP32[$pRet>>2] = 0;
 $2 = HEAP32[$1>>2]|0;
 $3 = ($2|0)!=(0|0);
 if (!($3)) {
  $62 = HEAP32[$pRet>>2]|0;
  STACKTOP = sp;return ($62|0);
 }
 $4 = HEAP32[$1>>2]|0;
 $5 = HEAP32[$4>>2]|0;
 $6 = (($5) - 1)|0;
 $7 = $6<<4;
 $8 = (24 + ($7))|0;
 HEAP32[$nByte>>2] = $8;
 $9 = HEAP32[$0>>2]|0;
 $10 = HEAP32[$nByte>>2]|0;
 $11 = ($10|0)<(0);
 $12 = $11 << 31 >> 31;
 $13 = (_sqlite3DbMallocZero($9,$10,$12)|0);
 HEAP32[$pRet>>2] = $13;
 $14 = HEAP32[$pRet>>2]|0;
 $15 = ($14|0)!=(0|0);
 if (!($15)) {
  $62 = HEAP32[$pRet>>2]|0;
  STACKTOP = sp;return ($62|0);
 }
 $16 = HEAP32[$1>>2]|0;
 $17 = HEAP32[$16>>2]|0;
 $18 = HEAP32[$pRet>>2]|0;
 HEAP32[$18>>2] = $17;
 HEAP32[$i>>2] = 0;
 while(1) {
  $19 = HEAP32[$i>>2]|0;
  $20 = HEAP32[$1>>2]|0;
  $21 = HEAP32[$20>>2]|0;
  $22 = ($19|0)<($21|0);
  if (!($22)) {
   break;
  }
  $23 = HEAP32[$0>>2]|0;
  $24 = HEAP32[$i>>2]|0;
  $25 = HEAP32[$1>>2]|0;
  $26 = (($25) + 8|0);
  $27 = (($26) + ($24<<4)|0);
  $28 = (($27) + 8|0);
  $29 = HEAP32[$28>>2]|0;
  $30 = (_sqlite3SelectDup($23,$29,0)|0);
  $31 = HEAP32[$i>>2]|0;
  $32 = HEAP32[$pRet>>2]|0;
  $33 = (($32) + 8|0);
  $34 = (($33) + ($31<<4)|0);
  $35 = (($34) + 8|0);
  HEAP32[$35>>2] = $30;
  $36 = HEAP32[$0>>2]|0;
  $37 = HEAP32[$i>>2]|0;
  $38 = HEAP32[$1>>2]|0;
  $39 = (($38) + 8|0);
  $40 = (($39) + ($37<<4)|0);
  $41 = (($40) + 4|0);
  $42 = HEAP32[$41>>2]|0;
  $43 = (_sqlite3ExprListDup($36,$42,0)|0);
  $44 = HEAP32[$i>>2]|0;
  $45 = HEAP32[$pRet>>2]|0;
  $46 = (($45) + 8|0);
  $47 = (($46) + ($44<<4)|0);
  $48 = (($47) + 4|0);
  HEAP32[$48>>2] = $43;
  $49 = HEAP32[$0>>2]|0;
  $50 = HEAP32[$i>>2]|0;
  $51 = HEAP32[$1>>2]|0;
  $52 = (($51) + 8|0);
  $53 = (($52) + ($50<<4)|0);
  $54 = HEAP32[$53>>2]|0;
  $55 = (_sqlite3DbStrDup($49,$54)|0);
  $56 = HEAP32[$i>>2]|0;
  $57 = HEAP32[$pRet>>2]|0;
  $58 = (($57) + 8|0);
  $59 = (($58) + ($56<<4)|0);
  HEAP32[$59>>2] = $55;
  $60 = HEAP32[$i>>2]|0;
  $61 = (($60) + 1)|0;
  HEAP32[$i>>2] = $61;
 }
 $62 = HEAP32[$pRet>>2]|0;
 STACKTOP = sp;return ($62|0);
}
function _sqlite3IdListDup($db,$p) {
 $db = $db|0;
 $p = $p|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $i = 0, $pNew = 0, $pNewItem = 0, $pOldItem = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 24|0;
 $1 = sp + 20|0;
 $2 = sp + 16|0;
 $pNew = sp + 12|0;
 $i = sp + 8|0;
 $pNewItem = sp + 4|0;
 $pOldItem = sp;
 HEAP32[$1>>2] = $db;
 HEAP32[$2>>2] = $p;
 $3 = HEAP32[$2>>2]|0;
 $4 = ($3|0)==(0|0);
 if ($4) {
  HEAP32[$0>>2] = 0;
  $52 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($52|0);
 }
 $5 = HEAP32[$1>>2]|0;
 $6 = (_sqlite3DbMallocRaw($5,8,0)|0);
 HEAP32[$pNew>>2] = $6;
 $7 = HEAP32[$pNew>>2]|0;
 $8 = ($7|0)==(0|0);
 if ($8) {
  HEAP32[$0>>2] = 0;
  $52 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($52|0);
 }
 $9 = HEAP32[$2>>2]|0;
 $10 = (($9) + 4|0);
 $11 = HEAP32[$10>>2]|0;
 $12 = HEAP32[$pNew>>2]|0;
 $13 = (($12) + 4|0);
 HEAP32[$13>>2] = $11;
 $14 = HEAP32[$1>>2]|0;
 $15 = HEAP32[$2>>2]|0;
 $16 = (($15) + 4|0);
 $17 = HEAP32[$16>>2]|0;
 $18 = $17<<3;
 $19 = (_sqlite3DbMallocRaw($14,$18,0)|0);
 $20 = HEAP32[$pNew>>2]|0;
 HEAP32[$20>>2] = $19;
 $21 = HEAP32[$pNew>>2]|0;
 $22 = HEAP32[$21>>2]|0;
 $23 = ($22|0)==(0|0);
 if ($23) {
  $24 = HEAP32[$1>>2]|0;
  $25 = HEAP32[$pNew>>2]|0;
  _sqlite3DbFree($24,$25);
  HEAP32[$0>>2] = 0;
  $52 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($52|0);
 }
 HEAP32[$i>>2] = 0;
 while(1) {
  $26 = HEAP32[$i>>2]|0;
  $27 = HEAP32[$2>>2]|0;
  $28 = (($27) + 4|0);
  $29 = HEAP32[$28>>2]|0;
  $30 = ($26|0)<($29|0);
  if (!($30)) {
   break;
  }
  $31 = HEAP32[$i>>2]|0;
  $32 = HEAP32[$pNew>>2]|0;
  $33 = HEAP32[$32>>2]|0;
  $34 = (($33) + ($31<<3)|0);
  HEAP32[$pNewItem>>2] = $34;
  $35 = HEAP32[$i>>2]|0;
  $36 = HEAP32[$2>>2]|0;
  $37 = HEAP32[$36>>2]|0;
  $38 = (($37) + ($35<<3)|0);
  HEAP32[$pOldItem>>2] = $38;
  $39 = HEAP32[$1>>2]|0;
  $40 = HEAP32[$pOldItem>>2]|0;
  $41 = HEAP32[$40>>2]|0;
  $42 = (_sqlite3DbStrDup($39,$41)|0);
  $43 = HEAP32[$pNewItem>>2]|0;
  HEAP32[$43>>2] = $42;
  $44 = HEAP32[$pOldItem>>2]|0;
  $45 = (($44) + 4|0);
  $46 = HEAP32[$45>>2]|0;
  $47 = HEAP32[$pNewItem>>2]|0;
  $48 = (($47) + 4|0);
  HEAP32[$48>>2] = $46;
  $49 = HEAP32[$i>>2]|0;
  $50 = (($49) + 1)|0;
  HEAP32[$i>>2] = $50;
 }
 $51 = HEAP32[$pNew>>2]|0;
 HEAP32[$0>>2] = $51;
 $52 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($52|0);
}
function _vtabCallConstructor($db,$pTab,$pMod,$xConstruct,$pzErr) {
 $db = $db|0;
 $pTab = $pTab|0;
 $pMod = $pMod|0;
 $xConstruct = $xConstruct|0;
 $pzErr = $pzErr|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0;
 var $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0, $188 = 0;
 var $189 = 0, $19 = 0, $190 = 0, $191 = 0, $192 = 0, $193 = 0, $194 = 0, $195 = 0, $196 = 0, $197 = 0, $198 = 0, $199 = 0, $2 = 0, $20 = 0, $200 = 0, $201 = 0, $202 = 0, $203 = 0, $204 = 0, $205 = 0;
 var $206 = 0, $207 = 0, $208 = 0, $209 = 0, $21 = 0, $210 = 0, $211 = 0, $212 = 0, $213 = 0, $214 = 0, $215 = 0, $216 = 0, $217 = 0, $218 = 0, $219 = 0, $22 = 0, $220 = 0, $221 = 0, $222 = 0, $223 = 0;
 var $224 = 0, $225 = 0, $226 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0;
 var $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0;
 var $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0;
 var $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0;
 var $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $azArg = 0, $i = 0, $iCol = 0, $iDb = 0, $j = 0, $nArg = 0, $nDel = 0, $nType = 0, $or$cond = 0, $pPriorCtx = 0, $pVTable = 0, $rc = 0, $sCtx = 0;
 var $vararg_buffer8 = 0, $zErr = 0, $zFormat = 0, $zModuleName = 0, $zType = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 96|0;
 $vararg_buffer8 = sp;
 $0 = sp + 92|0;
 $1 = sp + 88|0;
 $2 = sp + 84|0;
 $3 = sp + 80|0;
 $4 = sp + 76|0;
 $5 = sp + 72|0;
 $sCtx = sp + 64|0;
 $pPriorCtx = sp + 60|0;
 $pVTable = sp + 56|0;
 $rc = sp + 52|0;
 $azArg = sp + 48|0;
 $nArg = sp + 44|0;
 $zErr = sp + 40|0;
 $zModuleName = sp + 36|0;
 $iDb = sp + 32|0;
 $zFormat = sp + 28|0;
 $iCol = sp + 24|0;
 $zType = sp + 20|0;
 $nType = sp + 16|0;
 $i = sp + 12|0;
 $j = sp + 8|0;
 $nDel = sp + 4|0;
 HEAP32[$1>>2] = $db;
 HEAP32[$2>>2] = $pTab;
 HEAP32[$3>>2] = $pMod;
 HEAP32[$4>>2] = $xConstruct;
 HEAP32[$5>>2] = $pzErr;
 $6 = HEAP32[$2>>2]|0;
 $7 = (($6) + 56|0);
 $8 = HEAP32[$7>>2]|0;
 HEAP32[$azArg>>2] = $8;
 $9 = HEAP32[$2>>2]|0;
 $10 = (($9) + 52|0);
 $11 = HEAP32[$10>>2]|0;
 HEAP32[$nArg>>2] = $11;
 HEAP32[$zErr>>2] = 0;
 $12 = HEAP32[$1>>2]|0;
 $13 = HEAP32[$2>>2]|0;
 $14 = HEAP32[$13>>2]|0;
 HEAP32[$vararg_buffer8>>2] = $14;
 $15 = (_sqlite3MPrintf($12,1200,$vararg_buffer8)|0);
 HEAP32[$zModuleName>>2] = $15;
 $16 = HEAP32[$zModuleName>>2]|0;
 $17 = ($16|0)!=(0|0);
 if (!($17)) {
  HEAP32[$0>>2] = 7;
  $226 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($226|0);
 }
 $18 = HEAP32[$1>>2]|0;
 $19 = (_sqlite3DbMallocZero($18,28,0)|0);
 HEAP32[$pVTable>>2] = $19;
 $20 = HEAP32[$pVTable>>2]|0;
 $21 = ($20|0)!=(0|0);
 $22 = HEAP32[$1>>2]|0;
 if (!($21)) {
  $23 = HEAP32[$zModuleName>>2]|0;
  _sqlite3DbFree($22,$23);
  HEAP32[$0>>2] = 7;
  $226 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($226|0);
 }
 $24 = HEAP32[$pVTable>>2]|0;
 HEAP32[$24>>2] = $22;
 $25 = HEAP32[$3>>2]|0;
 $26 = HEAP32[$pVTable>>2]|0;
 $27 = (($26) + 4|0);
 HEAP32[$27>>2] = $25;
 $28 = HEAP32[$1>>2]|0;
 $29 = HEAP32[$2>>2]|0;
 $30 = (($29) + 68|0);
 $31 = HEAP32[$30>>2]|0;
 $32 = (_sqlite3SchemaToIndex($28,$31)|0);
 HEAP32[$iDb>>2] = $32;
 $33 = HEAP32[$iDb>>2]|0;
 $34 = HEAP32[$1>>2]|0;
 $35 = (($34) + 16|0);
 $36 = HEAP32[$35>>2]|0;
 $37 = (($36) + ($33<<4)|0);
 $38 = HEAP32[$37>>2]|0;
 $39 = HEAP32[$2>>2]|0;
 $40 = (($39) + 56|0);
 $41 = HEAP32[$40>>2]|0;
 $42 = (($41) + 4|0);
 HEAP32[$42>>2] = $38;
 $43 = HEAP32[$2>>2]|0;
 $44 = (($sCtx) + 4|0);
 HEAP32[$44>>2] = $43;
 $45 = HEAP32[$pVTable>>2]|0;
 HEAP32[$sCtx>>2] = $45;
 $46 = HEAP32[$1>>2]|0;
 $47 = (($46) + 324|0);
 $48 = HEAP32[$47>>2]|0;
 HEAP32[$pPriorCtx>>2] = $48;
 $49 = HEAP32[$1>>2]|0;
 $50 = (($49) + 324|0);
 HEAP32[$50>>2] = $sCtx;
 $51 = HEAP32[$4>>2]|0;
 $52 = HEAP32[$1>>2]|0;
 $53 = HEAP32[$3>>2]|0;
 $54 = (($53) + 8|0);
 $55 = HEAP32[$54>>2]|0;
 $56 = HEAP32[$nArg>>2]|0;
 $57 = HEAP32[$azArg>>2]|0;
 $58 = HEAP32[$pVTable>>2]|0;
 $59 = (($58) + 8|0);
 $60 = (FUNCTION_TABLE_iiiiiii[$51 & 1]($52,$55,$56,$57,$59,$zErr)|0);
 HEAP32[$rc>>2] = $60;
 $61 = HEAP32[$pPriorCtx>>2]|0;
 $62 = HEAP32[$1>>2]|0;
 $63 = (($62) + 324|0);
 HEAP32[$63>>2] = $61;
 $64 = HEAP32[$rc>>2]|0;
 $65 = ($64|0)==(7);
 if ($65) {
  $66 = HEAP32[$1>>2]|0;
  $67 = (($66) + 64|0);
  HEAP8[$67>>0] = 1;
 }
 $68 = HEAP32[$rc>>2]|0;
 $69 = (0)!=($68|0);
 L12: do {
  if ($69) {
   $70 = HEAP32[$zErr>>2]|0;
   $71 = ($70|0)==(0|0);
   $72 = HEAP32[$1>>2]|0;
   if ($71) {
    $73 = HEAP32[$zModuleName>>2]|0;
    HEAP32[$vararg_buffer8>>2] = $73;
    $74 = (_sqlite3MPrintf($72,17968,$vararg_buffer8)|0);
    $75 = HEAP32[$5>>2]|0;
    HEAP32[$75>>2] = $74;
   } else {
    $76 = HEAP32[$zErr>>2]|0;
    HEAP32[$vararg_buffer8>>2] = $76;
    $77 = (_sqlite3MPrintf($72,1200,$vararg_buffer8)|0);
    $78 = HEAP32[$5>>2]|0;
    HEAP32[$78>>2] = $77;
    $79 = HEAP32[$zErr>>2]|0;
    _sqlite3_free($79);
   }
   $80 = HEAP32[$1>>2]|0;
   $81 = HEAP32[$pVTable>>2]|0;
   _sqlite3DbFree($80,$81);
  } else {
   $82 = HEAP32[$pVTable>>2]|0;
   $83 = (($82) + 8|0);
   $84 = HEAP32[$83>>2]|0;
   $85 = ($84|0)!=(0|0);
   if ($85) {
    $86 = HEAP32[$pVTable>>2]|0;
    $87 = (($86) + 8|0);
    $88 = HEAP32[$87>>2]|0;
    ;HEAP32[$88+0>>2]=0|0;HEAP32[$88+4>>2]=0|0;HEAP32[$88+8>>2]=0|0;
    $89 = HEAP32[$3>>2]|0;
    $90 = HEAP32[$89>>2]|0;
    $91 = HEAP32[$pVTable>>2]|0;
    $92 = (($91) + 8|0);
    $93 = HEAP32[$92>>2]|0;
    HEAP32[$93>>2] = $90;
    $94 = HEAP32[$pVTable>>2]|0;
    $95 = (($94) + 12|0);
    HEAP32[$95>>2] = 1;
    $96 = (($sCtx) + 4|0);
    $97 = HEAP32[$96>>2]|0;
    $98 = ($97|0)!=(0|0);
    if ($98) {
     HEAP32[$zFormat>>2] = 18000;
     $99 = HEAP32[$1>>2]|0;
     $100 = HEAP32[$zFormat>>2]|0;
     $101 = HEAP32[$2>>2]|0;
     $102 = HEAP32[$101>>2]|0;
     HEAP32[$vararg_buffer8>>2] = $102;
     $103 = (_sqlite3MPrintf($99,$100,$vararg_buffer8)|0);
     $104 = HEAP32[$5>>2]|0;
     HEAP32[$104>>2] = $103;
     $105 = HEAP32[$pVTable>>2]|0;
     _sqlite3VtabUnlock($105);
     HEAP32[$rc>>2] = 1;
     break;
    }
    $106 = HEAP32[$2>>2]|0;
    $107 = (($106) + 60|0);
    $108 = HEAP32[$107>>2]|0;
    $109 = HEAP32[$pVTable>>2]|0;
    $110 = (($109) + 24|0);
    HEAP32[$110>>2] = $108;
    $111 = HEAP32[$pVTable>>2]|0;
    $112 = HEAP32[$2>>2]|0;
    $113 = (($112) + 60|0);
    HEAP32[$113>>2] = $111;
    HEAP32[$iCol>>2] = 0;
    while(1) {
     $114 = HEAP32[$iCol>>2]|0;
     $115 = HEAP32[$2>>2]|0;
     $116 = (($115) + 38|0);
     $117 = HEAP16[$116>>1]|0;
     $118 = $117 << 16 >> 16;
     $119 = ($114|0)<($118|0);
     if (!($119)) {
      break L12;
     }
     $120 = HEAP32[$iCol>>2]|0;
     $121 = HEAP32[$2>>2]|0;
     $122 = (($121) + 4|0);
     $123 = HEAP32[$122>>2]|0;
     $124 = (($123) + (($120*24)|0)|0);
     $125 = (($124) + 12|0);
     $126 = HEAP32[$125>>2]|0;
     HEAP32[$zType>>2] = $126;
     HEAP32[$i>>2] = 0;
     $127 = HEAP32[$zType>>2]|0;
     $128 = ($127|0)!=(0|0);
     if ($128) {
      $129 = HEAP32[$zType>>2]|0;
      $130 = (_sqlite3Strlen30($129)|0);
      HEAP32[$nType>>2] = $130;
      $131 = HEAP32[$zType>>2]|0;
      $132 = (_sqlite3_strnicmp(18048,$131,6)|0);
      $133 = ($132|0)!=(0);
      if ($133) {
       label = 21;
      } else {
       $134 = HEAP32[$zType>>2]|0;
       $135 = (($134) + 6|0);
       $136 = HEAP8[$135>>0]|0;
       $137 = $136 << 24 >> 24;
       $138 = ($137|0)!=(0);
       if ($138) {
        $139 = HEAP32[$zType>>2]|0;
        $140 = (($139) + 6|0);
        $141 = HEAP8[$140>>0]|0;
        $142 = $141 << 24 >> 24;
        $143 = ($142|0)!=(32);
        if ($143) {
         label = 21;
        }
       }
      }
      L26: do {
       if ((label|0) == 21) {
        label = 0;
        HEAP32[$i>>2] = 0;
        while(1) {
         $144 = HEAP32[$i>>2]|0;
         $145 = HEAP32[$nType>>2]|0;
         $146 = ($144|0)<($145|0);
         if (!($146)) {
          break L26;
         }
         $147 = HEAP32[$i>>2]|0;
         $148 = HEAP32[$zType>>2]|0;
         $149 = (($148) + ($147)|0);
         $150 = (_sqlite3_strnicmp(18056,$149,7)|0);
         $151 = (0)==($150|0);
         if ($151) {
          $152 = HEAP32[$i>>2]|0;
          $153 = (($152) + 7)|0;
          $154 = HEAP32[$zType>>2]|0;
          $155 = (($154) + ($153)|0);
          $156 = HEAP8[$155>>0]|0;
          $157 = $156 << 24 >> 24;
          $158 = ($157|0)==(0);
          if ($158) {
           break;
          }
          $159 = HEAP32[$i>>2]|0;
          $160 = (($159) + 7)|0;
          $161 = HEAP32[$zType>>2]|0;
          $162 = (($161) + ($160)|0);
          $163 = HEAP8[$162>>0]|0;
          $164 = $163 << 24 >> 24;
          $165 = ($164|0)==(32);
          if ($165) {
           break;
          }
         }
         $168 = HEAP32[$i>>2]|0;
         $169 = (($168) + 1)|0;
         HEAP32[$i>>2] = $169;
        }
        $166 = HEAP32[$i>>2]|0;
        $167 = (($166) + 1)|0;
        HEAP32[$i>>2] = $167;
       }
      } while(0);
      $170 = HEAP32[$i>>2]|0;
      $171 = HEAP32[$nType>>2]|0;
      $172 = ($170|0)<($171|0);
      if ($172) {
       $173 = HEAP32[$i>>2]|0;
       $174 = (($173) + 6)|0;
       $175 = HEAP32[$zType>>2]|0;
       $176 = (($175) + ($174)|0);
       $177 = HEAP8[$176>>0]|0;
       $178 = $177 << 24 >> 24;
       $179 = ($178|0)!=(0);
       $180 = $179 ? 1 : 0;
       $181 = (6 + ($180))|0;
       HEAP32[$nDel>>2] = $181;
       $182 = HEAP32[$i>>2]|0;
       HEAP32[$j>>2] = $182;
       while(1) {
        $183 = HEAP32[$j>>2]|0;
        $184 = HEAP32[$nDel>>2]|0;
        $185 = (($183) + ($184))|0;
        $186 = HEAP32[$nType>>2]|0;
        $187 = ($185|0)<=($186|0);
        if (!($187)) {
         break;
        }
        $188 = HEAP32[$j>>2]|0;
        $189 = HEAP32[$nDel>>2]|0;
        $190 = (($188) + ($189))|0;
        $191 = HEAP32[$zType>>2]|0;
        $192 = (($191) + ($190)|0);
        $193 = HEAP8[$192>>0]|0;
        $194 = HEAP32[$j>>2]|0;
        $195 = HEAP32[$zType>>2]|0;
        $196 = (($195) + ($194)|0);
        HEAP8[$196>>0] = $193;
        $197 = HEAP32[$j>>2]|0;
        $198 = (($197) + 1)|0;
        HEAP32[$j>>2] = $198;
       }
       $199 = HEAP32[$i>>2]|0;
       $200 = HEAP32[$zType>>2]|0;
       $201 = (($200) + ($199)|0);
       $202 = HEAP8[$201>>0]|0;
       $203 = $202 << 24 >> 24;
       $204 = ($203|0)==(0);
       $205 = HEAP32[$i>>2]|0;
       $206 = ($205|0)>(0);
       $or$cond = $204 & $206;
       if ($or$cond) {
        $207 = HEAP32[$i>>2]|0;
        $208 = (($207) - 1)|0;
        $209 = HEAP32[$zType>>2]|0;
        $210 = (($209) + ($208)|0);
        HEAP8[$210>>0] = 0;
       }
       $211 = HEAP32[$iCol>>2]|0;
       $212 = HEAP32[$2>>2]|0;
       $213 = (($212) + 4|0);
       $214 = HEAP32[$213>>2]|0;
       $215 = (($214) + (($211*24)|0)|0);
       $216 = (($215) + 23|0);
       $217 = HEAP8[$216>>0]|0;
       $218 = $217&255;
       $219 = $218 | 2;
       $220 = $219&255;
       HEAP8[$216>>0] = $220;
      }
     }
     $221 = HEAP32[$iCol>>2]|0;
     $222 = (($221) + 1)|0;
     HEAP32[$iCol>>2] = $222;
    }
   }
  }
 } while(0);
 $223 = HEAP32[$1>>2]|0;
 $224 = HEAP32[$zModuleName>>2]|0;
 _sqlite3DbFree($223,$224);
 $225 = HEAP32[$rc>>2]|0;
 HEAP32[$0>>2] = $225;
 $226 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($226|0);
}
function _sqlite3ReadSchema($pParse) {
 $pParse = $pParse|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0;
 var $8 = 0, $9 = 0, $db = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $rc = sp + 4|0;
 $db = sp;
 HEAP32[$0>>2] = $pParse;
 HEAP32[$rc>>2] = 0;
 $1 = HEAP32[$0>>2]|0;
 $2 = HEAP32[$1>>2]|0;
 HEAP32[$db>>2] = $2;
 $3 = HEAP32[$db>>2]|0;
 $4 = (($3) + 140|0);
 $5 = (($4) + 5|0);
 $6 = HEAP8[$5>>0]|0;
 $7 = ($6<<24>>24)!=(0);
 if (!($7)) {
  $8 = HEAP32[$db>>2]|0;
  $9 = HEAP32[$0>>2]|0;
  $10 = (($9) + 4|0);
  $11 = (_sqlite3Init($8,$10)|0);
  HEAP32[$rc>>2] = $11;
 }
 $12 = HEAP32[$rc>>2]|0;
 $13 = ($12|0)!=(0);
 if (!($13)) {
  $21 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($21|0);
 }
 $14 = HEAP32[$rc>>2]|0;
 $15 = HEAP32[$0>>2]|0;
 $16 = (($15) + 12|0);
 HEAP32[$16>>2] = $14;
 $17 = HEAP32[$0>>2]|0;
 $18 = (($17) + 64|0);
 $19 = HEAP32[$18>>2]|0;
 $20 = (($19) + 1)|0;
 HEAP32[$18>>2] = $20;
 $21 = HEAP32[$rc>>2]|0;
 STACKTOP = sp;return ($21|0);
}
function _analyzeDatabase($pParse,$iDb) {
 $pParse = $pParse|0;
 $iDb = $iDb|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $db = 0, $iMem = 0, $iStatCur = 0, $iTab = 0, $k = 0, $pSchema = 0, $pTab = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $0 = sp + 32|0;
 $1 = sp + 28|0;
 $db = sp + 24|0;
 $pSchema = sp + 20|0;
 $k = sp + 16|0;
 $iStatCur = sp + 12|0;
 $iMem = sp + 8|0;
 $iTab = sp + 4|0;
 $pTab = sp;
 HEAP32[$0>>2] = $pParse;
 HEAP32[$1>>2] = $iDb;
 $2 = HEAP32[$0>>2]|0;
 $3 = HEAP32[$2>>2]|0;
 HEAP32[$db>>2] = $3;
 $4 = HEAP32[$1>>2]|0;
 $5 = HEAP32[$db>>2]|0;
 $6 = (($5) + 16|0);
 $7 = HEAP32[$6>>2]|0;
 $8 = (($7) + ($4<<4)|0);
 $9 = (($8) + 12|0);
 $10 = HEAP32[$9>>2]|0;
 HEAP32[$pSchema>>2] = $10;
 $11 = HEAP32[$0>>2]|0;
 $12 = HEAP32[$1>>2]|0;
 _sqlite3BeginWriteOperation($11,0,$12);
 $13 = HEAP32[$0>>2]|0;
 $14 = (($13) + 68|0);
 $15 = HEAP32[$14>>2]|0;
 HEAP32[$iStatCur>>2] = $15;
 $16 = HEAP32[$0>>2]|0;
 $17 = (($16) + 68|0);
 $18 = HEAP32[$17>>2]|0;
 $19 = (($18) + 3)|0;
 HEAP32[$17>>2] = $19;
 $20 = HEAP32[$0>>2]|0;
 $21 = HEAP32[$1>>2]|0;
 $22 = HEAP32[$iStatCur>>2]|0;
 _openStatTable($20,$21,$22,0,0);
 $23 = HEAP32[$0>>2]|0;
 $24 = (($23) + 72|0);
 $25 = HEAP32[$24>>2]|0;
 $26 = (($25) + 1)|0;
 HEAP32[$iMem>>2] = $26;
 $27 = HEAP32[$0>>2]|0;
 $28 = (($27) + 68|0);
 $29 = HEAP32[$28>>2]|0;
 HEAP32[$iTab>>2] = $29;
 $30 = HEAP32[$pSchema>>2]|0;
 $31 = (($30) + 8|0);
 $32 = (($31) + 8|0);
 $33 = HEAP32[$32>>2]|0;
 HEAP32[$k>>2] = $33;
 while(1) {
  $34 = HEAP32[$k>>2]|0;
  $35 = ($34|0)!=(0|0);
  if (!($35)) {
   break;
  }
  $36 = HEAP32[$k>>2]|0;
  $37 = (($36) + 8|0);
  $38 = HEAP32[$37>>2]|0;
  HEAP32[$pTab>>2] = $38;
  $39 = HEAP32[$0>>2]|0;
  $40 = HEAP32[$pTab>>2]|0;
  $41 = HEAP32[$iStatCur>>2]|0;
  $42 = HEAP32[$iMem>>2]|0;
  $43 = HEAP32[$iTab>>2]|0;
  _analyzeOneTable($39,$40,0,$41,$42,$43);
  $44 = HEAP32[$k>>2]|0;
  $45 = HEAP32[$44>>2]|0;
  HEAP32[$k>>2] = $45;
 }
 $46 = HEAP32[$0>>2]|0;
 $47 = HEAP32[$1>>2]|0;
 _loadAnalysis($46,$47);
 STACKTOP = sp;return;
}
function _sqlite3FindDb($db,$pName) {
 $db = $db|0;
 $pName = $pName|0;
 var $0 = 0, $1 = 0, $10 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $i = 0, $zName = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $1 = sp + 8|0;
 $i = sp + 4|0;
 $zName = sp;
 HEAP32[$0>>2] = $db;
 HEAP32[$1>>2] = $pName;
 $2 = HEAP32[$0>>2]|0;
 $3 = HEAP32[$1>>2]|0;
 $4 = (_sqlite3NameFromToken($2,$3)|0);
 HEAP32[$zName>>2] = $4;
 $5 = HEAP32[$0>>2]|0;
 $6 = HEAP32[$zName>>2]|0;
 $7 = (_sqlite3FindDbName($5,$6)|0);
 HEAP32[$i>>2] = $7;
 $8 = HEAP32[$0>>2]|0;
 $9 = HEAP32[$zName>>2]|0;
 _sqlite3DbFree($8,$9);
 $10 = HEAP32[$i>>2]|0;
 STACKTOP = sp;return ($10|0);
}
function _analyzeTable($pParse,$pTab,$pOnlyIdx) {
 $pParse = $pParse|0;
 $pTab = $pTab|0;
 $pOnlyIdx = $pOnlyIdx|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0;
 var $iDb = 0, $iStatCur = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 16|0;
 $1 = sp + 12|0;
 $2 = sp + 8|0;
 $iDb = sp + 4|0;
 $iStatCur = sp;
 HEAP32[$0>>2] = $pParse;
 HEAP32[$1>>2] = $pTab;
 HEAP32[$2>>2] = $pOnlyIdx;
 $3 = HEAP32[$0>>2]|0;
 $4 = HEAP32[$3>>2]|0;
 $5 = HEAP32[$1>>2]|0;
 $6 = (($5) + 68|0);
 $7 = HEAP32[$6>>2]|0;
 $8 = (_sqlite3SchemaToIndex($4,$7)|0);
 HEAP32[$iDb>>2] = $8;
 $9 = HEAP32[$0>>2]|0;
 $10 = HEAP32[$iDb>>2]|0;
 _sqlite3BeginWriteOperation($9,0,$10);
 $11 = HEAP32[$0>>2]|0;
 $12 = (($11) + 68|0);
 $13 = HEAP32[$12>>2]|0;
 HEAP32[$iStatCur>>2] = $13;
 $14 = HEAP32[$0>>2]|0;
 $15 = (($14) + 68|0);
 $16 = HEAP32[$15>>2]|0;
 $17 = (($16) + 3)|0;
 HEAP32[$15>>2] = $17;
 $18 = HEAP32[$2>>2]|0;
 $19 = ($18|0)!=(0|0);
 $20 = HEAP32[$0>>2]|0;
 $21 = HEAP32[$iDb>>2]|0;
 $22 = HEAP32[$iStatCur>>2]|0;
 if ($19) {
  $23 = HEAP32[$2>>2]|0;
  $24 = HEAP32[$23>>2]|0;
  _openStatTable($20,$21,$22,$24,18160);
 } else {
  $25 = HEAP32[$1>>2]|0;
  $26 = HEAP32[$25>>2]|0;
  _openStatTable($20,$21,$22,$26,18168);
 }
 $27 = HEAP32[$0>>2]|0;
 $28 = HEAP32[$1>>2]|0;
 $29 = HEAP32[$2>>2]|0;
 $30 = HEAP32[$iStatCur>>2]|0;
 $31 = HEAP32[$0>>2]|0;
 $32 = (($31) + 72|0);
 $33 = HEAP32[$32>>2]|0;
 $34 = (($33) + 1)|0;
 $35 = HEAP32[$0>>2]|0;
 $36 = (($35) + 68|0);
 $37 = HEAP32[$36>>2]|0;
 _analyzeOneTable($27,$28,$29,$30,$34,$37);
 $38 = HEAP32[$0>>2]|0;
 $39 = HEAP32[$iDb>>2]|0;
 _loadAnalysis($38,$39);
 STACKTOP = sp;return;
}
function _sqlite3TwoPartName($pParse,$pName1,$pName2,$pUnqual) {
 $pParse = $pParse|0;
 $pName1 = $pName1|0;
 $pName2 = $pName2|0;
 $pUnqual = $pUnqual|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $db = 0, $iDb = 0, $vararg_buffer1 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $vararg_buffer1 = sp;
 $0 = sp + 28|0;
 $1 = sp + 24|0;
 $2 = sp + 20|0;
 $3 = sp + 16|0;
 $4 = sp + 12|0;
 $iDb = sp + 8|0;
 $db = sp + 4|0;
 HEAP32[$1>>2] = $pParse;
 HEAP32[$2>>2] = $pName1;
 HEAP32[$3>>2] = $pName2;
 HEAP32[$4>>2] = $pUnqual;
 $5 = HEAP32[$1>>2]|0;
 $6 = HEAP32[$5>>2]|0;
 HEAP32[$db>>2] = $6;
 $7 = HEAP32[$3>>2]|0;
 $8 = ($7|0)!=(0|0);
 if ($8) {
  $9 = HEAP32[$3>>2]|0;
  $10 = (($9) + 4|0);
  $11 = HEAP32[$10>>2]|0;
  $12 = ($11>>>0)>(0);
  if ($12) {
   $13 = HEAP32[$db>>2]|0;
   $14 = (($13) + 140|0);
   $15 = (($14) + 5|0);
   $16 = HEAP8[$15>>0]|0;
   $17 = ($16<<24>>24)!=(0);
   if ($17) {
    $18 = HEAP32[$1>>2]|0;
    _sqlite3ErrorMsg($18,18112,$vararg_buffer1);
    $19 = HEAP32[$1>>2]|0;
    $20 = (($19) + 64|0);
    $21 = HEAP32[$20>>2]|0;
    $22 = (($21) + 1)|0;
    HEAP32[$20>>2] = $22;
    HEAP32[$0>>2] = -1;
    $44 = HEAP32[$0>>2]|0;
    STACKTOP = sp;return ($44|0);
   }
   $23 = HEAP32[$3>>2]|0;
   $24 = HEAP32[$4>>2]|0;
   HEAP32[$24>>2] = $23;
   $25 = HEAP32[$db>>2]|0;
   $26 = HEAP32[$2>>2]|0;
   $27 = (_sqlite3FindDb($25,$26)|0);
   HEAP32[$iDb>>2] = $27;
   $28 = HEAP32[$iDb>>2]|0;
   $29 = ($28|0)<(0);
   if ($29) {
    $30 = HEAP32[$1>>2]|0;
    $31 = HEAP32[$2>>2]|0;
    HEAP32[$vararg_buffer1>>2] = $31;
    _sqlite3ErrorMsg($30,18136,$vararg_buffer1);
    $32 = HEAP32[$1>>2]|0;
    $33 = (($32) + 64|0);
    $34 = HEAP32[$33>>2]|0;
    $35 = (($34) + 1)|0;
    HEAP32[$33>>2] = $35;
    HEAP32[$0>>2] = -1;
    $44 = HEAP32[$0>>2]|0;
    STACKTOP = sp;return ($44|0);
   }
  } else {
   label = 7;
  }
 } else {
  label = 7;
 }
 if ((label|0) == 7) {
  $36 = HEAP32[$db>>2]|0;
  $37 = (($36) + 140|0);
  $38 = (($37) + 4|0);
  $39 = HEAP8[$38>>0]|0;
  $40 = $39&255;
  HEAP32[$iDb>>2] = $40;
  $41 = HEAP32[$2>>2]|0;
  $42 = HEAP32[$4>>2]|0;
  HEAP32[$42>>2] = $41;
 }
 $43 = HEAP32[$iDb>>2]|0;
 HEAP32[$0>>2] = $43;
 $44 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($44|0);
}
function _openStatTable($pParse,$iDb,$iStatCur,$zWhere,$zWhereType) {
 $pParse = $pParse|0;
 $iDb = $iDb|0;
 $iStatCur = $iStatCur|0;
 $zWhere = $zWhere|0;
 $zWhereType = $zWhereType|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0;
 var $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $aCreateTbl = 0, $aRoot = 0, $db = 0, $i = 0, $pDb = 0, $pStat = 0;
 var $v = 0, $vararg_buffer3 = 0, $vararg_ptr1 = 0, $vararg_ptr2 = 0, $vararg_ptr6 = 0, $vararg_ptr7 = 0, $vararg_ptr8 = 0, $zTab = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 80|0;
 $vararg_buffer3 = sp;
 $0 = sp + 68|0;
 $1 = sp + 64|0;
 $2 = sp + 60|0;
 $3 = sp + 56|0;
 $4 = sp + 52|0;
 $i = sp + 48|0;
 $db = sp + 44|0;
 $pDb = sp + 40|0;
 $v = sp + 36|0;
 $aRoot = sp + 24|0;
 $aCreateTbl = sp + 72|0;
 $zTab = sp + 20|0;
 $pStat = sp + 16|0;
 HEAP32[$0>>2] = $pParse;
 HEAP32[$1>>2] = $iDb;
 HEAP32[$2>>2] = $iStatCur;
 HEAP32[$3>>2] = $zWhere;
 HEAP32[$4>>2] = $zWhereType;
 $5 = HEAP32[$0>>2]|0;
 $6 = HEAP32[$5>>2]|0;
 HEAP32[$db>>2] = $6;
 $7 = HEAP32[$0>>2]|0;
 $8 = (_sqlite3GetVdbe($7)|0);
 HEAP32[$v>>2] = $8;
 $9 = HEAP32[$v>>2]|0;
 $10 = ($9|0)==(0|0);
 if ($10) {
  STACKTOP = sp;return;
 }
 $11 = HEAP32[$1>>2]|0;
 $12 = HEAP32[$db>>2]|0;
 $13 = (($12) + 16|0);
 $14 = HEAP32[$13>>2]|0;
 $15 = (($14) + ($11<<4)|0);
 HEAP32[$pDb>>2] = $15;
 HEAP32[$i>>2] = 0;
 while(1) {
  $16 = HEAP32[$i>>2]|0;
  $17 = ($16|0)<(3);
  if (!($17)) {
   break;
  }
  $18 = HEAP32[$i>>2]|0;
  $19 = (18368 + ($18<<3)|0);
  $20 = HEAP32[$19>>2]|0;
  HEAP32[$zTab>>2] = $20;
  $21 = HEAP32[$db>>2]|0;
  $22 = HEAP32[$zTab>>2]|0;
  $23 = HEAP32[$pDb>>2]|0;
  $24 = HEAP32[$23>>2]|0;
  $25 = (_sqlite3FindTable($21,$22,$24)|0);
  HEAP32[$pStat>>2] = $25;
  $26 = ($25|0)==(0|0);
  do {
   if ($26) {
    $27 = HEAP32[$i>>2]|0;
    $28 = (18368 + ($27<<3)|0);
    $29 = (($28) + 4|0);
    $30 = HEAP32[$29>>2]|0;
    $31 = ($30|0)!=(0|0);
    if ($31) {
     $32 = HEAP32[$0>>2]|0;
     $33 = HEAP32[$pDb>>2]|0;
     $34 = HEAP32[$33>>2]|0;
     $35 = HEAP32[$zTab>>2]|0;
     $36 = HEAP32[$i>>2]|0;
     $37 = (18368 + ($36<<3)|0);
     $38 = (($37) + 4|0);
     $39 = HEAP32[$38>>2]|0;
     HEAP32[$vararg_buffer3>>2] = $34;
     $vararg_ptr1 = (($vararg_buffer3) + 4|0);
     HEAP32[$vararg_ptr1>>2] = $35;
     $vararg_ptr2 = (($vararg_buffer3) + 8|0);
     HEAP32[$vararg_ptr2>>2] = $39;
     _sqlite3NestedParse($32,18456,$vararg_buffer3);
     $40 = HEAP32[$0>>2]|0;
     $41 = (($40) + 388|0);
     $42 = HEAP32[$41>>2]|0;
     $43 = HEAP32[$i>>2]|0;
     $44 = (($aRoot) + ($43<<2)|0);
     HEAP32[$44>>2] = $42;
     $45 = HEAP32[$i>>2]|0;
     $46 = (($aCreateTbl) + ($45)|0);
     HEAP8[$46>>0] = 2;
    }
   } else {
    $47 = HEAP32[$pStat>>2]|0;
    $48 = (($47) + 32|0);
    $49 = HEAP32[$48>>2]|0;
    $50 = HEAP32[$i>>2]|0;
    $51 = (($aRoot) + ($50<<2)|0);
    HEAP32[$51>>2] = $49;
    $52 = HEAP32[$i>>2]|0;
    $53 = (($aCreateTbl) + ($52)|0);
    HEAP8[$53>>0] = 0;
    $54 = HEAP32[$0>>2]|0;
    $55 = HEAP32[$1>>2]|0;
    $56 = HEAP32[$i>>2]|0;
    $57 = (($aRoot) + ($56<<2)|0);
    $58 = HEAP32[$57>>2]|0;
    $59 = HEAP32[$zTab>>2]|0;
    _sqlite3TableLock($54,$55,$58,1,$59);
    $60 = HEAP32[$3>>2]|0;
    $61 = ($60|0)!=(0|0);
    if ($61) {
     $62 = HEAP32[$0>>2]|0;
     $63 = HEAP32[$pDb>>2]|0;
     $64 = HEAP32[$63>>2]|0;
     $65 = HEAP32[$zTab>>2]|0;
     $66 = HEAP32[$4>>2]|0;
     $67 = HEAP32[$3>>2]|0;
     HEAP32[$vararg_buffer3>>2] = $64;
     $vararg_ptr6 = (($vararg_buffer3) + 4|0);
     HEAP32[$vararg_ptr6>>2] = $65;
     $vararg_ptr7 = (($vararg_buffer3) + 8|0);
     HEAP32[$vararg_ptr7>>2] = $66;
     $vararg_ptr8 = (($vararg_buffer3) + 12|0);
     HEAP32[$vararg_ptr8>>2] = $67;
     _sqlite3NestedParse($62,18480,$vararg_buffer3);
     break;
    } else {
     $68 = HEAP32[$v>>2]|0;
     $69 = HEAP32[$i>>2]|0;
     $70 = (($aRoot) + ($69<<2)|0);
     $71 = HEAP32[$70>>2]|0;
     $72 = HEAP32[$1>>2]|0;
     (_sqlite3VdbeAddOp2($68,118,$71,$72)|0);
     break;
    }
   }
  } while(0);
  $73 = HEAP32[$i>>2]|0;
  $74 = (($73) + 1)|0;
  HEAP32[$i>>2] = $74;
 }
 HEAP32[$i>>2] = 0;
 while(1) {
  $75 = HEAP32[$i>>2]|0;
  $76 = (18368 + ($75<<3)|0);
  $77 = (($76) + 4|0);
  $78 = HEAP32[$77>>2]|0;
  $79 = ($78|0)!=(0|0);
  if (!($79)) {
   break;
  }
  $80 = HEAP32[$v>>2]|0;
  $81 = HEAP32[$2>>2]|0;
  $82 = HEAP32[$i>>2]|0;
  $83 = (($81) + ($82))|0;
  $84 = HEAP32[$i>>2]|0;
  $85 = (($aRoot) + ($84<<2)|0);
  $86 = HEAP32[$85>>2]|0;
  $87 = HEAP32[$1>>2]|0;
  (_sqlite3VdbeAddOp4Int($80,55,$83,$86,$87,3)|0);
  $88 = HEAP32[$v>>2]|0;
  $89 = HEAP32[$i>>2]|0;
  $90 = (($aCreateTbl) + ($89)|0);
  $91 = HEAP8[$90>>0]|0;
  _sqlite3VdbeChangeP5($88,$91);
  $92 = HEAP32[$i>>2]|0;
  $93 = (($92) + 1)|0;
  HEAP32[$i>>2] = $93;
 }
 STACKTOP = sp;return;
}
function _analyzeOneTable($pParse,$pTab,$pOnlyIdx,$iStatCur,$iMem,$iTab) {
 $pParse = $pParse|0;
 $pTab = $pTab|0;
 $pOnlyIdx = $pOnlyIdx|0;
 $iStatCur = $iStatCur|0;
 $iMem = $iMem|0;
 $iTab = $iTab|0;
 var $$sink = 0, $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0;
 var $115 = 0, $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0;
 var $133 = 0, $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0;
 var $151 = 0, $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0;
 var $17 = 0, $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0;
 var $188 = 0, $189 = 0, $19 = 0, $190 = 0, $191 = 0, $192 = 0, $193 = 0, $194 = 0, $195 = 0, $196 = 0, $197 = 0, $198 = 0, $199 = 0, $2 = 0, $20 = 0, $200 = 0, $201 = 0, $202 = 0, $203 = 0, $204 = 0;
 var $205 = 0, $206 = 0, $207 = 0, $208 = 0, $209 = 0, $21 = 0, $210 = 0, $211 = 0, $212 = 0, $213 = 0, $214 = 0, $215 = 0, $216 = 0, $217 = 0, $218 = 0, $219 = 0, $22 = 0, $220 = 0, $221 = 0, $222 = 0;
 var $223 = 0, $224 = 0, $225 = 0, $226 = 0, $227 = 0, $228 = 0, $229 = 0, $23 = 0, $230 = 0, $231 = 0, $232 = 0, $233 = 0, $234 = 0, $235 = 0, $236 = 0, $237 = 0, $238 = 0, $239 = 0, $24 = 0, $240 = 0;
 var $241 = 0, $242 = 0, $243 = 0, $244 = 0, $245 = 0, $246 = 0, $247 = 0, $248 = 0, $249 = 0, $25 = 0, $250 = 0, $251 = 0, $252 = 0, $253 = 0, $254 = 0, $255 = 0, $256 = 0, $257 = 0, $258 = 0, $259 = 0;
 var $26 = 0, $260 = 0, $261 = 0, $262 = 0, $263 = 0, $264 = 0, $265 = 0, $266 = 0, $267 = 0, $268 = 0, $269 = 0, $27 = 0, $270 = 0, $271 = 0, $272 = 0, $273 = 0, $274 = 0, $275 = 0, $276 = 0, $277 = 0;
 var $278 = 0, $279 = 0, $28 = 0, $280 = 0, $281 = 0, $282 = 0, $283 = 0, $284 = 0, $285 = 0, $286 = 0, $287 = 0, $288 = 0, $289 = 0, $29 = 0, $290 = 0, $291 = 0, $292 = 0, $293 = 0, $294 = 0, $295 = 0;
 var $296 = 0, $297 = 0, $298 = 0, $299 = 0, $3 = 0, $30 = 0, $300 = 0, $301 = 0, $302 = 0, $303 = 0, $304 = 0, $305 = 0, $306 = 0, $307 = 0, $308 = 0, $309 = 0, $31 = 0, $310 = 0, $311 = 0, $312 = 0;
 var $313 = 0, $314 = 0, $315 = 0, $316 = 0, $317 = 0, $318 = 0, $319 = 0, $32 = 0, $320 = 0, $321 = 0, $322 = 0, $323 = 0, $324 = 0, $325 = 0, $326 = 0, $327 = 0, $328 = 0, $329 = 0, $33 = 0, $330 = 0;
 var $331 = 0, $332 = 0, $333 = 0, $334 = 0, $335 = 0, $336 = 0, $337 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0;
 var $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0;
 var $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0;
 var $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $aGotoChng = 0;
 var $addrNextRow = 0, $addrRewind = 0, $db = 0, $endDistinctTest = 0, $i = 0, $iDb = 0, $iIdxCur = 0, $iTabCur = 0, $jZeroRows = 0, $nCol = 0, $nColTest = 0, $needTableCnt = 0, $or$cond = 0, $pColl = 0, $pIdx = 0, $regChng = 0, $regIdxname = 0, $regNewRowid = 0, $regPrev = 0, $regStat1 = 0;
 var $regStat4 = 0, $regTabname = 0, $regTemp = 0, $v = 0, $zIdxName = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 128|0;
 $0 = sp + 116|0;
 $1 = sp + 112|0;
 $2 = sp + 108|0;
 $3 = sp + 104|0;
 $4 = sp + 100|0;
 $5 = sp + 96|0;
 $db = sp + 92|0;
 $pIdx = sp + 88|0;
 $iIdxCur = sp + 84|0;
 $iTabCur = sp + 80|0;
 $v = sp + 76|0;
 $i = sp + 72|0;
 $jZeroRows = sp + 68|0;
 $iDb = sp + 64|0;
 $needTableCnt = sp + 120|0;
 $regNewRowid = sp + 60|0;
 $regStat4 = sp + 56|0;
 $regChng = sp + 52|0;
 $regTemp = sp + 48|0;
 $regTabname = sp + 44|0;
 $regIdxname = sp + 40|0;
 $regStat1 = sp + 36|0;
 $regPrev = sp + 32|0;
 $nCol = sp + 28|0;
 $addrRewind = sp + 24|0;
 $addrNextRow = sp + 20|0;
 $zIdxName = sp + 16|0;
 $nColTest = sp + 12|0;
 $endDistinctTest = sp + 8|0;
 $aGotoChng = sp + 4|0;
 $pColl = sp;
 HEAP32[$0>>2] = $pParse;
 HEAP32[$1>>2] = $pTab;
 HEAP32[$2>>2] = $pOnlyIdx;
 HEAP32[$3>>2] = $iStatCur;
 HEAP32[$4>>2] = $iMem;
 HEAP32[$5>>2] = $iTab;
 $6 = HEAP32[$0>>2]|0;
 $7 = HEAP32[$6>>2]|0;
 HEAP32[$db>>2] = $7;
 HEAP32[$jZeroRows>>2] = -1;
 HEAP8[$needTableCnt>>0] = 1;
 $8 = HEAP32[$4>>2]|0;
 $9 = (($8) + 1)|0;
 HEAP32[$4>>2] = $9;
 HEAP32[$regNewRowid>>2] = $8;
 $10 = HEAP32[$4>>2]|0;
 $11 = (($10) + 1)|0;
 HEAP32[$4>>2] = $11;
 HEAP32[$regStat4>>2] = $10;
 $12 = HEAP32[$4>>2]|0;
 $13 = (($12) + 1)|0;
 HEAP32[$4>>2] = $13;
 HEAP32[$regChng>>2] = $12;
 $14 = HEAP32[$4>>2]|0;
 $15 = (($14) + 1)|0;
 HEAP32[$4>>2] = $15;
 HEAP32[$regTemp>>2] = $14;
 $16 = HEAP32[$4>>2]|0;
 $17 = (($16) + 1)|0;
 HEAP32[$4>>2] = $17;
 HEAP32[$regTabname>>2] = $16;
 $18 = HEAP32[$4>>2]|0;
 $19 = (($18) + 1)|0;
 HEAP32[$4>>2] = $19;
 HEAP32[$regIdxname>>2] = $18;
 $20 = HEAP32[$4>>2]|0;
 $21 = (($20) + 1)|0;
 HEAP32[$4>>2] = $21;
 HEAP32[$regStat1>>2] = $20;
 $22 = HEAP32[$4>>2]|0;
 HEAP32[$regPrev>>2] = $22;
 $23 = HEAP32[$0>>2]|0;
 $24 = (($23) + 72|0);
 $25 = HEAP32[$24>>2]|0;
 $26 = HEAP32[$4>>2]|0;
 $27 = ($25|0)>($26|0);
 if ($27) {
  $28 = HEAP32[$0>>2]|0;
  $29 = (($28) + 72|0);
  $30 = HEAP32[$29>>2]|0;
  $34 = $30;
 } else {
  $31 = HEAP32[$4>>2]|0;
  $34 = $31;
 }
 $32 = HEAP32[$0>>2]|0;
 $33 = (($32) + 72|0);
 HEAP32[$33>>2] = $34;
 $35 = HEAP32[$0>>2]|0;
 $36 = (_sqlite3GetVdbe($35)|0);
 HEAP32[$v>>2] = $36;
 $37 = HEAP32[$v>>2]|0;
 $38 = ($37|0)==(0|0);
 $39 = HEAP32[$1>>2]|0;
 $40 = ($39|0)==(0|0);
 $or$cond = $38 | $40;
 if ($or$cond) {
  STACKTOP = sp;return;
 }
 $41 = HEAP32[$1>>2]|0;
 $42 = (($41) + 32|0);
 $43 = HEAP32[$42>>2]|0;
 $44 = ($43|0)==(0);
 if ($44) {
  STACKTOP = sp;return;
 }
 $45 = HEAP32[$1>>2]|0;
 $46 = HEAP32[$45>>2]|0;
 $47 = (_sqlite3_strnicmp($46,15144,7)|0);
 $48 = ($47|0)==(0);
 if ($48) {
  STACKTOP = sp;return;
 }
 $49 = HEAP32[$db>>2]|0;
 $50 = HEAP32[$1>>2]|0;
 $51 = (($50) + 68|0);
 $52 = HEAP32[$51>>2]|0;
 $53 = (_sqlite3SchemaToIndex($49,$52)|0);
 HEAP32[$iDb>>2] = $53;
 $54 = HEAP32[$0>>2]|0;
 $55 = HEAP32[$1>>2]|0;
 $56 = HEAP32[$55>>2]|0;
 $57 = HEAP32[$iDb>>2]|0;
 $58 = HEAP32[$db>>2]|0;
 $59 = (($58) + 16|0);
 $60 = HEAP32[$59>>2]|0;
 $61 = (($60) + ($57<<4)|0);
 $62 = HEAP32[$61>>2]|0;
 $63 = (_sqlite3AuthCheck($54,28,$56,0,$62)|0);
 $64 = ($63|0)!=(0);
 if ($64) {
  STACKTOP = sp;return;
 }
 $65 = HEAP32[$0>>2]|0;
 $66 = HEAP32[$iDb>>2]|0;
 $67 = HEAP32[$1>>2]|0;
 $68 = (($67) + 32|0);
 $69 = HEAP32[$68>>2]|0;
 $70 = HEAP32[$1>>2]|0;
 $71 = HEAP32[$70>>2]|0;
 _sqlite3TableLock($65,$66,$69,0,$71);
 $72 = HEAP32[$5>>2]|0;
 $73 = (($72) + 1)|0;
 HEAP32[$5>>2] = $73;
 HEAP32[$iTabCur>>2] = $72;
 $74 = HEAP32[$5>>2]|0;
 $75 = (($74) + 1)|0;
 HEAP32[$5>>2] = $75;
 HEAP32[$iIdxCur>>2] = $74;
 $76 = HEAP32[$0>>2]|0;
 $77 = (($76) + 68|0);
 $78 = HEAP32[$77>>2]|0;
 $79 = HEAP32[$5>>2]|0;
 $80 = ($78|0)>($79|0);
 if ($80) {
  $81 = HEAP32[$0>>2]|0;
  $82 = (($81) + 68|0);
  $83 = HEAP32[$82>>2]|0;
  $87 = $83;
 } else {
  $84 = HEAP32[$5>>2]|0;
  $87 = $84;
 }
 $85 = HEAP32[$0>>2]|0;
 $86 = (($85) + 68|0);
 HEAP32[$86>>2] = $87;
 $88 = HEAP32[$0>>2]|0;
 $89 = HEAP32[$iTabCur>>2]|0;
 $90 = HEAP32[$iDb>>2]|0;
 $91 = HEAP32[$1>>2]|0;
 _sqlite3OpenTable($88,$89,$90,$91,54);
 $92 = HEAP32[$v>>2]|0;
 $93 = HEAP32[$regTabname>>2]|0;
 $94 = HEAP32[$1>>2]|0;
 $95 = HEAP32[$94>>2]|0;
 (_sqlite3VdbeAddOp4($92,97,0,$93,0,$95,0)|0);
 $96 = HEAP32[$1>>2]|0;
 $97 = (($96) + 8|0);
 $98 = HEAP32[$97>>2]|0;
 HEAP32[$pIdx>>2] = $98;
 while(1) {
  $99 = HEAP32[$pIdx>>2]|0;
  $100 = ($99|0)!=(0|0);
  $101 = HEAP32[$2>>2]|0;
  if (!($100)) {
   break;
  }
  $102 = ($101|0)!=(0|0);
  if ($102) {
   $103 = HEAP32[$2>>2]|0;
   $104 = HEAP32[$pIdx>>2]|0;
   $105 = ($103|0)!=($104|0);
   if (!($105)) {
    label = 15;
   }
  } else {
   label = 15;
  }
  do {
   if ((label|0) == 15) {
    label = 0;
    $106 = HEAP32[$pIdx>>2]|0;
    $107 = (($106) + 36|0);
    $108 = HEAP32[$107>>2]|0;
    $109 = ($108|0)==(0|0);
    if ($109) {
     HEAP8[$needTableCnt>>0] = 0;
    }
    $110 = HEAP32[$1>>2]|0;
    $111 = (($110) + 44|0);
    $112 = HEAP8[$111>>0]|0;
    $113 = $112&255;
    $114 = $113 & 32;
    $115 = ($114|0)==(0);
    if ($115) {
     label = 20;
    } else {
     $116 = HEAP32[$pIdx>>2]|0;
     $117 = (($116) + 55|0);
     $118 = HEAP8[$117>>0]|0;
     $119 = $118 & 3;
     $120 = $119&255;
     $121 = ($120|0)==(2);
     if ($121) {
      $122 = HEAP32[$pIdx>>2]|0;
      $123 = (($122) + 50|0);
      $124 = HEAP16[$123>>1]|0;
      $125 = $124&65535;
      HEAP32[$nCol>>2] = $125;
      $126 = HEAP32[$1>>2]|0;
      $127 = HEAP32[$126>>2]|0;
      HEAP32[$zIdxName>>2] = $127;
      $128 = HEAP32[$nCol>>2]|0;
      $129 = (($128) - 1)|0;
      HEAP32[$nColTest>>2] = $129;
     } else {
      label = 20;
     }
    }
    if ((label|0) == 20) {
     label = 0;
     $130 = HEAP32[$pIdx>>2]|0;
     $131 = (($130) + 52|0);
     $132 = HEAP16[$131>>1]|0;
     $133 = $132&65535;
     HEAP32[$nCol>>2] = $133;
     $134 = HEAP32[$pIdx>>2]|0;
     $135 = HEAP32[$134>>2]|0;
     HEAP32[$zIdxName>>2] = $135;
     $136 = HEAP32[$pIdx>>2]|0;
     $137 = (($136) + 55|0);
     $138 = HEAP8[$137>>0]|0;
     $139 = ($138&255) >>> 3;
     $140 = $139 & 1;
     $141 = $140&255;
     $142 = ($141|0)!=(0);
     if ($142) {
      $143 = HEAP32[$pIdx>>2]|0;
      $144 = (($143) + 50|0);
      $145 = HEAP16[$144>>1]|0;
      $146 = $145&65535;
      $$sink = $146;
     } else {
      $147 = HEAP32[$nCol>>2]|0;
      $$sink = $147;
     }
     $148 = (($$sink) - 1)|0;
     HEAP32[$nColTest>>2] = $148;
    }
    $149 = HEAP32[$v>>2]|0;
    $150 = HEAP32[$regIdxname>>2]|0;
    $151 = HEAP32[$zIdxName>>2]|0;
    (_sqlite3VdbeAddOp4($149,97,0,$150,0,$151,0)|0);
    $152 = HEAP32[$0>>2]|0;
    $153 = (($152) + 72|0);
    $154 = HEAP32[$153>>2]|0;
    $155 = HEAP32[$regPrev>>2]|0;
    $156 = HEAP32[$nColTest>>2]|0;
    $157 = (($155) + ($156))|0;
    $158 = ($154|0)>($157|0);
    if ($158) {
     $159 = HEAP32[$0>>2]|0;
     $160 = (($159) + 72|0);
     $161 = HEAP32[$160>>2]|0;
     $167 = $161;
    } else {
     $162 = HEAP32[$regPrev>>2]|0;
     $163 = HEAP32[$nColTest>>2]|0;
     $164 = (($162) + ($163))|0;
     $167 = $164;
    }
    $165 = HEAP32[$0>>2]|0;
    $166 = (($165) + 72|0);
    HEAP32[$166>>2] = $167;
    $168 = HEAP32[$v>>2]|0;
    $169 = HEAP32[$iIdxCur>>2]|0;
    $170 = HEAP32[$pIdx>>2]|0;
    $171 = (($170) + 44|0);
    $172 = HEAP32[$171>>2]|0;
    $173 = HEAP32[$iDb>>2]|0;
    (_sqlite3VdbeAddOp3($168,54,$169,$172,$173)|0);
    $174 = HEAP32[$0>>2]|0;
    $175 = HEAP32[$pIdx>>2]|0;
    _sqlite3VdbeSetP4KeyInfo($174,$175);
    $176 = HEAP32[$v>>2]|0;
    $177 = HEAP32[$nCol>>2]|0;
    $178 = HEAP32[$regStat4>>2]|0;
    $179 = (($178) + 1)|0;
    (_sqlite3VdbeAddOp2($176,25,$177,$179)|0);
    $180 = HEAP32[$v>>2]|0;
    $181 = HEAP32[$pIdx>>2]|0;
    $182 = (($181) + 50|0);
    $183 = HEAP16[$182>>1]|0;
    $184 = $183&65535;
    $185 = HEAP32[$regStat4>>2]|0;
    $186 = (($185) + 2)|0;
    (_sqlite3VdbeAddOp2($180,25,$184,$186)|0);
    $187 = HEAP32[$v>>2]|0;
    $188 = HEAP32[$regStat4>>2]|0;
    $189 = (($188) + 1)|0;
    $190 = HEAP32[$regStat4>>2]|0;
    (_sqlite3VdbeAddOp3($187,1,0,$189,$190)|0);
    $191 = HEAP32[$v>>2]|0;
    _sqlite3VdbeChangeP4($191,-1,18176,-5);
    $192 = HEAP32[$v>>2]|0;
    _sqlite3VdbeChangeP5($192,2);
    $193 = HEAP32[$v>>2]|0;
    $194 = HEAP32[$iIdxCur>>2]|0;
    $195 = (_sqlite3VdbeAddOp1($193,108,$194)|0);
    HEAP32[$addrRewind>>2] = $195;
    $196 = HEAP32[$v>>2]|0;
    $197 = HEAP32[$regChng>>2]|0;
    (_sqlite3VdbeAddOp2($196,25,0,$197)|0);
    $198 = HEAP32[$v>>2]|0;
    $199 = (_sqlite3VdbeCurrentAddr($198)|0);
    HEAP32[$addrNextRow>>2] = $199;
    $200 = HEAP32[$nColTest>>2]|0;
    $201 = ($200|0)>(0);
    if ($201) {
     $202 = HEAP32[$v>>2]|0;
     $203 = (_sqlite3VdbeMakeLabel($202)|0);
     HEAP32[$endDistinctTest>>2] = $203;
     $204 = HEAP32[$db>>2]|0;
     $205 = HEAP32[$nColTest>>2]|0;
     $206 = $205<<2;
     $207 = (_sqlite3DbMallocRaw($204,$206,0)|0);
     HEAP32[$aGotoChng>>2] = $207;
     $208 = HEAP32[$aGotoChng>>2]|0;
     $209 = ($208|0)==(0|0);
     if ($209) {
      break;
     }
     $210 = HEAP32[$v>>2]|0;
     (_sqlite3VdbeAddOp0($210,16)|0);
     $211 = HEAP32[$v>>2]|0;
     $212 = (_sqlite3VdbeCurrentAddr($211)|0);
     HEAP32[$addrNextRow>>2] = $212;
     $213 = HEAP32[$nColTest>>2]|0;
     $214 = ($213|0)==(1);
     if ($214) {
      $215 = HEAP32[$pIdx>>2]|0;
      $216 = (($215) + 50|0);
      $217 = HEAP16[$216>>1]|0;
      $218 = $217&65535;
      $219 = ($218|0)==(1);
      if ($219) {
       $220 = HEAP32[$pIdx>>2]|0;
       $221 = (($220) + 54|0);
       $222 = HEAP8[$221>>0]|0;
       $223 = $222&255;
       $224 = ($223|0)!=(0);
       if ($224) {
        $225 = HEAP32[$v>>2]|0;
        $226 = HEAP32[$regPrev>>2]|0;
        $227 = HEAP32[$endDistinctTest>>2]|0;
        (_sqlite3VdbeAddOp2($225,77,$226,$227)|0);
       }
      }
     }
     HEAP32[$i>>2] = 0;
     while(1) {
      $228 = HEAP32[$i>>2]|0;
      $229 = HEAP32[$nColTest>>2]|0;
      $230 = ($228|0)<($229|0);
      if (!($230)) {
       break;
      }
      $231 = HEAP32[$0>>2]|0;
      $232 = HEAP32[$i>>2]|0;
      $233 = HEAP32[$pIdx>>2]|0;
      $234 = (($233) + 32|0);
      $235 = HEAP32[$234>>2]|0;
      $236 = (($235) + ($232<<2)|0);
      $237 = HEAP32[$236>>2]|0;
      $238 = (_sqlite3LocateCollSeq($231,$237)|0);
      HEAP32[$pColl>>2] = $238;
      $239 = HEAP32[$v>>2]|0;
      $240 = HEAP32[$i>>2]|0;
      $241 = HEAP32[$regChng>>2]|0;
      (_sqlite3VdbeAddOp2($239,25,$240,$241)|0);
      $242 = HEAP32[$v>>2]|0;
      $243 = HEAP32[$iIdxCur>>2]|0;
      $244 = HEAP32[$i>>2]|0;
      $245 = HEAP32[$regTemp>>2]|0;
      (_sqlite3VdbeAddOp3($242,47,$243,$244,$245)|0);
      $246 = HEAP32[$v>>2]|0;
      $247 = HEAP32[$regTemp>>2]|0;
      $248 = HEAP32[$regPrev>>2]|0;
      $249 = HEAP32[$i>>2]|0;
      $250 = (($248) + ($249))|0;
      $251 = HEAP32[$pColl>>2]|0;
      $252 = (_sqlite3VdbeAddOp4($246,78,$247,0,$250,$251,-4)|0);
      $253 = HEAP32[$i>>2]|0;
      $254 = HEAP32[$aGotoChng>>2]|0;
      $255 = (($254) + ($253<<2)|0);
      HEAP32[$255>>2] = $252;
      $256 = HEAP32[$v>>2]|0;
      _sqlite3VdbeChangeP5($256,-128);
      $257 = HEAP32[$i>>2]|0;
      $258 = (($257) + 1)|0;
      HEAP32[$i>>2] = $258;
     }
     $259 = HEAP32[$v>>2]|0;
     $260 = HEAP32[$nColTest>>2]|0;
     $261 = HEAP32[$regChng>>2]|0;
     (_sqlite3VdbeAddOp2($259,25,$260,$261)|0);
     $262 = HEAP32[$v>>2]|0;
     $263 = HEAP32[$endDistinctTest>>2]|0;
     (_sqlite3VdbeAddOp2($262,16,0,$263)|0);
     $264 = HEAP32[$v>>2]|0;
     $265 = HEAP32[$addrNextRow>>2]|0;
     $266 = (($265) - 1)|0;
     _sqlite3VdbeJumpHere($264,$266);
     HEAP32[$i>>2] = 0;
     while(1) {
      $267 = HEAP32[$i>>2]|0;
      $268 = HEAP32[$nColTest>>2]|0;
      $269 = ($267|0)<($268|0);
      $270 = HEAP32[$v>>2]|0;
      if (!($269)) {
       break;
      }
      $271 = HEAP32[$i>>2]|0;
      $272 = HEAP32[$aGotoChng>>2]|0;
      $273 = (($272) + ($271<<2)|0);
      $274 = HEAP32[$273>>2]|0;
      _sqlite3VdbeJumpHere($270,$274);
      $275 = HEAP32[$v>>2]|0;
      $276 = HEAP32[$iIdxCur>>2]|0;
      $277 = HEAP32[$i>>2]|0;
      $278 = HEAP32[$regPrev>>2]|0;
      $279 = HEAP32[$i>>2]|0;
      $280 = (($278) + ($279))|0;
      (_sqlite3VdbeAddOp3($275,47,$276,$277,$280)|0);
      $281 = HEAP32[$i>>2]|0;
      $282 = (($281) + 1)|0;
      HEAP32[$i>>2] = $282;
     }
     $283 = HEAP32[$endDistinctTest>>2]|0;
     _sqlite3VdbeResolveLabel($270,$283);
     $284 = HEAP32[$db>>2]|0;
     $285 = HEAP32[$aGotoChng>>2]|0;
     _sqlite3DbFree($284,$285);
    }
    $286 = HEAP32[$v>>2]|0;
    $287 = HEAP32[$regStat4>>2]|0;
    $288 = HEAP32[$regTemp>>2]|0;
    (_sqlite3VdbeAddOp3($286,1,1,$287,$288)|0);
    $289 = HEAP32[$v>>2]|0;
    _sqlite3VdbeChangeP4($289,-1,18216,-5);
    $290 = HEAP32[$v>>2]|0;
    _sqlite3VdbeChangeP5($290,2);
    $291 = HEAP32[$v>>2]|0;
    $292 = HEAP32[$iIdxCur>>2]|0;
    $293 = HEAP32[$addrNextRow>>2]|0;
    (_sqlite3VdbeAddOp2($291,9,$292,$293)|0);
    $294 = HEAP32[$v>>2]|0;
    $295 = HEAP32[$regStat4>>2]|0;
    $296 = HEAP32[$regStat1>>2]|0;
    _callStatGet($294,$295,0,$296);
    $297 = HEAP32[$v>>2]|0;
    $298 = HEAP32[$regTabname>>2]|0;
    $299 = HEAP32[$regTemp>>2]|0;
    (_sqlite3VdbeAddOp4($297,49,$298,3,$299,18256,0)|0);
    $300 = HEAP32[$v>>2]|0;
    $301 = HEAP32[$3>>2]|0;
    $302 = HEAP32[$regNewRowid>>2]|0;
    (_sqlite3VdbeAddOp2($300,74,$301,$302)|0);
    $303 = HEAP32[$v>>2]|0;
    $304 = HEAP32[$3>>2]|0;
    $305 = HEAP32[$regTemp>>2]|0;
    $306 = HEAP32[$regNewRowid>>2]|0;
    (_sqlite3VdbeAddOp3($303,75,$304,$305,$306)|0);
    $307 = HEAP32[$v>>2]|0;
    _sqlite3VdbeChangeP5($307,8);
    $308 = HEAP32[$v>>2]|0;
    $309 = HEAP32[$addrRewind>>2]|0;
    _sqlite3VdbeJumpHere($308,$309);
   }
  } while(0);
  $310 = HEAP32[$pIdx>>2]|0;
  $311 = (($310) + 20|0);
  $312 = HEAP32[$311>>2]|0;
  HEAP32[$pIdx>>2] = $312;
 }
 $313 = ($101|0)==(0|0);
 if (!($313)) {
  STACKTOP = sp;return;
 }
 $314 = HEAP8[$needTableCnt>>0]|0;
 $315 = $314&255;
 $316 = ($315|0)!=(0);
 if (!($316)) {
  STACKTOP = sp;return;
 }
 $317 = HEAP32[$v>>2]|0;
 $318 = HEAP32[$iTabCur>>2]|0;
 $319 = HEAP32[$regStat1>>2]|0;
 (_sqlite3VdbeAddOp2($317,50,$318,$319)|0);
 $320 = HEAP32[$v>>2]|0;
 $321 = HEAP32[$regStat1>>2]|0;
 $322 = (_sqlite3VdbeAddOp1($320,46,$321)|0);
 HEAP32[$jZeroRows>>2] = $322;
 $323 = HEAP32[$v>>2]|0;
 $324 = HEAP32[$regIdxname>>2]|0;
 (_sqlite3VdbeAddOp2($323,28,0,$324)|0);
 $325 = HEAP32[$v>>2]|0;
 $326 = HEAP32[$regTabname>>2]|0;
 $327 = HEAP32[$regTemp>>2]|0;
 (_sqlite3VdbeAddOp4($325,49,$326,3,$327,18256,0)|0);
 $328 = HEAP32[$v>>2]|0;
 $329 = HEAP32[$3>>2]|0;
 $330 = HEAP32[$regNewRowid>>2]|0;
 (_sqlite3VdbeAddOp2($328,74,$329,$330)|0);
 $331 = HEAP32[$v>>2]|0;
 $332 = HEAP32[$3>>2]|0;
 $333 = HEAP32[$regTemp>>2]|0;
 $334 = HEAP32[$regNewRowid>>2]|0;
 (_sqlite3VdbeAddOp3($331,75,$332,$333,$334)|0);
 $335 = HEAP32[$v>>2]|0;
 _sqlite3VdbeChangeP5($335,8);
 $336 = HEAP32[$v>>2]|0;
 $337 = HEAP32[$jZeroRows>>2]|0;
 _sqlite3VdbeJumpHere($336,$337);
 STACKTOP = sp;return;
}
function _loadAnalysis($pParse,$iDb) {
 $pParse = $pParse|0;
 $iDb = $iDb|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $v = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $1 = sp + 4|0;
 $v = sp;
 HEAP32[$0>>2] = $pParse;
 HEAP32[$1>>2] = $iDb;
 $2 = HEAP32[$0>>2]|0;
 $3 = (_sqlite3GetVdbe($2)|0);
 HEAP32[$v>>2] = $3;
 $4 = HEAP32[$v>>2]|0;
 $5 = ($4|0)!=(0|0);
 if (!($5)) {
  STACKTOP = sp;return;
 }
 $6 = HEAP32[$v>>2]|0;
 $7 = HEAP32[$1>>2]|0;
 (_sqlite3VdbeAddOp1($6,123,$7)|0);
 STACKTOP = sp;return;
}
function _sqlite3VdbeAddOp1($p,$op,$p1) {
 $p = $p|0;
 $op = $op|0;
 $p1 = $p1|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $1 = sp + 4|0;
 $2 = sp;
 HEAP32[$0>>2] = $p;
 HEAP32[$1>>2] = $op;
 HEAP32[$2>>2] = $p1;
 $3 = HEAP32[$0>>2]|0;
 $4 = HEAP32[$1>>2]|0;
 $5 = HEAP32[$2>>2]|0;
 $6 = (_sqlite3VdbeAddOp3($3,$4,$5,0,0)|0);
 STACKTOP = sp;return ($6|0);
}
function _sqlite3TableLock($pParse,$iDb,$iTab,$isWriteLock,$zName) {
 $pParse = $pParse|0;
 $iDb = $iDb|0;
 $iTab = $iTab|0;
 $isWriteLock = $isWriteLock|0;
 $zName = $zName|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0;
 var $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $9 = 0, $i = 0, $nBytes = 0, $p = 0, $pToplevel = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $0 = sp + 28|0;
 $1 = sp + 24|0;
 $2 = sp + 20|0;
 $3 = sp + 32|0;
 $4 = sp + 16|0;
 $pToplevel = sp + 12|0;
 $i = sp + 8|0;
 $nBytes = sp + 4|0;
 $p = sp;
 HEAP32[$0>>2] = $pParse;
 HEAP32[$1>>2] = $iDb;
 HEAP32[$2>>2] = $iTab;
 HEAP8[$3>>0] = $isWriteLock;
 HEAP32[$4>>2] = $zName;
 $5 = HEAP32[$0>>2]|0;
 $6 = (($5) + 408|0);
 $7 = HEAP32[$6>>2]|0;
 $8 = ($7|0)!=(0|0);
 $9 = HEAP32[$0>>2]|0;
 if ($8) {
  $10 = (($9) + 408|0);
  $11 = HEAP32[$10>>2]|0;
  $12 = $11;
 } else {
  $12 = $9;
 }
 HEAP32[$pToplevel>>2] = $12;
 HEAP32[$i>>2] = 0;
 while(1) {
  $13 = HEAP32[$i>>2]|0;
  $14 = HEAP32[$pToplevel>>2]|0;
  $15 = (($14) + 396|0);
  $16 = HEAP32[$15>>2]|0;
  $17 = ($13|0)<($16|0);
  if (!($17)) {
   break;
  }
  $18 = HEAP32[$i>>2]|0;
  $19 = HEAP32[$pToplevel>>2]|0;
  $20 = (($19) + 400|0);
  $21 = HEAP32[$20>>2]|0;
  $22 = (($21) + ($18<<4)|0);
  HEAP32[$p>>2] = $22;
  $23 = HEAP32[$p>>2]|0;
  $24 = HEAP32[$23>>2]|0;
  $25 = HEAP32[$1>>2]|0;
  $26 = ($24|0)==($25|0);
  if ($26) {
   $27 = HEAP32[$p>>2]|0;
   $28 = (($27) + 4|0);
   $29 = HEAP32[$28>>2]|0;
   $30 = HEAP32[$2>>2]|0;
   $31 = ($29|0)==($30|0);
   if ($31) {
    label = 7;
    break;
   }
  }
  $45 = HEAP32[$i>>2]|0;
  $46 = (($45) + 1)|0;
  HEAP32[$i>>2] = $46;
 }
 if ((label|0) == 7) {
  $32 = HEAP32[$p>>2]|0;
  $33 = (($32) + 8|0);
  $34 = HEAP8[$33>>0]|0;
  $35 = $34&255;
  $36 = ($35|0)!=(0);
  if ($36) {
   $41 = 1;
  } else {
   $37 = HEAP8[$3>>0]|0;
   $38 = $37&255;
   $39 = ($38|0)!=(0);
   $41 = $39;
  }
  $40 = $41&1;
  $42 = $40&255;
  $43 = HEAP32[$p>>2]|0;
  $44 = (($43) + 8|0);
  HEAP8[$44>>0] = $42;
  STACKTOP = sp;return;
 }
 $47 = HEAP32[$pToplevel>>2]|0;
 $48 = (($47) + 396|0);
 $49 = HEAP32[$48>>2]|0;
 $50 = (($49) + 1)|0;
 $51 = $50<<4;
 HEAP32[$nBytes>>2] = $51;
 $52 = HEAP32[$pToplevel>>2]|0;
 $53 = HEAP32[$52>>2]|0;
 $54 = HEAP32[$pToplevel>>2]|0;
 $55 = (($54) + 400|0);
 $56 = HEAP32[$55>>2]|0;
 $57 = HEAP32[$nBytes>>2]|0;
 $58 = ($57|0)<(0);
 $59 = $58 << 31 >> 31;
 $60 = (_sqlite3DbReallocOrFree($53,$56,$57,$59)|0);
 $61 = HEAP32[$pToplevel>>2]|0;
 $62 = (($61) + 400|0);
 HEAP32[$62>>2] = $60;
 $63 = HEAP32[$pToplevel>>2]|0;
 $64 = (($63) + 400|0);
 $65 = HEAP32[$64>>2]|0;
 $66 = ($65|0)!=(0|0);
 $67 = HEAP32[$pToplevel>>2]|0;
 $68 = (($67) + 396|0);
 if ($66) {
  $69 = HEAP32[$68>>2]|0;
  $70 = (($69) + 1)|0;
  HEAP32[$68>>2] = $70;
  $71 = HEAP32[$pToplevel>>2]|0;
  $72 = (($71) + 400|0);
  $73 = HEAP32[$72>>2]|0;
  $74 = (($73) + ($69<<4)|0);
  HEAP32[$p>>2] = $74;
  $75 = HEAP32[$1>>2]|0;
  $76 = HEAP32[$p>>2]|0;
  HEAP32[$76>>2] = $75;
  $77 = HEAP32[$2>>2]|0;
  $78 = HEAP32[$p>>2]|0;
  $79 = (($78) + 4|0);
  HEAP32[$79>>2] = $77;
  $80 = HEAP8[$3>>0]|0;
  $81 = HEAP32[$p>>2]|0;
  $82 = (($81) + 8|0);
  HEAP8[$82>>0] = $80;
  $83 = HEAP32[$4>>2]|0;
  $84 = HEAP32[$p>>2]|0;
  $85 = (($84) + 12|0);
  HEAP32[$85>>2] = $83;
  STACKTOP = sp;return;
 } else {
  HEAP32[$68>>2] = 0;
  $86 = HEAP32[$pToplevel>>2]|0;
  $87 = HEAP32[$86>>2]|0;
  $88 = (($87) + 64|0);
  HEAP8[$88>>0] = 1;
  STACKTOP = sp;return;
 }
}
function _sqlite3OpenTable($pParse,$iCur,$iDb,$pTab,$opcode) {
 $pParse = $pParse|0;
 $iCur = $iCur|0;
 $iDb = $iDb|0;
 $pTab = $pTab|0;
 $opcode = $opcode|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $pPk = 0, $v = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 24|0;
 $1 = sp + 20|0;
 $2 = sp + 16|0;
 $3 = sp + 12|0;
 $4 = sp + 8|0;
 $v = sp + 4|0;
 $pPk = sp;
 HEAP32[$0>>2] = $pParse;
 HEAP32[$1>>2] = $iCur;
 HEAP32[$2>>2] = $iDb;
 HEAP32[$3>>2] = $pTab;
 HEAP32[$4>>2] = $opcode;
 $5 = HEAP32[$0>>2]|0;
 $6 = (_sqlite3GetVdbe($5)|0);
 HEAP32[$v>>2] = $6;
 $7 = HEAP32[$0>>2]|0;
 $8 = HEAP32[$2>>2]|0;
 $9 = HEAP32[$3>>2]|0;
 $10 = (($9) + 32|0);
 $11 = HEAP32[$10>>2]|0;
 $12 = HEAP32[$4>>2]|0;
 $13 = ($12|0)==(55);
 $14 = $13 ? 1 : 0;
 $15 = $14&255;
 $16 = HEAP32[$3>>2]|0;
 $17 = HEAP32[$16>>2]|0;
 _sqlite3TableLock($7,$8,$11,$15,$17);
 $18 = HEAP32[$3>>2]|0;
 $19 = (($18) + 44|0);
 $20 = HEAP8[$19>>0]|0;
 $21 = $20&255;
 $22 = $21 & 32;
 $23 = ($22|0)==(0);
 if ($23) {
  $24 = HEAP32[$v>>2]|0;
  $25 = HEAP32[$4>>2]|0;
  $26 = HEAP32[$1>>2]|0;
  $27 = HEAP32[$3>>2]|0;
  $28 = (($27) + 32|0);
  $29 = HEAP32[$28>>2]|0;
  $30 = HEAP32[$2>>2]|0;
  $31 = HEAP32[$3>>2]|0;
  $32 = (($31) + 38|0);
  $33 = HEAP16[$32>>1]|0;
  $34 = $33 << 16 >> 16;
  (_sqlite3VdbeAddOp4Int($24,$25,$26,$29,$30,$34)|0);
  STACKTOP = sp;return;
 } else {
  $35 = HEAP32[$3>>2]|0;
  $36 = (_sqlite3PrimaryKeyIndex($35)|0);
  HEAP32[$pPk>>2] = $36;
  $37 = HEAP32[$v>>2]|0;
  $38 = HEAP32[$4>>2]|0;
  $39 = HEAP32[$1>>2]|0;
  $40 = HEAP32[$pPk>>2]|0;
  $41 = (($40) + 44|0);
  $42 = HEAP32[$41>>2]|0;
  $43 = HEAP32[$2>>2]|0;
  (_sqlite3VdbeAddOp3($37,$38,$39,$42,$43)|0);
  $44 = HEAP32[$0>>2]|0;
  $45 = HEAP32[$pPk>>2]|0;
  _sqlite3VdbeSetP4KeyInfo($44,$45);
  STACKTOP = sp;return;
 }
}
function _sqlite3VdbeSetP4KeyInfo($pParse,$pIdx) {
 $pParse = $pParse|0;
 $pIdx = $pIdx|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $v = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $1 = sp + 4|0;
 $v = sp;
 HEAP32[$0>>2] = $pParse;
 HEAP32[$1>>2] = $pIdx;
 $2 = HEAP32[$0>>2]|0;
 $3 = (($2) + 8|0);
 $4 = HEAP32[$3>>2]|0;
 HEAP32[$v>>2] = $4;
 $5 = HEAP32[$v>>2]|0;
 $6 = HEAP32[$0>>2]|0;
 $7 = HEAP32[$1>>2]|0;
 $8 = (_sqlite3KeyInfoOfIndex($6,$7)|0);
 _sqlite3VdbeChangeP4($5,-1,$8,-6);
 STACKTOP = sp;return;
}
function _sqlite3VdbeCurrentAddr($p) {
 $p = $p|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 HEAP32[$0>>2] = $p;
 $1 = HEAP32[$0>>2]|0;
 $2 = (($1) + 32|0);
 $3 = HEAP32[$2>>2]|0;
 STACKTOP = sp;return ($3|0);
}
function _sqlite3VdbeMakeLabel($v) {
 $v = $v|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $i = 0, $p = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $p = sp + 4|0;
 $i = sp;
 HEAP32[$0>>2] = $v;
 $1 = HEAP32[$0>>2]|0;
 $2 = (($1) + 24|0);
 $3 = HEAP32[$2>>2]|0;
 HEAP32[$p>>2] = $3;
 $4 = HEAP32[$p>>2]|0;
 $5 = (($4) + 108|0);
 $6 = HEAP32[$5>>2]|0;
 $7 = (($6) + 1)|0;
 HEAP32[$5>>2] = $7;
 HEAP32[$i>>2] = $6;
 $8 = HEAP32[$i>>2]|0;
 $9 = HEAP32[$i>>2]|0;
 $10 = (($9) - 1)|0;
 $11 = $8 & $10;
 $12 = ($11|0)==(0);
 if ($12) {
  $13 = HEAP32[$p>>2]|0;
  $14 = HEAP32[$13>>2]|0;
  $15 = HEAP32[$p>>2]|0;
  $16 = (($15) + 112|0);
  $17 = HEAP32[$16>>2]|0;
  $18 = HEAP32[$i>>2]|0;
  $19 = $18<<1;
  $20 = (($19) + 1)|0;
  $21 = $20<<2;
  $22 = (_sqlite3DbReallocOrFree($14,$17,$21,0)|0);
  $23 = HEAP32[$p>>2]|0;
  $24 = (($23) + 112|0);
  HEAP32[$24>>2] = $22;
 }
 $25 = HEAP32[$p>>2]|0;
 $26 = (($25) + 112|0);
 $27 = HEAP32[$26>>2]|0;
 $28 = ($27|0)!=(0|0);
 if (!($28)) {
  $34 = HEAP32[$i>>2]|0;
  $35 = (-1 - ($34))|0;
  STACKTOP = sp;return ($35|0);
 }
 $29 = HEAP32[$i>>2]|0;
 $30 = HEAP32[$p>>2]|0;
 $31 = (($30) + 112|0);
 $32 = HEAP32[$31>>2]|0;
 $33 = (($32) + ($29<<2)|0);
 HEAP32[$33>>2] = -1;
 $34 = HEAP32[$i>>2]|0;
 $35 = (-1 - ($34))|0;
 STACKTOP = sp;return ($35|0);
}
function _sqlite3LocateCollSeq($pParse,$zName) {
 $pParse = $pParse|0;
 $zName = $zName|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $db = 0, $enc = 0, $initbusy = 0, $pColl = 0, label = 0;
 var sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 12|0;
 $1 = sp + 8|0;
 $db = sp + 4|0;
 $enc = sp + 17|0;
 $initbusy = sp + 16|0;
 $pColl = sp;
 HEAP32[$0>>2] = $pParse;
 HEAP32[$1>>2] = $zName;
 $2 = HEAP32[$0>>2]|0;
 $3 = HEAP32[$2>>2]|0;
 HEAP32[$db>>2] = $3;
 $4 = HEAP32[$db>>2]|0;
 $5 = (($4) + 16|0);
 $6 = HEAP32[$5>>2]|0;
 $7 = (($6) + 12|0);
 $8 = HEAP32[$7>>2]|0;
 $9 = (($8) + 77|0);
 $10 = HEAP8[$9>>0]|0;
 HEAP8[$enc>>0] = $10;
 $11 = HEAP32[$db>>2]|0;
 $12 = (($11) + 140|0);
 $13 = (($12) + 5|0);
 $14 = HEAP8[$13>>0]|0;
 HEAP8[$initbusy>>0] = $14;
 $15 = HEAP32[$db>>2]|0;
 $16 = HEAP8[$enc>>0]|0;
 $17 = HEAP32[$1>>2]|0;
 $18 = HEAP8[$initbusy>>0]|0;
 $19 = $18&255;
 $20 = (_sqlite3FindCollSeq($15,$16,$17,$19)|0);
 HEAP32[$pColl>>2] = $20;
 $21 = HEAP8[$initbusy>>0]|0;
 $22 = ($21<<24>>24)!=(0);
 if ($22) {
  $34 = HEAP32[$pColl>>2]|0;
  STACKTOP = sp;return ($34|0);
 }
 $23 = HEAP32[$pColl>>2]|0;
 $24 = ($23|0)!=(0|0);
 if ($24) {
  $25 = HEAP32[$pColl>>2]|0;
  $26 = (($25) + 12|0);
  $27 = HEAP32[$26>>2]|0;
  $28 = ($27|0)!=(0|0);
  if ($28) {
   $34 = HEAP32[$pColl>>2]|0;
   STACKTOP = sp;return ($34|0);
  }
 }
 $29 = HEAP32[$0>>2]|0;
 $30 = HEAP8[$enc>>0]|0;
 $31 = HEAP32[$pColl>>2]|0;
 $32 = HEAP32[$1>>2]|0;
 $33 = (_sqlite3GetCollSeq($29,$30,$31,$32)|0);
 HEAP32[$pColl>>2] = $33;
 $34 = HEAP32[$pColl>>2]|0;
 STACKTOP = sp;return ($34|0);
}
function _sqlite3VdbeResolveLabel($v,$x) {
 $v = $v|0;
 $x = $x|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $j = 0, $p = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $1 = sp + 8|0;
 $p = sp + 4|0;
 $j = sp;
 HEAP32[$0>>2] = $v;
 HEAP32[$1>>2] = $x;
 $2 = HEAP32[$0>>2]|0;
 $3 = (($2) + 24|0);
 $4 = HEAP32[$3>>2]|0;
 HEAP32[$p>>2] = $4;
 $5 = HEAP32[$1>>2]|0;
 $6 = (-1 - ($5))|0;
 HEAP32[$j>>2] = $6;
 $7 = HEAP32[$j>>2]|0;
 $8 = ($7|0)>=(0);
 if ($8) {
  $9 = HEAP32[$p>>2]|0;
  $10 = (($9) + 112|0);
  $11 = HEAP32[$10>>2]|0;
  $12 = ($11|0)!=(0|0);
  if ($12) {
   $13 = HEAP32[$0>>2]|0;
   $14 = (($13) + 32|0);
   $15 = HEAP32[$14>>2]|0;
   $16 = HEAP32[$j>>2]|0;
   $17 = HEAP32[$p>>2]|0;
   $18 = (($17) + 112|0);
   $19 = HEAP32[$18>>2]|0;
   $20 = (($19) + ($16<<2)|0);
   HEAP32[$20>>2] = $15;
  }
 }
 $21 = HEAP32[$0>>2]|0;
 $22 = (($21) + 32|0);
 $23 = HEAP32[$22>>2]|0;
 $24 = (($23) - 1)|0;
 $25 = HEAP32[$p>>2]|0;
 $26 = (($25) + 88|0);
 HEAP32[$26>>2] = $24;
 STACKTOP = sp;return;
}
function _callStatGet($v,$regStat4,$iParam,$regOut) {
 $v = $v|0;
 $regStat4 = $regStat4|0;
 $iParam = $iParam|0;
 $regOut = $regOut|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $1 = sp + 8|0;
 $2 = sp + 4|0;
 $3 = sp;
 HEAP32[$0>>2] = $v;
 HEAP32[$1>>2] = $regStat4;
 HEAP32[$2>>2] = $iParam;
 HEAP32[$3>>2] = $regOut;
 $4 = HEAP32[$0>>2]|0;
 $5 = HEAP32[$1>>2]|0;
 $6 = HEAP32[$3>>2]|0;
 (_sqlite3VdbeAddOp3($4,1,0,$5,$6)|0);
 $7 = HEAP32[$0>>2]|0;
 _sqlite3VdbeChangeP4($7,-1,18264,-5);
 $8 = HEAP32[$0>>2]|0;
 _sqlite3VdbeChangeP5($8,1);
 STACKTOP = sp;return;
}
function _statGet($context,$argc,$argv) {
 $context = $context|0;
 $argc = $argc|0;
 $argv = $argv|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0;
 var $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $9 = 0, $i = 0, $iVal = 0, $nDistinct = 0, $p = 0, $vararg_buffer1 = 0, $z = 0, $zRet = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 64|0;
 $vararg_buffer1 = sp + 16|0;
 $0 = sp + 48|0;
 $1 = sp + 44|0;
 $2 = sp + 40|0;
 $p = sp + 36|0;
 $z = sp + 32|0;
 $i = sp + 28|0;
 $zRet = sp + 24|0;
 $nDistinct = sp + 8|0;
 $iVal = sp;
 HEAP32[$0>>2] = $context;
 HEAP32[$1>>2] = $argc;
 HEAP32[$2>>2] = $argv;
 $3 = HEAP32[$2>>2]|0;
 $4 = HEAP32[$3>>2]|0;
 $5 = (_sqlite3_value_blob($4)|0);
 HEAP32[$p>>2] = $5;
 $6 = HEAP32[$p>>2]|0;
 $7 = (($6) + 12|0);
 $8 = HEAP32[$7>>2]|0;
 $9 = (($8) + 1)|0;
 $10 = ($9*25)|0;
 $11 = ($10|0)<(0);
 $12 = $11 << 31 >> 31;
 $13 = (_sqlite3MallocZero($10,$12)|0);
 HEAP32[$zRet>>2] = $13;
 $14 = HEAP32[$zRet>>2]|0;
 $15 = ($14|0)==(0|0);
 if ($15) {
  $16 = HEAP32[$0>>2]|0;
  _sqlite3_result_error_nomem($16);
  STACKTOP = sp;return;
 }
 $17 = HEAP32[$zRet>>2]|0;
 $18 = HEAP32[$p>>2]|0;
 $19 = HEAP32[$18>>2]|0;
 $20 = $vararg_buffer1;
 $21 = $20;
 HEAP32[$21>>2] = $19;
 $22 = (($20) + 4)|0;
 $23 = $22;
 HEAP32[$23>>2] = 0;
 (_sqlite3_snprintf(24,$17,18320,$vararg_buffer1)|0);
 $24 = HEAP32[$zRet>>2]|0;
 $25 = HEAP32[$zRet>>2]|0;
 $26 = (_sqlite3Strlen30($25)|0);
 $27 = (($24) + ($26)|0);
 HEAP32[$z>>2] = $27;
 HEAP32[$i>>2] = 0;
 while(1) {
  $28 = HEAP32[$i>>2]|0;
  $29 = HEAP32[$p>>2]|0;
  $30 = (($29) + 12|0);
  $31 = HEAP32[$30>>2]|0;
  $32 = ($28|0)<($31|0);
  if (!($32)) {
   break;
  }
  $33 = HEAP32[$i>>2]|0;
  $34 = HEAP32[$p>>2]|0;
  $35 = (($34) + 20|0);
  $36 = (($35) + 4|0);
  $37 = HEAP32[$36>>2]|0;
  $38 = (($37) + ($33<<2)|0);
  $39 = HEAP32[$38>>2]|0;
  $40 = (($39) + 1)|0;
  $41 = $nDistinct;
  $42 = $41;
  HEAP32[$42>>2] = $40;
  $43 = (($41) + 4)|0;
  $44 = $43;
  HEAP32[$44>>2] = 0;
  $45 = HEAP32[$p>>2]|0;
  $46 = HEAP32[$45>>2]|0;
  $47 = $nDistinct;
  $48 = $47;
  $49 = HEAP32[$48>>2]|0;
  $50 = (($47) + 4)|0;
  $51 = $50;
  $52 = HEAP32[$51>>2]|0;
  $53 = (_i64Add(($46|0),0,($49|0),($52|0))|0);
  $54 = tempRet0;
  $55 = (_i64Subtract(($53|0),($54|0),1,0)|0);
  $56 = tempRet0;
  $57 = $nDistinct;
  $58 = $57;
  $59 = HEAP32[$58>>2]|0;
  $60 = (($57) + 4)|0;
  $61 = $60;
  $62 = HEAP32[$61>>2]|0;
  $63 = (___udivdi3(($55|0),($56|0),($59|0),($62|0))|0);
  $64 = tempRet0;
  $65 = $iVal;
  $66 = $65;
  HEAP32[$66>>2] = $63;
  $67 = (($65) + 4)|0;
  $68 = $67;
  HEAP32[$68>>2] = $64;
  $69 = HEAP32[$z>>2]|0;
  $70 = $iVal;
  $71 = $70;
  $72 = HEAP32[$71>>2]|0;
  $73 = (($70) + 4)|0;
  $74 = $73;
  $75 = HEAP32[$74>>2]|0;
  $76 = $vararg_buffer1;
  $77 = $76;
  HEAP32[$77>>2] = $72;
  $78 = (($76) + 4)|0;
  $79 = $78;
  HEAP32[$79>>2] = $75;
  (_sqlite3_snprintf(24,$69,18328,$vararg_buffer1)|0);
  $80 = HEAP32[$z>>2]|0;
  $81 = (_sqlite3Strlen30($80)|0);
  $82 = HEAP32[$z>>2]|0;
  $83 = (($82) + ($81)|0);
  HEAP32[$z>>2] = $83;
  $84 = HEAP32[$i>>2]|0;
  $85 = (($84) + 1)|0;
  HEAP32[$i>>2] = $85;
 }
 $86 = HEAP32[$0>>2]|0;
 $87 = HEAP32[$zRet>>2]|0;
 _sqlite3_result_text($86,$87,-1,13);
 STACKTOP = sp;return;
}
function _statPush($context,$argc,$argv) {
 $context = $context|0;
 $argc = $argc|0;
 $argv = $argv|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $7 = 0, $8 = 0, $9 = 0, $i = 0, $iChng = 0, $p = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 20|0;
 $1 = sp + 16|0;
 $2 = sp + 12|0;
 $i = sp + 8|0;
 $p = sp + 4|0;
 $iChng = sp;
 HEAP32[$0>>2] = $context;
 HEAP32[$1>>2] = $argc;
 HEAP32[$2>>2] = $argv;
 $3 = HEAP32[$2>>2]|0;
 $4 = HEAP32[$3>>2]|0;
 $5 = (_sqlite3_value_blob($4)|0);
 HEAP32[$p>>2] = $5;
 $6 = HEAP32[$2>>2]|0;
 $7 = (($6) + 4|0);
 $8 = HEAP32[$7>>2]|0;
 $9 = (_sqlite3_value_int($8)|0);
 HEAP32[$iChng>>2] = $9;
 $10 = HEAP32[$p>>2]|0;
 $11 = HEAP32[$10>>2]|0;
 $12 = ($11|0)==(0);
 if ($12) {
  HEAP32[$i>>2] = 0;
  while(1) {
   $13 = HEAP32[$i>>2]|0;
   $14 = HEAP32[$p>>2]|0;
   $15 = (($14) + 8|0);
   $16 = HEAP32[$15>>2]|0;
   $17 = ($13|0)<($16|0);
   if (!($17)) {
    break;
   }
   $18 = HEAP32[$i>>2]|0;
   $19 = HEAP32[$p>>2]|0;
   $20 = (($19) + 20|0);
   $21 = HEAP32[$20>>2]|0;
   $22 = (($21) + ($18<<2)|0);
   HEAP32[$22>>2] = 1;
   $23 = HEAP32[$i>>2]|0;
   $24 = (($23) + 1)|0;
   HEAP32[$i>>2] = $24;
  }
  $60 = HEAP32[$p>>2]|0;
  $61 = HEAP32[$60>>2]|0;
  $62 = (($61) + 1)|0;
  HEAP32[$60>>2] = $62;
  STACKTOP = sp;return;
 }
 $25 = HEAP32[$p>>2]|0;
 $26 = HEAP32[$iChng>>2]|0;
 _samplePushPrevious($25,$26);
 HEAP32[$i>>2] = 0;
 while(1) {
  $27 = HEAP32[$i>>2]|0;
  $28 = HEAP32[$iChng>>2]|0;
  $29 = ($27|0)<($28|0);
  if (!($29)) {
   break;
  }
  $30 = HEAP32[$i>>2]|0;
  $31 = HEAP32[$p>>2]|0;
  $32 = (($31) + 20|0);
  $33 = HEAP32[$32>>2]|0;
  $34 = (($33) + ($30<<2)|0);
  $35 = HEAP32[$34>>2]|0;
  $36 = (($35) + 1)|0;
  HEAP32[$34>>2] = $36;
  $37 = HEAP32[$i>>2]|0;
  $38 = (($37) + 1)|0;
  HEAP32[$i>>2] = $38;
 }
 $39 = HEAP32[$iChng>>2]|0;
 HEAP32[$i>>2] = $39;
 while(1) {
  $40 = HEAP32[$i>>2]|0;
  $41 = HEAP32[$p>>2]|0;
  $42 = (($41) + 8|0);
  $43 = HEAP32[$42>>2]|0;
  $44 = ($40|0)<($43|0);
  if (!($44)) {
   break;
  }
  $45 = HEAP32[$i>>2]|0;
  $46 = HEAP32[$p>>2]|0;
  $47 = (($46) + 20|0);
  $48 = (($47) + 4|0);
  $49 = HEAP32[$48>>2]|0;
  $50 = (($49) + ($45<<2)|0);
  $51 = HEAP32[$50>>2]|0;
  $52 = (($51) + 1)|0;
  HEAP32[$50>>2] = $52;
  $53 = HEAP32[$i>>2]|0;
  $54 = HEAP32[$p>>2]|0;
  $55 = (($54) + 20|0);
  $56 = HEAP32[$55>>2]|0;
  $57 = (($56) + ($53<<2)|0);
  HEAP32[$57>>2] = 1;
  $58 = HEAP32[$i>>2]|0;
  $59 = (($58) + 1)|0;
  HEAP32[$i>>2] = $59;
 }
 $60 = HEAP32[$p>>2]|0;
 $61 = HEAP32[$60>>2]|0;
 $62 = (($61) + 1)|0;
 HEAP32[$60>>2] = $62;
 STACKTOP = sp;return;
}
function _samplePushPrevious($p,$iChng) {
 $p = $p|0;
 $iChng = $iChng|0;
 var $0 = 0, $1 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $1 = sp;
 HEAP32[$0>>2] = $p;
 HEAP32[$1>>2] = $iChng;
 STACKTOP = sp;return;
}
function _statInit($context,$argc,$argv) {
 $context = $context|0;
 $argc = $argc|0;
 $argv = $argv|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $db = 0, $n = 0, $nCol = 0, $nColUp = 0, $nKeyCol = 0, $p = 0;
 var label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $0 = sp + 32|0;
 $1 = sp + 28|0;
 $2 = sp + 24|0;
 $p = sp + 20|0;
 $nCol = sp + 16|0;
 $nKeyCol = sp + 12|0;
 $nColUp = sp + 8|0;
 $n = sp + 4|0;
 $db = sp;
 HEAP32[$0>>2] = $context;
 HEAP32[$1>>2] = $argc;
 HEAP32[$2>>2] = $argv;
 $3 = HEAP32[$2>>2]|0;
 $4 = HEAP32[$3>>2]|0;
 $5 = (_sqlite3_value_int($4)|0);
 HEAP32[$nCol>>2] = $5;
 $6 = HEAP32[$nCol>>2]|0;
 $7 = (($6) + 1)|0;
 $8 = $7 & -2;
 HEAP32[$nColUp>>2] = $8;
 $9 = HEAP32[$2>>2]|0;
 $10 = (($9) + 4|0);
 $11 = HEAP32[$10>>2]|0;
 $12 = (_sqlite3_value_int($11)|0);
 HEAP32[$nKeyCol>>2] = $12;
 $13 = HEAP32[$nColUp>>2]|0;
 $14 = $13<<2;
 $15 = (56 + ($14))|0;
 $16 = HEAP32[$nColUp>>2]|0;
 $17 = $16<<2;
 $18 = (($15) + ($17))|0;
 HEAP32[$n>>2] = $18;
 $19 = HEAP32[$0>>2]|0;
 $20 = (_sqlite3_context_db_handle($19)|0);
 HEAP32[$db>>2] = $20;
 $21 = HEAP32[$db>>2]|0;
 $22 = HEAP32[$n>>2]|0;
 $23 = ($22|0)<(0);
 $24 = $23 << 31 >> 31;
 $25 = (_sqlite3DbMallocZero($21,$22,$24)|0);
 HEAP32[$p>>2] = $25;
 $26 = HEAP32[$p>>2]|0;
 $27 = ($26|0)==(0|0);
 if ($27) {
  $28 = HEAP32[$0>>2]|0;
  _sqlite3_result_error_nomem($28);
  STACKTOP = sp;return;
 } else {
  $29 = HEAP32[$db>>2]|0;
  $30 = HEAP32[$p>>2]|0;
  $31 = (($30) + 52|0);
  HEAP32[$31>>2] = $29;
  $32 = HEAP32[$p>>2]|0;
  HEAP32[$32>>2] = 0;
  $33 = HEAP32[$nCol>>2]|0;
  $34 = HEAP32[$p>>2]|0;
  $35 = (($34) + 8|0);
  HEAP32[$35>>2] = $33;
  $36 = HEAP32[$nKeyCol>>2]|0;
  $37 = HEAP32[$p>>2]|0;
  $38 = (($37) + 12|0);
  HEAP32[$38>>2] = $36;
  $39 = HEAP32[$p>>2]|0;
  $40 = (($39) + 56|0);
  $41 = HEAP32[$p>>2]|0;
  $42 = (($41) + 20|0);
  $43 = (($42) + 4|0);
  HEAP32[$43>>2] = $40;
  $44 = HEAP32[$nColUp>>2]|0;
  $45 = HEAP32[$p>>2]|0;
  $46 = (($45) + 20|0);
  $47 = (($46) + 4|0);
  $48 = HEAP32[$47>>2]|0;
  $49 = (($48) + ($44<<2)|0);
  $50 = HEAP32[$p>>2]|0;
  $51 = (($50) + 20|0);
  HEAP32[$51>>2] = $49;
  $52 = HEAP32[$0>>2]|0;
  $53 = HEAP32[$p>>2]|0;
  _sqlite3_result_blob($52,$53,56,16);
  STACKTOP = sp;return;
 }
}
function _stat4Destructor($pOld) {
 $pOld = $pOld|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $p = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $p = sp;
 HEAP32[$0>>2] = $pOld;
 $1 = HEAP32[$0>>2]|0;
 HEAP32[$p>>2] = $1;
 $2 = HEAP32[$p>>2]|0;
 $3 = (($2) + 52|0);
 $4 = HEAP32[$3>>2]|0;
 $5 = HEAP32[$p>>2]|0;
 _sqlite3DbFree($4,$5);
 STACKTOP = sp;return;
}
function _sqlite3KeyInfoOfIndex($pParse,$pIdx) {
 $pParse = $pParse|0;
 $pIdx = $pIdx|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0;
 var $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $i = 0;
 var $nCol = 0, $nKey = 0, $pKey = 0, $zColl = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 28|0;
 $1 = sp + 24|0;
 $2 = sp + 20|0;
 $i = sp + 16|0;
 $nCol = sp + 12|0;
 $nKey = sp + 8|0;
 $pKey = sp + 4|0;
 $zColl = sp;
 HEAP32[$1>>2] = $pParse;
 HEAP32[$2>>2] = $pIdx;
 $3 = HEAP32[$1>>2]|0;
 $4 = (($3) + 64|0);
 $5 = HEAP32[$4>>2]|0;
 $6 = ($5|0)!=(0);
 if ($6) {
  HEAP32[$0>>2] = 0;
  $98 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($98|0);
 }
 $7 = HEAP32[$2>>2]|0;
 $8 = (($7) + 40|0);
 $9 = HEAP32[$8>>2]|0;
 $10 = ($9|0)!=(0|0);
 if ($10) {
  $11 = HEAP32[$2>>2]|0;
  $12 = (($11) + 40|0);
  $13 = HEAP32[$12>>2]|0;
  $14 = (($13) + 12|0);
  $15 = HEAP32[$14>>2]|0;
  $16 = HEAP32[$1>>2]|0;
  $17 = HEAP32[$16>>2]|0;
  $18 = ($15|0)!=($17|0);
  if ($18) {
   $19 = HEAP32[$2>>2]|0;
   $20 = (($19) + 40|0);
   $21 = HEAP32[$20>>2]|0;
   _sqlite3KeyInfoUnref($21);
   $22 = HEAP32[$2>>2]|0;
   $23 = (($22) + 40|0);
   HEAP32[$23>>2] = 0;
  }
 }
 $24 = HEAP32[$2>>2]|0;
 $25 = (($24) + 40|0);
 $26 = HEAP32[$25>>2]|0;
 $27 = ($26|0)==(0|0);
 do {
  if ($27) {
   $28 = HEAP32[$2>>2]|0;
   $29 = (($28) + 52|0);
   $30 = HEAP16[$29>>1]|0;
   $31 = $30&65535;
   HEAP32[$nCol>>2] = $31;
   $32 = HEAP32[$2>>2]|0;
   $33 = (($32) + 50|0);
   $34 = HEAP16[$33>>1]|0;
   $35 = $34&65535;
   HEAP32[$nKey>>2] = $35;
   $36 = HEAP32[$2>>2]|0;
   $37 = (($36) + 55|0);
   $38 = HEAP8[$37>>0]|0;
   $39 = ($38&255) >>> 3;
   $40 = $39 & 1;
   $41 = $40&255;
   $42 = ($41|0)!=(0);
   $43 = HEAP32[$1>>2]|0;
   $44 = HEAP32[$43>>2]|0;
   if ($42) {
    $45 = HEAP32[$nKey>>2]|0;
    $46 = HEAP32[$nCol>>2]|0;
    $47 = HEAP32[$nKey>>2]|0;
    $48 = (($46) - ($47))|0;
    $49 = (_sqlite3KeyInfoAlloc($44,$45,$48)|0);
    HEAP32[$pKey>>2] = $49;
   } else {
    $50 = HEAP32[$nCol>>2]|0;
    $51 = (_sqlite3KeyInfoAlloc($44,$50,0)|0);
    HEAP32[$pKey>>2] = $51;
   }
   $52 = HEAP32[$pKey>>2]|0;
   $53 = ($52|0)!=(0|0);
   if ($53) {
    HEAP32[$i>>2] = 0;
    while(1) {
     $54 = HEAP32[$i>>2]|0;
     $55 = HEAP32[$nCol>>2]|0;
     $56 = ($54|0)<($55|0);
     if (!($56)) {
      break;
     }
     $57 = HEAP32[$i>>2]|0;
     $58 = HEAP32[$2>>2]|0;
     $59 = (($58) + 32|0);
     $60 = HEAP32[$59>>2]|0;
     $61 = (($60) + ($57<<2)|0);
     $62 = HEAP32[$61>>2]|0;
     HEAP32[$zColl>>2] = $62;
     $63 = HEAP32[$zColl>>2]|0;
     $64 = (_strcmp($63,3320)|0);
     $65 = ($64|0)==(0);
     if ($65) {
      $73 = 0;
     } else {
      $66 = HEAP32[$1>>2]|0;
      $67 = HEAP32[$zColl>>2]|0;
      $68 = (_sqlite3LocateCollSeq($66,$67)|0);
      $73 = $68;
     }
     $69 = HEAP32[$i>>2]|0;
     $70 = HEAP32[$pKey>>2]|0;
     $71 = (($70) + 20|0);
     $72 = (($71) + ($69<<2)|0);
     HEAP32[$72>>2] = $73;
     $74 = HEAP32[$i>>2]|0;
     $75 = HEAP32[$2>>2]|0;
     $76 = (($75) + 28|0);
     $77 = HEAP32[$76>>2]|0;
     $78 = (($77) + ($74)|0);
     $79 = HEAP8[$78>>0]|0;
     $80 = HEAP32[$i>>2]|0;
     $81 = HEAP32[$pKey>>2]|0;
     $82 = (($81) + 16|0);
     $83 = HEAP32[$82>>2]|0;
     $84 = (($83) + ($80)|0);
     HEAP8[$84>>0] = $79;
     $85 = HEAP32[$i>>2]|0;
     $86 = (($85) + 1)|0;
     HEAP32[$i>>2] = $86;
    }
    $87 = HEAP32[$1>>2]|0;
    $88 = (($87) + 64|0);
    $89 = HEAP32[$88>>2]|0;
    $90 = ($89|0)!=(0);
    $91 = HEAP32[$pKey>>2]|0;
    if ($90) {
     _sqlite3KeyInfoUnref($91);
     break;
    } else {
     $92 = HEAP32[$2>>2]|0;
     $93 = (($92) + 40|0);
     HEAP32[$93>>2] = $91;
     break;
    }
   }
  }
 } while(0);
 $94 = HEAP32[$2>>2]|0;
 $95 = (($94) + 40|0);
 $96 = HEAP32[$95>>2]|0;
 $97 = (_sqlite3KeyInfoRef($96)|0);
 HEAP32[$0>>2] = $97;
 $98 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($98|0);
}
function _sqlite3KeyInfoAlloc($db,$N,$X) {
 $db = $db|0;
 $N = $N|0;
 $X = $X|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $5 = 0, $6 = 0;
 var $7 = 0, $8 = 0, $9 = 0, $p = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $1 = sp + 8|0;
 $2 = sp + 4|0;
 $p = sp;
 HEAP32[$0>>2] = $db;
 HEAP32[$1>>2] = $N;
 HEAP32[$2>>2] = $X;
 $3 = HEAP32[$1>>2]|0;
 $4 = HEAP32[$2>>2]|0;
 $5 = (($3) + ($4))|0;
 $6 = ($5*5)|0;
 $7 = (24 + ($6))|0;
 $8 = (_sqlite3DbMallocZero(0,$7,0)|0);
 HEAP32[$p>>2] = $8;
 $9 = HEAP32[$p>>2]|0;
 $10 = ($9|0)!=(0|0);
 if ($10) {
  $11 = HEAP32[$1>>2]|0;
  $12 = HEAP32[$2>>2]|0;
  $13 = (($11) + ($12))|0;
  $14 = HEAP32[$p>>2]|0;
  $15 = (($14) + 20|0);
  $16 = (($15) + ($13<<2)|0);
  $17 = HEAP32[$p>>2]|0;
  $18 = (($17) + 16|0);
  HEAP32[$18>>2] = $16;
  $19 = HEAP32[$1>>2]|0;
  $20 = $19&65535;
  $21 = HEAP32[$p>>2]|0;
  $22 = (($21) + 6|0);
  HEAP16[$22>>1] = $20;
  $23 = HEAP32[$2>>2]|0;
  $24 = $23&65535;
  $25 = HEAP32[$p>>2]|0;
  $26 = (($25) + 8|0);
  HEAP16[$26>>1] = $24;
  $27 = HEAP32[$0>>2]|0;
  $28 = (($27) + 16|0);
  $29 = HEAP32[$28>>2]|0;
  $30 = (($29) + 12|0);
  $31 = HEAP32[$30>>2]|0;
  $32 = (($31) + 77|0);
  $33 = HEAP8[$32>>0]|0;
  $34 = HEAP32[$p>>2]|0;
  $35 = (($34) + 4|0);
  HEAP8[$35>>0] = $33;
  $36 = HEAP32[$0>>2]|0;
  $37 = HEAP32[$p>>2]|0;
  $38 = (($37) + 12|0);
  HEAP32[$38>>2] = $36;
  $39 = HEAP32[$p>>2]|0;
  HEAP32[$39>>2] = 1;
  $42 = HEAP32[$p>>2]|0;
  STACKTOP = sp;return ($42|0);
 } else {
  $40 = HEAP32[$0>>2]|0;
  $41 = (($40) + 64|0);
  HEAP8[$41>>0] = 1;
  $42 = HEAP32[$p>>2]|0;
  STACKTOP = sp;return ($42|0);
 }
 return 0|0;
}
function _sqlite3KeyInfoRef($p) {
 $p = $p|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 HEAP32[$0>>2] = $p;
 $1 = HEAP32[$0>>2]|0;
 $2 = ($1|0)!=(0|0);
 if ($2) {
  $3 = HEAP32[$0>>2]|0;
  $4 = HEAP32[$3>>2]|0;
  $5 = (($4) + 1)|0;
  HEAP32[$3>>2] = $5;
 }
 $6 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($6|0);
}
function _sqlite3PrimaryKeyIndex($pTab) {
 $pTab = $pTab|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $p = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $p = sp;
 HEAP32[$0>>2] = $pTab;
 $1 = HEAP32[$0>>2]|0;
 $2 = (($1) + 8|0);
 $3 = HEAP32[$2>>2]|0;
 HEAP32[$p>>2] = $3;
 while(1) {
  $4 = HEAP32[$p>>2]|0;
  $5 = ($4|0)!=(0|0);
  if (!($5)) {
   label = 5;
   break;
  }
  $6 = HEAP32[$p>>2]|0;
  $7 = (($6) + 55|0);
  $8 = HEAP8[$7>>0]|0;
  $9 = $8 & 3;
  $10 = $9&255;
  $11 = ($10|0)==(2);
  if ($11) {
   label = 5;
   break;
  }
  $12 = HEAP32[$p>>2]|0;
  $13 = (($12) + 20|0);
  $14 = HEAP32[$13>>2]|0;
  HEAP32[$p>>2] = $14;
 }
 if ((label|0) == 5) {
  $15 = HEAP32[$p>>2]|0;
  STACKTOP = sp;return ($15|0);
 }
 return 0|0;
}
function _sqlite3Init($db,$pzErrMsg) {
 $db = $db|0;
 $pzErrMsg = $pzErrMsg|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $8 = 0, $9 = 0, $commit_internal = 0, $i = 0, $or$cond = 0, $or$cond3 = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 16|0;
 $1 = sp + 12|0;
 $i = sp + 8|0;
 $rc = sp + 4|0;
 $commit_internal = sp;
 HEAP32[$0>>2] = $db;
 HEAP32[$1>>2] = $pzErrMsg;
 $2 = HEAP32[$0>>2]|0;
 $3 = (($2) + 24|0);
 $4 = HEAP32[$3>>2]|0;
 $5 = $4 & 2;
 $6 = ($5|0)!=(0);
 $7 = $6 ^ 1;
 $8 = $7&1;
 HEAP32[$commit_internal>>2] = $8;
 HEAP32[$rc>>2] = 0;
 $9 = HEAP32[$0>>2]|0;
 $10 = (($9) + 140|0);
 $11 = (($10) + 5|0);
 HEAP8[$11>>0] = 1;
 HEAP32[$i>>2] = 0;
 while(1) {
  $12 = HEAP32[$rc>>2]|0;
  $13 = ($12|0)==(0);
  if (!($13)) {
   break;
  }
  $14 = HEAP32[$i>>2]|0;
  $15 = HEAP32[$0>>2]|0;
  $16 = (($15) + 20|0);
  $17 = HEAP32[$16>>2]|0;
  $18 = ($14|0)<($17|0);
  if (!($18)) {
   break;
  }
  $19 = HEAP32[$i>>2]|0;
  $20 = HEAP32[$0>>2]|0;
  $21 = (($20) + 16|0);
  $22 = HEAP32[$21>>2]|0;
  $23 = (($22) + ($19<<4)|0);
  $24 = (($23) + 12|0);
  $25 = HEAP32[$24>>2]|0;
  $26 = (($25) + 78|0);
  $27 = HEAP16[$26>>1]|0;
  $28 = $27&65535;
  $29 = $28 & 1;
  $30 = ($29|0)==(1);
  $31 = HEAP32[$i>>2]|0;
  $32 = ($31|0)==(1);
  $or$cond = $30 | $32;
  if (!($or$cond)) {
   $33 = HEAP32[$0>>2]|0;
   $34 = HEAP32[$i>>2]|0;
   $35 = HEAP32[$1>>2]|0;
   $36 = (_sqlite3InitOne($33,$34,$35)|0);
   HEAP32[$rc>>2] = $36;
   $37 = HEAP32[$rc>>2]|0;
   $38 = ($37|0)!=(0);
   if ($38) {
    $39 = HEAP32[$0>>2]|0;
    $40 = HEAP32[$i>>2]|0;
    _sqlite3ResetOneSchema($39,$40);
   }
  }
  $41 = HEAP32[$i>>2]|0;
  $42 = (($41) + 1)|0;
  HEAP32[$i>>2] = $42;
 }
 $43 = HEAP32[$rc>>2]|0;
 $44 = ($43|0)==(0);
 if ($44) {
  $45 = HEAP32[$0>>2]|0;
  $46 = (($45) + 16|0);
  $47 = HEAP32[$46>>2]|0;
  $48 = (($47) + 16|0);
  $49 = (($48) + 12|0);
  $50 = HEAP32[$49>>2]|0;
  $51 = (($50) + 78|0);
  $52 = HEAP16[$51>>1]|0;
  $53 = $52&65535;
  $54 = $53 & 1;
  $55 = ($54|0)==(1);
  if (!($55)) {
   $56 = HEAP32[$0>>2]|0;
   $57 = HEAP32[$1>>2]|0;
   $58 = (_sqlite3InitOne($56,1,$57)|0);
   HEAP32[$rc>>2] = $58;
   $59 = HEAP32[$rc>>2]|0;
   $60 = ($59|0)!=(0);
   if ($60) {
    $61 = HEAP32[$0>>2]|0;
    _sqlite3ResetOneSchema($61,1);
   }
  }
 }
 $62 = HEAP32[$0>>2]|0;
 $63 = (($62) + 140|0);
 $64 = (($63) + 5|0);
 HEAP8[$64>>0] = 0;
 $65 = HEAP32[$rc>>2]|0;
 $66 = ($65|0)==(0);
 $67 = HEAP32[$commit_internal>>2]|0;
 $68 = ($67|0)!=(0);
 $or$cond3 = $66 & $68;
 if (!($or$cond3)) {
  $70 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($70|0);
 }
 $69 = HEAP32[$0>>2]|0;
 _sqlite3CommitInternalChanges($69);
 $70 = HEAP32[$rc>>2]|0;
 STACKTOP = sp;return ($70|0);
}
function _sqlite3InitOne($db,$iDb,$pzErrMsg) {
 $db = $db|0;
 $iDb = $iDb|0;
 $pzErrMsg = $pzErrMsg|0;
 var $$ = 0, $$1 = 0, $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0;
 var $114 = 0, $115 = 0, $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0;
 var $132 = 0, $133 = 0, $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0;
 var $150 = 0, $151 = 0, $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0;
 var $169 = 0, $17 = 0, $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0;
 var $187 = 0, $188 = 0, $189 = 0, $19 = 0, $190 = 0, $191 = 0, $192 = 0, $193 = 0, $194 = 0, $195 = 0, $196 = 0, $197 = 0, $198 = 0, $199 = 0, $2 = 0, $20 = 0, $200 = 0, $201 = 0, $202 = 0, $203 = 0;
 var $204 = 0, $205 = 0, $206 = 0, $207 = 0, $208 = 0, $209 = 0, $21 = 0, $210 = 0, $211 = 0, $212 = 0, $213 = 0, $214 = 0, $215 = 0, $216 = 0, $217 = 0, $218 = 0, $219 = 0, $22 = 0, $220 = 0, $221 = 0;
 var $222 = 0, $223 = 0, $224 = 0, $225 = 0, $226 = 0, $227 = 0, $228 = 0, $229 = 0, $23 = 0, $230 = 0, $231 = 0, $232 = 0, $233 = 0, $234 = 0, $235 = 0, $236 = 0, $237 = 0, $238 = 0, $239 = 0, $24 = 0;
 var $240 = 0, $241 = 0, $242 = 0, $243 = 0, $244 = 0, $245 = 0, $246 = 0, $247 = 0, $248 = 0, $249 = 0, $25 = 0, $250 = 0, $251 = 0, $252 = 0, $253 = 0, $254 = 0, $255 = 0, $256 = 0, $257 = 0, $258 = 0;
 var $259 = 0, $26 = 0, $260 = 0, $261 = 0, $262 = 0, $263 = 0, $264 = 0, $265 = 0, $266 = 0, $267 = 0, $268 = 0, $269 = 0, $27 = 0, $270 = 0, $271 = 0, $272 = 0, $273 = 0, $274 = 0, $275 = 0, $276 = 0;
 var $277 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0;
 var $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0;
 var $azArg = 0, $encoding = 0, $i = 0, $initData = 0, $meta = 0, $openedTransaction = 0, $or$cond = 0, $pDb = 0, $pTab = 0, $rc = 0, $size = 0, $vararg_buffer7 = 0, $vararg_ptr10 = 0, $xAuth = 0, $zMasterName = 0, $zMasterSchema = 0, $zSql = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 128|0;
 $vararg_buffer7 = sp;
 $0 = sp + 120|0;
 $1 = sp + 116|0;
 $2 = sp + 112|0;
 $3 = sp + 108|0;
 $rc = sp + 104|0;
 $i = sp + 100|0;
 $size = sp + 96|0;
 $pTab = sp + 92|0;
 $pDb = sp + 88|0;
 $azArg = sp + 72|0;
 $meta = sp + 48|0;
 $initData = sp + 32|0;
 $zMasterSchema = sp + 24|0;
 $zMasterName = sp + 20|0;
 $openedTransaction = sp + 16|0;
 $encoding = sp + 124|0;
 $zSql = sp + 12|0;
 $xAuth = sp + 8|0;
 HEAP32[$1>>2] = $db;
 HEAP32[$2>>2] = $iDb;
 HEAP32[$3>>2] = $pzErrMsg;
 HEAP32[$openedTransaction>>2] = 0;
 $4 = HEAP32[$2>>2]|0;
 $5 = ($4|0)==(1);
 if ($5) {
  HEAP32[$zMasterSchema>>2] = 18616;
 } else {
  HEAP32[$zMasterSchema>>2] = 18512;
 }
 $6 = HEAP32[$2>>2]|0;
 $7 = ($6|0)==(1);
 $8 = $7 ? 14904 : 14928;
 HEAP32[$zMasterName>>2] = $8;
 $9 = HEAP32[$zMasterName>>2]|0;
 HEAP32[$azArg>>2] = $9;
 $10 = (($azArg) + 4|0);
 HEAP32[$10>>2] = 18736;
 $11 = HEAP32[$zMasterSchema>>2]|0;
 $12 = (($azArg) + 8|0);
 HEAP32[$12>>2] = $11;
 $13 = (($azArg) + 12|0);
 HEAP32[$13>>2] = 0;
 $14 = HEAP32[$1>>2]|0;
 HEAP32[$initData>>2] = $14;
 $15 = HEAP32[$2>>2]|0;
 $16 = (($initData) + 8|0);
 HEAP32[$16>>2] = $15;
 $17 = (($initData) + 12|0);
 HEAP32[$17>>2] = 0;
 $18 = HEAP32[$3>>2]|0;
 $19 = (($initData) + 4|0);
 HEAP32[$19>>2] = $18;
 (_sqlite3InitCallback($initData,3,$azArg,0)|0);
 $20 = (($initData) + 12|0);
 $21 = HEAP32[$20>>2]|0;
 $22 = ($21|0)!=(0);
 if ($22) {
  $23 = (($initData) + 12|0);
  $24 = HEAP32[$23>>2]|0;
  HEAP32[$rc>>2] = $24;
 } else {
  $25 = HEAP32[$1>>2]|0;
  $26 = HEAP32[$zMasterName>>2]|0;
  $27 = HEAP32[$2>>2]|0;
  $28 = HEAP32[$1>>2]|0;
  $29 = (($28) + 16|0);
  $30 = HEAP32[$29>>2]|0;
  $31 = (($30) + ($27<<4)|0);
  $32 = HEAP32[$31>>2]|0;
  $33 = (_sqlite3FindTable($25,$26,$32)|0);
  HEAP32[$pTab>>2] = $33;
  $34 = HEAP32[$pTab>>2]|0;
  $35 = ($34|0)!=(0|0);
  if ($35) {
   $36 = HEAP32[$pTab>>2]|0;
   $37 = (($36) + 44|0);
   $38 = HEAP8[$37>>0]|0;
   $39 = $38&255;
   $40 = $39 | 1;
   $41 = $40&255;
   HEAP8[$37>>0] = $41;
  }
  $42 = HEAP32[$2>>2]|0;
  $43 = HEAP32[$1>>2]|0;
  $44 = (($43) + 16|0);
  $45 = HEAP32[$44>>2]|0;
  $46 = (($45) + ($42<<4)|0);
  HEAP32[$pDb>>2] = $46;
  $47 = HEAP32[$pDb>>2]|0;
  $48 = (($47) + 4|0);
  $49 = HEAP32[$48>>2]|0;
  $50 = ($49|0)==(0|0);
  if ($50) {
   $51 = HEAP32[$2>>2]|0;
   $52 = ($51|0)==(1);
   if ($52) {
    $53 = HEAP32[$1>>2]|0;
    $54 = (($53) + 16|0);
    $55 = HEAP32[$54>>2]|0;
    $56 = (($55) + 16|0);
    $57 = (($56) + 12|0);
    $58 = HEAP32[$57>>2]|0;
    $59 = (($58) + 78|0);
    $60 = HEAP16[$59>>1]|0;
    $61 = $60&65535;
    $62 = $61 | 1;
    $63 = $62&65535;
    HEAP16[$59>>1] = $63;
   }
   HEAP32[$0>>2] = 0;
   $277 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($277|0);
  }
  $64 = HEAP32[$pDb>>2]|0;
  $65 = (($64) + 4|0);
  $66 = HEAP32[$65>>2]|0;
  _sqlite3BtreeEnter($66);
  $67 = HEAP32[$pDb>>2]|0;
  $68 = (($67) + 4|0);
  $69 = HEAP32[$68>>2]|0;
  $70 = (_sqlite3BtreeIsInReadTrans($69)|0);
  $71 = ($70|0)!=(0);
  do {
   if ($71) {
    label = 16;
   } else {
    $72 = HEAP32[$pDb>>2]|0;
    $73 = (($72) + 4|0);
    $74 = HEAP32[$73>>2]|0;
    $75 = (_sqlite3BtreeBeginTrans($74,0)|0);
    HEAP32[$rc>>2] = $75;
    $76 = HEAP32[$rc>>2]|0;
    $77 = ($76|0)!=(0);
    if ($77) {
     $78 = HEAP32[$3>>2]|0;
     $79 = HEAP32[$1>>2]|0;
     $80 = HEAP32[$rc>>2]|0;
     $81 = (_sqlite3ErrStr($80)|0);
     HEAP32[$vararg_buffer7>>2] = $81;
     _sqlite3SetString($78,$79,1200,$vararg_buffer7);
     break;
    } else {
     HEAP32[$openedTransaction>>2] = 1;
     label = 16;
     break;
    }
   }
  } while(0);
  L22: do {
   if ((label|0) == 16) {
    HEAP32[$i>>2] = 0;
    while(1) {
     $82 = HEAP32[$i>>2]|0;
     $83 = ($82|0)<(5);
     if (!($83)) {
      break;
     }
     $84 = HEAP32[$pDb>>2]|0;
     $85 = (($84) + 4|0);
     $86 = HEAP32[$85>>2]|0;
     $87 = HEAP32[$i>>2]|0;
     $88 = (($87) + 1)|0;
     $89 = HEAP32[$i>>2]|0;
     $90 = (($meta) + ($89<<2)|0);
     _sqlite3BtreeGetMeta($86,$88,$90);
     $91 = HEAP32[$i>>2]|0;
     $92 = (($91) + 1)|0;
     HEAP32[$i>>2] = $92;
    }
    $93 = HEAP32[$meta>>2]|0;
    $94 = HEAP32[$pDb>>2]|0;
    $95 = (($94) + 12|0);
    $96 = HEAP32[$95>>2]|0;
    HEAP32[$96>>2] = $93;
    $97 = (($meta) + 16|0);
    $98 = HEAP32[$97>>2]|0;
    $99 = ($98|0)!=(0);
    $100 = HEAP32[$2>>2]|0;
    do {
     if ($99) {
      $101 = ($100|0)==(0);
      $102 = (($meta) + 16|0);
      $103 = HEAP32[$102>>2]|0;
      if ($101) {
       $104 = $103&255;
       $105 = $104&255;
       $106 = $105 & 3;
       $107 = $106&255;
       HEAP8[$encoding>>0] = $107;
       $108 = HEAP8[$encoding>>0]|0;
       $109 = $108&255;
       $110 = ($109|0)==(0);
       $$ = $110 ? 1 : $107;
       HEAP8[$encoding>>0] = $$;
       $111 = HEAP8[$encoding>>0]|0;
       $112 = HEAP32[$1>>2]|0;
       $113 = (($112) + 16|0);
       $114 = HEAP32[$113>>2]|0;
       $115 = (($114) + 12|0);
       $116 = HEAP32[$115>>2]|0;
       $117 = (($116) + 77|0);
       HEAP8[$117>>0] = $111;
       break;
      }
      $118 = HEAP32[$1>>2]|0;
      $119 = (($118) + 16|0);
      $120 = HEAP32[$119>>2]|0;
      $121 = (($120) + 12|0);
      $122 = HEAP32[$121>>2]|0;
      $123 = (($122) + 77|0);
      $124 = HEAP8[$123>>0]|0;
      $125 = $124&255;
      $126 = ($103|0)!=($125|0);
      if ($126) {
       $127 = HEAP32[$3>>2]|0;
       $128 = HEAP32[$1>>2]|0;
       _sqlite3SetString($127,$128,18744,$vararg_buffer7);
       HEAP32[$rc>>2] = 1;
       break L22;
      }
     } else {
      $129 = HEAP32[$1>>2]|0;
      $130 = (($129) + 16|0);
      $131 = HEAP32[$130>>2]|0;
      $132 = (($131) + ($100<<4)|0);
      $133 = (($132) + 12|0);
      $134 = HEAP32[$133>>2]|0;
      $135 = (($134) + 78|0);
      $136 = HEAP16[$135>>1]|0;
      $137 = $136&65535;
      $138 = $137 | 4;
      $139 = $138&65535;
      HEAP16[$135>>1] = $139;
     }
    } while(0);
    $140 = HEAP32[$1>>2]|0;
    $141 = (($140) + 16|0);
    $142 = HEAP32[$141>>2]|0;
    $143 = (($142) + 12|0);
    $144 = HEAP32[$143>>2]|0;
    $145 = (($144) + 77|0);
    $146 = HEAP8[$145>>0]|0;
    $147 = HEAP32[$pDb>>2]|0;
    $148 = (($147) + 12|0);
    $149 = HEAP32[$148>>2]|0;
    $150 = (($149) + 77|0);
    HEAP8[$150>>0] = $146;
    $151 = HEAP32[$pDb>>2]|0;
    $152 = (($151) + 12|0);
    $153 = HEAP32[$152>>2]|0;
    $154 = (($153) + 80|0);
    $155 = HEAP32[$154>>2]|0;
    $156 = ($155|0)==(0);
    if ($156) {
     $157 = (($meta) + 8|0);
     $158 = HEAP32[$157>>2]|0;
     $159 = (_sqlite3AbsInt32($158)|0);
     HEAP32[$size>>2] = $159;
     $160 = HEAP32[$size>>2]|0;
     $161 = ($160|0)==(0);
     $$1 = $161 ? 2000 : $159;
     HEAP32[$size>>2] = $$1;
     $162 = HEAP32[$size>>2]|0;
     $163 = HEAP32[$pDb>>2]|0;
     $164 = (($163) + 12|0);
     $165 = HEAP32[$164>>2]|0;
     $166 = (($165) + 80|0);
     HEAP32[$166>>2] = $162;
     $167 = HEAP32[$pDb>>2]|0;
     $168 = (($167) + 4|0);
     $169 = HEAP32[$168>>2]|0;
     $170 = HEAP32[$pDb>>2]|0;
     $171 = (($170) + 12|0);
     $172 = HEAP32[$171>>2]|0;
     $173 = (($172) + 80|0);
     $174 = HEAP32[$173>>2]|0;
     (_sqlite3BtreeSetCacheSize($169,$174)|0);
    }
    $175 = (($meta) + 4|0);
    $176 = HEAP32[$175>>2]|0;
    $177 = $176&255;
    $178 = HEAP32[$pDb>>2]|0;
    $179 = (($178) + 12|0);
    $180 = HEAP32[$179>>2]|0;
    $181 = (($180) + 76|0);
    HEAP8[$181>>0] = $177;
    $182 = HEAP32[$pDb>>2]|0;
    $183 = (($182) + 12|0);
    $184 = HEAP32[$183>>2]|0;
    $185 = (($184) + 76|0);
    $186 = HEAP8[$185>>0]|0;
    $187 = $186&255;
    $188 = ($187|0)==(0);
    if ($188) {
     $189 = HEAP32[$pDb>>2]|0;
     $190 = (($189) + 12|0);
     $191 = HEAP32[$190>>2]|0;
     $192 = (($191) + 76|0);
     HEAP8[$192>>0] = 1;
    }
    $193 = HEAP32[$pDb>>2]|0;
    $194 = (($193) + 12|0);
    $195 = HEAP32[$194>>2]|0;
    $196 = (($195) + 76|0);
    $197 = HEAP8[$196>>0]|0;
    $198 = $197&255;
    $199 = ($198|0)>(4);
    if ($199) {
     $200 = HEAP32[$3>>2]|0;
     $201 = HEAP32[$1>>2]|0;
     _sqlite3SetString($200,$201,18816,$vararg_buffer7);
     HEAP32[$rc>>2] = 1;
     break;
    }
    $202 = HEAP32[$2>>2]|0;
    $203 = ($202|0)==(0);
    if ($203) {
     $204 = (($meta) + 4|0);
     $205 = HEAP32[$204>>2]|0;
     $206 = ($205|0)>=(4);
     if ($206) {
      $207 = HEAP32[$1>>2]|0;
      $208 = (($207) + 24|0);
      $209 = HEAP32[$208>>2]|0;
      $210 = $209 & -32769;
      HEAP32[$208>>2] = $210;
     }
    }
    $211 = HEAP32[$1>>2]|0;
    $212 = HEAP32[$2>>2]|0;
    $213 = HEAP32[$1>>2]|0;
    $214 = (($213) + 16|0);
    $215 = HEAP32[$214>>2]|0;
    $216 = (($215) + ($212<<4)|0);
    $217 = HEAP32[$216>>2]|0;
    $218 = HEAP32[$zMasterName>>2]|0;
    HEAP32[$vararg_buffer7>>2] = $217;
    $vararg_ptr10 = (($vararg_buffer7) + 4|0);
    HEAP32[$vararg_ptr10>>2] = $218;
    $219 = (_sqlite3MPrintf($211,18840,$vararg_buffer7)|0);
    HEAP32[$zSql>>2] = $219;
    $220 = HEAP32[$1>>2]|0;
    $221 = (($220) + 284|0);
    $222 = HEAP32[$221>>2]|0;
    HEAP32[$xAuth>>2] = $222;
    $223 = HEAP32[$1>>2]|0;
    $224 = (($223) + 284|0);
    HEAP32[$224>>2] = 0;
    $225 = HEAP32[$1>>2]|0;
    $226 = HEAP32[$zSql>>2]|0;
    $227 = (_sqlite3_exec($225,$226,6,$initData,0)|0);
    HEAP32[$rc>>2] = $227;
    $228 = HEAP32[$xAuth>>2]|0;
    $229 = HEAP32[$1>>2]|0;
    $230 = (($229) + 284|0);
    HEAP32[$230>>2] = $228;
    $231 = HEAP32[$rc>>2]|0;
    $232 = ($231|0)==(0);
    if ($232) {
     $233 = (($initData) + 12|0);
     $234 = HEAP32[$233>>2]|0;
     HEAP32[$rc>>2] = $234;
    }
    $235 = HEAP32[$1>>2]|0;
    $236 = HEAP32[$zSql>>2]|0;
    _sqlite3DbFree($235,$236);
    $237 = HEAP32[$rc>>2]|0;
    $238 = ($237|0)==(0);
    if ($238) {
     $239 = HEAP32[$1>>2]|0;
     $240 = HEAP32[$2>>2]|0;
     (_sqlite3AnalysisLoad($239,$240)|0);
    }
    $241 = HEAP32[$1>>2]|0;
    $242 = (($241) + 64|0);
    $243 = HEAP8[$242>>0]|0;
    $244 = ($243<<24>>24)!=(0);
    if ($244) {
     HEAP32[$rc>>2] = 7;
     $245 = HEAP32[$1>>2]|0;
     _sqlite3ResetAllSchemasOfConnection($245);
    }
    $246 = HEAP32[$rc>>2]|0;
    $247 = ($246|0)==(0);
    if (!($247)) {
     $248 = HEAP32[$1>>2]|0;
     $249 = (($248) + 24|0);
     $250 = HEAP32[$249>>2]|0;
     $251 = $250 & 65536;
     $252 = ($251|0)!=(0);
     if (!($252)) {
      break;
     }
    }
    $253 = HEAP32[$2>>2]|0;
    $254 = HEAP32[$1>>2]|0;
    $255 = (($254) + 16|0);
    $256 = HEAP32[$255>>2]|0;
    $257 = (($256) + ($253<<4)|0);
    $258 = (($257) + 12|0);
    $259 = HEAP32[$258>>2]|0;
    $260 = (($259) + 78|0);
    $261 = HEAP16[$260>>1]|0;
    $262 = $261&65535;
    $263 = $262 | 1;
    $264 = $263&65535;
    HEAP16[$260>>1] = $264;
    HEAP32[$rc>>2] = 0;
   }
  } while(0);
  $265 = HEAP32[$openedTransaction>>2]|0;
  $266 = ($265|0)!=(0);
  if ($266) {
   $267 = HEAP32[$pDb>>2]|0;
   $268 = (($267) + 4|0);
   $269 = HEAP32[$268>>2]|0;
   (_sqlite3BtreeCommit($269)|0);
  }
 }
 $270 = HEAP32[$rc>>2]|0;
 $271 = ($270|0)==(7);
 $272 = HEAP32[$rc>>2]|0;
 $273 = ($272|0)==(3082);
 $or$cond = $271 | $273;
 if ($or$cond) {
  $274 = HEAP32[$1>>2]|0;
  $275 = (($274) + 64|0);
  HEAP8[$275>>0] = 1;
 }
 $276 = HEAP32[$rc>>2]|0;
 HEAP32[$0>>2] = $276;
 $277 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($277|0);
}
function _sqlite3ResetOneSchema($db,$iDb) {
 $db = $db|0;
 $iDb = $iDb|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $pDb = 0;
 var label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $1 = sp + 4|0;
 $pDb = sp;
 HEAP32[$0>>2] = $db;
 HEAP32[$1>>2] = $iDb;
 $2 = HEAP32[$1>>2]|0;
 $3 = HEAP32[$0>>2]|0;
 $4 = (($3) + 16|0);
 $5 = HEAP32[$4>>2]|0;
 $6 = (($5) + ($2<<4)|0);
 HEAP32[$pDb>>2] = $6;
 $7 = HEAP32[$pDb>>2]|0;
 $8 = (($7) + 12|0);
 $9 = HEAP32[$8>>2]|0;
 _sqlite3SchemaClear($9);
 $10 = HEAP32[$1>>2]|0;
 $11 = ($10|0)!=(1);
 if (!($11)) {
  STACKTOP = sp;return;
 }
 $12 = HEAP32[$0>>2]|0;
 $13 = (($12) + 16|0);
 $14 = HEAP32[$13>>2]|0;
 $15 = (($14) + 16|0);
 HEAP32[$pDb>>2] = $15;
 $16 = HEAP32[$pDb>>2]|0;
 $17 = (($16) + 12|0);
 $18 = HEAP32[$17>>2]|0;
 _sqlite3SchemaClear($18);
 STACKTOP = sp;return;
}
function _sqlite3CommitInternalChanges($db) {
 $db = $db|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 HEAP32[$0>>2] = $db;
 $1 = HEAP32[$0>>2]|0;
 $2 = (($1) + 24|0);
 $3 = HEAP32[$2>>2]|0;
 $4 = $3 & -3;
 HEAP32[$2>>2] = $4;
 STACKTOP = sp;return;
}
function _sqlite3InitCallback($pInit,$argc,$argv,$NotUsed) {
 $pInit = $pInit|0;
 $argc = $argc|0;
 $argv = $argv|0;
 $NotUsed = $NotUsed|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0;
 var $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0;
 var $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0;
 var $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0;
 var $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0;
 var $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $db = 0, $iDb = 0, $pData = 0, $pIndex = 0, $pStmt = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $0 = sp + 40|0;
 $1 = sp + 36|0;
 $2 = sp + 32|0;
 $3 = sp + 28|0;
 $4 = sp + 24|0;
 $pData = sp + 20|0;
 $db = sp + 16|0;
 $iDb = sp + 12|0;
 $rc = sp + 8|0;
 $pStmt = sp + 4|0;
 $pIndex = sp;
 HEAP32[$1>>2] = $pInit;
 HEAP32[$2>>2] = $argc;
 HEAP32[$3>>2] = $argv;
 HEAP32[$4>>2] = $NotUsed;
 $5 = HEAP32[$1>>2]|0;
 HEAP32[$pData>>2] = $5;
 $6 = HEAP32[$pData>>2]|0;
 $7 = HEAP32[$6>>2]|0;
 HEAP32[$db>>2] = $7;
 $8 = HEAP32[$pData>>2]|0;
 $9 = (($8) + 8|0);
 $10 = HEAP32[$9>>2]|0;
 HEAP32[$iDb>>2] = $10;
 $11 = HEAP32[$iDb>>2]|0;
 $12 = HEAP32[$db>>2]|0;
 $13 = (($12) + 16|0);
 $14 = HEAP32[$13>>2]|0;
 $15 = (($14) + ($11<<4)|0);
 $16 = (($15) + 12|0);
 $17 = HEAP32[$16>>2]|0;
 $18 = (($17) + 78|0);
 $19 = HEAP16[$18>>1]|0;
 $20 = $19&65535;
 $21 = $20 & -5;
 $22 = $21&65535;
 HEAP16[$18>>1] = $22;
 $23 = HEAP32[$db>>2]|0;
 $24 = (($23) + 64|0);
 $25 = HEAP8[$24>>0]|0;
 $26 = ($25<<24>>24)!=(0);
 if ($26) {
  $27 = HEAP32[$pData>>2]|0;
  $28 = HEAP32[$3>>2]|0;
  $29 = HEAP32[$28>>2]|0;
  _corruptSchema($27,$29,0);
  HEAP32[$0>>2] = 1;
  $124 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($124|0);
 }
 $30 = HEAP32[$3>>2]|0;
 $31 = ($30|0)==(0|0);
 if ($31) {
  HEAP32[$0>>2] = 0;
  $124 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($124|0);
 }
 $32 = HEAP32[$3>>2]|0;
 $33 = (($32) + 4|0);
 $34 = HEAP32[$33>>2]|0;
 $35 = ($34|0)==(0|0);
 do {
  if ($35) {
   $36 = HEAP32[$pData>>2]|0;
   $37 = HEAP32[$3>>2]|0;
   $38 = HEAP32[$37>>2]|0;
   _corruptSchema($36,$38,0);
  } else {
   $39 = HEAP32[$3>>2]|0;
   $40 = (($39) + 8|0);
   $41 = HEAP32[$40>>2]|0;
   $42 = ($41|0)!=(0|0);
   if ($42) {
    $43 = HEAP32[$3>>2]|0;
    $44 = (($43) + 8|0);
    $45 = HEAP32[$44>>2]|0;
    $46 = HEAP8[$45>>0]|0;
    $47 = $46 << 24 >> 24;
    $48 = ($47|0)!=(0);
    if ($48) {
     $49 = HEAP32[$iDb>>2]|0;
     $50 = $49&255;
     $51 = HEAP32[$db>>2]|0;
     $52 = (($51) + 140|0);
     $53 = (($52) + 4|0);
     HEAP8[$53>>0] = $50;
     $54 = HEAP32[$3>>2]|0;
     $55 = (($54) + 4|0);
     $56 = HEAP32[$55>>2]|0;
     $57 = (_sqlite3Atoi($56)|0);
     $58 = HEAP32[$db>>2]|0;
     $59 = (($58) + 140|0);
     HEAP32[$59>>2] = $57;
     $60 = HEAP32[$db>>2]|0;
     $61 = (($60) + 140|0);
     $62 = (($61) + 6|0);
     HEAP8[$62>>0] = 0;
     $63 = HEAP32[$db>>2]|0;
     $64 = HEAP32[$3>>2]|0;
     $65 = (($64) + 8|0);
     $66 = HEAP32[$65>>2]|0;
     (_sqlite3_prepare($63,$66,-1,$pStmt,0)|0);
     $67 = HEAP32[$db>>2]|0;
     $68 = (($67) + 52|0);
     $69 = HEAP32[$68>>2]|0;
     HEAP32[$rc>>2] = $69;
     $70 = HEAP32[$db>>2]|0;
     $71 = (($70) + 140|0);
     $72 = (($71) + 4|0);
     HEAP8[$72>>0] = 0;
     $73 = HEAP32[$rc>>2]|0;
     $74 = (0)!=($73|0);
     do {
      if ($74) {
       $75 = HEAP32[$db>>2]|0;
       $76 = (($75) + 140|0);
       $77 = (($76) + 6|0);
       $78 = HEAP8[$77>>0]|0;
       $79 = ($78<<24>>24)!=(0);
       if (!($79)) {
        $80 = HEAP32[$rc>>2]|0;
        $81 = HEAP32[$pData>>2]|0;
        $82 = (($81) + 12|0);
        HEAP32[$82>>2] = $80;
        $83 = HEAP32[$rc>>2]|0;
        $84 = ($83|0)==(7);
        if ($84) {
         $85 = HEAP32[$db>>2]|0;
         $86 = (($85) + 64|0);
         HEAP8[$86>>0] = 1;
         break;
        }
        $87 = HEAP32[$rc>>2]|0;
        $88 = ($87|0)!=(9);
        if ($88) {
         $89 = HEAP32[$rc>>2]|0;
         $90 = $89 & 255;
         $91 = ($90|0)!=(6);
         if ($91) {
          $92 = HEAP32[$pData>>2]|0;
          $93 = HEAP32[$3>>2]|0;
          $94 = HEAP32[$93>>2]|0;
          $95 = HEAP32[$db>>2]|0;
          $96 = (_sqlite3_errmsg($95)|0);
          _corruptSchema($92,$94,$96);
         }
        }
       }
      }
     } while(0);
     $97 = HEAP32[$pStmt>>2]|0;
     (_sqlite3_finalize($97)|0);
     break;
    }
   }
   $98 = HEAP32[$3>>2]|0;
   $99 = HEAP32[$98>>2]|0;
   $100 = ($99|0)==(0|0);
   if ($100) {
    $101 = HEAP32[$pData>>2]|0;
    _corruptSchema($101,0,0);
    break;
   }
   $102 = HEAP32[$db>>2]|0;
   $103 = HEAP32[$3>>2]|0;
   $104 = HEAP32[$103>>2]|0;
   $105 = HEAP32[$iDb>>2]|0;
   $106 = HEAP32[$db>>2]|0;
   $107 = (($106) + 16|0);
   $108 = HEAP32[$107>>2]|0;
   $109 = (($108) + ($105<<4)|0);
   $110 = HEAP32[$109>>2]|0;
   $111 = (_sqlite3FindIndex($102,$104,$110)|0);
   HEAP32[$pIndex>>2] = $111;
   $112 = HEAP32[$pIndex>>2]|0;
   $113 = ($112|0)==(0|0);
   if (!($113)) {
    $114 = HEAP32[$3>>2]|0;
    $115 = (($114) + 4|0);
    $116 = HEAP32[$115>>2]|0;
    $117 = HEAP32[$pIndex>>2]|0;
    $118 = (($117) + 44|0);
    $119 = (_sqlite3GetInt32($116,$118)|0);
    $120 = ($119|0)==(0);
    if ($120) {
     $121 = HEAP32[$pData>>2]|0;
     $122 = HEAP32[$3>>2]|0;
     $123 = HEAP32[$122>>2]|0;
     _corruptSchema($121,$123,18992);
    }
   }
  }
 } while(0);
 HEAP32[$0>>2] = 0;
 $124 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($124|0);
}
function _sqlite3AbsInt32($x) {
 $x = $x|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $1 = sp;
 HEAP32[$1>>2] = $x;
 $2 = HEAP32[$1>>2]|0;
 $3 = ($2|0)>=(0);
 $4 = HEAP32[$1>>2]|0;
 do {
  if ($3) {
   HEAP32[$0>>2] = $4;
  } else {
   $5 = ($4|0)==(-2147483648);
   if ($5) {
    HEAP32[$0>>2] = 2147483647;
    break;
   } else {
    $6 = HEAP32[$1>>2]|0;
    $7 = (0 - ($6))|0;
    HEAP32[$0>>2] = $7;
    break;
   }
  }
 } while(0);
 $8 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($8|0);
}
function _sqlite3BtreeSetCacheSize($p,$mxPage) {
 $p = $p|0;
 $mxPage = $mxPage|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $pBt = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $1 = sp + 4|0;
 $pBt = sp;
 HEAP32[$0>>2] = $p;
 HEAP32[$1>>2] = $mxPage;
 $2 = HEAP32[$0>>2]|0;
 $3 = (($2) + 4|0);
 $4 = HEAP32[$3>>2]|0;
 HEAP32[$pBt>>2] = $4;
 $5 = HEAP32[$0>>2]|0;
 _sqlite3BtreeEnter($5);
 $6 = HEAP32[$pBt>>2]|0;
 $7 = HEAP32[$6>>2]|0;
 $8 = HEAP32[$1>>2]|0;
 _sqlite3PagerSetCachesize($7,$8);
 STACKTOP = sp;return 0;
}
function _sqlite3AnalysisLoad($db,$iDb) {
 $db = $db|0;
 $iDb = $iDb|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $i = 0, $pIdx = 0, $rc = 0, $sInfo = 0, $vararg_buffer = 0, $zSql = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $vararg_buffer = sp;
 $0 = sp + 40|0;
 $1 = sp + 36|0;
 $2 = sp + 32|0;
 $sInfo = sp + 24|0;
 $i = sp + 16|0;
 $zSql = sp + 12|0;
 $rc = sp + 8|0;
 $pIdx = sp + 4|0;
 HEAP32[$1>>2] = $db;
 HEAP32[$2>>2] = $iDb;
 $3 = HEAP32[$2>>2]|0;
 $4 = HEAP32[$1>>2]|0;
 $5 = (($4) + 16|0);
 $6 = HEAP32[$5>>2]|0;
 $7 = (($6) + ($3<<4)|0);
 $8 = (($7) + 12|0);
 $9 = HEAP32[$8>>2]|0;
 $10 = (($9) + 24|0);
 $11 = (($10) + 8|0);
 $12 = HEAP32[$11>>2]|0;
 HEAP32[$i>>2] = $12;
 while(1) {
  $13 = HEAP32[$i>>2]|0;
  $14 = ($13|0)!=(0|0);
  if (!($14)) {
   break;
  }
  $15 = HEAP32[$i>>2]|0;
  $16 = (($15) + 8|0);
  $17 = HEAP32[$16>>2]|0;
  HEAP32[$pIdx>>2] = $17;
  $18 = HEAP32[$pIdx>>2]|0;
  _sqlite3DefaultRowEst($18);
  $19 = HEAP32[$i>>2]|0;
  $20 = HEAP32[$19>>2]|0;
  HEAP32[$i>>2] = $20;
 }
 $21 = HEAP32[$1>>2]|0;
 HEAP32[$sInfo>>2] = $21;
 $22 = HEAP32[$2>>2]|0;
 $23 = HEAP32[$1>>2]|0;
 $24 = (($23) + 16|0);
 $25 = HEAP32[$24>>2]|0;
 $26 = (($25) + ($22<<4)|0);
 $27 = HEAP32[$26>>2]|0;
 $28 = (($sInfo) + 4|0);
 HEAP32[$28>>2] = $27;
 $29 = HEAP32[$1>>2]|0;
 $30 = (($sInfo) + 4|0);
 $31 = HEAP32[$30>>2]|0;
 $32 = (_sqlite3FindTable($29,18392,$31)|0);
 $33 = ($32|0)==(0|0);
 if ($33) {
  HEAP32[$0>>2] = 1;
  $50 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($50|0);
 }
 $34 = HEAP32[$1>>2]|0;
 $35 = (($sInfo) + 4|0);
 $36 = HEAP32[$35>>2]|0;
 HEAP32[$vararg_buffer>>2] = $36;
 $37 = (_sqlite3MPrintf($34,18896,$vararg_buffer)|0);
 HEAP32[$zSql>>2] = $37;
 $38 = HEAP32[$zSql>>2]|0;
 $39 = ($38|0)==(0|0);
 if ($39) {
  HEAP32[$rc>>2] = 7;
 } else {
  $40 = HEAP32[$1>>2]|0;
  $41 = HEAP32[$zSql>>2]|0;
  $42 = (_sqlite3_exec($40,$41,7,$sInfo,0)|0);
  HEAP32[$rc>>2] = $42;
  $43 = HEAP32[$1>>2]|0;
  $44 = HEAP32[$zSql>>2]|0;
  _sqlite3DbFree($43,$44);
 }
 $45 = HEAP32[$rc>>2]|0;
 $46 = ($45|0)==(7);
 if ($46) {
  $47 = HEAP32[$1>>2]|0;
  $48 = (($47) + 64|0);
  HEAP8[$48>>0] = 1;
 }
 $49 = HEAP32[$rc>>2]|0;
 HEAP32[$0>>2] = $49;
 $50 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($50|0);
}
function _sqlite3BtreeCommit($p) {
 $p = $p|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $rc = sp;
 HEAP32[$0>>2] = $p;
 $1 = HEAP32[$0>>2]|0;
 _sqlite3BtreeEnter($1);
 $2 = HEAP32[$0>>2]|0;
 $3 = (_sqlite3BtreeCommitPhaseOne($2,0)|0);
 HEAP32[$rc>>2] = $3;
 $4 = HEAP32[$rc>>2]|0;
 $5 = ($4|0)==(0);
 if (!($5)) {
  $8 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($8|0);
 }
 $6 = HEAP32[$0>>2]|0;
 $7 = (_sqlite3BtreeCommitPhaseTwo($6,0)|0);
 HEAP32[$rc>>2] = $7;
 $8 = HEAP32[$rc>>2]|0;
 STACKTOP = sp;return ($8|0);
}
function _sqlite3DefaultRowEst($pIdx) {
 $pIdx = $pIdx|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $a = 0, $aVal = 0, $i = 0, $nCopy = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 12|0;
 $aVal = sp + 16|0;
 $a = sp + 8|0;
 $nCopy = sp + 4|0;
 $i = sp;
 HEAP32[$0>>2] = $pIdx;
 ;HEAP16[$aVal+0>>1]=HEAP16[18976+0>>1]|0;HEAP16[$aVal+2>>1]=HEAP16[18976+2>>1]|0;HEAP16[$aVal+4>>1]=HEAP16[18976+4>>1]|0;HEAP16[$aVal+6>>1]=HEAP16[18976+6>>1]|0;HEAP16[$aVal+8>>1]=HEAP16[18976+8>>1]|0;
 $1 = HEAP32[$0>>2]|0;
 $2 = (($1) + 8|0);
 $3 = HEAP32[$2>>2]|0;
 HEAP32[$a>>2] = $3;
 $4 = HEAP32[$0>>2]|0;
 $5 = (($4) + 50|0);
 $6 = HEAP16[$5>>1]|0;
 $7 = $6&65535;
 $8 = (5)<($7|0);
 if ($8) {
  $13 = 5;
 } else {
  $9 = HEAP32[$0>>2]|0;
  $10 = (($9) + 50|0);
  $11 = HEAP16[$10>>1]|0;
  $12 = $11&65535;
  $13 = $12;
 }
 HEAP32[$nCopy>>2] = $13;
 $14 = HEAP32[$0>>2]|0;
 $15 = (($14) + 12|0);
 $16 = HEAP32[$15>>2]|0;
 $17 = (($16) + 28|0);
 $18 = HEAP16[$17>>1]|0;
 $19 = HEAP32[$a>>2]|0;
 HEAP16[$19>>1] = $18;
 $20 = HEAP32[$a>>2]|0;
 $21 = HEAP16[$20>>1]|0;
 $22 = $21 << 16 >> 16;
 $23 = ($22|0)<(33);
 if ($23) {
  $24 = HEAP32[$a>>2]|0;
  HEAP16[$24>>1] = 33;
 }
 $25 = HEAP32[$a>>2]|0;
 $26 = (($25) + 2|0);
 $27 = HEAP32[$nCopy>>2]|0;
 $28 = $27<<1;
 _memcpy(($26|0),($aVal|0),($28|0))|0;
 $29 = HEAP32[$nCopy>>2]|0;
 $30 = (($29) + 1)|0;
 HEAP32[$i>>2] = $30;
 while(1) {
  $31 = HEAP32[$i>>2]|0;
  $32 = HEAP32[$0>>2]|0;
  $33 = (($32) + 50|0);
  $34 = HEAP16[$33>>1]|0;
  $35 = $34&65535;
  $36 = ($31|0)<=($35|0);
  if (!($36)) {
   break;
  }
  $37 = HEAP32[$i>>2]|0;
  $38 = HEAP32[$a>>2]|0;
  $39 = (($38) + ($37<<1)|0);
  HEAP16[$39>>1] = 23;
  $40 = HEAP32[$i>>2]|0;
  $41 = (($40) + 1)|0;
  HEAP32[$i>>2] = $41;
 }
 $42 = HEAP32[$0>>2]|0;
 $43 = (($42) + 54|0);
 $44 = HEAP8[$43>>0]|0;
 $45 = $44&255;
 $46 = ($45|0)!=(0);
 if (!($46)) {
  STACKTOP = sp;return;
 }
 $47 = HEAP32[$0>>2]|0;
 $48 = (($47) + 50|0);
 $49 = HEAP16[$48>>1]|0;
 $50 = $49&65535;
 $51 = HEAP32[$a>>2]|0;
 $52 = (($51) + ($50<<1)|0);
 HEAP16[$52>>1] = 0;
 STACKTOP = sp;return;
}
function _analysisLoader($pData,$argc,$argv,$NotUsed) {
 $pData = $pData|0;
 $argc = $argc|0;
 $argv = $argv|0;
 $NotUsed = $NotUsed|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0;
 var $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $aiRowEst = 0, $fakeIdx = 0, $nCol = 0, $pIndex = 0, $pInfo = 0, $pTable = 0, $z = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 112|0;
 $0 = sp + 96|0;
 $1 = sp + 92|0;
 $2 = sp + 88|0;
 $3 = sp + 84|0;
 $4 = sp + 80|0;
 $pInfo = sp + 76|0;
 $pIndex = sp + 72|0;
 $pTable = sp + 68|0;
 $z = sp + 64|0;
 $nCol = sp + 60|0;
 $aiRowEst = sp + 56|0;
 $fakeIdx = sp;
 HEAP32[$1>>2] = $pData;
 HEAP32[$2>>2] = $argc;
 HEAP32[$3>>2] = $argv;
 HEAP32[$4>>2] = $NotUsed;
 $5 = HEAP32[$1>>2]|0;
 HEAP32[$pInfo>>2] = $5;
 $6 = HEAP32[$3>>2]|0;
 $7 = ($6|0)==(0|0);
 if (!($7)) {
  $8 = HEAP32[$3>>2]|0;
  $9 = HEAP32[$8>>2]|0;
  $10 = ($9|0)==(0|0);
  if (!($10)) {
   $11 = HEAP32[$3>>2]|0;
   $12 = (($11) + 8|0);
   $13 = HEAP32[$12>>2]|0;
   $14 = ($13|0)==(0|0);
   if (!($14)) {
    $15 = HEAP32[$pInfo>>2]|0;
    $16 = HEAP32[$15>>2]|0;
    $17 = HEAP32[$3>>2]|0;
    $18 = HEAP32[$17>>2]|0;
    $19 = HEAP32[$pInfo>>2]|0;
    $20 = (($19) + 4|0);
    $21 = HEAP32[$20>>2]|0;
    $22 = (_sqlite3FindTable($16,$18,$21)|0);
    HEAP32[$pTable>>2] = $22;
    $23 = HEAP32[$pTable>>2]|0;
    $24 = ($23|0)==(0|0);
    if ($24) {
     HEAP32[$0>>2] = 0;
     $89 = HEAP32[$0>>2]|0;
     STACKTOP = sp;return ($89|0);
    }
    $25 = HEAP32[$3>>2]|0;
    $26 = (($25) + 4|0);
    $27 = HEAP32[$26>>2]|0;
    $28 = ($27|0)==(0|0);
    do {
     if ($28) {
      HEAP32[$pIndex>>2] = 0;
     } else {
      $29 = HEAP32[$3>>2]|0;
      $30 = HEAP32[$29>>2]|0;
      $31 = HEAP32[$3>>2]|0;
      $32 = (($31) + 4|0);
      $33 = HEAP32[$32>>2]|0;
      $34 = (_sqlite3_stricmp($30,$33)|0);
      $35 = ($34|0)==(0);
      if ($35) {
       $36 = HEAP32[$pTable>>2]|0;
       $37 = (_sqlite3PrimaryKeyIndex($36)|0);
       HEAP32[$pIndex>>2] = $37;
       break;
      } else {
       $38 = HEAP32[$pInfo>>2]|0;
       $39 = HEAP32[$38>>2]|0;
       $40 = HEAP32[$3>>2]|0;
       $41 = (($40) + 4|0);
       $42 = HEAP32[$41>>2]|0;
       $43 = HEAP32[$pInfo>>2]|0;
       $44 = (($43) + 4|0);
       $45 = HEAP32[$44>>2]|0;
       $46 = (_sqlite3FindIndex($39,$42,$45)|0);
       HEAP32[$pIndex>>2] = $46;
       break;
      }
     }
    } while(0);
    $47 = HEAP32[$3>>2]|0;
    $48 = (($47) + 8|0);
    $49 = HEAP32[$48>>2]|0;
    HEAP32[$z>>2] = $49;
    $50 = HEAP32[$pIndex>>2]|0;
    $51 = ($50|0)!=(0|0);
    if ($51) {
     $52 = HEAP32[$pIndex>>2]|0;
     $53 = (($52) + 50|0);
     $54 = HEAP16[$53>>1]|0;
     $55 = $54&65535;
     $56 = (($55) + 1)|0;
     HEAP32[$nCol>>2] = $56;
     HEAP32[$aiRowEst>>2] = 0;
     $57 = HEAP32[$pIndex>>2]|0;
     $58 = (($57) + 55|0);
     $59 = HEAP8[$58>>0]|0;
     $60 = $59 & -5;
     HEAP8[$58>>0] = $60;
     $61 = HEAP32[$z>>2]|0;
     $62 = HEAP32[$nCol>>2]|0;
     $63 = HEAP32[$aiRowEst>>2]|0;
     $64 = HEAP32[$pIndex>>2]|0;
     $65 = (($64) + 8|0);
     $66 = HEAP32[$65>>2]|0;
     $67 = HEAP32[$pIndex>>2]|0;
     _decodeIntArray($61,$62,$63,$66,$67);
     $68 = HEAP32[$pIndex>>2]|0;
     $69 = (($68) + 36|0);
     $70 = HEAP32[$69>>2]|0;
     $71 = ($70|0)==(0|0);
     if ($71) {
      $72 = HEAP32[$pIndex>>2]|0;
      $73 = (($72) + 8|0);
      $74 = HEAP32[$73>>2]|0;
      $75 = HEAP16[$74>>1]|0;
      $76 = HEAP32[$pTable>>2]|0;
      $77 = (($76) + 28|0);
      HEAP16[$77>>1] = $75;
     }
    } else {
     $78 = HEAP32[$pTable>>2]|0;
     $79 = (($78) + 42|0);
     $80 = HEAP16[$79>>1]|0;
     $81 = (($fakeIdx) + 48|0);
     HEAP16[$81>>1] = $80;
     $82 = HEAP32[$z>>2]|0;
     $83 = HEAP32[$pTable>>2]|0;
     $84 = (($83) + 28|0);
     _decodeIntArray($82,1,0,$84,$fakeIdx);
     $85 = (($fakeIdx) + 48|0);
     $86 = HEAP16[$85>>1]|0;
     $87 = HEAP32[$pTable>>2]|0;
     $88 = (($87) + 42|0);
     HEAP16[$88>>1] = $86;
    }
    HEAP32[$0>>2] = 0;
    $89 = HEAP32[$0>>2]|0;
    STACKTOP = sp;return ($89|0);
   }
  }
 }
 HEAP32[$0>>2] = 0;
 $89 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($89|0);
}
function _decodeIntArray($zIntArray,$nOut,$aOut,$aLog,$pIndex) {
 $zIntArray = $zIntArray|0;
 $nOut = $nOut|0;
 $aOut = $aOut|0;
 $aLog = $aLog|0;
 $pIndex = $pIndex|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $8 = 0, $9 = 0, $c = 0, $i = 0, $or$cond = 0;
 var $v = 0, $z = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $0 = sp + 32|0;
 $1 = sp + 28|0;
 $2 = sp + 24|0;
 $3 = sp + 20|0;
 $4 = sp + 16|0;
 $z = sp + 12|0;
 $c = sp + 8|0;
 $i = sp + 4|0;
 $v = sp;
 HEAP32[$0>>2] = $zIntArray;
 HEAP32[$1>>2] = $nOut;
 HEAP32[$2>>2] = $aOut;
 HEAP32[$3>>2] = $aLog;
 HEAP32[$4>>2] = $pIndex;
 $5 = HEAP32[$0>>2]|0;
 HEAP32[$z>>2] = $5;
 HEAP32[$i>>2] = 0;
 while(1) {
  $6 = HEAP32[$z>>2]|0;
  $7 = HEAP8[$6>>0]|0;
  $8 = $7 << 24 >> 24;
  $9 = ($8|0)!=(0);
  if (!($9)) {
   break;
  }
  $10 = HEAP32[$i>>2]|0;
  $11 = HEAP32[$1>>2]|0;
  $12 = ($10|0)<($11|0);
  if (!($12)) {
   break;
  }
  HEAP32[$v>>2] = 0;
  while(1) {
   $13 = HEAP32[$z>>2]|0;
   $14 = HEAP8[$13>>0]|0;
   $15 = $14 << 24 >> 24;
   HEAP32[$c>>2] = $15;
   $16 = ($15|0)>=(48);
   $17 = HEAP32[$c>>2]|0;
   $18 = ($17|0)<=(57);
   $or$cond = $16 & $18;
   if (!($or$cond)) {
    break;
   }
   $19 = HEAP32[$v>>2]|0;
   $20 = ($19*10)|0;
   $21 = HEAP32[$c>>2]|0;
   $22 = (($20) + ($21))|0;
   $23 = (($22) - 48)|0;
   HEAP32[$v>>2] = $23;
   $24 = HEAP32[$z>>2]|0;
   $25 = (($24) + 1|0);
   HEAP32[$z>>2] = $25;
  }
  $26 = HEAP32[$v>>2]|0;
  $27 = (_sqlite3LogEst($26,0)|0);
  $28 = HEAP32[$i>>2]|0;
  $29 = HEAP32[$3>>2]|0;
  $30 = (($29) + ($28<<1)|0);
  HEAP16[$30>>1] = $27;
  $31 = HEAP32[$z>>2]|0;
  $32 = HEAP8[$31>>0]|0;
  $33 = $32 << 24 >> 24;
  $34 = ($33|0)==(32);
  if ($34) {
   $35 = HEAP32[$z>>2]|0;
   $36 = (($35) + 1|0);
   HEAP32[$z>>2] = $36;
  }
  $37 = HEAP32[$i>>2]|0;
  $38 = (($37) + 1)|0;
  HEAP32[$i>>2] = $38;
 }
 L12: while(1) {
  $39 = HEAP32[$z>>2]|0;
  $40 = HEAP8[$39>>0]|0;
  $41 = ($40<<24>>24)!=(0);
  if (!($41)) {
   break;
  }
  $42 = HEAP32[$z>>2]|0;
  $43 = (_sqlite3_strglob(18944,$42)|0);
  $44 = ($43|0)==(0);
  if ($44) {
   $45 = HEAP32[$4>>2]|0;
   $46 = (($45) + 55|0);
   $47 = HEAP8[$46>>0]|0;
   $48 = $47 & -5;
   $49 = $48 | 4;
   HEAP8[$46>>0] = $49;
  } else {
   $50 = HEAP32[$z>>2]|0;
   $51 = (_sqlite3_strglob(18960,$50)|0);
   $52 = ($51|0)==(0);
   if ($52) {
    $53 = HEAP32[$z>>2]|0;
    $54 = (($53) + 3|0);
    $55 = (_sqlite3Atoi($54)|0);
    $56 = ($55|0)<(0);
    $57 = $56 << 31 >> 31;
    $58 = (_sqlite3LogEst($55,$57)|0);
    $59 = HEAP32[$4>>2]|0;
    $60 = (($59) + 48|0);
    HEAP16[$60>>1] = $58;
   }
  }
  while(1) {
   $61 = HEAP32[$z>>2]|0;
   $62 = HEAP8[$61>>0]|0;
   $63 = $62 << 24 >> 24;
   $64 = ($63|0)!=(0);
   if (!($64)) {
    break;
   }
   $65 = HEAP32[$z>>2]|0;
   $66 = HEAP8[$65>>0]|0;
   $67 = $66 << 24 >> 24;
   $68 = ($67|0)!=(32);
   if (!($68)) {
    break;
   }
   $69 = HEAP32[$z>>2]|0;
   $70 = (($69) + 1|0);
   HEAP32[$z>>2] = $70;
  }
  while(1) {
   $71 = HEAP32[$z>>2]|0;
   $72 = HEAP8[$71>>0]|0;
   $73 = $72 << 24 >> 24;
   $74 = ($73|0)==(32);
   if (!($74)) {
    continue L12;
   }
   $75 = HEAP32[$z>>2]|0;
   $76 = (($75) + 1|0);
   HEAP32[$z>>2] = $76;
  }
 }
 STACKTOP = sp;return;
}
function _corruptSchema($pData,$zObj,$zExtra) {
 $pData = $pData|0;
 $zObj = $zObj|0;
 $zExtra = $zExtra|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $db = 0, $vararg_buffer1 = 0, $vararg_ptr4 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $vararg_buffer1 = sp;
 $0 = sp + 20|0;
 $1 = sp + 16|0;
 $2 = sp + 12|0;
 $db = sp + 8|0;
 HEAP32[$0>>2] = $pData;
 HEAP32[$1>>2] = $zObj;
 HEAP32[$2>>2] = $zExtra;
 $3 = HEAP32[$0>>2]|0;
 $4 = HEAP32[$3>>2]|0;
 HEAP32[$db>>2] = $4;
 $5 = HEAP32[$db>>2]|0;
 $6 = (($5) + 64|0);
 $7 = HEAP8[$6>>0]|0;
 $8 = ($7<<24>>24)!=(0);
 if (!($8)) {
  $9 = HEAP32[$db>>2]|0;
  $10 = (($9) + 24|0);
  $11 = HEAP32[$10>>2]|0;
  $12 = $11 & 65536;
  $13 = ($12|0)==(0);
  if ($13) {
   $14 = HEAP32[$1>>2]|0;
   $15 = ($14|0)==(0|0);
   if ($15) {
    HEAP32[$1>>2] = 19016;
   }
   $16 = HEAP32[$0>>2]|0;
   $17 = (($16) + 4|0);
   $18 = HEAP32[$17>>2]|0;
   $19 = HEAP32[$db>>2]|0;
   $20 = HEAP32[$1>>2]|0;
   HEAP32[$vararg_buffer1>>2] = $20;
   _sqlite3SetString($18,$19,19024,$vararg_buffer1);
   $21 = HEAP32[$2>>2]|0;
   $22 = ($21|0)!=(0|0);
   if ($22) {
    $23 = HEAP32[$db>>2]|0;
    $24 = HEAP32[$0>>2]|0;
    $25 = (($24) + 4|0);
    $26 = HEAP32[$25>>2]|0;
    $27 = HEAP32[$26>>2]|0;
    $28 = HEAP32[$0>>2]|0;
    $29 = (($28) + 4|0);
    $30 = HEAP32[$29>>2]|0;
    $31 = HEAP32[$30>>2]|0;
    $32 = HEAP32[$2>>2]|0;
    HEAP32[$vararg_buffer1>>2] = $31;
    $vararg_ptr4 = (($vararg_buffer1) + 4|0);
    HEAP32[$vararg_ptr4>>2] = $32;
    $33 = (_sqlite3MAppendf($23,$27,19056,$vararg_buffer1)|0);
    $34 = HEAP32[$0>>2]|0;
    $35 = (($34) + 4|0);
    $36 = HEAP32[$35>>2]|0;
    HEAP32[$36>>2] = $33;
   }
  }
 }
 $37 = HEAP32[$db>>2]|0;
 $38 = (($37) + 64|0);
 $39 = HEAP8[$38>>0]|0;
 $40 = $39&255;
 $41 = ($40|0)!=(0);
 if ($41) {
  $45 = 7;
  $43 = HEAP32[$0>>2]|0;
  $44 = (($43) + 12|0);
  HEAP32[$44>>2] = $45;
  STACKTOP = sp;return;
 }
 $42 = (_sqlite3CorruptError(103217)|0);
 $45 = $42;
 $43 = HEAP32[$0>>2]|0;
 $44 = (($43) + 12|0);
 HEAP32[$44>>2] = $45;
 STACKTOP = sp;return;
}
function _sqlite3MAppendf($db,$zStr,$zFormat,$varargs) {
 $db = $db|0;
 $zStr = $zStr|0;
 $zFormat = $zFormat|0;
 $varargs = $varargs|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $ap = 0, $z = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $0 = sp + 32|0;
 $1 = sp + 28|0;
 $2 = sp + 24|0;
 $ap = sp + 8|0;
 $z = sp;
 HEAP32[$0>>2] = $db;
 HEAP32[$1>>2] = $zStr;
 HEAP32[$2>>2] = $zFormat;
 HEAP32[$ap>>2] = $varargs;
 $3 = HEAP32[$0>>2]|0;
 $4 = HEAP32[$2>>2]|0;
 $5 = (_sqlite3VMPrintf($3,$4,$ap)|0);
 HEAP32[$z>>2] = $5;
 $6 = HEAP32[$0>>2]|0;
 $7 = HEAP32[$1>>2]|0;
 _sqlite3DbFree($6,$7);
 $8 = HEAP32[$z>>2]|0;
 STACKTOP = sp;return ($8|0);
}
function _reindexDatabases($pParse,$zColl) {
 $pParse = $pParse|0;
 $zColl = $zColl|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $db = 0, $iDb = 0, $k = 0, $pDb = 0, $pTab = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 24|0;
 $1 = sp + 20|0;
 $pDb = sp + 16|0;
 $iDb = sp + 12|0;
 $db = sp + 8|0;
 $k = sp + 4|0;
 $pTab = sp;
 HEAP32[$0>>2] = $pParse;
 HEAP32[$1>>2] = $zColl;
 $2 = HEAP32[$0>>2]|0;
 $3 = HEAP32[$2>>2]|0;
 HEAP32[$db>>2] = $3;
 HEAP32[$iDb>>2] = 0;
 $4 = HEAP32[$db>>2]|0;
 $5 = (($4) + 16|0);
 $6 = HEAP32[$5>>2]|0;
 HEAP32[$pDb>>2] = $6;
 while(1) {
  $7 = HEAP32[$iDb>>2]|0;
  $8 = HEAP32[$db>>2]|0;
  $9 = (($8) + 20|0);
  $10 = HEAP32[$9>>2]|0;
  $11 = ($7|0)<($10|0);
  if (!($11)) {
   break;
  }
  $12 = HEAP32[$pDb>>2]|0;
  $13 = (($12) + 12|0);
  $14 = HEAP32[$13>>2]|0;
  $15 = (($14) + 8|0);
  $16 = (($15) + 8|0);
  $17 = HEAP32[$16>>2]|0;
  HEAP32[$k>>2] = $17;
  while(1) {
   $18 = HEAP32[$k>>2]|0;
   $19 = ($18|0)!=(0|0);
   if (!($19)) {
    break;
   }
   $20 = HEAP32[$k>>2]|0;
   $21 = (($20) + 8|0);
   $22 = HEAP32[$21>>2]|0;
   HEAP32[$pTab>>2] = $22;
   $23 = HEAP32[$0>>2]|0;
   $24 = HEAP32[$pTab>>2]|0;
   $25 = HEAP32[$1>>2]|0;
   _reindexTable($23,$24,$25);
   $26 = HEAP32[$k>>2]|0;
   $27 = HEAP32[$26>>2]|0;
   HEAP32[$k>>2] = $27;
  }
  $28 = HEAP32[$iDb>>2]|0;
  $29 = (($28) + 1)|0;
  HEAP32[$iDb>>2] = $29;
  $30 = HEAP32[$pDb>>2]|0;
  $31 = (($30) + 16|0);
  HEAP32[$pDb>>2] = $31;
 }
 STACKTOP = sp;return;
}
function _reindexTable($pParse,$pTab,$zColl) {
 $pParse = $pParse|0;
 $pTab = $pTab|0;
 $zColl = $zColl|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $iDb = 0, $pIndex = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 16|0;
 $1 = sp + 12|0;
 $2 = sp + 8|0;
 $pIndex = sp + 4|0;
 $iDb = sp;
 HEAP32[$0>>2] = $pParse;
 HEAP32[$1>>2] = $pTab;
 HEAP32[$2>>2] = $zColl;
 $3 = HEAP32[$1>>2]|0;
 $4 = (($3) + 8|0);
 $5 = HEAP32[$4>>2]|0;
 HEAP32[$pIndex>>2] = $5;
 while(1) {
  $6 = HEAP32[$pIndex>>2]|0;
  $7 = ($6|0)!=(0|0);
  if (!($7)) {
   break;
  }
  $8 = HEAP32[$2>>2]|0;
  $9 = ($8|0)==(0|0);
  if ($9) {
   label = 5;
  } else {
   $10 = HEAP32[$2>>2]|0;
   $11 = HEAP32[$pIndex>>2]|0;
   $12 = (_collationMatch($10,$11)|0);
   $13 = ($12|0)!=(0);
   if ($13) {
    label = 5;
   }
  }
  if ((label|0) == 5) {
   label = 0;
   $14 = HEAP32[$0>>2]|0;
   $15 = HEAP32[$14>>2]|0;
   $16 = HEAP32[$1>>2]|0;
   $17 = (($16) + 68|0);
   $18 = HEAP32[$17>>2]|0;
   $19 = (_sqlite3SchemaToIndex($15,$18)|0);
   HEAP32[$iDb>>2] = $19;
   $20 = HEAP32[$0>>2]|0;
   $21 = HEAP32[$iDb>>2]|0;
   _sqlite3BeginWriteOperation($20,0,$21);
   $22 = HEAP32[$0>>2]|0;
   $23 = HEAP32[$pIndex>>2]|0;
   _sqlite3RefillIndex($22,$23,-1);
  }
  $24 = HEAP32[$pIndex>>2]|0;
  $25 = (($24) + 20|0);
  $26 = HEAP32[$25>>2]|0;
  HEAP32[$pIndex>>2] = $26;
 }
 STACKTOP = sp;return;
}
function _sqlite3RefillIndex($pParse,$pIndex,$memRootPage) {
 $pParse = $pParse|0;
 $pIndex = $pIndex|0;
 $memRootPage = $memRootPage|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0;
 var $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0;
 var $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0;
 var $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0;
 var $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $addr1 = 0, $addr2 = 0, $db = 0, $iDb = 0, $iIdx = 0, $iPartIdxLabel = 0, $iSorter = 0, $iTab = 0;
 var $j2 = 0, $or$cond = 0, $pKey = 0, $pTab = 0, $regRecord = 0, $tnum = 0, $v = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 80|0;
 $0 = sp + 64|0;
 $1 = sp + 60|0;
 $2 = sp + 56|0;
 $pTab = sp + 52|0;
 $iTab = sp + 48|0;
 $iIdx = sp + 44|0;
 $iSorter = sp + 40|0;
 $addr1 = sp + 36|0;
 $addr2 = sp + 32|0;
 $tnum = sp + 28|0;
 $iPartIdxLabel = sp + 24|0;
 $v = sp + 20|0;
 $pKey = sp + 16|0;
 $regRecord = sp + 12|0;
 $db = sp + 8|0;
 $iDb = sp + 4|0;
 $j2 = sp;
 HEAP32[$0>>2] = $pParse;
 HEAP32[$1>>2] = $pIndex;
 HEAP32[$2>>2] = $memRootPage;
 $3 = HEAP32[$1>>2]|0;
 $4 = (($3) + 12|0);
 $5 = HEAP32[$4>>2]|0;
 HEAP32[$pTab>>2] = $5;
 $6 = HEAP32[$0>>2]|0;
 $7 = (($6) + 68|0);
 $8 = HEAP32[$7>>2]|0;
 $9 = (($8) + 1)|0;
 HEAP32[$7>>2] = $9;
 HEAP32[$iTab>>2] = $8;
 $10 = HEAP32[$0>>2]|0;
 $11 = (($10) + 68|0);
 $12 = HEAP32[$11>>2]|0;
 $13 = (($12) + 1)|0;
 HEAP32[$11>>2] = $13;
 HEAP32[$iIdx>>2] = $12;
 $14 = HEAP32[$0>>2]|0;
 $15 = HEAP32[$14>>2]|0;
 HEAP32[$db>>2] = $15;
 $16 = HEAP32[$db>>2]|0;
 $17 = HEAP32[$1>>2]|0;
 $18 = (($17) + 24|0);
 $19 = HEAP32[$18>>2]|0;
 $20 = (_sqlite3SchemaToIndex($16,$19)|0);
 HEAP32[$iDb>>2] = $20;
 $21 = HEAP32[$0>>2]|0;
 $22 = HEAP32[$1>>2]|0;
 $23 = HEAP32[$22>>2]|0;
 $24 = HEAP32[$iDb>>2]|0;
 $25 = HEAP32[$db>>2]|0;
 $26 = (($25) + 16|0);
 $27 = HEAP32[$26>>2]|0;
 $28 = (($27) + ($24<<4)|0);
 $29 = HEAP32[$28>>2]|0;
 $30 = (_sqlite3AuthCheck($21,27,$23,0,$29)|0);
 $31 = ($30|0)!=(0);
 if ($31) {
  STACKTOP = sp;return;
 }
 $32 = HEAP32[$0>>2]|0;
 $33 = HEAP32[$iDb>>2]|0;
 $34 = HEAP32[$pTab>>2]|0;
 $35 = (($34) + 32|0);
 $36 = HEAP32[$35>>2]|0;
 $37 = HEAP32[$pTab>>2]|0;
 $38 = HEAP32[$37>>2]|0;
 _sqlite3TableLock($32,$33,$36,1,$38);
 $39 = HEAP32[$0>>2]|0;
 $40 = (_sqlite3GetVdbe($39)|0);
 HEAP32[$v>>2] = $40;
 $41 = HEAP32[$v>>2]|0;
 $42 = ($41|0)==(0|0);
 if ($42) {
  STACKTOP = sp;return;
 }
 $43 = HEAP32[$2>>2]|0;
 $44 = ($43|0)>=(0);
 if ($44) {
  $45 = HEAP32[$2>>2]|0;
  HEAP32[$tnum>>2] = $45;
 } else {
  $46 = HEAP32[$1>>2]|0;
  $47 = (($46) + 44|0);
  $48 = HEAP32[$47>>2]|0;
  HEAP32[$tnum>>2] = $48;
 }
 $49 = HEAP32[$0>>2]|0;
 $50 = HEAP32[$1>>2]|0;
 $51 = (_sqlite3KeyInfoOfIndex($49,$50)|0);
 HEAP32[$pKey>>2] = $51;
 $52 = HEAP32[$0>>2]|0;
 $53 = (($52) + 68|0);
 $54 = HEAP32[$53>>2]|0;
 $55 = (($54) + 1)|0;
 HEAP32[$53>>2] = $55;
 HEAP32[$iSorter>>2] = $54;
 $56 = HEAP32[$v>>2]|0;
 $57 = HEAP32[$iSorter>>2]|0;
 $58 = HEAP32[$1>>2]|0;
 $59 = (($58) + 50|0);
 $60 = HEAP16[$59>>1]|0;
 $61 = $60&65535;
 $62 = HEAP32[$pKey>>2]|0;
 $63 = (_sqlite3KeyInfoRef($62)|0);
 (_sqlite3VdbeAddOp4($56,58,$57,0,$61,$63,-6)|0);
 $64 = HEAP32[$0>>2]|0;
 $65 = HEAP32[$iTab>>2]|0;
 $66 = HEAP32[$iDb>>2]|0;
 $67 = HEAP32[$pTab>>2]|0;
 _sqlite3OpenTable($64,$65,$66,$67,54);
 $68 = HEAP32[$v>>2]|0;
 $69 = HEAP32[$iTab>>2]|0;
 $70 = (_sqlite3VdbeAddOp2($68,108,$69,0)|0);
 HEAP32[$addr1>>2] = $70;
 $71 = HEAP32[$0>>2]|0;
 $72 = (_sqlite3GetTempReg($71)|0);
 HEAP32[$regRecord>>2] = $72;
 $73 = HEAP32[$0>>2]|0;
 $74 = HEAP32[$1>>2]|0;
 $75 = HEAP32[$iTab>>2]|0;
 $76 = HEAP32[$regRecord>>2]|0;
 (_sqlite3GenerateIndexKey($73,$74,$75,$76,0,$iPartIdxLabel,0,0)|0);
 $77 = HEAP32[$v>>2]|0;
 $78 = HEAP32[$iSorter>>2]|0;
 $79 = HEAP32[$regRecord>>2]|0;
 (_sqlite3VdbeAddOp2($77,109,$78,$79)|0);
 $80 = HEAP32[$0>>2]|0;
 $81 = HEAP32[$iPartIdxLabel>>2]|0;
 _sqlite3ResolvePartIdxLabel($80,$81);
 $82 = HEAP32[$v>>2]|0;
 $83 = HEAP32[$iTab>>2]|0;
 $84 = HEAP32[$addr1>>2]|0;
 $85 = (($84) + 1)|0;
 (_sqlite3VdbeAddOp2($82,9,$83,$85)|0);
 $86 = HEAP32[$v>>2]|0;
 $87 = HEAP32[$addr1>>2]|0;
 _sqlite3VdbeJumpHere($86,$87);
 $88 = HEAP32[$2>>2]|0;
 $89 = ($88|0)<(0);
 if ($89) {
  $90 = HEAP32[$v>>2]|0;
  $91 = HEAP32[$tnum>>2]|0;
  $92 = HEAP32[$iDb>>2]|0;
  (_sqlite3VdbeAddOp2($90,118,$91,$92)|0);
 }
 $93 = HEAP32[$v>>2]|0;
 $94 = HEAP32[$iIdx>>2]|0;
 $95 = HEAP32[$tnum>>2]|0;
 $96 = HEAP32[$iDb>>2]|0;
 $97 = HEAP32[$pKey>>2]|0;
 (_sqlite3VdbeAddOp4($93,55,$94,$95,$96,$97,-6)|0);
 $98 = HEAP32[$v>>2]|0;
 $99 = HEAP32[$2>>2]|0;
 $100 = ($99|0)>=(0);
 $101 = $100 ? 2 : 0;
 $102 = 1 | $101;
 $103 = $102&255;
 _sqlite3VdbeChangeP5($98,$103);
 $104 = HEAP32[$v>>2]|0;
 $105 = HEAP32[$iSorter>>2]|0;
 $106 = (_sqlite3VdbeAddOp2($104,106,$105,0)|0);
 HEAP32[$addr1>>2] = $106;
 $107 = HEAP32[$1>>2]|0;
 $108 = (($107) + 54|0);
 $109 = HEAP8[$108>>0]|0;
 $110 = $109&255;
 $111 = ($110|0)!=(0);
 $112 = HEAP32[$pKey>>2]|0;
 $113 = ($112|0)!=(0|0);
 $or$cond = $111 & $113;
 $114 = HEAP32[$v>>2]|0;
 $115 = (_sqlite3VdbeCurrentAddr($114)|0);
 if ($or$cond) {
  $116 = (($115) + 3)|0;
  HEAP32[$j2>>2] = $116;
  $117 = HEAP32[$v>>2]|0;
  $118 = HEAP32[$j2>>2]|0;
  (_sqlite3VdbeAddOp2($117,16,0,$118)|0);
  $119 = HEAP32[$v>>2]|0;
  $120 = (_sqlite3VdbeCurrentAddr($119)|0);
  HEAP32[$addr2>>2] = $120;
  $121 = HEAP32[$v>>2]|0;
  $122 = HEAP32[$iSorter>>2]|0;
  $123 = HEAP32[$j2>>2]|0;
  $124 = HEAP32[$regRecord>>2]|0;
  $125 = HEAP32[$1>>2]|0;
  $126 = (($125) + 50|0);
  $127 = HEAP16[$126>>1]|0;
  $128 = $127&65535;
  (_sqlite3VdbeAddOp4Int($121,99,$122,$123,$124,$128)|0);
  $129 = HEAP32[$0>>2]|0;
  $130 = HEAP32[$1>>2]|0;
  _sqlite3UniqueConstraint($129,2,$130);
 } else {
  HEAP32[$addr2>>2] = $115;
 }
 $131 = HEAP32[$v>>2]|0;
 $132 = HEAP32[$iSorter>>2]|0;
 $133 = HEAP32[$regRecord>>2]|0;
 $134 = HEAP32[$iIdx>>2]|0;
 (_sqlite3VdbeAddOp3($131,100,$132,$133,$134)|0);
 $135 = HEAP32[$v>>2]|0;
 $136 = HEAP32[$iIdx>>2]|0;
 $137 = HEAP32[$regRecord>>2]|0;
 (_sqlite3VdbeAddOp3($135,110,$136,$137,1)|0);
 $138 = HEAP32[$v>>2]|0;
 _sqlite3VdbeChangeP5($138,16);
 $139 = HEAP32[$0>>2]|0;
 $140 = HEAP32[$regRecord>>2]|0;
 _sqlite3ReleaseTempReg($139,$140);
 $141 = HEAP32[$v>>2]|0;
 $142 = HEAP32[$iSorter>>2]|0;
 $143 = HEAP32[$addr2>>2]|0;
 (_sqlite3VdbeAddOp2($141,5,$142,$143)|0);
 $144 = HEAP32[$v>>2]|0;
 $145 = HEAP32[$addr1>>2]|0;
 _sqlite3VdbeJumpHere($144,$145);
 $146 = HEAP32[$v>>2]|0;
 $147 = HEAP32[$iTab>>2]|0;
 (_sqlite3VdbeAddOp1($146,61,$147)|0);
 $148 = HEAP32[$v>>2]|0;
 $149 = HEAP32[$iIdx>>2]|0;
 (_sqlite3VdbeAddOp1($148,61,$149)|0);
 $150 = HEAP32[$v>>2]|0;
 $151 = HEAP32[$iSorter>>2]|0;
 (_sqlite3VdbeAddOp1($150,61,$151)|0);
 STACKTOP = sp;return;
}
function _sqlite3GenerateIndexKey($pParse,$pIdx,$iDataCur,$regOut,$prefixOnly,$piPartIdxLabel,$pPrior,$regPrior) {
 $pParse = $pParse|0;
 $pIdx = $pIdx|0;
 $iDataCur = $iDataCur|0;
 $regOut = $regOut|0;
 $prefixOnly = $prefixOnly|0;
 $piPartIdxLabel = $piPartIdxLabel|0;
 $pPrior = $pPrior|0;
 $regPrior = $regPrior|0;
 var $$sink = 0, $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0;
 var $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0;
 var $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0;
 var $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0;
 var $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0;
 var $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $j = 0, $nCol = 0, $pTab = 0, $regBase = 0, $v = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 64|0;
 $0 = sp + 48|0;
 $1 = sp + 44|0;
 $2 = sp + 40|0;
 $3 = sp + 36|0;
 $4 = sp + 32|0;
 $5 = sp + 28|0;
 $6 = sp + 24|0;
 $7 = sp + 20|0;
 $v = sp + 16|0;
 $j = sp + 12|0;
 $pTab = sp + 8|0;
 $regBase = sp + 4|0;
 $nCol = sp;
 HEAP32[$0>>2] = $pParse;
 HEAP32[$1>>2] = $pIdx;
 HEAP32[$2>>2] = $iDataCur;
 HEAP32[$3>>2] = $regOut;
 HEAP32[$4>>2] = $prefixOnly;
 HEAP32[$5>>2] = $piPartIdxLabel;
 HEAP32[$6>>2] = $pPrior;
 HEAP32[$7>>2] = $regPrior;
 $8 = HEAP32[$0>>2]|0;
 $9 = (($8) + 8|0);
 $10 = HEAP32[$9>>2]|0;
 HEAP32[$v>>2] = $10;
 $11 = HEAP32[$1>>2]|0;
 $12 = (($11) + 12|0);
 $13 = HEAP32[$12>>2]|0;
 HEAP32[$pTab>>2] = $13;
 $14 = HEAP32[$5>>2]|0;
 $15 = ($14|0)!=(0|0);
 do {
  if ($15) {
   $16 = HEAP32[$1>>2]|0;
   $17 = (($16) + 36|0);
   $18 = HEAP32[$17>>2]|0;
   $19 = ($18|0)!=(0|0);
   if ($19) {
    $20 = HEAP32[$v>>2]|0;
    $21 = (_sqlite3VdbeMakeLabel($20)|0);
    $22 = HEAP32[$5>>2]|0;
    HEAP32[$22>>2] = $21;
    $23 = HEAP32[$2>>2]|0;
    $24 = HEAP32[$0>>2]|0;
    $25 = (($24) + 96|0);
    HEAP32[$25>>2] = $23;
    $26 = HEAP32[$0>>2]|0;
    _sqlite3ExprCachePush($26);
    $27 = HEAP32[$0>>2]|0;
    $28 = HEAP32[$1>>2]|0;
    $29 = (($28) + 36|0);
    $30 = HEAP32[$29>>2]|0;
    $31 = HEAP32[$5>>2]|0;
    $32 = HEAP32[$31>>2]|0;
    _sqlite3ExprIfFalse($27,$30,$32,16);
    break;
   } else {
    $33 = HEAP32[$5>>2]|0;
    HEAP32[$33>>2] = 0;
    break;
   }
  }
 } while(0);
 $34 = HEAP32[$4>>2]|0;
 $35 = ($34|0)!=(0);
 if ($35) {
  $36 = HEAP32[$1>>2]|0;
  $37 = (($36) + 55|0);
  $38 = HEAP8[$37>>0]|0;
  $39 = ($38&255) >>> 3;
  $40 = $39 & 1;
  $41 = $40&255;
  $42 = ($41|0)!=(0);
  if ($42) {
   $43 = HEAP32[$1>>2]|0;
   $44 = (($43) + 50|0);
   $45 = HEAP16[$44>>1]|0;
   $$sink = $45;
  } else {
   label = 8;
  }
 } else {
  label = 8;
 }
 if ((label|0) == 8) {
  $46 = HEAP32[$1>>2]|0;
  $47 = (($46) + 52|0);
  $48 = HEAP16[$47>>1]|0;
  $$sink = $48;
 }
 $49 = $$sink&65535;
 HEAP32[$nCol>>2] = $49;
 $50 = HEAP32[$0>>2]|0;
 $51 = HEAP32[$nCol>>2]|0;
 $52 = (_sqlite3GetTempRange($50,$51)|0);
 HEAP32[$regBase>>2] = $52;
 $53 = HEAP32[$6>>2]|0;
 $54 = ($53|0)!=(0|0);
 do {
  if ($54) {
   $55 = HEAP32[$regBase>>2]|0;
   $56 = HEAP32[$7>>2]|0;
   $57 = ($55|0)!=($56|0);
   if (!($57)) {
    $58 = HEAP32[$6>>2]|0;
    $59 = (($58) + 36|0);
    $60 = HEAP32[$59>>2]|0;
    $61 = ($60|0)!=(0|0);
    if (!($61)) {
     break;
    }
   }
   HEAP32[$6>>2] = 0;
  }
 } while(0);
 HEAP32[$j>>2] = 0;
 while(1) {
  $62 = HEAP32[$j>>2]|0;
  $63 = HEAP32[$nCol>>2]|0;
  $64 = ($62|0)<($63|0);
  if (!($64)) {
   break;
  }
  $65 = HEAP32[$6>>2]|0;
  $66 = ($65|0)!=(0|0);
  if ($66) {
   $67 = HEAP32[$j>>2]|0;
   $68 = HEAP32[$6>>2]|0;
   $69 = (($68) + 4|0);
   $70 = HEAP32[$69>>2]|0;
   $71 = (($70) + ($67<<1)|0);
   $72 = HEAP16[$71>>1]|0;
   $73 = $72 << 16 >> 16;
   $74 = HEAP32[$j>>2]|0;
   $75 = HEAP32[$1>>2]|0;
   $76 = (($75) + 4|0);
   $77 = HEAP32[$76>>2]|0;
   $78 = (($77) + ($74<<1)|0);
   $79 = HEAP16[$78>>1]|0;
   $80 = $79 << 16 >> 16;
   $81 = ($73|0)==($80|0);
   if (!($81)) {
    label = 17;
   }
  } else {
   label = 17;
  }
  if ((label|0) == 17) {
   label = 0;
   $82 = HEAP32[$v>>2]|0;
   $83 = HEAP32[$pTab>>2]|0;
   $84 = HEAP32[$2>>2]|0;
   $85 = HEAP32[$j>>2]|0;
   $86 = HEAP32[$1>>2]|0;
   $87 = (($86) + 4|0);
   $88 = HEAP32[$87>>2]|0;
   $89 = (($88) + ($85<<1)|0);
   $90 = HEAP16[$89>>1]|0;
   $91 = $90 << 16 >> 16;
   $92 = HEAP32[$regBase>>2]|0;
   $93 = HEAP32[$j>>2]|0;
   $94 = (($92) + ($93))|0;
   _sqlite3ExprCodeGetColumnOfTable($82,$83,$84,$91,$94);
   $95 = HEAP32[$v>>2]|0;
   (_sqlite3VdbeDeletePriorOpcode($95,39)|0);
  }
  $96 = HEAP32[$j>>2]|0;
  $97 = (($96) + 1)|0;
  HEAP32[$j>>2] = $97;
 }
 $98 = HEAP32[$3>>2]|0;
 $99 = ($98|0)!=(0);
 if (!($99)) {
  $104 = HEAP32[$0>>2]|0;
  $105 = HEAP32[$regBase>>2]|0;
  $106 = HEAP32[$nCol>>2]|0;
  _sqlite3ReleaseTempRange($104,$105,$106);
  $107 = HEAP32[$regBase>>2]|0;
  STACKTOP = sp;return ($107|0);
 }
 $100 = HEAP32[$v>>2]|0;
 $101 = HEAP32[$regBase>>2]|0;
 $102 = HEAP32[$nCol>>2]|0;
 $103 = HEAP32[$3>>2]|0;
 (_sqlite3VdbeAddOp3($100,49,$101,$102,$103)|0);
 $104 = HEAP32[$0>>2]|0;
 $105 = HEAP32[$regBase>>2]|0;
 $106 = HEAP32[$nCol>>2]|0;
 _sqlite3ReleaseTempRange($104,$105,$106);
 $107 = HEAP32[$regBase>>2]|0;
 STACKTOP = sp;return ($107|0);
}
function _sqlite3ResolvePartIdxLabel($pParse,$iLabel) {
 $pParse = $pParse|0;
 $iLabel = $iLabel|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $1 = sp;
 HEAP32[$0>>2] = $pParse;
 HEAP32[$1>>2] = $iLabel;
 $2 = HEAP32[$1>>2]|0;
 $3 = ($2|0)!=(0);
 if (!($3)) {
  STACKTOP = sp;return;
 }
 $4 = HEAP32[$0>>2]|0;
 $5 = (($4) + 8|0);
 $6 = HEAP32[$5>>2]|0;
 $7 = HEAP32[$1>>2]|0;
 _sqlite3VdbeResolveLabel($6,$7);
 $8 = HEAP32[$0>>2]|0;
 _sqlite3ExprCachePop($8);
 STACKTOP = sp;return;
}
function _sqlite3UniqueConstraint($pParse,$onError,$pIdx) {
 $pParse = $pParse|0;
 $onError = $onError|0;
 $pIdx = $pIdx|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $5 = 0;
 var $6 = 0, $7 = 0, $8 = 0, $9 = 0, $errMsg = 0, $j = 0, $pTab = 0, $zCol = 0, $zErr = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 64|0;
 $0 = sp + 52|0;
 $1 = sp + 48|0;
 $2 = sp + 44|0;
 $zErr = sp + 40|0;
 $j = sp + 36|0;
 $errMsg = sp + 8|0;
 $pTab = sp + 4|0;
 $zCol = sp;
 HEAP32[$0>>2] = $pParse;
 HEAP32[$1>>2] = $onError;
 HEAP32[$2>>2] = $pIdx;
 $3 = HEAP32[$2>>2]|0;
 $4 = (($3) + 12|0);
 $5 = HEAP32[$4>>2]|0;
 HEAP32[$pTab>>2] = $5;
 _sqlite3StrAccumInit($errMsg,0,0,200);
 $6 = HEAP32[$0>>2]|0;
 $7 = HEAP32[$6>>2]|0;
 HEAP32[$errMsg>>2] = $7;
 HEAP32[$j>>2] = 0;
 while(1) {
  $8 = HEAP32[$j>>2]|0;
  $9 = HEAP32[$2>>2]|0;
  $10 = (($9) + 50|0);
  $11 = HEAP16[$10>>1]|0;
  $12 = $11&65535;
  $13 = ($8|0)<($12|0);
  if (!($13)) {
   break;
  }
  $14 = HEAP32[$j>>2]|0;
  $15 = HEAP32[$2>>2]|0;
  $16 = (($15) + 4|0);
  $17 = HEAP32[$16>>2]|0;
  $18 = (($17) + ($14<<1)|0);
  $19 = HEAP16[$18>>1]|0;
  $20 = $19 << 16 >> 16;
  $21 = HEAP32[$pTab>>2]|0;
  $22 = (($21) + 4|0);
  $23 = HEAP32[$22>>2]|0;
  $24 = (($23) + (($20*24)|0)|0);
  $25 = HEAP32[$24>>2]|0;
  HEAP32[$zCol>>2] = $25;
  $26 = HEAP32[$j>>2]|0;
  $27 = ($26|0)!=(0);
  if ($27) {
   _sqlite3StrAccumAppend($errMsg,19112,2);
  }
  $28 = HEAP32[$pTab>>2]|0;
  $29 = HEAP32[$28>>2]|0;
  _sqlite3StrAccumAppendAll($errMsg,$29);
  _sqlite3StrAccumAppend($errMsg,19120,1);
  $30 = HEAP32[$zCol>>2]|0;
  _sqlite3StrAccumAppendAll($errMsg,$30);
  $31 = HEAP32[$j>>2]|0;
  $32 = (($31) + 1)|0;
  HEAP32[$j>>2] = $32;
 }
 $33 = (_sqlite3StrAccumFinish($errMsg)|0);
 HEAP32[$zErr>>2] = $33;
 $34 = HEAP32[$0>>2]|0;
 $35 = HEAP32[$2>>2]|0;
 $36 = (($35) + 55|0);
 $37 = HEAP8[$36>>0]|0;
 $38 = $37 & 3;
 $39 = $38&255;
 $40 = ($39|0)==(2);
 $41 = $40 ? 1555 : 2067;
 $42 = HEAP32[$1>>2]|0;
 $43 = HEAP32[$zErr>>2]|0;
 _sqlite3HaltConstraint($34,$41,$42,$43,-1,2);
 STACKTOP = sp;return;
}
function _sqlite3StrAccumAppendAll($p,$z) {
 $p = $p|0;
 $z = $z|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $1 = sp;
 HEAP32[$0>>2] = $p;
 HEAP32[$1>>2] = $z;
 $2 = HEAP32[$0>>2]|0;
 $3 = HEAP32[$1>>2]|0;
 $4 = HEAP32[$1>>2]|0;
 $5 = (_sqlite3Strlen30($4)|0);
 _sqlite3StrAccumAppend($2,$3,$5);
 STACKTOP = sp;return;
}
function _sqlite3HaltConstraint($pParse,$errCode,$onError,$p4,$p4type,$p5Errmsg) {
 $pParse = $pParse|0;
 $errCode = $errCode|0;
 $onError = $onError|0;
 $p4 = $p4|0;
 $p4type = $p4type|0;
 $p5Errmsg = $p5Errmsg|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0;
 var $9 = 0, $v = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 16|0;
 $1 = sp + 12|0;
 $2 = sp + 8|0;
 $3 = sp + 4|0;
 $4 = sp + 21|0;
 $5 = sp + 20|0;
 $v = sp;
 HEAP32[$0>>2] = $pParse;
 HEAP32[$1>>2] = $errCode;
 HEAP32[$2>>2] = $onError;
 HEAP32[$3>>2] = $p4;
 HEAP8[$4>>0] = $p4type;
 HEAP8[$5>>0] = $p5Errmsg;
 $6 = HEAP32[$0>>2]|0;
 $7 = (_sqlite3GetVdbe($6)|0);
 HEAP32[$v>>2] = $7;
 $8 = HEAP32[$2>>2]|0;
 $9 = ($8|0)==(2);
 if ($9) {
  $10 = HEAP32[$0>>2]|0;
  _sqlite3MayAbort($10);
 }
 $11 = HEAP32[$v>>2]|0;
 $12 = HEAP32[$1>>2]|0;
 $13 = HEAP32[$2>>2]|0;
 $14 = HEAP32[$3>>2]|0;
 $15 = HEAP8[$4>>0]|0;
 $16 = $15 << 24 >> 24;
 (_sqlite3VdbeAddOp4($11,24,$12,$13,0,$14,$16)|0);
 $17 = HEAP8[$5>>0]|0;
 $18 = ($17<<24>>24)!=(0);
 if (!($18)) {
  STACKTOP = sp;return;
 }
 $19 = HEAP32[$v>>2]|0;
 $20 = HEAP8[$5>>0]|0;
 _sqlite3VdbeChangeP5($19,$20);
 STACKTOP = sp;return;
}
function _sqlite3ExprCachePop($pParse) {
 $pParse = $pParse|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $i = 0, $p = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $i = sp + 4|0;
 $p = sp;
 HEAP32[$0>>2] = $pParse;
 $1 = HEAP32[$0>>2]|0;
 $2 = (($1) + 100|0);
 $3 = HEAP32[$2>>2]|0;
 $4 = (($3) + -1)|0;
 HEAP32[$2>>2] = $4;
 HEAP32[$i>>2] = 0;
 $5 = HEAP32[$0>>2]|0;
 $6 = (($5) + 116|0);
 HEAP32[$p>>2] = $6;
 while(1) {
  $7 = HEAP32[$i>>2]|0;
  $8 = ($7|0)<(10);
  if (!($8)) {
   break;
  }
  $9 = HEAP32[$p>>2]|0;
  $10 = (($9) + 12|0);
  $11 = HEAP32[$10>>2]|0;
  $12 = ($11|0)!=(0);
  if ($12) {
   $13 = HEAP32[$p>>2]|0;
   $14 = (($13) + 8|0);
   $15 = HEAP32[$14>>2]|0;
   $16 = HEAP32[$0>>2]|0;
   $17 = (($16) + 100|0);
   $18 = HEAP32[$17>>2]|0;
   $19 = ($15|0)>($18|0);
   if ($19) {
    $20 = HEAP32[$0>>2]|0;
    $21 = HEAP32[$p>>2]|0;
    _cacheEntryClear($20,$21);
    $22 = HEAP32[$p>>2]|0;
    $23 = (($22) + 12|0);
    HEAP32[$23>>2] = 0;
   }
  }
  $24 = HEAP32[$i>>2]|0;
  $25 = (($24) + 1)|0;
  HEAP32[$i>>2] = $25;
  $26 = HEAP32[$p>>2]|0;
  $27 = (($26) + 20|0);
  HEAP32[$p>>2] = $27;
 }
 STACKTOP = sp;return;
}
function _cacheEntryClear($pParse,$p) {
 $pParse = $pParse|0;
 $p = $p|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $3 = 0, $4 = 0, $5 = 0;
 var $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $1 = sp;
 HEAP32[$0>>2] = $pParse;
 HEAP32[$1>>2] = $p;
 $2 = HEAP32[$1>>2]|0;
 $3 = (($2) + 6|0);
 $4 = HEAP8[$3>>0]|0;
 $5 = ($4<<24>>24)!=(0);
 if (!($5)) {
  STACKTOP = sp;return;
 }
 $6 = HEAP32[$0>>2]|0;
 $7 = (($6) + 19|0);
 $8 = HEAP8[$7>>0]|0;
 $9 = $8&255;
 $10 = ($9|0)<(8);
 if ($10) {
  $11 = HEAP32[$1>>2]|0;
  $12 = (($11) + 12|0);
  $13 = HEAP32[$12>>2]|0;
  $14 = HEAP32[$0>>2]|0;
  $15 = (($14) + 19|0);
  $16 = HEAP8[$15>>0]|0;
  $17 = (($16) + 1)<<24>>24;
  HEAP8[$15>>0] = $17;
  $18 = $16&255;
  $19 = HEAP32[$0>>2]|0;
  $20 = (($19) + 24|0);
  $21 = (($20) + ($18<<2)|0);
  HEAP32[$21>>2] = $13;
 }
 $22 = HEAP32[$1>>2]|0;
 $23 = (($22) + 6|0);
 HEAP8[$23>>0] = 0;
 STACKTOP = sp;return;
}
function _sqlite3ExprCachePush($pParse) {
 $pParse = $pParse|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 HEAP32[$0>>2] = $pParse;
 $1 = HEAP32[$0>>2]|0;
 $2 = (($1) + 100|0);
 $3 = HEAP32[$2>>2]|0;
 $4 = (($3) + 1)|0;
 HEAP32[$2>>2] = $4;
 STACKTOP = sp;return;
}
function _sqlite3ExprIfFalse($pParse,$pExpr,$dest,$jumpIfNull) {
 $pParse = $pParse|0;
 $pExpr = $pExpr|0;
 $dest = $dest|0;
 $jumpIfNull = $jumpIfNull|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0;
 var $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0;
 var $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0;
 var $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0;
 var $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $d2 = 0, $destIfNull = 0, $op = 0, $or$cond = 0, $r1 = 0;
 var $r2 = 0, $regFree1 = 0, $regFree2 = 0, $v = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $0 = sp + 44|0;
 $1 = sp + 40|0;
 $2 = sp + 36|0;
 $3 = sp + 32|0;
 $v = sp + 28|0;
 $op = sp + 24|0;
 $regFree1 = sp + 20|0;
 $regFree2 = sp + 16|0;
 $r1 = sp + 12|0;
 $r2 = sp + 8|0;
 $d2 = sp + 4|0;
 $destIfNull = sp;
 HEAP32[$0>>2] = $pParse;
 HEAP32[$1>>2] = $pExpr;
 HEAP32[$2>>2] = $dest;
 HEAP32[$3>>2] = $jumpIfNull;
 $4 = HEAP32[$0>>2]|0;
 $5 = (($4) + 8|0);
 $6 = HEAP32[$5>>2]|0;
 HEAP32[$v>>2] = $6;
 HEAP32[$op>>2] = 0;
 HEAP32[$regFree1>>2] = 0;
 HEAP32[$regFree2>>2] = 0;
 $7 = HEAP32[$v>>2]|0;
 $8 = ($7|0)==(0|0);
 $9 = HEAP32[$1>>2]|0;
 $10 = ($9|0)==(0|0);
 $or$cond = $8 | $10;
 if ($or$cond) {
  STACKTOP = sp;return;
 }
 $11 = HEAP32[$1>>2]|0;
 $12 = HEAP8[$11>>0]|0;
 $13 = $12&255;
 $14 = (($13) + 0)|0;
 $15 = $14 ^ 1;
 $16 = (($15) - 0)|0;
 HEAP32[$op>>2] = $16;
 $17 = HEAP32[$1>>2]|0;
 $18 = HEAP8[$17>>0]|0;
 $19 = $18&255;
 L4: do {
  switch ($19|0) {
  case 79: case 78: case 83: case 80: case 81: case 82:  {
   $59 = HEAP32[$0>>2]|0;
   $60 = HEAP32[$1>>2]|0;
   $61 = (($60) + 12|0);
   $62 = HEAP32[$61>>2]|0;
   $63 = (_sqlite3ExprCodeTemp($59,$62,$regFree1)|0);
   HEAP32[$r1>>2] = $63;
   $64 = HEAP32[$0>>2]|0;
   $65 = HEAP32[$1>>2]|0;
   $66 = (($65) + 16|0);
   $67 = HEAP32[$66>>2]|0;
   $68 = (_sqlite3ExprCodeTemp($64,$67,$regFree2)|0);
   HEAP32[$r2>>2] = $68;
   $69 = HEAP32[$0>>2]|0;
   $70 = HEAP32[$1>>2]|0;
   $71 = (($70) + 12|0);
   $72 = HEAP32[$71>>2]|0;
   $73 = HEAP32[$1>>2]|0;
   $74 = (($73) + 16|0);
   $75 = HEAP32[$74>>2]|0;
   $76 = HEAP32[$op>>2]|0;
   $77 = HEAP32[$r1>>2]|0;
   $78 = HEAP32[$r2>>2]|0;
   $79 = HEAP32[$2>>2]|0;
   $80 = HEAP32[$3>>2]|0;
   (_codeCompare($69,$72,$75,$76,$77,$78,$79,$80)|0);
   break;
  }
  case 72:  {
   $20 = HEAP32[$0>>2]|0;
   $21 = HEAP32[$1>>2]|0;
   $22 = (($21) + 12|0);
   $23 = HEAP32[$22>>2]|0;
   $24 = HEAP32[$2>>2]|0;
   $25 = HEAP32[$3>>2]|0;
   _sqlite3ExprIfFalse($20,$23,$24,$25);
   $26 = HEAP32[$0>>2]|0;
   _sqlite3ExprCachePush($26);
   $27 = HEAP32[$0>>2]|0;
   $28 = HEAP32[$1>>2]|0;
   $29 = (($28) + 16|0);
   $30 = HEAP32[$29>>2]|0;
   $31 = HEAP32[$2>>2]|0;
   $32 = HEAP32[$3>>2]|0;
   _sqlite3ExprIfFalse($27,$30,$31,$32);
   $33 = HEAP32[$0>>2]|0;
   _sqlite3ExprCachePop($33);
   break;
  }
  case 71:  {
   $34 = HEAP32[$v>>2]|0;
   $35 = (_sqlite3VdbeMakeLabel($34)|0);
   HEAP32[$d2>>2] = $35;
   $36 = HEAP32[$0>>2]|0;
   $37 = HEAP32[$1>>2]|0;
   $38 = (($37) + 12|0);
   $39 = HEAP32[$38>>2]|0;
   $40 = HEAP32[$d2>>2]|0;
   $41 = HEAP32[$3>>2]|0;
   $42 = $41 ^ 16;
   _sqlite3ExprIfTrue($36,$39,$40,$42);
   $43 = HEAP32[$0>>2]|0;
   _sqlite3ExprCachePush($43);
   $44 = HEAP32[$0>>2]|0;
   $45 = HEAP32[$1>>2]|0;
   $46 = (($45) + 16|0);
   $47 = HEAP32[$46>>2]|0;
   $48 = HEAP32[$2>>2]|0;
   $49 = HEAP32[$3>>2]|0;
   _sqlite3ExprIfFalse($44,$47,$48,$49);
   $50 = HEAP32[$v>>2]|0;
   $51 = HEAP32[$d2>>2]|0;
   _sqlite3VdbeResolveLabel($50,$51);
   $52 = HEAP32[$0>>2]|0;
   _sqlite3ExprCachePop($52);
   break;
  }
  case 77: case 76:  {
   $107 = HEAP32[$0>>2]|0;
   $108 = HEAP32[$1>>2]|0;
   $109 = (($108) + 12|0);
   $110 = HEAP32[$109>>2]|0;
   $111 = (_sqlite3ExprCodeTemp($107,$110,$regFree1)|0);
   HEAP32[$r1>>2] = $111;
   $112 = HEAP32[$v>>2]|0;
   $113 = HEAP32[$op>>2]|0;
   $114 = HEAP32[$r1>>2]|0;
   $115 = HEAP32[$2>>2]|0;
   (_sqlite3VdbeAddOp2($112,$113,$114,$115)|0);
   break;
  }
  case 74:  {
   $116 = HEAP32[$0>>2]|0;
   $117 = HEAP32[$1>>2]|0;
   $118 = HEAP32[$2>>2]|0;
   $119 = HEAP32[$3>>2]|0;
   _exprCodeBetween($116,$117,$118,0,$119);
   break;
  }
  case 75:  {
   $120 = HEAP32[$3>>2]|0;
   $121 = ($120|0)!=(0);
   if ($121) {
    $122 = HEAP32[$0>>2]|0;
    $123 = HEAP32[$1>>2]|0;
    $124 = HEAP32[$2>>2]|0;
    $125 = HEAP32[$2>>2]|0;
    _sqlite3ExprCodeIN($122,$123,$124,$125);
    break L4;
   } else {
    $126 = HEAP32[$v>>2]|0;
    $127 = (_sqlite3VdbeMakeLabel($126)|0);
    HEAP32[$destIfNull>>2] = $127;
    $128 = HEAP32[$0>>2]|0;
    $129 = HEAP32[$1>>2]|0;
    $130 = HEAP32[$2>>2]|0;
    $131 = HEAP32[$destIfNull>>2]|0;
    _sqlite3ExprCodeIN($128,$129,$130,$131);
    $132 = HEAP32[$v>>2]|0;
    $133 = HEAP32[$destIfNull>>2]|0;
    _sqlite3VdbeResolveLabel($132,$133);
    break L4;
   }
   break;
  }
  case 148: case 73:  {
   $81 = HEAP32[$0>>2]|0;
   $82 = HEAP32[$1>>2]|0;
   $83 = (($82) + 12|0);
   $84 = HEAP32[$83>>2]|0;
   $85 = (_sqlite3ExprCodeTemp($81,$84,$regFree1)|0);
   HEAP32[$r1>>2] = $85;
   $86 = HEAP32[$0>>2]|0;
   $87 = HEAP32[$1>>2]|0;
   $88 = (($87) + 16|0);
   $89 = HEAP32[$88>>2]|0;
   $90 = (_sqlite3ExprCodeTemp($86,$89,$regFree2)|0);
   HEAP32[$r2>>2] = $90;
   $91 = HEAP32[$1>>2]|0;
   $92 = HEAP8[$91>>0]|0;
   $93 = $92&255;
   $94 = ($93|0)==(73);
   $95 = $94 ? 78 : 79;
   HEAP32[$op>>2] = $95;
   $96 = HEAP32[$0>>2]|0;
   $97 = HEAP32[$1>>2]|0;
   $98 = (($97) + 12|0);
   $99 = HEAP32[$98>>2]|0;
   $100 = HEAP32[$1>>2]|0;
   $101 = (($100) + 16|0);
   $102 = HEAP32[$101>>2]|0;
   $103 = HEAP32[$op>>2]|0;
   $104 = HEAP32[$r1>>2]|0;
   $105 = HEAP32[$r2>>2]|0;
   $106 = HEAP32[$2>>2]|0;
   (_codeCompare($96,$99,$102,$103,$104,$105,$106,128)|0);
   break;
  }
  case 19:  {
   $53 = HEAP32[$0>>2]|0;
   $54 = HEAP32[$1>>2]|0;
   $55 = (($54) + 12|0);
   $56 = HEAP32[$55>>2]|0;
   $57 = HEAP32[$2>>2]|0;
   $58 = HEAP32[$3>>2]|0;
   _sqlite3ExprIfTrue($53,$56,$57,$58);
   break;
  }
  default: {
   $134 = HEAP32[$1>>2]|0;
   $135 = (_exprAlwaysFalse($134)|0);
   $136 = ($135|0)!=(0);
   if ($136) {
    $137 = HEAP32[$v>>2]|0;
    $138 = HEAP32[$2>>2]|0;
    (_sqlite3VdbeAddOp2($137,16,0,$138)|0);
    break L4;
   }
   $139 = HEAP32[$1>>2]|0;
   $140 = (_exprAlwaysTrue($139)|0);
   $141 = ($140|0)!=(0);
   if (!($141)) {
    $142 = HEAP32[$0>>2]|0;
    $143 = HEAP32[$1>>2]|0;
    $144 = (_sqlite3ExprCodeTemp($142,$143,$regFree1)|0);
    HEAP32[$r1>>2] = $144;
    $145 = HEAP32[$v>>2]|0;
    $146 = HEAP32[$r1>>2]|0;
    $147 = HEAP32[$2>>2]|0;
    $148 = HEAP32[$3>>2]|0;
    $149 = ($148|0)!=(0);
    $150 = $149&1;
    (_sqlite3VdbeAddOp3($145,46,$146,$147,$150)|0);
   }
  }
  }
 } while(0);
 $151 = HEAP32[$0>>2]|0;
 $152 = HEAP32[$regFree1>>2]|0;
 _sqlite3ReleaseTempReg($151,$152);
 $153 = HEAP32[$0>>2]|0;
 $154 = HEAP32[$regFree2>>2]|0;
 _sqlite3ReleaseTempReg($153,$154);
 STACKTOP = sp;return;
}
function _sqlite3GetTempRange($pParse,$nReg) {
 $pParse = $pParse|0;
 $nReg = $nReg|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $i = 0, $n = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $1 = sp + 8|0;
 $i = sp + 4|0;
 $n = sp;
 HEAP32[$0>>2] = $pParse;
 HEAP32[$1>>2] = $nReg;
 $2 = HEAP32[$0>>2]|0;
 $3 = (($2) + 60|0);
 $4 = HEAP32[$3>>2]|0;
 HEAP32[$i>>2] = $4;
 $5 = HEAP32[$0>>2]|0;
 $6 = (($5) + 56|0);
 $7 = HEAP32[$6>>2]|0;
 HEAP32[$n>>2] = $7;
 $8 = HEAP32[$1>>2]|0;
 $9 = HEAP32[$n>>2]|0;
 $10 = ($8|0)<=($9|0);
 if ($10) {
  $11 = HEAP32[$1>>2]|0;
  $12 = HEAP32[$0>>2]|0;
  $13 = (($12) + 60|0);
  $14 = HEAP32[$13>>2]|0;
  $15 = (($14) + ($11))|0;
  HEAP32[$13>>2] = $15;
  $16 = HEAP32[$1>>2]|0;
  $17 = HEAP32[$0>>2]|0;
  $18 = (($17) + 56|0);
  $19 = HEAP32[$18>>2]|0;
  $20 = (($19) - ($16))|0;
  HEAP32[$18>>2] = $20;
  $30 = HEAP32[$i>>2]|0;
  STACKTOP = sp;return ($30|0);
 } else {
  $21 = HEAP32[$0>>2]|0;
  $22 = (($21) + 72|0);
  $23 = HEAP32[$22>>2]|0;
  $24 = (($23) + 1)|0;
  HEAP32[$i>>2] = $24;
  $25 = HEAP32[$1>>2]|0;
  $26 = HEAP32[$0>>2]|0;
  $27 = (($26) + 72|0);
  $28 = HEAP32[$27>>2]|0;
  $29 = (($28) + ($25))|0;
  HEAP32[$27>>2] = $29;
  $30 = HEAP32[$i>>2]|0;
  STACKTOP = sp;return ($30|0);
 }
 return 0|0;
}
function _sqlite3ExprCodeGetColumnOfTable($v,$pTab,$iTabCur,$iCol,$regOut) {
 $v = $v|0;
 $pTab = $pTab|0;
 $iTabCur = $iTabCur|0;
 $iCol = $iCol|0;
 $regOut = $regOut|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $op = 0, $x = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 24|0;
 $1 = sp + 20|0;
 $2 = sp + 16|0;
 $3 = sp + 12|0;
 $4 = sp + 8|0;
 $op = sp + 4|0;
 $x = sp;
 HEAP32[$0>>2] = $v;
 HEAP32[$1>>2] = $pTab;
 HEAP32[$2>>2] = $iTabCur;
 HEAP32[$3>>2] = $iCol;
 HEAP32[$4>>2] = $regOut;
 $5 = HEAP32[$3>>2]|0;
 $6 = ($5|0)<(0);
 if ($6) {
  label = 3;
 } else {
  $7 = HEAP32[$3>>2]|0;
  $8 = HEAP32[$1>>2]|0;
  $9 = (($8) + 36|0);
  $10 = HEAP16[$9>>1]|0;
  $11 = $10 << 16 >> 16;
  $12 = ($7|0)==($11|0);
  if ($12) {
   label = 3;
  } else {
   $16 = HEAP32[$1>>2]|0;
   $17 = (($16) + 44|0);
   $18 = HEAP8[$17>>0]|0;
   $19 = $18&255;
   $20 = $19 & 16;
   $21 = ($20|0)!=(0);
   $22 = $21 ? 148 : 47;
   HEAP32[$op>>2] = $22;
   $23 = HEAP32[$3>>2]|0;
   HEAP32[$x>>2] = $23;
   $24 = HEAP32[$1>>2]|0;
   $25 = (($24) + 44|0);
   $26 = HEAP8[$25>>0]|0;
   $27 = $26&255;
   $28 = $27 & 32;
   $29 = ($28|0)==(0);
   if (!($29)) {
    $30 = HEAP32[$1>>2]|0;
    $31 = (_sqlite3PrimaryKeyIndex($30)|0);
    $32 = HEAP32[$3>>2]|0;
    $33 = $32&65535;
    $34 = (_sqlite3ColumnOfIndex($31,$33)|0);
    $35 = $34 << 16 >> 16;
    HEAP32[$x>>2] = $35;
   }
   $36 = HEAP32[$0>>2]|0;
   $37 = HEAP32[$op>>2]|0;
   $38 = HEAP32[$2>>2]|0;
   $39 = HEAP32[$x>>2]|0;
   $40 = HEAP32[$4>>2]|0;
   (_sqlite3VdbeAddOp3($36,$37,$38,$39,$40)|0);
  }
 }
 if ((label|0) == 3) {
  $13 = HEAP32[$0>>2]|0;
  $14 = HEAP32[$2>>2]|0;
  $15 = HEAP32[$4>>2]|0;
  (_sqlite3VdbeAddOp2($13,103,$14,$15)|0);
 }
 $41 = HEAP32[$3>>2]|0;
 $42 = ($41|0)>=(0);
 if (!($42)) {
  STACKTOP = sp;return;
 }
 $43 = HEAP32[$0>>2]|0;
 $44 = HEAP32[$1>>2]|0;
 $45 = HEAP32[$3>>2]|0;
 $46 = HEAP32[$4>>2]|0;
 _sqlite3ColumnDefault($43,$44,$45,$46);
 STACKTOP = sp;return;
}
function _sqlite3VdbeDeletePriorOpcode($p,$op) {
 $p = $p|0;
 $op = $op|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $1 = sp;
 $2 = sp + 8|0;
 HEAP32[$1>>2] = $p;
 HEAP8[$2>>0] = $op;
 $3 = HEAP32[$1>>2]|0;
 $4 = (($3) + 32|0);
 $5 = HEAP32[$4>>2]|0;
 $6 = (($5) - 1)|0;
 $7 = HEAP32[$1>>2]|0;
 $8 = (($7) + 24|0);
 $9 = HEAP32[$8>>2]|0;
 $10 = (($9) + 88|0);
 $11 = HEAP32[$10>>2]|0;
 $12 = ($6|0)>($11|0);
 if ($12) {
  $13 = HEAP32[$1>>2]|0;
  $14 = (($13) + 32|0);
  $15 = HEAP32[$14>>2]|0;
  $16 = (($15) - 1)|0;
  $17 = HEAP32[$1>>2]|0;
  $18 = (($17) + 4|0);
  $19 = HEAP32[$18>>2]|0;
  $20 = (($19) + (($16*20)|0)|0);
  $21 = HEAP8[$20>>0]|0;
  $22 = $21&255;
  $23 = HEAP8[$2>>0]|0;
  $24 = $23&255;
  $25 = ($22|0)==($24|0);
  if ($25) {
   $26 = HEAP32[$1>>2]|0;
   $27 = HEAP32[$1>>2]|0;
   $28 = (($27) + 32|0);
   $29 = HEAP32[$28>>2]|0;
   $30 = (($29) - 1)|0;
   _sqlite3VdbeChangeToNoop($26,$30);
   HEAP32[$0>>2] = 1;
   $31 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($31|0);
  }
 }
 HEAP32[$0>>2] = 0;
 $31 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($31|0);
}
function _sqlite3ReleaseTempRange($pParse,$iReg,$nReg) {
 $pParse = $pParse|0;
 $iReg = $iReg|0;
 $nReg = $nReg|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $1 = sp + 4|0;
 $2 = sp;
 HEAP32[$0>>2] = $pParse;
 HEAP32[$1>>2] = $iReg;
 HEAP32[$2>>2] = $nReg;
 $3 = HEAP32[$0>>2]|0;
 $4 = HEAP32[$1>>2]|0;
 $5 = HEAP32[$2>>2]|0;
 _sqlite3ExprCacheRemove($3,$4,$5);
 $6 = HEAP32[$2>>2]|0;
 $7 = HEAP32[$0>>2]|0;
 $8 = (($7) + 56|0);
 $9 = HEAP32[$8>>2]|0;
 $10 = ($6|0)>($9|0);
 if (!($10)) {
  STACKTOP = sp;return;
 }
 $11 = HEAP32[$2>>2]|0;
 $12 = HEAP32[$0>>2]|0;
 $13 = (($12) + 56|0);
 HEAP32[$13>>2] = $11;
 $14 = HEAP32[$1>>2]|0;
 $15 = HEAP32[$0>>2]|0;
 $16 = (($15) + 60|0);
 HEAP32[$16>>2] = $14;
 STACKTOP = sp;return;
}
function _sqlite3ExprCacheRemove($pParse,$iReg,$nReg) {
 $pParse = $pParse|0;
 $iReg = $iReg|0;
 $nReg = $nReg|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $i = 0, $iLast = 0, $p = 0, $r = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 24|0;
 $1 = sp + 20|0;
 $2 = sp + 16|0;
 $i = sp + 12|0;
 $iLast = sp + 8|0;
 $p = sp + 4|0;
 $r = sp;
 HEAP32[$0>>2] = $pParse;
 HEAP32[$1>>2] = $iReg;
 HEAP32[$2>>2] = $nReg;
 $3 = HEAP32[$1>>2]|0;
 $4 = HEAP32[$2>>2]|0;
 $5 = (($3) + ($4))|0;
 $6 = (($5) - 1)|0;
 HEAP32[$iLast>>2] = $6;
 HEAP32[$i>>2] = 0;
 $7 = HEAP32[$0>>2]|0;
 $8 = (($7) + 116|0);
 HEAP32[$p>>2] = $8;
 while(1) {
  $9 = HEAP32[$i>>2]|0;
  $10 = ($9|0)<(10);
  if (!($10)) {
   break;
  }
  $11 = HEAP32[$p>>2]|0;
  $12 = (($11) + 12|0);
  $13 = HEAP32[$12>>2]|0;
  HEAP32[$r>>2] = $13;
  $14 = HEAP32[$r>>2]|0;
  $15 = HEAP32[$1>>2]|0;
  $16 = ($14|0)>=($15|0);
  if ($16) {
   $17 = HEAP32[$r>>2]|0;
   $18 = HEAP32[$iLast>>2]|0;
   $19 = ($17|0)<=($18|0);
   if ($19) {
    $20 = HEAP32[$0>>2]|0;
    $21 = HEAP32[$p>>2]|0;
    _cacheEntryClear($20,$21);
    $22 = HEAP32[$p>>2]|0;
    $23 = (($22) + 12|0);
    HEAP32[$23>>2] = 0;
   }
  }
  $24 = HEAP32[$i>>2]|0;
  $25 = (($24) + 1)|0;
  HEAP32[$i>>2] = $25;
  $26 = HEAP32[$p>>2]|0;
  $27 = (($26) + 20|0);
  HEAP32[$p>>2] = $27;
 }
 STACKTOP = sp;return;
}
function _sqlite3ColumnOfIndex($pIdx,$iCol) {
 $pIdx = $pIdx|0;
 $iCol = $iCol|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0;
 var $7 = 0, $8 = 0, $9 = 0, $i = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 10|0;
 $1 = sp + 4|0;
 $2 = sp + 8|0;
 $i = sp;
 HEAP32[$1>>2] = $pIdx;
 HEAP16[$2>>1] = $iCol;
 HEAP32[$i>>2] = 0;
 while(1) {
  $3 = HEAP32[$i>>2]|0;
  $4 = HEAP32[$1>>2]|0;
  $5 = (($4) + 52|0);
  $6 = HEAP16[$5>>1]|0;
  $7 = $6&65535;
  $8 = ($3|0)<($7|0);
  if (!($8)) {
   label = 6;
   break;
  }
  $9 = HEAP16[$2>>1]|0;
  $10 = $9 << 16 >> 16;
  $11 = HEAP32[$i>>2]|0;
  $12 = HEAP32[$1>>2]|0;
  $13 = (($12) + 4|0);
  $14 = HEAP32[$13>>2]|0;
  $15 = (($14) + ($11<<1)|0);
  $16 = HEAP16[$15>>1]|0;
  $17 = $16 << 16 >> 16;
  $18 = ($10|0)==($17|0);
  $19 = HEAP32[$i>>2]|0;
  if ($18) {
   label = 4;
   break;
  }
  $21 = (($19) + 1)|0;
  HEAP32[$i>>2] = $21;
 }
 if ((label|0) == 4) {
  $20 = $19&65535;
  HEAP16[$0>>1] = $20;
  $22 = HEAP16[$0>>1]|0;
  STACKTOP = sp;return ($22|0);
 }
 else if ((label|0) == 6) {
  HEAP16[$0>>1] = -1;
  $22 = HEAP16[$0>>1]|0;
  STACKTOP = sp;return ($22|0);
 }
 return 0|0;
}
function _sqlite3ColumnDefault($v,$pTab,$i,$iReg) {
 $v = $v|0;
 $pTab = $pTab|0;
 $i = $i|0;
 $iReg = $iReg|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $enc = 0, $pCol = 0, $pValue = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 20|0;
 $1 = sp + 16|0;
 $2 = sp + 12|0;
 $3 = sp + 8|0;
 $pValue = sp + 4|0;
 $enc = sp + 24|0;
 $pCol = sp;
 HEAP32[$0>>2] = $v;
 HEAP32[$1>>2] = $pTab;
 HEAP32[$2>>2] = $i;
 HEAP32[$3>>2] = $iReg;
 $4 = HEAP32[$1>>2]|0;
 $5 = (($4) + 12|0);
 $6 = HEAP32[$5>>2]|0;
 $7 = ($6|0)!=(0|0);
 if ($7) {
  STACKTOP = sp;return;
 }
 HEAP32[$pValue>>2] = 0;
 $8 = HEAP32[$0>>2]|0;
 $9 = (_sqlite3VdbeDb($8)|0);
 $10 = (($9) + 16|0);
 $11 = HEAP32[$10>>2]|0;
 $12 = (($11) + 12|0);
 $13 = HEAP32[$12>>2]|0;
 $14 = (($13) + 77|0);
 $15 = HEAP8[$14>>0]|0;
 HEAP8[$enc>>0] = $15;
 $16 = HEAP32[$2>>2]|0;
 $17 = HEAP32[$1>>2]|0;
 $18 = (($17) + 4|0);
 $19 = HEAP32[$18>>2]|0;
 $20 = (($19) + (($16*24)|0)|0);
 HEAP32[$pCol>>2] = $20;
 $21 = HEAP32[$0>>2]|0;
 $22 = (_sqlite3VdbeDb($21)|0);
 $23 = HEAP32[$pCol>>2]|0;
 $24 = (($23) + 4|0);
 $25 = HEAP32[$24>>2]|0;
 $26 = HEAP8[$enc>>0]|0;
 $27 = HEAP32[$pCol>>2]|0;
 $28 = (($27) + 21|0);
 $29 = HEAP8[$28>>0]|0;
 (_sqlite3ValueFromExpr($22,$25,$26,$29,$pValue)|0);
 $30 = HEAP32[$pValue>>2]|0;
 $31 = ($30|0)!=(0|0);
 if ($31) {
  $32 = HEAP32[$0>>2]|0;
  $33 = HEAP32[$pValue>>2]|0;
  _sqlite3VdbeChangeP4($32,-1,$33,-8);
 }
 $34 = HEAP32[$2>>2]|0;
 $35 = HEAP32[$1>>2]|0;
 $36 = (($35) + 4|0);
 $37 = HEAP32[$36>>2]|0;
 $38 = (($37) + (($34*24)|0)|0);
 $39 = (($38) + 21|0);
 $40 = HEAP8[$39>>0]|0;
 $41 = $40 << 24 >> 24;
 $42 = ($41|0)==(69);
 if (!($42)) {
  STACKTOP = sp;return;
 }
 $43 = HEAP32[$0>>2]|0;
 $44 = HEAP32[$3>>2]|0;
 (_sqlite3VdbeAddOp1($43,39,$44)|0);
 STACKTOP = sp;return;
}
function _sqlite3VdbeDb($v) {
 $v = $v|0;
 var $0 = 0, $1 = 0, $2 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 HEAP32[$0>>2] = $v;
 $1 = HEAP32[$0>>2]|0;
 $2 = HEAP32[$1>>2]|0;
 STACKTOP = sp;return ($2|0);
}
function _sqlite3ExprIfTrue($pParse,$pExpr,$dest,$jumpIfNull) {
 $pParse = $pParse|0;
 $pExpr = $pExpr|0;
 $dest = $dest|0;
 $jumpIfNull = $jumpIfNull|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0;
 var $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0;
 var $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0;
 var $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0;
 var $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0;
 var $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $d2 = 0, $destIfFalse = 0, $destIfNull = 0, $op = 0, $or$cond = 0, $r1 = 0, $r2 = 0, $regFree1 = 0, $regFree2 = 0, $v = 0, label = 0;
 var sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 64|0;
 $0 = sp + 48|0;
 $1 = sp + 44|0;
 $2 = sp + 40|0;
 $3 = sp + 36|0;
 $v = sp + 32|0;
 $op = sp + 28|0;
 $regFree1 = sp + 24|0;
 $regFree2 = sp + 20|0;
 $r1 = sp + 16|0;
 $r2 = sp + 12|0;
 $d2 = sp + 8|0;
 $destIfFalse = sp + 4|0;
 $destIfNull = sp;
 HEAP32[$0>>2] = $pParse;
 HEAP32[$1>>2] = $pExpr;
 HEAP32[$2>>2] = $dest;
 HEAP32[$3>>2] = $jumpIfNull;
 $4 = HEAP32[$0>>2]|0;
 $5 = (($4) + 8|0);
 $6 = HEAP32[$5>>2]|0;
 HEAP32[$v>>2] = $6;
 HEAP32[$op>>2] = 0;
 HEAP32[$regFree1>>2] = 0;
 HEAP32[$regFree2>>2] = 0;
 $7 = HEAP32[$v>>2]|0;
 $8 = ($7|0)==(0|0);
 $9 = HEAP32[$1>>2]|0;
 $10 = ($9|0)==(0|0);
 $or$cond = $8 | $10;
 if ($or$cond) {
  STACKTOP = sp;return;
 }
 $11 = HEAP32[$1>>2]|0;
 $12 = HEAP8[$11>>0]|0;
 $13 = $12&255;
 HEAP32[$op>>2] = $13;
 $14 = HEAP32[$op>>2]|0;
 L4: do {
  switch ($14|0) {
  case 72:  {
   $15 = HEAP32[$v>>2]|0;
   $16 = (_sqlite3VdbeMakeLabel($15)|0);
   HEAP32[$d2>>2] = $16;
   $17 = HEAP32[$0>>2]|0;
   $18 = HEAP32[$1>>2]|0;
   $19 = (($18) + 12|0);
   $20 = HEAP32[$19>>2]|0;
   $21 = HEAP32[$d2>>2]|0;
   $22 = HEAP32[$3>>2]|0;
   $23 = $22 ^ 16;
   _sqlite3ExprIfFalse($17,$20,$21,$23);
   $24 = HEAP32[$0>>2]|0;
   _sqlite3ExprCachePush($24);
   $25 = HEAP32[$0>>2]|0;
   $26 = HEAP32[$1>>2]|0;
   $27 = (($26) + 16|0);
   $28 = HEAP32[$27>>2]|0;
   $29 = HEAP32[$2>>2]|0;
   $30 = HEAP32[$3>>2]|0;
   _sqlite3ExprIfTrue($25,$28,$29,$30);
   $31 = HEAP32[$v>>2]|0;
   $32 = HEAP32[$d2>>2]|0;
   _sqlite3VdbeResolveLabel($31,$32);
   $33 = HEAP32[$0>>2]|0;
   _sqlite3ExprCachePop($33);
   break;
  }
  case 71:  {
   $34 = HEAP32[$0>>2]|0;
   $35 = HEAP32[$1>>2]|0;
   $36 = (($35) + 12|0);
   $37 = HEAP32[$36>>2]|0;
   $38 = HEAP32[$2>>2]|0;
   $39 = HEAP32[$3>>2]|0;
   _sqlite3ExprIfTrue($34,$37,$38,$39);
   $40 = HEAP32[$0>>2]|0;
   _sqlite3ExprCachePush($40);
   $41 = HEAP32[$0>>2]|0;
   $42 = HEAP32[$1>>2]|0;
   $43 = (($42) + 16|0);
   $44 = HEAP32[$43>>2]|0;
   $45 = HEAP32[$2>>2]|0;
   $46 = HEAP32[$3>>2]|0;
   _sqlite3ExprIfTrue($41,$44,$45,$46);
   $47 = HEAP32[$0>>2]|0;
   _sqlite3ExprCachePop($47);
   break;
  }
  case 74:  {
   $109 = HEAP32[$0>>2]|0;
   $110 = HEAP32[$1>>2]|0;
   $111 = HEAP32[$2>>2]|0;
   $112 = HEAP32[$3>>2]|0;
   _exprCodeBetween($109,$110,$111,1,$112);
   break;
  }
  case 75:  {
   $113 = HEAP32[$v>>2]|0;
   $114 = (_sqlite3VdbeMakeLabel($113)|0);
   HEAP32[$destIfFalse>>2] = $114;
   $115 = HEAP32[$3>>2]|0;
   $116 = ($115|0)!=(0);
   $117 = HEAP32[$2>>2]|0;
   $118 = HEAP32[$destIfFalse>>2]|0;
   $119 = $116 ? $117 : $118;
   HEAP32[$destIfNull>>2] = $119;
   $120 = HEAP32[$0>>2]|0;
   $121 = HEAP32[$1>>2]|0;
   $122 = HEAP32[$destIfFalse>>2]|0;
   $123 = HEAP32[$destIfNull>>2]|0;
   _sqlite3ExprCodeIN($120,$121,$122,$123);
   $124 = HEAP32[$v>>2]|0;
   $125 = HEAP32[$2>>2]|0;
   (_sqlite3VdbeAddOp2($124,16,0,$125)|0);
   $126 = HEAP32[$v>>2]|0;
   $127 = HEAP32[$destIfFalse>>2]|0;
   _sqlite3VdbeResolveLabel($126,$127);
   break;
  }
  case 148: case 73:  {
   $76 = HEAP32[$0>>2]|0;
   $77 = HEAP32[$1>>2]|0;
   $78 = (($77) + 12|0);
   $79 = HEAP32[$78>>2]|0;
   $80 = (_sqlite3ExprCodeTemp($76,$79,$regFree1)|0);
   HEAP32[$r1>>2] = $80;
   $81 = HEAP32[$0>>2]|0;
   $82 = HEAP32[$1>>2]|0;
   $83 = (($82) + 16|0);
   $84 = HEAP32[$83>>2]|0;
   $85 = (_sqlite3ExprCodeTemp($81,$84,$regFree2)|0);
   HEAP32[$r2>>2] = $85;
   $86 = HEAP32[$op>>2]|0;
   $87 = ($86|0)==(73);
   $88 = $87 ? 79 : 78;
   HEAP32[$op>>2] = $88;
   $89 = HEAP32[$0>>2]|0;
   $90 = HEAP32[$1>>2]|0;
   $91 = (($90) + 12|0);
   $92 = HEAP32[$91>>2]|0;
   $93 = HEAP32[$1>>2]|0;
   $94 = (($93) + 16|0);
   $95 = HEAP32[$94>>2]|0;
   $96 = HEAP32[$op>>2]|0;
   $97 = HEAP32[$r1>>2]|0;
   $98 = HEAP32[$r2>>2]|0;
   $99 = HEAP32[$2>>2]|0;
   (_codeCompare($89,$92,$95,$96,$97,$98,$99,128)|0);
   break;
  }
  case 79: case 78: case 83: case 80: case 81: case 82:  {
   $54 = HEAP32[$0>>2]|0;
   $55 = HEAP32[$1>>2]|0;
   $56 = (($55) + 12|0);
   $57 = HEAP32[$56>>2]|0;
   $58 = (_sqlite3ExprCodeTemp($54,$57,$regFree1)|0);
   HEAP32[$r1>>2] = $58;
   $59 = HEAP32[$0>>2]|0;
   $60 = HEAP32[$1>>2]|0;
   $61 = (($60) + 16|0);
   $62 = HEAP32[$61>>2]|0;
   $63 = (_sqlite3ExprCodeTemp($59,$62,$regFree2)|0);
   HEAP32[$r2>>2] = $63;
   $64 = HEAP32[$0>>2]|0;
   $65 = HEAP32[$1>>2]|0;
   $66 = (($65) + 12|0);
   $67 = HEAP32[$66>>2]|0;
   $68 = HEAP32[$1>>2]|0;
   $69 = (($68) + 16|0);
   $70 = HEAP32[$69>>2]|0;
   $71 = HEAP32[$op>>2]|0;
   $72 = HEAP32[$r1>>2]|0;
   $73 = HEAP32[$r2>>2]|0;
   $74 = HEAP32[$2>>2]|0;
   $75 = HEAP32[$3>>2]|0;
   (_codeCompare($64,$67,$70,$71,$72,$73,$74,$75)|0);
   break;
  }
  case 77: case 76:  {
   $100 = HEAP32[$0>>2]|0;
   $101 = HEAP32[$1>>2]|0;
   $102 = (($101) + 12|0);
   $103 = HEAP32[$102>>2]|0;
   $104 = (_sqlite3ExprCodeTemp($100,$103,$regFree1)|0);
   HEAP32[$r1>>2] = $104;
   $105 = HEAP32[$v>>2]|0;
   $106 = HEAP32[$op>>2]|0;
   $107 = HEAP32[$r1>>2]|0;
   $108 = HEAP32[$2>>2]|0;
   (_sqlite3VdbeAddOp2($105,$106,$107,$108)|0);
   break;
  }
  case 19:  {
   $48 = HEAP32[$0>>2]|0;
   $49 = HEAP32[$1>>2]|0;
   $50 = (($49) + 12|0);
   $51 = HEAP32[$50>>2]|0;
   $52 = HEAP32[$2>>2]|0;
   $53 = HEAP32[$3>>2]|0;
   _sqlite3ExprIfFalse($48,$51,$52,$53);
   break;
  }
  default: {
   $128 = HEAP32[$1>>2]|0;
   $129 = (_exprAlwaysTrue($128)|0);
   $130 = ($129|0)!=(0);
   if ($130) {
    $131 = HEAP32[$v>>2]|0;
    $132 = HEAP32[$2>>2]|0;
    (_sqlite3VdbeAddOp2($131,16,0,$132)|0);
    break L4;
   }
   $133 = HEAP32[$1>>2]|0;
   $134 = (_exprAlwaysFalse($133)|0);
   $135 = ($134|0)!=(0);
   if (!($135)) {
    $136 = HEAP32[$0>>2]|0;
    $137 = HEAP32[$1>>2]|0;
    $138 = (_sqlite3ExprCodeTemp($136,$137,$regFree1)|0);
    HEAP32[$r1>>2] = $138;
    $139 = HEAP32[$v>>2]|0;
    $140 = HEAP32[$r1>>2]|0;
    $141 = HEAP32[$2>>2]|0;
    $142 = HEAP32[$3>>2]|0;
    $143 = ($142|0)!=(0);
    $144 = $143&1;
    (_sqlite3VdbeAddOp3($139,45,$140,$141,$144)|0);
   }
  }
  }
 } while(0);
 $145 = HEAP32[$0>>2]|0;
 $146 = HEAP32[$regFree1>>2]|0;
 _sqlite3ReleaseTempReg($145,$146);
 $147 = HEAP32[$0>>2]|0;
 $148 = HEAP32[$regFree2>>2]|0;
 _sqlite3ReleaseTempReg($147,$148);
 STACKTOP = sp;return;
}
function _sqlite3ExprCodeTemp($pParse,$pExpr,$pReg) {
 $pParse = $pParse|0;
 $pExpr = $pExpr|0;
 $pReg = $pReg|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $8 = 0, $9 = 0, $i = 0, $p = 0, $pItem = 0, $r1 = 0, $r2 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $0 = sp + 32|0;
 $1 = sp + 28|0;
 $2 = sp + 24|0;
 $3 = sp + 20|0;
 $r2 = sp + 16|0;
 $p = sp + 12|0;
 $i = sp + 8|0;
 $pItem = sp + 4|0;
 $r1 = sp;
 HEAP32[$1>>2] = $pParse;
 HEAP32[$2>>2] = $pExpr;
 HEAP32[$3>>2] = $pReg;
 $4 = HEAP32[$2>>2]|0;
 $5 = (_sqlite3ExprSkipCollate($4)|0);
 HEAP32[$2>>2] = $5;
 $6 = HEAP32[$1>>2]|0;
 $7 = (($6) + 23|0);
 $8 = HEAP8[$7>>0]|0;
 $9 = $8&255;
 $10 = ($9|0)!=(0);
 if ($10) {
  $11 = HEAP32[$2>>2]|0;
  $12 = HEAP8[$11>>0]|0;
  $13 = $12&255;
  $14 = ($13|0)!=(159);
  if ($14) {
   $15 = HEAP32[$2>>2]|0;
   $16 = (_sqlite3ExprIsConstantNotJoin($15)|0);
   $17 = ($16|0)!=(0);
   if ($17) {
    $18 = HEAP32[$1>>2]|0;
    $19 = (($18) + 316|0);
    $20 = HEAP32[$19>>2]|0;
    HEAP32[$p>>2] = $20;
    $21 = HEAP32[$3>>2]|0;
    HEAP32[$21>>2] = 0;
    $22 = HEAP32[$p>>2]|0;
    $23 = ($22|0)!=(0|0);
    L5: do {
     if ($23) {
      $24 = HEAP32[$p>>2]|0;
      $25 = (($24) + 4|0);
      $26 = HEAP32[$25>>2]|0;
      HEAP32[$pItem>>2] = $26;
      $27 = HEAP32[$p>>2]|0;
      $28 = HEAP32[$27>>2]|0;
      HEAP32[$i>>2] = $28;
      while(1) {
       $29 = HEAP32[$i>>2]|0;
       $30 = ($29|0)>(0);
       if (!($30)) {
        break L5;
       }
       $31 = HEAP32[$pItem>>2]|0;
       $32 = (($31) + 13|0);
       $33 = HEAP8[$32>>0]|0;
       $34 = ($33&255) >>> 2;
       $35 = $34 & 1;
       $36 = $35&255;
       $37 = ($36|0)!=(0);
       if ($37) {
        $38 = HEAP32[$pItem>>2]|0;
        $39 = HEAP32[$38>>2]|0;
        $40 = HEAP32[$2>>2]|0;
        $41 = (_sqlite3ExprCompare($39,$40,-1)|0);
        $42 = ($41|0)==(0);
        if ($42) {
         break;
        }
       }
       $46 = HEAP32[$pItem>>2]|0;
       $47 = (($46) + 20|0);
       HEAP32[$pItem>>2] = $47;
       $48 = HEAP32[$i>>2]|0;
       $49 = (($48) + -1)|0;
       HEAP32[$i>>2] = $49;
      }
      $43 = HEAP32[$pItem>>2]|0;
      $44 = (($43) + 16|0);
      $45 = HEAP32[$44>>2]|0;
      HEAP32[$0>>2] = $45;
      $72 = HEAP32[$0>>2]|0;
      STACKTOP = sp;return ($72|0);
     }
    } while(0);
    $50 = HEAP32[$1>>2]|0;
    $51 = (($50) + 72|0);
    $52 = HEAP32[$51>>2]|0;
    $53 = (($52) + 1)|0;
    HEAP32[$51>>2] = $53;
    HEAP32[$r2>>2] = $53;
    $54 = HEAP32[$1>>2]|0;
    $55 = HEAP32[$2>>2]|0;
    $56 = HEAP32[$r2>>2]|0;
    _sqlite3ExprCodeAtInit($54,$55,$56,1);
   } else {
    label = 12;
   }
  } else {
   label = 12;
  }
 } else {
  label = 12;
 }
 do {
  if ((label|0) == 12) {
   $57 = HEAP32[$1>>2]|0;
   $58 = (_sqlite3GetTempReg($57)|0);
   HEAP32[$r1>>2] = $58;
   $59 = HEAP32[$1>>2]|0;
   $60 = HEAP32[$2>>2]|0;
   $61 = HEAP32[$r1>>2]|0;
   $62 = (_sqlite3ExprCodeTarget($59,$60,$61)|0);
   HEAP32[$r2>>2] = $62;
   $63 = HEAP32[$r2>>2]|0;
   $64 = HEAP32[$r1>>2]|0;
   $65 = ($63|0)==($64|0);
   if ($65) {
    $66 = HEAP32[$r1>>2]|0;
    $67 = HEAP32[$3>>2]|0;
    HEAP32[$67>>2] = $66;
    break;
   } else {
    $68 = HEAP32[$1>>2]|0;
    $69 = HEAP32[$r1>>2]|0;
    _sqlite3ReleaseTempReg($68,$69);
    $70 = HEAP32[$3>>2]|0;
    HEAP32[$70>>2] = 0;
    break;
   }
  }
 } while(0);
 $71 = HEAP32[$r2>>2]|0;
 HEAP32[$0>>2] = $71;
 $72 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($72|0);
}
function _codeCompare($pParse,$pLeft,$pRight,$opcode,$in1,$in2,$dest,$jumpIfNull) {
 $pParse = $pParse|0;
 $pLeft = $pLeft|0;
 $pRight = $pRight|0;
 $opcode = $opcode|0;
 $in1 = $in1|0;
 $in2 = $in2|0;
 $dest = $dest|0;
 $jumpIfNull = $jumpIfNull|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $addr = 0, $p4 = 0, $p5 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $0 = sp + 40|0;
 $1 = sp + 36|0;
 $2 = sp + 32|0;
 $3 = sp + 28|0;
 $4 = sp + 24|0;
 $5 = sp + 20|0;
 $6 = sp + 16|0;
 $7 = sp + 12|0;
 $p5 = sp + 8|0;
 $addr = sp + 4|0;
 $p4 = sp;
 HEAP32[$0>>2] = $pParse;
 HEAP32[$1>>2] = $pLeft;
 HEAP32[$2>>2] = $pRight;
 HEAP32[$3>>2] = $opcode;
 HEAP32[$4>>2] = $in1;
 HEAP32[$5>>2] = $in2;
 HEAP32[$6>>2] = $dest;
 HEAP32[$7>>2] = $jumpIfNull;
 $8 = HEAP32[$0>>2]|0;
 $9 = HEAP32[$1>>2]|0;
 $10 = HEAP32[$2>>2]|0;
 $11 = (_sqlite3BinaryCompareCollSeq($8,$9,$10)|0);
 HEAP32[$p4>>2] = $11;
 $12 = HEAP32[$1>>2]|0;
 $13 = HEAP32[$2>>2]|0;
 $14 = HEAP32[$7>>2]|0;
 $15 = (_binaryCompareP5($12,$13,$14)|0);
 $16 = $15&255;
 HEAP32[$p5>>2] = $16;
 $17 = HEAP32[$0>>2]|0;
 $18 = (($17) + 8|0);
 $19 = HEAP32[$18>>2]|0;
 $20 = HEAP32[$3>>2]|0;
 $21 = HEAP32[$5>>2]|0;
 $22 = HEAP32[$6>>2]|0;
 $23 = HEAP32[$4>>2]|0;
 $24 = HEAP32[$p4>>2]|0;
 $25 = (_sqlite3VdbeAddOp4($19,$20,$21,$22,$23,$24,-4)|0);
 HEAP32[$addr>>2] = $25;
 $26 = HEAP32[$0>>2]|0;
 $27 = (($26) + 8|0);
 $28 = HEAP32[$27>>2]|0;
 $29 = HEAP32[$p5>>2]|0;
 $30 = $29&255;
 _sqlite3VdbeChangeP5($28,$30);
 $31 = HEAP32[$addr>>2]|0;
 STACKTOP = sp;return ($31|0);
}
function _exprCodeBetween($pParse,$pExpr,$dest,$jumpIfTrue,$jumpIfNull) {
 $pParse = $pParse|0;
 $pExpr = $pExpr|0;
 $dest = $dest|0;
 $jumpIfTrue = $jumpIfTrue|0;
 $jumpIfNull = $jumpIfNull|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $compLeft = 0, $compRight = 0, $exprAnd = 0, $exprX = 0;
 var $regFree1 = 0, dest = 0, label = 0, sp = 0, src = 0, stop = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 224|0;
 $0 = sp + 216|0;
 $1 = sp + 212|0;
 $2 = sp + 208|0;
 $3 = sp + 204|0;
 $4 = sp + 200|0;
 $exprAnd = sp + 152|0;
 $compLeft = sp + 104|0;
 $compRight = sp + 56|0;
 $exprX = sp + 8|0;
 $regFree1 = sp;
 HEAP32[$0>>2] = $pParse;
 HEAP32[$1>>2] = $pExpr;
 HEAP32[$2>>2] = $dest;
 HEAP32[$3>>2] = $jumpIfTrue;
 HEAP32[$4>>2] = $jumpIfNull;
 HEAP32[$regFree1>>2] = 0;
 $5 = HEAP32[$1>>2]|0;
 $6 = (($5) + 12|0);
 $7 = HEAP32[$6>>2]|0;
 dest=$exprX+0|0; src=$7+0|0; stop=dest+48|0; do { HEAP32[dest>>2]=HEAP32[src>>2]|0; dest=dest+4|0; src=src+4|0; } while ((dest|0) < (stop|0));
 HEAP8[$exprAnd>>0] = 72;
 $8 = (($exprAnd) + 12|0);
 HEAP32[$8>>2] = $compLeft;
 $9 = (($exprAnd) + 16|0);
 HEAP32[$9>>2] = $compRight;
 HEAP8[$compLeft>>0] = 83;
 $10 = (($compLeft) + 12|0);
 HEAP32[$10>>2] = $exprX;
 $11 = HEAP32[$1>>2]|0;
 $12 = (($11) + 20|0);
 $13 = HEAP32[$12>>2]|0;
 $14 = (($13) + 4|0);
 $15 = HEAP32[$14>>2]|0;
 $16 = HEAP32[$15>>2]|0;
 $17 = (($compLeft) + 16|0);
 HEAP32[$17>>2] = $16;
 HEAP8[$compRight>>0] = 81;
 $18 = (($compRight) + 12|0);
 HEAP32[$18>>2] = $exprX;
 $19 = HEAP32[$1>>2]|0;
 $20 = (($19) + 20|0);
 $21 = HEAP32[$20>>2]|0;
 $22 = (($21) + 4|0);
 $23 = HEAP32[$22>>2]|0;
 $24 = (($23) + 20|0);
 $25 = HEAP32[$24>>2]|0;
 $26 = (($compRight) + 16|0);
 HEAP32[$26>>2] = $25;
 $27 = HEAP32[$0>>2]|0;
 $28 = (_sqlite3ExprCodeTemp($27,$exprX,$regFree1)|0);
 _exprToRegister($exprX,$28);
 $29 = HEAP32[$3>>2]|0;
 $30 = ($29|0)!=(0);
 $31 = HEAP32[$0>>2]|0;
 $32 = HEAP32[$2>>2]|0;
 $33 = HEAP32[$4>>2]|0;
 if ($30) {
  _sqlite3ExprIfTrue($31,$exprAnd,$32,$33);
  $34 = HEAP32[$0>>2]|0;
  $35 = HEAP32[$regFree1>>2]|0;
  _sqlite3ReleaseTempReg($34,$35);
  STACKTOP = sp;return;
 } else {
  _sqlite3ExprIfFalse($31,$exprAnd,$32,$33);
  $34 = HEAP32[$0>>2]|0;
  $35 = HEAP32[$regFree1>>2]|0;
  _sqlite3ReleaseTempReg($34,$35);
  STACKTOP = sp;return;
 }
}
function _sqlite3ExprCodeIN($pParse,$pExpr,$destIfFalse,$destIfNull) {
 $pParse = $pParse|0;
 $pExpr = $pExpr|0;
 $destIfFalse = $destIfFalse|0;
 $destIfNull = $destIfNull|0;
 var $$rRhsHasNull = 0, $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0;
 var $115 = 0, $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0;
 var $133 = 0, $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0;
 var $151 = 0, $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0;
 var $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0;
 var $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0;
 var $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0;
 var $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0;
 var $98 = 0, $99 = 0, $addr1 = 0, $affinity = 0, $eType = 0, $ii = 0, $j1 = 0, $labelOk = 0, $pColl = 0, $pList = 0, $r1 = 0, $r2 = 0, $rRhsHasNull = 0, $regCkNull = 0, $regToFree = 0, $v = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 80|0;
 $0 = sp + 64|0;
 $1 = sp + 60|0;
 $2 = sp + 56|0;
 $3 = sp + 52|0;
 $rRhsHasNull = sp + 48|0;
 $affinity = sp + 68|0;
 $eType = sp + 44|0;
 $r1 = sp + 40|0;
 $v = sp + 36|0;
 $pList = sp + 32|0;
 $pColl = sp + 28|0;
 $labelOk = sp + 24|0;
 $r2 = sp + 20|0;
 $regToFree = sp + 16|0;
 $regCkNull = sp + 12|0;
 $ii = sp + 8|0;
 $addr1 = sp + 4|0;
 $j1 = sp;
 HEAP32[$0>>2] = $pParse;
 HEAP32[$1>>2] = $pExpr;
 HEAP32[$2>>2] = $destIfFalse;
 HEAP32[$3>>2] = $destIfNull;
 HEAP32[$rRhsHasNull>>2] = 0;
 $4 = HEAP32[$0>>2]|0;
 $5 = (($4) + 8|0);
 $6 = HEAP32[$5>>2]|0;
 HEAP32[$v>>2] = $6;
 $7 = HEAP32[$0>>2]|0;
 $8 = HEAP32[$1>>2]|0;
 $9 = HEAP32[$2>>2]|0;
 $10 = HEAP32[$3>>2]|0;
 $11 = ($9|0)==($10|0);
 $$rRhsHasNull = $11 ? 0 : $rRhsHasNull;
 $12 = (_sqlite3FindInIndex($7,$8,3,$$rRhsHasNull)|0);
 HEAP32[$eType>>2] = $12;
 $13 = HEAP32[$1>>2]|0;
 $14 = (_comparisonAffinity($13)|0);
 HEAP8[$affinity>>0] = $14;
 $15 = HEAP32[$0>>2]|0;
 _sqlite3ExprCachePush($15);
 $16 = HEAP32[$0>>2]|0;
 $17 = (_sqlite3GetTempReg($16)|0);
 HEAP32[$r1>>2] = $17;
 $18 = HEAP32[$0>>2]|0;
 $19 = HEAP32[$1>>2]|0;
 $20 = (($19) + 12|0);
 $21 = HEAP32[$20>>2]|0;
 $22 = HEAP32[$r1>>2]|0;
 _sqlite3ExprCode($18,$21,$22);
 $23 = HEAP32[$eType>>2]|0;
 $24 = ($23|0)==(5);
 $25 = HEAP32[$1>>2]|0;
 if (!($24)) {
  $110 = (($25) + 12|0);
  $111 = HEAP32[$110>>2]|0;
  $112 = (_sqlite3ExprCanBeNull($111)|0);
  $113 = ($112|0)!=(0);
  do {
   if ($113) {
    $114 = HEAP32[$3>>2]|0;
    $115 = HEAP32[$2>>2]|0;
    $116 = ($114|0)==($115|0);
    $117 = HEAP32[$v>>2]|0;
    $118 = HEAP32[$r1>>2]|0;
    if ($116) {
     $119 = HEAP32[$3>>2]|0;
     (_sqlite3VdbeAddOp2($117,76,$118,$119)|0);
     break;
    } else {
     $120 = (_sqlite3VdbeAddOp1($117,77,$118)|0);
     HEAP32[$addr1>>2] = $120;
     $121 = HEAP32[$v>>2]|0;
     $122 = HEAP32[$1>>2]|0;
     $123 = (($122) + 28|0);
     $124 = HEAP32[$123>>2]|0;
     $125 = HEAP32[$2>>2]|0;
     (_sqlite3VdbeAddOp2($121,108,$124,$125)|0);
     $126 = HEAP32[$v>>2]|0;
     $127 = HEAP32[$3>>2]|0;
     (_sqlite3VdbeAddOp2($126,16,0,$127)|0);
     $128 = HEAP32[$v>>2]|0;
     $129 = HEAP32[$addr1>>2]|0;
     _sqlite3VdbeJumpHere($128,$129);
     break;
    }
   }
  } while(0);
  $130 = HEAP32[$eType>>2]|0;
  $131 = ($130|0)==(1);
  $132 = HEAP32[$v>>2]|0;
  $133 = HEAP32[$r1>>2]|0;
  if ($131) {
   $134 = HEAP32[$2>>2]|0;
   (_sqlite3VdbeAddOp2($132,38,$133,$134)|0);
   $135 = HEAP32[$v>>2]|0;
   $136 = HEAP32[$1>>2]|0;
   $137 = (($136) + 28|0);
   $138 = HEAP32[$137>>2]|0;
   $139 = HEAP32[$2>>2]|0;
   $140 = HEAP32[$r1>>2]|0;
   (_sqlite3VdbeAddOp3($135,70,$138,$139,$140)|0);
   $158 = HEAP32[$0>>2]|0;
   $159 = HEAP32[$r1>>2]|0;
   _sqlite3ReleaseTempReg($158,$159);
   $160 = HEAP32[$0>>2]|0;
   _sqlite3ExprCachePop($160);
   STACKTOP = sp;return;
  }
  (_sqlite3VdbeAddOp4($132,48,$133,1,0,$affinity,1)|0);
  $141 = HEAP32[$rRhsHasNull>>2]|0;
  $142 = ($141|0)==(0);
  $143 = HEAP32[$v>>2]|0;
  $144 = HEAP32[$1>>2]|0;
  $145 = (($144) + 28|0);
  $146 = HEAP32[$145>>2]|0;
  if ($142) {
   $147 = HEAP32[$2>>2]|0;
   $148 = HEAP32[$r1>>2]|0;
   (_sqlite3VdbeAddOp4Int($143,68,$146,$147,$148,1)|0);
   $158 = HEAP32[$0>>2]|0;
   $159 = HEAP32[$r1>>2]|0;
   _sqlite3ReleaseTempReg($158,$159);
   $160 = HEAP32[$0>>2]|0;
   _sqlite3ExprCachePop($160);
   STACKTOP = sp;return;
  } else {
   $149 = HEAP32[$r1>>2]|0;
   $150 = (_sqlite3VdbeAddOp4Int($143,69,$146,0,$149,1)|0);
   HEAP32[$j1>>2] = $150;
   $151 = HEAP32[$v>>2]|0;
   $152 = HEAP32[$rRhsHasNull>>2]|0;
   $153 = HEAP32[$3>>2]|0;
   (_sqlite3VdbeAddOp2($151,76,$152,$153)|0);
   $154 = HEAP32[$v>>2]|0;
   $155 = HEAP32[$2>>2]|0;
   (_sqlite3VdbeAddOp2($154,16,0,$155)|0);
   $156 = HEAP32[$v>>2]|0;
   $157 = HEAP32[$j1>>2]|0;
   _sqlite3VdbeJumpHere($156,$157);
   $158 = HEAP32[$0>>2]|0;
   $159 = HEAP32[$r1>>2]|0;
   _sqlite3ReleaseTempReg($158,$159);
   $160 = HEAP32[$0>>2]|0;
   _sqlite3ExprCachePop($160);
   STACKTOP = sp;return;
  }
 }
 $26 = (($25) + 20|0);
 $27 = HEAP32[$26>>2]|0;
 HEAP32[$pList>>2] = $27;
 $28 = HEAP32[$0>>2]|0;
 $29 = HEAP32[$1>>2]|0;
 $30 = (($29) + 12|0);
 $31 = HEAP32[$30>>2]|0;
 $32 = (_sqlite3ExprCollSeq($28,$31)|0);
 HEAP32[$pColl>>2] = $32;
 $33 = HEAP32[$v>>2]|0;
 $34 = (_sqlite3VdbeMakeLabel($33)|0);
 HEAP32[$labelOk>>2] = $34;
 HEAP32[$regCkNull>>2] = 0;
 $35 = HEAP32[$3>>2]|0;
 $36 = HEAP32[$2>>2]|0;
 $37 = ($35|0)!=($36|0);
 if ($37) {
  $38 = HEAP32[$0>>2]|0;
  $39 = (_sqlite3GetTempReg($38)|0);
  HEAP32[$regCkNull>>2] = $39;
  $40 = HEAP32[$v>>2]|0;
  $41 = HEAP32[$r1>>2]|0;
  $42 = HEAP32[$r1>>2]|0;
  $43 = HEAP32[$regCkNull>>2]|0;
  (_sqlite3VdbeAddOp3($40,85,$41,$42,$43)|0);
 }
 HEAP32[$ii>>2] = 0;
 while(1) {
  $44 = HEAP32[$ii>>2]|0;
  $45 = HEAP32[$pList>>2]|0;
  $46 = HEAP32[$45>>2]|0;
  $47 = ($44|0)<($46|0);
  if (!($47)) {
   break;
  }
  $48 = HEAP32[$0>>2]|0;
  $49 = HEAP32[$ii>>2]|0;
  $50 = HEAP32[$pList>>2]|0;
  $51 = (($50) + 4|0);
  $52 = HEAP32[$51>>2]|0;
  $53 = (($52) + (($49*20)|0)|0);
  $54 = HEAP32[$53>>2]|0;
  $55 = (_sqlite3ExprCodeTemp($48,$54,$regToFree)|0);
  HEAP32[$r2>>2] = $55;
  $56 = HEAP32[$regCkNull>>2]|0;
  $57 = ($56|0)!=(0);
  if ($57) {
   $58 = HEAP32[$ii>>2]|0;
   $59 = HEAP32[$pList>>2]|0;
   $60 = (($59) + 4|0);
   $61 = HEAP32[$60>>2]|0;
   $62 = (($61) + (($58*20)|0)|0);
   $63 = HEAP32[$62>>2]|0;
   $64 = (_sqlite3ExprCanBeNull($63)|0);
   $65 = ($64|0)!=(0);
   if ($65) {
    $66 = HEAP32[$v>>2]|0;
    $67 = HEAP32[$regCkNull>>2]|0;
    $68 = HEAP32[$r2>>2]|0;
    $69 = HEAP32[$regCkNull>>2]|0;
    (_sqlite3VdbeAddOp3($66,85,$67,$68,$69)|0);
   }
  }
  $70 = HEAP32[$ii>>2]|0;
  $71 = HEAP32[$pList>>2]|0;
  $72 = HEAP32[$71>>2]|0;
  $73 = (($72) - 1)|0;
  $74 = ($70|0)<($73|0);
  if ($74) {
   label = 11;
  } else {
   $75 = HEAP32[$3>>2]|0;
   $76 = HEAP32[$2>>2]|0;
   $77 = ($75|0)!=($76|0);
   if ($77) {
    label = 11;
   } else {
    $85 = HEAP32[$v>>2]|0;
    $86 = HEAP32[$r1>>2]|0;
    $87 = HEAP32[$2>>2]|0;
    $88 = HEAP32[$r2>>2]|0;
    $89 = HEAP32[$pColl>>2]|0;
    (_sqlite3VdbeAddOp4($85,78,$86,$87,$88,$89,-4)|0);
    $90 = HEAP32[$v>>2]|0;
    $91 = HEAP8[$affinity>>0]|0;
    $92 = $91 << 24 >> 24;
    $93 = $92 | 16;
    $94 = $93&255;
    _sqlite3VdbeChangeP5($90,$94);
   }
  }
  if ((label|0) == 11) {
   label = 0;
   $78 = HEAP32[$v>>2]|0;
   $79 = HEAP32[$r1>>2]|0;
   $80 = HEAP32[$labelOk>>2]|0;
   $81 = HEAP32[$r2>>2]|0;
   $82 = HEAP32[$pColl>>2]|0;
   (_sqlite3VdbeAddOp4($78,79,$79,$80,$81,$82,-4)|0);
   $83 = HEAP32[$v>>2]|0;
   $84 = HEAP8[$affinity>>0]|0;
   _sqlite3VdbeChangeP5($83,$84);
  }
  $95 = HEAP32[$0>>2]|0;
  $96 = HEAP32[$regToFree>>2]|0;
  _sqlite3ReleaseTempReg($95,$96);
  $97 = HEAP32[$ii>>2]|0;
  $98 = (($97) + 1)|0;
  HEAP32[$ii>>2] = $98;
 }
 $99 = HEAP32[$regCkNull>>2]|0;
 $100 = ($99|0)!=(0);
 if ($100) {
  $101 = HEAP32[$v>>2]|0;
  $102 = HEAP32[$regCkNull>>2]|0;
  $103 = HEAP32[$3>>2]|0;
  (_sqlite3VdbeAddOp2($101,76,$102,$103)|0);
  $104 = HEAP32[$v>>2]|0;
  $105 = HEAP32[$2>>2]|0;
  (_sqlite3VdbeAddOp2($104,16,0,$105)|0);
 }
 $106 = HEAP32[$v>>2]|0;
 $107 = HEAP32[$labelOk>>2]|0;
 _sqlite3VdbeResolveLabel($106,$107);
 $108 = HEAP32[$0>>2]|0;
 $109 = HEAP32[$regCkNull>>2]|0;
 _sqlite3ReleaseTempReg($108,$109);
 $158 = HEAP32[$0>>2]|0;
 $159 = HEAP32[$r1>>2]|0;
 _sqlite3ReleaseTempReg($158,$159);
 $160 = HEAP32[$0>>2]|0;
 _sqlite3ExprCachePop($160);
 STACKTOP = sp;return;
}
function _exprAlwaysTrue($p) {
 $p = $p|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $v = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $1 = sp + 4|0;
 $v = sp;
 HEAP32[$1>>2] = $p;
 HEAP32[$v>>2] = 0;
 $2 = HEAP32[$1>>2]|0;
 $3 = (($2) + 4|0);
 $4 = HEAP32[$3>>2]|0;
 $5 = $4 & 1;
 $6 = ($5|0)!=(0);
 do {
  if ($6) {
   HEAP32[$0>>2] = 0;
  } else {
   $7 = HEAP32[$1>>2]|0;
   $8 = (_sqlite3ExprIsInteger($7,$v)|0);
   $9 = ($8|0)!=(0);
   if ($9) {
    $10 = HEAP32[$v>>2]|0;
    $11 = ($10|0)!=(0);
    $12 = $11&1;
    HEAP32[$0>>2] = $12;
    break;
   } else {
    HEAP32[$0>>2] = 0;
    break;
   }
  }
 } while(0);
 $13 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($13|0);
}
function _sqlite3FindInIndex($pParse,$pX,$inFlags,$prRhsHasNull) {
 $pParse = $pParse|0;
 $pX = $pX|0;
 $inFlags = $inFlags|0;
 $prRhsHasNull = $prRhsHasNull|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0;
 var $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0, $188 = 0;
 var $189 = 0, $19 = 0, $190 = 0, $191 = 0, $192 = 0, $193 = 0, $194 = 0, $195 = 0, $196 = 0, $197 = 0, $198 = 0, $199 = 0, $2 = 0, $20 = 0, $200 = 0, $201 = 0, $202 = 0, $203 = 0, $204 = 0, $205 = 0;
 var $206 = 0, $207 = 0, $208 = 0, $209 = 0, $21 = 0, $210 = 0, $211 = 0, $212 = 0, $213 = 0, $214 = 0, $215 = 0, $216 = 0, $217 = 0, $218 = 0, $219 = 0, $22 = 0, $220 = 0, $221 = 0, $222 = 0, $223 = 0;
 var $224 = 0, $225 = 0, $226 = 0, $227 = 0, $228 = 0, $229 = 0, $23 = 0, $230 = 0, $231 = 0, $232 = 0, $233 = 0, $234 = 0, $235 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0;
 var $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0;
 var $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0;
 var $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0;
 var $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $affinity_ok = 0, $db = 0, $eType = 0, $iAddr = 0;
 var $iAddr1 = 0, $iCol = 0, $iDb = 0, $iTab = 0, $mustBeUnique = 0, $or$cond = 0, $or$cond3 = 0, $p = 0, $pExpr = 0, $pIdx = 0, $pReq = 0, $pTab = 0, $rMayHaveNull = 0, $savedNQueryLoop = 0, $v = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 80|0;
 $0 = sp + 72|0;
 $1 = sp + 68|0;
 $2 = sp + 64|0;
 $3 = sp + 60|0;
 $p = sp + 56|0;
 $eType = sp + 52|0;
 $iTab = sp + 48|0;
 $mustBeUnique = sp + 44|0;
 $v = sp + 40|0;
 $db = sp + 36|0;
 $pTab = sp + 32|0;
 $pExpr = sp + 28|0;
 $iCol = sp + 78|0;
 $iDb = sp + 76|0;
 $iAddr = sp + 24|0;
 $pIdx = sp + 20|0;
 $pReq = sp + 16|0;
 $affinity_ok = sp + 12|0;
 $iAddr1 = sp + 8|0;
 $savedNQueryLoop = sp + 4|0;
 $rMayHaveNull = sp;
 HEAP32[$0>>2] = $pParse;
 HEAP32[$1>>2] = $pX;
 HEAP32[$2>>2] = $inFlags;
 HEAP32[$3>>2] = $prRhsHasNull;
 HEAP32[$eType>>2] = 0;
 $4 = HEAP32[$0>>2]|0;
 $5 = (($4) + 68|0);
 $6 = HEAP32[$5>>2]|0;
 $7 = (($6) + 1)|0;
 HEAP32[$5>>2] = $7;
 HEAP32[$iTab>>2] = $6;
 $8 = HEAP32[$0>>2]|0;
 $9 = (_sqlite3GetVdbe($8)|0);
 HEAP32[$v>>2] = $9;
 $10 = HEAP32[$2>>2]|0;
 $11 = $10 & 4;
 $12 = ($11|0)!=(0);
 $13 = $12&1;
 HEAP32[$mustBeUnique>>2] = $13;
 $14 = HEAP32[$1>>2]|0;
 $15 = (($14) + 4|0);
 $16 = HEAP32[$15>>2]|0;
 $17 = $16 & 2048;
 $18 = ($17|0)!=(0);
 if ($18) {
  $19 = HEAP32[$1>>2]|0;
  $20 = (($19) + 20|0);
  $21 = HEAP32[$20>>2]|0;
  $22 = $21;
 } else {
  $22 = 0;
 }
 HEAP32[$p>>2] = $22;
 $23 = HEAP32[$0>>2]|0;
 $24 = (($23) + 64|0);
 $25 = HEAP32[$24>>2]|0;
 $26 = ($25|0)==(0);
 L4: do {
  if ($26) {
   $27 = HEAP32[$p>>2]|0;
   $28 = (_isCandidateForInOpt($27)|0);
   $29 = ($28|0)!=(0);
   if ($29) {
    $30 = HEAP32[$0>>2]|0;
    $31 = HEAP32[$30>>2]|0;
    HEAP32[$db>>2] = $31;
    $32 = HEAP32[$p>>2]|0;
    $33 = (($32) + 32|0);
    $34 = HEAP32[$33>>2]|0;
    $35 = (($34) + 8|0);
    $36 = (($35) + 16|0);
    $37 = HEAP32[$36>>2]|0;
    HEAP32[$pTab>>2] = $37;
    $38 = HEAP32[$p>>2]|0;
    $39 = HEAP32[$38>>2]|0;
    $40 = (($39) + 4|0);
    $41 = HEAP32[$40>>2]|0;
    $42 = HEAP32[$41>>2]|0;
    HEAP32[$pExpr>>2] = $42;
    $43 = HEAP32[$pExpr>>2]|0;
    $44 = (($43) + 32|0);
    $45 = HEAP16[$44>>1]|0;
    HEAP16[$iCol>>1] = $45;
    $46 = HEAP32[$db>>2]|0;
    $47 = HEAP32[$pTab>>2]|0;
    $48 = (($47) + 68|0);
    $49 = HEAP32[$48>>2]|0;
    $50 = (_sqlite3SchemaToIndex($46,$49)|0);
    $51 = $50&65535;
    HEAP16[$iDb>>1] = $51;
    $52 = HEAP32[$0>>2]|0;
    $53 = HEAP16[$iDb>>1]|0;
    $54 = $53 << 16 >> 16;
    _sqlite3CodeVerifySchema($52,$54);
    $55 = HEAP32[$0>>2]|0;
    $56 = HEAP16[$iDb>>1]|0;
    $57 = $56 << 16 >> 16;
    $58 = HEAP32[$pTab>>2]|0;
    $59 = (($58) + 32|0);
    $60 = HEAP32[$59>>2]|0;
    $61 = HEAP32[$pTab>>2]|0;
    $62 = HEAP32[$61>>2]|0;
    _sqlite3TableLock($55,$57,$60,0,$62);
    $63 = HEAP16[$iCol>>1]|0;
    $64 = $63 << 16 >> 16;
    $65 = ($64|0)<(0);
    $66 = HEAP32[$0>>2]|0;
    if ($65) {
     $67 = (_sqlite3CodeOnce($66)|0);
     HEAP32[$iAddr>>2] = $67;
     $68 = HEAP32[$0>>2]|0;
     $69 = HEAP32[$iTab>>2]|0;
     $70 = HEAP16[$iDb>>1]|0;
     $71 = $70 << 16 >> 16;
     $72 = HEAP32[$pTab>>2]|0;
     _sqlite3OpenTable($68,$69,$71,$72,54);
     HEAP32[$eType>>2] = 1;
     $73 = HEAP32[$v>>2]|0;
     $74 = HEAP32[$iAddr>>2]|0;
     _sqlite3VdbeJumpHere($73,$74);
     break;
    }
    $75 = HEAP32[$1>>2]|0;
    $76 = (($75) + 12|0);
    $77 = HEAP32[$76>>2]|0;
    $78 = HEAP32[$pExpr>>2]|0;
    $79 = (_sqlite3BinaryCompareCollSeq($66,$77,$78)|0);
    HEAP32[$pReq>>2] = $79;
    $80 = HEAP32[$1>>2]|0;
    $81 = HEAP16[$iCol>>1]|0;
    $82 = $81 << 16 >> 16;
    $83 = HEAP32[$pTab>>2]|0;
    $84 = (($83) + 4|0);
    $85 = HEAP32[$84>>2]|0;
    $86 = (($85) + (($82*24)|0)|0);
    $87 = (($86) + 21|0);
    $88 = HEAP8[$87>>0]|0;
    $89 = (_sqlite3IndexAffinityOk($80,$88)|0);
    HEAP32[$affinity_ok>>2] = $89;
    $90 = HEAP32[$pTab>>2]|0;
    $91 = (($90) + 8|0);
    $92 = HEAP32[$91>>2]|0;
    HEAP32[$pIdx>>2] = $92;
    while(1) {
     $93 = HEAP32[$pIdx>>2]|0;
     $94 = ($93|0)!=(0|0);
     $95 = HEAP32[$eType>>2]|0;
     $96 = ($95|0)==(0);
     $or$cond = $94 & $96;
     $97 = HEAP32[$affinity_ok>>2]|0;
     $98 = ($97|0)!=(0);
     $or$cond3 = $or$cond & $98;
     if (!($or$cond3)) {
      break L4;
     }
     $99 = HEAP32[$pIdx>>2]|0;
     $100 = (($99) + 4|0);
     $101 = HEAP32[$100>>2]|0;
     $102 = HEAP16[$101>>1]|0;
     $103 = $102 << 16 >> 16;
     $104 = HEAP16[$iCol>>1]|0;
     $105 = $104 << 16 >> 16;
     $106 = ($103|0)==($105|0);
     do {
      if ($106) {
       $107 = HEAP32[$db>>2]|0;
       $108 = HEAP32[$db>>2]|0;
       $109 = (($108) + 16|0);
       $110 = HEAP32[$109>>2]|0;
       $111 = (($110) + 12|0);
       $112 = HEAP32[$111>>2]|0;
       $113 = (($112) + 77|0);
       $114 = HEAP8[$113>>0]|0;
       $115 = HEAP32[$pIdx>>2]|0;
       $116 = (($115) + 32|0);
       $117 = HEAP32[$116>>2]|0;
       $118 = HEAP32[$117>>2]|0;
       $119 = (_sqlite3FindCollSeq($107,$114,$118,0)|0);
       $120 = HEAP32[$pReq>>2]|0;
       $121 = ($119|0)==($120|0);
       if ($121) {
        $122 = HEAP32[$mustBeUnique>>2]|0;
        $123 = ($122|0)!=(0);
        if ($123) {
         $124 = HEAP32[$pIdx>>2]|0;
         $125 = (($124) + 50|0);
         $126 = HEAP16[$125>>1]|0;
         $127 = $126&65535;
         $128 = ($127|0)==(1);
         if (!($128)) {
          break;
         }
         $129 = HEAP32[$pIdx>>2]|0;
         $130 = (($129) + 54|0);
         $131 = HEAP8[$130>>0]|0;
         $132 = $131&255;
         $133 = ($132|0)!=(0);
         if (!($133)) {
          break;
         }
        }
        $134 = HEAP32[$0>>2]|0;
        $135 = (_sqlite3CodeOnce($134)|0);
        HEAP32[$iAddr1>>2] = $135;
        $136 = HEAP32[$v>>2]|0;
        $137 = HEAP32[$iTab>>2]|0;
        $138 = HEAP32[$pIdx>>2]|0;
        $139 = (($138) + 44|0);
        $140 = HEAP32[$139>>2]|0;
        $141 = HEAP16[$iDb>>1]|0;
        $142 = $141 << 16 >> 16;
        (_sqlite3VdbeAddOp3($136,54,$137,$140,$142)|0);
        $143 = HEAP32[$0>>2]|0;
        $144 = HEAP32[$pIdx>>2]|0;
        _sqlite3VdbeSetP4KeyInfo($143,$144);
        $145 = HEAP32[$pIdx>>2]|0;
        $146 = (($145) + 28|0);
        $147 = HEAP32[$146>>2]|0;
        $148 = HEAP8[$147>>0]|0;
        $149 = $148&255;
        $150 = (3 + ($149))|0;
        HEAP32[$eType>>2] = $150;
        $151 = HEAP32[$3>>2]|0;
        $152 = ($151|0)!=(0|0);
        if ($152) {
         $153 = HEAP16[$iCol>>1]|0;
         $154 = $153 << 16 >> 16;
         $155 = HEAP32[$pTab>>2]|0;
         $156 = (($155) + 4|0);
         $157 = HEAP32[$156>>2]|0;
         $158 = (($157) + (($154*24)|0)|0);
         $159 = (($158) + 20|0);
         $160 = HEAP8[$159>>0]|0;
         $161 = ($160<<24>>24)!=(0);
         if (!($161)) {
          $162 = HEAP32[$0>>2]|0;
          $163 = (($162) + 72|0);
          $164 = HEAP32[$163>>2]|0;
          $165 = (($164) + 1)|0;
          HEAP32[$163>>2] = $165;
          $166 = HEAP32[$3>>2]|0;
          HEAP32[$166>>2] = $165;
          $167 = HEAP32[$v>>2]|0;
          $168 = HEAP32[$iTab>>2]|0;
          $169 = HEAP32[$3>>2]|0;
          $170 = HEAP32[$169>>2]|0;
          _sqlite3SetHasNullFlag($167,$168,$170);
         }
        }
        $171 = HEAP32[$v>>2]|0;
        $172 = HEAP32[$iAddr1>>2]|0;
        _sqlite3VdbeJumpHere($171,$172);
       }
      }
     } while(0);
     $173 = HEAP32[$pIdx>>2]|0;
     $174 = (($173) + 20|0);
     $175 = HEAP32[$174>>2]|0;
     HEAP32[$pIdx>>2] = $175;
    }
   }
  }
 } while(0);
 $176 = HEAP32[$eType>>2]|0;
 $177 = ($176|0)==(0);
 do {
  if ($177) {
   $178 = HEAP32[$2>>2]|0;
   $179 = $178 & 1;
   $180 = ($179|0)!=(0);
   if ($180) {
    $181 = HEAP32[$1>>2]|0;
    $182 = (($181) + 4|0);
    $183 = HEAP32[$182>>2]|0;
    $184 = $183 & 2048;
    $185 = ($184|0)!=(0);
    if (!($185)) {
     $186 = HEAP32[$1>>2]|0;
     $187 = (_sqlite3InRhsIsConstant($186)|0);
     $188 = ($187|0)!=(0);
     if ($188) {
      $189 = HEAP32[$1>>2]|0;
      $190 = (($189) + 20|0);
      $191 = HEAP32[$190>>2]|0;
      $192 = HEAP32[$191>>2]|0;
      $193 = ($192|0)<=(2);
      if (!($193)) {
       break;
      }
     }
     HEAP32[$eType>>2] = 5;
    }
   }
  }
 } while(0);
 $194 = HEAP32[$eType>>2]|0;
 $195 = ($194|0)==(0);
 if (!($195)) {
  $232 = HEAP32[$iTab>>2]|0;
  $233 = HEAP32[$1>>2]|0;
  $234 = (($233) + 28|0);
  HEAP32[$234>>2] = $232;
  $235 = HEAP32[$eType>>2]|0;
  STACKTOP = sp;return ($235|0);
 }
 $196 = HEAP32[$0>>2]|0;
 $197 = (($196) + 424|0);
 $198 = HEAP32[$197>>2]|0;
 HEAP32[$savedNQueryLoop>>2] = $198;
 HEAP32[$rMayHaveNull>>2] = 0;
 HEAP32[$eType>>2] = 2;
 $199 = HEAP32[$2>>2]|0;
 $200 = $199 & 4;
 $201 = ($200|0)!=(0);
 if ($201) {
  $202 = HEAP32[$0>>2]|0;
  $203 = (($202) + 424|0);
  HEAP32[$203>>2] = 0;
  $204 = HEAP32[$1>>2]|0;
  $205 = (($204) + 12|0);
  $206 = HEAP32[$205>>2]|0;
  $207 = (($206) + 32|0);
  $208 = HEAP16[$207>>1]|0;
  $209 = $208 << 16 >> 16;
  $210 = ($209|0)<(0);
  if ($210) {
   $211 = HEAP32[$1>>2]|0;
   $212 = (($211) + 4|0);
   $213 = HEAP32[$212>>2]|0;
   $214 = $213 & 2048;
   $215 = ($214|0)!=(0);
   if (!($215)) {
    HEAP32[$eType>>2] = 1;
   }
  }
 } else {
  $216 = HEAP32[$3>>2]|0;
  $217 = ($216|0)!=(0|0);
  if ($217) {
   $218 = HEAP32[$0>>2]|0;
   $219 = (($218) + 72|0);
   $220 = HEAP32[$219>>2]|0;
   $221 = (($220) + 1)|0;
   HEAP32[$219>>2] = $221;
   HEAP32[$rMayHaveNull>>2] = $221;
   $222 = HEAP32[$3>>2]|0;
   HEAP32[$222>>2] = $221;
  }
 }
 $223 = HEAP32[$0>>2]|0;
 $224 = HEAP32[$1>>2]|0;
 $225 = HEAP32[$rMayHaveNull>>2]|0;
 $226 = HEAP32[$eType>>2]|0;
 $227 = ($226|0)==(1);
 $228 = $227&1;
 (_sqlite3CodeSubselect($223,$224,$225,$228)|0);
 $229 = HEAP32[$savedNQueryLoop>>2]|0;
 $230 = HEAP32[$0>>2]|0;
 $231 = (($230) + 424|0);
 HEAP32[$231>>2] = $229;
 $235 = HEAP32[$eType>>2]|0;
 STACKTOP = sp;return ($235|0);
}
function _comparisonAffinity($pExpr) {
 $pExpr = $pExpr|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $aff = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 $aff = sp + 4|0;
 HEAP32[$0>>2] = $pExpr;
 $1 = HEAP32[$0>>2]|0;
 $2 = (($1) + 12|0);
 $3 = HEAP32[$2>>2]|0;
 $4 = (_sqlite3ExprAffinity($3)|0);
 HEAP8[$aff>>0] = $4;
 $5 = HEAP32[$0>>2]|0;
 $6 = (($5) + 16|0);
 $7 = HEAP32[$6>>2]|0;
 $8 = ($7|0)!=(0|0);
 $9 = HEAP32[$0>>2]|0;
 if ($8) {
  $10 = (($9) + 16|0);
  $11 = HEAP32[$10>>2]|0;
  $12 = HEAP8[$aff>>0]|0;
  $13 = (_sqlite3CompareAffinity($11,$12)|0);
  HEAP8[$aff>>0] = $13;
  $29 = HEAP8[$aff>>0]|0;
  STACKTOP = sp;return ($29|0);
 }
 $14 = (($9) + 4|0);
 $15 = HEAP32[$14>>2]|0;
 $16 = $15 & 2048;
 $17 = ($16|0)!=(0);
 if ($17) {
  $18 = HEAP32[$0>>2]|0;
  $19 = (($18) + 20|0);
  $20 = HEAP32[$19>>2]|0;
  $21 = HEAP32[$20>>2]|0;
  $22 = (($21) + 4|0);
  $23 = HEAP32[$22>>2]|0;
  $24 = HEAP32[$23>>2]|0;
  $25 = HEAP8[$aff>>0]|0;
  $26 = (_sqlite3CompareAffinity($24,$25)|0);
  HEAP8[$aff>>0] = $26;
  $29 = HEAP8[$aff>>0]|0;
  STACKTOP = sp;return ($29|0);
 }
 $27 = HEAP8[$aff>>0]|0;
 $28 = ($27<<24>>24)!=(0);
 if ($28) {
  $29 = HEAP8[$aff>>0]|0;
  STACKTOP = sp;return ($29|0);
 }
 HEAP8[$aff>>0] = 65;
 $29 = HEAP8[$aff>>0]|0;
 STACKTOP = sp;return ($29|0);
}
function _sqlite3ExprCode($pParse,$pExpr,$target) {
 $pParse = $pParse|0;
 $pExpr = $pExpr|0;
 $target = $target|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $inReg = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $1 = sp + 8|0;
 $2 = sp + 4|0;
 $inReg = sp;
 HEAP32[$0>>2] = $pParse;
 HEAP32[$1>>2] = $pExpr;
 HEAP32[$2>>2] = $target;
 $3 = HEAP32[$1>>2]|0;
 $4 = ($3|0)!=(0|0);
 if ($4) {
  $5 = HEAP32[$1>>2]|0;
  $6 = HEAP8[$5>>0]|0;
  $7 = $6&255;
  $8 = ($7|0)==(159);
  if ($8) {
   $9 = HEAP32[$0>>2]|0;
   $10 = (($9) + 8|0);
   $11 = HEAP32[$10>>2]|0;
   $12 = HEAP32[$1>>2]|0;
   $13 = (($12) + 28|0);
   $14 = HEAP32[$13>>2]|0;
   $15 = HEAP32[$2>>2]|0;
   (_sqlite3VdbeAddOp2($11,33,$14,$15)|0);
   STACKTOP = sp;return;
  }
 }
 $16 = HEAP32[$0>>2]|0;
 $17 = HEAP32[$1>>2]|0;
 $18 = HEAP32[$2>>2]|0;
 $19 = (_sqlite3ExprCodeTarget($16,$17,$18)|0);
 HEAP32[$inReg>>2] = $19;
 $20 = HEAP32[$inReg>>2]|0;
 $21 = HEAP32[$2>>2]|0;
 $22 = ($20|0)!=($21|0);
 if (!($22)) {
  STACKTOP = sp;return;
 }
 $23 = HEAP32[$0>>2]|0;
 $24 = (($23) + 8|0);
 $25 = HEAP32[$24>>2]|0;
 $26 = ($25|0)!=(0|0);
 if (!($26)) {
  STACKTOP = sp;return;
 }
 $27 = HEAP32[$0>>2]|0;
 $28 = (($27) + 8|0);
 $29 = HEAP32[$28>>2]|0;
 $30 = HEAP32[$inReg>>2]|0;
 $31 = HEAP32[$2>>2]|0;
 (_sqlite3VdbeAddOp2($29,34,$30,$31)|0);
 STACKTOP = sp;return;
}
function _sqlite3ExprCanBeNull($p) {
 $p = $p|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $op = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $1 = sp;
 $op = sp + 8|0;
 HEAP32[$1>>2] = $p;
 while(1) {
  $2 = HEAP32[$1>>2]|0;
  $3 = HEAP8[$2>>0]|0;
  $4 = $3&255;
  $5 = ($4|0)==(158);
  if ($5) {
   $49 = 1;
  } else {
   $6 = HEAP32[$1>>2]|0;
   $7 = HEAP8[$6>>0]|0;
   $8 = $7&255;
   $9 = ($8|0)==(157);
   $49 = $9;
  }
  $10 = HEAP32[$1>>2]|0;
  if (!($49)) {
   break;
  }
  $11 = (($10) + 12|0);
  $12 = HEAP32[$11>>2]|0;
  HEAP32[$1>>2] = $12;
 }
 $13 = HEAP8[$10>>0]|0;
 HEAP8[$op>>0] = $13;
 $14 = HEAP8[$op>>0]|0;
 $15 = $14&255;
 $16 = ($15|0)==(159);
 if ($16) {
  $17 = HEAP32[$1>>2]|0;
  $18 = (($17) + 38|0);
  $19 = HEAP8[$18>>0]|0;
  HEAP8[$op>>0] = $19;
 }
 $20 = HEAP8[$op>>0]|0;
 $21 = $20&255;
 switch ($21|0) {
 case 134: case 133: case 97: case 132:  {
  HEAP32[$0>>2] = 0;
  $48 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($48|0);
  break;
 }
 case 154:  {
  $22 = HEAP32[$1>>2]|0;
  $23 = (($22) + 4|0);
  $24 = HEAP32[$23>>2]|0;
  $25 = $24 & 1048576;
  $26 = ($25|0)!=(0);
  if ($26) {
   $47 = 1;
  } else {
   $27 = HEAP32[$1>>2]|0;
   $28 = (($27) + 32|0);
   $29 = HEAP16[$28>>1]|0;
   $30 = $29 << 16 >> 16;
   $31 = ($30|0)>=(0);
   if ($31) {
    $32 = HEAP32[$1>>2]|0;
    $33 = (($32) + 32|0);
    $34 = HEAP16[$33>>1]|0;
    $35 = $34 << 16 >> 16;
    $36 = HEAP32[$1>>2]|0;
    $37 = (($36) + 44|0);
    $38 = HEAP32[$37>>2]|0;
    $39 = (($38) + 4|0);
    $40 = HEAP32[$39>>2]|0;
    $41 = (($40) + (($35*24)|0)|0);
    $42 = (($41) + 20|0);
    $43 = HEAP8[$42>>0]|0;
    $44 = $43&255;
    $45 = ($44|0)==(0);
    $47 = $45;
   } else {
    $47 = 0;
   }
  }
  $46 = $47&1;
  HEAP32[$0>>2] = $46;
  $48 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($48|0);
  break;
 }
 default: {
  HEAP32[$0>>2] = 1;
  $48 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($48|0);
 }
 }
 return 0|0;
}
function _sqlite3ExprCodeTarget($pParse,$pExpr,$target) {
 $pParse = $pParse|0;
 $pExpr = $pExpr|0;
 $target = $target|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0;
 var $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0, $188 = 0;
 var $189 = 0, $19 = 0, $190 = 0, $191 = 0, $192 = 0, $193 = 0, $194 = 0, $195 = 0, $196 = 0, $197 = 0, $198 = 0, $199 = 0, $2 = 0, $20 = 0, $200 = 0, $201 = 0, $202 = 0, $203 = 0, $204 = 0, $205 = 0;
 var $206 = 0, $207 = 0, $208 = 0, $209 = 0, $21 = 0, $210 = 0, $211 = 0, $212 = 0, $213 = 0, $214 = 0, $215 = 0, $216 = 0, $217 = 0, $218 = 0, $219 = 0, $22 = 0, $220 = 0, $221 = 0, $222 = 0, $223 = 0;
 var $224 = 0, $225 = 0, $226 = 0, $227 = 0, $228 = 0, $229 = 0, $23 = 0, $230 = 0, $231 = 0, $232 = 0, $233 = 0, $234 = 0, $235 = 0, $236 = 0, $237 = 0, $238 = 0, $239 = 0, $24 = 0, $240 = 0, $241 = 0;
 var $242 = 0, $243 = 0, $244 = 0, $245 = 0, $246 = 0, $247 = 0, $248 = 0, $249 = 0, $25 = 0, $250 = 0, $251 = 0, $252 = 0, $253 = 0, $254 = 0, $255 = 0, $256 = 0, $257 = 0, $258 = 0, $259 = 0, $26 = 0;
 var $260 = 0, $261 = 0, $262 = 0, $263 = 0, $264 = 0, $265 = 0, $266 = 0, $267 = 0, $268 = 0, $269 = 0, $27 = 0, $270 = 0, $271 = 0, $272 = 0, $273 = 0, $274 = 0, $275 = 0, $276 = 0, $277 = 0, $278 = 0;
 var $279 = 0, $28 = 0, $280 = 0, $281 = 0, $282 = 0, $283 = 0, $284 = 0, $285 = 0, $286 = 0, $287 = 0, $288 = 0, $289 = 0, $29 = 0, $290 = 0, $291 = 0, $292 = 0, $293 = 0, $294 = 0, $295 = 0, $296 = 0;
 var $297 = 0, $298 = 0, $299 = 0, $3 = 0, $30 = 0, $300 = 0, $301 = 0, $302 = 0, $303 = 0, $304 = 0, $305 = 0, $306 = 0, $307 = 0, $308 = 0, $309 = 0, $31 = 0, $310 = 0, $311 = 0, $312 = 0, $313 = 0;
 var $314 = 0, $315 = 0, $316 = 0, $317 = 0, $318 = 0, $319 = 0, $32 = 0, $320 = 0, $321 = 0, $322 = 0, $323 = 0, $324 = 0, $325 = 0, $326 = 0, $327 = 0, $328 = 0, $329 = 0, $33 = 0, $330 = 0, $331 = 0;
 var $332 = 0, $333 = 0, $334 = 0, $335 = 0, $336 = 0, $337 = 0, $338 = 0, $339 = 0, $34 = 0, $340 = 0, $341 = 0, $342 = 0, $343 = 0, $344 = 0, $345 = 0, $346 = 0, $347 = 0, $348 = 0, $349 = 0, $35 = 0;
 var $350 = 0, $351 = 0, $352 = 0, $353 = 0, $354 = 0, $355 = 0, $356 = 0, $357 = 0, $358 = 0, $359 = 0, $36 = 0, $360 = 0, $361 = 0, $362 = 0, $363 = 0, $364 = 0, $365 = 0, $366 = 0, $367 = 0, $368 = 0;
 var $369 = 0, $37 = 0, $370 = 0, $371 = 0, $372 = 0, $373 = 0, $374 = 0, $375 = 0, $376 = 0, $377 = 0, $378 = 0, $379 = 0, $38 = 0, $380 = 0, $381 = 0, $382 = 0, $383 = 0, $384 = 0, $385 = 0, $386 = 0;
 var $387 = 0, $388 = 0, $389 = 0, $39 = 0, $390 = 0, $391 = 0, $392 = 0, $393 = 0, $394 = 0, $395 = 0, $396 = 0, $397 = 0, $398 = 0, $399 = 0, $4 = 0, $40 = 0, $400 = 0, $401 = 0, $402 = 0, $403 = 0;
 var $404 = 0, $405 = 0, $406 = 0, $407 = 0, $408 = 0, $409 = 0, $41 = 0, $410 = 0, $411 = 0, $412 = 0, $413 = 0, $414 = 0, $415 = 0, $416 = 0, $417 = 0, $418 = 0, $419 = 0, $42 = 0, $420 = 0, $421 = 0;
 var $422 = 0, $423 = 0, $424 = 0, $425 = 0, $426 = 0, $427 = 0, $428 = 0, $429 = 0, $43 = 0, $430 = 0, $431 = 0, $432 = 0, $433 = 0, $434 = 0, $435 = 0, $436 = 0, $437 = 0, $438 = 0, $439 = 0, $44 = 0;
 var $440 = 0, $441 = 0, $442 = 0, $443 = 0, $444 = 0, $445 = 0, $446 = 0, $447 = 0, $448 = 0, $449 = 0, $45 = 0, $450 = 0, $451 = 0, $452 = 0, $453 = 0, $454 = 0, $455 = 0, $456 = 0, $457 = 0, $458 = 0;
 var $459 = 0, $46 = 0, $460 = 0, $461 = 0, $462 = 0, $463 = 0, $464 = 0, $465 = 0, $466 = 0, $467 = 0, $468 = 0, $469 = 0, $47 = 0, $470 = 0, $471 = 0, $472 = 0, $473 = 0, $474 = 0, $475 = 0, $476 = 0;
 var $477 = 0, $478 = 0, $479 = 0, $48 = 0, $480 = 0, $481 = 0, $482 = 0, $483 = 0, $484 = 0, $485 = 0, $486 = 0, $487 = 0, $488 = 0, $489 = 0, $49 = 0, $490 = 0, $491 = 0, $492 = 0, $493 = 0, $494 = 0;
 var $495 = 0, $496 = 0, $497 = 0, $498 = 0, $499 = 0, $5 = 0, $50 = 0, $500 = 0, $501 = 0, $502 = 0, $503 = 0, $504 = 0, $505 = 0, $506 = 0, $507 = 0, $508 = 0, $509 = 0, $51 = 0, $510 = 0, $511 = 0;
 var $512 = 0, $513 = 0, $514 = 0, $515 = 0, $516 = 0, $517 = 0, $518 = 0, $519 = 0, $52 = 0, $520 = 0, $521 = 0, $522 = 0, $523 = 0, $524 = 0, $525 = 0, $526 = 0, $527 = 0, $528 = 0, $529 = 0, $53 = 0;
 var $530 = 0, $531 = 0, $532 = 0, $533 = 0, $534 = 0, $535 = 0, $536 = 0, $537 = 0, $538 = 0, $539 = 0, $54 = 0, $540 = 0, $541 = 0, $542 = 0, $543 = 0, $544 = 0, $545 = 0, $546 = 0, $547 = 0, $548 = 0;
 var $549 = 0, $55 = 0, $550 = 0, $551 = 0, $552 = 0, $553 = 0, $554 = 0, $555 = 0, $556 = 0, $557 = 0, $558 = 0, $559 = 0, $56 = 0, $560 = 0, $561 = 0, $562 = 0, $563 = 0, $564 = 0, $565 = 0, $566 = 0;
 var $567 = 0, $568 = 0, $569 = 0, $57 = 0, $570 = 0, $571 = 0, $572 = 0, $573 = 0, $574 = 0, $575 = 0, $576 = 0, $577 = 0, $578 = 0, $579 = 0, $58 = 0, $580 = 0, $581 = 0, $582 = 0, $583 = 0, $584 = 0;
 var $585 = 0, $586 = 0, $587 = 0, $588 = 0, $589 = 0, $59 = 0, $590 = 0, $591 = 0, $592 = 0, $593 = 0, $594 = 0, $595 = 0, $596 = 0, $597 = 0, $598 = 0, $599 = 0, $6 = 0, $60 = 0, $600 = 0, $601 = 0;
 var $602 = 0, $603 = 0, $604 = 0, $605 = 0, $606 = 0, $607 = 0, $608 = 0, $609 = 0, $61 = 0, $610 = 0, $611 = 0, $612 = 0, $613 = 0, $614 = 0, $615 = 0, $616 = 0, $617 = 0, $618 = 0, $619 = 0, $62 = 0;
 var $620 = 0, $621 = 0, $622 = 0, $623 = 0, $624 = 0, $625 = 0, $626 = 0, $627 = 0, $628 = 0, $629 = 0, $63 = 0, $630 = 0, $631 = 0, $632 = 0, $633 = 0, $634 = 0, $635 = 0, $636 = 0, $637 = 0, $638 = 0;
 var $639 = 0, $64 = 0, $640 = 0, $641 = 0, $642 = 0, $643 = 0, $644 = 0, $645 = 0, $646 = 0, $647 = 0, $648 = 0, $649 = 0, $65 = 0, $650 = 0, $651 = 0, $652 = 0, $653 = 0, $654 = 0, $655 = 0, $656 = 0;
 var $657 = 0, $658 = 0, $659 = 0, $66 = 0, $660 = 0, $661 = 0, $662 = 0, $663 = 0, $664 = 0, $665 = 0, $666 = 0, $667 = 0, $668 = 0, $669 = 0, $67 = 0, $670 = 0, $671 = 0, $672 = 0, $673 = 0, $674 = 0;
 var $675 = 0, $676 = 0, $677 = 0, $678 = 0, $679 = 0, $68 = 0, $680 = 0, $681 = 0, $682 = 0, $683 = 0, $684 = 0, $685 = 0, $686 = 0, $687 = 0, $688 = 0, $689 = 0, $69 = 0, $690 = 0, $691 = 0, $692 = 0;
 var $693 = 0, $694 = 0, $695 = 0, $696 = 0, $697 = 0, $698 = 0, $699 = 0, $7 = 0, $70 = 0, $700 = 0, $701 = 0, $702 = 0, $703 = 0, $704 = 0, $705 = 0, $706 = 0, $707 = 0, $708 = 0, $709 = 0, $71 = 0;
 var $710 = 0, $711 = 0, $712 = 0, $713 = 0, $714 = 0, $715 = 0, $716 = 0, $717 = 0, $718 = 0, $719 = 0, $72 = 0, $720 = 0, $721 = 0, $722 = 0, $723 = 0, $724 = 0, $725 = 0, $726 = 0, $727 = 0, $728 = 0;
 var $729 = 0, $73 = 0, $730 = 0, $731 = 0, $732 = 0, $733 = 0, $734 = 0, $735 = 0, $736 = 0, $737 = 0, $738 = 0, $739 = 0, $74 = 0, $740 = 0, $741 = 0, $742 = 0, $743 = 0, $744 = 0, $745 = 0, $746 = 0;
 var $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0;
 var $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $aListelem = 0, $addr = 0, $constMask = 0, $db = 0, $destIfFalse = 0, $destIfNull = 0, $enc = 0, $endCoalesce = 0, $endLabel = 0, $exprOp = 0, $i = 0, $i2 = 0, $iTab = 0;
 var $inReg = 0, $n = 0, $nExpr = 0, $nFarg = 0, $nId = 0, $nextCase = 0, $op = 0, $opCompare = 0, $or$cond = 0, $or$cond3 = 0, $p1 = 0, $pAggInfo = 0, $pCol = 0, $pColl = 0, $pDef = 0, $pEList = 0, $pFarg = 0, $pInfo = 0, $pLItem = 0, $pLeft = 0;
 var $pLeft1 = 0, $pRight = 0, $pTab = 0, $pTest = 0, $pX = 0, $r1 = 0, $r2 = 0, $r3 = 0, $r4 = 0, $regFree1 = 0, $regFree2 = 0, $tempX = 0, $v = 0, $vararg_buffer8 = 0, $vararg_ptr7 = 0, $z = 0, $zBlob = 0, $zId = 0, dest = 0, label = 0;
 var sp = 0, src = 0, stop = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 304|0;
 $vararg_buffer8 = sp;
 $0 = sp + 292|0;
 $1 = sp + 288|0;
 $2 = sp + 284|0;
 $3 = sp + 280|0;
 $v = sp + 276|0;
 $op = sp + 272|0;
 $inReg = sp + 268|0;
 $regFree1 = sp + 264|0;
 $regFree2 = sp + 260|0;
 $r1 = sp + 256|0;
 $r2 = sp + 252|0;
 $r3 = sp + 248|0;
 $r4 = sp + 244|0;
 $db = sp + 240|0;
 $tempX = sp + 192|0;
 $pAggInfo = sp + 184|0;
 $pCol = sp + 180|0;
 $iTab = sp + 176|0;
 $n = sp + 172|0;
 $z = sp + 168|0;
 $zBlob = sp + 164|0;
 $pLeft = sp + 160|0;
 $addr = sp + 156|0;
 $pInfo = sp + 152|0;
 $pFarg = sp + 148|0;
 $nFarg = sp + 144|0;
 $pDef = sp + 140|0;
 $nId = sp + 136|0;
 $zId = sp + 132|0;
 $constMask = sp + 128|0;
 $i = sp + 124|0;
 $enc = sp + 297|0;
 $pColl = sp + 120|0;
 $endCoalesce = sp + 116|0;
 $exprOp = sp + 296|0;
 $destIfFalse = sp + 112|0;
 $destIfNull = sp + 108|0;
 $pLeft1 = sp + 104|0;
 $pLItem = sp + 100|0;
 $pRight = sp + 96|0;
 $pTab = sp + 92|0;
 $p1 = sp + 88|0;
 $endLabel = sp + 84|0;
 $nextCase = sp + 80|0;
 $nExpr = sp + 76|0;
 $i2 = sp + 72|0;
 $pEList = sp + 68|0;
 $aListelem = sp + 64|0;
 $opCompare = sp + 16|0;
 $pX = sp + 12|0;
 $pTest = sp + 8|0;
 HEAP32[$1>>2] = $pParse;
 HEAP32[$2>>2] = $pExpr;
 HEAP32[$3>>2] = $target;
 $4 = HEAP32[$1>>2]|0;
 $5 = (($4) + 8|0);
 $6 = HEAP32[$5>>2]|0;
 HEAP32[$v>>2] = $6;
 $7 = HEAP32[$3>>2]|0;
 HEAP32[$inReg>>2] = $7;
 HEAP32[$regFree1>>2] = 0;
 HEAP32[$regFree2>>2] = 0;
 $8 = HEAP32[$1>>2]|0;
 $9 = HEAP32[$8>>2]|0;
 HEAP32[$db>>2] = $9;
 $10 = HEAP32[$v>>2]|0;
 $11 = ($10|0)==(0|0);
 if ($11) {
  HEAP32[$0>>2] = 0;
  $746 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($746|0);
 }
 $12 = HEAP32[$2>>2]|0;
 $13 = ($12|0)==(0|0);
 if ($13) {
  HEAP32[$op>>2] = 101;
 } else {
  $14 = HEAP32[$2>>2]|0;
  $15 = HEAP8[$14>>0]|0;
  $16 = $15&255;
  HEAP32[$op>>2] = $16;
 }
 $17 = HEAP32[$op>>2]|0;
 L9: do {
  switch ($17|0) {
  case 19: case 96:  {
   $261 = HEAP32[$1>>2]|0;
   $262 = HEAP32[$2>>2]|0;
   $263 = (($262) + 12|0);
   $264 = HEAP32[$263>>2]|0;
   $265 = (_sqlite3ExprCodeTemp($261,$264,$regFree1)|0);
   HEAP32[$r1>>2] = $265;
   $266 = HEAP32[$3>>2]|0;
   HEAP32[$inReg>>2] = $266;
   $267 = HEAP32[$v>>2]|0;
   $268 = HEAP32[$op>>2]|0;
   $269 = HEAP32[$r1>>2]|0;
   $270 = HEAP32[$inReg>>2]|0;
   (_sqlite3VdbeAddOp2($267,$268,$269,$270)|0);
   break;
  }
  case 157:  {
   $228 = HEAP32[$2>>2]|0;
   $229 = (($228) + 12|0);
   $230 = HEAP32[$229>>2]|0;
   HEAP32[$pLeft>>2] = $230;
   $231 = HEAP32[$pLeft>>2]|0;
   $232 = HEAP8[$231>>0]|0;
   $233 = $232&255;
   $234 = ($233|0)==(132);
   do {
    if ($234) {
     $235 = HEAP32[$1>>2]|0;
     $236 = HEAP32[$pLeft>>2]|0;
     $237 = HEAP32[$3>>2]|0;
     _codeInteger($235,$236,1,$237);
    } else {
     $238 = HEAP32[$pLeft>>2]|0;
     $239 = HEAP8[$238>>0]|0;
     $240 = $239&255;
     $241 = ($240|0)==(133);
     if ($241) {
      $242 = HEAP32[$v>>2]|0;
      $243 = HEAP32[$pLeft>>2]|0;
      $244 = (($243) + 8|0);
      $245 = HEAP32[$244>>2]|0;
      $246 = HEAP32[$3>>2]|0;
      _codeReal($242,$245,1,$246);
      break;
     } else {
      HEAP8[$tempX>>0] = -124;
      $247 = (($tempX) + 4|0);
      HEAP32[$247>>2] = 17408;
      $248 = (($tempX) + 8|0);
      HEAP32[$248>>2] = 0;
      $249 = HEAP32[$1>>2]|0;
      $250 = (_sqlite3ExprCodeTemp($249,$tempX,$regFree1)|0);
      HEAP32[$r1>>2] = $250;
      $251 = HEAP32[$1>>2]|0;
      $252 = HEAP32[$2>>2]|0;
      $253 = (($252) + 12|0);
      $254 = HEAP32[$253>>2]|0;
      $255 = (_sqlite3ExprCodeTemp($251,$254,$regFree2)|0);
      HEAP32[$r2>>2] = $255;
      $256 = HEAP32[$v>>2]|0;
      $257 = HEAP32[$r2>>2]|0;
      $258 = HEAP32[$r1>>2]|0;
      $259 = HEAP32[$3>>2]|0;
      (_sqlite3VdbeAddOp3($256,90,$257,$258,$259)|0);
      break;
     }
    }
   } while(0);
   $260 = HEAP32[$3>>2]|0;
   HEAP32[$inReg>>2] = $260;
   break;
  }
  case 24:  {
   $140 = HEAP32[$1>>2]|0;
   $141 = HEAP32[$2>>2]|0;
   $142 = (($141) + 12|0);
   $143 = HEAP32[$142>>2]|0;
   $144 = HEAP32[$3>>2]|0;
   $145 = (_sqlite3ExprCodeTarget($140,$143,$144)|0);
   HEAP32[$inReg>>2] = $145;
   break;
  }
  case 79: case 78: case 83: case 80: case 81: case 82:  {
   $168 = HEAP32[$1>>2]|0;
   $169 = HEAP32[$2>>2]|0;
   $170 = (($169) + 12|0);
   $171 = HEAP32[$170>>2]|0;
   $172 = (_sqlite3ExprCodeTemp($168,$171,$regFree1)|0);
   HEAP32[$r1>>2] = $172;
   $173 = HEAP32[$1>>2]|0;
   $174 = HEAP32[$2>>2]|0;
   $175 = (($174) + 16|0);
   $176 = HEAP32[$175>>2]|0;
   $177 = (_sqlite3ExprCodeTemp($173,$176,$regFree2)|0);
   HEAP32[$r2>>2] = $177;
   $178 = HEAP32[$1>>2]|0;
   $179 = HEAP32[$2>>2]|0;
   $180 = (($179) + 12|0);
   $181 = HEAP32[$180>>2]|0;
   $182 = HEAP32[$2>>2]|0;
   $183 = (($182) + 16|0);
   $184 = HEAP32[$183>>2]|0;
   $185 = HEAP32[$op>>2]|0;
   $186 = HEAP32[$r1>>2]|0;
   $187 = HEAP32[$r2>>2]|0;
   $188 = HEAP32[$inReg>>2]|0;
   (_codeCompare($178,$181,$184,$185,$186,$187,$188,32)|0);
   break;
  }
  case 148: case 73:  {
   $189 = HEAP32[$1>>2]|0;
   $190 = HEAP32[$2>>2]|0;
   $191 = (($190) + 12|0);
   $192 = HEAP32[$191>>2]|0;
   $193 = (_sqlite3ExprCodeTemp($189,$192,$regFree1)|0);
   HEAP32[$r1>>2] = $193;
   $194 = HEAP32[$1>>2]|0;
   $195 = HEAP32[$2>>2]|0;
   $196 = (($195) + 16|0);
   $197 = HEAP32[$196>>2]|0;
   $198 = (_sqlite3ExprCodeTemp($194,$197,$regFree2)|0);
   HEAP32[$r2>>2] = $198;
   $199 = HEAP32[$op>>2]|0;
   $200 = ($199|0)==(73);
   $201 = $200 ? 79 : 78;
   HEAP32[$op>>2] = $201;
   $202 = HEAP32[$1>>2]|0;
   $203 = HEAP32[$2>>2]|0;
   $204 = (($203) + 12|0);
   $205 = HEAP32[$204>>2]|0;
   $206 = HEAP32[$2>>2]|0;
   $207 = (($206) + 16|0);
   $208 = HEAP32[$207>>2]|0;
   $209 = HEAP32[$op>>2]|0;
   $210 = HEAP32[$r1>>2]|0;
   $211 = HEAP32[$r2>>2]|0;
   $212 = HEAP32[$inReg>>2]|0;
   (_codeCompare($202,$205,$208,$209,$210,$211,$212,160)|0);
   break;
  }
  case 153:  {
   HEAP32[$constMask>>2] = 0;
   $305 = HEAP32[$db>>2]|0;
   $306 = (($305) + 16|0);
   $307 = HEAP32[$306>>2]|0;
   $308 = (($307) + 12|0);
   $309 = HEAP32[$308>>2]|0;
   $310 = (($309) + 77|0);
   $311 = HEAP8[$310>>0]|0;
   HEAP8[$enc>>0] = $311;
   HEAP32[$pColl>>2] = 0;
   $312 = HEAP32[$2>>2]|0;
   $313 = (($312) + 4|0);
   $314 = HEAP32[$313>>2]|0;
   $315 = $314 & 16384;
   $316 = ($315|0)!=(0);
   if ($316) {
    HEAP32[$pFarg>>2] = 0;
   } else {
    $317 = HEAP32[$2>>2]|0;
    $318 = (($317) + 20|0);
    $319 = HEAP32[$318>>2]|0;
    HEAP32[$pFarg>>2] = $319;
   }
   $320 = HEAP32[$pFarg>>2]|0;
   $321 = ($320|0)!=(0|0);
   if ($321) {
    $322 = HEAP32[$pFarg>>2]|0;
    $323 = HEAP32[$322>>2]|0;
    $324 = $323;
   } else {
    $324 = 0;
   }
   HEAP32[$nFarg>>2] = $324;
   $325 = HEAP32[$2>>2]|0;
   $326 = (($325) + 8|0);
   $327 = HEAP32[$326>>2]|0;
   HEAP32[$zId>>2] = $327;
   $328 = HEAP32[$zId>>2]|0;
   $329 = (_sqlite3Strlen30($328)|0);
   HEAP32[$nId>>2] = $329;
   $330 = HEAP32[$db>>2]|0;
   $331 = HEAP32[$zId>>2]|0;
   $332 = HEAP32[$nId>>2]|0;
   $333 = HEAP32[$nFarg>>2]|0;
   $334 = HEAP8[$enc>>0]|0;
   $335 = (_sqlite3FindFunction($330,$331,$332,$333,$334,0)|0);
   HEAP32[$pDef>>2] = $335;
   $336 = HEAP32[$pDef>>2]|0;
   $337 = ($336|0)==(0|0);
   if (!($337)) {
    $338 = HEAP32[$pDef>>2]|0;
    $339 = (($338) + 12|0);
    $340 = HEAP32[$339>>2]|0;
    $341 = ($340|0)==(0|0);
    if (!($341)) {
     $345 = HEAP32[$pDef>>2]|0;
     $346 = (($345) + 2|0);
     $347 = HEAP16[$346>>1]|0;
     $348 = $347&65535;
     $349 = $348 & 512;
     $350 = ($349|0)!=(0);
     if ($350) {
      $351 = HEAP32[$v>>2]|0;
      $352 = (_sqlite3VdbeMakeLabel($351)|0);
      HEAP32[$endCoalesce>>2] = $352;
      $353 = HEAP32[$1>>2]|0;
      $354 = HEAP32[$pFarg>>2]|0;
      $355 = (($354) + 4|0);
      $356 = HEAP32[$355>>2]|0;
      $357 = HEAP32[$356>>2]|0;
      $358 = HEAP32[$3>>2]|0;
      _sqlite3ExprCode($353,$357,$358);
      HEAP32[$i>>2] = 1;
      while(1) {
       $359 = HEAP32[$i>>2]|0;
       $360 = HEAP32[$nFarg>>2]|0;
       $361 = ($359|0)<($360|0);
       $362 = HEAP32[$v>>2]|0;
       if (!($361)) {
        break;
       }
       $363 = HEAP32[$3>>2]|0;
       $364 = HEAP32[$endCoalesce>>2]|0;
       (_sqlite3VdbeAddOp2($362,77,$363,$364)|0);
       $365 = HEAP32[$1>>2]|0;
       $366 = HEAP32[$3>>2]|0;
       _sqlite3ExprCacheRemove($365,$366,1);
       $367 = HEAP32[$1>>2]|0;
       _sqlite3ExprCachePush($367);
       $368 = HEAP32[$1>>2]|0;
       $369 = HEAP32[$i>>2]|0;
       $370 = HEAP32[$pFarg>>2]|0;
       $371 = (($370) + 4|0);
       $372 = HEAP32[$371>>2]|0;
       $373 = (($372) + (($369*20)|0)|0);
       $374 = HEAP32[$373>>2]|0;
       $375 = HEAP32[$3>>2]|0;
       _sqlite3ExprCode($368,$374,$375);
       $376 = HEAP32[$1>>2]|0;
       _sqlite3ExprCachePop($376);
       $377 = HEAP32[$i>>2]|0;
       $378 = (($377) + 1)|0;
       HEAP32[$i>>2] = $378;
      }
      $379 = HEAP32[$endCoalesce>>2]|0;
      _sqlite3VdbeResolveLabel($362,$379);
      break L9;
     }
     $380 = HEAP32[$pDef>>2]|0;
     $381 = (($380) + 2|0);
     $382 = HEAP16[$381>>1]|0;
     $383 = $382&65535;
     $384 = $383 & 1024;
     $385 = ($384|0)!=(0);
     if ($385) {
      $386 = HEAP32[$1>>2]|0;
      $387 = HEAP32[$pFarg>>2]|0;
      $388 = (($387) + 4|0);
      $389 = HEAP32[$388>>2]|0;
      $390 = HEAP32[$389>>2]|0;
      $391 = HEAP32[$3>>2]|0;
      _sqlite3ExprCode($386,$390,$391);
      break L9;
     }
     HEAP32[$i>>2] = 0;
     while(1) {
      $392 = HEAP32[$i>>2]|0;
      $393 = HEAP32[$nFarg>>2]|0;
      $394 = ($392|0)<($393|0);
      if (!($394)) {
       break;
      }
      $395 = HEAP32[$i>>2]|0;
      $396 = ($395|0)<(32);
      if ($396) {
       $397 = HEAP32[$i>>2]|0;
       $398 = HEAP32[$pFarg>>2]|0;
       $399 = (($398) + 4|0);
       $400 = HEAP32[$399>>2]|0;
       $401 = (($400) + (($397*20)|0)|0);
       $402 = HEAP32[$401>>2]|0;
       $403 = (_sqlite3ExprIsConstant($402)|0);
       $404 = ($403|0)!=(0);
       if ($404) {
        $405 = HEAP32[$i>>2]|0;
        $406 = 1 << $405;
        $407 = HEAP32[$constMask>>2]|0;
        $408 = $407 | $406;
        HEAP32[$constMask>>2] = $408;
       }
      }
      $409 = HEAP32[$pDef>>2]|0;
      $410 = (($409) + 2|0);
      $411 = HEAP16[$410>>1]|0;
      $412 = $411&65535;
      $413 = $412 & 32;
      $414 = ($413|0)==(0);
      $415 = HEAP32[$pColl>>2]|0;
      $416 = ($415|0)!=(0|0);
      $or$cond = $414 | $416;
      if (!($or$cond)) {
       $417 = HEAP32[$1>>2]|0;
       $418 = HEAP32[$i>>2]|0;
       $419 = HEAP32[$pFarg>>2]|0;
       $420 = (($419) + 4|0);
       $421 = HEAP32[$420>>2]|0;
       $422 = (($421) + (($418*20)|0)|0);
       $423 = HEAP32[$422>>2]|0;
       $424 = (_sqlite3ExprCollSeq($417,$423)|0);
       HEAP32[$pColl>>2] = $424;
      }
      $425 = HEAP32[$i>>2]|0;
      $426 = (($425) + 1)|0;
      HEAP32[$i>>2] = $426;
     }
     $427 = HEAP32[$pFarg>>2]|0;
     $428 = ($427|0)!=(0|0);
     if ($428) {
      $429 = HEAP32[$constMask>>2]|0;
      $430 = ($429|0)!=(0);
      $431 = HEAP32[$1>>2]|0;
      if ($430) {
       $432 = (($431) + 72|0);
       $433 = HEAP32[$432>>2]|0;
       $434 = (($433) + 1)|0;
       HEAP32[$r1>>2] = $434;
       $435 = HEAP32[$nFarg>>2]|0;
       $436 = HEAP32[$1>>2]|0;
       $437 = (($436) + 72|0);
       $438 = HEAP32[$437>>2]|0;
       $439 = (($438) + ($435))|0;
       HEAP32[$437>>2] = $439;
      } else {
       $440 = HEAP32[$nFarg>>2]|0;
       $441 = (_sqlite3GetTempRange($431,$440)|0);
       HEAP32[$r1>>2] = $441;
      }
      $442 = HEAP32[$pDef>>2]|0;
      $443 = (($442) + 2|0);
      $444 = HEAP16[$443>>1]|0;
      $445 = $444&65535;
      $446 = $445 & 192;
      $447 = ($446|0)!=(0);
      do {
       if ($447) {
        $448 = HEAP32[$pFarg>>2]|0;
        $449 = (($448) + 4|0);
        $450 = HEAP32[$449>>2]|0;
        $451 = HEAP32[$450>>2]|0;
        $452 = HEAP8[$451>>0]|0;
        HEAP8[$exprOp>>0] = $452;
        $453 = HEAP8[$exprOp>>0]|0;
        $454 = $453&255;
        $455 = ($454|0)==(154);
        if (!($455)) {
         $456 = HEAP8[$exprOp>>0]|0;
         $457 = $456&255;
         $458 = ($457|0)==(156);
         if (!($458)) {
          break;
         }
        }
        $459 = HEAP32[$pDef>>2]|0;
        $460 = (($459) + 2|0);
        $461 = HEAP16[$460>>1]|0;
        $462 = $461&65535;
        $463 = $462 & 192;
        $464 = $463&255;
        $465 = HEAP32[$pFarg>>2]|0;
        $466 = (($465) + 4|0);
        $467 = HEAP32[$466>>2]|0;
        $468 = HEAP32[$467>>2]|0;
        $469 = (($468) + 38|0);
        HEAP8[$469>>0] = $464;
       }
      } while(0);
      $470 = HEAP32[$1>>2]|0;
      _sqlite3ExprCachePush($470);
      $471 = HEAP32[$1>>2]|0;
      $472 = HEAP32[$pFarg>>2]|0;
      $473 = HEAP32[$r1>>2]|0;
      (_sqlite3ExprCodeExprList($471,$472,$473,3)|0);
      $474 = HEAP32[$1>>2]|0;
      _sqlite3ExprCachePop($474);
     } else {
      HEAP32[$r1>>2] = 0;
     }
     $475 = HEAP32[$nFarg>>2]|0;
     $476 = ($475|0)>=(2);
     if ($476) {
      $477 = HEAP32[$2>>2]|0;
      $478 = (($477) + 4|0);
      $479 = HEAP32[$478>>2]|0;
      $480 = $479 & 128;
      $481 = ($480|0)!=(0);
      if ($481) {
       $482 = HEAP32[$db>>2]|0;
       $483 = HEAP32[$pDef>>2]|0;
       $484 = HEAP32[$nFarg>>2]|0;
       $485 = HEAP32[$pFarg>>2]|0;
       $486 = (($485) + 4|0);
       $487 = HEAP32[$486>>2]|0;
       $488 = (($487) + 20|0);
       $489 = HEAP32[$488>>2]|0;
       $490 = (_sqlite3VtabOverloadFunction($482,$483,$484,$489)|0);
       HEAP32[$pDef>>2] = $490;
      } else {
       label = 78;
      }
     } else {
      label = 78;
     }
     if ((label|0) == 78) {
      $491 = HEAP32[$nFarg>>2]|0;
      $492 = ($491|0)>(0);
      if ($492) {
       $493 = HEAP32[$db>>2]|0;
       $494 = HEAP32[$pDef>>2]|0;
       $495 = HEAP32[$nFarg>>2]|0;
       $496 = HEAP32[$pFarg>>2]|0;
       $497 = (($496) + 4|0);
       $498 = HEAP32[$497>>2]|0;
       $499 = HEAP32[$498>>2]|0;
       $500 = (_sqlite3VtabOverloadFunction($493,$494,$495,$499)|0);
       HEAP32[$pDef>>2] = $500;
      }
     }
     $501 = HEAP32[$pDef>>2]|0;
     $502 = (($501) + 2|0);
     $503 = HEAP16[$502>>1]|0;
     $504 = $503&65535;
     $505 = $504 & 32;
     $506 = ($505|0)!=(0);
     if ($506) {
      $507 = HEAP32[$pColl>>2]|0;
      $508 = ($507|0)!=(0|0);
      if (!($508)) {
       $509 = HEAP32[$db>>2]|0;
       $510 = (($509) + 8|0);
       $511 = HEAP32[$510>>2]|0;
       HEAP32[$pColl>>2] = $511;
      }
      $512 = HEAP32[$v>>2]|0;
      $513 = HEAP32[$pColl>>2]|0;
      (_sqlite3VdbeAddOp4($512,36,0,0,0,$513,-4)|0);
     }
     $514 = HEAP32[$v>>2]|0;
     $515 = HEAP32[$constMask>>2]|0;
     $516 = HEAP32[$r1>>2]|0;
     $517 = HEAP32[$3>>2]|0;
     $518 = HEAP32[$pDef>>2]|0;
     (_sqlite3VdbeAddOp4($514,1,$515,$516,$517,$518,-5)|0);
     $519 = HEAP32[$v>>2]|0;
     $520 = HEAP32[$nFarg>>2]|0;
     $521 = $520&255;
     _sqlite3VdbeChangeP5($519,$521);
     $522 = HEAP32[$nFarg>>2]|0;
     $523 = ($522|0)!=(0);
     $524 = HEAP32[$constMask>>2]|0;
     $525 = ($524|0)==(0);
     $or$cond3 = $523 & $525;
     if (!($or$cond3)) {
      break L9;
     }
     $526 = HEAP32[$1>>2]|0;
     $527 = HEAP32[$r1>>2]|0;
     $528 = HEAP32[$nFarg>>2]|0;
     _sqlite3ReleaseTempRange($526,$527,$528);
     break L9;
    }
   }
   $342 = HEAP32[$1>>2]|0;
   $343 = HEAP32[$nId>>2]|0;
   $344 = HEAP32[$zId>>2]|0;
   HEAP32[$vararg_buffer8>>2] = $343;
   $vararg_ptr7 = (($vararg_buffer8) + 4|0);
   HEAP32[$vararg_ptr7>>2] = $344;
   _sqlite3ErrorMsg($342,19160,$vararg_buffer8);
   break;
  }
  case 38:  {
   $146 = HEAP32[$1>>2]|0;
   $147 = HEAP32[$2>>2]|0;
   $148 = (($147) + 12|0);
   $149 = HEAP32[$148>>2]|0;
   $150 = HEAP32[$3>>2]|0;
   $151 = (_sqlite3ExprCodeTarget($146,$149,$150)|0);
   HEAP32[$inReg>>2] = $151;
   $152 = HEAP32[$inReg>>2]|0;
   $153 = HEAP32[$3>>2]|0;
   $154 = ($152|0)!=($153|0);
   if ($154) {
    $155 = HEAP32[$v>>2]|0;
    $156 = HEAP32[$inReg>>2]|0;
    $157 = HEAP32[$3>>2]|0;
    (_sqlite3VdbeAddOp2($155,34,$156,$157)|0);
    $158 = HEAP32[$3>>2]|0;
    HEAP32[$inReg>>2] = $158;
   }
   $159 = HEAP32[$v>>2]|0;
   $160 = HEAP32[$3>>2]|0;
   $161 = HEAP32[$2>>2]|0;
   $162 = (($161) + 8|0);
   $163 = HEAP32[$162>>2]|0;
   $164 = (_sqlite3AffinityType($163,0)|0);
   $165 = $164 << 24 >> 24;
   (_sqlite3VdbeAddOp2($159,40,$160,$165)|0);
   $166 = HEAP32[$1>>2]|0;
   $167 = HEAP32[$inReg>>2]|0;
   _sqlite3ExprCacheAffinityChange($166,$167,1);
   break;
  }
  case 101:  {
   $94 = HEAP32[$v>>2]|0;
   $95 = HEAP32[$3>>2]|0;
   (_sqlite3VdbeAddOp2($94,28,0,$95)|0);
   break;
  }
  case 159:  {
   $137 = HEAP32[$2>>2]|0;
   $138 = (($137) + 28|0);
   $139 = HEAP32[$138>>2]|0;
   HEAP32[$inReg>>2] = $139;
   break;
  }
  case 135:  {
   $113 = HEAP32[$v>>2]|0;
   $114 = HEAP32[$2>>2]|0;
   $115 = (($114) + 32|0);
   $116 = HEAP16[$115>>1]|0;
   $117 = $116 << 16 >> 16;
   $118 = HEAP32[$3>>2]|0;
   (_sqlite3VdbeAddOp2($113,31,$117,$118)|0);
   $119 = HEAP32[$2>>2]|0;
   $120 = (($119) + 8|0);
   $121 = HEAP32[$120>>2]|0;
   $122 = (($121) + 1|0);
   $123 = HEAP8[$122>>0]|0;
   $124 = $123 << 24 >> 24;
   $125 = ($124|0)!=(0);
   if ($125) {
    $126 = HEAP32[$v>>2]|0;
    $127 = HEAP32[$2>>2]|0;
    $128 = (($127) + 32|0);
    $129 = HEAP16[$128>>1]|0;
    $130 = $129 << 16 >> 16;
    $131 = (($130) - 1)|0;
    $132 = HEAP32[$1>>2]|0;
    $133 = (($132) + 472|0);
    $134 = HEAP32[$133>>2]|0;
    $135 = (($134) + ($131<<2)|0);
    $136 = HEAP32[$135>>2]|0;
    _sqlite3VdbeChangeP4($126,-1,$136,-2);
   }
   break;
  }
  case 97:  {
   $89 = HEAP32[$v>>2]|0;
   $90 = HEAP32[$3>>2]|0;
   $91 = HEAP32[$2>>2]|0;
   $92 = (($91) + 8|0);
   $93 = HEAP32[$92>>2]|0;
   (_sqlite3VdbeAddOp4($89,97,0,$90,0,$93,0)|0);
   break;
  }
  case 133:  {
   $84 = HEAP32[$v>>2]|0;
   $85 = HEAP32[$2>>2]|0;
   $86 = (($85) + 8|0);
   $87 = HEAP32[$86>>2]|0;
   $88 = HEAP32[$3>>2]|0;
   _codeReal($84,$87,0,$88);
   break;
  }
  case 132:  {
   $81 = HEAP32[$1>>2]|0;
   $82 = HEAP32[$2>>2]|0;
   $83 = HEAP32[$3>>2]|0;
   _codeInteger($81,$82,0,$83);
   break;
  }
  case 155:  {
   $286 = HEAP32[$2>>2]|0;
   $287 = (($286) + 40|0);
   $288 = HEAP32[$287>>2]|0;
   HEAP32[$pInfo>>2] = $288;
   $289 = HEAP32[$pInfo>>2]|0;
   $290 = ($289|0)==(0|0);
   if ($290) {
    $291 = HEAP32[$1>>2]|0;
    $292 = HEAP32[$2>>2]|0;
    $293 = (($292) + 8|0);
    $294 = HEAP32[$293>>2]|0;
    HEAP32[$vararg_buffer8>>2] = $294;
    _sqlite3ErrorMsg($291,19128,$vararg_buffer8);
    break L9;
   } else {
    $295 = HEAP32[$2>>2]|0;
    $296 = (($295) + 34|0);
    $297 = HEAP16[$296>>1]|0;
    $298 = $297 << 16 >> 16;
    $299 = HEAP32[$pInfo>>2]|0;
    $300 = (($299) + 40|0);
    $301 = HEAP32[$300>>2]|0;
    $302 = (($301) + ($298<<4)|0);
    $303 = (($302) + 8|0);
    $304 = HEAP32[$303>>2]|0;
    HEAP32[$inReg>>2] = $304;
    break L9;
   }
   break;
  }
  case 77: case 76:  {
   $271 = HEAP32[$v>>2]|0;
   $272 = HEAP32[$3>>2]|0;
   (_sqlite3VdbeAddOp2($271,25,1,$272)|0);
   $273 = HEAP32[$1>>2]|0;
   $274 = HEAP32[$2>>2]|0;
   $275 = (($274) + 12|0);
   $276 = HEAP32[$275>>2]|0;
   $277 = (_sqlite3ExprCodeTemp($273,$276,$regFree1)|0);
   HEAP32[$r1>>2] = $277;
   $278 = HEAP32[$v>>2]|0;
   $279 = HEAP32[$op>>2]|0;
   $280 = HEAP32[$r1>>2]|0;
   $281 = (_sqlite3VdbeAddOp1($278,$279,$280)|0);
   HEAP32[$addr>>2] = $281;
   $282 = HEAP32[$v>>2]|0;
   $283 = HEAP32[$3>>2]|0;
   (_sqlite3VdbeAddOp2($282,25,0,$283)|0);
   $284 = HEAP32[$v>>2]|0;
   $285 = HEAP32[$addr>>2]|0;
   _sqlite3VdbeJumpHere($284,$285);
   break;
  }
  case 154:  {
   label = 11;
   break;
  }
  case 75:  {
   $532 = HEAP32[$v>>2]|0;
   $533 = (_sqlite3VdbeMakeLabel($532)|0);
   HEAP32[$destIfFalse>>2] = $533;
   $534 = HEAP32[$v>>2]|0;
   $535 = (_sqlite3VdbeMakeLabel($534)|0);
   HEAP32[$destIfNull>>2] = $535;
   $536 = HEAP32[$v>>2]|0;
   $537 = HEAP32[$3>>2]|0;
   (_sqlite3VdbeAddOp2($536,28,0,$537)|0);
   $538 = HEAP32[$1>>2]|0;
   $539 = HEAP32[$2>>2]|0;
   $540 = HEAP32[$destIfFalse>>2]|0;
   $541 = HEAP32[$destIfNull>>2]|0;
   _sqlite3ExprCodeIN($538,$539,$540,$541);
   $542 = HEAP32[$v>>2]|0;
   $543 = HEAP32[$3>>2]|0;
   (_sqlite3VdbeAddOp2($542,25,1,$543)|0);
   $544 = HEAP32[$v>>2]|0;
   $545 = HEAP32[$destIfFalse>>2]|0;
   _sqlite3VdbeResolveLabel($544,$545);
   $546 = HEAP32[$v>>2]|0;
   $547 = HEAP32[$3>>2]|0;
   (_sqlite3VdbeAddOp2($546,37,$547,0)|0);
   $548 = HEAP32[$v>>2]|0;
   $549 = HEAP32[$destIfNull>>2]|0;
   _sqlite3VdbeResolveLabel($548,$549);
   break;
  }
  case 119: case 20:  {
   $529 = HEAP32[$1>>2]|0;
   $530 = HEAP32[$2>>2]|0;
   $531 = (_sqlite3CodeSubselect($529,$530,0,0)|0);
   HEAP32[$inReg>>2] = $531;
   break;
  }
  case 156:  {
   $18 = HEAP32[$2>>2]|0;
   $19 = (($18) + 40|0);
   $20 = HEAP32[$19>>2]|0;
   HEAP32[$pAggInfo>>2] = $20;
   $21 = HEAP32[$2>>2]|0;
   $22 = (($21) + 34|0);
   $23 = HEAP16[$22>>1]|0;
   $24 = $23 << 16 >> 16;
   $25 = HEAP32[$pAggInfo>>2]|0;
   $26 = (($25) + 28|0);
   $27 = HEAP32[$26>>2]|0;
   $28 = (($27) + (($24*24)|0)|0);
   HEAP32[$pCol>>2] = $28;
   $29 = HEAP32[$pAggInfo>>2]|0;
   $30 = HEAP8[$29>>0]|0;
   $31 = ($30<<24>>24)!=(0);
   if (!($31)) {
    $32 = HEAP32[$pCol>>2]|0;
    $33 = (($32) + 16|0);
    $34 = HEAP32[$33>>2]|0;
    HEAP32[$inReg>>2] = $34;
    break L9;
   }
   $35 = HEAP32[$pAggInfo>>2]|0;
   $36 = (($35) + 1|0);
   $37 = HEAP8[$36>>0]|0;
   $38 = ($37<<24>>24)!=(0);
   if ($38) {
    $39 = HEAP32[$v>>2]|0;
    $40 = HEAP32[$pAggInfo>>2]|0;
    $41 = (($40) + 8|0);
    $42 = HEAP32[$41>>2]|0;
    $43 = HEAP32[$pCol>>2]|0;
    $44 = (($43) + 12|0);
    $45 = HEAP32[$44>>2]|0;
    $46 = HEAP32[$3>>2]|0;
    (_sqlite3VdbeAddOp3($39,47,$42,$45,$46)|0);
   } else {
    label = 11;
   }
   break;
  }
  case 94: case 88: case 87: case 92: case 86: case 85: case 93: case 90: case 91: case 89: case 71: case 72:  {
   $213 = HEAP32[$1>>2]|0;
   $214 = HEAP32[$2>>2]|0;
   $215 = (($214) + 12|0);
   $216 = HEAP32[$215>>2]|0;
   $217 = (_sqlite3ExprCodeTemp($213,$216,$regFree1)|0);
   HEAP32[$r1>>2] = $217;
   $218 = HEAP32[$1>>2]|0;
   $219 = HEAP32[$2>>2]|0;
   $220 = (($219) + 16|0);
   $221 = HEAP32[$220>>2]|0;
   $222 = (_sqlite3ExprCodeTemp($218,$221,$regFree2)|0);
   HEAP32[$r2>>2] = $222;
   $223 = HEAP32[$v>>2]|0;
   $224 = HEAP32[$op>>2]|0;
   $225 = HEAP32[$r2>>2]|0;
   $226 = HEAP32[$r1>>2]|0;
   $227 = HEAP32[$3>>2]|0;
   (_sqlite3VdbeAddOp3($223,$224,$225,$226,$227)|0);
   break;
  }
  case 158: case 95:  {
   $599 = HEAP32[$1>>2]|0;
   $600 = HEAP32[$2>>2]|0;
   $601 = (($600) + 12|0);
   $602 = HEAP32[$601>>2]|0;
   $603 = HEAP32[$3>>2]|0;
   $604 = (_sqlite3ExprCodeTarget($599,$602,$603)|0);
   HEAP32[$inReg>>2] = $604;
   break;
  }
  case 74:  {
   $550 = HEAP32[$2>>2]|0;
   $551 = (($550) + 12|0);
   $552 = HEAP32[$551>>2]|0;
   HEAP32[$pLeft1>>2] = $552;
   $553 = HEAP32[$2>>2]|0;
   $554 = (($553) + 20|0);
   $555 = HEAP32[$554>>2]|0;
   $556 = (($555) + 4|0);
   $557 = HEAP32[$556>>2]|0;
   HEAP32[$pLItem>>2] = $557;
   $558 = HEAP32[$pLItem>>2]|0;
   $559 = HEAP32[$558>>2]|0;
   HEAP32[$pRight>>2] = $559;
   $560 = HEAP32[$1>>2]|0;
   $561 = HEAP32[$pLeft1>>2]|0;
   $562 = (_sqlite3ExprCodeTemp($560,$561,$regFree1)|0);
   HEAP32[$r1>>2] = $562;
   $563 = HEAP32[$1>>2]|0;
   $564 = HEAP32[$pRight>>2]|0;
   $565 = (_sqlite3ExprCodeTemp($563,$564,$regFree2)|0);
   HEAP32[$r2>>2] = $565;
   $566 = HEAP32[$1>>2]|0;
   $567 = (_sqlite3GetTempReg($566)|0);
   HEAP32[$r3>>2] = $567;
   $568 = HEAP32[$1>>2]|0;
   $569 = (_sqlite3GetTempReg($568)|0);
   HEAP32[$r4>>2] = $569;
   $570 = HEAP32[$1>>2]|0;
   $571 = HEAP32[$pLeft1>>2]|0;
   $572 = HEAP32[$pRight>>2]|0;
   $573 = HEAP32[$r1>>2]|0;
   $574 = HEAP32[$r2>>2]|0;
   $575 = HEAP32[$r3>>2]|0;
   (_codeCompare($570,$571,$572,83,$573,$574,$575,32)|0);
   $576 = HEAP32[$pLItem>>2]|0;
   $577 = (($576) + 20|0);
   HEAP32[$pLItem>>2] = $577;
   $578 = HEAP32[$pLItem>>2]|0;
   $579 = HEAP32[$578>>2]|0;
   HEAP32[$pRight>>2] = $579;
   $580 = HEAP32[$1>>2]|0;
   $581 = HEAP32[$regFree2>>2]|0;
   _sqlite3ReleaseTempReg($580,$581);
   $582 = HEAP32[$1>>2]|0;
   $583 = HEAP32[$pRight>>2]|0;
   $584 = (_sqlite3ExprCodeTemp($582,$583,$regFree2)|0);
   HEAP32[$r2>>2] = $584;
   $585 = HEAP32[$1>>2]|0;
   $586 = HEAP32[$pLeft1>>2]|0;
   $587 = HEAP32[$pRight>>2]|0;
   $588 = HEAP32[$r1>>2]|0;
   $589 = HEAP32[$r2>>2]|0;
   $590 = HEAP32[$r4>>2]|0;
   (_codeCompare($585,$586,$587,81,$588,$589,$590,32)|0);
   $591 = HEAP32[$v>>2]|0;
   $592 = HEAP32[$r3>>2]|0;
   $593 = HEAP32[$r4>>2]|0;
   $594 = HEAP32[$3>>2]|0;
   (_sqlite3VdbeAddOp3($591,72,$592,$593,$594)|0);
   $595 = HEAP32[$1>>2]|0;
   $596 = HEAP32[$r3>>2]|0;
   _sqlite3ReleaseTempReg($595,$596);
   $597 = HEAP32[$1>>2]|0;
   $598 = HEAP32[$r4>>2]|0;
   _sqlite3ReleaseTempReg($597,$598);
   break;
  }
  case 62:  {
   $605 = HEAP32[$2>>2]|0;
   $606 = (($605) + 44|0);
   $607 = HEAP32[$606>>2]|0;
   HEAP32[$pTab>>2] = $607;
   $608 = HEAP32[$2>>2]|0;
   $609 = (($608) + 28|0);
   $610 = HEAP32[$609>>2]|0;
   $611 = HEAP32[$pTab>>2]|0;
   $612 = (($611) + 38|0);
   $613 = HEAP16[$612>>1]|0;
   $614 = $613 << 16 >> 16;
   $615 = (($614) + 1)|0;
   $616 = Math_imul($610, $615)|0;
   $617 = (($616) + 1)|0;
   $618 = HEAP32[$2>>2]|0;
   $619 = (($618) + 32|0);
   $620 = HEAP16[$619>>1]|0;
   $621 = $620 << 16 >> 16;
   $622 = (($617) + ($621))|0;
   HEAP32[$p1>>2] = $622;
   $623 = HEAP32[$v>>2]|0;
   $624 = HEAP32[$p1>>2]|0;
   $625 = HEAP32[$3>>2]|0;
   (_sqlite3VdbeAddOp2($623,132,$624,$625)|0);
   $626 = HEAP32[$2>>2]|0;
   $627 = (($626) + 32|0);
   $628 = HEAP16[$627>>1]|0;
   $629 = $628 << 16 >> 16;
   $630 = ($629|0)>=(0);
   if ($630) {
    $631 = HEAP32[$2>>2]|0;
    $632 = (($631) + 32|0);
    $633 = HEAP16[$632>>1]|0;
    $634 = $633 << 16 >> 16;
    $635 = HEAP32[$pTab>>2]|0;
    $636 = (($635) + 4|0);
    $637 = HEAP32[$636>>2]|0;
    $638 = (($637) + (($634*24)|0)|0);
    $639 = (($638) + 21|0);
    $640 = HEAP8[$639>>0]|0;
    $641 = $640 << 24 >> 24;
    $642 = ($641|0)==(69);
    if ($642) {
     $643 = HEAP32[$v>>2]|0;
     $644 = HEAP32[$3>>2]|0;
     (_sqlite3VdbeAddOp1($643,39,$644)|0);
    }
   }
   break;
  }
  case 134:  {
   $96 = HEAP32[$2>>2]|0;
   $97 = (($96) + 8|0);
   $98 = HEAP32[$97>>2]|0;
   $99 = (($98) + 2|0);
   HEAP32[$z>>2] = $99;
   $100 = HEAP32[$z>>2]|0;
   $101 = (_sqlite3Strlen30($100)|0);
   $102 = (($101) - 1)|0;
   HEAP32[$n>>2] = $102;
   $103 = HEAP32[$v>>2]|0;
   $104 = (_sqlite3VdbeDb($103)|0);
   $105 = HEAP32[$z>>2]|0;
   $106 = HEAP32[$n>>2]|0;
   $107 = (_sqlite3HexToBlob($104,$105,$106)|0);
   HEAP32[$zBlob>>2] = $107;
   $108 = HEAP32[$v>>2]|0;
   $109 = HEAP32[$n>>2]|0;
   $110 = (($109|0) / 2)&-1;
   $111 = HEAP32[$3>>2]|0;
   $112 = HEAP32[$zBlob>>2]|0;
   (_sqlite3VdbeAddOp4($108,30,$110,$111,0,$112,-1)|0);
   break;
  }
  case 57:  {
   $713 = HEAP32[$1>>2]|0;
   $714 = (($713) + 412|0);
   $715 = HEAP32[$714>>2]|0;
   $716 = ($715|0)!=(0|0);
   if (!($716)) {
    $717 = HEAP32[$1>>2]|0;
    _sqlite3ErrorMsg($717,19192,$vararg_buffer8);
    HEAP32[$0>>2] = 0;
    $746 = HEAP32[$0>>2]|0;
    STACKTOP = sp;return ($746|0);
   }
   $718 = HEAP32[$2>>2]|0;
   $719 = (($718) + 1|0);
   $720 = HEAP8[$719>>0]|0;
   $721 = $720 << 24 >> 24;
   $722 = ($721|0)==(2);
   if ($722) {
    $723 = HEAP32[$1>>2]|0;
    _sqlite3MayAbort($723);
   }
   $724 = HEAP32[$2>>2]|0;
   $725 = (($724) + 1|0);
   $726 = HEAP8[$725>>0]|0;
   $727 = $726 << 24 >> 24;
   $728 = ($727|0)==(4);
   if ($728) {
    $729 = HEAP32[$v>>2]|0;
    $730 = HEAP32[$2>>2]|0;
    $731 = (($730) + 8|0);
    $732 = HEAP32[$731>>2]|0;
    (_sqlite3VdbeAddOp4($729,24,0,4,0,$732,0)|0);
    break L9;
   } else {
    $733 = HEAP32[$1>>2]|0;
    $734 = HEAP32[$2>>2]|0;
    $735 = (($734) + 1|0);
    $736 = HEAP8[$735>>0]|0;
    $737 = $736 << 24 >> 24;
    $738 = HEAP32[$2>>2]|0;
    $739 = (($738) + 8|0);
    $740 = HEAP32[$739>>2]|0;
    _sqlite3HaltConstraint($733,1811,$737,$740,0,0);
    break L9;
   }
   break;
  }
  default: {
   HEAP32[$pTest>>2] = 0;
   $645 = HEAP32[$2>>2]|0;
   $646 = (($645) + 20|0);
   $647 = HEAP32[$646>>2]|0;
   HEAP32[$pEList>>2] = $647;
   $648 = HEAP32[$pEList>>2]|0;
   $649 = (($648) + 4|0);
   $650 = HEAP32[$649>>2]|0;
   HEAP32[$aListelem>>2] = $650;
   $651 = HEAP32[$pEList>>2]|0;
   $652 = HEAP32[$651>>2]|0;
   HEAP32[$nExpr>>2] = $652;
   $653 = HEAP32[$v>>2]|0;
   $654 = (_sqlite3VdbeMakeLabel($653)|0);
   HEAP32[$endLabel>>2] = $654;
   $655 = HEAP32[$2>>2]|0;
   $656 = (($655) + 12|0);
   $657 = HEAP32[$656>>2]|0;
   HEAP32[$pX>>2] = $657;
   $658 = ($657|0)!=(0|0);
   if ($658) {
    $659 = HEAP32[$pX>>2]|0;
    dest=$tempX+0|0; src=$659+0|0; stop=dest+48|0; do { HEAP32[dest>>2]=HEAP32[src>>2]|0; dest=dest+4|0; src=src+4|0; } while ((dest|0) < (stop|0));
    $660 = HEAP32[$1>>2]|0;
    $661 = HEAP32[$pX>>2]|0;
    $662 = (_sqlite3ExprCodeTemp($660,$661,$regFree1)|0);
    _exprToRegister($tempX,$662);
    HEAP8[$opCompare>>0] = 79;
    $663 = (($opCompare) + 12|0);
    HEAP32[$663>>2] = $tempX;
    HEAP32[$pTest>>2] = $opCompare;
    HEAP32[$regFree1>>2] = 0;
   }
   HEAP32[$i2>>2] = 0;
   while(1) {
    $664 = HEAP32[$i2>>2]|0;
    $665 = HEAP32[$nExpr>>2]|0;
    $666 = (($665) - 1)|0;
    $667 = ($664|0)<($666|0);
    if (!($667)) {
     break;
    }
    $668 = HEAP32[$1>>2]|0;
    _sqlite3ExprCachePush($668);
    $669 = HEAP32[$pX>>2]|0;
    $670 = ($669|0)!=(0|0);
    $671 = HEAP32[$i2>>2]|0;
    $672 = HEAP32[$aListelem>>2]|0;
    $673 = (($672) + (($671*20)|0)|0);
    $674 = HEAP32[$673>>2]|0;
    if ($670) {
     $675 = (($opCompare) + 16|0);
     HEAP32[$675>>2] = $674;
    } else {
     HEAP32[$pTest>>2] = $674;
    }
    $676 = HEAP32[$v>>2]|0;
    $677 = (_sqlite3VdbeMakeLabel($676)|0);
    HEAP32[$nextCase>>2] = $677;
    $678 = HEAP32[$1>>2]|0;
    $679 = HEAP32[$pTest>>2]|0;
    $680 = HEAP32[$nextCase>>2]|0;
    _sqlite3ExprIfFalse($678,$679,$680,16);
    $681 = HEAP32[$1>>2]|0;
    $682 = HEAP32[$i2>>2]|0;
    $683 = (($682) + 1)|0;
    $684 = HEAP32[$aListelem>>2]|0;
    $685 = (($684) + (($683*20)|0)|0);
    $686 = HEAP32[$685>>2]|0;
    $687 = HEAP32[$3>>2]|0;
    _sqlite3ExprCode($681,$686,$687);
    $688 = HEAP32[$v>>2]|0;
    $689 = HEAP32[$endLabel>>2]|0;
    (_sqlite3VdbeAddOp2($688,16,0,$689)|0);
    $690 = HEAP32[$1>>2]|0;
    _sqlite3ExprCachePop($690);
    $691 = HEAP32[$v>>2]|0;
    $692 = HEAP32[$nextCase>>2]|0;
    _sqlite3VdbeResolveLabel($691,$692);
    $693 = HEAP32[$i2>>2]|0;
    $694 = (($693) + 2)|0;
    HEAP32[$i2>>2] = $694;
   }
   $695 = HEAP32[$nExpr>>2]|0;
   $696 = $695 & 1;
   $697 = ($696|0)!=(0);
   if ($697) {
    $698 = HEAP32[$1>>2]|0;
    _sqlite3ExprCachePush($698);
    $699 = HEAP32[$1>>2]|0;
    $700 = HEAP32[$nExpr>>2]|0;
    $701 = (($700) - 1)|0;
    $702 = HEAP32[$pEList>>2]|0;
    $703 = (($702) + 4|0);
    $704 = HEAP32[$703>>2]|0;
    $705 = (($704) + (($701*20)|0)|0);
    $706 = HEAP32[$705>>2]|0;
    $707 = HEAP32[$3>>2]|0;
    _sqlite3ExprCode($699,$706,$707);
    $708 = HEAP32[$1>>2]|0;
    _sqlite3ExprCachePop($708);
   } else {
    $709 = HEAP32[$v>>2]|0;
    $710 = HEAP32[$3>>2]|0;
    (_sqlite3VdbeAddOp2($709,28,0,$710)|0);
   }
   $711 = HEAP32[$v>>2]|0;
   $712 = HEAP32[$endLabel>>2]|0;
   _sqlite3VdbeResolveLabel($711,$712);
  }
  }
 } while(0);
 L140: do {
  if ((label|0) == 11) {
   $47 = HEAP32[$2>>2]|0;
   $48 = (($47) + 28|0);
   $49 = HEAP32[$48>>2]|0;
   HEAP32[$iTab>>2] = $49;
   $50 = HEAP32[$iTab>>2]|0;
   $51 = ($50|0)<(0);
   do {
    if ($51) {
     $52 = HEAP32[$1>>2]|0;
     $53 = (($52) + 92|0);
     $54 = HEAP32[$53>>2]|0;
     $55 = ($54|0)>(0);
     if ($55) {
      $56 = HEAP32[$2>>2]|0;
      $57 = (($56) + 32|0);
      $58 = HEAP16[$57>>1]|0;
      $59 = $58 << 16 >> 16;
      $60 = HEAP32[$1>>2]|0;
      $61 = (($60) + 92|0);
      $62 = HEAP32[$61>>2]|0;
      $63 = (($59) + ($62))|0;
      HEAP32[$inReg>>2] = $63;
      break L140;
     } else {
      $64 = HEAP32[$1>>2]|0;
      $65 = (($64) + 96|0);
      $66 = HEAP32[$65>>2]|0;
      HEAP32[$iTab>>2] = $66;
      break;
     }
    }
   } while(0);
   $67 = HEAP32[$1>>2]|0;
   $68 = HEAP32[$2>>2]|0;
   $69 = (($68) + 44|0);
   $70 = HEAP32[$69>>2]|0;
   $71 = HEAP32[$2>>2]|0;
   $72 = (($71) + 32|0);
   $73 = HEAP16[$72>>1]|0;
   $74 = $73 << 16 >> 16;
   $75 = HEAP32[$iTab>>2]|0;
   $76 = HEAP32[$3>>2]|0;
   $77 = HEAP32[$2>>2]|0;
   $78 = (($77) + 38|0);
   $79 = HEAP8[$78>>0]|0;
   $80 = (_sqlite3ExprCodeGetColumn($67,$70,$74,$75,$76,$79)|0);
   HEAP32[$inReg>>2] = $80;
  }
 } while(0);
 $741 = HEAP32[$1>>2]|0;
 $742 = HEAP32[$regFree1>>2]|0;
 _sqlite3ReleaseTempReg($741,$742);
 $743 = HEAP32[$1>>2]|0;
 $744 = HEAP32[$regFree2>>2]|0;
 _sqlite3ReleaseTempReg($743,$744);
 $745 = HEAP32[$inReg>>2]|0;
 HEAP32[$0>>2] = $745;
 $746 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($746|0);
}
function _sqlite3ExprCodeGetColumn($pParse,$pTab,$iColumn,$iTable,$iReg,$p5) {
 $pParse = $pParse|0;
 $pTab = $pTab|0;
 $iColumn = $iColumn|0;
 $iTable = $iTable|0;
 $iReg = $iReg|0;
 $p5 = $p5|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0;
 var $i = 0, $p = 0, $v = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $0 = sp + 32|0;
 $1 = sp + 28|0;
 $2 = sp + 24|0;
 $3 = sp + 20|0;
 $4 = sp + 16|0;
 $5 = sp + 12|0;
 $6 = sp + 36|0;
 $v = sp + 8|0;
 $i = sp + 4|0;
 $p = sp;
 HEAP32[$1>>2] = $pParse;
 HEAP32[$2>>2] = $pTab;
 HEAP32[$3>>2] = $iColumn;
 HEAP32[$4>>2] = $iTable;
 HEAP32[$5>>2] = $iReg;
 HEAP8[$6>>0] = $p5;
 $7 = HEAP32[$1>>2]|0;
 $8 = (($7) + 8|0);
 $9 = HEAP32[$8>>2]|0;
 HEAP32[$v>>2] = $9;
 HEAP32[$i>>2] = 0;
 $10 = HEAP32[$1>>2]|0;
 $11 = (($10) + 116|0);
 HEAP32[$p>>2] = $11;
 while(1) {
  $12 = HEAP32[$i>>2]|0;
  $13 = ($12|0)<(10);
  if (!($13)) {
   break;
  }
  $14 = HEAP32[$p>>2]|0;
  $15 = (($14) + 12|0);
  $16 = HEAP32[$15>>2]|0;
  $17 = ($16|0)>(0);
  if ($17) {
   $18 = HEAP32[$p>>2]|0;
   $19 = HEAP32[$18>>2]|0;
   $20 = HEAP32[$4>>2]|0;
   $21 = ($19|0)==($20|0);
   if ($21) {
    $22 = HEAP32[$p>>2]|0;
    $23 = (($22) + 4|0);
    $24 = HEAP16[$23>>1]|0;
    $25 = $24 << 16 >> 16;
    $26 = HEAP32[$3>>2]|0;
    $27 = ($25|0)==($26|0);
    if ($27) {
     label = 6;
     break;
    }
   }
  }
  $41 = HEAP32[$i>>2]|0;
  $42 = (($41) + 1)|0;
  HEAP32[$i>>2] = $42;
  $43 = HEAP32[$p>>2]|0;
  $44 = (($43) + 20|0);
  HEAP32[$p>>2] = $44;
 }
 if ((label|0) == 6) {
  $28 = HEAP32[$1>>2]|0;
  $29 = (($28) + 104|0);
  $30 = HEAP32[$29>>2]|0;
  $31 = (($30) + 1)|0;
  HEAP32[$29>>2] = $31;
  $32 = HEAP32[$p>>2]|0;
  $33 = (($32) + 16|0);
  HEAP32[$33>>2] = $30;
  $34 = HEAP32[$1>>2]|0;
  $35 = HEAP32[$p>>2]|0;
  $36 = (($35) + 12|0);
  $37 = HEAP32[$36>>2]|0;
  _sqlite3ExprCachePinRegister($34,$37);
  $38 = HEAP32[$p>>2]|0;
  $39 = (($38) + 12|0);
  $40 = HEAP32[$39>>2]|0;
  HEAP32[$0>>2] = $40;
  $59 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($59|0);
 }
 $45 = HEAP32[$v>>2]|0;
 $46 = HEAP32[$2>>2]|0;
 $47 = HEAP32[$4>>2]|0;
 $48 = HEAP32[$3>>2]|0;
 $49 = HEAP32[$5>>2]|0;
 _sqlite3ExprCodeGetColumnOfTable($45,$46,$47,$48,$49);
 $50 = HEAP8[$6>>0]|0;
 $51 = ($50<<24>>24)!=(0);
 if ($51) {
  $52 = HEAP32[$v>>2]|0;
  $53 = HEAP8[$6>>0]|0;
  _sqlite3VdbeChangeP5($52,$53);
 } else {
  $54 = HEAP32[$1>>2]|0;
  $55 = HEAP32[$4>>2]|0;
  $56 = HEAP32[$3>>2]|0;
  $57 = HEAP32[$5>>2]|0;
  _sqlite3ExprCacheStore($54,$55,$56,$57);
 }
 $58 = HEAP32[$5>>2]|0;
 HEAP32[$0>>2] = $58;
 $59 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($59|0);
}
function _codeInteger($pParse,$pExpr,$negFlag,$iMem) {
 $pParse = $pParse|0;
 $pExpr = $pExpr|0;
 $negFlag = $negFlag|0;
 $iMem = $iMem|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $7 = 0, $8 = 0, $9 = 0, $c = 0, $i = 0, $or$cond = 0, $v = 0, $value = 0, $vararg_buffer = 0, $z = 0, $zV = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $vararg_buffer = sp + 8|0;
 $0 = sp + 44|0;
 $1 = sp + 40|0;
 $2 = sp + 36|0;
 $3 = sp + 32|0;
 $v = sp + 28|0;
 $i = sp + 24|0;
 $c = sp + 20|0;
 $value = sp;
 $z = sp + 16|0;
 $zV = sp + 12|0;
 HEAP32[$0>>2] = $pParse;
 HEAP32[$1>>2] = $pExpr;
 HEAP32[$2>>2] = $negFlag;
 HEAP32[$3>>2] = $iMem;
 $4 = HEAP32[$0>>2]|0;
 $5 = (($4) + 8|0);
 $6 = HEAP32[$5>>2]|0;
 HEAP32[$v>>2] = $6;
 $7 = HEAP32[$1>>2]|0;
 $8 = (($7) + 4|0);
 $9 = HEAP32[$8>>2]|0;
 $10 = $9 & 1024;
 $11 = ($10|0)!=(0);
 $12 = HEAP32[$1>>2]|0;
 $13 = (($12) + 8|0);
 if ($11) {
  $14 = HEAP32[$13>>2]|0;
  HEAP32[$i>>2] = $14;
  $15 = HEAP32[$2>>2]|0;
  $16 = ($15|0)!=(0);
  if ($16) {
   $17 = HEAP32[$i>>2]|0;
   $18 = (0 - ($17))|0;
   HEAP32[$i>>2] = $18;
  }
  $19 = HEAP32[$v>>2]|0;
  $20 = HEAP32[$i>>2]|0;
  $21 = HEAP32[$3>>2]|0;
  (_sqlite3VdbeAddOp2($19,25,$20,$21)|0);
  STACKTOP = sp;return;
 }
 $22 = HEAP32[$13>>2]|0;
 HEAP32[$z>>2] = $22;
 $23 = HEAP32[$z>>2]|0;
 $24 = (_sqlite3DecOrHexToI64($23,$value)|0);
 HEAP32[$c>>2] = $24;
 $25 = HEAP32[$c>>2]|0;
 $26 = ($25|0)==(0);
 if (!($26)) {
  $27 = HEAP32[$c>>2]|0;
  $28 = ($27|0)==(2);
  $29 = HEAP32[$2>>2]|0;
  $30 = ($29|0)!=(0);
  $or$cond = $28 & $30;
  if (!($or$cond)) {
   $54 = HEAP32[$z>>2]|0;
   $55 = (_sqlite3_strnicmp($54,19336,2)|0);
   $56 = ($55|0)==(0);
   if ($56) {
    $57 = HEAP32[$0>>2]|0;
    $58 = HEAP32[$z>>2]|0;
    HEAP32[$vararg_buffer>>2] = $58;
    _sqlite3ErrorMsg($57,19344,$vararg_buffer);
    STACKTOP = sp;return;
   } else {
    $59 = HEAP32[$v>>2]|0;
    $60 = HEAP32[$z>>2]|0;
    $61 = HEAP32[$2>>2]|0;
    $62 = HEAP32[$3>>2]|0;
    _codeReal($59,$60,$61,$62);
    STACKTOP = sp;return;
   }
  }
 }
 $31 = HEAP32[$2>>2]|0;
 $32 = ($31|0)!=(0);
 if ($32) {
  $33 = HEAP32[$c>>2]|0;
  $34 = ($33|0)==(2);
  if ($34) {
   $45 = 0;$48 = -2147483648;
  } else {
   $35 = $value;
   $36 = $35;
   $37 = HEAP32[$36>>2]|0;
   $38 = (($35) + 4)|0;
   $39 = $38;
   $40 = HEAP32[$39>>2]|0;
   $41 = (_i64Subtract(0,0,($37|0),($40|0))|0);
   $42 = tempRet0;
   $45 = $41;$48 = $42;
  }
  $43 = $value;
  $44 = $43;
  HEAP32[$44>>2] = $45;
  $46 = (($43) + 4)|0;
  $47 = $46;
  HEAP32[$47>>2] = $48;
 }
 $49 = HEAP32[$v>>2]|0;
 $50 = (_dup8bytes($49,$value)|0);
 HEAP32[$zV>>2] = $50;
 $51 = HEAP32[$v>>2]|0;
 $52 = HEAP32[$3>>2]|0;
 $53 = HEAP32[$zV>>2]|0;
 (_sqlite3VdbeAddOp4($51,26,0,$52,0,$53,-13)|0);
 STACKTOP = sp;return;
}
function _codeReal($v,$z,$negateFlag,$iMem) {
 $v = $v|0;
 $z = $z|0;
 $negateFlag = $negateFlag|0;
 $iMem = $iMem|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0.0, $12 = 0.0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $value = 0, $zV = 0;
 var label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 24|0;
 $1 = sp + 20|0;
 $2 = sp + 16|0;
 $3 = sp + 12|0;
 $value = sp;
 $zV = sp + 8|0;
 HEAP32[$0>>2] = $v;
 HEAP32[$1>>2] = $z;
 HEAP32[$2>>2] = $negateFlag;
 HEAP32[$3>>2] = $iMem;
 $4 = HEAP32[$1>>2]|0;
 $5 = ($4|0)!=(0|0);
 if (!($5)) {
  STACKTOP = sp;return;
 }
 $6 = HEAP32[$1>>2]|0;
 $7 = HEAP32[$1>>2]|0;
 $8 = (_sqlite3Strlen30($7)|0);
 (_sqlite3AtoF($6,$value,$8,1)|0);
 $9 = HEAP32[$2>>2]|0;
 $10 = ($9|0)!=(0);
 if ($10) {
  $11 = +HEAPF64[$value>>3];
  $12 = -$11;
  HEAPF64[$value>>3] = $12;
 }
 $13 = HEAP32[$0>>2]|0;
 $14 = (_dup8bytes($13,$value)|0);
 HEAP32[$zV>>2] = $14;
 $15 = HEAP32[$0>>2]|0;
 $16 = HEAP32[$3>>2]|0;
 $17 = HEAP32[$zV>>2]|0;
 (_sqlite3VdbeAddOp4($15,133,0,$16,0,$17,-12)|0);
 STACKTOP = sp;return;
}
function _sqlite3ExprCacheAffinityChange($pParse,$iStart,$iCount) {
 $pParse = $pParse|0;
 $iStart = $iStart|0;
 $iCount = $iCount|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $1 = sp + 4|0;
 $2 = sp;
 HEAP32[$0>>2] = $pParse;
 HEAP32[$1>>2] = $iStart;
 HEAP32[$2>>2] = $iCount;
 $3 = HEAP32[$0>>2]|0;
 $4 = HEAP32[$1>>2]|0;
 $5 = HEAP32[$2>>2]|0;
 _sqlite3ExprCacheRemove($3,$4,$5);
 STACKTOP = sp;return;
}
function _sqlite3ExprIsConstant($p) {
 $p = $p|0;
 var $0 = 0, $1 = 0, $2 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 HEAP32[$0>>2] = $p;
 $1 = HEAP32[$0>>2]|0;
 $2 = (_exprIsConst($1,1)|0);
 STACKTOP = sp;return ($2|0);
}
function _sqlite3ExprCodeExprList($pParse,$pList,$target,$flags) {
 $pParse = $pParse|0;
 $pList = $pList|0;
 $target = $target|0;
 $flags = $flags|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0;
 var $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0;
 var $copyOp = 0, $i = 0, $inReg = 0, $n = 0, $pExpr = 0, $pItem = 0, $pOp = 0, $v = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $0 = sp + 36|0;
 $1 = sp + 32|0;
 $2 = sp + 28|0;
 $3 = sp + 41|0;
 $pItem = sp + 24|0;
 $i = sp + 20|0;
 $n = sp + 16|0;
 $copyOp = sp + 40|0;
 $pExpr = sp + 12|0;
 $inReg = sp + 8|0;
 $pOp = sp + 4|0;
 $v = sp;
 HEAP32[$0>>2] = $pParse;
 HEAP32[$1>>2] = $pList;
 HEAP32[$2>>2] = $target;
 HEAP8[$3>>0] = $flags;
 $4 = HEAP8[$3>>0]|0;
 $5 = $4&255;
 $6 = $5 & 1;
 $7 = ($6|0)!=(0);
 $8 = $7 ? 33 : 34;
 $9 = $8&255;
 HEAP8[$copyOp>>0] = $9;
 $10 = HEAP32[$1>>2]|0;
 $11 = HEAP32[$10>>2]|0;
 HEAP32[$n>>2] = $11;
 $12 = HEAP32[$0>>2]|0;
 $13 = (($12) + 23|0);
 $14 = HEAP8[$13>>0]|0;
 $15 = ($14<<24>>24)!=(0);
 if (!($15)) {
  $16 = HEAP8[$3>>0]|0;
  $17 = $16&255;
  $18 = $17 & -3;
  $19 = $18&255;
  HEAP8[$3>>0] = $19;
 }
 $20 = HEAP32[$1>>2]|0;
 $21 = (($20) + 4|0);
 $22 = HEAP32[$21>>2]|0;
 HEAP32[$pItem>>2] = $22;
 HEAP32[$i>>2] = 0;
 while(1) {
  $23 = HEAP32[$i>>2]|0;
  $24 = HEAP32[$n>>2]|0;
  $25 = ($23|0)<($24|0);
  if (!($25)) {
   break;
  }
  $26 = HEAP32[$pItem>>2]|0;
  $27 = HEAP32[$26>>2]|0;
  HEAP32[$pExpr>>2] = $27;
  $28 = HEAP8[$3>>0]|0;
  $29 = $28&255;
  $30 = $29 & 2;
  $31 = ($30|0)!=(0);
  if ($31) {
   $32 = HEAP32[$pExpr>>2]|0;
   $33 = (_sqlite3ExprIsConstant($32)|0);
   $34 = ($33|0)!=(0);
   if ($34) {
    $35 = HEAP32[$0>>2]|0;
    $36 = HEAP32[$pExpr>>2]|0;
    $37 = HEAP32[$2>>2]|0;
    $38 = HEAP32[$i>>2]|0;
    $39 = (($37) + ($38))|0;
    _sqlite3ExprCodeAtInit($35,$36,$39,0);
   } else {
    label = 8;
   }
  } else {
   label = 8;
  }
  do {
   if ((label|0) == 8) {
    label = 0;
    $40 = HEAP32[$0>>2]|0;
    $41 = HEAP32[$pExpr>>2]|0;
    $42 = HEAP32[$2>>2]|0;
    $43 = HEAP32[$i>>2]|0;
    $44 = (($42) + ($43))|0;
    $45 = (_sqlite3ExprCodeTarget($40,$41,$44)|0);
    HEAP32[$inReg>>2] = $45;
    $46 = HEAP32[$inReg>>2]|0;
    $47 = HEAP32[$2>>2]|0;
    $48 = HEAP32[$i>>2]|0;
    $49 = (($47) + ($48))|0;
    $50 = ($46|0)!=($49|0);
    if ($50) {
     $51 = HEAP32[$0>>2]|0;
     $52 = (($51) + 8|0);
     $53 = HEAP32[$52>>2]|0;
     HEAP32[$v>>2] = $53;
     $54 = HEAP8[$copyOp>>0]|0;
     $55 = $54&255;
     $56 = ($55|0)==(33);
     if ($56) {
      $57 = HEAP32[$v>>2]|0;
      $58 = (_sqlite3VdbeGetOp($57,-1)|0);
      HEAP32[$pOp>>2] = $58;
      $59 = HEAP8[$58>>0]|0;
      $60 = $59&255;
      $61 = ($60|0)==(33);
      if ($61) {
       $62 = HEAP32[$pOp>>2]|0;
       $63 = (($62) + 4|0);
       $64 = HEAP32[$63>>2]|0;
       $65 = HEAP32[$pOp>>2]|0;
       $66 = (($65) + 12|0);
       $67 = HEAP32[$66>>2]|0;
       $68 = (($64) + ($67))|0;
       $69 = (($68) + 1)|0;
       $70 = HEAP32[$inReg>>2]|0;
       $71 = ($69|0)==($70|0);
       if ($71) {
        $72 = HEAP32[$pOp>>2]|0;
        $73 = (($72) + 8|0);
        $74 = HEAP32[$73>>2]|0;
        $75 = HEAP32[$pOp>>2]|0;
        $76 = (($75) + 12|0);
        $77 = HEAP32[$76>>2]|0;
        $78 = (($74) + ($77))|0;
        $79 = (($78) + 1)|0;
        $80 = HEAP32[$2>>2]|0;
        $81 = HEAP32[$i>>2]|0;
        $82 = (($80) + ($81))|0;
        $83 = ($79|0)==($82|0);
        if ($83) {
         $84 = HEAP32[$pOp>>2]|0;
         $85 = (($84) + 12|0);
         $86 = HEAP32[$85>>2]|0;
         $87 = (($86) + 1)|0;
         HEAP32[$85>>2] = $87;
         break;
        }
       }
      }
     }
     $88 = HEAP32[$v>>2]|0;
     $89 = HEAP8[$copyOp>>0]|0;
     $90 = $89&255;
     $91 = HEAP32[$inReg>>2]|0;
     $92 = HEAP32[$2>>2]|0;
     $93 = HEAP32[$i>>2]|0;
     $94 = (($92) + ($93))|0;
     (_sqlite3VdbeAddOp2($88,$90,$91,$94)|0);
    }
   }
  } while(0);
  $95 = HEAP32[$i>>2]|0;
  $96 = (($95) + 1)|0;
  HEAP32[$i>>2] = $96;
  $97 = HEAP32[$pItem>>2]|0;
  $98 = (($97) + 20|0);
  HEAP32[$pItem>>2] = $98;
 }
 $99 = HEAP32[$n>>2]|0;
 STACKTOP = sp;return ($99|0);
}
function _sqlite3VtabOverloadFunction($db,$pDef,$nArg,$pExpr) {
 $db = $db|0;
 $pDef = $pDef|0;
 $nArg = $nArg|0;
 $pExpr = $pExpr|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $12 = 0, $13 = 0, $14 = 0;
 var $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0;
 var $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0;
 var $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0;
 var $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0;
 var $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $pArg = 0, $pMod = 0, $pNew = 0, $pTab = 0, $pVtab = 0, $rc = 0, $xFunc = 0;
 var $z = 0, $zLowerName = 0, dest = 0, label = 0, sp = 0, src = 0, stop = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 64|0;
 $0 = sp + 56|0;
 $1 = sp + 52|0;
 $2 = sp + 48|0;
 $3 = sp + 44|0;
 $4 = sp + 40|0;
 $pTab = sp + 36|0;
 $pVtab = sp + 32|0;
 $pMod = sp + 28|0;
 $xFunc = sp + 24|0;
 $pArg = sp + 20|0;
 $pNew = sp + 16|0;
 $rc = sp + 12|0;
 $zLowerName = sp + 8|0;
 $z = sp + 4|0;
 $5 = sp;
 HEAP32[$1>>2] = $db;
 HEAP32[$2>>2] = $pDef;
 HEAP32[$3>>2] = $nArg;
 HEAP32[$4>>2] = $pExpr;
 HEAP32[$xFunc>>2] = 0;
 HEAP32[$pArg>>2] = 0;
 HEAP32[$rc>>2] = 0;
 $6 = HEAP32[$4>>2]|0;
 $7 = ($6|0)==(0|0);
 if ($7) {
  $8 = HEAP32[$2>>2]|0;
  HEAP32[$0>>2] = $8;
  $112 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($112|0);
 }
 $9 = HEAP32[$4>>2]|0;
 $10 = HEAP8[$9>>0]|0;
 $11 = $10&255;
 $12 = ($11|0)!=(154);
 if ($12) {
  $13 = HEAP32[$2>>2]|0;
  HEAP32[$0>>2] = $13;
  $112 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($112|0);
 }
 $14 = HEAP32[$4>>2]|0;
 $15 = (($14) + 44|0);
 $16 = HEAP32[$15>>2]|0;
 HEAP32[$pTab>>2] = $16;
 $17 = HEAP32[$pTab>>2]|0;
 $18 = ($17|0)==(0|0);
 if ($18) {
  $19 = HEAP32[$2>>2]|0;
  HEAP32[$0>>2] = $19;
  $112 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($112|0);
 }
 $20 = HEAP32[$pTab>>2]|0;
 $21 = (($20) + 44|0);
 $22 = HEAP8[$21>>0]|0;
 $23 = $22&255;
 $24 = $23 & 16;
 $25 = ($24|0)==(0);
 if ($25) {
  $26 = HEAP32[$2>>2]|0;
  HEAP32[$0>>2] = $26;
  $112 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($112|0);
 }
 $27 = HEAP32[$1>>2]|0;
 $28 = HEAP32[$pTab>>2]|0;
 $29 = (_sqlite3GetVTable($27,$28)|0);
 $30 = (($29) + 8|0);
 $31 = HEAP32[$30>>2]|0;
 HEAP32[$pVtab>>2] = $31;
 $32 = HEAP32[$pVtab>>2]|0;
 $33 = HEAP32[$32>>2]|0;
 HEAP32[$pMod>>2] = $33;
 $34 = HEAP32[$pMod>>2]|0;
 $35 = (($34) + 72|0);
 $36 = HEAP32[$35>>2]|0;
 $37 = ($36|0)==(0|0);
 if ($37) {
  $38 = HEAP32[$2>>2]|0;
  HEAP32[$0>>2] = $38;
  $112 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($112|0);
 }
 $39 = HEAP32[$1>>2]|0;
 $40 = HEAP32[$2>>2]|0;
 $41 = (($40) + 24|0);
 $42 = HEAP32[$41>>2]|0;
 $43 = (_sqlite3DbStrDup($39,$42)|0);
 HEAP32[$zLowerName>>2] = $43;
 $44 = HEAP32[$zLowerName>>2]|0;
 $45 = ($44|0)!=(0|0);
 if ($45) {
  $46 = HEAP32[$zLowerName>>2]|0;
  HEAP32[$z>>2] = $46;
  while(1) {
   $47 = HEAP32[$z>>2]|0;
   $48 = HEAP8[$47>>0]|0;
   $49 = ($48<<24>>24)!=(0);
   if (!($49)) {
    break;
   }
   $50 = HEAP32[$z>>2]|0;
   $51 = HEAP8[$50>>0]|0;
   $52 = $51&255;
   $53 = (440 + ($52)|0);
   $54 = HEAP8[$53>>0]|0;
   $55 = HEAP32[$z>>2]|0;
   HEAP8[$55>>0] = $54;
   $56 = HEAP32[$z>>2]|0;
   $57 = (($56) + 1|0);
   HEAP32[$z>>2] = $57;
  }
  $58 = HEAP32[$pMod>>2]|0;
  $59 = (($58) + 72|0);
  $60 = HEAP32[$59>>2]|0;
  $61 = HEAP32[$pVtab>>2]|0;
  $62 = HEAP32[$3>>2]|0;
  $63 = HEAP32[$zLowerName>>2]|0;
  HEAP32[$5>>2] = $xFunc;
  $64 = HEAP32[$5>>2]|0;
  $65 = (FUNCTION_TABLE_iiiiii[$60 & 15]($61,$62,$63,$64,$pArg)|0);
  HEAP32[$rc>>2] = $65;
  $66 = HEAP32[$1>>2]|0;
  $67 = HEAP32[$zLowerName>>2]|0;
  _sqlite3DbFree($66,$67);
 }
 $68 = HEAP32[$rc>>2]|0;
 $69 = ($68|0)==(0);
 if ($69) {
  $70 = HEAP32[$2>>2]|0;
  HEAP32[$0>>2] = $70;
  $112 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($112|0);
 }
 $71 = HEAP32[$1>>2]|0;
 $72 = HEAP32[$2>>2]|0;
 $73 = (($72) + 24|0);
 $74 = HEAP32[$73>>2]|0;
 $75 = (_sqlite3Strlen30($74)|0);
 $76 = (36 + ($75))|0;
 $77 = (($76) + 1)|0;
 $78 = (_sqlite3DbMallocZero($71,$77,0)|0);
 HEAP32[$pNew>>2] = $78;
 $79 = HEAP32[$pNew>>2]|0;
 $80 = ($79|0)==(0|0);
 if ($80) {
  $81 = HEAP32[$2>>2]|0;
  HEAP32[$0>>2] = $81;
  $112 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($112|0);
 } else {
  $82 = HEAP32[$pNew>>2]|0;
  $83 = HEAP32[$2>>2]|0;
  dest=$82+0|0; src=$83+0|0; stop=dest+36|0; do { HEAP32[dest>>2]=HEAP32[src>>2]|0; dest=dest+4|0; src=src+4|0; } while ((dest|0) < (stop|0));
  $84 = HEAP32[$pNew>>2]|0;
  $85 = (($84) + 36|0);
  $86 = HEAP32[$pNew>>2]|0;
  $87 = (($86) + 24|0);
  HEAP32[$87>>2] = $85;
  $88 = HEAP32[$pNew>>2]|0;
  $89 = (($88) + 24|0);
  $90 = HEAP32[$89>>2]|0;
  $91 = HEAP32[$2>>2]|0;
  $92 = (($91) + 24|0);
  $93 = HEAP32[$92>>2]|0;
  $94 = HEAP32[$2>>2]|0;
  $95 = (($94) + 24|0);
  $96 = HEAP32[$95>>2]|0;
  $97 = (_sqlite3Strlen30($96)|0);
  $98 = (($97) + 1)|0;
  _memcpy(($90|0),($93|0),($98|0))|0;
  $99 = HEAP32[$xFunc>>2]|0;
  $100 = HEAP32[$pNew>>2]|0;
  $101 = (($100) + 12|0);
  HEAP32[$101>>2] = $99;
  $102 = HEAP32[$pArg>>2]|0;
  $103 = HEAP32[$pNew>>2]|0;
  $104 = (($103) + 4|0);
  HEAP32[$104>>2] = $102;
  $105 = HEAP32[$pNew>>2]|0;
  $106 = (($105) + 2|0);
  $107 = HEAP16[$106>>1]|0;
  $108 = $107&65535;
  $109 = $108 | 16;
  $110 = $109&65535;
  HEAP16[$106>>1] = $110;
  $111 = HEAP32[$pNew>>2]|0;
  HEAP32[$0>>2] = $111;
  $112 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($112|0);
 }
 return 0|0;
}
function _sqlite3CodeSubselect($pParse,$pExpr,$rHasNullFlag,$isRowid) {
 $pParse = $pParse|0;
 $pExpr = $pExpr|0;
 $rHasNullFlag = $rHasNullFlag|0;
 $isRowid = $isRowid|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0;
 var $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0, $188 = 0;
 var $189 = 0, $19 = 0, $190 = 0, $191 = 0, $192 = 0, $193 = 0, $194 = 0, $195 = 0, $196 = 0, $197 = 0, $198 = 0, $199 = 0, $2 = 0, $20 = 0, $200 = 0, $201 = 0, $202 = 0, $203 = 0, $204 = 0, $205 = 0;
 var $206 = 0, $207 = 0, $208 = 0, $209 = 0, $21 = 0, $210 = 0, $211 = 0, $212 = 0, $213 = 0, $214 = 0, $215 = 0, $216 = 0, $217 = 0, $218 = 0, $219 = 0, $22 = 0, $220 = 0, $221 = 0, $222 = 0, $223 = 0;
 var $224 = 0, $225 = 0, $226 = 0, $227 = 0, $228 = 0, $229 = 0, $23 = 0, $230 = 0, $231 = 0, $232 = 0, $233 = 0, $234 = 0, $235 = 0, $236 = 0, $237 = 0, $238 = 0, $239 = 0, $24 = 0, $240 = 0, $241 = 0;
 var $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0;
 var $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0;
 var $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0;
 var $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0;
 var $98 = 0, $99 = 0, $addr = 0, $affinity = 0, $cond = 0, $dest = 0, $dest1 = 0, $i = 0, $iValToIns = 0, $jmpIfDynamic = 0, $pE2 = 0, $pEList = 0, $pItem = 0, $pKeyInfo = 0, $pLeft = 0, $pList = 0, $pSel = 0, $pSelect = 0, $r1 = 0, $r2 = 0;
 var $r3 = 0, $rReg = 0, $v = 0, $vararg_buffer = 0, $vararg_ptr1 = 0, $vararg_ptr2 = 0, $zMsg = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 160|0;
 $vararg_buffer = sp;
 $0 = sp + 140|0;
 $1 = sp + 136|0;
 $2 = sp + 132|0;
 $3 = sp + 128|0;
 $4 = sp + 124|0;
 $jmpIfDynamic = sp + 120|0;
 $rReg = sp + 116|0;
 $v = sp + 112|0;
 $zMsg = sp + 108|0;
 $affinity = sp + 144|0;
 $addr = sp + 104|0;
 $pLeft = sp + 100|0;
 $pKeyInfo = sp + 96|0;
 $pSelect = sp + 92|0;
 $dest = sp + 72|0;
 $pEList = sp + 68|0;
 $i = sp + 64|0;
 $pList = sp + 60|0;
 $pItem = sp + 56|0;
 $r1 = sp + 52|0;
 $r2 = sp + 48|0;
 $r3 = sp + 44|0;
 $pE2 = sp + 40|0;
 $iValToIns = sp + 36|0;
 $pSel = sp + 32|0;
 $dest1 = sp + 12|0;
 HEAP32[$1>>2] = $pParse;
 HEAP32[$2>>2] = $pExpr;
 HEAP32[$3>>2] = $rHasNullFlag;
 HEAP32[$4>>2] = $isRowid;
 HEAP32[$jmpIfDynamic>>2] = -1;
 HEAP32[$rReg>>2] = 0;
 $5 = HEAP32[$1>>2]|0;
 $6 = (_sqlite3GetVdbe($5)|0);
 HEAP32[$v>>2] = $6;
 $7 = HEAP32[$v>>2]|0;
 $8 = ($7|0)==(0|0);
 if ($8) {
  HEAP32[$0>>2] = 0;
  $241 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($241|0);
 }
 $9 = HEAP32[$1>>2]|0;
 _sqlite3ExprCachePush($9);
 $10 = HEAP32[$2>>2]|0;
 $11 = (($10) + 4|0);
 $12 = HEAP32[$11>>2]|0;
 $13 = $12 & 32;
 $14 = ($13|0)!=(0);
 if (!($14)) {
  $15 = HEAP32[$1>>2]|0;
  $16 = (_sqlite3CodeOnce($15)|0);
  HEAP32[$jmpIfDynamic>>2] = $16;
 }
 $17 = HEAP32[$1>>2]|0;
 $18 = (($17) + 450|0);
 $19 = HEAP8[$18>>0]|0;
 $20 = $19&255;
 $21 = ($20|0)==(2);
 if ($21) {
  $22 = HEAP32[$1>>2]|0;
  $23 = HEAP32[$22>>2]|0;
  $24 = HEAP32[$jmpIfDynamic>>2]|0;
  $25 = ($24|0)>=(0);
  $26 = $25 ? 1216 : 19280;
  $27 = HEAP32[$2>>2]|0;
  $28 = HEAP8[$27>>0]|0;
  $29 = $28&255;
  $30 = ($29|0)==(75);
  $31 = $30 ? 19296 : 19304;
  $32 = HEAP32[$1>>2]|0;
  $33 = (($32) + 468|0);
  $34 = HEAP32[$33>>2]|0;
  HEAP32[$vararg_buffer>>2] = $26;
  $vararg_ptr1 = (($vararg_buffer) + 4|0);
  HEAP32[$vararg_ptr1>>2] = $31;
  $vararg_ptr2 = (($vararg_buffer) + 8|0);
  HEAP32[$vararg_ptr2>>2] = $34;
  $35 = (_sqlite3MPrintf($23,19248,$vararg_buffer)|0);
  HEAP32[$zMsg>>2] = $35;
  $36 = HEAP32[$v>>2]|0;
  $37 = HEAP32[$1>>2]|0;
  $38 = (($37) + 464|0);
  $39 = HEAP32[$38>>2]|0;
  $40 = HEAP32[$zMsg>>2]|0;
  (_sqlite3VdbeAddOp4($36,155,$39,0,0,$40,-1)|0);
 }
 $41 = HEAP32[$2>>2]|0;
 $42 = HEAP8[$41>>0]|0;
 $43 = $42&255;
 $cond = ($43|0)==(75);
 $44 = HEAP32[$2>>2]|0;
 do {
  if ($cond) {
   $45 = (($44) + 12|0);
   $46 = HEAP32[$45>>2]|0;
   HEAP32[$pLeft>>2] = $46;
   HEAP32[$pKeyInfo>>2] = 0;
   $47 = HEAP32[$pLeft>>2]|0;
   $48 = (_sqlite3ExprAffinity($47)|0);
   HEAP8[$affinity>>0] = $48;
   $49 = HEAP32[$1>>2]|0;
   $50 = (($49) + 68|0);
   $51 = HEAP32[$50>>2]|0;
   $52 = (($51) + 1)|0;
   HEAP32[$50>>2] = $52;
   $53 = HEAP32[$2>>2]|0;
   $54 = (($53) + 28|0);
   HEAP32[$54>>2] = $51;
   $55 = HEAP32[$v>>2]|0;
   $56 = HEAP32[$2>>2]|0;
   $57 = (($56) + 28|0);
   $58 = HEAP32[$57>>2]|0;
   $59 = HEAP32[$4>>2]|0;
   $60 = ($59|0)!=(0);
   $61 = $60 ^ 1;
   $62 = $61&1;
   $63 = (_sqlite3VdbeAddOp2($55,57,$58,$62)|0);
   HEAP32[$addr>>2] = $63;
   $64 = HEAP32[$4>>2]|0;
   $65 = ($64|0)!=(0);
   if ($65) {
    $69 = 0;
   } else {
    $66 = HEAP32[$1>>2]|0;
    $67 = HEAP32[$66>>2]|0;
    $68 = (_sqlite3KeyInfoAlloc($67,1,1)|0);
    $69 = $68;
   }
   HEAP32[$pKeyInfo>>2] = $69;
   $70 = HEAP32[$2>>2]|0;
   $71 = (($70) + 4|0);
   $72 = HEAP32[$71>>2]|0;
   $73 = $72 & 2048;
   $74 = ($73|0)!=(0);
   $75 = HEAP32[$2>>2]|0;
   $76 = (($75) + 20|0);
   do {
    if ($74) {
     $77 = HEAP32[$76>>2]|0;
     HEAP32[$pSelect>>2] = $77;
     $78 = HEAP32[$2>>2]|0;
     $79 = (($78) + 28|0);
     $80 = HEAP32[$79>>2]|0;
     _sqlite3SelectDestInit($dest,11,$80);
     $81 = HEAP8[$affinity>>0]|0;
     $82 = (($dest) + 1|0);
     HEAP8[$82>>0] = $81;
     $83 = HEAP32[$pSelect>>2]|0;
     $84 = (($83) + 8|0);
     HEAP32[$84>>2] = 0;
     $85 = HEAP32[$1>>2]|0;
     $86 = HEAP32[$pSelect>>2]|0;
     $87 = (_sqlite3Select($85,$86,$dest)|0);
     $88 = ($87|0)!=(0);
     if (!($88)) {
      $90 = HEAP32[$pSelect>>2]|0;
      $91 = HEAP32[$90>>2]|0;
      HEAP32[$pEList>>2] = $91;
      $92 = HEAP32[$1>>2]|0;
      $93 = HEAP32[$2>>2]|0;
      $94 = (($93) + 12|0);
      $95 = HEAP32[$94>>2]|0;
      $96 = HEAP32[$pEList>>2]|0;
      $97 = (($96) + 4|0);
      $98 = HEAP32[$97>>2]|0;
      $99 = HEAP32[$98>>2]|0;
      $100 = (_sqlite3BinaryCompareCollSeq($92,$95,$99)|0);
      $101 = HEAP32[$pKeyInfo>>2]|0;
      $102 = (($101) + 20|0);
      HEAP32[$102>>2] = $100;
      break;
     }
     $89 = HEAP32[$pKeyInfo>>2]|0;
     _sqlite3KeyInfoUnref($89);
     HEAP32[$0>>2] = 0;
     $241 = HEAP32[$0>>2]|0;
     STACKTOP = sp;return ($241|0);
    } else {
     $103 = HEAP32[$76>>2]|0;
     $104 = ($103|0)!=(0|0);
     if ($104) {
      $105 = HEAP32[$2>>2]|0;
      $106 = (($105) + 20|0);
      $107 = HEAP32[$106>>2]|0;
      HEAP32[$pList>>2] = $107;
      $108 = HEAP8[$affinity>>0]|0;
      $109 = ($108<<24>>24)!=(0);
      if (!($109)) {
       HEAP8[$affinity>>0] = 65;
      }
      $110 = HEAP32[$pKeyInfo>>2]|0;
      $111 = ($110|0)!=(0|0);
      if ($111) {
       $112 = HEAP32[$1>>2]|0;
       $113 = HEAP32[$2>>2]|0;
       $114 = (($113) + 12|0);
       $115 = HEAP32[$114>>2]|0;
       $116 = (_sqlite3ExprCollSeq($112,$115)|0);
       $117 = HEAP32[$pKeyInfo>>2]|0;
       $118 = (($117) + 20|0);
       HEAP32[$118>>2] = $116;
      }
      $119 = HEAP32[$1>>2]|0;
      $120 = (_sqlite3GetTempReg($119)|0);
      HEAP32[$r1>>2] = $120;
      $121 = HEAP32[$1>>2]|0;
      $122 = (_sqlite3GetTempReg($121)|0);
      HEAP32[$r2>>2] = $122;
      $123 = HEAP32[$4>>2]|0;
      $124 = ($123|0)!=(0);
      if ($124) {
       $125 = HEAP32[$v>>2]|0;
       $126 = HEAP32[$r2>>2]|0;
       (_sqlite3VdbeAddOp2($125,28,0,$126)|0);
      }
      $127 = HEAP32[$pList>>2]|0;
      $128 = HEAP32[$127>>2]|0;
      HEAP32[$i>>2] = $128;
      $129 = HEAP32[$pList>>2]|0;
      $130 = (($129) + 4|0);
      $131 = HEAP32[$130>>2]|0;
      HEAP32[$pItem>>2] = $131;
      while(1) {
       $132 = HEAP32[$i>>2]|0;
       $133 = ($132|0)>(0);
       if (!($133)) {
        break;
       }
       $134 = HEAP32[$pItem>>2]|0;
       $135 = HEAP32[$134>>2]|0;
       HEAP32[$pE2>>2] = $135;
       $136 = HEAP32[$jmpIfDynamic>>2]|0;
       $137 = ($136|0)>=(0);
       if ($137) {
        $138 = HEAP32[$pE2>>2]|0;
        $139 = (_sqlite3ExprIsConstant($138)|0);
        $140 = ($139|0)!=(0);
        if (!($140)) {
         $141 = HEAP32[$v>>2]|0;
         $142 = HEAP32[$jmpIfDynamic>>2]|0;
         _sqlite3VdbeChangeToNoop($141,$142);
         HEAP32[$jmpIfDynamic>>2] = -1;
        }
       }
       $143 = HEAP32[$4>>2]|0;
       $144 = ($143|0)!=(0);
       if ($144) {
        $145 = HEAP32[$pE2>>2]|0;
        $146 = (_sqlite3ExprIsInteger($145,$iValToIns)|0);
        $147 = ($146|0)!=(0);
        if ($147) {
         $148 = HEAP32[$v>>2]|0;
         $149 = HEAP32[$2>>2]|0;
         $150 = (($149) + 28|0);
         $151 = HEAP32[$150>>2]|0;
         $152 = HEAP32[$r2>>2]|0;
         $153 = HEAP32[$iValToIns>>2]|0;
         (_sqlite3VdbeAddOp3($148,84,$151,$152,$153)|0);
        } else {
         label = 29;
        }
       } else {
        label = 29;
       }
       do {
        if ((label|0) == 29) {
         label = 0;
         $154 = HEAP32[$1>>2]|0;
         $155 = HEAP32[$pE2>>2]|0;
         $156 = HEAP32[$r1>>2]|0;
         $157 = (_sqlite3ExprCodeTarget($154,$155,$156)|0);
         HEAP32[$r3>>2] = $157;
         $158 = HEAP32[$4>>2]|0;
         $159 = ($158|0)!=(0);
         $160 = HEAP32[$v>>2]|0;
         $161 = HEAP32[$r3>>2]|0;
         if ($159) {
          $162 = HEAP32[$v>>2]|0;
          $163 = (_sqlite3VdbeCurrentAddr($162)|0);
          $164 = (($163) + 2)|0;
          (_sqlite3VdbeAddOp2($160,38,$161,$164)|0);
          $165 = HEAP32[$v>>2]|0;
          $166 = HEAP32[$2>>2]|0;
          $167 = (($166) + 28|0);
          $168 = HEAP32[$167>>2]|0;
          $169 = HEAP32[$r2>>2]|0;
          $170 = HEAP32[$r3>>2]|0;
          (_sqlite3VdbeAddOp3($165,75,$168,$169,$170)|0);
          break;
         } else {
          $171 = HEAP32[$r2>>2]|0;
          (_sqlite3VdbeAddOp4($160,49,$161,1,$171,$affinity,1)|0);
          $172 = HEAP32[$1>>2]|0;
          $173 = HEAP32[$r3>>2]|0;
          _sqlite3ExprCacheAffinityChange($172,$173,1);
          $174 = HEAP32[$v>>2]|0;
          $175 = HEAP32[$2>>2]|0;
          $176 = (($175) + 28|0);
          $177 = HEAP32[$176>>2]|0;
          $178 = HEAP32[$r2>>2]|0;
          (_sqlite3VdbeAddOp2($174,110,$177,$178)|0);
          break;
         }
        }
       } while(0);
       $179 = HEAP32[$i>>2]|0;
       $180 = (($179) + -1)|0;
       HEAP32[$i>>2] = $180;
       $181 = HEAP32[$pItem>>2]|0;
       $182 = (($181) + 20|0);
       HEAP32[$pItem>>2] = $182;
      }
      $183 = HEAP32[$1>>2]|0;
      $184 = HEAP32[$r1>>2]|0;
      _sqlite3ReleaseTempReg($183,$184);
      $185 = HEAP32[$1>>2]|0;
      $186 = HEAP32[$r2>>2]|0;
      _sqlite3ReleaseTempReg($185,$186);
     }
    }
   } while(0);
   $187 = HEAP32[$pKeyInfo>>2]|0;
   $188 = ($187|0)!=(0|0);
   if ($188) {
    $189 = HEAP32[$v>>2]|0;
    $190 = HEAP32[$addr>>2]|0;
    $191 = HEAP32[$pKeyInfo>>2]|0;
    _sqlite3VdbeChangeP4($189,$190,$191,-6);
   }
  } else {
   $192 = (($44) + 20|0);
   $193 = HEAP32[$192>>2]|0;
   HEAP32[$pSel>>2] = $193;
   $194 = HEAP32[$1>>2]|0;
   $195 = (($194) + 72|0);
   $196 = HEAP32[$195>>2]|0;
   $197 = (($196) + 1)|0;
   HEAP32[$195>>2] = $197;
   _sqlite3SelectDestInit($dest1,0,$197);
   $198 = HEAP32[$2>>2]|0;
   $199 = HEAP8[$198>>0]|0;
   $200 = $199&255;
   $201 = ($200|0)==(119);
   if ($201) {
    HEAP8[$dest1>>0] = 10;
    $202 = (($dest1) + 4|0);
    $203 = HEAP32[$202>>2]|0;
    $204 = (($dest1) + 8|0);
    HEAP32[$204>>2] = $203;
    $205 = HEAP32[$v>>2]|0;
    $206 = (($dest1) + 4|0);
    $207 = HEAP32[$206>>2]|0;
    (_sqlite3VdbeAddOp2($205,28,0,$207)|0);
   } else {
    HEAP8[$dest1>>0] = 3;
    $208 = HEAP32[$v>>2]|0;
    $209 = (($dest1) + 4|0);
    $210 = HEAP32[$209>>2]|0;
    (_sqlite3VdbeAddOp2($208,25,0,$210)|0);
   }
   $211 = HEAP32[$1>>2]|0;
   $212 = HEAP32[$211>>2]|0;
   $213 = HEAP32[$pSel>>2]|0;
   $214 = (($213) + 60|0);
   $215 = HEAP32[$214>>2]|0;
   _sqlite3ExprDelete($212,$215);
   $216 = HEAP32[$1>>2]|0;
   $217 = (_sqlite3PExpr($216,132,0,0,((10152 + 8|0)))|0);
   $218 = HEAP32[$pSel>>2]|0;
   $219 = (($218) + 60|0);
   HEAP32[$219>>2] = $217;
   $220 = HEAP32[$pSel>>2]|0;
   $221 = (($220) + 8|0);
   HEAP32[$221>>2] = 0;
   $222 = HEAP32[$1>>2]|0;
   $223 = HEAP32[$pSel>>2]|0;
   $224 = (_sqlite3Select($222,$223,$dest1)|0);
   $225 = ($224|0)!=(0);
   if (!($225)) {
    $226 = (($dest1) + 4|0);
    $227 = HEAP32[$226>>2]|0;
    HEAP32[$rReg>>2] = $227;
    break;
   }
   HEAP32[$0>>2] = 0;
   $241 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($241|0);
  }
 } while(0);
 $228 = HEAP32[$3>>2]|0;
 $229 = ($228|0)!=(0);
 if ($229) {
  $230 = HEAP32[$v>>2]|0;
  $231 = HEAP32[$2>>2]|0;
  $232 = (($231) + 28|0);
  $233 = HEAP32[$232>>2]|0;
  $234 = HEAP32[$3>>2]|0;
  _sqlite3SetHasNullFlag($230,$233,$234);
 }
 $235 = HEAP32[$jmpIfDynamic>>2]|0;
 $236 = ($235|0)>=(0);
 if ($236) {
  $237 = HEAP32[$v>>2]|0;
  $238 = HEAP32[$jmpIfDynamic>>2]|0;
  _sqlite3VdbeJumpHere($237,$238);
 }
 $239 = HEAP32[$1>>2]|0;
 _sqlite3ExprCachePop($239);
 $240 = HEAP32[$rReg>>2]|0;
 HEAP32[$0>>2] = $240;
 $241 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($241|0);
}
function _exprToRegister($p,$iReg) {
 $p = $p|0;
 $iReg = $iReg|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $1 = sp;
 HEAP32[$0>>2] = $p;
 HEAP32[$1>>2] = $iReg;
 $2 = HEAP32[$0>>2]|0;
 $3 = HEAP8[$2>>0]|0;
 $4 = HEAP32[$0>>2]|0;
 $5 = (($4) + 38|0);
 HEAP8[$5>>0] = $3;
 $6 = HEAP32[$0>>2]|0;
 HEAP8[$6>>0] = -97;
 $7 = HEAP32[$1>>2]|0;
 $8 = HEAP32[$0>>2]|0;
 $9 = (($8) + 28|0);
 HEAP32[$9>>2] = $7;
 $10 = HEAP32[$0>>2]|0;
 $11 = (($10) + 4|0);
 $12 = HEAP32[$11>>2]|0;
 $13 = $12 & -4097;
 HEAP32[$11>>2] = $13;
 STACKTOP = sp;return;
}
function _sqlite3CodeOnce($pParse) {
 $pParse = $pParse|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $v = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $v = sp;
 HEAP32[$0>>2] = $pParse;
 $1 = HEAP32[$0>>2]|0;
 $2 = (_sqlite3GetVdbe($1)|0);
 HEAP32[$v>>2] = $2;
 $3 = HEAP32[$v>>2]|0;
 $4 = HEAP32[$0>>2]|0;
 $5 = (($4) + 80|0);
 $6 = HEAP32[$5>>2]|0;
 $7 = (($6) + 1)|0;
 HEAP32[$5>>2] = $7;
 $8 = (_sqlite3VdbeAddOp1($3,44,$6)|0);
 STACKTOP = sp;return ($8|0);
}
function _sqlite3SelectDestInit($pDest,$eDest,$iParm) {
 $pDest = $pDest|0;
 $eDest = $eDest|0;
 $iParm = $iParm|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $1 = sp + 4|0;
 $2 = sp;
 HEAP32[$0>>2] = $pDest;
 HEAP32[$1>>2] = $eDest;
 HEAP32[$2>>2] = $iParm;
 $3 = HEAP32[$1>>2]|0;
 $4 = $3&255;
 $5 = HEAP32[$0>>2]|0;
 HEAP8[$5>>0] = $4;
 $6 = HEAP32[$2>>2]|0;
 $7 = HEAP32[$0>>2]|0;
 $8 = (($7) + 4|0);
 HEAP32[$8>>2] = $6;
 $9 = HEAP32[$0>>2]|0;
 $10 = (($9) + 1|0);
 HEAP8[$10>>0] = 0;
 $11 = HEAP32[$0>>2]|0;
 $12 = (($11) + 8|0);
 HEAP32[$12>>2] = 0;
 $13 = HEAP32[$0>>2]|0;
 $14 = (($13) + 12|0);
 HEAP32[$14>>2] = 0;
 STACKTOP = sp;return;
}
function _sqlite3BinaryCompareCollSeq($pParse,$pLeft,$pRight) {
 $pParse = $pParse|0;
 $pLeft = $pLeft|0;
 $pRight = $pRight|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $pColl = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $1 = sp + 8|0;
 $2 = sp + 4|0;
 $pColl = sp;
 HEAP32[$0>>2] = $pParse;
 HEAP32[$1>>2] = $pLeft;
 HEAP32[$2>>2] = $pRight;
 $3 = HEAP32[$1>>2]|0;
 $4 = (($3) + 4|0);
 $5 = HEAP32[$4>>2]|0;
 $6 = $5 & 256;
 $7 = ($6|0)!=(0);
 if ($7) {
  $8 = HEAP32[$0>>2]|0;
  $9 = HEAP32[$1>>2]|0;
  $10 = (_sqlite3ExprCollSeq($8,$9)|0);
  HEAP32[$pColl>>2] = $10;
  $29 = HEAP32[$pColl>>2]|0;
  STACKTOP = sp;return ($29|0);
 }
 $11 = HEAP32[$2>>2]|0;
 $12 = ($11|0)!=(0|0);
 if ($12) {
  $13 = HEAP32[$2>>2]|0;
  $14 = (($13) + 4|0);
  $15 = HEAP32[$14>>2]|0;
  $16 = $15 & 256;
  $17 = ($16|0)!=(0);
  if ($17) {
   $18 = HEAP32[$0>>2]|0;
   $19 = HEAP32[$2>>2]|0;
   $20 = (_sqlite3ExprCollSeq($18,$19)|0);
   HEAP32[$pColl>>2] = $20;
   $29 = HEAP32[$pColl>>2]|0;
   STACKTOP = sp;return ($29|0);
  }
 }
 $21 = HEAP32[$0>>2]|0;
 $22 = HEAP32[$1>>2]|0;
 $23 = (_sqlite3ExprCollSeq($21,$22)|0);
 HEAP32[$pColl>>2] = $23;
 $24 = HEAP32[$pColl>>2]|0;
 $25 = ($24|0)!=(0|0);
 if ($25) {
  $29 = HEAP32[$pColl>>2]|0;
  STACKTOP = sp;return ($29|0);
 }
 $26 = HEAP32[$0>>2]|0;
 $27 = HEAP32[$2>>2]|0;
 $28 = (_sqlite3ExprCollSeq($26,$27)|0);
 HEAP32[$pColl>>2] = $28;
 $29 = HEAP32[$pColl>>2]|0;
 STACKTOP = sp;return ($29|0);
}
function _sqlite3SetHasNullFlag($v,$iCur,$regHasNull) {
 $v = $v|0;
 $iCur = $iCur|0;
 $regHasNull = $regHasNull|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $j1 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $1 = sp + 8|0;
 $2 = sp + 4|0;
 $j1 = sp;
 HEAP32[$0>>2] = $v;
 HEAP32[$1>>2] = $iCur;
 HEAP32[$2>>2] = $regHasNull;
 $3 = HEAP32[$0>>2]|0;
 $4 = HEAP32[$2>>2]|0;
 (_sqlite3VdbeAddOp2($3,25,0,$4)|0);
 $5 = HEAP32[$0>>2]|0;
 $6 = HEAP32[$1>>2]|0;
 $7 = (_sqlite3VdbeAddOp1($5,108,$6)|0);
 HEAP32[$j1>>2] = $7;
 $8 = HEAP32[$0>>2]|0;
 $9 = HEAP32[$1>>2]|0;
 $10 = HEAP32[$2>>2]|0;
 (_sqlite3VdbeAddOp3($8,47,$9,0,$10)|0);
 $11 = HEAP32[$0>>2]|0;
 _sqlite3VdbeChangeP5($11,-128);
 $12 = HEAP32[$0>>2]|0;
 $13 = HEAP32[$j1>>2]|0;
 _sqlite3VdbeJumpHere($12,$13);
 STACKTOP = sp;return;
}
function _sqlite3ExprCodeAtInit($pParse,$pExpr,$regDest,$reusable) {
 $pParse = $pParse|0;
 $pExpr = $pExpr|0;
 $regDest = $regDest|0;
 $reusable = $reusable|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0;
 var $p = 0, $pItem = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 16|0;
 $1 = sp + 12|0;
 $2 = sp + 8|0;
 $3 = sp + 20|0;
 $p = sp + 4|0;
 $pItem = sp;
 HEAP32[$0>>2] = $pParse;
 HEAP32[$1>>2] = $pExpr;
 HEAP32[$2>>2] = $regDest;
 HEAP8[$3>>0] = $reusable;
 $4 = HEAP32[$0>>2]|0;
 $5 = (($4) + 316|0);
 $6 = HEAP32[$5>>2]|0;
 HEAP32[$p>>2] = $6;
 $7 = HEAP32[$0>>2]|0;
 $8 = HEAP32[$7>>2]|0;
 $9 = HEAP32[$1>>2]|0;
 $10 = (_sqlite3ExprDup($8,$9,0)|0);
 HEAP32[$1>>2] = $10;
 $11 = HEAP32[$0>>2]|0;
 $12 = HEAP32[$p>>2]|0;
 $13 = HEAP32[$1>>2]|0;
 $14 = (_sqlite3ExprListAppend($11,$12,$13)|0);
 HEAP32[$p>>2] = $14;
 $15 = HEAP32[$p>>2]|0;
 $16 = ($15|0)!=(0|0);
 if (!($16)) {
  $37 = HEAP32[$p>>2]|0;
  $38 = HEAP32[$0>>2]|0;
  $39 = (($38) + 316|0);
  HEAP32[$39>>2] = $37;
  STACKTOP = sp;return;
 }
 $17 = HEAP32[$p>>2]|0;
 $18 = HEAP32[$17>>2]|0;
 $19 = (($18) - 1)|0;
 $20 = HEAP32[$p>>2]|0;
 $21 = (($20) + 4|0);
 $22 = HEAP32[$21>>2]|0;
 $23 = (($22) + (($19*20)|0)|0);
 HEAP32[$pItem>>2] = $23;
 $24 = HEAP32[$2>>2]|0;
 $25 = HEAP32[$pItem>>2]|0;
 $26 = (($25) + 16|0);
 HEAP32[$26>>2] = $24;
 $27 = HEAP8[$3>>0]|0;
 $28 = $27&255;
 $29 = HEAP32[$pItem>>2]|0;
 $30 = (($29) + 13|0);
 $31 = $28&255;
 $32 = HEAP8[$30>>0]|0;
 $33 = $31 & 1;
 $34 = ($33 << 2)&255;
 $35 = $32 & -5;
 $36 = $35 | $34;
 HEAP8[$30>>0] = $36;
 $37 = HEAP32[$p>>2]|0;
 $38 = HEAP32[$0>>2]|0;
 $39 = (($38) + 316|0);
 HEAP32[$39>>2] = $37;
 STACKTOP = sp;return;
}
function _sqlite3VdbeGetOp($p,$addr) {
 $p = $p|0;
 $addr = $addr|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0;
 var label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $1 = sp + 4|0;
 $2 = sp;
 HEAP32[$1>>2] = $p;
 HEAP32[$2>>2] = $addr;
 $3 = HEAP32[$2>>2]|0;
 $4 = ($3|0)<(0);
 if ($4) {
  $5 = HEAP32[$1>>2]|0;
  $6 = (($5) + 32|0);
  $7 = HEAP32[$6>>2]|0;
  $8 = (($7) - 1)|0;
  HEAP32[$2>>2] = $8;
 }
 $9 = HEAP32[$1>>2]|0;
 $10 = HEAP32[$9>>2]|0;
 $11 = (($10) + 64|0);
 $12 = HEAP8[$11>>0]|0;
 $13 = ($12<<24>>24)!=(0);
 if ($13) {
  HEAP32[$0>>2] = 19312;
  $19 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($19|0);
 } else {
  $14 = HEAP32[$2>>2]|0;
  $15 = HEAP32[$1>>2]|0;
  $16 = (($15) + 4|0);
  $17 = HEAP32[$16>>2]|0;
  $18 = (($17) + (($14*20)|0)|0);
  HEAP32[$0>>2] = $18;
  $19 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($19|0);
 }
 return 0|0;
}
function _exprIsConst($p,$initFlag) {
 $p = $p|0;
 $initFlag = $initFlag|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $w = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 28|0;
 $1 = sp + 24|0;
 $w = sp;
 HEAP32[$0>>2] = $p;
 HEAP32[$1>>2] = $initFlag;
 ;HEAP32[$w+0>>2]=0|0;HEAP32[$w+4>>2]=0|0;HEAP32[$w+8>>2]=0|0;HEAP32[$w+12>>2]=0|0;HEAP32[$w+16>>2]=0|0;HEAP32[$w+20>>2]=0|0;
 $2 = HEAP32[$1>>2]|0;
 $3 = (($w) + 20|0);
 HEAP32[$3>>2] = $2;
 HEAP32[$w>>2] = 44;
 $4 = (($w) + 4|0);
 HEAP32[$4>>2] = 45;
 $5 = HEAP32[$0>>2]|0;
 (_sqlite3WalkExpr($w,$5)|0);
 $6 = (($w) + 20|0);
 $7 = HEAP32[$6>>2]|0;
 STACKTOP = sp;return ($7|0);
}
function _exprNodeIsConstant($pWalker,$pExpr) {
 $pWalker = $pWalker|0;
 $pExpr = $pExpr|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0;
 var label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $1 = sp + 4|0;
 $2 = sp;
 HEAP32[$1>>2] = $pWalker;
 HEAP32[$2>>2] = $pExpr;
 $3 = HEAP32[$1>>2]|0;
 $4 = (($3) + 20|0);
 $5 = HEAP32[$4>>2]|0;
 $6 = ($5|0)==(2);
 if ($6) {
  $7 = HEAP32[$2>>2]|0;
  $8 = (($7) + 4|0);
  $9 = HEAP32[$8>>2]|0;
  $10 = $9 & 1;
  $11 = ($10|0)!=(0);
  if ($11) {
   $12 = HEAP32[$1>>2]|0;
   $13 = (($12) + 20|0);
   HEAP32[$13>>2] = 0;
   HEAP32[$0>>2] = 2;
   $39 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($39|0);
  }
 }
 $14 = HEAP32[$2>>2]|0;
 $15 = HEAP8[$14>>0]|0;
 $16 = $15&255;
 L6: do {
  switch ($16|0) {
  case 135:  {
   $28 = HEAP32[$1>>2]|0;
   $29 = (($28) + 20|0);
   $30 = HEAP32[$29>>2]|0;
   $31 = ($30|0)==(4);
   if ($31) {
    $32 = HEAP32[$2>>2]|0;
    HEAP8[$32>>0] = 101;
    label = 13;
    break L6;
   }
   $33 = HEAP32[$1>>2]|0;
   $34 = (($33) + 20|0);
   $35 = HEAP32[$34>>2]|0;
   $36 = ($35|0)==(3);
   if ($36) {
    $37 = HEAP32[$1>>2]|0;
    $38 = (($37) + 20|0);
    HEAP32[$38>>2] = 0;
    HEAP32[$0>>2] = 2;
    $39 = HEAP32[$0>>2]|0;
    STACKTOP = sp;return ($39|0);
   } else {
    label = 13;
   }
   break;
  }
  case 153:  {
   $17 = HEAP32[$1>>2]|0;
   $18 = (($17) + 20|0);
   $19 = HEAP32[$18>>2]|0;
   $20 = ($19|0)>=(3);
   if (!($20)) {
    $21 = HEAP32[$2>>2]|0;
    $22 = (($21) + 4|0);
    $23 = HEAP32[$22>>2]|0;
    $24 = $23 & 524288;
    $25 = ($24|0)!=(0);
    if (!($25)) {
     label = 8;
     break L6;
    }
   }
   HEAP32[$0>>2] = 0;
   $39 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($39|0);
   break;
  }
  case 156: case 155: case 154: case 27:  {
   label = 8;
   break;
  }
  default: {
   label = 13;
  }
  }
 } while(0);
 if ((label|0) == 8) {
  $26 = HEAP32[$1>>2]|0;
  $27 = (($26) + 20|0);
  HEAP32[$27>>2] = 0;
  HEAP32[$0>>2] = 2;
  $39 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($39|0);
 }
 else if ((label|0) == 13) {
  HEAP32[$0>>2] = 0;
  $39 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($39|0);
 }
 return 0|0;
}
function _selectNodeIsConstant($pWalker,$NotUsed) {
 $pWalker = $pWalker|0;
 $NotUsed = $NotUsed|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $1 = sp;
 HEAP32[$0>>2] = $pWalker;
 HEAP32[$1>>2] = $NotUsed;
 $2 = HEAP32[$0>>2]|0;
 $3 = (($2) + 20|0);
 HEAP32[$3>>2] = 0;
 STACKTOP = sp;return 2;
}
function _dup8bytes($v,$in) {
 $v = $v|0;
 $in = $in|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $out = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $1 = sp + 4|0;
 $out = sp;
 HEAP32[$0>>2] = $v;
 HEAP32[$1>>2] = $in;
 $2 = HEAP32[$0>>2]|0;
 $3 = (_sqlite3VdbeDb($2)|0);
 $4 = (_sqlite3DbMallocRaw($3,8,0)|0);
 HEAP32[$out>>2] = $4;
 $5 = HEAP32[$out>>2]|0;
 $6 = ($5|0)!=(0|0);
 if (!($6)) {
  $9 = HEAP32[$out>>2]|0;
  STACKTOP = sp;return ($9|0);
 }
 $7 = HEAP32[$out>>2]|0;
 $8 = HEAP32[$1>>2]|0;
 ;HEAP8[$7+0>>0]=HEAP8[$8+0>>0]|0;HEAP8[$7+1>>0]=HEAP8[$8+1>>0]|0;HEAP8[$7+2>>0]=HEAP8[$8+2>>0]|0;HEAP8[$7+3>>0]=HEAP8[$8+3>>0]|0;HEAP8[$7+4>>0]=HEAP8[$8+4>>0]|0;HEAP8[$7+5>>0]=HEAP8[$8+5>>0]|0;HEAP8[$7+6>>0]=HEAP8[$8+6>>0]|0;HEAP8[$7+7>>0]=HEAP8[$8+7>>0]|0;
 $9 = HEAP32[$out>>2]|0;
 STACKTOP = sp;return ($9|0);
}
function _sqlite3ExprCachePinRegister($pParse,$iReg) {
 $pParse = $pParse|0;
 $iReg = $iReg|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $i = 0, $p = 0, label = 0;
 var sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $1 = sp + 8|0;
 $i = sp + 4|0;
 $p = sp;
 HEAP32[$0>>2] = $pParse;
 HEAP32[$1>>2] = $iReg;
 HEAP32[$i>>2] = 0;
 $2 = HEAP32[$0>>2]|0;
 $3 = (($2) + 116|0);
 HEAP32[$p>>2] = $3;
 while(1) {
  $4 = HEAP32[$i>>2]|0;
  $5 = ($4|0)<(10);
  if (!($5)) {
   break;
  }
  $6 = HEAP32[$p>>2]|0;
  $7 = (($6) + 12|0);
  $8 = HEAP32[$7>>2]|0;
  $9 = HEAP32[$1>>2]|0;
  $10 = ($8|0)==($9|0);
  if ($10) {
   $11 = HEAP32[$p>>2]|0;
   $12 = (($11) + 6|0);
   HEAP8[$12>>0] = 0;
  }
  $13 = HEAP32[$i>>2]|0;
  $14 = (($13) + 1)|0;
  HEAP32[$i>>2] = $14;
  $15 = HEAP32[$p>>2]|0;
  $16 = (($15) + 20|0);
  HEAP32[$p>>2] = $16;
 }
 STACKTOP = sp;return;
}
function _sqlite3ExprCacheStore($pParse,$iTab,$iCol,$iReg) {
 $pParse = $pParse|0;
 $iTab = $iTab|0;
 $iCol = $iCol|0;
 $iReg = $iReg|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0;
 var $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $i = 0, $idxLru = 0, $minLru = 0, $p = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 28|0;
 $1 = sp + 24|0;
 $2 = sp + 20|0;
 $3 = sp + 16|0;
 $i = sp + 12|0;
 $minLru = sp + 8|0;
 $idxLru = sp + 4|0;
 $p = sp;
 HEAP32[$0>>2] = $pParse;
 HEAP32[$1>>2] = $iTab;
 HEAP32[$2>>2] = $iCol;
 HEAP32[$3>>2] = $iReg;
 $4 = HEAP32[$0>>2]|0;
 $5 = HEAP32[$4>>2]|0;
 $6 = (($5) + 60|0);
 $7 = HEAP16[$6>>1]|0;
 $8 = $7&65535;
 $9 = $8 & 2;
 $10 = ($9|0)!=(0);
 if ($10) {
  STACKTOP = sp;return;
 }
 HEAP32[$i>>2] = 0;
 $11 = HEAP32[$0>>2]|0;
 $12 = (($11) + 116|0);
 HEAP32[$p>>2] = $12;
 while(1) {
  $13 = HEAP32[$i>>2]|0;
  $14 = ($13|0)<(10);
  if (!($14)) {
   break;
  }
  $15 = HEAP32[$p>>2]|0;
  $16 = (($15) + 12|0);
  $17 = HEAP32[$16>>2]|0;
  $18 = ($17|0)==(0);
  if ($18) {
   label = 5;
   break;
  }
  $41 = HEAP32[$i>>2]|0;
  $42 = (($41) + 1)|0;
  HEAP32[$i>>2] = $42;
  $43 = HEAP32[$p>>2]|0;
  $44 = (($43) + 20|0);
  HEAP32[$p>>2] = $44;
 }
 if ((label|0) == 5) {
  $19 = HEAP32[$0>>2]|0;
  $20 = (($19) + 100|0);
  $21 = HEAP32[$20>>2]|0;
  $22 = HEAP32[$p>>2]|0;
  $23 = (($22) + 8|0);
  HEAP32[$23>>2] = $21;
  $24 = HEAP32[$1>>2]|0;
  $25 = HEAP32[$p>>2]|0;
  HEAP32[$25>>2] = $24;
  $26 = HEAP32[$2>>2]|0;
  $27 = $26&65535;
  $28 = HEAP32[$p>>2]|0;
  $29 = (($28) + 4|0);
  HEAP16[$29>>1] = $27;
  $30 = HEAP32[$3>>2]|0;
  $31 = HEAP32[$p>>2]|0;
  $32 = (($31) + 12|0);
  HEAP32[$32>>2] = $30;
  $33 = HEAP32[$p>>2]|0;
  $34 = (($33) + 6|0);
  HEAP8[$34>>0] = 0;
  $35 = HEAP32[$0>>2]|0;
  $36 = (($35) + 104|0);
  $37 = HEAP32[$36>>2]|0;
  $38 = (($37) + 1)|0;
  HEAP32[$36>>2] = $38;
  $39 = HEAP32[$p>>2]|0;
  $40 = (($39) + 16|0);
  HEAP32[$40>>2] = $37;
  STACKTOP = sp;return;
 }
 HEAP32[$minLru>>2] = 2147483647;
 HEAP32[$idxLru>>2] = -1;
 HEAP32[$i>>2] = 0;
 $45 = HEAP32[$0>>2]|0;
 $46 = (($45) + 116|0);
 HEAP32[$p>>2] = $46;
 while(1) {
  $47 = HEAP32[$i>>2]|0;
  $48 = ($47|0)<(10);
  if (!($48)) {
   break;
  }
  $49 = HEAP32[$p>>2]|0;
  $50 = (($49) + 16|0);
  $51 = HEAP32[$50>>2]|0;
  $52 = HEAP32[$minLru>>2]|0;
  $53 = ($51|0)<($52|0);
  if ($53) {
   $54 = HEAP32[$i>>2]|0;
   HEAP32[$idxLru>>2] = $54;
   $55 = HEAP32[$p>>2]|0;
   $56 = (($55) + 16|0);
   $57 = HEAP32[$56>>2]|0;
   HEAP32[$minLru>>2] = $57;
  }
  $58 = HEAP32[$i>>2]|0;
  $59 = (($58) + 1)|0;
  HEAP32[$i>>2] = $59;
  $60 = HEAP32[$p>>2]|0;
  $61 = (($60) + 20|0);
  HEAP32[$p>>2] = $61;
 }
 $62 = HEAP32[$idxLru>>2]|0;
 $63 = ($62|0)>=(0);
 if (!($63)) {
  STACKTOP = sp;return;
 }
 $64 = HEAP32[$idxLru>>2]|0;
 $65 = HEAP32[$0>>2]|0;
 $66 = (($65) + 116|0);
 $67 = (($66) + (($64*20)|0)|0);
 HEAP32[$p>>2] = $67;
 $68 = HEAP32[$0>>2]|0;
 $69 = (($68) + 100|0);
 $70 = HEAP32[$69>>2]|0;
 $71 = HEAP32[$p>>2]|0;
 $72 = (($71) + 8|0);
 HEAP32[$72>>2] = $70;
 $73 = HEAP32[$1>>2]|0;
 $74 = HEAP32[$p>>2]|0;
 HEAP32[$74>>2] = $73;
 $75 = HEAP32[$2>>2]|0;
 $76 = $75&65535;
 $77 = HEAP32[$p>>2]|0;
 $78 = (($77) + 4|0);
 HEAP16[$78>>1] = $76;
 $79 = HEAP32[$3>>2]|0;
 $80 = HEAP32[$p>>2]|0;
 $81 = (($80) + 12|0);
 HEAP32[$81>>2] = $79;
 $82 = HEAP32[$p>>2]|0;
 $83 = (($82) + 6|0);
 HEAP8[$83>>0] = 0;
 $84 = HEAP32[$0>>2]|0;
 $85 = (($84) + 104|0);
 $86 = HEAP32[$85>>2]|0;
 $87 = (($86) + 1)|0;
 HEAP32[$85>>2] = $87;
 $88 = HEAP32[$p>>2]|0;
 $89 = (($88) + 16|0);
 HEAP32[$89>>2] = $86;
 STACKTOP = sp;return;
}
function _sqlite3CompareAffinity($pExpr,$aff2) {
 $pExpr = $pExpr|0;
 $aff2 = $aff2|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $aff1 = 0, $or$cond = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 6|0;
 $1 = sp;
 $2 = sp + 5|0;
 $aff1 = sp + 4|0;
 HEAP32[$1>>2] = $pExpr;
 HEAP8[$2>>0] = $aff2;
 $3 = HEAP32[$1>>2]|0;
 $4 = (_sqlite3ExprAffinity($3)|0);
 HEAP8[$aff1>>0] = $4;
 $5 = HEAP8[$aff1>>0]|0;
 $6 = $5 << 24 >> 24;
 $7 = ($6|0)!=(0);
 if ($7) {
  $8 = HEAP8[$2>>0]|0;
  $9 = $8 << 24 >> 24;
  $10 = ($9|0)!=(0);
  if ($10) {
   $11 = HEAP8[$aff1>>0]|0;
   $12 = $11 << 24 >> 24;
   $13 = ($12|0)>=(67);
   if (!($13)) {
    $14 = HEAP8[$2>>0]|0;
    $15 = $14 << 24 >> 24;
    $16 = ($15|0)>=(67);
    if (!($16)) {
     HEAP8[$0>>0] = 65;
     $27 = HEAP8[$0>>0]|0;
     STACKTOP = sp;return ($27|0);
    }
   }
   HEAP8[$0>>0] = 67;
   $27 = HEAP8[$0>>0]|0;
   STACKTOP = sp;return ($27|0);
  }
 }
 $17 = HEAP8[$aff1>>0]|0;
 $18 = ($17<<24>>24)!=(0);
 $19 = HEAP8[$2>>0]|0;
 $20 = ($19<<24>>24)!=(0);
 $or$cond = $18 | $20;
 if ($or$cond) {
  $21 = HEAP8[$aff1>>0]|0;
  $22 = $21 << 24 >> 24;
  $23 = HEAP8[$2>>0]|0;
  $24 = $23 << 24 >> 24;
  $25 = (($22) + ($24))|0;
  $26 = $25&255;
  HEAP8[$0>>0] = $26;
  $27 = HEAP8[$0>>0]|0;
  STACKTOP = sp;return ($27|0);
 } else {
  HEAP8[$0>>0] = 65;
  $27 = HEAP8[$0>>0]|0;
  STACKTOP = sp;return ($27|0);
 }
 return 0|0;
}
function _isCandidateForInOpt($p) {
 $p = $p|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $pEList = 0, $pSrc = 0;
 var $pTab = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 16|0;
 $1 = sp + 12|0;
 $pSrc = sp + 8|0;
 $pEList = sp + 4|0;
 $pTab = sp;
 HEAP32[$1>>2] = $p;
 $2 = HEAP32[$1>>2]|0;
 $3 = ($2|0)==(0|0);
 do {
  if ($3) {
   HEAP32[$0>>2] = 0;
  } else {
   $4 = HEAP32[$1>>2]|0;
   $5 = (($4) + 52|0);
   $6 = HEAP32[$5>>2]|0;
   $7 = ($6|0)!=(0|0);
   if ($7) {
    HEAP32[$0>>2] = 0;
    break;
   }
   $8 = HEAP32[$1>>2]|0;
   $9 = (($8) + 6|0);
   $10 = HEAP16[$9>>1]|0;
   $11 = $10&65535;
   $12 = $11 & 5;
   $13 = ($12|0)!=(0);
   if ($13) {
    HEAP32[$0>>2] = 0;
    break;
   }
   $14 = HEAP32[$1>>2]|0;
   $15 = (($14) + 60|0);
   $16 = HEAP32[$15>>2]|0;
   $17 = ($16|0)!=(0|0);
   if ($17) {
    HEAP32[$0>>2] = 0;
    break;
   }
   $18 = HEAP32[$1>>2]|0;
   $19 = (($18) + 36|0);
   $20 = HEAP32[$19>>2]|0;
   $21 = ($20|0)!=(0|0);
   if ($21) {
    HEAP32[$0>>2] = 0;
    break;
   }
   $22 = HEAP32[$1>>2]|0;
   $23 = (($22) + 32|0);
   $24 = HEAP32[$23>>2]|0;
   HEAP32[$pSrc>>2] = $24;
   $25 = HEAP32[$pSrc>>2]|0;
   $26 = HEAP32[$25>>2]|0;
   $27 = ($26|0)!=(1);
   if ($27) {
    HEAP32[$0>>2] = 0;
    break;
   }
   $28 = HEAP32[$pSrc>>2]|0;
   $29 = (($28) + 8|0);
   $30 = (($29) + 20|0);
   $31 = HEAP32[$30>>2]|0;
   $32 = ($31|0)!=(0|0);
   if ($32) {
    HEAP32[$0>>2] = 0;
    break;
   }
   $33 = HEAP32[$pSrc>>2]|0;
   $34 = (($33) + 8|0);
   $35 = (($34) + 16|0);
   $36 = HEAP32[$35>>2]|0;
   HEAP32[$pTab>>2] = $36;
   $37 = HEAP32[$pTab>>2]|0;
   $38 = ($37|0)==(0|0);
   if ($38) {
    HEAP32[$0>>2] = 0;
    break;
   }
   $39 = HEAP32[$pTab>>2]|0;
   $40 = (($39) + 44|0);
   $41 = HEAP8[$40>>0]|0;
   $42 = $41&255;
   $43 = $42 & 16;
   $44 = ($43|0)!=(0);
   if ($44) {
    HEAP32[$0>>2] = 0;
    break;
   }
   $45 = HEAP32[$1>>2]|0;
   $46 = HEAP32[$45>>2]|0;
   HEAP32[$pEList>>2] = $46;
   $47 = HEAP32[$pEList>>2]|0;
   $48 = HEAP32[$47>>2]|0;
   $49 = ($48|0)!=(1);
   if ($49) {
    HEAP32[$0>>2] = 0;
    break;
   }
   $50 = HEAP32[$pEList>>2]|0;
   $51 = (($50) + 4|0);
   $52 = HEAP32[$51>>2]|0;
   $53 = HEAP32[$52>>2]|0;
   $54 = HEAP8[$53>>0]|0;
   $55 = $54&255;
   $56 = ($55|0)!=(154);
   if ($56) {
    HEAP32[$0>>2] = 0;
    break;
   } else {
    HEAP32[$0>>2] = 1;
    break;
   }
  }
 } while(0);
 $57 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($57|0);
}
function _sqlite3IndexAffinityOk($pExpr,$idx_affinity) {
 $pExpr = $pExpr|0;
 $idx_affinity = $idx_affinity|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $aff = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $1 = sp;
 $2 = sp + 9|0;
 $aff = sp + 8|0;
 HEAP32[$1>>2] = $pExpr;
 HEAP8[$2>>0] = $idx_affinity;
 $3 = HEAP32[$1>>2]|0;
 $4 = (_comparisonAffinity($3)|0);
 HEAP8[$aff>>0] = $4;
 $5 = HEAP8[$aff>>0]|0;
 $6 = $5 << 24 >> 24;
 if ((($6|0) == 66)) {
  $7 = HEAP8[$2>>0]|0;
  $8 = $7 << 24 >> 24;
  $9 = ($8|0)==(66);
  $10 = $9&1;
  HEAP32[$0>>2] = $10;
 } else if ((($6|0) == 65)) {
  HEAP32[$0>>2] = 1;
 } else {
  $11 = HEAP8[$2>>0]|0;
  $12 = $11 << 24 >> 24;
  $13 = ($12|0)>=(67);
  $14 = $13&1;
  HEAP32[$0>>2] = $14;
 }
 $15 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($15|0);
}
function _sqlite3InRhsIsConstant($pIn) {
 $pIn = $pIn|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $pLHS = 0, $res = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $pLHS = sp + 4|0;
 $res = sp;
 HEAP32[$0>>2] = $pIn;
 $1 = HEAP32[$0>>2]|0;
 $2 = (($1) + 12|0);
 $3 = HEAP32[$2>>2]|0;
 HEAP32[$pLHS>>2] = $3;
 $4 = HEAP32[$0>>2]|0;
 $5 = (($4) + 12|0);
 HEAP32[$5>>2] = 0;
 $6 = HEAP32[$0>>2]|0;
 $7 = (_sqlite3ExprIsConstant($6)|0);
 HEAP32[$res>>2] = $7;
 $8 = HEAP32[$pLHS>>2]|0;
 $9 = HEAP32[$0>>2]|0;
 $10 = (($9) + 12|0);
 HEAP32[$10>>2] = $8;
 $11 = HEAP32[$res>>2]|0;
 STACKTOP = sp;return ($11|0);
}
function _binaryCompareP5($pExpr1,$pExpr2,$jumpIfNull) {
 $pExpr1 = $pExpr1|0;
 $pExpr2 = $pExpr2|0;
 $jumpIfNull = $jumpIfNull|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $aff = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $1 = sp + 4|0;
 $2 = sp;
 $aff = sp + 12|0;
 HEAP32[$0>>2] = $pExpr1;
 HEAP32[$1>>2] = $pExpr2;
 HEAP32[$2>>2] = $jumpIfNull;
 $3 = HEAP32[$1>>2]|0;
 $4 = (_sqlite3ExprAffinity($3)|0);
 HEAP8[$aff>>0] = $4;
 $5 = HEAP32[$0>>2]|0;
 $6 = HEAP8[$aff>>0]|0;
 $7 = (_sqlite3CompareAffinity($5,$6)|0);
 $8 = $7&255;
 $9 = HEAP32[$2>>2]|0;
 $10 = $9&255;
 $11 = $10&255;
 $12 = $8 | $11;
 $13 = $12&255;
 HEAP8[$aff>>0] = $13;
 $14 = HEAP8[$aff>>0]|0;
 STACKTOP = sp;return ($14|0);
}
function _sqlite3ExprIsConstantNotJoin($p) {
 $p = $p|0;
 var $0 = 0, $1 = 0, $2 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 HEAP32[$0>>2] = $p;
 $1 = HEAP32[$0>>2]|0;
 $2 = (_exprIsConst($1,2)|0);
 STACKTOP = sp;return ($2|0);
}
function _collationMatch($zColl,$pIndex) {
 $zColl = $zColl|0;
 $pIndex = $pIndex|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $i = 0, $z = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 16|0;
 $1 = sp + 12|0;
 $2 = sp + 8|0;
 $i = sp + 4|0;
 $z = sp;
 HEAP32[$1>>2] = $zColl;
 HEAP32[$2>>2] = $pIndex;
 HEAP32[$i>>2] = 0;
 while(1) {
  $3 = HEAP32[$i>>2]|0;
  $4 = HEAP32[$2>>2]|0;
  $5 = (($4) + 52|0);
  $6 = HEAP16[$5>>1]|0;
  $7 = $6&65535;
  $8 = ($3|0)<($7|0);
  if (!($8)) {
   label = 7;
   break;
  }
  $9 = HEAP32[$i>>2]|0;
  $10 = HEAP32[$2>>2]|0;
  $11 = (($10) + 32|0);
  $12 = HEAP32[$11>>2]|0;
  $13 = (($12) + ($9<<2)|0);
  $14 = HEAP32[$13>>2]|0;
  HEAP32[$z>>2] = $14;
  $15 = HEAP32[$i>>2]|0;
  $16 = HEAP32[$2>>2]|0;
  $17 = (($16) + 4|0);
  $18 = HEAP32[$17>>2]|0;
  $19 = (($18) + ($15<<1)|0);
  $20 = HEAP16[$19>>1]|0;
  $21 = $20 << 16 >> 16;
  $22 = ($21|0)>=(0);
  if ($22) {
   $23 = HEAP32[$z>>2]|0;
   $24 = HEAP32[$1>>2]|0;
   $25 = (_sqlite3_stricmp($23,$24)|0);
   $26 = (0)==($25|0);
   if ($26) {
    label = 5;
    break;
   }
  }
  $27 = HEAP32[$i>>2]|0;
  $28 = (($27) + 1)|0;
  HEAP32[$i>>2] = $28;
 }
 if ((label|0) == 5) {
  HEAP32[$0>>2] = 1;
  $29 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($29|0);
 }
 else if ((label|0) == 7) {
  HEAP32[$0>>2] = 0;
  $29 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($29|0);
 }
 return 0|0;
}
function _detachFunc($context,$NotUsed,$argv) {
 $context = $context|0;
 $NotUsed = $NotUsed|0;
 $argv = $argv|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $7 = 0, $8 = 0, $9 = 0, $db = 0, $i = 0, $pDb = 0, $vararg_buffer6 = 0, $zErr = 0, $zName = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 160|0;
 $vararg_buffer6 = sp;
 $0 = sp + 28|0;
 $1 = sp + 24|0;
 $2 = sp + 20|0;
 $zName = sp + 16|0;
 $db = sp + 12|0;
 $i = sp + 8|0;
 $pDb = sp + 4|0;
 $zErr = sp + 32|0;
 HEAP32[$0>>2] = $context;
 HEAP32[$1>>2] = $NotUsed;
 HEAP32[$2>>2] = $argv;
 $3 = HEAP32[$2>>2]|0;
 $4 = HEAP32[$3>>2]|0;
 $5 = (_sqlite3_value_text($4)|0);
 HEAP32[$zName>>2] = $5;
 $6 = HEAP32[$0>>2]|0;
 $7 = (_sqlite3_context_db_handle($6)|0);
 HEAP32[$db>>2] = $7;
 HEAP32[$pDb>>2] = 0;
 $8 = HEAP32[$zName>>2]|0;
 $9 = ($8|0)==(0|0);
 if ($9) {
  HEAP32[$zName>>2] = 1216;
 }
 HEAP32[$i>>2] = 0;
 while(1) {
  $10 = HEAP32[$i>>2]|0;
  $11 = HEAP32[$db>>2]|0;
  $12 = (($11) + 20|0);
  $13 = HEAP32[$12>>2]|0;
  $14 = ($10|0)<($13|0);
  if (!($14)) {
   break;
  }
  $15 = HEAP32[$i>>2]|0;
  $16 = HEAP32[$db>>2]|0;
  $17 = (($16) + 16|0);
  $18 = HEAP32[$17>>2]|0;
  $19 = (($18) + ($15<<4)|0);
  HEAP32[$pDb>>2] = $19;
  $20 = HEAP32[$pDb>>2]|0;
  $21 = (($20) + 4|0);
  $22 = HEAP32[$21>>2]|0;
  $23 = ($22|0)==(0|0);
  if (!($23)) {
   $24 = HEAP32[$pDb>>2]|0;
   $25 = HEAP32[$24>>2]|0;
   $26 = HEAP32[$zName>>2]|0;
   $27 = (_sqlite3_stricmp($25,$26)|0);
   $28 = ($27|0)==(0);
   if ($28) {
    break;
   }
  }
  $29 = HEAP32[$i>>2]|0;
  $30 = (($29) + 1)|0;
  HEAP32[$i>>2] = $30;
 }
 $31 = HEAP32[$i>>2]|0;
 $32 = HEAP32[$db>>2]|0;
 $33 = (($32) + 20|0);
 $34 = HEAP32[$33>>2]|0;
 $35 = ($31|0)>=($34|0);
 do {
  if ($35) {
   $36 = HEAP32[$zName>>2]|0;
   HEAP32[$vararg_buffer6>>2] = $36;
   (_sqlite3_snprintf(128,$zErr,19424,$vararg_buffer6)|0);
  } else {
   $37 = HEAP32[$i>>2]|0;
   $38 = ($37|0)<(2);
   if ($38) {
    $39 = HEAP32[$zName>>2]|0;
    HEAP32[$vararg_buffer6>>2] = $39;
    (_sqlite3_snprintf(128,$zErr,19448,$vararg_buffer6)|0);
    break;
   }
   $40 = HEAP32[$db>>2]|0;
   $41 = (($40) + 62|0);
   $42 = HEAP8[$41>>0]|0;
   $43 = ($42<<24>>24)!=(0);
   if (!($43)) {
    (_sqlite3_snprintf(128,$zErr,19480,$vararg_buffer6)|0);
    break;
   }
   $44 = HEAP32[$pDb>>2]|0;
   $45 = (($44) + 4|0);
   $46 = HEAP32[$45>>2]|0;
   $47 = (_sqlite3BtreeIsInReadTrans($46)|0);
   $48 = ($47|0)!=(0);
   if (!($48)) {
    $49 = HEAP32[$pDb>>2]|0;
    $50 = (($49) + 4|0);
    $51 = HEAP32[$50>>2]|0;
    $52 = (_sqlite3BtreeIsInBackup($51)|0);
    $53 = ($52|0)!=(0);
    if (!($53)) {
     $55 = HEAP32[$pDb>>2]|0;
     $56 = (($55) + 4|0);
     $57 = HEAP32[$56>>2]|0;
     (_sqlite3BtreeClose($57)|0);
     $58 = HEAP32[$pDb>>2]|0;
     $59 = (($58) + 4|0);
     HEAP32[$59>>2] = 0;
     $60 = HEAP32[$pDb>>2]|0;
     $61 = (($60) + 12|0);
     HEAP32[$61>>2] = 0;
     $62 = HEAP32[$db>>2]|0;
     _sqlite3ResetAllSchemasOfConnection($62);
     STACKTOP = sp;return;
    }
   }
   $54 = HEAP32[$zName>>2]|0;
   HEAP32[$vararg_buffer6>>2] = $54;
   (_sqlite3_snprintf(128,$zErr,19528,$vararg_buffer6)|0);
  }
 } while(0);
 $63 = HEAP32[$0>>2]|0;
 _sqlite3_result_error($63,$zErr,-1);
 STACKTOP = sp;return;
}
function _codeAttach($pParse,$type,$pFunc,$pAuthArg,$pFilename,$pDbname,$pKey) {
 $pParse = $pParse|0;
 $type = $type|0;
 $pFunc = $pFunc|0;
 $pAuthArg = $pAuthArg|0;
 $pFilename = $pFilename|0;
 $pDbname = $pDbname|0;
 $pKey = $pKey|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $9 = 0;
 var $db = 0, $rc = 0, $regArgs = 0, $sName = 0, $v = 0, $zAuthArg = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 80|0;
 $0 = sp + 76|0;
 $1 = sp + 72|0;
 $2 = sp + 68|0;
 $3 = sp + 64|0;
 $4 = sp + 60|0;
 $5 = sp + 56|0;
 $6 = sp + 52|0;
 $rc = sp + 48|0;
 $sName = sp + 16|0;
 $v = sp + 12|0;
 $db = sp + 8|0;
 $regArgs = sp + 4|0;
 $zAuthArg = sp;
 HEAP32[$0>>2] = $pParse;
 HEAP32[$1>>2] = $type;
 HEAP32[$2>>2] = $pFunc;
 HEAP32[$3>>2] = $pAuthArg;
 HEAP32[$4>>2] = $pFilename;
 HEAP32[$5>>2] = $pDbname;
 HEAP32[$6>>2] = $pKey;
 $7 = HEAP32[$0>>2]|0;
 $8 = HEAP32[$7>>2]|0;
 HEAP32[$db>>2] = $8;
 ;HEAP32[$sName+0>>2]=0|0;HEAP32[$sName+4>>2]=0|0;HEAP32[$sName+8>>2]=0|0;HEAP32[$sName+12>>2]=0|0;HEAP32[$sName+16>>2]=0|0;HEAP32[$sName+20>>2]=0|0;HEAP32[$sName+24>>2]=0|0;HEAP32[$sName+28>>2]=0|0;
 $9 = HEAP32[$0>>2]|0;
 HEAP32[$sName>>2] = $9;
 $10 = HEAP32[$4>>2]|0;
 $11 = (_resolveAttachExpr($sName,$10)|0);
 HEAP32[$rc>>2] = $11;
 $12 = (0)!=($11|0);
 do {
  if ($12) {
   label = 4;
  } else {
   $13 = HEAP32[$5>>2]|0;
   $14 = (_resolveAttachExpr($sName,$13)|0);
   HEAP32[$rc>>2] = $14;
   $15 = (0)!=($14|0);
   if ($15) {
    label = 4;
   } else {
    $16 = HEAP32[$6>>2]|0;
    $17 = (_resolveAttachExpr($sName,$16)|0);
    HEAP32[$rc>>2] = $17;
    $18 = (0)!=($17|0);
    if ($18) {
     label = 4;
    } else {
     $23 = HEAP32[$3>>2]|0;
     $24 = ($23|0)!=(0|0);
     if ($24) {
      $25 = HEAP32[$3>>2]|0;
      $26 = HEAP8[$25>>0]|0;
      $27 = $26&255;
      $28 = ($27|0)==(97);
      if ($28) {
       $29 = HEAP32[$3>>2]|0;
       $30 = (($29) + 8|0);
       $31 = HEAP32[$30>>2]|0;
       HEAP32[$zAuthArg>>2] = $31;
      } else {
       HEAP32[$zAuthArg>>2] = 0;
      }
      $32 = HEAP32[$0>>2]|0;
      $33 = HEAP32[$1>>2]|0;
      $34 = HEAP32[$zAuthArg>>2]|0;
      $35 = (_sqlite3AuthCheck($32,$33,$34,0,0)|0);
      HEAP32[$rc>>2] = $35;
      $36 = HEAP32[$rc>>2]|0;
      $37 = ($36|0)!=(0);
      if ($37) {
       break;
      }
     }
     $38 = HEAP32[$0>>2]|0;
     $39 = (_sqlite3GetVdbe($38)|0);
     HEAP32[$v>>2] = $39;
     $40 = HEAP32[$0>>2]|0;
     $41 = (_sqlite3GetTempRange($40,4)|0);
     HEAP32[$regArgs>>2] = $41;
     $42 = HEAP32[$0>>2]|0;
     $43 = HEAP32[$4>>2]|0;
     $44 = HEAP32[$regArgs>>2]|0;
     _sqlite3ExprCode($42,$43,$44);
     $45 = HEAP32[$0>>2]|0;
     $46 = HEAP32[$5>>2]|0;
     $47 = HEAP32[$regArgs>>2]|0;
     $48 = (($47) + 1)|0;
     _sqlite3ExprCode($45,$46,$48);
     $49 = HEAP32[$0>>2]|0;
     $50 = HEAP32[$6>>2]|0;
     $51 = HEAP32[$regArgs>>2]|0;
     $52 = (($51) + 2)|0;
     _sqlite3ExprCode($49,$50,$52);
     $53 = HEAP32[$v>>2]|0;
     $54 = ($53|0)!=(0|0);
     if ($54) {
      $55 = HEAP32[$v>>2]|0;
      $56 = HEAP32[$regArgs>>2]|0;
      $57 = (($56) + 3)|0;
      $58 = HEAP32[$2>>2]|0;
      $59 = HEAP16[$58>>1]|0;
      $60 = $59 << 16 >> 16;
      $61 = (($57) - ($60))|0;
      $62 = HEAP32[$regArgs>>2]|0;
      $63 = (($62) + 3)|0;
      (_sqlite3VdbeAddOp3($55,1,0,$61,$63)|0);
      $64 = HEAP32[$v>>2]|0;
      $65 = HEAP32[$2>>2]|0;
      $66 = HEAP16[$65>>1]|0;
      $67 = $66&255;
      _sqlite3VdbeChangeP5($64,$67);
      $68 = HEAP32[$v>>2]|0;
      $69 = HEAP32[$2>>2]|0;
      _sqlite3VdbeChangeP4($68,-1,$69,-5);
      $70 = HEAP32[$v>>2]|0;
      $71 = HEAP32[$1>>2]|0;
      $72 = ($71|0)==(24);
      $73 = $72&1;
      (_sqlite3VdbeAddOp1($70,142,$73)|0);
     }
    }
   }
  }
 } while(0);
 if ((label|0) == 4) {
  $19 = HEAP32[$0>>2]|0;
  $20 = (($19) + 64|0);
  $21 = HEAP32[$20>>2]|0;
  $22 = (($21) + 1)|0;
  HEAP32[$20>>2] = $22;
 }
 $74 = HEAP32[$db>>2]|0;
 $75 = HEAP32[$4>>2]|0;
 _sqlite3ExprDelete($74,$75);
 $76 = HEAP32[$db>>2]|0;
 $77 = HEAP32[$5>>2]|0;
 _sqlite3ExprDelete($76,$77);
 $78 = HEAP32[$db>>2]|0;
 $79 = HEAP32[$6>>2]|0;
 _sqlite3ExprDelete($78,$79);
 STACKTOP = sp;return;
}
function _resolveAttachExpr($pName,$pExpr) {
 $pName = $pName|0;
 $pExpr = $pExpr|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $1 = sp + 4|0;
 $rc = sp;
 HEAP32[$0>>2] = $pName;
 HEAP32[$1>>2] = $pExpr;
 HEAP32[$rc>>2] = 0;
 $2 = HEAP32[$1>>2]|0;
 $3 = ($2|0)!=(0|0);
 do {
  if ($3) {
   $4 = HEAP32[$1>>2]|0;
   $5 = HEAP8[$4>>0]|0;
   $6 = $5&255;
   $7 = ($6|0)!=(27);
   if ($7) {
    $8 = HEAP32[$0>>2]|0;
    $9 = HEAP32[$1>>2]|0;
    $10 = (_sqlite3ResolveExprNames($8,$9)|0);
    HEAP32[$rc>>2] = $10;
    break;
   } else {
    $11 = HEAP32[$1>>2]|0;
    HEAP8[$11>>0] = 97;
    break;
   }
  }
 } while(0);
 $12 = HEAP32[$rc>>2]|0;
 STACKTOP = sp;return ($12|0);
}
function _attachFunc($context,$NotUsed,$argv) {
 $context = $context|0;
 $NotUsed = $NotUsed|0;
 $argv = $argv|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0;
 var $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0, $188 = 0;
 var $189 = 0, $19 = 0, $190 = 0, $191 = 0, $192 = 0, $193 = 0, $194 = 0, $195 = 0, $196 = 0, $197 = 0, $198 = 0, $199 = 0, $2 = 0, $20 = 0, $200 = 0, $201 = 0, $202 = 0, $203 = 0, $204 = 0, $205 = 0;
 var $206 = 0, $207 = 0, $208 = 0, $209 = 0, $21 = 0, $210 = 0, $211 = 0, $212 = 0, $213 = 0, $214 = 0, $215 = 0, $216 = 0, $217 = 0, $218 = 0, $219 = 0, $22 = 0, $220 = 0, $221 = 0, $222 = 0, $223 = 0;
 var $224 = 0, $225 = 0, $226 = 0, $227 = 0, $228 = 0, $229 = 0, $23 = 0, $230 = 0, $231 = 0, $232 = 0, $233 = 0, $234 = 0, $235 = 0, $236 = 0, $237 = 0, $238 = 0, $239 = 0, $24 = 0, $240 = 0, $241 = 0;
 var $242 = 0, $243 = 0, $244 = 0, $245 = 0, $246 = 0, $247 = 0, $248 = 0, $249 = 0, $25 = 0, $250 = 0, $251 = 0, $252 = 0, $253 = 0, $254 = 0, $255 = 0, $256 = 0, $257 = 0, $258 = 0, $259 = 0, $26 = 0;
 var $260 = 0, $261 = 0, $262 = 0, $263 = 0, $264 = 0, $265 = 0, $266 = 0, $267 = 0, $268 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0;
 var $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0;
 var $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0;
 var $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0;
 var $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $aNew = 0, $db = 0, $flags = 0, $i = 0, $iDb = 0, $or$cond = 0, $pPager = 0, $pVfs = 0, $rc = 0, $vararg_buffer13 = 0, $z = 0;
 var $zErr = 0, $zErrDyn = 0, $zFile = 0, $zName = 0, $zPath = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 80|0;
 $vararg_buffer13 = sp;
 $0 = sp + 68|0;
 $1 = sp + 64|0;
 $2 = sp + 60|0;
 $i = sp + 56|0;
 $rc = sp + 52|0;
 $db = sp + 48|0;
 $zName = sp + 44|0;
 $zFile = sp + 40|0;
 $zPath = sp + 36|0;
 $zErr = sp + 32|0;
 $flags = sp + 28|0;
 $aNew = sp + 24|0;
 $zErrDyn = sp + 20|0;
 $pVfs = sp + 16|0;
 $z = sp + 12|0;
 $pPager = sp + 8|0;
 $iDb = sp + 4|0;
 HEAP32[$0>>2] = $context;
 HEAP32[$1>>2] = $NotUsed;
 HEAP32[$2>>2] = $argv;
 HEAP32[$rc>>2] = 0;
 $3 = HEAP32[$0>>2]|0;
 $4 = (_sqlite3_context_db_handle($3)|0);
 HEAP32[$db>>2] = $4;
 HEAP32[$zPath>>2] = 0;
 HEAP32[$zErr>>2] = 0;
 HEAP32[$zErrDyn>>2] = 0;
 $5 = HEAP32[$2>>2]|0;
 $6 = HEAP32[$5>>2]|0;
 $7 = (_sqlite3_value_text($6)|0);
 HEAP32[$zFile>>2] = $7;
 $8 = HEAP32[$2>>2]|0;
 $9 = (($8) + 4|0);
 $10 = HEAP32[$9>>2]|0;
 $11 = (_sqlite3_value_text($10)|0);
 HEAP32[$zName>>2] = $11;
 $12 = HEAP32[$zFile>>2]|0;
 $13 = ($12|0)==(0|0);
 if ($13) {
  HEAP32[$zFile>>2] = 1216;
 }
 $14 = HEAP32[$zName>>2]|0;
 $15 = ($14|0)==(0|0);
 if ($15) {
  HEAP32[$zName>>2] = 1216;
 }
 $16 = HEAP32[$db>>2]|0;
 $17 = (($16) + 20|0);
 $18 = HEAP32[$17>>2]|0;
 $19 = HEAP32[$db>>2]|0;
 $20 = (($19) + 88|0);
 $21 = (($20) + 28|0);
 $22 = HEAP32[$21>>2]|0;
 $23 = (($22) + 2)|0;
 $24 = ($18|0)>=($23|0);
 $25 = HEAP32[$db>>2]|0;
 do {
  if ($24) {
   $26 = HEAP32[$db>>2]|0;
   $27 = (($26) + 88|0);
   $28 = (($27) + 28|0);
   $29 = HEAP32[$28>>2]|0;
   HEAP32[$vararg_buffer13>>2] = $29;
   $30 = (_sqlite3MPrintf($25,19608,$vararg_buffer13)|0);
   HEAP32[$zErrDyn>>2] = $30;
  } else {
   $31 = (($25) + 62|0);
   $32 = HEAP8[$31>>0]|0;
   $33 = ($32<<24>>24)!=(0);
   if (!($33)) {
    $34 = HEAP32[$db>>2]|0;
    $35 = (_sqlite3MPrintf($34,19648,$vararg_buffer13)|0);
    HEAP32[$zErrDyn>>2] = $35;
    break;
   }
   HEAP32[$i>>2] = 0;
   while(1) {
    $36 = HEAP32[$i>>2]|0;
    $37 = HEAP32[$db>>2]|0;
    $38 = (($37) + 20|0);
    $39 = HEAP32[$38>>2]|0;
    $40 = ($36|0)<($39|0);
    if (!($40)) {
     break;
    }
    $41 = HEAP32[$i>>2]|0;
    $42 = HEAP32[$db>>2]|0;
    $43 = (($42) + 16|0);
    $44 = HEAP32[$43>>2]|0;
    $45 = (($44) + ($41<<4)|0);
    $46 = HEAP32[$45>>2]|0;
    HEAP32[$z>>2] = $46;
    $47 = HEAP32[$z>>2]|0;
    $48 = HEAP32[$zName>>2]|0;
    $49 = (_sqlite3_stricmp($47,$48)|0);
    $50 = ($49|0)==(0);
    if ($50) {
     label = 12;
     break;
    }
    $54 = HEAP32[$i>>2]|0;
    $55 = (($54) + 1)|0;
    HEAP32[$i>>2] = $55;
   }
   if ((label|0) == 12) {
    $51 = HEAP32[$db>>2]|0;
    $52 = HEAP32[$zName>>2]|0;
    HEAP32[$vararg_buffer13>>2] = $52;
    $53 = (_sqlite3MPrintf($51,19696,$vararg_buffer13)|0);
    HEAP32[$zErrDyn>>2] = $53;
    break;
   }
   $56 = HEAP32[$db>>2]|0;
   $57 = (($56) + 16|0);
   $58 = HEAP32[$57>>2]|0;
   $59 = HEAP32[$db>>2]|0;
   $60 = (($59) + 456|0);
   $61 = ($58|0)==($60|0);
   $62 = HEAP32[$db>>2]|0;
   do {
    if ($61) {
     $63 = (_sqlite3DbMallocRaw($62,48,0)|0);
     HEAP32[$aNew>>2] = $63;
     $64 = HEAP32[$aNew>>2]|0;
     $65 = ($64|0)==(0|0);
     if ($65) {
      STACKTOP = sp;return;
     } else {
      $66 = HEAP32[$aNew>>2]|0;
      $67 = HEAP32[$db>>2]|0;
      $68 = (($67) + 16|0);
      $69 = HEAP32[$68>>2]|0;
      ;HEAP32[$66+0>>2]=HEAP32[$69+0>>2]|0;HEAP32[$66+4>>2]=HEAP32[$69+4>>2]|0;HEAP32[$66+8>>2]=HEAP32[$69+8>>2]|0;HEAP32[$66+12>>2]=HEAP32[$69+12>>2]|0;HEAP32[$66+16>>2]=HEAP32[$69+16>>2]|0;HEAP32[$66+20>>2]=HEAP32[$69+20>>2]|0;HEAP32[$66+24>>2]=HEAP32[$69+24>>2]|0;HEAP32[$66+28>>2]=HEAP32[$69+28>>2]|0;
      break;
     }
    } else {
     $70 = HEAP32[$db>>2]|0;
     $71 = (($70) + 16|0);
     $72 = HEAP32[$71>>2]|0;
     $73 = HEAP32[$db>>2]|0;
     $74 = (($73) + 20|0);
     $75 = HEAP32[$74>>2]|0;
     $76 = (($75) + 1)|0;
     $77 = $76<<4;
     $78 = (_sqlite3DbRealloc($62,$72,$77,0)|0);
     HEAP32[$aNew>>2] = $78;
     $79 = HEAP32[$aNew>>2]|0;
     $80 = ($79|0)==(0|0);
     if ($80) {
      STACKTOP = sp;return;
     }
    }
   } while(0);
   $81 = HEAP32[$aNew>>2]|0;
   $82 = HEAP32[$db>>2]|0;
   $83 = (($82) + 16|0);
   HEAP32[$83>>2] = $81;
   $84 = HEAP32[$db>>2]|0;
   $85 = (($84) + 20|0);
   $86 = HEAP32[$85>>2]|0;
   $87 = HEAP32[$db>>2]|0;
   $88 = (($87) + 16|0);
   $89 = HEAP32[$88>>2]|0;
   $90 = (($89) + ($86<<4)|0);
   HEAP32[$aNew>>2] = $90;
   $91 = HEAP32[$aNew>>2]|0;
   ;HEAP32[$91+0>>2]=0|0;HEAP32[$91+4>>2]=0|0;HEAP32[$91+8>>2]=0|0;HEAP32[$91+12>>2]=0|0;
   $92 = HEAP32[$db>>2]|0;
   $93 = (($92) + 48|0);
   $94 = HEAP32[$93>>2]|0;
   HEAP32[$flags>>2] = $94;
   $95 = HEAP32[$db>>2]|0;
   $96 = HEAP32[$95>>2]|0;
   $97 = (($96) + 16|0);
   $98 = HEAP32[$97>>2]|0;
   $99 = HEAP32[$zFile>>2]|0;
   $100 = (_sqlite3ParseUri($98,$99,$flags,$pVfs,$zPath,$zErr)|0);
   HEAP32[$rc>>2] = $100;
   $101 = HEAP32[$rc>>2]|0;
   $102 = ($101|0)!=(0);
   if ($102) {
    $103 = HEAP32[$rc>>2]|0;
    $104 = ($103|0)==(7);
    if ($104) {
     $105 = HEAP32[$db>>2]|0;
     $106 = (($105) + 64|0);
     HEAP8[$106>>0] = 1;
    }
    $107 = HEAP32[$0>>2]|0;
    $108 = HEAP32[$zErr>>2]|0;
    _sqlite3_result_error($107,$108,-1);
    $109 = HEAP32[$zErr>>2]|0;
    _sqlite3_free($109);
    STACKTOP = sp;return;
   }
   $110 = HEAP32[$flags>>2]|0;
   $111 = $110 | 256;
   HEAP32[$flags>>2] = $111;
   $112 = HEAP32[$pVfs>>2]|0;
   $113 = HEAP32[$zPath>>2]|0;
   $114 = HEAP32[$db>>2]|0;
   $115 = HEAP32[$aNew>>2]|0;
   $116 = (($115) + 4|0);
   $117 = HEAP32[$flags>>2]|0;
   $118 = (_sqlite3BtreeOpen($112,$113,$114,$116,0,$117)|0);
   HEAP32[$rc>>2] = $118;
   $119 = HEAP32[$zPath>>2]|0;
   _sqlite3_free($119);
   $120 = HEAP32[$db>>2]|0;
   $121 = (($120) + 20|0);
   $122 = HEAP32[$121>>2]|0;
   $123 = (($122) + 1)|0;
   HEAP32[$121>>2] = $123;
   $124 = HEAP32[$rc>>2]|0;
   $125 = ($124|0)==(19);
   if ($125) {
    HEAP32[$rc>>2] = 1;
    $126 = HEAP32[$db>>2]|0;
    $127 = (_sqlite3MPrintf($126,19728,$vararg_buffer13)|0);
    HEAP32[$zErrDyn>>2] = $127;
   } else {
    $128 = HEAP32[$rc>>2]|0;
    $129 = ($128|0)==(0);
    if ($129) {
     $130 = HEAP32[$db>>2]|0;
     $131 = HEAP32[$aNew>>2]|0;
     $132 = (($131) + 4|0);
     $133 = HEAP32[$132>>2]|0;
     $134 = (_sqlite3SchemaGet($130,$133)|0);
     $135 = HEAP32[$aNew>>2]|0;
     $136 = (($135) + 12|0);
     HEAP32[$136>>2] = $134;
     $137 = HEAP32[$aNew>>2]|0;
     $138 = (($137) + 12|0);
     $139 = HEAP32[$138>>2]|0;
     $140 = ($139|0)!=(0|0);
     if ($140) {
      $141 = HEAP32[$aNew>>2]|0;
      $142 = (($141) + 12|0);
      $143 = HEAP32[$142>>2]|0;
      $144 = (($143) + 76|0);
      $145 = HEAP8[$144>>0]|0;
      $146 = $145&255;
      $147 = ($146|0)!=(0);
      if ($147) {
       $148 = HEAP32[$aNew>>2]|0;
       $149 = (($148) + 12|0);
       $150 = HEAP32[$149>>2]|0;
       $151 = (($150) + 77|0);
       $152 = HEAP8[$151>>0]|0;
       $153 = $152&255;
       $154 = HEAP32[$db>>2]|0;
       $155 = (($154) + 16|0);
       $156 = HEAP32[$155>>2]|0;
       $157 = (($156) + 12|0);
       $158 = HEAP32[$157>>2]|0;
       $159 = (($158) + 77|0);
       $160 = HEAP8[$159>>0]|0;
       $161 = $160&255;
       $162 = ($153|0)!=($161|0);
       if ($162) {
        $163 = HEAP32[$db>>2]|0;
        $164 = (_sqlite3MPrintf($163,18744,$vararg_buffer13)|0);
        HEAP32[$zErrDyn>>2] = $164;
        HEAP32[$rc>>2] = 1;
       }
      }
     } else {
      HEAP32[$rc>>2] = 7;
     }
     $165 = HEAP32[$aNew>>2]|0;
     $166 = (($165) + 4|0);
     $167 = HEAP32[$166>>2]|0;
     $168 = (_sqlite3BtreePager($167)|0);
     HEAP32[$pPager>>2] = $168;
     $169 = HEAP32[$pPager>>2]|0;
     $170 = HEAP32[$db>>2]|0;
     $171 = (($170) + 65|0);
     $172 = HEAP8[$171>>0]|0;
     $173 = $172&255;
     (_sqlite3PagerLockingMode($169,$173)|0);
     $174 = HEAP32[$aNew>>2]|0;
     $175 = (($174) + 4|0);
     $176 = HEAP32[$175>>2]|0;
     $177 = HEAP32[$db>>2]|0;
     $178 = (($177) + 16|0);
     $179 = HEAP32[$178>>2]|0;
     $180 = (($179) + 4|0);
     $181 = HEAP32[$180>>2]|0;
     $182 = (_sqlite3BtreeSecureDelete($181,-1)|0);
     (_sqlite3BtreeSecureDelete($176,$182)|0);
     $183 = HEAP32[$aNew>>2]|0;
     $184 = (($183) + 4|0);
     $185 = HEAP32[$184>>2]|0;
     $186 = HEAP32[$db>>2]|0;
     $187 = (($186) + 24|0);
     $188 = HEAP32[$187>>2]|0;
     $189 = $188 & 28;
     $190 = 3 | $189;
     (_sqlite3BtreeSetPagerFlags($185,$190)|0);
    }
   }
   $191 = HEAP32[$aNew>>2]|0;
   $192 = (($191) + 8|0);
   HEAP8[$192>>0] = 3;
   $193 = HEAP32[$db>>2]|0;
   $194 = HEAP32[$zName>>2]|0;
   $195 = (_sqlite3DbStrDup($193,$194)|0);
   $196 = HEAP32[$aNew>>2]|0;
   HEAP32[$196>>2] = $195;
   $197 = HEAP32[$rc>>2]|0;
   $198 = ($197|0)==(0);
   if ($198) {
    $199 = HEAP32[$aNew>>2]|0;
    $200 = HEAP32[$199>>2]|0;
    $201 = ($200|0)==(0|0);
    if ($201) {
     HEAP32[$rc>>2] = 7;
    }
   }
   $202 = HEAP32[$rc>>2]|0;
   $203 = ($202|0)==(0);
   if ($203) {
    $204 = HEAP32[$db>>2]|0;
    _sqlite3BtreeEnterAll($204);
    $205 = HEAP32[$db>>2]|0;
    $206 = (_sqlite3Init($205,$zErrDyn)|0);
    HEAP32[$rc>>2] = $206;
   }
   $207 = HEAP32[$rc>>2]|0;
   $208 = ($207|0)!=(0);
   if (!($208)) {
    STACKTOP = sp;return;
   }
   $209 = HEAP32[$db>>2]|0;
   $210 = (($209) + 20|0);
   $211 = HEAP32[$210>>2]|0;
   $212 = (($211) - 1)|0;
   HEAP32[$iDb>>2] = $212;
   $213 = HEAP32[$iDb>>2]|0;
   $214 = HEAP32[$db>>2]|0;
   $215 = (($214) + 16|0);
   $216 = HEAP32[$215>>2]|0;
   $217 = (($216) + ($213<<4)|0);
   $218 = (($217) + 4|0);
   $219 = HEAP32[$218>>2]|0;
   $220 = ($219|0)!=(0|0);
   if ($220) {
    $221 = HEAP32[$iDb>>2]|0;
    $222 = HEAP32[$db>>2]|0;
    $223 = (($222) + 16|0);
    $224 = HEAP32[$223>>2]|0;
    $225 = (($224) + ($221<<4)|0);
    $226 = (($225) + 4|0);
    $227 = HEAP32[$226>>2]|0;
    (_sqlite3BtreeClose($227)|0);
    $228 = HEAP32[$iDb>>2]|0;
    $229 = HEAP32[$db>>2]|0;
    $230 = (($229) + 16|0);
    $231 = HEAP32[$230>>2]|0;
    $232 = (($231) + ($228<<4)|0);
    $233 = (($232) + 4|0);
    HEAP32[$233>>2] = 0;
    $234 = HEAP32[$iDb>>2]|0;
    $235 = HEAP32[$db>>2]|0;
    $236 = (($235) + 16|0);
    $237 = HEAP32[$236>>2]|0;
    $238 = (($237) + ($234<<4)|0);
    $239 = (($238) + 12|0);
    HEAP32[$239>>2] = 0;
   }
   $240 = HEAP32[$db>>2]|0;
   _sqlite3ResetAllSchemasOfConnection($240);
   $241 = HEAP32[$iDb>>2]|0;
   $242 = HEAP32[$db>>2]|0;
   $243 = (($242) + 20|0);
   HEAP32[$243>>2] = $241;
   $244 = HEAP32[$rc>>2]|0;
   $245 = ($244|0)==(7);
   $246 = HEAP32[$rc>>2]|0;
   $247 = ($246|0)==(3082);
   $or$cond = $245 | $247;
   if ($or$cond) {
    $248 = HEAP32[$db>>2]|0;
    $249 = (($248) + 64|0);
    HEAP8[$249>>0] = 1;
    $250 = HEAP32[$db>>2]|0;
    $251 = HEAP32[$zErrDyn>>2]|0;
    _sqlite3DbFree($250,$251);
    $252 = HEAP32[$db>>2]|0;
    $253 = (_sqlite3MPrintf($252,5400,$vararg_buffer13)|0);
    HEAP32[$zErrDyn>>2] = $253;
    break;
   }
   $254 = HEAP32[$zErrDyn>>2]|0;
   $255 = ($254|0)==(0|0);
   if ($255) {
    $256 = HEAP32[$db>>2]|0;
    $257 = HEAP32[$zFile>>2]|0;
    HEAP32[$vararg_buffer13>>2] = $257;
    $258 = (_sqlite3MPrintf($256,19760,$vararg_buffer13)|0);
    HEAP32[$zErrDyn>>2] = $258;
   }
  }
 } while(0);
 $259 = HEAP32[$zErrDyn>>2]|0;
 $260 = ($259|0)!=(0|0);
 if ($260) {
  $261 = HEAP32[$0>>2]|0;
  $262 = HEAP32[$zErrDyn>>2]|0;
  _sqlite3_result_error($261,$262,-1);
  $263 = HEAP32[$db>>2]|0;
  $264 = HEAP32[$zErrDyn>>2]|0;
  _sqlite3DbFree($263,$264);
 }
 $265 = HEAP32[$rc>>2]|0;
 $266 = ($265|0)!=(0);
 if (!($266)) {
  STACKTOP = sp;return;
 }
 $267 = HEAP32[$0>>2]|0;
 $268 = HEAP32[$rc>>2]|0;
 _sqlite3_result_error_code($267,$268);
 STACKTOP = sp;return;
}
function _sqlite3PagerLockingMode($pPager,$eMode) {
 $pPager = $pPager|0;
 $eMode = $eMode|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0;
 var $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $1 = sp;
 HEAP32[$0>>2] = $pPager;
 HEAP32[$1>>2] = $eMode;
 $2 = HEAP32[$1>>2]|0;
 $3 = ($2|0)>=(0);
 if ($3) {
  $4 = HEAP32[$0>>2]|0;
  $5 = (($4) + 12|0);
  $6 = HEAP8[$5>>0]|0;
  $7 = ($6<<24>>24)!=(0);
  if (!($7)) {
   $8 = HEAP32[$0>>2]|0;
   $9 = (($8) + 208|0);
   $10 = HEAP32[$9>>2]|0;
   $11 = (_sqlite3WalHeapMemory($10)|0);
   $12 = ($11|0)!=(0);
   if (!($12)) {
    $13 = HEAP32[$1>>2]|0;
    $14 = $13&255;
    $15 = HEAP32[$0>>2]|0;
    $16 = (($15) + 4|0);
    HEAP8[$16>>0] = $14;
   }
  }
 }
 $17 = HEAP32[$0>>2]|0;
 $18 = (($17) + 4|0);
 $19 = HEAP8[$18>>0]|0;
 $20 = $19&255;
 STACKTOP = sp;return ($20|0);
}
function _sqlite3BtreeSecureDelete($p,$newFlag) {
 $p = $p|0;
 $newFlag = $newFlag|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $b = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $1 = sp + 8|0;
 $2 = sp + 4|0;
 $b = sp;
 HEAP32[$1>>2] = $p;
 HEAP32[$2>>2] = $newFlag;
 $3 = HEAP32[$1>>2]|0;
 $4 = ($3|0)==(0|0);
 if ($4) {
  HEAP32[$0>>2] = 0;
  $36 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($36|0);
 }
 $5 = HEAP32[$1>>2]|0;
 _sqlite3BtreeEnter($5);
 $6 = HEAP32[$2>>2]|0;
 $7 = ($6|0)>=(0);
 if ($7) {
  $8 = HEAP32[$1>>2]|0;
  $9 = (($8) + 4|0);
  $10 = HEAP32[$9>>2]|0;
  $11 = (($10) + 22|0);
  $12 = HEAP16[$11>>1]|0;
  $13 = $12&65535;
  $14 = $13 & -5;
  $15 = $14&65535;
  HEAP16[$11>>1] = $15;
  $16 = HEAP32[$2>>2]|0;
  $17 = ($16|0)!=(0);
  if ($17) {
   $18 = HEAP32[$1>>2]|0;
   $19 = (($18) + 4|0);
   $20 = HEAP32[$19>>2]|0;
   $21 = (($20) + 22|0);
   $22 = HEAP16[$21>>1]|0;
   $23 = $22&65535;
   $24 = $23 | 4;
   $25 = $24&65535;
   HEAP16[$21>>1] = $25;
  }
 }
 $26 = HEAP32[$1>>2]|0;
 $27 = (($26) + 4|0);
 $28 = HEAP32[$27>>2]|0;
 $29 = (($28) + 22|0);
 $30 = HEAP16[$29>>1]|0;
 $31 = $30&65535;
 $32 = $31 & 4;
 $33 = ($32|0)!=(0);
 $34 = $33&1;
 HEAP32[$b>>2] = $34;
 $35 = HEAP32[$b>>2]|0;
 HEAP32[$0>>2] = $35;
 $36 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($36|0);
}
function _sqlite3BtreeSetPagerFlags($p,$pgFlags) {
 $p = $p|0;
 $pgFlags = $pgFlags|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $pBt = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $1 = sp + 4|0;
 $pBt = sp;
 HEAP32[$0>>2] = $p;
 HEAP32[$1>>2] = $pgFlags;
 $2 = HEAP32[$0>>2]|0;
 $3 = (($2) + 4|0);
 $4 = HEAP32[$3>>2]|0;
 HEAP32[$pBt>>2] = $4;
 $5 = HEAP32[$0>>2]|0;
 _sqlite3BtreeEnter($5);
 $6 = HEAP32[$pBt>>2]|0;
 $7 = HEAP32[$6>>2]|0;
 $8 = HEAP32[$1>>2]|0;
 _sqlite3PagerSetFlags($7,$8);
 STACKTOP = sp;return 0;
}
function _sqlite3PagerSetFlags($pPager,$pgFlags) {
 $pPager = $pPager|0;
 $pgFlags = $pgFlags|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $8 = 0, $9 = 0, $level = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $1 = sp + 4|0;
 $level = sp;
 HEAP32[$0>>2] = $pPager;
 HEAP32[$1>>2] = $pgFlags;
 $2 = HEAP32[$1>>2]|0;
 $3 = $2 & 3;
 HEAP32[$level>>2] = $3;
 $4 = HEAP32[$level>>2]|0;
 $5 = ($4|0)==(1);
 if ($5) {
  $12 = 1;
 } else {
  $6 = HEAP32[$0>>2]|0;
  $7 = (($6) + 12|0);
  $8 = HEAP8[$7>>0]|0;
  $9 = $8&255;
  $10 = ($9|0)!=(0);
  $12 = $10;
 }
 $11 = $12 ? 1 : 0;
 $13 = $11&255;
 $14 = HEAP32[$0>>2]|0;
 $15 = (($14) + 7|0);
 HEAP8[$15>>0] = $13;
 $16 = HEAP32[$level>>2]|0;
 $17 = ($16|0)==(3);
 if ($17) {
  $18 = HEAP32[$0>>2]|0;
  $19 = (($18) + 12|0);
  $20 = HEAP8[$19>>0]|0;
  $21 = ($20<<24>>24)!=(0);
  $22 = $21 ^ 1;
  $24 = $22;
 } else {
  $24 = 0;
 }
 $23 = $24 ? 1 : 0;
 $25 = $23&255;
 $26 = HEAP32[$0>>2]|0;
 $27 = (($26) + 8|0);
 HEAP8[$27>>0] = $25;
 $28 = HEAP32[$0>>2]|0;
 $29 = (($28) + 7|0);
 $30 = HEAP8[$29>>0]|0;
 $31 = ($30<<24>>24)!=(0);
 do {
  if ($31) {
   $32 = HEAP32[$0>>2]|0;
   $33 = (($32) + 11|0);
   HEAP8[$33>>0] = 0;
   $34 = HEAP32[$0>>2]|0;
   $35 = (($34) + 9|0);
   HEAP8[$35>>0] = 0;
  } else {
   $36 = HEAP32[$1>>2]|0;
   $37 = $36 & 4;
   $38 = ($37|0)!=(0);
   if ($38) {
    $39 = HEAP32[$0>>2]|0;
    $40 = (($39) + 11|0);
    HEAP8[$40>>0] = 3;
    $41 = HEAP32[$0>>2]|0;
    $42 = (($41) + 9|0);
    HEAP8[$42>>0] = 3;
    break;
   }
   $43 = HEAP32[$1>>2]|0;
   $44 = $43 & 8;
   $45 = ($44|0)!=(0);
   $46 = HEAP32[$0>>2]|0;
   $47 = (($46) + 11|0);
   HEAP8[$47>>0] = 2;
   $48 = HEAP32[$0>>2]|0;
   $49 = (($48) + 9|0);
   if ($45) {
    HEAP8[$49>>0] = 3;
    break;
   } else {
    HEAP8[$49>>0] = 2;
    break;
   }
  }
 } while(0);
 $50 = HEAP32[$0>>2]|0;
 $51 = (($50) + 11|0);
 $52 = HEAP8[$51>>0]|0;
 $53 = HEAP32[$0>>2]|0;
 $54 = (($53) + 10|0);
 HEAP8[$54>>0] = $52;
 $55 = HEAP32[$0>>2]|0;
 $56 = (($55) + 8|0);
 $57 = HEAP8[$56>>0]|0;
 $58 = ($57<<24>>24)!=(0);
 if ($58) {
  $59 = HEAP32[$0>>2]|0;
  $60 = (($59) + 10|0);
  $61 = HEAP8[$60>>0]|0;
  $62 = $61&255;
  $63 = $62 | 32;
  $64 = $63&255;
  HEAP8[$60>>0] = $64;
 }
 $65 = HEAP32[$1>>2]|0;
 $66 = $65 & 16;
 $67 = ($66|0)!=(0);
 $68 = HEAP32[$0>>2]|0;
 $69 = (($68) + 20|0);
 $70 = HEAP8[$69>>0]|0;
 $71 = $70&255;
 if ($67) {
  $72 = $71 & -2;
  $73 = $72&255;
  HEAP8[$69>>0] = $73;
  STACKTOP = sp;return;
 } else {
  $74 = $71 | 1;
  $75 = $74&255;
  HEAP8[$69>>0] = $75;
  STACKTOP = sp;return;
 }
}
function _sqlite3WalHeapMemory($pWal) {
 $pWal = $pWal|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 HEAP32[$0>>2] = $pWal;
 $1 = HEAP32[$0>>2]|0;
 $2 = ($1|0)!=(0|0);
 if (!($2)) {
  $9 = 0;
  $8 = $9&1;
  STACKTOP = sp;return ($8|0);
 }
 $3 = HEAP32[$0>>2]|0;
 $4 = (($3) + 43|0);
 $5 = HEAP8[$4>>0]|0;
 $6 = $5&255;
 $7 = ($6|0)==(2);
 $9 = $7;
 $8 = $9&1;
 STACKTOP = sp;return ($8|0);
}
function _sqlite3CodeVerifyNamedSchema($pParse,$zDb) {
 $pParse = $pParse|0;
 $zDb = $zDb|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $db = 0, $i = 0, $pDb = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 16|0;
 $1 = sp + 12|0;
 $db = sp + 8|0;
 $i = sp + 4|0;
 $pDb = sp;
 HEAP32[$0>>2] = $pParse;
 HEAP32[$1>>2] = $zDb;
 $2 = HEAP32[$0>>2]|0;
 $3 = HEAP32[$2>>2]|0;
 HEAP32[$db>>2] = $3;
 HEAP32[$i>>2] = 0;
 while(1) {
  $4 = HEAP32[$i>>2]|0;
  $5 = HEAP32[$db>>2]|0;
  $6 = (($5) + 20|0);
  $7 = HEAP32[$6>>2]|0;
  $8 = ($4|0)<($7|0);
  if (!($8)) {
   break;
  }
  $9 = HEAP32[$i>>2]|0;
  $10 = HEAP32[$db>>2]|0;
  $11 = (($10) + 16|0);
  $12 = HEAP32[$11>>2]|0;
  $13 = (($12) + ($9<<4)|0);
  HEAP32[$pDb>>2] = $13;
  $14 = HEAP32[$pDb>>2]|0;
  $15 = (($14) + 4|0);
  $16 = HEAP32[$15>>2]|0;
  $17 = ($16|0)!=(0|0);
  do {
   if ($17) {
    $18 = HEAP32[$1>>2]|0;
    $19 = ($18|0)!=(0|0);
    if ($19) {
     $20 = HEAP32[$1>>2]|0;
     $21 = HEAP32[$pDb>>2]|0;
     $22 = HEAP32[$21>>2]|0;
     $23 = (_sqlite3_stricmp($20,$22)|0);
     $24 = (0)==($23|0);
     if (!($24)) {
      break;
     }
    }
    $25 = HEAP32[$0>>2]|0;
    $26 = HEAP32[$i>>2]|0;
    _sqlite3CodeVerifySchema($25,$26);
   }
  } while(0);
  $27 = HEAP32[$i>>2]|0;
  $28 = (($27) + 1)|0;
  HEAP32[$i>>2] = $28;
 }
 STACKTOP = sp;return;
}
function _sqlite3DropTriggerPtr($pParse,$pTrigger) {
 $pParse = $pParse|0;
 $pTrigger = $pTrigger|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $7 = 0, $8 = 0, $9 = 0, $base = 0, $code = 0, $db = 0, $iDb = 0, $pTable = 0, $v = 0, $zDb = 0, $zTab = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $0 = sp + 36|0;
 $1 = sp + 32|0;
 $pTable = sp + 28|0;
 $v = sp + 24|0;
 $db = sp + 20|0;
 $iDb = sp + 16|0;
 $code = sp + 12|0;
 $zDb = sp + 8|0;
 $zTab = sp + 4|0;
 $base = sp;
 HEAP32[$0>>2] = $pParse;
 HEAP32[$1>>2] = $pTrigger;
 $2 = HEAP32[$0>>2]|0;
 $3 = HEAP32[$2>>2]|0;
 HEAP32[$db>>2] = $3;
 $4 = HEAP32[$0>>2]|0;
 $5 = HEAP32[$4>>2]|0;
 $6 = HEAP32[$1>>2]|0;
 $7 = (($6) + 20|0);
 $8 = HEAP32[$7>>2]|0;
 $9 = (_sqlite3SchemaToIndex($5,$8)|0);
 HEAP32[$iDb>>2] = $9;
 $10 = HEAP32[$1>>2]|0;
 $11 = (_tableOfTrigger($10)|0);
 HEAP32[$pTable>>2] = $11;
 HEAP32[$code>>2] = 16;
 $12 = HEAP32[$iDb>>2]|0;
 $13 = HEAP32[$db>>2]|0;
 $14 = (($13) + 16|0);
 $15 = HEAP32[$14>>2]|0;
 $16 = (($15) + ($12<<4)|0);
 $17 = HEAP32[$16>>2]|0;
 HEAP32[$zDb>>2] = $17;
 $18 = HEAP32[$iDb>>2]|0;
 $19 = ($18|0)==(1);
 $20 = $19 ? 14904 : 14928;
 HEAP32[$zTab>>2] = $20;
 $21 = HEAP32[$iDb>>2]|0;
 $22 = ($21|0)==(1);
 if ($22) {
  HEAP32[$code>>2] = 14;
 }
 $23 = HEAP32[$0>>2]|0;
 $24 = HEAP32[$code>>2]|0;
 $25 = HEAP32[$1>>2]|0;
 $26 = HEAP32[$25>>2]|0;
 $27 = HEAP32[$pTable>>2]|0;
 $28 = HEAP32[$27>>2]|0;
 $29 = HEAP32[$zDb>>2]|0;
 $30 = (_sqlite3AuthCheck($23,$24,$26,$28,$29)|0);
 $31 = ($30|0)!=(0);
 if ($31) {
  STACKTOP = sp;return;
 }
 $32 = HEAP32[$0>>2]|0;
 $33 = HEAP32[$zTab>>2]|0;
 $34 = HEAP32[$zDb>>2]|0;
 $35 = (_sqlite3AuthCheck($32,9,$33,0,$34)|0);
 $36 = ($35|0)!=(0);
 if ($36) {
  STACKTOP = sp;return;
 }
 $37 = HEAP32[$0>>2]|0;
 $38 = (_sqlite3GetVdbe($37)|0);
 HEAP32[$v>>2] = $38;
 $39 = ($38|0)!=(0|0);
 if (!($39)) {
  STACKTOP = sp;return;
 }
 $40 = HEAP32[$0>>2]|0;
 $41 = HEAP32[$iDb>>2]|0;
 _sqlite3BeginWriteOperation($40,0,$41);
 $42 = HEAP32[$0>>2]|0;
 $43 = HEAP32[$iDb>>2]|0;
 _sqlite3OpenMasterTable($42,$43);
 $44 = HEAP32[$v>>2]|0;
 $45 = (_sqlite3VdbeAddOpList($44,9,19816,0)|0);
 HEAP32[$base>>2] = $45;
 $46 = HEAP32[$v>>2]|0;
 $47 = HEAP32[$base>>2]|0;
 $48 = (($47) + 1)|0;
 $49 = HEAP32[$1>>2]|0;
 $50 = HEAP32[$49>>2]|0;
 _sqlite3VdbeChangeP4($46,$48,$50,0);
 $51 = HEAP32[$v>>2]|0;
 $52 = HEAP32[$base>>2]|0;
 $53 = (($52) + 4)|0;
 _sqlite3VdbeChangeP4($51,$53,1488,-2);
 $54 = HEAP32[$0>>2]|0;
 $55 = HEAP32[$iDb>>2]|0;
 _sqlite3ChangeCookie($54,$55);
 $56 = HEAP32[$v>>2]|0;
 (_sqlite3VdbeAddOp2($56,61,0,0)|0);
 $57 = HEAP32[$v>>2]|0;
 $58 = HEAP32[$iDb>>2]|0;
 $59 = HEAP32[$1>>2]|0;
 $60 = HEAP32[$59>>2]|0;
 (_sqlite3VdbeAddOp4($57,126,$58,0,0,$60,0)|0);
 $61 = HEAP32[$0>>2]|0;
 $62 = (($61) + 72|0);
 $63 = HEAP32[$62>>2]|0;
 $64 = ($63|0)<(3);
 if (!($64)) {
  STACKTOP = sp;return;
 }
 $65 = HEAP32[$0>>2]|0;
 $66 = (($65) + 72|0);
 HEAP32[$66>>2] = 3;
 STACKTOP = sp;return;
}
function _tableOfTrigger($pTrigger) {
 $pTrigger = $pTrigger|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 HEAP32[$0>>2] = $pTrigger;
 $1 = HEAP32[$0>>2]|0;
 $2 = (($1) + 24|0);
 $3 = HEAP32[$2>>2]|0;
 $4 = (($3) + 8|0);
 $5 = HEAP32[$0>>2]|0;
 $6 = (($5) + 4|0);
 $7 = HEAP32[$6>>2]|0;
 $8 = (_sqlite3HashFind($4,$7)|0);
 STACKTOP = sp;return ($8|0);
}
function _sqlite3OpenMasterTable($p,$iDb) {
 $p = $p|0;
 $iDb = $iDb|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $v = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $1 = sp + 4|0;
 $v = sp;
 HEAP32[$0>>2] = $p;
 HEAP32[$1>>2] = $iDb;
 $2 = HEAP32[$0>>2]|0;
 $3 = (_sqlite3GetVdbe($2)|0);
 HEAP32[$v>>2] = $3;
 $4 = HEAP32[$0>>2]|0;
 $5 = HEAP32[$1>>2]|0;
 $6 = HEAP32[$1>>2]|0;
 $7 = ($6|0)==(1);
 $8 = $7 ? 14904 : 14928;
 _sqlite3TableLock($4,$5,1,1,$8);
 $9 = HEAP32[$v>>2]|0;
 $10 = HEAP32[$1>>2]|0;
 (_sqlite3VdbeAddOp4Int($9,55,0,1,$10,5)|0);
 $11 = HEAP32[$0>>2]|0;
 $12 = (($11) + 68|0);
 $13 = HEAP32[$12>>2]|0;
 $14 = ($13|0)==(0);
 if (!($14)) {
  STACKTOP = sp;return;
 }
 $15 = HEAP32[$0>>2]|0;
 $16 = (($15) + 68|0);
 HEAP32[$16>>2] = 1;
 STACKTOP = sp;return;
}
function _triggerStepAllocate($db,$op,$pName) {
 $db = $db|0;
 $op = $op|0;
 $pName = $pName|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $pTriggerStep = 0, $z = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 12|0;
 $1 = sp + 16|0;
 $2 = sp + 8|0;
 $pTriggerStep = sp + 4|0;
 $z = sp;
 HEAP32[$0>>2] = $db;
 HEAP8[$1>>0] = $op;
 HEAP32[$2>>2] = $pName;
 $3 = HEAP32[$0>>2]|0;
 $4 = HEAP32[$2>>2]|0;
 $5 = (($4) + 4|0);
 $6 = HEAP32[$5>>2]|0;
 $7 = (40 + ($6))|0;
 $8 = (_sqlite3DbMallocZero($3,$7,0)|0);
 HEAP32[$pTriggerStep>>2] = $8;
 $9 = HEAP32[$pTriggerStep>>2]|0;
 $10 = ($9|0)!=(0|0);
 if (!($10)) {
  $30 = HEAP32[$pTriggerStep>>2]|0;
  STACKTOP = sp;return ($30|0);
 }
 $11 = HEAP32[$pTriggerStep>>2]|0;
 $12 = (($11) + 40|0);
 HEAP32[$z>>2] = $12;
 $13 = HEAP32[$z>>2]|0;
 $14 = HEAP32[$2>>2]|0;
 $15 = HEAP32[$14>>2]|0;
 $16 = HEAP32[$2>>2]|0;
 $17 = (($16) + 4|0);
 $18 = HEAP32[$17>>2]|0;
 _memcpy(($13|0),($15|0),($18|0))|0;
 $19 = HEAP32[$z>>2]|0;
 $20 = HEAP32[$pTriggerStep>>2]|0;
 $21 = (($20) + 12|0);
 HEAP32[$21>>2] = $19;
 $22 = HEAP32[$2>>2]|0;
 $23 = (($22) + 4|0);
 $24 = HEAP32[$23>>2]|0;
 $25 = HEAP32[$pTriggerStep>>2]|0;
 $26 = (($25) + 12|0);
 $27 = (($26) + 4|0);
 HEAP32[$27>>2] = $24;
 $28 = HEAP8[$1>>0]|0;
 $29 = HEAP32[$pTriggerStep>>2]|0;
 HEAP8[$29>>0] = $28;
 $30 = HEAP32[$pTriggerStep>>2]|0;
 STACKTOP = sp;return ($30|0);
}
function _sqlite3SrcListLookup($pParse,$pSrc) {
 $pParse = $pParse|0;
 $pSrc = $pSrc|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $3 = 0;
 var $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $pItem = 0, $pTab = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $1 = sp + 8|0;
 $pItem = sp + 4|0;
 $pTab = sp;
 HEAP32[$0>>2] = $pParse;
 HEAP32[$1>>2] = $pSrc;
 $2 = HEAP32[$1>>2]|0;
 $3 = (($2) + 8|0);
 HEAP32[$pItem>>2] = $3;
 $4 = HEAP32[$0>>2]|0;
 $5 = HEAP32[$pItem>>2]|0;
 $6 = (_sqlite3LocateTableItem($4,0,$5)|0);
 HEAP32[$pTab>>2] = $6;
 $7 = HEAP32[$0>>2]|0;
 $8 = HEAP32[$7>>2]|0;
 $9 = HEAP32[$pItem>>2]|0;
 $10 = (($9) + 16|0);
 $11 = HEAP32[$10>>2]|0;
 _sqlite3DeleteTable($8,$11);
 $12 = HEAP32[$pTab>>2]|0;
 $13 = HEAP32[$pItem>>2]|0;
 $14 = (($13) + 16|0);
 HEAP32[$14>>2] = $12;
 $15 = HEAP32[$pTab>>2]|0;
 $16 = ($15|0)!=(0|0);
 if ($16) {
  $17 = HEAP32[$pTab>>2]|0;
  $18 = (($17) + 40|0);
  $19 = HEAP16[$18>>1]|0;
  $20 = (($19) + 1)<<16>>16;
  HEAP16[$18>>1] = $20;
 }
 $21 = HEAP32[$0>>2]|0;
 $22 = HEAP32[$pItem>>2]|0;
 $23 = (_sqlite3IndexedByLookup($21,$22)|0);
 $24 = ($23|0)!=(0);
 if (!($24)) {
  $25 = HEAP32[$pTab>>2]|0;
  STACKTOP = sp;return ($25|0);
 }
 HEAP32[$pTab>>2] = 0;
 $25 = HEAP32[$pTab>>2]|0;
 STACKTOP = sp;return ($25|0);
}
function _sqlite3FixInit($pFix,$pParse,$iDb,$zType,$pName) {
 $pFix = $pFix|0;
 $pParse = $pParse|0;
 $iDb = $iDb|0;
 $zType = $zType|0;
 $pName = $pName|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $db = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 20|0;
 $1 = sp + 16|0;
 $2 = sp + 12|0;
 $3 = sp + 8|0;
 $4 = sp + 4|0;
 $db = sp;
 HEAP32[$0>>2] = $pFix;
 HEAP32[$1>>2] = $pParse;
 HEAP32[$2>>2] = $iDb;
 HEAP32[$3>>2] = $zType;
 HEAP32[$4>>2] = $pName;
 $5 = HEAP32[$1>>2]|0;
 $6 = HEAP32[$5>>2]|0;
 HEAP32[$db>>2] = $6;
 $7 = HEAP32[$1>>2]|0;
 $8 = HEAP32[$0>>2]|0;
 HEAP32[$8>>2] = $7;
 $9 = HEAP32[$2>>2]|0;
 $10 = HEAP32[$db>>2]|0;
 $11 = (($10) + 16|0);
 $12 = HEAP32[$11>>2]|0;
 $13 = (($12) + ($9<<4)|0);
 $14 = HEAP32[$13>>2]|0;
 $15 = HEAP32[$0>>2]|0;
 $16 = (($15) + 12|0);
 HEAP32[$16>>2] = $14;
 $17 = HEAP32[$2>>2]|0;
 $18 = HEAP32[$db>>2]|0;
 $19 = (($18) + 16|0);
 $20 = HEAP32[$19>>2]|0;
 $21 = (($20) + ($17<<4)|0);
 $22 = (($21) + 12|0);
 $23 = HEAP32[$22>>2]|0;
 $24 = HEAP32[$0>>2]|0;
 $25 = (($24) + 4|0);
 HEAP32[$25>>2] = $23;
 $26 = HEAP32[$3>>2]|0;
 $27 = HEAP32[$0>>2]|0;
 $28 = (($27) + 16|0);
 HEAP32[$28>>2] = $26;
 $29 = HEAP32[$4>>2]|0;
 $30 = HEAP32[$0>>2]|0;
 $31 = (($30) + 20|0);
 HEAP32[$31>>2] = $29;
 $32 = HEAP32[$2>>2]|0;
 $33 = ($32|0)==(1);
 $34 = $33&1;
 $35 = HEAP32[$0>>2]|0;
 $36 = (($35) + 8|0);
 HEAP32[$36>>2] = $34;
 STACKTOP = sp;return;
}
function _sqlite3FixSrcList($pFix,$pList) {
 $pFix = $pFix|0;
 $pList = $pList|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $7 = 0, $8 = 0, $9 = 0, $i = 0, $pItem = 0, $vararg_buffer = 0, $vararg_ptr1 = 0, $vararg_ptr2 = 0, $zDb = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $vararg_buffer = sp;
 $0 = sp + 32|0;
 $1 = sp + 28|0;
 $2 = sp + 24|0;
 $i = sp + 20|0;
 $zDb = sp + 16|0;
 $pItem = sp + 12|0;
 HEAP32[$1>>2] = $pFix;
 HEAP32[$2>>2] = $pList;
 $3 = HEAP32[$2>>2]|0;
 $4 = ($3|0)==(0|0);
 if ($4) {
  HEAP32[$0>>2] = 0;
  $67 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($67|0);
 }
 $5 = HEAP32[$1>>2]|0;
 $6 = (($5) + 12|0);
 $7 = HEAP32[$6>>2]|0;
 HEAP32[$zDb>>2] = $7;
 HEAP32[$i>>2] = 0;
 $8 = HEAP32[$2>>2]|0;
 $9 = (($8) + 8|0);
 HEAP32[$pItem>>2] = $9;
 while(1) {
  $10 = HEAP32[$i>>2]|0;
  $11 = HEAP32[$2>>2]|0;
  $12 = HEAP32[$11>>2]|0;
  $13 = ($10|0)<($12|0);
  if (!($13)) {
   label = 15;
   break;
  }
  $14 = HEAP32[$1>>2]|0;
  $15 = (($14) + 8|0);
  $16 = HEAP32[$15>>2]|0;
  $17 = ($16|0)==(0);
  if ($17) {
   $18 = HEAP32[$pItem>>2]|0;
   $19 = (($18) + 4|0);
   $20 = HEAP32[$19>>2]|0;
   $21 = ($20|0)!=(0|0);
   if ($21) {
    $22 = HEAP32[$pItem>>2]|0;
    $23 = (($22) + 4|0);
    $24 = HEAP32[$23>>2]|0;
    $25 = HEAP32[$zDb>>2]|0;
    $26 = (_sqlite3_stricmp($24,$25)|0);
    $27 = ($26|0)!=(0);
    if ($27) {
     label = 8;
     break;
    }
   }
   $39 = HEAP32[$1>>2]|0;
   $40 = HEAP32[$39>>2]|0;
   $41 = HEAP32[$40>>2]|0;
   $42 = HEAP32[$pItem>>2]|0;
   $43 = (($42) + 4|0);
   $44 = HEAP32[$43>>2]|0;
   _sqlite3DbFree($41,$44);
   $45 = HEAP32[$pItem>>2]|0;
   $46 = (($45) + 4|0);
   HEAP32[$46>>2] = 0;
   $47 = HEAP32[$1>>2]|0;
   $48 = (($47) + 4|0);
   $49 = HEAP32[$48>>2]|0;
   $50 = HEAP32[$pItem>>2]|0;
   HEAP32[$50>>2] = $49;
  }
  $51 = HEAP32[$1>>2]|0;
  $52 = HEAP32[$pItem>>2]|0;
  $53 = (($52) + 20|0);
  $54 = HEAP32[$53>>2]|0;
  $55 = (_sqlite3FixSelect($51,$54)|0);
  $56 = ($55|0)!=(0);
  if ($56) {
   label = 11;
   break;
  }
  $57 = HEAP32[$1>>2]|0;
  $58 = HEAP32[$pItem>>2]|0;
  $59 = (($58) + 44|0);
  $60 = HEAP32[$59>>2]|0;
  $61 = (_sqlite3FixExpr($57,$60)|0);
  $62 = ($61|0)!=(0);
  if ($62) {
   label = 13;
   break;
  }
  $63 = HEAP32[$i>>2]|0;
  $64 = (($63) + 1)|0;
  HEAP32[$i>>2] = $64;
  $65 = HEAP32[$pItem>>2]|0;
  $66 = (($65) + 72|0);
  HEAP32[$pItem>>2] = $66;
 }
 if ((label|0) == 8) {
  $28 = HEAP32[$1>>2]|0;
  $29 = HEAP32[$28>>2]|0;
  $30 = HEAP32[$1>>2]|0;
  $31 = (($30) + 16|0);
  $32 = HEAP32[$31>>2]|0;
  $33 = HEAP32[$1>>2]|0;
  $34 = (($33) + 20|0);
  $35 = HEAP32[$34>>2]|0;
  $36 = HEAP32[$pItem>>2]|0;
  $37 = (($36) + 4|0);
  $38 = HEAP32[$37>>2]|0;
  HEAP32[$vararg_buffer>>2] = $32;
  $vararg_ptr1 = (($vararg_buffer) + 4|0);
  HEAP32[$vararg_ptr1>>2] = $35;
  $vararg_ptr2 = (($vararg_buffer) + 8|0);
  HEAP32[$vararg_ptr2>>2] = $38;
  _sqlite3ErrorMsg($29,20128,$vararg_buffer);
  HEAP32[$0>>2] = 1;
  $67 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($67|0);
 }
 else if ((label|0) == 11) {
  HEAP32[$0>>2] = 1;
  $67 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($67|0);
 }
 else if ((label|0) == 13) {
  HEAP32[$0>>2] = 1;
  $67 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($67|0);
 }
 else if ((label|0) == 15) {
  HEAP32[$0>>2] = 0;
  $67 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($67|0);
 }
 return 0|0;
}
function _sqlite3FixSelect($pFix,$pSelect) {
 $pFix = $pFix|0;
 $pSelect = $pSelect|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $1 = sp + 4|0;
 $2 = sp;
 HEAP32[$1>>2] = $pFix;
 HEAP32[$2>>2] = $pSelect;
 while(1) {
  $3 = HEAP32[$2>>2]|0;
  $4 = ($3|0)!=(0|0);
  if (!($4)) {
   label = 20;
   break;
  }
  $5 = HEAP32[$1>>2]|0;
  $6 = HEAP32[$2>>2]|0;
  $7 = HEAP32[$6>>2]|0;
  $8 = (_sqlite3FixExprList($5,$7)|0);
  $9 = ($8|0)!=(0);
  if ($9) {
   label = 4;
   break;
  }
  $10 = HEAP32[$1>>2]|0;
  $11 = HEAP32[$2>>2]|0;
  $12 = (($11) + 32|0);
  $13 = HEAP32[$12>>2]|0;
  $14 = (_sqlite3FixSrcList($10,$13)|0);
  $15 = ($14|0)!=(0);
  if ($15) {
   label = 6;
   break;
  }
  $16 = HEAP32[$1>>2]|0;
  $17 = HEAP32[$2>>2]|0;
  $18 = (($17) + 36|0);
  $19 = HEAP32[$18>>2]|0;
  $20 = (_sqlite3FixExpr($16,$19)|0);
  $21 = ($20|0)!=(0);
  if ($21) {
   label = 8;
   break;
  }
  $22 = HEAP32[$1>>2]|0;
  $23 = HEAP32[$2>>2]|0;
  $24 = (($23) + 40|0);
  $25 = HEAP32[$24>>2]|0;
  $26 = (_sqlite3FixExprList($22,$25)|0);
  $27 = ($26|0)!=(0);
  if ($27) {
   label = 10;
   break;
  }
  $28 = HEAP32[$1>>2]|0;
  $29 = HEAP32[$2>>2]|0;
  $30 = (($29) + 44|0);
  $31 = HEAP32[$30>>2]|0;
  $32 = (_sqlite3FixExpr($28,$31)|0);
  $33 = ($32|0)!=(0);
  if ($33) {
   label = 12;
   break;
  }
  $34 = HEAP32[$1>>2]|0;
  $35 = HEAP32[$2>>2]|0;
  $36 = (($35) + 48|0);
  $37 = HEAP32[$36>>2]|0;
  $38 = (_sqlite3FixExprList($34,$37)|0);
  $39 = ($38|0)!=(0);
  if ($39) {
   label = 14;
   break;
  }
  $40 = HEAP32[$1>>2]|0;
  $41 = HEAP32[$2>>2]|0;
  $42 = (($41) + 60|0);
  $43 = HEAP32[$42>>2]|0;
  $44 = (_sqlite3FixExpr($40,$43)|0);
  $45 = ($44|0)!=(0);
  if ($45) {
   label = 16;
   break;
  }
  $46 = HEAP32[$1>>2]|0;
  $47 = HEAP32[$2>>2]|0;
  $48 = (($47) + 64|0);
  $49 = HEAP32[$48>>2]|0;
  $50 = (_sqlite3FixExpr($46,$49)|0);
  $51 = ($50|0)!=(0);
  if ($51) {
   label = 18;
   break;
  }
  $52 = HEAP32[$2>>2]|0;
  $53 = (($52) + 52|0);
  $54 = HEAP32[$53>>2]|0;
  HEAP32[$2>>2] = $54;
 }
 if ((label|0) == 4) {
  HEAP32[$0>>2] = 1;
 }
 else if ((label|0) == 6) {
  HEAP32[$0>>2] = 1;
 }
 else if ((label|0) == 8) {
  HEAP32[$0>>2] = 1;
 }
 else if ((label|0) == 10) {
  HEAP32[$0>>2] = 1;
 }
 else if ((label|0) == 12) {
  HEAP32[$0>>2] = 1;
 }
 else if ((label|0) == 14) {
  HEAP32[$0>>2] = 1;
 }
 else if ((label|0) == 16) {
  HEAP32[$0>>2] = 1;
 }
 else if ((label|0) == 18) {
  HEAP32[$0>>2] = 1;
 }
 else if ((label|0) == 20) {
  HEAP32[$0>>2] = 0;
 }
 $55 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($55|0);
}
function _sqlite3FixExpr($pFix,$pExpr) {
 $pFix = $pFix|0;
 $pExpr = $pExpr|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $vararg_buffer = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $vararg_buffer = sp;
 $0 = sp + 12|0;
 $1 = sp + 8|0;
 $2 = sp + 4|0;
 HEAP32[$1>>2] = $pFix;
 HEAP32[$2>>2] = $pExpr;
 while(1) {
  $3 = HEAP32[$2>>2]|0;
  $4 = ($3|0)!=(0|0);
  if (!($4)) {
   label = 16;
   break;
  }
  $5 = HEAP32[$2>>2]|0;
  $6 = HEAP8[$5>>0]|0;
  $7 = $6&255;
  $8 = ($7|0)==(135);
  if ($8) {
   $9 = HEAP32[$1>>2]|0;
   $10 = HEAP32[$9>>2]|0;
   $11 = HEAP32[$10>>2]|0;
   $12 = (($11) + 140|0);
   $13 = (($12) + 5|0);
   $14 = HEAP8[$13>>0]|0;
   $15 = ($14<<24>>24)!=(0);
   if (!($15)) {
    label = 6;
    break;
   }
   $16 = HEAP32[$2>>2]|0;
   HEAP8[$16>>0] = 101;
  }
  $22 = HEAP32[$2>>2]|0;
  $23 = (($22) + 4|0);
  $24 = HEAP32[$23>>2]|0;
  $25 = $24 & 16384;
  $26 = ($25|0)!=(0);
  if ($26) {
   label = 16;
   break;
  }
  $27 = HEAP32[$2>>2]|0;
  $28 = (($27) + 4|0);
  $29 = HEAP32[$28>>2]|0;
  $30 = $29 & 2048;
  $31 = ($30|0)!=(0);
  $32 = HEAP32[$1>>2]|0;
  $33 = HEAP32[$2>>2]|0;
  $34 = (($33) + 20|0);
  if ($31) {
   $35 = HEAP32[$34>>2]|0;
   $36 = (_sqlite3FixSelect($32,$35)|0);
   $37 = ($36|0)!=(0);
   if ($37) {
    label = 10;
    break;
   }
  } else {
   $38 = HEAP32[$34>>2]|0;
   $39 = (_sqlite3FixExprList($32,$38)|0);
   $40 = ($39|0)!=(0);
   if ($40) {
    label = 12;
    break;
   }
  }
  $41 = HEAP32[$1>>2]|0;
  $42 = HEAP32[$2>>2]|0;
  $43 = (($42) + 16|0);
  $44 = HEAP32[$43>>2]|0;
  $45 = (_sqlite3FixExpr($41,$44)|0);
  $46 = ($45|0)!=(0);
  if ($46) {
   label = 14;
   break;
  }
  $47 = HEAP32[$2>>2]|0;
  $48 = (($47) + 12|0);
  $49 = HEAP32[$48>>2]|0;
  HEAP32[$2>>2] = $49;
 }
 if ((label|0) == 6) {
  $17 = HEAP32[$1>>2]|0;
  $18 = HEAP32[$17>>2]|0;
  $19 = HEAP32[$1>>2]|0;
  $20 = (($19) + 16|0);
  $21 = HEAP32[$20>>2]|0;
  HEAP32[$vararg_buffer>>2] = $21;
  _sqlite3ErrorMsg($18,20176,$vararg_buffer);
  HEAP32[$0>>2] = 1;
  $50 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($50|0);
 }
 else if ((label|0) == 10) {
  HEAP32[$0>>2] = 1;
  $50 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($50|0);
 }
 else if ((label|0) == 12) {
  HEAP32[$0>>2] = 1;
  $50 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($50|0);
 }
 else if ((label|0) == 14) {
  HEAP32[$0>>2] = 1;
  $50 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($50|0);
 }
 else if ((label|0) == 16) {
  HEAP32[$0>>2] = 0;
  $50 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($50|0);
 }
 return 0|0;
}
function _sqlite3FixExprList($pFix,$pList) {
 $pFix = $pFix|0;
 $pList = $pList|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0;
 var $8 = 0, $9 = 0, $i = 0, $pItem = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 16|0;
 $1 = sp + 12|0;
 $2 = sp + 8|0;
 $i = sp + 4|0;
 $pItem = sp;
 HEAP32[$1>>2] = $pFix;
 HEAP32[$2>>2] = $pList;
 $3 = HEAP32[$2>>2]|0;
 $4 = ($3|0)==(0|0);
 if ($4) {
  HEAP32[$0>>2] = 0;
  $21 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($21|0);
 }
 HEAP32[$i>>2] = 0;
 $5 = HEAP32[$2>>2]|0;
 $6 = (($5) + 4|0);
 $7 = HEAP32[$6>>2]|0;
 HEAP32[$pItem>>2] = $7;
 while(1) {
  $8 = HEAP32[$i>>2]|0;
  $9 = HEAP32[$2>>2]|0;
  $10 = HEAP32[$9>>2]|0;
  $11 = ($8|0)<($10|0);
  if (!($11)) {
   label = 8;
   break;
  }
  $12 = HEAP32[$1>>2]|0;
  $13 = HEAP32[$pItem>>2]|0;
  $14 = HEAP32[$13>>2]|0;
  $15 = (_sqlite3FixExpr($12,$14)|0);
  $16 = ($15|0)!=(0);
  if ($16) {
   label = 6;
   break;
  }
  $17 = HEAP32[$i>>2]|0;
  $18 = (($17) + 1)|0;
  HEAP32[$i>>2] = $18;
  $19 = HEAP32[$pItem>>2]|0;
  $20 = (($19) + 20|0);
  HEAP32[$pItem>>2] = $20;
 }
 if ((label|0) == 6) {
  HEAP32[$0>>2] = 1;
  $21 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($21|0);
 }
 else if ((label|0) == 8) {
  HEAP32[$0>>2] = 0;
  $21 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($21|0);
 }
 return 0|0;
}
function _sqlite3FixTriggerStep($pFix,$pStep) {
 $pFix = $pFix|0;
 $pStep = $pStep|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $1 = sp + 4|0;
 $2 = sp;
 HEAP32[$1>>2] = $pFix;
 HEAP32[$2>>2] = $pStep;
 while(1) {
  $3 = HEAP32[$2>>2]|0;
  $4 = ($3|0)!=(0|0);
  if (!($4)) {
   label = 10;
   break;
  }
  $5 = HEAP32[$1>>2]|0;
  $6 = HEAP32[$2>>2]|0;
  $7 = (($6) + 8|0);
  $8 = HEAP32[$7>>2]|0;
  $9 = (_sqlite3FixSelect($5,$8)|0);
  $10 = ($9|0)!=(0);
  if ($10) {
   label = 4;
   break;
  }
  $11 = HEAP32[$1>>2]|0;
  $12 = HEAP32[$2>>2]|0;
  $13 = (($12) + 20|0);
  $14 = HEAP32[$13>>2]|0;
  $15 = (_sqlite3FixExpr($11,$14)|0);
  $16 = ($15|0)!=(0);
  if ($16) {
   label = 6;
   break;
  }
  $17 = HEAP32[$1>>2]|0;
  $18 = HEAP32[$2>>2]|0;
  $19 = (($18) + 24|0);
  $20 = HEAP32[$19>>2]|0;
  $21 = (_sqlite3FixExprList($17,$20)|0);
  $22 = ($21|0)!=(0);
  if ($22) {
   label = 8;
   break;
  }
  $23 = HEAP32[$2>>2]|0;
  $24 = (($23) + 32|0);
  $25 = HEAP32[$24>>2]|0;
  HEAP32[$2>>2] = $25;
 }
 if ((label|0) == 4) {
  HEAP32[$0>>2] = 1;
 }
 else if ((label|0) == 6) {
  HEAP32[$0>>2] = 1;
 }
 else if ((label|0) == 8) {
  HEAP32[$0>>2] = 1;
 }
 else if ((label|0) == 10) {
  HEAP32[$0>>2] = 0;
 }
 $26 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($26|0);
}
function _sqlite3VdbeRunOnlyOnce($p) {
 $p = $p|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 HEAP32[$0>>2] = $p;
 $1 = HEAP32[$0>>2]|0;
 $2 = (($1) + 88|0);
 $3 = HEAP16[$2>>1]|0;
 $4 = $3 & -65;
 $5 = $4 | 64;
 HEAP16[$2>>1] = $5;
 STACKTOP = sp;return;
}
function _sqlite3VdbeSetNumCols($p,$nResColumn) {
 $p = $p|0;
 $nResColumn = $nResColumn|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $5 = 0, $6 = 0;
 var $7 = 0, $8 = 0, $9 = 0, $db = 0, $n = 0, $pColName = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 16|0;
 $1 = sp + 12|0;
 $pColName = sp + 8|0;
 $n = sp + 4|0;
 $db = sp;
 HEAP32[$0>>2] = $p;
 HEAP32[$1>>2] = $nResColumn;
 $2 = HEAP32[$0>>2]|0;
 $3 = HEAP32[$2>>2]|0;
 HEAP32[$db>>2] = $3;
 $4 = HEAP32[$0>>2]|0;
 $5 = (($4) + 16|0);
 $6 = HEAP32[$5>>2]|0;
 $7 = HEAP32[$0>>2]|0;
 $8 = (($7) + 84|0);
 $9 = HEAP16[$8>>1]|0;
 $10 = $9&65535;
 $11 = $10<<1;
 _releaseMemArray($6,$11);
 $12 = HEAP32[$db>>2]|0;
 $13 = HEAP32[$0>>2]|0;
 $14 = (($13) + 16|0);
 $15 = HEAP32[$14>>2]|0;
 _sqlite3DbFree($12,$15);
 $16 = HEAP32[$1>>2]|0;
 $17 = $16<<1;
 HEAP32[$n>>2] = $17;
 $18 = HEAP32[$1>>2]|0;
 $19 = $18&65535;
 $20 = HEAP32[$0>>2]|0;
 $21 = (($20) + 84|0);
 HEAP16[$21>>1] = $19;
 $22 = HEAP32[$db>>2]|0;
 $23 = HEAP32[$n>>2]|0;
 $24 = ($23*40)|0;
 $25 = (_sqlite3DbMallocZero($22,$24,0)|0);
 HEAP32[$pColName>>2] = $25;
 $26 = HEAP32[$0>>2]|0;
 $27 = (($26) + 16|0);
 HEAP32[$27>>2] = $25;
 $28 = HEAP32[$0>>2]|0;
 $29 = (($28) + 16|0);
 $30 = HEAP32[$29>>2]|0;
 $31 = ($30|0)==(0|0);
 if ($31) {
  STACKTOP = sp;return;
 }
 while(1) {
  $32 = HEAP32[$n>>2]|0;
  $33 = (($32) + -1)|0;
  HEAP32[$n>>2] = $33;
  $34 = ($32|0)>(0);
  if (!($34)) {
   break;
  }
  $35 = HEAP32[$pColName>>2]|0;
  $36 = (($35) + 8|0);
  HEAP16[$36>>1] = 1;
  $37 = HEAP32[$0>>2]|0;
  $38 = HEAP32[$37>>2]|0;
  $39 = HEAP32[$pColName>>2]|0;
  $40 = (($39) + 32|0);
  HEAP32[$40>>2] = $38;
  $41 = HEAP32[$pColName>>2]|0;
  $42 = (($41) + 40|0);
  HEAP32[$pColName>>2] = $42;
 }
 STACKTOP = sp;return;
}
function _sqlite3VdbeSetColName($p,$idx,$var,$zName,$xDel) {
 $p = $p|0;
 $idx = $idx|0;
 $var = $var|0;
 $zName = $zName|0;
 $xDel = $xDel|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $pColName = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 28|0;
 $1 = sp + 24|0;
 $2 = sp + 20|0;
 $3 = sp + 16|0;
 $4 = sp + 12|0;
 $5 = sp + 8|0;
 $rc = sp + 4|0;
 $pColName = sp;
 HEAP32[$1>>2] = $p;
 HEAP32[$2>>2] = $idx;
 HEAP32[$3>>2] = $var;
 HEAP32[$4>>2] = $zName;
 HEAP32[$5>>2] = $xDel;
 $6 = HEAP32[$1>>2]|0;
 $7 = HEAP32[$6>>2]|0;
 $8 = (($7) + 64|0);
 $9 = HEAP8[$8>>0]|0;
 $10 = ($9<<24>>24)!=(0);
 if ($10) {
  HEAP32[$0>>2] = 7;
  $28 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($28|0);
 } else {
  $11 = HEAP32[$2>>2]|0;
  $12 = HEAP32[$3>>2]|0;
  $13 = HEAP32[$1>>2]|0;
  $14 = (($13) + 84|0);
  $15 = HEAP16[$14>>1]|0;
  $16 = $15&65535;
  $17 = Math_imul($12, $16)|0;
  $18 = (($11) + ($17))|0;
  $19 = HEAP32[$1>>2]|0;
  $20 = (($19) + 16|0);
  $21 = HEAP32[$20>>2]|0;
  $22 = (($21) + (($18*40)|0)|0);
  HEAP32[$pColName>>2] = $22;
  $23 = HEAP32[$pColName>>2]|0;
  $24 = HEAP32[$4>>2]|0;
  $25 = HEAP32[$5>>2]|0;
  $26 = (_sqlite3VdbeMemSetStr($23,$24,-1,1,$25)|0);
  HEAP32[$rc>>2] = $26;
  $27 = HEAP32[$rc>>2]|0;
  HEAP32[$0>>2] = $27;
  $28 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($28|0);
 }
 return 0|0;
}
function _returnSingleInt($pParse,$zLabel,$0,$1) {
 $pParse = $pParse|0;
 $zLabel = $zLabel|0;
 $0 = $0|0;
 $1 = $1|0;
 var $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0;
 var $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $mem = 0, $pI64 = 0, $v = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $2 = sp + 24|0;
 $3 = sp + 20|0;
 $4 = sp;
 $v = sp + 16|0;
 $mem = sp + 12|0;
 $pI64 = sp + 8|0;
 HEAP32[$2>>2] = $pParse;
 HEAP32[$3>>2] = $zLabel;
 $5 = $4;
 $6 = $5;
 HEAP32[$6>>2] = $0;
 $7 = (($5) + 4)|0;
 $8 = $7;
 HEAP32[$8>>2] = $1;
 $9 = HEAP32[$2>>2]|0;
 $10 = (_sqlite3GetVdbe($9)|0);
 HEAP32[$v>>2] = $10;
 $11 = HEAP32[$2>>2]|0;
 $12 = (($11) + 72|0);
 $13 = HEAP32[$12>>2]|0;
 $14 = (($13) + 1)|0;
 HEAP32[$12>>2] = $14;
 HEAP32[$mem>>2] = $14;
 $15 = HEAP32[$2>>2]|0;
 $16 = HEAP32[$15>>2]|0;
 $17 = (_sqlite3DbMallocRaw($16,8,0)|0);
 HEAP32[$pI64>>2] = $17;
 $18 = HEAP32[$pI64>>2]|0;
 $19 = ($18|0)!=(0|0);
 if ($19) {
  $20 = HEAP32[$pI64>>2]|0;
  ;HEAP32[$20+0>>2]=HEAP32[$4+0>>2]|0;HEAP32[$20+4>>2]=HEAP32[$4+4>>2]|0;
 }
 $21 = HEAP32[$v>>2]|0;
 $22 = HEAP32[$mem>>2]|0;
 $23 = HEAP32[$pI64>>2]|0;
 (_sqlite3VdbeAddOp4($21,26,0,$22,0,$23,-13)|0);
 $24 = HEAP32[$v>>2]|0;
 _sqlite3VdbeSetNumCols($24,1);
 $25 = HEAP32[$v>>2]|0;
 $26 = HEAP32[$3>>2]|0;
 (_sqlite3VdbeSetColName($25,0,0,$26,0)|0);
 $27 = HEAP32[$v>>2]|0;
 $28 = HEAP32[$mem>>2]|0;
 (_sqlite3VdbeAddOp2($27,35,$28,1)|0);
 STACKTOP = sp;return;
}
function _getLockingMode($z) {
 $z = $z|0;
 var $0 = 0, $1 = 0, $10 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $1 = sp;
 HEAP32[$1>>2] = $z;
 $2 = HEAP32[$1>>2]|0;
 $3 = ($2|0)!=(0|0);
 do {
  if ($3) {
   $4 = HEAP32[$1>>2]|0;
   $5 = (_sqlite3_stricmp($4,21072)|0);
   $6 = (0)==($5|0);
   if ($6) {
    HEAP32[$0>>2] = 1;
    break;
   }
   $7 = HEAP32[$1>>2]|0;
   $8 = (_sqlite3_stricmp($7,21064)|0);
   $9 = (0)==($8|0);
   if ($9) {
    HEAP32[$0>>2] = 0;
   } else {
    label = 6;
   }
  } else {
   label = 6;
  }
 } while(0);
 if ((label|0) == 6) {
  HEAP32[$0>>2] = -1;
 }
 $10 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($10|0);
}
function _sqlite3JournalModename($eMode) {
 $eMode = $eMode|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $1 = sp;
 HEAP32[$1>>2] = $eMode;
 $2 = HEAP32[$1>>2]|0;
 $3 = ($2|0)==(6);
 if ($3) {
  HEAP32[$0>>2] = 0;
 } else {
  $4 = HEAP32[$1>>2]|0;
  $5 = (22296 + ($4<<2)|0);
  $6 = HEAP32[$5>>2]|0;
  HEAP32[$0>>2] = $6;
 }
 $7 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($7|0);
}
function _sqlite3PagerJournalSizeLimit($pPager,$0,$1) {
 $pPager = $pPager|0;
 $0 = $0|0;
 $1 = $1|0;
 var $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0;
 var $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0;
 var $47 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $2 = sp + 8|0;
 $3 = sp;
 HEAP32[$2>>2] = $pPager;
 $4 = $3;
 $5 = $4;
 HEAP32[$5>>2] = $0;
 $6 = (($4) + 4)|0;
 $7 = $6;
 HEAP32[$7>>2] = $1;
 $8 = $3;
 $9 = $8;
 $10 = HEAP32[$9>>2]|0;
 $11 = (($8) + 4)|0;
 $12 = $11;
 $13 = HEAP32[$12>>2]|0;
 $14 = ($13|0)>(-1);
 $15 = ($10>>>0)>=(4294967295);
 $16 = ($13|0)==(-1);
 $17 = $16 & $15;
 $18 = $14 | $17;
 if (!($18)) {
  $40 = HEAP32[$2>>2]|0;
  $41 = (($40) + 160|0);
  $42 = $41;
  $43 = $42;
  $44 = HEAP32[$43>>2]|0;
  $45 = (($42) + 4)|0;
  $46 = $45;
  $47 = HEAP32[$46>>2]|0;
  tempRet0 = $47;
  STACKTOP = sp;return ($44|0);
 }
 $19 = $3;
 $20 = $19;
 $21 = HEAP32[$20>>2]|0;
 $22 = (($19) + 4)|0;
 $23 = $22;
 $24 = HEAP32[$23>>2]|0;
 $25 = HEAP32[$2>>2]|0;
 $26 = (($25) + 160|0);
 $27 = $26;
 $28 = $27;
 HEAP32[$28>>2] = $21;
 $29 = (($27) + 4)|0;
 $30 = $29;
 HEAP32[$30>>2] = $24;
 $31 = HEAP32[$2>>2]|0;
 $32 = (($31) + 208|0);
 $33 = HEAP32[$32>>2]|0;
 $34 = $3;
 $35 = $34;
 $36 = HEAP32[$35>>2]|0;
 $37 = (($34) + 4)|0;
 $38 = $37;
 $39 = HEAP32[$38>>2]|0;
 _sqlite3WalLimit($33,$36,$39);
 $40 = HEAP32[$2>>2]|0;
 $41 = (($40) + 160|0);
 $42 = $41;
 $43 = $42;
 $44 = HEAP32[$43>>2]|0;
 $45 = (($42) + 4)|0;
 $46 = $45;
 $47 = HEAP32[$46>>2]|0;
 tempRet0 = $47;
 STACKTOP = sp;return ($44|0);
}
function _sqlite3BtreeGetAutoVacuum($p) {
 $p = $p|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $rc = 0, label = 0;
 var sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $rc = sp;
 HEAP32[$0>>2] = $p;
 $1 = HEAP32[$0>>2]|0;
 _sqlite3BtreeEnter($1);
 $2 = HEAP32[$0>>2]|0;
 $3 = (($2) + 4|0);
 $4 = HEAP32[$3>>2]|0;
 $5 = (($4) + 17|0);
 $6 = HEAP8[$5>>0]|0;
 $7 = ($6<<24>>24)!=(0);
 if (!($7)) {
  $16 = 0;
  HEAP32[$rc>>2] = $16;
  $17 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($17|0);
 }
 $8 = HEAP32[$0>>2]|0;
 $9 = (($8) + 4|0);
 $10 = HEAP32[$9>>2]|0;
 $11 = (($10) + 18|0);
 $12 = HEAP8[$11>>0]|0;
 $13 = ($12<<24>>24)!=(0);
 $14 = $13 ^ 1;
 $15 = $14 ? 1 : 2;
 $16 = $15;
 HEAP32[$rc>>2] = $16;
 $17 = HEAP32[$rc>>2]|0;
 STACKTOP = sp;return ($17|0);
}
function _getAutoVacuum($z) {
 $z = $z|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0;
 var $8 = 0, $9 = 0, $i = 0, $or$cond = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $1 = sp + 4|0;
 $i = sp;
 HEAP32[$1>>2] = $z;
 $2 = HEAP32[$1>>2]|0;
 $3 = (_sqlite3_stricmp($2,22272)|0);
 $4 = (0)==($3|0);
 do {
  if ($4) {
   HEAP32[$0>>2] = 0;
  } else {
   $5 = HEAP32[$1>>2]|0;
   $6 = (_sqlite3_stricmp($5,21976)|0);
   $7 = (0)==($6|0);
   if ($7) {
    HEAP32[$0>>2] = 1;
    break;
   }
   $8 = HEAP32[$1>>2]|0;
   $9 = (_sqlite3_stricmp($8,22280)|0);
   $10 = (0)==($9|0);
   if ($10) {
    HEAP32[$0>>2] = 2;
    break;
   } else {
    $11 = HEAP32[$1>>2]|0;
    $12 = (_sqlite3Atoi($11)|0);
    HEAP32[$i>>2] = $12;
    $13 = HEAP32[$i>>2]|0;
    $14 = ($13|0)>=(0);
    $15 = HEAP32[$i>>2]|0;
    $16 = ($15|0)<=(2);
    $or$cond = $14 & $16;
    $17 = HEAP32[$i>>2]|0;
    $18 = $or$cond ? $17 : 0;
    $19 = $18&255;
    $20 = $19&255;
    HEAP32[$0>>2] = $20;
    break;
   }
  }
 } while(0);
 $21 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($21|0);
}
function _sqlite3BtreeSetAutoVacuum($p,$autoVacuum) {
 $p = $p|0;
 $autoVacuum = $autoVacuum|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $av = 0, $pBt = 0;
 var $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 12|0;
 $1 = sp + 8|0;
 $pBt = sp + 4|0;
 $rc = sp;
 $av = sp + 16|0;
 HEAP32[$0>>2] = $p;
 HEAP32[$1>>2] = $autoVacuum;
 $2 = HEAP32[$0>>2]|0;
 $3 = (($2) + 4|0);
 $4 = HEAP32[$3>>2]|0;
 HEAP32[$pBt>>2] = $4;
 HEAP32[$rc>>2] = 0;
 $5 = HEAP32[$1>>2]|0;
 $6 = $5&255;
 HEAP8[$av>>0] = $6;
 $7 = HEAP32[$0>>2]|0;
 _sqlite3BtreeEnter($7);
 $8 = HEAP32[$pBt>>2]|0;
 $9 = (($8) + 22|0);
 $10 = HEAP16[$9>>1]|0;
 $11 = $10&65535;
 $12 = $11 & 2;
 $13 = ($12|0)!=(0);
 if ($13) {
  $14 = HEAP8[$av>>0]|0;
  $15 = $14&255;
  $16 = ($15|0)!=(0);
  $17 = $16 ? 1 : 0;
  $18 = HEAP32[$pBt>>2]|0;
  $19 = (($18) + 17|0);
  $20 = HEAP8[$19>>0]|0;
  $21 = $20&255;
  $22 = ($17|0)!=($21|0);
  if ($22) {
   HEAP32[$rc>>2] = 8;
   $37 = HEAP32[$rc>>2]|0;
   STACKTOP = sp;return ($37|0);
  }
 }
 $23 = HEAP8[$av>>0]|0;
 $24 = $23&255;
 $25 = ($24|0)!=(0);
 $26 = $25 ? 1 : 0;
 $27 = $26&255;
 $28 = HEAP32[$pBt>>2]|0;
 $29 = (($28) + 17|0);
 HEAP8[$29>>0] = $27;
 $30 = HEAP8[$av>>0]|0;
 $31 = $30&255;
 $32 = ($31|0)==(2);
 $33 = $32 ? 1 : 0;
 $34 = $33&255;
 $35 = HEAP32[$pBt>>2]|0;
 $36 = (($35) + 18|0);
 HEAP8[$36>>0] = $34;
 $37 = HEAP32[$rc>>2]|0;
 STACKTOP = sp;return ($37|0);
}
function _changeTempStorage($pParse,$zStorageType) {
 $pParse = $pParse|0;
 $zStorageType = $zStorageType|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0;
 var $9 = 0, $db = 0, $ts = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 16|0;
 $1 = sp + 12|0;
 $2 = sp + 8|0;
 $ts = sp + 4|0;
 $db = sp;
 HEAP32[$1>>2] = $pParse;
 HEAP32[$2>>2] = $zStorageType;
 $3 = HEAP32[$2>>2]|0;
 $4 = (_getTempStore($3)|0);
 HEAP32[$ts>>2] = $4;
 $5 = HEAP32[$1>>2]|0;
 $6 = HEAP32[$5>>2]|0;
 HEAP32[$db>>2] = $6;
 $7 = HEAP32[$db>>2]|0;
 $8 = (($7) + 63|0);
 $9 = HEAP8[$8>>0]|0;
 $10 = $9&255;
 $11 = HEAP32[$ts>>2]|0;
 $12 = ($10|0)==($11|0);
 if ($12) {
  HEAP32[$0>>2] = 0;
  $20 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($20|0);
 }
 $13 = HEAP32[$1>>2]|0;
 $14 = (_invalidateTempStorage($13)|0);
 $15 = ($14|0)!=(0);
 if ($15) {
  HEAP32[$0>>2] = 1;
  $20 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($20|0);
 } else {
  $16 = HEAP32[$ts>>2]|0;
  $17 = $16&255;
  $18 = HEAP32[$db>>2]|0;
  $19 = (($18) + 63|0);
  HEAP8[$19>>0] = $17;
  HEAP32[$0>>2] = 0;
  $20 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($20|0);
 }
 return 0|0;
}
function _invalidateTempStorage($pParse) {
 $pParse = $pParse|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $db = 0, $vararg_buffer = 0, label = 0;
 var sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $vararg_buffer = sp;
 $0 = sp + 12|0;
 $1 = sp + 8|0;
 $db = sp + 4|0;
 HEAP32[$1>>2] = $pParse;
 $2 = HEAP32[$1>>2]|0;
 $3 = HEAP32[$2>>2]|0;
 HEAP32[$db>>2] = $3;
 $4 = HEAP32[$db>>2]|0;
 $5 = (($4) + 16|0);
 $6 = HEAP32[$5>>2]|0;
 $7 = (($6) + 16|0);
 $8 = (($7) + 4|0);
 $9 = HEAP32[$8>>2]|0;
 $10 = ($9|0)!=(0|0);
 do {
  if ($10) {
   $11 = HEAP32[$db>>2]|0;
   $12 = (($11) + 62|0);
   $13 = HEAP8[$12>>0]|0;
   $14 = ($13<<24>>24)!=(0);
   if ($14) {
    $15 = HEAP32[$db>>2]|0;
    $16 = (($15) + 16|0);
    $17 = HEAP32[$16>>2]|0;
    $18 = (($17) + 16|0);
    $19 = (($18) + 4|0);
    $20 = HEAP32[$19>>2]|0;
    $21 = (_sqlite3BtreeIsInReadTrans($20)|0);
    $22 = ($21|0)!=(0);
    if (!($22)) {
     $24 = HEAP32[$db>>2]|0;
     $25 = (($24) + 16|0);
     $26 = HEAP32[$25>>2]|0;
     $27 = (($26) + 16|0);
     $28 = (($27) + 4|0);
     $29 = HEAP32[$28>>2]|0;
     (_sqlite3BtreeClose($29)|0);
     $30 = HEAP32[$db>>2]|0;
     $31 = (($30) + 16|0);
     $32 = HEAP32[$31>>2]|0;
     $33 = (($32) + 16|0);
     $34 = (($33) + 4|0);
     HEAP32[$34>>2] = 0;
     $35 = HEAP32[$db>>2]|0;
     _sqlite3ResetAllSchemasOfConnection($35);
     break;
    }
   }
   $23 = HEAP32[$1>>2]|0;
   _sqlite3ErrorMsg($23,22208,$vararg_buffer);
   HEAP32[$0>>2] = 1;
   $36 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($36|0);
  }
 } while(0);
 HEAP32[$0>>2] = 0;
 $36 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($36|0);
}
function _setAllPagerFlags($db) {
 $db = $db|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $n = 0, $pDb = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $pDb = sp + 4|0;
 $n = sp;
 HEAP32[$0>>2] = $db;
 $1 = HEAP32[$0>>2]|0;
 $2 = (($1) + 62|0);
 $3 = HEAP8[$2>>0]|0;
 $4 = ($3<<24>>24)!=(0);
 if (!($4)) {
  STACKTOP = sp;return;
 }
 $5 = HEAP32[$0>>2]|0;
 $6 = (($5) + 16|0);
 $7 = HEAP32[$6>>2]|0;
 HEAP32[$pDb>>2] = $7;
 $8 = HEAP32[$0>>2]|0;
 $9 = (($8) + 20|0);
 $10 = HEAP32[$9>>2]|0;
 HEAP32[$n>>2] = $10;
 while(1) {
  $11 = HEAP32[$n>>2]|0;
  $12 = (($11) + -1)|0;
  HEAP32[$n>>2] = $12;
  $13 = ($11|0)>(0);
  if (!($13)) {
   break;
  }
  $14 = HEAP32[$pDb>>2]|0;
  $15 = (($14) + 4|0);
  $16 = HEAP32[$15>>2]|0;
  $17 = ($16|0)!=(0|0);
  if ($17) {
   $18 = HEAP32[$pDb>>2]|0;
   $19 = (($18) + 4|0);
   $20 = HEAP32[$19>>2]|0;
   $21 = HEAP32[$pDb>>2]|0;
   $22 = (($21) + 8|0);
   $23 = HEAP8[$22>>0]|0;
   $24 = $23&255;
   $25 = HEAP32[$0>>2]|0;
   $26 = (($25) + 24|0);
   $27 = HEAP32[$26>>2]|0;
   $28 = $27 & 28;
   $29 = $24 | $28;
   (_sqlite3BtreeSetPagerFlags($20,$29)|0);
  }
  $30 = HEAP32[$pDb>>2]|0;
  $31 = (($30) + 16|0);
  HEAP32[$pDb>>2] = $31;
 }
 STACKTOP = sp;return;
}
function _sqlite3LogEstToInt($x) {
 $x = $x|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0;
 var $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0;
 var $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0;
 var $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0;
 var $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0;
 var $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $n = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 8|0;
 $1 = sp + 16|0;
 $n = sp;
 HEAP16[$1>>1] = $x;
 $2 = HEAP16[$1>>1]|0;
 $3 = $2 << 16 >> 16;
 $4 = ($3|0)<(10);
 if ($4) {
  $5 = $0;
  $6 = $5;
  HEAP32[$6>>2] = 1;
  $7 = (($5) + 4)|0;
  $8 = $7;
  HEAP32[$8>>2] = 0;
  $104 = $0;
  $105 = $104;
  $106 = HEAP32[$105>>2]|0;
  $107 = (($104) + 4)|0;
  $108 = $107;
  $109 = HEAP32[$108>>2]|0;
  tempRet0 = $109;
  STACKTOP = sp;return ($106|0);
 }
 $9 = HEAP16[$1>>1]|0;
 $10 = $9 << 16 >> 16;
 $11 = (($10|0) % 10)&-1;
 $12 = ($11|0)<(0);
 $13 = $12 << 31 >> 31;
 $14 = $n;
 $15 = $14;
 HEAP32[$15>>2] = $11;
 $16 = (($14) + 4)|0;
 $17 = $16;
 HEAP32[$17>>2] = $13;
 $18 = HEAP16[$1>>1]|0;
 $19 = $18 << 16 >> 16;
 $20 = (($19|0) / 10)&-1;
 $21 = $20&65535;
 HEAP16[$1>>1] = $21;
 $22 = $n;
 $23 = $22;
 $24 = HEAP32[$23>>2]|0;
 $25 = (($22) + 4)|0;
 $26 = $25;
 $27 = HEAP32[$26>>2]|0;
 $28 = ($27>>>0)>(0);
 $29 = ($24>>>0)>=(5);
 $30 = ($27|0)==(0);
 $31 = $30 & $29;
 $32 = $28 | $31;
 $33 = $n;
 $34 = $33;
 $35 = HEAP32[$34>>2]|0;
 $36 = (($33) + 4)|0;
 $37 = $36;
 $38 = HEAP32[$37>>2]|0;
 if ($32) {
  $39 = (_i64Subtract(($35|0),($38|0),2,0)|0);
  $40 = tempRet0;
  $41 = $n;
  $42 = $41;
  HEAP32[$42>>2] = $39;
  $43 = (($41) + 4)|0;
  $44 = $43;
  HEAP32[$44>>2] = $40;
 } else {
  $45 = ($38>>>0)>(0);
  $46 = ($35>>>0)>=(1);
  $47 = ($38|0)==(0);
  $48 = $47 & $46;
  $49 = $45 | $48;
  if ($49) {
   $50 = $n;
   $51 = $50;
   $52 = HEAP32[$51>>2]|0;
   $53 = (($50) + 4)|0;
   $54 = $53;
   $55 = HEAP32[$54>>2]|0;
   $56 = (_i64Subtract(($52|0),($55|0),1,0)|0);
   $57 = tempRet0;
   $58 = $n;
   $59 = $58;
   HEAP32[$59>>2] = $56;
   $60 = (($58) + 4)|0;
   $61 = $60;
   HEAP32[$61>>2] = $57;
  }
 }
 $62 = HEAP16[$1>>1]|0;
 $63 = $62 << 16 >> 16;
 $64 = ($63|0)>=(3);
 if (!($64)) {
  $87 = $n;
  $88 = $87;
  $89 = HEAP32[$88>>2]|0;
  $90 = (($87) + 4)|0;
  $91 = $90;
  $92 = HEAP32[$91>>2]|0;
  $93 = (_i64Add(($89|0),($92|0),8,0)|0);
  $94 = tempRet0;
  $95 = HEAP16[$1>>1]|0;
  $96 = $95 << 16 >> 16;
  $97 = (3 - ($96))|0;
  $98 = (_bitshift64Lshr(($93|0),($94|0),($97|0))|0);
  $99 = tempRet0;
  $100 = $0;
  $101 = $100;
  HEAP32[$101>>2] = $98;
  $102 = (($100) + 4)|0;
  $103 = $102;
  HEAP32[$103>>2] = $99;
  $104 = $0;
  $105 = $104;
  $106 = HEAP32[$105>>2]|0;
  $107 = (($104) + 4)|0;
  $108 = $107;
  $109 = HEAP32[$108>>2]|0;
  tempRet0 = $109;
  STACKTOP = sp;return ($106|0);
 }
 $65 = HEAP16[$1>>1]|0;
 $66 = $65 << 16 >> 16;
 $67 = ($66|0)>(60);
 if ($67) {
  $83 = -1;$86 = 2147483647;
 } else {
  $68 = $n;
  $69 = $68;
  $70 = HEAP32[$69>>2]|0;
  $71 = (($68) + 4)|0;
  $72 = $71;
  $73 = HEAP32[$72>>2]|0;
  $74 = (_i64Add(($70|0),($73|0),8,0)|0);
  $75 = tempRet0;
  $76 = HEAP16[$1>>1]|0;
  $77 = $76 << 16 >> 16;
  $78 = (($77) - 3)|0;
  $79 = (_bitshift64Shl(($74|0),($75|0),($78|0))|0);
  $80 = tempRet0;
  $83 = $79;$86 = $80;
 }
 $81 = $0;
 $82 = $81;
 HEAP32[$82>>2] = $83;
 $84 = (($81) + 4)|0;
 $85 = $84;
 HEAP32[$85>>2] = $86;
 $104 = $0;
 $105 = $104;
 $106 = HEAP32[$105>>2]|0;
 $107 = (($104) + 4)|0;
 $108 = $107;
 $109 = HEAP32[$108>>2]|0;
 tempRet0 = $109;
 STACKTOP = sp;return ($106|0);
}
function _actionName($action) {
 $action = $action|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $zName = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $zName = sp;
 HEAP8[$0>>0] = $action;
 $1 = HEAP8[$0>>0]|0;
 $2 = $1&255;
 if ((($2|0) == 9)) {
  HEAP32[$zName>>2] = 22168;
 } else if ((($2|0) == 7)) {
  HEAP32[$zName>>2] = 22136;
 } else if ((($2|0) == 8)) {
  HEAP32[$zName>>2] = 22152;
 } else if ((($2|0) == 6)) {
  HEAP32[$zName>>2] = 22176;
 } else {
  HEAP32[$zName>>2] = 22192;
 }
 $3 = HEAP32[$zName>>2]|0;
 STACKTOP = sp;return ($3|0);
}
function _sqlite3FkLocateIndex($pParse,$pParent,$pFKey,$ppIdx,$paiCol) {
 $pParse = $pParse|0;
 $pParent = $pParent|0;
 $pFKey = $pFKey|0;
 $ppIdx = $ppIdx|0;
 $paiCol = $paiCol|0;
 var $$ = 0, $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0;
 var $115 = 0, $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0;
 var $133 = 0, $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0;
 var $151 = 0, $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0;
 var $17 = 0, $170 = 0, $171 = 0, $172 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0;
 var $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0;
 var $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0;
 var $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0;
 var $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $aiCol = 0, $i = 0, $i1 = 0, $iCol = 0, $j = 0, $nCol = 0;
 var $pIdx = 0, $vararg_buffer = 0, $vararg_ptr1 = 0, $zDfltColl = 0, $zIdxCol = 0, $zKey = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 80|0;
 $vararg_buffer = sp;
 $0 = sp + 64|0;
 $1 = sp + 60|0;
 $2 = sp + 56|0;
 $3 = sp + 52|0;
 $4 = sp + 48|0;
 $5 = sp + 44|0;
 $pIdx = sp + 40|0;
 $aiCol = sp + 36|0;
 $nCol = sp + 32|0;
 $zKey = sp + 28|0;
 $i = sp + 24|0;
 $i1 = sp + 20|0;
 $j = sp + 16|0;
 $iCol = sp + 68|0;
 $zDfltColl = sp + 12|0;
 $zIdxCol = sp + 8|0;
 HEAP32[$1>>2] = $pParse;
 HEAP32[$2>>2] = $pParent;
 HEAP32[$3>>2] = $pFKey;
 HEAP32[$4>>2] = $ppIdx;
 HEAP32[$5>>2] = $paiCol;
 HEAP32[$pIdx>>2] = 0;
 HEAP32[$aiCol>>2] = 0;
 $6 = HEAP32[$3>>2]|0;
 $7 = (($6) + 20|0);
 $8 = HEAP32[$7>>2]|0;
 HEAP32[$nCol>>2] = $8;
 $9 = HEAP32[$3>>2]|0;
 $10 = (($9) + 36|0);
 $11 = (($10) + 4|0);
 $12 = HEAP32[$11>>2]|0;
 HEAP32[$zKey>>2] = $12;
 $13 = HEAP32[$nCol>>2]|0;
 $14 = ($13|0)==(1);
 do {
  if ($14) {
   $15 = HEAP32[$2>>2]|0;
   $16 = (($15) + 36|0);
   $17 = HEAP16[$16>>1]|0;
   $18 = $17 << 16 >> 16;
   $19 = ($18|0)>=(0);
   if ($19) {
    $20 = HEAP32[$zKey>>2]|0;
    $21 = ($20|0)!=(0|0);
    if (!($21)) {
     HEAP32[$0>>2] = 0;
     $172 = HEAP32[$0>>2]|0;
     STACKTOP = sp;return ($172|0);
    }
    $22 = HEAP32[$2>>2]|0;
    $23 = (($22) + 36|0);
    $24 = HEAP16[$23>>1]|0;
    $25 = $24 << 16 >> 16;
    $26 = HEAP32[$2>>2]|0;
    $27 = (($26) + 4|0);
    $28 = HEAP32[$27>>2]|0;
    $29 = (($28) + (($25*24)|0)|0);
    $30 = HEAP32[$29>>2]|0;
    $31 = HEAP32[$zKey>>2]|0;
    $32 = (_sqlite3_stricmp($30,$31)|0);
    $33 = ($32|0)!=(0);
    if (!($33)) {
     HEAP32[$0>>2] = 0;
     $172 = HEAP32[$0>>2]|0;
     STACKTOP = sp;return ($172|0);
    }
   }
  } else {
   $34 = HEAP32[$5>>2]|0;
   $35 = ($34|0)!=(0|0);
   if ($35) {
    $36 = HEAP32[$1>>2]|0;
    $37 = HEAP32[$36>>2]|0;
    $38 = HEAP32[$nCol>>2]|0;
    $39 = $38<<2;
    $40 = (_sqlite3DbMallocRaw($37,$39,0)|0);
    HEAP32[$aiCol>>2] = $40;
    $41 = HEAP32[$aiCol>>2]|0;
    $42 = ($41|0)!=(0|0);
    if ($42) {
     $43 = HEAP32[$aiCol>>2]|0;
     $44 = HEAP32[$5>>2]|0;
     HEAP32[$44>>2] = $43;
     break;
    }
    HEAP32[$0>>2] = 1;
    $172 = HEAP32[$0>>2]|0;
    STACKTOP = sp;return ($172|0);
   }
  }
 } while(0);
 $45 = HEAP32[$2>>2]|0;
 $46 = (($45) + 8|0);
 $47 = HEAP32[$46>>2]|0;
 HEAP32[$pIdx>>2] = $47;
 L17: while(1) {
  $48 = HEAP32[$pIdx>>2]|0;
  $49 = ($48|0)!=(0|0);
  if (!($49)) {
   break;
  }
  $50 = HEAP32[$pIdx>>2]|0;
  $51 = (($50) + 50|0);
  $52 = HEAP16[$51>>1]|0;
  $53 = $52&65535;
  $54 = HEAP32[$nCol>>2]|0;
  $55 = ($53|0)==($54|0);
  do {
   if ($55) {
    $56 = HEAP32[$pIdx>>2]|0;
    $57 = (($56) + 54|0);
    $58 = HEAP8[$57>>0]|0;
    $59 = $58&255;
    $60 = ($59|0)!=(0);
    if ($60) {
     $61 = HEAP32[$zKey>>2]|0;
     $62 = ($61|0)==(0|0);
     if ($62) {
      $63 = HEAP32[$pIdx>>2]|0;
      $64 = (($63) + 55|0);
      $65 = HEAP8[$64>>0]|0;
      $66 = $65 & 3;
      $67 = $66&255;
      $68 = ($67|0)==(2);
      if ($68) {
       label = 17;
       break L17;
      } else {
       break;
      }
     }
     HEAP32[$i1>>2] = 0;
     while(1) {
      $84 = HEAP32[$i1>>2]|0;
      $85 = HEAP32[$nCol>>2]|0;
      $86 = ($84|0)<($85|0);
      if (!($86)) {
       break;
      }
      $87 = HEAP32[$i1>>2]|0;
      $88 = HEAP32[$pIdx>>2]|0;
      $89 = (($88) + 4|0);
      $90 = HEAP32[$89>>2]|0;
      $91 = (($90) + ($87<<1)|0);
      $92 = HEAP16[$91>>1]|0;
      HEAP16[$iCol>>1] = $92;
      $93 = HEAP16[$iCol>>1]|0;
      $94 = $93 << 16 >> 16;
      $95 = HEAP32[$2>>2]|0;
      $96 = (($95) + 4|0);
      $97 = HEAP32[$96>>2]|0;
      $98 = (($97) + (($94*24)|0)|0);
      $99 = (($98) + 16|0);
      $100 = HEAP32[$99>>2]|0;
      HEAP32[$zDfltColl>>2] = $100;
      $101 = HEAP32[$zDfltColl>>2]|0;
      $102 = ($101|0)!=(0|0);
      $$ = $102 ? $100 : 3320;
      HEAP32[$zDfltColl>>2] = $$;
      $103 = HEAP32[$i1>>2]|0;
      $104 = HEAP32[$pIdx>>2]|0;
      $105 = (($104) + 32|0);
      $106 = HEAP32[$105>>2]|0;
      $107 = (($106) + ($103<<2)|0);
      $108 = HEAP32[$107>>2]|0;
      $109 = HEAP32[$zDfltColl>>2]|0;
      $110 = (_sqlite3_stricmp($108,$109)|0);
      $111 = ($110|0)!=(0);
      if ($111) {
       break;
      }
      $112 = HEAP16[$iCol>>1]|0;
      $113 = $112 << 16 >> 16;
      $114 = HEAP32[$2>>2]|0;
      $115 = (($114) + 4|0);
      $116 = HEAP32[$115>>2]|0;
      $117 = (($116) + (($113*24)|0)|0);
      $118 = HEAP32[$117>>2]|0;
      HEAP32[$zIdxCol>>2] = $118;
      HEAP32[$j>>2] = 0;
      while(1) {
       $119 = HEAP32[$j>>2]|0;
       $120 = HEAP32[$nCol>>2]|0;
       $121 = ($119|0)<($120|0);
       if (!($121)) {
        break;
       }
       $122 = HEAP32[$j>>2]|0;
       $123 = HEAP32[$3>>2]|0;
       $124 = (($123) + 36|0);
       $125 = (($124) + ($122<<3)|0);
       $126 = (($125) + 4|0);
       $127 = HEAP32[$126>>2]|0;
       $128 = HEAP32[$zIdxCol>>2]|0;
       $129 = (_sqlite3_stricmp($127,$128)|0);
       $130 = ($129|0)==(0);
       if ($130) {
        label = 27;
        break;
       }
       $141 = HEAP32[$j>>2]|0;
       $142 = (($141) + 1)|0;
       HEAP32[$j>>2] = $142;
      }
      if ((label|0) == 27) {
       label = 0;
       $131 = HEAP32[$aiCol>>2]|0;
       $132 = ($131|0)!=(0|0);
       if ($132) {
        $133 = HEAP32[$j>>2]|0;
        $134 = HEAP32[$3>>2]|0;
        $135 = (($134) + 36|0);
        $136 = (($135) + ($133<<3)|0);
        $137 = HEAP32[$136>>2]|0;
        $138 = HEAP32[$i1>>2]|0;
        $139 = HEAP32[$aiCol>>2]|0;
        $140 = (($139) + ($138<<2)|0);
        HEAP32[$140>>2] = $137;
       }
      }
      $143 = HEAP32[$j>>2]|0;
      $144 = HEAP32[$nCol>>2]|0;
      $145 = ($143|0)==($144|0);
      if ($145) {
       break;
      }
      $146 = HEAP32[$i1>>2]|0;
      $147 = (($146) + 1)|0;
      HEAP32[$i1>>2] = $147;
     }
     $148 = HEAP32[$i1>>2]|0;
     $149 = HEAP32[$nCol>>2]|0;
     $150 = ($148|0)==($149|0);
     if ($150) {
      break L17;
     }
    }
   }
  } while(0);
  $151 = HEAP32[$pIdx>>2]|0;
  $152 = (($151) + 20|0);
  $153 = HEAP32[$152>>2]|0;
  HEAP32[$pIdx>>2] = $153;
 }
 L41: do {
  if ((label|0) == 17) {
   $69 = HEAP32[$aiCol>>2]|0;
   $70 = ($69|0)!=(0|0);
   if ($70) {
    HEAP32[$i>>2] = 0;
    while(1) {
     $71 = HEAP32[$i>>2]|0;
     $72 = HEAP32[$nCol>>2]|0;
     $73 = ($71|0)<($72|0);
     if (!($73)) {
      break L41;
     }
     $74 = HEAP32[$i>>2]|0;
     $75 = HEAP32[$3>>2]|0;
     $76 = (($75) + 36|0);
     $77 = (($76) + ($74<<3)|0);
     $78 = HEAP32[$77>>2]|0;
     $79 = HEAP32[$i>>2]|0;
     $80 = HEAP32[$aiCol>>2]|0;
     $81 = (($80) + ($79<<2)|0);
     HEAP32[$81>>2] = $78;
     $82 = HEAP32[$i>>2]|0;
     $83 = (($82) + 1)|0;
     HEAP32[$i>>2] = $83;
    }
   }
  }
 } while(0);
 $154 = HEAP32[$pIdx>>2]|0;
 $155 = ($154|0)!=(0|0);
 if ($155) {
  $170 = HEAP32[$pIdx>>2]|0;
  $171 = HEAP32[$4>>2]|0;
  HEAP32[$171>>2] = $170;
  HEAP32[$0>>2] = 0;
  $172 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($172|0);
 }
 $156 = HEAP32[$1>>2]|0;
 $157 = (($156) + 438|0);
 $158 = HEAP8[$157>>0]|0;
 $159 = ($158<<24>>24)!=(0);
 if (!($159)) {
  $160 = HEAP32[$1>>2]|0;
  $161 = HEAP32[$3>>2]|0;
  $162 = HEAP32[$161>>2]|0;
  $163 = HEAP32[$162>>2]|0;
  $164 = HEAP32[$3>>2]|0;
  $165 = (($164) + 8|0);
  $166 = HEAP32[$165>>2]|0;
  HEAP32[$vararg_buffer>>2] = $163;
  $vararg_ptr1 = (($vararg_buffer) + 4|0);
  HEAP32[$vararg_ptr1>>2] = $166;
  _sqlite3ErrorMsg($160,22088,$vararg_buffer);
 }
 $167 = HEAP32[$1>>2]|0;
 $168 = HEAP32[$167>>2]|0;
 $169 = HEAP32[$aiCol>>2]|0;
 _sqlite3DbFree($168,$169);
 HEAP32[$0>>2] = 1;
 $172 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($172|0);
}
function _sqlite3IndexAffinityStr($v,$pIdx) {
 $v = $v|0;
 $pIdx = $pIdx|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $7 = 0, $8 = 0, $9 = 0, $db = 0, $n = 0, $pTab = 0, $x = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 20|0;
 $1 = sp + 16|0;
 $2 = sp + 12|0;
 $n = sp + 8|0;
 $pTab = sp + 4|0;
 $db = sp;
 $x = sp + 24|0;
 HEAP32[$1>>2] = $v;
 HEAP32[$2>>2] = $pIdx;
 $3 = HEAP32[$2>>2]|0;
 $4 = (($3) + 16|0);
 $5 = HEAP32[$4>>2]|0;
 $6 = ($5|0)!=(0|0);
 if (!($6)) {
  $7 = HEAP32[$2>>2]|0;
  $8 = (($7) + 12|0);
  $9 = HEAP32[$8>>2]|0;
  HEAP32[$pTab>>2] = $9;
  $10 = HEAP32[$1>>2]|0;
  $11 = (_sqlite3VdbeDb($10)|0);
  HEAP32[$db>>2] = $11;
  $12 = HEAP32[$2>>2]|0;
  $13 = (($12) + 52|0);
  $14 = HEAP16[$13>>1]|0;
  $15 = $14&65535;
  $16 = (($15) + 1)|0;
  $17 = ($16|0)<(0);
  $18 = $17 << 31 >> 31;
  $19 = (_sqlite3DbMallocRaw(0,$16,$18)|0);
  $20 = HEAP32[$2>>2]|0;
  $21 = (($20) + 16|0);
  HEAP32[$21>>2] = $19;
  $22 = HEAP32[$2>>2]|0;
  $23 = (($22) + 16|0);
  $24 = HEAP32[$23>>2]|0;
  $25 = ($24|0)!=(0|0);
  if (!($25)) {
   $26 = HEAP32[$db>>2]|0;
   $27 = (($26) + 64|0);
   HEAP8[$27>>0] = 1;
   HEAP32[$0>>2] = 0;
   $67 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($67|0);
  }
  HEAP32[$n>>2] = 0;
  while(1) {
   $28 = HEAP32[$n>>2]|0;
   $29 = HEAP32[$2>>2]|0;
   $30 = (($29) + 52|0);
   $31 = HEAP16[$30>>1]|0;
   $32 = $31&65535;
   $33 = ($28|0)<($32|0);
   $34 = HEAP32[$n>>2]|0;
   $35 = HEAP32[$2>>2]|0;
   if (!($33)) {
    break;
   }
   $36 = (($35) + 4|0);
   $37 = HEAP32[$36>>2]|0;
   $38 = (($37) + ($34<<1)|0);
   $39 = HEAP16[$38>>1]|0;
   HEAP16[$x>>1] = $39;
   $40 = HEAP16[$x>>1]|0;
   $41 = $40 << 16 >> 16;
   $42 = ($41|0)<(0);
   if ($42) {
    $53 = 68;
   } else {
    $43 = HEAP16[$x>>1]|0;
    $44 = $43 << 16 >> 16;
    $45 = HEAP32[$pTab>>2]|0;
    $46 = (($45) + 4|0);
    $47 = HEAP32[$46>>2]|0;
    $48 = (($47) + (($44*24)|0)|0);
    $49 = (($48) + 21|0);
    $50 = HEAP8[$49>>0]|0;
    $51 = $50 << 24 >> 24;
    $53 = $51;
   }
   $52 = $53&255;
   $54 = HEAP32[$n>>2]|0;
   $55 = HEAP32[$2>>2]|0;
   $56 = (($55) + 16|0);
   $57 = HEAP32[$56>>2]|0;
   $58 = (($57) + ($54)|0);
   HEAP8[$58>>0] = $52;
   $59 = HEAP32[$n>>2]|0;
   $60 = (($59) + 1)|0;
   HEAP32[$n>>2] = $60;
  }
  $61 = (($35) + 16|0);
  $62 = HEAP32[$61>>2]|0;
  $63 = (($62) + ($34)|0);
  HEAP8[$63>>0] = 0;
 }
 $64 = HEAP32[$2>>2]|0;
 $65 = (($64) + 16|0);
 $66 = HEAP32[$65>>2]|0;
 HEAP32[$0>>2] = $66;
 $67 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($67|0);
}
function _sqlite3RegisterLikeFunctions($db,$caseSensitive) {
 $db = $db|0;
 $caseSensitive = $caseSensitive|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $pInfo = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $1 = sp + 4|0;
 $pInfo = sp;
 HEAP32[$0>>2] = $db;
 HEAP32[$1>>2] = $caseSensitive;
 $2 = HEAP32[$1>>2]|0;
 $3 = ($2|0)!=(0);
 if ($3) {
  HEAP32[$pInfo>>2] = 22080;
 } else {
  HEAP32[$pInfo>>2] = 8776;
 }
 $4 = HEAP32[$0>>2]|0;
 $5 = HEAP32[$pInfo>>2]|0;
 (_sqlite3CreateFunc($4,8784,2,1,$5,34,0,0,0)|0);
 $6 = HEAP32[$0>>2]|0;
 $7 = HEAP32[$pInfo>>2]|0;
 (_sqlite3CreateFunc($6,8784,3,1,$7,34,0,0,0)|0);
 $8 = HEAP32[$0>>2]|0;
 (_sqlite3CreateFunc($8,8768,2,1,1208,34,0,0,0)|0);
 $9 = HEAP32[$0>>2]|0;
 _setLikeOptFlag($9,8768,12);
 $10 = HEAP32[$0>>2]|0;
 $11 = HEAP32[$1>>2]|0;
 $12 = ($11|0)!=(0);
 $13 = $12 ? 12 : 4;
 $14 = $13&255;
 _setLikeOptFlag($10,8784,$14);
 STACKTOP = sp;return;
}
function _sqlite3ExprCacheClear($pParse) {
 $pParse = $pParse|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $i = 0, $p = 0, label = 0;
 var sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $i = sp + 4|0;
 $p = sp;
 HEAP32[$0>>2] = $pParse;
 HEAP32[$i>>2] = 0;
 $1 = HEAP32[$0>>2]|0;
 $2 = (($1) + 116|0);
 HEAP32[$p>>2] = $2;
 while(1) {
  $3 = HEAP32[$i>>2]|0;
  $4 = ($3|0)<(10);
  if (!($4)) {
   break;
  }
  $5 = HEAP32[$p>>2]|0;
  $6 = (($5) + 12|0);
  $7 = HEAP32[$6>>2]|0;
  $8 = ($7|0)!=(0);
  if ($8) {
   $9 = HEAP32[$0>>2]|0;
   $10 = HEAP32[$p>>2]|0;
   _cacheEntryClear($9,$10);
   $11 = HEAP32[$p>>2]|0;
   $12 = (($11) + 12|0);
   HEAP32[$12>>2] = 0;
  }
  $13 = HEAP32[$i>>2]|0;
  $14 = (($13) + 1)|0;
  HEAP32[$i>>2] = $14;
  $15 = HEAP32[$p>>2]|0;
  $16 = (($15) + 20|0);
  HEAP32[$p>>2] = $16;
 }
 STACKTOP = sp;return;
}
function _sqlite3OpenTableAndIndices($pParse,$pTab,$op,$iBase,$aToOpen,$piDataCur,$piIdxCur) {
 $pParse = $pParse|0;
 $pTab = $pTab|0;
 $op = $op|0;
 $iBase = $iBase|0;
 $aToOpen = $aToOpen|0;
 $piDataCur = $piDataCur|0;
 $piIdxCur = $piIdxCur|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0;
 var $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0;
 var $i = 0, $iDataCur = 0, $iDb = 0, $iIdxCur = 0, $or$cond = 0, $pIdx = 0, $v = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 64|0;
 $0 = sp + 52|0;
 $1 = sp + 48|0;
 $2 = sp + 44|0;
 $3 = sp + 40|0;
 $4 = sp + 36|0;
 $5 = sp + 32|0;
 $6 = sp + 28|0;
 $7 = sp + 24|0;
 $i = sp + 20|0;
 $iDb = sp + 16|0;
 $iDataCur = sp + 12|0;
 $pIdx = sp + 8|0;
 $v = sp + 4|0;
 $iIdxCur = sp;
 HEAP32[$1>>2] = $pParse;
 HEAP32[$2>>2] = $pTab;
 HEAP32[$3>>2] = $op;
 HEAP32[$4>>2] = $iBase;
 HEAP32[$5>>2] = $aToOpen;
 HEAP32[$6>>2] = $piDataCur;
 HEAP32[$7>>2] = $piIdxCur;
 $8 = HEAP32[$2>>2]|0;
 $9 = (($8) + 44|0);
 $10 = HEAP8[$9>>0]|0;
 $11 = $10&255;
 $12 = $11 & 16;
 $13 = ($12|0)!=(0);
 if ($13) {
  HEAP32[$0>>2] = 0;
  $119 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($119|0);
 }
 $14 = HEAP32[$1>>2]|0;
 $15 = HEAP32[$14>>2]|0;
 $16 = HEAP32[$2>>2]|0;
 $17 = (($16) + 68|0);
 $18 = HEAP32[$17>>2]|0;
 $19 = (_sqlite3SchemaToIndex($15,$18)|0);
 HEAP32[$iDb>>2] = $19;
 $20 = HEAP32[$1>>2]|0;
 $21 = (_sqlite3GetVdbe($20)|0);
 HEAP32[$v>>2] = $21;
 $22 = HEAP32[$4>>2]|0;
 $23 = ($22|0)<(0);
 if ($23) {
  $24 = HEAP32[$1>>2]|0;
  $25 = (($24) + 68|0);
  $26 = HEAP32[$25>>2]|0;
  HEAP32[$4>>2] = $26;
 }
 $27 = HEAP32[$4>>2]|0;
 $28 = (($27) + 1)|0;
 HEAP32[$4>>2] = $28;
 HEAP32[$iDataCur>>2] = $27;
 $29 = HEAP32[$6>>2]|0;
 $30 = ($29|0)!=(0|0);
 if ($30) {
  $31 = HEAP32[$iDataCur>>2]|0;
  $32 = HEAP32[$6>>2]|0;
  HEAP32[$32>>2] = $31;
 }
 $33 = HEAP32[$2>>2]|0;
 $34 = (($33) + 44|0);
 $35 = HEAP8[$34>>0]|0;
 $36 = $35&255;
 $37 = $36 & 32;
 $38 = ($37|0)==(0);
 do {
  if ($38) {
   $39 = HEAP32[$5>>2]|0;
   $40 = ($39|0)==(0|0);
   if (!($40)) {
    $41 = HEAP32[$5>>2]|0;
    $42 = HEAP8[$41>>0]|0;
    $43 = $42&255;
    $44 = ($43|0)!=(0);
    if (!($44)) {
     label = 11;
     break;
    }
   }
   $45 = HEAP32[$1>>2]|0;
   $46 = HEAP32[$iDataCur>>2]|0;
   $47 = HEAP32[$iDb>>2]|0;
   $48 = HEAP32[$2>>2]|0;
   $49 = HEAP32[$3>>2]|0;
   _sqlite3OpenTable($45,$46,$47,$48,$49);
  } else {
   label = 11;
  }
 } while(0);
 if ((label|0) == 11) {
  $50 = HEAP32[$1>>2]|0;
  $51 = HEAP32[$iDb>>2]|0;
  $52 = HEAP32[$2>>2]|0;
  $53 = (($52) + 32|0);
  $54 = HEAP32[$53>>2]|0;
  $55 = HEAP32[$3>>2]|0;
  $56 = ($55|0)==(55);
  $57 = $56&1;
  $58 = $57&255;
  $59 = HEAP32[$2>>2]|0;
  $60 = HEAP32[$59>>2]|0;
  _sqlite3TableLock($50,$51,$54,$58,$60);
 }
 $61 = HEAP32[$7>>2]|0;
 $62 = ($61|0)!=(0|0);
 if ($62) {
  $63 = HEAP32[$4>>2]|0;
  $64 = HEAP32[$7>>2]|0;
  HEAP32[$64>>2] = $63;
 }
 HEAP32[$i>>2] = 0;
 $65 = HEAP32[$2>>2]|0;
 $66 = (($65) + 8|0);
 $67 = HEAP32[$66>>2]|0;
 HEAP32[$pIdx>>2] = $67;
 while(1) {
  $68 = HEAP32[$pIdx>>2]|0;
  $69 = ($68|0)!=(0|0);
  $70 = HEAP32[$4>>2]|0;
  if (!($69)) {
   break;
  }
  $71 = (($70) + 1)|0;
  HEAP32[$4>>2] = $71;
  HEAP32[$iIdxCur>>2] = $70;
  $72 = HEAP32[$pIdx>>2]|0;
  $73 = (($72) + 55|0);
  $74 = HEAP8[$73>>0]|0;
  $75 = $74 & 3;
  $76 = $75&255;
  $77 = ($76|0)==(2);
  if ($77) {
   $78 = HEAP32[$2>>2]|0;
   $79 = (($78) + 44|0);
   $80 = HEAP8[$79>>0]|0;
   $81 = $80&255;
   $82 = $81 & 32;
   $83 = ($82|0)!=(0);
   $84 = HEAP32[$6>>2]|0;
   $85 = ($84|0)!=(0|0);
   $or$cond = $83 & $85;
   if ($or$cond) {
    $86 = HEAP32[$iIdxCur>>2]|0;
    $87 = HEAP32[$6>>2]|0;
    HEAP32[$87>>2] = $86;
   }
  }
  $88 = HEAP32[$5>>2]|0;
  $89 = ($88|0)==(0|0);
  if ($89) {
   label = 21;
  } else {
   $90 = HEAP32[$i>>2]|0;
   $91 = (($90) + 1)|0;
   $92 = HEAP32[$5>>2]|0;
   $93 = (($92) + ($91)|0);
   $94 = HEAP8[$93>>0]|0;
   $95 = $94&255;
   $96 = ($95|0)!=(0);
   if ($96) {
    label = 21;
   }
  }
  if ((label|0) == 21) {
   label = 0;
   $97 = HEAP32[$v>>2]|0;
   $98 = HEAP32[$3>>2]|0;
   $99 = HEAP32[$iIdxCur>>2]|0;
   $100 = HEAP32[$pIdx>>2]|0;
   $101 = (($100) + 44|0);
   $102 = HEAP32[$101>>2]|0;
   $103 = HEAP32[$iDb>>2]|0;
   (_sqlite3VdbeAddOp3($97,$98,$99,$102,$103)|0);
   $104 = HEAP32[$1>>2]|0;
   $105 = HEAP32[$pIdx>>2]|0;
   _sqlite3VdbeSetP4KeyInfo($104,$105);
  }
  $106 = HEAP32[$pIdx>>2]|0;
  $107 = (($106) + 20|0);
  $108 = HEAP32[$107>>2]|0;
  HEAP32[$pIdx>>2] = $108;
  $109 = HEAP32[$i>>2]|0;
  $110 = (($109) + 1)|0;
  HEAP32[$i>>2] = $110;
 }
 $111 = HEAP32[$1>>2]|0;
 $112 = (($111) + 68|0);
 $113 = HEAP32[$112>>2]|0;
 $114 = ($70|0)>($113|0);
 if ($114) {
  $115 = HEAP32[$4>>2]|0;
  $116 = HEAP32[$1>>2]|0;
  $117 = (($116) + 68|0);
  HEAP32[$117>>2] = $115;
 }
 $118 = HEAP32[$i>>2]|0;
 HEAP32[$0>>2] = $118;
 $119 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($119|0);
}
function _setLikeOptFlag($db,$zName,$flagVal) {
 $db = $db|0;
 $zName = $zName|0;
 $flagVal = $flagVal|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $pDef = 0, label = 0;
 var sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $1 = sp + 4|0;
 $2 = sp + 12|0;
 $pDef = sp;
 HEAP32[$0>>2] = $db;
 HEAP32[$1>>2] = $zName;
 HEAP8[$2>>0] = $flagVal;
 $3 = HEAP32[$0>>2]|0;
 $4 = HEAP32[$1>>2]|0;
 $5 = HEAP32[$1>>2]|0;
 $6 = (_sqlite3Strlen30($5)|0);
 $7 = (_sqlite3FindFunction($3,$4,$6,2,1,0)|0);
 HEAP32[$pDef>>2] = $7;
 $8 = HEAP32[$pDef>>2]|0;
 $9 = ($8|0)!=(0|0);
 if (!($9)) {
  STACKTOP = sp;return;
 }
 $10 = HEAP8[$2>>0]|0;
 $11 = $10&255;
 $12 = HEAP32[$pDef>>2]|0;
 $13 = (($12) + 2|0);
 $14 = HEAP16[$13>>1]|0;
 $15 = $14&65535;
 $16 = $15 | $11;
 $17 = $16&65535;
 HEAP16[$13>>1] = $17;
 STACKTOP = sp;return;
}
function _getTempStore($z) {
 $z = $z|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0;
 var $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $1 = sp;
 HEAP32[$1>>2] = $z;
 $2 = HEAP32[$1>>2]|0;
 $3 = HEAP8[$2>>0]|0;
 $4 = $3 << 24 >> 24;
 $5 = ($4|0)>=(48);
 if ($5) {
  $6 = HEAP32[$1>>2]|0;
  $7 = HEAP8[$6>>0]|0;
  $8 = $7 << 24 >> 24;
  $9 = ($8|0)<=(50);
  if ($9) {
   $10 = HEAP32[$1>>2]|0;
   $11 = HEAP8[$10>>0]|0;
   $12 = $11 << 24 >> 24;
   $13 = (($12) - 48)|0;
   HEAP32[$0>>2] = $13;
  } else {
   label = 4;
  }
 } else {
  label = 4;
 }
 do {
  if ((label|0) == 4) {
   $14 = HEAP32[$1>>2]|0;
   $15 = (_sqlite3_stricmp($14,21448)|0);
   $16 = ($15|0)==(0);
   if ($16) {
    HEAP32[$0>>2] = 1;
    break;
   }
   $17 = HEAP32[$1>>2]|0;
   $18 = (_sqlite3_stricmp($17,4792)|0);
   $19 = ($18|0)==(0);
   if ($19) {
    HEAP32[$0>>2] = 2;
    break;
   } else {
    HEAP32[$0>>2] = 0;
    break;
   }
  }
 } while(0);
 $20 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($20|0);
}
function _sqlite3WalLimit($pWal,$0,$1) {
 $pWal = $pWal|0;
 $0 = $0|0;
 $1 = $1|0;
 var $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0;
 var label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $2 = sp + 8|0;
 $3 = sp;
 HEAP32[$2>>2] = $pWal;
 $4 = $3;
 $5 = $4;
 HEAP32[$5>>2] = $0;
 $6 = (($4) + 4)|0;
 $7 = $6;
 HEAP32[$7>>2] = $1;
 $8 = HEAP32[$2>>2]|0;
 $9 = ($8|0)!=(0|0);
 if (!($9)) {
  STACKTOP = sp;return;
 }
 $10 = $3;
 $11 = $10;
 $12 = HEAP32[$11>>2]|0;
 $13 = (($10) + 4)|0;
 $14 = $13;
 $15 = HEAP32[$14>>2]|0;
 $16 = HEAP32[$2>>2]|0;
 $17 = (($16) + 16|0);
 $18 = $17;
 $19 = $18;
 HEAP32[$19>>2] = $12;
 $20 = (($18) + 4)|0;
 $21 = $20;
 HEAP32[$21>>2] = $15;
 STACKTOP = sp;return;
}
function _releaseMemArray($p,$N) {
 $p = $p|0;
 $N = $N|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $db = 0, $malloc_failed = 0, $or$cond = 0, $pEnd = 0;
 var label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 12|0;
 $1 = sp + 8|0;
 $pEnd = sp + 4|0;
 $db = sp;
 $malloc_failed = sp + 16|0;
 HEAP32[$0>>2] = $p;
 HEAP32[$1>>2] = $N;
 $2 = HEAP32[$0>>2]|0;
 $3 = ($2|0)!=(0|0);
 $4 = HEAP32[$1>>2]|0;
 $5 = ($4|0)!=(0);
 $or$cond = $3 & $5;
 if (!($or$cond)) {
  STACKTOP = sp;return;
 }
 $6 = HEAP32[$1>>2]|0;
 $7 = HEAP32[$0>>2]|0;
 $8 = (($7) + (($6*40)|0)|0);
 HEAP32[$pEnd>>2] = $8;
 $9 = HEAP32[$0>>2]|0;
 $10 = (($9) + 32|0);
 $11 = HEAP32[$10>>2]|0;
 HEAP32[$db>>2] = $11;
 $12 = HEAP32[$db>>2]|0;
 $13 = (($12) + 64|0);
 $14 = HEAP8[$13>>0]|0;
 HEAP8[$malloc_failed>>0] = $14;
 $15 = HEAP32[$db>>2]|0;
 $16 = (($15) + 520|0);
 $17 = HEAP32[$16>>2]|0;
 $18 = ($17|0)!=(0|0);
 if ($18) {
  while(1) {
   $19 = HEAP32[$0>>2]|0;
   $20 = (($19) + 24|0);
   $21 = HEAP32[$20>>2]|0;
   $22 = ($21|0)!=(0);
   if ($22) {
    $23 = HEAP32[$db>>2]|0;
    $24 = HEAP32[$0>>2]|0;
    $25 = (($24) + 20|0);
    $26 = HEAP32[$25>>2]|0;
    _sqlite3DbFree($23,$26);
   }
   $27 = HEAP32[$0>>2]|0;
   $28 = (($27) + 40|0);
   HEAP32[$0>>2] = $28;
   $29 = HEAP32[$pEnd>>2]|0;
   $30 = ($28>>>0)<($29>>>0);
   if (!($30)) {
    break;
   }
  }
  STACKTOP = sp;return;
 }
 while(1) {
  $31 = HEAP32[$0>>2]|0;
  $32 = (($31) + 8|0);
  $33 = HEAP16[$32>>1]|0;
  $34 = $33&65535;
  $35 = $34 & 9312;
  $36 = ($35|0)!=(0);
  $37 = HEAP32[$0>>2]|0;
  if ($36) {
   _sqlite3VdbeMemRelease($37);
  } else {
   $38 = (($37) + 24|0);
   $39 = HEAP32[$38>>2]|0;
   $40 = ($39|0)!=(0);
   if ($40) {
    $41 = HEAP32[$db>>2]|0;
    $42 = HEAP32[$0>>2]|0;
    $43 = (($42) + 20|0);
    $44 = HEAP32[$43>>2]|0;
    _sqlite3DbFree($41,$44);
    $45 = HEAP32[$0>>2]|0;
    $46 = (($45) + 24|0);
    HEAP32[$46>>2] = 0;
   }
  }
  $47 = HEAP32[$0>>2]|0;
  $48 = (($47) + 8|0);
  HEAP16[$48>>1] = 128;
  $49 = HEAP32[$0>>2]|0;
  $50 = (($49) + 40|0);
  HEAP32[$0>>2] = $50;
  $51 = HEAP32[$pEnd>>2]|0;
  $52 = ($50>>>0)<($51>>>0);
  if (!($52)) {
   break;
  }
 }
 $53 = HEAP8[$malloc_failed>>0]|0;
 $54 = HEAP32[$db>>2]|0;
 $55 = (($54) + 64|0);
 HEAP8[$55>>0] = $53;
 STACKTOP = sp;return;
}
function _sqlite3ClearStatTables($pParse,$iDb,$zType,$zName) {
 $pParse = $pParse|0;
 $iDb = $iDb|0;
 $zType = $zType|0;
 $zName = $zName|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $3 = 0, $4 = 0;
 var $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $i = 0, $vararg_buffer1 = 0, $vararg_ptr4 = 0, $vararg_ptr5 = 0, $vararg_ptr6 = 0, $zDbName = 0, $zTab = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 64|0;
 $vararg_buffer1 = sp;
 $0 = sp + 36|0;
 $1 = sp + 32|0;
 $2 = sp + 28|0;
 $3 = sp + 24|0;
 $i = sp + 20|0;
 $zDbName = sp + 16|0;
 $zTab = sp + 40|0;
 HEAP32[$0>>2] = $pParse;
 HEAP32[$1>>2] = $iDb;
 HEAP32[$2>>2] = $zType;
 HEAP32[$3>>2] = $zName;
 $4 = HEAP32[$1>>2]|0;
 $5 = HEAP32[$0>>2]|0;
 $6 = HEAP32[$5>>2]|0;
 $7 = (($6) + 16|0);
 $8 = HEAP32[$7>>2]|0;
 $9 = (($8) + ($4<<4)|0);
 $10 = HEAP32[$9>>2]|0;
 HEAP32[$zDbName>>2] = $10;
 HEAP32[$i>>2] = 1;
 while(1) {
  $11 = HEAP32[$i>>2]|0;
  $12 = ($11|0)<=(4);
  if (!($12)) {
   break;
  }
  $13 = HEAP32[$i>>2]|0;
  HEAP32[$vararg_buffer1>>2] = $13;
  (_sqlite3_snprintf(24,$zTab,23336,$vararg_buffer1)|0);
  $14 = HEAP32[$0>>2]|0;
  $15 = HEAP32[$14>>2]|0;
  $16 = HEAP32[$zDbName>>2]|0;
  $17 = (_sqlite3FindTable($15,$zTab,$16)|0);
  $18 = ($17|0)!=(0|0);
  if ($18) {
   $19 = HEAP32[$0>>2]|0;
   $20 = HEAP32[$zDbName>>2]|0;
   $21 = HEAP32[$2>>2]|0;
   $22 = HEAP32[$3>>2]|0;
   HEAP32[$vararg_buffer1>>2] = $20;
   $vararg_ptr4 = (($vararg_buffer1) + 4|0);
   HEAP32[$vararg_ptr4>>2] = $zTab;
   $vararg_ptr5 = (($vararg_buffer1) + 8|0);
   HEAP32[$vararg_ptr5>>2] = $21;
   $vararg_ptr6 = (($vararg_buffer1) + 12|0);
   HEAP32[$vararg_ptr6>>2] = $22;
   _sqlite3NestedParse($19,18480,$vararg_buffer1);
  }
  $23 = HEAP32[$i>>2]|0;
  $24 = (($23) + 1)|0;
  HEAP32[$i>>2] = $24;
 }
 STACKTOP = sp;return;
}
function _destroyRootPage($pParse,$iTable,$iDb) {
 $pParse = $pParse|0;
 $iTable = $iTable|0;
 $iDb = $iDb|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $r1 = 0, $v = 0, $vararg_buffer = 0, $vararg_ptr1 = 0, $vararg_ptr2 = 0, $vararg_ptr3 = 0, $vararg_ptr4 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $vararg_buffer = sp;
 $0 = sp + 36|0;
 $1 = sp + 32|0;
 $2 = sp + 28|0;
 $v = sp + 24|0;
 $r1 = sp + 20|0;
 HEAP32[$0>>2] = $pParse;
 HEAP32[$1>>2] = $iTable;
 HEAP32[$2>>2] = $iDb;
 $3 = HEAP32[$0>>2]|0;
 $4 = (_sqlite3GetVdbe($3)|0);
 HEAP32[$v>>2] = $4;
 $5 = HEAP32[$0>>2]|0;
 $6 = (_sqlite3GetTempReg($5)|0);
 HEAP32[$r1>>2] = $6;
 $7 = HEAP32[$v>>2]|0;
 $8 = HEAP32[$1>>2]|0;
 $9 = HEAP32[$r1>>2]|0;
 $10 = HEAP32[$2>>2]|0;
 (_sqlite3VdbeAddOp3($7,117,$8,$9,$10)|0);
 $11 = HEAP32[$0>>2]|0;
 _sqlite3MayAbort($11);
 $12 = HEAP32[$0>>2]|0;
 $13 = HEAP32[$2>>2]|0;
 $14 = HEAP32[$0>>2]|0;
 $15 = HEAP32[$14>>2]|0;
 $16 = (($15) + 16|0);
 $17 = HEAP32[$16>>2]|0;
 $18 = (($17) + ($13<<4)|0);
 $19 = HEAP32[$18>>2]|0;
 $20 = HEAP32[$2>>2]|0;
 $21 = ($20|0)==(1);
 $22 = $21 ? 14904 : 14928;
 $23 = HEAP32[$1>>2]|0;
 $24 = HEAP32[$r1>>2]|0;
 $25 = HEAP32[$r1>>2]|0;
 HEAP32[$vararg_buffer>>2] = $19;
 $vararg_ptr1 = (($vararg_buffer) + 4|0);
 HEAP32[$vararg_ptr1>>2] = $22;
 $vararg_ptr2 = (($vararg_buffer) + 8|0);
 HEAP32[$vararg_ptr2>>2] = $23;
 $vararg_ptr3 = (($vararg_buffer) + 12|0);
 HEAP32[$vararg_ptr3>>2] = $24;
 $vararg_ptr4 = (($vararg_buffer) + 16|0);
 HEAP32[$vararg_ptr4>>2] = $25;
 _sqlite3NestedParse($12,23280,$vararg_buffer);
 $26 = HEAP32[$0>>2]|0;
 $27 = HEAP32[$r1>>2]|0;
 _sqlite3ReleaseTempReg($26,$27);
 STACKTOP = sp;return;
}
function _sqlite3ArrayAllocate($db,$pArray,$szEntry,$pnEntry,$pIdx) {
 $db = $db|0;
 $pArray = $pArray|0;
 $szEntry = $szEntry|0;
 $pnEntry = $pnEntry|0;
 $pIdx = $pIdx|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $n = 0, $pNew = 0, $sz = 0, $z = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $0 = sp + 36|0;
 $1 = sp + 32|0;
 $2 = sp + 28|0;
 $3 = sp + 24|0;
 $4 = sp + 20|0;
 $5 = sp + 16|0;
 $z = sp + 12|0;
 $n = sp + 8|0;
 $sz = sp + 4|0;
 $pNew = sp;
 HEAP32[$1>>2] = $db;
 HEAP32[$2>>2] = $pArray;
 HEAP32[$3>>2] = $szEntry;
 HEAP32[$4>>2] = $pnEntry;
 HEAP32[$5>>2] = $pIdx;
 $6 = HEAP32[$4>>2]|0;
 $7 = HEAP32[$6>>2]|0;
 HEAP32[$n>>2] = $7;
 $8 = HEAP32[$n>>2]|0;
 $9 = HEAP32[$n>>2]|0;
 $10 = (($9) - 1)|0;
 $11 = $8 & $10;
 $12 = ($11|0)==(0);
 do {
  if ($12) {
   $13 = HEAP32[$n>>2]|0;
   $14 = ($13|0)==(0);
   if ($14) {
    $17 = 1;
   } else {
    $15 = HEAP32[$n>>2]|0;
    $16 = $15<<1;
    $17 = $16;
   }
   HEAP32[$sz>>2] = $17;
   $18 = HEAP32[$1>>2]|0;
   $19 = HEAP32[$2>>2]|0;
   $20 = HEAP32[$sz>>2]|0;
   $21 = HEAP32[$3>>2]|0;
   $22 = Math_imul($20, $21)|0;
   $23 = ($22|0)<(0);
   $24 = $23 << 31 >> 31;
   $25 = (_sqlite3DbRealloc($18,$19,$22,$24)|0);
   HEAP32[$pNew>>2] = $25;
   $26 = HEAP32[$pNew>>2]|0;
   $27 = ($26|0)==(0|0);
   if (!($27)) {
    $30 = HEAP32[$pNew>>2]|0;
    HEAP32[$2>>2] = $30;
    break;
   }
   $28 = HEAP32[$5>>2]|0;
   HEAP32[$28>>2] = -1;
   $29 = HEAP32[$2>>2]|0;
   HEAP32[$0>>2] = $29;
   $44 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($44|0);
  }
 } while(0);
 $31 = HEAP32[$2>>2]|0;
 HEAP32[$z>>2] = $31;
 $32 = HEAP32[$n>>2]|0;
 $33 = HEAP32[$3>>2]|0;
 $34 = Math_imul($32, $33)|0;
 $35 = HEAP32[$z>>2]|0;
 $36 = (($35) + ($34)|0);
 $37 = HEAP32[$3>>2]|0;
 _memset(($36|0),0,($37|0))|0;
 $38 = HEAP32[$n>>2]|0;
 $39 = HEAP32[$5>>2]|0;
 HEAP32[$39>>2] = $38;
 $40 = HEAP32[$4>>2]|0;
 $41 = HEAP32[$40>>2]|0;
 $42 = (($41) + 1)|0;
 HEAP32[$40>>2] = $42;
 $43 = HEAP32[$2>>2]|0;
 HEAP32[$0>>2] = $43;
 $44 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($44|0);
}
function _sqlite3TriggersExist($pParse,$pTab,$op,$pChanges,$pMask) {
 $pParse = $pParse|0;
 $pTab = $pTab|0;
 $op = $op|0;
 $pChanges = $pChanges|0;
 $pMask = $pMask|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $mask = 0, $p = 0, $pList = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 28|0;
 $1 = sp + 24|0;
 $2 = sp + 20|0;
 $3 = sp + 16|0;
 $4 = sp + 12|0;
 $mask = sp + 8|0;
 $pList = sp + 4|0;
 $p = sp;
 HEAP32[$0>>2] = $pParse;
 HEAP32[$1>>2] = $pTab;
 HEAP32[$2>>2] = $op;
 HEAP32[$3>>2] = $pChanges;
 HEAP32[$4>>2] = $pMask;
 HEAP32[$mask>>2] = 0;
 HEAP32[$pList>>2] = 0;
 $5 = HEAP32[$0>>2]|0;
 $6 = HEAP32[$5>>2]|0;
 $7 = (($6) + 24|0);
 $8 = HEAP32[$7>>2]|0;
 $9 = $8 & 8388608;
 $10 = ($9|0)!=(0);
 if ($10) {
  $11 = HEAP32[$0>>2]|0;
  $12 = HEAP32[$1>>2]|0;
  $13 = (_sqlite3TriggerList($11,$12)|0);
  HEAP32[$pList>>2] = $13;
 }
 $14 = HEAP32[$pList>>2]|0;
 HEAP32[$p>>2] = $14;
 while(1) {
  $15 = HEAP32[$p>>2]|0;
  $16 = ($15|0)!=(0|0);
  if (!($16)) {
   break;
  }
  $17 = HEAP32[$p>>2]|0;
  $18 = (($17) + 8|0);
  $19 = HEAP8[$18>>0]|0;
  $20 = $19&255;
  $21 = HEAP32[$2>>2]|0;
  $22 = ($20|0)==($21|0);
  if ($22) {
   $23 = HEAP32[$p>>2]|0;
   $24 = (($23) + 16|0);
   $25 = HEAP32[$24>>2]|0;
   $26 = HEAP32[$3>>2]|0;
   $27 = (_checkColumnOverlap($25,$26)|0);
   $28 = ($27|0)!=(0);
   if ($28) {
    $29 = HEAP32[$p>>2]|0;
    $30 = (($29) + 9|0);
    $31 = HEAP8[$30>>0]|0;
    $32 = $31&255;
    $33 = HEAP32[$mask>>2]|0;
    $34 = $33 | $32;
    HEAP32[$mask>>2] = $34;
   }
  }
  $35 = HEAP32[$p>>2]|0;
  $36 = (($35) + 32|0);
  $37 = HEAP32[$36>>2]|0;
  HEAP32[$p>>2] = $37;
 }
 $38 = HEAP32[$4>>2]|0;
 $39 = ($38|0)!=(0|0);
 if (!($39)) {
  $42 = HEAP32[$mask>>2]|0;
  $43 = ($42|0)!=(0);
  $44 = HEAP32[$pList>>2]|0;
  $45 = $43 ? $44 : 0;
  STACKTOP = sp;return ($45|0);
 }
 $40 = HEAP32[$mask>>2]|0;
 $41 = HEAP32[$4>>2]|0;
 HEAP32[$41>>2] = $40;
 $42 = HEAP32[$mask>>2]|0;
 $43 = ($42|0)!=(0);
 $44 = HEAP32[$pList>>2]|0;
 $45 = $43 ? $44 : 0;
 STACKTOP = sp;return ($45|0);
}
function _sqlite3IsReadOnly($pParse,$pTab,$viewOk) {
 $pParse = $pParse|0;
 $pTab = $pTab|0;
 $viewOk = $viewOk|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $vararg_buffer1 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $vararg_buffer1 = sp;
 $0 = sp + 16|0;
 $1 = sp + 12|0;
 $2 = sp + 8|0;
 $3 = sp + 4|0;
 HEAP32[$1>>2] = $pParse;
 HEAP32[$2>>2] = $pTab;
 HEAP32[$3>>2] = $viewOk;
 $4 = HEAP32[$2>>2]|0;
 $5 = (($4) + 44|0);
 $6 = HEAP8[$5>>0]|0;
 $7 = $6&255;
 $8 = $7 & 16;
 $9 = ($8|0)!=(0);
 if ($9) {
  $10 = HEAP32[$1>>2]|0;
  $11 = HEAP32[$10>>2]|0;
  $12 = HEAP32[$2>>2]|0;
  $13 = (_sqlite3GetVTable($11,$12)|0);
  $14 = (($13) + 4|0);
  $15 = HEAP32[$14>>2]|0;
  $16 = HEAP32[$15>>2]|0;
  $17 = (($16) + 52|0);
  $18 = HEAP32[$17>>2]|0;
  $19 = ($18|0)==(0|0);
  if (!($19)) {
   label = 3;
  }
 } else {
  label = 3;
 }
 do {
  if ((label|0) == 3) {
   $20 = HEAP32[$2>>2]|0;
   $21 = (($20) + 44|0);
   $22 = HEAP8[$21>>0]|0;
   $23 = $22&255;
   $24 = $23 & 1;
   $25 = ($24|0)!=(0);
   if ($25) {
    $26 = HEAP32[$1>>2]|0;
    $27 = HEAP32[$26>>2]|0;
    $28 = (($27) + 24|0);
    $29 = HEAP32[$28>>2]|0;
    $30 = $29 & 2048;
    $31 = ($30|0)==(0);
    if ($31) {
     $32 = HEAP32[$1>>2]|0;
     $33 = (($32) + 18|0);
     $34 = HEAP8[$33>>0]|0;
     $35 = $34&255;
     $36 = ($35|0)==(0);
     if ($36) {
      break;
     }
    }
   }
   $40 = HEAP32[$3>>2]|0;
   $41 = ($40|0)!=(0);
   if (!($41)) {
    $42 = HEAP32[$2>>2]|0;
    $43 = (($42) + 12|0);
    $44 = HEAP32[$43>>2]|0;
    $45 = ($44|0)!=(0|0);
    if ($45) {
     $46 = HEAP32[$1>>2]|0;
     $47 = HEAP32[$2>>2]|0;
     $48 = HEAP32[$47>>2]|0;
     HEAP32[$vararg_buffer1>>2] = $48;
     _sqlite3ErrorMsg($46,24480,$vararg_buffer1);
     HEAP32[$0>>2] = 1;
     $49 = HEAP32[$0>>2]|0;
     STACKTOP = sp;return ($49|0);
    }
   }
   HEAP32[$0>>2] = 0;
   $49 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($49|0);
  }
 } while(0);
 $37 = HEAP32[$1>>2]|0;
 $38 = HEAP32[$2>>2]|0;
 $39 = HEAP32[$38>>2]|0;
 HEAP32[$vararg_buffer1>>2] = $39;
 _sqlite3ErrorMsg($37,24448,$vararg_buffer1);
 HEAP32[$0>>2] = 1;
 $49 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($49|0);
}
function _sqlite3VdbeCountChanges($v) {
 $v = $v|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 HEAP32[$0>>2] = $v;
 $1 = HEAP32[$0>>2]|0;
 $2 = (($1) + 88|0);
 $3 = HEAP16[$2>>1]|0;
 $4 = $3 & -17;
 $5 = $4 | 16;
 HEAP16[$2>>1] = $5;
 STACKTOP = sp;return;
}
function _xferOptimization($pParse,$pDest,$pSelect,$onError,$iDbDest) {
 $pParse = $pParse|0;
 $pDest = $pDest|0;
 $pSelect = $pSelect|0;
 $onError = $onError|0;
 $iDbDest = $iDbDest|0;
 var $$old = 0, $$old1 = 0, $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0;
 var $114 = 0, $115 = 0, $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0;
 var $132 = 0, $133 = 0, $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0;
 var $150 = 0, $151 = 0, $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0;
 var $169 = 0, $17 = 0, $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0;
 var $187 = 0, $188 = 0, $189 = 0, $19 = 0, $190 = 0, $191 = 0, $192 = 0, $193 = 0, $194 = 0, $195 = 0, $196 = 0, $197 = 0, $198 = 0, $199 = 0, $2 = 0, $20 = 0, $200 = 0, $201 = 0, $202 = 0, $203 = 0;
 var $204 = 0, $205 = 0, $206 = 0, $207 = 0, $208 = 0, $209 = 0, $21 = 0, $210 = 0, $211 = 0, $212 = 0, $213 = 0, $214 = 0, $215 = 0, $216 = 0, $217 = 0, $218 = 0, $219 = 0, $22 = 0, $220 = 0, $221 = 0;
 var $222 = 0, $223 = 0, $224 = 0, $225 = 0, $226 = 0, $227 = 0, $228 = 0, $229 = 0, $23 = 0, $230 = 0, $231 = 0, $232 = 0, $233 = 0, $234 = 0, $235 = 0, $236 = 0, $237 = 0, $238 = 0, $239 = 0, $24 = 0;
 var $240 = 0, $241 = 0, $242 = 0, $243 = 0, $244 = 0, $245 = 0, $246 = 0, $247 = 0, $248 = 0, $249 = 0, $25 = 0, $250 = 0, $251 = 0, $252 = 0, $253 = 0, $254 = 0, $255 = 0, $256 = 0, $257 = 0, $258 = 0;
 var $259 = 0, $26 = 0, $260 = 0, $261 = 0, $262 = 0, $263 = 0, $264 = 0, $265 = 0, $266 = 0, $267 = 0, $268 = 0, $269 = 0, $27 = 0, $270 = 0, $271 = 0, $272 = 0, $273 = 0, $274 = 0, $275 = 0, $276 = 0;
 var $277 = 0, $278 = 0, $279 = 0, $28 = 0, $280 = 0, $281 = 0, $282 = 0, $283 = 0, $284 = 0, $285 = 0, $286 = 0, $287 = 0, $288 = 0, $289 = 0, $29 = 0, $290 = 0, $291 = 0, $292 = 0, $293 = 0, $294 = 0;
 var $295 = 0, $296 = 0, $297 = 0, $298 = 0, $299 = 0, $3 = 0, $30 = 0, $300 = 0, $301 = 0, $302 = 0, $303 = 0, $304 = 0, $305 = 0, $306 = 0, $307 = 0, $308 = 0, $309 = 0, $31 = 0, $310 = 0, $311 = 0;
 var $312 = 0, $313 = 0, $314 = 0, $315 = 0, $316 = 0, $317 = 0, $318 = 0, $319 = 0, $32 = 0, $320 = 0, $321 = 0, $322 = 0, $323 = 0, $324 = 0, $325 = 0, $326 = 0, $327 = 0, $328 = 0, $329 = 0, $33 = 0;
 var $330 = 0, $331 = 0, $332 = 0, $333 = 0, $334 = 0, $335 = 0, $336 = 0, $337 = 0, $338 = 0, $339 = 0, $34 = 0, $340 = 0, $341 = 0, $342 = 0, $343 = 0, $344 = 0, $345 = 0, $346 = 0, $347 = 0, $348 = 0;
 var $349 = 0, $35 = 0, $350 = 0, $351 = 0, $352 = 0, $353 = 0, $354 = 0, $355 = 0, $356 = 0, $357 = 0, $358 = 0, $359 = 0, $36 = 0, $360 = 0, $361 = 0, $362 = 0, $363 = 0, $364 = 0, $365 = 0, $366 = 0;
 var $367 = 0, $368 = 0, $369 = 0, $37 = 0, $370 = 0, $371 = 0, $372 = 0, $373 = 0, $374 = 0, $375 = 0, $376 = 0, $377 = 0, $378 = 0, $379 = 0, $38 = 0, $380 = 0, $381 = 0, $382 = 0, $383 = 0, $384 = 0;
 var $385 = 0, $386 = 0, $387 = 0, $388 = 0, $389 = 0, $39 = 0, $390 = 0, $391 = 0, $392 = 0, $393 = 0, $394 = 0, $395 = 0, $396 = 0, $397 = 0, $398 = 0, $399 = 0, $4 = 0, $40 = 0, $400 = 0, $401 = 0;
 var $402 = 0, $403 = 0, $404 = 0, $405 = 0, $406 = 0, $407 = 0, $408 = 0, $409 = 0, $41 = 0, $410 = 0, $411 = 0, $412 = 0, $413 = 0, $414 = 0, $415 = 0, $416 = 0, $417 = 0, $418 = 0, $419 = 0, $42 = 0;
 var $420 = 0, $421 = 0, $422 = 0, $423 = 0, $424 = 0, $425 = 0, $426 = 0, $427 = 0, $428 = 0, $429 = 0, $43 = 0, $430 = 0, $431 = 0, $432 = 0, $433 = 0, $434 = 0, $435 = 0, $436 = 0, $437 = 0, $438 = 0;
 var $439 = 0, $44 = 0, $440 = 0, $441 = 0, $442 = 0, $443 = 0, $444 = 0, $445 = 0, $446 = 0, $447 = 0, $448 = 0, $449 = 0, $45 = 0, $450 = 0, $451 = 0, $452 = 0, $453 = 0, $454 = 0, $455 = 0, $456 = 0;
 var $457 = 0, $458 = 0, $459 = 0, $46 = 0, $460 = 0, $461 = 0, $462 = 0, $463 = 0, $464 = 0, $465 = 0, $466 = 0, $467 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0;
 var $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0;
 var $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0;
 var $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $addr1 = 0, $addr2 = 0, $destHasUniqueIdx = 0, $emptyDestTest = 0, $emptySrcTest = 0, $i = 0, $iDbSrc = 0, $iDest = 0, $iSrc = 0, $or$cond = 0;
 var $or$cond4 = 0, $pDestCol = 0, $pDestIdx = 0, $pEList = 0, $pItem = 0, $pSrc = 0, $pSrcCol = 0, $pSrcIdx = 0, $regAutoinc = 0, $regData = 0, $regRowid = 0, $v = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 112|0;
 $0 = sp + 100|0;
 $1 = sp + 96|0;
 $2 = sp + 92|0;
 $3 = sp + 88|0;
 $4 = sp + 84|0;
 $5 = sp + 80|0;
 $pEList = sp + 76|0;
 $pSrc = sp + 72|0;
 $pSrcIdx = sp + 68|0;
 $pDestIdx = sp + 64|0;
 $pItem = sp + 60|0;
 $i = sp + 56|0;
 $iDbSrc = sp + 52|0;
 $iSrc = sp + 48|0;
 $iDest = sp + 44|0;
 $addr1 = sp + 40|0;
 $addr2 = sp + 36|0;
 $emptyDestTest = sp + 32|0;
 $emptySrcTest = sp + 28|0;
 $v = sp + 24|0;
 $regAutoinc = sp + 20|0;
 $destHasUniqueIdx = sp + 16|0;
 $regData = sp + 12|0;
 $regRowid = sp + 8|0;
 $pDestCol = sp + 4|0;
 $pSrcCol = sp;
 HEAP32[$1>>2] = $pParse;
 HEAP32[$2>>2] = $pDest;
 HEAP32[$3>>2] = $pSelect;
 HEAP32[$4>>2] = $onError;
 HEAP32[$5>>2] = $iDbDest;
 HEAP32[$emptyDestTest>>2] = 0;
 HEAP32[$emptySrcTest>>2] = 0;
 HEAP32[$destHasUniqueIdx>>2] = 0;
 $6 = HEAP32[$3>>2]|0;
 $7 = ($6|0)==(0|0);
 if ($7) {
  HEAP32[$0>>2] = 0;
  $467 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($467|0);
 }
 $8 = HEAP32[$1>>2]|0;
 $9 = (($8) + 532|0);
 $10 = HEAP32[$9>>2]|0;
 $11 = ($10|0)!=(0|0);
 if (!($11)) {
  $12 = HEAP32[$3>>2]|0;
  $13 = (($12) + 68|0);
  $14 = HEAP32[$13>>2]|0;
  $15 = ($14|0)!=(0|0);
  if (!($15)) {
   $16 = HEAP32[$1>>2]|0;
   $17 = HEAP32[$2>>2]|0;
   $18 = (_sqlite3TriggerList($16,$17)|0);
   $19 = ($18|0)!=(0|0);
   if ($19) {
    HEAP32[$0>>2] = 0;
    $467 = HEAP32[$0>>2]|0;
    STACKTOP = sp;return ($467|0);
   }
   $20 = HEAP32[$2>>2]|0;
   $21 = (($20) + 44|0);
   $22 = HEAP8[$21>>0]|0;
   $23 = $22&255;
   $24 = $23 & 16;
   $25 = ($24|0)!=(0);
   if ($25) {
    HEAP32[$0>>2] = 0;
    $467 = HEAP32[$0>>2]|0;
    STACKTOP = sp;return ($467|0);
   }
   $26 = HEAP32[$4>>2]|0;
   $27 = ($26|0)==(10);
   if ($27) {
    $28 = HEAP32[$2>>2]|0;
    $29 = (($28) + 36|0);
    $30 = HEAP16[$29>>1]|0;
    $31 = $30 << 16 >> 16;
    $32 = ($31|0)>=(0);
    if ($32) {
     $33 = HEAP32[$2>>2]|0;
     $34 = (($33) + 45|0);
     $35 = HEAP8[$34>>0]|0;
     $36 = $35&255;
     HEAP32[$4>>2] = $36;
    }
    $37 = HEAP32[$4>>2]|0;
    $38 = ($37|0)==(10);
    if ($38) {
     HEAP32[$4>>2] = 2;
    }
   }
   $39 = HEAP32[$3>>2]|0;
   $40 = (($39) + 32|0);
   $41 = HEAP32[$40>>2]|0;
   $42 = HEAP32[$41>>2]|0;
   $43 = ($42|0)!=(1);
   if ($43) {
    HEAP32[$0>>2] = 0;
    $467 = HEAP32[$0>>2]|0;
    STACKTOP = sp;return ($467|0);
   }
   $44 = HEAP32[$3>>2]|0;
   $45 = (($44) + 32|0);
   $46 = HEAP32[$45>>2]|0;
   $47 = (($46) + 8|0);
   $48 = (($47) + 20|0);
   $49 = HEAP32[$48>>2]|0;
   $50 = ($49|0)!=(0|0);
   if ($50) {
    HEAP32[$0>>2] = 0;
    $467 = HEAP32[$0>>2]|0;
    STACKTOP = sp;return ($467|0);
   }
   $51 = HEAP32[$3>>2]|0;
   $52 = (($51) + 36|0);
   $53 = HEAP32[$52>>2]|0;
   $54 = ($53|0)!=(0|0);
   if ($54) {
    HEAP32[$0>>2] = 0;
    $467 = HEAP32[$0>>2]|0;
    STACKTOP = sp;return ($467|0);
   }
   $55 = HEAP32[$3>>2]|0;
   $56 = (($55) + 48|0);
   $57 = HEAP32[$56>>2]|0;
   $58 = ($57|0)!=(0|0);
   if ($58) {
    HEAP32[$0>>2] = 0;
    $467 = HEAP32[$0>>2]|0;
    STACKTOP = sp;return ($467|0);
   }
   $59 = HEAP32[$3>>2]|0;
   $60 = (($59) + 40|0);
   $61 = HEAP32[$60>>2]|0;
   $62 = ($61|0)!=(0|0);
   if ($62) {
    HEAP32[$0>>2] = 0;
    $467 = HEAP32[$0>>2]|0;
    STACKTOP = sp;return ($467|0);
   }
   $63 = HEAP32[$3>>2]|0;
   $64 = (($63) + 60|0);
   $65 = HEAP32[$64>>2]|0;
   $66 = ($65|0)!=(0|0);
   if ($66) {
    HEAP32[$0>>2] = 0;
    $467 = HEAP32[$0>>2]|0;
    STACKTOP = sp;return ($467|0);
   }
   $67 = HEAP32[$3>>2]|0;
   $68 = (($67) + 52|0);
   $69 = HEAP32[$68>>2]|0;
   $70 = ($69|0)!=(0|0);
   if ($70) {
    HEAP32[$0>>2] = 0;
    $467 = HEAP32[$0>>2]|0;
    STACKTOP = sp;return ($467|0);
   }
   $71 = HEAP32[$3>>2]|0;
   $72 = (($71) + 6|0);
   $73 = HEAP16[$72>>1]|0;
   $74 = $73&65535;
   $75 = $74 & 1;
   $76 = ($75|0)!=(0);
   if ($76) {
    HEAP32[$0>>2] = 0;
    $467 = HEAP32[$0>>2]|0;
    STACKTOP = sp;return ($467|0);
   }
   $77 = HEAP32[$3>>2]|0;
   $78 = HEAP32[$77>>2]|0;
   HEAP32[$pEList>>2] = $78;
   $79 = HEAP32[$pEList>>2]|0;
   $80 = HEAP32[$79>>2]|0;
   $81 = ($80|0)!=(1);
   if ($81) {
    HEAP32[$0>>2] = 0;
    $467 = HEAP32[$0>>2]|0;
    STACKTOP = sp;return ($467|0);
   }
   $82 = HEAP32[$pEList>>2]|0;
   $83 = (($82) + 4|0);
   $84 = HEAP32[$83>>2]|0;
   $85 = HEAP32[$84>>2]|0;
   $86 = HEAP8[$85>>0]|0;
   $87 = $86&255;
   $88 = ($87|0)!=(116);
   if ($88) {
    HEAP32[$0>>2] = 0;
    $467 = HEAP32[$0>>2]|0;
    STACKTOP = sp;return ($467|0);
   }
   $89 = HEAP32[$3>>2]|0;
   $90 = (($89) + 32|0);
   $91 = HEAP32[$90>>2]|0;
   $92 = (($91) + 8|0);
   HEAP32[$pItem>>2] = $92;
   $93 = HEAP32[$1>>2]|0;
   $94 = HEAP32[$pItem>>2]|0;
   $95 = (_sqlite3LocateTableItem($93,0,$94)|0);
   HEAP32[$pSrc>>2] = $95;
   $96 = HEAP32[$pSrc>>2]|0;
   $97 = ($96|0)==(0|0);
   if ($97) {
    HEAP32[$0>>2] = 0;
    $467 = HEAP32[$0>>2]|0;
    STACKTOP = sp;return ($467|0);
   }
   $98 = HEAP32[$pSrc>>2]|0;
   $99 = HEAP32[$2>>2]|0;
   $100 = ($98|0)==($99|0);
   if ($100) {
    HEAP32[$0>>2] = 0;
    $467 = HEAP32[$0>>2]|0;
    STACKTOP = sp;return ($467|0);
   }
   $101 = HEAP32[$2>>2]|0;
   $102 = (($101) + 44|0);
   $103 = HEAP8[$102>>0]|0;
   $104 = $103&255;
   $105 = $104 & 32;
   $106 = ($105|0)==(0);
   $107 = $106&1;
   $108 = HEAP32[$pSrc>>2]|0;
   $109 = (($108) + 44|0);
   $110 = HEAP8[$109>>0]|0;
   $111 = $110&255;
   $112 = $111 & 32;
   $113 = ($112|0)==(0);
   $114 = $113&1;
   $115 = ($107|0)!=($114|0);
   if ($115) {
    HEAP32[$0>>2] = 0;
    $467 = HEAP32[$0>>2]|0;
    STACKTOP = sp;return ($467|0);
   }
   $116 = HEAP32[$pSrc>>2]|0;
   $117 = (($116) + 44|0);
   $118 = HEAP8[$117>>0]|0;
   $119 = $118&255;
   $120 = $119 & 16;
   $121 = ($120|0)!=(0);
   if ($121) {
    HEAP32[$0>>2] = 0;
    $467 = HEAP32[$0>>2]|0;
    STACKTOP = sp;return ($467|0);
   }
   $122 = HEAP32[$pSrc>>2]|0;
   $123 = (($122) + 12|0);
   $124 = HEAP32[$123>>2]|0;
   $125 = ($124|0)!=(0|0);
   if ($125) {
    HEAP32[$0>>2] = 0;
    $467 = HEAP32[$0>>2]|0;
    STACKTOP = sp;return ($467|0);
   }
   $126 = HEAP32[$2>>2]|0;
   $127 = (($126) + 38|0);
   $128 = HEAP16[$127>>1]|0;
   $129 = $128 << 16 >> 16;
   $130 = HEAP32[$pSrc>>2]|0;
   $131 = (($130) + 38|0);
   $132 = HEAP16[$131>>1]|0;
   $133 = $132 << 16 >> 16;
   $134 = ($129|0)!=($133|0);
   if ($134) {
    HEAP32[$0>>2] = 0;
    $467 = HEAP32[$0>>2]|0;
    STACKTOP = sp;return ($467|0);
   }
   $135 = HEAP32[$2>>2]|0;
   $136 = (($135) + 36|0);
   $137 = HEAP16[$136>>1]|0;
   $138 = $137 << 16 >> 16;
   $139 = HEAP32[$pSrc>>2]|0;
   $140 = (($139) + 36|0);
   $141 = HEAP16[$140>>1]|0;
   $142 = $141 << 16 >> 16;
   $143 = ($138|0)!=($142|0);
   if ($143) {
    HEAP32[$0>>2] = 0;
    $467 = HEAP32[$0>>2]|0;
    STACKTOP = sp;return ($467|0);
   }
   HEAP32[$i>>2] = 0;
   L91: while(1) {
    $144 = HEAP32[$i>>2]|0;
    $145 = HEAP32[$2>>2]|0;
    $146 = (($145) + 38|0);
    $147 = HEAP16[$146>>1]|0;
    $148 = $147 << 16 >> 16;
    $149 = ($144|0)<($148|0);
    if (!($149)) {
     label = 64;
     break;
    }
    $150 = HEAP32[$i>>2]|0;
    $151 = HEAP32[$2>>2]|0;
    $152 = (($151) + 4|0);
    $153 = HEAP32[$152>>2]|0;
    $154 = (($153) + (($150*24)|0)|0);
    HEAP32[$pDestCol>>2] = $154;
    $155 = HEAP32[$i>>2]|0;
    $156 = HEAP32[$pSrc>>2]|0;
    $157 = (($156) + 4|0);
    $158 = HEAP32[$157>>2]|0;
    $159 = (($158) + (($155*24)|0)|0);
    HEAP32[$pSrcCol>>2] = $159;
    $160 = HEAP32[$pDestCol>>2]|0;
    $161 = (($160) + 21|0);
    $162 = HEAP8[$161>>0]|0;
    $163 = $162 << 24 >> 24;
    $164 = HEAP32[$pSrcCol>>2]|0;
    $165 = (($164) + 21|0);
    $166 = HEAP8[$165>>0]|0;
    $167 = $166 << 24 >> 24;
    $168 = ($163|0)!=($167|0);
    if ($168) {
     label = 52;
     break;
    }
    $169 = HEAP32[$pDestCol>>2]|0;
    $170 = (($169) + 16|0);
    $171 = HEAP32[$170>>2]|0;
    $172 = HEAP32[$pSrcCol>>2]|0;
    $173 = (($172) + 16|0);
    $174 = HEAP32[$173>>2]|0;
    $175 = (_xferCompatibleCollation($171,$174)|0);
    $176 = ($175|0)!=(0);
    if (!($176)) {
     label = 54;
     break;
    }
    $177 = HEAP32[$pDestCol>>2]|0;
    $178 = (($177) + 20|0);
    $179 = HEAP8[$178>>0]|0;
    $180 = $179&255;
    $181 = ($180|0)!=(0);
    if ($181) {
     $182 = HEAP32[$pSrcCol>>2]|0;
     $183 = (($182) + 20|0);
     $184 = HEAP8[$183>>0]|0;
     $185 = ($184<<24>>24)!=(0);
     if (!($185)) {
      label = 57;
      break;
     }
    }
    $186 = HEAP32[$i>>2]|0;
    $187 = ($186|0)>(0);
    do {
     if ($187) {
      $188 = HEAP32[$pDestCol>>2]|0;
      $189 = (($188) + 8|0);
      $190 = HEAP32[$189>>2]|0;
      $191 = ($190|0)==(0|0);
      $192 = $191&1;
      $193 = HEAP32[$pSrcCol>>2]|0;
      $194 = (($193) + 8|0);
      $195 = HEAP32[$194>>2]|0;
      $196 = ($195|0)==(0|0);
      $197 = $196&1;
      $198 = ($192|0)!=($197|0);
      if ($198) {
       label = 62;
       break L91;
      }
      $199 = HEAP32[$pDestCol>>2]|0;
      $200 = (($199) + 8|0);
      $201 = HEAP32[$200>>2]|0;
      $202 = ($201|0)!=(0|0);
      if (!($202)) {
       break;
      }
      $203 = HEAP32[$pDestCol>>2]|0;
      $204 = (($203) + 8|0);
      $205 = HEAP32[$204>>2]|0;
      $206 = HEAP32[$pSrcCol>>2]|0;
      $207 = (($206) + 8|0);
      $208 = HEAP32[$207>>2]|0;
      $209 = (_strcmp($205,$208)|0);
      $210 = ($209|0)!=(0);
      if ($210) {
       label = 62;
       break L91;
      }
     }
    } while(0);
    $211 = HEAP32[$i>>2]|0;
    $212 = (($211) + 1)|0;
    HEAP32[$i>>2] = $212;
   }
   if ((label|0) == 52) {
    HEAP32[$0>>2] = 0;
    $467 = HEAP32[$0>>2]|0;
    STACKTOP = sp;return ($467|0);
   }
   else if ((label|0) == 54) {
    HEAP32[$0>>2] = 0;
    $467 = HEAP32[$0>>2]|0;
    STACKTOP = sp;return ($467|0);
   }
   else if ((label|0) == 57) {
    HEAP32[$0>>2] = 0;
    $467 = HEAP32[$0>>2]|0;
    STACKTOP = sp;return ($467|0);
   }
   else if ((label|0) == 62) {
    HEAP32[$0>>2] = 0;
    $467 = HEAP32[$0>>2]|0;
    STACKTOP = sp;return ($467|0);
   }
   else if ((label|0) == 64) {
    $213 = HEAP32[$2>>2]|0;
    $214 = (($213) + 8|0);
    $215 = HEAP32[$214>>2]|0;
    HEAP32[$pDestIdx>>2] = $215;
    while(1) {
     $216 = HEAP32[$pDestIdx>>2]|0;
     $217 = ($216|0)!=(0|0);
     if (!($217)) {
      break;
     }
     $218 = HEAP32[$pDestIdx>>2]|0;
     $219 = (($218) + 54|0);
     $220 = HEAP8[$219>>0]|0;
     $221 = $220&255;
     $222 = ($221|0)!=(0);
     if ($222) {
      HEAP32[$destHasUniqueIdx>>2] = 1;
     }
     $223 = HEAP32[$pSrc>>2]|0;
     $224 = (($223) + 8|0);
     $225 = HEAP32[$224>>2]|0;
     HEAP32[$pSrcIdx>>2] = $225;
     while(1) {
      $226 = HEAP32[$pSrcIdx>>2]|0;
      $227 = ($226|0)!=(0|0);
      if (!($227)) {
       break;
      }
      $228 = HEAP32[$pDestIdx>>2]|0;
      $229 = HEAP32[$pSrcIdx>>2]|0;
      $230 = (_xferCompatibleIndex($228,$229)|0);
      $231 = ($230|0)!=(0);
      if ($231) {
       break;
      }
      $232 = HEAP32[$pSrcIdx>>2]|0;
      $233 = (($232) + 20|0);
      $234 = HEAP32[$233>>2]|0;
      HEAP32[$pSrcIdx>>2] = $234;
     }
     $235 = HEAP32[$pSrcIdx>>2]|0;
     $236 = ($235|0)==(0|0);
     if ($236) {
      label = 73;
      break;
     }
     $237 = HEAP32[$pDestIdx>>2]|0;
     $238 = (($237) + 20|0);
     $239 = HEAP32[$238>>2]|0;
     HEAP32[$pDestIdx>>2] = $239;
    }
    if ((label|0) == 73) {
     HEAP32[$0>>2] = 0;
     $467 = HEAP32[$0>>2]|0;
     STACKTOP = sp;return ($467|0);
    }
    $240 = HEAP32[$2>>2]|0;
    $241 = (($240) + 24|0);
    $242 = HEAP32[$241>>2]|0;
    $243 = ($242|0)!=(0|0);
    do {
     if ($243) {
      $244 = HEAP32[$pSrc>>2]|0;
      $245 = (($244) + 24|0);
      $246 = HEAP32[$245>>2]|0;
      $247 = HEAP32[$2>>2]|0;
      $248 = (($247) + 24|0);
      $249 = HEAP32[$248>>2]|0;
      $250 = (_sqlite3ExprListCompare($246,$249,-1)|0);
      $251 = ($250|0)!=(0);
      if (!($251)) {
       break;
      }
      HEAP32[$0>>2] = 0;
      $467 = HEAP32[$0>>2]|0;
      STACKTOP = sp;return ($467|0);
     }
    } while(0);
    $252 = HEAP32[$1>>2]|0;
    $253 = HEAP32[$252>>2]|0;
    $254 = (($253) + 24|0);
    $255 = HEAP32[$254>>2]|0;
    $256 = $255 & 524288;
    $257 = ($256|0)!=(0);
    do {
     if ($257) {
      $258 = HEAP32[$2>>2]|0;
      $259 = (($258) + 16|0);
      $260 = HEAP32[$259>>2]|0;
      $261 = ($260|0)!=(0|0);
      if (!($261)) {
       break;
      }
      HEAP32[$0>>2] = 0;
      $467 = HEAP32[$0>>2]|0;
      STACKTOP = sp;return ($467|0);
     }
    } while(0);
    $262 = HEAP32[$1>>2]|0;
    $263 = HEAP32[$262>>2]|0;
    $264 = (($263) + 24|0);
    $265 = HEAP32[$264>>2]|0;
    $266 = $265 & 128;
    $267 = ($266|0)!=(0);
    if ($267) {
     HEAP32[$0>>2] = 0;
     $467 = HEAP32[$0>>2]|0;
     STACKTOP = sp;return ($467|0);
    }
    $268 = HEAP32[$1>>2]|0;
    $269 = HEAP32[$268>>2]|0;
    $270 = HEAP32[$pSrc>>2]|0;
    $271 = (($270) + 68|0);
    $272 = HEAP32[$271>>2]|0;
    $273 = (_sqlite3SchemaToIndex($269,$272)|0);
    HEAP32[$iDbSrc>>2] = $273;
    $274 = HEAP32[$1>>2]|0;
    $275 = (_sqlite3GetVdbe($274)|0);
    HEAP32[$v>>2] = $275;
    $276 = HEAP32[$1>>2]|0;
    $277 = HEAP32[$iDbSrc>>2]|0;
    _sqlite3CodeVerifySchema($276,$277);
    $278 = HEAP32[$1>>2]|0;
    $279 = (($278) + 68|0);
    $280 = HEAP32[$279>>2]|0;
    $281 = (($280) + 1)|0;
    HEAP32[$279>>2] = $281;
    HEAP32[$iSrc>>2] = $280;
    $282 = HEAP32[$1>>2]|0;
    $283 = (($282) + 68|0);
    $284 = HEAP32[$283>>2]|0;
    $285 = (($284) + 1)|0;
    HEAP32[$283>>2] = $285;
    HEAP32[$iDest>>2] = $284;
    $286 = HEAP32[$1>>2]|0;
    $287 = HEAP32[$5>>2]|0;
    $288 = HEAP32[$2>>2]|0;
    $289 = (_autoIncBegin($286,$287,$288)|0);
    HEAP32[$regAutoinc>>2] = $289;
    $290 = HEAP32[$1>>2]|0;
    $291 = (_sqlite3GetTempReg($290)|0);
    HEAP32[$regData>>2] = $291;
    $292 = HEAP32[$1>>2]|0;
    $293 = (_sqlite3GetTempReg($292)|0);
    HEAP32[$regRowid>>2] = $293;
    $294 = HEAP32[$1>>2]|0;
    $295 = HEAP32[$iDest>>2]|0;
    $296 = HEAP32[$5>>2]|0;
    $297 = HEAP32[$2>>2]|0;
    _sqlite3OpenTable($294,$295,$296,$297,55);
    $298 = HEAP32[$2>>2]|0;
    $299 = (($298) + 36|0);
    $300 = HEAP16[$299>>1]|0;
    $301 = $300 << 16 >> 16;
    $302 = ($301|0)<(0);
    if ($302) {
     $303 = HEAP32[$2>>2]|0;
     $304 = (($303) + 8|0);
     $305 = HEAP32[$304>>2]|0;
     $306 = ($305|0)!=(0|0);
     $307 = HEAP32[$destHasUniqueIdx>>2]|0;
     $308 = ($307|0)!=(0);
     $or$cond = $306 | $308;
     if ($or$cond) {
      label = 87;
     } else {
      label = 86;
     }
    } else {
     $$old = HEAP32[$destHasUniqueIdx>>2]|0;
     $$old1 = ($$old|0)!=(0);
     if ($$old1) {
      label = 87;
     } else {
      label = 86;
     }
    }
    if ((label|0) == 86) {
     $309 = HEAP32[$4>>2]|0;
     $310 = ($309|0)!=(2);
     $311 = HEAP32[$4>>2]|0;
     $312 = ($311|0)!=(1);
     $or$cond4 = $310 & $312;
     if ($or$cond4) {
      label = 87;
     }
    }
    if ((label|0) == 87) {
     $313 = HEAP32[$v>>2]|0;
     $314 = HEAP32[$iDest>>2]|0;
     $315 = (_sqlite3VdbeAddOp2($313,108,$314,0)|0);
     HEAP32[$addr1>>2] = $315;
     $316 = HEAP32[$v>>2]|0;
     $317 = (_sqlite3VdbeAddOp2($316,16,0,0)|0);
     HEAP32[$emptyDestTest>>2] = $317;
     $318 = HEAP32[$v>>2]|0;
     $319 = HEAP32[$addr1>>2]|0;
     _sqlite3VdbeJumpHere($318,$319);
    }
    $320 = HEAP32[$pSrc>>2]|0;
    $321 = (($320) + 44|0);
    $322 = HEAP8[$321>>0]|0;
    $323 = $322&255;
    $324 = $323 & 32;
    $325 = ($324|0)==(0);
    $326 = HEAP32[$1>>2]|0;
    if ($325) {
     $327 = HEAP32[$iSrc>>2]|0;
     $328 = HEAP32[$iDbSrc>>2]|0;
     $329 = HEAP32[$pSrc>>2]|0;
     _sqlite3OpenTable($326,$327,$328,$329,54);
     $330 = HEAP32[$v>>2]|0;
     $331 = HEAP32[$iSrc>>2]|0;
     $332 = (_sqlite3VdbeAddOp2($330,108,$331,0)|0);
     HEAP32[$emptySrcTest>>2] = $332;
     $333 = HEAP32[$2>>2]|0;
     $334 = (($333) + 36|0);
     $335 = HEAP16[$334>>1]|0;
     $336 = $335 << 16 >> 16;
     $337 = ($336|0)>=(0);
     do {
      if ($337) {
       $338 = HEAP32[$v>>2]|0;
       $339 = HEAP32[$iSrc>>2]|0;
       $340 = HEAP32[$regRowid>>2]|0;
       $341 = (_sqlite3VdbeAddOp2($338,103,$339,$340)|0);
       HEAP32[$addr1>>2] = $341;
       $342 = HEAP32[$v>>2]|0;
       $343 = HEAP32[$iDest>>2]|0;
       $344 = HEAP32[$regRowid>>2]|0;
       $345 = (_sqlite3VdbeAddOp3($342,70,$343,0,$344)|0);
       HEAP32[$addr2>>2] = $345;
       $346 = HEAP32[$1>>2]|0;
       $347 = HEAP32[$4>>2]|0;
       $348 = HEAP32[$2>>2]|0;
       _sqlite3RowidConstraint($346,$347,$348);
       $349 = HEAP32[$v>>2]|0;
       $350 = HEAP32[$addr2>>2]|0;
       _sqlite3VdbeJumpHere($349,$350);
       $351 = HEAP32[$1>>2]|0;
       $352 = HEAP32[$regAutoinc>>2]|0;
       $353 = HEAP32[$regRowid>>2]|0;
       _autoIncStep($351,$352,$353);
      } else {
       $354 = HEAP32[$2>>2]|0;
       $355 = (($354) + 8|0);
       $356 = HEAP32[$355>>2]|0;
       $357 = ($356|0)==(0|0);
       $358 = HEAP32[$v>>2]|0;
       if ($357) {
        $359 = HEAP32[$iDest>>2]|0;
        $360 = HEAP32[$regRowid>>2]|0;
        $361 = (_sqlite3VdbeAddOp2($358,74,$359,$360)|0);
        HEAP32[$addr1>>2] = $361;
        break;
       } else {
        $362 = HEAP32[$iSrc>>2]|0;
        $363 = HEAP32[$regRowid>>2]|0;
        $364 = (_sqlite3VdbeAddOp2($358,103,$362,$363)|0);
        HEAP32[$addr1>>2] = $364;
        break;
       }
      }
     } while(0);
     $365 = HEAP32[$v>>2]|0;
     $366 = HEAP32[$iSrc>>2]|0;
     $367 = HEAP32[$regData>>2]|0;
     (_sqlite3VdbeAddOp2($365,102,$366,$367)|0);
     $368 = HEAP32[$v>>2]|0;
     $369 = HEAP32[$iDest>>2]|0;
     $370 = HEAP32[$regData>>2]|0;
     $371 = HEAP32[$regRowid>>2]|0;
     (_sqlite3VdbeAddOp3($368,75,$369,$370,$371)|0);
     $372 = HEAP32[$v>>2]|0;
     _sqlite3VdbeChangeP5($372,11);
     $373 = HEAP32[$v>>2]|0;
     $374 = HEAP32[$2>>2]|0;
     $375 = HEAP32[$374>>2]|0;
     _sqlite3VdbeChangeP4($373,-1,$375,0);
     $376 = HEAP32[$v>>2]|0;
     $377 = HEAP32[$iSrc>>2]|0;
     $378 = HEAP32[$addr1>>2]|0;
     (_sqlite3VdbeAddOp2($376,9,$377,$378)|0);
     $379 = HEAP32[$v>>2]|0;
     $380 = HEAP32[$iSrc>>2]|0;
     (_sqlite3VdbeAddOp2($379,61,$380,0)|0);
     $381 = HEAP32[$v>>2]|0;
     $382 = HEAP32[$iDest>>2]|0;
     (_sqlite3VdbeAddOp2($381,61,$382,0)|0);
    } else {
     $383 = HEAP32[$5>>2]|0;
     $384 = HEAP32[$2>>2]|0;
     $385 = (($384) + 32|0);
     $386 = HEAP32[$385>>2]|0;
     $387 = HEAP32[$2>>2]|0;
     $388 = HEAP32[$387>>2]|0;
     _sqlite3TableLock($326,$383,$386,1,$388);
     $389 = HEAP32[$1>>2]|0;
     $390 = HEAP32[$iDbSrc>>2]|0;
     $391 = HEAP32[$pSrc>>2]|0;
     $392 = (($391) + 32|0);
     $393 = HEAP32[$392>>2]|0;
     $394 = HEAP32[$pSrc>>2]|0;
     $395 = HEAP32[$394>>2]|0;
     _sqlite3TableLock($389,$390,$393,0,$395);
    }
    $396 = HEAP32[$2>>2]|0;
    $397 = (($396) + 8|0);
    $398 = HEAP32[$397>>2]|0;
    HEAP32[$pDestIdx>>2] = $398;
    while(1) {
     $399 = HEAP32[$pDestIdx>>2]|0;
     $400 = ($399|0)!=(0|0);
     if (!($400)) {
      break;
     }
     $401 = HEAP32[$pSrc>>2]|0;
     $402 = (($401) + 8|0);
     $403 = HEAP32[$402>>2]|0;
     HEAP32[$pSrcIdx>>2] = $403;
     while(1) {
      $404 = HEAP32[$pSrcIdx>>2]|0;
      $405 = ($404|0)!=(0|0);
      if (!($405)) {
       break;
      }
      $406 = HEAP32[$pDestIdx>>2]|0;
      $407 = HEAP32[$pSrcIdx>>2]|0;
      $408 = (_xferCompatibleIndex($406,$407)|0);
      $409 = ($408|0)!=(0);
      if ($409) {
       break;
      }
      $410 = HEAP32[$pSrcIdx>>2]|0;
      $411 = (($410) + 20|0);
      $412 = HEAP32[$411>>2]|0;
      HEAP32[$pSrcIdx>>2] = $412;
     }
     $413 = HEAP32[$v>>2]|0;
     $414 = HEAP32[$iSrc>>2]|0;
     $415 = HEAP32[$pSrcIdx>>2]|0;
     $416 = (($415) + 44|0);
     $417 = HEAP32[$416>>2]|0;
     $418 = HEAP32[$iDbSrc>>2]|0;
     (_sqlite3VdbeAddOp3($413,54,$414,$417,$418)|0);
     $419 = HEAP32[$1>>2]|0;
     $420 = HEAP32[$pSrcIdx>>2]|0;
     _sqlite3VdbeSetP4KeyInfo($419,$420);
     $421 = HEAP32[$v>>2]|0;
     $422 = HEAP32[$iDest>>2]|0;
     $423 = HEAP32[$pDestIdx>>2]|0;
     $424 = (($423) + 44|0);
     $425 = HEAP32[$424>>2]|0;
     $426 = HEAP32[$5>>2]|0;
     (_sqlite3VdbeAddOp3($421,55,$422,$425,$426)|0);
     $427 = HEAP32[$1>>2]|0;
     $428 = HEAP32[$pDestIdx>>2]|0;
     _sqlite3VdbeSetP4KeyInfo($427,$428);
     $429 = HEAP32[$v>>2]|0;
     _sqlite3VdbeChangeP5($429,1);
     $430 = HEAP32[$v>>2]|0;
     $431 = HEAP32[$iSrc>>2]|0;
     $432 = (_sqlite3VdbeAddOp2($430,108,$431,0)|0);
     HEAP32[$addr1>>2] = $432;
     $433 = HEAP32[$v>>2]|0;
     $434 = HEAP32[$iSrc>>2]|0;
     $435 = HEAP32[$regData>>2]|0;
     (_sqlite3VdbeAddOp2($433,101,$434,$435)|0);
     $436 = HEAP32[$v>>2]|0;
     $437 = HEAP32[$iDest>>2]|0;
     $438 = HEAP32[$regData>>2]|0;
     (_sqlite3VdbeAddOp3($436,110,$437,$438,1)|0);
     $439 = HEAP32[$v>>2]|0;
     $440 = HEAP32[$iSrc>>2]|0;
     $441 = HEAP32[$addr1>>2]|0;
     $442 = (($441) + 1)|0;
     (_sqlite3VdbeAddOp2($439,9,$440,$442)|0);
     $443 = HEAP32[$v>>2]|0;
     $444 = HEAP32[$addr1>>2]|0;
     _sqlite3VdbeJumpHere($443,$444);
     $445 = HEAP32[$v>>2]|0;
     $446 = HEAP32[$iSrc>>2]|0;
     (_sqlite3VdbeAddOp2($445,61,$446,0)|0);
     $447 = HEAP32[$v>>2]|0;
     $448 = HEAP32[$iDest>>2]|0;
     (_sqlite3VdbeAddOp2($447,61,$448,0)|0);
     $449 = HEAP32[$pDestIdx>>2]|0;
     $450 = (($449) + 20|0);
     $451 = HEAP32[$450>>2]|0;
     HEAP32[$pDestIdx>>2] = $451;
    }
    $452 = HEAP32[$emptySrcTest>>2]|0;
    $453 = ($452|0)!=(0);
    if ($453) {
     $454 = HEAP32[$v>>2]|0;
     $455 = HEAP32[$emptySrcTest>>2]|0;
     _sqlite3VdbeJumpHere($454,$455);
    }
    $456 = HEAP32[$1>>2]|0;
    $457 = HEAP32[$regRowid>>2]|0;
    _sqlite3ReleaseTempReg($456,$457);
    $458 = HEAP32[$1>>2]|0;
    $459 = HEAP32[$regData>>2]|0;
    _sqlite3ReleaseTempReg($458,$459);
    $460 = HEAP32[$emptyDestTest>>2]|0;
    $461 = ($460|0)!=(0);
    if ($461) {
     $462 = HEAP32[$v>>2]|0;
     (_sqlite3VdbeAddOp2($462,24,0,0)|0);
     $463 = HEAP32[$v>>2]|0;
     $464 = HEAP32[$emptyDestTest>>2]|0;
     _sqlite3VdbeJumpHere($463,$464);
     $465 = HEAP32[$v>>2]|0;
     $466 = HEAP32[$iDest>>2]|0;
     (_sqlite3VdbeAddOp2($465,61,$466,0)|0);
     HEAP32[$0>>2] = 0;
     $467 = HEAP32[$0>>2]|0;
     STACKTOP = sp;return ($467|0);
    } else {
     HEAP32[$0>>2] = 1;
     $467 = HEAP32[$0>>2]|0;
     STACKTOP = sp;return ($467|0);
    }
   }
  }
 }
 HEAP32[$0>>2] = 0;
 $467 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($467|0);
}
function _autoIncBegin($pParse,$iDb,$pTab) {
 $pParse = $pParse|0;
 $iDb = $iDb|0;
 $pTab = $pTab|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $7 = 0, $8 = 0, $9 = 0, $memId = 0, $pInfo = 0, $pToplevel = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 24|0;
 $1 = sp + 20|0;
 $2 = sp + 16|0;
 $3 = sp + 12|0;
 $memId = sp + 8|0;
 $pToplevel = sp + 4|0;
 $pInfo = sp;
 HEAP32[$1>>2] = $pParse;
 HEAP32[$2>>2] = $iDb;
 HEAP32[$3>>2] = $pTab;
 HEAP32[$memId>>2] = 0;
 $4 = HEAP32[$3>>2]|0;
 $5 = (($4) + 44|0);
 $6 = HEAP8[$5>>0]|0;
 $7 = $6&255;
 $8 = $7 & 8;
 $9 = ($8|0)!=(0);
 if ($9) {
  $10 = HEAP32[$1>>2]|0;
  $11 = (($10) + 408|0);
  $12 = HEAP32[$11>>2]|0;
  $13 = ($12|0)!=(0|0);
  $14 = HEAP32[$1>>2]|0;
  if ($13) {
   $15 = (($14) + 408|0);
   $16 = HEAP32[$15>>2]|0;
   $17 = $16;
  } else {
   $17 = $14;
  }
  HEAP32[$pToplevel>>2] = $17;
  $18 = HEAP32[$pToplevel>>2]|0;
  $19 = (($18) + 404|0);
  $20 = HEAP32[$19>>2]|0;
  HEAP32[$pInfo>>2] = $20;
  while(1) {
   $21 = HEAP32[$pInfo>>2]|0;
   $22 = ($21|0)!=(0|0);
   if ($22) {
    $23 = HEAP32[$pInfo>>2]|0;
    $24 = (($23) + 4|0);
    $25 = HEAP32[$24>>2]|0;
    $26 = HEAP32[$3>>2]|0;
    $27 = ($25|0)!=($26|0);
    $68 = $27;
   } else {
    $68 = 0;
   }
   $28 = HEAP32[$pInfo>>2]|0;
   if (!($68)) {
    break;
   }
   $29 = HEAP32[$28>>2]|0;
   HEAP32[$pInfo>>2] = $29;
  }
  $30 = ($28|0)==(0|0);
  do {
   if ($30) {
    $31 = HEAP32[$1>>2]|0;
    $32 = HEAP32[$31>>2]|0;
    $33 = (_sqlite3DbMallocRaw($32,16,0)|0);
    HEAP32[$pInfo>>2] = $33;
    $34 = HEAP32[$pInfo>>2]|0;
    $35 = ($34|0)==(0|0);
    if (!($35)) {
     $36 = HEAP32[$pToplevel>>2]|0;
     $37 = (($36) + 404|0);
     $38 = HEAP32[$37>>2]|0;
     $39 = HEAP32[$pInfo>>2]|0;
     HEAP32[$39>>2] = $38;
     $40 = HEAP32[$pInfo>>2]|0;
     $41 = HEAP32[$pToplevel>>2]|0;
     $42 = (($41) + 404|0);
     HEAP32[$42>>2] = $40;
     $43 = HEAP32[$3>>2]|0;
     $44 = HEAP32[$pInfo>>2]|0;
     $45 = (($44) + 4|0);
     HEAP32[$45>>2] = $43;
     $46 = HEAP32[$2>>2]|0;
     $47 = HEAP32[$pInfo>>2]|0;
     $48 = (($47) + 8|0);
     HEAP32[$48>>2] = $46;
     $49 = HEAP32[$pToplevel>>2]|0;
     $50 = (($49) + 72|0);
     $51 = HEAP32[$50>>2]|0;
     $52 = (($51) + 1)|0;
     HEAP32[$50>>2] = $52;
     $53 = HEAP32[$pToplevel>>2]|0;
     $54 = (($53) + 72|0);
     $55 = HEAP32[$54>>2]|0;
     $56 = (($55) + 1)|0;
     HEAP32[$54>>2] = $56;
     $57 = HEAP32[$pInfo>>2]|0;
     $58 = (($57) + 12|0);
     HEAP32[$58>>2] = $56;
     $59 = HEAP32[$pToplevel>>2]|0;
     $60 = (($59) + 72|0);
     $61 = HEAP32[$60>>2]|0;
     $62 = (($61) + 1)|0;
     HEAP32[$60>>2] = $62;
     break;
    }
    HEAP32[$0>>2] = 0;
    $67 = HEAP32[$0>>2]|0;
    STACKTOP = sp;return ($67|0);
   }
  } while(0);
  $63 = HEAP32[$pInfo>>2]|0;
  $64 = (($63) + 12|0);
  $65 = HEAP32[$64>>2]|0;
  HEAP32[$memId>>2] = $65;
 }
 $66 = HEAP32[$memId>>2]|0;
 HEAP32[$0>>2] = $66;
 $67 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($67|0);
}
function _readsTable($p,$iDb,$pTab) {
 $p = $p|0;
 $iDb = $iDb|0;
 $pTab = $pTab|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $7 = 0, $8 = 0, $9 = 0, $i = 0, $iEnd = 0, $pIndex = 0, $pOp = 0, $pVTab = 0, $tnum = 0, $v = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $0 = sp + 40|0;
 $1 = sp + 36|0;
 $2 = sp + 32|0;
 $3 = sp + 28|0;
 $v = sp + 24|0;
 $i = sp + 20|0;
 $iEnd = sp + 16|0;
 $pVTab = sp + 12|0;
 $pOp = sp + 8|0;
 $pIndex = sp + 4|0;
 $tnum = sp;
 HEAP32[$1>>2] = $p;
 HEAP32[$2>>2] = $iDb;
 HEAP32[$3>>2] = $pTab;
 $4 = HEAP32[$1>>2]|0;
 $5 = (_sqlite3GetVdbe($4)|0);
 HEAP32[$v>>2] = $5;
 $6 = HEAP32[$v>>2]|0;
 $7 = (_sqlite3VdbeCurrentAddr($6)|0);
 HEAP32[$iEnd>>2] = $7;
 $8 = HEAP32[$3>>2]|0;
 $9 = (($8) + 44|0);
 $10 = HEAP8[$9>>0]|0;
 $11 = $10&255;
 $12 = $11 & 16;
 $13 = ($12|0)!=(0);
 if ($13) {
  $14 = HEAP32[$1>>2]|0;
  $15 = HEAP32[$14>>2]|0;
  $16 = HEAP32[$3>>2]|0;
  $17 = (_sqlite3GetVTable($15,$16)|0);
  $18 = $17;
 } else {
  $18 = 0;
 }
 HEAP32[$pVTab>>2] = $18;
 HEAP32[$i>>2] = 1;
 L4: while(1) {
  $19 = HEAP32[$i>>2]|0;
  $20 = HEAP32[$iEnd>>2]|0;
  $21 = ($19|0)<($20|0);
  if (!($21)) {
   label = 18;
   break;
  }
  $22 = HEAP32[$v>>2]|0;
  $23 = HEAP32[$i>>2]|0;
  $24 = (_sqlite3VdbeGetOp($22,$23)|0);
  HEAP32[$pOp>>2] = $24;
  $25 = HEAP32[$pOp>>2]|0;
  $26 = HEAP8[$25>>0]|0;
  $27 = $26&255;
  $28 = ($27|0)==(54);
  L7: do {
   if ($28) {
    $29 = HEAP32[$pOp>>2]|0;
    $30 = (($29) + 12|0);
    $31 = HEAP32[$30>>2]|0;
    $32 = HEAP32[$2>>2]|0;
    $33 = ($31|0)==($32|0);
    if ($33) {
     $34 = HEAP32[$pOp>>2]|0;
     $35 = (($34) + 8|0);
     $36 = HEAP32[$35>>2]|0;
     HEAP32[$tnum>>2] = $36;
     $37 = HEAP32[$tnum>>2]|0;
     $38 = HEAP32[$3>>2]|0;
     $39 = (($38) + 32|0);
     $40 = HEAP32[$39>>2]|0;
     $41 = ($37|0)==($40|0);
     if ($41) {
      label = 8;
      break L4;
     }
     $42 = HEAP32[$3>>2]|0;
     $43 = (($42) + 8|0);
     $44 = HEAP32[$43>>2]|0;
     HEAP32[$pIndex>>2] = $44;
     while(1) {
      $45 = HEAP32[$pIndex>>2]|0;
      $46 = ($45|0)!=(0|0);
      if (!($46)) {
       break L7;
      }
      $47 = HEAP32[$tnum>>2]|0;
      $48 = HEAP32[$pIndex>>2]|0;
      $49 = (($48) + 44|0);
      $50 = HEAP32[$49>>2]|0;
      $51 = ($47|0)==($50|0);
      if ($51) {
       label = 12;
       break L4;
      }
      $52 = HEAP32[$pIndex>>2]|0;
      $53 = (($52) + 20|0);
      $54 = HEAP32[$53>>2]|0;
      HEAP32[$pIndex>>2] = $54;
     }
    }
   }
  } while(0);
  $55 = HEAP32[$pOp>>2]|0;
  $56 = HEAP8[$55>>0]|0;
  $57 = $56&255;
  $58 = ($57|0)==(147);
  if ($58) {
   $59 = HEAP32[$pOp>>2]|0;
   $60 = (($59) + 16|0);
   $61 = HEAP32[$60>>2]|0;
   $62 = HEAP32[$pVTab>>2]|0;
   $63 = ($61|0)==($62|0);
   if ($63) {
    label = 16;
    break;
   }
  }
  $64 = HEAP32[$i>>2]|0;
  $65 = (($64) + 1)|0;
  HEAP32[$i>>2] = $65;
 }
 if ((label|0) == 8) {
  HEAP32[$0>>2] = 1;
  $66 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($66|0);
 }
 else if ((label|0) == 12) {
  HEAP32[$0>>2] = 1;
  $66 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($66|0);
 }
 else if ((label|0) == 16) {
  HEAP32[$0>>2] = 1;
  $66 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($66|0);
 }
 else if ((label|0) == 18) {
  HEAP32[$0>>2] = 0;
  $66 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($66|0);
 }
 return 0|0;
}
function _sqlite3ExprCodeAndCache($pParse,$pExpr,$target) {
 $pParse = $pParse|0;
 $pExpr = $pExpr|0;
 $target = $target|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $iMem = 0, $v = 0;
 var label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 16|0;
 $1 = sp + 12|0;
 $2 = sp + 8|0;
 $v = sp + 4|0;
 $iMem = sp;
 HEAP32[$0>>2] = $pParse;
 HEAP32[$1>>2] = $pExpr;
 HEAP32[$2>>2] = $target;
 $3 = HEAP32[$0>>2]|0;
 $4 = (($3) + 8|0);
 $5 = HEAP32[$4>>2]|0;
 HEAP32[$v>>2] = $5;
 $6 = HEAP32[$0>>2]|0;
 $7 = HEAP32[$1>>2]|0;
 $8 = HEAP32[$2>>2]|0;
 _sqlite3ExprCode($6,$7,$8);
 $9 = HEAP32[$0>>2]|0;
 $10 = (($9) + 72|0);
 $11 = HEAP32[$10>>2]|0;
 $12 = (($11) + 1)|0;
 HEAP32[$10>>2] = $12;
 HEAP32[$iMem>>2] = $12;
 $13 = HEAP32[$v>>2]|0;
 $14 = HEAP32[$2>>2]|0;
 $15 = HEAP32[$iMem>>2]|0;
 (_sqlite3VdbeAddOp2($13,33,$14,$15)|0);
 $16 = HEAP32[$1>>2]|0;
 $17 = HEAP32[$iMem>>2]|0;
 _exprToRegister($16,$17);
 STACKTOP = sp;return;
}
function _sqlite3TableAffinity($v,$pTab,$iReg) {
 $v = $v|0;
 $pTab = $pTab|0;
 $iReg = $iReg|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $7 = 0, $8 = 0, $9 = 0, $db = 0, $i = 0, $zColAff = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 20|0;
 $1 = sp + 16|0;
 $2 = sp + 12|0;
 $i = sp + 8|0;
 $zColAff = sp + 4|0;
 $db = sp;
 HEAP32[$0>>2] = $v;
 HEAP32[$1>>2] = $pTab;
 HEAP32[$2>>2] = $iReg;
 $3 = HEAP32[$1>>2]|0;
 $4 = (($3) + 20|0);
 $5 = HEAP32[$4>>2]|0;
 HEAP32[$zColAff>>2] = $5;
 $6 = HEAP32[$zColAff>>2]|0;
 $7 = ($6|0)==(0|0);
 if ($7) {
  $8 = HEAP32[$0>>2]|0;
  $9 = (_sqlite3VdbeDb($8)|0);
  HEAP32[$db>>2] = $9;
  $10 = HEAP32[$1>>2]|0;
  $11 = (($10) + 38|0);
  $12 = HEAP16[$11>>1]|0;
  $13 = $12 << 16 >> 16;
  $14 = (($13) + 1)|0;
  $15 = ($14|0)<(0);
  $16 = $15 << 31 >> 31;
  $17 = (_sqlite3DbMallocRaw(0,$14,$16)|0);
  HEAP32[$zColAff>>2] = $17;
  $18 = HEAP32[$zColAff>>2]|0;
  $19 = ($18|0)!=(0|0);
  if (!($19)) {
   $20 = HEAP32[$db>>2]|0;
   $21 = (($20) + 64|0);
   HEAP8[$21>>0] = 1;
   STACKTOP = sp;return;
  }
  HEAP32[$i>>2] = 0;
  while(1) {
   $22 = HEAP32[$i>>2]|0;
   $23 = HEAP32[$1>>2]|0;
   $24 = (($23) + 38|0);
   $25 = HEAP16[$24>>1]|0;
   $26 = $25 << 16 >> 16;
   $27 = ($22|0)<($26|0);
   if (!($27)) {
    break;
   }
   $28 = HEAP32[$i>>2]|0;
   $29 = HEAP32[$1>>2]|0;
   $30 = (($29) + 4|0);
   $31 = HEAP32[$30>>2]|0;
   $32 = (($31) + (($28*24)|0)|0);
   $33 = (($32) + 21|0);
   $34 = HEAP8[$33>>0]|0;
   $35 = HEAP32[$i>>2]|0;
   $36 = HEAP32[$zColAff>>2]|0;
   $37 = (($36) + ($35)|0);
   HEAP8[$37>>0] = $34;
   $38 = HEAP32[$i>>2]|0;
   $39 = (($38) + 1)|0;
   HEAP32[$i>>2] = $39;
  }
  while(1) {
   $40 = HEAP32[$i>>2]|0;
   $41 = (($40) + -1)|0;
   HEAP32[$i>>2] = $41;
   $42 = HEAP32[$zColAff>>2]|0;
   $43 = (($42) + ($40)|0);
   HEAP8[$43>>0] = 0;
   $44 = HEAP32[$i>>2]|0;
   $45 = ($44|0)>=(0);
   if (!($45)) {
    break;
   }
   $46 = HEAP32[$i>>2]|0;
   $47 = HEAP32[$zColAff>>2]|0;
   $48 = (($47) + ($46)|0);
   $49 = HEAP8[$48>>0]|0;
   $50 = $49 << 24 >> 24;
   $51 = ($50|0)==(65);
   if (!($51)) {
    break;
   }
  }
  $52 = HEAP32[$zColAff>>2]|0;
  $53 = HEAP32[$1>>2]|0;
  $54 = (($53) + 20|0);
  HEAP32[$54>>2] = $52;
 }
 $55 = HEAP32[$zColAff>>2]|0;
 $56 = (_sqlite3Strlen30($55)|0);
 HEAP32[$i>>2] = $56;
 $57 = HEAP32[$i>>2]|0;
 $58 = ($57|0)!=(0);
 if (!($58)) {
  STACKTOP = sp;return;
 }
 $59 = HEAP32[$2>>2]|0;
 $60 = ($59|0)!=(0);
 $61 = HEAP32[$0>>2]|0;
 if ($60) {
  $62 = HEAP32[$2>>2]|0;
  $63 = HEAP32[$i>>2]|0;
  $64 = HEAP32[$zColAff>>2]|0;
  $65 = HEAP32[$i>>2]|0;
  (_sqlite3VdbeAddOp4($61,48,$62,$63,0,$64,$65)|0);
  STACKTOP = sp;return;
 } else {
  $66 = HEAP32[$zColAff>>2]|0;
  $67 = HEAP32[$i>>2]|0;
  _sqlite3VdbeChangeP4($61,-1,$66,$67);
  STACKTOP = sp;return;
 }
}
function _sqlite3CodeRowTrigger($pParse,$pTrigger,$op,$pChanges,$tr_tm,$pTab,$reg,$orconf,$ignoreJump) {
 $pParse = $pParse|0;
 $pTrigger = $pTrigger|0;
 $op = $op|0;
 $pChanges = $pChanges|0;
 $tr_tm = $tr_tm|0;
 $pTab = $pTab|0;
 $reg = $reg|0;
 $orconf = $orconf|0;
 $ignoreJump = $ignoreJump|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $p = 0;
 var label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $0 = sp + 36|0;
 $1 = sp + 32|0;
 $2 = sp + 28|0;
 $3 = sp + 24|0;
 $4 = sp + 20|0;
 $5 = sp + 16|0;
 $6 = sp + 12|0;
 $7 = sp + 8|0;
 $8 = sp + 4|0;
 $p = sp;
 HEAP32[$0>>2] = $pParse;
 HEAP32[$1>>2] = $pTrigger;
 HEAP32[$2>>2] = $op;
 HEAP32[$3>>2] = $pChanges;
 HEAP32[$4>>2] = $tr_tm;
 HEAP32[$5>>2] = $pTab;
 HEAP32[$6>>2] = $reg;
 HEAP32[$7>>2] = $orconf;
 HEAP32[$8>>2] = $ignoreJump;
 $9 = HEAP32[$1>>2]|0;
 HEAP32[$p>>2] = $9;
 while(1) {
  $10 = HEAP32[$p>>2]|0;
  $11 = ($10|0)!=(0|0);
  if (!($11)) {
   break;
  }
  $12 = HEAP32[$p>>2]|0;
  $13 = (($12) + 8|0);
  $14 = HEAP8[$13>>0]|0;
  $15 = $14&255;
  $16 = HEAP32[$2>>2]|0;
  $17 = ($15|0)==($16|0);
  if ($17) {
   $18 = HEAP32[$p>>2]|0;
   $19 = (($18) + 9|0);
   $20 = HEAP8[$19>>0]|0;
   $21 = $20&255;
   $22 = HEAP32[$4>>2]|0;
   $23 = ($21|0)==($22|0);
   if ($23) {
    $24 = HEAP32[$p>>2]|0;
    $25 = (($24) + 16|0);
    $26 = HEAP32[$25>>2]|0;
    $27 = HEAP32[$3>>2]|0;
    $28 = (_checkColumnOverlap($26,$27)|0);
    $29 = ($28|0)!=(0);
    if ($29) {
     $30 = HEAP32[$0>>2]|0;
     $31 = HEAP32[$p>>2]|0;
     $32 = HEAP32[$5>>2]|0;
     $33 = HEAP32[$6>>2]|0;
     $34 = HEAP32[$7>>2]|0;
     $35 = HEAP32[$8>>2]|0;
     _sqlite3CodeRowTriggerDirect($30,$31,$32,$33,$34,$35);
    }
   }
  }
  $36 = HEAP32[$p>>2]|0;
  $37 = (($36) + 32|0);
  $38 = HEAP32[$37>>2]|0;
  HEAP32[$p>>2] = $38;
 }
 STACKTOP = sp;return;
}
function _autoIncStep($pParse,$memId,$regRowid) {
 $pParse = $pParse|0;
 $memId = $memId|0;
 $regRowid = $regRowid|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $1 = sp + 4|0;
 $2 = sp;
 HEAP32[$0>>2] = $pParse;
 HEAP32[$1>>2] = $memId;
 HEAP32[$2>>2] = $regRowid;
 $3 = HEAP32[$1>>2]|0;
 $4 = ($3|0)>(0);
 if (!($4)) {
  STACKTOP = sp;return;
 }
 $5 = HEAP32[$0>>2]|0;
 $6 = (($5) + 8|0);
 $7 = HEAP32[$6>>2]|0;
 $8 = HEAP32[$1>>2]|0;
 $9 = HEAP32[$2>>2]|0;
 (_sqlite3VdbeAddOp2($7,136,$8,$9)|0);
 STACKTOP = sp;return;
}
function _sqlite3ExprCodeFactorable($pParse,$pExpr,$target) {
 $pParse = $pParse|0;
 $pExpr = $pExpr|0;
 $target = $target|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $1 = sp + 4|0;
 $2 = sp;
 HEAP32[$0>>2] = $pParse;
 HEAP32[$1>>2] = $pExpr;
 HEAP32[$2>>2] = $target;
 $3 = HEAP32[$0>>2]|0;
 $4 = (($3) + 23|0);
 $5 = HEAP8[$4>>0]|0;
 $6 = $5&255;
 $7 = ($6|0)!=(0);
 if ($7) {
  $8 = HEAP32[$1>>2]|0;
  $9 = (_sqlite3ExprIsConstant($8)|0);
  $10 = ($9|0)!=(0);
  if ($10) {
   $11 = HEAP32[$0>>2]|0;
   $12 = HEAP32[$1>>2]|0;
   $13 = HEAP32[$2>>2]|0;
   _sqlite3ExprCodeAtInit($11,$12,$13,0);
   STACKTOP = sp;return;
  }
 }
 $14 = HEAP32[$0>>2]|0;
 $15 = HEAP32[$1>>2]|0;
 $16 = HEAP32[$2>>2]|0;
 _sqlite3ExprCode($14,$15,$16);
 STACKTOP = sp;return;
}
function _sqlite3VtabMakeWritable($pParse,$pTab) {
 $pParse = $pParse|0;
 $pTab = $pTab|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $apVtabLock = 0, $i = 0, $n = 0, $pToplevel = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 20|0;
 $1 = sp + 16|0;
 $pToplevel = sp + 12|0;
 $i = sp + 8|0;
 $n = sp + 4|0;
 $apVtabLock = sp;
 HEAP32[$0>>2] = $pParse;
 HEAP32[$1>>2] = $pTab;
 $2 = HEAP32[$0>>2]|0;
 $3 = (($2) + 408|0);
 $4 = HEAP32[$3>>2]|0;
 $5 = ($4|0)!=(0|0);
 $6 = HEAP32[$0>>2]|0;
 if ($5) {
  $7 = (($6) + 408|0);
  $8 = HEAP32[$7>>2]|0;
  $9 = $8;
 } else {
  $9 = $6;
 }
 HEAP32[$pToplevel>>2] = $9;
 HEAP32[$i>>2] = 0;
 while(1) {
  $10 = HEAP32[$i>>2]|0;
  $11 = HEAP32[$pToplevel>>2]|0;
  $12 = (($11) + 452|0);
  $13 = HEAP32[$12>>2]|0;
  $14 = ($10|0)<($13|0);
  if (!($14)) {
   break;
  }
  $15 = HEAP32[$1>>2]|0;
  $16 = HEAP32[$i>>2]|0;
  $17 = HEAP32[$pToplevel>>2]|0;
  $18 = (($17) + 520|0);
  $19 = HEAP32[$18>>2]|0;
  $20 = (($19) + ($16<<2)|0);
  $21 = HEAP32[$20>>2]|0;
  $22 = ($15|0)==($21|0);
  if ($22) {
   label = 10;
   break;
  }
  $23 = HEAP32[$i>>2]|0;
  $24 = (($23) + 1)|0;
  HEAP32[$i>>2] = $24;
 }
 if ((label|0) == 10) {
  STACKTOP = sp;return;
 }
 $25 = HEAP32[$pToplevel>>2]|0;
 $26 = (($25) + 452|0);
 $27 = HEAP32[$26>>2]|0;
 $28 = (($27) + 1)|0;
 $29 = $28<<2;
 HEAP32[$n>>2] = $29;
 $30 = HEAP32[$pToplevel>>2]|0;
 $31 = (($30) + 520|0);
 $32 = HEAP32[$31>>2]|0;
 $33 = HEAP32[$n>>2]|0;
 $34 = (_sqlite3_realloc($32,$33)|0);
 HEAP32[$apVtabLock>>2] = $34;
 $35 = HEAP32[$apVtabLock>>2]|0;
 $36 = ($35|0)!=(0|0);
 if ($36) {
  $37 = HEAP32[$apVtabLock>>2]|0;
  $38 = HEAP32[$pToplevel>>2]|0;
  $39 = (($38) + 520|0);
  HEAP32[$39>>2] = $37;
  $40 = HEAP32[$1>>2]|0;
  $41 = HEAP32[$pToplevel>>2]|0;
  $42 = (($41) + 452|0);
  $43 = HEAP32[$42>>2]|0;
  $44 = (($43) + 1)|0;
  HEAP32[$42>>2] = $44;
  $45 = HEAP32[$pToplevel>>2]|0;
  $46 = (($45) + 520|0);
  $47 = HEAP32[$46>>2]|0;
  $48 = (($47) + ($43<<2)|0);
  HEAP32[$48>>2] = $40;
  STACKTOP = sp;return;
 } else {
  $49 = HEAP32[$pToplevel>>2]|0;
  $50 = HEAP32[$49>>2]|0;
  $51 = (($50) + 64|0);
  HEAP8[$51>>0] = 1;
  STACKTOP = sp;return;
 }
}
function _sqlite3GenerateConstraintChecks($pParse,$pTab,$aRegIdx,$iDataCur,$iIdxCur,$regNewData,$regOldData,$pkChng,$overrideError,$ignoreDest,$pbMayReplace) {
 $pParse = $pParse|0;
 $pTab = $pTab|0;
 $aRegIdx = $aRegIdx|0;
 $iDataCur = $iDataCur|0;
 $iIdxCur = $iIdxCur|0;
 $regNewData = $regNewData|0;
 $regOldData = $regOldData|0;
 $pkChng = $pkChng|0;
 $overrideError = $overrideError|0;
 $ignoreDest = $ignoreDest|0;
 $pbMayReplace = $pbMayReplace|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0;
 var $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0, $188 = 0;
 var $189 = 0, $19 = 0, $190 = 0, $191 = 0, $192 = 0, $193 = 0, $194 = 0, $195 = 0, $196 = 0, $197 = 0, $198 = 0, $199 = 0, $2 = 0, $20 = 0, $200 = 0, $201 = 0, $202 = 0, $203 = 0, $204 = 0, $205 = 0;
 var $206 = 0, $207 = 0, $208 = 0, $209 = 0, $21 = 0, $210 = 0, $211 = 0, $212 = 0, $213 = 0, $214 = 0, $215 = 0, $216 = 0, $217 = 0, $218 = 0, $219 = 0, $22 = 0, $220 = 0, $221 = 0, $222 = 0, $223 = 0;
 var $224 = 0, $225 = 0, $226 = 0, $227 = 0, $228 = 0, $229 = 0, $23 = 0, $230 = 0, $231 = 0, $232 = 0, $233 = 0, $234 = 0, $235 = 0, $236 = 0, $237 = 0, $238 = 0, $239 = 0, $24 = 0, $240 = 0, $241 = 0;
 var $242 = 0, $243 = 0, $244 = 0, $245 = 0, $246 = 0, $247 = 0, $248 = 0, $249 = 0, $25 = 0, $250 = 0, $251 = 0, $252 = 0, $253 = 0, $254 = 0, $255 = 0, $256 = 0, $257 = 0, $258 = 0, $259 = 0, $26 = 0;
 var $260 = 0, $261 = 0, $262 = 0, $263 = 0, $264 = 0, $265 = 0, $266 = 0, $267 = 0, $268 = 0, $269 = 0, $27 = 0, $270 = 0, $271 = 0, $272 = 0, $273 = 0, $274 = 0, $275 = 0, $276 = 0, $277 = 0, $278 = 0;
 var $279 = 0, $28 = 0, $280 = 0, $281 = 0, $282 = 0, $283 = 0, $284 = 0, $285 = 0, $286 = 0, $287 = 0, $288 = 0, $289 = 0, $29 = 0, $290 = 0, $291 = 0, $292 = 0, $293 = 0, $294 = 0, $295 = 0, $296 = 0;
 var $297 = 0, $298 = 0, $299 = 0, $3 = 0, $30 = 0, $300 = 0, $301 = 0, $302 = 0, $303 = 0, $304 = 0, $305 = 0, $306 = 0, $307 = 0, $308 = 0, $309 = 0, $31 = 0, $310 = 0, $311 = 0, $312 = 0, $313 = 0;
 var $314 = 0, $315 = 0, $316 = 0, $317 = 0, $318 = 0, $319 = 0, $32 = 0, $320 = 0, $321 = 0, $322 = 0, $323 = 0, $324 = 0, $325 = 0, $326 = 0, $327 = 0, $328 = 0, $329 = 0, $33 = 0, $330 = 0, $331 = 0;
 var $332 = 0, $333 = 0, $334 = 0, $335 = 0, $336 = 0, $337 = 0, $338 = 0, $339 = 0, $34 = 0, $340 = 0, $341 = 0, $342 = 0, $343 = 0, $344 = 0, $345 = 0, $346 = 0, $347 = 0, $348 = 0, $349 = 0, $35 = 0;
 var $350 = 0, $351 = 0, $352 = 0, $353 = 0, $354 = 0, $355 = 0, $356 = 0, $357 = 0, $358 = 0, $359 = 0, $36 = 0, $360 = 0, $361 = 0, $362 = 0, $363 = 0, $364 = 0, $365 = 0, $366 = 0, $367 = 0, $368 = 0;
 var $369 = 0, $37 = 0, $370 = 0, $371 = 0, $372 = 0, $373 = 0, $374 = 0, $375 = 0, $376 = 0, $377 = 0, $378 = 0, $379 = 0, $38 = 0, $380 = 0, $381 = 0, $382 = 0, $383 = 0, $384 = 0, $385 = 0, $386 = 0;
 var $387 = 0, $388 = 0, $389 = 0, $39 = 0, $390 = 0, $391 = 0, $392 = 0, $393 = 0, $394 = 0, $395 = 0, $396 = 0, $397 = 0, $398 = 0, $399 = 0, $4 = 0, $40 = 0, $400 = 0, $401 = 0, $402 = 0, $403 = 0;
 var $404 = 0, $405 = 0, $406 = 0, $407 = 0, $408 = 0, $409 = 0, $41 = 0, $410 = 0, $411 = 0, $412 = 0, $413 = 0, $414 = 0, $415 = 0, $416 = 0, $417 = 0, $418 = 0, $419 = 0, $42 = 0, $420 = 0, $421 = 0;
 var $422 = 0, $423 = 0, $424 = 0, $425 = 0, $426 = 0, $427 = 0, $428 = 0, $429 = 0, $43 = 0, $430 = 0, $431 = 0, $432 = 0, $433 = 0, $434 = 0, $435 = 0, $436 = 0, $437 = 0, $438 = 0, $439 = 0, $44 = 0;
 var $440 = 0, $441 = 0, $442 = 0, $443 = 0, $444 = 0, $445 = 0, $446 = 0, $447 = 0, $448 = 0, $449 = 0, $45 = 0, $450 = 0, $451 = 0, $452 = 0, $453 = 0, $454 = 0, $455 = 0, $456 = 0, $457 = 0, $458 = 0;
 var $459 = 0, $46 = 0, $460 = 0, $461 = 0, $462 = 0, $463 = 0, $464 = 0, $465 = 0, $466 = 0, $467 = 0, $468 = 0, $469 = 0, $47 = 0, $470 = 0, $471 = 0, $472 = 0, $473 = 0, $474 = 0, $475 = 0, $476 = 0;
 var $477 = 0, $478 = 0, $479 = 0, $48 = 0, $480 = 0, $481 = 0, $482 = 0, $483 = 0, $484 = 0, $485 = 0, $486 = 0, $487 = 0, $488 = 0, $489 = 0, $49 = 0, $490 = 0, $491 = 0, $492 = 0, $493 = 0, $494 = 0;
 var $495 = 0, $496 = 0, $497 = 0, $498 = 0, $499 = 0, $5 = 0, $50 = 0, $500 = 0, $501 = 0, $502 = 0, $503 = 0, $504 = 0, $505 = 0, $506 = 0, $507 = 0, $508 = 0, $509 = 0, $51 = 0, $510 = 0, $511 = 0;
 var $512 = 0, $513 = 0, $514 = 0, $515 = 0, $516 = 0, $517 = 0, $518 = 0, $519 = 0, $52 = 0, $520 = 0, $521 = 0, $522 = 0, $523 = 0, $524 = 0, $525 = 0, $526 = 0, $527 = 0, $528 = 0, $529 = 0, $53 = 0;
 var $530 = 0, $531 = 0, $532 = 0, $533 = 0, $534 = 0, $535 = 0, $536 = 0, $537 = 0, $538 = 0, $539 = 0, $54 = 0, $540 = 0, $541 = 0, $542 = 0, $543 = 0, $544 = 0, $545 = 0, $546 = 0, $547 = 0, $548 = 0;
 var $549 = 0, $55 = 0, $550 = 0, $551 = 0, $552 = 0, $553 = 0, $554 = 0, $555 = 0, $556 = 0, $557 = 0, $558 = 0, $559 = 0, $56 = 0, $560 = 0, $561 = 0, $562 = 0, $563 = 0, $564 = 0, $565 = 0, $566 = 0;
 var $567 = 0, $568 = 0, $569 = 0, $57 = 0, $570 = 0, $571 = 0, $572 = 0, $573 = 0, $574 = 0, $575 = 0, $576 = 0, $577 = 0, $578 = 0, $579 = 0, $58 = 0, $580 = 0, $581 = 0, $582 = 0, $583 = 0, $584 = 0;
 var $585 = 0, $586 = 0, $587 = 0, $588 = 0, $589 = 0, $59 = 0, $590 = 0, $591 = 0, $592 = 0, $593 = 0, $594 = 0, $595 = 0, $596 = 0, $597 = 0, $598 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0;
 var $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0;
 var $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $addrJump = 0;
 var $addrRowidOk = 0, $addrUniqueOk = 0, $allOk = 0, $bAffinityDone = 0, $db = 0, $i = 0, $iField = 0, $iThisCur = 0, $ipkBottom = 0, $ipkTop = 0, $isUpdate = 0, $ix = 0, $j1 = 0, $nCol = 0, $nPkField = 0, $onError = 0, $op = 0, $or$cond = 0, $or$cond3 = 0, $p4 = 0;
 var $pCheck = 0, $pIdx = 0, $pPk = 0, $pTrigger = 0, $pTrigger2 = 0, $regCmp = 0, $regIdx = 0, $regR = 0, $regRowid = 0, $seenReplace = 0, $v = 0, $vararg_buffer = 0, $vararg_ptr4 = 0, $x = 0, $x1 = 0, $zMsg = 0, $zName = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 176|0;
 $vararg_buffer = sp;
 $0 = sp + 168|0;
 $1 = sp + 164|0;
 $2 = sp + 160|0;
 $3 = sp + 156|0;
 $4 = sp + 152|0;
 $5 = sp + 148|0;
 $6 = sp + 144|0;
 $7 = sp + 175|0;
 $8 = sp + 174|0;
 $9 = sp + 140|0;
 $10 = sp + 136|0;
 $v = sp + 132|0;
 $pIdx = sp + 128|0;
 $pPk = sp + 124|0;
 $db = sp + 120|0;
 $i = sp + 116|0;
 $ix = sp + 112|0;
 $nCol = sp + 108|0;
 $onError = sp + 104|0;
 $j1 = sp + 100|0;
 $seenReplace = sp + 96|0;
 $nPkField = sp + 92|0;
 $ipkTop = sp + 88|0;
 $ipkBottom = sp + 84|0;
 $isUpdate = sp + 173|0;
 $bAffinityDone = sp + 172|0;
 $regRowid = sp + 80|0;
 $zMsg = sp + 76|0;
 $pCheck = sp + 72|0;
 $allOk = sp + 68|0;
 $zName = sp + 64|0;
 $addrRowidOk = sp + 60|0;
 $pTrigger = sp + 56|0;
 $regIdx = sp + 52|0;
 $regR = sp + 48|0;
 $iThisCur = sp + 44|0;
 $addrUniqueOk = sp + 40|0;
 $iField = sp + 36|0;
 $x = sp + 32|0;
 $x1 = sp + 28|0;
 $addrJump = sp + 24|0;
 $op = sp + 20|0;
 $regCmp = sp + 16|0;
 $p4 = sp + 12|0;
 $pTrigger2 = sp + 8|0;
 HEAP32[$0>>2] = $pParse;
 HEAP32[$1>>2] = $pTab;
 HEAP32[$2>>2] = $aRegIdx;
 HEAP32[$3>>2] = $iDataCur;
 HEAP32[$4>>2] = $iIdxCur;
 HEAP32[$5>>2] = $regNewData;
 HEAP32[$6>>2] = $regOldData;
 HEAP8[$7>>0] = $pkChng;
 HEAP8[$8>>0] = $overrideError;
 HEAP32[$9>>2] = $ignoreDest;
 HEAP32[$10>>2] = $pbMayReplace;
 HEAP32[$pPk>>2] = 0;
 HEAP32[$seenReplace>>2] = 0;
 HEAP32[$ipkTop>>2] = 0;
 HEAP32[$ipkBottom>>2] = 0;
 HEAP8[$bAffinityDone>>0] = 0;
 HEAP32[$regRowid>>2] = -1;
 $11 = HEAP32[$6>>2]|0;
 $12 = ($11|0)!=(0);
 $13 = $12&1;
 $14 = $13&255;
 HEAP8[$isUpdate>>0] = $14;
 $15 = HEAP32[$0>>2]|0;
 $16 = HEAP32[$15>>2]|0;
 HEAP32[$db>>2] = $16;
 $17 = HEAP32[$0>>2]|0;
 $18 = (_sqlite3GetVdbe($17)|0);
 HEAP32[$v>>2] = $18;
 $19 = HEAP32[$1>>2]|0;
 $20 = (($19) + 38|0);
 $21 = HEAP16[$20>>1]|0;
 $22 = $21 << 16 >> 16;
 HEAP32[$nCol>>2] = $22;
 $23 = HEAP32[$1>>2]|0;
 $24 = (($23) + 44|0);
 $25 = HEAP8[$24>>0]|0;
 $26 = $25&255;
 $27 = $26 & 32;
 $28 = ($27|0)==(0);
 if ($28) {
  HEAP32[$pPk>>2] = 0;
  HEAP32[$nPkField>>2] = 1;
 } else {
  $29 = HEAP32[$1>>2]|0;
  $30 = (_sqlite3PrimaryKeyIndex($29)|0);
  HEAP32[$pPk>>2] = $30;
  $31 = HEAP32[$pPk>>2]|0;
  $32 = (($31) + 50|0);
  $33 = HEAP16[$32>>1]|0;
  $34 = $33&65535;
  HEAP32[$nPkField>>2] = $34;
 }
 HEAP32[$i>>2] = 0;
 while(1) {
  $35 = HEAP32[$i>>2]|0;
  $36 = HEAP32[$nCol>>2]|0;
  $37 = ($35|0)<($36|0);
  if (!($37)) {
   break;
  }
  $38 = HEAP32[$i>>2]|0;
  $39 = HEAP32[$1>>2]|0;
  $40 = (($39) + 36|0);
  $41 = HEAP16[$40>>1]|0;
  $42 = $41 << 16 >> 16;
  $43 = ($38|0)==($42|0);
  do {
   if (!($43)) {
    $44 = HEAP32[$i>>2]|0;
    $45 = HEAP32[$1>>2]|0;
    $46 = (($45) + 4|0);
    $47 = HEAP32[$46>>2]|0;
    $48 = (($47) + (($44*24)|0)|0);
    $49 = (($48) + 20|0);
    $50 = HEAP8[$49>>0]|0;
    $51 = $50&255;
    HEAP32[$onError>>2] = $51;
    $52 = HEAP32[$onError>>2]|0;
    $53 = ($52|0)==(0);
    if (!($53)) {
     $54 = HEAP8[$8>>0]|0;
     $55 = $54&255;
     $56 = ($55|0)!=(10);
     if ($56) {
      $57 = HEAP8[$8>>0]|0;
      $58 = $57&255;
      HEAP32[$onError>>2] = $58;
     } else {
      $59 = HEAP32[$onError>>2]|0;
      $60 = ($59|0)==(10);
      if ($60) {
       HEAP32[$onError>>2] = 2;
      }
     }
     $61 = HEAP32[$onError>>2]|0;
     $62 = ($61|0)==(5);
     if ($62) {
      $63 = HEAP32[$i>>2]|0;
      $64 = HEAP32[$1>>2]|0;
      $65 = (($64) + 4|0);
      $66 = HEAP32[$65>>2]|0;
      $67 = (($66) + (($63*24)|0)|0);
      $68 = (($67) + 4|0);
      $69 = HEAP32[$68>>2]|0;
      $70 = ($69|0)==(0|0);
      if ($70) {
       HEAP32[$onError>>2] = 2;
      }
     }
     $71 = HEAP32[$onError>>2]|0;
     if ((($71|0) == 4)) {
      $91 = HEAP32[$v>>2]|0;
      $92 = HEAP32[$5>>2]|0;
      $93 = (($92) + 1)|0;
      $94 = HEAP32[$i>>2]|0;
      $95 = (($93) + ($94))|0;
      $96 = HEAP32[$9>>2]|0;
      (_sqlite3VdbeAddOp2($91,76,$95,$96)|0);
      break;
     } else if ((($71|0) == 2)) {
      $72 = HEAP32[$0>>2]|0;
      _sqlite3MayAbort($72);
     } else if (!((($71|0) == 3) | (($71|0) == 1))) {
      $97 = HEAP32[$v>>2]|0;
      $98 = HEAP32[$5>>2]|0;
      $99 = (($98) + 1)|0;
      $100 = HEAP32[$i>>2]|0;
      $101 = (($99) + ($100))|0;
      $102 = (_sqlite3VdbeAddOp1($97,77,$101)|0);
      HEAP32[$j1>>2] = $102;
      $103 = HEAP32[$0>>2]|0;
      $104 = HEAP32[$i>>2]|0;
      $105 = HEAP32[$1>>2]|0;
      $106 = (($105) + 4|0);
      $107 = HEAP32[$106>>2]|0;
      $108 = (($107) + (($104*24)|0)|0);
      $109 = (($108) + 4|0);
      $110 = HEAP32[$109>>2]|0;
      $111 = HEAP32[$5>>2]|0;
      $112 = (($111) + 1)|0;
      $113 = HEAP32[$i>>2]|0;
      $114 = (($112) + ($113))|0;
      _sqlite3ExprCode($103,$110,$114);
      $115 = HEAP32[$v>>2]|0;
      $116 = HEAP32[$j1>>2]|0;
      _sqlite3VdbeJumpHere($115,$116);
      break;
     }
     $73 = HEAP32[$db>>2]|0;
     $74 = HEAP32[$1>>2]|0;
     $75 = HEAP32[$74>>2]|0;
     $76 = HEAP32[$i>>2]|0;
     $77 = HEAP32[$1>>2]|0;
     $78 = (($77) + 4|0);
     $79 = HEAP32[$78>>2]|0;
     $80 = (($79) + (($76*24)|0)|0);
     $81 = HEAP32[$80>>2]|0;
     HEAP32[$vararg_buffer>>2] = $75;
     $vararg_ptr4 = (($vararg_buffer) + 4|0);
     HEAP32[$vararg_ptr4>>2] = $81;
     $82 = (_sqlite3MPrintf($73,17448,$vararg_buffer)|0);
     HEAP32[$zMsg>>2] = $82;
     $83 = HEAP32[$v>>2]|0;
     $84 = HEAP32[$onError>>2]|0;
     $85 = HEAP32[$5>>2]|0;
     $86 = (($85) + 1)|0;
     $87 = HEAP32[$i>>2]|0;
     $88 = (($86) + ($87))|0;
     $89 = HEAP32[$zMsg>>2]|0;
     (_sqlite3VdbeAddOp4($83,23,1299,$84,$88,$89,-1)|0);
     $90 = HEAP32[$v>>2]|0;
     _sqlite3VdbeChangeP5($90,1);
    }
   }
  } while(0);
  $117 = HEAP32[$i>>2]|0;
  $118 = (($117) + 1)|0;
  HEAP32[$i>>2] = $118;
 }
 $119 = HEAP32[$1>>2]|0;
 $120 = (($119) + 24|0);
 $121 = HEAP32[$120>>2]|0;
 $122 = ($121|0)!=(0|0);
 L27: do {
  if ($122) {
   $123 = HEAP32[$db>>2]|0;
   $124 = (($123) + 24|0);
   $125 = HEAP32[$124>>2]|0;
   $126 = $125 & 8192;
   $127 = ($126|0)==(0);
   if ($127) {
    $128 = HEAP32[$1>>2]|0;
    $129 = (($128) + 24|0);
    $130 = HEAP32[$129>>2]|0;
    HEAP32[$pCheck>>2] = $130;
    $131 = HEAP32[$5>>2]|0;
    $132 = (($131) + 1)|0;
    $133 = HEAP32[$0>>2]|0;
    $134 = (($133) + 92|0);
    HEAP32[$134>>2] = $132;
    $135 = HEAP8[$8>>0]|0;
    $136 = $135&255;
    $137 = ($136|0)!=(10);
    if ($137) {
     $138 = HEAP8[$8>>0]|0;
     $139 = $138&255;
     $140 = $139;
    } else {
     $140 = 2;
    }
    HEAP32[$onError>>2] = $140;
    HEAP32[$i>>2] = 0;
    while(1) {
     $141 = HEAP32[$i>>2]|0;
     $142 = HEAP32[$pCheck>>2]|0;
     $143 = HEAP32[$142>>2]|0;
     $144 = ($141|0)<($143|0);
     if (!($144)) {
      break L27;
     }
     $145 = HEAP32[$v>>2]|0;
     $146 = (_sqlite3VdbeMakeLabel($145)|0);
     HEAP32[$allOk>>2] = $146;
     $147 = HEAP32[$0>>2]|0;
     $148 = HEAP32[$i>>2]|0;
     $149 = HEAP32[$pCheck>>2]|0;
     $150 = (($149) + 4|0);
     $151 = HEAP32[$150>>2]|0;
     $152 = (($151) + (($148*20)|0)|0);
     $153 = HEAP32[$152>>2]|0;
     $154 = HEAP32[$allOk>>2]|0;
     _sqlite3ExprIfTrue($147,$153,$154,16);
     $155 = HEAP32[$onError>>2]|0;
     $156 = ($155|0)==(4);
     if ($156) {
      $157 = HEAP32[$v>>2]|0;
      $158 = HEAP32[$9>>2]|0;
      (_sqlite3VdbeAddOp2($157,16,0,$158)|0);
     } else {
      $159 = HEAP32[$i>>2]|0;
      $160 = HEAP32[$pCheck>>2]|0;
      $161 = (($160) + 4|0);
      $162 = HEAP32[$161>>2]|0;
      $163 = (($162) + (($159*20)|0)|0);
      $164 = (($163) + 4|0);
      $165 = HEAP32[$164>>2]|0;
      HEAP32[$zName>>2] = $165;
      $166 = HEAP32[$zName>>2]|0;
      $167 = ($166|0)==(0|0);
      if ($167) {
       $168 = HEAP32[$1>>2]|0;
       $169 = HEAP32[$168>>2]|0;
       HEAP32[$zName>>2] = $169;
      }
      $170 = HEAP32[$onError>>2]|0;
      $171 = ($170|0)==(5);
      if ($171) {
       HEAP32[$onError>>2] = 2;
      }
      $172 = HEAP32[$0>>2]|0;
      $173 = HEAP32[$onError>>2]|0;
      $174 = HEAP32[$zName>>2]|0;
      _sqlite3HaltConstraint($172,275,$173,$174,0,3);
     }
     $175 = HEAP32[$v>>2]|0;
     $176 = HEAP32[$allOk>>2]|0;
     _sqlite3VdbeResolveLabel($175,$176);
     $177 = HEAP32[$i>>2]|0;
     $178 = (($177) + 1)|0;
     HEAP32[$i>>2] = $178;
    }
   }
  }
 } while(0);
 $179 = HEAP8[$7>>0]|0;
 $180 = $179&255;
 $181 = ($180|0)!=(0);
 $182 = HEAP32[$pPk>>2]|0;
 $183 = ($182|0)==(0|0);
 $or$cond = $181 & $183;
 if ($or$cond) {
  $184 = HEAP32[$v>>2]|0;
  $185 = (_sqlite3VdbeMakeLabel($184)|0);
  HEAP32[$addrRowidOk>>2] = $185;
  $186 = HEAP32[$1>>2]|0;
  $187 = (($186) + 45|0);
  $188 = HEAP8[$187>>0]|0;
  $189 = $188&255;
  HEAP32[$onError>>2] = $189;
  $190 = HEAP8[$8>>0]|0;
  $191 = $190&255;
  $192 = ($191|0)!=(10);
  if ($192) {
   $193 = HEAP8[$8>>0]|0;
   $194 = $193&255;
   HEAP32[$onError>>2] = $194;
  } else {
   $195 = HEAP32[$onError>>2]|0;
   $196 = ($195|0)==(10);
   if ($196) {
    HEAP32[$onError>>2] = 2;
   }
  }
  $197 = HEAP8[$isUpdate>>0]|0;
  $198 = ($197<<24>>24)!=(0);
  if ($198) {
   $199 = HEAP32[$v>>2]|0;
   $200 = HEAP32[$5>>2]|0;
   $201 = HEAP32[$addrRowidOk>>2]|0;
   $202 = HEAP32[$6>>2]|0;
   (_sqlite3VdbeAddOp3($199,79,$200,$201,$202)|0);
   $203 = HEAP32[$v>>2]|0;
   _sqlite3VdbeChangeP5($203,-112);
  }
  $204 = HEAP32[$onError>>2]|0;
  $205 = ($204|0)==(5);
  L57: do {
   if ($205) {
    $206 = HEAP8[$8>>0]|0;
    $207 = $206&255;
    $208 = ($207|0)!=(5);
    if ($208) {
     $209 = HEAP32[$1>>2]|0;
     $210 = (($209) + 8|0);
     $211 = HEAP32[$210>>2]|0;
     HEAP32[$pIdx>>2] = $211;
     while(1) {
      $212 = HEAP32[$pIdx>>2]|0;
      $213 = ($212|0)!=(0|0);
      if (!($213)) {
       break L57;
      }
      $214 = HEAP32[$pIdx>>2]|0;
      $215 = (($214) + 54|0);
      $216 = HEAP8[$215>>0]|0;
      $217 = $216&255;
      $218 = ($217|0)==(4);
      if ($218) {
       break;
      }
      $219 = HEAP32[$pIdx>>2]|0;
      $220 = (($219) + 54|0);
      $221 = HEAP8[$220>>0]|0;
      $222 = $221&255;
      $223 = ($222|0)==(3);
      if ($223) {
       break;
      }
      $226 = HEAP32[$pIdx>>2]|0;
      $227 = (($226) + 20|0);
      $228 = HEAP32[$227>>2]|0;
      HEAP32[$pIdx>>2] = $228;
     }
     $224 = HEAP32[$v>>2]|0;
     $225 = (_sqlite3VdbeAddOp0($224,16)|0);
     HEAP32[$ipkTop>>2] = $225;
    }
   }
  } while(0);
  $229 = HEAP32[$v>>2]|0;
  $230 = HEAP32[$3>>2]|0;
  $231 = HEAP32[$addrRowidOk>>2]|0;
  $232 = HEAP32[$5>>2]|0;
  (_sqlite3VdbeAddOp3($229,70,$230,$231,$232)|0);
  $233 = HEAP32[$onError>>2]|0;
  switch ($233|0) {
  case 5:  {
   HEAP32[$pTrigger>>2] = 0;
   $237 = HEAP32[$db>>2]|0;
   $238 = (($237) + 24|0);
   $239 = HEAP32[$238>>2]|0;
   $240 = $239 & 262144;
   $241 = ($240|0)!=(0);
   if ($241) {
    $242 = HEAP32[$0>>2]|0;
    $243 = HEAP32[$1>>2]|0;
    $244 = (_sqlite3TriggersExist($242,$243,109,0,0)|0);
    HEAP32[$pTrigger>>2] = $244;
   }
   $245 = HEAP32[$pTrigger>>2]|0;
   $246 = ($245|0)!=(0|0);
   if ($246) {
    label = 57;
   } else {
    $247 = HEAP32[$0>>2]|0;
    $248 = HEAP32[$1>>2]|0;
    $249 = (_sqlite3FkRequired($247,$248,0,0)|0);
    $250 = ($249|0)!=(0);
    if ($250) {
     label = 57;
    } else {
     $258 = HEAP32[$1>>2]|0;
     $259 = (($258) + 8|0);
     $260 = HEAP32[$259>>2]|0;
     $261 = ($260|0)!=(0|0);
     if ($261) {
      $262 = HEAP32[$0>>2]|0;
      _sqlite3MultiWrite($262);
      $263 = HEAP32[$0>>2]|0;
      $264 = HEAP32[$1>>2]|0;
      $265 = HEAP32[$3>>2]|0;
      $266 = HEAP32[$4>>2]|0;
      _sqlite3GenerateRowIndexDelete($263,$264,$265,$266,0);
     }
    }
   }
   if ((label|0) == 57) {
    $251 = HEAP32[$0>>2]|0;
    _sqlite3MultiWrite($251);
    $252 = HEAP32[$0>>2]|0;
    $253 = HEAP32[$1>>2]|0;
    $254 = HEAP32[$pTrigger>>2]|0;
    $255 = HEAP32[$3>>2]|0;
    $256 = HEAP32[$4>>2]|0;
    $257 = HEAP32[$5>>2]|0;
    _sqlite3GenerateRowDelete($252,$253,$254,$255,$256,$257,1,0,5,1);
   }
   HEAP32[$seenReplace>>2] = 1;
   break;
  }
  case 3: case 2: case 1:  {
   label = 52;
   break;
  }
  case 4:  {
   $267 = HEAP32[$v>>2]|0;
   $268 = HEAP32[$9>>2]|0;
   (_sqlite3VdbeAddOp2($267,16,0,$268)|0);
   break;
  }
  default: {
   HEAP32[$onError>>2] = 2;
   label = 52;
  }
  }
  if ((label|0) == 52) {
   $234 = HEAP32[$0>>2]|0;
   $235 = HEAP32[$onError>>2]|0;
   $236 = HEAP32[$1>>2]|0;
   _sqlite3RowidConstraint($234,$235,$236);
  }
  $269 = HEAP32[$v>>2]|0;
  $270 = HEAP32[$addrRowidOk>>2]|0;
  _sqlite3VdbeResolveLabel($269,$270);
  $271 = HEAP32[$ipkTop>>2]|0;
  $272 = ($271|0)!=(0);
  if ($272) {
   $273 = HEAP32[$v>>2]|0;
   $274 = (_sqlite3VdbeAddOp0($273,16)|0);
   HEAP32[$ipkBottom>>2] = $274;
   $275 = HEAP32[$v>>2]|0;
   $276 = HEAP32[$ipkTop>>2]|0;
   _sqlite3VdbeJumpHere($275,$276);
  }
 }
 HEAP32[$ix>>2] = 0;
 $277 = HEAP32[$1>>2]|0;
 $278 = (($277) + 8|0);
 $279 = HEAP32[$278>>2]|0;
 HEAP32[$pIdx>>2] = $279;
 while(1) {
  $280 = HEAP32[$pIdx>>2]|0;
  $281 = ($280|0)!=(0|0);
  if (!($281)) {
   break;
  }
  $282 = HEAP32[$ix>>2]|0;
  $283 = HEAP32[$2>>2]|0;
  $284 = (($283) + ($282<<2)|0);
  $285 = HEAP32[$284>>2]|0;
  $286 = ($285|0)==(0);
  do {
   if (!($286)) {
    $287 = HEAP8[$bAffinityDone>>0]|0;
    $288 = $287&255;
    $289 = ($288|0)==(0);
    if ($289) {
     $290 = HEAP32[$v>>2]|0;
     $291 = HEAP32[$1>>2]|0;
     $292 = HEAP32[$5>>2]|0;
     $293 = (($292) + 1)|0;
     _sqlite3TableAffinity($290,$291,$293);
     HEAP8[$bAffinityDone>>0] = 1;
    }
    $294 = HEAP32[$4>>2]|0;
    $295 = HEAP32[$ix>>2]|0;
    $296 = (($294) + ($295))|0;
    HEAP32[$iThisCur>>2] = $296;
    $297 = HEAP32[$v>>2]|0;
    $298 = (_sqlite3VdbeMakeLabel($297)|0);
    HEAP32[$addrUniqueOk>>2] = $298;
    $299 = HEAP32[$pIdx>>2]|0;
    $300 = (($299) + 36|0);
    $301 = HEAP32[$300>>2]|0;
    $302 = ($301|0)!=(0|0);
    if ($302) {
     $303 = HEAP32[$v>>2]|0;
     $304 = HEAP32[$ix>>2]|0;
     $305 = HEAP32[$2>>2]|0;
     $306 = (($305) + ($304<<2)|0);
     $307 = HEAP32[$306>>2]|0;
     (_sqlite3VdbeAddOp2($303,28,0,$307)|0);
     $308 = HEAP32[$5>>2]|0;
     $309 = (($308) + 1)|0;
     $310 = HEAP32[$0>>2]|0;
     $311 = (($310) + 92|0);
     HEAP32[$311>>2] = $309;
     $312 = HEAP32[$0>>2]|0;
     $313 = HEAP32[$pIdx>>2]|0;
     $314 = (($313) + 36|0);
     $315 = HEAP32[$314>>2]|0;
     $316 = HEAP32[$addrUniqueOk>>2]|0;
     _sqlite3ExprIfFalse($312,$315,$316,16);
     $317 = HEAP32[$0>>2]|0;
     $318 = (($317) + 92|0);
     HEAP32[$318>>2] = 0;
    }
    $319 = HEAP32[$0>>2]|0;
    $320 = HEAP32[$pIdx>>2]|0;
    $321 = (($320) + 52|0);
    $322 = HEAP16[$321>>1]|0;
    $323 = $322&65535;
    $324 = (_sqlite3GetTempRange($319,$323)|0);
    HEAP32[$regIdx>>2] = $324;
    HEAP32[$i>>2] = 0;
    while(1) {
     $325 = HEAP32[$i>>2]|0;
     $326 = HEAP32[$pIdx>>2]|0;
     $327 = (($326) + 52|0);
     $328 = HEAP16[$327>>1]|0;
     $329 = $328&65535;
     $330 = ($325|0)<($329|0);
     if (!($330)) {
      break;
     }
     $331 = HEAP32[$i>>2]|0;
     $332 = HEAP32[$pIdx>>2]|0;
     $333 = (($332) + 4|0);
     $334 = HEAP32[$333>>2]|0;
     $335 = (($334) + ($331<<1)|0);
     $336 = HEAP16[$335>>1]|0;
     $337 = $336 << 16 >> 16;
     HEAP32[$iField>>2] = $337;
     $338 = HEAP32[$iField>>2]|0;
     $339 = ($338|0)<(0);
     if ($339) {
      label = 75;
     } else {
      $340 = HEAP32[$iField>>2]|0;
      $341 = HEAP32[$1>>2]|0;
      $342 = (($341) + 36|0);
      $343 = HEAP16[$342>>1]|0;
      $344 = $343 << 16 >> 16;
      $345 = ($340|0)==($344|0);
      if ($345) {
       label = 75;
      } else {
       $360 = HEAP32[$iField>>2]|0;
       $361 = HEAP32[$5>>2]|0;
       $362 = (($360) + ($361))|0;
       $363 = (($362) + 1)|0;
       HEAP32[$x>>2] = $363;
       label = 80;
      }
     }
     if ((label|0) == 75) {
      label = 0;
      $346 = HEAP32[$regRowid>>2]|0;
      $347 = HEAP32[$regIdx>>2]|0;
      $348 = HEAP32[$i>>2]|0;
      $349 = (($347) + ($348))|0;
      $350 = ($346|0)==($349|0);
      if (!($350)) {
       $351 = HEAP32[$5>>2]|0;
       HEAP32[$x>>2] = $351;
       $352 = HEAP32[$pIdx>>2]|0;
       $353 = (($352) + 36|0);
       $354 = HEAP32[$353>>2]|0;
       $355 = ($354|0)!=(0|0);
       if ($355) {
        $359 = -1;
       } else {
        $356 = HEAP32[$regIdx>>2]|0;
        $357 = HEAP32[$i>>2]|0;
        $358 = (($356) + ($357))|0;
        $359 = $358;
       }
       HEAP32[$regRowid>>2] = $359;
       label = 80;
      }
     }
     if ((label|0) == 80) {
      label = 0;
      $364 = HEAP32[$v>>2]|0;
      $365 = HEAP32[$x>>2]|0;
      $366 = HEAP32[$regIdx>>2]|0;
      $367 = HEAP32[$i>>2]|0;
      $368 = (($366) + ($367))|0;
      (_sqlite3VdbeAddOp2($364,34,$365,$368)|0);
     }
     $369 = HEAP32[$i>>2]|0;
     $370 = (($369) + 1)|0;
     HEAP32[$i>>2] = $370;
    }
    $371 = HEAP32[$v>>2]|0;
    $372 = HEAP32[$regIdx>>2]|0;
    $373 = HEAP32[$pIdx>>2]|0;
    $374 = (($373) + 52|0);
    $375 = HEAP16[$374>>1]|0;
    $376 = $375&65535;
    $377 = HEAP32[$ix>>2]|0;
    $378 = HEAP32[$2>>2]|0;
    $379 = (($378) + ($377<<2)|0);
    $380 = HEAP32[$379>>2]|0;
    (_sqlite3VdbeAddOp3($371,49,$372,$376,$380)|0);
    $381 = HEAP32[$0>>2]|0;
    $382 = HEAP32[$regIdx>>2]|0;
    $383 = HEAP32[$pIdx>>2]|0;
    $384 = (($383) + 52|0);
    $385 = HEAP16[$384>>1]|0;
    $386 = $385&65535;
    _sqlite3ExprCacheAffinityChange($381,$382,$386);
    $387 = HEAP8[$isUpdate>>0]|0;
    $388 = $387&255;
    $389 = ($388|0)!=(0);
    if ($389) {
     $390 = HEAP32[$pPk>>2]|0;
     $391 = HEAP32[$pIdx>>2]|0;
     $392 = ($390|0)==($391|0);
     if ($392) {
      $393 = HEAP8[$7>>0]|0;
      $394 = $393&255;
      $395 = ($394|0)==(0);
      if ($395) {
       $396 = HEAP32[$v>>2]|0;
       $397 = HEAP32[$addrUniqueOk>>2]|0;
       _sqlite3VdbeResolveLabel($396,$397);
       break;
      }
     }
    }
    $398 = HEAP32[$pIdx>>2]|0;
    $399 = (($398) + 54|0);
    $400 = HEAP8[$399>>0]|0;
    $401 = $400&255;
    HEAP32[$onError>>2] = $401;
    $402 = HEAP32[$onError>>2]|0;
    $403 = ($402|0)==(0);
    if ($403) {
     $404 = HEAP32[$0>>2]|0;
     $405 = HEAP32[$regIdx>>2]|0;
     $406 = HEAP32[$pIdx>>2]|0;
     $407 = (($406) + 52|0);
     $408 = HEAP16[$407>>1]|0;
     $409 = $408&65535;
     _sqlite3ReleaseTempRange($404,$405,$409);
     $410 = HEAP32[$v>>2]|0;
     $411 = HEAP32[$addrUniqueOk>>2]|0;
     _sqlite3VdbeResolveLabel($410,$411);
     break;
    }
    $412 = HEAP8[$8>>0]|0;
    $413 = $412&255;
    $414 = ($413|0)!=(10);
    if ($414) {
     $415 = HEAP8[$8>>0]|0;
     $416 = $415&255;
     HEAP32[$onError>>2] = $416;
    } else {
     $417 = HEAP32[$onError>>2]|0;
     $418 = ($417|0)==(10);
     if ($418) {
      HEAP32[$onError>>2] = 2;
     }
    }
    $419 = HEAP32[$v>>2]|0;
    $420 = HEAP32[$iThisCur>>2]|0;
    $421 = HEAP32[$addrUniqueOk>>2]|0;
    $422 = HEAP32[$regIdx>>2]|0;
    $423 = HEAP32[$pIdx>>2]|0;
    $424 = (($423) + 50|0);
    $425 = HEAP16[$424>>1]|0;
    $426 = $425&65535;
    (_sqlite3VdbeAddOp4Int($419,67,$420,$421,$422,$426)|0);
    $427 = HEAP32[$pIdx>>2]|0;
    $428 = HEAP32[$pPk>>2]|0;
    $429 = ($427|0)==($428|0);
    if ($429) {
     $430 = HEAP32[$regIdx>>2]|0;
     $434 = $430;
    } else {
     $431 = HEAP32[$0>>2]|0;
     $432 = HEAP32[$nPkField>>2]|0;
     $433 = (_sqlite3GetTempRange($431,$432)|0);
     $434 = $433;
    }
    HEAP32[$regR>>2] = $434;
    $435 = HEAP8[$isUpdate>>0]|0;
    $436 = $435&255;
    $437 = ($436|0)!=(0);
    $438 = HEAP32[$onError>>2]|0;
    $439 = ($438|0)==(5);
    $or$cond3 = $437 | $439;
    L130: do {
     if ($or$cond3) {
      $440 = HEAP32[$1>>2]|0;
      $441 = (($440) + 44|0);
      $442 = HEAP8[$441>>0]|0;
      $443 = $442&255;
      $444 = $443 & 32;
      $445 = ($444|0)==(0);
      if ($445) {
       $446 = HEAP32[$v>>2]|0;
       $447 = HEAP32[$iThisCur>>2]|0;
       $448 = HEAP32[$regR>>2]|0;
       (_sqlite3VdbeAddOp2($446,112,$447,$448)|0);
       $449 = HEAP8[$isUpdate>>0]|0;
       $450 = ($449<<24>>24)!=(0);
       if (!($450)) {
        break;
       }
       $451 = HEAP32[$v>>2]|0;
       $452 = HEAP32[$regR>>2]|0;
       $453 = HEAP32[$addrUniqueOk>>2]|0;
       $454 = HEAP32[$6>>2]|0;
       (_sqlite3VdbeAddOp3($451,79,$452,$453,$454)|0);
       $455 = HEAP32[$v>>2]|0;
       _sqlite3VdbeChangeP5($455,-112);
       break;
      }
      $456 = HEAP32[$pIdx>>2]|0;
      $457 = HEAP32[$pPk>>2]|0;
      $458 = ($456|0)!=($457|0);
      L136: do {
       if ($458) {
        HEAP32[$i>>2] = 0;
        while(1) {
         $459 = HEAP32[$i>>2]|0;
         $460 = HEAP32[$pPk>>2]|0;
         $461 = (($460) + 50|0);
         $462 = HEAP16[$461>>1]|0;
         $463 = $462&65535;
         $464 = ($459|0)<($463|0);
         if (!($464)) {
          break L136;
         }
         $465 = HEAP32[$pIdx>>2]|0;
         $466 = HEAP32[$i>>2]|0;
         $467 = HEAP32[$pPk>>2]|0;
         $468 = (($467) + 4|0);
         $469 = HEAP32[$468>>2]|0;
         $470 = (($469) + ($466<<1)|0);
         $471 = HEAP16[$470>>1]|0;
         $472 = (_sqlite3ColumnOfIndex($465,$471)|0);
         $473 = $472 << 16 >> 16;
         HEAP32[$x1>>2] = $473;
         $474 = HEAP32[$v>>2]|0;
         $475 = HEAP32[$iThisCur>>2]|0;
         $476 = HEAP32[$x1>>2]|0;
         $477 = HEAP32[$regR>>2]|0;
         $478 = HEAP32[$i>>2]|0;
         $479 = (($477) + ($478))|0;
         (_sqlite3VdbeAddOp3($474,47,$475,$476,$479)|0);
         $480 = HEAP32[$i>>2]|0;
         $481 = (($480) + 1)|0;
         HEAP32[$i>>2] = $481;
        }
       }
      } while(0);
      $482 = HEAP8[$isUpdate>>0]|0;
      $483 = ($482<<24>>24)!=(0);
      if ($483) {
       $484 = HEAP32[$v>>2]|0;
       $485 = (_sqlite3VdbeCurrentAddr($484)|0);
       $486 = HEAP32[$pPk>>2]|0;
       $487 = (($486) + 50|0);
       $488 = HEAP16[$487>>1]|0;
       $489 = $488&65535;
       $490 = (($485) + ($489))|0;
       HEAP32[$addrJump>>2] = $490;
       HEAP32[$op>>2] = 78;
       $491 = HEAP32[$pIdx>>2]|0;
       $492 = (($491) + 55|0);
       $493 = HEAP8[$492>>0]|0;
       $494 = $493 & 3;
       $495 = $494&255;
       $496 = ($495|0)==(2);
       $497 = HEAP32[$regIdx>>2]|0;
       $498 = HEAP32[$regR>>2]|0;
       $499 = $496 ? $497 : $498;
       HEAP32[$regCmp>>2] = $499;
       HEAP32[$i>>2] = 0;
       while(1) {
        $500 = HEAP32[$i>>2]|0;
        $501 = HEAP32[$pPk>>2]|0;
        $502 = (($501) + 50|0);
        $503 = HEAP16[$502>>1]|0;
        $504 = $503&65535;
        $505 = ($500|0)<($504|0);
        if (!($505)) {
         break L130;
        }
        $506 = HEAP32[$0>>2]|0;
        $507 = HEAP32[$i>>2]|0;
        $508 = HEAP32[$pPk>>2]|0;
        $509 = (($508) + 32|0);
        $510 = HEAP32[$509>>2]|0;
        $511 = (($510) + ($507<<2)|0);
        $512 = HEAP32[$511>>2]|0;
        $513 = (_sqlite3LocateCollSeq($506,$512)|0);
        HEAP32[$p4>>2] = $513;
        $514 = HEAP32[$i>>2]|0;
        $515 = HEAP32[$pPk>>2]|0;
        $516 = (($515) + 4|0);
        $517 = HEAP32[$516>>2]|0;
        $518 = (($517) + ($514<<1)|0);
        $519 = HEAP16[$518>>1]|0;
        $520 = $519 << 16 >> 16;
        HEAP32[$x1>>2] = $520;
        $521 = HEAP32[$i>>2]|0;
        $522 = HEAP32[$pPk>>2]|0;
        $523 = (($522) + 50|0);
        $524 = HEAP16[$523>>1]|0;
        $525 = $524&65535;
        $526 = (($525) - 1)|0;
        $527 = ($521|0)==($526|0);
        if ($527) {
         $528 = HEAP32[$addrUniqueOk>>2]|0;
         HEAP32[$addrJump>>2] = $528;
         HEAP32[$op>>2] = 79;
        }
        $529 = HEAP32[$v>>2]|0;
        $530 = HEAP32[$op>>2]|0;
        $531 = HEAP32[$6>>2]|0;
        $532 = (($531) + 1)|0;
        $533 = HEAP32[$x1>>2]|0;
        $534 = (($532) + ($533))|0;
        $535 = HEAP32[$addrJump>>2]|0;
        $536 = HEAP32[$regCmp>>2]|0;
        $537 = HEAP32[$i>>2]|0;
        $538 = (($536) + ($537))|0;
        $539 = HEAP32[$p4>>2]|0;
        (_sqlite3VdbeAddOp4($529,$530,$534,$535,$538,$539,-4)|0);
        $540 = HEAP32[$v>>2]|0;
        _sqlite3VdbeChangeP5($540,-112);
        $541 = HEAP32[$i>>2]|0;
        $542 = (($541) + 1)|0;
        HEAP32[$i>>2] = $542;
       }
      }
     }
    } while(0);
    $543 = HEAP32[$onError>>2]|0;
    if ((($543|0) == 4)) {
     $547 = HEAP32[$v>>2]|0;
     $548 = HEAP32[$9>>2]|0;
     (_sqlite3VdbeAddOp2($547,16,0,$548)|0);
    } else if ((($543|0) == 3) | (($543|0) == 2) | (($543|0) == 1)) {
     $544 = HEAP32[$0>>2]|0;
     $545 = HEAP32[$onError>>2]|0;
     $546 = HEAP32[$pIdx>>2]|0;
     _sqlite3UniqueConstraint($544,$545,$546);
    } else {
     HEAP32[$pTrigger2>>2] = 0;
     $549 = HEAP32[$0>>2]|0;
     _sqlite3MultiWrite($549);
     $550 = HEAP32[$db>>2]|0;
     $551 = (($550) + 24|0);
     $552 = HEAP32[$551>>2]|0;
     $553 = $552 & 262144;
     $554 = ($553|0)!=(0);
     if ($554) {
      $555 = HEAP32[$0>>2]|0;
      $556 = HEAP32[$1>>2]|0;
      $557 = (_sqlite3TriggersExist($555,$556,109,0,0)|0);
      HEAP32[$pTrigger2>>2] = $557;
     }
     $558 = HEAP32[$0>>2]|0;
     $559 = HEAP32[$1>>2]|0;
     $560 = HEAP32[$pTrigger2>>2]|0;
     $561 = HEAP32[$3>>2]|0;
     $562 = HEAP32[$4>>2]|0;
     $563 = HEAP32[$regR>>2]|0;
     $564 = HEAP32[$nPkField>>2]|0;
     $565 = $564&65535;
     $566 = HEAP32[$pIdx>>2]|0;
     $567 = HEAP32[$pPk>>2]|0;
     $568 = ($566|0)==($567|0);
     $569 = $568&1;
     $570 = $569&255;
     _sqlite3GenerateRowDelete($558,$559,$560,$561,$562,$563,$565,0,5,$570);
     HEAP32[$seenReplace>>2] = 1;
    }
    $571 = HEAP32[$v>>2]|0;
    $572 = HEAP32[$addrUniqueOk>>2]|0;
    _sqlite3VdbeResolveLabel($571,$572);
    $573 = HEAP32[$0>>2]|0;
    $574 = HEAP32[$regIdx>>2]|0;
    $575 = HEAP32[$pIdx>>2]|0;
    $576 = (($575) + 52|0);
    $577 = HEAP16[$576>>1]|0;
    $578 = $577&65535;
    _sqlite3ReleaseTempRange($573,$574,$578);
    $579 = HEAP32[$regR>>2]|0;
    $580 = HEAP32[$regIdx>>2]|0;
    $581 = ($579|0)!=($580|0);
    if ($581) {
     $582 = HEAP32[$0>>2]|0;
     $583 = HEAP32[$regR>>2]|0;
     $584 = HEAP32[$nPkField>>2]|0;
     _sqlite3ReleaseTempRange($582,$583,$584);
    }
   }
  } while(0);
  $585 = HEAP32[$pIdx>>2]|0;
  $586 = (($585) + 20|0);
  $587 = HEAP32[$586>>2]|0;
  HEAP32[$pIdx>>2] = $587;
  $588 = HEAP32[$ix>>2]|0;
  $589 = (($588) + 1)|0;
  HEAP32[$ix>>2] = $589;
 }
 $590 = HEAP32[$ipkTop>>2]|0;
 $591 = ($590|0)!=(0);
 if (!($591)) {
  $597 = HEAP32[$seenReplace>>2]|0;
  $598 = HEAP32[$10>>2]|0;
  HEAP32[$598>>2] = $597;
  STACKTOP = sp;return;
 }
 $592 = HEAP32[$v>>2]|0;
 $593 = HEAP32[$ipkTop>>2]|0;
 $594 = (($593) + 1)|0;
 (_sqlite3VdbeAddOp2($592,16,0,$594)|0);
 $595 = HEAP32[$v>>2]|0;
 $596 = HEAP32[$ipkBottom>>2]|0;
 _sqlite3VdbeJumpHere($595,$596);
 $597 = HEAP32[$seenReplace>>2]|0;
 $598 = HEAP32[$10>>2]|0;
 HEAP32[$598>>2] = $597;
 STACKTOP = sp;return;
}
function _sqlite3FkCheck($pParse,$pTab,$regOld,$regNew,$aChange,$bChngRowid) {
 $pParse = $pParse|0;
 $pTab = $pTab|0;
 $regOld = $regOld|0;
 $regNew = $regNew|0;
 $aChange = $aChange|0;
 $bChngRowid = $bChngRowid|0;
 var $$sink = 0, $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0;
 var $115 = 0, $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0;
 var $133 = 0, $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0;
 var $151 = 0, $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0;
 var $17 = 0, $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0;
 var $188 = 0, $189 = 0, $19 = 0, $190 = 0, $191 = 0, $192 = 0, $193 = 0, $194 = 0, $195 = 0, $196 = 0, $197 = 0, $198 = 0, $199 = 0, $2 = 0, $20 = 0, $200 = 0, $201 = 0, $202 = 0, $203 = 0, $204 = 0;
 var $205 = 0, $206 = 0, $207 = 0, $208 = 0, $209 = 0, $21 = 0, $210 = 0, $211 = 0, $212 = 0, $213 = 0, $214 = 0, $215 = 0, $216 = 0, $217 = 0, $218 = 0, $219 = 0, $22 = 0, $220 = 0, $221 = 0, $222 = 0;
 var $223 = 0, $224 = 0, $225 = 0, $226 = 0, $227 = 0, $228 = 0, $229 = 0, $23 = 0, $230 = 0, $231 = 0, $232 = 0, $233 = 0, $234 = 0, $235 = 0, $236 = 0, $237 = 0, $238 = 0, $239 = 0, $24 = 0, $240 = 0;
 var $241 = 0, $242 = 0, $243 = 0, $244 = 0, $245 = 0, $246 = 0, $247 = 0, $248 = 0, $249 = 0, $25 = 0, $250 = 0, $251 = 0, $252 = 0, $253 = 0, $254 = 0, $255 = 0, $256 = 0, $257 = 0, $258 = 0, $259 = 0;
 var $26 = 0, $260 = 0, $261 = 0, $262 = 0, $263 = 0, $264 = 0, $265 = 0, $266 = 0, $267 = 0, $268 = 0, $269 = 0, $27 = 0, $270 = 0, $271 = 0, $272 = 0, $273 = 0, $274 = 0, $275 = 0, $276 = 0, $277 = 0;
 var $278 = 0, $279 = 0, $28 = 0, $280 = 0, $281 = 0, $282 = 0, $283 = 0, $284 = 0, $285 = 0, $286 = 0, $287 = 0, $288 = 0, $289 = 0, $29 = 0, $290 = 0, $291 = 0, $292 = 0, $293 = 0, $294 = 0, $295 = 0;
 var $296 = 0, $297 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0;
 var $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0;
 var $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0;
 var $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $aiCol = 0;
 var $aiCol2 = 0, $aiFree = 0, $db = 0, $i = 0, $iCol = 0, $iDb = 0, $iJump = 0, $iReg = 0, $isIgnore = 0, $isIgnoreErrors = 0, $pFKey = 0, $pIdx = 0, $pIdx1 = 0, $pItem = 0, $pSrc = 0, $pTo = 0, $rcauth = 0, $v = 0, $zCol = 0, $zDb = 0;
 var label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 112|0;
 $0 = sp + 104|0;
 $1 = sp + 100|0;
 $2 = sp + 96|0;
 $3 = sp + 92|0;
 $4 = sp + 88|0;
 $5 = sp + 84|0;
 $db = sp + 80|0;
 $pFKey = sp + 76|0;
 $iDb = sp + 72|0;
 $zDb = sp + 68|0;
 $isIgnoreErrors = sp + 64|0;
 $pTo = sp + 60|0;
 $pIdx = sp + 56|0;
 $aiFree = sp + 52|0;
 $aiCol = sp + 48|0;
 $iCol = sp + 44|0;
 $i = sp + 40|0;
 $isIgnore = sp + 36|0;
 $v = sp + 32|0;
 $iJump = sp + 28|0;
 $iReg = sp + 24|0;
 $rcauth = sp + 20|0;
 $zCol = sp + 16|0;
 $pIdx1 = sp + 12|0;
 $pSrc = sp + 8|0;
 $aiCol2 = sp + 4|0;
 $pItem = sp;
 HEAP32[$0>>2] = $pParse;
 HEAP32[$1>>2] = $pTab;
 HEAP32[$2>>2] = $regOld;
 HEAP32[$3>>2] = $regNew;
 HEAP32[$4>>2] = $aChange;
 HEAP32[$5>>2] = $bChngRowid;
 $6 = HEAP32[$0>>2]|0;
 $7 = HEAP32[$6>>2]|0;
 HEAP32[$db>>2] = $7;
 $8 = HEAP32[$0>>2]|0;
 $9 = (($8) + 438|0);
 $10 = HEAP8[$9>>0]|0;
 $11 = $10&255;
 HEAP32[$isIgnoreErrors>>2] = $11;
 $12 = HEAP32[$db>>2]|0;
 $13 = (($12) + 24|0);
 $14 = HEAP32[$13>>2]|0;
 $15 = $14 & 524288;
 $16 = ($15|0)==(0);
 if ($16) {
  STACKTOP = sp;return;
 }
 $17 = HEAP32[$db>>2]|0;
 $18 = HEAP32[$1>>2]|0;
 $19 = (($18) + 68|0);
 $20 = HEAP32[$19>>2]|0;
 $21 = (_sqlite3SchemaToIndex($17,$20)|0);
 HEAP32[$iDb>>2] = $21;
 $22 = HEAP32[$iDb>>2]|0;
 $23 = HEAP32[$db>>2]|0;
 $24 = (($23) + 16|0);
 $25 = HEAP32[$24>>2]|0;
 $26 = (($25) + ($22<<4)|0);
 $27 = HEAP32[$26>>2]|0;
 HEAP32[$zDb>>2] = $27;
 $28 = HEAP32[$1>>2]|0;
 $29 = (($28) + 16|0);
 $30 = HEAP32[$29>>2]|0;
 HEAP32[$pFKey>>2] = $30;
 L4: while(1) {
  $31 = HEAP32[$pFKey>>2]|0;
  $32 = ($31|0)!=(0|0);
  if (!($32)) {
   label = 38;
   break;
  }
  HEAP32[$pIdx>>2] = 0;
  HEAP32[$aiFree>>2] = 0;
  HEAP32[$isIgnore>>2] = 0;
  $33 = HEAP32[$4>>2]|0;
  $34 = ($33|0)!=(0|0);
  if ($34) {
   $35 = HEAP32[$1>>2]|0;
   $36 = HEAP32[$35>>2]|0;
   $37 = HEAP32[$pFKey>>2]|0;
   $38 = (($37) + 8|0);
   $39 = HEAP32[$38>>2]|0;
   $40 = (_sqlite3_stricmp($36,$39)|0);
   $41 = ($40|0)!=(0);
   if ($41) {
    $42 = HEAP32[$1>>2]|0;
    $43 = HEAP32[$pFKey>>2]|0;
    $44 = HEAP32[$4>>2]|0;
    $45 = HEAP32[$5>>2]|0;
    $46 = (_fkChildIsModified($42,$43,$44,$45)|0);
    $47 = ($46|0)==(0);
    if (!($47)) {
     label = 7;
    }
   } else {
    label = 7;
   }
  } else {
   label = 7;
  }
  do {
   if ((label|0) == 7) {
    label = 0;
    $48 = HEAP32[$0>>2]|0;
    $49 = (($48) + 438|0);
    $50 = HEAP8[$49>>0]|0;
    $51 = ($50<<24>>24)!=(0);
    if ($51) {
     $52 = HEAP32[$db>>2]|0;
     $53 = HEAP32[$pFKey>>2]|0;
     $54 = (($53) + 8|0);
     $55 = HEAP32[$54>>2]|0;
     $56 = HEAP32[$zDb>>2]|0;
     $57 = (_sqlite3FindTable($52,$55,$56)|0);
     HEAP32[$pTo>>2] = $57;
    } else {
     $58 = HEAP32[$0>>2]|0;
     $59 = HEAP32[$pFKey>>2]|0;
     $60 = (($59) + 8|0);
     $61 = HEAP32[$60>>2]|0;
     $62 = HEAP32[$zDb>>2]|0;
     $63 = (_sqlite3LocateTable($58,0,$61,$62)|0);
     HEAP32[$pTo>>2] = $63;
    }
    $64 = HEAP32[$pTo>>2]|0;
    $65 = ($64|0)!=(0|0);
    if ($65) {
     $66 = HEAP32[$0>>2]|0;
     $67 = HEAP32[$pTo>>2]|0;
     $68 = HEAP32[$pFKey>>2]|0;
     $69 = (_sqlite3FkLocateIndex($66,$67,$68,$pIdx,$aiFree)|0);
     $70 = ($69|0)!=(0);
     if (!($70)) {
      $112 = HEAP32[$aiFree>>2]|0;
      $113 = ($112|0)!=(0|0);
      if ($113) {
       $114 = HEAP32[$aiFree>>2]|0;
       HEAP32[$aiCol>>2] = $114;
      } else {
       $115 = HEAP32[$pFKey>>2]|0;
       $116 = (($115) + 36|0);
       $117 = HEAP32[$116>>2]|0;
       HEAP32[$iCol>>2] = $117;
       HEAP32[$aiCol>>2] = $iCol;
      }
      HEAP32[$i>>2] = 0;
      while(1) {
       $118 = HEAP32[$i>>2]|0;
       $119 = HEAP32[$pFKey>>2]|0;
       $120 = (($119) + 20|0);
       $121 = HEAP32[$120>>2]|0;
       $122 = ($118|0)<($121|0);
       if (!($122)) {
        break;
       }
       $123 = HEAP32[$i>>2]|0;
       $124 = HEAP32[$aiCol>>2]|0;
       $125 = (($124) + ($123<<2)|0);
       $126 = HEAP32[$125>>2]|0;
       $127 = HEAP32[$1>>2]|0;
       $128 = (($127) + 36|0);
       $129 = HEAP16[$128>>1]|0;
       $130 = $129 << 16 >> 16;
       $131 = ($126|0)==($130|0);
       if ($131) {
        $132 = HEAP32[$i>>2]|0;
        $133 = HEAP32[$aiCol>>2]|0;
        $134 = (($133) + ($132<<2)|0);
        HEAP32[$134>>2] = -1;
       }
       $135 = HEAP32[$db>>2]|0;
       $136 = (($135) + 284|0);
       $137 = HEAP32[$136>>2]|0;
       $138 = ($137|0)!=(0|0);
       if ($138) {
        $139 = HEAP32[$pIdx>>2]|0;
        $140 = ($139|0)!=(0|0);
        if ($140) {
         $141 = HEAP32[$i>>2]|0;
         $142 = HEAP32[$pIdx>>2]|0;
         $143 = (($142) + 4|0);
         $144 = HEAP32[$143>>2]|0;
         $145 = (($144) + ($141<<1)|0);
         $146 = HEAP16[$145>>1]|0;
         $$sink = $146;
        } else {
         $147 = HEAP32[$pTo>>2]|0;
         $148 = (($147) + 36|0);
         $149 = HEAP16[$148>>1]|0;
         $$sink = $149;
        }
        $150 = $$sink << 16 >> 16;
        $151 = HEAP32[$pTo>>2]|0;
        $152 = (($151) + 4|0);
        $153 = HEAP32[$152>>2]|0;
        $154 = (($153) + (($150*24)|0)|0);
        $155 = HEAP32[$154>>2]|0;
        HEAP32[$zCol>>2] = $155;
        $156 = HEAP32[$0>>2]|0;
        $157 = HEAP32[$pTo>>2]|0;
        $158 = HEAP32[$157>>2]|0;
        $159 = HEAP32[$zCol>>2]|0;
        $160 = HEAP32[$iDb>>2]|0;
        $161 = (_sqlite3AuthReadCol($156,$158,$159,$160)|0);
        HEAP32[$rcauth>>2] = $161;
        $162 = HEAP32[$rcauth>>2]|0;
        $163 = ($162|0)==(2);
        $164 = $163&1;
        HEAP32[$isIgnore>>2] = $164;
       }
       $165 = HEAP32[$i>>2]|0;
       $166 = (($165) + 1)|0;
       HEAP32[$i>>2] = $166;
      }
      $167 = HEAP32[$0>>2]|0;
      $168 = HEAP32[$iDb>>2]|0;
      $169 = HEAP32[$pTo>>2]|0;
      $170 = (($169) + 32|0);
      $171 = HEAP32[$170>>2]|0;
      $172 = HEAP32[$pTo>>2]|0;
      $173 = HEAP32[$172>>2]|0;
      _sqlite3TableLock($167,$168,$171,0,$173);
      $174 = HEAP32[$0>>2]|0;
      $175 = (($174) + 68|0);
      $176 = HEAP32[$175>>2]|0;
      $177 = (($176) + 1)|0;
      HEAP32[$175>>2] = $177;
      $178 = HEAP32[$2>>2]|0;
      $179 = ($178|0)!=(0);
      if ($179) {
       $180 = HEAP32[$0>>2]|0;
       $181 = HEAP32[$iDb>>2]|0;
       $182 = HEAP32[$pTo>>2]|0;
       $183 = HEAP32[$pIdx>>2]|0;
       $184 = HEAP32[$pFKey>>2]|0;
       $185 = HEAP32[$aiCol>>2]|0;
       $186 = HEAP32[$2>>2]|0;
       $187 = HEAP32[$isIgnore>>2]|0;
       _fkLookupParent($180,$181,$182,$183,$184,$185,$186,-1,$187);
      }
      $188 = HEAP32[$3>>2]|0;
      $189 = ($188|0)!=(0);
      if ($189) {
       $190 = HEAP32[$0>>2]|0;
       $191 = HEAP32[$iDb>>2]|0;
       $192 = HEAP32[$pTo>>2]|0;
       $193 = HEAP32[$pIdx>>2]|0;
       $194 = HEAP32[$pFKey>>2]|0;
       $195 = HEAP32[$aiCol>>2]|0;
       $196 = HEAP32[$3>>2]|0;
       $197 = HEAP32[$isIgnore>>2]|0;
       _fkLookupParent($190,$191,$192,$193,$194,$195,$196,1,$197);
      }
      $198 = HEAP32[$db>>2]|0;
      $199 = HEAP32[$aiFree>>2]|0;
      _sqlite3DbFree($198,$199);
      break;
     }
    }
    $71 = HEAP32[$isIgnoreErrors>>2]|0;
    $72 = ($71|0)!=(0);
    if (!($72)) {
     label = 57;
     break L4;
    }
    $73 = HEAP32[$db>>2]|0;
    $74 = (($73) + 64|0);
    $75 = HEAP8[$74>>0]|0;
    $76 = $75&255;
    $77 = ($76|0)!=(0);
    if ($77) {
     label = 57;
     break L4;
    }
    $78 = HEAP32[$pTo>>2]|0;
    $79 = ($78|0)==(0|0);
    if ($79) {
     $80 = HEAP32[$0>>2]|0;
     $81 = (_sqlite3GetVdbe($80)|0);
     HEAP32[$v>>2] = $81;
     $82 = HEAP32[$v>>2]|0;
     $83 = (_sqlite3VdbeCurrentAddr($82)|0);
     $84 = HEAP32[$pFKey>>2]|0;
     $85 = (($84) + 20|0);
     $86 = HEAP32[$85>>2]|0;
     $87 = (($83) + ($86))|0;
     $88 = (($87) + 1)|0;
     HEAP32[$iJump>>2] = $88;
     HEAP32[$i>>2] = 0;
     while(1) {
      $89 = HEAP32[$i>>2]|0;
      $90 = HEAP32[$pFKey>>2]|0;
      $91 = (($90) + 20|0);
      $92 = HEAP32[$91>>2]|0;
      $93 = ($89|0)<($92|0);
      if (!($93)) {
       break;
      }
      $94 = HEAP32[$i>>2]|0;
      $95 = HEAP32[$pFKey>>2]|0;
      $96 = (($95) + 36|0);
      $97 = (($96) + ($94<<3)|0);
      $98 = HEAP32[$97>>2]|0;
      $99 = HEAP32[$2>>2]|0;
      $100 = (($98) + ($99))|0;
      $101 = (($100) + 1)|0;
      HEAP32[$iReg>>2] = $101;
      $102 = HEAP32[$v>>2]|0;
      $103 = HEAP32[$iReg>>2]|0;
      $104 = HEAP32[$iJump>>2]|0;
      (_sqlite3VdbeAddOp2($102,76,$103,$104)|0);
      $105 = HEAP32[$i>>2]|0;
      $106 = (($105) + 1)|0;
      HEAP32[$i>>2] = $106;
     }
     $107 = HEAP32[$v>>2]|0;
     $108 = HEAP32[$pFKey>>2]|0;
     $109 = (($108) + 24|0);
     $110 = HEAP8[$109>>0]|0;
     $111 = $110&255;
     (_sqlite3VdbeAddOp2($107,134,$111,-1)|0);
    }
   }
  } while(0);
  $200 = HEAP32[$pFKey>>2]|0;
  $201 = (($200) + 4|0);
  $202 = HEAP32[$201>>2]|0;
  HEAP32[$pFKey>>2] = $202;
 }
 if ((label|0) == 38) {
  $203 = HEAP32[$1>>2]|0;
  $204 = (_sqlite3FkReferences($203)|0);
  HEAP32[$pFKey>>2] = $204;
  L56: while(1) {
   $205 = HEAP32[$pFKey>>2]|0;
   $206 = ($205|0)!=(0|0);
   if (!($206)) {
    label = 57;
    break;
   }
   HEAP32[$pIdx1>>2] = 0;
   HEAP32[$aiCol2>>2] = 0;
   $207 = HEAP32[$4>>2]|0;
   $208 = ($207|0)!=(0|0);
   if ($208) {
    $209 = HEAP32[$1>>2]|0;
    $210 = HEAP32[$pFKey>>2]|0;
    $211 = HEAP32[$4>>2]|0;
    $212 = HEAP32[$5>>2]|0;
    $213 = (_fkParentIsModified($209,$210,$211,$212)|0);
    $214 = ($213|0)==(0);
    if (!($214)) {
     label = 42;
    }
   } else {
    label = 42;
   }
   do {
    if ((label|0) == 42) {
     label = 0;
     $215 = HEAP32[$pFKey>>2]|0;
     $216 = (($215) + 24|0);
     $217 = HEAP8[$216>>0]|0;
     $218 = ($217<<24>>24)!=(0);
     if (!($218)) {
      $219 = HEAP32[$db>>2]|0;
      $220 = (($219) + 24|0);
      $221 = HEAP32[$220>>2]|0;
      $222 = $221 & 16777216;
      $223 = ($222|0)!=(0);
      if (!($223)) {
       $224 = HEAP32[$0>>2]|0;
       $225 = (($224) + 408|0);
       $226 = HEAP32[$225>>2]|0;
       $227 = ($226|0)!=(0|0);
       if (!($227)) {
        $228 = HEAP32[$0>>2]|0;
        $229 = (($228) + 20|0);
        $230 = HEAP8[$229>>0]|0;
        $231 = ($230<<24>>24)!=(0);
        if (!($231)) {
         break;
        }
       }
      }
     }
     $232 = HEAP32[$0>>2]|0;
     $233 = HEAP32[$1>>2]|0;
     $234 = HEAP32[$pFKey>>2]|0;
     $235 = (_sqlite3FkLocateIndex($232,$233,$234,$pIdx1,$aiCol2)|0);
     $236 = ($235|0)!=(0);
     if ($236) {
      $237 = HEAP32[$isIgnoreErrors>>2]|0;
      $238 = ($237|0)!=(0);
      if (!($238)) {
       label = 57;
       break L56;
      }
      $239 = HEAP32[$db>>2]|0;
      $240 = (($239) + 64|0);
      $241 = HEAP8[$240>>0]|0;
      $242 = $241&255;
      $243 = ($242|0)!=(0);
      if ($243) {
       label = 57;
       break L56;
      } else {
       break;
      }
     }
     $244 = HEAP32[$db>>2]|0;
     $245 = (_sqlite3SrcListAppend($244,0,0,0)|0);
     HEAP32[$pSrc>>2] = $245;
     $246 = HEAP32[$pSrc>>2]|0;
     $247 = ($246|0)!=(0|0);
     if ($247) {
      $248 = HEAP32[$pSrc>>2]|0;
      $249 = (($248) + 8|0);
      HEAP32[$pItem>>2] = $249;
      $250 = HEAP32[$pFKey>>2]|0;
      $251 = HEAP32[$250>>2]|0;
      $252 = HEAP32[$pItem>>2]|0;
      $253 = (($252) + 16|0);
      HEAP32[$253>>2] = $251;
      $254 = HEAP32[$pFKey>>2]|0;
      $255 = HEAP32[$254>>2]|0;
      $256 = HEAP32[$255>>2]|0;
      $257 = HEAP32[$pItem>>2]|0;
      $258 = (($257) + 8|0);
      HEAP32[$258>>2] = $256;
      $259 = HEAP32[$pItem>>2]|0;
      $260 = (($259) + 16|0);
      $261 = HEAP32[$260>>2]|0;
      $262 = (($261) + 40|0);
      $263 = HEAP16[$262>>1]|0;
      $264 = (($263) + 1)<<16>>16;
      HEAP16[$262>>1] = $264;
      $265 = HEAP32[$0>>2]|0;
      $266 = (($265) + 68|0);
      $267 = HEAP32[$266>>2]|0;
      $268 = (($267) + 1)|0;
      HEAP32[$266>>2] = $268;
      $269 = HEAP32[$pItem>>2]|0;
      $270 = (($269) + 40|0);
      HEAP32[$270>>2] = $267;
      $271 = HEAP32[$3>>2]|0;
      $272 = ($271|0)!=(0);
      if ($272) {
       $273 = HEAP32[$0>>2]|0;
       $274 = HEAP32[$pSrc>>2]|0;
       $275 = HEAP32[$1>>2]|0;
       $276 = HEAP32[$pIdx1>>2]|0;
       $277 = HEAP32[$pFKey>>2]|0;
       $278 = HEAP32[$aiCol2>>2]|0;
       $279 = HEAP32[$3>>2]|0;
       _fkScanChildren($273,$274,$275,$276,$277,$278,$279,-1);
      }
      $280 = HEAP32[$2>>2]|0;
      $281 = ($280|0)!=(0);
      if ($281) {
       $282 = HEAP32[$0>>2]|0;
       $283 = HEAP32[$pSrc>>2]|0;
       $284 = HEAP32[$1>>2]|0;
       $285 = HEAP32[$pIdx1>>2]|0;
       $286 = HEAP32[$pFKey>>2]|0;
       $287 = HEAP32[$aiCol2>>2]|0;
       $288 = HEAP32[$2>>2]|0;
       _fkScanChildren($282,$283,$284,$285,$286,$287,$288,1);
      }
      $289 = HEAP32[$pItem>>2]|0;
      $290 = (($289) + 8|0);
      HEAP32[$290>>2] = 0;
      $291 = HEAP32[$db>>2]|0;
      $292 = HEAP32[$pSrc>>2]|0;
      _sqlite3SrcListDelete($291,$292);
     }
     $293 = HEAP32[$db>>2]|0;
     $294 = HEAP32[$aiCol2>>2]|0;
     _sqlite3DbFree($293,$294);
    }
   } while(0);
   $295 = HEAP32[$pFKey>>2]|0;
   $296 = (($295) + 12|0);
   $297 = HEAP32[$296>>2]|0;
   HEAP32[$pFKey>>2] = $297;
  }
  if ((label|0) == 57) {
   STACKTOP = sp;return;
  }
 }
 else if ((label|0) == 57) {
  STACKTOP = sp;return;
 }
}
function _sqlite3CompleteInsertion($pParse,$pTab,$iDataCur,$iIdxCur,$regNewData,$aRegIdx,$isUpdate,$appendBias,$useSeekResult) {
 $pParse = $pParse|0;
 $pTab = $pTab|0;
 $iDataCur = $iDataCur|0;
 $iIdxCur = $iIdxCur|0;
 $regNewData = $regNewData|0;
 $aRegIdx = $aRegIdx|0;
 $isUpdate = $isUpdate|0;
 $appendBias = $appendBias|0;
 $useSeekResult = $useSeekResult|0;
 var $$ = 0, $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0;
 var $115 = 0, $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $14 = 0, $15 = 0;
 var $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0;
 var $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0;
 var $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0;
 var $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0;
 var $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $bAffinityDone = 0, $i = 0, $pIdx = 0, $pik_flags = 0, $regData = 0, $regRec = 0, $v = 0, label = 0;
 var sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 64|0;
 $0 = sp + 52|0;
 $1 = sp + 48|0;
 $2 = sp + 44|0;
 $3 = sp + 40|0;
 $4 = sp + 36|0;
 $5 = sp + 32|0;
 $6 = sp + 28|0;
 $7 = sp + 24|0;
 $8 = sp + 20|0;
 $v = sp + 16|0;
 $pIdx = sp + 12|0;
 $pik_flags = sp + 57|0;
 $regData = sp + 8|0;
 $regRec = sp + 4|0;
 $i = sp;
 $bAffinityDone = sp + 56|0;
 HEAP32[$0>>2] = $pParse;
 HEAP32[$1>>2] = $pTab;
 HEAP32[$2>>2] = $iDataCur;
 HEAP32[$3>>2] = $iIdxCur;
 HEAP32[$4>>2] = $regNewData;
 HEAP32[$5>>2] = $aRegIdx;
 HEAP32[$6>>2] = $isUpdate;
 HEAP32[$7>>2] = $appendBias;
 HEAP32[$8>>2] = $useSeekResult;
 HEAP8[$bAffinityDone>>0] = 0;
 $9 = HEAP32[$0>>2]|0;
 $10 = (_sqlite3GetVdbe($9)|0);
 HEAP32[$v>>2] = $10;
 HEAP32[$i>>2] = 0;
 $11 = HEAP32[$1>>2]|0;
 $12 = (($11) + 8|0);
 $13 = HEAP32[$12>>2]|0;
 HEAP32[$pIdx>>2] = $13;
 while(1) {
  $14 = HEAP32[$pIdx>>2]|0;
  $15 = ($14|0)!=(0|0);
  if (!($15)) {
   break;
  }
  $16 = HEAP32[$i>>2]|0;
  $17 = HEAP32[$5>>2]|0;
  $18 = (($17) + ($16<<2)|0);
  $19 = HEAP32[$18>>2]|0;
  $20 = ($19|0)==(0);
  if (!($20)) {
   HEAP8[$bAffinityDone>>0] = 1;
   $21 = HEAP32[$pIdx>>2]|0;
   $22 = (($21) + 36|0);
   $23 = HEAP32[$22>>2]|0;
   $24 = ($23|0)!=(0|0);
   if ($24) {
    $25 = HEAP32[$v>>2]|0;
    $26 = HEAP32[$i>>2]|0;
    $27 = HEAP32[$5>>2]|0;
    $28 = (($27) + ($26<<2)|0);
    $29 = HEAP32[$28>>2]|0;
    $30 = HEAP32[$v>>2]|0;
    $31 = (_sqlite3VdbeCurrentAddr($30)|0);
    $32 = (($31) + 2)|0;
    (_sqlite3VdbeAddOp2($25,76,$29,$32)|0);
   }
   $33 = HEAP32[$v>>2]|0;
   $34 = HEAP32[$3>>2]|0;
   $35 = HEAP32[$i>>2]|0;
   $36 = (($34) + ($35))|0;
   $37 = HEAP32[$i>>2]|0;
   $38 = HEAP32[$5>>2]|0;
   $39 = (($38) + ($37<<2)|0);
   $40 = HEAP32[$39>>2]|0;
   (_sqlite3VdbeAddOp2($33,110,$36,$40)|0);
   HEAP8[$pik_flags>>0] = 0;
   $41 = HEAP32[$8>>2]|0;
   $42 = ($41|0)!=(0);
   $$ = $42 ? 16 : 0;
   HEAP8[$pik_flags>>0] = $$;
   $43 = HEAP32[$pIdx>>2]|0;
   $44 = (($43) + 55|0);
   $45 = HEAP8[$44>>0]|0;
   $46 = $45 & 3;
   $47 = $46&255;
   $48 = ($47|0)==(2);
   if ($48) {
    $49 = HEAP32[$1>>2]|0;
    $50 = (($49) + 44|0);
    $51 = HEAP8[$50>>0]|0;
    $52 = $51&255;
    $53 = $52 & 32;
    $54 = ($53|0)==(0);
    if (!($54)) {
     $55 = HEAP8[$pik_flags>>0]|0;
     $56 = $55&255;
     $57 = $56 | 1;
     $58 = $57&255;
     HEAP8[$pik_flags>>0] = $58;
    }
   }
   $59 = HEAP8[$pik_flags>>0]|0;
   $60 = ($59<<24>>24)!=(0);
   if ($60) {
    $61 = HEAP32[$v>>2]|0;
    $62 = HEAP8[$pik_flags>>0]|0;
    _sqlite3VdbeChangeP5($61,$62);
   }
  }
  $63 = HEAP32[$pIdx>>2]|0;
  $64 = (($63) + 20|0);
  $65 = HEAP32[$64>>2]|0;
  HEAP32[$pIdx>>2] = $65;
  $66 = HEAP32[$i>>2]|0;
  $67 = (($66) + 1)|0;
  HEAP32[$i>>2] = $67;
 }
 $68 = HEAP32[$1>>2]|0;
 $69 = (($68) + 44|0);
 $70 = HEAP8[$69>>0]|0;
 $71 = $70&255;
 $72 = $71 & 32;
 $73 = ($72|0)==(0);
 if (!($73)) {
  STACKTOP = sp;return;
 }
 $74 = HEAP32[$4>>2]|0;
 $75 = (($74) + 1)|0;
 HEAP32[$regData>>2] = $75;
 $76 = HEAP32[$0>>2]|0;
 $77 = (_sqlite3GetTempReg($76)|0);
 HEAP32[$regRec>>2] = $77;
 $78 = HEAP32[$v>>2]|0;
 $79 = HEAP32[$regData>>2]|0;
 $80 = HEAP32[$1>>2]|0;
 $81 = (($80) + 38|0);
 $82 = HEAP16[$81>>1]|0;
 $83 = $82 << 16 >> 16;
 $84 = HEAP32[$regRec>>2]|0;
 (_sqlite3VdbeAddOp3($78,49,$79,$83,$84)|0);
 $85 = HEAP8[$bAffinityDone>>0]|0;
 $86 = ($85<<24>>24)!=(0);
 if (!($86)) {
  $87 = HEAP32[$v>>2]|0;
  $88 = HEAP32[$1>>2]|0;
  _sqlite3TableAffinity($87,$88,0);
 }
 $89 = HEAP32[$0>>2]|0;
 $90 = HEAP32[$regData>>2]|0;
 $91 = HEAP32[$1>>2]|0;
 $92 = (($91) + 38|0);
 $93 = HEAP16[$92>>1]|0;
 $94 = $93 << 16 >> 16;
 _sqlite3ExprCacheAffinityChange($89,$90,$94);
 $95 = HEAP32[$0>>2]|0;
 $96 = (($95) + 18|0);
 $97 = HEAP8[$96>>0]|0;
 $98 = ($97<<24>>24)!=(0);
 if ($98) {
  HEAP8[$pik_flags>>0] = 0;
 } else {
  HEAP8[$pik_flags>>0] = 1;
  $99 = HEAP32[$6>>2]|0;
  $100 = ($99|0)!=(0);
  $101 = $100 ? 4 : 2;
  $102 = HEAP8[$pik_flags>>0]|0;
  $103 = $102&255;
  $104 = $103 | $101;
  $105 = $104&255;
  HEAP8[$pik_flags>>0] = $105;
 }
 $106 = HEAP32[$7>>2]|0;
 $107 = ($106|0)!=(0);
 if ($107) {
  $108 = HEAP8[$pik_flags>>0]|0;
  $109 = $108&255;
  $110 = $109 | 8;
  $111 = $110&255;
  HEAP8[$pik_flags>>0] = $111;
 }
 $112 = HEAP32[$8>>2]|0;
 $113 = ($112|0)!=(0);
 if ($113) {
  $114 = HEAP8[$pik_flags>>0]|0;
  $115 = $114&255;
  $116 = $115 | 16;
  $117 = $116&255;
  HEAP8[$pik_flags>>0] = $117;
 }
 $118 = HEAP32[$v>>2]|0;
 $119 = HEAP32[$2>>2]|0;
 $120 = HEAP32[$regRec>>2]|0;
 $121 = HEAP32[$4>>2]|0;
 (_sqlite3VdbeAddOp3($118,75,$119,$120,$121)|0);
 $122 = HEAP32[$0>>2]|0;
 $123 = (($122) + 18|0);
 $124 = HEAP8[$123>>0]|0;
 $125 = ($124<<24>>24)!=(0);
 if (!($125)) {
  $126 = HEAP32[$v>>2]|0;
  $127 = HEAP32[$1>>2]|0;
  $128 = HEAP32[$127>>2]|0;
  _sqlite3VdbeChangeP4($126,-1,$128,0);
 }
 $129 = HEAP32[$v>>2]|0;
 $130 = HEAP8[$pik_flags>>0]|0;
 _sqlite3VdbeChangeP5($129,$130);
 STACKTOP = sp;return;
}
function _sqlite3AutoincrementEnd($pParse) {
 $pParse = $pParse|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $db = 0, $iRec = 0, $j1 = 0, $memId = 0, $p = 0;
 var $pDb = 0, $v = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 28|0;
 $p = sp + 24|0;
 $v = sp + 20|0;
 $db = sp + 16|0;
 $pDb = sp + 12|0;
 $j1 = sp + 8|0;
 $iRec = sp + 4|0;
 $memId = sp;
 HEAP32[$0>>2] = $pParse;
 $1 = HEAP32[$0>>2]|0;
 $2 = (($1) + 8|0);
 $3 = HEAP32[$2>>2]|0;
 HEAP32[$v>>2] = $3;
 $4 = HEAP32[$0>>2]|0;
 $5 = HEAP32[$4>>2]|0;
 HEAP32[$db>>2] = $5;
 $6 = HEAP32[$0>>2]|0;
 $7 = (($6) + 404|0);
 $8 = HEAP32[$7>>2]|0;
 HEAP32[$p>>2] = $8;
 while(1) {
  $9 = HEAP32[$p>>2]|0;
  $10 = ($9|0)!=(0|0);
  if (!($10)) {
   break;
  }
  $11 = HEAP32[$p>>2]|0;
  $12 = (($11) + 8|0);
  $13 = HEAP32[$12>>2]|0;
  $14 = HEAP32[$db>>2]|0;
  $15 = (($14) + 16|0);
  $16 = HEAP32[$15>>2]|0;
  $17 = (($16) + ($13<<4)|0);
  HEAP32[$pDb>>2] = $17;
  $18 = HEAP32[$p>>2]|0;
  $19 = (($18) + 12|0);
  $20 = HEAP32[$19>>2]|0;
  HEAP32[$memId>>2] = $20;
  $21 = HEAP32[$0>>2]|0;
  $22 = (_sqlite3GetTempReg($21)|0);
  HEAP32[$iRec>>2] = $22;
  $23 = HEAP32[$0>>2]|0;
  $24 = HEAP32[$p>>2]|0;
  $25 = (($24) + 8|0);
  $26 = HEAP32[$25>>2]|0;
  $27 = HEAP32[$pDb>>2]|0;
  $28 = (($27) + 12|0);
  $29 = HEAP32[$28>>2]|0;
  $30 = (($29) + 72|0);
  $31 = HEAP32[$30>>2]|0;
  _sqlite3OpenTable($23,0,$26,$31,55);
  $32 = HEAP32[$v>>2]|0;
  $33 = HEAP32[$memId>>2]|0;
  $34 = (($33) + 1)|0;
  $35 = (_sqlite3VdbeAddOp1($32,77,$34)|0);
  HEAP32[$j1>>2] = $35;
  $36 = HEAP32[$v>>2]|0;
  $37 = HEAP32[$memId>>2]|0;
  $38 = (($37) + 1)|0;
  (_sqlite3VdbeAddOp2($36,74,0,$38)|0);
  $39 = HEAP32[$v>>2]|0;
  $40 = HEAP32[$j1>>2]|0;
  _sqlite3VdbeJumpHere($39,$40);
  $41 = HEAP32[$v>>2]|0;
  $42 = HEAP32[$memId>>2]|0;
  $43 = (($42) - 1)|0;
  $44 = HEAP32[$iRec>>2]|0;
  (_sqlite3VdbeAddOp3($41,49,$43,2,$44)|0);
  $45 = HEAP32[$v>>2]|0;
  $46 = HEAP32[$iRec>>2]|0;
  $47 = HEAP32[$memId>>2]|0;
  $48 = (($47) + 1)|0;
  (_sqlite3VdbeAddOp3($45,75,0,$46,$48)|0);
  $49 = HEAP32[$v>>2]|0;
  _sqlite3VdbeChangeP5($49,8);
  $50 = HEAP32[$v>>2]|0;
  (_sqlite3VdbeAddOp0($50,61)|0);
  $51 = HEAP32[$0>>2]|0;
  $52 = HEAP32[$iRec>>2]|0;
  _sqlite3ReleaseTempReg($51,$52);
  $53 = HEAP32[$p>>2]|0;
  $54 = HEAP32[$53>>2]|0;
  HEAP32[$p>>2] = $54;
 }
 STACKTOP = sp;return;
}
function _fkChildIsModified($pTab,$p,$aChange,$bChngRowid) {
 $pTab = $pTab|0;
 $p = $p|0;
 $aChange = $aChange|0;
 $bChngRowid = $bChngRowid|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $i = 0, $iChildKey = 0, $or$cond = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 24|0;
 $1 = sp + 20|0;
 $2 = sp + 16|0;
 $3 = sp + 12|0;
 $4 = sp + 8|0;
 $i = sp + 4|0;
 $iChildKey = sp;
 HEAP32[$1>>2] = $pTab;
 HEAP32[$2>>2] = $p;
 HEAP32[$3>>2] = $aChange;
 HEAP32[$4>>2] = $bChngRowid;
 HEAP32[$i>>2] = 0;
 while(1) {
  $5 = HEAP32[$i>>2]|0;
  $6 = HEAP32[$2>>2]|0;
  $7 = (($6) + 20|0);
  $8 = HEAP32[$7>>2]|0;
  $9 = ($5|0)<($8|0);
  if (!($9)) {
   label = 8;
   break;
  }
  $10 = HEAP32[$i>>2]|0;
  $11 = HEAP32[$2>>2]|0;
  $12 = (($11) + 36|0);
  $13 = (($12) + ($10<<3)|0);
  $14 = HEAP32[$13>>2]|0;
  HEAP32[$iChildKey>>2] = $14;
  $15 = HEAP32[$iChildKey>>2]|0;
  $16 = HEAP32[$3>>2]|0;
  $17 = (($16) + ($15<<2)|0);
  $18 = HEAP32[$17>>2]|0;
  $19 = ($18|0)>=(0);
  if ($19) {
   label = 4;
   break;
  }
  $20 = HEAP32[$iChildKey>>2]|0;
  $21 = HEAP32[$1>>2]|0;
  $22 = (($21) + 36|0);
  $23 = HEAP16[$22>>1]|0;
  $24 = $23 << 16 >> 16;
  $25 = ($20|0)==($24|0);
  $26 = HEAP32[$4>>2]|0;
  $27 = ($26|0)!=(0);
  $or$cond = $25 & $27;
  if ($or$cond) {
   label = 6;
   break;
  }
  $28 = HEAP32[$i>>2]|0;
  $29 = (($28) + 1)|0;
  HEAP32[$i>>2] = $29;
 }
 if ((label|0) == 4) {
  HEAP32[$0>>2] = 1;
  $30 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($30|0);
 }
 else if ((label|0) == 6) {
  HEAP32[$0>>2] = 1;
  $30 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($30|0);
 }
 else if ((label|0) == 8) {
  HEAP32[$0>>2] = 0;
  $30 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($30|0);
 }
 return 0|0;
}
function _fkLookupParent($pParse,$iDb,$pTab,$pIdx,$pFKey,$aiCol,$regData,$nIncr,$isIgnore) {
 $pParse = $pParse|0;
 $iDb = $iDb|0;
 $pTab = $pTab|0;
 $pIdx = $pIdx|0;
 $pFKey = $pFKey|0;
 $aiCol = $aiCol|0;
 $regData = $regData|0;
 $nIncr = $nIncr|0;
 $isIgnore = $isIgnore|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0;
 var $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0, $188 = 0;
 var $189 = 0, $19 = 0, $190 = 0, $191 = 0, $192 = 0, $193 = 0, $194 = 0, $195 = 0, $196 = 0, $197 = 0, $198 = 0, $199 = 0, $2 = 0, $20 = 0, $200 = 0, $201 = 0, $202 = 0, $203 = 0, $204 = 0, $205 = 0;
 var $206 = 0, $207 = 0, $208 = 0, $209 = 0, $21 = 0, $210 = 0, $211 = 0, $212 = 0, $213 = 0, $214 = 0, $215 = 0, $216 = 0, $217 = 0, $218 = 0, $219 = 0, $22 = 0, $220 = 0, $221 = 0, $222 = 0, $223 = 0;
 var $224 = 0, $225 = 0, $226 = 0, $227 = 0, $228 = 0, $229 = 0, $23 = 0, $230 = 0, $231 = 0, $232 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0;
 var $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0;
 var $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0;
 var $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0;
 var $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $i = 0, $iChild = 0, $iCur = 0, $iJump = 0, $iMustBeInt = 0, $iOk = 0, $iParent = 0;
 var $iReg = 0, $nCol = 0, $or$cond = 0, $or$cond3 = 0, $regRec = 0, $regTemp = 0, $regTemp1 = 0, $v = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 96|0;
 $0 = sp + 84|0;
 $1 = sp + 80|0;
 $2 = sp + 76|0;
 $3 = sp + 72|0;
 $4 = sp + 68|0;
 $5 = sp + 64|0;
 $6 = sp + 60|0;
 $7 = sp + 56|0;
 $8 = sp + 52|0;
 $i = sp + 48|0;
 $v = sp + 44|0;
 $iCur = sp + 40|0;
 $iOk = sp + 36|0;
 $iReg = sp + 32|0;
 $iMustBeInt = sp + 28|0;
 $regTemp = sp + 24|0;
 $nCol = sp + 20|0;
 $regTemp1 = sp + 16|0;
 $regRec = sp + 12|0;
 $iJump = sp + 8|0;
 $iChild = sp + 4|0;
 $iParent = sp;
 HEAP32[$0>>2] = $pParse;
 HEAP32[$1>>2] = $iDb;
 HEAP32[$2>>2] = $pTab;
 HEAP32[$3>>2] = $pIdx;
 HEAP32[$4>>2] = $pFKey;
 HEAP32[$5>>2] = $aiCol;
 HEAP32[$6>>2] = $regData;
 HEAP32[$7>>2] = $nIncr;
 HEAP32[$8>>2] = $isIgnore;
 $9 = HEAP32[$0>>2]|0;
 $10 = (_sqlite3GetVdbe($9)|0);
 HEAP32[$v>>2] = $10;
 $11 = HEAP32[$0>>2]|0;
 $12 = (($11) + 68|0);
 $13 = HEAP32[$12>>2]|0;
 $14 = (($13) - 1)|0;
 HEAP32[$iCur>>2] = $14;
 $15 = HEAP32[$v>>2]|0;
 $16 = (_sqlite3VdbeMakeLabel($15)|0);
 HEAP32[$iOk>>2] = $16;
 $17 = HEAP32[$7>>2]|0;
 $18 = ($17|0)<(0);
 if ($18) {
  $19 = HEAP32[$v>>2]|0;
  $20 = HEAP32[$4>>2]|0;
  $21 = (($20) + 24|0);
  $22 = HEAP8[$21>>0]|0;
  $23 = $22&255;
  $24 = HEAP32[$iOk>>2]|0;
  (_sqlite3VdbeAddOp2($19,135,$23,$24)|0);
 }
 HEAP32[$i>>2] = 0;
 while(1) {
  $25 = HEAP32[$i>>2]|0;
  $26 = HEAP32[$4>>2]|0;
  $27 = (($26) + 20|0);
  $28 = HEAP32[$27>>2]|0;
  $29 = ($25|0)<($28|0);
  if (!($29)) {
   break;
  }
  $30 = HEAP32[$i>>2]|0;
  $31 = HEAP32[$5>>2]|0;
  $32 = (($31) + ($30<<2)|0);
  $33 = HEAP32[$32>>2]|0;
  $34 = HEAP32[$6>>2]|0;
  $35 = (($33) + ($34))|0;
  $36 = (($35) + 1)|0;
  HEAP32[$iReg>>2] = $36;
  $37 = HEAP32[$v>>2]|0;
  $38 = HEAP32[$iReg>>2]|0;
  $39 = HEAP32[$iOk>>2]|0;
  (_sqlite3VdbeAddOp2($37,76,$38,$39)|0);
  $40 = HEAP32[$i>>2]|0;
  $41 = (($40) + 1)|0;
  HEAP32[$i>>2] = $41;
 }
 $42 = HEAP32[$8>>2]|0;
 $43 = ($42|0)==(0);
 do {
  if ($43) {
   $44 = HEAP32[$3>>2]|0;
   $45 = ($44|0)==(0|0);
   if ($45) {
    $46 = HEAP32[$0>>2]|0;
    $47 = (_sqlite3GetTempReg($46)|0);
    HEAP32[$regTemp>>2] = $47;
    $48 = HEAP32[$v>>2]|0;
    $49 = HEAP32[$5>>2]|0;
    $50 = HEAP32[$49>>2]|0;
    $51 = (($50) + 1)|0;
    $52 = HEAP32[$6>>2]|0;
    $53 = (($51) + ($52))|0;
    $54 = HEAP32[$regTemp>>2]|0;
    (_sqlite3VdbeAddOp2($48,34,$53,$54)|0);
    $55 = HEAP32[$v>>2]|0;
    $56 = HEAP32[$regTemp>>2]|0;
    $57 = (_sqlite3VdbeAddOp2($55,38,$56,0)|0);
    HEAP32[$iMustBeInt>>2] = $57;
    $58 = HEAP32[$2>>2]|0;
    $59 = HEAP32[$4>>2]|0;
    $60 = HEAP32[$59>>2]|0;
    $61 = ($58|0)==($60|0);
    $62 = HEAP32[$7>>2]|0;
    $63 = ($62|0)==(1);
    $or$cond = $61 & $63;
    if ($or$cond) {
     $64 = HEAP32[$v>>2]|0;
     $65 = HEAP32[$6>>2]|0;
     $66 = HEAP32[$iOk>>2]|0;
     $67 = HEAP32[$regTemp>>2]|0;
     (_sqlite3VdbeAddOp3($64,79,$65,$66,$67)|0);
     $68 = HEAP32[$v>>2]|0;
     _sqlite3VdbeChangeP5($68,-112);
    }
    $69 = HEAP32[$0>>2]|0;
    $70 = HEAP32[$iCur>>2]|0;
    $71 = HEAP32[$1>>2]|0;
    $72 = HEAP32[$2>>2]|0;
    _sqlite3OpenTable($69,$70,$71,$72,54);
    $73 = HEAP32[$v>>2]|0;
    $74 = HEAP32[$iCur>>2]|0;
    $75 = HEAP32[$regTemp>>2]|0;
    (_sqlite3VdbeAddOp3($73,70,$74,0,$75)|0);
    $76 = HEAP32[$v>>2]|0;
    $77 = HEAP32[$iOk>>2]|0;
    (_sqlite3VdbeAddOp2($76,16,0,$77)|0);
    $78 = HEAP32[$v>>2]|0;
    $79 = HEAP32[$v>>2]|0;
    $80 = (_sqlite3VdbeCurrentAddr($79)|0);
    $81 = (($80) - 2)|0;
    _sqlite3VdbeJumpHere($78,$81);
    $82 = HEAP32[$v>>2]|0;
    $83 = HEAP32[$iMustBeInt>>2]|0;
    _sqlite3VdbeJumpHere($82,$83);
    $84 = HEAP32[$0>>2]|0;
    $85 = HEAP32[$regTemp>>2]|0;
    _sqlite3ReleaseTempReg($84,$85);
    break;
   }
   $86 = HEAP32[$4>>2]|0;
   $87 = (($86) + 20|0);
   $88 = HEAP32[$87>>2]|0;
   HEAP32[$nCol>>2] = $88;
   $89 = HEAP32[$0>>2]|0;
   $90 = HEAP32[$nCol>>2]|0;
   $91 = (_sqlite3GetTempRange($89,$90)|0);
   HEAP32[$regTemp1>>2] = $91;
   $92 = HEAP32[$0>>2]|0;
   $93 = (_sqlite3GetTempReg($92)|0);
   HEAP32[$regRec>>2] = $93;
   $94 = HEAP32[$v>>2]|0;
   $95 = HEAP32[$iCur>>2]|0;
   $96 = HEAP32[$3>>2]|0;
   $97 = (($96) + 44|0);
   $98 = HEAP32[$97>>2]|0;
   $99 = HEAP32[$1>>2]|0;
   (_sqlite3VdbeAddOp3($94,54,$95,$98,$99)|0);
   $100 = HEAP32[$0>>2]|0;
   $101 = HEAP32[$3>>2]|0;
   _sqlite3VdbeSetP4KeyInfo($100,$101);
   HEAP32[$i>>2] = 0;
   while(1) {
    $102 = HEAP32[$i>>2]|0;
    $103 = HEAP32[$nCol>>2]|0;
    $104 = ($102|0)<($103|0);
    if (!($104)) {
     break;
    }
    $105 = HEAP32[$v>>2]|0;
    $106 = HEAP32[$i>>2]|0;
    $107 = HEAP32[$5>>2]|0;
    $108 = (($107) + ($106<<2)|0);
    $109 = HEAP32[$108>>2]|0;
    $110 = (($109) + 1)|0;
    $111 = HEAP32[$6>>2]|0;
    $112 = (($110) + ($111))|0;
    $113 = HEAP32[$regTemp1>>2]|0;
    $114 = HEAP32[$i>>2]|0;
    $115 = (($113) + ($114))|0;
    (_sqlite3VdbeAddOp2($105,33,$112,$115)|0);
    $116 = HEAP32[$i>>2]|0;
    $117 = (($116) + 1)|0;
    HEAP32[$i>>2] = $117;
   }
   $118 = HEAP32[$2>>2]|0;
   $119 = HEAP32[$4>>2]|0;
   $120 = HEAP32[$119>>2]|0;
   $121 = ($118|0)==($120|0);
   $122 = HEAP32[$7>>2]|0;
   $123 = ($122|0)==(1);
   $or$cond3 = $121 & $123;
   if ($or$cond3) {
    $124 = HEAP32[$v>>2]|0;
    $125 = (_sqlite3VdbeCurrentAddr($124)|0);
    $126 = HEAP32[$nCol>>2]|0;
    $127 = (($125) + ($126))|0;
    $128 = (($127) + 1)|0;
    HEAP32[$iJump>>2] = $128;
    HEAP32[$i>>2] = 0;
    while(1) {
     $129 = HEAP32[$i>>2]|0;
     $130 = HEAP32[$nCol>>2]|0;
     $131 = ($129|0)<($130|0);
     if (!($131)) {
      break;
     }
     $132 = HEAP32[$i>>2]|0;
     $133 = HEAP32[$5>>2]|0;
     $134 = (($133) + ($132<<2)|0);
     $135 = HEAP32[$134>>2]|0;
     $136 = (($135) + 1)|0;
     $137 = HEAP32[$6>>2]|0;
     $138 = (($136) + ($137))|0;
     HEAP32[$iChild>>2] = $138;
     $139 = HEAP32[$i>>2]|0;
     $140 = HEAP32[$3>>2]|0;
     $141 = (($140) + 4|0);
     $142 = HEAP32[$141>>2]|0;
     $143 = (($142) + ($139<<1)|0);
     $144 = HEAP16[$143>>1]|0;
     $145 = $144 << 16 >> 16;
     $146 = (($145) + 1)|0;
     $147 = HEAP32[$6>>2]|0;
     $148 = (($146) + ($147))|0;
     HEAP32[$iParent>>2] = $148;
     $149 = HEAP32[$i>>2]|0;
     $150 = HEAP32[$3>>2]|0;
     $151 = (($150) + 4|0);
     $152 = HEAP32[$151>>2]|0;
     $153 = (($152) + ($149<<1)|0);
     $154 = HEAP16[$153>>1]|0;
     $155 = $154 << 16 >> 16;
     $156 = HEAP32[$2>>2]|0;
     $157 = (($156) + 36|0);
     $158 = HEAP16[$157>>1]|0;
     $159 = $158 << 16 >> 16;
     $160 = ($155|0)==($159|0);
     if ($160) {
      $161 = HEAP32[$6>>2]|0;
      HEAP32[$iParent>>2] = $161;
     }
     $162 = HEAP32[$v>>2]|0;
     $163 = HEAP32[$iChild>>2]|0;
     $164 = HEAP32[$iJump>>2]|0;
     $165 = HEAP32[$iParent>>2]|0;
     (_sqlite3VdbeAddOp3($162,78,$163,$164,$165)|0);
     $166 = HEAP32[$v>>2]|0;
     _sqlite3VdbeChangeP5($166,16);
     $167 = HEAP32[$i>>2]|0;
     $168 = (($167) + 1)|0;
     HEAP32[$i>>2] = $168;
    }
    $169 = HEAP32[$v>>2]|0;
    $170 = HEAP32[$iOk>>2]|0;
    (_sqlite3VdbeAddOp2($169,16,0,$170)|0);
   }
   $171 = HEAP32[$v>>2]|0;
   $172 = HEAP32[$regTemp1>>2]|0;
   $173 = HEAP32[$nCol>>2]|0;
   $174 = HEAP32[$regRec>>2]|0;
   $175 = HEAP32[$v>>2]|0;
   $176 = HEAP32[$3>>2]|0;
   $177 = (_sqlite3IndexAffinityStr($175,$176)|0);
   $178 = HEAP32[$nCol>>2]|0;
   (_sqlite3VdbeAddOp4($171,49,$172,$173,$174,$177,$178)|0);
   $179 = HEAP32[$v>>2]|0;
   $180 = HEAP32[$iCur>>2]|0;
   $181 = HEAP32[$iOk>>2]|0;
   $182 = HEAP32[$regRec>>2]|0;
   (_sqlite3VdbeAddOp4Int($179,69,$180,$181,$182,0)|0);
   $183 = HEAP32[$0>>2]|0;
   $184 = HEAP32[$regRec>>2]|0;
   _sqlite3ReleaseTempReg($183,$184);
   $185 = HEAP32[$0>>2]|0;
   $186 = HEAP32[$regTemp1>>2]|0;
   $187 = HEAP32[$nCol>>2]|0;
   _sqlite3ReleaseTempRange($185,$186,$187);
  }
 } while(0);
 $188 = HEAP32[$4>>2]|0;
 $189 = (($188) + 24|0);
 $190 = HEAP8[$189>>0]|0;
 $191 = ($190<<24>>24)!=(0);
 if (!($191)) {
  $192 = HEAP32[$0>>2]|0;
  $193 = HEAP32[$192>>2]|0;
  $194 = (($193) + 24|0);
  $195 = HEAP32[$194>>2]|0;
  $196 = $195 & 16777216;
  $197 = ($196|0)!=(0);
  if (!($197)) {
   $198 = HEAP32[$0>>2]|0;
   $199 = (($198) + 408|0);
   $200 = HEAP32[$199>>2]|0;
   $201 = ($200|0)!=(0|0);
   if (!($201)) {
    $202 = HEAP32[$0>>2]|0;
    $203 = (($202) + 20|0);
    $204 = HEAP8[$203>>0]|0;
    $205 = ($204<<24>>24)!=(0);
    if (!($205)) {
     $206 = HEAP32[$0>>2]|0;
     _sqlite3HaltConstraint($206,787,2,0,-2,4);
     $229 = HEAP32[$v>>2]|0;
     $230 = HEAP32[$iOk>>2]|0;
     _sqlite3VdbeResolveLabel($229,$230);
     $231 = HEAP32[$v>>2]|0;
     $232 = HEAP32[$iCur>>2]|0;
     (_sqlite3VdbeAddOp1($231,61,$232)|0);
     STACKTOP = sp;return;
    }
   }
  }
 }
 $207 = HEAP32[$7>>2]|0;
 $208 = ($207|0)>(0);
 if ($208) {
  $209 = HEAP32[$4>>2]|0;
  $210 = (($209) + 24|0);
  $211 = HEAP8[$210>>0]|0;
  $212 = $211&255;
  $213 = ($212|0)==(0);
  if ($213) {
   $214 = HEAP32[$0>>2]|0;
   $215 = (($214) + 408|0);
   $216 = HEAP32[$215>>2]|0;
   $217 = ($216|0)!=(0|0);
   $218 = HEAP32[$0>>2]|0;
   if ($217) {
    $219 = (($218) + 408|0);
    $220 = HEAP32[$219>>2]|0;
    $222 = $220;
   } else {
    $222 = $218;
   }
   $221 = (($222) + 21|0);
   HEAP8[$221>>0] = 1;
  }
 }
 $223 = HEAP32[$v>>2]|0;
 $224 = HEAP32[$4>>2]|0;
 $225 = (($224) + 24|0);
 $226 = HEAP8[$225>>0]|0;
 $227 = $226&255;
 $228 = HEAP32[$7>>2]|0;
 (_sqlite3VdbeAddOp2($223,134,$227,$228)|0);
 $229 = HEAP32[$v>>2]|0;
 $230 = HEAP32[$iOk>>2]|0;
 _sqlite3VdbeResolveLabel($229,$230);
 $231 = HEAP32[$v>>2]|0;
 $232 = HEAP32[$iCur>>2]|0;
 (_sqlite3VdbeAddOp1($231,61,$232)|0);
 STACKTOP = sp;return;
}
function _fkParentIsModified($pTab,$p,$aChange,$bChngRowid) {
 $pTab = $pTab|0;
 $p = $p|0;
 $aChange = $aChange|0;
 $bChngRowid = $bChngRowid|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $i = 0, $iKey = 0, $or$cond = 0;
 var $pCol = 0, $zKey = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $0 = sp + 32|0;
 $1 = sp + 28|0;
 $2 = sp + 24|0;
 $3 = sp + 20|0;
 $4 = sp + 16|0;
 $i = sp + 12|0;
 $zKey = sp + 8|0;
 $iKey = sp + 4|0;
 $pCol = sp;
 HEAP32[$1>>2] = $pTab;
 HEAP32[$2>>2] = $p;
 HEAP32[$3>>2] = $aChange;
 HEAP32[$4>>2] = $bChngRowid;
 HEAP32[$i>>2] = 0;
 L1: while(1) {
  $5 = HEAP32[$i>>2]|0;
  $6 = HEAP32[$2>>2]|0;
  $7 = (($6) + 20|0);
  $8 = HEAP32[$7>>2]|0;
  $9 = ($5|0)<($8|0);
  if (!($9)) {
   label = 14;
   break;
  }
  $10 = HEAP32[$i>>2]|0;
  $11 = HEAP32[$2>>2]|0;
  $12 = (($11) + 36|0);
  $13 = (($12) + ($10<<3)|0);
  $14 = (($13) + 4|0);
  $15 = HEAP32[$14>>2]|0;
  HEAP32[$zKey>>2] = $15;
  HEAP32[$iKey>>2] = 0;
  while(1) {
   $16 = HEAP32[$iKey>>2]|0;
   $17 = HEAP32[$1>>2]|0;
   $18 = (($17) + 38|0);
   $19 = HEAP16[$18>>1]|0;
   $20 = $19 << 16 >> 16;
   $21 = ($16|0)<($20|0);
   if (!($21)) {
    break;
   }
   $22 = HEAP32[$iKey>>2]|0;
   $23 = HEAP32[$3>>2]|0;
   $24 = (($23) + ($22<<2)|0);
   $25 = HEAP32[$24>>2]|0;
   $26 = ($25|0)>=(0);
   if ($26) {
    label = 7;
   } else {
    $27 = HEAP32[$iKey>>2]|0;
    $28 = HEAP32[$1>>2]|0;
    $29 = (($28) + 36|0);
    $30 = HEAP16[$29>>1]|0;
    $31 = $30 << 16 >> 16;
    $32 = ($27|0)==($31|0);
    $33 = HEAP32[$4>>2]|0;
    $34 = ($33|0)!=(0);
    $or$cond = $32 & $34;
    if ($or$cond) {
     label = 7;
    }
   }
   do {
    if ((label|0) == 7) {
     label = 0;
     $35 = HEAP32[$iKey>>2]|0;
     $36 = HEAP32[$1>>2]|0;
     $37 = (($36) + 4|0);
     $38 = HEAP32[$37>>2]|0;
     $39 = (($38) + (($35*24)|0)|0);
     HEAP32[$pCol>>2] = $39;
     $40 = HEAP32[$zKey>>2]|0;
     $41 = ($40|0)!=(0|0);
     $42 = HEAP32[$pCol>>2]|0;
     if ($41) {
      $43 = HEAP32[$42>>2]|0;
      $44 = HEAP32[$zKey>>2]|0;
      $45 = (_sqlite3_stricmp($43,$44)|0);
      $46 = (0)==($45|0);
      if ($46) {
       label = 9;
       break L1;
      } else {
       break;
      }
     } else {
      $47 = (($42) + 23|0);
      $48 = HEAP8[$47>>0]|0;
      $49 = $48&255;
      $50 = $49 & 1;
      $51 = ($50|0)!=(0);
      if ($51) {
       label = 11;
       break L1;
      } else {
       break;
      }
     }
    }
   } while(0);
   $52 = HEAP32[$iKey>>2]|0;
   $53 = (($52) + 1)|0;
   HEAP32[$iKey>>2] = $53;
  }
  $54 = HEAP32[$i>>2]|0;
  $55 = (($54) + 1)|0;
  HEAP32[$i>>2] = $55;
 }
 if ((label|0) == 9) {
  HEAP32[$0>>2] = 1;
  $56 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($56|0);
 }
 else if ((label|0) == 11) {
  HEAP32[$0>>2] = 1;
  $56 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($56|0);
 }
 else if ((label|0) == 14) {
  HEAP32[$0>>2] = 0;
  $56 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($56|0);
 }
 return 0|0;
}
function _fkScanChildren($pParse,$pSrc,$pTab,$pIdx,$pFKey,$aiCol,$regData,$nIncr) {
 $pParse = $pParse|0;
 $pSrc = $pSrc|0;
 $pTab = $pTab|0;
 $pIdx = $pIdx|0;
 $pFKey = $pFKey|0;
 $aiCol = $aiCol|0;
 $regData = $regData|0;
 $nIncr = $nIncr|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0;
 var $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0;
 var $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0;
 var $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0;
 var $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0;
 var $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0;
 var $97 = 0, $98 = 0, $99 = 0, $db = 0, $i = 0, $iCol = 0, $iCol4 = 0, $iFkIfZero = 0, $or$cond = 0, $pAll = 0, $pEq = 0, $pEq3 = 0, $pLeft = 0, $pLeft1 = 0, $pNe = 0, $pPk = 0, $pRight = 0, $pRight2 = 0, $pWInfo = 0, $pWhere = 0;
 var $sNameContext = 0, $v = 0, $zCol = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 144|0;
 $0 = sp + 128|0;
 $1 = sp + 124|0;
 $2 = sp + 120|0;
 $3 = sp + 116|0;
 $4 = sp + 112|0;
 $5 = sp + 108|0;
 $6 = sp + 104|0;
 $7 = sp + 100|0;
 $db = sp + 96|0;
 $i = sp + 92|0;
 $pWhere = sp + 88|0;
 $sNameContext = sp + 56|0;
 $pWInfo = sp + 48|0;
 $iFkIfZero = sp + 44|0;
 $v = sp + 40|0;
 $pLeft = sp + 36|0;
 $pRight = sp + 32|0;
 $pEq = sp + 28|0;
 $iCol = sp + 134|0;
 $zCol = sp + 24|0;
 $pNe = sp + 20|0;
 $pLeft1 = sp + 16|0;
 $pRight2 = sp + 12|0;
 $pEq3 = sp + 8|0;
 $pAll = sp + 4|0;
 $pPk = sp;
 $iCol4 = sp + 132|0;
 HEAP32[$0>>2] = $pParse;
 HEAP32[$1>>2] = $pSrc;
 HEAP32[$2>>2] = $pTab;
 HEAP32[$3>>2] = $pIdx;
 HEAP32[$4>>2] = $pFKey;
 HEAP32[$5>>2] = $aiCol;
 HEAP32[$6>>2] = $regData;
 HEAP32[$7>>2] = $nIncr;
 $8 = HEAP32[$0>>2]|0;
 $9 = HEAP32[$8>>2]|0;
 HEAP32[$db>>2] = $9;
 HEAP32[$pWhere>>2] = 0;
 HEAP32[$iFkIfZero>>2] = 0;
 $10 = HEAP32[$0>>2]|0;
 $11 = (_sqlite3GetVdbe($10)|0);
 HEAP32[$v>>2] = $11;
 $12 = HEAP32[$7>>2]|0;
 $13 = ($12|0)<(0);
 if ($13) {
  $14 = HEAP32[$v>>2]|0;
  $15 = HEAP32[$4>>2]|0;
  $16 = (($15) + 24|0);
  $17 = HEAP8[$16>>0]|0;
  $18 = $17&255;
  $19 = (_sqlite3VdbeAddOp2($14,135,$18,0)|0);
  HEAP32[$iFkIfZero>>2] = $19;
 }
 HEAP32[$i>>2] = 0;
 while(1) {
  $20 = HEAP32[$i>>2]|0;
  $21 = HEAP32[$4>>2]|0;
  $22 = (($21) + 20|0);
  $23 = HEAP32[$22>>2]|0;
  $24 = ($20|0)<($23|0);
  if (!($24)) {
   break;
  }
  $25 = HEAP32[$3>>2]|0;
  $26 = ($25|0)!=(0|0);
  if ($26) {
   $27 = HEAP32[$i>>2]|0;
   $28 = HEAP32[$3>>2]|0;
   $29 = (($28) + 4|0);
   $30 = HEAP32[$29>>2]|0;
   $31 = (($30) + ($27<<1)|0);
   $32 = HEAP16[$31>>1]|0;
   $33 = $32 << 16 >> 16;
   $35 = $33;
  } else {
   $35 = -1;
  }
  $34 = $35&65535;
  HEAP16[$iCol>>1] = $34;
  $36 = HEAP32[$0>>2]|0;
  $37 = HEAP32[$2>>2]|0;
  $38 = HEAP32[$6>>2]|0;
  $39 = HEAP16[$iCol>>1]|0;
  $40 = (_exprTableRegister($36,$37,$38,$39)|0);
  HEAP32[$pLeft>>2] = $40;
  $41 = HEAP32[$5>>2]|0;
  $42 = ($41|0)!=(0|0);
  if ($42) {
   $43 = HEAP32[$i>>2]|0;
   $44 = HEAP32[$5>>2]|0;
   $45 = (($44) + ($43<<2)|0);
   $46 = HEAP32[$45>>2]|0;
   $51 = $46;
  } else {
   $47 = HEAP32[$4>>2]|0;
   $48 = (($47) + 36|0);
   $49 = HEAP32[$48>>2]|0;
   $51 = $49;
  }
  $50 = $51&65535;
  HEAP16[$iCol>>1] = $50;
  $52 = HEAP16[$iCol>>1]|0;
  $53 = $52 << 16 >> 16;
  $54 = HEAP32[$4>>2]|0;
  $55 = HEAP32[$54>>2]|0;
  $56 = (($55) + 4|0);
  $57 = HEAP32[$56>>2]|0;
  $58 = (($57) + (($53*24)|0)|0);
  $59 = HEAP32[$58>>2]|0;
  HEAP32[$zCol>>2] = $59;
  $60 = HEAP32[$db>>2]|0;
  $61 = HEAP32[$zCol>>2]|0;
  $62 = (_sqlite3Expr($60,27,$61)|0);
  HEAP32[$pRight>>2] = $62;
  $63 = HEAP32[$0>>2]|0;
  $64 = HEAP32[$pLeft>>2]|0;
  $65 = HEAP32[$pRight>>2]|0;
  $66 = (_sqlite3PExpr($63,79,$64,$65,0)|0);
  HEAP32[$pEq>>2] = $66;
  $67 = HEAP32[$db>>2]|0;
  $68 = HEAP32[$pWhere>>2]|0;
  $69 = HEAP32[$pEq>>2]|0;
  $70 = (_sqlite3ExprAnd($67,$68,$69)|0);
  HEAP32[$pWhere>>2] = $70;
  $71 = HEAP32[$i>>2]|0;
  $72 = (($71) + 1)|0;
  HEAP32[$i>>2] = $72;
 }
 $73 = HEAP32[$2>>2]|0;
 $74 = HEAP32[$4>>2]|0;
 $75 = HEAP32[$74>>2]|0;
 $76 = ($73|0)==($75|0);
 $77 = HEAP32[$7>>2]|0;
 $78 = ($77|0)>(0);
 $or$cond = $76 & $78;
 if ($or$cond) {
  $79 = HEAP32[$2>>2]|0;
  $80 = (($79) + 44|0);
  $81 = HEAP8[$80>>0]|0;
  $82 = $81&255;
  $83 = $82 & 32;
  $84 = ($83|0)==(0);
  if ($84) {
   $85 = HEAP32[$0>>2]|0;
   $86 = HEAP32[$2>>2]|0;
   $87 = HEAP32[$6>>2]|0;
   $88 = (_exprTableRegister($85,$86,$87,-1)|0);
   HEAP32[$pLeft1>>2] = $88;
   $89 = HEAP32[$db>>2]|0;
   $90 = HEAP32[$2>>2]|0;
   $91 = HEAP32[$1>>2]|0;
   $92 = (($91) + 8|0);
   $93 = (($92) + 40|0);
   $94 = HEAP32[$93>>2]|0;
   $95 = (_exprTableColumn($89,$90,$94,-1)|0);
   HEAP32[$pRight2>>2] = $95;
   $96 = HEAP32[$0>>2]|0;
   $97 = HEAP32[$pLeft1>>2]|0;
   $98 = HEAP32[$pRight2>>2]|0;
   $99 = (_sqlite3PExpr($96,78,$97,$98,0)|0);
   HEAP32[$pNe>>2] = $99;
  } else {
   HEAP32[$pAll>>2] = 0;
   $100 = HEAP32[$2>>2]|0;
   $101 = (_sqlite3PrimaryKeyIndex($100)|0);
   HEAP32[$pPk>>2] = $101;
   HEAP32[$i>>2] = 0;
   while(1) {
    $102 = HEAP32[$i>>2]|0;
    $103 = HEAP32[$pPk>>2]|0;
    $104 = (($103) + 50|0);
    $105 = HEAP16[$104>>1]|0;
    $106 = $105&65535;
    $107 = ($102|0)<($106|0);
    if (!($107)) {
     break;
    }
    $108 = HEAP32[$i>>2]|0;
    $109 = HEAP32[$3>>2]|0;
    $110 = (($109) + 4|0);
    $111 = HEAP32[$110>>2]|0;
    $112 = (($111) + ($108<<1)|0);
    $113 = HEAP16[$112>>1]|0;
    HEAP16[$iCol4>>1] = $113;
    $114 = HEAP32[$0>>2]|0;
    $115 = HEAP32[$2>>2]|0;
    $116 = HEAP32[$6>>2]|0;
    $117 = HEAP16[$iCol4>>1]|0;
    $118 = (_exprTableRegister($114,$115,$116,$117)|0);
    HEAP32[$pLeft1>>2] = $118;
    $119 = HEAP32[$db>>2]|0;
    $120 = HEAP32[$2>>2]|0;
    $121 = HEAP32[$1>>2]|0;
    $122 = (($121) + 8|0);
    $123 = (($122) + 40|0);
    $124 = HEAP32[$123>>2]|0;
    $125 = HEAP16[$iCol4>>1]|0;
    $126 = (_exprTableColumn($119,$120,$124,$125)|0);
    HEAP32[$pRight2>>2] = $126;
    $127 = HEAP32[$0>>2]|0;
    $128 = HEAP32[$pLeft1>>2]|0;
    $129 = HEAP32[$pRight2>>2]|0;
    $130 = (_sqlite3PExpr($127,79,$128,$129,0)|0);
    HEAP32[$pEq3>>2] = $130;
    $131 = HEAP32[$db>>2]|0;
    $132 = HEAP32[$pAll>>2]|0;
    $133 = HEAP32[$pEq3>>2]|0;
    $134 = (_sqlite3ExprAnd($131,$132,$133)|0);
    HEAP32[$pAll>>2] = $134;
    $135 = HEAP32[$i>>2]|0;
    $136 = (($135) + 1)|0;
    HEAP32[$i>>2] = $136;
   }
   $137 = HEAP32[$0>>2]|0;
   $138 = HEAP32[$pAll>>2]|0;
   $139 = (_sqlite3PExpr($137,19,$138,0,0)|0);
   HEAP32[$pNe>>2] = $139;
  }
  $140 = HEAP32[$db>>2]|0;
  $141 = HEAP32[$pWhere>>2]|0;
  $142 = HEAP32[$pNe>>2]|0;
  $143 = (_sqlite3ExprAnd($140,$141,$142)|0);
  HEAP32[$pWhere>>2] = $143;
 }
 ;HEAP32[$sNameContext+0>>2]=0|0;HEAP32[$sNameContext+4>>2]=0|0;HEAP32[$sNameContext+8>>2]=0|0;HEAP32[$sNameContext+12>>2]=0|0;HEAP32[$sNameContext+16>>2]=0|0;HEAP32[$sNameContext+20>>2]=0|0;HEAP32[$sNameContext+24>>2]=0|0;HEAP32[$sNameContext+28>>2]=0|0;
 $144 = HEAP32[$1>>2]|0;
 $145 = (($sNameContext) + 4|0);
 HEAP32[$145>>2] = $144;
 $146 = HEAP32[$0>>2]|0;
 HEAP32[$sNameContext>>2] = $146;
 $147 = HEAP32[$pWhere>>2]|0;
 (_sqlite3ResolveExprNames($sNameContext,$147)|0);
 $148 = HEAP32[$0>>2]|0;
 $149 = HEAP32[$1>>2]|0;
 $150 = HEAP32[$pWhere>>2]|0;
 $151 = (_sqlite3WhereBegin($148,$149,$150,0,0,0,0)|0);
 HEAP32[$pWInfo>>2] = $151;
 $152 = HEAP32[$7>>2]|0;
 $153 = ($152|0)>(0);
 if ($153) {
  $154 = HEAP32[$4>>2]|0;
  $155 = (($154) + 24|0);
  $156 = HEAP8[$155>>0]|0;
  $157 = $156&255;
  $158 = ($157|0)==(0);
  if ($158) {
   $159 = HEAP32[$0>>2]|0;
   $160 = (($159) + 408|0);
   $161 = HEAP32[$160>>2]|0;
   $162 = ($161|0)!=(0|0);
   $163 = HEAP32[$0>>2]|0;
   if ($162) {
    $164 = (($163) + 408|0);
    $165 = HEAP32[$164>>2]|0;
    $167 = $165;
   } else {
    $167 = $163;
   }
   $166 = (($167) + 21|0);
   HEAP8[$166>>0] = 1;
  }
 }
 $168 = HEAP32[$v>>2]|0;
 $169 = HEAP32[$4>>2]|0;
 $170 = (($169) + 24|0);
 $171 = HEAP8[$170>>0]|0;
 $172 = $171&255;
 $173 = HEAP32[$7>>2]|0;
 (_sqlite3VdbeAddOp2($168,134,$172,$173)|0);
 $174 = HEAP32[$pWInfo>>2]|0;
 $175 = ($174|0)!=(0|0);
 if ($175) {
  $176 = HEAP32[$pWInfo>>2]|0;
  _sqlite3WhereEnd($176);
 }
 $177 = HEAP32[$db>>2]|0;
 $178 = HEAP32[$pWhere>>2]|0;
 _sqlite3ExprDelete($177,$178);
 $179 = HEAP32[$iFkIfZero>>2]|0;
 $180 = ($179|0)!=(0);
 if (!($180)) {
  STACKTOP = sp;return;
 }
 $181 = HEAP32[$v>>2]|0;
 $182 = HEAP32[$iFkIfZero>>2]|0;
 _sqlite3VdbeJumpHere($181,$182);
 STACKTOP = sp;return;
}
function _exprTableRegister($pParse,$pTab,$regBase,$iCol) {
 $pParse = $pParse|0;
 $pTab = $pTab|0;
 $regBase = $regBase|0;
 $iCol = $iCol|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $db = 0, $pCol = 0, $pExpr = 0;
 var $zColl = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 24|0;
 $1 = sp + 20|0;
 $2 = sp + 16|0;
 $3 = sp + 28|0;
 $pExpr = sp + 12|0;
 $pCol = sp + 8|0;
 $zColl = sp + 4|0;
 $db = sp;
 HEAP32[$0>>2] = $pParse;
 HEAP32[$1>>2] = $pTab;
 HEAP32[$2>>2] = $regBase;
 HEAP16[$3>>1] = $iCol;
 $4 = HEAP32[$0>>2]|0;
 $5 = HEAP32[$4>>2]|0;
 HEAP32[$db>>2] = $5;
 $6 = HEAP32[$db>>2]|0;
 $7 = (_sqlite3Expr($6,159,0)|0);
 HEAP32[$pExpr>>2] = $7;
 $8 = HEAP32[$pExpr>>2]|0;
 $9 = ($8|0)!=(0|0);
 if (!($9)) {
  $56 = HEAP32[$pExpr>>2]|0;
  STACKTOP = sp;return ($56|0);
 }
 $10 = HEAP16[$3>>1]|0;
 $11 = $10 << 16 >> 16;
 $12 = ($11|0)>=(0);
 if ($12) {
  $13 = HEAP16[$3>>1]|0;
  $14 = $13 << 16 >> 16;
  $15 = HEAP32[$1>>2]|0;
  $16 = (($15) + 36|0);
  $17 = HEAP16[$16>>1]|0;
  $18 = $17 << 16 >> 16;
  $19 = ($14|0)!=($18|0);
  if ($19) {
   $20 = HEAP16[$3>>1]|0;
   $21 = $20 << 16 >> 16;
   $22 = HEAP32[$1>>2]|0;
   $23 = (($22) + 4|0);
   $24 = HEAP32[$23>>2]|0;
   $25 = (($24) + (($21*24)|0)|0);
   HEAP32[$pCol>>2] = $25;
   $26 = HEAP32[$2>>2]|0;
   $27 = HEAP16[$3>>1]|0;
   $28 = $27 << 16 >> 16;
   $29 = (($26) + ($28))|0;
   $30 = (($29) + 1)|0;
   $31 = HEAP32[$pExpr>>2]|0;
   $32 = (($31) + 28|0);
   HEAP32[$32>>2] = $30;
   $33 = HEAP32[$pCol>>2]|0;
   $34 = (($33) + 21|0);
   $35 = HEAP8[$34>>0]|0;
   $36 = HEAP32[$pExpr>>2]|0;
   $37 = (($36) + 1|0);
   HEAP8[$37>>0] = $35;
   $38 = HEAP32[$pCol>>2]|0;
   $39 = (($38) + 16|0);
   $40 = HEAP32[$39>>2]|0;
   HEAP32[$zColl>>2] = $40;
   $41 = HEAP32[$zColl>>2]|0;
   $42 = ($41|0)==(0|0);
   if ($42) {
    $43 = HEAP32[$db>>2]|0;
    $44 = (($43) + 8|0);
    $45 = HEAP32[$44>>2]|0;
    $46 = HEAP32[$45>>2]|0;
    HEAP32[$zColl>>2] = $46;
   }
   $47 = HEAP32[$0>>2]|0;
   $48 = HEAP32[$pExpr>>2]|0;
   $49 = HEAP32[$zColl>>2]|0;
   $50 = (_sqlite3ExprAddCollateString($47,$48,$49)|0);
   HEAP32[$pExpr>>2] = $50;
   $56 = HEAP32[$pExpr>>2]|0;
   STACKTOP = sp;return ($56|0);
  }
 }
 $51 = HEAP32[$2>>2]|0;
 $52 = HEAP32[$pExpr>>2]|0;
 $53 = (($52) + 28|0);
 HEAP32[$53>>2] = $51;
 $54 = HEAP32[$pExpr>>2]|0;
 $55 = (($54) + 1|0);
 HEAP8[$55>>0] = 68;
 $56 = HEAP32[$pExpr>>2]|0;
 STACKTOP = sp;return ($56|0);
}
function _exprTableColumn($db,$pTab,$iCursor,$iCol) {
 $db = $db|0;
 $pTab = $pTab|0;
 $iCursor = $iCursor|0;
 $iCol = $iCol|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $pExpr = 0, label = 0;
 var sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 12|0;
 $1 = sp + 8|0;
 $2 = sp + 4|0;
 $3 = sp + 16|0;
 $pExpr = sp;
 HEAP32[$0>>2] = $db;
 HEAP32[$1>>2] = $pTab;
 HEAP32[$2>>2] = $iCursor;
 HEAP16[$3>>1] = $iCol;
 $4 = HEAP32[$0>>2]|0;
 $5 = (_sqlite3Expr($4,154,0)|0);
 HEAP32[$pExpr>>2] = $5;
 $6 = HEAP32[$pExpr>>2]|0;
 $7 = ($6|0)!=(0|0);
 if (!($7)) {
  $17 = HEAP32[$pExpr>>2]|0;
  STACKTOP = sp;return ($17|0);
 }
 $8 = HEAP32[$1>>2]|0;
 $9 = HEAP32[$pExpr>>2]|0;
 $10 = (($9) + 44|0);
 HEAP32[$10>>2] = $8;
 $11 = HEAP32[$2>>2]|0;
 $12 = HEAP32[$pExpr>>2]|0;
 $13 = (($12) + 28|0);
 HEAP32[$13>>2] = $11;
 $14 = HEAP16[$3>>1]|0;
 $15 = HEAP32[$pExpr>>2]|0;
 $16 = (($15) + 32|0);
 HEAP16[$16>>1] = $14;
 $17 = HEAP32[$pExpr>>2]|0;
 STACKTOP = sp;return ($17|0);
}
function _sqlite3WhereBegin($pParse,$pTabList,$pWhere,$pOrderBy,$pResultSet,$wctrlFlags,$iIdxCur) {
 $pParse = $pParse|0;
 $pTabList = $pTabList|0;
 $pWhere = $pWhere|0;
 $pOrderBy = $pOrderBy|0;
 $pResultSet = $pResultSet|0;
 $wctrlFlags = $wctrlFlags|0;
 $iIdxCur = $iIdxCur|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0;
 var $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0, $188 = 0;
 var $189 = 0, $19 = 0, $190 = 0, $191 = 0, $192 = 0, $193 = 0, $194 = 0, $195 = 0, $196 = 0, $197 = 0, $198 = 0, $199 = 0, $2 = 0, $20 = 0, $200 = 0, $201 = 0, $202 = 0, $203 = 0, $204 = 0, $205 = 0;
 var $206 = 0, $207 = 0, $208 = 0, $209 = 0, $21 = 0, $210 = 0, $211 = 0, $212 = 0, $213 = 0, $214 = 0, $215 = 0, $216 = 0, $217 = 0, $218 = 0, $219 = 0, $22 = 0, $220 = 0, $221 = 0, $222 = 0, $223 = 0;
 var $224 = 0, $225 = 0, $226 = 0, $227 = 0, $228 = 0, $229 = 0, $23 = 0, $230 = 0, $231 = 0, $232 = 0, $233 = 0, $234 = 0, $235 = 0, $236 = 0, $237 = 0, $238 = 0, $239 = 0, $24 = 0, $240 = 0, $241 = 0;
 var $242 = 0, $243 = 0, $244 = 0, $245 = 0, $246 = 0, $247 = 0, $248 = 0, $249 = 0, $25 = 0, $250 = 0, $251 = 0, $252 = 0, $253 = 0, $254 = 0, $255 = 0, $256 = 0, $257 = 0, $258 = 0, $259 = 0, $26 = 0;
 var $260 = 0, $261 = 0, $262 = 0, $263 = 0, $264 = 0, $265 = 0, $266 = 0, $267 = 0, $268 = 0, $269 = 0, $27 = 0, $270 = 0, $271 = 0, $272 = 0, $273 = 0, $274 = 0, $275 = 0, $276 = 0, $277 = 0, $278 = 0;
 var $279 = 0, $28 = 0, $280 = 0, $281 = 0, $282 = 0, $283 = 0, $284 = 0, $285 = 0, $286 = 0, $287 = 0, $288 = 0, $289 = 0, $29 = 0, $290 = 0, $291 = 0, $292 = 0, $293 = 0, $294 = 0, $295 = 0, $296 = 0;
 var $297 = 0, $298 = 0, $299 = 0, $3 = 0, $30 = 0, $300 = 0, $301 = 0, $302 = 0, $303 = 0, $304 = 0, $305 = 0, $306 = 0, $307 = 0, $308 = 0, $309 = 0, $31 = 0, $310 = 0, $311 = 0, $312 = 0, $313 = 0;
 var $314 = 0, $315 = 0, $316 = 0, $317 = 0, $318 = 0, $319 = 0, $32 = 0, $320 = 0, $321 = 0, $322 = 0, $323 = 0, $324 = 0, $325 = 0, $326 = 0, $327 = 0, $328 = 0, $329 = 0, $33 = 0, $330 = 0, $331 = 0;
 var $332 = 0, $333 = 0, $334 = 0, $335 = 0, $336 = 0, $337 = 0, $338 = 0, $339 = 0, $34 = 0, $340 = 0, $341 = 0, $342 = 0, $343 = 0, $344 = 0, $345 = 0, $346 = 0, $347 = 0, $348 = 0, $349 = 0, $35 = 0;
 var $350 = 0, $351 = 0, $352 = 0, $353 = 0, $354 = 0, $355 = 0, $356 = 0, $357 = 0, $358 = 0, $359 = 0, $36 = 0, $360 = 0, $361 = 0, $362 = 0, $363 = 0, $364 = 0, $365 = 0, $366 = 0, $367 = 0, $368 = 0;
 var $369 = 0, $37 = 0, $370 = 0, $371 = 0, $372 = 0, $373 = 0, $374 = 0, $375 = 0, $376 = 0, $377 = 0, $378 = 0, $379 = 0, $38 = 0, $380 = 0, $381 = 0, $382 = 0, $383 = 0, $384 = 0, $385 = 0, $386 = 0;
 var $387 = 0, $388 = 0, $389 = 0, $39 = 0, $390 = 0, $391 = 0, $392 = 0, $393 = 0, $394 = 0, $395 = 0, $396 = 0, $397 = 0, $398 = 0, $399 = 0, $4 = 0, $40 = 0, $400 = 0, $401 = 0, $402 = 0, $403 = 0;
 var $404 = 0, $405 = 0, $406 = 0, $407 = 0, $408 = 0, $409 = 0, $41 = 0, $410 = 0, $411 = 0, $412 = 0, $413 = 0, $414 = 0, $415 = 0, $416 = 0, $417 = 0, $418 = 0, $419 = 0, $42 = 0, $420 = 0, $421 = 0;
 var $422 = 0, $423 = 0, $424 = 0, $425 = 0, $426 = 0, $427 = 0, $428 = 0, $429 = 0, $43 = 0, $430 = 0, $431 = 0, $432 = 0, $433 = 0, $434 = 0, $435 = 0, $436 = 0, $437 = 0, $438 = 0, $439 = 0, $44 = 0;
 var $440 = 0, $441 = 0, $442 = 0, $443 = 0, $444 = 0, $445 = 0, $446 = 0, $447 = 0, $448 = 0, $449 = 0, $45 = 0, $450 = 0, $451 = 0, $452 = 0, $453 = 0, $454 = 0, $455 = 0, $456 = 0, $457 = 0, $458 = 0;
 var $459 = 0, $46 = 0, $460 = 0, $461 = 0, $462 = 0, $463 = 0, $464 = 0, $465 = 0, $466 = 0, $467 = 0, $468 = 0, $469 = 0, $47 = 0, $470 = 0, $471 = 0, $472 = 0, $473 = 0, $474 = 0, $475 = 0, $476 = 0;
 var $477 = 0, $478 = 0, $479 = 0, $48 = 0, $480 = 0, $481 = 0, $482 = 0, $483 = 0, $484 = 0, $485 = 0, $486 = 0, $487 = 0, $488 = 0, $489 = 0, $49 = 0, $490 = 0, $491 = 0, $492 = 0, $493 = 0, $494 = 0;
 var $495 = 0, $496 = 0, $497 = 0, $498 = 0, $499 = 0, $5 = 0, $50 = 0, $500 = 0, $501 = 0, $502 = 0, $503 = 0, $504 = 0, $505 = 0, $506 = 0, $507 = 0, $508 = 0, $509 = 0, $51 = 0, $510 = 0, $511 = 0;
 var $512 = 0, $513 = 0, $514 = 0, $515 = 0, $516 = 0, $517 = 0, $518 = 0, $519 = 0, $52 = 0, $520 = 0, $521 = 0, $522 = 0, $523 = 0, $524 = 0, $525 = 0, $526 = 0, $527 = 0, $528 = 0, $529 = 0, $53 = 0;
 var $530 = 0, $531 = 0, $532 = 0, $533 = 0, $534 = 0, $535 = 0, $536 = 0, $537 = 0, $538 = 0, $539 = 0, $54 = 0, $540 = 0, $541 = 0, $542 = 0, $543 = 0, $544 = 0, $545 = 0, $546 = 0, $547 = 0, $548 = 0;
 var $549 = 0, $55 = 0, $550 = 0, $551 = 0, $552 = 0, $553 = 0, $554 = 0, $555 = 0, $556 = 0, $557 = 0, $558 = 0, $559 = 0, $56 = 0, $560 = 0, $561 = 0, $562 = 0, $563 = 0, $564 = 0, $565 = 0, $566 = 0;
 var $567 = 0, $568 = 0, $569 = 0, $57 = 0, $570 = 0, $571 = 0, $572 = 0, $573 = 0, $574 = 0, $575 = 0, $576 = 0, $577 = 0, $578 = 0, $579 = 0, $58 = 0, $580 = 0, $581 = 0, $582 = 0, $583 = 0, $584 = 0;
 var $585 = 0, $586 = 0, $587 = 0, $588 = 0, $589 = 0, $59 = 0, $590 = 0, $591 = 0, $592 = 0, $593 = 0, $594 = 0, $595 = 0, $596 = 0, $597 = 0, $598 = 0, $599 = 0, $6 = 0, $60 = 0, $600 = 0, $601 = 0;
 var $602 = 0, $603 = 0, $604 = 0, $605 = 0, $606 = 0, $607 = 0, $608 = 0, $609 = 0, $61 = 0, $610 = 0, $611 = 0, $612 = 0, $613 = 0, $614 = 0, $615 = 0, $616 = 0, $617 = 0, $618 = 0, $619 = 0, $62 = 0;
 var $620 = 0, $621 = 0, $622 = 0, $623 = 0, $624 = 0, $625 = 0, $626 = 0, $627 = 0, $628 = 0, $629 = 0, $63 = 0, $630 = 0, $631 = 0, $632 = 0, $633 = 0, $634 = 0, $635 = 0, $636 = 0, $637 = 0, $638 = 0;
 var $639 = 0, $64 = 0, $640 = 0, $641 = 0, $642 = 0, $643 = 0, $644 = 0, $645 = 0, $646 = 0, $647 = 0, $648 = 0, $649 = 0, $65 = 0, $650 = 0, $651 = 0, $652 = 0, $653 = 0, $654 = 0, $655 = 0, $656 = 0;
 var $657 = 0, $658 = 0, $659 = 0, $66 = 0, $660 = 0, $661 = 0, $662 = 0, $663 = 0, $664 = 0, $665 = 0, $666 = 0, $667 = 0, $668 = 0, $669 = 0, $67 = 0, $670 = 0, $671 = 0, $672 = 0, $673 = 0, $674 = 0;
 var $675 = 0, $676 = 0, $677 = 0, $678 = 0, $679 = 0, $68 = 0, $680 = 0, $681 = 0, $682 = 0, $683 = 0, $684 = 0, $685 = 0, $686 = 0, $687 = 0, $688 = 0, $689 = 0, $69 = 0, $690 = 0, $691 = 0, $692 = 0;
 var $693 = 0, $694 = 0, $695 = 0, $696 = 0, $697 = 0, $698 = 0, $699 = 0, $7 = 0, $70 = 0, $700 = 0, $701 = 0, $702 = 0, $703 = 0, $704 = 0, $705 = 0, $706 = 0, $707 = 0, $708 = 0, $709 = 0, $71 = 0;
 var $710 = 0, $711 = 0, $712 = 0, $713 = 0, $714 = 0, $715 = 0, $716 = 0, $717 = 0, $718 = 0, $719 = 0, $72 = 0, $720 = 0, $721 = 0, $722 = 0, $723 = 0, $724 = 0, $725 = 0, $726 = 0, $727 = 0, $728 = 0;
 var $729 = 0, $73 = 0, $730 = 0, $731 = 0, $732 = 0, $733 = 0, $734 = 0, $735 = 0, $736 = 0, $737 = 0, $738 = 0, $739 = 0, $74 = 0, $740 = 0, $741 = 0, $742 = 0, $743 = 0, $744 = 0, $745 = 0, $746 = 0;
 var $747 = 0, $748 = 0, $749 = 0, $75 = 0, $750 = 0, $751 = 0, $752 = 0, $753 = 0, $754 = 0, $755 = 0, $756 = 0, $757 = 0, $758 = 0, $759 = 0, $76 = 0, $760 = 0, $761 = 0, $762 = 0, $763 = 0, $764 = 0;
 var $765 = 0, $766 = 0, $767 = 0, $768 = 0, $769 = 0, $77 = 0, $770 = 0, $771 = 0, $772 = 0, $773 = 0, $774 = 0, $775 = 0, $776 = 0, $777 = 0, $778 = 0, $779 = 0, $78 = 0, $780 = 0, $781 = 0, $782 = 0;
 var $783 = 0, $784 = 0, $785 = 0, $786 = 0, $787 = 0, $788 = 0, $789 = 0, $79 = 0, $790 = 0, $791 = 0, $792 = 0, $793 = 0, $794 = 0, $795 = 0, $796 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0;
 var $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $b = 0, $db = 0, $iCur = 0;
 var $iDb = 0, $iIndexCur = 0, $ii = 0, $n = 0, $nByteWInfo = 0, $nTabList = 0, $notReady = 0, $op = 0, $op1 = 0, $or$cond = 0, $pEnd = 0, $pIx = 0, $pJ = 0, $pLevel = 0, $pLoop = 0, $pMaskSet = 0, $pTab = 0, $pTabItem = 0, $pTerm = 0, $pVTab = 0;
 var $pWInfo = 0, $rc = 0, $sWLB = 0, $tabUsed = 0, $v = 0, $vararg_buffer = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 176|0;
 $vararg_buffer = sp + 24|0;
 $0 = sp + 164|0;
 $1 = sp + 160|0;
 $2 = sp + 156|0;
 $3 = sp + 152|0;
 $4 = sp + 148|0;
 $5 = sp + 144|0;
 $6 = sp + 168|0;
 $7 = sp + 140|0;
 $nByteWInfo = sp + 136|0;
 $nTabList = sp + 132|0;
 $pWInfo = sp + 128|0;
 $v = sp + 124|0;
 $notReady = sp + 16|0;
 $sWLB = sp + 104|0;
 $pMaskSet = sp + 100|0;
 $pLevel = sp + 96|0;
 $pLoop = sp + 92|0;
 $ii = sp + 88|0;
 $db = sp + 84|0;
 $rc = sp + 80|0;
 $tabUsed = sp + 8|0;
 $pTerm = sp + 76|0;
 $pEnd = sp + 72|0;
 $pTab = sp + 68|0;
 $iDb = sp + 64|0;
 $pTabItem = sp + 60|0;
 $pVTab = sp + 56|0;
 $iCur = sp + 52|0;
 $op = sp + 48|0;
 $b = sp;
 $n = sp + 44|0;
 $pIx = sp + 40|0;
 $iIndexCur = sp + 36|0;
 $op1 = sp + 32|0;
 $pJ = sp + 28|0;
 HEAP32[$1>>2] = $pParse;
 HEAP32[$2>>2] = $pTabList;
 HEAP32[$3>>2] = $pWhere;
 HEAP32[$4>>2] = $pOrderBy;
 HEAP32[$5>>2] = $pResultSet;
 HEAP16[$6>>1] = $wctrlFlags;
 HEAP32[$7>>2] = $iIdxCur;
 $8 = HEAP32[$1>>2]|0;
 $9 = (($8) + 8|0);
 $10 = HEAP32[$9>>2]|0;
 HEAP32[$v>>2] = $10;
 $11 = HEAP32[$1>>2]|0;
 $12 = HEAP32[$11>>2]|0;
 HEAP32[$db>>2] = $12;
 ;HEAP32[$sWLB+0>>2]=0|0;HEAP32[$sWLB+4>>2]=0|0;HEAP32[$sWLB+8>>2]=0|0;HEAP32[$sWLB+12>>2]=0|0;HEAP32[$sWLB+16>>2]=0|0;
 $13 = HEAP32[$4>>2]|0;
 $14 = ($13|0)!=(0|0);
 if ($14) {
  $15 = HEAP32[$4>>2]|0;
  $16 = HEAP32[$15>>2]|0;
  $17 = ($16|0)>=(64);
  if ($17) {
   HEAP32[$4>>2] = 0;
  }
 }
 $18 = HEAP32[$4>>2]|0;
 $19 = (($sWLB) + 8|0);
 HEAP32[$19>>2] = $18;
 $20 = HEAP32[$db>>2]|0;
 $21 = (($20) + 60|0);
 $22 = HEAP16[$21>>1]|0;
 $23 = $22&65535;
 $24 = $23 & 32;
 $25 = ($24|0)!=(0);
 if ($25) {
  $26 = HEAP16[$6>>1]|0;
  $27 = $26&65535;
  $28 = $27 & -1025;
  $29 = $28&65535;
  HEAP16[$6>>1] = $29;
 }
 $30 = HEAP32[$2>>2]|0;
 $31 = HEAP32[$30>>2]|0;
 $32 = ($31|0)>(64);
 if ($32) {
  $33 = HEAP32[$1>>2]|0;
  HEAP32[$vararg_buffer>>2] = 64;
  _sqlite3ErrorMsg($33,23568,$vararg_buffer);
  HEAP32[$0>>2] = 0;
  $796 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($796|0);
 }
 $34 = HEAP16[$6>>1]|0;
 $35 = $34&65535;
 $36 = $35 & 64;
 $37 = ($36|0)!=(0);
 if ($37) {
  $40 = 1;
 } else {
  $38 = HEAP32[$2>>2]|0;
  $39 = HEAP32[$38>>2]|0;
  $40 = $39;
 }
 HEAP32[$nTabList>>2] = $40;
 $41 = HEAP32[$nTabList>>2]|0;
 $42 = (($41) - 1)|0;
 $43 = ($42*72)|0;
 $44 = (808 + ($43))|0;
 $45 = (($44) + 7)|0;
 $46 = $45 & -8;
 HEAP32[$nByteWInfo>>2] = $46;
 $47 = HEAP32[$db>>2]|0;
 $48 = HEAP32[$nByteWInfo>>2]|0;
 $49 = (($48) + 72)|0;
 $50 = (_sqlite3DbMallocZero($47,$49,0)|0);
 HEAP32[$pWInfo>>2] = $50;
 $51 = HEAP32[$db>>2]|0;
 $52 = (($51) + 64|0);
 $53 = HEAP8[$52>>0]|0;
 $54 = ($53<<24>>24)!=(0);
 L15: do {
  if ($54) {
   $55 = HEAP32[$db>>2]|0;
   $56 = HEAP32[$pWInfo>>2]|0;
   _sqlite3DbFree($55,$56);
   HEAP32[$pWInfo>>2] = 0;
  } else {
   $57 = HEAP32[$pWInfo>>2]|0;
   $58 = (($57) + 60|0);
   $59 = (($58) + 4|0);
   HEAP32[$59>>2] = -1;
   $60 = HEAP32[$pWInfo>>2]|0;
   $61 = (($60) + 60|0);
   HEAP32[$61>>2] = -1;
   $62 = HEAP32[$nTabList>>2]|0;
   $63 = $62&255;
   $64 = HEAP32[$pWInfo>>2]|0;
   $65 = (($64) + 41|0);
   HEAP8[$65>>0] = $63;
   $66 = HEAP32[$1>>2]|0;
   $67 = HEAP32[$pWInfo>>2]|0;
   HEAP32[$67>>2] = $66;
   $68 = HEAP32[$2>>2]|0;
   $69 = HEAP32[$pWInfo>>2]|0;
   $70 = (($69) + 4|0);
   HEAP32[$70>>2] = $68;
   $71 = HEAP32[$4>>2]|0;
   $72 = HEAP32[$pWInfo>>2]|0;
   $73 = (($72) + 8|0);
   HEAP32[$73>>2] = $71;
   $74 = HEAP32[$5>>2]|0;
   $75 = HEAP32[$pWInfo>>2]|0;
   $76 = (($75) + 12|0);
   HEAP32[$76>>2] = $74;
   $77 = HEAP32[$v>>2]|0;
   $78 = (_sqlite3VdbeMakeLabel($77)|0);
   $79 = HEAP32[$pWInfo>>2]|0;
   $80 = (($79) + 48|0);
   HEAP32[$80>>2] = $78;
   $81 = HEAP32[$pWInfo>>2]|0;
   $82 = (($81) + 52|0);
   HEAP32[$82>>2] = $78;
   $83 = HEAP16[$6>>1]|0;
   $84 = HEAP32[$pWInfo>>2]|0;
   $85 = (($84) + 34|0);
   HEAP16[$85>>1] = $83;
   $86 = HEAP32[$1>>2]|0;
   $87 = (($86) + 424|0);
   $88 = HEAP32[$87>>2]|0;
   $89 = HEAP32[$pWInfo>>2]|0;
   $90 = (($89) + 56|0);
   HEAP32[$90>>2] = $88;
   $91 = HEAP32[$pWInfo>>2]|0;
   $92 = (($91) + 68|0);
   HEAP32[$pMaskSet>>2] = $92;
   $93 = HEAP32[$pWInfo>>2]|0;
   HEAP32[$sWLB>>2] = $93;
   $94 = HEAP32[$pWInfo>>2]|0;
   $95 = (($94) + 328|0);
   $96 = (($sWLB) + 4|0);
   HEAP32[$96>>2] = $95;
   $97 = HEAP32[$pWInfo>>2]|0;
   $98 = HEAP32[$nByteWInfo>>2]|0;
   $99 = (($97) + ($98)|0);
   $100 = (($sWLB) + 12|0);
   HEAP32[$100>>2] = $99;
   $101 = (($sWLB) + 12|0);
   $102 = HEAP32[$101>>2]|0;
   _whereLoopInit($102);
   $103 = HEAP32[$pMaskSet>>2]|0;
   HEAP32[$103>>2] = 0;
   $104 = HEAP32[$pWInfo>>2]|0;
   $105 = (($104) + 328|0);
   $106 = HEAP32[$pWInfo>>2]|0;
   _whereClauseInit($105,$106);
   $107 = HEAP32[$pWInfo>>2]|0;
   $108 = (($107) + 328|0);
   $109 = HEAP32[$3>>2]|0;
   _whereSplit($108,$109,72);
   HEAP32[$ii>>2] = 0;
   while(1) {
    $110 = HEAP32[$ii>>2]|0;
    $111 = (($sWLB) + 4|0);
    $112 = HEAP32[$111>>2]|0;
    $113 = (($112) + 12|0);
    $114 = HEAP32[$113>>2]|0;
    $115 = ($110|0)<($114|0);
    $116 = HEAP32[$nTabList>>2]|0;
    $117 = ($116|0)==(0);
    if (!($115)) {
     break;
    }
    if ($117) {
     label = 16;
    } else {
     $118 = HEAP32[$ii>>2]|0;
     $119 = (($sWLB) + 4|0);
     $120 = HEAP32[$119>>2]|0;
     $121 = (($120) + 20|0);
     $122 = HEAP32[$121>>2]|0;
     $123 = (($122) + (($118*48)|0)|0);
     $124 = HEAP32[$123>>2]|0;
     $125 = (_sqlite3ExprIsConstantNotJoin($124)|0);
     $126 = ($125|0)!=(0);
     if ($126) {
      label = 16;
     }
    }
    if ((label|0) == 16) {
     label = 0;
     $127 = HEAP32[$1>>2]|0;
     $128 = HEAP32[$ii>>2]|0;
     $129 = (($sWLB) + 4|0);
     $130 = HEAP32[$129>>2]|0;
     $131 = (($130) + 20|0);
     $132 = HEAP32[$131>>2]|0;
     $133 = (($132) + (($128*48)|0)|0);
     $134 = HEAP32[$133>>2]|0;
     $135 = HEAP32[$pWInfo>>2]|0;
     $136 = (($135) + 52|0);
     $137 = HEAP32[$136>>2]|0;
     _sqlite3ExprIfFalse($127,$134,$137,16);
     $138 = HEAP32[$ii>>2]|0;
     $139 = (($sWLB) + 4|0);
     $140 = HEAP32[$139>>2]|0;
     $141 = (($140) + 20|0);
     $142 = HEAP32[$141>>2]|0;
     $143 = (($142) + (($138*48)|0)|0);
     $144 = (($143) + 20|0);
     $145 = HEAP8[$144>>0]|0;
     $146 = $145&255;
     $147 = $146 | 4;
     $148 = $147&255;
     HEAP8[$144>>0] = $148;
    }
    $149 = HEAP32[$ii>>2]|0;
    $150 = (($149) + 1)|0;
    HEAP32[$ii>>2] = $150;
   }
   if ($117) {
    $151 = HEAP32[$4>>2]|0;
    $152 = ($151|0)!=(0|0);
    if ($152) {
     $153 = HEAP32[$4>>2]|0;
     $154 = HEAP32[$153>>2]|0;
     $155 = $154&255;
     $156 = HEAP32[$pWInfo>>2]|0;
     $157 = (($156) + 36|0);
     HEAP8[$157>>0] = $155;
    }
    $158 = HEAP16[$6>>1]|0;
    $159 = $158&65535;
    $160 = $159 & 1024;
    $161 = ($160|0)!=(0);
    if ($161) {
     $162 = HEAP32[$pWInfo>>2]|0;
     $163 = (($162) + 40|0);
     HEAP8[$163>>0] = 1;
    }
   }
   HEAP32[$ii>>2] = 0;
   while(1) {
    $164 = HEAP32[$ii>>2]|0;
    $165 = HEAP32[$2>>2]|0;
    $166 = HEAP32[$165>>2]|0;
    $167 = ($164|0)<($166|0);
    if (!($167)) {
     break;
    }
    $168 = HEAP32[$pMaskSet>>2]|0;
    $169 = HEAP32[$ii>>2]|0;
    $170 = HEAP32[$2>>2]|0;
    $171 = (($170) + 8|0);
    $172 = (($171) + (($169*72)|0)|0);
    $173 = (($172) + 40|0);
    $174 = HEAP32[$173>>2]|0;
    _createMask($168,$174);
    $175 = HEAP32[$ii>>2]|0;
    $176 = (($175) + 1)|0;
    HEAP32[$ii>>2] = $176;
   }
   $177 = HEAP32[$2>>2]|0;
   $178 = HEAP32[$pWInfo>>2]|0;
   $179 = (($178) + 328|0);
   _exprAnalyzeAll($177,$179);
   $180 = HEAP32[$db>>2]|0;
   $181 = (($180) + 64|0);
   $182 = HEAP8[$181>>0]|0;
   $183 = ($182<<24>>24)!=(0);
   if (!($183)) {
    $184 = HEAP16[$6>>1]|0;
    $185 = $184&65535;
    $186 = $185 & 1024;
    $187 = ($186|0)!=(0);
    do {
     if ($187) {
      $188 = HEAP32[$1>>2]|0;
      $189 = HEAP32[$2>>2]|0;
      $190 = HEAP32[$pWInfo>>2]|0;
      $191 = (($190) + 328|0);
      $192 = HEAP32[$5>>2]|0;
      $193 = (_isDistinctRedundant($188,$189,$191,$192)|0);
      $194 = ($193|0)!=(0);
      if ($194) {
       $195 = HEAP32[$pWInfo>>2]|0;
       $196 = (($195) + 40|0);
       HEAP8[$196>>0] = 1;
       break;
      }
      $197 = HEAP32[$4>>2]|0;
      $198 = ($197|0)==(0|0);
      if ($198) {
       $199 = HEAP32[$pWInfo>>2]|0;
       $200 = (($199) + 34|0);
       $201 = HEAP16[$200>>1]|0;
       $202 = $201&65535;
       $203 = $202 | 512;
       $204 = $203&65535;
       HEAP16[$200>>1] = $204;
       $205 = HEAP32[$5>>2]|0;
       $206 = HEAP32[$pWInfo>>2]|0;
       $207 = (($206) + 8|0);
       HEAP32[$207>>2] = $205;
      }
     }
    } while(0);
    $208 = HEAP32[$nTabList>>2]|0;
    $209 = ($208|0)!=(1);
    if ($209) {
     label = 34;
    } else {
     $210 = (_whereShortCut($sWLB)|0);
     $211 = ($210|0)==(0);
     if ($211) {
      label = 34;
     }
    }
    if ((label|0) == 34) {
     $212 = (_whereLoopAddAll($sWLB)|0);
     HEAP32[$rc>>2] = $212;
     $213 = HEAP32[$rc>>2]|0;
     $214 = ($213|0)!=(0);
     if ($214) {
      break;
     }
     $215 = HEAP32[$pWInfo>>2]|0;
     (_wherePathSolver($215,0)|0);
     $216 = HEAP32[$db>>2]|0;
     $217 = (($216) + 64|0);
     $218 = HEAP8[$217>>0]|0;
     $219 = ($218<<24>>24)!=(0);
     if ($219) {
      break;
     }
     $220 = HEAP32[$pWInfo>>2]|0;
     $221 = (($220) + 8|0);
     $222 = HEAP32[$221>>2]|0;
     $223 = ($222|0)!=(0|0);
     if ($223) {
      $224 = HEAP32[$pWInfo>>2]|0;
      $225 = HEAP32[$pWInfo>>2]|0;
      $226 = (($225) + 32|0);
      $227 = HEAP16[$226>>1]|0;
      $228 = $227 << 16 >> 16;
      $229 = (($228) + 1)|0;
      $230 = $229&65535;
      (_wherePathSolver($224,$230)|0);
      $231 = HEAP32[$db>>2]|0;
      $232 = (($231) + 64|0);
      $233 = HEAP8[$232>>0]|0;
      $234 = ($233<<24>>24)!=(0);
      if ($234) {
       break;
      }
     }
    }
    $235 = HEAP32[$pWInfo>>2]|0;
    $236 = (($235) + 8|0);
    $237 = HEAP32[$236>>2]|0;
    $238 = ($237|0)==(0|0);
    if ($238) {
     $239 = HEAP32[$db>>2]|0;
     $240 = (($239) + 24|0);
     $241 = HEAP32[$240>>2]|0;
     $242 = $241 & 131072;
     $243 = ($242|0)!=(0);
     if ($243) {
      $244 = HEAP32[$pWInfo>>2]|0;
      $245 = (($244) + 24|0);
      $246 = $245;
      $247 = $246;
      HEAP32[$247>>2] = -1;
      $248 = (($246) + 4)|0;
      $249 = $248;
      HEAP32[$249>>2] = -1;
     }
    }
    $250 = HEAP32[$1>>2]|0;
    $251 = (($250) + 64|0);
    $252 = HEAP32[$251>>2]|0;
    $253 = ($252|0)!=(0);
    if (!($253)) {
     $254 = HEAP32[$db>>2]|0;
     $255 = (($254) + 64|0);
     $256 = HEAP8[$255>>0]|0;
     $257 = $256&255;
     $258 = ($257|0)!=(0);
     if (!($258)) {
      $259 = HEAP32[$pWInfo>>2]|0;
      $260 = (($259) + 41|0);
      $261 = HEAP8[$260>>0]|0;
      $262 = $261&255;
      $263 = ($262|0)>=(2);
      $264 = HEAP32[$5>>2]|0;
      $265 = ($264|0)!=(0|0);
      $or$cond = $263 & $265;
      L60: do {
       if ($or$cond) {
        $266 = HEAP32[$db>>2]|0;
        $267 = (($266) + 60|0);
        $268 = HEAP16[$267>>1]|0;
        $269 = $268&65535;
        $270 = $269 & 1024;
        $271 = ($270|0)==(0);
        if ($271) {
         $272 = HEAP32[$pMaskSet>>2]|0;
         $273 = HEAP32[$5>>2]|0;
         $274 = (_exprListTableUsage($272,$273)|0);
         $275 = tempRet0;
         $276 = $tabUsed;
         $277 = $276;
         HEAP32[$277>>2] = $274;
         $278 = (($276) + 4)|0;
         $279 = $278;
         HEAP32[$279>>2] = $275;
         $280 = (($sWLB) + 8|0);
         $281 = HEAP32[$280>>2]|0;
         $282 = ($281|0)!=(0|0);
         if ($282) {
          $283 = HEAP32[$pMaskSet>>2]|0;
          $284 = (($sWLB) + 8|0);
          $285 = HEAP32[$284>>2]|0;
          $286 = (_exprListTableUsage($283,$285)|0);
          $287 = tempRet0;
          $288 = $tabUsed;
          $289 = $288;
          $290 = HEAP32[$289>>2]|0;
          $291 = (($288) + 4)|0;
          $292 = $291;
          $293 = HEAP32[$292>>2]|0;
          $294 = $290 | $286;
          $295 = $293 | $287;
          $296 = $tabUsed;
          $297 = $296;
          HEAP32[$297>>2] = $294;
          $298 = (($296) + 4)|0;
          $299 = $298;
          HEAP32[$299>>2] = $295;
         }
         while(1) {
          $300 = HEAP32[$pWInfo>>2]|0;
          $301 = (($300) + 41|0);
          $302 = HEAP8[$301>>0]|0;
          $303 = $302&255;
          $304 = ($303|0)>=(2);
          if (!($304)) {
           break L60;
          }
          $305 = HEAP32[$pWInfo>>2]|0;
          $306 = (($305) + 41|0);
          $307 = HEAP8[$306>>0]|0;
          $308 = $307&255;
          $309 = (($308) - 1)|0;
          $310 = HEAP32[$pWInfo>>2]|0;
          $311 = (($310) + 736|0);
          $312 = (($311) + (($309*72)|0)|0);
          $313 = (($312) + 56|0);
          $314 = HEAP32[$313>>2]|0;
          HEAP32[$pLoop>>2] = $314;
          $315 = HEAP32[$pLoop>>2]|0;
          $316 = (($315) + 16|0);
          $317 = HEAP8[$316>>0]|0;
          $318 = $317&255;
          $319 = HEAP32[$pWInfo>>2]|0;
          $320 = (($319) + 4|0);
          $321 = HEAP32[$320>>2]|0;
          $322 = (($321) + 8|0);
          $323 = (($322) + (($318*72)|0)|0);
          $324 = (($323) + 36|0);
          $325 = HEAP8[$324>>0]|0;
          $326 = $325&255;
          $327 = $326 & 8;
          $328 = ($327|0)==(0);
          if ($328) {
           break L60;
          }
          $329 = HEAP16[$6>>1]|0;
          $330 = $329&65535;
          $331 = $330 & 1024;
          $332 = ($331|0)==(0);
          if ($332) {
           $333 = HEAP32[$pLoop>>2]|0;
           $334 = (($333) + 36|0);
           $335 = HEAP32[$334>>2]|0;
           $336 = $335 & 4096;
           $337 = ($336|0)==(0);
           if ($337) {
            break L60;
           }
          }
          $338 = $tabUsed;
          $339 = $338;
          $340 = HEAP32[$339>>2]|0;
          $341 = (($338) + 4)|0;
          $342 = $341;
          $343 = HEAP32[$342>>2]|0;
          $344 = HEAP32[$pLoop>>2]|0;
          $345 = (($344) + 8|0);
          $346 = $345;
          $347 = $346;
          $348 = HEAP32[$347>>2]|0;
          $349 = (($346) + 4)|0;
          $350 = $349;
          $351 = HEAP32[$350>>2]|0;
          $352 = $340 & $348;
          $353 = $343 & $351;
          $354 = ($352|0)!=(0);
          $355 = ($353|0)!=(0);
          $356 = $354 | $355;
          if ($356) {
           break L60;
          }
          $357 = (($sWLB) + 4|0);
          $358 = HEAP32[$357>>2]|0;
          $359 = (($358) + 20|0);
          $360 = HEAP32[$359>>2]|0;
          $361 = (($sWLB) + 4|0);
          $362 = HEAP32[$361>>2]|0;
          $363 = (($362) + 12|0);
          $364 = HEAP32[$363>>2]|0;
          $365 = (($360) + (($364*48)|0)|0);
          HEAP32[$pEnd>>2] = $365;
          $366 = (($sWLB) + 4|0);
          $367 = HEAP32[$366>>2]|0;
          $368 = (($367) + 20|0);
          $369 = HEAP32[$368>>2]|0;
          HEAP32[$pTerm>>2] = $369;
          while(1) {
           $370 = HEAP32[$pTerm>>2]|0;
           $371 = HEAP32[$pEnd>>2]|0;
           $372 = ($370>>>0)<($371>>>0);
           if (!($372)) {
            break;
           }
           $373 = HEAP32[$pTerm>>2]|0;
           $374 = (($373) + 40|0);
           $375 = $374;
           $376 = $375;
           $377 = HEAP32[$376>>2]|0;
           $378 = (($375) + 4)|0;
           $379 = $378;
           $380 = HEAP32[$379>>2]|0;
           $381 = HEAP32[$pLoop>>2]|0;
           $382 = (($381) + 8|0);
           $383 = $382;
           $384 = $383;
           $385 = HEAP32[$384>>2]|0;
           $386 = (($383) + 4)|0;
           $387 = $386;
           $388 = HEAP32[$387>>2]|0;
           $389 = $377 & $385;
           $390 = $380 & $388;
           $391 = ($389|0)!=(0);
           $392 = ($390|0)!=(0);
           $393 = $391 | $392;
           if ($393) {
            $394 = HEAP32[$pTerm>>2]|0;
            $395 = HEAP32[$394>>2]|0;
            $396 = (($395) + 4|0);
            $397 = HEAP32[$396>>2]|0;
            $398 = $397 & 1;
            $399 = ($398|0)!=(0);
            if (!($399)) {
             break;
            }
           }
           $400 = HEAP32[$pTerm>>2]|0;
           $401 = (($400) + 48|0);
           HEAP32[$pTerm>>2] = $401;
          }
          $402 = HEAP32[$pTerm>>2]|0;
          $403 = HEAP32[$pEnd>>2]|0;
          $404 = ($402>>>0)<($403>>>0);
          if ($404) {
           break L60;
          }
          $405 = HEAP32[$pWInfo>>2]|0;
          $406 = (($405) + 41|0);
          $407 = HEAP8[$406>>0]|0;
          $408 = (($407) + -1)<<24>>24;
          HEAP8[$406>>0] = $408;
          $409 = HEAP32[$nTabList>>2]|0;
          $410 = (($409) + -1)|0;
          HEAP32[$nTabList>>2] = $410;
         }
        }
       }
      } while(0);
      $411 = HEAP32[$pWInfo>>2]|0;
      $412 = (($411) + 32|0);
      $413 = HEAP16[$412>>1]|0;
      $414 = $413 << 16 >> 16;
      $415 = HEAP32[$pWInfo>>2]|0;
      $416 = HEAP32[$415>>2]|0;
      $417 = (($416) + 424|0);
      $418 = HEAP32[$417>>2]|0;
      $419 = (($418) + ($414))|0;
      HEAP32[$417>>2] = $419;
      $420 = HEAP16[$6>>1]|0;
      $421 = $420&65535;
      $422 = $421 & 4;
      $423 = ($422|0)!=(0);
      if ($423) {
       $424 = HEAP32[$pWInfo>>2]|0;
       $425 = (($424) + 736|0);
       $426 = (($425) + 56|0);
       $427 = HEAP32[$426>>2]|0;
       $428 = (($427) + 36|0);
       $429 = HEAP32[$428>>2]|0;
       $430 = $429 & 4096;
       $431 = ($430|0)!=(0);
       if ($431) {
        $432 = HEAP32[$pWInfo>>2]|0;
        $433 = (($432) + 38|0);
        HEAP8[$433>>0] = 1;
        $434 = HEAP32[$2>>2]|0;
        $435 = (($434) + 8|0);
        $436 = (($435) + 16|0);
        $437 = HEAP32[$436>>2]|0;
        $438 = (($437) + 44|0);
        $439 = HEAP8[$438>>0]|0;
        $440 = $439&255;
        $441 = $440 & 32;
        $442 = ($441|0)==(0);
        if ($442) {
         $443 = HEAP32[$pWInfo>>2]|0;
         $444 = (($443) + 736|0);
         $445 = (($444) + 56|0);
         $446 = HEAP32[$445>>2]|0;
         $447 = (($446) + 36|0);
         $448 = HEAP32[$447>>2]|0;
         $449 = $448 & -65;
         HEAP32[$447>>2] = $449;
        }
       }
      }
      $450 = $notReady;
      $451 = $450;
      HEAP32[$451>>2] = -1;
      $452 = (($450) + 4)|0;
      $453 = $452;
      HEAP32[$453>>2] = -1;
      HEAP32[$ii>>2] = 0;
      $454 = HEAP32[$pWInfo>>2]|0;
      $455 = (($454) + 736|0);
      HEAP32[$pLevel>>2] = $455;
      while(1) {
       $456 = HEAP32[$ii>>2]|0;
       $457 = HEAP32[$nTabList>>2]|0;
       $458 = ($456|0)<($457|0);
       if (!($458)) {
        break;
       }
       $459 = HEAP32[$pLevel>>2]|0;
       $460 = (($459) + 36|0);
       $461 = HEAP8[$460>>0]|0;
       $462 = $461&255;
       $463 = HEAP32[$2>>2]|0;
       $464 = (($463) + 8|0);
       $465 = (($464) + (($462*72)|0)|0);
       HEAP32[$pTabItem>>2] = $465;
       $466 = HEAP32[$pTabItem>>2]|0;
       $467 = (($466) + 16|0);
       $468 = HEAP32[$467>>2]|0;
       HEAP32[$pTab>>2] = $468;
       $469 = HEAP32[$db>>2]|0;
       $470 = HEAP32[$pTab>>2]|0;
       $471 = (($470) + 68|0);
       $472 = HEAP32[$471>>2]|0;
       $473 = (_sqlite3SchemaToIndex($469,$472)|0);
       HEAP32[$iDb>>2] = $473;
       $474 = HEAP32[$pLevel>>2]|0;
       $475 = (($474) + 56|0);
       $476 = HEAP32[$475>>2]|0;
       HEAP32[$pLoop>>2] = $476;
       $477 = HEAP32[$pTab>>2]|0;
       $478 = (($477) + 44|0);
       $479 = HEAP8[$478>>0]|0;
       $480 = $479&255;
       $481 = $480 & 2;
       $482 = ($481|0)!=(0);
       L90: do {
        if (!($482)) {
         $483 = HEAP32[$pTab>>2]|0;
         $484 = (($483) + 12|0);
         $485 = HEAP32[$484>>2]|0;
         $486 = ($485|0)!=(0|0);
         if (!($486)) {
          $487 = HEAP32[$pLoop>>2]|0;
          $488 = (($487) + 36|0);
          $489 = HEAP32[$488>>2]|0;
          $490 = $489 & 1024;
          $491 = ($490|0)!=(0);
          if ($491) {
           $492 = HEAP32[$db>>2]|0;
           $493 = HEAP32[$pTab>>2]|0;
           $494 = (_sqlite3GetVTable($492,$493)|0);
           HEAP32[$pVTab>>2] = $494;
           $495 = HEAP32[$pTabItem>>2]|0;
           $496 = (($495) + 40|0);
           $497 = HEAP32[$496>>2]|0;
           HEAP32[$iCur>>2] = $497;
           $498 = HEAP32[$v>>2]|0;
           $499 = HEAP32[$iCur>>2]|0;
           $500 = HEAP32[$pVTab>>2]|0;
           (_sqlite3VdbeAddOp4($498,147,$499,0,0,$500,-10)|0);
           break;
          }
          $501 = HEAP32[$pTab>>2]|0;
          $502 = (($501) + 44|0);
          $503 = HEAP8[$502>>0]|0;
          $504 = $503&255;
          $505 = $504 & 16;
          $506 = ($505|0)!=(0);
          if ($506) {
           break;
          }
          $507 = HEAP32[$pLoop>>2]|0;
          $508 = (($507) + 36|0);
          $509 = HEAP32[$508>>2]|0;
          $510 = $509 & 64;
          $511 = ($510|0)==(0);
          do {
           if ($511) {
            $512 = HEAP16[$6>>1]|0;
            $513 = $512&65535;
            $514 = $513 & 16;
            $515 = ($514|0)==(0);
            if (!($515)) {
             break;
            }
            HEAP32[$op>>2] = 54;
            $516 = HEAP32[$pWInfo>>2]|0;
            $517 = (($516) + 38|0);
            $518 = HEAP8[$517>>0]|0;
            $519 = ($518<<24>>24)!=(0);
            if ($519) {
             HEAP32[$op>>2] = 55;
             $520 = HEAP32[$pTabItem>>2]|0;
             $521 = (($520) + 40|0);
             $522 = HEAP32[$521>>2]|0;
             $523 = HEAP32[$pWInfo>>2]|0;
             $524 = (($523) + 60|0);
             HEAP32[$524>>2] = $522;
            }
            $525 = HEAP32[$1>>2]|0;
            $526 = HEAP32[$pTabItem>>2]|0;
            $527 = (($526) + 40|0);
            $528 = HEAP32[$527>>2]|0;
            $529 = HEAP32[$iDb>>2]|0;
            $530 = HEAP32[$pTab>>2]|0;
            $531 = HEAP32[$op>>2]|0;
            _sqlite3OpenTable($525,$528,$529,$530,$531);
            $532 = HEAP32[$pWInfo>>2]|0;
            $533 = (($532) + 38|0);
            $534 = HEAP8[$533>>0]|0;
            $535 = ($534<<24>>24)!=(0);
            if ($535) {
             break L90;
            }
            $536 = HEAP32[$pTab>>2]|0;
            $537 = (($536) + 38|0);
            $538 = HEAP16[$537>>1]|0;
            $539 = $538 << 16 >> 16;
            $540 = ($539|0)<(64);
            if (!($540)) {
             break L90;
            }
            $541 = HEAP32[$pTab>>2]|0;
            $542 = (($541) + 44|0);
            $543 = HEAP8[$542>>0]|0;
            $544 = $543&255;
            $545 = $544 & 32;
            $546 = ($545|0)==(0);
            if (!($546)) {
             break L90;
            }
            $547 = HEAP32[$pTabItem>>2]|0;
            $548 = (($547) + 56|0);
            $549 = $548;
            $550 = $549;
            $551 = HEAP32[$550>>2]|0;
            $552 = (($549) + 4)|0;
            $553 = $552;
            $554 = HEAP32[$553>>2]|0;
            $555 = $b;
            $556 = $555;
            HEAP32[$556>>2] = $551;
            $557 = (($555) + 4)|0;
            $558 = $557;
            HEAP32[$558>>2] = $554;
            HEAP32[$n>>2] = 0;
            while(1) {
             $559 = $b;
             $560 = $559;
             $561 = HEAP32[$560>>2]|0;
             $562 = (($559) + 4)|0;
             $563 = $562;
             $564 = HEAP32[$563>>2]|0;
             $565 = ($561|0)!=(0);
             $566 = ($564|0)!=(0);
             $567 = $565 | $566;
             if (!($567)) {
              break;
             }
             $568 = $b;
             $569 = $568;
             $570 = HEAP32[$569>>2]|0;
             $571 = (($568) + 4)|0;
             $572 = $571;
             $573 = HEAP32[$572>>2]|0;
             $574 = (_bitshift64Lshr(($570|0),($573|0),1)|0);
             $575 = tempRet0;
             $576 = $b;
             $577 = $576;
             HEAP32[$577>>2] = $574;
             $578 = (($576) + 4)|0;
             $579 = $578;
             HEAP32[$579>>2] = $575;
             $580 = HEAP32[$n>>2]|0;
             $581 = (($580) + 1)|0;
             HEAP32[$n>>2] = $581;
            }
            $582 = HEAP32[$v>>2]|0;
            $583 = HEAP32[$v>>2]|0;
            $584 = (_sqlite3VdbeCurrentAddr($583)|0);
            $585 = (($584) - 1)|0;
            $586 = HEAP32[$n>>2]|0;
            $587 = $586;
            _sqlite3VdbeChangeP4($582,$585,$587,-14);
            break L90;
           }
          } while(0);
          $588 = HEAP32[$1>>2]|0;
          $589 = HEAP32[$iDb>>2]|0;
          $590 = HEAP32[$pTab>>2]|0;
          $591 = (($590) + 32|0);
          $592 = HEAP32[$591>>2]|0;
          $593 = HEAP32[$pTab>>2]|0;
          $594 = HEAP32[$593>>2]|0;
          _sqlite3TableLock($588,$589,$592,0,$594);
         }
        }
       } while(0);
       $595 = HEAP32[$pLoop>>2]|0;
       $596 = (($595) + 36|0);
       $597 = HEAP32[$596>>2]|0;
       $598 = $597 & 512;
       $599 = ($598|0)!=(0);
       do {
        if ($599) {
         $600 = HEAP32[$pLoop>>2]|0;
         $601 = (($600) + 24|0);
         $602 = (($601) + 4|0);
         $603 = HEAP32[$602>>2]|0;
         HEAP32[$pIx>>2] = $603;
         HEAP32[$op1>>2] = 54;
         $604 = HEAP32[$pTab>>2]|0;
         $605 = (($604) + 44|0);
         $606 = HEAP8[$605>>0]|0;
         $607 = $606&255;
         $608 = $607 & 32;
         $609 = ($608|0)==(0);
         do {
          if ($609) {
           label = 87;
          } else {
           $610 = HEAP32[$pIx>>2]|0;
           $611 = (($610) + 55|0);
           $612 = HEAP8[$611>>0]|0;
           $613 = $612 & 3;
           $614 = $613&255;
           $615 = ($614|0)==(2);
           if (!($615)) {
            label = 87;
            break;
           }
           $616 = HEAP16[$6>>1]|0;
           $617 = $616&65535;
           $618 = $617 & 64;
           $619 = ($618|0)!=(0);
           if (!($619)) {
            label = 87;
            break;
           }
           $620 = HEAP32[$pLevel>>2]|0;
           $621 = (($620) + 4|0);
           $622 = HEAP32[$621>>2]|0;
           HEAP32[$iIndexCur>>2] = $622;
           HEAP32[$op1>>2] = 0;
          }
         } while(0);
         L118: do {
          if ((label|0) == 87) {
           label = 0;
           $623 = HEAP32[$pWInfo>>2]|0;
           $624 = (($623) + 38|0);
           $625 = HEAP8[$624>>0]|0;
           $626 = ($625<<24>>24)!=(0);
           if ($626) {
            $627 = HEAP32[$pTabItem>>2]|0;
            $628 = (($627) + 16|0);
            $629 = HEAP32[$628>>2]|0;
            $630 = (($629) + 8|0);
            $631 = HEAP32[$630>>2]|0;
            HEAP32[$pJ>>2] = $631;
            $632 = HEAP32[$7>>2]|0;
            HEAP32[$iIndexCur>>2] = $632;
            while(1) {
             $633 = HEAP32[$pJ>>2]|0;
             $634 = ($633|0)!=(0|0);
             if (!($634)) {
              break;
             }
             $635 = HEAP32[$pJ>>2]|0;
             $636 = HEAP32[$pIx>>2]|0;
             $637 = ($635|0)!=($636|0);
             if (!($637)) {
              break;
             }
             $638 = HEAP32[$iIndexCur>>2]|0;
             $639 = (($638) + 1)|0;
             HEAP32[$iIndexCur>>2] = $639;
             $640 = HEAP32[$pJ>>2]|0;
             $641 = (($640) + 20|0);
             $642 = HEAP32[$641>>2]|0;
             HEAP32[$pJ>>2] = $642;
            }
            HEAP32[$op1>>2] = 55;
            $643 = HEAP32[$iIndexCur>>2]|0;
            $644 = HEAP32[$pWInfo>>2]|0;
            $645 = (($644) + 60|0);
            $646 = (($645) + 4|0);
            HEAP32[$646>>2] = $643;
            break;
           } else {
            $647 = HEAP32[$7>>2]|0;
            $648 = ($647|0)!=(0);
            do {
             if ($648) {
              $649 = HEAP16[$6>>1]|0;
              $650 = $649&65535;
              $651 = $650 & 64;
              $652 = ($651|0)!=(0);
              if (!($652)) {
               break;
              }
              $653 = HEAP32[$7>>2]|0;
              HEAP32[$iIndexCur>>2] = $653;
              $654 = HEAP16[$6>>1]|0;
              $655 = $654&65535;
              $656 = $655 & 4096;
              $657 = ($656|0)!=(0);
              if (!($657)) {
               break L118;
              }
              HEAP32[$op1>>2] = 53;
              break L118;
             }
            } while(0);
            $658 = HEAP32[$1>>2]|0;
            $659 = (($658) + 68|0);
            $660 = HEAP32[$659>>2]|0;
            $661 = (($660) + 1)|0;
            HEAP32[$659>>2] = $661;
            HEAP32[$iIndexCur>>2] = $660;
            break;
           }
          }
         } while(0);
         $662 = HEAP32[$iIndexCur>>2]|0;
         $663 = HEAP32[$pLevel>>2]|0;
         $664 = (($663) + 8|0);
         HEAP32[$664>>2] = $662;
         $665 = HEAP32[$op1>>2]|0;
         $666 = ($665|0)!=(0);
         if (!($666)) {
          break;
         }
         $667 = HEAP32[$v>>2]|0;
         $668 = HEAP32[$op1>>2]|0;
         $669 = HEAP32[$iIndexCur>>2]|0;
         $670 = HEAP32[$pIx>>2]|0;
         $671 = (($670) + 44|0);
         $672 = HEAP32[$671>>2]|0;
         $673 = HEAP32[$iDb>>2]|0;
         (_sqlite3VdbeAddOp3($667,$668,$669,$672,$673)|0);
         $674 = HEAP32[$1>>2]|0;
         $675 = HEAP32[$pIx>>2]|0;
         _sqlite3VdbeSetP4KeyInfo($674,$675);
        }
       } while(0);
       $676 = HEAP32[$iDb>>2]|0;
       $677 = ($676|0)>=(0);
       if ($677) {
        $678 = HEAP32[$1>>2]|0;
        $679 = HEAP32[$iDb>>2]|0;
        _sqlite3CodeVerifySchema($678,$679);
       }
       $680 = HEAP32[$pWInfo>>2]|0;
       $681 = (($680) + 68|0);
       $682 = HEAP32[$pTabItem>>2]|0;
       $683 = (($682) + 40|0);
       $684 = HEAP32[$683>>2]|0;
       $685 = (_getMask($681,$684)|0);
       $686 = tempRet0;
       $687 = $685 ^ -1;
       $688 = $686 ^ -1;
       $689 = $notReady;
       $690 = $689;
       $691 = HEAP32[$690>>2]|0;
       $692 = (($689) + 4)|0;
       $693 = $692;
       $694 = HEAP32[$693>>2]|0;
       $695 = $691 & $687;
       $696 = $694 & $688;
       $697 = $notReady;
       $698 = $697;
       HEAP32[$698>>2] = $695;
       $699 = (($697) + 4)|0;
       $700 = $699;
       HEAP32[$700>>2] = $696;
       $701 = HEAP32[$ii>>2]|0;
       $702 = (($701) + 1)|0;
       HEAP32[$ii>>2] = $702;
       $703 = HEAP32[$pLevel>>2]|0;
       $704 = (($703) + 72|0);
       HEAP32[$pLevel>>2] = $704;
      }
      $705 = HEAP32[$v>>2]|0;
      $706 = (_sqlite3VdbeCurrentAddr($705)|0);
      $707 = HEAP32[$pWInfo>>2]|0;
      $708 = (($707) + 44|0);
      HEAP32[$708>>2] = $706;
      $709 = HEAP32[$db>>2]|0;
      $710 = (($709) + 64|0);
      $711 = HEAP8[$710>>0]|0;
      $712 = ($711<<24>>24)!=(0);
      if (!($712)) {
       $713 = $notReady;
       $714 = $713;
       HEAP32[$714>>2] = -1;
       $715 = (($713) + 4)|0;
       $716 = $715;
       HEAP32[$716>>2] = -1;
       HEAP32[$ii>>2] = 0;
       while(1) {
        $717 = HEAP32[$ii>>2]|0;
        $718 = HEAP32[$nTabList>>2]|0;
        $719 = ($717|0)<($718|0);
        if (!($719)) {
         break;
        }
        $720 = HEAP32[$ii>>2]|0;
        $721 = HEAP32[$pWInfo>>2]|0;
        $722 = (($721) + 736|0);
        $723 = (($722) + (($720*72)|0)|0);
        HEAP32[$pLevel>>2] = $723;
        $724 = HEAP32[$pLevel>>2]|0;
        $725 = (($724) + 56|0);
        $726 = HEAP32[$725>>2]|0;
        $727 = (($726) + 36|0);
        $728 = HEAP32[$727>>2]|0;
        $729 = $728 & 16384;
        $730 = ($729|0)!=(0);
        if ($730) {
         $731 = HEAP32[$1>>2]|0;
         $732 = HEAP32[$pWInfo>>2]|0;
         $733 = (($732) + 328|0);
         $734 = HEAP32[$pLevel>>2]|0;
         $735 = (($734) + 36|0);
         $736 = HEAP8[$735>>0]|0;
         $737 = $736&255;
         $738 = HEAP32[$2>>2]|0;
         $739 = (($738) + 8|0);
         $740 = (($739) + (($737*72)|0)|0);
         $741 = $notReady;
         $742 = $741;
         $743 = HEAP32[$742>>2]|0;
         $744 = (($741) + 4)|0;
         $745 = $744;
         $746 = HEAP32[$745>>2]|0;
         $747 = HEAP32[$pLevel>>2]|0;
         _constructAutomaticIndex($731,$733,$740,$743,$746,$747);
         $748 = HEAP32[$db>>2]|0;
         $749 = (($748) + 64|0);
         $750 = HEAP8[$749>>0]|0;
         $751 = ($750<<24>>24)!=(0);
         if ($751) {
          break L15;
         }
        }
        $752 = HEAP32[$1>>2]|0;
        $753 = HEAP32[$2>>2]|0;
        $754 = HEAP32[$pLevel>>2]|0;
        $755 = HEAP32[$ii>>2]|0;
        $756 = HEAP32[$pLevel>>2]|0;
        $757 = (($756) + 36|0);
        $758 = HEAP8[$757>>0]|0;
        $759 = $758&255;
        $760 = HEAP16[$6>>1]|0;
        _explainOneScan($752,$753,$754,$755,$759,$760);
        $761 = HEAP32[$v>>2]|0;
        $762 = (_sqlite3VdbeCurrentAddr($761)|0);
        $763 = HEAP32[$pLevel>>2]|0;
        $764 = (($763) + 32|0);
        HEAP32[$764>>2] = $762;
        $765 = HEAP32[$pWInfo>>2]|0;
        $766 = HEAP32[$ii>>2]|0;
        $767 = $notReady;
        $768 = $767;
        $769 = HEAP32[$768>>2]|0;
        $770 = (($767) + 4)|0;
        $771 = $770;
        $772 = HEAP32[$771>>2]|0;
        $773 = (_codeOneLoopStart($765,$766,$769,$772)|0);
        $774 = tempRet0;
        $775 = $notReady;
        $776 = $775;
        HEAP32[$776>>2] = $773;
        $777 = (($775) + 4)|0;
        $778 = $777;
        HEAP32[$778>>2] = $774;
        $779 = HEAP32[$pLevel>>2]|0;
        $780 = (($779) + 24|0);
        $781 = HEAP32[$780>>2]|0;
        $782 = HEAP32[$pWInfo>>2]|0;
        $783 = (($782) + 48|0);
        HEAP32[$783>>2] = $781;
        $784 = HEAP32[$ii>>2]|0;
        $785 = (($784) + 1)|0;
        HEAP32[$ii>>2] = $785;
       }
       $786 = HEAP32[$pWInfo>>2]|0;
       HEAP32[$0>>2] = $786;
       $796 = HEAP32[$0>>2]|0;
       STACKTOP = sp;return ($796|0);
      }
     }
    }
   }
  }
 } while(0);
 $787 = HEAP32[$pWInfo>>2]|0;
 $788 = ($787|0)!=(0|0);
 if ($788) {
  $789 = HEAP32[$pWInfo>>2]|0;
  $790 = (($789) + 56|0);
  $791 = HEAP32[$790>>2]|0;
  $792 = HEAP32[$1>>2]|0;
  $793 = (($792) + 424|0);
  HEAP32[$793>>2] = $791;
  $794 = HEAP32[$db>>2]|0;
  $795 = HEAP32[$pWInfo>>2]|0;
  _whereInfoFree($794,$795);
 }
 HEAP32[$0>>2] = 0;
 $796 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($796|0);
}
function _sqlite3WhereEnd($pWInfo) {
 $pWInfo = $pWInfo|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0;
 var $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0, $188 = 0;
 var $189 = 0, $19 = 0, $190 = 0, $191 = 0, $192 = 0, $193 = 0, $194 = 0, $195 = 0, $196 = 0, $197 = 0, $198 = 0, $199 = 0, $2 = 0, $20 = 0, $200 = 0, $201 = 0, $202 = 0, $203 = 0, $204 = 0, $205 = 0;
 var $206 = 0, $207 = 0, $208 = 0, $209 = 0, $21 = 0, $210 = 0, $211 = 0, $212 = 0, $213 = 0, $214 = 0, $215 = 0, $216 = 0, $217 = 0, $218 = 0, $219 = 0, $22 = 0, $220 = 0, $221 = 0, $222 = 0, $223 = 0;
 var $224 = 0, $225 = 0, $226 = 0, $227 = 0, $228 = 0, $229 = 0, $23 = 0, $230 = 0, $231 = 0, $232 = 0, $233 = 0, $234 = 0, $235 = 0, $236 = 0, $237 = 0, $238 = 0, $239 = 0, $24 = 0, $240 = 0, $241 = 0;
 var $242 = 0, $243 = 0, $244 = 0, $245 = 0, $246 = 0, $247 = 0, $248 = 0, $249 = 0, $25 = 0, $250 = 0, $251 = 0, $252 = 0, $253 = 0, $254 = 0, $255 = 0, $256 = 0, $257 = 0, $258 = 0, $259 = 0, $26 = 0;
 var $260 = 0, $261 = 0, $262 = 0, $263 = 0, $264 = 0, $265 = 0, $266 = 0, $267 = 0, $268 = 0, $269 = 0, $27 = 0, $270 = 0, $271 = 0, $272 = 0, $273 = 0, $274 = 0, $275 = 0, $276 = 0, $277 = 0, $278 = 0;
 var $279 = 0, $28 = 0, $280 = 0, $281 = 0, $282 = 0, $283 = 0, $284 = 0, $285 = 0, $286 = 0, $287 = 0, $288 = 0, $289 = 0, $29 = 0, $290 = 0, $291 = 0, $292 = 0, $293 = 0, $294 = 0, $295 = 0, $296 = 0;
 var $297 = 0, $298 = 0, $299 = 0, $3 = 0, $30 = 0, $300 = 0, $301 = 0, $302 = 0, $303 = 0, $304 = 0, $305 = 0, $306 = 0, $307 = 0, $308 = 0, $309 = 0, $31 = 0, $310 = 0, $311 = 0, $312 = 0, $313 = 0;
 var $314 = 0, $315 = 0, $316 = 0, $317 = 0, $318 = 0, $319 = 0, $32 = 0, $320 = 0, $321 = 0, $322 = 0, $323 = 0, $324 = 0, $325 = 0, $326 = 0, $327 = 0, $328 = 0, $329 = 0, $33 = 0, $330 = 0, $331 = 0;
 var $332 = 0, $333 = 0, $334 = 0, $335 = 0, $336 = 0, $337 = 0, $338 = 0, $339 = 0, $34 = 0, $340 = 0, $341 = 0, $342 = 0, $343 = 0, $344 = 0, $345 = 0, $346 = 0, $347 = 0, $348 = 0, $349 = 0, $35 = 0;
 var $350 = 0, $351 = 0, $352 = 0, $353 = 0, $354 = 0, $355 = 0, $356 = 0, $357 = 0, $358 = 0, $359 = 0, $36 = 0, $360 = 0, $361 = 0, $362 = 0, $363 = 0, $364 = 0, $365 = 0, $366 = 0, $367 = 0, $368 = 0;
 var $369 = 0, $37 = 0, $370 = 0, $371 = 0, $372 = 0, $373 = 0, $374 = 0, $375 = 0, $376 = 0, $377 = 0, $378 = 0, $379 = 0, $38 = 0, $380 = 0, $381 = 0, $382 = 0, $383 = 0, $384 = 0, $385 = 0, $386 = 0;
 var $387 = 0, $388 = 0, $389 = 0, $39 = 0, $390 = 0, $391 = 0, $392 = 0, $393 = 0, $394 = 0, $395 = 0, $396 = 0, $397 = 0, $398 = 0, $399 = 0, $4 = 0, $40 = 0, $400 = 0, $401 = 0, $402 = 0, $403 = 0;
 var $404 = 0, $405 = 0, $406 = 0, $407 = 0, $408 = 0, $409 = 0, $41 = 0, $410 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0;
 var $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0;
 var $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0;
 var $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $addr = 0, $db = 0, $i = 0, $j = 0, $k = 0, $last = 0, $pIdx = 0, $pIn = 0, $pLevel = 0;
 var $pLoop = 0, $pOp = 0, $pParse = 0, $pPk = 0, $pTab = 0, $pTabItem = 0, $pTabList = 0, $v = 0, $ws = 0, $x = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 80|0;
 $0 = sp + 76|0;
 $pParse = sp + 72|0;
 $v = sp + 68|0;
 $i = sp + 64|0;
 $pLevel = sp + 60|0;
 $pLoop = sp + 56|0;
 $pTabList = sp + 52|0;
 $db = sp + 48|0;
 $addr = sp + 44|0;
 $pIn = sp + 40|0;
 $j = sp + 36|0;
 $k = sp + 32|0;
 $last = sp + 28|0;
 $pOp = sp + 24|0;
 $pIdx = sp + 20|0;
 $pTabItem = sp + 16|0;
 $pTab = sp + 12|0;
 $ws = sp + 8|0;
 $x = sp + 4|0;
 $pPk = sp;
 HEAP32[$0>>2] = $pWInfo;
 $1 = HEAP32[$0>>2]|0;
 $2 = HEAP32[$1>>2]|0;
 HEAP32[$pParse>>2] = $2;
 $3 = HEAP32[$pParse>>2]|0;
 $4 = (($3) + 8|0);
 $5 = HEAP32[$4>>2]|0;
 HEAP32[$v>>2] = $5;
 $6 = HEAP32[$0>>2]|0;
 $7 = (($6) + 4|0);
 $8 = HEAP32[$7>>2]|0;
 HEAP32[$pTabList>>2] = $8;
 $9 = HEAP32[$pParse>>2]|0;
 $10 = HEAP32[$9>>2]|0;
 HEAP32[$db>>2] = $10;
 $11 = HEAP32[$pParse>>2]|0;
 _sqlite3ExprCacheClear($11);
 $12 = HEAP32[$0>>2]|0;
 $13 = (($12) + 41|0);
 $14 = HEAP8[$13>>0]|0;
 $15 = $14&255;
 $16 = (($15) - 1)|0;
 HEAP32[$i>>2] = $16;
 while(1) {
  $17 = HEAP32[$i>>2]|0;
  $18 = ($17|0)>=(0);
  if (!($18)) {
   break;
  }
  $19 = HEAP32[$i>>2]|0;
  $20 = HEAP32[$0>>2]|0;
  $21 = (($20) + 736|0);
  $22 = (($21) + (($19*72)|0)|0);
  HEAP32[$pLevel>>2] = $22;
  $23 = HEAP32[$pLevel>>2]|0;
  $24 = (($23) + 56|0);
  $25 = HEAP32[$24>>2]|0;
  HEAP32[$pLoop>>2] = $25;
  $26 = HEAP32[$v>>2]|0;
  $27 = HEAP32[$pLevel>>2]|0;
  $28 = (($27) + 24|0);
  $29 = HEAP32[$28>>2]|0;
  _sqlite3VdbeResolveLabel($26,$29);
  $30 = HEAP32[$pLevel>>2]|0;
  $31 = (($30) + 37|0);
  $32 = HEAP8[$31>>0]|0;
  $33 = $32&255;
  $34 = ($33|0)!=(154);
  if ($34) {
   $35 = HEAP32[$v>>2]|0;
   $36 = HEAP32[$pLevel>>2]|0;
   $37 = (($36) + 37|0);
   $38 = HEAP8[$37>>0]|0;
   $39 = $38&255;
   $40 = HEAP32[$pLevel>>2]|0;
   $41 = (($40) + 40|0);
   $42 = HEAP32[$41>>2]|0;
   $43 = HEAP32[$pLevel>>2]|0;
   $44 = (($43) + 44|0);
   $45 = HEAP32[$44>>2]|0;
   $46 = HEAP32[$pLevel>>2]|0;
   $47 = (($46) + 38|0);
   $48 = HEAP8[$47>>0]|0;
   $49 = $48&255;
   (_sqlite3VdbeAddOp3($35,$39,$42,$45,$49)|0);
   $50 = HEAP32[$v>>2]|0;
   $51 = HEAP32[$pLevel>>2]|0;
   $52 = (($51) + 39|0);
   $53 = HEAP8[$52>>0]|0;
   _sqlite3VdbeChangeP5($50,$53);
  }
  $54 = HEAP32[$pLoop>>2]|0;
  $55 = (($54) + 36|0);
  $56 = HEAP32[$55>>2]|0;
  $57 = $56 & 2048;
  $58 = ($57|0)!=(0);
  if ($58) {
   $59 = HEAP32[$pLevel>>2]|0;
   $60 = (($59) + 48|0);
   $61 = HEAP32[$60>>2]|0;
   $62 = ($61|0)>(0);
   if ($62) {
    $63 = HEAP32[$v>>2]|0;
    $64 = HEAP32[$pLevel>>2]|0;
    $65 = (($64) + 16|0);
    $66 = HEAP32[$65>>2]|0;
    _sqlite3VdbeResolveLabel($63,$66);
    $67 = HEAP32[$pLevel>>2]|0;
    $68 = (($67) + 48|0);
    $69 = HEAP32[$68>>2]|0;
    HEAP32[$j>>2] = $69;
    $70 = HEAP32[$j>>2]|0;
    $71 = (($70) - 1)|0;
    $72 = HEAP32[$pLevel>>2]|0;
    $73 = (($72) + 48|0);
    $74 = (($73) + 4|0);
    $75 = HEAP32[$74>>2]|0;
    $76 = (($75) + (($71*12)|0)|0);
    HEAP32[$pIn>>2] = $76;
    while(1) {
     $77 = HEAP32[$j>>2]|0;
     $78 = ($77|0)>(0);
     if (!($78)) {
      break;
     }
     $79 = HEAP32[$v>>2]|0;
     $80 = HEAP32[$pIn>>2]|0;
     $81 = (($80) + 4|0);
     $82 = HEAP32[$81>>2]|0;
     $83 = (($82) + 1)|0;
     _sqlite3VdbeJumpHere($79,$83);
     $84 = HEAP32[$v>>2]|0;
     $85 = HEAP32[$pIn>>2]|0;
     $86 = (($85) + 8|0);
     $87 = HEAP8[$86>>0]|0;
     $88 = $87&255;
     $89 = HEAP32[$pIn>>2]|0;
     $90 = HEAP32[$89>>2]|0;
     $91 = HEAP32[$pIn>>2]|0;
     $92 = (($91) + 4|0);
     $93 = HEAP32[$92>>2]|0;
     (_sqlite3VdbeAddOp2($84,$88,$90,$93)|0);
     $94 = HEAP32[$v>>2]|0;
     $95 = HEAP32[$pIn>>2]|0;
     $96 = (($95) + 4|0);
     $97 = HEAP32[$96>>2]|0;
     $98 = (($97) - 1)|0;
     _sqlite3VdbeJumpHere($94,$98);
     $99 = HEAP32[$j>>2]|0;
     $100 = (($99) + -1)|0;
     HEAP32[$j>>2] = $100;
     $101 = HEAP32[$pIn>>2]|0;
     $102 = (($101) + -12|0);
     HEAP32[$pIn>>2] = $102;
    }
    $103 = HEAP32[$db>>2]|0;
    $104 = HEAP32[$pLevel>>2]|0;
    $105 = (($104) + 48|0);
    $106 = (($105) + 4|0);
    $107 = HEAP32[$106>>2]|0;
    _sqlite3DbFree($103,$107);
   }
  }
  $108 = HEAP32[$v>>2]|0;
  $109 = HEAP32[$pLevel>>2]|0;
  $110 = (($109) + 12|0);
  $111 = HEAP32[$110>>2]|0;
  _sqlite3VdbeResolveLabel($108,$111);
  $112 = HEAP32[$pLevel>>2]|0;
  $113 = (($112) + 20|0);
  $114 = HEAP32[$113>>2]|0;
  $115 = ($114|0)!=(0);
  if ($115) {
   $116 = HEAP32[$v>>2]|0;
   $117 = HEAP32[$pLevel>>2]|0;
   $118 = (($117) + 20|0);
   $119 = HEAP32[$118>>2]|0;
   (_sqlite3VdbeAddOp2($116,16,0,$119)|0);
   $120 = HEAP32[$v>>2]|0;
   $121 = HEAP32[$pLevel>>2]|0;
   $122 = (($121) + 20|0);
   $123 = HEAP32[$122>>2]|0;
   _sqlite3VdbeJumpHere($120,$123);
   $124 = HEAP32[$v>>2]|0;
   $125 = HEAP32[$pLevel>>2]|0;
   $126 = (($125) + 20|0);
   $127 = HEAP32[$126>>2]|0;
   $128 = (($127) - 2)|0;
   _sqlite3VdbeJumpHere($124,$128);
  }
  $129 = HEAP32[$pLevel>>2]|0;
  $130 = HEAP32[$129>>2]|0;
  $131 = ($130|0)!=(0);
  if ($131) {
   $132 = HEAP32[$v>>2]|0;
   $133 = HEAP32[$pLevel>>2]|0;
   $134 = HEAP32[$133>>2]|0;
   $135 = (_sqlite3VdbeAddOp1($132,137,$134)|0);
   HEAP32[$addr>>2] = $135;
   $136 = HEAP32[$pLoop>>2]|0;
   $137 = (($136) + 36|0);
   $138 = HEAP32[$137>>2]|0;
   $139 = $138 & 64;
   $140 = ($139|0)==(0);
   if ($140) {
    $141 = HEAP32[$v>>2]|0;
    $142 = HEAP32[$i>>2]|0;
    $143 = HEAP32[$pTabList>>2]|0;
    $144 = (($143) + 8|0);
    $145 = (($144) + (($142*72)|0)|0);
    $146 = (($145) + 40|0);
    $147 = HEAP32[$146>>2]|0;
    (_sqlite3VdbeAddOp1($141,104,$147)|0);
   }
   $148 = HEAP32[$pLoop>>2]|0;
   $149 = (($148) + 36|0);
   $150 = HEAP32[$149>>2]|0;
   $151 = $150 & 512;
   $152 = ($151|0)!=(0);
   if ($152) {
    $153 = HEAP32[$v>>2]|0;
    $154 = HEAP32[$pLevel>>2]|0;
    $155 = (($154) + 8|0);
    $156 = HEAP32[$155>>2]|0;
    (_sqlite3VdbeAddOp1($153,104,$156)|0);
   }
   $157 = HEAP32[$pLevel>>2]|0;
   $158 = (($157) + 37|0);
   $159 = HEAP8[$158>>0]|0;
   $160 = $159&255;
   $161 = ($160|0)==(18);
   $162 = HEAP32[$v>>2]|0;
   $163 = HEAP32[$pLevel>>2]|0;
   if ($161) {
    $164 = (($163) + 40|0);
    $165 = HEAP32[$164>>2]|0;
    $166 = HEAP32[$pLevel>>2]|0;
    $167 = (($166) + 28|0);
    $168 = HEAP32[$167>>2]|0;
    (_sqlite3VdbeAddOp2($162,17,$165,$168)|0);
   } else {
    $169 = (($163) + 28|0);
    $170 = HEAP32[$169>>2]|0;
    (_sqlite3VdbeAddOp2($162,16,0,$170)|0);
   }
   $171 = HEAP32[$v>>2]|0;
   $172 = HEAP32[$addr>>2]|0;
   _sqlite3VdbeJumpHere($171,$172);
  }
  $173 = HEAP32[$i>>2]|0;
  $174 = (($173) + -1)|0;
  HEAP32[$i>>2] = $174;
 }
 $175 = HEAP32[$v>>2]|0;
 $176 = HEAP32[$0>>2]|0;
 $177 = (($176) + 52|0);
 $178 = HEAP32[$177>>2]|0;
 _sqlite3VdbeResolveLabel($175,$178);
 HEAP32[$i>>2] = 0;
 $179 = HEAP32[$0>>2]|0;
 $180 = (($179) + 736|0);
 HEAP32[$pLevel>>2] = $180;
 while(1) {
  $181 = HEAP32[$i>>2]|0;
  $182 = HEAP32[$0>>2]|0;
  $183 = (($182) + 41|0);
  $184 = HEAP8[$183>>0]|0;
  $185 = $184&255;
  $186 = ($181|0)<($185|0);
  if (!($186)) {
   break;
  }
  HEAP32[$pIdx>>2] = 0;
  $187 = HEAP32[$pLevel>>2]|0;
  $188 = (($187) + 36|0);
  $189 = HEAP8[$188>>0]|0;
  $190 = $189&255;
  $191 = HEAP32[$pTabList>>2]|0;
  $192 = (($191) + 8|0);
  $193 = (($192) + (($190*72)|0)|0);
  HEAP32[$pTabItem>>2] = $193;
  $194 = HEAP32[$pTabItem>>2]|0;
  $195 = (($194) + 16|0);
  $196 = HEAP32[$195>>2]|0;
  HEAP32[$pTab>>2] = $196;
  $197 = HEAP32[$pLevel>>2]|0;
  $198 = (($197) + 56|0);
  $199 = HEAP32[$198>>2]|0;
  HEAP32[$pLoop>>2] = $199;
  $200 = HEAP32[$pTabItem>>2]|0;
  $201 = (($200) + 37|0);
  $202 = HEAP8[$201>>0]|0;
  $203 = ($202&255) >>> 2;
  $204 = $203 & 1;
  $205 = $204&255;
  $206 = ($205|0)!=(0);
  L35: do {
   if ($206) {
    $207 = HEAP32[$db>>2]|0;
    $208 = (($207) + 64|0);
    $209 = HEAP8[$208>>0]|0;
    $210 = ($209<<24>>24)!=(0);
    if ($210) {
     label = 35;
    } else {
     $211 = HEAP32[$v>>2]|0;
     $212 = (_sqlite3VdbeCurrentAddr($211)|0);
     HEAP32[$last>>2] = $212;
     $213 = HEAP32[$pLevel>>2]|0;
     $214 = (($213) + 32|0);
     $215 = HEAP32[$214>>2]|0;
     HEAP32[$k>>2] = $215;
     $216 = HEAP32[$v>>2]|0;
     $217 = HEAP32[$k>>2]|0;
     $218 = (_sqlite3VdbeGetOp($216,$217)|0);
     HEAP32[$pOp>>2] = $218;
     while(1) {
      $219 = HEAP32[$k>>2]|0;
      $220 = HEAP32[$last>>2]|0;
      $221 = ($219|0)<($220|0);
      if (!($221)) {
       break L35;
      }
      $222 = HEAP32[$pOp>>2]|0;
      $223 = (($222) + 4|0);
      $224 = HEAP32[$223>>2]|0;
      $225 = HEAP32[$pLevel>>2]|0;
      $226 = (($225) + 4|0);
      $227 = HEAP32[$226>>2]|0;
      $228 = ($224|0)!=($227|0);
      do {
       if (!($228)) {
        $229 = HEAP32[$pOp>>2]|0;
        $230 = HEAP8[$229>>0]|0;
        $231 = $230&255;
        $232 = ($231|0)==(47);
        $233 = HEAP32[$pOp>>2]|0;
        if ($232) {
         HEAP8[$233>>0] = 33;
         $234 = HEAP32[$pOp>>2]|0;
         $235 = (($234) + 8|0);
         $236 = HEAP32[$235>>2]|0;
         $237 = HEAP32[$pTabItem>>2]|0;
         $238 = (($237) + 32|0);
         $239 = HEAP32[$238>>2]|0;
         $240 = (($236) + ($239))|0;
         $241 = HEAP32[$pOp>>2]|0;
         $242 = (($241) + 4|0);
         HEAP32[$242>>2] = $240;
         $243 = HEAP32[$pOp>>2]|0;
         $244 = (($243) + 12|0);
         $245 = HEAP32[$244>>2]|0;
         $246 = HEAP32[$pOp>>2]|0;
         $247 = (($246) + 8|0);
         HEAP32[$247>>2] = $245;
         $248 = HEAP32[$pOp>>2]|0;
         $249 = (($248) + 12|0);
         HEAP32[$249>>2] = 0;
         break;
        }
        $250 = HEAP8[$233>>0]|0;
        $251 = $250&255;
        $252 = ($251|0)==(103);
        if ($252) {
         $253 = HEAP32[$pOp>>2]|0;
         HEAP8[$253>>0] = 28;
         $254 = HEAP32[$pOp>>2]|0;
         $255 = (($254) + 4|0);
         HEAP32[$255>>2] = 0;
         $256 = HEAP32[$pOp>>2]|0;
         $257 = (($256) + 12|0);
         HEAP32[$257>>2] = 0;
        }
       }
      } while(0);
      $258 = HEAP32[$k>>2]|0;
      $259 = (($258) + 1)|0;
      HEAP32[$k>>2] = $259;
      $260 = HEAP32[$pOp>>2]|0;
      $261 = (($260) + 20|0);
      HEAP32[$pOp>>2] = $261;
     }
    }
   } else {
    label = 35;
   }
  } while(0);
  L48: do {
   if ((label|0) == 35) {
    label = 0;
    $262 = HEAP32[$pTab>>2]|0;
    $263 = (($262) + 44|0);
    $264 = HEAP8[$263>>0]|0;
    $265 = $264&255;
    $266 = $265 & 2;
    $267 = ($266|0)==(0);
    if ($267) {
     $268 = HEAP32[$pTab>>2]|0;
     $269 = (($268) + 12|0);
     $270 = HEAP32[$269>>2]|0;
     $271 = ($270|0)==(0|0);
     if ($271) {
      $272 = HEAP32[$0>>2]|0;
      $273 = (($272) + 34|0);
      $274 = HEAP16[$273>>1]|0;
      $275 = $274&65535;
      $276 = $275 & 16;
      $277 = ($276|0)==(0);
      if ($277) {
       $278 = HEAP32[$pLoop>>2]|0;
       $279 = (($278) + 36|0);
       $280 = HEAP32[$279>>2]|0;
       HEAP32[$ws>>2] = $280;
       $281 = HEAP32[$0>>2]|0;
       $282 = (($281) + 38|0);
       $283 = HEAP8[$282>>0]|0;
       $284 = ($283<<24>>24)!=(0);
       if (!($284)) {
        $285 = HEAP32[$ws>>2]|0;
        $286 = $285 & 64;
        $287 = ($286|0)==(0);
        if ($287) {
         $288 = HEAP32[$v>>2]|0;
         $289 = HEAP32[$pTabItem>>2]|0;
         $290 = (($289) + 40|0);
         $291 = HEAP32[$290>>2]|0;
         (_sqlite3VdbeAddOp1($288,61,$291)|0);
        }
       }
       $292 = HEAP32[$ws>>2]|0;
       $293 = $292 & 512;
       $294 = ($293|0)!=(0);
       if ($294) {
        $295 = HEAP32[$ws>>2]|0;
        $296 = $295 & 16640;
        $297 = ($296|0)==(0);
        if ($297) {
         $298 = HEAP32[$pLevel>>2]|0;
         $299 = (($298) + 8|0);
         $300 = HEAP32[$299>>2]|0;
         $301 = HEAP32[$0>>2]|0;
         $302 = (($301) + 60|0);
         $303 = (($302) + 4|0);
         $304 = HEAP32[$303>>2]|0;
         $305 = ($300|0)!=($304|0);
         if ($305) {
          $306 = HEAP32[$v>>2]|0;
          $307 = HEAP32[$pLevel>>2]|0;
          $308 = (($307) + 8|0);
          $309 = HEAP32[$308>>2]|0;
          (_sqlite3VdbeAddOp1($306,61,$309)|0);
         }
        }
       }
      }
     }
    }
    $310 = HEAP32[$pLoop>>2]|0;
    $311 = (($310) + 36|0);
    $312 = HEAP32[$311>>2]|0;
    $313 = $312 & 576;
    $314 = ($313|0)!=(0);
    $315 = HEAP32[$pLoop>>2]|0;
    if ($314) {
     $316 = (($315) + 24|0);
     $317 = (($316) + 4|0);
     $318 = HEAP32[$317>>2]|0;
     HEAP32[$pIdx>>2] = $318;
    } else {
     $319 = (($315) + 36|0);
     $320 = HEAP32[$319>>2]|0;
     $321 = $320 & 8192;
     $322 = ($321|0)!=(0);
     if ($322) {
      $323 = HEAP32[$pLevel>>2]|0;
      $324 = (($323) + 48|0);
      $325 = HEAP32[$324>>2]|0;
      HEAP32[$pIdx>>2] = $325;
     }
    }
    $326 = HEAP32[$pIdx>>2]|0;
    $327 = ($326|0)!=(0|0);
    if ($327) {
     $328 = HEAP32[$db>>2]|0;
     $329 = (($328) + 64|0);
     $330 = HEAP8[$329>>0]|0;
     $331 = ($330<<24>>24)!=(0);
     if (!($331)) {
      $332 = HEAP32[$v>>2]|0;
      $333 = (_sqlite3VdbeCurrentAddr($332)|0);
      HEAP32[$last>>2] = $333;
      $334 = HEAP32[$pLevel>>2]|0;
      $335 = (($334) + 32|0);
      $336 = HEAP32[$335>>2]|0;
      HEAP32[$k>>2] = $336;
      $337 = HEAP32[$v>>2]|0;
      $338 = HEAP32[$k>>2]|0;
      $339 = (_sqlite3VdbeGetOp($337,$338)|0);
      HEAP32[$pOp>>2] = $339;
      while(1) {
       $340 = HEAP32[$k>>2]|0;
       $341 = HEAP32[$last>>2]|0;
       $342 = ($340|0)<($341|0);
       if (!($342)) {
        break L48;
       }
       $343 = HEAP32[$pOp>>2]|0;
       $344 = (($343) + 4|0);
       $345 = HEAP32[$344>>2]|0;
       $346 = HEAP32[$pLevel>>2]|0;
       $347 = (($346) + 4|0);
       $348 = HEAP32[$347>>2]|0;
       $349 = ($345|0)!=($348|0);
       do {
        if (!($349)) {
         $350 = HEAP32[$pOp>>2]|0;
         $351 = HEAP8[$350>>0]|0;
         $352 = $351&255;
         $353 = ($352|0)==(47);
         $354 = HEAP32[$pOp>>2]|0;
         if (!($353)) {
          $387 = HEAP8[$354>>0]|0;
          $388 = $387&255;
          $389 = ($388|0)==(103);
          if (!($389)) {
           break;
          }
          $390 = HEAP32[$pLevel>>2]|0;
          $391 = (($390) + 8|0);
          $392 = HEAP32[$391>>2]|0;
          $393 = HEAP32[$pOp>>2]|0;
          $394 = (($393) + 4|0);
          HEAP32[$394>>2] = $392;
          $395 = HEAP32[$pOp>>2]|0;
          HEAP8[$395>>0] = 112;
          break;
         }
         $355 = (($354) + 8|0);
         $356 = HEAP32[$355>>2]|0;
         HEAP32[$x>>2] = $356;
         $357 = HEAP32[$pTab>>2]|0;
         $358 = (($357) + 44|0);
         $359 = HEAP8[$358>>0]|0;
         $360 = $359&255;
         $361 = $360 & 32;
         $362 = ($361|0)==(0);
         if (!($362)) {
          $363 = HEAP32[$pTab>>2]|0;
          $364 = (_sqlite3PrimaryKeyIndex($363)|0);
          HEAP32[$pPk>>2] = $364;
          $365 = HEAP32[$x>>2]|0;
          $366 = HEAP32[$pPk>>2]|0;
          $367 = (($366) + 4|0);
          $368 = HEAP32[$367>>2]|0;
          $369 = (($368) + ($365<<1)|0);
          $370 = HEAP16[$369>>1]|0;
          $371 = $370 << 16 >> 16;
          HEAP32[$x>>2] = $371;
         }
         $372 = HEAP32[$pIdx>>2]|0;
         $373 = HEAP32[$x>>2]|0;
         $374 = $373&65535;
         $375 = (_sqlite3ColumnOfIndex($372,$374)|0);
         $376 = $375 << 16 >> 16;
         HEAP32[$x>>2] = $376;
         $377 = HEAP32[$x>>2]|0;
         $378 = ($377|0)>=(0);
         if ($378) {
          $379 = HEAP32[$x>>2]|0;
          $380 = HEAP32[$pOp>>2]|0;
          $381 = (($380) + 8|0);
          HEAP32[$381>>2] = $379;
          $382 = HEAP32[$pLevel>>2]|0;
          $383 = (($382) + 8|0);
          $384 = HEAP32[$383>>2]|0;
          $385 = HEAP32[$pOp>>2]|0;
          $386 = (($385) + 4|0);
          HEAP32[$386>>2] = $384;
         }
        }
       } while(0);
       $396 = HEAP32[$k>>2]|0;
       $397 = (($396) + 1)|0;
       HEAP32[$k>>2] = $397;
       $398 = HEAP32[$pOp>>2]|0;
       $399 = (($398) + 20|0);
       HEAP32[$pOp>>2] = $399;
      }
     }
    }
   }
  } while(0);
  $400 = HEAP32[$i>>2]|0;
  $401 = (($400) + 1)|0;
  HEAP32[$i>>2] = $401;
  $402 = HEAP32[$pLevel>>2]|0;
  $403 = (($402) + 72|0);
  HEAP32[$pLevel>>2] = $403;
 }
 $404 = HEAP32[$0>>2]|0;
 $405 = (($404) + 56|0);
 $406 = HEAP32[$405>>2]|0;
 $407 = HEAP32[$pParse>>2]|0;
 $408 = (($407) + 424|0);
 HEAP32[$408>>2] = $406;
 $409 = HEAP32[$db>>2]|0;
 $410 = HEAP32[$0>>2]|0;
 _whereInfoFree($409,$410);
 STACKTOP = sp;return;
}
function _whereInfoFree($db,$pWInfo) {
 $db = $db|0;
 $pWInfo = $pWInfo|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0;
 var $8 = 0, $9 = 0, $p = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $1 = sp + 4|0;
 $p = sp;
 HEAP32[$0>>2] = $db;
 HEAP32[$1>>2] = $pWInfo;
 $2 = HEAP32[$1>>2]|0;
 $3 = ($2|0)!=(0|0);
 if (!($3)) {
  STACKTOP = sp;return;
 }
 $4 = HEAP32[$1>>2]|0;
 $5 = (($4) + 328|0);
 _whereClauseClear($5);
 while(1) {
  $6 = HEAP32[$1>>2]|0;
  $7 = (($6) + 16|0);
  $8 = HEAP32[$7>>2]|0;
  $9 = ($8|0)!=(0|0);
  if (!($9)) {
   break;
  }
  $10 = HEAP32[$1>>2]|0;
  $11 = (($10) + 16|0);
  $12 = HEAP32[$11>>2]|0;
  HEAP32[$p>>2] = $12;
  $13 = HEAP32[$p>>2]|0;
  $14 = (($13) + 48|0);
  $15 = HEAP32[$14>>2]|0;
  $16 = HEAP32[$1>>2]|0;
  $17 = (($16) + 16|0);
  HEAP32[$17>>2] = $15;
  $18 = HEAP32[$0>>2]|0;
  $19 = HEAP32[$p>>2]|0;
  _whereLoopDelete($18,$19);
 }
 $20 = HEAP32[$0>>2]|0;
 $21 = HEAP32[$1>>2]|0;
 _sqlite3DbFree($20,$21);
 STACKTOP = sp;return;
}
function _whereClauseClear($pWC) {
 $pWC = $pWC|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $a = 0, $db = 0, $i = 0;
 var label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $i = sp + 8|0;
 $a = sp + 4|0;
 $db = sp;
 HEAP32[$0>>2] = $pWC;
 $1 = HEAP32[$0>>2]|0;
 $2 = HEAP32[$1>>2]|0;
 $3 = HEAP32[$2>>2]|0;
 $4 = HEAP32[$3>>2]|0;
 HEAP32[$db>>2] = $4;
 $5 = HEAP32[$0>>2]|0;
 $6 = (($5) + 12|0);
 $7 = HEAP32[$6>>2]|0;
 $8 = (($7) - 1)|0;
 HEAP32[$i>>2] = $8;
 $9 = HEAP32[$0>>2]|0;
 $10 = (($9) + 20|0);
 $11 = HEAP32[$10>>2]|0;
 HEAP32[$a>>2] = $11;
 while(1) {
  $12 = HEAP32[$i>>2]|0;
  $13 = ($12|0)>=(0);
  if (!($13)) {
   break;
  }
  $14 = HEAP32[$a>>2]|0;
  $15 = (($14) + 20|0);
  $16 = HEAP8[$15>>0]|0;
  $17 = $16&255;
  $18 = $17 & 1;
  $19 = ($18|0)!=(0);
  if ($19) {
   $20 = HEAP32[$db>>2]|0;
   $21 = HEAP32[$a>>2]|0;
   $22 = HEAP32[$21>>2]|0;
   _sqlite3ExprDelete($20,$22);
  }
  $23 = HEAP32[$a>>2]|0;
  $24 = (($23) + 20|0);
  $25 = HEAP8[$24>>0]|0;
  $26 = $25&255;
  $27 = $26 & 16;
  $28 = ($27|0)!=(0);
  if ($28) {
   $29 = HEAP32[$db>>2]|0;
   $30 = HEAP32[$a>>2]|0;
   $31 = (($30) + 12|0);
   $32 = HEAP32[$31>>2]|0;
   _whereOrInfoDelete($29,$32);
  } else {
   $33 = HEAP32[$a>>2]|0;
   $34 = (($33) + 20|0);
   $35 = HEAP8[$34>>0]|0;
   $36 = $35&255;
   $37 = $36 & 32;
   $38 = ($37|0)!=(0);
   if ($38) {
    $39 = HEAP32[$db>>2]|0;
    $40 = HEAP32[$a>>2]|0;
    $41 = (($40) + 12|0);
    $42 = HEAP32[$41>>2]|0;
    _whereAndInfoDelete($39,$42);
   }
  }
  $43 = HEAP32[$i>>2]|0;
  $44 = (($43) + -1)|0;
  HEAP32[$i>>2] = $44;
  $45 = HEAP32[$a>>2]|0;
  $46 = (($45) + 48|0);
  HEAP32[$a>>2] = $46;
 }
 $47 = HEAP32[$0>>2]|0;
 $48 = (($47) + 20|0);
 $49 = HEAP32[$48>>2]|0;
 $50 = HEAP32[$0>>2]|0;
 $51 = (($50) + 24|0);
 $52 = ($49|0)!=($51|0);
 if (!($52)) {
  STACKTOP = sp;return;
 }
 $53 = HEAP32[$db>>2]|0;
 $54 = HEAP32[$0>>2]|0;
 $55 = (($54) + 20|0);
 $56 = HEAP32[$55>>2]|0;
 _sqlite3DbFree($53,$56);
 STACKTOP = sp;return;
}
function _whereLoopDelete($db,$p) {
 $db = $db|0;
 $p = $p|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $1 = sp;
 HEAP32[$0>>2] = $db;
 HEAP32[$1>>2] = $p;
 $2 = HEAP32[$0>>2]|0;
 $3 = HEAP32[$1>>2]|0;
 _whereLoopClear($2,$3);
 $4 = HEAP32[$0>>2]|0;
 $5 = HEAP32[$1>>2]|0;
 _sqlite3DbFree($4,$5);
 STACKTOP = sp;return;
}
function _whereLoopClear($db,$p) {
 $db = $db|0;
 $p = $p|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $1 = sp;
 HEAP32[$0>>2] = $db;
 HEAP32[$1>>2] = $p;
 $2 = HEAP32[$1>>2]|0;
 $3 = (($2) + 44|0);
 $4 = HEAP32[$3>>2]|0;
 $5 = HEAP32[$1>>2]|0;
 $6 = (($5) + 52|0);
 $7 = ($4|0)!=($6|0);
 if ($7) {
  $8 = HEAP32[$0>>2]|0;
  $9 = HEAP32[$1>>2]|0;
  $10 = (($9) + 44|0);
  $11 = HEAP32[$10>>2]|0;
  _sqlite3DbFree($8,$11);
 }
 $12 = HEAP32[$0>>2]|0;
 $13 = HEAP32[$1>>2]|0;
 _whereLoopClearUnion($12,$13);
 $14 = HEAP32[$1>>2]|0;
 _whereLoopInit($14);
 STACKTOP = sp;return;
}
function _whereLoopClearUnion($db,$p) {
 $db = $db|0;
 $p = $p|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0;
 var sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $1 = sp;
 HEAP32[$0>>2] = $db;
 HEAP32[$1>>2] = $p;
 $2 = HEAP32[$1>>2]|0;
 $3 = (($2) + 36|0);
 $4 = HEAP32[$3>>2]|0;
 $5 = $4 & 17408;
 $6 = ($5|0)!=(0);
 if (!($6)) {
  STACKTOP = sp;return;
 }
 $7 = HEAP32[$1>>2]|0;
 $8 = (($7) + 36|0);
 $9 = HEAP32[$8>>2]|0;
 $10 = $9 & 1024;
 $11 = ($10|0)!=(0);
 if ($11) {
  $12 = HEAP32[$1>>2]|0;
  $13 = (($12) + 24|0);
  $14 = (($13) + 4|0);
  $15 = HEAP8[$14>>0]|0;
  $16 = $15&255;
  $17 = ($16|0)!=(0);
  if ($17) {
   $18 = HEAP32[$1>>2]|0;
   $19 = (($18) + 24|0);
   $20 = (($19) + 8|0);
   $21 = HEAP32[$20>>2]|0;
   _sqlite3_free($21);
   $22 = HEAP32[$1>>2]|0;
   $23 = (($22) + 24|0);
   $24 = (($23) + 4|0);
   HEAP8[$24>>0] = 0;
   $25 = HEAP32[$1>>2]|0;
   $26 = (($25) + 24|0);
   $27 = (($26) + 8|0);
   HEAP32[$27>>2] = 0;
   STACKTOP = sp;return;
  }
 }
 $28 = HEAP32[$1>>2]|0;
 $29 = (($28) + 36|0);
 $30 = HEAP32[$29>>2]|0;
 $31 = $30 & 16384;
 $32 = ($31|0)!=(0);
 if (!($32)) {
  STACKTOP = sp;return;
 }
 $33 = HEAP32[$1>>2]|0;
 $34 = (($33) + 24|0);
 $35 = (($34) + 4|0);
 $36 = HEAP32[$35>>2]|0;
 $37 = ($36|0)!=(0|0);
 if (!($37)) {
  STACKTOP = sp;return;
 }
 $38 = HEAP32[$0>>2]|0;
 $39 = HEAP32[$1>>2]|0;
 $40 = (($39) + 24|0);
 $41 = (($40) + 4|0);
 $42 = HEAP32[$41>>2]|0;
 $43 = (($42) + 16|0);
 $44 = HEAP32[$43>>2]|0;
 _sqlite3DbFree($38,$44);
 $45 = HEAP32[$1>>2]|0;
 $46 = (($45) + 24|0);
 $47 = (($46) + 4|0);
 $48 = HEAP32[$47>>2]|0;
 $49 = (($48) + 40|0);
 $50 = HEAP32[$49>>2]|0;
 _sqlite3KeyInfoUnref($50);
 $51 = HEAP32[$0>>2]|0;
 $52 = HEAP32[$1>>2]|0;
 $53 = (($52) + 24|0);
 $54 = (($53) + 4|0);
 $55 = HEAP32[$54>>2]|0;
 _sqlite3DbFree($51,$55);
 $56 = HEAP32[$1>>2]|0;
 $57 = (($56) + 24|0);
 $58 = (($57) + 4|0);
 HEAP32[$58>>2] = 0;
 STACKTOP = sp;return;
}
function _whereLoopInit($p) {
 $p = $p|0;
 var $0 = 0, $1 = 0, $10 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 HEAP32[$0>>2] = $p;
 $1 = HEAP32[$0>>2]|0;
 $2 = (($1) + 52|0);
 $3 = HEAP32[$0>>2]|0;
 $4 = (($3) + 44|0);
 HEAP32[$4>>2] = $2;
 $5 = HEAP32[$0>>2]|0;
 $6 = (($5) + 40|0);
 HEAP16[$6>>1] = 0;
 $7 = HEAP32[$0>>2]|0;
 $8 = (($7) + 42|0);
 HEAP16[$8>>1] = 4;
 $9 = HEAP32[$0>>2]|0;
 $10 = (($9) + 36|0);
 HEAP32[$10>>2] = 0;
 STACKTOP = sp;return;
}
function _whereOrInfoDelete($db,$p) {
 $db = $db|0;
 $p = $p|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $1 = sp;
 HEAP32[$0>>2] = $db;
 HEAP32[$1>>2] = $p;
 $2 = HEAP32[$1>>2]|0;
 _whereClauseClear($2);
 $3 = HEAP32[$0>>2]|0;
 $4 = HEAP32[$1>>2]|0;
 _sqlite3DbFree($3,$4);
 STACKTOP = sp;return;
}
function _whereAndInfoDelete($db,$p) {
 $db = $db|0;
 $p = $p|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $1 = sp;
 HEAP32[$0>>2] = $db;
 HEAP32[$1>>2] = $p;
 $2 = HEAP32[$1>>2]|0;
 _whereClauseClear($2);
 $3 = HEAP32[$0>>2]|0;
 $4 = HEAP32[$1>>2]|0;
 _sqlite3DbFree($3,$4);
 STACKTOP = sp;return;
}
function _whereClauseInit($pWC,$pWInfo) {
 $pWC = $pWC|0;
 $pWInfo = $pWInfo|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $1 = sp;
 HEAP32[$0>>2] = $pWC;
 HEAP32[$1>>2] = $pWInfo;
 $2 = HEAP32[$1>>2]|0;
 $3 = HEAP32[$0>>2]|0;
 HEAP32[$3>>2] = $2;
 $4 = HEAP32[$0>>2]|0;
 $5 = (($4) + 4|0);
 HEAP32[$5>>2] = 0;
 $6 = HEAP32[$0>>2]|0;
 $7 = (($6) + 12|0);
 HEAP32[$7>>2] = 0;
 $8 = HEAP32[$0>>2]|0;
 $9 = (($8) + 16|0);
 HEAP32[$9>>2] = 8;
 $10 = HEAP32[$0>>2]|0;
 $11 = (($10) + 24|0);
 $12 = HEAP32[$0>>2]|0;
 $13 = (($12) + 20|0);
 HEAP32[$13>>2] = $11;
 STACKTOP = sp;return;
}
function _whereSplit($pWC,$pExpr,$op) {
 $pWC = $pWC|0;
 $pExpr = $pExpr|0;
 $op = $op|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $3 = 0, $4 = 0, $5 = 0;
 var $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $1 = sp;
 $2 = sp + 8|0;
 HEAP32[$0>>2] = $pWC;
 HEAP32[$1>>2] = $pExpr;
 HEAP8[$2>>0] = $op;
 $3 = HEAP8[$2>>0]|0;
 $4 = HEAP32[$0>>2]|0;
 $5 = (($4) + 8|0);
 HEAP8[$5>>0] = $3;
 $6 = HEAP32[$1>>2]|0;
 $7 = ($6|0)==(0|0);
 if ($7) {
  STACKTOP = sp;return;
 }
 $8 = HEAP32[$1>>2]|0;
 $9 = HEAP8[$8>>0]|0;
 $10 = $9&255;
 $11 = HEAP8[$2>>0]|0;
 $12 = $11&255;
 $13 = ($10|0)!=($12|0);
 $14 = HEAP32[$0>>2]|0;
 $15 = HEAP32[$1>>2]|0;
 if ($13) {
  (_whereClauseInsert($14,$15,0)|0);
  STACKTOP = sp;return;
 } else {
  $16 = (($15) + 12|0);
  $17 = HEAP32[$16>>2]|0;
  $18 = HEAP8[$2>>0]|0;
  _whereSplit($14,$17,$18);
  $19 = HEAP32[$0>>2]|0;
  $20 = HEAP32[$1>>2]|0;
  $21 = (($20) + 16|0);
  $22 = HEAP32[$21>>2]|0;
  $23 = HEAP8[$2>>0]|0;
  _whereSplit($19,$22,$23);
  STACKTOP = sp;return;
 }
}
function _createMask($pMaskSet,$iCursor) {
 $pMaskSet = $pMaskSet|0;
 $iCursor = $iCursor|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $1 = sp;
 HEAP32[$0>>2] = $pMaskSet;
 HEAP32[$1>>2] = $iCursor;
 $2 = HEAP32[$1>>2]|0;
 $3 = HEAP32[$0>>2]|0;
 $4 = HEAP32[$3>>2]|0;
 $5 = (($4) + 1)|0;
 HEAP32[$3>>2] = $5;
 $6 = HEAP32[$0>>2]|0;
 $7 = (($6) + 4|0);
 $8 = (($7) + ($4<<2)|0);
 HEAP32[$8>>2] = $2;
 STACKTOP = sp;return;
}
function _exprAnalyzeAll($pTabList,$pWC) {
 $pTabList = $pTabList|0;
 $pWC = $pWC|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $i = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $1 = sp + 4|0;
 $i = sp;
 HEAP32[$0>>2] = $pTabList;
 HEAP32[$1>>2] = $pWC;
 $2 = HEAP32[$1>>2]|0;
 $3 = (($2) + 12|0);
 $4 = HEAP32[$3>>2]|0;
 $5 = (($4) - 1)|0;
 HEAP32[$i>>2] = $5;
 while(1) {
  $6 = HEAP32[$i>>2]|0;
  $7 = ($6|0)>=(0);
  if (!($7)) {
   break;
  }
  $8 = HEAP32[$0>>2]|0;
  $9 = HEAP32[$1>>2]|0;
  $10 = HEAP32[$i>>2]|0;
  _exprAnalyze($8,$9,$10);
  $11 = HEAP32[$i>>2]|0;
  $12 = (($11) + -1)|0;
  HEAP32[$i>>2] = $12;
 }
 STACKTOP = sp;return;
}
function _isDistinctRedundant($pParse,$pTabList,$pWC,$pDistinct) {
 $pParse = $pParse|0;
 $pTabList = $pTabList|0;
 $pWC = $pWC|0;
 $pDistinct = $pDistinct|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0;
 var $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0;
 var $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0;
 var $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0;
 var $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0;
 var $98 = 0, $99 = 0, $i = 0, $iBase = 0, $iCol = 0, $iIdxCol = 0, $p = 0, $pIdx = 0, $pTab = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $0 = sp + 40|0;
 $1 = sp + 36|0;
 $2 = sp + 32|0;
 $3 = sp + 28|0;
 $4 = sp + 24|0;
 $pTab = sp + 20|0;
 $pIdx = sp + 16|0;
 $i = sp + 12|0;
 $iBase = sp + 8|0;
 $p = sp + 4|0;
 $iCol = sp + 44|0;
 $iIdxCol = sp;
 HEAP32[$1>>2] = $pParse;
 HEAP32[$2>>2] = $pTabList;
 HEAP32[$3>>2] = $pWC;
 HEAP32[$4>>2] = $pDistinct;
 $5 = HEAP32[$2>>2]|0;
 $6 = HEAP32[$5>>2]|0;
 $7 = ($6|0)!=(1);
 if ($7) {
  HEAP32[$0>>2] = 0;
  $101 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($101|0);
 }
 $8 = HEAP32[$2>>2]|0;
 $9 = (($8) + 8|0);
 $10 = (($9) + 40|0);
 $11 = HEAP32[$10>>2]|0;
 HEAP32[$iBase>>2] = $11;
 $12 = HEAP32[$2>>2]|0;
 $13 = (($12) + 8|0);
 $14 = (($13) + 16|0);
 $15 = HEAP32[$14>>2]|0;
 HEAP32[$pTab>>2] = $15;
 HEAP32[$i>>2] = 0;
 while(1) {
  $16 = HEAP32[$i>>2]|0;
  $17 = HEAP32[$4>>2]|0;
  $18 = HEAP32[$17>>2]|0;
  $19 = ($16|0)<($18|0);
  if (!($19)) {
   break;
  }
  $20 = HEAP32[$i>>2]|0;
  $21 = HEAP32[$4>>2]|0;
  $22 = (($21) + 4|0);
  $23 = HEAP32[$22>>2]|0;
  $24 = (($23) + (($20*20)|0)|0);
  $25 = HEAP32[$24>>2]|0;
  $26 = (_sqlite3ExprSkipCollate($25)|0);
  HEAP32[$p>>2] = $26;
  $27 = HEAP32[$p>>2]|0;
  $28 = HEAP8[$27>>0]|0;
  $29 = $28&255;
  $30 = ($29|0)==(154);
  if ($30) {
   $31 = HEAP32[$p>>2]|0;
   $32 = (($31) + 28|0);
   $33 = HEAP32[$32>>2]|0;
   $34 = HEAP32[$iBase>>2]|0;
   $35 = ($33|0)==($34|0);
   if ($35) {
    $36 = HEAP32[$p>>2]|0;
    $37 = (($36) + 32|0);
    $38 = HEAP16[$37>>1]|0;
    $39 = $38 << 16 >> 16;
    $40 = ($39|0)<(0);
    if ($40) {
     label = 8;
     break;
    }
   }
  }
  $41 = HEAP32[$i>>2]|0;
  $42 = (($41) + 1)|0;
  HEAP32[$i>>2] = $42;
 }
 if ((label|0) == 8) {
  HEAP32[$0>>2] = 1;
  $101 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($101|0);
 }
 $43 = HEAP32[$pTab>>2]|0;
 $44 = (($43) + 8|0);
 $45 = HEAP32[$44>>2]|0;
 HEAP32[$pIdx>>2] = $45;
 while(1) {
  $46 = HEAP32[$pIdx>>2]|0;
  $47 = ($46|0)!=(0|0);
  if (!($47)) {
   label = 22;
   break;
  }
  $48 = HEAP32[$pIdx>>2]|0;
  $49 = (($48) + 54|0);
  $50 = HEAP8[$49>>0]|0;
  $51 = $50&255;
  $52 = ($51|0)!=(0);
  if ($52) {
   HEAP32[$i>>2] = 0;
   while(1) {
    $53 = HEAP32[$i>>2]|0;
    $54 = HEAP32[$pIdx>>2]|0;
    $55 = (($54) + 50|0);
    $56 = HEAP16[$55>>1]|0;
    $57 = $56&65535;
    $58 = ($53|0)<($57|0);
    if (!($58)) {
     break;
    }
    $59 = HEAP32[$i>>2]|0;
    $60 = HEAP32[$pIdx>>2]|0;
    $61 = (($60) + 4|0);
    $62 = HEAP32[$61>>2]|0;
    $63 = (($62) + ($59<<1)|0);
    $64 = HEAP16[$63>>1]|0;
    HEAP16[$iCol>>1] = $64;
    $65 = HEAP32[$3>>2]|0;
    $66 = HEAP32[$iBase>>2]|0;
    $67 = HEAP16[$iCol>>1]|0;
    $68 = $67 << 16 >> 16;
    $69 = HEAP32[$pIdx>>2]|0;
    $70 = (_findTerm($65,$66,$68,-1,-1,2,$69)|0);
    $71 = (0|0)==($70|0);
    if ($71) {
     $72 = HEAP32[$1>>2]|0;
     $73 = HEAP32[$4>>2]|0;
     $74 = HEAP32[$iBase>>2]|0;
     $75 = HEAP32[$pIdx>>2]|0;
     $76 = HEAP32[$i>>2]|0;
     $77 = (_findIndexCol($72,$73,$74,$75,$76)|0);
     HEAP32[$iIdxCol>>2] = $77;
     $78 = HEAP32[$iIdxCol>>2]|0;
     $79 = ($78|0)<(0);
     if ($79) {
      break;
     }
     $80 = HEAP16[$iCol>>1]|0;
     $81 = $80 << 16 >> 16;
     $82 = HEAP32[$pTab>>2]|0;
     $83 = (($82) + 4|0);
     $84 = HEAP32[$83>>2]|0;
     $85 = (($84) + (($81*24)|0)|0);
     $86 = (($85) + 20|0);
     $87 = HEAP8[$86>>0]|0;
     $88 = $87&255;
     $89 = ($88|0)==(0);
     if ($89) {
      break;
     }
    }
    $90 = HEAP32[$i>>2]|0;
    $91 = (($90) + 1)|0;
    HEAP32[$i>>2] = $91;
   }
   $92 = HEAP32[$i>>2]|0;
   $93 = HEAP32[$pIdx>>2]|0;
   $94 = (($93) + 50|0);
   $95 = HEAP16[$94>>1]|0;
   $96 = $95&65535;
   $97 = ($92|0)==($96|0);
   if ($97) {
    label = 20;
    break;
   }
  }
  $98 = HEAP32[$pIdx>>2]|0;
  $99 = (($98) + 20|0);
  $100 = HEAP32[$99>>2]|0;
  HEAP32[$pIdx>>2] = $100;
 }
 if ((label|0) == 20) {
  HEAP32[$0>>2] = 1;
  $101 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($101|0);
 }
 else if ((label|0) == 22) {
  HEAP32[$0>>2] = 0;
  $101 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($101|0);
 }
 return 0|0;
}
function _whereShortCut($pBuilder) {
 $pBuilder = $pBuilder|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0;
 var $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0, $188 = 0;
 var $189 = 0, $19 = 0, $190 = 0, $191 = 0, $192 = 0, $193 = 0, $194 = 0, $195 = 0, $196 = 0, $197 = 0, $198 = 0, $199 = 0, $2 = 0, $20 = 0, $200 = 0, $201 = 0, $202 = 0, $203 = 0, $21 = 0, $22 = 0;
 var $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0;
 var $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0;
 var $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0;
 var $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0;
 var $96 = 0, $97 = 0, $98 = 0, $99 = 0, $iCur = 0, $j = 0, $pIdx = 0, $pItem = 0, $pLoop = 0, $pTab = 0, $pTerm = 0, $pWC = 0, $pWInfo = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $0 = sp + 40|0;
 $1 = sp + 36|0;
 $pWInfo = sp + 32|0;
 $pItem = sp + 28|0;
 $pWC = sp + 24|0;
 $pTerm = sp + 20|0;
 $pLoop = sp + 16|0;
 $iCur = sp + 12|0;
 $j = sp + 8|0;
 $pTab = sp + 4|0;
 $pIdx = sp;
 HEAP32[$1>>2] = $pBuilder;
 $2 = HEAP32[$1>>2]|0;
 $3 = HEAP32[$2>>2]|0;
 HEAP32[$pWInfo>>2] = $3;
 $4 = HEAP32[$pWInfo>>2]|0;
 $5 = (($4) + 34|0);
 $6 = HEAP16[$5>>1]|0;
 $7 = $6&65535;
 $8 = $7 & 32;
 $9 = ($8|0)!=(0);
 if ($9) {
  HEAP32[$0>>2] = 0;
  $203 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($203|0);
 }
 $10 = HEAP32[$pWInfo>>2]|0;
 $11 = (($10) + 4|0);
 $12 = HEAP32[$11>>2]|0;
 $13 = (($12) + 8|0);
 HEAP32[$pItem>>2] = $13;
 $14 = HEAP32[$pItem>>2]|0;
 $15 = (($14) + 16|0);
 $16 = HEAP32[$15>>2]|0;
 HEAP32[$pTab>>2] = $16;
 $17 = HEAP32[$pTab>>2]|0;
 $18 = (($17) + 44|0);
 $19 = HEAP8[$18>>0]|0;
 $20 = $19&255;
 $21 = $20 & 16;
 $22 = ($21|0)!=(0);
 if ($22) {
  HEAP32[$0>>2] = 0;
  $203 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($203|0);
 }
 $23 = HEAP32[$pItem>>2]|0;
 $24 = (($23) + 64|0);
 $25 = HEAP32[$24>>2]|0;
 $26 = ($25|0)!=(0|0);
 if ($26) {
  HEAP32[$0>>2] = 0;
  $203 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($203|0);
 }
 $27 = HEAP32[$pItem>>2]|0;
 $28 = (($27) + 40|0);
 $29 = HEAP32[$28>>2]|0;
 HEAP32[$iCur>>2] = $29;
 $30 = HEAP32[$pWInfo>>2]|0;
 $31 = (($30) + 328|0);
 HEAP32[$pWC>>2] = $31;
 $32 = HEAP32[$1>>2]|0;
 $33 = (($32) + 12|0);
 $34 = HEAP32[$33>>2]|0;
 HEAP32[$pLoop>>2] = $34;
 $35 = HEAP32[$pLoop>>2]|0;
 $36 = (($35) + 36|0);
 HEAP32[$36>>2] = 0;
 $37 = HEAP32[$pLoop>>2]|0;
 $38 = (($37) + 24|0);
 $39 = (($38) + 2|0);
 HEAP16[$39>>1] = 0;
 $40 = HEAP32[$pWC>>2]|0;
 $41 = HEAP32[$iCur>>2]|0;
 $42 = (_findTerm($40,$41,-1,0,0,2,0)|0);
 HEAP32[$pTerm>>2] = $42;
 $43 = HEAP32[$pTerm>>2]|0;
 $44 = ($43|0)!=(0|0);
 L13: do {
  if ($44) {
   $45 = HEAP32[$pLoop>>2]|0;
   $46 = (($45) + 36|0);
   HEAP32[$46>>2] = 4353;
   $47 = HEAP32[$pTerm>>2]|0;
   $48 = HEAP32[$pLoop>>2]|0;
   $49 = (($48) + 44|0);
   $50 = HEAP32[$49>>2]|0;
   HEAP32[$50>>2] = $47;
   $51 = HEAP32[$pLoop>>2]|0;
   $52 = (($51) + 40|0);
   HEAP16[$52>>1] = 1;
   $53 = HEAP32[$pLoop>>2]|0;
   $54 = (($53) + 24|0);
   HEAP16[$54>>1] = 1;
   $55 = HEAP32[$pLoop>>2]|0;
   $56 = (($55) + 20|0);
   HEAP16[$56>>1] = 33;
  } else {
   $57 = HEAP32[$pTab>>2]|0;
   $58 = (($57) + 8|0);
   $59 = HEAP32[$58>>2]|0;
   HEAP32[$pIdx>>2] = $59;
   while(1) {
    $60 = HEAP32[$pIdx>>2]|0;
    $61 = ($60|0)!=(0|0);
    if (!($61)) {
     break L13;
    }
    $62 = HEAP32[$pIdx>>2]|0;
    $63 = (($62) + 54|0);
    $64 = HEAP8[$63>>0]|0;
    $65 = $64&255;
    $66 = ($65|0)!=(0);
    if ($66) {
     $67 = HEAP32[$pIdx>>2]|0;
     $68 = (($67) + 36|0);
     $69 = HEAP32[$68>>2]|0;
     $70 = ($69|0)!=(0|0);
     if (!($70)) {
      $71 = HEAP32[$pIdx>>2]|0;
      $72 = (($71) + 50|0);
      $73 = HEAP16[$72>>1]|0;
      $74 = $73&65535;
      $75 = ($74|0)>(4);
      if (!($75)) {
       HEAP32[$j>>2] = 0;
       while(1) {
        $76 = HEAP32[$j>>2]|0;
        $77 = HEAP32[$pIdx>>2]|0;
        $78 = (($77) + 50|0);
        $79 = HEAP16[$78>>1]|0;
        $80 = $79&65535;
        $81 = ($76|0)<($80|0);
        if (!($81)) {
         break;
        }
        $82 = HEAP32[$pWC>>2]|0;
        $83 = HEAP32[$iCur>>2]|0;
        $84 = HEAP32[$j>>2]|0;
        $85 = HEAP32[$pIdx>>2]|0;
        $86 = (($85) + 4|0);
        $87 = HEAP32[$86>>2]|0;
        $88 = (($87) + ($84<<1)|0);
        $89 = HEAP16[$88>>1]|0;
        $90 = $89 << 16 >> 16;
        $91 = HEAP32[$pIdx>>2]|0;
        $92 = (_findTerm($82,$83,$90,0,0,2,$91)|0);
        HEAP32[$pTerm>>2] = $92;
        $93 = HEAP32[$pTerm>>2]|0;
        $94 = ($93|0)==(0|0);
        if ($94) {
         break;
        }
        $95 = HEAP32[$pTerm>>2]|0;
        $96 = HEAP32[$j>>2]|0;
        $97 = HEAP32[$pLoop>>2]|0;
        $98 = (($97) + 44|0);
        $99 = HEAP32[$98>>2]|0;
        $100 = (($99) + ($96<<2)|0);
        HEAP32[$100>>2] = $95;
        $101 = HEAP32[$j>>2]|0;
        $102 = (($101) + 1)|0;
        HEAP32[$j>>2] = $102;
       }
       $103 = HEAP32[$j>>2]|0;
       $104 = HEAP32[$pIdx>>2]|0;
       $105 = (($104) + 50|0);
       $106 = HEAP16[$105>>1]|0;
       $107 = $106&65535;
       $108 = ($103|0)!=($107|0);
       if (!($108)) {
        break;
       }
      }
     }
    }
    $154 = HEAP32[$pIdx>>2]|0;
    $155 = (($154) + 20|0);
    $156 = HEAP32[$155>>2]|0;
    HEAP32[$pIdx>>2] = $156;
   }
   $109 = HEAP32[$pLoop>>2]|0;
   $110 = (($109) + 36|0);
   HEAP32[$110>>2] = 4609;
   $111 = HEAP32[$pIdx>>2]|0;
   $112 = (($111) + 55|0);
   $113 = HEAP8[$112>>0]|0;
   $114 = ($113&255) >>> 5;
   $115 = $114 & 1;
   $116 = $115&255;
   $117 = ($116|0)!=(0);
   if ($117) {
    label = 21;
   } else {
    $118 = HEAP32[$pItem>>2]|0;
    $119 = (($118) + 56|0);
    $120 = $119;
    $121 = $120;
    $122 = HEAP32[$121>>2]|0;
    $123 = (($120) + 4)|0;
    $124 = $123;
    $125 = HEAP32[$124>>2]|0;
    $126 = HEAP32[$pIdx>>2]|0;
    $127 = (_columnsInIndex($126)|0);
    $128 = tempRet0;
    $129 = $127 ^ -1;
    $130 = $128 ^ -1;
    $131 = $122 & $129;
    $132 = $125 & $130;
    $133 = ($131|0)==(0);
    $134 = ($132|0)==(0);
    $135 = $133 & $134;
    if ($135) {
     label = 21;
    }
   }
   if ((label|0) == 21) {
    $136 = HEAP32[$pLoop>>2]|0;
    $137 = (($136) + 36|0);
    $138 = HEAP32[$137>>2]|0;
    $139 = $138 | 64;
    HEAP32[$137>>2] = $139;
   }
   $140 = HEAP32[$j>>2]|0;
   $141 = $140&65535;
   $142 = HEAP32[$pLoop>>2]|0;
   $143 = (($142) + 40|0);
   HEAP16[$143>>1] = $141;
   $144 = HEAP32[$j>>2]|0;
   $145 = $144&65535;
   $146 = HEAP32[$pLoop>>2]|0;
   $147 = (($146) + 24|0);
   HEAP16[$147>>1] = $145;
   $148 = HEAP32[$pIdx>>2]|0;
   $149 = HEAP32[$pLoop>>2]|0;
   $150 = (($149) + 24|0);
   $151 = (($150) + 4|0);
   HEAP32[$151>>2] = $148;
   $152 = HEAP32[$pLoop>>2]|0;
   $153 = (($152) + 20|0);
   HEAP16[$153>>1] = 39;
  }
 } while(0);
 $157 = HEAP32[$pLoop>>2]|0;
 $158 = (($157) + 36|0);
 $159 = HEAP32[$158>>2]|0;
 $160 = ($159|0)!=(0);
 if (!($160)) {
  HEAP32[$0>>2] = 0;
  $203 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($203|0);
 }
 $161 = HEAP32[$pLoop>>2]|0;
 $162 = (($161) + 22|0);
 HEAP16[$162>>1] = 1;
 $163 = HEAP32[$pLoop>>2]|0;
 $164 = HEAP32[$pWInfo>>2]|0;
 $165 = (($164) + 736|0);
 $166 = (($165) + 56|0);
 HEAP32[$166>>2] = $163;
 $167 = HEAP32[$pWInfo>>2]|0;
 $168 = (($167) + 68|0);
 $169 = HEAP32[$iCur>>2]|0;
 $170 = (_getMask($168,$169)|0);
 $171 = tempRet0;
 $172 = HEAP32[$pLoop>>2]|0;
 $173 = (($172) + 8|0);
 $174 = $173;
 $175 = $174;
 HEAP32[$175>>2] = $170;
 $176 = (($174) + 4)|0;
 $177 = $176;
 HEAP32[$177>>2] = $171;
 $178 = HEAP32[$iCur>>2]|0;
 $179 = HEAP32[$pWInfo>>2]|0;
 $180 = (($179) + 736|0);
 $181 = (($180) + 4|0);
 HEAP32[$181>>2] = $178;
 $182 = HEAP32[$pWInfo>>2]|0;
 $183 = (($182) + 32|0);
 HEAP16[$183>>1] = 1;
 $184 = HEAP32[$pWInfo>>2]|0;
 $185 = (($184) + 8|0);
 $186 = HEAP32[$185>>2]|0;
 $187 = ($186|0)!=(0|0);
 if ($187) {
  $188 = HEAP32[$pWInfo>>2]|0;
  $189 = (($188) + 8|0);
  $190 = HEAP32[$189>>2]|0;
  $191 = HEAP32[$190>>2]|0;
  $192 = $191&255;
  $193 = HEAP32[$pWInfo>>2]|0;
  $194 = (($193) + 36|0);
  HEAP8[$194>>0] = $192;
 }
 $195 = HEAP32[$pWInfo>>2]|0;
 $196 = (($195) + 34|0);
 $197 = HEAP16[$196>>1]|0;
 $198 = $197&65535;
 $199 = $198 & 1024;
 $200 = ($199|0)!=(0);
 if ($200) {
  $201 = HEAP32[$pWInfo>>2]|0;
  $202 = (($201) + 40|0);
  HEAP8[$202>>0] = 1;
 }
 HEAP32[$0>>2] = 1;
 $203 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($203|0);
}
function _whereLoopAddAll($pBuilder) {
 $pBuilder = $pBuilder|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0;
 var $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0;
 var $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0;
 var $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0;
 var $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0;
 var $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $db = 0, $iTab = 0, $mExtra = 0, $mPrior = 0, $nTabList = 0, $pItem = 0, $pNew = 0, $pTabList = 0, $pWInfo = 0, $priorJoinType = 0;
 var $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 64|0;
 $0 = sp + 48|0;
 $pWInfo = sp + 44|0;
 $mExtra = sp + 8|0;
 $mPrior = sp;
 $iTab = sp + 40|0;
 $pTabList = sp + 36|0;
 $pItem = sp + 32|0;
 $db = sp + 28|0;
 $nTabList = sp + 24|0;
 $rc = sp + 20|0;
 $priorJoinType = sp + 52|0;
 $pNew = sp + 16|0;
 HEAP32[$0>>2] = $pBuilder;
 $1 = HEAP32[$0>>2]|0;
 $2 = HEAP32[$1>>2]|0;
 HEAP32[$pWInfo>>2] = $2;
 $3 = $mExtra;
 $4 = $3;
 HEAP32[$4>>2] = 0;
 $5 = (($3) + 4)|0;
 $6 = $5;
 HEAP32[$6>>2] = 0;
 $7 = $mPrior;
 $8 = $7;
 HEAP32[$8>>2] = 0;
 $9 = (($7) + 4)|0;
 $10 = $9;
 HEAP32[$10>>2] = 0;
 $11 = HEAP32[$pWInfo>>2]|0;
 $12 = (($11) + 4|0);
 $13 = HEAP32[$12>>2]|0;
 HEAP32[$pTabList>>2] = $13;
 $14 = HEAP32[$pWInfo>>2]|0;
 $15 = HEAP32[$14>>2]|0;
 $16 = HEAP32[$15>>2]|0;
 HEAP32[$db>>2] = $16;
 $17 = HEAP32[$pWInfo>>2]|0;
 $18 = (($17) + 41|0);
 $19 = HEAP8[$18>>0]|0;
 $20 = $19&255;
 HEAP32[$nTabList>>2] = $20;
 HEAP32[$rc>>2] = 0;
 HEAP8[$priorJoinType>>0] = 0;
 $21 = HEAP32[$0>>2]|0;
 $22 = (($21) + 12|0);
 $23 = HEAP32[$22>>2]|0;
 HEAP32[$pNew>>2] = $23;
 $24 = HEAP32[$pNew>>2]|0;
 _whereLoopInit($24);
 HEAP32[$iTab>>2] = 0;
 $25 = HEAP32[$pTabList>>2]|0;
 $26 = (($25) + 8|0);
 HEAP32[$pItem>>2] = $26;
 while(1) {
  $27 = HEAP32[$iTab>>2]|0;
  $28 = HEAP32[$nTabList>>2]|0;
  $29 = ($27|0)<($28|0);
  if (!($29)) {
   label = 13;
   break;
  }
  $30 = HEAP32[$iTab>>2]|0;
  $31 = $30&255;
  $32 = HEAP32[$pNew>>2]|0;
  $33 = (($32) + 16|0);
  HEAP8[$33>>0] = $31;
  $34 = HEAP32[$pWInfo>>2]|0;
  $35 = (($34) + 68|0);
  $36 = HEAP32[$pItem>>2]|0;
  $37 = (($36) + 40|0);
  $38 = HEAP32[$37>>2]|0;
  $39 = (_getMask($35,$38)|0);
  $40 = tempRet0;
  $41 = HEAP32[$pNew>>2]|0;
  $42 = (($41) + 8|0);
  $43 = $42;
  $44 = $43;
  HEAP32[$44>>2] = $39;
  $45 = (($43) + 4)|0;
  $46 = $45;
  HEAP32[$46>>2] = $40;
  $47 = HEAP32[$pItem>>2]|0;
  $48 = (($47) + 36|0);
  $49 = HEAP8[$48>>0]|0;
  $50 = $49&255;
  $51 = HEAP8[$priorJoinType>>0]|0;
  $52 = $51&255;
  $53 = $50 | $52;
  $54 = $53 & 10;
  $55 = ($54|0)!=(0);
  if ($55) {
   $56 = $mPrior;
   $57 = $56;
   $58 = HEAP32[$57>>2]|0;
   $59 = (($56) + 4)|0;
   $60 = $59;
   $61 = HEAP32[$60>>2]|0;
   $62 = $mExtra;
   $63 = $62;
   HEAP32[$63>>2] = $58;
   $64 = (($62) + 4)|0;
   $65 = $64;
   HEAP32[$65>>2] = $61;
  }
  $66 = HEAP32[$pItem>>2]|0;
  $67 = (($66) + 36|0);
  $68 = HEAP8[$67>>0]|0;
  HEAP8[$priorJoinType>>0] = $68;
  $69 = HEAP32[$pItem>>2]|0;
  $70 = (($69) + 16|0);
  $71 = HEAP32[$70>>2]|0;
  $72 = (($71) + 44|0);
  $73 = HEAP8[$72>>0]|0;
  $74 = $73&255;
  $75 = $74 & 16;
  $76 = ($75|0)!=(0);
  $77 = HEAP32[$0>>2]|0;
  $78 = $mExtra;
  $79 = $78;
  $80 = HEAP32[$79>>2]|0;
  $81 = (($78) + 4)|0;
  $82 = $81;
  $83 = HEAP32[$82>>2]|0;
  if ($76) {
   $84 = (_whereLoopAddVirtual($77,$80,$83)|0);
   HEAP32[$rc>>2] = $84;
  } else {
   $85 = (_whereLoopAddBtree($77,$80,$83)|0);
   HEAP32[$rc>>2] = $85;
  }
  $86 = HEAP32[$rc>>2]|0;
  $87 = ($86|0)==(0);
  if ($87) {
   $88 = HEAP32[$0>>2]|0;
   $89 = $mExtra;
   $90 = $89;
   $91 = HEAP32[$90>>2]|0;
   $92 = (($89) + 4)|0;
   $93 = $92;
   $94 = HEAP32[$93>>2]|0;
   $95 = (_whereLoopAddOr($88,$91,$94)|0);
   HEAP32[$rc>>2] = $95;
  }
  $96 = HEAP32[$pNew>>2]|0;
  $97 = (($96) + 8|0);
  $98 = $97;
  $99 = $98;
  $100 = HEAP32[$99>>2]|0;
  $101 = (($98) + 4)|0;
  $102 = $101;
  $103 = HEAP32[$102>>2]|0;
  $104 = $mPrior;
  $105 = $104;
  $106 = HEAP32[$105>>2]|0;
  $107 = (($104) + 4)|0;
  $108 = $107;
  $109 = HEAP32[$108>>2]|0;
  $110 = $106 | $100;
  $111 = $109 | $103;
  $112 = $mPrior;
  $113 = $112;
  HEAP32[$113>>2] = $110;
  $114 = (($112) + 4)|0;
  $115 = $114;
  HEAP32[$115>>2] = $111;
  $116 = HEAP32[$rc>>2]|0;
  $117 = ($116|0)!=(0);
  if ($117) {
   label = 13;
   break;
  }
  $118 = HEAP32[$db>>2]|0;
  $119 = (($118) + 64|0);
  $120 = HEAP8[$119>>0]|0;
  $121 = $120&255;
  $122 = ($121|0)!=(0);
  if ($122) {
   label = 13;
   break;
  }
  $123 = HEAP32[$iTab>>2]|0;
  $124 = (($123) + 1)|0;
  HEAP32[$iTab>>2] = $124;
  $125 = HEAP32[$pItem>>2]|0;
  $126 = (($125) + 72|0);
  HEAP32[$pItem>>2] = $126;
 }
 if ((label|0) == 13) {
  $127 = HEAP32[$db>>2]|0;
  $128 = HEAP32[$pNew>>2]|0;
  _whereLoopClear($127,$128);
  $129 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($129|0);
 }
 return 0|0;
}
function _wherePathSolver($pWInfo,$nRowEst) {
 $pWInfo = $pWInfo|0;
 $nRowEst = $nRowEst|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0;
 var $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0, $188 = 0;
 var $189 = 0, $19 = 0, $190 = 0, $191 = 0, $192 = 0, $193 = 0, $194 = 0, $195 = 0, $196 = 0, $197 = 0, $198 = 0, $199 = 0, $2 = 0, $20 = 0, $200 = 0, $201 = 0, $202 = 0, $203 = 0, $204 = 0, $205 = 0;
 var $206 = 0, $207 = 0, $208 = 0, $209 = 0, $21 = 0, $210 = 0, $211 = 0, $212 = 0, $213 = 0, $214 = 0, $215 = 0, $216 = 0, $217 = 0, $218 = 0, $219 = 0, $22 = 0, $220 = 0, $221 = 0, $222 = 0, $223 = 0;
 var $224 = 0, $225 = 0, $226 = 0, $227 = 0, $228 = 0, $229 = 0, $23 = 0, $230 = 0, $231 = 0, $232 = 0, $233 = 0, $234 = 0, $235 = 0, $236 = 0, $237 = 0, $238 = 0, $239 = 0, $24 = 0, $240 = 0, $241 = 0;
 var $242 = 0, $243 = 0, $244 = 0, $245 = 0, $246 = 0, $247 = 0, $248 = 0, $249 = 0, $25 = 0, $250 = 0, $251 = 0, $252 = 0, $253 = 0, $254 = 0, $255 = 0, $256 = 0, $257 = 0, $258 = 0, $259 = 0, $26 = 0;
 var $260 = 0, $261 = 0, $262 = 0, $263 = 0, $264 = 0, $265 = 0, $266 = 0, $267 = 0, $268 = 0, $269 = 0, $27 = 0, $270 = 0, $271 = 0, $272 = 0, $273 = 0, $274 = 0, $275 = 0, $276 = 0, $277 = 0, $278 = 0;
 var $279 = 0, $28 = 0, $280 = 0, $281 = 0, $282 = 0, $283 = 0, $284 = 0, $285 = 0, $286 = 0, $287 = 0, $288 = 0, $289 = 0, $29 = 0, $290 = 0, $291 = 0, $292 = 0, $293 = 0, $294 = 0, $295 = 0, $296 = 0;
 var $297 = 0, $298 = 0, $299 = 0, $3 = 0, $30 = 0, $300 = 0, $301 = 0, $302 = 0, $303 = 0, $304 = 0, $305 = 0, $306 = 0, $307 = 0, $308 = 0, $309 = 0, $31 = 0, $310 = 0, $311 = 0, $312 = 0, $313 = 0;
 var $314 = 0, $315 = 0, $316 = 0, $317 = 0, $318 = 0, $319 = 0, $32 = 0, $320 = 0, $321 = 0, $322 = 0, $323 = 0, $324 = 0, $325 = 0, $326 = 0, $327 = 0, $328 = 0, $329 = 0, $33 = 0, $330 = 0, $331 = 0;
 var $332 = 0, $333 = 0, $334 = 0, $335 = 0, $336 = 0, $337 = 0, $338 = 0, $339 = 0, $34 = 0, $340 = 0, $341 = 0, $342 = 0, $343 = 0, $344 = 0, $345 = 0, $346 = 0, $347 = 0, $348 = 0, $349 = 0, $35 = 0;
 var $350 = 0, $351 = 0, $352 = 0, $353 = 0, $354 = 0, $355 = 0, $356 = 0, $357 = 0, $358 = 0, $359 = 0, $36 = 0, $360 = 0, $361 = 0, $362 = 0, $363 = 0, $364 = 0, $365 = 0, $366 = 0, $367 = 0, $368 = 0;
 var $369 = 0, $37 = 0, $370 = 0, $371 = 0, $372 = 0, $373 = 0, $374 = 0, $375 = 0, $376 = 0, $377 = 0, $378 = 0, $379 = 0, $38 = 0, $380 = 0, $381 = 0, $382 = 0, $383 = 0, $384 = 0, $385 = 0, $386 = 0;
 var $387 = 0, $388 = 0, $389 = 0, $39 = 0, $390 = 0, $391 = 0, $392 = 0, $393 = 0, $394 = 0, $395 = 0, $396 = 0, $397 = 0, $398 = 0, $399 = 0, $4 = 0, $40 = 0, $400 = 0, $401 = 0, $402 = 0, $403 = 0;
 var $404 = 0, $405 = 0, $406 = 0, $407 = 0, $408 = 0, $409 = 0, $41 = 0, $410 = 0, $411 = 0, $412 = 0, $413 = 0, $414 = 0, $415 = 0, $416 = 0, $417 = 0, $418 = 0, $419 = 0, $42 = 0, $420 = 0, $421 = 0;
 var $422 = 0, $423 = 0, $424 = 0, $425 = 0, $426 = 0, $427 = 0, $428 = 0, $429 = 0, $43 = 0, $430 = 0, $431 = 0, $432 = 0, $433 = 0, $434 = 0, $435 = 0, $436 = 0, $437 = 0, $438 = 0, $439 = 0, $44 = 0;
 var $440 = 0, $441 = 0, $442 = 0, $443 = 0, $444 = 0, $445 = 0, $446 = 0, $447 = 0, $448 = 0, $449 = 0, $45 = 0, $450 = 0, $451 = 0, $452 = 0, $453 = 0, $454 = 0, $455 = 0, $456 = 0, $457 = 0, $458 = 0;
 var $459 = 0, $46 = 0, $460 = 0, $461 = 0, $462 = 0, $463 = 0, $464 = 0, $465 = 0, $466 = 0, $467 = 0, $468 = 0, $469 = 0, $47 = 0, $470 = 0, $471 = 0, $472 = 0, $473 = 0, $474 = 0, $475 = 0, $476 = 0;
 var $477 = 0, $478 = 0, $479 = 0, $48 = 0, $480 = 0, $481 = 0, $482 = 0, $483 = 0, $484 = 0, $485 = 0, $486 = 0, $487 = 0, $488 = 0, $489 = 0, $49 = 0, $490 = 0, $491 = 0, $492 = 0, $493 = 0, $494 = 0;
 var $495 = 0, $496 = 0, $497 = 0, $498 = 0, $499 = 0, $5 = 0, $50 = 0, $500 = 0, $501 = 0, $502 = 0, $503 = 0, $504 = 0, $505 = 0, $506 = 0, $507 = 0, $508 = 0, $509 = 0, $51 = 0, $510 = 0, $511 = 0;
 var $512 = 0, $513 = 0, $514 = 0, $515 = 0, $516 = 0, $517 = 0, $518 = 0, $519 = 0, $52 = 0, $520 = 0, $521 = 0, $522 = 0, $523 = 0, $524 = 0, $525 = 0, $526 = 0, $527 = 0, $528 = 0, $529 = 0, $53 = 0;
 var $530 = 0, $531 = 0, $532 = 0, $533 = 0, $534 = 0, $535 = 0, $536 = 0, $537 = 0, $538 = 0, $539 = 0, $54 = 0, $540 = 0, $541 = 0, $542 = 0, $543 = 0, $544 = 0, $545 = 0, $546 = 0, $547 = 0, $548 = 0;
 var $549 = 0, $55 = 0, $550 = 0, $551 = 0, $552 = 0, $553 = 0, $554 = 0, $555 = 0, $556 = 0, $557 = 0, $558 = 0, $559 = 0, $56 = 0, $560 = 0, $561 = 0, $562 = 0, $563 = 0, $564 = 0, $565 = 0, $566 = 0;
 var $567 = 0, $568 = 0, $569 = 0, $57 = 0, $570 = 0, $571 = 0, $572 = 0, $573 = 0, $574 = 0, $575 = 0, $576 = 0, $577 = 0, $578 = 0, $579 = 0, $58 = 0, $580 = 0, $581 = 0, $582 = 0, $583 = 0, $584 = 0;
 var $585 = 0, $586 = 0, $587 = 0, $588 = 0, $589 = 0, $59 = 0, $590 = 0, $591 = 0, $592 = 0, $593 = 0, $594 = 0, $595 = 0, $596 = 0, $597 = 0, $598 = 0, $599 = 0, $6 = 0, $60 = 0, $600 = 0, $601 = 0;
 var $602 = 0, $603 = 0, $604 = 0, $605 = 0, $606 = 0, $607 = 0, $608 = 0, $609 = 0, $61 = 0, $610 = 0, $611 = 0, $612 = 0, $613 = 0, $614 = 0, $615 = 0, $616 = 0, $617 = 0, $618 = 0, $619 = 0, $62 = 0;
 var $620 = 0, $621 = 0, $622 = 0, $623 = 0, $624 = 0, $625 = 0, $626 = 0, $627 = 0, $628 = 0, $629 = 0, $63 = 0, $630 = 0, $631 = 0, $632 = 0, $633 = 0, $634 = 0, $635 = 0, $636 = 0, $637 = 0, $638 = 0;
 var $639 = 0, $64 = 0, $640 = 0, $641 = 0, $642 = 0, $643 = 0, $644 = 0, $645 = 0, $646 = 0, $647 = 0, $648 = 0, $649 = 0, $65 = 0, $650 = 0, $651 = 0, $652 = 0, $653 = 0, $654 = 0, $655 = 0, $656 = 0;
 var $657 = 0, $658 = 0, $659 = 0, $66 = 0, $660 = 0, $661 = 0, $662 = 0, $663 = 0, $664 = 0, $665 = 0, $666 = 0, $667 = 0, $668 = 0, $669 = 0, $67 = 0, $670 = 0, $671 = 0, $672 = 0, $673 = 0, $674 = 0;
 var $675 = 0, $676 = 0, $677 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0;
 var $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $aFrom = 0, $aSortCost = 0;
 var $aTo = 0, $db = 0, $iLoop = 0, $ii = 0, $isOrdered = 0, $jj = 0, $maskNew = 0, $mxChoice = 0, $mxCost = 0, $mxI = 0, $mxUnsorted = 0, $nFrom = 0, $nLoop = 0, $nOrder = 0, $nOrderBy = 0, $nOut = 0, $nSpace = 0, $nTo = 0, $notUsed = 0, $pFrom = 0;
 var $pLevel = 0, $pParse = 0, $pSpace = 0, $pTo = 0, $pWLoop = 0, $pX = 0, $rCost = 0, $rUnsorted = 0, $rc = 0, $revMask = 0, $revMask1 = 0, $vararg_buffer = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 160|0;
 $vararg_buffer = sp + 32|0;
 $0 = sp + 132|0;
 $1 = sp + 128|0;
 $2 = sp + 146|0;
 $mxChoice = sp + 124|0;
 $nLoop = sp + 120|0;
 $pParse = sp + 116|0;
 $db = sp + 112|0;
 $iLoop = sp + 108|0;
 $ii = sp + 104|0;
 $jj = sp + 100|0;
 $mxI = sp + 96|0;
 $nOrderBy = sp + 92|0;
 $mxCost = sp + 144|0;
 $mxUnsorted = sp + 142|0;
 $nTo = sp + 88|0;
 $nFrom = sp + 84|0;
 $aFrom = sp + 80|0;
 $aTo = sp + 76|0;
 $pFrom = sp + 72|0;
 $pTo = sp + 68|0;
 $pWLoop = sp + 64|0;
 $pX = sp + 60|0;
 $aSortCost = sp + 56|0;
 $pSpace = sp + 52|0;
 $nSpace = sp + 48|0;
 $nOut = sp + 140|0;
 $rCost = sp + 138|0;
 $rUnsorted = sp + 136|0;
 $isOrdered = sp + 148|0;
 $maskNew = sp + 24|0;
 $revMask = sp + 16|0;
 $pLevel = sp + 44|0;
 $notUsed = sp + 8|0;
 $rc = sp + 40|0;
 $revMask1 = sp;
 $nOrder = sp + 36|0;
 HEAP32[$1>>2] = $pWInfo;
 HEAP16[$2>>1] = $nRowEst;
 HEAP32[$mxI>>2] = 0;
 HEAP16[$mxCost>>1] = 0;
 HEAP16[$mxUnsorted>>1] = 0;
 HEAP32[$aSortCost>>2] = 0;
 $3 = HEAP32[$1>>2]|0;
 $4 = HEAP32[$3>>2]|0;
 HEAP32[$pParse>>2] = $4;
 $5 = HEAP32[$pParse>>2]|0;
 $6 = HEAP32[$5>>2]|0;
 HEAP32[$db>>2] = $6;
 $7 = HEAP32[$1>>2]|0;
 $8 = (($7) + 41|0);
 $9 = HEAP8[$8>>0]|0;
 $10 = $9&255;
 HEAP32[$nLoop>>2] = $10;
 $11 = HEAP32[$nLoop>>2]|0;
 $12 = ($11|0)<=(1);
 if ($12) {
  $16 = 1;
 } else {
  $13 = HEAP32[$nLoop>>2]|0;
  $14 = ($13|0)==(2);
  $15 = $14 ? 5 : 10;
  $16 = $15;
 }
 HEAP32[$mxChoice>>2] = $16;
 $17 = HEAP32[$1>>2]|0;
 $18 = (($17) + 8|0);
 $19 = HEAP32[$18>>2]|0;
 $20 = ($19|0)==(0|0);
 if ($20) {
  label = 5;
 } else {
  $21 = HEAP16[$2>>1]|0;
  $22 = $21 << 16 >> 16;
  $23 = ($22|0)==(0);
  if ($23) {
   label = 5;
  } else {
   $24 = HEAP32[$1>>2]|0;
   $25 = (($24) + 8|0);
   $26 = HEAP32[$25>>2]|0;
   $27 = HEAP32[$26>>2]|0;
   HEAP32[$nOrderBy>>2] = $27;
  }
 }
 if ((label|0) == 5) {
  HEAP32[$nOrderBy>>2] = 0;
 }
 $28 = HEAP32[$nLoop>>2]|0;
 $29 = $28<<2;
 $30 = (32 + ($29))|0;
 $31 = HEAP32[$mxChoice>>2]|0;
 $32 = Math_imul($30, $31)|0;
 $33 = $32<<1;
 HEAP32[$nSpace>>2] = $33;
 $34 = HEAP32[$nOrderBy>>2]|0;
 $35 = $34<<1;
 $36 = HEAP32[$nSpace>>2]|0;
 $37 = (($36) + ($35))|0;
 HEAP32[$nSpace>>2] = $37;
 $38 = HEAP32[$db>>2]|0;
 $39 = HEAP32[$nSpace>>2]|0;
 $40 = ($39|0)<(0);
 $41 = $40 << 31 >> 31;
 $42 = (_sqlite3DbMallocRaw($38,$39,$41)|0);
 HEAP32[$pSpace>>2] = $42;
 $43 = HEAP32[$pSpace>>2]|0;
 $44 = ($43|0)==(0|0);
 if ($44) {
  HEAP32[$0>>2] = 7;
  $677 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($677|0);
 }
 $45 = HEAP32[$pSpace>>2]|0;
 HEAP32[$aTo>>2] = $45;
 $46 = HEAP32[$aTo>>2]|0;
 $47 = HEAP32[$mxChoice>>2]|0;
 $48 = (($46) + ($47<<5)|0);
 HEAP32[$aFrom>>2] = $48;
 $49 = HEAP32[$aFrom>>2]|0;
 ;HEAP32[$49+0>>2]=0|0;HEAP32[$49+4>>2]=0|0;HEAP32[$49+8>>2]=0|0;HEAP32[$49+12>>2]=0|0;HEAP32[$49+16>>2]=0|0;HEAP32[$49+20>>2]=0|0;HEAP32[$49+24>>2]=0|0;HEAP32[$49+28>>2]=0|0;
 $50 = HEAP32[$aFrom>>2]|0;
 $51 = HEAP32[$mxChoice>>2]|0;
 $52 = (($50) + ($51<<5)|0);
 HEAP32[$pX>>2] = $52;
 $53 = HEAP32[$mxChoice>>2]|0;
 $54 = $53<<1;
 HEAP32[$ii>>2] = $54;
 $55 = HEAP32[$aTo>>2]|0;
 HEAP32[$pFrom>>2] = $55;
 while(1) {
  $56 = HEAP32[$ii>>2]|0;
  $57 = ($56|0)>(0);
  if (!($57)) {
   break;
  }
  $58 = HEAP32[$pX>>2]|0;
  $59 = HEAP32[$pFrom>>2]|0;
  $60 = (($59) + 24|0);
  HEAP32[$60>>2] = $58;
  $61 = HEAP32[$ii>>2]|0;
  $62 = (($61) + -1)|0;
  HEAP32[$ii>>2] = $62;
  $63 = HEAP32[$pFrom>>2]|0;
  $64 = (($63) + 32|0);
  HEAP32[$pFrom>>2] = $64;
  $65 = HEAP32[$nLoop>>2]|0;
  $66 = HEAP32[$pX>>2]|0;
  $67 = (($66) + ($65<<2)|0);
  HEAP32[$pX>>2] = $67;
 }
 $68 = HEAP32[$nOrderBy>>2]|0;
 $69 = ($68|0)!=(0);
 if ($69) {
  $70 = HEAP32[$pX>>2]|0;
  HEAP32[$aSortCost>>2] = $70;
  $71 = HEAP32[$aSortCost>>2]|0;
  $72 = HEAP32[$nOrderBy>>2]|0;
  $73 = $72<<1;
  _memset(($71|0),0,($73|0))|0;
 }
 $74 = HEAP32[$pParse>>2]|0;
 $75 = (($74) + 424|0);
 $76 = HEAP32[$75>>2]|0;
 $77 = ($76>>>0)<(46);
 if ($77) {
  $78 = HEAP32[$pParse>>2]|0;
  $79 = (($78) + 424|0);
  $80 = HEAP32[$79>>2]|0;
  $82 = $80;
 } else {
  $82 = 46;
 }
 $81 = $82&65535;
 $83 = HEAP32[$aFrom>>2]|0;
 $84 = (($83) + 16|0);
 HEAP16[$84>>1] = $81;
 HEAP32[$nFrom>>2] = 1;
 $85 = HEAP32[$nOrderBy>>2]|0;
 $86 = ($85|0)!=(0);
 if ($86) {
  $87 = HEAP32[$nLoop>>2]|0;
  $88 = ($87|0)>(0);
  $89 = HEAP32[$nOrderBy>>2]|0;
  $90 = $88 ? -1 : $89;
  $91 = $90&255;
  $92 = HEAP32[$aFrom>>2]|0;
  $93 = (($92) + 22|0);
  HEAP8[$93>>0] = $91;
 }
 HEAP32[$iLoop>>2] = 0;
 while(1) {
  $94 = HEAP32[$iLoop>>2]|0;
  $95 = HEAP32[$nLoop>>2]|0;
  $96 = ($94|0)<($95|0);
  if (!($96)) {
   break;
  }
  HEAP32[$nTo>>2] = 0;
  HEAP32[$ii>>2] = 0;
  $97 = HEAP32[$aFrom>>2]|0;
  HEAP32[$pFrom>>2] = $97;
  while(1) {
   $98 = HEAP32[$ii>>2]|0;
   $99 = HEAP32[$nFrom>>2]|0;
   $100 = ($98|0)<($99|0);
   if (!($100)) {
    break;
   }
   $101 = HEAP32[$1>>2]|0;
   $102 = (($101) + 16|0);
   $103 = HEAP32[$102>>2]|0;
   HEAP32[$pWLoop>>2] = $103;
   while(1) {
    $104 = HEAP32[$pWLoop>>2]|0;
    $105 = ($104|0)!=(0|0);
    if (!($105)) {
     break;
    }
    $106 = HEAP32[$pFrom>>2]|0;
    $107 = (($106) + 22|0);
    $108 = HEAP8[$107>>0]|0;
    HEAP8[$isOrdered>>0] = $108;
    $109 = $revMask;
    $110 = $109;
    HEAP32[$110>>2] = 0;
    $111 = (($109) + 4)|0;
    $112 = $111;
    HEAP32[$112>>2] = 0;
    $113 = HEAP32[$pWLoop>>2]|0;
    $114 = $113;
    $115 = $114;
    $116 = HEAP32[$115>>2]|0;
    $117 = (($114) + 4)|0;
    $118 = $117;
    $119 = HEAP32[$118>>2]|0;
    $120 = HEAP32[$pFrom>>2]|0;
    $121 = $120;
    $122 = $121;
    $123 = HEAP32[$122>>2]|0;
    $124 = (($121) + 4)|0;
    $125 = $124;
    $126 = HEAP32[$125>>2]|0;
    $127 = $123 ^ -1;
    $128 = $126 ^ -1;
    $129 = $116 & $127;
    $130 = $119 & $128;
    $131 = ($129|0)!=(0);
    $132 = ($130|0)!=(0);
    $133 = $131 | $132;
    L36: do {
     if (!($133)) {
      $134 = HEAP32[$pWLoop>>2]|0;
      $135 = (($134) + 8|0);
      $136 = $135;
      $137 = $136;
      $138 = HEAP32[$137>>2]|0;
      $139 = (($136) + 4)|0;
      $140 = $139;
      $141 = HEAP32[$140>>2]|0;
      $142 = HEAP32[$pFrom>>2]|0;
      $143 = $142;
      $144 = $143;
      $145 = HEAP32[$144>>2]|0;
      $146 = (($143) + 4)|0;
      $147 = $146;
      $148 = HEAP32[$147>>2]|0;
      $149 = $138 & $145;
      $150 = $141 & $148;
      $151 = ($149|0)!=(0);
      $152 = ($150|0)!=(0);
      $153 = $151 | $152;
      if (!($153)) {
       $154 = HEAP32[$pWLoop>>2]|0;
       $155 = (($154) + 18|0);
       $156 = HEAP16[$155>>1]|0;
       $157 = HEAP32[$pWLoop>>2]|0;
       $158 = (($157) + 20|0);
       $159 = HEAP16[$158>>1]|0;
       $160 = $159 << 16 >> 16;
       $161 = HEAP32[$pFrom>>2]|0;
       $162 = (($161) + 16|0);
       $163 = HEAP16[$162>>1]|0;
       $164 = $163 << 16 >> 16;
       $165 = (($160) + ($164))|0;
       $166 = $165&65535;
       $167 = (_sqlite3LogEstAdd($156,$166)|0);
       HEAP16[$rUnsorted>>1] = $167;
       $168 = HEAP16[$rUnsorted>>1]|0;
       $169 = HEAP32[$pFrom>>2]|0;
       $170 = (($169) + 20|0);
       $171 = HEAP16[$170>>1]|0;
       $172 = (_sqlite3LogEstAdd($168,$171)|0);
       HEAP16[$rUnsorted>>1] = $172;
       $173 = HEAP32[$pFrom>>2]|0;
       $174 = (($173) + 16|0);
       $175 = HEAP16[$174>>1]|0;
       $176 = $175 << 16 >> 16;
       $177 = HEAP32[$pWLoop>>2]|0;
       $178 = (($177) + 22|0);
       $179 = HEAP16[$178>>1]|0;
       $180 = $179 << 16 >> 16;
       $181 = (($176) + ($180))|0;
       $182 = $181&65535;
       HEAP16[$nOut>>1] = $182;
       $183 = HEAP32[$pFrom>>2]|0;
       $184 = $183;
       $185 = $184;
       $186 = HEAP32[$185>>2]|0;
       $187 = (($184) + 4)|0;
       $188 = $187;
       $189 = HEAP32[$188>>2]|0;
       $190 = HEAP32[$pWLoop>>2]|0;
       $191 = (($190) + 8|0);
       $192 = $191;
       $193 = $192;
       $194 = HEAP32[$193>>2]|0;
       $195 = (($192) + 4)|0;
       $196 = $195;
       $197 = HEAP32[$196>>2]|0;
       $198 = $186 | $194;
       $199 = $189 | $197;
       $200 = $maskNew;
       $201 = $200;
       HEAP32[$201>>2] = $198;
       $202 = (($200) + 4)|0;
       $203 = $202;
       HEAP32[$203>>2] = $199;
       $204 = HEAP8[$isOrdered>>0]|0;
       $205 = $204 << 24 >> 24;
       $206 = ($205|0)<(0);
       if ($206) {
        $207 = HEAP32[$1>>2]|0;
        $208 = HEAP32[$1>>2]|0;
        $209 = (($208) + 8|0);
        $210 = HEAP32[$209>>2]|0;
        $211 = HEAP32[$pFrom>>2]|0;
        $212 = HEAP32[$1>>2]|0;
        $213 = (($212) + 34|0);
        $214 = HEAP16[$213>>1]|0;
        $215 = HEAP32[$iLoop>>2]|0;
        $216 = $215&65535;
        $217 = HEAP32[$pWLoop>>2]|0;
        $218 = (_wherePathSatisfiesOrderBy($207,$210,$211,$214,$216,$217,$revMask)|0);
        HEAP8[$isOrdered>>0] = $218;
       } else {
        $219 = HEAP32[$pFrom>>2]|0;
        $220 = (($219) + 8|0);
        $221 = $220;
        $222 = $221;
        $223 = HEAP32[$222>>2]|0;
        $224 = (($221) + 4)|0;
        $225 = $224;
        $226 = HEAP32[$225>>2]|0;
        $227 = $revMask;
        $228 = $227;
        HEAP32[$228>>2] = $223;
        $229 = (($227) + 4)|0;
        $230 = $229;
        HEAP32[$230>>2] = $226;
       }
       $231 = HEAP8[$isOrdered>>0]|0;
       $232 = $231 << 24 >> 24;
       $233 = ($232|0)>=(0);
       if ($233) {
        $234 = HEAP8[$isOrdered>>0]|0;
        $235 = $234 << 24 >> 24;
        $236 = HEAP32[$nOrderBy>>2]|0;
        $237 = ($235|0)<($236|0);
        if ($237) {
         $238 = HEAP8[$isOrdered>>0]|0;
         $239 = $238 << 24 >> 24;
         $240 = HEAP32[$aSortCost>>2]|0;
         $241 = (($240) + ($239<<1)|0);
         $242 = HEAP16[$241>>1]|0;
         $243 = $242 << 16 >> 16;
         $244 = ($243|0)==(0);
         if ($244) {
          $245 = HEAP32[$1>>2]|0;
          $246 = HEAP16[$2>>1]|0;
          $247 = HEAP32[$nOrderBy>>2]|0;
          $248 = HEAP8[$isOrdered>>0]|0;
          $249 = $248 << 24 >> 24;
          $250 = (_whereSortingCost($245,$246,$247,$249)|0);
          $251 = HEAP8[$isOrdered>>0]|0;
          $252 = $251 << 24 >> 24;
          $253 = HEAP32[$aSortCost>>2]|0;
          $254 = (($253) + ($252<<1)|0);
          HEAP16[$254>>1] = $250;
         }
         $255 = HEAP16[$rUnsorted>>1]|0;
         $256 = HEAP8[$isOrdered>>0]|0;
         $257 = $256 << 24 >> 24;
         $258 = HEAP32[$aSortCost>>2]|0;
         $259 = (($258) + ($257<<1)|0);
         $260 = HEAP16[$259>>1]|0;
         $261 = (_sqlite3LogEstAdd($255,$260)|0);
         HEAP16[$rCost>>1] = $261;
        } else {
         label = 34;
        }
       } else {
        label = 34;
       }
       if ((label|0) == 34) {
        label = 0;
        $262 = HEAP16[$rUnsorted>>1]|0;
        HEAP16[$rCost>>1] = $262;
       }
       HEAP32[$jj>>2] = 0;
       $263 = HEAP32[$aTo>>2]|0;
       HEAP32[$pTo>>2] = $263;
       while(1) {
        $264 = HEAP32[$jj>>2]|0;
        $265 = HEAP32[$nTo>>2]|0;
        $266 = ($264|0)<($265|0);
        if (!($266)) {
         break;
        }
        $267 = HEAP32[$pTo>>2]|0;
        $268 = $267;
        $269 = $268;
        $270 = HEAP32[$269>>2]|0;
        $271 = (($268) + 4)|0;
        $272 = $271;
        $273 = HEAP32[$272>>2]|0;
        $274 = $maskNew;
        $275 = $274;
        $276 = HEAP32[$275>>2]|0;
        $277 = (($274) + 4)|0;
        $278 = $277;
        $279 = HEAP32[$278>>2]|0;
        $280 = ($270|0)==($276|0);
        $281 = ($273|0)==($279|0);
        $282 = $280 & $281;
        if ($282) {
         $283 = HEAP32[$pTo>>2]|0;
         $284 = (($283) + 22|0);
         $285 = HEAP8[$284>>0]|0;
         $286 = $285 << 24 >> 24;
         $287 = HEAP8[$isOrdered>>0]|0;
         $288 = $287 << 24 >> 24;
         $289 = $286 ^ $288;
         $290 = $289 & 128;
         $291 = ($290|0)==(0);
         if ($291) {
          break;
         }
        }
        $292 = HEAP32[$jj>>2]|0;
        $293 = (($292) + 1)|0;
        HEAP32[$jj>>2] = $293;
        $294 = HEAP32[$pTo>>2]|0;
        $295 = (($294) + 32|0);
        HEAP32[$pTo>>2] = $295;
       }
       $296 = HEAP32[$jj>>2]|0;
       $297 = HEAP32[$nTo>>2]|0;
       $298 = ($296|0)>=($297|0);
       do {
        if ($298) {
         $299 = HEAP32[$nTo>>2]|0;
         $300 = HEAP32[$mxChoice>>2]|0;
         $301 = ($299|0)>=($300|0);
         do {
          if ($301) {
           $302 = HEAP16[$rCost>>1]|0;
           $303 = $302 << 16 >> 16;
           $304 = HEAP16[$mxCost>>1]|0;
           $305 = $304 << 16 >> 16;
           $306 = ($303|0)>($305|0);
           if ($306) {
            break L36;
           }
           $307 = HEAP16[$rCost>>1]|0;
           $308 = $307 << 16 >> 16;
           $309 = HEAP16[$mxCost>>1]|0;
           $310 = $309 << 16 >> 16;
           $311 = ($308|0)==($310|0);
           if (!($311)) {
            break;
           }
           $312 = HEAP16[$rUnsorted>>1]|0;
           $313 = $312 << 16 >> 16;
           $314 = HEAP16[$mxUnsorted>>1]|0;
           $315 = $314 << 16 >> 16;
           $316 = ($313|0)>=($315|0);
           if ($316) {
            break L36;
           }
          }
         } while(0);
         $317 = HEAP32[$nTo>>2]|0;
         $318 = HEAP32[$mxChoice>>2]|0;
         $319 = ($317|0)<($318|0);
         if ($319) {
          $320 = HEAP32[$nTo>>2]|0;
          $321 = (($320) + 1)|0;
          HEAP32[$nTo>>2] = $321;
          HEAP32[$jj>>2] = $320;
         } else {
          $322 = HEAP32[$mxI>>2]|0;
          HEAP32[$jj>>2] = $322;
         }
         $323 = HEAP32[$jj>>2]|0;
         $324 = HEAP32[$aTo>>2]|0;
         $325 = (($324) + ($323<<5)|0);
         HEAP32[$pTo>>2] = $325;
        } else {
         $326 = HEAP32[$pTo>>2]|0;
         $327 = (($326) + 18|0);
         $328 = HEAP16[$327>>1]|0;
         $329 = $328 << 16 >> 16;
         $330 = HEAP16[$rCost>>1]|0;
         $331 = $330 << 16 >> 16;
         $332 = ($329|0)<($331|0);
         if ($332) {
          break L36;
         }
         $333 = HEAP32[$pTo>>2]|0;
         $334 = (($333) + 18|0);
         $335 = HEAP16[$334>>1]|0;
         $336 = $335 << 16 >> 16;
         $337 = HEAP16[$rCost>>1]|0;
         $338 = $337 << 16 >> 16;
         $339 = ($336|0)==($338|0);
         if (!($339)) {
          break;
         }
         $340 = HEAP32[$pTo>>2]|0;
         $341 = (($340) + 16|0);
         $342 = HEAP16[$341>>1]|0;
         $343 = $342 << 16 >> 16;
         $344 = HEAP16[$nOut>>1]|0;
         $345 = $344 << 16 >> 16;
         $346 = ($343|0)<=($345|0);
         if ($346) {
          break L36;
         }
        }
       } while(0);
       $347 = HEAP32[$pFrom>>2]|0;
       $348 = $347;
       $349 = $348;
       $350 = HEAP32[$349>>2]|0;
       $351 = (($348) + 4)|0;
       $352 = $351;
       $353 = HEAP32[$352>>2]|0;
       $354 = HEAP32[$pWLoop>>2]|0;
       $355 = (($354) + 8|0);
       $356 = $355;
       $357 = $356;
       $358 = HEAP32[$357>>2]|0;
       $359 = (($356) + 4)|0;
       $360 = $359;
       $361 = HEAP32[$360>>2]|0;
       $362 = $350 | $358;
       $363 = $353 | $361;
       $364 = HEAP32[$pTo>>2]|0;
       $365 = $364;
       $366 = $365;
       HEAP32[$366>>2] = $362;
       $367 = (($365) + 4)|0;
       $368 = $367;
       HEAP32[$368>>2] = $363;
       $369 = $revMask;
       $370 = $369;
       $371 = HEAP32[$370>>2]|0;
       $372 = (($369) + 4)|0;
       $373 = $372;
       $374 = HEAP32[$373>>2]|0;
       $375 = HEAP32[$pTo>>2]|0;
       $376 = (($375) + 8|0);
       $377 = $376;
       $378 = $377;
       HEAP32[$378>>2] = $371;
       $379 = (($377) + 4)|0;
       $380 = $379;
       HEAP32[$380>>2] = $374;
       $381 = HEAP16[$nOut>>1]|0;
       $382 = HEAP32[$pTo>>2]|0;
       $383 = (($382) + 16|0);
       HEAP16[$383>>1] = $381;
       $384 = HEAP16[$rCost>>1]|0;
       $385 = HEAP32[$pTo>>2]|0;
       $386 = (($385) + 18|0);
       HEAP16[$386>>1] = $384;
       $387 = HEAP16[$rUnsorted>>1]|0;
       $388 = HEAP32[$pTo>>2]|0;
       $389 = (($388) + 20|0);
       HEAP16[$389>>1] = $387;
       $390 = HEAP8[$isOrdered>>0]|0;
       $391 = HEAP32[$pTo>>2]|0;
       $392 = (($391) + 22|0);
       HEAP8[$392>>0] = $390;
       $393 = HEAP32[$pTo>>2]|0;
       $394 = (($393) + 24|0);
       $395 = HEAP32[$394>>2]|0;
       $396 = HEAP32[$pFrom>>2]|0;
       $397 = (($396) + 24|0);
       $398 = HEAP32[$397>>2]|0;
       $399 = HEAP32[$iLoop>>2]|0;
       $400 = $399<<2;
       _memcpy(($395|0),($398|0),($400|0))|0;
       $401 = HEAP32[$pWLoop>>2]|0;
       $402 = HEAP32[$iLoop>>2]|0;
       $403 = HEAP32[$pTo>>2]|0;
       $404 = (($403) + 24|0);
       $405 = HEAP32[$404>>2]|0;
       $406 = (($405) + ($402<<2)|0);
       HEAP32[$406>>2] = $401;
       $407 = HEAP32[$nTo>>2]|0;
       $408 = HEAP32[$mxChoice>>2]|0;
       $409 = ($407|0)>=($408|0);
       if ($409) {
        HEAP32[$mxI>>2] = 0;
        $410 = HEAP32[$aTo>>2]|0;
        $411 = (($410) + 18|0);
        $412 = HEAP16[$411>>1]|0;
        HEAP16[$mxCost>>1] = $412;
        $413 = HEAP32[$aTo>>2]|0;
        $414 = (($413) + 16|0);
        $415 = HEAP16[$414>>1]|0;
        HEAP16[$mxUnsorted>>1] = $415;
        HEAP32[$jj>>2] = 1;
        $416 = HEAP32[$aTo>>2]|0;
        $417 = (($416) + 32|0);
        HEAP32[$pTo>>2] = $417;
        while(1) {
         $418 = HEAP32[$jj>>2]|0;
         $419 = HEAP32[$mxChoice>>2]|0;
         $420 = ($418|0)<($419|0);
         if (!($420)) {
          break L36;
         }
         $421 = HEAP32[$pTo>>2]|0;
         $422 = (($421) + 18|0);
         $423 = HEAP16[$422>>1]|0;
         $424 = $423 << 16 >> 16;
         $425 = HEAP16[$mxCost>>1]|0;
         $426 = $425 << 16 >> 16;
         $427 = ($424|0)>($426|0);
         do {
          if ($427) {
           label = 58;
          } else {
           $428 = HEAP32[$pTo>>2]|0;
           $429 = (($428) + 18|0);
           $430 = HEAP16[$429>>1]|0;
           $431 = $430 << 16 >> 16;
           $432 = HEAP16[$mxCost>>1]|0;
           $433 = $432 << 16 >> 16;
           $434 = ($431|0)==($433|0);
           if (!($434)) {
            break;
           }
           $435 = HEAP32[$pTo>>2]|0;
           $436 = (($435) + 20|0);
           $437 = HEAP16[$436>>1]|0;
           $438 = $437 << 16 >> 16;
           $439 = HEAP16[$mxUnsorted>>1]|0;
           $440 = $439 << 16 >> 16;
           $441 = ($438|0)>($440|0);
           if ($441) {
            label = 58;
           }
          }
         } while(0);
         if ((label|0) == 58) {
          label = 0;
          $442 = HEAP32[$pTo>>2]|0;
          $443 = (($442) + 18|0);
          $444 = HEAP16[$443>>1]|0;
          HEAP16[$mxCost>>1] = $444;
          $445 = HEAP32[$pTo>>2]|0;
          $446 = (($445) + 20|0);
          $447 = HEAP16[$446>>1]|0;
          HEAP16[$mxUnsorted>>1] = $447;
          $448 = HEAP32[$jj>>2]|0;
          HEAP32[$mxI>>2] = $448;
         }
         $449 = HEAP32[$jj>>2]|0;
         $450 = (($449) + 1)|0;
         HEAP32[$jj>>2] = $450;
         $451 = HEAP32[$pTo>>2]|0;
         $452 = (($451) + 32|0);
         HEAP32[$pTo>>2] = $452;
        }
       }
      }
     }
    } while(0);
    $453 = HEAP32[$pWLoop>>2]|0;
    $454 = (($453) + 48|0);
    $455 = HEAP32[$454>>2]|0;
    HEAP32[$pWLoop>>2] = $455;
   }
   $456 = HEAP32[$ii>>2]|0;
   $457 = (($456) + 1)|0;
   HEAP32[$ii>>2] = $457;
   $458 = HEAP32[$pFrom>>2]|0;
   $459 = (($458) + 32|0);
   HEAP32[$pFrom>>2] = $459;
  }
  $460 = HEAP32[$aTo>>2]|0;
  HEAP32[$pFrom>>2] = $460;
  $461 = HEAP32[$aFrom>>2]|0;
  HEAP32[$aTo>>2] = $461;
  $462 = HEAP32[$pFrom>>2]|0;
  HEAP32[$aFrom>>2] = $462;
  $463 = HEAP32[$nTo>>2]|0;
  HEAP32[$nFrom>>2] = $463;
  $464 = HEAP32[$iLoop>>2]|0;
  $465 = (($464) + 1)|0;
  HEAP32[$iLoop>>2] = $465;
 }
 $466 = HEAP32[$nFrom>>2]|0;
 $467 = ($466|0)==(0);
 if ($467) {
  $468 = HEAP32[$pParse>>2]|0;
  _sqlite3ErrorMsg($468,24032,$vararg_buffer);
  $469 = HEAP32[$db>>2]|0;
  $470 = HEAP32[$pSpace>>2]|0;
  _sqlite3DbFree($469,$470);
  HEAP32[$0>>2] = 1;
  $677 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($677|0);
 }
 $471 = HEAP32[$aFrom>>2]|0;
 HEAP32[$pFrom>>2] = $471;
 HEAP32[$ii>>2] = 1;
 while(1) {
  $472 = HEAP32[$ii>>2]|0;
  $473 = HEAP32[$nFrom>>2]|0;
  $474 = ($472|0)<($473|0);
  if (!($474)) {
   break;
  }
  $475 = HEAP32[$pFrom>>2]|0;
  $476 = (($475) + 18|0);
  $477 = HEAP16[$476>>1]|0;
  $478 = $477 << 16 >> 16;
  $479 = HEAP32[$ii>>2]|0;
  $480 = HEAP32[$aFrom>>2]|0;
  $481 = (($480) + ($479<<5)|0);
  $482 = (($481) + 18|0);
  $483 = HEAP16[$482>>1]|0;
  $484 = $483 << 16 >> 16;
  $485 = ($478|0)>($484|0);
  if ($485) {
   $486 = HEAP32[$ii>>2]|0;
   $487 = HEAP32[$aFrom>>2]|0;
   $488 = (($487) + ($486<<5)|0);
   HEAP32[$pFrom>>2] = $488;
  }
  $489 = HEAP32[$ii>>2]|0;
  $490 = (($489) + 1)|0;
  HEAP32[$ii>>2] = $490;
 }
 HEAP32[$iLoop>>2] = 0;
 while(1) {
  $491 = HEAP32[$iLoop>>2]|0;
  $492 = HEAP32[$nLoop>>2]|0;
  $493 = ($491|0)<($492|0);
  $494 = HEAP32[$1>>2]|0;
  if (!($493)) {
   break;
  }
  $495 = (($494) + 736|0);
  $496 = HEAP32[$iLoop>>2]|0;
  $497 = (($495) + (($496*72)|0)|0);
  HEAP32[$pLevel>>2] = $497;
  $498 = HEAP32[$iLoop>>2]|0;
  $499 = HEAP32[$pFrom>>2]|0;
  $500 = (($499) + 24|0);
  $501 = HEAP32[$500>>2]|0;
  $502 = (($501) + ($498<<2)|0);
  $503 = HEAP32[$502>>2]|0;
  HEAP32[$pWLoop>>2] = $503;
  $504 = HEAP32[$pLevel>>2]|0;
  $505 = (($504) + 56|0);
  HEAP32[$505>>2] = $503;
  $506 = HEAP32[$pWLoop>>2]|0;
  $507 = (($506) + 16|0);
  $508 = HEAP8[$507>>0]|0;
  $509 = HEAP32[$pLevel>>2]|0;
  $510 = (($509) + 36|0);
  HEAP8[$510>>0] = $508;
  $511 = HEAP32[$pLevel>>2]|0;
  $512 = (($511) + 36|0);
  $513 = HEAP8[$512>>0]|0;
  $514 = $513&255;
  $515 = HEAP32[$1>>2]|0;
  $516 = (($515) + 4|0);
  $517 = HEAP32[$516>>2]|0;
  $518 = (($517) + 8|0);
  $519 = (($518) + (($514*72)|0)|0);
  $520 = (($519) + 40|0);
  $521 = HEAP32[$520>>2]|0;
  $522 = HEAP32[$pLevel>>2]|0;
  $523 = (($522) + 4|0);
  HEAP32[$523>>2] = $521;
  $524 = HEAP32[$iLoop>>2]|0;
  $525 = (($524) + 1)|0;
  HEAP32[$iLoop>>2] = $525;
 }
 $526 = (($494) + 34|0);
 $527 = HEAP16[$526>>1]|0;
 $528 = $527&65535;
 $529 = $528 & 1024;
 $530 = ($529|0)!=(0);
 if ($530) {
  $531 = HEAP32[$1>>2]|0;
  $532 = (($531) + 34|0);
  $533 = HEAP16[$532>>1]|0;
  $534 = $533&65535;
  $535 = $534 & 512;
  $536 = ($535|0)==(0);
  if ($536) {
   $537 = HEAP32[$1>>2]|0;
   $538 = (($537) + 40|0);
   $539 = HEAP8[$538>>0]|0;
   $540 = $539&255;
   $541 = ($540|0)==(0);
   if ($541) {
    $542 = HEAP16[$2>>1]|0;
    $543 = $542 << 16 >> 16;
    $544 = ($543|0)!=(0);
    if ($544) {
     $545 = HEAP32[$1>>2]|0;
     $546 = HEAP32[$1>>2]|0;
     $547 = (($546) + 12|0);
     $548 = HEAP32[$547>>2]|0;
     $549 = HEAP32[$pFrom>>2]|0;
     $550 = HEAP32[$nLoop>>2]|0;
     $551 = (($550) - 1)|0;
     $552 = $551&65535;
     $553 = HEAP32[$nLoop>>2]|0;
     $554 = (($553) - 1)|0;
     $555 = HEAP32[$pFrom>>2]|0;
     $556 = (($555) + 24|0);
     $557 = HEAP32[$556>>2]|0;
     $558 = (($557) + ($554<<2)|0);
     $559 = HEAP32[$558>>2]|0;
     $560 = (_wherePathSatisfiesOrderBy($545,$548,$549,512,$552,$559,$notUsed)|0);
     $561 = $560 << 24 >> 24;
     HEAP32[$rc>>2] = $561;
     $562 = HEAP32[$rc>>2]|0;
     $563 = HEAP32[$1>>2]|0;
     $564 = (($563) + 12|0);
     $565 = HEAP32[$564>>2]|0;
     $566 = HEAP32[$565>>2]|0;
     $567 = ($562|0)==($566|0);
     if ($567) {
      $568 = HEAP32[$1>>2]|0;
      $569 = (($568) + 40|0);
      HEAP8[$569>>0] = 2;
     }
    }
   }
  }
 }
 $570 = HEAP32[$1>>2]|0;
 $571 = (($570) + 8|0);
 $572 = HEAP32[$571>>2]|0;
 $573 = ($572|0)!=(0|0);
 if ($573) {
  $574 = HEAP32[$1>>2]|0;
  $575 = (($574) + 34|0);
  $576 = HEAP16[$575>>1]|0;
  $577 = $576&65535;
  $578 = $577 & 512;
  $579 = ($578|0)!=(0);
  $580 = HEAP32[$pFrom>>2]|0;
  $581 = (($580) + 22|0);
  $582 = HEAP8[$581>>0]|0;
  if ($579) {
   $583 = $582 << 24 >> 24;
   $584 = HEAP32[$1>>2]|0;
   $585 = (($584) + 8|0);
   $586 = HEAP32[$585>>2]|0;
   $587 = HEAP32[$586>>2]|0;
   $588 = ($583|0)==($587|0);
   if ($588) {
    $589 = HEAP32[$1>>2]|0;
    $590 = (($589) + 40|0);
    HEAP8[$590>>0] = 2;
   }
  } else {
   $591 = HEAP32[$1>>2]|0;
   $592 = (($591) + 36|0);
   HEAP8[$592>>0] = $582;
   $593 = HEAP32[$1>>2]|0;
   $594 = (($593) + 36|0);
   $595 = HEAP8[$594>>0]|0;
   $596 = $595 << 24 >> 24;
   $597 = ($596|0)<(0);
   if ($597) {
    $598 = HEAP32[$1>>2]|0;
    $599 = (($598) + 36|0);
    HEAP8[$599>>0] = 0;
   }
   $600 = HEAP32[$pFrom>>2]|0;
   $601 = (($600) + 8|0);
   $602 = $601;
   $603 = $602;
   $604 = HEAP32[$603>>2]|0;
   $605 = (($602) + 4)|0;
   $606 = $605;
   $607 = HEAP32[$606>>2]|0;
   $608 = HEAP32[$1>>2]|0;
   $609 = (($608) + 24|0);
   $610 = $609;
   $611 = $610;
   HEAP32[$611>>2] = $604;
   $612 = (($610) + 4)|0;
   $613 = $612;
   HEAP32[$613>>2] = $607;
  }
  $614 = HEAP32[$1>>2]|0;
  $615 = (($614) + 34|0);
  $616 = HEAP16[$615>>1]|0;
  $617 = $616&65535;
  $618 = $617 & 2048;
  $619 = ($618|0)!=(0);
  if ($619) {
   $620 = HEAP32[$1>>2]|0;
   $621 = (($620) + 36|0);
   $622 = HEAP8[$621>>0]|0;
   $623 = $622 << 24 >> 24;
   $624 = HEAP32[$1>>2]|0;
   $625 = (($624) + 8|0);
   $626 = HEAP32[$625>>2]|0;
   $627 = HEAP32[$626>>2]|0;
   $628 = ($623|0)==($627|0);
   if ($628) {
    $629 = $revMask1;
    $630 = $629;
    HEAP32[$630>>2] = 0;
    $631 = (($629) + 4)|0;
    $632 = $631;
    HEAP32[$632>>2] = 0;
    $633 = HEAP32[$1>>2]|0;
    $634 = HEAP32[$1>>2]|0;
    $635 = (($634) + 8|0);
    $636 = HEAP32[$635>>2]|0;
    $637 = HEAP32[$pFrom>>2]|0;
    $638 = HEAP32[$nLoop>>2]|0;
    $639 = (($638) - 1)|0;
    $640 = $639&65535;
    $641 = HEAP32[$nLoop>>2]|0;
    $642 = (($641) - 1)|0;
    $643 = HEAP32[$pFrom>>2]|0;
    $644 = (($643) + 24|0);
    $645 = HEAP32[$644>>2]|0;
    $646 = (($645) + ($642<<2)|0);
    $647 = HEAP32[$646>>2]|0;
    $648 = (_wherePathSatisfiesOrderBy($633,$636,$637,0,$640,$647,$revMask1)|0);
    $649 = $648 << 24 >> 24;
    HEAP32[$nOrder>>2] = $649;
    $650 = HEAP32[$nOrder>>2]|0;
    $651 = HEAP32[$1>>2]|0;
    $652 = (($651) + 8|0);
    $653 = HEAP32[$652>>2]|0;
    $654 = HEAP32[$653>>2]|0;
    $655 = ($650|0)==($654|0);
    if ($655) {
     $656 = HEAP32[$1>>2]|0;
     $657 = (($656) + 37|0);
     HEAP8[$657>>0] = 1;
     $658 = $revMask1;
     $659 = $658;
     $660 = HEAP32[$659>>2]|0;
     $661 = (($658) + 4)|0;
     $662 = $661;
     $663 = HEAP32[$662>>2]|0;
     $664 = HEAP32[$1>>2]|0;
     $665 = (($664) + 24|0);
     $666 = $665;
     $667 = $666;
     HEAP32[$667>>2] = $660;
     $668 = (($666) + 4)|0;
     $669 = $668;
     HEAP32[$669>>2] = $663;
    }
   }
  }
 }
 $670 = HEAP32[$pFrom>>2]|0;
 $671 = (($670) + 16|0);
 $672 = HEAP16[$671>>1]|0;
 $673 = HEAP32[$1>>2]|0;
 $674 = (($673) + 32|0);
 HEAP16[$674>>1] = $672;
 $675 = HEAP32[$db>>2]|0;
 $676 = HEAP32[$pSpace>>2]|0;
 _sqlite3DbFree($675,$676);
 HEAP32[$0>>2] = 0;
 $677 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($677|0);
}
function _exprListTableUsage($pMaskSet,$pList) {
 $pMaskSet = $pMaskSet|0;
 $pList = $pList|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0;
 var $9 = 0, $i = 0, $mask = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 16|0;
 $1 = sp + 12|0;
 $i = sp + 8|0;
 $mask = sp;
 HEAP32[$0>>2] = $pMaskSet;
 HEAP32[$1>>2] = $pList;
 $2 = $mask;
 $3 = $2;
 HEAP32[$3>>2] = 0;
 $4 = (($2) + 4)|0;
 $5 = $4;
 HEAP32[$5>>2] = 0;
 $6 = HEAP32[$1>>2]|0;
 $7 = ($6|0)!=(0|0);
 if (!($7)) {
  $35 = $mask;
  $36 = $35;
  $37 = HEAP32[$36>>2]|0;
  $38 = (($35) + 4)|0;
  $39 = $38;
  $40 = HEAP32[$39>>2]|0;
  tempRet0 = $40;
  STACKTOP = sp;return ($37|0);
 }
 HEAP32[$i>>2] = 0;
 while(1) {
  $8 = HEAP32[$i>>2]|0;
  $9 = HEAP32[$1>>2]|0;
  $10 = HEAP32[$9>>2]|0;
  $11 = ($8|0)<($10|0);
  if (!($11)) {
   break;
  }
  $12 = HEAP32[$0>>2]|0;
  $13 = HEAP32[$i>>2]|0;
  $14 = HEAP32[$1>>2]|0;
  $15 = (($14) + 4|0);
  $16 = HEAP32[$15>>2]|0;
  $17 = (($16) + (($13*20)|0)|0);
  $18 = HEAP32[$17>>2]|0;
  $19 = (_exprTableUsage($12,$18)|0);
  $20 = tempRet0;
  $21 = $mask;
  $22 = $21;
  $23 = HEAP32[$22>>2]|0;
  $24 = (($21) + 4)|0;
  $25 = $24;
  $26 = HEAP32[$25>>2]|0;
  $27 = $23 | $19;
  $28 = $26 | $20;
  $29 = $mask;
  $30 = $29;
  HEAP32[$30>>2] = $27;
  $31 = (($29) + 4)|0;
  $32 = $31;
  HEAP32[$32>>2] = $28;
  $33 = HEAP32[$i>>2]|0;
  $34 = (($33) + 1)|0;
  HEAP32[$i>>2] = $34;
 }
 $35 = $mask;
 $36 = $35;
 $37 = HEAP32[$36>>2]|0;
 $38 = (($35) + 4)|0;
 $39 = $38;
 $40 = HEAP32[$39>>2]|0;
 tempRet0 = $40;
 STACKTOP = sp;return ($37|0);
}
function _getMask($pMaskSet,$iCursor) {
 $pMaskSet = $pMaskSet|0;
 $iCursor = $iCursor|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $i = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp;
 $1 = sp + 16|0;
 $2 = sp + 12|0;
 $i = sp + 8|0;
 HEAP32[$1>>2] = $pMaskSet;
 HEAP32[$2>>2] = $iCursor;
 HEAP32[$i>>2] = 0;
 while(1) {
  $3 = HEAP32[$i>>2]|0;
  $4 = HEAP32[$1>>2]|0;
  $5 = HEAP32[$4>>2]|0;
  $6 = ($3|0)<($5|0);
  if (!($6)) {
   label = 6;
   break;
  }
  $7 = HEAP32[$i>>2]|0;
  $8 = HEAP32[$1>>2]|0;
  $9 = (($8) + 4|0);
  $10 = (($9) + ($7<<2)|0);
  $11 = HEAP32[$10>>2]|0;
  $12 = HEAP32[$2>>2]|0;
  $13 = ($11|0)==($12|0);
  $14 = HEAP32[$i>>2]|0;
  if ($13) {
   label = 4;
   break;
  }
  $21 = (($14) + 1)|0;
  HEAP32[$i>>2] = $21;
 }
 if ((label|0) == 4) {
  $15 = (_bitshift64Shl(1,0,($14|0))|0);
  $16 = tempRet0;
  $17 = $0;
  $18 = $17;
  HEAP32[$18>>2] = $15;
  $19 = (($17) + 4)|0;
  $20 = $19;
  HEAP32[$20>>2] = $16;
  $26 = $0;
  $27 = $26;
  $28 = HEAP32[$27>>2]|0;
  $29 = (($26) + 4)|0;
  $30 = $29;
  $31 = HEAP32[$30>>2]|0;
  tempRet0 = $31;
  STACKTOP = sp;return ($28|0);
 }
 else if ((label|0) == 6) {
  $22 = $0;
  $23 = $22;
  HEAP32[$23>>2] = 0;
  $24 = (($22) + 4)|0;
  $25 = $24;
  HEAP32[$25>>2] = 0;
  $26 = $0;
  $27 = $26;
  $28 = HEAP32[$27>>2]|0;
  $29 = (($26) + 4)|0;
  $30 = $29;
  $31 = HEAP32[$30>>2]|0;
  tempRet0 = $31;
  STACKTOP = sp;return ($28|0);
 }
 return 0|0;
}
function _constructAutomaticIndex($pParse,$pWC,$pSrc,$0,$1,$pLevel) {
 $pParse = $pParse|0;
 $pWC = $pWC|0;
 $pSrc = $pSrc|0;
 $0 = $0|0;
 $1 = $1|0;
 $pLevel = $pLevel|0;
 var $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0, $116 = 0, $117 = 0;
 var $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0, $134 = 0, $135 = 0;
 var $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0, $152 = 0, $153 = 0;
 var $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0, $170 = 0, $171 = 0;
 var $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0, $188 = 0, $189 = 0, $19 = 0;
 var $190 = 0, $191 = 0, $192 = 0, $193 = 0, $194 = 0, $195 = 0, $196 = 0, $197 = 0, $198 = 0, $199 = 0, $2 = 0, $20 = 0, $200 = 0, $201 = 0, $202 = 0, $203 = 0, $204 = 0, $205 = 0, $206 = 0, $207 = 0;
 var $208 = 0, $209 = 0, $21 = 0, $210 = 0, $211 = 0, $212 = 0, $213 = 0, $214 = 0, $215 = 0, $216 = 0, $217 = 0, $218 = 0, $219 = 0, $22 = 0, $220 = 0, $221 = 0, $222 = 0, $223 = 0, $224 = 0, $225 = 0;
 var $226 = 0, $227 = 0, $228 = 0, $229 = 0, $23 = 0, $230 = 0, $231 = 0, $232 = 0, $233 = 0, $234 = 0, $235 = 0, $236 = 0, $237 = 0, $238 = 0, $239 = 0, $24 = 0, $240 = 0, $241 = 0, $242 = 0, $243 = 0;
 var $244 = 0, $245 = 0, $246 = 0, $247 = 0, $248 = 0, $249 = 0, $25 = 0, $250 = 0, $251 = 0, $252 = 0, $253 = 0, $254 = 0, $255 = 0, $256 = 0, $257 = 0, $258 = 0, $259 = 0, $26 = 0, $260 = 0, $261 = 0;
 var $262 = 0, $263 = 0, $264 = 0, $265 = 0, $266 = 0, $267 = 0, $268 = 0, $269 = 0, $27 = 0, $270 = 0, $271 = 0, $272 = 0, $273 = 0, $274 = 0, $275 = 0, $276 = 0, $277 = 0, $278 = 0, $279 = 0, $28 = 0;
 var $280 = 0, $281 = 0, $282 = 0, $283 = 0, $284 = 0, $285 = 0, $286 = 0, $287 = 0, $288 = 0, $289 = 0, $29 = 0, $290 = 0, $291 = 0, $292 = 0, $293 = 0, $294 = 0, $295 = 0, $296 = 0, $297 = 0, $298 = 0;
 var $299 = 0, $3 = 0, $30 = 0, $300 = 0, $301 = 0, $302 = 0, $303 = 0, $304 = 0, $305 = 0, $306 = 0, $307 = 0, $308 = 0, $309 = 0, $31 = 0, $310 = 0, $311 = 0, $312 = 0, $313 = 0, $314 = 0, $315 = 0;
 var $316 = 0, $317 = 0, $318 = 0, $319 = 0, $32 = 0, $320 = 0, $321 = 0, $322 = 0, $323 = 0, $324 = 0, $325 = 0, $326 = 0, $327 = 0, $328 = 0, $329 = 0, $33 = 0, $330 = 0, $331 = 0, $332 = 0, $333 = 0;
 var $334 = 0, $335 = 0, $336 = 0, $337 = 0, $338 = 0, $339 = 0, $34 = 0, $340 = 0, $341 = 0, $342 = 0, $343 = 0, $344 = 0, $345 = 0, $346 = 0, $347 = 0, $348 = 0, $349 = 0, $35 = 0, $350 = 0, $351 = 0;
 var $352 = 0, $353 = 0, $354 = 0, $355 = 0, $356 = 0, $357 = 0, $358 = 0, $359 = 0, $36 = 0, $360 = 0, $361 = 0, $362 = 0, $363 = 0, $364 = 0, $365 = 0, $366 = 0, $367 = 0, $368 = 0, $369 = 0, $37 = 0;
 var $370 = 0, $371 = 0, $372 = 0, $373 = 0, $374 = 0, $375 = 0, $376 = 0, $377 = 0, $378 = 0, $379 = 0, $38 = 0, $380 = 0, $381 = 0, $382 = 0, $383 = 0, $384 = 0, $385 = 0, $386 = 0, $387 = 0, $388 = 0;
 var $389 = 0, $39 = 0, $390 = 0, $391 = 0, $392 = 0, $393 = 0, $394 = 0, $395 = 0, $396 = 0, $397 = 0, $398 = 0, $399 = 0, $4 = 0, $40 = 0, $400 = 0, $401 = 0, $402 = 0, $403 = 0, $404 = 0, $405 = 0;
 var $406 = 0, $407 = 0, $408 = 0, $409 = 0, $41 = 0, $410 = 0, $411 = 0, $412 = 0, $413 = 0, $414 = 0, $415 = 0, $416 = 0, $417 = 0, $418 = 0, $419 = 0, $42 = 0, $420 = 0, $421 = 0, $422 = 0, $423 = 0;
 var $424 = 0, $425 = 0, $426 = 0, $427 = 0, $428 = 0, $429 = 0, $43 = 0, $430 = 0, $431 = 0, $432 = 0, $433 = 0, $434 = 0, $435 = 0, $436 = 0, $437 = 0, $438 = 0, $439 = 0, $44 = 0, $440 = 0, $441 = 0;
 var $442 = 0, $443 = 0, $444 = 0, $445 = 0, $446 = 0, $447 = 0, $448 = 0, $449 = 0, $45 = 0, $450 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0;
 var $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0;
 var $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0;
 var $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $addrInit = 0, $addrTop = 0, $cMask = 0, $cMask2 = 0, $extraCols = 0, $i = 0, $iCol = 0, $iCol1 = 0, $idxCols = 0, $mxBitCol = 0, $n = 0;
 var $nKeyCol = 0, $pColl = 0, $pIdx = 0, $pLoop = 0, $pTable = 0, $pTerm = 0, $pWCEnd = 0, $pX = 0, $regRecord = 0, $sentWarning = 0, $v = 0, $vararg_buffer = 0, $vararg_ptr1 = 0, $zNotUsed = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 144|0;
 $vararg_buffer = sp + 40|0;
 $2 = sp + 132|0;
 $3 = sp + 128|0;
 $4 = sp + 124|0;
 $5 = sp + 32|0;
 $6 = sp + 120|0;
 $nKeyCol = sp + 116|0;
 $pTerm = sp + 112|0;
 $pWCEnd = sp + 108|0;
 $pIdx = sp + 104|0;
 $v = sp + 100|0;
 $addrInit = sp + 96|0;
 $pTable = sp + 92|0;
 $addrTop = sp + 88|0;
 $regRecord = sp + 84|0;
 $n = sp + 80|0;
 $i = sp + 76|0;
 $mxBitCol = sp + 72|0;
 $pColl = sp + 68|0;
 $pLoop = sp + 64|0;
 $zNotUsed = sp + 60|0;
 $idxCols = sp + 24|0;
 $extraCols = sp + 16|0;
 $sentWarning = sp + 136|0;
 $iCol = sp + 56|0;
 $cMask = sp + 8|0;
 $iCol1 = sp + 52|0;
 $cMask2 = sp;
 $pX = sp + 48|0;
 HEAP32[$2>>2] = $pParse;
 HEAP32[$3>>2] = $pWC;
 HEAP32[$4>>2] = $pSrc;
 $7 = $5;
 $8 = $7;
 HEAP32[$8>>2] = $0;
 $9 = (($7) + 4)|0;
 $10 = $9;
 HEAP32[$10>>2] = $1;
 HEAP32[$6>>2] = $pLevel;
 HEAP8[$sentWarning>>0] = 0;
 $11 = HEAP32[$2>>2]|0;
 $12 = (($11) + 8|0);
 $13 = HEAP32[$12>>2]|0;
 HEAP32[$v>>2] = $13;
 $14 = HEAP32[$2>>2]|0;
 $15 = (_sqlite3CodeOnce($14)|0);
 HEAP32[$addrInit>>2] = $15;
 HEAP32[$nKeyCol>>2] = 0;
 $16 = HEAP32[$4>>2]|0;
 $17 = (($16) + 16|0);
 $18 = HEAP32[$17>>2]|0;
 HEAP32[$pTable>>2] = $18;
 $19 = HEAP32[$3>>2]|0;
 $20 = (($19) + 12|0);
 $21 = HEAP32[$20>>2]|0;
 $22 = HEAP32[$3>>2]|0;
 $23 = (($22) + 20|0);
 $24 = HEAP32[$23>>2]|0;
 $25 = (($24) + (($21*48)|0)|0);
 HEAP32[$pWCEnd>>2] = $25;
 $26 = HEAP32[$6>>2]|0;
 $27 = (($26) + 56|0);
 $28 = HEAP32[$27>>2]|0;
 HEAP32[$pLoop>>2] = $28;
 $29 = $idxCols;
 $30 = $29;
 HEAP32[$30>>2] = 0;
 $31 = (($29) + 4)|0;
 $32 = $31;
 HEAP32[$32>>2] = 0;
 $33 = HEAP32[$3>>2]|0;
 $34 = (($33) + 20|0);
 $35 = HEAP32[$34>>2]|0;
 HEAP32[$pTerm>>2] = $35;
 while(1) {
  $36 = HEAP32[$pTerm>>2]|0;
  $37 = HEAP32[$pWCEnd>>2]|0;
  $38 = ($36>>>0)<($37>>>0);
  if (!($38)) {
   break;
  }
  $39 = HEAP32[$pTerm>>2]|0;
  $40 = HEAP32[$4>>2]|0;
  $41 = $5;
  $42 = $41;
  $43 = HEAP32[$42>>2]|0;
  $44 = (($41) + 4)|0;
  $45 = $44;
  $46 = HEAP32[$45>>2]|0;
  $47 = (_termCanDriveIndex($39,$40,$43,$46)|0);
  $48 = ($47|0)!=(0);
  if ($48) {
   $49 = HEAP32[$pTerm>>2]|0;
   $50 = (($49) + 12|0);
   $51 = HEAP32[$50>>2]|0;
   HEAP32[$iCol>>2] = $51;
   $52 = HEAP32[$iCol>>2]|0;
   $53 = ($52|0)>=(64);
   if ($53) {
    $59 = 0;$62 = -2147483648;
   } else {
    $54 = HEAP32[$iCol>>2]|0;
    $55 = (_bitshift64Shl(1,0,($54|0))|0);
    $56 = tempRet0;
    $59 = $55;$62 = $56;
   }
   $57 = $cMask;
   $58 = $57;
   HEAP32[$58>>2] = $59;
   $60 = (($57) + 4)|0;
   $61 = $60;
   HEAP32[$61>>2] = $62;
   $63 = HEAP8[$sentWarning>>0]|0;
   $64 = ($63<<24>>24)!=(0);
   if (!($64)) {
    $65 = HEAP32[$pTable>>2]|0;
    $66 = HEAP32[$65>>2]|0;
    $67 = HEAP32[$iCol>>2]|0;
    $68 = HEAP32[$pTable>>2]|0;
    $69 = (($68) + 4|0);
    $70 = HEAP32[$69>>2]|0;
    $71 = (($70) + (($67*24)|0)|0);
    $72 = HEAP32[$71>>2]|0;
    HEAP32[$vararg_buffer>>2] = $66;
    $vararg_ptr1 = (($vararg_buffer) + 4|0);
    HEAP32[$vararg_ptr1>>2] = $72;
    _sqlite3_log(284,23984,$vararg_buffer);
    HEAP8[$sentWarning>>0] = 1;
   }
   $73 = $idxCols;
   $74 = $73;
   $75 = HEAP32[$74>>2]|0;
   $76 = (($73) + 4)|0;
   $77 = $76;
   $78 = HEAP32[$77>>2]|0;
   $79 = $cMask;
   $80 = $79;
   $81 = HEAP32[$80>>2]|0;
   $82 = (($79) + 4)|0;
   $83 = $82;
   $84 = HEAP32[$83>>2]|0;
   $85 = $75 & $81;
   $86 = $78 & $84;
   $87 = ($85|0)==(0);
   $88 = ($86|0)==(0);
   $89 = $87 & $88;
   if ($89) {
    $90 = HEAP32[$2>>2]|0;
    $91 = HEAP32[$90>>2]|0;
    $92 = HEAP32[$pLoop>>2]|0;
    $93 = HEAP32[$nKeyCol>>2]|0;
    $94 = (($93) + 1)|0;
    $95 = (_whereLoopResize($91,$92,$94)|0);
    $96 = ($95|0)!=(0);
    if ($96) {
     label = 42;
     break;
    }
    $97 = HEAP32[$pTerm>>2]|0;
    $98 = HEAP32[$nKeyCol>>2]|0;
    $99 = (($98) + 1)|0;
    HEAP32[$nKeyCol>>2] = $99;
    $100 = HEAP32[$pLoop>>2]|0;
    $101 = (($100) + 44|0);
    $102 = HEAP32[$101>>2]|0;
    $103 = (($102) + ($98<<2)|0);
    HEAP32[$103>>2] = $97;
    $104 = $cMask;
    $105 = $104;
    $106 = HEAP32[$105>>2]|0;
    $107 = (($104) + 4)|0;
    $108 = $107;
    $109 = HEAP32[$108>>2]|0;
    $110 = $idxCols;
    $111 = $110;
    $112 = HEAP32[$111>>2]|0;
    $113 = (($110) + 4)|0;
    $114 = $113;
    $115 = HEAP32[$114>>2]|0;
    $116 = $112 | $106;
    $117 = $115 | $109;
    $118 = $idxCols;
    $119 = $118;
    HEAP32[$119>>2] = $116;
    $120 = (($118) + 4)|0;
    $121 = $120;
    HEAP32[$121>>2] = $117;
   }
  }
  $122 = HEAP32[$pTerm>>2]|0;
  $123 = (($122) + 48|0);
  HEAP32[$pTerm>>2] = $123;
 }
 if ((label|0) == 42) {
  STACKTOP = sp;return;
 }
 $124 = HEAP32[$nKeyCol>>2]|0;
 $125 = $124&65535;
 $126 = HEAP32[$pLoop>>2]|0;
 $127 = (($126) + 40|0);
 HEAP16[$127>>1] = $125;
 $128 = HEAP32[$pLoop>>2]|0;
 $129 = (($128) + 24|0);
 HEAP16[$129>>1] = $125;
 $130 = HEAP32[$pLoop>>2]|0;
 $131 = (($130) + 36|0);
 HEAP32[$131>>2] = 16961;
 $132 = HEAP32[$4>>2]|0;
 $133 = (($132) + 56|0);
 $134 = $133;
 $135 = $134;
 $136 = HEAP32[$135>>2]|0;
 $137 = (($134) + 4)|0;
 $138 = $137;
 $139 = HEAP32[$138>>2]|0;
 $140 = $idxCols;
 $141 = $140;
 $142 = HEAP32[$141>>2]|0;
 $143 = (($140) + 4)|0;
 $144 = $143;
 $145 = HEAP32[$144>>2]|0;
 $146 = $142 ^ -1;
 $147 = $145 ^ -1;
 $148 = $147 | -2147483648;
 $149 = $136 & $146;
 $150 = $139 & $148;
 $151 = $extraCols;
 $152 = $151;
 HEAP32[$152>>2] = $149;
 $153 = (($151) + 4)|0;
 $154 = $153;
 HEAP32[$154>>2] = $150;
 $155 = HEAP32[$pTable>>2]|0;
 $156 = (($155) + 38|0);
 $157 = HEAP16[$156>>1]|0;
 $158 = $157 << 16 >> 16;
 $159 = ($158|0)>=(63);
 if ($159) {
  $164 = 63;
 } else {
  $160 = HEAP32[$pTable>>2]|0;
  $161 = (($160) + 38|0);
  $162 = HEAP16[$161>>1]|0;
  $163 = $162 << 16 >> 16;
  $164 = $163;
 }
 HEAP32[$mxBitCol>>2] = $164;
 HEAP32[$i>>2] = 0;
 while(1) {
  $165 = HEAP32[$i>>2]|0;
  $166 = HEAP32[$mxBitCol>>2]|0;
  $167 = ($165|0)<($166|0);
  if (!($167)) {
   break;
  }
  $168 = $extraCols;
  $169 = $168;
  $170 = HEAP32[$169>>2]|0;
  $171 = (($168) + 4)|0;
  $172 = $171;
  $173 = HEAP32[$172>>2]|0;
  $174 = HEAP32[$i>>2]|0;
  $175 = (_bitshift64Shl(1,0,($174|0))|0);
  $176 = tempRet0;
  $177 = $170 & $175;
  $178 = $173 & $176;
  $179 = ($177|0)!=(0);
  $180 = ($178|0)!=(0);
  $181 = $179 | $180;
  if ($181) {
   $182 = HEAP32[$nKeyCol>>2]|0;
   $183 = (($182) + 1)|0;
   HEAP32[$nKeyCol>>2] = $183;
  }
  $184 = HEAP32[$i>>2]|0;
  $185 = (($184) + 1)|0;
  HEAP32[$i>>2] = $185;
 }
 $186 = HEAP32[$4>>2]|0;
 $187 = (($186) + 56|0);
 $188 = $187;
 $189 = $188;
 $190 = HEAP32[$189>>2]|0;
 $191 = (($188) + 4)|0;
 $192 = $191;
 $193 = HEAP32[$192>>2]|0;
 $194 = $193 & -2147483648;
 $195 = (0)!=(0);
 $196 = ($194|0)!=(0);
 $197 = $195 | $196;
 if ($197) {
  $198 = HEAP32[$pTable>>2]|0;
  $199 = (($198) + 38|0);
  $200 = HEAP16[$199>>1]|0;
  $201 = $200 << 16 >> 16;
  $202 = (($201) - 64)|0;
  $203 = (($202) + 1)|0;
  $204 = HEAP32[$nKeyCol>>2]|0;
  $205 = (($204) + ($203))|0;
  HEAP32[$nKeyCol>>2] = $205;
 }
 $206 = HEAP32[$pLoop>>2]|0;
 $207 = (($206) + 36|0);
 $208 = HEAP32[$207>>2]|0;
 $209 = $208 | 65;
 HEAP32[$207>>2] = $209;
 $210 = HEAP32[$2>>2]|0;
 $211 = HEAP32[$210>>2]|0;
 $212 = HEAP32[$nKeyCol>>2]|0;
 $213 = (($212) + 1)|0;
 $214 = $213&65535;
 $215 = (_sqlite3AllocateIndexObject($211,$214,0,$zNotUsed)|0);
 HEAP32[$pIdx>>2] = $215;
 $216 = HEAP32[$pIdx>>2]|0;
 $217 = ($216|0)==(0|0);
 if ($217) {
  STACKTOP = sp;return;
 }
 $218 = HEAP32[$pIdx>>2]|0;
 $219 = HEAP32[$pLoop>>2]|0;
 $220 = (($219) + 24|0);
 $221 = (($220) + 4|0);
 HEAP32[$221>>2] = $218;
 $222 = HEAP32[$pIdx>>2]|0;
 HEAP32[$222>>2] = 24016;
 $223 = HEAP32[$pTable>>2]|0;
 $224 = HEAP32[$pIdx>>2]|0;
 $225 = (($224) + 12|0);
 HEAP32[$225>>2] = $223;
 HEAP32[$n>>2] = 0;
 $226 = $idxCols;
 $227 = $226;
 HEAP32[$227>>2] = 0;
 $228 = (($226) + 4)|0;
 $229 = $228;
 HEAP32[$229>>2] = 0;
 $230 = HEAP32[$3>>2]|0;
 $231 = (($230) + 20|0);
 $232 = HEAP32[$231>>2]|0;
 HEAP32[$pTerm>>2] = $232;
 while(1) {
  $233 = HEAP32[$pTerm>>2]|0;
  $234 = HEAP32[$pWCEnd>>2]|0;
  $235 = ($233>>>0)<($234>>>0);
  if (!($235)) {
   break;
  }
  $236 = HEAP32[$pTerm>>2]|0;
  $237 = HEAP32[$4>>2]|0;
  $238 = $5;
  $239 = $238;
  $240 = HEAP32[$239>>2]|0;
  $241 = (($238) + 4)|0;
  $242 = $241;
  $243 = HEAP32[$242>>2]|0;
  $244 = (_termCanDriveIndex($236,$237,$240,$243)|0);
  $245 = ($244|0)!=(0);
  if ($245) {
   $246 = HEAP32[$pTerm>>2]|0;
   $247 = (($246) + 12|0);
   $248 = HEAP32[$247>>2]|0;
   HEAP32[$iCol1>>2] = $248;
   $249 = HEAP32[$iCol1>>2]|0;
   $250 = ($249|0)>=(64);
   if ($250) {
    $256 = 0;$259 = -2147483648;
   } else {
    $251 = HEAP32[$iCol1>>2]|0;
    $252 = (_bitshift64Shl(1,0,($251|0))|0);
    $253 = tempRet0;
    $256 = $252;$259 = $253;
   }
   $254 = $cMask2;
   $255 = $254;
   HEAP32[$255>>2] = $256;
   $257 = (($254) + 4)|0;
   $258 = $257;
   HEAP32[$258>>2] = $259;
   $260 = $idxCols;
   $261 = $260;
   $262 = HEAP32[$261>>2]|0;
   $263 = (($260) + 4)|0;
   $264 = $263;
   $265 = HEAP32[$264>>2]|0;
   $266 = $cMask2;
   $267 = $266;
   $268 = HEAP32[$267>>2]|0;
   $269 = (($266) + 4)|0;
   $270 = $269;
   $271 = HEAP32[$270>>2]|0;
   $272 = $262 & $268;
   $273 = $265 & $271;
   $274 = ($272|0)==(0);
   $275 = ($273|0)==(0);
   $276 = $274 & $275;
   if ($276) {
    $277 = HEAP32[$pTerm>>2]|0;
    $278 = HEAP32[$277>>2]|0;
    HEAP32[$pX>>2] = $278;
    $279 = $cMask2;
    $280 = $279;
    $281 = HEAP32[$280>>2]|0;
    $282 = (($279) + 4)|0;
    $283 = $282;
    $284 = HEAP32[$283>>2]|0;
    $285 = $idxCols;
    $286 = $285;
    $287 = HEAP32[$286>>2]|0;
    $288 = (($285) + 4)|0;
    $289 = $288;
    $290 = HEAP32[$289>>2]|0;
    $291 = $287 | $281;
    $292 = $290 | $284;
    $293 = $idxCols;
    $294 = $293;
    HEAP32[$294>>2] = $291;
    $295 = (($293) + 4)|0;
    $296 = $295;
    HEAP32[$296>>2] = $292;
    $297 = HEAP32[$pTerm>>2]|0;
    $298 = (($297) + 12|0);
    $299 = HEAP32[$298>>2]|0;
    $300 = $299&65535;
    $301 = HEAP32[$n>>2]|0;
    $302 = HEAP32[$pIdx>>2]|0;
    $303 = (($302) + 4|0);
    $304 = HEAP32[$303>>2]|0;
    $305 = (($304) + ($301<<1)|0);
    HEAP16[$305>>1] = $300;
    $306 = HEAP32[$2>>2]|0;
    $307 = HEAP32[$pX>>2]|0;
    $308 = (($307) + 12|0);
    $309 = HEAP32[$308>>2]|0;
    $310 = HEAP32[$pX>>2]|0;
    $311 = (($310) + 16|0);
    $312 = HEAP32[$311>>2]|0;
    $313 = (_sqlite3BinaryCompareCollSeq($306,$309,$312)|0);
    HEAP32[$pColl>>2] = $313;
    $314 = HEAP32[$pColl>>2]|0;
    $315 = ($314|0)!=(0|0);
    if ($315) {
     $316 = HEAP32[$pColl>>2]|0;
     $317 = HEAP32[$316>>2]|0;
     $323 = $317;
    } else {
     $323 = 3320;
    }
    $318 = HEAP32[$n>>2]|0;
    $319 = HEAP32[$pIdx>>2]|0;
    $320 = (($319) + 32|0);
    $321 = HEAP32[$320>>2]|0;
    $322 = (($321) + ($318<<2)|0);
    HEAP32[$322>>2] = $323;
    $324 = HEAP32[$n>>2]|0;
    $325 = (($324) + 1)|0;
    HEAP32[$n>>2] = $325;
   }
  }
  $326 = HEAP32[$pTerm>>2]|0;
  $327 = (($326) + 48|0);
  HEAP32[$pTerm>>2] = $327;
 }
 HEAP32[$i>>2] = 0;
 while(1) {
  $328 = HEAP32[$i>>2]|0;
  $329 = HEAP32[$mxBitCol>>2]|0;
  $330 = ($328|0)<($329|0);
  if (!($330)) {
   break;
  }
  $331 = $extraCols;
  $332 = $331;
  $333 = HEAP32[$332>>2]|0;
  $334 = (($331) + 4)|0;
  $335 = $334;
  $336 = HEAP32[$335>>2]|0;
  $337 = HEAP32[$i>>2]|0;
  $338 = (_bitshift64Shl(1,0,($337|0))|0);
  $339 = tempRet0;
  $340 = $333 & $338;
  $341 = $336 & $339;
  $342 = ($340|0)!=(0);
  $343 = ($341|0)!=(0);
  $344 = $342 | $343;
  if ($344) {
   $345 = HEAP32[$i>>2]|0;
   $346 = $345&65535;
   $347 = HEAP32[$n>>2]|0;
   $348 = HEAP32[$pIdx>>2]|0;
   $349 = (($348) + 4|0);
   $350 = HEAP32[$349>>2]|0;
   $351 = (($350) + ($347<<1)|0);
   HEAP16[$351>>1] = $346;
   $352 = HEAP32[$n>>2]|0;
   $353 = HEAP32[$pIdx>>2]|0;
   $354 = (($353) + 32|0);
   $355 = HEAP32[$354>>2]|0;
   $356 = (($355) + ($352<<2)|0);
   HEAP32[$356>>2] = 3320;
   $357 = HEAP32[$n>>2]|0;
   $358 = (($357) + 1)|0;
   HEAP32[$n>>2] = $358;
  }
  $359 = HEAP32[$i>>2]|0;
  $360 = (($359) + 1)|0;
  HEAP32[$i>>2] = $360;
 }
 $361 = HEAP32[$4>>2]|0;
 $362 = (($361) + 56|0);
 $363 = $362;
 $364 = $363;
 $365 = HEAP32[$364>>2]|0;
 $366 = (($363) + 4)|0;
 $367 = $366;
 $368 = HEAP32[$367>>2]|0;
 $369 = $368 & -2147483648;
 $370 = (0)!=(0);
 $371 = ($369|0)!=(0);
 $372 = $370 | $371;
 L55: do {
  if ($372) {
   HEAP32[$i>>2] = 63;
   while(1) {
    $373 = HEAP32[$i>>2]|0;
    $374 = HEAP32[$pTable>>2]|0;
    $375 = (($374) + 38|0);
    $376 = HEAP16[$375>>1]|0;
    $377 = $376 << 16 >> 16;
    $378 = ($373|0)<($377|0);
    if (!($378)) {
     break L55;
    }
    $379 = HEAP32[$i>>2]|0;
    $380 = $379&65535;
    $381 = HEAP32[$n>>2]|0;
    $382 = HEAP32[$pIdx>>2]|0;
    $383 = (($382) + 4|0);
    $384 = HEAP32[$383>>2]|0;
    $385 = (($384) + ($381<<1)|0);
    HEAP16[$385>>1] = $380;
    $386 = HEAP32[$n>>2]|0;
    $387 = HEAP32[$pIdx>>2]|0;
    $388 = (($387) + 32|0);
    $389 = HEAP32[$388>>2]|0;
    $390 = (($389) + ($386<<2)|0);
    HEAP32[$390>>2] = 3320;
    $391 = HEAP32[$n>>2]|0;
    $392 = (($391) + 1)|0;
    HEAP32[$n>>2] = $392;
    $393 = HEAP32[$i>>2]|0;
    $394 = (($393) + 1)|0;
    HEAP32[$i>>2] = $394;
   }
  }
 } while(0);
 $395 = HEAP32[$n>>2]|0;
 $396 = HEAP32[$pIdx>>2]|0;
 $397 = (($396) + 4|0);
 $398 = HEAP32[$397>>2]|0;
 $399 = (($398) + ($395<<1)|0);
 HEAP16[$399>>1] = -1;
 $400 = HEAP32[$n>>2]|0;
 $401 = HEAP32[$pIdx>>2]|0;
 $402 = (($401) + 32|0);
 $403 = HEAP32[$402>>2]|0;
 $404 = (($403) + ($400<<2)|0);
 HEAP32[$404>>2] = 3320;
 $405 = HEAP32[$2>>2]|0;
 $406 = (($405) + 68|0);
 $407 = HEAP32[$406>>2]|0;
 $408 = (($407) + 1)|0;
 HEAP32[$406>>2] = $408;
 $409 = HEAP32[$6>>2]|0;
 $410 = (($409) + 8|0);
 HEAP32[$410>>2] = $407;
 $411 = HEAP32[$v>>2]|0;
 $412 = HEAP32[$6>>2]|0;
 $413 = (($412) + 8|0);
 $414 = HEAP32[$413>>2]|0;
 $415 = HEAP32[$nKeyCol>>2]|0;
 $416 = (($415) + 1)|0;
 (_sqlite3VdbeAddOp2($411,56,$414,$416)|0);
 $417 = HEAP32[$2>>2]|0;
 $418 = HEAP32[$pIdx>>2]|0;
 _sqlite3VdbeSetP4KeyInfo($417,$418);
 $419 = HEAP32[$v>>2]|0;
 $420 = HEAP32[$6>>2]|0;
 $421 = (($420) + 4|0);
 $422 = HEAP32[$421>>2]|0;
 $423 = (_sqlite3VdbeAddOp1($419,108,$422)|0);
 HEAP32[$addrTop>>2] = $423;
 $424 = HEAP32[$2>>2]|0;
 $425 = (_sqlite3GetTempReg($424)|0);
 HEAP32[$regRecord>>2] = $425;
 $426 = HEAP32[$2>>2]|0;
 $427 = HEAP32[$pIdx>>2]|0;
 $428 = HEAP32[$6>>2]|0;
 $429 = (($428) + 4|0);
 $430 = HEAP32[$429>>2]|0;
 $431 = HEAP32[$regRecord>>2]|0;
 (_sqlite3GenerateIndexKey($426,$427,$430,$431,0,0,0,0)|0);
 $432 = HEAP32[$v>>2]|0;
 $433 = HEAP32[$6>>2]|0;
 $434 = (($433) + 8|0);
 $435 = HEAP32[$434>>2]|0;
 $436 = HEAP32[$regRecord>>2]|0;
 (_sqlite3VdbeAddOp2($432,110,$435,$436)|0);
 $437 = HEAP32[$v>>2]|0;
 _sqlite3VdbeChangeP5($437,16);
 $438 = HEAP32[$v>>2]|0;
 $439 = HEAP32[$6>>2]|0;
 $440 = (($439) + 4|0);
 $441 = HEAP32[$440>>2]|0;
 $442 = HEAP32[$addrTop>>2]|0;
 $443 = (($442) + 1)|0;
 (_sqlite3VdbeAddOp2($438,9,$441,$443)|0);
 $444 = HEAP32[$v>>2]|0;
 _sqlite3VdbeChangeP5($444,3);
 $445 = HEAP32[$v>>2]|0;
 $446 = HEAP32[$addrTop>>2]|0;
 _sqlite3VdbeJumpHere($445,$446);
 $447 = HEAP32[$2>>2]|0;
 $448 = HEAP32[$regRecord>>2]|0;
 _sqlite3ReleaseTempReg($447,$448);
 $449 = HEAP32[$v>>2]|0;
 $450 = HEAP32[$addrInit>>2]|0;
 _sqlite3VdbeJumpHere($449,$450);
 STACKTOP = sp;return;
}
function _explainOneScan($pParse,$pTabList,$pLevel,$iLevel,$iFrom,$wctrlFlags) {
 $pParse = $pParse|0;
 $pTabList = $pTabList|0;
 $pLevel = $pLevel|0;
 $iLevel = $iLevel|0;
 $iFrom = $iFrom|0;
 $wctrlFlags = $wctrlFlags|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0;
 var $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0;
 var $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0;
 var $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0;
 var $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0;
 var $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $db = 0, $flags = 0, $iId = 0, $isSearch = 0, $pIdx = 0, $pItem = 0, $pLoop = 0, $str = 0, $v = 0, $vararg_buffer10 = 0, $vararg_ptr13 = 0, $zBuf = 0, $zFmt = 0;
 var $zMsg = 0, $zRange = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 208|0;
 $vararg_buffer10 = sp;
 $0 = sp + 96|0;
 $1 = sp + 92|0;
 $2 = sp + 88|0;
 $3 = sp + 84|0;
 $4 = sp + 80|0;
 $5 = sp + 100|0;
 $pItem = sp + 76|0;
 $v = sp + 72|0;
 $db = sp + 68|0;
 $iId = sp + 64|0;
 $isSearch = sp + 60|0;
 $pLoop = sp + 56|0;
 $flags = sp + 52|0;
 $zMsg = sp + 48|0;
 $str = sp + 20|0;
 $zBuf = sp + 104|0;
 $zFmt = sp + 16|0;
 $pIdx = sp + 12|0;
 $zRange = sp + 8|0;
 HEAP32[$0>>2] = $pParse;
 HEAP32[$1>>2] = $pTabList;
 HEAP32[$2>>2] = $pLevel;
 HEAP32[$3>>2] = $iLevel;
 HEAP32[$4>>2] = $iFrom;
 HEAP16[$5>>1] = $wctrlFlags;
 $6 = HEAP32[$0>>2]|0;
 $7 = (($6) + 450|0);
 $8 = HEAP8[$7>>0]|0;
 $9 = $8&255;
 $10 = ($9|0)==(2);
 if (!($10)) {
  STACKTOP = sp;return;
 }
 $11 = HEAP32[$2>>2]|0;
 $12 = (($11) + 36|0);
 $13 = HEAP8[$12>>0]|0;
 $14 = $13&255;
 $15 = HEAP32[$1>>2]|0;
 $16 = (($15) + 8|0);
 $17 = (($16) + (($14*72)|0)|0);
 HEAP32[$pItem>>2] = $17;
 $18 = HEAP32[$0>>2]|0;
 $19 = (($18) + 8|0);
 $20 = HEAP32[$19>>2]|0;
 HEAP32[$v>>2] = $20;
 $21 = HEAP32[$0>>2]|0;
 $22 = HEAP32[$21>>2]|0;
 HEAP32[$db>>2] = $22;
 $23 = HEAP32[$0>>2]|0;
 $24 = (($23) + 464|0);
 $25 = HEAP32[$24>>2]|0;
 HEAP32[$iId>>2] = $25;
 $26 = HEAP32[$2>>2]|0;
 $27 = (($26) + 56|0);
 $28 = HEAP32[$27>>2]|0;
 HEAP32[$pLoop>>2] = $28;
 $29 = HEAP32[$pLoop>>2]|0;
 $30 = (($29) + 36|0);
 $31 = HEAP32[$30>>2]|0;
 HEAP32[$flags>>2] = $31;
 $32 = HEAP32[$flags>>2]|0;
 $33 = $32 & 8192;
 $34 = ($33|0)!=(0);
 if ($34) {
  STACKTOP = sp;return;
 }
 $35 = HEAP16[$5>>1]|0;
 $36 = $35&65535;
 $37 = $36 & 64;
 $38 = ($37|0)!=(0);
 if ($38) {
  STACKTOP = sp;return;
 }
 $39 = HEAP32[$flags>>2]|0;
 $40 = $39 & 48;
 $41 = ($40|0)!=(0);
 do {
  if ($41) {
   $55 = 1;
  } else {
   $42 = HEAP32[$flags>>2]|0;
   $43 = $42 & 1024;
   $44 = ($43|0)==(0);
   if ($44) {
    $45 = HEAP32[$pLoop>>2]|0;
    $46 = (($45) + 24|0);
    $47 = HEAP16[$46>>1]|0;
    $48 = $47&65535;
    $49 = ($48|0)>(0);
    if ($49) {
     $55 = 1;
     break;
    }
   }
   $50 = HEAP16[$5>>1]|0;
   $51 = $50&65535;
   $52 = $51 & 3;
   $53 = ($52|0)!=(0);
   $55 = $53;
  }
 } while(0);
 $54 = $55&1;
 HEAP32[$isSearch>>2] = $54;
 _sqlite3StrAccumInit($str,$zBuf,100,1000000000);
 $56 = HEAP32[$db>>2]|0;
 HEAP32[$str>>2] = $56;
 $57 = HEAP32[$isSearch>>2]|0;
 $58 = ($57|0)!=(0);
 $59 = $58 ? 23640 : 23648;
 _sqlite3StrAccumAppendAll($str,$59);
 $60 = HEAP32[$pItem>>2]|0;
 $61 = (($60) + 20|0);
 $62 = HEAP32[$61>>2]|0;
 $63 = ($62|0)!=(0|0);
 $64 = HEAP32[$pItem>>2]|0;
 if ($63) {
  $65 = (($64) + 38|0);
  $66 = HEAP8[$65>>0]|0;
  $67 = $66&255;
  HEAP32[$vararg_buffer10>>2] = $67;
  _sqlite3XPrintf($str,0,23656,$vararg_buffer10);
 } else {
  $68 = (($64) + 8|0);
  $69 = HEAP32[$68>>2]|0;
  HEAP32[$vararg_buffer10>>2] = $69;
  _sqlite3XPrintf($str,0,23672,$vararg_buffer10);
 }
 $70 = HEAP32[$pItem>>2]|0;
 $71 = (($70) + 12|0);
 $72 = HEAP32[$71>>2]|0;
 $73 = ($72|0)!=(0|0);
 if ($73) {
  $74 = HEAP32[$pItem>>2]|0;
  $75 = (($74) + 12|0);
  $76 = HEAP32[$75>>2]|0;
  HEAP32[$vararg_buffer10>>2] = $76;
  _sqlite3XPrintf($str,0,23688,$vararg_buffer10);
 }
 $77 = HEAP32[$flags>>2]|0;
 $78 = $77 & 1280;
 $79 = ($78|0)==(0);
 do {
  if ($79) {
   HEAP32[$zFmt>>2] = 0;
   $80 = HEAP32[$pLoop>>2]|0;
   $81 = (($80) + 24|0);
   $82 = (($81) + 4|0);
   $83 = HEAP32[$82>>2]|0;
   HEAP32[$pIdx>>2] = $83;
   $84 = HEAP32[$pItem>>2]|0;
   $85 = (($84) + 16|0);
   $86 = HEAP32[$85>>2]|0;
   $87 = (($86) + 44|0);
   $88 = HEAP8[$87>>0]|0;
   $89 = $88&255;
   $90 = $89 & 32;
   $91 = ($90|0)==(0);
   if ($91) {
    label = 18;
   } else {
    $92 = HEAP32[$pIdx>>2]|0;
    $93 = (($92) + 55|0);
    $94 = HEAP8[$93>>0]|0;
    $95 = $94 & 3;
    $96 = $95&255;
    $97 = ($96|0)==(2);
    if ($97) {
     $98 = HEAP32[$isSearch>>2]|0;
     $99 = ($98|0)!=(0);
     if ($99) {
      HEAP32[$zFmt>>2] = 23696;
     }
    } else {
     label = 18;
    }
   }
   do {
    if ((label|0) == 18) {
     $100 = HEAP32[$flags>>2]|0;
     $101 = $100 & 16384;
     $102 = ($101|0)!=(0);
     if ($102) {
      HEAP32[$zFmt>>2] = 23712;
      break;
     }
     $103 = HEAP32[$flags>>2]|0;
     $104 = $103 & 64;
     $105 = ($104|0)!=(0);
     if ($105) {
      HEAP32[$zFmt>>2] = 23744;
      break;
     } else {
      HEAP32[$zFmt>>2] = 23768;
      break;
     }
    }
   } while(0);
   $106 = HEAP32[$zFmt>>2]|0;
   $107 = ($106|0)!=(0|0);
   if ($107) {
    _sqlite3StrAccumAppend($str,23784,7);
    $108 = HEAP32[$zFmt>>2]|0;
    $109 = HEAP32[$pIdx>>2]|0;
    $110 = HEAP32[$109>>2]|0;
    HEAP32[$vararg_buffer10>>2] = $110;
    _sqlite3XPrintf($str,0,$108,$vararg_buffer10);
    $111 = HEAP32[$pLoop>>2]|0;
    $112 = HEAP32[$pItem>>2]|0;
    $113 = (($112) + 16|0);
    $114 = HEAP32[$113>>2]|0;
    _explainIndexRange($str,$111,$114);
   }
  } else {
   $115 = HEAP32[$flags>>2]|0;
   $116 = $115 & 256;
   $117 = ($116|0)!=(0);
   if ($117) {
    $118 = HEAP32[$flags>>2]|0;
    $119 = $118 & 15;
    $120 = ($119|0)!=(0);
    if ($120) {
     $121 = HEAP32[$flags>>2]|0;
     $122 = $121 & 5;
     $123 = ($122|0)!=(0);
     do {
      if ($123) {
       HEAP32[$zRange>>2] = 23792;
      } else {
       $124 = HEAP32[$flags>>2]|0;
       $125 = $124 & 48;
       $126 = ($125|0)==(48);
       if ($126) {
        HEAP32[$zRange>>2] = 23808;
        break;
       }
       $127 = HEAP32[$flags>>2]|0;
       $128 = $127 & 32;
       $129 = ($128|0)!=(0);
       if ($129) {
        HEAP32[$zRange>>2] = 23832;
        break;
       } else {
        HEAP32[$zRange>>2] = 23848;
        break;
       }
      }
     } while(0);
     _sqlite3StrAccumAppendAll($str,23864);
     $130 = HEAP32[$zRange>>2]|0;
     _sqlite3StrAccumAppendAll($str,$130);
     break;
    }
   }
   $131 = HEAP32[$flags>>2]|0;
   $132 = $131 & 1024;
   $133 = ($132|0)!=(0);
   if ($133) {
    $134 = HEAP32[$pLoop>>2]|0;
    $135 = (($134) + 24|0);
    $136 = HEAP32[$135>>2]|0;
    $137 = HEAP32[$pLoop>>2]|0;
    $138 = (($137) + 24|0);
    $139 = (($138) + 8|0);
    $140 = HEAP32[$139>>2]|0;
    HEAP32[$vararg_buffer10>>2] = $136;
    $vararg_ptr13 = (($vararg_buffer10) + 4|0);
    HEAP32[$vararg_ptr13>>2] = $140;
    _sqlite3XPrintf($str,0,23896,$vararg_buffer10);
   }
  }
 } while(0);
 $141 = (_sqlite3StrAccumFinish($str)|0);
 HEAP32[$zMsg>>2] = $141;
 $142 = HEAP32[$v>>2]|0;
 $143 = HEAP32[$iId>>2]|0;
 $144 = HEAP32[$3>>2]|0;
 $145 = HEAP32[$4>>2]|0;
 $146 = HEAP32[$zMsg>>2]|0;
 (_sqlite3VdbeAddOp4($142,155,$143,$144,$145,$146,-1)|0);
 STACKTOP = sp;return;
}
function _codeOneLoopStart($pWInfo,$iLevel,$0,$1) {
 $pWInfo = $pWInfo|0;
 $iLevel = $iLevel|0;
 $0 = $0|0;
 $1 = $1|0;
 var $10 = 0, $100 = 0, $1000 = 0, $1001 = 0, $1002 = 0, $1003 = 0, $1004 = 0, $1005 = 0, $1006 = 0, $1007 = 0, $1008 = 0, $1009 = 0, $101 = 0, $1010 = 0, $1011 = 0, $1012 = 0, $1013 = 0, $1014 = 0, $1015 = 0, $1016 = 0;
 var $1017 = 0, $1018 = 0, $1019 = 0, $102 = 0, $1020 = 0, $1021 = 0, $1022 = 0, $1023 = 0, $1024 = 0, $1025 = 0, $1026 = 0, $1027 = 0, $1028 = 0, $1029 = 0, $103 = 0, $1030 = 0, $1031 = 0, $1032 = 0, $1033 = 0, $1034 = 0;
 var $1035 = 0, $1036 = 0, $1037 = 0, $1038 = 0, $1039 = 0, $104 = 0, $1040 = 0, $1041 = 0, $1042 = 0, $1043 = 0, $1044 = 0, $1045 = 0, $1046 = 0, $1047 = 0, $1048 = 0, $1049 = 0, $105 = 0, $1050 = 0, $1051 = 0, $1052 = 0;
 var $1053 = 0, $1054 = 0, $1055 = 0, $1056 = 0, $1057 = 0, $1058 = 0, $1059 = 0, $106 = 0, $1060 = 0, $1061 = 0, $1062 = 0, $1063 = 0, $1064 = 0, $1065 = 0, $1066 = 0, $1067 = 0, $1068 = 0, $1069 = 0, $107 = 0, $1070 = 0;
 var $1071 = 0, $1072 = 0, $1073 = 0, $1074 = 0, $1075 = 0, $1076 = 0, $1077 = 0, $1078 = 0, $1079 = 0, $108 = 0, $1080 = 0, $1081 = 0, $1082 = 0, $1083 = 0, $1084 = 0, $1085 = 0, $1086 = 0, $1087 = 0, $1088 = 0, $1089 = 0;
 var $109 = 0, $1090 = 0, $1091 = 0, $1092 = 0, $1093 = 0, $1094 = 0, $1095 = 0, $1096 = 0, $1097 = 0, $1098 = 0, $1099 = 0, $11 = 0, $110 = 0, $1100 = 0, $1101 = 0, $1102 = 0, $1103 = 0, $1104 = 0, $1105 = 0, $1106 = 0;
 var $1107 = 0, $1108 = 0, $1109 = 0, $111 = 0, $1110 = 0, $1111 = 0, $1112 = 0, $1113 = 0, $1114 = 0, $1115 = 0, $1116 = 0, $1117 = 0, $1118 = 0, $1119 = 0, $112 = 0, $1120 = 0, $1121 = 0, $1122 = 0, $1123 = 0, $1124 = 0;
 var $1125 = 0, $1126 = 0, $1127 = 0, $1128 = 0, $1129 = 0, $113 = 0, $1130 = 0, $1131 = 0, $1132 = 0, $1133 = 0, $1134 = 0, $1135 = 0, $1136 = 0, $1137 = 0, $1138 = 0, $1139 = 0, $114 = 0, $1140 = 0, $1141 = 0, $1142 = 0;
 var $1143 = 0, $1144 = 0, $1145 = 0, $1146 = 0, $1147 = 0, $1148 = 0, $1149 = 0, $115 = 0, $1150 = 0, $1151 = 0, $1152 = 0, $1153 = 0, $1154 = 0, $1155 = 0, $1156 = 0, $1157 = 0, $1158 = 0, $1159 = 0, $116 = 0, $1160 = 0;
 var $1161 = 0, $1162 = 0, $1163 = 0, $1164 = 0, $1165 = 0, $1166 = 0, $1167 = 0, $1168 = 0, $1169 = 0, $117 = 0, $1170 = 0, $1171 = 0, $1172 = 0, $1173 = 0, $1174 = 0, $1175 = 0, $1176 = 0, $1177 = 0, $1178 = 0, $1179 = 0;
 var $118 = 0, $1180 = 0, $1181 = 0, $1182 = 0, $1183 = 0, $1184 = 0, $1185 = 0, $1186 = 0, $1187 = 0, $1188 = 0, $1189 = 0, $119 = 0, $1190 = 0, $1191 = 0, $1192 = 0, $1193 = 0, $1194 = 0, $1195 = 0, $1196 = 0, $1197 = 0;
 var $1198 = 0, $1199 = 0, $12 = 0, $120 = 0, $1200 = 0, $1201 = 0, $1202 = 0, $1203 = 0, $1204 = 0, $1205 = 0, $1206 = 0, $1207 = 0, $1208 = 0, $1209 = 0, $121 = 0, $1210 = 0, $1211 = 0, $1212 = 0, $1213 = 0, $1214 = 0;
 var $1215 = 0, $1216 = 0, $1217 = 0, $1218 = 0, $1219 = 0, $122 = 0, $1220 = 0, $1221 = 0, $1222 = 0, $1223 = 0, $1224 = 0, $1225 = 0, $1226 = 0, $1227 = 0, $1228 = 0, $1229 = 0, $123 = 0, $1230 = 0, $1231 = 0, $1232 = 0;
 var $1233 = 0, $1234 = 0, $1235 = 0, $1236 = 0, $1237 = 0, $1238 = 0, $1239 = 0, $124 = 0, $1240 = 0, $1241 = 0, $1242 = 0, $1243 = 0, $1244 = 0, $1245 = 0, $1246 = 0, $1247 = 0, $1248 = 0, $1249 = 0, $125 = 0, $1250 = 0;
 var $1251 = 0, $1252 = 0, $1253 = 0, $1254 = 0, $1255 = 0, $1256 = 0, $1257 = 0, $1258 = 0, $1259 = 0, $126 = 0, $1260 = 0, $1261 = 0, $1262 = 0, $1263 = 0, $1264 = 0, $1265 = 0, $1266 = 0, $1267 = 0, $1268 = 0, $1269 = 0;
 var $127 = 0, $1270 = 0, $1271 = 0, $1272 = 0, $1273 = 0, $1274 = 0, $1275 = 0, $1276 = 0, $1277 = 0, $1278 = 0, $1279 = 0, $128 = 0, $1280 = 0, $1281 = 0, $1282 = 0, $1283 = 0, $1284 = 0, $1285 = 0, $1286 = 0, $1287 = 0;
 var $1288 = 0, $1289 = 0, $129 = 0, $1290 = 0, $1291 = 0, $1292 = 0, $1293 = 0, $1294 = 0, $1295 = 0, $1296 = 0, $1297 = 0, $1298 = 0, $1299 = 0, $13 = 0, $130 = 0, $1300 = 0, $1301 = 0, $1302 = 0, $1303 = 0, $1304 = 0;
 var $1305 = 0, $1306 = 0, $1307 = 0, $1308 = 0, $1309 = 0, $131 = 0, $1310 = 0, $1311 = 0, $1312 = 0, $1313 = 0, $1314 = 0, $1315 = 0, $1316 = 0, $1317 = 0, $1318 = 0, $1319 = 0, $132 = 0, $1320 = 0, $1321 = 0, $1322 = 0;
 var $1323 = 0, $1324 = 0, $1325 = 0, $1326 = 0, $1327 = 0, $1328 = 0, $1329 = 0, $133 = 0, $1330 = 0, $1331 = 0, $1332 = 0, $1333 = 0, $1334 = 0, $1335 = 0, $1336 = 0, $1337 = 0, $1338 = 0, $1339 = 0, $134 = 0, $1340 = 0;
 var $1341 = 0, $1342 = 0, $1343 = 0, $1344 = 0, $1345 = 0, $1346 = 0, $1347 = 0, $1348 = 0, $1349 = 0, $135 = 0, $1350 = 0, $1351 = 0, $1352 = 0, $1353 = 0, $1354 = 0, $1355 = 0, $1356 = 0, $1357 = 0, $1358 = 0, $1359 = 0;
 var $136 = 0, $1360 = 0, $1361 = 0, $1362 = 0, $1363 = 0, $1364 = 0, $1365 = 0, $1366 = 0, $1367 = 0, $1368 = 0, $1369 = 0, $137 = 0, $1370 = 0, $1371 = 0, $1372 = 0, $1373 = 0, $1374 = 0, $1375 = 0, $1376 = 0, $1377 = 0;
 var $1378 = 0, $1379 = 0, $138 = 0, $1380 = 0, $1381 = 0, $1382 = 0, $1383 = 0, $1384 = 0, $1385 = 0, $1386 = 0, $1387 = 0, $1388 = 0, $1389 = 0, $139 = 0, $1390 = 0, $1391 = 0, $1392 = 0, $1393 = 0, $1394 = 0, $1395 = 0;
 var $1396 = 0, $1397 = 0, $1398 = 0, $1399 = 0, $14 = 0, $140 = 0, $1400 = 0, $1401 = 0, $1402 = 0, $1403 = 0, $1404 = 0, $1405 = 0, $1406 = 0, $1407 = 0, $1408 = 0, $1409 = 0, $141 = 0, $1410 = 0, $1411 = 0, $1412 = 0;
 var $1413 = 0, $1414 = 0, $1415 = 0, $1416 = 0, $1417 = 0, $1418 = 0, $1419 = 0, $142 = 0, $1420 = 0, $1421 = 0, $1422 = 0, $1423 = 0, $1424 = 0, $1425 = 0, $1426 = 0, $1427 = 0, $1428 = 0, $1429 = 0, $143 = 0, $1430 = 0;
 var $1431 = 0, $1432 = 0, $1433 = 0, $1434 = 0, $1435 = 0, $1436 = 0, $1437 = 0, $1438 = 0, $1439 = 0, $144 = 0, $1440 = 0, $1441 = 0, $1442 = 0, $1443 = 0, $1444 = 0, $1445 = 0, $1446 = 0, $1447 = 0, $1448 = 0, $1449 = 0;
 var $145 = 0, $1450 = 0, $1451 = 0, $1452 = 0, $1453 = 0, $1454 = 0, $1455 = 0, $1456 = 0, $1457 = 0, $1458 = 0, $1459 = 0, $146 = 0, $1460 = 0, $1461 = 0, $1462 = 0, $1463 = 0, $1464 = 0, $1465 = 0, $1466 = 0, $1467 = 0;
 var $1468 = 0, $1469 = 0, $147 = 0, $1470 = 0, $1471 = 0, $1472 = 0, $1473 = 0, $1474 = 0, $1475 = 0, $1476 = 0, $1477 = 0, $1478 = 0, $1479 = 0, $148 = 0, $1480 = 0, $1481 = 0, $1482 = 0, $1483 = 0, $1484 = 0, $1485 = 0;
 var $1486 = 0, $1487 = 0, $1488 = 0, $1489 = 0, $149 = 0, $1490 = 0, $1491 = 0, $1492 = 0, $1493 = 0, $1494 = 0, $1495 = 0, $1496 = 0, $1497 = 0, $1498 = 0, $1499 = 0, $15 = 0, $150 = 0, $1500 = 0, $1501 = 0, $1502 = 0;
 var $1503 = 0, $1504 = 0, $1505 = 0, $1506 = 0, $1507 = 0, $1508 = 0, $1509 = 0, $151 = 0, $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0;
 var $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0, $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0;
 var $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0, $188 = 0, $189 = 0, $19 = 0, $190 = 0, $191 = 0, $192 = 0, $193 = 0, $194 = 0, $195 = 0, $196 = 0, $197 = 0, $198 = 0, $199 = 0;
 var $2 = 0, $20 = 0, $200 = 0, $201 = 0, $202 = 0, $203 = 0, $204 = 0, $205 = 0, $206 = 0, $207 = 0, $208 = 0, $209 = 0, $21 = 0, $210 = 0, $211 = 0, $212 = 0, $213 = 0, $214 = 0, $215 = 0, $216 = 0;
 var $217 = 0, $218 = 0, $219 = 0, $22 = 0, $220 = 0, $221 = 0, $222 = 0, $223 = 0, $224 = 0, $225 = 0, $226 = 0, $227 = 0, $228 = 0, $229 = 0, $23 = 0, $230 = 0, $231 = 0, $232 = 0, $233 = 0, $234 = 0;
 var $235 = 0, $236 = 0, $237 = 0, $238 = 0, $239 = 0, $24 = 0, $240 = 0, $241 = 0, $242 = 0, $243 = 0, $244 = 0, $245 = 0, $246 = 0, $247 = 0, $248 = 0, $249 = 0, $25 = 0, $250 = 0, $251 = 0, $252 = 0;
 var $253 = 0, $254 = 0, $255 = 0, $256 = 0, $257 = 0, $258 = 0, $259 = 0, $26 = 0, $260 = 0, $261 = 0, $262 = 0, $263 = 0, $264 = 0, $265 = 0, $266 = 0, $267 = 0, $268 = 0, $269 = 0, $27 = 0, $270 = 0;
 var $271 = 0, $272 = 0, $273 = 0, $274 = 0, $275 = 0, $276 = 0, $277 = 0, $278 = 0, $279 = 0, $28 = 0, $280 = 0, $281 = 0, $282 = 0, $283 = 0, $284 = 0, $285 = 0, $286 = 0, $287 = 0, $288 = 0, $289 = 0;
 var $29 = 0, $290 = 0, $291 = 0, $292 = 0, $293 = 0, $294 = 0, $295 = 0, $296 = 0, $297 = 0, $298 = 0, $299 = 0, $3 = 0, $30 = 0, $300 = 0, $301 = 0, $302 = 0, $303 = 0, $304 = 0, $305 = 0, $306 = 0;
 var $307 = 0, $308 = 0, $309 = 0, $31 = 0, $310 = 0, $311 = 0, $312 = 0, $313 = 0, $314 = 0, $315 = 0, $316 = 0, $317 = 0, $318 = 0, $319 = 0, $32 = 0, $320 = 0, $321 = 0, $322 = 0, $323 = 0, $324 = 0;
 var $325 = 0, $326 = 0, $327 = 0, $328 = 0, $329 = 0, $33 = 0, $330 = 0, $331 = 0, $332 = 0, $333 = 0, $334 = 0, $335 = 0, $336 = 0, $337 = 0, $338 = 0, $339 = 0, $34 = 0, $340 = 0, $341 = 0, $342 = 0;
 var $343 = 0, $344 = 0, $345 = 0, $346 = 0, $347 = 0, $348 = 0, $349 = 0, $35 = 0, $350 = 0, $351 = 0, $352 = 0, $353 = 0, $354 = 0, $355 = 0, $356 = 0, $357 = 0, $358 = 0, $359 = 0, $36 = 0, $360 = 0;
 var $361 = 0, $362 = 0, $363 = 0, $364 = 0, $365 = 0, $366 = 0, $367 = 0, $368 = 0, $369 = 0, $37 = 0, $370 = 0, $371 = 0, $372 = 0, $373 = 0, $374 = 0, $375 = 0, $376 = 0, $377 = 0, $378 = 0, $379 = 0;
 var $38 = 0, $380 = 0, $381 = 0, $382 = 0, $383 = 0, $384 = 0, $385 = 0, $386 = 0, $387 = 0, $388 = 0, $389 = 0, $39 = 0, $390 = 0, $391 = 0, $392 = 0, $393 = 0, $394 = 0, $395 = 0, $396 = 0, $397 = 0;
 var $398 = 0, $399 = 0, $4 = 0, $40 = 0, $400 = 0, $401 = 0, $402 = 0, $403 = 0, $404 = 0, $405 = 0, $406 = 0, $407 = 0, $408 = 0, $409 = 0, $41 = 0, $410 = 0, $411 = 0, $412 = 0, $413 = 0, $414 = 0;
 var $415 = 0, $416 = 0, $417 = 0, $418 = 0, $419 = 0, $42 = 0, $420 = 0, $421 = 0, $422 = 0, $423 = 0, $424 = 0, $425 = 0, $426 = 0, $427 = 0, $428 = 0, $429 = 0, $43 = 0, $430 = 0, $431 = 0, $432 = 0;
 var $433 = 0, $434 = 0, $435 = 0, $436 = 0, $437 = 0, $438 = 0, $439 = 0, $44 = 0, $440 = 0, $441 = 0, $442 = 0, $443 = 0, $444 = 0, $445 = 0, $446 = 0, $447 = 0, $448 = 0, $449 = 0, $45 = 0, $450 = 0;
 var $451 = 0, $452 = 0, $453 = 0, $454 = 0, $455 = 0, $456 = 0, $457 = 0, $458 = 0, $459 = 0, $46 = 0, $460 = 0, $461 = 0, $462 = 0, $463 = 0, $464 = 0, $465 = 0, $466 = 0, $467 = 0, $468 = 0, $469 = 0;
 var $47 = 0, $470 = 0, $471 = 0, $472 = 0, $473 = 0, $474 = 0, $475 = 0, $476 = 0, $477 = 0, $478 = 0, $479 = 0, $48 = 0, $480 = 0, $481 = 0, $482 = 0, $483 = 0, $484 = 0, $485 = 0, $486 = 0, $487 = 0;
 var $488 = 0, $489 = 0, $49 = 0, $490 = 0, $491 = 0, $492 = 0, $493 = 0, $494 = 0, $495 = 0, $496 = 0, $497 = 0, $498 = 0, $499 = 0, $5 = 0, $50 = 0, $500 = 0, $501 = 0, $502 = 0, $503 = 0, $504 = 0;
 var $505 = 0, $506 = 0, $507 = 0, $508 = 0, $509 = 0, $51 = 0, $510 = 0, $511 = 0, $512 = 0, $513 = 0, $514 = 0, $515 = 0, $516 = 0, $517 = 0, $518 = 0, $519 = 0, $52 = 0, $520 = 0, $521 = 0, $522 = 0;
 var $523 = 0, $524 = 0, $525 = 0, $526 = 0, $527 = 0, $528 = 0, $529 = 0, $53 = 0, $530 = 0, $531 = 0, $532 = 0, $533 = 0, $534 = 0, $535 = 0, $536 = 0, $537 = 0, $538 = 0, $539 = 0, $54 = 0, $540 = 0;
 var $541 = 0, $542 = 0, $543 = 0, $544 = 0, $545 = 0, $546 = 0, $547 = 0, $548 = 0, $549 = 0, $55 = 0, $550 = 0, $551 = 0, $552 = 0, $553 = 0, $554 = 0, $555 = 0, $556 = 0, $557 = 0, $558 = 0, $559 = 0;
 var $56 = 0, $560 = 0, $561 = 0, $562 = 0, $563 = 0, $564 = 0, $565 = 0, $566 = 0, $567 = 0, $568 = 0, $569 = 0, $57 = 0, $570 = 0, $571 = 0, $572 = 0, $573 = 0, $574 = 0, $575 = 0, $576 = 0, $577 = 0;
 var $578 = 0, $579 = 0, $58 = 0, $580 = 0, $581 = 0, $582 = 0, $583 = 0, $584 = 0, $585 = 0, $586 = 0, $587 = 0, $588 = 0, $589 = 0, $59 = 0, $590 = 0, $591 = 0, $592 = 0, $593 = 0, $594 = 0, $595 = 0;
 var $596 = 0, $597 = 0, $598 = 0, $599 = 0, $6 = 0, $60 = 0, $600 = 0, $601 = 0, $602 = 0, $603 = 0, $604 = 0, $605 = 0, $606 = 0, $607 = 0, $608 = 0, $609 = 0, $61 = 0, $610 = 0, $611 = 0, $612 = 0;
 var $613 = 0, $614 = 0, $615 = 0, $616 = 0, $617 = 0, $618 = 0, $619 = 0, $62 = 0, $620 = 0, $621 = 0, $622 = 0, $623 = 0, $624 = 0, $625 = 0, $626 = 0, $627 = 0, $628 = 0, $629 = 0, $63 = 0, $630 = 0;
 var $631 = 0, $632 = 0, $633 = 0, $634 = 0, $635 = 0, $636 = 0, $637 = 0, $638 = 0, $639 = 0, $64 = 0, $640 = 0, $641 = 0, $642 = 0, $643 = 0, $644 = 0, $645 = 0, $646 = 0, $647 = 0, $648 = 0, $649 = 0;
 var $65 = 0, $650 = 0, $651 = 0, $652 = 0, $653 = 0, $654 = 0, $655 = 0, $656 = 0, $657 = 0, $658 = 0, $659 = 0, $66 = 0, $660 = 0, $661 = 0, $662 = 0, $663 = 0, $664 = 0, $665 = 0, $666 = 0, $667 = 0;
 var $668 = 0, $669 = 0, $67 = 0, $670 = 0, $671 = 0, $672 = 0, $673 = 0, $674 = 0, $675 = 0, $676 = 0, $677 = 0, $678 = 0, $679 = 0, $68 = 0, $680 = 0, $681 = 0, $682 = 0, $683 = 0, $684 = 0, $685 = 0;
 var $686 = 0, $687 = 0, $688 = 0, $689 = 0, $69 = 0, $690 = 0, $691 = 0, $692 = 0, $693 = 0, $694 = 0, $695 = 0, $696 = 0, $697 = 0, $698 = 0, $699 = 0, $7 = 0, $70 = 0, $700 = 0, $701 = 0, $702 = 0;
 var $703 = 0, $704 = 0, $705 = 0, $706 = 0, $707 = 0, $708 = 0, $709 = 0, $71 = 0, $710 = 0, $711 = 0, $712 = 0, $713 = 0, $714 = 0, $715 = 0, $716 = 0, $717 = 0, $718 = 0, $719 = 0, $72 = 0, $720 = 0;
 var $721 = 0, $722 = 0, $723 = 0, $724 = 0, $725 = 0, $726 = 0, $727 = 0, $728 = 0, $729 = 0, $73 = 0, $730 = 0, $731 = 0, $732 = 0, $733 = 0, $734 = 0, $735 = 0, $736 = 0, $737 = 0, $738 = 0, $739 = 0;
 var $74 = 0, $740 = 0, $741 = 0, $742 = 0, $743 = 0, $744 = 0, $745 = 0, $746 = 0, $747 = 0, $748 = 0, $749 = 0, $75 = 0, $750 = 0, $751 = 0, $752 = 0, $753 = 0, $754 = 0, $755 = 0, $756 = 0, $757 = 0;
 var $758 = 0, $759 = 0, $76 = 0, $760 = 0, $761 = 0, $762 = 0, $763 = 0, $764 = 0, $765 = 0, $766 = 0, $767 = 0, $768 = 0, $769 = 0, $77 = 0, $770 = 0, $771 = 0, $772 = 0, $773 = 0, $774 = 0, $775 = 0;
 var $776 = 0, $777 = 0, $778 = 0, $779 = 0, $78 = 0, $780 = 0, $781 = 0, $782 = 0, $783 = 0, $784 = 0, $785 = 0, $786 = 0, $787 = 0, $788 = 0, $789 = 0, $79 = 0, $790 = 0, $791 = 0, $792 = 0, $793 = 0;
 var $794 = 0, $795 = 0, $796 = 0, $797 = 0, $798 = 0, $799 = 0, $8 = 0, $80 = 0, $800 = 0, $801 = 0, $802 = 0, $803 = 0, $804 = 0, $805 = 0, $806 = 0, $807 = 0, $808 = 0, $809 = 0, $81 = 0, $810 = 0;
 var $811 = 0, $812 = 0, $813 = 0, $814 = 0, $815 = 0, $816 = 0, $817 = 0, $818 = 0, $819 = 0, $82 = 0, $820 = 0, $821 = 0, $822 = 0, $823 = 0, $824 = 0, $825 = 0, $826 = 0, $827 = 0, $828 = 0, $829 = 0;
 var $83 = 0, $830 = 0, $831 = 0, $832 = 0, $833 = 0, $834 = 0, $835 = 0, $836 = 0, $837 = 0, $838 = 0, $839 = 0, $84 = 0, $840 = 0, $841 = 0, $842 = 0, $843 = 0, $844 = 0, $845 = 0, $846 = 0, $847 = 0;
 var $848 = 0, $849 = 0, $85 = 0, $850 = 0, $851 = 0, $852 = 0, $853 = 0, $854 = 0, $855 = 0, $856 = 0, $857 = 0, $858 = 0, $859 = 0, $86 = 0, $860 = 0, $861 = 0, $862 = 0, $863 = 0, $864 = 0, $865 = 0;
 var $866 = 0, $867 = 0, $868 = 0, $869 = 0, $87 = 0, $870 = 0, $871 = 0, $872 = 0, $873 = 0, $874 = 0, $875 = 0, $876 = 0, $877 = 0, $878 = 0, $879 = 0, $88 = 0, $880 = 0, $881 = 0, $882 = 0, $883 = 0;
 var $884 = 0, $885 = 0, $886 = 0, $887 = 0, $888 = 0, $889 = 0, $89 = 0, $890 = 0, $891 = 0, $892 = 0, $893 = 0, $894 = 0, $895 = 0, $896 = 0, $897 = 0, $898 = 0, $899 = 0, $9 = 0, $90 = 0, $900 = 0;
 var $901 = 0, $902 = 0, $903 = 0, $904 = 0, $905 = 0, $906 = 0, $907 = 0, $908 = 0, $909 = 0, $91 = 0, $910 = 0, $911 = 0, $912 = 0, $913 = 0, $914 = 0, $915 = 0, $916 = 0, $917 = 0, $918 = 0, $919 = 0;
 var $92 = 0, $920 = 0, $921 = 0, $922 = 0, $923 = 0, $924 = 0, $925 = 0, $926 = 0, $927 = 0, $928 = 0, $929 = 0, $93 = 0, $930 = 0, $931 = 0, $932 = 0, $933 = 0, $934 = 0, $935 = 0, $936 = 0, $937 = 0;
 var $938 = 0, $939 = 0, $94 = 0, $940 = 0, $941 = 0, $942 = 0, $943 = 0, $944 = 0, $945 = 0, $946 = 0, $947 = 0, $948 = 0, $949 = 0, $95 = 0, $950 = 0, $951 = 0, $952 = 0, $953 = 0, $954 = 0, $955 = 0;
 var $956 = 0, $957 = 0, $958 = 0, $959 = 0, $96 = 0, $960 = 0, $961 = 0, $962 = 0, $963 = 0, $964 = 0, $965 = 0, $966 = 0, $967 = 0, $968 = 0, $969 = 0, $97 = 0, $970 = 0, $971 = 0, $972 = 0, $973 = 0;
 var $974 = 0, $975 = 0, $976 = 0, $977 = 0, $978 = 0, $979 = 0, $98 = 0, $980 = 0, $981 = 0, $982 = 0, $983 = 0, $984 = 0, $985 = 0, $986 = 0, $987 = 0, $988 = 0, $989 = 0, $99 = 0, $990 = 0, $991 = 0;
 var $992 = 0, $993 = 0, $994 = 0, $995 = 0, $996 = 0, $997 = 0, $998 = 0, $999 = 0, $addrBrk = 0, $addrCont = 0, $addrNotFound = 0, $addrNxt = 0, $bRev = 0, $bSeekPastNull = 0, $bStopAtNull = 0, $cEndAff = 0, $db = 0, $endEq = 0, $iCol = 0, $iCovCur = 0;
 var $iCur = 0, $iIdxCur = 0, $iLoopBody = 0, $iPk = 0, $iReg = 0, $iReleaseReg = 0, $iRetInit = 0, $iRowidReg = 0, $iSet = 0, $iTarget = 0, $iTerm = 0, $ii = 0, $j = 0, $j1 = 0, $k = 0, $memEndValue = 0, $nConstraint = 0, $nConstraint2 = 0, $nEq = 0, $nExtraReg = 0;
 var $nNotReady = 0, $nPk = 0, $omitTable = 0, $op = 0, $or$cond = 0, $origSrc = 0, $pAlt = 0, $pAndExpr = 0, $pCov = 0, $pE = 0, $pE7 = 0, $pEAlt = 0, $pEnd = 0, $pExpr = 0, $pIdx = 0, $pLevel = 0, $pLoop = 0, $pOrExpr = 0, $pOrTab = 0, $pOrTerm = 0;
 var $pOrWc = 0, $pParse = 0, $pPk = 0, $pPk5 = 0, $pPk6 = 0, $pRangeEnd = 0, $pRangeStart = 0, $pRight = 0, $pRight4 = 0, $pStart = 0, $pSubLoop = 0, $pSubWInfo = 0, $pTab = 0, $pTabItem = 0, $pTerm = 0, $pWC = 0, $pX = 0, $pX1 = 0, $r = 0, $r1 = 0;
 var $rTemp = 0, $regBase = 0, $regReturn = 0, $regRowid = 0, $regRowset = 0, $regYield = 0, $start = 0, $startEq = 0, $start_constraints = 0, $t = 0, $t3 = 0, $testOp = 0, $untestedTerms = 0, $v = 0, $wctrlFlags = 0, $zStartAff = 0, dest = 0, label = 0, sp = 0, src = 0;
 var stop = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 368|0;
 $2 = sp + 8|0;
 $3 = sp + 344|0;
 $4 = sp + 340|0;
 $5 = sp;
 $j = sp + 336|0;
 $k = sp + 332|0;
 $iCur = sp + 328|0;
 $addrNxt = sp + 324|0;
 $omitTable = sp + 320|0;
 $bRev = sp + 316|0;
 $pLevel = sp + 312|0;
 $pLoop = sp + 308|0;
 $pWC = sp + 304|0;
 $pTerm = sp + 300|0;
 $pParse = sp + 296|0;
 $db = sp + 292|0;
 $v = sp + 288|0;
 $pTabItem = sp + 284|0;
 $addrBrk = sp + 280|0;
 $addrCont = sp + 276|0;
 $iRowidReg = sp + 272|0;
 $iReleaseReg = sp + 268|0;
 $regYield = sp + 264|0;
 $iReg = sp + 260|0;
 $addrNotFound = sp + 256|0;
 $nConstraint = sp + 252|0;
 $iTarget = sp + 248|0;
 $testOp = sp + 244|0;
 $start = sp + 240|0;
 $memEndValue = sp + 236|0;
 $pStart = sp + 232|0;
 $pEnd = sp + 228|0;
 $pX = sp + 224|0;
 $r1 = sp + 220|0;
 $rTemp = sp + 216|0;
 $pX1 = sp + 212|0;
 $nEq = sp + 350|0;
 $regBase = sp + 208|0;
 $pRangeStart = sp + 204|0;
 $pRangeEnd = sp + 200|0;
 $startEq = sp + 196|0;
 $endEq = sp + 192|0;
 $start_constraints = sp + 188|0;
 $nConstraint2 = sp + 184|0;
 $pIdx = sp + 180|0;
 $iIdxCur = sp + 176|0;
 $nExtraReg = sp + 172|0;
 $op = sp + 168|0;
 $zStartAff = sp + 164|0;
 $cEndAff = sp + 355|0;
 $bSeekPastNull = sp + 354|0;
 $bStopAtNull = sp + 353|0;
 $t = sp + 160|0;
 $t3 = sp + 352|0;
 $pRight = sp + 156|0;
 $pRight4 = sp + 152|0;
 $pPk = sp + 148|0;
 $pOrWc = sp + 144|0;
 $pOrTab = sp + 140|0;
 $pCov = sp + 136|0;
 $iCovCur = sp + 132|0;
 $regReturn = sp + 128|0;
 $regRowset = sp + 124|0;
 $regRowid = sp + 120|0;
 $iLoopBody = sp + 116|0;
 $iRetInit = sp + 112|0;
 $untestedTerms = sp + 108|0;
 $ii = sp + 104|0;
 $wctrlFlags = sp + 348|0;
 $pAndExpr = sp + 100|0;
 $pTab = sp + 96|0;
 $nNotReady = sp + 92|0;
 $origSrc = sp + 88|0;
 $pPk5 = sp + 84|0;
 $iTerm = sp + 80|0;
 $pExpr = sp + 76|0;
 $pOrTerm = sp + 72|0;
 $pSubWInfo = sp + 68|0;
 $pOrExpr = sp + 64|0;
 $j1 = sp + 60|0;
 $pSubLoop = sp + 56|0;
 $r = sp + 52|0;
 $iSet = sp + 48|0;
 $pPk6 = sp + 44|0;
 $nPk = sp + 40|0;
 $iPk = sp + 36|0;
 $iCol = sp + 32|0;
 $pE = sp + 28|0;
 $pE7 = sp + 24|0;
 $pEAlt = sp + 20|0;
 $pAlt = sp + 16|0;
 HEAP32[$3>>2] = $pWInfo;
 HEAP32[$4>>2] = $iLevel;
 $6 = $5;
 $7 = $6;
 HEAP32[$7>>2] = $0;
 $8 = (($6) + 4)|0;
 $9 = $8;
 HEAP32[$9>>2] = $1;
 HEAP32[$iRowidReg>>2] = 0;
 HEAP32[$iReleaseReg>>2] = 0;
 $10 = HEAP32[$3>>2]|0;
 $11 = HEAP32[$10>>2]|0;
 HEAP32[$pParse>>2] = $11;
 $12 = HEAP32[$pParse>>2]|0;
 $13 = (($12) + 8|0);
 $14 = HEAP32[$13>>2]|0;
 HEAP32[$v>>2] = $14;
 $15 = HEAP32[$3>>2]|0;
 $16 = (($15) + 328|0);
 HEAP32[$pWC>>2] = $16;
 $17 = HEAP32[$pParse>>2]|0;
 $18 = HEAP32[$17>>2]|0;
 HEAP32[$db>>2] = $18;
 $19 = HEAP32[$4>>2]|0;
 $20 = HEAP32[$3>>2]|0;
 $21 = (($20) + 736|0);
 $22 = (($21) + (($19*72)|0)|0);
 HEAP32[$pLevel>>2] = $22;
 $23 = HEAP32[$pLevel>>2]|0;
 $24 = (($23) + 56|0);
 $25 = HEAP32[$24>>2]|0;
 HEAP32[$pLoop>>2] = $25;
 $26 = HEAP32[$pLevel>>2]|0;
 $27 = (($26) + 36|0);
 $28 = HEAP8[$27>>0]|0;
 $29 = $28&255;
 $30 = HEAP32[$3>>2]|0;
 $31 = (($30) + 4|0);
 $32 = HEAP32[$31>>2]|0;
 $33 = (($32) + 8|0);
 $34 = (($33) + (($29*72)|0)|0);
 HEAP32[$pTabItem>>2] = $34;
 $35 = HEAP32[$pTabItem>>2]|0;
 $36 = (($35) + 40|0);
 $37 = HEAP32[$36>>2]|0;
 HEAP32[$iCur>>2] = $37;
 $38 = $5;
 $39 = $38;
 $40 = HEAP32[$39>>2]|0;
 $41 = (($38) + 4)|0;
 $42 = $41;
 $43 = HEAP32[$42>>2]|0;
 $44 = HEAP32[$3>>2]|0;
 $45 = (($44) + 68|0);
 $46 = HEAP32[$iCur>>2]|0;
 $47 = (_getMask($45,$46)|0);
 $48 = tempRet0;
 $49 = $47 ^ -1;
 $50 = $48 ^ -1;
 $51 = $40 & $49;
 $52 = $43 & $50;
 $53 = HEAP32[$pLevel>>2]|0;
 $54 = (($53) + 64|0);
 $55 = $54;
 $56 = $55;
 HEAP32[$56>>2] = $51;
 $57 = (($55) + 4)|0;
 $58 = $57;
 HEAP32[$58>>2] = $52;
 $59 = HEAP32[$3>>2]|0;
 $60 = (($59) + 24|0);
 $61 = $60;
 $62 = $61;
 $63 = HEAP32[$62>>2]|0;
 $64 = (($61) + 4)|0;
 $65 = $64;
 $66 = HEAP32[$65>>2]|0;
 $67 = HEAP32[$4>>2]|0;
 $68 = (_bitshift64Lshr(($63|0),($66|0),($67|0))|0);
 $69 = tempRet0;
 $70 = $68 & 1;
 HEAP32[$bRev>>2] = $70;
 $71 = HEAP32[$pLoop>>2]|0;
 $72 = (($71) + 36|0);
 $73 = HEAP32[$72>>2]|0;
 $74 = $73 & 64;
 $75 = ($74|0)!=(0);
 if ($75) {
  $76 = HEAP32[$3>>2]|0;
  $77 = (($76) + 34|0);
  $78 = HEAP16[$77>>1]|0;
  $79 = $78&65535;
  $80 = $79 & 32;
  $81 = ($80|0)==(0);
  $83 = $81;
 } else {
  $83 = 0;
 }
 $82 = $83&1;
 HEAP32[$omitTable>>2] = $82;
 $84 = HEAP32[$v>>2]|0;
 $85 = (_sqlite3VdbeMakeLabel($84)|0);
 $86 = HEAP32[$pLevel>>2]|0;
 $87 = (($86) + 16|0);
 HEAP32[$87>>2] = $85;
 $88 = HEAP32[$pLevel>>2]|0;
 $89 = (($88) + 12|0);
 HEAP32[$89>>2] = $85;
 HEAP32[$addrBrk>>2] = $85;
 $90 = HEAP32[$v>>2]|0;
 $91 = (_sqlite3VdbeMakeLabel($90)|0);
 $92 = HEAP32[$pLevel>>2]|0;
 $93 = (($92) + 24|0);
 HEAP32[$93>>2] = $91;
 HEAP32[$addrCont>>2] = $91;
 $94 = HEAP32[$pLevel>>2]|0;
 $95 = (($94) + 36|0);
 $96 = HEAP8[$95>>0]|0;
 $97 = $96&255;
 $98 = ($97|0)>(0);
 if ($98) {
  $99 = HEAP32[$pTabItem>>2]|0;
  $100 = (($99) + 36|0);
  $101 = HEAP8[$100>>0]|0;
  $102 = $101&255;
  $103 = $102 & 8;
  $104 = ($103|0)!=(0);
  if ($104) {
   $105 = HEAP32[$pParse>>2]|0;
   $106 = (($105) + 72|0);
   $107 = HEAP32[$106>>2]|0;
   $108 = (($107) + 1)|0;
   HEAP32[$106>>2] = $108;
   $109 = HEAP32[$pLevel>>2]|0;
   HEAP32[$109>>2] = $108;
   $110 = HEAP32[$v>>2]|0;
   $111 = HEAP32[$pLevel>>2]|0;
   $112 = HEAP32[$111>>2]|0;
   (_sqlite3VdbeAddOp2($110,25,0,$112)|0);
  }
 }
 $113 = HEAP32[$pTabItem>>2]|0;
 $114 = (($113) + 37|0);
 $115 = HEAP8[$114>>0]|0;
 $116 = ($115&255) >>> 2;
 $117 = $116 & 1;
 $118 = $117&255;
 $119 = ($118|0)!=(0);
 do {
  if ($119) {
   $120 = HEAP32[$pTabItem>>2]|0;
   $121 = (($120) + 28|0);
   $122 = HEAP32[$121>>2]|0;
   HEAP32[$regYield>>2] = $122;
   $123 = HEAP32[$v>>2]|0;
   $124 = HEAP32[$regYield>>2]|0;
   $125 = HEAP32[$pTabItem>>2]|0;
   $126 = (($125) + 24|0);
   $127 = HEAP32[$126>>2]|0;
   (_sqlite3VdbeAddOp3($123,20,$124,0,$127)|0);
   $128 = HEAP32[$v>>2]|0;
   $129 = HEAP32[$regYield>>2]|0;
   $130 = HEAP32[$addrBrk>>2]|0;
   $131 = (_sqlite3VdbeAddOp2($128,22,$129,$130)|0);
   $132 = HEAP32[$pLevel>>2]|0;
   $133 = (($132) + 44|0);
   HEAP32[$133>>2] = $131;
   $134 = HEAP32[$pLevel>>2]|0;
   $135 = (($134) + 37|0);
   HEAP8[$135>>0] = 16;
  } else {
   $136 = HEAP32[$pLoop>>2]|0;
   $137 = (($136) + 36|0);
   $138 = HEAP32[$137>>2]|0;
   $139 = $138 & 1024;
   $140 = ($139|0)!=(0);
   $141 = HEAP32[$pLoop>>2]|0;
   if ($140) {
    $142 = (($141) + 40|0);
    $143 = HEAP16[$142>>1]|0;
    $144 = $143&65535;
    HEAP32[$nConstraint>>2] = $144;
    $145 = HEAP32[$pParse>>2]|0;
    _sqlite3ExprCachePush($145);
    $146 = HEAP32[$pParse>>2]|0;
    $147 = HEAP32[$nConstraint>>2]|0;
    $148 = (($147) + 2)|0;
    $149 = (_sqlite3GetTempRange($146,$148)|0);
    HEAP32[$iReg>>2] = $149;
    $150 = HEAP32[$pLevel>>2]|0;
    $151 = (($150) + 12|0);
    $152 = HEAP32[$151>>2]|0;
    HEAP32[$addrNotFound>>2] = $152;
    HEAP32[$j>>2] = 0;
    while(1) {
     $153 = HEAP32[$j>>2]|0;
     $154 = HEAP32[$nConstraint>>2]|0;
     $155 = ($153|0)<($154|0);
     if (!($155)) {
      break;
     }
     $156 = HEAP32[$iReg>>2]|0;
     $157 = HEAP32[$j>>2]|0;
     $158 = (($156) + ($157))|0;
     $159 = (($158) + 2)|0;
     HEAP32[$iTarget>>2] = $159;
     $160 = HEAP32[$j>>2]|0;
     $161 = HEAP32[$pLoop>>2]|0;
     $162 = (($161) + 44|0);
     $163 = HEAP32[$162>>2]|0;
     $164 = (($163) + ($160<<2)|0);
     $165 = HEAP32[$164>>2]|0;
     HEAP32[$pTerm>>2] = $165;
     $166 = HEAP32[$pTerm>>2]|0;
     $167 = ($166|0)==(0|0);
     do {
      if (!($167)) {
       $168 = HEAP32[$pTerm>>2]|0;
       $169 = (($168) + 18|0);
       $170 = HEAP16[$169>>1]|0;
       $171 = $170&65535;
       $172 = $171 & 1;
       $173 = ($172|0)!=(0);
       $174 = HEAP32[$pParse>>2]|0;
       $175 = HEAP32[$pTerm>>2]|0;
       if ($173) {
        $176 = HEAP32[$pLevel>>2]|0;
        $177 = HEAP32[$j>>2]|0;
        $178 = HEAP32[$bRev>>2]|0;
        $179 = HEAP32[$iTarget>>2]|0;
        (_codeEqualityTerm($174,$175,$176,$177,$178,$179)|0);
        $180 = HEAP32[$pLevel>>2]|0;
        $181 = (($180) + 16|0);
        $182 = HEAP32[$181>>2]|0;
        HEAP32[$addrNotFound>>2] = $182;
        break;
       } else {
        $183 = HEAP32[$175>>2]|0;
        $184 = (($183) + 16|0);
        $185 = HEAP32[$184>>2]|0;
        $186 = HEAP32[$iTarget>>2]|0;
        _sqlite3ExprCode($174,$185,$186);
        break;
       }
      }
     } while(0);
     $187 = HEAP32[$j>>2]|0;
     $188 = (($187) + 1)|0;
     HEAP32[$j>>2] = $188;
    }
    $189 = HEAP32[$v>>2]|0;
    $190 = HEAP32[$pLoop>>2]|0;
    $191 = (($190) + 24|0);
    $192 = HEAP32[$191>>2]|0;
    $193 = HEAP32[$iReg>>2]|0;
    (_sqlite3VdbeAddOp2($189,25,$192,$193)|0);
    $194 = HEAP32[$v>>2]|0;
    $195 = HEAP32[$nConstraint>>2]|0;
    $196 = HEAP32[$iReg>>2]|0;
    $197 = (($196) + 1)|0;
    (_sqlite3VdbeAddOp2($194,25,$195,$197)|0);
    $198 = HEAP32[$v>>2]|0;
    $199 = HEAP32[$iCur>>2]|0;
    $200 = HEAP32[$addrNotFound>>2]|0;
    $201 = HEAP32[$iReg>>2]|0;
    $202 = HEAP32[$pLoop>>2]|0;
    $203 = (($202) + 24|0);
    $204 = (($203) + 8|0);
    $205 = HEAP32[$204>>2]|0;
    $206 = HEAP32[$pLoop>>2]|0;
    $207 = (($206) + 24|0);
    $208 = (($207) + 4|0);
    $209 = HEAP8[$208>>0]|0;
    $210 = $209&255;
    $211 = ($210|0)!=(0);
    $212 = $211 ? -11 : -2;
    (_sqlite3VdbeAddOp4($198,14,$199,$200,$201,$205,$212)|0);
    $213 = HEAP32[$pLoop>>2]|0;
    $214 = (($213) + 24|0);
    $215 = (($214) + 4|0);
    HEAP8[$215>>0] = 0;
    HEAP32[$j>>2] = 0;
    while(1) {
     $216 = HEAP32[$j>>2]|0;
     $217 = HEAP32[$nConstraint>>2]|0;
     $218 = ($216|0)<($217|0);
     $219 = HEAP32[$j>>2]|0;
     $220 = ($219|0)<(16);
     $or$cond = $218 & $220;
     if (!($or$cond)) {
      break;
     }
     $221 = HEAP32[$pLoop>>2]|0;
     $222 = (($221) + 24|0);
     $223 = (($222) + 6|0);
     $224 = HEAP16[$223>>1]|0;
     $225 = $224&65535;
     $226 = HEAP32[$j>>2]|0;
     $227 = $225 >> $226;
     $228 = $227 & 1;
     $229 = ($228|0)!=(0);
     if ($229) {
      $230 = HEAP32[$pLevel>>2]|0;
      $231 = HEAP32[$j>>2]|0;
      $232 = HEAP32[$pLoop>>2]|0;
      $233 = (($232) + 44|0);
      $234 = HEAP32[$233>>2]|0;
      $235 = (($234) + ($231<<2)|0);
      $236 = HEAP32[$235>>2]|0;
      _disableTerm($230,$236);
     }
     $237 = HEAP32[$j>>2]|0;
     $238 = (($237) + 1)|0;
     HEAP32[$j>>2] = $238;
    }
    $239 = HEAP32[$pLevel>>2]|0;
    $240 = (($239) + 37|0);
    HEAP8[$240>>0] = -107;
    $241 = HEAP32[$iCur>>2]|0;
    $242 = HEAP32[$pLevel>>2]|0;
    $243 = (($242) + 40|0);
    HEAP32[$243>>2] = $241;
    $244 = HEAP32[$v>>2]|0;
    $245 = (_sqlite3VdbeCurrentAddr($244)|0);
    $246 = HEAP32[$pLevel>>2]|0;
    $247 = (($246) + 44|0);
    HEAP32[$247>>2] = $245;
    $248 = HEAP32[$pParse>>2]|0;
    $249 = HEAP32[$iReg>>2]|0;
    $250 = HEAP32[$nConstraint>>2]|0;
    $251 = (($250) + 2)|0;
    _sqlite3ReleaseTempRange($248,$249,$251);
    $252 = HEAP32[$pParse>>2]|0;
    _sqlite3ExprCachePop($252);
    break;
   }
   $253 = (($141) + 36|0);
   $254 = HEAP32[$253>>2]|0;
   $255 = $254 & 256;
   $256 = ($255|0)!=(0);
   if ($256) {
    $257 = HEAP32[$pLoop>>2]|0;
    $258 = (($257) + 36|0);
    $259 = HEAP32[$258>>2]|0;
    $260 = $259 & 5;
    $261 = ($260|0)!=(0);
    if ($261) {
     $262 = HEAP32[$pLoop>>2]|0;
     $263 = (($262) + 44|0);
     $264 = HEAP32[$263>>2]|0;
     $265 = HEAP32[$264>>2]|0;
     HEAP32[$pTerm>>2] = $265;
     $266 = HEAP32[$pParse>>2]|0;
     $267 = (($266) + 72|0);
     $268 = HEAP32[$267>>2]|0;
     $269 = (($268) + 1)|0;
     HEAP32[$267>>2] = $269;
     HEAP32[$iReleaseReg>>2] = $269;
     $270 = HEAP32[$pParse>>2]|0;
     $271 = HEAP32[$pTerm>>2]|0;
     $272 = HEAP32[$pLevel>>2]|0;
     $273 = HEAP32[$bRev>>2]|0;
     $274 = HEAP32[$iReleaseReg>>2]|0;
     $275 = (_codeEqualityTerm($270,$271,$272,0,$273,$274)|0);
     HEAP32[$iRowidReg>>2] = $275;
     $276 = HEAP32[$iRowidReg>>2]|0;
     $277 = HEAP32[$iReleaseReg>>2]|0;
     $278 = ($276|0)!=($277|0);
     if ($278) {
      $279 = HEAP32[$pParse>>2]|0;
      $280 = HEAP32[$iReleaseReg>>2]|0;
      _sqlite3ReleaseTempReg($279,$280);
     }
     $281 = HEAP32[$pLevel>>2]|0;
     $282 = (($281) + 16|0);
     $283 = HEAP32[$282>>2]|0;
     HEAP32[$addrNxt>>2] = $283;
     $284 = HEAP32[$v>>2]|0;
     $285 = HEAP32[$iRowidReg>>2]|0;
     $286 = HEAP32[$addrNxt>>2]|0;
     (_sqlite3VdbeAddOp2($284,38,$285,$286)|0);
     $287 = HEAP32[$v>>2]|0;
     $288 = HEAP32[$iCur>>2]|0;
     $289 = HEAP32[$addrNxt>>2]|0;
     $290 = HEAP32[$iRowidReg>>2]|0;
     (_sqlite3VdbeAddOp3($287,70,$288,$289,$290)|0);
     $291 = HEAP32[$pParse>>2]|0;
     $292 = HEAP32[$iRowidReg>>2]|0;
     _sqlite3ExprCacheAffinityChange($291,$292,1);
     $293 = HEAP32[$pParse>>2]|0;
     $294 = HEAP32[$iCur>>2]|0;
     $295 = HEAP32[$iRowidReg>>2]|0;
     _sqlite3ExprCacheStore($293,$294,-1,$295);
     $296 = HEAP32[$pLevel>>2]|0;
     $297 = (($296) + 37|0);
     HEAP8[$297>>0] = -102;
     break;
    }
   }
   $298 = HEAP32[$pLoop>>2]|0;
   $299 = (($298) + 36|0);
   $300 = HEAP32[$299>>2]|0;
   $301 = $300 & 256;
   $302 = ($301|0)!=(0);
   if ($302) {
    $303 = HEAP32[$pLoop>>2]|0;
    $304 = (($303) + 36|0);
    $305 = HEAP32[$304>>2]|0;
    $306 = $305 & 2;
    $307 = ($306|0)!=(0);
    if ($307) {
     HEAP32[$testOp>>2] = 154;
     HEAP32[$memEndValue>>2] = 0;
     HEAP32[$j>>2] = 0;
     HEAP32[$pEnd>>2] = 0;
     HEAP32[$pStart>>2] = 0;
     $308 = HEAP32[$pLoop>>2]|0;
     $309 = (($308) + 36|0);
     $310 = HEAP32[$309>>2]|0;
     $311 = $310 & 32;
     $312 = ($311|0)!=(0);
     if ($312) {
      $313 = HEAP32[$j>>2]|0;
      $314 = (($313) + 1)|0;
      HEAP32[$j>>2] = $314;
      $315 = HEAP32[$pLoop>>2]|0;
      $316 = (($315) + 44|0);
      $317 = HEAP32[$316>>2]|0;
      $318 = (($317) + ($313<<2)|0);
      $319 = HEAP32[$318>>2]|0;
      HEAP32[$pStart>>2] = $319;
     }
     $320 = HEAP32[$pLoop>>2]|0;
     $321 = (($320) + 36|0);
     $322 = HEAP32[$321>>2]|0;
     $323 = $322 & 16;
     $324 = ($323|0)!=(0);
     if ($324) {
      $325 = HEAP32[$j>>2]|0;
      $326 = (($325) + 1)|0;
      HEAP32[$j>>2] = $326;
      $327 = HEAP32[$pLoop>>2]|0;
      $328 = (($327) + 44|0);
      $329 = HEAP32[$328>>2]|0;
      $330 = (($329) + ($325<<2)|0);
      $331 = HEAP32[$330>>2]|0;
      HEAP32[$pEnd>>2] = $331;
     }
     $332 = HEAP32[$bRev>>2]|0;
     $333 = ($332|0)!=(0);
     if ($333) {
      $334 = HEAP32[$pStart>>2]|0;
      HEAP32[$pTerm>>2] = $334;
      $335 = HEAP32[$pEnd>>2]|0;
      HEAP32[$pStart>>2] = $335;
      $336 = HEAP32[$pTerm>>2]|0;
      HEAP32[$pEnd>>2] = $336;
     }
     $337 = HEAP32[$pStart>>2]|0;
     $338 = ($337|0)!=(0|0);
     if ($338) {
      $339 = HEAP32[$pStart>>2]|0;
      $340 = HEAP32[$339>>2]|0;
      HEAP32[$pX>>2] = $340;
      $341 = HEAP32[$pParse>>2]|0;
      $342 = HEAP32[$pX>>2]|0;
      $343 = (($342) + 16|0);
      $344 = HEAP32[$343>>2]|0;
      $345 = (_sqlite3ExprCodeTemp($341,$344,$rTemp)|0);
      HEAP32[$r1>>2] = $345;
      $346 = HEAP32[$v>>2]|0;
      $347 = HEAP32[$pX>>2]|0;
      $348 = HEAP8[$347>>0]|0;
      $349 = $348&255;
      $350 = (($349) - 80)|0;
      $351 = (23600 + ($350)|0);
      $352 = HEAP8[$351>>0]|0;
      $353 = $352&255;
      $354 = HEAP32[$iCur>>2]|0;
      $355 = HEAP32[$addrBrk>>2]|0;
      $356 = HEAP32[$r1>>2]|0;
      (_sqlite3VdbeAddOp3($346,$353,$354,$355,$356)|0);
      $357 = HEAP32[$pParse>>2]|0;
      $358 = HEAP32[$r1>>2]|0;
      _sqlite3ExprCacheAffinityChange($357,$358,1);
      $359 = HEAP32[$pParse>>2]|0;
      $360 = HEAP32[$rTemp>>2]|0;
      _sqlite3ReleaseTempReg($359,$360);
      $361 = HEAP32[$pLevel>>2]|0;
      $362 = HEAP32[$pStart>>2]|0;
      _disableTerm($361,$362);
     } else {
      $363 = HEAP32[$v>>2]|0;
      $364 = HEAP32[$bRev>>2]|0;
      $365 = ($364|0)!=(0);
      $366 = $365 ? 105 : 108;
      $367 = HEAP32[$iCur>>2]|0;
      $368 = HEAP32[$addrBrk>>2]|0;
      (_sqlite3VdbeAddOp2($363,$366,$367,$368)|0);
     }
     $369 = HEAP32[$pEnd>>2]|0;
     $370 = ($369|0)!=(0|0);
     if ($370) {
      $371 = HEAP32[$pEnd>>2]|0;
      $372 = HEAP32[$371>>2]|0;
      HEAP32[$pX1>>2] = $372;
      $373 = HEAP32[$pParse>>2]|0;
      $374 = (($373) + 72|0);
      $375 = HEAP32[$374>>2]|0;
      $376 = (($375) + 1)|0;
      HEAP32[$374>>2] = $376;
      HEAP32[$memEndValue>>2] = $376;
      $377 = HEAP32[$pParse>>2]|0;
      $378 = HEAP32[$pX1>>2]|0;
      $379 = (($378) + 16|0);
      $380 = HEAP32[$379>>2]|0;
      $381 = HEAP32[$memEndValue>>2]|0;
      _sqlite3ExprCode($377,$380,$381);
      $382 = HEAP32[$pX1>>2]|0;
      $383 = HEAP8[$382>>0]|0;
      $384 = $383&255;
      $385 = ($384|0)==(82);
      if ($385) {
       label = 41;
      } else {
       $386 = HEAP32[$pX1>>2]|0;
       $387 = HEAP8[$386>>0]|0;
       $388 = $387&255;
       $389 = ($388|0)==(80);
       if ($389) {
        label = 41;
       } else {
        $393 = HEAP32[$bRev>>2]|0;
        $394 = ($393|0)!=(0);
        $395 = $394 ? 82 : 80;
        HEAP32[$testOp>>2] = $395;
       }
      }
      if ((label|0) == 41) {
       $390 = HEAP32[$bRev>>2]|0;
       $391 = ($390|0)!=(0);
       $392 = $391 ? 81 : 83;
       HEAP32[$testOp>>2] = $392;
      }
      $396 = HEAP32[$pLevel>>2]|0;
      $397 = HEAP32[$pEnd>>2]|0;
      _disableTerm($396,$397);
     }
     $398 = HEAP32[$v>>2]|0;
     $399 = (_sqlite3VdbeCurrentAddr($398)|0);
     HEAP32[$start>>2] = $399;
     $400 = HEAP32[$bRev>>2]|0;
     $401 = ($400|0)!=(0);
     $402 = $401 ? 8 : 9;
     $403 = $402&255;
     $404 = HEAP32[$pLevel>>2]|0;
     $405 = (($404) + 37|0);
     HEAP8[$405>>0] = $403;
     $406 = HEAP32[$iCur>>2]|0;
     $407 = HEAP32[$pLevel>>2]|0;
     $408 = (($407) + 40|0);
     HEAP32[$408>>2] = $406;
     $409 = HEAP32[$start>>2]|0;
     $410 = HEAP32[$pLevel>>2]|0;
     $411 = (($410) + 44|0);
     HEAP32[$411>>2] = $409;
     $412 = HEAP32[$testOp>>2]|0;
     $413 = ($412|0)!=(154);
     if (!($413)) {
      break;
     }
     $414 = HEAP32[$pParse>>2]|0;
     $415 = (($414) + 72|0);
     $416 = HEAP32[$415>>2]|0;
     $417 = (($416) + 1)|0;
     HEAP32[$415>>2] = $417;
     HEAP32[$iRowidReg>>2] = $417;
     $418 = HEAP32[$v>>2]|0;
     $419 = HEAP32[$iCur>>2]|0;
     $420 = HEAP32[$iRowidReg>>2]|0;
     (_sqlite3VdbeAddOp2($418,103,$419,$420)|0);
     $421 = HEAP32[$pParse>>2]|0;
     $422 = HEAP32[$iCur>>2]|0;
     $423 = HEAP32[$iRowidReg>>2]|0;
     _sqlite3ExprCacheStore($421,$422,-1,$423);
     $424 = HEAP32[$v>>2]|0;
     $425 = HEAP32[$testOp>>2]|0;
     $426 = HEAP32[$memEndValue>>2]|0;
     $427 = HEAP32[$addrBrk>>2]|0;
     $428 = HEAP32[$iRowidReg>>2]|0;
     (_sqlite3VdbeAddOp3($424,$425,$426,$427,$428)|0);
     $429 = HEAP32[$v>>2]|0;
     _sqlite3VdbeChangeP5($429,83);
     break;
    }
   }
   $430 = HEAP32[$pLoop>>2]|0;
   $431 = (($430) + 36|0);
   $432 = HEAP32[$431>>2]|0;
   $433 = $432 & 512;
   $434 = ($433|0)!=(0);
   $435 = HEAP32[$pLoop>>2]|0;
   if ($434) {
    $436 = (($435) + 24|0);
    $437 = HEAP16[$436>>1]|0;
    HEAP16[$nEq>>1] = $437;
    HEAP32[$pRangeStart>>2] = 0;
    HEAP32[$pRangeEnd>>2] = 0;
    HEAP32[$nExtraReg>>2] = 0;
    HEAP8[$cEndAff>>0] = 0;
    HEAP8[$bSeekPastNull>>0] = 0;
    HEAP8[$bStopAtNull>>0] = 0;
    $438 = HEAP32[$pLoop>>2]|0;
    $439 = (($438) + 24|0);
    $440 = (($439) + 4|0);
    $441 = HEAP32[$440>>2]|0;
    HEAP32[$pIdx>>2] = $441;
    $442 = HEAP32[$pLevel>>2]|0;
    $443 = (($442) + 8|0);
    $444 = HEAP32[$443>>2]|0;
    HEAP32[$iIdxCur>>2] = $444;
    $445 = HEAP32[$3>>2]|0;
    $446 = (($445) + 34|0);
    $447 = HEAP16[$446>>1]|0;
    $448 = $447&65535;
    $449 = $448 & 1;
    $450 = ($449|0)!=(0);
    if ($450) {
     $451 = HEAP32[$3>>2]|0;
     $452 = (($451) + 36|0);
     $453 = HEAP8[$452>>0]|0;
     $454 = $453 << 24 >> 24;
     $455 = ($454|0)>(0);
     if ($455) {
      $456 = HEAP32[$pIdx>>2]|0;
      $457 = (($456) + 50|0);
      $458 = HEAP16[$457>>1]|0;
      $459 = $458&65535;
      $460 = HEAP16[$nEq>>1]|0;
      $461 = $460&65535;
      $462 = ($459|0)>($461|0);
      if ($462) {
       HEAP8[$bSeekPastNull>>0] = 1;
       HEAP32[$nExtraReg>>2] = 1;
      }
     }
    }
    $463 = HEAP16[$nEq>>1]|0;
    $464 = $463&65535;
    HEAP32[$j>>2] = $464;
    $465 = HEAP32[$pLoop>>2]|0;
    $466 = (($465) + 36|0);
    $467 = HEAP32[$466>>2]|0;
    $468 = $467 & 32;
    $469 = ($468|0)!=(0);
    if ($469) {
     $470 = HEAP32[$j>>2]|0;
     $471 = (($470) + 1)|0;
     HEAP32[$j>>2] = $471;
     $472 = HEAP32[$pLoop>>2]|0;
     $473 = (($472) + 44|0);
     $474 = HEAP32[$473>>2]|0;
     $475 = (($474) + ($470<<2)|0);
     $476 = HEAP32[$475>>2]|0;
     HEAP32[$pRangeStart>>2] = $476;
     HEAP32[$nExtraReg>>2] = 1;
    }
    $477 = HEAP32[$pLoop>>2]|0;
    $478 = (($477) + 36|0);
    $479 = HEAP32[$478>>2]|0;
    $480 = $479 & 16;
    $481 = ($480|0)!=(0);
    if ($481) {
     $482 = HEAP32[$j>>2]|0;
     $483 = (($482) + 1)|0;
     HEAP32[$j>>2] = $483;
     $484 = HEAP32[$pLoop>>2]|0;
     $485 = (($484) + 44|0);
     $486 = HEAP32[$485>>2]|0;
     $487 = (($486) + ($482<<2)|0);
     $488 = HEAP32[$487>>2]|0;
     HEAP32[$pRangeEnd>>2] = $488;
     HEAP32[$nExtraReg>>2] = 1;
     $489 = HEAP32[$pRangeStart>>2]|0;
     $490 = ($489|0)==(0|0);
     if ($490) {
      $491 = HEAP16[$nEq>>1]|0;
      $492 = $491&65535;
      $493 = HEAP32[$pIdx>>2]|0;
      $494 = (($493) + 4|0);
      $495 = HEAP32[$494>>2]|0;
      $496 = (($495) + ($492<<1)|0);
      $497 = HEAP16[$496>>1]|0;
      $498 = $497 << 16 >> 16;
      HEAP32[$j>>2] = $498;
      $499 = ($498|0)>=(0);
      if ($499) {
       $500 = HEAP32[$j>>2]|0;
       $501 = HEAP32[$pIdx>>2]|0;
       $502 = (($501) + 12|0);
       $503 = HEAP32[$502>>2]|0;
       $504 = (($503) + 4|0);
       $505 = HEAP32[$504>>2]|0;
       $506 = (($505) + (($500*24)|0)|0);
       $507 = (($506) + 20|0);
       $508 = HEAP8[$507>>0]|0;
       $509 = $508&255;
       $510 = ($509|0)==(0);
       if ($510) {
        HEAP8[$bSeekPastNull>>0] = 1;
       }
      }
     }
    }
    $511 = HEAP32[$pParse>>2]|0;
    $512 = HEAP32[$pLevel>>2]|0;
    $513 = HEAP32[$bRev>>2]|0;
    $514 = HEAP32[$nExtraReg>>2]|0;
    $515 = (_codeAllEqualityTerms($511,$512,$513,$514,$zStartAff)|0);
    HEAP32[$regBase>>2] = $515;
    $516 = HEAP32[$zStartAff>>2]|0;
    $517 = ($516|0)!=(0|0);
    if ($517) {
     $518 = HEAP16[$nEq>>1]|0;
     $519 = $518&65535;
     $520 = HEAP32[$zStartAff>>2]|0;
     $521 = (($520) + ($519)|0);
     $522 = HEAP8[$521>>0]|0;
     HEAP8[$cEndAff>>0] = $522;
    }
    $523 = HEAP32[$pLevel>>2]|0;
    $524 = (($523) + 16|0);
    $525 = HEAP32[$524>>2]|0;
    HEAP32[$addrNxt>>2] = $525;
    $526 = HEAP16[$nEq>>1]|0;
    $527 = $526&65535;
    $528 = HEAP32[$pIdx>>2]|0;
    $529 = (($528) + 50|0);
    $530 = HEAP16[$529>>1]|0;
    $531 = $530&65535;
    $532 = ($527|0)<($531|0);
    if ($532) {
     $533 = HEAP32[$bRev>>2]|0;
     $534 = HEAP16[$nEq>>1]|0;
     $535 = $534&65535;
     $536 = HEAP32[$pIdx>>2]|0;
     $537 = (($536) + 28|0);
     $538 = HEAP32[$537>>2]|0;
     $539 = (($538) + ($535)|0);
     $540 = HEAP8[$539>>0]|0;
     $541 = $540&255;
     $542 = ($541|0)==(0);
     $543 = $542&1;
     $544 = ($533|0)==($543|0);
     if ($544) {
      label = 64;
     } else {
      label = 62;
     }
    } else {
     label = 62;
    }
    if ((label|0) == 62) {
     $545 = HEAP32[$bRev>>2]|0;
     $546 = ($545|0)!=(0);
     if ($546) {
      $547 = HEAP32[$pIdx>>2]|0;
      $548 = (($547) + 50|0);
      $549 = HEAP16[$548>>1]|0;
      $550 = $549&65535;
      $551 = HEAP16[$nEq>>1]|0;
      $552 = $551&65535;
      $553 = ($550|0)==($552|0);
      if ($553) {
       label = 64;
      }
     }
    }
    if ((label|0) == 64) {
     $554 = HEAP32[$pRangeEnd>>2]|0;
     HEAP32[$t>>2] = $554;
     $555 = HEAP32[$pRangeStart>>2]|0;
     HEAP32[$pRangeEnd>>2] = $555;
     $556 = HEAP32[$t>>2]|0;
     HEAP32[$pRangeStart>>2] = $556;
     $557 = HEAP8[$bSeekPastNull>>0]|0;
     HEAP8[$t3>>0] = $557;
     $558 = HEAP8[$bStopAtNull>>0]|0;
     HEAP8[$bSeekPastNull>>0] = $558;
     $559 = HEAP8[$t3>>0]|0;
     HEAP8[$bStopAtNull>>0] = $559;
    }
    $560 = HEAP32[$pRangeStart>>2]|0;
    $561 = ($560|0)!=(0|0);
    if ($561) {
     $562 = HEAP32[$pRangeStart>>2]|0;
     $563 = (($562) + 18|0);
     $564 = HEAP16[$563>>1]|0;
     $565 = $564&65535;
     $566 = $565 & 40;
     $567 = ($566|0)!=(0);
     $569 = $567;
    } else {
     $569 = 1;
    }
    $568 = $569&1;
    HEAP32[$startEq>>2] = $568;
    $570 = HEAP32[$pRangeEnd>>2]|0;
    $571 = ($570|0)!=(0|0);
    if ($571) {
     $572 = HEAP32[$pRangeEnd>>2]|0;
     $573 = (($572) + 18|0);
     $574 = HEAP16[$573>>1]|0;
     $575 = $574&65535;
     $576 = $575 & 40;
     $577 = ($576|0)!=(0);
     $579 = $577;
    } else {
     $579 = 1;
    }
    $578 = $579&1;
    HEAP32[$endEq>>2] = $578;
    $580 = HEAP32[$pRangeStart>>2]|0;
    $581 = ($580|0)!=(0|0);
    if ($581) {
     $586 = 1;
    } else {
     $582 = HEAP16[$nEq>>1]|0;
     $583 = $582&65535;
     $584 = ($583|0)>(0);
     $586 = $584;
    }
    $585 = $586&1;
    HEAP32[$start_constraints>>2] = $585;
    $587 = HEAP16[$nEq>>1]|0;
    $588 = $587&65535;
    HEAP32[$nConstraint2>>2] = $588;
    $589 = HEAP32[$pRangeStart>>2]|0;
    $590 = ($589|0)!=(0|0);
    if ($590) {
     $591 = HEAP32[$pRangeStart>>2]|0;
     $592 = HEAP32[$591>>2]|0;
     $593 = (($592) + 16|0);
     $594 = HEAP32[$593>>2]|0;
     HEAP32[$pRight>>2] = $594;
     $595 = HEAP32[$pParse>>2]|0;
     $596 = HEAP32[$pRight>>2]|0;
     $597 = HEAP32[$regBase>>2]|0;
     $598 = HEAP16[$nEq>>1]|0;
     $599 = $598&65535;
     $600 = (($597) + ($599))|0;
     _sqlite3ExprCode($595,$596,$600);
     $601 = HEAP32[$pRangeStart>>2]|0;
     $602 = (($601) + 20|0);
     $603 = HEAP8[$602>>0]|0;
     $604 = $603&255;
     $605 = $604 & 0;
     $606 = ($605|0)==(0);
     do {
      if ($606) {
       $607 = HEAP32[$pRight>>2]|0;
       $608 = (_sqlite3ExprCanBeNull($607)|0);
       $609 = ($608|0)!=(0);
       if (!($609)) {
        break;
       }
       $610 = HEAP32[$v>>2]|0;
       $611 = HEAP32[$regBase>>2]|0;
       $612 = HEAP16[$nEq>>1]|0;
       $613 = $612&65535;
       $614 = (($611) + ($613))|0;
       $615 = HEAP32[$addrNxt>>2]|0;
       (_sqlite3VdbeAddOp2($610,76,$614,$615)|0);
      }
     } while(0);
     $616 = HEAP32[$zStartAff>>2]|0;
     $617 = ($616|0)!=(0|0);
     do {
      if ($617) {
       $618 = HEAP32[$pRight>>2]|0;
       $619 = HEAP16[$nEq>>1]|0;
       $620 = $619&65535;
       $621 = HEAP32[$zStartAff>>2]|0;
       $622 = (($621) + ($620)|0);
       $623 = HEAP8[$622>>0]|0;
       $624 = (_sqlite3CompareAffinity($618,$623)|0);
       $625 = $624 << 24 >> 24;
       $626 = ($625|0)==(65);
       if ($626) {
        $627 = HEAP16[$nEq>>1]|0;
        $628 = $627&65535;
        $629 = HEAP32[$zStartAff>>2]|0;
        $630 = (($629) + ($628)|0);
        HEAP8[$630>>0] = 65;
       }
       $631 = HEAP32[$pRight>>2]|0;
       $632 = HEAP16[$nEq>>1]|0;
       $633 = $632&65535;
       $634 = HEAP32[$zStartAff>>2]|0;
       $635 = (($634) + ($633)|0);
       $636 = HEAP8[$635>>0]|0;
       $637 = (_sqlite3ExprNeedsNoAffinityChange($631,$636)|0);
       $638 = ($637|0)!=(0);
       if (!($638)) {
        break;
       }
       $639 = HEAP16[$nEq>>1]|0;
       $640 = $639&65535;
       $641 = HEAP32[$zStartAff>>2]|0;
       $642 = (($641) + ($640)|0);
       HEAP8[$642>>0] = 65;
      }
     } while(0);
     $643 = HEAP32[$nConstraint2>>2]|0;
     $644 = (($643) + 1)|0;
     HEAP32[$nConstraint2>>2] = $644;
    } else {
     $645 = HEAP8[$bSeekPastNull>>0]|0;
     $646 = ($645<<24>>24)!=(0);
     if ($646) {
      $647 = HEAP32[$v>>2]|0;
      $648 = HEAP32[$regBase>>2]|0;
      $649 = HEAP16[$nEq>>1]|0;
      $650 = $649&65535;
      $651 = (($648) + ($650))|0;
      (_sqlite3VdbeAddOp2($647,28,0,$651)|0);
      $652 = HEAP32[$nConstraint2>>2]|0;
      $653 = (($652) + 1)|0;
      HEAP32[$nConstraint2>>2] = $653;
      HEAP32[$startEq>>2] = 0;
      HEAP32[$start_constraints>>2] = 1;
     }
    }
    $654 = HEAP32[$pParse>>2]|0;
    $655 = HEAP32[$regBase>>2]|0;
    $656 = HEAP32[$nConstraint2>>2]|0;
    $657 = HEAP8[$bSeekPastNull>>0]|0;
    $658 = $657&255;
    $659 = (($656) - ($658))|0;
    $660 = HEAP32[$zStartAff>>2]|0;
    _codeApplyAffinity($654,$655,$659,$660);
    $661 = HEAP32[$start_constraints>>2]|0;
    $662 = $661 << 2;
    $663 = HEAP32[$startEq>>2]|0;
    $664 = $663 << 1;
    $665 = (($662) + ($664))|0;
    $666 = HEAP32[$bRev>>2]|0;
    $667 = (($665) + ($666))|0;
    $668 = (23608 + ($667)|0);
    $669 = HEAP8[$668>>0]|0;
    $670 = $669&255;
    HEAP32[$op>>2] = $670;
    $671 = HEAP32[$v>>2]|0;
    $672 = HEAP32[$op>>2]|0;
    $673 = HEAP32[$iIdxCur>>2]|0;
    $674 = HEAP32[$addrNxt>>2]|0;
    $675 = HEAP32[$regBase>>2]|0;
    $676 = HEAP32[$nConstraint2>>2]|0;
    (_sqlite3VdbeAddOp4Int($671,$672,$673,$674,$675,$676)|0);
    $677 = HEAP16[$nEq>>1]|0;
    $678 = $677&65535;
    HEAP32[$nConstraint2>>2] = $678;
    $679 = HEAP32[$pRangeEnd>>2]|0;
    $680 = ($679|0)!=(0|0);
    do {
     if ($680) {
      $681 = HEAP32[$pRangeEnd>>2]|0;
      $682 = HEAP32[$681>>2]|0;
      $683 = (($682) + 16|0);
      $684 = HEAP32[$683>>2]|0;
      HEAP32[$pRight4>>2] = $684;
      $685 = HEAP32[$pParse>>2]|0;
      $686 = HEAP32[$regBase>>2]|0;
      $687 = HEAP16[$nEq>>1]|0;
      $688 = $687&65535;
      $689 = (($686) + ($688))|0;
      _sqlite3ExprCacheRemove($685,$689,1);
      $690 = HEAP32[$pParse>>2]|0;
      $691 = HEAP32[$pRight4>>2]|0;
      $692 = HEAP32[$regBase>>2]|0;
      $693 = HEAP16[$nEq>>1]|0;
      $694 = $693&65535;
      $695 = (($692) + ($694))|0;
      _sqlite3ExprCode($690,$691,$695);
      $696 = HEAP32[$pRangeEnd>>2]|0;
      $697 = (($696) + 20|0);
      $698 = HEAP8[$697>>0]|0;
      $699 = $698&255;
      $700 = $699 & 0;
      $701 = ($700|0)==(0);
      do {
       if ($701) {
        $702 = HEAP32[$pRight4>>2]|0;
        $703 = (_sqlite3ExprCanBeNull($702)|0);
        $704 = ($703|0)!=(0);
        if (!($704)) {
         break;
        }
        $705 = HEAP32[$v>>2]|0;
        $706 = HEAP32[$regBase>>2]|0;
        $707 = HEAP16[$nEq>>1]|0;
        $708 = $707&65535;
        $709 = (($706) + ($708))|0;
        $710 = HEAP32[$addrNxt>>2]|0;
        (_sqlite3VdbeAddOp2($705,76,$709,$710)|0);
       }
      } while(0);
      $711 = HEAP32[$pRight4>>2]|0;
      $712 = HEAP8[$cEndAff>>0]|0;
      $713 = (_sqlite3CompareAffinity($711,$712)|0);
      $714 = $713 << 24 >> 24;
      $715 = ($714|0)!=(65);
      do {
       if ($715) {
        $716 = HEAP32[$pRight4>>2]|0;
        $717 = HEAP8[$cEndAff>>0]|0;
        $718 = (_sqlite3ExprNeedsNoAffinityChange($716,$717)|0);
        $719 = ($718|0)!=(0);
        if ($719) {
         break;
        }
        $720 = HEAP32[$pParse>>2]|0;
        $721 = HEAP32[$regBase>>2]|0;
        $722 = HEAP16[$nEq>>1]|0;
        $723 = $722&65535;
        $724 = (($721) + ($723))|0;
        _codeApplyAffinity($720,$724,1,$cEndAff);
       }
      } while(0);
      $725 = HEAP32[$nConstraint2>>2]|0;
      $726 = (($725) + 1)|0;
      HEAP32[$nConstraint2>>2] = $726;
     } else {
      $727 = HEAP8[$bStopAtNull>>0]|0;
      $728 = ($727<<24>>24)!=(0);
      if (!($728)) {
       break;
      }
      $729 = HEAP32[$v>>2]|0;
      $730 = HEAP32[$regBase>>2]|0;
      $731 = HEAP16[$nEq>>1]|0;
      $732 = $731&65535;
      $733 = (($730) + ($732))|0;
      (_sqlite3VdbeAddOp2($729,28,0,$733)|0);
      HEAP32[$endEq>>2] = 0;
      $734 = HEAP32[$nConstraint2>>2]|0;
      $735 = (($734) + 1)|0;
      HEAP32[$nConstraint2>>2] = $735;
     }
    } while(0);
    $736 = HEAP32[$db>>2]|0;
    $737 = HEAP32[$zStartAff>>2]|0;
    _sqlite3DbFree($736,$737);
    $738 = HEAP32[$v>>2]|0;
    $739 = (_sqlite3VdbeCurrentAddr($738)|0);
    $740 = HEAP32[$pLevel>>2]|0;
    $741 = (($740) + 44|0);
    HEAP32[$741>>2] = $739;
    $742 = HEAP32[$nConstraint2>>2]|0;
    $743 = ($742|0)!=(0);
    if ($743) {
     $744 = HEAP32[$bRev>>2]|0;
     $745 = $744<<1;
     $746 = HEAP32[$endEq>>2]|0;
     $747 = (($745) + ($746))|0;
     $748 = (23616 + ($747)|0);
     $749 = HEAP8[$748>>0]|0;
     $750 = $749&255;
     HEAP32[$op>>2] = $750;
     $751 = HEAP32[$v>>2]|0;
     $752 = HEAP32[$op>>2]|0;
     $753 = HEAP32[$iIdxCur>>2]|0;
     $754 = HEAP32[$addrNxt>>2]|0;
     $755 = HEAP32[$regBase>>2]|0;
     $756 = HEAP32[$nConstraint2>>2]|0;
     (_sqlite3VdbeAddOp4Int($751,$752,$753,$754,$755,$756)|0);
    }
    $757 = HEAP32[$pLevel>>2]|0;
    $758 = HEAP32[$pRangeStart>>2]|0;
    _disableTerm($757,$758);
    $759 = HEAP32[$pLevel>>2]|0;
    $760 = HEAP32[$pRangeEnd>>2]|0;
    _disableTerm($759,$760);
    $761 = HEAP32[$omitTable>>2]|0;
    $762 = ($761|0)!=(0);
    do {
     if (!($762)) {
      $763 = HEAP32[$pIdx>>2]|0;
      $764 = (($763) + 12|0);
      $765 = HEAP32[$764>>2]|0;
      $766 = (($765) + 44|0);
      $767 = HEAP8[$766>>0]|0;
      $768 = $767&255;
      $769 = $768 & 32;
      $770 = ($769|0)==(0);
      if ($770) {
       $771 = HEAP32[$pParse>>2]|0;
       $772 = (($771) + 72|0);
       $773 = HEAP32[$772>>2]|0;
       $774 = (($773) + 1)|0;
       HEAP32[$772>>2] = $774;
       HEAP32[$iRowidReg>>2] = $774;
       $775 = HEAP32[$v>>2]|0;
       $776 = HEAP32[$iIdxCur>>2]|0;
       $777 = HEAP32[$iRowidReg>>2]|0;
       (_sqlite3VdbeAddOp2($775,112,$776,$777)|0);
       $778 = HEAP32[$pParse>>2]|0;
       $779 = HEAP32[$iCur>>2]|0;
       $780 = HEAP32[$iRowidReg>>2]|0;
       _sqlite3ExprCacheStore($778,$779,-1,$780);
       $781 = HEAP32[$v>>2]|0;
       $782 = HEAP32[$iCur>>2]|0;
       $783 = HEAP32[$iRowidReg>>2]|0;
       (_sqlite3VdbeAddOp2($781,66,$782,$783)|0);
       break;
      }
      $784 = HEAP32[$iCur>>2]|0;
      $785 = HEAP32[$iIdxCur>>2]|0;
      $786 = ($784|0)!=($785|0);
      if (!($786)) {
       break;
      }
      $787 = HEAP32[$pIdx>>2]|0;
      $788 = (($787) + 12|0);
      $789 = HEAP32[$788>>2]|0;
      $790 = (_sqlite3PrimaryKeyIndex($789)|0);
      HEAP32[$pPk>>2] = $790;
      $791 = HEAP32[$pParse>>2]|0;
      $792 = HEAP32[$pPk>>2]|0;
      $793 = (($792) + 50|0);
      $794 = HEAP16[$793>>1]|0;
      $795 = $794&65535;
      $796 = (_sqlite3GetTempRange($791,$795)|0);
      HEAP32[$iRowidReg>>2] = $796;
      HEAP32[$j>>2] = 0;
      while(1) {
       $797 = HEAP32[$j>>2]|0;
       $798 = HEAP32[$pPk>>2]|0;
       $799 = (($798) + 50|0);
       $800 = HEAP16[$799>>1]|0;
       $801 = $800&65535;
       $802 = ($797|0)<($801|0);
       if (!($802)) {
        break;
       }
       $803 = HEAP32[$pIdx>>2]|0;
       $804 = HEAP32[$j>>2]|0;
       $805 = HEAP32[$pPk>>2]|0;
       $806 = (($805) + 4|0);
       $807 = HEAP32[$806>>2]|0;
       $808 = (($807) + ($804<<1)|0);
       $809 = HEAP16[$808>>1]|0;
       $810 = (_sqlite3ColumnOfIndex($803,$809)|0);
       $811 = $810 << 16 >> 16;
       HEAP32[$k>>2] = $811;
       $812 = HEAP32[$v>>2]|0;
       $813 = HEAP32[$iIdxCur>>2]|0;
       $814 = HEAP32[$k>>2]|0;
       $815 = HEAP32[$iRowidReg>>2]|0;
       $816 = HEAP32[$j>>2]|0;
       $817 = (($815) + ($816))|0;
       (_sqlite3VdbeAddOp3($812,47,$813,$814,$817)|0);
       $818 = HEAP32[$j>>2]|0;
       $819 = (($818) + 1)|0;
       HEAP32[$j>>2] = $819;
      }
      $820 = HEAP32[$v>>2]|0;
      $821 = HEAP32[$iCur>>2]|0;
      $822 = HEAP32[$addrCont>>2]|0;
      $823 = HEAP32[$iRowidReg>>2]|0;
      $824 = HEAP32[$pPk>>2]|0;
      $825 = (($824) + 50|0);
      $826 = HEAP16[$825>>1]|0;
      $827 = $826&65535;
      (_sqlite3VdbeAddOp4Int($820,68,$821,$822,$823,$827)|0);
     }
    } while(0);
    $828 = HEAP32[$pLoop>>2]|0;
    $829 = (($828) + 36|0);
    $830 = HEAP32[$829>>2]|0;
    $831 = $830 & 4096;
    $832 = ($831|0)!=(0);
    do {
     if ($832) {
      $833 = HEAP32[$pLevel>>2]|0;
      $834 = (($833) + 37|0);
      HEAP8[$834>>0] = -102;
     } else {
      $835 = HEAP32[$bRev>>2]|0;
      $836 = ($835|0)!=(0);
      $837 = HEAP32[$pLevel>>2]|0;
      $838 = (($837) + 37|0);
      if ($836) {
       HEAP8[$838>>0] = 8;
       break;
      } else {
       HEAP8[$838>>0] = 9;
       break;
      }
     }
    } while(0);
    $839 = HEAP32[$iIdxCur>>2]|0;
    $840 = HEAP32[$pLevel>>2]|0;
    $841 = (($840) + 40|0);
    HEAP32[$841>>2] = $839;
    $842 = HEAP32[$pLoop>>2]|0;
    $843 = (($842) + 36|0);
    $844 = HEAP32[$843>>2]|0;
    $845 = $844 & 65536;
    $846 = ($845|0)!=(0);
    $847 = $846 ? 1 : 0;
    $848 = $847&255;
    $849 = HEAP32[$pLevel>>2]|0;
    $850 = (($849) + 38|0);
    HEAP8[$850>>0] = $848;
    $851 = HEAP32[$pLoop>>2]|0;
    $852 = (($851) + 36|0);
    $853 = HEAP32[$852>>2]|0;
    $854 = $853 & 15;
    $855 = ($854|0)==(0);
    if (!($855)) {
     break;
    }
    $856 = HEAP32[$pLevel>>2]|0;
    $857 = (($856) + 39|0);
    HEAP8[$857>>0] = 1;
    break;
   }
   $858 = (($435) + 36|0);
   $859 = HEAP32[$858>>2]|0;
   $860 = $859 & 8192;
   $861 = ($860|0)!=(0);
   if (!($861)) {
    $1272 = HEAP32[$pTabItem>>2]|0;
    $1273 = (($1272) + 37|0);
    $1274 = HEAP8[$1273>>0]|0;
    $1275 = ($1274&255) >>> 3;
    $1276 = $1275 & 1;
    $1277 = $1276&255;
    $1278 = ($1277|0)!=(0);
    if ($1278) {
     $1279 = HEAP32[$pLevel>>2]|0;
     $1280 = (($1279) + 37|0);
     HEAP8[$1280>>0] = -102;
     break;
    } else {
     $1281 = HEAP32[$bRev>>2]|0;
     $1282 = (23624 + ($1281)|0);
     $1283 = HEAP8[$1282>>0]|0;
     $1284 = HEAP32[$pLevel>>2]|0;
     $1285 = (($1284) + 37|0);
     HEAP8[$1285>>0] = $1283;
     $1286 = HEAP32[$iCur>>2]|0;
     $1287 = HEAP32[$pLevel>>2]|0;
     $1288 = (($1287) + 40|0);
     HEAP32[$1288>>2] = $1286;
     $1289 = HEAP32[$v>>2]|0;
     $1290 = HEAP32[$bRev>>2]|0;
     $1291 = (23632 + ($1290)|0);
     $1292 = HEAP8[$1291>>0]|0;
     $1293 = $1292&255;
     $1294 = HEAP32[$iCur>>2]|0;
     $1295 = HEAP32[$addrBrk>>2]|0;
     $1296 = (_sqlite3VdbeAddOp2($1289,$1293,$1294,$1295)|0);
     $1297 = (1 + ($1296))|0;
     $1298 = HEAP32[$pLevel>>2]|0;
     $1299 = (($1298) + 44|0);
     HEAP32[$1299>>2] = $1297;
     $1300 = HEAP32[$pLevel>>2]|0;
     $1301 = (($1300) + 39|0);
     HEAP8[$1301>>0] = 1;
     break;
    }
   }
   HEAP32[$pCov>>2] = 0;
   $862 = HEAP32[$pParse>>2]|0;
   $863 = (($862) + 68|0);
   $864 = HEAP32[$863>>2]|0;
   $865 = (($864) + 1)|0;
   HEAP32[$863>>2] = $865;
   HEAP32[$iCovCur>>2] = $864;
   $866 = HEAP32[$pParse>>2]|0;
   $867 = (($866) + 72|0);
   $868 = HEAP32[$867>>2]|0;
   $869 = (($868) + 1)|0;
   HEAP32[$867>>2] = $869;
   HEAP32[$regReturn>>2] = $869;
   HEAP32[$regRowset>>2] = 0;
   HEAP32[$regRowid>>2] = 0;
   $870 = HEAP32[$v>>2]|0;
   $871 = (_sqlite3VdbeMakeLabel($870)|0);
   HEAP32[$iLoopBody>>2] = $871;
   HEAP32[$untestedTerms>>2] = 0;
   HEAP32[$pAndExpr>>2] = 0;
   $872 = HEAP32[$pTabItem>>2]|0;
   $873 = (($872) + 16|0);
   $874 = HEAP32[$873>>2]|0;
   HEAP32[$pTab>>2] = $874;
   $875 = HEAP32[$pLoop>>2]|0;
   $876 = (($875) + 44|0);
   $877 = HEAP32[$876>>2]|0;
   $878 = HEAP32[$877>>2]|0;
   HEAP32[$pTerm>>2] = $878;
   $879 = HEAP32[$pTerm>>2]|0;
   $880 = (($879) + 12|0);
   $881 = HEAP32[$880>>2]|0;
   HEAP32[$pOrWc>>2] = $881;
   $882 = HEAP32[$pLevel>>2]|0;
   $883 = (($882) + 37|0);
   HEAP8[$883>>0] = 18;
   $884 = HEAP32[$regReturn>>2]|0;
   $885 = HEAP32[$pLevel>>2]|0;
   $886 = (($885) + 40|0);
   HEAP32[$886>>2] = $884;
   $887 = HEAP32[$3>>2]|0;
   $888 = (($887) + 41|0);
   $889 = HEAP8[$888>>0]|0;
   $890 = $889&255;
   $891 = ($890|0)>(1);
   $892 = HEAP32[$3>>2]|0;
   L159: do {
    if ($891) {
     $893 = (($892) + 41|0);
     $894 = HEAP8[$893>>0]|0;
     $895 = $894&255;
     $896 = HEAP32[$4>>2]|0;
     $897 = (($895) - ($896))|0;
     $898 = (($897) - 1)|0;
     HEAP32[$nNotReady>>2] = $898;
     $899 = HEAP32[$db>>2]|0;
     $900 = HEAP32[$nNotReady>>2]|0;
     $901 = ($900*72)|0;
     $902 = (80 + ($901))|0;
     $903 = (_sqlite3DbMallocRaw($899,$902,0)|0);
     HEAP32[$pOrTab>>2] = $903;
     $904 = HEAP32[$pOrTab>>2]|0;
     $905 = ($904|0)==(0|0);
     if ($905) {
      $906 = $5;
      $907 = $906;
      $908 = HEAP32[$907>>2]|0;
      $909 = (($906) + 4)|0;
      $910 = $909;
      $911 = HEAP32[$910>>2]|0;
      $912 = $2;
      $913 = $912;
      HEAP32[$913>>2] = $908;
      $914 = (($912) + 4)|0;
      $915 = $914;
      HEAP32[$915>>2] = $911;
      $1504 = $2;
      $1505 = $1504;
      $1506 = HEAP32[$1505>>2]|0;
      $1507 = (($1504) + 4)|0;
      $1508 = $1507;
      $1509 = HEAP32[$1508>>2]|0;
      tempRet0 = $1509;
      STACKTOP = sp;return ($1506|0);
     }
     $916 = HEAP32[$nNotReady>>2]|0;
     $917 = (($916) + 1)|0;
     $918 = $917&255;
     $919 = $918&255;
     $920 = HEAP32[$pOrTab>>2]|0;
     $921 = (($920) + 4|0);
     HEAP32[$921>>2] = $919;
     $922 = HEAP32[$pOrTab>>2]|0;
     $923 = (($922) + 4|0);
     $924 = HEAP32[$923>>2]|0;
     $925 = HEAP32[$pOrTab>>2]|0;
     HEAP32[$925>>2] = $924;
     $926 = HEAP32[$pOrTab>>2]|0;
     $927 = (($926) + 8|0);
     $928 = HEAP32[$pTabItem>>2]|0;
     dest=$927+0|0; src=$928+0|0; stop=dest+72|0; do { HEAP32[dest>>2]=HEAP32[src>>2]|0; dest=dest+4|0; src=src+4|0; } while ((dest|0) < (stop|0));
     $929 = HEAP32[$3>>2]|0;
     $930 = (($929) + 4|0);
     $931 = HEAP32[$930>>2]|0;
     $932 = (($931) + 8|0);
     HEAP32[$origSrc>>2] = $932;
     HEAP32[$k>>2] = 1;
     while(1) {
      $933 = HEAP32[$k>>2]|0;
      $934 = HEAP32[$nNotReady>>2]|0;
      $935 = ($933|0)<=($934|0);
      if (!($935)) {
       break L159;
      }
      $936 = HEAP32[$k>>2]|0;
      $937 = HEAP32[$pOrTab>>2]|0;
      $938 = (($937) + 8|0);
      $939 = (($938) + (($936*72)|0)|0);
      $940 = HEAP32[$k>>2]|0;
      $941 = HEAP32[$pLevel>>2]|0;
      $942 = (($941) + (($940*72)|0)|0);
      $943 = (($942) + 36|0);
      $944 = HEAP8[$943>>0]|0;
      $945 = $944&255;
      $946 = HEAP32[$origSrc>>2]|0;
      $947 = (($946) + (($945*72)|0)|0);
      dest=$939+0|0; src=$947+0|0; stop=dest+72|0; do { HEAP32[dest>>2]=HEAP32[src>>2]|0; dest=dest+4|0; src=src+4|0; } while ((dest|0) < (stop|0));
      $948 = HEAP32[$k>>2]|0;
      $949 = (($948) + 1)|0;
      HEAP32[$k>>2] = $949;
     }
    } else {
     $950 = (($892) + 4|0);
     $951 = HEAP32[$950>>2]|0;
     HEAP32[$pOrTab>>2] = $951;
    }
   } while(0);
   $952 = HEAP32[$3>>2]|0;
   $953 = (($952) + 34|0);
   $954 = HEAP16[$953>>1]|0;
   $955 = $954&65535;
   $956 = $955 & 8;
   $957 = ($956|0)==(0);
   if ($957) {
    $958 = HEAP32[$pTab>>2]|0;
    $959 = (($958) + 44|0);
    $960 = HEAP8[$959>>0]|0;
    $961 = $960&255;
    $962 = $961 & 32;
    $963 = ($962|0)==(0);
    if ($963) {
     $964 = HEAP32[$pParse>>2]|0;
     $965 = (($964) + 72|0);
     $966 = HEAP32[$965>>2]|0;
     $967 = (($966) + 1)|0;
     HEAP32[$965>>2] = $967;
     HEAP32[$regRowset>>2] = $967;
     $968 = HEAP32[$v>>2]|0;
     $969 = HEAP32[$regRowset>>2]|0;
     (_sqlite3VdbeAddOp2($968,28,0,$969)|0);
    } else {
     $970 = HEAP32[$pTab>>2]|0;
     $971 = (_sqlite3PrimaryKeyIndex($970)|0);
     HEAP32[$pPk5>>2] = $971;
     $972 = HEAP32[$pParse>>2]|0;
     $973 = (($972) + 68|0);
     $974 = HEAP32[$973>>2]|0;
     $975 = (($974) + 1)|0;
     HEAP32[$973>>2] = $975;
     HEAP32[$regRowset>>2] = $974;
     $976 = HEAP32[$v>>2]|0;
     $977 = HEAP32[$regRowset>>2]|0;
     $978 = HEAP32[$pPk5>>2]|0;
     $979 = (($978) + 50|0);
     $980 = HEAP16[$979>>1]|0;
     $981 = $980&65535;
     (_sqlite3VdbeAddOp2($976,57,$977,$981)|0);
     $982 = HEAP32[$pParse>>2]|0;
     $983 = HEAP32[$pPk5>>2]|0;
     _sqlite3VdbeSetP4KeyInfo($982,$983);
    }
    $984 = HEAP32[$pParse>>2]|0;
    $985 = (($984) + 72|0);
    $986 = HEAP32[$985>>2]|0;
    $987 = (($986) + 1)|0;
    HEAP32[$985>>2] = $987;
    HEAP32[$regRowid>>2] = $987;
   }
   $988 = HEAP32[$v>>2]|0;
   $989 = HEAP32[$regReturn>>2]|0;
   $990 = (_sqlite3VdbeAddOp2($988,25,0,$989)|0);
   HEAP32[$iRetInit>>2] = $990;
   $991 = HEAP32[$pWC>>2]|0;
   $992 = (($991) + 12|0);
   $993 = HEAP32[$992>>2]|0;
   $994 = ($993|0)>(1);
   if ($994) {
    HEAP32[$iTerm>>2] = 0;
    while(1) {
     $995 = HEAP32[$iTerm>>2]|0;
     $996 = HEAP32[$pWC>>2]|0;
     $997 = (($996) + 12|0);
     $998 = HEAP32[$997>>2]|0;
     $999 = ($995|0)<($998|0);
     if (!($999)) {
      break;
     }
     $1000 = HEAP32[$iTerm>>2]|0;
     $1001 = HEAP32[$pWC>>2]|0;
     $1002 = (($1001) + 20|0);
     $1003 = HEAP32[$1002>>2]|0;
     $1004 = (($1003) + (($1000*48)|0)|0);
     $1005 = HEAP32[$1004>>2]|0;
     HEAP32[$pExpr>>2] = $1005;
     $1006 = HEAP32[$iTerm>>2]|0;
     $1007 = HEAP32[$pWC>>2]|0;
     $1008 = (($1007) + 20|0);
     $1009 = HEAP32[$1008>>2]|0;
     $1010 = (($1009) + (($1006*48)|0)|0);
     $1011 = HEAP32[$pTerm>>2]|0;
     $1012 = ($1010|0)==($1011|0);
     if (!($1012)) {
      $1013 = HEAP32[$pExpr>>2]|0;
      $1014 = (($1013) + 4|0);
      $1015 = HEAP32[$1014>>2]|0;
      $1016 = $1015 & 1;
      $1017 = ($1016|0)!=(0);
      if (!($1017)) {
       $1018 = HEAP32[$iTerm>>2]|0;
       $1019 = HEAP32[$pWC>>2]|0;
       $1020 = (($1019) + 20|0);
       $1021 = HEAP32[$1020>>2]|0;
       $1022 = (($1021) + (($1018*48)|0)|0);
       $1023 = (($1022) + 20|0);
       $1024 = HEAP8[$1023>>0]|0;
       $1025 = $1024&255;
       $1026 = $1025 & 18;
       $1027 = ($1026|0)!=(0);
       if (!($1027)) {
        $1028 = HEAP32[$iTerm>>2]|0;
        $1029 = HEAP32[$pWC>>2]|0;
        $1030 = (($1029) + 20|0);
        $1031 = HEAP32[$1030>>2]|0;
        $1032 = (($1031) + (($1028*48)|0)|0);
        $1033 = (($1032) + 18|0);
        $1034 = HEAP16[$1033>>1]|0;
        $1035 = $1034&65535;
        $1036 = $1035 & 4095;
        $1037 = ($1036|0)==(0);
        if (!($1037)) {
         $1038 = HEAP32[$db>>2]|0;
         $1039 = HEAP32[$pExpr>>2]|0;
         $1040 = (_sqlite3ExprDup($1038,$1039,0)|0);
         HEAP32[$pExpr>>2] = $1040;
         $1041 = HEAP32[$db>>2]|0;
         $1042 = HEAP32[$pAndExpr>>2]|0;
         $1043 = HEAP32[$pExpr>>2]|0;
         $1044 = (_sqlite3ExprAnd($1041,$1042,$1043)|0);
         HEAP32[$pAndExpr>>2] = $1044;
        }
       }
      }
     }
     $1045 = HEAP32[$iTerm>>2]|0;
     $1046 = (($1045) + 1)|0;
     HEAP32[$iTerm>>2] = $1046;
    }
    $1047 = HEAP32[$pAndExpr>>2]|0;
    $1048 = ($1047|0)!=(0|0);
    if ($1048) {
     $1049 = HEAP32[$pParse>>2]|0;
     $1050 = HEAP32[$pAndExpr>>2]|0;
     $1051 = (_sqlite3PExpr($1049,72,0,$1050,0)|0);
     HEAP32[$pAndExpr>>2] = $1051;
    }
   }
   HEAP16[$wctrlFlags>>1] = 112;
   HEAP32[$ii>>2] = 0;
   while(1) {
    $1052 = HEAP32[$ii>>2]|0;
    $1053 = HEAP32[$pOrWc>>2]|0;
    $1054 = (($1053) + 12|0);
    $1055 = HEAP32[$1054>>2]|0;
    $1056 = ($1052|0)<($1055|0);
    if (!($1056)) {
     break;
    }
    $1057 = HEAP32[$ii>>2]|0;
    $1058 = HEAP32[$pOrWc>>2]|0;
    $1059 = (($1058) + 20|0);
    $1060 = HEAP32[$1059>>2]|0;
    $1061 = (($1060) + (($1057*48)|0)|0);
    HEAP32[$pOrTerm>>2] = $1061;
    $1062 = HEAP32[$pOrTerm>>2]|0;
    $1063 = (($1062) + 8|0);
    $1064 = HEAP32[$1063>>2]|0;
    $1065 = HEAP32[$iCur>>2]|0;
    $1066 = ($1064|0)==($1065|0);
    if ($1066) {
     label = 138;
    } else {
     $1067 = HEAP32[$pOrTerm>>2]|0;
     $1068 = (($1067) + 18|0);
     $1069 = HEAP16[$1068>>1]|0;
     $1070 = $1069&65535;
     $1071 = $1070 & 512;
     $1072 = ($1071|0)!=(0);
     if ($1072) {
      label = 138;
     }
    }
    if ((label|0) == 138) {
     label = 0;
     $1073 = HEAP32[$pOrTerm>>2]|0;
     $1074 = HEAP32[$1073>>2]|0;
     HEAP32[$pOrExpr>>2] = $1074;
     HEAP32[$j1>>2] = 0;
     $1075 = HEAP32[$pAndExpr>>2]|0;
     $1076 = ($1075|0)!=(0|0);
     if ($1076) {
      $1077 = HEAP32[$pOrExpr>>2]|0;
      $1078 = (($1077) + 4|0);
      $1079 = HEAP32[$1078>>2]|0;
      $1080 = $1079 & 1;
      $1081 = ($1080|0)!=(0);
      if (!($1081)) {
       $1082 = HEAP32[$pOrExpr>>2]|0;
       $1083 = HEAP32[$pAndExpr>>2]|0;
       $1084 = (($1083) + 12|0);
       HEAP32[$1084>>2] = $1082;
       $1085 = HEAP32[$pAndExpr>>2]|0;
       HEAP32[$pOrExpr>>2] = $1085;
      }
     }
     $1086 = HEAP32[$pParse>>2]|0;
     $1087 = HEAP32[$pOrTab>>2]|0;
     $1088 = HEAP32[$pOrExpr>>2]|0;
     $1089 = HEAP16[$wctrlFlags>>1]|0;
     $1090 = HEAP32[$iCovCur>>2]|0;
     $1091 = (_sqlite3WhereBegin($1086,$1087,$1088,0,0,$1089,$1090)|0);
     HEAP32[$pSubWInfo>>2] = $1091;
     $1092 = HEAP32[$pSubWInfo>>2]|0;
     $1093 = ($1092|0)!=(0|0);
     if ($1093) {
      $1094 = HEAP32[$pParse>>2]|0;
      $1095 = HEAP32[$pOrTab>>2]|0;
      $1096 = HEAP32[$pSubWInfo>>2]|0;
      $1097 = (($1096) + 736|0);
      $1098 = HEAP32[$4>>2]|0;
      $1099 = HEAP32[$pLevel>>2]|0;
      $1100 = (($1099) + 36|0);
      $1101 = HEAP8[$1100>>0]|0;
      $1102 = $1101&255;
      _explainOneScan($1094,$1095,$1097,$1098,$1102,0);
      $1103 = HEAP32[$3>>2]|0;
      $1104 = (($1103) + 34|0);
      $1105 = HEAP16[$1104>>1]|0;
      $1106 = $1105&65535;
      $1107 = $1106 & 8;
      $1108 = ($1107|0)==(0);
      do {
       if ($1108) {
        $1109 = HEAP32[$ii>>2]|0;
        $1110 = HEAP32[$pOrWc>>2]|0;
        $1111 = (($1110) + 12|0);
        $1112 = HEAP32[$1111>>2]|0;
        $1113 = (($1112) - 1)|0;
        $1114 = ($1109|0)==($1113|0);
        $1115 = HEAP32[$ii>>2]|0;
        $1116 = $1114 ? -1 : $1115;
        HEAP32[$iSet>>2] = $1116;
        $1117 = HEAP32[$pTab>>2]|0;
        $1118 = (($1117) + 44|0);
        $1119 = HEAP8[$1118>>0]|0;
        $1120 = $1119&255;
        $1121 = $1120 & 32;
        $1122 = ($1121|0)==(0);
        if ($1122) {
         $1123 = HEAP32[$pParse>>2]|0;
         $1124 = HEAP32[$pTab>>2]|0;
         $1125 = HEAP32[$iCur>>2]|0;
         $1126 = HEAP32[$regRowid>>2]|0;
         $1127 = (_sqlite3ExprCodeGetColumn($1123,$1124,-1,$1125,$1126,0)|0);
         HEAP32[$r>>2] = $1127;
         $1128 = HEAP32[$v>>2]|0;
         $1129 = HEAP32[$regRowset>>2]|0;
         $1130 = HEAP32[$r>>2]|0;
         $1131 = HEAP32[$iSet>>2]|0;
         $1132 = (_sqlite3VdbeAddOp4Int($1128,130,$1129,0,$1130,$1131)|0);
         HEAP32[$j1>>2] = $1132;
         break;
        }
        $1133 = HEAP32[$pTab>>2]|0;
        $1134 = (_sqlite3PrimaryKeyIndex($1133)|0);
        HEAP32[$pPk6>>2] = $1134;
        $1135 = HEAP32[$pPk6>>2]|0;
        $1136 = (($1135) + 50|0);
        $1137 = HEAP16[$1136>>1]|0;
        $1138 = $1137&65535;
        HEAP32[$nPk>>2] = $1138;
        $1139 = HEAP32[$pParse>>2]|0;
        $1140 = HEAP32[$nPk>>2]|0;
        $1141 = (_sqlite3GetTempRange($1139,$1140)|0);
        HEAP32[$r>>2] = $1141;
        HEAP32[$iPk>>2] = 0;
        while(1) {
         $1142 = HEAP32[$iPk>>2]|0;
         $1143 = HEAP32[$nPk>>2]|0;
         $1144 = ($1142|0)<($1143|0);
         if (!($1144)) {
          break;
         }
         $1145 = HEAP32[$iPk>>2]|0;
         $1146 = HEAP32[$pPk6>>2]|0;
         $1147 = (($1146) + 4|0);
         $1148 = HEAP32[$1147>>2]|0;
         $1149 = (($1148) + ($1145<<1)|0);
         $1150 = HEAP16[$1149>>1]|0;
         $1151 = $1150 << 16 >> 16;
         HEAP32[$iCol>>2] = $1151;
         $1152 = HEAP32[$pParse>>2]|0;
         $1153 = HEAP32[$pTab>>2]|0;
         $1154 = HEAP32[$iCol>>2]|0;
         $1155 = HEAP32[$iCur>>2]|0;
         $1156 = HEAP32[$r>>2]|0;
         $1157 = HEAP32[$iPk>>2]|0;
         $1158 = (($1156) + ($1157))|0;
         (_sqlite3ExprCodeGetColumn($1152,$1153,$1154,$1155,$1158,0)|0);
         $1159 = HEAP32[$iPk>>2]|0;
         $1160 = (($1159) + 1)|0;
         HEAP32[$iPk>>2] = $1160;
        }
        $1161 = HEAP32[$iSet>>2]|0;
        $1162 = ($1161|0)!=(0);
        if ($1162) {
         $1163 = HEAP32[$v>>2]|0;
         $1164 = HEAP32[$regRowset>>2]|0;
         $1165 = HEAP32[$r>>2]|0;
         $1166 = HEAP32[$nPk>>2]|0;
         $1167 = (_sqlite3VdbeAddOp4Int($1163,69,$1164,0,$1165,$1166)|0);
         HEAP32[$j1>>2] = $1167;
        }
        $1168 = HEAP32[$iSet>>2]|0;
        $1169 = ($1168|0)>=(0);
        do {
         if ($1169) {
          $1170 = HEAP32[$v>>2]|0;
          $1171 = HEAP32[$r>>2]|0;
          $1172 = HEAP32[$nPk>>2]|0;
          $1173 = HEAP32[$regRowid>>2]|0;
          (_sqlite3VdbeAddOp3($1170,49,$1171,$1172,$1173)|0);
          $1174 = HEAP32[$v>>2]|0;
          $1175 = HEAP32[$regRowset>>2]|0;
          $1176 = HEAP32[$regRowid>>2]|0;
          (_sqlite3VdbeAddOp3($1174,110,$1175,$1176,0)|0);
          $1177 = HEAP32[$iSet>>2]|0;
          $1178 = ($1177|0)!=(0);
          if (!($1178)) {
           break;
          }
          $1179 = HEAP32[$v>>2]|0;
          _sqlite3VdbeChangeP5($1179,16);
         }
        } while(0);
        $1180 = HEAP32[$pParse>>2]|0;
        $1181 = HEAP32[$r>>2]|0;
        $1182 = HEAP32[$nPk>>2]|0;
        _sqlite3ReleaseTempRange($1180,$1181,$1182);
       }
      } while(0);
      $1183 = HEAP32[$v>>2]|0;
      $1184 = HEAP32[$regReturn>>2]|0;
      $1185 = HEAP32[$iLoopBody>>2]|0;
      (_sqlite3VdbeAddOp2($1183,17,$1184,$1185)|0);
      $1186 = HEAP32[$j1>>2]|0;
      $1187 = ($1186|0)!=(0);
      if ($1187) {
       $1188 = HEAP32[$v>>2]|0;
       $1189 = HEAP32[$j1>>2]|0;
       _sqlite3VdbeJumpHere($1188,$1189);
      }
      $1190 = HEAP32[$pSubWInfo>>2]|0;
      $1191 = (($1190) + 39|0);
      $1192 = HEAP8[$1191>>0]|0;
      $1193 = ($1192<<24>>24)!=(0);
      if ($1193) {
       HEAP32[$untestedTerms>>2] = 1;
      }
      $1194 = HEAP32[$pSubWInfo>>2]|0;
      $1195 = (($1194) + 736|0);
      $1196 = (($1195) + 56|0);
      $1197 = HEAP32[$1196>>2]|0;
      HEAP32[$pSubLoop>>2] = $1197;
      $1198 = HEAP32[$pSubLoop>>2]|0;
      $1199 = (($1198) + 36|0);
      $1200 = HEAP32[$1199>>2]|0;
      $1201 = $1200 & 512;
      $1202 = ($1201|0)!=(0);
      do {
       if ($1202) {
        $1203 = HEAP32[$ii>>2]|0;
        $1204 = ($1203|0)==(0);
        if (!($1204)) {
         $1205 = HEAP32[$pSubLoop>>2]|0;
         $1206 = (($1205) + 24|0);
         $1207 = (($1206) + 4|0);
         $1208 = HEAP32[$1207>>2]|0;
         $1209 = HEAP32[$pCov>>2]|0;
         $1210 = ($1208|0)==($1209|0);
         if (!($1210)) {
          label = 164;
          break;
         }
        }
        $1211 = HEAP32[$pTab>>2]|0;
        $1212 = (($1211) + 44|0);
        $1213 = HEAP8[$1212>>0]|0;
        $1214 = $1213&255;
        $1215 = $1214 & 32;
        $1216 = ($1215|0)==(0);
        if (!($1216)) {
         $1217 = HEAP32[$pSubLoop>>2]|0;
         $1218 = (($1217) + 24|0);
         $1219 = (($1218) + 4|0);
         $1220 = HEAP32[$1219>>2]|0;
         $1221 = (($1220) + 55|0);
         $1222 = HEAP8[$1221>>0]|0;
         $1223 = $1222 & 3;
         $1224 = $1223&255;
         $1225 = ($1224|0)==(2);
         if ($1225) {
          label = 164;
          break;
         }
        }
        $1226 = HEAP32[$pSubLoop>>2]|0;
        $1227 = (($1226) + 24|0);
        $1228 = (($1227) + 4|0);
        $1229 = HEAP32[$1228>>2]|0;
        HEAP32[$pCov>>2] = $1229;
        $1230 = HEAP16[$wctrlFlags>>1]|0;
        $1231 = $1230&65535;
        $1232 = $1231 | 4096;
        $1233 = $1232&65535;
        HEAP16[$wctrlFlags>>1] = $1233;
       } else {
        label = 164;
       }
      } while(0);
      if ((label|0) == 164) {
       label = 0;
       HEAP32[$pCov>>2] = 0;
      }
      $1234 = HEAP32[$pSubWInfo>>2]|0;
      _sqlite3WhereEnd($1234);
     }
    }
    $1235 = HEAP32[$ii>>2]|0;
    $1236 = (($1235) + 1)|0;
    HEAP32[$ii>>2] = $1236;
   }
   $1237 = HEAP32[$pCov>>2]|0;
   $1238 = HEAP32[$pLevel>>2]|0;
   $1239 = (($1238) + 48|0);
   HEAP32[$1239>>2] = $1237;
   $1240 = HEAP32[$pCov>>2]|0;
   $1241 = ($1240|0)!=(0|0);
   if ($1241) {
    $1242 = HEAP32[$iCovCur>>2]|0;
    $1243 = HEAP32[$pLevel>>2]|0;
    $1244 = (($1243) + 8|0);
    HEAP32[$1244>>2] = $1242;
   }
   $1245 = HEAP32[$pAndExpr>>2]|0;
   $1246 = ($1245|0)!=(0|0);
   if ($1246) {
    $1247 = HEAP32[$pAndExpr>>2]|0;
    $1248 = (($1247) + 12|0);
    HEAP32[$1248>>2] = 0;
    $1249 = HEAP32[$db>>2]|0;
    $1250 = HEAP32[$pAndExpr>>2]|0;
    _sqlite3ExprDelete($1249,$1250);
   }
   $1251 = HEAP32[$v>>2]|0;
   $1252 = HEAP32[$iRetInit>>2]|0;
   $1253 = HEAP32[$v>>2]|0;
   $1254 = (_sqlite3VdbeCurrentAddr($1253)|0);
   _sqlite3VdbeChangeP1($1251,$1252,$1254);
   $1255 = HEAP32[$v>>2]|0;
   $1256 = HEAP32[$pLevel>>2]|0;
   $1257 = (($1256) + 12|0);
   $1258 = HEAP32[$1257>>2]|0;
   (_sqlite3VdbeAddOp2($1255,16,0,$1258)|0);
   $1259 = HEAP32[$v>>2]|0;
   $1260 = HEAP32[$iLoopBody>>2]|0;
   _sqlite3VdbeResolveLabel($1259,$1260);
   $1261 = HEAP32[$3>>2]|0;
   $1262 = (($1261) + 41|0);
   $1263 = HEAP8[$1262>>0]|0;
   $1264 = $1263&255;
   $1265 = ($1264|0)>(1);
   if ($1265) {
    $1266 = HEAP32[$db>>2]|0;
    $1267 = HEAP32[$pOrTab>>2]|0;
    _sqlite3DbFree($1266,$1267);
   }
   $1268 = HEAP32[$untestedTerms>>2]|0;
   $1269 = ($1268|0)!=(0);
   if (!($1269)) {
    $1270 = HEAP32[$pLevel>>2]|0;
    $1271 = HEAP32[$pTerm>>2]|0;
    _disableTerm($1270,$1271);
   }
  }
 } while(0);
 $1302 = HEAP32[$pWC>>2]|0;
 $1303 = (($1302) + 20|0);
 $1304 = HEAP32[$1303>>2]|0;
 HEAP32[$pTerm>>2] = $1304;
 $1305 = HEAP32[$pWC>>2]|0;
 $1306 = (($1305) + 12|0);
 $1307 = HEAP32[$1306>>2]|0;
 HEAP32[$j>>2] = $1307;
 while(1) {
  $1308 = HEAP32[$j>>2]|0;
  $1309 = ($1308|0)>(0);
  if (!($1309)) {
   break;
  }
  $1310 = HEAP32[$pTerm>>2]|0;
  $1311 = (($1310) + 20|0);
  $1312 = HEAP8[$1311>>0]|0;
  $1313 = $1312&255;
  $1314 = $1313 & 6;
  $1315 = ($1314|0)!=(0);
  do {
   if (!($1315)) {
    $1316 = HEAP32[$pTerm>>2]|0;
    $1317 = (($1316) + 40|0);
    $1318 = $1317;
    $1319 = $1318;
    $1320 = HEAP32[$1319>>2]|0;
    $1321 = (($1318) + 4)|0;
    $1322 = $1321;
    $1323 = HEAP32[$1322>>2]|0;
    $1324 = HEAP32[$pLevel>>2]|0;
    $1325 = (($1324) + 64|0);
    $1326 = $1325;
    $1327 = $1326;
    $1328 = HEAP32[$1327>>2]|0;
    $1329 = (($1326) + 4)|0;
    $1330 = $1329;
    $1331 = HEAP32[$1330>>2]|0;
    $1332 = $1320 & $1328;
    $1333 = $1323 & $1331;
    $1334 = ($1332|0)!=(0);
    $1335 = ($1333|0)!=(0);
    $1336 = $1334 | $1335;
    if ($1336) {
     $1337 = HEAP32[$3>>2]|0;
     $1338 = (($1337) + 39|0);
     HEAP8[$1338>>0] = 1;
     break;
    }
    $1339 = HEAP32[$pTerm>>2]|0;
    $1340 = HEAP32[$1339>>2]|0;
    HEAP32[$pE>>2] = $1340;
    $1341 = HEAP32[$pLevel>>2]|0;
    $1342 = HEAP32[$1341>>2]|0;
    $1343 = ($1342|0)!=(0);
    if ($1343) {
     $1344 = HEAP32[$pE>>2]|0;
     $1345 = (($1344) + 4|0);
     $1346 = HEAP32[$1345>>2]|0;
     $1347 = $1346 & 1;
     $1348 = ($1347|0)!=(0);
     if (!($1348)) {
      break;
     }
    }
    $1349 = HEAP32[$pParse>>2]|0;
    $1350 = HEAP32[$pE>>2]|0;
    $1351 = HEAP32[$addrCont>>2]|0;
    _sqlite3ExprIfFalse($1349,$1350,$1351,16);
    $1352 = HEAP32[$pTerm>>2]|0;
    $1353 = (($1352) + 20|0);
    $1354 = HEAP8[$1353>>0]|0;
    $1355 = $1354&255;
    $1356 = $1355 | 4;
    $1357 = $1356&255;
    HEAP8[$1353>>0] = $1357;
   }
  } while(0);
  $1358 = HEAP32[$j>>2]|0;
  $1359 = (($1358) + -1)|0;
  HEAP32[$j>>2] = $1359;
  $1360 = HEAP32[$pTerm>>2]|0;
  $1361 = (($1360) + 48|0);
  HEAP32[$pTerm>>2] = $1361;
 }
 $1362 = HEAP32[$pWC>>2]|0;
 $1363 = (($1362) + 20|0);
 $1364 = HEAP32[$1363>>2]|0;
 HEAP32[$pTerm>>2] = $1364;
 $1365 = HEAP32[$pWC>>2]|0;
 $1366 = (($1365) + 12|0);
 $1367 = HEAP32[$1366>>2]|0;
 HEAP32[$j>>2] = $1367;
 while(1) {
  $1368 = HEAP32[$j>>2]|0;
  $1369 = ($1368|0)>(0);
  if (!($1369)) {
   break;
  }
  $1370 = HEAP32[$pTerm>>2]|0;
  $1371 = (($1370) + 20|0);
  $1372 = HEAP8[$1371>>0]|0;
  $1373 = $1372&255;
  $1374 = $1373 & 6;
  $1375 = ($1374|0)!=(0);
  if (!($1375)) {
   $1376 = HEAP32[$pTerm>>2]|0;
   $1377 = (($1376) + 18|0);
   $1378 = HEAP16[$1377>>1]|0;
   $1379 = $1378&65535;
   $1380 = ($1379|0)!=(1026);
   if (!($1380)) {
    $1381 = HEAP32[$pTerm>>2]|0;
    $1382 = (($1381) + 8|0);
    $1383 = HEAP32[$1382>>2]|0;
    $1384 = HEAP32[$iCur>>2]|0;
    $1385 = ($1383|0)!=($1384|0);
    if (!($1385)) {
     $1386 = HEAP32[$pLevel>>2]|0;
     $1387 = HEAP32[$1386>>2]|0;
     $1388 = ($1387|0)!=(0);
     if (!($1388)) {
      $1389 = HEAP32[$pTerm>>2]|0;
      $1390 = HEAP32[$1389>>2]|0;
      HEAP32[$pE7>>2] = $1390;
      $1391 = HEAP32[$pWC>>2]|0;
      $1392 = HEAP32[$iCur>>2]|0;
      $1393 = HEAP32[$pTerm>>2]|0;
      $1394 = (($1393) + 12|0);
      $1395 = HEAP32[$1394>>2]|0;
      $1396 = $5;
      $1397 = $1396;
      $1398 = HEAP32[$1397>>2]|0;
      $1399 = (($1396) + 4)|0;
      $1400 = $1399;
      $1401 = HEAP32[$1400>>2]|0;
      $1402 = (_findTerm($1391,$1392,$1395,$1398,$1401,3,0)|0);
      HEAP32[$pAlt>>2] = $1402;
      $1403 = HEAP32[$pAlt>>2]|0;
      $1404 = ($1403|0)==(0|0);
      if (!($1404)) {
       $1405 = HEAP32[$pAlt>>2]|0;
       $1406 = (($1405) + 20|0);
       $1407 = HEAP8[$1406>>0]|0;
       $1408 = $1407&255;
       $1409 = $1408 & 4;
       $1410 = ($1409|0)!=(0);
       if (!($1410)) {
        $1411 = HEAP32[$db>>2]|0;
        $1412 = (_sqlite3DbMallocRaw($1411,48,0)|0);
        HEAP32[$pEAlt>>2] = $1412;
        $1413 = HEAP32[$pEAlt>>2]|0;
        $1414 = ($1413|0)!=(0|0);
        if ($1414) {
         $1415 = HEAP32[$pEAlt>>2]|0;
         $1416 = HEAP32[$pAlt>>2]|0;
         $1417 = HEAP32[$1416>>2]|0;
         dest=$1415+0|0; src=$1417+0|0; stop=dest+48|0; do { HEAP32[dest>>2]=HEAP32[src>>2]|0; dest=dest+4|0; src=src+4|0; } while ((dest|0) < (stop|0));
         $1418 = HEAP32[$pE7>>2]|0;
         $1419 = (($1418) + 12|0);
         $1420 = HEAP32[$1419>>2]|0;
         $1421 = HEAP32[$pEAlt>>2]|0;
         $1422 = (($1421) + 12|0);
         HEAP32[$1422>>2] = $1420;
         $1423 = HEAP32[$pParse>>2]|0;
         $1424 = HEAP32[$pEAlt>>2]|0;
         $1425 = HEAP32[$addrCont>>2]|0;
         _sqlite3ExprIfFalse($1423,$1424,$1425,16);
         $1426 = HEAP32[$db>>2]|0;
         $1427 = HEAP32[$pEAlt>>2]|0;
         _sqlite3DbFree($1426,$1427);
        }
       }
      }
     }
    }
   }
  }
  $1428 = HEAP32[$j>>2]|0;
  $1429 = (($1428) + -1)|0;
  HEAP32[$j>>2] = $1429;
  $1430 = HEAP32[$pTerm>>2]|0;
  $1431 = (($1430) + 48|0);
  HEAP32[$pTerm>>2] = $1431;
 }
 $1432 = HEAP32[$pLevel>>2]|0;
 $1433 = HEAP32[$1432>>2]|0;
 $1434 = ($1433|0)!=(0);
 L276: do {
  if ($1434) {
   $1435 = HEAP32[$v>>2]|0;
   $1436 = (_sqlite3VdbeCurrentAddr($1435)|0);
   $1437 = HEAP32[$pLevel>>2]|0;
   $1438 = (($1437) + 28|0);
   HEAP32[$1438>>2] = $1436;
   $1439 = HEAP32[$v>>2]|0;
   $1440 = HEAP32[$pLevel>>2]|0;
   $1441 = HEAP32[$1440>>2]|0;
   (_sqlite3VdbeAddOp2($1439,25,1,$1441)|0);
   $1442 = HEAP32[$pParse>>2]|0;
   _sqlite3ExprCacheClear($1442);
   $1443 = HEAP32[$pWC>>2]|0;
   $1444 = (($1443) + 20|0);
   $1445 = HEAP32[$1444>>2]|0;
   HEAP32[$pTerm>>2] = $1445;
   HEAP32[$j>>2] = 0;
   while(1) {
    $1446 = HEAP32[$j>>2]|0;
    $1447 = HEAP32[$pWC>>2]|0;
    $1448 = (($1447) + 12|0);
    $1449 = HEAP32[$1448>>2]|0;
    $1450 = ($1446|0)<($1449|0);
    if (!($1450)) {
     break L276;
    }
    $1451 = HEAP32[$pTerm>>2]|0;
    $1452 = (($1451) + 20|0);
    $1453 = HEAP8[$1452>>0]|0;
    $1454 = $1453&255;
    $1455 = $1454 & 6;
    $1456 = ($1455|0)!=(0);
    if (!($1456)) {
     $1457 = HEAP32[$pTerm>>2]|0;
     $1458 = (($1457) + 40|0);
     $1459 = $1458;
     $1460 = $1459;
     $1461 = HEAP32[$1460>>2]|0;
     $1462 = (($1459) + 4)|0;
     $1463 = $1462;
     $1464 = HEAP32[$1463>>2]|0;
     $1465 = HEAP32[$pLevel>>2]|0;
     $1466 = (($1465) + 64|0);
     $1467 = $1466;
     $1468 = $1467;
     $1469 = HEAP32[$1468>>2]|0;
     $1470 = (($1467) + 4)|0;
     $1471 = $1470;
     $1472 = HEAP32[$1471>>2]|0;
     $1473 = $1461 & $1469;
     $1474 = $1464 & $1472;
     $1475 = ($1473|0)!=(0);
     $1476 = ($1474|0)!=(0);
     $1477 = $1475 | $1476;
     if (!($1477)) {
      $1478 = HEAP32[$pParse>>2]|0;
      $1479 = HEAP32[$pTerm>>2]|0;
      $1480 = HEAP32[$1479>>2]|0;
      $1481 = HEAP32[$addrCont>>2]|0;
      _sqlite3ExprIfFalse($1478,$1480,$1481,16);
      $1482 = HEAP32[$pTerm>>2]|0;
      $1483 = (($1482) + 20|0);
      $1484 = HEAP8[$1483>>0]|0;
      $1485 = $1484&255;
      $1486 = $1485 | 4;
      $1487 = $1486&255;
      HEAP8[$1483>>0] = $1487;
     }
    }
    $1488 = HEAP32[$j>>2]|0;
    $1489 = (($1488) + 1)|0;
    HEAP32[$j>>2] = $1489;
    $1490 = HEAP32[$pTerm>>2]|0;
    $1491 = (($1490) + 48|0);
    HEAP32[$pTerm>>2] = $1491;
   }
  }
 } while(0);
 $1492 = HEAP32[$pLevel>>2]|0;
 $1493 = (($1492) + 64|0);
 $1494 = $1493;
 $1495 = $1494;
 $1496 = HEAP32[$1495>>2]|0;
 $1497 = (($1494) + 4)|0;
 $1498 = $1497;
 $1499 = HEAP32[$1498>>2]|0;
 $1500 = $2;
 $1501 = $1500;
 HEAP32[$1501>>2] = $1496;
 $1502 = (($1500) + 4)|0;
 $1503 = $1502;
 HEAP32[$1503>>2] = $1499;
 $1504 = $2;
 $1505 = $1504;
 $1506 = HEAP32[$1505>>2]|0;
 $1507 = (($1504) + 4)|0;
 $1508 = $1507;
 $1509 = HEAP32[$1508>>2]|0;
 tempRet0 = $1509;
 STACKTOP = sp;return ($1506|0);
}
function _codeEqualityTerm($pParse,$pTerm,$pLevel,$iEq,$bRev,$iTarget) {
 $pParse = $pParse|0;
 $pTerm = $pTerm|0;
 $pLevel = $pLevel|0;
 $iEq = $iEq|0;
 $bRev = $bRev|0;
 $iTarget = $iTarget|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0;
 var $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0;
 var $eType = 0, $iReg = 0, $iTab = 0, $pIn = 0, $pLoop = 0, $pX = 0, $v = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 64|0;
 $0 = sp + 48|0;
 $1 = sp + 44|0;
 $2 = sp + 40|0;
 $3 = sp + 36|0;
 $4 = sp + 32|0;
 $5 = sp + 28|0;
 $pX = sp + 24|0;
 $v = sp + 20|0;
 $iReg = sp + 16|0;
 $eType = sp + 12|0;
 $iTab = sp + 8|0;
 $pIn = sp + 4|0;
 $pLoop = sp;
 HEAP32[$0>>2] = $pParse;
 HEAP32[$1>>2] = $pTerm;
 HEAP32[$2>>2] = $pLevel;
 HEAP32[$3>>2] = $iEq;
 HEAP32[$4>>2] = $bRev;
 HEAP32[$5>>2] = $iTarget;
 $6 = HEAP32[$1>>2]|0;
 $7 = HEAP32[$6>>2]|0;
 HEAP32[$pX>>2] = $7;
 $8 = HEAP32[$0>>2]|0;
 $9 = (($8) + 8|0);
 $10 = HEAP32[$9>>2]|0;
 HEAP32[$v>>2] = $10;
 $11 = HEAP32[$pX>>2]|0;
 $12 = HEAP8[$11>>0]|0;
 $13 = $12&255;
 $14 = ($13|0)==(79);
 if ($14) {
  $15 = HEAP32[$0>>2]|0;
  $16 = HEAP32[$pX>>2]|0;
  $17 = (($16) + 16|0);
  $18 = HEAP32[$17>>2]|0;
  $19 = HEAP32[$5>>2]|0;
  $20 = (_sqlite3ExprCodeTarget($15,$18,$19)|0);
  HEAP32[$iReg>>2] = $20;
  $137 = HEAP32[$2>>2]|0;
  $138 = HEAP32[$1>>2]|0;
  _disableTerm($137,$138);
  $139 = HEAP32[$iReg>>2]|0;
  STACKTOP = sp;return ($139|0);
 }
 $21 = HEAP32[$pX>>2]|0;
 $22 = HEAP8[$21>>0]|0;
 $23 = $22&255;
 $24 = ($23|0)==(76);
 if ($24) {
  $25 = HEAP32[$5>>2]|0;
  HEAP32[$iReg>>2] = $25;
  $26 = HEAP32[$v>>2]|0;
  $27 = HEAP32[$iReg>>2]|0;
  (_sqlite3VdbeAddOp2($26,28,0,$27)|0);
  $137 = HEAP32[$2>>2]|0;
  $138 = HEAP32[$1>>2]|0;
  _disableTerm($137,$138);
  $139 = HEAP32[$iReg>>2]|0;
  STACKTOP = sp;return ($139|0);
 }
 $28 = HEAP32[$2>>2]|0;
 $29 = (($28) + 56|0);
 $30 = HEAP32[$29>>2]|0;
 HEAP32[$pLoop>>2] = $30;
 $31 = HEAP32[$pLoop>>2]|0;
 $32 = (($31) + 36|0);
 $33 = HEAP32[$32>>2]|0;
 $34 = $33 & 1024;
 $35 = ($34|0)==(0);
 if ($35) {
  $36 = HEAP32[$pLoop>>2]|0;
  $37 = (($36) + 24|0);
  $38 = (($37) + 4|0);
  $39 = HEAP32[$38>>2]|0;
  $40 = ($39|0)!=(0|0);
  if ($40) {
   $41 = HEAP32[$3>>2]|0;
   $42 = HEAP32[$pLoop>>2]|0;
   $43 = (($42) + 24|0);
   $44 = (($43) + 4|0);
   $45 = HEAP32[$44>>2]|0;
   $46 = (($45) + 28|0);
   $47 = HEAP32[$46>>2]|0;
   $48 = (($47) + ($41)|0);
   $49 = HEAP8[$48>>0]|0;
   $50 = $49&255;
   $51 = ($50|0)!=(0);
   if ($51) {
    $52 = HEAP32[$4>>2]|0;
    $53 = ($52|0)!=(0);
    $54 = $53 ^ 1;
    $55 = $54&1;
    HEAP32[$4>>2] = $55;
   }
  }
 }
 $56 = HEAP32[$5>>2]|0;
 HEAP32[$iReg>>2] = $56;
 $57 = HEAP32[$0>>2]|0;
 $58 = HEAP32[$pX>>2]|0;
 $59 = (_sqlite3FindInIndex($57,$58,4,0)|0);
 HEAP32[$eType>>2] = $59;
 $60 = HEAP32[$eType>>2]|0;
 $61 = ($60|0)==(4);
 if ($61) {
  $62 = HEAP32[$4>>2]|0;
  $63 = ($62|0)!=(0);
  $64 = $63 ^ 1;
  $65 = $64&1;
  HEAP32[$4>>2] = $65;
 }
 $66 = HEAP32[$pX>>2]|0;
 $67 = (($66) + 28|0);
 $68 = HEAP32[$67>>2]|0;
 HEAP32[$iTab>>2] = $68;
 $69 = HEAP32[$v>>2]|0;
 $70 = HEAP32[$4>>2]|0;
 $71 = ($70|0)!=(0);
 $72 = $71 ? 105 : 108;
 $73 = HEAP32[$iTab>>2]|0;
 (_sqlite3VdbeAddOp2($69,$72,$73,0)|0);
 $74 = HEAP32[$pLoop>>2]|0;
 $75 = (($74) + 36|0);
 $76 = HEAP32[$75>>2]|0;
 $77 = $76 | 2048;
 HEAP32[$75>>2] = $77;
 $78 = HEAP32[$2>>2]|0;
 $79 = (($78) + 48|0);
 $80 = HEAP32[$79>>2]|0;
 $81 = ($80|0)==(0);
 if ($81) {
  $82 = HEAP32[$v>>2]|0;
  $83 = (_sqlite3VdbeMakeLabel($82)|0);
  $84 = HEAP32[$2>>2]|0;
  $85 = (($84) + 16|0);
  HEAP32[$85>>2] = $83;
 }
 $86 = HEAP32[$2>>2]|0;
 $87 = (($86) + 48|0);
 $88 = HEAP32[$87>>2]|0;
 $89 = (($88) + 1)|0;
 HEAP32[$87>>2] = $89;
 $90 = HEAP32[$0>>2]|0;
 $91 = HEAP32[$90>>2]|0;
 $92 = HEAP32[$2>>2]|0;
 $93 = (($92) + 48|0);
 $94 = (($93) + 4|0);
 $95 = HEAP32[$94>>2]|0;
 $96 = HEAP32[$2>>2]|0;
 $97 = (($96) + 48|0);
 $98 = HEAP32[$97>>2]|0;
 $99 = ($98*12)|0;
 $100 = (_sqlite3DbReallocOrFree($91,$95,$99,0)|0);
 $101 = HEAP32[$2>>2]|0;
 $102 = (($101) + 48|0);
 $103 = (($102) + 4|0);
 HEAP32[$103>>2] = $100;
 $104 = HEAP32[$2>>2]|0;
 $105 = (($104) + 48|0);
 $106 = (($105) + 4|0);
 $107 = HEAP32[$106>>2]|0;
 HEAP32[$pIn>>2] = $107;
 $108 = HEAP32[$pIn>>2]|0;
 $109 = ($108|0)!=(0|0);
 $110 = HEAP32[$2>>2]|0;
 $111 = (($110) + 48|0);
 if (!($109)) {
  HEAP32[$111>>2] = 0;
  $137 = HEAP32[$2>>2]|0;
  $138 = HEAP32[$1>>2]|0;
  _disableTerm($137,$138);
  $139 = HEAP32[$iReg>>2]|0;
  STACKTOP = sp;return ($139|0);
 }
 $112 = HEAP32[$111>>2]|0;
 $113 = (($112) - 1)|0;
 $114 = HEAP32[$pIn>>2]|0;
 $115 = (($114) + (($113*12)|0)|0);
 HEAP32[$pIn>>2] = $115;
 $116 = HEAP32[$iTab>>2]|0;
 $117 = HEAP32[$pIn>>2]|0;
 HEAP32[$117>>2] = $116;
 $118 = HEAP32[$eType>>2]|0;
 $119 = ($118|0)==(1);
 $120 = HEAP32[$v>>2]|0;
 $121 = HEAP32[$iTab>>2]|0;
 $122 = HEAP32[$iReg>>2]|0;
 if ($119) {
  $123 = (_sqlite3VdbeAddOp2($120,103,$121,$122)|0);
  $124 = HEAP32[$pIn>>2]|0;
  $125 = (($124) + 4|0);
  HEAP32[$125>>2] = $123;
 } else {
  $126 = (_sqlite3VdbeAddOp3($120,47,$121,0,$122)|0);
  $127 = HEAP32[$pIn>>2]|0;
  $128 = (($127) + 4|0);
  HEAP32[$128>>2] = $126;
 }
 $129 = HEAP32[$4>>2]|0;
 $130 = ($129|0)!=(0);
 $131 = $130 ? 6 : 7;
 $132 = $131&255;
 $133 = HEAP32[$pIn>>2]|0;
 $134 = (($133) + 8|0);
 HEAP8[$134>>0] = $132;
 $135 = HEAP32[$v>>2]|0;
 $136 = HEAP32[$iReg>>2]|0;
 (_sqlite3VdbeAddOp1($135,76,$136)|0);
 $137 = HEAP32[$2>>2]|0;
 $138 = HEAP32[$1>>2]|0;
 _disableTerm($137,$138);
 $139 = HEAP32[$iReg>>2]|0;
 STACKTOP = sp;return ($139|0);
}
function _disableTerm($pLevel,$pTerm) {
 $pLevel = $pLevel|0;
 $pTerm = $pTerm|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $7 = 0, $8 = 0, $9 = 0, $pOther = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $1 = sp + 4|0;
 $pOther = sp;
 HEAP32[$0>>2] = $pLevel;
 HEAP32[$1>>2] = $pTerm;
 $2 = HEAP32[$1>>2]|0;
 $3 = ($2|0)!=(0|0);
 if (!($3)) {
  STACKTOP = sp;return;
 }
 $4 = HEAP32[$1>>2]|0;
 $5 = (($4) + 20|0);
 $6 = HEAP8[$5>>0]|0;
 $7 = $6&255;
 $8 = $7 & 4;
 $9 = ($8|0)==(0);
 if (!($9)) {
  STACKTOP = sp;return;
 }
 $10 = HEAP32[$0>>2]|0;
 $11 = HEAP32[$10>>2]|0;
 $12 = ($11|0)==(0);
 if (!($12)) {
  $13 = HEAP32[$1>>2]|0;
  $14 = HEAP32[$13>>2]|0;
  $15 = (($14) + 4|0);
  $16 = HEAP32[$15>>2]|0;
  $17 = $16 & 1;
  $18 = ($17|0)!=(0);
  if (!($18)) {
   STACKTOP = sp;return;
  }
 }
 $19 = HEAP32[$0>>2]|0;
 $20 = (($19) + 64|0);
 $21 = $20;
 $22 = $21;
 $23 = HEAP32[$22>>2]|0;
 $24 = (($21) + 4)|0;
 $25 = $24;
 $26 = HEAP32[$25>>2]|0;
 $27 = HEAP32[$1>>2]|0;
 $28 = (($27) + 40|0);
 $29 = $28;
 $30 = $29;
 $31 = HEAP32[$30>>2]|0;
 $32 = (($29) + 4)|0;
 $33 = $32;
 $34 = HEAP32[$33>>2]|0;
 $35 = $23 & $31;
 $36 = $26 & $34;
 $37 = ($35|0)==(0);
 $38 = ($36|0)==(0);
 $39 = $37 & $38;
 if (!($39)) {
  STACKTOP = sp;return;
 }
 $40 = HEAP32[$1>>2]|0;
 $41 = (($40) + 20|0);
 $42 = HEAP8[$41>>0]|0;
 $43 = $42&255;
 $44 = $43 | 4;
 $45 = $44&255;
 HEAP8[$41>>0] = $45;
 $46 = HEAP32[$1>>2]|0;
 $47 = (($46) + 4|0);
 $48 = HEAP32[$47>>2]|0;
 $49 = ($48|0)>=(0);
 if (!($49)) {
  STACKTOP = sp;return;
 }
 $50 = HEAP32[$1>>2]|0;
 $51 = (($50) + 4|0);
 $52 = HEAP32[$51>>2]|0;
 $53 = HEAP32[$1>>2]|0;
 $54 = (($53) + 24|0);
 $55 = HEAP32[$54>>2]|0;
 $56 = (($55) + 20|0);
 $57 = HEAP32[$56>>2]|0;
 $58 = (($57) + (($52*48)|0)|0);
 HEAP32[$pOther>>2] = $58;
 $59 = HEAP32[$pOther>>2]|0;
 $60 = (($59) + 21|0);
 $61 = HEAP8[$60>>0]|0;
 $62 = (($61) + -1)<<24>>24;
 HEAP8[$60>>0] = $62;
 $63 = $62&255;
 $64 = ($63|0)==(0);
 if (!($64)) {
  STACKTOP = sp;return;
 }
 $65 = HEAP32[$0>>2]|0;
 $66 = HEAP32[$pOther>>2]|0;
 _disableTerm($65,$66);
 STACKTOP = sp;return;
}
function _codeAllEqualityTerms($pParse,$pLevel,$bRev,$nExtraReg,$pzAff) {
 $pParse = $pParse|0;
 $pLevel = $pLevel|0;
 $bRev = $bRev|0;
 $nExtraReg = $nExtraReg|0;
 $pzAff = $pzAff|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $17 = 0, $18 = 0;
 var $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0;
 var $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0;
 var $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0;
 var $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0;
 var $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $iIdxCur = 0, $j = 0, $nEq = 0, $nReg = 0, $nSkip = 0, $pIdx = 0, $pLoop = 0, $pRight = 0, $pTerm = 0, $r1 = 0, $regBase = 0;
 var $v = 0, $zAff = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 80|0;
 $0 = sp + 60|0;
 $1 = sp + 56|0;
 $2 = sp + 52|0;
 $3 = sp + 48|0;
 $4 = sp + 44|0;
 $nEq = sp + 66|0;
 $nSkip = sp + 64|0;
 $v = sp + 40|0;
 $pIdx = sp + 36|0;
 $pTerm = sp + 32|0;
 $pLoop = sp + 28|0;
 $j = sp + 24|0;
 $regBase = sp + 20|0;
 $nReg = sp + 16|0;
 $zAff = sp + 12|0;
 $iIdxCur = sp + 8|0;
 $r1 = sp + 4|0;
 $pRight = sp;
 HEAP32[$0>>2] = $pParse;
 HEAP32[$1>>2] = $pLevel;
 HEAP32[$2>>2] = $bRev;
 HEAP32[$3>>2] = $nExtraReg;
 HEAP32[$4>>2] = $pzAff;
 $5 = HEAP32[$0>>2]|0;
 $6 = (($5) + 8|0);
 $7 = HEAP32[$6>>2]|0;
 HEAP32[$v>>2] = $7;
 $8 = HEAP32[$1>>2]|0;
 $9 = (($8) + 56|0);
 $10 = HEAP32[$9>>2]|0;
 HEAP32[$pLoop>>2] = $10;
 $11 = HEAP32[$pLoop>>2]|0;
 $12 = (($11) + 24|0);
 $13 = HEAP16[$12>>1]|0;
 HEAP16[$nEq>>1] = $13;
 $14 = HEAP32[$pLoop>>2]|0;
 $15 = (($14) + 24|0);
 $16 = (($15) + 2|0);
 $17 = HEAP16[$16>>1]|0;
 HEAP16[$nSkip>>1] = $17;
 $18 = HEAP32[$pLoop>>2]|0;
 $19 = (($18) + 24|0);
 $20 = (($19) + 4|0);
 $21 = HEAP32[$20>>2]|0;
 HEAP32[$pIdx>>2] = $21;
 $22 = HEAP32[$0>>2]|0;
 $23 = (($22) + 72|0);
 $24 = HEAP32[$23>>2]|0;
 $25 = (($24) + 1)|0;
 HEAP32[$regBase>>2] = $25;
 $26 = HEAP32[$pLoop>>2]|0;
 $27 = (($26) + 24|0);
 $28 = HEAP16[$27>>1]|0;
 $29 = $28&65535;
 $30 = HEAP32[$3>>2]|0;
 $31 = (($29) + ($30))|0;
 HEAP32[$nReg>>2] = $31;
 $32 = HEAP32[$nReg>>2]|0;
 $33 = HEAP32[$0>>2]|0;
 $34 = (($33) + 72|0);
 $35 = HEAP32[$34>>2]|0;
 $36 = (($35) + ($32))|0;
 HEAP32[$34>>2] = $36;
 $37 = HEAP32[$0>>2]|0;
 $38 = HEAP32[$37>>2]|0;
 $39 = HEAP32[$v>>2]|0;
 $40 = HEAP32[$pIdx>>2]|0;
 $41 = (_sqlite3IndexAffinityStr($39,$40)|0);
 $42 = (_sqlite3DbStrDup($38,$41)|0);
 HEAP32[$zAff>>2] = $42;
 $43 = HEAP32[$zAff>>2]|0;
 $44 = ($43|0)!=(0|0);
 if (!($44)) {
  $45 = HEAP32[$0>>2]|0;
  $46 = HEAP32[$45>>2]|0;
  $47 = (($46) + 64|0);
  HEAP8[$47>>0] = 1;
 }
 $48 = HEAP16[$nSkip>>1]|0;
 $49 = ($48<<16>>16)!=(0);
 L4: do {
  if ($49) {
   $50 = HEAP32[$1>>2]|0;
   $51 = (($50) + 8|0);
   $52 = HEAP32[$51>>2]|0;
   HEAP32[$iIdxCur>>2] = $52;
   $53 = HEAP32[$v>>2]|0;
   $54 = HEAP32[$2>>2]|0;
   $55 = ($54|0)!=(0);
   $56 = $55 ? 105 : 108;
   $57 = HEAP32[$iIdxCur>>2]|0;
   (_sqlite3VdbeAddOp1($53,$56,$57)|0);
   $58 = HEAP32[$v>>2]|0;
   $59 = (_sqlite3VdbeAddOp0($58,16)|0);
   HEAP32[$j>>2] = $59;
   $60 = HEAP32[$v>>2]|0;
   $61 = HEAP32[$2>>2]|0;
   $62 = ($61|0)!=(0);
   $63 = $62 ? 62 : 65;
   $64 = HEAP32[$iIdxCur>>2]|0;
   $65 = HEAP32[$regBase>>2]|0;
   $66 = HEAP16[$nSkip>>1]|0;
   $67 = $66&65535;
   $68 = (_sqlite3VdbeAddOp4Int($60,$63,$64,0,$65,$67)|0);
   $69 = HEAP32[$1>>2]|0;
   $70 = (($69) + 20|0);
   HEAP32[$70>>2] = $68;
   $71 = HEAP32[$v>>2]|0;
   $72 = HEAP32[$j>>2]|0;
   _sqlite3VdbeJumpHere($71,$72);
   HEAP32[$j>>2] = 0;
   while(1) {
    $73 = HEAP32[$j>>2]|0;
    $74 = HEAP16[$nSkip>>1]|0;
    $75 = $74&65535;
    $76 = ($73|0)<($75|0);
    if (!($76)) {
     break L4;
    }
    $77 = HEAP32[$v>>2]|0;
    $78 = HEAP32[$iIdxCur>>2]|0;
    $79 = HEAP32[$j>>2]|0;
    $80 = HEAP32[$regBase>>2]|0;
    $81 = HEAP32[$j>>2]|0;
    $82 = (($80) + ($81))|0;
    (_sqlite3VdbeAddOp3($77,47,$78,$79,$82)|0);
    $83 = HEAP32[$j>>2]|0;
    $84 = (($83) + 1)|0;
    HEAP32[$j>>2] = $84;
   }
  }
 } while(0);
 $85 = HEAP16[$nSkip>>1]|0;
 $86 = $85&65535;
 HEAP32[$j>>2] = $86;
 while(1) {
  $87 = HEAP32[$j>>2]|0;
  $88 = HEAP16[$nEq>>1]|0;
  $89 = $88&65535;
  $90 = ($87|0)<($89|0);
  if (!($90)) {
   break;
  }
  $91 = HEAP32[$j>>2]|0;
  $92 = HEAP32[$pLoop>>2]|0;
  $93 = (($92) + 44|0);
  $94 = HEAP32[$93>>2]|0;
  $95 = (($94) + ($91<<2)|0);
  $96 = HEAP32[$95>>2]|0;
  HEAP32[$pTerm>>2] = $96;
  $97 = HEAP32[$0>>2]|0;
  $98 = HEAP32[$pTerm>>2]|0;
  $99 = HEAP32[$1>>2]|0;
  $100 = HEAP32[$j>>2]|0;
  $101 = HEAP32[$2>>2]|0;
  $102 = HEAP32[$regBase>>2]|0;
  $103 = HEAP32[$j>>2]|0;
  $104 = (($102) + ($103))|0;
  $105 = (_codeEqualityTerm($97,$98,$99,$100,$101,$104)|0);
  HEAP32[$r1>>2] = $105;
  $106 = HEAP32[$r1>>2]|0;
  $107 = HEAP32[$regBase>>2]|0;
  $108 = HEAP32[$j>>2]|0;
  $109 = (($107) + ($108))|0;
  $110 = ($106|0)!=($109|0);
  do {
   if ($110) {
    $111 = HEAP32[$nReg>>2]|0;
    $112 = ($111|0)==(1);
    if ($112) {
     $113 = HEAP32[$0>>2]|0;
     $114 = HEAP32[$regBase>>2]|0;
     _sqlite3ReleaseTempReg($113,$114);
     $115 = HEAP32[$r1>>2]|0;
     HEAP32[$regBase>>2] = $115;
     break;
    } else {
     $116 = HEAP32[$v>>2]|0;
     $117 = HEAP32[$r1>>2]|0;
     $118 = HEAP32[$regBase>>2]|0;
     $119 = HEAP32[$j>>2]|0;
     $120 = (($118) + ($119))|0;
     (_sqlite3VdbeAddOp2($116,34,$117,$120)|0);
     break;
    }
   }
  } while(0);
  $121 = HEAP32[$pTerm>>2]|0;
  $122 = (($121) + 18|0);
  $123 = HEAP16[$122>>1]|0;
  $124 = $123&65535;
  $125 = $124 & 129;
  $126 = ($125|0)==(0);
  if ($126) {
   $127 = HEAP32[$pTerm>>2]|0;
   $128 = HEAP32[$127>>2]|0;
   $129 = (($128) + 16|0);
   $130 = HEAP32[$129>>2]|0;
   HEAP32[$pRight>>2] = $130;
   $131 = HEAP32[$pRight>>2]|0;
   $132 = (_sqlite3ExprCanBeNull($131)|0);
   $133 = ($132|0)!=(0);
   if ($133) {
    $134 = HEAP32[$v>>2]|0;
    $135 = HEAP32[$regBase>>2]|0;
    $136 = HEAP32[$j>>2]|0;
    $137 = (($135) + ($136))|0;
    $138 = HEAP32[$1>>2]|0;
    $139 = (($138) + 12|0);
    $140 = HEAP32[$139>>2]|0;
    (_sqlite3VdbeAddOp2($134,76,$137,$140)|0);
   }
   $141 = HEAP32[$zAff>>2]|0;
   $142 = ($141|0)!=(0|0);
   if ($142) {
    $143 = HEAP32[$pRight>>2]|0;
    $144 = HEAP32[$j>>2]|0;
    $145 = HEAP32[$zAff>>2]|0;
    $146 = (($145) + ($144)|0);
    $147 = HEAP8[$146>>0]|0;
    $148 = (_sqlite3CompareAffinity($143,$147)|0);
    $149 = $148 << 24 >> 24;
    $150 = ($149|0)==(65);
    if ($150) {
     $151 = HEAP32[$j>>2]|0;
     $152 = HEAP32[$zAff>>2]|0;
     $153 = (($152) + ($151)|0);
     HEAP8[$153>>0] = 65;
    }
    $154 = HEAP32[$pRight>>2]|0;
    $155 = HEAP32[$j>>2]|0;
    $156 = HEAP32[$zAff>>2]|0;
    $157 = (($156) + ($155)|0);
    $158 = HEAP8[$157>>0]|0;
    $159 = (_sqlite3ExprNeedsNoAffinityChange($154,$158)|0);
    $160 = ($159|0)!=(0);
    if ($160) {
     $161 = HEAP32[$j>>2]|0;
     $162 = HEAP32[$zAff>>2]|0;
     $163 = (($162) + ($161)|0);
     HEAP8[$163>>0] = 65;
    }
   }
  }
  $164 = HEAP32[$j>>2]|0;
  $165 = (($164) + 1)|0;
  HEAP32[$j>>2] = $165;
 }
 $166 = HEAP32[$zAff>>2]|0;
 $167 = HEAP32[$4>>2]|0;
 HEAP32[$167>>2] = $166;
 $168 = HEAP32[$regBase>>2]|0;
 STACKTOP = sp;return ($168|0);
}
function _sqlite3ExprNeedsNoAffinityChange($p,$aff) {
 $p = $p|0;
 $aff = $aff|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $7 = 0, $8 = 0;
 var $9 = 0, $op = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $1 = sp;
 $2 = sp + 9|0;
 $op = sp + 8|0;
 HEAP32[$1>>2] = $p;
 HEAP8[$2>>0] = $aff;
 $3 = HEAP8[$2>>0]|0;
 $4 = $3 << 24 >> 24;
 $5 = ($4|0)==(65);
 L1: do {
  if ($5) {
   HEAP32[$0>>2] = 1;
  } else {
   while(1) {
    $6 = HEAP32[$1>>2]|0;
    $7 = HEAP8[$6>>0]|0;
    $8 = $7&255;
    $9 = ($8|0)==(158);
    if ($9) {
     $60 = 1;
    } else {
     $10 = HEAP32[$1>>2]|0;
     $11 = HEAP8[$10>>0]|0;
     $12 = $11&255;
     $13 = ($12|0)==(157);
     $60 = $13;
    }
    $14 = HEAP32[$1>>2]|0;
    if (!($60)) {
     break;
    }
    $15 = (($14) + 12|0);
    $16 = HEAP32[$15>>2]|0;
    HEAP32[$1>>2] = $16;
   }
   $17 = HEAP8[$14>>0]|0;
   HEAP8[$op>>0] = $17;
   $18 = HEAP8[$op>>0]|0;
   $19 = $18&255;
   $20 = ($19|0)==(159);
   if ($20) {
    $21 = HEAP32[$1>>2]|0;
    $22 = (($21) + 38|0);
    $23 = HEAP8[$22>>0]|0;
    HEAP8[$op>>0] = $23;
   }
   $24 = HEAP8[$op>>0]|0;
   $25 = $24&255;
   switch ($25|0) {
   case 132:  {
    $26 = HEAP8[$2>>0]|0;
    $27 = $26 << 24 >> 24;
    $28 = ($27|0)==(68);
    if ($28) {
     $33 = 1;
    } else {
     $29 = HEAP8[$2>>0]|0;
     $30 = $29 << 24 >> 24;
     $31 = ($30|0)==(67);
     $33 = $31;
    }
    $32 = $33&1;
    HEAP32[$0>>2] = $32;
    break L1;
    break;
   }
   case 97:  {
    $42 = HEAP8[$2>>0]|0;
    $43 = $42 << 24 >> 24;
    $44 = ($43|0)==(66);
    $45 = $44&1;
    HEAP32[$0>>2] = $45;
    break L1;
    break;
   }
   case 134:  {
    HEAP32[$0>>2] = 1;
    break L1;
    break;
   }
   case 133:  {
    $34 = HEAP8[$2>>0]|0;
    $35 = $34 << 24 >> 24;
    $36 = ($35|0)==(69);
    if ($36) {
     $41 = 1;
    } else {
     $37 = HEAP8[$2>>0]|0;
     $38 = $37 << 24 >> 24;
     $39 = ($38|0)==(67);
     $41 = $39;
    }
    $40 = $41&1;
    HEAP32[$0>>2] = $40;
    break L1;
    break;
   }
   case 154:  {
    $46 = HEAP32[$1>>2]|0;
    $47 = (($46) + 32|0);
    $48 = HEAP16[$47>>1]|0;
    $49 = $48 << 16 >> 16;
    $50 = ($49|0)<(0);
    if ($50) {
     $51 = HEAP8[$2>>0]|0;
     $52 = $51 << 24 >> 24;
     $53 = ($52|0)==(68);
     if ($53) {
      $58 = 1;
     } else {
      $54 = HEAP8[$2>>0]|0;
      $55 = $54 << 24 >> 24;
      $56 = ($55|0)==(67);
      $58 = $56;
     }
    } else {
     $58 = 0;
    }
    $57 = $58&1;
    HEAP32[$0>>2] = $57;
    break L1;
    break;
   }
   default: {
    HEAP32[$0>>2] = 0;
    break L1;
   }
   }
  }
 } while(0);
 $59 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($59|0);
}
function _codeApplyAffinity($pParse,$base,$n,$zAff) {
 $pParse = $pParse|0;
 $base = $base|0;
 $n = $n|0;
 $zAff = $zAff|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $5 = 0, $6 = 0;
 var $7 = 0, $8 = 0, $9 = 0, $v = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 16|0;
 $1 = sp + 12|0;
 $2 = sp + 8|0;
 $3 = sp + 4|0;
 $v = sp;
 HEAP32[$0>>2] = $pParse;
 HEAP32[$1>>2] = $base;
 HEAP32[$2>>2] = $n;
 HEAP32[$3>>2] = $zAff;
 $4 = HEAP32[$0>>2]|0;
 $5 = (($4) + 8|0);
 $6 = HEAP32[$5>>2]|0;
 HEAP32[$v>>2] = $6;
 $7 = HEAP32[$3>>2]|0;
 $8 = ($7|0)==(0|0);
 if ($8) {
  STACKTOP = sp;return;
 }
 while(1) {
  $9 = HEAP32[$2>>2]|0;
  $10 = ($9|0)>(0);
  if (!($10)) {
   break;
  }
  $11 = HEAP32[$3>>2]|0;
  $12 = HEAP8[$11>>0]|0;
  $13 = $12 << 24 >> 24;
  $14 = ($13|0)==(65);
  if (!($14)) {
   break;
  }
  $15 = HEAP32[$2>>2]|0;
  $16 = (($15) + -1)|0;
  HEAP32[$2>>2] = $16;
  $17 = HEAP32[$1>>2]|0;
  $18 = (($17) + 1)|0;
  HEAP32[$1>>2] = $18;
  $19 = HEAP32[$3>>2]|0;
  $20 = (($19) + 1|0);
  HEAP32[$3>>2] = $20;
 }
 while(1) {
  $21 = HEAP32[$2>>2]|0;
  $22 = ($21|0)>(1);
  if ($22) {
   $23 = HEAP32[$2>>2]|0;
   $24 = (($23) - 1)|0;
   $25 = HEAP32[$3>>2]|0;
   $26 = (($25) + ($24)|0);
   $27 = HEAP8[$26>>0]|0;
   $28 = $27 << 24 >> 24;
   $29 = ($28|0)==(65);
   $42 = $29;
  } else {
   $42 = 0;
  }
  $30 = HEAP32[$2>>2]|0;
  if (!($42)) {
   break;
  }
  $31 = (($30) + -1)|0;
  HEAP32[$2>>2] = $31;
 }
 $32 = ($30|0)>(0);
 if (!($32)) {
  STACKTOP = sp;return;
 }
 $33 = HEAP32[$v>>2]|0;
 $34 = HEAP32[$1>>2]|0;
 $35 = HEAP32[$2>>2]|0;
 (_sqlite3VdbeAddOp2($33,48,$34,$35)|0);
 $36 = HEAP32[$v>>2]|0;
 $37 = HEAP32[$3>>2]|0;
 $38 = HEAP32[$2>>2]|0;
 _sqlite3VdbeChangeP4($36,-1,$37,$38);
 $39 = HEAP32[$0>>2]|0;
 $40 = HEAP32[$1>>2]|0;
 $41 = HEAP32[$2>>2]|0;
 _sqlite3ExprCacheAffinityChange($39,$40,$41);
 STACKTOP = sp;return;
}
function _findTerm($pWC,$iCur,$iColumn,$0,$1,$op,$pIdx) {
 $pWC = $pWC|0;
 $iCur = $iCur|0;
 $iColumn = $iColumn|0;
 $0 = $0|0;
 $1 = $1|0;
 $op = $op|0;
 $pIdx = $pIdx|0;
 var $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0;
 var $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0;
 var $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $7 = 0;
 var $8 = 0, $9 = 0, $p = 0, $pResult = 0, $scan = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 160|0;
 $2 = sp + 148|0;
 $3 = sp + 144|0;
 $4 = sp + 140|0;
 $5 = sp + 136|0;
 $6 = sp;
 $7 = sp + 132|0;
 $8 = sp + 128|0;
 $pResult = sp + 124|0;
 $p = sp + 120|0;
 $scan = sp + 8|0;
 HEAP32[$3>>2] = $pWC;
 HEAP32[$4>>2] = $iCur;
 HEAP32[$5>>2] = $iColumn;
 $9 = $6;
 $10 = $9;
 HEAP32[$10>>2] = $0;
 $11 = (($9) + 4)|0;
 $12 = $11;
 HEAP32[$12>>2] = $1;
 HEAP32[$7>>2] = $op;
 HEAP32[$8>>2] = $pIdx;
 HEAP32[$pResult>>2] = 0;
 $13 = HEAP32[$3>>2]|0;
 $14 = HEAP32[$4>>2]|0;
 $15 = HEAP32[$5>>2]|0;
 $16 = HEAP32[$7>>2]|0;
 $17 = HEAP32[$8>>2]|0;
 $18 = (_whereScanInit($scan,$13,$14,$15,$16,$17)|0);
 HEAP32[$p>>2] = $18;
 while(1) {
  $19 = HEAP32[$p>>2]|0;
  $20 = ($19|0)!=(0|0);
  if (!($20)) {
   label = 10;
   break;
  }
  $21 = HEAP32[$p>>2]|0;
  $22 = (($21) + 32|0);
  $23 = $22;
  $24 = $23;
  $25 = HEAP32[$24>>2]|0;
  $26 = (($23) + 4)|0;
  $27 = $26;
  $28 = HEAP32[$27>>2]|0;
  $29 = $6;
  $30 = $29;
  $31 = HEAP32[$30>>2]|0;
  $32 = (($29) + 4)|0;
  $33 = $32;
  $34 = HEAP32[$33>>2]|0;
  $35 = $25 & $31;
  $36 = $28 & $34;
  $37 = ($35|0)==(0);
  $38 = ($36|0)==(0);
  $39 = $37 & $38;
  if ($39) {
   $40 = HEAP32[$p>>2]|0;
   $41 = (($40) + 32|0);
   $42 = $41;
   $43 = $42;
   $44 = HEAP32[$43>>2]|0;
   $45 = (($42) + 4)|0;
   $46 = $45;
   $47 = HEAP32[$46>>2]|0;
   $48 = ($44|0)==(0);
   $49 = ($47|0)==(0);
   $50 = $48 & $49;
   if ($50) {
    $51 = HEAP32[$p>>2]|0;
    $52 = (($51) + 18|0);
    $53 = HEAP16[$52>>1]|0;
    $54 = $53&65535;
    $55 = $54 & 2;
    $56 = ($55|0)!=(0);
    if ($56) {
     label = 6;
     break;
    }
   }
   $58 = HEAP32[$pResult>>2]|0;
   $59 = ($58|0)==(0|0);
   if ($59) {
    $60 = HEAP32[$p>>2]|0;
    HEAP32[$pResult>>2] = $60;
   }
  }
  $61 = (_whereScanNext($scan)|0);
  HEAP32[$p>>2] = $61;
 }
 if ((label|0) == 6) {
  $57 = HEAP32[$p>>2]|0;
  HEAP32[$2>>2] = $57;
  $63 = HEAP32[$2>>2]|0;
  STACKTOP = sp;return ($63|0);
 }
 else if ((label|0) == 10) {
  $62 = HEAP32[$pResult>>2]|0;
  HEAP32[$2>>2] = $62;
  $63 = HEAP32[$2>>2]|0;
  STACKTOP = sp;return ($63|0);
 }
 return 0|0;
}
function _whereScanInit($pScan,$pWC,$iCur,$iColumn,$opMask,$pIdx) {
 $pScan = $pScan|0;
 $pWC = $pWC|0;
 $iCur = $iCur|0;
 $iColumn = $iColumn|0;
 $opMask = $opMask|0;
 $pIdx = $pIdx|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $8 = 0, $9 = 0, $j = 0, $or$cond = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 28|0;
 $1 = sp + 24|0;
 $2 = sp + 20|0;
 $3 = sp + 16|0;
 $4 = sp + 12|0;
 $5 = sp + 8|0;
 $6 = sp + 4|0;
 $j = sp;
 HEAP32[$1>>2] = $pScan;
 HEAP32[$2>>2] = $pWC;
 HEAP32[$3>>2] = $iCur;
 HEAP32[$4>>2] = $iColumn;
 HEAP32[$5>>2] = $opMask;
 HEAP32[$6>>2] = $pIdx;
 $7 = HEAP32[$2>>2]|0;
 $8 = HEAP32[$1>>2]|0;
 HEAP32[$8>>2] = $7;
 $9 = HEAP32[$2>>2]|0;
 $10 = HEAP32[$1>>2]|0;
 $11 = (($10) + 4|0);
 HEAP32[$11>>2] = $9;
 $12 = HEAP32[$6>>2]|0;
 $13 = ($12|0)!=(0|0);
 $14 = HEAP32[$4>>2]|0;
 $15 = ($14|0)>=(0);
 $or$cond = $13 & $15;
 do {
  if ($or$cond) {
   $16 = HEAP32[$4>>2]|0;
   $17 = HEAP32[$6>>2]|0;
   $18 = (($17) + 12|0);
   $19 = HEAP32[$18>>2]|0;
   $20 = (($19) + 4|0);
   $21 = HEAP32[$20>>2]|0;
   $22 = (($21) + (($16*24)|0)|0);
   $23 = (($22) + 21|0);
   $24 = HEAP8[$23>>0]|0;
   $25 = HEAP32[$1>>2]|0;
   $26 = (($25) + 12|0);
   HEAP8[$26>>0] = $24;
   HEAP32[$j>>2] = 0;
   while(1) {
    $27 = HEAP32[$j>>2]|0;
    $28 = HEAP32[$6>>2]|0;
    $29 = (($28) + 4|0);
    $30 = HEAP32[$29>>2]|0;
    $31 = (($30) + ($27<<1)|0);
    $32 = HEAP16[$31>>1]|0;
    $33 = $32 << 16 >> 16;
    $34 = HEAP32[$4>>2]|0;
    $35 = ($33|0)!=($34|0);
    $36 = HEAP32[$j>>2]|0;
    $37 = HEAP32[$6>>2]|0;
    if (!($35)) {
     label = 7;
     break;
    }
    $38 = (($37) + 52|0);
    $39 = HEAP16[$38>>1]|0;
    $40 = $39&65535;
    $41 = ($36|0)>($40|0);
    if ($41) {
     break;
    }
    $42 = HEAP32[$j>>2]|0;
    $43 = (($42) + 1)|0;
    HEAP32[$j>>2] = $43;
   }
   if ((label|0) == 7) {
    $44 = (($37) + 32|0);
    $45 = HEAP32[$44>>2]|0;
    $46 = (($45) + ($36<<2)|0);
    $47 = HEAP32[$46>>2]|0;
    $48 = HEAP32[$1>>2]|0;
    $49 = (($48) + 8|0);
    HEAP32[$49>>2] = $47;
    break;
   }
   HEAP32[$0>>2] = 0;
   $72 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($72|0);
  } else {
   $50 = HEAP32[$1>>2]|0;
   $51 = (($50) + 12|0);
   HEAP8[$51>>0] = 0;
   $52 = HEAP32[$1>>2]|0;
   $53 = (($52) + 8|0);
   HEAP32[$53>>2] = 0;
  }
 } while(0);
 $54 = HEAP32[$5>>2]|0;
 $55 = HEAP32[$1>>2]|0;
 $56 = (($55) + 16|0);
 HEAP32[$56>>2] = $54;
 $57 = HEAP32[$1>>2]|0;
 $58 = (($57) + 20|0);
 HEAP32[$58>>2] = 0;
 $59 = HEAP32[$3>>2]|0;
 $60 = HEAP32[$1>>2]|0;
 $61 = (($60) + 24|0);
 HEAP32[$61>>2] = $59;
 $62 = HEAP32[$4>>2]|0;
 $63 = HEAP32[$1>>2]|0;
 $64 = (($63) + 24|0);
 $65 = (($64) + 4|0);
 HEAP32[$65>>2] = $62;
 $66 = HEAP32[$1>>2]|0;
 $67 = (($66) + 13|0);
 HEAP8[$67>>0] = 2;
 $68 = HEAP32[$1>>2]|0;
 $69 = (($68) + 14|0);
 HEAP8[$69>>0] = 2;
 $70 = HEAP32[$1>>2]|0;
 $71 = (_whereScanNext($70)|0);
 HEAP32[$0>>2] = $71;
 $72 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($72|0);
}
function _whereScanNext($pScan) {
 $pScan = $pScan|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0;
 var $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0, $188 = 0;
 var $189 = 0, $19 = 0, $190 = 0, $191 = 0, $192 = 0, $193 = 0, $194 = 0, $195 = 0, $196 = 0, $197 = 0, $198 = 0, $199 = 0, $2 = 0, $20 = 0, $200 = 0, $201 = 0, $202 = 0, $203 = 0, $204 = 0, $205 = 0;
 var $206 = 0, $207 = 0, $208 = 0, $209 = 0, $21 = 0, $210 = 0, $211 = 0, $212 = 0, $213 = 0, $214 = 0, $215 = 0, $216 = 0, $217 = 0, $218 = 0, $219 = 0, $22 = 0, $220 = 0, $221 = 0, $222 = 0, $223 = 0;
 var $224 = 0, $225 = 0, $226 = 0, $227 = 0, $228 = 0, $229 = 0, $23 = 0, $230 = 0, $231 = 0, $232 = 0, $233 = 0, $234 = 0, $235 = 0, $236 = 0, $237 = 0, $238 = 0, $239 = 0, $24 = 0, $240 = 0, $241 = 0;
 var $242 = 0, $243 = 0, $244 = 0, $245 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0;
 var $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0;
 var $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0;
 var $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0;
 var $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $iColumn = 0, $iCur = 0, $j = 0, $k = 0, $pColl = 0, $pParse = 0, $pTerm = 0, $pWC = 0, $pX = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $0 = sp + 40|0;
 $1 = sp + 36|0;
 $iCur = sp + 32|0;
 $iColumn = sp + 28|0;
 $pX = sp + 24|0;
 $pWC = sp + 20|0;
 $pTerm = sp + 16|0;
 $k = sp + 12|0;
 $j = sp + 8|0;
 $pColl = sp + 4|0;
 $pParse = sp;
 HEAP32[$1>>2] = $pScan;
 $2 = HEAP32[$1>>2]|0;
 $3 = (($2) + 20|0);
 $4 = HEAP32[$3>>2]|0;
 HEAP32[$k>>2] = $4;
 L1: while(1) {
  $5 = HEAP32[$1>>2]|0;
  $6 = (($5) + 14|0);
  $7 = HEAP8[$6>>0]|0;
  $8 = $7&255;
  $9 = HEAP32[$1>>2]|0;
  $10 = (($9) + 13|0);
  $11 = HEAP8[$10>>0]|0;
  $12 = $11&255;
  $13 = ($8|0)<=($12|0);
  if (!($13)) {
   label = 35;
   break;
  }
  $14 = HEAP32[$1>>2]|0;
  $15 = (($14) + 14|0);
  $16 = HEAP8[$15>>0]|0;
  $17 = $16&255;
  $18 = (($17) - 2)|0;
  $19 = HEAP32[$1>>2]|0;
  $20 = (($19) + 24|0);
  $21 = (($20) + ($18<<2)|0);
  $22 = HEAP32[$21>>2]|0;
  HEAP32[$iCur>>2] = $22;
  $23 = HEAP32[$1>>2]|0;
  $24 = (($23) + 14|0);
  $25 = HEAP8[$24>>0]|0;
  $26 = $25&255;
  $27 = (($26) - 1)|0;
  $28 = HEAP32[$1>>2]|0;
  $29 = (($28) + 24|0);
  $30 = (($29) + ($27<<2)|0);
  $31 = HEAP32[$30>>2]|0;
  HEAP32[$iColumn>>2] = $31;
  while(1) {
   $32 = HEAP32[$1>>2]|0;
   $33 = (($32) + 4|0);
   $34 = HEAP32[$33>>2]|0;
   HEAP32[$pWC>>2] = $34;
   $35 = ($34|0)!=(0|0);
   if (!($35)) {
    break;
   }
   $36 = HEAP32[$pWC>>2]|0;
   $37 = (($36) + 20|0);
   $38 = HEAP32[$37>>2]|0;
   $39 = HEAP32[$k>>2]|0;
   $40 = (($38) + (($39*48)|0)|0);
   HEAP32[$pTerm>>2] = $40;
   while(1) {
    $41 = HEAP32[$k>>2]|0;
    $42 = HEAP32[$pWC>>2]|0;
    $43 = (($42) + 12|0);
    $44 = HEAP32[$43>>2]|0;
    $45 = ($41|0)<($44|0);
    if (!($45)) {
     break;
    }
    $46 = HEAP32[$pTerm>>2]|0;
    $47 = (($46) + 8|0);
    $48 = HEAP32[$47>>2]|0;
    $49 = HEAP32[$iCur>>2]|0;
    $50 = ($48|0)==($49|0);
    do {
     if ($50) {
      $51 = HEAP32[$pTerm>>2]|0;
      $52 = (($51) + 12|0);
      $53 = HEAP32[$52>>2]|0;
      $54 = HEAP32[$iColumn>>2]|0;
      $55 = ($53|0)==($54|0);
      if ($55) {
       $56 = HEAP32[$1>>2]|0;
       $57 = (($56) + 14|0);
       $58 = HEAP8[$57>>0]|0;
       $59 = $58&255;
       $60 = ($59|0)<=(2);
       if (!($60)) {
        $61 = HEAP32[$pTerm>>2]|0;
        $62 = HEAP32[$61>>2]|0;
        $63 = (($62) + 4|0);
        $64 = HEAP32[$63>>2]|0;
        $65 = $64 & 1;
        $66 = ($65|0)!=(0);
        if ($66) {
         break;
        }
       }
       $67 = HEAP32[$pTerm>>2]|0;
       $68 = (($67) + 18|0);
       $69 = HEAP16[$68>>1]|0;
       $70 = $69&65535;
       $71 = $70 & 1024;
       $72 = ($71|0)!=(0);
       if ($72) {
        $73 = HEAP32[$1>>2]|0;
        $74 = (($73) + 13|0);
        $75 = HEAP8[$74>>0]|0;
        $76 = $75&255;
        $77 = ($76|0)<(22);
        if ($77) {
         $78 = HEAP32[$pTerm>>2]|0;
         $79 = HEAP32[$78>>2]|0;
         $80 = (($79) + 16|0);
         $81 = HEAP32[$80>>2]|0;
         $82 = (_sqlite3ExprSkipCollate($81)|0);
         HEAP32[$pX>>2] = $82;
         HEAP32[$j>>2] = 0;
         while(1) {
          $83 = HEAP32[$j>>2]|0;
          $84 = HEAP32[$1>>2]|0;
          $85 = (($84) + 13|0);
          $86 = HEAP8[$85>>0]|0;
          $87 = $86&255;
          $88 = ($83|0)<($87|0);
          if (!($88)) {
           break;
          }
          $89 = HEAP32[$j>>2]|0;
          $90 = HEAP32[$1>>2]|0;
          $91 = (($90) + 24|0);
          $92 = (($91) + ($89<<2)|0);
          $93 = HEAP32[$92>>2]|0;
          $94 = HEAP32[$pX>>2]|0;
          $95 = (($94) + 28|0);
          $96 = HEAP32[$95>>2]|0;
          $97 = ($93|0)==($96|0);
          if ($97) {
           $98 = HEAP32[$j>>2]|0;
           $99 = (($98) + 1)|0;
           $100 = HEAP32[$1>>2]|0;
           $101 = (($100) + 24|0);
           $102 = (($101) + ($99<<2)|0);
           $103 = HEAP32[$102>>2]|0;
           $104 = HEAP32[$pX>>2]|0;
           $105 = (($104) + 32|0);
           $106 = HEAP16[$105>>1]|0;
           $107 = $106 << 16 >> 16;
           $108 = ($103|0)==($107|0);
           if ($108) {
            break;
           }
          }
          $109 = HEAP32[$j>>2]|0;
          $110 = (($109) + 2)|0;
          HEAP32[$j>>2] = $110;
         }
         $111 = HEAP32[$j>>2]|0;
         $112 = HEAP32[$1>>2]|0;
         $113 = (($112) + 13|0);
         $114 = HEAP8[$113>>0]|0;
         $115 = $114&255;
         $116 = ($111|0)==($115|0);
         if ($116) {
          $117 = HEAP32[$pX>>2]|0;
          $118 = (($117) + 28|0);
          $119 = HEAP32[$118>>2]|0;
          $120 = HEAP32[$j>>2]|0;
          $121 = HEAP32[$1>>2]|0;
          $122 = (($121) + 24|0);
          $123 = (($122) + ($120<<2)|0);
          HEAP32[$123>>2] = $119;
          $124 = HEAP32[$pX>>2]|0;
          $125 = (($124) + 32|0);
          $126 = HEAP16[$125>>1]|0;
          $127 = $126 << 16 >> 16;
          $128 = HEAP32[$j>>2]|0;
          $129 = (($128) + 1)|0;
          $130 = HEAP32[$1>>2]|0;
          $131 = (($130) + 24|0);
          $132 = (($131) + ($129<<2)|0);
          HEAP32[$132>>2] = $127;
          $133 = HEAP32[$1>>2]|0;
          $134 = (($133) + 13|0);
          $135 = HEAP8[$134>>0]|0;
          $136 = $135&255;
          $137 = (($136) + 2)|0;
          $138 = $137&255;
          HEAP8[$134>>0] = $138;
         }
        }
       }
       $139 = HEAP32[$pTerm>>2]|0;
       $140 = (($139) + 18|0);
       $141 = HEAP16[$140>>1]|0;
       $142 = $141&65535;
       $143 = HEAP32[$1>>2]|0;
       $144 = (($143) + 16|0);
       $145 = HEAP32[$144>>2]|0;
       $146 = $142 & $145;
       $147 = ($146|0)!=(0);
       if ($147) {
        $148 = HEAP32[$1>>2]|0;
        $149 = (($148) + 8|0);
        $150 = HEAP32[$149>>2]|0;
        $151 = ($150|0)!=(0|0);
        if ($151) {
         $152 = HEAP32[$pTerm>>2]|0;
         $153 = (($152) + 18|0);
         $154 = HEAP16[$153>>1]|0;
         $155 = $154&65535;
         $156 = $155 & 128;
         $157 = ($156|0)==(0);
         if ($157) {
          $158 = HEAP32[$pWC>>2]|0;
          $159 = HEAP32[$158>>2]|0;
          $160 = HEAP32[$159>>2]|0;
          HEAP32[$pParse>>2] = $160;
          $161 = HEAP32[$pTerm>>2]|0;
          $162 = HEAP32[$161>>2]|0;
          HEAP32[$pX>>2] = $162;
          $163 = HEAP32[$pX>>2]|0;
          $164 = HEAP32[$1>>2]|0;
          $165 = (($164) + 12|0);
          $166 = HEAP8[$165>>0]|0;
          $167 = (_sqlite3IndexAffinityOk($163,$166)|0);
          $168 = ($167|0)!=(0);
          if (!($168)) {
           break;
          }
          $169 = HEAP32[$pParse>>2]|0;
          $170 = HEAP32[$pX>>2]|0;
          $171 = (($170) + 12|0);
          $172 = HEAP32[$171>>2]|0;
          $173 = HEAP32[$pX>>2]|0;
          $174 = (($173) + 16|0);
          $175 = HEAP32[$174>>2]|0;
          $176 = (_sqlite3BinaryCompareCollSeq($169,$172,$175)|0);
          HEAP32[$pColl>>2] = $176;
          $177 = HEAP32[$pColl>>2]|0;
          $178 = ($177|0)==(0|0);
          if ($178) {
           $179 = HEAP32[$pParse>>2]|0;
           $180 = HEAP32[$179>>2]|0;
           $181 = (($180) + 8|0);
           $182 = HEAP32[$181>>2]|0;
           HEAP32[$pColl>>2] = $182;
          }
          $183 = HEAP32[$pColl>>2]|0;
          $184 = HEAP32[$183>>2]|0;
          $185 = HEAP32[$1>>2]|0;
          $186 = (($185) + 8|0);
          $187 = HEAP32[$186>>2]|0;
          $188 = (_sqlite3_stricmp($184,$187)|0);
          $189 = ($188|0)!=(0);
          if ($189) {
           break;
          }
         }
        }
        $190 = HEAP32[$pTerm>>2]|0;
        $191 = (($190) + 18|0);
        $192 = HEAP16[$191>>1]|0;
        $193 = $192&65535;
        $194 = $193 & 2;
        $195 = ($194|0)!=(0);
        if (!($195)) {
         label = 31;
         break L1;
        }
        $196 = HEAP32[$pTerm>>2]|0;
        $197 = HEAP32[$196>>2]|0;
        $198 = (($197) + 16|0);
        $199 = HEAP32[$198>>2]|0;
        HEAP32[$pX>>2] = $199;
        $200 = HEAP8[$199>>0]|0;
        $201 = $200&255;
        $202 = ($201|0)==(154);
        if (!($202)) {
         label = 31;
         break L1;
        }
        $203 = HEAP32[$pX>>2]|0;
        $204 = (($203) + 28|0);
        $205 = HEAP32[$204>>2]|0;
        $206 = HEAP32[$1>>2]|0;
        $207 = (($206) + 24|0);
        $208 = HEAP32[$207>>2]|0;
        $209 = ($205|0)==($208|0);
        if (!($209)) {
         label = 31;
         break L1;
        }
        $210 = HEAP32[$pX>>2]|0;
        $211 = (($210) + 32|0);
        $212 = HEAP16[$211>>1]|0;
        $213 = $212 << 16 >> 16;
        $214 = HEAP32[$1>>2]|0;
        $215 = (($214) + 24|0);
        $216 = (($215) + 4|0);
        $217 = HEAP32[$216>>2]|0;
        $218 = ($213|0)==($217|0);
        if (!($218)) {
         label = 31;
         break L1;
        }
       }
      }
     }
    } while(0);
    $224 = HEAP32[$k>>2]|0;
    $225 = (($224) + 1)|0;
    HEAP32[$k>>2] = $225;
    $226 = HEAP32[$pTerm>>2]|0;
    $227 = (($226) + 48|0);
    HEAP32[$pTerm>>2] = $227;
   }
   $228 = HEAP32[$1>>2]|0;
   $229 = (($228) + 4|0);
   $230 = HEAP32[$229>>2]|0;
   $231 = (($230) + 4|0);
   $232 = HEAP32[$231>>2]|0;
   $233 = HEAP32[$1>>2]|0;
   $234 = (($233) + 4|0);
   HEAP32[$234>>2] = $232;
   HEAP32[$k>>2] = 0;
  }
  $235 = HEAP32[$1>>2]|0;
  $236 = HEAP32[$235>>2]|0;
  $237 = HEAP32[$1>>2]|0;
  $238 = (($237) + 4|0);
  HEAP32[$238>>2] = $236;
  HEAP32[$k>>2] = 0;
  $239 = HEAP32[$1>>2]|0;
  $240 = (($239) + 14|0);
  $241 = HEAP8[$240>>0]|0;
  $242 = $241&255;
  $243 = (($242) + 2)|0;
  $244 = $243&255;
  HEAP8[$240>>0] = $244;
 }
 if ((label|0) == 31) {
  $219 = HEAP32[$k>>2]|0;
  $220 = (($219) + 1)|0;
  $221 = HEAP32[$1>>2]|0;
  $222 = (($221) + 20|0);
  HEAP32[$222>>2] = $220;
  $223 = HEAP32[$pTerm>>2]|0;
  HEAP32[$0>>2] = $223;
  $245 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($245|0);
 }
 else if ((label|0) == 35) {
  HEAP32[$0>>2] = 0;
  $245 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($245|0);
 }
 return 0|0;
}
function _explainIndexRange($pStr,$pLoop,$pTab) {
 $pStr = $pStr|0;
 $pLoop = $pLoop|0;
 $pTab = $pTab|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0;
 var $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0;
 var $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0;
 var $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0;
 var $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0;
 var $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $aCol = 0, $aiColumn = 0, $i = 0, $j = 0, $nEq = 0, $nSkip = 0, $pIndex = 0, $vararg_buffer = 0, $z = 0, $z1 = 0;
 var $z2 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 64|0;
 $vararg_buffer = sp;
 $0 = sp + 44|0;
 $1 = sp + 40|0;
 $2 = sp + 36|0;
 $pIndex = sp + 32|0;
 $nEq = sp + 50|0;
 $nSkip = sp + 48|0;
 $i = sp + 28|0;
 $j = sp + 24|0;
 $aCol = sp + 20|0;
 $aiColumn = sp + 16|0;
 $z = sp + 12|0;
 $z1 = sp + 8|0;
 $z2 = sp + 4|0;
 HEAP32[$0>>2] = $pStr;
 HEAP32[$1>>2] = $pLoop;
 HEAP32[$2>>2] = $pTab;
 $3 = HEAP32[$1>>2]|0;
 $4 = (($3) + 24|0);
 $5 = (($4) + 4|0);
 $6 = HEAP32[$5>>2]|0;
 HEAP32[$pIndex>>2] = $6;
 $7 = HEAP32[$1>>2]|0;
 $8 = (($7) + 24|0);
 $9 = HEAP16[$8>>1]|0;
 HEAP16[$nEq>>1] = $9;
 $10 = HEAP32[$1>>2]|0;
 $11 = (($10) + 24|0);
 $12 = (($11) + 2|0);
 $13 = HEAP16[$12>>1]|0;
 HEAP16[$nSkip>>1] = $13;
 $14 = HEAP32[$2>>2]|0;
 $15 = (($14) + 4|0);
 $16 = HEAP32[$15>>2]|0;
 HEAP32[$aCol>>2] = $16;
 $17 = HEAP32[$pIndex>>2]|0;
 $18 = (($17) + 4|0);
 $19 = HEAP32[$18>>2]|0;
 HEAP32[$aiColumn>>2] = $19;
 $20 = HEAP16[$nEq>>1]|0;
 $21 = $20&65535;
 $22 = ($21|0)==(0);
 if ($22) {
  $23 = HEAP32[$1>>2]|0;
  $24 = (($23) + 36|0);
  $25 = HEAP32[$24>>2]|0;
  $26 = $25 & 48;
  $27 = ($26|0)==(0);
  if ($27) {
   STACKTOP = sp;return;
  }
 }
 $28 = HEAP32[$0>>2]|0;
 _sqlite3StrAccumAppend($28,23928,2);
 HEAP32[$i>>2] = 0;
 while(1) {
  $29 = HEAP32[$i>>2]|0;
  $30 = HEAP16[$nEq>>1]|0;
  $31 = $30&65535;
  $32 = ($29|0)<($31|0);
  $33 = HEAP32[$i>>2]|0;
  if (!($32)) {
   break;
  }
  $34 = HEAP32[$aiColumn>>2]|0;
  $35 = (($34) + ($33<<1)|0);
  $36 = HEAP16[$35>>1]|0;
  $37 = $36 << 16 >> 16;
  $38 = ($37|0)<(0);
  if ($38) {
   $47 = 9992;
  } else {
   $39 = HEAP32[$i>>2]|0;
   $40 = HEAP32[$aiColumn>>2]|0;
   $41 = (($40) + ($39<<1)|0);
   $42 = HEAP16[$41>>1]|0;
   $43 = $42 << 16 >> 16;
   $44 = HEAP32[$aCol>>2]|0;
   $45 = (($44) + (($43*24)|0)|0);
   $46 = HEAP32[$45>>2]|0;
   $47 = $46;
  }
  HEAP32[$z>>2] = $47;
  $48 = HEAP32[$i>>2]|0;
  $49 = HEAP16[$nSkip>>1]|0;
  $50 = $49&65535;
  $51 = ($48|0)>=($50|0);
  if ($51) {
   $52 = HEAP32[$0>>2]|0;
   $53 = HEAP32[$i>>2]|0;
   $54 = HEAP32[$z>>2]|0;
   _explainAppendTerm($52,$53,$54,23936);
  } else {
   $55 = HEAP32[$i>>2]|0;
   $56 = ($55|0)!=(0);
   if ($56) {
    $57 = HEAP32[$0>>2]|0;
    _sqlite3StrAccumAppend($57,23944,5);
   }
   $58 = HEAP32[$0>>2]|0;
   $59 = HEAP32[$z>>2]|0;
   HEAP32[$vararg_buffer>>2] = $59;
   _sqlite3XPrintf($58,0,23952,$vararg_buffer);
  }
  $60 = HEAP32[$i>>2]|0;
  $61 = (($60) + 1)|0;
  HEAP32[$i>>2] = $61;
 }
 HEAP32[$j>>2] = $33;
 $62 = HEAP32[$1>>2]|0;
 $63 = (($62) + 36|0);
 $64 = HEAP32[$63>>2]|0;
 $65 = $64 & 32;
 $66 = ($65|0)!=(0);
 if ($66) {
  $67 = HEAP32[$j>>2]|0;
  $68 = HEAP32[$aiColumn>>2]|0;
  $69 = (($68) + ($67<<1)|0);
  $70 = HEAP16[$69>>1]|0;
  $71 = $70 << 16 >> 16;
  $72 = ($71|0)<(0);
  if ($72) {
   $81 = 9992;
  } else {
   $73 = HEAP32[$j>>2]|0;
   $74 = HEAP32[$aiColumn>>2]|0;
   $75 = (($74) + ($73<<1)|0);
   $76 = HEAP16[$75>>1]|0;
   $77 = $76 << 16 >> 16;
   $78 = HEAP32[$aCol>>2]|0;
   $79 = (($78) + (($77*24)|0)|0);
   $80 = HEAP32[$79>>2]|0;
   $81 = $80;
  }
  HEAP32[$z1>>2] = $81;
  $82 = HEAP32[$0>>2]|0;
  $83 = HEAP32[$i>>2]|0;
  $84 = (($83) + 1)|0;
  HEAP32[$i>>2] = $84;
  $85 = HEAP32[$z1>>2]|0;
  _explainAppendTerm($82,$83,$85,23960);
 }
 $86 = HEAP32[$1>>2]|0;
 $87 = (($86) + 36|0);
 $88 = HEAP32[$87>>2]|0;
 $89 = $88 & 16;
 $90 = ($89|0)!=(0);
 if ($90) {
  $91 = HEAP32[$j>>2]|0;
  $92 = HEAP32[$aiColumn>>2]|0;
  $93 = (($92) + ($91<<1)|0);
  $94 = HEAP16[$93>>1]|0;
  $95 = $94 << 16 >> 16;
  $96 = ($95|0)<(0);
  if ($96) {
   $105 = 9992;
  } else {
   $97 = HEAP32[$j>>2]|0;
   $98 = HEAP32[$aiColumn>>2]|0;
   $99 = (($98) + ($97<<1)|0);
   $100 = HEAP16[$99>>1]|0;
   $101 = $100 << 16 >> 16;
   $102 = HEAP32[$aCol>>2]|0;
   $103 = (($102) + (($101*24)|0)|0);
   $104 = HEAP32[$103>>2]|0;
   $105 = $104;
  }
  HEAP32[$z2>>2] = $105;
  $106 = HEAP32[$0>>2]|0;
  $107 = HEAP32[$i>>2]|0;
  $108 = HEAP32[$z2>>2]|0;
  _explainAppendTerm($106,$107,$108,23968);
 }
 $109 = HEAP32[$0>>2]|0;
 _sqlite3StrAccumAppend($109,23976,1);
 STACKTOP = sp;return;
}
function _explainAppendTerm($pStr,$iTerm,$zColumn,$zOp) {
 $pStr = $pStr|0;
 $iTerm = $iTerm|0;
 $zColumn = $zColumn|0;
 $zOp = $zOp|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $1 = sp + 8|0;
 $2 = sp + 4|0;
 $3 = sp;
 HEAP32[$0>>2] = $pStr;
 HEAP32[$1>>2] = $iTerm;
 HEAP32[$2>>2] = $zColumn;
 HEAP32[$3>>2] = $zOp;
 $4 = HEAP32[$1>>2]|0;
 $5 = ($4|0)!=(0);
 if ($5) {
  $6 = HEAP32[$0>>2]|0;
  _sqlite3StrAccumAppend($6,23944,5);
 }
 $7 = HEAP32[$0>>2]|0;
 $8 = HEAP32[$2>>2]|0;
 _sqlite3StrAccumAppendAll($7,$8);
 $9 = HEAP32[$0>>2]|0;
 $10 = HEAP32[$3>>2]|0;
 _sqlite3StrAccumAppend($9,$10,1);
 $11 = HEAP32[$0>>2]|0;
 _sqlite3StrAccumAppend($11,19016,1);
 STACKTOP = sp;return;
}
function _termCanDriveIndex($pTerm,$pSrc,$0,$1) {
 $pTerm = $pTerm|0;
 $pSrc = $pSrc|0;
 $0 = $0|0;
 $1 = $1|0;
 var $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0;
 var $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0;
 var $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $7 = 0, $8 = 0;
 var $9 = 0, $aff = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $2 = sp + 16|0;
 $3 = sp + 12|0;
 $4 = sp + 8|0;
 $5 = sp;
 $aff = sp + 20|0;
 HEAP32[$3>>2] = $pTerm;
 HEAP32[$4>>2] = $pSrc;
 $6 = $5;
 $7 = $6;
 HEAP32[$7>>2] = $0;
 $8 = (($6) + 4)|0;
 $9 = $8;
 HEAP32[$9>>2] = $1;
 $10 = HEAP32[$3>>2]|0;
 $11 = (($10) + 8|0);
 $12 = HEAP32[$11>>2]|0;
 $13 = HEAP32[$4>>2]|0;
 $14 = (($13) + 40|0);
 $15 = HEAP32[$14>>2]|0;
 $16 = ($12|0)!=($15|0);
 if ($16) {
  HEAP32[$2>>2] = 0;
  $62 = HEAP32[$2>>2]|0;
  STACKTOP = sp;return ($62|0);
 }
 $17 = HEAP32[$3>>2]|0;
 $18 = (($17) + 18|0);
 $19 = HEAP16[$18>>1]|0;
 $20 = $19&65535;
 $21 = $20 & 2;
 $22 = ($21|0)==(0);
 if ($22) {
  HEAP32[$2>>2] = 0;
  $62 = HEAP32[$2>>2]|0;
  STACKTOP = sp;return ($62|0);
 }
 $23 = HEAP32[$3>>2]|0;
 $24 = (($23) + 32|0);
 $25 = $24;
 $26 = $25;
 $27 = HEAP32[$26>>2]|0;
 $28 = (($25) + 4)|0;
 $29 = $28;
 $30 = HEAP32[$29>>2]|0;
 $31 = $5;
 $32 = $31;
 $33 = HEAP32[$32>>2]|0;
 $34 = (($31) + 4)|0;
 $35 = $34;
 $36 = HEAP32[$35>>2]|0;
 $37 = $27 & $33;
 $38 = $30 & $36;
 $39 = ($37|0)!=(0);
 $40 = ($38|0)!=(0);
 $41 = $39 | $40;
 if ($41) {
  HEAP32[$2>>2] = 0;
  $62 = HEAP32[$2>>2]|0;
  STACKTOP = sp;return ($62|0);
 }
 $42 = HEAP32[$3>>2]|0;
 $43 = (($42) + 12|0);
 $44 = HEAP32[$43>>2]|0;
 $45 = ($44|0)<(0);
 if ($45) {
  HEAP32[$2>>2] = 0;
  $62 = HEAP32[$2>>2]|0;
  STACKTOP = sp;return ($62|0);
 }
 $46 = HEAP32[$3>>2]|0;
 $47 = (($46) + 12|0);
 $48 = HEAP32[$47>>2]|0;
 $49 = HEAP32[$4>>2]|0;
 $50 = (($49) + 16|0);
 $51 = HEAP32[$50>>2]|0;
 $52 = (($51) + 4|0);
 $53 = HEAP32[$52>>2]|0;
 $54 = (($53) + (($48*24)|0)|0);
 $55 = (($54) + 21|0);
 $56 = HEAP8[$55>>0]|0;
 HEAP8[$aff>>0] = $56;
 $57 = HEAP32[$3>>2]|0;
 $58 = HEAP32[$57>>2]|0;
 $59 = HEAP8[$aff>>0]|0;
 $60 = (_sqlite3IndexAffinityOk($58,$59)|0);
 $61 = ($60|0)!=(0);
 if ($61) {
  HEAP32[$2>>2] = 1;
  $62 = HEAP32[$2>>2]|0;
  STACKTOP = sp;return ($62|0);
 } else {
  HEAP32[$2>>2] = 0;
  $62 = HEAP32[$2>>2]|0;
  STACKTOP = sp;return ($62|0);
 }
 return 0|0;
}
function _whereLoopResize($db,$p,$n) {
 $db = $db|0;
 $p = $p|0;
 $n = $n|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $paNew = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 16|0;
 $1 = sp + 12|0;
 $2 = sp + 8|0;
 $3 = sp + 4|0;
 $paNew = sp;
 HEAP32[$1>>2] = $db;
 HEAP32[$2>>2] = $p;
 HEAP32[$3>>2] = $n;
 $4 = HEAP32[$2>>2]|0;
 $5 = (($4) + 42|0);
 $6 = HEAP16[$5>>1]|0;
 $7 = $6&65535;
 $8 = HEAP32[$3>>2]|0;
 $9 = ($7|0)>=($8|0);
 if ($9) {
  HEAP32[$0>>2] = 0;
  $45 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($45|0);
 }
 $10 = HEAP32[$3>>2]|0;
 $11 = (($10) + 7)|0;
 $12 = $11 & -8;
 HEAP32[$3>>2] = $12;
 $13 = HEAP32[$1>>2]|0;
 $14 = HEAP32[$3>>2]|0;
 $15 = $14<<2;
 $16 = (_sqlite3DbMallocRaw($13,$15,0)|0);
 HEAP32[$paNew>>2] = $16;
 $17 = HEAP32[$paNew>>2]|0;
 $18 = ($17|0)==(0|0);
 if ($18) {
  HEAP32[$0>>2] = 7;
  $45 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($45|0);
 }
 $19 = HEAP32[$paNew>>2]|0;
 $20 = HEAP32[$2>>2]|0;
 $21 = (($20) + 44|0);
 $22 = HEAP32[$21>>2]|0;
 $23 = HEAP32[$2>>2]|0;
 $24 = (($23) + 42|0);
 $25 = HEAP16[$24>>1]|0;
 $26 = $25&65535;
 $27 = $26<<2;
 _memcpy(($19|0),($22|0),($27|0))|0;
 $28 = HEAP32[$2>>2]|0;
 $29 = (($28) + 44|0);
 $30 = HEAP32[$29>>2]|0;
 $31 = HEAP32[$2>>2]|0;
 $32 = (($31) + 52|0);
 $33 = ($30|0)!=($32|0);
 if ($33) {
  $34 = HEAP32[$1>>2]|0;
  $35 = HEAP32[$2>>2]|0;
  $36 = (($35) + 44|0);
  $37 = HEAP32[$36>>2]|0;
  _sqlite3DbFree($34,$37);
 }
 $38 = HEAP32[$paNew>>2]|0;
 $39 = HEAP32[$2>>2]|0;
 $40 = (($39) + 44|0);
 HEAP32[$40>>2] = $38;
 $41 = HEAP32[$3>>2]|0;
 $42 = $41&65535;
 $43 = HEAP32[$2>>2]|0;
 $44 = (($43) + 42|0);
 HEAP16[$44>>1] = $42;
 HEAP32[$0>>2] = 0;
 $45 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($45|0);
}
function _sqlite3AllocateIndexObject($db,$nCol,$nExtra,$ppExtra) {
 $db = $db|0;
 $nCol = $nCol|0;
 $nExtra = $nExtra|0;
 $ppExtra = $ppExtra|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $9 = 0;
 var $nByte = 0, $p = 0, $pExtra = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 20|0;
 $1 = sp + 24|0;
 $2 = sp + 16|0;
 $3 = sp + 12|0;
 $p = sp + 8|0;
 $nByte = sp + 4|0;
 $pExtra = sp;
 HEAP32[$0>>2] = $db;
 HEAP16[$1>>1] = $nCol;
 HEAP32[$2>>2] = $nExtra;
 HEAP32[$3>>2] = $ppExtra;
 $4 = HEAP16[$1>>1]|0;
 $5 = $4 << 16 >> 16;
 $6 = $5<<2;
 $7 = (($6) + 7)|0;
 $8 = $7 & -8;
 $9 = (56 + ($8))|0;
 $10 = HEAP16[$1>>1]|0;
 $11 = $10 << 16 >> 16;
 $12 = (($11) + 1)|0;
 $13 = $12<<1;
 $14 = HEAP16[$1>>1]|0;
 $15 = $14 << 16 >> 16;
 $16 = $15<<1;
 $17 = (($13) + ($16))|0;
 $18 = HEAP16[$1>>1]|0;
 $19 = $18 << 16 >> 16;
 $20 = $19;
 $21 = (($17) + ($20))|0;
 $22 = (($21) + 7)|0;
 $23 = $22 & -8;
 $24 = (($9) + ($23))|0;
 HEAP32[$nByte>>2] = $24;
 $25 = HEAP32[$0>>2]|0;
 $26 = HEAP32[$nByte>>2]|0;
 $27 = HEAP32[$2>>2]|0;
 $28 = (($26) + ($27))|0;
 $29 = ($28|0)<(0);
 $30 = $29 << 31 >> 31;
 $31 = (_sqlite3DbMallocZero($25,$28,$30)|0);
 HEAP32[$p>>2] = $31;
 $32 = HEAP32[$p>>2]|0;
 $33 = ($32|0)!=(0|0);
 if (!($33)) {
  $79 = HEAP32[$p>>2]|0;
  STACKTOP = sp;return ($79|0);
 }
 $34 = HEAP32[$p>>2]|0;
 $35 = (($34) + 56|0);
 HEAP32[$pExtra>>2] = $35;
 $36 = HEAP32[$pExtra>>2]|0;
 $37 = HEAP32[$p>>2]|0;
 $38 = (($37) + 32|0);
 HEAP32[$38>>2] = $36;
 $39 = HEAP16[$1>>1]|0;
 $40 = $39 << 16 >> 16;
 $41 = $40<<2;
 $42 = (($41) + 7)|0;
 $43 = $42 & -8;
 $44 = HEAP32[$pExtra>>2]|0;
 $45 = (($44) + ($43)|0);
 HEAP32[$pExtra>>2] = $45;
 $46 = HEAP32[$pExtra>>2]|0;
 $47 = HEAP32[$p>>2]|0;
 $48 = (($47) + 8|0);
 HEAP32[$48>>2] = $46;
 $49 = HEAP16[$1>>1]|0;
 $50 = $49 << 16 >> 16;
 $51 = (($50) + 1)|0;
 $52 = $51<<1;
 $53 = HEAP32[$pExtra>>2]|0;
 $54 = (($53) + ($52)|0);
 HEAP32[$pExtra>>2] = $54;
 $55 = HEAP32[$pExtra>>2]|0;
 $56 = HEAP32[$p>>2]|0;
 $57 = (($56) + 4|0);
 HEAP32[$57>>2] = $55;
 $58 = HEAP16[$1>>1]|0;
 $59 = $58 << 16 >> 16;
 $60 = $59<<1;
 $61 = HEAP32[$pExtra>>2]|0;
 $62 = (($61) + ($60)|0);
 HEAP32[$pExtra>>2] = $62;
 $63 = HEAP32[$pExtra>>2]|0;
 $64 = HEAP32[$p>>2]|0;
 $65 = (($64) + 28|0);
 HEAP32[$65>>2] = $63;
 $66 = HEAP16[$1>>1]|0;
 $67 = HEAP32[$p>>2]|0;
 $68 = (($67) + 52|0);
 HEAP16[$68>>1] = $66;
 $69 = HEAP16[$1>>1]|0;
 $70 = $69 << 16 >> 16;
 $71 = (($70) - 1)|0;
 $72 = $71&65535;
 $73 = HEAP32[$p>>2]|0;
 $74 = (($73) + 50|0);
 HEAP16[$74>>1] = $72;
 $75 = HEAP32[$p>>2]|0;
 $76 = HEAP32[$nByte>>2]|0;
 $77 = (($75) + ($76)|0);
 $78 = HEAP32[$3>>2]|0;
 HEAP32[$78>>2] = $77;
 $79 = HEAP32[$p>>2]|0;
 STACKTOP = sp;return ($79|0);
}
function _exprTableUsage($pMaskSet,$p) {
 $pMaskSet = $pMaskSet|0;
 $p = $p|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0;
 var $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0;
 var $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0;
 var $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0;
 var $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $mask = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 8|0;
 $1 = sp + 20|0;
 $2 = sp + 16|0;
 $mask = sp;
 HEAP32[$1>>2] = $pMaskSet;
 HEAP32[$2>>2] = $p;
 $3 = $mask;
 $4 = $3;
 HEAP32[$4>>2] = 0;
 $5 = (($3) + 4)|0;
 $6 = $5;
 HEAP32[$6>>2] = 0;
 $7 = HEAP32[$2>>2]|0;
 $8 = ($7|0)==(0|0);
 if ($8) {
  $9 = $0;
  $10 = $9;
  HEAP32[$10>>2] = 0;
  $11 = (($9) + 4)|0;
  $12 = $11;
  HEAP32[$12>>2] = 0;
  $111 = $0;
  $112 = $111;
  $113 = HEAP32[$112>>2]|0;
  $114 = (($111) + 4)|0;
  $115 = $114;
  $116 = HEAP32[$115>>2]|0;
  tempRet0 = $116;
  STACKTOP = sp;return ($113|0);
 }
 $13 = HEAP32[$2>>2]|0;
 $14 = HEAP8[$13>>0]|0;
 $15 = $14&255;
 $16 = ($15|0)==(154);
 $17 = HEAP32[$1>>2]|0;
 $18 = HEAP32[$2>>2]|0;
 if ($16) {
  $19 = (($18) + 28|0);
  $20 = HEAP32[$19>>2]|0;
  $21 = (_getMask($17,$20)|0);
  $22 = tempRet0;
  $23 = $mask;
  $24 = $23;
  HEAP32[$24>>2] = $21;
  $25 = (($23) + 4)|0;
  $26 = $25;
  HEAP32[$26>>2] = $22;
  $27 = $mask;
  $28 = $27;
  $29 = HEAP32[$28>>2]|0;
  $30 = (($27) + 4)|0;
  $31 = $30;
  $32 = HEAP32[$31>>2]|0;
  $33 = $0;
  $34 = $33;
  HEAP32[$34>>2] = $29;
  $35 = (($33) + 4)|0;
  $36 = $35;
  HEAP32[$36>>2] = $32;
  $111 = $0;
  $112 = $111;
  $113 = HEAP32[$112>>2]|0;
  $114 = (($111) + 4)|0;
  $115 = $114;
  $116 = HEAP32[$115>>2]|0;
  tempRet0 = $116;
  STACKTOP = sp;return ($113|0);
 }
 $37 = (($18) + 16|0);
 $38 = HEAP32[$37>>2]|0;
 $39 = (_exprTableUsage($17,$38)|0);
 $40 = tempRet0;
 $41 = $mask;
 $42 = $41;
 HEAP32[$42>>2] = $39;
 $43 = (($41) + 4)|0;
 $44 = $43;
 HEAP32[$44>>2] = $40;
 $45 = HEAP32[$1>>2]|0;
 $46 = HEAP32[$2>>2]|0;
 $47 = (($46) + 12|0);
 $48 = HEAP32[$47>>2]|0;
 $49 = (_exprTableUsage($45,$48)|0);
 $50 = tempRet0;
 $51 = $mask;
 $52 = $51;
 $53 = HEAP32[$52>>2]|0;
 $54 = (($51) + 4)|0;
 $55 = $54;
 $56 = HEAP32[$55>>2]|0;
 $57 = $53 | $49;
 $58 = $56 | $50;
 $59 = $mask;
 $60 = $59;
 HEAP32[$60>>2] = $57;
 $61 = (($59) + 4)|0;
 $62 = $61;
 HEAP32[$62>>2] = $58;
 $63 = HEAP32[$2>>2]|0;
 $64 = (($63) + 4|0);
 $65 = HEAP32[$64>>2]|0;
 $66 = $65 & 2048;
 $67 = ($66|0)!=(0);
 $68 = HEAP32[$1>>2]|0;
 $69 = HEAP32[$2>>2]|0;
 $70 = (($69) + 20|0);
 if ($67) {
  $71 = HEAP32[$70>>2]|0;
  $72 = (_exprSelectTableUsage($68,$71)|0);
  $73 = tempRet0;
  $74 = $mask;
  $75 = $74;
  $76 = HEAP32[$75>>2]|0;
  $77 = (($74) + 4)|0;
  $78 = $77;
  $79 = HEAP32[$78>>2]|0;
  $80 = $76 | $72;
  $81 = $79 | $73;
  $82 = $mask;
  $83 = $82;
  HEAP32[$83>>2] = $80;
  $84 = (($82) + 4)|0;
  $85 = $84;
  HEAP32[$85>>2] = $81;
 } else {
  $86 = HEAP32[$70>>2]|0;
  $87 = (_exprListTableUsage($68,$86)|0);
  $88 = tempRet0;
  $89 = $mask;
  $90 = $89;
  $91 = HEAP32[$90>>2]|0;
  $92 = (($89) + 4)|0;
  $93 = $92;
  $94 = HEAP32[$93>>2]|0;
  $95 = $91 | $87;
  $96 = $94 | $88;
  $97 = $mask;
  $98 = $97;
  HEAP32[$98>>2] = $95;
  $99 = (($97) + 4)|0;
  $100 = $99;
  HEAP32[$100>>2] = $96;
 }
 $101 = $mask;
 $102 = $101;
 $103 = HEAP32[$102>>2]|0;
 $104 = (($101) + 4)|0;
 $105 = $104;
 $106 = HEAP32[$105>>2]|0;
 $107 = $0;
 $108 = $107;
 HEAP32[$108>>2] = $103;
 $109 = (($107) + 4)|0;
 $110 = $109;
 HEAP32[$110>>2] = $106;
 $111 = $0;
 $112 = $111;
 $113 = HEAP32[$112>>2]|0;
 $114 = (($111) + 4)|0;
 $115 = $114;
 $116 = HEAP32[$115>>2]|0;
 tempRet0 = $116;
 STACKTOP = sp;return ($113|0);
}
function _exprSelectTableUsage($pMaskSet,$pS) {
 $pMaskSet = $pMaskSet|0;
 $pS = $pS|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0;
 var $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0;
 var $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0;
 var $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0;
 var $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $i = 0;
 var $mask = 0, $pSrc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 20|0;
 $1 = sp + 16|0;
 $mask = sp;
 $pSrc = sp + 12|0;
 $i = sp + 8|0;
 HEAP32[$0>>2] = $pMaskSet;
 HEAP32[$1>>2] = $pS;
 $2 = $mask;
 $3 = $2;
 HEAP32[$3>>2] = 0;
 $4 = (($2) + 4)|0;
 $5 = $4;
 HEAP32[$5>>2] = 0;
 while(1) {
  $6 = HEAP32[$1>>2]|0;
  $7 = ($6|0)!=(0|0);
  if (!($7)) {
   break;
  }
  $8 = HEAP32[$1>>2]|0;
  $9 = (($8) + 32|0);
  $10 = HEAP32[$9>>2]|0;
  HEAP32[$pSrc>>2] = $10;
  $11 = HEAP32[$0>>2]|0;
  $12 = HEAP32[$1>>2]|0;
  $13 = HEAP32[$12>>2]|0;
  $14 = (_exprListTableUsage($11,$13)|0);
  $15 = tempRet0;
  $16 = $mask;
  $17 = $16;
  $18 = HEAP32[$17>>2]|0;
  $19 = (($16) + 4)|0;
  $20 = $19;
  $21 = HEAP32[$20>>2]|0;
  $22 = $18 | $14;
  $23 = $21 | $15;
  $24 = $mask;
  $25 = $24;
  HEAP32[$25>>2] = $22;
  $26 = (($24) + 4)|0;
  $27 = $26;
  HEAP32[$27>>2] = $23;
  $28 = HEAP32[$0>>2]|0;
  $29 = HEAP32[$1>>2]|0;
  $30 = (($29) + 40|0);
  $31 = HEAP32[$30>>2]|0;
  $32 = (_exprListTableUsage($28,$31)|0);
  $33 = tempRet0;
  $34 = $mask;
  $35 = $34;
  $36 = HEAP32[$35>>2]|0;
  $37 = (($34) + 4)|0;
  $38 = $37;
  $39 = HEAP32[$38>>2]|0;
  $40 = $36 | $32;
  $41 = $39 | $33;
  $42 = $mask;
  $43 = $42;
  HEAP32[$43>>2] = $40;
  $44 = (($42) + 4)|0;
  $45 = $44;
  HEAP32[$45>>2] = $41;
  $46 = HEAP32[$0>>2]|0;
  $47 = HEAP32[$1>>2]|0;
  $48 = (($47) + 48|0);
  $49 = HEAP32[$48>>2]|0;
  $50 = (_exprListTableUsage($46,$49)|0);
  $51 = tempRet0;
  $52 = $mask;
  $53 = $52;
  $54 = HEAP32[$53>>2]|0;
  $55 = (($52) + 4)|0;
  $56 = $55;
  $57 = HEAP32[$56>>2]|0;
  $58 = $54 | $50;
  $59 = $57 | $51;
  $60 = $mask;
  $61 = $60;
  HEAP32[$61>>2] = $58;
  $62 = (($60) + 4)|0;
  $63 = $62;
  HEAP32[$63>>2] = $59;
  $64 = HEAP32[$0>>2]|0;
  $65 = HEAP32[$1>>2]|0;
  $66 = (($65) + 36|0);
  $67 = HEAP32[$66>>2]|0;
  $68 = (_exprTableUsage($64,$67)|0);
  $69 = tempRet0;
  $70 = $mask;
  $71 = $70;
  $72 = HEAP32[$71>>2]|0;
  $73 = (($70) + 4)|0;
  $74 = $73;
  $75 = HEAP32[$74>>2]|0;
  $76 = $72 | $68;
  $77 = $75 | $69;
  $78 = $mask;
  $79 = $78;
  HEAP32[$79>>2] = $76;
  $80 = (($78) + 4)|0;
  $81 = $80;
  HEAP32[$81>>2] = $77;
  $82 = HEAP32[$0>>2]|0;
  $83 = HEAP32[$1>>2]|0;
  $84 = (($83) + 44|0);
  $85 = HEAP32[$84>>2]|0;
  $86 = (_exprTableUsage($82,$85)|0);
  $87 = tempRet0;
  $88 = $mask;
  $89 = $88;
  $90 = HEAP32[$89>>2]|0;
  $91 = (($88) + 4)|0;
  $92 = $91;
  $93 = HEAP32[$92>>2]|0;
  $94 = $90 | $86;
  $95 = $93 | $87;
  $96 = $mask;
  $97 = $96;
  HEAP32[$97>>2] = $94;
  $98 = (($96) + 4)|0;
  $99 = $98;
  HEAP32[$99>>2] = $95;
  $100 = HEAP32[$pSrc>>2]|0;
  $101 = ($100|0)!=(0|0);
  L4: do {
   if ($101) {
    HEAP32[$i>>2] = 0;
    while(1) {
     $102 = HEAP32[$i>>2]|0;
     $103 = HEAP32[$pSrc>>2]|0;
     $104 = HEAP32[$103>>2]|0;
     $105 = ($102|0)<($104|0);
     if (!($105)) {
      break L4;
     }
     $106 = HEAP32[$0>>2]|0;
     $107 = HEAP32[$i>>2]|0;
     $108 = HEAP32[$pSrc>>2]|0;
     $109 = (($108) + 8|0);
     $110 = (($109) + (($107*72)|0)|0);
     $111 = (($110) + 20|0);
     $112 = HEAP32[$111>>2]|0;
     $113 = (_exprSelectTableUsage($106,$112)|0);
     $114 = tempRet0;
     $115 = $mask;
     $116 = $115;
     $117 = HEAP32[$116>>2]|0;
     $118 = (($115) + 4)|0;
     $119 = $118;
     $120 = HEAP32[$119>>2]|0;
     $121 = $117 | $113;
     $122 = $120 | $114;
     $123 = $mask;
     $124 = $123;
     HEAP32[$124>>2] = $121;
     $125 = (($123) + 4)|0;
     $126 = $125;
     HEAP32[$126>>2] = $122;
     $127 = HEAP32[$0>>2]|0;
     $128 = HEAP32[$i>>2]|0;
     $129 = HEAP32[$pSrc>>2]|0;
     $130 = (($129) + 8|0);
     $131 = (($130) + (($128*72)|0)|0);
     $132 = (($131) + 44|0);
     $133 = HEAP32[$132>>2]|0;
     $134 = (_exprTableUsage($127,$133)|0);
     $135 = tempRet0;
     $136 = $mask;
     $137 = $136;
     $138 = HEAP32[$137>>2]|0;
     $139 = (($136) + 4)|0;
     $140 = $139;
     $141 = HEAP32[$140>>2]|0;
     $142 = $138 | $134;
     $143 = $141 | $135;
     $144 = $mask;
     $145 = $144;
     HEAP32[$145>>2] = $142;
     $146 = (($144) + 4)|0;
     $147 = $146;
     HEAP32[$147>>2] = $143;
     $148 = HEAP32[$i>>2]|0;
     $149 = (($148) + 1)|0;
     HEAP32[$i>>2] = $149;
    }
   }
  } while(0);
  $150 = HEAP32[$1>>2]|0;
  $151 = (($150) + 52|0);
  $152 = HEAP32[$151>>2]|0;
  HEAP32[$1>>2] = $152;
 }
 $153 = $mask;
 $154 = $153;
 $155 = HEAP32[$154>>2]|0;
 $156 = (($153) + 4)|0;
 $157 = $156;
 $158 = HEAP32[$157>>2]|0;
 tempRet0 = $158;
 STACKTOP = sp;return ($155|0);
}
function _sqlite3LogEstAdd($a,$b) {
 $a = $a|0;
 $b = $b|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $7 = 0, $8 = 0;
 var $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $1 = sp + 2|0;
 $2 = sp;
 HEAP16[$1>>1] = $a;
 HEAP16[$2>>1] = $b;
 $3 = HEAP16[$1>>1]|0;
 $4 = $3 << 16 >> 16;
 $5 = HEAP16[$2>>1]|0;
 $6 = $5 << 16 >> 16;
 $7 = ($4|0)>=($6|0);
 if ($7) {
  $8 = HEAP16[$1>>1]|0;
  $9 = $8 << 16 >> 16;
  $10 = HEAP16[$2>>1]|0;
  $11 = $10 << 16 >> 16;
  $12 = (($11) + 49)|0;
  $13 = ($9|0)>($12|0);
  $14 = HEAP16[$1>>1]|0;
  if ($13) {
   HEAP16[$0>>1] = $14;
   $60 = HEAP16[$0>>1]|0;
   STACKTOP = sp;return ($60|0);
  }
  $15 = $14 << 16 >> 16;
  $16 = HEAP16[$2>>1]|0;
  $17 = $16 << 16 >> 16;
  $18 = (($17) + 31)|0;
  $19 = ($15|0)>($18|0);
  $20 = HEAP16[$1>>1]|0;
  $21 = $20 << 16 >> 16;
  if ($19) {
   $22 = (($21) + 1)|0;
   $23 = $22&65535;
   HEAP16[$0>>1] = $23;
   $60 = HEAP16[$0>>1]|0;
   STACKTOP = sp;return ($60|0);
  } else {
   $24 = HEAP16[$1>>1]|0;
   $25 = $24 << 16 >> 16;
   $26 = HEAP16[$2>>1]|0;
   $27 = $26 << 16 >> 16;
   $28 = (($25) - ($27))|0;
   $29 = (24056 + ($28)|0);
   $30 = HEAP8[$29>>0]|0;
   $31 = $30&255;
   $32 = (($21) + ($31))|0;
   $33 = $32&65535;
   HEAP16[$0>>1] = $33;
   $60 = HEAP16[$0>>1]|0;
   STACKTOP = sp;return ($60|0);
  }
 } else {
  $34 = HEAP16[$2>>1]|0;
  $35 = $34 << 16 >> 16;
  $36 = HEAP16[$1>>1]|0;
  $37 = $36 << 16 >> 16;
  $38 = (($37) + 49)|0;
  $39 = ($35|0)>($38|0);
  $40 = HEAP16[$2>>1]|0;
  if ($39) {
   HEAP16[$0>>1] = $40;
   $60 = HEAP16[$0>>1]|0;
   STACKTOP = sp;return ($60|0);
  }
  $41 = $40 << 16 >> 16;
  $42 = HEAP16[$1>>1]|0;
  $43 = $42 << 16 >> 16;
  $44 = (($43) + 31)|0;
  $45 = ($41|0)>($44|0);
  $46 = HEAP16[$2>>1]|0;
  $47 = $46 << 16 >> 16;
  if ($45) {
   $48 = (($47) + 1)|0;
   $49 = $48&65535;
   HEAP16[$0>>1] = $49;
   $60 = HEAP16[$0>>1]|0;
   STACKTOP = sp;return ($60|0);
  } else {
   $50 = HEAP16[$2>>1]|0;
   $51 = $50 << 16 >> 16;
   $52 = HEAP16[$1>>1]|0;
   $53 = $52 << 16 >> 16;
   $54 = (($51) - ($53))|0;
   $55 = (24056 + ($54)|0);
   $56 = HEAP8[$55>>0]|0;
   $57 = $56&255;
   $58 = (($47) + ($57))|0;
   $59 = $58&65535;
   HEAP16[$0>>1] = $59;
   $60 = HEAP16[$0>>1]|0;
   STACKTOP = sp;return ($60|0);
  }
 }
 return 0|0;
}
function _wherePathSatisfiesOrderBy($pWInfo,$pOrderBy,$pPath,$wctrlFlags,$nLoop,$pLast,$pRevMask) {
 $pWInfo = $pWInfo|0;
 $pOrderBy = $pOrderBy|0;
 $pPath = $pPath|0;
 $wctrlFlags = $wctrlFlags|0;
 $nLoop = $nLoop|0;
 $pLast = $pLast|0;
 $pRevMask = $pRevMask|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0;
 var $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0, $188 = 0;
 var $189 = 0, $19 = 0, $190 = 0, $191 = 0, $192 = 0, $193 = 0, $194 = 0, $195 = 0, $196 = 0, $197 = 0, $198 = 0, $199 = 0, $2 = 0, $20 = 0, $200 = 0, $201 = 0, $202 = 0, $203 = 0, $204 = 0, $205 = 0;
 var $206 = 0, $207 = 0, $208 = 0, $209 = 0, $21 = 0, $210 = 0, $211 = 0, $212 = 0, $213 = 0, $214 = 0, $215 = 0, $216 = 0, $217 = 0, $218 = 0, $219 = 0, $22 = 0, $220 = 0, $221 = 0, $222 = 0, $223 = 0;
 var $224 = 0, $225 = 0, $226 = 0, $227 = 0, $228 = 0, $229 = 0, $23 = 0, $230 = 0, $231 = 0, $232 = 0, $233 = 0, $234 = 0, $235 = 0, $236 = 0, $237 = 0, $238 = 0, $239 = 0, $24 = 0, $240 = 0, $241 = 0;
 var $242 = 0, $243 = 0, $244 = 0, $245 = 0, $246 = 0, $247 = 0, $248 = 0, $249 = 0, $25 = 0, $250 = 0, $251 = 0, $252 = 0, $253 = 0, $254 = 0, $255 = 0, $256 = 0, $257 = 0, $258 = 0, $259 = 0, $26 = 0;
 var $260 = 0, $261 = 0, $262 = 0, $263 = 0, $264 = 0, $265 = 0, $266 = 0, $267 = 0, $268 = 0, $269 = 0, $27 = 0, $270 = 0, $271 = 0, $272 = 0, $273 = 0, $274 = 0, $275 = 0, $276 = 0, $277 = 0, $278 = 0;
 var $279 = 0, $28 = 0, $280 = 0, $281 = 0, $282 = 0, $283 = 0, $284 = 0, $285 = 0, $286 = 0, $287 = 0, $288 = 0, $289 = 0, $29 = 0, $290 = 0, $291 = 0, $292 = 0, $293 = 0, $294 = 0, $295 = 0, $296 = 0;
 var $297 = 0, $298 = 0, $299 = 0, $3 = 0, $30 = 0, $300 = 0, $301 = 0, $302 = 0, $303 = 0, $304 = 0, $305 = 0, $306 = 0, $307 = 0, $308 = 0, $309 = 0, $31 = 0, $310 = 0, $311 = 0, $312 = 0, $313 = 0;
 var $314 = 0, $315 = 0, $316 = 0, $317 = 0, $318 = 0, $319 = 0, $32 = 0, $320 = 0, $321 = 0, $322 = 0, $323 = 0, $324 = 0, $325 = 0, $326 = 0, $327 = 0, $328 = 0, $329 = 0, $33 = 0, $330 = 0, $331 = 0;
 var $332 = 0, $333 = 0, $334 = 0, $335 = 0, $336 = 0, $337 = 0, $338 = 0, $339 = 0, $34 = 0, $340 = 0, $341 = 0, $342 = 0, $343 = 0, $344 = 0, $345 = 0, $346 = 0, $347 = 0, $348 = 0, $349 = 0, $35 = 0;
 var $350 = 0, $351 = 0, $352 = 0, $353 = 0, $354 = 0, $355 = 0, $356 = 0, $357 = 0, $358 = 0, $359 = 0, $36 = 0, $360 = 0, $361 = 0, $362 = 0, $363 = 0, $364 = 0, $365 = 0, $366 = 0, $367 = 0, $368 = 0;
 var $369 = 0, $37 = 0, $370 = 0, $371 = 0, $372 = 0, $373 = 0, $374 = 0, $375 = 0, $376 = 0, $377 = 0, $378 = 0, $379 = 0, $38 = 0, $380 = 0, $381 = 0, $382 = 0, $383 = 0, $384 = 0, $385 = 0, $386 = 0;
 var $387 = 0, $388 = 0, $389 = 0, $39 = 0, $390 = 0, $391 = 0, $392 = 0, $393 = 0, $394 = 0, $395 = 0, $396 = 0, $397 = 0, $398 = 0, $399 = 0, $4 = 0, $40 = 0, $400 = 0, $401 = 0, $402 = 0, $403 = 0;
 var $404 = 0, $405 = 0, $406 = 0, $407 = 0, $408 = 0, $409 = 0, $41 = 0, $410 = 0, $411 = 0, $412 = 0, $413 = 0, $414 = 0, $415 = 0, $416 = 0, $417 = 0, $418 = 0, $419 = 0, $42 = 0, $420 = 0, $421 = 0;
 var $422 = 0, $423 = 0, $424 = 0, $425 = 0, $426 = 0, $427 = 0, $428 = 0, $429 = 0, $43 = 0, $430 = 0, $431 = 0, $432 = 0, $433 = 0, $434 = 0, $435 = 0, $436 = 0, $437 = 0, $438 = 0, $439 = 0, $44 = 0;
 var $440 = 0, $441 = 0, $442 = 0, $443 = 0, $444 = 0, $445 = 0, $446 = 0, $447 = 0, $448 = 0, $449 = 0, $45 = 0, $450 = 0, $451 = 0, $452 = 0, $453 = 0, $454 = 0, $455 = 0, $456 = 0, $457 = 0, $458 = 0;
 var $459 = 0, $46 = 0, $460 = 0, $461 = 0, $462 = 0, $463 = 0, $464 = 0, $465 = 0, $466 = 0, $467 = 0, $468 = 0, $469 = 0, $47 = 0, $470 = 0, $471 = 0, $472 = 0, $473 = 0, $474 = 0, $475 = 0, $476 = 0;
 var $477 = 0, $478 = 0, $479 = 0, $48 = 0, $480 = 0, $481 = 0, $482 = 0, $483 = 0, $484 = 0, $485 = 0, $486 = 0, $487 = 0, $488 = 0, $489 = 0, $49 = 0, $490 = 0, $491 = 0, $492 = 0, $493 = 0, $494 = 0;
 var $495 = 0, $496 = 0, $497 = 0, $498 = 0, $499 = 0, $5 = 0, $50 = 0, $500 = 0, $501 = 0, $502 = 0, $503 = 0, $504 = 0, $505 = 0, $506 = 0, $507 = 0, $508 = 0, $509 = 0, $51 = 0, $510 = 0, $511 = 0;
 var $512 = 0, $513 = 0, $514 = 0, $515 = 0, $516 = 0, $517 = 0, $518 = 0, $519 = 0, $52 = 0, $520 = 0, $521 = 0, $522 = 0, $523 = 0, $524 = 0, $525 = 0, $526 = 0, $527 = 0, $528 = 0, $529 = 0, $53 = 0;
 var $530 = 0, $531 = 0, $532 = 0, $533 = 0, $534 = 0, $535 = 0, $536 = 0, $537 = 0, $538 = 0, $539 = 0, $54 = 0, $540 = 0, $541 = 0, $542 = 0, $543 = 0, $544 = 0, $545 = 0, $546 = 0, $547 = 0, $548 = 0;
 var $549 = 0, $55 = 0, $550 = 0, $551 = 0, $552 = 0, $553 = 0, $554 = 0, $555 = 0, $556 = 0, $557 = 0, $558 = 0, $559 = 0, $56 = 0, $560 = 0, $561 = 0, $562 = 0, $563 = 0, $564 = 0, $565 = 0, $566 = 0;
 var $567 = 0, $568 = 0, $569 = 0, $57 = 0, $570 = 0, $571 = 0, $572 = 0, $573 = 0, $574 = 0, $575 = 0, $576 = 0, $577 = 0, $578 = 0, $579 = 0, $58 = 0, $580 = 0, $581 = 0, $582 = 0, $583 = 0, $584 = 0;
 var $585 = 0, $586 = 0, $587 = 0, $588 = 0, $589 = 0, $59 = 0, $590 = 0, $591 = 0, $592 = 0, $593 = 0, $594 = 0, $595 = 0, $596 = 0, $597 = 0, $598 = 0, $599 = 0, $6 = 0, $60 = 0, $600 = 0, $601 = 0;
 var $602 = 0, $603 = 0, $604 = 0, $605 = 0, $606 = 0, $607 = 0, $608 = 0, $609 = 0, $61 = 0, $610 = 0, $611 = 0, $612 = 0, $613 = 0, $614 = 0, $615 = 0, $616 = 0, $617 = 0, $618 = 0, $619 = 0, $62 = 0;
 var $620 = 0, $621 = 0, $622 = 0, $623 = 0, $624 = 0, $625 = 0, $626 = 0, $627 = 0, $628 = 0, $629 = 0, $63 = 0, $630 = 0, $631 = 0, $632 = 0, $633 = 0, $634 = 0, $635 = 0, $636 = 0, $637 = 0, $638 = 0;
 var $639 = 0, $64 = 0, $640 = 0, $641 = 0, $642 = 0, $643 = 0, $644 = 0, $645 = 0, $646 = 0, $647 = 0, $648 = 0, $649 = 0, $65 = 0, $650 = 0, $651 = 0, $652 = 0, $653 = 0, $654 = 0, $655 = 0, $656 = 0;
 var $657 = 0, $658 = 0, $659 = 0, $66 = 0, $660 = 0, $661 = 0, $662 = 0, $663 = 0, $664 = 0, $665 = 0, $666 = 0, $667 = 0, $668 = 0, $669 = 0, $67 = 0, $670 = 0, $671 = 0, $672 = 0, $673 = 0, $674 = 0;
 var $675 = 0, $676 = 0, $677 = 0, $678 = 0, $679 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0;
 var $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0;
 var $bOnce = 0, $db = 0, $distinctColumns = 0, $i = 0, $iColumn = 0, $iCur = 0, $iLoop = 0, $isMatch = 0, $isOrderDistinct = 0, $j = 0, $m = 0, $mTerm = 0, $nColumn = 0, $nKeyCol = 0, $nOrderBy = 0, $obDone = 0, $obSat = 0, $or$cond = 0, $orderDistinctMask = 0, $p = 0;
 var $pColl = 0, $pIndex = 0, $pLoop = 0, $pOBExpr = 0, $pTerm = 0, $ready = 0, $rev = 0, $revIdx = 0, $revSet = 0, $z1 = 0, $z2 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 144|0;
 $0 = sp + 141|0;
 $1 = sp + 120|0;
 $2 = sp + 116|0;
 $3 = sp + 112|0;
 $4 = sp + 132|0;
 $5 = sp + 130|0;
 $6 = sp + 108|0;
 $7 = sp + 104|0;
 $revSet = sp + 140|0;
 $rev = sp + 139|0;
 $revIdx = sp + 138|0;
 $isOrderDistinct = sp + 137|0;
 $distinctColumns = sp + 136|0;
 $isMatch = sp + 135|0;
 $nKeyCol = sp + 128|0;
 $nColumn = sp + 126|0;
 $nOrderBy = sp + 124|0;
 $iLoop = sp + 100|0;
 $i = sp + 96|0;
 $j = sp + 92|0;
 $iCur = sp + 88|0;
 $iColumn = sp + 84|0;
 $pLoop = sp + 80|0;
 $pTerm = sp + 76|0;
 $pOBExpr = sp + 72|0;
 $pColl = sp + 68|0;
 $pIndex = sp + 64|0;
 $db = sp + 60|0;
 $obSat = sp + 40|0;
 $obDone = sp + 32|0;
 $orderDistinctMask = sp + 24|0;
 $ready = sp + 16|0;
 $z1 = sp + 56|0;
 $z2 = sp + 52|0;
 $bOnce = sp + 134|0;
 $p = sp + 48|0;
 $mTerm = sp + 8|0;
 $m = sp;
 HEAP32[$1>>2] = $pWInfo;
 HEAP32[$2>>2] = $pOrderBy;
 HEAP32[$3>>2] = $pPath;
 HEAP16[$4>>1] = $wctrlFlags;
 HEAP16[$5>>1] = $nLoop;
 HEAP32[$6>>2] = $pLast;
 HEAP32[$7>>2] = $pRevMask;
 HEAP32[$pLoop>>2] = 0;
 $8 = HEAP32[$1>>2]|0;
 $9 = HEAP32[$8>>2]|0;
 $10 = HEAP32[$9>>2]|0;
 HEAP32[$db>>2] = $10;
 $11 = $obSat;
 $12 = $11;
 HEAP32[$12>>2] = 0;
 $13 = (($11) + 4)|0;
 $14 = $13;
 HEAP32[$14>>2] = 0;
 $15 = HEAP16[$5>>1]|0;
 $16 = $15&65535;
 $17 = ($16|0)!=(0);
 if ($17) {
  $18 = HEAP32[$db>>2]|0;
  $19 = (($18) + 60|0);
  $20 = HEAP16[$19>>1]|0;
  $21 = $20&65535;
  $22 = $21 & 128;
  $23 = ($22|0)!=(0);
  if ($23) {
   HEAP8[$0>>0] = 0;
   $679 = HEAP8[$0>>0]|0;
   STACKTOP = sp;return ($679|0);
  }
 }
 $24 = HEAP32[$2>>2]|0;
 $25 = HEAP32[$24>>2]|0;
 $26 = $25&65535;
 HEAP16[$nOrderBy>>1] = $26;
 $27 = HEAP16[$nOrderBy>>1]|0;
 $28 = $27&65535;
 $29 = ($28|0)>(63);
 if ($29) {
  HEAP8[$0>>0] = 0;
  $679 = HEAP8[$0>>0]|0;
  STACKTOP = sp;return ($679|0);
 }
 HEAP8[$isOrderDistinct>>0] = 1;
 $30 = HEAP16[$nOrderBy>>1]|0;
 $31 = $30&65535;
 $32 = (_bitshift64Shl(1,0,($31|0))|0);
 $33 = tempRet0;
 $34 = (_i64Subtract(($32|0),($33|0),1,0)|0);
 $35 = tempRet0;
 $36 = $obDone;
 $37 = $36;
 HEAP32[$37>>2] = $34;
 $38 = (($36) + 4)|0;
 $39 = $38;
 HEAP32[$39>>2] = $35;
 $40 = $orderDistinctMask;
 $41 = $40;
 HEAP32[$41>>2] = 0;
 $42 = (($40) + 4)|0;
 $43 = $42;
 HEAP32[$43>>2] = 0;
 $44 = $ready;
 $45 = $44;
 HEAP32[$45>>2] = 0;
 $46 = (($44) + 4)|0;
 $47 = $46;
 HEAP32[$47>>2] = 0;
 HEAP32[$iLoop>>2] = 0;
 while(1) {
  $48 = HEAP8[$isOrderDistinct>>0]|0;
  $49 = $48&255;
  $50 = ($49|0)!=(0);
  if (!($50)) {
   break;
  }
  $51 = $obSat;
  $52 = $51;
  $53 = HEAP32[$52>>2]|0;
  $54 = (($51) + 4)|0;
  $55 = $54;
  $56 = HEAP32[$55>>2]|0;
  $57 = $obDone;
  $58 = $57;
  $59 = HEAP32[$58>>2]|0;
  $60 = (($57) + 4)|0;
  $61 = $60;
  $62 = HEAP32[$61>>2]|0;
  $63 = ($56>>>0)<($62>>>0);
  $64 = ($53>>>0)<($59>>>0);
  $65 = ($56|0)==($62|0);
  $66 = $65 & $64;
  $67 = $63 | $66;
  if (!($67)) {
   break;
  }
  $68 = HEAP32[$iLoop>>2]|0;
  $69 = HEAP16[$5>>1]|0;
  $70 = $69&65535;
  $71 = ($68|0)<=($70|0);
  if (!($71)) {
   break;
  }
  $72 = HEAP32[$iLoop>>2]|0;
  $73 = ($72|0)>(0);
  if ($73) {
   $74 = HEAP32[$pLoop>>2]|0;
   $75 = (($74) + 8|0);
   $76 = $75;
   $77 = $76;
   $78 = HEAP32[$77>>2]|0;
   $79 = (($76) + 4)|0;
   $80 = $79;
   $81 = HEAP32[$80>>2]|0;
   $82 = $ready;
   $83 = $82;
   $84 = HEAP32[$83>>2]|0;
   $85 = (($82) + 4)|0;
   $86 = $85;
   $87 = HEAP32[$86>>2]|0;
   $88 = $84 | $78;
   $89 = $87 | $81;
   $90 = $ready;
   $91 = $90;
   HEAP32[$91>>2] = $88;
   $92 = (($90) + 4)|0;
   $93 = $92;
   HEAP32[$93>>2] = $89;
  }
  $94 = HEAP32[$iLoop>>2]|0;
  $95 = HEAP16[$5>>1]|0;
  $96 = $95&65535;
  $97 = ($94|0)<($96|0);
  if ($97) {
   $98 = HEAP32[$iLoop>>2]|0;
   $99 = HEAP32[$3>>2]|0;
   $100 = (($99) + 24|0);
   $101 = HEAP32[$100>>2]|0;
   $102 = (($101) + ($98<<2)|0);
   $103 = HEAP32[$102>>2]|0;
   $105 = $103;
  } else {
   $104 = HEAP32[$6>>2]|0;
   $105 = $104;
  }
  HEAP32[$pLoop>>2] = $105;
  $106 = HEAP32[$pLoop>>2]|0;
  $107 = (($106) + 36|0);
  $108 = HEAP32[$107>>2]|0;
  $109 = $108 & 1024;
  $110 = ($109|0)!=(0);
  $111 = HEAP32[$pLoop>>2]|0;
  if ($110) {
   label = 16;
   break;
  }
  $126 = (($111) + 16|0);
  $127 = HEAP8[$126>>0]|0;
  $128 = $127&255;
  $129 = HEAP32[$1>>2]|0;
  $130 = (($129) + 4|0);
  $131 = HEAP32[$130>>2]|0;
  $132 = (($131) + 8|0);
  $133 = (($132) + (($128*72)|0)|0);
  $134 = (($133) + 40|0);
  $135 = HEAP32[$134>>2]|0;
  HEAP32[$iCur>>2] = $135;
  HEAP32[$i>>2] = 0;
  while(1) {
   $136 = HEAP32[$i>>2]|0;
   $137 = HEAP16[$nOrderBy>>1]|0;
   $138 = $137&65535;
   $139 = ($136|0)<($138|0);
   if (!($139)) {
    break;
   }
   $140 = HEAP32[$i>>2]|0;
   $141 = (_bitshift64Shl(1,0,($140|0))|0);
   $142 = tempRet0;
   $143 = $obSat;
   $144 = $143;
   $145 = HEAP32[$144>>2]|0;
   $146 = (($143) + 4)|0;
   $147 = $146;
   $148 = HEAP32[$147>>2]|0;
   $149 = $141 & $145;
   $150 = $142 & $148;
   $151 = ($149|0)!=(0);
   $152 = ($150|0)!=(0);
   $153 = $151 | $152;
   do {
    if (!($153)) {
     $154 = HEAP32[$i>>2]|0;
     $155 = HEAP32[$2>>2]|0;
     $156 = (($155) + 4|0);
     $157 = HEAP32[$156>>2]|0;
     $158 = (($157) + (($154*20)|0)|0);
     $159 = HEAP32[$158>>2]|0;
     $160 = (_sqlite3ExprSkipCollate($159)|0);
     HEAP32[$pOBExpr>>2] = $160;
     $161 = HEAP32[$pOBExpr>>2]|0;
     $162 = HEAP8[$161>>0]|0;
     $163 = $162&255;
     $164 = ($163|0)!=(154);
     if (!($164)) {
      $165 = HEAP32[$pOBExpr>>2]|0;
      $166 = (($165) + 28|0);
      $167 = HEAP32[$166>>2]|0;
      $168 = HEAP32[$iCur>>2]|0;
      $169 = ($167|0)!=($168|0);
      if (!($169)) {
       $170 = HEAP32[$1>>2]|0;
       $171 = (($170) + 328|0);
       $172 = HEAP32[$iCur>>2]|0;
       $173 = HEAP32[$pOBExpr>>2]|0;
       $174 = (($173) + 32|0);
       $175 = HEAP16[$174>>1]|0;
       $176 = $175 << 16 >> 16;
       $177 = $ready;
       $178 = $177;
       $179 = HEAP32[$178>>2]|0;
       $180 = (($177) + 4)|0;
       $181 = $180;
       $182 = HEAP32[$181>>2]|0;
       $183 = $179 ^ -1;
       $184 = $182 ^ -1;
       $185 = (_findTerm($171,$172,$176,$183,$184,130,0)|0);
       HEAP32[$pTerm>>2] = $185;
       $186 = HEAP32[$pTerm>>2]|0;
       $187 = ($186|0)==(0|0);
       if (!($187)) {
        $188 = HEAP32[$pTerm>>2]|0;
        $189 = (($188) + 18|0);
        $190 = HEAP16[$189>>1]|0;
        $191 = $190&65535;
        $192 = $191 & 2;
        $193 = ($192|0)!=(0);
        if ($193) {
         $194 = HEAP32[$pOBExpr>>2]|0;
         $195 = (($194) + 32|0);
         $196 = HEAP16[$195>>1]|0;
         $197 = $196 << 16 >> 16;
         $198 = ($197|0)>=(0);
         if ($198) {
          $199 = HEAP32[$1>>2]|0;
          $200 = HEAP32[$199>>2]|0;
          $201 = HEAP32[$i>>2]|0;
          $202 = HEAP32[$2>>2]|0;
          $203 = (($202) + 4|0);
          $204 = HEAP32[$203>>2]|0;
          $205 = (($204) + (($201*20)|0)|0);
          $206 = HEAP32[$205>>2]|0;
          $207 = (_sqlite3ExprCollSeq($200,$206)|0);
          HEAP32[$pColl>>2] = $207;
          $208 = HEAP32[$pColl>>2]|0;
          $209 = ($208|0)!=(0|0);
          if (!($209)) {
           $210 = HEAP32[$db>>2]|0;
           $211 = (($210) + 8|0);
           $212 = HEAP32[$211>>2]|0;
           HEAP32[$pColl>>2] = $212;
          }
          $213 = HEAP32[$pColl>>2]|0;
          $214 = HEAP32[$213>>2]|0;
          HEAP32[$z1>>2] = $214;
          $215 = HEAP32[$1>>2]|0;
          $216 = HEAP32[$215>>2]|0;
          $217 = HEAP32[$pTerm>>2]|0;
          $218 = HEAP32[$217>>2]|0;
          $219 = (_sqlite3ExprCollSeq($216,$218)|0);
          HEAP32[$pColl>>2] = $219;
          $220 = HEAP32[$pColl>>2]|0;
          $221 = ($220|0)!=(0|0);
          if (!($221)) {
           $222 = HEAP32[$db>>2]|0;
           $223 = (($222) + 8|0);
           $224 = HEAP32[$223>>2]|0;
           HEAP32[$pColl>>2] = $224;
          }
          $225 = HEAP32[$pColl>>2]|0;
          $226 = HEAP32[$225>>2]|0;
          HEAP32[$z2>>2] = $226;
          $227 = HEAP32[$z1>>2]|0;
          $228 = HEAP32[$z2>>2]|0;
          $229 = (_sqlite3_stricmp($227,$228)|0);
          $230 = ($229|0)!=(0);
          if ($230) {
           break;
          }
         }
        }
        $231 = HEAP32[$i>>2]|0;
        $232 = (_bitshift64Shl(1,0,($231|0))|0);
        $233 = tempRet0;
        $234 = $obSat;
        $235 = $234;
        $236 = HEAP32[$235>>2]|0;
        $237 = (($234) + 4)|0;
        $238 = $237;
        $239 = HEAP32[$238>>2]|0;
        $240 = $236 | $232;
        $241 = $239 | $233;
        $242 = $obSat;
        $243 = $242;
        HEAP32[$243>>2] = $240;
        $244 = (($242) + 4)|0;
        $245 = $244;
        HEAP32[$245>>2] = $241;
       }
      }
     }
    }
   } while(0);
   $246 = HEAP32[$i>>2]|0;
   $247 = (($246) + 1)|0;
   HEAP32[$i>>2] = $247;
  }
  $248 = HEAP32[$pLoop>>2]|0;
  $249 = (($248) + 36|0);
  $250 = HEAP32[$249>>2]|0;
  $251 = $250 & 4096;
  $252 = ($251|0)==(0);
  if ($252) {
   $253 = HEAP32[$pLoop>>2]|0;
   $254 = (($253) + 36|0);
   $255 = HEAP32[$254>>2]|0;
   $256 = $255 & 256;
   $257 = ($256|0)!=(0);
   if ($257) {
    HEAP32[$pIndex>>2] = 0;
    HEAP16[$nKeyCol>>1] = 0;
    HEAP16[$nColumn>>1] = 1;
   } else {
    $258 = HEAP32[$pLoop>>2]|0;
    $259 = (($258) + 24|0);
    $260 = (($259) + 4|0);
    $261 = HEAP32[$260>>2]|0;
    HEAP32[$pIndex>>2] = $261;
    $262 = ($261|0)==(0|0);
    if ($262) {
     label = 38;
     break;
    }
    $263 = HEAP32[$pIndex>>2]|0;
    $264 = (($263) + 55|0);
    $265 = HEAP8[$264>>0]|0;
    $266 = ($265&255) >>> 2;
    $267 = $266 & 1;
    $268 = $267&255;
    $269 = ($268|0)!=(0);
    if ($269) {
     label = 38;
     break;
    }
    $270 = HEAP32[$pIndex>>2]|0;
    $271 = (($270) + 50|0);
    $272 = HEAP16[$271>>1]|0;
    HEAP16[$nKeyCol>>1] = $272;
    $273 = HEAP32[$pIndex>>2]|0;
    $274 = (($273) + 52|0);
    $275 = HEAP16[$274>>1]|0;
    HEAP16[$nColumn>>1] = $275;
    $276 = HEAP32[$pIndex>>2]|0;
    $277 = (($276) + 54|0);
    $278 = HEAP8[$277>>0]|0;
    $279 = $278&255;
    $280 = ($279|0)!=(0);
    $281 = $280&1;
    $282 = $281&255;
    HEAP8[$isOrderDistinct>>0] = $282;
   }
   HEAP8[$revSet>>0] = 0;
   HEAP8[$rev>>0] = 0;
   HEAP8[$distinctColumns>>0] = 0;
   HEAP32[$j>>2] = 0;
   while(1) {
    $283 = HEAP32[$j>>2]|0;
    $284 = HEAP16[$nColumn>>1]|0;
    $285 = $284&65535;
    $286 = ($283|0)<($285|0);
    if (!($286)) {
     break;
    }
    $287 = HEAP32[$j>>2]|0;
    $288 = HEAP32[$pLoop>>2]|0;
    $289 = (($288) + 24|0);
    $290 = HEAP16[$289>>1]|0;
    $291 = $290&65535;
    $292 = ($287|0)<($291|0);
    if ($292) {
     $293 = HEAP32[$pLoop>>2]|0;
     $294 = (($293) + 24|0);
     $295 = (($294) + 2|0);
     $296 = HEAP16[$295>>1]|0;
     $297 = $296&65535;
     $298 = ($297|0)==(0);
     if ($298) {
      $299 = HEAP32[$j>>2]|0;
      $300 = HEAP32[$pLoop>>2]|0;
      $301 = (($300) + 44|0);
      $302 = HEAP32[$301>>2]|0;
      $303 = (($302) + ($299<<2)|0);
      $304 = HEAP32[$303>>2]|0;
      $305 = (($304) + 18|0);
      $306 = HEAP16[$305>>1]|0;
      $307 = $306&65535;
      HEAP32[$i>>2] = $307;
      $308 = $307 & 130;
      $309 = ($308|0)!=(0);
      if ($309) {
       $310 = HEAP32[$i>>2]|0;
       $311 = $310 & 128;
       $312 = ($311|0)!=(0);
       if ($312) {
        HEAP8[$isOrderDistinct>>0] = 0;
       }
      } else {
       label = 47;
      }
     } else {
      label = 47;
     }
    } else {
     label = 47;
    }
    if ((label|0) == 47) {
     label = 0;
     $313 = HEAP32[$pIndex>>2]|0;
     $314 = ($313|0)!=(0|0);
     if ($314) {
      $315 = HEAP32[$j>>2]|0;
      $316 = HEAP32[$pIndex>>2]|0;
      $317 = (($316) + 4|0);
      $318 = HEAP32[$317>>2]|0;
      $319 = (($318) + ($315<<1)|0);
      $320 = HEAP16[$319>>1]|0;
      $321 = $320 << 16 >> 16;
      HEAP32[$iColumn>>2] = $321;
      $322 = HEAP32[$j>>2]|0;
      $323 = HEAP32[$pIndex>>2]|0;
      $324 = (($323) + 28|0);
      $325 = HEAP32[$324>>2]|0;
      $326 = (($325) + ($322)|0);
      $327 = HEAP8[$326>>0]|0;
      HEAP8[$revIdx>>0] = $327;
      $328 = HEAP32[$iColumn>>2]|0;
      $329 = HEAP32[$pIndex>>2]|0;
      $330 = (($329) + 12|0);
      $331 = HEAP32[$330>>2]|0;
      $332 = (($331) + 36|0);
      $333 = HEAP16[$332>>1]|0;
      $334 = $333 << 16 >> 16;
      $335 = ($328|0)==($334|0);
      if ($335) {
       HEAP32[$iColumn>>2] = -1;
      }
     } else {
      HEAP32[$iColumn>>2] = -1;
      HEAP8[$revIdx>>0] = 0;
     }
     $336 = HEAP8[$isOrderDistinct>>0]|0;
     $337 = $336&255;
     $338 = ($337|0)!=(0);
     $339 = HEAP32[$iColumn>>2]|0;
     $340 = ($339|0)>=(0);
     $or$cond = $338 & $340;
     do {
      if ($or$cond) {
       $341 = HEAP32[$j>>2]|0;
       $342 = HEAP32[$pLoop>>2]|0;
       $343 = (($342) + 24|0);
       $344 = HEAP16[$343>>1]|0;
       $345 = $344&65535;
       $346 = ($341|0)>=($345|0);
       if ($346) {
        $347 = HEAP32[$iColumn>>2]|0;
        $348 = HEAP32[$pIndex>>2]|0;
        $349 = (($348) + 12|0);
        $350 = HEAP32[$349>>2]|0;
        $351 = (($350) + 4|0);
        $352 = HEAP32[$351>>2]|0;
        $353 = (($352) + (($347*24)|0)|0);
        $354 = (($353) + 20|0);
        $355 = HEAP8[$354>>0]|0;
        $356 = $355&255;
        $357 = ($356|0)==(0);
        if (!($357)) {
         break;
        }
        HEAP8[$isOrderDistinct>>0] = 0;
       }
      }
     } while(0);
     HEAP8[$bOnce>>0] = 1;
     HEAP8[$isMatch>>0] = 0;
     HEAP32[$i>>2] = 0;
     L71: while(1) {
      $358 = HEAP8[$bOnce>>0]|0;
      $359 = $358&255;
      $360 = ($359|0)!=(0);
      if (!($360)) {
       break;
      }
      $361 = HEAP32[$i>>2]|0;
      $362 = HEAP16[$nOrderBy>>1]|0;
      $363 = $362&65535;
      $364 = ($361|0)<($363|0);
      if (!($364)) {
       break;
      }
      $365 = HEAP32[$i>>2]|0;
      $366 = (_bitshift64Shl(1,0,($365|0))|0);
      $367 = tempRet0;
      $368 = $obSat;
      $369 = $368;
      $370 = HEAP32[$369>>2]|0;
      $371 = (($368) + 4)|0;
      $372 = $371;
      $373 = HEAP32[$372>>2]|0;
      $374 = $366 & $370;
      $375 = $367 & $373;
      $376 = ($374|0)!=(0);
      $377 = ($375|0)!=(0);
      $378 = $376 | $377;
      do {
       if (!($378)) {
        $379 = HEAP32[$i>>2]|0;
        $380 = HEAP32[$2>>2]|0;
        $381 = (($380) + 4|0);
        $382 = HEAP32[$381>>2]|0;
        $383 = (($382) + (($379*20)|0)|0);
        $384 = HEAP32[$383>>2]|0;
        $385 = (_sqlite3ExprSkipCollate($384)|0);
        HEAP32[$pOBExpr>>2] = $385;
        $386 = HEAP16[$4>>1]|0;
        $387 = $386&65535;
        $388 = $387 & 768;
        $389 = ($388|0)==(0);
        if ($389) {
         HEAP8[$bOnce>>0] = 0;
        }
        $390 = HEAP32[$pOBExpr>>2]|0;
        $391 = HEAP8[$390>>0]|0;
        $392 = $391&255;
        $393 = ($392|0)!=(154);
        if ($393) {
         break;
        }
        $394 = HEAP32[$pOBExpr>>2]|0;
        $395 = (($394) + 28|0);
        $396 = HEAP32[$395>>2]|0;
        $397 = HEAP32[$iCur>>2]|0;
        $398 = ($396|0)!=($397|0);
        if ($398) {
         break;
        }
        $399 = HEAP32[$pOBExpr>>2]|0;
        $400 = (($399) + 32|0);
        $401 = HEAP16[$400>>1]|0;
        $402 = $401 << 16 >> 16;
        $403 = HEAP32[$iColumn>>2]|0;
        $404 = ($402|0)!=($403|0);
        if ($404) {
         break;
        }
        $405 = HEAP32[$iColumn>>2]|0;
        $406 = ($405|0)>=(0);
        if (!($406)) {
         label = 68;
         break L71;
        }
        $407 = HEAP32[$1>>2]|0;
        $408 = HEAP32[$407>>2]|0;
        $409 = HEAP32[$i>>2]|0;
        $410 = HEAP32[$2>>2]|0;
        $411 = (($410) + 4|0);
        $412 = HEAP32[$411>>2]|0;
        $413 = (($412) + (($409*20)|0)|0);
        $414 = HEAP32[$413>>2]|0;
        $415 = (_sqlite3ExprCollSeq($408,$414)|0);
        HEAP32[$pColl>>2] = $415;
        $416 = HEAP32[$pColl>>2]|0;
        $417 = ($416|0)!=(0|0);
        if (!($417)) {
         $418 = HEAP32[$db>>2]|0;
         $419 = (($418) + 8|0);
         $420 = HEAP32[$419>>2]|0;
         HEAP32[$pColl>>2] = $420;
        }
        $421 = HEAP32[$pColl>>2]|0;
        $422 = HEAP32[$421>>2]|0;
        $423 = HEAP32[$j>>2]|0;
        $424 = HEAP32[$pIndex>>2]|0;
        $425 = (($424) + 32|0);
        $426 = HEAP32[$425>>2]|0;
        $427 = (($426) + ($423<<2)|0);
        $428 = HEAP32[$427>>2]|0;
        $429 = (_sqlite3_stricmp($422,$428)|0);
        $430 = ($429|0)!=(0);
        if (!($430)) {
         label = 68;
         break L71;
        }
       }
      } while(0);
      $431 = HEAP32[$i>>2]|0;
      $432 = (($431) + 1)|0;
      HEAP32[$i>>2] = $432;
     }
     if ((label|0) == 68) {
      label = 0;
      HEAP8[$isMatch>>0] = 1;
     }
     $433 = HEAP8[$isMatch>>0]|0;
     $434 = $433&255;
     $435 = ($434|0)!=(0);
     do {
      if ($435) {
       $436 = HEAP16[$4>>1]|0;
       $437 = $436&65535;
       $438 = $437 & 256;
       $439 = ($438|0)==(0);
       if (!($439)) {
        break;
       }
       $440 = HEAP8[$revSet>>0]|0;
       $441 = ($440<<24>>24)!=(0);
       if ($441) {
        $442 = HEAP8[$rev>>0]|0;
        $443 = $442&255;
        $444 = HEAP8[$revIdx>>0]|0;
        $445 = $444&255;
        $446 = $443 ^ $445;
        $447 = HEAP32[$i>>2]|0;
        $448 = HEAP32[$2>>2]|0;
        $449 = (($448) + 4|0);
        $450 = HEAP32[$449>>2]|0;
        $451 = (($450) + (($447*20)|0)|0);
        $452 = (($451) + 12|0);
        $453 = HEAP8[$452>>0]|0;
        $454 = $453&255;
        $455 = ($446|0)!=($454|0);
        if (!($455)) {
         break;
        }
        HEAP8[$isMatch>>0] = 0;
        break;
       }
       $456 = HEAP8[$revIdx>>0]|0;
       $457 = $456&255;
       $458 = HEAP32[$i>>2]|0;
       $459 = HEAP32[$2>>2]|0;
       $460 = (($459) + 4|0);
       $461 = HEAP32[$460>>2]|0;
       $462 = (($461) + (($458*20)|0)|0);
       $463 = (($462) + 12|0);
       $464 = HEAP8[$463>>0]|0;
       $465 = $464&255;
       $466 = $457 ^ $465;
       $467 = $466&255;
       HEAP8[$rev>>0] = $467;
       $468 = HEAP8[$rev>>0]|0;
       $469 = ($468<<24>>24)!=(0);
       if ($469) {
        $470 = HEAP32[$iLoop>>2]|0;
        $471 = (_bitshift64Shl(1,0,($470|0))|0);
        $472 = tempRet0;
        $473 = HEAP32[$7>>2]|0;
        $474 = $473;
        $475 = $474;
        $476 = HEAP32[$475>>2]|0;
        $477 = (($474) + 4)|0;
        $478 = $477;
        $479 = HEAP32[$478>>2]|0;
        $480 = $476 | $471;
        $481 = $479 | $472;
        $482 = $473;
        $483 = $482;
        HEAP32[$483>>2] = $480;
        $484 = (($482) + 4)|0;
        $485 = $484;
        HEAP32[$485>>2] = $481;
       }
       HEAP8[$revSet>>0] = 1;
      }
     } while(0);
     $486 = HEAP8[$isMatch>>0]|0;
     $487 = ($486<<24>>24)!=(0);
     if (!($487)) {
      label = 82;
      break;
     }
     $488 = HEAP32[$iColumn>>2]|0;
     $489 = ($488|0)<(0);
     if ($489) {
      HEAP8[$distinctColumns>>0] = 1;
     }
     $490 = HEAP32[$i>>2]|0;
     $491 = (_bitshift64Shl(1,0,($490|0))|0);
     $492 = tempRet0;
     $493 = $obSat;
     $494 = $493;
     $495 = HEAP32[$494>>2]|0;
     $496 = (($493) + 4)|0;
     $497 = $496;
     $498 = HEAP32[$497>>2]|0;
     $499 = $495 | $491;
     $500 = $498 | $492;
     $501 = $obSat;
     $502 = $501;
     HEAP32[$502>>2] = $499;
     $503 = (($501) + 4)|0;
     $504 = $503;
     HEAP32[$504>>2] = $500;
    }
    $511 = HEAP32[$j>>2]|0;
    $512 = (($511) + 1)|0;
    HEAP32[$j>>2] = $512;
   }
   do {
    if ((label|0) == 82) {
     label = 0;
     $505 = HEAP32[$j>>2]|0;
     $506 = ($505|0)==(0);
     if (!($506)) {
      $507 = HEAP32[$j>>2]|0;
      $508 = HEAP16[$nKeyCol>>1]|0;
      $509 = $508&65535;
      $510 = ($507|0)<($509|0);
      if (!($510)) {
       break;
      }
     }
     HEAP8[$isOrderDistinct>>0] = 0;
    }
   } while(0);
   $513 = HEAP8[$distinctColumns>>0]|0;
   $514 = ($513<<24>>24)!=(0);
   if ($514) {
    HEAP8[$isOrderDistinct>>0] = 1;
   }
  }
  $515 = HEAP8[$isOrderDistinct>>0]|0;
  $516 = ($515<<24>>24)!=(0);
  L115: do {
   if ($516) {
    $517 = HEAP32[$pLoop>>2]|0;
    $518 = (($517) + 8|0);
    $519 = $518;
    $520 = $519;
    $521 = HEAP32[$520>>2]|0;
    $522 = (($519) + 4)|0;
    $523 = $522;
    $524 = HEAP32[$523>>2]|0;
    $525 = $orderDistinctMask;
    $526 = $525;
    $527 = HEAP32[$526>>2]|0;
    $528 = (($525) + 4)|0;
    $529 = $528;
    $530 = HEAP32[$529>>2]|0;
    $531 = $527 | $521;
    $532 = $530 | $524;
    $533 = $orderDistinctMask;
    $534 = $533;
    HEAP32[$534>>2] = $531;
    $535 = (($533) + 4)|0;
    $536 = $535;
    HEAP32[$536>>2] = $532;
    HEAP32[$i>>2] = 0;
    while(1) {
     $537 = HEAP32[$i>>2]|0;
     $538 = HEAP16[$nOrderBy>>1]|0;
     $539 = $538&65535;
     $540 = ($537|0)<($539|0);
     if (!($540)) {
      break L115;
     }
     $541 = HEAP32[$i>>2]|0;
     $542 = (_bitshift64Shl(1,0,($541|0))|0);
     $543 = tempRet0;
     $544 = $obSat;
     $545 = $544;
     $546 = HEAP32[$545>>2]|0;
     $547 = (($544) + 4)|0;
     $548 = $547;
     $549 = HEAP32[$548>>2]|0;
     $550 = $542 & $546;
     $551 = $543 & $549;
     $552 = ($550|0)!=(0);
     $553 = ($551|0)!=(0);
     $554 = $552 | $553;
     do {
      if (!($554)) {
       $555 = HEAP32[$i>>2]|0;
       $556 = HEAP32[$2>>2]|0;
       $557 = (($556) + 4|0);
       $558 = HEAP32[$557>>2]|0;
       $559 = (($558) + (($555*20)|0)|0);
       $560 = HEAP32[$559>>2]|0;
       HEAP32[$p>>2] = $560;
       $561 = HEAP32[$1>>2]|0;
       $562 = (($561) + 68|0);
       $563 = HEAP32[$p>>2]|0;
       $564 = (_exprTableUsage($562,$563)|0);
       $565 = tempRet0;
       $566 = $mTerm;
       $567 = $566;
       HEAP32[$567>>2] = $564;
       $568 = (($566) + 4)|0;
       $569 = $568;
       HEAP32[$569>>2] = $565;
       $570 = $mTerm;
       $571 = $570;
       $572 = HEAP32[$571>>2]|0;
       $573 = (($570) + 4)|0;
       $574 = $573;
       $575 = HEAP32[$574>>2]|0;
       $576 = ($572|0)==(0);
       $577 = ($575|0)==(0);
       $578 = $576 & $577;
       if ($578) {
        $579 = HEAP32[$p>>2]|0;
        $580 = (_sqlite3ExprIsConstant($579)|0);
        $581 = ($580|0)!=(0);
        if (!($581)) {
         break;
        }
       }
       $582 = $mTerm;
       $583 = $582;
       $584 = HEAP32[$583>>2]|0;
       $585 = (($582) + 4)|0;
       $586 = $585;
       $587 = HEAP32[$586>>2]|0;
       $588 = $orderDistinctMask;
       $589 = $588;
       $590 = HEAP32[$589>>2]|0;
       $591 = (($588) + 4)|0;
       $592 = $591;
       $593 = HEAP32[$592>>2]|0;
       $594 = $590 ^ -1;
       $595 = $593 ^ -1;
       $596 = $584 & $594;
       $597 = $587 & $595;
       $598 = ($596|0)==(0);
       $599 = ($597|0)==(0);
       $600 = $598 & $599;
       if ($600) {
        $601 = HEAP32[$i>>2]|0;
        $602 = (_bitshift64Shl(1,0,($601|0))|0);
        $603 = tempRet0;
        $604 = $obSat;
        $605 = $604;
        $606 = HEAP32[$605>>2]|0;
        $607 = (($604) + 4)|0;
        $608 = $607;
        $609 = HEAP32[$608>>2]|0;
        $610 = $606 | $602;
        $611 = $609 | $603;
        $612 = $obSat;
        $613 = $612;
        HEAP32[$613>>2] = $610;
        $614 = (($612) + 4)|0;
        $615 = $614;
        HEAP32[$615>>2] = $611;
       }
      }
     } while(0);
     $616 = HEAP32[$i>>2]|0;
     $617 = (($616) + 1)|0;
     HEAP32[$i>>2] = $617;
    }
   }
  } while(0);
  $618 = HEAP32[$iLoop>>2]|0;
  $619 = (($618) + 1)|0;
  HEAP32[$iLoop>>2] = $619;
 }
 if ((label|0) == 16) {
  $112 = (($111) + 24|0);
  $113 = (($112) + 5|0);
  $114 = HEAP8[$113>>0]|0;
  $115 = ($114<<24>>24)!=(0);
  if ($115) {
   $116 = $obDone;
   $117 = $116;
   $118 = HEAP32[$117>>2]|0;
   $119 = (($116) + 4)|0;
   $120 = $119;
   $121 = HEAP32[$120>>2]|0;
   $122 = $obSat;
   $123 = $122;
   HEAP32[$123>>2] = $118;
   $124 = (($122) + 4)|0;
   $125 = $124;
   HEAP32[$125>>2] = $121;
  }
 }
 else if ((label|0) == 38) {
  HEAP8[$0>>0] = 0;
  $679 = HEAP8[$0>>0]|0;
  STACKTOP = sp;return ($679|0);
 }
 $620 = $obSat;
 $621 = $620;
 $622 = HEAP32[$621>>2]|0;
 $623 = (($620) + 4)|0;
 $624 = $623;
 $625 = HEAP32[$624>>2]|0;
 $626 = $obDone;
 $627 = $626;
 $628 = HEAP32[$627>>2]|0;
 $629 = (($626) + 4)|0;
 $630 = $629;
 $631 = HEAP32[$630>>2]|0;
 $632 = ($622|0)==($628|0);
 $633 = ($625|0)==($631|0);
 $634 = $632 & $633;
 if ($634) {
  $635 = HEAP16[$nOrderBy>>1]|0;
  $636 = $635&255;
  HEAP8[$0>>0] = $636;
  $679 = HEAP8[$0>>0]|0;
  STACKTOP = sp;return ($679|0);
 }
 $637 = HEAP8[$isOrderDistinct>>0]|0;
 $638 = ($637<<24>>24)!=(0);
 if ($638) {
  HEAP8[$0>>0] = -1;
  $679 = HEAP8[$0>>0]|0;
  STACKTOP = sp;return ($679|0);
 }
 $639 = HEAP16[$nOrderBy>>1]|0;
 $640 = $639&65535;
 $641 = (($640) - 1)|0;
 HEAP32[$i>>2] = $641;
 while(1) {
  $642 = HEAP32[$i>>2]|0;
  $643 = ($642|0)>(0);
  if (!($643)) {
   label = 106;
   break;
  }
  $644 = HEAP32[$i>>2]|0;
  $645 = (_bitshift64Shl(1,0,($644|0))|0);
  $646 = tempRet0;
  $647 = (_i64Subtract(($645|0),($646|0),1,0)|0);
  $648 = tempRet0;
  $649 = $m;
  $650 = $649;
  HEAP32[$650>>2] = $647;
  $651 = (($649) + 4)|0;
  $652 = $651;
  HEAP32[$652>>2] = $648;
  $653 = $obSat;
  $654 = $653;
  $655 = HEAP32[$654>>2]|0;
  $656 = (($653) + 4)|0;
  $657 = $656;
  $658 = HEAP32[$657>>2]|0;
  $659 = $m;
  $660 = $659;
  $661 = HEAP32[$660>>2]|0;
  $662 = (($659) + 4)|0;
  $663 = $662;
  $664 = HEAP32[$663>>2]|0;
  $665 = $655 & $661;
  $666 = $658 & $664;
  $667 = $m;
  $668 = $667;
  $669 = HEAP32[$668>>2]|0;
  $670 = (($667) + 4)|0;
  $671 = $670;
  $672 = HEAP32[$671>>2]|0;
  $673 = ($665|0)==($669|0);
  $674 = ($666|0)==($672|0);
  $675 = $673 & $674;
  $676 = HEAP32[$i>>2]|0;
  if ($675) {
   label = 104;
   break;
  }
  $678 = (($676) + -1)|0;
  HEAP32[$i>>2] = $678;
 }
 if ((label|0) == 104) {
  $677 = $676&255;
  HEAP8[$0>>0] = $677;
  $679 = HEAP8[$0>>0]|0;
  STACKTOP = sp;return ($679|0);
 }
 else if ((label|0) == 106) {
  HEAP8[$0>>0] = 0;
  $679 = HEAP8[$0>>0]|0;
  STACKTOP = sp;return ($679|0);
 }
 return 0|0;
}
function _whereSortingCost($pWInfo,$nRow,$nOrderBy,$nSorted) {
 $pWInfo = $pWInfo|0;
 $nRow = $nRow|0;
 $nOrderBy = $nOrderBy|0;
 $nSorted = $nSorted|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $rScale = 0, $rSortCost = 0;
 var label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 8|0;
 $1 = sp + 16|0;
 $2 = sp + 4|0;
 $3 = sp;
 $rScale = sp + 14|0;
 $rSortCost = sp + 12|0;
 HEAP32[$0>>2] = $pWInfo;
 HEAP16[$1>>1] = $nRow;
 HEAP32[$2>>2] = $nOrderBy;
 HEAP32[$3>>2] = $nSorted;
 $4 = HEAP32[$2>>2]|0;
 $5 = HEAP32[$3>>2]|0;
 $6 = (($4) - ($5))|0;
 $7 = ($6*100)|0;
 $8 = HEAP32[$2>>2]|0;
 $9 = (($7|0) / ($8|0))&-1;
 $10 = ($9|0)<(0);
 $11 = $10 << 31 >> 31;
 $12 = (_sqlite3LogEst($9,$11)|0);
 $13 = $12 << 16 >> 16;
 $14 = (($13) - 66)|0;
 $15 = $14&65535;
 HEAP16[$rScale>>1] = $15;
 $16 = HEAP16[$1>>1]|0;
 $17 = $16 << 16 >> 16;
 $18 = HEAP16[$1>>1]|0;
 $19 = (_estLog($18)|0);
 $20 = $19 << 16 >> 16;
 $21 = (($17) + ($20))|0;
 $22 = HEAP16[$rScale>>1]|0;
 $23 = $22 << 16 >> 16;
 $24 = (($21) + ($23))|0;
 $25 = (($24) + 16)|0;
 $26 = $25&65535;
 HEAP16[$rSortCost>>1] = $26;
 $27 = HEAP32[$0>>2]|0;
 $28 = (($27) + 34|0);
 $29 = HEAP16[$28>>1]|0;
 $30 = $29&65535;
 $31 = $30 & 1024;
 $32 = ($31|0)!=(0);
 if (!($32)) {
  $37 = HEAP16[$rSortCost>>1]|0;
  STACKTOP = sp;return ($37|0);
 }
 $33 = HEAP16[$rSortCost>>1]|0;
 $34 = $33 << 16 >> 16;
 $35 = (($34) + 16)|0;
 $36 = $35&65535;
 HEAP16[$rSortCost>>1] = $36;
 $37 = HEAP16[$rSortCost>>1]|0;
 STACKTOP = sp;return ($37|0);
}
function _estLog($N) {
 $N = $N|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 HEAP16[$0>>1] = $N;
 $1 = HEAP16[$0>>1]|0;
 $2 = $1 << 16 >> 16;
 $3 = ($2|0)<=(10);
 if ($3) {
  $12 = 0;
  $11 = $12&65535;
  STACKTOP = sp;return ($11|0);
 }
 $4 = HEAP16[$0>>1]|0;
 $5 = $4 << 16 >> 16;
 $6 = ($5|0)<(0);
 $7 = $6 << 31 >> 31;
 $8 = (_sqlite3LogEst($5,$7)|0);
 $9 = $8 << 16 >> 16;
 $10 = (($9) - 33)|0;
 $12 = $10;
 $11 = $12&65535;
 STACKTOP = sp;return ($11|0);
}
function _whereLoopAddVirtual($pBuilder,$0,$1) {
 $pBuilder = $pBuilder|0;
 $0 = $0|0;
 $1 = $1|0;
 var $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0, $116 = 0, $117 = 0;
 var $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0, $134 = 0, $135 = 0;
 var $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0, $152 = 0, $153 = 0;
 var $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0, $170 = 0, $171 = 0;
 var $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0, $188 = 0, $189 = 0, $19 = 0;
 var $190 = 0, $191 = 0, $192 = 0, $193 = 0, $194 = 0, $195 = 0, $196 = 0, $197 = 0, $198 = 0, $199 = 0, $2 = 0, $20 = 0, $200 = 0, $201 = 0, $202 = 0, $203 = 0, $204 = 0, $205 = 0, $206 = 0, $207 = 0;
 var $208 = 0, $209 = 0, $21 = 0, $210 = 0, $211 = 0, $212 = 0, $213 = 0, $214 = 0, $215 = 0, $216 = 0, $217 = 0, $218 = 0, $219 = 0, $22 = 0, $220 = 0, $221 = 0, $222 = 0, $223 = 0, $224 = 0, $225 = 0;
 var $226 = 0, $227 = 0, $228 = 0, $229 = 0, $23 = 0, $230 = 0, $231 = 0, $232 = 0, $233 = 0, $234 = 0, $235 = 0, $236 = 0, $237 = 0, $238 = 0, $239 = 0, $24 = 0, $240 = 0, $241 = 0, $242 = 0, $243 = 0;
 var $244 = 0, $245 = 0, $246 = 0, $247 = 0, $248 = 0, $249 = 0, $25 = 0, $250 = 0, $251 = 0, $252 = 0, $253 = 0, $254 = 0, $255 = 0, $256 = 0, $257 = 0, $258 = 0, $259 = 0, $26 = 0, $260 = 0, $261 = 0;
 var $262 = 0, $263 = 0, $264 = 0, $265 = 0, $266 = 0, $267 = 0, $268 = 0, $269 = 0, $27 = 0, $270 = 0, $271 = 0, $272 = 0, $273 = 0, $274 = 0, $275 = 0, $276 = 0, $277 = 0, $278 = 0, $279 = 0, $28 = 0;
 var $280 = 0, $281 = 0, $282 = 0, $283 = 0, $284 = 0, $285 = 0, $286 = 0, $287 = 0, $288 = 0, $289 = 0, $29 = 0, $290 = 0, $291 = 0, $292 = 0, $293 = 0, $294 = 0, $295 = 0, $296 = 0, $297 = 0, $298 = 0;
 var $299 = 0, $3 = 0, $30 = 0, $300 = 0, $301 = 0, $302 = 0, $303 = 0, $304 = 0, $305 = 0, $306 = 0, $307 = 0, $308 = 0, $309 = 0, $31 = 0, $310 = 0, $311 = 0, $312 = 0, $313 = 0, $314 = 0, $315 = 0;
 var $316 = 0, $317 = 0, $318 = 0, $319 = 0, $32 = 0, $320 = 0, $321 = 0, $322 = 0, $323 = 0, $324 = 0, $325 = 0, $326 = 0, $327 = 0, $328 = 0, $329 = 0, $33 = 0, $330 = 0, $331 = 0, $332 = 0, $333 = 0;
 var $334 = 0, $335 = 0, $336 = 0, $337 = 0, $338 = 0, $339 = 0, $34 = 0, $340 = 0, $341 = 0, $342 = 0, $343 = 0, $344 = 0, $345 = 0, $346 = 0, $347 = 0, $348 = 0, $349 = 0, $35 = 0, $350 = 0, $351 = 0;
 var $352 = 0, $353 = 0, $354 = 0, $355 = 0, $356 = 0, $357 = 0, $358 = 0, $359 = 0, $36 = 0, $360 = 0, $361 = 0, $362 = 0, $363 = 0, $364 = 0, $365 = 0, $366 = 0.0, $367 = 0, $368 = 0, $369 = 0, $37 = 0;
 var $370 = 0, $371 = 0, $372 = 0, $373 = 0, $374 = 0, $375 = 0, $376 = 0, $377 = 0, $378 = 0, $379 = 0, $38 = 0, $380 = 0, $381 = 0, $382 = 0, $383 = 0, $384 = 0, $385 = 0, $386 = 0, $387 = 0, $388 = 0;
 var $389 = 0, $39 = 0, $390 = 0, $391 = 0, $392 = 0, $393 = 0, $394 = 0, $395 = 0, $396 = 0, $397 = 0, $398 = 0, $399 = 0, $4 = 0, $40 = 0, $400 = 0, $401 = 0, $402 = 0, $403 = 0, $404 = 0, $405 = 0;
 var $406 = 0, $407 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0;
 var $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0;
 var $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0;
 var $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $db = 0, $i = 0, $iPhase = 0, $iTerm = 0, $j = 0, $mxTerm = 0, $nConstraint = 0, $or$cond = 0, $or$cond3 = 0, $pIdxCons = 0, $pIdxInfo = 0, $pNew = 0, $pParse = 0, $pSrc = 0;
 var $pTab = 0, $pTerm = 0, $pUsage = 0, $pWC = 0, $pWInfo = 0, $rc = 0, $seenIn = 0, $seenVar = 0, $vararg_buffer = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 112|0;
 $vararg_buffer = sp + 8|0;
 $2 = sp + 96|0;
 $3 = sp + 92|0;
 $4 = sp;
 $pWInfo = sp + 88|0;
 $pParse = sp + 84|0;
 $pWC = sp + 80|0;
 $pSrc = sp + 76|0;
 $pTab = sp + 72|0;
 $db = sp + 68|0;
 $pIdxInfo = sp + 64|0;
 $pIdxCons = sp + 60|0;
 $pUsage = sp + 56|0;
 $pTerm = sp + 52|0;
 $i = sp + 48|0;
 $j = sp + 44|0;
 $iTerm = sp + 40|0;
 $mxTerm = sp + 36|0;
 $nConstraint = sp + 32|0;
 $seenIn = sp + 28|0;
 $seenVar = sp + 24|0;
 $iPhase = sp + 20|0;
 $pNew = sp + 16|0;
 $rc = sp + 12|0;
 HEAP32[$3>>2] = $pBuilder;
 $5 = $4;
 $6 = $5;
 HEAP32[$6>>2] = $0;
 $7 = (($5) + 4)|0;
 $8 = $7;
 HEAP32[$8>>2] = $1;
 HEAP32[$seenIn>>2] = 0;
 HEAP32[$seenVar>>2] = 0;
 HEAP32[$rc>>2] = 0;
 $9 = HEAP32[$3>>2]|0;
 $10 = HEAP32[$9>>2]|0;
 HEAP32[$pWInfo>>2] = $10;
 $11 = HEAP32[$pWInfo>>2]|0;
 $12 = HEAP32[$11>>2]|0;
 HEAP32[$pParse>>2] = $12;
 $13 = HEAP32[$pParse>>2]|0;
 $14 = HEAP32[$13>>2]|0;
 HEAP32[$db>>2] = $14;
 $15 = HEAP32[$3>>2]|0;
 $16 = (($15) + 4|0);
 $17 = HEAP32[$16>>2]|0;
 HEAP32[$pWC>>2] = $17;
 $18 = HEAP32[$3>>2]|0;
 $19 = (($18) + 12|0);
 $20 = HEAP32[$19>>2]|0;
 HEAP32[$pNew>>2] = $20;
 $21 = HEAP32[$pNew>>2]|0;
 $22 = (($21) + 16|0);
 $23 = HEAP8[$22>>0]|0;
 $24 = $23&255;
 $25 = HEAP32[$pWInfo>>2]|0;
 $26 = (($25) + 4|0);
 $27 = HEAP32[$26>>2]|0;
 $28 = (($27) + 8|0);
 $29 = (($28) + (($24*72)|0)|0);
 HEAP32[$pSrc>>2] = $29;
 $30 = HEAP32[$pSrc>>2]|0;
 $31 = (($30) + 16|0);
 $32 = HEAP32[$31>>2]|0;
 HEAP32[$pTab>>2] = $32;
 $33 = HEAP32[$pParse>>2]|0;
 $34 = HEAP32[$pWC>>2]|0;
 $35 = HEAP32[$pSrc>>2]|0;
 $36 = HEAP32[$3>>2]|0;
 $37 = (($36) + 8|0);
 $38 = HEAP32[$37>>2]|0;
 $39 = (_allocateIndexInfo($33,$34,$35,$38)|0);
 HEAP32[$pIdxInfo>>2] = $39;
 $40 = HEAP32[$pIdxInfo>>2]|0;
 $41 = ($40|0)==(0|0);
 if ($41) {
  HEAP32[$2>>2] = 7;
  $407 = HEAP32[$2>>2]|0;
  STACKTOP = sp;return ($407|0);
 }
 $42 = HEAP32[$pNew>>2]|0;
 $43 = $42;
 $44 = $43;
 HEAP32[$44>>2] = 0;
 $45 = (($43) + 4)|0;
 $46 = $45;
 HEAP32[$46>>2] = 0;
 $47 = HEAP32[$pNew>>2]|0;
 $48 = (($47) + 18|0);
 HEAP16[$48>>1] = 0;
 $49 = HEAP32[$pNew>>2]|0;
 $50 = (($49) + 36|0);
 HEAP32[$50>>2] = 1024;
 $51 = HEAP32[$pNew>>2]|0;
 $52 = (($51) + 40|0);
 HEAP16[$52>>1] = 0;
 $53 = HEAP32[$pNew>>2]|0;
 $54 = (($53) + 24|0);
 $55 = (($54) + 4|0);
 HEAP8[$55>>0] = 0;
 $56 = HEAP32[$pIdxInfo>>2]|0;
 $57 = (($56) + 16|0);
 $58 = HEAP32[$57>>2]|0;
 HEAP32[$pUsage>>2] = $58;
 $59 = HEAP32[$pIdxInfo>>2]|0;
 $60 = HEAP32[$59>>2]|0;
 HEAP32[$nConstraint>>2] = $60;
 $61 = HEAP32[$db>>2]|0;
 $62 = HEAP32[$pNew>>2]|0;
 $63 = HEAP32[$nConstraint>>2]|0;
 $64 = (_whereLoopResize($61,$62,$63)|0);
 $65 = ($64|0)!=(0);
 if ($65) {
  $66 = HEAP32[$db>>2]|0;
  $67 = HEAP32[$pIdxInfo>>2]|0;
  _sqlite3DbFree($66,$67);
  HEAP32[$2>>2] = 7;
  $407 = HEAP32[$2>>2]|0;
  STACKTOP = sp;return ($407|0);
 }
 HEAP32[$iPhase>>2] = 0;
 L9: while(1) {
  $68 = HEAP32[$iPhase>>2]|0;
  $69 = ($68|0)<=(3);
  if (!($69)) {
   break;
  }
  $70 = HEAP32[$seenIn>>2]|0;
  $71 = ($70|0)!=(0);
  if (!($71)) {
   $72 = HEAP32[$iPhase>>2]|0;
   $73 = $72 & 1;
   $74 = ($73|0)!=(0);
   if ($74) {
    $75 = HEAP32[$iPhase>>2]|0;
    $76 = (($75) + 1)|0;
    HEAP32[$iPhase>>2] = $76;
    $77 = HEAP32[$iPhase>>2]|0;
    $78 = ($77|0)>(3);
    if ($78) {
     break;
    }
   }
  }
  $79 = HEAP32[$seenVar>>2]|0;
  $80 = ($79|0)==(0);
  $81 = HEAP32[$iPhase>>2]|0;
  $82 = ($81|0)>(1);
  $or$cond = $80 & $82;
  if ($or$cond) {
   break;
  }
  $83 = HEAP32[$pIdxInfo>>2]|0;
  $84 = (($83) + 4|0);
  $85 = HEAP32[$84>>2]|0;
  HEAP32[$pIdxCons>>2] = $85;
  HEAP32[$i>>2] = 0;
  while(1) {
   $86 = HEAP32[$i>>2]|0;
   $87 = HEAP32[$pIdxInfo>>2]|0;
   $88 = HEAP32[$87>>2]|0;
   $89 = ($86|0)<($88|0);
   if (!($89)) {
    break;
   }
   $90 = HEAP32[$pIdxCons>>2]|0;
   $91 = (($90) + 8|0);
   $92 = HEAP32[$91>>2]|0;
   HEAP32[$j>>2] = $92;
   $93 = HEAP32[$j>>2]|0;
   $94 = HEAP32[$pWC>>2]|0;
   $95 = (($94) + 20|0);
   $96 = HEAP32[$95>>2]|0;
   $97 = (($96) + (($93*48)|0)|0);
   HEAP32[$pTerm>>2] = $97;
   $98 = HEAP32[$iPhase>>2]|0;
   do {
    if ((($98|0) == 2)) {
     $141 = HEAP32[$pTerm>>2]|0;
     $142 = (($141) + 18|0);
     $143 = HEAP16[$142>>1]|0;
     $144 = $143&65535;
     $145 = $144 & 1;
     $146 = ($145|0)==(0);
     $147 = $146&1;
     $148 = $147&255;
     $149 = HEAP32[$pIdxCons>>2]|0;
     $150 = (($149) + 5|0);
     HEAP8[$150>>0] = $148;
    } else if ((($98|0) == 0)) {
     $99 = HEAP32[$pIdxCons>>2]|0;
     $100 = (($99) + 5|0);
     HEAP8[$100>>0] = 0;
     $101 = HEAP32[$pTerm>>2]|0;
     $102 = (($101) + 18|0);
     $103 = HEAP16[$102>>1]|0;
     $104 = $103&65535;
     $105 = $104 & 1;
     $106 = ($105|0)!=(0);
     if ($106) {
      HEAP32[$seenIn>>2] = 1;
     }
     $107 = HEAP32[$pTerm>>2]|0;
     $108 = (($107) + 32|0);
     $109 = $108;
     $110 = $109;
     $111 = HEAP32[$110>>2]|0;
     $112 = (($109) + 4)|0;
     $113 = $112;
     $114 = HEAP32[$113>>2]|0;
     $115 = ($111|0)!=(0);
     $116 = ($114|0)!=(0);
     $117 = $115 | $116;
     if ($117) {
      HEAP32[$seenVar>>2] = 1;
      break;
     }
     $118 = HEAP32[$pTerm>>2]|0;
     $119 = (($118) + 18|0);
     $120 = HEAP16[$119>>1]|0;
     $121 = $120&65535;
     $122 = $121 & 1;
     $123 = ($122|0)==(0);
     if ($123) {
      $124 = HEAP32[$pIdxCons>>2]|0;
      $125 = (($124) + 5|0);
      HEAP8[$125>>0] = 1;
     }
    } else if ((($98|0) == 1)) {
     $126 = HEAP32[$pTerm>>2]|0;
     $127 = (($126) + 32|0);
     $128 = $127;
     $129 = $128;
     $130 = HEAP32[$129>>2]|0;
     $131 = (($128) + 4)|0;
     $132 = $131;
     $133 = HEAP32[$132>>2]|0;
     $134 = ($130|0)==(0);
     $135 = ($133|0)==(0);
     $136 = $134 & $135;
     $137 = $136&1;
     $138 = $137&255;
     $139 = HEAP32[$pIdxCons>>2]|0;
     $140 = (($139) + 5|0);
     HEAP8[$140>>0] = $138;
    } else {
     $151 = HEAP32[$pIdxCons>>2]|0;
     $152 = (($151) + 5|0);
     HEAP8[$152>>0] = 1;
    }
   } while(0);
   $153 = HEAP32[$i>>2]|0;
   $154 = (($153) + 1)|0;
   HEAP32[$i>>2] = $154;
   $155 = HEAP32[$pIdxCons>>2]|0;
   $156 = (($155) + 12|0);
   HEAP32[$pIdxCons>>2] = $156;
  }
  $157 = HEAP32[$pUsage>>2]|0;
  $158 = HEAP32[$pIdxInfo>>2]|0;
  $159 = HEAP32[$158>>2]|0;
  $160 = $159<<3;
  _memset(($157|0),0,($160|0))|0;
  $161 = HEAP32[$pIdxInfo>>2]|0;
  $162 = (($161) + 28|0);
  $163 = HEAP32[$162>>2]|0;
  $164 = ($163|0)!=(0);
  if ($164) {
   $165 = HEAP32[$pIdxInfo>>2]|0;
   $166 = (($165) + 24|0);
   $167 = HEAP32[$166>>2]|0;
   _sqlite3_free($167);
  }
  $168 = HEAP32[$pIdxInfo>>2]|0;
  $169 = (($168) + 24|0);
  HEAP32[$169>>2] = 0;
  $170 = HEAP32[$pIdxInfo>>2]|0;
  $171 = (($170) + 20|0);
  HEAP32[$171>>2] = 0;
  $172 = HEAP32[$pIdxInfo>>2]|0;
  $173 = (($172) + 28|0);
  HEAP32[$173>>2] = 0;
  $174 = HEAP32[$pIdxInfo>>2]|0;
  $175 = (($174) + 32|0);
  HEAP32[$175>>2] = 0;
  $176 = HEAP32[$pIdxInfo>>2]|0;
  $177 = (($176) + 40|0);
  HEAPF64[$177>>3] = 4.9999999999999998E+98;
  $178 = HEAP32[$pIdxInfo>>2]|0;
  $179 = (($178) + 48|0);
  $180 = $179;
  $181 = $180;
  HEAP32[$181>>2] = 25;
  $182 = (($180) + 4)|0;
  $183 = $182;
  HEAP32[$183>>2] = 0;
  $184 = HEAP32[$pParse>>2]|0;
  $185 = HEAP32[$pTab>>2]|0;
  $186 = HEAP32[$pIdxInfo>>2]|0;
  $187 = (_vtabBestIndex($184,$185,$186)|0);
  HEAP32[$rc>>2] = $187;
  $188 = HEAP32[$rc>>2]|0;
  $189 = ($188|0)!=(0);
  if ($189) {
   break;
  }
  $190 = HEAP32[$pIdxInfo>>2]|0;
  $191 = (($190) + 4|0);
  $192 = HEAP32[$191>>2]|0;
  HEAP32[$pIdxCons>>2] = $192;
  $193 = $4;
  $194 = $193;
  $195 = HEAP32[$194>>2]|0;
  $196 = (($193) + 4)|0;
  $197 = $196;
  $198 = HEAP32[$197>>2]|0;
  $199 = HEAP32[$pNew>>2]|0;
  $200 = $199;
  $201 = $200;
  HEAP32[$201>>2] = $195;
  $202 = (($200) + 4)|0;
  $203 = $202;
  HEAP32[$203>>2] = $198;
  HEAP32[$mxTerm>>2] = -1;
  HEAP32[$i>>2] = 0;
  while(1) {
   $204 = HEAP32[$i>>2]|0;
   $205 = HEAP32[$nConstraint>>2]|0;
   $206 = ($204|0)<($205|0);
   if (!($206)) {
    break;
   }
   $207 = HEAP32[$i>>2]|0;
   $208 = HEAP32[$pNew>>2]|0;
   $209 = (($208) + 44|0);
   $210 = HEAP32[$209>>2]|0;
   $211 = (($210) + ($207<<2)|0);
   HEAP32[$211>>2] = 0;
   $212 = HEAP32[$i>>2]|0;
   $213 = (($212) + 1)|0;
   HEAP32[$i>>2] = $213;
  }
  $214 = HEAP32[$pNew>>2]|0;
  $215 = (($214) + 24|0);
  $216 = (($215) + 6|0);
  HEAP16[$216>>1] = 0;
  HEAP32[$i>>2] = 0;
  while(1) {
   $217 = HEAP32[$i>>2]|0;
   $218 = HEAP32[$nConstraint>>2]|0;
   $219 = ($217|0)<($218|0);
   if (!($219)) {
    break;
   }
   $220 = HEAP32[$i>>2]|0;
   $221 = HEAP32[$pUsage>>2]|0;
   $222 = (($221) + ($220<<3)|0);
   $223 = HEAP32[$222>>2]|0;
   $224 = (($223) - 1)|0;
   HEAP32[$iTerm>>2] = $224;
   $225 = ($224|0)>=(0);
   if ($225) {
    $226 = HEAP32[$pIdxCons>>2]|0;
    $227 = (($226) + 8|0);
    $228 = HEAP32[$227>>2]|0;
    HEAP32[$j>>2] = $228;
    $229 = HEAP32[$iTerm>>2]|0;
    $230 = HEAP32[$nConstraint>>2]|0;
    $231 = ($229|0)>=($230|0);
    $232 = HEAP32[$j>>2]|0;
    $233 = ($232|0)<(0);
    $or$cond3 = $231 | $233;
    if ($or$cond3) {
     label = 36;
     break L9;
    }
    $234 = HEAP32[$j>>2]|0;
    $235 = HEAP32[$pWC>>2]|0;
    $236 = (($235) + 12|0);
    $237 = HEAP32[$236>>2]|0;
    $238 = ($234|0)>=($237|0);
    if ($238) {
     label = 36;
     break L9;
    }
    $239 = HEAP32[$iTerm>>2]|0;
    $240 = HEAP32[$pNew>>2]|0;
    $241 = (($240) + 44|0);
    $242 = HEAP32[$241>>2]|0;
    $243 = (($242) + ($239<<2)|0);
    $244 = HEAP32[$243>>2]|0;
    $245 = ($244|0)!=(0|0);
    if ($245) {
     label = 36;
     break L9;
    }
    $249 = HEAP32[$j>>2]|0;
    $250 = HEAP32[$pWC>>2]|0;
    $251 = (($250) + 20|0);
    $252 = HEAP32[$251>>2]|0;
    $253 = (($252) + (($249*48)|0)|0);
    HEAP32[$pTerm>>2] = $253;
    $254 = HEAP32[$pTerm>>2]|0;
    $255 = (($254) + 32|0);
    $256 = $255;
    $257 = $256;
    $258 = HEAP32[$257>>2]|0;
    $259 = (($256) + 4)|0;
    $260 = $259;
    $261 = HEAP32[$260>>2]|0;
    $262 = HEAP32[$pNew>>2]|0;
    $263 = $262;
    $264 = $263;
    $265 = HEAP32[$264>>2]|0;
    $266 = (($263) + 4)|0;
    $267 = $266;
    $268 = HEAP32[$267>>2]|0;
    $269 = $265 | $258;
    $270 = $268 | $261;
    $271 = $262;
    $272 = $271;
    HEAP32[$272>>2] = $269;
    $273 = (($271) + 4)|0;
    $274 = $273;
    HEAP32[$274>>2] = $270;
    $275 = HEAP32[$pTerm>>2]|0;
    $276 = HEAP32[$iTerm>>2]|0;
    $277 = HEAP32[$pNew>>2]|0;
    $278 = (($277) + 44|0);
    $279 = HEAP32[$278>>2]|0;
    $280 = (($279) + ($276<<2)|0);
    HEAP32[$280>>2] = $275;
    $281 = HEAP32[$iTerm>>2]|0;
    $282 = HEAP32[$mxTerm>>2]|0;
    $283 = ($281|0)>($282|0);
    if ($283) {
     $284 = HEAP32[$iTerm>>2]|0;
     HEAP32[$mxTerm>>2] = $284;
    }
    $285 = HEAP32[$iTerm>>2]|0;
    $286 = ($285|0)<(16);
    if ($286) {
     $287 = HEAP32[$i>>2]|0;
     $288 = HEAP32[$pUsage>>2]|0;
     $289 = (($288) + ($287<<3)|0);
     $290 = (($289) + 4|0);
     $291 = HEAP8[$290>>0]|0;
     $292 = $291&255;
     $293 = ($292|0)!=(0);
     if ($293) {
      $294 = HEAP32[$iTerm>>2]|0;
      $295 = 1 << $294;
      $296 = HEAP32[$pNew>>2]|0;
      $297 = (($296) + 24|0);
      $298 = (($297) + 6|0);
      $299 = HEAP16[$298>>1]|0;
      $300 = $299&65535;
      $301 = $300 | $295;
      $302 = $301&65535;
      HEAP16[$298>>1] = $302;
     }
    }
    $303 = HEAP32[$pTerm>>2]|0;
    $304 = (($303) + 18|0);
    $305 = HEAP16[$304>>1]|0;
    $306 = $305&65535;
    $307 = $306 & 1;
    $308 = ($307|0)!=(0);
    if ($308) {
     $309 = HEAP32[$i>>2]|0;
     $310 = HEAP32[$pUsage>>2]|0;
     $311 = (($310) + ($309<<3)|0);
     $312 = (($311) + 4|0);
     $313 = HEAP8[$312>>0]|0;
     $314 = $313&255;
     $315 = ($314|0)==(0);
     if ($315) {
      break;
     }
     $316 = HEAP32[$pIdxInfo>>2]|0;
     $317 = (($316) + 32|0);
     HEAP32[$317>>2] = 0;
    }
   }
   $318 = HEAP32[$i>>2]|0;
   $319 = (($318) + 1)|0;
   HEAP32[$i>>2] = $319;
   $320 = HEAP32[$pIdxCons>>2]|0;
   $321 = (($320) + 12|0);
   HEAP32[$pIdxCons>>2] = $321;
  }
  $322 = HEAP32[$i>>2]|0;
  $323 = HEAP32[$nConstraint>>2]|0;
  $324 = ($322|0)>=($323|0);
  if ($324) {
   $325 = HEAP32[$mxTerm>>2]|0;
   $326 = (($325) + 1)|0;
   $327 = $326&65535;
   $328 = HEAP32[$pNew>>2]|0;
   $329 = (($328) + 40|0);
   HEAP16[$329>>1] = $327;
   $330 = HEAP32[$pIdxInfo>>2]|0;
   $331 = (($330) + 20|0);
   $332 = HEAP32[$331>>2]|0;
   $333 = HEAP32[$pNew>>2]|0;
   $334 = (($333) + 24|0);
   HEAP32[$334>>2] = $332;
   $335 = HEAP32[$pIdxInfo>>2]|0;
   $336 = (($335) + 28|0);
   $337 = HEAP32[$336>>2]|0;
   $338 = $337&255;
   $339 = HEAP32[$pNew>>2]|0;
   $340 = (($339) + 24|0);
   $341 = (($340) + 4|0);
   HEAP8[$341>>0] = $338;
   $342 = HEAP32[$pIdxInfo>>2]|0;
   $343 = (($342) + 28|0);
   HEAP32[$343>>2] = 0;
   $344 = HEAP32[$pIdxInfo>>2]|0;
   $345 = (($344) + 24|0);
   $346 = HEAP32[$345>>2]|0;
   $347 = HEAP32[$pNew>>2]|0;
   $348 = (($347) + 24|0);
   $349 = (($348) + 8|0);
   HEAP32[$349>>2] = $346;
   $350 = HEAP32[$pIdxInfo>>2]|0;
   $351 = (($350) + 32|0);
   $352 = HEAP32[$351>>2]|0;
   $353 = ($352|0)!=(0);
   if ($353) {
    $354 = HEAP32[$pIdxInfo>>2]|0;
    $355 = (($354) + 8|0);
    $356 = HEAP32[$355>>2]|0;
    $358 = $356;
   } else {
    $358 = 0;
   }
   $357 = $358&255;
   $359 = HEAP32[$pNew>>2]|0;
   $360 = (($359) + 24|0);
   $361 = (($360) + 5|0);
   HEAP8[$361>>0] = $357;
   $362 = HEAP32[$pNew>>2]|0;
   $363 = (($362) + 18|0);
   HEAP16[$363>>1] = 0;
   $364 = HEAP32[$pIdxInfo>>2]|0;
   $365 = (($364) + 40|0);
   $366 = +HEAPF64[$365>>3];
   $367 = (_sqlite3LogEstFromDouble($366)|0);
   $368 = HEAP32[$pNew>>2]|0;
   $369 = (($368) + 20|0);
   HEAP16[$369>>1] = $367;
   $370 = HEAP32[$pIdxInfo>>2]|0;
   $371 = (($370) + 48|0);
   $372 = $371;
   $373 = $372;
   $374 = HEAP32[$373>>2]|0;
   $375 = (($372) + 4)|0;
   $376 = $375;
   $377 = HEAP32[$376>>2]|0;
   $378 = (_sqlite3LogEst($374,$377)|0);
   $379 = HEAP32[$pNew>>2]|0;
   $380 = (($379) + 22|0);
   HEAP16[$380>>1] = $378;
   $381 = HEAP32[$3>>2]|0;
   $382 = HEAP32[$pNew>>2]|0;
   (_whereLoopInsert($381,$382)|0);
   $383 = HEAP32[$pNew>>2]|0;
   $384 = (($383) + 24|0);
   $385 = (($384) + 4|0);
   $386 = HEAP8[$385>>0]|0;
   $387 = ($386<<24>>24)!=(0);
   if ($387) {
    $388 = HEAP32[$pNew>>2]|0;
    $389 = (($388) + 24|0);
    $390 = (($389) + 8|0);
    $391 = HEAP32[$390>>2]|0;
    _sqlite3_free($391);
    $392 = HEAP32[$pNew>>2]|0;
    $393 = (($392) + 24|0);
    $394 = (($393) + 4|0);
    HEAP8[$394>>0] = 0;
   }
  }
  $395 = HEAP32[$iPhase>>2]|0;
  $396 = (($395) + 1)|0;
  HEAP32[$iPhase>>2] = $396;
 }
 if ((label|0) == 36) {
  HEAP32[$rc>>2] = 1;
  $246 = HEAP32[$pParse>>2]|0;
  $247 = HEAP32[$pTab>>2]|0;
  $248 = HEAP32[$247>>2]|0;
  HEAP32[$vararg_buffer>>2] = $248;
  _sqlite3ErrorMsg($246,24088,$vararg_buffer);
 }
 $397 = HEAP32[$pIdxInfo>>2]|0;
 $398 = (($397) + 28|0);
 $399 = HEAP32[$398>>2]|0;
 $400 = ($399|0)!=(0);
 if ($400) {
  $401 = HEAP32[$pIdxInfo>>2]|0;
  $402 = (($401) + 24|0);
  $403 = HEAP32[$402>>2]|0;
  _sqlite3_free($403);
 }
 $404 = HEAP32[$db>>2]|0;
 $405 = HEAP32[$pIdxInfo>>2]|0;
 _sqlite3DbFree($404,$405);
 $406 = HEAP32[$rc>>2]|0;
 HEAP32[$2>>2] = $406;
 $407 = HEAP32[$2>>2]|0;
 STACKTOP = sp;return ($407|0);
}
function _whereLoopAddBtree($pBuilder,$0,$1) {
 $pBuilder = $pBuilder|0;
 $0 = $0|0;
 $1 = $1|0;
 var $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0, $116 = 0, $117 = 0;
 var $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0, $134 = 0, $135 = 0;
 var $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0, $152 = 0, $153 = 0;
 var $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0, $170 = 0, $171 = 0;
 var $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0, $188 = 0, $189 = 0, $19 = 0;
 var $190 = 0, $191 = 0, $192 = 0, $193 = 0, $194 = 0, $195 = 0, $196 = 0, $197 = 0, $198 = 0, $199 = 0, $2 = 0, $20 = 0, $200 = 0, $201 = 0, $202 = 0, $203 = 0, $204 = 0, $205 = 0, $206 = 0, $207 = 0;
 var $208 = 0, $209 = 0, $21 = 0, $210 = 0, $211 = 0, $212 = 0, $213 = 0, $214 = 0, $215 = 0, $216 = 0, $217 = 0, $218 = 0, $219 = 0, $22 = 0, $220 = 0, $221 = 0, $222 = 0, $223 = 0, $224 = 0, $225 = 0;
 var $226 = 0, $227 = 0, $228 = 0, $229 = 0, $23 = 0, $230 = 0, $231 = 0, $232 = 0, $233 = 0, $234 = 0, $235 = 0, $236 = 0, $237 = 0, $238 = 0, $239 = 0, $24 = 0, $240 = 0, $241 = 0, $242 = 0, $243 = 0;
 var $244 = 0, $245 = 0, $246 = 0, $247 = 0, $248 = 0, $249 = 0, $25 = 0, $250 = 0, $251 = 0, $252 = 0, $253 = 0, $254 = 0, $255 = 0, $256 = 0, $257 = 0, $258 = 0, $259 = 0, $26 = 0, $260 = 0, $261 = 0;
 var $262 = 0, $263 = 0, $264 = 0, $265 = 0, $266 = 0, $267 = 0, $268 = 0, $269 = 0, $27 = 0, $270 = 0, $271 = 0, $272 = 0, $273 = 0, $274 = 0, $275 = 0, $276 = 0, $277 = 0, $278 = 0, $279 = 0, $28 = 0;
 var $280 = 0, $281 = 0, $282 = 0, $283 = 0, $284 = 0, $285 = 0, $286 = 0, $287 = 0, $288 = 0, $289 = 0, $29 = 0, $290 = 0, $291 = 0, $292 = 0, $293 = 0, $294 = 0, $295 = 0, $296 = 0, $297 = 0, $298 = 0;
 var $299 = 0, $3 = 0, $30 = 0, $300 = 0, $301 = 0, $302 = 0, $303 = 0, $304 = 0, $305 = 0, $306 = 0, $307 = 0, $308 = 0, $309 = 0, $31 = 0, $310 = 0, $311 = 0, $312 = 0, $313 = 0, $314 = 0, $315 = 0;
 var $316 = 0, $317 = 0, $318 = 0, $319 = 0, $32 = 0, $320 = 0, $321 = 0, $322 = 0, $323 = 0, $324 = 0, $325 = 0, $326 = 0, $327 = 0, $328 = 0, $329 = 0, $33 = 0, $330 = 0, $331 = 0, $332 = 0, $333 = 0;
 var $334 = 0, $335 = 0, $336 = 0, $337 = 0, $338 = 0, $339 = 0, $34 = 0, $340 = 0, $341 = 0, $342 = 0, $343 = 0, $344 = 0, $345 = 0, $346 = 0, $347 = 0, $348 = 0, $349 = 0, $35 = 0, $350 = 0, $351 = 0;
 var $352 = 0, $353 = 0, $354 = 0, $355 = 0, $356 = 0, $357 = 0, $358 = 0, $359 = 0, $36 = 0, $360 = 0, $361 = 0, $362 = 0, $363 = 0, $364 = 0, $365 = 0, $366 = 0, $367 = 0, $368 = 0, $369 = 0, $37 = 0;
 var $370 = 0, $371 = 0, $372 = 0, $373 = 0, $374 = 0, $375 = 0, $376 = 0, $377 = 0, $378 = 0, $379 = 0, $38 = 0, $380 = 0, $381 = 0, $382 = 0, $383 = 0, $384 = 0, $385 = 0, $386 = 0, $387 = 0, $388 = 0;
 var $389 = 0, $39 = 0, $390 = 0, $391 = 0, $392 = 0, $393 = 0, $394 = 0, $395 = 0, $396 = 0, $397 = 0, $398 = 0, $399 = 0, $4 = 0, $40 = 0, $400 = 0, $401 = 0, $402 = 0, $403 = 0, $404 = 0, $405 = 0;
 var $406 = 0, $407 = 0, $408 = 0, $409 = 0, $41 = 0, $410 = 0, $411 = 0, $412 = 0, $413 = 0, $414 = 0, $415 = 0, $416 = 0, $417 = 0, $418 = 0, $419 = 0, $42 = 0, $420 = 0, $421 = 0, $422 = 0, $423 = 0;
 var $424 = 0, $425 = 0, $426 = 0, $427 = 0, $428 = 0, $429 = 0, $43 = 0, $430 = 0, $431 = 0, $432 = 0, $433 = 0, $434 = 0, $435 = 0, $436 = 0, $437 = 0, $438 = 0, $439 = 0, $44 = 0, $440 = 0, $441 = 0;
 var $442 = 0, $443 = 0, $444 = 0, $445 = 0, $446 = 0, $447 = 0, $448 = 0, $449 = 0, $45 = 0, $450 = 0, $451 = 0, $452 = 0, $453 = 0, $454 = 0, $455 = 0, $456 = 0, $457 = 0, $458 = 0, $459 = 0, $46 = 0;
 var $460 = 0, $461 = 0, $462 = 0, $463 = 0, $464 = 0, $465 = 0, $466 = 0, $467 = 0, $468 = 0, $469 = 0, $47 = 0, $470 = 0, $471 = 0, $472 = 0, $473 = 0, $474 = 0, $475 = 0, $476 = 0, $477 = 0, $478 = 0;
 var $479 = 0, $48 = 0, $480 = 0, $481 = 0, $482 = 0, $483 = 0, $484 = 0, $485 = 0, $486 = 0, $487 = 0, $488 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0;
 var $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0;
 var $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0;
 var $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $aiColumnPk = 0, $aiRowEstPk = 0, $b = 0, $iSortIdx = 0, $m = 0, $or$cond = 0, $pFirst = 0, $pNew = 0, $pProbe = 0, $pSrc = 0, $pTab = 0, $pTabList = 0, $pTerm = 0;
 var $pWC = 0, $pWCEnd = 0, $pWInfo = 0, $rLogSize = 0, $rSize = 0, $rc = 0, $sPk = 0, dest = 0, label = 0, sp = 0, stop = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 144|0;
 $2 = sp + 128|0;
 $3 = sp + 8|0;
 $pWInfo = sp + 124|0;
 $pProbe = sp + 120|0;
 $sPk = sp + 64|0;
 $aiRowEstPk = sp + 140|0;
 $aiColumnPk = sp + 136|0;
 $pTabList = sp + 56|0;
 $pSrc = sp + 52|0;
 $pNew = sp + 48|0;
 $rc = sp + 44|0;
 $iSortIdx = sp + 40|0;
 $b = sp + 36|0;
 $rSize = sp + 134|0;
 $rLogSize = sp + 132|0;
 $pWC = sp + 32|0;
 $pTab = sp + 28|0;
 $pFirst = sp + 24|0;
 $pTerm = sp + 20|0;
 $pWCEnd = sp + 16|0;
 $m = sp;
 HEAP32[$2>>2] = $pBuilder;
 $4 = $3;
 $5 = $4;
 HEAP32[$5>>2] = $0;
 $6 = (($4) + 4)|0;
 $7 = $6;
 HEAP32[$7>>2] = $1;
 HEAP16[$aiColumnPk>>1] = -1;
 HEAP32[$rc>>2] = 0;
 HEAP32[$iSortIdx>>2] = 1;
 $8 = HEAP32[$2>>2]|0;
 $9 = (($8) + 12|0);
 $10 = HEAP32[$9>>2]|0;
 HEAP32[$pNew>>2] = $10;
 $11 = HEAP32[$2>>2]|0;
 $12 = HEAP32[$11>>2]|0;
 HEAP32[$pWInfo>>2] = $12;
 $13 = HEAP32[$pWInfo>>2]|0;
 $14 = (($13) + 4|0);
 $15 = HEAP32[$14>>2]|0;
 HEAP32[$pTabList>>2] = $15;
 $16 = HEAP32[$pTabList>>2]|0;
 $17 = (($16) + 8|0);
 $18 = HEAP32[$pNew>>2]|0;
 $19 = (($18) + 16|0);
 $20 = HEAP8[$19>>0]|0;
 $21 = $20&255;
 $22 = (($17) + (($21*72)|0)|0);
 HEAP32[$pSrc>>2] = $22;
 $23 = HEAP32[$pSrc>>2]|0;
 $24 = (($23) + 16|0);
 $25 = HEAP32[$24>>2]|0;
 HEAP32[$pTab>>2] = $25;
 $26 = HEAP32[$2>>2]|0;
 $27 = (($26) + 4|0);
 $28 = HEAP32[$27>>2]|0;
 HEAP32[$pWC>>2] = $28;
 $29 = HEAP32[$pSrc>>2]|0;
 $30 = (($29) + 68|0);
 $31 = HEAP32[$30>>2]|0;
 $32 = ($31|0)!=(0|0);
 do {
  if ($32) {
   $33 = HEAP32[$pSrc>>2]|0;
   $34 = (($33) + 68|0);
   $35 = HEAP32[$34>>2]|0;
   HEAP32[$pProbe>>2] = $35;
  } else {
   $36 = HEAP32[$pTab>>2]|0;
   $37 = (($36) + 44|0);
   $38 = HEAP8[$37>>0]|0;
   $39 = $38&255;
   $40 = $39 & 32;
   $41 = ($40|0)==(0);
   if (!($41)) {
    $42 = HEAP32[$pTab>>2]|0;
    $43 = (($42) + 8|0);
    $44 = HEAP32[$43>>2]|0;
    HEAP32[$pProbe>>2] = $44;
    break;
   }
   dest=$sPk+0|0; stop=dest+56|0; do { HEAP32[dest>>2]=0|0; dest=dest+4|0; } while ((dest|0) < (stop|0));
   $45 = (($sPk) + 50|0);
   HEAP16[$45>>1] = 1;
   $46 = (($sPk) + 52|0);
   HEAP16[$46>>1] = 1;
   $47 = (($sPk) + 4|0);
   HEAP32[$47>>2] = $aiColumnPk;
   $48 = (($sPk) + 8|0);
   HEAP32[$48>>2] = $aiRowEstPk;
   $49 = (($sPk) + 54|0);
   HEAP8[$49>>0] = 5;
   $50 = HEAP32[$pTab>>2]|0;
   $51 = (($sPk) + 12|0);
   HEAP32[$51>>2] = $50;
   $52 = HEAP32[$pTab>>2]|0;
   $53 = (($52) + 42|0);
   $54 = HEAP16[$53>>1]|0;
   $55 = (($sPk) + 48|0);
   HEAP16[$55>>1] = $54;
   $56 = HEAP32[$pTab>>2]|0;
   $57 = (($56) + 28|0);
   $58 = HEAP16[$57>>1]|0;
   HEAP16[$aiRowEstPk>>1] = $58;
   $59 = (($aiRowEstPk) + 2|0);
   HEAP16[$59>>1] = 0;
   $60 = HEAP32[$pSrc>>2]|0;
   $61 = (($60) + 16|0);
   $62 = HEAP32[$61>>2]|0;
   $63 = (($62) + 8|0);
   $64 = HEAP32[$63>>2]|0;
   HEAP32[$pFirst>>2] = $64;
   $65 = HEAP32[$pSrc>>2]|0;
   $66 = (($65) + 37|0);
   $67 = HEAP8[$66>>0]|0;
   $68 = $67 & 1;
   $69 = $68&255;
   $70 = ($69|0)==(0);
   if ($70) {
    $71 = HEAP32[$pFirst>>2]|0;
    $72 = (($sPk) + 20|0);
    HEAP32[$72>>2] = $71;
   }
   HEAP32[$pProbe>>2] = $sPk;
  }
 } while(0);
 $73 = HEAP32[$pTab>>2]|0;
 $74 = (($73) + 28|0);
 $75 = HEAP16[$74>>1]|0;
 HEAP16[$rSize>>1] = $75;
 $76 = HEAP16[$rSize>>1]|0;
 $77 = (_estLog($76)|0);
 HEAP16[$rLogSize>>1] = $77;
 $78 = HEAP32[$2>>2]|0;
 $79 = (($78) + 16|0);
 $80 = HEAP32[$79>>2]|0;
 $81 = ($80|0)!=(0|0);
 L11: do {
  if (!($81)) {
   $82 = HEAP32[$pWInfo>>2]|0;
   $83 = HEAP32[$82>>2]|0;
   $84 = HEAP32[$83>>2]|0;
   $85 = (($84) + 24|0);
   $86 = HEAP32[$85>>2]|0;
   $87 = $86 & 1048576;
   $88 = ($87|0)!=(0);
   if ($88) {
    $89 = HEAP32[$pSrc>>2]|0;
    $90 = (($89) + 68|0);
    $91 = HEAP32[$90>>2]|0;
    $92 = ($91|0)==(0|0);
    if ($92) {
     $93 = HEAP32[$pSrc>>2]|0;
     $94 = (($93) + 37|0);
     $95 = HEAP8[$94>>0]|0;
     $96 = ($95&255) >>> 2;
     $97 = $96 & 1;
     $98 = $97&255;
     $99 = ($98|0)!=(0);
     if (!($99)) {
      $100 = HEAP32[$pSrc>>2]|0;
      $101 = (($100) + 37|0);
      $102 = HEAP8[$101>>0]|0;
      $103 = $102 & 1;
      $104 = $103&255;
      $105 = ($104|0)!=(0);
      if (!($105)) {
       $106 = HEAP32[$pTab>>2]|0;
       $107 = (($106) + 44|0);
       $108 = HEAP8[$107>>0]|0;
       $109 = $108&255;
       $110 = $109 & 32;
       $111 = ($110|0)==(0);
       if ($111) {
        $112 = HEAP32[$pSrc>>2]|0;
        $113 = (($112) + 37|0);
        $114 = HEAP8[$113>>0]|0;
        $115 = ($114&255) >>> 1;
        $116 = $115 & 1;
        $117 = $116&255;
        $118 = ($117|0)!=(0);
        if (!($118)) {
         $119 = HEAP32[$pSrc>>2]|0;
         $120 = (($119) + 37|0);
         $121 = HEAP8[$120>>0]|0;
         $122 = ($121&255) >>> 3;
         $123 = $122 & 1;
         $124 = $123&255;
         $125 = ($124|0)!=(0);
         if (!($125)) {
          $126 = HEAP32[$pWC>>2]|0;
          $127 = (($126) + 20|0);
          $128 = HEAP32[$127>>2]|0;
          $129 = HEAP32[$pWC>>2]|0;
          $130 = (($129) + 12|0);
          $131 = HEAP32[$130>>2]|0;
          $132 = (($128) + (($131*48)|0)|0);
          HEAP32[$pWCEnd>>2] = $132;
          $133 = HEAP32[$pWC>>2]|0;
          $134 = (($133) + 20|0);
          $135 = HEAP32[$134>>2]|0;
          HEAP32[$pTerm>>2] = $135;
          while(1) {
           $136 = HEAP32[$rc>>2]|0;
           $137 = ($136|0)==(0);
           if (!($137)) {
            break L11;
           }
           $138 = HEAP32[$pTerm>>2]|0;
           $139 = HEAP32[$pWCEnd>>2]|0;
           $140 = ($138>>>0)<($139>>>0);
           if (!($140)) {
            break L11;
           }
           $141 = HEAP32[$pTerm>>2]|0;
           $142 = (($141) + 32|0);
           $143 = $142;
           $144 = $143;
           $145 = HEAP32[$144>>2]|0;
           $146 = (($143) + 4)|0;
           $147 = $146;
           $148 = HEAP32[$147>>2]|0;
           $149 = HEAP32[$pNew>>2]|0;
           $150 = (($149) + 8|0);
           $151 = $150;
           $152 = $151;
           $153 = HEAP32[$152>>2]|0;
           $154 = (($151) + 4)|0;
           $155 = $154;
           $156 = HEAP32[$155>>2]|0;
           $157 = $145 & $153;
           $158 = $148 & $156;
           $159 = ($157|0)!=(0);
           $160 = ($158|0)!=(0);
           $161 = $159 | $160;
           if (!($161)) {
            $162 = HEAP32[$pTerm>>2]|0;
            $163 = HEAP32[$pSrc>>2]|0;
            $164 = (_termCanDriveIndex($162,$163,0,0)|0);
            $165 = ($164|0)!=(0);
            if ($165) {
             $166 = HEAP32[$pNew>>2]|0;
             $167 = (($166) + 24|0);
             HEAP16[$167>>1] = 1;
             $168 = HEAP32[$pNew>>2]|0;
             $169 = (($168) + 24|0);
             $170 = (($169) + 2|0);
             HEAP16[$170>>1] = 0;
             $171 = HEAP32[$pNew>>2]|0;
             $172 = (($171) + 24|0);
             $173 = (($172) + 4|0);
             HEAP32[$173>>2] = 0;
             $174 = HEAP32[$pNew>>2]|0;
             $175 = (($174) + 40|0);
             HEAP16[$175>>1] = 1;
             $176 = HEAP32[$pTerm>>2]|0;
             $177 = HEAP32[$pNew>>2]|0;
             $178 = (($177) + 44|0);
             $179 = HEAP32[$178>>2]|0;
             HEAP32[$179>>2] = $176;
             $180 = HEAP16[$rLogSize>>1]|0;
             $181 = $180 << 16 >> 16;
             $182 = HEAP16[$rSize>>1]|0;
             $183 = $182 << 16 >> 16;
             $184 = (($181) + ($183))|0;
             $185 = (($184) + 4)|0;
             $186 = $185&65535;
             $187 = HEAP32[$pNew>>2]|0;
             $188 = (($187) + 18|0);
             HEAP16[$188>>1] = $186;
             $189 = HEAP32[$pTab>>2]|0;
             $190 = (($189) + 12|0);
             $191 = HEAP32[$190>>2]|0;
             $192 = ($191|0)==(0|0);
             if ($192) {
              $193 = HEAP32[$pTab>>2]|0;
              $194 = (($193) + 44|0);
              $195 = HEAP8[$194>>0]|0;
              $196 = $195&255;
              $197 = $196 & 2;
              $198 = ($197|0)==(0);
              if ($198) {
               $199 = HEAP32[$pNew>>2]|0;
               $200 = (($199) + 18|0);
               $201 = HEAP16[$200>>1]|0;
               $202 = $201 << 16 >> 16;
               $203 = (($202) + 24)|0;
               $204 = $203&65535;
               HEAP16[$200>>1] = $204;
              }
             }
             $205 = HEAP32[$pNew>>2]|0;
             $206 = (($205) + 22|0);
             HEAP16[$206>>1] = 43;
             $207 = HEAP16[$rLogSize>>1]|0;
             $208 = HEAP32[$pNew>>2]|0;
             $209 = (($208) + 22|0);
             $210 = HEAP16[$209>>1]|0;
             $211 = (_sqlite3LogEstAdd($207,$210)|0);
             $212 = HEAP32[$pNew>>2]|0;
             $213 = (($212) + 20|0);
             HEAP16[$213>>1] = $211;
             $214 = HEAP32[$pNew>>2]|0;
             $215 = (($214) + 36|0);
             HEAP32[$215>>2] = 16384;
             $216 = $3;
             $217 = $216;
             $218 = HEAP32[$217>>2]|0;
             $219 = (($216) + 4)|0;
             $220 = $219;
             $221 = HEAP32[$220>>2]|0;
             $222 = HEAP32[$pTerm>>2]|0;
             $223 = (($222) + 32|0);
             $224 = $223;
             $225 = $224;
             $226 = HEAP32[$225>>2]|0;
             $227 = (($224) + 4)|0;
             $228 = $227;
             $229 = HEAP32[$228>>2]|0;
             $230 = $218 | $226;
             $231 = $221 | $229;
             $232 = HEAP32[$pNew>>2]|0;
             $233 = $232;
             $234 = $233;
             HEAP32[$234>>2] = $230;
             $235 = (($233) + 4)|0;
             $236 = $235;
             HEAP32[$236>>2] = $231;
             $237 = HEAP32[$2>>2]|0;
             $238 = HEAP32[$pNew>>2]|0;
             $239 = (_whereLoopInsert($237,$238)|0);
             HEAP32[$rc>>2] = $239;
            }
           }
           $240 = HEAP32[$pTerm>>2]|0;
           $241 = (($240) + 48|0);
           HEAP32[$pTerm>>2] = $241;
          }
         }
        }
       }
      }
     }
    }
   }
  }
 } while(0);
 L32: while(1) {
  $242 = HEAP32[$rc>>2]|0;
  $243 = ($242|0)==(0);
  $244 = HEAP32[$pProbe>>2]|0;
  $245 = ($244|0)!=(0|0);
  $or$cond = $243 & $245;
  if (!($or$cond)) {
   label = 47;
   break;
  }
  $246 = HEAP32[$pProbe>>2]|0;
  $247 = (($246) + 36|0);
  $248 = HEAP32[$247>>2]|0;
  $249 = ($248|0)!=(0|0);
  if ($249) {
   $250 = HEAP32[$pSrc>>2]|0;
   $251 = (($250) + 40|0);
   $252 = HEAP32[$251>>2]|0;
   $253 = HEAP32[$pWC>>2]|0;
   $254 = HEAP32[$pProbe>>2]|0;
   $255 = (($254) + 36|0);
   $256 = HEAP32[$255>>2]|0;
   $257 = (_whereUsablePartialIndex($252,$253,$256)|0);
   $258 = ($257|0)!=(0);
   if ($258) {
    label = 29;
   }
  } else {
   label = 29;
  }
  if ((label|0) == 29) {
   label = 0;
   $259 = HEAP32[$pProbe>>2]|0;
   $260 = (($259) + 8|0);
   $261 = HEAP32[$260>>2]|0;
   $262 = HEAP16[$261>>1]|0;
   HEAP16[$rSize>>1] = $262;
   $263 = HEAP32[$pNew>>2]|0;
   $264 = (($263) + 24|0);
   HEAP16[$264>>1] = 0;
   $265 = HEAP32[$pNew>>2]|0;
   $266 = (($265) + 24|0);
   $267 = (($266) + 2|0);
   HEAP16[$267>>1] = 0;
   $268 = HEAP32[$pNew>>2]|0;
   $269 = (($268) + 40|0);
   HEAP16[$269>>1] = 0;
   $270 = HEAP32[$pNew>>2]|0;
   $271 = (($270) + 17|0);
   HEAP8[$271>>0] = 0;
   $272 = HEAP32[$pNew>>2]|0;
   $273 = (($272) + 18|0);
   HEAP16[$273>>1] = 0;
   $274 = $3;
   $275 = $274;
   $276 = HEAP32[$275>>2]|0;
   $277 = (($274) + 4)|0;
   $278 = $277;
   $279 = HEAP32[$278>>2]|0;
   $280 = HEAP32[$pNew>>2]|0;
   $281 = $280;
   $282 = $281;
   HEAP32[$282>>2] = $276;
   $283 = (($281) + 4)|0;
   $284 = $283;
   HEAP32[$284>>2] = $279;
   $285 = HEAP16[$rSize>>1]|0;
   $286 = HEAP32[$pNew>>2]|0;
   $287 = (($286) + 22|0);
   HEAP16[$287>>1] = $285;
   $288 = HEAP32[$pProbe>>2]|0;
   $289 = HEAP32[$pNew>>2]|0;
   $290 = (($289) + 24|0);
   $291 = (($290) + 4|0);
   HEAP32[$291>>2] = $288;
   $292 = HEAP32[$2>>2]|0;
   $293 = HEAP32[$pProbe>>2]|0;
   $294 = HEAP32[$pSrc>>2]|0;
   $295 = (($294) + 40|0);
   $296 = HEAP32[$295>>2]|0;
   $297 = (_indexMightHelpWithOrderBy($292,$293,$296)|0);
   HEAP32[$b>>2] = $297;
   $298 = HEAP32[$pProbe>>2]|0;
   $299 = (($298) + 44|0);
   $300 = HEAP32[$299>>2]|0;
   $301 = ($300|0)<=(0);
   do {
    if ($301) {
     $302 = HEAP32[$pNew>>2]|0;
     $303 = (($302) + 36|0);
     HEAP32[$303>>2] = 256;
     $304 = HEAP32[$b>>2]|0;
     $305 = ($304|0)!=(0);
     $306 = HEAP32[$iSortIdx>>2]|0;
     $307 = $305 ? $306 : 0;
     $308 = $307&255;
     $309 = HEAP32[$pNew>>2]|0;
     $310 = (($309) + 17|0);
     HEAP8[$310>>0] = $308;
     $311 = HEAP16[$rSize>>1]|0;
     $312 = $311 << 16 >> 16;
     $313 = (($312) + 16)|0;
     $314 = $313&65535;
     $315 = HEAP32[$pNew>>2]|0;
     $316 = (($315) + 20|0);
     HEAP16[$316>>1] = $314;
     $317 = HEAP32[$pWC>>2]|0;
     $318 = HEAP32[$pNew>>2]|0;
     $319 = HEAP16[$rSize>>1]|0;
     _whereLoopOutputAdjust($317,$318,$319);
     $320 = HEAP32[$2>>2]|0;
     $321 = HEAP32[$pNew>>2]|0;
     $322 = (_whereLoopInsert($320,$321)|0);
     HEAP32[$rc>>2] = $322;
     $323 = HEAP16[$rSize>>1]|0;
     $324 = HEAP32[$pNew>>2]|0;
     $325 = (($324) + 22|0);
     HEAP16[$325>>1] = $323;
     $326 = HEAP32[$rc>>2]|0;
     $327 = ($326|0)!=(0);
     if ($327) {
      label = 47;
      break L32;
     }
    } else {
     $328 = HEAP32[$pProbe>>2]|0;
     $329 = (($328) + 55|0);
     $330 = HEAP8[$329>>0]|0;
     $331 = ($330&255) >>> 5;
     $332 = $331 & 1;
     $333 = $332&255;
     $334 = ($333|0)!=(0);
     if ($334) {
      $335 = HEAP32[$pNew>>2]|0;
      $336 = (($335) + 36|0);
      HEAP32[$336>>2] = 576;
      $337 = $m;
      $338 = $337;
      HEAP32[$338>>2] = 0;
      $339 = (($337) + 4)|0;
      $340 = $339;
      HEAP32[$340>>2] = 0;
     } else {
      $341 = HEAP32[$pSrc>>2]|0;
      $342 = (($341) + 56|0);
      $343 = $342;
      $344 = $343;
      $345 = HEAP32[$344>>2]|0;
      $346 = (($343) + 4)|0;
      $347 = $346;
      $348 = HEAP32[$347>>2]|0;
      $349 = HEAP32[$pProbe>>2]|0;
      $350 = (_columnsInIndex($349)|0);
      $351 = tempRet0;
      $352 = $350 ^ -1;
      $353 = $351 ^ -1;
      $354 = $345 & $352;
      $355 = $348 & $353;
      $356 = $m;
      $357 = $356;
      HEAP32[$357>>2] = $354;
      $358 = (($356) + 4)|0;
      $359 = $358;
      HEAP32[$359>>2] = $355;
      $360 = $m;
      $361 = $360;
      $362 = HEAP32[$361>>2]|0;
      $363 = (($360) + 4)|0;
      $364 = $363;
      $365 = HEAP32[$364>>2]|0;
      $366 = ($362|0)==(0);
      $367 = ($365|0)==(0);
      $368 = $366 & $367;
      $369 = $368 ? 576 : 512;
      $370 = HEAP32[$pNew>>2]|0;
      $371 = (($370) + 36|0);
      HEAP32[$371>>2] = $369;
     }
     $372 = HEAP32[$b>>2]|0;
     $373 = ($372|0)!=(0);
     if (!($373)) {
      $374 = HEAP32[$pTab>>2]|0;
      $375 = (($374) + 44|0);
      $376 = HEAP8[$375>>0]|0;
      $377 = $376&255;
      $378 = $377 & 32;
      $379 = ($378|0)==(0);
      if ($379) {
       $380 = $m;
       $381 = $380;
       $382 = HEAP32[$381>>2]|0;
       $383 = (($380) + 4)|0;
       $384 = $383;
       $385 = HEAP32[$384>>2]|0;
       $386 = ($382|0)==(0);
       $387 = ($385|0)==(0);
       $388 = $386 & $387;
       if (!($388)) {
        break;
       }
       $389 = HEAP32[$pProbe>>2]|0;
       $390 = (($389) + 55|0);
       $391 = HEAP8[$390>>0]|0;
       $392 = ($391&255) >>> 2;
       $393 = $392 & 1;
       $394 = $393&255;
       $395 = ($394|0)==(0);
       if (!($395)) {
        break;
       }
       $396 = HEAP32[$pProbe>>2]|0;
       $397 = (($396) + 48|0);
       $398 = HEAP16[$397>>1]|0;
       $399 = $398 << 16 >> 16;
       $400 = HEAP32[$pTab>>2]|0;
       $401 = (($400) + 42|0);
       $402 = HEAP16[$401>>1]|0;
       $403 = $402 << 16 >> 16;
       $404 = ($399|0)<($403|0);
       if (!($404)) {
        break;
       }
       $405 = HEAP32[$pWInfo>>2]|0;
       $406 = (($405) + 34|0);
       $407 = HEAP16[$406>>1]|0;
       $408 = $407&65535;
       $409 = $408 & 4;
       $410 = ($409|0)==(0);
       if (!($410)) {
        break;
       }
       $411 = HEAP32[((38640 + 16|0))>>2]|0;
       $412 = ($411|0)!=(0);
       if (!($412)) {
        break;
       }
       $413 = HEAP32[$pWInfo>>2]|0;
       $414 = HEAP32[$413>>2]|0;
       $415 = HEAP32[$414>>2]|0;
       $416 = (($415) + 60|0);
       $417 = HEAP16[$416>>1]|0;
       $418 = $417&65535;
       $419 = $418 & 64;
       $420 = ($419|0)==(0);
       if (!($420)) {
        break;
       }
      }
     }
     $421 = HEAP32[$b>>2]|0;
     $422 = ($421|0)!=(0);
     $423 = HEAP32[$iSortIdx>>2]|0;
     $424 = $422 ? $423 : 0;
     $425 = $424&255;
     $426 = HEAP32[$pNew>>2]|0;
     $427 = (($426) + 17|0);
     HEAP8[$427>>0] = $425;
     $428 = HEAP16[$rSize>>1]|0;
     $429 = $428 << 16 >> 16;
     $430 = (($429) + 1)|0;
     $431 = HEAP32[$pProbe>>2]|0;
     $432 = (($431) + 48|0);
     $433 = HEAP16[$432>>1]|0;
     $434 = $433 << 16 >> 16;
     $435 = ($434*15)|0;
     $436 = HEAP32[$pTab>>2]|0;
     $437 = (($436) + 42|0);
     $438 = HEAP16[$437>>1]|0;
     $439 = $438 << 16 >> 16;
     $440 = (($435|0) / ($439|0))&-1;
     $441 = (($430) + ($440))|0;
     $442 = $441&65535;
     $443 = HEAP32[$pNew>>2]|0;
     $444 = (($443) + 20|0);
     HEAP16[$444>>1] = $442;
     $445 = $m;
     $446 = $445;
     $447 = HEAP32[$446>>2]|0;
     $448 = (($445) + 4)|0;
     $449 = $448;
     $450 = HEAP32[$449>>2]|0;
     $451 = ($447|0)!=(0);
     $452 = ($450|0)!=(0);
     $453 = $451 | $452;
     if ($453) {
      $454 = HEAP32[$pNew>>2]|0;
      $455 = (($454) + 20|0);
      $456 = HEAP16[$455>>1]|0;
      $457 = HEAP16[$rSize>>1]|0;
      $458 = $457 << 16 >> 16;
      $459 = (($458) + 16)|0;
      $460 = $459&65535;
      $461 = (_sqlite3LogEstAdd($456,$460)|0);
      $462 = HEAP32[$pNew>>2]|0;
      $463 = (($462) + 20|0);
      HEAP16[$463>>1] = $461;
     }
     $464 = HEAP32[$pWC>>2]|0;
     $465 = HEAP32[$pNew>>2]|0;
     $466 = HEAP16[$rSize>>1]|0;
     _whereLoopOutputAdjust($464,$465,$466);
     $467 = HEAP32[$2>>2]|0;
     $468 = HEAP32[$pNew>>2]|0;
     $469 = (_whereLoopInsert($467,$468)|0);
     HEAP32[$rc>>2] = $469;
     $470 = HEAP16[$rSize>>1]|0;
     $471 = HEAP32[$pNew>>2]|0;
     $472 = (($471) + 22|0);
     HEAP16[$472>>1] = $470;
     $473 = HEAP32[$rc>>2]|0;
     $474 = ($473|0)!=(0);
     if ($474) {
      label = 47;
      break L32;
     }
    }
   } while(0);
   $475 = HEAP32[$2>>2]|0;
   $476 = HEAP32[$pSrc>>2]|0;
   $477 = HEAP32[$pProbe>>2]|0;
   $478 = (_whereLoopAddBtreeIndex($475,$476,$477,0)|0);
   HEAP32[$rc>>2] = $478;
   $479 = HEAP32[$pSrc>>2]|0;
   $480 = (($479) + 68|0);
   $481 = HEAP32[$480>>2]|0;
   $482 = ($481|0)!=(0|0);
   if ($482) {
    label = 47;
    break;
   }
  }
  $483 = HEAP32[$pProbe>>2]|0;
  $484 = (($483) + 20|0);
  $485 = HEAP32[$484>>2]|0;
  HEAP32[$pProbe>>2] = $485;
  $486 = HEAP32[$iSortIdx>>2]|0;
  $487 = (($486) + 1)|0;
  HEAP32[$iSortIdx>>2] = $487;
 }
 if ((label|0) == 47) {
  $488 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($488|0);
 }
 return 0|0;
}
function _whereLoopAddOr($pBuilder,$0,$1) {
 $pBuilder = $pBuilder|0;
 $0 = $0|0;
 $1 = $1|0;
 var $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0, $116 = 0, $117 = 0;
 var $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0, $134 = 0, $135 = 0;
 var $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0, $152 = 0, $153 = 0;
 var $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0, $170 = 0, $171 = 0;
 var $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0, $188 = 0, $189 = 0, $19 = 0;
 var $190 = 0, $191 = 0, $192 = 0, $193 = 0, $194 = 0, $195 = 0, $196 = 0, $197 = 0, $198 = 0, $199 = 0, $2 = 0, $20 = 0, $200 = 0, $201 = 0, $202 = 0, $203 = 0, $204 = 0, $205 = 0, $206 = 0, $207 = 0;
 var $208 = 0, $209 = 0, $21 = 0, $210 = 0, $211 = 0, $212 = 0, $213 = 0, $214 = 0, $215 = 0, $216 = 0, $217 = 0, $218 = 0, $219 = 0, $22 = 0, $220 = 0, $221 = 0, $222 = 0, $223 = 0, $224 = 0, $225 = 0;
 var $226 = 0, $227 = 0, $228 = 0, $229 = 0, $23 = 0, $230 = 0, $231 = 0, $232 = 0, $233 = 0, $234 = 0, $235 = 0, $236 = 0, $237 = 0, $238 = 0, $239 = 0, $24 = 0, $240 = 0, $241 = 0, $242 = 0, $243 = 0;
 var $244 = 0, $245 = 0, $246 = 0, $247 = 0, $248 = 0, $249 = 0, $25 = 0, $250 = 0, $251 = 0, $252 = 0, $253 = 0, $254 = 0, $255 = 0, $256 = 0, $257 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0;
 var $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0;
 var $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0;
 var $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0;
 var $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $i = 0, $iCur = 0, $j = 0, $once = 0;
 var $or$cond = 0, $pItem = 0, $pNew = 0, $pOrTerm = 0, $pOrWC = 0, $pOrWCEnd = 0, $pTerm = 0, $pWC = 0, $pWCEnd = 0, $pWInfo = 0, $rc = 0, $sCur = 0, $sPrev = 0, $sSubBuild = 0, $sSum = 0, $tempWC = 0, dest = 0, label = 0, sp = 0, stop = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 672|0;
 $2 = sp + 660|0;
 $3 = sp + 576|0;
 $pWInfo = sp + 656|0;
 $pWC = sp + 652|0;
 $pNew = sp + 648|0;
 $pTerm = sp + 644|0;
 $pWCEnd = sp + 640|0;
 $rc = sp + 636|0;
 $iCur = sp + 632|0;
 $tempWC = sp + 168|0;
 $sSubBuild = sp + 612|0;
 $sSum = sp + 112|0;
 $sCur = sp + 56|0;
 $pItem = sp + 608|0;
 $pOrWC = sp + 604|0;
 $pOrWCEnd = sp + 600|0;
 $pOrTerm = sp + 596|0;
 $once = sp + 592|0;
 $i = sp + 588|0;
 $j = sp + 584|0;
 $sPrev = sp;
 HEAP32[$2>>2] = $pBuilder;
 $4 = $3;
 $5 = $4;
 HEAP32[$5>>2] = $0;
 $6 = (($4) + 4)|0;
 $7 = $6;
 HEAP32[$7>>2] = $1;
 $8 = HEAP32[$2>>2]|0;
 $9 = HEAP32[$8>>2]|0;
 HEAP32[$pWInfo>>2] = $9;
 HEAP32[$rc>>2] = 0;
 $10 = HEAP32[$2>>2]|0;
 $11 = (($10) + 4|0);
 $12 = HEAP32[$11>>2]|0;
 HEAP32[$pWC>>2] = $12;
 $13 = HEAP32[$pWC>>2]|0;
 $14 = (($13) + 20|0);
 $15 = HEAP32[$14>>2]|0;
 $16 = HEAP32[$pWC>>2]|0;
 $17 = (($16) + 12|0);
 $18 = HEAP32[$17>>2]|0;
 $19 = (($15) + (($18*48)|0)|0);
 HEAP32[$pWCEnd>>2] = $19;
 $20 = HEAP32[$2>>2]|0;
 $21 = (($20) + 12|0);
 $22 = HEAP32[$21>>2]|0;
 HEAP32[$pNew>>2] = $22;
 dest=$sSum+0|0; stop=dest+56|0; do { HEAP32[dest>>2]=0|0; dest=dest+4|0; } while ((dest|0) < (stop|0));
 $23 = HEAP32[$pWInfo>>2]|0;
 $24 = (($23) + 4|0);
 $25 = HEAP32[$24>>2]|0;
 $26 = (($25) + 8|0);
 $27 = HEAP32[$pNew>>2]|0;
 $28 = (($27) + 16|0);
 $29 = HEAP8[$28>>0]|0;
 $30 = $29&255;
 $31 = (($26) + (($30*72)|0)|0);
 HEAP32[$pItem>>2] = $31;
 $32 = HEAP32[$pItem>>2]|0;
 $33 = (($32) + 40|0);
 $34 = HEAP32[$33>>2]|0;
 HEAP32[$iCur>>2] = $34;
 $35 = HEAP32[$pWC>>2]|0;
 $36 = (($35) + 20|0);
 $37 = HEAP32[$36>>2]|0;
 HEAP32[$pTerm>>2] = $37;
 while(1) {
  $38 = HEAP32[$pTerm>>2]|0;
  $39 = HEAP32[$pWCEnd>>2]|0;
  $40 = ($38>>>0)<($39>>>0);
  $41 = HEAP32[$rc>>2]|0;
  $42 = ($41|0)==(0);
  $or$cond = $40 & $42;
  if (!($or$cond)) {
   break;
  }
  $43 = HEAP32[$pTerm>>2]|0;
  $44 = (($43) + 18|0);
  $45 = HEAP16[$44>>1]|0;
  $46 = $45&65535;
  $47 = $46 & 256;
  $48 = ($47|0)!=(0);
  L4: do {
   if ($48) {
    $49 = HEAP32[$pTerm>>2]|0;
    $50 = (($49) + 12|0);
    $51 = HEAP32[$50>>2]|0;
    $52 = (($51) + 408|0);
    $53 = $52;
    $54 = $53;
    $55 = HEAP32[$54>>2]|0;
    $56 = (($53) + 4)|0;
    $57 = $56;
    $58 = HEAP32[$57>>2]|0;
    $59 = HEAP32[$pNew>>2]|0;
    $60 = (($59) + 8|0);
    $61 = $60;
    $62 = $61;
    $63 = HEAP32[$62>>2]|0;
    $64 = (($61) + 4)|0;
    $65 = $64;
    $66 = HEAP32[$65>>2]|0;
    $67 = $55 & $63;
    $68 = $58 & $66;
    $69 = ($67|0)!=(0);
    $70 = ($68|0)!=(0);
    $71 = $69 | $70;
    if ($71) {
     $72 = HEAP32[$pTerm>>2]|0;
     $73 = (($72) + 12|0);
     $74 = HEAP32[$73>>2]|0;
     HEAP32[$pOrWC>>2] = $74;
     $75 = HEAP32[$pOrWC>>2]|0;
     $76 = (($75) + 12|0);
     $77 = HEAP32[$76>>2]|0;
     $78 = HEAP32[$pOrWC>>2]|0;
     $79 = (($78) + 20|0);
     $80 = HEAP32[$79>>2]|0;
     $81 = (($80) + (($77*48)|0)|0);
     HEAP32[$pOrWCEnd>>2] = $81;
     HEAP32[$once>>2] = 1;
     $82 = HEAP32[$2>>2]|0;
     ;HEAP32[$sSubBuild+0>>2]=HEAP32[$82+0>>2]|0;HEAP32[$sSubBuild+4>>2]=HEAP32[$82+4>>2]|0;HEAP32[$sSubBuild+8>>2]=HEAP32[$82+8>>2]|0;HEAP32[$sSubBuild+12>>2]=HEAP32[$82+12>>2]|0;HEAP32[$sSubBuild+16>>2]=HEAP32[$82+16>>2]|0;
     $83 = (($sSubBuild) + 8|0);
     HEAP32[$83>>2] = 0;
     $84 = (($sSubBuild) + 16|0);
     HEAP32[$84>>2] = $sCur;
     $85 = HEAP32[$pOrWC>>2]|0;
     $86 = (($85) + 20|0);
     $87 = HEAP32[$86>>2]|0;
     HEAP32[$pOrTerm>>2] = $87;
     L7: while(1) {
      $88 = HEAP32[$pOrTerm>>2]|0;
      $89 = HEAP32[$pOrWCEnd>>2]|0;
      $90 = ($88>>>0)<($89>>>0);
      if (!($90)) {
       break;
      }
      $91 = HEAP32[$pOrTerm>>2]|0;
      $92 = (($91) + 18|0);
      $93 = HEAP16[$92>>1]|0;
      $94 = $93&65535;
      $95 = $94 & 512;
      $96 = ($95|0)!=(0);
      $97 = HEAP32[$pOrTerm>>2]|0;
      if ($96) {
       $98 = (($97) + 12|0);
       $99 = HEAP32[$98>>2]|0;
       $100 = (($sSubBuild) + 4|0);
       HEAP32[$100>>2] = $99;
       label = 11;
      } else {
       $101 = (($97) + 8|0);
       $102 = HEAP32[$101>>2]|0;
       $103 = HEAP32[$iCur>>2]|0;
       $104 = ($102|0)==($103|0);
       if ($104) {
        $105 = HEAP32[$pWC>>2]|0;
        $106 = HEAP32[$105>>2]|0;
        HEAP32[$tempWC>>2] = $106;
        $107 = HEAP32[$pWC>>2]|0;
        $108 = (($tempWC) + 4|0);
        HEAP32[$108>>2] = $107;
        $109 = (($tempWC) + 8|0);
        HEAP8[$109>>0] = 72;
        $110 = (($tempWC) + 12|0);
        HEAP32[$110>>2] = 1;
        $111 = HEAP32[$pOrTerm>>2]|0;
        $112 = (($tempWC) + 20|0);
        HEAP32[$112>>2] = $111;
        $113 = (($sSubBuild) + 4|0);
        HEAP32[$113>>2] = $tempWC;
        label = 11;
       }
      }
      L14: do {
       if ((label|0) == 11) {
        label = 0;
        HEAP16[$sCur>>1] = 0;
        $114 = HEAP32[$pItem>>2]|0;
        $115 = (($114) + 16|0);
        $116 = HEAP32[$115>>2]|0;
        $117 = (($116) + 44|0);
        $118 = HEAP8[$117>>0]|0;
        $119 = $118&255;
        $120 = $119 & 16;
        $121 = ($120|0)!=(0);
        $122 = $3;
        $123 = $122;
        $124 = HEAP32[$123>>2]|0;
        $125 = (($122) + 4)|0;
        $126 = $125;
        $127 = HEAP32[$126>>2]|0;
        if ($121) {
         $128 = (_whereLoopAddVirtual($sSubBuild,$124,$127)|0);
         HEAP32[$rc>>2] = $128;
        } else {
         $129 = (_whereLoopAddBtree($sSubBuild,$124,$127)|0);
         HEAP32[$rc>>2] = $129;
        }
        $130 = HEAP32[$rc>>2]|0;
        $131 = ($130|0)==(0);
        if ($131) {
         $132 = $3;
         $133 = $132;
         $134 = HEAP32[$133>>2]|0;
         $135 = (($132) + 4)|0;
         $136 = $135;
         $137 = HEAP32[$136>>2]|0;
         $138 = (_whereLoopAddOr($sSubBuild,$134,$137)|0);
         HEAP32[$rc>>2] = $138;
        }
        $139 = HEAP16[$sCur>>1]|0;
        $140 = $139&65535;
        $141 = ($140|0)==(0);
        if ($141) {
         label = 17;
         break L7;
        }
        $142 = HEAP32[$once>>2]|0;
        $143 = ($142|0)!=(0);
        if ($143) {
         _whereOrMove($sSum,$sCur);
         HEAP32[$once>>2] = 0;
         break;
        }
        _whereOrMove($sPrev,$sSum);
        HEAP16[$sSum>>1] = 0;
        HEAP32[$i>>2] = 0;
        while(1) {
         $144 = HEAP32[$i>>2]|0;
         $145 = HEAP16[$sPrev>>1]|0;
         $146 = $145&65535;
         $147 = ($144|0)<($146|0);
         if (!($147)) {
          break L14;
         }
         HEAP32[$j>>2] = 0;
         while(1) {
          $148 = HEAP32[$j>>2]|0;
          $149 = HEAP16[$sCur>>1]|0;
          $150 = $149&65535;
          $151 = ($148|0)<($150|0);
          $152 = HEAP32[$i>>2]|0;
          if (!($151)) {
           break;
          }
          $153 = (($sPrev) + 8|0);
          $154 = (($153) + ($152<<4)|0);
          $155 = $154;
          $156 = $155;
          $157 = HEAP32[$156>>2]|0;
          $158 = (($155) + 4)|0;
          $159 = $158;
          $160 = HEAP32[$159>>2]|0;
          $161 = HEAP32[$j>>2]|0;
          $162 = (($sCur) + 8|0);
          $163 = (($162) + ($161<<4)|0);
          $164 = $163;
          $165 = $164;
          $166 = HEAP32[$165>>2]|0;
          $167 = (($164) + 4)|0;
          $168 = $167;
          $169 = HEAP32[$168>>2]|0;
          $170 = $157 | $166;
          $171 = $160 | $169;
          $172 = HEAP32[$i>>2]|0;
          $173 = (($sPrev) + 8|0);
          $174 = (($173) + ($172<<4)|0);
          $175 = (($174) + 8|0);
          $176 = HEAP16[$175>>1]|0;
          $177 = HEAP32[$j>>2]|0;
          $178 = (($sCur) + 8|0);
          $179 = (($178) + ($177<<4)|0);
          $180 = (($179) + 8|0);
          $181 = HEAP16[$180>>1]|0;
          $182 = (_sqlite3LogEstAdd($176,$181)|0);
          $183 = HEAP32[$i>>2]|0;
          $184 = (($sPrev) + 8|0);
          $185 = (($184) + ($183<<4)|0);
          $186 = (($185) + 10|0);
          $187 = HEAP16[$186>>1]|0;
          $188 = HEAP32[$j>>2]|0;
          $189 = (($sCur) + 8|0);
          $190 = (($189) + ($188<<4)|0);
          $191 = (($190) + 10|0);
          $192 = HEAP16[$191>>1]|0;
          $193 = (_sqlite3LogEstAdd($187,$192)|0);
          (_whereOrInsert($sSum,$170,$171,$182,$193)|0);
          $194 = HEAP32[$j>>2]|0;
          $195 = (($194) + 1)|0;
          HEAP32[$j>>2] = $195;
         }
         $196 = (($152) + 1)|0;
         HEAP32[$i>>2] = $196;
        }
       }
      } while(0);
      $197 = HEAP32[$pOrTerm>>2]|0;
      $198 = (($197) + 48|0);
      HEAP32[$pOrTerm>>2] = $198;
     }
     if ((label|0) == 17) {
      label = 0;
      HEAP16[$sSum>>1] = 0;
     }
     $199 = HEAP32[$pNew>>2]|0;
     $200 = (($199) + 40|0);
     HEAP16[$200>>1] = 1;
     $201 = HEAP32[$pTerm>>2]|0;
     $202 = HEAP32[$pNew>>2]|0;
     $203 = (($202) + 44|0);
     $204 = HEAP32[$203>>2]|0;
     HEAP32[$204>>2] = $201;
     $205 = HEAP32[$pNew>>2]|0;
     $206 = (($205) + 36|0);
     HEAP32[$206>>2] = 8192;
     $207 = HEAP32[$pNew>>2]|0;
     $208 = (($207) + 18|0);
     HEAP16[$208>>1] = 0;
     $209 = HEAP32[$pNew>>2]|0;
     $210 = (($209) + 17|0);
     HEAP8[$210>>0] = 0;
     $211 = HEAP32[$pNew>>2]|0;
     $212 = (($211) + 24|0);
     ;HEAP32[$212+0>>2]=0|0;HEAP32[$212+4>>2]=0|0;HEAP32[$212+8>>2]=0|0;
     HEAP32[$i>>2] = 0;
     while(1) {
      $213 = HEAP32[$rc>>2]|0;
      $214 = ($213|0)==(0);
      if (!($214)) {
       break L4;
      }
      $215 = HEAP32[$i>>2]|0;
      $216 = HEAP16[$sSum>>1]|0;
      $217 = $216&65535;
      $218 = ($215|0)<($217|0);
      if (!($218)) {
       break L4;
      }
      $219 = HEAP32[$i>>2]|0;
      $220 = (($sSum) + 8|0);
      $221 = (($220) + ($219<<4)|0);
      $222 = (($221) + 8|0);
      $223 = HEAP16[$222>>1]|0;
      $224 = $223 << 16 >> 16;
      $225 = (($224) + 1)|0;
      $226 = $225&65535;
      $227 = HEAP32[$pNew>>2]|0;
      $228 = (($227) + 20|0);
      HEAP16[$228>>1] = $226;
      $229 = HEAP32[$i>>2]|0;
      $230 = (($sSum) + 8|0);
      $231 = (($230) + ($229<<4)|0);
      $232 = (($231) + 10|0);
      $233 = HEAP16[$232>>1]|0;
      $234 = HEAP32[$pNew>>2]|0;
      $235 = (($234) + 22|0);
      HEAP16[$235>>1] = $233;
      $236 = HEAP32[$i>>2]|0;
      $237 = (($sSum) + 8|0);
      $238 = (($237) + ($236<<4)|0);
      $239 = $238;
      $240 = $239;
      $241 = HEAP32[$240>>2]|0;
      $242 = (($239) + 4)|0;
      $243 = $242;
      $244 = HEAP32[$243>>2]|0;
      $245 = HEAP32[$pNew>>2]|0;
      $246 = $245;
      $247 = $246;
      HEAP32[$247>>2] = $241;
      $248 = (($246) + 4)|0;
      $249 = $248;
      HEAP32[$249>>2] = $244;
      $250 = HEAP32[$2>>2]|0;
      $251 = HEAP32[$pNew>>2]|0;
      $252 = (_whereLoopInsert($250,$251)|0);
      HEAP32[$rc>>2] = $252;
      $253 = HEAP32[$i>>2]|0;
      $254 = (($253) + 1)|0;
      HEAP32[$i>>2] = $254;
     }
    }
   }
  } while(0);
  $255 = HEAP32[$pTerm>>2]|0;
  $256 = (($255) + 48|0);
  HEAP32[$pTerm>>2] = $256;
 }
 $257 = HEAP32[$rc>>2]|0;
 STACKTOP = sp;return ($257|0);
}
function _whereOrMove($pDest,$pSrc) {
 $pDest = $pDest|0;
 $pSrc = $pSrc|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $1 = sp;
 HEAP32[$0>>2] = $pDest;
 HEAP32[$1>>2] = $pSrc;
 $2 = HEAP32[$1>>2]|0;
 $3 = HEAP16[$2>>1]|0;
 $4 = HEAP32[$0>>2]|0;
 HEAP16[$4>>1] = $3;
 $5 = HEAP32[$0>>2]|0;
 $6 = (($5) + 8|0);
 $7 = HEAP32[$1>>2]|0;
 $8 = (($7) + 8|0);
 $9 = HEAP32[$0>>2]|0;
 $10 = HEAP16[$9>>1]|0;
 $11 = $10&65535;
 $12 = $11<<4;
 _memcpy(($6|0),($8|0),($12|0))|0;
 STACKTOP = sp;return;
}
function _whereOrInsert($pSet,$0,$1,$rRun,$nOut) {
 $pSet = $pSet|0;
 $0 = $0|0;
 $1 = $1|0;
 $rRun = $rRun|0;
 $nOut = $nOut|0;
 var $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0, $116 = 0, $117 = 0;
 var $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0, $134 = 0, $135 = 0;
 var $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0, $152 = 0, $153 = 0;
 var $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0;
 var $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0;
 var $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0;
 var $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0;
 var $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $i = 0, $p = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $2 = sp + 16|0;
 $3 = sp + 12|0;
 $4 = sp;
 $5 = sp + 24|0;
 $6 = sp + 22|0;
 $i = sp + 20|0;
 $p = sp + 8|0;
 HEAP32[$3>>2] = $pSet;
 $7 = $4;
 $8 = $7;
 HEAP32[$8>>2] = $0;
 $9 = (($7) + 4)|0;
 $10 = $9;
 HEAP32[$10>>2] = $1;
 HEAP16[$5>>1] = $rRun;
 HEAP16[$6>>1] = $nOut;
 $11 = HEAP32[$3>>2]|0;
 $12 = HEAP16[$11>>1]|0;
 HEAP16[$i>>1] = $12;
 $13 = HEAP32[$3>>2]|0;
 $14 = (($13) + 8|0);
 HEAP32[$p>>2] = $14;
 while(1) {
  $15 = HEAP16[$i>>1]|0;
  $16 = $15&65535;
  $17 = ($16|0)>(0);
  if (!($17)) {
   label = 9;
   break;
  }
  $18 = HEAP16[$5>>1]|0;
  $19 = $18 << 16 >> 16;
  $20 = HEAP32[$p>>2]|0;
  $21 = (($20) + 8|0);
  $22 = HEAP16[$21>>1]|0;
  $23 = $22 << 16 >> 16;
  $24 = ($19|0)<=($23|0);
  if ($24) {
   $25 = $4;
   $26 = $25;
   $27 = HEAP32[$26>>2]|0;
   $28 = (($25) + 4)|0;
   $29 = $28;
   $30 = HEAP32[$29>>2]|0;
   $31 = HEAP32[$p>>2]|0;
   $32 = $31;
   $33 = $32;
   $34 = HEAP32[$33>>2]|0;
   $35 = (($32) + 4)|0;
   $36 = $35;
   $37 = HEAP32[$36>>2]|0;
   $38 = $27 & $34;
   $39 = $30 & $37;
   $40 = $4;
   $41 = $40;
   $42 = HEAP32[$41>>2]|0;
   $43 = (($40) + 4)|0;
   $44 = $43;
   $45 = HEAP32[$44>>2]|0;
   $46 = ($38|0)==($42|0);
   $47 = ($39|0)==($45|0);
   $48 = $46 & $47;
   if ($48) {
    break;
   }
  }
  $49 = HEAP32[$p>>2]|0;
  $50 = (($49) + 8|0);
  $51 = HEAP16[$50>>1]|0;
  $52 = $51 << 16 >> 16;
  $53 = HEAP16[$5>>1]|0;
  $54 = $53 << 16 >> 16;
  $55 = ($52|0)<=($54|0);
  if ($55) {
   $56 = HEAP32[$p>>2]|0;
   $57 = $56;
   $58 = $57;
   $59 = HEAP32[$58>>2]|0;
   $60 = (($57) + 4)|0;
   $61 = $60;
   $62 = HEAP32[$61>>2]|0;
   $63 = $4;
   $64 = $63;
   $65 = HEAP32[$64>>2]|0;
   $66 = (($63) + 4)|0;
   $67 = $66;
   $68 = HEAP32[$67>>2]|0;
   $69 = $59 & $65;
   $70 = $62 & $68;
   $71 = HEAP32[$p>>2]|0;
   $72 = $71;
   $73 = $72;
   $74 = HEAP32[$73>>2]|0;
   $75 = (($72) + 4)|0;
   $76 = $75;
   $77 = HEAP32[$76>>2]|0;
   $78 = ($69|0)==($74|0);
   $79 = ($70|0)==($77|0);
   $80 = $78 & $79;
   if ($80) {
    label = 7;
    break;
   }
  }
  $81 = HEAP16[$i>>1]|0;
  $82 = (($81) + -1)<<16>>16;
  HEAP16[$i>>1] = $82;
  $83 = HEAP32[$p>>2]|0;
  $84 = (($83) + 16|0);
  HEAP32[$p>>2] = $84;
 }
 if ((label|0) == 7) {
  HEAP32[$2>>2] = 0;
  $153 = HEAP32[$2>>2]|0;
  STACKTOP = sp;return ($153|0);
 }
 do {
  if ((label|0) == 9) {
   $85 = HEAP32[$3>>2]|0;
   $86 = HEAP16[$85>>1]|0;
   $87 = $86&65535;
   $88 = ($87|0)<(3);
   $89 = HEAP32[$3>>2]|0;
   if ($88) {
    $90 = HEAP16[$89>>1]|0;
    $91 = (($90) + 1)<<16>>16;
    HEAP16[$89>>1] = $91;
    $92 = $90&65535;
    $93 = HEAP32[$3>>2]|0;
    $94 = (($93) + 8|0);
    $95 = (($94) + ($92<<4)|0);
    HEAP32[$p>>2] = $95;
    $96 = HEAP16[$6>>1]|0;
    $97 = HEAP32[$p>>2]|0;
    $98 = (($97) + 10|0);
    HEAP16[$98>>1] = $96;
    break;
   }
   $99 = (($89) + 8|0);
   HEAP32[$p>>2] = $99;
   HEAP16[$i>>1] = 1;
   while(1) {
    $100 = HEAP16[$i>>1]|0;
    $101 = $100&65535;
    $102 = HEAP32[$3>>2]|0;
    $103 = HEAP16[$102>>1]|0;
    $104 = $103&65535;
    $105 = ($101|0)<($104|0);
    $106 = HEAP32[$p>>2]|0;
    $107 = (($106) + 8|0);
    $108 = HEAP16[$107>>1]|0;
    $109 = $108 << 16 >> 16;
    if (!($105)) {
     break;
    }
    $110 = HEAP16[$i>>1]|0;
    $111 = $110&65535;
    $112 = HEAP32[$3>>2]|0;
    $113 = (($112) + 8|0);
    $114 = (($113) + ($111<<4)|0);
    $115 = (($114) + 8|0);
    $116 = HEAP16[$115>>1]|0;
    $117 = $116 << 16 >> 16;
    $118 = ($109|0)>($117|0);
    if ($118) {
     $119 = HEAP32[$3>>2]|0;
     $120 = (($119) + 8|0);
     $121 = HEAP16[$i>>1]|0;
     $122 = $121&65535;
     $123 = (($120) + ($122<<4)|0);
     HEAP32[$p>>2] = $123;
    }
    $124 = HEAP16[$i>>1]|0;
    $125 = (($124) + 1)<<16>>16;
    HEAP16[$i>>1] = $125;
   }
   $126 = HEAP16[$5>>1]|0;
   $127 = $126 << 16 >> 16;
   $128 = ($109|0)<=($127|0);
   if ($128) {
    HEAP32[$2>>2] = 0;
    $153 = HEAP32[$2>>2]|0;
    STACKTOP = sp;return ($153|0);
   }
  }
 } while(0);
 $129 = $4;
 $130 = $129;
 $131 = HEAP32[$130>>2]|0;
 $132 = (($129) + 4)|0;
 $133 = $132;
 $134 = HEAP32[$133>>2]|0;
 $135 = HEAP32[$p>>2]|0;
 $136 = $135;
 $137 = $136;
 HEAP32[$137>>2] = $131;
 $138 = (($136) + 4)|0;
 $139 = $138;
 HEAP32[$139>>2] = $134;
 $140 = HEAP16[$5>>1]|0;
 $141 = HEAP32[$p>>2]|0;
 $142 = (($141) + 8|0);
 HEAP16[$142>>1] = $140;
 $143 = HEAP32[$p>>2]|0;
 $144 = (($143) + 10|0);
 $145 = HEAP16[$144>>1]|0;
 $146 = $145 << 16 >> 16;
 $147 = HEAP16[$6>>1]|0;
 $148 = $147 << 16 >> 16;
 $149 = ($146|0)>($148|0);
 if ($149) {
  $150 = HEAP16[$6>>1]|0;
  $151 = HEAP32[$p>>2]|0;
  $152 = (($151) + 10|0);
  HEAP16[$152>>1] = $150;
 }
 HEAP32[$2>>2] = 1;
 $153 = HEAP32[$2>>2]|0;
 STACKTOP = sp;return ($153|0);
}
function _whereLoopInsert($pBuilder,$pTemplate) {
 $pBuilder = $pBuilder|0;
 $pTemplate = $pTemplate|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0;
 var $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $db = 0, $p = 0, $pIndex = 0, $pToDel = 0, $pWInfo = 0, $ppPrev = 0, $ppTail = 0, label = 0;
 var sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $0 = sp + 36|0;
 $1 = sp + 32|0;
 $2 = sp + 28|0;
 $ppPrev = sp + 24|0;
 $p = sp + 20|0;
 $pWInfo = sp + 16|0;
 $db = sp + 12|0;
 $ppTail = sp + 8|0;
 $pToDel = sp + 4|0;
 $pIndex = sp;
 HEAP32[$1>>2] = $pBuilder;
 HEAP32[$2>>2] = $pTemplate;
 $3 = HEAP32[$1>>2]|0;
 $4 = HEAP32[$3>>2]|0;
 HEAP32[$pWInfo>>2] = $4;
 $5 = HEAP32[$pWInfo>>2]|0;
 $6 = HEAP32[$5>>2]|0;
 $7 = HEAP32[$6>>2]|0;
 HEAP32[$db>>2] = $7;
 $8 = HEAP32[$1>>2]|0;
 $9 = (($8) + 16|0);
 $10 = HEAP32[$9>>2]|0;
 $11 = ($10|0)!=(0|0);
 if ($11) {
  $12 = HEAP32[$1>>2]|0;
  $13 = (($12) + 16|0);
  $14 = HEAP32[$13>>2]|0;
  $15 = HEAP32[$2>>2]|0;
  $16 = $15;
  $17 = $16;
  $18 = HEAP32[$17>>2]|0;
  $19 = (($16) + 4)|0;
  $20 = $19;
  $21 = HEAP32[$20>>2]|0;
  $22 = HEAP32[$2>>2]|0;
  $23 = (($22) + 20|0);
  $24 = HEAP16[$23>>1]|0;
  $25 = HEAP32[$2>>2]|0;
  $26 = (($25) + 22|0);
  $27 = HEAP16[$26>>1]|0;
  (_whereOrInsert($14,$18,$21,$24,$27)|0);
  HEAP32[$0>>2] = 0;
  $91 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($91|0);
 }
 $28 = HEAP32[$pWInfo>>2]|0;
 $29 = (($28) + 16|0);
 $30 = HEAP32[$29>>2]|0;
 $31 = HEAP32[$2>>2]|0;
 _whereLoopAdjustCost($30,$31);
 $32 = HEAP32[$pWInfo>>2]|0;
 $33 = (($32) + 16|0);
 $34 = HEAP32[$2>>2]|0;
 $35 = (_whereLoopFindLesser($33,$34)|0);
 HEAP32[$ppPrev>>2] = $35;
 $36 = HEAP32[$ppPrev>>2]|0;
 $37 = ($36|0)==(0|0);
 if ($37) {
  HEAP32[$0>>2] = 0;
  $91 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($91|0);
 }
 $38 = HEAP32[$ppPrev>>2]|0;
 $39 = HEAP32[$38>>2]|0;
 HEAP32[$p>>2] = $39;
 $40 = HEAP32[$p>>2]|0;
 $41 = ($40|0)==(0|0);
 L9: do {
  if ($41) {
   $42 = HEAP32[$db>>2]|0;
   $43 = (_sqlite3DbMallocRaw($42,72,0)|0);
   HEAP32[$p>>2] = $43;
   $44 = HEAP32[$ppPrev>>2]|0;
   HEAP32[$44>>2] = $43;
   $45 = HEAP32[$p>>2]|0;
   $46 = ($45|0)==(0|0);
   if (!($46)) {
    $47 = HEAP32[$p>>2]|0;
    _whereLoopInit($47);
    $48 = HEAP32[$p>>2]|0;
    $49 = (($48) + 48|0);
    HEAP32[$49>>2] = 0;
    break;
   }
   HEAP32[$0>>2] = 7;
   $91 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($91|0);
  } else {
   $50 = HEAP32[$p>>2]|0;
   $51 = (($50) + 48|0);
   HEAP32[$ppTail>>2] = $51;
   while(1) {
    $52 = HEAP32[$ppTail>>2]|0;
    $53 = HEAP32[$52>>2]|0;
    $54 = ($53|0)!=(0|0);
    if (!($54)) {
     break L9;
    }
    $55 = HEAP32[$ppTail>>2]|0;
    $56 = HEAP32[$2>>2]|0;
    $57 = (_whereLoopFindLesser($55,$56)|0);
    HEAP32[$ppTail>>2] = $57;
    $58 = HEAP32[$ppTail>>2]|0;
    $59 = ($58|0)==(0|0);
    if ($59) {
     break L9;
    }
    $60 = HEAP32[$ppTail>>2]|0;
    $61 = HEAP32[$60>>2]|0;
    HEAP32[$pToDel>>2] = $61;
    $62 = HEAP32[$pToDel>>2]|0;
    $63 = ($62|0)==(0|0);
    if ($63) {
     break L9;
    }
    $64 = HEAP32[$pToDel>>2]|0;
    $65 = (($64) + 48|0);
    $66 = HEAP32[$65>>2]|0;
    $67 = HEAP32[$ppTail>>2]|0;
    HEAP32[$67>>2] = $66;
    $68 = HEAP32[$db>>2]|0;
    $69 = HEAP32[$pToDel>>2]|0;
    _whereLoopDelete($68,$69);
   }
  }
 } while(0);
 $70 = HEAP32[$db>>2]|0;
 $71 = HEAP32[$p>>2]|0;
 $72 = HEAP32[$2>>2]|0;
 (_whereLoopXfer($70,$71,$72)|0);
 $73 = HEAP32[$p>>2]|0;
 $74 = (($73) + 36|0);
 $75 = HEAP32[$74>>2]|0;
 $76 = $75 & 1024;
 $77 = ($76|0)==(0);
 if ($77) {
  $78 = HEAP32[$p>>2]|0;
  $79 = (($78) + 24|0);
  $80 = (($79) + 4|0);
  $81 = HEAP32[$80>>2]|0;
  HEAP32[$pIndex>>2] = $81;
  $82 = HEAP32[$pIndex>>2]|0;
  $83 = ($82|0)!=(0|0);
  if ($83) {
   $84 = HEAP32[$pIndex>>2]|0;
   $85 = (($84) + 44|0);
   $86 = HEAP32[$85>>2]|0;
   $87 = ($86|0)==(0);
   if ($87) {
    $88 = HEAP32[$p>>2]|0;
    $89 = (($88) + 24|0);
    $90 = (($89) + 4|0);
    HEAP32[$90>>2] = 0;
   }
  }
 }
 HEAP32[$0>>2] = 0;
 $91 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($91|0);
}
function _whereLoopAdjustCost($p,$pTemplate) {
 $p = $p|0;
 $pTemplate = $pTemplate|0;
 var $$old = 0, $$old1 = 0, $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0;
 var $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0;
 var $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0;
 var $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $8 = 0, $9 = 0, $or$cond = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $1 = sp;
 HEAP32[$0>>2] = $p;
 HEAP32[$1>>2] = $pTemplate;
 $2 = HEAP32[$1>>2]|0;
 $3 = (($2) + 36|0);
 $4 = HEAP32[$3>>2]|0;
 $5 = $4 & 512;
 $6 = ($5|0)==(0);
 if ($6) {
  STACKTOP = sp;return;
 }
 $7 = HEAP32[$1>>2]|0;
 $8 = (($7) + 36|0);
 $9 = HEAP32[$8>>2]|0;
 $10 = $9 & 32768;
 $11 = ($10|0)==(0);
 $12 = HEAP32[$0>>2]|0;
 $13 = ($12|0)!=(0|0);
 $or$cond = $11 & $13;
 if (!($or$cond)) {
  STACKTOP = sp;return;
 }
 while(1) {
  $14 = HEAP32[$0>>2]|0;
  $15 = (($14) + 16|0);
  $16 = HEAP8[$15>>0]|0;
  $17 = $16&255;
  $18 = HEAP32[$1>>2]|0;
  $19 = (($18) + 16|0);
  $20 = HEAP8[$19>>0]|0;
  $21 = $20&255;
  $22 = ($17|0)!=($21|0);
  do {
   if (!($22)) {
    $23 = HEAP32[$0>>2]|0;
    $24 = (($23) + 36|0);
    $25 = HEAP32[$24>>2]|0;
    $26 = $25 & 512;
    $27 = ($26|0)==(0);
    if (!($27)) {
     $28 = HEAP32[$0>>2]|0;
     $29 = (($28) + 36|0);
     $30 = HEAP32[$29>>2]|0;
     $31 = $30 & 32768;
     $32 = ($31|0)!=(0);
     if (!($32)) {
      $33 = HEAP32[$0>>2]|0;
      $34 = HEAP32[$1>>2]|0;
      $35 = (_whereLoopCheaperProperSubset($33,$34)|0);
      $36 = ($35|0)!=(0);
      if ($36) {
       $37 = HEAP32[$0>>2]|0;
       $38 = (($37) + 20|0);
       $39 = HEAP16[$38>>1]|0;
       $40 = HEAP32[$1>>2]|0;
       $41 = (($40) + 20|0);
       HEAP16[$41>>1] = $39;
       $42 = HEAP32[$0>>2]|0;
       $43 = (($42) + 22|0);
       $44 = HEAP16[$43>>1]|0;
       $45 = $44 << 16 >> 16;
       $46 = (($45) - 1)|0;
       $47 = $46&65535;
       $48 = HEAP32[$1>>2]|0;
       $49 = (($48) + 22|0);
       HEAP16[$49>>1] = $47;
       break;
      }
      $50 = HEAP32[$1>>2]|0;
      $51 = HEAP32[$0>>2]|0;
      $52 = (_whereLoopCheaperProperSubset($50,$51)|0);
      $53 = ($52|0)!=(0);
      if ($53) {
       $54 = HEAP32[$0>>2]|0;
       $55 = (($54) + 20|0);
       $56 = HEAP16[$55>>1]|0;
       $57 = HEAP32[$1>>2]|0;
       $58 = (($57) + 20|0);
       HEAP16[$58>>1] = $56;
       $59 = HEAP32[$0>>2]|0;
       $60 = (($59) + 22|0);
       $61 = HEAP16[$60>>1]|0;
       $62 = $61 << 16 >> 16;
       $63 = (($62) + 1)|0;
       $64 = $63&65535;
       $65 = HEAP32[$1>>2]|0;
       $66 = (($65) + 22|0);
       HEAP16[$66>>1] = $64;
      }
     }
    }
   }
  } while(0);
  $67 = HEAP32[$0>>2]|0;
  $68 = (($67) + 48|0);
  $69 = HEAP32[$68>>2]|0;
  HEAP32[$0>>2] = $69;
  $$old = HEAP32[$0>>2]|0;
  $$old1 = ($$old|0)!=(0|0);
  if (!($$old1)) {
   break;
  }
 }
 STACKTOP = sp;return;
}
function _whereLoopFindLesser($ppPrev,$pTemplate) {
 $ppPrev = $ppPrev|0;
 $pTemplate = $pTemplate|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $17 = 0, $18 = 0;
 var $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0;
 var $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0;
 var $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0;
 var $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0;
 var $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $p = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $1 = sp + 8|0;
 $2 = sp + 4|0;
 $p = sp;
 HEAP32[$1>>2] = $ppPrev;
 HEAP32[$2>>2] = $pTemplate;
 $3 = HEAP32[$1>>2]|0;
 $4 = HEAP32[$3>>2]|0;
 HEAP32[$p>>2] = $4;
 while(1) {
  $5 = HEAP32[$p>>2]|0;
  $6 = ($5|0)!=(0|0);
  if (!($6)) {
   label = 18;
   break;
  }
  $7 = HEAP32[$p>>2]|0;
  $8 = (($7) + 16|0);
  $9 = HEAP8[$8>>0]|0;
  $10 = $9&255;
  $11 = HEAP32[$2>>2]|0;
  $12 = (($11) + 16|0);
  $13 = HEAP8[$12>>0]|0;
  $14 = $13&255;
  $15 = ($10|0)!=($14|0);
  if (!($15)) {
   $16 = HEAP32[$p>>2]|0;
   $17 = (($16) + 17|0);
   $18 = HEAP8[$17>>0]|0;
   $19 = $18&255;
   $20 = HEAP32[$2>>2]|0;
   $21 = (($20) + 17|0);
   $22 = HEAP8[$21>>0]|0;
   $23 = $22&255;
   $24 = ($19|0)!=($23|0);
   if (!($24)) {
    $25 = HEAP32[$p>>2]|0;
    $26 = (($25) + 36|0);
    $27 = HEAP32[$26>>2]|0;
    $28 = $27 & 16384;
    $29 = ($28|0)!=(0);
    if ($29) {
     $30 = HEAP32[$2>>2]|0;
     $31 = (($30) + 36|0);
     $32 = HEAP32[$31>>2]|0;
     $33 = $32 & 512;
     $34 = ($33|0)!=(0);
     if ($34) {
      $35 = HEAP32[$2>>2]|0;
      $36 = (($35) + 36|0);
      $37 = HEAP32[$36>>2]|0;
      $38 = $37 & 1;
      $39 = ($38|0)!=(0);
      if ($39) {
       $40 = HEAP32[$p>>2]|0;
       $41 = $40;
       $42 = $41;
       $43 = HEAP32[$42>>2]|0;
       $44 = (($41) + 4)|0;
       $45 = $44;
       $46 = HEAP32[$45>>2]|0;
       $47 = HEAP32[$2>>2]|0;
       $48 = $47;
       $49 = $48;
       $50 = HEAP32[$49>>2]|0;
       $51 = (($48) + 4)|0;
       $52 = $51;
       $53 = HEAP32[$52>>2]|0;
       $54 = $43 & $50;
       $55 = $46 & $53;
       $56 = HEAP32[$2>>2]|0;
       $57 = $56;
       $58 = $57;
       $59 = HEAP32[$58>>2]|0;
       $60 = (($57) + 4)|0;
       $61 = $60;
       $62 = HEAP32[$61>>2]|0;
       $63 = ($54|0)==($59|0);
       $64 = ($55|0)==($62|0);
       $65 = $63 & $64;
       if ($65) {
        label = 18;
        break;
       }
      }
     }
    }
    $66 = HEAP32[$p>>2]|0;
    $67 = $66;
    $68 = $67;
    $69 = HEAP32[$68>>2]|0;
    $70 = (($67) + 4)|0;
    $71 = $70;
    $72 = HEAP32[$71>>2]|0;
    $73 = HEAP32[$2>>2]|0;
    $74 = $73;
    $75 = $74;
    $76 = HEAP32[$75>>2]|0;
    $77 = (($74) + 4)|0;
    $78 = $77;
    $79 = HEAP32[$78>>2]|0;
    $80 = $69 & $76;
    $81 = $72 & $79;
    $82 = HEAP32[$p>>2]|0;
    $83 = $82;
    $84 = $83;
    $85 = HEAP32[$84>>2]|0;
    $86 = (($83) + 4)|0;
    $87 = $86;
    $88 = HEAP32[$87>>2]|0;
    $89 = ($80|0)==($85|0);
    $90 = ($81|0)==($88|0);
    $91 = $89 & $90;
    if ($91) {
     $92 = HEAP32[$p>>2]|0;
     $93 = (($92) + 18|0);
     $94 = HEAP16[$93>>1]|0;
     $95 = $94 << 16 >> 16;
     $96 = HEAP32[$2>>2]|0;
     $97 = (($96) + 18|0);
     $98 = HEAP16[$97>>1]|0;
     $99 = $98 << 16 >> 16;
     $100 = ($95|0)<=($99|0);
     if ($100) {
      $101 = HEAP32[$p>>2]|0;
      $102 = (($101) + 20|0);
      $103 = HEAP16[$102>>1]|0;
      $104 = $103 << 16 >> 16;
      $105 = HEAP32[$2>>2]|0;
      $106 = (($105) + 20|0);
      $107 = HEAP16[$106>>1]|0;
      $108 = $107 << 16 >> 16;
      $109 = ($104|0)<=($108|0);
      if ($109) {
       $110 = HEAP32[$p>>2]|0;
       $111 = (($110) + 22|0);
       $112 = HEAP16[$111>>1]|0;
       $113 = $112 << 16 >> 16;
       $114 = HEAP32[$2>>2]|0;
       $115 = (($114) + 22|0);
       $116 = HEAP16[$115>>1]|0;
       $117 = $116 << 16 >> 16;
       $118 = ($113|0)<=($117|0);
       if ($118) {
        label = 13;
        break;
       }
      }
     }
    }
    $119 = HEAP32[$p>>2]|0;
    $120 = $119;
    $121 = $120;
    $122 = HEAP32[$121>>2]|0;
    $123 = (($120) + 4)|0;
    $124 = $123;
    $125 = HEAP32[$124>>2]|0;
    $126 = HEAP32[$2>>2]|0;
    $127 = $126;
    $128 = $127;
    $129 = HEAP32[$128>>2]|0;
    $130 = (($127) + 4)|0;
    $131 = $130;
    $132 = HEAP32[$131>>2]|0;
    $133 = $122 & $129;
    $134 = $125 & $132;
    $135 = HEAP32[$2>>2]|0;
    $136 = $135;
    $137 = $136;
    $138 = HEAP32[$137>>2]|0;
    $139 = (($136) + 4)|0;
    $140 = $139;
    $141 = HEAP32[$140>>2]|0;
    $142 = ($133|0)==($138|0);
    $143 = ($134|0)==($141|0);
    $144 = $142 & $143;
    if ($144) {
     $145 = HEAP32[$p>>2]|0;
     $146 = (($145) + 20|0);
     $147 = HEAP16[$146>>1]|0;
     $148 = $147 << 16 >> 16;
     $149 = HEAP32[$2>>2]|0;
     $150 = (($149) + 20|0);
     $151 = HEAP16[$150>>1]|0;
     $152 = $151 << 16 >> 16;
     $153 = ($148|0)>=($152|0);
     if ($153) {
      $154 = HEAP32[$p>>2]|0;
      $155 = (($154) + 22|0);
      $156 = HEAP16[$155>>1]|0;
      $157 = $156 << 16 >> 16;
      $158 = HEAP32[$2>>2]|0;
      $159 = (($158) + 22|0);
      $160 = HEAP16[$159>>1]|0;
      $161 = $160 << 16 >> 16;
      $162 = ($157|0)>=($161|0);
      if ($162) {
       label = 18;
       break;
      }
     }
    }
   }
  }
  $163 = HEAP32[$p>>2]|0;
  $164 = (($163) + 48|0);
  HEAP32[$1>>2] = $164;
  $165 = HEAP32[$1>>2]|0;
  $166 = HEAP32[$165>>2]|0;
  HEAP32[$p>>2] = $166;
 }
 if ((label|0) == 13) {
  HEAP32[$0>>2] = 0;
  $168 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($168|0);
 }
 else if ((label|0) == 18) {
  $167 = HEAP32[$1>>2]|0;
  HEAP32[$0>>2] = $167;
  $168 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($168|0);
 }
 return 0|0;
}
function _whereLoopXfer($db,$pTo,$pFrom) {
 $db = $db|0;
 $pTo = $pTo|0;
 $pFrom = $pFrom|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $5 = 0;
 var $6 = 0, $7 = 0, $8 = 0, $9 = 0, dest = 0, label = 0, sp = 0, src = 0, stop = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $1 = sp + 8|0;
 $2 = sp + 4|0;
 $3 = sp;
 HEAP32[$1>>2] = $db;
 HEAP32[$2>>2] = $pTo;
 HEAP32[$3>>2] = $pFrom;
 $4 = HEAP32[$1>>2]|0;
 $5 = HEAP32[$2>>2]|0;
 _whereLoopClearUnion($4,$5);
 $6 = HEAP32[$1>>2]|0;
 $7 = HEAP32[$2>>2]|0;
 $8 = HEAP32[$3>>2]|0;
 $9 = (($8) + 40|0);
 $10 = HEAP16[$9>>1]|0;
 $11 = $10&65535;
 $12 = (_whereLoopResize($6,$7,$11)|0);
 $13 = ($12|0)!=(0);
 $14 = HEAP32[$2>>2]|0;
 if ($13) {
  $15 = (($14) + 24|0);
  ;HEAP32[$15+0>>2]=0|0;HEAP32[$15+4>>2]=0|0;HEAP32[$15+8>>2]=0|0;
  HEAP32[$0>>2] = 7;
  $43 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($43|0);
 }
 $16 = HEAP32[$3>>2]|0;
 dest=$14+0|0; src=$16+0|0; stop=dest+40|0; do { HEAP32[dest>>2]=HEAP32[src>>2]|0; dest=dest+4|0; src=src+4|0; } while ((dest|0) < (stop|0));HEAP16[$14+40>>1]=HEAP16[$16+40>>1]|0;
 $17 = HEAP32[$2>>2]|0;
 $18 = (($17) + 44|0);
 $19 = HEAP32[$18>>2]|0;
 $20 = HEAP32[$3>>2]|0;
 $21 = (($20) + 44|0);
 $22 = HEAP32[$21>>2]|0;
 $23 = HEAP32[$2>>2]|0;
 $24 = (($23) + 40|0);
 $25 = HEAP16[$24>>1]|0;
 $26 = $25&65535;
 $27 = $26<<2;
 _memcpy(($19|0),($22|0),($27|0))|0;
 $28 = HEAP32[$3>>2]|0;
 $29 = (($28) + 36|0);
 $30 = HEAP32[$29>>2]|0;
 $31 = $30 & 1024;
 $32 = ($31|0)!=(0);
 $33 = HEAP32[$3>>2]|0;
 if ($32) {
  $34 = (($33) + 24|0);
  $35 = (($34) + 4|0);
  HEAP8[$35>>0] = 0;
 } else {
  $36 = (($33) + 36|0);
  $37 = HEAP32[$36>>2]|0;
  $38 = $37 & 16384;
  $39 = ($38|0)!=(0);
  if ($39) {
   $40 = HEAP32[$3>>2]|0;
   $41 = (($40) + 24|0);
   $42 = (($41) + 4|0);
   HEAP32[$42>>2] = 0;
  }
 }
 HEAP32[$0>>2] = 0;
 $43 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($43|0);
}
function _whereLoopCheaperProperSubset($pX,$pY) {
 $pX = $pX|0;
 $pY = $pY|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $8 = 0, $9 = 0, $i = 0, $j = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 16|0;
 $1 = sp + 12|0;
 $2 = sp + 8|0;
 $i = sp + 4|0;
 $j = sp;
 HEAP32[$1>>2] = $pX;
 HEAP32[$2>>2] = $pY;
 $3 = HEAP32[$1>>2]|0;
 $4 = (($3) + 40|0);
 $5 = HEAP16[$4>>1]|0;
 $6 = $5&65535;
 $7 = HEAP32[$2>>2]|0;
 $8 = (($7) + 40|0);
 $9 = HEAP16[$8>>1]|0;
 $10 = $9&65535;
 $11 = ($6|0)>=($10|0);
 if ($11) {
  HEAP32[$0>>2] = 0;
  $72 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($72|0);
 }
 $12 = HEAP32[$1>>2]|0;
 $13 = (($12) + 20|0);
 $14 = HEAP16[$13>>1]|0;
 $15 = $14 << 16 >> 16;
 $16 = HEAP32[$2>>2]|0;
 $17 = (($16) + 20|0);
 $18 = HEAP16[$17>>1]|0;
 $19 = $18 << 16 >> 16;
 $20 = ($15|0)>=($19|0);
 if ($20) {
  $21 = HEAP32[$1>>2]|0;
  $22 = (($21) + 20|0);
  $23 = HEAP16[$22>>1]|0;
  $24 = $23 << 16 >> 16;
  $25 = HEAP32[$2>>2]|0;
  $26 = (($25) + 20|0);
  $27 = HEAP16[$26>>1]|0;
  $28 = $27 << 16 >> 16;
  $29 = ($24|0)>($28|0);
  if ($29) {
   HEAP32[$0>>2] = 0;
   $72 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($72|0);
  }
  $30 = HEAP32[$1>>2]|0;
  $31 = (($30) + 22|0);
  $32 = HEAP16[$31>>1]|0;
  $33 = $32 << 16 >> 16;
  $34 = HEAP32[$2>>2]|0;
  $35 = (($34) + 22|0);
  $36 = HEAP16[$35>>1]|0;
  $37 = $36 << 16 >> 16;
  $38 = ($33|0)>($37|0);
  if ($38) {
   HEAP32[$0>>2] = 0;
   $72 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($72|0);
  }
 }
 $39 = HEAP32[$1>>2]|0;
 $40 = (($39) + 40|0);
 $41 = HEAP16[$40>>1]|0;
 $42 = $41&65535;
 $43 = (($42) - 1)|0;
 HEAP32[$i>>2] = $43;
 while(1) {
  $44 = HEAP32[$i>>2]|0;
  $45 = ($44|0)>=(0);
  if (!($45)) {
   label = 17;
   break;
  }
  $46 = HEAP32[$2>>2]|0;
  $47 = (($46) + 40|0);
  $48 = HEAP16[$47>>1]|0;
  $49 = $48&65535;
  $50 = (($49) - 1)|0;
  HEAP32[$j>>2] = $50;
  while(1) {
   $51 = HEAP32[$j>>2]|0;
   $52 = ($51|0)>=(0);
   if (!($52)) {
    break;
   }
   $53 = HEAP32[$j>>2]|0;
   $54 = HEAP32[$2>>2]|0;
   $55 = (($54) + 44|0);
   $56 = HEAP32[$55>>2]|0;
   $57 = (($56) + ($53<<2)|0);
   $58 = HEAP32[$57>>2]|0;
   $59 = HEAP32[$i>>2]|0;
   $60 = HEAP32[$1>>2]|0;
   $61 = (($60) + 44|0);
   $62 = HEAP32[$61>>2]|0;
   $63 = (($62) + ($59<<2)|0);
   $64 = HEAP32[$63>>2]|0;
   $65 = ($58|0)==($64|0);
   if ($65) {
    break;
   }
   $66 = HEAP32[$j>>2]|0;
   $67 = (($66) + -1)|0;
   HEAP32[$j>>2] = $67;
  }
  $68 = HEAP32[$j>>2]|0;
  $69 = ($68|0)<(0);
  if ($69) {
   label = 15;
   break;
  }
  $70 = HEAP32[$i>>2]|0;
  $71 = (($70) + -1)|0;
  HEAP32[$i>>2] = $71;
 }
 if ((label|0) == 15) {
  HEAP32[$0>>2] = 0;
  $72 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($72|0);
 }
 else if ((label|0) == 17) {
  HEAP32[$0>>2] = 1;
  $72 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($72|0);
 }
 return 0|0;
}
function _whereUsablePartialIndex($iTab,$pWC,$pWhere) {
 $iTab = $iTab|0;
 $pWC = $pWC|0;
 $pWhere = $pWhere|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0;
 var $7 = 0, $8 = 0, $9 = 0, $i = 0, $pTerm = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 20|0;
 $1 = sp + 16|0;
 $2 = sp + 12|0;
 $3 = sp + 8|0;
 $i = sp + 4|0;
 $pTerm = sp;
 HEAP32[$1>>2] = $iTab;
 HEAP32[$2>>2] = $pWC;
 HEAP32[$3>>2] = $pWhere;
 HEAP32[$i>>2] = 0;
 $4 = HEAP32[$2>>2]|0;
 $5 = (($4) + 20|0);
 $6 = HEAP32[$5>>2]|0;
 HEAP32[$pTerm>>2] = $6;
 while(1) {
  $7 = HEAP32[$i>>2]|0;
  $8 = HEAP32[$2>>2]|0;
  $9 = (($8) + 12|0);
  $10 = HEAP32[$9>>2]|0;
  $11 = ($7|0)<($10|0);
  if (!($11)) {
   label = 6;
   break;
  }
  $12 = HEAP32[$pTerm>>2]|0;
  $13 = HEAP32[$12>>2]|0;
  $14 = HEAP32[$3>>2]|0;
  $15 = HEAP32[$1>>2]|0;
  $16 = (_sqlite3ExprImpliesExpr($13,$14,$15)|0);
  $17 = ($16|0)!=(0);
  if ($17) {
   label = 4;
   break;
  }
  $18 = HEAP32[$i>>2]|0;
  $19 = (($18) + 1)|0;
  HEAP32[$i>>2] = $19;
  $20 = HEAP32[$pTerm>>2]|0;
  $21 = (($20) + 48|0);
  HEAP32[$pTerm>>2] = $21;
 }
 if ((label|0) == 4) {
  HEAP32[$0>>2] = 1;
  $22 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($22|0);
 }
 else if ((label|0) == 6) {
  HEAP32[$0>>2] = 0;
  $22 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($22|0);
 }
 return 0|0;
}
function _indexMightHelpWithOrderBy($pBuilder,$pIndex,$iCursor) {
 $pBuilder = $pBuilder|0;
 $pIndex = $pIndex|0;
 $iCursor = $iCursor|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $7 = 0, $8 = 0, $9 = 0, $ii = 0, $jj = 0, $pExpr = 0, $pOB = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 28|0;
 $1 = sp + 24|0;
 $2 = sp + 20|0;
 $3 = sp + 16|0;
 $pOB = sp + 12|0;
 $ii = sp + 8|0;
 $jj = sp + 4|0;
 $pExpr = sp;
 HEAP32[$1>>2] = $pBuilder;
 HEAP32[$2>>2] = $pIndex;
 HEAP32[$3>>2] = $iCursor;
 $4 = HEAP32[$2>>2]|0;
 $5 = (($4) + 55|0);
 $6 = HEAP8[$5>>0]|0;
 $7 = ($6&255) >>> 2;
 $8 = $7 & 1;
 $9 = $8&255;
 $10 = ($9|0)!=(0);
 if ($10) {
  HEAP32[$0>>2] = 0;
  $63 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($63|0);
 }
 $11 = HEAP32[$1>>2]|0;
 $12 = HEAP32[$11>>2]|0;
 $13 = (($12) + 8|0);
 $14 = HEAP32[$13>>2]|0;
 HEAP32[$pOB>>2] = $14;
 $15 = ($14|0)==(0|0);
 if ($15) {
  HEAP32[$0>>2] = 0;
  $63 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($63|0);
 }
 HEAP32[$ii>>2] = 0;
 L9: while(1) {
  $16 = HEAP32[$ii>>2]|0;
  $17 = HEAP32[$pOB>>2]|0;
  $18 = HEAP32[$17>>2]|0;
  $19 = ($16|0)<($18|0);
  if (!($19)) {
   label = 18;
   break;
  }
  $20 = HEAP32[$ii>>2]|0;
  $21 = HEAP32[$pOB>>2]|0;
  $22 = (($21) + 4|0);
  $23 = HEAP32[$22>>2]|0;
  $24 = (($23) + (($20*20)|0)|0);
  $25 = HEAP32[$24>>2]|0;
  $26 = (_sqlite3ExprSkipCollate($25)|0);
  HEAP32[$pExpr>>2] = $26;
  $27 = HEAP32[$pExpr>>2]|0;
  $28 = HEAP8[$27>>0]|0;
  $29 = $28&255;
  $30 = ($29|0)!=(154);
  if ($30) {
   label = 8;
   break;
  }
  $31 = HEAP32[$pExpr>>2]|0;
  $32 = (($31) + 28|0);
  $33 = HEAP32[$32>>2]|0;
  $34 = HEAP32[$3>>2]|0;
  $35 = ($33|0)==($34|0);
  L13: do {
   if ($35) {
    $36 = HEAP32[$pExpr>>2]|0;
    $37 = (($36) + 32|0);
    $38 = HEAP16[$37>>1]|0;
    $39 = $38 << 16 >> 16;
    $40 = ($39|0)<(0);
    if ($40) {
     label = 11;
     break L9;
    }
    HEAP32[$jj>>2] = 0;
    while(1) {
     $41 = HEAP32[$jj>>2]|0;
     $42 = HEAP32[$2>>2]|0;
     $43 = (($42) + 50|0);
     $44 = HEAP16[$43>>1]|0;
     $45 = $44&65535;
     $46 = ($41|0)<($45|0);
     if (!($46)) {
      break L13;
     }
     $47 = HEAP32[$pExpr>>2]|0;
     $48 = (($47) + 32|0);
     $49 = HEAP16[$48>>1]|0;
     $50 = $49 << 16 >> 16;
     $51 = HEAP32[$jj>>2]|0;
     $52 = HEAP32[$2>>2]|0;
     $53 = (($52) + 4|0);
     $54 = HEAP32[$53>>2]|0;
     $55 = (($54) + ($51<<1)|0);
     $56 = HEAP16[$55>>1]|0;
     $57 = $56 << 16 >> 16;
     $58 = ($50|0)==($57|0);
     if ($58) {
      label = 15;
      break L9;
     }
     $59 = HEAP32[$jj>>2]|0;
     $60 = (($59) + 1)|0;
     HEAP32[$jj>>2] = $60;
    }
   }
  } while(0);
  $61 = HEAP32[$ii>>2]|0;
  $62 = (($61) + 1)|0;
  HEAP32[$ii>>2] = $62;
 }
 if ((label|0) == 8) {
  HEAP32[$0>>2] = 0;
  $63 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($63|0);
 }
 else if ((label|0) == 11) {
  HEAP32[$0>>2] = 1;
  $63 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($63|0);
 }
 else if ((label|0) == 15) {
  HEAP32[$0>>2] = 1;
  $63 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($63|0);
 }
 else if ((label|0) == 18) {
  HEAP32[$0>>2] = 0;
  $63 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($63|0);
 }
 return 0|0;
}
function _whereLoopOutputAdjust($pWC,$pLoop,$nRow) {
 $pWC = $pWC|0;
 $pLoop = $pLoop|0;
 $nRow = $nRow|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0;
 var $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0;
 var $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0;
 var $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0;
 var $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0;
 var $98 = 0, $99 = 0, $i = 0, $j = 0, $nEq = 0, $notAllowed = 0, $pTerm = 0, $pX = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $0 = sp + 32|0;
 $1 = sp + 28|0;
 $2 = sp + 36|0;
 $pTerm = sp + 24|0;
 $pX = sp + 20|0;
 $notAllowed = sp;
 $i = sp + 16|0;
 $j = sp + 12|0;
 $nEq = sp + 8|0;
 HEAP32[$0>>2] = $pWC;
 HEAP32[$1>>2] = $pLoop;
 HEAP16[$2>>1] = $nRow;
 $3 = HEAP32[$1>>2]|0;
 $4 = $3;
 $5 = $4;
 $6 = HEAP32[$5>>2]|0;
 $7 = (($4) + 4)|0;
 $8 = $7;
 $9 = HEAP32[$8>>2]|0;
 $10 = HEAP32[$1>>2]|0;
 $11 = (($10) + 8|0);
 $12 = $11;
 $13 = $12;
 $14 = HEAP32[$13>>2]|0;
 $15 = (($12) + 4)|0;
 $16 = $15;
 $17 = HEAP32[$16>>2]|0;
 $18 = $6 | $14;
 $19 = $9 | $17;
 $20 = $18 ^ -1;
 $21 = $19 ^ -1;
 $22 = $notAllowed;
 $23 = $22;
 HEAP32[$23>>2] = $20;
 $24 = (($22) + 4)|0;
 $25 = $24;
 HEAP32[$25>>2] = $21;
 HEAP32[$nEq>>2] = 0;
 $26 = HEAP32[$0>>2]|0;
 $27 = (($26) + 12|0);
 $28 = HEAP32[$27>>2]|0;
 HEAP32[$i>>2] = $28;
 $29 = HEAP32[$0>>2]|0;
 $30 = (($29) + 20|0);
 $31 = HEAP32[$30>>2]|0;
 HEAP32[$pTerm>>2] = $31;
 while(1) {
  $32 = HEAP32[$i>>2]|0;
  $33 = ($32|0)>(0);
  if (!($33)) {
   break;
  }
  $34 = HEAP32[$pTerm>>2]|0;
  $35 = (($34) + 20|0);
  $36 = HEAP8[$35>>0]|0;
  $37 = $36&255;
  $38 = $37 & 2;
  $39 = ($38|0)!=(0);
  if ($39) {
   break;
  }
  $40 = HEAP32[$pTerm>>2]|0;
  $41 = (($40) + 40|0);
  $42 = $41;
  $43 = $42;
  $44 = HEAP32[$43>>2]|0;
  $45 = (($42) + 4)|0;
  $46 = $45;
  $47 = HEAP32[$46>>2]|0;
  $48 = HEAP32[$1>>2]|0;
  $49 = (($48) + 8|0);
  $50 = $49;
  $51 = $50;
  $52 = HEAP32[$51>>2]|0;
  $53 = (($50) + 4)|0;
  $54 = $53;
  $55 = HEAP32[$54>>2]|0;
  $56 = $44 & $52;
  $57 = $47 & $55;
  $58 = ($56|0)==(0);
  $59 = ($57|0)==(0);
  $60 = $58 & $59;
  do {
   if (!($60)) {
    $61 = HEAP32[$pTerm>>2]|0;
    $62 = (($61) + 40|0);
    $63 = $62;
    $64 = $63;
    $65 = HEAP32[$64>>2]|0;
    $66 = (($63) + 4)|0;
    $67 = $66;
    $68 = HEAP32[$67>>2]|0;
    $69 = $notAllowed;
    $70 = $69;
    $71 = HEAP32[$70>>2]|0;
    $72 = (($69) + 4)|0;
    $73 = $72;
    $74 = HEAP32[$73>>2]|0;
    $75 = $65 & $71;
    $76 = $68 & $74;
    $77 = ($75|0)!=(0);
    $78 = ($76|0)!=(0);
    $79 = $77 | $78;
    if (!($79)) {
     $80 = HEAP32[$1>>2]|0;
     $81 = (($80) + 40|0);
     $82 = HEAP16[$81>>1]|0;
     $83 = $82&65535;
     $84 = (($83) - 1)|0;
     HEAP32[$j>>2] = $84;
     while(1) {
      $85 = HEAP32[$j>>2]|0;
      $86 = ($85|0)>=(0);
      if (!($86)) {
       break;
      }
      $87 = HEAP32[$j>>2]|0;
      $88 = HEAP32[$1>>2]|0;
      $89 = (($88) + 44|0);
      $90 = HEAP32[$89>>2]|0;
      $91 = (($90) + ($87<<2)|0);
      $92 = HEAP32[$91>>2]|0;
      HEAP32[$pX>>2] = $92;
      $93 = HEAP32[$pX>>2]|0;
      $94 = ($93|0)==(0|0);
      if (!($94)) {
       $95 = HEAP32[$pX>>2]|0;
       $96 = HEAP32[$pTerm>>2]|0;
       $97 = ($95|0)==($96|0);
       if ($97) {
        break;
       }
       $98 = HEAP32[$pX>>2]|0;
       $99 = (($98) + 4|0);
       $100 = HEAP32[$99>>2]|0;
       $101 = ($100|0)>=(0);
       if ($101) {
        $102 = HEAP32[$pX>>2]|0;
        $103 = (($102) + 4|0);
        $104 = HEAP32[$103>>2]|0;
        $105 = HEAP32[$0>>2]|0;
        $106 = (($105) + 20|0);
        $107 = HEAP32[$106>>2]|0;
        $108 = (($107) + (($104*48)|0)|0);
        $109 = HEAP32[$pTerm>>2]|0;
        $110 = ($108|0)==($109|0);
        if ($110) {
         break;
        }
       }
      }
      $111 = HEAP32[$j>>2]|0;
      $112 = (($111) + -1)|0;
      HEAP32[$j>>2] = $112;
     }
     $113 = HEAP32[$j>>2]|0;
     $114 = ($113|0)<(0);
     if ($114) {
      $115 = HEAP32[$pTerm>>2]|0;
      $116 = (($115) + 16|0);
      $117 = HEAP16[$116>>1]|0;
      $118 = $117 << 16 >> 16;
      $119 = ($118|0)<=(0);
      if ($119) {
       $120 = HEAP32[$pTerm>>2]|0;
       $121 = (($120) + 16|0);
       $122 = HEAP16[$121>>1]|0;
       $123 = $122 << 16 >> 16;
       $124 = HEAP32[$1>>2]|0;
       $125 = (($124) + 22|0);
       $126 = HEAP16[$125>>1]|0;
       $127 = $126 << 16 >> 16;
       $128 = (($127) + ($123))|0;
       $129 = $128&65535;
       HEAP16[$125>>1] = $129;
       break;
      }
      $130 = HEAP32[$1>>2]|0;
      $131 = (($130) + 22|0);
      $132 = HEAP16[$131>>1]|0;
      $133 = (($132) + -1)<<16>>16;
      HEAP16[$131>>1] = $133;
      $134 = HEAP32[$pTerm>>2]|0;
      $135 = (($134) + 18|0);
      $136 = HEAP16[$135>>1]|0;
      $137 = $136&65535;
      $138 = $137 & 2;
      $139 = ($138|0)!=(0);
      if ($139) {
       $140 = HEAP32[$nEq>>2]|0;
       $141 = (($140) + 1)|0;
       HEAP32[$nEq>>2] = $141;
      }
     }
    }
   }
  } while(0);
  $142 = HEAP32[$i>>2]|0;
  $143 = (($142) + -1)|0;
  HEAP32[$i>>2] = $143;
  $144 = HEAP32[$pTerm>>2]|0;
  $145 = (($144) + 48|0);
  HEAP32[$pTerm>>2] = $145;
 }
 $146 = HEAP32[$nEq>>2]|0;
 $147 = ($146|0)!=(0);
 if (!($147)) {
  STACKTOP = sp;return;
 }
 $148 = HEAP32[$1>>2]|0;
 $149 = (($148) + 22|0);
 $150 = HEAP16[$149>>1]|0;
 $151 = $150 << 16 >> 16;
 $152 = HEAP16[$2>>1]|0;
 $153 = $152 << 16 >> 16;
 $154 = (($153) - 10)|0;
 $155 = ($151|0)>($154|0);
 if (!($155)) {
  STACKTOP = sp;return;
 }
 $156 = HEAP16[$2>>1]|0;
 $157 = $156 << 16 >> 16;
 $158 = (($157) - 10)|0;
 $159 = $158&65535;
 $160 = HEAP32[$1>>2]|0;
 $161 = (($160) + 22|0);
 HEAP16[$161>>1] = $159;
 STACKTOP = sp;return;
}
function _columnsInIndex($pIdx) {
 $pIdx = $pIdx|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $j = 0, $m = 0, $or$cond = 0, $x = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 16|0;
 $m = sp;
 $j = sp + 12|0;
 $x = sp + 8|0;
 HEAP32[$0>>2] = $pIdx;
 $1 = $m;
 $2 = $1;
 HEAP32[$2>>2] = 0;
 $3 = (($1) + 4)|0;
 $4 = $3;
 HEAP32[$4>>2] = 0;
 $5 = HEAP32[$0>>2]|0;
 $6 = (($5) + 52|0);
 $7 = HEAP16[$6>>1]|0;
 $8 = $7&65535;
 $9 = (($8) - 1)|0;
 HEAP32[$j>>2] = $9;
 while(1) {
  $10 = HEAP32[$j>>2]|0;
  $11 = ($10|0)>=(0);
  if (!($11)) {
   break;
  }
  $12 = HEAP32[$j>>2]|0;
  $13 = HEAP32[$0>>2]|0;
  $14 = (($13) + 4|0);
  $15 = HEAP32[$14>>2]|0;
  $16 = (($15) + ($12<<1)|0);
  $17 = HEAP16[$16>>1]|0;
  $18 = $17 << 16 >> 16;
  HEAP32[$x>>2] = $18;
  $19 = HEAP32[$x>>2]|0;
  $20 = ($19|0)>=(0);
  $21 = HEAP32[$x>>2]|0;
  $22 = ($21|0)<(63);
  $or$cond = $20 & $22;
  if ($or$cond) {
   $23 = HEAP32[$x>>2]|0;
   $24 = (_bitshift64Shl(1,0,($23|0))|0);
   $25 = tempRet0;
   $26 = $m;
   $27 = $26;
   $28 = HEAP32[$27>>2]|0;
   $29 = (($26) + 4)|0;
   $30 = $29;
   $31 = HEAP32[$30>>2]|0;
   $32 = $28 | $24;
   $33 = $31 | $25;
   $34 = $m;
   $35 = $34;
   HEAP32[$35>>2] = $32;
   $36 = (($34) + 4)|0;
   $37 = $36;
   HEAP32[$37>>2] = $33;
  }
  $38 = HEAP32[$j>>2]|0;
  $39 = (($38) + -1)|0;
  HEAP32[$j>>2] = $39;
 }
 $40 = $m;
 $41 = $40;
 $42 = HEAP32[$41>>2]|0;
 $43 = (($40) + 4)|0;
 $44 = $43;
 $45 = HEAP32[$44>>2]|0;
 tempRet0 = $45;
 STACKTOP = sp;return ($42|0);
}
function _whereLoopAddBtreeIndex($pBuilder,$pSrc,$pProbe,$nInMul) {
 $pBuilder = $pBuilder|0;
 $pSrc = $pSrc|0;
 $pProbe = $pProbe|0;
 $nInMul = $nInMul|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0;
 var $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0, $188 = 0;
 var $189 = 0, $19 = 0, $190 = 0, $191 = 0, $192 = 0, $193 = 0, $194 = 0, $195 = 0, $196 = 0, $197 = 0, $198 = 0, $199 = 0, $2 = 0, $20 = 0, $200 = 0, $201 = 0, $202 = 0, $203 = 0, $204 = 0, $205 = 0;
 var $206 = 0, $207 = 0, $208 = 0, $209 = 0, $21 = 0, $210 = 0, $211 = 0, $212 = 0, $213 = 0, $214 = 0, $215 = 0, $216 = 0, $217 = 0, $218 = 0, $219 = 0, $22 = 0, $220 = 0, $221 = 0, $222 = 0, $223 = 0;
 var $224 = 0, $225 = 0, $226 = 0, $227 = 0, $228 = 0, $229 = 0, $23 = 0, $230 = 0, $231 = 0, $232 = 0, $233 = 0, $234 = 0, $235 = 0, $236 = 0, $237 = 0, $238 = 0, $239 = 0, $24 = 0, $240 = 0, $241 = 0;
 var $242 = 0, $243 = 0, $244 = 0, $245 = 0, $246 = 0, $247 = 0, $248 = 0, $249 = 0, $25 = 0, $250 = 0, $251 = 0, $252 = 0, $253 = 0, $254 = 0, $255 = 0, $256 = 0, $257 = 0, $258 = 0, $259 = 0, $26 = 0;
 var $260 = 0, $261 = 0, $262 = 0, $263 = 0, $264 = 0, $265 = 0, $266 = 0, $267 = 0, $268 = 0, $269 = 0, $27 = 0, $270 = 0, $271 = 0, $272 = 0, $273 = 0, $274 = 0, $275 = 0, $276 = 0, $277 = 0, $278 = 0;
 var $279 = 0, $28 = 0, $280 = 0, $281 = 0, $282 = 0, $283 = 0, $284 = 0, $285 = 0, $286 = 0, $287 = 0, $288 = 0, $289 = 0, $29 = 0, $290 = 0, $291 = 0, $292 = 0, $293 = 0, $294 = 0, $295 = 0, $296 = 0;
 var $297 = 0, $298 = 0, $299 = 0, $3 = 0, $30 = 0, $300 = 0, $301 = 0, $302 = 0, $303 = 0, $304 = 0, $305 = 0, $306 = 0, $307 = 0, $308 = 0, $309 = 0, $31 = 0, $310 = 0, $311 = 0, $312 = 0, $313 = 0;
 var $314 = 0, $315 = 0, $316 = 0, $317 = 0, $318 = 0, $319 = 0, $32 = 0, $320 = 0, $321 = 0, $322 = 0, $323 = 0, $324 = 0, $325 = 0, $326 = 0, $327 = 0, $328 = 0, $329 = 0, $33 = 0, $330 = 0, $331 = 0;
 var $332 = 0, $333 = 0, $334 = 0, $335 = 0, $336 = 0, $337 = 0, $338 = 0, $339 = 0, $34 = 0, $340 = 0, $341 = 0, $342 = 0, $343 = 0, $344 = 0, $345 = 0, $346 = 0, $347 = 0, $348 = 0, $349 = 0, $35 = 0;
 var $350 = 0, $351 = 0, $352 = 0, $353 = 0, $354 = 0, $355 = 0, $356 = 0, $357 = 0, $358 = 0, $359 = 0, $36 = 0, $360 = 0, $361 = 0, $362 = 0, $363 = 0, $364 = 0, $365 = 0, $366 = 0, $367 = 0, $368 = 0;
 var $369 = 0, $37 = 0, $370 = 0, $371 = 0, $372 = 0, $373 = 0, $374 = 0, $375 = 0, $376 = 0, $377 = 0, $378 = 0, $379 = 0, $38 = 0, $380 = 0, $381 = 0, $382 = 0, $383 = 0, $384 = 0, $385 = 0, $386 = 0;
 var $387 = 0, $388 = 0, $389 = 0, $39 = 0, $390 = 0, $391 = 0, $392 = 0, $393 = 0, $394 = 0, $395 = 0, $396 = 0, $397 = 0, $398 = 0, $399 = 0, $4 = 0, $40 = 0, $400 = 0, $401 = 0, $402 = 0, $403 = 0;
 var $404 = 0, $405 = 0, $406 = 0, $407 = 0, $408 = 0, $409 = 0, $41 = 0, $410 = 0, $411 = 0, $412 = 0, $413 = 0, $414 = 0, $415 = 0, $416 = 0, $417 = 0, $418 = 0, $419 = 0, $42 = 0, $420 = 0, $421 = 0;
 var $422 = 0, $423 = 0, $424 = 0, $425 = 0, $426 = 0, $427 = 0, $428 = 0, $429 = 0, $43 = 0, $430 = 0, $431 = 0, $432 = 0, $433 = 0, $434 = 0, $435 = 0, $436 = 0, $437 = 0, $438 = 0, $439 = 0, $44 = 0;
 var $440 = 0, $441 = 0, $442 = 0, $443 = 0, $444 = 0, $445 = 0, $446 = 0, $447 = 0, $448 = 0, $449 = 0, $45 = 0, $450 = 0, $451 = 0, $452 = 0, $453 = 0, $454 = 0, $455 = 0, $456 = 0, $457 = 0, $458 = 0;
 var $459 = 0, $46 = 0, $460 = 0, $461 = 0, $462 = 0, $463 = 0, $464 = 0, $465 = 0, $466 = 0, $467 = 0, $468 = 0, $469 = 0, $47 = 0, $470 = 0, $471 = 0, $472 = 0, $473 = 0, $474 = 0, $475 = 0, $476 = 0;
 var $477 = 0, $478 = 0, $479 = 0, $48 = 0, $480 = 0, $481 = 0, $482 = 0, $483 = 0, $484 = 0, $485 = 0, $486 = 0, $487 = 0, $488 = 0, $489 = 0, $49 = 0, $490 = 0, $491 = 0, $492 = 0, $493 = 0, $494 = 0;
 var $495 = 0, $496 = 0, $497 = 0, $498 = 0, $499 = 0, $5 = 0, $50 = 0, $500 = 0, $501 = 0, $502 = 0, $503 = 0, $504 = 0, $505 = 0, $506 = 0, $507 = 0, $508 = 0, $509 = 0, $51 = 0, $510 = 0, $511 = 0;
 var $512 = 0, $513 = 0, $514 = 0, $515 = 0, $516 = 0, $517 = 0, $518 = 0, $519 = 0, $52 = 0, $520 = 0, $521 = 0, $522 = 0, $523 = 0, $524 = 0, $525 = 0, $526 = 0, $527 = 0, $528 = 0, $529 = 0, $53 = 0;
 var $530 = 0, $531 = 0, $532 = 0, $533 = 0, $534 = 0, $535 = 0, $536 = 0, $537 = 0, $538 = 0, $539 = 0, $54 = 0, $540 = 0, $541 = 0, $542 = 0, $543 = 0, $544 = 0, $545 = 0, $546 = 0, $547 = 0, $548 = 0;
 var $549 = 0, $55 = 0, $550 = 0, $551 = 0, $552 = 0, $553 = 0, $554 = 0, $555 = 0, $556 = 0, $557 = 0, $558 = 0, $559 = 0, $56 = 0, $560 = 0, $561 = 0, $562 = 0, $563 = 0, $564 = 0, $565 = 0, $566 = 0;
 var $567 = 0, $568 = 0, $569 = 0, $57 = 0, $570 = 0, $571 = 0, $572 = 0, $573 = 0, $574 = 0, $575 = 0, $576 = 0, $577 = 0, $578 = 0, $579 = 0, $58 = 0, $580 = 0, $581 = 0, $582 = 0, $583 = 0, $584 = 0;
 var $585 = 0, $586 = 0, $587 = 0, $588 = 0, $589 = 0, $59 = 0, $590 = 0, $591 = 0, $592 = 0, $593 = 0, $594 = 0, $595 = 0, $596 = 0, $597 = 0, $598 = 0, $599 = 0, $6 = 0, $60 = 0, $600 = 0, $601 = 0;
 var $602 = 0, $603 = 0, $604 = 0, $605 = 0, $606 = 0, $607 = 0, $608 = 0, $609 = 0, $61 = 0, $610 = 0, $611 = 0, $612 = 0, $613 = 0, $614 = 0, $615 = 0, $616 = 0, $617 = 0, $618 = 0, $619 = 0, $62 = 0;
 var $620 = 0, $621 = 0, $622 = 0, $623 = 0, $624 = 0, $625 = 0, $626 = 0, $627 = 0, $628 = 0, $629 = 0, $63 = 0, $630 = 0, $631 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0;
 var $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0;
 var $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $db = 0, $eOp = 0, $iCol = 0, $nEq = 0, $nIn = 0, $nIter = 0, $nOutUnadjusted = 0, $opMask = 0;
 var $or$cond = 0, $or$cond3 = 0, $pBtm = 0, $pExpr = 0, $pNew = 0, $pParse = 0, $pTerm = 0, $pTop = 0, $pWInfo = 0, $rCostIdx = 0, $rLogSize = 0, $rSize = 0, $rc = 0, $saved_nEq = 0, $saved_nLTerm = 0, $saved_nOut = 0, $saved_nSkip = 0, $saved_prereq = 0, $saved_wsFlags = 0, $scan = 0;
 var label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 224|0;
 $0 = sp + 188|0;
 $1 = sp + 184|0;
 $2 = sp + 180|0;
 $3 = sp + 176|0;
 $4 = sp + 212|0;
 $pWInfo = sp + 172|0;
 $pParse = sp + 168|0;
 $db = sp + 164|0;
 $pNew = sp + 160|0;
 $pTerm = sp + 156|0;
 $opMask = sp + 152|0;
 $scan = sp + 40|0;
 $saved_prereq = sp;
 $saved_nLTerm = sp + 210|0;
 $saved_nEq = sp + 208|0;
 $saved_nSkip = sp + 206|0;
 $saved_wsFlags = sp + 36|0;
 $saved_nOut = sp + 204|0;
 $iCol = sp + 32|0;
 $rc = sp + 28|0;
 $rSize = sp + 202|0;
 $rLogSize = sp + 200|0;
 $pTop = sp + 24|0;
 $pBtm = sp + 20|0;
 $nIter = sp + 198|0;
 $eOp = sp + 196|0;
 $rCostIdx = sp + 194|0;
 $nOutUnadjusted = sp + 192|0;
 $nIn = sp + 16|0;
 $pExpr = sp + 12|0;
 $nEq = sp + 8|0;
 HEAP32[$1>>2] = $pBuilder;
 HEAP32[$2>>2] = $pSrc;
 HEAP32[$3>>2] = $pProbe;
 HEAP16[$4>>1] = $nInMul;
 $5 = HEAP32[$1>>2]|0;
 $6 = HEAP32[$5>>2]|0;
 HEAP32[$pWInfo>>2] = $6;
 $7 = HEAP32[$pWInfo>>2]|0;
 $8 = HEAP32[$7>>2]|0;
 HEAP32[$pParse>>2] = $8;
 $9 = HEAP32[$pParse>>2]|0;
 $10 = HEAP32[$9>>2]|0;
 HEAP32[$db>>2] = $10;
 HEAP32[$rc>>2] = 0;
 HEAP32[$pTop>>2] = 0;
 HEAP32[$pBtm>>2] = 0;
 $11 = HEAP32[$1>>2]|0;
 $12 = (($11) + 12|0);
 $13 = HEAP32[$12>>2]|0;
 HEAP32[$pNew>>2] = $13;
 $14 = HEAP32[$db>>2]|0;
 $15 = (($14) + 64|0);
 $16 = HEAP8[$15>>0]|0;
 $17 = ($16<<24>>24)!=(0);
 if ($17) {
  HEAP32[$0>>2] = 7;
  $631 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($631|0);
 }
 $18 = HEAP32[$pNew>>2]|0;
 $19 = (($18) + 36|0);
 $20 = HEAP32[$19>>2]|0;
 $21 = $20 & 32;
 $22 = ($21|0)!=(0);
 do {
  if ($22) {
   HEAP32[$opMask>>2] = 24;
  } else {
   $23 = HEAP32[$3>>2]|0;
   $24 = (($23) + 44|0);
   $25 = HEAP32[$24>>2]|0;
   $26 = ($25|0)<=(0);
   if (!($26)) {
    $27 = HEAP32[$2>>2]|0;
    $28 = (($27) + 36|0);
    $29 = HEAP8[$28>>0]|0;
    $30 = $29&255;
    $31 = $30 & 8;
    $32 = ($31|0)!=(0);
    if (!($32)) {
     HEAP32[$opMask>>2] = 191;
     break;
    }
   }
   HEAP32[$opMask>>2] = 63;
  }
 } while(0);
 $33 = HEAP32[$3>>2]|0;
 $34 = (($33) + 55|0);
 $35 = HEAP8[$34>>0]|0;
 $36 = ($35&255) >>> 2;
 $37 = $36 & 1;
 $38 = $37&255;
 $39 = ($38|0)!=(0);
 if ($39) {
  $40 = HEAP32[$opMask>>2]|0;
  $41 = $40 & -61;
  HEAP32[$opMask>>2] = $41;
 }
 $42 = HEAP32[$pNew>>2]|0;
 $43 = (($42) + 24|0);
 $44 = HEAP16[$43>>1]|0;
 $45 = $44&65535;
 $46 = HEAP32[$3>>2]|0;
 $47 = (($46) + 4|0);
 $48 = HEAP32[$47>>2]|0;
 $49 = (($48) + ($45<<1)|0);
 $50 = HEAP16[$49>>1]|0;
 $51 = $50 << 16 >> 16;
 HEAP32[$iCol>>2] = $51;
 $52 = HEAP32[$1>>2]|0;
 $53 = (($52) + 4|0);
 $54 = HEAP32[$53>>2]|0;
 $55 = HEAP32[$2>>2]|0;
 $56 = (($55) + 40|0);
 $57 = HEAP32[$56>>2]|0;
 $58 = HEAP32[$iCol>>2]|0;
 $59 = HEAP32[$opMask>>2]|0;
 $60 = HEAP32[$3>>2]|0;
 $61 = (_whereScanInit($scan,$54,$57,$58,$59,$60)|0);
 HEAP32[$pTerm>>2] = $61;
 $62 = HEAP32[$pNew>>2]|0;
 $63 = (($62) + 24|0);
 $64 = HEAP16[$63>>1]|0;
 HEAP16[$saved_nEq>>1] = $64;
 $65 = HEAP32[$pNew>>2]|0;
 $66 = (($65) + 24|0);
 $67 = (($66) + 2|0);
 $68 = HEAP16[$67>>1]|0;
 HEAP16[$saved_nSkip>>1] = $68;
 $69 = HEAP32[$pNew>>2]|0;
 $70 = (($69) + 40|0);
 $71 = HEAP16[$70>>1]|0;
 HEAP16[$saved_nLTerm>>1] = $71;
 $72 = HEAP32[$pNew>>2]|0;
 $73 = (($72) + 36|0);
 $74 = HEAP32[$73>>2]|0;
 HEAP32[$saved_wsFlags>>2] = $74;
 $75 = HEAP32[$pNew>>2]|0;
 $76 = $75;
 $77 = $76;
 $78 = HEAP32[$77>>2]|0;
 $79 = (($76) + 4)|0;
 $80 = $79;
 $81 = HEAP32[$80>>2]|0;
 $82 = $saved_prereq;
 $83 = $82;
 HEAP32[$83>>2] = $78;
 $84 = (($82) + 4)|0;
 $85 = $84;
 HEAP32[$85>>2] = $81;
 $86 = HEAP32[$pNew>>2]|0;
 $87 = (($86) + 22|0);
 $88 = HEAP16[$87>>1]|0;
 HEAP16[$saved_nOut>>1] = $88;
 $89 = HEAP32[$pNew>>2]|0;
 $90 = (($89) + 18|0);
 HEAP16[$90>>1] = 0;
 $91 = HEAP32[$3>>2]|0;
 $92 = (($91) + 8|0);
 $93 = HEAP32[$92>>2]|0;
 $94 = HEAP16[$93>>1]|0;
 HEAP16[$rSize>>1] = $94;
 $95 = HEAP16[$rSize>>1]|0;
 $96 = (_estLog($95)|0);
 HEAP16[$rLogSize>>1] = $96;
 $97 = HEAP16[$saved_nEq>>1]|0;
 $98 = $97&65535;
 $99 = HEAP16[$saved_nSkip>>1]|0;
 $100 = $99&65535;
 $101 = ($98|0)==($100|0);
 if ($101) {
  $102 = HEAP16[$saved_nEq>>1]|0;
  $103 = $102&65535;
  $104 = (($103) + 1)|0;
  $105 = HEAP32[$3>>2]|0;
  $106 = (($105) + 50|0);
  $107 = HEAP16[$106>>1]|0;
  $108 = $107&65535;
  $109 = ($104|0)<($108|0);
  if ($109) {
   $110 = HEAP16[$saved_nEq>>1]|0;
   $111 = $110&65535;
   $112 = (($111) + 1)|0;
   $113 = HEAP32[$3>>2]|0;
   $114 = (($113) + 8|0);
   $115 = HEAP32[$114>>2]|0;
   $116 = (($115) + ($112<<1)|0);
   $117 = HEAP16[$116>>1]|0;
   $118 = $117 << 16 >> 16;
   $119 = ($118|0)>=(42);
   if ($119) {
    $120 = HEAP32[$db>>2]|0;
    $121 = HEAP32[$pNew>>2]|0;
    $122 = HEAP32[$pNew>>2]|0;
    $123 = (($122) + 40|0);
    $124 = HEAP16[$123>>1]|0;
    $125 = $124&65535;
    $126 = (($125) + 1)|0;
    $127 = (_whereLoopResize($120,$121,$126)|0);
    HEAP32[$rc>>2] = $127;
    $128 = ($127|0)==(0);
    if ($128) {
     $129 = HEAP32[$pNew>>2]|0;
     $130 = (($129) + 24|0);
     $131 = HEAP16[$130>>1]|0;
     $132 = (($131) + 1)<<16>>16;
     HEAP16[$130>>1] = $132;
     $133 = HEAP32[$pNew>>2]|0;
     $134 = (($133) + 24|0);
     $135 = (($134) + 2|0);
     $136 = HEAP16[$135>>1]|0;
     $137 = (($136) + 1)<<16>>16;
     HEAP16[$135>>1] = $137;
     $138 = HEAP32[$pNew>>2]|0;
     $139 = (($138) + 40|0);
     $140 = HEAP16[$139>>1]|0;
     $141 = (($140) + 1)<<16>>16;
     HEAP16[$139>>1] = $141;
     $142 = $140&65535;
     $143 = HEAP32[$pNew>>2]|0;
     $144 = (($143) + 44|0);
     $145 = HEAP32[$144>>2]|0;
     $146 = (($145) + ($142<<2)|0);
     HEAP32[$146>>2] = 0;
     $147 = HEAP32[$pNew>>2]|0;
     $148 = (($147) + 36|0);
     $149 = HEAP32[$148>>2]|0;
     $150 = $149 | 32768;
     HEAP32[$148>>2] = $150;
     $151 = HEAP16[$saved_nEq>>1]|0;
     $152 = $151&65535;
     $153 = HEAP32[$3>>2]|0;
     $154 = (($153) + 8|0);
     $155 = HEAP32[$154>>2]|0;
     $156 = (($155) + ($152<<1)|0);
     $157 = HEAP16[$156>>1]|0;
     $158 = $157 << 16 >> 16;
     $159 = HEAP16[$saved_nEq>>1]|0;
     $160 = $159&65535;
     $161 = (($160) + 1)|0;
     $162 = HEAP32[$3>>2]|0;
     $163 = (($162) + 8|0);
     $164 = HEAP32[$163>>2]|0;
     $165 = (($164) + ($161<<1)|0);
     $166 = HEAP16[$165>>1]|0;
     $167 = $166 << 16 >> 16;
     $168 = (($158) - ($167))|0;
     $169 = $168&65535;
     HEAP16[$nIter>>1] = $169;
     $170 = HEAP32[$pTerm>>2]|0;
     $171 = ($170|0)!=(0|0);
     if ($171) {
      $172 = HEAP16[$nIter>>1]|0;
      $173 = $172 << 16 >> 16;
      $174 = (($173) + 10)|0;
      $175 = $174&65535;
      HEAP16[$nIter>>1] = $175;
     }
     $176 = HEAP16[$nIter>>1]|0;
     $177 = $176 << 16 >> 16;
     $178 = HEAP32[$pNew>>2]|0;
     $179 = (($178) + 22|0);
     $180 = HEAP16[$179>>1]|0;
     $181 = $180 << 16 >> 16;
     $182 = (($181) - ($177))|0;
     $183 = $182&65535;
     HEAP16[$179>>1] = $183;
     $184 = HEAP16[$nIter>>1]|0;
     $185 = $184 << 16 >> 16;
     $186 = (($185) + 5)|0;
     $187 = $186&65535;
     HEAP16[$nIter>>1] = $187;
     $188 = HEAP32[$1>>2]|0;
     $189 = HEAP32[$2>>2]|0;
     $190 = HEAP32[$3>>2]|0;
     $191 = HEAP16[$nIter>>1]|0;
     $192 = $191 << 16 >> 16;
     $193 = HEAP16[$4>>1]|0;
     $194 = $193 << 16 >> 16;
     $195 = (($192) + ($194))|0;
     $196 = $195&65535;
     (_whereLoopAddBtreeIndex($188,$189,$190,$196)|0);
     $197 = HEAP16[$saved_nOut>>1]|0;
     $198 = HEAP32[$pNew>>2]|0;
     $199 = (($198) + 22|0);
     HEAP16[$199>>1] = $197;
     $200 = HEAP16[$saved_nEq>>1]|0;
     $201 = HEAP32[$pNew>>2]|0;
     $202 = (($201) + 24|0);
     HEAP16[$202>>1] = $200;
     $203 = HEAP16[$saved_nSkip>>1]|0;
     $204 = HEAP32[$pNew>>2]|0;
     $205 = (($204) + 24|0);
     $206 = (($205) + 2|0);
     HEAP16[$206>>1] = $203;
    }
   }
  }
 }
 while(1) {
  $207 = HEAP32[$rc>>2]|0;
  $208 = ($207|0)==(0);
  $209 = HEAP32[$pTerm>>2]|0;
  $210 = ($209|0)!=(0|0);
  $or$cond = $208 & $210;
  if (!($or$cond)) {
   break;
  }
  $211 = HEAP32[$pTerm>>2]|0;
  $212 = (($211) + 18|0);
  $213 = HEAP16[$212>>1]|0;
  HEAP16[$eOp>>1] = $213;
  HEAP32[$nIn>>2] = 0;
  $214 = HEAP16[$eOp>>1]|0;
  $215 = $214&65535;
  $216 = ($215|0)==(128);
  if ($216) {
   label = 21;
  } else {
   $217 = HEAP32[$pTerm>>2]|0;
   $218 = (($217) + 20|0);
   $219 = HEAP8[$218>>0]|0;
   $220 = $219&255;
   $221 = $220 & 0;
   $222 = ($221|0)!=(0);
   if ($222) {
    label = 21;
   } else {
    label = 23;
   }
  }
  if ((label|0) == 21) {
   label = 0;
   $223 = HEAP32[$iCol>>2]|0;
   $224 = ($223|0)<(0);
   if (!($224)) {
    $225 = HEAP32[$iCol>>2]|0;
    $226 = HEAP32[$2>>2]|0;
    $227 = (($226) + 16|0);
    $228 = HEAP32[$227>>2]|0;
    $229 = (($228) + 4|0);
    $230 = HEAP32[$229>>2]|0;
    $231 = (($230) + (($225*24)|0)|0);
    $232 = (($231) + 20|0);
    $233 = HEAP8[$232>>0]|0;
    $234 = $233&255;
    $235 = ($234|0)!=(0);
    if (!($235)) {
     label = 23;
    }
   }
  }
  if ((label|0) == 23) {
   label = 0;
   $236 = HEAP32[$pTerm>>2]|0;
   $237 = (($236) + 32|0);
   $238 = $237;
   $239 = $238;
   $240 = HEAP32[$239>>2]|0;
   $241 = (($238) + 4)|0;
   $242 = $241;
   $243 = HEAP32[$242>>2]|0;
   $244 = HEAP32[$pNew>>2]|0;
   $245 = (($244) + 8|0);
   $246 = $245;
   $247 = $246;
   $248 = HEAP32[$247>>2]|0;
   $249 = (($246) + 4)|0;
   $250 = $249;
   $251 = HEAP32[$250>>2]|0;
   $252 = $240 & $248;
   $253 = $243 & $251;
   $254 = ($252|0)!=(0);
   $255 = ($253|0)!=(0);
   $256 = $254 | $255;
   if (!($256)) {
    $257 = HEAP32[$saved_wsFlags>>2]|0;
    $258 = HEAP32[$pNew>>2]|0;
    $259 = (($258) + 36|0);
    HEAP32[$259>>2] = $257;
    $260 = HEAP16[$saved_nEq>>1]|0;
    $261 = HEAP32[$pNew>>2]|0;
    $262 = (($261) + 24|0);
    HEAP16[$262>>1] = $260;
    $263 = HEAP16[$saved_nLTerm>>1]|0;
    $264 = HEAP32[$pNew>>2]|0;
    $265 = (($264) + 40|0);
    HEAP16[$265>>1] = $263;
    $266 = HEAP32[$db>>2]|0;
    $267 = HEAP32[$pNew>>2]|0;
    $268 = HEAP32[$pNew>>2]|0;
    $269 = (($268) + 40|0);
    $270 = HEAP16[$269>>1]|0;
    $271 = $270&65535;
    $272 = (($271) + 1)|0;
    $273 = (_whereLoopResize($266,$267,$272)|0);
    $274 = ($273|0)!=(0);
    if ($274) {
     break;
    }
    $275 = HEAP32[$pTerm>>2]|0;
    $276 = HEAP32[$pNew>>2]|0;
    $277 = (($276) + 40|0);
    $278 = HEAP16[$277>>1]|0;
    $279 = (($278) + 1)<<16>>16;
    HEAP16[$277>>1] = $279;
    $280 = $278&65535;
    $281 = HEAP32[$pNew>>2]|0;
    $282 = (($281) + 44|0);
    $283 = HEAP32[$282>>2]|0;
    $284 = (($283) + ($280<<2)|0);
    HEAP32[$284>>2] = $275;
    $285 = $saved_prereq;
    $286 = $285;
    $287 = HEAP32[$286>>2]|0;
    $288 = (($285) + 4)|0;
    $289 = $288;
    $290 = HEAP32[$289>>2]|0;
    $291 = HEAP32[$pTerm>>2]|0;
    $292 = (($291) + 32|0);
    $293 = $292;
    $294 = $293;
    $295 = HEAP32[$294>>2]|0;
    $296 = (($293) + 4)|0;
    $297 = $296;
    $298 = HEAP32[$297>>2]|0;
    $299 = $287 | $295;
    $300 = $290 | $298;
    $301 = HEAP32[$pNew>>2]|0;
    $302 = (($301) + 8|0);
    $303 = $302;
    $304 = $303;
    $305 = HEAP32[$304>>2]|0;
    $306 = (($303) + 4)|0;
    $307 = $306;
    $308 = HEAP32[$307>>2]|0;
    $309 = $305 ^ -1;
    $310 = $308 ^ -1;
    $311 = $299 & $309;
    $312 = $300 & $310;
    $313 = HEAP32[$pNew>>2]|0;
    $314 = $313;
    $315 = $314;
    HEAP32[$315>>2] = $311;
    $316 = (($314) + 4)|0;
    $317 = $316;
    HEAP32[$317>>2] = $312;
    $318 = HEAP16[$eOp>>1]|0;
    $319 = $318&65535;
    $320 = $319 & 1;
    $321 = ($320|0)!=(0);
    do {
     if ($321) {
      $322 = HEAP32[$pTerm>>2]|0;
      $323 = HEAP32[$322>>2]|0;
      HEAP32[$pExpr>>2] = $323;
      $324 = HEAP32[$pNew>>2]|0;
      $325 = (($324) + 36|0);
      $326 = HEAP32[$325>>2]|0;
      $327 = $326 | 4;
      HEAP32[$325>>2] = $327;
      $328 = HEAP32[$pExpr>>2]|0;
      $329 = (($328) + 4|0);
      $330 = HEAP32[$329>>2]|0;
      $331 = $330 & 2048;
      $332 = ($331|0)!=(0);
      if ($332) {
       HEAP32[$nIn>>2] = 46;
       break;
      }
      $333 = HEAP32[$pExpr>>2]|0;
      $334 = (($333) + 20|0);
      $335 = HEAP32[$334>>2]|0;
      $336 = ($335|0)!=(0|0);
      if ($336) {
       $337 = HEAP32[$pExpr>>2]|0;
       $338 = (($337) + 20|0);
       $339 = HEAP32[$338>>2]|0;
       $340 = HEAP32[$339>>2]|0;
       $341 = ($340|0)!=(0);
       if ($341) {
        $342 = HEAP32[$pExpr>>2]|0;
        $343 = (($342) + 20|0);
        $344 = HEAP32[$343>>2]|0;
        $345 = HEAP32[$344>>2]|0;
        $346 = ($345|0)<(0);
        $347 = $346 << 31 >> 31;
        $348 = (_sqlite3LogEst($345,$347)|0);
        $349 = $348 << 16 >> 16;
        HEAP32[$nIn>>2] = $349;
       }
      }
     } else {
      $350 = HEAP16[$eOp>>1]|0;
      $351 = $350&65535;
      $352 = $351 & 2;
      $353 = ($352|0)!=(0);
      if ($353) {
       $354 = HEAP32[$pNew>>2]|0;
       $355 = (($354) + 36|0);
       $356 = HEAP32[$355>>2]|0;
       $357 = $356 | 1;
       HEAP32[$355>>2] = $357;
       $358 = HEAP32[$iCol>>2]|0;
       $359 = ($358|0)<(0);
       if (!($359)) {
        $360 = HEAP16[$4>>1]|0;
        $361 = $360 << 16 >> 16;
        $362 = ($361|0)==(0);
        if (!($362)) {
         break;
        }
        $363 = HEAP32[$pNew>>2]|0;
        $364 = (($363) + 24|0);
        $365 = HEAP16[$364>>1]|0;
        $366 = $365&65535;
        $367 = HEAP32[$3>>2]|0;
        $368 = (($367) + 50|0);
        $369 = HEAP16[$368>>1]|0;
        $370 = $369&65535;
        $371 = (($370) - 1)|0;
        $372 = ($366|0)==($371|0);
        if (!($372)) {
         break;
        }
       }
       $373 = HEAP32[$iCol>>2]|0;
       $374 = ($373|0)>=(0);
       if ($374) {
        $375 = HEAP32[$3>>2]|0;
        $376 = (($375) + 54|0);
        $377 = HEAP8[$376>>0]|0;
        $378 = $377&255;
        $379 = ($378|0)!=(0);
        if (!($379)) {
         $380 = HEAP32[$pNew>>2]|0;
         $381 = (($380) + 36|0);
         $382 = HEAP32[$381>>2]|0;
         $383 = $382 | 65536;
         HEAP32[$381>>2] = $383;
         break;
        }
       }
       $384 = HEAP32[$pNew>>2]|0;
       $385 = (($384) + 36|0);
       $386 = HEAP32[$385>>2]|0;
       $387 = $386 | 4096;
       HEAP32[$385>>2] = $387;
       break;
      } else {
       $388 = HEAP16[$eOp>>1]|0;
       $389 = $388&65535;
       $390 = $389 & 128;
       $391 = ($390|0)!=(0);
       if ($391) {
        $392 = HEAP32[$pNew>>2]|0;
        $393 = (($392) + 36|0);
        $394 = HEAP32[$393>>2]|0;
        $395 = $394 | 8;
        HEAP32[$393>>2] = $395;
        break;
       }
       $396 = HEAP16[$eOp>>1]|0;
       $397 = $396&65535;
       $398 = $397 & 36;
       $399 = ($398|0)!=(0);
       $400 = HEAP32[$pNew>>2]|0;
       $401 = (($400) + 36|0);
       $402 = HEAP32[$401>>2]|0;
       if ($399) {
        $403 = $402 | 34;
        HEAP32[$401>>2] = $403;
        $404 = HEAP32[$pTerm>>2]|0;
        HEAP32[$pBtm>>2] = $404;
        HEAP32[$pTop>>2] = 0;
        break;
       }
       $405 = $402 | 18;
       HEAP32[$401>>2] = $405;
       $406 = HEAP32[$pTerm>>2]|0;
       HEAP32[$pTop>>2] = $406;
       $407 = HEAP32[$pNew>>2]|0;
       $408 = (($407) + 36|0);
       $409 = HEAP32[$408>>2]|0;
       $410 = $409 & 32;
       $411 = ($410|0)!=(0);
       if ($411) {
        $412 = HEAP32[$pNew>>2]|0;
        $413 = (($412) + 40|0);
        $414 = HEAP16[$413>>1]|0;
        $415 = $414&65535;
        $416 = (($415) - 2)|0;
        $417 = HEAP32[$pNew>>2]|0;
        $418 = (($417) + 44|0);
        $419 = HEAP32[$418>>2]|0;
        $420 = (($419) + ($416<<2)|0);
        $421 = HEAP32[$420>>2]|0;
        $422 = $421;
       } else {
        $422 = 0;
       }
       HEAP32[$pBtm>>2] = $422;
       break;
      }
     }
    } while(0);
    $423 = HEAP32[$pNew>>2]|0;
    $424 = (($423) + 36|0);
    $425 = HEAP32[$424>>2]|0;
    $426 = $425 & 2;
    $427 = ($426|0)!=(0);
    do {
     if ($427) {
      $428 = HEAP32[$pParse>>2]|0;
      $429 = HEAP32[$1>>2]|0;
      $430 = HEAP32[$pBtm>>2]|0;
      $431 = HEAP32[$pTop>>2]|0;
      $432 = HEAP32[$pNew>>2]|0;
      (_whereRangeScanEst($428,$429,$430,$431,$432)|0);
     } else {
      $433 = HEAP32[$pNew>>2]|0;
      $434 = (($433) + 24|0);
      $435 = HEAP16[$434>>1]|0;
      $436 = (($435) + 1)<<16>>16;
      HEAP16[$434>>1] = $436;
      $437 = $436&65535;
      HEAP32[$nEq>>2] = $437;
      $438 = HEAP32[$pTerm>>2]|0;
      $439 = (($438) + 16|0);
      $440 = HEAP16[$439>>1]|0;
      $441 = $440 << 16 >> 16;
      $442 = ($441|0)<=(0);
      $443 = HEAP32[$iCol>>2]|0;
      $444 = ($443|0)>=(0);
      $or$cond3 = $442 & $444;
      if ($or$cond3) {
       $445 = HEAP32[$pTerm>>2]|0;
       $446 = (($445) + 16|0);
       $447 = HEAP16[$446>>1]|0;
       $448 = $447 << 16 >> 16;
       $449 = HEAP32[$pNew>>2]|0;
       $450 = (($449) + 22|0);
       $451 = HEAP16[$450>>1]|0;
       $452 = $451 << 16 >> 16;
       $453 = (($452) + ($448))|0;
       $454 = $453&65535;
       HEAP16[$450>>1] = $454;
       $455 = HEAP32[$nIn>>2]|0;
       $456 = HEAP32[$pNew>>2]|0;
       $457 = (($456) + 22|0);
       $458 = HEAP16[$457>>1]|0;
       $459 = $458 << 16 >> 16;
       $460 = (($459) - ($455))|0;
       $461 = $460&65535;
       HEAP16[$457>>1] = $461;
       break;
      }
      $462 = HEAP32[$nEq>>2]|0;
      $463 = HEAP32[$3>>2]|0;
      $464 = (($463) + 8|0);
      $465 = HEAP32[$464>>2]|0;
      $466 = (($465) + ($462<<1)|0);
      $467 = HEAP16[$466>>1]|0;
      $468 = $467 << 16 >> 16;
      $469 = HEAP32[$nEq>>2]|0;
      $470 = (($469) - 1)|0;
      $471 = HEAP32[$3>>2]|0;
      $472 = (($471) + 8|0);
      $473 = HEAP32[$472>>2]|0;
      $474 = (($473) + ($470<<1)|0);
      $475 = HEAP16[$474>>1]|0;
      $476 = $475 << 16 >> 16;
      $477 = (($468) - ($476))|0;
      $478 = HEAP32[$pNew>>2]|0;
      $479 = (($478) + 22|0);
      $480 = HEAP16[$479>>1]|0;
      $481 = $480 << 16 >> 16;
      $482 = (($481) + ($477))|0;
      $483 = $482&65535;
      HEAP16[$479>>1] = $483;
      $484 = HEAP16[$eOp>>1]|0;
      $485 = $484&65535;
      $486 = $485 & 128;
      $487 = ($486|0)!=(0);
      if ($487) {
       $488 = HEAP32[$pNew>>2]|0;
       $489 = (($488) + 22|0);
       $490 = HEAP16[$489>>1]|0;
       $491 = $490 << 16 >> 16;
       $492 = (($491) + 10)|0;
       $493 = $492&65535;
       HEAP16[$489>>1] = $493;
      }
     }
    } while(0);
    $494 = HEAP32[$pNew>>2]|0;
    $495 = (($494) + 22|0);
    $496 = HEAP16[$495>>1]|0;
    $497 = $496 << 16 >> 16;
    $498 = (($497) + 1)|0;
    $499 = HEAP32[$3>>2]|0;
    $500 = (($499) + 48|0);
    $501 = HEAP16[$500>>1]|0;
    $502 = $501 << 16 >> 16;
    $503 = ($502*15)|0;
    $504 = HEAP32[$2>>2]|0;
    $505 = (($504) + 16|0);
    $506 = HEAP32[$505>>2]|0;
    $507 = (($506) + 42|0);
    $508 = HEAP16[$507>>1]|0;
    $509 = $508 << 16 >> 16;
    $510 = (($503|0) / ($509|0))&-1;
    $511 = (($498) + ($510))|0;
    $512 = $511&65535;
    HEAP16[$rCostIdx>>1] = $512;
    $513 = HEAP16[$rLogSize>>1]|0;
    $514 = HEAP16[$rCostIdx>>1]|0;
    $515 = (_sqlite3LogEstAdd($513,$514)|0);
    $516 = HEAP32[$pNew>>2]|0;
    $517 = (($516) + 20|0);
    HEAP16[$517>>1] = $515;
    $518 = HEAP32[$pNew>>2]|0;
    $519 = (($518) + 36|0);
    $520 = HEAP32[$519>>2]|0;
    $521 = $520 & 320;
    $522 = ($521|0)==(0);
    if ($522) {
     $523 = HEAP32[$pNew>>2]|0;
     $524 = (($523) + 20|0);
     $525 = HEAP16[$524>>1]|0;
     $526 = HEAP32[$pNew>>2]|0;
     $527 = (($526) + 22|0);
     $528 = HEAP16[$527>>1]|0;
     $529 = $528 << 16 >> 16;
     $530 = (($529) + 16)|0;
     $531 = $530&65535;
     $532 = (_sqlite3LogEstAdd($525,$531)|0);
     $533 = HEAP32[$pNew>>2]|0;
     $534 = (($533) + 20|0);
     HEAP16[$534>>1] = $532;
    }
    $535 = HEAP32[$pNew>>2]|0;
    $536 = (($535) + 22|0);
    $537 = HEAP16[$536>>1]|0;
    HEAP16[$nOutUnadjusted>>1] = $537;
    $538 = HEAP16[$4>>1]|0;
    $539 = $538 << 16 >> 16;
    $540 = HEAP32[$nIn>>2]|0;
    $541 = (($539) + ($540))|0;
    $542 = HEAP32[$pNew>>2]|0;
    $543 = (($542) + 20|0);
    $544 = HEAP16[$543>>1]|0;
    $545 = $544 << 16 >> 16;
    $546 = (($545) + ($541))|0;
    $547 = $546&65535;
    HEAP16[$543>>1] = $547;
    $548 = HEAP16[$4>>1]|0;
    $549 = $548 << 16 >> 16;
    $550 = HEAP32[$nIn>>2]|0;
    $551 = (($549) + ($550))|0;
    $552 = HEAP32[$pNew>>2]|0;
    $553 = (($552) + 22|0);
    $554 = HEAP16[$553>>1]|0;
    $555 = $554 << 16 >> 16;
    $556 = (($555) + ($551))|0;
    $557 = $556&65535;
    HEAP16[$553>>1] = $557;
    $558 = HEAP32[$1>>2]|0;
    $559 = (($558) + 4|0);
    $560 = HEAP32[$559>>2]|0;
    $561 = HEAP32[$pNew>>2]|0;
    $562 = HEAP16[$rSize>>1]|0;
    _whereLoopOutputAdjust($560,$561,$562);
    $563 = HEAP32[$1>>2]|0;
    $564 = HEAP32[$pNew>>2]|0;
    $565 = (_whereLoopInsert($563,$564)|0);
    HEAP32[$rc>>2] = $565;
    $566 = HEAP32[$pNew>>2]|0;
    $567 = (($566) + 36|0);
    $568 = HEAP32[$567>>2]|0;
    $569 = $568 & 2;
    $570 = ($569|0)!=(0);
    if ($570) {
     $571 = HEAP16[$saved_nOut>>1]|0;
     $572 = HEAP32[$pNew>>2]|0;
     $573 = (($572) + 22|0);
     HEAP16[$573>>1] = $571;
    } else {
     $574 = HEAP16[$nOutUnadjusted>>1]|0;
     $575 = HEAP32[$pNew>>2]|0;
     $576 = (($575) + 22|0);
     HEAP16[$576>>1] = $574;
    }
    $577 = HEAP32[$pNew>>2]|0;
    $578 = (($577) + 36|0);
    $579 = HEAP32[$578>>2]|0;
    $580 = $579 & 16;
    $581 = ($580|0)==(0);
    if ($581) {
     $582 = HEAP32[$pNew>>2]|0;
     $583 = (($582) + 24|0);
     $584 = HEAP16[$583>>1]|0;
     $585 = $584&65535;
     $586 = HEAP32[$3>>2]|0;
     $587 = (($586) + 52|0);
     $588 = HEAP16[$587>>1]|0;
     $589 = $588&65535;
     $590 = ($585|0)<($589|0);
     if ($590) {
      $591 = HEAP32[$1>>2]|0;
      $592 = HEAP32[$2>>2]|0;
      $593 = HEAP32[$3>>2]|0;
      $594 = HEAP16[$4>>1]|0;
      $595 = $594 << 16 >> 16;
      $596 = HEAP32[$nIn>>2]|0;
      $597 = (($595) + ($596))|0;
      $598 = $597&65535;
      (_whereLoopAddBtreeIndex($591,$592,$593,$598)|0);
     }
    }
    $599 = HEAP16[$saved_nOut>>1]|0;
    $600 = HEAP32[$pNew>>2]|0;
    $601 = (($600) + 22|0);
    HEAP16[$601>>1] = $599;
   }
  }
  $602 = (_whereScanNext($scan)|0);
  HEAP32[$pTerm>>2] = $602;
 }
 $603 = $saved_prereq;
 $604 = $603;
 $605 = HEAP32[$604>>2]|0;
 $606 = (($603) + 4)|0;
 $607 = $606;
 $608 = HEAP32[$607>>2]|0;
 $609 = HEAP32[$pNew>>2]|0;
 $610 = $609;
 $611 = $610;
 HEAP32[$611>>2] = $605;
 $612 = (($610) + 4)|0;
 $613 = $612;
 HEAP32[$613>>2] = $608;
 $614 = HEAP16[$saved_nEq>>1]|0;
 $615 = HEAP32[$pNew>>2]|0;
 $616 = (($615) + 24|0);
 HEAP16[$616>>1] = $614;
 $617 = HEAP16[$saved_nSkip>>1]|0;
 $618 = HEAP32[$pNew>>2]|0;
 $619 = (($618) + 24|0);
 $620 = (($619) + 2|0);
 HEAP16[$620>>1] = $617;
 $621 = HEAP32[$saved_wsFlags>>2]|0;
 $622 = HEAP32[$pNew>>2]|0;
 $623 = (($622) + 36|0);
 HEAP32[$623>>2] = $621;
 $624 = HEAP16[$saved_nOut>>1]|0;
 $625 = HEAP32[$pNew>>2]|0;
 $626 = (($625) + 22|0);
 HEAP16[$626>>1] = $624;
 $627 = HEAP16[$saved_nLTerm>>1]|0;
 $628 = HEAP32[$pNew>>2]|0;
 $629 = (($628) + 40|0);
 HEAP16[$629>>1] = $627;
 $630 = HEAP32[$rc>>2]|0;
 HEAP32[$0>>2] = $630;
 $631 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($631|0);
}
function _whereRangeScanEst($pParse,$pBuilder,$pLower,$pUpper,$pLoop) {
 $pParse = $pParse|0;
 $pBuilder = $pBuilder|0;
 $pLower = $pLower|0;
 $pUpper = $pUpper|0;
 $pLoop = $pLoop|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $nNew = 0, $nOut = 0, $or$cond = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 24|0;
 $1 = sp + 20|0;
 $2 = sp + 16|0;
 $3 = sp + 12|0;
 $4 = sp + 8|0;
 $rc = sp + 4|0;
 $nOut = sp;
 $nNew = sp + 28|0;
 HEAP32[$0>>2] = $pParse;
 HEAP32[$1>>2] = $pBuilder;
 HEAP32[$2>>2] = $pLower;
 HEAP32[$3>>2] = $pUpper;
 HEAP32[$4>>2] = $pLoop;
 HEAP32[$rc>>2] = 0;
 $5 = HEAP32[$4>>2]|0;
 $6 = (($5) + 22|0);
 $7 = HEAP16[$6>>1]|0;
 $8 = $7 << 16 >> 16;
 HEAP32[$nOut>>2] = $8;
 $9 = HEAP32[$2>>2]|0;
 $10 = HEAP32[$nOut>>2]|0;
 $11 = $10&65535;
 $12 = (_whereRangeAdjust($9,$11)|0);
 HEAP16[$nNew>>1] = $12;
 $13 = HEAP32[$3>>2]|0;
 $14 = HEAP16[$nNew>>1]|0;
 $15 = (_whereRangeAdjust($13,$14)|0);
 HEAP16[$nNew>>1] = $15;
 $16 = HEAP32[$2>>2]|0;
 $17 = ($16|0)!=(0|0);
 $18 = HEAP32[$3>>2]|0;
 $19 = ($18|0)!=(0|0);
 $or$cond = $17 & $19;
 if ($or$cond) {
  $20 = HEAP16[$nNew>>1]|0;
  $21 = $20 << 16 >> 16;
  $22 = (($21) - 20)|0;
  $23 = $22&65535;
  HEAP16[$nNew>>1] = $23;
 }
 $24 = HEAP32[$2>>2]|0;
 $25 = ($24|0)!=(0|0);
 $26 = $25&1;
 $27 = HEAP32[$3>>2]|0;
 $28 = ($27|0)!=(0|0);
 $29 = $28&1;
 $30 = (($26) + ($29))|0;
 $31 = HEAP32[$nOut>>2]|0;
 $32 = (($31) - ($30))|0;
 HEAP32[$nOut>>2] = $32;
 $33 = HEAP16[$nNew>>1]|0;
 $34 = $33 << 16 >> 16;
 $35 = ($34|0)<(10);
 if ($35) {
  HEAP16[$nNew>>1] = 10;
 }
 $36 = HEAP16[$nNew>>1]|0;
 $37 = $36 << 16 >> 16;
 $38 = HEAP32[$nOut>>2]|0;
 $39 = ($37|0)<($38|0);
 if (!($39)) {
  $42 = HEAP32[$nOut>>2]|0;
  $43 = $42&65535;
  $44 = HEAP32[$4>>2]|0;
  $45 = (($44) + 22|0);
  HEAP16[$45>>1] = $43;
  $46 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($46|0);
 }
 $40 = HEAP16[$nNew>>1]|0;
 $41 = $40 << 16 >> 16;
 HEAP32[$nOut>>2] = $41;
 $42 = HEAP32[$nOut>>2]|0;
 $43 = $42&65535;
 $44 = HEAP32[$4>>2]|0;
 $45 = (($44) + 22|0);
 HEAP16[$45>>1] = $43;
 $46 = HEAP32[$rc>>2]|0;
 STACKTOP = sp;return ($46|0);
}
function _whereRangeAdjust($pTerm,$nNew) {
 $pTerm = $pTerm|0;
 $nNew = $nNew|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $nRet = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 $1 = sp + 6|0;
 $nRet = sp + 4|0;
 HEAP32[$0>>2] = $pTerm;
 HEAP16[$1>>1] = $nNew;
 $2 = HEAP16[$1>>1]|0;
 HEAP16[$nRet>>1] = $2;
 $3 = HEAP32[$0>>2]|0;
 $4 = ($3|0)!=(0|0);
 do {
  if ($4) {
   $5 = HEAP32[$0>>2]|0;
   $6 = (($5) + 16|0);
   $7 = HEAP16[$6>>1]|0;
   $8 = $7 << 16 >> 16;
   $9 = ($8|0)<=(0);
   $10 = HEAP32[$0>>2]|0;
   if ($9) {
    $11 = (($10) + 16|0);
    $12 = HEAP16[$11>>1]|0;
    $13 = $12 << 16 >> 16;
    $14 = HEAP16[$nRet>>1]|0;
    $15 = $14 << 16 >> 16;
    $16 = (($15) + ($13))|0;
    $17 = $16&65535;
    HEAP16[$nRet>>1] = $17;
    break;
   }
   $18 = (($10) + 20|0);
   $19 = HEAP8[$18>>0]|0;
   $20 = $19&255;
   $21 = $20 & 0;
   $22 = ($21|0)==(0);
   if ($22) {
    $23 = HEAP16[$nRet>>1]|0;
    $24 = $23 << 16 >> 16;
    $25 = (($24) - 20)|0;
    $26 = $25&65535;
    HEAP16[$nRet>>1] = $26;
   }
  }
 } while(0);
 $27 = HEAP16[$nRet>>1]|0;
 STACKTOP = sp;return ($27|0);
}
function _sqlite3ExprImpliesExpr($pE1,$pE2,$iTab) {
 $pE1 = $pE1|0;
 $pE2 = $pE2|0;
 $iTab = $iTab|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $1 = sp + 8|0;
 $2 = sp + 4|0;
 $3 = sp;
 HEAP32[$1>>2] = $pE1;
 HEAP32[$2>>2] = $pE2;
 HEAP32[$3>>2] = $iTab;
 $4 = HEAP32[$1>>2]|0;
 $5 = HEAP32[$2>>2]|0;
 $6 = HEAP32[$3>>2]|0;
 $7 = (_sqlite3ExprCompare($4,$5,$6)|0);
 $8 = ($7|0)==(0);
 if ($8) {
  HEAP32[$0>>2] = 1;
  $48 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($48|0);
 }
 $9 = HEAP32[$2>>2]|0;
 $10 = HEAP8[$9>>0]|0;
 $11 = $10&255;
 $12 = ($11|0)==(71);
 do {
  if ($12) {
   $13 = HEAP32[$1>>2]|0;
   $14 = HEAP32[$2>>2]|0;
   $15 = (($14) + 12|0);
   $16 = HEAP32[$15>>2]|0;
   $17 = HEAP32[$3>>2]|0;
   $18 = (_sqlite3ExprImpliesExpr($13,$16,$17)|0);
   $19 = ($18|0)!=(0);
   if (!($19)) {
    $20 = HEAP32[$1>>2]|0;
    $21 = HEAP32[$2>>2]|0;
    $22 = (($21) + 16|0);
    $23 = HEAP32[$22>>2]|0;
    $24 = HEAP32[$3>>2]|0;
    $25 = (_sqlite3ExprImpliesExpr($20,$23,$24)|0);
    $26 = ($25|0)!=(0);
    if (!($26)) {
     break;
    }
   }
   HEAP32[$0>>2] = 1;
   $48 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($48|0);
  }
 } while(0);
 $27 = HEAP32[$2>>2]|0;
 $28 = HEAP8[$27>>0]|0;
 $29 = $28&255;
 $30 = ($29|0)==(77);
 if ($30) {
  $31 = HEAP32[$1>>2]|0;
  $32 = (($31) + 12|0);
  $33 = HEAP32[$32>>2]|0;
  $34 = HEAP32[$2>>2]|0;
  $35 = (($34) + 12|0);
  $36 = HEAP32[$35>>2]|0;
  $37 = HEAP32[$3>>2]|0;
  $38 = (_sqlite3ExprCompare($33,$36,$37)|0);
  $39 = ($38|0)==(0);
  if ($39) {
   $40 = HEAP32[$1>>2]|0;
   $41 = HEAP8[$40>>0]|0;
   $42 = $41&255;
   $43 = ($42|0)!=(76);
   if ($43) {
    $44 = HEAP32[$1>>2]|0;
    $45 = HEAP8[$44>>0]|0;
    $46 = $45&255;
    $47 = ($46|0)!=(73);
    if ($47) {
     HEAP32[$0>>2] = 1;
     $48 = HEAP32[$0>>2]|0;
     STACKTOP = sp;return ($48|0);
    }
   }
  }
 }
 HEAP32[$0>>2] = 0;
 $48 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($48|0);
}
function _allocateIndexInfo($pParse,$pWC,$pSrc,$pOrderBy) {
 $pParse = $pParse|0;
 $pWC = $pWC|0;
 $pSrc = $pSrc|0;
 $pOrderBy = $pOrderBy|0;
 var $$ = 0, $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0;
 var $115 = 0, $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0;
 var $133 = 0, $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0;
 var $151 = 0, $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0;
 var $17 = 0, $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0;
 var $188 = 0, $189 = 0, $19 = 0, $190 = 0, $191 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0;
 var $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0;
 var $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0;
 var $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0;
 var $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $i = 0, $j = 0, $n = 0, $nOrderBy = 0, $nTerm = 0, $op = 0, $pExpr = 0;
 var $pExpr1 = 0, $pIdxCons = 0, $pIdxInfo = 0, $pIdxOrderBy = 0, $pTerm = 0, $pUsage = 0, $vararg_buffer = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 80|0;
 $vararg_buffer = sp;
 $0 = sp + 68|0;
 $1 = sp + 64|0;
 $2 = sp + 60|0;
 $3 = sp + 56|0;
 $4 = sp + 52|0;
 $i = sp + 48|0;
 $j = sp + 44|0;
 $nTerm = sp + 40|0;
 $pIdxCons = sp + 36|0;
 $pIdxOrderBy = sp + 32|0;
 $pUsage = sp + 28|0;
 $pTerm = sp + 24|0;
 $nOrderBy = sp + 20|0;
 $pIdxInfo = sp + 16|0;
 $n = sp + 12|0;
 $pExpr = sp + 8|0;
 $op = sp + 72|0;
 $pExpr1 = sp + 4|0;
 HEAP32[$1>>2] = $pParse;
 HEAP32[$2>>2] = $pWC;
 HEAP32[$3>>2] = $pSrc;
 HEAP32[$4>>2] = $pOrderBy;
 HEAP32[$nTerm>>2] = 0;
 HEAP32[$i>>2] = 0;
 $5 = HEAP32[$2>>2]|0;
 $6 = (($5) + 20|0);
 $7 = HEAP32[$6>>2]|0;
 HEAP32[$pTerm>>2] = $7;
 while(1) {
  $8 = HEAP32[$i>>2]|0;
  $9 = HEAP32[$2>>2]|0;
  $10 = (($9) + 12|0);
  $11 = HEAP32[$10>>2]|0;
  $12 = ($8|0)<($11|0);
  if (!($12)) {
   break;
  }
  $13 = HEAP32[$pTerm>>2]|0;
  $14 = (($13) + 8|0);
  $15 = HEAP32[$14>>2]|0;
  $16 = HEAP32[$3>>2]|0;
  $17 = (($16) + 40|0);
  $18 = HEAP32[$17>>2]|0;
  $19 = ($15|0)!=($18|0);
  if (!($19)) {
   $20 = HEAP32[$pTerm>>2]|0;
   $21 = (($20) + 18|0);
   $22 = HEAP16[$21>>1]|0;
   $23 = $22&65535;
   $24 = $23 & -1153;
   $25 = ($24|0)==(0);
   if (!($25)) {
    $26 = HEAP32[$pTerm>>2]|0;
    $27 = (($26) + 20|0);
    $28 = HEAP8[$27>>0]|0;
    $29 = $28&255;
    $30 = $29 & 0;
    $31 = ($30|0)!=(0);
    if (!($31)) {
     $32 = HEAP32[$nTerm>>2]|0;
     $33 = (($32) + 1)|0;
     HEAP32[$nTerm>>2] = $33;
    }
   }
  }
  $34 = HEAP32[$i>>2]|0;
  $35 = (($34) + 1)|0;
  HEAP32[$i>>2] = $35;
  $36 = HEAP32[$pTerm>>2]|0;
  $37 = (($36) + 48|0);
  HEAP32[$pTerm>>2] = $37;
 }
 HEAP32[$nOrderBy>>2] = 0;
 $38 = HEAP32[$4>>2]|0;
 $39 = ($38|0)!=(0|0);
 if ($39) {
  $40 = HEAP32[$4>>2]|0;
  $41 = HEAP32[$40>>2]|0;
  HEAP32[$n>>2] = $41;
  HEAP32[$i>>2] = 0;
  while(1) {
   $42 = HEAP32[$i>>2]|0;
   $43 = HEAP32[$n>>2]|0;
   $44 = ($42|0)<($43|0);
   if (!($44)) {
    break;
   }
   $45 = HEAP32[$i>>2]|0;
   $46 = HEAP32[$4>>2]|0;
   $47 = (($46) + 4|0);
   $48 = HEAP32[$47>>2]|0;
   $49 = (($48) + (($45*20)|0)|0);
   $50 = HEAP32[$49>>2]|0;
   HEAP32[$pExpr>>2] = $50;
   $51 = HEAP32[$pExpr>>2]|0;
   $52 = HEAP8[$51>>0]|0;
   $53 = $52&255;
   $54 = ($53|0)!=(154);
   if ($54) {
    break;
   }
   $55 = HEAP32[$pExpr>>2]|0;
   $56 = (($55) + 28|0);
   $57 = HEAP32[$56>>2]|0;
   $58 = HEAP32[$3>>2]|0;
   $59 = (($58) + 40|0);
   $60 = HEAP32[$59>>2]|0;
   $61 = ($57|0)!=($60|0);
   if ($61) {
    break;
   }
   $62 = HEAP32[$i>>2]|0;
   $63 = (($62) + 1)|0;
   HEAP32[$i>>2] = $63;
  }
  $64 = HEAP32[$i>>2]|0;
  $65 = HEAP32[$n>>2]|0;
  $66 = ($64|0)==($65|0);
  if ($66) {
   $67 = HEAP32[$n>>2]|0;
   HEAP32[$nOrderBy>>2] = $67;
  }
 }
 $68 = HEAP32[$1>>2]|0;
 $69 = HEAP32[$68>>2]|0;
 $70 = HEAP32[$nTerm>>2]|0;
 $71 = ($70*20)|0;
 $72 = (56 + ($71))|0;
 $73 = HEAP32[$nOrderBy>>2]|0;
 $74 = $73<<3;
 $75 = (($72) + ($74))|0;
 $76 = (_sqlite3DbMallocZero($69,$75,0)|0);
 HEAP32[$pIdxInfo>>2] = $76;
 $77 = HEAP32[$pIdxInfo>>2]|0;
 $78 = ($77|0)==(0|0);
 if ($78) {
  $79 = HEAP32[$1>>2]|0;
  _sqlite3ErrorMsg($79,5400,$vararg_buffer);
  HEAP32[$0>>2] = 0;
  $191 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($191|0);
 }
 $80 = HEAP32[$pIdxInfo>>2]|0;
 $81 = (($80) + 56|0);
 HEAP32[$pIdxCons>>2] = $81;
 $82 = HEAP32[$nTerm>>2]|0;
 $83 = HEAP32[$pIdxCons>>2]|0;
 $84 = (($83) + (($82*12)|0)|0);
 HEAP32[$pIdxOrderBy>>2] = $84;
 $85 = HEAP32[$nOrderBy>>2]|0;
 $86 = HEAP32[$pIdxOrderBy>>2]|0;
 $87 = (($86) + ($85<<3)|0);
 HEAP32[$pUsage>>2] = $87;
 $88 = HEAP32[$nTerm>>2]|0;
 $89 = HEAP32[$pIdxInfo>>2]|0;
 HEAP32[$89>>2] = $88;
 $90 = HEAP32[$nOrderBy>>2]|0;
 $91 = HEAP32[$pIdxInfo>>2]|0;
 $92 = (($91) + 8|0);
 HEAP32[$92>>2] = $90;
 $93 = HEAP32[$pIdxCons>>2]|0;
 $94 = HEAP32[$pIdxInfo>>2]|0;
 $95 = (($94) + 4|0);
 HEAP32[$95>>2] = $93;
 $96 = HEAP32[$pIdxOrderBy>>2]|0;
 $97 = HEAP32[$pIdxInfo>>2]|0;
 $98 = (($97) + 12|0);
 HEAP32[$98>>2] = $96;
 $99 = HEAP32[$pUsage>>2]|0;
 $100 = HEAP32[$pIdxInfo>>2]|0;
 $101 = (($100) + 16|0);
 HEAP32[$101>>2] = $99;
 HEAP32[$j>>2] = 0;
 HEAP32[$i>>2] = 0;
 $102 = HEAP32[$2>>2]|0;
 $103 = (($102) + 20|0);
 $104 = HEAP32[$103>>2]|0;
 HEAP32[$pTerm>>2] = $104;
 while(1) {
  $105 = HEAP32[$i>>2]|0;
  $106 = HEAP32[$2>>2]|0;
  $107 = (($106) + 12|0);
  $108 = HEAP32[$107>>2]|0;
  $109 = ($105|0)<($108|0);
  if (!($109)) {
   break;
  }
  $110 = HEAP32[$pTerm>>2]|0;
  $111 = (($110) + 8|0);
  $112 = HEAP32[$111>>2]|0;
  $113 = HEAP32[$3>>2]|0;
  $114 = (($113) + 40|0);
  $115 = HEAP32[$114>>2]|0;
  $116 = ($112|0)!=($115|0);
  if (!($116)) {
   $117 = HEAP32[$pTerm>>2]|0;
   $118 = (($117) + 18|0);
   $119 = HEAP16[$118>>1]|0;
   $120 = $119&65535;
   $121 = $120 & -1153;
   $122 = ($121|0)==(0);
   if (!($122)) {
    $123 = HEAP32[$pTerm>>2]|0;
    $124 = (($123) + 20|0);
    $125 = HEAP8[$124>>0]|0;
    $126 = $125&255;
    $127 = $126 & 0;
    $128 = ($127|0)!=(0);
    if (!($128)) {
     $129 = HEAP32[$pTerm>>2]|0;
     $130 = (($129) + 12|0);
     $131 = HEAP32[$130>>2]|0;
     $132 = HEAP32[$j>>2]|0;
     $133 = HEAP32[$pIdxCons>>2]|0;
     $134 = (($133) + (($132*12)|0)|0);
     HEAP32[$134>>2] = $131;
     $135 = HEAP32[$i>>2]|0;
     $136 = HEAP32[$j>>2]|0;
     $137 = HEAP32[$pIdxCons>>2]|0;
     $138 = (($137) + (($136*12)|0)|0);
     $139 = (($138) + 8|0);
     HEAP32[$139>>2] = $135;
     $140 = HEAP32[$pTerm>>2]|0;
     $141 = (($140) + 18|0);
     $142 = HEAP16[$141>>1]|0;
     $143 = $142&255;
     $144 = $143&255;
     $145 = $144 & 4095;
     $146 = $145&255;
     HEAP8[$op>>0] = $146;
     $147 = HEAP8[$op>>0]|0;
     $148 = $147&255;
     $149 = ($148|0)==(1);
     $$ = $149 ? 2 : $146;
     HEAP8[$op>>0] = $$;
     $150 = HEAP8[$op>>0]|0;
     $151 = HEAP32[$j>>2]|0;
     $152 = HEAP32[$pIdxCons>>2]|0;
     $153 = (($152) + (($151*12)|0)|0);
     $154 = (($153) + 4|0);
     HEAP8[$154>>0] = $150;
     $155 = HEAP32[$j>>2]|0;
     $156 = (($155) + 1)|0;
     HEAP32[$j>>2] = $156;
    }
   }
  }
  $157 = HEAP32[$i>>2]|0;
  $158 = (($157) + 1)|0;
  HEAP32[$i>>2] = $158;
  $159 = HEAP32[$pTerm>>2]|0;
  $160 = (($159) + 48|0);
  HEAP32[$pTerm>>2] = $160;
 }
 HEAP32[$i>>2] = 0;
 while(1) {
  $161 = HEAP32[$i>>2]|0;
  $162 = HEAP32[$nOrderBy>>2]|0;
  $163 = ($161|0)<($162|0);
  if (!($163)) {
   break;
  }
  $164 = HEAP32[$i>>2]|0;
  $165 = HEAP32[$4>>2]|0;
  $166 = (($165) + 4|0);
  $167 = HEAP32[$166>>2]|0;
  $168 = (($167) + (($164*20)|0)|0);
  $169 = HEAP32[$168>>2]|0;
  HEAP32[$pExpr1>>2] = $169;
  $170 = HEAP32[$pExpr1>>2]|0;
  $171 = (($170) + 32|0);
  $172 = HEAP16[$171>>1]|0;
  $173 = $172 << 16 >> 16;
  $174 = HEAP32[$i>>2]|0;
  $175 = HEAP32[$pIdxOrderBy>>2]|0;
  $176 = (($175) + ($174<<3)|0);
  HEAP32[$176>>2] = $173;
  $177 = HEAP32[$i>>2]|0;
  $178 = HEAP32[$4>>2]|0;
  $179 = (($178) + 4|0);
  $180 = HEAP32[$179>>2]|0;
  $181 = (($180) + (($177*20)|0)|0);
  $182 = (($181) + 12|0);
  $183 = HEAP8[$182>>0]|0;
  $184 = HEAP32[$i>>2]|0;
  $185 = HEAP32[$pIdxOrderBy>>2]|0;
  $186 = (($185) + ($184<<3)|0);
  $187 = (($186) + 4|0);
  HEAP8[$187>>0] = $183;
  $188 = HEAP32[$i>>2]|0;
  $189 = (($188) + 1)|0;
  HEAP32[$i>>2] = $189;
 }
 $190 = HEAP32[$pIdxInfo>>2]|0;
 HEAP32[$0>>2] = $190;
 $191 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($191|0);
}
function _vtabBestIndex($pParse,$pTab,$p) {
 $pParse = $pParse|0;
 $pTab = $pTab|0;
 $p = $p|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $7 = 0, $8 = 0, $9 = 0, $i = 0, $pVtab = 0, $rc = 0, $vararg_buffer4 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $vararg_buffer4 = sp;
 $0 = sp + 24|0;
 $1 = sp + 20|0;
 $2 = sp + 16|0;
 $pVtab = sp + 12|0;
 $i = sp + 8|0;
 $rc = sp + 4|0;
 HEAP32[$0>>2] = $pParse;
 HEAP32[$1>>2] = $pTab;
 HEAP32[$2>>2] = $p;
 $3 = HEAP32[$0>>2]|0;
 $4 = HEAP32[$3>>2]|0;
 $5 = HEAP32[$1>>2]|0;
 $6 = (_sqlite3GetVTable($4,$5)|0);
 $7 = (($6) + 8|0);
 $8 = HEAP32[$7>>2]|0;
 HEAP32[$pVtab>>2] = $8;
 $9 = HEAP32[$pVtab>>2]|0;
 $10 = HEAP32[$9>>2]|0;
 $11 = (($10) + 12|0);
 $12 = HEAP32[$11>>2]|0;
 $13 = HEAP32[$pVtab>>2]|0;
 $14 = HEAP32[$2>>2]|0;
 $15 = (FUNCTION_TABLE_iii[$12 & 63]($13,$14)|0);
 HEAP32[$rc>>2] = $15;
 $16 = HEAP32[$rc>>2]|0;
 $17 = ($16|0)!=(0);
 do {
  if ($17) {
   $18 = HEAP32[$rc>>2]|0;
   $19 = ($18|0)==(7);
   if ($19) {
    $20 = HEAP32[$0>>2]|0;
    $21 = HEAP32[$20>>2]|0;
    $22 = (($21) + 64|0);
    HEAP8[$22>>0] = 1;
    break;
   }
   $23 = HEAP32[$pVtab>>2]|0;
   $24 = (($23) + 8|0);
   $25 = HEAP32[$24>>2]|0;
   $26 = ($25|0)!=(0|0);
   $27 = HEAP32[$0>>2]|0;
   if ($26) {
    $30 = HEAP32[$pVtab>>2]|0;
    $31 = (($30) + 8|0);
    $32 = HEAP32[$31>>2]|0;
    HEAP32[$vararg_buffer4>>2] = $32;
    _sqlite3ErrorMsg($27,1200,$vararg_buffer4);
    break;
   } else {
    $28 = HEAP32[$rc>>2]|0;
    $29 = (_sqlite3ErrStr($28)|0);
    HEAP32[$vararg_buffer4>>2] = $29;
    _sqlite3ErrorMsg($27,1200,$vararg_buffer4);
    break;
   }
  }
 } while(0);
 $33 = HEAP32[$pVtab>>2]|0;
 $34 = (($33) + 8|0);
 $35 = HEAP32[$34>>2]|0;
 _sqlite3_free($35);
 $36 = HEAP32[$pVtab>>2]|0;
 $37 = (($36) + 8|0);
 HEAP32[$37>>2] = 0;
 HEAP32[$i>>2] = 0;
 while(1) {
  $38 = HEAP32[$i>>2]|0;
  $39 = HEAP32[$2>>2]|0;
  $40 = HEAP32[$39>>2]|0;
  $41 = ($38|0)<($40|0);
  if (!($41)) {
   break;
  }
  $42 = HEAP32[$i>>2]|0;
  $43 = HEAP32[$2>>2]|0;
  $44 = (($43) + 4|0);
  $45 = HEAP32[$44>>2]|0;
  $46 = (($45) + (($42*12)|0)|0);
  $47 = (($46) + 5|0);
  $48 = HEAP8[$47>>0]|0;
  $49 = ($48<<24>>24)!=(0);
  if (!($49)) {
   $50 = HEAP32[$i>>2]|0;
   $51 = HEAP32[$2>>2]|0;
   $52 = (($51) + 16|0);
   $53 = HEAP32[$52>>2]|0;
   $54 = (($53) + ($50<<3)|0);
   $55 = HEAP32[$54>>2]|0;
   $56 = ($55|0)>(0);
   if ($56) {
    $57 = HEAP32[$0>>2]|0;
    $58 = HEAP32[$1>>2]|0;
    $59 = HEAP32[$58>>2]|0;
    HEAP32[$vararg_buffer4>>2] = $59;
    _sqlite3ErrorMsg($57,24120,$vararg_buffer4);
   }
  }
  $60 = HEAP32[$i>>2]|0;
  $61 = (($60) + 1)|0;
  HEAP32[$i>>2] = $61;
 }
 $62 = HEAP32[$0>>2]|0;
 $63 = (($62) + 64|0);
 $64 = HEAP32[$63>>2]|0;
 STACKTOP = sp;return ($64|0);
}
function _sqlite3LogEstFromDouble($x) {
 $x = +$x;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0.0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $3 = 0;
 var $4 = 0.0, $5 = 0, $6 = 0.0, $7 = 0, $8 = 0, $9 = 0, $a = 0, $e = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 18|0;
 $1 = sp + 8|0;
 $a = sp;
 $e = sp + 16|0;
 HEAPF64[$1>>3] = $x;
 $2 = +HEAPF64[$1>>3];
 $3 = $2 <= 1.0;
 if ($3) {
  HEAP16[$0>>1] = 0;
  $25 = HEAP16[$0>>1]|0;
  STACKTOP = sp;return ($25|0);
 }
 $4 = +HEAPF64[$1>>3];
 $5 = $4 <= 2.0E+9;
 if ($5) {
  $6 = +HEAPF64[$1>>3];
  $7 = (~~$6)>>>0;
  $8 = +Math_abs($6) >= 1.0 ? $6 > 0.0 ? (Math_min(+Math_floor($6 / 4294967296.0), 4294967295.0) | 0) >>> 0 : ~~+Math_ceil(($6 - +(~~$6 >>> 0)) / 4294967296.0) >>> 0 : 0;
  $9 = (_sqlite3LogEst($7,$8)|0);
  HEAP16[$0>>1] = $9;
  $25 = HEAP16[$0>>1]|0;
  STACKTOP = sp;return ($25|0);
 } else {
  ;HEAP32[$a+0>>2]=HEAP32[$1+0>>2]|0;HEAP32[$a+4>>2]=HEAP32[$1+4>>2]|0;
  $10 = $a;
  $11 = $10;
  $12 = HEAP32[$11>>2]|0;
  $13 = (($10) + 4)|0;
  $14 = $13;
  $15 = HEAP32[$14>>2]|0;
  $16 = (_bitshift64Lshr(($12|0),($15|0),52)|0);
  $17 = tempRet0;
  $18 = (_i64Subtract(($16|0),($17|0),1022,0)|0);
  $19 = tempRet0;
  $20 = $18&65535;
  HEAP16[$e>>1] = $20;
  $21 = HEAP16[$e>>1]|0;
  $22 = $21 << 16 >> 16;
  $23 = ($22*10)|0;
  $24 = $23&65535;
  HEAP16[$0>>1] = $24;
  $25 = HEAP16[$0>>1]|0;
  STACKTOP = sp;return ($25|0);
 }
 return 0|0;
}
function _findIndexCol($pParse,$pList,$iBase,$pIdx,$iCol) {
 $pParse = $pParse|0;
 $pList = $pList|0;
 $iBase = $iBase|0;
 $pIdx = $pIdx|0;
 $iCol = $iCol|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $7 = 0, $8 = 0, $9 = 0, $i = 0, $p = 0, $pColl = 0, $zColl = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $0 = sp + 36|0;
 $1 = sp + 32|0;
 $2 = sp + 28|0;
 $3 = sp + 24|0;
 $4 = sp + 20|0;
 $5 = sp + 16|0;
 $i = sp + 12|0;
 $zColl = sp + 8|0;
 $p = sp + 4|0;
 $pColl = sp;
 HEAP32[$1>>2] = $pParse;
 HEAP32[$2>>2] = $pList;
 HEAP32[$3>>2] = $iBase;
 HEAP32[$4>>2] = $pIdx;
 HEAP32[$5>>2] = $iCol;
 $6 = HEAP32[$5>>2]|0;
 $7 = HEAP32[$4>>2]|0;
 $8 = (($7) + 32|0);
 $9 = HEAP32[$8>>2]|0;
 $10 = (($9) + ($6<<2)|0);
 $11 = HEAP32[$10>>2]|0;
 HEAP32[$zColl>>2] = $11;
 HEAP32[$i>>2] = 0;
 while(1) {
  $12 = HEAP32[$i>>2]|0;
  $13 = HEAP32[$2>>2]|0;
  $14 = HEAP32[$13>>2]|0;
  $15 = ($12|0)<($14|0);
  if (!($15)) {
   label = 10;
   break;
  }
  $16 = HEAP32[$i>>2]|0;
  $17 = HEAP32[$2>>2]|0;
  $18 = (($17) + 4|0);
  $19 = HEAP32[$18>>2]|0;
  $20 = (($19) + (($16*20)|0)|0);
  $21 = HEAP32[$20>>2]|0;
  $22 = (_sqlite3ExprSkipCollate($21)|0);
  HEAP32[$p>>2] = $22;
  $23 = HEAP32[$p>>2]|0;
  $24 = HEAP8[$23>>0]|0;
  $25 = $24&255;
  $26 = ($25|0)==(154);
  if ($26) {
   $27 = HEAP32[$p>>2]|0;
   $28 = (($27) + 32|0);
   $29 = HEAP16[$28>>1]|0;
   $30 = $29 << 16 >> 16;
   $31 = HEAP32[$5>>2]|0;
   $32 = HEAP32[$4>>2]|0;
   $33 = (($32) + 4|0);
   $34 = HEAP32[$33>>2]|0;
   $35 = (($34) + ($31<<1)|0);
   $36 = HEAP16[$35>>1]|0;
   $37 = $36 << 16 >> 16;
   $38 = ($30|0)==($37|0);
   if ($38) {
    $39 = HEAP32[$p>>2]|0;
    $40 = (($39) + 28|0);
    $41 = HEAP32[$40>>2]|0;
    $42 = HEAP32[$3>>2]|0;
    $43 = ($41|0)==($42|0);
    if ($43) {
     $44 = HEAP32[$1>>2]|0;
     $45 = HEAP32[$i>>2]|0;
     $46 = HEAP32[$2>>2]|0;
     $47 = (($46) + 4|0);
     $48 = HEAP32[$47>>2]|0;
     $49 = (($48) + (($45*20)|0)|0);
     $50 = HEAP32[$49>>2]|0;
     $51 = (_sqlite3ExprCollSeq($44,$50)|0);
     HEAP32[$pColl>>2] = $51;
     $52 = HEAP32[$pColl>>2]|0;
     $53 = ($52|0)!=(0|0);
     if ($53) {
      $54 = HEAP32[$pColl>>2]|0;
      $55 = HEAP32[$54>>2]|0;
      $56 = HEAP32[$zColl>>2]|0;
      $57 = (_sqlite3_stricmp($55,$56)|0);
      $58 = (0)==($57|0);
      if ($58) {
       label = 8;
       break;
      }
     }
    }
   }
  }
  $60 = HEAP32[$i>>2]|0;
  $61 = (($60) + 1)|0;
  HEAP32[$i>>2] = $61;
 }
 if ((label|0) == 8) {
  $59 = HEAP32[$i>>2]|0;
  HEAP32[$0>>2] = $59;
  $62 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($62|0);
 }
 else if ((label|0) == 10) {
  HEAP32[$0>>2] = -1;
  $62 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($62|0);
 }
 return 0|0;
}
function _exprAnalyze($pSrc,$pWC,$idxTerm) {
 $pSrc = $pSrc|0;
 $pWC = $pWC|0;
 $idxTerm = $idxTerm|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0;
 var $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0, $188 = 0;
 var $189 = 0, $19 = 0, $190 = 0, $191 = 0, $192 = 0, $193 = 0, $194 = 0, $195 = 0, $196 = 0, $197 = 0, $198 = 0, $199 = 0, $2 = 0, $20 = 0, $200 = 0, $201 = 0, $202 = 0, $203 = 0, $204 = 0, $205 = 0;
 var $206 = 0, $207 = 0, $208 = 0, $209 = 0, $21 = 0, $210 = 0, $211 = 0, $212 = 0, $213 = 0, $214 = 0, $215 = 0, $216 = 0, $217 = 0, $218 = 0, $219 = 0, $22 = 0, $220 = 0, $221 = 0, $222 = 0, $223 = 0;
 var $224 = 0, $225 = 0, $226 = 0, $227 = 0, $228 = 0, $229 = 0, $23 = 0, $230 = 0, $231 = 0, $232 = 0, $233 = 0, $234 = 0, $235 = 0, $236 = 0, $237 = 0, $238 = 0, $239 = 0, $24 = 0, $240 = 0, $241 = 0;
 var $242 = 0, $243 = 0, $244 = 0, $245 = 0, $246 = 0, $247 = 0, $248 = 0, $249 = 0, $25 = 0, $250 = 0, $251 = 0, $252 = 0, $253 = 0, $254 = 0, $255 = 0, $256 = 0, $257 = 0, $258 = 0, $259 = 0, $26 = 0;
 var $260 = 0, $261 = 0, $262 = 0, $263 = 0, $264 = 0, $265 = 0, $266 = 0, $267 = 0, $268 = 0, $269 = 0, $27 = 0, $270 = 0, $271 = 0, $272 = 0, $273 = 0, $274 = 0, $275 = 0, $276 = 0, $277 = 0, $278 = 0;
 var $279 = 0, $28 = 0, $280 = 0, $281 = 0, $282 = 0, $283 = 0, $284 = 0, $285 = 0, $286 = 0, $287 = 0, $288 = 0, $289 = 0, $29 = 0, $290 = 0, $291 = 0, $292 = 0, $293 = 0, $294 = 0, $295 = 0, $296 = 0;
 var $297 = 0, $298 = 0, $299 = 0, $3 = 0, $30 = 0, $300 = 0, $301 = 0, $302 = 0, $303 = 0, $304 = 0, $305 = 0, $306 = 0, $307 = 0, $308 = 0, $309 = 0, $31 = 0, $310 = 0, $311 = 0, $312 = 0, $313 = 0;
 var $314 = 0, $315 = 0, $316 = 0, $317 = 0, $318 = 0, $319 = 0, $32 = 0, $320 = 0, $321 = 0, $322 = 0, $323 = 0, $324 = 0, $325 = 0, $326 = 0, $327 = 0, $328 = 0, $329 = 0, $33 = 0, $330 = 0, $331 = 0;
 var $332 = 0, $333 = 0, $334 = 0, $335 = 0, $336 = 0, $337 = 0, $338 = 0, $339 = 0, $34 = 0, $340 = 0, $341 = 0, $342 = 0, $343 = 0, $344 = 0, $345 = 0, $346 = 0, $347 = 0, $348 = 0, $349 = 0, $35 = 0;
 var $350 = 0, $351 = 0, $352 = 0, $353 = 0, $354 = 0, $355 = 0, $356 = 0, $357 = 0, $358 = 0, $359 = 0, $36 = 0, $360 = 0, $361 = 0, $362 = 0, $363 = 0, $364 = 0, $365 = 0, $366 = 0, $367 = 0, $368 = 0;
 var $369 = 0, $37 = 0, $370 = 0, $371 = 0, $372 = 0, $373 = 0, $374 = 0, $375 = 0, $376 = 0, $377 = 0, $378 = 0, $379 = 0, $38 = 0, $380 = 0, $381 = 0, $382 = 0, $383 = 0, $384 = 0, $385 = 0, $386 = 0;
 var $387 = 0, $388 = 0, $389 = 0, $39 = 0, $390 = 0, $391 = 0, $392 = 0, $393 = 0, $394 = 0, $395 = 0, $396 = 0, $397 = 0, $398 = 0, $399 = 0, $4 = 0, $40 = 0, $400 = 0, $401 = 0, $402 = 0, $403 = 0;
 var $404 = 0, $405 = 0, $406 = 0, $407 = 0, $408 = 0, $409 = 0, $41 = 0, $410 = 0, $411 = 0, $412 = 0, $413 = 0, $414 = 0, $415 = 0, $416 = 0, $417 = 0, $418 = 0, $419 = 0, $42 = 0, $420 = 0, $421 = 0;
 var $422 = 0, $423 = 0, $424 = 0, $425 = 0, $426 = 0, $427 = 0, $428 = 0, $429 = 0, $43 = 0, $430 = 0, $431 = 0, $432 = 0, $433 = 0, $434 = 0, $435 = 0, $436 = 0, $437 = 0, $438 = 0, $439 = 0, $44 = 0;
 var $440 = 0, $441 = 0, $442 = 0, $443 = 0, $444 = 0, $445 = 0, $446 = 0, $447 = 0, $448 = 0, $449 = 0, $45 = 0, $450 = 0, $451 = 0, $452 = 0, $453 = 0, $454 = 0, $455 = 0, $456 = 0, $457 = 0, $458 = 0;
 var $459 = 0, $46 = 0, $460 = 0, $461 = 0, $462 = 0, $463 = 0, $464 = 0, $465 = 0, $466 = 0, $467 = 0, $468 = 0, $469 = 0, $47 = 0, $470 = 0, $471 = 0, $472 = 0, $473 = 0, $474 = 0, $475 = 0, $476 = 0;
 var $477 = 0, $478 = 0, $479 = 0, $48 = 0, $480 = 0, $481 = 0, $482 = 0, $483 = 0, $484 = 0, $485 = 0, $486 = 0, $487 = 0, $488 = 0, $489 = 0, $49 = 0, $490 = 0, $491 = 0, $492 = 0, $493 = 0, $494 = 0;
 var $495 = 0, $496 = 0, $497 = 0, $498 = 0, $499 = 0, $5 = 0, $50 = 0, $500 = 0, $501 = 0, $502 = 0, $503 = 0, $504 = 0, $505 = 0, $506 = 0, $507 = 0, $508 = 0, $509 = 0, $51 = 0, $510 = 0, $511 = 0;
 var $512 = 0, $513 = 0, $514 = 0, $515 = 0, $516 = 0, $517 = 0, $518 = 0, $519 = 0, $52 = 0, $520 = 0, $521 = 0, $522 = 0, $523 = 0, $524 = 0, $525 = 0, $526 = 0, $527 = 0, $528 = 0, $529 = 0, $53 = 0;
 var $530 = 0, $531 = 0, $532 = 0, $533 = 0, $534 = 0, $535 = 0, $536 = 0, $537 = 0, $538 = 0, $539 = 0, $54 = 0, $540 = 0, $541 = 0, $542 = 0, $543 = 0, $544 = 0, $545 = 0, $546 = 0, $547 = 0, $548 = 0;
 var $549 = 0, $55 = 0, $550 = 0, $551 = 0, $552 = 0, $553 = 0, $554 = 0, $555 = 0, $556 = 0, $557 = 0, $558 = 0, $559 = 0, $56 = 0, $560 = 0, $561 = 0, $562 = 0, $563 = 0, $564 = 0, $565 = 0, $566 = 0;
 var $567 = 0, $568 = 0, $569 = 0, $57 = 0, $570 = 0, $571 = 0, $572 = 0, $573 = 0, $574 = 0, $575 = 0, $576 = 0, $577 = 0, $578 = 0, $579 = 0, $58 = 0, $580 = 0, $581 = 0, $582 = 0, $583 = 0, $584 = 0;
 var $585 = 0, $586 = 0, $587 = 0, $588 = 0, $589 = 0, $59 = 0, $590 = 0, $591 = 0, $592 = 0, $593 = 0, $594 = 0, $595 = 0, $596 = 0, $597 = 0, $598 = 0, $599 = 0, $6 = 0, $60 = 0, $600 = 0, $601 = 0;
 var $602 = 0, $603 = 0, $604 = 0, $605 = 0, $606 = 0, $607 = 0, $608 = 0, $609 = 0, $61 = 0, $610 = 0, $611 = 0, $612 = 0, $613 = 0, $614 = 0, $615 = 0, $616 = 0, $617 = 0, $618 = 0, $619 = 0, $62 = 0;
 var $620 = 0, $621 = 0, $622 = 0, $623 = 0, $624 = 0, $625 = 0, $626 = 0, $627 = 0, $628 = 0, $629 = 0, $63 = 0, $630 = 0, $631 = 0, $632 = 0, $633 = 0, $634 = 0, $635 = 0, $636 = 0, $637 = 0, $638 = 0;
 var $639 = 0, $64 = 0, $640 = 0, $641 = 0, $642 = 0, $643 = 0, $644 = 0, $645 = 0, $646 = 0, $647 = 0, $648 = 0, $649 = 0, $65 = 0, $650 = 0, $651 = 0, $652 = 0, $653 = 0, $654 = 0, $655 = 0, $656 = 0;
 var $657 = 0, $658 = 0, $659 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0;
 var $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0;
 var $c = 0, $db = 0, $eExtraOp = 0, $extraRight = 0, $i = 0, $idxNew = 0, $idxNew1 = 0, $idxNew13 = 0, $idxNew2 = 0, $idxNew4 = 0, $isComplete = 0, $noCase = 0, $op = 0, $opMask = 0, $pC = 0, $pDup = 0, $pExpr = 0, $pLeft = 0, $pLeft2 = 0, $pLeft6 = 0;
 var $pList = 0, $pMaskSet = 0, $pNew = 0, $pNewExpr = 0, $pNewExpr1 = 0, $pNewExpr2 = 0, $pNewExpr7 = 0, $pNewTerm = 0, $pParse = 0, $pRight = 0, $pRight5 = 0, $pStr1 = 0, $pStr2 = 0, $pTerm = 0, $pWInfo = 0, $prereqAll = 0, $prereqColumn = 0, $prereqExpr = 0, $prereqLeft = 0, $sCollSeqName = 0;
 var $x = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 208|0;
 $0 = sp + 188|0;
 $1 = sp + 184|0;
 $2 = sp + 180|0;
 $pWInfo = sp + 176|0;
 $pTerm = sp + 172|0;
 $pMaskSet = sp + 168|0;
 $pExpr = sp + 164|0;
 $prereqLeft = sp + 40|0;
 $prereqAll = sp + 32|0;
 $extraRight = sp + 24|0;
 $pStr1 = sp + 160|0;
 $isComplete = sp + 156|0;
 $noCase = sp + 152|0;
 $op = sp + 148|0;
 $pParse = sp + 144|0;
 $db = sp + 140|0;
 $x = sp + 16|0;
 $pLeft = sp + 136|0;
 $pRight = sp + 132|0;
 $opMask = sp + 194|0;
 $pNew = sp + 128|0;
 $pDup = sp + 124|0;
 $eExtraOp = sp + 192|0;
 $idxNew = sp + 120|0;
 $pList = sp + 116|0;
 $i = sp + 112|0;
 $pNewExpr = sp + 108|0;
 $idxNew1 = sp + 104|0;
 $pLeft2 = sp + 100|0;
 $pStr2 = sp + 96|0;
 $pNewExpr1 = sp + 92|0;
 $pNewExpr2 = sp + 88|0;
 $idxNew13 = sp + 84|0;
 $idxNew2 = sp + 80|0;
 $sCollSeqName = sp + 72|0;
 $c = sp + 196|0;
 $pC = sp + 68|0;
 $idxNew4 = sp + 64|0;
 $pRight5 = sp + 60|0;
 $pLeft6 = sp + 56|0;
 $pNewTerm = sp + 52|0;
 $prereqColumn = sp + 8|0;
 $prereqExpr = sp;
 $pNewExpr7 = sp + 48|0;
 HEAP32[$0>>2] = $pSrc;
 HEAP32[$1>>2] = $pWC;
 HEAP32[$2>>2] = $idxTerm;
 $3 = HEAP32[$1>>2]|0;
 $4 = HEAP32[$3>>2]|0;
 HEAP32[$pWInfo>>2] = $4;
 $5 = $extraRight;
 $6 = $5;
 HEAP32[$6>>2] = 0;
 $7 = (($5) + 4)|0;
 $8 = $7;
 HEAP32[$8>>2] = 0;
 HEAP32[$pStr1>>2] = 0;
 HEAP32[$isComplete>>2] = 0;
 HEAP32[$noCase>>2] = 0;
 $9 = HEAP32[$pWInfo>>2]|0;
 $10 = HEAP32[$9>>2]|0;
 HEAP32[$pParse>>2] = $10;
 $11 = HEAP32[$pParse>>2]|0;
 $12 = HEAP32[$11>>2]|0;
 HEAP32[$db>>2] = $12;
 $13 = HEAP32[$db>>2]|0;
 $14 = (($13) + 64|0);
 $15 = HEAP8[$14>>0]|0;
 $16 = ($15<<24>>24)!=(0);
 if ($16) {
  STACKTOP = sp;return;
 }
 $17 = HEAP32[$2>>2]|0;
 $18 = HEAP32[$1>>2]|0;
 $19 = (($18) + 20|0);
 $20 = HEAP32[$19>>2]|0;
 $21 = (($20) + (($17*48)|0)|0);
 HEAP32[$pTerm>>2] = $21;
 $22 = HEAP32[$pWInfo>>2]|0;
 $23 = (($22) + 68|0);
 HEAP32[$pMaskSet>>2] = $23;
 $24 = HEAP32[$pTerm>>2]|0;
 $25 = HEAP32[$24>>2]|0;
 HEAP32[$pExpr>>2] = $25;
 $26 = HEAP32[$pMaskSet>>2]|0;
 $27 = HEAP32[$pExpr>>2]|0;
 $28 = (($27) + 12|0);
 $29 = HEAP32[$28>>2]|0;
 $30 = (_exprTableUsage($26,$29)|0);
 $31 = tempRet0;
 $32 = $prereqLeft;
 $33 = $32;
 HEAP32[$33>>2] = $30;
 $34 = (($32) + 4)|0;
 $35 = $34;
 HEAP32[$35>>2] = $31;
 $36 = HEAP32[$pExpr>>2]|0;
 $37 = HEAP8[$36>>0]|0;
 $38 = $37&255;
 HEAP32[$op>>2] = $38;
 $39 = HEAP32[$op>>2]|0;
 $40 = ($39|0)==(75);
 do {
  if ($40) {
   $41 = HEAP32[$pExpr>>2]|0;
   $42 = (($41) + 4|0);
   $43 = HEAP32[$42>>2]|0;
   $44 = $43 & 2048;
   $45 = ($44|0)!=(0);
   $46 = HEAP32[$pMaskSet>>2]|0;
   $47 = HEAP32[$pExpr>>2]|0;
   $48 = (($47) + 20|0);
   if ($45) {
    $49 = HEAP32[$48>>2]|0;
    $50 = (_exprSelectTableUsage($46,$49)|0);
    $51 = tempRet0;
    $52 = HEAP32[$pTerm>>2]|0;
    $53 = (($52) + 32|0);
    $54 = $53;
    $55 = $54;
    HEAP32[$55>>2] = $50;
    $56 = (($54) + 4)|0;
    $57 = $56;
    HEAP32[$57>>2] = $51;
    break;
   } else {
    $58 = HEAP32[$48>>2]|0;
    $59 = (_exprListTableUsage($46,$58)|0);
    $60 = tempRet0;
    $61 = HEAP32[$pTerm>>2]|0;
    $62 = (($61) + 32|0);
    $63 = $62;
    $64 = $63;
    HEAP32[$64>>2] = $59;
    $65 = (($63) + 4)|0;
    $66 = $65;
    HEAP32[$66>>2] = $60;
    break;
   }
  } else {
   $67 = HEAP32[$op>>2]|0;
   $68 = ($67|0)==(76);
   if ($68) {
    $69 = HEAP32[$pTerm>>2]|0;
    $70 = (($69) + 32|0);
    $71 = $70;
    $72 = $71;
    HEAP32[$72>>2] = 0;
    $73 = (($71) + 4)|0;
    $74 = $73;
    HEAP32[$74>>2] = 0;
    break;
   } else {
    $75 = HEAP32[$pMaskSet>>2]|0;
    $76 = HEAP32[$pExpr>>2]|0;
    $77 = (($76) + 16|0);
    $78 = HEAP32[$77>>2]|0;
    $79 = (_exprTableUsage($75,$78)|0);
    $80 = tempRet0;
    $81 = HEAP32[$pTerm>>2]|0;
    $82 = (($81) + 32|0);
    $83 = $82;
    $84 = $83;
    HEAP32[$84>>2] = $79;
    $85 = (($83) + 4)|0;
    $86 = $85;
    HEAP32[$86>>2] = $80;
    break;
   }
  }
 } while(0);
 $87 = HEAP32[$pMaskSet>>2]|0;
 $88 = HEAP32[$pExpr>>2]|0;
 $89 = (_exprTableUsage($87,$88)|0);
 $90 = tempRet0;
 $91 = $prereqAll;
 $92 = $91;
 HEAP32[$92>>2] = $89;
 $93 = (($91) + 4)|0;
 $94 = $93;
 HEAP32[$94>>2] = $90;
 $95 = HEAP32[$pExpr>>2]|0;
 $96 = (($95) + 4|0);
 $97 = HEAP32[$96>>2]|0;
 $98 = $97 & 1;
 $99 = ($98|0)!=(0);
 if ($99) {
  $100 = HEAP32[$pMaskSet>>2]|0;
  $101 = HEAP32[$pExpr>>2]|0;
  $102 = (($101) + 36|0);
  $103 = HEAP16[$102>>1]|0;
  $104 = $103 << 16 >> 16;
  $105 = (_getMask($100,$104)|0);
  $106 = tempRet0;
  $107 = $x;
  $108 = $107;
  HEAP32[$108>>2] = $105;
  $109 = (($107) + 4)|0;
  $110 = $109;
  HEAP32[$110>>2] = $106;
  $111 = $x;
  $112 = $111;
  $113 = HEAP32[$112>>2]|0;
  $114 = (($111) + 4)|0;
  $115 = $114;
  $116 = HEAP32[$115>>2]|0;
  $117 = $prereqAll;
  $118 = $117;
  $119 = HEAP32[$118>>2]|0;
  $120 = (($117) + 4)|0;
  $121 = $120;
  $122 = HEAP32[$121>>2]|0;
  $123 = $119 | $113;
  $124 = $122 | $116;
  $125 = $prereqAll;
  $126 = $125;
  HEAP32[$126>>2] = $123;
  $127 = (($125) + 4)|0;
  $128 = $127;
  HEAP32[$128>>2] = $124;
  $129 = $x;
  $130 = $129;
  $131 = HEAP32[$130>>2]|0;
  $132 = (($129) + 4)|0;
  $133 = $132;
  $134 = HEAP32[$133>>2]|0;
  $135 = (_i64Subtract(($131|0),($134|0),1,0)|0);
  $136 = tempRet0;
  $137 = $extraRight;
  $138 = $137;
  HEAP32[$138>>2] = $135;
  $139 = (($137) + 4)|0;
  $140 = $139;
  HEAP32[$140>>2] = $136;
 }
 $141 = $prereqAll;
 $142 = $141;
 $143 = HEAP32[$142>>2]|0;
 $144 = (($141) + 4)|0;
 $145 = $144;
 $146 = HEAP32[$145>>2]|0;
 $147 = HEAP32[$pTerm>>2]|0;
 $148 = (($147) + 40|0);
 $149 = $148;
 $150 = $149;
 HEAP32[$150>>2] = $143;
 $151 = (($149) + 4)|0;
 $152 = $151;
 HEAP32[$152>>2] = $146;
 $153 = HEAP32[$pTerm>>2]|0;
 $154 = (($153) + 8|0);
 HEAP32[$154>>2] = -1;
 $155 = HEAP32[$pTerm>>2]|0;
 $156 = (($155) + 4|0);
 HEAP32[$156>>2] = -1;
 $157 = HEAP32[$pTerm>>2]|0;
 $158 = (($157) + 18|0);
 HEAP16[$158>>1] = 0;
 $159 = HEAP32[$op>>2]|0;
 $160 = (_allowedOp($159)|0);
 $161 = ($160|0)!=(0);
 $162 = HEAP32[$pExpr>>2]|0;
 do {
  if ($161) {
   $163 = (($162) + 12|0);
   $164 = HEAP32[$163>>2]|0;
   $165 = (_sqlite3ExprSkipCollate($164)|0);
   HEAP32[$pLeft>>2] = $165;
   $166 = HEAP32[$pExpr>>2]|0;
   $167 = (($166) + 16|0);
   $168 = HEAP32[$167>>2]|0;
   $169 = (_sqlite3ExprSkipCollate($168)|0);
   HEAP32[$pRight>>2] = $169;
   $170 = HEAP32[$pTerm>>2]|0;
   $171 = (($170) + 32|0);
   $172 = $171;
   $173 = $172;
   $174 = HEAP32[$173>>2]|0;
   $175 = (($172) + 4)|0;
   $176 = $175;
   $177 = HEAP32[$176>>2]|0;
   $178 = $prereqLeft;
   $179 = $178;
   $180 = HEAP32[$179>>2]|0;
   $181 = (($178) + 4)|0;
   $182 = $181;
   $183 = HEAP32[$182>>2]|0;
   $184 = $174 & $180;
   $185 = $177 & $183;
   $186 = ($184|0)==(0);
   $187 = ($185|0)==(0);
   $188 = $186 & $187;
   $189 = $188 ? 4095 : 1024;
   $190 = $189&65535;
   HEAP16[$opMask>>1] = $190;
   $191 = HEAP32[$pLeft>>2]|0;
   $192 = HEAP8[$191>>0]|0;
   $193 = $192&255;
   $194 = ($193|0)==(154);
   if ($194) {
    $195 = HEAP32[$pLeft>>2]|0;
    $196 = (($195) + 28|0);
    $197 = HEAP32[$196>>2]|0;
    $198 = HEAP32[$pTerm>>2]|0;
    $199 = (($198) + 8|0);
    HEAP32[$199>>2] = $197;
    $200 = HEAP32[$pLeft>>2]|0;
    $201 = (($200) + 32|0);
    $202 = HEAP16[$201>>1]|0;
    $203 = $202 << 16 >> 16;
    $204 = HEAP32[$pTerm>>2]|0;
    $205 = (($204) + 12|0);
    HEAP32[$205>>2] = $203;
    $206 = HEAP32[$op>>2]|0;
    $207 = (_operatorMask($206)|0);
    $208 = $207&65535;
    $209 = HEAP16[$opMask>>1]|0;
    $210 = $209&65535;
    $211 = $208 & $210;
    $212 = $211&65535;
    $213 = HEAP32[$pTerm>>2]|0;
    $214 = (($213) + 18|0);
    HEAP16[$214>>1] = $212;
   }
   $215 = HEAP32[$pRight>>2]|0;
   $216 = ($215|0)!=(0|0);
   if ($216) {
    $217 = HEAP32[$pRight>>2]|0;
    $218 = HEAP8[$217>>0]|0;
    $219 = $218&255;
    $220 = ($219|0)==(154);
    if ($220) {
     HEAP16[$eExtraOp>>1] = 0;
     $221 = HEAP32[$pTerm>>2]|0;
     $222 = (($221) + 8|0);
     $223 = HEAP32[$222>>2]|0;
     $224 = ($223|0)>=(0);
     if ($224) {
      $225 = HEAP32[$db>>2]|0;
      $226 = HEAP32[$pExpr>>2]|0;
      $227 = (_sqlite3ExprDup($225,$226,0)|0);
      HEAP32[$pDup>>2] = $227;
      $228 = HEAP32[$db>>2]|0;
      $229 = (($228) + 64|0);
      $230 = HEAP8[$229>>0]|0;
      $231 = ($230<<24>>24)!=(0);
      if ($231) {
       $232 = HEAP32[$db>>2]|0;
       $233 = HEAP32[$pDup>>2]|0;
       _sqlite3ExprDelete($232,$233);
       STACKTOP = sp;return;
      }
      $234 = HEAP32[$1>>2]|0;
      $235 = HEAP32[$pDup>>2]|0;
      $236 = (_whereClauseInsert($234,$235,3)|0);
      HEAP32[$idxNew>>2] = $236;
      $237 = HEAP32[$idxNew>>2]|0;
      $238 = ($237|0)==(0);
      if ($238) {
       STACKTOP = sp;return;
      }
      $239 = HEAP32[$idxNew>>2]|0;
      $240 = HEAP32[$1>>2]|0;
      $241 = (($240) + 20|0);
      $242 = HEAP32[$241>>2]|0;
      $243 = (($242) + (($239*48)|0)|0);
      HEAP32[$pNew>>2] = $243;
      $244 = HEAP32[$2>>2]|0;
      $245 = HEAP32[$pNew>>2]|0;
      $246 = (($245) + 4|0);
      HEAP32[$246>>2] = $244;
      $247 = HEAP32[$2>>2]|0;
      $248 = HEAP32[$1>>2]|0;
      $249 = (($248) + 20|0);
      $250 = HEAP32[$249>>2]|0;
      $251 = (($250) + (($247*48)|0)|0);
      HEAP32[$pTerm>>2] = $251;
      $252 = HEAP32[$pTerm>>2]|0;
      $253 = (($252) + 21|0);
      HEAP8[$253>>0] = 1;
      $254 = HEAP32[$pTerm>>2]|0;
      $255 = (($254) + 20|0);
      $256 = HEAP8[$255>>0]|0;
      $257 = $256&255;
      $258 = $257 | 8;
      $259 = $258&255;
      HEAP8[$255>>0] = $259;
      $260 = HEAP32[$pExpr>>2]|0;
      $261 = HEAP8[$260>>0]|0;
      $262 = $261&255;
      $263 = ($262|0)==(79);
      if ($263) {
       $264 = HEAP32[$pExpr>>2]|0;
       $265 = (($264) + 4|0);
       $266 = HEAP32[$265>>2]|0;
       $267 = $266 & 1;
       $268 = ($267|0)!=(0);
       if (!($268)) {
        $269 = HEAP32[$db>>2]|0;
        $270 = (($269) + 60|0);
        $271 = HEAP16[$270>>1]|0;
        $272 = $271&65535;
        $273 = $272 & 512;
        $274 = ($273|0)==(0);
        if ($274) {
         $275 = HEAP32[$pTerm>>2]|0;
         $276 = (($275) + 18|0);
         $277 = HEAP16[$276>>1]|0;
         $278 = $277&65535;
         $279 = $278 | 1024;
         $280 = $279&65535;
         HEAP16[$276>>1] = $280;
         HEAP16[$eExtraOp>>1] = 1024;
        }
       }
      }
     } else {
      $281 = HEAP32[$pExpr>>2]|0;
      HEAP32[$pDup>>2] = $281;
      $282 = HEAP32[$pTerm>>2]|0;
      HEAP32[$pNew>>2] = $282;
     }
     $283 = HEAP32[$pParse>>2]|0;
     $284 = HEAP32[$pDup>>2]|0;
     _exprCommute($283,$284);
     $285 = HEAP32[$pDup>>2]|0;
     $286 = (($285) + 12|0);
     $287 = HEAP32[$286>>2]|0;
     $288 = (_sqlite3ExprSkipCollate($287)|0);
     HEAP32[$pLeft>>2] = $288;
     $289 = HEAP32[$pLeft>>2]|0;
     $290 = (($289) + 28|0);
     $291 = HEAP32[$290>>2]|0;
     $292 = HEAP32[$pNew>>2]|0;
     $293 = (($292) + 8|0);
     HEAP32[$293>>2] = $291;
     $294 = HEAP32[$pLeft>>2]|0;
     $295 = (($294) + 32|0);
     $296 = HEAP16[$295>>1]|0;
     $297 = $296 << 16 >> 16;
     $298 = HEAP32[$pNew>>2]|0;
     $299 = (($298) + 12|0);
     HEAP32[$299>>2] = $297;
     $300 = $prereqLeft;
     $301 = $300;
     $302 = HEAP32[$301>>2]|0;
     $303 = (($300) + 4)|0;
     $304 = $303;
     $305 = HEAP32[$304>>2]|0;
     $306 = $extraRight;
     $307 = $306;
     $308 = HEAP32[$307>>2]|0;
     $309 = (($306) + 4)|0;
     $310 = $309;
     $311 = HEAP32[$310>>2]|0;
     $312 = $302 | $308;
     $313 = $305 | $311;
     $314 = HEAP32[$pNew>>2]|0;
     $315 = (($314) + 32|0);
     $316 = $315;
     $317 = $316;
     HEAP32[$317>>2] = $312;
     $318 = (($316) + 4)|0;
     $319 = $318;
     HEAP32[$319>>2] = $313;
     $320 = $prereqAll;
     $321 = $320;
     $322 = HEAP32[$321>>2]|0;
     $323 = (($320) + 4)|0;
     $324 = $323;
     $325 = HEAP32[$324>>2]|0;
     $326 = HEAP32[$pNew>>2]|0;
     $327 = (($326) + 40|0);
     $328 = $327;
     $329 = $328;
     HEAP32[$329>>2] = $322;
     $330 = (($328) + 4)|0;
     $331 = $330;
     HEAP32[$331>>2] = $325;
     $332 = HEAP32[$pDup>>2]|0;
     $333 = HEAP8[$332>>0]|0;
     $334 = $333&255;
     $335 = (_operatorMask($334)|0);
     $336 = $335&65535;
     $337 = HEAP16[$eExtraOp>>1]|0;
     $338 = $337&65535;
     $339 = (($336) + ($338))|0;
     $340 = HEAP16[$opMask>>1]|0;
     $341 = $340&65535;
     $342 = $339 & $341;
     $343 = $342&65535;
     $344 = HEAP32[$pNew>>2]|0;
     $345 = (($344) + 18|0);
     HEAP16[$345>>1] = $343;
    }
   }
  } else {
   $346 = HEAP8[$162>>0]|0;
   $347 = $346&255;
   $348 = ($347|0)==(74);
   if ($348) {
    $349 = HEAP32[$1>>2]|0;
    $350 = (($349) + 8|0);
    $351 = HEAP8[$350>>0]|0;
    $352 = $351&255;
    $353 = ($352|0)==(72);
    if ($353) {
     $354 = HEAP32[$pExpr>>2]|0;
     $355 = (($354) + 20|0);
     $356 = HEAP32[$355>>2]|0;
     HEAP32[$pList>>2] = $356;
     HEAP32[$i>>2] = 0;
     while(1) {
      $357 = HEAP32[$i>>2]|0;
      $358 = ($357|0)<(2);
      if (!($358)) {
       break;
      }
      $359 = HEAP32[$pParse>>2]|0;
      $360 = HEAP32[$i>>2]|0;
      $361 = (24168 + ($360)|0);
      $362 = HEAP8[$361>>0]|0;
      $363 = $362&255;
      $364 = HEAP32[$db>>2]|0;
      $365 = HEAP32[$pExpr>>2]|0;
      $366 = (($365) + 12|0);
      $367 = HEAP32[$366>>2]|0;
      $368 = (_sqlite3ExprDup($364,$367,0)|0);
      $369 = HEAP32[$db>>2]|0;
      $370 = HEAP32[$i>>2]|0;
      $371 = HEAP32[$pList>>2]|0;
      $372 = (($371) + 4|0);
      $373 = HEAP32[$372>>2]|0;
      $374 = (($373) + (($370*20)|0)|0);
      $375 = HEAP32[$374>>2]|0;
      $376 = (_sqlite3ExprDup($369,$375,0)|0);
      $377 = (_sqlite3PExpr($359,$363,$368,$376,0)|0);
      HEAP32[$pNewExpr>>2] = $377;
      $378 = HEAP32[$pNewExpr>>2]|0;
      $379 = HEAP32[$pExpr>>2]|0;
      _transferJoinMarkings($378,$379);
      $380 = HEAP32[$1>>2]|0;
      $381 = HEAP32[$pNewExpr>>2]|0;
      $382 = (_whereClauseInsert($380,$381,3)|0);
      HEAP32[$idxNew1>>2] = $382;
      $383 = HEAP32[$0>>2]|0;
      $384 = HEAP32[$1>>2]|0;
      $385 = HEAP32[$idxNew1>>2]|0;
      _exprAnalyze($383,$384,$385);
      $386 = HEAP32[$2>>2]|0;
      $387 = HEAP32[$1>>2]|0;
      $388 = (($387) + 20|0);
      $389 = HEAP32[$388>>2]|0;
      $390 = (($389) + (($386*48)|0)|0);
      HEAP32[$pTerm>>2] = $390;
      $391 = HEAP32[$2>>2]|0;
      $392 = HEAP32[$idxNew1>>2]|0;
      $393 = HEAP32[$1>>2]|0;
      $394 = (($393) + 20|0);
      $395 = HEAP32[$394>>2]|0;
      $396 = (($395) + (($392*48)|0)|0);
      $397 = (($396) + 4|0);
      HEAP32[$397>>2] = $391;
      $398 = HEAP32[$i>>2]|0;
      $399 = (($398) + 1)|0;
      HEAP32[$i>>2] = $399;
     }
     $400 = HEAP32[$pTerm>>2]|0;
     $401 = (($400) + 21|0);
     HEAP8[$401>>0] = 2;
     break;
    }
   }
   $402 = HEAP32[$pExpr>>2]|0;
   $403 = HEAP8[$402>>0]|0;
   $404 = $403&255;
   $405 = ($404|0)==(71);
   if ($405) {
    $406 = HEAP32[$0>>2]|0;
    $407 = HEAP32[$1>>2]|0;
    $408 = HEAP32[$2>>2]|0;
    _exprAnalyzeOrTerm($406,$407,$408);
    $409 = HEAP32[$2>>2]|0;
    $410 = HEAP32[$1>>2]|0;
    $411 = (($410) + 20|0);
    $412 = HEAP32[$411>>2]|0;
    $413 = (($412) + (($409*48)|0)|0);
    HEAP32[$pTerm>>2] = $413;
   }
  }
 } while(0);
 $414 = HEAP32[$1>>2]|0;
 $415 = (($414) + 8|0);
 $416 = HEAP8[$415>>0]|0;
 $417 = $416&255;
 $418 = ($417|0)==(72);
 if ($418) {
  $419 = HEAP32[$pParse>>2]|0;
  $420 = HEAP32[$pExpr>>2]|0;
  $421 = (_isLikeOrGlob($419,$420,$pStr1,$isComplete,$noCase)|0);
  $422 = ($421|0)!=(0);
  if ($422) {
   $423 = HEAP32[$pExpr>>2]|0;
   $424 = (($423) + 20|0);
   $425 = HEAP32[$424>>2]|0;
   $426 = (($425) + 4|0);
   $427 = HEAP32[$426>>2]|0;
   $428 = (($427) + 20|0);
   $429 = HEAP32[$428>>2]|0;
   HEAP32[$pLeft2>>2] = $429;
   $430 = HEAP32[$db>>2]|0;
   $431 = HEAP32[$pStr1>>2]|0;
   $432 = (_sqlite3ExprDup($430,$431,0)|0);
   HEAP32[$pStr2>>2] = $432;
   $433 = HEAP32[$db>>2]|0;
   $434 = (($433) + 64|0);
   $435 = HEAP8[$434>>0]|0;
   $436 = ($435<<24>>24)!=(0);
   if (!($436)) {
    $437 = HEAP32[$pStr2>>2]|0;
    $438 = (($437) + 8|0);
    $439 = HEAP32[$438>>2]|0;
    $440 = (_sqlite3Strlen30($439)|0);
    $441 = (($440) - 1)|0;
    $442 = HEAP32[$pStr2>>2]|0;
    $443 = (($442) + 8|0);
    $444 = HEAP32[$443>>2]|0;
    $445 = (($444) + ($441)|0);
    HEAP32[$pC>>2] = $445;
    $446 = HEAP32[$pC>>2]|0;
    $447 = HEAP8[$446>>0]|0;
    HEAP8[$c>>0] = $447;
    $448 = HEAP32[$noCase>>2]|0;
    $449 = ($448|0)!=(0);
    if ($449) {
     $450 = HEAP8[$c>>0]|0;
     $451 = $450&255;
     $452 = ($451|0)==(64);
     if ($452) {
      HEAP32[$isComplete>>2] = 0;
     }
     $453 = HEAP8[$c>>0]|0;
     $454 = $453&255;
     $455 = (440 + ($454)|0);
     $456 = HEAP8[$455>>0]|0;
     HEAP8[$c>>0] = $456;
    }
    $457 = HEAP8[$c>>0]|0;
    $458 = $457&255;
    $459 = (($458) + 1)|0;
    $460 = $459&255;
    $461 = HEAP32[$pC>>2]|0;
    HEAP8[$461>>0] = $460;
   }
   $462 = HEAP32[$noCase>>2]|0;
   $463 = ($462|0)!=(0);
   $464 = $463 ? 3336 : 3320;
   HEAP32[$sCollSeqName>>2] = $464;
   $465 = (($sCollSeqName) + 4|0);
   HEAP32[$465>>2] = 6;
   $466 = HEAP32[$db>>2]|0;
   $467 = HEAP32[$pLeft2>>2]|0;
   $468 = (_sqlite3ExprDup($466,$467,0)|0);
   HEAP32[$pNewExpr1>>2] = $468;
   $469 = HEAP32[$pParse>>2]|0;
   $470 = HEAP32[$pParse>>2]|0;
   $471 = HEAP32[$pNewExpr1>>2]|0;
   $472 = (_sqlite3ExprAddCollateToken($470,$471,$sCollSeqName)|0);
   $473 = HEAP32[$pStr1>>2]|0;
   $474 = (_sqlite3PExpr($469,83,$472,$473,0)|0);
   HEAP32[$pNewExpr1>>2] = $474;
   $475 = HEAP32[$pNewExpr1>>2]|0;
   $476 = HEAP32[$pExpr>>2]|0;
   _transferJoinMarkings($475,$476);
   $477 = HEAP32[$1>>2]|0;
   $478 = HEAP32[$pNewExpr1>>2]|0;
   $479 = (_whereClauseInsert($477,$478,3)|0);
   HEAP32[$idxNew13>>2] = $479;
   $480 = HEAP32[$0>>2]|0;
   $481 = HEAP32[$1>>2]|0;
   $482 = HEAP32[$idxNew13>>2]|0;
   _exprAnalyze($480,$481,$482);
   $483 = HEAP32[$db>>2]|0;
   $484 = HEAP32[$pLeft2>>2]|0;
   $485 = (_sqlite3ExprDup($483,$484,0)|0);
   HEAP32[$pNewExpr2>>2] = $485;
   $486 = HEAP32[$pParse>>2]|0;
   $487 = HEAP32[$pParse>>2]|0;
   $488 = HEAP32[$pNewExpr2>>2]|0;
   $489 = (_sqlite3ExprAddCollateToken($487,$488,$sCollSeqName)|0);
   $490 = HEAP32[$pStr2>>2]|0;
   $491 = (_sqlite3PExpr($486,82,$489,$490,0)|0);
   HEAP32[$pNewExpr2>>2] = $491;
   $492 = HEAP32[$pNewExpr2>>2]|0;
   $493 = HEAP32[$pExpr>>2]|0;
   _transferJoinMarkings($492,$493);
   $494 = HEAP32[$1>>2]|0;
   $495 = HEAP32[$pNewExpr2>>2]|0;
   $496 = (_whereClauseInsert($494,$495,3)|0);
   HEAP32[$idxNew2>>2] = $496;
   $497 = HEAP32[$0>>2]|0;
   $498 = HEAP32[$1>>2]|0;
   $499 = HEAP32[$idxNew2>>2]|0;
   _exprAnalyze($497,$498,$499);
   $500 = HEAP32[$2>>2]|0;
   $501 = HEAP32[$1>>2]|0;
   $502 = (($501) + 20|0);
   $503 = HEAP32[$502>>2]|0;
   $504 = (($503) + (($500*48)|0)|0);
   HEAP32[$pTerm>>2] = $504;
   $505 = HEAP32[$isComplete>>2]|0;
   $506 = ($505|0)!=(0);
   if ($506) {
    $507 = HEAP32[$2>>2]|0;
    $508 = HEAP32[$idxNew13>>2]|0;
    $509 = HEAP32[$1>>2]|0;
    $510 = (($509) + 20|0);
    $511 = HEAP32[$510>>2]|0;
    $512 = (($511) + (($508*48)|0)|0);
    $513 = (($512) + 4|0);
    HEAP32[$513>>2] = $507;
    $514 = HEAP32[$2>>2]|0;
    $515 = HEAP32[$idxNew2>>2]|0;
    $516 = HEAP32[$1>>2]|0;
    $517 = (($516) + 20|0);
    $518 = HEAP32[$517>>2]|0;
    $519 = (($518) + (($515*48)|0)|0);
    $520 = (($519) + 4|0);
    HEAP32[$520>>2] = $514;
    $521 = HEAP32[$pTerm>>2]|0;
    $522 = (($521) + 21|0);
    HEAP8[$522>>0] = 2;
   }
  }
 }
 $523 = HEAP32[$pExpr>>2]|0;
 $524 = (_isMatchOfColumn($523)|0);
 $525 = ($524|0)!=(0);
 if ($525) {
  $526 = HEAP32[$pExpr>>2]|0;
  $527 = (($526) + 20|0);
  $528 = HEAP32[$527>>2]|0;
  $529 = (($528) + 4|0);
  $530 = HEAP32[$529>>2]|0;
  $531 = HEAP32[$530>>2]|0;
  HEAP32[$pRight5>>2] = $531;
  $532 = HEAP32[$pExpr>>2]|0;
  $533 = (($532) + 20|0);
  $534 = HEAP32[$533>>2]|0;
  $535 = (($534) + 4|0);
  $536 = HEAP32[$535>>2]|0;
  $537 = (($536) + 20|0);
  $538 = HEAP32[$537>>2]|0;
  HEAP32[$pLeft6>>2] = $538;
  $539 = HEAP32[$pMaskSet>>2]|0;
  $540 = HEAP32[$pRight5>>2]|0;
  $541 = (_exprTableUsage($539,$540)|0);
  $542 = tempRet0;
  $543 = $prereqExpr;
  $544 = $543;
  HEAP32[$544>>2] = $541;
  $545 = (($543) + 4)|0;
  $546 = $545;
  HEAP32[$546>>2] = $542;
  $547 = HEAP32[$pMaskSet>>2]|0;
  $548 = HEAP32[$pLeft6>>2]|0;
  $549 = (_exprTableUsage($547,$548)|0);
  $550 = tempRet0;
  $551 = $prereqColumn;
  $552 = $551;
  HEAP32[$552>>2] = $549;
  $553 = (($551) + 4)|0;
  $554 = $553;
  HEAP32[$554>>2] = $550;
  $555 = $prereqExpr;
  $556 = $555;
  $557 = HEAP32[$556>>2]|0;
  $558 = (($555) + 4)|0;
  $559 = $558;
  $560 = HEAP32[$559>>2]|0;
  $561 = $prereqColumn;
  $562 = $561;
  $563 = HEAP32[$562>>2]|0;
  $564 = (($561) + 4)|0;
  $565 = $564;
  $566 = HEAP32[$565>>2]|0;
  $567 = $557 & $563;
  $568 = $560 & $566;
  $569 = ($567|0)==(0);
  $570 = ($568|0)==(0);
  $571 = $569 & $570;
  if ($571) {
   $572 = HEAP32[$pParse>>2]|0;
   $573 = HEAP32[$db>>2]|0;
   $574 = HEAP32[$pRight5>>2]|0;
   $575 = (_sqlite3ExprDup($573,$574,0)|0);
   $576 = (_sqlite3PExpr($572,51,0,$575,0)|0);
   HEAP32[$pNewExpr7>>2] = $576;
   $577 = HEAP32[$1>>2]|0;
   $578 = HEAP32[$pNewExpr7>>2]|0;
   $579 = (_whereClauseInsert($577,$578,3)|0);
   HEAP32[$idxNew4>>2] = $579;
   $580 = HEAP32[$idxNew4>>2]|0;
   $581 = HEAP32[$1>>2]|0;
   $582 = (($581) + 20|0);
   $583 = HEAP32[$582>>2]|0;
   $584 = (($583) + (($580*48)|0)|0);
   HEAP32[$pNewTerm>>2] = $584;
   $585 = $prereqExpr;
   $586 = $585;
   $587 = HEAP32[$586>>2]|0;
   $588 = (($585) + 4)|0;
   $589 = $588;
   $590 = HEAP32[$589>>2]|0;
   $591 = HEAP32[$pNewTerm>>2]|0;
   $592 = (($591) + 32|0);
   $593 = $592;
   $594 = $593;
   HEAP32[$594>>2] = $587;
   $595 = (($593) + 4)|0;
   $596 = $595;
   HEAP32[$596>>2] = $590;
   $597 = HEAP32[$pLeft6>>2]|0;
   $598 = (($597) + 28|0);
   $599 = HEAP32[$598>>2]|0;
   $600 = HEAP32[$pNewTerm>>2]|0;
   $601 = (($600) + 8|0);
   HEAP32[$601>>2] = $599;
   $602 = HEAP32[$pLeft6>>2]|0;
   $603 = (($602) + 32|0);
   $604 = HEAP16[$603>>1]|0;
   $605 = $604 << 16 >> 16;
   $606 = HEAP32[$pNewTerm>>2]|0;
   $607 = (($606) + 12|0);
   HEAP32[$607>>2] = $605;
   $608 = HEAP32[$pNewTerm>>2]|0;
   $609 = (($608) + 18|0);
   HEAP16[$609>>1] = 64;
   $610 = HEAP32[$2>>2]|0;
   $611 = HEAP32[$pNewTerm>>2]|0;
   $612 = (($611) + 4|0);
   HEAP32[$612>>2] = $610;
   $613 = HEAP32[$2>>2]|0;
   $614 = HEAP32[$1>>2]|0;
   $615 = (($614) + 20|0);
   $616 = HEAP32[$615>>2]|0;
   $617 = (($616) + (($613*48)|0)|0);
   HEAP32[$pTerm>>2] = $617;
   $618 = HEAP32[$pTerm>>2]|0;
   $619 = (($618) + 21|0);
   HEAP8[$619>>0] = 1;
   $620 = HEAP32[$pTerm>>2]|0;
   $621 = (($620) + 20|0);
   $622 = HEAP8[$621>>0]|0;
   $623 = $622&255;
   $624 = $623 | 8;
   $625 = $624&255;
   HEAP8[$621>>0] = $625;
   $626 = HEAP32[$pTerm>>2]|0;
   $627 = (($626) + 40|0);
   $628 = $627;
   $629 = $628;
   $630 = HEAP32[$629>>2]|0;
   $631 = (($628) + 4)|0;
   $632 = $631;
   $633 = HEAP32[$632>>2]|0;
   $634 = HEAP32[$pNewTerm>>2]|0;
   $635 = (($634) + 40|0);
   $636 = $635;
   $637 = $636;
   HEAP32[$637>>2] = $630;
   $638 = (($636) + 4)|0;
   $639 = $638;
   HEAP32[$639>>2] = $633;
  }
 }
 $640 = $extraRight;
 $641 = $640;
 $642 = HEAP32[$641>>2]|0;
 $643 = (($640) + 4)|0;
 $644 = $643;
 $645 = HEAP32[$644>>2]|0;
 $646 = HEAP32[$pTerm>>2]|0;
 $647 = (($646) + 32|0);
 $648 = $647;
 $649 = $648;
 $650 = HEAP32[$649>>2]|0;
 $651 = (($648) + 4)|0;
 $652 = $651;
 $653 = HEAP32[$652>>2]|0;
 $654 = $650 | $642;
 $655 = $653 | $645;
 $656 = $647;
 $657 = $656;
 HEAP32[$657>>2] = $654;
 $658 = (($656) + 4)|0;
 $659 = $658;
 HEAP32[$659>>2] = $655;
 STACKTOP = sp;return;
}
function _allowedOp($op) {
 $op = $op|0;
 var $0 = 0, $1 = 0, $10 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $or$cond = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 HEAP32[$0>>2] = $op;
 $1 = HEAP32[$0>>2]|0;
 $2 = ($1|0)==(75);
 if ($2) {
  $10 = 1;
 } else {
  $3 = HEAP32[$0>>2]|0;
  $4 = ($3|0)>=(79);
  $5 = HEAP32[$0>>2]|0;
  $6 = ($5|0)<=(83);
  $or$cond = $4 & $6;
  if ($or$cond) {
   $10 = 1;
  } else {
   $7 = HEAP32[$0>>2]|0;
   $8 = ($7|0)==(76);
   $10 = $8;
  }
 }
 $9 = $10&1;
 STACKTOP = sp;return ($9|0);
}
function _operatorMask($op) {
 $op = $op|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $c = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 $c = sp + 4|0;
 HEAP32[$0>>2] = $op;
 $1 = HEAP32[$0>>2]|0;
 $2 = ($1|0)==(75);
 do {
  if ($2) {
   HEAP16[$c>>1] = 1;
  } else {
   $3 = HEAP32[$0>>2]|0;
   $4 = ($3|0)==(76);
   if ($4) {
    HEAP16[$c>>1] = 128;
    break;
   } else {
    $5 = HEAP32[$0>>2]|0;
    $6 = (($5) - 79)|0;
    $7 = 2 << $6;
    $8 = $7&65535;
    HEAP16[$c>>1] = $8;
    break;
   }
  }
 } while(0);
 $9 = HEAP16[$c>>1]|0;
 STACKTOP = sp;return ($9|0);
}
function _whereClauseInsert($pWC,$p,$wtFlags) {
 $pWC = $pWC|0;
 $p = $p|0;
 $wtFlags = $wtFlags|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0;
 var $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0;
 var $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0;
 var $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0;
 var $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0;
 var $97 = 0, $98 = 0, $99 = 0, $db = 0, $idx = 0, $pOld = 0, $pTerm = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 24|0;
 $1 = sp + 20|0;
 $2 = sp + 16|0;
 $3 = sp + 28|0;
 $pTerm = sp + 12|0;
 $idx = sp + 8|0;
 $pOld = sp + 4|0;
 $db = sp;
 HEAP32[$1>>2] = $pWC;
 HEAP32[$2>>2] = $p;
 HEAP8[$3>>0] = $wtFlags;
 $4 = HEAP32[$1>>2]|0;
 $5 = (($4) + 12|0);
 $6 = HEAP32[$5>>2]|0;
 $7 = HEAP32[$1>>2]|0;
 $8 = (($7) + 16|0);
 $9 = HEAP32[$8>>2]|0;
 $10 = ($6|0)>=($9|0);
 do {
  if ($10) {
   $11 = HEAP32[$1>>2]|0;
   $12 = (($11) + 20|0);
   $13 = HEAP32[$12>>2]|0;
   HEAP32[$pOld>>2] = $13;
   $14 = HEAP32[$1>>2]|0;
   $15 = HEAP32[$14>>2]|0;
   $16 = HEAP32[$15>>2]|0;
   $17 = HEAP32[$16>>2]|0;
   HEAP32[$db>>2] = $17;
   $18 = HEAP32[$db>>2]|0;
   $19 = HEAP32[$1>>2]|0;
   $20 = (($19) + 16|0);
   $21 = HEAP32[$20>>2]|0;
   $22 = ($21*48)|0;
   $23 = $22<<1;
   $24 = (_sqlite3DbMallocRaw($18,$23,0)|0);
   $25 = HEAP32[$1>>2]|0;
   $26 = (($25) + 20|0);
   HEAP32[$26>>2] = $24;
   $27 = HEAP32[$1>>2]|0;
   $28 = (($27) + 20|0);
   $29 = HEAP32[$28>>2]|0;
   $30 = ($29|0)==(0|0);
   if (!($30)) {
    $40 = HEAP32[$1>>2]|0;
    $41 = (($40) + 20|0);
    $42 = HEAP32[$41>>2]|0;
    $43 = HEAP32[$pOld>>2]|0;
    $44 = HEAP32[$1>>2]|0;
    $45 = (($44) + 12|0);
    $46 = HEAP32[$45>>2]|0;
    $47 = ($46*48)|0;
    _memcpy(($42|0),($43|0),($47|0))|0;
    $48 = HEAP32[$pOld>>2]|0;
    $49 = HEAP32[$1>>2]|0;
    $50 = (($49) + 24|0);
    $51 = ($48|0)!=($50|0);
    if ($51) {
     $52 = HEAP32[$db>>2]|0;
     $53 = HEAP32[$pOld>>2]|0;
     _sqlite3DbFree($52,$53);
    }
    $54 = HEAP32[$db>>2]|0;
    $55 = HEAP32[$1>>2]|0;
    $56 = (($55) + 20|0);
    $57 = HEAP32[$56>>2]|0;
    $58 = (_sqlite3DbMallocSize($54,$57)|0);
    $59 = (($58>>>0) / 48)&-1;
    $60 = HEAP32[$1>>2]|0;
    $61 = (($60) + 16|0);
    HEAP32[$61>>2] = $59;
    break;
   }
   $31 = HEAP8[$3>>0]|0;
   $32 = $31&255;
   $33 = $32 & 1;
   $34 = ($33|0)!=(0);
   if ($34) {
    $35 = HEAP32[$db>>2]|0;
    $36 = HEAP32[$2>>2]|0;
    _sqlite3ExprDelete($35,$36);
   }
   $37 = HEAP32[$pOld>>2]|0;
   $38 = HEAP32[$1>>2]|0;
   $39 = (($38) + 20|0);
   HEAP32[$39>>2] = $37;
   HEAP32[$0>>2] = 0;
   $102 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($102|0);
  }
 } while(0);
 $62 = HEAP32[$1>>2]|0;
 $63 = (($62) + 12|0);
 $64 = HEAP32[$63>>2]|0;
 $65 = (($64) + 1)|0;
 HEAP32[$63>>2] = $65;
 HEAP32[$idx>>2] = $64;
 $66 = HEAP32[$1>>2]|0;
 $67 = (($66) + 20|0);
 $68 = HEAP32[$67>>2]|0;
 $69 = (($68) + (($64*48)|0)|0);
 HEAP32[$pTerm>>2] = $69;
 $70 = HEAP32[$2>>2]|0;
 $71 = ($70|0)!=(0|0);
 if ($71) {
  $72 = HEAP32[$2>>2]|0;
  $73 = (($72) + 4|0);
  $74 = HEAP32[$73>>2]|0;
  $75 = $74 & 262144;
  $76 = ($75|0)!=(0);
  if ($76) {
   $77 = HEAP32[$2>>2]|0;
   $78 = (($77) + 28|0);
   $79 = HEAP32[$78>>2]|0;
   $80 = ($79|0)<(0);
   $81 = $80 << 31 >> 31;
   $82 = (_sqlite3LogEst($79,$81)|0);
   $83 = $82 << 16 >> 16;
   $84 = (($83) - 99)|0;
   $85 = $84&65535;
   $86 = HEAP32[$pTerm>>2]|0;
   $87 = (($86) + 16|0);
   HEAP16[$87>>1] = $85;
  } else {
   label = 12;
  }
 } else {
  label = 12;
 }
 if ((label|0) == 12) {
  $88 = HEAP32[$pTerm>>2]|0;
  $89 = (($88) + 16|0);
  HEAP16[$89>>1] = 1;
 }
 $90 = HEAP32[$2>>2]|0;
 $91 = (_sqlite3ExprSkipCollate($90)|0);
 $92 = HEAP32[$pTerm>>2]|0;
 HEAP32[$92>>2] = $91;
 $93 = HEAP8[$3>>0]|0;
 $94 = HEAP32[$pTerm>>2]|0;
 $95 = (($94) + 20|0);
 HEAP8[$95>>0] = $93;
 $96 = HEAP32[$1>>2]|0;
 $97 = HEAP32[$pTerm>>2]|0;
 $98 = (($97) + 24|0);
 HEAP32[$98>>2] = $96;
 $99 = HEAP32[$pTerm>>2]|0;
 $100 = (($99) + 4|0);
 HEAP32[$100>>2] = -1;
 $101 = HEAP32[$idx>>2]|0;
 HEAP32[$0>>2] = $101;
 $102 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($102|0);
}
function _exprCommute($pParse,$pExpr) {
 $pParse = $pParse|0;
 $pExpr = $pExpr|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $7 = 0, $8 = 0, $9 = 0, $expLeft = 0, $expRight = 0, $t = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $1 = sp + 4|0;
 $expRight = sp + 14|0;
 $expLeft = sp + 12|0;
 $t = sp;
 HEAP32[$0>>2] = $pParse;
 HEAP32[$1>>2] = $pExpr;
 $2 = HEAP32[$1>>2]|0;
 $3 = (($2) + 16|0);
 $4 = HEAP32[$3>>2]|0;
 $5 = (($4) + 4|0);
 $6 = HEAP32[$5>>2]|0;
 $7 = $6 & 256;
 $8 = $7&65535;
 HEAP16[$expRight>>1] = $8;
 $9 = HEAP32[$1>>2]|0;
 $10 = (($9) + 12|0);
 $11 = HEAP32[$10>>2]|0;
 $12 = (($11) + 4|0);
 $13 = HEAP32[$12>>2]|0;
 $14 = $13 & 256;
 $15 = $14&65535;
 HEAP16[$expLeft>>1] = $15;
 $16 = HEAP16[$expRight>>1]|0;
 $17 = $16&65535;
 $18 = HEAP16[$expLeft>>1]|0;
 $19 = $18&65535;
 $20 = ($17|0)==($19|0);
 do {
  if ($20) {
   $21 = HEAP16[$expRight>>1]|0;
   $22 = ($21<<16>>16)!=(0);
   if ($22) {
    $23 = HEAP32[$1>>2]|0;
    $24 = (($23) + 16|0);
    $25 = HEAP32[$24>>2]|0;
    $26 = (($25) + 4|0);
    $27 = HEAP32[$26>>2]|0;
    $28 = $27 & -257;
    HEAP32[$26>>2] = $28;
    break;
   }
   $29 = HEAP32[$0>>2]|0;
   $30 = HEAP32[$1>>2]|0;
   $31 = (($30) + 12|0);
   $32 = HEAP32[$31>>2]|0;
   $33 = (_sqlite3ExprCollSeq($29,$32)|0);
   $34 = ($33|0)!=(0|0);
   if ($34) {
    $35 = HEAP32[$1>>2]|0;
    $36 = (($35) + 12|0);
    $37 = HEAP32[$36>>2]|0;
    $38 = (($37) + 4|0);
    $39 = HEAP32[$38>>2]|0;
    $40 = $39 | 256;
    HEAP32[$38>>2] = $40;
   }
  }
 } while(0);
 $41 = HEAP32[$1>>2]|0;
 $42 = (($41) + 16|0);
 $43 = HEAP32[$42>>2]|0;
 HEAP32[$t>>2] = $43;
 $44 = HEAP32[$1>>2]|0;
 $45 = (($44) + 12|0);
 $46 = HEAP32[$45>>2]|0;
 $47 = HEAP32[$1>>2]|0;
 $48 = (($47) + 16|0);
 HEAP32[$48>>2] = $46;
 $49 = HEAP32[$t>>2]|0;
 $50 = HEAP32[$1>>2]|0;
 $51 = (($50) + 12|0);
 HEAP32[$51>>2] = $49;
 $52 = HEAP32[$1>>2]|0;
 $53 = HEAP8[$52>>0]|0;
 $54 = $53&255;
 $55 = ($54|0)>=(80);
 if (!($55)) {
  STACKTOP = sp;return;
 }
 $56 = HEAP32[$1>>2]|0;
 $57 = HEAP8[$56>>0]|0;
 $58 = $57&255;
 $59 = (($58) - 80)|0;
 $60 = $59 ^ 2;
 $61 = (($60) + 80)|0;
 $62 = $61&255;
 $63 = HEAP32[$1>>2]|0;
 HEAP8[$63>>0] = $62;
 STACKTOP = sp;return;
}
function _transferJoinMarkings($pDerived,$pBase) {
 $pDerived = $pDerived|0;
 $pBase = $pBase|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $1 = sp;
 HEAP32[$0>>2] = $pDerived;
 HEAP32[$1>>2] = $pBase;
 $2 = HEAP32[$0>>2]|0;
 $3 = ($2|0)!=(0|0);
 if (!($3)) {
  STACKTOP = sp;return;
 }
 $4 = HEAP32[$1>>2]|0;
 $5 = (($4) + 4|0);
 $6 = HEAP32[$5>>2]|0;
 $7 = $6 & 1;
 $8 = HEAP32[$0>>2]|0;
 $9 = (($8) + 4|0);
 $10 = HEAP32[$9>>2]|0;
 $11 = $10 | $7;
 HEAP32[$9>>2] = $11;
 $12 = HEAP32[$1>>2]|0;
 $13 = (($12) + 36|0);
 $14 = HEAP16[$13>>1]|0;
 $15 = HEAP32[$0>>2]|0;
 $16 = (($15) + 36|0);
 HEAP16[$16>>1] = $14;
 STACKTOP = sp;return;
}
function _exprAnalyzeOrTerm($pSrc,$pWC,$idxTerm) {
 $pSrc = $pSrc|0;
 $pWC = $pWC|0;
 $idxTerm = $idxTerm|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0;
 var $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0, $188 = 0;
 var $189 = 0, $19 = 0, $190 = 0, $191 = 0, $192 = 0, $193 = 0, $194 = 0, $195 = 0, $196 = 0, $197 = 0, $198 = 0, $199 = 0, $2 = 0, $20 = 0, $200 = 0, $201 = 0, $202 = 0, $203 = 0, $204 = 0, $205 = 0;
 var $206 = 0, $207 = 0, $208 = 0, $209 = 0, $21 = 0, $210 = 0, $211 = 0, $212 = 0, $213 = 0, $214 = 0, $215 = 0, $216 = 0, $217 = 0, $218 = 0, $219 = 0, $22 = 0, $220 = 0, $221 = 0, $222 = 0, $223 = 0;
 var $224 = 0, $225 = 0, $226 = 0, $227 = 0, $228 = 0, $229 = 0, $23 = 0, $230 = 0, $231 = 0, $232 = 0, $233 = 0, $234 = 0, $235 = 0, $236 = 0, $237 = 0, $238 = 0, $239 = 0, $24 = 0, $240 = 0, $241 = 0;
 var $242 = 0, $243 = 0, $244 = 0, $245 = 0, $246 = 0, $247 = 0, $248 = 0, $249 = 0, $25 = 0, $250 = 0, $251 = 0, $252 = 0, $253 = 0, $254 = 0, $255 = 0, $256 = 0, $257 = 0, $258 = 0, $259 = 0, $26 = 0;
 var $260 = 0, $261 = 0, $262 = 0, $263 = 0, $264 = 0, $265 = 0, $266 = 0, $267 = 0, $268 = 0, $269 = 0, $27 = 0, $270 = 0, $271 = 0, $272 = 0, $273 = 0, $274 = 0, $275 = 0, $276 = 0, $277 = 0, $278 = 0;
 var $279 = 0, $28 = 0, $280 = 0, $281 = 0, $282 = 0, $283 = 0, $284 = 0, $285 = 0, $286 = 0, $287 = 0, $288 = 0, $289 = 0, $29 = 0, $290 = 0, $291 = 0, $292 = 0, $293 = 0, $294 = 0, $295 = 0, $296 = 0;
 var $297 = 0, $298 = 0, $299 = 0, $3 = 0, $30 = 0, $300 = 0, $301 = 0, $302 = 0, $303 = 0, $304 = 0, $305 = 0, $306 = 0, $307 = 0, $308 = 0, $309 = 0, $31 = 0, $310 = 0, $311 = 0, $312 = 0, $313 = 0;
 var $314 = 0, $315 = 0, $316 = 0, $317 = 0, $318 = 0, $319 = 0, $32 = 0, $320 = 0, $321 = 0, $322 = 0, $323 = 0, $324 = 0, $325 = 0, $326 = 0, $327 = 0, $328 = 0, $329 = 0, $33 = 0, $330 = 0, $331 = 0;
 var $332 = 0, $333 = 0, $334 = 0, $335 = 0, $336 = 0, $337 = 0, $338 = 0, $339 = 0, $34 = 0, $340 = 0, $341 = 0, $342 = 0, $343 = 0, $344 = 0, $345 = 0, $346 = 0, $347 = 0, $348 = 0, $349 = 0, $35 = 0;
 var $350 = 0, $351 = 0, $352 = 0, $353 = 0, $354 = 0, $355 = 0, $356 = 0, $357 = 0, $358 = 0, $359 = 0, $36 = 0, $360 = 0, $361 = 0, $362 = 0, $363 = 0, $364 = 0, $365 = 0, $366 = 0, $367 = 0, $368 = 0;
 var $369 = 0, $37 = 0, $370 = 0, $371 = 0, $372 = 0, $373 = 0, $374 = 0, $375 = 0, $376 = 0, $377 = 0, $378 = 0, $379 = 0, $38 = 0, $380 = 0, $381 = 0, $382 = 0, $383 = 0, $384 = 0, $385 = 0, $386 = 0;
 var $387 = 0, $388 = 0, $389 = 0, $39 = 0, $390 = 0, $391 = 0, $392 = 0, $393 = 0, $394 = 0, $395 = 0, $396 = 0, $397 = 0, $398 = 0, $399 = 0, $4 = 0, $40 = 0, $400 = 0, $401 = 0, $402 = 0, $403 = 0;
 var $404 = 0, $405 = 0, $406 = 0, $407 = 0, $408 = 0, $409 = 0, $41 = 0, $410 = 0, $411 = 0, $412 = 0, $413 = 0, $414 = 0, $415 = 0, $416 = 0, $417 = 0, $418 = 0, $419 = 0, $42 = 0, $420 = 0, $421 = 0;
 var $422 = 0, $423 = 0, $424 = 0, $425 = 0, $426 = 0, $427 = 0, $428 = 0, $429 = 0, $43 = 0, $430 = 0, $431 = 0, $432 = 0, $433 = 0, $434 = 0, $435 = 0, $436 = 0, $437 = 0, $438 = 0, $439 = 0, $44 = 0;
 var $440 = 0, $441 = 0, $442 = 0, $443 = 0, $444 = 0, $445 = 0, $446 = 0, $447 = 0, $448 = 0, $449 = 0, $45 = 0, $450 = 0, $451 = 0, $452 = 0, $453 = 0, $454 = 0, $455 = 0, $456 = 0, $457 = 0, $458 = 0;
 var $459 = 0, $46 = 0, $460 = 0, $461 = 0, $462 = 0, $463 = 0, $464 = 0, $465 = 0, $466 = 0, $467 = 0, $468 = 0, $469 = 0, $47 = 0, $470 = 0, $471 = 0, $472 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0;
 var $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0;
 var $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0;
 var $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $affLeft = 0, $affRight = 0, $b = 0, $b1 = 0, $chngToIN = 0, $db = 0, $i = 0;
 var $iColumn = 0, $iCursor = 0, $idxNew = 0, $indexable = 0, $j = 0, $j2 = 0, $okToChngToIN = 0, $or$cond = 0, $or$cond3 = 0, $or$cond5 = 0, $pAndInfo = 0, $pAndTerm = 0, $pAndWC = 0, $pDup = 0, $pExpr = 0, $pLeft = 0, $pList = 0, $pNew = 0, $pOrInfo = 0, $pOrTerm = 0;
 var $pOrWc = 0, $pOther = 0, $pParse = 0, $pTerm = 0, $pWInfo = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 144|0;
 $0 = sp + 140|0;
 $1 = sp + 136|0;
 $2 = sp + 132|0;
 $pWInfo = sp + 128|0;
 $pParse = sp + 124|0;
 $db = sp + 120|0;
 $pTerm = sp + 116|0;
 $pExpr = sp + 112|0;
 $i = sp + 108|0;
 $pOrWc = sp + 104|0;
 $pOrTerm = sp + 100|0;
 $pOrInfo = sp + 96|0;
 $chngToIN = sp + 24|0;
 $indexable = sp + 16|0;
 $pAndInfo = sp + 92|0;
 $pAndWC = sp + 88|0;
 $pAndTerm = sp + 84|0;
 $j = sp + 80|0;
 $b = sp + 8|0;
 $b1 = sp;
 $pOther = sp + 76|0;
 $okToChngToIN = sp + 72|0;
 $iColumn = sp + 68|0;
 $iCursor = sp + 64|0;
 $j2 = sp + 60|0;
 $affLeft = sp + 56|0;
 $affRight = sp + 52|0;
 $pDup = sp + 48|0;
 $pList = sp + 44|0;
 $pLeft = sp + 40|0;
 $pNew = sp + 36|0;
 $idxNew = sp + 32|0;
 HEAP32[$0>>2] = $pSrc;
 HEAP32[$1>>2] = $pWC;
 HEAP32[$2>>2] = $idxTerm;
 $3 = HEAP32[$1>>2]|0;
 $4 = HEAP32[$3>>2]|0;
 HEAP32[$pWInfo>>2] = $4;
 $5 = HEAP32[$pWInfo>>2]|0;
 $6 = HEAP32[$5>>2]|0;
 HEAP32[$pParse>>2] = $6;
 $7 = HEAP32[$pParse>>2]|0;
 $8 = HEAP32[$7>>2]|0;
 HEAP32[$db>>2] = $8;
 $9 = HEAP32[$2>>2]|0;
 $10 = HEAP32[$1>>2]|0;
 $11 = (($10) + 20|0);
 $12 = HEAP32[$11>>2]|0;
 $13 = (($12) + (($9*48)|0)|0);
 HEAP32[$pTerm>>2] = $13;
 $14 = HEAP32[$pTerm>>2]|0;
 $15 = HEAP32[$14>>2]|0;
 HEAP32[$pExpr>>2] = $15;
 $16 = HEAP32[$db>>2]|0;
 $17 = (_sqlite3DbMallocZero($16,416,0)|0);
 HEAP32[$pOrInfo>>2] = $17;
 $18 = HEAP32[$pTerm>>2]|0;
 $19 = (($18) + 12|0);
 HEAP32[$19>>2] = $17;
 $20 = HEAP32[$pOrInfo>>2]|0;
 $21 = ($20|0)==(0|0);
 if ($21) {
  STACKTOP = sp;return;
 }
 $22 = HEAP32[$pTerm>>2]|0;
 $23 = (($22) + 20|0);
 $24 = HEAP8[$23>>0]|0;
 $25 = $24&255;
 $26 = $25 | 16;
 $27 = $26&255;
 HEAP8[$23>>0] = $27;
 $28 = HEAP32[$pOrInfo>>2]|0;
 HEAP32[$pOrWc>>2] = $28;
 $29 = HEAP32[$pOrWc>>2]|0;
 $30 = HEAP32[$pWInfo>>2]|0;
 _whereClauseInit($29,$30);
 $31 = HEAP32[$pOrWc>>2]|0;
 $32 = HEAP32[$pExpr>>2]|0;
 _whereSplit($31,$32,71);
 $33 = HEAP32[$0>>2]|0;
 $34 = HEAP32[$pOrWc>>2]|0;
 _exprAnalyzeAll($33,$34);
 $35 = HEAP32[$db>>2]|0;
 $36 = (($35) + 64|0);
 $37 = HEAP8[$36>>0]|0;
 $38 = ($37<<24>>24)!=(0);
 if ($38) {
  STACKTOP = sp;return;
 }
 $39 = $indexable;
 $40 = $39;
 HEAP32[$40>>2] = -1;
 $41 = (($39) + 4)|0;
 $42 = $41;
 HEAP32[$42>>2] = -1;
 $43 = $chngToIN;
 $44 = $43;
 HEAP32[$44>>2] = -1;
 $45 = (($43) + 4)|0;
 $46 = $45;
 HEAP32[$46>>2] = -1;
 $47 = HEAP32[$pOrWc>>2]|0;
 $48 = (($47) + 12|0);
 $49 = HEAP32[$48>>2]|0;
 $50 = (($49) - 1)|0;
 HEAP32[$i>>2] = $50;
 $51 = HEAP32[$pOrWc>>2]|0;
 $52 = (($51) + 20|0);
 $53 = HEAP32[$52>>2]|0;
 HEAP32[$pOrTerm>>2] = $53;
 while(1) {
  $54 = HEAP32[$i>>2]|0;
  $55 = ($54|0)>=(0);
  $56 = $indexable;
  $57 = $56;
  $58 = HEAP32[$57>>2]|0;
  $59 = (($56) + 4)|0;
  $60 = $59;
  $61 = HEAP32[$60>>2]|0;
  $62 = ($58|0)!=(0);
  $63 = ($61|0)!=(0);
  $64 = $62 | $63;
  $or$cond = $55 & $64;
  if (!($or$cond)) {
   break;
  }
  $65 = HEAP32[$pOrTerm>>2]|0;
  $66 = (($65) + 18|0);
  $67 = HEAP16[$66>>1]|0;
  $68 = $67&65535;
  $69 = $68 & 255;
  $70 = ($69|0)==(0);
  do {
   if ($70) {
    $71 = $chngToIN;
    $72 = $71;
    HEAP32[$72>>2] = 0;
    $73 = (($71) + 4)|0;
    $74 = $73;
    HEAP32[$74>>2] = 0;
    $75 = HEAP32[$db>>2]|0;
    $76 = (_sqlite3DbMallocRaw($75,408,0)|0);
    HEAP32[$pAndInfo>>2] = $76;
    $77 = HEAP32[$pAndInfo>>2]|0;
    $78 = ($77|0)!=(0|0);
    if ($78) {
     $79 = $b;
     $80 = $79;
     HEAP32[$80>>2] = 0;
     $81 = (($79) + 4)|0;
     $82 = $81;
     HEAP32[$82>>2] = 0;
     $83 = HEAP32[$pAndInfo>>2]|0;
     $84 = HEAP32[$pOrTerm>>2]|0;
     $85 = (($84) + 12|0);
     HEAP32[$85>>2] = $83;
     $86 = HEAP32[$pOrTerm>>2]|0;
     $87 = (($86) + 20|0);
     $88 = HEAP8[$87>>0]|0;
     $89 = $88&255;
     $90 = $89 | 32;
     $91 = $90&255;
     HEAP8[$87>>0] = $91;
     $92 = HEAP32[$pOrTerm>>2]|0;
     $93 = (($92) + 18|0);
     HEAP16[$93>>1] = 512;
     $94 = HEAP32[$pAndInfo>>2]|0;
     HEAP32[$pAndWC>>2] = $94;
     $95 = HEAP32[$pAndWC>>2]|0;
     $96 = HEAP32[$1>>2]|0;
     $97 = HEAP32[$96>>2]|0;
     _whereClauseInit($95,$97);
     $98 = HEAP32[$pAndWC>>2]|0;
     $99 = HEAP32[$pOrTerm>>2]|0;
     $100 = HEAP32[$99>>2]|0;
     _whereSplit($98,$100,72);
     $101 = HEAP32[$0>>2]|0;
     $102 = HEAP32[$pAndWC>>2]|0;
     _exprAnalyzeAll($101,$102);
     $103 = HEAP32[$1>>2]|0;
     $104 = HEAP32[$pAndWC>>2]|0;
     $105 = (($104) + 4|0);
     HEAP32[$105>>2] = $103;
     $106 = HEAP32[$db>>2]|0;
     $107 = (($106) + 64|0);
     $108 = HEAP8[$107>>0]|0;
     $109 = ($108<<24>>24)!=(0);
     L21: do {
      if (!($109)) {
       HEAP32[$j>>2] = 0;
       $110 = HEAP32[$pAndWC>>2]|0;
       $111 = (($110) + 20|0);
       $112 = HEAP32[$111>>2]|0;
       HEAP32[$pAndTerm>>2] = $112;
       while(1) {
        $113 = HEAP32[$j>>2]|0;
        $114 = HEAP32[$pAndWC>>2]|0;
        $115 = (($114) + 12|0);
        $116 = HEAP32[$115>>2]|0;
        $117 = ($113|0)<($116|0);
        if (!($117)) {
         break L21;
        }
        $118 = HEAP32[$pAndTerm>>2]|0;
        $119 = HEAP32[$118>>2]|0;
        $120 = HEAP8[$119>>0]|0;
        $121 = $120&255;
        $122 = (_allowedOp($121)|0);
        $123 = ($122|0)!=(0);
        if ($123) {
         $124 = HEAP32[$pWInfo>>2]|0;
         $125 = (($124) + 68|0);
         $126 = HEAP32[$pAndTerm>>2]|0;
         $127 = (($126) + 8|0);
         $128 = HEAP32[$127>>2]|0;
         $129 = (_getMask($125,$128)|0);
         $130 = tempRet0;
         $131 = $b;
         $132 = $131;
         $133 = HEAP32[$132>>2]|0;
         $134 = (($131) + 4)|0;
         $135 = $134;
         $136 = HEAP32[$135>>2]|0;
         $137 = $133 | $129;
         $138 = $136 | $130;
         $139 = $b;
         $140 = $139;
         HEAP32[$140>>2] = $137;
         $141 = (($139) + 4)|0;
         $142 = $141;
         HEAP32[$142>>2] = $138;
        }
        $143 = HEAP32[$j>>2]|0;
        $144 = (($143) + 1)|0;
        HEAP32[$j>>2] = $144;
        $145 = HEAP32[$pAndTerm>>2]|0;
        $146 = (($145) + 48|0);
        HEAP32[$pAndTerm>>2] = $146;
       }
      }
     } while(0);
     $147 = $b;
     $148 = $147;
     $149 = HEAP32[$148>>2]|0;
     $150 = (($147) + 4)|0;
     $151 = $150;
     $152 = HEAP32[$151>>2]|0;
     $153 = $indexable;
     $154 = $153;
     $155 = HEAP32[$154>>2]|0;
     $156 = (($153) + 4)|0;
     $157 = $156;
     $158 = HEAP32[$157>>2]|0;
     $159 = $155 & $149;
     $160 = $158 & $152;
     $161 = $indexable;
     $162 = $161;
     HEAP32[$162>>2] = $159;
     $163 = (($161) + 4)|0;
     $164 = $163;
     HEAP32[$164>>2] = $160;
    }
   } else {
    $165 = HEAP32[$pOrTerm>>2]|0;
    $166 = (($165) + 20|0);
    $167 = HEAP8[$166>>0]|0;
    $168 = $167&255;
    $169 = $168 & 8;
    $170 = ($169|0)!=(0);
    if (!($170)) {
     $171 = HEAP32[$pWInfo>>2]|0;
     $172 = (($171) + 68|0);
     $173 = HEAP32[$pOrTerm>>2]|0;
     $174 = (($173) + 8|0);
     $175 = HEAP32[$174>>2]|0;
     $176 = (_getMask($172,$175)|0);
     $177 = tempRet0;
     $178 = $b1;
     $179 = $178;
     HEAP32[$179>>2] = $176;
     $180 = (($178) + 4)|0;
     $181 = $180;
     HEAP32[$181>>2] = $177;
     $182 = HEAP32[$pOrTerm>>2]|0;
     $183 = (($182) + 20|0);
     $184 = HEAP8[$183>>0]|0;
     $185 = $184&255;
     $186 = $185 & 2;
     $187 = ($186|0)!=(0);
     if ($187) {
      $188 = HEAP32[$pOrTerm>>2]|0;
      $189 = (($188) + 4|0);
      $190 = HEAP32[$189>>2]|0;
      $191 = HEAP32[$pOrWc>>2]|0;
      $192 = (($191) + 20|0);
      $193 = HEAP32[$192>>2]|0;
      $194 = (($193) + (($190*48)|0)|0);
      HEAP32[$pOther>>2] = $194;
      $195 = HEAP32[$pWInfo>>2]|0;
      $196 = (($195) + 68|0);
      $197 = HEAP32[$pOther>>2]|0;
      $198 = (($197) + 8|0);
      $199 = HEAP32[$198>>2]|0;
      $200 = (_getMask($196,$199)|0);
      $201 = tempRet0;
      $202 = $b1;
      $203 = $202;
      $204 = HEAP32[$203>>2]|0;
      $205 = (($202) + 4)|0;
      $206 = $205;
      $207 = HEAP32[$206>>2]|0;
      $208 = $204 | $200;
      $209 = $207 | $201;
      $210 = $b1;
      $211 = $210;
      HEAP32[$211>>2] = $208;
      $212 = (($210) + 4)|0;
      $213 = $212;
      HEAP32[$213>>2] = $209;
     }
     $214 = $b1;
     $215 = $214;
     $216 = HEAP32[$215>>2]|0;
     $217 = (($214) + 4)|0;
     $218 = $217;
     $219 = HEAP32[$218>>2]|0;
     $220 = $indexable;
     $221 = $220;
     $222 = HEAP32[$221>>2]|0;
     $223 = (($220) + 4)|0;
     $224 = $223;
     $225 = HEAP32[$224>>2]|0;
     $226 = $222 & $216;
     $227 = $225 & $219;
     $228 = $indexable;
     $229 = $228;
     HEAP32[$229>>2] = $226;
     $230 = (($228) + 4)|0;
     $231 = $230;
     HEAP32[$231>>2] = $227;
     $232 = HEAP32[$pOrTerm>>2]|0;
     $233 = (($232) + 18|0);
     $234 = HEAP16[$233>>1]|0;
     $235 = $234&65535;
     $236 = $235 & 2;
     $237 = ($236|0)==(0);
     if ($237) {
      $238 = $chngToIN;
      $239 = $238;
      HEAP32[$239>>2] = 0;
      $240 = (($238) + 4)|0;
      $241 = $240;
      HEAP32[$241>>2] = 0;
      break;
     } else {
      $242 = $b1;
      $243 = $242;
      $244 = HEAP32[$243>>2]|0;
      $245 = (($242) + 4)|0;
      $246 = $245;
      $247 = HEAP32[$246>>2]|0;
      $248 = $chngToIN;
      $249 = $248;
      $250 = HEAP32[$249>>2]|0;
      $251 = (($248) + 4)|0;
      $252 = $251;
      $253 = HEAP32[$252>>2]|0;
      $254 = $250 & $244;
      $255 = $253 & $247;
      $256 = $chngToIN;
      $257 = $256;
      HEAP32[$257>>2] = $254;
      $258 = (($256) + 4)|0;
      $259 = $258;
      HEAP32[$259>>2] = $255;
      break;
     }
    }
   }
  } while(0);
  $260 = HEAP32[$i>>2]|0;
  $261 = (($260) + -1)|0;
  HEAP32[$i>>2] = $261;
  $262 = HEAP32[$pOrTerm>>2]|0;
  $263 = (($262) + 48|0);
  HEAP32[$pOrTerm>>2] = $263;
 }
 $264 = $indexable;
 $265 = $264;
 $266 = HEAP32[$265>>2]|0;
 $267 = (($264) + 4)|0;
 $268 = $267;
 $269 = HEAP32[$268>>2]|0;
 $270 = HEAP32[$pOrInfo>>2]|0;
 $271 = (($270) + 408|0);
 $272 = $271;
 $273 = $272;
 HEAP32[$273>>2] = $266;
 $274 = (($272) + 4)|0;
 $275 = $274;
 HEAP32[$275>>2] = $269;
 $276 = $indexable;
 $277 = $276;
 $278 = HEAP32[$277>>2]|0;
 $279 = (($276) + 4)|0;
 $280 = $279;
 $281 = HEAP32[$280>>2]|0;
 $282 = ($278|0)==(0);
 $283 = ($281|0)==(0);
 $284 = $282 & $283;
 $285 = $284 ? 0 : 256;
 $286 = $285&65535;
 $287 = HEAP32[$pTerm>>2]|0;
 $288 = (($287) + 18|0);
 HEAP16[$288>>1] = $286;
 $289 = $chngToIN;
 $290 = $289;
 $291 = HEAP32[$290>>2]|0;
 $292 = (($289) + 4)|0;
 $293 = $292;
 $294 = HEAP32[$293>>2]|0;
 $295 = ($291|0)!=(0);
 $296 = ($294|0)!=(0);
 $297 = $295 | $296;
 if (!($297)) {
  STACKTOP = sp;return;
 }
 HEAP32[$okToChngToIN>>2] = 0;
 HEAP32[$iColumn>>2] = -1;
 HEAP32[$iCursor>>2] = -1;
 HEAP32[$j2>>2] = 0;
 HEAP32[$j2>>2] = 0;
 while(1) {
  $298 = HEAP32[$j2>>2]|0;
  $299 = ($298|0)>=(2);
  $300 = HEAP32[$okToChngToIN>>2]|0;
  $301 = ($300|0)!=(0);
  $or$cond3 = $299 | $301;
  if ($or$cond3) {
   break;
  }
  $302 = HEAP32[$pOrWc>>2]|0;
  $303 = (($302) + 20|0);
  $304 = HEAP32[$303>>2]|0;
  HEAP32[$pOrTerm>>2] = $304;
  $305 = HEAP32[$pOrWc>>2]|0;
  $306 = (($305) + 12|0);
  $307 = HEAP32[$306>>2]|0;
  $308 = (($307) - 1)|0;
  HEAP32[$i>>2] = $308;
  while(1) {
   $309 = HEAP32[$i>>2]|0;
   $310 = ($309|0)>=(0);
   if (!($310)) {
    break;
   }
   $311 = HEAP32[$pOrTerm>>2]|0;
   $312 = (($311) + 20|0);
   $313 = HEAP8[$312>>0]|0;
   $314 = $313&255;
   $315 = $314 & -65;
   $316 = $315&255;
   HEAP8[$312>>0] = $316;
   $317 = HEAP32[$pOrTerm>>2]|0;
   $318 = (($317) + 8|0);
   $319 = HEAP32[$318>>2]|0;
   $320 = HEAP32[$iCursor>>2]|0;
   $321 = ($319|0)==($320|0);
   if (!($321)) {
    $322 = $chngToIN;
    $323 = $322;
    $324 = HEAP32[$323>>2]|0;
    $325 = (($322) + 4)|0;
    $326 = $325;
    $327 = HEAP32[$326>>2]|0;
    $328 = HEAP32[$pWInfo>>2]|0;
    $329 = (($328) + 68|0);
    $330 = HEAP32[$pOrTerm>>2]|0;
    $331 = (($330) + 8|0);
    $332 = HEAP32[$331>>2]|0;
    $333 = (_getMask($329,$332)|0);
    $334 = tempRet0;
    $335 = $324 & $333;
    $336 = $327 & $334;
    $337 = ($335|0)==(0);
    $338 = ($336|0)==(0);
    $339 = $337 & $338;
    if (!($339)) {
     label = 28;
     break;
    }
   }
   $346 = HEAP32[$i>>2]|0;
   $347 = (($346) + -1)|0;
   HEAP32[$i>>2] = $347;
   $348 = HEAP32[$pOrTerm>>2]|0;
   $349 = (($348) + 48|0);
   HEAP32[$pOrTerm>>2] = $349;
  }
  if ((label|0) == 28) {
   label = 0;
   $340 = HEAP32[$pOrTerm>>2]|0;
   $341 = (($340) + 12|0);
   $342 = HEAP32[$341>>2]|0;
   HEAP32[$iColumn>>2] = $342;
   $343 = HEAP32[$pOrTerm>>2]|0;
   $344 = (($343) + 8|0);
   $345 = HEAP32[$344>>2]|0;
   HEAP32[$iCursor>>2] = $345;
  }
  $350 = HEAP32[$i>>2]|0;
  $351 = ($350|0)<(0);
  if ($351) {
   break;
  }
  HEAP32[$okToChngToIN>>2] = 1;
  while(1) {
   $352 = HEAP32[$i>>2]|0;
   $353 = ($352|0)>=(0);
   $354 = HEAP32[$okToChngToIN>>2]|0;
   $355 = ($354|0)!=(0);
   $or$cond5 = $353 & $355;
   if (!($or$cond5)) {
    break;
   }
   $356 = HEAP32[$pOrTerm>>2]|0;
   $357 = (($356) + 8|0);
   $358 = HEAP32[$357>>2]|0;
   $359 = HEAP32[$iCursor>>2]|0;
   $360 = ($358|0)!=($359|0);
   $361 = HEAP32[$pOrTerm>>2]|0;
   do {
    if ($360) {
     $362 = (($361) + 20|0);
     $363 = HEAP8[$362>>0]|0;
     $364 = $363&255;
     $365 = $364 & -65;
     $366 = $365&255;
     HEAP8[$362>>0] = $366;
    } else {
     $367 = (($361) + 12|0);
     $368 = HEAP32[$367>>2]|0;
     $369 = HEAP32[$iColumn>>2]|0;
     $370 = ($368|0)!=($369|0);
     if ($370) {
      HEAP32[$okToChngToIN>>2] = 0;
      break;
     }
     $371 = HEAP32[$pOrTerm>>2]|0;
     $372 = HEAP32[$371>>2]|0;
     $373 = (($372) + 16|0);
     $374 = HEAP32[$373>>2]|0;
     $375 = (_sqlite3ExprAffinity($374)|0);
     $376 = $375 << 24 >> 24;
     HEAP32[$affRight>>2] = $376;
     $377 = HEAP32[$pOrTerm>>2]|0;
     $378 = HEAP32[$377>>2]|0;
     $379 = (($378) + 12|0);
     $380 = HEAP32[$379>>2]|0;
     $381 = (_sqlite3ExprAffinity($380)|0);
     $382 = $381 << 24 >> 24;
     HEAP32[$affLeft>>2] = $382;
     $383 = HEAP32[$affRight>>2]|0;
     $384 = ($383|0)!=(0);
     if ($384) {
      $385 = HEAP32[$affRight>>2]|0;
      $386 = HEAP32[$affLeft>>2]|0;
      $387 = ($385|0)!=($386|0);
      if ($387) {
       HEAP32[$okToChngToIN>>2] = 0;
       break;
      }
     }
     $388 = HEAP32[$pOrTerm>>2]|0;
     $389 = (($388) + 20|0);
     $390 = HEAP8[$389>>0]|0;
     $391 = $390&255;
     $392 = $391 | 64;
     $393 = $392&255;
     HEAP8[$389>>0] = $393;
    }
   } while(0);
   $394 = HEAP32[$i>>2]|0;
   $395 = (($394) + -1)|0;
   HEAP32[$i>>2] = $395;
   $396 = HEAP32[$pOrTerm>>2]|0;
   $397 = (($396) + 48|0);
   HEAP32[$pOrTerm>>2] = $397;
  }
  $398 = HEAP32[$j2>>2]|0;
  $399 = (($398) + 1)|0;
  HEAP32[$j2>>2] = $399;
 }
 $400 = HEAP32[$okToChngToIN>>2]|0;
 $401 = ($400|0)!=(0);
 if (!($401)) {
  STACKTOP = sp;return;
 }
 HEAP32[$pList>>2] = 0;
 HEAP32[$pLeft>>2] = 0;
 $402 = HEAP32[$pOrWc>>2]|0;
 $403 = (($402) + 12|0);
 $404 = HEAP32[$403>>2]|0;
 $405 = (($404) - 1)|0;
 HEAP32[$i>>2] = $405;
 $406 = HEAP32[$pOrWc>>2]|0;
 $407 = (($406) + 20|0);
 $408 = HEAP32[$407>>2]|0;
 HEAP32[$pOrTerm>>2] = $408;
 while(1) {
  $409 = HEAP32[$i>>2]|0;
  $410 = ($409|0)>=(0);
  if (!($410)) {
   break;
  }
  $411 = HEAP32[$pOrTerm>>2]|0;
  $412 = (($411) + 20|0);
  $413 = HEAP8[$412>>0]|0;
  $414 = $413&255;
  $415 = $414 & 64;
  $416 = ($415|0)==(0);
  if (!($416)) {
   $417 = HEAP32[$db>>2]|0;
   $418 = HEAP32[$pOrTerm>>2]|0;
   $419 = HEAP32[$418>>2]|0;
   $420 = (($419) + 16|0);
   $421 = HEAP32[$420>>2]|0;
   $422 = (_sqlite3ExprDup($417,$421,0)|0);
   HEAP32[$pDup>>2] = $422;
   $423 = HEAP32[$pWInfo>>2]|0;
   $424 = HEAP32[$423>>2]|0;
   $425 = HEAP32[$pList>>2]|0;
   $426 = HEAP32[$pDup>>2]|0;
   $427 = (_sqlite3ExprListAppend($424,$425,$426)|0);
   HEAP32[$pList>>2] = $427;
   $428 = HEAP32[$pOrTerm>>2]|0;
   $429 = HEAP32[$428>>2]|0;
   $430 = (($429) + 12|0);
   $431 = HEAP32[$430>>2]|0;
   HEAP32[$pLeft>>2] = $431;
  }
  $432 = HEAP32[$i>>2]|0;
  $433 = (($432) + -1)|0;
  HEAP32[$i>>2] = $433;
  $434 = HEAP32[$pOrTerm>>2]|0;
  $435 = (($434) + 48|0);
  HEAP32[$pOrTerm>>2] = $435;
 }
 $436 = HEAP32[$db>>2]|0;
 $437 = HEAP32[$pLeft>>2]|0;
 $438 = (_sqlite3ExprDup($436,$437,0)|0);
 HEAP32[$pDup>>2] = $438;
 $439 = HEAP32[$pParse>>2]|0;
 $440 = HEAP32[$pDup>>2]|0;
 $441 = (_sqlite3PExpr($439,75,$440,0,0)|0);
 HEAP32[$pNew>>2] = $441;
 $442 = HEAP32[$pNew>>2]|0;
 $443 = ($442|0)!=(0|0);
 if ($443) {
  $444 = HEAP32[$pNew>>2]|0;
  $445 = HEAP32[$pExpr>>2]|0;
  _transferJoinMarkings($444,$445);
  $446 = HEAP32[$pList>>2]|0;
  $447 = HEAP32[$pNew>>2]|0;
  $448 = (($447) + 20|0);
  HEAP32[$448>>2] = $446;
  $449 = HEAP32[$1>>2]|0;
  $450 = HEAP32[$pNew>>2]|0;
  $451 = (_whereClauseInsert($449,$450,3)|0);
  HEAP32[$idxNew>>2] = $451;
  $452 = HEAP32[$0>>2]|0;
  $453 = HEAP32[$1>>2]|0;
  $454 = HEAP32[$idxNew>>2]|0;
  _exprAnalyze($452,$453,$454);
  $455 = HEAP32[$2>>2]|0;
  $456 = HEAP32[$1>>2]|0;
  $457 = (($456) + 20|0);
  $458 = HEAP32[$457>>2]|0;
  $459 = (($458) + (($455*48)|0)|0);
  HEAP32[$pTerm>>2] = $459;
  $460 = HEAP32[$2>>2]|0;
  $461 = HEAP32[$idxNew>>2]|0;
  $462 = HEAP32[$1>>2]|0;
  $463 = (($462) + 20|0);
  $464 = HEAP32[$463>>2]|0;
  $465 = (($464) + (($461*48)|0)|0);
  $466 = (($465) + 4|0);
  HEAP32[$466>>2] = $460;
  $467 = HEAP32[$pTerm>>2]|0;
  $468 = (($467) + 21|0);
  HEAP8[$468>>0] = 1;
 } else {
  $469 = HEAP32[$db>>2]|0;
  $470 = HEAP32[$pList>>2]|0;
  _sqlite3ExprListDelete($469,$470);
 }
 $471 = HEAP32[$pTerm>>2]|0;
 $472 = (($471) + 18|0);
 HEAP16[$472>>1] = 2048;
 STACKTOP = sp;return;
}
function _isLikeOrGlob($pParse,$pExpr,$ppPrefix,$pisComplete,$pnoCase) {
 $pParse = $pParse|0;
 $pExpr = $pExpr|0;
 $ppPrefix = $ppPrefix|0;
 $pisComplete = $pisComplete|0;
 $pnoCase = $pnoCase|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $17 = 0, $18 = 0, $19 = 0;
 var $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0;
 var $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0;
 var $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0;
 var $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0;
 var $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $c = 0, $cnt = 0, $db = 0, $iCol = 0, $op = 0, $pLeft = 0, $pList = 0, $pPrefix = 0, $pReprepare = 0, $pRight = 0, $pVal = 0, $r1 = 0;
 var $v = 0, $wc = 0, $z = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 96|0;
 $0 = sp + 76|0;
 $1 = sp + 72|0;
 $2 = sp + 68|0;
 $3 = sp + 64|0;
 $4 = sp + 60|0;
 $5 = sp + 56|0;
 $z = sp + 52|0;
 $pRight = sp + 48|0;
 $pLeft = sp + 44|0;
 $pList = sp + 40|0;
 $c = sp + 36|0;
 $cnt = sp + 32|0;
 $wc = sp + 80|0;
 $db = sp + 28|0;
 $pVal = sp + 24|0;
 $op = sp + 20|0;
 $pReprepare = sp + 16|0;
 $iCol = sp + 12|0;
 $pPrefix = sp + 8|0;
 $v = sp + 4|0;
 $r1 = sp;
 HEAP32[$1>>2] = $pParse;
 HEAP32[$2>>2] = $pExpr;
 HEAP32[$3>>2] = $ppPrefix;
 HEAP32[$4>>2] = $pisComplete;
 HEAP32[$5>>2] = $pnoCase;
 HEAP32[$z>>2] = 0;
 $6 = HEAP32[$1>>2]|0;
 $7 = HEAP32[$6>>2]|0;
 HEAP32[$db>>2] = $7;
 HEAP32[$pVal>>2] = 0;
 $8 = HEAP32[$db>>2]|0;
 $9 = HEAP32[$2>>2]|0;
 $10 = HEAP32[$5>>2]|0;
 $11 = (_sqlite3IsLikeFunction($8,$9,$10,$wc)|0);
 $12 = ($11|0)!=(0);
 if (!($12)) {
  HEAP32[$0>>2] = 0;
  $166 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($166|0);
 }
 $13 = HEAP32[$2>>2]|0;
 $14 = (($13) + 20|0);
 $15 = HEAP32[$14>>2]|0;
 HEAP32[$pList>>2] = $15;
 $16 = HEAP32[$pList>>2]|0;
 $17 = (($16) + 4|0);
 $18 = HEAP32[$17>>2]|0;
 $19 = (($18) + 20|0);
 $20 = HEAP32[$19>>2]|0;
 HEAP32[$pLeft>>2] = $20;
 $21 = HEAP32[$pLeft>>2]|0;
 $22 = HEAP8[$21>>0]|0;
 $23 = $22&255;
 $24 = ($23|0)!=(154);
 if (!($24)) {
  $25 = HEAP32[$pLeft>>2]|0;
  $26 = (_sqlite3ExprAffinity($25)|0);
  $27 = $26 << 24 >> 24;
  $28 = ($27|0)!=(66);
  if (!($28)) {
   $29 = HEAP32[$pLeft>>2]|0;
   $30 = (($29) + 44|0);
   $31 = HEAP32[$30>>2]|0;
   $32 = (($31) + 44|0);
   $33 = HEAP8[$32>>0]|0;
   $34 = $33&255;
   $35 = $34 & 16;
   $36 = ($35|0)!=(0);
   if (!($36)) {
    $37 = HEAP32[$pList>>2]|0;
    $38 = (($37) + 4|0);
    $39 = HEAP32[$38>>2]|0;
    $40 = HEAP32[$39>>2]|0;
    $41 = (_sqlite3ExprSkipCollate($40)|0);
    HEAP32[$pRight>>2] = $41;
    $42 = HEAP32[$pRight>>2]|0;
    $43 = HEAP8[$42>>0]|0;
    $44 = $43&255;
    HEAP32[$op>>2] = $44;
    $45 = HEAP32[$op>>2]|0;
    $46 = ($45|0)==(135);
    if ($46) {
     $47 = HEAP32[$1>>2]|0;
     $48 = (($47) + 476|0);
     $49 = HEAP32[$48>>2]|0;
     HEAP32[$pReprepare>>2] = $49;
     $50 = HEAP32[$pRight>>2]|0;
     $51 = (($50) + 32|0);
     $52 = HEAP16[$51>>1]|0;
     $53 = $52 << 16 >> 16;
     HEAP32[$iCol>>2] = $53;
     $54 = HEAP32[$pReprepare>>2]|0;
     $55 = HEAP32[$iCol>>2]|0;
     $56 = (_sqlite3VdbeGetBoundValue($54,$55,65)|0);
     HEAP32[$pVal>>2] = $56;
     $57 = HEAP32[$pVal>>2]|0;
     $58 = ($57|0)!=(0|0);
     if ($58) {
      $59 = HEAP32[$pVal>>2]|0;
      $60 = (_sqlite3_value_type($59)|0);
      $61 = ($60|0)==(3);
      if ($61) {
       $62 = HEAP32[$pVal>>2]|0;
       $63 = (_sqlite3_value_text($62)|0);
       HEAP32[$z>>2] = $63;
      }
     }
     $64 = HEAP32[$1>>2]|0;
     $65 = (($64) + 8|0);
     $66 = HEAP32[$65>>2]|0;
     $67 = HEAP32[$iCol>>2]|0;
     _sqlite3VdbeSetVarmask($66,$67);
    } else {
     $68 = HEAP32[$op>>2]|0;
     $69 = ($68|0)==(97);
     if ($69) {
      $70 = HEAP32[$pRight>>2]|0;
      $71 = (($70) + 8|0);
      $72 = HEAP32[$71>>2]|0;
      HEAP32[$z>>2] = $72;
     }
    }
    $73 = HEAP32[$z>>2]|0;
    $74 = ($73|0)!=(0|0);
    do {
     if ($74) {
      HEAP32[$cnt>>2] = 0;
      while(1) {
       $75 = HEAP32[$cnt>>2]|0;
       $76 = HEAP32[$z>>2]|0;
       $77 = (($76) + ($75)|0);
       $78 = HEAP8[$77>>0]|0;
       $79 = $78 << 24 >> 24;
       HEAP32[$c>>2] = $79;
       $80 = ($79|0)!=(0);
       if ($80) {
        $81 = HEAP32[$c>>2]|0;
        $82 = HEAP8[$wc>>0]|0;
        $83 = $82 << 24 >> 24;
        $84 = ($81|0)!=($83|0);
        if ($84) {
         $85 = HEAP32[$c>>2]|0;
         $86 = (($wc) + 1|0);
         $87 = HEAP8[$86>>0]|0;
         $88 = $87 << 24 >> 24;
         $89 = ($85|0)!=($88|0);
         if ($89) {
          $90 = HEAP32[$c>>2]|0;
          $91 = (($wc) + 2|0);
          $92 = HEAP8[$91>>0]|0;
          $93 = $92 << 24 >> 24;
          $94 = ($90|0)!=($93|0);
          $167 = $94;
         } else {
          $167 = 0;
         }
        } else {
         $167 = 0;
        }
       } else {
        $167 = 0;
       }
       $95 = HEAP32[$cnt>>2]|0;
       if (!($167)) {
        break;
       }
       $96 = (($95) + 1)|0;
       HEAP32[$cnt>>2] = $96;
      }
      $97 = ($95|0)!=(0);
      if ($97) {
       $98 = HEAP32[$cnt>>2]|0;
       $99 = (($98) - 1)|0;
       $100 = HEAP32[$z>>2]|0;
       $101 = (($100) + ($99)|0);
       $102 = HEAP8[$101>>0]|0;
       $103 = $102&255;
       $104 = (255)!=($103|0);
       if ($104) {
        $105 = HEAP32[$c>>2]|0;
        $106 = HEAP8[$wc>>0]|0;
        $107 = $106 << 24 >> 24;
        $108 = ($105|0)==($107|0);
        if ($108) {
         $109 = HEAP32[$cnt>>2]|0;
         $110 = (($109) + 1)|0;
         $111 = HEAP32[$z>>2]|0;
         $112 = (($111) + ($110)|0);
         $113 = HEAP8[$112>>0]|0;
         $114 = $113 << 24 >> 24;
         $115 = ($114|0)==(0);
         $117 = $115;
        } else {
         $117 = 0;
        }
        $116 = $117&1;
        $118 = HEAP32[$4>>2]|0;
        HEAP32[$118>>2] = $116;
        $119 = HEAP32[$db>>2]|0;
        $120 = HEAP32[$z>>2]|0;
        $121 = (_sqlite3Expr($119,97,$120)|0);
        HEAP32[$pPrefix>>2] = $121;
        $122 = HEAP32[$pPrefix>>2]|0;
        $123 = ($122|0)!=(0|0);
        if ($123) {
         $124 = HEAP32[$cnt>>2]|0;
         $125 = HEAP32[$pPrefix>>2]|0;
         $126 = (($125) + 8|0);
         $127 = HEAP32[$126>>2]|0;
         $128 = (($127) + ($124)|0);
         HEAP8[$128>>0] = 0;
        }
        $129 = HEAP32[$pPrefix>>2]|0;
        $130 = HEAP32[$3>>2]|0;
        HEAP32[$130>>2] = $129;
        $131 = HEAP32[$op>>2]|0;
        $132 = ($131|0)==(135);
        if (!($132)) {
         break;
        }
        $133 = HEAP32[$1>>2]|0;
        $134 = (($133) + 8|0);
        $135 = HEAP32[$134>>2]|0;
        HEAP32[$v>>2] = $135;
        $136 = HEAP32[$v>>2]|0;
        $137 = HEAP32[$pRight>>2]|0;
        $138 = (($137) + 32|0);
        $139 = HEAP16[$138>>1]|0;
        $140 = $139 << 16 >> 16;
        _sqlite3VdbeSetVarmask($136,$140);
        $141 = HEAP32[$4>>2]|0;
        $142 = HEAP32[$141>>2]|0;
        $143 = ($142|0)!=(0);
        if (!($143)) {
         break;
        }
        $144 = HEAP32[$pRight>>2]|0;
        $145 = (($144) + 8|0);
        $146 = HEAP32[$145>>2]|0;
        $147 = (($146) + 1|0);
        $148 = HEAP8[$147>>0]|0;
        $149 = $148 << 24 >> 24;
        $150 = ($149|0)!=(0);
        if (!($150)) {
         break;
        }
        $151 = HEAP32[$1>>2]|0;
        $152 = (_sqlite3GetTempReg($151)|0);
        HEAP32[$r1>>2] = $152;
        $153 = HEAP32[$1>>2]|0;
        $154 = HEAP32[$pRight>>2]|0;
        $155 = HEAP32[$r1>>2]|0;
        (_sqlite3ExprCodeTarget($153,$154,$155)|0);
        $156 = HEAP32[$v>>2]|0;
        $157 = HEAP32[$v>>2]|0;
        $158 = (_sqlite3VdbeCurrentAddr($157)|0);
        $159 = (($158) - 1)|0;
        _sqlite3VdbeChangeP3($156,$159,0);
        $160 = HEAP32[$1>>2]|0;
        $161 = HEAP32[$r1>>2]|0;
        _sqlite3ReleaseTempReg($160,$161);
        break;
       }
      }
      HEAP32[$z>>2] = 0;
     }
    } while(0);
    $162 = HEAP32[$pVal>>2]|0;
    _sqlite3ValueFree($162);
    $163 = HEAP32[$z>>2]|0;
    $164 = ($163|0)!=(0|0);
    $165 = $164&1;
    HEAP32[$0>>2] = $165;
    $166 = HEAP32[$0>>2]|0;
    STACKTOP = sp;return ($166|0);
   }
  }
 }
 HEAP32[$0>>2] = 0;
 $166 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($166|0);
}
function _isMatchOfColumn($pExpr) {
 $pExpr = $pExpr|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $3 = 0;
 var $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $pList = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $1 = sp + 4|0;
 $pList = sp;
 HEAP32[$1>>2] = $pExpr;
 $2 = HEAP32[$1>>2]|0;
 $3 = HEAP8[$2>>0]|0;
 $4 = $3&255;
 $5 = ($4|0)!=(153);
 do {
  if ($5) {
   HEAP32[$0>>2] = 0;
  } else {
   $6 = HEAP32[$1>>2]|0;
   $7 = (($6) + 8|0);
   $8 = HEAP32[$7>>2]|0;
   $9 = (_sqlite3_stricmp($8,21512)|0);
   $10 = ($9|0)!=(0);
   if ($10) {
    HEAP32[$0>>2] = 0;
    break;
   }
   $11 = HEAP32[$1>>2]|0;
   $12 = (($11) + 20|0);
   $13 = HEAP32[$12>>2]|0;
   HEAP32[$pList>>2] = $13;
   $14 = HEAP32[$pList>>2]|0;
   $15 = HEAP32[$14>>2]|0;
   $16 = ($15|0)!=(2);
   if ($16) {
    HEAP32[$0>>2] = 0;
    break;
   }
   $17 = HEAP32[$pList>>2]|0;
   $18 = (($17) + 4|0);
   $19 = HEAP32[$18>>2]|0;
   $20 = (($19) + 20|0);
   $21 = HEAP32[$20>>2]|0;
   $22 = HEAP8[$21>>0]|0;
   $23 = $22&255;
   $24 = ($23|0)!=(154);
   if ($24) {
    HEAP32[$0>>2] = 0;
    break;
   } else {
    HEAP32[$0>>2] = 1;
    break;
   }
  }
 } while(0);
 $25 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($25|0);
}
function _sqlite3IsLikeFunction($db,$pExpr,$pIsNocase,$aWc) {
 $db = $db|0;
 $pExpr = $pExpr|0;
 $pIsNocase = $pIsNocase|0;
 $aWc = $aWc|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $pDef = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 20|0;
 $1 = sp + 16|0;
 $2 = sp + 12|0;
 $3 = sp + 8|0;
 $4 = sp + 4|0;
 $pDef = sp;
 HEAP32[$1>>2] = $db;
 HEAP32[$2>>2] = $pExpr;
 HEAP32[$3>>2] = $pIsNocase;
 HEAP32[$4>>2] = $aWc;
 $5 = HEAP32[$2>>2]|0;
 $6 = HEAP8[$5>>0]|0;
 $7 = $6&255;
 $8 = ($7|0)!=(153);
 if (!($8)) {
  $9 = HEAP32[$2>>2]|0;
  $10 = (($9) + 20|0);
  $11 = HEAP32[$10>>2]|0;
  $12 = ($11|0)!=(0|0);
  if ($12) {
   $13 = HEAP32[$2>>2]|0;
   $14 = (($13) + 20|0);
   $15 = HEAP32[$14>>2]|0;
   $16 = HEAP32[$15>>2]|0;
   $17 = ($16|0)!=(2);
   if (!($17)) {
    $18 = HEAP32[$1>>2]|0;
    $19 = HEAP32[$2>>2]|0;
    $20 = (($19) + 8|0);
    $21 = HEAP32[$20>>2]|0;
    $22 = HEAP32[$2>>2]|0;
    $23 = (($22) + 8|0);
    $24 = HEAP32[$23>>2]|0;
    $25 = (_sqlite3Strlen30($24)|0);
    $26 = (_sqlite3FindFunction($18,$21,$25,2,1,0)|0);
    HEAP32[$pDef>>2] = $26;
    $27 = HEAP32[$pDef>>2]|0;
    $28 = ($27|0)==(0|0);
    if (!($28)) {
     $29 = HEAP32[$pDef>>2]|0;
     $30 = (($29) + 2|0);
     $31 = HEAP16[$30>>1]|0;
     $32 = $31&65535;
     $33 = $32 & 4;
     $34 = ($33|0)==(0);
     if (!($34)) {
      $35 = HEAP32[$4>>2]|0;
      $36 = HEAP32[$pDef>>2]|0;
      $37 = (($36) + 4|0);
      $38 = HEAP32[$37>>2]|0;
      ;HEAP8[$35+0>>0]=HEAP8[$38+0>>0]|0;HEAP8[$35+1>>0]=HEAP8[$38+1>>0]|0;HEAP8[$35+2>>0]=HEAP8[$38+2>>0]|0;
      $39 = HEAP32[$pDef>>2]|0;
      $40 = (($39) + 2|0);
      $41 = HEAP16[$40>>1]|0;
      $42 = $41&65535;
      $43 = $42 & 8;
      $44 = ($43|0)==(0);
      $45 = $44&1;
      $46 = HEAP32[$3>>2]|0;
      HEAP32[$46>>2] = $45;
      HEAP32[$0>>2] = 1;
      $47 = HEAP32[$0>>2]|0;
      STACKTOP = sp;return ($47|0);
     }
    }
    HEAP32[$0>>2] = 0;
    $47 = HEAP32[$0>>2]|0;
    STACKTOP = sp;return ($47|0);
   }
  }
 }
 HEAP32[$0>>2] = 0;
 $47 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($47|0);
}
function _sqlite3VdbeGetBoundValue($v,$iVar,$aff) {
 $v = $v|0;
 $iVar = $iVar|0;
 $aff = $aff|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $pMem = 0, $pRet = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 16|0;
 $1 = sp + 12|0;
 $2 = sp + 8|0;
 $3 = sp + 20|0;
 $pMem = sp + 4|0;
 $pRet = sp;
 HEAP32[$1>>2] = $v;
 HEAP32[$2>>2] = $iVar;
 HEAP8[$3>>0] = $aff;
 $4 = HEAP32[$1>>2]|0;
 $5 = ($4|0)!=(0|0);
 if ($5) {
  $6 = HEAP32[$2>>2]|0;
  $7 = (($6) - 1)|0;
  $8 = HEAP32[$1>>2]|0;
  $9 = (($8) + 60|0);
  $10 = HEAP32[$9>>2]|0;
  $11 = (($10) + (($7*40)|0)|0);
  HEAP32[$pMem>>2] = $11;
  $12 = HEAP32[$pMem>>2]|0;
  $13 = (($12) + 8|0);
  $14 = HEAP16[$13>>1]|0;
  $15 = $14&65535;
  $16 = $15 & 1;
  $17 = (0)==($16|0);
  if ($17) {
   $18 = HEAP32[$1>>2]|0;
   $19 = HEAP32[$18>>2]|0;
   $20 = (_sqlite3ValueNew($19)|0);
   HEAP32[$pRet>>2] = $20;
   $21 = HEAP32[$pRet>>2]|0;
   $22 = ($21|0)!=(0|0);
   if ($22) {
    $23 = HEAP32[$pRet>>2]|0;
    $24 = HEAP32[$pMem>>2]|0;
    (_sqlite3VdbeMemCopy($23,$24)|0);
    $25 = HEAP32[$pRet>>2]|0;
    $26 = HEAP8[$3>>0]|0;
    _sqlite3ValueApplyAffinity($25,$26,1);
   }
   $27 = HEAP32[$pRet>>2]|0;
   HEAP32[$0>>2] = $27;
   $28 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($28|0);
  }
 }
 HEAP32[$0>>2] = 0;
 $28 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($28|0);
}
function _sqlite3VdbeSetVarmask($v,$iVar) {
 $v = $v|0;
 $iVar = $iVar|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $1 = sp;
 HEAP32[$0>>2] = $v;
 HEAP32[$1>>2] = $iVar;
 $2 = HEAP32[$1>>2]|0;
 $3 = ($2|0)>(32);
 if ($3) {
  $4 = HEAP32[$0>>2]|0;
  $5 = (($4) + 188|0);
  HEAP32[$5>>2] = -1;
  STACKTOP = sp;return;
 } else {
  $6 = HEAP32[$1>>2]|0;
  $7 = (($6) - 1)|0;
  $8 = 1 << $7;
  $9 = HEAP32[$0>>2]|0;
  $10 = (($9) + 188|0);
  $11 = HEAP32[$10>>2]|0;
  $12 = $11 | $8;
  HEAP32[$10>>2] = $12;
  STACKTOP = sp;return;
 }
}
function _sqlite3RowidConstraint($pParse,$onError,$pTab) {
 $pParse = $pParse|0;
 $onError = $onError|0;
 $pTab = $pTab|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $rc = 0, $vararg_buffer2 = 0, $vararg_ptr1 = 0, $zMsg = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $vararg_buffer2 = sp;
 $0 = sp + 24|0;
 $1 = sp + 20|0;
 $2 = sp + 16|0;
 $zMsg = sp + 12|0;
 $rc = sp + 8|0;
 HEAP32[$0>>2] = $pParse;
 HEAP32[$1>>2] = $onError;
 HEAP32[$2>>2] = $pTab;
 $3 = HEAP32[$2>>2]|0;
 $4 = (($3) + 36|0);
 $5 = HEAP16[$4>>1]|0;
 $6 = $5 << 16 >> 16;
 $7 = ($6|0)>=(0);
 $8 = HEAP32[$0>>2]|0;
 $9 = HEAP32[$8>>2]|0;
 $10 = HEAP32[$2>>2]|0;
 $11 = HEAP32[$10>>2]|0;
 if ($7) {
  $12 = HEAP32[$2>>2]|0;
  $13 = (($12) + 36|0);
  $14 = HEAP16[$13>>1]|0;
  $15 = $14 << 16 >> 16;
  $16 = HEAP32[$2>>2]|0;
  $17 = (($16) + 4|0);
  $18 = HEAP32[$17>>2]|0;
  $19 = (($18) + (($15*24)|0)|0);
  $20 = HEAP32[$19>>2]|0;
  HEAP32[$vararg_buffer2>>2] = $11;
  $vararg_ptr1 = (($vararg_buffer2) + 4|0);
  HEAP32[$vararg_ptr1>>2] = $20;
  $21 = (_sqlite3MPrintf($9,17448,$vararg_buffer2)|0);
  HEAP32[$zMsg>>2] = $21;
  HEAP32[$rc>>2] = 1555;
  $23 = HEAP32[$0>>2]|0;
  $24 = HEAP32[$rc>>2]|0;
  $25 = HEAP32[$1>>2]|0;
  $26 = HEAP32[$zMsg>>2]|0;
  _sqlite3HaltConstraint($23,$24,$25,$26,-1,2);
  STACKTOP = sp;return;
 } else {
  HEAP32[$vararg_buffer2>>2] = $11;
  $22 = (_sqlite3MPrintf($9,24432,$vararg_buffer2)|0);
  HEAP32[$zMsg>>2] = $22;
  HEAP32[$rc>>2] = 2579;
  $23 = HEAP32[$0>>2]|0;
  $24 = HEAP32[$rc>>2]|0;
  $25 = HEAP32[$1>>2]|0;
  $26 = HEAP32[$zMsg>>2]|0;
  _sqlite3HaltConstraint($23,$24,$25,$26,-1,2);
  STACKTOP = sp;return;
 }
}
function _sqlite3FkRequired($pParse,$pTab,$aChange,$chngRowid) {
 $pParse = $pParse|0;
 $pTab = $pTab|0;
 $aChange = $aChange|0;
 $chngRowid = $chngRowid|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $p = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 20|0;
 $1 = sp + 16|0;
 $2 = sp + 12|0;
 $3 = sp + 8|0;
 $4 = sp + 4|0;
 $p = sp;
 HEAP32[$1>>2] = $pParse;
 HEAP32[$2>>2] = $pTab;
 HEAP32[$3>>2] = $aChange;
 HEAP32[$4>>2] = $chngRowid;
 $5 = HEAP32[$1>>2]|0;
 $6 = HEAP32[$5>>2]|0;
 $7 = (($6) + 24|0);
 $8 = HEAP32[$7>>2]|0;
 $9 = $8 & 524288;
 $10 = ($9|0)!=(0);
 L1: do {
  if ($10) {
   $11 = HEAP32[$3>>2]|0;
   $12 = ($11|0)!=(0|0);
   $13 = HEAP32[$2>>2]|0;
   if (!($12)) {
    $14 = (_sqlite3FkReferences($13)|0);
    $15 = ($14|0)!=(0|0);
    if ($15) {
     $21 = 1;
    } else {
     $16 = HEAP32[$2>>2]|0;
     $17 = (($16) + 16|0);
     $18 = HEAP32[$17>>2]|0;
     $19 = ($18|0)!=(0|0);
     $21 = $19;
    }
    $20 = $21&1;
    HEAP32[$0>>2] = $20;
    $47 = HEAP32[$0>>2]|0;
    STACKTOP = sp;return ($47|0);
   }
   $22 = (($13) + 16|0);
   $23 = HEAP32[$22>>2]|0;
   HEAP32[$p>>2] = $23;
   while(1) {
    $24 = HEAP32[$p>>2]|0;
    $25 = ($24|0)!=(0|0);
    $26 = HEAP32[$2>>2]|0;
    if (!($25)) {
     break;
    }
    $27 = HEAP32[$p>>2]|0;
    $28 = HEAP32[$3>>2]|0;
    $29 = HEAP32[$4>>2]|0;
    $30 = (_fkChildIsModified($26,$27,$28,$29)|0);
    $31 = ($30|0)!=(0);
    if ($31) {
     label = 9;
     break;
    }
    $32 = HEAP32[$p>>2]|0;
    $33 = (($32) + 4|0);
    $34 = HEAP32[$33>>2]|0;
    HEAP32[$p>>2] = $34;
   }
   if ((label|0) == 9) {
    HEAP32[$0>>2] = 1;
    $47 = HEAP32[$0>>2]|0;
    STACKTOP = sp;return ($47|0);
   }
   $35 = (_sqlite3FkReferences($26)|0);
   HEAP32[$p>>2] = $35;
   while(1) {
    $36 = HEAP32[$p>>2]|0;
    $37 = ($36|0)!=(0|0);
    if (!($37)) {
     break L1;
    }
    $38 = HEAP32[$2>>2]|0;
    $39 = HEAP32[$p>>2]|0;
    $40 = HEAP32[$3>>2]|0;
    $41 = HEAP32[$4>>2]|0;
    $42 = (_fkParentIsModified($38,$39,$40,$41)|0);
    $43 = ($42|0)!=(0);
    if ($43) {
     break;
    }
    $44 = HEAP32[$p>>2]|0;
    $45 = (($44) + 12|0);
    $46 = HEAP32[$45>>2]|0;
    HEAP32[$p>>2] = $46;
   }
   HEAP32[$0>>2] = 1;
   $47 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($47|0);
  }
 } while(0);
 HEAP32[$0>>2] = 0;
 $47 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($47|0);
}
function _sqlite3MultiWrite($pParse) {
 $pParse = $pParse|0;
 var $0 = 0, $1 = 0, $10 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $pToplevel = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $pToplevel = sp;
 HEAP32[$0>>2] = $pParse;
 $1 = HEAP32[$0>>2]|0;
 $2 = (($1) + 408|0);
 $3 = HEAP32[$2>>2]|0;
 $4 = ($3|0)!=(0|0);
 $5 = HEAP32[$0>>2]|0;
 if ($4) {
  $6 = (($5) + 408|0);
  $7 = HEAP32[$6>>2]|0;
  $8 = $7;
 } else {
  $8 = $5;
 }
 HEAP32[$pToplevel>>2] = $8;
 $9 = HEAP32[$pToplevel>>2]|0;
 $10 = (($9) + 20|0);
 HEAP8[$10>>0] = 1;
 STACKTOP = sp;return;
}
function _sqlite3GenerateRowDelete($pParse,$pTab,$pTrigger,$iDataCur,$iIdxCur,$iPk,$nPk,$count,$onconf,$bNoSeek) {
 $pParse = $pParse|0;
 $pTab = $pTab|0;
 $pTrigger = $pTrigger|0;
 $iDataCur = $iDataCur|0;
 $iIdxCur = $iIdxCur|0;
 $iPk = $iPk|0;
 $nPk = $nPk|0;
 $count = $count|0;
 $onconf = $onconf|0;
 $bNoSeek = $bNoSeek|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0;
 var $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0;
 var $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0;
 var $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0;
 var $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0;
 var $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $addrStart = 0, $iCol = 0, $iLabel = 0, $iOld = 0, $mask = 0, $opSeek = 0, $or$cond = 0, $v = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 64|0;
 $0 = sp + 44|0;
 $1 = sp + 40|0;
 $2 = sp + 36|0;
 $3 = sp + 32|0;
 $4 = sp + 28|0;
 $5 = sp + 24|0;
 $6 = sp + 48|0;
 $7 = sp + 53|0;
 $8 = sp + 52|0;
 $9 = sp + 51|0;
 $v = sp + 20|0;
 $iOld = sp + 16|0;
 $iLabel = sp + 12|0;
 $opSeek = sp + 50|0;
 $mask = sp + 8|0;
 $iCol = sp + 4|0;
 $addrStart = sp;
 HEAP32[$0>>2] = $pParse;
 HEAP32[$1>>2] = $pTab;
 HEAP32[$2>>2] = $pTrigger;
 HEAP32[$3>>2] = $iDataCur;
 HEAP32[$4>>2] = $iIdxCur;
 HEAP32[$5>>2] = $iPk;
 HEAP16[$6>>1] = $nPk;
 HEAP8[$7>>0] = $count;
 HEAP8[$8>>0] = $onconf;
 HEAP8[$9>>0] = $bNoSeek;
 $10 = HEAP32[$0>>2]|0;
 $11 = (($10) + 8|0);
 $12 = HEAP32[$11>>2]|0;
 HEAP32[$v>>2] = $12;
 HEAP32[$iOld>>2] = 0;
 $13 = HEAP32[$v>>2]|0;
 $14 = (_sqlite3VdbeMakeLabel($13)|0);
 HEAP32[$iLabel>>2] = $14;
 $15 = HEAP32[$1>>2]|0;
 $16 = (($15) + 44|0);
 $17 = HEAP8[$16>>0]|0;
 $18 = $17&255;
 $19 = $18 & 32;
 $20 = ($19|0)==(0);
 $21 = $20 ? 70 : 68;
 $22 = $21&255;
 HEAP8[$opSeek>>0] = $22;
 $23 = HEAP8[$9>>0]|0;
 $24 = ($23<<24>>24)!=(0);
 if (!($24)) {
  $25 = HEAP32[$v>>2]|0;
  $26 = HEAP8[$opSeek>>0]|0;
  $27 = $26&255;
  $28 = HEAP32[$3>>2]|0;
  $29 = HEAP32[$iLabel>>2]|0;
  $30 = HEAP32[$5>>2]|0;
  $31 = HEAP16[$6>>1]|0;
  $32 = $31 << 16 >> 16;
  (_sqlite3VdbeAddOp4Int($25,$27,$28,$29,$30,$32)|0);
 }
 $33 = HEAP32[$0>>2]|0;
 $34 = HEAP32[$1>>2]|0;
 $35 = (_sqlite3FkRequired($33,$34,0,0)|0);
 $36 = ($35|0)!=(0);
 $37 = HEAP32[$2>>2]|0;
 $38 = ($37|0)!=(0|0);
 $or$cond = $36 | $38;
 if ($or$cond) {
  $39 = HEAP32[$0>>2]|0;
  $40 = HEAP32[$2>>2]|0;
  $41 = HEAP32[$1>>2]|0;
  $42 = HEAP8[$8>>0]|0;
  $43 = $42&255;
  $44 = (_sqlite3TriggerColmask($39,$40,0,0,3,$41,$43)|0);
  HEAP32[$mask>>2] = $44;
  $45 = HEAP32[$0>>2]|0;
  $46 = HEAP32[$1>>2]|0;
  $47 = (_sqlite3FkOldmask($45,$46)|0);
  $48 = HEAP32[$mask>>2]|0;
  $49 = $48 | $47;
  HEAP32[$mask>>2] = $49;
  $50 = HEAP32[$0>>2]|0;
  $51 = (($50) + 72|0);
  $52 = HEAP32[$51>>2]|0;
  $53 = (($52) + 1)|0;
  HEAP32[$iOld>>2] = $53;
  $54 = HEAP32[$1>>2]|0;
  $55 = (($54) + 38|0);
  $56 = HEAP16[$55>>1]|0;
  $57 = $56 << 16 >> 16;
  $58 = (1 + ($57))|0;
  $59 = HEAP32[$0>>2]|0;
  $60 = (($59) + 72|0);
  $61 = HEAP32[$60>>2]|0;
  $62 = (($61) + ($58))|0;
  HEAP32[$60>>2] = $62;
  $63 = HEAP32[$v>>2]|0;
  $64 = HEAP32[$5>>2]|0;
  $65 = HEAP32[$iOld>>2]|0;
  (_sqlite3VdbeAddOp2($63,33,$64,$65)|0);
  HEAP32[$iCol>>2] = 0;
  while(1) {
   $66 = HEAP32[$iCol>>2]|0;
   $67 = HEAP32[$1>>2]|0;
   $68 = (($67) + 38|0);
   $69 = HEAP16[$68>>1]|0;
   $70 = $69 << 16 >> 16;
   $71 = ($66|0)<($70|0);
   if (!($71)) {
    break;
   }
   $72 = HEAP32[$mask>>2]|0;
   $73 = ($72|0)==(-1);
   if ($73) {
    label = 9;
   } else {
    $74 = HEAP32[$iCol>>2]|0;
    $75 = ($74|0)<=(31);
    if ($75) {
     $76 = HEAP32[$mask>>2]|0;
     $77 = HEAP32[$iCol>>2]|0;
     $78 = 1 << $77;
     $79 = $76 & $78;
     $80 = ($79|0)!=(0);
     if ($80) {
      label = 9;
     }
    }
   }
   if ((label|0) == 9) {
    label = 0;
    $81 = HEAP32[$v>>2]|0;
    $82 = HEAP32[$1>>2]|0;
    $83 = HEAP32[$3>>2]|0;
    $84 = HEAP32[$iCol>>2]|0;
    $85 = HEAP32[$iOld>>2]|0;
    $86 = HEAP32[$iCol>>2]|0;
    $87 = (($85) + ($86))|0;
    $88 = (($87) + 1)|0;
    _sqlite3ExprCodeGetColumnOfTable($81,$82,$83,$84,$88);
   }
   $89 = HEAP32[$iCol>>2]|0;
   $90 = (($89) + 1)|0;
   HEAP32[$iCol>>2] = $90;
  }
  $91 = HEAP32[$v>>2]|0;
  $92 = (_sqlite3VdbeCurrentAddr($91)|0);
  HEAP32[$addrStart>>2] = $92;
  $93 = HEAP32[$0>>2]|0;
  $94 = HEAP32[$2>>2]|0;
  $95 = HEAP32[$1>>2]|0;
  $96 = HEAP32[$iOld>>2]|0;
  $97 = HEAP8[$8>>0]|0;
  $98 = $97&255;
  $99 = HEAP32[$iLabel>>2]|0;
  _sqlite3CodeRowTrigger($93,$94,109,0,1,$95,$96,$98,$99);
  $100 = HEAP32[$addrStart>>2]|0;
  $101 = HEAP32[$v>>2]|0;
  $102 = (_sqlite3VdbeCurrentAddr($101)|0);
  $103 = ($100|0)<($102|0);
  if ($103) {
   $104 = HEAP32[$v>>2]|0;
   $105 = HEAP8[$opSeek>>0]|0;
   $106 = $105&255;
   $107 = HEAP32[$3>>2]|0;
   $108 = HEAP32[$iLabel>>2]|0;
   $109 = HEAP32[$5>>2]|0;
   $110 = HEAP16[$6>>1]|0;
   $111 = $110 << 16 >> 16;
   (_sqlite3VdbeAddOp4Int($104,$106,$107,$108,$109,$111)|0);
  }
  $112 = HEAP32[$0>>2]|0;
  $113 = HEAP32[$1>>2]|0;
  $114 = HEAP32[$iOld>>2]|0;
  _sqlite3FkCheck($112,$113,$114,0,0,0);
 }
 $115 = HEAP32[$1>>2]|0;
 $116 = (($115) + 12|0);
 $117 = HEAP32[$116>>2]|0;
 $118 = ($117|0)==(0|0);
 if (!($118)) {
  $134 = HEAP32[$0>>2]|0;
  $135 = HEAP32[$1>>2]|0;
  $136 = HEAP32[$iOld>>2]|0;
  _sqlite3FkActions($134,$135,0,$136,0,0);
  $137 = HEAP32[$0>>2]|0;
  $138 = HEAP32[$2>>2]|0;
  $139 = HEAP32[$1>>2]|0;
  $140 = HEAP32[$iOld>>2]|0;
  $141 = HEAP8[$8>>0]|0;
  $142 = $141&255;
  $143 = HEAP32[$iLabel>>2]|0;
  _sqlite3CodeRowTrigger($137,$138,109,0,2,$139,$140,$142,$143);
  $144 = HEAP32[$v>>2]|0;
  $145 = HEAP32[$iLabel>>2]|0;
  _sqlite3VdbeResolveLabel($144,$145);
  STACKTOP = sp;return;
 }
 $119 = HEAP32[$0>>2]|0;
 $120 = HEAP32[$1>>2]|0;
 $121 = HEAP32[$3>>2]|0;
 $122 = HEAP32[$4>>2]|0;
 _sqlite3GenerateRowIndexDelete($119,$120,$121,$122,0);
 $123 = HEAP32[$v>>2]|0;
 $124 = HEAP32[$3>>2]|0;
 $125 = HEAP8[$7>>0]|0;
 $126 = $125&255;
 $127 = ($126|0)!=(0);
 $128 = $127 ? 1 : 0;
 (_sqlite3VdbeAddOp2($123,95,$124,$128)|0);
 $129 = HEAP8[$7>>0]|0;
 $130 = ($129<<24>>24)!=(0);
 if (!($130)) {
  $134 = HEAP32[$0>>2]|0;
  $135 = HEAP32[$1>>2]|0;
  $136 = HEAP32[$iOld>>2]|0;
  _sqlite3FkActions($134,$135,0,$136,0,0);
  $137 = HEAP32[$0>>2]|0;
  $138 = HEAP32[$2>>2]|0;
  $139 = HEAP32[$1>>2]|0;
  $140 = HEAP32[$iOld>>2]|0;
  $141 = HEAP8[$8>>0]|0;
  $142 = $141&255;
  $143 = HEAP32[$iLabel>>2]|0;
  _sqlite3CodeRowTrigger($137,$138,109,0,2,$139,$140,$142,$143);
  $144 = HEAP32[$v>>2]|0;
  $145 = HEAP32[$iLabel>>2]|0;
  _sqlite3VdbeResolveLabel($144,$145);
  STACKTOP = sp;return;
 }
 $131 = HEAP32[$v>>2]|0;
 $132 = HEAP32[$1>>2]|0;
 $133 = HEAP32[$132>>2]|0;
 _sqlite3VdbeChangeP4($131,-1,$133,0);
 $134 = HEAP32[$0>>2]|0;
 $135 = HEAP32[$1>>2]|0;
 $136 = HEAP32[$iOld>>2]|0;
 _sqlite3FkActions($134,$135,0,$136,0,0);
 $137 = HEAP32[$0>>2]|0;
 $138 = HEAP32[$2>>2]|0;
 $139 = HEAP32[$1>>2]|0;
 $140 = HEAP32[$iOld>>2]|0;
 $141 = HEAP8[$8>>0]|0;
 $142 = $141&255;
 $143 = HEAP32[$iLabel>>2]|0;
 _sqlite3CodeRowTrigger($137,$138,109,0,2,$139,$140,$142,$143);
 $144 = HEAP32[$v>>2]|0;
 $145 = HEAP32[$iLabel>>2]|0;
 _sqlite3VdbeResolveLabel($144,$145);
 STACKTOP = sp;return;
}
function _sqlite3GenerateRowIndexDelete($pParse,$pTab,$iDataCur,$iIdxCur,$aRegIdx) {
 $pParse = $pParse|0;
 $pTab = $pTab|0;
 $iDataCur = $iDataCur|0;
 $iIdxCur = $iIdxCur|0;
 $aRegIdx = $aRegIdx|0;
 var $$sink = 0, $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0;
 var $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0;
 var $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0;
 var $62 = 0, $63 = 0, $7 = 0, $8 = 0, $9 = 0, $i = 0, $iPartIdxLabel = 0, $pIdx = 0, $pPk = 0, $pPrior = 0, $r1 = 0, $v = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $0 = sp + 44|0;
 $1 = sp + 40|0;
 $2 = sp + 36|0;
 $3 = sp + 32|0;
 $4 = sp + 28|0;
 $i = sp + 24|0;
 $r1 = sp + 20|0;
 $iPartIdxLabel = sp + 16|0;
 $pIdx = sp + 12|0;
 $pPrior = sp + 8|0;
 $v = sp + 4|0;
 $pPk = sp;
 HEAP32[$0>>2] = $pParse;
 HEAP32[$1>>2] = $pTab;
 HEAP32[$2>>2] = $iDataCur;
 HEAP32[$3>>2] = $iIdxCur;
 HEAP32[$4>>2] = $aRegIdx;
 HEAP32[$r1>>2] = -1;
 HEAP32[$pPrior>>2] = 0;
 $5 = HEAP32[$0>>2]|0;
 $6 = (($5) + 8|0);
 $7 = HEAP32[$6>>2]|0;
 HEAP32[$v>>2] = $7;
 $8 = HEAP32[$1>>2]|0;
 $9 = (($8) + 44|0);
 $10 = HEAP8[$9>>0]|0;
 $11 = $10&255;
 $12 = $11 & 32;
 $13 = ($12|0)==(0);
 if ($13) {
  $16 = 0;
 } else {
  $14 = HEAP32[$1>>2]|0;
  $15 = (_sqlite3PrimaryKeyIndex($14)|0);
  $16 = $15;
 }
 HEAP32[$pPk>>2] = $16;
 HEAP32[$i>>2] = 0;
 $17 = HEAP32[$1>>2]|0;
 $18 = (($17) + 8|0);
 $19 = HEAP32[$18>>2]|0;
 HEAP32[$pIdx>>2] = $19;
 while(1) {
  $20 = HEAP32[$pIdx>>2]|0;
  $21 = ($20|0)!=(0|0);
  if (!($21)) {
   break;
  }
  $22 = HEAP32[$4>>2]|0;
  $23 = ($22|0)!=(0|0);
  if ($23) {
   $24 = HEAP32[$i>>2]|0;
   $25 = HEAP32[$4>>2]|0;
   $26 = (($25) + ($24<<2)|0);
   $27 = HEAP32[$26>>2]|0;
   $28 = ($27|0)==(0);
   if (!($28)) {
    label = 7;
   }
  } else {
   label = 7;
  }
  if ((label|0) == 7) {
   label = 0;
   $29 = HEAP32[$pIdx>>2]|0;
   $30 = HEAP32[$pPk>>2]|0;
   $31 = ($29|0)==($30|0);
   if (!($31)) {
    $32 = HEAP32[$0>>2]|0;
    $33 = HEAP32[$pIdx>>2]|0;
    $34 = HEAP32[$2>>2]|0;
    $35 = HEAP32[$pPrior>>2]|0;
    $36 = HEAP32[$r1>>2]|0;
    $37 = (_sqlite3GenerateIndexKey($32,$33,$34,0,1,$iPartIdxLabel,$35,$36)|0);
    HEAP32[$r1>>2] = $37;
    $38 = HEAP32[$v>>2]|0;
    $39 = HEAP32[$3>>2]|0;
    $40 = HEAP32[$i>>2]|0;
    $41 = (($39) + ($40))|0;
    $42 = HEAP32[$r1>>2]|0;
    $43 = HEAP32[$pIdx>>2]|0;
    $44 = (($43) + 55|0);
    $45 = HEAP8[$44>>0]|0;
    $46 = ($45&255) >>> 3;
    $47 = $46 & 1;
    $48 = $47&255;
    $49 = ($48|0)!=(0);
    $50 = HEAP32[$pIdx>>2]|0;
    if ($49) {
     $51 = (($50) + 50|0);
     $52 = HEAP16[$51>>1]|0;
     $$sink = $52;
    } else {
     $53 = (($50) + 52|0);
     $54 = HEAP16[$53>>1]|0;
     $$sink = $54;
    }
    $55 = $$sink&65535;
    (_sqlite3VdbeAddOp3($38,111,$41,$42,$55)|0);
    $56 = HEAP32[$0>>2]|0;
    $57 = HEAP32[$iPartIdxLabel>>2]|0;
    _sqlite3ResolvePartIdxLabel($56,$57);
    $58 = HEAP32[$pIdx>>2]|0;
    HEAP32[$pPrior>>2] = $58;
   }
  }
  $59 = HEAP32[$i>>2]|0;
  $60 = (($59) + 1)|0;
  HEAP32[$i>>2] = $60;
  $61 = HEAP32[$pIdx>>2]|0;
  $62 = (($61) + 20|0);
  $63 = HEAP32[$62>>2]|0;
  HEAP32[$pIdx>>2] = $63;
 }
 STACKTOP = sp;return;
}
function _sqlite3TriggerColmask($pParse,$pTrigger,$pChanges,$isNew,$tr_tm,$pTab,$orconf) {
 $pParse = $pParse|0;
 $pTrigger = $pTrigger|0;
 $pChanges = $pChanges|0;
 $isNew = $isNew|0;
 $tr_tm = $tr_tm|0;
 $pTab = $pTab|0;
 $orconf = $orconf|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $mask = 0, $op = 0, $p = 0, $pPrg = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $0 = sp + 40|0;
 $1 = sp + 36|0;
 $2 = sp + 32|0;
 $3 = sp + 28|0;
 $4 = sp + 24|0;
 $5 = sp + 20|0;
 $6 = sp + 16|0;
 $op = sp + 12|0;
 $mask = sp + 8|0;
 $p = sp + 4|0;
 $pPrg = sp;
 HEAP32[$0>>2] = $pParse;
 HEAP32[$1>>2] = $pTrigger;
 HEAP32[$2>>2] = $pChanges;
 HEAP32[$3>>2] = $isNew;
 HEAP32[$4>>2] = $tr_tm;
 HEAP32[$5>>2] = $pTab;
 HEAP32[$6>>2] = $orconf;
 $7 = HEAP32[$2>>2]|0;
 $8 = ($7|0)!=(0|0);
 $9 = $8 ? 110 : 109;
 HEAP32[$op>>2] = $9;
 HEAP32[$mask>>2] = 0;
 $10 = HEAP32[$1>>2]|0;
 HEAP32[$p>>2] = $10;
 while(1) {
  $11 = HEAP32[$p>>2]|0;
  $12 = ($11|0)!=(0|0);
  if (!($12)) {
   break;
  }
  $13 = HEAP32[$p>>2]|0;
  $14 = (($13) + 8|0);
  $15 = HEAP8[$14>>0]|0;
  $16 = $15&255;
  $17 = HEAP32[$op>>2]|0;
  $18 = ($16|0)==($17|0);
  if ($18) {
   $19 = HEAP32[$4>>2]|0;
   $20 = HEAP32[$p>>2]|0;
   $21 = (($20) + 9|0);
   $22 = HEAP8[$21>>0]|0;
   $23 = $22&255;
   $24 = $19 & $23;
   $25 = ($24|0)!=(0);
   if ($25) {
    $26 = HEAP32[$p>>2]|0;
    $27 = (($26) + 16|0);
    $28 = HEAP32[$27>>2]|0;
    $29 = HEAP32[$2>>2]|0;
    $30 = (_checkColumnOverlap($28,$29)|0);
    $31 = ($30|0)!=(0);
    if ($31) {
     $32 = HEAP32[$0>>2]|0;
     $33 = HEAP32[$p>>2]|0;
     $34 = HEAP32[$5>>2]|0;
     $35 = HEAP32[$6>>2]|0;
     $36 = (_getRowTrigger($32,$33,$34,$35)|0);
     HEAP32[$pPrg>>2] = $36;
     $37 = HEAP32[$pPrg>>2]|0;
     $38 = ($37|0)!=(0|0);
     if ($38) {
      $39 = HEAP32[$3>>2]|0;
      $40 = HEAP32[$pPrg>>2]|0;
      $41 = (($40) + 16|0);
      $42 = (($41) + ($39<<2)|0);
      $43 = HEAP32[$42>>2]|0;
      $44 = HEAP32[$mask>>2]|0;
      $45 = $44 | $43;
      HEAP32[$mask>>2] = $45;
     }
    }
   }
  }
  $46 = HEAP32[$p>>2]|0;
  $47 = (($46) + 32|0);
  $48 = HEAP32[$47>>2]|0;
  HEAP32[$p>>2] = $48;
 }
 $49 = HEAP32[$mask>>2]|0;
 STACKTOP = sp;return ($49|0);
}
function _sqlite3FkOldmask($pParse,$pTab) {
 $pParse = $pParse|0;
 $pTab = $pTab|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $8 = 0, $9 = 0, $i = 0, $mask = 0;
 var $p = 0, $pIdx = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 20|0;
 $1 = sp + 16|0;
 $mask = sp + 12|0;
 $p = sp + 8|0;
 $i = sp + 4|0;
 $pIdx = sp;
 HEAP32[$0>>2] = $pParse;
 HEAP32[$1>>2] = $pTab;
 HEAP32[$mask>>2] = 0;
 $2 = HEAP32[$0>>2]|0;
 $3 = HEAP32[$2>>2]|0;
 $4 = (($3) + 24|0);
 $5 = HEAP32[$4>>2]|0;
 $6 = $5 & 524288;
 $7 = ($6|0)!=(0);
 if (!($7)) {
  $77 = HEAP32[$mask>>2]|0;
  STACKTOP = sp;return ($77|0);
 }
 $8 = HEAP32[$1>>2]|0;
 $9 = (($8) + 16|0);
 $10 = HEAP32[$9>>2]|0;
 HEAP32[$p>>2] = $10;
 while(1) {
  $11 = HEAP32[$p>>2]|0;
  $12 = ($11|0)!=(0|0);
  if (!($12)) {
   break;
  }
  HEAP32[$i>>2] = 0;
  while(1) {
   $13 = HEAP32[$i>>2]|0;
   $14 = HEAP32[$p>>2]|0;
   $15 = (($14) + 20|0);
   $16 = HEAP32[$15>>2]|0;
   $17 = ($13|0)<($16|0);
   if (!($17)) {
    break;
   }
   $18 = HEAP32[$i>>2]|0;
   $19 = HEAP32[$p>>2]|0;
   $20 = (($19) + 36|0);
   $21 = (($20) + ($18<<3)|0);
   $22 = HEAP32[$21>>2]|0;
   $23 = ($22|0)>(31);
   if ($23) {
    $32 = -1;
   } else {
    $24 = HEAP32[$i>>2]|0;
    $25 = HEAP32[$p>>2]|0;
    $26 = (($25) + 36|0);
    $27 = (($26) + ($24<<3)|0);
    $28 = HEAP32[$27>>2]|0;
    $29 = 1 << $28;
    $32 = $29;
   }
   $30 = HEAP32[$mask>>2]|0;
   $31 = $30 | $32;
   HEAP32[$mask>>2] = $31;
   $33 = HEAP32[$i>>2]|0;
   $34 = (($33) + 1)|0;
   HEAP32[$i>>2] = $34;
  }
  $35 = HEAP32[$p>>2]|0;
  $36 = (($35) + 4|0);
  $37 = HEAP32[$36>>2]|0;
  HEAP32[$p>>2] = $37;
 }
 $38 = HEAP32[$1>>2]|0;
 $39 = (_sqlite3FkReferences($38)|0);
 HEAP32[$p>>2] = $39;
 while(1) {
  $40 = HEAP32[$p>>2]|0;
  $41 = ($40|0)!=(0|0);
  if (!($41)) {
   break;
  }
  HEAP32[$pIdx>>2] = 0;
  $42 = HEAP32[$0>>2]|0;
  $43 = HEAP32[$1>>2]|0;
  $44 = HEAP32[$p>>2]|0;
  (_sqlite3FkLocateIndex($42,$43,$44,$pIdx,0)|0);
  $45 = HEAP32[$pIdx>>2]|0;
  $46 = ($45|0)!=(0|0);
  L18: do {
   if ($46) {
    HEAP32[$i>>2] = 0;
    while(1) {
     $47 = HEAP32[$i>>2]|0;
     $48 = HEAP32[$pIdx>>2]|0;
     $49 = (($48) + 50|0);
     $50 = HEAP16[$49>>1]|0;
     $51 = $50&65535;
     $52 = ($47|0)<($51|0);
     if (!($52)) {
      break L18;
     }
     $53 = HEAP32[$i>>2]|0;
     $54 = HEAP32[$pIdx>>2]|0;
     $55 = (($54) + 4|0);
     $56 = HEAP32[$55>>2]|0;
     $57 = (($56) + ($53<<1)|0);
     $58 = HEAP16[$57>>1]|0;
     $59 = $58 << 16 >> 16;
     $60 = ($59|0)>(31);
     if ($60) {
      $71 = -1;
     } else {
      $61 = HEAP32[$i>>2]|0;
      $62 = HEAP32[$pIdx>>2]|0;
      $63 = (($62) + 4|0);
      $64 = HEAP32[$63>>2]|0;
      $65 = (($64) + ($61<<1)|0);
      $66 = HEAP16[$65>>1]|0;
      $67 = $66 << 16 >> 16;
      $68 = 1 << $67;
      $71 = $68;
     }
     $69 = HEAP32[$mask>>2]|0;
     $70 = $69 | $71;
     HEAP32[$mask>>2] = $70;
     $72 = HEAP32[$i>>2]|0;
     $73 = (($72) + 1)|0;
     HEAP32[$i>>2] = $73;
    }
   }
  } while(0);
  $74 = HEAP32[$p>>2]|0;
  $75 = (($74) + 12|0);
  $76 = HEAP32[$75>>2]|0;
  HEAP32[$p>>2] = $76;
 }
 $77 = HEAP32[$mask>>2]|0;
 STACKTOP = sp;return ($77|0);
}
function _sqlite3FkActions($pParse,$pTab,$pChanges,$regOld,$aChange,$bChngRowid) {
 $pParse = $pParse|0;
 $pTab = $pTab|0;
 $pChanges = $pChanges|0;
 $regOld = $regOld|0;
 $aChange = $aChange|0;
 $bChngRowid = $bChngRowid|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $pAct = 0, $pFKey = 0;
 var label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 28|0;
 $1 = sp + 24|0;
 $2 = sp + 20|0;
 $3 = sp + 16|0;
 $4 = sp + 12|0;
 $5 = sp + 8|0;
 $pFKey = sp + 4|0;
 $pAct = sp;
 HEAP32[$0>>2] = $pParse;
 HEAP32[$1>>2] = $pTab;
 HEAP32[$2>>2] = $pChanges;
 HEAP32[$3>>2] = $regOld;
 HEAP32[$4>>2] = $aChange;
 HEAP32[$5>>2] = $bChngRowid;
 $6 = HEAP32[$0>>2]|0;
 $7 = HEAP32[$6>>2]|0;
 $8 = (($7) + 24|0);
 $9 = HEAP32[$8>>2]|0;
 $10 = $9 & 524288;
 $11 = ($10|0)!=(0);
 if (!($11)) {
  STACKTOP = sp;return;
 }
 $12 = HEAP32[$1>>2]|0;
 $13 = (_sqlite3FkReferences($12)|0);
 HEAP32[$pFKey>>2] = $13;
 while(1) {
  $14 = HEAP32[$pFKey>>2]|0;
  $15 = ($14|0)!=(0|0);
  if (!($15)) {
   break;
  }
  $16 = HEAP32[$4>>2]|0;
  $17 = ($16|0)==(0|0);
  if ($17) {
   label = 6;
  } else {
   $18 = HEAP32[$1>>2]|0;
   $19 = HEAP32[$pFKey>>2]|0;
   $20 = HEAP32[$4>>2]|0;
   $21 = HEAP32[$5>>2]|0;
   $22 = (_fkParentIsModified($18,$19,$20,$21)|0);
   $23 = ($22|0)!=(0);
   if ($23) {
    label = 6;
   }
  }
  if ((label|0) == 6) {
   label = 0;
   $24 = HEAP32[$0>>2]|0;
   $25 = HEAP32[$1>>2]|0;
   $26 = HEAP32[$pFKey>>2]|0;
   $27 = HEAP32[$2>>2]|0;
   $28 = (_fkActionTrigger($24,$25,$26,$27)|0);
   HEAP32[$pAct>>2] = $28;
   $29 = HEAP32[$pAct>>2]|0;
   $30 = ($29|0)!=(0|0);
   if ($30) {
    $31 = HEAP32[$0>>2]|0;
    $32 = HEAP32[$pAct>>2]|0;
    $33 = HEAP32[$1>>2]|0;
    $34 = HEAP32[$3>>2]|0;
    _sqlite3CodeRowTriggerDirect($31,$32,$33,$34,2,0);
   }
  }
  $35 = HEAP32[$pFKey>>2]|0;
  $36 = (($35) + 12|0);
  $37 = HEAP32[$36>>2]|0;
  HEAP32[$pFKey>>2] = $37;
 }
 STACKTOP = sp;return;
}
function _fkActionTrigger($pParse,$pTab,$pFKey,$pChanges) {
 $pParse = $pParse|0;
 $pTab = $pTab|0;
 $pFKey = $pFKey|0;
 $pChanges = $pChanges|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0;
 var $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0, $188 = 0;
 var $189 = 0, $19 = 0, $190 = 0, $191 = 0, $192 = 0, $193 = 0, $194 = 0, $195 = 0, $196 = 0, $197 = 0, $198 = 0, $199 = 0, $2 = 0, $20 = 0, $200 = 0, $201 = 0, $202 = 0, $203 = 0, $204 = 0, $205 = 0;
 var $206 = 0, $207 = 0, $208 = 0, $209 = 0, $21 = 0, $210 = 0, $211 = 0, $212 = 0, $213 = 0, $214 = 0, $215 = 0, $216 = 0, $217 = 0, $218 = 0, $219 = 0, $22 = 0, $220 = 0, $221 = 0, $222 = 0, $223 = 0;
 var $224 = 0, $225 = 0, $226 = 0, $227 = 0, $228 = 0, $229 = 0, $23 = 0, $230 = 0, $231 = 0, $232 = 0, $233 = 0, $234 = 0, $235 = 0, $236 = 0, $237 = 0, $238 = 0, $239 = 0, $24 = 0, $240 = 0, $241 = 0;
 var $242 = 0, $243 = 0, $244 = 0, $245 = 0, $246 = 0, $247 = 0, $248 = 0, $249 = 0, $25 = 0, $250 = 0, $251 = 0, $252 = 0, $253 = 0, $254 = 0, $255 = 0, $256 = 0, $257 = 0, $258 = 0, $259 = 0, $26 = 0;
 var $260 = 0, $261 = 0, $262 = 0, $263 = 0, $264 = 0, $265 = 0, $266 = 0, $267 = 0, $268 = 0, $269 = 0, $27 = 0, $270 = 0, $271 = 0, $272 = 0, $273 = 0, $274 = 0, $275 = 0, $276 = 0, $277 = 0, $278 = 0;
 var $279 = 0, $28 = 0, $280 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0;
 var $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0;
 var $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0;
 var $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0;
 var $99 = 0, $action = 0, $aiCol = 0, $db = 0, $enableLookaside = 0, $i = 0, $iAction = 0, $iFromCol = 0, $nFrom = 0, $or$cond = 0, $or$cond3 = 0, $pDflt = 0, $pEq = 0, $pIdx = 0, $pList = 0, $pNew = 0, $pRaise = 0, $pSelect = 0, $pStep = 0, $pTrigger = 0;
 var $pWhen = 0, $pWhere = 0, $tFrom = 0, $tFromCol = 0, $tNew = 0, $tOld = 0, $tToCol = 0, $zFrom = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 144|0;
 $0 = sp + 136|0;
 $1 = sp + 132|0;
 $2 = sp + 128|0;
 $3 = sp + 124|0;
 $4 = sp + 120|0;
 $db = sp + 116|0;
 $action = sp + 112|0;
 $pTrigger = sp + 108|0;
 $iAction = sp + 104|0;
 $enableLookaside = sp + 140|0;
 $zFrom = sp + 100|0;
 $nFrom = sp + 96|0;
 $pIdx = sp + 92|0;
 $aiCol = sp + 88|0;
 $pStep = sp + 84|0;
 $pWhere = sp + 80|0;
 $pList = sp + 76|0;
 $pSelect = sp + 72|0;
 $i = sp + 68|0;
 $pWhen = sp + 64|0;
 $tOld = sp + 56|0;
 $tNew = sp + 48|0;
 $tFromCol = sp + 40|0;
 $tToCol = sp + 32|0;
 $iFromCol = sp + 28|0;
 $pEq = sp + 24|0;
 $pNew = sp + 20|0;
 $pDflt = sp + 16|0;
 $tFrom = sp + 8|0;
 $pRaise = sp;
 HEAP32[$1>>2] = $pParse;
 HEAP32[$2>>2] = $pTab;
 HEAP32[$3>>2] = $pFKey;
 HEAP32[$4>>2] = $pChanges;
 $5 = HEAP32[$1>>2]|0;
 $6 = HEAP32[$5>>2]|0;
 HEAP32[$db>>2] = $6;
 $7 = HEAP32[$4>>2]|0;
 $8 = ($7|0)!=(0|0);
 $9 = $8&1;
 HEAP32[$iAction>>2] = $9;
 $10 = HEAP32[$iAction>>2]|0;
 $11 = HEAP32[$3>>2]|0;
 $12 = (($11) + 25|0);
 $13 = (($12) + ($10)|0);
 $14 = HEAP8[$13>>0]|0;
 $15 = $14&255;
 HEAP32[$action>>2] = $15;
 $16 = HEAP32[$iAction>>2]|0;
 $17 = HEAP32[$3>>2]|0;
 $18 = (($17) + 28|0);
 $19 = (($18) + ($16<<2)|0);
 $20 = HEAP32[$19>>2]|0;
 HEAP32[$pTrigger>>2] = $20;
 $21 = HEAP32[$action>>2]|0;
 $22 = ($21|0)==(0);
 $23 = HEAP32[$pTrigger>>2]|0;
 $24 = ($23|0)!=(0|0);
 $or$cond = $22 | $24;
 if (!($or$cond)) {
  HEAP32[$pIdx>>2] = 0;
  HEAP32[$aiCol>>2] = 0;
  HEAP32[$pStep>>2] = 0;
  HEAP32[$pWhere>>2] = 0;
  HEAP32[$pList>>2] = 0;
  HEAP32[$pSelect>>2] = 0;
  HEAP32[$pWhen>>2] = 0;
  $25 = HEAP32[$1>>2]|0;
  $26 = HEAP32[$2>>2]|0;
  $27 = HEAP32[$3>>2]|0;
  $28 = (_sqlite3FkLocateIndex($25,$26,$27,$pIdx,$aiCol)|0);
  $29 = ($28|0)!=(0);
  if ($29) {
   HEAP32[$0>>2] = 0;
   $280 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($280|0);
  }
  HEAP32[$i>>2] = 0;
  while(1) {
   $30 = HEAP32[$i>>2]|0;
   $31 = HEAP32[$3>>2]|0;
   $32 = (($31) + 20|0);
   $33 = HEAP32[$32>>2]|0;
   $34 = ($30|0)<($33|0);
   if (!($34)) {
    break;
   }
   ;HEAP32[$tOld+0>>2]=HEAP32[24376+0>>2]|0;HEAP32[$tOld+4>>2]=HEAP32[24376+4>>2]|0;
   ;HEAP32[$tNew+0>>2]=HEAP32[24384+0>>2]|0;HEAP32[$tNew+4>>2]=HEAP32[24384+4>>2]|0;
   $35 = HEAP32[$aiCol>>2]|0;
   $36 = ($35|0)!=(0|0);
   if ($36) {
    $37 = HEAP32[$i>>2]|0;
    $38 = HEAP32[$aiCol>>2]|0;
    $39 = (($38) + ($37<<2)|0);
    $40 = HEAP32[$39>>2]|0;
    $44 = $40;
   } else {
    $41 = HEAP32[$3>>2]|0;
    $42 = (($41) + 36|0);
    $43 = HEAP32[$42>>2]|0;
    $44 = $43;
   }
   HEAP32[$iFromCol>>2] = $44;
   $45 = HEAP32[$pIdx>>2]|0;
   $46 = ($45|0)!=(0|0);
   if ($46) {
    $47 = HEAP32[$i>>2]|0;
    $48 = HEAP32[$pIdx>>2]|0;
    $49 = (($48) + 4|0);
    $50 = HEAP32[$49>>2]|0;
    $51 = (($50) + ($47<<1)|0);
    $52 = HEAP16[$51>>1]|0;
    $53 = $52 << 16 >> 16;
    $54 = HEAP32[$2>>2]|0;
    $55 = (($54) + 4|0);
    $56 = HEAP32[$55>>2]|0;
    $57 = (($56) + (($53*24)|0)|0);
    $58 = HEAP32[$57>>2]|0;
    $59 = $58;
   } else {
    $59 = 24392;
   }
   HEAP32[$tToCol>>2] = $59;
   $60 = HEAP32[$iFromCol>>2]|0;
   $61 = HEAP32[$3>>2]|0;
   $62 = HEAP32[$61>>2]|0;
   $63 = (($62) + 4|0);
   $64 = HEAP32[$63>>2]|0;
   $65 = (($64) + (($60*24)|0)|0);
   $66 = HEAP32[$65>>2]|0;
   HEAP32[$tFromCol>>2] = $66;
   $67 = HEAP32[$tToCol>>2]|0;
   $68 = (_sqlite3Strlen30($67)|0);
   $69 = (($tToCol) + 4|0);
   HEAP32[$69>>2] = $68;
   $70 = HEAP32[$tFromCol>>2]|0;
   $71 = (_sqlite3Strlen30($70)|0);
   $72 = (($tFromCol) + 4|0);
   HEAP32[$72>>2] = $71;
   $73 = HEAP32[$1>>2]|0;
   $74 = HEAP32[$1>>2]|0;
   $75 = HEAP32[$1>>2]|0;
   $76 = (_sqlite3PExpr($75,27,0,0,$tOld)|0);
   $77 = HEAP32[$1>>2]|0;
   $78 = (_sqlite3PExpr($77,27,0,0,$tToCol)|0);
   $79 = (_sqlite3PExpr($74,122,$76,$78,0)|0);
   $80 = HEAP32[$1>>2]|0;
   $81 = (_sqlite3PExpr($80,27,0,0,$tFromCol)|0);
   $82 = (_sqlite3PExpr($73,79,$79,$81,0)|0);
   HEAP32[$pEq>>2] = $82;
   $83 = HEAP32[$db>>2]|0;
   $84 = HEAP32[$pWhere>>2]|0;
   $85 = HEAP32[$pEq>>2]|0;
   $86 = (_sqlite3ExprAnd($83,$84,$85)|0);
   HEAP32[$pWhere>>2] = $86;
   $87 = HEAP32[$4>>2]|0;
   $88 = ($87|0)!=(0|0);
   if ($88) {
    $89 = HEAP32[$1>>2]|0;
    $90 = HEAP32[$1>>2]|0;
    $91 = HEAP32[$1>>2]|0;
    $92 = (_sqlite3PExpr($91,27,0,0,$tOld)|0);
    $93 = HEAP32[$1>>2]|0;
    $94 = (_sqlite3PExpr($93,27,0,0,$tToCol)|0);
    $95 = (_sqlite3PExpr($90,122,$92,$94,0)|0);
    $96 = HEAP32[$1>>2]|0;
    $97 = HEAP32[$1>>2]|0;
    $98 = (_sqlite3PExpr($97,27,0,0,$tNew)|0);
    $99 = HEAP32[$1>>2]|0;
    $100 = (_sqlite3PExpr($99,27,0,0,$tToCol)|0);
    $101 = (_sqlite3PExpr($96,122,$98,$100,0)|0);
    $102 = (_sqlite3PExpr($89,73,$95,$101,0)|0);
    HEAP32[$pEq>>2] = $102;
    $103 = HEAP32[$db>>2]|0;
    $104 = HEAP32[$pWhen>>2]|0;
    $105 = HEAP32[$pEq>>2]|0;
    $106 = (_sqlite3ExprAnd($103,$104,$105)|0);
    HEAP32[$pWhen>>2] = $106;
   }
   $107 = HEAP32[$action>>2]|0;
   $108 = ($107|0)!=(6);
   if ($108) {
    $109 = HEAP32[$action>>2]|0;
    $110 = ($109|0)!=(9);
    $111 = HEAP32[$4>>2]|0;
    $112 = ($111|0)!=(0|0);
    $or$cond3 = $110 | $112;
    if ($or$cond3) {
     $113 = HEAP32[$action>>2]|0;
     $114 = ($113|0)==(9);
     do {
      if ($114) {
       $115 = HEAP32[$1>>2]|0;
       $116 = HEAP32[$1>>2]|0;
       $117 = (_sqlite3PExpr($116,27,0,0,$tNew)|0);
       $118 = HEAP32[$1>>2]|0;
       $119 = (_sqlite3PExpr($118,27,0,0,$tToCol)|0);
       $120 = (_sqlite3PExpr($115,122,$117,$119,0)|0);
       HEAP32[$pNew>>2] = $120;
      } else {
       $121 = HEAP32[$action>>2]|0;
       $122 = ($121|0)==(8);
       if (!($122)) {
        $138 = HEAP32[$1>>2]|0;
        $139 = (_sqlite3PExpr($138,101,0,0,0)|0);
        HEAP32[$pNew>>2] = $139;
        break;
       }
       $123 = HEAP32[$iFromCol>>2]|0;
       $124 = HEAP32[$3>>2]|0;
       $125 = HEAP32[$124>>2]|0;
       $126 = (($125) + 4|0);
       $127 = HEAP32[$126>>2]|0;
       $128 = (($127) + (($123*24)|0)|0);
       $129 = (($128) + 4|0);
       $130 = HEAP32[$129>>2]|0;
       HEAP32[$pDflt>>2] = $130;
       $131 = HEAP32[$pDflt>>2]|0;
       $132 = ($131|0)!=(0|0);
       if ($132) {
        $133 = HEAP32[$db>>2]|0;
        $134 = HEAP32[$pDflt>>2]|0;
        $135 = (_sqlite3ExprDup($133,$134,0)|0);
        HEAP32[$pNew>>2] = $135;
        break;
       } else {
        $136 = HEAP32[$1>>2]|0;
        $137 = (_sqlite3PExpr($136,101,0,0,0)|0);
        HEAP32[$pNew>>2] = $137;
        break;
       }
      }
     } while(0);
     $140 = HEAP32[$1>>2]|0;
     $141 = HEAP32[$pList>>2]|0;
     $142 = HEAP32[$pNew>>2]|0;
     $143 = (_sqlite3ExprListAppend($140,$141,$142)|0);
     HEAP32[$pList>>2] = $143;
     $144 = HEAP32[$1>>2]|0;
     $145 = HEAP32[$pList>>2]|0;
     _sqlite3ExprListSetName($144,$145,$tFromCol,0);
    }
   }
   $146 = HEAP32[$i>>2]|0;
   $147 = (($146) + 1)|0;
   HEAP32[$i>>2] = $147;
  }
  $148 = HEAP32[$db>>2]|0;
  $149 = HEAP32[$aiCol>>2]|0;
  _sqlite3DbFree($148,$149);
  $150 = HEAP32[$3>>2]|0;
  $151 = HEAP32[$150>>2]|0;
  $152 = HEAP32[$151>>2]|0;
  HEAP32[$zFrom>>2] = $152;
  $153 = HEAP32[$zFrom>>2]|0;
  $154 = (_sqlite3Strlen30($153)|0);
  HEAP32[$nFrom>>2] = $154;
  $155 = HEAP32[$action>>2]|0;
  $156 = ($155|0)==(6);
  if ($156) {
   $157 = HEAP32[$zFrom>>2]|0;
   HEAP32[$tFrom>>2] = $157;
   $158 = HEAP32[$nFrom>>2]|0;
   $159 = (($tFrom) + 4|0);
   HEAP32[$159>>2] = $158;
   $160 = HEAP32[$db>>2]|0;
   $161 = (_sqlite3Expr($160,57,24400)|0);
   HEAP32[$pRaise>>2] = $161;
   $162 = HEAP32[$pRaise>>2]|0;
   $163 = ($162|0)!=(0|0);
   if ($163) {
    $164 = HEAP32[$pRaise>>2]|0;
    $165 = (($164) + 1|0);
    HEAP8[$165>>0] = 2;
   }
   $166 = HEAP32[$1>>2]|0;
   $167 = HEAP32[$1>>2]|0;
   $168 = HEAP32[$pRaise>>2]|0;
   $169 = (_sqlite3ExprListAppend($167,0,$168)|0);
   $170 = HEAP32[$db>>2]|0;
   $171 = (_sqlite3SrcListAppend($170,0,$tFrom,0)|0);
   $172 = HEAP32[$pWhere>>2]|0;
   $173 = (_sqlite3SelectNew($166,$169,$171,$172,0,0,0,0,0,0)|0);
   HEAP32[$pSelect>>2] = $173;
   HEAP32[$pWhere>>2] = 0;
  }
  $174 = HEAP32[$db>>2]|0;
  $175 = (($174) + 248|0);
  $176 = (($175) + 2|0);
  $177 = HEAP8[$176>>0]|0;
  HEAP8[$enableLookaside>>0] = $177;
  $178 = HEAP32[$db>>2]|0;
  $179 = (($178) + 248|0);
  $180 = (($179) + 2|0);
  HEAP8[$180>>0] = 0;
  $181 = HEAP32[$db>>2]|0;
  $182 = HEAP32[$nFrom>>2]|0;
  $183 = (76 + ($182))|0;
  $184 = (($183) + 1)|0;
  $185 = (_sqlite3DbMallocZero($181,$184,0)|0);
  HEAP32[$pTrigger>>2] = $185;
  $186 = HEAP32[$pTrigger>>2]|0;
  $187 = ($186|0)!=(0|0);
  if ($187) {
   $188 = HEAP32[$pTrigger>>2]|0;
   $189 = (($188) + 36|0);
   $190 = HEAP32[$pTrigger>>2]|0;
   $191 = (($190) + 28|0);
   HEAP32[$191>>2] = $189;
   HEAP32[$pStep>>2] = $189;
   $192 = HEAP32[$pStep>>2]|0;
   $193 = (($192) + 40|0);
   $194 = HEAP32[$pStep>>2]|0;
   $195 = (($194) + 12|0);
   HEAP32[$195>>2] = $193;
   $196 = HEAP32[$nFrom>>2]|0;
   $197 = HEAP32[$pStep>>2]|0;
   $198 = (($197) + 12|0);
   $199 = (($198) + 4|0);
   HEAP32[$199>>2] = $196;
   $200 = HEAP32[$pStep>>2]|0;
   $201 = (($200) + 12|0);
   $202 = HEAP32[$201>>2]|0;
   $203 = HEAP32[$zFrom>>2]|0;
   $204 = HEAP32[$nFrom>>2]|0;
   _memcpy(($202|0),($203|0),($204|0))|0;
   $205 = HEAP32[$db>>2]|0;
   $206 = HEAP32[$pWhere>>2]|0;
   $207 = (_sqlite3ExprDup($205,$206,1)|0);
   $208 = HEAP32[$pStep>>2]|0;
   $209 = (($208) + 20|0);
   HEAP32[$209>>2] = $207;
   $210 = HEAP32[$db>>2]|0;
   $211 = HEAP32[$pList>>2]|0;
   $212 = (_sqlite3ExprListDup($210,$211,1)|0);
   $213 = HEAP32[$pStep>>2]|0;
   $214 = (($213) + 24|0);
   HEAP32[$214>>2] = $212;
   $215 = HEAP32[$db>>2]|0;
   $216 = HEAP32[$pSelect>>2]|0;
   $217 = (_sqlite3SelectDup($215,$216,1)|0);
   $218 = HEAP32[$pStep>>2]|0;
   $219 = (($218) + 8|0);
   HEAP32[$219>>2] = $217;
   $220 = HEAP32[$pWhen>>2]|0;
   $221 = ($220|0)!=(0|0);
   if ($221) {
    $222 = HEAP32[$1>>2]|0;
    $223 = HEAP32[$pWhen>>2]|0;
    $224 = (_sqlite3PExpr($222,19,$223,0,0)|0);
    HEAP32[$pWhen>>2] = $224;
    $225 = HEAP32[$db>>2]|0;
    $226 = HEAP32[$pWhen>>2]|0;
    $227 = (_sqlite3ExprDup($225,$226,1)|0);
    $228 = HEAP32[$pTrigger>>2]|0;
    $229 = (($228) + 12|0);
    HEAP32[$229>>2] = $227;
   }
  }
  $230 = HEAP8[$enableLookaside>>0]|0;
  $231 = HEAP32[$db>>2]|0;
  $232 = (($231) + 248|0);
  $233 = (($232) + 2|0);
  HEAP8[$233>>0] = $230;
  $234 = HEAP32[$db>>2]|0;
  $235 = HEAP32[$pWhere>>2]|0;
  _sqlite3ExprDelete($234,$235);
  $236 = HEAP32[$db>>2]|0;
  $237 = HEAP32[$pWhen>>2]|0;
  _sqlite3ExprDelete($236,$237);
  $238 = HEAP32[$db>>2]|0;
  $239 = HEAP32[$pList>>2]|0;
  _sqlite3ExprListDelete($238,$239);
  $240 = HEAP32[$db>>2]|0;
  $241 = HEAP32[$pSelect>>2]|0;
  _sqlite3SelectDelete($240,$241);
  $242 = HEAP32[$db>>2]|0;
  $243 = (($242) + 64|0);
  $244 = HEAP8[$243>>0]|0;
  $245 = $244&255;
  $246 = ($245|0)==(1);
  if ($246) {
   $247 = HEAP32[$db>>2]|0;
   $248 = HEAP32[$pTrigger>>2]|0;
   _fkTriggerDelete($247,$248);
   HEAP32[$0>>2] = 0;
   $280 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($280|0);
  }
  $249 = HEAP32[$action>>2]|0;
  if ((($249|0) == 9)) {
   $251 = HEAP32[$4>>2]|0;
   $252 = ($251|0)!=(0|0);
   if ($252) {
    label = 37;
   } else {
    $253 = HEAP32[$pStep>>2]|0;
    HEAP8[$253>>0] = 109;
   }
  } else if ((($249|0) == 6)) {
   $250 = HEAP32[$pStep>>2]|0;
   HEAP8[$250>>0] = 119;
  } else {
   label = 37;
  }
  if ((label|0) == 37) {
   $254 = HEAP32[$pStep>>2]|0;
   HEAP8[$254>>0] = 110;
  }
  $255 = HEAP32[$pTrigger>>2]|0;
  $256 = HEAP32[$pStep>>2]|0;
  $257 = (($256) + 4|0);
  HEAP32[$257>>2] = $255;
  $258 = HEAP32[$2>>2]|0;
  $259 = (($258) + 68|0);
  $260 = HEAP32[$259>>2]|0;
  $261 = HEAP32[$pTrigger>>2]|0;
  $262 = (($261) + 20|0);
  HEAP32[$262>>2] = $260;
  $263 = HEAP32[$2>>2]|0;
  $264 = (($263) + 68|0);
  $265 = HEAP32[$264>>2]|0;
  $266 = HEAP32[$pTrigger>>2]|0;
  $267 = (($266) + 24|0);
  HEAP32[$267>>2] = $265;
  $268 = HEAP32[$pTrigger>>2]|0;
  $269 = HEAP32[$iAction>>2]|0;
  $270 = HEAP32[$3>>2]|0;
  $271 = (($270) + 28|0);
  $272 = (($271) + ($269<<2)|0);
  HEAP32[$272>>2] = $268;
  $273 = HEAP32[$4>>2]|0;
  $274 = ($273|0)!=(0|0);
  $275 = $274 ? 110 : 109;
  $276 = $275&255;
  $277 = HEAP32[$pTrigger>>2]|0;
  $278 = (($277) + 8|0);
  HEAP8[$278>>0] = $276;
 }
 $279 = HEAP32[$pTrigger>>2]|0;
 HEAP32[$0>>2] = $279;
 $280 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($280|0);
}
function _sqlite3CodeRowTriggerDirect($pParse,$p,$pTab,$reg,$orconf,$ignoreJump) {
 $pParse = $pParse|0;
 $p = $p|0;
 $pTab = $pTab|0;
 $reg = $reg|0;
 $orconf = $orconf|0;
 $ignoreJump = $ignoreJump|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0;
 var $bRecursive = 0, $pPrg = 0, $v = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $0 = sp + 32|0;
 $1 = sp + 28|0;
 $2 = sp + 24|0;
 $3 = sp + 20|0;
 $4 = sp + 16|0;
 $5 = sp + 12|0;
 $v = sp + 8|0;
 $pPrg = sp + 4|0;
 $bRecursive = sp;
 HEAP32[$0>>2] = $pParse;
 HEAP32[$1>>2] = $p;
 HEAP32[$2>>2] = $pTab;
 HEAP32[$3>>2] = $reg;
 HEAP32[$4>>2] = $orconf;
 HEAP32[$5>>2] = $ignoreJump;
 $6 = HEAP32[$0>>2]|0;
 $7 = (_sqlite3GetVdbe($6)|0);
 HEAP32[$v>>2] = $7;
 $8 = HEAP32[$0>>2]|0;
 $9 = HEAP32[$1>>2]|0;
 $10 = HEAP32[$2>>2]|0;
 $11 = HEAP32[$4>>2]|0;
 $12 = (_getRowTrigger($8,$9,$10,$11)|0);
 HEAP32[$pPrg>>2] = $12;
 $13 = HEAP32[$pPrg>>2]|0;
 $14 = ($13|0)!=(0|0);
 if (!($14)) {
  STACKTOP = sp;return;
 }
 $15 = HEAP32[$1>>2]|0;
 $16 = HEAP32[$15>>2]|0;
 $17 = ($16|0)!=(0|0);
 if ($17) {
  $18 = HEAP32[$0>>2]|0;
  $19 = HEAP32[$18>>2]|0;
  $20 = (($19) + 24|0);
  $21 = HEAP32[$20>>2]|0;
  $22 = $21 & 262144;
  $23 = (0)==($22|0);
  $25 = $23;
 } else {
  $25 = 0;
 }
 $24 = $25&1;
 HEAP32[$bRecursive>>2] = $24;
 $26 = HEAP32[$v>>2]|0;
 $27 = HEAP32[$3>>2]|0;
 $28 = HEAP32[$5>>2]|0;
 $29 = HEAP32[$0>>2]|0;
 $30 = (($29) + 72|0);
 $31 = HEAP32[$30>>2]|0;
 $32 = (($31) + 1)|0;
 HEAP32[$30>>2] = $32;
 (_sqlite3VdbeAddOp3($26,131,$27,$28,$32)|0);
 $33 = HEAP32[$v>>2]|0;
 $34 = HEAP32[$pPrg>>2]|0;
 $35 = (($34) + 8|0);
 $36 = HEAP32[$35>>2]|0;
 _sqlite3VdbeChangeP4($33,-1,$36,-18);
 $37 = HEAP32[$v>>2]|0;
 $38 = HEAP32[$bRecursive>>2]|0;
 $39 = $38&255;
 _sqlite3VdbeChangeP5($37,$39);
 STACKTOP = sp;return;
}
function _getRowTrigger($pParse,$pTrigger,$pTab,$orconf) {
 $pParse = $pParse|0;
 $pTrigger = $pTrigger|0;
 $pTab = $pTab|0;
 $orconf = $orconf|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $pPrg = 0, $pRoot = 0, label = 0;
 var sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 20|0;
 $1 = sp + 16|0;
 $2 = sp + 12|0;
 $3 = sp + 8|0;
 $pRoot = sp + 4|0;
 $pPrg = sp;
 HEAP32[$0>>2] = $pParse;
 HEAP32[$1>>2] = $pTrigger;
 HEAP32[$2>>2] = $pTab;
 HEAP32[$3>>2] = $orconf;
 $4 = HEAP32[$0>>2]|0;
 $5 = (($4) + 408|0);
 $6 = HEAP32[$5>>2]|0;
 $7 = ($6|0)!=(0|0);
 $8 = HEAP32[$0>>2]|0;
 if ($7) {
  $9 = (($8) + 408|0);
  $10 = HEAP32[$9>>2]|0;
  $11 = $10;
 } else {
  $11 = $8;
 }
 HEAP32[$pRoot>>2] = $11;
 $12 = HEAP32[$pRoot>>2]|0;
 $13 = (($12) + 528|0);
 $14 = HEAP32[$13>>2]|0;
 HEAP32[$pPrg>>2] = $14;
 while(1) {
  $15 = HEAP32[$pPrg>>2]|0;
  $16 = ($15|0)!=(0|0);
  if (!($16)) {
   break;
  }
  $17 = HEAP32[$pPrg>>2]|0;
  $18 = HEAP32[$17>>2]|0;
  $19 = HEAP32[$1>>2]|0;
  $20 = ($18|0)!=($19|0);
  if (!($20)) {
   $21 = HEAP32[$pPrg>>2]|0;
   $22 = (($21) + 12|0);
   $23 = HEAP32[$22>>2]|0;
   $24 = HEAP32[$3>>2]|0;
   $25 = ($23|0)!=($24|0);
   if (!($25)) {
    break;
   }
  }
  $26 = HEAP32[$pPrg>>2]|0;
  $27 = (($26) + 4|0);
  $28 = HEAP32[$27>>2]|0;
  HEAP32[$pPrg>>2] = $28;
 }
 $29 = HEAP32[$pPrg>>2]|0;
 $30 = ($29|0)!=(0|0);
 if ($30) {
  $36 = HEAP32[$pPrg>>2]|0;
  STACKTOP = sp;return ($36|0);
 }
 $31 = HEAP32[$0>>2]|0;
 $32 = HEAP32[$1>>2]|0;
 $33 = HEAP32[$2>>2]|0;
 $34 = HEAP32[$3>>2]|0;
 $35 = (_codeRowTrigger($31,$32,$33,$34)|0);
 HEAP32[$pPrg>>2] = $35;
 $36 = HEAP32[$pPrg>>2]|0;
 STACKTOP = sp;return ($36|0);
}
function _codeRowTrigger($pParse,$pTrigger,$pTab,$orconf) {
 $pParse = $pParse|0;
 $pTrigger = $pTrigger|0;
 $pTab = $pTab|0;
 $orconf = $orconf|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0;
 var $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0;
 var $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0;
 var $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0;
 var $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0;
 var $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $db = 0, $iEndTrigger = 0, $pPrg = 0, $pProgram = 0, $pSubParse = 0, $pTop = 0, $pWhen = 0, $sNC = 0, $v = 0, $vararg_buffer = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 96|0;
 $vararg_buffer = sp;
 $0 = sp + 84|0;
 $1 = sp + 80|0;
 $2 = sp + 76|0;
 $3 = sp + 72|0;
 $4 = sp + 68|0;
 $pTop = sp + 64|0;
 $db = sp + 60|0;
 $pPrg = sp + 56|0;
 $pWhen = sp + 52|0;
 $v = sp + 48|0;
 $sNC = sp + 16|0;
 $pProgram = sp + 12|0;
 $pSubParse = sp + 8|0;
 $iEndTrigger = sp + 4|0;
 HEAP32[$1>>2] = $pParse;
 HEAP32[$2>>2] = $pTrigger;
 HEAP32[$3>>2] = $pTab;
 HEAP32[$4>>2] = $orconf;
 $5 = HEAP32[$1>>2]|0;
 $6 = (($5) + 408|0);
 $7 = HEAP32[$6>>2]|0;
 $8 = ($7|0)!=(0|0);
 $9 = HEAP32[$1>>2]|0;
 if ($8) {
  $10 = (($9) + 408|0);
  $11 = HEAP32[$10>>2]|0;
  $12 = $11;
 } else {
  $12 = $9;
 }
 HEAP32[$pTop>>2] = $12;
 $13 = HEAP32[$1>>2]|0;
 $14 = HEAP32[$13>>2]|0;
 HEAP32[$db>>2] = $14;
 HEAP32[$pWhen>>2] = 0;
 HEAP32[$pProgram>>2] = 0;
 HEAP32[$iEndTrigger>>2] = 0;
 $15 = HEAP32[$db>>2]|0;
 $16 = (_sqlite3DbMallocZero($15,24,0)|0);
 HEAP32[$pPrg>>2] = $16;
 $17 = HEAP32[$pPrg>>2]|0;
 $18 = ($17|0)!=(0|0);
 if (!($18)) {
  HEAP32[$0>>2] = 0;
  $165 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($165|0);
 }
 $19 = HEAP32[$pTop>>2]|0;
 $20 = (($19) + 528|0);
 $21 = HEAP32[$20>>2]|0;
 $22 = HEAP32[$pPrg>>2]|0;
 $23 = (($22) + 4|0);
 HEAP32[$23>>2] = $21;
 $24 = HEAP32[$pPrg>>2]|0;
 $25 = HEAP32[$pTop>>2]|0;
 $26 = (($25) + 528|0);
 HEAP32[$26>>2] = $24;
 $27 = HEAP32[$db>>2]|0;
 $28 = (_sqlite3DbMallocZero($27,28,0)|0);
 HEAP32[$pProgram>>2] = $28;
 $29 = HEAP32[$pPrg>>2]|0;
 $30 = (($29) + 8|0);
 HEAP32[$30>>2] = $28;
 $31 = HEAP32[$pProgram>>2]|0;
 $32 = ($31|0)!=(0|0);
 if (!($32)) {
  HEAP32[$0>>2] = 0;
  $165 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($165|0);
 }
 $33 = HEAP32[$pTop>>2]|0;
 $34 = (($33) + 8|0);
 $35 = HEAP32[$34>>2]|0;
 $36 = HEAP32[$pProgram>>2]|0;
 _sqlite3VdbeLinkSubProgram($35,$36);
 $37 = HEAP32[$2>>2]|0;
 $38 = HEAP32[$pPrg>>2]|0;
 HEAP32[$38>>2] = $37;
 $39 = HEAP32[$4>>2]|0;
 $40 = HEAP32[$pPrg>>2]|0;
 $41 = (($40) + 12|0);
 HEAP32[$41>>2] = $39;
 $42 = HEAP32[$pPrg>>2]|0;
 $43 = (($42) + 16|0);
 HEAP32[$43>>2] = -1;
 $44 = HEAP32[$pPrg>>2]|0;
 $45 = (($44) + 16|0);
 $46 = (($45) + 4|0);
 HEAP32[$46>>2] = -1;
 $47 = HEAP32[$db>>2]|0;
 $48 = (_sqlite3DbMallocZero($47,536,0)|0);
 HEAP32[$pSubParse>>2] = $48;
 $49 = HEAP32[$pSubParse>>2]|0;
 $50 = ($49|0)!=(0|0);
 if (!($50)) {
  HEAP32[$0>>2] = 0;
  $165 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($165|0);
 }
 ;HEAP32[$sNC+0>>2]=0|0;HEAP32[$sNC+4>>2]=0|0;HEAP32[$sNC+8>>2]=0|0;HEAP32[$sNC+12>>2]=0|0;HEAP32[$sNC+16>>2]=0|0;HEAP32[$sNC+20>>2]=0|0;HEAP32[$sNC+24>>2]=0|0;HEAP32[$sNC+28>>2]=0|0;
 $51 = HEAP32[$pSubParse>>2]|0;
 HEAP32[$sNC>>2] = $51;
 $52 = HEAP32[$db>>2]|0;
 $53 = HEAP32[$pSubParse>>2]|0;
 HEAP32[$53>>2] = $52;
 $54 = HEAP32[$3>>2]|0;
 $55 = HEAP32[$pSubParse>>2]|0;
 $56 = (($55) + 412|0);
 HEAP32[$56>>2] = $54;
 $57 = HEAP32[$pTop>>2]|0;
 $58 = HEAP32[$pSubParse>>2]|0;
 $59 = (($58) + 408|0);
 HEAP32[$59>>2] = $57;
 $60 = HEAP32[$2>>2]|0;
 $61 = HEAP32[$60>>2]|0;
 $62 = HEAP32[$pSubParse>>2]|0;
 $63 = (($62) + 492|0);
 HEAP32[$63>>2] = $61;
 $64 = HEAP32[$2>>2]|0;
 $65 = (($64) + 8|0);
 $66 = HEAP8[$65>>0]|0;
 $67 = HEAP32[$pSubParse>>2]|0;
 $68 = (($67) + 436|0);
 HEAP8[$68>>0] = $66;
 $69 = HEAP32[$1>>2]|0;
 $70 = (($69) + 424|0);
 $71 = HEAP32[$70>>2]|0;
 $72 = HEAP32[$pSubParse>>2]|0;
 $73 = (($72) + 424|0);
 HEAP32[$73>>2] = $71;
 $74 = HEAP32[$pSubParse>>2]|0;
 $75 = (_sqlite3GetVdbe($74)|0);
 HEAP32[$v>>2] = $75;
 $76 = HEAP32[$v>>2]|0;
 $77 = ($76|0)!=(0|0);
 if ($77) {
  $78 = HEAP32[$v>>2]|0;
  $79 = HEAP32[$db>>2]|0;
  $80 = HEAP32[$2>>2]|0;
  $81 = HEAP32[$80>>2]|0;
  HEAP32[$vararg_buffer>>2] = $81;
  $82 = (_sqlite3MPrintf($79,24176,$vararg_buffer)|0);
  _sqlite3VdbeChangeP4($78,-1,$82,-1);
  $83 = HEAP32[$2>>2]|0;
  $84 = (($83) + 12|0);
  $85 = HEAP32[$84>>2]|0;
  $86 = ($85|0)!=(0|0);
  if ($86) {
   $87 = HEAP32[$db>>2]|0;
   $88 = HEAP32[$2>>2]|0;
   $89 = (($88) + 12|0);
   $90 = HEAP32[$89>>2]|0;
   $91 = (_sqlite3ExprDup($87,$90,0)|0);
   HEAP32[$pWhen>>2] = $91;
   $92 = HEAP32[$pWhen>>2]|0;
   $93 = (_sqlite3ResolveExprNames($sNC,$92)|0);
   $94 = (0)==($93|0);
   if ($94) {
    $95 = HEAP32[$db>>2]|0;
    $96 = (($95) + 64|0);
    $97 = HEAP8[$96>>0]|0;
    $98 = $97&255;
    $99 = ($98|0)==(0);
    if ($99) {
     $100 = HEAP32[$v>>2]|0;
     $101 = (_sqlite3VdbeMakeLabel($100)|0);
     HEAP32[$iEndTrigger>>2] = $101;
     $102 = HEAP32[$pSubParse>>2]|0;
     $103 = HEAP32[$pWhen>>2]|0;
     $104 = HEAP32[$iEndTrigger>>2]|0;
     _sqlite3ExprIfFalse($102,$103,$104,16);
    }
   }
   $105 = HEAP32[$db>>2]|0;
   $106 = HEAP32[$pWhen>>2]|0;
   _sqlite3ExprDelete($105,$106);
  }
  $107 = HEAP32[$pSubParse>>2]|0;
  $108 = HEAP32[$2>>2]|0;
  $109 = (($108) + 28|0);
  $110 = HEAP32[$109>>2]|0;
  $111 = HEAP32[$4>>2]|0;
  (_codeTriggerProgram($107,$110,$111)|0);
  $112 = HEAP32[$iEndTrigger>>2]|0;
  $113 = ($112|0)!=(0);
  if ($113) {
   $114 = HEAP32[$v>>2]|0;
   $115 = HEAP32[$iEndTrigger>>2]|0;
   _sqlite3VdbeResolveLabel($114,$115);
  }
  $116 = HEAP32[$v>>2]|0;
  (_sqlite3VdbeAddOp0($116,24)|0);
  $117 = HEAP32[$1>>2]|0;
  $118 = HEAP32[$pSubParse>>2]|0;
  _transferParseError($117,$118);
  $119 = HEAP32[$db>>2]|0;
  $120 = (($119) + 64|0);
  $121 = HEAP8[$120>>0]|0;
  $122 = $121&255;
  $123 = ($122|0)==(0);
  if ($123) {
   $124 = HEAP32[$v>>2]|0;
   $125 = HEAP32[$pProgram>>2]|0;
   $126 = (($125) + 4|0);
   $127 = HEAP32[$pTop>>2]|0;
   $128 = (($127) + 392|0);
   $129 = (_sqlite3VdbeTakeOpArray($124,$126,$128)|0);
   $130 = HEAP32[$pProgram>>2]|0;
   HEAP32[$130>>2] = $129;
  }
  $131 = HEAP32[$pSubParse>>2]|0;
  $132 = (($131) + 72|0);
  $133 = HEAP32[$132>>2]|0;
  $134 = HEAP32[$pProgram>>2]|0;
  $135 = (($134) + 8|0);
  HEAP32[$135>>2] = $133;
  $136 = HEAP32[$pSubParse>>2]|0;
  $137 = (($136) + 68|0);
  $138 = HEAP32[$137>>2]|0;
  $139 = HEAP32[$pProgram>>2]|0;
  $140 = (($139) + 12|0);
  HEAP32[$140>>2] = $138;
  $141 = HEAP32[$pSubParse>>2]|0;
  $142 = (($141) + 80|0);
  $143 = HEAP32[$142>>2]|0;
  $144 = HEAP32[$pProgram>>2]|0;
  $145 = (($144) + 16|0);
  HEAP32[$145>>2] = $143;
  $146 = HEAP32[$2>>2]|0;
  $147 = HEAP32[$pProgram>>2]|0;
  $148 = (($147) + 20|0);
  HEAP32[$148>>2] = $146;
  $149 = HEAP32[$pSubParse>>2]|0;
  $150 = (($149) + 428|0);
  $151 = HEAP32[$150>>2]|0;
  $152 = HEAP32[$pPrg>>2]|0;
  $153 = (($152) + 16|0);
  HEAP32[$153>>2] = $151;
  $154 = HEAP32[$pSubParse>>2]|0;
  $155 = (($154) + 432|0);
  $156 = HEAP32[$155>>2]|0;
  $157 = HEAP32[$pPrg>>2]|0;
  $158 = (($157) + 16|0);
  $159 = (($158) + 4|0);
  HEAP32[$159>>2] = $156;
  $160 = HEAP32[$v>>2]|0;
  _sqlite3VdbeDelete($160);
 }
 $161 = HEAP32[$pSubParse>>2]|0;
 _sqlite3ParserReset($161);
 $162 = HEAP32[$db>>2]|0;
 $163 = HEAP32[$pSubParse>>2]|0;
 _sqlite3DbFree($162,$163);
 $164 = HEAP32[$pPrg>>2]|0;
 HEAP32[$0>>2] = $164;
 $165 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($165|0);
}
function _sqlite3VdbeLinkSubProgram($pVdbe,$p) {
 $pVdbe = $pVdbe|0;
 $p = $p|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $1 = sp;
 HEAP32[$0>>2] = $pVdbe;
 HEAP32[$1>>2] = $p;
 $2 = HEAP32[$0>>2]|0;
 $3 = (($2) + 192|0);
 $4 = HEAP32[$3>>2]|0;
 $5 = HEAP32[$1>>2]|0;
 $6 = (($5) + 24|0);
 HEAP32[$6>>2] = $4;
 $7 = HEAP32[$1>>2]|0;
 $8 = HEAP32[$0>>2]|0;
 $9 = (($8) + 192|0);
 HEAP32[$9>>2] = $7;
 STACKTOP = sp;return;
}
function _codeTriggerProgram($pParse,$pStepList,$orconf) {
 $pParse = $pParse|0;
 $pStepList = $pStepList|0;
 $orconf = $orconf|0;
 var $$sink = 0, $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0;
 var $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0;
 var $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0;
 var $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0;
 var $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $9 = 0, $db = 0, $pSelect = 0, $pStep = 0, $sDest = 0, $v = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $0 = sp + 44|0;
 $1 = sp + 40|0;
 $2 = sp + 36|0;
 $pStep = sp + 32|0;
 $v = sp + 28|0;
 $db = sp + 24|0;
 $sDest = sp + 4|0;
 $pSelect = sp;
 HEAP32[$0>>2] = $pParse;
 HEAP32[$1>>2] = $pStepList;
 HEAP32[$2>>2] = $orconf;
 $3 = HEAP32[$0>>2]|0;
 $4 = (($3) + 8|0);
 $5 = HEAP32[$4>>2]|0;
 HEAP32[$v>>2] = $5;
 $6 = HEAP32[$0>>2]|0;
 $7 = HEAP32[$6>>2]|0;
 HEAP32[$db>>2] = $7;
 $8 = HEAP32[$1>>2]|0;
 HEAP32[$pStep>>2] = $8;
 while(1) {
  $9 = HEAP32[$pStep>>2]|0;
  $10 = ($9|0)!=(0|0);
  if (!($10)) {
   break;
  }
  $11 = HEAP32[$2>>2]|0;
  $12 = ($11|0)==(10);
  if ($12) {
   $13 = HEAP32[$pStep>>2]|0;
   $14 = (($13) + 1|0);
   $15 = HEAP8[$14>>0]|0;
   $$sink = $15;
  } else {
   $16 = HEAP32[$2>>2]|0;
   $17 = $16&255;
   $$sink = $17;
  }
  $18 = $$sink&255;
  $19 = $18&255;
  $20 = HEAP32[$0>>2]|0;
  $21 = (($20) + 437|0);
  HEAP8[$21>>0] = $19;
  $22 = HEAP32[$pStep>>2]|0;
  $23 = HEAP8[$22>>0]|0;
  $24 = $23&255;
  if ((($24|0) == 109)) {
   $61 = HEAP32[$0>>2]|0;
   $62 = HEAP32[$0>>2]|0;
   $63 = HEAP32[$pStep>>2]|0;
   $64 = (_targetSrcList($62,$63)|0);
   $65 = HEAP32[$db>>2]|0;
   $66 = HEAP32[$pStep>>2]|0;
   $67 = (($66) + 20|0);
   $68 = HEAP32[$67>>2]|0;
   $69 = (_sqlite3ExprDup($65,$68,0)|0);
   _sqlite3DeleteFrom($61,$64,$69);
  } else if ((($24|0) == 108)) {
   $43 = HEAP32[$0>>2]|0;
   $44 = HEAP32[$0>>2]|0;
   $45 = HEAP32[$pStep>>2]|0;
   $46 = (_targetSrcList($44,$45)|0);
   $47 = HEAP32[$db>>2]|0;
   $48 = HEAP32[$pStep>>2]|0;
   $49 = (($48) + 8|0);
   $50 = HEAP32[$49>>2]|0;
   $51 = (_sqlite3SelectDup($47,$50,0)|0);
   $52 = HEAP32[$db>>2]|0;
   $53 = HEAP32[$pStep>>2]|0;
   $54 = (($53) + 28|0);
   $55 = HEAP32[$54>>2]|0;
   $56 = (_sqlite3IdListDup($52,$55)|0);
   $57 = HEAP32[$0>>2]|0;
   $58 = (($57) + 437|0);
   $59 = HEAP8[$58>>0]|0;
   $60 = $59&255;
   _sqlite3Insert($43,$46,$51,$56,$60);
  } else if ((($24|0) == 110)) {
   $25 = HEAP32[$0>>2]|0;
   $26 = HEAP32[$0>>2]|0;
   $27 = HEAP32[$pStep>>2]|0;
   $28 = (_targetSrcList($26,$27)|0);
   $29 = HEAP32[$db>>2]|0;
   $30 = HEAP32[$pStep>>2]|0;
   $31 = (($30) + 24|0);
   $32 = HEAP32[$31>>2]|0;
   $33 = (_sqlite3ExprListDup($29,$32,0)|0);
   $34 = HEAP32[$db>>2]|0;
   $35 = HEAP32[$pStep>>2]|0;
   $36 = (($35) + 20|0);
   $37 = HEAP32[$36>>2]|0;
   $38 = (_sqlite3ExprDup($34,$37,0)|0);
   $39 = HEAP32[$0>>2]|0;
   $40 = (($39) + 437|0);
   $41 = HEAP8[$40>>0]|0;
   $42 = $41&255;
   _sqlite3Update($25,$28,$33,$38,$42);
  } else {
   $70 = HEAP32[$db>>2]|0;
   $71 = HEAP32[$pStep>>2]|0;
   $72 = (($71) + 8|0);
   $73 = HEAP32[$72>>2]|0;
   $74 = (_sqlite3SelectDup($70,$73,0)|0);
   HEAP32[$pSelect>>2] = $74;
   _sqlite3SelectDestInit($sDest,4,0);
   $75 = HEAP32[$0>>2]|0;
   $76 = HEAP32[$pSelect>>2]|0;
   (_sqlite3Select($75,$76,$sDest)|0);
   $77 = HEAP32[$db>>2]|0;
   $78 = HEAP32[$pSelect>>2]|0;
   _sqlite3SelectDelete($77,$78);
  }
  $79 = HEAP32[$pStep>>2]|0;
  $80 = HEAP8[$79>>0]|0;
  $81 = $80&255;
  $82 = ($81|0)!=(119);
  if ($82) {
   $83 = HEAP32[$v>>2]|0;
   (_sqlite3VdbeAddOp0($83,98)|0);
  }
  $84 = HEAP32[$pStep>>2]|0;
  $85 = (($84) + 32|0);
  $86 = HEAP32[$85>>2]|0;
  HEAP32[$pStep>>2] = $86;
 }
 STACKTOP = sp;return 0;
}
function _transferParseError($pTo,$pFrom) {
 $pTo = $pTo|0;
 $pFrom = $pFrom|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0;
 var label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $1 = sp;
 HEAP32[$0>>2] = $pTo;
 HEAP32[$1>>2] = $pFrom;
 $2 = HEAP32[$0>>2]|0;
 $3 = (($2) + 64|0);
 $4 = HEAP32[$3>>2]|0;
 $5 = ($4|0)==(0);
 $6 = HEAP32[$1>>2]|0;
 if ($5) {
  $7 = (($6) + 4|0);
  $8 = HEAP32[$7>>2]|0;
  $9 = HEAP32[$0>>2]|0;
  $10 = (($9) + 4|0);
  HEAP32[$10>>2] = $8;
  $11 = HEAP32[$1>>2]|0;
  $12 = (($11) + 64|0);
  $13 = HEAP32[$12>>2]|0;
  $14 = HEAP32[$0>>2]|0;
  $15 = (($14) + 64|0);
  HEAP32[$15>>2] = $13;
  STACKTOP = sp;return;
 } else {
  $16 = HEAP32[$6>>2]|0;
  $17 = HEAP32[$1>>2]|0;
  $18 = (($17) + 4|0);
  $19 = HEAP32[$18>>2]|0;
  _sqlite3DbFree($16,$19);
  STACKTOP = sp;return;
 }
}
function _sqlite3VdbeTakeOpArray($p,$pnOp,$pnMaxArg) {
 $p = $p|0;
 $pnOp = $pnOp|0;
 $pnMaxArg = $pnMaxArg|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $aOp = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $1 = sp + 8|0;
 $2 = sp + 4|0;
 $aOp = sp;
 HEAP32[$0>>2] = $p;
 HEAP32[$1>>2] = $pnOp;
 HEAP32[$2>>2] = $pnMaxArg;
 $3 = HEAP32[$0>>2]|0;
 $4 = (($3) + 4|0);
 $5 = HEAP32[$4>>2]|0;
 HEAP32[$aOp>>2] = $5;
 $6 = HEAP32[$0>>2]|0;
 $7 = HEAP32[$2>>2]|0;
 _resolveP2Values($6,$7);
 $8 = HEAP32[$0>>2]|0;
 $9 = (($8) + 32|0);
 $10 = HEAP32[$9>>2]|0;
 $11 = HEAP32[$1>>2]|0;
 HEAP32[$11>>2] = $10;
 $12 = HEAP32[$0>>2]|0;
 $13 = (($12) + 4|0);
 HEAP32[$13>>2] = 0;
 $14 = HEAP32[$aOp>>2]|0;
 STACKTOP = sp;return ($14|0);
}
function _resolveP2Values($p,$pMaxFuncArgs) {
 $p = $p|0;
 $pMaxFuncArgs = $pMaxFuncArgs|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0;
 var $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0;
 var $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0;
 var $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0;
 var $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0;
 var $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $aLabel = 0, $i = 0, $n = 0, $nMaxArgs = 0, $opcode = 0, $pOp = 0, $pParse = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $0 = sp + 28|0;
 $1 = sp + 24|0;
 $i = sp + 20|0;
 $nMaxArgs = sp + 16|0;
 $pOp = sp + 12|0;
 $pParse = sp + 8|0;
 $aLabel = sp + 4|0;
 $opcode = sp + 32|0;
 $n = sp;
 HEAP32[$0>>2] = $p;
 HEAP32[$1>>2] = $pMaxFuncArgs;
 $2 = HEAP32[$1>>2]|0;
 $3 = HEAP32[$2>>2]|0;
 HEAP32[$nMaxArgs>>2] = $3;
 $4 = HEAP32[$0>>2]|0;
 $5 = (($4) + 24|0);
 $6 = HEAP32[$5>>2]|0;
 HEAP32[$pParse>>2] = $6;
 $7 = HEAP32[$pParse>>2]|0;
 $8 = (($7) + 112|0);
 $9 = HEAP32[$8>>2]|0;
 HEAP32[$aLabel>>2] = $9;
 $10 = HEAP32[$0>>2]|0;
 $11 = (($10) + 88|0);
 $12 = HEAP16[$11>>1]|0;
 $13 = $12 & -257;
 $14 = $13 | 256;
 HEAP16[$11>>1] = $14;
 $15 = HEAP32[$0>>2]|0;
 $16 = (($15) + 88|0);
 $17 = HEAP16[$16>>1]|0;
 $18 = $17 & -513;
 HEAP16[$16>>1] = $18;
 $19 = HEAP32[$0>>2]|0;
 $20 = (($19) + 4|0);
 $21 = HEAP32[$20>>2]|0;
 HEAP32[$pOp>>2] = $21;
 $22 = HEAP32[$0>>2]|0;
 $23 = (($22) + 32|0);
 $24 = HEAP32[$23>>2]|0;
 $25 = (($24) - 1)|0;
 HEAP32[$i>>2] = $25;
 while(1) {
  $26 = HEAP32[$i>>2]|0;
  $27 = ($26|0)>=(0);
  if (!($27)) {
   break;
  }
  $28 = HEAP32[$pOp>>2]|0;
  $29 = HEAP8[$28>>0]|0;
  HEAP8[$opcode>>0] = $29;
  $30 = HEAP8[$opcode>>0]|0;
  $31 = $30&255;
  switch ($31|0) {
  case 12: case 13: case 11:  {
   $55 = HEAP32[$0>>2]|0;
   $56 = (($55) + 88|0);
   $57 = HEAP16[$56>>1]|0;
   $58 = $57 & -257;
   HEAP16[$56>>1] = $58;
   $59 = HEAP32[$0>>2]|0;
   $60 = (($59) + 88|0);
   $61 = HEAP16[$60>>1]|0;
   $62 = $61 & -513;
   $63 = $62 | 512;
   HEAP16[$60>>1] = $63;
   break;
  }
  case 10: case 1:  {
   $32 = HEAP32[$pOp>>2]|0;
   $33 = (($32) + 3|0);
   $34 = HEAP8[$33>>0]|0;
   $35 = $34&255;
   $36 = HEAP32[$nMaxArgs>>2]|0;
   $37 = ($35|0)>($36|0);
   if ($37) {
    $38 = HEAP32[$pOp>>2]|0;
    $39 = (($38) + 3|0);
    $40 = HEAP8[$39>>0]|0;
    $41 = $40&255;
    HEAP32[$nMaxArgs>>2] = $41;
   }
   break;
  }
  case 14:  {
   $72 = HEAP32[$pOp>>2]|0;
   $73 = (($72) + -20|0);
   $74 = (($73) + 4|0);
   $75 = HEAP32[$74>>2]|0;
   HEAP32[$n>>2] = $75;
   $76 = HEAP32[$n>>2]|0;
   $77 = HEAP32[$nMaxArgs>>2]|0;
   $78 = ($76|0)>($77|0);
   if ($78) {
    $79 = HEAP32[$n>>2]|0;
    HEAP32[$nMaxArgs>>2] = $79;
   }
   break;
  }
  case 4:  {
   $42 = HEAP32[$pOp>>2]|0;
   $43 = (($42) + 8|0);
   $44 = HEAP32[$43>>2]|0;
   $45 = ($44|0)!=(0);
   if ($45) {
    $46 = HEAP32[$0>>2]|0;
    $47 = (($46) + 88|0);
    $48 = HEAP16[$47>>1]|0;
    $49 = $48 & -257;
    HEAP16[$47>>1] = $49;
    label = 8;
   } else {
    label = 8;
   }
   break;
  }
  case 2: case 3:  {
   label = 8;
   break;
  }
  case 15:  {
   $64 = HEAP32[$pOp>>2]|0;
   $65 = (($64) + 8|0);
   $66 = HEAP32[$65>>2]|0;
   $67 = HEAP32[$nMaxArgs>>2]|0;
   $68 = ($66|0)>($67|0);
   if ($68) {
    $69 = HEAP32[$pOp>>2]|0;
    $70 = (($69) + 8|0);
    $71 = HEAP32[$70>>2]|0;
    HEAP32[$nMaxArgs>>2] = $71;
   }
   break;
  }
  case 5: case 7: case 9:  {
   $80 = HEAP32[$pOp>>2]|0;
   $81 = (($80) + 16|0);
   HEAP32[$81>>2] = 46;
   $82 = HEAP32[$pOp>>2]|0;
   $83 = (($82) + 1|0);
   HEAP8[$83>>0] = -19;
   break;
  }
  case 6: case 8:  {
   $84 = HEAP32[$pOp>>2]|0;
   $85 = (($84) + 16|0);
   HEAP32[$85>>2] = 47;
   $86 = HEAP32[$pOp>>2]|0;
   $87 = (($86) + 1|0);
   HEAP8[$87>>0] = -19;
   break;
  }
  default: {
  }
  }
  if ((label|0) == 8) {
   label = 0;
   $50 = HEAP32[$0>>2]|0;
   $51 = (($50) + 88|0);
   $52 = HEAP16[$51>>1]|0;
   $53 = $52 & -513;
   $54 = $53 | 512;
   HEAP16[$51>>1] = $54;
  }
  $88 = HEAP8[$opcode>>0]|0;
  $89 = $88&255;
  $90 = (24192 + ($89)|0);
  $91 = HEAP8[$90>>0]|0;
  $92 = HEAP32[$pOp>>2]|0;
  $93 = (($92) + 2|0);
  HEAP8[$93>>0] = $91;
  $94 = HEAP32[$pOp>>2]|0;
  $95 = (($94) + 2|0);
  $96 = HEAP8[$95>>0]|0;
  $97 = $96&255;
  $98 = $97 & 1;
  $99 = ($98|0)!=(0);
  if ($99) {
   $100 = HEAP32[$pOp>>2]|0;
   $101 = (($100) + 8|0);
   $102 = HEAP32[$101>>2]|0;
   $103 = ($102|0)<(0);
   if ($103) {
    $104 = HEAP32[$pOp>>2]|0;
    $105 = (($104) + 8|0);
    $106 = HEAP32[$105>>2]|0;
    $107 = (-1 - ($106))|0;
    $108 = HEAP32[$aLabel>>2]|0;
    $109 = (($108) + ($107<<2)|0);
    $110 = HEAP32[$109>>2]|0;
    $111 = HEAP32[$pOp>>2]|0;
    $112 = (($111) + 8|0);
    HEAP32[$112>>2] = $110;
   }
  }
  $113 = HEAP32[$i>>2]|0;
  $114 = (($113) + -1)|0;
  HEAP32[$i>>2] = $114;
  $115 = HEAP32[$pOp>>2]|0;
  $116 = (($115) + 20|0);
  HEAP32[$pOp>>2] = $116;
 }
 $117 = HEAP32[$0>>2]|0;
 $118 = HEAP32[$117>>2]|0;
 $119 = HEAP32[$pParse>>2]|0;
 $120 = (($119) + 112|0);
 $121 = HEAP32[$120>>2]|0;
 _sqlite3DbFree($118,$121);
 $122 = HEAP32[$pParse>>2]|0;
 $123 = (($122) + 112|0);
 HEAP32[$123>>2] = 0;
 $124 = HEAP32[$pParse>>2]|0;
 $125 = (($124) + 108|0);
 HEAP32[$125>>2] = 0;
 $126 = HEAP32[$nMaxArgs>>2]|0;
 $127 = HEAP32[$1>>2]|0;
 HEAP32[$127>>2] = $126;
 STACKTOP = sp;return;
}
function _sqlite3BtreeNext($pCur,$pRes) {
 $pCur = $pCur|0;
 $pRes = $pRes|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $7 = 0;
 var $8 = 0, $9 = 0, $pPage = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $1 = sp + 8|0;
 $2 = sp + 4|0;
 $pPage = sp;
 HEAP32[$1>>2] = $pCur;
 HEAP32[$2>>2] = $pRes;
 $3 = HEAP32[$1>>2]|0;
 $4 = (($3) + 24|0);
 $5 = (($4) + 20|0);
 HEAP16[$5>>1] = 0;
 $6 = HEAP32[$1>>2]|0;
 $7 = (($6) + 72|0);
 $8 = HEAP8[$7>>0]|0;
 $9 = $8&255;
 $10 = $9 & -7;
 $11 = $10&255;
 HEAP8[$7>>0] = $11;
 $12 = HEAP32[$2>>2]|0;
 HEAP32[$12>>2] = 0;
 $13 = HEAP32[$1>>2]|0;
 $14 = (($13) + 73|0);
 $15 = HEAP8[$14>>0]|0;
 $16 = $15&255;
 $17 = ($16|0)!=(1);
 $18 = HEAP32[$1>>2]|0;
 if ($17) {
  $19 = HEAP32[$2>>2]|0;
  $20 = (_btreeNext($18,$19)|0);
  HEAP32[$0>>2] = $20;
  $61 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($61|0);
 }
 $21 = (($18) + 76|0);
 $22 = HEAP16[$21>>1]|0;
 $23 = $22 << 16 >> 16;
 $24 = HEAP32[$1>>2]|0;
 $25 = (($24) + 120|0);
 $26 = (($25) + ($23<<2)|0);
 $27 = HEAP32[$26>>2]|0;
 HEAP32[$pPage>>2] = $27;
 $28 = HEAP32[$1>>2]|0;
 $29 = (($28) + 76|0);
 $30 = HEAP16[$29>>1]|0;
 $31 = $30 << 16 >> 16;
 $32 = HEAP32[$1>>2]|0;
 $33 = (($32) + 78|0);
 $34 = (($33) + ($31<<1)|0);
 $35 = HEAP16[$34>>1]|0;
 $36 = (($35) + 1)<<16>>16;
 HEAP16[$34>>1] = $36;
 $37 = $36&65535;
 $38 = HEAP32[$pPage>>2]|0;
 $39 = (($38) + 18|0);
 $40 = HEAP16[$39>>1]|0;
 $41 = $40&65535;
 $42 = ($37|0)>=($41|0);
 if ($42) {
  $43 = HEAP32[$1>>2]|0;
  $44 = (($43) + 76|0);
  $45 = HEAP16[$44>>1]|0;
  $46 = $45 << 16 >> 16;
  $47 = HEAP32[$1>>2]|0;
  $48 = (($47) + 78|0);
  $49 = (($48) + ($46<<1)|0);
  $50 = HEAP16[$49>>1]|0;
  $51 = (($50) + -1)<<16>>16;
  HEAP16[$49>>1] = $51;
  $52 = HEAP32[$1>>2]|0;
  $53 = HEAP32[$2>>2]|0;
  $54 = (_btreeNext($52,$53)|0);
  HEAP32[$0>>2] = $54;
  $61 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($61|0);
 }
 $55 = HEAP32[$pPage>>2]|0;
 $56 = (($55) + 5|0);
 $57 = HEAP8[$56>>0]|0;
 $58 = ($57<<24>>24)!=(0);
 if ($58) {
  HEAP32[$0>>2] = 0;
  $61 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($61|0);
 } else {
  $59 = HEAP32[$1>>2]|0;
  $60 = (_moveToLeftmost($59)|0);
  HEAP32[$0>>2] = $60;
  $61 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($61|0);
 }
 return 0|0;
}
function _sqlite3BtreePrevious($pCur,$pRes) {
 $pCur = $pCur|0;
 $pRes = $pRes|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $1 = sp + 4|0;
 $2 = sp;
 HEAP32[$1>>2] = $pCur;
 HEAP32[$2>>2] = $pRes;
 $3 = HEAP32[$2>>2]|0;
 HEAP32[$3>>2] = 0;
 $4 = HEAP32[$1>>2]|0;
 $5 = (($4) + 72|0);
 $6 = HEAP8[$5>>0]|0;
 $7 = $6&255;
 $8 = $7 & -15;
 $9 = $8&255;
 HEAP8[$5>>0] = $9;
 $10 = HEAP32[$1>>2]|0;
 $11 = (($10) + 24|0);
 $12 = (($11) + 20|0);
 HEAP16[$12>>1] = 0;
 $13 = HEAP32[$1>>2]|0;
 $14 = (($13) + 73|0);
 $15 = HEAP8[$14>>0]|0;
 $16 = $15&255;
 $17 = ($16|0)!=(1);
 if (!($17)) {
  $18 = HEAP32[$1>>2]|0;
  $19 = (($18) + 76|0);
  $20 = HEAP16[$19>>1]|0;
  $21 = $20 << 16 >> 16;
  $22 = HEAP32[$1>>2]|0;
  $23 = (($22) + 78|0);
  $24 = (($23) + ($21<<1)|0);
  $25 = HEAP16[$24>>1]|0;
  $26 = $25&65535;
  $27 = ($26|0)==(0);
  if (!($27)) {
   $28 = HEAP32[$1>>2]|0;
   $29 = (($28) + 76|0);
   $30 = HEAP16[$29>>1]|0;
   $31 = $30 << 16 >> 16;
   $32 = HEAP32[$1>>2]|0;
   $33 = (($32) + 120|0);
   $34 = (($33) + ($31<<2)|0);
   $35 = HEAP32[$34>>2]|0;
   $36 = (($35) + 5|0);
   $37 = HEAP8[$36>>0]|0;
   $38 = $37&255;
   $39 = ($38|0)==(0);
   if (!($39)) {
    $43 = HEAP32[$1>>2]|0;
    $44 = (($43) + 76|0);
    $45 = HEAP16[$44>>1]|0;
    $46 = $45 << 16 >> 16;
    $47 = HEAP32[$1>>2]|0;
    $48 = (($47) + 78|0);
    $49 = (($48) + ($46<<1)|0);
    $50 = HEAP16[$49>>1]|0;
    $51 = (($50) + -1)<<16>>16;
    HEAP16[$49>>1] = $51;
    HEAP32[$0>>2] = 0;
    $52 = HEAP32[$0>>2]|0;
    STACKTOP = sp;return ($52|0);
   }
  }
 }
 $40 = HEAP32[$1>>2]|0;
 $41 = HEAP32[$2>>2]|0;
 $42 = (_btreePrevious($40,$41)|0);
 HEAP32[$0>>2] = $42;
 $52 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($52|0);
}
function _btreePrevious($pCur,$pRes) {
 $pCur = $pCur|0;
 $pRes = $pRes|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0;
 var $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0;
 var $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0;
 var $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0;
 var $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $idx = 0;
 var $pPage = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 20|0;
 $1 = sp + 16|0;
 $2 = sp + 12|0;
 $rc = sp + 8|0;
 $pPage = sp + 4|0;
 $idx = sp;
 HEAP32[$1>>2] = $pCur;
 HEAP32[$2>>2] = $pRes;
 $3 = HEAP32[$1>>2]|0;
 $4 = (($3) + 73|0);
 $5 = HEAP8[$4>>0]|0;
 $6 = $5&255;
 $7 = ($6|0)!=(1);
 if ($7) {
  $8 = HEAP32[$1>>2]|0;
  $9 = (($8) + 73|0);
  $10 = HEAP8[$9>>0]|0;
  $11 = $10&255;
  $12 = ($11|0)>=(3);
  if ($12) {
   $13 = HEAP32[$1>>2]|0;
   $14 = (_btreeRestoreCursorPosition($13)|0);
   $15 = $14;
  } else {
   $15 = 0;
  }
  HEAP32[$rc>>2] = $15;
  $16 = HEAP32[$rc>>2]|0;
  $17 = ($16|0)!=(0);
  if ($17) {
   $18 = HEAP32[$rc>>2]|0;
   HEAP32[$0>>2] = $18;
   $138 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($138|0);
  }
  $19 = HEAP32[$1>>2]|0;
  $20 = (($19) + 73|0);
  $21 = HEAP8[$20>>0]|0;
  $22 = $21&255;
  $23 = (0)==($22|0);
  if ($23) {
   $24 = HEAP32[$2>>2]|0;
   HEAP32[$24>>2] = 1;
   HEAP32[$0>>2] = 0;
   $138 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($138|0);
  }
  $25 = HEAP32[$1>>2]|0;
  $26 = (($25) + 68|0);
  $27 = HEAP32[$26>>2]|0;
  $28 = ($27|0)!=(0);
  if ($28) {
   $29 = HEAP32[$1>>2]|0;
   $30 = (($29) + 73|0);
   HEAP8[$30>>0] = 1;
   $31 = HEAP32[$1>>2]|0;
   $32 = (($31) + 68|0);
   $33 = HEAP32[$32>>2]|0;
   $34 = ($33|0)<(0);
   $35 = HEAP32[$1>>2]|0;
   $36 = (($35) + 68|0);
   HEAP32[$36>>2] = 0;
   if ($34) {
    HEAP32[$0>>2] = 0;
    $138 = HEAP32[$0>>2]|0;
    STACKTOP = sp;return ($138|0);
   }
  }
 }
 $37 = HEAP32[$1>>2]|0;
 $38 = (($37) + 76|0);
 $39 = HEAP16[$38>>1]|0;
 $40 = $39 << 16 >> 16;
 $41 = HEAP32[$1>>2]|0;
 $42 = (($41) + 120|0);
 $43 = (($42) + ($40<<2)|0);
 $44 = HEAP32[$43>>2]|0;
 HEAP32[$pPage>>2] = $44;
 $45 = HEAP32[$pPage>>2]|0;
 $46 = (($45) + 5|0);
 $47 = HEAP8[$46>>0]|0;
 $48 = ($47<<24>>24)!=(0);
 do {
  if ($48) {
   while(1) {
    $94 = HEAP32[$1>>2]|0;
    $95 = (($94) + 76|0);
    $96 = HEAP16[$95>>1]|0;
    $97 = $96 << 16 >> 16;
    $98 = HEAP32[$1>>2]|0;
    $99 = (($98) + 78|0);
    $100 = (($99) + ($97<<1)|0);
    $101 = HEAP16[$100>>1]|0;
    $102 = $101&65535;
    $103 = ($102|0)==(0);
    $104 = HEAP32[$1>>2]|0;
    $105 = (($104) + 76|0);
    $106 = HEAP16[$105>>1]|0;
    $107 = $106 << 16 >> 16;
    if (!($103)) {
     break;
    }
    $108 = ($107|0)==(0);
    $109 = HEAP32[$1>>2]|0;
    if ($108) {
     label = 17;
     break;
    }
    _moveToParent($109);
   }
   if ((label|0) == 17) {
    $110 = (($109) + 73|0);
    HEAP8[$110>>0] = 0;
    $111 = HEAP32[$2>>2]|0;
    HEAP32[$111>>2] = 1;
    HEAP32[$0>>2] = 0;
    $138 = HEAP32[$0>>2]|0;
    STACKTOP = sp;return ($138|0);
   }
   $112 = HEAP32[$1>>2]|0;
   $113 = (($112) + 78|0);
   $114 = (($113) + ($107<<1)|0);
   $115 = HEAP16[$114>>1]|0;
   $116 = (($115) + -1)<<16>>16;
   HEAP16[$114>>1] = $116;
   $117 = HEAP32[$1>>2]|0;
   $118 = (($117) + 76|0);
   $119 = HEAP16[$118>>1]|0;
   $120 = $119 << 16 >> 16;
   $121 = HEAP32[$1>>2]|0;
   $122 = (($121) + 120|0);
   $123 = (($122) + ($120<<2)|0);
   $124 = HEAP32[$123>>2]|0;
   HEAP32[$pPage>>2] = $124;
   $125 = HEAP32[$pPage>>2]|0;
   $126 = (($125) + 2|0);
   $127 = HEAP8[$126>>0]|0;
   $128 = $127&255;
   $129 = ($128|0)!=(0);
   if ($129) {
    $130 = HEAP32[$pPage>>2]|0;
    $131 = (($130) + 5|0);
    $132 = HEAP8[$131>>0]|0;
    $133 = ($132<<24>>24)!=(0);
    if (!($133)) {
     $134 = HEAP32[$1>>2]|0;
     $135 = HEAP32[$2>>2]|0;
     $136 = (_sqlite3BtreePrevious($134,$135)|0);
     HEAP32[$rc>>2] = $136;
     break;
    }
   }
   HEAP32[$rc>>2] = 0;
  } else {
   $49 = HEAP32[$1>>2]|0;
   $50 = (($49) + 76|0);
   $51 = HEAP16[$50>>1]|0;
   $52 = $51 << 16 >> 16;
   $53 = HEAP32[$1>>2]|0;
   $54 = (($53) + 78|0);
   $55 = (($54) + ($52<<1)|0);
   $56 = HEAP16[$55>>1]|0;
   $57 = $56&65535;
   HEAP32[$idx>>2] = $57;
   $58 = HEAP32[$1>>2]|0;
   $59 = HEAP32[$pPage>>2]|0;
   $60 = (($59) + 56|0);
   $61 = HEAP32[$60>>2]|0;
   $62 = HEAP32[$pPage>>2]|0;
   $63 = (($62) + 20|0);
   $64 = HEAP16[$63>>1]|0;
   $65 = $64&65535;
   $66 = HEAP32[$idx>>2]|0;
   $67 = $66<<1;
   $68 = HEAP32[$pPage>>2]|0;
   $69 = (($68) + 64|0);
   $70 = HEAP32[$69>>2]|0;
   $71 = (($70) + ($67)|0);
   $72 = HEAP8[$71>>0]|0;
   $73 = $72&255;
   $74 = $73 << 8;
   $75 = HEAP32[$idx>>2]|0;
   $76 = $75<<1;
   $77 = HEAP32[$pPage>>2]|0;
   $78 = (($77) + 64|0);
   $79 = HEAP32[$78>>2]|0;
   $80 = (($79) + ($76)|0);
   $81 = (($80) + 1|0);
   $82 = HEAP8[$81>>0]|0;
   $83 = $82&255;
   $84 = $74 | $83;
   $85 = $65 & $84;
   $86 = (($61) + ($85)|0);
   $87 = (_sqlite3Get4byte($86)|0);
   $88 = (_moveToChild($58,$87)|0);
   HEAP32[$rc>>2] = $88;
   $89 = HEAP32[$rc>>2]|0;
   $90 = ($89|0)!=(0);
   if (!($90)) {
    $92 = HEAP32[$1>>2]|0;
    $93 = (_moveToRightmost($92)|0);
    HEAP32[$rc>>2] = $93;
    break;
   }
   $91 = HEAP32[$rc>>2]|0;
   HEAP32[$0>>2] = $91;
   $138 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($138|0);
  }
 } while(0);
 $137 = HEAP32[$rc>>2]|0;
 HEAP32[$0>>2] = $137;
 $138 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($138|0);
}
function _btreeRestoreCursorPosition($pCur) {
 $pCur = $pCur|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $1 = sp + 4|0;
 $rc = sp;
 HEAP32[$1>>2] = $pCur;
 $2 = HEAP32[$1>>2]|0;
 $3 = (($2) + 73|0);
 $4 = HEAP8[$3>>0]|0;
 $5 = $4&255;
 $6 = ($5|0)==(4);
 $7 = HEAP32[$1>>2]|0;
 if ($6) {
  $8 = (($7) + 68|0);
  $9 = HEAP32[$8>>2]|0;
  HEAP32[$0>>2] = $9;
  $45 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($45|0);
 }
 $10 = (($7) + 73|0);
 HEAP8[$10>>0] = 0;
 $11 = HEAP32[$1>>2]|0;
 $12 = HEAP32[$1>>2]|0;
 $13 = (($12) + 56|0);
 $14 = HEAP32[$13>>2]|0;
 $15 = HEAP32[$1>>2]|0;
 $16 = (($15) + 48|0);
 $17 = $16;
 $18 = $17;
 $19 = HEAP32[$18>>2]|0;
 $20 = (($17) + 4)|0;
 $21 = $20;
 $22 = HEAP32[$21>>2]|0;
 $23 = HEAP32[$1>>2]|0;
 $24 = (($23) + 68|0);
 $25 = (_btreeMoveto($11,$14,$19,$22,0,$24)|0);
 HEAP32[$rc>>2] = $25;
 $26 = HEAP32[$rc>>2]|0;
 $27 = ($26|0)==(0);
 if ($27) {
  $28 = HEAP32[$1>>2]|0;
  $29 = (($28) + 56|0);
  $30 = HEAP32[$29>>2]|0;
  _sqlite3_free($30);
  $31 = HEAP32[$1>>2]|0;
  $32 = (($31) + 56|0);
  HEAP32[$32>>2] = 0;
  $33 = HEAP32[$1>>2]|0;
  $34 = (($33) + 68|0);
  $35 = HEAP32[$34>>2]|0;
  $36 = ($35|0)!=(0);
  if ($36) {
   $37 = HEAP32[$1>>2]|0;
   $38 = (($37) + 73|0);
   $39 = HEAP8[$38>>0]|0;
   $40 = $39&255;
   $41 = ($40|0)==(1);
   if ($41) {
    $42 = HEAP32[$1>>2]|0;
    $43 = (($42) + 73|0);
    HEAP8[$43>>0] = 2;
   }
  }
 }
 $44 = HEAP32[$rc>>2]|0;
 HEAP32[$0>>2] = $44;
 $45 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($45|0);
}
function _moveToChild($pCur,$newPgno) {
 $pCur = $pCur|0;
 $newPgno = $newPgno|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $8 = 0, $9 = 0, $i = 0, $pBt = 0, $pNewPage = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 24|0;
 $1 = sp + 20|0;
 $2 = sp + 16|0;
 $rc = sp + 12|0;
 $i = sp + 8|0;
 $pNewPage = sp + 4|0;
 $pBt = sp;
 HEAP32[$1>>2] = $pCur;
 HEAP32[$2>>2] = $newPgno;
 $3 = HEAP32[$1>>2]|0;
 $4 = (($3) + 76|0);
 $5 = HEAP16[$4>>1]|0;
 $6 = $5 << 16 >> 16;
 HEAP32[$i>>2] = $6;
 $7 = HEAP32[$1>>2]|0;
 $8 = (($7) + 4|0);
 $9 = HEAP32[$8>>2]|0;
 HEAP32[$pBt>>2] = $9;
 $10 = HEAP32[$1>>2]|0;
 $11 = (($10) + 76|0);
 $12 = HEAP16[$11>>1]|0;
 $13 = $12 << 16 >> 16;
 $14 = ($13|0)>=(19);
 if ($14) {
  $15 = (_sqlite3CorruptError(56561)|0);
  HEAP32[$0>>2] = $15;
  $72 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($72|0);
 }
 $16 = HEAP32[$pBt>>2]|0;
 $17 = HEAP32[$2>>2]|0;
 $18 = HEAP32[$1>>2]|0;
 $19 = (($18) + 72|0);
 $20 = HEAP8[$19>>0]|0;
 $21 = $20&255;
 $22 = $21 & 1;
 $23 = ($22|0)==(0);
 $24 = $23 ? 2 : 0;
 $25 = (_getAndInitPage($16,$17,$pNewPage,$24)|0);
 HEAP32[$rc>>2] = $25;
 $26 = HEAP32[$rc>>2]|0;
 $27 = ($26|0)!=(0);
 if ($27) {
  $28 = HEAP32[$rc>>2]|0;
  HEAP32[$0>>2] = $28;
  $72 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($72|0);
 }
 $29 = HEAP32[$pNewPage>>2]|0;
 $30 = HEAP32[$i>>2]|0;
 $31 = (($30) + 1)|0;
 $32 = HEAP32[$1>>2]|0;
 $33 = (($32) + 120|0);
 $34 = (($33) + ($31<<2)|0);
 HEAP32[$34>>2] = $29;
 $35 = HEAP32[$i>>2]|0;
 $36 = (($35) + 1)|0;
 $37 = HEAP32[$1>>2]|0;
 $38 = (($37) + 78|0);
 $39 = (($38) + ($36<<1)|0);
 HEAP16[$39>>1] = 0;
 $40 = HEAP32[$1>>2]|0;
 $41 = (($40) + 76|0);
 $42 = HEAP16[$41>>1]|0;
 $43 = (($42) + 1)<<16>>16;
 HEAP16[$41>>1] = $43;
 $44 = HEAP32[$1>>2]|0;
 $45 = (($44) + 24|0);
 $46 = (($45) + 20|0);
 HEAP16[$46>>1] = 0;
 $47 = HEAP32[$1>>2]|0;
 $48 = (($47) + 72|0);
 $49 = HEAP8[$48>>0]|0;
 $50 = $49&255;
 $51 = $50 & -7;
 $52 = $51&255;
 HEAP8[$48>>0] = $52;
 $53 = HEAP32[$pNewPage>>2]|0;
 $54 = (($53) + 18|0);
 $55 = HEAP16[$54>>1]|0;
 $56 = $55&65535;
 $57 = ($56|0)<(1);
 if (!($57)) {
  $58 = HEAP32[$pNewPage>>2]|0;
  $59 = (($58) + 2|0);
  $60 = HEAP8[$59>>0]|0;
  $61 = $60&255;
  $62 = HEAP32[$i>>2]|0;
  $63 = HEAP32[$1>>2]|0;
  $64 = (($63) + 120|0);
  $65 = (($64) + ($62<<2)|0);
  $66 = HEAP32[$65>>2]|0;
  $67 = (($66) + 2|0);
  $68 = HEAP8[$67>>0]|0;
  $69 = $68&255;
  $70 = ($61|0)!=($69|0);
  if (!($70)) {
   HEAP32[$0>>2] = 0;
   $72 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($72|0);
  }
 }
 $71 = (_sqlite3CorruptError(56573)|0);
 HEAP32[$0>>2] = $71;
 $72 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($72|0);
}
function _moveToRightmost($pCur) {
 $pCur = $pCur|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $pPage = 0, $pgno = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 16|0;
 $1 = sp + 12|0;
 $pgno = sp + 8|0;
 $rc = sp + 4|0;
 $pPage = sp;
 HEAP32[$1>>2] = $pCur;
 HEAP32[$rc>>2] = 0;
 HEAP32[$pPage>>2] = 0;
 while(1) {
  $2 = HEAP32[$1>>2]|0;
  $3 = (($2) + 76|0);
  $4 = HEAP16[$3>>1]|0;
  $5 = $4 << 16 >> 16;
  $6 = HEAP32[$1>>2]|0;
  $7 = (($6) + 120|0);
  $8 = (($7) + ($5<<2)|0);
  $9 = HEAP32[$8>>2]|0;
  HEAP32[$pPage>>2] = $9;
  $10 = (($9) + 5|0);
  $11 = HEAP8[$10>>0]|0;
  $12 = ($11<<24>>24)!=(0);
  $13 = $12 ^ 1;
  $14 = HEAP32[$pPage>>2]|0;
  if (!($13)) {
   label = 5;
   break;
  }
  $15 = (($14) + 6|0);
  $16 = HEAP8[$15>>0]|0;
  $17 = $16&255;
  $18 = (($17) + 8)|0;
  $19 = HEAP32[$pPage>>2]|0;
  $20 = (($19) + 56|0);
  $21 = HEAP32[$20>>2]|0;
  $22 = (($21) + ($18)|0);
  $23 = (_sqlite3Get4byte($22)|0);
  HEAP32[$pgno>>2] = $23;
  $24 = HEAP32[$pPage>>2]|0;
  $25 = (($24) + 18|0);
  $26 = HEAP16[$25>>1]|0;
  $27 = HEAP32[$1>>2]|0;
  $28 = (($27) + 76|0);
  $29 = HEAP16[$28>>1]|0;
  $30 = $29 << 16 >> 16;
  $31 = HEAP32[$1>>2]|0;
  $32 = (($31) + 78|0);
  $33 = (($32) + ($30<<1)|0);
  HEAP16[$33>>1] = $26;
  $34 = HEAP32[$1>>2]|0;
  $35 = HEAP32[$pgno>>2]|0;
  $36 = (_moveToChild($34,$35)|0);
  HEAP32[$rc>>2] = $36;
  $37 = HEAP32[$rc>>2]|0;
  $38 = ($37|0)!=(0);
  if ($38) {
   label = 4;
   break;
  }
 }
 if ((label|0) == 4) {
  $39 = HEAP32[$rc>>2]|0;
  HEAP32[$0>>2] = $39;
  $52 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($52|0);
 }
 else if ((label|0) == 5) {
  $40 = (($14) + 18|0);
  $41 = HEAP16[$40>>1]|0;
  $42 = $41&65535;
  $43 = (($42) - 1)|0;
  $44 = $43&65535;
  $45 = HEAP32[$1>>2]|0;
  $46 = (($45) + 76|0);
  $47 = HEAP16[$46>>1]|0;
  $48 = $47 << 16 >> 16;
  $49 = HEAP32[$1>>2]|0;
  $50 = (($49) + 78|0);
  $51 = (($50) + ($48<<1)|0);
  HEAP16[$51>>1] = $44;
  HEAP32[$0>>2] = 0;
  $52 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($52|0);
 }
 return 0|0;
}
function _moveToParent($pCur) {
 $pCur = $pCur|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0;
 var $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 HEAP32[$0>>2] = $pCur;
 $1 = HEAP32[$0>>2]|0;
 $2 = (($1) + 76|0);
 $3 = HEAP16[$2>>1]|0;
 $4 = $3 << 16 >> 16;
 $5 = HEAP32[$0>>2]|0;
 $6 = (($5) + 120|0);
 $7 = (($6) + ($4<<2)|0);
 $8 = HEAP32[$7>>2]|0;
 _releasePage($8);
 $9 = HEAP32[$0>>2]|0;
 $10 = (($9) + 76|0);
 $11 = HEAP16[$10>>1]|0;
 $12 = (($11) + -1)<<16>>16;
 HEAP16[$10>>1] = $12;
 $13 = HEAP32[$0>>2]|0;
 $14 = (($13) + 24|0);
 $15 = (($14) + 20|0);
 HEAP16[$15>>1] = 0;
 $16 = HEAP32[$0>>2]|0;
 $17 = (($16) + 72|0);
 $18 = HEAP8[$17>>0]|0;
 $19 = $18&255;
 $20 = $19 & -7;
 $21 = $20&255;
 HEAP8[$17>>0] = $21;
 STACKTOP = sp;return;
}
function _getAndInitPage($pBt,$pgno,$ppPage,$bReadonly) {
 $pBt = $pBt|0;
 $pgno = $pgno|0;
 $ppPage = $ppPage|0;
 $bReadonly = $bReadonly|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 16|0;
 $1 = sp + 12|0;
 $2 = sp + 8|0;
 $3 = sp + 4|0;
 $rc = sp;
 HEAP32[$0>>2] = $pBt;
 HEAP32[$1>>2] = $pgno;
 HEAP32[$2>>2] = $ppPage;
 HEAP32[$3>>2] = $bReadonly;
 $4 = HEAP32[$1>>2]|0;
 $5 = HEAP32[$0>>2]|0;
 $6 = (_btreePagecount($5)|0);
 $7 = ($4>>>0)>($6>>>0);
 if ($7) {
  $8 = (_sqlite3CorruptError(53903)|0);
  HEAP32[$rc>>2] = $8;
 } else {
  $9 = HEAP32[$0>>2]|0;
  $10 = HEAP32[$1>>2]|0;
  $11 = HEAP32[$2>>2]|0;
  $12 = HEAP32[$3>>2]|0;
  $13 = (_btreeGetPage($9,$10,$11,$12)|0);
  HEAP32[$rc>>2] = $13;
  $14 = HEAP32[$rc>>2]|0;
  $15 = ($14|0)==(0);
  if ($15) {
   $16 = HEAP32[$2>>2]|0;
   $17 = HEAP32[$16>>2]|0;
   $18 = HEAP8[$17>>0]|0;
   $19 = $18&255;
   $20 = ($19|0)==(0);
   if ($20) {
    $21 = HEAP32[$2>>2]|0;
    $22 = HEAP32[$21>>2]|0;
    $23 = (_btreeInitPage($22)|0);
    HEAP32[$rc>>2] = $23;
    $24 = HEAP32[$rc>>2]|0;
    $25 = ($24|0)!=(0);
    if ($25) {
     $26 = HEAP32[$2>>2]|0;
     $27 = HEAP32[$26>>2]|0;
     _releasePage($27);
    }
   }
  }
 }
 $28 = HEAP32[$rc>>2]|0;
 STACKTOP = sp;return ($28|0);
}
function _btreePagecount($pBt) {
 $pBt = $pBt|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 HEAP32[$0>>2] = $pBt;
 $1 = HEAP32[$0>>2]|0;
 $2 = (($1) + 44|0);
 $3 = HEAP32[$2>>2]|0;
 STACKTOP = sp;return ($3|0);
}
function _btreeGetPage($pBt,$pgno,$ppPage,$flags) {
 $pBt = $pBt|0;
 $pgno = $pgno|0;
 $ppPage = $ppPage|0;
 $flags = $flags|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $pDbPage = 0;
 var $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 24|0;
 $1 = sp + 20|0;
 $2 = sp + 16|0;
 $3 = sp + 12|0;
 $4 = sp + 8|0;
 $rc = sp + 4|0;
 $pDbPage = sp;
 HEAP32[$1>>2] = $pBt;
 HEAP32[$2>>2] = $pgno;
 HEAP32[$3>>2] = $ppPage;
 HEAP32[$4>>2] = $flags;
 $5 = HEAP32[$1>>2]|0;
 $6 = HEAP32[$5>>2]|0;
 $7 = HEAP32[$2>>2]|0;
 $8 = HEAP32[$4>>2]|0;
 $9 = (_sqlite3PagerAcquire($6,$7,$pDbPage,$8)|0);
 HEAP32[$rc>>2] = $9;
 $10 = HEAP32[$rc>>2]|0;
 $11 = ($10|0)!=(0);
 if ($11) {
  $12 = HEAP32[$rc>>2]|0;
  HEAP32[$0>>2] = $12;
  $18 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($18|0);
 } else {
  $13 = HEAP32[$pDbPage>>2]|0;
  $14 = HEAP32[$2>>2]|0;
  $15 = HEAP32[$1>>2]|0;
  $16 = (_btreePageFromDbPage($13,$14,$15)|0);
  $17 = HEAP32[$3>>2]|0;
  HEAP32[$17>>2] = $16;
  HEAP32[$0>>2] = 0;
  $18 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($18|0);
 }
 return 0|0;
}
function _btreePageFromDbPage($pDbPage,$pgno,$pBt) {
 $pDbPage = $pDbPage|0;
 $pgno = $pgno|0;
 $pBt = $pBt|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $pPage = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $1 = sp + 8|0;
 $2 = sp + 4|0;
 $pPage = sp;
 HEAP32[$0>>2] = $pDbPage;
 HEAP32[$1>>2] = $pgno;
 HEAP32[$2>>2] = $pBt;
 $3 = HEAP32[$0>>2]|0;
 $4 = (_sqlite3PagerGetExtra($3)|0);
 HEAP32[$pPage>>2] = $4;
 $5 = HEAP32[$0>>2]|0;
 $6 = (_sqlite3PagerGetData($5)|0);
 $7 = HEAP32[$pPage>>2]|0;
 $8 = (($7) + 56|0);
 HEAP32[$8>>2] = $6;
 $9 = HEAP32[$0>>2]|0;
 $10 = HEAP32[$pPage>>2]|0;
 $11 = (($10) + 68|0);
 HEAP32[$11>>2] = $9;
 $12 = HEAP32[$2>>2]|0;
 $13 = HEAP32[$pPage>>2]|0;
 $14 = (($13) + 52|0);
 HEAP32[$14>>2] = $12;
 $15 = HEAP32[$1>>2]|0;
 $16 = HEAP32[$pPage>>2]|0;
 $17 = (($16) + 72|0);
 HEAP32[$17>>2] = $15;
 $18 = HEAP32[$pPage>>2]|0;
 $19 = (($18) + 72|0);
 $20 = HEAP32[$19>>2]|0;
 $21 = ($20|0)==(1);
 $22 = $21 ? 100 : 0;
 $23 = $22&255;
 $24 = HEAP32[$pPage>>2]|0;
 $25 = (($24) + 6|0);
 HEAP8[$25>>0] = $23;
 $26 = HEAP32[$pPage>>2]|0;
 STACKTOP = sp;return ($26|0);
}
function _btreeMoveto($pCur,$pKey,$0,$1,$bias,$pRes) {
 $pCur = $pCur|0;
 $pKey = $pKey|0;
 $0 = $0|0;
 $1 = $1|0;
 $bias = $bias|0;
 $pRes = $pRes|0;
 var $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0;
 var $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0;
 var $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $7 = 0;
 var $8 = 0, $9 = 0, $aSpace = 0, $pFree = 0, $pIdxKey = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 240|0;
 $2 = sp + 36|0;
 $3 = sp + 32|0;
 $4 = sp + 28|0;
 $5 = sp;
 $6 = sp + 24|0;
 $7 = sp + 20|0;
 $rc = sp + 16|0;
 $pIdxKey = sp + 12|0;
 $aSpace = sp + 40|0;
 $pFree = sp + 8|0;
 HEAP32[$3>>2] = $pCur;
 HEAP32[$4>>2] = $pKey;
 $8 = $5;
 $9 = $8;
 HEAP32[$9>>2] = $0;
 $10 = (($8) + 4)|0;
 $11 = $10;
 HEAP32[$11>>2] = $1;
 HEAP32[$6>>2] = $bias;
 HEAP32[$7>>2] = $pRes;
 HEAP32[$pFree>>2] = 0;
 $12 = HEAP32[$4>>2]|0;
 $13 = ($12|0)!=(0|0);
 if ($13) {
  $14 = HEAP32[$3>>2]|0;
  $15 = (($14) + 16|0);
  $16 = HEAP32[$15>>2]|0;
  $17 = (_sqlite3VdbeAllocUnpackedRecord($16,$aSpace,200,$pFree)|0);
  HEAP32[$pIdxKey>>2] = $17;
  $18 = HEAP32[$pIdxKey>>2]|0;
  $19 = ($18|0)==(0|0);
  if ($19) {
   HEAP32[$2>>2] = 7;
   $63 = HEAP32[$2>>2]|0;
   STACKTOP = sp;return ($63|0);
  }
  $20 = HEAP32[$3>>2]|0;
  $21 = (($20) + 16|0);
  $22 = HEAP32[$21>>2]|0;
  $23 = $5;
  $24 = $23;
  $25 = HEAP32[$24>>2]|0;
  $26 = (($23) + 4)|0;
  $27 = $26;
  $28 = HEAP32[$27>>2]|0;
  $29 = HEAP32[$4>>2]|0;
  $30 = HEAP32[$pIdxKey>>2]|0;
  _sqlite3VdbeRecordUnpack($22,$25,$29,$30);
  $31 = HEAP32[$pIdxKey>>2]|0;
  $32 = (($31) + 4|0);
  $33 = HEAP16[$32>>1]|0;
  $34 = $33&65535;
  $35 = ($34|0)==(0);
  if ($35) {
   $36 = HEAP32[$3>>2]|0;
   $37 = (($36) + 16|0);
   $38 = HEAP32[$37>>2]|0;
   $39 = (($38) + 12|0);
   $40 = HEAP32[$39>>2]|0;
   $41 = HEAP32[$pFree>>2]|0;
   _sqlite3DbFree($40,$41);
   $42 = (_sqlite3CorruptError(52917)|0);
   HEAP32[$2>>2] = $42;
   $63 = HEAP32[$2>>2]|0;
   STACKTOP = sp;return ($63|0);
  }
 } else {
  HEAP32[$pIdxKey>>2] = 0;
 }
 $43 = HEAP32[$3>>2]|0;
 $44 = HEAP32[$pIdxKey>>2]|0;
 $45 = $5;
 $46 = $45;
 $47 = HEAP32[$46>>2]|0;
 $48 = (($45) + 4)|0;
 $49 = $48;
 $50 = HEAP32[$49>>2]|0;
 $51 = HEAP32[$6>>2]|0;
 $52 = HEAP32[$7>>2]|0;
 $53 = (_sqlite3BtreeMovetoUnpacked($43,$44,$47,$50,$51,$52)|0);
 HEAP32[$rc>>2] = $53;
 $54 = HEAP32[$pFree>>2]|0;
 $55 = ($54|0)!=(0|0);
 if ($55) {
  $56 = HEAP32[$3>>2]|0;
  $57 = (($56) + 16|0);
  $58 = HEAP32[$57>>2]|0;
  $59 = (($58) + 12|0);
  $60 = HEAP32[$59>>2]|0;
  $61 = HEAP32[$pFree>>2]|0;
  _sqlite3DbFree($60,$61);
 }
 $62 = HEAP32[$rc>>2]|0;
 HEAP32[$2>>2] = $62;
 $63 = HEAP32[$2>>2]|0;
 STACKTOP = sp;return ($63|0);
}
function _sqlite3VdbeAllocUnpackedRecord($pKeyInfo,$pSpace,$szSpace,$ppFree) {
 $pKeyInfo = $pKeyInfo|0;
 $pSpace = $pSpace|0;
 $szSpace = $szSpace|0;
 $ppFree = $ppFree|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $nByte = 0, $nOff = 0, $p = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 28|0;
 $1 = sp + 24|0;
 $2 = sp + 20|0;
 $3 = sp + 16|0;
 $4 = sp + 12|0;
 $p = sp + 8|0;
 $nOff = sp + 4|0;
 $nByte = sp;
 HEAP32[$1>>2] = $pKeyInfo;
 HEAP32[$2>>2] = $pSpace;
 HEAP32[$3>>2] = $szSpace;
 HEAP32[$4>>2] = $ppFree;
 $5 = HEAP32[$2>>2]|0;
 $6 = $5;
 $7 = $6 & 7;
 $8 = (8 - ($7))|0;
 $9 = $8 & 7;
 HEAP32[$nOff>>2] = $9;
 $10 = HEAP32[$1>>2]|0;
 $11 = (($10) + 6|0);
 $12 = HEAP16[$11>>1]|0;
 $13 = $12&65535;
 $14 = (($13) + 1)|0;
 $15 = ($14*40)|0;
 $16 = (24 + ($15))|0;
 HEAP32[$nByte>>2] = $16;
 $17 = HEAP32[$nByte>>2]|0;
 $18 = HEAP32[$3>>2]|0;
 $19 = HEAP32[$nOff>>2]|0;
 $20 = (($18) + ($19))|0;
 $21 = ($17|0)>($20|0);
 if ($21) {
  $22 = HEAP32[$1>>2]|0;
  $23 = (($22) + 12|0);
  $24 = HEAP32[$23>>2]|0;
  $25 = HEAP32[$nByte>>2]|0;
  $26 = ($25|0)<(0);
  $27 = $26 << 31 >> 31;
  $28 = (_sqlite3DbMallocRaw($24,$25,$27)|0);
  HEAP32[$p>>2] = $28;
  $29 = HEAP32[$p>>2]|0;
  $30 = HEAP32[$4>>2]|0;
  HEAP32[$30>>2] = $29;
  $31 = HEAP32[$p>>2]|0;
  $32 = ($31|0)!=(0|0);
  if (!($32)) {
   HEAP32[$0>>2] = 0;
   $52 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($52|0);
  }
 } else {
  $33 = HEAP32[$nOff>>2]|0;
  $34 = HEAP32[$2>>2]|0;
  $35 = (($34) + ($33)|0);
  HEAP32[$p>>2] = $35;
  $36 = HEAP32[$4>>2]|0;
  HEAP32[$36>>2] = 0;
 }
 $37 = HEAP32[$p>>2]|0;
 $38 = (($37) + 24|0);
 $39 = HEAP32[$p>>2]|0;
 $40 = (($39) + 8|0);
 HEAP32[$40>>2] = $38;
 $41 = HEAP32[$1>>2]|0;
 $42 = HEAP32[$p>>2]|0;
 HEAP32[$42>>2] = $41;
 $43 = HEAP32[$1>>2]|0;
 $44 = (($43) + 6|0);
 $45 = HEAP16[$44>>1]|0;
 $46 = $45&65535;
 $47 = (($46) + 1)|0;
 $48 = $47&65535;
 $49 = HEAP32[$p>>2]|0;
 $50 = (($49) + 4|0);
 HEAP16[$50>>1] = $48;
 $51 = HEAP32[$p>>2]|0;
 HEAP32[$0>>2] = $51;
 $52 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($52|0);
}
function _sqlite3VdbeRecordUnpack($pKeyInfo,$nKey,$pKey,$p) {
 $pKeyInfo = $pKeyInfo|0;
 $nKey = $nKey|0;
 $pKey = $pKey|0;
 $p = $p|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $9 = 0;
 var $aKey = 0, $d = 0, $idx = 0, $pMem = 0, $serial_type = 0, $szHdr = 0, $u = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $0 = sp + 36|0;
 $1 = sp + 32|0;
 $2 = sp + 28|0;
 $3 = sp + 24|0;
 $aKey = sp + 20|0;
 $d = sp + 16|0;
 $idx = sp + 12|0;
 $u = sp + 40|0;
 $szHdr = sp + 8|0;
 $pMem = sp + 4|0;
 $serial_type = sp;
 HEAP32[$0>>2] = $pKeyInfo;
 HEAP32[$1>>2] = $nKey;
 HEAP32[$2>>2] = $pKey;
 HEAP32[$3>>2] = $p;
 $4 = HEAP32[$2>>2]|0;
 HEAP32[$aKey>>2] = $4;
 $5 = HEAP32[$3>>2]|0;
 $6 = (($5) + 8|0);
 $7 = HEAP32[$6>>2]|0;
 HEAP32[$pMem>>2] = $7;
 $8 = HEAP32[$3>>2]|0;
 $9 = (($8) + 6|0);
 HEAP8[$9>>0] = 0;
 $10 = HEAP32[$aKey>>2]|0;
 $11 = HEAP8[$10>>0]|0;
 $12 = $11&255;
 $13 = ($12|0)<(128);
 $14 = HEAP32[$aKey>>2]|0;
 if ($13) {
  $15 = HEAP8[$14>>0]|0;
  $16 = $15&255;
  HEAP32[$szHdr>>2] = $16;
  $20 = 1;
 } else {
  $17 = (_sqlite3GetVarint32($14,$szHdr)|0);
  $18 = $17&255;
  $20 = $18;
 }
 $19 = $20&255;
 $21 = $19&255;
 HEAP32[$idx>>2] = $21;
 $22 = HEAP32[$szHdr>>2]|0;
 HEAP32[$d>>2] = $22;
 HEAP16[$u>>1] = 0;
 while(1) {
  $23 = HEAP32[$idx>>2]|0;
  $24 = HEAP32[$szHdr>>2]|0;
  $25 = ($23>>>0)<($24>>>0);
  if (!($25)) {
   label = 11;
   break;
  }
  $26 = HEAP32[$d>>2]|0;
  $27 = HEAP32[$1>>2]|0;
  $28 = ($26|0)<=($27|0);
  if (!($28)) {
   label = 11;
   break;
  }
  $29 = HEAP32[$idx>>2]|0;
  $30 = HEAP32[$aKey>>2]|0;
  $31 = (($30) + ($29)|0);
  $32 = HEAP8[$31>>0]|0;
  $33 = $32&255;
  $34 = ($33|0)<(128);
  $35 = HEAP32[$idx>>2]|0;
  $36 = HEAP32[$aKey>>2]|0;
  $37 = (($36) + ($35)|0);
  if ($34) {
   $38 = HEAP8[$37>>0]|0;
   $39 = $38&255;
   HEAP32[$serial_type>>2] = $39;
   $43 = 1;
  } else {
   $40 = (_sqlite3GetVarint32($37,$serial_type)|0);
   $41 = $40&255;
   $43 = $41;
  }
  $42 = $43&255;
  $44 = $42&255;
  $45 = HEAP32[$idx>>2]|0;
  $46 = (($45) + ($44))|0;
  HEAP32[$idx>>2] = $46;
  $47 = HEAP32[$0>>2]|0;
  $48 = (($47) + 4|0);
  $49 = HEAP8[$48>>0]|0;
  $50 = HEAP32[$pMem>>2]|0;
  $51 = (($50) + 10|0);
  HEAP8[$51>>0] = $49;
  $52 = HEAP32[$0>>2]|0;
  $53 = (($52) + 12|0);
  $54 = HEAP32[$53>>2]|0;
  $55 = HEAP32[$pMem>>2]|0;
  $56 = (($55) + 32|0);
  HEAP32[$56>>2] = $54;
  $57 = HEAP32[$pMem>>2]|0;
  $58 = (($57) + 24|0);
  HEAP32[$58>>2] = 0;
  $59 = HEAP32[$d>>2]|0;
  $60 = HEAP32[$aKey>>2]|0;
  $61 = (($60) + ($59)|0);
  $62 = HEAP32[$serial_type>>2]|0;
  $63 = HEAP32[$pMem>>2]|0;
  $64 = (_sqlite3VdbeSerialGet($61,$62,$63)|0);
  $65 = HEAP32[$d>>2]|0;
  $66 = (($65) + ($64))|0;
  HEAP32[$d>>2] = $66;
  $67 = HEAP32[$pMem>>2]|0;
  $68 = (($67) + 40|0);
  HEAP32[$pMem>>2] = $68;
  $69 = HEAP16[$u>>1]|0;
  $70 = (($69) + 1)<<16>>16;
  HEAP16[$u>>1] = $70;
  $71 = $70&65535;
  $72 = HEAP32[$3>>2]|0;
  $73 = (($72) + 4|0);
  $74 = HEAP16[$73>>1]|0;
  $75 = $74&65535;
  $76 = ($71|0)>=($75|0);
  if ($76) {
   label = 11;
   break;
  }
 }
 if ((label|0) == 11) {
  $77 = HEAP16[$u>>1]|0;
  $78 = HEAP32[$3>>2]|0;
  $79 = (($78) + 4|0);
  HEAP16[$79>>1] = $77;
  STACKTOP = sp;return;
 }
}
function _sqlite3BtreeMovetoUnpacked($pCur,$pIdxKey,$0,$1,$biasRight,$pRes) {
 $pCur = $pCur|0;
 $pIdxKey = $pIdxKey|0;
 $0 = $0|0;
 $1 = $1|0;
 $biasRight = $biasRight|0;
 $pRes = $pRes|0;
 var $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0, $116 = 0, $117 = 0;
 var $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0, $134 = 0, $135 = 0;
 var $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0, $152 = 0, $153 = 0;
 var $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0, $170 = 0, $171 = 0;
 var $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0, $188 = 0, $189 = 0, $19 = 0;
 var $190 = 0, $191 = 0, $192 = 0, $193 = 0, $194 = 0, $195 = 0, $196 = 0, $197 = 0, $198 = 0, $199 = 0, $2 = 0, $20 = 0, $200 = 0, $201 = 0, $202 = 0, $203 = 0, $204 = 0, $205 = 0, $206 = 0, $207 = 0;
 var $208 = 0, $209 = 0, $21 = 0, $210 = 0, $211 = 0, $212 = 0, $213 = 0, $214 = 0, $215 = 0, $216 = 0, $217 = 0, $218 = 0, $219 = 0, $22 = 0, $220 = 0, $221 = 0, $222 = 0, $223 = 0, $224 = 0, $225 = 0;
 var $226 = 0, $227 = 0, $228 = 0, $229 = 0, $23 = 0, $230 = 0, $231 = 0, $232 = 0, $233 = 0, $234 = 0, $235 = 0, $236 = 0, $237 = 0, $238 = 0, $239 = 0, $24 = 0, $240 = 0, $241 = 0, $242 = 0, $243 = 0;
 var $244 = 0, $245 = 0, $246 = 0, $247 = 0, $248 = 0, $249 = 0, $25 = 0, $250 = 0, $251 = 0, $252 = 0, $253 = 0, $254 = 0, $255 = 0, $256 = 0, $257 = 0, $258 = 0, $259 = 0, $26 = 0, $260 = 0, $261 = 0;
 var $262 = 0, $263 = 0, $264 = 0, $265 = 0, $266 = 0, $267 = 0, $268 = 0, $269 = 0, $27 = 0, $270 = 0, $271 = 0, $272 = 0, $273 = 0, $274 = 0, $275 = 0, $276 = 0, $277 = 0, $278 = 0, $279 = 0, $28 = 0;
 var $280 = 0, $281 = 0, $282 = 0, $283 = 0, $284 = 0, $285 = 0, $286 = 0, $287 = 0, $288 = 0, $289 = 0, $29 = 0, $290 = 0, $291 = 0, $292 = 0, $293 = 0, $294 = 0, $295 = 0, $296 = 0, $297 = 0, $298 = 0;
 var $299 = 0, $3 = 0, $30 = 0, $300 = 0, $301 = 0, $302 = 0, $303 = 0, $304 = 0, $305 = 0, $306 = 0, $307 = 0, $308 = 0, $309 = 0, $31 = 0, $310 = 0, $311 = 0, $312 = 0, $313 = 0, $314 = 0, $315 = 0;
 var $316 = 0, $317 = 0, $318 = 0, $319 = 0, $32 = 0, $320 = 0, $321 = 0, $322 = 0, $323 = 0, $324 = 0, $325 = 0, $326 = 0, $327 = 0, $328 = 0, $329 = 0, $33 = 0, $330 = 0, $331 = 0, $332 = 0, $333 = 0;
 var $334 = 0, $335 = 0, $336 = 0, $337 = 0, $338 = 0, $339 = 0, $34 = 0, $340 = 0, $341 = 0, $342 = 0, $343 = 0, $344 = 0, $345 = 0, $346 = 0, $347 = 0, $348 = 0, $349 = 0, $35 = 0, $350 = 0, $351 = 0;
 var $352 = 0, $353 = 0, $354 = 0, $355 = 0, $356 = 0, $357 = 0, $358 = 0, $359 = 0, $36 = 0, $360 = 0, $361 = 0, $362 = 0, $363 = 0, $364 = 0, $365 = 0, $366 = 0, $367 = 0, $368 = 0, $369 = 0, $37 = 0;
 var $370 = 0, $371 = 0, $372 = 0, $373 = 0, $374 = 0, $375 = 0, $376 = 0, $377 = 0, $378 = 0, $379 = 0, $38 = 0, $380 = 0, $381 = 0, $382 = 0, $383 = 0, $384 = 0, $385 = 0, $386 = 0, $387 = 0, $388 = 0;
 var $389 = 0, $39 = 0, $390 = 0, $391 = 0, $392 = 0, $393 = 0, $394 = 0, $395 = 0, $396 = 0, $397 = 0, $398 = 0, $399 = 0, $4 = 0, $40 = 0, $400 = 0, $401 = 0, $402 = 0, $403 = 0, $404 = 0, $405 = 0;
 var $406 = 0, $407 = 0, $408 = 0, $409 = 0, $41 = 0, $410 = 0, $411 = 0, $412 = 0, $413 = 0, $414 = 0, $415 = 0, $416 = 0, $417 = 0, $418 = 0, $419 = 0, $42 = 0, $420 = 0, $421 = 0, $422 = 0, $423 = 0;
 var $424 = 0, $425 = 0, $426 = 0, $427 = 0, $428 = 0, $429 = 0, $43 = 0, $430 = 0, $431 = 0, $432 = 0, $433 = 0, $434 = 0, $435 = 0, $436 = 0, $437 = 0, $438 = 0, $439 = 0, $44 = 0, $440 = 0, $441 = 0;
 var $442 = 0, $443 = 0, $444 = 0, $445 = 0, $446 = 0, $447 = 0, $448 = 0, $449 = 0, $45 = 0, $450 = 0, $451 = 0, $452 = 0, $453 = 0, $454 = 0, $455 = 0, $456 = 0, $457 = 0, $458 = 0, $459 = 0, $46 = 0;
 var $460 = 0, $461 = 0, $462 = 0, $463 = 0, $464 = 0, $465 = 0, $466 = 0, $467 = 0, $468 = 0, $469 = 0, $47 = 0, $470 = 0, $471 = 0, $472 = 0, $473 = 0, $474 = 0, $475 = 0, $476 = 0, $477 = 0, $478 = 0;
 var $479 = 0, $48 = 0, $480 = 0, $481 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0;
 var $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0;
 var $c = 0, $chldPg = 0, $idx = 0, $lwr = 0, $nCell = 0, $nCellKey = 0, $pCell = 0, $pCellBody = 0, $pCellKey = 0, $pPage = 0, $rc = 0, $upr = 0, $xRecordCompare = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 96|0;
 $2 = sp + 80|0;
 $3 = sp + 76|0;
 $4 = sp + 72|0;
 $5 = sp + 8|0;
 $6 = sp + 68|0;
 $7 = sp + 64|0;
 $rc = sp + 60|0;
 $xRecordCompare = sp + 56|0;
 $lwr = sp + 52|0;
 $upr = sp + 48|0;
 $idx = sp + 44|0;
 $c = sp + 40|0;
 $chldPg = sp + 36|0;
 $pPage = sp + 32|0;
 $pCell = sp + 28|0;
 $nCellKey = sp;
 $nCell = sp + 24|0;
 $pCellKey = sp + 20|0;
 $pCellBody = sp + 16|0;
 HEAP32[$3>>2] = $pCur;
 HEAP32[$4>>2] = $pIdxKey;
 $8 = $5;
 $9 = $8;
 HEAP32[$9>>2] = $0;
 $10 = (($8) + 4)|0;
 $11 = $10;
 HEAP32[$11>>2] = $1;
 HEAP32[$6>>2] = $biasRight;
 HEAP32[$7>>2] = $pRes;
 $12 = HEAP32[$3>>2]|0;
 $13 = (($12) + 73|0);
 $14 = HEAP8[$13>>0]|0;
 $15 = $14&255;
 $16 = ($15|0)==(1);
 if ($16) {
  $17 = HEAP32[$3>>2]|0;
  $18 = (($17) + 72|0);
  $19 = HEAP8[$18>>0]|0;
  $20 = $19&255;
  $21 = $20 & 2;
  $22 = ($21|0)!=(0);
  if ($22) {
   $23 = HEAP32[$3>>2]|0;
   $24 = (($23) + 120|0);
   $25 = HEAP32[$24>>2]|0;
   $26 = (($25) + 2|0);
   $27 = HEAP8[$26>>0]|0;
   $28 = $27&255;
   $29 = ($28|0)!=(0);
   if ($29) {
    $30 = HEAP32[$3>>2]|0;
    $31 = (($30) + 24|0);
    $32 = $31;
    $33 = $32;
    $34 = HEAP32[$33>>2]|0;
    $35 = (($32) + 4)|0;
    $36 = $35;
    $37 = HEAP32[$36>>2]|0;
    $38 = $5;
    $39 = $38;
    $40 = HEAP32[$39>>2]|0;
    $41 = (($38) + 4)|0;
    $42 = $41;
    $43 = HEAP32[$42>>2]|0;
    $44 = ($34|0)==($40|0);
    $45 = ($37|0)==($43|0);
    $46 = $44 & $45;
    if ($46) {
     $47 = HEAP32[$7>>2]|0;
     HEAP32[$47>>2] = 0;
     HEAP32[$2>>2] = 0;
     $481 = HEAP32[$2>>2]|0;
     STACKTOP = sp;return ($481|0);
    }
    $48 = HEAP32[$3>>2]|0;
    $49 = (($48) + 72|0);
    $50 = HEAP8[$49>>0]|0;
    $51 = $50&255;
    $52 = $51 & 8;
    $53 = ($52|0)!=(0);
    if ($53) {
     $54 = HEAP32[$3>>2]|0;
     $55 = (($54) + 24|0);
     $56 = $55;
     $57 = $56;
     $58 = HEAP32[$57>>2]|0;
     $59 = (($56) + 4)|0;
     $60 = $59;
     $61 = HEAP32[$60>>2]|0;
     $62 = $5;
     $63 = $62;
     $64 = HEAP32[$63>>2]|0;
     $65 = (($62) + 4)|0;
     $66 = $65;
     $67 = HEAP32[$66>>2]|0;
     $68 = ($61|0)<($67|0);
     $69 = ($58>>>0)<($64>>>0);
     $70 = ($61|0)==($67|0);
     $71 = $70 & $69;
     $72 = $68 | $71;
     if ($72) {
      $73 = HEAP32[$7>>2]|0;
      HEAP32[$73>>2] = -1;
      HEAP32[$2>>2] = 0;
      $481 = HEAP32[$2>>2]|0;
      STACKTOP = sp;return ($481|0);
     }
    }
   }
  }
 }
 $74 = HEAP32[$4>>2]|0;
 $75 = ($74|0)!=(0|0);
 if ($75) {
  $76 = HEAP32[$4>>2]|0;
  $77 = (_sqlite3VdbeFindCompare($76)|0);
  HEAP32[$xRecordCompare>>2] = $77;
  $78 = HEAP32[$4>>2]|0;
  $79 = (($78) + 7|0);
  HEAP8[$79>>0] = 0;
 } else {
  HEAP32[$xRecordCompare>>2] = 0;
 }
 $80 = HEAP32[$3>>2]|0;
 $81 = (_moveToRoot($80)|0);
 HEAP32[$rc>>2] = $81;
 $82 = HEAP32[$rc>>2]|0;
 $83 = ($82|0)!=(0);
 if ($83) {
  $84 = HEAP32[$rc>>2]|0;
  HEAP32[$2>>2] = $84;
  $481 = HEAP32[$2>>2]|0;
  STACKTOP = sp;return ($481|0);
 }
 $85 = HEAP32[$3>>2]|0;
 $86 = (($85) + 73|0);
 $87 = HEAP8[$86>>0]|0;
 $88 = $87&255;
 $89 = ($88|0)==(0);
 if ($89) {
  $90 = HEAP32[$7>>2]|0;
  HEAP32[$90>>2] = -1;
  HEAP32[$2>>2] = 0;
  $481 = HEAP32[$2>>2]|0;
  STACKTOP = sp;return ($481|0);
 }
 L24: while(1) {
  $91 = HEAP32[$3>>2]|0;
  $92 = (($91) + 76|0);
  $93 = HEAP16[$92>>1]|0;
  $94 = $93 << 16 >> 16;
  $95 = HEAP32[$3>>2]|0;
  $96 = (($95) + 120|0);
  $97 = (($96) + ($94<<2)|0);
  $98 = HEAP32[$97>>2]|0;
  HEAP32[$pPage>>2] = $98;
  HEAP32[$lwr>>2] = 0;
  $99 = HEAP32[$pPage>>2]|0;
  $100 = (($99) + 18|0);
  $101 = HEAP16[$100>>1]|0;
  $102 = $101&65535;
  $103 = (($102) - 1)|0;
  HEAP32[$upr>>2] = $103;
  $104 = HEAP32[$upr>>2]|0;
  $105 = HEAP32[$6>>2]|0;
  $106 = (1 - ($105))|0;
  $107 = $104 >> $106;
  HEAP32[$idx>>2] = $107;
  $108 = HEAP32[$idx>>2]|0;
  $109 = $108&65535;
  $110 = HEAP32[$3>>2]|0;
  $111 = (($110) + 76|0);
  $112 = HEAP16[$111>>1]|0;
  $113 = $112 << 16 >> 16;
  $114 = HEAP32[$3>>2]|0;
  $115 = (($114) + 78|0);
  $116 = (($115) + ($113<<1)|0);
  HEAP16[$116>>1] = $109;
  $117 = HEAP32[$xRecordCompare>>2]|0;
  $118 = ($117|0)==(0|0);
  L26: do {
   if ($118) {
    while(1) {
     $119 = HEAP32[$pPage>>2]|0;
     $120 = (($119) + 56|0);
     $121 = HEAP32[$120>>2]|0;
     $122 = HEAP32[$pPage>>2]|0;
     $123 = (($122) + 20|0);
     $124 = HEAP16[$123>>1]|0;
     $125 = $124&65535;
     $126 = HEAP32[$idx>>2]|0;
     $127 = $126<<1;
     $128 = HEAP32[$pPage>>2]|0;
     $129 = (($128) + 64|0);
     $130 = HEAP32[$129>>2]|0;
     $131 = (($130) + ($127)|0);
     $132 = HEAP8[$131>>0]|0;
     $133 = $132&255;
     $134 = $133 << 8;
     $135 = HEAP32[$idx>>2]|0;
     $136 = $135<<1;
     $137 = HEAP32[$pPage>>2]|0;
     $138 = (($137) + 64|0);
     $139 = HEAP32[$138>>2]|0;
     $140 = (($139) + ($136)|0);
     $141 = (($140) + 1|0);
     $142 = HEAP8[$141>>0]|0;
     $143 = $142&255;
     $144 = $134 | $143;
     $145 = $125 & $144;
     $146 = (($121) + ($145)|0);
     $147 = HEAP32[$pPage>>2]|0;
     $148 = (($147) + 7|0);
     $149 = HEAP8[$148>>0]|0;
     $150 = $149&255;
     $151 = (($146) + ($150)|0);
     HEAP32[$pCell>>2] = $151;
     $152 = HEAP32[$pPage>>2]|0;
     $153 = (($152) + 3|0);
     $154 = HEAP8[$153>>0]|0;
     $155 = ($154<<24>>24)!=(0);
     L29: do {
      if ($155) {
       while(1) {
        $156 = HEAP32[$pCell>>2]|0;
        $157 = (($156) + 1|0);
        HEAP32[$pCell>>2] = $157;
        $158 = HEAP8[$156>>0]|0;
        $159 = $158&255;
        $160 = (128)<=($159|0);
        if (!($160)) {
         break L29;
        }
        $161 = HEAP32[$pCell>>2]|0;
        $162 = HEAP32[$pPage>>2]|0;
        $163 = (($162) + 60|0);
        $164 = HEAP32[$163>>2]|0;
        $165 = ($161>>>0)>=($164>>>0);
        if ($165) {
         label = 20;
         break L24;
        }
       }
      }
     } while(0);
     $167 = HEAP32[$pCell>>2]|0;
     (_sqlite3GetVarint($167,$nCellKey)|0);
     $168 = $nCellKey;
     $169 = $168;
     $170 = HEAP32[$169>>2]|0;
     $171 = (($168) + 4)|0;
     $172 = $171;
     $173 = HEAP32[$172>>2]|0;
     $174 = $5;
     $175 = $174;
     $176 = HEAP32[$175>>2]|0;
     $177 = (($174) + 4)|0;
     $178 = $177;
     $179 = HEAP32[$178>>2]|0;
     $180 = ($173|0)<($179|0);
     $181 = ($170>>>0)<($176>>>0);
     $182 = ($173|0)==($179|0);
     $183 = $182 & $181;
     $184 = $180 | $183;
     if ($184) {
      $185 = HEAP32[$idx>>2]|0;
      $186 = (($185) + 1)|0;
      HEAP32[$lwr>>2] = $186;
      $187 = HEAP32[$lwr>>2]|0;
      $188 = HEAP32[$upr>>2]|0;
      $189 = ($187|0)>($188|0);
      if ($189) {
       label = 23;
       break;
      }
     } else {
      $190 = $nCellKey;
      $191 = $190;
      $192 = HEAP32[$191>>2]|0;
      $193 = (($190) + 4)|0;
      $194 = $193;
      $195 = HEAP32[$194>>2]|0;
      $196 = $5;
      $197 = $196;
      $198 = HEAP32[$197>>2]|0;
      $199 = (($196) + 4)|0;
      $200 = $199;
      $201 = HEAP32[$200>>2]|0;
      $202 = ($195|0)>($201|0);
      $203 = ($192>>>0)>($198>>>0);
      $204 = ($195|0)==($201|0);
      $205 = $204 & $203;
      $206 = $202 | $205;
      if (!($206)) {
       label = 27;
       break;
      }
      $207 = HEAP32[$idx>>2]|0;
      $208 = (($207) - 1)|0;
      HEAP32[$upr>>2] = $208;
      $209 = HEAP32[$lwr>>2]|0;
      $210 = HEAP32[$upr>>2]|0;
      $211 = ($209|0)>($210|0);
      if ($211) {
       label = 26;
       break;
      }
     }
     $245 = HEAP32[$lwr>>2]|0;
     $246 = HEAP32[$upr>>2]|0;
     $247 = (($245) + ($246))|0;
     $248 = $247 >> 1;
     HEAP32[$idx>>2] = $248;
    }
    if ((label|0) == 23) {
     label = 0;
     HEAP32[$c>>2] = -1;
     label = 49;
     break;
    }
    else if ((label|0) == 26) {
     label = 0;
     HEAP32[$c>>2] = 1;
     label = 49;
     break;
    }
    else if ((label|0) == 27) {
     label = 0;
     $212 = HEAP32[$3>>2]|0;
     $213 = (($212) + 72|0);
     $214 = HEAP8[$213>>0]|0;
     $215 = $214&255;
     $216 = $215 | 2;
     $217 = $216&255;
     HEAP8[$213>>0] = $217;
     $218 = $nCellKey;
     $219 = $218;
     $220 = HEAP32[$219>>2]|0;
     $221 = (($218) + 4)|0;
     $222 = $221;
     $223 = HEAP32[$222>>2]|0;
     $224 = HEAP32[$3>>2]|0;
     $225 = (($224) + 24|0);
     $226 = $225;
     $227 = $226;
     HEAP32[$227>>2] = $220;
     $228 = (($226) + 4)|0;
     $229 = $228;
     HEAP32[$229>>2] = $223;
     $230 = HEAP32[$idx>>2]|0;
     $231 = $230&65535;
     $232 = HEAP32[$3>>2]|0;
     $233 = (($232) + 76|0);
     $234 = HEAP16[$233>>1]|0;
     $235 = $234 << 16 >> 16;
     $236 = HEAP32[$3>>2]|0;
     $237 = (($236) + 78|0);
     $238 = (($237) + ($235<<1)|0);
     HEAP16[$238>>1] = $231;
     $239 = HEAP32[$pPage>>2]|0;
     $240 = (($239) + 5|0);
     $241 = HEAP8[$240>>0]|0;
     $242 = ($241<<24>>24)!=(0);
     if ($242) {
      label = 29;
      break L24;
     }
     $243 = HEAP32[$idx>>2]|0;
     HEAP32[$lwr>>2] = $243;
     break;
    }
   } else {
    while(1) {
     $249 = HEAP32[$pPage>>2]|0;
     $250 = (($249) + 56|0);
     $251 = HEAP32[$250>>2]|0;
     $252 = HEAP32[$pPage>>2]|0;
     $253 = (($252) + 20|0);
     $254 = HEAP16[$253>>1]|0;
     $255 = $254&65535;
     $256 = HEAP32[$idx>>2]|0;
     $257 = $256<<1;
     $258 = HEAP32[$pPage>>2]|0;
     $259 = (($258) + 64|0);
     $260 = HEAP32[$259>>2]|0;
     $261 = (($260) + ($257)|0);
     $262 = HEAP8[$261>>0]|0;
     $263 = $262&255;
     $264 = $263 << 8;
     $265 = HEAP32[$idx>>2]|0;
     $266 = $265<<1;
     $267 = HEAP32[$pPage>>2]|0;
     $268 = (($267) + 64|0);
     $269 = HEAP32[$268>>2]|0;
     $270 = (($269) + ($266)|0);
     $271 = (($270) + 1|0);
     $272 = HEAP8[$271>>0]|0;
     $273 = $272&255;
     $274 = $264 | $273;
     $275 = $255 & $274;
     $276 = (($251) + ($275)|0);
     $277 = HEAP32[$pPage>>2]|0;
     $278 = (($277) + 7|0);
     $279 = HEAP8[$278>>0]|0;
     $280 = $279&255;
     $281 = (($276) + ($280)|0);
     HEAP32[$pCell>>2] = $281;
     $282 = HEAP32[$pCell>>2]|0;
     $283 = HEAP8[$282>>0]|0;
     $284 = $283&255;
     HEAP32[$nCell>>2] = $284;
     $285 = HEAP32[$nCell>>2]|0;
     $286 = HEAP32[$pPage>>2]|0;
     $287 = (($286) + 8|0);
     $288 = HEAP8[$287>>0]|0;
     $289 = $288&255;
     $290 = ($285|0)<=($289|0);
     do {
      if ($290) {
       $291 = HEAP32[$xRecordCompare>>2]|0;
       $292 = HEAP32[$nCell>>2]|0;
       $293 = HEAP32[$pCell>>2]|0;
       $294 = (($293) + 1|0);
       $295 = HEAP32[$4>>2]|0;
       $296 = (FUNCTION_TABLE_iiii[$291 & 31]($292,$294,$295)|0);
       HEAP32[$c>>2] = $296;
      } else {
       $297 = HEAP32[$pCell>>2]|0;
       $298 = (($297) + 1|0);
       $299 = HEAP8[$298>>0]|0;
       $300 = $299&255;
       $301 = $300 & 128;
       $302 = ($301|0)!=(0);
       if (!($302)) {
        $303 = HEAP32[$nCell>>2]|0;
        $304 = $303 & 127;
        $305 = $304 << 7;
        $306 = HEAP32[$pCell>>2]|0;
        $307 = (($306) + 1|0);
        $308 = HEAP8[$307>>0]|0;
        $309 = $308&255;
        $310 = (($305) + ($309))|0;
        HEAP32[$nCell>>2] = $310;
        $311 = HEAP32[$pPage>>2]|0;
        $312 = (($311) + 10|0);
        $313 = HEAP16[$312>>1]|0;
        $314 = $313&65535;
        $315 = ($310|0)<=($314|0);
        if ($315) {
         $316 = HEAP32[$xRecordCompare>>2]|0;
         $317 = HEAP32[$nCell>>2]|0;
         $318 = HEAP32[$pCell>>2]|0;
         $319 = (($318) + 2|0);
         $320 = HEAP32[$4>>2]|0;
         $321 = (FUNCTION_TABLE_iiii[$316 & 31]($317,$319,$320)|0);
         HEAP32[$c>>2] = $321;
         break;
        }
       }
       $322 = HEAP32[$pCell>>2]|0;
       $323 = HEAP32[$pPage>>2]|0;
       $324 = (($323) + 7|0);
       $325 = HEAP8[$324>>0]|0;
       $326 = $325&255;
       $327 = (0 - ($326))|0;
       $328 = (($322) + ($327)|0);
       HEAP32[$pCellBody>>2] = $328;
       $329 = HEAP32[$pPage>>2]|0;
       $330 = HEAP32[$pCellBody>>2]|0;
       $331 = HEAP32[$3>>2]|0;
       $332 = (($331) + 24|0);
       _btreeParseCellPtr($329,$330,$332);
       $333 = HEAP32[$3>>2]|0;
       $334 = (($333) + 24|0);
       $335 = $334;
       $336 = $335;
       $337 = HEAP32[$336>>2]|0;
       $338 = (($335) + 4)|0;
       $339 = $338;
       $340 = HEAP32[$339>>2]|0;
       HEAP32[$nCell>>2] = $337;
       $341 = HEAP32[$nCell>>2]|0;
       $342 = ($341|0)<(0);
       $343 = $342 << 31 >> 31;
       $344 = (_sqlite3Malloc($341,$343)|0);
       HEAP32[$pCellKey>>2] = $344;
       $345 = HEAP32[$pCellKey>>2]|0;
       $346 = ($345|0)==(0|0);
       if ($346) {
        label = 37;
        break L24;
       }
       $347 = HEAP32[$idx>>2]|0;
       $348 = $347&65535;
       $349 = HEAP32[$3>>2]|0;
       $350 = (($349) + 76|0);
       $351 = HEAP16[$350>>1]|0;
       $352 = $351 << 16 >> 16;
       $353 = HEAP32[$3>>2]|0;
       $354 = (($353) + 78|0);
       $355 = (($354) + ($352<<1)|0);
       HEAP16[$355>>1] = $348;
       $356 = HEAP32[$3>>2]|0;
       $357 = HEAP32[$nCell>>2]|0;
       $358 = HEAP32[$pCellKey>>2]|0;
       $359 = (_accessPayload($356,0,$357,$358,2)|0);
       HEAP32[$rc>>2] = $359;
       $360 = HEAP32[$rc>>2]|0;
       $361 = ($360|0)!=(0);
       if ($361) {
        label = 39;
        break L24;
       }
       $363 = HEAP32[$xRecordCompare>>2]|0;
       $364 = HEAP32[$nCell>>2]|0;
       $365 = HEAP32[$pCellKey>>2]|0;
       $366 = HEAP32[$4>>2]|0;
       $367 = (FUNCTION_TABLE_iiii[$363 & 31]($364,$365,$366)|0);
       HEAP32[$c>>2] = $367;
       $368 = HEAP32[$pCellKey>>2]|0;
       _sqlite3_free($368);
      }
     } while(0);
     $369 = HEAP32[$c>>2]|0;
     $370 = ($369|0)<(0);
     if ($370) {
      $371 = HEAP32[$idx>>2]|0;
      $372 = (($371) + 1)|0;
      HEAP32[$lwr>>2] = $372;
     } else {
      $373 = HEAP32[$c>>2]|0;
      $374 = ($373|0)>(0);
      if (!($374)) {
       label = 45;
       break L24;
      }
      $375 = HEAP32[$idx>>2]|0;
      $376 = (($375) - 1)|0;
      HEAP32[$upr>>2] = $376;
     }
     $391 = HEAP32[$lwr>>2]|0;
     $392 = HEAP32[$upr>>2]|0;
     $393 = ($391|0)>($392|0);
     if ($393) {
      label = 49;
      break L26;
     }
     $394 = HEAP32[$lwr>>2]|0;
     $395 = HEAP32[$upr>>2]|0;
     $396 = (($394) + ($395))|0;
     $397 = $396 >> 1;
     HEAP32[$idx>>2] = $397;
    }
   }
  } while(0);
  if ((label|0) == 49) {
   label = 0;
   $398 = HEAP32[$pPage>>2]|0;
   $399 = (($398) + 5|0);
   $400 = HEAP8[$399>>0]|0;
   $401 = ($400<<24>>24)!=(0);
   if ($401) {
    label = 50;
    break;
   }
  }
  $413 = HEAP32[$lwr>>2]|0;
  $414 = HEAP32[$pPage>>2]|0;
  $415 = (($414) + 18|0);
  $416 = HEAP16[$415>>1]|0;
  $417 = $416&65535;
  $418 = ($413|0)>=($417|0);
  $419 = HEAP32[$pPage>>2]|0;
  if ($418) {
   $420 = (($419) + 6|0);
   $421 = HEAP8[$420>>0]|0;
   $422 = $421&255;
   $423 = (($422) + 8)|0;
   $424 = HEAP32[$pPage>>2]|0;
   $425 = (($424) + 56|0);
   $426 = HEAP32[$425>>2]|0;
   $427 = (($426) + ($423)|0);
   $428 = (_sqlite3Get4byte($427)|0);
   HEAP32[$chldPg>>2] = $428;
  } else {
   $429 = (($419) + 56|0);
   $430 = HEAP32[$429>>2]|0;
   $431 = HEAP32[$pPage>>2]|0;
   $432 = (($431) + 20|0);
   $433 = HEAP16[$432>>1]|0;
   $434 = $433&65535;
   $435 = HEAP32[$lwr>>2]|0;
   $436 = $435<<1;
   $437 = HEAP32[$pPage>>2]|0;
   $438 = (($437) + 64|0);
   $439 = HEAP32[$438>>2]|0;
   $440 = (($439) + ($436)|0);
   $441 = HEAP8[$440>>0]|0;
   $442 = $441&255;
   $443 = $442 << 8;
   $444 = HEAP32[$lwr>>2]|0;
   $445 = $444<<1;
   $446 = HEAP32[$pPage>>2]|0;
   $447 = (($446) + 64|0);
   $448 = HEAP32[$447>>2]|0;
   $449 = (($448) + ($445)|0);
   $450 = (($449) + 1|0);
   $451 = HEAP8[$450>>0]|0;
   $452 = $451&255;
   $453 = $443 | $452;
   $454 = $434 & $453;
   $455 = (($430) + ($454)|0);
   $456 = (_sqlite3Get4byte($455)|0);
   HEAP32[$chldPg>>2] = $456;
  }
  $457 = HEAP32[$lwr>>2]|0;
  $458 = $457&65535;
  $459 = HEAP32[$3>>2]|0;
  $460 = (($459) + 76|0);
  $461 = HEAP16[$460>>1]|0;
  $462 = $461 << 16 >> 16;
  $463 = HEAP32[$3>>2]|0;
  $464 = (($463) + 78|0);
  $465 = (($464) + ($462<<1)|0);
  HEAP16[$465>>1] = $458;
  $466 = HEAP32[$3>>2]|0;
  $467 = HEAP32[$chldPg>>2]|0;
  $468 = (_moveToChild($466,$467)|0);
  HEAP32[$rc>>2] = $468;
  $469 = HEAP32[$rc>>2]|0;
  $470 = ($469|0)!=(0);
  if ($470) {
   break;
  }
 }
 if ((label|0) == 20) {
  $166 = (_sqlite3CorruptError(56945)|0);
  HEAP32[$2>>2] = $166;
  $481 = HEAP32[$2>>2]|0;
  STACKTOP = sp;return ($481|0);
 }
 else if ((label|0) == 29) {
  $244 = HEAP32[$7>>2]|0;
  HEAP32[$244>>2] = 0;
  HEAP32[$rc>>2] = 0;
 }
 else if ((label|0) == 37) {
  HEAP32[$rc>>2] = 7;
 }
 else if ((label|0) == 39) {
  $362 = HEAP32[$pCellKey>>2]|0;
  _sqlite3_free($362);
 }
 else if ((label|0) == 45) {
  $377 = HEAP32[$7>>2]|0;
  HEAP32[$377>>2] = 0;
  HEAP32[$rc>>2] = 0;
  $378 = HEAP32[$idx>>2]|0;
  $379 = $378&65535;
  $380 = HEAP32[$3>>2]|0;
  $381 = (($380) + 76|0);
  $382 = HEAP16[$381>>1]|0;
  $383 = $382 << 16 >> 16;
  $384 = HEAP32[$3>>2]|0;
  $385 = (($384) + 78|0);
  $386 = (($385) + ($383<<1)|0);
  HEAP16[$386>>1] = $379;
  $387 = HEAP32[$4>>2]|0;
  $388 = (($387) + 7|0);
  $389 = HEAP8[$388>>0]|0;
  $390 = ($389<<24>>24)!=(0);
  if ($390) {
   HEAP32[$rc>>2] = 11;
  }
 }
 else if ((label|0) == 50) {
  $402 = HEAP32[$idx>>2]|0;
  $403 = $402&65535;
  $404 = HEAP32[$3>>2]|0;
  $405 = (($404) + 76|0);
  $406 = HEAP16[$405>>1]|0;
  $407 = $406 << 16 >> 16;
  $408 = HEAP32[$3>>2]|0;
  $409 = (($408) + 78|0);
  $410 = (($409) + ($407<<1)|0);
  HEAP16[$410>>1] = $403;
  $411 = HEAP32[$c>>2]|0;
  $412 = HEAP32[$7>>2]|0;
  HEAP32[$412>>2] = $411;
  HEAP32[$rc>>2] = 0;
 }
 $471 = HEAP32[$3>>2]|0;
 $472 = (($471) + 24|0);
 $473 = (($472) + 20|0);
 HEAP16[$473>>1] = 0;
 $474 = HEAP32[$3>>2]|0;
 $475 = (($474) + 72|0);
 $476 = HEAP8[$475>>0]|0;
 $477 = $476&255;
 $478 = $477 & -7;
 $479 = $478&255;
 HEAP8[$475>>0] = $479;
 $480 = HEAP32[$rc>>2]|0;
 HEAP32[$2>>2] = $480;
 $481 = HEAP32[$2>>2]|0;
 STACKTOP = sp;return ($481|0);
}
function _sqlite3VdbeFindCompare($p) {
 $p = $p|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $5 = 0;
 var $6 = 0, $7 = 0, $8 = 0, $9 = 0, $flags = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $1 = sp + 4|0;
 $flags = sp;
 HEAP32[$1>>2] = $p;
 $2 = HEAP32[$1>>2]|0;
 $3 = HEAP32[$2>>2]|0;
 $4 = (($3) + 6|0);
 $5 = HEAP16[$4>>1]|0;
 $6 = $5&65535;
 $7 = HEAP32[$1>>2]|0;
 $8 = HEAP32[$7>>2]|0;
 $9 = (($8) + 8|0);
 $10 = HEAP16[$9>>1]|0;
 $11 = $10&65535;
 $12 = (($6) + ($11))|0;
 $13 = ($12|0)<=(13);
 if ($13) {
  $14 = HEAP32[$1>>2]|0;
  $15 = (($14) + 8|0);
  $16 = HEAP32[$15>>2]|0;
  $17 = (($16) + 8|0);
  $18 = HEAP16[$17>>1]|0;
  $19 = $18&65535;
  HEAP32[$flags>>2] = $19;
  $20 = HEAP32[$1>>2]|0;
  $21 = HEAP32[$20>>2]|0;
  $22 = (($21) + 16|0);
  $23 = HEAP32[$22>>2]|0;
  $24 = HEAP8[$23>>0]|0;
  $25 = ($24<<24>>24)!=(0);
  $26 = HEAP32[$1>>2]|0;
  $27 = (($26) + 12|0);
  if ($25) {
   HEAP32[$27>>2] = 1;
   $28 = HEAP32[$1>>2]|0;
   $29 = (($28) + 16|0);
   HEAP32[$29>>2] = -1;
  } else {
   HEAP32[$27>>2] = -1;
   $30 = HEAP32[$1>>2]|0;
   $31 = (($30) + 16|0);
   HEAP32[$31>>2] = 1;
  }
  $32 = HEAP32[$flags>>2]|0;
  $33 = $32 & 4;
  $34 = ($33|0)!=(0);
  if ($34) {
   HEAP32[$0>>2] = 15;
   $43 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($43|0);
  }
  $35 = HEAP32[$flags>>2]|0;
  $36 = $35 & 25;
  $37 = ($36|0)==(0);
  if ($37) {
   $38 = HEAP32[$1>>2]|0;
   $39 = HEAP32[$38>>2]|0;
   $40 = (($39) + 20|0);
   $41 = HEAP32[$40>>2]|0;
   $42 = ($41|0)==(0|0);
   if ($42) {
    HEAP32[$0>>2] = 16;
    $43 = HEAP32[$0>>2]|0;
    STACKTOP = sp;return ($43|0);
   }
  }
 }
 HEAP32[$0>>2] = 17;
 $43 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($43|0);
}
function _moveToRoot($pCur) {
 $pCur = $pCur|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0;
 var $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0;
 var $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0;
 var $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0;
 var $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0;
 var $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $pRoot = 0, $rc = 0, $subpage = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 16|0;
 $1 = sp + 12|0;
 $pRoot = sp + 8|0;
 $rc = sp + 4|0;
 $subpage = sp;
 HEAP32[$1>>2] = $pCur;
 HEAP32[$rc>>2] = 0;
 $2 = HEAP32[$1>>2]|0;
 $3 = (($2) + 73|0);
 $4 = HEAP8[$3>>0]|0;
 $5 = $4&255;
 $6 = ($5|0)>=(3);
 do {
  if ($6) {
   $7 = HEAP32[$1>>2]|0;
   $8 = (($7) + 73|0);
   $9 = HEAP8[$8>>0]|0;
   $10 = $9&255;
   $11 = ($10|0)==(4);
   $12 = HEAP32[$1>>2]|0;
   if (!($11)) {
    _sqlite3BtreeClearCursor($12);
    break;
   }
   $13 = (($12) + 68|0);
   $14 = HEAP32[$13>>2]|0;
   HEAP32[$0>>2] = $14;
   $124 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($124|0);
  }
 } while(0);
 $15 = HEAP32[$1>>2]|0;
 $16 = (($15) + 76|0);
 $17 = HEAP16[$16>>1]|0;
 $18 = $17 << 16 >> 16;
 $19 = ($18|0)>=(0);
 L8: do {
  if ($19) {
   while(1) {
    $20 = HEAP32[$1>>2]|0;
    $21 = (($20) + 76|0);
    $22 = HEAP16[$21>>1]|0;
    $23 = ($22<<16>>16)!=(0);
    if (!($23)) {
     break L8;
    }
    $24 = HEAP32[$1>>2]|0;
    $25 = (($24) + 76|0);
    $26 = HEAP16[$25>>1]|0;
    $27 = (($26) + -1)<<16>>16;
    HEAP16[$25>>1] = $27;
    $28 = $26 << 16 >> 16;
    $29 = HEAP32[$1>>2]|0;
    $30 = (($29) + 120|0);
    $31 = (($30) + ($28<<2)|0);
    $32 = HEAP32[$31>>2]|0;
    _releasePage($32);
   }
  } else {
   $33 = HEAP32[$1>>2]|0;
   $34 = (($33) + 60|0);
   $35 = HEAP32[$34>>2]|0;
   $36 = ($35|0)==(0);
   $37 = HEAP32[$1>>2]|0;
   if ($36) {
    $38 = (($37) + 73|0);
    HEAP8[$38>>0] = 0;
    HEAP32[$0>>2] = 0;
    $124 = HEAP32[$0>>2]|0;
    STACKTOP = sp;return ($124|0);
   }
   $39 = HEAP32[$37>>2]|0;
   $40 = (($39) + 4|0);
   $41 = HEAP32[$40>>2]|0;
   $42 = HEAP32[$1>>2]|0;
   $43 = (($42) + 60|0);
   $44 = HEAP32[$43>>2]|0;
   $45 = HEAP32[$1>>2]|0;
   $46 = (($45) + 120|0);
   $47 = HEAP32[$1>>2]|0;
   $48 = (($47) + 72|0);
   $49 = HEAP8[$48>>0]|0;
   $50 = $49&255;
   $51 = $50 & 1;
   $52 = ($51|0)==(0);
   $53 = $52 ? 2 : 0;
   $54 = (_getAndInitPage($41,$44,$46,$53)|0);
   HEAP32[$rc>>2] = $54;
   $55 = HEAP32[$rc>>2]|0;
   $56 = ($55|0)!=(0);
   $57 = HEAP32[$1>>2]|0;
   if (!($56)) {
    $60 = (($57) + 76|0);
    HEAP16[$60>>1] = 0;
    break;
   }
   $58 = (($57) + 73|0);
   HEAP8[$58>>0] = 0;
   $59 = HEAP32[$rc>>2]|0;
   HEAP32[$0>>2] = $59;
   $124 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($124|0);
  }
 } while(0);
 $61 = HEAP32[$1>>2]|0;
 $62 = (($61) + 120|0);
 $63 = HEAP32[$62>>2]|0;
 HEAP32[$pRoot>>2] = $63;
 $64 = HEAP32[$pRoot>>2]|0;
 $65 = HEAP8[$64>>0]|0;
 $66 = $65&255;
 $67 = ($66|0)==(0);
 if (!($67)) {
  $68 = HEAP32[$1>>2]|0;
  $69 = (($68) + 16|0);
  $70 = HEAP32[$69>>2]|0;
  $71 = ($70|0)==(0|0);
  $72 = $71&1;
  $73 = HEAP32[$pRoot>>2]|0;
  $74 = (($73) + 2|0);
  $75 = HEAP8[$74>>0]|0;
  $76 = $75&255;
  $77 = ($72|0)!=($76|0);
  if (!($77)) {
   $79 = HEAP32[$1>>2]|0;
   $80 = (($79) + 78|0);
   HEAP16[$80>>1] = 0;
   $81 = HEAP32[$1>>2]|0;
   $82 = (($81) + 24|0);
   $83 = (($82) + 20|0);
   HEAP16[$83>>1] = 0;
   $84 = HEAP32[$1>>2]|0;
   $85 = (($84) + 72|0);
   $86 = HEAP8[$85>>0]|0;
   $87 = $86&255;
   $88 = $87 & -15;
   $89 = $88&255;
   HEAP8[$85>>0] = $89;
   $90 = HEAP32[$pRoot>>2]|0;
   $91 = (($90) + 18|0);
   $92 = HEAP16[$91>>1]|0;
   $93 = $92&65535;
   $94 = ($93|0)>(0);
   do {
    if ($94) {
     $95 = HEAP32[$1>>2]|0;
     $96 = (($95) + 73|0);
     HEAP8[$96>>0] = 1;
    } else {
     $97 = HEAP32[$pRoot>>2]|0;
     $98 = (($97) + 5|0);
     $99 = HEAP8[$98>>0]|0;
     $100 = ($99<<24>>24)!=(0);
     if ($100) {
      $121 = HEAP32[$1>>2]|0;
      $122 = (($121) + 73|0);
      HEAP8[$122>>0] = 0;
      break;
     }
     $101 = HEAP32[$pRoot>>2]|0;
     $102 = (($101) + 72|0);
     $103 = HEAP32[$102>>2]|0;
     $104 = ($103|0)!=(1);
     if (!($104)) {
      $106 = HEAP32[$pRoot>>2]|0;
      $107 = (($106) + 6|0);
      $108 = HEAP8[$107>>0]|0;
      $109 = $108&255;
      $110 = (($109) + 8)|0;
      $111 = HEAP32[$pRoot>>2]|0;
      $112 = (($111) + 56|0);
      $113 = HEAP32[$112>>2]|0;
      $114 = (($113) + ($110)|0);
      $115 = (_sqlite3Get4byte($114)|0);
      HEAP32[$subpage>>2] = $115;
      $116 = HEAP32[$1>>2]|0;
      $117 = (($116) + 73|0);
      HEAP8[$117>>0] = 1;
      $118 = HEAP32[$1>>2]|0;
      $119 = HEAP32[$subpage>>2]|0;
      $120 = (_moveToChild($118,$119)|0);
      HEAP32[$rc>>2] = $120;
      break;
     }
     $105 = (_sqlite3CorruptError(56709)|0);
     HEAP32[$0>>2] = $105;
     $124 = HEAP32[$0>>2]|0;
     STACKTOP = sp;return ($124|0);
    }
   } while(0);
   $123 = HEAP32[$rc>>2]|0;
   HEAP32[$0>>2] = $123;
   $124 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($124|0);
  }
 }
 $78 = (_sqlite3CorruptError(56698)|0);
 HEAP32[$0>>2] = $78;
 $124 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($124|0);
}
function _sqlite3GetVarint($p,$v) {
 $p = $p|0;
 $v = $v|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0;
 var $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0, $188 = 0;
 var $189 = 0, $19 = 0, $190 = 0, $191 = 0, $192 = 0, $193 = 0, $194 = 0, $195 = 0, $196 = 0, $197 = 0, $198 = 0, $199 = 0, $2 = 0, $20 = 0, $200 = 0, $201 = 0, $202 = 0, $203 = 0, $204 = 0, $205 = 0;
 var $206 = 0, $207 = 0, $208 = 0, $209 = 0, $21 = 0, $210 = 0, $211 = 0, $212 = 0, $213 = 0, $214 = 0, $215 = 0, $216 = 0, $217 = 0, $218 = 0, $219 = 0, $22 = 0, $220 = 0, $221 = 0, $222 = 0, $223 = 0;
 var $224 = 0, $225 = 0, $226 = 0, $227 = 0, $228 = 0, $229 = 0, $23 = 0, $230 = 0, $231 = 0, $232 = 0, $233 = 0, $234 = 0, $235 = 0, $236 = 0, $237 = 0, $238 = 0, $239 = 0, $24 = 0, $240 = 0, $241 = 0;
 var $242 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0;
 var $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0;
 var $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0;
 var $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0;
 var $97 = 0, $98 = 0, $99 = 0, $a = 0, $b = 0, $s = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 20|0;
 $1 = sp + 16|0;
 $2 = sp + 12|0;
 $a = sp + 8|0;
 $b = sp + 4|0;
 $s = sp;
 HEAP32[$1>>2] = $p;
 HEAP32[$2>>2] = $v;
 $3 = HEAP32[$1>>2]|0;
 $4 = HEAP8[$3>>0]|0;
 $5 = $4&255;
 HEAP32[$a>>2] = $5;
 $6 = HEAP32[$a>>2]|0;
 $7 = $6 & 128;
 $8 = ($7|0)!=(0);
 if (!($8)) {
  $9 = HEAP32[$a>>2]|0;
  $10 = HEAP32[$2>>2]|0;
  $11 = $10;
  $12 = $11;
  HEAP32[$12>>2] = $9;
  $13 = (($11) + 4)|0;
  $14 = $13;
  HEAP32[$14>>2] = 0;
  HEAP8[$0>>0] = 1;
  $242 = HEAP8[$0>>0]|0;
  STACKTOP = sp;return ($242|0);
 }
 $15 = HEAP32[$1>>2]|0;
 $16 = (($15) + 1|0);
 HEAP32[$1>>2] = $16;
 $17 = HEAP32[$1>>2]|0;
 $18 = HEAP8[$17>>0]|0;
 $19 = $18&255;
 HEAP32[$b>>2] = $19;
 $20 = HEAP32[$b>>2]|0;
 $21 = $20 & 128;
 $22 = ($21|0)!=(0);
 if (!($22)) {
  $23 = HEAP32[$a>>2]|0;
  $24 = $23 & 127;
  HEAP32[$a>>2] = $24;
  $25 = HEAP32[$a>>2]|0;
  $26 = $25 << 7;
  HEAP32[$a>>2] = $26;
  $27 = HEAP32[$b>>2]|0;
  $28 = HEAP32[$a>>2]|0;
  $29 = $28 | $27;
  HEAP32[$a>>2] = $29;
  $30 = HEAP32[$a>>2]|0;
  $31 = HEAP32[$2>>2]|0;
  $32 = $31;
  $33 = $32;
  HEAP32[$33>>2] = $30;
  $34 = (($32) + 4)|0;
  $35 = $34;
  HEAP32[$35>>2] = 0;
  HEAP8[$0>>0] = 2;
  $242 = HEAP8[$0>>0]|0;
  STACKTOP = sp;return ($242|0);
 }
 $36 = HEAP32[$1>>2]|0;
 $37 = (($36) + 1|0);
 HEAP32[$1>>2] = $37;
 $38 = HEAP32[$a>>2]|0;
 $39 = $38 << 14;
 HEAP32[$a>>2] = $39;
 $40 = HEAP32[$1>>2]|0;
 $41 = HEAP8[$40>>0]|0;
 $42 = $41&255;
 $43 = HEAP32[$a>>2]|0;
 $44 = $43 | $42;
 HEAP32[$a>>2] = $44;
 $45 = HEAP32[$a>>2]|0;
 $46 = $45 & 128;
 $47 = ($46|0)!=(0);
 $48 = HEAP32[$a>>2]|0;
 $49 = $48 & 2080895;
 HEAP32[$a>>2] = $49;
 if (!($47)) {
  $50 = HEAP32[$b>>2]|0;
  $51 = $50 & 127;
  HEAP32[$b>>2] = $51;
  $52 = HEAP32[$b>>2]|0;
  $53 = $52 << 7;
  HEAP32[$b>>2] = $53;
  $54 = HEAP32[$b>>2]|0;
  $55 = HEAP32[$a>>2]|0;
  $56 = $55 | $54;
  HEAP32[$a>>2] = $56;
  $57 = HEAP32[$a>>2]|0;
  $58 = HEAP32[$2>>2]|0;
  $59 = $58;
  $60 = $59;
  HEAP32[$60>>2] = $57;
  $61 = (($59) + 4)|0;
  $62 = $61;
  HEAP32[$62>>2] = 0;
  HEAP8[$0>>0] = 3;
  $242 = HEAP8[$0>>0]|0;
  STACKTOP = sp;return ($242|0);
 }
 $63 = HEAP32[$1>>2]|0;
 $64 = (($63) + 1|0);
 HEAP32[$1>>2] = $64;
 $65 = HEAP32[$b>>2]|0;
 $66 = $65 << 14;
 HEAP32[$b>>2] = $66;
 $67 = HEAP32[$1>>2]|0;
 $68 = HEAP8[$67>>0]|0;
 $69 = $68&255;
 $70 = HEAP32[$b>>2]|0;
 $71 = $70 | $69;
 HEAP32[$b>>2] = $71;
 $72 = HEAP32[$b>>2]|0;
 $73 = $72 & 128;
 $74 = ($73|0)!=(0);
 $75 = HEAP32[$b>>2]|0;
 $76 = $75 & 2080895;
 HEAP32[$b>>2] = $76;
 $77 = HEAP32[$a>>2]|0;
 if (!($74)) {
  $78 = $77 << 7;
  HEAP32[$a>>2] = $78;
  $79 = HEAP32[$b>>2]|0;
  $80 = HEAP32[$a>>2]|0;
  $81 = $80 | $79;
  HEAP32[$a>>2] = $81;
  $82 = HEAP32[$a>>2]|0;
  $83 = HEAP32[$2>>2]|0;
  $84 = $83;
  $85 = $84;
  HEAP32[$85>>2] = $82;
  $86 = (($84) + 4)|0;
  $87 = $86;
  HEAP32[$87>>2] = 0;
  HEAP8[$0>>0] = 4;
  $242 = HEAP8[$0>>0]|0;
  STACKTOP = sp;return ($242|0);
 }
 HEAP32[$s>>2] = $77;
 $88 = HEAP32[$1>>2]|0;
 $89 = (($88) + 1|0);
 HEAP32[$1>>2] = $89;
 $90 = HEAP32[$a>>2]|0;
 $91 = $90 << 14;
 HEAP32[$a>>2] = $91;
 $92 = HEAP32[$1>>2]|0;
 $93 = HEAP8[$92>>0]|0;
 $94 = $93&255;
 $95 = HEAP32[$a>>2]|0;
 $96 = $95 | $94;
 HEAP32[$a>>2] = $96;
 $97 = HEAP32[$a>>2]|0;
 $98 = $97 & 128;
 $99 = ($98|0)!=(0);
 if (!($99)) {
  $100 = HEAP32[$b>>2]|0;
  $101 = $100 << 7;
  HEAP32[$b>>2] = $101;
  $102 = HEAP32[$b>>2]|0;
  $103 = HEAP32[$a>>2]|0;
  $104 = $103 | $102;
  HEAP32[$a>>2] = $104;
  $105 = HEAP32[$s>>2]|0;
  $106 = $105 >>> 18;
  HEAP32[$s>>2] = $106;
  $107 = HEAP32[$s>>2]|0;
  $108 = HEAP32[$a>>2]|0;
  $109 = HEAP32[$2>>2]|0;
  $110 = $109;
  $111 = $110;
  HEAP32[$111>>2] = $108;
  $112 = (($110) + 4)|0;
  $113 = $112;
  HEAP32[$113>>2] = $107;
  HEAP8[$0>>0] = 5;
  $242 = HEAP8[$0>>0]|0;
  STACKTOP = sp;return ($242|0);
 }
 $114 = HEAP32[$s>>2]|0;
 $115 = $114 << 7;
 HEAP32[$s>>2] = $115;
 $116 = HEAP32[$b>>2]|0;
 $117 = HEAP32[$s>>2]|0;
 $118 = $117 | $116;
 HEAP32[$s>>2] = $118;
 $119 = HEAP32[$1>>2]|0;
 $120 = (($119) + 1|0);
 HEAP32[$1>>2] = $120;
 $121 = HEAP32[$b>>2]|0;
 $122 = $121 << 14;
 HEAP32[$b>>2] = $122;
 $123 = HEAP32[$1>>2]|0;
 $124 = HEAP8[$123>>0]|0;
 $125 = $124&255;
 $126 = HEAP32[$b>>2]|0;
 $127 = $126 | $125;
 HEAP32[$b>>2] = $127;
 $128 = HEAP32[$b>>2]|0;
 $129 = $128 & 128;
 $130 = ($129|0)!=(0);
 if (!($130)) {
  $131 = HEAP32[$a>>2]|0;
  $132 = $131 & 2080895;
  HEAP32[$a>>2] = $132;
  $133 = HEAP32[$a>>2]|0;
  $134 = $133 << 7;
  HEAP32[$a>>2] = $134;
  $135 = HEAP32[$b>>2]|0;
  $136 = HEAP32[$a>>2]|0;
  $137 = $136 | $135;
  HEAP32[$a>>2] = $137;
  $138 = HEAP32[$s>>2]|0;
  $139 = $138 >>> 18;
  HEAP32[$s>>2] = $139;
  $140 = HEAP32[$s>>2]|0;
  $141 = HEAP32[$a>>2]|0;
  $142 = HEAP32[$2>>2]|0;
  $143 = $142;
  $144 = $143;
  HEAP32[$144>>2] = $141;
  $145 = (($143) + 4)|0;
  $146 = $145;
  HEAP32[$146>>2] = $140;
  HEAP8[$0>>0] = 6;
  $242 = HEAP8[$0>>0]|0;
  STACKTOP = sp;return ($242|0);
 }
 $147 = HEAP32[$1>>2]|0;
 $148 = (($147) + 1|0);
 HEAP32[$1>>2] = $148;
 $149 = HEAP32[$a>>2]|0;
 $150 = $149 << 14;
 HEAP32[$a>>2] = $150;
 $151 = HEAP32[$1>>2]|0;
 $152 = HEAP8[$151>>0]|0;
 $153 = $152&255;
 $154 = HEAP32[$a>>2]|0;
 $155 = $154 | $153;
 HEAP32[$a>>2] = $155;
 $156 = HEAP32[$a>>2]|0;
 $157 = $156 & 128;
 $158 = ($157|0)!=(0);
 $159 = HEAP32[$a>>2]|0;
 if (!($158)) {
  $160 = $159 & -266354561;
  HEAP32[$a>>2] = $160;
  $161 = HEAP32[$b>>2]|0;
  $162 = $161 & 2080895;
  HEAP32[$b>>2] = $162;
  $163 = HEAP32[$b>>2]|0;
  $164 = $163 << 7;
  HEAP32[$b>>2] = $164;
  $165 = HEAP32[$b>>2]|0;
  $166 = HEAP32[$a>>2]|0;
  $167 = $166 | $165;
  HEAP32[$a>>2] = $167;
  $168 = HEAP32[$s>>2]|0;
  $169 = $168 >>> 11;
  HEAP32[$s>>2] = $169;
  $170 = HEAP32[$s>>2]|0;
  $171 = HEAP32[$a>>2]|0;
  $172 = HEAP32[$2>>2]|0;
  $173 = $172;
  $174 = $173;
  HEAP32[$174>>2] = $171;
  $175 = (($173) + 4)|0;
  $176 = $175;
  HEAP32[$176>>2] = $170;
  HEAP8[$0>>0] = 7;
  $242 = HEAP8[$0>>0]|0;
  STACKTOP = sp;return ($242|0);
 }
 $177 = $159 & 2080895;
 HEAP32[$a>>2] = $177;
 $178 = HEAP32[$1>>2]|0;
 $179 = (($178) + 1|0);
 HEAP32[$1>>2] = $179;
 $180 = HEAP32[$b>>2]|0;
 $181 = $180 << 14;
 HEAP32[$b>>2] = $181;
 $182 = HEAP32[$1>>2]|0;
 $183 = HEAP8[$182>>0]|0;
 $184 = $183&255;
 $185 = HEAP32[$b>>2]|0;
 $186 = $185 | $184;
 HEAP32[$b>>2] = $186;
 $187 = HEAP32[$b>>2]|0;
 $188 = $187 & 128;
 $189 = ($188|0)!=(0);
 if ($189) {
  $206 = HEAP32[$1>>2]|0;
  $207 = (($206) + 1|0);
  HEAP32[$1>>2] = $207;
  $208 = HEAP32[$a>>2]|0;
  $209 = $208 << 15;
  HEAP32[$a>>2] = $209;
  $210 = HEAP32[$1>>2]|0;
  $211 = HEAP8[$210>>0]|0;
  $212 = $211&255;
  $213 = HEAP32[$a>>2]|0;
  $214 = $213 | $212;
  HEAP32[$a>>2] = $214;
  $215 = HEAP32[$b>>2]|0;
  $216 = $215 & 2080895;
  HEAP32[$b>>2] = $216;
  $217 = HEAP32[$b>>2]|0;
  $218 = $217 << 8;
  HEAP32[$b>>2] = $218;
  $219 = HEAP32[$b>>2]|0;
  $220 = HEAP32[$a>>2]|0;
  $221 = $220 | $219;
  HEAP32[$a>>2] = $221;
  $222 = HEAP32[$s>>2]|0;
  $223 = $222 << 4;
  HEAP32[$s>>2] = $223;
  $224 = HEAP32[$1>>2]|0;
  $225 = (($224) + -4|0);
  $226 = HEAP8[$225>>0]|0;
  $227 = $226&255;
  HEAP32[$b>>2] = $227;
  $228 = HEAP32[$b>>2]|0;
  $229 = $228 & 127;
  HEAP32[$b>>2] = $229;
  $230 = HEAP32[$b>>2]|0;
  $231 = $230 >>> 3;
  HEAP32[$b>>2] = $231;
  $232 = HEAP32[$b>>2]|0;
  $233 = HEAP32[$s>>2]|0;
  $234 = $233 | $232;
  HEAP32[$s>>2] = $234;
  $235 = HEAP32[$s>>2]|0;
  $236 = HEAP32[$a>>2]|0;
  $237 = HEAP32[$2>>2]|0;
  $238 = $237;
  $239 = $238;
  HEAP32[$239>>2] = $236;
  $240 = (($238) + 4)|0;
  $241 = $240;
  HEAP32[$241>>2] = $235;
  HEAP8[$0>>0] = 9;
  $242 = HEAP8[$0>>0]|0;
  STACKTOP = sp;return ($242|0);
 } else {
  $190 = HEAP32[$b>>2]|0;
  $191 = $190 & -266354561;
  HEAP32[$b>>2] = $191;
  $192 = HEAP32[$a>>2]|0;
  $193 = $192 << 7;
  HEAP32[$a>>2] = $193;
  $194 = HEAP32[$b>>2]|0;
  $195 = HEAP32[$a>>2]|0;
  $196 = $195 | $194;
  HEAP32[$a>>2] = $196;
  $197 = HEAP32[$s>>2]|0;
  $198 = $197 >>> 4;
  HEAP32[$s>>2] = $198;
  $199 = HEAP32[$s>>2]|0;
  $200 = HEAP32[$a>>2]|0;
  $201 = HEAP32[$2>>2]|0;
  $202 = $201;
  $203 = $202;
  HEAP32[$203>>2] = $200;
  $204 = (($202) + 4)|0;
  $205 = $204;
  HEAP32[$205>>2] = $199;
  HEAP8[$0>>0] = 8;
  $242 = HEAP8[$0>>0]|0;
  STACKTOP = sp;return ($242|0);
 }
 return 0|0;
}
function _btreeParseCellPtr($pPage,$pCell,$pInfo) {
 $pPage = $pPage|0;
 $pCell = $pCell|0;
 $pInfo = $pInfo|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0;
 var $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0;
 var $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0;
 var $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0;
 var $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0;
 var $96 = 0, $97 = 0, $98 = 0, $99 = 0, $maxLocal = 0, $minLocal = 0, $nPayload = 0, $pIter = 0, $surplus = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 28|0;
 $1 = sp + 24|0;
 $2 = sp + 20|0;
 $pIter = sp + 16|0;
 $nPayload = sp + 12|0;
 $minLocal = sp + 8|0;
 $maxLocal = sp + 4|0;
 $surplus = sp;
 HEAP32[$0>>2] = $pPage;
 HEAP32[$1>>2] = $pCell;
 HEAP32[$2>>2] = $pInfo;
 $3 = HEAP32[$0>>2]|0;
 $4 = (($3) + 3|0);
 $5 = HEAP8[$4>>0]|0;
 $6 = ($5<<24>>24)!=(0);
 if ($6) {
  $7 = HEAP32[$1>>2]|0;
  $8 = HEAP32[$1>>2]|0;
  $9 = HEAP8[$8>>0]|0;
  $10 = $9&255;
  $11 = ($10|0)<(128);
  $12 = HEAP32[$1>>2]|0;
  if ($11) {
   $13 = HEAP8[$12>>0]|0;
   $14 = $13&255;
   HEAP32[$nPayload>>2] = $14;
   $18 = 1;
  } else {
   $15 = (_sqlite3GetVarint32($12,$nPayload)|0);
   $16 = $15&255;
   $18 = $16;
  }
  $17 = $18&255;
  $19 = $17&255;
  $20 = (($7) + ($19)|0);
  HEAP32[$pIter>>2] = $20;
  $21 = HEAP32[$pIter>>2]|0;
  $22 = HEAP32[$2>>2]|0;
  $23 = (_sqlite3GetVarint($21,$22)|0);
  $24 = $23&255;
  $25 = HEAP32[$pIter>>2]|0;
  $26 = (($25) + ($24)|0);
  HEAP32[$pIter>>2] = $26;
 } else {
  $27 = HEAP32[$0>>2]|0;
  $28 = (($27) + 4|0);
  $29 = HEAP8[$28>>0]|0;
  $30 = ($29<<24>>24)!=(0);
  $31 = HEAP32[$1>>2]|0;
  if ($30) {
   $32 = (($31) + 4|0);
   $33 = HEAP32[$2>>2]|0;
   $34 = (_sqlite3GetVarint($32,$33)|0);
   $35 = $34&255;
   $36 = (4 + ($35))|0;
   $37 = $36&65535;
   $38 = HEAP32[$2>>2]|0;
   $39 = (($38) + 20|0);
   HEAP16[$39>>1] = $37;
   $40 = HEAP32[$2>>2]|0;
   $41 = (($40) + 12|0);
   HEAP32[$41>>2] = 0;
   $42 = HEAP32[$2>>2]|0;
   $43 = (($42) + 16|0);
   HEAP16[$43>>1] = 0;
   $44 = HEAP32[$2>>2]|0;
   $45 = (($44) + 18|0);
   HEAP16[$45>>1] = 0;
   $46 = HEAP32[$2>>2]|0;
   $47 = (($46) + 8|0);
   HEAP32[$47>>2] = 0;
   STACKTOP = sp;return;
  }
  $48 = HEAP32[$0>>2]|0;
  $49 = (($48) + 7|0);
  $50 = HEAP8[$49>>0]|0;
  $51 = $50&255;
  $52 = (($31) + ($51)|0);
  HEAP32[$pIter>>2] = $52;
  $53 = HEAP32[$pIter>>2]|0;
  $54 = HEAP8[$53>>0]|0;
  $55 = $54&255;
  $56 = ($55|0)<(128);
  $57 = HEAP32[$pIter>>2]|0;
  if ($56) {
   $58 = HEAP8[$57>>0]|0;
   $59 = $58&255;
   HEAP32[$nPayload>>2] = $59;
   $63 = 1;
  } else {
   $60 = (_sqlite3GetVarint32($57,$nPayload)|0);
   $61 = $60&255;
   $63 = $61;
  }
  $62 = $63&255;
  $64 = $62&255;
  $65 = HEAP32[$pIter>>2]|0;
  $66 = (($65) + ($64)|0);
  HEAP32[$pIter>>2] = $66;
  $67 = HEAP32[$nPayload>>2]|0;
  $68 = HEAP32[$2>>2]|0;
  $69 = $68;
  $70 = $69;
  HEAP32[$70>>2] = $67;
  $71 = (($69) + 4)|0;
  $72 = $71;
  HEAP32[$72>>2] = 0;
 }
 $73 = HEAP32[$nPayload>>2]|0;
 $74 = HEAP32[$2>>2]|0;
 $75 = (($74) + 12|0);
 HEAP32[$75>>2] = $73;
 $76 = HEAP32[$pIter>>2]|0;
 $77 = HEAP32[$2>>2]|0;
 $78 = (($77) + 8|0);
 HEAP32[$78>>2] = $76;
 $79 = HEAP32[$nPayload>>2]|0;
 $80 = HEAP32[$0>>2]|0;
 $81 = (($80) + 10|0);
 $82 = HEAP16[$81>>1]|0;
 $83 = $82&65535;
 $84 = ($79>>>0)<=($83>>>0);
 if ($84) {
  $85 = HEAP32[$nPayload>>2]|0;
  $86 = HEAP32[$pIter>>2]|0;
  $87 = HEAP32[$1>>2]|0;
  $88 = $86;
  $89 = $87;
  $90 = (($88) - ($89))|0;
  $91 = $90&65535;
  $92 = $91&65535;
  $93 = (($85) + ($92))|0;
  $94 = $93&65535;
  $95 = HEAP32[$2>>2]|0;
  $96 = (($95) + 20|0);
  HEAP16[$96>>1] = $94;
  $97 = HEAP32[$2>>2]|0;
  $98 = (($97) + 20|0);
  $99 = HEAP16[$98>>1]|0;
  $100 = $99&65535;
  $101 = ($100|0)<(4);
  if ($101) {
   $102 = HEAP32[$2>>2]|0;
   $103 = (($102) + 20|0);
   HEAP16[$103>>1] = 4;
  }
  $104 = HEAP32[$nPayload>>2]|0;
  $105 = $104&65535;
  $106 = HEAP32[$2>>2]|0;
  $107 = (($106) + 16|0);
  HEAP16[$107>>1] = $105;
  $108 = HEAP32[$2>>2]|0;
  $109 = (($108) + 18|0);
  HEAP16[$109>>1] = 0;
  STACKTOP = sp;return;
 }
 $110 = HEAP32[$0>>2]|0;
 $111 = (($110) + 12|0);
 $112 = HEAP16[$111>>1]|0;
 $113 = $112&65535;
 HEAP32[$minLocal>>2] = $113;
 $114 = HEAP32[$0>>2]|0;
 $115 = (($114) + 10|0);
 $116 = HEAP16[$115>>1]|0;
 $117 = $116&65535;
 HEAP32[$maxLocal>>2] = $117;
 $118 = HEAP32[$minLocal>>2]|0;
 $119 = HEAP32[$nPayload>>2]|0;
 $120 = HEAP32[$minLocal>>2]|0;
 $121 = (($119) - ($120))|0;
 $122 = HEAP32[$0>>2]|0;
 $123 = (($122) + 52|0);
 $124 = HEAP32[$123>>2]|0;
 $125 = (($124) + 36|0);
 $126 = HEAP32[$125>>2]|0;
 $127 = (($126) - 4)|0;
 $128 = (($121>>>0) % ($127>>>0))&-1;
 $129 = (($118) + ($128))|0;
 HEAP32[$surplus>>2] = $129;
 $130 = HEAP32[$surplus>>2]|0;
 $131 = HEAP32[$maxLocal>>2]|0;
 $132 = ($130|0)<=($131|0);
 if ($132) {
  $133 = HEAP32[$surplus>>2]|0;
  $134 = $133&65535;
  $135 = HEAP32[$2>>2]|0;
  $136 = (($135) + 16|0);
  HEAP16[$136>>1] = $134;
 } else {
  $137 = HEAP32[$minLocal>>2]|0;
  $138 = $137&65535;
  $139 = HEAP32[$2>>2]|0;
  $140 = (($139) + 16|0);
  HEAP16[$140>>1] = $138;
 }
 $141 = HEAP32[$2>>2]|0;
 $142 = (($141) + 16|0);
 $143 = HEAP16[$142>>1]|0;
 $144 = $143&65535;
 $145 = HEAP32[$2>>2]|0;
 $146 = (($145) + 8|0);
 $147 = HEAP32[$146>>2]|0;
 $148 = (($147) + ($144)|0);
 $149 = HEAP32[$1>>2]|0;
 $150 = $148;
 $151 = $149;
 $152 = (($150) - ($151))|0;
 $153 = $152&65535;
 $154 = HEAP32[$2>>2]|0;
 $155 = (($154) + 18|0);
 HEAP16[$155>>1] = $153;
 $156 = HEAP32[$2>>2]|0;
 $157 = (($156) + 18|0);
 $158 = HEAP16[$157>>1]|0;
 $159 = $158&65535;
 $160 = (($159) + 4)|0;
 $161 = $160&65535;
 $162 = HEAP32[$2>>2]|0;
 $163 = (($162) + 20|0);
 HEAP16[$163>>1] = $161;
 STACKTOP = sp;return;
}
function _accessPayload($pCur,$offset,$amt,$pBuf,$eOp) {
 $pCur = $pCur|0;
 $offset = $offset|0;
 $amt = $amt|0;
 $pBuf = $pBuf|0;
 $eOp = $eOp|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0;
 var $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0, $188 = 0;
 var $189 = 0, $19 = 0, $190 = 0, $191 = 0, $192 = 0, $193 = 0, $194 = 0, $195 = 0, $196 = 0, $197 = 0, $198 = 0, $199 = 0, $2 = 0, $20 = 0, $200 = 0, $201 = 0, $202 = 0, $203 = 0, $204 = 0, $205 = 0;
 var $206 = 0, $207 = 0, $208 = 0, $209 = 0, $21 = 0, $210 = 0, $211 = 0, $212 = 0, $213 = 0, $214 = 0, $215 = 0, $216 = 0, $217 = 0, $218 = 0, $219 = 0, $22 = 0, $220 = 0, $221 = 0, $222 = 0, $223 = 0;
 var $224 = 0, $225 = 0, $226 = 0, $227 = 0, $228 = 0, $229 = 0, $23 = 0, $230 = 0, $231 = 0, $232 = 0, $233 = 0, $234 = 0, $235 = 0, $236 = 0, $237 = 0, $238 = 0, $239 = 0, $24 = 0, $240 = 0, $241 = 0;
 var $242 = 0, $243 = 0, $244 = 0, $245 = 0, $246 = 0, $247 = 0, $248 = 0, $249 = 0, $25 = 0, $250 = 0, $251 = 0, $252 = 0, $253 = 0, $254 = 0, $255 = 0, $256 = 0, $257 = 0, $258 = 0, $259 = 0, $26 = 0;
 var $260 = 0, $261 = 0, $262 = 0, $263 = 0, $264 = 0, $265 = 0, $266 = 0, $267 = 0, $268 = 0, $269 = 0, $27 = 0, $270 = 0, $271 = 0, $272 = 0, $273 = 0, $274 = 0, $275 = 0, $276 = 0, $277 = 0, $278 = 0;
 var $279 = 0, $28 = 0, $280 = 0, $281 = 0, $282 = 0, $283 = 0, $284 = 0, $285 = 0, $286 = 0, $287 = 0, $288 = 0, $289 = 0, $29 = 0, $290 = 0, $291 = 0, $292 = 0, $293 = 0, $294 = 0, $295 = 0, $296 = 0;
 var $297 = 0, $298 = 0, $299 = 0, $3 = 0, $30 = 0, $300 = 0, $301 = 0, $302 = 0, $303 = 0, $304 = 0, $305 = 0, $306 = 0, $307 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0;
 var $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0;
 var $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0;
 var $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0;
 var $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $a = 0, $a1 = 0, $aNew = 0, $aPayload = 0, $iIdx = 0, $iPage = 0, $nOvfl = 0, $nextPage = 0, $or$cond = 0, $or$cond3 = 0, $or$cond5 = 0, $or$cond7 = 0;
 var $ovflSize = 0, $pBt = 0, $pDbPage = 0, $pPage = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 80|0;
 $0 = sp + 72|0;
 $1 = sp + 68|0;
 $2 = sp + 64|0;
 $3 = sp + 60|0;
 $4 = sp + 56|0;
 $5 = sp + 52|0;
 $aPayload = sp + 48|0;
 $rc = sp + 44|0;
 $iIdx = sp + 40|0;
 $pPage = sp + 36|0;
 $pBt = sp + 32|0;
 $iPage = sp + 28|0;
 $a = sp + 24|0;
 $ovflSize = sp + 20|0;
 $nextPage = sp + 16|0;
 $nOvfl = sp + 12|0;
 $aNew = sp + 8|0;
 $a1 = sp + 4|0;
 $pDbPage = sp;
 HEAP32[$1>>2] = $pCur;
 HEAP32[$2>>2] = $offset;
 HEAP32[$3>>2] = $amt;
 HEAP32[$4>>2] = $pBuf;
 HEAP32[$5>>2] = $eOp;
 HEAP32[$rc>>2] = 0;
 HEAP32[$iIdx>>2] = 0;
 $6 = HEAP32[$1>>2]|0;
 $7 = (($6) + 76|0);
 $8 = HEAP16[$7>>1]|0;
 $9 = $8 << 16 >> 16;
 $10 = HEAP32[$1>>2]|0;
 $11 = (($10) + 120|0);
 $12 = (($11) + ($9<<2)|0);
 $13 = HEAP32[$12>>2]|0;
 HEAP32[$pPage>>2] = $13;
 $14 = HEAP32[$1>>2]|0;
 $15 = (($14) + 4|0);
 $16 = HEAP32[$15>>2]|0;
 HEAP32[$pBt>>2] = $16;
 $17 = HEAP32[$1>>2]|0;
 $18 = (($17) + 24|0);
 $19 = (($18) + 20|0);
 $20 = HEAP16[$19>>1]|0;
 $21 = $20&65535;
 $22 = ($21|0)==(0);
 if ($22) {
  $23 = HEAP32[$1>>2]|0;
  $24 = (($23) + 76|0);
  $25 = HEAP16[$24>>1]|0;
  $26 = $25 << 16 >> 16;
  HEAP32[$iPage>>2] = $26;
  $27 = HEAP32[$iPage>>2]|0;
  $28 = HEAP32[$1>>2]|0;
  $29 = (($28) + 120|0);
  $30 = (($29) + ($27<<2)|0);
  $31 = HEAP32[$30>>2]|0;
  $32 = HEAP32[$iPage>>2]|0;
  $33 = HEAP32[$1>>2]|0;
  $34 = (($33) + 78|0);
  $35 = (($34) + ($32<<1)|0);
  $36 = HEAP16[$35>>1]|0;
  $37 = $36&65535;
  $38 = HEAP32[$1>>2]|0;
  $39 = (($38) + 24|0);
  _btreeParseCell($31,$37,$39);
  $40 = HEAP32[$1>>2]|0;
  $41 = (($40) + 72|0);
  $42 = HEAP8[$41>>0]|0;
  $43 = $42&255;
  $44 = $43 | 2;
  $45 = $44&255;
  HEAP8[$41>>0] = $45;
 }
 $46 = HEAP32[$1>>2]|0;
 $47 = (($46) + 24|0);
 $48 = (($47) + 8|0);
 $49 = HEAP32[$48>>2]|0;
 HEAP32[$aPayload>>2] = $49;
 $50 = HEAP32[$1>>2]|0;
 $51 = (($50) + 24|0);
 $52 = (($51) + 16|0);
 $53 = HEAP16[$52>>1]|0;
 $54 = $53&65535;
 $55 = HEAP32[$aPayload>>2]|0;
 $56 = (($55) + ($54)|0);
 $57 = HEAP32[$pBt>>2]|0;
 $58 = (($57) + 36|0);
 $59 = HEAP32[$58>>2]|0;
 $60 = HEAP32[$pPage>>2]|0;
 $61 = (($60) + 56|0);
 $62 = HEAP32[$61>>2]|0;
 $63 = (($62) + ($59)|0);
 $64 = ($56>>>0)>($63>>>0);
 if ($64) {
  $65 = (_sqlite3CorruptError(56278)|0);
  HEAP32[$0>>2] = $65;
  $307 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($307|0);
 }
 $66 = HEAP32[$2>>2]|0;
 $67 = HEAP32[$1>>2]|0;
 $68 = (($67) + 24|0);
 $69 = (($68) + 16|0);
 $70 = HEAP16[$69>>1]|0;
 $71 = $70&65535;
 $72 = ($66>>>0)<($71>>>0);
 if ($72) {
  $73 = HEAP32[$3>>2]|0;
  HEAP32[$a>>2] = $73;
  $74 = HEAP32[$a>>2]|0;
  $75 = HEAP32[$2>>2]|0;
  $76 = (($74) + ($75))|0;
  $77 = HEAP32[$1>>2]|0;
  $78 = (($77) + 24|0);
  $79 = (($78) + 16|0);
  $80 = HEAP16[$79>>1]|0;
  $81 = $80&65535;
  $82 = ($76>>>0)>($81>>>0);
  if ($82) {
   $83 = HEAP32[$1>>2]|0;
   $84 = (($83) + 24|0);
   $85 = (($84) + 16|0);
   $86 = HEAP16[$85>>1]|0;
   $87 = $86&65535;
   $88 = HEAP32[$2>>2]|0;
   $89 = (($87) - ($88))|0;
   HEAP32[$a>>2] = $89;
  }
  $90 = HEAP32[$2>>2]|0;
  $91 = HEAP32[$aPayload>>2]|0;
  $92 = (($91) + ($90)|0);
  $93 = HEAP32[$4>>2]|0;
  $94 = HEAP32[$a>>2]|0;
  $95 = HEAP32[$5>>2]|0;
  $96 = $95 & 1;
  $97 = HEAP32[$pPage>>2]|0;
  $98 = (($97) + 68|0);
  $99 = HEAP32[$98>>2]|0;
  $100 = (_copyPayload($92,$93,$94,$96,$99)|0);
  HEAP32[$rc>>2] = $100;
  HEAP32[$2>>2] = 0;
  $101 = HEAP32[$a>>2]|0;
  $102 = HEAP32[$4>>2]|0;
  $103 = (($102) + ($101)|0);
  HEAP32[$4>>2] = $103;
  $104 = HEAP32[$a>>2]|0;
  $105 = HEAP32[$3>>2]|0;
  $106 = (($105) - ($104))|0;
  HEAP32[$3>>2] = $106;
 } else {
  $107 = HEAP32[$1>>2]|0;
  $108 = (($107) + 24|0);
  $109 = (($108) + 16|0);
  $110 = HEAP16[$109>>1]|0;
  $111 = $110&65535;
  $112 = HEAP32[$2>>2]|0;
  $113 = (($112) - ($111))|0;
  HEAP32[$2>>2] = $113;
 }
 $114 = HEAP32[$rc>>2]|0;
 $115 = ($114|0)==(0);
 $116 = HEAP32[$3>>2]|0;
 $117 = ($116>>>0)>(0);
 $or$cond = $115 & $117;
 L15: do {
  if ($or$cond) {
   $118 = HEAP32[$pBt>>2]|0;
   $119 = (($118) + 36|0);
   $120 = HEAP32[$119>>2]|0;
   $121 = (($120) - 4)|0;
   HEAP32[$ovflSize>>2] = $121;
   $122 = HEAP32[$1>>2]|0;
   $123 = (($122) + 24|0);
   $124 = (($123) + 16|0);
   $125 = HEAP16[$124>>1]|0;
   $126 = $125&65535;
   $127 = HEAP32[$aPayload>>2]|0;
   $128 = (($127) + ($126)|0);
   $129 = (_sqlite3Get4byte($128)|0);
   HEAP32[$nextPage>>2] = $129;
   $130 = HEAP32[$5>>2]|0;
   $131 = ($130|0)!=(2);
   if ($131) {
    $132 = HEAP32[$1>>2]|0;
    $133 = (($132) + 72|0);
    $134 = HEAP8[$133>>0]|0;
    $135 = $134&255;
    $136 = $135 & 4;
    $137 = ($136|0)==(0);
    if ($137) {
     $138 = HEAP32[$1>>2]|0;
     $139 = (($138) + 24|0);
     $140 = (($139) + 12|0);
     $141 = HEAP32[$140>>2]|0;
     $142 = HEAP32[$1>>2]|0;
     $143 = (($142) + 24|0);
     $144 = (($143) + 16|0);
     $145 = HEAP16[$144>>1]|0;
     $146 = $145&65535;
     $147 = (($141) - ($146))|0;
     $148 = HEAP32[$ovflSize>>2]|0;
     $149 = (($147) + ($148))|0;
     $150 = (($149) - 1)|0;
     $151 = HEAP32[$ovflSize>>2]|0;
     $152 = (($150>>>0) / ($151>>>0))&-1;
     HEAP32[$nOvfl>>2] = $152;
     $153 = HEAP32[$nOvfl>>2]|0;
     $154 = HEAP32[$1>>2]|0;
     $155 = (($154) + 64|0);
     $156 = HEAP32[$155>>2]|0;
     $157 = ($153|0)>($156|0);
     do {
      if ($157) {
       $158 = HEAP32[$1>>2]|0;
       $159 = HEAP32[$158>>2]|0;
       $160 = HEAP32[$159>>2]|0;
       $161 = HEAP32[$1>>2]|0;
       $162 = (($161) + 20|0);
       $163 = HEAP32[$162>>2]|0;
       $164 = HEAP32[$nOvfl>>2]|0;
       $165 = $164<<1;
       $166 = $165<<2;
       $167 = (_sqlite3DbRealloc($160,$163,$166,0)|0);
       HEAP32[$aNew>>2] = $167;
       $168 = HEAP32[$aNew>>2]|0;
       $169 = ($168|0)==(0|0);
       if ($169) {
        HEAP32[$rc>>2] = 7;
        break;
       } else {
        $170 = HEAP32[$nOvfl>>2]|0;
        $171 = $170<<1;
        $172 = HEAP32[$1>>2]|0;
        $173 = (($172) + 64|0);
        HEAP32[$173>>2] = $171;
        $174 = HEAP32[$aNew>>2]|0;
        $175 = HEAP32[$1>>2]|0;
        $176 = (($175) + 20|0);
        HEAP32[$176>>2] = $174;
        break;
       }
      }
     } while(0);
     $177 = HEAP32[$rc>>2]|0;
     $178 = ($177|0)==(0);
     if ($178) {
      $179 = HEAP32[$1>>2]|0;
      $180 = (($179) + 20|0);
      $181 = HEAP32[$180>>2]|0;
      $182 = HEAP32[$nOvfl>>2]|0;
      $183 = $182<<2;
      _memset(($181|0),0,($183|0))|0;
      $184 = HEAP32[$1>>2]|0;
      $185 = (($184) + 72|0);
      $186 = HEAP8[$185>>0]|0;
      $187 = $186&255;
      $188 = $187 | 4;
      $189 = $188&255;
      HEAP8[$185>>0] = $189;
     }
    }
   }
   $190 = HEAP32[$1>>2]|0;
   $191 = (($190) + 72|0);
   $192 = HEAP8[$191>>0]|0;
   $193 = $192&255;
   $194 = $193 & 4;
   $195 = ($194|0)!=(0);
   if ($195) {
    $196 = HEAP32[$2>>2]|0;
    $197 = HEAP32[$ovflSize>>2]|0;
    $198 = (($196>>>0) / ($197>>>0))&-1;
    $199 = HEAP32[$1>>2]|0;
    $200 = (($199) + 20|0);
    $201 = HEAP32[$200>>2]|0;
    $202 = (($201) + ($198<<2)|0);
    $203 = HEAP32[$202>>2]|0;
    $204 = ($203|0)!=(0);
    if ($204) {
     $205 = HEAP32[$2>>2]|0;
     $206 = HEAP32[$ovflSize>>2]|0;
     $207 = (($205>>>0) / ($206>>>0))&-1;
     HEAP32[$iIdx>>2] = $207;
     $208 = HEAP32[$iIdx>>2]|0;
     $209 = HEAP32[$1>>2]|0;
     $210 = (($209) + 20|0);
     $211 = HEAP32[$210>>2]|0;
     $212 = (($211) + ($208<<2)|0);
     $213 = HEAP32[$212>>2]|0;
     HEAP32[$nextPage>>2] = $213;
     $214 = HEAP32[$2>>2]|0;
     $215 = HEAP32[$ovflSize>>2]|0;
     $216 = (($214>>>0) % ($215>>>0))&-1;
     HEAP32[$2>>2] = $216;
    }
   }
   while(1) {
    $217 = HEAP32[$rc>>2]|0;
    $218 = ($217|0)==(0);
    $219 = HEAP32[$3>>2]|0;
    $220 = ($219>>>0)>(0);
    $or$cond3 = $218 & $220;
    $221 = HEAP32[$nextPage>>2]|0;
    $222 = ($221|0)!=(0);
    $or$cond5 = $or$cond3 & $222;
    if (!($or$cond5)) {
     break L15;
    }
    $223 = HEAP32[$1>>2]|0;
    $224 = (($223) + 72|0);
    $225 = HEAP8[$224>>0]|0;
    $226 = $225&255;
    $227 = $226 & 4;
    $228 = ($227|0)!=(0);
    if ($228) {
     $229 = HEAP32[$nextPage>>2]|0;
     $230 = HEAP32[$iIdx>>2]|0;
     $231 = HEAP32[$1>>2]|0;
     $232 = (($231) + 20|0);
     $233 = HEAP32[$232>>2]|0;
     $234 = (($233) + ($230<<2)|0);
     HEAP32[$234>>2] = $229;
    }
    $235 = HEAP32[$2>>2]|0;
    $236 = HEAP32[$ovflSize>>2]|0;
    $237 = ($235>>>0)>=($236>>>0);
    if ($237) {
     $238 = HEAP32[$iIdx>>2]|0;
     $239 = (($238) + 1)|0;
     $240 = HEAP32[$1>>2]|0;
     $241 = (($240) + 20|0);
     $242 = HEAP32[$241>>2]|0;
     $243 = (($242) + ($239<<2)|0);
     $244 = HEAP32[$243>>2]|0;
     $245 = ($244|0)!=(0);
     if ($245) {
      $246 = HEAP32[$iIdx>>2]|0;
      $247 = (($246) + 1)|0;
      $248 = HEAP32[$1>>2]|0;
      $249 = (($248) + 20|0);
      $250 = HEAP32[$249>>2]|0;
      $251 = (($250) + ($247<<2)|0);
      $252 = HEAP32[$251>>2]|0;
      HEAP32[$nextPage>>2] = $252;
     } else {
      $253 = HEAP32[$pBt>>2]|0;
      $254 = HEAP32[$nextPage>>2]|0;
      $255 = (_getOverflowPage($253,$254,0,$nextPage)|0);
      HEAP32[$rc>>2] = $255;
     }
     $256 = HEAP32[$ovflSize>>2]|0;
     $257 = HEAP32[$2>>2]|0;
     $258 = (($257) - ($256))|0;
     HEAP32[$2>>2] = $258;
    } else {
     $259 = HEAP32[$3>>2]|0;
     HEAP32[$a1>>2] = $259;
     $260 = HEAP32[$a1>>2]|0;
     $261 = HEAP32[$2>>2]|0;
     $262 = (($260) + ($261))|0;
     $263 = HEAP32[$ovflSize>>2]|0;
     $264 = ($262>>>0)>($263>>>0);
     if ($264) {
      $265 = HEAP32[$ovflSize>>2]|0;
      $266 = HEAP32[$2>>2]|0;
      $267 = (($265) - ($266))|0;
      HEAP32[$a1>>2] = $267;
     }
     $268 = HEAP32[$pBt>>2]|0;
     $269 = HEAP32[$268>>2]|0;
     $270 = HEAP32[$nextPage>>2]|0;
     $271 = HEAP32[$5>>2]|0;
     $272 = $271 & 1;
     $273 = ($272|0)==(0);
     $274 = $273 ? 2 : 0;
     $275 = (_sqlite3PagerAcquire($269,$270,$pDbPage,$274)|0);
     HEAP32[$rc>>2] = $275;
     $276 = HEAP32[$rc>>2]|0;
     $277 = ($276|0)==(0);
     if ($277) {
      $278 = HEAP32[$pDbPage>>2]|0;
      $279 = (_sqlite3PagerGetData($278)|0);
      HEAP32[$aPayload>>2] = $279;
      $280 = HEAP32[$aPayload>>2]|0;
      $281 = (_sqlite3Get4byte($280)|0);
      HEAP32[$nextPage>>2] = $281;
      $282 = HEAP32[$2>>2]|0;
      $283 = (($282) + 4)|0;
      $284 = HEAP32[$aPayload>>2]|0;
      $285 = (($284) + ($283)|0);
      $286 = HEAP32[$4>>2]|0;
      $287 = HEAP32[$a1>>2]|0;
      $288 = HEAP32[$5>>2]|0;
      $289 = $288 & 1;
      $290 = HEAP32[$pDbPage>>2]|0;
      $291 = (_copyPayload($285,$286,$287,$289,$290)|0);
      HEAP32[$rc>>2] = $291;
      $292 = HEAP32[$pDbPage>>2]|0;
      _sqlite3PagerUnref($292);
      HEAP32[$2>>2] = 0;
     }
     $293 = HEAP32[$a1>>2]|0;
     $294 = HEAP32[$3>>2]|0;
     $295 = (($294) - ($293))|0;
     HEAP32[$3>>2] = $295;
     $296 = HEAP32[$a1>>2]|0;
     $297 = HEAP32[$4>>2]|0;
     $298 = (($297) + ($296)|0);
     HEAP32[$4>>2] = $298;
    }
    $299 = HEAP32[$iIdx>>2]|0;
    $300 = (($299) + 1)|0;
    HEAP32[$iIdx>>2] = $300;
   }
  }
 } while(0);
 $301 = HEAP32[$rc>>2]|0;
 $302 = ($301|0)==(0);
 $303 = HEAP32[$3>>2]|0;
 $304 = ($303>>>0)>(0);
 $or$cond7 = $302 & $304;
 if ($or$cond7) {
  $305 = (_sqlite3CorruptError(56431)|0);
  HEAP32[$0>>2] = $305;
  $307 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($307|0);
 } else {
  $306 = HEAP32[$rc>>2]|0;
  HEAP32[$0>>2] = $306;
  $307 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($307|0);
 }
 return 0|0;
}
function _btreeParseCell($pPage,$iCell,$pInfo) {
 $pPage = $pPage|0;
 $iCell = $iCell|0;
 $pInfo = $pInfo|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $1 = sp + 4|0;
 $2 = sp;
 HEAP32[$0>>2] = $pPage;
 HEAP32[$1>>2] = $iCell;
 HEAP32[$2>>2] = $pInfo;
 $3 = HEAP32[$0>>2]|0;
 $4 = HEAP32[$0>>2]|0;
 $5 = (($4) + 56|0);
 $6 = HEAP32[$5>>2]|0;
 $7 = HEAP32[$0>>2]|0;
 $8 = (($7) + 20|0);
 $9 = HEAP16[$8>>1]|0;
 $10 = $9&65535;
 $11 = HEAP32[$1>>2]|0;
 $12 = $11<<1;
 $13 = HEAP32[$0>>2]|0;
 $14 = (($13) + 64|0);
 $15 = HEAP32[$14>>2]|0;
 $16 = (($15) + ($12)|0);
 $17 = HEAP8[$16>>0]|0;
 $18 = $17&255;
 $19 = $18 << 8;
 $20 = HEAP32[$1>>2]|0;
 $21 = $20<<1;
 $22 = HEAP32[$0>>2]|0;
 $23 = (($22) + 64|0);
 $24 = HEAP32[$23>>2]|0;
 $25 = (($24) + ($21)|0);
 $26 = (($25) + 1|0);
 $27 = HEAP8[$26>>0]|0;
 $28 = $27&255;
 $29 = $19 | $28;
 $30 = $10 & $29;
 $31 = (($6) + ($30)|0);
 $32 = HEAP32[$2>>2]|0;
 _btreeParseCellPtr($3,$31,$32);
 STACKTOP = sp;return;
}
function _copyPayload($pPayload,$pBuf,$nByte,$eOp,$pDbPage) {
 $pPayload = $pPayload|0;
 $pBuf = $pBuf|0;
 $nByte = $nByte|0;
 $eOp = $eOp|0;
 $pDbPage = $pDbPage|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0;
 var $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 24|0;
 $1 = sp + 20|0;
 $2 = sp + 16|0;
 $3 = sp + 12|0;
 $4 = sp + 8|0;
 $5 = sp + 4|0;
 $rc = sp;
 HEAP32[$1>>2] = $pPayload;
 HEAP32[$2>>2] = $pBuf;
 HEAP32[$3>>2] = $nByte;
 HEAP32[$4>>2] = $eOp;
 HEAP32[$5>>2] = $pDbPage;
 $6 = HEAP32[$4>>2]|0;
 $7 = ($6|0)!=(0);
 do {
  if ($7) {
   $8 = HEAP32[$5>>2]|0;
   $9 = (_sqlite3PagerWrite($8)|0);
   HEAP32[$rc>>2] = $9;
   $10 = HEAP32[$rc>>2]|0;
   $11 = ($10|0)!=(0);
   if (!($11)) {
    $13 = HEAP32[$1>>2]|0;
    $14 = HEAP32[$2>>2]|0;
    $15 = HEAP32[$3>>2]|0;
    _memcpy(($13|0),($14|0),($15|0))|0;
    break;
   }
   $12 = HEAP32[$rc>>2]|0;
   HEAP32[$0>>2] = $12;
   $19 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($19|0);
  } else {
   $16 = HEAP32[$2>>2]|0;
   $17 = HEAP32[$1>>2]|0;
   $18 = HEAP32[$3>>2]|0;
   _memcpy(($16|0),($17|0),($18|0))|0;
  }
 } while(0);
 HEAP32[$0>>2] = 0;
 $19 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($19|0);
}
function _getOverflowPage($pBt,$ovfl,$ppPage,$pPgnoNext) {
 $pBt = $pBt|0;
 $ovfl = $ovfl|0;
 $ppPage = $ppPage|0;
 $pPgnoNext = $pPgnoNext|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $7 = 0, $8 = 0, $9 = 0, $eType = 0, $iGuess = 0, $next = 0, $pPage = 0, $pgno = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $0 = sp + 32|0;
 $1 = sp + 28|0;
 $2 = sp + 24|0;
 $3 = sp + 20|0;
 $next = sp + 16|0;
 $pPage = sp + 12|0;
 $rc = sp + 8|0;
 $pgno = sp + 4|0;
 $iGuess = sp;
 $eType = sp + 36|0;
 HEAP32[$0>>2] = $pBt;
 HEAP32[$1>>2] = $ovfl;
 HEAP32[$2>>2] = $ppPage;
 HEAP32[$3>>2] = $pPgnoNext;
 HEAP32[$next>>2] = 0;
 HEAP32[$pPage>>2] = 0;
 HEAP32[$rc>>2] = 0;
 $4 = HEAP32[$0>>2]|0;
 $5 = (($4) + 17|0);
 $6 = HEAP8[$5>>0]|0;
 $7 = ($6<<24>>24)!=(0);
 if ($7) {
  $8 = HEAP32[$1>>2]|0;
  $9 = (($8) + 1)|0;
  HEAP32[$iGuess>>2] = $9;
  while(1) {
   $10 = HEAP32[$0>>2]|0;
   $11 = HEAP32[$iGuess>>2]|0;
   $12 = (_ptrmapPageno($10,$11)|0);
   $13 = HEAP32[$iGuess>>2]|0;
   $14 = ($12|0)==($13|0);
   if ($14) {
    $64 = 1;
   } else {
    $15 = HEAP32[$iGuess>>2]|0;
    $16 = HEAP32[1128>>2]|0;
    $17 = HEAP32[$0>>2]|0;
    $18 = (($17) + 32|0);
    $19 = HEAP32[$18>>2]|0;
    $20 = (($16>>>0) / ($19>>>0))&-1;
    $21 = (($20) + 1)|0;
    $22 = ($15|0)==($21|0);
    $64 = $22;
   }
   $23 = HEAP32[$iGuess>>2]|0;
   if (!($64)) {
    break;
   }
   $24 = (($23) + 1)|0;
   HEAP32[$iGuess>>2] = $24;
  }
  $25 = HEAP32[$0>>2]|0;
  $26 = (_btreePagecount($25)|0);
  $27 = ($23>>>0)<=($26>>>0);
  if ($27) {
   $28 = HEAP32[$0>>2]|0;
   $29 = HEAP32[$iGuess>>2]|0;
   $30 = (_ptrmapGet($28,$29,$eType,$pgno)|0);
   HEAP32[$rc>>2] = $30;
   $31 = HEAP32[$rc>>2]|0;
   $32 = ($31|0)==(0);
   if ($32) {
    $33 = HEAP8[$eType>>0]|0;
    $34 = $33&255;
    $35 = ($34|0)==(4);
    if ($35) {
     $36 = HEAP32[$pgno>>2]|0;
     $37 = HEAP32[$1>>2]|0;
     $38 = ($36|0)==($37|0);
     if ($38) {
      $39 = HEAP32[$iGuess>>2]|0;
      HEAP32[$next>>2] = $39;
      HEAP32[$rc>>2] = 101;
     }
    }
   }
  }
 }
 $40 = HEAP32[$rc>>2]|0;
 $41 = ($40|0)==(0);
 if ($41) {
  $42 = HEAP32[$0>>2]|0;
  $43 = HEAP32[$1>>2]|0;
  $44 = HEAP32[$2>>2]|0;
  $45 = ($44|0)==(0|0);
  $46 = $45 ? 2 : 0;
  $47 = (_btreeGetPage($42,$43,$pPage,$46)|0);
  HEAP32[$rc>>2] = $47;
  $48 = HEAP32[$rc>>2]|0;
  $49 = ($48|0)==(0);
  if ($49) {
   $50 = HEAP32[$pPage>>2]|0;
   $51 = (($50) + 56|0);
   $52 = HEAP32[$51>>2]|0;
   $53 = (_sqlite3Get4byte($52)|0);
   HEAP32[$next>>2] = $53;
  }
 }
 $54 = HEAP32[$next>>2]|0;
 $55 = HEAP32[$3>>2]|0;
 HEAP32[$55>>2] = $54;
 $56 = HEAP32[$2>>2]|0;
 $57 = ($56|0)!=(0|0);
 $58 = HEAP32[$pPage>>2]|0;
 if ($57) {
  $59 = HEAP32[$2>>2]|0;
  HEAP32[$59>>2] = $58;
  $60 = HEAP32[$rc>>2]|0;
  $61 = ($60|0)==(101);
  $62 = HEAP32[$rc>>2]|0;
  $63 = $61 ? 0 : $62;
  STACKTOP = sp;return ($63|0);
 } else {
  _releasePage($58);
  $60 = HEAP32[$rc>>2]|0;
  $61 = ($60|0)==(101);
  $62 = HEAP32[$rc>>2]|0;
  $63 = $61 ? 0 : $62;
  STACKTOP = sp;return ($63|0);
 }
 return 0|0;
}
function _ptrmapPageno($pBt,$pgno) {
 $pBt = $pBt|0;
 $pgno = $pgno|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $iPtrMap = 0, $nPagesPerMapPage = 0, $ret = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 20|0;
 $1 = sp + 16|0;
 $2 = sp + 12|0;
 $nPagesPerMapPage = sp + 8|0;
 $iPtrMap = sp + 4|0;
 $ret = sp;
 HEAP32[$1>>2] = $pBt;
 HEAP32[$2>>2] = $pgno;
 $3 = HEAP32[$2>>2]|0;
 $4 = ($3>>>0)<(2);
 if ($4) {
  HEAP32[$0>>2] = 0;
  $29 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($29|0);
 }
 $5 = HEAP32[$1>>2]|0;
 $6 = (($5) + 36|0);
 $7 = HEAP32[$6>>2]|0;
 $8 = (($7>>>0) / 5)&-1;
 $9 = (($8) + 1)|0;
 HEAP32[$nPagesPerMapPage>>2] = $9;
 $10 = HEAP32[$2>>2]|0;
 $11 = (($10) - 2)|0;
 $12 = HEAP32[$nPagesPerMapPage>>2]|0;
 $13 = (($11>>>0) / ($12>>>0))&-1;
 HEAP32[$iPtrMap>>2] = $13;
 $14 = HEAP32[$iPtrMap>>2]|0;
 $15 = HEAP32[$nPagesPerMapPage>>2]|0;
 $16 = Math_imul($14, $15)|0;
 $17 = (($16) + 2)|0;
 HEAP32[$ret>>2] = $17;
 $18 = HEAP32[$ret>>2]|0;
 $19 = HEAP32[1128>>2]|0;
 $20 = HEAP32[$1>>2]|0;
 $21 = (($20) + 32|0);
 $22 = HEAP32[$21>>2]|0;
 $23 = (($19>>>0) / ($22>>>0))&-1;
 $24 = (($23) + 1)|0;
 $25 = ($18|0)==($24|0);
 if ($25) {
  $26 = HEAP32[$ret>>2]|0;
  $27 = (($26) + 1)|0;
  HEAP32[$ret>>2] = $27;
 }
 $28 = HEAP32[$ret>>2]|0;
 HEAP32[$0>>2] = $28;
 $29 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($29|0);
}
function _ptrmapGet($pBt,$key,$pEType,$pPgno) {
 $pBt = $pBt|0;
 $key = $key|0;
 $pEType = $pEType|0;
 $pPgno = $pPgno|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $iPtrmap = 0, $offset = 0, $pDbPage = 0, $pPtrmap = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $0 = sp + 36|0;
 $1 = sp + 32|0;
 $2 = sp + 28|0;
 $3 = sp + 24|0;
 $4 = sp + 20|0;
 $pDbPage = sp + 16|0;
 $iPtrmap = sp + 12|0;
 $pPtrmap = sp + 8|0;
 $offset = sp + 4|0;
 $rc = sp;
 HEAP32[$1>>2] = $pBt;
 HEAP32[$2>>2] = $key;
 HEAP32[$3>>2] = $pEType;
 HEAP32[$4>>2] = $pPgno;
 $5 = HEAP32[$1>>2]|0;
 $6 = HEAP32[$2>>2]|0;
 $7 = (_ptrmapPageno($5,$6)|0);
 HEAP32[$iPtrmap>>2] = $7;
 $8 = HEAP32[$1>>2]|0;
 $9 = HEAP32[$8>>2]|0;
 $10 = HEAP32[$iPtrmap>>2]|0;
 $11 = (_sqlite3PagerAcquire($9,$10,$pDbPage,0)|0);
 HEAP32[$rc>>2] = $11;
 $12 = HEAP32[$rc>>2]|0;
 $13 = ($12|0)!=(0);
 if ($13) {
  $14 = HEAP32[$rc>>2]|0;
  HEAP32[$0>>2] = $14;
  $49 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($49|0);
 }
 $15 = HEAP32[$pDbPage>>2]|0;
 $16 = (_sqlite3PagerGetData($15)|0);
 HEAP32[$pPtrmap>>2] = $16;
 $17 = HEAP32[$2>>2]|0;
 $18 = HEAP32[$iPtrmap>>2]|0;
 $19 = (($17) - ($18))|0;
 $20 = (($19) - 1)|0;
 $21 = ($20*5)|0;
 HEAP32[$offset>>2] = $21;
 $22 = HEAP32[$offset>>2]|0;
 $23 = ($22|0)<(0);
 if ($23) {
  $24 = HEAP32[$pDbPage>>2]|0;
  _sqlite3PagerUnref($24);
  $25 = (_sqlite3CorruptError(53113)|0);
  HEAP32[$0>>2] = $25;
  $49 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($49|0);
 }
 $26 = HEAP32[$offset>>2]|0;
 $27 = HEAP32[$pPtrmap>>2]|0;
 $28 = (($27) + ($26)|0);
 $29 = HEAP8[$28>>0]|0;
 $30 = HEAP32[$3>>2]|0;
 HEAP8[$30>>0] = $29;
 $31 = HEAP32[$4>>2]|0;
 $32 = ($31|0)!=(0|0);
 if ($32) {
  $33 = HEAP32[$offset>>2]|0;
  $34 = (($33) + 1)|0;
  $35 = HEAP32[$pPtrmap>>2]|0;
  $36 = (($35) + ($34)|0);
  $37 = (_sqlite3Get4byte($36)|0);
  $38 = HEAP32[$4>>2]|0;
  HEAP32[$38>>2] = $37;
 }
 $39 = HEAP32[$pDbPage>>2]|0;
 _sqlite3PagerUnref($39);
 $40 = HEAP32[$3>>2]|0;
 $41 = HEAP8[$40>>0]|0;
 $42 = $41&255;
 $43 = ($42|0)<(1);
 if (!($43)) {
  $44 = HEAP32[$3>>2]|0;
  $45 = HEAP8[$44>>0]|0;
  $46 = $45&255;
  $47 = ($46|0)>(5);
  if (!($47)) {
   HEAP32[$0>>2] = 0;
   $49 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($49|0);
  }
 }
 $48 = (_sqlite3CorruptError(53121)|0);
 HEAP32[$0>>2] = $48;
 $49 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($49|0);
}
function _sqlite3GetVarint32($p,$v) {
 $p = $p|0;
 $v = $v|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $8 = 0, $9 = 0, $a = 0, $b = 0, $n = 0, $v64 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 25|0;
 $1 = sp + 20|0;
 $2 = sp + 16|0;
 $a = sp + 12|0;
 $b = sp + 8|0;
 $v64 = sp;
 $n = sp + 24|0;
 HEAP32[$1>>2] = $p;
 HEAP32[$2>>2] = $v;
 $3 = HEAP32[$1>>2]|0;
 $4 = HEAP8[$3>>0]|0;
 $5 = $4&255;
 HEAP32[$a>>2] = $5;
 $6 = HEAP32[$1>>2]|0;
 $7 = (($6) + 1|0);
 HEAP32[$1>>2] = $7;
 $8 = HEAP32[$1>>2]|0;
 $9 = HEAP8[$8>>0]|0;
 $10 = $9&255;
 HEAP32[$b>>2] = $10;
 $11 = HEAP32[$b>>2]|0;
 $12 = $11 & 128;
 $13 = ($12|0)!=(0);
 if (!($13)) {
  $14 = HEAP32[$a>>2]|0;
  $15 = $14 & 127;
  HEAP32[$a>>2] = $15;
  $16 = HEAP32[$a>>2]|0;
  $17 = $16 << 7;
  HEAP32[$a>>2] = $17;
  $18 = HEAP32[$a>>2]|0;
  $19 = HEAP32[$b>>2]|0;
  $20 = $18 | $19;
  $21 = HEAP32[$2>>2]|0;
  HEAP32[$21>>2] = $20;
  HEAP8[$0>>0] = 2;
  $72 = HEAP8[$0>>0]|0;
  STACKTOP = sp;return ($72|0);
 }
 $22 = HEAP32[$1>>2]|0;
 $23 = (($22) + 1|0);
 HEAP32[$1>>2] = $23;
 $24 = HEAP32[$a>>2]|0;
 $25 = $24 << 14;
 HEAP32[$a>>2] = $25;
 $26 = HEAP32[$1>>2]|0;
 $27 = HEAP8[$26>>0]|0;
 $28 = $27&255;
 $29 = HEAP32[$a>>2]|0;
 $30 = $29 | $28;
 HEAP32[$a>>2] = $30;
 $31 = HEAP32[$a>>2]|0;
 $32 = $31 & 128;
 $33 = ($32|0)!=(0);
 if (!($33)) {
  $34 = HEAP32[$a>>2]|0;
  $35 = $34 & 2080895;
  HEAP32[$a>>2] = $35;
  $36 = HEAP32[$b>>2]|0;
  $37 = $36 & 127;
  HEAP32[$b>>2] = $37;
  $38 = HEAP32[$b>>2]|0;
  $39 = $38 << 7;
  HEAP32[$b>>2] = $39;
  $40 = HEAP32[$a>>2]|0;
  $41 = HEAP32[$b>>2]|0;
  $42 = $40 | $41;
  $43 = HEAP32[$2>>2]|0;
  HEAP32[$43>>2] = $42;
  HEAP8[$0>>0] = 3;
  $72 = HEAP8[$0>>0]|0;
  STACKTOP = sp;return ($72|0);
 }
 $44 = HEAP32[$1>>2]|0;
 $45 = (($44) + -2|0);
 HEAP32[$1>>2] = $45;
 $46 = HEAP32[$1>>2]|0;
 $47 = (_sqlite3GetVarint($46,$v64)|0);
 HEAP8[$n>>0] = $47;
 $48 = $v64;
 $49 = $48;
 $50 = HEAP32[$49>>2]|0;
 $51 = (($48) + 4)|0;
 $52 = $51;
 $53 = HEAP32[$52>>2]|0;
 $54 = $v64;
 $55 = $54;
 $56 = HEAP32[$55>>2]|0;
 $57 = (($54) + 4)|0;
 $58 = $57;
 $59 = HEAP32[$58>>2]|0;
 $60 = ($50|0)!=($56|0);
 $61 = (0)!=($59|0);
 $62 = $60 | $61;
 if ($62) {
  $63 = HEAP32[$2>>2]|0;
  HEAP32[$63>>2] = -1;
 } else {
  $64 = $v64;
  $65 = $64;
  $66 = HEAP32[$65>>2]|0;
  $67 = (($64) + 4)|0;
  $68 = $67;
  $69 = HEAP32[$68>>2]|0;
  $70 = HEAP32[$2>>2]|0;
  HEAP32[$70>>2] = $66;
 }
 $71 = HEAP8[$n>>0]|0;
 HEAP8[$0>>0] = $71;
 $72 = HEAP8[$0>>0]|0;
 STACKTOP = sp;return ($72|0);
}
function _vdbeRecordCompareInt($nKey1,$pKey1,$pPKey2) {
 $nKey1 = $nKey1|0;
 $pKey1 = $pKey1|0;
 $pPKey2 = $pPKey2|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0;
 var $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0, $188 = 0;
 var $189 = 0, $19 = 0, $190 = 0, $191 = 0, $192 = 0, $193 = 0, $194 = 0, $195 = 0, $196 = 0, $197 = 0, $198 = 0, $199 = 0, $2 = 0, $20 = 0, $200 = 0, $201 = 0, $202 = 0, $203 = 0, $204 = 0, $205 = 0;
 var $206 = 0, $207 = 0, $208 = 0, $209 = 0, $21 = 0, $210 = 0, $211 = 0, $212 = 0, $213 = 0, $214 = 0, $215 = 0, $216 = 0, $217 = 0, $218 = 0, $219 = 0, $22 = 0, $220 = 0, $221 = 0, $222 = 0, $223 = 0;
 var $224 = 0, $225 = 0, $226 = 0, $227 = 0, $228 = 0, $229 = 0, $23 = 0, $230 = 0, $231 = 0, $232 = 0, $233 = 0, $234 = 0, $235 = 0, $236 = 0, $237 = 0, $238 = 0, $239 = 0, $24 = 0, $240 = 0, $241 = 0;
 var $242 = 0, $243 = 0, $244 = 0, $245 = 0, $246 = 0, $247 = 0, $248 = 0, $249 = 0, $25 = 0, $250 = 0, $251 = 0, $252 = 0, $253 = 0, $254 = 0, $255 = 0, $256 = 0, $257 = 0, $258 = 0, $259 = 0, $26 = 0;
 var $260 = 0, $261 = 0, $262 = 0, $263 = 0, $264 = 0, $265 = 0, $266 = 0, $267 = 0, $268 = 0, $269 = 0, $27 = 0, $270 = 0, $271 = 0, $272 = 0, $273 = 0, $274 = 0, $275 = 0, $276 = 0, $277 = 0, $278 = 0;
 var $279 = 0, $28 = 0, $280 = 0, $281 = 0, $282 = 0, $283 = 0, $284 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0;
 var $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0;
 var $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0;
 var $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0;
 var $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $aKey = 0, $lhs = 0, $res = 0, $serial_type = 0, $v = 0, $x = 0, $y = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 64|0;
 $0 = sp + 52|0;
 $1 = sp + 48|0;
 $2 = sp + 44|0;
 $3 = sp + 40|0;
 $aKey = sp + 36|0;
 $serial_type = sp + 32|0;
 $res = sp + 28|0;
 $y = sp + 24|0;
 $x = sp + 16|0;
 $v = sp + 8|0;
 $lhs = sp;
 HEAP32[$1>>2] = $nKey1;
 HEAP32[$2>>2] = $pKey1;
 HEAP32[$3>>2] = $pPKey2;
 $4 = HEAP32[$2>>2]|0;
 $5 = HEAP8[$4>>0]|0;
 $6 = $5&255;
 $7 = $6 & 63;
 $8 = HEAP32[$2>>2]|0;
 $9 = (($8) + ($7)|0);
 HEAP32[$aKey>>2] = $9;
 $10 = HEAP32[$2>>2]|0;
 $11 = (($10) + 1|0);
 $12 = HEAP8[$11>>0]|0;
 $13 = $12&255;
 HEAP32[$serial_type>>2] = $13;
 $14 = HEAP32[$3>>2]|0;
 $15 = (($14) + 8|0);
 $16 = HEAP32[$15>>2]|0;
 $17 = $16;
 $18 = $17;
 $19 = HEAP32[$18>>2]|0;
 $20 = (($17) + 4)|0;
 $21 = $20;
 $22 = HEAP32[$21>>2]|0;
 $23 = $v;
 $24 = $23;
 HEAP32[$24>>2] = $19;
 $25 = (($23) + 4)|0;
 $26 = $25;
 HEAP32[$26>>2] = $22;
 $27 = HEAP32[$serial_type>>2]|0;
 do {
  switch ($27|0) {
  case 2:  {
   $37 = HEAP32[$aKey>>2]|0;
   $38 = HEAP8[$37>>0]|0;
   $39 = $38 << 24 >> 24;
   $40 = $39<<8;
   $41 = HEAP32[$aKey>>2]|0;
   $42 = (($41) + 1|0);
   $43 = HEAP8[$42>>0]|0;
   $44 = $43&255;
   $45 = $40 | $44;
   $46 = ($45|0)<(0);
   $47 = $46 << 31 >> 31;
   $48 = $lhs;
   $49 = $48;
   HEAP32[$49>>2] = $45;
   $50 = (($48) + 4)|0;
   $51 = $50;
   HEAP32[$51>>2] = $47;
   break;
  }
  case 3:  {
   $52 = HEAP32[$aKey>>2]|0;
   $53 = HEAP8[$52>>0]|0;
   $54 = $53 << 24 >> 24;
   $55 = $54<<16;
   $56 = HEAP32[$aKey>>2]|0;
   $57 = (($56) + 1|0);
   $58 = HEAP8[$57>>0]|0;
   $59 = $58&255;
   $60 = $59 << 8;
   $61 = $55 | $60;
   $62 = HEAP32[$aKey>>2]|0;
   $63 = (($62) + 2|0);
   $64 = HEAP8[$63>>0]|0;
   $65 = $64&255;
   $66 = $61 | $65;
   $67 = ($66|0)<(0);
   $68 = $67 << 31 >> 31;
   $69 = $lhs;
   $70 = $69;
   HEAP32[$70>>2] = $66;
   $71 = (($69) + 4)|0;
   $72 = $71;
   HEAP32[$72>>2] = $68;
   break;
  }
  case 7: case 0:  {
   $223 = HEAP32[$1>>2]|0;
   $224 = HEAP32[$2>>2]|0;
   $225 = HEAP32[$3>>2]|0;
   $226 = (_sqlite3VdbeRecordCompare($223,$224,$225)|0);
   HEAP32[$0>>2] = $226;
   $284 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($284|0);
   break;
  }
  case 4:  {
   $73 = HEAP32[$aKey>>2]|0;
   $74 = HEAP8[$73>>0]|0;
   $75 = $74&255;
   $76 = $75 << 24;
   $77 = HEAP32[$aKey>>2]|0;
   $78 = (($77) + 1|0);
   $79 = HEAP8[$78>>0]|0;
   $80 = $79&255;
   $81 = $80 << 16;
   $82 = $76 | $81;
   $83 = HEAP32[$aKey>>2]|0;
   $84 = (($83) + 2|0);
   $85 = HEAP8[$84>>0]|0;
   $86 = $85&255;
   $87 = $86 << 8;
   $88 = $82 | $87;
   $89 = HEAP32[$aKey>>2]|0;
   $90 = (($89) + 3|0);
   $91 = HEAP8[$90>>0]|0;
   $92 = $91&255;
   $93 = $88 | $92;
   HEAP32[$y>>2] = $93;
   $94 = HEAP32[$y>>2]|0;
   $95 = ($94|0)<(0);
   $96 = $95 << 31 >> 31;
   $97 = $lhs;
   $98 = $97;
   HEAP32[$98>>2] = $94;
   $99 = (($97) + 4)|0;
   $100 = $99;
   HEAP32[$100>>2] = $96;
   break;
  }
  case 9:  {
   $219 = $lhs;
   $220 = $219;
   HEAP32[$220>>2] = 1;
   $221 = (($219) + 4)|0;
   $222 = $221;
   HEAP32[$222>>2] = 0;
   break;
  }
  case 8:  {
   $215 = $lhs;
   $216 = $215;
   HEAP32[$216>>2] = 0;
   $217 = (($215) + 4)|0;
   $218 = $217;
   HEAP32[$218>>2] = 0;
   break;
  }
  case 1:  {
   $28 = HEAP32[$aKey>>2]|0;
   $29 = HEAP8[$28>>0]|0;
   $30 = $29 << 24 >> 24;
   $31 = ($30|0)<(0);
   $32 = $31 << 31 >> 31;
   $33 = $lhs;
   $34 = $33;
   HEAP32[$34>>2] = $30;
   $35 = (($33) + 4)|0;
   $36 = $35;
   HEAP32[$36>>2] = $32;
   break;
  }
  case 5:  {
   $101 = HEAP32[$aKey>>2]|0;
   $102 = (($101) + 2|0);
   $103 = HEAP8[$102>>0]|0;
   $104 = $103&255;
   $105 = $104 << 24;
   $106 = HEAP32[$aKey>>2]|0;
   $107 = (($106) + 2|0);
   $108 = (($107) + 1|0);
   $109 = HEAP8[$108>>0]|0;
   $110 = $109&255;
   $111 = $110 << 16;
   $112 = $105 | $111;
   $113 = HEAP32[$aKey>>2]|0;
   $114 = (($113) + 2|0);
   $115 = (($114) + 2|0);
   $116 = HEAP8[$115>>0]|0;
   $117 = $116&255;
   $118 = $117 << 8;
   $119 = $112 | $118;
   $120 = HEAP32[$aKey>>2]|0;
   $121 = (($120) + 2|0);
   $122 = (($121) + 3|0);
   $123 = HEAP8[$122>>0]|0;
   $124 = $123&255;
   $125 = $119 | $124;
   $126 = HEAP32[$aKey>>2]|0;
   $127 = HEAP8[$126>>0]|0;
   $128 = $127 << 24 >> 24;
   $129 = $128<<8;
   $130 = HEAP32[$aKey>>2]|0;
   $131 = (($130) + 1|0);
   $132 = HEAP8[$131>>0]|0;
   $133 = $132&255;
   $134 = $129 | $133;
   $135 = ($134|0)<(0);
   $136 = $135 << 31 >> 31;
   $137 = (___muldi3(0,1,($134|0),($136|0))|0);
   $138 = tempRet0;
   $139 = (_i64Add(($125|0),0,($137|0),($138|0))|0);
   $140 = tempRet0;
   $141 = $lhs;
   $142 = $141;
   HEAP32[$142>>2] = $139;
   $143 = (($141) + 4)|0;
   $144 = $143;
   HEAP32[$144>>2] = $140;
   break;
  }
  case 6:  {
   $145 = HEAP32[$aKey>>2]|0;
   $146 = HEAP8[$145>>0]|0;
   $147 = $146&255;
   $148 = $147 << 24;
   $149 = HEAP32[$aKey>>2]|0;
   $150 = (($149) + 1|0);
   $151 = HEAP8[$150>>0]|0;
   $152 = $151&255;
   $153 = $152 << 16;
   $154 = $148 | $153;
   $155 = HEAP32[$aKey>>2]|0;
   $156 = (($155) + 2|0);
   $157 = HEAP8[$156>>0]|0;
   $158 = $157&255;
   $159 = $158 << 8;
   $160 = $154 | $159;
   $161 = HEAP32[$aKey>>2]|0;
   $162 = (($161) + 3|0);
   $163 = HEAP8[$162>>0]|0;
   $164 = $163&255;
   $165 = $160 | $164;
   $166 = $x;
   $167 = $166;
   HEAP32[$167>>2] = $165;
   $168 = (($166) + 4)|0;
   $169 = $168;
   HEAP32[$169>>2] = 0;
   $170 = $x;
   $171 = $170;
   $172 = HEAP32[$171>>2]|0;
   $173 = (($170) + 4)|0;
   $174 = $173;
   $175 = HEAP32[$174>>2]|0;
   $176 = HEAP32[$aKey>>2]|0;
   $177 = (($176) + 4|0);
   $178 = HEAP8[$177>>0]|0;
   $179 = $178&255;
   $180 = $179 << 24;
   $181 = HEAP32[$aKey>>2]|0;
   $182 = (($181) + 4|0);
   $183 = (($182) + 1|0);
   $184 = HEAP8[$183>>0]|0;
   $185 = $184&255;
   $186 = $185 << 16;
   $187 = $180 | $186;
   $188 = HEAP32[$aKey>>2]|0;
   $189 = (($188) + 4|0);
   $190 = (($189) + 2|0);
   $191 = HEAP8[$190>>0]|0;
   $192 = $191&255;
   $193 = $192 << 8;
   $194 = $187 | $193;
   $195 = HEAP32[$aKey>>2]|0;
   $196 = (($195) + 4|0);
   $197 = (($196) + 3|0);
   $198 = HEAP8[$197>>0]|0;
   $199 = $198&255;
   $200 = $194 | $199;
   $201 = $x;
   $202 = $201;
   HEAP32[$202>>2] = $200;
   $203 = (($201) + 4)|0;
   $204 = $203;
   HEAP32[$204>>2] = $172;
   $205 = $x;
   $206 = $205;
   $207 = HEAP32[$206>>2]|0;
   $208 = (($205) + 4)|0;
   $209 = $208;
   $210 = HEAP32[$209>>2]|0;
   $211 = $lhs;
   $212 = $211;
   HEAP32[$212>>2] = $207;
   $213 = (($211) + 4)|0;
   $214 = $213;
   HEAP32[$214>>2] = $210;
   break;
  }
  default: {
   $227 = HEAP32[$1>>2]|0;
   $228 = HEAP32[$2>>2]|0;
   $229 = HEAP32[$3>>2]|0;
   $230 = (_sqlite3VdbeRecordCompare($227,$228,$229)|0);
   HEAP32[$0>>2] = $230;
   $284 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($284|0);
  }
  }
 } while(0);
 $231 = $v;
 $232 = $231;
 $233 = HEAP32[$232>>2]|0;
 $234 = (($231) + 4)|0;
 $235 = $234;
 $236 = HEAP32[$235>>2]|0;
 $237 = $lhs;
 $238 = $237;
 $239 = HEAP32[$238>>2]|0;
 $240 = (($237) + 4)|0;
 $241 = $240;
 $242 = HEAP32[$241>>2]|0;
 $243 = ($236|0)>($242|0);
 $244 = ($233>>>0)>($239>>>0);
 $245 = ($236|0)==($242|0);
 $246 = $245 & $244;
 $247 = $243 | $246;
 do {
  if ($247) {
   $248 = HEAP32[$3>>2]|0;
   $249 = (($248) + 12|0);
   $250 = HEAP32[$249>>2]|0;
   HEAP32[$res>>2] = $250;
  } else {
   $251 = $v;
   $252 = $251;
   $253 = HEAP32[$252>>2]|0;
   $254 = (($251) + 4)|0;
   $255 = $254;
   $256 = HEAP32[$255>>2]|0;
   $257 = $lhs;
   $258 = $257;
   $259 = HEAP32[$258>>2]|0;
   $260 = (($257) + 4)|0;
   $261 = $260;
   $262 = HEAP32[$261>>2]|0;
   $263 = ($256|0)<($262|0);
   $264 = ($253>>>0)<($259>>>0);
   $265 = ($256|0)==($262|0);
   $266 = $265 & $264;
   $267 = $263 | $266;
   $268 = HEAP32[$3>>2]|0;
   if ($267) {
    $269 = (($268) + 16|0);
    $270 = HEAP32[$269>>2]|0;
    HEAP32[$res>>2] = $270;
    break;
   }
   $271 = (($268) + 4|0);
   $272 = HEAP16[$271>>1]|0;
   $273 = $272&65535;
   $274 = ($273|0)>(1);
   if ($274) {
    $275 = HEAP32[$1>>2]|0;
    $276 = HEAP32[$2>>2]|0;
    $277 = HEAP32[$3>>2]|0;
    $278 = (_vdbeRecordCompareWithSkip($275,$276,$277,1)|0);
    HEAP32[$res>>2] = $278;
    break;
   } else {
    $279 = HEAP32[$3>>2]|0;
    $280 = (($279) + 6|0);
    $281 = HEAP8[$280>>0]|0;
    $282 = $281 << 24 >> 24;
    HEAP32[$res>>2] = $282;
    break;
   }
  }
 } while(0);
 $283 = HEAP32[$res>>2]|0;
 HEAP32[$0>>2] = $283;
 $284 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($284|0);
}
function _vdbeRecordCompareString($nKey1,$pKey1,$pPKey2) {
 $nKey1 = $nKey1|0;
 $pKey1 = $pKey1|0;
 $pPKey2 = $pPKey2|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0;
 var $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0;
 var $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0;
 var $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0;
 var $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0;
 var $96 = 0, $97 = 0, $98 = 0, $99 = 0, $aKey1 = 0, $nCmp = 0, $nStr = 0, $res = 0, $serial_type = 0, $szHdr = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $0 = sp + 36|0;
 $1 = sp + 32|0;
 $2 = sp + 28|0;
 $3 = sp + 24|0;
 $aKey1 = sp + 20|0;
 $serial_type = sp + 16|0;
 $res = sp + 12|0;
 $nCmp = sp + 8|0;
 $nStr = sp + 4|0;
 $szHdr = sp;
 HEAP32[$1>>2] = $nKey1;
 HEAP32[$2>>2] = $pKey1;
 HEAP32[$3>>2] = $pPKey2;
 $4 = HEAP32[$2>>2]|0;
 HEAP32[$aKey1>>2] = $4;
 $5 = HEAP32[$aKey1>>2]|0;
 $6 = (($5) + 1|0);
 $7 = HEAP8[$6>>0]|0;
 $8 = $7&255;
 $9 = ($8|0)<(128);
 $10 = HEAP32[$aKey1>>2]|0;
 $11 = (($10) + 1|0);
 if ($9) {
  $12 = HEAP8[$11>>0]|0;
  $13 = $12&255;
  HEAP32[$serial_type>>2] = $13;
 } else {
  (_sqlite3GetVarint32($11,$serial_type)|0);
 }
 $14 = HEAP32[$serial_type>>2]|0;
 $15 = ($14|0)<(12);
 do {
  if ($15) {
   $16 = HEAP32[$3>>2]|0;
   $17 = (($16) + 12|0);
   $18 = HEAP32[$17>>2]|0;
   HEAP32[$res>>2] = $18;
  } else {
   $19 = HEAP32[$serial_type>>2]|0;
   $20 = $19 & 1;
   $21 = ($20|0)!=(0);
   if (!($21)) {
    $22 = HEAP32[$3>>2]|0;
    $23 = (($22) + 16|0);
    $24 = HEAP32[$23>>2]|0;
    HEAP32[$res>>2] = $24;
    break;
   }
   $25 = HEAP32[$aKey1>>2]|0;
   $26 = HEAP8[$25>>0]|0;
   $27 = $26&255;
   HEAP32[$szHdr>>2] = $27;
   $28 = HEAP32[$serial_type>>2]|0;
   $29 = (($28) - 12)|0;
   $30 = (($29|0) / 2)&-1;
   HEAP32[$nStr>>2] = $30;
   $31 = HEAP32[$szHdr>>2]|0;
   $32 = HEAP32[$nStr>>2]|0;
   $33 = (($31) + ($32))|0;
   $34 = HEAP32[$1>>2]|0;
   $35 = ($33|0)>($34|0);
   if ($35) {
    $36 = (_sqlite3CorruptError(67026)|0);
    $37 = $36&255;
    $38 = HEAP32[$3>>2]|0;
    $39 = (($38) + 7|0);
    HEAP8[$39>>0] = $37;
    HEAP32[$0>>2] = 0;
    $103 = HEAP32[$0>>2]|0;
    STACKTOP = sp;return ($103|0);
   }
   $40 = HEAP32[$3>>2]|0;
   $41 = (($40) + 8|0);
   $42 = HEAP32[$41>>2]|0;
   $43 = (($42) + 12|0);
   $44 = HEAP32[$43>>2]|0;
   $45 = HEAP32[$nStr>>2]|0;
   $46 = ($44|0)<($45|0);
   if ($46) {
    $47 = HEAP32[$3>>2]|0;
    $48 = (($47) + 8|0);
    $49 = HEAP32[$48>>2]|0;
    $50 = (($49) + 12|0);
    $51 = HEAP32[$50>>2]|0;
    $53 = $51;
   } else {
    $52 = HEAP32[$nStr>>2]|0;
    $53 = $52;
   }
   HEAP32[$nCmp>>2] = $53;
   $54 = HEAP32[$szHdr>>2]|0;
   $55 = HEAP32[$aKey1>>2]|0;
   $56 = (($55) + ($54)|0);
   $57 = HEAP32[$3>>2]|0;
   $58 = (($57) + 8|0);
   $59 = HEAP32[$58>>2]|0;
   $60 = (($59) + 16|0);
   $61 = HEAP32[$60>>2]|0;
   $62 = HEAP32[$nCmp>>2]|0;
   $63 = (_memcmp($56,$61,$62)|0);
   HEAP32[$res>>2] = $63;
   $64 = HEAP32[$res>>2]|0;
   $65 = ($64|0)==(0);
   if (!($65)) {
    $95 = HEAP32[$res>>2]|0;
    $96 = ($95|0)>(0);
    $97 = HEAP32[$3>>2]|0;
    if ($96) {
     $98 = (($97) + 16|0);
     $99 = HEAP32[$98>>2]|0;
     HEAP32[$res>>2] = $99;
     break;
    } else {
     $100 = (($97) + 12|0);
     $101 = HEAP32[$100>>2]|0;
     HEAP32[$res>>2] = $101;
     break;
    }
   }
   $66 = HEAP32[$nStr>>2]|0;
   $67 = HEAP32[$3>>2]|0;
   $68 = (($67) + 8|0);
   $69 = HEAP32[$68>>2]|0;
   $70 = (($69) + 12|0);
   $71 = HEAP32[$70>>2]|0;
   $72 = (($66) - ($71))|0;
   HEAP32[$res>>2] = $72;
   $73 = HEAP32[$res>>2]|0;
   $74 = ($73|0)==(0);
   if ($74) {
    $75 = HEAP32[$3>>2]|0;
    $76 = (($75) + 4|0);
    $77 = HEAP16[$76>>1]|0;
    $78 = $77&65535;
    $79 = ($78|0)>(1);
    if ($79) {
     $80 = HEAP32[$1>>2]|0;
     $81 = HEAP32[$2>>2]|0;
     $82 = HEAP32[$3>>2]|0;
     $83 = (_vdbeRecordCompareWithSkip($80,$81,$82,1)|0);
     HEAP32[$res>>2] = $83;
     break;
    } else {
     $84 = HEAP32[$3>>2]|0;
     $85 = (($84) + 6|0);
     $86 = HEAP8[$85>>0]|0;
     $87 = $86 << 24 >> 24;
     HEAP32[$res>>2] = $87;
     break;
    }
   } else {
    $88 = HEAP32[$res>>2]|0;
    $89 = ($88|0)>(0);
    $90 = HEAP32[$3>>2]|0;
    if ($89) {
     $91 = (($90) + 16|0);
     $92 = HEAP32[$91>>2]|0;
     HEAP32[$res>>2] = $92;
     break;
    } else {
     $93 = (($90) + 12|0);
     $94 = HEAP32[$93>>2]|0;
     HEAP32[$res>>2] = $94;
     break;
    }
   }
  }
 } while(0);
 $102 = HEAP32[$res>>2]|0;
 HEAP32[$0>>2] = $102;
 $103 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($103|0);
}
function _sqlite3VdbeRecordCompare($nKey1,$pKey1,$pPKey2) {
 $nKey1 = $nKey1|0;
 $pKey1 = $pKey1|0;
 $pPKey2 = $pPKey2|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $1 = sp + 4|0;
 $2 = sp;
 HEAP32[$0>>2] = $nKey1;
 HEAP32[$1>>2] = $pKey1;
 HEAP32[$2>>2] = $pPKey2;
 $3 = HEAP32[$0>>2]|0;
 $4 = HEAP32[$1>>2]|0;
 $5 = HEAP32[$2>>2]|0;
 $6 = (_vdbeRecordCompareWithSkip($3,$4,$5,0)|0);
 STACKTOP = sp;return ($6|0);
}
function _vdbeRecordCompareWithSkip($nKey1,$pKey1,$pPKey2,$bSkip) {
 $nKey1 = $nKey1|0;
 $pKey1 = $pKey1|0;
 $pPKey2 = $pPKey2|0;
 $bSkip = $bSkip|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0.0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0.0, $17 = 0;
 var $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0.0, $177 = 0.0, $178 = 0.0, $179 = 0, $18 = 0, $180 = 0.0, $181 = 0.0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0, $188 = 0;
 var $189 = 0, $19 = 0, $190 = 0, $191 = 0, $192 = 0, $193 = 0, $194 = 0, $195 = 0, $196 = 0, $197 = 0, $198 = 0, $199 = 0, $2 = 0, $20 = 0, $200 = 0, $201 = 0, $202 = 0, $203 = 0, $204 = 0, $205 = 0;
 var $206 = 0, $207 = 0, $208 = 0, $209 = 0, $21 = 0, $210 = 0, $211 = 0, $212 = 0, $213 = 0, $214 = 0, $215 = 0, $216 = 0, $217 = 0, $218 = 0, $219 = 0, $22 = 0, $220 = 0, $221 = 0, $222 = 0, $223 = 0;
 var $224 = 0, $225 = 0, $226 = 0, $227 = 0, $228 = 0, $229 = 0, $23 = 0, $230 = 0, $231 = 0, $232 = 0, $233 = 0, $234 = 0, $235 = 0, $236 = 0, $237 = 0, $238 = 0, $239 = 0, $24 = 0, $240 = 0, $241 = 0;
 var $242 = 0, $243 = 0, $244 = 0, $245 = 0, $246 = 0, $247 = 0, $248 = 0, $249 = 0, $25 = 0, $250 = 0, $251 = 0, $252 = 0, $253 = 0, $254 = 0, $255 = 0, $256 = 0, $257 = 0, $258 = 0, $259 = 0, $26 = 0;
 var $260 = 0, $261 = 0, $262 = 0, $263 = 0, $264 = 0, $265 = 0, $266 = 0, $267 = 0, $268 = 0, $269 = 0, $27 = 0, $270 = 0, $271 = 0, $272 = 0, $273 = 0, $274 = 0, $275 = 0, $276 = 0, $277 = 0, $278 = 0;
 var $279 = 0, $28 = 0, $280 = 0, $281 = 0, $282 = 0, $283 = 0, $284 = 0, $285 = 0, $286 = 0, $287 = 0, $288 = 0, $289 = 0, $29 = 0, $290 = 0, $291 = 0, $292 = 0, $293 = 0, $294 = 0, $295 = 0, $296 = 0;
 var $297 = 0, $298 = 0, $299 = 0, $3 = 0, $30 = 0, $300 = 0, $301 = 0, $302 = 0, $303 = 0, $304 = 0, $305 = 0, $306 = 0, $307 = 0, $308 = 0, $309 = 0, $31 = 0, $310 = 0, $311 = 0, $312 = 0, $313 = 0;
 var $314 = 0, $315 = 0, $316 = 0, $317 = 0, $318 = 0, $319 = 0, $32 = 0, $320 = 0, $321 = 0, $322 = 0, $323 = 0, $324 = 0, $325 = 0, $326 = 0, $327 = 0, $328 = 0, $329 = 0, $33 = 0, $330 = 0, $331 = 0;
 var $332 = 0, $333 = 0, $334 = 0, $335 = 0, $336 = 0, $337 = 0, $338 = 0, $339 = 0, $34 = 0, $340 = 0, $341 = 0, $342 = 0, $343 = 0, $344 = 0, $345 = 0, $346 = 0, $347 = 0, $348 = 0, $349 = 0, $35 = 0;
 var $350 = 0, $351 = 0, $352 = 0, $353 = 0, $354 = 0, $355 = 0, $356 = 0, $357 = 0, $358 = 0, $359 = 0, $36 = 0, $360 = 0, $361 = 0, $362 = 0, $363 = 0, $364 = 0, $365 = 0, $366 = 0, $367 = 0, $368 = 0;
 var $369 = 0, $37 = 0, $370 = 0, $371 = 0, $372 = 0, $373 = 0, $374 = 0, $375 = 0, $376 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0;
 var $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0;
 var $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0.0, $81 = 0, $82 = 0, $83 = 0;
 var $84 = 0, $85 = 0.0, $86 = 0.0, $87 = 0, $88 = 0.0, $89 = 0.0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $aKey1 = 0, $d1 = 0, $i = 0;
 var $idx1 = 0, $lhs = 0, $lhs3 = 0, $mem1 = 0, $nCmp = 0, $nCmp4 = 0, $nStr = 0, $pKeyInfo = 0, $pRhs = 0, $rc = 0, $rhs = 0, $rhs1 = 0, $rhs2 = 0, $s1 = 0, $serial_type = 0, $szHdr1 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 160|0;
 $0 = sp + 148|0;
 $1 = sp + 144|0;
 $2 = sp + 140|0;
 $3 = sp + 136|0;
 $4 = sp + 132|0;
 $d1 = sp + 128|0;
 $i = sp + 124|0;
 $szHdr1 = sp + 120|0;
 $idx1 = sp + 116|0;
 $rc = sp + 112|0;
 $pRhs = sp + 108|0;
 $pKeyInfo = sp + 104|0;
 $aKey1 = sp + 100|0;
 $mem1 = sp + 40|0;
 $s1 = sp + 96|0;
 $serial_type = sp + 92|0;
 $rhs = sp + 32|0;
 $lhs = sp + 24|0;
 $rhs1 = sp + 16|0;
 $rhs2 = sp + 8|0;
 $lhs3 = sp;
 $nCmp = sp + 88|0;
 $nStr = sp + 84|0;
 $nCmp4 = sp + 80|0;
 HEAP32[$1>>2] = $nKey1;
 HEAP32[$2>>2] = $pKey1;
 HEAP32[$3>>2] = $pPKey2;
 HEAP32[$4>>2] = $bSkip;
 HEAP32[$rc>>2] = 0;
 $5 = HEAP32[$3>>2]|0;
 $6 = (($5) + 8|0);
 $7 = HEAP32[$6>>2]|0;
 HEAP32[$pRhs>>2] = $7;
 $8 = HEAP32[$3>>2]|0;
 $9 = HEAP32[$8>>2]|0;
 HEAP32[$pKeyInfo>>2] = $9;
 $10 = HEAP32[$2>>2]|0;
 HEAP32[$aKey1>>2] = $10;
 $11 = HEAP32[$4>>2]|0;
 $12 = ($11|0)!=(0);
 $13 = HEAP32[$aKey1>>2]|0;
 do {
  if ($12) {
   $14 = (($13) + 1|0);
   $15 = HEAP8[$14>>0]|0;
   $16 = $15&255;
   $17 = ($16|0)<(128);
   $18 = HEAP32[$aKey1>>2]|0;
   $19 = (($18) + 1|0);
   if ($17) {
    $20 = HEAP8[$19>>0]|0;
    $21 = $20&255;
    HEAP32[$s1>>2] = $21;
    $25 = 1;
   } else {
    $22 = (_sqlite3GetVarint32($19,$s1)|0);
    $23 = $22&255;
    $25 = $23;
   }
   $24 = $25&255;
   $26 = $24&255;
   $27 = (1 + ($26))|0;
   HEAP32[$idx1>>2] = $27;
   $28 = HEAP32[$aKey1>>2]|0;
   $29 = HEAP8[$28>>0]|0;
   $30 = $29&255;
   HEAP32[$szHdr1>>2] = $30;
   $31 = HEAP32[$szHdr1>>2]|0;
   $32 = HEAP32[$s1>>2]|0;
   $33 = (_sqlite3VdbeSerialTypeLen($32)|0);
   $34 = (($31) + ($33))|0;
   HEAP32[$d1>>2] = $34;
   HEAP32[$i>>2] = 1;
   $35 = HEAP32[$pRhs>>2]|0;
   $36 = (($35) + 40|0);
   HEAP32[$pRhs>>2] = $36;
  } else {
   $37 = HEAP8[$13>>0]|0;
   $38 = $37&255;
   $39 = ($38|0)<(128);
   $40 = HEAP32[$aKey1>>2]|0;
   if ($39) {
    $41 = HEAP8[$40>>0]|0;
    $42 = $41&255;
    HEAP32[$szHdr1>>2] = $42;
    $46 = 1;
   } else {
    $43 = (_sqlite3GetVarint32($40,$szHdr1)|0);
    $44 = $43&255;
    $46 = $44;
   }
   $45 = $46&255;
   $47 = $45&255;
   HEAP32[$idx1>>2] = $47;
   $48 = HEAP32[$szHdr1>>2]|0;
   HEAP32[$d1>>2] = $48;
   $49 = HEAP32[$d1>>2]|0;
   $50 = HEAP32[$1>>2]|0;
   $51 = ($49>>>0)>($50>>>0);
   if (!($51)) {
    HEAP32[$i>>2] = 0;
    break;
   }
   $52 = (_sqlite3CorruptError(66745)|0);
   $53 = $52&255;
   $54 = HEAP32[$3>>2]|0;
   $55 = (($54) + 7|0);
   HEAP8[$55>>0] = $53;
   HEAP32[$0>>2] = 0;
   $376 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($376|0);
  }
 } while(0);
 L16: while(1) {
  $56 = HEAP32[$pRhs>>2]|0;
  $57 = (($56) + 8|0);
  $58 = HEAP16[$57>>1]|0;
  $59 = $58&65535;
  $60 = $59 & 4;
  $61 = ($60|0)!=(0);
  do {
   if ($61) {
    $62 = HEAP32[$idx1>>2]|0;
    $63 = HEAP32[$aKey1>>2]|0;
    $64 = (($63) + ($62)|0);
    $65 = HEAP8[$64>>0]|0;
    $66 = $65&255;
    HEAP32[$serial_type>>2] = $66;
    $67 = HEAP32[$serial_type>>2]|0;
    $68 = ($67>>>0)>=(12);
    if ($68) {
     HEAP32[$rc>>2] = 1;
     break;
    }
    $69 = HEAP32[$serial_type>>2]|0;
    $70 = ($69|0)==(0);
    if ($70) {
     HEAP32[$rc>>2] = -1;
     break;
    }
    $71 = HEAP32[$serial_type>>2]|0;
    $72 = ($71|0)==(7);
    if ($72) {
     $73 = HEAP32[$pRhs>>2]|0;
     $74 = $73;
     $75 = $74;
     $76 = HEAP32[$75>>2]|0;
     $77 = (($74) + 4)|0;
     $78 = $77;
     $79 = HEAP32[$78>>2]|0;
     $80 = (+($76>>>0)) + (4294967296.0*(+($79|0)));
     HEAPF64[$rhs>>3] = $80;
     $81 = HEAP32[$d1>>2]|0;
     $82 = HEAP32[$aKey1>>2]|0;
     $83 = (($82) + ($81)|0);
     $84 = HEAP32[$serial_type>>2]|0;
     (_sqlite3VdbeSerialGet($83,$84,$mem1)|0);
     $85 = +HEAPF64[$mem1>>3];
     $86 = +HEAPF64[$rhs>>3];
     $87 = $85 < $86;
     if ($87) {
      HEAP32[$rc>>2] = -1;
      break;
     }
     $88 = +HEAPF64[$mem1>>3];
     $89 = +HEAPF64[$rhs>>3];
     $90 = $88 > $89;
     if (!($90)) {
      break;
     }
     HEAP32[$rc>>2] = 1;
     break;
    } else {
     $91 = HEAP32[$serial_type>>2]|0;
     $92 = HEAP32[$d1>>2]|0;
     $93 = HEAP32[$aKey1>>2]|0;
     $94 = (($93) + ($92)|0);
     $95 = (_vdbeRecordDecodeInt($91,$94)|0);
     $96 = tempRet0;
     $97 = $lhs;
     $98 = $97;
     HEAP32[$98>>2] = $95;
     $99 = (($97) + 4)|0;
     $100 = $99;
     HEAP32[$100>>2] = $96;
     $101 = HEAP32[$pRhs>>2]|0;
     $102 = $101;
     $103 = $102;
     $104 = HEAP32[$103>>2]|0;
     $105 = (($102) + 4)|0;
     $106 = $105;
     $107 = HEAP32[$106>>2]|0;
     $108 = $rhs1;
     $109 = $108;
     HEAP32[$109>>2] = $104;
     $110 = (($108) + 4)|0;
     $111 = $110;
     HEAP32[$111>>2] = $107;
     $112 = $lhs;
     $113 = $112;
     $114 = HEAP32[$113>>2]|0;
     $115 = (($112) + 4)|0;
     $116 = $115;
     $117 = HEAP32[$116>>2]|0;
     $118 = $rhs1;
     $119 = $118;
     $120 = HEAP32[$119>>2]|0;
     $121 = (($118) + 4)|0;
     $122 = $121;
     $123 = HEAP32[$122>>2]|0;
     $124 = ($117|0)<($123|0);
     $125 = ($114>>>0)<($120>>>0);
     $126 = ($117|0)==($123|0);
     $127 = $126 & $125;
     $128 = $124 | $127;
     if ($128) {
      HEAP32[$rc>>2] = -1;
      break;
     }
     $129 = $lhs;
     $130 = $129;
     $131 = HEAP32[$130>>2]|0;
     $132 = (($129) + 4)|0;
     $133 = $132;
     $134 = HEAP32[$133>>2]|0;
     $135 = $rhs1;
     $136 = $135;
     $137 = HEAP32[$136>>2]|0;
     $138 = (($135) + 4)|0;
     $139 = $138;
     $140 = HEAP32[$139>>2]|0;
     $141 = ($134|0)>($140|0);
     $142 = ($131>>>0)>($137>>>0);
     $143 = ($134|0)==($140|0);
     $144 = $143 & $142;
     $145 = $141 | $144;
     if (!($145)) {
      break;
     }
     HEAP32[$rc>>2] = 1;
     break;
    }
   } else {
    $146 = HEAP32[$pRhs>>2]|0;
    $147 = (($146) + 8|0);
    $148 = HEAP16[$147>>1]|0;
    $149 = $148&65535;
    $150 = $149 & 8;
    $151 = ($150|0)!=(0);
    if ($151) {
     $152 = HEAP32[$idx1>>2]|0;
     $153 = HEAP32[$aKey1>>2]|0;
     $154 = (($153) + ($152)|0);
     $155 = HEAP8[$154>>0]|0;
     $156 = $155&255;
     HEAP32[$serial_type>>2] = $156;
     $157 = HEAP32[$serial_type>>2]|0;
     $158 = ($157>>>0)>=(12);
     if ($158) {
      HEAP32[$rc>>2] = 1;
      break;
     }
     $159 = HEAP32[$serial_type>>2]|0;
     $160 = ($159|0)==(0);
     if ($160) {
      HEAP32[$rc>>2] = -1;
      break;
     }
     $161 = HEAP32[$pRhs>>2]|0;
     $162 = +HEAPF64[$161>>3];
     HEAPF64[$rhs2>>3] = $162;
     $163 = HEAP32[$d1>>2]|0;
     $164 = HEAP32[$aKey1>>2]|0;
     $165 = (($164) + ($163)|0);
     $166 = HEAP32[$serial_type>>2]|0;
     (_sqlite3VdbeSerialGet($165,$166,$mem1)|0);
     $167 = HEAP32[$serial_type>>2]|0;
     $168 = ($167|0)==(7);
     if ($168) {
      $169 = +HEAPF64[$mem1>>3];
      HEAPF64[$lhs3>>3] = $169;
     } else {
      $170 = $mem1;
      $171 = $170;
      $172 = HEAP32[$171>>2]|0;
      $173 = (($170) + 4)|0;
      $174 = $173;
      $175 = HEAP32[$174>>2]|0;
      $176 = (+($172>>>0)) + (4294967296.0*(+($175|0)));
      HEAPF64[$lhs3>>3] = $176;
     }
     $177 = +HEAPF64[$lhs3>>3];
     $178 = +HEAPF64[$rhs2>>3];
     $179 = $177 < $178;
     if ($179) {
      HEAP32[$rc>>2] = -1;
      break;
     }
     $180 = +HEAPF64[$lhs3>>3];
     $181 = +HEAPF64[$rhs2>>3];
     $182 = $180 > $181;
     if (!($182)) {
      break;
     }
     HEAP32[$rc>>2] = 1;
     break;
    }
    $183 = HEAP32[$pRhs>>2]|0;
    $184 = (($183) + 8|0);
    $185 = HEAP16[$184>>1]|0;
    $186 = $185&65535;
    $187 = $186 & 2;
    $188 = ($187|0)!=(0);
    if (!($188)) {
     $275 = HEAP32[$pRhs>>2]|0;
     $276 = (($275) + 8|0);
     $277 = HEAP16[$276>>1]|0;
     $278 = $277&65535;
     $279 = $278 & 16;
     $280 = ($279|0)!=(0);
     $281 = HEAP32[$idx1>>2]|0;
     $282 = HEAP32[$aKey1>>2]|0;
     $283 = (($282) + ($281)|0);
     $284 = HEAP8[$283>>0]|0;
     $285 = $284&255;
     if (!($280)) {
      HEAP32[$serial_type>>2] = $285;
      $334 = HEAP32[$serial_type>>2]|0;
      $335 = ($334|0)!=(0);
      $336 = $335&1;
      HEAP32[$rc>>2] = $336;
      break;
     }
     $286 = ($285|0)<(128);
     $287 = HEAP32[$idx1>>2]|0;
     $288 = HEAP32[$aKey1>>2]|0;
     $289 = (($288) + ($287)|0);
     if ($286) {
      $290 = HEAP8[$289>>0]|0;
      $291 = $290&255;
      HEAP32[$serial_type>>2] = $291;
     } else {
      (_sqlite3GetVarint32($289,$serial_type)|0);
     }
     $292 = HEAP32[$serial_type>>2]|0;
     $293 = ($292>>>0)<(12);
     if (!($293)) {
      $294 = HEAP32[$serial_type>>2]|0;
      $295 = $294 & 1;
      $296 = ($295|0)!=(0);
      if (!($296)) {
       $297 = HEAP32[$serial_type>>2]|0;
       $298 = (($297) - 12)|0;
       $299 = (($298>>>0) / 2)&-1;
       HEAP32[$nStr>>2] = $299;
       $300 = HEAP32[$d1>>2]|0;
       $301 = HEAP32[$nStr>>2]|0;
       $302 = (($300) + ($301))|0;
       $303 = HEAP32[$1>>2]|0;
       $304 = ($302>>>0)>($303>>>0);
       if ($304) {
        label = 63;
        break L16;
       }
       $309 = HEAP32[$nStr>>2]|0;
       $310 = HEAP32[$pRhs>>2]|0;
       $311 = (($310) + 12|0);
       $312 = HEAP32[$311>>2]|0;
       $313 = ($309|0)<($312|0);
       if ($313) {
        $314 = HEAP32[$nStr>>2]|0;
        $318 = $314;
       } else {
        $315 = HEAP32[$pRhs>>2]|0;
        $316 = (($315) + 12|0);
        $317 = HEAP32[$316>>2]|0;
        $318 = $317;
       }
       HEAP32[$nCmp4>>2] = $318;
       $319 = HEAP32[$d1>>2]|0;
       $320 = HEAP32[$aKey1>>2]|0;
       $321 = (($320) + ($319)|0);
       $322 = HEAP32[$pRhs>>2]|0;
       $323 = (($322) + 16|0);
       $324 = HEAP32[$323>>2]|0;
       $325 = HEAP32[$nCmp4>>2]|0;
       $326 = (_memcmp($321,$324,$325)|0);
       HEAP32[$rc>>2] = $326;
       $327 = HEAP32[$rc>>2]|0;
       $328 = ($327|0)==(0);
       if (!($328)) {
        break;
       }
       $329 = HEAP32[$nStr>>2]|0;
       $330 = HEAP32[$pRhs>>2]|0;
       $331 = (($330) + 12|0);
       $332 = HEAP32[$331>>2]|0;
       $333 = (($329) - ($332))|0;
       HEAP32[$rc>>2] = $333;
       break;
      }
     }
     HEAP32[$rc>>2] = -1;
     break;
    }
    $189 = HEAP32[$idx1>>2]|0;
    $190 = HEAP32[$aKey1>>2]|0;
    $191 = (($190) + ($189)|0);
    $192 = HEAP8[$191>>0]|0;
    $193 = $192&255;
    $194 = ($193|0)<(128);
    $195 = HEAP32[$idx1>>2]|0;
    $196 = HEAP32[$aKey1>>2]|0;
    $197 = (($196) + ($195)|0);
    if ($194) {
     $198 = HEAP8[$197>>0]|0;
     $199 = $198&255;
     HEAP32[$serial_type>>2] = $199;
    } else {
     (_sqlite3GetVarint32($197,$serial_type)|0);
    }
    $200 = HEAP32[$serial_type>>2]|0;
    $201 = ($200>>>0)<(12);
    if ($201) {
     HEAP32[$rc>>2] = -1;
     break;
    }
    $202 = HEAP32[$serial_type>>2]|0;
    $203 = $202 & 1;
    $204 = ($203|0)!=(0);
    if (!($204)) {
     HEAP32[$rc>>2] = 1;
     break;
    }
    $205 = HEAP32[$serial_type>>2]|0;
    $206 = (($205) - 12)|0;
    $207 = (($206>>>0) / 2)&-1;
    $208 = (($mem1) + 12|0);
    HEAP32[$208>>2] = $207;
    $209 = HEAP32[$d1>>2]|0;
    $210 = (($mem1) + 12|0);
    $211 = HEAP32[$210>>2]|0;
    $212 = (($209) + ($211))|0;
    $213 = HEAP32[$1>>2]|0;
    $214 = ($212>>>0)>($213>>>0);
    if ($214) {
     label = 47;
     break L16;
    }
    $219 = HEAP32[$i>>2]|0;
    $220 = HEAP32[$pKeyInfo>>2]|0;
    $221 = (($220) + 20|0);
    $222 = (($221) + ($219<<2)|0);
    $223 = HEAP32[$222>>2]|0;
    $224 = ($223|0)!=(0|0);
    if ($224) {
     $225 = HEAP32[$pKeyInfo>>2]|0;
     $226 = (($225) + 4|0);
     $227 = HEAP8[$226>>0]|0;
     $228 = (($mem1) + 10|0);
     HEAP8[$228>>0] = $227;
     $229 = HEAP32[$pKeyInfo>>2]|0;
     $230 = (($229) + 12|0);
     $231 = HEAP32[$230>>2]|0;
     $232 = (($mem1) + 32|0);
     HEAP32[$232>>2] = $231;
     $233 = (($mem1) + 8|0);
     HEAP16[$233>>1] = 2;
     $234 = HEAP32[$d1>>2]|0;
     $235 = HEAP32[$aKey1>>2]|0;
     $236 = (($235) + ($234)|0);
     $237 = (($mem1) + 16|0);
     HEAP32[$237>>2] = $236;
     $238 = HEAP32[$pRhs>>2]|0;
     $239 = HEAP32[$i>>2]|0;
     $240 = HEAP32[$pKeyInfo>>2]|0;
     $241 = (($240) + 20|0);
     $242 = (($241) + ($239<<2)|0);
     $243 = HEAP32[$242>>2]|0;
     $244 = HEAP32[$3>>2]|0;
     $245 = (($244) + 7|0);
     $246 = (_vdbeCompareMemString($mem1,$238,$243,$245)|0);
     HEAP32[$rc>>2] = $246;
     break;
    }
    $247 = (($mem1) + 12|0);
    $248 = HEAP32[$247>>2]|0;
    $249 = HEAP32[$pRhs>>2]|0;
    $250 = (($249) + 12|0);
    $251 = HEAP32[$250>>2]|0;
    $252 = ($248|0)<($251|0);
    if ($252) {
     $253 = (($mem1) + 12|0);
     $254 = HEAP32[$253>>2]|0;
     $258 = $254;
    } else {
     $255 = HEAP32[$pRhs>>2]|0;
     $256 = (($255) + 12|0);
     $257 = HEAP32[$256>>2]|0;
     $258 = $257;
    }
    HEAP32[$nCmp>>2] = $258;
    $259 = HEAP32[$d1>>2]|0;
    $260 = HEAP32[$aKey1>>2]|0;
    $261 = (($260) + ($259)|0);
    $262 = HEAP32[$pRhs>>2]|0;
    $263 = (($262) + 16|0);
    $264 = HEAP32[$263>>2]|0;
    $265 = HEAP32[$nCmp>>2]|0;
    $266 = (_memcmp($261,$264,$265)|0);
    HEAP32[$rc>>2] = $266;
    $267 = HEAP32[$rc>>2]|0;
    $268 = ($267|0)==(0);
    if ($268) {
     $269 = (($mem1) + 12|0);
     $270 = HEAP32[$269>>2]|0;
     $271 = HEAP32[$pRhs>>2]|0;
     $272 = (($271) + 12|0);
     $273 = HEAP32[$272>>2]|0;
     $274 = (($270) - ($273))|0;
     HEAP32[$rc>>2] = $274;
    }
   }
  } while(0);
  $337 = HEAP32[$rc>>2]|0;
  $338 = ($337|0)!=(0);
  $339 = HEAP32[$i>>2]|0;
  if ($338) {
   label = 71;
   break;
  }
  $349 = (($339) + 1)|0;
  HEAP32[$i>>2] = $349;
  $350 = HEAP32[$pRhs>>2]|0;
  $351 = (($350) + 40|0);
  HEAP32[$pRhs>>2] = $351;
  $352 = HEAP32[$serial_type>>2]|0;
  $353 = (_sqlite3VdbeSerialTypeLen($352)|0);
  $354 = HEAP32[$d1>>2]|0;
  $355 = (($354) + ($353))|0;
  HEAP32[$d1>>2] = $355;
  $356 = HEAP32[$serial_type>>2]|0;
  $357 = (_sqlite3VarintLen($356,0)|0);
  $358 = HEAP32[$idx1>>2]|0;
  $359 = (($358) + ($357))|0;
  HEAP32[$idx1>>2] = $359;
  $360 = HEAP32[$idx1>>2]|0;
  $361 = HEAP32[$szHdr1>>2]|0;
  $362 = ($360>>>0)<($361>>>0);
  if (!($362)) {
   label = 77;
   break;
  }
  $363 = HEAP32[$i>>2]|0;
  $364 = HEAP32[$3>>2]|0;
  $365 = (($364) + 4|0);
  $366 = HEAP16[$365>>1]|0;
  $367 = $366&65535;
  $368 = ($363|0)<($367|0);
  if (!($368)) {
   label = 77;
   break;
  }
  $369 = HEAP32[$d1>>2]|0;
  $370 = HEAP32[$1>>2]|0;
  $371 = ($369>>>0)<=($370>>>0);
  if (!($371)) {
   label = 77;
   break;
  }
 }
 if ((label|0) == 47) {
  $215 = (_sqlite3CorruptError(66824)|0);
  $216 = $215&255;
  $217 = HEAP32[$3>>2]|0;
  $218 = (($217) + 7|0);
  HEAP8[$218>>0] = $216;
  HEAP32[$0>>2] = 0;
  $376 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($376|0);
 }
 else if ((label|0) == 63) {
  $305 = (_sqlite3CorruptError(66853)|0);
  $306 = $305&255;
  $307 = HEAP32[$3>>2]|0;
  $308 = (($307) + 7|0);
  HEAP8[$308>>0] = $306;
  HEAP32[$0>>2] = 0;
  $376 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($376|0);
 }
 else if ((label|0) == 71) {
  $340 = HEAP32[$pKeyInfo>>2]|0;
  $341 = (($340) + 16|0);
  $342 = HEAP32[$341>>2]|0;
  $343 = (($342) + ($339)|0);
  $344 = HEAP8[$343>>0]|0;
  $345 = ($344<<24>>24)!=(0);
  if ($345) {
   $346 = HEAP32[$rc>>2]|0;
   $347 = (0 - ($346))|0;
   HEAP32[$rc>>2] = $347;
  }
  $348 = HEAP32[$rc>>2]|0;
  HEAP32[$0>>2] = $348;
  $376 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($376|0);
 }
 else if ((label|0) == 77) {
  $372 = HEAP32[$3>>2]|0;
  $373 = (($372) + 6|0);
  $374 = HEAP8[$373>>0]|0;
  $375 = $374 << 24 >> 24;
  HEAP32[$0>>2] = $375;
  $376 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($376|0);
 }
 return 0|0;
}
function _sqlite3VdbeSerialTypeLen($serial_type) {
 $serial_type = $serial_type|0;
 var $0 = 0, $1 = 0, $10 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $1 = sp;
 HEAP32[$1>>2] = $serial_type;
 $2 = HEAP32[$1>>2]|0;
 $3 = ($2>>>0)>=(12);
 $4 = HEAP32[$1>>2]|0;
 if ($3) {
  $5 = (($4) - 12)|0;
  $6 = (($5>>>0) / 2)&-1;
  HEAP32[$0>>2] = $6;
  $10 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($10|0);
 } else {
  $7 = (24360 + ($4)|0);
  $8 = HEAP8[$7>>0]|0;
  $9 = $8&255;
  HEAP32[$0>>2] = $9;
  $10 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($10|0);
 }
 return 0|0;
}
function _sqlite3VdbeSerialGet($buf,$serial_type,$pMem) {
 $buf = $buf|0;
 $serial_type = $serial_type|0;
 $pMem = $pMem|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $17 = 0, $18 = 0;
 var $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0;
 var $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0;
 var $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0;
 var $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0;
 var $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $1 = sp + 8|0;
 $2 = sp + 4|0;
 $3 = sp;
 HEAP32[$1>>2] = $buf;
 HEAP32[$2>>2] = $serial_type;
 HEAP32[$3>>2] = $pMem;
 $4 = HEAP32[$2>>2]|0;
 switch ($4|0) {
 case 4:  {
  $61 = HEAP32[$1>>2]|0;
  $62 = HEAP8[$61>>0]|0;
  $63 = $62 << 24 >> 24;
  $64 = $63<<24;
  $65 = HEAP32[$1>>2]|0;
  $66 = (($65) + 1|0);
  $67 = HEAP8[$66>>0]|0;
  $68 = $67&255;
  $69 = $68 << 16;
  $70 = $64 | $69;
  $71 = HEAP32[$1>>2]|0;
  $72 = (($71) + 2|0);
  $73 = HEAP8[$72>>0]|0;
  $74 = $73&255;
  $75 = $74 << 8;
  $76 = $70 | $75;
  $77 = HEAP32[$1>>2]|0;
  $78 = (($77) + 3|0);
  $79 = HEAP8[$78>>0]|0;
  $80 = $79&255;
  $81 = $76 | $80;
  $82 = ($81|0)<(0);
  $83 = $82 << 31 >> 31;
  $84 = HEAP32[$3>>2]|0;
  $85 = $84;
  $86 = $85;
  HEAP32[$86>>2] = $81;
  $87 = (($85) + 4)|0;
  $88 = $87;
  HEAP32[$88>>2] = $83;
  $89 = HEAP32[$3>>2]|0;
  $90 = (($89) + 8|0);
  HEAP16[$90>>1] = 4;
  HEAP32[$0>>2] = 4;
  $168 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($168|0);
  break;
 }
 case 0: case 11: case 10:  {
  $5 = HEAP32[$3>>2]|0;
  $6 = (($5) + 8|0);
  HEAP16[$6>>1] = 1;
  HEAP32[$0>>2] = 0;
  $168 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($168|0);
  break;
 }
 case 9: case 8:  {
  $142 = HEAP32[$2>>2]|0;
  $143 = (($142) - 8)|0;
  $144 = HEAP32[$3>>2]|0;
  $145 = $144;
  $146 = $145;
  HEAP32[$146>>2] = $143;
  $147 = (($145) + 4)|0;
  $148 = $147;
  HEAP32[$148>>2] = 0;
  $149 = HEAP32[$3>>2]|0;
  $150 = (($149) + 8|0);
  HEAP16[$150>>1] = 4;
  HEAP32[$0>>2] = 0;
  $168 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($168|0);
  break;
 }
 case 3:  {
  $37 = HEAP32[$1>>2]|0;
  $38 = HEAP8[$37>>0]|0;
  $39 = $38 << 24 >> 24;
  $40 = $39<<16;
  $41 = HEAP32[$1>>2]|0;
  $42 = (($41) + 1|0);
  $43 = HEAP8[$42>>0]|0;
  $44 = $43&255;
  $45 = $44 << 8;
  $46 = $40 | $45;
  $47 = HEAP32[$1>>2]|0;
  $48 = (($47) + 2|0);
  $49 = HEAP8[$48>>0]|0;
  $50 = $49&255;
  $51 = $46 | $50;
  $52 = ($51|0)<(0);
  $53 = $52 << 31 >> 31;
  $54 = HEAP32[$3>>2]|0;
  $55 = $54;
  $56 = $55;
  HEAP32[$56>>2] = $51;
  $57 = (($55) + 4)|0;
  $58 = $57;
  HEAP32[$58>>2] = $53;
  $59 = HEAP32[$3>>2]|0;
  $60 = (($59) + 8|0);
  HEAP16[$60>>1] = 4;
  HEAP32[$0>>2] = 3;
  $168 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($168|0);
  break;
 }
 case 7: case 6:  {
  $138 = HEAP32[$1>>2]|0;
  $139 = HEAP32[$2>>2]|0;
  $140 = HEAP32[$3>>2]|0;
  $141 = (_serialGet($138,$139,$140)|0);
  HEAP32[$0>>2] = $141;
  $168 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($168|0);
  break;
 }
 case 5:  {
  $91 = HEAP32[$1>>2]|0;
  $92 = (($91) + 2|0);
  $93 = HEAP8[$92>>0]|0;
  $94 = $93&255;
  $95 = $94 << 24;
  $96 = HEAP32[$1>>2]|0;
  $97 = (($96) + 2|0);
  $98 = (($97) + 1|0);
  $99 = HEAP8[$98>>0]|0;
  $100 = $99&255;
  $101 = $100 << 16;
  $102 = $95 | $101;
  $103 = HEAP32[$1>>2]|0;
  $104 = (($103) + 2|0);
  $105 = (($104) + 2|0);
  $106 = HEAP8[$105>>0]|0;
  $107 = $106&255;
  $108 = $107 << 8;
  $109 = $102 | $108;
  $110 = HEAP32[$1>>2]|0;
  $111 = (($110) + 2|0);
  $112 = (($111) + 3|0);
  $113 = HEAP8[$112>>0]|0;
  $114 = $113&255;
  $115 = $109 | $114;
  $116 = HEAP32[$1>>2]|0;
  $117 = HEAP8[$116>>0]|0;
  $118 = $117 << 24 >> 24;
  $119 = $118<<8;
  $120 = HEAP32[$1>>2]|0;
  $121 = (($120) + 1|0);
  $122 = HEAP8[$121>>0]|0;
  $123 = $122&255;
  $124 = $119 | $123;
  $125 = ($124|0)<(0);
  $126 = $125 << 31 >> 31;
  $127 = (___muldi3(0,1,($124|0),($126|0))|0);
  $128 = tempRet0;
  $129 = (_i64Add(($115|0),0,($127|0),($128|0))|0);
  $130 = tempRet0;
  $131 = HEAP32[$3>>2]|0;
  $132 = $131;
  $133 = $132;
  HEAP32[$133>>2] = $129;
  $134 = (($132) + 4)|0;
  $135 = $134;
  HEAP32[$135>>2] = $130;
  $136 = HEAP32[$3>>2]|0;
  $137 = (($136) + 8|0);
  HEAP16[$137>>1] = 4;
  HEAP32[$0>>2] = 6;
  $168 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($168|0);
  break;
 }
 case 2:  {
  $19 = HEAP32[$1>>2]|0;
  $20 = HEAP8[$19>>0]|0;
  $21 = $20 << 24 >> 24;
  $22 = $21<<8;
  $23 = HEAP32[$1>>2]|0;
  $24 = (($23) + 1|0);
  $25 = HEAP8[$24>>0]|0;
  $26 = $25&255;
  $27 = $22 | $26;
  $28 = ($27|0)<(0);
  $29 = $28 << 31 >> 31;
  $30 = HEAP32[$3>>2]|0;
  $31 = $30;
  $32 = $31;
  HEAP32[$32>>2] = $27;
  $33 = (($31) + 4)|0;
  $34 = $33;
  HEAP32[$34>>2] = $29;
  $35 = HEAP32[$3>>2]|0;
  $36 = (($35) + 8|0);
  HEAP16[$36>>1] = 4;
  HEAP32[$0>>2] = 2;
  $168 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($168|0);
  break;
 }
 case 1:  {
  $7 = HEAP32[$1>>2]|0;
  $8 = HEAP8[$7>>0]|0;
  $9 = $8 << 24 >> 24;
  $10 = ($9|0)<(0);
  $11 = $10 << 31 >> 31;
  $12 = HEAP32[$3>>2]|0;
  $13 = $12;
  $14 = $13;
  HEAP32[$14>>2] = $9;
  $15 = (($13) + 4)|0;
  $16 = $15;
  HEAP32[$16>>2] = $11;
  $17 = HEAP32[$3>>2]|0;
  $18 = (($17) + 8|0);
  HEAP16[$18>>1] = 4;
  HEAP32[$0>>2] = 1;
  $168 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($168|0);
  break;
 }
 default: {
  $151 = HEAP32[$1>>2]|0;
  $152 = HEAP32[$3>>2]|0;
  $153 = (($152) + 16|0);
  HEAP32[$153>>2] = $151;
  $154 = HEAP32[$2>>2]|0;
  $155 = (($154) - 12)|0;
  $156 = (($155>>>0) / 2)&-1;
  $157 = HEAP32[$3>>2]|0;
  $158 = (($157) + 12|0);
  HEAP32[$158>>2] = $156;
  $159 = HEAP32[$2>>2]|0;
  $160 = $159 & 1;
  $161 = (24352 + ($160<<1)|0);
  $162 = HEAP16[$161>>1]|0;
  $163 = HEAP32[$3>>2]|0;
  $164 = (($163) + 8|0);
  HEAP16[$164>>1] = $162;
  $165 = HEAP32[$3>>2]|0;
  $166 = (($165) + 12|0);
  $167 = HEAP32[$166>>2]|0;
  HEAP32[$0>>2] = $167;
  $168 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($168|0);
 }
 }
 return 0|0;
}
function _vdbeRecordDecodeInt($serial_type,$aKey) {
 $serial_type = $serial_type|0;
 $aKey = $aKey|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0;
 var $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0, $188 = 0;
 var $189 = 0, $19 = 0, $190 = 0, $191 = 0, $192 = 0, $193 = 0, $194 = 0, $195 = 0, $196 = 0, $197 = 0, $198 = 0, $199 = 0, $2 = 0, $20 = 0, $200 = 0, $201 = 0, $202 = 0, $21 = 0, $22 = 0, $23 = 0;
 var $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0;
 var $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0;
 var $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0;
 var $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0;
 var $97 = 0, $98 = 0, $99 = 0, $x = 0, $y = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 8|0;
 $1 = sp + 24|0;
 $2 = sp + 20|0;
 $y = sp + 16|0;
 $x = sp;
 HEAP32[$1>>2] = $serial_type;
 HEAP32[$2>>2] = $aKey;
 $3 = HEAP32[$1>>2]|0;
 switch ($3|0) {
 case 4:  {
  $49 = HEAP32[$2>>2]|0;
  $50 = HEAP8[$49>>0]|0;
  $51 = $50&255;
  $52 = $51 << 24;
  $53 = HEAP32[$2>>2]|0;
  $54 = (($53) + 1|0);
  $55 = HEAP8[$54>>0]|0;
  $56 = $55&255;
  $57 = $56 << 16;
  $58 = $52 | $57;
  $59 = HEAP32[$2>>2]|0;
  $60 = (($59) + 2|0);
  $61 = HEAP8[$60>>0]|0;
  $62 = $61&255;
  $63 = $62 << 8;
  $64 = $58 | $63;
  $65 = HEAP32[$2>>2]|0;
  $66 = (($65) + 3|0);
  $67 = HEAP8[$66>>0]|0;
  $68 = $67&255;
  $69 = $64 | $68;
  HEAP32[$y>>2] = $69;
  $70 = HEAP32[$y>>2]|0;
  $71 = ($70|0)<(0);
  $72 = $71 << 31 >> 31;
  $73 = $0;
  $74 = $73;
  HEAP32[$74>>2] = $70;
  $75 = (($73) + 4)|0;
  $76 = $75;
  HEAP32[$76>>2] = $72;
  break;
 }
 case 3:  {
  $28 = HEAP32[$2>>2]|0;
  $29 = HEAP8[$28>>0]|0;
  $30 = $29 << 24 >> 24;
  $31 = $30<<16;
  $32 = HEAP32[$2>>2]|0;
  $33 = (($32) + 1|0);
  $34 = HEAP8[$33>>0]|0;
  $35 = $34&255;
  $36 = $35 << 8;
  $37 = $31 | $36;
  $38 = HEAP32[$2>>2]|0;
  $39 = (($38) + 2|0);
  $40 = HEAP8[$39>>0]|0;
  $41 = $40&255;
  $42 = $37 | $41;
  $43 = ($42|0)<(0);
  $44 = $43 << 31 >> 31;
  $45 = $0;
  $46 = $45;
  HEAP32[$46>>2] = $42;
  $47 = (($45) + 4)|0;
  $48 = $47;
  HEAP32[$48>>2] = $44;
  break;
 }
 case 2:  {
  $13 = HEAP32[$2>>2]|0;
  $14 = HEAP8[$13>>0]|0;
  $15 = $14 << 24 >> 24;
  $16 = $15<<8;
  $17 = HEAP32[$2>>2]|0;
  $18 = (($17) + 1|0);
  $19 = HEAP8[$18>>0]|0;
  $20 = $19&255;
  $21 = $16 | $20;
  $22 = ($21|0)<(0);
  $23 = $22 << 31 >> 31;
  $24 = $0;
  $25 = $24;
  HEAP32[$25>>2] = $21;
  $26 = (($24) + 4)|0;
  $27 = $26;
  HEAP32[$27>>2] = $23;
  break;
 }
 case 6:  {
  $121 = HEAP32[$2>>2]|0;
  $122 = HEAP8[$121>>0]|0;
  $123 = $122&255;
  $124 = $123 << 24;
  $125 = HEAP32[$2>>2]|0;
  $126 = (($125) + 1|0);
  $127 = HEAP8[$126>>0]|0;
  $128 = $127&255;
  $129 = $128 << 16;
  $130 = $124 | $129;
  $131 = HEAP32[$2>>2]|0;
  $132 = (($131) + 2|0);
  $133 = HEAP8[$132>>0]|0;
  $134 = $133&255;
  $135 = $134 << 8;
  $136 = $130 | $135;
  $137 = HEAP32[$2>>2]|0;
  $138 = (($137) + 3|0);
  $139 = HEAP8[$138>>0]|0;
  $140 = $139&255;
  $141 = $136 | $140;
  $142 = $x;
  $143 = $142;
  HEAP32[$143>>2] = $141;
  $144 = (($142) + 4)|0;
  $145 = $144;
  HEAP32[$145>>2] = 0;
  $146 = $x;
  $147 = $146;
  $148 = HEAP32[$147>>2]|0;
  $149 = (($146) + 4)|0;
  $150 = $149;
  $151 = HEAP32[$150>>2]|0;
  $152 = HEAP32[$2>>2]|0;
  $153 = (($152) + 4|0);
  $154 = HEAP8[$153>>0]|0;
  $155 = $154&255;
  $156 = $155 << 24;
  $157 = HEAP32[$2>>2]|0;
  $158 = (($157) + 4|0);
  $159 = (($158) + 1|0);
  $160 = HEAP8[$159>>0]|0;
  $161 = $160&255;
  $162 = $161 << 16;
  $163 = $156 | $162;
  $164 = HEAP32[$2>>2]|0;
  $165 = (($164) + 4|0);
  $166 = (($165) + 2|0);
  $167 = HEAP8[$166>>0]|0;
  $168 = $167&255;
  $169 = $168 << 8;
  $170 = $163 | $169;
  $171 = HEAP32[$2>>2]|0;
  $172 = (($171) + 4|0);
  $173 = (($172) + 3|0);
  $174 = HEAP8[$173>>0]|0;
  $175 = $174&255;
  $176 = $170 | $175;
  $177 = $x;
  $178 = $177;
  HEAP32[$178>>2] = $176;
  $179 = (($177) + 4)|0;
  $180 = $179;
  HEAP32[$180>>2] = $148;
  $181 = $x;
  $182 = $181;
  $183 = HEAP32[$182>>2]|0;
  $184 = (($181) + 4)|0;
  $185 = $184;
  $186 = HEAP32[$185>>2]|0;
  $187 = $0;
  $188 = $187;
  HEAP32[$188>>2] = $183;
  $189 = (($187) + 4)|0;
  $190 = $189;
  HEAP32[$190>>2] = $186;
  break;
 }
 case 1: case 0:  {
  $4 = HEAP32[$2>>2]|0;
  $5 = HEAP8[$4>>0]|0;
  $6 = $5 << 24 >> 24;
  $7 = ($6|0)<(0);
  $8 = $7 << 31 >> 31;
  $9 = $0;
  $10 = $9;
  HEAP32[$10>>2] = $6;
  $11 = (($9) + 4)|0;
  $12 = $11;
  HEAP32[$12>>2] = $8;
  break;
 }
 case 5:  {
  $77 = HEAP32[$2>>2]|0;
  $78 = (($77) + 2|0);
  $79 = HEAP8[$78>>0]|0;
  $80 = $79&255;
  $81 = $80 << 24;
  $82 = HEAP32[$2>>2]|0;
  $83 = (($82) + 2|0);
  $84 = (($83) + 1|0);
  $85 = HEAP8[$84>>0]|0;
  $86 = $85&255;
  $87 = $86 << 16;
  $88 = $81 | $87;
  $89 = HEAP32[$2>>2]|0;
  $90 = (($89) + 2|0);
  $91 = (($90) + 2|0);
  $92 = HEAP8[$91>>0]|0;
  $93 = $92&255;
  $94 = $93 << 8;
  $95 = $88 | $94;
  $96 = HEAP32[$2>>2]|0;
  $97 = (($96) + 2|0);
  $98 = (($97) + 3|0);
  $99 = HEAP8[$98>>0]|0;
  $100 = $99&255;
  $101 = $95 | $100;
  $102 = HEAP32[$2>>2]|0;
  $103 = HEAP8[$102>>0]|0;
  $104 = $103 << 24 >> 24;
  $105 = $104<<8;
  $106 = HEAP32[$2>>2]|0;
  $107 = (($106) + 1|0);
  $108 = HEAP8[$107>>0]|0;
  $109 = $108&255;
  $110 = $105 | $109;
  $111 = ($110|0)<(0);
  $112 = $111 << 31 >> 31;
  $113 = (___muldi3(0,1,($110|0),($112|0))|0);
  $114 = tempRet0;
  $115 = (_i64Add(($101|0),0,($113|0),($114|0))|0);
  $116 = tempRet0;
  $117 = $0;
  $118 = $117;
  HEAP32[$118>>2] = $115;
  $119 = (($117) + 4)|0;
  $120 = $119;
  HEAP32[$120>>2] = $116;
  break;
 }
 default: {
  $191 = HEAP32[$1>>2]|0;
  $192 = (($191) - 8)|0;
  $193 = $0;
  $194 = $193;
  HEAP32[$194>>2] = $192;
  $195 = (($193) + 4)|0;
  $196 = $195;
  HEAP32[$196>>2] = 0;
 }
 }
 $197 = $0;
 $198 = $197;
 $199 = HEAP32[$198>>2]|0;
 $200 = (($197) + 4)|0;
 $201 = $200;
 $202 = HEAP32[$201>>2]|0;
 tempRet0 = $202;
 STACKTOP = sp;return ($199|0);
}
function _sqlite3VarintLen($0,$1) {
 $0 = $0|0;
 $1 = $1|0;
 var $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0;
 var $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $i = 0, $or$cond = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $2 = sp;
 $i = sp + 8|0;
 $3 = $2;
 $4 = $3;
 HEAP32[$4>>2] = $0;
 $5 = (($3) + 4)|0;
 $6 = $5;
 HEAP32[$6>>2] = $1;
 HEAP32[$i>>2] = 0;
 while(1) {
  $7 = HEAP32[$i>>2]|0;
  $8 = (($7) + 1)|0;
  HEAP32[$i>>2] = $8;
  $9 = $2;
  $10 = $9;
  $11 = HEAP32[$10>>2]|0;
  $12 = (($9) + 4)|0;
  $13 = $12;
  $14 = HEAP32[$13>>2]|0;
  $15 = (_bitshift64Lshr(($11|0),($14|0),7)|0);
  $16 = tempRet0;
  $17 = $2;
  $18 = $17;
  HEAP32[$18>>2] = $15;
  $19 = (($17) + 4)|0;
  $20 = $19;
  HEAP32[$20>>2] = $16;
  $21 = $2;
  $22 = $21;
  $23 = HEAP32[$22>>2]|0;
  $24 = (($21) + 4)|0;
  $25 = $24;
  $26 = HEAP32[$25>>2]|0;
  $27 = ($23|0)!=(0);
  $28 = ($26|0)!=(0);
  $29 = $27 | $28;
  $30 = HEAP32[$i>>2]|0;
  $31 = ($30|0)<(9);
  $or$cond = $29 & $31;
  if (!($or$cond)) {
   break;
  }
 }
 $32 = HEAP32[$i>>2]|0;
 STACKTOP = sp;return ($32|0);
}
function _serialGet($buf,$serial_type,$pMem) {
 $buf = $buf|0;
 $serial_type = $serial_type|0;
 $pMem = $pMem|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0;
 var $81 = 0, $82 = 0, $83 = 0.0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $x = 0, $y = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 20|0;
 $1 = sp + 16|0;
 $2 = sp + 12|0;
 $x = sp;
 $y = sp + 8|0;
 HEAP32[$0>>2] = $buf;
 HEAP32[$1>>2] = $serial_type;
 HEAP32[$2>>2] = $pMem;
 $3 = HEAP32[$0>>2]|0;
 $4 = HEAP8[$3>>0]|0;
 $5 = $4&255;
 $6 = $5 << 24;
 $7 = HEAP32[$0>>2]|0;
 $8 = (($7) + 1|0);
 $9 = HEAP8[$8>>0]|0;
 $10 = $9&255;
 $11 = $10 << 16;
 $12 = $6 | $11;
 $13 = HEAP32[$0>>2]|0;
 $14 = (($13) + 2|0);
 $15 = HEAP8[$14>>0]|0;
 $16 = $15&255;
 $17 = $16 << 8;
 $18 = $12 | $17;
 $19 = HEAP32[$0>>2]|0;
 $20 = (($19) + 3|0);
 $21 = HEAP8[$20>>0]|0;
 $22 = $21&255;
 $23 = $18 | $22;
 $24 = $x;
 $25 = $24;
 HEAP32[$25>>2] = $23;
 $26 = (($24) + 4)|0;
 $27 = $26;
 HEAP32[$27>>2] = 0;
 $28 = HEAP32[$0>>2]|0;
 $29 = (($28) + 4|0);
 $30 = HEAP8[$29>>0]|0;
 $31 = $30&255;
 $32 = $31 << 24;
 $33 = HEAP32[$0>>2]|0;
 $34 = (($33) + 4|0);
 $35 = (($34) + 1|0);
 $36 = HEAP8[$35>>0]|0;
 $37 = $36&255;
 $38 = $37 << 16;
 $39 = $32 | $38;
 $40 = HEAP32[$0>>2]|0;
 $41 = (($40) + 4|0);
 $42 = (($41) + 2|0);
 $43 = HEAP8[$42>>0]|0;
 $44 = $43&255;
 $45 = $44 << 8;
 $46 = $39 | $45;
 $47 = HEAP32[$0>>2]|0;
 $48 = (($47) + 4|0);
 $49 = (($48) + 3|0);
 $50 = HEAP8[$49>>0]|0;
 $51 = $50&255;
 $52 = $46 | $51;
 HEAP32[$y>>2] = $52;
 $53 = $x;
 $54 = $53;
 $55 = HEAP32[$54>>2]|0;
 $56 = (($53) + 4)|0;
 $57 = $56;
 $58 = HEAP32[$57>>2]|0;
 $59 = HEAP32[$y>>2]|0;
 $60 = (_i64Add(0,($55|0),($59|0),0)|0);
 $61 = tempRet0;
 $62 = $x;
 $63 = $62;
 HEAP32[$63>>2] = $60;
 $64 = (($62) + 4)|0;
 $65 = $64;
 HEAP32[$65>>2] = $61;
 $66 = HEAP32[$1>>2]|0;
 $67 = ($66|0)==(6);
 if ($67) {
  $68 = $x;
  $69 = $68;
  $70 = HEAP32[$69>>2]|0;
  $71 = (($68) + 4)|0;
  $72 = $71;
  $73 = HEAP32[$72>>2]|0;
  $74 = HEAP32[$2>>2]|0;
  $75 = $74;
  $76 = $75;
  HEAP32[$76>>2] = $70;
  $77 = (($75) + 4)|0;
  $78 = $77;
  HEAP32[$78>>2] = $73;
  $79 = HEAP32[$2>>2]|0;
  $80 = (($79) + 8|0);
  HEAP16[$80>>1] = 4;
  STACKTOP = sp;return 8;
 } else {
  $81 = HEAP32[$2>>2]|0;
  ;HEAP32[$81+0>>2]=HEAP32[$x+0>>2]|0;HEAP32[$81+4>>2]=HEAP32[$x+4>>2]|0;
  $82 = HEAP32[$2>>2]|0;
  $83 = +HEAPF64[$82>>3];
  $84 = (_sqlite3IsNaN($83)|0);
  $85 = ($84|0)!=(0);
  $86 = $85 ? 1 : 8;
  $87 = $86&65535;
  $88 = HEAP32[$2>>2]|0;
  $89 = (($88) + 8|0);
  HEAP16[$89>>1] = $87;
  STACKTOP = sp;return 8;
 }
 return 0|0;
}
function _sqlite3IsNaN($x) {
 $x = +$x;
 var $0 = 0, $1 = 0.0, $2 = 0.0, $3 = 0.0, $4 = 0.0, $5 = 0, $6 = 0, $7 = 0, $rc = 0, $y = 0, $z = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 16|0;
 $rc = sp + 24|0;
 $y = sp + 8|0;
 $z = sp;
 HEAPF64[$0>>3] = $x;
 $1 = +HEAPF64[$0>>3];
 HEAPF64[$y>>3] = $1;
 $2 = +HEAPF64[$y>>3];
 HEAPF64[$z>>3] = $2;
 $3 = +HEAPF64[$y>>3];
 $4 = +HEAPF64[$z>>3];
 $5 = $3 != $4;
 $6 = $5&1;
 HEAP32[$rc>>2] = $6;
 $7 = HEAP32[$rc>>2]|0;
 STACKTOP = sp;return ($7|0);
}
function _btreeNext($pCur,$pRes) {
 $pCur = $pCur|0;
 $pRes = $pRes|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0;
 var $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0;
 var $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0;
 var $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0;
 var $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0;
 var $97 = 0, $98 = 0, $99 = 0, $idx = 0, $pPage = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 20|0;
 $1 = sp + 16|0;
 $2 = sp + 12|0;
 $rc = sp + 8|0;
 $idx = sp + 4|0;
 $pPage = sp;
 HEAP32[$1>>2] = $pCur;
 HEAP32[$2>>2] = $pRes;
 $3 = HEAP32[$1>>2]|0;
 $4 = (($3) + 73|0);
 $5 = HEAP8[$4>>0]|0;
 $6 = $5&255;
 $7 = ($6|0)!=(1);
 if ($7) {
  $8 = HEAP32[$1>>2]|0;
  $9 = (($8) + 73|0);
  $10 = HEAP8[$9>>0]|0;
  $11 = $10&255;
  $12 = ($11|0)>=(3);
  if ($12) {
   $13 = HEAP32[$1>>2]|0;
   $14 = (_btreeRestoreCursorPosition($13)|0);
   $15 = $14;
  } else {
   $15 = 0;
  }
  HEAP32[$rc>>2] = $15;
  $16 = HEAP32[$rc>>2]|0;
  $17 = ($16|0)!=(0);
  if ($17) {
   $18 = HEAP32[$rc>>2]|0;
   HEAP32[$0>>2] = $18;
   $122 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($122|0);
  }
  $19 = HEAP32[$1>>2]|0;
  $20 = (($19) + 73|0);
  $21 = HEAP8[$20>>0]|0;
  $22 = $21&255;
  $23 = (0)==($22|0);
  if ($23) {
   $24 = HEAP32[$2>>2]|0;
   HEAP32[$24>>2] = 1;
   HEAP32[$0>>2] = 0;
   $122 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($122|0);
  }
  $25 = HEAP32[$1>>2]|0;
  $26 = (($25) + 68|0);
  $27 = HEAP32[$26>>2]|0;
  $28 = ($27|0)!=(0);
  if ($28) {
   $29 = HEAP32[$1>>2]|0;
   $30 = (($29) + 73|0);
   HEAP8[$30>>0] = 1;
   $31 = HEAP32[$1>>2]|0;
   $32 = (($31) + 68|0);
   $33 = HEAP32[$32>>2]|0;
   $34 = ($33|0)>(0);
   $35 = HEAP32[$1>>2]|0;
   $36 = (($35) + 68|0);
   HEAP32[$36>>2] = 0;
   if ($34) {
    HEAP32[$0>>2] = 0;
    $122 = HEAP32[$0>>2]|0;
    STACKTOP = sp;return ($122|0);
   }
  }
 }
 $37 = HEAP32[$1>>2]|0;
 $38 = (($37) + 76|0);
 $39 = HEAP16[$38>>1]|0;
 $40 = $39 << 16 >> 16;
 $41 = HEAP32[$1>>2]|0;
 $42 = (($41) + 120|0);
 $43 = (($42) + ($40<<2)|0);
 $44 = HEAP32[$43>>2]|0;
 HEAP32[$pPage>>2] = $44;
 $45 = HEAP32[$1>>2]|0;
 $46 = (($45) + 76|0);
 $47 = HEAP16[$46>>1]|0;
 $48 = $47 << 16 >> 16;
 $49 = HEAP32[$1>>2]|0;
 $50 = (($49) + 78|0);
 $51 = (($50) + ($48<<1)|0);
 $52 = HEAP16[$51>>1]|0;
 $53 = (($52) + 1)<<16>>16;
 HEAP16[$51>>1] = $53;
 $54 = $53&65535;
 HEAP32[$idx>>2] = $54;
 $55 = HEAP32[$idx>>2]|0;
 $56 = HEAP32[$pPage>>2]|0;
 $57 = (($56) + 18|0);
 $58 = HEAP16[$57>>1]|0;
 $59 = $58&65535;
 $60 = ($55|0)>=($59|0);
 $61 = HEAP32[$pPage>>2]|0;
 $62 = (($61) + 5|0);
 $63 = HEAP8[$62>>0]|0;
 $64 = ($63<<24>>24)!=(0);
 if (!($60)) {
  if ($64) {
   HEAP32[$0>>2] = 0;
   $122 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($122|0);
  } else {
   $120 = HEAP32[$1>>2]|0;
   $121 = (_moveToLeftmost($120)|0);
   HEAP32[$0>>2] = $121;
   $122 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($122|0);
  }
 }
 if (!($64)) {
  $65 = HEAP32[$1>>2]|0;
  $66 = HEAP32[$pPage>>2]|0;
  $67 = (($66) + 6|0);
  $68 = HEAP8[$67>>0]|0;
  $69 = $68&255;
  $70 = (($69) + 8)|0;
  $71 = HEAP32[$pPage>>2]|0;
  $72 = (($71) + 56|0);
  $73 = HEAP32[$72>>2]|0;
  $74 = (($73) + ($70)|0);
  $75 = (_sqlite3Get4byte($74)|0);
  $76 = (_moveToChild($65,$75)|0);
  HEAP32[$rc>>2] = $76;
  $77 = HEAP32[$rc>>2]|0;
  $78 = ($77|0)!=(0);
  if ($78) {
   $79 = HEAP32[$rc>>2]|0;
   HEAP32[$0>>2] = $79;
   $122 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($122|0);
  } else {
   $80 = HEAP32[$1>>2]|0;
   $81 = (_moveToLeftmost($80)|0);
   HEAP32[$0>>2] = $81;
   $122 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($122|0);
  }
 }
 while(1) {
  $82 = HEAP32[$1>>2]|0;
  $83 = (($82) + 76|0);
  $84 = HEAP16[$83>>1]|0;
  $85 = $84 << 16 >> 16;
  $86 = ($85|0)==(0);
  if ($86) {
   label = 17;
   break;
  }
  $90 = HEAP32[$1>>2]|0;
  _moveToParent($90);
  $91 = HEAP32[$1>>2]|0;
  $92 = (($91) + 76|0);
  $93 = HEAP16[$92>>1]|0;
  $94 = $93 << 16 >> 16;
  $95 = HEAP32[$1>>2]|0;
  $96 = (($95) + 120|0);
  $97 = (($96) + ($94<<2)|0);
  $98 = HEAP32[$97>>2]|0;
  HEAP32[$pPage>>2] = $98;
  $99 = HEAP32[$1>>2]|0;
  $100 = (($99) + 76|0);
  $101 = HEAP16[$100>>1]|0;
  $102 = $101 << 16 >> 16;
  $103 = HEAP32[$1>>2]|0;
  $104 = (($103) + 78|0);
  $105 = (($104) + ($102<<1)|0);
  $106 = HEAP16[$105>>1]|0;
  $107 = $106&65535;
  $108 = HEAP32[$pPage>>2]|0;
  $109 = (($108) + 18|0);
  $110 = HEAP16[$109>>1]|0;
  $111 = $110&65535;
  $112 = ($107|0)>=($111|0);
  if (!($112)) {
   break;
  }
 }
 if ((label|0) == 17) {
  $87 = HEAP32[$2>>2]|0;
  HEAP32[$87>>2] = 1;
  $88 = HEAP32[$1>>2]|0;
  $89 = (($88) + 73|0);
  HEAP8[$89>>0] = 0;
  HEAP32[$0>>2] = 0;
  $122 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($122|0);
 }
 $113 = HEAP32[$pPage>>2]|0;
 $114 = (($113) + 2|0);
 $115 = HEAP8[$114>>0]|0;
 $116 = ($115<<24>>24)!=(0);
 if ($116) {
  $117 = HEAP32[$1>>2]|0;
  $118 = HEAP32[$2>>2]|0;
  $119 = (_sqlite3BtreeNext($117,$118)|0);
  HEAP32[$0>>2] = $119;
  $122 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($122|0);
 } else {
  HEAP32[$0>>2] = 0;
  $122 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($122|0);
 }
 return 0|0;
}
function _moveToLeftmost($pCur) {
 $pCur = $pCur|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $7 = 0, $8 = 0, $9 = 0, $pPage = 0, $pgno = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $pgno = sp + 8|0;
 $rc = sp + 4|0;
 $pPage = sp;
 HEAP32[$0>>2] = $pCur;
 HEAP32[$rc>>2] = 0;
 while(1) {
  $1 = HEAP32[$rc>>2]|0;
  $2 = ($1|0)==(0);
  if (!($2)) {
   label = 5;
   break;
  }
  $3 = HEAP32[$0>>2]|0;
  $4 = (($3) + 76|0);
  $5 = HEAP16[$4>>1]|0;
  $6 = $5 << 16 >> 16;
  $7 = HEAP32[$0>>2]|0;
  $8 = (($7) + 120|0);
  $9 = (($8) + ($6<<2)|0);
  $10 = HEAP32[$9>>2]|0;
  HEAP32[$pPage>>2] = $10;
  $11 = (($10) + 5|0);
  $12 = HEAP8[$11>>0]|0;
  $13 = ($12<<24>>24)!=(0);
  $14 = $13 ^ 1;
  if (!($14)) {
   label = 5;
   break;
  }
  $15 = HEAP32[$pPage>>2]|0;
  $16 = (($15) + 56|0);
  $17 = HEAP32[$16>>2]|0;
  $18 = HEAP32[$pPage>>2]|0;
  $19 = (($18) + 20|0);
  $20 = HEAP16[$19>>1]|0;
  $21 = $20&65535;
  $22 = HEAP32[$0>>2]|0;
  $23 = (($22) + 76|0);
  $24 = HEAP16[$23>>1]|0;
  $25 = $24 << 16 >> 16;
  $26 = HEAP32[$0>>2]|0;
  $27 = (($26) + 78|0);
  $28 = (($27) + ($25<<1)|0);
  $29 = HEAP16[$28>>1]|0;
  $30 = $29&65535;
  $31 = $30<<1;
  $32 = HEAP32[$pPage>>2]|0;
  $33 = (($32) + 64|0);
  $34 = HEAP32[$33>>2]|0;
  $35 = (($34) + ($31)|0);
  $36 = HEAP8[$35>>0]|0;
  $37 = $36&255;
  $38 = $37 << 8;
  $39 = HEAP32[$0>>2]|0;
  $40 = (($39) + 76|0);
  $41 = HEAP16[$40>>1]|0;
  $42 = $41 << 16 >> 16;
  $43 = HEAP32[$0>>2]|0;
  $44 = (($43) + 78|0);
  $45 = (($44) + ($42<<1)|0);
  $46 = HEAP16[$45>>1]|0;
  $47 = $46&65535;
  $48 = $47<<1;
  $49 = HEAP32[$pPage>>2]|0;
  $50 = (($49) + 64|0);
  $51 = HEAP32[$50>>2]|0;
  $52 = (($51) + ($48)|0);
  $53 = (($52) + 1|0);
  $54 = HEAP8[$53>>0]|0;
  $55 = $54&255;
  $56 = $38 | $55;
  $57 = $21 & $56;
  $58 = (($17) + ($57)|0);
  $59 = (_sqlite3Get4byte($58)|0);
  HEAP32[$pgno>>2] = $59;
  $60 = HEAP32[$0>>2]|0;
  $61 = HEAP32[$pgno>>2]|0;
  $62 = (_moveToChild($60,$61)|0);
  HEAP32[$rc>>2] = $62;
 }
 if ((label|0) == 5) {
  $63 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($63|0);
 }
 return 0|0;
}
function _targetSrcList($pParse,$pStep) {
 $pParse = $pParse|0;
 $pStep = $pStep|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $db = 0;
 var $iDb = 0, $or$cond = 0, $pSrc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 16|0;
 $1 = sp + 12|0;
 $iDb = sp + 8|0;
 $pSrc = sp + 4|0;
 $db = sp;
 HEAP32[$0>>2] = $pParse;
 HEAP32[$1>>2] = $pStep;
 $2 = HEAP32[$0>>2]|0;
 $3 = HEAP32[$2>>2]|0;
 $4 = HEAP32[$1>>2]|0;
 $5 = (($4) + 12|0);
 $6 = (_sqlite3SrcListAppend($3,0,$5,0)|0);
 HEAP32[$pSrc>>2] = $6;
 $7 = HEAP32[$pSrc>>2]|0;
 $8 = ($7|0)!=(0|0);
 if (!($8)) {
  $38 = HEAP32[$pSrc>>2]|0;
  STACKTOP = sp;return ($38|0);
 }
 $9 = HEAP32[$0>>2]|0;
 $10 = HEAP32[$9>>2]|0;
 $11 = HEAP32[$1>>2]|0;
 $12 = (($11) + 4|0);
 $13 = HEAP32[$12>>2]|0;
 $14 = (($13) + 20|0);
 $15 = HEAP32[$14>>2]|0;
 $16 = (_sqlite3SchemaToIndex($10,$15)|0);
 HEAP32[$iDb>>2] = $16;
 $17 = HEAP32[$iDb>>2]|0;
 $18 = ($17|0)==(0);
 $19 = HEAP32[$iDb>>2]|0;
 $20 = ($19|0)>=(2);
 $or$cond = $18 | $20;
 if (!($or$cond)) {
  $38 = HEAP32[$pSrc>>2]|0;
  STACKTOP = sp;return ($38|0);
 }
 $21 = HEAP32[$0>>2]|0;
 $22 = HEAP32[$21>>2]|0;
 HEAP32[$db>>2] = $22;
 $23 = HEAP32[$db>>2]|0;
 $24 = HEAP32[$iDb>>2]|0;
 $25 = HEAP32[$db>>2]|0;
 $26 = (($25) + 16|0);
 $27 = HEAP32[$26>>2]|0;
 $28 = (($27) + ($24<<4)|0);
 $29 = HEAP32[$28>>2]|0;
 $30 = (_sqlite3DbStrDup($23,$29)|0);
 $31 = HEAP32[$pSrc>>2]|0;
 $32 = HEAP32[$31>>2]|0;
 $33 = (($32) - 1)|0;
 $34 = HEAP32[$pSrc>>2]|0;
 $35 = (($34) + 8|0);
 $36 = (($35) + (($33*72)|0)|0);
 $37 = (($36) + 4|0);
 HEAP32[$37>>2] = $30;
 $38 = HEAP32[$pSrc>>2]|0;
 STACKTOP = sp;return ($38|0);
}
function _fkTriggerDelete($dbMem,$p) {
 $dbMem = $dbMem|0;
 $p = $p|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $3 = 0, $4 = 0;
 var $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $pStep = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $1 = sp + 4|0;
 $pStep = sp;
 HEAP32[$0>>2] = $dbMem;
 HEAP32[$1>>2] = $p;
 $2 = HEAP32[$1>>2]|0;
 $3 = ($2|0)!=(0|0);
 if (!($3)) {
  STACKTOP = sp;return;
 }
 $4 = HEAP32[$1>>2]|0;
 $5 = (($4) + 28|0);
 $6 = HEAP32[$5>>2]|0;
 HEAP32[$pStep>>2] = $6;
 $7 = HEAP32[$0>>2]|0;
 $8 = HEAP32[$pStep>>2]|0;
 $9 = (($8) + 20|0);
 $10 = HEAP32[$9>>2]|0;
 _sqlite3ExprDelete($7,$10);
 $11 = HEAP32[$0>>2]|0;
 $12 = HEAP32[$pStep>>2]|0;
 $13 = (($12) + 24|0);
 $14 = HEAP32[$13>>2]|0;
 _sqlite3ExprListDelete($11,$14);
 $15 = HEAP32[$0>>2]|0;
 $16 = HEAP32[$pStep>>2]|0;
 $17 = (($16) + 8|0);
 $18 = HEAP32[$17>>2]|0;
 _sqlite3SelectDelete($15,$18);
 $19 = HEAP32[$0>>2]|0;
 $20 = HEAP32[$1>>2]|0;
 $21 = (($20) + 12|0);
 $22 = HEAP32[$21>>2]|0;
 _sqlite3ExprDelete($19,$22);
 $23 = HEAP32[$0>>2]|0;
 $24 = HEAP32[$1>>2]|0;
 _sqlite3DbFree($23,$24);
 STACKTOP = sp;return;
}
function _checkColumnOverlap($pIdList,$pEList) {
 $pIdList = $pIdList|0;
 $pEList = $pEList|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $3 = 0, $4 = 0, $5 = 0;
 var $6 = 0, $7 = 0, $8 = 0, $9 = 0, $e = 0, $or$cond = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $1 = sp + 8|0;
 $2 = sp + 4|0;
 $e = sp;
 HEAP32[$1>>2] = $pIdList;
 HEAP32[$2>>2] = $pEList;
 $3 = HEAP32[$1>>2]|0;
 $4 = ($3|0)==(0|0);
 $5 = HEAP32[$2>>2]|0;
 $6 = ($5|0)==(0|0);
 $or$cond = $4 | $6;
 if ($or$cond) {
  HEAP32[$0>>2] = 1;
  $23 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($23|0);
 }
 HEAP32[$e>>2] = 0;
 while(1) {
  $7 = HEAP32[$e>>2]|0;
  $8 = HEAP32[$2>>2]|0;
  $9 = HEAP32[$8>>2]|0;
  $10 = ($7|0)<($9|0);
  if (!($10)) {
   label = 8;
   break;
  }
  $11 = HEAP32[$1>>2]|0;
  $12 = HEAP32[$e>>2]|0;
  $13 = HEAP32[$2>>2]|0;
  $14 = (($13) + 4|0);
  $15 = HEAP32[$14>>2]|0;
  $16 = (($15) + (($12*20)|0)|0);
  $17 = (($16) + 4|0);
  $18 = HEAP32[$17>>2]|0;
  $19 = (_sqlite3IdListIndex($11,$18)|0);
  $20 = ($19|0)>=(0);
  if ($20) {
   label = 6;
   break;
  }
  $21 = HEAP32[$e>>2]|0;
  $22 = (($21) + 1)|0;
  HEAP32[$e>>2] = $22;
 }
 if ((label|0) == 6) {
  HEAP32[$0>>2] = 1;
  $23 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($23|0);
 }
 else if ((label|0) == 8) {
  HEAP32[$0>>2] = 0;
  $23 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($23|0);
 }
 return 0|0;
}
function _xferCompatibleCollation($z1,$z2) {
 $z1 = $z1|0;
 $z2 = $z2|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $1 = sp + 4|0;
 $2 = sp;
 HEAP32[$1>>2] = $z1;
 HEAP32[$2>>2] = $z2;
 $3 = HEAP32[$1>>2]|0;
 $4 = ($3|0)==(0|0);
 $5 = HEAP32[$2>>2]|0;
 $6 = ($5|0)==(0|0);
 do {
  if ($4) {
   $7 = $6&1;
   HEAP32[$0>>2] = $7;
  } else {
   if ($6) {
    HEAP32[$0>>2] = 0;
    break;
   } else {
    $8 = HEAP32[$1>>2]|0;
    $9 = HEAP32[$2>>2]|0;
    $10 = (_sqlite3_stricmp($8,$9)|0);
    $11 = ($10|0)==(0);
    $12 = $11&1;
    HEAP32[$0>>2] = $12;
    break;
   }
  }
 } while(0);
 $13 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($13|0);
}
function _xferCompatibleIndex($pDest,$pSrc) {
 $pDest = $pDest|0;
 $pSrc = $pSrc|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0;
 var $81 = 0, $9 = 0, $i = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $1 = sp + 8|0;
 $2 = sp + 4|0;
 $i = sp;
 HEAP32[$1>>2] = $pDest;
 HEAP32[$2>>2] = $pSrc;
 $3 = HEAP32[$1>>2]|0;
 $4 = (($3) + 50|0);
 $5 = HEAP16[$4>>1]|0;
 $6 = $5&65535;
 $7 = HEAP32[$2>>2]|0;
 $8 = (($7) + 50|0);
 $9 = HEAP16[$8>>1]|0;
 $10 = $9&65535;
 $11 = ($6|0)!=($10|0);
 if ($11) {
  HEAP32[$0>>2] = 0;
  $81 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($81|0);
 }
 $12 = HEAP32[$1>>2]|0;
 $13 = (($12) + 54|0);
 $14 = HEAP8[$13>>0]|0;
 $15 = $14&255;
 $16 = HEAP32[$2>>2]|0;
 $17 = (($16) + 54|0);
 $18 = HEAP8[$17>>0]|0;
 $19 = $18&255;
 $20 = ($15|0)!=($19|0);
 if ($20) {
  HEAP32[$0>>2] = 0;
  $81 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($81|0);
 }
 HEAP32[$i>>2] = 0;
 while(1) {
  $21 = HEAP32[$i>>2]|0;
  $22 = HEAP32[$2>>2]|0;
  $23 = (($22) + 50|0);
  $24 = HEAP16[$23>>1]|0;
  $25 = $24&65535;
  $26 = ($21|0)<($25|0);
  if (!($26)) {
   label = 14;
   break;
  }
  $27 = HEAP32[$i>>2]|0;
  $28 = HEAP32[$2>>2]|0;
  $29 = (($28) + 4|0);
  $30 = HEAP32[$29>>2]|0;
  $31 = (($30) + ($27<<1)|0);
  $32 = HEAP16[$31>>1]|0;
  $33 = $32 << 16 >> 16;
  $34 = HEAP32[$i>>2]|0;
  $35 = HEAP32[$1>>2]|0;
  $36 = (($35) + 4|0);
  $37 = HEAP32[$36>>2]|0;
  $38 = (($37) + ($34<<1)|0);
  $39 = HEAP16[$38>>1]|0;
  $40 = $39 << 16 >> 16;
  $41 = ($33|0)!=($40|0);
  if ($41) {
   label = 8;
   break;
  }
  $42 = HEAP32[$i>>2]|0;
  $43 = HEAP32[$2>>2]|0;
  $44 = (($43) + 28|0);
  $45 = HEAP32[$44>>2]|0;
  $46 = (($45) + ($42)|0);
  $47 = HEAP8[$46>>0]|0;
  $48 = $47&255;
  $49 = HEAP32[$i>>2]|0;
  $50 = HEAP32[$1>>2]|0;
  $51 = (($50) + 28|0);
  $52 = HEAP32[$51>>2]|0;
  $53 = (($52) + ($49)|0);
  $54 = HEAP8[$53>>0]|0;
  $55 = $54&255;
  $56 = ($48|0)!=($55|0);
  if ($56) {
   label = 10;
   break;
  }
  $57 = HEAP32[$i>>2]|0;
  $58 = HEAP32[$2>>2]|0;
  $59 = (($58) + 32|0);
  $60 = HEAP32[$59>>2]|0;
  $61 = (($60) + ($57<<2)|0);
  $62 = HEAP32[$61>>2]|0;
  $63 = HEAP32[$i>>2]|0;
  $64 = HEAP32[$1>>2]|0;
  $65 = (($64) + 32|0);
  $66 = HEAP32[$65>>2]|0;
  $67 = (($66) + ($63<<2)|0);
  $68 = HEAP32[$67>>2]|0;
  $69 = (_xferCompatibleCollation($62,$68)|0);
  $70 = ($69|0)!=(0);
  if (!($70)) {
   label = 12;
   break;
  }
  $71 = HEAP32[$i>>2]|0;
  $72 = (($71) + 1)|0;
  HEAP32[$i>>2] = $72;
 }
 if ((label|0) == 8) {
  HEAP32[$0>>2] = 0;
  $81 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($81|0);
 }
 else if ((label|0) == 10) {
  HEAP32[$0>>2] = 0;
  $81 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($81|0);
 }
 else if ((label|0) == 12) {
  HEAP32[$0>>2] = 0;
  $81 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($81|0);
 }
 else if ((label|0) == 14) {
  $73 = HEAP32[$2>>2]|0;
  $74 = (($73) + 36|0);
  $75 = HEAP32[$74>>2]|0;
  $76 = HEAP32[$1>>2]|0;
  $77 = (($76) + 36|0);
  $78 = HEAP32[$77>>2]|0;
  $79 = (_sqlite3ExprCompare($75,$78,-1)|0);
  $80 = ($79|0)!=(0);
  if ($80) {
   HEAP32[$0>>2] = 0;
   $81 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($81|0);
  } else {
   HEAP32[$0>>2] = 1;
   $81 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($81|0);
  }
 }
 return 0|0;
}
function _updateVirtualTable($pParse,$pSrc,$pTab,$pChanges,$pRowid,$aXRef,$pWhere,$onError) {
 $pParse = $pParse|0;
 $pSrc = $pSrc|0;
 $pTab = $pTab|0;
 $pChanges = $pChanges|0;
 $pRowid = $pRowid|0;
 $aXRef = $aXRef|0;
 $pWhere = $pWhere|0;
 $onError = $onError|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0;
 var $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0;
 var $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0;
 var $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0;
 var $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $addr = 0;
 var $db = 0, $dest = 0, $ephemTab = 0, $i = 0, $iReg = 0, $pEList = 0, $pExpr = 0, $pSelect = 0, $pVTab = 0, $v = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 96|0;
 $0 = sp + 88|0;
 $1 = sp + 84|0;
 $2 = sp + 80|0;
 $3 = sp + 76|0;
 $4 = sp + 72|0;
 $5 = sp + 68|0;
 $6 = sp + 64|0;
 $7 = sp + 60|0;
 $v = sp + 56|0;
 $pEList = sp + 52|0;
 $pSelect = sp + 48|0;
 $pExpr = sp + 44|0;
 $ephemTab = sp + 40|0;
 $i = sp + 36|0;
 $addr = sp + 32|0;
 $iReg = sp + 28|0;
 $db = sp + 24|0;
 $pVTab = sp + 20|0;
 $dest = sp;
 HEAP32[$0>>2] = $pParse;
 HEAP32[$1>>2] = $pSrc;
 HEAP32[$2>>2] = $pTab;
 HEAP32[$3>>2] = $pChanges;
 HEAP32[$4>>2] = $pRowid;
 HEAP32[$5>>2] = $aXRef;
 HEAP32[$6>>2] = $pWhere;
 HEAP32[$7>>2] = $onError;
 $8 = HEAP32[$0>>2]|0;
 $9 = (($8) + 8|0);
 $10 = HEAP32[$9>>2]|0;
 HEAP32[$v>>2] = $10;
 HEAP32[$pEList>>2] = 0;
 HEAP32[$pSelect>>2] = 0;
 $11 = HEAP32[$0>>2]|0;
 $12 = HEAP32[$11>>2]|0;
 HEAP32[$db>>2] = $12;
 $13 = HEAP32[$db>>2]|0;
 $14 = HEAP32[$2>>2]|0;
 $15 = (_sqlite3GetVTable($13,$14)|0);
 HEAP32[$pVTab>>2] = $15;
 $16 = HEAP32[$0>>2]|0;
 $17 = HEAP32[$db>>2]|0;
 $18 = (_sqlite3Expr($17,27,24560)|0);
 $19 = (_sqlite3ExprListAppend($16,0,$18)|0);
 HEAP32[$pEList>>2] = $19;
 $20 = HEAP32[$4>>2]|0;
 $21 = ($20|0)!=(0|0);
 if ($21) {
  $22 = HEAP32[$0>>2]|0;
  $23 = HEAP32[$pEList>>2]|0;
  $24 = HEAP32[$db>>2]|0;
  $25 = HEAP32[$4>>2]|0;
  $26 = (_sqlite3ExprDup($24,$25,0)|0);
  $27 = (_sqlite3ExprListAppend($22,$23,$26)|0);
  HEAP32[$pEList>>2] = $27;
 }
 HEAP32[$i>>2] = 0;
 while(1) {
  $28 = HEAP32[$i>>2]|0;
  $29 = HEAP32[$2>>2]|0;
  $30 = (($29) + 38|0);
  $31 = HEAP16[$30>>1]|0;
  $32 = $31 << 16 >> 16;
  $33 = ($28|0)<($32|0);
  if (!($33)) {
   break;
  }
  $34 = HEAP32[$i>>2]|0;
  $35 = HEAP32[$5>>2]|0;
  $36 = (($35) + ($34<<2)|0);
  $37 = HEAP32[$36>>2]|0;
  $38 = ($37|0)>=(0);
  $39 = HEAP32[$db>>2]|0;
  $40 = HEAP32[$i>>2]|0;
  if ($38) {
   $41 = HEAP32[$5>>2]|0;
   $42 = (($41) + ($40<<2)|0);
   $43 = HEAP32[$42>>2]|0;
   $44 = HEAP32[$3>>2]|0;
   $45 = (($44) + 4|0);
   $46 = HEAP32[$45>>2]|0;
   $47 = (($46) + (($43*20)|0)|0);
   $48 = HEAP32[$47>>2]|0;
   $49 = (_sqlite3ExprDup($39,$48,0)|0);
   HEAP32[$pExpr>>2] = $49;
  } else {
   $50 = HEAP32[$2>>2]|0;
   $51 = (($50) + 4|0);
   $52 = HEAP32[$51>>2]|0;
   $53 = (($52) + (($40*24)|0)|0);
   $54 = HEAP32[$53>>2]|0;
   $55 = (_sqlite3Expr($39,27,$54)|0);
   HEAP32[$pExpr>>2] = $55;
  }
  $56 = HEAP32[$0>>2]|0;
  $57 = HEAP32[$pEList>>2]|0;
  $58 = HEAP32[$pExpr>>2]|0;
  $59 = (_sqlite3ExprListAppend($56,$57,$58)|0);
  HEAP32[$pEList>>2] = $59;
  $60 = HEAP32[$i>>2]|0;
  $61 = (($60) + 1)|0;
  HEAP32[$i>>2] = $61;
 }
 $62 = HEAP32[$0>>2]|0;
 $63 = HEAP32[$pEList>>2]|0;
 $64 = HEAP32[$1>>2]|0;
 $65 = HEAP32[$6>>2]|0;
 $66 = (_sqlite3SelectNew($62,$63,$64,$65,0,0,0,0,0,0)|0);
 HEAP32[$pSelect>>2] = $66;
 $67 = HEAP32[$0>>2]|0;
 $68 = (($67) + 68|0);
 $69 = HEAP32[$68>>2]|0;
 $70 = (($69) + 1)|0;
 HEAP32[$68>>2] = $70;
 HEAP32[$ephemTab>>2] = $69;
 $71 = HEAP32[$v>>2]|0;
 $72 = HEAP32[$ephemTab>>2]|0;
 $73 = HEAP32[$2>>2]|0;
 $74 = (($73) + 38|0);
 $75 = HEAP16[$74>>1]|0;
 $76 = $75 << 16 >> 16;
 $77 = (($76) + 1)|0;
 $78 = HEAP32[$4>>2]|0;
 $79 = ($78|0)!=(0|0);
 $80 = $79&1;
 $81 = (($77) + ($80))|0;
 (_sqlite3VdbeAddOp2($71,57,$72,$81)|0);
 $82 = HEAP32[$v>>2]|0;
 _sqlite3VdbeChangeP5($82,8);
 $83 = HEAP32[$ephemTab>>2]|0;
 _sqlite3SelectDestInit($dest,14,$83);
 $84 = HEAP32[$0>>2]|0;
 $85 = HEAP32[$pSelect>>2]|0;
 (_sqlite3Select($84,$85,$dest)|0);
 $86 = HEAP32[$0>>2]|0;
 $87 = (($86) + 72|0);
 $88 = HEAP32[$87>>2]|0;
 $89 = (($88) + 1)|0;
 HEAP32[$87>>2] = $89;
 HEAP32[$iReg>>2] = $89;
 $90 = HEAP32[$2>>2]|0;
 $91 = (($90) + 38|0);
 $92 = HEAP16[$91>>1]|0;
 $93 = $92 << 16 >> 16;
 $94 = (($93) + 1)|0;
 $95 = HEAP32[$0>>2]|0;
 $96 = (($95) + 72|0);
 $97 = HEAP32[$96>>2]|0;
 $98 = (($97) + ($94))|0;
 HEAP32[$96>>2] = $98;
 $99 = HEAP32[$v>>2]|0;
 $100 = HEAP32[$ephemTab>>2]|0;
 $101 = (_sqlite3VdbeAddOp2($99,108,$100,0)|0);
 HEAP32[$addr>>2] = $101;
 $102 = HEAP32[$v>>2]|0;
 $103 = HEAP32[$ephemTab>>2]|0;
 $104 = HEAP32[$iReg>>2]|0;
 (_sqlite3VdbeAddOp3($102,47,$103,0,$104)|0);
 $105 = HEAP32[$v>>2]|0;
 $106 = HEAP32[$ephemTab>>2]|0;
 $107 = HEAP32[$4>>2]|0;
 $108 = ($107|0)!=(0|0);
 $109 = $108 ? 1 : 0;
 $110 = HEAP32[$iReg>>2]|0;
 $111 = (($110) + 1)|0;
 (_sqlite3VdbeAddOp3($105,47,$106,$109,$111)|0);
 HEAP32[$i>>2] = 0;
 while(1) {
  $112 = HEAP32[$i>>2]|0;
  $113 = HEAP32[$2>>2]|0;
  $114 = (($113) + 38|0);
  $115 = HEAP16[$114>>1]|0;
  $116 = $115 << 16 >> 16;
  $117 = ($112|0)<($116|0);
  if (!($117)) {
   break;
  }
  $118 = HEAP32[$v>>2]|0;
  $119 = HEAP32[$ephemTab>>2]|0;
  $120 = HEAP32[$i>>2]|0;
  $121 = (($120) + 1)|0;
  $122 = HEAP32[$4>>2]|0;
  $123 = ($122|0)!=(0|0);
  $124 = $123&1;
  $125 = (($121) + ($124))|0;
  $126 = HEAP32[$iReg>>2]|0;
  $127 = (($126) + 2)|0;
  $128 = HEAP32[$i>>2]|0;
  $129 = (($127) + ($128))|0;
  (_sqlite3VdbeAddOp3($118,47,$119,$125,$129)|0);
  $130 = HEAP32[$i>>2]|0;
  $131 = (($130) + 1)|0;
  HEAP32[$i>>2] = $131;
 }
 $132 = HEAP32[$0>>2]|0;
 $133 = HEAP32[$2>>2]|0;
 _sqlite3VtabMakeWritable($132,$133);
 $134 = HEAP32[$v>>2]|0;
 $135 = HEAP32[$2>>2]|0;
 $136 = (($135) + 38|0);
 $137 = HEAP16[$136>>1]|0;
 $138 = $137 << 16 >> 16;
 $139 = (($138) + 2)|0;
 $140 = HEAP32[$iReg>>2]|0;
 $141 = HEAP32[$pVTab>>2]|0;
 (_sqlite3VdbeAddOp4($134,15,0,$139,$140,$141,-10)|0);
 $142 = HEAP32[$v>>2]|0;
 $143 = HEAP32[$7>>2]|0;
 $144 = ($143|0)==(10);
 $145 = HEAP32[$7>>2]|0;
 $146 = $144 ? 2 : $145;
 $147 = $146&255;
 _sqlite3VdbeChangeP5($142,$147);
 $148 = HEAP32[$0>>2]|0;
 _sqlite3MayAbort($148);
 $149 = HEAP32[$v>>2]|0;
 $150 = HEAP32[$ephemTab>>2]|0;
 $151 = HEAP32[$addr>>2]|0;
 $152 = (($151) + 1)|0;
 (_sqlite3VdbeAddOp2($149,9,$150,$152)|0);
 $153 = HEAP32[$v>>2]|0;
 $154 = HEAP32[$addr>>2]|0;
 _sqlite3VdbeJumpHere($153,$154);
 $155 = HEAP32[$v>>2]|0;
 $156 = HEAP32[$ephemTab>>2]|0;
 (_sqlite3VdbeAddOp2($155,61,$156,0)|0);
 $157 = HEAP32[$db>>2]|0;
 $158 = HEAP32[$pSelect>>2]|0;
 _sqlite3SelectDelete($157,$158);
 STACKTOP = sp;return;
}
function _sqlite3AuthContextPush($pParse,$pContext,$zContext) {
 $pParse = $pParse|0;
 $pContext = $pContext|0;
 $zContext = $zContext|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $1 = sp + 4|0;
 $2 = sp;
 HEAP32[$0>>2] = $pParse;
 HEAP32[$1>>2] = $pContext;
 HEAP32[$2>>2] = $zContext;
 $3 = HEAP32[$0>>2]|0;
 $4 = HEAP32[$1>>2]|0;
 $5 = (($4) + 4|0);
 HEAP32[$5>>2] = $3;
 $6 = HEAP32[$0>>2]|0;
 $7 = (($6) + 492|0);
 $8 = HEAP32[$7>>2]|0;
 $9 = HEAP32[$1>>2]|0;
 HEAP32[$9>>2] = $8;
 $10 = HEAP32[$2>>2]|0;
 $11 = HEAP32[$0>>2]|0;
 $12 = (($11) + 492|0);
 HEAP32[$12>>2] = $10;
 STACKTOP = sp;return;
}
function _sqlite3MaterializeView($pParse,$pView,$pWhere,$iCur) {
 $pParse = $pParse|0;
 $pView = $pView|0;
 $pWhere = $pWhere|0;
 $iCur = $iCur|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $db = 0, $dest = 0, $iDb = 0, $pFrom = 0, $pSel = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 64|0;
 $0 = sp + 48|0;
 $1 = sp + 44|0;
 $2 = sp + 40|0;
 $3 = sp + 36|0;
 $dest = sp + 16|0;
 $pSel = sp + 12|0;
 $pFrom = sp + 8|0;
 $db = sp + 4|0;
 $iDb = sp;
 HEAP32[$0>>2] = $pParse;
 HEAP32[$1>>2] = $pView;
 HEAP32[$2>>2] = $pWhere;
 HEAP32[$3>>2] = $iCur;
 $4 = HEAP32[$0>>2]|0;
 $5 = HEAP32[$4>>2]|0;
 HEAP32[$db>>2] = $5;
 $6 = HEAP32[$db>>2]|0;
 $7 = HEAP32[$1>>2]|0;
 $8 = (($7) + 68|0);
 $9 = HEAP32[$8>>2]|0;
 $10 = (_sqlite3SchemaToIndex($6,$9)|0);
 HEAP32[$iDb>>2] = $10;
 $11 = HEAP32[$db>>2]|0;
 $12 = HEAP32[$2>>2]|0;
 $13 = (_sqlite3ExprDup($11,$12,0)|0);
 HEAP32[$2>>2] = $13;
 $14 = HEAP32[$db>>2]|0;
 $15 = (_sqlite3SrcListAppend($14,0,0,0)|0);
 HEAP32[$pFrom>>2] = $15;
 $16 = HEAP32[$pFrom>>2]|0;
 $17 = ($16|0)!=(0|0);
 if ($17) {
  $18 = HEAP32[$db>>2]|0;
  $19 = HEAP32[$1>>2]|0;
  $20 = HEAP32[$19>>2]|0;
  $21 = (_sqlite3DbStrDup($18,$20)|0);
  $22 = HEAP32[$pFrom>>2]|0;
  $23 = (($22) + 8|0);
  $24 = (($23) + 8|0);
  HEAP32[$24>>2] = $21;
  $25 = HEAP32[$db>>2]|0;
  $26 = HEAP32[$iDb>>2]|0;
  $27 = HEAP32[$db>>2]|0;
  $28 = (($27) + 16|0);
  $29 = HEAP32[$28>>2]|0;
  $30 = (($29) + ($26<<4)|0);
  $31 = HEAP32[$30>>2]|0;
  $32 = (_sqlite3DbStrDup($25,$31)|0);
  $33 = HEAP32[$pFrom>>2]|0;
  $34 = (($33) + 8|0);
  $35 = (($34) + 4|0);
  HEAP32[$35>>2] = $32;
 }
 $36 = HEAP32[$0>>2]|0;
 $37 = HEAP32[$pFrom>>2]|0;
 $38 = HEAP32[$2>>2]|0;
 $39 = (_sqlite3SelectNew($36,0,$37,$38,0,0,0,0,0,0)|0);
 HEAP32[$pSel>>2] = $39;
 $40 = HEAP32[$3>>2]|0;
 _sqlite3SelectDestInit($dest,12,$40);
 $41 = HEAP32[$0>>2]|0;
 $42 = HEAP32[$pSel>>2]|0;
 (_sqlite3Select($41,$42,$dest)|0);
 $43 = HEAP32[$db>>2]|0;
 $44 = HEAP32[$pSel>>2]|0;
 _sqlite3SelectDelete($43,$44);
 STACKTOP = sp;return;
}
function _sqlite3WhereOkOnePass($pWInfo,$aiCur) {
 $pWInfo = $pWInfo|0;
 $aiCur = $aiCur|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $1 = sp;
 HEAP32[$0>>2] = $pWInfo;
 HEAP32[$1>>2] = $aiCur;
 $2 = HEAP32[$1>>2]|0;
 $3 = HEAP32[$0>>2]|0;
 $4 = (($3) + 60|0);
 ;HEAP32[$2+0>>2]=HEAP32[$4+0>>2]|0;HEAP32[$2+4>>2]=HEAP32[$4+4>>2]|0;
 $5 = HEAP32[$0>>2]|0;
 $6 = (($5) + 38|0);
 $7 = HEAP8[$6>>0]|0;
 $8 = $7&255;
 STACKTOP = sp;return ($8|0);
}
function _sqlite3AuthContextPop($pContext) {
 $pContext = $pContext|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 HEAP32[$0>>2] = $pContext;
 $1 = HEAP32[$0>>2]|0;
 $2 = (($1) + 4|0);
 $3 = HEAP32[$2>>2]|0;
 $4 = ($3|0)!=(0|0);
 if (!($4)) {
  STACKTOP = sp;return;
 }
 $5 = HEAP32[$0>>2]|0;
 $6 = HEAP32[$5>>2]|0;
 $7 = HEAP32[$0>>2]|0;
 $8 = (($7) + 4|0);
 $9 = HEAP32[$8>>2]|0;
 $10 = (($9) + 492|0);
 HEAP32[$10>>2] = $6;
 $11 = HEAP32[$0>>2]|0;
 $12 = (($11) + 4|0);
 HEAP32[$12>>2] = 0;
 STACKTOP = sp;return;
}
function _sqlite3SrcListEnlarge($db,$pSrc,$nExtra,$iStart) {
 $db = $db|0;
 $pSrc = $pSrc|0;
 $nExtra = $nExtra|0;
 $iStart = $iStart|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $9 = 0;
 var $i = 0, $nAlloc = 0, $nGot = 0, $pNew = 0, dest = 0, label = 0, sp = 0, src = 0, stop = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $0 = sp + 32|0;
 $1 = sp + 28|0;
 $2 = sp + 24|0;
 $3 = sp + 20|0;
 $4 = sp + 16|0;
 $i = sp + 12|0;
 $pNew = sp + 8|0;
 $nAlloc = sp + 4|0;
 $nGot = sp;
 HEAP32[$1>>2] = $db;
 HEAP32[$2>>2] = $pSrc;
 HEAP32[$3>>2] = $nExtra;
 HEAP32[$4>>2] = $iStart;
 $5 = HEAP32[$2>>2]|0;
 $6 = HEAP32[$5>>2]|0;
 $7 = HEAP32[$3>>2]|0;
 $8 = (($6) + ($7))|0;
 $9 = HEAP32[$2>>2]|0;
 $10 = (($9) + 4|0);
 $11 = HEAP32[$10>>2]|0;
 $12 = ($8>>>0)>($11>>>0);
 do {
  if ($12) {
   $13 = HEAP32[$2>>2]|0;
   $14 = HEAP32[$13>>2]|0;
   $15 = HEAP32[$3>>2]|0;
   $16 = (($14) + ($15))|0;
   HEAP32[$nAlloc>>2] = $16;
   $17 = HEAP32[$1>>2]|0;
   $18 = HEAP32[$2>>2]|0;
   $19 = HEAP32[$nAlloc>>2]|0;
   $20 = (($19) - 1)|0;
   $21 = ($20*72)|0;
   $22 = (80 + ($21))|0;
   $23 = (_sqlite3DbRealloc($17,$18,$22,0)|0);
   HEAP32[$pNew>>2] = $23;
   $24 = HEAP32[$pNew>>2]|0;
   $25 = ($24|0)==(0|0);
   if (!($25)) {
    $27 = HEAP32[$pNew>>2]|0;
    HEAP32[$2>>2] = $27;
    $28 = HEAP32[$1>>2]|0;
    $29 = HEAP32[$pNew>>2]|0;
    $30 = (_sqlite3DbMallocSize($28,$29)|0);
    $31 = (($30) - 80)|0;
    $32 = (($31>>>0) / 72)&-1;
    $33 = (($32) + 1)|0;
    HEAP32[$nGot>>2] = $33;
    $34 = HEAP32[$nGot>>2]|0;
    $35 = HEAP32[$2>>2]|0;
    $36 = (($35) + 4|0);
    HEAP32[$36>>2] = $34;
    break;
   }
   $26 = HEAP32[$2>>2]|0;
   HEAP32[$0>>2] = $26;
   $79 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($79|0);
  }
 } while(0);
 $37 = HEAP32[$2>>2]|0;
 $38 = HEAP32[$37>>2]|0;
 $39 = (($38) - 1)|0;
 HEAP32[$i>>2] = $39;
 while(1) {
  $40 = HEAP32[$i>>2]|0;
  $41 = HEAP32[$4>>2]|0;
  $42 = ($40|0)>=($41|0);
  if (!($42)) {
   break;
  }
  $43 = HEAP32[$i>>2]|0;
  $44 = HEAP32[$3>>2]|0;
  $45 = (($43) + ($44))|0;
  $46 = HEAP32[$2>>2]|0;
  $47 = (($46) + 8|0);
  $48 = (($47) + (($45*72)|0)|0);
  $49 = HEAP32[$i>>2]|0;
  $50 = HEAP32[$2>>2]|0;
  $51 = (($50) + 8|0);
  $52 = (($51) + (($49*72)|0)|0);
  dest=$48+0|0; src=$52+0|0; stop=dest+72|0; do { HEAP32[dest>>2]=HEAP32[src>>2]|0; dest=dest+4|0; src=src+4|0; } while ((dest|0) < (stop|0));
  $53 = HEAP32[$i>>2]|0;
  $54 = (($53) + -1)|0;
  HEAP32[$i>>2] = $54;
 }
 $55 = HEAP32[$3>>2]|0;
 $56 = HEAP32[$2>>2]|0;
 $57 = HEAP32[$56>>2]|0;
 $58 = (($57) + ($55))|0;
 HEAP32[$56>>2] = $58;
 $59 = HEAP32[$4>>2]|0;
 $60 = HEAP32[$2>>2]|0;
 $61 = (($60) + 8|0);
 $62 = (($61) + (($59*72)|0)|0);
 $63 = HEAP32[$3>>2]|0;
 $64 = ($63*72)|0;
 _memset(($62|0),0,($64|0))|0;
 $65 = HEAP32[$4>>2]|0;
 HEAP32[$i>>2] = $65;
 while(1) {
  $66 = HEAP32[$i>>2]|0;
  $67 = HEAP32[$4>>2]|0;
  $68 = HEAP32[$3>>2]|0;
  $69 = (($67) + ($68))|0;
  $70 = ($66|0)<($69|0);
  if (!($70)) {
   break;
  }
  $71 = HEAP32[$i>>2]|0;
  $72 = HEAP32[$2>>2]|0;
  $73 = (($72) + 8|0);
  $74 = (($73) + (($71*72)|0)|0);
  $75 = (($74) + 40|0);
  HEAP32[$75>>2] = -1;
  $76 = HEAP32[$i>>2]|0;
  $77 = (($76) + 1)|0;
  HEAP32[$i>>2] = $77;
 }
 $78 = HEAP32[$2>>2]|0;
 HEAP32[$0>>2] = $78;
 $79 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($79|0);
}
function _checkForMultiColumnSelectError($pParse,$pDest,$nExpr) {
 $pParse = $pParse|0;
 $pDest = $pDest|0;
 $nExpr = $nExpr|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $eDest = 0, $or$cond = 0, $vararg_buffer = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $vararg_buffer = sp;
 $0 = sp + 20|0;
 $1 = sp + 16|0;
 $2 = sp + 12|0;
 $3 = sp + 8|0;
 $eDest = sp + 4|0;
 HEAP32[$1>>2] = $pParse;
 HEAP32[$2>>2] = $pDest;
 HEAP32[$3>>2] = $nExpr;
 $4 = HEAP32[$2>>2]|0;
 $5 = HEAP8[$4>>0]|0;
 $6 = $5&255;
 HEAP32[$eDest>>2] = $6;
 $7 = HEAP32[$3>>2]|0;
 $8 = ($7|0)>(1);
 if ($8) {
  $9 = HEAP32[$eDest>>2]|0;
  $10 = ($9|0)==(10);
  $11 = HEAP32[$eDest>>2]|0;
  $12 = ($11|0)==(11);
  $or$cond = $10 | $12;
  if ($or$cond) {
   $13 = HEAP32[$1>>2]|0;
   _sqlite3ErrorMsg($13,25392,$vararg_buffer);
   HEAP32[$0>>2] = 1;
   $14 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($14|0);
  }
 }
 HEAP32[$0>>2] = 0;
 $14 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($14|0);
}
function _sqlite3SelectExprHeight($p) {
 $p = $p|0;
 var $0 = 0, $1 = 0, $2 = 0, $nHeight = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $nHeight = sp;
 HEAP32[$0>>2] = $p;
 HEAP32[$nHeight>>2] = 0;
 $1 = HEAP32[$0>>2]|0;
 _heightOfSelect($1,$nHeight);
 $2 = HEAP32[$nHeight>>2]|0;
 STACKTOP = sp;return ($2|0);
}
function _flattenSubquery($pParse,$p,$iFrom,$isAgg,$subqueryIsAgg) {
 $pParse = $pParse|0;
 $p = $p|0;
 $iFrom = $iFrom|0;
 $isAgg = $isAgg|0;
 $subqueryIsAgg = $subqueryIsAgg|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0;
 var $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0, $188 = 0;
 var $189 = 0, $19 = 0, $190 = 0, $191 = 0, $192 = 0, $193 = 0, $194 = 0, $195 = 0, $196 = 0, $197 = 0, $198 = 0, $199 = 0, $2 = 0, $20 = 0, $200 = 0, $201 = 0, $202 = 0, $203 = 0, $204 = 0, $205 = 0;
 var $206 = 0, $207 = 0, $208 = 0, $209 = 0, $21 = 0, $210 = 0, $211 = 0, $212 = 0, $213 = 0, $214 = 0, $215 = 0, $216 = 0, $217 = 0, $218 = 0, $219 = 0, $22 = 0, $220 = 0, $221 = 0, $222 = 0, $223 = 0;
 var $224 = 0, $225 = 0, $226 = 0, $227 = 0, $228 = 0, $229 = 0, $23 = 0, $230 = 0, $231 = 0, $232 = 0, $233 = 0, $234 = 0, $235 = 0, $236 = 0, $237 = 0, $238 = 0, $239 = 0, $24 = 0, $240 = 0, $241 = 0;
 var $242 = 0, $243 = 0, $244 = 0, $245 = 0, $246 = 0, $247 = 0, $248 = 0, $249 = 0, $25 = 0, $250 = 0, $251 = 0, $252 = 0, $253 = 0, $254 = 0, $255 = 0, $256 = 0, $257 = 0, $258 = 0, $259 = 0, $26 = 0;
 var $260 = 0, $261 = 0, $262 = 0, $263 = 0, $264 = 0, $265 = 0, $266 = 0, $267 = 0, $268 = 0, $269 = 0, $27 = 0, $270 = 0, $271 = 0, $272 = 0, $273 = 0, $274 = 0, $275 = 0, $276 = 0, $277 = 0, $278 = 0;
 var $279 = 0, $28 = 0, $280 = 0, $281 = 0, $282 = 0, $283 = 0, $284 = 0, $285 = 0, $286 = 0, $287 = 0, $288 = 0, $289 = 0, $29 = 0, $290 = 0, $291 = 0, $292 = 0, $293 = 0, $294 = 0, $295 = 0, $296 = 0;
 var $297 = 0, $298 = 0, $299 = 0, $3 = 0, $30 = 0, $300 = 0, $301 = 0, $302 = 0, $303 = 0, $304 = 0, $305 = 0, $306 = 0, $307 = 0, $308 = 0, $309 = 0, $31 = 0, $310 = 0, $311 = 0, $312 = 0, $313 = 0;
 var $314 = 0, $315 = 0, $316 = 0, $317 = 0, $318 = 0, $319 = 0, $32 = 0, $320 = 0, $321 = 0, $322 = 0, $323 = 0, $324 = 0, $325 = 0, $326 = 0, $327 = 0, $328 = 0, $329 = 0, $33 = 0, $330 = 0, $331 = 0;
 var $332 = 0, $333 = 0, $334 = 0, $335 = 0, $336 = 0, $337 = 0, $338 = 0, $339 = 0, $34 = 0, $340 = 0, $341 = 0, $342 = 0, $343 = 0, $344 = 0, $345 = 0, $346 = 0, $347 = 0, $348 = 0, $349 = 0, $35 = 0;
 var $350 = 0, $351 = 0, $352 = 0, $353 = 0, $354 = 0, $355 = 0, $356 = 0, $357 = 0, $358 = 0, $359 = 0, $36 = 0, $360 = 0, $361 = 0, $362 = 0, $363 = 0, $364 = 0, $365 = 0, $366 = 0, $367 = 0, $368 = 0;
 var $369 = 0, $37 = 0, $370 = 0, $371 = 0, $372 = 0, $373 = 0, $374 = 0, $375 = 0, $376 = 0, $377 = 0, $378 = 0, $379 = 0, $38 = 0, $380 = 0, $381 = 0, $382 = 0, $383 = 0, $384 = 0, $385 = 0, $386 = 0;
 var $387 = 0, $388 = 0, $389 = 0, $39 = 0, $390 = 0, $391 = 0, $392 = 0, $393 = 0, $394 = 0, $395 = 0, $396 = 0, $397 = 0, $398 = 0, $399 = 0, $4 = 0, $40 = 0, $400 = 0, $401 = 0, $402 = 0, $403 = 0;
 var $404 = 0, $405 = 0, $406 = 0, $407 = 0, $408 = 0, $409 = 0, $41 = 0, $410 = 0, $411 = 0, $412 = 0, $413 = 0, $414 = 0, $415 = 0, $416 = 0, $417 = 0, $418 = 0, $419 = 0, $42 = 0, $420 = 0, $421 = 0;
 var $422 = 0, $423 = 0, $424 = 0, $425 = 0, $426 = 0, $427 = 0, $428 = 0, $429 = 0, $43 = 0, $430 = 0, $431 = 0, $432 = 0, $433 = 0, $434 = 0, $435 = 0, $436 = 0, $437 = 0, $438 = 0, $439 = 0, $44 = 0;
 var $440 = 0, $441 = 0, $442 = 0, $443 = 0, $444 = 0, $445 = 0, $446 = 0, $447 = 0, $448 = 0, $449 = 0, $45 = 0, $450 = 0, $451 = 0, $452 = 0, $453 = 0, $454 = 0, $455 = 0, $456 = 0, $457 = 0, $458 = 0;
 var $459 = 0, $46 = 0, $460 = 0, $461 = 0, $462 = 0, $463 = 0, $464 = 0, $465 = 0, $466 = 0, $467 = 0, $468 = 0, $469 = 0, $47 = 0, $470 = 0, $471 = 0, $472 = 0, $473 = 0, $474 = 0, $475 = 0, $476 = 0;
 var $477 = 0, $478 = 0, $479 = 0, $48 = 0, $480 = 0, $481 = 0, $482 = 0, $483 = 0, $484 = 0, $485 = 0, $486 = 0, $487 = 0, $488 = 0, $489 = 0, $49 = 0, $490 = 0, $491 = 0, $492 = 0, $493 = 0, $494 = 0;
 var $495 = 0, $496 = 0, $497 = 0, $498 = 0, $499 = 0, $5 = 0, $50 = 0, $500 = 0, $501 = 0, $502 = 0, $503 = 0, $504 = 0, $505 = 0, $506 = 0, $507 = 0, $508 = 0, $509 = 0, $51 = 0, $510 = 0, $511 = 0;
 var $512 = 0, $513 = 0, $514 = 0, $515 = 0, $516 = 0, $517 = 0, $518 = 0, $519 = 0, $52 = 0, $520 = 0, $521 = 0, $522 = 0, $523 = 0, $524 = 0, $525 = 0, $526 = 0, $527 = 0, $528 = 0, $529 = 0, $53 = 0;
 var $530 = 0, $531 = 0, $532 = 0, $533 = 0, $534 = 0, $535 = 0, $536 = 0, $537 = 0, $538 = 0, $539 = 0, $54 = 0, $540 = 0, $541 = 0, $542 = 0, $543 = 0, $544 = 0, $545 = 0, $546 = 0, $547 = 0, $548 = 0;
 var $549 = 0, $55 = 0, $550 = 0, $551 = 0, $552 = 0, $553 = 0, $554 = 0, $555 = 0, $556 = 0, $557 = 0, $558 = 0, $559 = 0, $56 = 0, $560 = 0, $561 = 0, $562 = 0, $563 = 0, $564 = 0, $565 = 0, $566 = 0;
 var $567 = 0, $568 = 0, $569 = 0, $57 = 0, $570 = 0, $571 = 0, $572 = 0, $573 = 0, $574 = 0, $575 = 0, $576 = 0, $577 = 0, $578 = 0, $579 = 0, $58 = 0, $580 = 0, $581 = 0, $582 = 0, $583 = 0, $584 = 0;
 var $585 = 0, $586 = 0, $587 = 0, $588 = 0, $589 = 0, $59 = 0, $590 = 0, $591 = 0, $592 = 0, $593 = 0, $594 = 0, $595 = 0, $596 = 0, $597 = 0, $598 = 0, $599 = 0, $6 = 0, $60 = 0, $600 = 0, $601 = 0;
 var $602 = 0, $603 = 0, $604 = 0, $605 = 0, $606 = 0, $607 = 0, $608 = 0, $609 = 0, $61 = 0, $610 = 0, $611 = 0, $612 = 0, $613 = 0, $614 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0;
 var $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0;
 var $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $db = 0, $i = 0, $iParent = 0, $ii = 0, $jointype = 0;
 var $nSubSrc = 0, $or$cond = 0, $or$cond3 = 0, $or$cond5 = 0, $pLimit = 0, $pList = 0, $pNew = 0, $pOffset = 0, $pOrderBy = 0, $pOrderBy1 = 0, $pParent = 0, $pPrior = 0, $pSrc = 0, $pSub = 0, $pSub1 = 0, $pSubSrc = 0, $pSubitem = 0, $pTabToDel = 0, $pToplevel = 0, $pWhere = 0;
 var $zName = 0, $zSavedAuthContext = 0, dest = 0, label = 0, sp = 0, src = 0, stop = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 128|0;
 $0 = sp + 112|0;
 $1 = sp + 108|0;
 $2 = sp + 104|0;
 $3 = sp + 100|0;
 $4 = sp + 96|0;
 $5 = sp + 92|0;
 $zSavedAuthContext = sp + 88|0;
 $pParent = sp + 84|0;
 $pSub = sp + 80|0;
 $pSub1 = sp + 76|0;
 $pSrc = sp + 72|0;
 $pSubSrc = sp + 68|0;
 $pList = sp + 64|0;
 $iParent = sp + 60|0;
 $i = sp + 56|0;
 $pWhere = sp + 52|0;
 $pSubitem = sp + 48|0;
 $db = sp + 44|0;
 $ii = sp + 40|0;
 $pNew = sp + 36|0;
 $pOrderBy = sp + 32|0;
 $pLimit = sp + 28|0;
 $pOffset = sp + 24|0;
 $pPrior = sp + 20|0;
 $pTabToDel = sp + 16|0;
 $pToplevel = sp + 12|0;
 $nSubSrc = sp + 8|0;
 $jointype = sp + 116|0;
 $zName = sp + 4|0;
 $pOrderBy1 = sp;
 HEAP32[$1>>2] = $pParse;
 HEAP32[$2>>2] = $p;
 HEAP32[$3>>2] = $iFrom;
 HEAP32[$4>>2] = $isAgg;
 HEAP32[$5>>2] = $subqueryIsAgg;
 $6 = HEAP32[$1>>2]|0;
 $7 = (($6) + 492|0);
 $8 = HEAP32[$7>>2]|0;
 HEAP32[$zSavedAuthContext>>2] = $8;
 $9 = HEAP32[$1>>2]|0;
 $10 = HEAP32[$9>>2]|0;
 HEAP32[$db>>2] = $10;
 $11 = HEAP32[$db>>2]|0;
 $12 = (($11) + 60|0);
 $13 = HEAP16[$12>>1]|0;
 $14 = $13&65535;
 $15 = $14 & 1;
 $16 = ($15|0)!=(0);
 if ($16) {
  HEAP32[$0>>2] = 0;
  $614 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($614|0);
 }
 $17 = HEAP32[$2>>2]|0;
 $18 = (($17) + 32|0);
 $19 = HEAP32[$18>>2]|0;
 HEAP32[$pSrc>>2] = $19;
 $20 = HEAP32[$3>>2]|0;
 $21 = HEAP32[$pSrc>>2]|0;
 $22 = (($21) + 8|0);
 $23 = (($22) + (($20*72)|0)|0);
 HEAP32[$pSubitem>>2] = $23;
 $24 = HEAP32[$pSubitem>>2]|0;
 $25 = (($24) + 40|0);
 $26 = HEAP32[$25>>2]|0;
 HEAP32[$iParent>>2] = $26;
 $27 = HEAP32[$pSubitem>>2]|0;
 $28 = (($27) + 20|0);
 $29 = HEAP32[$28>>2]|0;
 HEAP32[$pSub>>2] = $29;
 $30 = HEAP32[$4>>2]|0;
 $31 = ($30|0)!=(0);
 $32 = HEAP32[$5>>2]|0;
 $33 = ($32|0)!=(0);
 $or$cond = $31 & $33;
 if ($or$cond) {
  HEAP32[$0>>2] = 0;
  $614 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($614|0);
 }
 $34 = HEAP32[$5>>2]|0;
 $35 = ($34|0)!=(0);
 if ($35) {
  $36 = HEAP32[$pSrc>>2]|0;
  $37 = HEAP32[$36>>2]|0;
  $38 = ($37|0)>(1);
  if ($38) {
   HEAP32[$0>>2] = 0;
   $614 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($614|0);
  }
 }
 $39 = HEAP32[$pSub>>2]|0;
 $40 = (($39) + 32|0);
 $41 = HEAP32[$40>>2]|0;
 HEAP32[$pSubSrc>>2] = $41;
 $42 = HEAP32[$pSub>>2]|0;
 $43 = (($42) + 60|0);
 $44 = HEAP32[$43>>2]|0;
 $45 = ($44|0)!=(0|0);
 if ($45) {
  $46 = HEAP32[$2>>2]|0;
  $47 = (($46) + 60|0);
  $48 = HEAP32[$47>>2]|0;
  $49 = ($48|0)!=(0|0);
  if ($49) {
   HEAP32[$0>>2] = 0;
   $614 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($614|0);
  }
 }
 $50 = HEAP32[$pSub>>2]|0;
 $51 = (($50) + 64|0);
 $52 = HEAP32[$51>>2]|0;
 $53 = ($52|0)!=(0|0);
 if ($53) {
  HEAP32[$0>>2] = 0;
  $614 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($614|0);
 }
 $54 = HEAP32[$2>>2]|0;
 $55 = (($54) + 6|0);
 $56 = HEAP16[$55>>1]|0;
 $57 = $56&65535;
 $58 = $57 & 64;
 $59 = ($58|0)!=(0);
 if ($59) {
  $60 = HEAP32[$pSub>>2]|0;
  $61 = (($60) + 60|0);
  $62 = HEAP32[$61>>2]|0;
  $63 = ($62|0)!=(0|0);
  if ($63) {
   HEAP32[$0>>2] = 0;
   $614 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($614|0);
  }
 }
 $64 = HEAP32[$pSubSrc>>2]|0;
 $65 = HEAP32[$64>>2]|0;
 $66 = ($65|0)==(0);
 if ($66) {
  HEAP32[$0>>2] = 0;
  $614 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($614|0);
 }
 $67 = HEAP32[$pSub>>2]|0;
 $68 = (($67) + 6|0);
 $69 = HEAP16[$68>>1]|0;
 $70 = $69&65535;
 $71 = $70 & 1;
 $72 = ($71|0)!=(0);
 if ($72) {
  HEAP32[$0>>2] = 0;
  $614 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($614|0);
 }
 $73 = HEAP32[$pSub>>2]|0;
 $74 = (($73) + 60|0);
 $75 = HEAP32[$74>>2]|0;
 $76 = ($75|0)!=(0|0);
 if ($76) {
  $77 = HEAP32[$pSrc>>2]|0;
  $78 = HEAP32[$77>>2]|0;
  $79 = ($78|0)>(1);
  $80 = HEAP32[$4>>2]|0;
  $81 = ($80|0)!=(0);
  $or$cond3 = $79 | $81;
  if ($or$cond3) {
   HEAP32[$0>>2] = 0;
   $614 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($614|0);
  }
 }
 $82 = HEAP32[$2>>2]|0;
 $83 = (($82) + 6|0);
 $84 = HEAP16[$83>>1]|0;
 $85 = $84&65535;
 $86 = $85 & 1;
 $87 = ($86|0)!=(0);
 $88 = HEAP32[$5>>2]|0;
 $89 = ($88|0)!=(0);
 $or$cond5 = $87 & $89;
 if ($or$cond5) {
  HEAP32[$0>>2] = 0;
  $614 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($614|0);
 }
 $90 = HEAP32[$2>>2]|0;
 $91 = (($90) + 48|0);
 $92 = HEAP32[$91>>2]|0;
 $93 = ($92|0)!=(0|0);
 if ($93) {
  $94 = HEAP32[$pSub>>2]|0;
  $95 = (($94) + 48|0);
  $96 = HEAP32[$95>>2]|0;
  $97 = ($96|0)!=(0|0);
  if ($97) {
   HEAP32[$0>>2] = 0;
   $614 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($614|0);
  }
 }
 $98 = HEAP32[$4>>2]|0;
 $99 = ($98|0)!=(0);
 if ($99) {
  $100 = HEAP32[$pSub>>2]|0;
  $101 = (($100) + 48|0);
  $102 = HEAP32[$101>>2]|0;
  $103 = ($102|0)!=(0|0);
  if ($103) {
   HEAP32[$0>>2] = 0;
   $614 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($614|0);
  }
 }
 $104 = HEAP32[$pSub>>2]|0;
 $105 = (($104) + 60|0);
 $106 = HEAP32[$105>>2]|0;
 $107 = ($106|0)!=(0|0);
 if ($107) {
  $108 = HEAP32[$2>>2]|0;
  $109 = (($108) + 36|0);
  $110 = HEAP32[$109>>2]|0;
  $111 = ($110|0)!=(0|0);
  if ($111) {
   HEAP32[$0>>2] = 0;
   $614 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($614|0);
  }
 }
 $112 = HEAP32[$pSub>>2]|0;
 $113 = (($112) + 60|0);
 $114 = HEAP32[$113>>2]|0;
 $115 = ($114|0)!=(0|0);
 if ($115) {
  $116 = HEAP32[$2>>2]|0;
  $117 = (($116) + 6|0);
  $118 = HEAP16[$117>>1]|0;
  $119 = $118&65535;
  $120 = $119 & 1;
  $121 = ($120|0)!=(0);
  if ($121) {
   HEAP32[$0>>2] = 0;
   $614 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($614|0);
  }
 }
 $122 = HEAP32[$pSub>>2]|0;
 $123 = (($122) + 6|0);
 $124 = HEAP16[$123>>1]|0;
 $125 = $124&65535;
 $126 = $125 & 6144;
 $127 = ($126|0)!=(0);
 if ($127) {
  HEAP32[$0>>2] = 0;
  $614 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($614|0);
 }
 $128 = HEAP32[$2>>2]|0;
 $129 = (($128) + 6|0);
 $130 = HEAP16[$129>>1]|0;
 $131 = $130&65535;
 $132 = $131 & 2048;
 $133 = ($132|0)!=(0);
 if ($133) {
  $134 = HEAP32[$pSub>>2]|0;
  $135 = (($134) + 52|0);
  $136 = HEAP32[$135>>2]|0;
  $137 = ($136|0)!=(0|0);
  if ($137) {
   HEAP32[$0>>2] = 0;
   $614 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($614|0);
  }
 }
 $138 = HEAP32[$pSubitem>>2]|0;
 $139 = (($138) + 36|0);
 $140 = HEAP8[$139>>0]|0;
 $141 = $140&255;
 $142 = $141 & 32;
 $143 = ($142|0)!=(0);
 if ($143) {
  HEAP32[$0>>2] = 0;
  $614 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($614|0);
 }
 $144 = HEAP32[$pSub>>2]|0;
 $145 = (($144) + 52|0);
 $146 = HEAP32[$145>>2]|0;
 $147 = ($146|0)!=(0|0);
 L78: do {
  if ($147) {
   $148 = HEAP32[$pSub>>2]|0;
   $149 = (($148) + 48|0);
   $150 = HEAP32[$149>>2]|0;
   $151 = ($150|0)!=(0|0);
   if ($151) {
    HEAP32[$0>>2] = 0;
    $614 = HEAP32[$0>>2]|0;
    STACKTOP = sp;return ($614|0);
   }
   $152 = HEAP32[$4>>2]|0;
   $153 = ($152|0)!=(0);
   do {
    if (!($153)) {
     $154 = HEAP32[$2>>2]|0;
     $155 = (($154) + 6|0);
     $156 = HEAP16[$155>>1]|0;
     $157 = $156&65535;
     $158 = $157 & 1;
     $159 = ($158|0)!=(0);
     if ($159) {
      break;
     }
     $160 = HEAP32[$pSrc>>2]|0;
     $161 = HEAP32[$160>>2]|0;
     $162 = ($161|0)!=(1);
     if ($162) {
      break;
     }
     $163 = HEAP32[$pSub>>2]|0;
     HEAP32[$pSub1>>2] = $163;
     while(1) {
      $164 = HEAP32[$pSub1>>2]|0;
      $165 = ($164|0)!=(0|0);
      if (!($165)) {
       break;
      }
      $166 = HEAP32[$pSub1>>2]|0;
      $167 = (($166) + 6|0);
      $168 = HEAP16[$167>>1]|0;
      $169 = $168&65535;
      $170 = $169 & 5;
      $171 = ($170|0)!=(0);
      if ($171) {
       label = 58;
       break;
      }
      $172 = HEAP32[$pSub1>>2]|0;
      $173 = (($172) + 52|0);
      $174 = HEAP32[$173>>2]|0;
      $175 = ($174|0)!=(0|0);
      if ($175) {
       $176 = HEAP32[$pSub1>>2]|0;
       $177 = (($176) + 4|0);
       $178 = HEAP8[$177>>0]|0;
       $179 = $178&255;
       $180 = ($179|0)!=(116);
       if ($180) {
        label = 58;
        break;
       }
      }
      $181 = HEAP32[$pSub1>>2]|0;
      $182 = (($181) + 32|0);
      $183 = HEAP32[$182>>2]|0;
      $184 = HEAP32[$183>>2]|0;
      $185 = ($184|0)<(1);
      if ($185) {
       label = 58;
       break;
      }
      $186 = HEAP32[$pSub>>2]|0;
      $187 = HEAP32[$186>>2]|0;
      $188 = HEAP32[$187>>2]|0;
      $189 = HEAP32[$pSub1>>2]|0;
      $190 = HEAP32[$189>>2]|0;
      $191 = HEAP32[$190>>2]|0;
      $192 = ($188|0)!=($191|0);
      if ($192) {
       label = 58;
       break;
      }
      $193 = HEAP32[$pSub1>>2]|0;
      $194 = (($193) + 52|0);
      $195 = HEAP32[$194>>2]|0;
      HEAP32[$pSub1>>2] = $195;
     }
     if ((label|0) == 58) {
      HEAP32[$0>>2] = 0;
      $614 = HEAP32[$0>>2]|0;
      STACKTOP = sp;return ($614|0);
     }
     $196 = HEAP32[$2>>2]|0;
     $197 = (($196) + 48|0);
     $198 = HEAP32[$197>>2]|0;
     $199 = ($198|0)!=(0|0);
     if (!($199)) {
      break L78;
     }
     HEAP32[$ii>>2] = 0;
     while(1) {
      $200 = HEAP32[$ii>>2]|0;
      $201 = HEAP32[$2>>2]|0;
      $202 = (($201) + 48|0);
      $203 = HEAP32[$202>>2]|0;
      $204 = HEAP32[$203>>2]|0;
      $205 = ($200|0)<($204|0);
      if (!($205)) {
       break L78;
      }
      $206 = HEAP32[$ii>>2]|0;
      $207 = HEAP32[$2>>2]|0;
      $208 = (($207) + 48|0);
      $209 = HEAP32[$208>>2]|0;
      $210 = (($209) + 4|0);
      $211 = HEAP32[$210>>2]|0;
      $212 = (($211) + (($206*20)|0)|0);
      $213 = (($212) + 16|0);
      $214 = HEAP16[$213>>1]|0;
      $215 = $214&65535;
      $216 = ($215|0)==(0);
      if ($216) {
       break;
      }
      $217 = HEAP32[$ii>>2]|0;
      $218 = (($217) + 1)|0;
      HEAP32[$ii>>2] = $218;
     }
     HEAP32[$0>>2] = 0;
     $614 = HEAP32[$0>>2]|0;
     STACKTOP = sp;return ($614|0);
    }
   } while(0);
   HEAP32[$0>>2] = 0;
   $614 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($614|0);
  }
 } while(0);
 $219 = HEAP32[$pSubitem>>2]|0;
 $220 = (($219) + 8|0);
 $221 = HEAP32[$220>>2]|0;
 $222 = HEAP32[$1>>2]|0;
 $223 = (($222) + 492|0);
 HEAP32[$223>>2] = $221;
 $224 = HEAP32[$1>>2]|0;
 (_sqlite3AuthCheck($224,21,0,0,0)|0);
 $225 = HEAP32[$zSavedAuthContext>>2]|0;
 $226 = HEAP32[$1>>2]|0;
 $227 = (($226) + 492|0);
 HEAP32[$227>>2] = $225;
 $228 = HEAP32[$pSub>>2]|0;
 $229 = (($228) + 52|0);
 $230 = HEAP32[$229>>2]|0;
 HEAP32[$pSub>>2] = $230;
 while(1) {
  $231 = HEAP32[$pSub>>2]|0;
  $232 = ($231|0)!=(0|0);
  if (!($232)) {
   break;
  }
  $233 = HEAP32[$2>>2]|0;
  $234 = (($233) + 48|0);
  $235 = HEAP32[$234>>2]|0;
  HEAP32[$pOrderBy>>2] = $235;
  $236 = HEAP32[$2>>2]|0;
  $237 = (($236) + 60|0);
  $238 = HEAP32[$237>>2]|0;
  HEAP32[$pLimit>>2] = $238;
  $239 = HEAP32[$2>>2]|0;
  $240 = (($239) + 64|0);
  $241 = HEAP32[$240>>2]|0;
  HEAP32[$pOffset>>2] = $241;
  $242 = HEAP32[$2>>2]|0;
  $243 = (($242) + 52|0);
  $244 = HEAP32[$243>>2]|0;
  HEAP32[$pPrior>>2] = $244;
  $245 = HEAP32[$2>>2]|0;
  $246 = (($245) + 48|0);
  HEAP32[$246>>2] = 0;
  $247 = HEAP32[$2>>2]|0;
  $248 = (($247) + 32|0);
  HEAP32[$248>>2] = 0;
  $249 = HEAP32[$2>>2]|0;
  $250 = (($249) + 52|0);
  HEAP32[$250>>2] = 0;
  $251 = HEAP32[$2>>2]|0;
  $252 = (($251) + 60|0);
  HEAP32[$252>>2] = 0;
  $253 = HEAP32[$2>>2]|0;
  $254 = (($253) + 64|0);
  HEAP32[$254>>2] = 0;
  $255 = HEAP32[$db>>2]|0;
  $256 = HEAP32[$2>>2]|0;
  $257 = (_sqlite3SelectDup($255,$256,0)|0);
  HEAP32[$pNew>>2] = $257;
  $258 = HEAP32[$pOffset>>2]|0;
  $259 = HEAP32[$2>>2]|0;
  $260 = (($259) + 64|0);
  HEAP32[$260>>2] = $258;
  $261 = HEAP32[$pLimit>>2]|0;
  $262 = HEAP32[$2>>2]|0;
  $263 = (($262) + 60|0);
  HEAP32[$263>>2] = $261;
  $264 = HEAP32[$pOrderBy>>2]|0;
  $265 = HEAP32[$2>>2]|0;
  $266 = (($265) + 48|0);
  HEAP32[$266>>2] = $264;
  $267 = HEAP32[$pSrc>>2]|0;
  $268 = HEAP32[$2>>2]|0;
  $269 = (($268) + 32|0);
  HEAP32[$269>>2] = $267;
  $270 = HEAP32[$2>>2]|0;
  $271 = (($270) + 4|0);
  HEAP8[$271>>0] = 116;
  $272 = HEAP32[$pNew>>2]|0;
  $273 = ($272|0)==(0|0);
  $274 = HEAP32[$pPrior>>2]|0;
  if ($273) {
   $275 = HEAP32[$2>>2]|0;
   $276 = (($275) + 52|0);
   HEAP32[$276>>2] = $274;
  } else {
   $277 = HEAP32[$pNew>>2]|0;
   $278 = (($277) + 52|0);
   HEAP32[$278>>2] = $274;
   $279 = HEAP32[$pPrior>>2]|0;
   $280 = ($279|0)!=(0|0);
   if ($280) {
    $281 = HEAP32[$pNew>>2]|0;
    $282 = HEAP32[$pPrior>>2]|0;
    $283 = (($282) + 56|0);
    HEAP32[$283>>2] = $281;
   }
   $284 = HEAP32[$2>>2]|0;
   $285 = HEAP32[$pNew>>2]|0;
   $286 = (($285) + 56|0);
   HEAP32[$286>>2] = $284;
   $287 = HEAP32[$pNew>>2]|0;
   $288 = HEAP32[$2>>2]|0;
   $289 = (($288) + 52|0);
   HEAP32[$289>>2] = $287;
  }
  $290 = HEAP32[$db>>2]|0;
  $291 = (($290) + 64|0);
  $292 = HEAP8[$291>>0]|0;
  $293 = ($292<<24>>24)!=(0);
  if ($293) {
   label = 74;
   break;
  }
  $294 = HEAP32[$pSub>>2]|0;
  $295 = (($294) + 52|0);
  $296 = HEAP32[$295>>2]|0;
  HEAP32[$pSub>>2] = $296;
 }
 if ((label|0) == 74) {
  HEAP32[$0>>2] = 1;
  $614 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($614|0);
 }
 $297 = HEAP32[$pSubitem>>2]|0;
 $298 = (($297) + 20|0);
 $299 = HEAP32[$298>>2]|0;
 HEAP32[$pSub1>>2] = $299;
 HEAP32[$pSub>>2] = $299;
 $300 = HEAP32[$db>>2]|0;
 $301 = HEAP32[$pSubitem>>2]|0;
 $302 = (($301) + 4|0);
 $303 = HEAP32[$302>>2]|0;
 _sqlite3DbFree($300,$303);
 $304 = HEAP32[$db>>2]|0;
 $305 = HEAP32[$pSubitem>>2]|0;
 $306 = (($305) + 8|0);
 $307 = HEAP32[$306>>2]|0;
 _sqlite3DbFree($304,$307);
 $308 = HEAP32[$db>>2]|0;
 $309 = HEAP32[$pSubitem>>2]|0;
 $310 = (($309) + 12|0);
 $311 = HEAP32[$310>>2]|0;
 _sqlite3DbFree($308,$311);
 $312 = HEAP32[$pSubitem>>2]|0;
 $313 = (($312) + 4|0);
 HEAP32[$313>>2] = 0;
 $314 = HEAP32[$pSubitem>>2]|0;
 $315 = (($314) + 8|0);
 HEAP32[$315>>2] = 0;
 $316 = HEAP32[$pSubitem>>2]|0;
 $317 = (($316) + 12|0);
 HEAP32[$317>>2] = 0;
 $318 = HEAP32[$pSubitem>>2]|0;
 $319 = (($318) + 20|0);
 HEAP32[$319>>2] = 0;
 $320 = HEAP32[$pSubitem>>2]|0;
 $321 = (($320) + 16|0);
 $322 = HEAP32[$321>>2]|0;
 $323 = ($322|0)!=(0|0);
 if ($323) {
  $324 = HEAP32[$pSubitem>>2]|0;
  $325 = (($324) + 16|0);
  $326 = HEAP32[$325>>2]|0;
  HEAP32[$pTabToDel>>2] = $326;
  $327 = HEAP32[$pTabToDel>>2]|0;
  $328 = (($327) + 40|0);
  $329 = HEAP16[$328>>1]|0;
  $330 = $329&65535;
  $331 = ($330|0)==(1);
  if ($331) {
   $332 = HEAP32[$1>>2]|0;
   $333 = (($332) + 408|0);
   $334 = HEAP32[$333>>2]|0;
   $335 = ($334|0)!=(0|0);
   $336 = HEAP32[$1>>2]|0;
   if ($335) {
    $337 = (($336) + 408|0);
    $338 = HEAP32[$337>>2]|0;
    $339 = $338;
   } else {
    $339 = $336;
   }
   HEAP32[$pToplevel>>2] = $339;
   $340 = HEAP32[$pToplevel>>2]|0;
   $341 = (($340) + 524|0);
   $342 = HEAP32[$341>>2]|0;
   $343 = HEAP32[$pTabToDel>>2]|0;
   $344 = (($343) + 72|0);
   HEAP32[$344>>2] = $342;
   $345 = HEAP32[$pTabToDel>>2]|0;
   $346 = HEAP32[$pToplevel>>2]|0;
   $347 = (($346) + 524|0);
   HEAP32[$347>>2] = $345;
  } else {
   $348 = HEAP32[$pTabToDel>>2]|0;
   $349 = (($348) + 40|0);
   $350 = HEAP16[$349>>1]|0;
   $351 = (($350) + -1)<<16>>16;
   HEAP16[$349>>1] = $351;
  }
  $352 = HEAP32[$pSubitem>>2]|0;
  $353 = (($352) + 16|0);
  HEAP32[$353>>2] = 0;
 }
 $354 = HEAP32[$2>>2]|0;
 HEAP32[$pParent>>2] = $354;
 while(1) {
  $355 = HEAP32[$pParent>>2]|0;
  $356 = ($355|0)!=(0|0);
  if (!($356)) {
   break;
  }
  HEAP8[$jointype>>0] = 0;
  $357 = HEAP32[$pSub>>2]|0;
  $358 = (($357) + 32|0);
  $359 = HEAP32[$358>>2]|0;
  HEAP32[$pSubSrc>>2] = $359;
  $360 = HEAP32[$pSubSrc>>2]|0;
  $361 = HEAP32[$360>>2]|0;
  HEAP32[$nSubSrc>>2] = $361;
  $362 = HEAP32[$pParent>>2]|0;
  $363 = (($362) + 32|0);
  $364 = HEAP32[$363>>2]|0;
  HEAP32[$pSrc>>2] = $364;
  $365 = HEAP32[$pSrc>>2]|0;
  $366 = ($365|0)!=(0|0);
  if ($366) {
   $367 = HEAP32[$pSubitem>>2]|0;
   $368 = (($367) + 36|0);
   $369 = HEAP8[$368>>0]|0;
   HEAP8[$jointype>>0] = $369;
  } else {
   $370 = HEAP32[$db>>2]|0;
   $371 = (_sqlite3SrcListAppend($370,0,0,0)|0);
   $372 = HEAP32[$pParent>>2]|0;
   $373 = (($372) + 32|0);
   HEAP32[$373>>2] = $371;
   HEAP32[$pSrc>>2] = $371;
   $374 = HEAP32[$pSrc>>2]|0;
   $375 = ($374|0)==(0|0);
   if ($375) {
    break;
   }
  }
  $376 = HEAP32[$nSubSrc>>2]|0;
  $377 = ($376|0)>(1);
  if ($377) {
   $378 = HEAP32[$db>>2]|0;
   $379 = HEAP32[$pSrc>>2]|0;
   $380 = HEAP32[$nSubSrc>>2]|0;
   $381 = (($380) - 1)|0;
   $382 = HEAP32[$3>>2]|0;
   $383 = (($382) + 1)|0;
   $384 = (_sqlite3SrcListEnlarge($378,$379,$381,$383)|0);
   HEAP32[$pSrc>>2] = $384;
   $385 = HEAP32[$pParent>>2]|0;
   $386 = (($385) + 32|0);
   HEAP32[$386>>2] = $384;
   $387 = HEAP32[$db>>2]|0;
   $388 = (($387) + 64|0);
   $389 = HEAP8[$388>>0]|0;
   $390 = ($389<<24>>24)!=(0);
   if ($390) {
    break;
   }
  }
  HEAP32[$i>>2] = 0;
  while(1) {
   $391 = HEAP32[$i>>2]|0;
   $392 = HEAP32[$nSubSrc>>2]|0;
   $393 = ($391|0)<($392|0);
   if (!($393)) {
    break;
   }
   $394 = HEAP32[$db>>2]|0;
   $395 = HEAP32[$i>>2]|0;
   $396 = HEAP32[$3>>2]|0;
   $397 = (($395) + ($396))|0;
   $398 = HEAP32[$pSrc>>2]|0;
   $399 = (($398) + 8|0);
   $400 = (($399) + (($397*72)|0)|0);
   $401 = (($400) + 48|0);
   $402 = HEAP32[$401>>2]|0;
   _sqlite3IdListDelete($394,$402);
   $403 = HEAP32[$i>>2]|0;
   $404 = HEAP32[$3>>2]|0;
   $405 = (($403) + ($404))|0;
   $406 = HEAP32[$pSrc>>2]|0;
   $407 = (($406) + 8|0);
   $408 = (($407) + (($405*72)|0)|0);
   $409 = HEAP32[$i>>2]|0;
   $410 = HEAP32[$pSubSrc>>2]|0;
   $411 = (($410) + 8|0);
   $412 = (($411) + (($409*72)|0)|0);
   dest=$408+0|0; src=$412+0|0; stop=dest+72|0; do { HEAP32[dest>>2]=HEAP32[src>>2]|0; dest=dest+4|0; src=src+4|0; } while ((dest|0) < (stop|0));
   $413 = HEAP32[$i>>2]|0;
   $414 = HEAP32[$pSubSrc>>2]|0;
   $415 = (($414) + 8|0);
   $416 = (($415) + (($413*72)|0)|0);
   dest=$416+0|0; stop=dest+72|0; do { HEAP32[dest>>2]=0|0; dest=dest+4|0; } while ((dest|0) < (stop|0));
   $417 = HEAP32[$i>>2]|0;
   $418 = (($417) + 1)|0;
   HEAP32[$i>>2] = $418;
  }
  $419 = HEAP8[$jointype>>0]|0;
  $420 = HEAP32[$3>>2]|0;
  $421 = HEAP32[$pSrc>>2]|0;
  $422 = (($421) + 8|0);
  $423 = (($422) + (($420*72)|0)|0);
  $424 = (($423) + 36|0);
  HEAP8[$424>>0] = $419;
  $425 = HEAP32[$pParent>>2]|0;
  $426 = HEAP32[$425>>2]|0;
  HEAP32[$pList>>2] = $426;
  HEAP32[$i>>2] = 0;
  while(1) {
   $427 = HEAP32[$i>>2]|0;
   $428 = HEAP32[$pList>>2]|0;
   $429 = HEAP32[$428>>2]|0;
   $430 = ($427|0)<($429|0);
   if (!($430)) {
    break;
   }
   $431 = HEAP32[$i>>2]|0;
   $432 = HEAP32[$pList>>2]|0;
   $433 = (($432) + 4|0);
   $434 = HEAP32[$433>>2]|0;
   $435 = (($434) + (($431*20)|0)|0);
   $436 = (($435) + 4|0);
   $437 = HEAP32[$436>>2]|0;
   $438 = ($437|0)==(0|0);
   if ($438) {
    $439 = HEAP32[$db>>2]|0;
    $440 = HEAP32[$i>>2]|0;
    $441 = HEAP32[$pList>>2]|0;
    $442 = (($441) + 4|0);
    $443 = HEAP32[$442>>2]|0;
    $444 = (($443) + (($440*20)|0)|0);
    $445 = (($444) + 8|0);
    $446 = HEAP32[$445>>2]|0;
    $447 = (_sqlite3DbStrDup($439,$446)|0);
    HEAP32[$zName>>2] = $447;
    $448 = HEAP32[$zName>>2]|0;
    (_sqlite3Dequote($448)|0);
    $449 = HEAP32[$zName>>2]|0;
    $450 = HEAP32[$i>>2]|0;
    $451 = HEAP32[$pList>>2]|0;
    $452 = (($451) + 4|0);
    $453 = HEAP32[$452>>2]|0;
    $454 = (($453) + (($450*20)|0)|0);
    $455 = (($454) + 4|0);
    HEAP32[$455>>2] = $449;
   }
   $456 = HEAP32[$i>>2]|0;
   $457 = (($456) + 1)|0;
   HEAP32[$i>>2] = $457;
  }
  $458 = HEAP32[$db>>2]|0;
  $459 = HEAP32[$pParent>>2]|0;
  $460 = HEAP32[$459>>2]|0;
  $461 = HEAP32[$iParent>>2]|0;
  $462 = HEAP32[$pSub>>2]|0;
  $463 = HEAP32[$462>>2]|0;
  _substExprList($458,$460,$461,$463);
  $464 = HEAP32[$4>>2]|0;
  $465 = ($464|0)!=(0);
  if ($465) {
   $466 = HEAP32[$db>>2]|0;
   $467 = HEAP32[$pParent>>2]|0;
   $468 = (($467) + 40|0);
   $469 = HEAP32[$468>>2]|0;
   $470 = HEAP32[$iParent>>2]|0;
   $471 = HEAP32[$pSub>>2]|0;
   $472 = HEAP32[$471>>2]|0;
   _substExprList($466,$469,$470,$472);
   $473 = HEAP32[$db>>2]|0;
   $474 = HEAP32[$pParent>>2]|0;
   $475 = (($474) + 44|0);
   $476 = HEAP32[$475>>2]|0;
   $477 = HEAP32[$iParent>>2]|0;
   $478 = HEAP32[$pSub>>2]|0;
   $479 = HEAP32[$478>>2]|0;
   $480 = (_substExpr($473,$476,$477,$479)|0);
   $481 = HEAP32[$pParent>>2]|0;
   $482 = (($481) + 44|0);
   HEAP32[$482>>2] = $480;
  }
  $483 = HEAP32[$pSub>>2]|0;
  $484 = (($483) + 48|0);
  $485 = HEAP32[$484>>2]|0;
  $486 = ($485|0)!=(0|0);
  do {
   if ($486) {
    $487 = HEAP32[$pSub>>2]|0;
    $488 = (($487) + 48|0);
    $489 = HEAP32[$488>>2]|0;
    HEAP32[$pOrderBy1>>2] = $489;
    HEAP32[$i>>2] = 0;
    while(1) {
     $490 = HEAP32[$i>>2]|0;
     $491 = HEAP32[$pOrderBy1>>2]|0;
     $492 = HEAP32[$491>>2]|0;
     $493 = ($490|0)<($492|0);
     if (!($493)) {
      break;
     }
     $494 = HEAP32[$i>>2]|0;
     $495 = HEAP32[$pOrderBy1>>2]|0;
     $496 = (($495) + 4|0);
     $497 = HEAP32[$496>>2]|0;
     $498 = (($497) + (($494*20)|0)|0);
     $499 = (($498) + 16|0);
     HEAP16[$499>>1] = 0;
     $500 = HEAP32[$i>>2]|0;
     $501 = (($500) + 1)|0;
     HEAP32[$i>>2] = $501;
    }
    $502 = HEAP32[$pOrderBy1>>2]|0;
    $503 = HEAP32[$pParent>>2]|0;
    $504 = (($503) + 48|0);
    HEAP32[$504>>2] = $502;
    $505 = HEAP32[$pSub>>2]|0;
    $506 = (($505) + 48|0);
    HEAP32[$506>>2] = 0;
   } else {
    $507 = HEAP32[$pParent>>2]|0;
    $508 = (($507) + 48|0);
    $509 = HEAP32[$508>>2]|0;
    $510 = ($509|0)!=(0|0);
    if (!($510)) {
     break;
    }
    $511 = HEAP32[$db>>2]|0;
    $512 = HEAP32[$pParent>>2]|0;
    $513 = (($512) + 48|0);
    $514 = HEAP32[$513>>2]|0;
    $515 = HEAP32[$iParent>>2]|0;
    $516 = HEAP32[$pSub>>2]|0;
    $517 = HEAP32[$516>>2]|0;
    _substExprList($511,$514,$515,$517);
   }
  } while(0);
  $518 = HEAP32[$pSub>>2]|0;
  $519 = (($518) + 36|0);
  $520 = HEAP32[$519>>2]|0;
  $521 = ($520|0)!=(0|0);
  if ($521) {
   $522 = HEAP32[$db>>2]|0;
   $523 = HEAP32[$pSub>>2]|0;
   $524 = (($523) + 36|0);
   $525 = HEAP32[$524>>2]|0;
   $526 = (_sqlite3ExprDup($522,$525,0)|0);
   HEAP32[$pWhere>>2] = $526;
  } else {
   HEAP32[$pWhere>>2] = 0;
  }
  $527 = HEAP32[$5>>2]|0;
  $528 = ($527|0)!=(0);
  if ($528) {
   $529 = HEAP32[$pParent>>2]|0;
   $530 = (($529) + 36|0);
   $531 = HEAP32[$530>>2]|0;
   $532 = HEAP32[$pParent>>2]|0;
   $533 = (($532) + 44|0);
   HEAP32[$533>>2] = $531;
   $534 = HEAP32[$pWhere>>2]|0;
   $535 = HEAP32[$pParent>>2]|0;
   $536 = (($535) + 36|0);
   HEAP32[$536>>2] = $534;
   $537 = HEAP32[$db>>2]|0;
   $538 = HEAP32[$pParent>>2]|0;
   $539 = (($538) + 44|0);
   $540 = HEAP32[$539>>2]|0;
   $541 = HEAP32[$iParent>>2]|0;
   $542 = HEAP32[$pSub>>2]|0;
   $543 = HEAP32[$542>>2]|0;
   $544 = (_substExpr($537,$540,$541,$543)|0);
   $545 = HEAP32[$pParent>>2]|0;
   $546 = (($545) + 44|0);
   HEAP32[$546>>2] = $544;
   $547 = HEAP32[$db>>2]|0;
   $548 = HEAP32[$pParent>>2]|0;
   $549 = (($548) + 44|0);
   $550 = HEAP32[$549>>2]|0;
   $551 = HEAP32[$db>>2]|0;
   $552 = HEAP32[$pSub>>2]|0;
   $553 = (($552) + 44|0);
   $554 = HEAP32[$553>>2]|0;
   $555 = (_sqlite3ExprDup($551,$554,0)|0);
   $556 = (_sqlite3ExprAnd($547,$550,$555)|0);
   $557 = HEAP32[$pParent>>2]|0;
   $558 = (($557) + 44|0);
   HEAP32[$558>>2] = $556;
   $559 = HEAP32[$db>>2]|0;
   $560 = HEAP32[$pSub>>2]|0;
   $561 = (($560) + 40|0);
   $562 = HEAP32[$561>>2]|0;
   $563 = (_sqlite3ExprListDup($559,$562,0)|0);
   $564 = HEAP32[$pParent>>2]|0;
   $565 = (($564) + 40|0);
   HEAP32[$565>>2] = $563;
  } else {
   $566 = HEAP32[$db>>2]|0;
   $567 = HEAP32[$pParent>>2]|0;
   $568 = (($567) + 36|0);
   $569 = HEAP32[$568>>2]|0;
   $570 = HEAP32[$iParent>>2]|0;
   $571 = HEAP32[$pSub>>2]|0;
   $572 = HEAP32[$571>>2]|0;
   $573 = (_substExpr($566,$569,$570,$572)|0);
   $574 = HEAP32[$pParent>>2]|0;
   $575 = (($574) + 36|0);
   HEAP32[$575>>2] = $573;
   $576 = HEAP32[$db>>2]|0;
   $577 = HEAP32[$pParent>>2]|0;
   $578 = (($577) + 36|0);
   $579 = HEAP32[$578>>2]|0;
   $580 = HEAP32[$pWhere>>2]|0;
   $581 = (_sqlite3ExprAnd($576,$579,$580)|0);
   $582 = HEAP32[$pParent>>2]|0;
   $583 = (($582) + 36|0);
   HEAP32[$583>>2] = $581;
  }
  $584 = HEAP32[$pSub>>2]|0;
  $585 = (($584) + 6|0);
  $586 = HEAP16[$585>>1]|0;
  $587 = $586&65535;
  $588 = $587 & 1;
  $589 = HEAP32[$pParent>>2]|0;
  $590 = (($589) + 6|0);
  $591 = HEAP16[$590>>1]|0;
  $592 = $591&65535;
  $593 = $592 | $588;
  $594 = $593&65535;
  HEAP16[$590>>1] = $594;
  $595 = HEAP32[$pSub>>2]|0;
  $596 = (($595) + 60|0);
  $597 = HEAP32[$596>>2]|0;
  $598 = ($597|0)!=(0|0);
  if ($598) {
   $599 = HEAP32[$pSub>>2]|0;
   $600 = (($599) + 60|0);
   $601 = HEAP32[$600>>2]|0;
   $602 = HEAP32[$pParent>>2]|0;
   $603 = (($602) + 60|0);
   HEAP32[$603>>2] = $601;
   $604 = HEAP32[$pSub>>2]|0;
   $605 = (($604) + 60|0);
   HEAP32[$605>>2] = 0;
  }
  $606 = HEAP32[$pParent>>2]|0;
  $607 = (($606) + 52|0);
  $608 = HEAP32[$607>>2]|0;
  HEAP32[$pParent>>2] = $608;
  $609 = HEAP32[$pSub>>2]|0;
  $610 = (($609) + 52|0);
  $611 = HEAP32[$610>>2]|0;
  HEAP32[$pSub>>2] = $611;
 }
 $612 = HEAP32[$db>>2]|0;
 $613 = HEAP32[$pSub1>>2]|0;
 _sqlite3SelectDelete($612,$613);
 HEAP32[$0>>2] = 1;
 $614 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($614|0);
}
function _sqlite3ClearTempRegCache($pParse) {
 $pParse = $pParse|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 HEAP32[$0>>2] = $pParse;
 $1 = HEAP32[$0>>2]|0;
 $2 = (($1) + 19|0);
 HEAP8[$2>>0] = 0;
 $3 = HEAP32[$0>>2]|0;
 $4 = (($3) + 56|0);
 HEAP32[$4>>2] = 0;
 STACKTOP = sp;return;
}
function _multiSelect($pParse,$p,$pDest) {
 $pParse = $pParse|0;
 $p = $p|0;
 $pDest = $pDest|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0;
 var $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0, $188 = 0;
 var $189 = 0, $19 = 0, $190 = 0, $191 = 0, $192 = 0, $193 = 0, $194 = 0, $195 = 0, $196 = 0, $197 = 0, $198 = 0, $199 = 0, $2 = 0, $20 = 0, $200 = 0, $201 = 0, $202 = 0, $203 = 0, $204 = 0, $205 = 0;
 var $206 = 0, $207 = 0, $208 = 0, $209 = 0, $21 = 0, $210 = 0, $211 = 0, $212 = 0, $213 = 0, $214 = 0, $215 = 0, $216 = 0, $217 = 0, $218 = 0, $219 = 0, $22 = 0, $220 = 0, $221 = 0, $222 = 0, $223 = 0;
 var $224 = 0, $225 = 0, $226 = 0, $227 = 0, $228 = 0, $229 = 0, $23 = 0, $230 = 0, $231 = 0, $232 = 0, $233 = 0, $234 = 0, $235 = 0, $236 = 0, $237 = 0, $238 = 0, $239 = 0, $24 = 0, $240 = 0, $241 = 0;
 var $242 = 0, $243 = 0, $244 = 0, $245 = 0, $246 = 0, $247 = 0, $248 = 0, $249 = 0, $25 = 0, $250 = 0, $251 = 0, $252 = 0, $253 = 0, $254 = 0, $255 = 0, $256 = 0, $257 = 0, $258 = 0, $259 = 0, $26 = 0;
 var $260 = 0, $261 = 0, $262 = 0, $263 = 0, $264 = 0, $265 = 0, $266 = 0, $267 = 0, $268 = 0, $269 = 0, $27 = 0, $270 = 0, $271 = 0, $272 = 0, $273 = 0, $274 = 0, $275 = 0, $276 = 0, $277 = 0, $278 = 0;
 var $279 = 0, $28 = 0, $280 = 0, $281 = 0, $282 = 0, $283 = 0, $284 = 0, $285 = 0, $286 = 0, $287 = 0, $288 = 0, $289 = 0, $29 = 0, $290 = 0, $291 = 0, $292 = 0, $293 = 0, $294 = 0, $295 = 0, $296 = 0;
 var $297 = 0, $298 = 0, $299 = 0, $3 = 0, $30 = 0, $300 = 0, $301 = 0, $302 = 0, $303 = 0, $304 = 0, $305 = 0, $306 = 0, $307 = 0, $308 = 0, $309 = 0, $31 = 0, $310 = 0, $311 = 0, $312 = 0, $313 = 0;
 var $314 = 0, $315 = 0, $316 = 0, $317 = 0, $318 = 0, $319 = 0, $32 = 0, $320 = 0, $321 = 0, $322 = 0, $323 = 0, $324 = 0, $325 = 0, $326 = 0, $327 = 0, $328 = 0, $329 = 0, $33 = 0, $330 = 0, $331 = 0;
 var $332 = 0, $333 = 0, $334 = 0, $335 = 0, $336 = 0, $337 = 0, $338 = 0, $339 = 0, $34 = 0, $340 = 0, $341 = 0, $342 = 0, $343 = 0, $344 = 0, $345 = 0, $346 = 0, $347 = 0, $348 = 0, $349 = 0, $35 = 0;
 var $350 = 0, $351 = 0, $352 = 0, $353 = 0, $354 = 0, $355 = 0, $356 = 0, $357 = 0, $358 = 0, $359 = 0, $36 = 0, $360 = 0, $361 = 0, $362 = 0, $363 = 0, $364 = 0, $365 = 0, $366 = 0, $367 = 0, $368 = 0;
 var $369 = 0, $37 = 0, $370 = 0, $371 = 0, $372 = 0, $373 = 0, $374 = 0, $375 = 0, $376 = 0, $377 = 0, $378 = 0, $379 = 0, $38 = 0, $380 = 0, $381 = 0, $382 = 0, $383 = 0, $384 = 0, $385 = 0, $386 = 0;
 var $387 = 0, $388 = 0, $389 = 0, $39 = 0, $390 = 0, $391 = 0, $392 = 0, $393 = 0, $394 = 0, $395 = 0, $396 = 0, $397 = 0, $398 = 0, $399 = 0, $4 = 0, $40 = 0, $400 = 0, $401 = 0, $402 = 0, $403 = 0;
 var $404 = 0, $405 = 0, $406 = 0, $407 = 0, $408 = 0, $409 = 0, $41 = 0, $410 = 0, $411 = 0, $412 = 0, $413 = 0, $414 = 0, $415 = 0, $416 = 0, $417 = 0, $418 = 0, $419 = 0, $42 = 0, $420 = 0, $421 = 0;
 var $422 = 0, $423 = 0, $424 = 0, $425 = 0, $426 = 0, $427 = 0, $428 = 0, $429 = 0, $43 = 0, $430 = 0, $431 = 0, $432 = 0, $433 = 0, $434 = 0, $435 = 0, $436 = 0, $437 = 0, $438 = 0, $439 = 0, $44 = 0;
 var $440 = 0, $441 = 0, $442 = 0, $443 = 0, $444 = 0, $445 = 0, $446 = 0, $447 = 0, $448 = 0, $449 = 0, $45 = 0, $450 = 0, $451 = 0, $452 = 0, $453 = 0, $454 = 0, $455 = 0, $456 = 0, $457 = 0, $458 = 0;
 var $459 = 0, $46 = 0, $460 = 0, $461 = 0, $462 = 0, $463 = 0, $464 = 0, $465 = 0, $466 = 0, $467 = 0, $468 = 0, $469 = 0, $47 = 0, $470 = 0, $471 = 0, $472 = 0, $473 = 0, $474 = 0, $475 = 0, $476 = 0;
 var $477 = 0, $478 = 0, $479 = 0, $48 = 0, $480 = 0, $481 = 0, $482 = 0, $483 = 0, $484 = 0, $485 = 0, $486 = 0, $487 = 0, $488 = 0, $489 = 0, $49 = 0, $490 = 0, $491 = 0, $492 = 0, $493 = 0, $494 = 0;
 var $495 = 0, $496 = 0, $497 = 0, $498 = 0, $499 = 0, $5 = 0, $50 = 0, $500 = 0, $501 = 0, $502 = 0, $503 = 0, $504 = 0, $505 = 0, $506 = 0, $507 = 0, $508 = 0, $509 = 0, $51 = 0, $510 = 0, $511 = 0;
 var $512 = 0, $513 = 0, $514 = 0, $515 = 0, $516 = 0, $517 = 0, $518 = 0, $519 = 0, $52 = 0, $520 = 0, $521 = 0, $522 = 0, $523 = 0, $524 = 0, $525 = 0, $526 = 0, $527 = 0, $528 = 0, $529 = 0, $53 = 0;
 var $530 = 0, $531 = 0, $532 = 0, $533 = 0, $534 = 0, $535 = 0, $536 = 0, $537 = 0, $538 = 0, $539 = 0, $54 = 0, $540 = 0, $541 = 0, $542 = 0, $543 = 0, $544 = 0, $545 = 0, $546 = 0, $547 = 0, $548 = 0;
 var $549 = 0, $55 = 0, $550 = 0, $551 = 0, $552 = 0, $553 = 0, $554 = 0, $555 = 0, $556 = 0, $557 = 0, $558 = 0, $559 = 0, $56 = 0, $560 = 0, $561 = 0, $562 = 0, $563 = 0, $564 = 0, $565 = 0, $566 = 0;
 var $567 = 0, $568 = 0, $569 = 0, $57 = 0, $570 = 0, $571 = 0, $572 = 0, $573 = 0, $574 = 0, $575 = 0, $576 = 0, $577 = 0, $578 = 0, $579 = 0, $58 = 0, $580 = 0, $581 = 0, $582 = 0, $583 = 0, $584 = 0;
 var $585 = 0, $586 = 0, $587 = 0, $588 = 0, $589 = 0, $59 = 0, $590 = 0, $591 = 0, $592 = 0, $593 = 0, $594 = 0, $595 = 0, $596 = 0, $597 = 0, $598 = 0, $599 = 0, $6 = 0, $60 = 0, $600 = 0, $601 = 0;
 var $602 = 0, $603 = 0, $604 = 0, $605 = 0, $606 = 0, $607 = 0, $608 = 0, $609 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0;
 var $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0;
 var $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $addr = 0, $addr1 = 0, $addr7 = 0, $addr9 = 0, $apColl = 0, $db = 0, $dest = 0, $i = 0, $iBreak = 0, $iBreak3 = 0;
 var $iCont = 0, $iCont2 = 0, $iStart = 0, $iStart4 = 0, $iSub1 = 0, $iSub2 = 0, $intersectdest = 0, $nCol = 0, $nLimit = 0, $op = 0, $or$cond = 0, $pDelete = 0, $pFirst = 0, $pFirst8 = 0, $pKeyInfo = 0, $pLimit = 0, $pLimit5 = 0, $pLoop = 0, $pOffset = 0, $pOffset6 = 0;
 var $pPrior = 0, $priorOp = 0, $r1 = 0, $rc = 0, $tab1 = 0, $tab2 = 0, $unionTab = 0, $uniondest = 0, $v = 0, $vararg_buffer7 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 224|0;
 $vararg_buffer7 = sp;
 $0 = sp + 212|0;
 $1 = sp + 208|0;
 $2 = sp + 204|0;
 $3 = sp + 200|0;
 $rc = sp + 196|0;
 $pPrior = sp + 192|0;
 $v = sp + 188|0;
 $dest = sp + 168|0;
 $pDelete = sp + 164|0;
 $db = sp + 160|0;
 $iSub1 = sp + 156|0;
 $iSub2 = sp + 152|0;
 $addr = sp + 148|0;
 $nLimit = sp + 144|0;
 $unionTab = sp + 140|0;
 $op = sp + 216|0;
 $priorOp = sp + 136|0;
 $pLimit = sp + 132|0;
 $pOffset = sp + 128|0;
 $addr1 = sp + 124|0;
 $uniondest = sp + 104|0;
 $iCont = sp + 100|0;
 $iBreak = sp + 96|0;
 $iStart = sp + 92|0;
 $pFirst = sp + 88|0;
 $tab1 = sp + 84|0;
 $tab2 = sp + 80|0;
 $iCont2 = sp + 76|0;
 $iBreak3 = sp + 72|0;
 $iStart4 = sp + 68|0;
 $pLimit5 = sp + 64|0;
 $pOffset6 = sp + 60|0;
 $addr7 = sp + 56|0;
 $intersectdest = sp + 36|0;
 $r1 = sp + 32|0;
 $pFirst8 = sp + 28|0;
 $i = sp + 24|0;
 $pKeyInfo = sp + 20|0;
 $pLoop = sp + 16|0;
 $apColl = sp + 12|0;
 $nCol = sp + 8|0;
 $addr9 = sp + 4|0;
 HEAP32[$1>>2] = $pParse;
 HEAP32[$2>>2] = $p;
 HEAP32[$3>>2] = $pDest;
 HEAP32[$rc>>2] = 0;
 HEAP32[$pDelete>>2] = 0;
 HEAP32[$iSub1>>2] = 0;
 HEAP32[$iSub2>>2] = 0;
 $4 = HEAP32[$1>>2]|0;
 $5 = HEAP32[$4>>2]|0;
 HEAP32[$db>>2] = $5;
 $6 = HEAP32[$2>>2]|0;
 $7 = (($6) + 52|0);
 $8 = HEAP32[$7>>2]|0;
 HEAP32[$pPrior>>2] = $8;
 $9 = HEAP32[$3>>2]|0;
 ;HEAP32[$dest+0>>2]=HEAP32[$9+0>>2]|0;HEAP32[$dest+4>>2]=HEAP32[$9+4>>2]|0;HEAP32[$dest+8>>2]=HEAP32[$9+8>>2]|0;HEAP32[$dest+12>>2]=HEAP32[$9+12>>2]|0;HEAP32[$dest+16>>2]=HEAP32[$9+16>>2]|0;
 $10 = HEAP32[$pPrior>>2]|0;
 $11 = (($10) + 48|0);
 $12 = HEAP32[$11>>2]|0;
 $13 = ($12|0)!=(0|0);
 L1: do {
  if ($13) {
   $14 = HEAP32[$1>>2]|0;
   $15 = HEAP32[$2>>2]|0;
   $16 = (($15) + 4|0);
   $17 = HEAP8[$16>>0]|0;
   $18 = $17&255;
   $19 = (_selectOpName($18)|0);
   HEAP32[$vararg_buffer7>>2] = $19;
   _sqlite3ErrorMsg($14,25048,$vararg_buffer7);
   HEAP32[$rc>>2] = 1;
  } else {
   $20 = HEAP32[$pPrior>>2]|0;
   $21 = (($20) + 60|0);
   $22 = HEAP32[$21>>2]|0;
   $23 = ($22|0)!=(0|0);
   $24 = HEAP32[$1>>2]|0;
   if ($23) {
    $25 = HEAP32[$2>>2]|0;
    $26 = (($25) + 4|0);
    $27 = HEAP8[$26>>0]|0;
    $28 = $27&255;
    $29 = (_selectOpName($28)|0);
    HEAP32[$vararg_buffer7>>2] = $29;
    _sqlite3ErrorMsg($24,25096,$vararg_buffer7);
    HEAP32[$rc>>2] = 1;
    break;
   }
   $30 = (_sqlite3GetVdbe($24)|0);
   HEAP32[$v>>2] = $30;
   $31 = HEAP8[$dest>>0]|0;
   $32 = $31&255;
   $33 = ($32|0)==(12);
   if ($33) {
    $34 = HEAP32[$v>>2]|0;
    $35 = (($dest) + 4|0);
    $36 = HEAP32[$35>>2]|0;
    $37 = HEAP32[$2>>2]|0;
    $38 = HEAP32[$37>>2]|0;
    $39 = HEAP32[$38>>2]|0;
    (_sqlite3VdbeAddOp2($34,57,$36,$39)|0);
    $40 = HEAP32[$v>>2]|0;
    _sqlite3VdbeChangeP5($40,8);
    HEAP8[$dest>>0] = 14;
   }
   $41 = HEAP32[$2>>2]|0;
   $42 = HEAP32[$41>>2]|0;
   $43 = HEAP32[$42>>2]|0;
   $44 = HEAP32[$pPrior>>2]|0;
   $45 = HEAP32[$44>>2]|0;
   $46 = HEAP32[$45>>2]|0;
   $47 = ($43|0)!=($46|0);
   $48 = HEAP32[$2>>2]|0;
   $49 = (($48) + 6|0);
   $50 = HEAP16[$49>>1]|0;
   $51 = $50&65535;
   if ($47) {
    $52 = $51 & 128;
    $53 = ($52|0)!=(0);
    $54 = HEAP32[$1>>2]|0;
    if ($53) {
     _sqlite3ErrorMsg($54,25144,$vararg_buffer7);
    } else {
     $55 = HEAP32[$2>>2]|0;
     $56 = (($55) + 4|0);
     $57 = HEAP8[$56>>0]|0;
     $58 = $57&255;
     $59 = (_selectOpName($58)|0);
     HEAP32[$vararg_buffer7>>2] = $59;
     _sqlite3ErrorMsg($54,25192,$vararg_buffer7);
    }
    HEAP32[$rc>>2] = 1;
    break;
   }
   $60 = $51 & 2048;
   $61 = ($60|0)!=(0);
   do {
    if ($61) {
     $62 = HEAP32[$1>>2]|0;
     $63 = HEAP32[$2>>2]|0;
     _generateWithRecursiveQuery($62,$63,$dest);
    } else {
     $64 = HEAP32[$2>>2]|0;
     $65 = (($64) + 48|0);
     $66 = HEAP32[$65>>2]|0;
     $67 = ($66|0)!=(0|0);
     if ($67) {
      $68 = HEAP32[$1>>2]|0;
      $69 = HEAP32[$2>>2]|0;
      $70 = HEAP32[$3>>2]|0;
      $71 = (_multiSelectOrderBy($68,$69,$70)|0);
      HEAP32[$0>>2] = $71;
      $609 = HEAP32[$0>>2]|0;
      STACKTOP = sp;return ($609|0);
     }
     $72 = HEAP32[$2>>2]|0;
     $73 = (($72) + 4|0);
     $74 = HEAP8[$73>>0]|0;
     $75 = $74&255;
     if ((($75|0) == 115) | (($75|0) == 117)) {
      HEAP8[$op>>0] = 0;
      HEAP32[$priorOp>>2] = 1;
      $203 = HEAP8[$dest>>0]|0;
      $204 = $203&255;
      $205 = HEAP32[$priorOp>>2]|0;
      $206 = ($204|0)==($205|0);
      if ($206) {
       $207 = (($dest) + 4|0);
       $208 = HEAP32[$207>>2]|0;
       HEAP32[$unionTab>>2] = $208;
      } else {
       $209 = HEAP32[$1>>2]|0;
       $210 = (($209) + 68|0);
       $211 = HEAP32[$210>>2]|0;
       $212 = (($211) + 1)|0;
       HEAP32[$210>>2] = $212;
       HEAP32[$unionTab>>2] = $211;
       $213 = HEAP32[$v>>2]|0;
       $214 = HEAP32[$unionTab>>2]|0;
       $215 = (_sqlite3VdbeAddOp2($213,57,$214,0)|0);
       HEAP32[$addr1>>2] = $215;
       $216 = HEAP32[$addr1>>2]|0;
       $217 = HEAP32[$2>>2]|0;
       $218 = (($217) + 16|0);
       HEAP32[$218>>2] = $216;
       $219 = HEAP32[$2>>2]|0;
       $220 = (_findRightmost($219)|0);
       $221 = (($220) + 6|0);
       $222 = HEAP16[$221>>1]|0;
       $223 = $222&65535;
       $224 = $223 | 8;
       $225 = $224&65535;
       HEAP16[$221>>1] = $225;
      }
      $226 = HEAP32[$priorOp>>2]|0;
      $227 = HEAP32[$unionTab>>2]|0;
      _sqlite3SelectDestInit($uniondest,$226,$227);
      $228 = HEAP32[$1>>2]|0;
      $229 = (($228) + 468|0);
      $230 = HEAP32[$229>>2]|0;
      HEAP32[$iSub1>>2] = $230;
      $231 = HEAP32[$1>>2]|0;
      $232 = HEAP32[$pPrior>>2]|0;
      $233 = (_sqlite3Select($231,$232,$uniondest)|0);
      HEAP32[$rc>>2] = $233;
      $234 = HEAP32[$rc>>2]|0;
      $235 = ($234|0)!=(0);
      if ($235) {
       break L1;
      }
      $236 = HEAP32[$2>>2]|0;
      $237 = (($236) + 4|0);
      $238 = HEAP8[$237>>0]|0;
      $239 = $238&255;
      $240 = ($239|0)==(117);
      if ($240) {
       HEAP8[$op>>0] = 2;
      } else {
       HEAP8[$op>>0] = 1;
      }
      $241 = HEAP32[$2>>2]|0;
      $242 = (($241) + 52|0);
      HEAP32[$242>>2] = 0;
      $243 = HEAP32[$2>>2]|0;
      $244 = (($243) + 60|0);
      $245 = HEAP32[$244>>2]|0;
      HEAP32[$pLimit>>2] = $245;
      $246 = HEAP32[$2>>2]|0;
      $247 = (($246) + 60|0);
      HEAP32[$247>>2] = 0;
      $248 = HEAP32[$2>>2]|0;
      $249 = (($248) + 64|0);
      $250 = HEAP32[$249>>2]|0;
      HEAP32[$pOffset>>2] = $250;
      $251 = HEAP32[$2>>2]|0;
      $252 = (($251) + 64|0);
      HEAP32[$252>>2] = 0;
      $253 = HEAP8[$op>>0]|0;
      HEAP8[$uniondest>>0] = $253;
      $254 = HEAP32[$1>>2]|0;
      $255 = (($254) + 468|0);
      $256 = HEAP32[$255>>2]|0;
      HEAP32[$iSub2>>2] = $256;
      $257 = HEAP32[$1>>2]|0;
      $258 = HEAP32[$2>>2]|0;
      $259 = (_sqlite3Select($257,$258,$uniondest)|0);
      HEAP32[$rc>>2] = $259;
      $260 = HEAP32[$db>>2]|0;
      $261 = HEAP32[$2>>2]|0;
      $262 = (($261) + 48|0);
      $263 = HEAP32[$262>>2]|0;
      _sqlite3ExprListDelete($260,$263);
      $264 = HEAP32[$2>>2]|0;
      $265 = (($264) + 52|0);
      $266 = HEAP32[$265>>2]|0;
      HEAP32[$pDelete>>2] = $266;
      $267 = HEAP32[$pPrior>>2]|0;
      $268 = HEAP32[$2>>2]|0;
      $269 = (($268) + 52|0);
      HEAP32[$269>>2] = $267;
      $270 = HEAP32[$2>>2]|0;
      $271 = (($270) + 48|0);
      HEAP32[$271>>2] = 0;
      $272 = HEAP32[$2>>2]|0;
      $273 = (($272) + 4|0);
      $274 = HEAP8[$273>>0]|0;
      $275 = $274&255;
      $276 = ($275|0)==(115);
      if ($276) {
       $277 = HEAP32[$pPrior>>2]|0;
       $278 = (($277) + 24|0);
       $279 = $278;
       $280 = $279;
       $281 = HEAP32[$280>>2]|0;
       $282 = (($279) + 4)|0;
       $283 = $282;
       $284 = HEAP32[$283>>2]|0;
       $285 = HEAP32[$2>>2]|0;
       $286 = (($285) + 24|0);
       $287 = $286;
       $288 = $287;
       $289 = HEAP32[$288>>2]|0;
       $290 = (($287) + 4)|0;
       $291 = $290;
       $292 = HEAP32[$291>>2]|0;
       $293 = (_i64Add(($289|0),($292|0),($281|0),($284|0))|0);
       $294 = tempRet0;
       $295 = $286;
       $296 = $295;
       HEAP32[$296>>2] = $293;
       $297 = (($295) + 4)|0;
       $298 = $297;
       HEAP32[$298>>2] = $294;
      }
      $299 = HEAP32[$db>>2]|0;
      $300 = HEAP32[$2>>2]|0;
      $301 = (($300) + 60|0);
      $302 = HEAP32[$301>>2]|0;
      _sqlite3ExprDelete($299,$302);
      $303 = HEAP32[$pLimit>>2]|0;
      $304 = HEAP32[$2>>2]|0;
      $305 = (($304) + 60|0);
      HEAP32[$305>>2] = $303;
      $306 = HEAP32[$pOffset>>2]|0;
      $307 = HEAP32[$2>>2]|0;
      $308 = (($307) + 64|0);
      HEAP32[$308>>2] = $306;
      $309 = HEAP32[$2>>2]|0;
      $310 = (($309) + 8|0);
      HEAP32[$310>>2] = 0;
      $311 = HEAP32[$2>>2]|0;
      $312 = (($311) + 12|0);
      HEAP32[$312>>2] = 0;
      $313 = HEAP8[$dest>>0]|0;
      $314 = $313&255;
      $315 = HEAP32[$priorOp>>2]|0;
      $316 = ($314|0)!=($315|0);
      if (!($316)) {
       break;
      }
      $317 = HEAP8[$dest>>0]|0;
      $318 = $317&255;
      $319 = ($318|0)==(9);
      if ($319) {
       $320 = HEAP32[$2>>2]|0;
       HEAP32[$pFirst>>2] = $320;
       while(1) {
        $321 = HEAP32[$pFirst>>2]|0;
        $322 = (($321) + 52|0);
        $323 = HEAP32[$322>>2]|0;
        $324 = ($323|0)!=(0|0);
        if (!($324)) {
         break;
        }
        $325 = HEAP32[$pFirst>>2]|0;
        $326 = (($325) + 52|0);
        $327 = HEAP32[$326>>2]|0;
        HEAP32[$pFirst>>2] = $327;
       }
       $328 = HEAP32[$1>>2]|0;
       $329 = HEAP32[$pFirst>>2]|0;
       $330 = HEAP32[$329>>2]|0;
       _generateColumnNames($328,0,$330);
      }
      $331 = HEAP32[$v>>2]|0;
      $332 = (_sqlite3VdbeMakeLabel($331)|0);
      HEAP32[$iBreak>>2] = $332;
      $333 = HEAP32[$v>>2]|0;
      $334 = (_sqlite3VdbeMakeLabel($333)|0);
      HEAP32[$iCont>>2] = $334;
      $335 = HEAP32[$1>>2]|0;
      $336 = HEAP32[$2>>2]|0;
      $337 = HEAP32[$iBreak>>2]|0;
      _computeLimitRegisters($335,$336,$337);
      $338 = HEAP32[$v>>2]|0;
      $339 = HEAP32[$unionTab>>2]|0;
      $340 = HEAP32[$iBreak>>2]|0;
      (_sqlite3VdbeAddOp2($338,108,$339,$340)|0);
      $341 = HEAP32[$v>>2]|0;
      $342 = (_sqlite3VdbeCurrentAddr($341)|0);
      HEAP32[$iStart>>2] = $342;
      $343 = HEAP32[$1>>2]|0;
      $344 = HEAP32[$2>>2]|0;
      $345 = HEAP32[$2>>2]|0;
      $346 = HEAP32[$345>>2]|0;
      $347 = HEAP32[$unionTab>>2]|0;
      $348 = HEAP32[$iCont>>2]|0;
      $349 = HEAP32[$iBreak>>2]|0;
      _selectInnerLoop($343,$344,$346,$347,0,0,$dest,$348,$349);
      $350 = HEAP32[$v>>2]|0;
      $351 = HEAP32[$iCont>>2]|0;
      _sqlite3VdbeResolveLabel($350,$351);
      $352 = HEAP32[$v>>2]|0;
      $353 = HEAP32[$unionTab>>2]|0;
      $354 = HEAP32[$iStart>>2]|0;
      (_sqlite3VdbeAddOp2($352,9,$353,$354)|0);
      $355 = HEAP32[$v>>2]|0;
      $356 = HEAP32[$iBreak>>2]|0;
      _sqlite3VdbeResolveLabel($355,$356);
      $357 = HEAP32[$v>>2]|0;
      $358 = HEAP32[$unionTab>>2]|0;
      (_sqlite3VdbeAddOp2($357,61,$358,0)|0);
      break;
     } else if ((($75|0) == 116)) {
      HEAP32[$addr>>2] = 0;
      $76 = HEAP32[$2>>2]|0;
      $77 = (($76) + 8|0);
      $78 = HEAP32[$77>>2]|0;
      $79 = HEAP32[$pPrior>>2]|0;
      $80 = (($79) + 8|0);
      HEAP32[$80>>2] = $78;
      $81 = HEAP32[$2>>2]|0;
      $82 = (($81) + 12|0);
      $83 = HEAP32[$82>>2]|0;
      $84 = HEAP32[$pPrior>>2]|0;
      $85 = (($84) + 12|0);
      HEAP32[$85>>2] = $83;
      $86 = HEAP32[$2>>2]|0;
      $87 = (($86) + 60|0);
      $88 = HEAP32[$87>>2]|0;
      $89 = HEAP32[$pPrior>>2]|0;
      $90 = (($89) + 60|0);
      HEAP32[$90>>2] = $88;
      $91 = HEAP32[$2>>2]|0;
      $92 = (($91) + 64|0);
      $93 = HEAP32[$92>>2]|0;
      $94 = HEAP32[$pPrior>>2]|0;
      $95 = (($94) + 64|0);
      HEAP32[$95>>2] = $93;
      $96 = HEAP32[$1>>2]|0;
      $97 = (($96) + 468|0);
      $98 = HEAP32[$97>>2]|0;
      HEAP32[$iSub1>>2] = $98;
      $99 = HEAP32[$1>>2]|0;
      $100 = HEAP32[$pPrior>>2]|0;
      $101 = (_sqlite3Select($99,$100,$dest)|0);
      HEAP32[$rc>>2] = $101;
      $102 = HEAP32[$2>>2]|0;
      $103 = (($102) + 60|0);
      HEAP32[$103>>2] = 0;
      $104 = HEAP32[$2>>2]|0;
      $105 = (($104) + 64|0);
      HEAP32[$105>>2] = 0;
      $106 = HEAP32[$rc>>2]|0;
      $107 = ($106|0)!=(0);
      if ($107) {
       break L1;
      }
      $108 = HEAP32[$2>>2]|0;
      $109 = (($108) + 52|0);
      HEAP32[$109>>2] = 0;
      $110 = HEAP32[$pPrior>>2]|0;
      $111 = (($110) + 8|0);
      $112 = HEAP32[$111>>2]|0;
      $113 = HEAP32[$2>>2]|0;
      $114 = (($113) + 8|0);
      HEAP32[$114>>2] = $112;
      $115 = HEAP32[$pPrior>>2]|0;
      $116 = (($115) + 12|0);
      $117 = HEAP32[$116>>2]|0;
      $118 = HEAP32[$2>>2]|0;
      $119 = (($118) + 12|0);
      HEAP32[$119>>2] = $117;
      $120 = HEAP32[$2>>2]|0;
      $121 = (($120) + 8|0);
      $122 = HEAP32[$121>>2]|0;
      $123 = ($122|0)!=(0);
      if ($123) {
       $124 = HEAP32[$v>>2]|0;
       $125 = HEAP32[$2>>2]|0;
       $126 = (($125) + 8|0);
       $127 = HEAP32[$126>>2]|0;
       $128 = (_sqlite3VdbeAddOp1($124,139,$127)|0);
       HEAP32[$addr>>2] = $128;
      }
      $129 = HEAP32[$1>>2]|0;
      $130 = (($129) + 468|0);
      $131 = HEAP32[$130>>2]|0;
      HEAP32[$iSub2>>2] = $131;
      $132 = HEAP32[$1>>2]|0;
      $133 = HEAP32[$2>>2]|0;
      $134 = (_sqlite3Select($132,$133,$dest)|0);
      HEAP32[$rc>>2] = $134;
      $135 = HEAP32[$2>>2]|0;
      $136 = (($135) + 52|0);
      $137 = HEAP32[$136>>2]|0;
      HEAP32[$pDelete>>2] = $137;
      $138 = HEAP32[$pPrior>>2]|0;
      $139 = HEAP32[$2>>2]|0;
      $140 = (($139) + 52|0);
      HEAP32[$140>>2] = $138;
      $141 = HEAP32[$pPrior>>2]|0;
      $142 = (($141) + 24|0);
      $143 = $142;
      $144 = $143;
      $145 = HEAP32[$144>>2]|0;
      $146 = (($143) + 4)|0;
      $147 = $146;
      $148 = HEAP32[$147>>2]|0;
      $149 = HEAP32[$2>>2]|0;
      $150 = (($149) + 24|0);
      $151 = $150;
      $152 = $151;
      $153 = HEAP32[$152>>2]|0;
      $154 = (($151) + 4)|0;
      $155 = $154;
      $156 = HEAP32[$155>>2]|0;
      $157 = (_i64Add(($153|0),($156|0),($145|0),($148|0))|0);
      $158 = tempRet0;
      $159 = $150;
      $160 = $159;
      HEAP32[$160>>2] = $157;
      $161 = (($159) + 4)|0;
      $162 = $161;
      HEAP32[$162>>2] = $158;
      $163 = HEAP32[$pPrior>>2]|0;
      $164 = (($163) + 60|0);
      $165 = HEAP32[$164>>2]|0;
      $166 = ($165|0)!=(0|0);
      if ($166) {
       $167 = HEAP32[$pPrior>>2]|0;
       $168 = (($167) + 60|0);
       $169 = HEAP32[$168>>2]|0;
       $170 = (_sqlite3ExprIsInteger($169,$nLimit)|0);
       $171 = ($170|0)!=(0);
       $172 = HEAP32[$nLimit>>2]|0;
       $173 = ($172|0)>(0);
       $or$cond = $171 & $173;
       if ($or$cond) {
        $174 = HEAP32[$2>>2]|0;
        $175 = (($174) + 24|0);
        $176 = $175;
        $177 = $176;
        $178 = HEAP32[$177>>2]|0;
        $179 = (($176) + 4)|0;
        $180 = $179;
        $181 = HEAP32[$180>>2]|0;
        $182 = HEAP32[$nLimit>>2]|0;
        $183 = ($182|0)<(0);
        $184 = $183 << 31 >> 31;
        $185 = ($181>>>0)>($184>>>0);
        $186 = ($178>>>0)>($182>>>0);
        $187 = ($181|0)==($184|0);
        $188 = $187 & $186;
        $189 = $185 | $188;
        if ($189) {
         $190 = HEAP32[$nLimit>>2]|0;
         $191 = ($190|0)<(0);
         $192 = $191 << 31 >> 31;
         $193 = HEAP32[$2>>2]|0;
         $194 = (($193) + 24|0);
         $195 = $194;
         $196 = $195;
         HEAP32[$196>>2] = $190;
         $197 = (($195) + 4)|0;
         $198 = $197;
         HEAP32[$198>>2] = $192;
        }
       }
      }
      $199 = HEAP32[$addr>>2]|0;
      $200 = ($199|0)!=(0);
      if (!($200)) {
       break;
      }
      $201 = HEAP32[$v>>2]|0;
      $202 = HEAP32[$addr>>2]|0;
      _sqlite3VdbeJumpHere($201,$202);
      break;
     } else {
      $359 = HEAP32[$1>>2]|0;
      $360 = (($359) + 68|0);
      $361 = HEAP32[$360>>2]|0;
      $362 = (($361) + 1)|0;
      HEAP32[$360>>2] = $362;
      HEAP32[$tab1>>2] = $361;
      $363 = HEAP32[$1>>2]|0;
      $364 = (($363) + 68|0);
      $365 = HEAP32[$364>>2]|0;
      $366 = (($365) + 1)|0;
      HEAP32[$364>>2] = $366;
      HEAP32[$tab2>>2] = $365;
      $367 = HEAP32[$v>>2]|0;
      $368 = HEAP32[$tab1>>2]|0;
      $369 = (_sqlite3VdbeAddOp2($367,57,$368,0)|0);
      HEAP32[$addr7>>2] = $369;
      $370 = HEAP32[$addr7>>2]|0;
      $371 = HEAP32[$2>>2]|0;
      $372 = (($371) + 16|0);
      HEAP32[$372>>2] = $370;
      $373 = HEAP32[$2>>2]|0;
      $374 = (_findRightmost($373)|0);
      $375 = (($374) + 6|0);
      $376 = HEAP16[$375>>1]|0;
      $377 = $376&65535;
      $378 = $377 | 8;
      $379 = $378&65535;
      HEAP16[$375>>1] = $379;
      $380 = HEAP32[$tab1>>2]|0;
      _sqlite3SelectDestInit($intersectdest,1,$380);
      $381 = HEAP32[$1>>2]|0;
      $382 = (($381) + 468|0);
      $383 = HEAP32[$382>>2]|0;
      HEAP32[$iSub1>>2] = $383;
      $384 = HEAP32[$1>>2]|0;
      $385 = HEAP32[$pPrior>>2]|0;
      $386 = (_sqlite3Select($384,$385,$intersectdest)|0);
      HEAP32[$rc>>2] = $386;
      $387 = HEAP32[$rc>>2]|0;
      $388 = ($387|0)!=(0);
      if ($388) {
       break L1;
      }
      $389 = HEAP32[$v>>2]|0;
      $390 = HEAP32[$tab2>>2]|0;
      $391 = (_sqlite3VdbeAddOp2($389,57,$390,0)|0);
      HEAP32[$addr7>>2] = $391;
      $392 = HEAP32[$addr7>>2]|0;
      $393 = HEAP32[$2>>2]|0;
      $394 = (($393) + 16|0);
      $395 = (($394) + 4|0);
      HEAP32[$395>>2] = $392;
      $396 = HEAP32[$2>>2]|0;
      $397 = (($396) + 52|0);
      HEAP32[$397>>2] = 0;
      $398 = HEAP32[$2>>2]|0;
      $399 = (($398) + 60|0);
      $400 = HEAP32[$399>>2]|0;
      HEAP32[$pLimit5>>2] = $400;
      $401 = HEAP32[$2>>2]|0;
      $402 = (($401) + 60|0);
      HEAP32[$402>>2] = 0;
      $403 = HEAP32[$2>>2]|0;
      $404 = (($403) + 64|0);
      $405 = HEAP32[$404>>2]|0;
      HEAP32[$pOffset6>>2] = $405;
      $406 = HEAP32[$2>>2]|0;
      $407 = (($406) + 64|0);
      HEAP32[$407>>2] = 0;
      $408 = HEAP32[$tab2>>2]|0;
      $409 = (($intersectdest) + 4|0);
      HEAP32[$409>>2] = $408;
      $410 = HEAP32[$1>>2]|0;
      $411 = (($410) + 468|0);
      $412 = HEAP32[$411>>2]|0;
      HEAP32[$iSub2>>2] = $412;
      $413 = HEAP32[$1>>2]|0;
      $414 = HEAP32[$2>>2]|0;
      $415 = (_sqlite3Select($413,$414,$intersectdest)|0);
      HEAP32[$rc>>2] = $415;
      $416 = HEAP32[$2>>2]|0;
      $417 = (($416) + 52|0);
      $418 = HEAP32[$417>>2]|0;
      HEAP32[$pDelete>>2] = $418;
      $419 = HEAP32[$pPrior>>2]|0;
      $420 = HEAP32[$2>>2]|0;
      $421 = (($420) + 52|0);
      HEAP32[$421>>2] = $419;
      $422 = HEAP32[$2>>2]|0;
      $423 = (($422) + 24|0);
      $424 = $423;
      $425 = $424;
      $426 = HEAP32[$425>>2]|0;
      $427 = (($424) + 4)|0;
      $428 = $427;
      $429 = HEAP32[$428>>2]|0;
      $430 = HEAP32[$pPrior>>2]|0;
      $431 = (($430) + 24|0);
      $432 = $431;
      $433 = $432;
      $434 = HEAP32[$433>>2]|0;
      $435 = (($432) + 4)|0;
      $436 = $435;
      $437 = HEAP32[$436>>2]|0;
      $438 = ($429>>>0)>($437>>>0);
      $439 = ($426>>>0)>($434>>>0);
      $440 = ($429|0)==($437|0);
      $441 = $440 & $439;
      $442 = $438 | $441;
      if ($442) {
       $443 = HEAP32[$pPrior>>2]|0;
       $444 = (($443) + 24|0);
       $445 = $444;
       $446 = $445;
       $447 = HEAP32[$446>>2]|0;
       $448 = (($445) + 4)|0;
       $449 = $448;
       $450 = HEAP32[$449>>2]|0;
       $451 = HEAP32[$2>>2]|0;
       $452 = (($451) + 24|0);
       $453 = $452;
       $454 = $453;
       HEAP32[$454>>2] = $447;
       $455 = (($453) + 4)|0;
       $456 = $455;
       HEAP32[$456>>2] = $450;
      }
      $457 = HEAP32[$db>>2]|0;
      $458 = HEAP32[$2>>2]|0;
      $459 = (($458) + 60|0);
      $460 = HEAP32[$459>>2]|0;
      _sqlite3ExprDelete($457,$460);
      $461 = HEAP32[$pLimit5>>2]|0;
      $462 = HEAP32[$2>>2]|0;
      $463 = (($462) + 60|0);
      HEAP32[$463>>2] = $461;
      $464 = HEAP32[$pOffset6>>2]|0;
      $465 = HEAP32[$2>>2]|0;
      $466 = (($465) + 64|0);
      HEAP32[$466>>2] = $464;
      $467 = HEAP8[$dest>>0]|0;
      $468 = $467&255;
      $469 = ($468|0)==(9);
      if ($469) {
       $470 = HEAP32[$2>>2]|0;
       HEAP32[$pFirst8>>2] = $470;
       while(1) {
        $471 = HEAP32[$pFirst8>>2]|0;
        $472 = (($471) + 52|0);
        $473 = HEAP32[$472>>2]|0;
        $474 = ($473|0)!=(0|0);
        if (!($474)) {
         break;
        }
        $475 = HEAP32[$pFirst8>>2]|0;
        $476 = (($475) + 52|0);
        $477 = HEAP32[$476>>2]|0;
        HEAP32[$pFirst8>>2] = $477;
       }
       $478 = HEAP32[$1>>2]|0;
       $479 = HEAP32[$pFirst8>>2]|0;
       $480 = HEAP32[$479>>2]|0;
       _generateColumnNames($478,0,$480);
      }
      $481 = HEAP32[$v>>2]|0;
      $482 = (_sqlite3VdbeMakeLabel($481)|0);
      HEAP32[$iBreak3>>2] = $482;
      $483 = HEAP32[$v>>2]|0;
      $484 = (_sqlite3VdbeMakeLabel($483)|0);
      HEAP32[$iCont2>>2] = $484;
      $485 = HEAP32[$1>>2]|0;
      $486 = HEAP32[$2>>2]|0;
      $487 = HEAP32[$iBreak3>>2]|0;
      _computeLimitRegisters($485,$486,$487);
      $488 = HEAP32[$v>>2]|0;
      $489 = HEAP32[$tab1>>2]|0;
      $490 = HEAP32[$iBreak3>>2]|0;
      (_sqlite3VdbeAddOp2($488,108,$489,$490)|0);
      $491 = HEAP32[$1>>2]|0;
      $492 = (_sqlite3GetTempReg($491)|0);
      HEAP32[$r1>>2] = $492;
      $493 = HEAP32[$v>>2]|0;
      $494 = HEAP32[$tab1>>2]|0;
      $495 = HEAP32[$r1>>2]|0;
      $496 = (_sqlite3VdbeAddOp2($493,101,$494,$495)|0);
      HEAP32[$iStart4>>2] = $496;
      $497 = HEAP32[$v>>2]|0;
      $498 = HEAP32[$tab2>>2]|0;
      $499 = HEAP32[$iCont2>>2]|0;
      $500 = HEAP32[$r1>>2]|0;
      (_sqlite3VdbeAddOp4Int($497,68,$498,$499,$500,0)|0);
      $501 = HEAP32[$1>>2]|0;
      $502 = HEAP32[$r1>>2]|0;
      _sqlite3ReleaseTempReg($501,$502);
      $503 = HEAP32[$1>>2]|0;
      $504 = HEAP32[$2>>2]|0;
      $505 = HEAP32[$2>>2]|0;
      $506 = HEAP32[$505>>2]|0;
      $507 = HEAP32[$tab1>>2]|0;
      $508 = HEAP32[$iCont2>>2]|0;
      $509 = HEAP32[$iBreak3>>2]|0;
      _selectInnerLoop($503,$504,$506,$507,0,0,$dest,$508,$509);
      $510 = HEAP32[$v>>2]|0;
      $511 = HEAP32[$iCont2>>2]|0;
      _sqlite3VdbeResolveLabel($510,$511);
      $512 = HEAP32[$v>>2]|0;
      $513 = HEAP32[$tab1>>2]|0;
      $514 = HEAP32[$iStart4>>2]|0;
      (_sqlite3VdbeAddOp2($512,9,$513,$514)|0);
      $515 = HEAP32[$v>>2]|0;
      $516 = HEAP32[$iBreak3>>2]|0;
      _sqlite3VdbeResolveLabel($515,$516);
      $517 = HEAP32[$v>>2]|0;
      $518 = HEAP32[$tab2>>2]|0;
      (_sqlite3VdbeAddOp2($517,61,$518,0)|0);
      $519 = HEAP32[$v>>2]|0;
      $520 = HEAP32[$tab1>>2]|0;
      (_sqlite3VdbeAddOp2($519,61,$520,0)|0);
      break;
     }
    }
   } while(0);
   $521 = HEAP32[$1>>2]|0;
   $522 = HEAP32[$2>>2]|0;
   $523 = (($522) + 4|0);
   $524 = HEAP8[$523>>0]|0;
   $525 = $524&255;
   $526 = HEAP32[$iSub1>>2]|0;
   $527 = HEAP32[$iSub2>>2]|0;
   $528 = HEAP32[$2>>2]|0;
   $529 = (($528) + 4|0);
   $530 = HEAP8[$529>>0]|0;
   $531 = $530&255;
   $532 = ($531|0)!=(116);
   $533 = $532&1;
   _explainComposite($521,$525,$526,$527,$533);
   $534 = HEAP32[$2>>2]|0;
   $535 = (($534) + 6|0);
   $536 = HEAP16[$535>>1]|0;
   $537 = $536&65535;
   $538 = $537 & 8;
   $539 = ($538|0)!=(0);
   if ($539) {
    $540 = HEAP32[$2>>2]|0;
    $541 = HEAP32[$540>>2]|0;
    $542 = HEAP32[$541>>2]|0;
    HEAP32[$nCol>>2] = $542;
    $543 = HEAP32[$db>>2]|0;
    $544 = HEAP32[$nCol>>2]|0;
    $545 = (_sqlite3KeyInfoAlloc($543,$544,1)|0);
    HEAP32[$pKeyInfo>>2] = $545;
    $546 = HEAP32[$pKeyInfo>>2]|0;
    $547 = ($546|0)!=(0|0);
    if (!($547)) {
     HEAP32[$rc>>2] = 7;
     break;
    }
    HEAP32[$i>>2] = 0;
    $548 = HEAP32[$pKeyInfo>>2]|0;
    $549 = (($548) + 20|0);
    HEAP32[$apColl>>2] = $549;
    while(1) {
     $550 = HEAP32[$i>>2]|0;
     $551 = HEAP32[$nCol>>2]|0;
     $552 = ($550|0)<($551|0);
     if (!($552)) {
      break;
     }
     $553 = HEAP32[$1>>2]|0;
     $554 = HEAP32[$2>>2]|0;
     $555 = HEAP32[$i>>2]|0;
     $556 = (_multiSelectCollSeq($553,$554,$555)|0);
     $557 = HEAP32[$apColl>>2]|0;
     HEAP32[$557>>2] = $556;
     $558 = HEAP32[$apColl>>2]|0;
     $559 = HEAP32[$558>>2]|0;
     $560 = (0|0)==($559|0);
     if ($560) {
      $561 = HEAP32[$db>>2]|0;
      $562 = (($561) + 8|0);
      $563 = HEAP32[$562>>2]|0;
      $564 = HEAP32[$apColl>>2]|0;
      HEAP32[$564>>2] = $563;
     }
     $565 = HEAP32[$i>>2]|0;
     $566 = (($565) + 1)|0;
     HEAP32[$i>>2] = $566;
     $567 = HEAP32[$apColl>>2]|0;
     $568 = (($567) + 4|0);
     HEAP32[$apColl>>2] = $568;
    }
    $569 = HEAP32[$2>>2]|0;
    HEAP32[$pLoop>>2] = $569;
    while(1) {
     $570 = HEAP32[$pLoop>>2]|0;
     $571 = ($570|0)!=(0|0);
     if (!($571)) {
      break;
     }
     HEAP32[$i>>2] = 0;
     while(1) {
      $572 = HEAP32[$i>>2]|0;
      $573 = ($572|0)<(2);
      if (!($573)) {
       break;
      }
      $574 = HEAP32[$i>>2]|0;
      $575 = HEAP32[$pLoop>>2]|0;
      $576 = (($575) + 16|0);
      $577 = (($576) + ($574<<2)|0);
      $578 = HEAP32[$577>>2]|0;
      HEAP32[$addr9>>2] = $578;
      $579 = HEAP32[$addr9>>2]|0;
      $580 = ($579|0)<(0);
      if ($580) {
       break;
      }
      $581 = HEAP32[$v>>2]|0;
      $582 = HEAP32[$addr9>>2]|0;
      $583 = HEAP32[$nCol>>2]|0;
      _sqlite3VdbeChangeP2($581,$582,$583);
      $584 = HEAP32[$v>>2]|0;
      $585 = HEAP32[$addr9>>2]|0;
      $586 = HEAP32[$pKeyInfo>>2]|0;
      $587 = (_sqlite3KeyInfoRef($586)|0);
      _sqlite3VdbeChangeP4($584,$585,$587,-6);
      $588 = HEAP32[$i>>2]|0;
      $589 = HEAP32[$pLoop>>2]|0;
      $590 = (($589) + 16|0);
      $591 = (($590) + ($588<<2)|0);
      HEAP32[$591>>2] = -1;
      $592 = HEAP32[$i>>2]|0;
      $593 = (($592) + 1)|0;
      HEAP32[$i>>2] = $593;
     }
     $594 = HEAP32[$pLoop>>2]|0;
     $595 = (($594) + 52|0);
     $596 = HEAP32[$595>>2]|0;
     HEAP32[$pLoop>>2] = $596;
    }
    $597 = HEAP32[$pKeyInfo>>2]|0;
    _sqlite3KeyInfoUnref($597);
   }
  }
 } while(0);
 $598 = (($dest) + 8|0);
 $599 = HEAP32[$598>>2]|0;
 $600 = HEAP32[$3>>2]|0;
 $601 = (($600) + 8|0);
 HEAP32[$601>>2] = $599;
 $602 = (($dest) + 12|0);
 $603 = HEAP32[$602>>2]|0;
 $604 = HEAP32[$3>>2]|0;
 $605 = (($604) + 12|0);
 HEAP32[$605>>2] = $603;
 $606 = HEAP32[$db>>2]|0;
 $607 = HEAP32[$pDelete>>2]|0;
 _sqlite3SelectDelete($606,$607);
 $608 = HEAP32[$rc>>2]|0;
 HEAP32[$0>>2] = $608;
 $609 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($609|0);
}
function _keyInfoFromExprList($pParse,$pList,$iStart,$nExtra) {
 $pParse = $pParse|0;
 $pList = $pList|0;
 $iStart = $iStart|0;
 $nExtra = $nExtra|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $db = 0, $i = 0, $nExpr = 0;
 var $pColl = 0, $pInfo = 0, $pItem = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $0 = sp + 36|0;
 $1 = sp + 32|0;
 $2 = sp + 28|0;
 $3 = sp + 24|0;
 $nExpr = sp + 20|0;
 $pInfo = sp + 16|0;
 $pItem = sp + 12|0;
 $db = sp + 8|0;
 $i = sp + 4|0;
 $pColl = sp;
 HEAP32[$0>>2] = $pParse;
 HEAP32[$1>>2] = $pList;
 HEAP32[$2>>2] = $iStart;
 HEAP32[$3>>2] = $nExtra;
 $4 = HEAP32[$0>>2]|0;
 $5 = HEAP32[$4>>2]|0;
 HEAP32[$db>>2] = $5;
 $6 = HEAP32[$1>>2]|0;
 $7 = HEAP32[$6>>2]|0;
 HEAP32[$nExpr>>2] = $7;
 $8 = HEAP32[$db>>2]|0;
 $9 = HEAP32[$nExpr>>2]|0;
 $10 = HEAP32[$3>>2]|0;
 $11 = (($9) + ($10))|0;
 $12 = HEAP32[$2>>2]|0;
 $13 = (($11) - ($12))|0;
 $14 = (_sqlite3KeyInfoAlloc($8,$13,1)|0);
 HEAP32[$pInfo>>2] = $14;
 $15 = HEAP32[$pInfo>>2]|0;
 $16 = ($15|0)!=(0|0);
 if (!($16)) {
  $56 = HEAP32[$pInfo>>2]|0;
  STACKTOP = sp;return ($56|0);
 }
 $17 = HEAP32[$2>>2]|0;
 HEAP32[$i>>2] = $17;
 $18 = HEAP32[$1>>2]|0;
 $19 = (($18) + 4|0);
 $20 = HEAP32[$19>>2]|0;
 $21 = HEAP32[$2>>2]|0;
 $22 = (($20) + (($21*20)|0)|0);
 HEAP32[$pItem>>2] = $22;
 while(1) {
  $23 = HEAP32[$i>>2]|0;
  $24 = HEAP32[$nExpr>>2]|0;
  $25 = ($23|0)<($24|0);
  if (!($25)) {
   break;
  }
  $26 = HEAP32[$0>>2]|0;
  $27 = HEAP32[$pItem>>2]|0;
  $28 = HEAP32[$27>>2]|0;
  $29 = (_sqlite3ExprCollSeq($26,$28)|0);
  HEAP32[$pColl>>2] = $29;
  $30 = HEAP32[$pColl>>2]|0;
  $31 = ($30|0)!=(0|0);
  if (!($31)) {
   $32 = HEAP32[$db>>2]|0;
   $33 = (($32) + 8|0);
   $34 = HEAP32[$33>>2]|0;
   HEAP32[$pColl>>2] = $34;
  }
  $35 = HEAP32[$pColl>>2]|0;
  $36 = HEAP32[$i>>2]|0;
  $37 = HEAP32[$2>>2]|0;
  $38 = (($36) - ($37))|0;
  $39 = HEAP32[$pInfo>>2]|0;
  $40 = (($39) + 20|0);
  $41 = (($40) + ($38<<2)|0);
  HEAP32[$41>>2] = $35;
  $42 = HEAP32[$pItem>>2]|0;
  $43 = (($42) + 12|0);
  $44 = HEAP8[$43>>0]|0;
  $45 = HEAP32[$i>>2]|0;
  $46 = HEAP32[$2>>2]|0;
  $47 = (($45) - ($46))|0;
  $48 = HEAP32[$pInfo>>2]|0;
  $49 = (($48) + 16|0);
  $50 = HEAP32[$49>>2]|0;
  $51 = (($50) + ($47)|0);
  HEAP8[$51>>0] = $44;
  $52 = HEAP32[$i>>2]|0;
  $53 = (($52) + 1)|0;
  HEAP32[$i>>2] = $53;
  $54 = HEAP32[$pItem>>2]|0;
  $55 = (($54) + 20|0);
  HEAP32[$pItem>>2] = $55;
 }
 $56 = HEAP32[$pInfo>>2]|0;
 STACKTOP = sp;return ($56|0);
}
function _computeLimitRegisters($pParse,$p,$iBreak) {
 $pParse = $pParse|0;
 $p = $p|0;
 $iBreak = $iBreak|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0;
 var $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0;
 var $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0;
 var $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0;
 var $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0;
 var $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $addr1 = 0, $iLimit = 0, $iOffset = 0, $n = 0, $v = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 28|0;
 $1 = sp + 24|0;
 $2 = sp + 20|0;
 $v = sp + 16|0;
 $iLimit = sp + 12|0;
 $iOffset = sp + 8|0;
 $addr1 = sp + 4|0;
 $n = sp;
 HEAP32[$0>>2] = $pParse;
 HEAP32[$1>>2] = $p;
 HEAP32[$2>>2] = $iBreak;
 HEAP32[$v>>2] = 0;
 HEAP32[$iLimit>>2] = 0;
 $3 = HEAP32[$1>>2]|0;
 $4 = (($3) + 8|0);
 $5 = HEAP32[$4>>2]|0;
 $6 = ($5|0)!=(0);
 if ($6) {
  STACKTOP = sp;return;
 }
 $7 = HEAP32[$0>>2]|0;
 _sqlite3ExprCacheClear($7);
 $8 = HEAP32[$1>>2]|0;
 $9 = (($8) + 60|0);
 $10 = HEAP32[$9>>2]|0;
 $11 = ($10|0)!=(0|0);
 if (!($11)) {
  STACKTOP = sp;return;
 }
 $12 = HEAP32[$0>>2]|0;
 $13 = (($12) + 72|0);
 $14 = HEAP32[$13>>2]|0;
 $15 = (($14) + 1)|0;
 HEAP32[$13>>2] = $15;
 HEAP32[$iLimit>>2] = $15;
 $16 = HEAP32[$1>>2]|0;
 $17 = (($16) + 8|0);
 HEAP32[$17>>2] = $15;
 $18 = HEAP32[$0>>2]|0;
 $19 = (_sqlite3GetVdbe($18)|0);
 HEAP32[$v>>2] = $19;
 $20 = HEAP32[$1>>2]|0;
 $21 = (($20) + 60|0);
 $22 = HEAP32[$21>>2]|0;
 $23 = (_sqlite3ExprIsInteger($22,$n)|0);
 $24 = ($23|0)!=(0);
 do {
  if ($24) {
   $25 = HEAP32[$v>>2]|0;
   $26 = HEAP32[$n>>2]|0;
   $27 = HEAP32[$iLimit>>2]|0;
   (_sqlite3VdbeAddOp2($25,25,$26,$27)|0);
   $28 = HEAP32[$n>>2]|0;
   $29 = ($28|0)==(0);
   if ($29) {
    $30 = HEAP32[$v>>2]|0;
    $31 = HEAP32[$2>>2]|0;
    (_sqlite3VdbeAddOp2($30,16,0,$31)|0);
    break;
   }
   $32 = HEAP32[$n>>2]|0;
   $33 = ($32|0)>=(0);
   if ($33) {
    $34 = HEAP32[$1>>2]|0;
    $35 = (($34) + 24|0);
    $36 = $35;
    $37 = $36;
    $38 = HEAP32[$37>>2]|0;
    $39 = (($36) + 4)|0;
    $40 = $39;
    $41 = HEAP32[$40>>2]|0;
    $42 = HEAP32[$n>>2]|0;
    $43 = ($42|0)<(0);
    $44 = $43 << 31 >> 31;
    $45 = ($41>>>0)>($44>>>0);
    $46 = ($38>>>0)>($42>>>0);
    $47 = ($41|0)==($44|0);
    $48 = $47 & $46;
    $49 = $45 | $48;
    if ($49) {
     $50 = HEAP32[$n>>2]|0;
     $51 = ($50|0)<(0);
     $52 = $51 << 31 >> 31;
     $53 = HEAP32[$1>>2]|0;
     $54 = (($53) + 24|0);
     $55 = $54;
     $56 = $55;
     HEAP32[$56>>2] = $50;
     $57 = (($55) + 4)|0;
     $58 = $57;
     HEAP32[$58>>2] = $52;
    }
   }
  } else {
   $59 = HEAP32[$0>>2]|0;
   $60 = HEAP32[$1>>2]|0;
   $61 = (($60) + 60|0);
   $62 = HEAP32[$61>>2]|0;
   $63 = HEAP32[$iLimit>>2]|0;
   _sqlite3ExprCode($59,$62,$63);
   $64 = HEAP32[$v>>2]|0;
   $65 = HEAP32[$iLimit>>2]|0;
   (_sqlite3VdbeAddOp1($64,38,$65)|0);
   $66 = HEAP32[$v>>2]|0;
   $67 = HEAP32[$iLimit>>2]|0;
   $68 = HEAP32[$2>>2]|0;
   (_sqlite3VdbeAddOp2($66,139,$67,$68)|0);
  }
 } while(0);
 $69 = HEAP32[$1>>2]|0;
 $70 = (($69) + 64|0);
 $71 = HEAP32[$70>>2]|0;
 $72 = ($71|0)!=(0|0);
 if (!($72)) {
  STACKTOP = sp;return;
 }
 $73 = HEAP32[$0>>2]|0;
 $74 = (($73) + 72|0);
 $75 = HEAP32[$74>>2]|0;
 $76 = (($75) + 1)|0;
 HEAP32[$74>>2] = $76;
 HEAP32[$iOffset>>2] = $76;
 $77 = HEAP32[$1>>2]|0;
 $78 = (($77) + 12|0);
 HEAP32[$78>>2] = $76;
 $79 = HEAP32[$0>>2]|0;
 $80 = (($79) + 72|0);
 $81 = HEAP32[$80>>2]|0;
 $82 = (($81) + 1)|0;
 HEAP32[$80>>2] = $82;
 $83 = HEAP32[$0>>2]|0;
 $84 = HEAP32[$1>>2]|0;
 $85 = (($84) + 64|0);
 $86 = HEAP32[$85>>2]|0;
 $87 = HEAP32[$iOffset>>2]|0;
 _sqlite3ExprCode($83,$86,$87);
 $88 = HEAP32[$v>>2]|0;
 $89 = HEAP32[$iOffset>>2]|0;
 (_sqlite3VdbeAddOp1($88,38,$89)|0);
 $90 = HEAP32[$v>>2]|0;
 $91 = HEAP32[$iOffset>>2]|0;
 $92 = (_sqlite3VdbeAddOp1($90,137,$91)|0);
 HEAP32[$addr1>>2] = $92;
 $93 = HEAP32[$v>>2]|0;
 $94 = HEAP32[$iOffset>>2]|0;
 (_sqlite3VdbeAddOp2($93,25,0,$94)|0);
 $95 = HEAP32[$v>>2]|0;
 $96 = HEAP32[$addr1>>2]|0;
 _sqlite3VdbeJumpHere($95,$96);
 $97 = HEAP32[$v>>2]|0;
 $98 = HEAP32[$iLimit>>2]|0;
 $99 = HEAP32[$iOffset>>2]|0;
 $100 = HEAP32[$iOffset>>2]|0;
 $101 = (($100) + 1)|0;
 (_sqlite3VdbeAddOp3($97,89,$98,$99,$101)|0);
 $102 = HEAP32[$v>>2]|0;
 $103 = HEAP32[$iLimit>>2]|0;
 $104 = (_sqlite3VdbeAddOp1($102,137,$103)|0);
 HEAP32[$addr1>>2] = $104;
 $105 = HEAP32[$v>>2]|0;
 $106 = HEAP32[$iOffset>>2]|0;
 $107 = (($106) + 1)|0;
 (_sqlite3VdbeAddOp2($105,25,-1,$107)|0);
 $108 = HEAP32[$v>>2]|0;
 $109 = HEAP32[$addr1>>2]|0;
 _sqlite3VdbeJumpHere($108,$109);
 STACKTOP = sp;return;
}
function _sqlite3WhereOutputRowCount($pWInfo) {
 $pWInfo = $pWInfo|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 HEAP32[$0>>2] = $pWInfo;
 $1 = HEAP32[$0>>2]|0;
 $2 = (($1) + 32|0);
 $3 = HEAP16[$2>>1]|0;
 $4 = (_sqlite3LogEstToInt($3)|0);
 $5 = tempRet0;
 tempRet0 = $5;
 STACKTOP = sp;return ($4|0);
}
function _sqlite3WhereIsDistinct($pWInfo) {
 $pWInfo = $pWInfo|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 HEAP32[$0>>2] = $pWInfo;
 $1 = HEAP32[$0>>2]|0;
 $2 = (($1) + 40|0);
 $3 = HEAP8[$2>>0]|0;
 $4 = $3&255;
 STACKTOP = sp;return ($4|0);
}
function _sqlite3WhereIsOrdered($pWInfo) {
 $pWInfo = $pWInfo|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 HEAP32[$0>>2] = $pWInfo;
 $1 = HEAP32[$0>>2]|0;
 $2 = (($1) + 36|0);
 $3 = HEAP8[$2>>0]|0;
 $4 = $3 << 24 >> 24;
 STACKTOP = sp;return ($4|0);
}
function _selectInnerLoop($pParse,$p,$pEList,$srcTab,$pSort,$pDistinct,$pDest,$iContinue,$iBreak) {
 $pParse = $pParse|0;
 $p = $p|0;
 $pEList = $pEList|0;
 $srcTab = $srcTab|0;
 $pSort = $pSort|0;
 $pDistinct = $pDistinct|0;
 $pDest = $pDest|0;
 $iContinue = $iContinue|0;
 $iBreak = $iBreak|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0;
 var $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0, $188 = 0;
 var $189 = 0, $19 = 0, $190 = 0, $191 = 0, $192 = 0, $193 = 0, $194 = 0, $195 = 0, $196 = 0, $197 = 0, $198 = 0, $199 = 0, $2 = 0, $20 = 0, $200 = 0, $201 = 0, $202 = 0, $203 = 0, $204 = 0, $205 = 0;
 var $206 = 0, $207 = 0, $208 = 0, $209 = 0, $21 = 0, $210 = 0, $211 = 0, $212 = 0, $213 = 0, $214 = 0, $215 = 0, $216 = 0, $217 = 0, $218 = 0, $219 = 0, $22 = 0, $220 = 0, $221 = 0, $222 = 0, $223 = 0;
 var $224 = 0, $225 = 0, $226 = 0, $227 = 0, $228 = 0, $229 = 0, $23 = 0, $230 = 0, $231 = 0, $232 = 0, $233 = 0, $234 = 0, $235 = 0, $236 = 0, $237 = 0, $238 = 0, $239 = 0, $24 = 0, $240 = 0, $241 = 0;
 var $242 = 0, $243 = 0, $244 = 0, $245 = 0, $246 = 0, $247 = 0, $248 = 0, $249 = 0, $25 = 0, $250 = 0, $251 = 0, $252 = 0, $253 = 0, $254 = 0, $255 = 0, $256 = 0, $257 = 0, $258 = 0, $259 = 0, $26 = 0;
 var $260 = 0, $261 = 0, $262 = 0, $263 = 0, $264 = 0, $265 = 0, $266 = 0, $267 = 0, $268 = 0, $269 = 0, $27 = 0, $270 = 0, $271 = 0, $272 = 0, $273 = 0, $274 = 0, $275 = 0, $276 = 0, $277 = 0, $278 = 0;
 var $279 = 0, $28 = 0, $280 = 0, $281 = 0, $282 = 0, $283 = 0, $284 = 0, $285 = 0, $286 = 0, $287 = 0, $288 = 0, $289 = 0, $29 = 0, $290 = 0, $291 = 0, $292 = 0, $293 = 0, $294 = 0, $295 = 0, $296 = 0;
 var $297 = 0, $298 = 0, $299 = 0, $3 = 0, $30 = 0, $300 = 0, $301 = 0, $302 = 0, $303 = 0, $304 = 0, $305 = 0, $306 = 0, $307 = 0, $308 = 0, $309 = 0, $31 = 0, $310 = 0, $311 = 0, $312 = 0, $313 = 0;
 var $314 = 0, $315 = 0, $316 = 0, $317 = 0, $318 = 0, $319 = 0, $32 = 0, $320 = 0, $321 = 0, $322 = 0, $323 = 0, $324 = 0, $325 = 0, $326 = 0, $327 = 0, $328 = 0, $329 = 0, $33 = 0, $330 = 0, $331 = 0;
 var $332 = 0, $333 = 0, $334 = 0, $335 = 0, $336 = 0, $337 = 0, $338 = 0, $339 = 0, $34 = 0, $340 = 0, $341 = 0, $342 = 0, $343 = 0, $344 = 0, $345 = 0, $346 = 0, $347 = 0, $348 = 0, $349 = 0, $35 = 0;
 var $350 = 0, $351 = 0, $352 = 0, $353 = 0, $354 = 0, $355 = 0, $356 = 0, $357 = 0, $358 = 0, $359 = 0, $36 = 0, $360 = 0, $361 = 0, $362 = 0, $363 = 0, $364 = 0, $365 = 0, $366 = 0, $367 = 0, $368 = 0;
 var $369 = 0, $37 = 0, $370 = 0, $371 = 0, $372 = 0, $373 = 0, $374 = 0, $375 = 0, $376 = 0, $377 = 0, $378 = 0, $379 = 0, $38 = 0, $380 = 0, $381 = 0, $382 = 0, $383 = 0, $384 = 0, $385 = 0, $386 = 0;
 var $387 = 0, $388 = 0, $389 = 0, $39 = 0, $390 = 0, $391 = 0, $392 = 0, $393 = 0, $394 = 0, $395 = 0, $396 = 0, $397 = 0, $398 = 0, $399 = 0, $4 = 0, $40 = 0, $400 = 0, $401 = 0, $402 = 0, $403 = 0;
 var $404 = 0, $405 = 0, $406 = 0, $407 = 0, $408 = 0, $409 = 0, $41 = 0, $410 = 0, $411 = 0, $412 = 0, $413 = 0, $414 = 0, $415 = 0, $416 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0;
 var $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0;
 var $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0;
 var $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $addr = 0, $addrTest = 0, $eDest = 0;
 var $hasDistinct = 0, $i = 0, $iJump = 0, $iParm = 0, $nKey = 0, $nPrefixReg = 0, $nResultCol = 0, $or$cond = 0, $pColl = 0, $pOp = 0, $pSO = 0, $r1 = 0, $r11 = 0, $r12 = 0, $r13 = 0, $r2 = 0, $r24 = 0, $r3 = 0, $regPrev = 0, $regResult = 0;
 var $v = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 128|0;
 $0 = sp + 124|0;
 $1 = sp + 120|0;
 $2 = sp + 116|0;
 $3 = sp + 112|0;
 $4 = sp + 108|0;
 $5 = sp + 104|0;
 $6 = sp + 100|0;
 $7 = sp + 96|0;
 $8 = sp + 92|0;
 $v = sp + 88|0;
 $i = sp + 84|0;
 $hasDistinct = sp + 80|0;
 $regResult = sp + 76|0;
 $eDest = sp + 72|0;
 $iParm = sp + 68|0;
 $nResultCol = sp + 64|0;
 $nPrefixReg = sp + 60|0;
 $pOp = sp + 56|0;
 $iJump = sp + 52|0;
 $regPrev = sp + 48|0;
 $pColl = sp + 44|0;
 $r1 = sp + 40|0;
 $r11 = sp + 36|0;
 $addr = sp + 32|0;
 $r2 = sp + 28|0;
 $r12 = sp + 24|0;
 $nKey = sp + 20|0;
 $r13 = sp + 16|0;
 $r24 = sp + 12|0;
 $r3 = sp + 8|0;
 $addrTest = sp + 4|0;
 $pSO = sp;
 HEAP32[$0>>2] = $pParse;
 HEAP32[$1>>2] = $p;
 HEAP32[$2>>2] = $pEList;
 HEAP32[$3>>2] = $srcTab;
 HEAP32[$4>>2] = $pSort;
 HEAP32[$5>>2] = $pDistinct;
 HEAP32[$6>>2] = $pDest;
 HEAP32[$7>>2] = $iContinue;
 HEAP32[$8>>2] = $iBreak;
 $9 = HEAP32[$0>>2]|0;
 $10 = (($9) + 8|0);
 $11 = HEAP32[$10>>2]|0;
 HEAP32[$v>>2] = $11;
 $12 = HEAP32[$6>>2]|0;
 $13 = HEAP8[$12>>0]|0;
 $14 = $13&255;
 HEAP32[$eDest>>2] = $14;
 $15 = HEAP32[$6>>2]|0;
 $16 = (($15) + 4|0);
 $17 = HEAP32[$16>>2]|0;
 HEAP32[$iParm>>2] = $17;
 HEAP32[$nPrefixReg>>2] = 0;
 $18 = HEAP32[$5>>2]|0;
 $19 = ($18|0)!=(0|0);
 if ($19) {
  $20 = HEAP32[$5>>2]|0;
  $21 = (($20) + 1|0);
  $22 = HEAP8[$21>>0]|0;
  $23 = $22&255;
  $24 = $23;
 } else {
  $24 = 0;
 }
 HEAP32[$hasDistinct>>2] = $24;
 $25 = HEAP32[$4>>2]|0;
 $26 = ($25|0)!=(0|0);
 if ($26) {
  $27 = HEAP32[$4>>2]|0;
  $28 = HEAP32[$27>>2]|0;
  $29 = ($28|0)==(0|0);
  if ($29) {
   HEAP32[$4>>2] = 0;
  }
 }
 $30 = HEAP32[$4>>2]|0;
 $31 = ($30|0)!=(0|0);
 $32 = HEAP32[$hasDistinct>>2]|0;
 $33 = ($32|0)!=(0);
 $or$cond = $31 | $33;
 if (!($or$cond)) {
  $34 = HEAP32[$v>>2]|0;
  $35 = HEAP32[$1>>2]|0;
  $36 = (($35) + 12|0);
  $37 = HEAP32[$36>>2]|0;
  $38 = HEAP32[$7>>2]|0;
  _codeOffset($34,$37,$38);
 }
 $39 = HEAP32[$2>>2]|0;
 $40 = HEAP32[$39>>2]|0;
 HEAP32[$nResultCol>>2] = $40;
 $41 = HEAP32[$6>>2]|0;
 $42 = (($41) + 8|0);
 $43 = HEAP32[$42>>2]|0;
 $44 = ($43|0)==(0);
 if ($44) {
  $45 = HEAP32[$4>>2]|0;
  $46 = ($45|0)!=(0|0);
  if ($46) {
   $47 = HEAP32[$4>>2]|0;
   $48 = HEAP32[$47>>2]|0;
   $49 = HEAP32[$48>>2]|0;
   HEAP32[$nPrefixReg>>2] = $49;
   $50 = HEAP32[$4>>2]|0;
   $51 = (($50) + 24|0);
   $52 = HEAP8[$51>>0]|0;
   $53 = $52&255;
   $54 = $53 & 1;
   $55 = ($54|0)!=(0);
   if (!($55)) {
    $56 = HEAP32[$nPrefixReg>>2]|0;
    $57 = (($56) + 1)|0;
    HEAP32[$nPrefixReg>>2] = $57;
   }
   $58 = HEAP32[$nPrefixReg>>2]|0;
   $59 = HEAP32[$0>>2]|0;
   $60 = (($59) + 72|0);
   $61 = HEAP32[$60>>2]|0;
   $62 = (($61) + ($58))|0;
   HEAP32[$60>>2] = $62;
  }
  $63 = HEAP32[$0>>2]|0;
  $64 = (($63) + 72|0);
  $65 = HEAP32[$64>>2]|0;
  $66 = (($65) + 1)|0;
  $67 = HEAP32[$6>>2]|0;
  $68 = (($67) + 8|0);
  HEAP32[$68>>2] = $66;
  $69 = HEAP32[$nResultCol>>2]|0;
  $70 = HEAP32[$0>>2]|0;
  $71 = (($70) + 72|0);
  $72 = HEAP32[$71>>2]|0;
  $73 = (($72) + ($69))|0;
  HEAP32[$71>>2] = $73;
 } else {
  $74 = HEAP32[$6>>2]|0;
  $75 = (($74) + 8|0);
  $76 = HEAP32[$75>>2]|0;
  $77 = HEAP32[$nResultCol>>2]|0;
  $78 = (($76) + ($77))|0;
  $79 = HEAP32[$0>>2]|0;
  $80 = (($79) + 72|0);
  $81 = HEAP32[$80>>2]|0;
  $82 = ($78|0)>($81|0);
  if ($82) {
   $83 = HEAP32[$nResultCol>>2]|0;
   $84 = HEAP32[$0>>2]|0;
   $85 = (($84) + 72|0);
   $86 = HEAP32[$85>>2]|0;
   $87 = (($86) + ($83))|0;
   HEAP32[$85>>2] = $87;
  }
 }
 $88 = HEAP32[$nResultCol>>2]|0;
 $89 = HEAP32[$6>>2]|0;
 $90 = (($89) + 12|0);
 HEAP32[$90>>2] = $88;
 $91 = HEAP32[$6>>2]|0;
 $92 = (($91) + 8|0);
 $93 = HEAP32[$92>>2]|0;
 HEAP32[$regResult>>2] = $93;
 $94 = HEAP32[$3>>2]|0;
 $95 = ($94|0)>=(0);
 L22: do {
  if ($95) {
   HEAP32[$i>>2] = 0;
   while(1) {
    $96 = HEAP32[$i>>2]|0;
    $97 = HEAP32[$nResultCol>>2]|0;
    $98 = ($96|0)<($97|0);
    if (!($98)) {
     break L22;
    }
    $99 = HEAP32[$v>>2]|0;
    $100 = HEAP32[$3>>2]|0;
    $101 = HEAP32[$i>>2]|0;
    $102 = HEAP32[$regResult>>2]|0;
    $103 = HEAP32[$i>>2]|0;
    $104 = (($102) + ($103))|0;
    (_sqlite3VdbeAddOp3($99,47,$100,$101,$104)|0);
    $105 = HEAP32[$i>>2]|0;
    $106 = (($105) + 1)|0;
    HEAP32[$i>>2] = $106;
   }
  } else {
   $107 = HEAP32[$eDest>>2]|0;
   $108 = ($107|0)!=(3);
   if ($108) {
    $109 = HEAP32[$0>>2]|0;
    $110 = HEAP32[$2>>2]|0;
    $111 = HEAP32[$regResult>>2]|0;
    $112 = HEAP32[$eDest>>2]|0;
    $113 = ($112|0)==(9);
    if ($113) {
     $117 = 1;
    } else {
     $114 = HEAP32[$eDest>>2]|0;
     $115 = ($114|0)==(13);
     $117 = $115;
    }
    $116 = $117 ? 1 : 0;
    $118 = $116&255;
    (_sqlite3ExprCodeExprList($109,$110,$111,$118)|0);
   }
  }
 } while(0);
 $119 = HEAP32[$hasDistinct>>2]|0;
 $120 = ($119|0)!=(0);
 if ($120) {
  $121 = HEAP32[$5>>2]|0;
  $122 = (($121) + 1|0);
  $123 = HEAP8[$122>>0]|0;
  $124 = $123&255;
  if ((($124|0) == 1)) {
   $190 = HEAP32[$v>>2]|0;
   $191 = HEAP32[$5>>2]|0;
   $192 = (($191) + 8|0);
   $193 = HEAP32[$192>>2]|0;
   _sqlite3VdbeChangeToNoop($190,$193);
  } else if ((($124|0) == 2)) {
   $125 = HEAP32[$0>>2]|0;
   $126 = (($125) + 72|0);
   $127 = HEAP32[$126>>2]|0;
   $128 = (($127) + 1)|0;
   HEAP32[$regPrev>>2] = $128;
   $129 = HEAP32[$nResultCol>>2]|0;
   $130 = HEAP32[$0>>2]|0;
   $131 = (($130) + 72|0);
   $132 = HEAP32[$131>>2]|0;
   $133 = (($132) + ($129))|0;
   HEAP32[$131>>2] = $133;
   $134 = HEAP32[$v>>2]|0;
   $135 = HEAP32[$5>>2]|0;
   $136 = (($135) + 8|0);
   $137 = HEAP32[$136>>2]|0;
   _sqlite3VdbeChangeToNoop($134,$137);
   $138 = HEAP32[$v>>2]|0;
   $139 = HEAP32[$5>>2]|0;
   $140 = (($139) + 8|0);
   $141 = HEAP32[$140>>2]|0;
   $142 = (_sqlite3VdbeGetOp($138,$141)|0);
   HEAP32[$pOp>>2] = $142;
   $143 = HEAP32[$pOp>>2]|0;
   HEAP8[$143>>0] = 28;
   $144 = HEAP32[$pOp>>2]|0;
   $145 = (($144) + 4|0);
   HEAP32[$145>>2] = 1;
   $146 = HEAP32[$regPrev>>2]|0;
   $147 = HEAP32[$pOp>>2]|0;
   $148 = (($147) + 8|0);
   HEAP32[$148>>2] = $146;
   $149 = HEAP32[$v>>2]|0;
   $150 = (_sqlite3VdbeCurrentAddr($149)|0);
   $151 = HEAP32[$nResultCol>>2]|0;
   $152 = (($150) + ($151))|0;
   HEAP32[$iJump>>2] = $152;
   HEAP32[$i>>2] = 0;
   while(1) {
    $153 = HEAP32[$i>>2]|0;
    $154 = HEAP32[$nResultCol>>2]|0;
    $155 = ($153|0)<($154|0);
    if (!($155)) {
     break;
    }
    $156 = HEAP32[$0>>2]|0;
    $157 = HEAP32[$i>>2]|0;
    $158 = HEAP32[$2>>2]|0;
    $159 = (($158) + 4|0);
    $160 = HEAP32[$159>>2]|0;
    $161 = (($160) + (($157*20)|0)|0);
    $162 = HEAP32[$161>>2]|0;
    $163 = (_sqlite3ExprCollSeq($156,$162)|0);
    HEAP32[$pColl>>2] = $163;
    $164 = HEAP32[$i>>2]|0;
    $165 = HEAP32[$nResultCol>>2]|0;
    $166 = (($165) - 1)|0;
    $167 = ($164|0)<($166|0);
    $168 = HEAP32[$v>>2]|0;
    $169 = HEAP32[$regResult>>2]|0;
    $170 = HEAP32[$i>>2]|0;
    $171 = (($169) + ($170))|0;
    if ($167) {
     $172 = HEAP32[$iJump>>2]|0;
     $173 = HEAP32[$regPrev>>2]|0;
     $174 = HEAP32[$i>>2]|0;
     $175 = (($173) + ($174))|0;
     (_sqlite3VdbeAddOp3($168,78,$171,$172,$175)|0);
    } else {
     $176 = HEAP32[$7>>2]|0;
     $177 = HEAP32[$regPrev>>2]|0;
     $178 = HEAP32[$i>>2]|0;
     $179 = (($177) + ($178))|0;
     (_sqlite3VdbeAddOp3($168,79,$171,$176,$179)|0);
    }
    $180 = HEAP32[$v>>2]|0;
    $181 = HEAP32[$pColl>>2]|0;
    _sqlite3VdbeChangeP4($180,-1,$181,-4);
    $182 = HEAP32[$v>>2]|0;
    _sqlite3VdbeChangeP5($182,-128);
    $183 = HEAP32[$i>>2]|0;
    $184 = (($183) + 1)|0;
    HEAP32[$i>>2] = $184;
   }
   $185 = HEAP32[$v>>2]|0;
   $186 = HEAP32[$regResult>>2]|0;
   $187 = HEAP32[$regPrev>>2]|0;
   $188 = HEAP32[$nResultCol>>2]|0;
   $189 = (($188) - 1)|0;
   (_sqlite3VdbeAddOp3($185,33,$186,$187,$189)|0);
  } else {
   $194 = HEAP32[$0>>2]|0;
   $195 = HEAP32[$5>>2]|0;
   $196 = (($195) + 4|0);
   $197 = HEAP32[$196>>2]|0;
   $198 = HEAP32[$7>>2]|0;
   $199 = HEAP32[$nResultCol>>2]|0;
   $200 = HEAP32[$regResult>>2]|0;
   _codeDistinct($194,$197,$198,$199,$200);
  }
  $201 = HEAP32[$4>>2]|0;
  $202 = ($201|0)==(0|0);
  if ($202) {
   $203 = HEAP32[$v>>2]|0;
   $204 = HEAP32[$1>>2]|0;
   $205 = (($204) + 12|0);
   $206 = HEAP32[$205>>2]|0;
   $207 = HEAP32[$7>>2]|0;
   _codeOffset($203,$206,$207);
  }
 }
 $208 = HEAP32[$eDest>>2]|0;
 L50: do {
  switch ($208|0) {
  case 10:  {
   $304 = HEAP32[$4>>2]|0;
   $305 = ($304|0)!=(0|0);
   if ($305) {
    $306 = HEAP32[$0>>2]|0;
    $307 = HEAP32[$4>>2]|0;
    $308 = HEAP32[$1>>2]|0;
    $309 = HEAP32[$regResult>>2]|0;
    $310 = HEAP32[$nPrefixReg>>2]|0;
    _pushOntoSorter($306,$307,$308,$309,1,$310);
   }
   break;
  }
  case 12: case 14: case 6: case 5:  {
   $224 = HEAP32[$0>>2]|0;
   $225 = HEAP32[$nPrefixReg>>2]|0;
   $226 = (($225) + 1)|0;
   $227 = (_sqlite3GetTempRange($224,$226)|0);
   HEAP32[$r11>>2] = $227;
   $228 = HEAP32[$v>>2]|0;
   $229 = HEAP32[$regResult>>2]|0;
   $230 = HEAP32[$nResultCol>>2]|0;
   $231 = HEAP32[$r11>>2]|0;
   $232 = HEAP32[$nPrefixReg>>2]|0;
   $233 = (($231) + ($232))|0;
   (_sqlite3VdbeAddOp3($228,49,$229,$230,$233)|0);
   $234 = HEAP32[$eDest>>2]|0;
   $235 = ($234|0)==(6);
   if ($235) {
    $236 = HEAP32[$v>>2]|0;
    $237 = (_sqlite3VdbeCurrentAddr($236)|0);
    $238 = (($237) + 4)|0;
    HEAP32[$addr>>2] = $238;
    $239 = HEAP32[$v>>2]|0;
    $240 = HEAP32[$iParm>>2]|0;
    $241 = (($240) + 1)|0;
    $242 = HEAP32[$addr>>2]|0;
    $243 = HEAP32[$r11>>2]|0;
    (_sqlite3VdbeAddOp4Int($239,69,$241,$242,$243,0)|0);
    $244 = HEAP32[$v>>2]|0;
    $245 = HEAP32[$iParm>>2]|0;
    $246 = (($245) + 1)|0;
    $247 = HEAP32[$r11>>2]|0;
    (_sqlite3VdbeAddOp2($244,110,$246,$247)|0);
   }
   $248 = HEAP32[$4>>2]|0;
   $249 = ($248|0)!=(0|0);
   $250 = HEAP32[$0>>2]|0;
   if ($249) {
    $251 = HEAP32[$4>>2]|0;
    $252 = HEAP32[$1>>2]|0;
    $253 = HEAP32[$r11>>2]|0;
    $254 = HEAP32[$nPrefixReg>>2]|0;
    $255 = (($253) + ($254))|0;
    $256 = HEAP32[$nPrefixReg>>2]|0;
    _pushOntoSorter($250,$251,$252,$255,1,$256);
   } else {
    $257 = (_sqlite3GetTempReg($250)|0);
    HEAP32[$r2>>2] = $257;
    $258 = HEAP32[$v>>2]|0;
    $259 = HEAP32[$iParm>>2]|0;
    $260 = HEAP32[$r2>>2]|0;
    (_sqlite3VdbeAddOp2($258,74,$259,$260)|0);
    $261 = HEAP32[$v>>2]|0;
    $262 = HEAP32[$iParm>>2]|0;
    $263 = HEAP32[$r11>>2]|0;
    $264 = HEAP32[$r2>>2]|0;
    (_sqlite3VdbeAddOp3($261,75,$262,$263,$264)|0);
    $265 = HEAP32[$v>>2]|0;
    _sqlite3VdbeChangeP5($265,8);
    $266 = HEAP32[$0>>2]|0;
    $267 = HEAP32[$r2>>2]|0;
    _sqlite3ReleaseTempReg($266,$267);
   }
   $268 = HEAP32[$0>>2]|0;
   $269 = HEAP32[$r11>>2]|0;
   $270 = HEAP32[$nPrefixReg>>2]|0;
   $271 = (($270) + 1)|0;
   _sqlite3ReleaseTempRange($268,$269,$271);
   break;
  }
  case 11:  {
   $272 = HEAP32[$2>>2]|0;
   $273 = (($272) + 4|0);
   $274 = HEAP32[$273>>2]|0;
   $275 = HEAP32[$274>>2]|0;
   $276 = HEAP32[$6>>2]|0;
   $277 = (($276) + 1|0);
   $278 = HEAP8[$277>>0]|0;
   $279 = (_sqlite3CompareAffinity($275,$278)|0);
   $280 = HEAP32[$6>>2]|0;
   $281 = (($280) + 1|0);
   HEAP8[$281>>0] = $279;
   $282 = HEAP32[$4>>2]|0;
   $283 = ($282|0)!=(0|0);
   $284 = HEAP32[$0>>2]|0;
   if ($283) {
    $285 = HEAP32[$4>>2]|0;
    $286 = HEAP32[$1>>2]|0;
    $287 = HEAP32[$regResult>>2]|0;
    $288 = HEAP32[$nPrefixReg>>2]|0;
    _pushOntoSorter($284,$285,$286,$287,1,$288);
    break L50;
   } else {
    $289 = (_sqlite3GetTempReg($284)|0);
    HEAP32[$r12>>2] = $289;
    $290 = HEAP32[$v>>2]|0;
    $291 = HEAP32[$regResult>>2]|0;
    $292 = HEAP32[$r12>>2]|0;
    $293 = HEAP32[$6>>2]|0;
    $294 = (($293) + 1|0);
    (_sqlite3VdbeAddOp4($290,49,$291,1,$292,$294,1)|0);
    $295 = HEAP32[$0>>2]|0;
    $296 = HEAP32[$regResult>>2]|0;
    _sqlite3ExprCacheAffinityChange($295,$296,1);
    $297 = HEAP32[$v>>2]|0;
    $298 = HEAP32[$iParm>>2]|0;
    $299 = HEAP32[$r12>>2]|0;
    (_sqlite3VdbeAddOp2($297,110,$298,$299)|0);
    $300 = HEAP32[$0>>2]|0;
    $301 = HEAP32[$r12>>2]|0;
    _sqlite3ReleaseTempReg($300,$301);
    break L50;
   }
   break;
  }
  case 3:  {
   $302 = HEAP32[$v>>2]|0;
   $303 = HEAP32[$iParm>>2]|0;
   (_sqlite3VdbeAddOp2($302,25,1,$303)|0);
   break;
  }
  case 7: case 8:  {
   HEAP32[$addrTest>>2] = 0;
   $330 = HEAP32[$6>>2]|0;
   $331 = (($330) + 16|0);
   $332 = HEAP32[$331>>2]|0;
   HEAP32[$pSO>>2] = $332;
   $333 = HEAP32[$pSO>>2]|0;
   $334 = HEAP32[$333>>2]|0;
   HEAP32[$nKey>>2] = $334;
   $335 = HEAP32[$0>>2]|0;
   $336 = (_sqlite3GetTempReg($335)|0);
   HEAP32[$r13>>2] = $336;
   $337 = HEAP32[$0>>2]|0;
   $338 = HEAP32[$nKey>>2]|0;
   $339 = (($338) + 2)|0;
   $340 = (_sqlite3GetTempRange($337,$339)|0);
   HEAP32[$r24>>2] = $340;
   $341 = HEAP32[$r24>>2]|0;
   $342 = HEAP32[$nKey>>2]|0;
   $343 = (($341) + ($342))|0;
   $344 = (($343) + 1)|0;
   HEAP32[$r3>>2] = $344;
   $345 = HEAP32[$eDest>>2]|0;
   $346 = ($345|0)==(8);
   if ($346) {
    $347 = HEAP32[$v>>2]|0;
    $348 = HEAP32[$iParm>>2]|0;
    $349 = (($348) + 1)|0;
    $350 = HEAP32[$regResult>>2]|0;
    $351 = HEAP32[$nResultCol>>2]|0;
    $352 = (_sqlite3VdbeAddOp4Int($347,69,$349,0,$350,$351)|0);
    HEAP32[$addrTest>>2] = $352;
   }
   $353 = HEAP32[$v>>2]|0;
   $354 = HEAP32[$regResult>>2]|0;
   $355 = HEAP32[$nResultCol>>2]|0;
   $356 = HEAP32[$r3>>2]|0;
   (_sqlite3VdbeAddOp3($353,49,$354,$355,$356)|0);
   $357 = HEAP32[$eDest>>2]|0;
   $358 = ($357|0)==(8);
   if ($358) {
    $359 = HEAP32[$v>>2]|0;
    $360 = HEAP32[$iParm>>2]|0;
    $361 = (($360) + 1)|0;
    $362 = HEAP32[$r3>>2]|0;
    (_sqlite3VdbeAddOp2($359,110,$361,$362)|0);
    $363 = HEAP32[$v>>2]|0;
    _sqlite3VdbeChangeP5($363,16);
   }
   HEAP32[$i>>2] = 0;
   while(1) {
    $364 = HEAP32[$i>>2]|0;
    $365 = HEAP32[$nKey>>2]|0;
    $366 = ($364|0)<($365|0);
    $367 = HEAP32[$v>>2]|0;
    if (!($366)) {
     break;
    }
    $368 = HEAP32[$regResult>>2]|0;
    $369 = HEAP32[$i>>2]|0;
    $370 = HEAP32[$pSO>>2]|0;
    $371 = (($370) + 4|0);
    $372 = HEAP32[$371>>2]|0;
    $373 = (($372) + (($369*20)|0)|0);
    $374 = (($373) + 16|0);
    $375 = HEAP16[$374>>1]|0;
    $376 = $375&65535;
    $377 = (($368) + ($376))|0;
    $378 = (($377) - 1)|0;
    $379 = HEAP32[$r24>>2]|0;
    $380 = HEAP32[$i>>2]|0;
    $381 = (($379) + ($380))|0;
    (_sqlite3VdbeAddOp2($367,34,$378,$381)|0);
    $382 = HEAP32[$i>>2]|0;
    $383 = (($382) + 1)|0;
    HEAP32[$i>>2] = $383;
   }
   $384 = HEAP32[$iParm>>2]|0;
   $385 = HEAP32[$r24>>2]|0;
   $386 = HEAP32[$nKey>>2]|0;
   $387 = (($385) + ($386))|0;
   (_sqlite3VdbeAddOp2($367,73,$384,$387)|0);
   $388 = HEAP32[$v>>2]|0;
   $389 = HEAP32[$r24>>2]|0;
   $390 = HEAP32[$nKey>>2]|0;
   $391 = (($390) + 2)|0;
   $392 = HEAP32[$r13>>2]|0;
   (_sqlite3VdbeAddOp3($388,49,$389,$391,$392)|0);
   $393 = HEAP32[$v>>2]|0;
   $394 = HEAP32[$iParm>>2]|0;
   $395 = HEAP32[$r13>>2]|0;
   (_sqlite3VdbeAddOp2($393,110,$394,$395)|0);
   $396 = HEAP32[$addrTest>>2]|0;
   $397 = ($396|0)!=(0);
   if ($397) {
    $398 = HEAP32[$v>>2]|0;
    $399 = HEAP32[$addrTest>>2]|0;
    _sqlite3VdbeJumpHere($398,$399);
   }
   $400 = HEAP32[$0>>2]|0;
   $401 = HEAP32[$r13>>2]|0;
   _sqlite3ReleaseTempReg($400,$401);
   $402 = HEAP32[$0>>2]|0;
   $403 = HEAP32[$r24>>2]|0;
   $404 = HEAP32[$nKey>>2]|0;
   $405 = (($404) + 2)|0;
   _sqlite3ReleaseTempRange($402,$403,$405);
   break;
  }
  case 9: case 13:  {
   $311 = HEAP32[$4>>2]|0;
   $312 = ($311|0)!=(0|0);
   if ($312) {
    $313 = HEAP32[$0>>2]|0;
    $314 = HEAP32[$4>>2]|0;
    $315 = HEAP32[$1>>2]|0;
    $316 = HEAP32[$regResult>>2]|0;
    $317 = HEAP32[$nResultCol>>2]|0;
    $318 = HEAP32[$nPrefixReg>>2]|0;
    _pushOntoSorter($313,$314,$315,$316,$317,$318);
    break L50;
   }
   $319 = HEAP32[$eDest>>2]|0;
   $320 = ($319|0)==(13);
   $321 = HEAP32[$v>>2]|0;
   if ($320) {
    $322 = HEAP32[$6>>2]|0;
    $323 = (($322) + 4|0);
    $324 = HEAP32[$323>>2]|0;
    (_sqlite3VdbeAddOp1($321,22,$324)|0);
    break L50;
   } else {
    $325 = HEAP32[$regResult>>2]|0;
    $326 = HEAP32[$nResultCol>>2]|0;
    (_sqlite3VdbeAddOp2($321,35,$325,$326)|0);
    $327 = HEAP32[$0>>2]|0;
    $328 = HEAP32[$regResult>>2]|0;
    $329 = HEAP32[$nResultCol>>2]|0;
    _sqlite3ExprCacheAffinityChange($327,$328,$329);
    break L50;
   }
   break;
  }
  case 2:  {
   $220 = HEAP32[$v>>2]|0;
   $221 = HEAP32[$iParm>>2]|0;
   $222 = HEAP32[$regResult>>2]|0;
   $223 = HEAP32[$nResultCol>>2]|0;
   (_sqlite3VdbeAddOp3($220,111,$221,$222,$223)|0);
   break;
  }
  case 1:  {
   $209 = HEAP32[$0>>2]|0;
   $210 = (_sqlite3GetTempReg($209)|0);
   HEAP32[$r1>>2] = $210;
   $211 = HEAP32[$v>>2]|0;
   $212 = HEAP32[$regResult>>2]|0;
   $213 = HEAP32[$nResultCol>>2]|0;
   $214 = HEAP32[$r1>>2]|0;
   (_sqlite3VdbeAddOp3($211,49,$212,$213,$214)|0);
   $215 = HEAP32[$v>>2]|0;
   $216 = HEAP32[$iParm>>2]|0;
   $217 = HEAP32[$r1>>2]|0;
   (_sqlite3VdbeAddOp2($215,110,$216,$217)|0);
   $218 = HEAP32[$0>>2]|0;
   $219 = HEAP32[$r1>>2]|0;
   _sqlite3ReleaseTempReg($218,$219);
   break;
  }
  default: {
  }
  }
 } while(0);
 $406 = HEAP32[$4>>2]|0;
 $407 = ($406|0)==(0|0);
 if (!($407)) {
  STACKTOP = sp;return;
 }
 $408 = HEAP32[$1>>2]|0;
 $409 = (($408) + 8|0);
 $410 = HEAP32[$409>>2]|0;
 $411 = ($410|0)!=(0);
 if (!($411)) {
  STACKTOP = sp;return;
 }
 $412 = HEAP32[$v>>2]|0;
 $413 = HEAP32[$1>>2]|0;
 $414 = (($413) + 8|0);
 $415 = HEAP32[$414>>2]|0;
 $416 = HEAP32[$8>>2]|0;
 (_sqlite3VdbeAddOp3($412,139,$415,$416,-1)|0);
 STACKTOP = sp;return;
}
function _sqlite3WhereContinueLabel($pWInfo) {
 $pWInfo = $pWInfo|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 HEAP32[$0>>2] = $pWInfo;
 $1 = HEAP32[$0>>2]|0;
 $2 = (($1) + 48|0);
 $3 = HEAP32[$2>>2]|0;
 STACKTOP = sp;return ($3|0);
}
function _sqlite3WhereBreakLabel($pWInfo) {
 $pWInfo = $pWInfo|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 HEAP32[$0>>2] = $pWInfo;
 $1 = HEAP32[$0>>2]|0;
 $2 = (($1) + 52|0);
 $3 = HEAP32[$2>>2]|0;
 STACKTOP = sp;return ($3|0);
}
function _sqlite3ExprAnalyzeAggList($pNC,$pList) {
 $pNC = $pNC|0;
 $pList = $pList|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $i = 0, $pItem = 0;
 var label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $1 = sp + 8|0;
 $pItem = sp + 4|0;
 $i = sp;
 HEAP32[$0>>2] = $pNC;
 HEAP32[$1>>2] = $pList;
 $2 = HEAP32[$1>>2]|0;
 $3 = ($2|0)!=(0|0);
 if (!($3)) {
  STACKTOP = sp;return;
 }
 $4 = HEAP32[$1>>2]|0;
 $5 = (($4) + 4|0);
 $6 = HEAP32[$5>>2]|0;
 HEAP32[$pItem>>2] = $6;
 HEAP32[$i>>2] = 0;
 while(1) {
  $7 = HEAP32[$i>>2]|0;
  $8 = HEAP32[$1>>2]|0;
  $9 = HEAP32[$8>>2]|0;
  $10 = ($7|0)<($9|0);
  if (!($10)) {
   break;
  }
  $11 = HEAP32[$0>>2]|0;
  $12 = HEAP32[$pItem>>2]|0;
  $13 = HEAP32[$12>>2]|0;
  _sqlite3ExprAnalyzeAggregates($11,$13);
  $14 = HEAP32[$i>>2]|0;
  $15 = (($14) + 1)|0;
  HEAP32[$i>>2] = $15;
  $16 = HEAP32[$pItem>>2]|0;
  $17 = (($16) + 20|0);
  HEAP32[$pItem>>2] = $17;
 }
 STACKTOP = sp;return;
}
function _sqlite3ExprAnalyzeAggregates($pNC,$pExpr) {
 $pNC = $pNC|0;
 $pExpr = $pExpr|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $w = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 28|0;
 $1 = sp + 24|0;
 $w = sp;
 HEAP32[$0>>2] = $pNC;
 HEAP32[$1>>2] = $pExpr;
 ;HEAP32[$w+0>>2]=0|0;HEAP32[$w+4>>2]=0|0;HEAP32[$w+8>>2]=0|0;HEAP32[$w+12>>2]=0|0;HEAP32[$w+16>>2]=0|0;HEAP32[$w+20>>2]=0|0;
 HEAP32[$w>>2] = 48;
 $2 = (($w) + 4|0);
 HEAP32[$2>>2] = 49;
 $3 = HEAP32[$0>>2]|0;
 $4 = (($w) + 20|0);
 HEAP32[$4>>2] = $3;
 $5 = HEAP32[$1>>2]|0;
 (_sqlite3WalkExpr($w,$5)|0);
 STACKTOP = sp;return;
}
function _explainTempTable($pParse,$zUsage) {
 $pParse = $pParse|0;
 $zUsage = $zUsage|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $v = 0;
 var $vararg_buffer = 0, $zMsg = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $vararg_buffer = sp;
 $0 = sp + 16|0;
 $1 = sp + 12|0;
 $v = sp + 8|0;
 $zMsg = sp + 4|0;
 HEAP32[$0>>2] = $pParse;
 HEAP32[$1>>2] = $zUsage;
 $2 = HEAP32[$0>>2]|0;
 $3 = (($2) + 450|0);
 $4 = HEAP8[$3>>0]|0;
 $5 = $4&255;
 $6 = ($5|0)==(2);
 if (!($6)) {
  STACKTOP = sp;return;
 }
 $7 = HEAP32[$0>>2]|0;
 $8 = (($7) + 8|0);
 $9 = HEAP32[$8>>2]|0;
 HEAP32[$v>>2] = $9;
 $10 = HEAP32[$0>>2]|0;
 $11 = HEAP32[$10>>2]|0;
 $12 = HEAP32[$1>>2]|0;
 HEAP32[$vararg_buffer>>2] = $12;
 $13 = (_sqlite3MPrintf($11,25024,$vararg_buffer)|0);
 HEAP32[$zMsg>>2] = $13;
 $14 = HEAP32[$v>>2]|0;
 $15 = HEAP32[$0>>2]|0;
 $16 = (($15) + 464|0);
 $17 = HEAP32[$16>>2]|0;
 $18 = HEAP32[$zMsg>>2]|0;
 (_sqlite3VdbeAddOp4($14,155,$17,0,0,$18,-1)|0);
 STACKTOP = sp;return;
}
function _sqlite3WhereIsSorted($pWInfo) {
 $pWInfo = $pWInfo|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 HEAP32[$0>>2] = $pWInfo;
 $1 = HEAP32[$0>>2]|0;
 $2 = (($1) + 37|0);
 $3 = HEAP8[$2>>0]|0;
 $4 = $3&255;
 STACKTOP = sp;return ($4|0);
}
function _sqlite3ExprCodeMove($pParse,$iFrom,$iTo,$nReg) {
 $pParse = $pParse|0;
 $iFrom = $iFrom|0;
 $iTo = $iTo|0;
 $nReg = $nReg|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $1 = sp + 8|0;
 $2 = sp + 4|0;
 $3 = sp;
 HEAP32[$0>>2] = $pParse;
 HEAP32[$1>>2] = $iFrom;
 HEAP32[$2>>2] = $iTo;
 HEAP32[$3>>2] = $nReg;
 $4 = HEAP32[$0>>2]|0;
 $5 = (($4) + 8|0);
 $6 = HEAP32[$5>>2]|0;
 $7 = HEAP32[$1>>2]|0;
 $8 = HEAP32[$2>>2]|0;
 $9 = HEAP32[$3>>2]|0;
 (_sqlite3VdbeAddOp3($6,32,$7,$8,$9)|0);
 $10 = HEAP32[$0>>2]|0;
 $11 = HEAP32[$1>>2]|0;
 $12 = HEAP32[$3>>2]|0;
 _sqlite3ExprCacheRemove($10,$11,$12);
 STACKTOP = sp;return;
}
function _updateAccumulator($pParse,$pAggInfo) {
 $pParse = $pParse|0;
 $pAggInfo = $pAggInfo|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0;
 var $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0;
 var $addrHitTest = 0, $addrNext = 0, $i = 0, $j = 0, $nArg = 0, $pC = 0, $pColl = 0, $pF = 0, $pItem = 0, $pList = 0, $regAgg = 0, $regHit = 0, $v = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 64|0;
 $0 = sp + 56|0;
 $1 = sp + 52|0;
 $v = sp + 48|0;
 $i = sp + 44|0;
 $regHit = sp + 40|0;
 $addrHitTest = sp + 36|0;
 $pF = sp + 32|0;
 $pC = sp + 28|0;
 $nArg = sp + 24|0;
 $addrNext = sp + 20|0;
 $regAgg = sp + 16|0;
 $pList = sp + 12|0;
 $pColl = sp + 8|0;
 $pItem = sp + 4|0;
 $j = sp;
 HEAP32[$0>>2] = $pParse;
 HEAP32[$1>>2] = $pAggInfo;
 $2 = HEAP32[$0>>2]|0;
 $3 = (($2) + 8|0);
 $4 = HEAP32[$3>>2]|0;
 HEAP32[$v>>2] = $4;
 HEAP32[$regHit>>2] = 0;
 HEAP32[$addrHitTest>>2] = 0;
 $5 = HEAP32[$1>>2]|0;
 HEAP8[$5>>0] = 1;
 HEAP32[$i>>2] = 0;
 $6 = HEAP32[$1>>2]|0;
 $7 = (($6) + 40|0);
 $8 = HEAP32[$7>>2]|0;
 HEAP32[$pF>>2] = $8;
 while(1) {
  $9 = HEAP32[$i>>2]|0;
  $10 = HEAP32[$1>>2]|0;
  $11 = (($10) + 44|0);
  $12 = HEAP32[$11>>2]|0;
  $13 = ($9|0)<($12|0);
  if (!($13)) {
   break;
  }
  HEAP32[$addrNext>>2] = 0;
  $14 = HEAP32[$pF>>2]|0;
  $15 = HEAP32[$14>>2]|0;
  $16 = (($15) + 20|0);
  $17 = HEAP32[$16>>2]|0;
  HEAP32[$pList>>2] = $17;
  $18 = HEAP32[$pList>>2]|0;
  $19 = ($18|0)!=(0|0);
  if ($19) {
   $20 = HEAP32[$pList>>2]|0;
   $21 = HEAP32[$20>>2]|0;
   HEAP32[$nArg>>2] = $21;
   $22 = HEAP32[$0>>2]|0;
   $23 = HEAP32[$nArg>>2]|0;
   $24 = (_sqlite3GetTempRange($22,$23)|0);
   HEAP32[$regAgg>>2] = $24;
   $25 = HEAP32[$0>>2]|0;
   $26 = HEAP32[$pList>>2]|0;
   $27 = HEAP32[$regAgg>>2]|0;
   (_sqlite3ExprCodeExprList($25,$26,$27,1)|0);
  } else {
   HEAP32[$nArg>>2] = 0;
   HEAP32[$regAgg>>2] = 0;
  }
  $28 = HEAP32[$pF>>2]|0;
  $29 = (($28) + 12|0);
  $30 = HEAP32[$29>>2]|0;
  $31 = ($30|0)>=(0);
  if ($31) {
   $32 = HEAP32[$v>>2]|0;
   $33 = (_sqlite3VdbeMakeLabel($32)|0);
   HEAP32[$addrNext>>2] = $33;
   $34 = HEAP32[$0>>2]|0;
   $35 = HEAP32[$pF>>2]|0;
   $36 = (($35) + 12|0);
   $37 = HEAP32[$36>>2]|0;
   $38 = HEAP32[$addrNext>>2]|0;
   $39 = HEAP32[$regAgg>>2]|0;
   _codeDistinct($34,$37,$38,1,$39);
  }
  $40 = HEAP32[$pF>>2]|0;
  $41 = (($40) + 4|0);
  $42 = HEAP32[$41>>2]|0;
  $43 = (($42) + 2|0);
  $44 = HEAP16[$43>>1]|0;
  $45 = $44&65535;
  $46 = $45 & 32;
  $47 = ($46|0)!=(0);
  if ($47) {
   HEAP32[$pColl>>2] = 0;
   HEAP32[$j>>2] = 0;
   $48 = HEAP32[$pList>>2]|0;
   $49 = (($48) + 4|0);
   $50 = HEAP32[$49>>2]|0;
   HEAP32[$pItem>>2] = $50;
   while(1) {
    $51 = HEAP32[$pColl>>2]|0;
    $52 = ($51|0)!=(0|0);
    if ($52) {
     break;
    }
    $53 = HEAP32[$j>>2]|0;
    $54 = HEAP32[$nArg>>2]|0;
    $55 = ($53|0)<($54|0);
    if (!($55)) {
     break;
    }
    $56 = HEAP32[$0>>2]|0;
    $57 = HEAP32[$pItem>>2]|0;
    $58 = HEAP32[$57>>2]|0;
    $59 = (_sqlite3ExprCollSeq($56,$58)|0);
    HEAP32[$pColl>>2] = $59;
    $60 = HEAP32[$j>>2]|0;
    $61 = (($60) + 1)|0;
    HEAP32[$j>>2] = $61;
    $62 = HEAP32[$pItem>>2]|0;
    $63 = (($62) + 20|0);
    HEAP32[$pItem>>2] = $63;
   }
   $64 = HEAP32[$pColl>>2]|0;
   $65 = ($64|0)!=(0|0);
   if (!($65)) {
    $66 = HEAP32[$0>>2]|0;
    $67 = HEAP32[$66>>2]|0;
    $68 = (($67) + 8|0);
    $69 = HEAP32[$68>>2]|0;
    HEAP32[$pColl>>2] = $69;
   }
   $70 = HEAP32[$regHit>>2]|0;
   $71 = ($70|0)==(0);
   if ($71) {
    $72 = HEAP32[$1>>2]|0;
    $73 = (($72) + 36|0);
    $74 = HEAP32[$73>>2]|0;
    $75 = ($74|0)!=(0);
    if ($75) {
     $76 = HEAP32[$0>>2]|0;
     $77 = (($76) + 72|0);
     $78 = HEAP32[$77>>2]|0;
     $79 = (($78) + 1)|0;
     HEAP32[$77>>2] = $79;
     HEAP32[$regHit>>2] = $79;
    }
   }
   $80 = HEAP32[$v>>2]|0;
   $81 = HEAP32[$regHit>>2]|0;
   $82 = HEAP32[$pColl>>2]|0;
   (_sqlite3VdbeAddOp4($80,36,$81,0,0,$82,-4)|0);
  }
  $83 = HEAP32[$v>>2]|0;
  $84 = HEAP32[$regAgg>>2]|0;
  $85 = HEAP32[$pF>>2]|0;
  $86 = (($85) + 8|0);
  $87 = HEAP32[$86>>2]|0;
  $88 = HEAP32[$pF>>2]|0;
  $89 = (($88) + 4|0);
  $90 = HEAP32[$89>>2]|0;
  (_sqlite3VdbeAddOp4($83,10,0,$84,$87,$90,-5)|0);
  $91 = HEAP32[$v>>2]|0;
  $92 = HEAP32[$nArg>>2]|0;
  $93 = $92&255;
  _sqlite3VdbeChangeP5($91,$93);
  $94 = HEAP32[$0>>2]|0;
  $95 = HEAP32[$regAgg>>2]|0;
  $96 = HEAP32[$nArg>>2]|0;
  _sqlite3ExprCacheAffinityChange($94,$95,$96);
  $97 = HEAP32[$0>>2]|0;
  $98 = HEAP32[$regAgg>>2]|0;
  $99 = HEAP32[$nArg>>2]|0;
  _sqlite3ReleaseTempRange($97,$98,$99);
  $100 = HEAP32[$addrNext>>2]|0;
  $101 = ($100|0)!=(0);
  if ($101) {
   $102 = HEAP32[$v>>2]|0;
   $103 = HEAP32[$addrNext>>2]|0;
   _sqlite3VdbeResolveLabel($102,$103);
   $104 = HEAP32[$0>>2]|0;
   _sqlite3ExprCacheClear($104);
  }
  $105 = HEAP32[$i>>2]|0;
  $106 = (($105) + 1)|0;
  HEAP32[$i>>2] = $106;
  $107 = HEAP32[$pF>>2]|0;
  $108 = (($107) + 16|0);
  HEAP32[$pF>>2] = $108;
 }
 $109 = HEAP32[$regHit>>2]|0;
 $110 = ($109|0)!=(0);
 if ($110) {
  $111 = HEAP32[$v>>2]|0;
  $112 = HEAP32[$regHit>>2]|0;
  $113 = (_sqlite3VdbeAddOp1($111,45,$112)|0);
  HEAP32[$addrHitTest>>2] = $113;
 }
 $114 = HEAP32[$0>>2]|0;
 _sqlite3ExprCacheClear($114);
 HEAP32[$i>>2] = 0;
 $115 = HEAP32[$1>>2]|0;
 $116 = (($115) + 28|0);
 $117 = HEAP32[$116>>2]|0;
 HEAP32[$pC>>2] = $117;
 while(1) {
  $118 = HEAP32[$i>>2]|0;
  $119 = HEAP32[$1>>2]|0;
  $120 = (($119) + 36|0);
  $121 = HEAP32[$120>>2]|0;
  $122 = ($118|0)<($121|0);
  if (!($122)) {
   break;
  }
  $123 = HEAP32[$0>>2]|0;
  $124 = HEAP32[$pC>>2]|0;
  $125 = (($124) + 20|0);
  $126 = HEAP32[$125>>2]|0;
  $127 = HEAP32[$pC>>2]|0;
  $128 = (($127) + 16|0);
  $129 = HEAP32[$128>>2]|0;
  _sqlite3ExprCode($123,$126,$129);
  $130 = HEAP32[$i>>2]|0;
  $131 = (($130) + 1)|0;
  HEAP32[$i>>2] = $131;
  $132 = HEAP32[$pC>>2]|0;
  $133 = (($132) + 24|0);
  HEAP32[$pC>>2] = $133;
 }
 $134 = HEAP32[$1>>2]|0;
 HEAP8[$134>>0] = 0;
 $135 = HEAP32[$0>>2]|0;
 _sqlite3ExprCacheClear($135);
 $136 = HEAP32[$addrHitTest>>2]|0;
 $137 = ($136|0)!=(0);
 if (!($137)) {
  STACKTOP = sp;return;
 }
 $138 = HEAP32[$v>>2]|0;
 $139 = HEAP32[$addrHitTest>>2]|0;
 _sqlite3VdbeJumpHere($138,$139);
 STACKTOP = sp;return;
}
function _finalizeAggFunctions($pParse,$pAggInfo) {
 $pParse = $pParse|0;
 $pAggInfo = $pAggInfo|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $i = 0, $pF = 0, $pList = 0, $v = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 20|0;
 $1 = sp + 16|0;
 $v = sp + 12|0;
 $i = sp + 8|0;
 $pF = sp + 4|0;
 $pList = sp;
 HEAP32[$0>>2] = $pParse;
 HEAP32[$1>>2] = $pAggInfo;
 $2 = HEAP32[$0>>2]|0;
 $3 = (($2) + 8|0);
 $4 = HEAP32[$3>>2]|0;
 HEAP32[$v>>2] = $4;
 HEAP32[$i>>2] = 0;
 $5 = HEAP32[$1>>2]|0;
 $6 = (($5) + 40|0);
 $7 = HEAP32[$6>>2]|0;
 HEAP32[$pF>>2] = $7;
 while(1) {
  $8 = HEAP32[$i>>2]|0;
  $9 = HEAP32[$1>>2]|0;
  $10 = (($9) + 44|0);
  $11 = HEAP32[$10>>2]|0;
  $12 = ($8|0)<($11|0);
  if (!($12)) {
   break;
  }
  $13 = HEAP32[$pF>>2]|0;
  $14 = HEAP32[$13>>2]|0;
  $15 = (($14) + 20|0);
  $16 = HEAP32[$15>>2]|0;
  HEAP32[$pList>>2] = $16;
  $17 = HEAP32[$v>>2]|0;
  $18 = HEAP32[$pF>>2]|0;
  $19 = (($18) + 8|0);
  $20 = HEAP32[$19>>2]|0;
  $21 = HEAP32[$pList>>2]|0;
  $22 = ($21|0)!=(0|0);
  if ($22) {
   $23 = HEAP32[$pList>>2]|0;
   $24 = HEAP32[$23>>2]|0;
   $28 = $24;
  } else {
   $28 = 0;
  }
  $25 = HEAP32[$pF>>2]|0;
  $26 = (($25) + 4|0);
  $27 = HEAP32[$26>>2]|0;
  (_sqlite3VdbeAddOp4($17,140,$20,$28,0,$27,-5)|0);
  $29 = HEAP32[$i>>2]|0;
  $30 = (($29) + 1)|0;
  HEAP32[$i>>2] = $30;
  $31 = HEAP32[$pF>>2]|0;
  $32 = (($31) + 16|0);
  HEAP32[$pF>>2] = $32;
 }
 STACKTOP = sp;return;
}
function _resetAccumulator($pParse,$pAggInfo) {
 $pParse = $pParse|0;
 $pAggInfo = $pAggInfo|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $7 = 0, $8 = 0;
 var $9 = 0, $i = 0, $nReg = 0, $pE = 0, $pFunc = 0, $pKeyInfo = 0, $v = 0, $vararg_buffer = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $vararg_buffer = sp;
 $0 = sp + 32|0;
 $1 = sp + 28|0;
 $v = sp + 24|0;
 $i = sp + 20|0;
 $pFunc = sp + 16|0;
 $nReg = sp + 12|0;
 $pE = sp + 8|0;
 $pKeyInfo = sp + 4|0;
 HEAP32[$0>>2] = $pParse;
 HEAP32[$1>>2] = $pAggInfo;
 $2 = HEAP32[$0>>2]|0;
 $3 = (($2) + 8|0);
 $4 = HEAP32[$3>>2]|0;
 HEAP32[$v>>2] = $4;
 $5 = HEAP32[$1>>2]|0;
 $6 = (($5) + 44|0);
 $7 = HEAP32[$6>>2]|0;
 $8 = HEAP32[$1>>2]|0;
 $9 = (($8) + 32|0);
 $10 = HEAP32[$9>>2]|0;
 $11 = (($7) + ($10))|0;
 HEAP32[$nReg>>2] = $11;
 $12 = HEAP32[$nReg>>2]|0;
 $13 = ($12|0)==(0);
 if ($13) {
  STACKTOP = sp;return;
 }
 $14 = HEAP32[$v>>2]|0;
 $15 = HEAP32[$1>>2]|0;
 $16 = (($15) + 16|0);
 $17 = HEAP32[$16>>2]|0;
 $18 = HEAP32[$1>>2]|0;
 $19 = (($18) + 20|0);
 $20 = HEAP32[$19>>2]|0;
 (_sqlite3VdbeAddOp3($14,28,0,$17,$20)|0);
 $21 = HEAP32[$1>>2]|0;
 $22 = (($21) + 40|0);
 $23 = HEAP32[$22>>2]|0;
 HEAP32[$pFunc>>2] = $23;
 HEAP32[$i>>2] = 0;
 while(1) {
  $24 = HEAP32[$i>>2]|0;
  $25 = HEAP32[$1>>2]|0;
  $26 = (($25) + 44|0);
  $27 = HEAP32[$26>>2]|0;
  $28 = ($24|0)<($27|0);
  if (!($28)) {
   break;
  }
  $29 = HEAP32[$pFunc>>2]|0;
  $30 = (($29) + 12|0);
  $31 = HEAP32[$30>>2]|0;
  $32 = ($31|0)>=(0);
  do {
   if ($32) {
    $33 = HEAP32[$pFunc>>2]|0;
    $34 = HEAP32[$33>>2]|0;
    HEAP32[$pE>>2] = $34;
    $35 = HEAP32[$pE>>2]|0;
    $36 = (($35) + 20|0);
    $37 = HEAP32[$36>>2]|0;
    $38 = ($37|0)==(0|0);
    if (!($38)) {
     $39 = HEAP32[$pE>>2]|0;
     $40 = (($39) + 20|0);
     $41 = HEAP32[$40>>2]|0;
     $42 = HEAP32[$41>>2]|0;
     $43 = ($42|0)!=(1);
     if (!($43)) {
      $47 = HEAP32[$0>>2]|0;
      $48 = HEAP32[$pE>>2]|0;
      $49 = (($48) + 20|0);
      $50 = HEAP32[$49>>2]|0;
      $51 = (_keyInfoFromExprList($47,$50,0,0)|0);
      HEAP32[$pKeyInfo>>2] = $51;
      $52 = HEAP32[$v>>2]|0;
      $53 = HEAP32[$pFunc>>2]|0;
      $54 = (($53) + 12|0);
      $55 = HEAP32[$54>>2]|0;
      $56 = HEAP32[$pKeyInfo>>2]|0;
      (_sqlite3VdbeAddOp4($52,57,$55,0,0,$56,-6)|0);
      break;
     }
    }
    $44 = HEAP32[$0>>2]|0;
    _sqlite3ErrorMsg($44,24968,$vararg_buffer);
    $45 = HEAP32[$pFunc>>2]|0;
    $46 = (($45) + 12|0);
    HEAP32[$46>>2] = -1;
   }
  } while(0);
  $57 = HEAP32[$i>>2]|0;
  $58 = (($57) + 1)|0;
  HEAP32[$i>>2] = $58;
  $59 = HEAP32[$pFunc>>2]|0;
  $60 = (($59) + 16|0);
  HEAP32[$pFunc>>2] = $60;
 }
 STACKTOP = sp;return;
}
function _isSimpleCount($p,$pAggInfo) {
 $p = $p|0;
 $pAggInfo = $pAggInfo|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $7 = 0, $8 = 0, $9 = 0, $pExpr = 0, $pTab = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 16|0;
 $1 = sp + 12|0;
 $2 = sp + 8|0;
 $pTab = sp + 4|0;
 $pExpr = sp;
 HEAP32[$1>>2] = $p;
 HEAP32[$2>>2] = $pAggInfo;
 $3 = HEAP32[$1>>2]|0;
 $4 = (($3) + 36|0);
 $5 = HEAP32[$4>>2]|0;
 $6 = ($5|0)!=(0|0);
 if (!($6)) {
  $7 = HEAP32[$1>>2]|0;
  $8 = HEAP32[$7>>2]|0;
  $9 = HEAP32[$8>>2]|0;
  $10 = ($9|0)!=(1);
  if (!($10)) {
   $11 = HEAP32[$1>>2]|0;
   $12 = (($11) + 32|0);
   $13 = HEAP32[$12>>2]|0;
   $14 = HEAP32[$13>>2]|0;
   $15 = ($14|0)!=(1);
   if (!($15)) {
    $16 = HEAP32[$1>>2]|0;
    $17 = (($16) + 32|0);
    $18 = HEAP32[$17>>2]|0;
    $19 = (($18) + 8|0);
    $20 = (($19) + 20|0);
    $21 = HEAP32[$20>>2]|0;
    $22 = ($21|0)!=(0|0);
    if (!($22)) {
     $23 = HEAP32[$1>>2]|0;
     $24 = (($23) + 32|0);
     $25 = HEAP32[$24>>2]|0;
     $26 = (($25) + 8|0);
     $27 = (($26) + 16|0);
     $28 = HEAP32[$27>>2]|0;
     HEAP32[$pTab>>2] = $28;
     $29 = HEAP32[$1>>2]|0;
     $30 = HEAP32[$29>>2]|0;
     $31 = (($30) + 4|0);
     $32 = HEAP32[$31>>2]|0;
     $33 = HEAP32[$32>>2]|0;
     HEAP32[$pExpr>>2] = $33;
     $34 = HEAP32[$pTab>>2]|0;
     $35 = (($34) + 44|0);
     $36 = HEAP8[$35>>0]|0;
     $37 = $36&255;
     $38 = $37 & 16;
     $39 = ($38|0)!=(0);
     if ($39) {
      HEAP32[$0>>2] = 0;
      $64 = HEAP32[$0>>2]|0;
      STACKTOP = sp;return ($64|0);
     }
     $40 = HEAP32[$pExpr>>2]|0;
     $41 = HEAP8[$40>>0]|0;
     $42 = $41&255;
     $43 = ($42|0)!=(155);
     if ($43) {
      HEAP32[$0>>2] = 0;
      $64 = HEAP32[$0>>2]|0;
      STACKTOP = sp;return ($64|0);
     }
     $44 = HEAP32[$2>>2]|0;
     $45 = (($44) + 44|0);
     $46 = HEAP32[$45>>2]|0;
     $47 = ($46|0)==(0);
     if ($47) {
      HEAP32[$0>>2] = 0;
      $64 = HEAP32[$0>>2]|0;
      STACKTOP = sp;return ($64|0);
     }
     $48 = HEAP32[$2>>2]|0;
     $49 = (($48) + 40|0);
     $50 = HEAP32[$49>>2]|0;
     $51 = (($50) + 4|0);
     $52 = HEAP32[$51>>2]|0;
     $53 = (($52) + 2|0);
     $54 = HEAP16[$53>>1]|0;
     $55 = $54&65535;
     $56 = $55 & 256;
     $57 = ($56|0)==(0);
     if ($57) {
      HEAP32[$0>>2] = 0;
      $64 = HEAP32[$0>>2]|0;
      STACKTOP = sp;return ($64|0);
     }
     $58 = HEAP32[$pExpr>>2]|0;
     $59 = (($58) + 4|0);
     $60 = HEAP32[$59>>2]|0;
     $61 = $60 & 16;
     $62 = ($61|0)!=(0);
     if ($62) {
      HEAP32[$0>>2] = 0;
      $64 = HEAP32[$0>>2]|0;
      STACKTOP = sp;return ($64|0);
     } else {
      $63 = HEAP32[$pTab>>2]|0;
      HEAP32[$0>>2] = $63;
      $64 = HEAP32[$0>>2]|0;
      STACKTOP = sp;return ($64|0);
     }
    }
   }
  }
 }
 HEAP32[$0>>2] = 0;
 $64 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($64|0);
}
function _explainSimpleCount($pParse,$pTab,$pIdx) {
 $pParse = $pParse|0;
 $pTab = $pTab|0;
 $pIdx = $pIdx|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $bCover = 0, $vararg_buffer = 0, $vararg_ptr1 = 0, $vararg_ptr2 = 0, $zEqp = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $vararg_buffer = sp;
 $0 = sp + 28|0;
 $1 = sp + 24|0;
 $2 = sp + 20|0;
 $bCover = sp + 16|0;
 $zEqp = sp + 12|0;
 HEAP32[$0>>2] = $pParse;
 HEAP32[$1>>2] = $pTab;
 HEAP32[$2>>2] = $pIdx;
 $3 = HEAP32[$0>>2]|0;
 $4 = (($3) + 450|0);
 $5 = HEAP8[$4>>0]|0;
 $6 = $5&255;
 $7 = ($6|0)==(2);
 if (!($7)) {
  STACKTOP = sp;return;
 }
 $8 = HEAP32[$2>>2]|0;
 $9 = ($8|0)!=(0|0);
 if ($9) {
  $10 = HEAP32[$1>>2]|0;
  $11 = (($10) + 44|0);
  $12 = HEAP8[$11>>0]|0;
  $13 = $12&255;
  $14 = $13 & 32;
  $15 = ($14|0)==(0);
  if ($15) {
   $24 = 1;
  } else {
   $16 = HEAP32[$2>>2]|0;
   $17 = (($16) + 55|0);
   $18 = HEAP8[$17>>0]|0;
   $19 = $18 & 3;
   $20 = $19&255;
   $21 = ($20|0)==(2);
   $22 = $21 ^ 1;
   $24 = $22;
  }
 } else {
  $24 = 0;
 }
 $23 = $24&1;
 HEAP32[$bCover>>2] = $23;
 $25 = HEAP32[$0>>2]|0;
 $26 = HEAP32[$25>>2]|0;
 $27 = HEAP32[$1>>2]|0;
 $28 = HEAP32[$27>>2]|0;
 $29 = HEAP32[$bCover>>2]|0;
 $30 = ($29|0)!=(0);
 $31 = $30 ? 24944 : 1216;
 $32 = HEAP32[$bCover>>2]|0;
 $33 = ($32|0)!=(0);
 if ($33) {
  $34 = HEAP32[$2>>2]|0;
  $35 = HEAP32[$34>>2]|0;
  $36 = $35;
 } else {
  $36 = 1216;
 }
 HEAP32[$vararg_buffer>>2] = $28;
 $vararg_ptr1 = (($vararg_buffer) + 4|0);
 HEAP32[$vararg_ptr1>>2] = $31;
 $vararg_ptr2 = (($vararg_buffer) + 8|0);
 HEAP32[$vararg_ptr2>>2] = $36;
 $37 = (_sqlite3MPrintf($26,24920,$vararg_buffer)|0);
 HEAP32[$zEqp>>2] = $37;
 $38 = HEAP32[$0>>2]|0;
 $39 = (($38) + 8|0);
 $40 = HEAP32[$39>>2]|0;
 $41 = HEAP32[$0>>2]|0;
 $42 = (($41) + 464|0);
 $43 = HEAP32[$42>>2]|0;
 $44 = HEAP32[$zEqp>>2]|0;
 (_sqlite3VdbeAddOp4($40,155,$43,0,0,$44,-1)|0);
 STACKTOP = sp;return;
}
function _minMaxQuery($pAggInfo,$ppMinMax) {
 $pAggInfo = $pAggInfo|0;
 $ppMinMax = $ppMinMax|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0;
 var $9 = 0, $eRet = 0, $pEList = 0, $pExpr = 0, $zFunc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 20|0;
 $1 = sp + 16|0;
 $eRet = sp + 12|0;
 $pExpr = sp + 8|0;
 $pEList = sp + 4|0;
 $zFunc = sp;
 HEAP32[$0>>2] = $pAggInfo;
 HEAP32[$1>>2] = $ppMinMax;
 HEAP32[$eRet>>2] = 0;
 $2 = HEAP32[$1>>2]|0;
 HEAP32[$2>>2] = 0;
 $3 = HEAP32[$0>>2]|0;
 $4 = (($3) + 44|0);
 $5 = HEAP32[$4>>2]|0;
 $6 = ($5|0)==(1);
 do {
  if ($6) {
   $7 = HEAP32[$0>>2]|0;
   $8 = (($7) + 40|0);
   $9 = HEAP32[$8>>2]|0;
   $10 = HEAP32[$9>>2]|0;
   HEAP32[$pExpr>>2] = $10;
   $11 = HEAP32[$pExpr>>2]|0;
   $12 = (($11) + 20|0);
   $13 = HEAP32[$12>>2]|0;
   HEAP32[$pEList>>2] = $13;
   $14 = HEAP32[$pEList>>2]|0;
   $15 = ($14|0)!=(0|0);
   if ($15) {
    $16 = HEAP32[$pEList>>2]|0;
    $17 = HEAP32[$16>>2]|0;
    $18 = ($17|0)==(1);
    if ($18) {
     $19 = HEAP32[$pEList>>2]|0;
     $20 = (($19) + 4|0);
     $21 = HEAP32[$20>>2]|0;
     $22 = HEAP32[$21>>2]|0;
     $23 = HEAP8[$22>>0]|0;
     $24 = $23&255;
     $25 = ($24|0)==(156);
     if ($25) {
      $26 = HEAP32[$pExpr>>2]|0;
      $27 = (($26) + 8|0);
      $28 = HEAP32[$27>>2]|0;
      HEAP32[$zFunc>>2] = $28;
      $29 = HEAP32[$zFunc>>2]|0;
      $30 = (_sqlite3_stricmp($29,8312)|0);
      $31 = ($30|0)==(0);
      if ($31) {
       HEAP32[$eRet>>2] = 1;
       $32 = HEAP32[$pEList>>2]|0;
       $33 = HEAP32[$1>>2]|0;
       HEAP32[$33>>2] = $32;
       break;
      }
      $34 = HEAP32[$zFunc>>2]|0;
      $35 = (_sqlite3_stricmp($34,8320)|0);
      $36 = ($35|0)==(0);
      if ($36) {
       HEAP32[$eRet>>2] = 2;
       $37 = HEAP32[$pEList>>2]|0;
       $38 = HEAP32[$1>>2]|0;
       HEAP32[$38>>2] = $37;
      }
     }
    }
   }
  }
 } while(0);
 $39 = HEAP32[$eRet>>2]|0;
 $40 = $39&255;
 STACKTOP = sp;return ($40|0);
}
function _generateSortTail($pParse,$p,$pSort,$nColumn,$pDest) {
 $pParse = $pParse|0;
 $p = $p|0;
 $pSort = $pSort|0;
 $nColumn = $nColumn|0;
 $pDest = $pDest|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0;
 var $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0, $188 = 0;
 var $189 = 0, $19 = 0, $190 = 0, $191 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0;
 var $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0;
 var $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0;
 var $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0;
 var $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $addr = 0, $addrBreak = 0, $addrContinue = 0, $addrOnce = 0, $bSeq = 0, $eDest = 0, $i = 0, $iParm = 0;
 var $iSortTab = 0, $iTab = 0, $nKey = 0, $nSortData = 0, $or$cond = 0, $pOrderBy = 0, $regRow = 0, $regRowid = 0, $regSortOut = 0, $v = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 96|0;
 $0 = sp + 84|0;
 $1 = sp + 80|0;
 $2 = sp + 76|0;
 $3 = sp + 72|0;
 $4 = sp + 68|0;
 $v = sp + 64|0;
 $addrBreak = sp + 60|0;
 $addrContinue = sp + 56|0;
 $addr = sp + 52|0;
 $addrOnce = sp + 48|0;
 $iTab = sp + 44|0;
 $pOrderBy = sp + 40|0;
 $eDest = sp + 36|0;
 $iParm = sp + 32|0;
 $regRow = sp + 28|0;
 $regRowid = sp + 24|0;
 $nKey = sp + 20|0;
 $iSortTab = sp + 16|0;
 $nSortData = sp + 12|0;
 $i = sp + 8|0;
 $bSeq = sp + 4|0;
 $regSortOut = sp;
 HEAP32[$0>>2] = $pParse;
 HEAP32[$1>>2] = $p;
 HEAP32[$2>>2] = $pSort;
 HEAP32[$3>>2] = $nColumn;
 HEAP32[$4>>2] = $pDest;
 $5 = HEAP32[$0>>2]|0;
 $6 = (($5) + 8|0);
 $7 = HEAP32[$6>>2]|0;
 HEAP32[$v>>2] = $7;
 $8 = HEAP32[$v>>2]|0;
 $9 = (_sqlite3VdbeMakeLabel($8)|0);
 HEAP32[$addrBreak>>2] = $9;
 $10 = HEAP32[$v>>2]|0;
 $11 = (_sqlite3VdbeMakeLabel($10)|0);
 HEAP32[$addrContinue>>2] = $11;
 HEAP32[$addrOnce>>2] = 0;
 $12 = HEAP32[$2>>2]|0;
 $13 = HEAP32[$12>>2]|0;
 HEAP32[$pOrderBy>>2] = $13;
 $14 = HEAP32[$4>>2]|0;
 $15 = HEAP8[$14>>0]|0;
 $16 = $15&255;
 HEAP32[$eDest>>2] = $16;
 $17 = HEAP32[$4>>2]|0;
 $18 = (($17) + 4|0);
 $19 = HEAP32[$18>>2]|0;
 HEAP32[$iParm>>2] = $19;
 $20 = HEAP32[$2>>2]|0;
 $21 = (($20) + 16|0);
 $22 = HEAP32[$21>>2]|0;
 $23 = ($22|0)!=(0);
 if ($23) {
  $24 = HEAP32[$v>>2]|0;
  $25 = HEAP32[$2>>2]|0;
  $26 = (($25) + 12|0);
  $27 = HEAP32[$26>>2]|0;
  $28 = HEAP32[$2>>2]|0;
  $29 = (($28) + 16|0);
  $30 = HEAP32[$29>>2]|0;
  (_sqlite3VdbeAddOp2($24,17,$27,$30)|0);
  $31 = HEAP32[$v>>2]|0;
  $32 = HEAP32[$addrBreak>>2]|0;
  (_sqlite3VdbeAddOp2($31,16,0,$32)|0);
  $33 = HEAP32[$v>>2]|0;
  $34 = HEAP32[$2>>2]|0;
  $35 = (($34) + 16|0);
  $36 = HEAP32[$35>>2]|0;
  _sqlite3VdbeResolveLabel($33,$36);
 }
 $37 = HEAP32[$2>>2]|0;
 $38 = (($37) + 8|0);
 $39 = HEAP32[$38>>2]|0;
 HEAP32[$iTab>>2] = $39;
 $40 = HEAP32[$eDest>>2]|0;
 $41 = ($40|0)==(9);
 $42 = HEAP32[$eDest>>2]|0;
 $43 = ($42|0)==(13);
 $or$cond = $41 | $43;
 if ($or$cond) {
  HEAP32[$regRowid>>2] = 0;
  $44 = HEAP32[$4>>2]|0;
  $45 = (($44) + 8|0);
  $46 = HEAP32[$45>>2]|0;
  HEAP32[$regRow>>2] = $46;
  $47 = HEAP32[$3>>2]|0;
  HEAP32[$nSortData>>2] = $47;
 } else {
  $48 = HEAP32[$0>>2]|0;
  $49 = (_sqlite3GetTempReg($48)|0);
  HEAP32[$regRowid>>2] = $49;
  $50 = HEAP32[$0>>2]|0;
  $51 = (_sqlite3GetTempReg($50)|0);
  HEAP32[$regRow>>2] = $51;
  HEAP32[$nSortData>>2] = 1;
 }
 $52 = HEAP32[$pOrderBy>>2]|0;
 $53 = HEAP32[$52>>2]|0;
 $54 = HEAP32[$2>>2]|0;
 $55 = (($54) + 4|0);
 $56 = HEAP32[$55>>2]|0;
 $57 = (($53) - ($56))|0;
 HEAP32[$nKey>>2] = $57;
 $58 = HEAP32[$2>>2]|0;
 $59 = (($58) + 24|0);
 $60 = HEAP8[$59>>0]|0;
 $61 = $60&255;
 $62 = $61 & 1;
 $63 = ($62|0)!=(0);
 if ($63) {
  $64 = HEAP32[$0>>2]|0;
  $65 = (($64) + 72|0);
  $66 = HEAP32[$65>>2]|0;
  $67 = (($66) + 1)|0;
  HEAP32[$65>>2] = $67;
  HEAP32[$regSortOut>>2] = $67;
  $68 = HEAP32[$0>>2]|0;
  $69 = (($68) + 68|0);
  $70 = HEAP32[$69>>2]|0;
  $71 = (($70) + 1)|0;
  HEAP32[$69>>2] = $71;
  HEAP32[$iSortTab>>2] = $70;
  $72 = HEAP32[$2>>2]|0;
  $73 = (($72) + 16|0);
  $74 = HEAP32[$73>>2]|0;
  $75 = ($74|0)!=(0);
  if ($75) {
   $76 = HEAP32[$0>>2]|0;
   $77 = (_sqlite3CodeOnce($76)|0);
   HEAP32[$addrOnce>>2] = $77;
  }
  $78 = HEAP32[$v>>2]|0;
  $79 = HEAP32[$iSortTab>>2]|0;
  $80 = HEAP32[$regSortOut>>2]|0;
  $81 = HEAP32[$nKey>>2]|0;
  $82 = (($81) + 1)|0;
  $83 = HEAP32[$nSortData>>2]|0;
  $84 = (($82) + ($83))|0;
  (_sqlite3VdbeAddOp3($78,60,$79,$80,$84)|0);
  $85 = HEAP32[$addrOnce>>2]|0;
  $86 = ($85|0)!=(0);
  if ($86) {
   $87 = HEAP32[$v>>2]|0;
   $88 = HEAP32[$addrOnce>>2]|0;
   _sqlite3VdbeJumpHere($87,$88);
  }
  $89 = HEAP32[$v>>2]|0;
  $90 = HEAP32[$iTab>>2]|0;
  $91 = HEAP32[$addrBreak>>2]|0;
  $92 = (_sqlite3VdbeAddOp2($89,106,$90,$91)|0);
  $93 = (1 + ($92))|0;
  HEAP32[$addr>>2] = $93;
  $94 = HEAP32[$v>>2]|0;
  $95 = HEAP32[$1>>2]|0;
  $96 = (($95) + 12|0);
  $97 = HEAP32[$96>>2]|0;
  $98 = HEAP32[$addrContinue>>2]|0;
  _codeOffset($94,$97,$98);
  $99 = HEAP32[$v>>2]|0;
  $100 = HEAP32[$iTab>>2]|0;
  $101 = HEAP32[$regSortOut>>2]|0;
  $102 = HEAP32[$iSortTab>>2]|0;
  (_sqlite3VdbeAddOp3($99,100,$100,$101,$102)|0);
  HEAP32[$bSeq>>2] = 0;
 } else {
  $103 = HEAP32[$v>>2]|0;
  $104 = HEAP32[$iTab>>2]|0;
  $105 = HEAP32[$addrBreak>>2]|0;
  $106 = (_sqlite3VdbeAddOp2($103,107,$104,$105)|0);
  $107 = (1 + ($106))|0;
  HEAP32[$addr>>2] = $107;
  $108 = HEAP32[$v>>2]|0;
  $109 = HEAP32[$1>>2]|0;
  $110 = (($109) + 12|0);
  $111 = HEAP32[$110>>2]|0;
  $112 = HEAP32[$addrContinue>>2]|0;
  _codeOffset($108,$111,$112);
  $113 = HEAP32[$iTab>>2]|0;
  HEAP32[$iSortTab>>2] = $113;
  HEAP32[$bSeq>>2] = 1;
 }
 HEAP32[$i>>2] = 0;
 while(1) {
  $114 = HEAP32[$i>>2]|0;
  $115 = HEAP32[$nSortData>>2]|0;
  $116 = ($114|0)<($115|0);
  if (!($116)) {
   break;
  }
  $117 = HEAP32[$v>>2]|0;
  $118 = HEAP32[$iSortTab>>2]|0;
  $119 = HEAP32[$nKey>>2]|0;
  $120 = HEAP32[$bSeq>>2]|0;
  $121 = (($119) + ($120))|0;
  $122 = HEAP32[$i>>2]|0;
  $123 = (($121) + ($122))|0;
  $124 = HEAP32[$regRow>>2]|0;
  $125 = HEAP32[$i>>2]|0;
  $126 = (($124) + ($125))|0;
  (_sqlite3VdbeAddOp3($117,47,$118,$123,$126)|0);
  $127 = HEAP32[$i>>2]|0;
  $128 = (($127) + 1)|0;
  HEAP32[$i>>2] = $128;
 }
 $129 = HEAP32[$eDest>>2]|0;
 do {
  if ((($129|0) == 10)) {
   $148 = HEAP32[$0>>2]|0;
   $149 = HEAP32[$regRow>>2]|0;
   $150 = HEAP32[$iParm>>2]|0;
   _sqlite3ExprCodeMove($148,$149,$150,1);
  } else if ((($129|0) == 11)) {
   $138 = HEAP32[$v>>2]|0;
   $139 = HEAP32[$regRow>>2]|0;
   $140 = HEAP32[$regRowid>>2]|0;
   $141 = HEAP32[$4>>2]|0;
   $142 = (($141) + 1|0);
   (_sqlite3VdbeAddOp4($138,49,$139,1,$140,$142,1)|0);
   $143 = HEAP32[$0>>2]|0;
   $144 = HEAP32[$regRow>>2]|0;
   _sqlite3ExprCacheAffinityChange($143,$144,1);
   $145 = HEAP32[$v>>2]|0;
   $146 = HEAP32[$iParm>>2]|0;
   $147 = HEAP32[$regRowid>>2]|0;
   (_sqlite3VdbeAddOp2($145,110,$146,$147)|0);
  } else if ((($129|0) == 12) | (($129|0) == 14)) {
   $130 = HEAP32[$v>>2]|0;
   $131 = HEAP32[$iParm>>2]|0;
   $132 = HEAP32[$regRowid>>2]|0;
   (_sqlite3VdbeAddOp2($130,74,$131,$132)|0);
   $133 = HEAP32[$v>>2]|0;
   $134 = HEAP32[$iParm>>2]|0;
   $135 = HEAP32[$regRow>>2]|0;
   $136 = HEAP32[$regRowid>>2]|0;
   (_sqlite3VdbeAddOp3($133,75,$134,$135,$136)|0);
   $137 = HEAP32[$v>>2]|0;
   _sqlite3VdbeChangeP5($137,8);
  } else {
   $151 = HEAP32[$eDest>>2]|0;
   $152 = ($151|0)==(9);
   $153 = HEAP32[$v>>2]|0;
   $154 = HEAP32[$4>>2]|0;
   if ($152) {
    $155 = (($154) + 8|0);
    $156 = HEAP32[$155>>2]|0;
    $157 = HEAP32[$3>>2]|0;
    (_sqlite3VdbeAddOp2($153,35,$156,$157)|0);
    $158 = HEAP32[$0>>2]|0;
    $159 = HEAP32[$4>>2]|0;
    $160 = (($159) + 8|0);
    $161 = HEAP32[$160>>2]|0;
    $162 = HEAP32[$3>>2]|0;
    _sqlite3ExprCacheAffinityChange($158,$161,$162);
    break;
   } else {
    $163 = (($154) + 4|0);
    $164 = HEAP32[$163>>2]|0;
    (_sqlite3VdbeAddOp1($153,22,$164)|0);
    break;
   }
  }
 } while(0);
 $165 = HEAP32[$regRowid>>2]|0;
 $166 = ($165|0)!=(0);
 if ($166) {
  $167 = HEAP32[$0>>2]|0;
  $168 = HEAP32[$regRow>>2]|0;
  _sqlite3ReleaseTempReg($167,$168);
  $169 = HEAP32[$0>>2]|0;
  $170 = HEAP32[$regRowid>>2]|0;
  _sqlite3ReleaseTempReg($169,$170);
 }
 $171 = HEAP32[$v>>2]|0;
 $172 = HEAP32[$addrContinue>>2]|0;
 _sqlite3VdbeResolveLabel($171,$172);
 $173 = HEAP32[$2>>2]|0;
 $174 = (($173) + 24|0);
 $175 = HEAP8[$174>>0]|0;
 $176 = $175&255;
 $177 = $176 & 1;
 $178 = ($177|0)!=(0);
 $179 = HEAP32[$v>>2]|0;
 $180 = HEAP32[$iTab>>2]|0;
 $181 = HEAP32[$addr>>2]|0;
 if ($178) {
  (_sqlite3VdbeAddOp2($179,5,$180,$181)|0);
 } else {
  (_sqlite3VdbeAddOp2($179,9,$180,$181)|0);
 }
 $182 = HEAP32[$2>>2]|0;
 $183 = (($182) + 12|0);
 $184 = HEAP32[$183>>2]|0;
 $185 = ($184|0)!=(0);
 if (!($185)) {
  $190 = HEAP32[$v>>2]|0;
  $191 = HEAP32[$addrBreak>>2]|0;
  _sqlite3VdbeResolveLabel($190,$191);
  STACKTOP = sp;return;
 }
 $186 = HEAP32[$v>>2]|0;
 $187 = HEAP32[$2>>2]|0;
 $188 = (($187) + 12|0);
 $189 = HEAP32[$188>>2]|0;
 (_sqlite3VdbeAddOp1($186,18,$189)|0);
 $190 = HEAP32[$v>>2]|0;
 $191 = HEAP32[$addrBreak>>2]|0;
 _sqlite3VdbeResolveLabel($190,$191);
 STACKTOP = sp;return;
}
function _generateColumnNames($pParse,$pTabList,$pEList) {
 $pParse = $pParse|0;
 $pTabList = $pTabList|0;
 $pEList = $pEList|0;
 var $$old = 0, $$old2 = 0, $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0;
 var $114 = 0, $115 = 0, $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0;
 var $132 = 0, $133 = 0, $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0;
 var $150 = 0, $151 = 0, $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0;
 var $169 = 0, $17 = 0, $170 = 0, $171 = 0, $172 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0;
 var $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0;
 var $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0;
 var $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0;
 var $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $db = 0, $fullNames = 0, $i = 0, $iCol = 0, $j = 0;
 var $or$cond = 0, $or$cond3 = 0, $or$cond6 = 0, $p = 0, $pTab = 0, $shortNames = 0, $v = 0, $vararg_buffer8 = 0, $vararg_ptr7 = 0, $z = 0, $zCol = 0, $zName = 0, $zName1 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 80|0;
 $vararg_buffer8 = sp;
 $0 = sp + 68|0;
 $1 = sp + 64|0;
 $2 = sp + 60|0;
 $v = sp + 56|0;
 $i = sp + 52|0;
 $j = sp + 48|0;
 $db = sp + 44|0;
 $fullNames = sp + 40|0;
 $shortNames = sp + 36|0;
 $p = sp + 32|0;
 $zName = sp + 28|0;
 $pTab = sp + 24|0;
 $zCol = sp + 20|0;
 $iCol = sp + 16|0;
 $zName1 = sp + 12|0;
 $z = sp + 8|0;
 HEAP32[$0>>2] = $pParse;
 HEAP32[$1>>2] = $pTabList;
 HEAP32[$2>>2] = $pEList;
 $3 = HEAP32[$0>>2]|0;
 $4 = (($3) + 8|0);
 $5 = HEAP32[$4>>2]|0;
 HEAP32[$v>>2] = $5;
 $6 = HEAP32[$0>>2]|0;
 $7 = HEAP32[$6>>2]|0;
 HEAP32[$db>>2] = $7;
 $8 = HEAP32[$0>>2]|0;
 $9 = (($8) + 450|0);
 $10 = HEAP8[$9>>0]|0;
 $11 = ($10<<24>>24)!=(0);
 if ($11) {
  STACKTOP = sp;return;
 }
 $12 = HEAP32[$0>>2]|0;
 $13 = (($12) + 16|0);
 $14 = HEAP8[$13>>0]|0;
 $15 = $14&255;
 $16 = ($15|0)!=(0);
 $17 = HEAP32[$v>>2]|0;
 $18 = ($17|0)==(0|0);
 $or$cond = $16 | $18;
 if ($or$cond) {
  STACKTOP = sp;return;
 }
 $19 = HEAP32[$db>>2]|0;
 $20 = (($19) + 64|0);
 $21 = HEAP8[$20>>0]|0;
 $22 = $21&255;
 $23 = ($22|0)!=(0);
 if ($23) {
  STACKTOP = sp;return;
 }
 $24 = HEAP32[$0>>2]|0;
 $25 = (($24) + 16|0);
 HEAP8[$25>>0] = 1;
 $26 = HEAP32[$db>>2]|0;
 $27 = (($26) + 24|0);
 $28 = HEAP32[$27>>2]|0;
 $29 = $28 & 32;
 $30 = ($29|0)!=(0);
 $31 = $30&1;
 HEAP32[$fullNames>>2] = $31;
 $32 = HEAP32[$db>>2]|0;
 $33 = (($32) + 24|0);
 $34 = HEAP32[$33>>2]|0;
 $35 = $34 & 64;
 $36 = ($35|0)!=(0);
 $37 = $36&1;
 HEAP32[$shortNames>>2] = $37;
 $38 = HEAP32[$v>>2]|0;
 $39 = HEAP32[$2>>2]|0;
 $40 = HEAP32[$39>>2]|0;
 _sqlite3VdbeSetNumCols($38,$40);
 HEAP32[$i>>2] = 0;
 while(1) {
  $41 = HEAP32[$i>>2]|0;
  $42 = HEAP32[$2>>2]|0;
  $43 = HEAP32[$42>>2]|0;
  $44 = ($41|0)<($43|0);
  if (!($44)) {
   break;
  }
  $45 = HEAP32[$i>>2]|0;
  $46 = HEAP32[$2>>2]|0;
  $47 = (($46) + 4|0);
  $48 = HEAP32[$47>>2]|0;
  $49 = (($48) + (($45*20)|0)|0);
  $50 = HEAP32[$49>>2]|0;
  HEAP32[$p>>2] = $50;
  $51 = HEAP32[$p>>2]|0;
  $52 = ($51|0)==(0|0);
  do {
   if (!($52)) {
    $53 = HEAP32[$i>>2]|0;
    $54 = HEAP32[$2>>2]|0;
    $55 = (($54) + 4|0);
    $56 = HEAP32[$55>>2]|0;
    $57 = (($56) + (($53*20)|0)|0);
    $58 = (($57) + 4|0);
    $59 = HEAP32[$58>>2]|0;
    $60 = ($59|0)!=(0|0);
    if ($60) {
     $61 = HEAP32[$i>>2]|0;
     $62 = HEAP32[$2>>2]|0;
     $63 = (($62) + 4|0);
     $64 = HEAP32[$63>>2]|0;
     $65 = (($64) + (($61*20)|0)|0);
     $66 = (($65) + 4|0);
     $67 = HEAP32[$66>>2]|0;
     HEAP32[$zName>>2] = $67;
     $68 = HEAP32[$v>>2]|0;
     $69 = HEAP32[$i>>2]|0;
     $70 = HEAP32[$zName>>2]|0;
     (_sqlite3VdbeSetColName($68,$69,0,$70,(-1))|0);
     break;
    }
    $71 = HEAP32[$p>>2]|0;
    $72 = HEAP8[$71>>0]|0;
    $73 = $72&255;
    $74 = ($73|0)==(154);
    if ($74) {
     $$old = HEAP32[$1>>2]|0;
     $$old2 = ($$old|0)!=(0|0);
     if (!($$old2)) {
      label = 26;
     }
    } else {
     $75 = HEAP32[$p>>2]|0;
     $76 = HEAP8[$75>>0]|0;
     $77 = $76&255;
     $78 = ($77|0)==(156);
     $79 = HEAP32[$1>>2]|0;
     $80 = ($79|0)!=(0|0);
     $or$cond3 = $78 & $80;
     if (!($or$cond3)) {
      label = 26;
     }
    }
    if ((label|0) == 26) {
     label = 0;
     $149 = HEAP32[$i>>2]|0;
     $150 = HEAP32[$2>>2]|0;
     $151 = (($150) + 4|0);
     $152 = HEAP32[$151>>2]|0;
     $153 = (($152) + (($149*20)|0)|0);
     $154 = (($153) + 8|0);
     $155 = HEAP32[$154>>2]|0;
     HEAP32[$z>>2] = $155;
     $156 = HEAP32[$z>>2]|0;
     $157 = ($156|0)==(0|0);
     $158 = HEAP32[$db>>2]|0;
     if ($157) {
      $159 = HEAP32[$i>>2]|0;
      $160 = (($159) + 1)|0;
      HEAP32[$vararg_buffer8>>2] = $160;
      $161 = (_sqlite3MPrintf($158,24904,$vararg_buffer8)|0);
      $164 = $161;
     } else {
      $162 = HEAP32[$z>>2]|0;
      $163 = (_sqlite3DbStrDup($158,$162)|0);
      $164 = $163;
     }
     HEAP32[$z>>2] = $164;
     $165 = HEAP32[$v>>2]|0;
     $166 = HEAP32[$i>>2]|0;
     $167 = HEAP32[$z>>2]|0;
     (_sqlite3VdbeSetColName($165,$166,0,$167,16)|0);
     break;
    }
    $81 = HEAP32[$p>>2]|0;
    $82 = (($81) + 32|0);
    $83 = HEAP16[$82>>1]|0;
    $84 = $83 << 16 >> 16;
    HEAP32[$iCol>>2] = $84;
    HEAP32[$j>>2] = 0;
    while(1) {
     $85 = HEAP32[$j>>2]|0;
     $86 = HEAP32[$1>>2]|0;
     $87 = HEAP32[$86>>2]|0;
     $88 = ($85|0)<($87|0);
     if (!($88)) {
      break;
     }
     $89 = HEAP32[$j>>2]|0;
     $90 = HEAP32[$1>>2]|0;
     $91 = (($90) + 8|0);
     $92 = (($91) + (($89*72)|0)|0);
     $93 = (($92) + 40|0);
     $94 = HEAP32[$93>>2]|0;
     $95 = HEAP32[$p>>2]|0;
     $96 = (($95) + 28|0);
     $97 = HEAP32[$96>>2]|0;
     $98 = ($94|0)==($97|0);
     if ($98) {
      break;
     }
     $99 = HEAP32[$j>>2]|0;
     $100 = (($99) + 1)|0;
     HEAP32[$j>>2] = $100;
    }
    $101 = HEAP32[$j>>2]|0;
    $102 = HEAP32[$1>>2]|0;
    $103 = (($102) + 8|0);
    $104 = (($103) + (($101*72)|0)|0);
    $105 = (($104) + 16|0);
    $106 = HEAP32[$105>>2]|0;
    HEAP32[$pTab>>2] = $106;
    $107 = HEAP32[$iCol>>2]|0;
    $108 = ($107|0)<(0);
    if ($108) {
     $109 = HEAP32[$pTab>>2]|0;
     $110 = (($109) + 36|0);
     $111 = HEAP16[$110>>1]|0;
     $112 = $111 << 16 >> 16;
     HEAP32[$iCol>>2] = $112;
    }
    $113 = HEAP32[$iCol>>2]|0;
    $114 = ($113|0)<(0);
    if ($114) {
     HEAP32[$zCol>>2] = 9992;
    } else {
     $115 = HEAP32[$iCol>>2]|0;
     $116 = HEAP32[$pTab>>2]|0;
     $117 = (($116) + 4|0);
     $118 = HEAP32[$117>>2]|0;
     $119 = (($118) + (($115*24)|0)|0);
     $120 = HEAP32[$119>>2]|0;
     HEAP32[$zCol>>2] = $120;
    }
    $121 = HEAP32[$shortNames>>2]|0;
    $122 = ($121|0)!=(0);
    $123 = HEAP32[$fullNames>>2]|0;
    $124 = ($123|0)!=(0);
    $or$cond6 = $122 | $124;
    if (!($or$cond6)) {
     $125 = HEAP32[$v>>2]|0;
     $126 = HEAP32[$i>>2]|0;
     $127 = HEAP32[$db>>2]|0;
     $128 = HEAP32[$i>>2]|0;
     $129 = HEAP32[$2>>2]|0;
     $130 = (($129) + 4|0);
     $131 = HEAP32[$130>>2]|0;
     $132 = (($131) + (($128*20)|0)|0);
     $133 = (($132) + 8|0);
     $134 = HEAP32[$133>>2]|0;
     $135 = (_sqlite3DbStrDup($127,$134)|0);
     (_sqlite3VdbeSetColName($125,$126,0,$135,16)|0);
     break;
    }
    $136 = HEAP32[$fullNames>>2]|0;
    $137 = ($136|0)!=(0);
    if ($137) {
     HEAP32[$zName1>>2] = 0;
     $138 = HEAP32[$db>>2]|0;
     $139 = HEAP32[$pTab>>2]|0;
     $140 = HEAP32[$139>>2]|0;
     $141 = HEAP32[$zCol>>2]|0;
     HEAP32[$vararg_buffer8>>2] = $140;
     $vararg_ptr7 = (($vararg_buffer8) + 4|0);
     HEAP32[$vararg_ptr7>>2] = $141;
     $142 = (_sqlite3MPrintf($138,17448,$vararg_buffer8)|0);
     HEAP32[$zName1>>2] = $142;
     $143 = HEAP32[$v>>2]|0;
     $144 = HEAP32[$i>>2]|0;
     $145 = HEAP32[$zName1>>2]|0;
     (_sqlite3VdbeSetColName($143,$144,0,$145,16)|0);
     break;
    } else {
     $146 = HEAP32[$v>>2]|0;
     $147 = HEAP32[$i>>2]|0;
     $148 = HEAP32[$zCol>>2]|0;
     (_sqlite3VdbeSetColName($146,$147,0,$148,(-1))|0);
     break;
    }
   }
  } while(0);
  $168 = HEAP32[$i>>2]|0;
  $169 = (($168) + 1)|0;
  HEAP32[$i>>2] = $169;
 }
 $170 = HEAP32[$0>>2]|0;
 $171 = HEAP32[$1>>2]|0;
 $172 = HEAP32[$2>>2]|0;
 _generateColumnTypes($170,$171,$172);
 STACKTOP = sp;return;
}
function _generateColumnTypes($pParse,$pTabList,$pEList) {
 $pParse = $pParse|0;
 $pTabList = $pTabList|0;
 $pEList = $pEList|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $3 = 0;
 var $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $i = 0, $p = 0, $sNC = 0, $v = 0, $zType = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 64|0;
 $0 = sp + 56|0;
 $1 = sp + 52|0;
 $2 = sp + 48|0;
 $v = sp + 44|0;
 $i = sp + 40|0;
 $sNC = sp + 8|0;
 $p = sp + 4|0;
 $zType = sp;
 HEAP32[$0>>2] = $pParse;
 HEAP32[$1>>2] = $pTabList;
 HEAP32[$2>>2] = $pEList;
 $3 = HEAP32[$0>>2]|0;
 $4 = (($3) + 8|0);
 $5 = HEAP32[$4>>2]|0;
 HEAP32[$v>>2] = $5;
 $6 = HEAP32[$1>>2]|0;
 $7 = (($sNC) + 4|0);
 HEAP32[$7>>2] = $6;
 $8 = HEAP32[$0>>2]|0;
 HEAP32[$sNC>>2] = $8;
 HEAP32[$i>>2] = 0;
 while(1) {
  $9 = HEAP32[$i>>2]|0;
  $10 = HEAP32[$2>>2]|0;
  $11 = HEAP32[$10>>2]|0;
  $12 = ($9|0)<($11|0);
  if (!($12)) {
   break;
  }
  $13 = HEAP32[$i>>2]|0;
  $14 = HEAP32[$2>>2]|0;
  $15 = (($14) + 4|0);
  $16 = HEAP32[$15>>2]|0;
  $17 = (($16) + (($13*20)|0)|0);
  $18 = HEAP32[$17>>2]|0;
  HEAP32[$p>>2] = $18;
  $19 = HEAP32[$p>>2]|0;
  $20 = (_columnTypeImpl($sNC,$19,0)|0);
  HEAP32[$zType>>2] = $20;
  $21 = HEAP32[$v>>2]|0;
  $22 = HEAP32[$i>>2]|0;
  $23 = HEAP32[$zType>>2]|0;
  (_sqlite3VdbeSetColName($21,$22,1,$23,(-1))|0);
  $24 = HEAP32[$i>>2]|0;
  $25 = (($24) + 1)|0;
  HEAP32[$i>>2] = $25;
 }
 STACKTOP = sp;return;
}
function _codeOffset($v,$iOffset,$iContinue) {
 $v = $v|0;
 $iOffset = $iOffset|0;
 $iContinue = $iContinue|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $addr = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $1 = sp + 8|0;
 $2 = sp + 4|0;
 $addr = sp;
 HEAP32[$0>>2] = $v;
 HEAP32[$1>>2] = $iOffset;
 HEAP32[$2>>2] = $iContinue;
 $3 = HEAP32[$1>>2]|0;
 $4 = ($3|0)>(0);
 if (!($4)) {
  STACKTOP = sp;return;
 }
 $5 = HEAP32[$0>>2]|0;
 $6 = HEAP32[$1>>2]|0;
 $7 = (_sqlite3VdbeAddOp3($5,138,$6,0,-1)|0);
 HEAP32[$addr>>2] = $7;
 $8 = HEAP32[$0>>2]|0;
 $9 = HEAP32[$2>>2]|0;
 (_sqlite3VdbeAddOp2($8,16,0,$9)|0);
 $10 = HEAP32[$0>>2]|0;
 $11 = HEAP32[$addr>>2]|0;
 _sqlite3VdbeJumpHere($10,$11);
 STACKTOP = sp;return;
}
function _codeDistinct($pParse,$iTab,$addrRepeat,$N,$iMem) {
 $pParse = $pParse|0;
 $iTab = $iTab|0;
 $addrRepeat = $addrRepeat|0;
 $N = $N|0;
 $iMem = $iMem|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $3 = 0, $4 = 0, $5 = 0;
 var $6 = 0, $7 = 0, $8 = 0, $9 = 0, $r1 = 0, $v = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 24|0;
 $1 = sp + 20|0;
 $2 = sp + 16|0;
 $3 = sp + 12|0;
 $4 = sp + 8|0;
 $v = sp + 4|0;
 $r1 = sp;
 HEAP32[$0>>2] = $pParse;
 HEAP32[$1>>2] = $iTab;
 HEAP32[$2>>2] = $addrRepeat;
 HEAP32[$3>>2] = $N;
 HEAP32[$4>>2] = $iMem;
 $5 = HEAP32[$0>>2]|0;
 $6 = (($5) + 8|0);
 $7 = HEAP32[$6>>2]|0;
 HEAP32[$v>>2] = $7;
 $8 = HEAP32[$0>>2]|0;
 $9 = (_sqlite3GetTempReg($8)|0);
 HEAP32[$r1>>2] = $9;
 $10 = HEAP32[$v>>2]|0;
 $11 = HEAP32[$1>>2]|0;
 $12 = HEAP32[$2>>2]|0;
 $13 = HEAP32[$4>>2]|0;
 $14 = HEAP32[$3>>2]|0;
 (_sqlite3VdbeAddOp4Int($10,69,$11,$12,$13,$14)|0);
 $15 = HEAP32[$v>>2]|0;
 $16 = HEAP32[$4>>2]|0;
 $17 = HEAP32[$3>>2]|0;
 $18 = HEAP32[$r1>>2]|0;
 (_sqlite3VdbeAddOp3($15,49,$16,$17,$18)|0);
 $19 = HEAP32[$v>>2]|0;
 $20 = HEAP32[$1>>2]|0;
 $21 = HEAP32[$r1>>2]|0;
 (_sqlite3VdbeAddOp2($19,110,$20,$21)|0);
 $22 = HEAP32[$0>>2]|0;
 $23 = HEAP32[$r1>>2]|0;
 _sqlite3ReleaseTempReg($22,$23);
 STACKTOP = sp;return;
}
function _analyzeAggregate($pWalker,$pExpr) {
 $pWalker = $pWalker|0;
 $pExpr = $pExpr|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0;
 var $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0, $188 = 0;
 var $189 = 0, $19 = 0, $190 = 0, $191 = 0, $192 = 0, $193 = 0, $194 = 0, $195 = 0, $196 = 0, $197 = 0, $198 = 0, $199 = 0, $2 = 0, $20 = 0, $200 = 0, $201 = 0, $202 = 0, $203 = 0, $204 = 0, $205 = 0;
 var $206 = 0, $207 = 0, $208 = 0, $209 = 0, $21 = 0, $210 = 0, $211 = 0, $212 = 0, $213 = 0, $214 = 0, $215 = 0, $216 = 0, $217 = 0, $218 = 0, $219 = 0, $22 = 0, $220 = 0, $221 = 0, $222 = 0, $223 = 0;
 var $224 = 0, $225 = 0, $226 = 0, $227 = 0, $228 = 0, $229 = 0, $23 = 0, $230 = 0, $231 = 0, $232 = 0, $233 = 0, $234 = 0, $235 = 0, $236 = 0, $237 = 0, $238 = 0, $239 = 0, $24 = 0, $240 = 0, $241 = 0;
 var $242 = 0, $243 = 0, $244 = 0, $245 = 0, $246 = 0, $247 = 0, $248 = 0, $249 = 0, $25 = 0, $250 = 0, $251 = 0, $252 = 0, $253 = 0, $254 = 0, $255 = 0, $256 = 0, $257 = 0, $258 = 0, $259 = 0, $26 = 0;
 var $260 = 0, $261 = 0, $262 = 0, $263 = 0, $264 = 0, $265 = 0, $266 = 0, $267 = 0, $268 = 0, $269 = 0, $27 = 0, $270 = 0, $271 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0;
 var $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0;
 var $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0;
 var $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0;
 var $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $enc = 0, $i = 0, $j = 0, $k = 0, $n = 0, $pAggInfo = 0, $pCol = 0, $pE = 0;
 var $pGB = 0, $pItem = 0, $pItem1 = 0, $pNC = 0, $pParse = 0, $pSrcList = 0, $pTerm = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 80|0;
 $0 = sp + 64|0;
 $1 = sp + 60|0;
 $2 = sp + 56|0;
 $i = sp + 52|0;
 $pNC = sp + 48|0;
 $pParse = sp + 44|0;
 $pSrcList = sp + 40|0;
 $pAggInfo = sp + 36|0;
 $pItem = sp + 32|0;
 $pCol = sp + 28|0;
 $k = sp + 24|0;
 $j = sp + 20|0;
 $n = sp + 16|0;
 $pGB = sp + 12|0;
 $pTerm = sp + 8|0;
 $pE = sp + 4|0;
 $pItem1 = sp;
 $enc = sp + 68|0;
 HEAP32[$1>>2] = $pWalker;
 HEAP32[$2>>2] = $pExpr;
 $3 = HEAP32[$1>>2]|0;
 $4 = (($3) + 20|0);
 $5 = HEAP32[$4>>2]|0;
 HEAP32[$pNC>>2] = $5;
 $6 = HEAP32[$pNC>>2]|0;
 $7 = HEAP32[$6>>2]|0;
 HEAP32[$pParse>>2] = $7;
 $8 = HEAP32[$pNC>>2]|0;
 $9 = (($8) + 4|0);
 $10 = HEAP32[$9>>2]|0;
 HEAP32[$pSrcList>>2] = $10;
 $11 = HEAP32[$pNC>>2]|0;
 $12 = (($11) + 12|0);
 $13 = HEAP32[$12>>2]|0;
 HEAP32[$pAggInfo>>2] = $13;
 $14 = HEAP32[$2>>2]|0;
 $15 = HEAP8[$14>>0]|0;
 $16 = $15&255;
 if ((($16|0) == 154) | (($16|0) == 156)) {
  $17 = HEAP32[$pSrcList>>2]|0;
  $18 = ($17|0)!=(0|0);
  L3: do {
   if ($18) {
    $19 = HEAP32[$pSrcList>>2]|0;
    $20 = (($19) + 8|0);
    HEAP32[$pItem>>2] = $20;
    HEAP32[$i>>2] = 0;
    while(1) {
     $21 = HEAP32[$i>>2]|0;
     $22 = HEAP32[$pSrcList>>2]|0;
     $23 = HEAP32[$22>>2]|0;
     $24 = ($21|0)<($23|0);
     if (!($24)) {
      break L3;
     }
     $25 = HEAP32[$2>>2]|0;
     $26 = (($25) + 28|0);
     $27 = HEAP32[$26>>2]|0;
     $28 = HEAP32[$pItem>>2]|0;
     $29 = (($28) + 40|0);
     $30 = HEAP32[$29>>2]|0;
     $31 = ($27|0)==($30|0);
     if ($31) {
      break;
     }
     $162 = HEAP32[$i>>2]|0;
     $163 = (($162) + 1)|0;
     HEAP32[$i>>2] = $163;
     $164 = HEAP32[$pItem>>2]|0;
     $165 = (($164) + 72|0);
     HEAP32[$pItem>>2] = $165;
    }
    $32 = HEAP32[$pAggInfo>>2]|0;
    $33 = (($32) + 28|0);
    $34 = HEAP32[$33>>2]|0;
    HEAP32[$pCol>>2] = $34;
    HEAP32[$k>>2] = 0;
    while(1) {
     $35 = HEAP32[$k>>2]|0;
     $36 = HEAP32[$pAggInfo>>2]|0;
     $37 = (($36) + 32|0);
     $38 = HEAP32[$37>>2]|0;
     $39 = ($35|0)<($38|0);
     if (!($39)) {
      break;
     }
     $40 = HEAP32[$pCol>>2]|0;
     $41 = (($40) + 4|0);
     $42 = HEAP32[$41>>2]|0;
     $43 = HEAP32[$2>>2]|0;
     $44 = (($43) + 28|0);
     $45 = HEAP32[$44>>2]|0;
     $46 = ($42|0)==($45|0);
     if ($46) {
      $47 = HEAP32[$pCol>>2]|0;
      $48 = (($47) + 8|0);
      $49 = HEAP32[$48>>2]|0;
      $50 = HEAP32[$2>>2]|0;
      $51 = (($50) + 32|0);
      $52 = HEAP16[$51>>1]|0;
      $53 = $52 << 16 >> 16;
      $54 = ($49|0)==($53|0);
      if ($54) {
       break;
      }
     }
     $55 = HEAP32[$k>>2]|0;
     $56 = (($55) + 1)|0;
     HEAP32[$k>>2] = $56;
     $57 = HEAP32[$pCol>>2]|0;
     $58 = (($57) + 24|0);
     HEAP32[$pCol>>2] = $58;
    }
    $59 = HEAP32[$k>>2]|0;
    $60 = HEAP32[$pAggInfo>>2]|0;
    $61 = (($60) + 32|0);
    $62 = HEAP32[$61>>2]|0;
    $63 = ($59|0)>=($62|0);
    if ($63) {
     $64 = HEAP32[$pParse>>2]|0;
     $65 = HEAP32[$64>>2]|0;
     $66 = HEAP32[$pAggInfo>>2]|0;
     $67 = (_addAggInfoColumn($65,$66)|0);
     HEAP32[$k>>2] = $67;
     $68 = ($67|0)>=(0);
     if ($68) {
      $69 = HEAP32[$k>>2]|0;
      $70 = HEAP32[$pAggInfo>>2]|0;
      $71 = (($70) + 28|0);
      $72 = HEAP32[$71>>2]|0;
      $73 = (($72) + (($69*24)|0)|0);
      HEAP32[$pCol>>2] = $73;
      $74 = HEAP32[$2>>2]|0;
      $75 = (($74) + 44|0);
      $76 = HEAP32[$75>>2]|0;
      $77 = HEAP32[$pCol>>2]|0;
      HEAP32[$77>>2] = $76;
      $78 = HEAP32[$2>>2]|0;
      $79 = (($78) + 28|0);
      $80 = HEAP32[$79>>2]|0;
      $81 = HEAP32[$pCol>>2]|0;
      $82 = (($81) + 4|0);
      HEAP32[$82>>2] = $80;
      $83 = HEAP32[$2>>2]|0;
      $84 = (($83) + 32|0);
      $85 = HEAP16[$84>>1]|0;
      $86 = $85 << 16 >> 16;
      $87 = HEAP32[$pCol>>2]|0;
      $88 = (($87) + 8|0);
      HEAP32[$88>>2] = $86;
      $89 = HEAP32[$pParse>>2]|0;
      $90 = (($89) + 72|0);
      $91 = HEAP32[$90>>2]|0;
      $92 = (($91) + 1)|0;
      HEAP32[$90>>2] = $92;
      $93 = HEAP32[$pCol>>2]|0;
      $94 = (($93) + 16|0);
      HEAP32[$94>>2] = $92;
      $95 = HEAP32[$pCol>>2]|0;
      $96 = (($95) + 12|0);
      HEAP32[$96>>2] = -1;
      $97 = HEAP32[$2>>2]|0;
      $98 = HEAP32[$pCol>>2]|0;
      $99 = (($98) + 20|0);
      HEAP32[$99>>2] = $97;
      $100 = HEAP32[$pAggInfo>>2]|0;
      $101 = (($100) + 24|0);
      $102 = HEAP32[$101>>2]|0;
      $103 = ($102|0)!=(0|0);
      L20: do {
       if ($103) {
        $104 = HEAP32[$pAggInfo>>2]|0;
        $105 = (($104) + 24|0);
        $106 = HEAP32[$105>>2]|0;
        HEAP32[$pGB>>2] = $106;
        $107 = HEAP32[$pGB>>2]|0;
        $108 = (($107) + 4|0);
        $109 = HEAP32[$108>>2]|0;
        HEAP32[$pTerm>>2] = $109;
        $110 = HEAP32[$pGB>>2]|0;
        $111 = HEAP32[$110>>2]|0;
        HEAP32[$n>>2] = $111;
        HEAP32[$j>>2] = 0;
        while(1) {
         $112 = HEAP32[$j>>2]|0;
         $113 = HEAP32[$n>>2]|0;
         $114 = ($112|0)<($113|0);
         if (!($114)) {
          break L20;
         }
         $115 = HEAP32[$pTerm>>2]|0;
         $116 = HEAP32[$115>>2]|0;
         HEAP32[$pE>>2] = $116;
         $117 = HEAP32[$pE>>2]|0;
         $118 = HEAP8[$117>>0]|0;
         $119 = $118&255;
         $120 = ($119|0)==(154);
         if ($120) {
          $121 = HEAP32[$pE>>2]|0;
          $122 = (($121) + 28|0);
          $123 = HEAP32[$122>>2]|0;
          $124 = HEAP32[$2>>2]|0;
          $125 = (($124) + 28|0);
          $126 = HEAP32[$125>>2]|0;
          $127 = ($123|0)==($126|0);
          if ($127) {
           $128 = HEAP32[$pE>>2]|0;
           $129 = (($128) + 32|0);
           $130 = HEAP16[$129>>1]|0;
           $131 = $130 << 16 >> 16;
           $132 = HEAP32[$2>>2]|0;
           $133 = (($132) + 32|0);
           $134 = HEAP16[$133>>1]|0;
           $135 = $134 << 16 >> 16;
           $136 = ($131|0)==($135|0);
           if ($136) {
            break;
           }
          }
         }
         $140 = HEAP32[$j>>2]|0;
         $141 = (($140) + 1)|0;
         HEAP32[$j>>2] = $141;
         $142 = HEAP32[$pTerm>>2]|0;
         $143 = (($142) + 20|0);
         HEAP32[$pTerm>>2] = $143;
        }
        $137 = HEAP32[$j>>2]|0;
        $138 = HEAP32[$pCol>>2]|0;
        $139 = (($138) + 12|0);
        HEAP32[$139>>2] = $137;
       }
      } while(0);
      $144 = HEAP32[$pCol>>2]|0;
      $145 = (($144) + 12|0);
      $146 = HEAP32[$145>>2]|0;
      $147 = ($146|0)<(0);
      if ($147) {
       $148 = HEAP32[$pAggInfo>>2]|0;
       $149 = (($148) + 12|0);
       $150 = HEAP32[$149>>2]|0;
       $151 = (($150) + 1)|0;
       HEAP32[$149>>2] = $151;
       $152 = HEAP32[$pCol>>2]|0;
       $153 = (($152) + 12|0);
       HEAP32[$153>>2] = $150;
      }
     }
    }
    $154 = HEAP32[$pAggInfo>>2]|0;
    $155 = HEAP32[$2>>2]|0;
    $156 = (($155) + 40|0);
    HEAP32[$156>>2] = $154;
    $157 = HEAP32[$2>>2]|0;
    HEAP8[$157>>0] = -100;
    $158 = HEAP32[$k>>2]|0;
    $159 = $158&65535;
    $160 = HEAP32[$2>>2]|0;
    $161 = (($160) + 34|0);
    HEAP16[$161>>1] = $159;
   }
  } while(0);
  HEAP32[$0>>2] = 1;
  $271 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($271|0);
 } else if ((($16|0) == 155)) {
  $166 = HEAP32[$pNC>>2]|0;
  $167 = (($166) + 28|0);
  $168 = HEAP16[$167>>1]|0;
  $169 = $168&65535;
  $170 = $169 & 8;
  $171 = ($170|0)==(0);
  if ($171) {
   $172 = HEAP32[$1>>2]|0;
   $173 = (($172) + 16|0);
   $174 = HEAP32[$173>>2]|0;
   $175 = HEAP32[$2>>2]|0;
   $176 = (($175) + 38|0);
   $177 = HEAP8[$176>>0]|0;
   $178 = $177&255;
   $179 = ($174|0)==($178|0);
   if ($179) {
    $180 = HEAP32[$pAggInfo>>2]|0;
    $181 = (($180) + 40|0);
    $182 = HEAP32[$181>>2]|0;
    HEAP32[$pItem1>>2] = $182;
    HEAP32[$i>>2] = 0;
    while(1) {
     $183 = HEAP32[$i>>2]|0;
     $184 = HEAP32[$pAggInfo>>2]|0;
     $185 = (($184) + 44|0);
     $186 = HEAP32[$185>>2]|0;
     $187 = ($183|0)<($186|0);
     if (!($187)) {
      break;
     }
     $188 = HEAP32[$pItem1>>2]|0;
     $189 = HEAP32[$188>>2]|0;
     $190 = HEAP32[$2>>2]|0;
     $191 = (_sqlite3ExprCompare($189,$190,-1)|0);
     $192 = ($191|0)==(0);
     if ($192) {
      break;
     }
     $193 = HEAP32[$i>>2]|0;
     $194 = (($193) + 1)|0;
     HEAP32[$i>>2] = $194;
     $195 = HEAP32[$pItem1>>2]|0;
     $196 = (($195) + 16|0);
     HEAP32[$pItem1>>2] = $196;
    }
    $197 = HEAP32[$i>>2]|0;
    $198 = HEAP32[$pAggInfo>>2]|0;
    $199 = (($198) + 44|0);
    $200 = HEAP32[$199>>2]|0;
    $201 = ($197|0)>=($200|0);
    do {
     if ($201) {
      $202 = HEAP32[$pParse>>2]|0;
      $203 = HEAP32[$202>>2]|0;
      $204 = (($203) + 16|0);
      $205 = HEAP32[$204>>2]|0;
      $206 = (($205) + 12|0);
      $207 = HEAP32[$206>>2]|0;
      $208 = (($207) + 77|0);
      $209 = HEAP8[$208>>0]|0;
      HEAP8[$enc>>0] = $209;
      $210 = HEAP32[$pParse>>2]|0;
      $211 = HEAP32[$210>>2]|0;
      $212 = HEAP32[$pAggInfo>>2]|0;
      $213 = (_addAggInfoFunc($211,$212)|0);
      HEAP32[$i>>2] = $213;
      $214 = HEAP32[$i>>2]|0;
      $215 = ($214|0)>=(0);
      if ($215) {
       $216 = HEAP32[$i>>2]|0;
       $217 = HEAP32[$pAggInfo>>2]|0;
       $218 = (($217) + 40|0);
       $219 = HEAP32[$218>>2]|0;
       $220 = (($219) + ($216<<4)|0);
       HEAP32[$pItem1>>2] = $220;
       $221 = HEAP32[$2>>2]|0;
       $222 = HEAP32[$pItem1>>2]|0;
       HEAP32[$222>>2] = $221;
       $223 = HEAP32[$pParse>>2]|0;
       $224 = (($223) + 72|0);
       $225 = HEAP32[$224>>2]|0;
       $226 = (($225) + 1)|0;
       HEAP32[$224>>2] = $226;
       $227 = HEAP32[$pItem1>>2]|0;
       $228 = (($227) + 8|0);
       HEAP32[$228>>2] = $226;
       $229 = HEAP32[$pParse>>2]|0;
       $230 = HEAP32[$229>>2]|0;
       $231 = HEAP32[$2>>2]|0;
       $232 = (($231) + 8|0);
       $233 = HEAP32[$232>>2]|0;
       $234 = HEAP32[$2>>2]|0;
       $235 = (($234) + 8|0);
       $236 = HEAP32[$235>>2]|0;
       $237 = (_sqlite3Strlen30($236)|0);
       $238 = HEAP32[$2>>2]|0;
       $239 = (($238) + 20|0);
       $240 = HEAP32[$239>>2]|0;
       $241 = ($240|0)!=(0|0);
       if ($241) {
        $242 = HEAP32[$2>>2]|0;
        $243 = (($242) + 20|0);
        $244 = HEAP32[$243>>2]|0;
        $245 = HEAP32[$244>>2]|0;
        $247 = $245;
       } else {
        $247 = 0;
       }
       $246 = HEAP8[$enc>>0]|0;
       $248 = (_sqlite3FindFunction($230,$233,$237,$247,$246,0)|0);
       $249 = HEAP32[$pItem1>>2]|0;
       $250 = (($249) + 4|0);
       HEAP32[$250>>2] = $248;
       $251 = HEAP32[$2>>2]|0;
       $252 = (($251) + 4|0);
       $253 = HEAP32[$252>>2]|0;
       $254 = $253 & 16;
       $255 = ($254|0)!=(0);
       if ($255) {
        $256 = HEAP32[$pParse>>2]|0;
        $257 = (($256) + 68|0);
        $258 = HEAP32[$257>>2]|0;
        $259 = (($258) + 1)|0;
        HEAP32[$257>>2] = $259;
        $260 = HEAP32[$pItem1>>2]|0;
        $261 = (($260) + 12|0);
        HEAP32[$261>>2] = $258;
        break;
       } else {
        $262 = HEAP32[$pItem1>>2]|0;
        $263 = (($262) + 12|0);
        HEAP32[$263>>2] = -1;
        break;
       }
      }
     }
    } while(0);
    $264 = HEAP32[$i>>2]|0;
    $265 = $264&65535;
    $266 = HEAP32[$2>>2]|0;
    $267 = (($266) + 34|0);
    HEAP16[$267>>1] = $265;
    $268 = HEAP32[$pAggInfo>>2]|0;
    $269 = HEAP32[$2>>2]|0;
    $270 = (($269) + 40|0);
    HEAP32[$270>>2] = $268;
    HEAP32[$0>>2] = 1;
    $271 = HEAP32[$0>>2]|0;
    STACKTOP = sp;return ($271|0);
   }
  }
  HEAP32[$0>>2] = 0;
  $271 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($271|0);
 } else {
  HEAP32[$0>>2] = 0;
  $271 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($271|0);
 }
 return 0|0;
}
function _analyzeAggregatesInSelect($pWalker,$pSelect) {
 $pWalker = $pWalker|0;
 $pSelect = $pSelect|0;
 var $0 = 0, $1 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $1 = sp;
 HEAP32[$0>>2] = $pWalker;
 HEAP32[$1>>2] = $pSelect;
 STACKTOP = sp;return 0;
}
function _addAggInfoColumn($db,$pInfo) {
 $db = $db|0;
 $pInfo = $pInfo|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $i = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $1 = sp + 4|0;
 $i = sp;
 HEAP32[$0>>2] = $db;
 HEAP32[$1>>2] = $pInfo;
 $2 = HEAP32[$0>>2]|0;
 $3 = HEAP32[$1>>2]|0;
 $4 = (($3) + 28|0);
 $5 = HEAP32[$4>>2]|0;
 $6 = HEAP32[$1>>2]|0;
 $7 = (($6) + 32|0);
 $8 = (_sqlite3ArrayAllocate($2,$5,24,$7,$i)|0);
 $9 = HEAP32[$1>>2]|0;
 $10 = (($9) + 28|0);
 HEAP32[$10>>2] = $8;
 $11 = HEAP32[$i>>2]|0;
 STACKTOP = sp;return ($11|0);
}
function _addAggInfoFunc($db,$pInfo) {
 $db = $db|0;
 $pInfo = $pInfo|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $i = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $1 = sp + 4|0;
 $i = sp;
 HEAP32[$0>>2] = $db;
 HEAP32[$1>>2] = $pInfo;
 $2 = HEAP32[$0>>2]|0;
 $3 = HEAP32[$1>>2]|0;
 $4 = (($3) + 40|0);
 $5 = HEAP32[$4>>2]|0;
 $6 = HEAP32[$1>>2]|0;
 $7 = (($6) + 44|0);
 $8 = (_sqlite3ArrayAllocate($2,$5,16,$7,$i)|0);
 $9 = HEAP32[$1>>2]|0;
 $10 = (($9) + 40|0);
 HEAP32[$10>>2] = $8;
 $11 = HEAP32[$i>>2]|0;
 STACKTOP = sp;return ($11|0);
}
function _pushOntoSorter($pParse,$pSort,$pSelect,$regData,$nData,$nPrefixReg) {
 $pParse = $pParse|0;
 $pSort = $pSort|0;
 $pSelect = $pSelect|0;
 $regData = $regData|0;
 $nData = $nData|0;
 $nPrefixReg = $nPrefixReg|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0;
 var $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0, $188 = 0;
 var $189 = 0, $19 = 0, $190 = 0, $191 = 0, $192 = 0, $193 = 0, $194 = 0, $195 = 0, $196 = 0, $197 = 0, $198 = 0, $199 = 0, $2 = 0, $20 = 0, $200 = 0, $201 = 0, $202 = 0, $203 = 0, $204 = 0, $205 = 0;
 var $206 = 0, $207 = 0, $208 = 0, $209 = 0, $21 = 0, $210 = 0, $211 = 0, $212 = 0, $213 = 0, $214 = 0, $215 = 0, $216 = 0, $217 = 0, $218 = 0, $219 = 0, $22 = 0, $220 = 0, $221 = 0, $222 = 0, $223 = 0;
 var $224 = 0, $225 = 0, $226 = 0, $227 = 0, $228 = 0, $229 = 0, $23 = 0, $230 = 0, $231 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0;
 var $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0;
 var $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0;
 var $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0;
 var $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $addr1 = 0, $addr2 = 0, $addrFirst = 0, $addrJmp = 0, $bSeq = 0, $iLimit = 0, $nBase = 0, $nExpr = 0;
 var $nKey = 0, $nOBSat = 0, $op = 0, $pKI = 0, $pOp = 0, $regBase = 0, $regPrevKey = 0, $regRecord = 0, $v = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 96|0;
 $0 = sp + 88|0;
 $1 = sp + 84|0;
 $2 = sp + 80|0;
 $3 = sp + 76|0;
 $4 = sp + 72|0;
 $5 = sp + 68|0;
 $v = sp + 64|0;
 $bSeq = sp + 60|0;
 $nExpr = sp + 56|0;
 $nBase = sp + 52|0;
 $regBase = sp + 48|0;
 $regRecord = sp + 44|0;
 $nOBSat = sp + 40|0;
 $op = sp + 36|0;
 $regPrevKey = sp + 32|0;
 $addrFirst = sp + 28|0;
 $addrJmp = sp + 24|0;
 $pOp = sp + 20|0;
 $nKey = sp + 16|0;
 $pKI = sp + 12|0;
 $addr1 = sp + 8|0;
 $addr2 = sp + 4|0;
 $iLimit = sp;
 HEAP32[$0>>2] = $pParse;
 HEAP32[$1>>2] = $pSort;
 HEAP32[$2>>2] = $pSelect;
 HEAP32[$3>>2] = $regData;
 HEAP32[$4>>2] = $nData;
 HEAP32[$5>>2] = $nPrefixReg;
 $6 = HEAP32[$0>>2]|0;
 $7 = (($6) + 8|0);
 $8 = HEAP32[$7>>2]|0;
 HEAP32[$v>>2] = $8;
 $9 = HEAP32[$1>>2]|0;
 $10 = (($9) + 24|0);
 $11 = HEAP8[$10>>0]|0;
 $12 = $11&255;
 $13 = $12 & 1;
 $14 = ($13|0)==(0);
 $15 = $14&1;
 HEAP32[$bSeq>>2] = $15;
 $16 = HEAP32[$1>>2]|0;
 $17 = HEAP32[$16>>2]|0;
 $18 = HEAP32[$17>>2]|0;
 HEAP32[$nExpr>>2] = $18;
 $19 = HEAP32[$nExpr>>2]|0;
 $20 = HEAP32[$bSeq>>2]|0;
 $21 = (($19) + ($20))|0;
 $22 = HEAP32[$4>>2]|0;
 $23 = (($21) + ($22))|0;
 HEAP32[$nBase>>2] = $23;
 $24 = HEAP32[$0>>2]|0;
 $25 = (($24) + 72|0);
 $26 = HEAP32[$25>>2]|0;
 $27 = (($26) + 1)|0;
 HEAP32[$25>>2] = $27;
 HEAP32[$regRecord>>2] = $27;
 $28 = HEAP32[$1>>2]|0;
 $29 = (($28) + 4|0);
 $30 = HEAP32[$29>>2]|0;
 HEAP32[$nOBSat>>2] = $30;
 $31 = HEAP32[$5>>2]|0;
 $32 = ($31|0)!=(0);
 if ($32) {
  $33 = HEAP32[$3>>2]|0;
  $34 = HEAP32[$nExpr>>2]|0;
  $35 = (($33) - ($34))|0;
  $36 = HEAP32[$bSeq>>2]|0;
  $37 = (($35) - ($36))|0;
  HEAP32[$regBase>>2] = $37;
 } else {
  $38 = HEAP32[$0>>2]|0;
  $39 = (($38) + 72|0);
  $40 = HEAP32[$39>>2]|0;
  $41 = (($40) + 1)|0;
  HEAP32[$regBase>>2] = $41;
  $42 = HEAP32[$nBase>>2]|0;
  $43 = HEAP32[$0>>2]|0;
  $44 = (($43) + 72|0);
  $45 = HEAP32[$44>>2]|0;
  $46 = (($45) + ($42))|0;
  HEAP32[$44>>2] = $46;
 }
 $47 = HEAP32[$0>>2]|0;
 $48 = HEAP32[$1>>2]|0;
 $49 = HEAP32[$48>>2]|0;
 $50 = HEAP32[$regBase>>2]|0;
 (_sqlite3ExprCodeExprList($47,$49,$50,1)|0);
 $51 = HEAP32[$bSeq>>2]|0;
 $52 = ($51|0)!=(0);
 if ($52) {
  $53 = HEAP32[$v>>2]|0;
  $54 = HEAP32[$1>>2]|0;
  $55 = (($54) + 8|0);
  $56 = HEAP32[$55>>2]|0;
  $57 = HEAP32[$regBase>>2]|0;
  $58 = HEAP32[$nExpr>>2]|0;
  $59 = (($57) + ($58))|0;
  (_sqlite3VdbeAddOp2($53,73,$56,$59)|0);
 }
 $60 = HEAP32[$5>>2]|0;
 $61 = ($60|0)==(0);
 if ($61) {
  $62 = HEAP32[$0>>2]|0;
  $63 = HEAP32[$3>>2]|0;
  $64 = HEAP32[$regBase>>2]|0;
  $65 = HEAP32[$nExpr>>2]|0;
  $66 = (($64) + ($65))|0;
  $67 = HEAP32[$bSeq>>2]|0;
  $68 = (($66) + ($67))|0;
  $69 = HEAP32[$4>>2]|0;
  _sqlite3ExprCodeMove($62,$63,$68,$69);
 }
 $70 = HEAP32[$v>>2]|0;
 $71 = HEAP32[$regBase>>2]|0;
 $72 = HEAP32[$nOBSat>>2]|0;
 $73 = (($71) + ($72))|0;
 $74 = HEAP32[$nBase>>2]|0;
 $75 = HEAP32[$nOBSat>>2]|0;
 $76 = (($74) - ($75))|0;
 $77 = HEAP32[$regRecord>>2]|0;
 (_sqlite3VdbeAddOp3($70,49,$73,$76,$77)|0);
 $78 = HEAP32[$nOBSat>>2]|0;
 $79 = ($78|0)>(0);
 do {
  if ($79) {
   $80 = HEAP32[$0>>2]|0;
   $81 = (($80) + 72|0);
   $82 = HEAP32[$81>>2]|0;
   $83 = (($82) + 1)|0;
   HEAP32[$regPrevKey>>2] = $83;
   $84 = HEAP32[$1>>2]|0;
   $85 = (($84) + 4|0);
   $86 = HEAP32[$85>>2]|0;
   $87 = HEAP32[$0>>2]|0;
   $88 = (($87) + 72|0);
   $89 = HEAP32[$88>>2]|0;
   $90 = (($89) + ($86))|0;
   HEAP32[$88>>2] = $90;
   $91 = HEAP32[$nExpr>>2]|0;
   $92 = HEAP32[$1>>2]|0;
   $93 = (($92) + 4|0);
   $94 = HEAP32[$93>>2]|0;
   $95 = (($91) - ($94))|0;
   $96 = HEAP32[$bSeq>>2]|0;
   $97 = (($95) + ($96))|0;
   HEAP32[$nKey>>2] = $97;
   $98 = HEAP32[$bSeq>>2]|0;
   $99 = ($98|0)!=(0);
   $100 = HEAP32[$v>>2]|0;
   if ($99) {
    $101 = HEAP32[$regBase>>2]|0;
    $102 = HEAP32[$nExpr>>2]|0;
    $103 = (($101) + ($102))|0;
    $104 = (_sqlite3VdbeAddOp1($100,46,$103)|0);
    HEAP32[$addrFirst>>2] = $104;
   } else {
    $105 = HEAP32[$1>>2]|0;
    $106 = (($105) + 8|0);
    $107 = HEAP32[$106>>2]|0;
    $108 = (_sqlite3VdbeAddOp1($100,59,$107)|0);
    HEAP32[$addrFirst>>2] = $108;
   }
   $109 = HEAP32[$v>>2]|0;
   $110 = HEAP32[$regPrevKey>>2]|0;
   $111 = HEAP32[$regBase>>2]|0;
   $112 = HEAP32[$1>>2]|0;
   $113 = (($112) + 4|0);
   $114 = HEAP32[$113>>2]|0;
   (_sqlite3VdbeAddOp3($109,42,$110,$111,$114)|0);
   $115 = HEAP32[$v>>2]|0;
   $116 = HEAP32[$1>>2]|0;
   $117 = (($116) + 20|0);
   $118 = HEAP32[$117>>2]|0;
   $119 = (_sqlite3VdbeGetOp($115,$118)|0);
   HEAP32[$pOp>>2] = $119;
   $120 = HEAP32[$0>>2]|0;
   $121 = HEAP32[$120>>2]|0;
   $122 = (($121) + 64|0);
   $123 = HEAP8[$122>>0]|0;
   $124 = ($123<<24>>24)!=(0);
   if ($124) {
    STACKTOP = sp;return;
   } else {
    $125 = HEAP32[$nKey>>2]|0;
    $126 = HEAP32[$4>>2]|0;
    $127 = (($125) + ($126))|0;
    $128 = HEAP32[$pOp>>2]|0;
    $129 = (($128) + 8|0);
    HEAP32[$129>>2] = $127;
    $130 = HEAP32[$pOp>>2]|0;
    $131 = (($130) + 16|0);
    $132 = HEAP32[$131>>2]|0;
    HEAP32[$pKI>>2] = $132;
    $133 = HEAP32[$pKI>>2]|0;
    $134 = (($133) + 16|0);
    $135 = HEAP32[$134>>2]|0;
    $136 = HEAP32[$pKI>>2]|0;
    $137 = (($136) + 6|0);
    $138 = HEAP16[$137>>1]|0;
    $139 = $138&65535;
    _memset(($135|0),0,($139|0))|0;
    $140 = HEAP32[$v>>2]|0;
    $141 = HEAP32[$pKI>>2]|0;
    _sqlite3VdbeChangeP4($140,-1,$141,-6);
    $142 = HEAP32[$0>>2]|0;
    $143 = HEAP32[$1>>2]|0;
    $144 = HEAP32[$143>>2]|0;
    $145 = HEAP32[$nOBSat>>2]|0;
    $146 = (_keyInfoFromExprList($142,$144,$145,1)|0);
    $147 = HEAP32[$pOp>>2]|0;
    $148 = (($147) + 16|0);
    HEAP32[$148>>2] = $146;
    $149 = HEAP32[$v>>2]|0;
    $150 = (_sqlite3VdbeCurrentAddr($149)|0);
    HEAP32[$addrJmp>>2] = $150;
    $151 = HEAP32[$v>>2]|0;
    $152 = HEAP32[$addrJmp>>2]|0;
    $153 = (($152) + 1)|0;
    $154 = HEAP32[$addrJmp>>2]|0;
    $155 = (($154) + 1)|0;
    (_sqlite3VdbeAddOp3($151,43,$153,0,$155)|0);
    $156 = HEAP32[$v>>2]|0;
    $157 = (_sqlite3VdbeMakeLabel($156)|0);
    $158 = HEAP32[$1>>2]|0;
    $159 = (($158) + 16|0);
    HEAP32[$159>>2] = $157;
    $160 = HEAP32[$0>>2]|0;
    $161 = (($160) + 72|0);
    $162 = HEAP32[$161>>2]|0;
    $163 = (($162) + 1)|0;
    HEAP32[$161>>2] = $163;
    $164 = HEAP32[$1>>2]|0;
    $165 = (($164) + 12|0);
    HEAP32[$165>>2] = $163;
    $166 = HEAP32[$v>>2]|0;
    $167 = HEAP32[$1>>2]|0;
    $168 = (($167) + 12|0);
    $169 = HEAP32[$168>>2]|0;
    $170 = HEAP32[$1>>2]|0;
    $171 = (($170) + 16|0);
    $172 = HEAP32[$171>>2]|0;
    (_sqlite3VdbeAddOp2($166,17,$169,$172)|0);
    $173 = HEAP32[$v>>2]|0;
    $174 = HEAP32[$1>>2]|0;
    $175 = (($174) + 8|0);
    $176 = HEAP32[$175>>2]|0;
    (_sqlite3VdbeAddOp1($173,119,$176)|0);
    $177 = HEAP32[$v>>2]|0;
    $178 = HEAP32[$addrFirst>>2]|0;
    _sqlite3VdbeJumpHere($177,$178);
    $179 = HEAP32[$0>>2]|0;
    $180 = HEAP32[$regBase>>2]|0;
    $181 = HEAP32[$regPrevKey>>2]|0;
    $182 = HEAP32[$1>>2]|0;
    $183 = (($182) + 4|0);
    $184 = HEAP32[$183>>2]|0;
    _sqlite3ExprCodeMove($179,$180,$181,$184);
    $185 = HEAP32[$v>>2]|0;
    $186 = HEAP32[$addrJmp>>2]|0;
    _sqlite3VdbeJumpHere($185,$186);
    break;
   }
  }
 } while(0);
 $187 = HEAP32[$1>>2]|0;
 $188 = (($187) + 24|0);
 $189 = HEAP8[$188>>0]|0;
 $190 = $189&255;
 $191 = $190 & 1;
 $192 = ($191|0)!=(0);
 if ($192) {
  HEAP32[$op>>2] = 109;
 } else {
  HEAP32[$op>>2] = 110;
 }
 $193 = HEAP32[$v>>2]|0;
 $194 = HEAP32[$op>>2]|0;
 $195 = HEAP32[$1>>2]|0;
 $196 = (($195) + 8|0);
 $197 = HEAP32[$196>>2]|0;
 $198 = HEAP32[$regRecord>>2]|0;
 (_sqlite3VdbeAddOp2($193,$194,$197,$198)|0);
 $199 = HEAP32[$2>>2]|0;
 $200 = (($199) + 8|0);
 $201 = HEAP32[$200>>2]|0;
 $202 = ($201|0)!=(0);
 if (!($202)) {
  STACKTOP = sp;return;
 }
 $203 = HEAP32[$2>>2]|0;
 $204 = (($203) + 12|0);
 $205 = HEAP32[$204>>2]|0;
 $206 = ($205|0)!=(0);
 $207 = HEAP32[$2>>2]|0;
 if ($206) {
  $208 = (($207) + 12|0);
  $209 = HEAP32[$208>>2]|0;
  $210 = (($209) + 1)|0;
  HEAP32[$iLimit>>2] = $210;
 } else {
  $211 = (($207) + 8|0);
  $212 = HEAP32[$211>>2]|0;
  HEAP32[$iLimit>>2] = $212;
 }
 $213 = HEAP32[$v>>2]|0;
 $214 = HEAP32[$iLimit>>2]|0;
 $215 = (_sqlite3VdbeAddOp1($213,139,$214)|0);
 HEAP32[$addr1>>2] = $215;
 $216 = HEAP32[$v>>2]|0;
 $217 = HEAP32[$iLimit>>2]|0;
 (_sqlite3VdbeAddOp2($216,37,$217,-1)|0);
 $218 = HEAP32[$v>>2]|0;
 $219 = (_sqlite3VdbeAddOp0($218,16)|0);
 HEAP32[$addr2>>2] = $219;
 $220 = HEAP32[$v>>2]|0;
 $221 = HEAP32[$addr1>>2]|0;
 _sqlite3VdbeJumpHere($220,$221);
 $222 = HEAP32[$v>>2]|0;
 $223 = HEAP32[$1>>2]|0;
 $224 = (($223) + 8|0);
 $225 = HEAP32[$224>>2]|0;
 (_sqlite3VdbeAddOp1($222,105,$225)|0);
 $226 = HEAP32[$v>>2]|0;
 $227 = HEAP32[$1>>2]|0;
 $228 = (($227) + 8|0);
 $229 = HEAP32[$228>>2]|0;
 (_sqlite3VdbeAddOp1($226,95,$229)|0);
 $230 = HEAP32[$v>>2]|0;
 $231 = HEAP32[$addr2>>2]|0;
 _sqlite3VdbeJumpHere($230,$231);
 STACKTOP = sp;return;
}
function _selectOpName($id) {
 $id = $id|0;
 var $0 = 0, $1 = 0, $2 = 0, $z = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $z = sp;
 HEAP32[$0>>2] = $id;
 $1 = HEAP32[$0>>2]|0;
 if ((($1|0) == 118)) {
  HEAP32[$z>>2] = 25360;
 } else if ((($1|0) == 116)) {
  HEAP32[$z>>2] = 25344;
 } else if ((($1|0) == 117)) {
  HEAP32[$z>>2] = 25376;
 } else {
  HEAP32[$z>>2] = 25384;
 }
 $2 = HEAP32[$z>>2]|0;
 STACKTOP = sp;return ($2|0);
}
function _generateWithRecursiveQuery($pParse,$p,$pDest) {
 $pParse = $pParse|0;
 $p = $p|0;
 $pDest = $pDest|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0;
 var $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0, $188 = 0;
 var $189 = 0, $19 = 0, $190 = 0, $191 = 0, $192 = 0, $193 = 0, $194 = 0, $195 = 0, $196 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0;
 var $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0;
 var $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0;
 var $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0;
 var $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $addrBreak = 0, $addrCont = 0, $addrTop = 0;
 var $destQueue = 0, $eDest = 0, $i = 0, $iCurrent = 0, $iDistinct = 0, $iQueue = 0, $nCol = 0, $pKeyInfo = 0, $pLimit = 0, $pOffset = 0, $pOrderBy = 0, $pSetup = 0, $pSrc = 0, $rc = 0, $regCurrent = 0, $regLimit = 0, $regOffset = 0, $v = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 112|0;
 $0 = sp + 108|0;
 $1 = sp + 104|0;
 $2 = sp + 100|0;
 $pSrc = sp + 96|0;
 $nCol = sp + 92|0;
 $v = sp + 88|0;
 $pSetup = sp + 84|0;
 $addrTop = sp + 80|0;
 $addrCont = sp + 76|0;
 $addrBreak = sp + 72|0;
 $iCurrent = sp + 68|0;
 $regCurrent = sp + 64|0;
 $iQueue = sp + 60|0;
 $iDistinct = sp + 56|0;
 $eDest = sp + 52|0;
 $destQueue = sp + 32|0;
 $i = sp + 28|0;
 $rc = sp + 24|0;
 $pOrderBy = sp + 20|0;
 $pLimit = sp + 16|0;
 $pOffset = sp + 12|0;
 $regLimit = sp + 8|0;
 $regOffset = sp + 4|0;
 $pKeyInfo = sp;
 HEAP32[$0>>2] = $pParse;
 HEAP32[$1>>2] = $p;
 HEAP32[$2>>2] = $pDest;
 $3 = HEAP32[$1>>2]|0;
 $4 = (($3) + 32|0);
 $5 = HEAP32[$4>>2]|0;
 HEAP32[$pSrc>>2] = $5;
 $6 = HEAP32[$1>>2]|0;
 $7 = HEAP32[$6>>2]|0;
 $8 = HEAP32[$7>>2]|0;
 HEAP32[$nCol>>2] = $8;
 $9 = HEAP32[$0>>2]|0;
 $10 = (($9) + 8|0);
 $11 = HEAP32[$10>>2]|0;
 HEAP32[$v>>2] = $11;
 $12 = HEAP32[$1>>2]|0;
 $13 = (($12) + 52|0);
 $14 = HEAP32[$13>>2]|0;
 HEAP32[$pSetup>>2] = $14;
 HEAP32[$iCurrent>>2] = 0;
 HEAP32[$iDistinct>>2] = 0;
 HEAP32[$eDest>>2] = 5;
 $15 = HEAP32[$0>>2]|0;
 $16 = (_sqlite3AuthCheck($15,33,0,0,0)|0);
 $17 = ($16|0)!=(0);
 if ($17) {
  STACKTOP = sp;return;
 }
 $18 = HEAP32[$v>>2]|0;
 $19 = (_sqlite3VdbeMakeLabel($18)|0);
 HEAP32[$addrBreak>>2] = $19;
 $20 = HEAP32[$0>>2]|0;
 $21 = HEAP32[$1>>2]|0;
 $22 = HEAP32[$addrBreak>>2]|0;
 _computeLimitRegisters($20,$21,$22);
 $23 = HEAP32[$1>>2]|0;
 $24 = (($23) + 60|0);
 $25 = HEAP32[$24>>2]|0;
 HEAP32[$pLimit>>2] = $25;
 $26 = HEAP32[$1>>2]|0;
 $27 = (($26) + 64|0);
 $28 = HEAP32[$27>>2]|0;
 HEAP32[$pOffset>>2] = $28;
 $29 = HEAP32[$1>>2]|0;
 $30 = (($29) + 8|0);
 $31 = HEAP32[$30>>2]|0;
 HEAP32[$regLimit>>2] = $31;
 $32 = HEAP32[$1>>2]|0;
 $33 = (($32) + 12|0);
 $34 = HEAP32[$33>>2]|0;
 HEAP32[$regOffset>>2] = $34;
 $35 = HEAP32[$1>>2]|0;
 $36 = (($35) + 64|0);
 HEAP32[$36>>2] = 0;
 $37 = HEAP32[$1>>2]|0;
 $38 = (($37) + 60|0);
 HEAP32[$38>>2] = 0;
 $39 = HEAP32[$1>>2]|0;
 $40 = (($39) + 12|0);
 HEAP32[$40>>2] = 0;
 $41 = HEAP32[$1>>2]|0;
 $42 = (($41) + 8|0);
 HEAP32[$42>>2] = 0;
 $43 = HEAP32[$1>>2]|0;
 $44 = (($43) + 48|0);
 $45 = HEAP32[$44>>2]|0;
 HEAP32[$pOrderBy>>2] = $45;
 HEAP32[$i>>2] = 0;
 while(1) {
  $46 = HEAP32[$i>>2]|0;
  $47 = HEAP32[$pSrc>>2]|0;
  $48 = HEAP32[$47>>2]|0;
  $49 = ($46|0)<($48|0);
  if (!($49)) {
   break;
  }
  $50 = HEAP32[$i>>2]|0;
  $51 = HEAP32[$pSrc>>2]|0;
  $52 = (($51) + 8|0);
  $53 = (($52) + (($50*72)|0)|0);
  $54 = (($53) + 37|0);
  $55 = HEAP8[$54>>0]|0;
  $56 = ($55&255) >>> 3;
  $57 = $56 & 1;
  $58 = $57&255;
  $59 = ($58|0)!=(0);
  $60 = HEAP32[$i>>2]|0;
  if ($59) {
   label = 5;
   break;
  }
  $66 = (($60) + 1)|0;
  HEAP32[$i>>2] = $66;
 }
 if ((label|0) == 5) {
  $61 = HEAP32[$pSrc>>2]|0;
  $62 = (($61) + 8|0);
  $63 = (($62) + (($60*72)|0)|0);
  $64 = (($63) + 40|0);
  $65 = HEAP32[$64>>2]|0;
  HEAP32[$iCurrent>>2] = $65;
 }
 $67 = HEAP32[$0>>2]|0;
 $68 = (($67) + 68|0);
 $69 = HEAP32[$68>>2]|0;
 $70 = (($69) + 1)|0;
 HEAP32[$68>>2] = $70;
 HEAP32[$iQueue>>2] = $69;
 $71 = HEAP32[$1>>2]|0;
 $72 = (($71) + 4|0);
 $73 = HEAP8[$72>>0]|0;
 $74 = $73&255;
 $75 = ($74|0)==(115);
 $76 = HEAP32[$pOrderBy>>2]|0;
 $77 = ($76|0)!=(0|0);
 if ($75) {
  $78 = $77 ? 8 : 6;
  HEAP32[$eDest>>2] = $78;
  $79 = HEAP32[$0>>2]|0;
  $80 = (($79) + 68|0);
  $81 = HEAP32[$80>>2]|0;
  $82 = (($81) + 1)|0;
  HEAP32[$80>>2] = $82;
  HEAP32[$iDistinct>>2] = $81;
 } else {
  $83 = $77 ? 7 : 5;
  HEAP32[$eDest>>2] = $83;
 }
 $84 = HEAP32[$eDest>>2]|0;
 $85 = HEAP32[$iQueue>>2]|0;
 _sqlite3SelectDestInit($destQueue,$84,$85);
 $86 = HEAP32[$0>>2]|0;
 $87 = (($86) + 72|0);
 $88 = HEAP32[$87>>2]|0;
 $89 = (($88) + 1)|0;
 HEAP32[$87>>2] = $89;
 HEAP32[$regCurrent>>2] = $89;
 $90 = HEAP32[$v>>2]|0;
 $91 = HEAP32[$iCurrent>>2]|0;
 $92 = HEAP32[$regCurrent>>2]|0;
 $93 = HEAP32[$nCol>>2]|0;
 (_sqlite3VdbeAddOp3($90,60,$91,$92,$93)|0);
 $94 = HEAP32[$pOrderBy>>2]|0;
 $95 = ($94|0)!=(0|0);
 if ($95) {
  $96 = HEAP32[$0>>2]|0;
  $97 = HEAP32[$1>>2]|0;
  $98 = (_multiSelectOrderByKeyInfo($96,$97,1)|0);
  HEAP32[$pKeyInfo>>2] = $98;
  $99 = HEAP32[$v>>2]|0;
  $100 = HEAP32[$iQueue>>2]|0;
  $101 = HEAP32[$pOrderBy>>2]|0;
  $102 = HEAP32[$101>>2]|0;
  $103 = (($102) + 2)|0;
  $104 = HEAP32[$pKeyInfo>>2]|0;
  (_sqlite3VdbeAddOp4($99,57,$100,$103,0,$104,-6)|0);
  $105 = HEAP32[$pOrderBy>>2]|0;
  $106 = (($destQueue) + 16|0);
  HEAP32[$106>>2] = $105;
 } else {
  $107 = HEAP32[$v>>2]|0;
  $108 = HEAP32[$iQueue>>2]|0;
  $109 = HEAP32[$nCol>>2]|0;
  (_sqlite3VdbeAddOp2($107,57,$108,$109)|0);
 }
 $110 = HEAP32[$iDistinct>>2]|0;
 $111 = ($110|0)!=(0);
 if ($111) {
  $112 = HEAP32[$v>>2]|0;
  $113 = HEAP32[$iDistinct>>2]|0;
  $114 = (_sqlite3VdbeAddOp2($112,57,$113,0)|0);
  $115 = HEAP32[$1>>2]|0;
  $116 = (($115) + 16|0);
  HEAP32[$116>>2] = $114;
  $117 = HEAP32[$1>>2]|0;
  $118 = (($117) + 6|0);
  $119 = HEAP16[$118>>1]|0;
  $120 = $119&65535;
  $121 = $120 | 8;
  $122 = $121&65535;
  HEAP16[$118>>1] = $122;
 }
 $123 = HEAP32[$1>>2]|0;
 $124 = (($123) + 48|0);
 HEAP32[$124>>2] = 0;
 $125 = HEAP32[$pSetup>>2]|0;
 $126 = (($125) + 56|0);
 HEAP32[$126>>2] = 0;
 $127 = HEAP32[$0>>2]|0;
 $128 = HEAP32[$pSetup>>2]|0;
 $129 = (_sqlite3Select($127,$128,$destQueue)|0);
 HEAP32[$rc>>2] = $129;
 $130 = HEAP32[$1>>2]|0;
 $131 = HEAP32[$pSetup>>2]|0;
 $132 = (($131) + 56|0);
 HEAP32[$132>>2] = $130;
 $133 = HEAP32[$rc>>2]|0;
 $134 = ($133|0)!=(0);
 if (!($134)) {
  $135 = HEAP32[$v>>2]|0;
  $136 = HEAP32[$iQueue>>2]|0;
  $137 = HEAP32[$addrBreak>>2]|0;
  $138 = (_sqlite3VdbeAddOp2($135,108,$136,$137)|0);
  HEAP32[$addrTop>>2] = $138;
  $139 = HEAP32[$v>>2]|0;
  $140 = HEAP32[$iCurrent>>2]|0;
  (_sqlite3VdbeAddOp1($139,104,$140)|0);
  $141 = HEAP32[$pOrderBy>>2]|0;
  $142 = ($141|0)!=(0|0);
  $143 = HEAP32[$v>>2]|0;
  $144 = HEAP32[$iQueue>>2]|0;
  if ($142) {
   $145 = HEAP32[$pOrderBy>>2]|0;
   $146 = HEAP32[$145>>2]|0;
   $147 = (($146) + 1)|0;
   $148 = HEAP32[$regCurrent>>2]|0;
   (_sqlite3VdbeAddOp3($143,47,$144,$147,$148)|0);
  } else {
   $149 = HEAP32[$regCurrent>>2]|0;
   (_sqlite3VdbeAddOp2($143,102,$144,$149)|0);
  }
  $150 = HEAP32[$v>>2]|0;
  $151 = HEAP32[$iQueue>>2]|0;
  (_sqlite3VdbeAddOp1($150,95,$151)|0);
  $152 = HEAP32[$v>>2]|0;
  $153 = (_sqlite3VdbeMakeLabel($152)|0);
  HEAP32[$addrCont>>2] = $153;
  $154 = HEAP32[$v>>2]|0;
  $155 = HEAP32[$regOffset>>2]|0;
  $156 = HEAP32[$addrCont>>2]|0;
  _codeOffset($154,$155,$156);
  $157 = HEAP32[$0>>2]|0;
  $158 = HEAP32[$1>>2]|0;
  $159 = HEAP32[$1>>2]|0;
  $160 = HEAP32[$159>>2]|0;
  $161 = HEAP32[$iCurrent>>2]|0;
  $162 = HEAP32[$2>>2]|0;
  $163 = HEAP32[$addrCont>>2]|0;
  $164 = HEAP32[$addrBreak>>2]|0;
  _selectInnerLoop($157,$158,$160,$161,0,0,$162,$163,$164);
  $165 = HEAP32[$regLimit>>2]|0;
  $166 = ($165|0)!=(0);
  if ($166) {
   $167 = HEAP32[$v>>2]|0;
   $168 = HEAP32[$regLimit>>2]|0;
   $169 = HEAP32[$addrBreak>>2]|0;
   (_sqlite3VdbeAddOp3($167,139,$168,$169,-1)|0);
  }
  $170 = HEAP32[$v>>2]|0;
  $171 = HEAP32[$addrCont>>2]|0;
  _sqlite3VdbeResolveLabel($170,$171);
  $172 = HEAP32[$1>>2]|0;
  $173 = (($172) + 52|0);
  HEAP32[$173>>2] = 0;
  $174 = HEAP32[$0>>2]|0;
  $175 = HEAP32[$1>>2]|0;
  (_sqlite3Select($174,$175,$destQueue)|0);
  $176 = HEAP32[$pSetup>>2]|0;
  $177 = HEAP32[$1>>2]|0;
  $178 = (($177) + 52|0);
  HEAP32[$178>>2] = $176;
  $179 = HEAP32[$v>>2]|0;
  $180 = HEAP32[$addrTop>>2]|0;
  (_sqlite3VdbeAddOp2($179,16,0,$180)|0);
  $181 = HEAP32[$v>>2]|0;
  $182 = HEAP32[$addrBreak>>2]|0;
  _sqlite3VdbeResolveLabel($181,$182);
 }
 $183 = HEAP32[$0>>2]|0;
 $184 = HEAP32[$183>>2]|0;
 $185 = HEAP32[$1>>2]|0;
 $186 = (($185) + 48|0);
 $187 = HEAP32[$186>>2]|0;
 _sqlite3ExprListDelete($184,$187);
 $188 = HEAP32[$pOrderBy>>2]|0;
 $189 = HEAP32[$1>>2]|0;
 $190 = (($189) + 48|0);
 HEAP32[$190>>2] = $188;
 $191 = HEAP32[$pLimit>>2]|0;
 $192 = HEAP32[$1>>2]|0;
 $193 = (($192) + 60|0);
 HEAP32[$193>>2] = $191;
 $194 = HEAP32[$pOffset>>2]|0;
 $195 = HEAP32[$1>>2]|0;
 $196 = (($195) + 64|0);
 HEAP32[$196>>2] = $194;
 STACKTOP = sp;return;
}
function _multiSelectOrderBy($pParse,$p,$pDest) {
 $pParse = $pParse|0;
 $p = $p|0;
 $pDest = $pDest|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0;
 var $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0, $188 = 0;
 var $189 = 0, $19 = 0, $190 = 0, $191 = 0, $192 = 0, $193 = 0, $194 = 0, $195 = 0, $196 = 0, $197 = 0, $198 = 0, $199 = 0, $2 = 0, $20 = 0, $200 = 0, $201 = 0, $202 = 0, $203 = 0, $204 = 0, $205 = 0;
 var $206 = 0, $207 = 0, $208 = 0, $209 = 0, $21 = 0, $210 = 0, $211 = 0, $212 = 0, $213 = 0, $214 = 0, $215 = 0, $216 = 0, $217 = 0, $218 = 0, $219 = 0, $22 = 0, $220 = 0, $221 = 0, $222 = 0, $223 = 0;
 var $224 = 0, $225 = 0, $226 = 0, $227 = 0, $228 = 0, $229 = 0, $23 = 0, $230 = 0, $231 = 0, $232 = 0, $233 = 0, $234 = 0, $235 = 0, $236 = 0, $237 = 0, $238 = 0, $239 = 0, $24 = 0, $240 = 0, $241 = 0;
 var $242 = 0, $243 = 0, $244 = 0, $245 = 0, $246 = 0, $247 = 0, $248 = 0, $249 = 0, $25 = 0, $250 = 0, $251 = 0, $252 = 0, $253 = 0, $254 = 0, $255 = 0, $256 = 0, $257 = 0, $258 = 0, $259 = 0, $26 = 0;
 var $260 = 0, $261 = 0, $262 = 0, $263 = 0, $264 = 0, $265 = 0, $266 = 0, $267 = 0, $268 = 0, $269 = 0, $27 = 0, $270 = 0, $271 = 0, $272 = 0, $273 = 0, $274 = 0, $275 = 0, $276 = 0, $277 = 0, $278 = 0;
 var $279 = 0, $28 = 0, $280 = 0, $281 = 0, $282 = 0, $283 = 0, $284 = 0, $285 = 0, $286 = 0, $287 = 0, $288 = 0, $289 = 0, $29 = 0, $290 = 0, $291 = 0, $292 = 0, $293 = 0, $294 = 0, $295 = 0, $296 = 0;
 var $297 = 0, $298 = 0, $299 = 0, $3 = 0, $30 = 0, $300 = 0, $301 = 0, $302 = 0, $303 = 0, $304 = 0, $305 = 0, $306 = 0, $307 = 0, $308 = 0, $309 = 0, $31 = 0, $310 = 0, $311 = 0, $312 = 0, $313 = 0;
 var $314 = 0, $315 = 0, $316 = 0, $317 = 0, $318 = 0, $319 = 0, $32 = 0, $320 = 0, $321 = 0, $322 = 0, $323 = 0, $324 = 0, $325 = 0, $326 = 0, $327 = 0, $328 = 0, $329 = 0, $33 = 0, $330 = 0, $331 = 0;
 var $332 = 0, $333 = 0, $334 = 0, $335 = 0, $336 = 0, $337 = 0, $338 = 0, $339 = 0, $34 = 0, $340 = 0, $341 = 0, $342 = 0, $343 = 0, $344 = 0, $345 = 0, $346 = 0, $347 = 0, $348 = 0, $349 = 0, $35 = 0;
 var $350 = 0, $351 = 0, $352 = 0, $353 = 0, $354 = 0, $355 = 0, $356 = 0, $357 = 0, $358 = 0, $359 = 0, $36 = 0, $360 = 0, $361 = 0, $362 = 0, $363 = 0, $364 = 0, $365 = 0, $366 = 0, $367 = 0, $368 = 0;
 var $369 = 0, $37 = 0, $370 = 0, $371 = 0, $372 = 0, $373 = 0, $374 = 0, $375 = 0, $376 = 0, $377 = 0, $378 = 0, $379 = 0, $38 = 0, $380 = 0, $381 = 0, $382 = 0, $383 = 0, $384 = 0, $385 = 0, $386 = 0;
 var $387 = 0, $388 = 0, $389 = 0, $39 = 0, $390 = 0, $391 = 0, $392 = 0, $393 = 0, $394 = 0, $395 = 0, $396 = 0, $397 = 0, $398 = 0, $399 = 0, $4 = 0, $40 = 0, $400 = 0, $401 = 0, $402 = 0, $403 = 0;
 var $404 = 0, $405 = 0, $406 = 0, $407 = 0, $408 = 0, $409 = 0, $41 = 0, $410 = 0, $411 = 0, $412 = 0, $413 = 0, $414 = 0, $415 = 0, $416 = 0, $417 = 0, $418 = 0, $419 = 0, $42 = 0, $420 = 0, $421 = 0;
 var $422 = 0, $423 = 0, $424 = 0, $425 = 0, $426 = 0, $427 = 0, $428 = 0, $429 = 0, $43 = 0, $430 = 0, $431 = 0, $432 = 0, $433 = 0, $434 = 0, $435 = 0, $436 = 0, $437 = 0, $438 = 0, $439 = 0, $44 = 0;
 var $440 = 0, $441 = 0, $442 = 0, $443 = 0, $444 = 0, $445 = 0, $446 = 0, $447 = 0, $448 = 0, $449 = 0, $45 = 0, $450 = 0, $451 = 0, $452 = 0, $453 = 0, $454 = 0, $455 = 0, $456 = 0, $457 = 0, $458 = 0;
 var $459 = 0, $46 = 0, $460 = 0, $461 = 0, $462 = 0, $463 = 0, $464 = 0, $465 = 0, $466 = 0, $467 = 0, $468 = 0, $469 = 0, $47 = 0, $470 = 0, $471 = 0, $472 = 0, $473 = 0, $474 = 0, $475 = 0, $476 = 0;
 var $477 = 0, $478 = 0, $479 = 0, $48 = 0, $480 = 0, $481 = 0, $482 = 0, $483 = 0, $484 = 0, $485 = 0, $486 = 0, $487 = 0, $488 = 0, $489 = 0, $49 = 0, $490 = 0, $491 = 0, $492 = 0, $493 = 0, $494 = 0;
 var $495 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0;
 var $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0;
 var $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $aPermute = 0, $addrAeqB = 0, $addrAgtB = 0, $addrAltB = 0;
 var $addrEofA = 0, $addrEofA_noB = 0, $addrEofB = 0, $addrOutA = 0, $addrOutB = 0, $addrSelectA = 0, $addrSelectB = 0, $db = 0, $destA = 0, $destB = 0, $i = 0, $iSub1 = 0, $iSub2 = 0, $j = 0, $j1 = 0, $labelCmpr = 0, $labelEnd = 0, $nExpr = 0, $nOrderBy = 0, $op = 0;
 var $or$cond = 0, $or$cond3 = 0, $or$cond5 = 0, $or$cond7 = 0, $pFirst = 0, $pItem = 0, $pItem1 = 0, $pKeyDup = 0, $pKeyMerge = 0, $pNew = 0, $pOrderBy = 0, $pPrior = 0, $regAddrA = 0, $regAddrB = 0, $regLimitA = 0, $regLimitB = 0, $regOutA = 0, $regOutB = 0, $regPrev = 0, $savedLimit = 0;
 var $savedOffset = 0, $v = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 224|0;
 $0 = sp + 212|0;
 $1 = sp + 208|0;
 $2 = sp + 204|0;
 $3 = sp + 200|0;
 $i = sp + 196|0;
 $j = sp + 192|0;
 $pPrior = sp + 188|0;
 $v = sp + 184|0;
 $destA = sp + 164|0;
 $destB = sp + 144|0;
 $regAddrA = sp + 140|0;
 $regAddrB = sp + 136|0;
 $addrSelectA = sp + 132|0;
 $addrSelectB = sp + 128|0;
 $regOutA = sp + 124|0;
 $regOutB = sp + 120|0;
 $addrOutA = sp + 116|0;
 $addrOutB = sp + 112|0;
 $addrEofA = sp + 108|0;
 $addrEofA_noB = sp + 104|0;
 $addrEofB = sp + 100|0;
 $addrAltB = sp + 96|0;
 $addrAeqB = sp + 92|0;
 $addrAgtB = sp + 88|0;
 $regLimitA = sp + 84|0;
 $regLimitB = sp + 80|0;
 $regPrev = sp + 76|0;
 $savedLimit = sp + 72|0;
 $savedOffset = sp + 68|0;
 $labelCmpr = sp + 64|0;
 $labelEnd = sp + 60|0;
 $j1 = sp + 56|0;
 $op = sp + 52|0;
 $pKeyDup = sp + 48|0;
 $pKeyMerge = sp + 44|0;
 $db = sp + 40|0;
 $pOrderBy = sp + 36|0;
 $nOrderBy = sp + 32|0;
 $aPermute = sp + 28|0;
 $iSub1 = sp + 24|0;
 $iSub2 = sp + 20|0;
 $pItem = sp + 16|0;
 $pNew = sp + 12|0;
 $pItem1 = sp + 8|0;
 $nExpr = sp + 4|0;
 $pFirst = sp;
 HEAP32[$1>>2] = $pParse;
 HEAP32[$2>>2] = $p;
 HEAP32[$3>>2] = $pDest;
 HEAP32[$addrOutB>>2] = 0;
 HEAP32[$pKeyDup>>2] = 0;
 $4 = HEAP32[$1>>2]|0;
 $5 = HEAP32[$4>>2]|0;
 HEAP32[$db>>2] = $5;
 $6 = HEAP32[$1>>2]|0;
 $7 = (($6) + 8|0);
 $8 = HEAP32[$7>>2]|0;
 HEAP32[$v>>2] = $8;
 $9 = HEAP32[$v>>2]|0;
 $10 = (_sqlite3VdbeMakeLabel($9)|0);
 HEAP32[$labelEnd>>2] = $10;
 $11 = HEAP32[$v>>2]|0;
 $12 = (_sqlite3VdbeMakeLabel($11)|0);
 HEAP32[$labelCmpr>>2] = $12;
 $13 = HEAP32[$2>>2]|0;
 $14 = (($13) + 4|0);
 $15 = HEAP8[$14>>0]|0;
 $16 = $15&255;
 HEAP32[$op>>2] = $16;
 $17 = HEAP32[$2>>2]|0;
 $18 = (($17) + 52|0);
 $19 = HEAP32[$18>>2]|0;
 HEAP32[$pPrior>>2] = $19;
 $20 = HEAP32[$2>>2]|0;
 $21 = (($20) + 48|0);
 $22 = HEAP32[$21>>2]|0;
 HEAP32[$pOrderBy>>2] = $22;
 $23 = HEAP32[$pOrderBy>>2]|0;
 $24 = HEAP32[$23>>2]|0;
 HEAP32[$nOrderBy>>2] = $24;
 $25 = HEAP32[$op>>2]|0;
 $26 = ($25|0)!=(116);
 L1: do {
  if ($26) {
   HEAP32[$i>>2] = 1;
   while(1) {
    $27 = HEAP32[$db>>2]|0;
    $28 = (($27) + 64|0);
    $29 = HEAP8[$28>>0]|0;
    $30 = $29&255;
    $31 = ($30|0)==(0);
    if (!($31)) {
     break L1;
    }
    $32 = HEAP32[$i>>2]|0;
    $33 = HEAP32[$2>>2]|0;
    $34 = HEAP32[$33>>2]|0;
    $35 = HEAP32[$34>>2]|0;
    $36 = ($32|0)<=($35|0);
    if (!($36)) {
     break L1;
    }
    HEAP32[$j>>2] = 0;
    $37 = HEAP32[$pOrderBy>>2]|0;
    $38 = (($37) + 4|0);
    $39 = HEAP32[$38>>2]|0;
    HEAP32[$pItem>>2] = $39;
    while(1) {
     $40 = HEAP32[$j>>2]|0;
     $41 = HEAP32[$nOrderBy>>2]|0;
     $42 = ($40|0)<($41|0);
     if (!($42)) {
      break;
     }
     $43 = HEAP32[$pItem>>2]|0;
     $44 = (($43) + 16|0);
     $45 = HEAP16[$44>>1]|0;
     $46 = $45&65535;
     $47 = HEAP32[$i>>2]|0;
     $48 = ($46|0)==($47|0);
     if ($48) {
      break;
     }
     $49 = HEAP32[$j>>2]|0;
     $50 = (($49) + 1)|0;
     HEAP32[$j>>2] = $50;
     $51 = HEAP32[$pItem>>2]|0;
     $52 = (($51) + 20|0);
     HEAP32[$pItem>>2] = $52;
    }
    $53 = HEAP32[$j>>2]|0;
    $54 = HEAP32[$nOrderBy>>2]|0;
    $55 = ($53|0)==($54|0);
    if ($55) {
     $56 = HEAP32[$db>>2]|0;
     $57 = (_sqlite3Expr($56,132,0)|0);
     HEAP32[$pNew>>2] = $57;
     $58 = HEAP32[$pNew>>2]|0;
     $59 = ($58|0)==(0|0);
     if ($59) {
      break;
     }
     $60 = HEAP32[$pNew>>2]|0;
     $61 = (($60) + 4|0);
     $62 = HEAP32[$61>>2]|0;
     $63 = $62 | 1024;
     HEAP32[$61>>2] = $63;
     $64 = HEAP32[$i>>2]|0;
     $65 = HEAP32[$pNew>>2]|0;
     $66 = (($65) + 8|0);
     HEAP32[$66>>2] = $64;
     $67 = HEAP32[$1>>2]|0;
     $68 = HEAP32[$pOrderBy>>2]|0;
     $69 = HEAP32[$pNew>>2]|0;
     $70 = (_sqlite3ExprListAppend($67,$68,$69)|0);
     HEAP32[$pOrderBy>>2] = $70;
     $71 = HEAP32[$pOrderBy>>2]|0;
     $72 = ($71|0)!=(0|0);
     if ($72) {
      $73 = HEAP32[$i>>2]|0;
      $74 = $73&65535;
      $75 = HEAP32[$nOrderBy>>2]|0;
      $76 = (($75) + 1)|0;
      HEAP32[$nOrderBy>>2] = $76;
      $77 = HEAP32[$pOrderBy>>2]|0;
      $78 = (($77) + 4|0);
      $79 = HEAP32[$78>>2]|0;
      $80 = (($79) + (($75*20)|0)|0);
      $81 = (($80) + 16|0);
      HEAP16[$81>>1] = $74;
     }
    }
    $82 = HEAP32[$i>>2]|0;
    $83 = (($82) + 1)|0;
    HEAP32[$i>>2] = $83;
   }
   HEAP32[$0>>2] = 7;
   $495 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($495|0);
  }
 } while(0);
 $84 = HEAP32[$db>>2]|0;
 $85 = HEAP32[$nOrderBy>>2]|0;
 $86 = $85<<2;
 $87 = (_sqlite3DbMallocRaw($84,$86,0)|0);
 HEAP32[$aPermute>>2] = $87;
 $88 = HEAP32[$aPermute>>2]|0;
 $89 = ($88|0)!=(0|0);
 if ($89) {
  HEAP32[$i>>2] = 0;
  $90 = HEAP32[$pOrderBy>>2]|0;
  $91 = (($90) + 4|0);
  $92 = HEAP32[$91>>2]|0;
  HEAP32[$pItem1>>2] = $92;
  while(1) {
   $93 = HEAP32[$i>>2]|0;
   $94 = HEAP32[$nOrderBy>>2]|0;
   $95 = ($93|0)<($94|0);
   if (!($95)) {
    break;
   }
   $96 = HEAP32[$pItem1>>2]|0;
   $97 = (($96) + 16|0);
   $98 = HEAP16[$97>>1]|0;
   $99 = $98&65535;
   $100 = (($99) - 1)|0;
   $101 = HEAP32[$i>>2]|0;
   $102 = HEAP32[$aPermute>>2]|0;
   $103 = (($102) + ($101<<2)|0);
   HEAP32[$103>>2] = $100;
   $104 = HEAP32[$i>>2]|0;
   $105 = (($104) + 1)|0;
   HEAP32[$i>>2] = $105;
   $106 = HEAP32[$pItem1>>2]|0;
   $107 = (($106) + 20|0);
   HEAP32[$pItem1>>2] = $107;
  }
  $108 = HEAP32[$1>>2]|0;
  $109 = HEAP32[$2>>2]|0;
  $110 = (_multiSelectOrderByKeyInfo($108,$109,1)|0);
  HEAP32[$pKeyMerge>>2] = $110;
 } else {
  HEAP32[$pKeyMerge>>2] = 0;
 }
 $111 = HEAP32[$pOrderBy>>2]|0;
 $112 = HEAP32[$2>>2]|0;
 $113 = (($112) + 48|0);
 HEAP32[$113>>2] = $111;
 $114 = HEAP32[$1>>2]|0;
 $115 = HEAP32[$114>>2]|0;
 $116 = HEAP32[$pOrderBy>>2]|0;
 $117 = (_sqlite3ExprListDup($115,$116,0)|0);
 $118 = HEAP32[$pPrior>>2]|0;
 $119 = (($118) + 48|0);
 HEAP32[$119>>2] = $117;
 $120 = HEAP32[$op>>2]|0;
 $121 = ($120|0)==(116);
 L28: do {
  if ($121) {
   HEAP32[$regPrev>>2] = 0;
  } else {
   $122 = HEAP32[$2>>2]|0;
   $123 = HEAP32[$122>>2]|0;
   $124 = HEAP32[$123>>2]|0;
   HEAP32[$nExpr>>2] = $124;
   $125 = HEAP32[$1>>2]|0;
   $126 = (($125) + 72|0);
   $127 = HEAP32[$126>>2]|0;
   $128 = (($127) + 1)|0;
   HEAP32[$regPrev>>2] = $128;
   $129 = HEAP32[$nExpr>>2]|0;
   $130 = (($129) + 1)|0;
   $131 = HEAP32[$1>>2]|0;
   $132 = (($131) + 72|0);
   $133 = HEAP32[$132>>2]|0;
   $134 = (($133) + ($130))|0;
   HEAP32[$132>>2] = $134;
   $135 = HEAP32[$v>>2]|0;
   $136 = HEAP32[$regPrev>>2]|0;
   (_sqlite3VdbeAddOp2($135,25,0,$136)|0);
   $137 = HEAP32[$db>>2]|0;
   $138 = HEAP32[$nExpr>>2]|0;
   $139 = (_sqlite3KeyInfoAlloc($137,$138,1)|0);
   HEAP32[$pKeyDup>>2] = $139;
   $140 = HEAP32[$pKeyDup>>2]|0;
   $141 = ($140|0)!=(0|0);
   if ($141) {
    HEAP32[$i>>2] = 0;
    while(1) {
     $142 = HEAP32[$i>>2]|0;
     $143 = HEAP32[$nExpr>>2]|0;
     $144 = ($142|0)<($143|0);
     if (!($144)) {
      break L28;
     }
     $145 = HEAP32[$1>>2]|0;
     $146 = HEAP32[$2>>2]|0;
     $147 = HEAP32[$i>>2]|0;
     $148 = (_multiSelectCollSeq($145,$146,$147)|0);
     $149 = HEAP32[$i>>2]|0;
     $150 = HEAP32[$pKeyDup>>2]|0;
     $151 = (($150) + 20|0);
     $152 = (($151) + ($149<<2)|0);
     HEAP32[$152>>2] = $148;
     $153 = HEAP32[$i>>2]|0;
     $154 = HEAP32[$pKeyDup>>2]|0;
     $155 = (($154) + 16|0);
     $156 = HEAP32[$155>>2]|0;
     $157 = (($156) + ($153)|0);
     HEAP8[$157>>0] = 0;
     $158 = HEAP32[$i>>2]|0;
     $159 = (($158) + 1)|0;
     HEAP32[$i>>2] = $159;
    }
   }
  }
 } while(0);
 $160 = HEAP32[$2>>2]|0;
 $161 = (($160) + 52|0);
 HEAP32[$161>>2] = 0;
 $162 = HEAP32[$pPrior>>2]|0;
 $163 = (($162) + 56|0);
 HEAP32[$163>>2] = 0;
 $164 = HEAP32[$1>>2]|0;
 $165 = HEAP32[$2>>2]|0;
 $166 = HEAP32[$2>>2]|0;
 $167 = (($166) + 48|0);
 $168 = HEAP32[$167>>2]|0;
 (_sqlite3ResolveOrderGroupBy($164,$165,$168,16480)|0);
 $169 = HEAP32[$pPrior>>2]|0;
 $170 = (($169) + 52|0);
 $171 = HEAP32[$170>>2]|0;
 $172 = ($171|0)==(0|0);
 if ($172) {
  $173 = HEAP32[$1>>2]|0;
  $174 = HEAP32[$pPrior>>2]|0;
  $175 = HEAP32[$pPrior>>2]|0;
  $176 = (($175) + 48|0);
  $177 = HEAP32[$176>>2]|0;
  (_sqlite3ResolveOrderGroupBy($173,$174,$177,16480)|0);
 }
 $178 = HEAP32[$1>>2]|0;
 $179 = HEAP32[$2>>2]|0;
 $180 = HEAP32[$labelEnd>>2]|0;
 _computeLimitRegisters($178,$179,$180);
 $181 = HEAP32[$2>>2]|0;
 $182 = (($181) + 8|0);
 $183 = HEAP32[$182>>2]|0;
 $184 = ($183|0)!=(0);
 $185 = HEAP32[$op>>2]|0;
 $186 = ($185|0)==(116);
 $or$cond = $184 & $186;
 if ($or$cond) {
  $187 = HEAP32[$1>>2]|0;
  $188 = (($187) + 72|0);
  $189 = HEAP32[$188>>2]|0;
  $190 = (($189) + 1)|0;
  HEAP32[$188>>2] = $190;
  HEAP32[$regLimitA>>2] = $190;
  $191 = HEAP32[$1>>2]|0;
  $192 = (($191) + 72|0);
  $193 = HEAP32[$192>>2]|0;
  $194 = (($193) + 1)|0;
  HEAP32[$192>>2] = $194;
  HEAP32[$regLimitB>>2] = $194;
  $195 = HEAP32[$v>>2]|0;
  $196 = HEAP32[$2>>2]|0;
  $197 = (($196) + 12|0);
  $198 = HEAP32[$197>>2]|0;
  $199 = ($198|0)!=(0);
  $200 = HEAP32[$2>>2]|0;
  if ($199) {
   $201 = (($200) + 12|0);
   $202 = HEAP32[$201>>2]|0;
   $203 = (($202) + 1)|0;
   $207 = $203;
  } else {
   $204 = (($200) + 8|0);
   $205 = HEAP32[$204>>2]|0;
   $207 = $205;
  }
  $206 = HEAP32[$regLimitA>>2]|0;
  (_sqlite3VdbeAddOp2($195,33,$207,$206)|0);
  $208 = HEAP32[$v>>2]|0;
  $209 = HEAP32[$regLimitA>>2]|0;
  $210 = HEAP32[$regLimitB>>2]|0;
  (_sqlite3VdbeAddOp2($208,33,$209,$210)|0);
 } else {
  HEAP32[$regLimitB>>2] = 0;
  HEAP32[$regLimitA>>2] = 0;
 }
 $211 = HEAP32[$db>>2]|0;
 $212 = HEAP32[$2>>2]|0;
 $213 = (($212) + 60|0);
 $214 = HEAP32[$213>>2]|0;
 _sqlite3ExprDelete($211,$214);
 $215 = HEAP32[$2>>2]|0;
 $216 = (($215) + 60|0);
 HEAP32[$216>>2] = 0;
 $217 = HEAP32[$db>>2]|0;
 $218 = HEAP32[$2>>2]|0;
 $219 = (($218) + 64|0);
 $220 = HEAP32[$219>>2]|0;
 _sqlite3ExprDelete($217,$220);
 $221 = HEAP32[$2>>2]|0;
 $222 = (($221) + 64|0);
 HEAP32[$222>>2] = 0;
 $223 = HEAP32[$1>>2]|0;
 $224 = (($223) + 72|0);
 $225 = HEAP32[$224>>2]|0;
 $226 = (($225) + 1)|0;
 HEAP32[$224>>2] = $226;
 HEAP32[$regAddrA>>2] = $226;
 $227 = HEAP32[$1>>2]|0;
 $228 = (($227) + 72|0);
 $229 = HEAP32[$228>>2]|0;
 $230 = (($229) + 1)|0;
 HEAP32[$228>>2] = $230;
 HEAP32[$regAddrB>>2] = $230;
 $231 = HEAP32[$1>>2]|0;
 $232 = (($231) + 72|0);
 $233 = HEAP32[$232>>2]|0;
 $234 = (($233) + 1)|0;
 HEAP32[$232>>2] = $234;
 HEAP32[$regOutA>>2] = $234;
 $235 = HEAP32[$1>>2]|0;
 $236 = (($235) + 72|0);
 $237 = HEAP32[$236>>2]|0;
 $238 = (($237) + 1)|0;
 HEAP32[$236>>2] = $238;
 HEAP32[$regOutB>>2] = $238;
 $239 = HEAP32[$regAddrA>>2]|0;
 _sqlite3SelectDestInit($destA,13,$239);
 $240 = HEAP32[$regAddrB>>2]|0;
 _sqlite3SelectDestInit($destB,13,$240);
 $241 = HEAP32[$v>>2]|0;
 $242 = (_sqlite3VdbeCurrentAddr($241)|0);
 $243 = (($242) + 1)|0;
 HEAP32[$addrSelectA>>2] = $243;
 $244 = HEAP32[$v>>2]|0;
 $245 = HEAP32[$regAddrA>>2]|0;
 $246 = HEAP32[$addrSelectA>>2]|0;
 $247 = (_sqlite3VdbeAddOp3($244,20,$245,0,$246)|0);
 HEAP32[$j1>>2] = $247;
 $248 = HEAP32[$regLimitA>>2]|0;
 $249 = HEAP32[$pPrior>>2]|0;
 $250 = (($249) + 8|0);
 HEAP32[$250>>2] = $248;
 $251 = HEAP32[$1>>2]|0;
 $252 = (($251) + 468|0);
 $253 = HEAP32[$252>>2]|0;
 HEAP32[$iSub1>>2] = $253;
 $254 = HEAP32[$1>>2]|0;
 $255 = HEAP32[$pPrior>>2]|0;
 (_sqlite3Select($254,$255,$destA)|0);
 $256 = HEAP32[$v>>2]|0;
 $257 = HEAP32[$regAddrA>>2]|0;
 (_sqlite3VdbeAddOp1($256,21,$257)|0);
 $258 = HEAP32[$v>>2]|0;
 $259 = HEAP32[$j1>>2]|0;
 _sqlite3VdbeJumpHere($258,$259);
 $260 = HEAP32[$v>>2]|0;
 $261 = (_sqlite3VdbeCurrentAddr($260)|0);
 $262 = (($261) + 1)|0;
 HEAP32[$addrSelectB>>2] = $262;
 $263 = HEAP32[$v>>2]|0;
 $264 = HEAP32[$regAddrB>>2]|0;
 $265 = HEAP32[$addrSelectB>>2]|0;
 $266 = (_sqlite3VdbeAddOp3($263,20,$264,0,$265)|0);
 HEAP32[$j1>>2] = $266;
 $267 = HEAP32[$2>>2]|0;
 $268 = (($267) + 8|0);
 $269 = HEAP32[$268>>2]|0;
 HEAP32[$savedLimit>>2] = $269;
 $270 = HEAP32[$2>>2]|0;
 $271 = (($270) + 12|0);
 $272 = HEAP32[$271>>2]|0;
 HEAP32[$savedOffset>>2] = $272;
 $273 = HEAP32[$regLimitB>>2]|0;
 $274 = HEAP32[$2>>2]|0;
 $275 = (($274) + 8|0);
 HEAP32[$275>>2] = $273;
 $276 = HEAP32[$2>>2]|0;
 $277 = (($276) + 12|0);
 HEAP32[$277>>2] = 0;
 $278 = HEAP32[$1>>2]|0;
 $279 = (($278) + 468|0);
 $280 = HEAP32[$279>>2]|0;
 HEAP32[$iSub2>>2] = $280;
 $281 = HEAP32[$1>>2]|0;
 $282 = HEAP32[$2>>2]|0;
 (_sqlite3Select($281,$282,$destB)|0);
 $283 = HEAP32[$savedLimit>>2]|0;
 $284 = HEAP32[$2>>2]|0;
 $285 = (($284) + 8|0);
 HEAP32[$285>>2] = $283;
 $286 = HEAP32[$savedOffset>>2]|0;
 $287 = HEAP32[$2>>2]|0;
 $288 = (($287) + 12|0);
 HEAP32[$288>>2] = $286;
 $289 = HEAP32[$v>>2]|0;
 $290 = HEAP32[$regAddrB>>2]|0;
 (_sqlite3VdbeAddOp1($289,21,$290)|0);
 $291 = HEAP32[$1>>2]|0;
 $292 = HEAP32[$2>>2]|0;
 $293 = HEAP32[$3>>2]|0;
 $294 = HEAP32[$regOutA>>2]|0;
 $295 = HEAP32[$regPrev>>2]|0;
 $296 = HEAP32[$pKeyDup>>2]|0;
 $297 = HEAP32[$labelEnd>>2]|0;
 $298 = (_generateOutputSubroutine($291,$292,$destA,$293,$294,$295,$296,$297)|0);
 HEAP32[$addrOutA>>2] = $298;
 $299 = HEAP32[$op>>2]|0;
 $300 = ($299|0)==(116);
 $301 = HEAP32[$op>>2]|0;
 $302 = ($301|0)==(115);
 $or$cond3 = $300 | $302;
 if ($or$cond3) {
  $303 = HEAP32[$1>>2]|0;
  $304 = HEAP32[$2>>2]|0;
  $305 = HEAP32[$3>>2]|0;
  $306 = HEAP32[$regOutB>>2]|0;
  $307 = HEAP32[$regPrev>>2]|0;
  $308 = HEAP32[$pKeyDup>>2]|0;
  $309 = HEAP32[$labelEnd>>2]|0;
  $310 = (_generateOutputSubroutine($303,$304,$destB,$305,$306,$307,$308,$309)|0);
  HEAP32[$addrOutB>>2] = $310;
 }
 $311 = HEAP32[$pKeyDup>>2]|0;
 _sqlite3KeyInfoUnref($311);
 $312 = HEAP32[$op>>2]|0;
 $313 = ($312|0)==(117);
 $314 = HEAP32[$op>>2]|0;
 $315 = ($314|0)==(118);
 $or$cond5 = $313 | $315;
 if ($or$cond5) {
  $316 = HEAP32[$labelEnd>>2]|0;
  HEAP32[$addrEofA>>2] = $316;
  HEAP32[$addrEofA_noB>>2] = $316;
 } else {
  $317 = HEAP32[$v>>2]|0;
  $318 = HEAP32[$regOutB>>2]|0;
  $319 = HEAP32[$addrOutB>>2]|0;
  $320 = (_sqlite3VdbeAddOp2($317,17,$318,$319)|0);
  HEAP32[$addrEofA>>2] = $320;
  $321 = HEAP32[$v>>2]|0;
  $322 = HEAP32[$regAddrB>>2]|0;
  $323 = HEAP32[$labelEnd>>2]|0;
  $324 = (_sqlite3VdbeAddOp2($321,22,$322,$323)|0);
  HEAP32[$addrEofA_noB>>2] = $324;
  $325 = HEAP32[$v>>2]|0;
  $326 = HEAP32[$addrEofA>>2]|0;
  (_sqlite3VdbeAddOp2($325,16,0,$326)|0);
  $327 = HEAP32[$pPrior>>2]|0;
  $328 = (($327) + 24|0);
  $329 = $328;
  $330 = $329;
  $331 = HEAP32[$330>>2]|0;
  $332 = (($329) + 4)|0;
  $333 = $332;
  $334 = HEAP32[$333>>2]|0;
  $335 = HEAP32[$2>>2]|0;
  $336 = (($335) + 24|0);
  $337 = $336;
  $338 = $337;
  $339 = HEAP32[$338>>2]|0;
  $340 = (($337) + 4)|0;
  $341 = $340;
  $342 = HEAP32[$341>>2]|0;
  $343 = (_i64Add(($339|0),($342|0),($331|0),($334|0))|0);
  $344 = tempRet0;
  $345 = $336;
  $346 = $345;
  HEAP32[$346>>2] = $343;
  $347 = (($345) + 4)|0;
  $348 = $347;
  HEAP32[$348>>2] = $344;
 }
 $349 = HEAP32[$op>>2]|0;
 $350 = ($349|0)==(118);
 if ($350) {
  $351 = HEAP32[$addrEofA>>2]|0;
  HEAP32[$addrEofB>>2] = $351;
  $352 = HEAP32[$2>>2]|0;
  $353 = (($352) + 24|0);
  $354 = $353;
  $355 = $354;
  $356 = HEAP32[$355>>2]|0;
  $357 = (($354) + 4)|0;
  $358 = $357;
  $359 = HEAP32[$358>>2]|0;
  $360 = HEAP32[$pPrior>>2]|0;
  $361 = (($360) + 24|0);
  $362 = $361;
  $363 = $362;
  $364 = HEAP32[$363>>2]|0;
  $365 = (($362) + 4)|0;
  $366 = $365;
  $367 = HEAP32[$366>>2]|0;
  $368 = ($359>>>0)>($367>>>0);
  $369 = ($356>>>0)>($364>>>0);
  $370 = ($359|0)==($367|0);
  $371 = $370 & $369;
  $372 = $368 | $371;
  if ($372) {
   $373 = HEAP32[$pPrior>>2]|0;
   $374 = (($373) + 24|0);
   $375 = $374;
   $376 = $375;
   $377 = HEAP32[$376>>2]|0;
   $378 = (($375) + 4)|0;
   $379 = $378;
   $380 = HEAP32[$379>>2]|0;
   $381 = HEAP32[$2>>2]|0;
   $382 = (($381) + 24|0);
   $383 = $382;
   $384 = $383;
   HEAP32[$384>>2] = $377;
   $385 = (($383) + 4)|0;
   $386 = $385;
   HEAP32[$386>>2] = $380;
  }
 } else {
  $387 = HEAP32[$v>>2]|0;
  $388 = HEAP32[$regOutA>>2]|0;
  $389 = HEAP32[$addrOutA>>2]|0;
  $390 = (_sqlite3VdbeAddOp2($387,17,$388,$389)|0);
  HEAP32[$addrEofB>>2] = $390;
  $391 = HEAP32[$v>>2]|0;
  $392 = HEAP32[$regAddrA>>2]|0;
  $393 = HEAP32[$labelEnd>>2]|0;
  (_sqlite3VdbeAddOp2($391,22,$392,$393)|0);
  $394 = HEAP32[$v>>2]|0;
  $395 = HEAP32[$addrEofB>>2]|0;
  (_sqlite3VdbeAddOp2($394,16,0,$395)|0);
 }
 $396 = HEAP32[$v>>2]|0;
 $397 = HEAP32[$regOutA>>2]|0;
 $398 = HEAP32[$addrOutA>>2]|0;
 $399 = (_sqlite3VdbeAddOp2($396,17,$397,$398)|0);
 HEAP32[$addrAltB>>2] = $399;
 $400 = HEAP32[$v>>2]|0;
 $401 = HEAP32[$regAddrA>>2]|0;
 $402 = HEAP32[$addrEofA>>2]|0;
 (_sqlite3VdbeAddOp2($400,22,$401,$402)|0);
 $403 = HEAP32[$v>>2]|0;
 $404 = HEAP32[$labelCmpr>>2]|0;
 (_sqlite3VdbeAddOp2($403,16,0,$404)|0);
 $405 = HEAP32[$op>>2]|0;
 $406 = ($405|0)==(116);
 do {
  if ($406) {
   $407 = HEAP32[$addrAltB>>2]|0;
   HEAP32[$addrAeqB>>2] = $407;
  } else {
   $408 = HEAP32[$op>>2]|0;
   $409 = ($408|0)==(118);
   if ($409) {
    $410 = HEAP32[$addrAltB>>2]|0;
    HEAP32[$addrAeqB>>2] = $410;
    $411 = HEAP32[$addrAltB>>2]|0;
    $412 = (($411) + 1)|0;
    HEAP32[$addrAltB>>2] = $412;
    break;
   } else {
    $413 = HEAP32[$v>>2]|0;
    $414 = HEAP32[$regAddrA>>2]|0;
    $415 = HEAP32[$addrEofA>>2]|0;
    $416 = (_sqlite3VdbeAddOp2($413,22,$414,$415)|0);
    HEAP32[$addrAeqB>>2] = $416;
    $417 = HEAP32[$v>>2]|0;
    $418 = HEAP32[$labelCmpr>>2]|0;
    (_sqlite3VdbeAddOp2($417,16,0,$418)|0);
    break;
   }
  }
 } while(0);
 $419 = HEAP32[$v>>2]|0;
 $420 = (_sqlite3VdbeCurrentAddr($419)|0);
 HEAP32[$addrAgtB>>2] = $420;
 $421 = HEAP32[$op>>2]|0;
 $422 = ($421|0)==(116);
 $423 = HEAP32[$op>>2]|0;
 $424 = ($423|0)==(115);
 $or$cond7 = $422 | $424;
 if ($or$cond7) {
  $425 = HEAP32[$v>>2]|0;
  $426 = HEAP32[$regOutB>>2]|0;
  $427 = HEAP32[$addrOutB>>2]|0;
  (_sqlite3VdbeAddOp2($425,17,$426,$427)|0);
 }
 $428 = HEAP32[$v>>2]|0;
 $429 = HEAP32[$regAddrB>>2]|0;
 $430 = HEAP32[$addrEofB>>2]|0;
 (_sqlite3VdbeAddOp2($428,22,$429,$430)|0);
 $431 = HEAP32[$v>>2]|0;
 $432 = HEAP32[$labelCmpr>>2]|0;
 (_sqlite3VdbeAddOp2($431,16,0,$432)|0);
 $433 = HEAP32[$v>>2]|0;
 $434 = HEAP32[$j1>>2]|0;
 _sqlite3VdbeJumpHere($433,$434);
 $435 = HEAP32[$v>>2]|0;
 $436 = HEAP32[$regAddrA>>2]|0;
 $437 = HEAP32[$addrEofA_noB>>2]|0;
 (_sqlite3VdbeAddOp2($435,22,$436,$437)|0);
 $438 = HEAP32[$v>>2]|0;
 $439 = HEAP32[$regAddrB>>2]|0;
 $440 = HEAP32[$addrEofB>>2]|0;
 (_sqlite3VdbeAddOp2($438,22,$439,$440)|0);
 $441 = HEAP32[$v>>2]|0;
 $442 = HEAP32[$labelCmpr>>2]|0;
 _sqlite3VdbeResolveLabel($441,$442);
 $443 = HEAP32[$v>>2]|0;
 $444 = HEAP32[$aPermute>>2]|0;
 (_sqlite3VdbeAddOp4($443,41,0,0,0,$444,-15)|0);
 $445 = HEAP32[$v>>2]|0;
 $446 = (($destA) + 8|0);
 $447 = HEAP32[$446>>2]|0;
 $448 = (($destB) + 8|0);
 $449 = HEAP32[$448>>2]|0;
 $450 = HEAP32[$nOrderBy>>2]|0;
 $451 = HEAP32[$pKeyMerge>>2]|0;
 (_sqlite3VdbeAddOp4($445,42,$447,$449,$450,$451,-6)|0);
 $452 = HEAP32[$v>>2]|0;
 _sqlite3VdbeChangeP5($452,1);
 $453 = HEAP32[$v>>2]|0;
 $454 = HEAP32[$addrAltB>>2]|0;
 $455 = HEAP32[$addrAeqB>>2]|0;
 $456 = HEAP32[$addrAgtB>>2]|0;
 (_sqlite3VdbeAddOp3($453,43,$454,$455,$456)|0);
 $457 = HEAP32[$v>>2]|0;
 $458 = HEAP32[$labelEnd>>2]|0;
 _sqlite3VdbeResolveLabel($457,$458);
 $459 = HEAP32[$3>>2]|0;
 $460 = HEAP8[$459>>0]|0;
 $461 = $460&255;
 $462 = ($461|0)==(9);
 if ($462) {
  $463 = HEAP32[$pPrior>>2]|0;
  HEAP32[$pFirst>>2] = $463;
  while(1) {
   $464 = HEAP32[$pFirst>>2]|0;
   $465 = (($464) + 52|0);
   $466 = HEAP32[$465>>2]|0;
   $467 = ($466|0)!=(0|0);
   if (!($467)) {
    break;
   }
   $468 = HEAP32[$pFirst>>2]|0;
   $469 = (($468) + 52|0);
   $470 = HEAP32[$469>>2]|0;
   HEAP32[$pFirst>>2] = $470;
  }
  $471 = HEAP32[$1>>2]|0;
  $472 = HEAP32[$pFirst>>2]|0;
  $473 = HEAP32[$472>>2]|0;
  _generateColumnNames($471,0,$473);
 }
 $474 = HEAP32[$2>>2]|0;
 $475 = (($474) + 52|0);
 $476 = HEAP32[$475>>2]|0;
 $477 = ($476|0)!=(0|0);
 if ($477) {
  $478 = HEAP32[$db>>2]|0;
  $479 = HEAP32[$2>>2]|0;
  $480 = (($479) + 52|0);
  $481 = HEAP32[$480>>2]|0;
  _sqlite3SelectDelete($478,$481);
 }
 $482 = HEAP32[$pPrior>>2]|0;
 $483 = HEAP32[$2>>2]|0;
 $484 = (($483) + 52|0);
 HEAP32[$484>>2] = $482;
 $485 = HEAP32[$2>>2]|0;
 $486 = HEAP32[$pPrior>>2]|0;
 $487 = (($486) + 56|0);
 HEAP32[$487>>2] = $485;
 $488 = HEAP32[$1>>2]|0;
 $489 = HEAP32[$2>>2]|0;
 $490 = (($489) + 4|0);
 $491 = HEAP8[$490>>0]|0;
 $492 = $491&255;
 $493 = HEAP32[$iSub1>>2]|0;
 $494 = HEAP32[$iSub2>>2]|0;
 _explainComposite($488,$492,$493,$494,0);
 HEAP32[$0>>2] = 0;
 $495 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($495|0);
}
function _explainComposite($pParse,$op,$iSub1,$iSub2,$bUseTmp) {
 $pParse = $pParse|0;
 $op = $op|0;
 $iSub1 = $iSub1|0;
 $iSub2 = $iSub2|0;
 $bUseTmp = $bUseTmp|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $v = 0, $vararg_buffer = 0, $vararg_ptr1 = 0, $vararg_ptr2 = 0, $vararg_ptr3 = 0, $zMsg = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $vararg_buffer = sp;
 $0 = sp + 40|0;
 $1 = sp + 36|0;
 $2 = sp + 32|0;
 $3 = sp + 28|0;
 $4 = sp + 24|0;
 $v = sp + 20|0;
 $zMsg = sp + 16|0;
 HEAP32[$0>>2] = $pParse;
 HEAP32[$1>>2] = $op;
 HEAP32[$2>>2] = $iSub1;
 HEAP32[$3>>2] = $iSub2;
 HEAP32[$4>>2] = $bUseTmp;
 $5 = HEAP32[$0>>2]|0;
 $6 = (($5) + 450|0);
 $7 = HEAP8[$6>>0]|0;
 $8 = $7&255;
 $9 = ($8|0)==(2);
 if (!($9)) {
  STACKTOP = sp;return;
 }
 $10 = HEAP32[$0>>2]|0;
 $11 = (($10) + 8|0);
 $12 = HEAP32[$11>>2]|0;
 HEAP32[$v>>2] = $12;
 $13 = HEAP32[$0>>2]|0;
 $14 = HEAP32[$13>>2]|0;
 $15 = HEAP32[$2>>2]|0;
 $16 = HEAP32[$3>>2]|0;
 $17 = HEAP32[$4>>2]|0;
 $18 = ($17|0)!=(0);
 $19 = $18 ? 25320 : 1216;
 $20 = HEAP32[$1>>2]|0;
 $21 = (_selectOpName($20)|0);
 HEAP32[$vararg_buffer>>2] = $15;
 $vararg_ptr1 = (($vararg_buffer) + 4|0);
 HEAP32[$vararg_ptr1>>2] = $16;
 $vararg_ptr2 = (($vararg_buffer) + 8|0);
 HEAP32[$vararg_ptr2>>2] = $19;
 $vararg_ptr3 = (($vararg_buffer) + 12|0);
 HEAP32[$vararg_ptr3>>2] = $21;
 $22 = (_sqlite3MPrintf($14,25280,$vararg_buffer)|0);
 HEAP32[$zMsg>>2] = $22;
 $23 = HEAP32[$v>>2]|0;
 $24 = HEAP32[$0>>2]|0;
 $25 = (($24) + 464|0);
 $26 = HEAP32[$25>>2]|0;
 $27 = HEAP32[$zMsg>>2]|0;
 (_sqlite3VdbeAddOp4($23,155,$26,0,0,$27,-1)|0);
 STACKTOP = sp;return;
}
function _multiSelectCollSeq($pParse,$p,$iCol) {
 $pParse = $pParse|0;
 $p = $p|0;
 $iCol = $iCol|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $pRet = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $1 = sp + 8|0;
 $2 = sp + 4|0;
 $pRet = sp;
 HEAP32[$0>>2] = $pParse;
 HEAP32[$1>>2] = $p;
 HEAP32[$2>>2] = $iCol;
 $3 = HEAP32[$1>>2]|0;
 $4 = (($3) + 52|0);
 $5 = HEAP32[$4>>2]|0;
 $6 = ($5|0)!=(0|0);
 if ($6) {
  $7 = HEAP32[$0>>2]|0;
  $8 = HEAP32[$1>>2]|0;
  $9 = (($8) + 52|0);
  $10 = HEAP32[$9>>2]|0;
  $11 = HEAP32[$2>>2]|0;
  $12 = (_multiSelectCollSeq($7,$10,$11)|0);
  HEAP32[$pRet>>2] = $12;
 } else {
  HEAP32[$pRet>>2] = 0;
 }
 $13 = HEAP32[$pRet>>2]|0;
 $14 = ($13|0)==(0|0);
 if (!($14)) {
  $29 = HEAP32[$pRet>>2]|0;
  STACKTOP = sp;return ($29|0);
 }
 $15 = HEAP32[$2>>2]|0;
 $16 = HEAP32[$1>>2]|0;
 $17 = HEAP32[$16>>2]|0;
 $18 = HEAP32[$17>>2]|0;
 $19 = ($15|0)<($18|0);
 if (!($19)) {
  $29 = HEAP32[$pRet>>2]|0;
  STACKTOP = sp;return ($29|0);
 }
 $20 = HEAP32[$0>>2]|0;
 $21 = HEAP32[$2>>2]|0;
 $22 = HEAP32[$1>>2]|0;
 $23 = HEAP32[$22>>2]|0;
 $24 = (($23) + 4|0);
 $25 = HEAP32[$24>>2]|0;
 $26 = (($25) + (($21*20)|0)|0);
 $27 = HEAP32[$26>>2]|0;
 $28 = (_sqlite3ExprCollSeq($20,$27)|0);
 HEAP32[$pRet>>2] = $28;
 $29 = HEAP32[$pRet>>2]|0;
 STACKTOP = sp;return ($29|0);
}
function _multiSelectOrderByKeyInfo($pParse,$p,$nExtra) {
 $pParse = $pParse|0;
 $p = $p|0;
 $nExtra = $nExtra|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $8 = 0, $9 = 0, $db = 0;
 var $i = 0, $nOrderBy = 0, $pColl = 0, $pItem = 0, $pOrderBy = 0, $pRet = 0, $pTerm = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $0 = sp + 40|0;
 $1 = sp + 36|0;
 $2 = sp + 32|0;
 $pOrderBy = sp + 28|0;
 $nOrderBy = sp + 24|0;
 $db = sp + 20|0;
 $pRet = sp + 16|0;
 $i = sp + 12|0;
 $pItem = sp + 8|0;
 $pTerm = sp + 4|0;
 $pColl = sp;
 HEAP32[$0>>2] = $pParse;
 HEAP32[$1>>2] = $p;
 HEAP32[$2>>2] = $nExtra;
 $3 = HEAP32[$1>>2]|0;
 $4 = (($3) + 48|0);
 $5 = HEAP32[$4>>2]|0;
 HEAP32[$pOrderBy>>2] = $5;
 $6 = HEAP32[$1>>2]|0;
 $7 = (($6) + 48|0);
 $8 = HEAP32[$7>>2]|0;
 $9 = HEAP32[$8>>2]|0;
 HEAP32[$nOrderBy>>2] = $9;
 $10 = HEAP32[$0>>2]|0;
 $11 = HEAP32[$10>>2]|0;
 HEAP32[$db>>2] = $11;
 $12 = HEAP32[$db>>2]|0;
 $13 = HEAP32[$nOrderBy>>2]|0;
 $14 = HEAP32[$2>>2]|0;
 $15 = (($13) + ($14))|0;
 $16 = (_sqlite3KeyInfoAlloc($12,$15,1)|0);
 HEAP32[$pRet>>2] = $16;
 $17 = HEAP32[$pRet>>2]|0;
 $18 = ($17|0)!=(0|0);
 if (!($18)) {
  $78 = HEAP32[$pRet>>2]|0;
  STACKTOP = sp;return ($78|0);
 }
 HEAP32[$i>>2] = 0;
 while(1) {
  $19 = HEAP32[$i>>2]|0;
  $20 = HEAP32[$nOrderBy>>2]|0;
  $21 = ($19|0)<($20|0);
  if (!($21)) {
   break;
  }
  $22 = HEAP32[$i>>2]|0;
  $23 = HEAP32[$pOrderBy>>2]|0;
  $24 = (($23) + 4|0);
  $25 = HEAP32[$24>>2]|0;
  $26 = (($25) + (($22*20)|0)|0);
  HEAP32[$pItem>>2] = $26;
  $27 = HEAP32[$pItem>>2]|0;
  $28 = HEAP32[$27>>2]|0;
  HEAP32[$pTerm>>2] = $28;
  $29 = HEAP32[$pTerm>>2]|0;
  $30 = (($29) + 4|0);
  $31 = HEAP32[$30>>2]|0;
  $32 = $31 & 256;
  $33 = ($32|0)!=(0);
  $34 = HEAP32[$0>>2]|0;
  if ($33) {
   $35 = HEAP32[$pTerm>>2]|0;
   $36 = (_sqlite3ExprCollSeq($34,$35)|0);
   HEAP32[$pColl>>2] = $36;
  } else {
   $37 = HEAP32[$1>>2]|0;
   $38 = HEAP32[$pItem>>2]|0;
   $39 = (($38) + 16|0);
   $40 = HEAP16[$39>>1]|0;
   $41 = $40&65535;
   $42 = (($41) - 1)|0;
   $43 = (_multiSelectCollSeq($34,$37,$42)|0);
   HEAP32[$pColl>>2] = $43;
   $44 = HEAP32[$pColl>>2]|0;
   $45 = ($44|0)==(0|0);
   if ($45) {
    $46 = HEAP32[$db>>2]|0;
    $47 = (($46) + 8|0);
    $48 = HEAP32[$47>>2]|0;
    HEAP32[$pColl>>2] = $48;
   }
   $49 = HEAP32[$0>>2]|0;
   $50 = HEAP32[$pTerm>>2]|0;
   $51 = HEAP32[$pColl>>2]|0;
   $52 = HEAP32[$51>>2]|0;
   $53 = (_sqlite3ExprAddCollateString($49,$50,$52)|0);
   $54 = HEAP32[$i>>2]|0;
   $55 = HEAP32[$pOrderBy>>2]|0;
   $56 = (($55) + 4|0);
   $57 = HEAP32[$56>>2]|0;
   $58 = (($57) + (($54*20)|0)|0);
   HEAP32[$58>>2] = $53;
  }
  $59 = HEAP32[$pColl>>2]|0;
  $60 = HEAP32[$i>>2]|0;
  $61 = HEAP32[$pRet>>2]|0;
  $62 = (($61) + 20|0);
  $63 = (($62) + ($60<<2)|0);
  HEAP32[$63>>2] = $59;
  $64 = HEAP32[$i>>2]|0;
  $65 = HEAP32[$pOrderBy>>2]|0;
  $66 = (($65) + 4|0);
  $67 = HEAP32[$66>>2]|0;
  $68 = (($67) + (($64*20)|0)|0);
  $69 = (($68) + 12|0);
  $70 = HEAP8[$69>>0]|0;
  $71 = HEAP32[$i>>2]|0;
  $72 = HEAP32[$pRet>>2]|0;
  $73 = (($72) + 16|0);
  $74 = HEAP32[$73>>2]|0;
  $75 = (($74) + ($71)|0);
  HEAP8[$75>>0] = $70;
  $76 = HEAP32[$i>>2]|0;
  $77 = (($76) + 1)|0;
  HEAP32[$i>>2] = $77;
 }
 $78 = HEAP32[$pRet>>2]|0;
 STACKTOP = sp;return ($78|0);
}
function _generateOutputSubroutine($pParse,$p,$pIn,$pDest,$regReturn,$regPrev,$pKeyInfo,$iBreak) {
 $pParse = $pParse|0;
 $p = $p|0;
 $pIn = $pIn|0;
 $pDest = $pDest|0;
 $regReturn = $regReturn|0;
 $regPrev = $regPrev|0;
 $pKeyInfo = $pKeyInfo|0;
 $iBreak = $iBreak|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0;
 var $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0, $188 = 0;
 var $189 = 0, $19 = 0, $190 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0;
 var $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0;
 var $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0;
 var $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0;
 var $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $addr = 0, $iContinue = 0, $j1 = 0, $j2 = 0, $r1 = 0, $r11 = 0, $r2 = 0, $v = 0, label = 0;
 var sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 80|0;
 $0 = sp + 64|0;
 $1 = sp + 60|0;
 $2 = sp + 56|0;
 $3 = sp + 52|0;
 $4 = sp + 48|0;
 $5 = sp + 44|0;
 $6 = sp + 40|0;
 $7 = sp + 36|0;
 $8 = sp + 32|0;
 $v = sp + 28|0;
 $iContinue = sp + 24|0;
 $addr = sp + 20|0;
 $j1 = sp + 16|0;
 $j2 = sp + 12|0;
 $r1 = sp + 8|0;
 $r2 = sp + 4|0;
 $r11 = sp;
 HEAP32[$1>>2] = $pParse;
 HEAP32[$2>>2] = $p;
 HEAP32[$3>>2] = $pIn;
 HEAP32[$4>>2] = $pDest;
 HEAP32[$5>>2] = $regReturn;
 HEAP32[$6>>2] = $regPrev;
 HEAP32[$7>>2] = $pKeyInfo;
 HEAP32[$8>>2] = $iBreak;
 $9 = HEAP32[$1>>2]|0;
 $10 = (($9) + 8|0);
 $11 = HEAP32[$10>>2]|0;
 HEAP32[$v>>2] = $11;
 $12 = HEAP32[$v>>2]|0;
 $13 = (_sqlite3VdbeCurrentAddr($12)|0);
 HEAP32[$addr>>2] = $13;
 $14 = HEAP32[$v>>2]|0;
 $15 = (_sqlite3VdbeMakeLabel($14)|0);
 HEAP32[$iContinue>>2] = $15;
 $16 = HEAP32[$6>>2]|0;
 $17 = ($16|0)!=(0);
 if ($17) {
  $18 = HEAP32[$v>>2]|0;
  $19 = HEAP32[$6>>2]|0;
  $20 = (_sqlite3VdbeAddOp1($18,46,$19)|0);
  HEAP32[$j1>>2] = $20;
  $21 = HEAP32[$v>>2]|0;
  $22 = HEAP32[$3>>2]|0;
  $23 = (($22) + 8|0);
  $24 = HEAP32[$23>>2]|0;
  $25 = HEAP32[$6>>2]|0;
  $26 = (($25) + 1)|0;
  $27 = HEAP32[$3>>2]|0;
  $28 = (($27) + 12|0);
  $29 = HEAP32[$28>>2]|0;
  $30 = HEAP32[$7>>2]|0;
  $31 = (_sqlite3KeyInfoRef($30)|0);
  $32 = (_sqlite3VdbeAddOp4($21,42,$24,$26,$29,$31,-6)|0);
  HEAP32[$j2>>2] = $32;
  $33 = HEAP32[$v>>2]|0;
  $34 = HEAP32[$j2>>2]|0;
  $35 = (($34) + 2)|0;
  $36 = HEAP32[$iContinue>>2]|0;
  $37 = HEAP32[$j2>>2]|0;
  $38 = (($37) + 2)|0;
  (_sqlite3VdbeAddOp3($33,43,$35,$36,$38)|0);
  $39 = HEAP32[$v>>2]|0;
  $40 = HEAP32[$j1>>2]|0;
  _sqlite3VdbeJumpHere($39,$40);
  $41 = HEAP32[$v>>2]|0;
  $42 = HEAP32[$3>>2]|0;
  $43 = (($42) + 8|0);
  $44 = HEAP32[$43>>2]|0;
  $45 = HEAP32[$6>>2]|0;
  $46 = (($45) + 1)|0;
  $47 = HEAP32[$3>>2]|0;
  $48 = (($47) + 12|0);
  $49 = HEAP32[$48>>2]|0;
  $50 = (($49) - 1)|0;
  (_sqlite3VdbeAddOp3($41,33,$44,$46,$50)|0);
  $51 = HEAP32[$v>>2]|0;
  $52 = HEAP32[$6>>2]|0;
  (_sqlite3VdbeAddOp2($51,25,1,$52)|0);
 }
 $53 = HEAP32[$1>>2]|0;
 $54 = HEAP32[$53>>2]|0;
 $55 = (($54) + 64|0);
 $56 = HEAP8[$55>>0]|0;
 $57 = ($56<<24>>24)!=(0);
 if ($57) {
  HEAP32[$0>>2] = 0;
  $190 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($190|0);
 }
 $58 = HEAP32[$v>>2]|0;
 $59 = HEAP32[$2>>2]|0;
 $60 = (($59) + 12|0);
 $61 = HEAP32[$60>>2]|0;
 $62 = HEAP32[$iContinue>>2]|0;
 _codeOffset($58,$61,$62);
 $63 = HEAP32[$4>>2]|0;
 $64 = HEAP8[$63>>0]|0;
 $65 = $64&255;
 switch ($65|0) {
 case 12: case 14:  {
  $66 = HEAP32[$1>>2]|0;
  $67 = (_sqlite3GetTempReg($66)|0);
  HEAP32[$r1>>2] = $67;
  $68 = HEAP32[$1>>2]|0;
  $69 = (_sqlite3GetTempReg($68)|0);
  HEAP32[$r2>>2] = $69;
  $70 = HEAP32[$v>>2]|0;
  $71 = HEAP32[$3>>2]|0;
  $72 = (($71) + 8|0);
  $73 = HEAP32[$72>>2]|0;
  $74 = HEAP32[$3>>2]|0;
  $75 = (($74) + 12|0);
  $76 = HEAP32[$75>>2]|0;
  $77 = HEAP32[$r1>>2]|0;
  (_sqlite3VdbeAddOp3($70,49,$73,$76,$77)|0);
  $78 = HEAP32[$v>>2]|0;
  $79 = HEAP32[$4>>2]|0;
  $80 = (($79) + 4|0);
  $81 = HEAP32[$80>>2]|0;
  $82 = HEAP32[$r2>>2]|0;
  (_sqlite3VdbeAddOp2($78,74,$81,$82)|0);
  $83 = HEAP32[$v>>2]|0;
  $84 = HEAP32[$4>>2]|0;
  $85 = (($84) + 4|0);
  $86 = HEAP32[$85>>2]|0;
  $87 = HEAP32[$r1>>2]|0;
  $88 = HEAP32[$r2>>2]|0;
  (_sqlite3VdbeAddOp3($83,75,$86,$87,$88)|0);
  $89 = HEAP32[$v>>2]|0;
  _sqlite3VdbeChangeP5($89,8);
  $90 = HEAP32[$1>>2]|0;
  $91 = HEAP32[$r2>>2]|0;
  _sqlite3ReleaseTempReg($90,$91);
  $92 = HEAP32[$1>>2]|0;
  $93 = HEAP32[$r1>>2]|0;
  _sqlite3ReleaseTempReg($92,$93);
  break;
 }
 case 11:  {
  $94 = HEAP32[$2>>2]|0;
  $95 = HEAP32[$94>>2]|0;
  $96 = (($95) + 4|0);
  $97 = HEAP32[$96>>2]|0;
  $98 = HEAP32[$97>>2]|0;
  $99 = HEAP32[$4>>2]|0;
  $100 = (($99) + 1|0);
  $101 = HEAP8[$100>>0]|0;
  $102 = (_sqlite3CompareAffinity($98,$101)|0);
  $103 = HEAP32[$4>>2]|0;
  $104 = (($103) + 1|0);
  HEAP8[$104>>0] = $102;
  $105 = HEAP32[$1>>2]|0;
  $106 = (_sqlite3GetTempReg($105)|0);
  HEAP32[$r11>>2] = $106;
  $107 = HEAP32[$v>>2]|0;
  $108 = HEAP32[$3>>2]|0;
  $109 = (($108) + 8|0);
  $110 = HEAP32[$109>>2]|0;
  $111 = HEAP32[$r11>>2]|0;
  $112 = HEAP32[$4>>2]|0;
  $113 = (($112) + 1|0);
  (_sqlite3VdbeAddOp4($107,49,$110,1,$111,$113,1)|0);
  $114 = HEAP32[$1>>2]|0;
  $115 = HEAP32[$3>>2]|0;
  $116 = (($115) + 8|0);
  $117 = HEAP32[$116>>2]|0;
  _sqlite3ExprCacheAffinityChange($114,$117,1);
  $118 = HEAP32[$v>>2]|0;
  $119 = HEAP32[$4>>2]|0;
  $120 = (($119) + 4|0);
  $121 = HEAP32[$120>>2]|0;
  $122 = HEAP32[$r11>>2]|0;
  (_sqlite3VdbeAddOp2($118,110,$121,$122)|0);
  $123 = HEAP32[$1>>2]|0;
  $124 = HEAP32[$r11>>2]|0;
  _sqlite3ReleaseTempReg($123,$124);
  break;
 }
 case 10:  {
  $125 = HEAP32[$1>>2]|0;
  $126 = HEAP32[$3>>2]|0;
  $127 = (($126) + 8|0);
  $128 = HEAP32[$127>>2]|0;
  $129 = HEAP32[$4>>2]|0;
  $130 = (($129) + 4|0);
  $131 = HEAP32[$130>>2]|0;
  _sqlite3ExprCodeMove($125,$128,$131,1);
  break;
 }
 case 13:  {
  $132 = HEAP32[$4>>2]|0;
  $133 = (($132) + 8|0);
  $134 = HEAP32[$133>>2]|0;
  $135 = ($134|0)==(0);
  if ($135) {
   $136 = HEAP32[$1>>2]|0;
   $137 = HEAP32[$3>>2]|0;
   $138 = (($137) + 12|0);
   $139 = HEAP32[$138>>2]|0;
   $140 = (_sqlite3GetTempRange($136,$139)|0);
   $141 = HEAP32[$4>>2]|0;
   $142 = (($141) + 8|0);
   HEAP32[$142>>2] = $140;
   $143 = HEAP32[$3>>2]|0;
   $144 = (($143) + 12|0);
   $145 = HEAP32[$144>>2]|0;
   $146 = HEAP32[$4>>2]|0;
   $147 = (($146) + 12|0);
   HEAP32[$147>>2] = $145;
  }
  $148 = HEAP32[$1>>2]|0;
  $149 = HEAP32[$3>>2]|0;
  $150 = (($149) + 8|0);
  $151 = HEAP32[$150>>2]|0;
  $152 = HEAP32[$4>>2]|0;
  $153 = (($152) + 8|0);
  $154 = HEAP32[$153>>2]|0;
  $155 = HEAP32[$4>>2]|0;
  $156 = (($155) + 12|0);
  $157 = HEAP32[$156>>2]|0;
  _sqlite3ExprCodeMove($148,$151,$154,$157);
  $158 = HEAP32[$v>>2]|0;
  $159 = HEAP32[$4>>2]|0;
  $160 = (($159) + 4|0);
  $161 = HEAP32[$160>>2]|0;
  (_sqlite3VdbeAddOp1($158,22,$161)|0);
  break;
 }
 default: {
  $162 = HEAP32[$v>>2]|0;
  $163 = HEAP32[$3>>2]|0;
  $164 = (($163) + 8|0);
  $165 = HEAP32[$164>>2]|0;
  $166 = HEAP32[$3>>2]|0;
  $167 = (($166) + 12|0);
  $168 = HEAP32[$167>>2]|0;
  (_sqlite3VdbeAddOp2($162,35,$165,$168)|0);
  $169 = HEAP32[$1>>2]|0;
  $170 = HEAP32[$3>>2]|0;
  $171 = (($170) + 8|0);
  $172 = HEAP32[$171>>2]|0;
  $173 = HEAP32[$3>>2]|0;
  $174 = (($173) + 12|0);
  $175 = HEAP32[$174>>2]|0;
  _sqlite3ExprCacheAffinityChange($169,$172,$175);
 }
 }
 $176 = HEAP32[$2>>2]|0;
 $177 = (($176) + 8|0);
 $178 = HEAP32[$177>>2]|0;
 $179 = ($178|0)!=(0);
 if ($179) {
  $180 = HEAP32[$v>>2]|0;
  $181 = HEAP32[$2>>2]|0;
  $182 = (($181) + 8|0);
  $183 = HEAP32[$182>>2]|0;
  $184 = HEAP32[$8>>2]|0;
  (_sqlite3VdbeAddOp3($180,139,$183,$184,-1)|0);
 }
 $185 = HEAP32[$v>>2]|0;
 $186 = HEAP32[$iContinue>>2]|0;
 _sqlite3VdbeResolveLabel($185,$186);
 $187 = HEAP32[$v>>2]|0;
 $188 = HEAP32[$5>>2]|0;
 (_sqlite3VdbeAddOp1($187,18,$188)|0);
 $189 = HEAP32[$addr>>2]|0;
 HEAP32[$0>>2] = $189;
 $190 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($190|0);
}
function _substExprList($db,$pList,$iTable,$pEList) {
 $db = $db|0;
 $pList = $pList|0;
 $iTable = $iTable|0;
 $pEList = $pEList|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $i = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 16|0;
 $1 = sp + 12|0;
 $2 = sp + 8|0;
 $3 = sp + 4|0;
 $i = sp;
 HEAP32[$0>>2] = $db;
 HEAP32[$1>>2] = $pList;
 HEAP32[$2>>2] = $iTable;
 HEAP32[$3>>2] = $pEList;
 $4 = HEAP32[$1>>2]|0;
 $5 = ($4|0)==(0|0);
 if ($5) {
  STACKTOP = sp;return;
 }
 HEAP32[$i>>2] = 0;
 while(1) {
  $6 = HEAP32[$i>>2]|0;
  $7 = HEAP32[$1>>2]|0;
  $8 = HEAP32[$7>>2]|0;
  $9 = ($6|0)<($8|0);
  if (!($9)) {
   break;
  }
  $10 = HEAP32[$0>>2]|0;
  $11 = HEAP32[$i>>2]|0;
  $12 = HEAP32[$1>>2]|0;
  $13 = (($12) + 4|0);
  $14 = HEAP32[$13>>2]|0;
  $15 = (($14) + (($11*20)|0)|0);
  $16 = HEAP32[$15>>2]|0;
  $17 = HEAP32[$2>>2]|0;
  $18 = HEAP32[$3>>2]|0;
  $19 = (_substExpr($10,$16,$17,$18)|0);
  $20 = HEAP32[$i>>2]|0;
  $21 = HEAP32[$1>>2]|0;
  $22 = (($21) + 4|0);
  $23 = HEAP32[$22>>2]|0;
  $24 = (($23) + (($20*20)|0)|0);
  HEAP32[$24>>2] = $19;
  $25 = HEAP32[$i>>2]|0;
  $26 = (($25) + 1)|0;
  HEAP32[$i>>2] = $26;
 }
 STACKTOP = sp;return;
}
function _substExpr($db,$pExpr,$iTable,$pEList) {
 $db = $db|0;
 $pExpr = $pExpr|0;
 $iTable = $iTable|0;
 $pEList = $pEList|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $8 = 0, $9 = 0, $pNew = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 20|0;
 $1 = sp + 16|0;
 $2 = sp + 12|0;
 $3 = sp + 8|0;
 $4 = sp + 4|0;
 $pNew = sp;
 HEAP32[$1>>2] = $db;
 HEAP32[$2>>2] = $pExpr;
 HEAP32[$3>>2] = $iTable;
 HEAP32[$4>>2] = $pEList;
 $5 = HEAP32[$2>>2]|0;
 $6 = ($5|0)==(0|0);
 if ($6) {
  HEAP32[$0>>2] = 0;
  $69 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($69|0);
 }
 $7 = HEAP32[$2>>2]|0;
 $8 = HEAP8[$7>>0]|0;
 $9 = $8&255;
 $10 = ($9|0)==(154);
 do {
  if ($10) {
   $11 = HEAP32[$2>>2]|0;
   $12 = (($11) + 28|0);
   $13 = HEAP32[$12>>2]|0;
   $14 = HEAP32[$3>>2]|0;
   $15 = ($13|0)==($14|0);
   if ($15) {
    $16 = HEAP32[$2>>2]|0;
    $17 = (($16) + 32|0);
    $18 = HEAP16[$17>>1]|0;
    $19 = $18 << 16 >> 16;
    $20 = ($19|0)<(0);
    if ($20) {
     $21 = HEAP32[$2>>2]|0;
     HEAP8[$21>>0] = 101;
     break;
    } else {
     $22 = HEAP32[$1>>2]|0;
     $23 = HEAP32[$2>>2]|0;
     $24 = (($23) + 32|0);
     $25 = HEAP16[$24>>1]|0;
     $26 = $25 << 16 >> 16;
     $27 = HEAP32[$4>>2]|0;
     $28 = (($27) + 4|0);
     $29 = HEAP32[$28>>2]|0;
     $30 = (($29) + (($26*20)|0)|0);
     $31 = HEAP32[$30>>2]|0;
     $32 = (_sqlite3ExprDup($22,$31,0)|0);
     HEAP32[$pNew>>2] = $32;
     $33 = HEAP32[$1>>2]|0;
     $34 = HEAP32[$2>>2]|0;
     _sqlite3ExprDelete($33,$34);
     $35 = HEAP32[$pNew>>2]|0;
     HEAP32[$2>>2] = $35;
     break;
    }
   } else {
    label = 8;
   }
  } else {
   label = 8;
  }
 } while(0);
 do {
  if ((label|0) == 8) {
   $36 = HEAP32[$1>>2]|0;
   $37 = HEAP32[$2>>2]|0;
   $38 = (($37) + 12|0);
   $39 = HEAP32[$38>>2]|0;
   $40 = HEAP32[$3>>2]|0;
   $41 = HEAP32[$4>>2]|0;
   $42 = (_substExpr($36,$39,$40,$41)|0);
   $43 = HEAP32[$2>>2]|0;
   $44 = (($43) + 12|0);
   HEAP32[$44>>2] = $42;
   $45 = HEAP32[$1>>2]|0;
   $46 = HEAP32[$2>>2]|0;
   $47 = (($46) + 16|0);
   $48 = HEAP32[$47>>2]|0;
   $49 = HEAP32[$3>>2]|0;
   $50 = HEAP32[$4>>2]|0;
   $51 = (_substExpr($45,$48,$49,$50)|0);
   $52 = HEAP32[$2>>2]|0;
   $53 = (($52) + 16|0);
   HEAP32[$53>>2] = $51;
   $54 = HEAP32[$2>>2]|0;
   $55 = (($54) + 4|0);
   $56 = HEAP32[$55>>2]|0;
   $57 = $56 & 2048;
   $58 = ($57|0)!=(0);
   $59 = HEAP32[$1>>2]|0;
   $60 = HEAP32[$2>>2]|0;
   $61 = (($60) + 20|0);
   if ($58) {
    $62 = HEAP32[$61>>2]|0;
    $63 = HEAP32[$3>>2]|0;
    $64 = HEAP32[$4>>2]|0;
    _substSelect($59,$62,$63,$64);
    break;
   } else {
    $65 = HEAP32[$61>>2]|0;
    $66 = HEAP32[$3>>2]|0;
    $67 = HEAP32[$4>>2]|0;
    _substExprList($59,$65,$66,$67);
    break;
   }
  }
 } while(0);
 $68 = HEAP32[$2>>2]|0;
 HEAP32[$0>>2] = $68;
 $69 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($69|0);
}
function _substSelect($db,$p,$iTable,$pEList) {
 $db = $db|0;
 $p = $p|0;
 $iTable = $iTable|0;
 $pEList = $pEList|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $7 = 0, $8 = 0, $9 = 0, $i = 0, $pItem = 0, $pSrc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 24|0;
 $1 = sp + 20|0;
 $2 = sp + 16|0;
 $3 = sp + 12|0;
 $pSrc = sp + 8|0;
 $pItem = sp + 4|0;
 $i = sp;
 HEAP32[$0>>2] = $db;
 HEAP32[$1>>2] = $p;
 HEAP32[$2>>2] = $iTable;
 HEAP32[$3>>2] = $pEList;
 $4 = HEAP32[$1>>2]|0;
 $5 = ($4|0)!=(0|0);
 if (!($5)) {
  STACKTOP = sp;return;
 }
 $6 = HEAP32[$0>>2]|0;
 $7 = HEAP32[$1>>2]|0;
 $8 = HEAP32[$7>>2]|0;
 $9 = HEAP32[$2>>2]|0;
 $10 = HEAP32[$3>>2]|0;
 _substExprList($6,$8,$9,$10);
 $11 = HEAP32[$0>>2]|0;
 $12 = HEAP32[$1>>2]|0;
 $13 = (($12) + 40|0);
 $14 = HEAP32[$13>>2]|0;
 $15 = HEAP32[$2>>2]|0;
 $16 = HEAP32[$3>>2]|0;
 _substExprList($11,$14,$15,$16);
 $17 = HEAP32[$0>>2]|0;
 $18 = HEAP32[$1>>2]|0;
 $19 = (($18) + 48|0);
 $20 = HEAP32[$19>>2]|0;
 $21 = HEAP32[$2>>2]|0;
 $22 = HEAP32[$3>>2]|0;
 _substExprList($17,$20,$21,$22);
 $23 = HEAP32[$0>>2]|0;
 $24 = HEAP32[$1>>2]|0;
 $25 = (($24) + 44|0);
 $26 = HEAP32[$25>>2]|0;
 $27 = HEAP32[$2>>2]|0;
 $28 = HEAP32[$3>>2]|0;
 $29 = (_substExpr($23,$26,$27,$28)|0);
 $30 = HEAP32[$1>>2]|0;
 $31 = (($30) + 44|0);
 HEAP32[$31>>2] = $29;
 $32 = HEAP32[$0>>2]|0;
 $33 = HEAP32[$1>>2]|0;
 $34 = (($33) + 36|0);
 $35 = HEAP32[$34>>2]|0;
 $36 = HEAP32[$2>>2]|0;
 $37 = HEAP32[$3>>2]|0;
 $38 = (_substExpr($32,$35,$36,$37)|0);
 $39 = HEAP32[$1>>2]|0;
 $40 = (($39) + 36|0);
 HEAP32[$40>>2] = $38;
 $41 = HEAP32[$0>>2]|0;
 $42 = HEAP32[$1>>2]|0;
 $43 = (($42) + 52|0);
 $44 = HEAP32[$43>>2]|0;
 $45 = HEAP32[$2>>2]|0;
 $46 = HEAP32[$3>>2]|0;
 _substSelect($41,$44,$45,$46);
 $47 = HEAP32[$1>>2]|0;
 $48 = (($47) + 32|0);
 $49 = HEAP32[$48>>2]|0;
 HEAP32[$pSrc>>2] = $49;
 $50 = HEAP32[$pSrc>>2]|0;
 $51 = ($50|0)!=(0|0);
 if (!($51)) {
  STACKTOP = sp;return;
 }
 $52 = HEAP32[$pSrc>>2]|0;
 $53 = HEAP32[$52>>2]|0;
 HEAP32[$i>>2] = $53;
 $54 = HEAP32[$pSrc>>2]|0;
 $55 = (($54) + 8|0);
 HEAP32[$pItem>>2] = $55;
 while(1) {
  $56 = HEAP32[$i>>2]|0;
  $57 = ($56|0)>(0);
  if (!($57)) {
   break;
  }
  $58 = HEAP32[$0>>2]|0;
  $59 = HEAP32[$pItem>>2]|0;
  $60 = (($59) + 20|0);
  $61 = HEAP32[$60>>2]|0;
  $62 = HEAP32[$2>>2]|0;
  $63 = HEAP32[$3>>2]|0;
  _substSelect($58,$61,$62,$63);
  $64 = HEAP32[$i>>2]|0;
  $65 = (($64) + -1)|0;
  HEAP32[$i>>2] = $65;
  $66 = HEAP32[$pItem>>2]|0;
  $67 = (($66) + 72|0);
  HEAP32[$pItem>>2] = $67;
 }
 STACKTOP = sp;return;
}
function _sqlite3FkDropTable($pParse,$pName,$pTab) {
 $pParse = $pParse|0;
 $pName = $pName|0;
 $pTab = $pTab|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $8 = 0, $9 = 0, $db = 0, $iSkip = 0, $p = 0, $v = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 24|0;
 $1 = sp + 20|0;
 $2 = sp + 16|0;
 $db = sp + 12|0;
 $iSkip = sp + 8|0;
 $v = sp + 4|0;
 $p = sp;
 HEAP32[$0>>2] = $pParse;
 HEAP32[$1>>2] = $pName;
 HEAP32[$2>>2] = $pTab;
 $3 = HEAP32[$0>>2]|0;
 $4 = HEAP32[$3>>2]|0;
 HEAP32[$db>>2] = $4;
 $5 = HEAP32[$db>>2]|0;
 $6 = (($5) + 24|0);
 $7 = HEAP32[$6>>2]|0;
 $8 = $7 & 524288;
 $9 = ($8|0)!=(0);
 if (!($9)) {
  STACKTOP = sp;return;
 }
 $10 = HEAP32[$2>>2]|0;
 $11 = (($10) + 44|0);
 $12 = HEAP8[$11>>0]|0;
 $13 = $12&255;
 $14 = $13 & 16;
 $15 = ($14|0)!=(0);
 if ($15) {
  STACKTOP = sp;return;
 }
 $16 = HEAP32[$2>>2]|0;
 $17 = (($16) + 12|0);
 $18 = HEAP32[$17>>2]|0;
 $19 = ($18|0)!=(0|0);
 if ($19) {
  STACKTOP = sp;return;
 }
 HEAP32[$iSkip>>2] = 0;
 $20 = HEAP32[$0>>2]|0;
 $21 = (_sqlite3GetVdbe($20)|0);
 HEAP32[$v>>2] = $21;
 $22 = HEAP32[$2>>2]|0;
 $23 = (_sqlite3FkReferences($22)|0);
 $24 = ($23|0)==(0|0);
 do {
  if ($24) {
   $25 = HEAP32[$2>>2]|0;
   $26 = (($25) + 16|0);
   $27 = HEAP32[$26>>2]|0;
   HEAP32[$p>>2] = $27;
   while(1) {
    $28 = HEAP32[$p>>2]|0;
    $29 = ($28|0)!=(0|0);
    if (!($29)) {
     break;
    }
    $30 = HEAP32[$p>>2]|0;
    $31 = (($30) + 24|0);
    $32 = HEAP8[$31>>0]|0;
    $33 = $32&255;
    $34 = ($33|0)!=(0);
    if ($34) {
     break;
    }
    $35 = HEAP32[$db>>2]|0;
    $36 = (($35) + 24|0);
    $37 = HEAP32[$36>>2]|0;
    $38 = $37 & 16777216;
    $39 = ($38|0)!=(0);
    if ($39) {
     break;
    }
    $40 = HEAP32[$p>>2]|0;
    $41 = (($40) + 4|0);
    $42 = HEAP32[$41>>2]|0;
    HEAP32[$p>>2] = $42;
   }
   $43 = HEAP32[$p>>2]|0;
   $44 = ($43|0)!=(0|0);
   if ($44) {
    $45 = HEAP32[$v>>2]|0;
    $46 = (_sqlite3VdbeMakeLabel($45)|0);
    HEAP32[$iSkip>>2] = $46;
    $47 = HEAP32[$v>>2]|0;
    $48 = HEAP32[$iSkip>>2]|0;
    (_sqlite3VdbeAddOp2($47,135,1,$48)|0);
    break;
   } else {
    STACKTOP = sp;return;
   }
  }
 } while(0);
 $49 = HEAP32[$0>>2]|0;
 $50 = (($49) + 438|0);
 HEAP8[$50>>0] = 1;
 $51 = HEAP32[$0>>2]|0;
 $52 = HEAP32[$db>>2]|0;
 $53 = HEAP32[$1>>2]|0;
 $54 = (_sqlite3SrcListDup($52,$53,0)|0);
 _sqlite3DeleteFrom($51,$54,0);
 $55 = HEAP32[$0>>2]|0;
 $56 = (($55) + 438|0);
 HEAP8[$56>>0] = 0;
 $57 = HEAP32[$db>>2]|0;
 $58 = (($57) + 24|0);
 $59 = HEAP32[$58>>2]|0;
 $60 = $59 & 16777216;
 $61 = ($60|0)==(0);
 if ($61) {
  $62 = HEAP32[$v>>2]|0;
  $63 = HEAP32[$v>>2]|0;
  $64 = (_sqlite3VdbeCurrentAddr($63)|0);
  $65 = (($64) + 2)|0;
  (_sqlite3VdbeAddOp2($62,135,0,$65)|0);
  $66 = HEAP32[$0>>2]|0;
  _sqlite3HaltConstraint($66,787,2,0,-2,4);
 }
 $67 = HEAP32[$iSkip>>2]|0;
 $68 = ($67|0)!=(0);
 if (!($68)) {
  STACKTOP = sp;return;
 }
 $69 = HEAP32[$v>>2]|0;
 $70 = HEAP32[$iSkip>>2]|0;
 _sqlite3VdbeResolveLabel($69,$70);
 STACKTOP = sp;return;
}
function _sqlite3CodeDropTable($pParse,$pTab,$iDb,$isView) {
 $pParse = $pParse|0;
 $pTab = $pTab|0;
 $iDb = $iDb|0;
 $isView = $isView|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $8 = 0, $9 = 0, $db = 0;
 var $pDb = 0, $pTrigger = 0, $v = 0, $vararg_buffer2 = 0, $vararg_ptr1 = 0, $vararg_ptr5 = 0, $vararg_ptr6 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $vararg_buffer2 = sp;
 $0 = sp + 40|0;
 $1 = sp + 36|0;
 $2 = sp + 32|0;
 $3 = sp + 28|0;
 $v = sp + 24|0;
 $db = sp + 20|0;
 $pTrigger = sp + 16|0;
 $pDb = sp + 12|0;
 HEAP32[$0>>2] = $pParse;
 HEAP32[$1>>2] = $pTab;
 HEAP32[$2>>2] = $iDb;
 HEAP32[$3>>2] = $isView;
 $4 = HEAP32[$0>>2]|0;
 $5 = HEAP32[$4>>2]|0;
 HEAP32[$db>>2] = $5;
 $6 = HEAP32[$2>>2]|0;
 $7 = HEAP32[$db>>2]|0;
 $8 = (($7) + 16|0);
 $9 = HEAP32[$8>>2]|0;
 $10 = (($9) + ($6<<4)|0);
 HEAP32[$pDb>>2] = $10;
 $11 = HEAP32[$0>>2]|0;
 $12 = (_sqlite3GetVdbe($11)|0);
 HEAP32[$v>>2] = $12;
 $13 = HEAP32[$0>>2]|0;
 $14 = HEAP32[$2>>2]|0;
 _sqlite3BeginWriteOperation($13,1,$14);
 $15 = HEAP32[$1>>2]|0;
 $16 = (($15) + 44|0);
 $17 = HEAP8[$16>>0]|0;
 $18 = $17&255;
 $19 = $18 & 16;
 $20 = ($19|0)!=(0);
 if ($20) {
  $21 = HEAP32[$v>>2]|0;
  (_sqlite3VdbeAddOp0($21,144)|0);
 }
 $22 = HEAP32[$0>>2]|0;
 $23 = HEAP32[$1>>2]|0;
 $24 = (_sqlite3TriggerList($22,$23)|0);
 HEAP32[$pTrigger>>2] = $24;
 while(1) {
  $25 = HEAP32[$pTrigger>>2]|0;
  $26 = ($25|0)!=(0|0);
  if (!($26)) {
   break;
  }
  $27 = HEAP32[$0>>2]|0;
  $28 = HEAP32[$pTrigger>>2]|0;
  _sqlite3DropTriggerPtr($27,$28);
  $29 = HEAP32[$pTrigger>>2]|0;
  $30 = (($29) + 32|0);
  $31 = HEAP32[$30>>2]|0;
  HEAP32[$pTrigger>>2] = $31;
 }
 $32 = HEAP32[$1>>2]|0;
 $33 = (($32) + 44|0);
 $34 = HEAP8[$33>>0]|0;
 $35 = $34&255;
 $36 = $35 & 8;
 $37 = ($36|0)!=(0);
 if ($37) {
  $38 = HEAP32[$0>>2]|0;
  $39 = HEAP32[$pDb>>2]|0;
  $40 = HEAP32[$39>>2]|0;
  $41 = HEAP32[$1>>2]|0;
  $42 = HEAP32[$41>>2]|0;
  HEAP32[$vararg_buffer2>>2] = $40;
  $vararg_ptr1 = (($vararg_buffer2) + 4|0);
  HEAP32[$vararg_ptr1>>2] = $42;
  _sqlite3NestedParse($38,25632,$vararg_buffer2);
 }
 $43 = HEAP32[$0>>2]|0;
 $44 = HEAP32[$pDb>>2]|0;
 $45 = HEAP32[$44>>2]|0;
 $46 = HEAP32[$2>>2]|0;
 $47 = ($46|0)==(1);
 $48 = $47 ? 14904 : 14928;
 $49 = HEAP32[$1>>2]|0;
 $50 = HEAP32[$49>>2]|0;
 HEAP32[$vararg_buffer2>>2] = $45;
 $vararg_ptr5 = (($vararg_buffer2) + 4|0);
 HEAP32[$vararg_ptr5>>2] = $48;
 $vararg_ptr6 = (($vararg_buffer2) + 8|0);
 HEAP32[$vararg_ptr6>>2] = $50;
 _sqlite3NestedParse($43,25680,$vararg_buffer2);
 $51 = HEAP32[$3>>2]|0;
 $52 = ($51|0)!=(0);
 if (!($52)) {
  $53 = HEAP32[$1>>2]|0;
  $54 = (($53) + 44|0);
  $55 = HEAP8[$54>>0]|0;
  $56 = $55&255;
  $57 = $56 & 16;
  $58 = ($57|0)!=(0);
  if (!($58)) {
   $59 = HEAP32[$0>>2]|0;
   $60 = HEAP32[$1>>2]|0;
   _destroyTable($59,$60);
  }
 }
 $61 = HEAP32[$1>>2]|0;
 $62 = (($61) + 44|0);
 $63 = HEAP8[$62>>0]|0;
 $64 = $63&255;
 $65 = $64 & 16;
 $66 = ($65|0)!=(0);
 if (!($66)) {
  $71 = HEAP32[$v>>2]|0;
  $72 = HEAP32[$2>>2]|0;
  $73 = HEAP32[$1>>2]|0;
  $74 = HEAP32[$73>>2]|0;
  (_sqlite3VdbeAddOp4($71,124,$72,0,0,$74,0)|0);
  $75 = HEAP32[$0>>2]|0;
  $76 = HEAP32[$2>>2]|0;
  _sqlite3ChangeCookie($75,$76);
  $77 = HEAP32[$db>>2]|0;
  $78 = HEAP32[$2>>2]|0;
  _sqliteViewResetAll($77,$78);
  STACKTOP = sp;return;
 }
 $67 = HEAP32[$v>>2]|0;
 $68 = HEAP32[$2>>2]|0;
 $69 = HEAP32[$1>>2]|0;
 $70 = HEAP32[$69>>2]|0;
 (_sqlite3VdbeAddOp4($67,146,$68,0,0,$70,0)|0);
 $71 = HEAP32[$v>>2]|0;
 $72 = HEAP32[$2>>2]|0;
 $73 = HEAP32[$1>>2]|0;
 $74 = HEAP32[$73>>2]|0;
 (_sqlite3VdbeAddOp4($71,124,$72,0,0,$74,0)|0);
 $75 = HEAP32[$0>>2]|0;
 $76 = HEAP32[$2>>2]|0;
 _sqlite3ChangeCookie($75,$76);
 $77 = HEAP32[$db>>2]|0;
 $78 = HEAP32[$2>>2]|0;
 _sqliteViewResetAll($77,$78);
 STACKTOP = sp;return;
}
function _destroyTable($pParse,$pTab) {
 $pParse = $pParse|0;
 $pTab = $pTab|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $5 = 0, $6 = 0;
 var $7 = 0, $8 = 0, $9 = 0, $iDb = 0, $iDestroyed = 0, $iIdx = 0, $iLargest = 0, $iTab = 0, $pIdx = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 28|0;
 $1 = sp + 24|0;
 $iTab = sp + 20|0;
 $iDestroyed = sp + 16|0;
 $pIdx = sp + 12|0;
 $iLargest = sp + 8|0;
 $iIdx = sp + 4|0;
 $iDb = sp;
 HEAP32[$0>>2] = $pParse;
 HEAP32[$1>>2] = $pTab;
 $2 = HEAP32[$1>>2]|0;
 $3 = (($2) + 32|0);
 $4 = HEAP32[$3>>2]|0;
 HEAP32[$iTab>>2] = $4;
 HEAP32[$iDestroyed>>2] = 0;
 while(1) {
  HEAP32[$iLargest>>2] = 0;
  $5 = HEAP32[$iDestroyed>>2]|0;
  $6 = ($5|0)==(0);
  if ($6) {
   label = 4;
  } else {
   $7 = HEAP32[$iTab>>2]|0;
   $8 = HEAP32[$iDestroyed>>2]|0;
   $9 = ($7|0)<($8|0);
   if ($9) {
    label = 4;
   }
  }
  if ((label|0) == 4) {
   label = 0;
   $10 = HEAP32[$iTab>>2]|0;
   HEAP32[$iLargest>>2] = $10;
  }
  $11 = HEAP32[$1>>2]|0;
  $12 = (($11) + 8|0);
  $13 = HEAP32[$12>>2]|0;
  HEAP32[$pIdx>>2] = $13;
  while(1) {
   $14 = HEAP32[$pIdx>>2]|0;
   $15 = ($14|0)!=(0|0);
   if (!($15)) {
    break;
   }
   $16 = HEAP32[$pIdx>>2]|0;
   $17 = (($16) + 44|0);
   $18 = HEAP32[$17>>2]|0;
   HEAP32[$iIdx>>2] = $18;
   $19 = HEAP32[$iDestroyed>>2]|0;
   $20 = ($19|0)==(0);
   if ($20) {
    label = 9;
   } else {
    $21 = HEAP32[$iIdx>>2]|0;
    $22 = HEAP32[$iDestroyed>>2]|0;
    $23 = ($21|0)<($22|0);
    if ($23) {
     label = 9;
    }
   }
   if ((label|0) == 9) {
    label = 0;
    $24 = HEAP32[$iIdx>>2]|0;
    $25 = HEAP32[$iLargest>>2]|0;
    $26 = ($24|0)>($25|0);
    if ($26) {
     $27 = HEAP32[$iIdx>>2]|0;
     HEAP32[$iLargest>>2] = $27;
    }
   }
   $28 = HEAP32[$pIdx>>2]|0;
   $29 = (($28) + 20|0);
   $30 = HEAP32[$29>>2]|0;
   HEAP32[$pIdx>>2] = $30;
  }
  $31 = HEAP32[$iLargest>>2]|0;
  $32 = ($31|0)==(0);
  if ($32) {
   break;
  }
  $33 = HEAP32[$0>>2]|0;
  $34 = HEAP32[$33>>2]|0;
  $35 = HEAP32[$1>>2]|0;
  $36 = (($35) + 68|0);
  $37 = HEAP32[$36>>2]|0;
  $38 = (_sqlite3SchemaToIndex($34,$37)|0);
  HEAP32[$iDb>>2] = $38;
  $39 = HEAP32[$0>>2]|0;
  $40 = HEAP32[$iLargest>>2]|0;
  $41 = HEAP32[$iDb>>2]|0;
  _destroyRootPage($39,$40,$41);
  $42 = HEAP32[$iLargest>>2]|0;
  HEAP32[$iDestroyed>>2] = $42;
 }
 STACKTOP = sp;return;
}
function _sqliteViewResetAll($db,$idx) {
 $db = $db|0;
 $idx = $idx|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $i = 0, $pTab = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $1 = sp + 8|0;
 $i = sp + 4|0;
 $pTab = sp;
 HEAP32[$0>>2] = $db;
 HEAP32[$1>>2] = $idx;
 $2 = HEAP32[$1>>2]|0;
 $3 = HEAP32[$0>>2]|0;
 $4 = (($3) + 16|0);
 $5 = HEAP32[$4>>2]|0;
 $6 = (($5) + ($2<<4)|0);
 $7 = (($6) + 12|0);
 $8 = HEAP32[$7>>2]|0;
 $9 = (($8) + 78|0);
 $10 = HEAP16[$9>>1]|0;
 $11 = $10&65535;
 $12 = $11 & 2;
 $13 = ($12|0)==(2);
 if (!($13)) {
  STACKTOP = sp;return;
 }
 $14 = HEAP32[$1>>2]|0;
 $15 = HEAP32[$0>>2]|0;
 $16 = (($15) + 16|0);
 $17 = HEAP32[$16>>2]|0;
 $18 = (($17) + ($14<<4)|0);
 $19 = (($18) + 12|0);
 $20 = HEAP32[$19>>2]|0;
 $21 = (($20) + 8|0);
 $22 = (($21) + 8|0);
 $23 = HEAP32[$22>>2]|0;
 HEAP32[$i>>2] = $23;
 while(1) {
  $24 = HEAP32[$i>>2]|0;
  $25 = ($24|0)!=(0|0);
  if (!($25)) {
   break;
  }
  $26 = HEAP32[$i>>2]|0;
  $27 = (($26) + 8|0);
  $28 = HEAP32[$27>>2]|0;
  HEAP32[$pTab>>2] = $28;
  $29 = HEAP32[$pTab>>2]|0;
  $30 = (($29) + 12|0);
  $31 = HEAP32[$30>>2]|0;
  $32 = ($31|0)!=(0|0);
  if ($32) {
   $33 = HEAP32[$0>>2]|0;
   $34 = HEAP32[$pTab>>2]|0;
   _sqliteDeleteColumnNames($33,$34);
   $35 = HEAP32[$pTab>>2]|0;
   $36 = (($35) + 4|0);
   HEAP32[$36>>2] = 0;
   $37 = HEAP32[$pTab>>2]|0;
   $38 = (($37) + 38|0);
   HEAP16[$38>>1] = 0;
  }
  $39 = HEAP32[$i>>2]|0;
  $40 = HEAP32[$39>>2]|0;
  HEAP32[$i>>2] = $40;
 }
 $41 = HEAP32[$1>>2]|0;
 $42 = HEAP32[$0>>2]|0;
 $43 = (($42) + 16|0);
 $44 = HEAP32[$43>>2]|0;
 $45 = (($44) + ($41<<4)|0);
 $46 = (($45) + 12|0);
 $47 = HEAP32[$46>>2]|0;
 $48 = (($47) + 78|0);
 $49 = HEAP16[$48>>1]|0;
 $50 = $49&65535;
 $51 = $50 & -3;
 $52 = $51&65535;
 HEAP16[$48>>1] = $52;
 STACKTOP = sp;return;
}
function _sqliteDeleteColumnNames($db,$pTable) {
 $db = $db|0;
 $pTable = $pTable|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $i = 0, $pCol = 0;
 var label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $1 = sp + 8|0;
 $i = sp + 4|0;
 $pCol = sp;
 HEAP32[$0>>2] = $db;
 HEAP32[$1>>2] = $pTable;
 $2 = HEAP32[$1>>2]|0;
 $3 = (($2) + 4|0);
 $4 = HEAP32[$3>>2]|0;
 HEAP32[$pCol>>2] = $4;
 $5 = ($4|0)!=(0|0);
 if (!($5)) {
  STACKTOP = sp;return;
 }
 HEAP32[$i>>2] = 0;
 while(1) {
  $6 = HEAP32[$i>>2]|0;
  $7 = HEAP32[$1>>2]|0;
  $8 = (($7) + 38|0);
  $9 = HEAP16[$8>>1]|0;
  $10 = $9 << 16 >> 16;
  $11 = ($6|0)<($10|0);
  $12 = HEAP32[$0>>2]|0;
  if (!($11)) {
   break;
  }
  $13 = HEAP32[$pCol>>2]|0;
  $14 = HEAP32[$13>>2]|0;
  _sqlite3DbFree($12,$14);
  $15 = HEAP32[$0>>2]|0;
  $16 = HEAP32[$pCol>>2]|0;
  $17 = (($16) + 4|0);
  $18 = HEAP32[$17>>2]|0;
  _sqlite3ExprDelete($15,$18);
  $19 = HEAP32[$0>>2]|0;
  $20 = HEAP32[$pCol>>2]|0;
  $21 = (($20) + 8|0);
  $22 = HEAP32[$21>>2]|0;
  _sqlite3DbFree($19,$22);
  $23 = HEAP32[$0>>2]|0;
  $24 = HEAP32[$pCol>>2]|0;
  $25 = (($24) + 12|0);
  $26 = HEAP32[$25>>2]|0;
  _sqlite3DbFree($23,$26);
  $27 = HEAP32[$0>>2]|0;
  $28 = HEAP32[$pCol>>2]|0;
  $29 = (($28) + 16|0);
  $30 = HEAP32[$29>>2]|0;
  _sqlite3DbFree($27,$30);
  $31 = HEAP32[$i>>2]|0;
  $32 = (($31) + 1)|0;
  HEAP32[$i>>2] = $32;
  $33 = HEAP32[$pCol>>2]|0;
  $34 = (($33) + 24|0);
  HEAP32[$pCol>>2] = $34;
 }
 $35 = HEAP32[$1>>2]|0;
 $36 = (($35) + 4|0);
 $37 = HEAP32[$36>>2]|0;
 _sqlite3DbFree($12,$37);
 STACKTOP = sp;return;
}
function _sqlite3ResolveSelfReference($pParse,$pTab,$type,$pExpr,$pList) {
 $pParse = $pParse|0;
 $pTab = $pTab|0;
 $type = $type|0;
 $pExpr = $pExpr|0;
 $pList = $pList|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $i = 0, $or$cond = 0;
 var $sNC = 0, $sSrc = 0, dest = 0, label = 0, sp = 0, stop = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 144|0;
 $0 = sp + 136|0;
 $1 = sp + 132|0;
 $2 = sp + 128|0;
 $3 = sp + 124|0;
 $4 = sp + 120|0;
 $sSrc = sp;
 $sNC = sp + 88|0;
 $i = sp + 80|0;
 HEAP32[$0>>2] = $pParse;
 HEAP32[$1>>2] = $pTab;
 HEAP32[$2>>2] = $type;
 HEAP32[$3>>2] = $pExpr;
 HEAP32[$4>>2] = $pList;
 ;HEAP32[$sNC+0>>2]=0|0;HEAP32[$sNC+4>>2]=0|0;HEAP32[$sNC+8>>2]=0|0;HEAP32[$sNC+12>>2]=0|0;HEAP32[$sNC+16>>2]=0|0;HEAP32[$sNC+20>>2]=0|0;HEAP32[$sNC+24>>2]=0|0;HEAP32[$sNC+28>>2]=0|0;
 dest=$sSrc+0|0; stop=dest+80|0; do { HEAP32[dest>>2]=0|0; dest=dest+4|0; } while ((dest|0) < (stop|0));
 HEAP32[$sSrc>>2] = 1;
 $5 = HEAP32[$1>>2]|0;
 $6 = HEAP32[$5>>2]|0;
 $7 = (($sSrc) + 8|0);
 $8 = (($7) + 8|0);
 HEAP32[$8>>2] = $6;
 $9 = HEAP32[$1>>2]|0;
 $10 = (($sSrc) + 8|0);
 $11 = (($10) + 16|0);
 HEAP32[$11>>2] = $9;
 $12 = (($sSrc) + 8|0);
 $13 = (($12) + 40|0);
 HEAP32[$13>>2] = -1;
 $14 = HEAP32[$0>>2]|0;
 HEAP32[$sNC>>2] = $14;
 $15 = (($sNC) + 4|0);
 HEAP32[$15>>2] = $sSrc;
 $16 = HEAP32[$2>>2]|0;
 $17 = $16&65535;
 $18 = (($sNC) + 28|0);
 HEAP16[$18>>1] = $17;
 $19 = HEAP32[$3>>2]|0;
 $20 = (_sqlite3ResolveExprNames($sNC,$19)|0);
 $21 = ($20|0)==(0);
 $22 = HEAP32[$4>>2]|0;
 $23 = ($22|0)!=(0|0);
 $or$cond = $21 & $23;
 if (!($or$cond)) {
  STACKTOP = sp;return;
 }
 HEAP32[$i>>2] = 0;
 while(1) {
  $24 = HEAP32[$i>>2]|0;
  $25 = HEAP32[$4>>2]|0;
  $26 = HEAP32[$25>>2]|0;
  $27 = ($24|0)<($26|0);
  if (!($27)) {
   label = 6;
   break;
  }
  $28 = HEAP32[$i>>2]|0;
  $29 = HEAP32[$4>>2]|0;
  $30 = (($29) + 4|0);
  $31 = HEAP32[$30>>2]|0;
  $32 = (($31) + (($28*20)|0)|0);
  $33 = HEAP32[$32>>2]|0;
  $34 = (_sqlite3ResolveExprNames($sNC,$33)|0);
  $35 = ($34|0)!=(0);
  if ($35) {
   label = 6;
   break;
  }
  $36 = HEAP32[$i>>2]|0;
  $37 = (($36) + 1)|0;
  HEAP32[$i>>2] = $37;
 }
 if ((label|0) == 6) {
  STACKTOP = sp;return;
 }
}
function _hasColumn($aiCol,$nCol,$x) {
 $aiCol = $aiCol|0;
 $nCol = $nCol|0;
 $x = $x|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $1 = sp + 8|0;
 $2 = sp + 4|0;
 $3 = sp;
 HEAP32[$1>>2] = $aiCol;
 HEAP32[$2>>2] = $nCol;
 HEAP32[$3>>2] = $x;
 while(1) {
  $4 = HEAP32[$2>>2]|0;
  $5 = (($4) + -1)|0;
  HEAP32[$2>>2] = $5;
  $6 = ($4|0)>(0);
  if (!($6)) {
   label = 5;
   break;
  }
  $7 = HEAP32[$3>>2]|0;
  $8 = HEAP32[$1>>2]|0;
  $9 = (($8) + 2|0);
  HEAP32[$1>>2] = $9;
  $10 = HEAP16[$8>>1]|0;
  $11 = $10 << 16 >> 16;
  $12 = ($7|0)==($11|0);
  if ($12) {
   label = 4;
   break;
  }
 }
 if ((label|0) == 4) {
  HEAP32[$0>>2] = 1;
  $13 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($13|0);
 }
 else if ((label|0) == 5) {
  HEAP32[$0>>2] = 0;
  $13 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($13|0);
 }
 return 0|0;
}
function _estimateIndexWidth($pIdx) {
 $pIdx = $pIdx|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $5 = 0, $6 = 0;
 var $7 = 0, $8 = 0, $9 = 0, $aCol = 0, $i = 0, $wIndex = 0, $x = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 12|0;
 $wIndex = sp + 8|0;
 $i = sp + 4|0;
 $aCol = sp;
 $x = sp + 16|0;
 HEAP32[$0>>2] = $pIdx;
 HEAP32[$wIndex>>2] = 0;
 $1 = HEAP32[$0>>2]|0;
 $2 = (($1) + 12|0);
 $3 = HEAP32[$2>>2]|0;
 $4 = (($3) + 4|0);
 $5 = HEAP32[$4>>2]|0;
 HEAP32[$aCol>>2] = $5;
 HEAP32[$i>>2] = 0;
 while(1) {
  $6 = HEAP32[$i>>2]|0;
  $7 = HEAP32[$0>>2]|0;
  $8 = (($7) + 52|0);
  $9 = HEAP16[$8>>1]|0;
  $10 = $9&65535;
  $11 = ($6|0)<($10|0);
  if (!($11)) {
   break;
  }
  $12 = HEAP32[$i>>2]|0;
  $13 = HEAP32[$0>>2]|0;
  $14 = (($13) + 4|0);
  $15 = HEAP32[$14>>2]|0;
  $16 = (($15) + ($12<<1)|0);
  $17 = HEAP16[$16>>1]|0;
  HEAP16[$x>>1] = $17;
  $18 = HEAP16[$x>>1]|0;
  $19 = $18 << 16 >> 16;
  $20 = ($19|0)<(0);
  if ($20) {
   $35 = 1;
  } else {
   $21 = HEAP32[$i>>2]|0;
   $22 = HEAP32[$0>>2]|0;
   $23 = (($22) + 4|0);
   $24 = HEAP32[$23>>2]|0;
   $25 = (($24) + ($21<<1)|0);
   $26 = HEAP16[$25>>1]|0;
   $27 = $26 << 16 >> 16;
   $28 = HEAP32[$aCol>>2]|0;
   $29 = (($28) + (($27*24)|0)|0);
   $30 = (($29) + 22|0);
   $31 = HEAP8[$30>>0]|0;
   $32 = $31&255;
   $35 = $32;
  }
  $33 = HEAP32[$wIndex>>2]|0;
  $34 = (($33) + ($35))|0;
  HEAP32[$wIndex>>2] = $34;
  $36 = HEAP32[$i>>2]|0;
  $37 = (($36) + 1)|0;
  HEAP32[$i>>2] = $37;
 }
 $38 = HEAP32[$wIndex>>2]|0;
 $39 = $38<<2;
 $40 = (_sqlite3LogEst($39,0)|0);
 $41 = HEAP32[$0>>2]|0;
 $42 = (($41) + 48|0);
 HEAP16[$42>>1] = $40;
 STACKTOP = sp;return;
}
function _freeIndex($db,$p) {
 $db = $db|0;
 $p = $p|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $1 = sp;
 HEAP32[$0>>2] = $db;
 HEAP32[$1>>2] = $p;
 $2 = HEAP32[$0>>2]|0;
 $3 = HEAP32[$1>>2]|0;
 _sqlite3DeleteIndexSamples($2,$3);
 $4 = HEAP32[$0>>2]|0;
 $5 = ($4|0)==(0|0);
 if ($5) {
  label = 3;
 } else {
  $6 = HEAP32[$0>>2]|0;
  $7 = (($6) + 520|0);
  $8 = HEAP32[$7>>2]|0;
  $9 = ($8|0)==(0|0);
  if ($9) {
   label = 3;
  }
 }
 if ((label|0) == 3) {
  $10 = HEAP32[$1>>2]|0;
  $11 = (($10) + 40|0);
  $12 = HEAP32[$11>>2]|0;
  _sqlite3KeyInfoUnref($12);
 }
 $13 = HEAP32[$0>>2]|0;
 $14 = HEAP32[$1>>2]|0;
 $15 = (($14) + 36|0);
 $16 = HEAP32[$15>>2]|0;
 _sqlite3ExprDelete($13,$16);
 $17 = HEAP32[$0>>2]|0;
 $18 = HEAP32[$1>>2]|0;
 $19 = (($18) + 16|0);
 $20 = HEAP32[$19>>2]|0;
 _sqlite3DbFree($17,$20);
 $21 = HEAP32[$1>>2]|0;
 $22 = (($21) + 55|0);
 $23 = HEAP8[$22>>0]|0;
 $24 = ($23&255) >>> 4;
 $25 = $24 & 1;
 $26 = $25&255;
 $27 = ($26|0)!=(0);
 if (!($27)) {
  $32 = HEAP32[$0>>2]|0;
  $33 = HEAP32[$1>>2]|0;
  _sqlite3DbFree($32,$33);
  STACKTOP = sp;return;
 }
 $28 = HEAP32[$0>>2]|0;
 $29 = HEAP32[$1>>2]|0;
 $30 = (($29) + 32|0);
 $31 = HEAP32[$30>>2]|0;
 _sqlite3DbFree($28,$31);
 $32 = HEAP32[$0>>2]|0;
 $33 = HEAP32[$1>>2]|0;
 _sqlite3DbFree($32,$33);
 STACKTOP = sp;return;
}
function _sqlite3DeleteIndexSamples($db,$pIdx) {
 $db = $db|0;
 $pIdx = $pIdx|0;
 var $0 = 0, $1 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $1 = sp;
 HEAP32[$0>>2] = $db;
 HEAP32[$1>>2] = $pIdx;
 STACKTOP = sp;return;
}
function _sqlite3ExprIsConstantOrFunction($p,$isInit) {
 $p = $p|0;
 $isInit = $isInit|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 $1 = sp + 4|0;
 HEAP32[$0>>2] = $p;
 HEAP8[$1>>0] = $isInit;
 $2 = HEAP32[$0>>2]|0;
 $3 = HEAP8[$1>>0]|0;
 $4 = $3&255;
 $5 = (3 + ($4))|0;
 $6 = (_exprIsConst($2,$5)|0);
 STACKTOP = sp;return ($6|0);
}
function _convertToWithoutRowidTable($pParse,$pTab) {
 $pParse = $pParse|0;
 $pTab = $pTab|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0;
 var $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0, $188 = 0;
 var $189 = 0, $19 = 0, $190 = 0, $191 = 0, $192 = 0, $193 = 0, $194 = 0, $195 = 0, $196 = 0, $197 = 0, $198 = 0, $199 = 0, $2 = 0, $20 = 0, $200 = 0, $201 = 0, $202 = 0, $203 = 0, $204 = 0, $205 = 0;
 var $206 = 0, $207 = 0, $208 = 0, $209 = 0, $21 = 0, $210 = 0, $211 = 0, $212 = 0, $213 = 0, $214 = 0, $215 = 0, $216 = 0, $217 = 0, $218 = 0, $219 = 0, $22 = 0, $220 = 0, $221 = 0, $222 = 0, $223 = 0;
 var $224 = 0, $225 = 0, $226 = 0, $227 = 0, $228 = 0, $229 = 0, $23 = 0, $230 = 0, $231 = 0, $232 = 0, $233 = 0, $234 = 0, $235 = 0, $236 = 0, $237 = 0, $238 = 0, $239 = 0, $24 = 0, $240 = 0, $241 = 0;
 var $242 = 0, $243 = 0, $244 = 0, $245 = 0, $246 = 0, $247 = 0, $248 = 0, $249 = 0, $25 = 0, $250 = 0, $251 = 0, $252 = 0, $253 = 0, $254 = 0, $255 = 0, $256 = 0, $257 = 0, $258 = 0, $259 = 0, $26 = 0;
 var $260 = 0, $261 = 0, $262 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0;
 var $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0;
 var $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0;
 var $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0;
 var $97 = 0, $98 = 0, $99 = 0, $db = 0, $i = 0, $j = 0, $n = 0, $nPk = 0, $pIdx = 0, $pList = 0, $pPk = 0, $v = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $0 = sp + 40|0;
 $1 = sp + 36|0;
 $pIdx = sp + 32|0;
 $pPk = sp + 28|0;
 $nPk = sp + 24|0;
 $i = sp + 20|0;
 $j = sp + 16|0;
 $db = sp + 12|0;
 $v = sp + 8|0;
 $pList = sp + 4|0;
 $n = sp;
 HEAP32[$0>>2] = $pParse;
 HEAP32[$1>>2] = $pTab;
 $2 = HEAP32[$0>>2]|0;
 $3 = HEAP32[$2>>2]|0;
 HEAP32[$db>>2] = $3;
 $4 = HEAP32[$0>>2]|0;
 $5 = (($4) + 8|0);
 $6 = HEAP32[$5>>2]|0;
 HEAP32[$v>>2] = $6;
 $7 = HEAP32[$0>>2]|0;
 $8 = (($7) + 416|0);
 $9 = HEAP32[$8>>2]|0;
 $10 = ($9|0)!=(0);
 if ($10) {
  $11 = HEAP32[$v>>2]|0;
  $12 = HEAP32[$0>>2]|0;
  $13 = (($12) + 416|0);
  $14 = HEAP32[$13>>2]|0;
  $15 = (_sqlite3VdbeGetOp($11,$14)|0);
  HEAP8[$15>>0] = 120;
 }
 $16 = HEAP32[$0>>2]|0;
 $17 = (($16) + 420|0);
 $18 = HEAP32[$17>>2]|0;
 $19 = ($18|0)!=(0);
 if ($19) {
  $20 = HEAP32[$v>>2]|0;
  $21 = HEAP32[$0>>2]|0;
  $22 = (($21) + 420|0);
  $23 = HEAP32[$22>>2]|0;
  $24 = (_sqlite3VdbeGetOp($20,$23)|0);
  HEAP8[$24>>0] = 16;
 }
 $25 = HEAP32[$1>>2]|0;
 $26 = (($25) + 36|0);
 $27 = HEAP16[$26>>1]|0;
 $28 = $27 << 16 >> 16;
 $29 = ($28|0)>=(0);
 do {
  if ($29) {
   $30 = HEAP32[$0>>2]|0;
   $31 = (_sqlite3ExprListAppend($30,0,0)|0);
   HEAP32[$pList>>2] = $31;
   $32 = HEAP32[$pList>>2]|0;
   $33 = ($32|0)==(0|0);
   if ($33) {
    STACKTOP = sp;return;
   }
   $34 = HEAP32[$0>>2]|0;
   $35 = HEAP32[$34>>2]|0;
   $36 = HEAP32[$1>>2]|0;
   $37 = (($36) + 36|0);
   $38 = HEAP16[$37>>1]|0;
   $39 = $38 << 16 >> 16;
   $40 = HEAP32[$1>>2]|0;
   $41 = (($40) + 4|0);
   $42 = HEAP32[$41>>2]|0;
   $43 = (($42) + (($39*24)|0)|0);
   $44 = HEAP32[$43>>2]|0;
   $45 = (_sqlite3DbStrDup($35,$44)|0);
   $46 = HEAP32[$pList>>2]|0;
   $47 = (($46) + 4|0);
   $48 = HEAP32[$47>>2]|0;
   $49 = (($48) + 4|0);
   HEAP32[$49>>2] = $45;
   $50 = HEAP32[$0>>2]|0;
   $51 = (($50) + 448|0);
   $52 = HEAP8[$51>>0]|0;
   $53 = HEAP32[$pList>>2]|0;
   $54 = (($53) + 4|0);
   $55 = HEAP32[$54>>2]|0;
   $56 = (($55) + 12|0);
   HEAP8[$56>>0] = $52;
   $57 = HEAP32[$0>>2]|0;
   $58 = HEAP32[$pList>>2]|0;
   $59 = HEAP32[$1>>2]|0;
   $60 = (($59) + 45|0);
   $61 = HEAP8[$60>>0]|0;
   $62 = $61&255;
   $63 = (_sqlite3CreateIndex($57,0,0,0,$58,$62,0,0,0,0)|0);
   HEAP32[$pPk>>2] = $63;
   $64 = HEAP32[$pPk>>2]|0;
   $65 = ($64|0)==(0|0);
   if ($65) {
    STACKTOP = sp;return;
   } else {
    $66 = HEAP32[$pPk>>2]|0;
    $67 = (($66) + 55|0);
    $68 = HEAP8[$67>>0]|0;
    $69 = $68 & -4;
    $70 = $69 | 2;
    HEAP8[$67>>0] = $70;
    $71 = HEAP32[$1>>2]|0;
    $72 = (($71) + 36|0);
    HEAP16[$72>>1] = -1;
    break;
   }
  } else {
   $73 = HEAP32[$1>>2]|0;
   $74 = (_sqlite3PrimaryKeyIndex($73)|0);
   HEAP32[$pPk>>2] = $74;
  }
 } while(0);
 $75 = HEAP32[$pPk>>2]|0;
 $76 = (($75) + 55|0);
 $77 = HEAP8[$76>>0]|0;
 $78 = $77 & -33;
 $79 = $78 | 32;
 HEAP8[$76>>0] = $79;
 $80 = HEAP32[$pPk>>2]|0;
 $81 = (($80) + 50|0);
 $82 = HEAP16[$81>>1]|0;
 $83 = $82&65535;
 HEAP32[$nPk>>2] = $83;
 HEAP32[$i>>2] = 0;
 while(1) {
  $84 = HEAP32[$i>>2]|0;
  $85 = HEAP32[$nPk>>2]|0;
  $86 = ($84|0)<($85|0);
  if (!($86)) {
   break;
  }
  $87 = HEAP32[$i>>2]|0;
  $88 = HEAP32[$pPk>>2]|0;
  $89 = (($88) + 4|0);
  $90 = HEAP32[$89>>2]|0;
  $91 = (($90) + ($87<<1)|0);
  $92 = HEAP16[$91>>1]|0;
  $93 = $92 << 16 >> 16;
  $94 = HEAP32[$1>>2]|0;
  $95 = (($94) + 4|0);
  $96 = HEAP32[$95>>2]|0;
  $97 = (($96) + (($93*24)|0)|0);
  $98 = (($97) + 20|0);
  HEAP8[$98>>0] = 1;
  $99 = HEAP32[$i>>2]|0;
  $100 = (($99) + 1)|0;
  HEAP32[$i>>2] = $100;
 }
 $101 = HEAP32[$pPk>>2]|0;
 $102 = (($101) + 55|0);
 $103 = HEAP8[$102>>0]|0;
 $104 = $103 & -9;
 $105 = $104 | 8;
 HEAP8[$102>>0] = $105;
 $106 = HEAP32[$1>>2]|0;
 $107 = (($106) + 32|0);
 $108 = HEAP32[$107>>2]|0;
 $109 = HEAP32[$pPk>>2]|0;
 $110 = (($109) + 44|0);
 HEAP32[$110>>2] = $108;
 $111 = HEAP32[$1>>2]|0;
 $112 = (($111) + 8|0);
 $113 = HEAP32[$112>>2]|0;
 HEAP32[$pIdx>>2] = $113;
 L21: while(1) {
  $114 = HEAP32[$pIdx>>2]|0;
  $115 = ($114|0)!=(0|0);
  if (!($115)) {
   break;
  }
  $116 = HEAP32[$pIdx>>2]|0;
  $117 = (($116) + 55|0);
  $118 = HEAP8[$117>>0]|0;
  $119 = $118 & 3;
  $120 = $119&255;
  $121 = ($120|0)==(2);
  L24: do {
   if (!($121)) {
    HEAP32[$n>>2] = 0;
    HEAP32[$i>>2] = 0;
    while(1) {
     $122 = HEAP32[$i>>2]|0;
     $123 = HEAP32[$nPk>>2]|0;
     $124 = ($122|0)<($123|0);
     if (!($124)) {
      break;
     }
     $125 = HEAP32[$pIdx>>2]|0;
     $126 = (($125) + 4|0);
     $127 = HEAP32[$126>>2]|0;
     $128 = HEAP32[$pIdx>>2]|0;
     $129 = (($128) + 50|0);
     $130 = HEAP16[$129>>1]|0;
     $131 = $130&65535;
     $132 = HEAP32[$i>>2]|0;
     $133 = HEAP32[$pPk>>2]|0;
     $134 = (($133) + 4|0);
     $135 = HEAP32[$134>>2]|0;
     $136 = (($135) + ($132<<1)|0);
     $137 = HEAP16[$136>>1]|0;
     $138 = $137 << 16 >> 16;
     $139 = (_hasColumn($127,$131,$138)|0);
     $140 = ($139|0)!=(0);
     if (!($140)) {
      $141 = HEAP32[$n>>2]|0;
      $142 = (($141) + 1)|0;
      HEAP32[$n>>2] = $142;
     }
     $143 = HEAP32[$i>>2]|0;
     $144 = (($143) + 1)|0;
     HEAP32[$i>>2] = $144;
    }
    $145 = HEAP32[$n>>2]|0;
    $146 = ($145|0)==(0);
    if ($146) {
     $147 = HEAP32[$pIdx>>2]|0;
     $148 = (($147) + 50|0);
     $149 = HEAP16[$148>>1]|0;
     $150 = HEAP32[$pIdx>>2]|0;
     $151 = (($150) + 52|0);
     HEAP16[$151>>1] = $149;
     break;
    }
    $152 = HEAP32[$db>>2]|0;
    $153 = HEAP32[$pIdx>>2]|0;
    $154 = HEAP32[$pIdx>>2]|0;
    $155 = (($154) + 50|0);
    $156 = HEAP16[$155>>1]|0;
    $157 = $156&65535;
    $158 = HEAP32[$n>>2]|0;
    $159 = (($157) + ($158))|0;
    $160 = (_resizeIndexObject($152,$153,$159)|0);
    $161 = ($160|0)!=(0);
    if ($161) {
     label = 38;
     break L21;
    }
    HEAP32[$i>>2] = 0;
    $162 = HEAP32[$pIdx>>2]|0;
    $163 = (($162) + 50|0);
    $164 = HEAP16[$163>>1]|0;
    $165 = $164&65535;
    HEAP32[$j>>2] = $165;
    while(1) {
     $166 = HEAP32[$i>>2]|0;
     $167 = HEAP32[$nPk>>2]|0;
     $168 = ($166|0)<($167|0);
     if (!($168)) {
      break L24;
     }
     $169 = HEAP32[$pIdx>>2]|0;
     $170 = (($169) + 4|0);
     $171 = HEAP32[$170>>2]|0;
     $172 = HEAP32[$pIdx>>2]|0;
     $173 = (($172) + 50|0);
     $174 = HEAP16[$173>>1]|0;
     $175 = $174&65535;
     $176 = HEAP32[$i>>2]|0;
     $177 = HEAP32[$pPk>>2]|0;
     $178 = (($177) + 4|0);
     $179 = HEAP32[$178>>2]|0;
     $180 = (($179) + ($176<<1)|0);
     $181 = HEAP16[$180>>1]|0;
     $182 = $181 << 16 >> 16;
     $183 = (_hasColumn($171,$175,$182)|0);
     $184 = ($183|0)!=(0);
     if (!($184)) {
      $185 = HEAP32[$i>>2]|0;
      $186 = HEAP32[$pPk>>2]|0;
      $187 = (($186) + 4|0);
      $188 = HEAP32[$187>>2]|0;
      $189 = (($188) + ($185<<1)|0);
      $190 = HEAP16[$189>>1]|0;
      $191 = HEAP32[$j>>2]|0;
      $192 = HEAP32[$pIdx>>2]|0;
      $193 = (($192) + 4|0);
      $194 = HEAP32[$193>>2]|0;
      $195 = (($194) + ($191<<1)|0);
      HEAP16[$195>>1] = $190;
      $196 = HEAP32[$i>>2]|0;
      $197 = HEAP32[$pPk>>2]|0;
      $198 = (($197) + 32|0);
      $199 = HEAP32[$198>>2]|0;
      $200 = (($199) + ($196<<2)|0);
      $201 = HEAP32[$200>>2]|0;
      $202 = HEAP32[$j>>2]|0;
      $203 = HEAP32[$pIdx>>2]|0;
      $204 = (($203) + 32|0);
      $205 = HEAP32[$204>>2]|0;
      $206 = (($205) + ($202<<2)|0);
      HEAP32[$206>>2] = $201;
      $207 = HEAP32[$j>>2]|0;
      $208 = (($207) + 1)|0;
      HEAP32[$j>>2] = $208;
     }
     $209 = HEAP32[$i>>2]|0;
     $210 = (($209) + 1)|0;
     HEAP32[$i>>2] = $210;
    }
   }
  } while(0);
  $211 = HEAP32[$pIdx>>2]|0;
  $212 = (($211) + 20|0);
  $213 = HEAP32[$212>>2]|0;
  HEAP32[$pIdx>>2] = $213;
 }
 if ((label|0) == 38) {
  STACKTOP = sp;return;
 }
 $214 = HEAP32[$nPk>>2]|0;
 $215 = HEAP32[$1>>2]|0;
 $216 = (($215) + 38|0);
 $217 = HEAP16[$216>>1]|0;
 $218 = $217 << 16 >> 16;
 $219 = ($214|0)<($218|0);
 if (!($219)) {
  $258 = HEAP32[$1>>2]|0;
  $259 = (($258) + 38|0);
  $260 = HEAP16[$259>>1]|0;
  $261 = HEAP32[$pPk>>2]|0;
  $262 = (($261) + 52|0);
  HEAP16[$262>>1] = $260;
  STACKTOP = sp;return;
 }
 $220 = HEAP32[$db>>2]|0;
 $221 = HEAP32[$pPk>>2]|0;
 $222 = HEAP32[$1>>2]|0;
 $223 = (($222) + 38|0);
 $224 = HEAP16[$223>>1]|0;
 $225 = $224 << 16 >> 16;
 $226 = (_resizeIndexObject($220,$221,$225)|0);
 $227 = ($226|0)!=(0);
 if ($227) {
  STACKTOP = sp;return;
 }
 HEAP32[$i>>2] = 0;
 $228 = HEAP32[$nPk>>2]|0;
 HEAP32[$j>>2] = $228;
 while(1) {
  $229 = HEAP32[$i>>2]|0;
  $230 = HEAP32[$1>>2]|0;
  $231 = (($230) + 38|0);
  $232 = HEAP16[$231>>1]|0;
  $233 = $232 << 16 >> 16;
  $234 = ($229|0)<($233|0);
  if (!($234)) {
   break;
  }
  $235 = HEAP32[$pPk>>2]|0;
  $236 = (($235) + 4|0);
  $237 = HEAP32[$236>>2]|0;
  $238 = HEAP32[$j>>2]|0;
  $239 = HEAP32[$i>>2]|0;
  $240 = (_hasColumn($237,$238,$239)|0);
  $241 = ($240|0)!=(0);
  if (!($241)) {
   $242 = HEAP32[$i>>2]|0;
   $243 = $242&65535;
   $244 = HEAP32[$j>>2]|0;
   $245 = HEAP32[$pPk>>2]|0;
   $246 = (($245) + 4|0);
   $247 = HEAP32[$246>>2]|0;
   $248 = (($247) + ($244<<1)|0);
   HEAP16[$248>>1] = $243;
   $249 = HEAP32[$j>>2]|0;
   $250 = HEAP32[$pPk>>2]|0;
   $251 = (($250) + 32|0);
   $252 = HEAP32[$251>>2]|0;
   $253 = (($252) + ($249<<2)|0);
   HEAP32[$253>>2] = 3320;
   $254 = HEAP32[$j>>2]|0;
   $255 = (($254) + 1)|0;
   HEAP32[$j>>2] = $255;
  }
  $256 = HEAP32[$i>>2]|0;
  $257 = (($256) + 1)|0;
  HEAP32[$i>>2] = $257;
 }
 STACKTOP = sp;return;
}
function _estimateTableWidth($pTab) {
 $pTab = $pTab|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $i = 0, $pTabCol = 0, $wTable = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $wTable = sp + 8|0;
 $pTabCol = sp + 4|0;
 $i = sp;
 HEAP32[$0>>2] = $pTab;
 HEAP32[$wTable>>2] = 0;
 $1 = HEAP32[$0>>2]|0;
 $2 = (($1) + 38|0);
 $3 = HEAP16[$2>>1]|0;
 $4 = $3 << 16 >> 16;
 HEAP32[$i>>2] = $4;
 $5 = HEAP32[$0>>2]|0;
 $6 = (($5) + 4|0);
 $7 = HEAP32[$6>>2]|0;
 HEAP32[$pTabCol>>2] = $7;
 while(1) {
  $8 = HEAP32[$i>>2]|0;
  $9 = ($8|0)>(0);
  if (!($9)) {
   break;
  }
  $10 = HEAP32[$pTabCol>>2]|0;
  $11 = (($10) + 22|0);
  $12 = HEAP8[$11>>0]|0;
  $13 = $12&255;
  $14 = HEAP32[$wTable>>2]|0;
  $15 = (($14) + ($13))|0;
  HEAP32[$wTable>>2] = $15;
  $16 = HEAP32[$i>>2]|0;
  $17 = (($16) + -1)|0;
  HEAP32[$i>>2] = $17;
  $18 = HEAP32[$pTabCol>>2]|0;
  $19 = (($18) + 24|0);
  HEAP32[$pTabCol>>2] = $19;
 }
 $20 = HEAP32[$0>>2]|0;
 $21 = (($20) + 36|0);
 $22 = HEAP16[$21>>1]|0;
 $23 = $22 << 16 >> 16;
 $24 = ($23|0)<(0);
 if (!($24)) {
  $27 = HEAP32[$wTable>>2]|0;
  $28 = $27<<2;
  $29 = (_sqlite3LogEst($28,0)|0);
  $30 = HEAP32[$0>>2]|0;
  $31 = (($30) + 42|0);
  HEAP16[$31>>1] = $29;
  STACKTOP = sp;return;
 }
 $25 = HEAP32[$wTable>>2]|0;
 $26 = (($25) + 1)|0;
 HEAP32[$wTable>>2] = $26;
 $27 = HEAP32[$wTable>>2]|0;
 $28 = $27<<2;
 $29 = (_sqlite3LogEst($28,0)|0);
 $30 = HEAP32[$0>>2]|0;
 $31 = (($30) + 42|0);
 HEAP16[$31>>1] = $29;
 STACKTOP = sp;return;
}
function _createTableStmt($db,$p) {
 $db = $db|0;
 $p = $p|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0;
 var $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0;
 var $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0;
 var $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0;
 var $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0;
 var $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $i = 0, $k = 0, $len = 0, $n = 0, $pCol = 0, $vararg_buffer3 = 0, $zEnd = 0, $zSep = 0, $zSep2 = 0, $zStmt = 0, $zType = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 64|0;
 $vararg_buffer3 = sp;
 $0 = sp + 52|0;
 $1 = sp + 48|0;
 $2 = sp + 44|0;
 $i = sp + 40|0;
 $k = sp + 36|0;
 $n = sp + 32|0;
 $zStmt = sp + 28|0;
 $zSep = sp + 24|0;
 $zSep2 = sp + 20|0;
 $zEnd = sp + 16|0;
 $pCol = sp + 12|0;
 $len = sp + 8|0;
 $zType = sp + 4|0;
 HEAP32[$1>>2] = $db;
 HEAP32[$2>>2] = $p;
 HEAP32[$n>>2] = 0;
 $3 = HEAP32[$2>>2]|0;
 $4 = (($3) + 4|0);
 $5 = HEAP32[$4>>2]|0;
 HEAP32[$pCol>>2] = $5;
 HEAP32[$i>>2] = 0;
 while(1) {
  $6 = HEAP32[$i>>2]|0;
  $7 = HEAP32[$2>>2]|0;
  $8 = (($7) + 38|0);
  $9 = HEAP16[$8>>1]|0;
  $10 = $9 << 16 >> 16;
  $11 = ($6|0)<($10|0);
  if (!($11)) {
   break;
  }
  $12 = HEAP32[$pCol>>2]|0;
  $13 = HEAP32[$12>>2]|0;
  $14 = (_identLength($13)|0);
  $15 = (($14) + 5)|0;
  $16 = HEAP32[$n>>2]|0;
  $17 = (($16) + ($15))|0;
  HEAP32[$n>>2] = $17;
  $18 = HEAP32[$i>>2]|0;
  $19 = (($18) + 1)|0;
  HEAP32[$i>>2] = $19;
  $20 = HEAP32[$pCol>>2]|0;
  $21 = (($20) + 24|0);
  HEAP32[$pCol>>2] = $21;
 }
 $22 = HEAP32[$2>>2]|0;
 $23 = HEAP32[$22>>2]|0;
 $24 = (_identLength($23)|0);
 $25 = HEAP32[$n>>2]|0;
 $26 = (($25) + ($24))|0;
 HEAP32[$n>>2] = $26;
 $27 = HEAP32[$n>>2]|0;
 $28 = ($27|0)<(50);
 if ($28) {
  HEAP32[$zSep>>2] = 1216;
  HEAP32[$zSep2>>2] = 9776;
  HEAP32[$zEnd>>2] = 23976;
 } else {
  HEAP32[$zSep>>2] = 26904;
  HEAP32[$zSep2>>2] = 26912;
  HEAP32[$zEnd>>2] = 26920;
 }
 $29 = HEAP32[$2>>2]|0;
 $30 = (($29) + 38|0);
 $31 = HEAP16[$30>>1]|0;
 $32 = $31 << 16 >> 16;
 $33 = ($32*6)|0;
 $34 = (35 + ($33))|0;
 $35 = HEAP32[$n>>2]|0;
 $36 = (($35) + ($34))|0;
 HEAP32[$n>>2] = $36;
 $37 = HEAP32[$n>>2]|0;
 $38 = ($37|0)<(0);
 $39 = $38 << 31 >> 31;
 $40 = (_sqlite3DbMallocRaw(0,$37,$39)|0);
 HEAP32[$zStmt>>2] = $40;
 $41 = HEAP32[$zStmt>>2]|0;
 $42 = ($41|0)==(0|0);
 if ($42) {
  $43 = HEAP32[$1>>2]|0;
  $44 = (($43) + 64|0);
  HEAP8[$44>>0] = 1;
  HEAP32[$0>>2] = 0;
  $105 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($105|0);
 }
 $45 = HEAP32[$n>>2]|0;
 $46 = HEAP32[$zStmt>>2]|0;
 (_sqlite3_snprintf($45,$46,26928,$vararg_buffer3)|0);
 $47 = HEAP32[$zStmt>>2]|0;
 $48 = (_sqlite3Strlen30($47)|0);
 HEAP32[$k>>2] = $48;
 $49 = HEAP32[$zStmt>>2]|0;
 $50 = HEAP32[$2>>2]|0;
 $51 = HEAP32[$50>>2]|0;
 _identPut($49,$k,$51);
 $52 = HEAP32[$k>>2]|0;
 $53 = (($52) + 1)|0;
 HEAP32[$k>>2] = $53;
 $54 = HEAP32[$zStmt>>2]|0;
 $55 = (($54) + ($52)|0);
 HEAP8[$55>>0] = 40;
 $56 = HEAP32[$2>>2]|0;
 $57 = (($56) + 4|0);
 $58 = HEAP32[$57>>2]|0;
 HEAP32[$pCol>>2] = $58;
 HEAP32[$i>>2] = 0;
 while(1) {
  $59 = HEAP32[$i>>2]|0;
  $60 = HEAP32[$2>>2]|0;
  $61 = (($60) + 38|0);
  $62 = HEAP16[$61>>1]|0;
  $63 = $62 << 16 >> 16;
  $64 = ($59|0)<($63|0);
  $65 = HEAP32[$n>>2]|0;
  $66 = HEAP32[$k>>2]|0;
  $67 = (($65) - ($66))|0;
  $68 = HEAP32[$k>>2]|0;
  $69 = HEAP32[$zStmt>>2]|0;
  $70 = (($69) + ($68)|0);
  if (!($64)) {
   break;
  }
  $71 = HEAP32[$zSep>>2]|0;
  (_sqlite3_snprintf($67,$70,$71,$vararg_buffer3)|0);
  $72 = HEAP32[$k>>2]|0;
  $73 = HEAP32[$zStmt>>2]|0;
  $74 = (($73) + ($72)|0);
  $75 = (_sqlite3Strlen30($74)|0);
  $76 = HEAP32[$k>>2]|0;
  $77 = (($76) + ($75))|0;
  HEAP32[$k>>2] = $77;
  $78 = HEAP32[$zSep2>>2]|0;
  HEAP32[$zSep>>2] = $78;
  $79 = HEAP32[$zStmt>>2]|0;
  $80 = HEAP32[$pCol>>2]|0;
  $81 = HEAP32[$80>>2]|0;
  _identPut($79,$k,$81);
  $82 = HEAP32[$pCol>>2]|0;
  $83 = (($82) + 21|0);
  $84 = HEAP8[$83>>0]|0;
  $85 = $84 << 24 >> 24;
  $86 = (($85) - 65)|0;
  $87 = (26944 + ($86<<2)|0);
  $88 = HEAP32[$87>>2]|0;
  HEAP32[$zType>>2] = $88;
  $89 = HEAP32[$zType>>2]|0;
  $90 = (_sqlite3Strlen30($89)|0);
  HEAP32[$len>>2] = $90;
  $91 = HEAP32[$k>>2]|0;
  $92 = HEAP32[$zStmt>>2]|0;
  $93 = (($92) + ($91)|0);
  $94 = HEAP32[$zType>>2]|0;
  $95 = HEAP32[$len>>2]|0;
  _memcpy(($93|0),($94|0),($95|0))|0;
  $96 = HEAP32[$len>>2]|0;
  $97 = HEAP32[$k>>2]|0;
  $98 = (($97) + ($96))|0;
  HEAP32[$k>>2] = $98;
  $99 = HEAP32[$i>>2]|0;
  $100 = (($99) + 1)|0;
  HEAP32[$i>>2] = $100;
  $101 = HEAP32[$pCol>>2]|0;
  $102 = (($101) + 24|0);
  HEAP32[$pCol>>2] = $102;
 }
 $103 = HEAP32[$zEnd>>2]|0;
 HEAP32[$vararg_buffer3>>2] = $103;
 (_sqlite3_snprintf($67,$70,1200,$vararg_buffer3)|0);
 $104 = HEAP32[$zStmt>>2]|0;
 HEAP32[$0>>2] = $104;
 $105 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($105|0);
}
function _identLength($z) {
 $z = $z|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $n = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $n = sp;
 HEAP32[$0>>2] = $z;
 HEAP32[$n>>2] = 0;
 while(1) {
  $1 = HEAP32[$0>>2]|0;
  $2 = HEAP8[$1>>0]|0;
  $3 = ($2<<24>>24)!=(0);
  if (!($3)) {
   break;
  }
  $4 = HEAP32[$0>>2]|0;
  $5 = HEAP8[$4>>0]|0;
  $6 = $5 << 24 >> 24;
  $7 = ($6|0)==(34);
  if ($7) {
   $8 = HEAP32[$n>>2]|0;
   $9 = (($8) + 1)|0;
   HEAP32[$n>>2] = $9;
  }
  $10 = HEAP32[$n>>2]|0;
  $11 = (($10) + 1)|0;
  HEAP32[$n>>2] = $11;
  $12 = HEAP32[$0>>2]|0;
  $13 = (($12) + 1|0);
  HEAP32[$0>>2] = $13;
 }
 $14 = HEAP32[$n>>2]|0;
 $15 = (($14) + 2)|0;
 STACKTOP = sp;return ($15|0);
}
function _identPut($z,$pIdx,$zSignedIdent) {
 $z = $z|0;
 $pIdx = $pIdx|0;
 $zSignedIdent = $zSignedIdent|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0;
 var $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $i = 0, $j = 0, $needQuote = 0, $zIdent = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 24|0;
 $1 = sp + 20|0;
 $2 = sp + 16|0;
 $zIdent = sp + 12|0;
 $i = sp + 8|0;
 $j = sp + 4|0;
 $needQuote = sp;
 HEAP32[$0>>2] = $z;
 HEAP32[$1>>2] = $pIdx;
 HEAP32[$2>>2] = $zSignedIdent;
 $3 = HEAP32[$2>>2]|0;
 HEAP32[$zIdent>>2] = $3;
 $4 = HEAP32[$1>>2]|0;
 $5 = HEAP32[$4>>2]|0;
 HEAP32[$i>>2] = $5;
 HEAP32[$j>>2] = 0;
 while(1) {
  $6 = HEAP32[$j>>2]|0;
  $7 = HEAP32[$zIdent>>2]|0;
  $8 = (($7) + ($6)|0);
  $9 = HEAP8[$8>>0]|0;
  $10 = ($9<<24>>24)!=(0);
  if (!($10)) {
   break;
  }
  $11 = HEAP32[$j>>2]|0;
  $12 = HEAP32[$zIdent>>2]|0;
  $13 = (($12) + ($11)|0);
  $14 = HEAP8[$13>>0]|0;
  $15 = $14&255;
  $16 = (1224 + ($15)|0);
  $17 = HEAP8[$16>>0]|0;
  $18 = $17&255;
  $19 = $18 & 6;
  $20 = ($19|0)!=(0);
  if (!($20)) {
   $21 = HEAP32[$j>>2]|0;
   $22 = HEAP32[$zIdent>>2]|0;
   $23 = (($22) + ($21)|0);
   $24 = HEAP8[$23>>0]|0;
   $25 = $24&255;
   $26 = ($25|0)!=(95);
   if ($26) {
    break;
   }
  }
  $27 = HEAP32[$j>>2]|0;
  $28 = (($27) + 1)|0;
  HEAP32[$j>>2] = $28;
 }
 $29 = HEAP32[$zIdent>>2]|0;
 $30 = HEAP8[$29>>0]|0;
 $31 = $30&255;
 $32 = (1224 + ($31)|0);
 $33 = HEAP8[$32>>0]|0;
 $34 = $33&255;
 $35 = $34 & 4;
 $36 = ($35|0)!=(0);
 if ($36) {
  $50 = 1;
 } else {
  $37 = HEAP32[$zIdent>>2]|0;
  $38 = HEAP32[$j>>2]|0;
  $39 = (_sqlite3KeywordCode($37,$38)|0);
  $40 = ($39|0)!=(27);
  if ($40) {
   $50 = 1;
  } else {
   $41 = HEAP32[$j>>2]|0;
   $42 = HEAP32[$zIdent>>2]|0;
   $43 = (($42) + ($41)|0);
   $44 = HEAP8[$43>>0]|0;
   $45 = $44&255;
   $46 = ($45|0)!=(0);
   if ($46) {
    $50 = 1;
   } else {
    $47 = HEAP32[$j>>2]|0;
    $48 = ($47|0)==(0);
    $50 = $48;
   }
  }
 }
 $49 = $50&1;
 HEAP32[$needQuote>>2] = $49;
 $51 = HEAP32[$needQuote>>2]|0;
 $52 = ($51|0)!=(0);
 if ($52) {
  $53 = HEAP32[$i>>2]|0;
  $54 = (($53) + 1)|0;
  HEAP32[$i>>2] = $54;
  $55 = HEAP32[$0>>2]|0;
  $56 = (($55) + ($53)|0);
  HEAP8[$56>>0] = 34;
 }
 HEAP32[$j>>2] = 0;
 while(1) {
  $57 = HEAP32[$j>>2]|0;
  $58 = HEAP32[$zIdent>>2]|0;
  $59 = (($58) + ($57)|0);
  $60 = HEAP8[$59>>0]|0;
  $61 = ($60<<24>>24)!=(0);
  if (!($61)) {
   break;
  }
  $62 = HEAP32[$j>>2]|0;
  $63 = HEAP32[$zIdent>>2]|0;
  $64 = (($63) + ($62)|0);
  $65 = HEAP8[$64>>0]|0;
  $66 = HEAP32[$i>>2]|0;
  $67 = (($66) + 1)|0;
  HEAP32[$i>>2] = $67;
  $68 = HEAP32[$0>>2]|0;
  $69 = (($68) + ($66)|0);
  HEAP8[$69>>0] = $65;
  $70 = HEAP32[$j>>2]|0;
  $71 = HEAP32[$zIdent>>2]|0;
  $72 = (($71) + ($70)|0);
  $73 = HEAP8[$72>>0]|0;
  $74 = $73&255;
  $75 = ($74|0)==(34);
  if ($75) {
   $76 = HEAP32[$i>>2]|0;
   $77 = (($76) + 1)|0;
   HEAP32[$i>>2] = $77;
   $78 = HEAP32[$0>>2]|0;
   $79 = (($78) + ($76)|0);
   HEAP8[$79>>0] = 34;
  }
  $80 = HEAP32[$j>>2]|0;
  $81 = (($80) + 1)|0;
  HEAP32[$j>>2] = $81;
 }
 $82 = HEAP32[$needQuote>>2]|0;
 $83 = ($82|0)!=(0);
 if (!($83)) {
  $88 = HEAP32[$i>>2]|0;
  $89 = HEAP32[$0>>2]|0;
  $90 = (($89) + ($88)|0);
  HEAP8[$90>>0] = 0;
  $91 = HEAP32[$i>>2]|0;
  $92 = HEAP32[$1>>2]|0;
  HEAP32[$92>>2] = $91;
  STACKTOP = sp;return;
 }
 $84 = HEAP32[$i>>2]|0;
 $85 = (($84) + 1)|0;
 HEAP32[$i>>2] = $85;
 $86 = HEAP32[$0>>2]|0;
 $87 = (($86) + ($84)|0);
 HEAP8[$87>>0] = 34;
 $88 = HEAP32[$i>>2]|0;
 $89 = HEAP32[$0>>2]|0;
 $90 = (($89) + ($88)|0);
 HEAP8[$90>>0] = 0;
 $91 = HEAP32[$i>>2]|0;
 $92 = HEAP32[$1>>2]|0;
 HEAP32[$92>>2] = $91;
 STACKTOP = sp;return;
}
function _resizeIndexObject($db,$pIdx,$N) {
 $db = $db|0;
 $pIdx = $pIdx|0;
 $N = $N|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $8 = 0, $9 = 0, $nByte = 0, $zExtra = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 20|0;
 $1 = sp + 16|0;
 $2 = sp + 12|0;
 $3 = sp + 8|0;
 $zExtra = sp + 4|0;
 $nByte = sp;
 HEAP32[$1>>2] = $db;
 HEAP32[$2>>2] = $pIdx;
 HEAP32[$3>>2] = $N;
 $4 = HEAP32[$2>>2]|0;
 $5 = (($4) + 52|0);
 $6 = HEAP16[$5>>1]|0;
 $7 = $6&65535;
 $8 = HEAP32[$3>>2]|0;
 $9 = ($7|0)>=($8|0);
 if ($9) {
  HEAP32[$0>>2] = 0;
  $71 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($71|0);
 }
 $10 = HEAP32[$3>>2]|0;
 $11 = ($10*7)|0;
 HEAP32[$nByte>>2] = $11;
 $12 = HEAP32[$1>>2]|0;
 $13 = HEAP32[$nByte>>2]|0;
 $14 = ($13|0)<(0);
 $15 = $14 << 31 >> 31;
 $16 = (_sqlite3DbMallocZero($12,$13,$15)|0);
 HEAP32[$zExtra>>2] = $16;
 $17 = HEAP32[$zExtra>>2]|0;
 $18 = ($17|0)==(0|0);
 if ($18) {
  HEAP32[$0>>2] = 7;
  $71 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($71|0);
 } else {
  $19 = HEAP32[$zExtra>>2]|0;
  $20 = HEAP32[$2>>2]|0;
  $21 = (($20) + 32|0);
  $22 = HEAP32[$21>>2]|0;
  $23 = HEAP32[$2>>2]|0;
  $24 = (($23) + 52|0);
  $25 = HEAP16[$24>>1]|0;
  $26 = $25&65535;
  $27 = $26<<2;
  _memcpy(($19|0),($22|0),($27|0))|0;
  $28 = HEAP32[$zExtra>>2]|0;
  $29 = HEAP32[$2>>2]|0;
  $30 = (($29) + 32|0);
  HEAP32[$30>>2] = $28;
  $31 = HEAP32[$3>>2]|0;
  $32 = $31<<2;
  $33 = HEAP32[$zExtra>>2]|0;
  $34 = (($33) + ($32)|0);
  HEAP32[$zExtra>>2] = $34;
  $35 = HEAP32[$zExtra>>2]|0;
  $36 = HEAP32[$2>>2]|0;
  $37 = (($36) + 4|0);
  $38 = HEAP32[$37>>2]|0;
  $39 = HEAP32[$2>>2]|0;
  $40 = (($39) + 52|0);
  $41 = HEAP16[$40>>1]|0;
  $42 = $41&65535;
  $43 = $42<<1;
  _memcpy(($35|0),($38|0),($43|0))|0;
  $44 = HEAP32[$zExtra>>2]|0;
  $45 = HEAP32[$2>>2]|0;
  $46 = (($45) + 4|0);
  HEAP32[$46>>2] = $44;
  $47 = HEAP32[$3>>2]|0;
  $48 = $47<<1;
  $49 = HEAP32[$zExtra>>2]|0;
  $50 = (($49) + ($48)|0);
  HEAP32[$zExtra>>2] = $50;
  $51 = HEAP32[$zExtra>>2]|0;
  $52 = HEAP32[$2>>2]|0;
  $53 = (($52) + 28|0);
  $54 = HEAP32[$53>>2]|0;
  $55 = HEAP32[$2>>2]|0;
  $56 = (($55) + 52|0);
  $57 = HEAP16[$56>>1]|0;
  $58 = $57&65535;
  _memcpy(($51|0),($54|0),($58|0))|0;
  $59 = HEAP32[$zExtra>>2]|0;
  $60 = HEAP32[$2>>2]|0;
  $61 = (($60) + 28|0);
  HEAP32[$61>>2] = $59;
  $62 = HEAP32[$3>>2]|0;
  $63 = $62&65535;
  $64 = HEAP32[$2>>2]|0;
  $65 = (($64) + 52|0);
  HEAP16[$65>>1] = $63;
  $66 = HEAP32[$2>>2]|0;
  $67 = (($66) + 55|0);
  $68 = HEAP8[$67>>0]|0;
  $69 = $68 & -17;
  $70 = $69 | 16;
  HEAP8[$67>>0] = $70;
  HEAP32[$0>>2] = 0;
  $71 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($71|0);
 }
 return 0|0;
}
function _codeTableLocks($pParse) {
 $pParse = $pParse|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $i = 0, $p = 0, $p1 = 0, $pVdbe = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 16|0;
 $i = sp + 12|0;
 $pVdbe = sp + 8|0;
 $p = sp + 4|0;
 $p1 = sp;
 HEAP32[$0>>2] = $pParse;
 $1 = HEAP32[$0>>2]|0;
 $2 = (_sqlite3GetVdbe($1)|0);
 HEAP32[$pVdbe>>2] = $2;
 HEAP32[$i>>2] = 0;
 while(1) {
  $3 = HEAP32[$i>>2]|0;
  $4 = HEAP32[$0>>2]|0;
  $5 = (($4) + 396|0);
  $6 = HEAP32[$5>>2]|0;
  $7 = ($3|0)<($6|0);
  if (!($7)) {
   break;
  }
  $8 = HEAP32[$i>>2]|0;
  $9 = HEAP32[$0>>2]|0;
  $10 = (($9) + 400|0);
  $11 = HEAP32[$10>>2]|0;
  $12 = (($11) + ($8<<4)|0);
  HEAP32[$p>>2] = $12;
  $13 = HEAP32[$p>>2]|0;
  $14 = HEAP32[$13>>2]|0;
  HEAP32[$p1>>2] = $14;
  $15 = HEAP32[$pVdbe>>2]|0;
  $16 = HEAP32[$p1>>2]|0;
  $17 = HEAP32[$p>>2]|0;
  $18 = (($17) + 4|0);
  $19 = HEAP32[$18>>2]|0;
  $20 = HEAP32[$p>>2]|0;
  $21 = (($20) + 8|0);
  $22 = HEAP8[$21>>0]|0;
  $23 = $22&255;
  $24 = HEAP32[$p>>2]|0;
  $25 = (($24) + 12|0);
  $26 = HEAP32[$25>>2]|0;
  (_sqlite3VdbeAddOp4($15,143,$16,$19,$23,$26,-2)|0);
  $27 = HEAP32[$i>>2]|0;
  $28 = (($27) + 1)|0;
  HEAP32[$i>>2] = $28;
 }
 STACKTOP = sp;return;
}
function _sqlite3AutoincrementBegin($pParse) {
 $pParse = $pParse|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $8 = 0, $9 = 0, $addr = 0, $db = 0, $memId = 0, $p = 0, $pDb = 0, $v = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 24|0;
 $p = sp + 20|0;
 $db = sp + 16|0;
 $pDb = sp + 12|0;
 $memId = sp + 8|0;
 $addr = sp + 4|0;
 $v = sp;
 HEAP32[$0>>2] = $pParse;
 $1 = HEAP32[$0>>2]|0;
 $2 = HEAP32[$1>>2]|0;
 HEAP32[$db>>2] = $2;
 $3 = HEAP32[$0>>2]|0;
 $4 = (($3) + 8|0);
 $5 = HEAP32[$4>>2]|0;
 HEAP32[$v>>2] = $5;
 $6 = HEAP32[$0>>2]|0;
 $7 = (($6) + 404|0);
 $8 = HEAP32[$7>>2]|0;
 HEAP32[$p>>2] = $8;
 while(1) {
  $9 = HEAP32[$p>>2]|0;
  $10 = ($9|0)!=(0|0);
  if (!($10)) {
   break;
  }
  $11 = HEAP32[$p>>2]|0;
  $12 = (($11) + 8|0);
  $13 = HEAP32[$12>>2]|0;
  $14 = HEAP32[$db>>2]|0;
  $15 = (($14) + 16|0);
  $16 = HEAP32[$15>>2]|0;
  $17 = (($16) + ($13<<4)|0);
  HEAP32[$pDb>>2] = $17;
  $18 = HEAP32[$p>>2]|0;
  $19 = (($18) + 12|0);
  $20 = HEAP32[$19>>2]|0;
  HEAP32[$memId>>2] = $20;
  $21 = HEAP32[$0>>2]|0;
  $22 = HEAP32[$p>>2]|0;
  $23 = (($22) + 8|0);
  $24 = HEAP32[$23>>2]|0;
  $25 = HEAP32[$pDb>>2]|0;
  $26 = (($25) + 12|0);
  $27 = HEAP32[$26>>2]|0;
  $28 = (($27) + 72|0);
  $29 = HEAP32[$28>>2]|0;
  _sqlite3OpenTable($21,0,$24,$29,54);
  $30 = HEAP32[$v>>2]|0;
  $31 = HEAP32[$memId>>2]|0;
  $32 = HEAP32[$memId>>2]|0;
  $33 = (($32) + 1)|0;
  (_sqlite3VdbeAddOp3($30,28,0,$31,$33)|0);
  $34 = HEAP32[$v>>2]|0;
  $35 = (_sqlite3VdbeCurrentAddr($34)|0);
  HEAP32[$addr>>2] = $35;
  $36 = HEAP32[$v>>2]|0;
  $37 = HEAP32[$memId>>2]|0;
  $38 = (($37) - 1)|0;
  $39 = HEAP32[$p>>2]|0;
  $40 = (($39) + 4|0);
  $41 = HEAP32[$40>>2]|0;
  $42 = HEAP32[$41>>2]|0;
  (_sqlite3VdbeAddOp4($36,97,0,$38,0,$42,0)|0);
  $43 = HEAP32[$v>>2]|0;
  $44 = HEAP32[$addr>>2]|0;
  $45 = (($44) + 9)|0;
  (_sqlite3VdbeAddOp2($43,108,0,$45)|0);
  $46 = HEAP32[$v>>2]|0;
  $47 = HEAP32[$memId>>2]|0;
  (_sqlite3VdbeAddOp3($46,47,0,0,$47)|0);
  $48 = HEAP32[$v>>2]|0;
  $49 = HEAP32[$memId>>2]|0;
  $50 = (($49) - 1)|0;
  $51 = HEAP32[$addr>>2]|0;
  $52 = (($51) + 7)|0;
  $53 = HEAP32[$memId>>2]|0;
  (_sqlite3VdbeAddOp3($48,78,$50,$52,$53)|0);
  $54 = HEAP32[$v>>2]|0;
  _sqlite3VdbeChangeP5($54,16);
  $55 = HEAP32[$v>>2]|0;
  $56 = HEAP32[$memId>>2]|0;
  $57 = (($56) + 1)|0;
  (_sqlite3VdbeAddOp2($55,103,0,$57)|0);
  $58 = HEAP32[$v>>2]|0;
  $59 = HEAP32[$memId>>2]|0;
  (_sqlite3VdbeAddOp3($58,47,0,1,$59)|0);
  $60 = HEAP32[$v>>2]|0;
  $61 = HEAP32[$addr>>2]|0;
  $62 = (($61) + 9)|0;
  (_sqlite3VdbeAddOp2($60,16,0,$62)|0);
  $63 = HEAP32[$v>>2]|0;
  $64 = HEAP32[$addr>>2]|0;
  $65 = (($64) + 2)|0;
  (_sqlite3VdbeAddOp2($63,9,0,$65)|0);
  $66 = HEAP32[$v>>2]|0;
  $67 = HEAP32[$memId>>2]|0;
  (_sqlite3VdbeAddOp2($66,25,0,$67)|0);
  $68 = HEAP32[$v>>2]|0;
  (_sqlite3VdbeAddOp0($68,61)|0);
  $69 = HEAP32[$p>>2]|0;
  $70 = HEAP32[$69>>2]|0;
  HEAP32[$p>>2] = $70;
 }
 STACKTOP = sp;return;
}
function _yyStackOverflow($yypParser,$yypMinor) {
 $yypParser = $yypParser|0;
 $yypMinor = $yypMinor|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $pParse = 0, $vararg_buffer = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $vararg_buffer = sp;
 $0 = sp + 12|0;
 $1 = sp + 8|0;
 $pParse = sp + 4|0;
 HEAP32[$0>>2] = $yypParser;
 HEAP32[$1>>2] = $yypMinor;
 $2 = HEAP32[$0>>2]|0;
 $3 = (($2) + 8|0);
 $4 = HEAP32[$3>>2]|0;
 HEAP32[$pParse>>2] = $4;
 $5 = HEAP32[$0>>2]|0;
 $6 = HEAP32[$5>>2]|0;
 $7 = (($6) + -1)|0;
 HEAP32[$5>>2] = $7;
 while(1) {
  $8 = HEAP32[$0>>2]|0;
  $9 = HEAP32[$8>>2]|0;
  $10 = ($9|0)>=(0);
  if (!($10)) {
   break;
  }
  $11 = HEAP32[$0>>2]|0;
  (_yy_pop_parser_stack($11)|0);
 }
 $12 = HEAP32[$pParse>>2]|0;
 _sqlite3ErrorMsg($12,27184,$vararg_buffer);
 $13 = HEAP32[$pParse>>2]|0;
 $14 = HEAP32[$0>>2]|0;
 $15 = (($14) + 8|0);
 HEAP32[$15>>2] = $13;
 STACKTOP = sp;return;
}
function _sqlite3Prepare($db,$zSql,$nBytes,$saveSqlFlag,$pReprepare,$ppStmt,$pzTail) {
 $db = $db|0;
 $zSql = $zSql|0;
 $nBytes = $nBytes|0;
 $saveSqlFlag = $saveSqlFlag|0;
 $pReprepare = $pReprepare|0;
 $ppStmt = $ppStmt|0;
 $pzTail = $pzTail|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0;
 var $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0, $188 = 0;
 var $189 = 0, $19 = 0, $190 = 0, $191 = 0, $192 = 0, $193 = 0, $194 = 0, $195 = 0, $196 = 0, $197 = 0, $198 = 0, $199 = 0, $2 = 0, $20 = 0, $200 = 0, $201 = 0, $202 = 0, $203 = 0, $204 = 0, $205 = 0;
 var $206 = 0, $207 = 0, $208 = 0, $209 = 0, $21 = 0, $210 = 0, $211 = 0, $212 = 0, $213 = 0, $214 = 0, $215 = 0, $216 = 0, $217 = 0, $218 = 0, $219 = 0, $22 = 0, $220 = 0, $221 = 0, $222 = 0, $223 = 0;
 var $224 = 0, $225 = 0, $226 = 0, $227 = 0, $228 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0;
 var $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0;
 var $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0;
 var $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0;
 var $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $i = 0, $iFirst = 0, $mx = 0, $mxLen = 0, $pBt = 0, $pParse = 0, $pT = 0, $pVdbe = 0, $rc = 0, $vararg_buffer3 = 0, $zDb = 0;
 var $zErrMsg = 0, $zSqlCopy = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 80|0;
 $vararg_buffer3 = sp;
 $0 = sp + 76|0;
 $1 = sp + 72|0;
 $2 = sp + 68|0;
 $3 = sp + 64|0;
 $4 = sp + 60|0;
 $5 = sp + 56|0;
 $6 = sp + 52|0;
 $pParse = sp + 48|0;
 $zErrMsg = sp + 44|0;
 $rc = sp + 40|0;
 $i = sp + 36|0;
 $pBt = sp + 32|0;
 $zDb = sp + 28|0;
 $zSqlCopy = sp + 24|0;
 $mxLen = sp + 20|0;
 $iFirst = sp + 16|0;
 $mx = sp + 12|0;
 $pVdbe = sp + 8|0;
 $pT = sp + 4|0;
 HEAP32[$0>>2] = $db;
 HEAP32[$1>>2] = $zSql;
 HEAP32[$2>>2] = $nBytes;
 HEAP32[$3>>2] = $saveSqlFlag;
 HEAP32[$4>>2] = $pReprepare;
 HEAP32[$5>>2] = $ppStmt;
 HEAP32[$6>>2] = $pzTail;
 HEAP32[$zErrMsg>>2] = 0;
 HEAP32[$rc>>2] = 0;
 $7 = HEAP32[$0>>2]|0;
 $8 = (_sqlite3DbMallocZero($7,536,0)|0);
 HEAP32[$pParse>>2] = $8;
 $9 = HEAP32[$pParse>>2]|0;
 $10 = ($9|0)==(0|0);
 if ($10) {
  HEAP32[$rc>>2] = 7;
  $222 = HEAP32[$pParse>>2]|0;
  _sqlite3ParserReset($222);
  $223 = HEAP32[$0>>2]|0;
  $224 = HEAP32[$pParse>>2]|0;
  _sqlite3DbFree($223,$224);
  $225 = HEAP32[$0>>2]|0;
  $226 = HEAP32[$rc>>2]|0;
  $227 = (_sqlite3ApiExit($225,$226)|0);
  HEAP32[$rc>>2] = $227;
  $228 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($228|0);
 }
 $11 = HEAP32[$4>>2]|0;
 $12 = HEAP32[$pParse>>2]|0;
 $13 = (($12) + 476|0);
 HEAP32[$13>>2] = $11;
 HEAP32[$i>>2] = 0;
 while(1) {
  $14 = HEAP32[$i>>2]|0;
  $15 = HEAP32[$0>>2]|0;
  $16 = (($15) + 20|0);
  $17 = HEAP32[$16>>2]|0;
  $18 = ($14|0)<($17|0);
  if (!($18)) {
   break;
  }
  $19 = HEAP32[$i>>2]|0;
  $20 = HEAP32[$0>>2]|0;
  $21 = (($20) + 16|0);
  $22 = HEAP32[$21>>2]|0;
  $23 = (($22) + ($19<<4)|0);
  $24 = (($23) + 4|0);
  $25 = HEAP32[$24>>2]|0;
  HEAP32[$pBt>>2] = $25;
  $26 = HEAP32[$pBt>>2]|0;
  $27 = ($26|0)!=(0|0);
  if ($27) {
   $28 = HEAP32[$pBt>>2]|0;
   $29 = (_sqlite3BtreeSchemaLocked($28)|0);
   HEAP32[$rc>>2] = $29;
   $30 = HEAP32[$rc>>2]|0;
   $31 = ($30|0)!=(0);
   if ($31) {
    label = 7;
    break;
   }
  }
  $41 = HEAP32[$i>>2]|0;
  $42 = (($41) + 1)|0;
  HEAP32[$i>>2] = $42;
 }
 if ((label|0) == 7) {
  $32 = HEAP32[$i>>2]|0;
  $33 = HEAP32[$0>>2]|0;
  $34 = (($33) + 16|0);
  $35 = HEAP32[$34>>2]|0;
  $36 = (($35) + ($32<<4)|0);
  $37 = HEAP32[$36>>2]|0;
  HEAP32[$zDb>>2] = $37;
  $38 = HEAP32[$0>>2]|0;
  $39 = HEAP32[$rc>>2]|0;
  $40 = HEAP32[$zDb>>2]|0;
  HEAP32[$vararg_buffer3>>2] = $40;
  _sqlite3ErrorWithMsg($38,$39,30936,$vararg_buffer3);
  $222 = HEAP32[$pParse>>2]|0;
  _sqlite3ParserReset($222);
  $223 = HEAP32[$0>>2]|0;
  $224 = HEAP32[$pParse>>2]|0;
  _sqlite3DbFree($223,$224);
  $225 = HEAP32[$0>>2]|0;
  $226 = HEAP32[$rc>>2]|0;
  $227 = (_sqlite3ApiExit($225,$226)|0);
  HEAP32[$rc>>2] = $227;
  $228 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($228|0);
 }
 $43 = HEAP32[$0>>2]|0;
 _sqlite3VtabUnlockList($43);
 $44 = HEAP32[$0>>2]|0;
 $45 = HEAP32[$pParse>>2]|0;
 HEAP32[$45>>2] = $44;
 $46 = HEAP32[$pParse>>2]|0;
 $47 = (($46) + 424|0);
 HEAP32[$47>>2] = 0;
 $48 = HEAP32[$2>>2]|0;
 $49 = ($48|0)>=(0);
 do {
  if ($49) {
   $50 = HEAP32[$2>>2]|0;
   $51 = ($50|0)==(0);
   if (!($51)) {
    $52 = HEAP32[$2>>2]|0;
    $53 = (($52) - 1)|0;
    $54 = HEAP32[$1>>2]|0;
    $55 = (($54) + ($53)|0);
    $56 = HEAP8[$55>>0]|0;
    $57 = $56 << 24 >> 24;
    $58 = ($57|0)!=(0);
    if (!($58)) {
     label = 17;
     break;
    }
   }
   $59 = HEAP32[$0>>2]|0;
   $60 = (($59) + 88|0);
   $61 = (($60) + 4|0);
   $62 = HEAP32[$61>>2]|0;
   HEAP32[$mxLen>>2] = $62;
   $63 = HEAP32[$2>>2]|0;
   $64 = HEAP32[$mxLen>>2]|0;
   $65 = ($63|0)>($64|0);
   $66 = HEAP32[$0>>2]|0;
   if ($65) {
    _sqlite3ErrorWithMsg($66,18,30968,$vararg_buffer3);
    $67 = HEAP32[$0>>2]|0;
    $68 = (_sqlite3ApiExit($67,18)|0);
    HEAP32[$rc>>2] = $68;
    $222 = HEAP32[$pParse>>2]|0;
    _sqlite3ParserReset($222);
    $223 = HEAP32[$0>>2]|0;
    $224 = HEAP32[$pParse>>2]|0;
    _sqlite3DbFree($223,$224);
    $225 = HEAP32[$0>>2]|0;
    $226 = HEAP32[$rc>>2]|0;
    $227 = (_sqlite3ApiExit($225,$226)|0);
    HEAP32[$rc>>2] = $227;
    $228 = HEAP32[$rc>>2]|0;
    STACKTOP = sp;return ($228|0);
   }
   $69 = HEAP32[$1>>2]|0;
   $70 = HEAP32[$2>>2]|0;
   $71 = ($70|0)<(0);
   $72 = $71 << 31 >> 31;
   $73 = (_sqlite3DbStrNDup($66,$69,$70,$72)|0);
   HEAP32[$zSqlCopy>>2] = $73;
   $74 = HEAP32[$zSqlCopy>>2]|0;
   $75 = ($74|0)!=(0|0);
   if ($75) {
    $76 = HEAP32[$pParse>>2]|0;
    $77 = HEAP32[$zSqlCopy>>2]|0;
    (_sqlite3RunParser($76,$77,$zErrMsg)|0);
    $78 = HEAP32[$0>>2]|0;
    $79 = HEAP32[$zSqlCopy>>2]|0;
    _sqlite3DbFree($78,$79);
    $80 = HEAP32[$pParse>>2]|0;
    $81 = (($80) + 480|0);
    $82 = HEAP32[$81>>2]|0;
    $83 = HEAP32[$zSqlCopy>>2]|0;
    $84 = $82;
    $85 = $83;
    $86 = (($84) - ($85))|0;
    $87 = HEAP32[$1>>2]|0;
    $88 = (($87) + ($86)|0);
    $89 = HEAP32[$pParse>>2]|0;
    $90 = (($89) + 480|0);
    HEAP32[$90>>2] = $88;
    break;
   } else {
    $91 = HEAP32[$2>>2]|0;
    $92 = HEAP32[$1>>2]|0;
    $93 = (($92) + ($91)|0);
    $94 = HEAP32[$pParse>>2]|0;
    $95 = (($94) + 480|0);
    HEAP32[$95>>2] = $93;
    break;
   }
  } else {
   label = 17;
  }
 } while(0);
 if ((label|0) == 17) {
  $96 = HEAP32[$pParse>>2]|0;
  $97 = HEAP32[$1>>2]|0;
  (_sqlite3RunParser($96,$97,$zErrMsg)|0);
 }
 $98 = HEAP32[$0>>2]|0;
 $99 = (($98) + 64|0);
 $100 = HEAP8[$99>>0]|0;
 $101 = ($100<<24>>24)!=(0);
 if ($101) {
  $102 = HEAP32[$pParse>>2]|0;
  $103 = (($102) + 12|0);
  HEAP32[$103>>2] = 7;
 }
 $104 = HEAP32[$pParse>>2]|0;
 $105 = (($104) + 12|0);
 $106 = HEAP32[$105>>2]|0;
 $107 = ($106|0)==(101);
 if ($107) {
  $108 = HEAP32[$pParse>>2]|0;
  $109 = (($108) + 12|0);
  HEAP32[$109>>2] = 0;
 }
 $110 = HEAP32[$pParse>>2]|0;
 $111 = (($110) + 17|0);
 $112 = HEAP8[$111>>0]|0;
 $113 = ($112<<24>>24)!=(0);
 if ($113) {
  $114 = HEAP32[$pParse>>2]|0;
  _schemaIsValid($114);
 }
 $115 = HEAP32[$0>>2]|0;
 $116 = (($115) + 64|0);
 $117 = HEAP8[$116>>0]|0;
 $118 = ($117<<24>>24)!=(0);
 if ($118) {
  $119 = HEAP32[$pParse>>2]|0;
  $120 = (($119) + 12|0);
  HEAP32[$120>>2] = 7;
 }
 $121 = HEAP32[$6>>2]|0;
 $122 = ($121|0)!=(0|0);
 if ($122) {
  $123 = HEAP32[$pParse>>2]|0;
  $124 = (($123) + 480|0);
  $125 = HEAP32[$124>>2]|0;
  $126 = HEAP32[$6>>2]|0;
  HEAP32[$126>>2] = $125;
 }
 $127 = HEAP32[$pParse>>2]|0;
 $128 = (($127) + 12|0);
 $129 = HEAP32[$128>>2]|0;
 HEAP32[$rc>>2] = $129;
 $130 = HEAP32[$rc>>2]|0;
 $131 = ($130|0)==(0);
 L45: do {
  if ($131) {
   $132 = HEAP32[$pParse>>2]|0;
   $133 = (($132) + 8|0);
   $134 = HEAP32[$133>>2]|0;
   $135 = ($134|0)!=(0|0);
   if ($135) {
    $136 = HEAP32[$pParse>>2]|0;
    $137 = (($136) + 450|0);
    $138 = HEAP8[$137>>0]|0;
    $139 = $138&255;
    $140 = ($139|0)!=(0);
    if ($140) {
     $141 = HEAP32[$pParse>>2]|0;
     $142 = (($141) + 450|0);
     $143 = HEAP8[$142>>0]|0;
     $144 = $143&255;
     $145 = ($144|0)==(2);
     $146 = HEAP32[$pParse>>2]|0;
     $147 = (($146) + 8|0);
     $148 = HEAP32[$147>>2]|0;
     if ($145) {
      _sqlite3VdbeSetNumCols($148,4);
      HEAP32[$iFirst>>2] = 8;
      HEAP32[$mx>>2] = 12;
     } else {
      _sqlite3VdbeSetNumCols($148,8);
      HEAP32[$iFirst>>2] = 0;
      HEAP32[$mx>>2] = 8;
     }
     $149 = HEAP32[$iFirst>>2]|0;
     HEAP32[$i>>2] = $149;
     while(1) {
      $150 = HEAP32[$i>>2]|0;
      $151 = HEAP32[$mx>>2]|0;
      $152 = ($150|0)<($151|0);
      if (!($152)) {
       break L45;
      }
      $153 = HEAP32[$pParse>>2]|0;
      $154 = (($153) + 8|0);
      $155 = HEAP32[$154>>2]|0;
      $156 = HEAP32[$i>>2]|0;
      $157 = HEAP32[$iFirst>>2]|0;
      $158 = (($156) - ($157))|0;
      $159 = HEAP32[$i>>2]|0;
      $160 = (30992 + ($159<<2)|0);
      $161 = HEAP32[$160>>2]|0;
      (_sqlite3VdbeSetColName($155,$158,0,$161,0)|0);
      $162 = HEAP32[$i>>2]|0;
      $163 = (($162) + 1)|0;
      HEAP32[$i>>2] = $163;
     }
    }
   }
  }
 } while(0);
 $164 = HEAP32[$0>>2]|0;
 $165 = (($164) + 140|0);
 $166 = (($165) + 5|0);
 $167 = HEAP8[$166>>0]|0;
 $168 = $167&255;
 $169 = ($168|0)==(0);
 if ($169) {
  $170 = HEAP32[$pParse>>2]|0;
  $171 = (($170) + 8|0);
  $172 = HEAP32[$171>>2]|0;
  HEAP32[$pVdbe>>2] = $172;
  $173 = HEAP32[$pVdbe>>2]|0;
  $174 = HEAP32[$1>>2]|0;
  $175 = HEAP32[$pParse>>2]|0;
  $176 = (($175) + 480|0);
  $177 = HEAP32[$176>>2]|0;
  $178 = HEAP32[$1>>2]|0;
  $179 = $177;
  $180 = $178;
  $181 = (($179) - ($180))|0;
  $182 = HEAP32[$3>>2]|0;
  _sqlite3VdbeSetSql($173,$174,$181,$182);
 }
 $183 = HEAP32[$pParse>>2]|0;
 $184 = (($183) + 8|0);
 $185 = HEAP32[$184>>2]|0;
 $186 = ($185|0)!=(0|0);
 do {
  if ($186) {
   $187 = HEAP32[$rc>>2]|0;
   $188 = ($187|0)!=(0);
   if (!($188)) {
    $189 = HEAP32[$0>>2]|0;
    $190 = (($189) + 64|0);
    $191 = HEAP8[$190>>0]|0;
    $192 = $191&255;
    $193 = ($192|0)!=(0);
    if (!($193)) {
     label = 43;
     break;
    }
   }
   $194 = HEAP32[$pParse>>2]|0;
   $195 = (($194) + 8|0);
   $196 = HEAP32[$195>>2]|0;
   (_sqlite3VdbeFinalize($196)|0);
  } else {
   label = 43;
  }
 } while(0);
 if ((label|0) == 43) {
  $197 = HEAP32[$pParse>>2]|0;
  $198 = (($197) + 8|0);
  $199 = HEAP32[$198>>2]|0;
  $200 = HEAP32[$5>>2]|0;
  HEAP32[$200>>2] = $199;
 }
 $201 = HEAP32[$zErrMsg>>2]|0;
 $202 = ($201|0)!=(0|0);
 $203 = HEAP32[$0>>2]|0;
 $204 = HEAP32[$rc>>2]|0;
 if ($202) {
  $205 = HEAP32[$zErrMsg>>2]|0;
  HEAP32[$vararg_buffer3>>2] = $205;
  _sqlite3ErrorWithMsg($203,$204,1200,$vararg_buffer3);
  $206 = HEAP32[$0>>2]|0;
  $207 = HEAP32[$zErrMsg>>2]|0;
  _sqlite3DbFree($206,$207);
 } else {
  _sqlite3Error($203,$204);
 }
 while(1) {
  $208 = HEAP32[$pParse>>2]|0;
  $209 = (($208) + 528|0);
  $210 = HEAP32[$209>>2]|0;
  $211 = ($210|0)!=(0|0);
  if (!($211)) {
   break;
  }
  $212 = HEAP32[$pParse>>2]|0;
  $213 = (($212) + 528|0);
  $214 = HEAP32[$213>>2]|0;
  HEAP32[$pT>>2] = $214;
  $215 = HEAP32[$pT>>2]|0;
  $216 = (($215) + 4|0);
  $217 = HEAP32[$216>>2]|0;
  $218 = HEAP32[$pParse>>2]|0;
  $219 = (($218) + 528|0);
  HEAP32[$219>>2] = $217;
  $220 = HEAP32[$0>>2]|0;
  $221 = HEAP32[$pT>>2]|0;
  _sqlite3DbFree($220,$221);
 }
 $222 = HEAP32[$pParse>>2]|0;
 _sqlite3ParserReset($222);
 $223 = HEAP32[$0>>2]|0;
 $224 = HEAP32[$pParse>>2]|0;
 _sqlite3DbFree($223,$224);
 $225 = HEAP32[$0>>2]|0;
 $226 = HEAP32[$rc>>2]|0;
 $227 = (_sqlite3ApiExit($225,$226)|0);
 HEAP32[$rc>>2] = $227;
 $228 = HEAP32[$rc>>2]|0;
 STACKTOP = sp;return ($228|0);
}
function _sqlite3BtreeSchemaLocked($p) {
 $p = $p|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $rc = sp;
 HEAP32[$0>>2] = $p;
 $1 = HEAP32[$0>>2]|0;
 _sqlite3BtreeEnter($1);
 $2 = HEAP32[$0>>2]|0;
 $3 = (_querySharedCacheTableLock($2,1,1)|0);
 HEAP32[$rc>>2] = $3;
 $4 = HEAP32[$rc>>2]|0;
 STACKTOP = sp;return ($4|0);
}
function _schemaIsValid($pParse) {
 $pParse = $pParse|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $cookie = 0, $db = 0, $iDb = 0, $openedTransaction = 0, $or$cond = 0, $pBt = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 24|0;
 $db = sp + 20|0;
 $iDb = sp + 16|0;
 $rc = sp + 12|0;
 $cookie = sp + 8|0;
 $openedTransaction = sp + 4|0;
 $pBt = sp;
 HEAP32[$0>>2] = $pParse;
 $1 = HEAP32[$0>>2]|0;
 $2 = HEAP32[$1>>2]|0;
 HEAP32[$db>>2] = $2;
 HEAP32[$iDb>>2] = 0;
 while(1) {
  $3 = HEAP32[$iDb>>2]|0;
  $4 = HEAP32[$db>>2]|0;
  $5 = (($4) + 20|0);
  $6 = HEAP32[$5>>2]|0;
  $7 = ($3|0)<($6|0);
  if (!($7)) {
   label = 14;
   break;
  }
  HEAP32[$openedTransaction>>2] = 0;
  $8 = HEAP32[$iDb>>2]|0;
  $9 = HEAP32[$db>>2]|0;
  $10 = (($9) + 16|0);
  $11 = HEAP32[$10>>2]|0;
  $12 = (($11) + ($8<<4)|0);
  $13 = (($12) + 4|0);
  $14 = HEAP32[$13>>2]|0;
  HEAP32[$pBt>>2] = $14;
  $15 = HEAP32[$pBt>>2]|0;
  $16 = ($15|0)==(0|0);
  if (!($16)) {
   $17 = HEAP32[$pBt>>2]|0;
   $18 = (_sqlite3BtreeIsInReadTrans($17)|0);
   $19 = ($18|0)!=(0);
   if (!($19)) {
    $20 = HEAP32[$pBt>>2]|0;
    $21 = (_sqlite3BtreeBeginTrans($20,0)|0);
    HEAP32[$rc>>2] = $21;
    $22 = HEAP32[$rc>>2]|0;
    $23 = ($22|0)==(7);
    $24 = HEAP32[$rc>>2]|0;
    $25 = ($24|0)==(3082);
    $or$cond = $23 | $25;
    if ($or$cond) {
     $26 = HEAP32[$db>>2]|0;
     $27 = (($26) + 64|0);
     HEAP8[$27>>0] = 1;
    }
    $28 = HEAP32[$rc>>2]|0;
    $29 = ($28|0)!=(0);
    if ($29) {
     label = 14;
     break;
    }
    HEAP32[$openedTransaction>>2] = 1;
   }
   $30 = HEAP32[$pBt>>2]|0;
   _sqlite3BtreeGetMeta($30,1,$cookie);
   $31 = HEAP32[$cookie>>2]|0;
   $32 = HEAP32[$iDb>>2]|0;
   $33 = HEAP32[$db>>2]|0;
   $34 = (($33) + 16|0);
   $35 = HEAP32[$34>>2]|0;
   $36 = (($35) + ($32<<4)|0);
   $37 = (($36) + 12|0);
   $38 = HEAP32[$37>>2]|0;
   $39 = HEAP32[$38>>2]|0;
   $40 = ($31|0)!=($39|0);
   if ($40) {
    $41 = HEAP32[$db>>2]|0;
    $42 = HEAP32[$iDb>>2]|0;
    _sqlite3ResetOneSchema($41,$42);
    $43 = HEAP32[$0>>2]|0;
    $44 = (($43) + 12|0);
    HEAP32[$44>>2] = 17;
   }
   $45 = HEAP32[$openedTransaction>>2]|0;
   $46 = ($45|0)!=(0);
   if ($46) {
    $47 = HEAP32[$pBt>>2]|0;
    (_sqlite3BtreeCommit($47)|0);
   }
  }
  $48 = HEAP32[$iDb>>2]|0;
  $49 = (($48) + 1)|0;
  HEAP32[$iDb>>2] = $49;
 }
 if ((label|0) == 14) {
  STACKTOP = sp;return;
 }
}
function _sqlite3VdbeSetSql($p,$z,$n,$isPrepareV2) {
 $p = $p|0;
 $z = $z|0;
 $n = $n|0;
 $isPrepareV2 = $isPrepareV2|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $3 = 0;
 var $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $1 = sp + 8|0;
 $2 = sp + 4|0;
 $3 = sp;
 HEAP32[$0>>2] = $p;
 HEAP32[$1>>2] = $z;
 HEAP32[$2>>2] = $n;
 HEAP32[$3>>2] = $isPrepareV2;
 $4 = HEAP32[$0>>2]|0;
 $5 = ($4|0)==(0|0);
 if ($5) {
  STACKTOP = sp;return;
 }
 $6 = HEAP32[$0>>2]|0;
 $7 = HEAP32[$6>>2]|0;
 $8 = HEAP32[$1>>2]|0;
 $9 = HEAP32[$2>>2]|0;
 $10 = ($9|0)<(0);
 $11 = $10 << 31 >> 31;
 $12 = (_sqlite3DbStrNDup($7,$8,$9,$11)|0);
 $13 = HEAP32[$0>>2]|0;
 $14 = (($13) + 168|0);
 HEAP32[$14>>2] = $12;
 $15 = HEAP32[$3>>2]|0;
 $16 = $15&255;
 $17 = $16&255;
 $18 = HEAP32[$0>>2]|0;
 $19 = (($18) + 88|0);
 $20 = $17&65535;
 $21 = HEAP16[$19>>1]|0;
 $22 = $20 & 1;
 $23 = ($22 << 10)&65535;
 $24 = $21 & -1025;
 $25 = $24 | $23;
 HEAP16[$19>>1] = $25;
 STACKTOP = sp;return;
}
function _querySharedCacheTableLock($p,$iTab,$eLock) {
 $p = $p|0;
 $iTab = $iTab|0;
 $eLock = $eLock|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $pBt = 0, $pIter = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 16|0;
 $1 = sp + 12|0;
 $2 = sp + 8|0;
 $3 = sp + 20|0;
 $pBt = sp + 4|0;
 $pIter = sp;
 HEAP32[$1>>2] = $p;
 HEAP32[$2>>2] = $iTab;
 HEAP8[$3>>0] = $eLock;
 $4 = HEAP32[$1>>2]|0;
 $5 = (($4) + 4|0);
 $6 = HEAP32[$5>>2]|0;
 HEAP32[$pBt>>2] = $6;
 $7 = HEAP32[$1>>2]|0;
 $8 = (($7) + 9|0);
 $9 = HEAP8[$8>>0]|0;
 $10 = ($9<<24>>24)!=(0);
 if (!($10)) {
  HEAP32[$0>>2] = 0;
  $55 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($55|0);
 }
 $11 = HEAP32[$pBt>>2]|0;
 $12 = (($11) + 76|0);
 $13 = HEAP32[$12>>2]|0;
 $14 = HEAP32[$1>>2]|0;
 $15 = ($13|0)!=($14|0);
 if ($15) {
  $16 = HEAP32[$pBt>>2]|0;
  $17 = (($16) + 22|0);
  $18 = HEAP16[$17>>1]|0;
  $19 = $18&65535;
  $20 = $19 & 32;
  $21 = ($20|0)!=(0);
  if ($21) {
   HEAP32[$0>>2] = 262;
   $55 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($55|0);
  }
 }
 $22 = HEAP32[$pBt>>2]|0;
 $23 = (($22) + 72|0);
 $24 = HEAP32[$23>>2]|0;
 HEAP32[$pIter>>2] = $24;
 while(1) {
  $25 = HEAP32[$pIter>>2]|0;
  $26 = ($25|0)!=(0|0);
  if (!($26)) {
   label = 15;
   break;
  }
  $27 = HEAP32[$pIter>>2]|0;
  $28 = HEAP32[$27>>2]|0;
  $29 = HEAP32[$1>>2]|0;
  $30 = ($28|0)!=($29|0);
  if ($30) {
   $31 = HEAP32[$pIter>>2]|0;
   $32 = (($31) + 4|0);
   $33 = HEAP32[$32>>2]|0;
   $34 = HEAP32[$2>>2]|0;
   $35 = ($33|0)==($34|0);
   if ($35) {
    $36 = HEAP32[$pIter>>2]|0;
    $37 = (($36) + 8|0);
    $38 = HEAP8[$37>>0]|0;
    $39 = $38&255;
    $40 = HEAP8[$3>>0]|0;
    $41 = $40&255;
    $42 = ($39|0)!=($41|0);
    if ($42) {
     break;
    }
   }
  }
  $52 = HEAP32[$pIter>>2]|0;
  $53 = (($52) + 12|0);
  $54 = HEAP32[$53>>2]|0;
  HEAP32[$pIter>>2] = $54;
 }
 if ((label|0) == 15) {
  HEAP32[$0>>2] = 0;
  $55 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($55|0);
 }
 $43 = HEAP8[$3>>0]|0;
 $44 = $43&255;
 $45 = ($44|0)==(2);
 if ($45) {
  $46 = HEAP32[$pBt>>2]|0;
  $47 = (($46) + 22|0);
  $48 = HEAP16[$47>>1]|0;
  $49 = $48&65535;
  $50 = $49 | 64;
  $51 = $50&65535;
  HEAP16[$47>>1] = $51;
 }
 HEAP32[$0>>2] = 262;
 $55 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($55|0);
}
function _sqlite3VdbeSwap($pA,$pB) {
 $pA = $pA|0;
 $pB = $pB|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $pTmp = 0, $tmp = 0, $zTmp = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 224|0;
 $0 = sp + 220|0;
 $1 = sp + 216|0;
 $tmp = sp;
 $pTmp = sp + 212|0;
 $zTmp = sp + 208|0;
 HEAP32[$0>>2] = $pA;
 HEAP32[$1>>2] = $pB;
 $2 = HEAP32[$0>>2]|0;
 _memcpy(($tmp|0),($2|0),208)|0;
 $3 = HEAP32[$0>>2]|0;
 $4 = HEAP32[$1>>2]|0;
 _memcpy(($3|0),($4|0),208)|0;
 $5 = HEAP32[$1>>2]|0;
 _memcpy(($5|0),($tmp|0),208)|0;
 $6 = HEAP32[$0>>2]|0;
 $7 = (($6) + 52|0);
 $8 = HEAP32[$7>>2]|0;
 HEAP32[$pTmp>>2] = $8;
 $9 = HEAP32[$1>>2]|0;
 $10 = (($9) + 52|0);
 $11 = HEAP32[$10>>2]|0;
 $12 = HEAP32[$0>>2]|0;
 $13 = (($12) + 52|0);
 HEAP32[$13>>2] = $11;
 $14 = HEAP32[$pTmp>>2]|0;
 $15 = HEAP32[$1>>2]|0;
 $16 = (($15) + 52|0);
 HEAP32[$16>>2] = $14;
 $17 = HEAP32[$0>>2]|0;
 $18 = (($17) + 48|0);
 $19 = HEAP32[$18>>2]|0;
 HEAP32[$pTmp>>2] = $19;
 $20 = HEAP32[$1>>2]|0;
 $21 = (($20) + 48|0);
 $22 = HEAP32[$21>>2]|0;
 $23 = HEAP32[$0>>2]|0;
 $24 = (($23) + 48|0);
 HEAP32[$24>>2] = $22;
 $25 = HEAP32[$pTmp>>2]|0;
 $26 = HEAP32[$1>>2]|0;
 $27 = (($26) + 48|0);
 HEAP32[$27>>2] = $25;
 $28 = HEAP32[$0>>2]|0;
 $29 = (($28) + 168|0);
 $30 = HEAP32[$29>>2]|0;
 HEAP32[$zTmp>>2] = $30;
 $31 = HEAP32[$1>>2]|0;
 $32 = (($31) + 168|0);
 $33 = HEAP32[$32>>2]|0;
 $34 = HEAP32[$0>>2]|0;
 $35 = (($34) + 168|0);
 HEAP32[$35>>2] = $33;
 $36 = HEAP32[$zTmp>>2]|0;
 $37 = HEAP32[$1>>2]|0;
 $38 = (($37) + 168|0);
 HEAP32[$38>>2] = $36;
 $39 = HEAP32[$0>>2]|0;
 $40 = (($39) + 88|0);
 $41 = HEAP16[$40>>1]|0;
 $42 = ($41&65535) >>> 10;
 $43 = $42 & 1;
 $44 = $43&65535;
 $45 = HEAP32[$1>>2]|0;
 $46 = (($45) + 88|0);
 $47 = $44&65535;
 $48 = HEAP16[$46>>1]|0;
 $49 = $47 & 1;
 $50 = ($49 << 10)&65535;
 $51 = $48 & -1025;
 $52 = $51 | $50;
 HEAP16[$46>>1] = $52;
 STACKTOP = sp;return;
}
function _sqlite3VdbeResetStepResult($p) {
 $p = $p|0;
 var $0 = 0, $1 = 0, $2 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 HEAP32[$0>>2] = $p;
 $1 = HEAP32[$0>>2]|0;
 $2 = (($1) + 80|0);
 HEAP32[$2>>2] = 0;
 STACKTOP = sp;return;
}
function _sqlite3FkDelete($db,$pTab) {
 $db = $db|0;
 $pTab = $pTab|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $7 = 0, $8 = 0, $9 = 0, $p = 0, $pFKey = 0, $pNext = 0, $z = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 20|0;
 $1 = sp + 16|0;
 $pFKey = sp + 12|0;
 $pNext = sp + 8|0;
 $p = sp + 4|0;
 $z = sp;
 HEAP32[$0>>2] = $db;
 HEAP32[$1>>2] = $pTab;
 $2 = HEAP32[$1>>2]|0;
 $3 = (($2) + 16|0);
 $4 = HEAP32[$3>>2]|0;
 HEAP32[$pFKey>>2] = $4;
 while(1) {
  $5 = HEAP32[$pFKey>>2]|0;
  $6 = ($5|0)!=(0|0);
  if (!($6)) {
   break;
  }
  $7 = HEAP32[$0>>2]|0;
  $8 = ($7|0)!=(0|0);
  if ($8) {
   $9 = HEAP32[$0>>2]|0;
   $10 = (($9) + 520|0);
   $11 = HEAP32[$10>>2]|0;
   $12 = ($11|0)==(0|0);
   if ($12) {
    label = 5;
   }
  } else {
   label = 5;
  }
  if ((label|0) == 5) {
   label = 0;
   $13 = HEAP32[$pFKey>>2]|0;
   $14 = (($13) + 16|0);
   $15 = HEAP32[$14>>2]|0;
   $16 = ($15|0)!=(0|0);
   $17 = HEAP32[$pFKey>>2]|0;
   $18 = (($17) + 12|0);
   $19 = HEAP32[$18>>2]|0;
   if ($16) {
    $20 = HEAP32[$pFKey>>2]|0;
    $21 = (($20) + 16|0);
    $22 = HEAP32[$21>>2]|0;
    $23 = (($22) + 12|0);
    HEAP32[$23>>2] = $19;
   } else {
    HEAP32[$p>>2] = $19;
    $24 = HEAP32[$p>>2]|0;
    $25 = ($24|0)!=(0|0);
    $26 = HEAP32[$pFKey>>2]|0;
    if ($25) {
     $27 = (($26) + 12|0);
     $28 = HEAP32[$27>>2]|0;
     $29 = (($28) + 8|0);
     $30 = HEAP32[$29>>2]|0;
     $33 = $30;
    } else {
     $31 = (($26) + 8|0);
     $32 = HEAP32[$31>>2]|0;
     $33 = $32;
    }
    HEAP32[$z>>2] = $33;
    $34 = HEAP32[$1>>2]|0;
    $35 = (($34) + 68|0);
    $36 = HEAP32[$35>>2]|0;
    $37 = (($36) + 56|0);
    $38 = HEAP32[$z>>2]|0;
    $39 = HEAP32[$p>>2]|0;
    (_sqlite3HashInsert($37,$38,$39)|0);
   }
   $40 = HEAP32[$pFKey>>2]|0;
   $41 = (($40) + 12|0);
   $42 = HEAP32[$41>>2]|0;
   $43 = ($42|0)!=(0|0);
   if ($43) {
    $44 = HEAP32[$pFKey>>2]|0;
    $45 = (($44) + 16|0);
    $46 = HEAP32[$45>>2]|0;
    $47 = HEAP32[$pFKey>>2]|0;
    $48 = (($47) + 12|0);
    $49 = HEAP32[$48>>2]|0;
    $50 = (($49) + 16|0);
    HEAP32[$50>>2] = $46;
   }
  }
  $51 = HEAP32[$0>>2]|0;
  $52 = HEAP32[$pFKey>>2]|0;
  $53 = (($52) + 28|0);
  $54 = HEAP32[$53>>2]|0;
  _fkTriggerDelete($51,$54);
  $55 = HEAP32[$0>>2]|0;
  $56 = HEAP32[$pFKey>>2]|0;
  $57 = (($56) + 28|0);
  $58 = (($57) + 4|0);
  $59 = HEAP32[$58>>2]|0;
  _fkTriggerDelete($55,$59);
  $60 = HEAP32[$pFKey>>2]|0;
  $61 = (($60) + 4|0);
  $62 = HEAP32[$61>>2]|0;
  HEAP32[$pNext>>2] = $62;
  $63 = HEAP32[$0>>2]|0;
  $64 = HEAP32[$pFKey>>2]|0;
  _sqlite3DbFree($63,$64);
  $65 = HEAP32[$pNext>>2]|0;
  HEAP32[$pFKey>>2] = $65;
 }
 STACKTOP = sp;return;
}
function _sqlite3VtabClear($db,$p) {
 $db = $db|0;
 $p = $p|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $i = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $1 = sp + 4|0;
 $i = sp;
 HEAP32[$0>>2] = $db;
 HEAP32[$1>>2] = $p;
 $2 = HEAP32[$0>>2]|0;
 $3 = ($2|0)!=(0|0);
 if ($3) {
  $4 = HEAP32[$0>>2]|0;
  $5 = (($4) + 520|0);
  $6 = HEAP32[$5>>2]|0;
  $7 = ($6|0)==(0|0);
  if ($7) {
   label = 3;
  }
 } else {
  label = 3;
 }
 if ((label|0) == 3) {
  $8 = HEAP32[$1>>2]|0;
  (_vtabDisconnectAll(0,$8)|0);
 }
 $9 = HEAP32[$1>>2]|0;
 $10 = (($9) + 56|0);
 $11 = HEAP32[$10>>2]|0;
 $12 = ($11|0)!=(0|0);
 if (!($12)) {
  STACKTOP = sp;return;
 }
 HEAP32[$i>>2] = 0;
 while(1) {
  $13 = HEAP32[$i>>2]|0;
  $14 = HEAP32[$1>>2]|0;
  $15 = (($14) + 52|0);
  $16 = HEAP32[$15>>2]|0;
  $17 = ($13|0)<($16|0);
  if (!($17)) {
   break;
  }
  $18 = HEAP32[$i>>2]|0;
  $19 = ($18|0)!=(1);
  if ($19) {
   $20 = HEAP32[$0>>2]|0;
   $21 = HEAP32[$i>>2]|0;
   $22 = HEAP32[$1>>2]|0;
   $23 = (($22) + 56|0);
   $24 = HEAP32[$23>>2]|0;
   $25 = (($24) + ($21<<2)|0);
   $26 = HEAP32[$25>>2]|0;
   _sqlite3DbFree($20,$26);
  }
  $27 = HEAP32[$i>>2]|0;
  $28 = (($27) + 1)|0;
  HEAP32[$i>>2] = $28;
 }
 $29 = HEAP32[$0>>2]|0;
 $30 = HEAP32[$1>>2]|0;
 $31 = (($30) + 56|0);
 $32 = HEAP32[$31>>2]|0;
 _sqlite3DbFree($29,$32);
 STACKTOP = sp;return;
}
function _vtabDisconnectAll($db,$p) {
 $db = $db|0;
 $p = $p|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $db2 = 0, $pNext = 0, $pRet = 0, $pVTable = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 20|0;
 $1 = sp + 16|0;
 $pRet = sp + 12|0;
 $pVTable = sp + 8|0;
 $db2 = sp + 4|0;
 $pNext = sp;
 HEAP32[$0>>2] = $db;
 HEAP32[$1>>2] = $p;
 HEAP32[$pRet>>2] = 0;
 $2 = HEAP32[$1>>2]|0;
 $3 = (($2) + 60|0);
 $4 = HEAP32[$3>>2]|0;
 HEAP32[$pVTable>>2] = $4;
 $5 = HEAP32[$1>>2]|0;
 $6 = (($5) + 60|0);
 HEAP32[$6>>2] = 0;
 while(1) {
  $7 = HEAP32[$pVTable>>2]|0;
  $8 = ($7|0)!=(0|0);
  if (!($8)) {
   break;
  }
  $9 = HEAP32[$pVTable>>2]|0;
  $10 = HEAP32[$9>>2]|0;
  HEAP32[$db2>>2] = $10;
  $11 = HEAP32[$pVTable>>2]|0;
  $12 = (($11) + 24|0);
  $13 = HEAP32[$12>>2]|0;
  HEAP32[$pNext>>2] = $13;
  $14 = HEAP32[$db2>>2]|0;
  $15 = HEAP32[$0>>2]|0;
  $16 = ($14|0)==($15|0);
  if ($16) {
   $17 = HEAP32[$pVTable>>2]|0;
   HEAP32[$pRet>>2] = $17;
   $18 = HEAP32[$pRet>>2]|0;
   $19 = HEAP32[$1>>2]|0;
   $20 = (($19) + 60|0);
   HEAP32[$20>>2] = $18;
   $21 = HEAP32[$pRet>>2]|0;
   $22 = (($21) + 24|0);
   HEAP32[$22>>2] = 0;
  } else {
   $23 = HEAP32[$db2>>2]|0;
   $24 = (($23) + 332|0);
   $25 = HEAP32[$24>>2]|0;
   $26 = HEAP32[$pVTable>>2]|0;
   $27 = (($26) + 24|0);
   HEAP32[$27>>2] = $25;
   $28 = HEAP32[$pVTable>>2]|0;
   $29 = HEAP32[$db2>>2]|0;
   $30 = (($29) + 332|0);
   HEAP32[$30>>2] = $28;
  }
  $31 = HEAP32[$pNext>>2]|0;
  HEAP32[$pVTable>>2] = $31;
 }
 $32 = HEAP32[$pRet>>2]|0;
 STACKTOP = sp;return ($32|0);
}
function _saveAllCursors($pBt,$iRoot,$pExcept) {
 $pBt = $pBt|0;
 $iRoot = $iRoot|0;
 $pExcept = $pExcept|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $p = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $1 = sp + 8|0;
 $2 = sp + 4|0;
 $p = sp;
 HEAP32[$0>>2] = $pBt;
 HEAP32[$1>>2] = $iRoot;
 HEAP32[$2>>2] = $pExcept;
 $3 = HEAP32[$0>>2]|0;
 $4 = (($3) + 8|0);
 $5 = HEAP32[$4>>2]|0;
 HEAP32[$p>>2] = $5;
 while(1) {
  $6 = HEAP32[$p>>2]|0;
  $7 = ($6|0)!=(0|0);
  if (!($7)) {
   break;
  }
  $8 = HEAP32[$p>>2]|0;
  $9 = HEAP32[$2>>2]|0;
  $10 = ($8|0)!=($9|0);
  if ($10) {
   $11 = HEAP32[$1>>2]|0;
   $12 = (0)==($11|0);
   if ($12) {
    break;
   }
   $13 = HEAP32[$p>>2]|0;
   $14 = (($13) + 60|0);
   $15 = HEAP32[$14>>2]|0;
   $16 = HEAP32[$1>>2]|0;
   $17 = ($15|0)==($16|0);
   if ($17) {
    break;
   }
  }
  $18 = HEAP32[$p>>2]|0;
  $19 = (($18) + 8|0);
  $20 = HEAP32[$19>>2]|0;
  HEAP32[$p>>2] = $20;
 }
 $21 = HEAP32[$p>>2]|0;
 $22 = ($21|0)!=(0|0);
 if (!($22)) {
  $27 = 0;
  STACKTOP = sp;return ($27|0);
 }
 $23 = HEAP32[$p>>2]|0;
 $24 = HEAP32[$1>>2]|0;
 $25 = HEAP32[$2>>2]|0;
 $26 = (_saveCursorsOnList($23,$24,$25)|0);
 $27 = $26;
 STACKTOP = sp;return ($27|0);
}
function _saveCursorsOnList($p,$iRoot,$pExcept) {
 $p = $p|0;
 $iRoot = $iRoot|0;
 $pExcept = $pExcept|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 16|0;
 $1 = sp + 12|0;
 $2 = sp + 8|0;
 $3 = sp + 4|0;
 $rc = sp;
 HEAP32[$1>>2] = $p;
 HEAP32[$2>>2] = $iRoot;
 HEAP32[$3>>2] = $pExcept;
 L1: while(1) {
  $4 = HEAP32[$1>>2]|0;
  $5 = HEAP32[$3>>2]|0;
  $6 = ($4|0)!=($5|0);
  do {
   if ($6) {
    $7 = HEAP32[$2>>2]|0;
    $8 = (0)==($7|0);
    if (!($8)) {
     $9 = HEAP32[$1>>2]|0;
     $10 = (($9) + 60|0);
     $11 = HEAP32[$10>>2]|0;
     $12 = HEAP32[$2>>2]|0;
     $13 = ($11|0)==($12|0);
     if (!($13)) {
      break;
     }
    }
    $14 = HEAP32[$1>>2]|0;
    $15 = (($14) + 73|0);
    $16 = HEAP8[$15>>0]|0;
    $17 = $16&255;
    $18 = ($17|0)==(1);
    $19 = HEAP32[$1>>2]|0;
    if ($18) {
     $20 = (_saveCursorPosition($19)|0);
     HEAP32[$rc>>2] = $20;
     $21 = HEAP32[$rc>>2]|0;
     $22 = (0)!=($21|0);
     if ($22) {
      label = 7;
      break L1;
     } else {
      break;
     }
    } else {
     _btreeReleaseAllCursorPages($19);
     break;
    }
   }
  } while(0);
  $24 = HEAP32[$1>>2]|0;
  $25 = (($24) + 8|0);
  $26 = HEAP32[$25>>2]|0;
  HEAP32[$1>>2] = $26;
  $27 = HEAP32[$1>>2]|0;
  $28 = ($27|0)!=(0|0);
  if (!($28)) {
   label = 10;
   break;
  }
 }
 if ((label|0) == 7) {
  $23 = HEAP32[$rc>>2]|0;
  HEAP32[$0>>2] = $23;
  $29 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($29|0);
 }
 else if ((label|0) == 10) {
  HEAP32[$0>>2] = 0;
  $29 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($29|0);
 }
 return 0|0;
}
function _saveCursorPosition($pCur) {
 $pCur = $pCur|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $pKey = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $rc = sp + 4|0;
 $pKey = sp;
 HEAP32[$0>>2] = $pCur;
 $1 = HEAP32[$0>>2]|0;
 $2 = HEAP32[$0>>2]|0;
 $3 = (($2) + 48|0);
 $4 = (_sqlite3BtreeKeySize($1,$3)|0);
 HEAP32[$rc>>2] = $4;
 $5 = HEAP32[$0>>2]|0;
 $6 = (($5) + 120|0);
 $7 = HEAP32[$6>>2]|0;
 $8 = (($7) + 2|0);
 $9 = HEAP8[$8>>0]|0;
 $10 = $9&255;
 $11 = (0)==($10|0);
 do {
  if ($11) {
   $12 = HEAP32[$0>>2]|0;
   $13 = (($12) + 48|0);
   $14 = $13;
   $15 = $14;
   $16 = HEAP32[$15>>2]|0;
   $17 = (($14) + 4)|0;
   $18 = $17;
   $19 = HEAP32[$18>>2]|0;
   $20 = (_sqlite3Malloc($16,$19)|0);
   HEAP32[$pKey>>2] = $20;
   $21 = HEAP32[$pKey>>2]|0;
   $22 = ($21|0)!=(0|0);
   if (!($22)) {
    HEAP32[$rc>>2] = 7;
    break;
   }
   $23 = HEAP32[$0>>2]|0;
   $24 = HEAP32[$0>>2]|0;
   $25 = (($24) + 48|0);
   $26 = $25;
   $27 = $26;
   $28 = HEAP32[$27>>2]|0;
   $29 = (($26) + 4)|0;
   $30 = $29;
   $31 = HEAP32[$30>>2]|0;
   $32 = HEAP32[$pKey>>2]|0;
   $33 = (_sqlite3BtreeKey($23,0,$28,$32)|0);
   HEAP32[$rc>>2] = $33;
   $34 = HEAP32[$rc>>2]|0;
   $35 = ($34|0)==(0);
   $36 = HEAP32[$pKey>>2]|0;
   if ($35) {
    $37 = HEAP32[$0>>2]|0;
    $38 = (($37) + 56|0);
    HEAP32[$38>>2] = $36;
    break;
   } else {
    _sqlite3_free($36);
    break;
   }
  }
 } while(0);
 $39 = HEAP32[$rc>>2]|0;
 $40 = ($39|0)==(0);
 if (!($40)) {
  $44 = HEAP32[$0>>2]|0;
  $45 = (($44) + 72|0);
  $46 = HEAP8[$45>>0]|0;
  $47 = $46&255;
  $48 = $47 & -5;
  $49 = $48&255;
  HEAP8[$45>>0] = $49;
  $50 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($50|0);
 }
 $41 = HEAP32[$0>>2]|0;
 _btreeReleaseAllCursorPages($41);
 $42 = HEAP32[$0>>2]|0;
 $43 = (($42) + 73|0);
 HEAP8[$43>>0] = 3;
 $44 = HEAP32[$0>>2]|0;
 $45 = (($44) + 72|0);
 $46 = HEAP8[$45>>0]|0;
 $47 = $46&255;
 $48 = $47 & -5;
 $49 = $48&255;
 HEAP8[$45>>0] = $49;
 $50 = HEAP32[$rc>>2]|0;
 STACKTOP = sp;return ($50|0);
}
function _btreeReleaseAllCursorPages($pCur) {
 $pCur = $pCur|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0;
 var $i = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $i = sp;
 HEAP32[$0>>2] = $pCur;
 HEAP32[$i>>2] = 0;
 while(1) {
  $1 = HEAP32[$i>>2]|0;
  $2 = HEAP32[$0>>2]|0;
  $3 = (($2) + 76|0);
  $4 = HEAP16[$3>>1]|0;
  $5 = $4 << 16 >> 16;
  $6 = ($1|0)<=($5|0);
  if (!($6)) {
   break;
  }
  $7 = HEAP32[$i>>2]|0;
  $8 = HEAP32[$0>>2]|0;
  $9 = (($8) + 120|0);
  $10 = (($9) + ($7<<2)|0);
  $11 = HEAP32[$10>>2]|0;
  _releasePage($11);
  $12 = HEAP32[$i>>2]|0;
  $13 = HEAP32[$0>>2]|0;
  $14 = (($13) + 120|0);
  $15 = (($14) + ($12<<2)|0);
  HEAP32[$15>>2] = 0;
  $16 = HEAP32[$i>>2]|0;
  $17 = (($16) + 1)|0;
  HEAP32[$i>>2] = $17;
 }
 $18 = HEAP32[$0>>2]|0;
 $19 = (($18) + 76|0);
 HEAP16[$19>>1] = -1;
 STACKTOP = sp;return;
}
function _sqlite3BtreeKeySize($pCur,$pSize) {
 $pCur = $pCur|0;
 $pSize = $pSize|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $5 = 0;
 var $6 = 0, $7 = 0, $8 = 0, $9 = 0, $iPage = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $1 = sp + 4|0;
 $iPage = sp;
 HEAP32[$0>>2] = $pCur;
 HEAP32[$1>>2] = $pSize;
 $2 = HEAP32[$0>>2]|0;
 $3 = (($2) + 24|0);
 $4 = (($3) + 20|0);
 $5 = HEAP16[$4>>1]|0;
 $6 = $5&65535;
 $7 = ($6|0)==(0);
 if ($7) {
  $8 = HEAP32[$0>>2]|0;
  $9 = (($8) + 76|0);
  $10 = HEAP16[$9>>1]|0;
  $11 = $10 << 16 >> 16;
  HEAP32[$iPage>>2] = $11;
  $12 = HEAP32[$iPage>>2]|0;
  $13 = HEAP32[$0>>2]|0;
  $14 = (($13) + 120|0);
  $15 = (($14) + ($12<<2)|0);
  $16 = HEAP32[$15>>2]|0;
  $17 = HEAP32[$iPage>>2]|0;
  $18 = HEAP32[$0>>2]|0;
  $19 = (($18) + 78|0);
  $20 = (($19) + ($17<<1)|0);
  $21 = HEAP16[$20>>1]|0;
  $22 = $21&65535;
  $23 = HEAP32[$0>>2]|0;
  $24 = (($23) + 24|0);
  _btreeParseCell($16,$22,$24);
  $25 = HEAP32[$0>>2]|0;
  $26 = (($25) + 72|0);
  $27 = HEAP8[$26>>0]|0;
  $28 = $27&255;
  $29 = $28 | 2;
  $30 = $29&255;
  HEAP8[$26>>0] = $30;
 }
 $31 = HEAP32[$0>>2]|0;
 $32 = (($31) + 24|0);
 $33 = $32;
 $34 = $33;
 $35 = HEAP32[$34>>2]|0;
 $36 = (($33) + 4)|0;
 $37 = $36;
 $38 = HEAP32[$37>>2]|0;
 $39 = HEAP32[$1>>2]|0;
 $40 = $39;
 $41 = $40;
 HEAP32[$41>>2] = $35;
 $42 = (($40) + 4)|0;
 $43 = $42;
 HEAP32[$43>>2] = $38;
 STACKTOP = sp;return 0;
}
function _sqlite3BtreeKey($pCur,$offset,$amt,$pBuf) {
 $pCur = $pCur|0;
 $offset = $offset|0;
 $amt = $amt|0;
 $pBuf = $pBuf|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $1 = sp + 8|0;
 $2 = sp + 4|0;
 $3 = sp;
 HEAP32[$0>>2] = $pCur;
 HEAP32[$1>>2] = $offset;
 HEAP32[$2>>2] = $amt;
 HEAP32[$3>>2] = $pBuf;
 $4 = HEAP32[$0>>2]|0;
 $5 = HEAP32[$1>>2]|0;
 $6 = HEAP32[$2>>2]|0;
 $7 = HEAP32[$3>>2]|0;
 $8 = (_accessPayload($4,$5,$6,$7,0)|0);
 STACKTOP = sp;return ($8|0);
}
function _allocSpace($pBuf,$nByte,$ppFrom,$pEnd,$pnByte) {
 $pBuf = $pBuf|0;
 $nByte = $nByte|0;
 $ppFrom = $ppFrom|0;
 $pEnd = $pEnd|0;
 $pnByte = $pnByte|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 20|0;
 $1 = sp + 16|0;
 $2 = sp + 12|0;
 $3 = sp + 8|0;
 $4 = sp + 4|0;
 $5 = sp;
 HEAP32[$1>>2] = $pBuf;
 HEAP32[$2>>2] = $nByte;
 HEAP32[$3>>2] = $ppFrom;
 HEAP32[$4>>2] = $pEnd;
 HEAP32[$5>>2] = $pnByte;
 $6 = HEAP32[$1>>2]|0;
 $7 = ($6|0)!=(0|0);
 if ($7) {
  $8 = HEAP32[$1>>2]|0;
  HEAP32[$0>>2] = $8;
  $29 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($29|0);
 }
 $9 = HEAP32[$2>>2]|0;
 $10 = (($9) + 7)|0;
 $11 = $10 & -8;
 HEAP32[$2>>2] = $11;
 $12 = HEAP32[$2>>2]|0;
 $13 = HEAP32[$3>>2]|0;
 $14 = HEAP32[$13>>2]|0;
 $15 = (($14) + ($12)|0);
 $16 = HEAP32[$4>>2]|0;
 $17 = ($15>>>0)<=($16>>>0);
 if ($17) {
  $18 = HEAP32[$3>>2]|0;
  $19 = HEAP32[$18>>2]|0;
  HEAP32[$1>>2] = $19;
  $20 = HEAP32[$2>>2]|0;
  $21 = HEAP32[$3>>2]|0;
  $22 = HEAP32[$21>>2]|0;
  $23 = (($22) + ($20)|0);
  HEAP32[$21>>2] = $23;
 } else {
  $24 = HEAP32[$2>>2]|0;
  $25 = HEAP32[$5>>2]|0;
  $26 = HEAP32[$25>>2]|0;
  $27 = (($26) + ($24))|0;
  HEAP32[$25>>2] = $27;
 }
 $28 = HEAP32[$1>>2]|0;
 HEAP32[$0>>2] = $28;
 $29 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($29|0);
}
function _freeP4($db,$p4type,$p4) {
 $db = $db|0;
 $p4type = $p4type|0;
 $p4 = $p4|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0;
 var $p = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $1 = sp + 8|0;
 $2 = sp + 4|0;
 $p = sp;
 HEAP32[$0>>2] = $db;
 HEAP32[$1>>2] = $p4type;
 HEAP32[$2>>2] = $p4;
 $3 = HEAP32[$2>>2]|0;
 $4 = ($3|0)!=(0|0);
 L1: do {
  if ($4) {
   $5 = HEAP32[$1>>2]|0;
   switch ($5|0) {
   case -10:  {
    $35 = HEAP32[$0>>2]|0;
    $36 = (($35) + 520|0);
    $37 = HEAP32[$36>>2]|0;
    $38 = ($37|0)==(0|0);
    if (!($38)) {
     break L1;
    }
    $39 = HEAP32[$2>>2]|0;
    _sqlite3VtabUnlock($39);
    break L1;
    break;
   }
   case -15: case -1: case -13: case -12:  {
    $6 = HEAP32[$0>>2]|0;
    $7 = HEAP32[$2>>2]|0;
    _sqlite3DbFree($6,$7);
    break L1;
    break;
   }
   case -5:  {
    $18 = HEAP32[$0>>2]|0;
    $19 = HEAP32[$2>>2]|0;
    _freeEphemeralFunction($18,$19);
    break L1;
    break;
   }
   case -8:  {
    $20 = HEAP32[$0>>2]|0;
    $21 = (($20) + 520|0);
    $22 = HEAP32[$21>>2]|0;
    $23 = ($22|0)==(0|0);
    $24 = HEAP32[$2>>2]|0;
    if ($23) {
     _sqlite3ValueFree($24);
     break L1;
    }
    HEAP32[$p>>2] = $24;
    $25 = HEAP32[$p>>2]|0;
    $26 = (($25) + 24|0);
    $27 = HEAP32[$26>>2]|0;
    $28 = ($27|0)!=(0);
    if ($28) {
     $29 = HEAP32[$0>>2]|0;
     $30 = HEAP32[$p>>2]|0;
     $31 = (($30) + 20|0);
     $32 = HEAP32[$31>>2]|0;
     _sqlite3DbFree($29,$32);
    }
    $33 = HEAP32[$0>>2]|0;
    $34 = HEAP32[$p>>2]|0;
    _sqlite3DbFree($33,$34);
    break L1;
    break;
   }
   case -6:  {
    $8 = HEAP32[$0>>2]|0;
    $9 = (($8) + 520|0);
    $10 = HEAP32[$9>>2]|0;
    $11 = ($10|0)==(0|0);
    if (!($11)) {
     break L1;
    }
    $12 = HEAP32[$2>>2]|0;
    _sqlite3KeyInfoUnref($12);
    break L1;
    break;
   }
   case -11:  {
    $13 = HEAP32[$0>>2]|0;
    $14 = (($13) + 520|0);
    $15 = HEAP32[$14>>2]|0;
    $16 = ($15|0)==(0|0);
    if (!($16)) {
     break L1;
    }
    $17 = HEAP32[$2>>2]|0;
    _sqlite3_free($17);
    break L1;
    break;
   }
   default: {
    break L1;
   }
   }
  }
 } while(0);
 STACKTOP = sp;return;
}
function _freeEphemeralFunction($db,$pDef) {
 $db = $db|0;
 $pDef = $pDef|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $1 = sp;
 HEAP32[$0>>2] = $db;
 HEAP32[$1>>2] = $pDef;
 $2 = HEAP32[$1>>2]|0;
 $3 = ($2|0)!=(0|0);
 if (!($3)) {
  STACKTOP = sp;return;
 }
 $4 = HEAP32[$1>>2]|0;
 $5 = (($4) + 2|0);
 $6 = HEAP16[$5>>1]|0;
 $7 = $6&65535;
 $8 = $7 & 16;
 $9 = ($8|0)!=(0);
 if (!($9)) {
  STACKTOP = sp;return;
 }
 $10 = HEAP32[$0>>2]|0;
 $11 = HEAP32[$1>>2]|0;
 _sqlite3DbFree($10,$11);
 STACKTOP = sp;return;
}
function _sqlite3VtabLock($pVTab) {
 $pVTab = $pVTab|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 HEAP32[$0>>2] = $pVTab;
 $1 = HEAP32[$0>>2]|0;
 $2 = (($1) + 12|0);
 $3 = HEAP32[$2>>2]|0;
 $4 = (($3) + 1)|0;
 HEAP32[$2>>2] = $4;
 STACKTOP = sp;return;
}
function _sqlite3VdbeMemMove($pTo,$pFrom) {
 $pTo = $pTo|0;
 $pFrom = $pFrom|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, dest = 0, label = 0, sp = 0, src = 0, stop = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $1 = sp;
 HEAP32[$0>>2] = $pTo;
 HEAP32[$1>>2] = $pFrom;
 $2 = HEAP32[$0>>2]|0;
 _sqlite3VdbeMemRelease($2);
 $3 = HEAP32[$0>>2]|0;
 $4 = HEAP32[$1>>2]|0;
 dest=$3+0|0; src=$4+0|0; stop=dest+40|0; do { HEAP32[dest>>2]=HEAP32[src>>2]|0; dest=dest+4|0; src=src+4|0; } while ((dest|0) < (stop|0));
 $5 = HEAP32[$1>>2]|0;
 $6 = (($5) + 8|0);
 HEAP16[$6>>1] = 1;
 $7 = HEAP32[$1>>2]|0;
 $8 = (($7) + 24|0);
 HEAP32[$8>>2] = 0;
 STACKTOP = sp;return;
}
function _columnNullValue() {
 var label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = sp;return (31224|0);
}
function _sqlite3VdbeList($p) {
 $p = $p|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0;
 var $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0, $188 = 0;
 var $189 = 0, $19 = 0, $190 = 0, $191 = 0, $192 = 0, $193 = 0, $194 = 0, $195 = 0, $196 = 0, $197 = 0, $198 = 0, $199 = 0, $2 = 0, $20 = 0, $200 = 0, $201 = 0, $202 = 0, $203 = 0, $204 = 0, $205 = 0;
 var $206 = 0, $207 = 0, $208 = 0, $209 = 0, $21 = 0, $210 = 0, $211 = 0, $212 = 0, $213 = 0, $214 = 0, $215 = 0, $216 = 0, $217 = 0, $218 = 0, $219 = 0, $22 = 0, $220 = 0, $221 = 0, $222 = 0, $223 = 0;
 var $224 = 0, $225 = 0, $226 = 0, $227 = 0, $228 = 0, $229 = 0, $23 = 0, $230 = 0, $231 = 0, $232 = 0, $233 = 0, $234 = 0, $235 = 0, $236 = 0, $237 = 0, $238 = 0, $239 = 0, $24 = 0, $240 = 0, $241 = 0;
 var $242 = 0, $243 = 0, $244 = 0, $245 = 0, $246 = 0, $247 = 0, $248 = 0, $249 = 0, $25 = 0, $250 = 0, $251 = 0, $252 = 0, $253 = 0, $254 = 0, $255 = 0, $256 = 0, $257 = 0, $258 = 0, $259 = 0, $26 = 0;
 var $260 = 0, $261 = 0, $262 = 0, $263 = 0, $264 = 0, $265 = 0, $266 = 0, $267 = 0, $268 = 0, $269 = 0, $27 = 0, $270 = 0, $271 = 0, $272 = 0, $273 = 0, $274 = 0, $275 = 0, $276 = 0, $277 = 0, $278 = 0;
 var $279 = 0, $28 = 0, $280 = 0, $281 = 0, $282 = 0, $283 = 0, $284 = 0, $285 = 0, $286 = 0, $287 = 0, $288 = 0, $289 = 0, $29 = 0, $290 = 0, $291 = 0, $292 = 0, $293 = 0, $294 = 0, $295 = 0, $296 = 0;
 var $297 = 0, $298 = 0, $299 = 0, $3 = 0, $30 = 0, $300 = 0, $301 = 0, $302 = 0, $303 = 0, $304 = 0, $305 = 0, $306 = 0, $307 = 0, $308 = 0, $309 = 0, $31 = 0, $310 = 0, $311 = 0, $312 = 0, $313 = 0;
 var $314 = 0, $315 = 0, $316 = 0, $317 = 0, $318 = 0, $319 = 0, $32 = 0, $320 = 0, $321 = 0, $322 = 0, $323 = 0, $324 = 0, $325 = 0, $326 = 0, $327 = 0, $328 = 0, $329 = 0, $33 = 0, $330 = 0, $331 = 0;
 var $332 = 0, $333 = 0, $334 = 0, $335 = 0, $336 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0;
 var $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0;
 var $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0;
 var $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $apSub = 0, $db = 0, $i = 0;
 var $j = 0, $j1 = 0, $nByte = 0, $nRow = 0, $nSub = 0, $pMem = 0, $pOp = 0, $pSub = 0, $rc = 0, $vararg_buffer2 = 0, $zP4 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 64|0;
 $vararg_buffer2 = sp;
 $0 = sp + 60|0;
 $1 = sp + 56|0;
 $nRow = sp + 52|0;
 $nSub = sp + 48|0;
 $apSub = sp + 44|0;
 $pSub = sp + 40|0;
 $db = sp + 36|0;
 $i = sp + 32|0;
 $rc = sp + 28|0;
 $pMem = sp + 24|0;
 $zP4 = sp + 20|0;
 $pOp = sp + 16|0;
 $j = sp + 12|0;
 $nByte = sp + 8|0;
 $j1 = sp + 4|0;
 HEAP32[$1>>2] = $p;
 HEAP32[$nSub>>2] = 0;
 HEAP32[$apSub>>2] = 0;
 HEAP32[$pSub>>2] = 0;
 $2 = HEAP32[$1>>2]|0;
 $3 = HEAP32[$2>>2]|0;
 HEAP32[$db>>2] = $3;
 HEAP32[$rc>>2] = 0;
 $4 = HEAP32[$1>>2]|0;
 $5 = (($4) + 8|0);
 $6 = HEAP32[$5>>2]|0;
 $7 = (($6) + 40|0);
 HEAP32[$pMem>>2] = $7;
 $8 = HEAP32[$pMem>>2]|0;
 _releaseMemArray($8,8);
 $9 = HEAP32[$1>>2]|0;
 $10 = (($9) + 20|0);
 HEAP32[$10>>2] = 0;
 $11 = HEAP32[$1>>2]|0;
 $12 = (($11) + 80|0);
 $13 = HEAP32[$12>>2]|0;
 $14 = ($13|0)==(7);
 if ($14) {
  $15 = HEAP32[$db>>2]|0;
  $16 = (($15) + 64|0);
  HEAP8[$16>>0] = 1;
  HEAP32[$0>>2] = 1;
  $336 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($336|0);
 }
 $17 = HEAP32[$1>>2]|0;
 $18 = (($17) + 32|0);
 $19 = HEAP32[$18>>2]|0;
 HEAP32[$nRow>>2] = $19;
 $20 = HEAP32[$1>>2]|0;
 $21 = (($20) + 88|0);
 $22 = HEAP16[$21>>1]|0;
 $23 = $22 & 3;
 $24 = $23&65535;
 $25 = ($24|0)==(1);
 L5: do {
  if ($25) {
   $26 = HEAP32[$1>>2]|0;
   $27 = (($26) + 8|0);
   $28 = HEAP32[$27>>2]|0;
   $29 = (($28) + 360|0);
   HEAP32[$pSub>>2] = $29;
   $30 = HEAP32[$pSub>>2]|0;
   $31 = (($30) + 8|0);
   $32 = HEAP16[$31>>1]|0;
   $33 = $32&65535;
   $34 = $33 & 16;
   $35 = ($34|0)!=(0);
   if ($35) {
    $36 = HEAP32[$pSub>>2]|0;
    $37 = (($36) + 12|0);
    $38 = HEAP32[$37>>2]|0;
    $39 = (($38>>>0) / 4)&-1;
    HEAP32[$nSub>>2] = $39;
    $40 = HEAP32[$pSub>>2]|0;
    $41 = (($40) + 16|0);
    $42 = HEAP32[$41>>2]|0;
    HEAP32[$apSub>>2] = $42;
   }
   HEAP32[$i>>2] = 0;
   while(1) {
    $43 = HEAP32[$i>>2]|0;
    $44 = HEAP32[$nSub>>2]|0;
    $45 = ($43|0)<($44|0);
    if (!($45)) {
     break L5;
    }
    $46 = HEAP32[$i>>2]|0;
    $47 = HEAP32[$apSub>>2]|0;
    $48 = (($47) + ($46<<2)|0);
    $49 = HEAP32[$48>>2]|0;
    $50 = (($49) + 4|0);
    $51 = HEAP32[$50>>2]|0;
    $52 = HEAP32[$nRow>>2]|0;
    $53 = (($52) + ($51))|0;
    HEAP32[$nRow>>2] = $53;
    $54 = HEAP32[$i>>2]|0;
    $55 = (($54) + 1)|0;
    HEAP32[$i>>2] = $55;
   }
  }
 } while(0);
 while(1) {
  $56 = HEAP32[$1>>2]|0;
  $57 = (($56) + 76|0);
  $58 = HEAP32[$57>>2]|0;
  $59 = (($58) + 1)|0;
  HEAP32[$57>>2] = $59;
  HEAP32[$i>>2] = $58;
  $60 = HEAP32[$i>>2]|0;
  $61 = HEAP32[$nRow>>2]|0;
  $62 = ($60|0)<($61|0);
  if (!($62)) {
   break;
  }
  $63 = HEAP32[$1>>2]|0;
  $64 = (($63) + 88|0);
  $65 = HEAP16[$64>>1]|0;
  $66 = $65 & 3;
  $67 = $66&65535;
  $68 = ($67|0)==(2);
  if (!($68)) {
   break;
  }
  $69 = HEAP32[$i>>2]|0;
  $70 = HEAP32[$1>>2]|0;
  $71 = (($70) + 4|0);
  $72 = HEAP32[$71>>2]|0;
  $73 = (($72) + (($69*20)|0)|0);
  $74 = HEAP8[$73>>0]|0;
  $75 = $74&255;
  $76 = ($75|0)!=(155);
  if (!($76)) {
   break;
  }
 }
 $77 = HEAP32[$i>>2]|0;
 $78 = HEAP32[$nRow>>2]|0;
 $79 = ($77|0)>=($78|0);
 do {
  if ($79) {
   $80 = HEAP32[$1>>2]|0;
   $81 = (($80) + 80|0);
   HEAP32[$81>>2] = 0;
   HEAP32[$rc>>2] = 101;
  } else {
   $82 = HEAP32[$db>>2]|0;
   $83 = (($82) + 240|0);
   $84 = HEAP32[$83>>2]|0;
   $85 = ($84|0)!=(0);
   if ($85) {
    $86 = HEAP32[$1>>2]|0;
    $87 = (($86) + 80|0);
    HEAP32[$87>>2] = 9;
    HEAP32[$rc>>2] = 1;
    $88 = HEAP32[$1>>2]|0;
    $89 = (($88) + 44|0);
    $90 = HEAP32[$db>>2]|0;
    $91 = HEAP32[$1>>2]|0;
    $92 = (($91) + 80|0);
    $93 = HEAP32[$92>>2]|0;
    $94 = (_sqlite3ErrStr($93)|0);
    HEAP32[$vararg_buffer2>>2] = $94;
    _sqlite3SetString($89,$90,1200,$vararg_buffer2);
    break;
   }
   $95 = HEAP32[$i>>2]|0;
   $96 = HEAP32[$1>>2]|0;
   $97 = (($96) + 32|0);
   $98 = HEAP32[$97>>2]|0;
   $99 = ($95|0)<($98|0);
   if ($99) {
    $100 = HEAP32[$i>>2]|0;
    $101 = HEAP32[$1>>2]|0;
    $102 = (($101) + 4|0);
    $103 = HEAP32[$102>>2]|0;
    $104 = (($103) + (($100*20)|0)|0);
    HEAP32[$pOp>>2] = $104;
   } else {
    $105 = HEAP32[$1>>2]|0;
    $106 = (($105) + 32|0);
    $107 = HEAP32[$106>>2]|0;
    $108 = HEAP32[$i>>2]|0;
    $109 = (($108) - ($107))|0;
    HEAP32[$i>>2] = $109;
    HEAP32[$j>>2] = 0;
    while(1) {
     $110 = HEAP32[$i>>2]|0;
     $111 = HEAP32[$j>>2]|0;
     $112 = HEAP32[$apSub>>2]|0;
     $113 = (($112) + ($111<<2)|0);
     $114 = HEAP32[$113>>2]|0;
     $115 = (($114) + 4|0);
     $116 = HEAP32[$115>>2]|0;
     $117 = ($110|0)>=($116|0);
     if (!($117)) {
      break;
     }
     $118 = HEAP32[$j>>2]|0;
     $119 = HEAP32[$apSub>>2]|0;
     $120 = (($119) + ($118<<2)|0);
     $121 = HEAP32[$120>>2]|0;
     $122 = (($121) + 4|0);
     $123 = HEAP32[$122>>2]|0;
     $124 = HEAP32[$i>>2]|0;
     $125 = (($124) - ($123))|0;
     HEAP32[$i>>2] = $125;
     $126 = HEAP32[$j>>2]|0;
     $127 = (($126) + 1)|0;
     HEAP32[$j>>2] = $127;
    }
    $128 = HEAP32[$i>>2]|0;
    $129 = HEAP32[$j>>2]|0;
    $130 = HEAP32[$apSub>>2]|0;
    $131 = (($130) + ($129<<2)|0);
    $132 = HEAP32[$131>>2]|0;
    $133 = HEAP32[$132>>2]|0;
    $134 = (($133) + (($128*20)|0)|0);
    HEAP32[$pOp>>2] = $134;
   }
   $135 = HEAP32[$1>>2]|0;
   $136 = (($135) + 88|0);
   $137 = HEAP16[$136>>1]|0;
   $138 = $137 & 3;
   $139 = $138&65535;
   $140 = ($139|0)==(1);
   if ($140) {
    $141 = HEAP32[$pMem>>2]|0;
    $142 = (($141) + 8|0);
    HEAP16[$142>>1] = 4;
    $143 = HEAP32[$i>>2]|0;
    $144 = ($143|0)<(0);
    $145 = $144 << 31 >> 31;
    $146 = HEAP32[$pMem>>2]|0;
    $147 = $146;
    $148 = $147;
    HEAP32[$148>>2] = $143;
    $149 = (($147) + 4)|0;
    $150 = $149;
    HEAP32[$150>>2] = $145;
    $151 = HEAP32[$pMem>>2]|0;
    $152 = (($151) + 40|0);
    HEAP32[$pMem>>2] = $152;
    $153 = HEAP32[$pMem>>2]|0;
    $154 = (($153) + 8|0);
    HEAP16[$154>>1] = 2562;
    $155 = HEAP32[$pOp>>2]|0;
    $156 = HEAP8[$155>>0]|0;
    $157 = $156&255;
    $158 = (_sqlite3OpcodeName($157)|0);
    $159 = HEAP32[$pMem>>2]|0;
    $160 = (($159) + 16|0);
    HEAP32[$160>>2] = $158;
    $161 = HEAP32[$pMem>>2]|0;
    $162 = (($161) + 16|0);
    $163 = HEAP32[$162>>2]|0;
    $164 = (_sqlite3Strlen30($163)|0);
    $165 = HEAP32[$pMem>>2]|0;
    $166 = (($165) + 12|0);
    HEAP32[$166>>2] = $164;
    $167 = HEAP32[$pMem>>2]|0;
    $168 = (($167) + 10|0);
    HEAP8[$168>>0] = 1;
    $169 = HEAP32[$pMem>>2]|0;
    $170 = (($169) + 40|0);
    HEAP32[$pMem>>2] = $170;
    $171 = HEAP32[$pOp>>2]|0;
    $172 = (($171) + 1|0);
    $173 = HEAP8[$172>>0]|0;
    $174 = $173 << 24 >> 24;
    $175 = ($174|0)==(-18);
    if ($175) {
     $176 = HEAP32[$nSub>>2]|0;
     $177 = (($176) + 1)|0;
     $178 = $177<<2;
     HEAP32[$nByte>>2] = $178;
     HEAP32[$j1>>2] = 0;
     while(1) {
      $179 = HEAP32[$j1>>2]|0;
      $180 = HEAP32[$nSub>>2]|0;
      $181 = ($179|0)<($180|0);
      if (!($181)) {
       break;
      }
      $182 = HEAP32[$j1>>2]|0;
      $183 = HEAP32[$apSub>>2]|0;
      $184 = (($183) + ($182<<2)|0);
      $185 = HEAP32[$184>>2]|0;
      $186 = HEAP32[$pOp>>2]|0;
      $187 = (($186) + 16|0);
      $188 = HEAP32[$187>>2]|0;
      $189 = ($185|0)==($188|0);
      if ($189) {
       break;
      }
      $190 = HEAP32[$j1>>2]|0;
      $191 = (($190) + 1)|0;
      HEAP32[$j1>>2] = $191;
     }
     $192 = HEAP32[$j1>>2]|0;
     $193 = HEAP32[$nSub>>2]|0;
     $194 = ($192|0)==($193|0);
     if ($194) {
      $195 = HEAP32[$pSub>>2]|0;
      $196 = HEAP32[$nByte>>2]|0;
      $197 = HEAP32[$nSub>>2]|0;
      $198 = ($197|0)!=(0);
      $199 = $198&1;
      $200 = (_sqlite3VdbeMemGrow($195,$196,$199)|0);
      $201 = (0)==($200|0);
      if ($201) {
       $202 = HEAP32[$pSub>>2]|0;
       $203 = (($202) + 16|0);
       $204 = HEAP32[$203>>2]|0;
       HEAP32[$apSub>>2] = $204;
       $205 = HEAP32[$pOp>>2]|0;
       $206 = (($205) + 16|0);
       $207 = HEAP32[$206>>2]|0;
       $208 = HEAP32[$nSub>>2]|0;
       $209 = (($208) + 1)|0;
       HEAP32[$nSub>>2] = $209;
       $210 = HEAP32[$apSub>>2]|0;
       $211 = (($210) + ($208<<2)|0);
       HEAP32[$211>>2] = $207;
       $212 = HEAP32[$pSub>>2]|0;
       $213 = (($212) + 8|0);
       $214 = HEAP16[$213>>1]|0;
       $215 = $214&65535;
       $216 = $215 | 16;
       $217 = $216&65535;
       HEAP16[$213>>1] = $217;
       $218 = HEAP32[$nSub>>2]|0;
       $219 = $218<<2;
       $220 = HEAP32[$pSub>>2]|0;
       $221 = (($220) + 12|0);
       HEAP32[$221>>2] = $219;
      }
     }
    }
   }
   $222 = HEAP32[$pMem>>2]|0;
   $223 = (($222) + 8|0);
   HEAP16[$223>>1] = 4;
   $224 = HEAP32[$pOp>>2]|0;
   $225 = (($224) + 4|0);
   $226 = HEAP32[$225>>2]|0;
   $227 = ($226|0)<(0);
   $228 = $227 << 31 >> 31;
   $229 = HEAP32[$pMem>>2]|0;
   $230 = $229;
   $231 = $230;
   HEAP32[$231>>2] = $226;
   $232 = (($230) + 4)|0;
   $233 = $232;
   HEAP32[$233>>2] = $228;
   $234 = HEAP32[$pMem>>2]|0;
   $235 = (($234) + 40|0);
   HEAP32[$pMem>>2] = $235;
   $236 = HEAP32[$pMem>>2]|0;
   $237 = (($236) + 8|0);
   HEAP16[$237>>1] = 4;
   $238 = HEAP32[$pOp>>2]|0;
   $239 = (($238) + 8|0);
   $240 = HEAP32[$239>>2]|0;
   $241 = ($240|0)<(0);
   $242 = $241 << 31 >> 31;
   $243 = HEAP32[$pMem>>2]|0;
   $244 = $243;
   $245 = $244;
   HEAP32[$245>>2] = $240;
   $246 = (($244) + 4)|0;
   $247 = $246;
   HEAP32[$247>>2] = $242;
   $248 = HEAP32[$pMem>>2]|0;
   $249 = (($248) + 40|0);
   HEAP32[$pMem>>2] = $249;
   $250 = HEAP32[$pMem>>2]|0;
   $251 = (($250) + 8|0);
   HEAP16[$251>>1] = 4;
   $252 = HEAP32[$pOp>>2]|0;
   $253 = (($252) + 12|0);
   $254 = HEAP32[$253>>2]|0;
   $255 = ($254|0)<(0);
   $256 = $255 << 31 >> 31;
   $257 = HEAP32[$pMem>>2]|0;
   $258 = $257;
   $259 = $258;
   HEAP32[$259>>2] = $254;
   $260 = (($258) + 4)|0;
   $261 = $260;
   HEAP32[$261>>2] = $256;
   $262 = HEAP32[$pMem>>2]|0;
   $263 = (($262) + 40|0);
   HEAP32[$pMem>>2] = $263;
   $264 = HEAP32[$pMem>>2]|0;
   $265 = (_sqlite3VdbeMemClearAndResize($264,32)|0);
   $266 = ($265|0)!=(0);
   if ($266) {
    HEAP32[$0>>2] = 1;
    $336 = HEAP32[$0>>2]|0;
    STACKTOP = sp;return ($336|0);
   }
   $267 = HEAP32[$pMem>>2]|0;
   $268 = (($267) + 8|0);
   HEAP16[$268>>1] = 514;
   $269 = HEAP32[$pOp>>2]|0;
   $270 = HEAP32[$pMem>>2]|0;
   $271 = (($270) + 16|0);
   $272 = HEAP32[$271>>2]|0;
   $273 = (_displayP4($269,$272,32)|0);
   HEAP32[$zP4>>2] = $273;
   $274 = HEAP32[$zP4>>2]|0;
   $275 = HEAP32[$pMem>>2]|0;
   $276 = (($275) + 16|0);
   $277 = HEAP32[$276>>2]|0;
   $278 = ($274|0)!=($277|0);
   $279 = HEAP32[$pMem>>2]|0;
   if ($278) {
    $280 = HEAP32[$zP4>>2]|0;
    (_sqlite3VdbeMemSetStr($279,$280,-1,1,0)|0);
   } else {
    $281 = (($279) + 16|0);
    $282 = HEAP32[$281>>2]|0;
    $283 = (_sqlite3Strlen30($282)|0);
    $284 = HEAP32[$pMem>>2]|0;
    $285 = (($284) + 12|0);
    HEAP32[$285>>2] = $283;
    $286 = HEAP32[$pMem>>2]|0;
    $287 = (($286) + 10|0);
    HEAP8[$287>>0] = 1;
   }
   $288 = HEAP32[$pMem>>2]|0;
   $289 = (($288) + 40|0);
   HEAP32[$pMem>>2] = $289;
   $290 = HEAP32[$1>>2]|0;
   $291 = (($290) + 88|0);
   $292 = HEAP16[$291>>1]|0;
   $293 = $292 & 3;
   $294 = $293&65535;
   $295 = ($294|0)==(1);
   do {
    if ($295) {
     $296 = HEAP32[$pMem>>2]|0;
     $297 = (_sqlite3VdbeMemClearAndResize($296,4)|0);
     $298 = ($297|0)!=(0);
     if (!($298)) {
      $299 = HEAP32[$pMem>>2]|0;
      $300 = (($299) + 8|0);
      HEAP16[$300>>1] = 514;
      $301 = HEAP32[$pMem>>2]|0;
      $302 = (($301) + 12|0);
      HEAP32[$302>>2] = 2;
      $303 = HEAP32[$pMem>>2]|0;
      $304 = (($303) + 16|0);
      $305 = HEAP32[$304>>2]|0;
      $306 = HEAP32[$pOp>>2]|0;
      $307 = (($306) + 3|0);
      $308 = HEAP8[$307>>0]|0;
      $309 = $308&255;
      HEAP32[$vararg_buffer2>>2] = $309;
      (_sqlite3_snprintf(3,$305,34488,$vararg_buffer2)|0);
      $310 = HEAP32[$pMem>>2]|0;
      $311 = (($310) + 10|0);
      HEAP8[$311>>0] = 1;
      $312 = HEAP32[$pMem>>2]|0;
      $313 = (($312) + 40|0);
      HEAP32[$pMem>>2] = $313;
      $314 = HEAP32[$pMem>>2]|0;
      $315 = (($314) + 8|0);
      HEAP16[$315>>1] = 1;
      break;
     }
     HEAP32[$0>>2] = 1;
     $336 = HEAP32[$0>>2]|0;
     STACKTOP = sp;return ($336|0);
    }
   } while(0);
   $316 = HEAP32[$1>>2]|0;
   $317 = (($316) + 88|0);
   $318 = HEAP16[$317>>1]|0;
   $319 = $318 & 3;
   $320 = $319&65535;
   $321 = (($320) - 1)|0;
   $322 = $321<<2;
   $323 = (8 - ($322))|0;
   $324 = $323&65535;
   $325 = HEAP32[$1>>2]|0;
   $326 = (($325) + 84|0);
   HEAP16[$326>>1] = $324;
   $327 = HEAP32[$1>>2]|0;
   $328 = (($327) + 8|0);
   $329 = HEAP32[$328>>2]|0;
   $330 = (($329) + 40|0);
   $331 = HEAP32[$1>>2]|0;
   $332 = (($331) + 20|0);
   HEAP32[$332>>2] = $330;
   $333 = HEAP32[$1>>2]|0;
   $334 = (($333) + 80|0);
   HEAP32[$334>>2] = 0;
   HEAP32[$rc>>2] = 100;
  }
 } while(0);
 $335 = HEAP32[$rc>>2]|0;
 HEAP32[$0>>2] = $335;
 $336 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($336|0);
}
function _sqlite3VdbeExec($p) {
 $p = $p|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $1000 = 0, $1001 = 0, $1002 = 0, $1003 = 0, $1004 = 0, $1005 = 0, $1006 = 0, $1007 = 0, $1008 = 0, $1009 = 0, $101 = 0, $1010 = 0, $1011 = 0, $1012 = 0, $1013 = 0, $1014 = 0;
 var $1015 = 0, $1016 = 0, $1017 = 0, $1018 = 0, $1019 = 0, $102 = 0, $1020 = 0, $1021 = 0, $1022 = 0, $1023 = 0, $1024 = 0, $1025 = 0, $1026 = 0, $1027 = 0, $1028 = 0, $1029 = 0, $103 = 0, $1030 = 0, $1031 = 0, $1032 = 0;
 var $1033 = 0, $1034 = 0, $1035 = 0, $1036 = 0, $1037 = 0, $1038 = 0, $1039 = 0, $104 = 0, $1040 = 0, $1041 = 0, $1042 = 0, $1043 = 0, $1044 = 0, $1045 = 0, $1046 = 0, $1047 = 0, $1048 = 0, $1049 = 0, $105 = 0, $1050 = 0;
 var $1051 = 0, $1052 = 0, $1053 = 0, $1054 = 0, $1055 = 0.0, $1056 = 0, $1057 = 0.0, $1058 = 0, $1059 = 0, $106 = 0, $1060 = 0, $1061 = 0.0, $1062 = 0.0, $1063 = 0.0, $1064 = 0.0, $1065 = 0.0, $1066 = 0.0, $1067 = 0.0, $1068 = 0.0, $1069 = 0.0;
 var $107 = 0, $1070 = 0.0, $1071 = 0, $1072 = 0.0, $1073 = 0.0, $1074 = 0.0, $1075 = 0.0, $1076 = 0, $1077 = 0, $1078 = 0, $1079 = 0, $108 = 0, $1080 = 0, $1081 = 0, $1082 = 0.0, $1083 = 0, $1084 = 0, $1085 = 0, $1086 = 0, $1087 = 0;
 var $1088 = 0, $1089 = 0, $109 = 0, $1090 = 0, $1091 = 0, $1092 = 0, $1093 = 0, $1094 = 0, $1095 = 0, $1096 = 0, $1097 = 0, $1098 = 0, $1099 = 0, $11 = 0, $110 = 0, $1100 = 0, $1101 = 0, $1102 = 0, $1103 = 0, $1104 = 0;
 var $1105 = 0, $1106 = 0, $1107 = 0, $1108 = 0, $1109 = 0, $111 = 0, $1110 = 0, $1111 = 0, $1112 = 0, $1113 = 0, $1114 = 0, $1115 = 0, $1116 = 0, $1117 = 0, $1118 = 0, $1119 = 0, $112 = 0, $1120 = 0, $1121 = 0, $1122 = 0;
 var $1123 = 0, $1124 = 0, $1125 = 0.0, $1126 = 0.0, $1127 = 0, $1128 = 0, $1129 = 0.0, $113 = 0, $1130 = 0, $1131 = 0, $1132 = 0, $1133 = 0, $1134 = 0, $1135 = 0, $1136 = 0, $1137 = 0, $1138 = 0, $1139 = 0, $114 = 0, $1140 = 0;
 var $1141 = 0, $1142 = 0, $1143 = 0, $1144 = 0, $1145 = 0, $1146 = 0, $1147 = 0, $1148 = 0, $1149 = 0, $115 = 0, $1150 = 0, $1151 = 0, $1152 = 0, $1153 = 0, $1154 = 0, $1155 = 0, $1156 = 0, $1157 = 0, $1158 = 0, $1159 = 0;
 var $116 = 0, $1160 = 0, $1161 = 0, $1162 = 0, $1163 = 0, $1164 = 0, $1165 = 0, $1166 = 0, $1167 = 0, $1168 = 0, $1169 = 0, $117 = 0, $1170 = 0, $1171 = 0, $1172 = 0, $1173 = 0, $1174 = 0, $1175 = 0, $1176 = 0, $1177 = 0;
 var $1178 = 0, $1179 = 0, $118 = 0, $1180 = 0, $1181 = 0, $1182 = 0, $1183 = 0, $1184 = 0, $1185 = 0, $1186 = 0, $1187 = 0, $1188 = 0, $1189 = 0, $119 = 0, $1190 = 0, $1191 = 0, $1192 = 0, $1193 = 0, $1194 = 0, $1195 = 0;
 var $1196 = 0, $1197 = 0, $1198 = 0, $1199 = 0, $12 = 0, $120 = 0, $1200 = 0, $1201 = 0, $1202 = 0, $1203 = 0, $1204 = 0, $1205 = 0, $1206 = 0, $1207 = 0, $1208 = 0, $1209 = 0, $121 = 0, $1210 = 0, $1211 = 0, $1212 = 0;
 var $1213 = 0, $1214 = 0, $1215 = 0, $1216 = 0, $1217 = 0, $1218 = 0, $1219 = 0, $122 = 0, $1220 = 0, $1221 = 0, $1222 = 0, $1223 = 0, $1224 = 0, $1225 = 0, $1226 = 0, $1227 = 0, $1228 = 0, $1229 = 0, $123 = 0, $1230 = 0;
 var $1231 = 0, $1232 = 0, $1233 = 0, $1234 = 0, $1235 = 0, $1236 = 0, $1237 = 0, $1238 = 0, $1239 = 0, $124 = 0, $1240 = 0, $1241 = 0, $1242 = 0, $1243 = 0, $1244 = 0, $1245 = 0, $1246 = 0, $1247 = 0, $1248 = 0, $1249 = 0;
 var $125 = 0, $1250 = 0, $1251 = 0, $1252 = 0, $1253 = 0, $1254 = 0, $1255 = 0, $1256 = 0, $1257 = 0, $1258 = 0, $1259 = 0, $126 = 0, $1260 = 0, $1261 = 0, $1262 = 0, $1263 = 0, $1264 = 0, $1265 = 0, $1266 = 0, $1267 = 0;
 var $1268 = 0, $1269 = 0, $127 = 0, $1270 = 0, $1271 = 0, $1272 = 0, $1273 = 0, $1274 = 0, $1275 = 0, $1276 = 0, $1277 = 0, $1278 = 0, $1279 = 0, $128 = 0, $1280 = 0, $1281 = 0, $1282 = 0, $1283 = 0, $1284 = 0, $1285 = 0;
 var $1286 = 0, $1287 = 0, $1288 = 0, $1289 = 0, $129 = 0, $1290 = 0, $1291 = 0, $1292 = 0, $1293 = 0, $1294 = 0, $1295 = 0, $1296 = 0, $1297 = 0, $1298 = 0, $1299 = 0, $13 = 0, $130 = 0, $1300 = 0, $1301 = 0, $1302 = 0;
 var $1303 = 0, $1304 = 0, $1305 = 0, $1306 = 0, $1307 = 0, $1308 = 0, $1309 = 0, $131 = 0, $1310 = 0, $1311 = 0, $1312 = 0, $1313 = 0, $1314 = 0, $1315 = 0, $1316 = 0, $1317 = 0, $1318 = 0, $1319 = 0, $132 = 0, $1320 = 0;
 var $1321 = 0, $1322 = 0, $1323 = 0, $1324 = 0, $1325 = 0, $1326 = 0, $1327 = 0, $1328 = 0, $1329 = 0, $133 = 0, $1330 = 0, $1331 = 0, $1332 = 0, $1333 = 0, $1334 = 0, $1335 = 0, $1336 = 0, $1337 = 0, $1338 = 0, $1339 = 0;
 var $134 = 0, $1340 = 0, $1341 = 0, $1342 = 0, $1343 = 0, $1344 = 0, $1345 = 0, $1346 = 0, $1347 = 0, $1348 = 0, $1349 = 0, $135 = 0, $1350 = 0, $1351 = 0, $1352 = 0, $1353 = 0, $1354 = 0, $1355 = 0, $1356 = 0, $1357 = 0;
 var $1358 = 0, $1359 = 0, $136 = 0, $1360 = 0, $1361 = 0, $1362 = 0, $1363 = 0, $1364 = 0, $1365 = 0, $1366 = 0, $1367 = 0, $1368 = 0, $1369 = 0, $137 = 0, $1370 = 0, $1371 = 0, $1372 = 0, $1373 = 0, $1374 = 0, $1375 = 0;
 var $1376 = 0, $1377 = 0, $1378 = 0, $1379 = 0, $138 = 0, $1380 = 0, $1381 = 0, $1382 = 0, $1383 = 0, $1384 = 0, $1385 = 0, $1386 = 0, $1387 = 0, $1388 = 0, $1389 = 0, $139 = 0, $1390 = 0, $1391 = 0, $1392 = 0, $1393 = 0;
 var $1394 = 0, $1395 = 0, $1396 = 0, $1397 = 0, $1398 = 0, $1399 = 0, $14 = 0, $140 = 0, $1400 = 0, $1401 = 0, $1402 = 0, $1403 = 0, $1404 = 0, $1405 = 0, $1406 = 0, $1407 = 0, $1408 = 0, $1409 = 0, $141 = 0, $1410 = 0;
 var $1411 = 0, $1412 = 0, $1413 = 0, $1414 = 0, $1415 = 0, $1416 = 0, $1417 = 0, $1418 = 0, $1419 = 0, $142 = 0, $1420 = 0, $1421 = 0, $1422 = 0, $1423 = 0, $1424 = 0, $1425 = 0, $1426 = 0, $1427 = 0, $1428 = 0, $1429 = 0;
 var $143 = 0, $1430 = 0, $1431 = 0, $1432 = 0, $1433 = 0, $1434 = 0, $1435 = 0, $1436 = 0, $1437 = 0, $1438 = 0, $1439 = 0, $144 = 0, $1440 = 0, $1441 = 0, $1442 = 0, $1443 = 0, $1444 = 0, $1445 = 0, $1446 = 0, $1447 = 0;
 var $1448 = 0, $1449 = 0, $145 = 0, $1450 = 0, $1451 = 0, $1452 = 0, $1453 = 0, $1454 = 0, $1455 = 0, $1456 = 0, $1457 = 0, $1458 = 0, $1459 = 0, $146 = 0, $1460 = 0, $1461 = 0, $1462 = 0, $1463 = 0, $1464 = 0, $1465 = 0;
 var $1466 = 0, $1467 = 0, $1468 = 0, $1469 = 0, $147 = 0, $1470 = 0, $1471 = 0, $1472 = 0, $1473 = 0, $1474 = 0, $1475 = 0, $1476 = 0, $1477 = 0, $1478 = 0, $1479 = 0, $148 = 0, $1480 = 0, $1481 = 0, $1482 = 0, $1483 = 0;
 var $1484 = 0, $1485 = 0, $1486 = 0, $1487 = 0, $1488 = 0, $1489 = 0, $149 = 0, $1490 = 0, $1491 = 0, $1492 = 0, $1493 = 0, $1494 = 0, $1495 = 0, $1496 = 0, $1497 = 0, $1498 = 0, $1499 = 0, $15 = 0, $150 = 0, $1500 = 0;
 var $1501 = 0, $1502 = 0, $1503 = 0, $1504 = 0, $1505 = 0, $1506 = 0, $1507 = 0, $1508 = 0, $1509 = 0, $151 = 0, $1510 = 0, $1511 = 0, $1512 = 0, $1513 = 0, $1514 = 0, $1515 = 0, $1516 = 0, $1517 = 0, $1518 = 0, $1519 = 0;
 var $152 = 0, $1520 = 0, $1521 = 0, $1522 = 0, $1523 = 0, $1524 = 0, $1525 = 0, $1526 = 0, $1527 = 0, $1528 = 0, $1529 = 0, $153 = 0, $1530 = 0, $1531 = 0, $1532 = 0, $1533 = 0, $1534 = 0, $1535 = 0, $1536 = 0, $1537 = 0;
 var $1538 = 0, $1539 = 0, $154 = 0, $1540 = 0, $1541 = 0, $1542 = 0, $1543 = 0, $1544 = 0, $1545 = 0, $1546 = 0, $1547 = 0, $1548 = 0, $1549 = 0, $155 = 0, $1550 = 0, $1551 = 0, $1552 = 0, $1553 = 0, $1554 = 0, $1555 = 0;
 var $1556 = 0, $1557 = 0, $1558 = 0, $1559 = 0, $156 = 0, $1560 = 0, $1561 = 0, $1562 = 0, $1563 = 0, $1564 = 0, $1565 = 0, $1566 = 0, $1567 = 0, $1568 = 0, $1569 = 0, $157 = 0, $1570 = 0, $1571 = 0, $1572 = 0, $1573 = 0;
 var $1574 = 0, $1575 = 0, $1576 = 0, $1577 = 0, $1578 = 0, $1579 = 0, $158 = 0, $1580 = 0, $1581 = 0, $1582 = 0, $1583 = 0, $1584 = 0, $1585 = 0, $1586 = 0, $1587 = 0, $1588 = 0, $1589 = 0, $159 = 0, $1590 = 0, $1591 = 0;
 var $1592 = 0, $1593 = 0, $1594 = 0, $1595 = 0, $1596 = 0, $1597 = 0, $1598 = 0, $1599 = 0, $16 = 0, $160 = 0, $1600 = 0, $1601 = 0, $1602 = 0, $1603 = 0, $1604 = 0, $1605 = 0, $1606 = 0, $1607 = 0, $1608 = 0, $1609 = 0;
 var $161 = 0, $1610 = 0, $1611 = 0, $1612 = 0, $1613 = 0, $1614 = 0, $1615 = 0, $1616 = 0, $1617 = 0, $1618 = 0, $1619 = 0, $162 = 0, $1620 = 0, $1621 = 0, $1622 = 0, $1623 = 0, $1624 = 0, $1625 = 0, $1626 = 0, $1627 = 0;
 var $1628 = 0, $1629 = 0, $163 = 0, $1630 = 0, $1631 = 0, $1632 = 0, $1633 = 0, $1634 = 0, $1635 = 0, $1636 = 0, $1637 = 0, $1638 = 0, $1639 = 0, $164 = 0, $1640 = 0, $1641 = 0, $1642 = 0, $1643 = 0, $1644 = 0, $1645 = 0;
 var $1646 = 0, $1647 = 0, $1648 = 0, $1649 = 0, $165 = 0, $1650 = 0, $1651 = 0, $1652 = 0, $1653 = 0, $1654 = 0, $1655 = 0, $1656 = 0, $1657 = 0, $1658 = 0, $1659 = 0, $166 = 0, $1660 = 0, $1661 = 0, $1662 = 0, $1663 = 0;
 var $1664 = 0, $1665 = 0, $1666 = 0, $1667 = 0, $1668 = 0, $1669 = 0, $167 = 0, $1670 = 0, $1671 = 0, $1672 = 0, $1673 = 0, $1674 = 0, $1675 = 0, $1676 = 0, $1677 = 0, $1678 = 0, $1679 = 0, $168 = 0, $1680 = 0, $1681 = 0;
 var $1682 = 0, $1683 = 0, $1684 = 0, $1685 = 0, $1686 = 0, $1687 = 0, $1688 = 0, $1689 = 0, $169 = 0, $1690 = 0, $1691 = 0, $1692 = 0, $1693 = 0, $1694 = 0, $1695 = 0, $1696 = 0, $1697 = 0, $1698 = 0, $1699 = 0, $17 = 0;
 var $170 = 0, $1700 = 0, $1701 = 0, $1702 = 0, $1703 = 0, $1704 = 0, $1705 = 0, $1706 = 0, $1707 = 0, $1708 = 0, $1709 = 0, $171 = 0, $1710 = 0, $1711 = 0, $1712 = 0, $1713 = 0, $1714 = 0, $1715 = 0, $1716 = 0, $1717 = 0;
 var $1718 = 0, $1719 = 0, $172 = 0, $1720 = 0, $1721 = 0, $1722 = 0, $1723 = 0, $1724 = 0, $1725 = 0, $1726 = 0, $1727 = 0, $1728 = 0, $1729 = 0, $173 = 0, $1730 = 0, $1731 = 0, $1732 = 0, $1733 = 0, $1734 = 0, $1735 = 0;
 var $1736 = 0, $1737 = 0, $1738 = 0, $1739 = 0, $174 = 0, $1740 = 0, $1741 = 0, $1742 = 0, $1743 = 0, $1744 = 0, $1745 = 0, $1746 = 0, $1747 = 0, $1748 = 0, $1749 = 0, $175 = 0, $1750 = 0, $1751 = 0, $1752 = 0, $1753 = 0;
 var $1754 = 0, $1755 = 0, $1756 = 0, $1757 = 0, $1758 = 0, $1759 = 0, $176 = 0, $1760 = 0, $1761 = 0, $1762 = 0, $1763 = 0, $1764 = 0, $1765 = 0, $1766 = 0, $1767 = 0, $1768 = 0, $1769 = 0, $177 = 0, $1770 = 0, $1771 = 0;
 var $1772 = 0, $1773 = 0, $1774 = 0, $1775 = 0, $1776 = 0, $1777 = 0, $1778 = 0, $1779 = 0, $178 = 0, $1780 = 0, $1781 = 0, $1782 = 0, $1783 = 0, $1784 = 0, $1785 = 0, $1786 = 0, $1787 = 0, $1788 = 0, $1789 = 0, $179 = 0;
 var $1790 = 0, $1791 = 0, $1792 = 0, $1793 = 0, $1794 = 0, $1795 = 0, $1796 = 0, $1797 = 0, $1798 = 0, $1799 = 0, $18 = 0, $180 = 0, $1800 = 0, $1801 = 0, $1802 = 0, $1803 = 0, $1804 = 0, $1805 = 0, $1806 = 0, $1807 = 0;
 var $1808 = 0, $1809 = 0, $181 = 0, $1810 = 0, $1811 = 0, $1812 = 0, $1813 = 0, $1814 = 0, $1815 = 0, $1816 = 0, $1817 = 0, $1818 = 0, $1819 = 0, $182 = 0, $1820 = 0, $1821 = 0, $1822 = 0, $1823 = 0, $1824 = 0, $1825 = 0;
 var $1826 = 0, $1827 = 0, $1828 = 0, $1829 = 0, $183 = 0, $1830 = 0, $1831 = 0, $1832 = 0, $1833 = 0, $1834 = 0, $1835 = 0, $1836 = 0, $1837 = 0, $1838 = 0, $1839 = 0, $184 = 0, $1840 = 0, $1841 = 0, $1842 = 0, $1843 = 0;
 var $1844 = 0, $1845 = 0, $1846 = 0, $1847 = 0, $1848 = 0, $1849 = 0, $185 = 0, $1850 = 0, $1851 = 0, $1852 = 0, $1853 = 0, $1854 = 0, $1855 = 0, $1856 = 0, $1857 = 0, $1858 = 0, $1859 = 0, $186 = 0, $1860 = 0, $1861 = 0;
 var $1862 = 0, $1863 = 0, $1864 = 0, $1865 = 0, $1866 = 0, $1867 = 0, $1868 = 0, $1869 = 0, $187 = 0, $1870 = 0, $1871 = 0, $1872 = 0, $1873 = 0, $1874 = 0, $1875 = 0, $1876 = 0, $1877 = 0, $1878 = 0, $1879 = 0, $188 = 0;
 var $1880 = 0, $1881 = 0, $1882 = 0, $1883 = 0, $1884 = 0, $1885 = 0, $1886 = 0, $1887 = 0, $1888 = 0, $1889 = 0, $189 = 0, $1890 = 0, $1891 = 0, $1892 = 0, $1893 = 0, $1894 = 0, $1895 = 0, $1896 = 0, $1897 = 0, $1898 = 0;
 var $1899 = 0, $19 = 0, $190 = 0, $1900 = 0, $1901 = 0, $1902 = 0, $1903 = 0, $1904 = 0, $1905 = 0, $1906 = 0, $1907 = 0, $1908 = 0, $1909 = 0, $191 = 0, $1910 = 0, $1911 = 0, $1912 = 0, $1913 = 0, $1914 = 0, $1915 = 0;
 var $1916 = 0, $1917 = 0, $1918 = 0, $1919 = 0, $192 = 0, $1920 = 0, $1921 = 0, $1922 = 0, $1923 = 0, $1924 = 0, $1925 = 0, $1926 = 0, $1927 = 0, $1928 = 0, $1929 = 0, $193 = 0, $1930 = 0, $1931 = 0, $1932 = 0, $1933 = 0;
 var $1934 = 0, $1935 = 0, $1936 = 0, $1937 = 0, $1938 = 0, $1939 = 0, $194 = 0, $1940 = 0, $1941 = 0, $1942 = 0, $1943 = 0, $1944 = 0, $1945 = 0, $1946 = 0, $1947 = 0, $1948 = 0, $1949 = 0, $195 = 0, $1950 = 0, $1951 = 0;
 var $1952 = 0, $1953 = 0, $1954 = 0, $1955 = 0, $1956 = 0, $1957 = 0, $1958 = 0, $1959 = 0, $196 = 0, $1960 = 0, $1961 = 0, $1962 = 0, $1963 = 0, $1964 = 0, $1965 = 0, $1966 = 0, $1967 = 0, $1968 = 0, $1969 = 0, $197 = 0;
 var $1970 = 0, $1971 = 0, $1972 = 0, $1973 = 0, $1974 = 0, $1975 = 0, $1976 = 0, $1977 = 0, $1978 = 0, $1979 = 0, $198 = 0, $1980 = 0, $1981 = 0, $1982 = 0, $1983 = 0, $1984 = 0, $1985 = 0, $1986 = 0, $1987 = 0, $1988 = 0;
 var $1989 = 0, $199 = 0, $1990 = 0, $1991 = 0, $1992 = 0, $1993 = 0, $1994 = 0, $1995 = 0, $1996 = 0, $1997 = 0, $1998 = 0, $1999 = 0, $2 = 0, $20 = 0, $200 = 0, $2000 = 0, $2001 = 0, $2002 = 0, $2003 = 0, $2004 = 0;
 var $2005 = 0, $2006 = 0, $2007 = 0, $2008 = 0, $2009 = 0, $201 = 0, $2010 = 0, $2011 = 0, $2012 = 0, $2013 = 0, $2014 = 0, $2015 = 0, $2016 = 0, $2017 = 0, $2018 = 0, $2019 = 0, $202 = 0, $2020 = 0, $2021 = 0, $2022 = 0;
 var $2023 = 0, $2024 = 0, $2025 = 0, $2026 = 0, $2027 = 0, $2028 = 0, $2029 = 0, $203 = 0, $2030 = 0, $2031 = 0, $2032 = 0, $2033 = 0, $2034 = 0, $2035 = 0, $2036 = 0, $2037 = 0, $2038 = 0, $2039 = 0, $204 = 0, $2040 = 0;
 var $2041 = 0, $2042 = 0, $2043 = 0, $2044 = 0, $2045 = 0, $2046 = 0, $2047 = 0, $2048 = 0, $2049 = 0, $205 = 0, $2050 = 0, $2051 = 0, $2052 = 0, $2053 = 0, $2054 = 0, $2055 = 0, $2056 = 0, $2057 = 0, $2058 = 0, $2059 = 0;
 var $206 = 0, $2060 = 0, $2061 = 0, $2062 = 0, $2063 = 0, $2064 = 0, $2065 = 0.0, $2066 = 0, $2067 = 0, $2068 = 0, $2069 = 0, $207 = 0, $2070 = 0, $2071 = 0, $2072 = 0, $2073 = 0, $2074 = 0, $2075 = 0, $2076 = 0, $2077 = 0;
 var $2078 = 0, $2079 = 0, $208 = 0, $2080 = 0, $2081 = 0, $2082 = 0, $2083 = 0, $2084 = 0, $2085 = 0, $2086 = 0, $2087 = 0, $2088 = 0, $2089 = 0, $209 = 0, $2090 = 0, $2091 = 0, $2092 = 0, $2093 = 0, $2094 = 0, $2095 = 0;
 var $2096 = 0, $2097 = 0, $2098 = 0, $2099 = 0, $21 = 0, $210 = 0, $2100 = 0, $2101 = 0, $2102 = 0, $2103 = 0, $2104 = 0, $2105 = 0, $2106 = 0, $2107 = 0, $2108 = 0, $2109 = 0, $211 = 0, $2110 = 0, $2111 = 0, $2112 = 0;
 var $2113 = 0, $2114 = 0, $2115 = 0, $2116 = 0, $2117 = 0, $2118 = 0, $2119 = 0, $212 = 0, $2120 = 0, $2121 = 0, $2122 = 0, $2123 = 0, $2124 = 0, $2125 = 0, $2126 = 0, $2127 = 0, $2128 = 0, $2129 = 0, $213 = 0, $2130 = 0;
 var $2131 = 0, $2132 = 0, $2133 = 0, $2134 = 0, $2135 = 0, $2136 = 0, $2137 = 0, $2138 = 0, $2139 = 0, $214 = 0, $2140 = 0, $2141 = 0, $2142 = 0, $2143 = 0, $2144 = 0, $2145 = 0, $2146 = 0, $2147 = 0, $2148 = 0, $2149 = 0;
 var $215 = 0, $2150 = 0, $2151 = 0, $2152 = 0, $2153 = 0, $2154 = 0, $2155 = 0, $2156 = 0, $2157 = 0, $2158 = 0, $2159 = 0, $216 = 0, $2160 = 0, $2161 = 0, $2162 = 0, $2163 = 0, $2164 = 0, $2165 = 0, $2166 = 0, $2167 = 0;
 var $2168 = 0, $2169 = 0, $217 = 0, $2170 = 0, $2171 = 0, $2172 = 0, $2173 = 0, $2174 = 0, $2175 = 0, $2176 = 0, $2177 = 0, $2178 = 0, $2179 = 0, $218 = 0, $2180 = 0, $2181 = 0, $2182 = 0, $2183 = 0, $2184 = 0, $2185 = 0;
 var $2186 = 0, $2187 = 0, $2188 = 0, $2189 = 0, $219 = 0, $2190 = 0, $2191 = 0, $2192 = 0, $2193 = 0, $2194 = 0, $2195 = 0, $2196 = 0, $2197 = 0, $2198 = 0, $2199 = 0, $22 = 0, $220 = 0, $2200 = 0, $2201 = 0, $2202 = 0;
 var $2203 = 0, $2204 = 0, $2205 = 0, $2206 = 0, $2207 = 0, $2208 = 0, $2209 = 0, $221 = 0, $2210 = 0, $2211 = 0, $2212 = 0, $2213 = 0, $2214 = 0, $2215 = 0, $2216 = 0, $2217 = 0, $2218 = 0, $2219 = 0, $222 = 0, $2220 = 0;
 var $2221 = 0, $2222 = 0, $2223 = 0, $2224 = 0, $2225 = 0, $2226 = 0, $2227 = 0, $2228 = 0, $2229 = 0, $223 = 0, $2230 = 0, $2231 = 0, $2232 = 0, $2233 = 0, $2234 = 0, $2235 = 0, $2236 = 0, $2237 = 0, $2238 = 0, $2239 = 0;
 var $224 = 0, $2240 = 0, $2241 = 0, $2242 = 0, $2243 = 0, $2244 = 0, $2245 = 0, $2246 = 0, $2247 = 0, $2248 = 0, $2249 = 0, $225 = 0, $2250 = 0, $2251 = 0, $2252 = 0, $2253 = 0, $2254 = 0, $2255 = 0, $2256 = 0, $2257 = 0;
 var $2258 = 0, $2259 = 0, $226 = 0, $2260 = 0, $2261 = 0, $2262 = 0, $2263 = 0, $2264 = 0, $2265 = 0, $2266 = 0, $2267 = 0, $2268 = 0, $2269 = 0, $227 = 0, $2270 = 0, $2271 = 0, $2272 = 0, $2273 = 0, $2274 = 0, $2275 = 0;
 var $2276 = 0, $2277 = 0, $2278 = 0, $2279 = 0, $228 = 0, $2280 = 0, $2281 = 0, $2282 = 0, $2283 = 0, $2284 = 0, $2285 = 0, $2286 = 0, $2287 = 0, $2288 = 0, $2289 = 0, $229 = 0, $2290 = 0, $2291 = 0, $2292 = 0, $2293 = 0;
 var $2294 = 0, $2295 = 0, $2296 = 0, $2297 = 0, $2298 = 0, $2299 = 0, $23 = 0, $230 = 0, $2300 = 0, $2301 = 0, $2302 = 0, $2303 = 0, $2304 = 0, $2305 = 0, $2306 = 0, $2307 = 0, $2308 = 0, $2309 = 0, $231 = 0, $2310 = 0;
 var $2311 = 0, $2312 = 0, $2313 = 0, $2314 = 0, $2315 = 0, $2316 = 0, $2317 = 0, $2318 = 0, $2319 = 0, $232 = 0, $2320 = 0, $2321 = 0, $2322 = 0, $2323 = 0, $2324 = 0, $2325 = 0, $2326 = 0, $2327 = 0, $2328 = 0, $2329 = 0;
 var $233 = 0, $2330 = 0, $2331 = 0, $2332 = 0, $2333 = 0, $2334 = 0, $2335 = 0, $2336 = 0, $2337 = 0, $2338 = 0, $2339 = 0, $234 = 0, $2340 = 0, $2341 = 0, $2342 = 0, $2343 = 0, $2344 = 0, $2345 = 0, $2346 = 0, $2347 = 0;
 var $2348 = 0, $2349 = 0, $235 = 0, $2350 = 0, $2351 = 0, $2352 = 0, $2353 = 0, $2354 = 0, $2355 = 0, $2356 = 0, $2357 = 0, $2358 = 0, $2359 = 0, $236 = 0, $2360 = 0, $2361 = 0, $2362 = 0, $2363 = 0, $2364 = 0, $2365 = 0;
 var $2366 = 0, $2367 = 0, $2368 = 0, $2369 = 0, $237 = 0, $2370 = 0, $2371 = 0, $2372 = 0, $2373 = 0, $2374 = 0, $2375 = 0, $2376 = 0, $2377 = 0, $2378 = 0, $2379 = 0, $238 = 0, $2380 = 0, $2381 = 0, $2382 = 0, $2383 = 0;
 var $2384 = 0, $2385 = 0, $2386 = 0, $2387 = 0, $2388 = 0, $2389 = 0, $239 = 0, $2390 = 0, $2391 = 0, $2392 = 0, $2393 = 0, $2394 = 0, $2395 = 0, $2396 = 0, $2397 = 0, $2398 = 0, $2399 = 0, $24 = 0, $240 = 0, $2400 = 0;
 var $2401 = 0, $2402 = 0, $2403 = 0, $2404 = 0, $2405 = 0, $2406 = 0, $2407 = 0, $2408 = 0, $2409 = 0, $241 = 0, $2410 = 0, $2411 = 0, $2412 = 0, $2413 = 0, $2414 = 0, $2415 = 0, $2416 = 0, $2417 = 0, $2418 = 0, $2419 = 0;
 var $242 = 0, $2420 = 0, $2421 = 0, $2422 = 0, $2423 = 0, $2424 = 0, $2425 = 0, $2426 = 0, $2427 = 0, $2428 = 0, $2429 = 0, $243 = 0, $2430 = 0, $2431 = 0, $2432 = 0, $2433 = 0, $2434 = 0, $2435 = 0, $2436 = 0, $2437 = 0;
 var $2438 = 0, $2439 = 0, $244 = 0, $2440 = 0, $2441 = 0, $2442 = 0, $2443 = 0, $2444 = 0, $2445 = 0, $2446 = 0, $2447 = 0, $2448 = 0, $2449 = 0, $245 = 0, $2450 = 0, $2451 = 0, $2452 = 0, $2453 = 0, $2454 = 0, $2455 = 0;
 var $2456 = 0, $2457 = 0, $2458 = 0, $2459 = 0, $246 = 0, $2460 = 0, $2461 = 0, $2462 = 0, $2463 = 0, $2464 = 0, $2465 = 0, $2466 = 0, $2467 = 0, $2468 = 0, $2469 = 0, $247 = 0, $2470 = 0, $2471 = 0, $2472 = 0, $2473 = 0;
 var $2474 = 0, $2475 = 0, $2476 = 0, $2477 = 0, $2478 = 0, $2479 = 0, $248 = 0, $2480 = 0, $2481 = 0, $2482 = 0, $2483 = 0, $2484 = 0, $2485 = 0, $2486 = 0, $2487 = 0, $2488 = 0, $2489 = 0, $249 = 0, $2490 = 0, $2491 = 0;
 var $2492 = 0, $2493 = 0, $2494 = 0, $2495 = 0, $2496 = 0, $2497 = 0, $2498 = 0, $2499 = 0, $25 = 0, $250 = 0, $2500 = 0, $2501 = 0, $2502 = 0, $2503 = 0, $2504 = 0, $2505 = 0, $2506 = 0, $2507 = 0, $2508 = 0, $2509 = 0;
 var $251 = 0, $2510 = 0, $2511 = 0, $2512 = 0, $2513 = 0, $2514 = 0, $2515 = 0, $2516 = 0, $2517 = 0, $2518 = 0, $2519 = 0, $252 = 0, $2520 = 0, $2521 = 0, $2522 = 0, $2523 = 0, $2524 = 0, $2525 = 0, $2526 = 0, $2527 = 0;
 var $2528 = 0, $2529 = 0, $253 = 0, $2530 = 0, $2531 = 0, $2532 = 0, $2533 = 0, $2534 = 0, $2535 = 0, $2536 = 0, $2537 = 0, $2538 = 0, $2539 = 0, $254 = 0, $2540 = 0, $2541 = 0, $2542 = 0, $2543 = 0, $2544 = 0, $2545 = 0;
 var $2546 = 0, $2547 = 0, $2548 = 0, $2549 = 0, $255 = 0, $2550 = 0, $2551 = 0, $2552 = 0, $2553 = 0, $2554 = 0, $2555 = 0, $2556 = 0, $2557 = 0, $2558 = 0, $2559 = 0, $256 = 0, $2560 = 0, $2561 = 0, $2562 = 0, $2563 = 0;
 var $2564 = 0, $2565 = 0, $2566 = 0, $2567 = 0, $2568 = 0, $2569 = 0, $257 = 0, $2570 = 0, $2571 = 0, $2572 = 0, $2573 = 0, $2574 = 0, $2575 = 0, $2576 = 0, $2577 = 0, $2578 = 0, $2579 = 0, $258 = 0, $2580 = 0, $2581 = 0;
 var $2582 = 0, $2583 = 0, $2584 = 0, $2585 = 0, $2586 = 0, $2587 = 0, $2588 = 0, $2589 = 0, $259 = 0, $2590 = 0, $2591 = 0, $2592 = 0, $2593 = 0, $2594 = 0, $2595 = 0, $2596 = 0, $2597 = 0, $2598 = 0, $2599 = 0, $26 = 0;
 var $260 = 0, $2600 = 0, $2601 = 0, $2602 = 0, $2603 = 0, $2604 = 0, $2605 = 0, $2606 = 0, $2607 = 0, $2608 = 0, $2609 = 0, $261 = 0, $2610 = 0, $2611 = 0, $2612 = 0, $2613 = 0, $2614 = 0, $2615 = 0, $2616 = 0, $2617 = 0;
 var $2618 = 0, $2619 = 0, $262 = 0, $2620 = 0, $2621 = 0, $2622 = 0, $2623 = 0, $2624 = 0, $2625 = 0, $2626 = 0, $2627 = 0, $2628 = 0, $2629 = 0, $263 = 0, $2630 = 0, $2631 = 0, $2632 = 0, $2633 = 0, $2634 = 0, $2635 = 0;
 var $2636 = 0, $2637 = 0, $2638 = 0, $2639 = 0, $264 = 0, $2640 = 0, $2641 = 0, $2642 = 0, $2643 = 0, $2644 = 0, $2645 = 0, $2646 = 0, $2647 = 0, $2648 = 0, $2649 = 0, $265 = 0, $2650 = 0, $2651 = 0, $2652 = 0, $2653 = 0;
 var $2654 = 0, $2655 = 0, $2656 = 0, $2657 = 0, $2658 = 0, $2659 = 0, $266 = 0, $2660 = 0, $2661 = 0, $2662 = 0, $2663 = 0, $2664 = 0, $2665 = 0, $2666 = 0, $2667 = 0, $2668 = 0, $2669 = 0, $267 = 0, $2670 = 0, $2671 = 0;
 var $2672 = 0, $2673 = 0, $2674 = 0, $2675 = 0, $2676 = 0, $2677 = 0, $2678 = 0, $2679 = 0, $268 = 0, $2680 = 0, $2681 = 0, $2682 = 0, $2683 = 0, $2684 = 0, $2685 = 0, $2686 = 0, $2687 = 0, $2688 = 0, $2689 = 0, $269 = 0;
 var $2690 = 0, $2691 = 0, $2692 = 0, $2693 = 0, $2694 = 0, $2695 = 0, $2696 = 0, $2697 = 0, $2698 = 0, $2699 = 0, $27 = 0, $270 = 0, $2700 = 0, $2701 = 0, $2702 = 0, $2703 = 0, $2704 = 0, $2705 = 0, $2706 = 0, $2707 = 0;
 var $2708 = 0, $2709 = 0, $271 = 0, $2710 = 0, $2711 = 0, $2712 = 0, $2713 = 0, $2714 = 0, $2715 = 0, $2716 = 0, $2717 = 0, $2718 = 0, $2719 = 0, $272 = 0, $2720 = 0, $2721 = 0, $2722 = 0, $2723 = 0, $2724 = 0, $2725 = 0;
 var $2726 = 0, $2727 = 0, $2728 = 0, $2729 = 0, $273 = 0, $2730 = 0, $2731 = 0, $2732 = 0, $2733 = 0, $2734 = 0, $2735 = 0, $2736 = 0, $2737 = 0, $2738 = 0, $2739 = 0, $274 = 0, $2740 = 0, $2741 = 0, $2742 = 0, $2743 = 0;
 var $2744 = 0, $2745 = 0, $2746 = 0, $2747 = 0, $2748 = 0, $2749 = 0, $275 = 0, $2750 = 0, $2751 = 0, $2752 = 0, $2753 = 0, $2754 = 0, $2755 = 0, $2756 = 0, $2757 = 0, $2758 = 0, $2759 = 0, $276 = 0, $2760 = 0, $2761 = 0;
 var $2762 = 0, $2763 = 0, $2764 = 0, $2765 = 0, $2766 = 0, $2767 = 0, $2768 = 0, $2769 = 0, $277 = 0, $2770 = 0, $2771 = 0, $2772 = 0, $2773 = 0, $2774 = 0, $2775 = 0, $2776 = 0, $2777 = 0, $2778 = 0, $2779 = 0, $278 = 0;
 var $2780 = 0, $2781 = 0, $2782 = 0, $2783 = 0, $2784 = 0, $2785 = 0, $2786 = 0, $2787 = 0, $2788 = 0, $2789 = 0, $279 = 0, $2790 = 0, $2791 = 0, $2792 = 0, $2793 = 0, $2794 = 0, $2795 = 0, $2796 = 0, $2797 = 0, $2798 = 0;
 var $2799 = 0, $28 = 0, $280 = 0, $2800 = 0, $2801 = 0, $2802 = 0, $2803 = 0, $2804 = 0, $2805 = 0, $2806 = 0, $2807 = 0, $2808 = 0, $2809 = 0, $281 = 0, $2810 = 0, $2811 = 0, $2812 = 0, $2813 = 0, $2814 = 0, $2815 = 0;
 var $2816 = 0, $2817 = 0, $2818 = 0, $2819 = 0, $282 = 0, $2820 = 0, $2821 = 0, $2822 = 0, $2823 = 0, $2824 = 0, $2825 = 0, $2826 = 0, $2827 = 0, $2828 = 0, $2829 = 0, $283 = 0, $2830 = 0, $2831 = 0, $2832 = 0, $2833 = 0;
 var $2834 = 0, $2835 = 0, $2836 = 0, $2837 = 0, $2838 = 0, $2839 = 0, $284 = 0, $2840 = 0, $2841 = 0, $2842 = 0, $2843 = 0, $2844 = 0, $2845 = 0, $2846 = 0, $2847 = 0, $2848 = 0, $2849 = 0, $285 = 0, $2850 = 0, $2851 = 0;
 var $2852 = 0, $2853 = 0, $2854 = 0, $2855 = 0, $2856 = 0, $2857 = 0, $2858 = 0, $2859 = 0, $286 = 0, $2860 = 0, $2861 = 0, $2862 = 0, $2863 = 0, $2864 = 0, $2865 = 0, $2866 = 0, $2867 = 0, $2868 = 0, $2869 = 0, $287 = 0;
 var $2870 = 0, $2871 = 0, $2872 = 0, $2873 = 0, $2874 = 0, $2875 = 0, $2876 = 0, $2877 = 0, $2878 = 0, $2879 = 0, $288 = 0, $2880 = 0, $2881 = 0, $2882 = 0, $2883 = 0, $2884 = 0, $2885 = 0, $2886 = 0, $2887 = 0, $2888 = 0;
 var $2889 = 0, $289 = 0, $2890 = 0, $2891 = 0, $2892 = 0, $2893 = 0, $2894 = 0, $2895 = 0, $2896 = 0, $2897 = 0, $2898 = 0, $2899 = 0, $29 = 0, $290 = 0, $2900 = 0, $2901 = 0, $2902 = 0, $2903 = 0, $2904 = 0, $2905 = 0;
 var $2906 = 0, $2907 = 0, $2908 = 0, $2909 = 0, $291 = 0, $2910 = 0, $2911 = 0, $2912 = 0, $2913 = 0, $2914 = 0, $2915 = 0, $2916 = 0, $2917 = 0, $2918 = 0, $2919 = 0, $292 = 0, $2920 = 0, $2921 = 0, $2922 = 0, $2923 = 0;
 var $2924 = 0, $2925 = 0, $2926 = 0, $2927 = 0, $2928 = 0, $2929 = 0, $293 = 0, $2930 = 0, $2931 = 0, $2932 = 0, $2933 = 0, $2934 = 0, $2935 = 0, $2936 = 0, $2937 = 0, $2938 = 0, $2939 = 0, $294 = 0, $2940 = 0, $2941 = 0;
 var $2942 = 0, $2943 = 0, $2944 = 0, $2945 = 0, $2946 = 0, $2947 = 0, $2948 = 0, $2949 = 0, $295 = 0, $2950 = 0, $2951 = 0, $2952 = 0, $2953 = 0, $2954 = 0, $2955 = 0, $2956 = 0, $2957 = 0, $2958 = 0, $2959 = 0, $296 = 0;
 var $2960 = 0, $2961 = 0, $2962 = 0, $2963 = 0, $2964 = 0, $2965 = 0, $2966 = 0, $2967 = 0, $2968 = 0, $2969 = 0, $297 = 0, $2970 = 0, $2971 = 0, $2972 = 0, $2973 = 0, $2974 = 0, $2975 = 0, $2976 = 0, $2977 = 0, $2978 = 0;
 var $2979 = 0, $298 = 0, $2980 = 0, $2981 = 0, $2982 = 0, $2983 = 0, $2984 = 0, $2985 = 0, $2986 = 0, $2987 = 0, $2988 = 0, $2989 = 0, $299 = 0, $2990 = 0, $2991 = 0, $2992 = 0, $2993 = 0, $2994 = 0, $2995 = 0, $2996 = 0;
 var $2997 = 0, $2998 = 0, $2999 = 0, $3 = 0, $30 = 0, $300 = 0, $3000 = 0, $3001 = 0, $3002 = 0, $3003 = 0, $3004 = 0, $3005 = 0, $3006 = 0, $3007 = 0, $3008 = 0, $3009 = 0, $301 = 0, $3010 = 0, $3011 = 0, $3012 = 0;
 var $3013 = 0, $3014 = 0, $3015 = 0, $3016 = 0, $3017 = 0, $3018 = 0, $3019 = 0, $302 = 0, $3020 = 0, $3021 = 0, $3022 = 0, $3023 = 0, $3024 = 0, $3025 = 0, $3026 = 0, $3027 = 0, $3028 = 0, $3029 = 0, $303 = 0, $3030 = 0;
 var $3031 = 0, $3032 = 0, $3033 = 0, $3034 = 0, $3035 = 0, $3036 = 0, $3037 = 0, $3038 = 0, $3039 = 0, $304 = 0, $3040 = 0, $3041 = 0, $3042 = 0, $3043 = 0, $3044 = 0, $3045 = 0, $3046 = 0, $3047 = 0, $3048 = 0, $3049 = 0;
 var $305 = 0, $3050 = 0, $3051 = 0, $3052 = 0, $3053 = 0, $3054 = 0, $3055 = 0, $3056 = 0, $3057 = 0, $3058 = 0, $3059 = 0, $306 = 0, $3060 = 0, $3061 = 0, $3062 = 0, $3063 = 0, $3064 = 0, $3065 = 0, $3066 = 0, $3067 = 0;
 var $3068 = 0, $3069 = 0, $307 = 0, $3070 = 0, $3071 = 0, $3072 = 0, $3073 = 0, $3074 = 0, $3075 = 0, $3076 = 0, $3077 = 0, $3078 = 0, $3079 = 0, $308 = 0, $3080 = 0, $3081 = 0, $3082 = 0, $3083 = 0, $3084 = 0, $3085 = 0;
 var $3086 = 0, $3087 = 0, $3088 = 0, $3089 = 0, $309 = 0, $3090 = 0, $3091 = 0, $3092 = 0, $3093 = 0, $3094 = 0, $3095 = 0, $3096 = 0, $3097 = 0, $3098 = 0, $3099 = 0, $31 = 0, $310 = 0, $3100 = 0, $3101 = 0, $3102 = 0;
 var $3103 = 0, $3104 = 0, $3105 = 0, $3106 = 0, $3107 = 0, $3108 = 0, $3109 = 0, $311 = 0, $3110 = 0, $3111 = 0, $3112 = 0, $3113 = 0, $3114 = 0, $3115 = 0, $3116 = 0, $3117 = 0, $3118 = 0, $3119 = 0, $312 = 0, $3120 = 0;
 var $3121 = 0, $3122 = 0, $3123 = 0, $3124 = 0, $3125 = 0, $3126 = 0, $3127 = 0, $3128 = 0, $3129 = 0, $313 = 0, $3130 = 0, $3131 = 0, $3132 = 0, $3133 = 0, $3134 = 0, $3135 = 0, $3136 = 0, $3137 = 0, $3138 = 0, $3139 = 0;
 var $314 = 0, $3140 = 0, $3141 = 0, $3142 = 0, $3143 = 0, $3144 = 0, $3145 = 0, $3146 = 0, $3147 = 0, $3148 = 0, $3149 = 0, $315 = 0, $3150 = 0, $3151 = 0, $3152 = 0, $3153 = 0, $3154 = 0, $3155 = 0, $3156 = 0, $3157 = 0;
 var $3158 = 0, $3159 = 0, $316 = 0, $3160 = 0, $3161 = 0, $3162 = 0, $3163 = 0, $3164 = 0, $3165 = 0, $3166 = 0, $3167 = 0, $3168 = 0, $3169 = 0, $317 = 0, $3170 = 0, $3171 = 0, $3172 = 0, $3173 = 0, $3174 = 0, $3175 = 0;
 var $3176 = 0, $3177 = 0, $3178 = 0, $3179 = 0, $318 = 0, $3180 = 0, $3181 = 0, $3182 = 0, $3183 = 0, $3184 = 0, $3185 = 0, $3186 = 0, $3187 = 0, $3188 = 0, $3189 = 0, $319 = 0, $3190 = 0, $3191 = 0, $3192 = 0, $3193 = 0;
 var $3194 = 0, $3195 = 0, $3196 = 0, $3197 = 0, $3198 = 0, $3199 = 0, $32 = 0, $320 = 0, $3200 = 0, $3201 = 0, $3202 = 0, $3203 = 0, $3204 = 0, $3205 = 0, $3206 = 0, $3207 = 0, $3208 = 0, $3209 = 0, $321 = 0, $3210 = 0;
 var $3211 = 0, $3212 = 0, $3213 = 0, $3214 = 0, $3215 = 0, $3216 = 0, $3217 = 0, $3218 = 0, $3219 = 0, $322 = 0, $3220 = 0, $3221 = 0, $3222 = 0, $3223 = 0, $3224 = 0, $3225 = 0, $3226 = 0, $3227 = 0, $3228 = 0, $3229 = 0;
 var $323 = 0, $3230 = 0, $3231 = 0, $3232 = 0, $3233 = 0, $3234 = 0, $3235 = 0, $3236 = 0, $3237 = 0, $3238 = 0, $3239 = 0, $324 = 0, $3240 = 0, $3241 = 0, $3242 = 0, $3243 = 0, $3244 = 0, $3245 = 0, $3246 = 0, $3247 = 0;
 var $3248 = 0, $3249 = 0, $325 = 0, $3250 = 0, $3251 = 0, $3252 = 0, $3253 = 0, $3254 = 0, $3255 = 0, $3256 = 0, $3257 = 0, $3258 = 0, $3259 = 0, $326 = 0, $3260 = 0, $3261 = 0, $3262 = 0, $3263 = 0, $3264 = 0, $3265 = 0;
 var $3266 = 0, $3267 = 0, $3268 = 0, $3269 = 0, $327 = 0, $3270 = 0, $3271 = 0, $3272 = 0, $3273 = 0, $3274 = 0, $3275 = 0, $3276 = 0, $3277 = 0, $3278 = 0, $3279 = 0, $328 = 0, $3280 = 0, $3281 = 0, $3282 = 0, $3283 = 0;
 var $3284 = 0, $3285 = 0, $3286 = 0, $3287 = 0, $3288 = 0, $3289 = 0, $329 = 0, $3290 = 0, $3291 = 0, $3292 = 0, $3293 = 0, $3294 = 0, $3295 = 0, $3296 = 0, $3297 = 0, $3298 = 0, $3299 = 0, $33 = 0, $330 = 0, $3300 = 0;
 var $3301 = 0, $3302 = 0, $3303 = 0, $3304 = 0, $3305 = 0, $3306 = 0, $3307 = 0, $3308 = 0, $3309 = 0, $331 = 0, $3310 = 0, $3311 = 0, $3312 = 0, $3313 = 0, $3314 = 0, $3315 = 0, $3316 = 0, $3317 = 0, $3318 = 0, $3319 = 0;
 var $332 = 0, $3320 = 0, $3321 = 0, $3322 = 0, $3323 = 0, $3324 = 0, $3325 = 0, $3326 = 0, $3327 = 0, $3328 = 0, $3329 = 0, $333 = 0, $3330 = 0, $3331 = 0, $3332 = 0, $3333 = 0, $3334 = 0, $3335 = 0, $3336 = 0, $3337 = 0;
 var $3338 = 0, $3339 = 0, $334 = 0, $3340 = 0, $3341 = 0, $3342 = 0, $3343 = 0, $3344 = 0, $3345 = 0, $3346 = 0, $3347 = 0, $3348 = 0, $3349 = 0, $335 = 0, $3350 = 0, $3351 = 0, $3352 = 0, $3353 = 0, $3354 = 0, $3355 = 0;
 var $3356 = 0, $3357 = 0, $3358 = 0, $3359 = 0, $336 = 0, $3360 = 0, $3361 = 0, $3362 = 0, $3363 = 0, $3364 = 0, $3365 = 0, $3366 = 0, $3367 = 0, $3368 = 0, $3369 = 0, $337 = 0, $3370 = 0, $3371 = 0, $3372 = 0, $3373 = 0;
 var $3374 = 0, $3375 = 0, $3376 = 0, $3377 = 0, $3378 = 0, $3379 = 0, $338 = 0, $3380 = 0, $3381 = 0, $3382 = 0, $3383 = 0, $3384 = 0, $3385 = 0, $3386 = 0, $3387 = 0, $3388 = 0, $3389 = 0, $339 = 0, $3390 = 0, $3391 = 0;
 var $3392 = 0, $3393 = 0, $3394 = 0, $3395 = 0, $3396 = 0, $3397 = 0, $3398 = 0, $3399 = 0, $34 = 0, $340 = 0, $3400 = 0, $3401 = 0, $3402 = 0, $3403 = 0, $3404 = 0, $3405 = 0, $3406 = 0, $3407 = 0, $3408 = 0, $3409 = 0;
 var $341 = 0, $3410 = 0, $3411 = 0, $3412 = 0, $3413 = 0, $3414 = 0, $3415 = 0, $3416 = 0, $3417 = 0, $3418 = 0, $3419 = 0, $342 = 0, $3420 = 0, $3421 = 0, $3422 = 0, $3423 = 0, $3424 = 0, $3425 = 0, $3426 = 0, $3427 = 0;
 var $3428 = 0, $3429 = 0, $343 = 0, $3430 = 0, $3431 = 0, $3432 = 0, $3433 = 0, $3434 = 0, $3435 = 0, $3436 = 0, $3437 = 0, $3438 = 0, $3439 = 0, $344 = 0, $3440 = 0, $3441 = 0, $3442 = 0, $3443 = 0, $3444 = 0, $3445 = 0;
 var $3446 = 0, $3447 = 0, $3448 = 0, $3449 = 0, $345 = 0, $3450 = 0, $3451 = 0, $3452 = 0, $3453 = 0, $3454 = 0, $3455 = 0, $3456 = 0, $3457 = 0, $3458 = 0, $3459 = 0, $346 = 0, $3460 = 0, $3461 = 0, $3462 = 0, $3463 = 0;
 var $3464 = 0, $3465 = 0, $3466 = 0, $3467 = 0, $3468 = 0, $3469 = 0, $347 = 0, $3470 = 0, $3471 = 0, $3472 = 0, $3473 = 0, $3474 = 0, $3475 = 0, $3476 = 0, $3477 = 0, $3478 = 0, $3479 = 0, $348 = 0, $3480 = 0, $3481 = 0;
 var $3482 = 0, $3483 = 0, $3484 = 0, $3485 = 0, $3486 = 0, $3487 = 0, $3488 = 0, $3489 = 0, $349 = 0, $3490 = 0, $3491 = 0, $3492 = 0, $3493 = 0, $3494 = 0, $3495 = 0, $3496 = 0, $3497 = 0, $3498 = 0, $3499 = 0, $35 = 0;
 var $350 = 0, $3500 = 0, $3501 = 0, $3502 = 0, $3503 = 0, $3504 = 0, $3505 = 0, $3506 = 0, $3507 = 0, $3508 = 0, $3509 = 0, $351 = 0, $3510 = 0, $3511 = 0, $3512 = 0, $3513 = 0, $3514 = 0, $3515 = 0, $3516 = 0, $3517 = 0;
 var $3518 = 0, $3519 = 0, $352 = 0, $3520 = 0, $3521 = 0, $3522 = 0, $3523 = 0, $3524 = 0, $3525 = 0, $3526 = 0, $3527 = 0, $3528 = 0, $3529 = 0, $353 = 0, $3530 = 0, $3531 = 0, $3532 = 0, $3533 = 0, $3534 = 0, $3535 = 0;
 var $3536 = 0, $3537 = 0, $3538 = 0, $3539 = 0, $354 = 0, $3540 = 0, $3541 = 0, $3542 = 0, $3543 = 0, $3544 = 0, $3545 = 0, $3546 = 0, $3547 = 0, $3548 = 0, $3549 = 0, $355 = 0, $3550 = 0, $3551 = 0, $3552 = 0, $3553 = 0;
 var $3554 = 0, $3555 = 0, $3556 = 0, $3557 = 0, $3558 = 0, $3559 = 0, $356 = 0, $3560 = 0, $3561 = 0, $3562 = 0, $3563 = 0, $3564 = 0, $3565 = 0, $3566 = 0, $3567 = 0, $3568 = 0, $3569 = 0, $357 = 0, $3570 = 0, $3571 = 0;
 var $3572 = 0, $3573 = 0, $3574 = 0, $3575 = 0, $3576 = 0, $3577 = 0, $3578 = 0, $3579 = 0, $358 = 0, $3580 = 0, $3581 = 0, $3582 = 0, $3583 = 0, $3584 = 0, $3585 = 0, $3586 = 0, $3587 = 0, $3588 = 0, $3589 = 0, $359 = 0;
 var $3590 = 0, $3591 = 0, $3592 = 0, $3593 = 0, $3594 = 0, $3595 = 0, $3596 = 0, $3597 = 0, $3598 = 0, $3599 = 0, $36 = 0, $360 = 0, $3600 = 0, $3601 = 0, $3602 = 0, $3603 = 0, $3604 = 0, $3605 = 0, $3606 = 0, $3607 = 0;
 var $3608 = 0, $3609 = 0, $361 = 0, $3610 = 0, $3611 = 0, $3612 = 0, $3613 = 0, $3614 = 0, $3615 = 0, $3616 = 0, $3617 = 0, $3618 = 0, $3619 = 0, $362 = 0, $3620 = 0, $3621 = 0, $3622 = 0, $3623 = 0, $3624 = 0, $3625 = 0;
 var $3626 = 0, $3627 = 0, $3628 = 0, $3629 = 0, $363 = 0, $3630 = 0, $3631 = 0, $3632 = 0, $3633 = 0, $3634 = 0, $3635 = 0, $3636 = 0, $3637 = 0, $3638 = 0, $3639 = 0, $364 = 0, $3640 = 0, $3641 = 0, $3642 = 0, $3643 = 0;
 var $3644 = 0, $3645 = 0, $3646 = 0, $3647 = 0, $3648 = 0, $3649 = 0, $365 = 0, $3650 = 0, $3651 = 0, $3652 = 0, $3653 = 0, $3654 = 0, $3655 = 0, $3656 = 0, $3657 = 0, $3658 = 0, $3659 = 0, $366 = 0, $3660 = 0, $3661 = 0;
 var $3662 = 0, $3663 = 0, $3664 = 0, $3665 = 0, $3666 = 0, $3667 = 0, $3668 = 0, $3669 = 0, $367 = 0, $3670 = 0, $3671 = 0, $3672 = 0, $3673 = 0, $3674 = 0, $3675 = 0, $3676 = 0, $3677 = 0, $3678 = 0, $3679 = 0, $368 = 0;
 var $3680 = 0, $3681 = 0, $3682 = 0, $3683 = 0, $3684 = 0, $3685 = 0, $3686 = 0, $3687 = 0, $3688 = 0, $3689 = 0, $369 = 0, $3690 = 0, $3691 = 0, $3692 = 0, $3693 = 0, $3694 = 0, $3695 = 0, $3696 = 0, $3697 = 0, $3698 = 0;
 var $3699 = 0, $37 = 0, $370 = 0, $3700 = 0, $3701 = 0, $3702 = 0, $3703 = 0, $3704 = 0, $3705 = 0, $3706 = 0, $3707 = 0, $3708 = 0, $3709 = 0, $371 = 0, $3710 = 0, $3711 = 0, $3712 = 0, $3713 = 0, $3714 = 0, $3715 = 0;
 var $3716 = 0, $3717 = 0, $3718 = 0, $3719 = 0, $372 = 0, $3720 = 0, $3721 = 0, $3722 = 0, $3723 = 0, $3724 = 0, $3725 = 0, $3726 = 0, $3727 = 0, $3728 = 0, $3729 = 0, $373 = 0, $3730 = 0, $3731 = 0, $3732 = 0, $3733 = 0;
 var $3734 = 0, $3735 = 0, $3736 = 0, $3737 = 0, $3738 = 0, $3739 = 0, $374 = 0, $3740 = 0, $3741 = 0, $3742 = 0, $3743 = 0, $3744 = 0, $3745 = 0, $3746 = 0, $3747 = 0, $3748 = 0, $3749 = 0, $375 = 0, $3750 = 0, $3751 = 0;
 var $3752 = 0, $3753 = 0, $3754 = 0, $3755 = 0, $3756 = 0, $3757 = 0, $3758 = 0, $3759 = 0, $376 = 0, $3760 = 0, $3761 = 0, $3762 = 0, $3763 = 0, $3764 = 0, $3765 = 0, $3766 = 0, $3767 = 0, $3768 = 0, $3769 = 0, $377 = 0;
 var $3770 = 0, $3771 = 0, $3772 = 0, $3773 = 0, $3774 = 0, $3775 = 0, $3776 = 0, $3777 = 0, $3778 = 0, $3779 = 0, $378 = 0, $3780 = 0, $3781 = 0, $3782 = 0, $3783 = 0, $3784 = 0, $3785 = 0, $3786 = 0, $3787 = 0, $3788 = 0;
 var $3789 = 0, $379 = 0, $3790 = 0, $3791 = 0, $3792 = 0, $3793 = 0, $3794 = 0, $3795 = 0, $3796 = 0, $3797 = 0, $3798 = 0, $3799 = 0, $38 = 0, $380 = 0, $3800 = 0, $3801 = 0, $3802 = 0, $3803 = 0, $3804 = 0, $3805 = 0;
 var $3806 = 0, $3807 = 0, $3808 = 0, $3809 = 0, $381 = 0, $3810 = 0.0, $3811 = 0, $3812 = 0, $3813 = 0, $3814 = 0, $3815 = 0, $3816 = 0, $3817 = 0.0, $3818 = 0, $3819 = 0, $382 = 0, $3820 = 0, $3821 = 0, $3822 = 0, $3823 = 0;
 var $3824 = 0, $3825 = 0.0, $3826 = 0, $3827 = 0, $3828 = 0, $3829 = 0, $383 = 0, $3830 = 0, $3831 = 0, $3832 = 0.0, $3833 = 0, $3834 = 0, $3835 = 0, $3836 = 0, $3837 = 0, $3838 = 0, $3839 = 0, $384 = 0, $3840 = 0, $3841 = 0;
 var $3842 = 0, $3843 = 0, $3844 = 0, $3845 = 0, $3846 = 0, $3847 = 0, $3848 = 0, $3849 = 0, $385 = 0, $3850 = 0, $3851 = 0, $3852 = 0, $3853 = 0, $3854 = 0, $3855 = 0, $3856 = 0, $3857 = 0, $3858 = 0, $3859 = 0, $386 = 0;
 var $3860 = 0, $3861 = 0, $3862 = 0, $3863 = 0, $3864 = 0, $3865 = 0, $3866 = 0, $3867 = 0, $3868 = 0, $3869 = 0, $387 = 0, $3870 = 0, $3871 = 0, $3872 = 0, $3873 = 0, $3874 = 0, $3875 = 0, $3876 = 0, $3877 = 0, $3878 = 0;
 var $3879 = 0, $388 = 0, $3880 = 0, $3881 = 0, $3882 = 0, $3883 = 0, $3884 = 0, $3885 = 0, $3886 = 0, $3887 = 0, $3888 = 0, $3889 = 0, $389 = 0, $3890 = 0, $3891 = 0, $3892 = 0, $3893 = 0, $3894 = 0, $3895 = 0, $3896 = 0;
 var $3897 = 0, $3898 = 0, $3899 = 0, $39 = 0, $390 = 0, $3900 = 0, $3901 = 0, $3902 = 0, $3903 = 0, $3904 = 0, $3905 = 0, $3906 = 0, $3907 = 0, $3908 = 0, $3909 = 0, $391 = 0, $3910 = 0, $3911 = 0, $3912 = 0, $3913 = 0;
 var $3914 = 0, $3915 = 0, $3916 = 0, $3917 = 0, $3918 = 0, $3919 = 0, $392 = 0, $3920 = 0, $3921 = 0, $3922 = 0, $3923 = 0, $3924 = 0, $3925 = 0, $3926 = 0, $3927 = 0, $3928 = 0, $3929 = 0, $393 = 0, $3930 = 0, $3931 = 0;
 var $3932 = 0, $3933 = 0, $3934 = 0, $3935 = 0, $3936 = 0, $3937 = 0, $3938 = 0, $3939 = 0, $394 = 0, $3940 = 0, $3941 = 0, $3942 = 0, $3943 = 0, $3944 = 0, $3945 = 0, $3946 = 0, $3947 = 0, $3948 = 0, $3949 = 0, $395 = 0;
 var $3950 = 0, $3951 = 0, $3952 = 0, $3953 = 0, $3954 = 0, $3955 = 0, $3956 = 0, $3957 = 0, $3958 = 0, $3959 = 0, $396 = 0, $3960 = 0, $3961 = 0, $3962 = 0, $3963 = 0, $3964 = 0, $3965 = 0, $3966 = 0, $3967 = 0, $3968 = 0;
 var $3969 = 0, $397 = 0, $3970 = 0, $3971 = 0, $3972 = 0, $3973 = 0, $3974 = 0, $3975 = 0, $3976 = 0, $3977 = 0, $3978 = 0, $3979 = 0, $398 = 0, $3980 = 0, $3981 = 0, $3982 = 0, $3983 = 0, $3984 = 0, $3985 = 0, $3986 = 0;
 var $3987 = 0, $3988 = 0, $3989 = 0, $399 = 0, $3990 = 0, $3991 = 0, $3992 = 0, $3993 = 0, $3994 = 0, $3995 = 0, $3996 = 0, $3997 = 0, $3998 = 0, $3999 = 0, $4 = 0, $40 = 0, $400 = 0, $4000 = 0, $4001 = 0, $4002 = 0;
 var $4003 = 0, $4004 = 0, $4005 = 0, $4006 = 0, $4007 = 0, $4008 = 0, $4009 = 0, $401 = 0, $4010 = 0, $4011 = 0, $4012 = 0, $4013 = 0, $4014 = 0, $4015 = 0, $4016 = 0, $4017 = 0, $4018 = 0, $4019 = 0, $402 = 0, $4020 = 0;
 var $4021 = 0, $4022 = 0, $4023 = 0, $4024 = 0, $4025 = 0, $4026 = 0, $4027 = 0, $4028 = 0, $4029 = 0, $403 = 0, $4030 = 0, $4031 = 0, $4032 = 0, $4033 = 0, $4034 = 0, $4035 = 0, $4036 = 0, $4037 = 0, $4038 = 0, $4039 = 0;
 var $404 = 0, $4040 = 0, $4041 = 0, $4042 = 0, $4043 = 0, $4044 = 0, $4045 = 0, $4046 = 0, $4047 = 0, $4048 = 0, $4049 = 0, $405 = 0, $4050 = 0, $4051 = 0, $4052 = 0, $4053 = 0, $4054 = 0, $4055 = 0, $4056 = 0, $4057 = 0;
 var $4058 = 0, $4059 = 0, $406 = 0, $4060 = 0, $4061 = 0, $4062 = 0, $4063 = 0, $4064 = 0, $4065 = 0, $4066 = 0, $4067 = 0, $4068 = 0, $4069 = 0, $407 = 0, $4070 = 0, $4071 = 0, $4072 = 0, $4073 = 0, $4074 = 0, $4075 = 0;
 var $4076 = 0, $4077 = 0, $4078 = 0, $4079 = 0, $408 = 0.0, $4080 = 0, $4081 = 0, $4082 = 0, $4083 = 0, $4084 = 0, $4085 = 0, $4086 = 0, $4087 = 0, $4088 = 0, $4089 = 0, $409 = 0, $4090 = 0, $4091 = 0, $4092 = 0, $4093 = 0;
 var $4094 = 0, $4095 = 0, $4096 = 0, $4097 = 0, $4098 = 0, $4099 = 0, $41 = 0, $410 = 0, $4100 = 0, $4101 = 0, $4102 = 0, $4103 = 0, $4104 = 0, $4105 = 0, $4106 = 0, $4107 = 0, $4108 = 0, $4109 = 0, $411 = 0, $4110 = 0;
 var $4111 = 0, $4112 = 0, $4113 = 0, $4114 = 0, $4115 = 0, $4116 = 0, $4117 = 0, $4118 = 0, $4119 = 0, $412 = 0, $4120 = 0, $4121 = 0, $4122 = 0, $4123 = 0, $4124 = 0, $4125 = 0, $4126 = 0, $4127 = 0, $4128 = 0, $4129 = 0;
 var $413 = 0, $4130 = 0, $4131 = 0, $4132 = 0, $4133 = 0, $4134 = 0, $4135 = 0, $4136 = 0, $4137 = 0, $4138 = 0, $4139 = 0, $414 = 0, $4140 = 0, $4141 = 0, $4142 = 0, $4143 = 0, $4144 = 0, $4145 = 0, $4146 = 0, $4147 = 0;
 var $4148 = 0, $4149 = 0, $415 = 0, $4150 = 0, $4151 = 0, $4152 = 0, $4153 = 0, $4154 = 0, $4155 = 0, $4156 = 0, $4157 = 0, $4158 = 0, $4159 = 0, $416 = 0, $4160 = 0, $4161 = 0, $4162 = 0, $4163 = 0, $4164 = 0, $4165 = 0;
 var $4166 = 0, $4167 = 0, $4168 = 0, $4169 = 0, $417 = 0, $4170 = 0, $4171 = 0, $4172 = 0, $4173 = 0, $4174 = 0, $4175 = 0, $4176 = 0, $4177 = 0, $4178 = 0, $4179 = 0, $418 = 0, $4180 = 0, $4181 = 0, $4182 = 0, $4183 = 0;
 var $4184 = 0, $4185 = 0, $4186 = 0, $4187 = 0, $4188 = 0, $4189 = 0, $419 = 0, $4190 = 0, $4191 = 0, $4192 = 0, $4193 = 0, $4194 = 0, $4195 = 0, $4196 = 0, $4197 = 0, $4198 = 0, $4199 = 0, $42 = 0, $420 = 0, $4200 = 0;
 var $4201 = 0, $4202 = 0, $4203 = 0, $4204 = 0, $4205 = 0, $4206 = 0, $4207 = 0, $4208 = 0, $4209 = 0, $421 = 0, $4210 = 0, $4211 = 0, $4212 = 0, $4213 = 0, $4214 = 0, $4215 = 0, $4216 = 0, $4217 = 0, $4218 = 0, $4219 = 0;
 var $422 = 0, $4220 = 0, $4221 = 0, $4222 = 0, $4223 = 0, $4224 = 0, $4225 = 0, $4226 = 0, $4227 = 0, $4228 = 0, $4229 = 0, $423 = 0, $4230 = 0, $4231 = 0, $4232 = 0, $4233 = 0, $4234 = 0, $4235 = 0, $4236 = 0, $4237 = 0;
 var $4238 = 0, $4239 = 0, $424 = 0, $4240 = 0, $4241 = 0, $4242 = 0, $4243 = 0, $4244 = 0, $4245 = 0, $4246 = 0, $4247 = 0, $4248 = 0, $4249 = 0, $425 = 0, $4250 = 0, $4251 = 0, $4252 = 0, $4253 = 0, $4254 = 0, $4255 = 0;
 var $4256 = 0, $4257 = 0, $4258 = 0, $4259 = 0, $426 = 0, $4260 = 0, $4261 = 0, $4262 = 0, $4263 = 0, $4264 = 0, $4265 = 0, $4266 = 0, $4267 = 0, $4268 = 0, $4269 = 0, $427 = 0, $4270 = 0, $4271 = 0, $4272 = 0, $4273 = 0;
 var $4274 = 0, $4275 = 0, $4276 = 0, $4277 = 0, $4278 = 0, $4279 = 0, $428 = 0, $4280 = 0, $4281 = 0, $4282 = 0, $4283 = 0, $4284 = 0, $4285 = 0, $4286 = 0, $4287 = 0, $4288 = 0, $4289 = 0, $429 = 0, $4290 = 0, $4291 = 0;
 var $4292 = 0, $4293 = 0, $4294 = 0, $4295 = 0, $4296 = 0, $4297 = 0, $4298 = 0, $4299 = 0, $43 = 0, $430 = 0, $4300 = 0, $4301 = 0, $4302 = 0, $4303 = 0, $4304 = 0, $4305 = 0, $4306 = 0, $4307 = 0, $4308 = 0, $4309 = 0;
 var $431 = 0, $4310 = 0, $4311 = 0, $4312 = 0, $4313 = 0, $4314 = 0, $4315 = 0, $4316 = 0, $4317 = 0, $4318 = 0, $4319 = 0, $432 = 0, $4320 = 0, $4321 = 0, $4322 = 0, $4323 = 0, $4324 = 0, $4325 = 0, $4326 = 0, $4327 = 0;
 var $4328 = 0, $4329 = 0, $433 = 0, $4330 = 0, $4331 = 0, $4332 = 0, $4333 = 0, $4334 = 0, $4335 = 0, $4336 = 0, $4337 = 0, $4338 = 0, $4339 = 0, $434 = 0, $4340 = 0, $4341 = 0, $4342 = 0, $4343 = 0, $4344 = 0, $4345 = 0;
 var $4346 = 0, $4347 = 0, $4348 = 0, $4349 = 0, $435 = 0, $4350 = 0, $4351 = 0, $4352 = 0, $4353 = 0, $4354 = 0, $4355 = 0, $4356 = 0, $4357 = 0, $4358 = 0, $4359 = 0, $436 = 0, $4360 = 0, $4361 = 0, $4362 = 0, $4363 = 0;
 var $4364 = 0, $4365 = 0, $4366 = 0, $4367 = 0, $4368 = 0, $4369 = 0, $437 = 0, $4370 = 0, $4371 = 0, $4372 = 0, $4373 = 0, $4374 = 0, $4375 = 0, $4376 = 0, $4377 = 0, $4378 = 0, $4379 = 0, $438 = 0, $4380 = 0, $4381 = 0;
 var $4382 = 0, $4383 = 0, $4384 = 0, $4385 = 0, $4386 = 0, $4387 = 0, $4388 = 0, $4389 = 0, $439 = 0, $4390 = 0, $4391 = 0, $4392 = 0, $4393 = 0, $4394 = 0, $4395 = 0, $4396 = 0, $4397 = 0, $4398 = 0, $4399 = 0, $44 = 0;
 var $440 = 0, $4400 = 0, $4401 = 0, $4402 = 0, $4403 = 0, $4404 = 0, $4405 = 0, $4406 = 0, $4407 = 0, $4408 = 0, $4409 = 0, $441 = 0, $4410 = 0, $4411 = 0, $4412 = 0, $4413 = 0, $4414 = 0, $4415 = 0, $4416 = 0, $4417 = 0;
 var $4418 = 0, $4419 = 0, $442 = 0, $4420 = 0, $4421 = 0, $4422 = 0, $4423 = 0, $4424 = 0, $4425 = 0, $4426 = 0, $4427 = 0, $4428 = 0, $4429 = 0, $443 = 0, $4430 = 0, $4431 = 0, $4432 = 0, $4433 = 0, $4434 = 0, $4435 = 0;
 var $4436 = 0, $4437 = 0, $4438 = 0, $4439 = 0, $444 = 0, $4440 = 0, $4441 = 0, $4442 = 0, $4443 = 0, $4444 = 0, $4445 = 0, $4446 = 0, $4447 = 0, $4448 = 0, $4449 = 0, $445 = 0, $4450 = 0, $4451 = 0, $4452 = 0, $4453 = 0;
 var $4454 = 0, $4455 = 0, $4456 = 0, $4457 = 0, $4458 = 0, $4459 = 0, $446 = 0, $4460 = 0, $4461 = 0, $4462 = 0, $4463 = 0, $4464 = 0, $4465 = 0, $4466 = 0, $4467 = 0, $4468 = 0, $4469 = 0, $447 = 0, $4470 = 0, $4471 = 0;
 var $4472 = 0, $4473 = 0, $4474 = 0, $4475 = 0, $4476 = 0, $4477 = 0, $4478 = 0, $4479 = 0, $448 = 0, $4480 = 0, $4481 = 0, $4482 = 0, $4483 = 0, $4484 = 0, $4485 = 0, $4486 = 0, $4487 = 0, $4488 = 0, $4489 = 0, $449 = 0;
 var $4490 = 0, $4491 = 0, $4492 = 0, $4493 = 0, $4494 = 0, $4495 = 0, $4496 = 0, $4497 = 0, $4498 = 0, $4499 = 0, $45 = 0, $450 = 0, $4500 = 0, $4501 = 0, $4502 = 0, $4503 = 0, $4504 = 0, $4505 = 0, $4506 = 0, $4507 = 0;
 var $4508 = 0, $4509 = 0, $451 = 0, $4510 = 0, $4511 = 0, $4512 = 0, $4513 = 0, $4514 = 0, $4515 = 0, $4516 = 0, $4517 = 0, $4518 = 0, $4519 = 0, $452 = 0, $4520 = 0, $4521 = 0, $4522 = 0, $4523 = 0, $4524 = 0, $4525 = 0;
 var $4526 = 0, $4527 = 0, $4528 = 0, $4529 = 0, $453 = 0, $4530 = 0, $4531 = 0, $4532 = 0, $4533 = 0, $4534 = 0, $4535 = 0, $4536 = 0, $4537 = 0, $4538 = 0, $4539 = 0, $454 = 0, $4540 = 0, $4541 = 0, $4542 = 0, $4543 = 0;
 var $4544 = 0, $4545 = 0, $4546 = 0, $4547 = 0, $4548 = 0, $4549 = 0, $455 = 0, $4550 = 0, $4551 = 0, $4552 = 0, $4553 = 0, $4554 = 0, $4555 = 0, $4556 = 0, $4557 = 0, $4558 = 0, $4559 = 0, $456 = 0, $4560 = 0, $4561 = 0;
 var $4562 = 0, $4563 = 0, $4564 = 0, $4565 = 0, $4566 = 0, $4567 = 0, $4568 = 0, $4569 = 0, $457 = 0, $4570 = 0, $4571 = 0, $4572 = 0, $4573 = 0, $4574 = 0, $4575 = 0, $4576 = 0, $4577 = 0, $4578 = 0, $4579 = 0, $458 = 0;
 var $4580 = 0, $4581 = 0, $4582 = 0, $4583 = 0, $4584 = 0, $4585 = 0, $4586 = 0, $4587 = 0, $4588 = 0, $4589 = 0, $459 = 0, $4590 = 0, $4591 = 0, $4592 = 0, $4593 = 0, $4594 = 0, $4595 = 0, $4596 = 0, $4597 = 0, $4598 = 0;
 var $4599 = 0, $46 = 0, $460 = 0, $4600 = 0, $4601 = 0, $4602 = 0, $4603 = 0, $4604 = 0, $4605 = 0, $4606 = 0, $4607 = 0, $4608 = 0, $4609 = 0, $461 = 0, $4610 = 0, $4611 = 0, $4612 = 0, $4613 = 0, $4614 = 0, $4615 = 0;
 var $4616 = 0, $4617 = 0, $4618 = 0, $4619 = 0, $462 = 0, $4620 = 0, $4621 = 0, $4622 = 0, $4623 = 0, $4624 = 0, $4625 = 0, $4626 = 0, $4627 = 0, $4628 = 0, $4629 = 0, $463 = 0, $4630 = 0, $4631 = 0, $4632 = 0, $4633 = 0;
 var $4634 = 0, $4635 = 0, $4636 = 0, $4637 = 0, $4638 = 0, $4639 = 0, $464 = 0, $4640 = 0, $4641 = 0, $4642 = 0, $4643 = 0, $4644 = 0, $4645 = 0, $4646 = 0, $4647 = 0, $4648 = 0, $4649 = 0, $465 = 0, $4650 = 0, $4651 = 0;
 var $4652 = 0, $4653 = 0, $4654 = 0, $4655 = 0, $4656 = 0, $4657 = 0, $4658 = 0, $4659 = 0, $466 = 0, $4660 = 0, $4661 = 0, $4662 = 0, $4663 = 0, $4664 = 0, $4665 = 0, $4666 = 0, $4667 = 0, $4668 = 0, $4669 = 0, $467 = 0;
 var $4670 = 0, $4671 = 0, $4672 = 0, $4673 = 0, $4674 = 0, $4675 = 0, $4676 = 0, $4677 = 0, $4678 = 0, $4679 = 0, $468 = 0, $4680 = 0, $4681 = 0, $4682 = 0, $4683 = 0, $4684 = 0, $4685 = 0, $4686 = 0, $4687 = 0, $4688 = 0;
 var $4689 = 0, $469 = 0, $4690 = 0, $4691 = 0, $4692 = 0, $4693 = 0, $4694 = 0, $4695 = 0, $4696 = 0, $4697 = 0, $4698 = 0, $4699 = 0, $47 = 0, $470 = 0, $4700 = 0, $4701 = 0, $4702 = 0, $4703 = 0, $4704 = 0, $4705 = 0;
 var $4706 = 0, $4707 = 0, $4708 = 0, $4709 = 0, $471 = 0, $4710 = 0, $4711 = 0, $4712 = 0, $4713 = 0, $4714 = 0, $4715 = 0, $4716 = 0, $4717 = 0, $4718 = 0, $4719 = 0, $472 = 0, $4720 = 0, $4721 = 0, $4722 = 0, $4723 = 0;
 var $4724 = 0, $4725 = 0, $4726 = 0, $4727 = 0, $4728 = 0, $4729 = 0, $473 = 0, $4730 = 0, $4731 = 0, $4732 = 0, $4733 = 0, $4734 = 0, $4735 = 0, $4736 = 0, $4737 = 0, $4738 = 0, $4739 = 0, $474 = 0, $4740 = 0, $4741 = 0;
 var $4742 = 0, $4743 = 0, $4744 = 0, $4745 = 0, $4746 = 0, $4747 = 0, $4748 = 0, $4749 = 0, $475 = 0, $4750 = 0, $4751 = 0, $4752 = 0, $4753 = 0, $4754 = 0, $4755 = 0, $4756 = 0, $4757 = 0, $4758 = 0, $4759 = 0, $476 = 0;
 var $4760 = 0, $4761 = 0, $4762 = 0, $4763 = 0, $4764 = 0, $4765 = 0, $4766 = 0, $4767 = 0, $4768 = 0, $4769 = 0, $477 = 0, $4770 = 0, $4771 = 0, $4772 = 0, $4773 = 0, $4774 = 0, $4775 = 0, $4776 = 0, $4777 = 0, $4778 = 0;
 var $4779 = 0, $478 = 0, $4780 = 0, $4781 = 0, $4782 = 0, $4783 = 0, $4784 = 0, $4785 = 0, $4786 = 0, $4787 = 0, $4788 = 0, $4789 = 0, $479 = 0, $4790 = 0, $4791 = 0, $4792 = 0, $4793 = 0, $4794 = 0, $4795 = 0, $4796 = 0;
 var $4797 = 0, $4798 = 0, $4799 = 0, $48 = 0, $480 = 0, $4800 = 0, $4801 = 0, $4802 = 0, $4803 = 0, $4804 = 0, $4805 = 0, $4806 = 0, $4807 = 0, $4808 = 0, $4809 = 0, $481 = 0, $4810 = 0, $4811 = 0, $4812 = 0, $4813 = 0;
 var $4814 = 0, $4815 = 0, $4816 = 0, $4817 = 0, $4818 = 0, $4819 = 0, $482 = 0, $4820 = 0, $4821 = 0, $4822 = 0, $4823 = 0, $4824 = 0, $4825 = 0, $4826 = 0, $4827 = 0, $4828 = 0, $4829 = 0, $483 = 0, $4830 = 0, $4831 = 0;
 var $4832 = 0, $4833 = 0, $4834 = 0, $4835 = 0, $4836 = 0, $4837 = 0, $4838 = 0, $4839 = 0, $484 = 0, $4840 = 0, $4841 = 0, $4842 = 0, $4843 = 0, $4844 = 0, $4845 = 0, $4846 = 0, $4847 = 0, $4848 = 0, $4849 = 0, $485 = 0;
 var $4850 = 0, $4851 = 0, $4852 = 0, $4853 = 0, $4854 = 0, $4855 = 0, $4856 = 0, $4857 = 0, $4858 = 0, $4859 = 0, $486 = 0, $4860 = 0, $4861 = 0, $4862 = 0, $4863 = 0, $4864 = 0, $4865 = 0, $4866 = 0, $4867 = 0, $4868 = 0;
 var $4869 = 0, $487 = 0, $4870 = 0, $4871 = 0, $4872 = 0, $4873 = 0, $4874 = 0, $4875 = 0, $4876 = 0, $4877 = 0, $4878 = 0, $4879 = 0, $488 = 0, $4880 = 0, $4881 = 0, $4882 = 0, $4883 = 0, $4884 = 0, $4885 = 0, $4886 = 0;
 var $4887 = 0, $4888 = 0, $4889 = 0, $489 = 0, $4890 = 0, $4891 = 0, $4892 = 0, $4893 = 0, $4894 = 0, $4895 = 0, $4896 = 0, $4897 = 0, $4898 = 0, $4899 = 0, $49 = 0, $490 = 0, $4900 = 0, $4901 = 0, $4902 = 0, $4903 = 0;
 var $4904 = 0, $4905 = 0, $4906 = 0, $4907 = 0, $4908 = 0, $4909 = 0, $491 = 0, $4910 = 0, $4911 = 0, $4912 = 0, $4913 = 0, $4914 = 0, $4915 = 0, $4916 = 0, $4917 = 0, $4918 = 0, $4919 = 0, $492 = 0, $4920 = 0, $4921 = 0;
 var $4922 = 0, $4923 = 0, $4924 = 0, $4925 = 0, $4926 = 0, $4927 = 0, $4928 = 0, $4929 = 0, $493 = 0, $4930 = 0, $4931 = 0, $4932 = 0, $4933 = 0, $4934 = 0, $4935 = 0, $4936 = 0, $4937 = 0, $4938 = 0, $4939 = 0, $494 = 0;
 var $4940 = 0, $4941 = 0, $4942 = 0, $4943 = 0, $4944 = 0, $4945 = 0, $4946 = 0, $4947 = 0, $4948 = 0, $4949 = 0, $495 = 0, $4950 = 0, $4951 = 0, $4952 = 0, $4953 = 0, $4954 = 0, $4955 = 0, $4956 = 0, $4957 = 0, $4958 = 0;
 var $4959 = 0, $496 = 0, $4960 = 0, $4961 = 0, $4962 = 0, $4963 = 0, $4964 = 0, $4965 = 0, $4966 = 0, $4967 = 0, $4968 = 0, $4969 = 0, $497 = 0, $4970 = 0, $4971 = 0, $4972 = 0, $4973 = 0, $4974 = 0, $4975 = 0, $4976 = 0;
 var $4977 = 0, $4978 = 0, $4979 = 0, $498 = 0, $4980 = 0, $4981 = 0, $4982 = 0, $4983 = 0, $4984 = 0, $4985 = 0, $4986 = 0, $4987 = 0, $4988 = 0, $4989 = 0, $499 = 0, $4990 = 0, $4991 = 0, $4992 = 0, $4993 = 0, $4994 = 0;
 var $4995 = 0, $4996 = 0, $4997 = 0, $4998 = 0, $4999 = 0, $5 = 0, $50 = 0, $500 = 0, $5000 = 0, $5001 = 0, $5002 = 0, $5003 = 0, $5004 = 0, $5005 = 0, $5006 = 0, $5007 = 0, $5008 = 0, $5009 = 0, $501 = 0, $5010 = 0;
 var $5011 = 0, $5012 = 0, $5013 = 0, $5014 = 0, $5015 = 0, $5016 = 0, $5017 = 0, $5018 = 0, $5019 = 0, $502 = 0, $5020 = 0, $5021 = 0, $5022 = 0, $5023 = 0, $5024 = 0, $5025 = 0, $5026 = 0, $5027 = 0, $5028 = 0, $5029 = 0;
 var $503 = 0, $5030 = 0, $5031 = 0, $5032 = 0, $5033 = 0, $5034 = 0, $5035 = 0, $5036 = 0, $5037 = 0, $5038 = 0, $5039 = 0, $504 = 0, $5040 = 0, $5041 = 0, $5042 = 0, $5043 = 0, $5044 = 0, $5045 = 0, $5046 = 0, $5047 = 0;
 var $5048 = 0, $5049 = 0, $505 = 0, $5050 = 0, $5051 = 0, $5052 = 0, $5053 = 0, $5054 = 0, $5055 = 0, $5056 = 0, $5057 = 0, $5058 = 0, $5059 = 0, $506 = 0, $5060 = 0, $5061 = 0, $5062 = 0, $5063 = 0, $5064 = 0, $5065 = 0;
 var $5066 = 0, $5067 = 0, $5068 = 0, $5069 = 0, $507 = 0, $5070 = 0, $5071 = 0, $5072 = 0, $5073 = 0, $5074 = 0, $5075 = 0, $5076 = 0, $5077 = 0, $5078 = 0, $5079 = 0, $508 = 0, $5080 = 0, $5081 = 0, $5082 = 0, $5083 = 0;
 var $5084 = 0, $5085 = 0, $5086 = 0, $5087 = 0, $5088 = 0, $5089 = 0, $509 = 0, $5090 = 0, $5091 = 0, $5092 = 0, $5093 = 0, $5094 = 0, $5095 = 0, $5096 = 0, $5097 = 0, $5098 = 0, $5099 = 0, $51 = 0, $510 = 0, $5100 = 0;
 var $5101 = 0, $5102 = 0, $5103 = 0, $5104 = 0, $5105 = 0, $5106 = 0, $5107 = 0, $5108 = 0, $5109 = 0, $511 = 0, $5110 = 0, $5111 = 0, $5112 = 0, $5113 = 0, $5114 = 0, $5115 = 0, $5116 = 0, $5117 = 0, $5118 = 0, $5119 = 0;
 var $512 = 0, $5120 = 0, $5121 = 0, $5122 = 0, $5123 = 0, $5124 = 0, $5125 = 0, $5126 = 0, $5127 = 0, $5128 = 0, $5129 = 0, $513 = 0, $5130 = 0, $5131 = 0, $5132 = 0, $5133 = 0, $5134 = 0, $5135 = 0, $5136 = 0, $5137 = 0;
 var $5138 = 0, $5139 = 0, $514 = 0, $5140 = 0, $5141 = 0, $5142 = 0, $5143 = 0, $5144 = 0, $5145 = 0, $5146 = 0, $5147 = 0, $5148 = 0, $5149 = 0, $515 = 0, $5150 = 0, $5151 = 0, $5152 = 0, $5153 = 0, $5154 = 0, $5155 = 0;
 var $5156 = 0, $5157 = 0, $5158 = 0, $5159 = 0, $516 = 0, $5160 = 0, $5161 = 0, $5162 = 0, $5163 = 0, $5164 = 0, $5165 = 0, $5166 = 0, $5167 = 0, $5168 = 0, $5169 = 0, $517 = 0, $5170 = 0, $5171 = 0, $5172 = 0, $5173 = 0;
 var $5174 = 0, $5175 = 0, $5176 = 0, $5177 = 0, $5178 = 0, $5179 = 0, $518 = 0, $5180 = 0, $5181 = 0, $5182 = 0, $5183 = 0, $5184 = 0, $5185 = 0, $5186 = 0, $5187 = 0, $5188 = 0, $5189 = 0, $519 = 0, $5190 = 0, $5191 = 0;
 var $5192 = 0, $5193 = 0, $5194 = 0, $5195 = 0, $5196 = 0, $5197 = 0, $5198 = 0, $5199 = 0, $52 = 0, $520 = 0, $5200 = 0, $5201 = 0, $5202 = 0, $5203 = 0, $5204 = 0, $5205 = 0, $5206 = 0, $5207 = 0, $5208 = 0, $5209 = 0;
 var $521 = 0, $5210 = 0, $5211 = 0, $5212 = 0, $5213 = 0, $5214 = 0, $5215 = 0, $5216 = 0, $5217 = 0, $5218 = 0, $5219 = 0, $522 = 0, $5220 = 0, $5221 = 0, $5222 = 0, $5223 = 0, $5224 = 0, $5225 = 0, $5226 = 0, $5227 = 0;
 var $5228 = 0, $5229 = 0, $523 = 0, $5230 = 0, $5231 = 0, $5232 = 0, $5233 = 0, $5234 = 0, $5235 = 0, $5236 = 0, $5237 = 0, $5238 = 0, $5239 = 0, $524 = 0, $5240 = 0, $5241 = 0, $5242 = 0, $5243 = 0, $5244 = 0, $5245 = 0;
 var $5246 = 0, $5247 = 0, $5248 = 0, $5249 = 0, $525 = 0, $5250 = 0, $5251 = 0, $5252 = 0, $5253 = 0, $5254 = 0, $5255 = 0, $5256 = 0, $5257 = 0, $5258 = 0, $5259 = 0, $526 = 0, $5260 = 0, $5261 = 0, $5262 = 0, $5263 = 0;
 var $5264 = 0, $5265 = 0, $5266 = 0, $5267 = 0, $5268 = 0, $5269 = 0, $527 = 0, $5270 = 0, $5271 = 0, $5272 = 0, $5273 = 0, $5274 = 0, $5275 = 0, $5276 = 0, $5277 = 0, $5278 = 0, $5279 = 0, $528 = 0, $5280 = 0, $5281 = 0;
 var $5282 = 0, $5283 = 0, $5284 = 0, $5285 = 0, $5286 = 0, $5287 = 0, $5288 = 0, $5289 = 0, $529 = 0, $5290 = 0, $5291 = 0, $5292 = 0, $5293 = 0, $5294 = 0, $5295 = 0, $5296 = 0, $5297 = 0, $5298 = 0, $5299 = 0, $53 = 0;
 var $530 = 0, $5300 = 0, $5301 = 0, $5302 = 0, $5303 = 0, $5304 = 0, $5305 = 0, $5306 = 0, $5307 = 0, $5308 = 0, $5309 = 0, $531 = 0, $5310 = 0, $5311 = 0, $5312 = 0, $5313 = 0, $5314 = 0, $5315 = 0, $5316 = 0, $5317 = 0;
 var $5318 = 0, $5319 = 0, $532 = 0, $5320 = 0, $5321 = 0, $5322 = 0, $5323 = 0, $5324 = 0, $5325 = 0, $5326 = 0, $5327 = 0, $5328 = 0, $5329 = 0, $533 = 0, $5330 = 0, $5331 = 0, $5332 = 0, $5333 = 0, $5334 = 0, $5335 = 0;
 var $5336 = 0, $5337 = 0, $5338 = 0, $5339 = 0, $534 = 0, $5340 = 0, $5341 = 0, $5342 = 0, $5343 = 0, $5344 = 0, $5345 = 0, $5346 = 0, $5347 = 0, $5348 = 0, $5349 = 0, $535 = 0, $5350 = 0, $5351 = 0, $5352 = 0, $5353 = 0;
 var $5354 = 0, $5355 = 0, $5356 = 0, $5357 = 0, $5358 = 0, $5359 = 0, $536 = 0, $5360 = 0, $5361 = 0, $5362 = 0, $5363 = 0, $5364 = 0, $5365 = 0, $5366 = 0, $5367 = 0, $5368 = 0, $5369 = 0, $537 = 0, $5370 = 0, $5371 = 0;
 var $5372 = 0, $5373 = 0, $5374 = 0, $5375 = 0, $5376 = 0, $5377 = 0, $5378 = 0, $5379 = 0, $538 = 0, $5380 = 0, $5381 = 0, $5382 = 0, $5383 = 0, $5384 = 0, $5385 = 0, $5386 = 0, $5387 = 0, $5388 = 0, $5389 = 0, $539 = 0;
 var $5390 = 0, $5391 = 0, $5392 = 0, $5393 = 0, $5394 = 0, $5395 = 0, $5396 = 0, $5397 = 0, $5398 = 0, $5399 = 0, $54 = 0, $540 = 0, $5400 = 0, $5401 = 0, $5402 = 0, $5403 = 0, $5404 = 0, $5405 = 0, $5406 = 0, $5407 = 0;
 var $5408 = 0, $5409 = 0, $541 = 0, $5410 = 0, $5411 = 0, $5412 = 0, $5413 = 0, $5414 = 0, $5415 = 0, $5416 = 0, $5417 = 0, $5418 = 0, $5419 = 0, $542 = 0, $5420 = 0, $5421 = 0, $5422 = 0, $5423 = 0, $5424 = 0, $5425 = 0;
 var $5426 = 0, $5427 = 0, $5428 = 0, $5429 = 0, $543 = 0, $5430 = 0, $5431 = 0, $5432 = 0, $5433 = 0, $5434 = 0, $5435 = 0, $5436 = 0, $5437 = 0, $5438 = 0, $5439 = 0, $544 = 0, $5440 = 0, $5441 = 0, $5442 = 0, $5443 = 0;
 var $5444 = 0, $5445 = 0, $5446 = 0, $5447 = 0, $5448 = 0, $5449 = 0, $545 = 0, $5450 = 0, $5451 = 0, $5452 = 0, $5453 = 0, $5454 = 0, $5455 = 0, $5456 = 0, $5457 = 0, $5458 = 0, $5459 = 0, $546 = 0, $5460 = 0, $5461 = 0;
 var $5462 = 0, $5463 = 0, $5464 = 0, $5465 = 0, $5466 = 0, $5467 = 0, $5468 = 0, $5469 = 0, $547 = 0, $5470 = 0, $5471 = 0, $5472 = 0, $5473 = 0, $5474 = 0, $5475 = 0, $5476 = 0, $5477 = 0, $5478 = 0, $5479 = 0, $548 = 0;
 var $5480 = 0, $5481 = 0, $5482 = 0, $5483 = 0, $5484 = 0, $5485 = 0, $5486 = 0, $5487 = 0, $5488 = 0, $5489 = 0, $549 = 0, $5490 = 0, $5491 = 0, $5492 = 0, $5493 = 0, $5494 = 0, $5495 = 0, $5496 = 0, $5497 = 0, $5498 = 0;
 var $5499 = 0, $55 = 0, $550 = 0, $5500 = 0, $5501 = 0, $5502 = 0, $5503 = 0, $5504 = 0, $5505 = 0, $5506 = 0, $5507 = 0, $5508 = 0, $5509 = 0, $551 = 0, $5510 = 0, $5511 = 0, $5512 = 0, $5513 = 0, $5514 = 0, $5515 = 0;
 var $5516 = 0, $5517 = 0, $5518 = 0, $5519 = 0, $552 = 0, $5520 = 0, $5521 = 0, $5522 = 0, $5523 = 0, $5524 = 0, $5525 = 0, $5526 = 0, $5527 = 0, $5528 = 0, $5529 = 0, $553 = 0, $5530 = 0, $5531 = 0, $5532 = 0, $5533 = 0;
 var $5534 = 0, $5535 = 0, $5536 = 0, $5537 = 0, $5538 = 0, $5539 = 0, $554 = 0, $5540 = 0, $5541 = 0, $5542 = 0, $5543 = 0, $5544 = 0, $5545 = 0, $5546 = 0, $5547 = 0, $5548 = 0, $5549 = 0, $555 = 0, $5550 = 0, $5551 = 0;
 var $5552 = 0, $5553 = 0, $5554 = 0, $5555 = 0, $5556 = 0, $5557 = 0, $5558 = 0, $5559 = 0, $556 = 0, $5560 = 0, $5561 = 0, $5562 = 0, $5563 = 0, $5564 = 0, $5565 = 0, $5566 = 0, $5567 = 0, $5568 = 0, $5569 = 0, $557 = 0;
 var $5570 = 0, $5571 = 0, $5572 = 0, $5573 = 0, $5574 = 0, $5575 = 0, $5576 = 0, $5577 = 0, $5578 = 0, $5579 = 0, $558 = 0, $5580 = 0, $5581 = 0, $5582 = 0, $5583 = 0, $5584 = 0, $5585 = 0, $5586 = 0, $5587 = 0, $5588 = 0;
 var $5589 = 0, $559 = 0, $5590 = 0, $5591 = 0, $5592 = 0, $5593 = 0, $5594 = 0, $5595 = 0, $5596 = 0, $5597 = 0, $5598 = 0, $5599 = 0, $56 = 0, $560 = 0, $5600 = 0, $5601 = 0, $5602 = 0, $5603 = 0, $5604 = 0, $5605 = 0;
 var $5606 = 0, $5607 = 0, $5608 = 0, $5609 = 0, $561 = 0, $5610 = 0, $5611 = 0, $5612 = 0, $5613 = 0, $5614 = 0, $5615 = 0, $5616 = 0, $5617 = 0, $5618 = 0, $5619 = 0, $562 = 0, $5620 = 0, $5621 = 0, $5622 = 0, $5623 = 0;
 var $5624 = 0, $5625 = 0, $5626 = 0, $5627 = 0, $5628 = 0, $5629 = 0, $563 = 0, $5630 = 0, $5631 = 0, $5632 = 0, $5633 = 0, $5634 = 0, $5635 = 0, $5636 = 0, $5637 = 0, $5638 = 0, $5639 = 0, $564 = 0, $5640 = 0, $5641 = 0;
 var $5642 = 0, $5643 = 0, $5644 = 0, $5645 = 0, $5646 = 0, $5647 = 0, $5648 = 0, $5649 = 0, $565 = 0, $5650 = 0, $5651 = 0, $5652 = 0, $5653 = 0, $5654 = 0, $5655 = 0, $5656 = 0, $5657 = 0, $5658 = 0, $5659 = 0, $566 = 0;
 var $5660 = 0, $5661 = 0, $5662 = 0, $5663 = 0, $5664 = 0, $5665 = 0, $5666 = 0, $5667 = 0, $5668 = 0, $5669 = 0, $567 = 0, $5670 = 0, $5671 = 0, $5672 = 0, $5673 = 0, $5674 = 0, $5675 = 0, $5676 = 0, $5677 = 0, $5678 = 0;
 var $5679 = 0, $568 = 0, $5680 = 0, $5681 = 0, $5682 = 0, $5683 = 0, $5684 = 0, $5685 = 0, $5686 = 0, $5687 = 0, $5688 = 0, $5689 = 0, $569 = 0, $5690 = 0, $5691 = 0, $5692 = 0, $5693 = 0, $5694 = 0, $5695 = 0, $5696 = 0;
 var $5697 = 0, $5698 = 0, $5699 = 0, $57 = 0, $570 = 0, $5700 = 0, $5701 = 0, $5702 = 0, $5703 = 0, $5704 = 0, $5705 = 0, $5706 = 0, $5707 = 0, $5708 = 0, $5709 = 0, $571 = 0, $5710 = 0, $5711 = 0, $5712 = 0, $5713 = 0;
 var $5714 = 0, $5715 = 0, $5716 = 0, $5717 = 0, $5718 = 0, $5719 = 0, $572 = 0, $5720 = 0, $5721 = 0, $5722 = 0, $5723 = 0, $5724 = 0, $5725 = 0, $5726 = 0, $5727 = 0, $5728 = 0, $5729 = 0, $573 = 0, $5730 = 0, $5731 = 0;
 var $5732 = 0, $5733 = 0, $5734 = 0, $5735 = 0, $5736 = 0, $5737 = 0, $5738 = 0, $5739 = 0, $574 = 0, $5740 = 0, $5741 = 0, $5742 = 0, $5743 = 0, $5744 = 0, $5745 = 0, $5746 = 0, $5747 = 0, $5748 = 0, $5749 = 0, $575 = 0;
 var $5750 = 0, $5751 = 0, $5752 = 0, $5753 = 0, $5754 = 0, $5755 = 0, $5756 = 0, $5757 = 0, $5758 = 0, $5759 = 0, $576 = 0, $5760 = 0, $5761 = 0, $5762 = 0, $5763 = 0, $5764 = 0, $5765 = 0, $5766 = 0, $5767 = 0, $5768 = 0;
 var $5769 = 0, $577 = 0, $5770 = 0, $5771 = 0, $5772 = 0, $5773 = 0, $5774 = 0, $5775 = 0, $5776 = 0, $5777 = 0, $5778 = 0, $5779 = 0, $578 = 0, $5780 = 0, $5781 = 0, $5782 = 0, $5783 = 0, $5784 = 0, $5785 = 0, $5786 = 0;
 var $5787 = 0, $5788 = 0, $5789 = 0, $579 = 0, $5790 = 0, $5791 = 0, $5792 = 0, $5793 = 0, $5794 = 0, $5795 = 0, $5796 = 0, $5797 = 0, $5798 = 0, $5799 = 0, $58 = 0, $580 = 0, $5800 = 0, $5801 = 0, $5802 = 0, $5803 = 0;
 var $5804 = 0, $5805 = 0, $5806 = 0, $5807 = 0, $5808 = 0, $5809 = 0, $581 = 0, $5810 = 0, $5811 = 0, $5812 = 0, $5813 = 0, $5814 = 0, $5815 = 0, $5816 = 0, $5817 = 0, $5818 = 0, $5819 = 0, $582 = 0, $5820 = 0, $5821 = 0;
 var $5822 = 0, $5823 = 0, $5824 = 0, $5825 = 0, $5826 = 0, $5827 = 0, $5828 = 0, $5829 = 0, $583 = 0, $5830 = 0, $5831 = 0, $5832 = 0, $5833 = 0, $5834 = 0, $5835 = 0, $5836 = 0, $5837 = 0, $5838 = 0, $5839 = 0, $584 = 0;
 var $5840 = 0, $5841 = 0, $5842 = 0, $5843 = 0, $5844 = 0, $5845 = 0, $5846 = 0, $5847 = 0, $5848 = 0, $5849 = 0, $585 = 0, $5850 = 0, $5851 = 0, $5852 = 0, $5853 = 0, $5854 = 0, $5855 = 0, $5856 = 0, $5857 = 0, $5858 = 0;
 var $5859 = 0, $586 = 0, $5860 = 0, $5861 = 0, $5862 = 0, $5863 = 0, $5864 = 0, $5865 = 0, $5866 = 0, $5867 = 0, $5868 = 0, $5869 = 0, $587 = 0, $5870 = 0, $5871 = 0, $5872 = 0, $5873 = 0, $5874 = 0, $5875 = 0, $5876 = 0;
 var $5877 = 0, $5878 = 0, $5879 = 0, $588 = 0, $5880 = 0, $5881 = 0, $5882 = 0, $5883 = 0, $5884 = 0, $5885 = 0, $5886 = 0, $5887 = 0, $5888 = 0, $5889 = 0, $589 = 0, $5890 = 0, $5891 = 0, $5892 = 0, $5893 = 0, $5894 = 0;
 var $5895 = 0, $5896 = 0, $5897 = 0, $5898 = 0, $5899 = 0, $59 = 0, $590 = 0, $5900 = 0, $5901 = 0, $5902 = 0, $5903 = 0, $5904 = 0, $5905 = 0, $5906 = 0, $5907 = 0, $5908 = 0, $5909 = 0, $591 = 0, $5910 = 0, $5911 = 0;
 var $5912 = 0, $5913 = 0, $5914 = 0, $5915 = 0, $5916 = 0, $5917 = 0, $5918 = 0, $5919 = 0, $592 = 0, $5920 = 0, $5921 = 0, $5922 = 0, $5923 = 0, $5924 = 0, $5925 = 0, $5926 = 0, $5927 = 0, $5928 = 0, $5929 = 0, $593 = 0;
 var $5930 = 0, $5931 = 0, $5932 = 0, $5933 = 0, $5934 = 0, $5935 = 0, $5936 = 0, $5937 = 0, $5938 = 0, $5939 = 0, $594 = 0, $5940 = 0, $5941 = 0, $5942 = 0, $5943 = 0, $5944 = 0, $5945 = 0, $5946 = 0, $5947 = 0, $5948 = 0;
 var $5949 = 0, $595 = 0, $5950 = 0, $5951 = 0, $5952 = 0, $5953 = 0, $5954 = 0, $5955 = 0, $5956 = 0, $5957 = 0, $5958 = 0, $5959 = 0, $596 = 0, $5960 = 0, $5961 = 0, $5962 = 0, $5963 = 0, $5964 = 0, $5965 = 0, $5966 = 0;
 var $5967 = 0, $5968 = 0, $5969 = 0, $597 = 0, $5970 = 0, $5971 = 0, $5972 = 0, $5973 = 0, $5974 = 0, $5975 = 0, $5976 = 0, $5977 = 0, $5978 = 0, $5979 = 0, $598 = 0, $5980 = 0, $5981 = 0, $5982 = 0, $5983 = 0, $5984 = 0;
 var $5985 = 0, $5986 = 0, $5987 = 0, $5988 = 0, $5989 = 0, $599 = 0, $5990 = 0, $5991 = 0, $5992 = 0, $5993 = 0, $5994 = 0, $5995 = 0, $5996 = 0, $5997 = 0, $5998 = 0, $5999 = 0, $6 = 0, $60 = 0, $600 = 0, $6000 = 0;
 var $6001 = 0, $6002 = 0, $6003 = 0, $6004 = 0, $6005 = 0, $6006 = 0, $6007 = 0, $6008 = 0, $6009 = 0, $601 = 0, $6010 = 0, $6011 = 0, $6012 = 0, $6013 = 0, $6014 = 0, $6015 = 0, $6016 = 0, $6017 = 0, $6018 = 0, $6019 = 0;
 var $602 = 0, $6020 = 0, $6021 = 0, $6022 = 0, $6023 = 0, $6024 = 0, $6025 = 0, $6026 = 0, $6027 = 0, $6028 = 0, $6029 = 0, $603 = 0, $6030 = 0, $6031 = 0, $6032 = 0, $6033 = 0, $6034 = 0, $6035 = 0, $6036 = 0, $6037 = 0;
 var $6038 = 0, $6039 = 0, $604 = 0, $6040 = 0, $6041 = 0, $6042 = 0, $6043 = 0, $6044 = 0, $6045 = 0, $6046 = 0, $6047 = 0, $6048 = 0, $6049 = 0, $605 = 0, $6050 = 0, $6051 = 0, $6052 = 0, $6053 = 0, $6054 = 0, $6055 = 0;
 var $6056 = 0, $6057 = 0, $6058 = 0, $6059 = 0, $606 = 0, $6060 = 0, $6061 = 0, $6062 = 0, $6063 = 0, $6064 = 0, $6065 = 0, $6066 = 0, $6067 = 0, $6068 = 0, $6069 = 0, $607 = 0, $6070 = 0, $6071 = 0, $6072 = 0, $6073 = 0;
 var $6074 = 0, $6075 = 0, $6076 = 0, $6077 = 0, $6078 = 0, $6079 = 0, $608 = 0, $6080 = 0, $6081 = 0, $6082 = 0, $6083 = 0, $6084 = 0, $6085 = 0, $6086 = 0, $6087 = 0, $6088 = 0, $6089 = 0, $609 = 0, $6090 = 0, $6091 = 0;
 var $6092 = 0, $6093 = 0, $6094 = 0, $6095 = 0, $6096 = 0, $6097 = 0, $6098 = 0, $6099 = 0, $61 = 0, $610 = 0, $6100 = 0, $6101 = 0, $6102 = 0, $6103 = 0, $6104 = 0, $6105 = 0, $6106 = 0, $6107 = 0, $6108 = 0, $6109 = 0;
 var $611 = 0, $6110 = 0, $6111 = 0, $6112 = 0, $6113 = 0, $6114 = 0, $6115 = 0, $6116 = 0, $6117 = 0, $6118 = 0, $6119 = 0, $612 = 0, $6120 = 0, $6121 = 0, $6122 = 0, $6123 = 0, $6124 = 0, $6125 = 0, $6126 = 0, $6127 = 0;
 var $6128 = 0, $6129 = 0, $613 = 0, $6130 = 0, $6131 = 0, $6132 = 0, $6133 = 0, $6134 = 0, $6135 = 0, $6136 = 0, $6137 = 0, $6138 = 0, $6139 = 0, $614 = 0, $6140 = 0, $6141 = 0, $6142 = 0, $6143 = 0, $6144 = 0, $6145 = 0;
 var $6146 = 0, $6147 = 0, $6148 = 0, $6149 = 0, $615 = 0, $6150 = 0, $6151 = 0, $6152 = 0, $6153 = 0, $6154 = 0, $6155 = 0, $6156 = 0, $6157 = 0, $6158 = 0, $6159 = 0, $616 = 0, $6160 = 0, $6161 = 0, $6162 = 0, $6163 = 0;
 var $6164 = 0, $6165 = 0, $6166 = 0, $6167 = 0, $6168 = 0, $6169 = 0, $617 = 0, $6170 = 0, $6171 = 0, $6172 = 0, $6173 = 0, $6174 = 0, $6175 = 0, $6176 = 0, $6177 = 0, $6178 = 0, $6179 = 0, $618 = 0, $6180 = 0, $6181 = 0;
 var $6182 = 0, $6183 = 0, $6184 = 0, $6185 = 0, $6186 = 0, $6187 = 0, $6188 = 0, $6189 = 0, $619 = 0, $6190 = 0, $6191 = 0, $6192 = 0, $6193 = 0, $6194 = 0, $6195 = 0, $6196 = 0, $6197 = 0, $6198 = 0, $6199 = 0, $62 = 0;
 var $620 = 0, $6200 = 0, $6201 = 0, $6202 = 0, $6203 = 0, $6204 = 0, $6205 = 0, $6206 = 0, $6207 = 0, $6208 = 0, $6209 = 0, $621 = 0, $6210 = 0, $6211 = 0, $6212 = 0, $6213 = 0, $6214 = 0, $6215 = 0, $6216 = 0, $6217 = 0;
 var $6218 = 0, $6219 = 0, $622 = 0, $6220 = 0, $6221 = 0, $6222 = 0, $6223 = 0, $6224 = 0, $6225 = 0, $6226 = 0, $6227 = 0, $6228 = 0, $6229 = 0, $623 = 0, $6230 = 0, $6231 = 0, $6232 = 0, $6233 = 0, $6234 = 0, $6235 = 0;
 var $6236 = 0, $6237 = 0, $6238 = 0, $6239 = 0, $624 = 0, $6240 = 0, $6241 = 0, $6242 = 0, $6243 = 0, $6244 = 0, $6245 = 0, $6246 = 0, $6247 = 0, $6248 = 0, $6249 = 0, $625 = 0, $6250 = 0, $6251 = 0, $6252 = 0, $6253 = 0;
 var $6254 = 0, $6255 = 0, $6256 = 0, $6257 = 0, $6258 = 0, $6259 = 0, $626 = 0, $6260 = 0, $6261 = 0, $6262 = 0, $6263 = 0, $6264 = 0, $6265 = 0, $6266 = 0, $6267 = 0, $6268 = 0, $6269 = 0, $627 = 0, $6270 = 0, $6271 = 0;
 var $6272 = 0, $6273 = 0, $6274 = 0, $6275 = 0, $6276 = 0, $6277 = 0, $6278 = 0, $6279 = 0, $628 = 0, $6280 = 0, $6281 = 0, $6282 = 0, $6283 = 0, $6284 = 0, $6285 = 0, $6286 = 0, $6287 = 0, $6288 = 0, $6289 = 0, $629 = 0;
 var $6290 = 0, $6291 = 0, $6292 = 0, $6293 = 0, $6294 = 0, $6295 = 0, $6296 = 0, $6297 = 0, $6298 = 0, $6299 = 0, $63 = 0, $630 = 0, $6300 = 0, $6301 = 0, $6302 = 0, $6303 = 0, $6304 = 0, $6305 = 0, $6306 = 0, $6307 = 0;
 var $6308 = 0, $6309 = 0, $631 = 0, $6310 = 0, $6311 = 0, $6312 = 0, $6313 = 0, $6314 = 0, $6315 = 0, $6316 = 0, $6317 = 0, $6318 = 0, $6319 = 0, $632 = 0, $6320 = 0, $6321 = 0, $6322 = 0, $6323 = 0, $6324 = 0, $6325 = 0;
 var $6326 = 0, $6327 = 0, $6328 = 0, $6329 = 0, $633 = 0, $6330 = 0, $6331 = 0, $6332 = 0, $6333 = 0, $6334 = 0, $6335 = 0, $6336 = 0, $6337 = 0, $6338 = 0, $6339 = 0, $634 = 0, $6340 = 0, $6341 = 0, $6342 = 0, $6343 = 0;
 var $6344 = 0, $6345 = 0, $6346 = 0, $6347 = 0, $6348 = 0, $6349 = 0, $635 = 0, $6350 = 0, $6351 = 0, $6352 = 0, $6353 = 0, $6354 = 0, $6355 = 0, $6356 = 0, $6357 = 0, $6358 = 0, $6359 = 0, $636 = 0, $6360 = 0, $6361 = 0;
 var $6362 = 0, $6363 = 0, $6364 = 0, $6365 = 0, $6366 = 0, $6367 = 0, $6368 = 0, $6369 = 0, $637 = 0, $6370 = 0, $6371 = 0, $6372 = 0, $6373 = 0, $6374 = 0, $6375 = 0, $6376 = 0, $6377 = 0, $6378 = 0, $6379 = 0, $638 = 0;
 var $6380 = 0, $6381 = 0, $6382 = 0, $6383 = 0, $6384 = 0, $6385 = 0, $6386 = 0, $6387 = 0, $6388 = 0, $6389 = 0, $639 = 0, $6390 = 0, $6391 = 0, $6392 = 0, $6393 = 0, $6394 = 0, $6395 = 0, $6396 = 0, $6397 = 0, $6398 = 0;
 var $6399 = 0, $64 = 0, $640 = 0, $6400 = 0, $6401 = 0, $6402 = 0, $6403 = 0, $6404 = 0, $6405 = 0, $6406 = 0, $6407 = 0, $6408 = 0, $6409 = 0, $641 = 0, $6410 = 0, $6411 = 0, $6412 = 0, $6413 = 0, $6414 = 0, $6415 = 0;
 var $6416 = 0, $6417 = 0, $6418 = 0, $6419 = 0, $642 = 0, $6420 = 0, $6421 = 0, $6422 = 0, $6423 = 0, $6424 = 0, $6425 = 0, $6426 = 0, $6427 = 0, $6428 = 0, $6429 = 0, $643 = 0, $6430 = 0, $6431 = 0, $6432 = 0, $6433 = 0;
 var $6434 = 0, $6435 = 0, $6436 = 0, $6437 = 0, $6438 = 0, $6439 = 0, $644 = 0, $6440 = 0, $6441 = 0, $6442 = 0, $6443 = 0, $6444 = 0, $6445 = 0, $6446 = 0, $6447 = 0, $6448 = 0, $6449 = 0, $645 = 0, $6450 = 0, $6451 = 0;
 var $6452 = 0, $6453 = 0, $6454 = 0, $6455 = 0, $6456 = 0, $6457 = 0, $6458 = 0, $6459 = 0, $646 = 0, $6460 = 0, $6461 = 0, $6462 = 0, $6463 = 0, $6464 = 0, $6465 = 0, $6466 = 0, $6467 = 0, $6468 = 0, $6469 = 0, $647 = 0;
 var $6470 = 0, $6471 = 0, $6472 = 0, $6473 = 0, $6474 = 0, $6475 = 0, $6476 = 0, $6477 = 0, $6478 = 0, $6479 = 0, $648 = 0, $6480 = 0, $6481 = 0, $6482 = 0, $6483 = 0, $6484 = 0, $6485 = 0, $6486 = 0, $6487 = 0, $6488 = 0;
 var $6489 = 0, $649 = 0, $6490 = 0, $6491 = 0, $6492 = 0, $6493 = 0, $6494 = 0, $6495 = 0, $6496 = 0, $6497 = 0, $6498 = 0, $6499 = 0, $65 = 0, $650 = 0, $6500 = 0, $6501 = 0, $6502 = 0, $6503 = 0, $6504 = 0, $6505 = 0;
 var $6506 = 0, $6507 = 0, $6508 = 0, $6509 = 0, $651 = 0, $6510 = 0, $6511 = 0, $6512 = 0, $6513 = 0, $6514 = 0, $6515 = 0, $6516 = 0, $6517 = 0, $6518 = 0, $6519 = 0, $652 = 0, $6520 = 0, $6521 = 0, $6522 = 0, $6523 = 0;
 var $6524 = 0, $6525 = 0, $6526 = 0, $6527 = 0, $6528 = 0, $6529 = 0, $653 = 0, $6530 = 0, $6531 = 0, $6532 = 0, $6533 = 0, $6534 = 0, $6535 = 0, $6536 = 0, $6537 = 0, $6538 = 0, $6539 = 0, $654 = 0, $6540 = 0, $6541 = 0;
 var $6542 = 0, $6543 = 0, $6544 = 0, $6545 = 0, $6546 = 0, $6547 = 0, $6548 = 0, $6549 = 0, $655 = 0, $6550 = 0, $6551 = 0, $6552 = 0, $6553 = 0, $6554 = 0, $6555 = 0, $6556 = 0, $6557 = 0, $6558 = 0, $6559 = 0, $656 = 0;
 var $6560 = 0, $6561 = 0, $6562 = 0, $6563 = 0, $6564 = 0, $6565 = 0, $6566 = 0, $6567 = 0, $6568 = 0, $6569 = 0, $657 = 0, $6570 = 0, $6571 = 0, $6572 = 0, $6573 = 0, $6574 = 0, $6575 = 0, $6576 = 0, $6577 = 0, $6578 = 0;
 var $6579 = 0, $658 = 0, $6580 = 0, $6581 = 0, $6582 = 0, $6583 = 0, $6584 = 0, $6585 = 0, $6586 = 0, $6587 = 0, $6588 = 0, $6589 = 0, $659 = 0, $6590 = 0, $6591 = 0, $6592 = 0, $6593 = 0, $6594 = 0, $6595 = 0, $6596 = 0;
 var $6597 = 0, $6598 = 0, $6599 = 0, $66 = 0, $660 = 0, $6600 = 0, $6601 = 0, $6602 = 0, $6603 = 0, $6604 = 0, $6605 = 0, $6606 = 0, $6607 = 0, $6608 = 0, $6609 = 0, $661 = 0, $6610 = 0, $6611 = 0, $6612 = 0, $6613 = 0;
 var $6614 = 0, $6615 = 0, $6616 = 0, $6617 = 0, $6618 = 0, $6619 = 0, $662 = 0, $6620 = 0, $6621 = 0, $6622 = 0, $6623 = 0, $6624 = 0, $6625 = 0, $6626 = 0, $6627 = 0, $6628 = 0, $6629 = 0, $663 = 0, $6630 = 0, $6631 = 0;
 var $6632 = 0, $6633 = 0, $6634 = 0, $6635 = 0, $6636 = 0, $6637 = 0, $6638 = 0, $6639 = 0, $664 = 0, $6640 = 0, $6641 = 0, $6642 = 0, $6643 = 0, $6644 = 0, $6645 = 0, $6646 = 0, $6647 = 0, $6648 = 0, $6649 = 0, $665 = 0;
 var $6650 = 0, $6651 = 0, $6652 = 0, $6653 = 0, $6654 = 0, $6655 = 0, $6656 = 0, $6657 = 0, $6658 = 0, $6659 = 0, $666 = 0, $6660 = 0, $6661 = 0, $6662 = 0, $6663 = 0, $6664 = 0, $6665 = 0, $6666 = 0, $6667 = 0, $6668 = 0;
 var $6669 = 0, $667 = 0, $6670 = 0, $6671 = 0, $6672 = 0, $6673 = 0, $6674 = 0, $6675 = 0, $6676 = 0, $6677 = 0, $6678 = 0, $6679 = 0, $668 = 0, $6680 = 0, $6681 = 0, $6682 = 0, $6683 = 0, $6684 = 0, $6685 = 0, $6686 = 0;
 var $6687 = 0, $6688 = 0, $6689 = 0, $669 = 0, $6690 = 0, $6691 = 0, $6692 = 0, $6693 = 0, $6694 = 0, $6695 = 0, $6696 = 0, $6697 = 0, $6698 = 0, $6699 = 0, $67 = 0, $670 = 0, $6700 = 0, $6701 = 0, $6702 = 0, $6703 = 0;
 var $6704 = 0, $6705 = 0, $6706 = 0, $6707 = 0, $6708 = 0, $6709 = 0, $671 = 0, $6710 = 0, $6711 = 0, $6712 = 0, $6713 = 0, $6714 = 0, $6715 = 0, $6716 = 0, $6717 = 0, $6718 = 0, $6719 = 0, $672 = 0, $6720 = 0, $6721 = 0;
 var $6722 = 0, $6723 = 0, $6724 = 0, $6725 = 0, $6726 = 0, $6727 = 0, $6728 = 0, $6729 = 0, $673 = 0, $6730 = 0, $6731 = 0, $6732 = 0, $6733 = 0, $6734 = 0, $6735 = 0, $6736 = 0, $6737 = 0, $6738 = 0, $6739 = 0, $674 = 0;
 var $6740 = 0, $6741 = 0, $6742 = 0, $6743 = 0, $6744 = 0, $6745 = 0, $6746 = 0, $6747 = 0, $6748 = 0, $6749 = 0, $675 = 0, $6750 = 0, $6751 = 0, $6752 = 0, $6753 = 0, $6754 = 0, $6755 = 0, $6756 = 0, $6757 = 0, $6758 = 0;
 var $6759 = 0, $676 = 0, $6760 = 0, $6761 = 0, $6762 = 0, $6763 = 0, $6764 = 0, $6765 = 0, $6766 = 0, $6767 = 0, $6768 = 0, $6769 = 0, $677 = 0, $6770 = 0, $6771 = 0, $6772 = 0, $6773 = 0, $6774 = 0, $6775 = 0, $6776 = 0;
 var $6777 = 0, $6778 = 0, $6779 = 0, $678 = 0, $6780 = 0, $6781 = 0, $6782 = 0, $6783 = 0, $6784 = 0, $6785 = 0, $6786 = 0, $6787 = 0, $6788 = 0, $6789 = 0, $679 = 0, $6790 = 0, $6791 = 0, $6792 = 0, $6793 = 0, $6794 = 0;
 var $6795 = 0, $6796 = 0, $6797 = 0, $6798 = 0, $6799 = 0, $68 = 0, $680 = 0, $6800 = 0, $6801 = 0, $6802 = 0, $6803 = 0, $6804 = 0, $6805 = 0, $6806 = 0, $6807 = 0, $6808 = 0, $6809 = 0, $681 = 0, $6810 = 0, $6811 = 0;
 var $6812 = 0, $6813 = 0, $6814 = 0, $6815 = 0, $6816 = 0, $6817 = 0, $6818 = 0, $6819 = 0, $682 = 0, $6820 = 0, $6821 = 0, $6822 = 0, $6823 = 0, $6824 = 0, $6825 = 0, $6826 = 0, $6827 = 0, $6828 = 0, $6829 = 0, $683 = 0;
 var $6830 = 0, $6831 = 0, $6832 = 0, $6833 = 0, $6834 = 0, $6835 = 0, $6836 = 0, $6837 = 0, $6838 = 0, $6839 = 0, $684 = 0, $6840 = 0, $6841 = 0, $6842 = 0, $6843 = 0, $6844 = 0, $6845 = 0, $6846 = 0, $6847 = 0, $6848 = 0;
 var $6849 = 0, $685 = 0, $6850 = 0, $6851 = 0, $6852 = 0, $6853 = 0, $6854 = 0, $6855 = 0, $6856 = 0, $6857 = 0, $6858 = 0, $6859 = 0, $686 = 0, $6860 = 0, $6861 = 0, $6862 = 0, $6863 = 0, $6864 = 0, $6865 = 0, $6866 = 0;
 var $6867 = 0, $6868 = 0, $6869 = 0, $687 = 0, $6870 = 0, $6871 = 0, $6872 = 0, $6873 = 0, $6874 = 0, $6875 = 0, $6876 = 0, $6877 = 0, $6878 = 0, $6879 = 0, $688 = 0, $6880 = 0, $6881 = 0, $6882 = 0, $6883 = 0, $6884 = 0;
 var $6885 = 0, $6886 = 0, $6887 = 0, $6888 = 0, $6889 = 0, $689 = 0, $6890 = 0, $6891 = 0, $6892 = 0, $6893 = 0, $6894 = 0, $6895 = 0, $6896 = 0, $6897 = 0, $6898 = 0, $6899 = 0, $69 = 0, $690 = 0, $6900 = 0, $6901 = 0;
 var $6902 = 0, $6903 = 0, $6904 = 0, $6905 = 0, $6906 = 0, $6907 = 0, $6908 = 0, $6909 = 0, $691 = 0, $6910 = 0, $6911 = 0, $6912 = 0, $6913 = 0, $6914 = 0, $6915 = 0, $6916 = 0, $6917 = 0, $6918 = 0, $6919 = 0, $692 = 0;
 var $6920 = 0, $6921 = 0, $6922 = 0, $6923 = 0, $6924 = 0, $6925 = 0, $6926 = 0, $6927 = 0, $6928 = 0, $6929 = 0, $693 = 0, $6930 = 0, $6931 = 0, $6932 = 0, $6933 = 0, $6934 = 0, $6935 = 0, $6936 = 0, $6937 = 0, $6938 = 0;
 var $6939 = 0, $694 = 0, $6940 = 0, $6941 = 0, $6942 = 0, $6943 = 0, $6944 = 0, $6945 = 0, $6946 = 0, $6947 = 0, $6948 = 0, $6949 = 0, $695 = 0, $6950 = 0, $6951 = 0, $6952 = 0, $6953 = 0, $6954 = 0, $6955 = 0, $6956 = 0;
 var $6957 = 0, $6958 = 0, $6959 = 0, $696 = 0, $6960 = 0, $6961 = 0, $6962 = 0, $6963 = 0, $6964 = 0, $6965 = 0, $6966 = 0, $6967 = 0, $6968 = 0, $6969 = 0, $697 = 0, $6970 = 0, $6971 = 0, $6972 = 0, $6973 = 0, $6974 = 0;
 var $6975 = 0, $6976 = 0, $6977 = 0, $6978 = 0, $6979 = 0, $698 = 0, $6980 = 0, $6981 = 0, $6982 = 0, $6983 = 0, $6984 = 0, $6985 = 0, $6986 = 0, $6987 = 0, $6988 = 0, $6989 = 0, $699 = 0, $6990 = 0, $6991 = 0, $6992 = 0;
 var $6993 = 0, $6994 = 0, $6995 = 0, $6996 = 0, $6997 = 0, $6998 = 0, $6999 = 0, $7 = 0, $70 = 0, $700 = 0, $7000 = 0, $7001 = 0, $7002 = 0, $7003 = 0, $7004 = 0, $7005 = 0, $7006 = 0, $7007 = 0, $7008 = 0, $7009 = 0;
 var $701 = 0, $7010 = 0, $7011 = 0, $7012 = 0, $7013 = 0, $7014 = 0, $702 = 0, $703 = 0, $704 = 0, $705 = 0, $706 = 0, $707 = 0, $708 = 0, $709 = 0, $71 = 0, $710 = 0, $711 = 0, $712 = 0, $713 = 0, $714 = 0;
 var $715 = 0, $716 = 0, $717 = 0, $718 = 0, $719 = 0, $72 = 0, $720 = 0, $721 = 0, $722 = 0, $723 = 0, $724 = 0, $725 = 0, $726 = 0, $727 = 0, $728 = 0, $729 = 0, $73 = 0, $730 = 0, $731 = 0, $732 = 0;
 var $733 = 0, $734 = 0, $735 = 0, $736 = 0, $737 = 0, $738 = 0, $739 = 0, $74 = 0, $740 = 0, $741 = 0, $742 = 0, $743 = 0, $744 = 0, $745 = 0, $746 = 0, $747 = 0, $748 = 0, $749 = 0, $75 = 0, $750 = 0;
 var $751 = 0, $752 = 0, $753 = 0, $754 = 0, $755 = 0, $756 = 0, $757 = 0, $758 = 0, $759 = 0, $76 = 0, $760 = 0, $761 = 0, $762 = 0, $763 = 0, $764 = 0, $765 = 0, $766 = 0, $767 = 0, $768 = 0, $769 = 0;
 var $77 = 0, $770 = 0, $771 = 0, $772 = 0, $773 = 0, $774 = 0, $775 = 0, $776 = 0, $777 = 0, $778 = 0, $779 = 0, $78 = 0, $780 = 0, $781 = 0, $782 = 0, $783 = 0, $784 = 0, $785 = 0, $786 = 0, $787 = 0;
 var $788 = 0, $789 = 0, $79 = 0, $790 = 0, $791 = 0, $792 = 0, $793 = 0, $794 = 0, $795 = 0, $796 = 0, $797 = 0, $798 = 0, $799 = 0, $8 = 0, $80 = 0, $800 = 0, $801 = 0, $802 = 0, $803 = 0, $804 = 0;
 var $805 = 0, $806 = 0, $807 = 0, $808 = 0, $809 = 0, $81 = 0, $810 = 0, $811 = 0, $812 = 0, $813 = 0, $814 = 0, $815 = 0, $816 = 0, $817 = 0, $818 = 0, $819 = 0, $82 = 0, $820 = 0, $821 = 0, $822 = 0;
 var $823 = 0, $824 = 0, $825 = 0, $826 = 0, $827 = 0, $828 = 0, $829 = 0, $83 = 0, $830 = 0, $831 = 0, $832 = 0, $833 = 0, $834 = 0, $835 = 0, $836 = 0, $837 = 0, $838 = 0, $839 = 0, $84 = 0, $840 = 0;
 var $841 = 0, $842 = 0, $843 = 0, $844 = 0, $845 = 0, $846 = 0, $847 = 0, $848 = 0, $849 = 0, $85 = 0, $850 = 0, $851 = 0, $852 = 0, $853 = 0, $854 = 0, $855 = 0, $856 = 0, $857 = 0, $858 = 0, $859 = 0;
 var $86 = 0, $860 = 0, $861 = 0, $862 = 0, $863 = 0, $864 = 0, $865 = 0, $866 = 0, $867 = 0, $868 = 0, $869 = 0, $87 = 0, $870 = 0, $871 = 0, $872 = 0, $873 = 0, $874 = 0, $875 = 0, $876 = 0, $877 = 0;
 var $878 = 0, $879 = 0, $88 = 0, $880 = 0, $881 = 0, $882 = 0, $883 = 0, $884 = 0, $885 = 0, $886 = 0, $887 = 0, $888 = 0, $889 = 0, $89 = 0, $890 = 0, $891 = 0, $892 = 0, $893 = 0, $894 = 0, $895 = 0;
 var $896 = 0, $897 = 0, $898 = 0, $899 = 0, $9 = 0, $90 = 0, $900 = 0, $901 = 0, $902 = 0, $903 = 0, $904 = 0, $905 = 0, $906 = 0, $907 = 0, $908 = 0, $909 = 0, $91 = 0, $910 = 0, $911 = 0, $912 = 0;
 var $913 = 0, $914 = 0, $915 = 0, $916 = 0, $917 = 0, $918 = 0, $919 = 0, $92 = 0, $920 = 0, $921 = 0, $922 = 0, $923 = 0, $924 = 0, $925 = 0, $926 = 0, $927 = 0, $928 = 0, $929 = 0, $93 = 0, $930 = 0;
 var $931 = 0, $932 = 0, $933 = 0, $934 = 0, $935 = 0, $936 = 0, $937 = 0, $938 = 0, $939 = 0, $94 = 0, $940 = 0, $941 = 0, $942 = 0, $943 = 0, $944 = 0, $945 = 0, $946 = 0, $947 = 0, $948 = 0, $949 = 0;
 var $95 = 0, $950 = 0, $951 = 0, $952 = 0, $953 = 0, $954 = 0, $955 = 0, $956 = 0, $957 = 0, $958 = 0, $959 = 0, $96 = 0, $960 = 0, $961 = 0, $962 = 0, $963 = 0, $964 = 0, $965 = 0, $966 = 0, $967 = 0;
 var $968 = 0, $969 = 0, $97 = 0, $970 = 0, $971 = 0, $972 = 0, $973 = 0, $974 = 0, $975 = 0, $976 = 0, $977 = 0, $978 = 0, $979 = 0, $98 = 0, $980 = 0, $981 = 0, $982 = 0, $983 = 0, $984 = 0, $985 = 0;
 var $986 = 0, $987 = 0, $988 = 0, $989 = 0, $99 = 0, $990 = 0, $991 = 0, $992 = 0, $993 = 0, $994 = 0, $995 = 0, $996 = 0, $997 = 0, $998 = 0, $999 = 0, $aMem = 0, $aOffset = 0, $aOp = 0, $aPermute = 0, $aRes = 0;
 var $aRoot = 0, $aTempRec = 0, $affinity = 0, $alreadyExists = 0, $apArg = 0, $apArg129 = 0, $apVal = 0, $apVal98 = 0, $avail = 0, $bIntint = 0, $bRev = 0, $c = 0, $cAff = 0, $cnt = 0, $cnt43 = 0, $ctx = 0, $ctx97 = 0, $db = 0, $desiredAutoCommit = 0, $eNew = 0;
 var $eOld = 0, $encoding = 0, $exists = 0, $file_format = 0, $flags = 0, $flags1 = 0, $flags3 = 0, $flags82 = 0, $fx = 0, $i = 0, $i100 = 0, $i11 = 0, $i114 = 0, $i127 = 0, $i14 = 0, $i2 = 0, $i7 = 0, $i93 = 0, $iA = 0, $iA4 = 0;
 var $iB = 0, $iB5 = 0, $iCnt = 0, $iCompare = 0, $iCookie = 0, $iDb = 0, $iDb22 = 0, $iDb79 = 0, $iDb84 = 0, $iGen = 0, $iKey = 0, $iKey40 = 0, $iKey46 = 0, $iMeta = 0, $iMeta18 = 0, $iMoved = 0, $iQuery = 0, $iRollback = 0, $iSavepoint = 0, $iSet = 0;
 var $idx = 0, $ii = 0, $ii33 = 0, $initData = 0, $isSchemaChange = 0, $isTransaction = 0, $isWriteLock = 0, $j = 0, $j85 = 0, $lastRowid = 0, $len = 0, $len15 = 0, $n = 0, $n1 = 0, $n3 = 0, $n56 = 0, $n6 = 0, $n64 = 0, $n92 = 0, $nArg = 0;
 var $nArg126 = 0, $nByte = 0, $nByte12 = 0, $nByte86 = 0, $nChange = 0, $nChange$ = 0, $nData = 0, $nEntry = 0, $nErr = 0, $nField = 0, $nField19 = 0, $nField31 = 0, $nHdr = 0, $nKey = 0, $nKeyCol = 0, $nMem = 0, $nName = 0, $nProgressLimit = 0, $nRoot = 0, $nVarint = 0;
 var $nVmStep = 0, $nZero = 0, $nZero48 = 0, $newMax = 0, $nullFlag = 0, $oc = 0, $offset = 0, $op = 0, $op49 = 0, $or$cond = 0, $or$cond11 = 0, $or$cond13 = 0, $or$cond15 = 0, $or$cond18 = 0, $or$cond20 = 0, $or$cond22 = 0, $or$cond24 = 0, $or$cond26 = 0, $or$cond3 = 0, $or$cond5 = 0;
 var $or$cond7 = 0, $or$cond9 = 0, $p1 = 0, $p1104 = 0, $p117 = 0, $p18 = 0, $p2 = 0, $p210 = 0, $p221 = 0, $p29 = 0, $pArg = 0, $pArgc = 0, $pBt = 0, $pBt102 = 0, $pBt103 = 0, $pBt131 = 0, $pC = 0, $pC27 = 0, $pC30 = 0, $pC32 = 0;
 var $pC34 = 0, $pC37 = 0, $pC41 = 0, $pC47 = 0, $pC50 = 0, $pC51 = 0, $pC53 = 0, $pC54 = 0, $pC57 = 0, $pC59 = 0, $pC60 = 0, $pC63 = 0, $pC66 = 0, $pC68 = 0, $pC70 = 0, $pC75 = 0, $pC76 = 0, $pC80 = 0, $pCaller = 0, $pColl = 0;
 var $pCrsr = 0, $pCrsr16 = 0, $pCrsr38 = 0, $pCrsr55 = 0, $pCrsr61 = 0, $pCrsr64 = 0, $pCrsr69 = 0, $pCrsr71 = 0, $pCrsr74 = 0, $pCur = 0, $pCur106 = 0, $pCur112 = 0, $pCur118 = 0, $pCur122 = 0, $pCur23 = 0, $pCx = 0, $pCx26 = 0, $pCx28 = 0, $pData = 0, $pData0 = 0;
 var $pDb = 0, $pDb24 = 0, $pDb83 = 0, $pDest = 0, $pDest117 = 0, $pEnd = 0, $pFrame = 0, $pFrame45 = 0, $pFrame88 = 0, $pFrame90 = 0, $pFrame91 = 0, $pFree = 0, $pIdxKey = 0, $pIn = 0, $pIn1 = 0, $pIn2 = 0, $pIn3 = 0, $pKey = 0, $pKeyInfo = 0, $pKeyInfo20 = 0;
 var $pKeyInfo25 = 0, $pLast = 0, $pMem = 0, $pMem101 = 0, $pMem44 = 0, $pMem87 = 0, $pMem94 = 0, $pMem99 = 0, $pModule = 0, $pModule108 = 0, $pModule109 = 0, $pModule116 = 0, $pModule120 = 0, $pModule125 = 0, $pName = 0, $pNew = 0, $pOp = 0, $pOut = 0, $pPager = 0, $pProgram = 0;
 var $pQuery = 0, $pRec = 0, $pRec95 = 0, $pReg = 0, $pRt = 0, $pSavepoint = 0, $pTmp = 0, $pVTab = 0, $pVar = 0, $pVdbe = 0, $pVtab = 0, $pVtab107 = 0, $pVtab111 = 0, $pVtab115 = 0, $pVtab119 = 0, $pVtab123 = 0, $pVtab124 = 0, $pVtabCursor = 0, $pVtabCursor110 = 0, $pX = 0;
 var $pX130 = 0, $payloadSize64 = 0, $pc = 0, $pcDest = 0, $pgno = 0, $pgno81 = 0, $pnErr = 0, $r = 0, $r36 = 0, $r73 = 0, $r78 = 0, $rA = 0, $rB = 0, $rc = 0, $res = 0, $res113 = 0, $res121 = 0, $res29 = 0, $res35 = 0, $res39 = 0;
 var $res42 = 0, $res52 = 0, $res62 = 0, $res65 = 0, $res67 = 0, $res72 = 0, $res77 = 0, $resetSchemaOnFault = 0, $rowid = 0, $rowid128 = 0, $sContext = 0, $sMem = 0, $seekResult = 0, $serial_type = 0, $szField = 0, $t = 0, $t89 = 0, $t96 = 0, $turnOnAC = 0, $type1 = 0;
 var $type2 = 0, $uA = 0, $v = 0, $v1 = 0, $v2 = 0, $v58 = 0, $val = 0, $vararg_buffer84 = 0, $vararg_ptr27 = 0, $vararg_ptr37 = 0, $vararg_ptr38 = 0, $vararg_ptr56 = 0, $vararg_ptr57 = 0, $vararg_ptr75 = 0, $vararg_ptr76 = 0, $vtabOnConflict = 0, $wrFlag = 0, $z = 0, $z105 = 0, $z132 = 0;
 var $zAffinity = 0, $zAffinity13 = 0, $zData = 0, $zDb = 0, $zEndHdr = 0, $zFilename = 0, $zHdr = 0, $zKey = 0, $zLogFmt = 0, $zMaster = 0, $zName = 0, $zNewRecord = 0, $zSql = 0, $zTbl = 0, $zTrace = 0, $zType = 0, dest = 0, label = 0, sp = 0, stop = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 1696|0;
 $vararg_buffer84 = sp + 256|0;
 $0 = sp + 1464|0;
 $pc = sp + 1460|0;
 $aOp = sp + 1456|0;
 $pOp = sp + 1452|0;
 $rc = sp + 1448|0;
 $db = sp + 1444|0;
 $resetSchemaOnFault = sp + 1680|0;
 $encoding = sp + 1679|0;
 $iCompare = sp + 1440|0;
 $nVmStep = sp + 1436|0;
 $nProgressLimit = sp + 1432|0;
 $aMem = sp + 1428|0;
 $pIn1 = sp + 1424|0;
 $pIn2 = sp + 1420|0;
 $pIn3 = sp + 1416|0;
 $pOut = sp + 1412|0;
 $aPermute = sp + 1408|0;
 $lastRowid = sp + 248|0;
 $pCaller = sp + 1404|0;
 $pcDest = sp + 1400|0;
 $zType = sp + 1396|0;
 $zLogFmt = sp + 1392|0;
 $pFrame = sp + 1388|0;
 $cnt = sp + 1384|0;
 $nullFlag = sp + 1480|0;
 $pVar = sp + 1380|0;
 $n = sp + 1376|0;
 $p1 = sp + 1372|0;
 $p2 = sp + 1368|0;
 $n1 = sp + 1364|0;
 $pMem = sp + 1360|0;
 $i = sp + 1356|0;
 $nByte = sp + 240|0;
 $bIntint = sp + 1678|0;
 $flags = sp + 1478|0;
 $type1 = sp + 1476|0;
 $type2 = sp + 1474|0;
 $iA = sp + 232|0;
 $iB = sp + 224|0;
 $rA = sp + 216|0;
 $rB = sp + 208|0;
 $i2 = sp + 1352|0;
 $pArg = sp + 1348|0;
 $ctx = sp + 1320|0;
 $apVal = sp + 1316|0;
 $n3 = sp + 1312|0;
 $iA4 = sp + 200|0;
 $uA = sp + 192|0;
 $iB5 = sp + 184|0;
 $op = sp + 1677|0;
 $res = sp + 1308|0;
 $affinity = sp + 1676|0;
 $flags1 = sp + 1472|0;
 $flags3 = sp + 1470|0;
 $n6 = sp + 1304|0;
 $i7 = sp + 1300|0;
 $p18 = sp + 1296|0;
 $p29 = sp + 1292|0;
 $pKeyInfo = sp + 1288|0;
 $idx = sp + 1284|0;
 $pColl = sp + 1280|0;
 $bRev = sp + 1276|0;
 $v1 = sp + 1272|0;
 $v2 = sp + 1268|0;
 $c = sp + 1264|0;
 $payloadSize64 = sp + 176|0;
 $p210 = sp + 1260|0;
 $pC = sp + 1256|0;
 $pCrsr = sp + 1252|0;
 $aOffset = sp + 1248|0;
 $len = sp + 1244|0;
 $i11 = sp + 1240|0;
 $pDest = sp + 1236|0;
 $sMem = sp + 136|0;
 $zData = sp + 1232|0;
 $zHdr = sp + 1228|0;
 $zEndHdr = sp + 1224|0;
 $offset = sp + 1220|0;
 $szField = sp + 1216|0;
 $avail = sp + 1212|0;
 $t = sp + 1208|0;
 $fx = sp + 1468|0;
 $pReg = sp + 1204|0;
 $zAffinity = sp + 1200|0;
 $cAff = sp + 1675|0;
 $zNewRecord = sp + 1196|0;
 $pRec = sp + 1192|0;
 $nData = sp + 128|0;
 $nHdr = sp + 1188|0;
 $nByte12 = sp + 120|0;
 $nZero = sp + 1184|0;
 $nVarint = sp + 1180|0;
 $serial_type = sp + 1176|0;
 $pData0 = sp + 1172|0;
 $pLast = sp + 1168|0;
 $nField = sp + 1164|0;
 $zAffinity13 = sp + 1160|0;
 $file_format = sp + 1156|0;
 $i14 = sp + 1152|0;
 $j = sp + 1148|0;
 $len15 = sp + 1144|0;
 $nEntry = sp + 112|0;
 $pCrsr16 = sp + 1140|0;
 $p117 = sp + 1136|0;
 $zName = sp + 1132|0;
 $nName = sp + 1128|0;
 $pNew = sp + 1124|0;
 $pSavepoint = sp + 1120|0;
 $pTmp = sp + 1116|0;
 $iSavepoint = sp + 1112|0;
 $ii = sp + 1108|0;
 $isTransaction = sp + 1104|0;
 $isSchemaChange = sp + 1100|0;
 $desiredAutoCommit = sp + 1096|0;
 $iRollback = sp + 1092|0;
 $turnOnAC = sp + 1088|0;
 $pBt = sp + 1084|0;
 $iMeta = sp + 1080|0;
 $iGen = sp + 1076|0;
 $iMeta18 = sp + 1072|0;
 $iDb = sp + 1068|0;
 $iCookie = sp + 1064|0;
 $pDb = sp + 1060|0;
 $pCur = sp + 1056|0;
 $nField19 = sp + 1052|0;
 $pKeyInfo20 = sp + 1048|0;
 $p221 = sp + 1044|0;
 $iDb22 = sp + 1040|0;
 $wrFlag = sp + 1036|0;
 $pX = sp + 1032|0;
 $pCur23 = sp + 1028|0;
 $pDb24 = sp + 1024|0;
 $pCx = sp + 1020|0;
 $pKeyInfo25 = sp + 1016|0;
 $pgno = sp + 1012|0;
 $pCx26 = sp + 1008|0;
 $pC27 = sp + 1004|0;
 $pCx28 = sp + 1000|0;
 $res29 = sp + 996|0;
 $oc = sp + 992|0;
 $pC30 = sp + 988|0;
 $r = sp + 968|0;
 $nField31 = sp + 964|0;
 $iKey = sp + 104|0;
 $pC32 = sp + 960|0;
 $alreadyExists = sp + 956|0;
 $ii33 = sp + 952|0;
 $pC34 = sp + 948|0;
 $res35 = sp + 944|0;
 $pFree = sp + 940|0;
 $pIdxKey = sp + 936|0;
 $r36 = sp + 916|0;
 $aTempRec = sp + 1484|0;
 $pC37 = sp + 912|0;
 $pCrsr38 = sp + 908|0;
 $res39 = sp + 904|0;
 $iKey40 = sp + 96|0;
 $v = sp + 88|0;
 $pC41 = sp + 900|0;
 $res42 = sp + 896|0;
 $cnt43 = sp + 892|0;
 $pMem44 = sp + 888|0;
 $pFrame45 = sp + 884|0;
 $pData = sp + 880|0;
 $pKey = sp + 876|0;
 $iKey46 = sp + 80|0;
 $pC47 = sp + 872|0;
 $nZero48 = sp + 868|0;
 $seekResult = sp + 864|0;
 $zDb = sp + 860|0;
 $zTbl = sp + 856|0;
 $op49 = sp + 852|0;
 $pC50 = sp + 848|0;
 $pC51 = sp + 844|0;
 $res52 = sp + 840|0;
 $nKeyCol = sp + 836|0;
 $pC53 = sp + 832|0;
 $pC54 = sp + 828|0;
 $pCrsr55 = sp + 824|0;
 $n56 = sp + 820|0;
 $n64 = sp + 72|0;
 $pC57 = sp + 816|0;
 $v58 = sp + 64|0;
 $pVtab = sp + 812|0;
 $pModule = sp + 808|0;
 $pC59 = sp + 804|0;
 $pC60 = sp + 800|0;
 $pCrsr61 = sp + 796|0;
 $res62 = sp + 792|0;
 $pC63 = sp + 788|0;
 $pCrsr64 = sp + 784|0;
 $res65 = sp + 780|0;
 $pC66 = sp + 776|0;
 $res67 = sp + 772|0;
 $pC68 = sp + 768|0;
 $pCrsr69 = sp + 764|0;
 $nKey = sp + 760|0;
 $zKey = sp + 756|0;
 $pC70 = sp + 752|0;
 $pCrsr71 = sp + 748|0;
 $res72 = sp + 744|0;
 $r73 = sp + 724|0;
 $pCrsr74 = sp + 720|0;
 $pC75 = sp + 716|0;
 $rowid = sp + 56|0;
 $pC76 = sp + 712|0;
 $res77 = sp + 708|0;
 $r78 = sp + 688|0;
 $iMoved = sp + 684|0;
 $iCnt = sp + 680|0;
 $pVdbe = sp + 676|0;
 $iDb79 = sp + 672|0;
 $nChange = sp + 668|0;
 $pC80 = sp + 664|0;
 $pgno81 = sp + 660|0;
 $flags82 = sp + 656|0;
 $pDb83 = sp + 652|0;
 $iDb84 = sp + 648|0;
 $zMaster = sp + 644|0;
 $zSql = sp + 640|0;
 $initData = sp + 624|0;
 $nRoot = sp + 616|0;
 $aRoot = sp + 612|0;
 $j85 = sp + 608|0;
 $nErr = sp + 604|0;
 $z = sp + 600|0;
 $pnErr = sp + 596|0;
 $val = sp + 48|0;
 $iSet = sp + 592|0;
 $exists = sp + 588|0;
 $nMem = sp + 584|0;
 $nByte86 = sp + 580|0;
 $pRt = sp + 576|0;
 $pMem87 = sp + 572|0;
 $pEnd = sp + 568|0;
 $pFrame88 = sp + 564|0;
 $pProgram = sp + 560|0;
 $t89 = sp + 556|0;
 $pFrame90 = sp + 552|0;
 $pIn = sp + 548|0;
 $pFrame91 = sp + 544|0;
 $n92 = sp + 540|0;
 $i93 = sp + 536|0;
 $pMem94 = sp + 532|0;
 $pRec95 = sp + 528|0;
 $t96 = sp + 8|0;
 $ctx97 = sp + 500|0;
 $apVal98 = sp + 496|0;
 $pMem99 = sp + 492|0;
 $i100 = sp + 488|0;
 $aRes = sp + 476|0;
 $pMem101 = sp + 472|0;
 $pBt102 = sp + 468|0;
 $pPager = sp + 464|0;
 $eNew = sp + 460|0;
 $eOld = sp + 456|0;
 $zFilename = sp + 452|0;
 $pBt103 = sp + 448|0;
 $isWriteLock = sp + 1483|0;
 $p1104 = sp + 444|0;
 $z105 = sp + 440|0;
 $pVTab = sp + 436|0;
 $pCur106 = sp + 432|0;
 $pVtabCursor = sp + 428|0;
 $pVtab107 = sp + 424|0;
 $pModule108 = sp + 420|0;
 $nArg = sp + 416|0;
 $iQuery = sp + 412|0;
 $pModule109 = sp + 408|0;
 $pQuery = sp + 404|0;
 $pArgc = sp + 400|0;
 $pVtabCursor110 = sp + 396|0;
 $pVtab111 = sp + 392|0;
 $pCur112 = sp + 388|0;
 $res113 = sp + 384|0;
 $i114 = sp + 380|0;
 $apArg = sp + 376|0;
 $pVtab115 = sp + 372|0;
 $pModule116 = sp + 368|0;
 $pDest117 = sp + 364|0;
 $sContext = sp + 336|0;
 $pCur118 = sp + 332|0;
 $pVtab119 = sp + 328|0;
 $pModule120 = sp + 324|0;
 $res121 = sp + 320|0;
 $pCur122 = sp + 316|0;
 $pVtab123 = sp + 312|0;
 $pName = sp + 308|0;
 $pVtab124 = sp + 304|0;
 $pModule125 = sp + 300|0;
 $nArg126 = sp + 296|0;
 $i127 = sp + 292|0;
 $rowid128 = sp;
 $apArg129 = sp + 288|0;
 $pX130 = sp + 284|0;
 $vtabOnConflict = sp + 1482|0;
 $newMax = sp + 280|0;
 $pBt131 = sp + 276|0;
 $zTrace = sp + 272|0;
 $z132 = sp + 268|0;
 HEAP32[$0>>2] = $p;
 HEAP32[$pc>>2] = 0;
 $1 = HEAP32[$0>>2]|0;
 $2 = (($1) + 4|0);
 $3 = HEAP32[$2>>2]|0;
 HEAP32[$aOp>>2] = $3;
 HEAP32[$rc>>2] = 0;
 $4 = HEAP32[$0>>2]|0;
 $5 = HEAP32[$4>>2]|0;
 HEAP32[$db>>2] = $5;
 HEAP8[$resetSchemaOnFault>>0] = 0;
 $6 = HEAP32[$db>>2]|0;
 $7 = (($6) + 16|0);
 $8 = HEAP32[$7>>2]|0;
 $9 = (($8) + 12|0);
 $10 = HEAP32[$9>>2]|0;
 $11 = (($10) + 77|0);
 $12 = HEAP8[$11>>0]|0;
 HEAP8[$encoding>>0] = $12;
 HEAP32[$iCompare>>2] = 0;
 HEAP32[$nVmStep>>2] = 0;
 HEAP32[$nProgressLimit>>2] = 0;
 $13 = HEAP32[$0>>2]|0;
 $14 = (($13) + 8|0);
 $15 = HEAP32[$14>>2]|0;
 HEAP32[$aMem>>2] = $15;
 HEAP32[$pIn1>>2] = 0;
 HEAP32[$pIn2>>2] = 0;
 HEAP32[$pIn3>>2] = 0;
 HEAP32[$pOut>>2] = 0;
 HEAP32[$aPermute>>2] = 0;
 $16 = HEAP32[$db>>2]|0;
 $17 = (($16) + 32|0);
 $18 = $17;
 $19 = $18;
 $20 = HEAP32[$19>>2]|0;
 $21 = (($18) + 4)|0;
 $22 = $21;
 $23 = HEAP32[$22>>2]|0;
 $24 = $lastRowid;
 $25 = $24;
 HEAP32[$25>>2] = $20;
 $26 = (($24) + 4)|0;
 $27 = $26;
 HEAP32[$27>>2] = $23;
 $28 = HEAP32[$0>>2]|0;
 $29 = (($28) + 80|0);
 $30 = HEAP32[$29>>2]|0;
 $31 = ($30|0)==(7);
 L1: do {
  if ($31) {
   label = 871;
  } else {
   $32 = HEAP32[$0>>2]|0;
   $33 = (($32) + 80|0);
   HEAP32[$33>>2] = 0;
   $34 = HEAP32[$0>>2]|0;
   $35 = (($34) + 136|0);
   $36 = $35;
   $37 = $36;
   HEAP32[$37>>2] = 0;
   $38 = (($36) + 4)|0;
   $39 = $38;
   HEAP32[$39>>2] = 0;
   $40 = HEAP32[$0>>2]|0;
   $41 = (($40) + 20|0);
   HEAP32[$41>>2] = 0;
   $42 = HEAP32[$db>>2]|0;
   $43 = (($42) + 444|0);
   $44 = (($43) + 8|0);
   HEAP32[$44>>2] = 0;
   $45 = HEAP32[$db>>2]|0;
   $46 = (($45) + 240|0);
   $47 = HEAP32[$46>>2]|0;
   $48 = ($47|0)!=(0);
   L3: do {
    if (!($48)) {
     $49 = HEAP32[$db>>2]|0;
     $50 = (($49) + 292|0);
     $51 = HEAP32[$50>>2]|0;
     $52 = ($51|0)!=(0|0);
     do {
      if ($52) {
       $53 = HEAP32[$0>>2]|0;
       $54 = (($53) + 108|0);
       $55 = (($54) + 16|0);
       $56 = HEAP32[$55>>2]|0;
       HEAP32[$nProgressLimit>>2] = $56;
       $57 = HEAP32[$nProgressLimit>>2]|0;
       $58 = ($57|0)==(0);
       $59 = HEAP32[$db>>2]|0;
       $60 = (($59) + 300|0);
       $61 = HEAP32[$60>>2]|0;
       if ($58) {
        HEAP32[$nProgressLimit>>2] = $61;
        break;
       } else {
        $62 = HEAP32[$nProgressLimit>>2]|0;
        $63 = (($62>>>0) % ($61>>>0))&-1;
        HEAP32[$nProgressLimit>>2] = $63;
        break;
       }
      }
     } while(0);
     $64 = HEAP32[$0>>2]|0;
     $65 = (($64) + 76|0);
     $66 = HEAP32[$65>>2]|0;
     HEAP32[$pc>>2] = $66;
     L11: while(1) {
      $67 = HEAP32[$rc>>2]|0;
      $68 = ($67|0)==(0);
      if (!($68)) {
       break L1;
      }
      $69 = HEAP32[$db>>2]|0;
      $70 = (($69) + 64|0);
      $71 = HEAP8[$70>>0]|0;
      $72 = ($71<<24>>24)!=(0);
      if ($72) {
       label = 871;
       break L1;
      }
      $73 = HEAP32[$nVmStep>>2]|0;
      $74 = (($73) + 1)|0;
      HEAP32[$nVmStep>>2] = $74;
      $75 = HEAP32[$pc>>2]|0;
      $76 = HEAP32[$aOp>>2]|0;
      $77 = (($76) + (($75*20)|0)|0);
      HEAP32[$pOp>>2] = $77;
      $78 = HEAP32[$pOp>>2]|0;
      $79 = (($78) + 2|0);
      $80 = HEAP8[$79>>0]|0;
      $81 = $80&255;
      $82 = $81 & 2;
      $83 = ($82|0)!=(0);
      if ($83) {
       $84 = HEAP32[$pOp>>2]|0;
       $85 = (($84) + 8|0);
       $86 = HEAP32[$85>>2]|0;
       $87 = HEAP32[$aMem>>2]|0;
       $88 = (($87) + (($86*40)|0)|0);
       HEAP32[$pOut>>2] = $88;
       $89 = HEAP32[$pOut>>2]|0;
       $90 = (($89) + 8|0);
       $91 = HEAP16[$90>>1]|0;
       $92 = $91&65535;
       $93 = $92 & 9312;
       $94 = ($93|0)!=(0);
       if ($94) {
        $95 = HEAP32[$pOut>>2]|0;
        _sqlite3VdbeMemSetNull($95);
       }
       $96 = HEAP32[$pOut>>2]|0;
       $97 = (($96) + 8|0);
       HEAP16[$97>>1] = 4;
      }
      $98 = HEAP32[$pOp>>2]|0;
      $99 = HEAP8[$98>>0]|0;
      $100 = $99&255;
      L21: do {
       switch ($100|0) {
       case 27:  {
        label = 57;
        break;
       }
       case 22:  {
        $214 = HEAP32[$pOp>>2]|0;
        $215 = (($214) + 4|0);
        $216 = HEAP32[$215>>2]|0;
        $217 = HEAP32[$aMem>>2]|0;
        $218 = (($217) + (($216*40)|0)|0);
        HEAP32[$pIn1>>2] = $218;
        $219 = HEAP32[$pIn1>>2]|0;
        $220 = (($219) + 8|0);
        HEAP16[$220>>1] = 4;
        $221 = HEAP32[$pIn1>>2]|0;
        $222 = $221;
        $223 = $222;
        $224 = HEAP32[$223>>2]|0;
        $225 = (($222) + 4)|0;
        $226 = $225;
        $227 = HEAP32[$226>>2]|0;
        HEAP32[$pcDest>>2] = $224;
        $228 = HEAP32[$pc>>2]|0;
        $229 = ($228|0)<(0);
        $230 = $229 << 31 >> 31;
        $231 = HEAP32[$pIn1>>2]|0;
        $232 = $231;
        $233 = $232;
        HEAP32[$233>>2] = $228;
        $234 = (($232) + 4)|0;
        $235 = $234;
        HEAP32[$235>>2] = $230;
        $236 = HEAP32[$pcDest>>2]|0;
        HEAP32[$pc>>2] = $236;
        break;
       }
       case 28:  {
        $483 = HEAP32[$pOp>>2]|0;
        $484 = (($483) + 12|0);
        $485 = HEAP32[$484>>2]|0;
        $486 = HEAP32[$pOp>>2]|0;
        $487 = (($486) + 8|0);
        $488 = HEAP32[$487>>2]|0;
        $489 = (($485) - ($488))|0;
        HEAP32[$cnt>>2] = $489;
        $490 = HEAP32[$pOp>>2]|0;
        $491 = (($490) + 4|0);
        $492 = HEAP32[$491>>2]|0;
        $493 = ($492|0)!=(0);
        $494 = $493 ? 257 : 1;
        $495 = $494&65535;
        HEAP16[$nullFlag>>1] = $495;
        $496 = HEAP32[$pOut>>2]|0;
        $497 = (($496) + 8|0);
        HEAP16[$497>>1] = $495;
        while(1) {
         $498 = HEAP32[$cnt>>2]|0;
         $499 = ($498|0)>(0);
         if (!($499)) {
          break L21;
         }
         $500 = HEAP32[$pOut>>2]|0;
         $501 = (($500) + 40|0);
         HEAP32[$pOut>>2] = $501;
         $502 = HEAP32[$pOut>>2]|0;
         _sqlite3VdbeMemSetNull($502);
         $503 = HEAP16[$nullFlag>>1]|0;
         $504 = HEAP32[$pOut>>2]|0;
         $505 = (($504) + 8|0);
         HEAP16[$505>>1] = $503;
         $506 = HEAP32[$cnt>>2]|0;
         $507 = (($506) + -1)|0;
         HEAP32[$cnt>>2] = $507;
        }
        break;
       }
       case 17:  {
        $135 = HEAP32[$pOp>>2]|0;
        $136 = (($135) + 4|0);
        $137 = HEAP32[$136>>2]|0;
        $138 = HEAP32[$aMem>>2]|0;
        $139 = (($138) + (($137*40)|0)|0);
        HEAP32[$pIn1>>2] = $139;
        $140 = HEAP32[$pIn1>>2]|0;
        $141 = (($140) + 8|0);
        HEAP16[$141>>1] = 4;
        $142 = HEAP32[$pc>>2]|0;
        $143 = ($142|0)<(0);
        $144 = $143 << 31 >> 31;
        $145 = HEAP32[$pIn1>>2]|0;
        $146 = $145;
        $147 = $146;
        HEAP32[$147>>2] = $142;
        $148 = (($146) + 4)|0;
        $149 = $148;
        HEAP32[$149>>2] = $144;
        $150 = HEAP32[$pOp>>2]|0;
        $151 = (($150) + 8|0);
        $152 = HEAP32[$151>>2]|0;
        $153 = (($152) - 1)|0;
        HEAP32[$pc>>2] = $153;
        break;
       }
       case 21:  {
        $194 = HEAP32[$pOp>>2]|0;
        $195 = (($194) + 4|0);
        $196 = HEAP32[$195>>2]|0;
        $197 = HEAP32[$aMem>>2]|0;
        $198 = (($197) + (($196*40)|0)|0);
        HEAP32[$pIn1>>2] = $198;
        $199 = HEAP32[$pIn1>>2]|0;
        $200 = $199;
        $201 = $200;
        $202 = HEAP32[$201>>2]|0;
        $203 = (($200) + 4)|0;
        $204 = $203;
        $205 = HEAP32[$204>>2]|0;
        $206 = HEAP32[$aOp>>2]|0;
        $207 = (($206) + (($202*20)|0)|0);
        HEAP32[$pCaller>>2] = $207;
        $208 = HEAP32[$pCaller>>2]|0;
        $209 = (($208) + 8|0);
        $210 = HEAP32[$209>>2]|0;
        $211 = (($210) - 1)|0;
        HEAP32[$pc>>2] = $211;
        $212 = HEAP32[$pIn1>>2]|0;
        $213 = (($212) + 8|0);
        HEAP16[$213>>1] = 128;
        break;
       }
       case 16:  {
        $101 = HEAP32[$pOp>>2]|0;
        $102 = (($101) + 8|0);
        $103 = HEAP32[$102>>2]|0;
        $104 = (($103) - 1)|0;
        HEAP32[$pc>>2] = $104;
        label = 16;
        break;
       }
       case 20:  {
        $168 = HEAP32[$pOp>>2]|0;
        $169 = (($168) + 4|0);
        $170 = HEAP32[$169>>2]|0;
        $171 = HEAP32[$aMem>>2]|0;
        $172 = (($171) + (($170*40)|0)|0);
        HEAP32[$pOut>>2] = $172;
        $173 = HEAP32[$pOp>>2]|0;
        $174 = (($173) + 12|0);
        $175 = HEAP32[$174>>2]|0;
        $176 = (($175) - 1)|0;
        $177 = ($176|0)<(0);
        $178 = $177 << 31 >> 31;
        $179 = HEAP32[$pOut>>2]|0;
        $180 = $179;
        $181 = $180;
        HEAP32[$181>>2] = $176;
        $182 = (($180) + 4)|0;
        $183 = $182;
        HEAP32[$183>>2] = $178;
        $184 = HEAP32[$pOut>>2]|0;
        $185 = (($184) + 8|0);
        HEAP16[$185>>1] = 4;
        $186 = HEAP32[$pOp>>2]|0;
        $187 = (($186) + 8|0);
        $188 = HEAP32[$187>>2]|0;
        $189 = ($188|0)!=(0);
        if ($189) {
         $190 = HEAP32[$pOp>>2]|0;
         $191 = (($190) + 8|0);
         $192 = HEAP32[$191>>2]|0;
         $193 = (($192) - 1)|0;
         HEAP32[$pc>>2] = $193;
        }
        break;
       }
       case 25:  {
        $379 = HEAP32[$pOp>>2]|0;
        $380 = (($379) + 4|0);
        $381 = HEAP32[$380>>2]|0;
        $382 = ($381|0)<(0);
        $383 = $382 << 31 >> 31;
        $384 = HEAP32[$pOut>>2]|0;
        $385 = $384;
        $386 = $385;
        HEAP32[$386>>2] = $381;
        $387 = (($385) + 4)|0;
        $388 = $387;
        HEAP32[$388>>2] = $383;
        break;
       }
       case 18:  {
        $154 = HEAP32[$pOp>>2]|0;
        $155 = (($154) + 4|0);
        $156 = HEAP32[$155>>2]|0;
        $157 = HEAP32[$aMem>>2]|0;
        $158 = (($157) + (($156*40)|0)|0);
        HEAP32[$pIn1>>2] = $158;
        $159 = HEAP32[$pIn1>>2]|0;
        $160 = $159;
        $161 = $160;
        $162 = HEAP32[$161>>2]|0;
        $163 = (($160) + 4)|0;
        $164 = $163;
        $165 = HEAP32[$164>>2]|0;
        HEAP32[$pc>>2] = $162;
        $166 = HEAP32[$pIn1>>2]|0;
        $167 = (($166) + 8|0);
        HEAP16[$167>>1] = 128;
        break;
       }
       case 30:  {
        $522 = HEAP32[$pOut>>2]|0;
        $523 = HEAP32[$pOp>>2]|0;
        $524 = (($523) + 16|0);
        $525 = HEAP32[$524>>2]|0;
        $526 = HEAP32[$pOp>>2]|0;
        $527 = (($526) + 4|0);
        $528 = HEAP32[$527>>2]|0;
        (_sqlite3VdbeMemSetStr($522,$525,$528,0,0)|0);
        $529 = HEAP8[$encoding>>0]|0;
        $530 = HEAP32[$pOut>>2]|0;
        $531 = (($530) + 10|0);
        HEAP8[$531>>0] = $529;
        break;
       }
       case 31:  {
        $532 = HEAP32[$pOp>>2]|0;
        $533 = (($532) + 4|0);
        $534 = HEAP32[$533>>2]|0;
        $535 = (($534) - 1)|0;
        $536 = HEAP32[$0>>2]|0;
        $537 = (($536) + 60|0);
        $538 = HEAP32[$537>>2]|0;
        $539 = (($538) + (($535*40)|0)|0);
        HEAP32[$pVar>>2] = $539;
        $540 = HEAP32[$pVar>>2]|0;
        $541 = (_sqlite3VdbeMemTooBig($540)|0);
        $542 = ($541|0)!=(0);
        if ($542) {
         label = 870;
         break L11;
        }
        $543 = HEAP32[$pOut>>2]|0;
        $544 = HEAP32[$pVar>>2]|0;
        _sqlite3VdbeMemShallowCopy($543,$544,2048);
        break;
       }
       case 133:  {
        $403 = HEAP32[$pOut>>2]|0;
        $404 = (($403) + 8|0);
        HEAP16[$404>>1] = 8;
        $405 = HEAP32[$pOp>>2]|0;
        $406 = (($405) + 16|0);
        $407 = HEAP32[$406>>2]|0;
        $408 = +HEAPF64[$407>>3];
        $409 = HEAP32[$pOut>>2]|0;
        HEAPF64[$409>>3] = $408;
        break;
       }
       case 97:  {
        $410 = HEAP32[$pOp>>2]|0;
        HEAP8[$410>>0] = 27;
        $411 = HEAP32[$pOp>>2]|0;
        $412 = (($411) + 16|0);
        $413 = HEAP32[$412>>2]|0;
        $414 = (_sqlite3Strlen30($413)|0);
        $415 = HEAP32[$pOp>>2]|0;
        $416 = (($415) + 4|0);
        HEAP32[$416>>2] = $414;
        $417 = HEAP8[$encoding>>0]|0;
        $418 = $417&255;
        $419 = ($418|0)!=(1);
        if ($419) {
         $420 = HEAP32[$pOut>>2]|0;
         $421 = HEAP32[$pOp>>2]|0;
         $422 = (($421) + 16|0);
         $423 = HEAP32[$422>>2]|0;
         $424 = (_sqlite3VdbeMemSetStr($420,$423,-1,1,0)|0);
         HEAP32[$rc>>2] = $424;
         $425 = HEAP32[$rc>>2]|0;
         $426 = ($425|0)==(18);
         if ($426) {
          label = 870;
          break L11;
         }
         $427 = HEAP32[$pOut>>2]|0;
         $428 = HEAP8[$encoding>>0]|0;
         $429 = $428&255;
         $430 = (_sqlite3VdbeChangeEncoding($427,$429)|0);
         $431 = (0)!=($430|0);
         if ($431) {
          label = 871;
          break L1;
         }
         $432 = HEAP32[$pOut>>2]|0;
         $433 = (($432) + 24|0);
         HEAP32[$433>>2] = 0;
         $434 = HEAP32[$pOut>>2]|0;
         $435 = (($434) + 8|0);
         $436 = HEAP16[$435>>1]|0;
         $437 = $436&65535;
         $438 = $437 | 2048;
         $439 = $438&65535;
         HEAP16[$435>>1] = $439;
         $440 = HEAP32[$pOp>>2]|0;
         $441 = (($440) + 1|0);
         $442 = HEAP8[$441>>0]|0;
         $443 = $442 << 24 >> 24;
         $444 = ($443|0)==(-1);
         if ($444) {
          $445 = HEAP32[$db>>2]|0;
          $446 = HEAP32[$pOp>>2]|0;
          $447 = (($446) + 16|0);
          $448 = HEAP32[$447>>2]|0;
          _sqlite3DbFree($445,$448);
         }
         $449 = HEAP32[$pOp>>2]|0;
         $450 = (($449) + 1|0);
         HEAP8[$450>>0] = -1;
         $451 = HEAP32[$pOut>>2]|0;
         $452 = (($451) + 16|0);
         $453 = HEAP32[$452>>2]|0;
         $454 = HEAP32[$pOp>>2]|0;
         $455 = (($454) + 16|0);
         HEAP32[$455>>2] = $453;
         $456 = HEAP32[$pOut>>2]|0;
         $457 = (($456) + 12|0);
         $458 = HEAP32[$457>>2]|0;
         $459 = HEAP32[$pOp>>2]|0;
         $460 = (($459) + 4|0);
         HEAP32[$460>>2] = $458;
        }
        $461 = HEAP32[$pOp>>2]|0;
        $462 = (($461) + 4|0);
        $463 = HEAP32[$462>>2]|0;
        $464 = HEAP32[$db>>2]|0;
        $465 = (($464) + 88|0);
        $466 = HEAP32[$465>>2]|0;
        $467 = ($463|0)>($466|0);
        if ($467) {
         label = 870;
         break L11;
        } else {
         label = 57;
        }
        break;
       }
       case 24:  {
        label = 28;
        break;
       }
       case 26:  {
        $389 = HEAP32[$pOp>>2]|0;
        $390 = (($389) + 16|0);
        $391 = HEAP32[$390>>2]|0;
        $392 = $391;
        $393 = $392;
        $394 = HEAP32[$393>>2]|0;
        $395 = (($392) + 4)|0;
        $396 = $395;
        $397 = HEAP32[$396>>2]|0;
        $398 = HEAP32[$pOut>>2]|0;
        $399 = $398;
        $400 = $399;
        HEAP32[$400>>2] = $394;
        $401 = (($399) + 4)|0;
        $402 = $401;
        HEAP32[$402>>2] = $397;
        break;
       }
       case 23:  {
        $237 = HEAP32[$pOp>>2]|0;
        $238 = (($237) + 12|0);
        $239 = HEAP32[$238>>2]|0;
        $240 = HEAP32[$aMem>>2]|0;
        $241 = (($240) + (($239*40)|0)|0);
        HEAP32[$pIn3>>2] = $241;
        $242 = HEAP32[$pIn3>>2]|0;
        $243 = (($242) + 8|0);
        $244 = HEAP16[$243>>1]|0;
        $245 = $244&65535;
        $246 = $245 & 1;
        $247 = ($246|0)==(0);
        if (!($247)) {
         label = 28;
        }
        break;
       }
       case 29:  {
        $508 = HEAP32[$pOp>>2]|0;
        $509 = (($508) + 4|0);
        $510 = HEAP32[$509>>2]|0;
        $511 = HEAP32[$aMem>>2]|0;
        $512 = (($511) + (($510*40)|0)|0);
        HEAP32[$pOut>>2] = $512;
        $513 = HEAP32[$pOut>>2]|0;
        $514 = (($513) + 8|0);
        $515 = HEAP16[$514>>1]|0;
        $516 = $515&65535;
        $517 = $516 | 1;
        $518 = $517 & -129;
        $519 = $518&65535;
        $520 = HEAP32[$pOut>>2]|0;
        $521 = (($520) + 8|0);
        HEAP16[$521>>1] = $519;
        break;
       }
       case 37:  {
        $1502 = HEAP32[$pOp>>2]|0;
        $1503 = (($1502) + 4|0);
        $1504 = HEAP32[$1503>>2]|0;
        $1505 = HEAP32[$aMem>>2]|0;
        $1506 = (($1505) + (($1504*40)|0)|0);
        HEAP32[$pIn1>>2] = $1506;
        $1507 = HEAP32[$pIn1>>2]|0;
        (_sqlite3VdbeMemIntegerify($1507)|0);
        $1508 = HEAP32[$pOp>>2]|0;
        $1509 = (($1508) + 8|0);
        $1510 = HEAP32[$1509>>2]|0;
        $1511 = ($1510|0)<(0);
        $1512 = $1511 << 31 >> 31;
        $1513 = HEAP32[$pIn1>>2]|0;
        $1514 = $1513;
        $1515 = $1514;
        $1516 = HEAP32[$1515>>2]|0;
        $1517 = (($1514) + 4)|0;
        $1518 = $1517;
        $1519 = HEAP32[$1518>>2]|0;
        $1520 = (_i64Add(($1516|0),($1519|0),($1510|0),($1512|0))|0);
        $1521 = tempRet0;
        $1522 = $1513;
        $1523 = $1522;
        HEAP32[$1523>>2] = $1520;
        $1524 = (($1522) + 4)|0;
        $1525 = $1524;
        HEAP32[$1525>>2] = $1521;
        break;
       }
       case 38:  {
        $1526 = HEAP32[$pOp>>2]|0;
        $1527 = (($1526) + 4|0);
        $1528 = HEAP32[$1527>>2]|0;
        $1529 = HEAP32[$aMem>>2]|0;
        $1530 = (($1529) + (($1528*40)|0)|0);
        HEAP32[$pIn1>>2] = $1530;
        $1531 = HEAP32[$pIn1>>2]|0;
        $1532 = (($1531) + 8|0);
        $1533 = HEAP16[$1532>>1]|0;
        $1534 = $1533&65535;
        $1535 = $1534 & 4;
        $1536 = ($1535|0)==(0);
        if ($1536) {
         $1537 = HEAP32[$pIn1>>2]|0;
         $1538 = HEAP8[$encoding>>0]|0;
         _applyAffinity($1537,67,$1538);
         $1539 = HEAP32[$pIn1>>2]|0;
         $1540 = (($1539) + 8|0);
         $1541 = HEAP16[$1540>>1]|0;
         $1542 = $1541&65535;
         $1543 = $1542 & 4;
         $1544 = ($1543|0)==(0);
         if ($1544) {
          $1545 = HEAP32[$pOp>>2]|0;
          $1546 = (($1545) + 8|0);
          $1547 = HEAP32[$1546>>2]|0;
          $1548 = ($1547|0)==(0);
          if ($1548) {
           label = 167;
           break L11;
          }
          $1549 = HEAP32[$pOp>>2]|0;
          $1550 = (($1549) + 8|0);
          $1551 = HEAP32[$1550>>2]|0;
          $1552 = (($1551) - 1)|0;
          HEAP32[$pc>>2] = $1552;
          break L21;
         }
        }
        $1553 = HEAP32[$pIn1>>2]|0;
        $1554 = (($1553) + 8|0);
        $1555 = HEAP16[$1554>>1]|0;
        $1556 = $1555&65535;
        $1557 = $1556 & -16896;
        $1558 = $1557 | 4;
        $1559 = $1558&65535;
        $1560 = HEAP32[$pIn1>>2]|0;
        $1561 = (($1560) + 8|0);
        HEAP16[$1561>>1] = $1559;
        break;
       }
       case 40:  {
        $1574 = HEAP32[$pOp>>2]|0;
        $1575 = (($1574) + 4|0);
        $1576 = HEAP32[$1575>>2]|0;
        $1577 = HEAP32[$aMem>>2]|0;
        $1578 = (($1577) + (($1576*40)|0)|0);
        HEAP32[$pIn1>>2] = $1578;
        $1579 = HEAP32[$pIn1>>2]|0;
        $1580 = (($1579) + 8|0);
        $1581 = HEAP16[$1580>>1]|0;
        $1582 = $1581&65535;
        $1583 = $1582 & 16384;
        $1584 = ($1583|0)!=(0);
        if ($1584) {
         $1585 = HEAP32[$pIn1>>2]|0;
         $1586 = (_sqlite3VdbeMemExpandBlob($1585)|0);
         $1587 = $1586;
        } else {
         $1587 = 0;
        }
        HEAP32[$rc>>2] = $1587;
        $1588 = HEAP32[$pIn1>>2]|0;
        $1589 = HEAP32[$pOp>>2]|0;
        $1590 = (($1589) + 8|0);
        $1591 = HEAP32[$1590>>2]|0;
        $1592 = $1591&255;
        $1593 = HEAP8[$encoding>>0]|0;
        _sqlite3VdbeMemCast($1588,$1592,$1593);
        break;
       }
       case 39:  {
        $1562 = HEAP32[$pOp>>2]|0;
        $1563 = (($1562) + 4|0);
        $1564 = HEAP32[$1563>>2]|0;
        $1565 = HEAP32[$aMem>>2]|0;
        $1566 = (($1565) + (($1564*40)|0)|0);
        HEAP32[$pIn1>>2] = $1566;
        $1567 = HEAP32[$pIn1>>2]|0;
        $1568 = (($1567) + 8|0);
        $1569 = HEAP16[$1568>>1]|0;
        $1570 = $1569&65535;
        $1571 = $1570 & 4;
        $1572 = ($1571|0)!=(0);
        if ($1572) {
         $1573 = HEAP32[$pIn1>>2]|0;
         (_sqlite3VdbeMemRealify($1573)|0);
        }
        break;
       }
       case 83: case 80: case 81: case 82: case 78: case 79:  {
        $1594 = HEAP32[$pOp>>2]|0;
        $1595 = (($1594) + 4|0);
        $1596 = HEAP32[$1595>>2]|0;
        $1597 = HEAP32[$aMem>>2]|0;
        $1598 = (($1597) + (($1596*40)|0)|0);
        HEAP32[$pIn1>>2] = $1598;
        $1599 = HEAP32[$pOp>>2]|0;
        $1600 = (($1599) + 12|0);
        $1601 = HEAP32[$1600>>2]|0;
        $1602 = HEAP32[$aMem>>2]|0;
        $1603 = (($1602) + (($1601*40)|0)|0);
        HEAP32[$pIn3>>2] = $1603;
        $1604 = HEAP32[$pIn1>>2]|0;
        $1605 = (($1604) + 8|0);
        $1606 = HEAP16[$1605>>1]|0;
        HEAP16[$flags1>>1] = $1606;
        $1607 = HEAP32[$pIn3>>2]|0;
        $1608 = (($1607) + 8|0);
        $1609 = HEAP16[$1608>>1]|0;
        HEAP16[$flags3>>1] = $1609;
        $1610 = HEAP16[$flags1>>1]|0;
        $1611 = $1610&65535;
        $1612 = HEAP16[$flags3>>1]|0;
        $1613 = $1612&65535;
        $1614 = $1611 | $1613;
        $1615 = $1614 & 1;
        $1616 = ($1615|0)!=(0);
        $1617 = HEAP32[$pOp>>2]|0;
        $1618 = (($1617) + 3|0);
        $1619 = HEAP8[$1618>>0]|0;
        $1620 = $1619&255;
        do {
         if ($1616) {
          $1621 = $1620 & 128;
          $1622 = ($1621|0)!=(0);
          if (!($1622)) {
           $1635 = HEAP32[$pOp>>2]|0;
           $1636 = (($1635) + 3|0);
           $1637 = HEAP8[$1636>>0]|0;
           $1638 = $1637&255;
           $1639 = $1638 & 32;
           $1640 = ($1639|0)!=(0);
           $1641 = HEAP32[$pOp>>2]|0;
           if ($1640) {
            $1642 = (($1641) + 8|0);
            $1643 = HEAP32[$1642>>2]|0;
            $1644 = HEAP32[$aMem>>2]|0;
            $1645 = (($1644) + (($1643*40)|0)|0);
            HEAP32[$pOut>>2] = $1645;
            $1646 = HEAP32[$pOut>>2]|0;
            $1647 = (($1646) + 8|0);
            $1648 = HEAP16[$1647>>1]|0;
            $1649 = $1648&65535;
            $1650 = $1649 & -16896;
            $1651 = $1650 | 1;
            $1652 = $1651&65535;
            $1653 = HEAP32[$pOut>>2]|0;
            $1654 = (($1653) + 8|0);
            HEAP16[$1654>>1] = $1652;
            break L21;
           }
           $1655 = (($1641) + 3|0);
           $1656 = HEAP8[$1655>>0]|0;
           $1657 = $1656&255;
           $1658 = $1657 & 16;
           $1659 = ($1658|0)!=(0);
           if (!($1659)) {
            break L21;
           }
           $1660 = HEAP32[$pOp>>2]|0;
           $1661 = (($1660) + 8|0);
           $1662 = HEAP32[$1661>>2]|0;
           $1663 = (($1662) - 1)|0;
           HEAP32[$pc>>2] = $1663;
           break L21;
          }
          $1623 = HEAP16[$flags1>>1]|0;
          $1624 = $1623&65535;
          $1625 = $1624 & 1;
          $1626 = ($1625|0)!=(0);
          if ($1626) {
           $1627 = HEAP16[$flags3>>1]|0;
           $1628 = $1627&65535;
           $1629 = $1628 & 1;
           $1630 = ($1629|0)!=(0);
           if ($1630) {
            $1631 = HEAP16[$flags3>>1]|0;
            $1632 = $1631&65535;
            $1633 = $1632 & 256;
            $1634 = ($1633|0)==(0);
            if ($1634) {
             HEAP32[$res>>2] = 0;
             break;
            }
           }
          }
          HEAP32[$res>>2] = 1;
         } else {
          $1664 = $1620 & 71;
          $1665 = $1664&255;
          HEAP8[$affinity>>0] = $1665;
          $1666 = HEAP8[$affinity>>0]|0;
          $1667 = $1666 << 24 >> 24;
          $1668 = ($1667|0)>=(67);
          if ($1668) {
           $1669 = HEAP32[$pIn1>>2]|0;
           $1670 = (($1669) + 8|0);
           $1671 = HEAP16[$1670>>1]|0;
           $1672 = $1671&65535;
           $1673 = $1672 & 14;
           $1674 = ($1673|0)==(2);
           if ($1674) {
            $1675 = HEAP32[$pIn1>>2]|0;
            _applyNumericAffinity($1675,0);
           }
           $1676 = HEAP32[$pIn3>>2]|0;
           $1677 = (($1676) + 8|0);
           $1678 = HEAP16[$1677>>1]|0;
           $1679 = $1678&65535;
           $1680 = $1679 & 14;
           $1681 = ($1680|0)==(2);
           if ($1681) {
            $1682 = HEAP32[$pIn3>>2]|0;
            _applyNumericAffinity($1682,0);
           }
          } else {
           $1683 = HEAP8[$affinity>>0]|0;
           $1684 = $1683 << 24 >> 24;
           $1685 = ($1684|0)==(66);
           if ($1685) {
            $1686 = HEAP32[$pIn1>>2]|0;
            $1687 = (($1686) + 8|0);
            $1688 = HEAP16[$1687>>1]|0;
            $1689 = $1688&65535;
            $1690 = $1689 & 2;
            $1691 = ($1690|0)==(0);
            if ($1691) {
             $1692 = HEAP32[$pIn1>>2]|0;
             $1693 = (($1692) + 8|0);
             $1694 = HEAP16[$1693>>1]|0;
             $1695 = $1694&65535;
             $1696 = $1695 & 12;
             $1697 = ($1696|0)!=(0);
             if ($1697) {
              $1698 = HEAP32[$pIn1>>2]|0;
              $1699 = HEAP8[$encoding>>0]|0;
              (_sqlite3VdbeMemStringify($1698,$1699,1)|0);
             }
            }
            $1700 = HEAP32[$pIn3>>2]|0;
            $1701 = (($1700) + 8|0);
            $1702 = HEAP16[$1701>>1]|0;
            $1703 = $1702&65535;
            $1704 = $1703 & 2;
            $1705 = ($1704|0)==(0);
            if ($1705) {
             $1706 = HEAP32[$pIn3>>2]|0;
             $1707 = (($1706) + 8|0);
             $1708 = HEAP16[$1707>>1]|0;
             $1709 = $1708&65535;
             $1710 = $1709 & 12;
             $1711 = ($1710|0)!=(0);
             if ($1711) {
              $1712 = HEAP32[$pIn3>>2]|0;
              $1713 = HEAP8[$encoding>>0]|0;
              (_sqlite3VdbeMemStringify($1712,$1713,1)|0);
             }
            }
           }
          }
          $1714 = HEAP32[$pIn1>>2]|0;
          $1715 = (($1714) + 8|0);
          $1716 = HEAP16[$1715>>1]|0;
          $1717 = $1716&65535;
          $1718 = $1717 & 16384;
          $1719 = ($1718|0)!=(0);
          if ($1719) {
           $1720 = HEAP32[$pIn1>>2]|0;
           (_sqlite3VdbeMemExpandBlob($1720)|0);
           $1721 = HEAP16[$flags1>>1]|0;
           $1722 = $1721&65535;
           $1723 = $1722 & -16385;
           $1724 = $1723&65535;
           HEAP16[$flags1>>1] = $1724;
          }
          $1725 = HEAP32[$pIn3>>2]|0;
          $1726 = (($1725) + 8|0);
          $1727 = HEAP16[$1726>>1]|0;
          $1728 = $1727&65535;
          $1729 = $1728 & 16384;
          $1730 = ($1729|0)!=(0);
          if ($1730) {
           $1731 = HEAP32[$pIn3>>2]|0;
           (_sqlite3VdbeMemExpandBlob($1731)|0);
           $1732 = HEAP16[$flags3>>1]|0;
           $1733 = $1732&65535;
           $1734 = $1733 & -16385;
           $1735 = $1734&65535;
           HEAP16[$flags3>>1] = $1735;
          }
          $1736 = HEAP32[$db>>2]|0;
          $1737 = (($1736) + 64|0);
          $1738 = HEAP8[$1737>>0]|0;
          $1739 = ($1738<<24>>24)!=(0);
          if ($1739) {
           label = 871;
           break L1;
          }
          $1740 = HEAP32[$pIn3>>2]|0;
          $1741 = HEAP32[$pIn1>>2]|0;
          $1742 = HEAP32[$pOp>>2]|0;
          $1743 = (($1742) + 16|0);
          $1744 = HEAP32[$1743>>2]|0;
          $1745 = (_sqlite3MemCompare($1740,$1741,$1744)|0);
          HEAP32[$res>>2] = $1745;
         }
        } while(0);
        $1746 = HEAP32[$pOp>>2]|0;
        $1747 = HEAP8[$1746>>0]|0;
        $1748 = $1747&255;
        switch ($1748|0) {
        case 79:  {
         $1749 = HEAP32[$res>>2]|0;
         $1750 = ($1749|0)==(0);
         $1751 = $1750&1;
         HEAP32[$res>>2] = $1751;
         break;
        }
        case 82:  {
         $1755 = HEAP32[$res>>2]|0;
         $1756 = ($1755|0)<(0);
         $1757 = $1756&1;
         HEAP32[$res>>2] = $1757;
         break;
        }
        case 81:  {
         $1758 = HEAP32[$res>>2]|0;
         $1759 = ($1758|0)<=(0);
         $1760 = $1759&1;
         HEAP32[$res>>2] = $1760;
         break;
        }
        case 80:  {
         $1761 = HEAP32[$res>>2]|0;
         $1762 = ($1761|0)>(0);
         $1763 = $1762&1;
         HEAP32[$res>>2] = $1763;
         break;
        }
        case 78:  {
         $1752 = HEAP32[$res>>2]|0;
         $1753 = ($1752|0)!=(0);
         $1754 = $1753&1;
         HEAP32[$res>>2] = $1754;
         break;
        }
        default: {
         $1764 = HEAP32[$res>>2]|0;
         $1765 = ($1764|0)>=(0);
         $1766 = $1765&1;
         HEAP32[$res>>2] = $1766;
        }
        }
        $1767 = HEAP32[$pOp>>2]|0;
        $1768 = (($1767) + 3|0);
        $1769 = HEAP8[$1768>>0]|0;
        $1770 = $1769&255;
        $1771 = $1770 & 32;
        $1772 = ($1771|0)!=(0);
        if ($1772) {
         $1773 = HEAP32[$pOp>>2]|0;
         $1774 = (($1773) + 8|0);
         $1775 = HEAP32[$1774>>2]|0;
         $1776 = HEAP32[$aMem>>2]|0;
         $1777 = (($1776) + (($1775*40)|0)|0);
         HEAP32[$pOut>>2] = $1777;
         $1778 = HEAP32[$pOut>>2]|0;
         $1779 = (($1778) + 8|0);
         $1780 = HEAP16[$1779>>1]|0;
         $1781 = $1780&65535;
         $1782 = $1781 & -16896;
         $1783 = $1782 | 4;
         $1784 = $1783&65535;
         $1785 = HEAP32[$pOut>>2]|0;
         $1786 = (($1785) + 8|0);
         HEAP16[$1786>>1] = $1784;
         $1787 = HEAP32[$res>>2]|0;
         $1788 = ($1787|0)<(0);
         $1789 = $1788 << 31 >> 31;
         $1790 = HEAP32[$pOut>>2]|0;
         $1791 = $1790;
         $1792 = $1791;
         HEAP32[$1792>>2] = $1787;
         $1793 = (($1791) + 4)|0;
         $1794 = $1793;
         HEAP32[$1794>>2] = $1789;
        } else {
         $1795 = HEAP32[$res>>2]|0;
         $1796 = ($1795|0)!=(0);
         if ($1796) {
          $1797 = HEAP32[$pOp>>2]|0;
          $1798 = (($1797) + 8|0);
          $1799 = HEAP32[$1798>>2]|0;
          $1800 = (($1799) - 1)|0;
          HEAP32[$pc>>2] = $1800;
         }
        }
        $1801 = HEAP16[$flags1>>1]|0;
        $1802 = HEAP32[$pIn1>>2]|0;
        $1803 = (($1802) + 8|0);
        HEAP16[$1803>>1] = $1801;
        $1804 = HEAP16[$flags3>>1]|0;
        $1805 = HEAP32[$pIn3>>2]|0;
        $1806 = (($1805) + 8|0);
        HEAP16[$1806>>1] = $1804;
        break;
       }
       case 34:  {
        $600 = HEAP32[$pOp>>2]|0;
        $601 = (($600) + 4|0);
        $602 = HEAP32[$601>>2]|0;
        $603 = HEAP32[$aMem>>2]|0;
        $604 = (($603) + (($602*40)|0)|0);
        HEAP32[$pIn1>>2] = $604;
        $605 = HEAP32[$pOp>>2]|0;
        $606 = (($605) + 8|0);
        $607 = HEAP32[$606>>2]|0;
        $608 = HEAP32[$aMem>>2]|0;
        $609 = (($608) + (($607*40)|0)|0);
        HEAP32[$pOut>>2] = $609;
        $610 = HEAP32[$pOut>>2]|0;
        $611 = HEAP32[$pIn1>>2]|0;
        _sqlite3VdbeMemShallowCopy($610,$611,4096);
        break;
       }
       case 35:  {
        $612 = HEAP32[$db>>2]|0;
        $613 = (($612) + 292|0);
        $614 = HEAP32[$613>>2]|0;
        $615 = ($614|0)!=(0|0);
        if ($615) {
         $616 = HEAP32[$nVmStep>>2]|0;
         $617 = HEAP32[$nProgressLimit>>2]|0;
         $618 = ($616>>>0)>=($617>>>0);
         if ($618) {
          $619 = HEAP32[$db>>2]|0;
          $620 = (($619) + 292|0);
          $621 = HEAP32[$620>>2]|0;
          $622 = HEAP32[$db>>2]|0;
          $623 = (($622) + 296|0);
          $624 = HEAP32[$623>>2]|0;
          $625 = (FUNCTION_TABLE_ii[$621 & 31]($624)|0);
          $626 = ($625|0)!=(0);
          if ($626) {
           label = 76;
           break L11;
          }
         }
        }
        $627 = HEAP32[$0>>2]|0;
        $628 = (_sqlite3VdbeCheckFk($627,0)|0);
        HEAP32[$rc>>2] = $628;
        $629 = (0)!=($628|0);
        if (!($629)) {
         $630 = HEAP32[$0>>2]|0;
         $631 = (_sqlite3VdbeCloseStatement($630,1)|0);
         HEAP32[$rc>>2] = $631;
         $632 = HEAP32[$rc>>2]|0;
         $633 = ($632|0)!=(0);
         if (!($633)) {
          label = 79;
          break L11;
         }
        }
        break;
       }
       case 1:  {
        $1160 = HEAP32[$pOp>>2]|0;
        $1161 = (($1160) + 3|0);
        $1162 = HEAP8[$1161>>0]|0;
        $1163 = $1162&255;
        HEAP32[$n3>>2] = $1163;
        $1164 = HEAP32[$0>>2]|0;
        $1165 = (($1164) + 12|0);
        $1166 = HEAP32[$1165>>2]|0;
        HEAP32[$apVal>>2] = $1166;
        $1167 = HEAP32[$pOp>>2]|0;
        $1168 = (($1167) + 12|0);
        $1169 = HEAP32[$1168>>2]|0;
        $1170 = HEAP32[$aMem>>2]|0;
        $1171 = (($1170) + (($1169*40)|0)|0);
        HEAP32[$ctx>>2] = $1171;
        $1172 = HEAP32[$pOp>>2]|0;
        $1173 = (($1172) + 8|0);
        $1174 = HEAP32[$1173>>2]|0;
        $1175 = HEAP32[$aMem>>2]|0;
        $1176 = (($1175) + (($1174*40)|0)|0);
        HEAP32[$pArg>>2] = $1176;
        HEAP32[$i2>>2] = 0;
        while(1) {
         $1177 = HEAP32[$i2>>2]|0;
         $1178 = HEAP32[$n3>>2]|0;
         $1179 = ($1177|0)<($1178|0);
         if (!($1179)) {
          break;
         }
         $1180 = HEAP32[$pArg>>2]|0;
         $1181 = HEAP32[$i2>>2]|0;
         $1182 = HEAP32[$apVal>>2]|0;
         $1183 = (($1182) + ($1181<<2)|0);
         HEAP32[$1183>>2] = $1180;
         $1184 = HEAP32[$pArg>>2]|0;
         $1185 = (($1184) + 8|0);
         $1186 = HEAP16[$1185>>1]|0;
         $1187 = $1186&65535;
         $1188 = $1187 & 4096;
         $1189 = ($1188|0)!=(0);
         if ($1189) {
          $1190 = HEAP32[$pArg>>2]|0;
          $1191 = (_sqlite3VdbeMemMakeWriteable($1190)|0);
          $1192 = ($1191|0)!=(0);
          if ($1192) {
           label = 871;
           break L1;
          }
         }
         $1193 = HEAP32[$i2>>2]|0;
         $1194 = (($1193) + 1)|0;
         HEAP32[$i2>>2] = $1194;
         $1195 = HEAP32[$pArg>>2]|0;
         $1196 = (($1195) + 40|0);
         HEAP32[$pArg>>2] = $1196;
        }
        $1197 = HEAP32[$pOp>>2]|0;
        $1198 = (($1197) + 16|0);
        $1199 = HEAP32[$1198>>2]|0;
        $1200 = (($ctx) + 4|0);
        HEAP32[$1200>>2] = $1199;
        $1201 = HEAP32[$pc>>2]|0;
        $1202 = (($ctx) + 16|0);
        HEAP32[$1202>>2] = $1201;
        $1203 = HEAP32[$0>>2]|0;
        $1204 = (($ctx) + 12|0);
        HEAP32[$1204>>2] = $1203;
        $1205 = HEAP32[$ctx>>2]|0;
        $1206 = (($1205) + 8|0);
        $1207 = HEAP16[$1206>>1]|0;
        $1208 = $1207&65535;
        $1209 = $1208 & -16896;
        $1210 = $1209 | 1;
        $1211 = $1210&65535;
        $1212 = HEAP32[$ctx>>2]|0;
        $1213 = (($1212) + 8|0);
        HEAP16[$1213>>1] = $1211;
        $1214 = (($ctx) + 25|0);
        HEAP8[$1214>>0] = 0;
        $1215 = $lastRowid;
        $1216 = $1215;
        $1217 = HEAP32[$1216>>2]|0;
        $1218 = (($1215) + 4)|0;
        $1219 = $1218;
        $1220 = HEAP32[$1219>>2]|0;
        $1221 = HEAP32[$db>>2]|0;
        $1222 = (($1221) + 32|0);
        $1223 = $1222;
        $1224 = $1223;
        HEAP32[$1224>>2] = $1217;
        $1225 = (($1223) + 4)|0;
        $1226 = $1225;
        HEAP32[$1226>>2] = $1220;
        $1227 = (($ctx) + 4|0);
        $1228 = HEAP32[$1227>>2]|0;
        $1229 = (($1228) + 12|0);
        $1230 = HEAP32[$1229>>2]|0;
        $1231 = HEAP32[$n3>>2]|0;
        $1232 = HEAP32[$apVal>>2]|0;
        FUNCTION_TABLE_viii[$1230 & 63]($ctx,$1231,$1232);
        $1233 = HEAP32[$db>>2]|0;
        $1234 = (($1233) + 32|0);
        $1235 = $1234;
        $1236 = $1235;
        $1237 = HEAP32[$1236>>2]|0;
        $1238 = (($1235) + 4)|0;
        $1239 = $1238;
        $1240 = HEAP32[$1239>>2]|0;
        $1241 = $lastRowid;
        $1242 = $1241;
        HEAP32[$1242>>2] = $1237;
        $1243 = (($1241) + 4)|0;
        $1244 = $1243;
        HEAP32[$1244>>2] = $1240;
        $1245 = (($ctx) + 25|0);
        $1246 = HEAP8[$1245>>0]|0;
        $1247 = ($1246<<24>>24)!=(0);
        if ($1247) {
         $1248 = (($ctx) + 20|0);
         $1249 = HEAP32[$1248>>2]|0;
         $1250 = ($1249|0)!=(0);
         if ($1250) {
          $1251 = HEAP32[$0>>2]|0;
          $1252 = (($1251) + 44|0);
          $1253 = HEAP32[$db>>2]|0;
          $1254 = HEAP32[$ctx>>2]|0;
          $1255 = (_sqlite3_value_text($1254)|0);
          HEAP32[$vararg_buffer84>>2] = $1255;
          _sqlite3SetString($1252,$1253,1200,$vararg_buffer84);
          $1256 = (($ctx) + 20|0);
          $1257 = HEAP32[$1256>>2]|0;
          HEAP32[$rc>>2] = $1257;
         }
         $1258 = HEAP32[$0>>2]|0;
         $1259 = HEAP32[$pc>>2]|0;
         $1260 = HEAP32[$pOp>>2]|0;
         $1261 = (($1260) + 4|0);
         $1262 = HEAP32[$1261>>2]|0;
         _sqlite3VdbeDeleteAuxData($1258,$1259,$1262);
        }
        $1263 = HEAP32[$ctx>>2]|0;
        $1264 = HEAP8[$encoding>>0]|0;
        $1265 = $1264&255;
        (_sqlite3VdbeChangeEncoding($1263,$1265)|0);
        $1266 = HEAP32[$ctx>>2]|0;
        $1267 = (_sqlite3VdbeMemTooBig($1266)|0);
        $1268 = ($1267|0)!=(0);
        if ($1268) {
         label = 870;
         break L11;
        }
        break;
       }
       case 36:  {
        $1151 = HEAP32[$pOp>>2]|0;
        $1152 = (($1151) + 4|0);
        $1153 = HEAP32[$1152>>2]|0;
        $1154 = ($1153|0)!=(0);
        if ($1154) {
         $1155 = HEAP32[$pOp>>2]|0;
         $1156 = (($1155) + 4|0);
         $1157 = HEAP32[$1156>>2]|0;
         $1158 = HEAP32[$aMem>>2]|0;
         $1159 = (($1158) + (($1157*40)|0)|0);
         _sqlite3VdbeMemSetInt64($1159,0,0);
        }
        break;
       }
       case 88: case 87: case 86: case 85:  {
        $1269 = HEAP32[$pOp>>2]|0;
        $1270 = (($1269) + 4|0);
        $1271 = HEAP32[$1270>>2]|0;
        $1272 = HEAP32[$aMem>>2]|0;
        $1273 = (($1272) + (($1271*40)|0)|0);
        HEAP32[$pIn1>>2] = $1273;
        $1274 = HEAP32[$pOp>>2]|0;
        $1275 = (($1274) + 8|0);
        $1276 = HEAP32[$1275>>2]|0;
        $1277 = HEAP32[$aMem>>2]|0;
        $1278 = (($1277) + (($1276*40)|0)|0);
        HEAP32[$pIn2>>2] = $1278;
        $1279 = HEAP32[$pOp>>2]|0;
        $1280 = (($1279) + 12|0);
        $1281 = HEAP32[$1280>>2]|0;
        $1282 = HEAP32[$aMem>>2]|0;
        $1283 = (($1282) + (($1281*40)|0)|0);
        HEAP32[$pOut>>2] = $1283;
        $1284 = HEAP32[$pIn1>>2]|0;
        $1285 = (($1284) + 8|0);
        $1286 = HEAP16[$1285>>1]|0;
        $1287 = $1286&65535;
        $1288 = HEAP32[$pIn2>>2]|0;
        $1289 = (($1288) + 8|0);
        $1290 = HEAP16[$1289>>1]|0;
        $1291 = $1290&65535;
        $1292 = $1287 | $1291;
        $1293 = $1292 & 1;
        $1294 = ($1293|0)!=(0);
        if ($1294) {
         $1295 = HEAP32[$pOut>>2]|0;
         _sqlite3VdbeMemSetNull($1295);
         break L21;
        }
        $1296 = HEAP32[$pIn2>>2]|0;
        $1297 = (_sqlite3VdbeIntValue($1296)|0);
        $1298 = tempRet0;
        $1299 = $iA4;
        $1300 = $1299;
        HEAP32[$1300>>2] = $1297;
        $1301 = (($1299) + 4)|0;
        $1302 = $1301;
        HEAP32[$1302>>2] = $1298;
        $1303 = HEAP32[$pIn1>>2]|0;
        $1304 = (_sqlite3VdbeIntValue($1303)|0);
        $1305 = tempRet0;
        $1306 = $iB5;
        $1307 = $1306;
        HEAP32[$1307>>2] = $1304;
        $1308 = (($1306) + 4)|0;
        $1309 = $1308;
        HEAP32[$1309>>2] = $1305;
        $1310 = HEAP32[$pOp>>2]|0;
        $1311 = HEAP8[$1310>>0]|0;
        HEAP8[$op>>0] = $1311;
        $1312 = HEAP8[$op>>0]|0;
        $1313 = $1312&255;
        $1314 = ($1313|0)==(85);
        do {
         if ($1314) {
          $1315 = $iB5;
          $1316 = $1315;
          $1317 = HEAP32[$1316>>2]|0;
          $1318 = (($1315) + 4)|0;
          $1319 = $1318;
          $1320 = HEAP32[$1319>>2]|0;
          $1321 = $iA4;
          $1322 = $1321;
          $1323 = HEAP32[$1322>>2]|0;
          $1324 = (($1321) + 4)|0;
          $1325 = $1324;
          $1326 = HEAP32[$1325>>2]|0;
          $1327 = $1323 & $1317;
          $1328 = $1326 & $1320;
          $1329 = $iA4;
          $1330 = $1329;
          HEAP32[$1330>>2] = $1327;
          $1331 = (($1329) + 4)|0;
          $1332 = $1331;
          HEAP32[$1332>>2] = $1328;
         } else {
          $1333 = HEAP8[$op>>0]|0;
          $1334 = $1333&255;
          $1335 = ($1334|0)==(86);
          $1336 = $iB5;
          $1337 = $1336;
          $1338 = HEAP32[$1337>>2]|0;
          $1339 = (($1336) + 4)|0;
          $1340 = $1339;
          $1341 = HEAP32[$1340>>2]|0;
          if ($1335) {
           $1342 = $iA4;
           $1343 = $1342;
           $1344 = HEAP32[$1343>>2]|0;
           $1345 = (($1342) + 4)|0;
           $1346 = $1345;
           $1347 = HEAP32[$1346>>2]|0;
           $1348 = $1344 | $1338;
           $1349 = $1347 | $1341;
           $1350 = $iA4;
           $1351 = $1350;
           HEAP32[$1351>>2] = $1348;
           $1352 = (($1350) + 4)|0;
           $1353 = $1352;
           HEAP32[$1353>>2] = $1349;
           break;
          }
          $1354 = ($1338|0)!=(0);
          $1355 = ($1341|0)!=(0);
          $1356 = $1354 | $1355;
          if ($1356) {
           $1357 = $iB5;
           $1358 = $1357;
           $1359 = HEAP32[$1358>>2]|0;
           $1360 = (($1357) + 4)|0;
           $1361 = $1360;
           $1362 = HEAP32[$1361>>2]|0;
           $1363 = ($1362|0)<(0);
           if ($1363) {
            $1364 = HEAP8[$op>>0]|0;
            $1365 = $1364&255;
            $1366 = (175 - ($1365))|0;
            $1367 = $1366&255;
            HEAP8[$op>>0] = $1367;
            $1368 = $iB5;
            $1369 = $1368;
            $1370 = HEAP32[$1369>>2]|0;
            $1371 = (($1368) + 4)|0;
            $1372 = $1371;
            $1373 = HEAP32[$1372>>2]|0;
            $1374 = ($1373|0)>(-1);
            $1375 = ($1370>>>0)>(4294967232);
            $1376 = ($1373|0)==(-1);
            $1377 = $1376 & $1375;
            $1378 = $1374 | $1377;
            if ($1378) {
             $1379 = $iB5;
             $1380 = $1379;
             $1381 = HEAP32[$1380>>2]|0;
             $1382 = (($1379) + 4)|0;
             $1383 = $1382;
             $1384 = HEAP32[$1383>>2]|0;
             $1385 = (_i64Subtract(0,0,($1381|0),($1384|0))|0);
             $1386 = tempRet0;
             $1389 = $1385;$1392 = $1386;
            } else {
             $1389 = 64;$1392 = 0;
            }
            $1387 = $iB5;
            $1388 = $1387;
            HEAP32[$1388>>2] = $1389;
            $1390 = (($1387) + 4)|0;
            $1391 = $1390;
            HEAP32[$1391>>2] = $1392;
           }
           $1393 = $iB5;
           $1394 = $1393;
           $1395 = HEAP32[$1394>>2]|0;
           $1396 = (($1393) + 4)|0;
           $1397 = $1396;
           $1398 = HEAP32[$1397>>2]|0;
           $1399 = ($1398|0)>(0);
           $1400 = ($1395>>>0)>=(64);
           $1401 = ($1398|0)==(0);
           $1402 = $1401 & $1400;
           $1403 = $1399 | $1402;
           if ($1403) {
            $1404 = $iA4;
            $1405 = $1404;
            $1406 = HEAP32[$1405>>2]|0;
            $1407 = (($1404) + 4)|0;
            $1408 = $1407;
            $1409 = HEAP32[$1408>>2]|0;
            $1410 = ($1409|0)>(0);
            $1411 = ($1406>>>0)>=(0);
            $1412 = ($1409|0)==(0);
            $1413 = $1412 & $1411;
            $1414 = $1410 | $1413;
            if ($1414) {
             $1419 = 1;
            } else {
             $1415 = HEAP8[$op>>0]|0;
             $1416 = $1415&255;
             $1417 = ($1416|0)==(87);
             $1419 = $1417;
            }
            $1418 = $1419 ? 0 : -1;
            $1420 = ($1418|0)<(0);
            $1421 = $1420 << 31 >> 31;
            $1422 = $iA4;
            $1423 = $1422;
            HEAP32[$1423>>2] = $1418;
            $1424 = (($1422) + 4)|0;
            $1425 = $1424;
            HEAP32[$1425>>2] = $1421;
            break;
           }
           ;HEAP32[$uA+0>>2]=HEAP32[$iA4+0>>2]|0;HEAP32[$uA+4>>2]=HEAP32[$iA4+4>>2]|0;
           $1426 = HEAP8[$op>>0]|0;
           $1427 = $1426&255;
           $1428 = ($1427|0)==(87);
           $1429 = $iB5;
           $1430 = $1429;
           $1431 = HEAP32[$1430>>2]|0;
           $1432 = (($1429) + 4)|0;
           $1433 = $1432;
           $1434 = HEAP32[$1433>>2]|0;
           $1435 = $uA;
           $1436 = $1435;
           $1437 = HEAP32[$1436>>2]|0;
           $1438 = (($1435) + 4)|0;
           $1439 = $1438;
           $1440 = HEAP32[$1439>>2]|0;
           if ($1428) {
            $1441 = (_bitshift64Shl(($1437|0),($1440|0),($1431|0))|0);
            $1442 = tempRet0;
            $1443 = $uA;
            $1444 = $1443;
            HEAP32[$1444>>2] = $1441;
            $1445 = (($1443) + 4)|0;
            $1446 = $1445;
            HEAP32[$1446>>2] = $1442;
           } else {
            $1447 = (_bitshift64Lshr(($1437|0),($1440|0),($1431|0))|0);
            $1448 = tempRet0;
            $1449 = $uA;
            $1450 = $1449;
            HEAP32[$1450>>2] = $1447;
            $1451 = (($1449) + 4)|0;
            $1452 = $1451;
            HEAP32[$1452>>2] = $1448;
            $1453 = $iA4;
            $1454 = $1453;
            $1455 = HEAP32[$1454>>2]|0;
            $1456 = (($1453) + 4)|0;
            $1457 = $1456;
            $1458 = HEAP32[$1457>>2]|0;
            $1459 = ($1458|0)<(0);
            if ($1459) {
             $1460 = $iB5;
             $1461 = $1460;
             $1462 = HEAP32[$1461>>2]|0;
             $1463 = (($1460) + 4)|0;
             $1464 = $1463;
             $1465 = HEAP32[$1464>>2]|0;
             $1466 = (_i64Subtract(64,0,($1462|0),($1465|0))|0);
             $1467 = tempRet0;
             $1468 = (_bitshift64Shl(-1,-1,($1466|0))|0);
             $1469 = tempRet0;
             $1470 = $uA;
             $1471 = $1470;
             $1472 = HEAP32[$1471>>2]|0;
             $1473 = (($1470) + 4)|0;
             $1474 = $1473;
             $1475 = HEAP32[$1474>>2]|0;
             $1476 = $1472 | $1468;
             $1477 = $1475 | $1469;
             $1478 = $uA;
             $1479 = $1478;
             HEAP32[$1479>>2] = $1476;
             $1480 = (($1478) + 4)|0;
             $1481 = $1480;
             HEAP32[$1481>>2] = $1477;
            }
           }
           ;HEAP32[$iA4+0>>2]=HEAP32[$uA+0>>2]|0;HEAP32[$iA4+4>>2]=HEAP32[$uA+4>>2]|0;
          }
         }
        } while(0);
        $1482 = $iA4;
        $1483 = $1482;
        $1484 = HEAP32[$1483>>2]|0;
        $1485 = (($1482) + 4)|0;
        $1486 = $1485;
        $1487 = HEAP32[$1486>>2]|0;
        $1488 = HEAP32[$pOut>>2]|0;
        $1489 = $1488;
        $1490 = $1489;
        HEAP32[$1490>>2] = $1484;
        $1491 = (($1489) + 4)|0;
        $1492 = $1491;
        HEAP32[$1492>>2] = $1487;
        $1493 = HEAP32[$pOut>>2]|0;
        $1494 = (($1493) + 8|0);
        $1495 = HEAP16[$1494>>1]|0;
        $1496 = $1495&65535;
        $1497 = $1496 & -16896;
        $1498 = $1497 | 4;
        $1499 = $1498&65535;
        $1500 = HEAP32[$pOut>>2]|0;
        $1501 = (($1500) + 8|0);
        HEAP16[$1501>>1] = $1499;
        break;
       }
       case 93: case 92: case 91: case 90: case 89:  {
        $860 = HEAP32[$pOp>>2]|0;
        $861 = (($860) + 4|0);
        $862 = HEAP32[$861>>2]|0;
        $863 = HEAP32[$aMem>>2]|0;
        $864 = (($863) + (($862*40)|0)|0);
        HEAP32[$pIn1>>2] = $864;
        $865 = HEAP32[$pIn1>>2]|0;
        $866 = (_numericType($865)|0);
        HEAP16[$type1>>1] = $866;
        $867 = HEAP32[$pOp>>2]|0;
        $868 = (($867) + 8|0);
        $869 = HEAP32[$868>>2]|0;
        $870 = HEAP32[$aMem>>2]|0;
        $871 = (($870) + (($869*40)|0)|0);
        HEAP32[$pIn2>>2] = $871;
        $872 = HEAP32[$pIn2>>2]|0;
        $873 = (_numericType($872)|0);
        HEAP16[$type2>>1] = $873;
        $874 = HEAP32[$pOp>>2]|0;
        $875 = (($874) + 12|0);
        $876 = HEAP32[$875>>2]|0;
        $877 = HEAP32[$aMem>>2]|0;
        $878 = (($877) + (($876*40)|0)|0);
        HEAP32[$pOut>>2] = $878;
        $879 = HEAP32[$pIn1>>2]|0;
        $880 = (($879) + 8|0);
        $881 = HEAP16[$880>>1]|0;
        $882 = $881&65535;
        $883 = HEAP32[$pIn2>>2]|0;
        $884 = (($883) + 8|0);
        $885 = HEAP16[$884>>1]|0;
        $886 = $885&65535;
        $887 = $882 | $886;
        $888 = $887&65535;
        HEAP16[$flags>>1] = $888;
        $889 = HEAP16[$flags>>1]|0;
        $890 = $889&65535;
        $891 = $890 & 1;
        $892 = ($891|0)!=(0);
        L168: do {
         if (!($892)) {
          $893 = HEAP16[$type1>>1]|0;
          $894 = $893&65535;
          $895 = HEAP16[$type2>>1]|0;
          $896 = $895&65535;
          $897 = $894 & $896;
          $898 = $897 & 4;
          $899 = ($898|0)!=(0);
          do {
           if ($899) {
            $900 = HEAP32[$pIn1>>2]|0;
            $901 = $900;
            $902 = $901;
            $903 = HEAP32[$902>>2]|0;
            $904 = (($901) + 4)|0;
            $905 = $904;
            $906 = HEAP32[$905>>2]|0;
            $907 = $iA;
            $908 = $907;
            HEAP32[$908>>2] = $903;
            $909 = (($907) + 4)|0;
            $910 = $909;
            HEAP32[$910>>2] = $906;
            $911 = HEAP32[$pIn2>>2]|0;
            $912 = $911;
            $913 = $912;
            $914 = HEAP32[$913>>2]|0;
            $915 = (($912) + 4)|0;
            $916 = $915;
            $917 = HEAP32[$916>>2]|0;
            $918 = $iB;
            $919 = $918;
            HEAP32[$919>>2] = $914;
            $920 = (($918) + 4)|0;
            $921 = $920;
            HEAP32[$921>>2] = $917;
            HEAP8[$bIntint>>0] = 1;
            $922 = HEAP32[$pOp>>2]|0;
            $923 = HEAP8[$922>>0]|0;
            $924 = $923&255;
            if ((($924|0) == 89)) {
             $925 = $iA;
             $926 = $925;
             $927 = HEAP32[$926>>2]|0;
             $928 = (($925) + 4)|0;
             $929 = $928;
             $930 = HEAP32[$929>>2]|0;
             $931 = (_sqlite3AddInt64($iB,$927,$930)|0);
             $932 = ($931|0)!=(0);
             if ($932) {
              break;
             }
            } else if ((($924|0) == 90)) {
             $933 = $iA;
             $934 = $933;
             $935 = HEAP32[$934>>2]|0;
             $936 = (($933) + 4)|0;
             $937 = $936;
             $938 = HEAP32[$937>>2]|0;
             $939 = (_sqlite3SubInt64($iB,$935,$938)|0);
             $940 = ($939|0)!=(0);
             if ($940) {
              break;
             }
            } else if ((($924|0) == 91)) {
             $941 = $iA;
             $942 = $941;
             $943 = HEAP32[$942>>2]|0;
             $944 = (($941) + 4)|0;
             $945 = $944;
             $946 = HEAP32[$945>>2]|0;
             $947 = (_sqlite3MulInt64($iB,$943,$946)|0);
             $948 = ($947|0)!=(0);
             if ($948) {
              break;
             }
            } else if ((($924|0) == 92)) {
             $949 = $iA;
             $950 = $949;
             $951 = HEAP32[$950>>2]|0;
             $952 = (($949) + 4)|0;
             $953 = $952;
             $954 = HEAP32[$953>>2]|0;
             $955 = ($951|0)==(0);
             $956 = ($954|0)==(0);
             $957 = $955 & $956;
             if ($957) {
              break L168;
             }
             $958 = $iA;
             $959 = $958;
             $960 = HEAP32[$959>>2]|0;
             $961 = (($958) + 4)|0;
             $962 = $961;
             $963 = HEAP32[$962>>2]|0;
             $964 = ($960|0)==(-1);
             $965 = ($963|0)==(-1);
             $966 = $964 & $965;
             $967 = $iB;
             $968 = $967;
             $969 = HEAP32[$968>>2]|0;
             $970 = (($967) + 4)|0;
             $971 = $970;
             $972 = HEAP32[$971>>2]|0;
             $973 = ($969|0)==(0);
             $974 = ($972|0)==(-2147483648);
             $975 = $973 & $974;
             $or$cond = $966 & $975;
             if ($or$cond) {
              break;
             }
             $976 = $iA;
             $977 = $976;
             $978 = HEAP32[$977>>2]|0;
             $979 = (($976) + 4)|0;
             $980 = $979;
             $981 = HEAP32[$980>>2]|0;
             $982 = $iB;
             $983 = $982;
             $984 = HEAP32[$983>>2]|0;
             $985 = (($982) + 4)|0;
             $986 = $985;
             $987 = HEAP32[$986>>2]|0;
             $988 = (___divdi3(($984|0),($987|0),($978|0),($981|0))|0);
             $989 = tempRet0;
             $990 = $iB;
             $991 = $990;
             HEAP32[$991>>2] = $988;
             $992 = (($990) + 4)|0;
             $993 = $992;
             HEAP32[$993>>2] = $989;
            } else {
             $994 = $iA;
             $995 = $994;
             $996 = HEAP32[$995>>2]|0;
             $997 = (($994) + 4)|0;
             $998 = $997;
             $999 = HEAP32[$998>>2]|0;
             $1000 = ($996|0)==(0);
             $1001 = ($999|0)==(0);
             $1002 = $1000 & $1001;
             if ($1002) {
              break L168;
             }
             $1003 = $iA;
             $1004 = $1003;
             $1005 = HEAP32[$1004>>2]|0;
             $1006 = (($1003) + 4)|0;
             $1007 = $1006;
             $1008 = HEAP32[$1007>>2]|0;
             $1009 = ($1005|0)==(-1);
             $1010 = ($1008|0)==(-1);
             $1011 = $1009 & $1010;
             if ($1011) {
              $1012 = $iA;
              $1013 = $1012;
              HEAP32[$1013>>2] = 1;
              $1014 = (($1012) + 4)|0;
              $1015 = $1014;
              HEAP32[$1015>>2] = 0;
             }
             $1016 = $iA;
             $1017 = $1016;
             $1018 = HEAP32[$1017>>2]|0;
             $1019 = (($1016) + 4)|0;
             $1020 = $1019;
             $1021 = HEAP32[$1020>>2]|0;
             $1022 = $iB;
             $1023 = $1022;
             $1024 = HEAP32[$1023>>2]|0;
             $1025 = (($1022) + 4)|0;
             $1026 = $1025;
             $1027 = HEAP32[$1026>>2]|0;
             $1028 = (___remdi3(($1024|0),($1027|0),($1018|0),($1021|0))|0);
             $1029 = tempRet0;
             $1030 = $iB;
             $1031 = $1030;
             HEAP32[$1031>>2] = $1028;
             $1032 = (($1030) + 4)|0;
             $1033 = $1032;
             HEAP32[$1033>>2] = $1029;
            }
            $1034 = $iB;
            $1035 = $1034;
            $1036 = HEAP32[$1035>>2]|0;
            $1037 = (($1034) + 4)|0;
            $1038 = $1037;
            $1039 = HEAP32[$1038>>2]|0;
            $1040 = HEAP32[$pOut>>2]|0;
            $1041 = $1040;
            $1042 = $1041;
            HEAP32[$1042>>2] = $1036;
            $1043 = (($1041) + 4)|0;
            $1044 = $1043;
            HEAP32[$1044>>2] = $1039;
            $1045 = HEAP32[$pOut>>2]|0;
            $1046 = (($1045) + 8|0);
            $1047 = HEAP16[$1046>>1]|0;
            $1048 = $1047&65535;
            $1049 = $1048 & -16896;
            $1050 = $1049 | 4;
            $1051 = $1050&65535;
            $1052 = HEAP32[$pOut>>2]|0;
            $1053 = (($1052) + 8|0);
            HEAP16[$1053>>1] = $1051;
            break L21;
           } else {
            HEAP8[$bIntint>>0] = 0;
           }
          } while(0);
          $1054 = HEAP32[$pIn1>>2]|0;
          $1055 = (+_sqlite3VdbeRealValue($1054));
          HEAPF64[$rA>>3] = $1055;
          $1056 = HEAP32[$pIn2>>2]|0;
          $1057 = (+_sqlite3VdbeRealValue($1056));
          HEAPF64[$rB>>3] = $1057;
          $1058 = HEAP32[$pOp>>2]|0;
          $1059 = HEAP8[$1058>>0]|0;
          $1060 = $1059&255;
          if ((($1060|0) == 89)) {
           $1061 = +HEAPF64[$rA>>3];
           $1062 = +HEAPF64[$rB>>3];
           $1063 = $1062 + $1061;
           HEAPF64[$rB>>3] = $1063;
          } else if ((($1060|0) == 90)) {
           $1064 = +HEAPF64[$rA>>3];
           $1065 = +HEAPF64[$rB>>3];
           $1066 = $1065 - $1064;
           HEAPF64[$rB>>3] = $1066;
          } else if ((($1060|0) == 91)) {
           $1067 = +HEAPF64[$rA>>3];
           $1068 = +HEAPF64[$rB>>3];
           $1069 = $1068 * $1067;
           HEAPF64[$rB>>3] = $1069;
          } else if ((($1060|0) == 92)) {
           $1070 = +HEAPF64[$rA>>3];
           $1071 = $1070 == 0.0;
           if ($1071) {
            break;
           }
           $1072 = +HEAPF64[$rA>>3];
           $1073 = +HEAPF64[$rB>>3];
           $1074 = $1073 / $1072;
           HEAPF64[$rB>>3] = $1074;
          } else {
           $1075 = +HEAPF64[$rA>>3];
           $1076 = (~~$1075)>>>0;
           $1077 = +Math_abs($1075) >= 1.0 ? $1075 > 0.0 ? (Math_min(+Math_floor($1075 / 4294967296.0), 4294967295.0) | 0) >>> 0 : ~~+Math_ceil(($1075 - +(~~$1075 >>> 0)) / 4294967296.0) >>> 0 : 0;
           $1078 = $iA;
           $1079 = $1078;
           HEAP32[$1079>>2] = $1076;
           $1080 = (($1078) + 4)|0;
           $1081 = $1080;
           HEAP32[$1081>>2] = $1077;
           $1082 = +HEAPF64[$rB>>3];
           $1083 = (~~$1082)>>>0;
           $1084 = +Math_abs($1082) >= 1.0 ? $1082 > 0.0 ? (Math_min(+Math_floor($1082 / 4294967296.0), 4294967295.0) | 0) >>> 0 : ~~+Math_ceil(($1082 - +(~~$1082 >>> 0)) / 4294967296.0) >>> 0 : 0;
           $1085 = $iB;
           $1086 = $1085;
           HEAP32[$1086>>2] = $1083;
           $1087 = (($1085) + 4)|0;
           $1088 = $1087;
           HEAP32[$1088>>2] = $1084;
           $1089 = $iA;
           $1090 = $1089;
           $1091 = HEAP32[$1090>>2]|0;
           $1092 = (($1089) + 4)|0;
           $1093 = $1092;
           $1094 = HEAP32[$1093>>2]|0;
           $1095 = ($1091|0)==(0);
           $1096 = ($1094|0)==(0);
           $1097 = $1095 & $1096;
           if ($1097) {
            break;
           }
           $1098 = $iA;
           $1099 = $1098;
           $1100 = HEAP32[$1099>>2]|0;
           $1101 = (($1098) + 4)|0;
           $1102 = $1101;
           $1103 = HEAP32[$1102>>2]|0;
           $1104 = ($1100|0)==(-1);
           $1105 = ($1103|0)==(-1);
           $1106 = $1104 & $1105;
           if ($1106) {
            $1107 = $iA;
            $1108 = $1107;
            HEAP32[$1108>>2] = 1;
            $1109 = (($1107) + 4)|0;
            $1110 = $1109;
            HEAP32[$1110>>2] = 0;
           }
           $1111 = $iB;
           $1112 = $1111;
           $1113 = HEAP32[$1112>>2]|0;
           $1114 = (($1111) + 4)|0;
           $1115 = $1114;
           $1116 = HEAP32[$1115>>2]|0;
           $1117 = $iA;
           $1118 = $1117;
           $1119 = HEAP32[$1118>>2]|0;
           $1120 = (($1117) + 4)|0;
           $1121 = $1120;
           $1122 = HEAP32[$1121>>2]|0;
           $1123 = (___remdi3(($1113|0),($1116|0),($1119|0),($1122|0))|0);
           $1124 = tempRet0;
           $1125 = (+($1123>>>0)) + (4294967296.0*(+($1124|0)));
           HEAPF64[$rB>>3] = $1125;
          }
          $1126 = +HEAPF64[$rB>>3];
          $1127 = (_sqlite3IsNaN($1126)|0);
          $1128 = ($1127|0)!=(0);
          if (!($1128)) {
           $1129 = +HEAPF64[$rB>>3];
           $1130 = HEAP32[$pOut>>2]|0;
           HEAPF64[$1130>>3] = $1129;
           $1131 = HEAP32[$pOut>>2]|0;
           $1132 = (($1131) + 8|0);
           $1133 = HEAP16[$1132>>1]|0;
           $1134 = $1133&65535;
           $1135 = $1134 & -16896;
           $1136 = $1135 | 8;
           $1137 = $1136&65535;
           $1138 = HEAP32[$pOut>>2]|0;
           $1139 = (($1138) + 8|0);
           HEAP16[$1139>>1] = $1137;
           $1140 = HEAP16[$type1>>1]|0;
           $1141 = $1140&65535;
           $1142 = HEAP16[$type2>>1]|0;
           $1143 = $1142&65535;
           $1144 = $1141 | $1143;
           $1145 = $1144 & 8;
           $1146 = ($1145|0)!=(0);
           $1147 = HEAP8[$bIntint>>0]|0;
           $1148 = ($1147<<24>>24)!=(0);
           $or$cond3 = $1146 | $1148;
           if ($or$cond3) {
            break L21;
           }
           $1149 = HEAP32[$pOut>>2]|0;
           _sqlite3VdbeIntegerAffinity($1149);
           break L21;
          }
         }
        } while(0);
        $1150 = HEAP32[$pOut>>2]|0;
        _sqlite3VdbeMemSetNull($1150);
        break;
       }
       case 94:  {
        $679 = HEAP32[$pOp>>2]|0;
        $680 = (($679) + 4|0);
        $681 = HEAP32[$680>>2]|0;
        $682 = HEAP32[$aMem>>2]|0;
        $683 = (($682) + (($681*40)|0)|0);
        HEAP32[$pIn1>>2] = $683;
        $684 = HEAP32[$pOp>>2]|0;
        $685 = (($684) + 8|0);
        $686 = HEAP32[$685>>2]|0;
        $687 = HEAP32[$aMem>>2]|0;
        $688 = (($687) + (($686*40)|0)|0);
        HEAP32[$pIn2>>2] = $688;
        $689 = HEAP32[$pOp>>2]|0;
        $690 = (($689) + 12|0);
        $691 = HEAP32[$690>>2]|0;
        $692 = HEAP32[$aMem>>2]|0;
        $693 = (($692) + (($691*40)|0)|0);
        HEAP32[$pOut>>2] = $693;
        $694 = HEAP32[$pIn1>>2]|0;
        $695 = (($694) + 8|0);
        $696 = HEAP16[$695>>1]|0;
        $697 = $696&65535;
        $698 = HEAP32[$pIn2>>2]|0;
        $699 = (($698) + 8|0);
        $700 = HEAP16[$699>>1]|0;
        $701 = $700&65535;
        $702 = $697 | $701;
        $703 = $702 & 1;
        $704 = ($703|0)!=(0);
        if ($704) {
         $705 = HEAP32[$pOut>>2]|0;
         _sqlite3VdbeMemSetNull($705);
         break L21;
        }
        $706 = HEAP32[$pIn1>>2]|0;
        $707 = (($706) + 8|0);
        $708 = HEAP16[$707>>1]|0;
        $709 = $708&65535;
        $710 = $709 & 16384;
        $711 = ($710|0)!=(0);
        if ($711) {
         $712 = HEAP32[$pIn1>>2]|0;
         $713 = (_sqlite3VdbeMemExpandBlob($712)|0);
         $714 = ($713|0)!=(0);
         if ($714) {
          label = 871;
          break L1;
         }
        }
        $715 = HEAP32[$pIn2>>2]|0;
        $716 = (($715) + 8|0);
        $717 = HEAP16[$716>>1]|0;
        $718 = $717&65535;
        $719 = $718 & 16384;
        $720 = ($719|0)!=(0);
        if ($720) {
         $721 = HEAP32[$pIn2>>2]|0;
         $722 = (_sqlite3VdbeMemExpandBlob($721)|0);
         $723 = ($722|0)!=(0);
         if ($723) {
          label = 871;
          break L1;
         }
        }
        $724 = HEAP32[$pIn1>>2]|0;
        $725 = (($724) + 8|0);
        $726 = HEAP16[$725>>1]|0;
        $727 = $726&65535;
        $728 = $727 & 18;
        $729 = ($728|0)==(0);
        if ($729) {
         $730 = HEAP32[$pIn1>>2]|0;
         $731 = HEAP8[$encoding>>0]|0;
         $732 = (_sqlite3VdbeMemStringify($730,$731,0)|0);
         $733 = ($732|0)!=(0);
         if ($733) {
          label = 871;
          break L1;
         }
        }
        $734 = HEAP32[$pIn2>>2]|0;
        $735 = (($734) + 8|0);
        $736 = HEAP16[$735>>1]|0;
        $737 = $736&65535;
        $738 = $737 & 18;
        $739 = ($738|0)==(0);
        if ($739) {
         $740 = HEAP32[$pIn2>>2]|0;
         $741 = HEAP8[$encoding>>0]|0;
         $742 = (_sqlite3VdbeMemStringify($740,$741,0)|0);
         $743 = ($742|0)!=(0);
         if ($743) {
          label = 871;
          break L1;
         }
        }
        $744 = HEAP32[$pIn1>>2]|0;
        $745 = (($744) + 12|0);
        $746 = HEAP32[$745>>2]|0;
        $747 = HEAP32[$pIn2>>2]|0;
        $748 = (($747) + 12|0);
        $749 = HEAP32[$748>>2]|0;
        $750 = (($746) + ($749))|0;
        $751 = ($750|0)<(0);
        $752 = $751 << 31 >> 31;
        $753 = $nByte;
        $754 = $753;
        HEAP32[$754>>2] = $750;
        $755 = (($753) + 4)|0;
        $756 = $755;
        HEAP32[$756>>2] = $752;
        $757 = $nByte;
        $758 = $757;
        $759 = HEAP32[$758>>2]|0;
        $760 = (($757) + 4)|0;
        $761 = $760;
        $762 = HEAP32[$761>>2]|0;
        $763 = HEAP32[$db>>2]|0;
        $764 = (($763) + 88|0);
        $765 = HEAP32[$764>>2]|0;
        $766 = ($765|0)<(0);
        $767 = $766 << 31 >> 31;
        $768 = ($762|0)>($767|0);
        $769 = ($759>>>0)>($765>>>0);
        $770 = ($762|0)==($767|0);
        $771 = $770 & $769;
        $772 = $768 | $771;
        if ($772) {
         label = 870;
         break L11;
        }
        $773 = HEAP32[$pOut>>2]|0;
        $774 = $nByte;
        $775 = $774;
        $776 = HEAP32[$775>>2]|0;
        $777 = (($774) + 4)|0;
        $778 = $777;
        $779 = HEAP32[$778>>2]|0;
        $780 = (($776) + 2)|0;
        $781 = HEAP32[$pOut>>2]|0;
        $782 = HEAP32[$pIn2>>2]|0;
        $783 = ($781|0)==($782|0);
        $784 = $783&1;
        $785 = (_sqlite3VdbeMemGrow($773,$780,$784)|0);
        $786 = ($785|0)!=(0);
        if ($786) {
         label = 871;
         break L1;
        }
        $787 = HEAP32[$pOut>>2]|0;
        $788 = (($787) + 8|0);
        $789 = HEAP16[$788>>1]|0;
        $790 = $789&65535;
        $791 = $790 & -16896;
        $792 = $791 | 2;
        $793 = $792&65535;
        $794 = HEAP32[$pOut>>2]|0;
        $795 = (($794) + 8|0);
        HEAP16[$795>>1] = $793;
        $796 = HEAP32[$pOut>>2]|0;
        $797 = HEAP32[$pIn2>>2]|0;
        $798 = ($796|0)!=($797|0);
        if ($798) {
         $799 = HEAP32[$pOut>>2]|0;
         $800 = (($799) + 16|0);
         $801 = HEAP32[$800>>2]|0;
         $802 = HEAP32[$pIn2>>2]|0;
         $803 = (($802) + 16|0);
         $804 = HEAP32[$803>>2]|0;
         $805 = HEAP32[$pIn2>>2]|0;
         $806 = (($805) + 12|0);
         $807 = HEAP32[$806>>2]|0;
         _memcpy(($801|0),($804|0),($807|0))|0;
        }
        $808 = HEAP32[$pIn2>>2]|0;
        $809 = (($808) + 12|0);
        $810 = HEAP32[$809>>2]|0;
        $811 = HEAP32[$pOut>>2]|0;
        $812 = (($811) + 16|0);
        $813 = HEAP32[$812>>2]|0;
        $814 = (($813) + ($810)|0);
        $815 = HEAP32[$pIn1>>2]|0;
        $816 = (($815) + 16|0);
        $817 = HEAP32[$816>>2]|0;
        $818 = HEAP32[$pIn1>>2]|0;
        $819 = (($818) + 12|0);
        $820 = HEAP32[$819>>2]|0;
        _memcpy(($814|0),($817|0),($820|0))|0;
        $821 = $nByte;
        $822 = $821;
        $823 = HEAP32[$822>>2]|0;
        $824 = (($821) + 4)|0;
        $825 = $824;
        $826 = HEAP32[$825>>2]|0;
        $827 = HEAP32[$pOut>>2]|0;
        $828 = (($827) + 16|0);
        $829 = HEAP32[$828>>2]|0;
        $830 = (($829) + ($823)|0);
        HEAP8[$830>>0] = 0;
        $831 = $nByte;
        $832 = $831;
        $833 = HEAP32[$832>>2]|0;
        $834 = (($831) + 4)|0;
        $835 = $834;
        $836 = HEAP32[$835>>2]|0;
        $837 = (_i64Add(($833|0),($836|0),1,0)|0);
        $838 = tempRet0;
        $839 = HEAP32[$pOut>>2]|0;
        $840 = (($839) + 16|0);
        $841 = HEAP32[$840>>2]|0;
        $842 = (($841) + ($837)|0);
        HEAP8[$842>>0] = 0;
        $843 = HEAP32[$pOut>>2]|0;
        $844 = (($843) + 8|0);
        $845 = HEAP16[$844>>1]|0;
        $846 = $845&65535;
        $847 = $846 | 512;
        $848 = $847&65535;
        HEAP16[$844>>1] = $848;
        $849 = $nByte;
        $850 = $849;
        $851 = HEAP32[$850>>2]|0;
        $852 = (($849) + 4)|0;
        $853 = $852;
        $854 = HEAP32[$853>>2]|0;
        $855 = HEAP32[$pOut>>2]|0;
        $856 = (($855) + 12|0);
        HEAP32[$856>>2] = $851;
        $857 = HEAP8[$encoding>>0]|0;
        $858 = HEAP32[$pOut>>2]|0;
        $859 = (($858) + 10|0);
        HEAP8[$859>>0] = $857;
        break;
       }
       case 33:  {
        $569 = HEAP32[$pOp>>2]|0;
        $570 = (($569) + 12|0);
        $571 = HEAP32[$570>>2]|0;
        HEAP32[$n1>>2] = $571;
        $572 = HEAP32[$pOp>>2]|0;
        $573 = (($572) + 4|0);
        $574 = HEAP32[$573>>2]|0;
        $575 = HEAP32[$aMem>>2]|0;
        $576 = (($575) + (($574*40)|0)|0);
        HEAP32[$pIn1>>2] = $576;
        $577 = HEAP32[$pOp>>2]|0;
        $578 = (($577) + 8|0);
        $579 = HEAP32[$578>>2]|0;
        $580 = HEAP32[$aMem>>2]|0;
        $581 = (($580) + (($579*40)|0)|0);
        HEAP32[$pOut>>2] = $581;
        while(1) {
         $582 = HEAP32[$pOut>>2]|0;
         $583 = HEAP32[$pIn1>>2]|0;
         _sqlite3VdbeMemShallowCopy($582,$583,4096);
         $584 = HEAP32[$pOut>>2]|0;
         $585 = (($584) + 8|0);
         $586 = HEAP16[$585>>1]|0;
         $587 = $586&65535;
         $588 = $587 & 4096;
         $589 = ($588|0)!=(0);
         if ($589) {
          $590 = HEAP32[$pOut>>2]|0;
          $591 = (_sqlite3VdbeMemMakeWriteable($590)|0);
          $592 = ($591|0)!=(0);
          if ($592) {
           label = 871;
           break L1;
          }
         }
         $593 = HEAP32[$n1>>2]|0;
         $594 = (($593) + -1)|0;
         HEAP32[$n1>>2] = $594;
         $595 = ($593|0)==(0);
         if ($595) {
          break L21;
         }
         $596 = HEAP32[$pOut>>2]|0;
         $597 = (($596) + 40|0);
         HEAP32[$pOut>>2] = $597;
         $598 = HEAP32[$pIn1>>2]|0;
         $599 = (($598) + 40|0);
         HEAP32[$pIn1>>2] = $599;
        }
        break;
       }
       case 32:  {
        $545 = HEAP32[$pOp>>2]|0;
        $546 = (($545) + 12|0);
        $547 = HEAP32[$546>>2]|0;
        HEAP32[$n>>2] = $547;
        $548 = HEAP32[$pOp>>2]|0;
        $549 = (($548) + 4|0);
        $550 = HEAP32[$549>>2]|0;
        HEAP32[$p1>>2] = $550;
        $551 = HEAP32[$pOp>>2]|0;
        $552 = (($551) + 8|0);
        $553 = HEAP32[$552>>2]|0;
        HEAP32[$p2>>2] = $553;
        $554 = HEAP32[$p1>>2]|0;
        $555 = HEAP32[$aMem>>2]|0;
        $556 = (($555) + (($554*40)|0)|0);
        HEAP32[$pIn1>>2] = $556;
        $557 = HEAP32[$p2>>2]|0;
        $558 = HEAP32[$aMem>>2]|0;
        $559 = (($558) + (($557*40)|0)|0);
        HEAP32[$pOut>>2] = $559;
        while(1) {
         $560 = HEAP32[$pOut>>2]|0;
         $561 = HEAP32[$pIn1>>2]|0;
         _sqlite3VdbeMemMove($560,$561);
         $562 = HEAP32[$pIn1>>2]|0;
         $563 = (($562) + 40|0);
         HEAP32[$pIn1>>2] = $563;
         $564 = HEAP32[$pOut>>2]|0;
         $565 = (($564) + 40|0);
         HEAP32[$pOut>>2] = $565;
         $566 = HEAP32[$n>>2]|0;
         $567 = (($566) + -1)|0;
         HEAP32[$n>>2] = $567;
         $568 = ($567|0)!=(0);
         if (!($568)) {
          break;
         }
        }
        break;
       }
       case 41:  {
        $1807 = HEAP32[$pOp>>2]|0;
        $1808 = (($1807) + 16|0);
        $1809 = HEAP32[$1808>>2]|0;
        HEAP32[$aPermute>>2] = $1809;
        break;
       }
       case 42:  {
        $1810 = HEAP32[$pOp>>2]|0;
        $1811 = (($1810) + 3|0);
        $1812 = HEAP8[$1811>>0]|0;
        $1813 = $1812&255;
        $1814 = $1813 & 1;
        $1815 = ($1814|0)==(0);
        if ($1815) {
         HEAP32[$aPermute>>2] = 0;
        }
        $1816 = HEAP32[$pOp>>2]|0;
        $1817 = (($1816) + 12|0);
        $1818 = HEAP32[$1817>>2]|0;
        HEAP32[$n6>>2] = $1818;
        $1819 = HEAP32[$pOp>>2]|0;
        $1820 = (($1819) + 16|0);
        $1821 = HEAP32[$1820>>2]|0;
        HEAP32[$pKeyInfo>>2] = $1821;
        $1822 = HEAP32[$pOp>>2]|0;
        $1823 = (($1822) + 4|0);
        $1824 = HEAP32[$1823>>2]|0;
        HEAP32[$p18>>2] = $1824;
        $1825 = HEAP32[$pOp>>2]|0;
        $1826 = (($1825) + 8|0);
        $1827 = HEAP32[$1826>>2]|0;
        HEAP32[$p29>>2] = $1827;
        HEAP32[$i7>>2] = 0;
        while(1) {
         $1828 = HEAP32[$i7>>2]|0;
         $1829 = HEAP32[$n6>>2]|0;
         $1830 = ($1828|0)<($1829|0);
         if (!($1830)) {
          break;
         }
         $1831 = HEAP32[$aPermute>>2]|0;
         $1832 = ($1831|0)!=(0|0);
         $1833 = HEAP32[$i7>>2]|0;
         if ($1832) {
          $1834 = HEAP32[$aPermute>>2]|0;
          $1835 = (($1834) + ($1833<<2)|0);
          $1836 = HEAP32[$1835>>2]|0;
          $1837 = $1836;
         } else {
          $1837 = $1833;
         }
         HEAP32[$idx>>2] = $1837;
         $1838 = HEAP32[$i7>>2]|0;
         $1839 = HEAP32[$pKeyInfo>>2]|0;
         $1840 = (($1839) + 20|0);
         $1841 = (($1840) + ($1838<<2)|0);
         $1842 = HEAP32[$1841>>2]|0;
         HEAP32[$pColl>>2] = $1842;
         $1843 = HEAP32[$i7>>2]|0;
         $1844 = HEAP32[$pKeyInfo>>2]|0;
         $1845 = (($1844) + 16|0);
         $1846 = HEAP32[$1845>>2]|0;
         $1847 = (($1846) + ($1843)|0);
         $1848 = HEAP8[$1847>>0]|0;
         $1849 = $1848&255;
         HEAP32[$bRev>>2] = $1849;
         $1850 = HEAP32[$p18>>2]|0;
         $1851 = HEAP32[$idx>>2]|0;
         $1852 = (($1850) + ($1851))|0;
         $1853 = HEAP32[$aMem>>2]|0;
         $1854 = (($1853) + (($1852*40)|0)|0);
         $1855 = HEAP32[$p29>>2]|0;
         $1856 = HEAP32[$idx>>2]|0;
         $1857 = (($1855) + ($1856))|0;
         $1858 = HEAP32[$aMem>>2]|0;
         $1859 = (($1858) + (($1857*40)|0)|0);
         $1860 = HEAP32[$pColl>>2]|0;
         $1861 = (_sqlite3MemCompare($1854,$1859,$1860)|0);
         HEAP32[$iCompare>>2] = $1861;
         $1862 = HEAP32[$iCompare>>2]|0;
         $1863 = ($1862|0)!=(0);
         if ($1863) {
          label = 224;
          break;
         }
         $1868 = HEAP32[$i7>>2]|0;
         $1869 = (($1868) + 1)|0;
         HEAP32[$i7>>2] = $1869;
        }
        if ((label|0) == 224) {
         label = 0;
         $1864 = HEAP32[$bRev>>2]|0;
         $1865 = ($1864|0)!=(0);
         if ($1865) {
          $1866 = HEAP32[$iCompare>>2]|0;
          $1867 = (0 - ($1866))|0;
          HEAP32[$iCompare>>2] = $1867;
         }
        }
        HEAP32[$aPermute>>2] = 0;
        break;
       }
       case 43:  {
        $1870 = HEAP32[$iCompare>>2]|0;
        $1871 = ($1870|0)<(0);
        if ($1871) {
         $1872 = HEAP32[$pOp>>2]|0;
         $1873 = (($1872) + 4|0);
         $1874 = HEAP32[$1873>>2]|0;
         $1875 = (($1874) - 1)|0;
         HEAP32[$pc>>2] = $1875;
         break L21;
        }
        $1876 = HEAP32[$iCompare>>2]|0;
        $1877 = ($1876|0)==(0);
        $1878 = HEAP32[$pOp>>2]|0;
        if ($1877) {
         $1879 = (($1878) + 8|0);
         $1880 = HEAP32[$1879>>2]|0;
         $1881 = (($1880) - 1)|0;
         HEAP32[$pc>>2] = $1881;
         break L21;
        } else {
         $1882 = (($1878) + 12|0);
         $1883 = HEAP32[$1882>>2]|0;
         $1884 = (($1883) - 1)|0;
         HEAP32[$pc>>2] = $1884;
         break L21;
        }
        break;
       }
       case 71: case 72:  {
        $1885 = HEAP32[$pOp>>2]|0;
        $1886 = (($1885) + 4|0);
        $1887 = HEAP32[$1886>>2]|0;
        $1888 = HEAP32[$aMem>>2]|0;
        $1889 = (($1888) + (($1887*40)|0)|0);
        HEAP32[$pIn1>>2] = $1889;
        $1890 = HEAP32[$pIn1>>2]|0;
        $1891 = (($1890) + 8|0);
        $1892 = HEAP16[$1891>>1]|0;
        $1893 = $1892&65535;
        $1894 = $1893 & 1;
        $1895 = ($1894|0)!=(0);
        if ($1895) {
         HEAP32[$v1>>2] = 2;
        } else {
         $1896 = HEAP32[$pIn1>>2]|0;
         $1897 = (_sqlite3VdbeIntValue($1896)|0);
         $1898 = tempRet0;
         $1899 = ($1897|0)!=(0);
         $1900 = ($1898|0)!=(0);
         $1901 = $1899 | $1900;
         $1902 = $1901&1;
         HEAP32[$v1>>2] = $1902;
        }
        $1903 = HEAP32[$pOp>>2]|0;
        $1904 = (($1903) + 8|0);
        $1905 = HEAP32[$1904>>2]|0;
        $1906 = HEAP32[$aMem>>2]|0;
        $1907 = (($1906) + (($1905*40)|0)|0);
        HEAP32[$pIn2>>2] = $1907;
        $1908 = HEAP32[$pIn2>>2]|0;
        $1909 = (($1908) + 8|0);
        $1910 = HEAP16[$1909>>1]|0;
        $1911 = $1910&65535;
        $1912 = $1911 & 1;
        $1913 = ($1912|0)!=(0);
        if ($1913) {
         HEAP32[$v2>>2] = 2;
        } else {
         $1914 = HEAP32[$pIn2>>2]|0;
         $1915 = (_sqlite3VdbeIntValue($1914)|0);
         $1916 = tempRet0;
         $1917 = ($1915|0)!=(0);
         $1918 = ($1916|0)!=(0);
         $1919 = $1917 | $1918;
         $1920 = $1919&1;
         HEAP32[$v2>>2] = $1920;
        }
        $1921 = HEAP32[$pOp>>2]|0;
        $1922 = HEAP8[$1921>>0]|0;
        $1923 = $1922&255;
        $1924 = ($1923|0)==(72);
        $1925 = HEAP32[$v1>>2]|0;
        $1926 = ($1925*3)|0;
        $1927 = HEAP32[$v2>>2]|0;
        $1928 = (($1926) + ($1927))|0;
        if ($1924) {
         $1929 = (31408 + ($1928)|0);
         $1930 = HEAP8[$1929>>0]|0;
         $1931 = $1930&255;
         HEAP32[$v1>>2] = $1931;
        } else {
         $1932 = (31424 + ($1928)|0);
         $1933 = HEAP8[$1932>>0]|0;
         $1934 = $1933&255;
         HEAP32[$v1>>2] = $1934;
        }
        $1935 = HEAP32[$pOp>>2]|0;
        $1936 = (($1935) + 12|0);
        $1937 = HEAP32[$1936>>2]|0;
        $1938 = HEAP32[$aMem>>2]|0;
        $1939 = (($1938) + (($1937*40)|0)|0);
        HEAP32[$pOut>>2] = $1939;
        $1940 = HEAP32[$v1>>2]|0;
        $1941 = ($1940|0)==(2);
        if ($1941) {
         $1942 = HEAP32[$pOut>>2]|0;
         $1943 = (($1942) + 8|0);
         $1944 = HEAP16[$1943>>1]|0;
         $1945 = $1944&65535;
         $1946 = $1945 & -16896;
         $1947 = $1946 | 1;
         $1948 = $1947&65535;
         $1949 = HEAP32[$pOut>>2]|0;
         $1950 = (($1949) + 8|0);
         HEAP16[$1950>>1] = $1948;
         break L21;
        } else {
         $1951 = HEAP32[$v1>>2]|0;
         $1952 = ($1951|0)<(0);
         $1953 = $1952 << 31 >> 31;
         $1954 = HEAP32[$pOut>>2]|0;
         $1955 = $1954;
         $1956 = $1955;
         HEAP32[$1956>>2] = $1951;
         $1957 = (($1955) + 4)|0;
         $1958 = $1957;
         HEAP32[$1958>>2] = $1953;
         $1959 = HEAP32[$pOut>>2]|0;
         $1960 = (($1959) + 8|0);
         $1961 = HEAP16[$1960>>1]|0;
         $1962 = $1961&65535;
         $1963 = $1962 & -16896;
         $1964 = $1963 | 4;
         $1965 = $1964&65535;
         $1966 = HEAP32[$pOut>>2]|0;
         $1967 = (($1966) + 8|0);
         HEAP16[$1967>>1] = $1965;
         break L21;
        }
        break;
       }
       case 19:  {
        $1968 = HEAP32[$pOp>>2]|0;
        $1969 = (($1968) + 4|0);
        $1970 = HEAP32[$1969>>2]|0;
        $1971 = HEAP32[$aMem>>2]|0;
        $1972 = (($1971) + (($1970*40)|0)|0);
        HEAP32[$pIn1>>2] = $1972;
        $1973 = HEAP32[$pOp>>2]|0;
        $1974 = (($1973) + 8|0);
        $1975 = HEAP32[$1974>>2]|0;
        $1976 = HEAP32[$aMem>>2]|0;
        $1977 = (($1976) + (($1975*40)|0)|0);
        HEAP32[$pOut>>2] = $1977;
        $1978 = HEAP32[$pOut>>2]|0;
        _sqlite3VdbeMemSetNull($1978);
        $1979 = HEAP32[$pIn1>>2]|0;
        $1980 = (($1979) + 8|0);
        $1981 = HEAP16[$1980>>1]|0;
        $1982 = $1981&65535;
        $1983 = $1982 & 1;
        $1984 = ($1983|0)==(0);
        if ($1984) {
         $1985 = HEAP32[$pOut>>2]|0;
         $1986 = (($1985) + 8|0);
         HEAP16[$1986>>1] = 4;
         $1987 = HEAP32[$pIn1>>2]|0;
         $1988 = (_sqlite3VdbeIntValue($1987)|0);
         $1989 = tempRet0;
         $1990 = ($1988|0)!=(0);
         $1991 = ($1989|0)!=(0);
         $1992 = $1990 | $1991;
         $1993 = $1992 ^ 1;
         $1994 = $1993&1;
         $1995 = ($1994|0)<(0);
         $1996 = $1995 << 31 >> 31;
         $1997 = HEAP32[$pOut>>2]|0;
         $1998 = $1997;
         $1999 = $1998;
         HEAP32[$1999>>2] = $1994;
         $2000 = (($1998) + 4)|0;
         $2001 = $2000;
         HEAP32[$2001>>2] = $1996;
        }
        break;
       }
       case 96:  {
        $2002 = HEAP32[$pOp>>2]|0;
        $2003 = (($2002) + 4|0);
        $2004 = HEAP32[$2003>>2]|0;
        $2005 = HEAP32[$aMem>>2]|0;
        $2006 = (($2005) + (($2004*40)|0)|0);
        HEAP32[$pIn1>>2] = $2006;
        $2007 = HEAP32[$pOp>>2]|0;
        $2008 = (($2007) + 8|0);
        $2009 = HEAP32[$2008>>2]|0;
        $2010 = HEAP32[$aMem>>2]|0;
        $2011 = (($2010) + (($2009*40)|0)|0);
        HEAP32[$pOut>>2] = $2011;
        $2012 = HEAP32[$pOut>>2]|0;
        _sqlite3VdbeMemSetNull($2012);
        $2013 = HEAP32[$pIn1>>2]|0;
        $2014 = (($2013) + 8|0);
        $2015 = HEAP16[$2014>>1]|0;
        $2016 = $2015&65535;
        $2017 = $2016 & 1;
        $2018 = ($2017|0)==(0);
        if ($2018) {
         $2019 = HEAP32[$pOut>>2]|0;
         $2020 = (($2019) + 8|0);
         HEAP16[$2020>>1] = 4;
         $2021 = HEAP32[$pIn1>>2]|0;
         $2022 = (_sqlite3VdbeIntValue($2021)|0);
         $2023 = tempRet0;
         $2024 = $2022 ^ -1;
         $2025 = $2023 ^ -1;
         $2026 = HEAP32[$pOut>>2]|0;
         $2027 = $2026;
         $2028 = $2027;
         HEAP32[$2028>>2] = $2024;
         $2029 = (($2027) + 4)|0;
         $2030 = $2029;
         HEAP32[$2030>>2] = $2025;
        }
        break;
       }
       case 44:  {
        $2031 = HEAP32[$pOp>>2]|0;
        $2032 = (($2031) + 4|0);
        $2033 = HEAP32[$2032>>2]|0;
        $2034 = HEAP32[$0>>2]|0;
        $2035 = (($2034) + 200|0);
        $2036 = HEAP32[$2035>>2]|0;
        $2037 = (($2036) + ($2033)|0);
        $2038 = HEAP8[$2037>>0]|0;
        $2039 = ($2038<<24>>24)!=(0);
        $2040 = HEAP32[$pOp>>2]|0;
        if ($2039) {
         $2041 = (($2040) + 8|0);
         $2042 = HEAP32[$2041>>2]|0;
         $2043 = (($2042) - 1)|0;
         HEAP32[$pc>>2] = $2043;
         break L21;
        } else {
         $2044 = (($2040) + 4|0);
         $2045 = HEAP32[$2044>>2]|0;
         $2046 = HEAP32[$0>>2]|0;
         $2047 = (($2046) + 200|0);
         $2048 = HEAP32[$2047>>2]|0;
         $2049 = (($2048) + ($2045)|0);
         HEAP8[$2049>>0] = 1;
         break L21;
        }
        break;
       }
       case 46: case 45:  {
        $2050 = HEAP32[$pOp>>2]|0;
        $2051 = (($2050) + 4|0);
        $2052 = HEAP32[$2051>>2]|0;
        $2053 = HEAP32[$aMem>>2]|0;
        $2054 = (($2053) + (($2052*40)|0)|0);
        HEAP32[$pIn1>>2] = $2054;
        $2055 = HEAP32[$pIn1>>2]|0;
        $2056 = (($2055) + 8|0);
        $2057 = HEAP16[$2056>>1]|0;
        $2058 = $2057&65535;
        $2059 = $2058 & 1;
        $2060 = ($2059|0)!=(0);
        if ($2060) {
         $2061 = HEAP32[$pOp>>2]|0;
         $2062 = (($2061) + 12|0);
         $2063 = HEAP32[$2062>>2]|0;
         HEAP32[$c>>2] = $2063;
        } else {
         $2064 = HEAP32[$pIn1>>2]|0;
         $2065 = (+_sqlite3VdbeRealValue($2064));
         $2066 = $2065 != 0.0;
         $2067 = $2066&1;
         HEAP32[$c>>2] = $2067;
         $2068 = HEAP32[$pOp>>2]|0;
         $2069 = HEAP8[$2068>>0]|0;
         $2070 = $2069&255;
         $2071 = ($2070|0)==(46);
         if ($2071) {
          $2072 = HEAP32[$c>>2]|0;
          $2073 = ($2072|0)!=(0);
          $2074 = $2073 ^ 1;
          $2075 = $2074&1;
          HEAP32[$c>>2] = $2075;
         }
        }
        $2076 = HEAP32[$c>>2]|0;
        $2077 = ($2076|0)!=(0);
        if ($2077) {
         $2078 = HEAP32[$pOp>>2]|0;
         $2079 = (($2078) + 8|0);
         $2080 = HEAP32[$2079>>2]|0;
         $2081 = (($2080) - 1)|0;
         HEAP32[$pc>>2] = $2081;
        }
        break;
       }
       case 76:  {
        $2082 = HEAP32[$pOp>>2]|0;
        $2083 = (($2082) + 4|0);
        $2084 = HEAP32[$2083>>2]|0;
        $2085 = HEAP32[$aMem>>2]|0;
        $2086 = (($2085) + (($2084*40)|0)|0);
        HEAP32[$pIn1>>2] = $2086;
        $2087 = HEAP32[$pIn1>>2]|0;
        $2088 = (($2087) + 8|0);
        $2089 = HEAP16[$2088>>1]|0;
        $2090 = $2089&65535;
        $2091 = $2090 & 1;
        $2092 = ($2091|0)!=(0);
        if ($2092) {
         $2093 = HEAP32[$pOp>>2]|0;
         $2094 = (($2093) + 8|0);
         $2095 = HEAP32[$2094>>2]|0;
         $2096 = (($2095) - 1)|0;
         HEAP32[$pc>>2] = $2096;
        }
        break;
       }
       case 77:  {
        $2097 = HEAP32[$pOp>>2]|0;
        $2098 = (($2097) + 4|0);
        $2099 = HEAP32[$2098>>2]|0;
        $2100 = HEAP32[$aMem>>2]|0;
        $2101 = (($2100) + (($2099*40)|0)|0);
        HEAP32[$pIn1>>2] = $2101;
        $2102 = HEAP32[$pIn1>>2]|0;
        $2103 = (($2102) + 8|0);
        $2104 = HEAP16[$2103>>1]|0;
        $2105 = $2104&65535;
        $2106 = $2105 & 1;
        $2107 = ($2106|0)==(0);
        if ($2107) {
         $2108 = HEAP32[$pOp>>2]|0;
         $2109 = (($2108) + 8|0);
         $2110 = HEAP32[$2109>>2]|0;
         $2111 = (($2110) - 1)|0;
         HEAP32[$pc>>2] = $2111;
        }
        break;
       }
       case 47:  {
        $2112 = HEAP32[$pOp>>2]|0;
        $2113 = (($2112) + 8|0);
        $2114 = HEAP32[$2113>>2]|0;
        HEAP32[$p210>>2] = $2114;
        $2115 = HEAP32[$pOp>>2]|0;
        $2116 = (($2115) + 12|0);
        $2117 = HEAP32[$2116>>2]|0;
        $2118 = HEAP32[$aMem>>2]|0;
        $2119 = (($2118) + (($2117*40)|0)|0);
        HEAP32[$pDest>>2] = $2119;
        $2120 = HEAP32[$pOp>>2]|0;
        $2121 = (($2120) + 4|0);
        $2122 = HEAP32[$2121>>2]|0;
        $2123 = HEAP32[$0>>2]|0;
        $2124 = (($2123) + 56|0);
        $2125 = HEAP32[$2124>>2]|0;
        $2126 = (($2125) + ($2122<<2)|0);
        $2127 = HEAP32[$2126>>2]|0;
        HEAP32[$pC>>2] = $2127;
        $2128 = HEAP32[$pC>>2]|0;
        $2129 = (($2128) + 80|0);
        $2130 = HEAP32[$2129>>2]|0;
        HEAP32[$aOffset>>2] = $2130;
        $2131 = HEAP32[$pC>>2]|0;
        $2132 = HEAP32[$2131>>2]|0;
        HEAP32[$pCrsr>>2] = $2132;
        $2133 = HEAP32[$pC>>2]|0;
        $2134 = (_sqlite3VdbeCursorMoveto($2133)|0);
        HEAP32[$rc>>2] = $2134;
        $2135 = HEAP32[$rc>>2]|0;
        $2136 = ($2135|0)!=(0);
        if ($2136) {
         break L11;
        }
        $2137 = HEAP32[$pC>>2]|0;
        $2138 = (($2137) + 60|0);
        $2139 = HEAP32[$2138>>2]|0;
        $2140 = HEAP32[$0>>2]|0;
        $2141 = (($2140) + 72|0);
        $2142 = HEAP32[$2141>>2]|0;
        $2143 = ($2139|0)!=($2142|0);
        $2144 = HEAP32[$pC>>2]|0;
        L293: do {
         if ($2143) {
          $2145 = (($2144) + 25|0);
          $2146 = HEAP8[$2145>>0]|0;
          $2147 = ($2146<<24>>24)!=(0);
          do {
           if ($2147) {
            $2148 = HEAP32[$pCrsr>>2]|0;
            $2149 = ($2148|0)==(0|0);
            if ($2149) {
             $2150 = HEAP32[$pC>>2]|0;
             $2151 = (($2150) + 16|0);
             $2152 = HEAP32[$2151>>2]|0;
             $2153 = HEAP32[$aMem>>2]|0;
             $2154 = (($2153) + (($2152*40)|0)|0);
             HEAP32[$pReg>>2] = $2154;
             $2155 = HEAP32[$pReg>>2]|0;
             $2156 = (($2155) + 12|0);
             $2157 = HEAP32[$2156>>2]|0;
             HEAP32[$avail>>2] = $2157;
             $2158 = HEAP32[$pC>>2]|0;
             $2159 = (($2158) + 68|0);
             HEAP32[$2159>>2] = $2157;
             $2160 = HEAP32[$pC>>2]|0;
             $2161 = (($2160) + 64|0);
             HEAP32[$2161>>2] = $2157;
             $2162 = HEAP32[$pReg>>2]|0;
             $2163 = (($2162) + 16|0);
             $2164 = HEAP32[$2163>>2]|0;
             $2165 = HEAP32[$pC>>2]|0;
             $2166 = (($2165) + 76|0);
             HEAP32[$2166>>2] = $2164;
             break;
            } else {
             $2167 = HEAP32[$pDest>>2]|0;
             _sqlite3VdbeMemSetNull($2167);
             break L293;
            }
           } else {
            $2168 = HEAP32[$pC>>2]|0;
            $2169 = (($2168) + 27|0);
            $2170 = HEAP8[$2169>>0]|0;
            $2171 = ($2170&255) >>> 2;
            $2172 = $2171 & 1;
            $2173 = $2172&255;
            $2174 = ($2173|0)==(0);
            $2175 = HEAP32[$pCrsr>>2]|0;
            if ($2174) {
             (_sqlite3BtreeKeySize($2175,$payloadSize64)|0);
             $2176 = HEAP32[$pCrsr>>2]|0;
             $2177 = (_sqlite3BtreeKeyFetch($2176,$avail)|0);
             $2178 = HEAP32[$pC>>2]|0;
             $2179 = (($2178) + 76|0);
             HEAP32[$2179>>2] = $2177;
             $2180 = $payloadSize64;
             $2181 = $2180;
             $2182 = HEAP32[$2181>>2]|0;
             $2183 = (($2180) + 4)|0;
             $2184 = $2183;
             $2185 = HEAP32[$2184>>2]|0;
             $2186 = HEAP32[$pC>>2]|0;
             $2187 = (($2186) + 64|0);
             HEAP32[$2187>>2] = $2182;
            } else {
             $2188 = HEAP32[$pC>>2]|0;
             $2189 = (($2188) + 64|0);
             (_sqlite3BtreeDataSize($2175,$2189)|0);
             $2190 = HEAP32[$pCrsr>>2]|0;
             $2191 = (_sqlite3BtreeDataFetch($2190,$avail)|0);
             $2192 = HEAP32[$pC>>2]|0;
             $2193 = (($2192) + 76|0);
             HEAP32[$2193>>2] = $2191;
            }
            $2194 = HEAP32[$pC>>2]|0;
            $2195 = (($2194) + 64|0);
            $2196 = HEAP32[$2195>>2]|0;
            $2197 = HEAP32[$avail>>2]|0;
            $2198 = ($2196>>>0)<=($2197>>>0);
            if ($2198) {
             $2199 = HEAP32[$pC>>2]|0;
             $2200 = (($2199) + 64|0);
             $2201 = HEAP32[$2200>>2]|0;
             $2202 = HEAP32[$pC>>2]|0;
             $2203 = (($2202) + 68|0);
             HEAP32[$2203>>2] = $2201;
            } else {
             $2204 = HEAP32[$avail>>2]|0;
             $2205 = HEAP32[$pC>>2]|0;
             $2206 = (($2205) + 68|0);
             HEAP32[$2206>>2] = $2204;
            }
            $2207 = HEAP32[$pC>>2]|0;
            $2208 = (($2207) + 64|0);
            $2209 = HEAP32[$2208>>2]|0;
            $2210 = HEAP32[$db>>2]|0;
            $2211 = (($2210) + 88|0);
            $2212 = HEAP32[$2211>>2]|0;
            $2213 = ($2209>>>0)>($2212>>>0);
            if ($2213) {
             label = 870;
             break L11;
            }
           }
          } while(0);
          $2214 = HEAP32[$0>>2]|0;
          $2215 = (($2214) + 72|0);
          $2216 = HEAP32[$2215>>2]|0;
          $2217 = HEAP32[$pC>>2]|0;
          $2218 = (($2217) + 60|0);
          HEAP32[$2218>>2] = $2216;
          $2219 = HEAP32[$pC>>2]|0;
          $2220 = (($2219) + 76|0);
          $2221 = HEAP32[$2220>>2]|0;
          $2222 = HEAP8[$2221>>0]|0;
          $2223 = $2222&255;
          $2224 = ($2223|0)<(128);
          $2225 = HEAP32[$pC>>2]|0;
          $2226 = (($2225) + 76|0);
          $2227 = HEAP32[$2226>>2]|0;
          if ($2224) {
           $2228 = HEAP8[$2227>>0]|0;
           $2229 = $2228&255;
           HEAP32[$offset>>2] = $2229;
           $2233 = 1;
          } else {
           $2230 = (_sqlite3GetVarint32($2227,$offset)|0);
           $2231 = $2230&255;
           $2233 = $2231;
          }
          $2232 = $2233&255;
          $2234 = $2232&255;
          $2235 = HEAP32[$pC>>2]|0;
          $2236 = (($2235) + 72|0);
          HEAP32[$2236>>2] = $2234;
          $2237 = HEAP32[$pC>>2]|0;
          $2238 = (($2237) + 22|0);
          HEAP16[$2238>>1] = 0;
          $2239 = HEAP32[$offset>>2]|0;
          $2240 = HEAP32[$aOffset>>2]|0;
          HEAP32[$2240>>2] = $2239;
          $2241 = HEAP32[$offset>>2]|0;
          $2242 = ($2241>>>0)>(98307);
          if (!($2242)) {
           $2243 = HEAP32[$offset>>2]|0;
           $2244 = HEAP32[$pC>>2]|0;
           $2245 = (($2244) + 64|0);
           $2246 = HEAP32[$2245>>2]|0;
           $2247 = ($2243>>>0)>($2246>>>0);
           if (!($2247)) {
            $2249 = HEAP32[$avail>>2]|0;
            $2250 = HEAP32[$offset>>2]|0;
            $2251 = ($2249>>>0)<($2250>>>0);
            if (!($2251)) {
             label = 284;
             break;
            }
            $2252 = HEAP32[$pC>>2]|0;
            $2253 = (($2252) + 76|0);
            HEAP32[$2253>>2] = 0;
            $2254 = HEAP32[$pC>>2]|0;
            $2255 = (($2254) + 68|0);
            HEAP32[$2255>>2] = 0;
            label = 284;
            break;
           }
          }
          $2248 = (_sqlite3CorruptError(71391)|0);
          HEAP32[$rc>>2] = $2248;
          break L21;
         } else {
          $2256 = (($2144) + 22|0);
          $2257 = HEAP16[$2256>>1]|0;
          $2258 = $2257&65535;
          $2259 = HEAP32[$p210>>2]|0;
          $2260 = ($2258|0)<=($2259|0);
          if ($2260) {
           label = 284;
          } else {
           label = 308;
          }
         }
        } while(0);
        do {
         if ((label|0) == 284) {
          label = 0;
          $2261 = HEAP32[$pC>>2]|0;
          $2262 = (($2261) + 72|0);
          $2263 = HEAP32[$2262>>2]|0;
          $2264 = HEAP32[$aOffset>>2]|0;
          $2265 = HEAP32[$2264>>2]|0;
          $2266 = ($2263>>>0)<($2265>>>0);
          do {
           if ($2266) {
            $2267 = HEAP32[$pC>>2]|0;
            $2268 = (($2267) + 76|0);
            $2269 = HEAP32[$2268>>2]|0;
            $2270 = ($2269|0)==(0|0);
            if ($2270) {
             dest=$sMem+0|0; stop=dest+40|0; do { HEAP32[dest>>2]=0|0; dest=dest+4|0; } while ((dest|0) < (stop|0));
             $2271 = HEAP32[$pCrsr>>2]|0;
             $2272 = HEAP32[$aOffset>>2]|0;
             $2273 = HEAP32[$2272>>2]|0;
             $2274 = HEAP32[$pC>>2]|0;
             $2275 = (($2274) + 27|0);
             $2276 = HEAP8[$2275>>0]|0;
             $2277 = ($2276&255) >>> 2;
             $2278 = $2277 & 1;
             $2279 = $2278&255;
             $2280 = ($2279|0)!=(0);
             $2281 = $2280 ^ 1;
             $2282 = $2281&1;
             $2283 = (_sqlite3VdbeMemFromBtree($2271,0,$2273,$2282,$sMem)|0);
             HEAP32[$rc>>2] = $2283;
             $2284 = HEAP32[$rc>>2]|0;
             $2285 = ($2284|0)!=(0);
             if ($2285) {
              break L21;
             }
             $2286 = (($sMem) + 16|0);
             $2287 = HEAP32[$2286>>2]|0;
             HEAP32[$zData>>2] = $2287;
            } else {
             $2288 = HEAP32[$pC>>2]|0;
             $2289 = (($2288) + 76|0);
             $2290 = HEAP32[$2289>>2]|0;
             HEAP32[$zData>>2] = $2290;
            }
            $2291 = HEAP32[$pC>>2]|0;
            $2292 = (($2291) + 22|0);
            $2293 = HEAP16[$2292>>1]|0;
            $2294 = $2293&65535;
            HEAP32[$i11>>2] = $2294;
            $2295 = HEAP32[$i11>>2]|0;
            $2296 = HEAP32[$aOffset>>2]|0;
            $2297 = (($2296) + ($2295<<2)|0);
            $2298 = HEAP32[$2297>>2]|0;
            HEAP32[$offset>>2] = $2298;
            $2299 = HEAP32[$zData>>2]|0;
            $2300 = HEAP32[$pC>>2]|0;
            $2301 = (($2300) + 72|0);
            $2302 = HEAP32[$2301>>2]|0;
            $2303 = (($2299) + ($2302)|0);
            HEAP32[$zHdr>>2] = $2303;
            $2304 = HEAP32[$zData>>2]|0;
            $2305 = HEAP32[$aOffset>>2]|0;
            $2306 = HEAP32[$2305>>2]|0;
            $2307 = (($2304) + ($2306)|0);
            HEAP32[$zEndHdr>>2] = $2307;
            while(1) {
             $2308 = HEAP32[$zHdr>>2]|0;
             $2309 = HEAP8[$2308>>0]|0;
             $2310 = $2309&255;
             $2311 = ($2310|0)<(128);
             $2312 = HEAP32[$zHdr>>2]|0;
             if ($2311) {
              $2313 = HEAP8[$2312>>0]|0;
              $2314 = $2313&255;
              HEAP32[$t>>2] = $2314;
              $2315 = HEAP32[$zHdr>>2]|0;
              $2316 = (($2315) + 1|0);
              HEAP32[$zHdr>>2] = $2316;
             } else {
              $2317 = (_sqlite3GetVarint32($2312,$t)|0);
              $2318 = $2317&255;
              $2319 = HEAP32[$zHdr>>2]|0;
              $2320 = (($2319) + ($2318)|0);
              HEAP32[$zHdr>>2] = $2320;
             }
             $2321 = HEAP32[$t>>2]|0;
             $2322 = HEAP32[$i11>>2]|0;
             $2323 = HEAP32[$pC>>2]|0;
             $2324 = (($2323) + 84|0);
             $2325 = (($2324) + ($2322<<2)|0);
             HEAP32[$2325>>2] = $2321;
             $2326 = HEAP32[$t>>2]|0;
             $2327 = (_sqlite3VdbeSerialTypeLen($2326)|0);
             HEAP32[$szField>>2] = $2327;
             $2328 = HEAP32[$szField>>2]|0;
             $2329 = HEAP32[$offset>>2]|0;
             $2330 = (($2329) + ($2328))|0;
             HEAP32[$offset>>2] = $2330;
             $2331 = HEAP32[$offset>>2]|0;
             $2332 = HEAP32[$szField>>2]|0;
             $2333 = ($2331>>>0)<($2332>>>0);
             if ($2333) {
              label = 294;
              break;
             }
             $2336 = HEAP32[$i11>>2]|0;
             $2337 = (($2336) + 1)|0;
             HEAP32[$i11>>2] = $2337;
             $2338 = HEAP32[$offset>>2]|0;
             $2339 = HEAP32[$i11>>2]|0;
             $2340 = HEAP32[$aOffset>>2]|0;
             $2341 = (($2340) + ($2339<<2)|0);
             HEAP32[$2341>>2] = $2338;
             $2342 = HEAP32[$i11>>2]|0;
             $2343 = HEAP32[$p210>>2]|0;
             $2344 = ($2342|0)<=($2343|0);
             if (!($2344)) {
              break;
             }
             $2345 = HEAP32[$zHdr>>2]|0;
             $2346 = HEAP32[$zEndHdr>>2]|0;
             $2347 = ($2345>>>0)<($2346>>>0);
             if (!($2347)) {
              break;
             }
            }
            if ((label|0) == 294) {
             label = 0;
             $2334 = HEAP32[$zEndHdr>>2]|0;
             $2335 = (($2334) + 1|0);
             HEAP32[$zHdr>>2] = $2335;
            }
            $2348 = HEAP32[$i11>>2]|0;
            $2349 = $2348&65535;
            $2350 = HEAP32[$pC>>2]|0;
            $2351 = (($2350) + 22|0);
            HEAP16[$2351>>1] = $2349;
            $2352 = HEAP32[$zHdr>>2]|0;
            $2353 = HEAP32[$zData>>2]|0;
            $2354 = $2352;
            $2355 = $2353;
            $2356 = (($2354) - ($2355))|0;
            $2357 = HEAP32[$pC>>2]|0;
            $2358 = (($2357) + 72|0);
            HEAP32[$2358>>2] = $2356;
            $2359 = HEAP32[$pC>>2]|0;
            $2360 = (($2359) + 76|0);
            $2361 = HEAP32[$2360>>2]|0;
            $2362 = ($2361|0)==(0|0);
            if ($2362) {
             _sqlite3VdbeMemRelease($sMem);
             $2363 = (($sMem) + 8|0);
             HEAP16[$2363>>1] = 1;
            }
            $2364 = HEAP32[$zHdr>>2]|0;
            $2365 = HEAP32[$zEndHdr>>2]|0;
            $2366 = ($2364>>>0)>=($2365>>>0);
            if ($2366) {
             $2367 = HEAP32[$zHdr>>2]|0;
             $2368 = HEAP32[$zEndHdr>>2]|0;
             $2369 = ($2367>>>0)>($2368>>>0);
             if (!($2369)) {
              $2370 = HEAP32[$offset>>2]|0;
              $2371 = HEAP32[$pC>>2]|0;
              $2372 = (($2371) + 64|0);
              $2373 = HEAP32[$2372>>2]|0;
              $2374 = ($2370|0)!=($2373|0);
              if (!($2374)) {
               label = 302;
              }
             }
            } else {
             label = 302;
            }
            if ((label|0) == 302) {
             label = 0;
             $2375 = HEAP32[$offset>>2]|0;
             $2376 = HEAP32[$pC>>2]|0;
             $2377 = (($2376) + 64|0);
             $2378 = HEAP32[$2377>>2]|0;
             $2379 = ($2375>>>0)>($2378>>>0);
             if (!($2379)) {
              break;
             }
            }
            $2380 = (_sqlite3CorruptError(71475)|0);
            HEAP32[$rc>>2] = $2380;
            break L21;
           }
          } while(0);
          $2381 = HEAP32[$pC>>2]|0;
          $2382 = (($2381) + 22|0);
          $2383 = HEAP16[$2382>>1]|0;
          $2384 = $2383&65535;
          $2385 = HEAP32[$p210>>2]|0;
          $2386 = ($2384|0)<=($2385|0);
          if ($2386) {
           $2387 = HEAP32[$pOp>>2]|0;
           $2388 = (($2387) + 1|0);
           $2389 = HEAP8[$2388>>0]|0;
           $2390 = $2389 << 24 >> 24;
           $2391 = ($2390|0)==(-8);
           $2392 = HEAP32[$pDest>>2]|0;
           if ($2391) {
            $2393 = HEAP32[$pOp>>2]|0;
            $2394 = (($2393) + 16|0);
            $2395 = HEAP32[$2394>>2]|0;
            _sqlite3VdbeMemShallowCopy($2392,$2395,2048);
            break;
           } else {
            _sqlite3VdbeMemSetNull($2392);
            break;
           }
          } else {
           label = 308;
          }
         }
        } while(0);
        if ((label|0) == 308) {
         label = 0;
         $2396 = HEAP32[$pDest>>2]|0;
         $2397 = (($2396) + 8|0);
         $2398 = HEAP16[$2397>>1]|0;
         $2399 = $2398&65535;
         $2400 = $2399 & 9312;
         $2401 = ($2400|0)!=(0);
         if ($2401) {
          $2402 = HEAP32[$pDest>>2]|0;
          _sqlite3VdbeMemSetNull($2402);
         }
         $2403 = HEAP32[$p210>>2]|0;
         $2404 = HEAP32[$pC>>2]|0;
         $2405 = (($2404) + 84|0);
         $2406 = (($2405) + ($2403<<2)|0);
         $2407 = HEAP32[$2406>>2]|0;
         HEAP32[$t>>2] = $2407;
         $2408 = HEAP32[$pC>>2]|0;
         $2409 = (($2408) + 68|0);
         $2410 = HEAP32[$2409>>2]|0;
         $2411 = HEAP32[$p210>>2]|0;
         $2412 = (($2411) + 1)|0;
         $2413 = HEAP32[$aOffset>>2]|0;
         $2414 = (($2413) + ($2412<<2)|0);
         $2415 = HEAP32[$2414>>2]|0;
         $2416 = ($2410>>>0)>=($2415>>>0);
         do {
          if ($2416) {
           $2417 = HEAP32[$pC>>2]|0;
           $2418 = (($2417) + 76|0);
           $2419 = HEAP32[$2418>>2]|0;
           $2420 = HEAP32[$p210>>2]|0;
           $2421 = HEAP32[$aOffset>>2]|0;
           $2422 = (($2421) + ($2420<<2)|0);
           $2423 = HEAP32[$2422>>2]|0;
           $2424 = (($2419) + ($2423)|0);
           $2425 = HEAP32[$t>>2]|0;
           $2426 = HEAP32[$pDest>>2]|0;
           (_sqlite3VdbeSerialGet($2424,$2425,$2426)|0);
          } else {
           $2427 = HEAP32[$pOp>>2]|0;
           $2428 = (($2427) + 3|0);
           $2429 = HEAP8[$2428>>0]|0;
           $2430 = $2429&255;
           $2431 = $2430 & 192;
           $2432 = ($2431|0)!=(0);
           do {
            if ($2432) {
             $2433 = HEAP32[$t>>2]|0;
             $2434 = ($2433>>>0)>=(12);
             if ($2434) {
              $2435 = HEAP32[$t>>2]|0;
              $2436 = $2435 & 1;
              $2437 = ($2436|0)==(0);
              if ($2437) {
               break;
              }
             }
             $2438 = HEAP32[$pOp>>2]|0;
             $2439 = (($2438) + 3|0);
             $2440 = HEAP8[$2439>>0]|0;
             $2441 = $2440&255;
             $2442 = $2441 & 128;
             $2443 = ($2442|0)!=(0);
             if (!($2443)) {
              label = 316;
             }
            } else {
             label = 316;
            }
           } while(0);
           if ((label|0) == 316) {
            label = 0;
            $2444 = HEAP32[$t>>2]|0;
            $2445 = (_sqlite3VdbeSerialTypeLen($2444)|0);
            HEAP32[$len>>2] = $2445;
            $2446 = ($2445|0)==(0);
            if (!($2446)) {
             $2452 = HEAP32[$pCrsr>>2]|0;
             $2453 = HEAP32[$p210>>2]|0;
             $2454 = HEAP32[$aOffset>>2]|0;
             $2455 = (($2454) + ($2453<<2)|0);
             $2456 = HEAP32[$2455>>2]|0;
             $2457 = HEAP32[$len>>2]|0;
             $2458 = HEAP32[$pC>>2]|0;
             $2459 = (($2458) + 27|0);
             $2460 = HEAP8[$2459>>0]|0;
             $2461 = ($2460&255) >>> 2;
             $2462 = $2461 & 1;
             $2463 = $2462&255;
             $2464 = ($2463|0)!=(0);
             $2465 = $2464 ^ 1;
             $2466 = $2465&1;
             $2467 = HEAP32[$pDest>>2]|0;
             $2468 = (_sqlite3VdbeMemFromBtree($2452,$2456,$2457,$2466,$2467)|0);
             HEAP32[$rc>>2] = $2468;
             $2469 = HEAP32[$rc>>2]|0;
             $2470 = ($2469|0)!=(0);
             if ($2470) {
              break L21;
             }
             $2471 = HEAP32[$pDest>>2]|0;
             $2472 = (($2471) + 16|0);
             $2473 = HEAP32[$2472>>2]|0;
             $2474 = HEAP32[$t>>2]|0;
             $2475 = HEAP32[$pDest>>2]|0;
             (_sqlite3VdbeSerialGet($2473,$2474,$2475)|0);
             $2476 = HEAP32[$pDest>>2]|0;
             $2477 = (($2476) + 8|0);
             $2478 = HEAP16[$2477>>1]|0;
             $2479 = $2478&65535;
             $2480 = $2479 & -4097;
             $2481 = $2480&65535;
             HEAP16[$2477>>1] = $2481;
             break;
            }
           }
           $2447 = HEAP32[$t>>2]|0;
           $2448 = ($2447>>>0)<=(13);
           if ($2448) {
            $2451 = $payloadSize64;
           } else {
            $2451 = 0;
           }
           $2449 = HEAP32[$t>>2]|0;
           $2450 = HEAP32[$pDest>>2]|0;
           (_sqlite3VdbeSerialGet($2451,$2449,$2450)|0);
          }
         } while(0);
         $2482 = HEAP8[$encoding>>0]|0;
         $2483 = HEAP32[$pDest>>2]|0;
         $2484 = (($2483) + 10|0);
         HEAP8[$2484>>0] = $2482;
        }
        $2485 = HEAP32[$pDest>>2]|0;
        $2486 = (($2485) + 8|0);
        $2487 = HEAP16[$2486>>1]|0;
        $2488 = $2487&65535;
        $2489 = $2488 & 4096;
        $2490 = ($2489|0)!=(0);
        if ($2490) {
         $2491 = HEAP32[$pDest>>2]|0;
         $2492 = (($2491) + 16|0);
         $2493 = HEAP32[$2492>>2]|0;
         $2494 = ($2493|0)!=(0|0);
         if ($2494) {
          $2495 = HEAP32[$pDest>>2]|0;
          $2496 = (($2495) + 8|0);
          $2497 = HEAP16[$2496>>1]|0;
          $2498 = $2497&65535;
          $2499 = $2498 & 18;
          $2500 = $2499&65535;
          HEAP16[$fx>>1] = $2500;
          $2501 = HEAP32[$pDest>>2]|0;
          $2502 = (($2501) + 16|0);
          $2503 = HEAP32[$2502>>2]|0;
          HEAP32[$zData>>2] = $2503;
          $2504 = HEAP32[$pDest>>2]|0;
          $2505 = (($2504) + 12|0);
          $2506 = HEAP32[$2505>>2]|0;
          HEAP32[$len>>2] = $2506;
          $2507 = HEAP32[$pDest>>2]|0;
          $2508 = HEAP32[$len>>2]|0;
          $2509 = (($2508) + 2)|0;
          $2510 = (_sqlite3VdbeMemClearAndResize($2507,$2509)|0);
          $2511 = ($2510|0)!=(0);
          if ($2511) {
           label = 871;
           break L1;
          }
          $2512 = HEAP32[$pDest>>2]|0;
          $2513 = (($2512) + 16|0);
          $2514 = HEAP32[$2513>>2]|0;
          $2515 = HEAP32[$zData>>2]|0;
          $2516 = HEAP32[$len>>2]|0;
          _memcpy(($2514|0),($2515|0),($2516|0))|0;
          $2517 = HEAP32[$len>>2]|0;
          $2518 = HEAP32[$pDest>>2]|0;
          $2519 = (($2518) + 16|0);
          $2520 = HEAP32[$2519>>2]|0;
          $2521 = (($2520) + ($2517)|0);
          HEAP8[$2521>>0] = 0;
          $2522 = HEAP32[$len>>2]|0;
          $2523 = (($2522) + 1)|0;
          $2524 = HEAP32[$pDest>>2]|0;
          $2525 = (($2524) + 16|0);
          $2526 = HEAP32[$2525>>2]|0;
          $2527 = (($2526) + ($2523)|0);
          HEAP8[$2527>>0] = 0;
          $2528 = HEAP16[$fx>>1]|0;
          $2529 = $2528&65535;
          $2530 = $2529 | 512;
          $2531 = $2530&65535;
          $2532 = HEAP32[$pDest>>2]|0;
          $2533 = (($2532) + 8|0);
          HEAP16[$2533>>1] = $2531;
         }
        }
        break;
       }
       case 48:  {
        $2534 = HEAP32[$pOp>>2]|0;
        $2535 = (($2534) + 16|0);
        $2536 = HEAP32[$2535>>2]|0;
        HEAP32[$zAffinity>>2] = $2536;
        $2537 = HEAP32[$pOp>>2]|0;
        $2538 = (($2537) + 4|0);
        $2539 = HEAP32[$2538>>2]|0;
        $2540 = HEAP32[$aMem>>2]|0;
        $2541 = (($2540) + (($2539*40)|0)|0);
        HEAP32[$pIn1>>2] = $2541;
        while(1) {
         $2542 = HEAP32[$zAffinity>>2]|0;
         $2543 = (($2542) + 1|0);
         HEAP32[$zAffinity>>2] = $2543;
         $2544 = HEAP8[$2542>>0]|0;
         HEAP8[$cAff>>0] = $2544;
         $2545 = $2544 << 24 >> 24;
         $2546 = ($2545|0)!=(0);
         if (!($2546)) {
          break L21;
         }
         $2547 = HEAP32[$pIn1>>2]|0;
         $2548 = HEAP8[$cAff>>0]|0;
         $2549 = HEAP8[$encoding>>0]|0;
         _applyAffinity($2547,$2548,$2549);
         $2550 = HEAP32[$pIn1>>2]|0;
         $2551 = (($2550) + 40|0);
         HEAP32[$pIn1>>2] = $2551;
        }
        break;
       }
       case 49:  {
        $2552 = $nData;
        $2553 = $2552;
        HEAP32[$2553>>2] = 0;
        $2554 = (($2552) + 4)|0;
        $2555 = $2554;
        HEAP32[$2555>>2] = 0;
        HEAP32[$nHdr>>2] = 0;
        HEAP32[$nZero>>2] = 0;
        $2556 = HEAP32[$pOp>>2]|0;
        $2557 = (($2556) + 4|0);
        $2558 = HEAP32[$2557>>2]|0;
        HEAP32[$nField>>2] = $2558;
        $2559 = HEAP32[$pOp>>2]|0;
        $2560 = (($2559) + 16|0);
        $2561 = HEAP32[$2560>>2]|0;
        HEAP32[$zAffinity13>>2] = $2561;
        $2562 = HEAP32[$nField>>2]|0;
        $2563 = HEAP32[$aMem>>2]|0;
        $2564 = (($2563) + (($2562*40)|0)|0);
        HEAP32[$pData0>>2] = $2564;
        $2565 = HEAP32[$pOp>>2]|0;
        $2566 = (($2565) + 8|0);
        $2567 = HEAP32[$2566>>2]|0;
        HEAP32[$nField>>2] = $2567;
        $2568 = HEAP32[$nField>>2]|0;
        $2569 = (($2568) - 1)|0;
        $2570 = HEAP32[$pData0>>2]|0;
        $2571 = (($2570) + (($2569*40)|0)|0);
        HEAP32[$pLast>>2] = $2571;
        $2572 = HEAP32[$0>>2]|0;
        $2573 = (($2572) + 87|0);
        $2574 = HEAP8[$2573>>0]|0;
        $2575 = $2574&255;
        HEAP32[$file_format>>2] = $2575;
        $2576 = HEAP32[$pOp>>2]|0;
        $2577 = (($2576) + 12|0);
        $2578 = HEAP32[$2577>>2]|0;
        $2579 = HEAP32[$aMem>>2]|0;
        $2580 = (($2579) + (($2578*40)|0)|0);
        HEAP32[$pOut>>2] = $2580;
        $2581 = HEAP32[$zAffinity13>>2]|0;
        $2582 = ($2581|0)!=(0|0);
        if ($2582) {
         $2583 = HEAP32[$pData0>>2]|0;
         HEAP32[$pRec>>2] = $2583;
         while(1) {
          $2584 = HEAP32[$pRec>>2]|0;
          $2585 = (($2584) + 40|0);
          HEAP32[$pRec>>2] = $2585;
          $2586 = HEAP32[$zAffinity13>>2]|0;
          $2587 = (($2586) + 1|0);
          HEAP32[$zAffinity13>>2] = $2587;
          $2588 = HEAP8[$2586>>0]|0;
          $2589 = HEAP8[$encoding>>0]|0;
          _applyAffinity($2584,$2588,$2589);
          $2590 = HEAP32[$zAffinity13>>2]|0;
          $2591 = HEAP8[$2590>>0]|0;
          $2592 = ($2591<<24>>24)!=(0);
          if (!($2592)) {
           break;
          }
         }
        }
        $2593 = HEAP32[$pLast>>2]|0;
        HEAP32[$pRec>>2] = $2593;
        while(1) {
         $2594 = HEAP32[$pRec>>2]|0;
         $2595 = HEAP32[$file_format>>2]|0;
         $2596 = (_sqlite3VdbeSerialType($2594,$2595)|0);
         HEAP32[$serial_type>>2] = $2596;
         $2597 = HEAP32[$pRec>>2]|0;
         $2598 = (($2597) + 28|0);
         HEAP32[$2598>>2] = $2596;
         $2599 = HEAP32[$serial_type>>2]|0;
         $2600 = (_sqlite3VdbeSerialTypeLen($2599)|0);
         HEAP32[$len15>>2] = $2600;
         $2601 = HEAP32[$pRec>>2]|0;
         $2602 = (($2601) + 8|0);
         $2603 = HEAP16[$2602>>1]|0;
         $2604 = $2603&65535;
         $2605 = $2604 & 16384;
         $2606 = ($2605|0)!=(0);
         do {
          if ($2606) {
           $2607 = $nData;
           $2608 = $2607;
           $2609 = HEAP32[$2608>>2]|0;
           $2610 = (($2607) + 4)|0;
           $2611 = $2610;
           $2612 = HEAP32[$2611>>2]|0;
           $2613 = ($2609|0)!=(0);
           $2614 = ($2612|0)!=(0);
           $2615 = $2613 | $2614;
           $2616 = HEAP32[$pRec>>2]|0;
           if ($2615) {
            (_sqlite3VdbeMemExpandBlob($2616)|0);
            break;
           } else {
            $2617 = HEAP32[$2616>>2]|0;
            $2618 = HEAP32[$nZero>>2]|0;
            $2619 = (($2618) + ($2617))|0;
            HEAP32[$nZero>>2] = $2619;
            $2620 = HEAP32[$pRec>>2]|0;
            $2621 = HEAP32[$2620>>2]|0;
            $2622 = HEAP32[$len15>>2]|0;
            $2623 = (($2622) - ($2621))|0;
            HEAP32[$len15>>2] = $2623;
            break;
           }
          }
         } while(0);
         $2624 = HEAP32[$len15>>2]|0;
         $2625 = ($2624|0)<(0);
         $2626 = $2625 << 31 >> 31;
         $2627 = $nData;
         $2628 = $2627;
         $2629 = HEAP32[$2628>>2]|0;
         $2630 = (($2627) + 4)|0;
         $2631 = $2630;
         $2632 = HEAP32[$2631>>2]|0;
         $2633 = (_i64Add(($2629|0),($2632|0),($2624|0),($2626|0))|0);
         $2634 = tempRet0;
         $2635 = $nData;
         $2636 = $2635;
         HEAP32[$2636>>2] = $2633;
         $2637 = (($2635) + 4)|0;
         $2638 = $2637;
         HEAP32[$2638>>2] = $2634;
         $2639 = HEAP32[$serial_type>>2]|0;
         $2640 = ($2639>>>0)<=(127);
         if ($2640) {
          $2645 = 1;
         } else {
          $2641 = HEAP32[$serial_type>>2]|0;
          $2642 = (_sqlite3VarintLen($2641,0)|0);
          $2645 = $2642;
         }
         $2643 = HEAP32[$nHdr>>2]|0;
         $2644 = (($2643) + ($2645))|0;
         HEAP32[$nHdr>>2] = $2644;
         $2646 = HEAP32[$pRec>>2]|0;
         $2647 = (($2646) + -40|0);
         HEAP32[$pRec>>2] = $2647;
         $2648 = HEAP32[$pData0>>2]|0;
         $2649 = ($2647>>>0)>=($2648>>>0);
         if (!($2649)) {
          break;
         }
        }
        $2650 = HEAP32[$nHdr>>2]|0;
        $2651 = ($2650|0)<=(126);
        $2652 = HEAP32[$nHdr>>2]|0;
        if ($2651) {
         $2653 = (($2652) + 1)|0;
         HEAP32[$nHdr>>2] = $2653;
        } else {
         $2654 = ($2652|0)<(0);
         $2655 = $2654 << 31 >> 31;
         $2656 = (_sqlite3VarintLen($2652,$2655)|0);
         HEAP32[$nVarint>>2] = $2656;
         $2657 = HEAP32[$nVarint>>2]|0;
         $2658 = HEAP32[$nHdr>>2]|0;
         $2659 = (($2658) + ($2657))|0;
         HEAP32[$nHdr>>2] = $2659;
         $2660 = HEAP32[$nVarint>>2]|0;
         $2661 = HEAP32[$nHdr>>2]|0;
         $2662 = ($2661|0)<(0);
         $2663 = $2662 << 31 >> 31;
         $2664 = (_sqlite3VarintLen($2661,$2663)|0);
         $2665 = ($2660|0)<($2664|0);
         if ($2665) {
          $2666 = HEAP32[$nHdr>>2]|0;
          $2667 = (($2666) + 1)|0;
          HEAP32[$nHdr>>2] = $2667;
         }
        }
        $2668 = HEAP32[$nHdr>>2]|0;
        $2669 = ($2668|0)<(0);
        $2670 = $2669 << 31 >> 31;
        $2671 = $nData;
        $2672 = $2671;
        $2673 = HEAP32[$2672>>2]|0;
        $2674 = (($2671) + 4)|0;
        $2675 = $2674;
        $2676 = HEAP32[$2675>>2]|0;
        $2677 = (_i64Add(($2668|0),($2670|0),($2673|0),($2676|0))|0);
        $2678 = tempRet0;
        $2679 = $nByte12;
        $2680 = $2679;
        HEAP32[$2680>>2] = $2677;
        $2681 = (($2679) + 4)|0;
        $2682 = $2681;
        HEAP32[$2682>>2] = $2678;
        $2683 = $nByte12;
        $2684 = $2683;
        $2685 = HEAP32[$2684>>2]|0;
        $2686 = (($2683) + 4)|0;
        $2687 = $2686;
        $2688 = HEAP32[$2687>>2]|0;
        $2689 = HEAP32[$db>>2]|0;
        $2690 = (($2689) + 88|0);
        $2691 = HEAP32[$2690>>2]|0;
        $2692 = ($2691|0)<(0);
        $2693 = $2692 << 31 >> 31;
        $2694 = ($2688|0)>($2693|0);
        $2695 = ($2685>>>0)>($2691>>>0);
        $2696 = ($2688|0)==($2693|0);
        $2697 = $2696 & $2695;
        $2698 = $2694 | $2697;
        if ($2698) {
         label = 870;
         break L11;
        }
        $2699 = HEAP32[$pOut>>2]|0;
        $2700 = $nByte12;
        $2701 = $2700;
        $2702 = HEAP32[$2701>>2]|0;
        $2703 = (($2700) + 4)|0;
        $2704 = $2703;
        $2705 = HEAP32[$2704>>2]|0;
        $2706 = (_sqlite3VdbeMemClearAndResize($2699,$2702)|0);
        $2707 = ($2706|0)!=(0);
        if ($2707) {
         label = 871;
         break L1;
        }
        $2708 = HEAP32[$pOut>>2]|0;
        $2709 = (($2708) + 16|0);
        $2710 = HEAP32[$2709>>2]|0;
        HEAP32[$zNewRecord>>2] = $2710;
        $2711 = HEAP32[$nHdr>>2]|0;
        $2712 = ($2711>>>0)<(128);
        if ($2712) {
         $2713 = HEAP32[$nHdr>>2]|0;
         $2714 = $2713&255;
         $2715 = HEAP32[$zNewRecord>>2]|0;
         HEAP8[$2715>>0] = $2714;
         $2722 = 1;
        } else {
         $2716 = HEAP32[$zNewRecord>>2]|0;
         $2717 = HEAP32[$nHdr>>2]|0;
         $2718 = ($2717|0)<(0);
         $2719 = $2718 << 31 >> 31;
         $2720 = (_sqlite3PutVarint($2716,$2717,$2719)|0);
         $2722 = $2720;
        }
        $2721 = $2722&255;
        $2723 = $2721&255;
        HEAP32[$i14>>2] = $2723;
        $2724 = HEAP32[$nHdr>>2]|0;
        HEAP32[$j>>2] = $2724;
        $2725 = HEAP32[$pData0>>2]|0;
        HEAP32[$pRec>>2] = $2725;
        while(1) {
         $2726 = HEAP32[$pRec>>2]|0;
         $2727 = (($2726) + 28|0);
         $2728 = HEAP32[$2727>>2]|0;
         HEAP32[$serial_type>>2] = $2728;
         $2729 = HEAP32[$serial_type>>2]|0;
         $2730 = ($2729>>>0)<(128);
         if ($2730) {
          $2731 = HEAP32[$serial_type>>2]|0;
          $2732 = $2731&255;
          $2733 = HEAP32[$i14>>2]|0;
          $2734 = HEAP32[$zNewRecord>>2]|0;
          $2735 = (($2734) + ($2733)|0);
          HEAP8[$2735>>0] = $2732;
          $2742 = 1;
         } else {
          $2736 = HEAP32[$i14>>2]|0;
          $2737 = HEAP32[$zNewRecord>>2]|0;
          $2738 = (($2737) + ($2736)|0);
          $2739 = HEAP32[$serial_type>>2]|0;
          $2740 = (_sqlite3PutVarint($2738,$2739,0)|0);
          $2742 = $2740;
         }
         $2741 = $2742&255;
         $2743 = $2741&255;
         $2744 = HEAP32[$i14>>2]|0;
         $2745 = (($2744) + ($2743))|0;
         HEAP32[$i14>>2] = $2745;
         $2746 = HEAP32[$j>>2]|0;
         $2747 = HEAP32[$zNewRecord>>2]|0;
         $2748 = (($2747) + ($2746)|0);
         $2749 = HEAP32[$pRec>>2]|0;
         $2750 = HEAP32[$serial_type>>2]|0;
         $2751 = (_sqlite3VdbeSerialPut($2748,$2749,$2750)|0);
         $2752 = HEAP32[$j>>2]|0;
         $2753 = (($2752) + ($2751))|0;
         HEAP32[$j>>2] = $2753;
         $2754 = HEAP32[$pRec>>2]|0;
         $2755 = (($2754) + 40|0);
         HEAP32[$pRec>>2] = $2755;
         $2756 = HEAP32[$pLast>>2]|0;
         $2757 = ($2755>>>0)<=($2756>>>0);
         if (!($2757)) {
          break;
         }
        }
        $2758 = $nByte12;
        $2759 = $2758;
        $2760 = HEAP32[$2759>>2]|0;
        $2761 = (($2758) + 4)|0;
        $2762 = $2761;
        $2763 = HEAP32[$2762>>2]|0;
        $2764 = HEAP32[$pOut>>2]|0;
        $2765 = (($2764) + 12|0);
        HEAP32[$2765>>2] = $2760;
        $2766 = HEAP32[$pOut>>2]|0;
        $2767 = (($2766) + 8|0);
        HEAP16[$2767>>1] = 16;
        $2768 = HEAP32[$nZero>>2]|0;
        $2769 = ($2768|0)!=(0);
        if ($2769) {
         $2770 = HEAP32[$nZero>>2]|0;
         $2771 = HEAP32[$pOut>>2]|0;
         HEAP32[$2771>>2] = $2770;
         $2772 = HEAP32[$pOut>>2]|0;
         $2773 = (($2772) + 8|0);
         $2774 = HEAP16[$2773>>1]|0;
         $2775 = $2774&65535;
         $2776 = $2775 | 16384;
         $2777 = $2776&65535;
         HEAP16[$2773>>1] = $2777;
        }
        $2778 = HEAP32[$pOut>>2]|0;
        $2779 = (($2778) + 10|0);
        HEAP8[$2779>>0] = 1;
        break;
       }
       case 50:  {
        $2780 = HEAP32[$pOp>>2]|0;
        $2781 = (($2780) + 4|0);
        $2782 = HEAP32[$2781>>2]|0;
        $2783 = HEAP32[$0>>2]|0;
        $2784 = (($2783) + 56|0);
        $2785 = HEAP32[$2784>>2]|0;
        $2786 = (($2785) + ($2782<<2)|0);
        $2787 = HEAP32[$2786>>2]|0;
        $2788 = HEAP32[$2787>>2]|0;
        HEAP32[$pCrsr16>>2] = $2788;
        $2789 = $nEntry;
        $2790 = $2789;
        HEAP32[$2790>>2] = 0;
        $2791 = (($2789) + 4)|0;
        $2792 = $2791;
        HEAP32[$2792>>2] = 0;
        $2793 = HEAP32[$pCrsr16>>2]|0;
        $2794 = (_sqlite3BtreeCount($2793,$nEntry)|0);
        HEAP32[$rc>>2] = $2794;
        $2795 = $nEntry;
        $2796 = $2795;
        $2797 = HEAP32[$2796>>2]|0;
        $2798 = (($2795) + 4)|0;
        $2799 = $2798;
        $2800 = HEAP32[$2799>>2]|0;
        $2801 = HEAP32[$pOut>>2]|0;
        $2802 = $2801;
        $2803 = $2802;
        HEAP32[$2803>>2] = $2797;
        $2804 = (($2802) + 4)|0;
        $2805 = $2804;
        HEAP32[$2805>>2] = $2800;
        break;
       }
       case 2:  {
        $2806 = HEAP32[$pOp>>2]|0;
        $2807 = (($2806) + 4|0);
        $2808 = HEAP32[$2807>>2]|0;
        HEAP32[$p117>>2] = $2808;
        $2809 = HEAP32[$pOp>>2]|0;
        $2810 = (($2809) + 16|0);
        $2811 = HEAP32[$2810>>2]|0;
        HEAP32[$zName>>2] = $2811;
        $2812 = HEAP32[$p117>>2]|0;
        $2813 = ($2812|0)==(0);
        if ($2813) {
         $2814 = HEAP32[$db>>2]|0;
         $2815 = (($2814) + 156|0);
         $2816 = HEAP32[$2815>>2]|0;
         $2817 = ($2816|0)>(0);
         if ($2817) {
          $2818 = HEAP32[$0>>2]|0;
          $2819 = (($2818) + 44|0);
          $2820 = HEAP32[$db>>2]|0;
          _sqlite3SetString($2819,$2820,31440,$vararg_buffer84);
          HEAP32[$rc>>2] = 5;
          break L21;
         }
         $2821 = HEAP32[$zName>>2]|0;
         $2822 = (_sqlite3Strlen30($2821)|0);
         HEAP32[$nName>>2] = $2822;
         $2823 = HEAP32[$db>>2]|0;
         $2824 = HEAP32[$db>>2]|0;
         $2825 = (($2824) + 500|0);
         $2826 = HEAP32[$2825>>2]|0;
         $2827 = HEAP32[$db>>2]|0;
         $2828 = (($2827) + 496|0);
         $2829 = HEAP32[$2828>>2]|0;
         $2830 = (($2826) + ($2829))|0;
         $2831 = (_sqlite3VtabSavepoint($2823,0,$2830)|0);
         HEAP32[$rc>>2] = $2831;
         $2832 = HEAP32[$rc>>2]|0;
         $2833 = ($2832|0)!=(0);
         if ($2833) {
          break L11;
         }
         $2834 = HEAP32[$db>>2]|0;
         $2835 = HEAP32[$nName>>2]|0;
         $2836 = (32 + ($2835))|0;
         $2837 = (($2836) + 1)|0;
         $2838 = (_sqlite3DbMallocRaw($2834,$2837,0)|0);
         HEAP32[$pNew>>2] = $2838;
         $2839 = HEAP32[$pNew>>2]|0;
         $2840 = ($2839|0)!=(0|0);
         if (!($2840)) {
          break L21;
         }
         $2841 = HEAP32[$pNew>>2]|0;
         $2842 = (($2841) + 32|0);
         $2843 = HEAP32[$pNew>>2]|0;
         HEAP32[$2843>>2] = $2842;
         $2844 = HEAP32[$pNew>>2]|0;
         $2845 = HEAP32[$2844>>2]|0;
         $2846 = HEAP32[$zName>>2]|0;
         $2847 = HEAP32[$nName>>2]|0;
         $2848 = (($2847) + 1)|0;
         _memcpy(($2845|0),($2846|0),($2848|0))|0;
         $2849 = HEAP32[$db>>2]|0;
         $2850 = (($2849) + 62|0);
         $2851 = HEAP8[$2850>>0]|0;
         $2852 = ($2851<<24>>24)!=(0);
         $2853 = HEAP32[$db>>2]|0;
         if ($2852) {
          $2854 = (($2853) + 62|0);
          HEAP8[$2854>>0] = 0;
          $2855 = HEAP32[$db>>2]|0;
          $2856 = (($2855) + 69|0);
          HEAP8[$2856>>0] = 1;
         } else {
          $2857 = (($2853) + 496|0);
          $2858 = HEAP32[$2857>>2]|0;
          $2859 = (($2858) + 1)|0;
          HEAP32[$2857>>2] = $2859;
         }
         $2860 = HEAP32[$db>>2]|0;
         $2861 = (($2860) + 488|0);
         $2862 = HEAP32[$2861>>2]|0;
         $2863 = HEAP32[$pNew>>2]|0;
         $2864 = (($2863) + 24|0);
         HEAP32[$2864>>2] = $2862;
         $2865 = HEAP32[$pNew>>2]|0;
         $2866 = HEAP32[$db>>2]|0;
         $2867 = (($2866) + 488|0);
         HEAP32[$2867>>2] = $2865;
         $2868 = HEAP32[$db>>2]|0;
         $2869 = (($2868) + 504|0);
         $2870 = $2869;
         $2871 = $2870;
         $2872 = HEAP32[$2871>>2]|0;
         $2873 = (($2870) + 4)|0;
         $2874 = $2873;
         $2875 = HEAP32[$2874>>2]|0;
         $2876 = HEAP32[$pNew>>2]|0;
         $2877 = (($2876) + 8|0);
         $2878 = $2877;
         $2879 = $2878;
         HEAP32[$2879>>2] = $2872;
         $2880 = (($2878) + 4)|0;
         $2881 = $2880;
         HEAP32[$2881>>2] = $2875;
         $2882 = HEAP32[$db>>2]|0;
         $2883 = (($2882) + 512|0);
         $2884 = $2883;
         $2885 = $2884;
         $2886 = HEAP32[$2885>>2]|0;
         $2887 = (($2884) + 4)|0;
         $2888 = $2887;
         $2889 = HEAP32[$2888>>2]|0;
         $2890 = HEAP32[$pNew>>2]|0;
         $2891 = (($2890) + 16|0);
         $2892 = $2891;
         $2893 = $2892;
         HEAP32[$2893>>2] = $2886;
         $2894 = (($2892) + 4)|0;
         $2895 = $2894;
         HEAP32[$2895>>2] = $2889;
         break L21;
        }
        HEAP32[$iSavepoint>>2] = 0;
        $2896 = HEAP32[$db>>2]|0;
        $2897 = (($2896) + 488|0);
        $2898 = HEAP32[$2897>>2]|0;
        HEAP32[$pSavepoint>>2] = $2898;
        while(1) {
         $2899 = HEAP32[$pSavepoint>>2]|0;
         $2900 = ($2899|0)!=(0|0);
         if (!($2900)) {
          break;
         }
         $2901 = HEAP32[$pSavepoint>>2]|0;
         $2902 = HEAP32[$2901>>2]|0;
         $2903 = HEAP32[$zName>>2]|0;
         $2904 = (_sqlite3_stricmp($2902,$2903)|0);
         $2905 = ($2904|0)!=(0);
         if (!($2905)) {
          break;
         }
         $2906 = HEAP32[$iSavepoint>>2]|0;
         $2907 = (($2906) + 1)|0;
         HEAP32[$iSavepoint>>2] = $2907;
         $2908 = HEAP32[$pSavepoint>>2]|0;
         $2909 = (($2908) + 24|0);
         $2910 = HEAP32[$2909>>2]|0;
         HEAP32[$pSavepoint>>2] = $2910;
        }
        $2911 = HEAP32[$pSavepoint>>2]|0;
        $2912 = ($2911|0)!=(0|0);
        if (!($2912)) {
         $2913 = HEAP32[$0>>2]|0;
         $2914 = (($2913) + 44|0);
         $2915 = HEAP32[$db>>2]|0;
         $2916 = HEAP32[$zName>>2]|0;
         HEAP32[$vararg_buffer84>>2] = $2916;
         _sqlite3SetString($2914,$2915,31496,$vararg_buffer84);
         HEAP32[$rc>>2] = 1;
         break L21;
        }
        $2917 = HEAP32[$db>>2]|0;
        $2918 = (($2917) + 156|0);
        $2919 = HEAP32[$2918>>2]|0;
        $2920 = ($2919|0)>(0);
        $2921 = HEAP32[$p117>>2]|0;
        $2922 = ($2921|0)==(1);
        $or$cond5 = $2920 & $2922;
        if ($or$cond5) {
         $2923 = HEAP32[$0>>2]|0;
         $2924 = (($2923) + 44|0);
         $2925 = HEAP32[$db>>2]|0;
         _sqlite3SetString($2924,$2925,31520,$vararg_buffer84);
         HEAP32[$rc>>2] = 5;
         break L21;
        }
        $2926 = HEAP32[$pSavepoint>>2]|0;
        $2927 = (($2926) + 24|0);
        $2928 = HEAP32[$2927>>2]|0;
        $2929 = ($2928|0)==(0|0);
        if ($2929) {
         $2930 = HEAP32[$db>>2]|0;
         $2931 = (($2930) + 69|0);
         $2932 = HEAP8[$2931>>0]|0;
         $2933 = $2932&255;
         $2934 = ($2933|0)!=(0);
         $2936 = $2934;
        } else {
         $2936 = 0;
        }
        $2935 = $2936&1;
        HEAP32[$isTransaction>>2] = $2935;
        $2937 = HEAP32[$isTransaction>>2]|0;
        $2938 = ($2937|0)!=(0);
        $2939 = HEAP32[$p117>>2]|0;
        $2940 = ($2939|0)==(1);
        $or$cond7 = $2938 & $2940;
        if ($or$cond7) {
         $2941 = HEAP32[$0>>2]|0;
         $2942 = (_sqlite3VdbeCheckFk($2941,1)|0);
         HEAP32[$rc>>2] = $2942;
         $2943 = ($2942|0)!=(0);
         if ($2943) {
          label = 869;
          break L11;
         }
         $2944 = HEAP32[$db>>2]|0;
         $2945 = (($2944) + 62|0);
         HEAP8[$2945>>0] = 1;
         $2946 = HEAP32[$0>>2]|0;
         $2947 = (_sqlite3VdbeHalt($2946)|0);
         $2948 = ($2947|0)==(5);
         if ($2948) {
          label = 381;
          break L11;
         }
         $2956 = HEAP32[$db>>2]|0;
         $2957 = (($2956) + 69|0);
         HEAP8[$2957>>0] = 0;
         $2958 = HEAP32[$0>>2]|0;
         $2959 = (($2958) + 80|0);
         $2960 = HEAP32[$2959>>2]|0;
         HEAP32[$rc>>2] = $2960;
        } else {
         $2961 = HEAP32[$db>>2]|0;
         $2962 = (($2961) + 496|0);
         $2963 = HEAP32[$2962>>2]|0;
         $2964 = HEAP32[$iSavepoint>>2]|0;
         $2965 = (($2963) - ($2964))|0;
         $2966 = (($2965) - 1)|0;
         HEAP32[$iSavepoint>>2] = $2966;
         $2967 = HEAP32[$p117>>2]|0;
         $2968 = ($2967|0)==(2);
         L456: do {
          if ($2968) {
           $2969 = HEAP32[$db>>2]|0;
           $2970 = (($2969) + 24|0);
           $2971 = HEAP32[$2970>>2]|0;
           $2972 = $2971 & 2;
           $2973 = ($2972|0)!=(0);
           $2974 = $2973&1;
           HEAP32[$isSchemaChange>>2] = $2974;
           HEAP32[$ii>>2] = 0;
           while(1) {
            $2975 = HEAP32[$ii>>2]|0;
            $2976 = HEAP32[$db>>2]|0;
            $2977 = (($2976) + 20|0);
            $2978 = HEAP32[$2977>>2]|0;
            $2979 = ($2975|0)<($2978|0);
            if (!($2979)) {
             break L456;
            }
            $2980 = HEAP32[$ii>>2]|0;
            $2981 = HEAP32[$db>>2]|0;
            $2982 = (($2981) + 16|0);
            $2983 = HEAP32[$2982>>2]|0;
            $2984 = (($2983) + ($2980<<4)|0);
            $2985 = (($2984) + 4|0);
            $2986 = HEAP32[$2985>>2]|0;
            $2987 = HEAP32[$isSchemaChange>>2]|0;
            $2988 = ($2987|0)==(0);
            $2989 = $2988&1;
            $2990 = (_sqlite3BtreeTripAllCursors($2986,516,$2989)|0);
            HEAP32[$rc>>2] = $2990;
            $2991 = HEAP32[$rc>>2]|0;
            $2992 = ($2991|0)!=(0);
            if ($2992) {
             break L11;
            }
            $2993 = HEAP32[$ii>>2]|0;
            $2994 = (($2993) + 1)|0;
            HEAP32[$ii>>2] = $2994;
           }
          } else {
           HEAP32[$isSchemaChange>>2] = 0;
          }
         } while(0);
         HEAP32[$ii>>2] = 0;
         while(1) {
          $2995 = HEAP32[$ii>>2]|0;
          $2996 = HEAP32[$db>>2]|0;
          $2997 = (($2996) + 20|0);
          $2998 = HEAP32[$2997>>2]|0;
          $2999 = ($2995|0)<($2998|0);
          if (!($2999)) {
           break;
          }
          $3000 = HEAP32[$ii>>2]|0;
          $3001 = HEAP32[$db>>2]|0;
          $3002 = (($3001) + 16|0);
          $3003 = HEAP32[$3002>>2]|0;
          $3004 = (($3003) + ($3000<<4)|0);
          $3005 = (($3004) + 4|0);
          $3006 = HEAP32[$3005>>2]|0;
          $3007 = HEAP32[$p117>>2]|0;
          $3008 = HEAP32[$iSavepoint>>2]|0;
          $3009 = (_sqlite3BtreeSavepoint($3006,$3007,$3008)|0);
          HEAP32[$rc>>2] = $3009;
          $3010 = HEAP32[$rc>>2]|0;
          $3011 = ($3010|0)!=(0);
          if ($3011) {
           break L11;
          }
          $3012 = HEAP32[$ii>>2]|0;
          $3013 = (($3012) + 1)|0;
          HEAP32[$ii>>2] = $3013;
         }
         $3014 = HEAP32[$isSchemaChange>>2]|0;
         $3015 = ($3014|0)!=(0);
         if ($3015) {
          $3016 = HEAP32[$db>>2]|0;
          _sqlite3ExpirePreparedStatements($3016);
          $3017 = HEAP32[$db>>2]|0;
          _sqlite3ResetAllSchemasOfConnection($3017);
          $3018 = HEAP32[$db>>2]|0;
          $3019 = (($3018) + 24|0);
          $3020 = HEAP32[$3019>>2]|0;
          $3021 = $3020 | 2;
          $3022 = HEAP32[$db>>2]|0;
          $3023 = (($3022) + 24|0);
          HEAP32[$3023>>2] = $3021;
         }
        }
        while(1) {
         $3024 = HEAP32[$db>>2]|0;
         $3025 = (($3024) + 488|0);
         $3026 = HEAP32[$3025>>2]|0;
         $3027 = HEAP32[$pSavepoint>>2]|0;
         $3028 = ($3026|0)!=($3027|0);
         if (!($3028)) {
          break;
         }
         $3029 = HEAP32[$db>>2]|0;
         $3030 = (($3029) + 488|0);
         $3031 = HEAP32[$3030>>2]|0;
         HEAP32[$pTmp>>2] = $3031;
         $3032 = HEAP32[$pTmp>>2]|0;
         $3033 = (($3032) + 24|0);
         $3034 = HEAP32[$3033>>2]|0;
         $3035 = HEAP32[$db>>2]|0;
         $3036 = (($3035) + 488|0);
         HEAP32[$3036>>2] = $3034;
         $3037 = HEAP32[$db>>2]|0;
         $3038 = HEAP32[$pTmp>>2]|0;
         _sqlite3DbFree($3037,$3038);
         $3039 = HEAP32[$db>>2]|0;
         $3040 = (($3039) + 496|0);
         $3041 = HEAP32[$3040>>2]|0;
         $3042 = (($3041) + -1)|0;
         HEAP32[$3040>>2] = $3042;
        }
        $3043 = HEAP32[$p117>>2]|0;
        $3044 = ($3043|0)==(1);
        $3045 = HEAP32[$pSavepoint>>2]|0;
        if ($3044) {
         $3046 = (($3045) + 24|0);
         $3047 = HEAP32[$3046>>2]|0;
         $3048 = HEAP32[$db>>2]|0;
         $3049 = (($3048) + 488|0);
         HEAP32[$3049>>2] = $3047;
         $3050 = HEAP32[$db>>2]|0;
         $3051 = HEAP32[$pSavepoint>>2]|0;
         _sqlite3DbFree($3050,$3051);
         $3052 = HEAP32[$isTransaction>>2]|0;
         $3053 = ($3052|0)!=(0);
         if (!($3053)) {
          $3054 = HEAP32[$db>>2]|0;
          $3055 = (($3054) + 496|0);
          $3056 = HEAP32[$3055>>2]|0;
          $3057 = (($3056) + -1)|0;
          HEAP32[$3055>>2] = $3057;
         }
        } else {
         $3058 = (($3045) + 8|0);
         $3059 = $3058;
         $3060 = $3059;
         $3061 = HEAP32[$3060>>2]|0;
         $3062 = (($3059) + 4)|0;
         $3063 = $3062;
         $3064 = HEAP32[$3063>>2]|0;
         $3065 = HEAP32[$db>>2]|0;
         $3066 = (($3065) + 504|0);
         $3067 = $3066;
         $3068 = $3067;
         HEAP32[$3068>>2] = $3061;
         $3069 = (($3067) + 4)|0;
         $3070 = $3069;
         HEAP32[$3070>>2] = $3064;
         $3071 = HEAP32[$pSavepoint>>2]|0;
         $3072 = (($3071) + 16|0);
         $3073 = $3072;
         $3074 = $3073;
         $3075 = HEAP32[$3074>>2]|0;
         $3076 = (($3073) + 4)|0;
         $3077 = $3076;
         $3078 = HEAP32[$3077>>2]|0;
         $3079 = HEAP32[$db>>2]|0;
         $3080 = (($3079) + 512|0);
         $3081 = $3080;
         $3082 = $3081;
         HEAP32[$3082>>2] = $3075;
         $3083 = (($3081) + 4)|0;
         $3084 = $3083;
         HEAP32[$3084>>2] = $3078;
        }
        $3085 = HEAP32[$isTransaction>>2]|0;
        $3086 = ($3085|0)!=(0);
        if (!($3086)) {
         $3087 = HEAP32[$db>>2]|0;
         $3088 = HEAP32[$p117>>2]|0;
         $3089 = HEAP32[$iSavepoint>>2]|0;
         $3090 = (_sqlite3VtabSavepoint($3087,$3088,$3089)|0);
         HEAP32[$rc>>2] = $3090;
         $3091 = HEAP32[$rc>>2]|0;
         $3092 = ($3091|0)!=(0);
         if ($3092) {
          break L11;
         }
        }
        break;
       }
       case 3:  {
        $3093 = HEAP32[$pOp>>2]|0;
        $3094 = (($3093) + 4|0);
        $3095 = HEAP32[$3094>>2]|0;
        HEAP32[$desiredAutoCommit>>2] = $3095;
        $3096 = HEAP32[$pOp>>2]|0;
        $3097 = (($3096) + 8|0);
        $3098 = HEAP32[$3097>>2]|0;
        HEAP32[$iRollback>>2] = $3098;
        $3099 = HEAP32[$desiredAutoCommit>>2]|0;
        $3100 = ($3099|0)!=(0);
        if ($3100) {
         $3101 = HEAP32[$db>>2]|0;
         $3102 = (($3101) + 62|0);
         $3103 = HEAP8[$3102>>0]|0;
         $3104 = ($3103<<24>>24)!=(0);
         $3105 = $3104 ^ 1;
         $3107 = $3105;
        } else {
         $3107 = 0;
        }
        $3106 = $3107&1;
        HEAP32[$turnOnAC>>2] = $3106;
        $3108 = HEAP32[$turnOnAC>>2]|0;
        $3109 = ($3108|0)==(0);
        $3110 = HEAP32[$iRollback>>2]|0;
        $3111 = ($3110|0)!=(0);
        $or$cond9 = $3109 | $3111;
        if (!($or$cond9)) {
         $3112 = HEAP32[$db>>2]|0;
         $3113 = (($3112) + 156|0);
         $3114 = HEAP32[$3113>>2]|0;
         $3115 = ($3114|0)>(0);
         if ($3115) {
          $3116 = HEAP32[$0>>2]|0;
          $3117 = (($3116) + 44|0);
          $3118 = HEAP32[$db>>2]|0;
          _sqlite3SetString($3117,$3118,31576,$vararg_buffer84);
          HEAP32[$rc>>2] = 5;
          break L21;
         }
        }
        $3119 = HEAP32[$desiredAutoCommit>>2]|0;
        $3120 = HEAP32[$db>>2]|0;
        $3121 = (($3120) + 62|0);
        $3122 = HEAP8[$3121>>0]|0;
        $3123 = $3122&255;
        $3124 = ($3119|0)!=($3123|0);
        if ($3124) {
         label = 409;
         break L11;
        }
        $3155 = HEAP32[$0>>2]|0;
        $3156 = (($3155) + 44|0);
        $3157 = HEAP32[$db>>2]|0;
        $3158 = HEAP32[$desiredAutoCommit>>2]|0;
        $3159 = ($3158|0)!=(0);
        if ($3159) {
         $3160 = HEAP32[$iRollback>>2]|0;
         $3161 = ($3160|0)!=(0);
         $3162 = $3161 ? 31680 : 31728;
         $3163 = $3162;
        } else {
         $3163 = 31632;
        }
        _sqlite3SetString($3156,$3157,$3163,$vararg_buffer84);
        HEAP32[$rc>>2] = 1;
        break;
       }
       case 4:  {
        $3164 = HEAP32[$pOp>>2]|0;
        $3165 = (($3164) + 8|0);
        $3166 = HEAP32[$3165>>2]|0;
        $3167 = ($3166|0)!=(0);
        if ($3167) {
         $3168 = HEAP32[$db>>2]|0;
         $3169 = (($3168) + 24|0);
         $3170 = HEAP32[$3169>>2]|0;
         $3171 = $3170 & 33554432;
         $3172 = ($3171|0)!=(0);
         if ($3172) {
          label = 422;
          break L11;
         }
        }
        $3173 = HEAP32[$pOp>>2]|0;
        $3174 = (($3173) + 4|0);
        $3175 = HEAP32[$3174>>2]|0;
        $3176 = HEAP32[$db>>2]|0;
        $3177 = (($3176) + 16|0);
        $3178 = HEAP32[$3177>>2]|0;
        $3179 = (($3178) + ($3175<<4)|0);
        $3180 = (($3179) + 4|0);
        $3181 = HEAP32[$3180>>2]|0;
        HEAP32[$pBt>>2] = $3181;
        $3182 = HEAP32[$pBt>>2]|0;
        $3183 = ($3182|0)!=(0|0);
        if ($3183) {
         $3184 = HEAP32[$pBt>>2]|0;
         $3185 = HEAP32[$pOp>>2]|0;
         $3186 = (($3185) + 8|0);
         $3187 = HEAP32[$3186>>2]|0;
         $3188 = (_sqlite3BtreeBeginTrans($3184,$3187)|0);
         HEAP32[$rc>>2] = $3188;
         $3189 = HEAP32[$rc>>2]|0;
         $3190 = ($3189|0)==(5);
         if ($3190) {
          label = 425;
          break L11;
         }
         $3196 = HEAP32[$rc>>2]|0;
         $3197 = ($3196|0)!=(0);
         if ($3197) {
          break L11;
         }
         $3198 = HEAP32[$pOp>>2]|0;
         $3199 = (($3198) + 8|0);
         $3200 = HEAP32[$3199>>2]|0;
         $3201 = ($3200|0)!=(0);
         do {
          if ($3201) {
           $3202 = HEAP32[$0>>2]|0;
           $3203 = (($3202) + 88|0);
           $3204 = HEAP16[$3203>>1]|0;
           $3205 = ($3204&65535) >>> 7;
           $3206 = $3205 & 1;
           $3207 = $3206&65535;
           $3208 = ($3207|0)!=(0);
           if ($3208) {
            $3209 = HEAP32[$db>>2]|0;
            $3210 = (($3209) + 62|0);
            $3211 = HEAP8[$3210>>0]|0;
            $3212 = $3211&255;
            $3213 = ($3212|0)==(0);
            if (!($3213)) {
             $3214 = HEAP32[$db>>2]|0;
             $3215 = (($3214) + 152|0);
             $3216 = HEAP32[$3215>>2]|0;
             $3217 = ($3216|0)>(1);
             if (!($3217)) {
              break;
             }
            }
            $3218 = HEAP32[$0>>2]|0;
            $3219 = (($3218) + 104|0);
            $3220 = HEAP32[$3219>>2]|0;
            $3221 = ($3220|0)==(0);
            if ($3221) {
             $3222 = HEAP32[$db>>2]|0;
             $3223 = (($3222) + 500|0);
             $3224 = HEAP32[$3223>>2]|0;
             $3225 = (($3224) + 1)|0;
             HEAP32[$3223>>2] = $3225;
             $3226 = HEAP32[$db>>2]|0;
             $3227 = (($3226) + 496|0);
             $3228 = HEAP32[$3227>>2]|0;
             $3229 = HEAP32[$db>>2]|0;
             $3230 = (($3229) + 500|0);
             $3231 = HEAP32[$3230>>2]|0;
             $3232 = (($3228) + ($3231))|0;
             $3233 = HEAP32[$0>>2]|0;
             $3234 = (($3233) + 104|0);
             HEAP32[$3234>>2] = $3232;
            }
            $3235 = HEAP32[$db>>2]|0;
            $3236 = HEAP32[$0>>2]|0;
            $3237 = (($3236) + 104|0);
            $3238 = HEAP32[$3237>>2]|0;
            $3239 = (($3238) - 1)|0;
            $3240 = (_sqlite3VtabSavepoint($3235,0,$3239)|0);
            HEAP32[$rc>>2] = $3240;
            $3241 = HEAP32[$rc>>2]|0;
            $3242 = ($3241|0)==(0);
            if ($3242) {
             $3243 = HEAP32[$pBt>>2]|0;
             $3244 = HEAP32[$0>>2]|0;
             $3245 = (($3244) + 104|0);
             $3246 = HEAP32[$3245>>2]|0;
             $3247 = (_sqlite3BtreeBeginStmt($3243,$3246)|0);
             HEAP32[$rc>>2] = $3247;
            }
            $3248 = HEAP32[$db>>2]|0;
            $3249 = (($3248) + 504|0);
            $3250 = $3249;
            $3251 = $3250;
            $3252 = HEAP32[$3251>>2]|0;
            $3253 = (($3250) + 4)|0;
            $3254 = $3253;
            $3255 = HEAP32[$3254>>2]|0;
            $3256 = HEAP32[$0>>2]|0;
            $3257 = (($3256) + 152|0);
            $3258 = $3257;
            $3259 = $3258;
            HEAP32[$3259>>2] = $3252;
            $3260 = (($3258) + 4)|0;
            $3261 = $3260;
            HEAP32[$3261>>2] = $3255;
            $3262 = HEAP32[$db>>2]|0;
            $3263 = (($3262) + 512|0);
            $3264 = $3263;
            $3265 = $3264;
            $3266 = HEAP32[$3265>>2]|0;
            $3267 = (($3264) + 4)|0;
            $3268 = $3267;
            $3269 = HEAP32[$3268>>2]|0;
            $3270 = HEAP32[$0>>2]|0;
            $3271 = (($3270) + 160|0);
            $3272 = $3271;
            $3273 = $3272;
            HEAP32[$3273>>2] = $3266;
            $3274 = (($3272) + 4)|0;
            $3275 = $3274;
            HEAP32[$3275>>2] = $3269;
           }
          }
         } while(0);
         $3276 = HEAP32[$pBt>>2]|0;
         _sqlite3BtreeGetMeta($3276,1,$iMeta);
         $3277 = HEAP32[$pOp>>2]|0;
         $3278 = (($3277) + 4|0);
         $3279 = HEAP32[$3278>>2]|0;
         $3280 = HEAP32[$db>>2]|0;
         $3281 = (($3280) + 16|0);
         $3282 = HEAP32[$3281>>2]|0;
         $3283 = (($3282) + ($3279<<4)|0);
         $3284 = (($3283) + 12|0);
         $3285 = HEAP32[$3284>>2]|0;
         $3286 = (($3285) + 4|0);
         $3287 = HEAP32[$3286>>2]|0;
         HEAP32[$iGen>>2] = $3287;
        } else {
         HEAP32[$iMeta>>2] = 0;
         HEAP32[$iGen>>2] = 0;
        }
        $3288 = HEAP32[$pOp>>2]|0;
        $3289 = (($3288) + 3|0);
        $3290 = HEAP8[$3289>>0]|0;
        $3291 = $3290&255;
        $3292 = ($3291|0)!=(0);
        if ($3292) {
         $3293 = HEAP32[$iMeta>>2]|0;
         $3294 = HEAP32[$pOp>>2]|0;
         $3295 = (($3294) + 12|0);
         $3296 = HEAP32[$3295>>2]|0;
         $3297 = ($3293|0)!=($3296|0);
         if (!($3297)) {
          $3298 = HEAP32[$iGen>>2]|0;
          $3299 = HEAP32[$pOp>>2]|0;
          $3300 = (($3299) + 16|0);
          $3301 = HEAP32[$3300>>2]|0;
          $3302 = ($3298|0)!=($3301|0);
          if (!($3302)) {
           break L21;
          }
         }
         $3303 = HEAP32[$db>>2]|0;
         $3304 = HEAP32[$0>>2]|0;
         $3305 = (($3304) + 44|0);
         $3306 = HEAP32[$3305>>2]|0;
         _sqlite3DbFree($3303,$3306);
         $3307 = HEAP32[$db>>2]|0;
         $3308 = (_sqlite3DbStrDup($3307,5664)|0);
         $3309 = HEAP32[$0>>2]|0;
         $3310 = (($3309) + 44|0);
         HEAP32[$3310>>2] = $3308;
         $3311 = HEAP32[$pOp>>2]|0;
         $3312 = (($3311) + 4|0);
         $3313 = HEAP32[$3312>>2]|0;
         $3314 = HEAP32[$db>>2]|0;
         $3315 = (($3314) + 16|0);
         $3316 = HEAP32[$3315>>2]|0;
         $3317 = (($3316) + ($3313<<4)|0);
         $3318 = (($3317) + 12|0);
         $3319 = HEAP32[$3318>>2]|0;
         $3320 = HEAP32[$3319>>2]|0;
         $3321 = HEAP32[$iMeta>>2]|0;
         $3322 = ($3320|0)!=($3321|0);
         if ($3322) {
          $3323 = HEAP32[$db>>2]|0;
          $3324 = HEAP32[$pOp>>2]|0;
          $3325 = (($3324) + 4|0);
          $3326 = HEAP32[$3325>>2]|0;
          _sqlite3ResetOneSchema($3323,$3326);
         }
         $3327 = HEAP32[$0>>2]|0;
         $3328 = (($3327) + 88|0);
         $3329 = HEAP16[$3328>>1]|0;
         $3330 = $3329 & -33;
         $3331 = $3330 | 32;
         HEAP16[$3328>>1] = $3331;
         HEAP32[$rc>>2] = 17;
        }
        break;
       }
       case 51:  {
        $3332 = HEAP32[$pOp>>2]|0;
        $3333 = (($3332) + 4|0);
        $3334 = HEAP32[$3333>>2]|0;
        HEAP32[$iDb>>2] = $3334;
        $3335 = HEAP32[$pOp>>2]|0;
        $3336 = (($3335) + 12|0);
        $3337 = HEAP32[$3336>>2]|0;
        HEAP32[$iCookie>>2] = $3337;
        $3338 = HEAP32[$iDb>>2]|0;
        $3339 = HEAP32[$db>>2]|0;
        $3340 = (($3339) + 16|0);
        $3341 = HEAP32[$3340>>2]|0;
        $3342 = (($3341) + ($3338<<4)|0);
        $3343 = (($3342) + 4|0);
        $3344 = HEAP32[$3343>>2]|0;
        $3345 = HEAP32[$iCookie>>2]|0;
        _sqlite3BtreeGetMeta($3344,$3345,$iMeta18);
        $3346 = HEAP32[$iMeta18>>2]|0;
        $3347 = ($3346|0)<(0);
        $3348 = $3347 << 31 >> 31;
        $3349 = HEAP32[$pOut>>2]|0;
        $3350 = $3349;
        $3351 = $3350;
        HEAP32[$3351>>2] = $3346;
        $3352 = (($3350) + 4)|0;
        $3353 = $3352;
        HEAP32[$3353>>2] = $3348;
        break;
       }
       case 52:  {
        $3354 = HEAP32[$pOp>>2]|0;
        $3355 = (($3354) + 4|0);
        $3356 = HEAP32[$3355>>2]|0;
        $3357 = HEAP32[$db>>2]|0;
        $3358 = (($3357) + 16|0);
        $3359 = HEAP32[$3358>>2]|0;
        $3360 = (($3359) + ($3356<<4)|0);
        HEAP32[$pDb>>2] = $3360;
        $3361 = HEAP32[$pOp>>2]|0;
        $3362 = (($3361) + 12|0);
        $3363 = HEAP32[$3362>>2]|0;
        $3364 = HEAP32[$aMem>>2]|0;
        $3365 = (($3364) + (($3363*40)|0)|0);
        HEAP32[$pIn3>>2] = $3365;
        $3366 = HEAP32[$pIn3>>2]|0;
        (_sqlite3VdbeMemIntegerify($3366)|0);
        $3367 = HEAP32[$pDb>>2]|0;
        $3368 = (($3367) + 4|0);
        $3369 = HEAP32[$3368>>2]|0;
        $3370 = HEAP32[$pOp>>2]|0;
        $3371 = (($3370) + 8|0);
        $3372 = HEAP32[$3371>>2]|0;
        $3373 = HEAP32[$pIn3>>2]|0;
        $3374 = $3373;
        $3375 = $3374;
        $3376 = HEAP32[$3375>>2]|0;
        $3377 = (($3374) + 4)|0;
        $3378 = $3377;
        $3379 = HEAP32[$3378>>2]|0;
        $3380 = (_sqlite3BtreeUpdateMeta($3369,$3372,$3376)|0);
        HEAP32[$rc>>2] = $3380;
        $3381 = HEAP32[$pOp>>2]|0;
        $3382 = (($3381) + 8|0);
        $3383 = HEAP32[$3382>>2]|0;
        $3384 = ($3383|0)==(1);
        if ($3384) {
         $3385 = HEAP32[$pIn3>>2]|0;
         $3386 = $3385;
         $3387 = $3386;
         $3388 = HEAP32[$3387>>2]|0;
         $3389 = (($3386) + 4)|0;
         $3390 = $3389;
         $3391 = HEAP32[$3390>>2]|0;
         $3392 = HEAP32[$pDb>>2]|0;
         $3393 = (($3392) + 12|0);
         $3394 = HEAP32[$3393>>2]|0;
         HEAP32[$3394>>2] = $3388;
         $3395 = HEAP32[$db>>2]|0;
         $3396 = (($3395) + 24|0);
         $3397 = HEAP32[$3396>>2]|0;
         $3398 = $3397 | 2;
         HEAP32[$3396>>2] = $3398;
        } else {
         $3399 = HEAP32[$pOp>>2]|0;
         $3400 = (($3399) + 8|0);
         $3401 = HEAP32[$3400>>2]|0;
         $3402 = ($3401|0)==(2);
         if ($3402) {
          $3403 = HEAP32[$pIn3>>2]|0;
          $3404 = $3403;
          $3405 = $3404;
          $3406 = HEAP32[$3405>>2]|0;
          $3407 = (($3404) + 4)|0;
          $3408 = $3407;
          $3409 = HEAP32[$3408>>2]|0;
          $3410 = $3406&255;
          $3411 = HEAP32[$pDb>>2]|0;
          $3412 = (($3411) + 12|0);
          $3413 = HEAP32[$3412>>2]|0;
          $3414 = (($3413) + 76|0);
          HEAP8[$3414>>0] = $3410;
         }
        }
        $3415 = HEAP32[$pOp>>2]|0;
        $3416 = (($3415) + 4|0);
        $3417 = HEAP32[$3416>>2]|0;
        $3418 = ($3417|0)==(1);
        if ($3418) {
         $3419 = HEAP32[$db>>2]|0;
         _sqlite3ExpirePreparedStatements($3419);
         $3420 = HEAP32[$0>>2]|0;
         $3421 = (($3420) + 88|0);
         $3422 = HEAP16[$3421>>1]|0;
         $3423 = $3422 & -33;
         HEAP16[$3421>>1] = $3423;
        }
        break;
       }
       case 55: case 54:  {
        label = 453;
        break;
       }
       case 53:  {
        $3424 = HEAP32[$pOp>>2]|0;
        $3425 = (($3424) + 4|0);
        $3426 = HEAP32[$3425>>2]|0;
        $3427 = HEAP32[$0>>2]|0;
        $3428 = (($3427) + 56|0);
        $3429 = HEAP32[$3428>>2]|0;
        $3430 = (($3429) + ($3426<<2)|0);
        $3431 = HEAP32[$3430>>2]|0;
        HEAP32[$pCur>>2] = $3431;
        $3432 = HEAP32[$pCur>>2]|0;
        $3433 = ($3432|0)!=(0|0);
        if ($3433) {
         $3434 = HEAP32[$pCur>>2]|0;
         $3435 = (($3434) + 28|0);
         $3436 = HEAP32[$3435>>2]|0;
         $3437 = HEAP32[$pOp>>2]|0;
         $3438 = (($3437) + 8|0);
         $3439 = HEAP32[$3438>>2]|0;
         $3440 = ($3436|0)==($3439|0);
         if (!($3440)) {
          label = 453;
         }
        } else {
         label = 453;
        }
        break;
       }
       case 57: case 56:  {
        $3578 = HEAP32[$0>>2]|0;
        $3579 = HEAP32[$pOp>>2]|0;
        $3580 = (($3579) + 4|0);
        $3581 = HEAP32[$3580>>2]|0;
        $3582 = HEAP32[$pOp>>2]|0;
        $3583 = (($3582) + 8|0);
        $3584 = HEAP32[$3583>>2]|0;
        $3585 = (_allocateCursor($3578,$3581,$3584,-1,1)|0);
        HEAP32[$pCx>>2] = $3585;
        $3586 = HEAP32[$pCx>>2]|0;
        $3587 = ($3586|0)==(0|0);
        if ($3587) {
         label = 871;
         break L1;
        }
        $3588 = HEAP32[$pCx>>2]|0;
        $3589 = (($3588) + 25|0);
        HEAP8[$3589>>0] = 1;
        $3590 = HEAP32[$pCx>>2]|0;
        $3591 = (($3590) + 27|0);
        $3592 = HEAP8[$3591>>0]|0;
        $3593 = $3592 & -2;
        $3594 = $3593 | 1;
        HEAP8[$3591>>0] = $3594;
        $3595 = HEAP32[$db>>2]|0;
        $3596 = HEAP32[$3595>>2]|0;
        $3597 = HEAP32[$db>>2]|0;
        $3598 = HEAP32[$pCx>>2]|0;
        $3599 = (($3598) + 4|0);
        $3600 = HEAP32[$pOp>>2]|0;
        $3601 = (($3600) + 3|0);
        $3602 = HEAP8[$3601>>0]|0;
        $3603 = $3602&255;
        $3604 = 5 | $3603;
        $3605 = (_sqlite3BtreeOpen($3596,0,$3597,$3599,$3604,1054)|0);
        HEAP32[$rc>>2] = $3605;
        $3606 = HEAP32[$rc>>2]|0;
        $3607 = ($3606|0)==(0);
        if ($3607) {
         $3608 = HEAP32[$pCx>>2]|0;
         $3609 = (($3608) + 4|0);
         $3610 = HEAP32[$3609>>2]|0;
         $3611 = (_sqlite3BtreeBeginTrans($3610,1)|0);
         HEAP32[$rc>>2] = $3611;
        }
        $3612 = HEAP32[$rc>>2]|0;
        $3613 = ($3612|0)==(0);
        do {
         if ($3613) {
          $3614 = HEAP32[$pOp>>2]|0;
          $3615 = (($3614) + 16|0);
          $3616 = HEAP32[$3615>>2]|0;
          HEAP32[$pKeyInfo25>>2] = $3616;
          $3617 = ($3616|0)!=(0|0);
          $3618 = HEAP32[$pCx>>2]|0;
          $3619 = (($3618) + 4|0);
          $3620 = HEAP32[$3619>>2]|0;
          if (!($3617)) {
           $3644 = HEAP32[$pCx>>2]|0;
           $3645 = HEAP32[$3644>>2]|0;
           $3646 = (_sqlite3BtreeCursor($3620,1,1,0,$3645)|0);
           HEAP32[$rc>>2] = $3646;
           $3647 = HEAP32[$pCx>>2]|0;
           $3648 = (($3647) + 27|0);
           $3649 = HEAP8[$3648>>0]|0;
           $3650 = $3649 & -5;
           $3651 = $3650 | 4;
           HEAP8[$3648>>0] = $3651;
           break;
          }
          $3621 = HEAP32[$pOp>>2]|0;
          $3622 = (($3621) + 3|0);
          $3623 = HEAP8[$3622>>0]|0;
          $3624 = $3623&255;
          $3625 = 2 | $3624;
          $3626 = (_sqlite3BtreeCreateTable($3620,$pgno,$3625)|0);
          HEAP32[$rc>>2] = $3626;
          $3627 = HEAP32[$rc>>2]|0;
          $3628 = ($3627|0)==(0);
          if ($3628) {
           $3629 = HEAP32[$pKeyInfo25>>2]|0;
           $3630 = HEAP32[$pCx>>2]|0;
           $3631 = (($3630) + 8|0);
           HEAP32[$3631>>2] = $3629;
           $3632 = HEAP32[$pCx>>2]|0;
           $3633 = (($3632) + 4|0);
           $3634 = HEAP32[$3633>>2]|0;
           $3635 = HEAP32[$pgno>>2]|0;
           $3636 = HEAP32[$pKeyInfo25>>2]|0;
           $3637 = HEAP32[$pCx>>2]|0;
           $3638 = HEAP32[$3637>>2]|0;
           $3639 = (_sqlite3BtreeCursor($3634,$3635,1,$3636,$3638)|0);
           HEAP32[$rc>>2] = $3639;
          }
          $3640 = HEAP32[$pCx>>2]|0;
          $3641 = (($3640) + 27|0);
          $3642 = HEAP8[$3641>>0]|0;
          $3643 = $3642 & -5;
          HEAP8[$3641>>0] = $3643;
         }
        } while(0);
        $3652 = HEAP32[$pOp>>2]|0;
        $3653 = (($3652) + 3|0);
        $3654 = HEAP8[$3653>>0]|0;
        $3655 = $3654&255;
        $3656 = ($3655|0)!=(8);
        $3657 = $3656&1;
        $3658 = HEAP32[$pCx>>2]|0;
        $3659 = (($3658) + 27|0);
        $3660 = $3657&255;
        $3661 = HEAP8[$3659>>0]|0;
        $3662 = $3660 & 1;
        $3663 = ($3662 << 3)&255;
        $3664 = $3661 & -9;
        $3665 = $3664 | $3663;
        HEAP8[$3659>>0] = $3665;
        break;
       }
       case 58:  {
        $3666 = HEAP32[$0>>2]|0;
        $3667 = HEAP32[$pOp>>2]|0;
        $3668 = (($3667) + 4|0);
        $3669 = HEAP32[$3668>>2]|0;
        $3670 = HEAP32[$pOp>>2]|0;
        $3671 = (($3670) + 8|0);
        $3672 = HEAP32[$3671>>2]|0;
        $3673 = (_allocateCursor($3666,$3669,$3672,-1,1)|0);
        HEAP32[$pCx26>>2] = $3673;
        $3674 = HEAP32[$pCx26>>2]|0;
        $3675 = ($3674|0)==(0|0);
        if ($3675) {
         label = 871;
         break L1;
        }
        $3676 = HEAP32[$pOp>>2]|0;
        $3677 = (($3676) + 16|0);
        $3678 = HEAP32[$3677>>2]|0;
        $3679 = HEAP32[$pCx26>>2]|0;
        $3680 = (($3679) + 8|0);
        HEAP32[$3680>>2] = $3678;
        $3681 = HEAP32[$db>>2]|0;
        $3682 = HEAP32[$pOp>>2]|0;
        $3683 = (($3682) + 12|0);
        $3684 = HEAP32[$3683>>2]|0;
        $3685 = HEAP32[$pCx26>>2]|0;
        $3686 = (_sqlite3VdbeSorterInit($3681,$3684,$3685)|0);
        HEAP32[$rc>>2] = $3686;
        break;
       }
       case 59:  {
        $3687 = HEAP32[$pOp>>2]|0;
        $3688 = (($3687) + 4|0);
        $3689 = HEAP32[$3688>>2]|0;
        $3690 = HEAP32[$0>>2]|0;
        $3691 = (($3690) + 56|0);
        $3692 = HEAP32[$3691>>2]|0;
        $3693 = (($3692) + ($3689<<2)|0);
        $3694 = HEAP32[$3693>>2]|0;
        HEAP32[$pC27>>2] = $3694;
        $3695 = HEAP32[$pC27>>2]|0;
        $3696 = (($3695) + 40|0);
        $3697 = $3696;
        $3698 = $3697;
        $3699 = HEAP32[$3698>>2]|0;
        $3700 = (($3697) + 4)|0;
        $3701 = $3700;
        $3702 = HEAP32[$3701>>2]|0;
        $3703 = (_i64Add(($3699|0),($3702|0),1,0)|0);
        $3704 = tempRet0;
        $3705 = $3696;
        $3706 = $3705;
        HEAP32[$3706>>2] = $3703;
        $3707 = (($3705) + 4)|0;
        $3708 = $3707;
        HEAP32[$3708>>2] = $3704;
        $3709 = ($3699|0)==(0);
        $3710 = ($3702|0)==(0);
        $3711 = $3709 & $3710;
        if ($3711) {
         $3712 = HEAP32[$pOp>>2]|0;
         $3713 = (($3712) + 8|0);
         $3714 = HEAP32[$3713>>2]|0;
         $3715 = (($3714) - 1)|0;
         HEAP32[$pc>>2] = $3715;
        }
        break;
       }
       case 60:  {
        $3716 = HEAP32[$0>>2]|0;
        $3717 = HEAP32[$pOp>>2]|0;
        $3718 = (($3717) + 4|0);
        $3719 = HEAP32[$3718>>2]|0;
        $3720 = HEAP32[$pOp>>2]|0;
        $3721 = (($3720) + 12|0);
        $3722 = HEAP32[$3721>>2]|0;
        $3723 = (_allocateCursor($3716,$3719,$3722,-1,0)|0);
        HEAP32[$pCx28>>2] = $3723;
        $3724 = HEAP32[$pCx28>>2]|0;
        $3725 = ($3724|0)==(0|0);
        if ($3725) {
         label = 871;
         break L1;
        }
        $3726 = HEAP32[$pCx28>>2]|0;
        $3727 = (($3726) + 25|0);
        HEAP8[$3727>>0] = 1;
        $3728 = HEAP32[$pOp>>2]|0;
        $3729 = (($3728) + 8|0);
        $3730 = HEAP32[$3729>>2]|0;
        $3731 = HEAP32[$pCx28>>2]|0;
        $3732 = (($3731) + 16|0);
        HEAP32[$3732>>2] = $3730;
        $3733 = HEAP32[$pCx28>>2]|0;
        $3734 = (($3733) + 27|0);
        $3735 = HEAP8[$3734>>0]|0;
        $3736 = $3735 & -5;
        $3737 = $3736 | 4;
        HEAP8[$3734>>0] = $3737;
        break;
       }
       case 61:  {
        $3738 = HEAP32[$0>>2]|0;
        $3739 = HEAP32[$pOp>>2]|0;
        $3740 = (($3739) + 4|0);
        $3741 = HEAP32[$3740>>2]|0;
        $3742 = HEAP32[$0>>2]|0;
        $3743 = (($3742) + 56|0);
        $3744 = HEAP32[$3743>>2]|0;
        $3745 = (($3744) + ($3741<<2)|0);
        $3746 = HEAP32[$3745>>2]|0;
        _sqlite3VdbeFreeCursor($3738,$3746);
        $3747 = HEAP32[$pOp>>2]|0;
        $3748 = (($3747) + 4|0);
        $3749 = HEAP32[$3748>>2]|0;
        $3750 = HEAP32[$0>>2]|0;
        $3751 = (($3750) + 56|0);
        $3752 = HEAP32[$3751>>2]|0;
        $3753 = (($3752) + ($3749<<2)|0);
        HEAP32[$3753>>2] = 0;
        break;
       }
       case 65: case 64: case 63: case 62:  {
        $3754 = HEAP32[$pOp>>2]|0;
        $3755 = (($3754) + 4|0);
        $3756 = HEAP32[$3755>>2]|0;
        $3757 = HEAP32[$0>>2]|0;
        $3758 = (($3757) + 56|0);
        $3759 = HEAP32[$3758>>2]|0;
        $3760 = (($3759) + ($3756<<2)|0);
        $3761 = HEAP32[$3760>>2]|0;
        HEAP32[$pC30>>2] = $3761;
        $3762 = HEAP32[$pOp>>2]|0;
        $3763 = HEAP8[$3762>>0]|0;
        $3764 = $3763&255;
        HEAP32[$oc>>2] = $3764;
        $3765 = HEAP32[$pC30>>2]|0;
        $3766 = (($3765) + 25|0);
        HEAP8[$3766>>0] = 0;
        $3767 = HEAP32[$pC30>>2]|0;
        $3768 = (($3767) + 27|0);
        $3769 = HEAP8[$3768>>0]|0;
        $3770 = ($3769&255) >>> 2;
        $3771 = $3770 & 1;
        $3772 = $3771&255;
        $3773 = ($3772|0)!=(0);
        $3774 = HEAP32[$pOp>>2]|0;
        if ($3773) {
         $3775 = (($3774) + 12|0);
         $3776 = HEAP32[$3775>>2]|0;
         $3777 = HEAP32[$aMem>>2]|0;
         $3778 = (($3777) + (($3776*40)|0)|0);
         HEAP32[$pIn3>>2] = $3778;
         $3779 = HEAP32[$pIn3>>2]|0;
         $3780 = (($3779) + 8|0);
         $3781 = HEAP16[$3780>>1]|0;
         $3782 = $3781&65535;
         $3783 = $3782 & 14;
         $3784 = ($3783|0)==(2);
         if ($3784) {
          $3785 = HEAP32[$pIn3>>2]|0;
          _applyNumericAffinity($3785,0);
         }
         $3786 = HEAP32[$pIn3>>2]|0;
         $3787 = (_sqlite3VdbeIntValue($3786)|0);
         $3788 = tempRet0;
         $3789 = $iKey;
         $3790 = $3789;
         HEAP32[$3790>>2] = $3787;
         $3791 = (($3789) + 4)|0;
         $3792 = $3791;
         HEAP32[$3792>>2] = $3788;
         $3793 = HEAP32[$pIn3>>2]|0;
         $3794 = (($3793) + 8|0);
         $3795 = HEAP16[$3794>>1]|0;
         $3796 = $3795&65535;
         $3797 = $3796 & 4;
         $3798 = ($3797|0)==(0);
         do {
          if ($3798) {
           $3799 = HEAP32[$pIn3>>2]|0;
           $3800 = (($3799) + 8|0);
           $3801 = HEAP16[$3800>>1]|0;
           $3802 = $3801&65535;
           $3803 = $3802 & 8;
           $3804 = ($3803|0)==(0);
           if ($3804) {
            $3805 = HEAP32[$pOp>>2]|0;
            $3806 = (($3805) + 8|0);
            $3807 = HEAP32[$3806>>2]|0;
            $3808 = (($3807) - 1)|0;
            HEAP32[$pc>>2] = $3808;
            break L21;
           }
           $3809 = HEAP32[$pIn3>>2]|0;
           $3810 = +HEAPF64[$3809>>3];
           $3811 = $iKey;
           $3812 = $3811;
           $3813 = HEAP32[$3812>>2]|0;
           $3814 = (($3811) + 4)|0;
           $3815 = $3814;
           $3816 = HEAP32[$3815>>2]|0;
           $3817 = (+($3813>>>0)) + (4294967296.0*(+($3816|0)));
           $3818 = $3810 < $3817;
           if ($3818) {
            $3819 = HEAP32[$oc>>2]|0;
            $3820 = $3819 & 1;
            $3821 = ($3820|0)==(1);
            if (!($3821)) {
             break;
            }
            $3822 = HEAP32[$oc>>2]|0;
            $3823 = (($3822) + -1)|0;
            HEAP32[$oc>>2] = $3823;
            break;
           }
           $3824 = HEAP32[$pIn3>>2]|0;
           $3825 = +HEAPF64[$3824>>3];
           $3826 = $iKey;
           $3827 = $3826;
           $3828 = HEAP32[$3827>>2]|0;
           $3829 = (($3826) + 4)|0;
           $3830 = $3829;
           $3831 = HEAP32[$3830>>2]|0;
           $3832 = (+($3828>>>0)) + (4294967296.0*(+($3831|0)));
           $3833 = $3825 > $3832;
           if ($3833) {
            $3834 = HEAP32[$oc>>2]|0;
            $3835 = $3834 & 1;
            $3836 = ($3835|0)==(0);
            if ($3836) {
             $3837 = HEAP32[$oc>>2]|0;
             $3838 = (($3837) + 1)|0;
             HEAP32[$oc>>2] = $3838;
            }
           }
          }
         } while(0);
         $3839 = HEAP32[$pC30>>2]|0;
         $3840 = HEAP32[$3839>>2]|0;
         $3841 = $iKey;
         $3842 = $3841;
         $3843 = HEAP32[$3842>>2]|0;
         $3844 = (($3841) + 4)|0;
         $3845 = $3844;
         $3846 = HEAP32[$3845>>2]|0;
         $3847 = (_sqlite3BtreeMovetoUnpacked($3840,0,$3843,$3846,0,$res29)|0);
         HEAP32[$rc>>2] = $3847;
         $3848 = $iKey;
         $3849 = $3848;
         $3850 = HEAP32[$3849>>2]|0;
         $3851 = (($3848) + 4)|0;
         $3852 = $3851;
         $3853 = HEAP32[$3852>>2]|0;
         $3854 = HEAP32[$pC30>>2]|0;
         $3855 = (($3854) + 48|0);
         $3856 = $3855;
         $3857 = $3856;
         HEAP32[$3857>>2] = $3850;
         $3858 = (($3856) + 4)|0;
         $3859 = $3858;
         HEAP32[$3859>>2] = $3853;
         $3860 = HEAP32[$rc>>2]|0;
         $3861 = ($3860|0)!=(0);
         if ($3861) {
          break L11;
         }
        } else {
         $3862 = (($3774) + 16|0);
         $3863 = HEAP32[$3862>>2]|0;
         HEAP32[$nField31>>2] = $3863;
         $3864 = HEAP32[$pC30>>2]|0;
         $3865 = (($3864) + 8|0);
         $3866 = HEAP32[$3865>>2]|0;
         HEAP32[$r>>2] = $3866;
         $3867 = HEAP32[$nField31>>2]|0;
         $3868 = $3867&65535;
         $3869 = (($r) + 4|0);
         HEAP16[$3869>>1] = $3868;
         $3870 = HEAP32[$oc>>2]|0;
         $3871 = (($3870) - 62)|0;
         $3872 = 1 & $3871;
         $3873 = ($3872|0)!=(0);
         $3874 = $3873 ? -1 : 1;
         $3875 = $3874&255;
         $3876 = (($r) + 6|0);
         HEAP8[$3876>>0] = $3875;
         $3877 = HEAP32[$pOp>>2]|0;
         $3878 = (($3877) + 12|0);
         $3879 = HEAP32[$3878>>2]|0;
         $3880 = HEAP32[$aMem>>2]|0;
         $3881 = (($3880) + (($3879*40)|0)|0);
         $3882 = (($r) + 8|0);
         HEAP32[$3882>>2] = $3881;
         $3883 = (($r) + 8|0);
         $3884 = HEAP32[$3883>>2]|0;
         $3885 = (($3884) + 8|0);
         $3886 = HEAP16[$3885>>1]|0;
         $3887 = $3886&65535;
         $3888 = $3887 & 16384;
         $3889 = ($3888|0)!=(0);
         if ($3889) {
          $3890 = (($r) + 8|0);
          $3891 = HEAP32[$3890>>2]|0;
          (_sqlite3VdbeMemExpandBlob($3891)|0);
         }
         $3892 = HEAP32[$pC30>>2]|0;
         $3893 = HEAP32[$3892>>2]|0;
         $3894 = (_sqlite3BtreeMovetoUnpacked($3893,$r,0,0,0,$res29)|0);
         HEAP32[$rc>>2] = $3894;
         $3895 = HEAP32[$rc>>2]|0;
         $3896 = ($3895|0)!=(0);
         if ($3896) {
          break L11;
         }
        }
        $3897 = HEAP32[$pC30>>2]|0;
        $3898 = (($3897) + 26|0);
        HEAP8[$3898>>0] = 0;
        $3899 = HEAP32[$pC30>>2]|0;
        $3900 = (($3899) + 60|0);
        HEAP32[$3900>>2] = 0;
        $3901 = HEAP32[$oc>>2]|0;
        $3902 = ($3901|0)>=(64);
        $3903 = HEAP32[$res29>>2]|0;
        do {
         if ($3902) {
          $3904 = ($3903|0)<(0);
          if (!($3904)) {
           $3905 = HEAP32[$res29>>2]|0;
           $3906 = ($3905|0)==(0);
           $3907 = HEAP32[$oc>>2]|0;
           $3908 = ($3907|0)==(65);
           $or$cond11 = $3906 & $3908;
           if (!($or$cond11)) {
            HEAP32[$res29>>2] = 0;
            break;
           }
          }
          HEAP32[$res29>>2] = 0;
          $3909 = HEAP32[$pC30>>2]|0;
          $3910 = HEAP32[$3909>>2]|0;
          $3911 = (_sqlite3BtreeNext($3910,$res29)|0);
          HEAP32[$rc>>2] = $3911;
          $3912 = HEAP32[$rc>>2]|0;
          $3913 = ($3912|0)!=(0);
          if ($3913) {
           break L11;
          }
         } else {
          $3914 = ($3903|0)>(0);
          if (!($3914)) {
           $3915 = HEAP32[$res29>>2]|0;
           $3916 = ($3915|0)==(0);
           $3917 = HEAP32[$oc>>2]|0;
           $3918 = ($3917|0)==(62);
           $or$cond13 = $3916 & $3918;
           if (!($or$cond13)) {
            $3924 = HEAP32[$pC30>>2]|0;
            $3925 = HEAP32[$3924>>2]|0;
            $3926 = (_sqlite3BtreeEof($3925)|0);
            HEAP32[$res29>>2] = $3926;
            break;
           }
          }
          HEAP32[$res29>>2] = 0;
          $3919 = HEAP32[$pC30>>2]|0;
          $3920 = HEAP32[$3919>>2]|0;
          $3921 = (_sqlite3BtreePrevious($3920,$res29)|0);
          HEAP32[$rc>>2] = $3921;
          $3922 = HEAP32[$rc>>2]|0;
          $3923 = ($3922|0)!=(0);
          if ($3923) {
           break L11;
          }
         }
        } while(0);
        $3927 = HEAP32[$res29>>2]|0;
        $3928 = ($3927|0)!=(0);
        if ($3928) {
         $3929 = HEAP32[$pOp>>2]|0;
         $3930 = (($3929) + 8|0);
         $3931 = HEAP32[$3930>>2]|0;
         $3932 = (($3931) - 1)|0;
         HEAP32[$pc>>2] = $3932;
        }
        break;
       }
       case 66:  {
        $3933 = HEAP32[$pOp>>2]|0;
        $3934 = (($3933) + 4|0);
        $3935 = HEAP32[$3934>>2]|0;
        $3936 = HEAP32[$0>>2]|0;
        $3937 = (($3936) + 56|0);
        $3938 = HEAP32[$3937>>2]|0;
        $3939 = (($3938) + ($3935<<2)|0);
        $3940 = HEAP32[$3939>>2]|0;
        HEAP32[$pC32>>2] = $3940;
        $3941 = HEAP32[$pC32>>2]|0;
        $3942 = (($3941) + 25|0);
        HEAP8[$3942>>0] = 0;
        $3943 = HEAP32[$pOp>>2]|0;
        $3944 = (($3943) + 8|0);
        $3945 = HEAP32[$3944>>2]|0;
        $3946 = HEAP32[$aMem>>2]|0;
        $3947 = (($3946) + (($3945*40)|0)|0);
        HEAP32[$pIn2>>2] = $3947;
        $3948 = HEAP32[$pIn2>>2]|0;
        $3949 = (_sqlite3VdbeIntValue($3948)|0);
        $3950 = tempRet0;
        $3951 = HEAP32[$pC32>>2]|0;
        $3952 = (($3951) + 48|0);
        $3953 = $3952;
        $3954 = $3953;
        HEAP32[$3954>>2] = $3949;
        $3955 = (($3953) + 4)|0;
        $3956 = $3955;
        HEAP32[$3956>>2] = $3950;
        $3957 = HEAP32[$pC32>>2]|0;
        $3958 = (($3957) + 26|0);
        HEAP8[$3958>>0] = 1;
        break;
       }
       case 69: case 68: case 67:  {
        $3959 = HEAP32[$pOp>>2]|0;
        $3960 = (($3959) + 4|0);
        $3961 = HEAP32[$3960>>2]|0;
        $3962 = HEAP32[$0>>2]|0;
        $3963 = (($3962) + 56|0);
        $3964 = HEAP32[$3963>>2]|0;
        $3965 = (($3964) + ($3961<<2)|0);
        $3966 = HEAP32[$3965>>2]|0;
        HEAP32[$pC34>>2] = $3966;
        $3967 = HEAP32[$pOp>>2]|0;
        $3968 = (($3967) + 12|0);
        $3969 = HEAP32[$3968>>2]|0;
        $3970 = HEAP32[$aMem>>2]|0;
        $3971 = (($3970) + (($3969*40)|0)|0);
        HEAP32[$pIn3>>2] = $3971;
        HEAP32[$pFree>>2] = 0;
        $3972 = HEAP32[$pOp>>2]|0;
        $3973 = (($3972) + 16|0);
        $3974 = HEAP32[$3973>>2]|0;
        $3975 = ($3974|0)>(0);
        $3976 = HEAP32[$pC34>>2]|0;
        $3977 = (($3976) + 8|0);
        $3978 = HEAP32[$3977>>2]|0;
        if ($3975) {
         HEAP32[$r36>>2] = $3978;
         $3979 = HEAP32[$pOp>>2]|0;
         $3980 = (($3979) + 16|0);
         $3981 = HEAP32[$3980>>2]|0;
         $3982 = $3981&65535;
         $3983 = (($r36) + 4|0);
         HEAP16[$3983>>1] = $3982;
         $3984 = HEAP32[$pIn3>>2]|0;
         $3985 = (($r36) + 8|0);
         HEAP32[$3985>>2] = $3984;
         HEAP32[$ii33>>2] = 0;
         while(1) {
          $3986 = HEAP32[$ii33>>2]|0;
          $3987 = (($r36) + 4|0);
          $3988 = HEAP16[$3987>>1]|0;
          $3989 = $3988&65535;
          $3990 = ($3986|0)<($3989|0);
          if (!($3990)) {
           break;
          }
          $3991 = HEAP32[$ii33>>2]|0;
          $3992 = (($r36) + 8|0);
          $3993 = HEAP32[$3992>>2]|0;
          $3994 = (($3993) + (($3991*40)|0)|0);
          $3995 = (($3994) + 8|0);
          $3996 = HEAP16[$3995>>1]|0;
          $3997 = $3996&65535;
          $3998 = $3997 & 16384;
          $3999 = ($3998|0)!=(0);
          if ($3999) {
           $4000 = HEAP32[$ii33>>2]|0;
           $4001 = (($r36) + 8|0);
           $4002 = HEAP32[$4001>>2]|0;
           $4003 = (($4002) + (($4000*40)|0)|0);
           (_sqlite3VdbeMemExpandBlob($4003)|0);
          }
          $4004 = HEAP32[$ii33>>2]|0;
          $4005 = (($4004) + 1)|0;
          HEAP32[$ii33>>2] = $4005;
         }
         HEAP32[$pIdxKey>>2] = $r36;
        } else {
         $4006 = (_sqlite3VdbeAllocUnpackedRecord($3978,$aTempRec,191,$pFree)|0);
         HEAP32[$pIdxKey>>2] = $4006;
         $4007 = HEAP32[$pIdxKey>>2]|0;
         $4008 = ($4007|0)==(0|0);
         if ($4008) {
          label = 871;
          break L1;
         }
         $4009 = HEAP32[$pC34>>2]|0;
         $4010 = (($4009) + 8|0);
         $4011 = HEAP32[$4010>>2]|0;
         $4012 = HEAP32[$pIn3>>2]|0;
         $4013 = (($4012) + 12|0);
         $4014 = HEAP32[$4013>>2]|0;
         $4015 = HEAP32[$pIn3>>2]|0;
         $4016 = (($4015) + 16|0);
         $4017 = HEAP32[$4016>>2]|0;
         $4018 = HEAP32[$pIdxKey>>2]|0;
         _sqlite3VdbeRecordUnpack($4011,$4014,$4017,$4018);
        }
        $4019 = HEAP32[$pIdxKey>>2]|0;
        $4020 = (($4019) + 6|0);
        HEAP8[$4020>>0] = 0;
        $4021 = HEAP32[$pOp>>2]|0;
        $4022 = HEAP8[$4021>>0]|0;
        $4023 = $4022&255;
        $4024 = ($4023|0)==(67);
        L603: do {
         if ($4024) {
          HEAP32[$ii33>>2] = 0;
          while(1) {
           $4025 = HEAP32[$ii33>>2]|0;
           $4026 = (($r36) + 4|0);
           $4027 = HEAP16[$4026>>1]|0;
           $4028 = $4027&65535;
           $4029 = ($4025|0)<($4028|0);
           if (!($4029)) {
            break L603;
           }
           $4030 = HEAP32[$ii33>>2]|0;
           $4031 = (($r36) + 8|0);
           $4032 = HEAP32[$4031>>2]|0;
           $4033 = (($4032) + (($4030*40)|0)|0);
           $4034 = (($4033) + 8|0);
           $4035 = HEAP16[$4034>>1]|0;
           $4036 = $4035&65535;
           $4037 = $4036 & 1;
           $4038 = ($4037|0)!=(0);
           if ($4038) {
            break;
           }
           $4043 = HEAP32[$ii33>>2]|0;
           $4044 = (($4043) + 1)|0;
           HEAP32[$ii33>>2] = $4044;
          }
          $4039 = HEAP32[$pOp>>2]|0;
          $4040 = (($4039) + 8|0);
          $4041 = HEAP32[$4040>>2]|0;
          $4042 = (($4041) - 1)|0;
          HEAP32[$pc>>2] = $4042;
         }
        } while(0);
        $4045 = HEAP32[$pC34>>2]|0;
        $4046 = HEAP32[$4045>>2]|0;
        $4047 = HEAP32[$pIdxKey>>2]|0;
        $4048 = (_sqlite3BtreeMovetoUnpacked($4046,$4047,0,0,0,$res35)|0);
        HEAP32[$rc>>2] = $4048;
        $4049 = HEAP32[$pOp>>2]|0;
        $4050 = (($4049) + 16|0);
        $4051 = HEAP32[$4050>>2]|0;
        $4052 = ($4051|0)==(0);
        if ($4052) {
         $4053 = HEAP32[$db>>2]|0;
         $4054 = HEAP32[$pFree>>2]|0;
         _sqlite3DbFree($4053,$4054);
        }
        $4055 = HEAP32[$rc>>2]|0;
        $4056 = ($4055|0)!=(0);
        if (!($4056)) {
         $4057 = HEAP32[$res35>>2]|0;
         $4058 = HEAP32[$pC34>>2]|0;
         $4059 = (($4058) + 12|0);
         HEAP32[$4059>>2] = $4057;
         $4060 = HEAP32[$res35>>2]|0;
         $4061 = ($4060|0)==(0);
         $4062 = $4061&1;
         HEAP32[$alreadyExists>>2] = $4062;
         $4063 = HEAP32[$alreadyExists>>2]|0;
         $4064 = (1 - ($4063))|0;
         $4065 = $4064&255;
         $4066 = HEAP32[$pC34>>2]|0;
         $4067 = (($4066) + 25|0);
         HEAP8[$4067>>0] = $4065;
         $4068 = HEAP32[$pC34>>2]|0;
         $4069 = (($4068) + 26|0);
         HEAP8[$4069>>0] = 0;
         $4070 = HEAP32[$pC34>>2]|0;
         $4071 = (($4070) + 60|0);
         HEAP32[$4071>>2] = 0;
         $4072 = HEAP32[$pOp>>2]|0;
         $4073 = HEAP8[$4072>>0]|0;
         $4074 = $4073&255;
         $4075 = ($4074|0)==(69);
         $4076 = HEAP32[$alreadyExists>>2]|0;
         $4077 = ($4076|0)!=(0);
         if ($4075) {
          if (!($4077)) {
           break L21;
          }
          $4078 = HEAP32[$pOp>>2]|0;
          $4079 = (($4078) + 8|0);
          $4080 = HEAP32[$4079>>2]|0;
          $4081 = (($4080) - 1)|0;
          HEAP32[$pc>>2] = $4081;
          break L21;
         } else {
          if ($4077) {
           break L21;
          }
          $4082 = HEAP32[$pOp>>2]|0;
          $4083 = (($4082) + 8|0);
          $4084 = HEAP32[$4083>>2]|0;
          $4085 = (($4084) - 1)|0;
          HEAP32[$pc>>2] = $4085;
          break L21;
         }
        }
        break;
       }
       case 73:  {
        $4147 = HEAP32[$pOp>>2]|0;
        $4148 = (($4147) + 4|0);
        $4149 = HEAP32[$4148>>2]|0;
        $4150 = HEAP32[$0>>2]|0;
        $4151 = (($4150) + 56|0);
        $4152 = HEAP32[$4151>>2]|0;
        $4153 = (($4152) + ($4149<<2)|0);
        $4154 = HEAP32[$4153>>2]|0;
        $4155 = (($4154) + 40|0);
        $4156 = $4155;
        $4157 = $4156;
        $4158 = HEAP32[$4157>>2]|0;
        $4159 = (($4156) + 4)|0;
        $4160 = $4159;
        $4161 = HEAP32[$4160>>2]|0;
        $4162 = (_i64Add(($4158|0),($4161|0),1,0)|0);
        $4163 = tempRet0;
        $4164 = $4155;
        $4165 = $4164;
        HEAP32[$4165>>2] = $4162;
        $4166 = (($4164) + 4)|0;
        $4167 = $4166;
        HEAP32[$4167>>2] = $4163;
        $4168 = HEAP32[$pOut>>2]|0;
        $4169 = $4168;
        $4170 = $4169;
        HEAP32[$4170>>2] = $4158;
        $4171 = (($4169) + 4)|0;
        $4172 = $4171;
        HEAP32[$4172>>2] = $4161;
        break;
       }
       case 70:  {
        $4086 = HEAP32[$pOp>>2]|0;
        $4087 = (($4086) + 12|0);
        $4088 = HEAP32[$4087>>2]|0;
        $4089 = HEAP32[$aMem>>2]|0;
        $4090 = (($4089) + (($4088*40)|0)|0);
        HEAP32[$pIn3>>2] = $4090;
        $4091 = HEAP32[$pOp>>2]|0;
        $4092 = (($4091) + 4|0);
        $4093 = HEAP32[$4092>>2]|0;
        $4094 = HEAP32[$0>>2]|0;
        $4095 = (($4094) + 56|0);
        $4096 = HEAP32[$4095>>2]|0;
        $4097 = (($4096) + ($4093<<2)|0);
        $4098 = HEAP32[$4097>>2]|0;
        HEAP32[$pC37>>2] = $4098;
        $4099 = HEAP32[$pC37>>2]|0;
        $4100 = HEAP32[$4099>>2]|0;
        HEAP32[$pCrsr38>>2] = $4100;
        HEAP32[$res39>>2] = 0;
        $4101 = HEAP32[$pIn3>>2]|0;
        $4102 = $4101;
        $4103 = $4102;
        $4104 = HEAP32[$4103>>2]|0;
        $4105 = (($4102) + 4)|0;
        $4106 = $4105;
        $4107 = HEAP32[$4106>>2]|0;
        $4108 = $iKey40;
        $4109 = $4108;
        HEAP32[$4109>>2] = $4104;
        $4110 = (($4108) + 4)|0;
        $4111 = $4110;
        HEAP32[$4111>>2] = $4107;
        $4112 = HEAP32[$pCrsr38>>2]|0;
        $4113 = $iKey40;
        $4114 = $4113;
        $4115 = HEAP32[$4114>>2]|0;
        $4116 = (($4113) + 4)|0;
        $4117 = $4116;
        $4118 = HEAP32[$4117>>2]|0;
        $4119 = (_sqlite3BtreeMovetoUnpacked($4112,0,$4115,$4118,0,$res39)|0);
        HEAP32[$rc>>2] = $4119;
        $4120 = $iKey40;
        $4121 = $4120;
        $4122 = HEAP32[$4121>>2]|0;
        $4123 = (($4120) + 4)|0;
        $4124 = $4123;
        $4125 = HEAP32[$4124>>2]|0;
        $4126 = HEAP32[$pC37>>2]|0;
        $4127 = (($4126) + 48|0);
        $4128 = $4127;
        $4129 = $4128;
        HEAP32[$4129>>2] = $4122;
        $4130 = (($4128) + 4)|0;
        $4131 = $4130;
        HEAP32[$4131>>2] = $4125;
        $4132 = HEAP32[$pC37>>2]|0;
        $4133 = (($4132) + 25|0);
        HEAP8[$4133>>0] = 0;
        $4134 = HEAP32[$pC37>>2]|0;
        $4135 = (($4134) + 60|0);
        HEAP32[$4135>>2] = 0;
        $4136 = HEAP32[$pC37>>2]|0;
        $4137 = (($4136) + 26|0);
        HEAP8[$4137>>0] = 0;
        $4138 = HEAP32[$res39>>2]|0;
        $4139 = ($4138|0)!=(0);
        if ($4139) {
         $4140 = HEAP32[$pOp>>2]|0;
         $4141 = (($4140) + 8|0);
         $4142 = HEAP32[$4141>>2]|0;
         $4143 = (($4142) - 1)|0;
         HEAP32[$pc>>2] = $4143;
        }
        $4144 = HEAP32[$res39>>2]|0;
        $4145 = HEAP32[$pC37>>2]|0;
        $4146 = (($4145) + 12|0);
        HEAP32[$4146>>2] = $4144;
        break;
       }
       case 74:  {
        $4173 = $v;
        $4174 = $4173;
        HEAP32[$4174>>2] = 0;
        $4175 = (($4173) + 4)|0;
        $4176 = $4175;
        HEAP32[$4176>>2] = 0;
        HEAP32[$res42>>2] = 0;
        $4177 = HEAP32[$pOp>>2]|0;
        $4178 = (($4177) + 4|0);
        $4179 = HEAP32[$4178>>2]|0;
        $4180 = HEAP32[$0>>2]|0;
        $4181 = (($4180) + 56|0);
        $4182 = HEAP32[$4181>>2]|0;
        $4183 = (($4182) + ($4179<<2)|0);
        $4184 = HEAP32[$4183>>2]|0;
        HEAP32[$pC41>>2] = $4184;
        $4185 = HEAP32[$pC41>>2]|0;
        $4186 = HEAP32[$4185>>2]|0;
        $4187 = ($4186|0)==(0|0);
        if (!($4187)) {
         $4188 = HEAP32[$pC41>>2]|0;
         $4189 = (($4188) + 27|0);
         $4190 = HEAP8[$4189>>0]|0;
         $4191 = ($4190&255) >>> 1;
         $4192 = $4191 & 1;
         $4193 = $4192&255;
         $4194 = ($4193|0)!=(0);
         do {
          if (!($4194)) {
           $4195 = HEAP32[$pC41>>2]|0;
           $4196 = HEAP32[$4195>>2]|0;
           $4197 = (_sqlite3BtreeLast($4196,$res42)|0);
           HEAP32[$rc>>2] = $4197;
           $4198 = HEAP32[$rc>>2]|0;
           $4199 = ($4198|0)!=(0);
           if ($4199) {
            break L11;
           }
           $4200 = HEAP32[$res42>>2]|0;
           $4201 = ($4200|0)!=(0);
           if ($4201) {
            $4202 = $v;
            $4203 = $4202;
            HEAP32[$4203>>2] = 1;
            $4204 = (($4202) + 4)|0;
            $4205 = $4204;
            HEAP32[$4205>>2] = 0;
            break;
           }
           $4206 = HEAP32[$pC41>>2]|0;
           $4207 = HEAP32[$4206>>2]|0;
           $4208 = (_sqlite3BtreeKeySize($4207,$v)|0);
           HEAP32[$rc>>2] = $4208;
           $4209 = $v;
           $4210 = $4209;
           $4211 = HEAP32[$4210>>2]|0;
           $4212 = (($4209) + 4)|0;
           $4213 = $4212;
           $4214 = HEAP32[$4213>>2]|0;
           $4215 = ($4214|0)>(2147483647);
           $4216 = ($4211>>>0)>=(4294967295);
           $4217 = ($4214|0)==(2147483647);
           $4218 = $4217 & $4216;
           $4219 = $4215 | $4218;
           if ($4219) {
            $4220 = HEAP32[$pC41>>2]|0;
            $4221 = (($4220) + 27|0);
            $4222 = HEAP8[$4221>>0]|0;
            $4223 = $4222 & -3;
            $4224 = $4223 | 2;
            HEAP8[$4221>>0] = $4224;
            break;
           } else {
            $4225 = $v;
            $4226 = $4225;
            $4227 = HEAP32[$4226>>2]|0;
            $4228 = (($4225) + 4)|0;
            $4229 = $4228;
            $4230 = HEAP32[$4229>>2]|0;
            $4231 = (_i64Add(($4227|0),($4230|0),1,0)|0);
            $4232 = tempRet0;
            $4233 = $v;
            $4234 = $4233;
            HEAP32[$4234>>2] = $4231;
            $4235 = (($4233) + 4)|0;
            $4236 = $4235;
            HEAP32[$4236>>2] = $4232;
            break;
           }
          }
         } while(0);
         $4237 = HEAP32[$pOp>>2]|0;
         $4238 = (($4237) + 12|0);
         $4239 = HEAP32[$4238>>2]|0;
         $4240 = ($4239|0)!=(0);
         if ($4240) {
          $4241 = HEAP32[$0>>2]|0;
          $4242 = (($4241) + 176|0);
          $4243 = HEAP32[$4242>>2]|0;
          $4244 = ($4243|0)!=(0|0);
          if ($4244) {
           $4245 = HEAP32[$0>>2]|0;
           $4246 = (($4245) + 176|0);
           $4247 = HEAP32[$4246>>2]|0;
           HEAP32[$pFrame45>>2] = $4247;
           while(1) {
            $4248 = HEAP32[$pFrame45>>2]|0;
            $4249 = (($4248) + 4|0);
            $4250 = HEAP32[$4249>>2]|0;
            $4251 = ($4250|0)!=(0|0);
            if (!($4251)) {
             break;
            }
            $4252 = HEAP32[$pFrame45>>2]|0;
            $4253 = (($4252) + 4|0);
            $4254 = HEAP32[$4253>>2]|0;
            HEAP32[$pFrame45>>2] = $4254;
           }
           $4255 = HEAP32[$pOp>>2]|0;
           $4256 = (($4255) + 12|0);
           $4257 = HEAP32[$4256>>2]|0;
           $4258 = HEAP32[$pFrame45>>2]|0;
           $4259 = (($4258) + 12|0);
           $4260 = HEAP32[$4259>>2]|0;
           $4261 = (($4260) + (($4257*40)|0)|0);
           HEAP32[$pMem44>>2] = $4261;
          } else {
           $4262 = HEAP32[$pOp>>2]|0;
           $4263 = (($4262) + 12|0);
           $4264 = HEAP32[$4263>>2]|0;
           $4265 = HEAP32[$aMem>>2]|0;
           $4266 = (($4265) + (($4264*40)|0)|0);
           HEAP32[$pMem44>>2] = $4266;
          }
          $4267 = HEAP32[$pMem44>>2]|0;
          (_sqlite3VdbeMemIntegerify($4267)|0);
          $4268 = HEAP32[$pMem44>>2]|0;
          $4269 = $4268;
          $4270 = $4269;
          $4271 = HEAP32[$4270>>2]|0;
          $4272 = (($4269) + 4)|0;
          $4273 = $4272;
          $4274 = HEAP32[$4273>>2]|0;
          $4275 = ($4271|0)==(-1);
          $4276 = ($4274|0)==(2147483647);
          $4277 = $4275 & $4276;
          if ($4277) {
           label = 557;
           break L11;
          }
          $4278 = HEAP32[$pC41>>2]|0;
          $4279 = (($4278) + 27|0);
          $4280 = HEAP8[$4279>>0]|0;
          $4281 = ($4280&255) >>> 1;
          $4282 = $4281 & 1;
          $4283 = $4282&255;
          $4284 = ($4283|0)!=(0);
          if ($4284) {
           label = 557;
           break L11;
          }
          $4285 = $v;
          $4286 = $4285;
          $4287 = HEAP32[$4286>>2]|0;
          $4288 = (($4285) + 4)|0;
          $4289 = $4288;
          $4290 = HEAP32[$4289>>2]|0;
          $4291 = HEAP32[$pMem44>>2]|0;
          $4292 = $4291;
          $4293 = $4292;
          $4294 = HEAP32[$4293>>2]|0;
          $4295 = (($4292) + 4)|0;
          $4296 = $4295;
          $4297 = HEAP32[$4296>>2]|0;
          $4298 = (_i64Add(($4294|0),($4297|0),1,0)|0);
          $4299 = tempRet0;
          $4300 = ($4290|0)<($4299|0);
          $4301 = ($4287>>>0)<($4298>>>0);
          $4302 = ($4290|0)==($4299|0);
          $4303 = $4302 & $4301;
          $4304 = $4300 | $4303;
          if ($4304) {
           $4305 = HEAP32[$pMem44>>2]|0;
           $4306 = $4305;
           $4307 = $4306;
           $4308 = HEAP32[$4307>>2]|0;
           $4309 = (($4306) + 4)|0;
           $4310 = $4309;
           $4311 = HEAP32[$4310>>2]|0;
           $4312 = (_i64Add(($4308|0),($4311|0),1,0)|0);
           $4313 = tempRet0;
           $4314 = $v;
           $4315 = $4314;
           HEAP32[$4315>>2] = $4312;
           $4316 = (($4314) + 4)|0;
           $4317 = $4316;
           HEAP32[$4317>>2] = $4313;
          }
          $4318 = $v;
          $4319 = $4318;
          $4320 = HEAP32[$4319>>2]|0;
          $4321 = (($4318) + 4)|0;
          $4322 = $4321;
          $4323 = HEAP32[$4322>>2]|0;
          $4324 = HEAP32[$pMem44>>2]|0;
          $4325 = $4324;
          $4326 = $4325;
          HEAP32[$4326>>2] = $4320;
          $4327 = (($4325) + 4)|0;
          $4328 = $4327;
          HEAP32[$4328>>2] = $4323;
         }
         $4329 = HEAP32[$pC41>>2]|0;
         $4330 = (($4329) + 27|0);
         $4331 = HEAP8[$4330>>0]|0;
         $4332 = ($4331&255) >>> 1;
         $4333 = $4332 & 1;
         $4334 = $4333&255;
         $4335 = ($4334|0)!=(0);
         if ($4335) {
          HEAP32[$cnt43>>2] = 0;
          while(1) {
           _sqlite3_randomness(8,$v);
           $4336 = $v;
           $4337 = $4336;
           $4338 = HEAP32[$4337>>2]|0;
           $4339 = (($4336) + 4)|0;
           $4340 = $4339;
           $4341 = HEAP32[$4340>>2]|0;
           $4342 = $4341 & 1073741823;
           $4343 = $v;
           $4344 = $4343;
           HEAP32[$4344>>2] = $4338;
           $4345 = (($4343) + 4)|0;
           $4346 = $4345;
           HEAP32[$4346>>2] = $4342;
           $4347 = $v;
           $4348 = $4347;
           $4349 = HEAP32[$4348>>2]|0;
           $4350 = (($4347) + 4)|0;
           $4351 = $4350;
           $4352 = HEAP32[$4351>>2]|0;
           $4353 = (_i64Add(($4349|0),($4352|0),1,0)|0);
           $4354 = tempRet0;
           $4355 = $v;
           $4356 = $4355;
           HEAP32[$4356>>2] = $4353;
           $4357 = (($4355) + 4)|0;
           $4358 = $4357;
           HEAP32[$4358>>2] = $4354;
           $4359 = HEAP32[$pC41>>2]|0;
           $4360 = HEAP32[$4359>>2]|0;
           $4361 = $v;
           $4362 = $4361;
           $4363 = HEAP32[$4362>>2]|0;
           $4364 = (($4361) + 4)|0;
           $4365 = $4364;
           $4366 = HEAP32[$4365>>2]|0;
           $4367 = (_sqlite3BtreeMovetoUnpacked($4360,0,$4363,$4366,0,$res42)|0);
           HEAP32[$rc>>2] = $4367;
           $4368 = ($4367|0)==(0);
           $4369 = HEAP32[$res42>>2]|0;
           $4370 = ($4369|0)==(0);
           $or$cond15 = $4368 & $4370;
           if (!($or$cond15)) {
            break;
           }
           $4371 = HEAP32[$cnt43>>2]|0;
           $4372 = (($4371) + 1)|0;
           HEAP32[$cnt43>>2] = $4372;
           $4373 = ($4372|0)<(100);
           if (!($4373)) {
            break;
           }
          }
          $4374 = HEAP32[$rc>>2]|0;
          $4375 = ($4374|0)==(0);
          $4376 = HEAP32[$res42>>2]|0;
          $4377 = ($4376|0)==(0);
          $or$cond18 = $4375 & $4377;
          if ($or$cond18) {
           label = 566;
           break L11;
          }
         }
         $4378 = HEAP32[$pC41>>2]|0;
         $4379 = (($4378) + 26|0);
         HEAP8[$4379>>0] = 0;
         $4380 = HEAP32[$pC41>>2]|0;
         $4381 = (($4380) + 60|0);
         HEAP32[$4381>>2] = 0;
        }
        $4382 = $v;
        $4383 = $4382;
        $4384 = HEAP32[$4383>>2]|0;
        $4385 = (($4382) + 4)|0;
        $4386 = $4385;
        $4387 = HEAP32[$4386>>2]|0;
        $4388 = HEAP32[$pOut>>2]|0;
        $4389 = $4388;
        $4390 = $4389;
        HEAP32[$4390>>2] = $4384;
        $4391 = (($4389) + 4)|0;
        $4392 = $4391;
        HEAP32[$4392>>2] = $4387;
        break;
       }
       case 84: case 75:  {
        $4393 = HEAP32[$pOp>>2]|0;
        $4394 = (($4393) + 8|0);
        $4395 = HEAP32[$4394>>2]|0;
        $4396 = HEAP32[$aMem>>2]|0;
        $4397 = (($4396) + (($4395*40)|0)|0);
        HEAP32[$pData>>2] = $4397;
        $4398 = HEAP32[$pOp>>2]|0;
        $4399 = (($4398) + 4|0);
        $4400 = HEAP32[$4399>>2]|0;
        $4401 = HEAP32[$0>>2]|0;
        $4402 = (($4401) + 56|0);
        $4403 = HEAP32[$4402>>2]|0;
        $4404 = (($4403) + ($4400<<2)|0);
        $4405 = HEAP32[$4404>>2]|0;
        HEAP32[$pC47>>2] = $4405;
        $4406 = HEAP32[$pOp>>2]|0;
        $4407 = HEAP8[$4406>>0]|0;
        $4408 = $4407&255;
        $4409 = ($4408|0)==(75);
        $4410 = HEAP32[$pOp>>2]|0;
        $4411 = (($4410) + 12|0);
        $4412 = HEAP32[$4411>>2]|0;
        if ($4409) {
         $4413 = HEAP32[$aMem>>2]|0;
         $4414 = (($4413) + (($4412*40)|0)|0);
         HEAP32[$pKey>>2] = $4414;
         $4415 = HEAP32[$pKey>>2]|0;
         $4416 = $4415;
         $4417 = $4416;
         $4418 = HEAP32[$4417>>2]|0;
         $4419 = (($4416) + 4)|0;
         $4420 = $4419;
         $4421 = HEAP32[$4420>>2]|0;
         $4422 = $iKey46;
         $4423 = $4422;
         HEAP32[$4423>>2] = $4418;
         $4424 = (($4422) + 4)|0;
         $4425 = $4424;
         HEAP32[$4425>>2] = $4421;
        } else {
         $4426 = ($4412|0)<(0);
         $4427 = $4426 << 31 >> 31;
         $4428 = $iKey46;
         $4429 = $4428;
         HEAP32[$4429>>2] = $4412;
         $4430 = (($4428) + 4)|0;
         $4431 = $4430;
         HEAP32[$4431>>2] = $4427;
        }
        $4432 = HEAP32[$pOp>>2]|0;
        $4433 = (($4432) + 3|0);
        $4434 = HEAP8[$4433>>0]|0;
        $4435 = $4434&255;
        $4436 = $4435 & 1;
        $4437 = ($4436|0)!=(0);
        if ($4437) {
         $4438 = HEAP32[$0>>2]|0;
         $4439 = (($4438) + 92|0);
         $4440 = HEAP32[$4439>>2]|0;
         $4441 = (($4440) + 1)|0;
         HEAP32[$4439>>2] = $4441;
        }
        $4442 = HEAP32[$pOp>>2]|0;
        $4443 = (($4442) + 3|0);
        $4444 = HEAP8[$4443>>0]|0;
        $4445 = $4444&255;
        $4446 = $4445 & 2;
        $4447 = ($4446|0)!=(0);
        if ($4447) {
         $4448 = $iKey46;
         $4449 = $4448;
         $4450 = HEAP32[$4449>>2]|0;
         $4451 = (($4448) + 4)|0;
         $4452 = $4451;
         $4453 = HEAP32[$4452>>2]|0;
         $4454 = $lastRowid;
         $4455 = $4454;
         HEAP32[$4455>>2] = $4450;
         $4456 = (($4454) + 4)|0;
         $4457 = $4456;
         HEAP32[$4457>>2] = $4453;
         $4458 = HEAP32[$db>>2]|0;
         $4459 = (($4458) + 32|0);
         $4460 = $4459;
         $4461 = $4460;
         HEAP32[$4461>>2] = $4450;
         $4462 = (($4460) + 4)|0;
         $4463 = $4462;
         HEAP32[$4463>>2] = $4453;
        }
        $4464 = HEAP32[$pData>>2]|0;
        $4465 = (($4464) + 8|0);
        $4466 = HEAP16[$4465>>1]|0;
        $4467 = $4466&65535;
        $4468 = $4467 & 1;
        $4469 = ($4468|0)!=(0);
        if ($4469) {
         $4470 = HEAP32[$pData>>2]|0;
         $4471 = (($4470) + 16|0);
         HEAP32[$4471>>2] = 0;
         $4472 = HEAP32[$pData>>2]|0;
         $4473 = (($4472) + 12|0);
         HEAP32[$4473>>2] = 0;
        }
        $4474 = HEAP32[$pOp>>2]|0;
        $4475 = (($4474) + 3|0);
        $4476 = HEAP8[$4475>>0]|0;
        $4477 = $4476&255;
        $4478 = $4477 & 16;
        $4479 = ($4478|0)!=(0);
        if ($4479) {
         $4480 = HEAP32[$pC47>>2]|0;
         $4481 = (($4480) + 12|0);
         $4482 = HEAP32[$4481>>2]|0;
         $4483 = $4482;
        } else {
         $4483 = 0;
        }
        HEAP32[$seekResult>>2] = $4483;
        $4484 = HEAP32[$pData>>2]|0;
        $4485 = (($4484) + 8|0);
        $4486 = HEAP16[$4485>>1]|0;
        $4487 = $4486&65535;
        $4488 = $4487 & 16384;
        $4489 = ($4488|0)!=(0);
        if ($4489) {
         $4490 = HEAP32[$pData>>2]|0;
         $4491 = HEAP32[$4490>>2]|0;
         HEAP32[$nZero48>>2] = $4491;
        } else {
         HEAP32[$nZero48>>2] = 0;
        }
        $4492 = HEAP32[$pC47>>2]|0;
        $4493 = HEAP32[$4492>>2]|0;
        $4494 = $iKey46;
        $4495 = $4494;
        $4496 = HEAP32[$4495>>2]|0;
        $4497 = (($4494) + 4)|0;
        $4498 = $4497;
        $4499 = HEAP32[$4498>>2]|0;
        $4500 = HEAP32[$pData>>2]|0;
        $4501 = (($4500) + 16|0);
        $4502 = HEAP32[$4501>>2]|0;
        $4503 = HEAP32[$pData>>2]|0;
        $4504 = (($4503) + 12|0);
        $4505 = HEAP32[$4504>>2]|0;
        $4506 = HEAP32[$nZero48>>2]|0;
        $4507 = HEAP32[$pOp>>2]|0;
        $4508 = (($4507) + 3|0);
        $4509 = HEAP8[$4508>>0]|0;
        $4510 = $4509&255;
        $4511 = $4510 & 8;
        $4512 = ($4511|0)!=(0);
        $4513 = $4512&1;
        $4514 = HEAP32[$seekResult>>2]|0;
        $4515 = (_sqlite3BtreeInsert($4493,0,$4496,$4499,$4502,$4505,$4506,$4513,$4514)|0);
        HEAP32[$rc>>2] = $4515;
        $4516 = HEAP32[$pC47>>2]|0;
        $4517 = (($4516) + 26|0);
        HEAP8[$4517>>0] = 0;
        $4518 = HEAP32[$pC47>>2]|0;
        $4519 = (($4518) + 60|0);
        HEAP32[$4519>>2] = 0;
        $4520 = HEAP32[$rc>>2]|0;
        $4521 = ($4520|0)==(0);
        if ($4521) {
         $4522 = HEAP32[$db>>2]|0;
         $4523 = (($4522) + 208|0);
         $4524 = HEAP32[$4523>>2]|0;
         $4525 = ($4524|0)!=(0|0);
         if ($4525) {
          $4526 = HEAP32[$pOp>>2]|0;
          $4527 = (($4526) + 16|0);
          $4528 = HEAP32[$4527>>2]|0;
          $4529 = ($4528|0)!=(0|0);
          if ($4529) {
           $4530 = HEAP32[$pC47>>2]|0;
           $4531 = (($4530) + 24|0);
           $4532 = HEAP8[$4531>>0]|0;
           $4533 = $4532 << 24 >> 24;
           $4534 = HEAP32[$db>>2]|0;
           $4535 = (($4534) + 16|0);
           $4536 = HEAP32[$4535>>2]|0;
           $4537 = (($4536) + ($4533<<4)|0);
           $4538 = HEAP32[$4537>>2]|0;
           HEAP32[$zDb>>2] = $4538;
           $4539 = HEAP32[$pOp>>2]|0;
           $4540 = (($4539) + 16|0);
           $4541 = HEAP32[$4540>>2]|0;
           HEAP32[$zTbl>>2] = $4541;
           $4542 = HEAP32[$pOp>>2]|0;
           $4543 = (($4542) + 3|0);
           $4544 = HEAP8[$4543>>0]|0;
           $4545 = $4544&255;
           $4546 = $4545 & 4;
           $4547 = ($4546|0)!=(0);
           $4548 = $4547 ? 23 : 18;
           HEAP32[$op49>>2] = $4548;
           $4549 = HEAP32[$db>>2]|0;
           $4550 = (($4549) + 208|0);
           $4551 = HEAP32[$4550>>2]|0;
           $4552 = HEAP32[$db>>2]|0;
           $4553 = (($4552) + 204|0);
           $4554 = HEAP32[$4553>>2]|0;
           $4555 = HEAP32[$op49>>2]|0;
           $4556 = HEAP32[$zDb>>2]|0;
           $4557 = HEAP32[$zTbl>>2]|0;
           $4558 = $iKey46;
           $4559 = $4558;
           $4560 = HEAP32[$4559>>2]|0;
           $4561 = (($4558) + 4)|0;
           $4562 = $4561;
           $4563 = HEAP32[$4562>>2]|0;
           FUNCTION_TABLE_viiiiii[$4551 & 0]($4554,$4555,$4556,$4557,$4560,$4563);
          }
         }
        }
        break;
       }
       case 95:  {
        $4564 = HEAP32[$pOp>>2]|0;
        $4565 = (($4564) + 4|0);
        $4566 = HEAP32[$4565>>2]|0;
        $4567 = HEAP32[$0>>2]|0;
        $4568 = (($4567) + 56|0);
        $4569 = HEAP32[$4568>>2]|0;
        $4570 = (($4569) + ($4566<<2)|0);
        $4571 = HEAP32[$4570>>2]|0;
        HEAP32[$pC50>>2] = $4571;
        $4572 = HEAP32[$pC50>>2]|0;
        $4573 = HEAP32[$4572>>2]|0;
        $4574 = (_sqlite3BtreeDelete($4573)|0);
        HEAP32[$rc>>2] = $4574;
        $4575 = HEAP32[$pC50>>2]|0;
        $4576 = (($4575) + 60|0);
        HEAP32[$4576>>2] = 0;
        $4577 = HEAP32[$rc>>2]|0;
        $4578 = ($4577|0)==(0);
        if ($4578) {
         $4579 = HEAP32[$db>>2]|0;
         $4580 = (($4579) + 208|0);
         $4581 = HEAP32[$4580>>2]|0;
         $4582 = ($4581|0)!=(0|0);
         if ($4582) {
          $4583 = HEAP32[$pOp>>2]|0;
          $4584 = (($4583) + 16|0);
          $4585 = HEAP32[$4584>>2]|0;
          $4586 = ($4585|0)!=(0|0);
          if ($4586) {
           $4587 = HEAP32[$pC50>>2]|0;
           $4588 = (($4587) + 27|0);
           $4589 = HEAP8[$4588>>0]|0;
           $4590 = ($4589&255) >>> 2;
           $4591 = $4590 & 1;
           $4592 = $4591&255;
           $4593 = ($4592|0)!=(0);
           if ($4593) {
            $4594 = HEAP32[$db>>2]|0;
            $4595 = (($4594) + 208|0);
            $4596 = HEAP32[$4595>>2]|0;
            $4597 = HEAP32[$db>>2]|0;
            $4598 = (($4597) + 204|0);
            $4599 = HEAP32[$4598>>2]|0;
            $4600 = HEAP32[$pC50>>2]|0;
            $4601 = (($4600) + 24|0);
            $4602 = HEAP8[$4601>>0]|0;
            $4603 = $4602 << 24 >> 24;
            $4604 = HEAP32[$db>>2]|0;
            $4605 = (($4604) + 16|0);
            $4606 = HEAP32[$4605>>2]|0;
            $4607 = (($4606) + ($4603<<4)|0);
            $4608 = HEAP32[$4607>>2]|0;
            $4609 = HEAP32[$pOp>>2]|0;
            $4610 = (($4609) + 16|0);
            $4611 = HEAP32[$4610>>2]|0;
            $4612 = HEAP32[$pC50>>2]|0;
            $4613 = (($4612) + 48|0);
            $4614 = $4613;
            $4615 = $4614;
            $4616 = HEAP32[$4615>>2]|0;
            $4617 = (($4614) + 4)|0;
            $4618 = $4617;
            $4619 = HEAP32[$4618>>2]|0;
            FUNCTION_TABLE_viiiiii[$4596 & 0]($4599,9,$4608,$4611,$4616,$4619);
           }
          }
         }
        }
        $4620 = HEAP32[$pOp>>2]|0;
        $4621 = (($4620) + 8|0);
        $4622 = HEAP32[$4621>>2]|0;
        $4623 = $4622 & 1;
        $4624 = ($4623|0)!=(0);
        if ($4624) {
         $4625 = HEAP32[$0>>2]|0;
         $4626 = (($4625) + 92|0);
         $4627 = HEAP32[$4626>>2]|0;
         $4628 = (($4627) + 1)|0;
         HEAP32[$4626>>2] = $4628;
        }
        break;
       }
       case 98:  {
        $4629 = HEAP32[$db>>2]|0;
        $4630 = HEAP32[$0>>2]|0;
        $4631 = (($4630) + 92|0);
        $4632 = HEAP32[$4631>>2]|0;
        _sqlite3VdbeSetChanges($4629,$4632);
        $4633 = HEAP32[$0>>2]|0;
        $4634 = (($4633) + 92|0);
        HEAP32[$4634>>2] = 0;
        break;
       }
       case 99:  {
        $4635 = HEAP32[$pOp>>2]|0;
        $4636 = (($4635) + 4|0);
        $4637 = HEAP32[$4636>>2]|0;
        $4638 = HEAP32[$0>>2]|0;
        $4639 = (($4638) + 56|0);
        $4640 = HEAP32[$4639>>2]|0;
        $4641 = (($4640) + ($4637<<2)|0);
        $4642 = HEAP32[$4641>>2]|0;
        HEAP32[$pC51>>2] = $4642;
        $4643 = HEAP32[$pOp>>2]|0;
        $4644 = (($4643) + 12|0);
        $4645 = HEAP32[$4644>>2]|0;
        $4646 = HEAP32[$aMem>>2]|0;
        $4647 = (($4646) + (($4645*40)|0)|0);
        HEAP32[$pIn3>>2] = $4647;
        $4648 = HEAP32[$pOp>>2]|0;
        $4649 = (($4648) + 16|0);
        $4650 = HEAP32[$4649>>2]|0;
        HEAP32[$nKeyCol>>2] = $4650;
        HEAP32[$res52>>2] = 0;
        $4651 = HEAP32[$pC51>>2]|0;
        $4652 = HEAP32[$pIn3>>2]|0;
        $4653 = HEAP32[$nKeyCol>>2]|0;
        $4654 = (_sqlite3VdbeSorterCompare($4651,$4652,$4653,$res52)|0);
        HEAP32[$rc>>2] = $4654;
        $4655 = HEAP32[$res52>>2]|0;
        $4656 = ($4655|0)!=(0);
        if ($4656) {
         $4657 = HEAP32[$pOp>>2]|0;
         $4658 = (($4657) + 8|0);
         $4659 = HEAP32[$4658>>2]|0;
         $4660 = (($4659) - 1)|0;
         HEAP32[$pc>>2] = $4660;
        }
        break;
       }
       case 100:  {
        $4661 = HEAP32[$pOp>>2]|0;
        $4662 = (($4661) + 8|0);
        $4663 = HEAP32[$4662>>2]|0;
        $4664 = HEAP32[$aMem>>2]|0;
        $4665 = (($4664) + (($4663*40)|0)|0);
        HEAP32[$pOut>>2] = $4665;
        $4666 = HEAP32[$pOp>>2]|0;
        $4667 = (($4666) + 4|0);
        $4668 = HEAP32[$4667>>2]|0;
        $4669 = HEAP32[$0>>2]|0;
        $4670 = (($4669) + 56|0);
        $4671 = HEAP32[$4670>>2]|0;
        $4672 = (($4671) + ($4668<<2)|0);
        $4673 = HEAP32[$4672>>2]|0;
        HEAP32[$pC53>>2] = $4673;
        $4674 = HEAP32[$pC53>>2]|0;
        $4675 = HEAP32[$pOut>>2]|0;
        $4676 = (_sqlite3VdbeSorterRowkey($4674,$4675)|0);
        HEAP32[$rc>>2] = $4676;
        $4677 = HEAP32[$pOp>>2]|0;
        $4678 = (($4677) + 12|0);
        $4679 = HEAP32[$4678>>2]|0;
        $4680 = HEAP32[$0>>2]|0;
        $4681 = (($4680) + 56|0);
        $4682 = HEAP32[$4681>>2]|0;
        $4683 = (($4682) + ($4679<<2)|0);
        $4684 = HEAP32[$4683>>2]|0;
        $4685 = (($4684) + 60|0);
        HEAP32[$4685>>2] = 0;
        break;
       }
       case 102: case 101:  {
        $4686 = HEAP32[$pOp>>2]|0;
        $4687 = (($4686) + 8|0);
        $4688 = HEAP32[$4687>>2]|0;
        $4689 = HEAP32[$aMem>>2]|0;
        $4690 = (($4689) + (($4688*40)|0)|0);
        HEAP32[$pOut>>2] = $4690;
        $4691 = HEAP32[$pOp>>2]|0;
        $4692 = (($4691) + 4|0);
        $4693 = HEAP32[$4692>>2]|0;
        $4694 = HEAP32[$0>>2]|0;
        $4695 = (($4694) + 56|0);
        $4696 = HEAP32[$4695>>2]|0;
        $4697 = (($4696) + ($4693<<2)|0);
        $4698 = HEAP32[$4697>>2]|0;
        HEAP32[$pC54>>2] = $4698;
        $4699 = HEAP32[$pC54>>2]|0;
        $4700 = HEAP32[$4699>>2]|0;
        HEAP32[$pCrsr55>>2] = $4700;
        $4701 = HEAP32[$pC54>>2]|0;
        $4702 = (($4701) + 27|0);
        $4703 = HEAP8[$4702>>0]|0;
        $4704 = ($4703&255) >>> 2;
        $4705 = $4704 & 1;
        $4706 = $4705&255;
        $4707 = ($4706|0)==(0);
        $4708 = HEAP32[$pCrsr55>>2]|0;
        if ($4707) {
         (_sqlite3BtreeKeySize($4708,$n64)|0);
         $4709 = $n64;
         $4710 = $4709;
         $4711 = HEAP32[$4710>>2]|0;
         $4712 = (($4709) + 4)|0;
         $4713 = $4712;
         $4714 = HEAP32[$4713>>2]|0;
         $4715 = HEAP32[$db>>2]|0;
         $4716 = (($4715) + 88|0);
         $4717 = HEAP32[$4716>>2]|0;
         $4718 = ($4717|0)<(0);
         $4719 = $4718 << 31 >> 31;
         $4720 = ($4714|0)>($4719|0);
         $4721 = ($4711>>>0)>($4717>>>0);
         $4722 = ($4714|0)==($4719|0);
         $4723 = $4722 & $4721;
         $4724 = $4720 | $4723;
         if ($4724) {
          label = 870;
          break L11;
         }
         $4725 = $n64;
         $4726 = $4725;
         $4727 = HEAP32[$4726>>2]|0;
         $4728 = (($4725) + 4)|0;
         $4729 = $4728;
         $4730 = HEAP32[$4729>>2]|0;
         HEAP32[$n56>>2] = $4727;
        } else {
         (_sqlite3BtreeDataSize($4708,$n56)|0);
         $4731 = HEAP32[$n56>>2]|0;
         $4732 = HEAP32[$db>>2]|0;
         $4733 = (($4732) + 88|0);
         $4734 = HEAP32[$4733>>2]|0;
         $4735 = ($4731>>>0)>($4734>>>0);
         if ($4735) {
          label = 870;
          break L11;
         }
        }
        $4736 = HEAP32[$pOut>>2]|0;
        $4737 = HEAP32[$n56>>2]|0;
        $4738 = ($4737>>>0)>(32);
        $4739 = HEAP32[$n56>>2]|0;
        $4740 = $4738 ? $4739 : 32;
        $4741 = (_sqlite3VdbeMemClearAndResize($4736,$4740)|0);
        $4742 = ($4741|0)!=(0);
        if ($4742) {
         label = 871;
         break L1;
        }
        $4743 = HEAP32[$n56>>2]|0;
        $4744 = HEAP32[$pOut>>2]|0;
        $4745 = (($4744) + 12|0);
        HEAP32[$4745>>2] = $4743;
        $4746 = HEAP32[$pOut>>2]|0;
        $4747 = (($4746) + 8|0);
        $4748 = HEAP16[$4747>>1]|0;
        $4749 = $4748&65535;
        $4750 = $4749 & -16896;
        $4751 = $4750 | 16;
        $4752 = $4751&65535;
        $4753 = HEAP32[$pOut>>2]|0;
        $4754 = (($4753) + 8|0);
        HEAP16[$4754>>1] = $4752;
        $4755 = HEAP32[$pC54>>2]|0;
        $4756 = (($4755) + 27|0);
        $4757 = HEAP8[$4756>>0]|0;
        $4758 = ($4757&255) >>> 2;
        $4759 = $4758 & 1;
        $4760 = $4759&255;
        $4761 = ($4760|0)==(0);
        $4762 = HEAP32[$pCrsr55>>2]|0;
        $4763 = HEAP32[$n56>>2]|0;
        $4764 = HEAP32[$pOut>>2]|0;
        $4765 = (($4764) + 16|0);
        $4766 = HEAP32[$4765>>2]|0;
        if ($4761) {
         $4767 = (_sqlite3BtreeKey($4762,0,$4763,$4766)|0);
         HEAP32[$rc>>2] = $4767;
        } else {
         $4768 = (_sqlite3BtreeData($4762,0,$4763,$4766)|0);
         HEAP32[$rc>>2] = $4768;
        }
        $4769 = HEAP32[$pOut>>2]|0;
        $4770 = (($4769) + 10|0);
        HEAP8[$4770>>0] = 1;
        break;
       }
       case 103:  {
        $4771 = HEAP32[$pOp>>2]|0;
        $4772 = (($4771) + 4|0);
        $4773 = HEAP32[$4772>>2]|0;
        $4774 = HEAP32[$0>>2]|0;
        $4775 = (($4774) + 56|0);
        $4776 = HEAP32[$4775>>2]|0;
        $4777 = (($4776) + ($4773<<2)|0);
        $4778 = HEAP32[$4777>>2]|0;
        HEAP32[$pC57>>2] = $4778;
        $4779 = HEAP32[$pC57>>2]|0;
        $4780 = (($4779) + 25|0);
        $4781 = HEAP8[$4780>>0]|0;
        $4782 = ($4781<<24>>24)!=(0);
        if ($4782) {
         $4783 = HEAP32[$pOut>>2]|0;
         $4784 = (($4783) + 8|0);
         HEAP16[$4784>>1] = 1;
         break L21;
        }
        $4785 = HEAP32[$pC57>>2]|0;
        $4786 = (($4785) + 26|0);
        $4787 = HEAP8[$4786>>0]|0;
        $4788 = ($4787<<24>>24)!=(0);
        $4789 = HEAP32[$pC57>>2]|0;
        do {
         if ($4788) {
          $4790 = (($4789) + 48|0);
          $4791 = $4790;
          $4792 = $4791;
          $4793 = HEAP32[$4792>>2]|0;
          $4794 = (($4791) + 4)|0;
          $4795 = $4794;
          $4796 = HEAP32[$4795>>2]|0;
          $4797 = $v58;
          $4798 = $4797;
          HEAP32[$4798>>2] = $4793;
          $4799 = (($4797) + 4)|0;
          $4800 = $4799;
          HEAP32[$4800>>2] = $4796;
         } else {
          $4801 = (($4789) + 32|0);
          $4802 = HEAP32[$4801>>2]|0;
          $4803 = ($4802|0)!=(0|0);
          $4804 = HEAP32[$pC57>>2]|0;
          if ($4803) {
           $4805 = (($4804) + 32|0);
           $4806 = HEAP32[$4805>>2]|0;
           $4807 = HEAP32[$4806>>2]|0;
           HEAP32[$pVtab>>2] = $4807;
           $4808 = HEAP32[$pVtab>>2]|0;
           $4809 = HEAP32[$4808>>2]|0;
           HEAP32[$pModule>>2] = $4809;
           $4810 = HEAP32[$pModule>>2]|0;
           $4811 = (($4810) + 48|0);
           $4812 = HEAP32[$4811>>2]|0;
           $4813 = HEAP32[$pC57>>2]|0;
           $4814 = (($4813) + 32|0);
           $4815 = HEAP32[$4814>>2]|0;
           $4816 = (FUNCTION_TABLE_iii[$4812 & 63]($4815,$v58)|0);
           HEAP32[$rc>>2] = $4816;
           $4817 = HEAP32[$0>>2]|0;
           $4818 = HEAP32[$pVtab>>2]|0;
           _sqlite3VtabImportErrmsg($4817,$4818);
           break;
          }
          $4819 = (_sqlite3VdbeCursorRestore($4804)|0);
          HEAP32[$rc>>2] = $4819;
          $4820 = HEAP32[$rc>>2]|0;
          $4821 = ($4820|0)!=(0);
          if ($4821) {
           break L11;
          }
          $4822 = HEAP32[$pC57>>2]|0;
          $4823 = (($4822) + 25|0);
          $4824 = HEAP8[$4823>>0]|0;
          $4825 = ($4824<<24>>24)!=(0);
          if ($4825) {
           $4826 = HEAP32[$pOut>>2]|0;
           $4827 = (($4826) + 8|0);
           HEAP16[$4827>>1] = 1;
           break L21;
          } else {
           $4828 = HEAP32[$pC57>>2]|0;
           $4829 = HEAP32[$4828>>2]|0;
           $4830 = (_sqlite3BtreeKeySize($4829,$v58)|0);
           HEAP32[$rc>>2] = $4830;
           break;
          }
         }
        } while(0);
        $4831 = $v58;
        $4832 = $4831;
        $4833 = HEAP32[$4832>>2]|0;
        $4834 = (($4831) + 4)|0;
        $4835 = $4834;
        $4836 = HEAP32[$4835>>2]|0;
        $4837 = HEAP32[$pOut>>2]|0;
        $4838 = $4837;
        $4839 = $4838;
        HEAP32[$4839>>2] = $4833;
        $4840 = (($4838) + 4)|0;
        $4841 = $4840;
        HEAP32[$4841>>2] = $4836;
        break;
       }
       case 105:  {
        $4859 = HEAP32[$pOp>>2]|0;
        $4860 = (($4859) + 4|0);
        $4861 = HEAP32[$4860>>2]|0;
        $4862 = HEAP32[$0>>2]|0;
        $4863 = (($4862) + 56|0);
        $4864 = HEAP32[$4863>>2]|0;
        $4865 = (($4864) + ($4861<<2)|0);
        $4866 = HEAP32[$4865>>2]|0;
        HEAP32[$pC60>>2] = $4866;
        $4867 = HEAP32[$pC60>>2]|0;
        $4868 = HEAP32[$4867>>2]|0;
        HEAP32[$pCrsr61>>2] = $4868;
        HEAP32[$res62>>2] = 0;
        $4869 = HEAP32[$pCrsr61>>2]|0;
        $4870 = (_sqlite3BtreeLast($4869,$res62)|0);
        HEAP32[$rc>>2] = $4870;
        $4871 = HEAP32[$res62>>2]|0;
        $4872 = $4871&255;
        $4873 = HEAP32[$pC60>>2]|0;
        $4874 = (($4873) + 25|0);
        HEAP8[$4874>>0] = $4872;
        $4875 = HEAP32[$pC60>>2]|0;
        $4876 = (($4875) + 26|0);
        HEAP8[$4876>>0] = 0;
        $4877 = HEAP32[$pC60>>2]|0;
        $4878 = (($4877) + 60|0);
        HEAP32[$4878>>2] = 0;
        $4879 = HEAP32[$pOp>>2]|0;
        $4880 = (($4879) + 8|0);
        $4881 = HEAP32[$4880>>2]|0;
        $4882 = ($4881|0)>(0);
        $4883 = HEAP32[$res62>>2]|0;
        $4884 = ($4883|0)!=(0);
        $or$cond26 = $4882 & $4884;
        if ($or$cond26) {
         $4885 = HEAP32[$pOp>>2]|0;
         $4886 = (($4885) + 8|0);
         $4887 = HEAP32[$4886>>2]|0;
         $4888 = (($4887) - 1)|0;
         HEAP32[$pc>>2] = $4888;
        }
        break;
       }
       case 104:  {
        $4842 = HEAP32[$pOp>>2]|0;
        $4843 = (($4842) + 4|0);
        $4844 = HEAP32[$4843>>2]|0;
        $4845 = HEAP32[$0>>2]|0;
        $4846 = (($4845) + 56|0);
        $4847 = HEAP32[$4846>>2]|0;
        $4848 = (($4847) + ($4844<<2)|0);
        $4849 = HEAP32[$4848>>2]|0;
        HEAP32[$pC59>>2] = $4849;
        $4850 = HEAP32[$pC59>>2]|0;
        $4851 = (($4850) + 25|0);
        HEAP8[$4851>>0] = 1;
        $4852 = HEAP32[$pC59>>2]|0;
        $4853 = (($4852) + 60|0);
        HEAP32[$4853>>2] = 0;
        $4854 = HEAP32[$pC59>>2]|0;
        $4855 = HEAP32[$4854>>2]|0;
        $4856 = ($4855|0)!=(0|0);
        if ($4856) {
         $4857 = HEAP32[$pC59>>2]|0;
         $4858 = HEAP32[$4857>>2]|0;
         _sqlite3BtreeClearCursor($4858);
        }
        break;
       }
       case 107: case 106:  {
        $4889 = HEAP32[$0>>2]|0;
        $4890 = (($4889) + 108|0);
        $4891 = (($4890) + 8|0);
        $4892 = HEAP32[$4891>>2]|0;
        $4893 = (($4892) + 1)|0;
        HEAP32[$4891>>2] = $4893;
        label = 623;
        break;
       }
       case 108:  {
        label = 623;
        break;
       }
       case 5:  {
        $4925 = HEAP32[$pOp>>2]|0;
        $4926 = (($4925) + 4|0);
        $4927 = HEAP32[$4926>>2]|0;
        $4928 = HEAP32[$0>>2]|0;
        $4929 = (($4928) + 56|0);
        $4930 = HEAP32[$4929>>2]|0;
        $4931 = (($4930) + ($4927<<2)|0);
        $4932 = HEAP32[$4931>>2]|0;
        HEAP32[$pC66>>2] = $4932;
        HEAP32[$res67>>2] = 0;
        $4933 = HEAP32[$db>>2]|0;
        $4934 = HEAP32[$pC66>>2]|0;
        $4935 = (_sqlite3VdbeSorterNext($4933,$4934,$res67)|0);
        HEAP32[$rc>>2] = $4935;
        label = 631;
        break;
       }
       case 7: case 6:  {
        $4936 = HEAP32[$pOp>>2]|0;
        $4937 = (($4936) + 4|0);
        $4938 = HEAP32[$4937>>2]|0;
        $4939 = HEAP32[$0>>2]|0;
        $4940 = (($4939) + 56|0);
        $4941 = HEAP32[$4940>>2]|0;
        $4942 = (($4941) + ($4938<<2)|0);
        $4943 = HEAP32[$4942>>2]|0;
        $4944 = ($4943|0)==(0|0);
        if (!($4944)) {
         label = 630;
        }
        break;
       }
       case 9: case 8:  {
        label = 630;
        break;
       }
       case 110: case 109:  {
        $4981 = HEAP32[$pOp>>2]|0;
        $4982 = (($4981) + 4|0);
        $4983 = HEAP32[$4982>>2]|0;
        $4984 = HEAP32[$0>>2]|0;
        $4985 = (($4984) + 56|0);
        $4986 = HEAP32[$4985>>2]|0;
        $4987 = (($4986) + ($4983<<2)|0);
        $4988 = HEAP32[$4987>>2]|0;
        HEAP32[$pC68>>2] = $4988;
        $4989 = HEAP32[$pOp>>2]|0;
        $4990 = (($4989) + 8|0);
        $4991 = HEAP32[$4990>>2]|0;
        $4992 = HEAP32[$aMem>>2]|0;
        $4993 = (($4992) + (($4991*40)|0)|0);
        HEAP32[$pIn2>>2] = $4993;
        $4994 = HEAP32[$pC68>>2]|0;
        $4995 = HEAP32[$4994>>2]|0;
        HEAP32[$pCrsr69>>2] = $4995;
        $4996 = HEAP32[$pOp>>2]|0;
        $4997 = (($4996) + 3|0);
        $4998 = HEAP8[$4997>>0]|0;
        $4999 = $4998&255;
        $5000 = $4999 & 1;
        $5001 = ($5000|0)!=(0);
        if ($5001) {
         $5002 = HEAP32[$0>>2]|0;
         $5003 = (($5002) + 92|0);
         $5004 = HEAP32[$5003>>2]|0;
         $5005 = (($5004) + 1)|0;
         HEAP32[$5003>>2] = $5005;
        }
        $5006 = HEAP32[$pIn2>>2]|0;
        $5007 = (($5006) + 8|0);
        $5008 = HEAP16[$5007>>1]|0;
        $5009 = $5008&65535;
        $5010 = $5009 & 16384;
        $5011 = ($5010|0)!=(0);
        if ($5011) {
         $5012 = HEAP32[$pIn2>>2]|0;
         $5013 = (_sqlite3VdbeMemExpandBlob($5012)|0);
         $5014 = $5013;
        } else {
         $5014 = 0;
        }
        HEAP32[$rc>>2] = $5014;
        $5015 = HEAP32[$rc>>2]|0;
        $5016 = ($5015|0)==(0);
        if ($5016) {
         $5017 = HEAP32[$pC68>>2]|0;
         $5018 = (($5017) + 56|0);
         $5019 = HEAP32[$5018>>2]|0;
         $5020 = ($5019|0)!=(0|0);
         if ($5020) {
          $5021 = HEAP32[$pC68>>2]|0;
          $5022 = HEAP32[$pIn2>>2]|0;
          $5023 = (_sqlite3VdbeSorterWrite($5021,$5022)|0);
          HEAP32[$rc>>2] = $5023;
          break L21;
         }
         $5024 = HEAP32[$pIn2>>2]|0;
         $5025 = (($5024) + 12|0);
         $5026 = HEAP32[$5025>>2]|0;
         HEAP32[$nKey>>2] = $5026;
         $5027 = HEAP32[$pIn2>>2]|0;
         $5028 = (($5027) + 16|0);
         $5029 = HEAP32[$5028>>2]|0;
         HEAP32[$zKey>>2] = $5029;
         $5030 = HEAP32[$pCrsr69>>2]|0;
         $5031 = HEAP32[$zKey>>2]|0;
         $5032 = HEAP32[$nKey>>2]|0;
         $5033 = ($5032|0)<(0);
         $5034 = $5033 << 31 >> 31;
         $5035 = HEAP32[$pOp>>2]|0;
         $5036 = (($5035) + 12|0);
         $5037 = HEAP32[$5036>>2]|0;
         $5038 = HEAP32[$pOp>>2]|0;
         $5039 = (($5038) + 3|0);
         $5040 = HEAP8[$5039>>0]|0;
         $5041 = $5040&255;
         $5042 = $5041 & 16;
         $5043 = ($5042|0)!=(0);
         if ($5043) {
          $5044 = HEAP32[$pC68>>2]|0;
          $5045 = (($5044) + 12|0);
          $5046 = HEAP32[$5045>>2]|0;
          $5047 = $5046;
         } else {
          $5047 = 0;
         }
         $5048 = (_sqlite3BtreeInsert($5030,$5031,$5032,$5034,1216,0,0,$5037,$5047)|0);
         HEAP32[$rc>>2] = $5048;
         $5049 = HEAP32[$pC68>>2]|0;
         $5050 = (($5049) + 60|0);
         HEAP32[$5050>>2] = 0;
        }
        break;
       }
       case 112:  {
        $5086 = HEAP32[$pOp>>2]|0;
        $5087 = (($5086) + 4|0);
        $5088 = HEAP32[$5087>>2]|0;
        $5089 = HEAP32[$0>>2]|0;
        $5090 = (($5089) + 56|0);
        $5091 = HEAP32[$5090>>2]|0;
        $5092 = (($5091) + ($5088<<2)|0);
        $5093 = HEAP32[$5092>>2]|0;
        HEAP32[$pC75>>2] = $5093;
        $5094 = HEAP32[$pC75>>2]|0;
        $5095 = HEAP32[$5094>>2]|0;
        HEAP32[$pCrsr74>>2] = $5095;
        $5096 = HEAP32[$pOut>>2]|0;
        $5097 = (($5096) + 8|0);
        HEAP16[$5097>>1] = 1;
        $5098 = HEAP32[$pC75>>2]|0;
        $5099 = (_sqlite3VdbeCursorRestore($5098)|0);
        HEAP32[$rc>>2] = $5099;
        $5100 = HEAP32[$rc>>2]|0;
        $5101 = ($5100|0)!=(0);
        if ($5101) {
         break L11;
        }
        $5102 = HEAP32[$pC75>>2]|0;
        $5103 = (($5102) + 25|0);
        $5104 = HEAP8[$5103>>0]|0;
        $5105 = ($5104<<24>>24)!=(0);
        if (!($5105)) {
         $5106 = $rowid;
         $5107 = $5106;
         HEAP32[$5107>>2] = 0;
         $5108 = (($5106) + 4)|0;
         $5109 = $5108;
         HEAP32[$5109>>2] = 0;
         $5110 = HEAP32[$db>>2]|0;
         $5111 = HEAP32[$pCrsr74>>2]|0;
         $5112 = (_sqlite3VdbeIdxRowid($5110,$5111,$rowid)|0);
         HEAP32[$rc>>2] = $5112;
         $5113 = HEAP32[$rc>>2]|0;
         $5114 = ($5113|0)!=(0);
         if ($5114) {
          break L11;
         }
         $5115 = $rowid;
         $5116 = $5115;
         $5117 = HEAP32[$5116>>2]|0;
         $5118 = (($5115) + 4)|0;
         $5119 = $5118;
         $5120 = HEAP32[$5119>>2]|0;
         $5121 = HEAP32[$pOut>>2]|0;
         $5122 = $5121;
         $5123 = $5122;
         HEAP32[$5123>>2] = $5117;
         $5124 = (($5122) + 4)|0;
         $5125 = $5124;
         HEAP32[$5125>>2] = $5120;
         $5126 = HEAP32[$pOut>>2]|0;
         $5127 = (($5126) + 8|0);
         HEAP16[$5127>>1] = 4;
        }
        break;
       }
       case 111:  {
        $5051 = HEAP32[$pOp>>2]|0;
        $5052 = (($5051) + 4|0);
        $5053 = HEAP32[$5052>>2]|0;
        $5054 = HEAP32[$0>>2]|0;
        $5055 = (($5054) + 56|0);
        $5056 = HEAP32[$5055>>2]|0;
        $5057 = (($5056) + ($5053<<2)|0);
        $5058 = HEAP32[$5057>>2]|0;
        HEAP32[$pC70>>2] = $5058;
        $5059 = HEAP32[$pC70>>2]|0;
        $5060 = HEAP32[$5059>>2]|0;
        HEAP32[$pCrsr71>>2] = $5060;
        $5061 = HEAP32[$pC70>>2]|0;
        $5062 = (($5061) + 8|0);
        $5063 = HEAP32[$5062>>2]|0;
        HEAP32[$r73>>2] = $5063;
        $5064 = HEAP32[$pOp>>2]|0;
        $5065 = (($5064) + 12|0);
        $5066 = HEAP32[$5065>>2]|0;
        $5067 = $5066&65535;
        $5068 = (($r73) + 4|0);
        HEAP16[$5068>>1] = $5067;
        $5069 = (($r73) + 6|0);
        HEAP8[$5069>>0] = 0;
        $5070 = HEAP32[$pOp>>2]|0;
        $5071 = (($5070) + 8|0);
        $5072 = HEAP32[$5071>>2]|0;
        $5073 = HEAP32[$aMem>>2]|0;
        $5074 = (($5073) + (($5072*40)|0)|0);
        $5075 = (($r73) + 8|0);
        HEAP32[$5075>>2] = $5074;
        $5076 = HEAP32[$pCrsr71>>2]|0;
        $5077 = (_sqlite3BtreeMovetoUnpacked($5076,$r73,0,0,0,$res72)|0);
        HEAP32[$rc>>2] = $5077;
        $5078 = HEAP32[$rc>>2]|0;
        $5079 = ($5078|0)==(0);
        $5080 = HEAP32[$res72>>2]|0;
        $5081 = ($5080|0)==(0);
        $or$cond20 = $5079 & $5081;
        if ($or$cond20) {
         $5082 = HEAP32[$pCrsr71>>2]|0;
         $5083 = (_sqlite3BtreeDelete($5082)|0);
         HEAP32[$rc>>2] = $5083;
        }
        $5084 = HEAP32[$pC70>>2]|0;
        $5085 = (($5084) + 60|0);
        HEAP32[$5085>>2] = 0;
        break;
       }
       case 116: case 115: case 114: case 113:  {
        $5128 = HEAP32[$pOp>>2]|0;
        $5129 = (($5128) + 4|0);
        $5130 = HEAP32[$5129>>2]|0;
        $5131 = HEAP32[$0>>2]|0;
        $5132 = (($5131) + 56|0);
        $5133 = HEAP32[$5132>>2]|0;
        $5134 = (($5133) + ($5130<<2)|0);
        $5135 = HEAP32[$5134>>2]|0;
        HEAP32[$pC76>>2] = $5135;
        $5136 = HEAP32[$pC76>>2]|0;
        $5137 = (($5136) + 8|0);
        $5138 = HEAP32[$5137>>2]|0;
        HEAP32[$r78>>2] = $5138;
        $5139 = HEAP32[$pOp>>2]|0;
        $5140 = (($5139) + 16|0);
        $5141 = HEAP32[$5140>>2]|0;
        $5142 = $5141&65535;
        $5143 = (($r78) + 4|0);
        HEAP16[$5143>>1] = $5142;
        $5144 = HEAP32[$pOp>>2]|0;
        $5145 = HEAP8[$5144>>0]|0;
        $5146 = $5145&255;
        $5147 = ($5146|0)<(115);
        $5148 = (($r78) + 6|0);
        if ($5147) {
         HEAP8[$5148>>0] = -1;
        } else {
         HEAP8[$5148>>0] = 0;
        }
        $5149 = HEAP32[$pOp>>2]|0;
        $5150 = (($5149) + 12|0);
        $5151 = HEAP32[$5150>>2]|0;
        $5152 = HEAP32[$aMem>>2]|0;
        $5153 = (($5152) + (($5151*40)|0)|0);
        $5154 = (($r78) + 8|0);
        HEAP32[$5154>>2] = $5153;
        HEAP32[$res77>>2] = 0;
        $5155 = HEAP32[$db>>2]|0;
        $5156 = HEAP32[$pC76>>2]|0;
        $5157 = (_sqlite3VdbeIdxKeyCompare($5155,$5156,$r78,$res77)|0);
        HEAP32[$rc>>2] = $5157;
        $5158 = HEAP32[$pOp>>2]|0;
        $5159 = HEAP8[$5158>>0]|0;
        $5160 = $5159&255;
        $5161 = $5160 & 1;
        $5162 = ($5161|0)==(1);
        $5163 = HEAP32[$res77>>2]|0;
        if ($5162) {
         $5164 = (0 - ($5163))|0;
         HEAP32[$res77>>2] = $5164;
        } else {
         $5165 = (($5163) + 1)|0;
         HEAP32[$res77>>2] = $5165;
        }
        $5166 = HEAP32[$res77>>2]|0;
        $5167 = ($5166|0)>(0);
        if ($5167) {
         $5168 = HEAP32[$pOp>>2]|0;
         $5169 = (($5168) + 8|0);
         $5170 = HEAP32[$5169>>2]|0;
         $5171 = (($5170) - 1)|0;
         HEAP32[$pc>>2] = $5171;
        }
        break;
       }
       case 117:  {
        HEAP32[$iCnt>>2] = 0;
        $5172 = HEAP32[$db>>2]|0;
        $5173 = (($5172) + 4|0);
        $5174 = HEAP32[$5173>>2]|0;
        HEAP32[$pVdbe>>2] = $5174;
        while(1) {
         $5175 = HEAP32[$pVdbe>>2]|0;
         $5176 = ($5175|0)!=(0|0);
         if (!($5176)) {
          break;
         }
         $5177 = HEAP32[$pVdbe>>2]|0;
         $5178 = (($5177) + 40|0);
         $5179 = HEAP32[$5178>>2]|0;
         $5180 = ($5179|0)==(-1108210269);
         if ($5180) {
          $5181 = HEAP32[$pVdbe>>2]|0;
          $5182 = (($5181) + 88|0);
          $5183 = HEAP16[$5182>>1]|0;
          $5184 = ($5183&65535) >>> 9;
          $5185 = $5184 & 1;
          $5186 = $5185&65535;
          $5187 = ($5186|0)!=(0);
          if ($5187) {
           $5188 = HEAP32[$pVdbe>>2]|0;
           $5189 = (($5188) + 88|0);
           $5190 = HEAP16[$5189>>1]|0;
           $5191 = ($5190&65535) >>> 2;
           $5192 = $5191 & 3;
           $5193 = $5192&65535;
           $5194 = ($5193|0)<(2);
           if ($5194) {
            $5195 = HEAP32[$pVdbe>>2]|0;
            $5196 = (($5195) + 76|0);
            $5197 = HEAP32[$5196>>2]|0;
            $5198 = ($5197|0)>=(0);
            if ($5198) {
             $5199 = HEAP32[$iCnt>>2]|0;
             $5200 = (($5199) + 1)|0;
             HEAP32[$iCnt>>2] = $5200;
            }
           }
          }
         }
         $5201 = HEAP32[$pVdbe>>2]|0;
         $5202 = (($5201) + 52|0);
         $5203 = HEAP32[$5202>>2]|0;
         HEAP32[$pVdbe>>2] = $5203;
        }
        $5204 = HEAP32[$pOut>>2]|0;
        $5205 = (($5204) + 8|0);
        HEAP16[$5205>>1] = 1;
        $5206 = HEAP32[$iCnt>>2]|0;
        $5207 = ($5206|0)>(1);
        if ($5207) {
         HEAP32[$rc>>2] = 6;
         $5208 = HEAP32[$0>>2]|0;
         $5209 = (($5208) + 86|0);
         HEAP8[$5209>>0] = 2;
         break L21;
        }
        $5210 = HEAP32[$pOp>>2]|0;
        $5211 = (($5210) + 12|0);
        $5212 = HEAP32[$5211>>2]|0;
        HEAP32[$iDb79>>2] = $5212;
        HEAP32[$iMoved>>2] = 0;
        $5213 = HEAP32[$iDb79>>2]|0;
        $5214 = HEAP32[$db>>2]|0;
        $5215 = (($5214) + 16|0);
        $5216 = HEAP32[$5215>>2]|0;
        $5217 = (($5216) + ($5213<<4)|0);
        $5218 = (($5217) + 4|0);
        $5219 = HEAP32[$5218>>2]|0;
        $5220 = HEAP32[$pOp>>2]|0;
        $5221 = (($5220) + 4|0);
        $5222 = HEAP32[$5221>>2]|0;
        $5223 = (_sqlite3BtreeDropTable($5219,$5222,$iMoved)|0);
        HEAP32[$rc>>2] = $5223;
        $5224 = HEAP32[$pOut>>2]|0;
        $5225 = (($5224) + 8|0);
        HEAP16[$5225>>1] = 4;
        $5226 = HEAP32[$iMoved>>2]|0;
        $5227 = ($5226|0)<(0);
        $5228 = $5227 << 31 >> 31;
        $5229 = HEAP32[$pOut>>2]|0;
        $5230 = $5229;
        $5231 = $5230;
        HEAP32[$5231>>2] = $5226;
        $5232 = (($5230) + 4)|0;
        $5233 = $5232;
        HEAP32[$5233>>2] = $5228;
        $5234 = HEAP32[$rc>>2]|0;
        $5235 = ($5234|0)==(0);
        $5236 = HEAP32[$iMoved>>2]|0;
        $5237 = ($5236|0)!=(0);
        $or$cond22 = $5235 & $5237;
        if ($or$cond22) {
         $5238 = HEAP32[$db>>2]|0;
         $5239 = HEAP32[$iDb79>>2]|0;
         $5240 = HEAP32[$iMoved>>2]|0;
         $5241 = HEAP32[$pOp>>2]|0;
         $5242 = (($5241) + 4|0);
         $5243 = HEAP32[$5242>>2]|0;
         _sqlite3RootPageMoved($5238,$5239,$5240,$5243);
         $5244 = HEAP32[$iDb79>>2]|0;
         $5245 = (($5244) + 1)|0;
         $5246 = $5245&255;
         HEAP8[$resetSchemaOnFault>>0] = $5246;
        }
        break;
       }
       case 119:  {
        $5297 = HEAP32[$pOp>>2]|0;
        $5298 = (($5297) + 4|0);
        $5299 = HEAP32[$5298>>2]|0;
        $5300 = HEAP32[$0>>2]|0;
        $5301 = (($5300) + 56|0);
        $5302 = HEAP32[$5301>>2]|0;
        $5303 = (($5302) + ($5299<<2)|0);
        $5304 = HEAP32[$5303>>2]|0;
        HEAP32[$pC80>>2] = $5304;
        $5305 = HEAP32[$pC80>>2]|0;
        $5306 = (($5305) + 56|0);
        $5307 = HEAP32[$5306>>2]|0;
        $5308 = ($5307|0)!=(0|0);
        if ($5308) {
         $5309 = HEAP32[$db>>2]|0;
         $5310 = HEAP32[$pC80>>2]|0;
         $5311 = (($5310) + 56|0);
         $5312 = HEAP32[$5311>>2]|0;
         _sqlite3VdbeSorterReset($5309,$5312);
         break L21;
        } else {
         $5313 = HEAP32[$pC80>>2]|0;
         $5314 = HEAP32[$5313>>2]|0;
         $5315 = (_sqlite3BtreeClearTableOfCursor($5314)|0);
         HEAP32[$rc>>2] = $5315;
         break L21;
        }
        break;
       }
       case 118:  {
        HEAP32[$nChange>>2] = 0;
        $5247 = HEAP32[$pOp>>2]|0;
        $5248 = (($5247) + 8|0);
        $5249 = HEAP32[$5248>>2]|0;
        $5250 = HEAP32[$db>>2]|0;
        $5251 = (($5250) + 16|0);
        $5252 = HEAP32[$5251>>2]|0;
        $5253 = (($5252) + ($5249<<4)|0);
        $5254 = (($5253) + 4|0);
        $5255 = HEAP32[$5254>>2]|0;
        $5256 = HEAP32[$pOp>>2]|0;
        $5257 = (($5256) + 4|0);
        $5258 = HEAP32[$5257>>2]|0;
        $5259 = HEAP32[$pOp>>2]|0;
        $5260 = (($5259) + 12|0);
        $5261 = HEAP32[$5260>>2]|0;
        $5262 = ($5261|0)!=(0);
        $nChange$ = $5262 ? $nChange : 0;
        $5263 = (_sqlite3BtreeClearTable($5255,$5258,$nChange$)|0);
        HEAP32[$rc>>2] = $5263;
        $5264 = HEAP32[$pOp>>2]|0;
        $5265 = (($5264) + 12|0);
        $5266 = HEAP32[$5265>>2]|0;
        $5267 = ($5266|0)!=(0);
        if ($5267) {
         $5268 = HEAP32[$nChange>>2]|0;
         $5269 = HEAP32[$0>>2]|0;
         $5270 = (($5269) + 92|0);
         $5271 = HEAP32[$5270>>2]|0;
         $5272 = (($5271) + ($5268))|0;
         HEAP32[$5270>>2] = $5272;
         $5273 = HEAP32[$pOp>>2]|0;
         $5274 = (($5273) + 12|0);
         $5275 = HEAP32[$5274>>2]|0;
         $5276 = ($5275|0)>(0);
         if ($5276) {
          $5277 = HEAP32[$nChange>>2]|0;
          $5278 = ($5277|0)<(0);
          $5279 = $5278 << 31 >> 31;
          $5280 = HEAP32[$pOp>>2]|0;
          $5281 = (($5280) + 12|0);
          $5282 = HEAP32[$5281>>2]|0;
          $5283 = HEAP32[$aMem>>2]|0;
          $5284 = (($5283) + (($5282*40)|0)|0);
          $5285 = $5284;
          $5286 = $5285;
          $5287 = HEAP32[$5286>>2]|0;
          $5288 = (($5285) + 4)|0;
          $5289 = $5288;
          $5290 = HEAP32[$5289>>2]|0;
          $5291 = (_i64Add(($5287|0),($5290|0),($5277|0),($5279|0))|0);
          $5292 = tempRet0;
          $5293 = $5284;
          $5294 = $5293;
          HEAP32[$5294>>2] = $5291;
          $5295 = (($5293) + 4)|0;
          $5296 = $5295;
          HEAP32[$5296>>2] = $5292;
         }
        }
        break;
       }
       case 121: case 120:  {
        HEAP32[$pgno81>>2] = 0;
        $5316 = HEAP32[$pOp>>2]|0;
        $5317 = (($5316) + 4|0);
        $5318 = HEAP32[$5317>>2]|0;
        $5319 = HEAP32[$db>>2]|0;
        $5320 = (($5319) + 16|0);
        $5321 = HEAP32[$5320>>2]|0;
        $5322 = (($5321) + ($5318<<4)|0);
        HEAP32[$pDb83>>2] = $5322;
        $5323 = HEAP32[$pOp>>2]|0;
        $5324 = HEAP8[$5323>>0]|0;
        $5325 = $5324&255;
        $5326 = ($5325|0)==(121);
        if ($5326) {
         HEAP32[$flags82>>2] = 1;
        } else {
         HEAP32[$flags82>>2] = 2;
        }
        $5327 = HEAP32[$pDb83>>2]|0;
        $5328 = (($5327) + 4|0);
        $5329 = HEAP32[$5328>>2]|0;
        $5330 = HEAP32[$flags82>>2]|0;
        $5331 = (_sqlite3BtreeCreateTable($5329,$pgno81,$5330)|0);
        HEAP32[$rc>>2] = $5331;
        $5332 = HEAP32[$pgno81>>2]|0;
        $5333 = ($5332|0)<(0);
        $5334 = $5333 << 31 >> 31;
        $5335 = HEAP32[$pOut>>2]|0;
        $5336 = $5335;
        $5337 = $5336;
        HEAP32[$5337>>2] = $5332;
        $5338 = (($5336) + 4)|0;
        $5339 = $5338;
        HEAP32[$5339>>2] = $5334;
        break;
       }
       case 123:  {
        $5389 = HEAP32[$db>>2]|0;
        $5390 = HEAP32[$pOp>>2]|0;
        $5391 = (($5390) + 4|0);
        $5392 = HEAP32[$5391>>2]|0;
        $5393 = (_sqlite3AnalysisLoad($5389,$5392)|0);
        HEAP32[$rc>>2] = $5393;
        break;
       }
       case 124:  {
        $5394 = HEAP32[$db>>2]|0;
        $5395 = HEAP32[$pOp>>2]|0;
        $5396 = (($5395) + 4|0);
        $5397 = HEAP32[$5396>>2]|0;
        $5398 = HEAP32[$pOp>>2]|0;
        $5399 = (($5398) + 16|0);
        $5400 = HEAP32[$5399>>2]|0;
        _sqlite3UnlinkAndDeleteTable($5394,$5397,$5400);
        break;
       }
       case 122:  {
        $5340 = HEAP32[$pOp>>2]|0;
        $5341 = (($5340) + 4|0);
        $5342 = HEAP32[$5341>>2]|0;
        HEAP32[$iDb84>>2] = $5342;
        $5343 = HEAP32[$iDb84>>2]|0;
        $5344 = ($5343|0)==(1);
        $5345 = $5344 ? 14904 : 14928;
        HEAP32[$zMaster>>2] = $5345;
        $5346 = HEAP32[$db>>2]|0;
        HEAP32[$initData>>2] = $5346;
        $5347 = HEAP32[$pOp>>2]|0;
        $5348 = (($5347) + 4|0);
        $5349 = HEAP32[$5348>>2]|0;
        $5350 = (($initData) + 8|0);
        HEAP32[$5350>>2] = $5349;
        $5351 = HEAP32[$0>>2]|0;
        $5352 = (($5351) + 44|0);
        $5353 = (($initData) + 4|0);
        HEAP32[$5353>>2] = $5352;
        $5354 = HEAP32[$db>>2]|0;
        $5355 = HEAP32[$iDb84>>2]|0;
        $5356 = HEAP32[$db>>2]|0;
        $5357 = (($5356) + 16|0);
        $5358 = HEAP32[$5357>>2]|0;
        $5359 = (($5358) + ($5355<<4)|0);
        $5360 = HEAP32[$5359>>2]|0;
        $5361 = HEAP32[$zMaster>>2]|0;
        $5362 = HEAP32[$pOp>>2]|0;
        $5363 = (($5362) + 16|0);
        $5364 = HEAP32[$5363>>2]|0;
        HEAP32[$vararg_buffer84>>2] = $5360;
        $vararg_ptr56 = (($vararg_buffer84) + 4|0);
        HEAP32[$vararg_ptr56>>2] = $5361;
        $vararg_ptr57 = (($vararg_buffer84) + 8|0);
        HEAP32[$vararg_ptr57>>2] = $5364;
        $5365 = (_sqlite3MPrintf($5354,31776,$vararg_buffer84)|0);
        HEAP32[$zSql>>2] = $5365;
        $5366 = HEAP32[$zSql>>2]|0;
        $5367 = ($5366|0)==(0|0);
        if ($5367) {
         HEAP32[$rc>>2] = 7;
        } else {
         $5368 = HEAP32[$db>>2]|0;
         $5369 = (($5368) + 140|0);
         $5370 = (($5369) + 5|0);
         HEAP8[$5370>>0] = 1;
         $5371 = (($initData) + 12|0);
         HEAP32[$5371>>2] = 0;
         $5372 = HEAP32[$db>>2]|0;
         $5373 = HEAP32[$zSql>>2]|0;
         $5374 = (_sqlite3_exec($5372,$5373,6,$initData,0)|0);
         HEAP32[$rc>>2] = $5374;
         $5375 = HEAP32[$rc>>2]|0;
         $5376 = ($5375|0)==(0);
         if ($5376) {
          $5377 = (($initData) + 12|0);
          $5378 = HEAP32[$5377>>2]|0;
          HEAP32[$rc>>2] = $5378;
         }
         $5379 = HEAP32[$db>>2]|0;
         $5380 = HEAP32[$zSql>>2]|0;
         _sqlite3DbFree($5379,$5380);
         $5381 = HEAP32[$db>>2]|0;
         $5382 = (($5381) + 140|0);
         $5383 = (($5382) + 5|0);
         HEAP8[$5383>>0] = 0;
        }
        $5384 = HEAP32[$rc>>2]|0;
        $5385 = ($5384|0)!=(0);
        if ($5385) {
         $5386 = HEAP32[$db>>2]|0;
         _sqlite3ResetAllSchemasOfConnection($5386);
        }
        $5387 = HEAP32[$rc>>2]|0;
        $5388 = ($5387|0)==(7);
        if ($5388) {
         label = 871;
         break L1;
        }
        break;
       }
       case 126:  {
        $5408 = HEAP32[$db>>2]|0;
        $5409 = HEAP32[$pOp>>2]|0;
        $5410 = (($5409) + 4|0);
        $5411 = HEAP32[$5410>>2]|0;
        $5412 = HEAP32[$pOp>>2]|0;
        $5413 = (($5412) + 16|0);
        $5414 = HEAP32[$5413>>2]|0;
        _sqlite3UnlinkAndDeleteTrigger($5408,$5411,$5414);
        break;
       }
       case 127:  {
        $5415 = HEAP32[$pOp>>2]|0;
        $5416 = (($5415) + 8|0);
        $5417 = HEAP32[$5416>>2]|0;
        HEAP32[$nRoot>>2] = $5417;
        $5418 = HEAP32[$db>>2]|0;
        $5419 = HEAP32[$nRoot>>2]|0;
        $5420 = (($5419) + 1)|0;
        $5421 = $5420<<2;
        $5422 = (_sqlite3DbMallocRaw($5418,$5421,0)|0);
        HEAP32[$aRoot>>2] = $5422;
        $5423 = HEAP32[$aRoot>>2]|0;
        $5424 = ($5423|0)==(0|0);
        if ($5424) {
         label = 871;
         break L1;
        }
        $5425 = HEAP32[$pOp>>2]|0;
        $5426 = (($5425) + 12|0);
        $5427 = HEAP32[$5426>>2]|0;
        $5428 = HEAP32[$aMem>>2]|0;
        $5429 = (($5428) + (($5427*40)|0)|0);
        HEAP32[$pnErr>>2] = $5429;
        $5430 = HEAP32[$pOp>>2]|0;
        $5431 = (($5430) + 4|0);
        $5432 = HEAP32[$5431>>2]|0;
        $5433 = HEAP32[$aMem>>2]|0;
        $5434 = (($5433) + (($5432*40)|0)|0);
        HEAP32[$pIn1>>2] = $5434;
        HEAP32[$j85>>2] = 0;
        while(1) {
         $5435 = HEAP32[$j85>>2]|0;
         $5436 = HEAP32[$nRoot>>2]|0;
         $5437 = ($5435|0)<($5436|0);
         $5438 = HEAP32[$j85>>2]|0;
         if (!($5437)) {
          break;
         }
         $5439 = HEAP32[$pIn1>>2]|0;
         $5440 = (($5439) + (($5438*40)|0)|0);
         $5441 = (_sqlite3VdbeIntValue($5440)|0);
         $5442 = tempRet0;
         $5443 = HEAP32[$j85>>2]|0;
         $5444 = HEAP32[$aRoot>>2]|0;
         $5445 = (($5444) + ($5443<<2)|0);
         HEAP32[$5445>>2] = $5441;
         $5446 = HEAP32[$j85>>2]|0;
         $5447 = (($5446) + 1)|0;
         HEAP32[$j85>>2] = $5447;
        }
        $5448 = HEAP32[$aRoot>>2]|0;
        $5449 = (($5448) + ($5438<<2)|0);
        HEAP32[$5449>>2] = 0;
        $5450 = HEAP32[$pOp>>2]|0;
        $5451 = (($5450) + 3|0);
        $5452 = HEAP8[$5451>>0]|0;
        $5453 = $5452&255;
        $5454 = HEAP32[$db>>2]|0;
        $5455 = (($5454) + 16|0);
        $5456 = HEAP32[$5455>>2]|0;
        $5457 = (($5456) + ($5453<<4)|0);
        $5458 = (($5457) + 4|0);
        $5459 = HEAP32[$5458>>2]|0;
        $5460 = HEAP32[$aRoot>>2]|0;
        $5461 = HEAP32[$nRoot>>2]|0;
        $5462 = HEAP32[$pnErr>>2]|0;
        $5463 = $5462;
        $5464 = $5463;
        $5465 = HEAP32[$5464>>2]|0;
        $5466 = (($5463) + 4)|0;
        $5467 = $5466;
        $5468 = HEAP32[$5467>>2]|0;
        $5469 = (_sqlite3BtreeIntegrityCheck($5459,$5460,$5461,$5465,$nErr)|0);
        HEAP32[$z>>2] = $5469;
        $5470 = HEAP32[$db>>2]|0;
        $5471 = HEAP32[$aRoot>>2]|0;
        _sqlite3DbFree($5470,$5471);
        $5472 = HEAP32[$nErr>>2]|0;
        $5473 = ($5472|0)<(0);
        $5474 = $5473 << 31 >> 31;
        $5475 = HEAP32[$pnErr>>2]|0;
        $5476 = $5475;
        $5477 = $5476;
        $5478 = HEAP32[$5477>>2]|0;
        $5479 = (($5476) + 4)|0;
        $5480 = $5479;
        $5481 = HEAP32[$5480>>2]|0;
        $5482 = (_i64Subtract(($5478|0),($5481|0),($5472|0),($5474|0))|0);
        $5483 = tempRet0;
        $5484 = $5475;
        $5485 = $5484;
        HEAP32[$5485>>2] = $5482;
        $5486 = (($5484) + 4)|0;
        $5487 = $5486;
        HEAP32[$5487>>2] = $5483;
        $5488 = HEAP32[$pIn1>>2]|0;
        _sqlite3VdbeMemSetNull($5488);
        $5489 = HEAP32[$nErr>>2]|0;
        $5490 = ($5489|0)==(0);
        if (!($5490)) {
         $5491 = HEAP32[$z>>2]|0;
         $5492 = ($5491|0)==(0|0);
         if ($5492) {
          label = 871;
          break L1;
         }
         $5493 = HEAP32[$pIn1>>2]|0;
         $5494 = HEAP32[$z>>2]|0;
         (_sqlite3VdbeMemSetStr($5493,$5494,-1,1,13)|0);
        }
        $5495 = HEAP32[$pIn1>>2]|0;
        $5496 = HEAP8[$encoding>>0]|0;
        $5497 = $5496&255;
        (_sqlite3VdbeChangeEncoding($5495,$5497)|0);
        break;
       }
       case 125:  {
        $5401 = HEAP32[$db>>2]|0;
        $5402 = HEAP32[$pOp>>2]|0;
        $5403 = (($5402) + 4|0);
        $5404 = HEAP32[$5403>>2]|0;
        $5405 = HEAP32[$pOp>>2]|0;
        $5406 = (($5405) + 16|0);
        $5407 = HEAP32[$5406>>2]|0;
        _sqlite3UnlinkAndDeleteIndex($5401,$5404,$5407);
        break;
       }
       case 128:  {
        $5498 = HEAP32[$pOp>>2]|0;
        $5499 = (($5498) + 4|0);
        $5500 = HEAP32[$5499>>2]|0;
        $5501 = HEAP32[$aMem>>2]|0;
        $5502 = (($5501) + (($5500*40)|0)|0);
        HEAP32[$pIn1>>2] = $5502;
        $5503 = HEAP32[$pOp>>2]|0;
        $5504 = (($5503) + 8|0);
        $5505 = HEAP32[$5504>>2]|0;
        $5506 = HEAP32[$aMem>>2]|0;
        $5507 = (($5506) + (($5505*40)|0)|0);
        HEAP32[$pIn2>>2] = $5507;
        $5508 = HEAP32[$pIn1>>2]|0;
        $5509 = (($5508) + 8|0);
        $5510 = HEAP16[$5509>>1]|0;
        $5511 = $5510&65535;
        $5512 = $5511 & 32;
        $5513 = ($5512|0)==(0);
        if ($5513) {
         $5514 = HEAP32[$pIn1>>2]|0;
         _sqlite3VdbeMemSetRowSet($5514);
         $5515 = HEAP32[$pIn1>>2]|0;
         $5516 = (($5515) + 8|0);
         $5517 = HEAP16[$5516>>1]|0;
         $5518 = $5517&65535;
         $5519 = $5518 & 32;
         $5520 = ($5519|0)==(0);
         if ($5520) {
          label = 871;
          break L1;
         }
        }
        $5521 = HEAP32[$pIn1>>2]|0;
        $5522 = HEAP32[$5521>>2]|0;
        $5523 = HEAP32[$pIn2>>2]|0;
        $5524 = $5523;
        $5525 = $5524;
        $5526 = HEAP32[$5525>>2]|0;
        $5527 = (($5524) + 4)|0;
        $5528 = $5527;
        $5529 = HEAP32[$5528>>2]|0;
        _sqlite3RowSetInsert($5522,$5526,$5529);
        break;
       }
       case 129:  {
        $5530 = HEAP32[$pOp>>2]|0;
        $5531 = (($5530) + 4|0);
        $5532 = HEAP32[$5531>>2]|0;
        $5533 = HEAP32[$aMem>>2]|0;
        $5534 = (($5533) + (($5532*40)|0)|0);
        HEAP32[$pIn1>>2] = $5534;
        $5535 = HEAP32[$pIn1>>2]|0;
        $5536 = (($5535) + 8|0);
        $5537 = HEAP16[$5536>>1]|0;
        $5538 = $5537&65535;
        $5539 = $5538 & 32;
        $5540 = ($5539|0)==(0);
        if (!($5540)) {
         $5541 = HEAP32[$pIn1>>2]|0;
         $5542 = HEAP32[$5541>>2]|0;
         $5543 = (_sqlite3RowSetNext($5542,$val)|0);
         $5544 = ($5543|0)==(0);
         if (!($5544)) {
          $5550 = HEAP32[$pOp>>2]|0;
          $5551 = (($5550) + 12|0);
          $5552 = HEAP32[$5551>>2]|0;
          $5553 = HEAP32[$aMem>>2]|0;
          $5554 = (($5553) + (($5552*40)|0)|0);
          $5555 = $val;
          $5556 = $5555;
          $5557 = HEAP32[$5556>>2]|0;
          $5558 = (($5555) + 4)|0;
          $5559 = $5558;
          $5560 = HEAP32[$5559>>2]|0;
          _sqlite3VdbeMemSetInt64($5554,$5557,$5560);
          label = 16;
          break L21;
         }
        }
        $5545 = HEAP32[$pIn1>>2]|0;
        _sqlite3VdbeMemSetNull($5545);
        $5546 = HEAP32[$pOp>>2]|0;
        $5547 = (($5546) + 8|0);
        $5548 = HEAP32[$5547>>2]|0;
        $5549 = (($5548) - 1)|0;
        HEAP32[$pc>>2] = $5549;
        label = 16;
        break;
       }
       case 130:  {
        $5561 = HEAP32[$pOp>>2]|0;
        $5562 = (($5561) + 4|0);
        $5563 = HEAP32[$5562>>2]|0;
        $5564 = HEAP32[$aMem>>2]|0;
        $5565 = (($5564) + (($5563*40)|0)|0);
        HEAP32[$pIn1>>2] = $5565;
        $5566 = HEAP32[$pOp>>2]|0;
        $5567 = (($5566) + 12|0);
        $5568 = HEAP32[$5567>>2]|0;
        $5569 = HEAP32[$aMem>>2]|0;
        $5570 = (($5569) + (($5568*40)|0)|0);
        HEAP32[$pIn3>>2] = $5570;
        $5571 = HEAP32[$pOp>>2]|0;
        $5572 = (($5571) + 16|0);
        $5573 = HEAP32[$5572>>2]|0;
        HEAP32[$iSet>>2] = $5573;
        $5574 = HEAP32[$pIn1>>2]|0;
        $5575 = (($5574) + 8|0);
        $5576 = HEAP16[$5575>>1]|0;
        $5577 = $5576&65535;
        $5578 = $5577 & 32;
        $5579 = ($5578|0)==(0);
        if ($5579) {
         $5580 = HEAP32[$pIn1>>2]|0;
         _sqlite3VdbeMemSetRowSet($5580);
         $5581 = HEAP32[$pIn1>>2]|0;
         $5582 = (($5581) + 8|0);
         $5583 = HEAP16[$5582>>1]|0;
         $5584 = $5583&65535;
         $5585 = $5584 & 32;
         $5586 = ($5585|0)==(0);
         if ($5586) {
          label = 871;
          break L1;
         }
        }
        $5587 = HEAP32[$iSet>>2]|0;
        $5588 = ($5587|0)!=(0);
        if ($5588) {
         $5589 = HEAP32[$pIn1>>2]|0;
         $5590 = HEAP32[$5589>>2]|0;
         $5591 = HEAP32[$iSet>>2]|0;
         $5592 = HEAP32[$pIn3>>2]|0;
         $5593 = $5592;
         $5594 = $5593;
         $5595 = HEAP32[$5594>>2]|0;
         $5596 = (($5593) + 4)|0;
         $5597 = $5596;
         $5598 = HEAP32[$5597>>2]|0;
         $5599 = (_sqlite3RowSetTest($5590,$5591,$5595,$5598)|0);
         HEAP32[$exists>>2] = $5599;
         $5600 = HEAP32[$exists>>2]|0;
         $5601 = ($5600|0)!=(0);
         if ($5601) {
          $5602 = HEAP32[$pOp>>2]|0;
          $5603 = (($5602) + 8|0);
          $5604 = HEAP32[$5603>>2]|0;
          $5605 = (($5604) - 1)|0;
          HEAP32[$pc>>2] = $5605;
          break L21;
         }
        }
        $5606 = HEAP32[$iSet>>2]|0;
        $5607 = ($5606|0)>=(0);
        if ($5607) {
         $5608 = HEAP32[$pIn1>>2]|0;
         $5609 = HEAP32[$5608>>2]|0;
         $5610 = HEAP32[$pIn3>>2]|0;
         $5611 = $5610;
         $5612 = $5611;
         $5613 = HEAP32[$5612>>2]|0;
         $5614 = (($5611) + 4)|0;
         $5615 = $5614;
         $5616 = HEAP32[$5615>>2]|0;
         _sqlite3RowSetInsert($5609,$5613,$5616);
        }
        break;
       }
       case 131:  {
        $5617 = HEAP32[$pOp>>2]|0;
        $5618 = (($5617) + 16|0);
        $5619 = HEAP32[$5618>>2]|0;
        HEAP32[$pProgram>>2] = $5619;
        $5620 = HEAP32[$pOp>>2]|0;
        $5621 = (($5620) + 12|0);
        $5622 = HEAP32[$5621>>2]|0;
        $5623 = HEAP32[$aMem>>2]|0;
        $5624 = (($5623) + (($5622*40)|0)|0);
        HEAP32[$pRt>>2] = $5624;
        $5625 = HEAP32[$pOp>>2]|0;
        $5626 = (($5625) + 3|0);
        $5627 = HEAP8[$5626>>0]|0;
        $5628 = ($5627<<24>>24)!=(0);
        if ($5628) {
         $5629 = HEAP32[$pProgram>>2]|0;
         $5630 = (($5629) + 20|0);
         $5631 = HEAP32[$5630>>2]|0;
         HEAP32[$t89>>2] = $5631;
         $5632 = HEAP32[$0>>2]|0;
         $5633 = (($5632) + 176|0);
         $5634 = HEAP32[$5633>>2]|0;
         HEAP32[$pFrame88>>2] = $5634;
         while(1) {
          $5635 = HEAP32[$pFrame88>>2]|0;
          $5636 = ($5635|0)!=(0|0);
          if (!($5636)) {
           break;
          }
          $5637 = HEAP32[$pFrame88>>2]|0;
          $5638 = (($5637) + 24|0);
          $5639 = HEAP32[$5638>>2]|0;
          $5640 = HEAP32[$t89>>2]|0;
          $5641 = ($5639|0)!=($5640|0);
          if (!($5641)) {
           break;
          }
          $5642 = HEAP32[$pFrame88>>2]|0;
          $5643 = (($5642) + 4|0);
          $5644 = HEAP32[$5643>>2]|0;
          HEAP32[$pFrame88>>2] = $5644;
         }
         $5645 = HEAP32[$pFrame88>>2]|0;
         $5646 = ($5645|0)!=(0|0);
         if ($5646) {
          break L21;
         }
        }
        $5647 = HEAP32[$0>>2]|0;
        $5648 = (($5647) + 184|0);
        $5649 = HEAP32[$5648>>2]|0;
        $5650 = HEAP32[$db>>2]|0;
        $5651 = (($5650) + 88|0);
        $5652 = (($5651) + 40|0);
        $5653 = HEAP32[$5652>>2]|0;
        $5654 = ($5649|0)>=($5653|0);
        if ($5654) {
         HEAP32[$rc>>2] = 1;
         $5655 = HEAP32[$0>>2]|0;
         $5656 = (($5655) + 44|0);
         $5657 = HEAP32[$db>>2]|0;
         _sqlite3SetString($5656,$5657,31840,$vararg_buffer84);
         break L21;
        }
        $5658 = HEAP32[$pRt>>2]|0;
        $5659 = (($5658) + 8|0);
        $5660 = HEAP16[$5659>>1]|0;
        $5661 = $5660&65535;
        $5662 = $5661 & 64;
        $5663 = ($5662|0)==(0);
        L845: do {
         if ($5663) {
          $5664 = HEAP32[$pProgram>>2]|0;
          $5665 = (($5664) + 8|0);
          $5666 = HEAP32[$5665>>2]|0;
          $5667 = HEAP32[$pProgram>>2]|0;
          $5668 = (($5667) + 12|0);
          $5669 = HEAP32[$5668>>2]|0;
          $5670 = (($5666) + ($5669))|0;
          HEAP32[$nMem>>2] = $5670;
          $5671 = HEAP32[$nMem>>2]|0;
          $5672 = ($5671*40)|0;
          $5673 = (72 + ($5672))|0;
          $5674 = HEAP32[$pProgram>>2]|0;
          $5675 = (($5674) + 12|0);
          $5676 = HEAP32[$5675>>2]|0;
          $5677 = $5676<<2;
          $5678 = (($5673) + ($5677))|0;
          $5679 = HEAP32[$pProgram>>2]|0;
          $5680 = (($5679) + 16|0);
          $5681 = HEAP32[$5680>>2]|0;
          $5682 = $5681;
          $5683 = (($5678) + ($5682))|0;
          HEAP32[$nByte86>>2] = $5683;
          $5684 = HEAP32[$db>>2]|0;
          $5685 = HEAP32[$nByte86>>2]|0;
          $5686 = ($5685|0)<(0);
          $5687 = $5686 << 31 >> 31;
          $5688 = (_sqlite3DbMallocZero($5684,$5685,$5687)|0);
          HEAP32[$pFrame88>>2] = $5688;
          $5689 = HEAP32[$pFrame88>>2]|0;
          $5690 = ($5689|0)!=(0|0);
          if (!($5690)) {
           label = 871;
           break L1;
          }
          $5691 = HEAP32[$pRt>>2]|0;
          _sqlite3VdbeMemRelease($5691);
          $5692 = HEAP32[$pRt>>2]|0;
          $5693 = (($5692) + 8|0);
          HEAP16[$5693>>1] = 64;
          $5694 = HEAP32[$pFrame88>>2]|0;
          $5695 = HEAP32[$pRt>>2]|0;
          HEAP32[$5695>>2] = $5694;
          $5696 = HEAP32[$0>>2]|0;
          $5697 = HEAP32[$pFrame88>>2]|0;
          HEAP32[$5697>>2] = $5696;
          $5698 = HEAP32[$nMem>>2]|0;
          $5699 = HEAP32[$pFrame88>>2]|0;
          $5700 = (($5699) + 60|0);
          HEAP32[$5700>>2] = $5698;
          $5701 = HEAP32[$pProgram>>2]|0;
          $5702 = (($5701) + 12|0);
          $5703 = HEAP32[$5702>>2]|0;
          $5704 = HEAP32[$pFrame88>>2]|0;
          $5705 = (($5704) + 64|0);
          HEAP32[$5705>>2] = $5703;
          $5706 = HEAP32[$pc>>2]|0;
          $5707 = HEAP32[$pFrame88>>2]|0;
          $5708 = (($5707) + 44|0);
          HEAP32[$5708>>2] = $5706;
          $5709 = HEAP32[$0>>2]|0;
          $5710 = (($5709) + 8|0);
          $5711 = HEAP32[$5710>>2]|0;
          $5712 = HEAP32[$pFrame88>>2]|0;
          $5713 = (($5712) + 12|0);
          HEAP32[$5713>>2] = $5711;
          $5714 = HEAP32[$0>>2]|0;
          $5715 = (($5714) + 28|0);
          $5716 = HEAP32[$5715>>2]|0;
          $5717 = HEAP32[$pFrame88>>2]|0;
          $5718 = (($5717) + 52|0);
          HEAP32[$5718>>2] = $5716;
          $5719 = HEAP32[$0>>2]|0;
          $5720 = (($5719) + 56|0);
          $5721 = HEAP32[$5720>>2]|0;
          $5722 = HEAP32[$pFrame88>>2]|0;
          $5723 = (($5722) + 20|0);
          HEAP32[$5723>>2] = $5721;
          $5724 = HEAP32[$0>>2]|0;
          $5725 = (($5724) + 36|0);
          $5726 = HEAP32[$5725>>2]|0;
          $5727 = HEAP32[$pFrame88>>2]|0;
          $5728 = (($5727) + 40|0);
          HEAP32[$5728>>2] = $5726;
          $5729 = HEAP32[$0>>2]|0;
          $5730 = (($5729) + 4|0);
          $5731 = HEAP32[$5730>>2]|0;
          $5732 = HEAP32[$pFrame88>>2]|0;
          $5733 = (($5732) + 8|0);
          HEAP32[$5733>>2] = $5731;
          $5734 = HEAP32[$0>>2]|0;
          $5735 = (($5734) + 32|0);
          $5736 = HEAP32[$5735>>2]|0;
          $5737 = HEAP32[$pFrame88>>2]|0;
          $5738 = (($5737) + 48|0);
          HEAP32[$5738>>2] = $5736;
          $5739 = HEAP32[$pProgram>>2]|0;
          $5740 = (($5739) + 20|0);
          $5741 = HEAP32[$5740>>2]|0;
          $5742 = HEAP32[$pFrame88>>2]|0;
          $5743 = (($5742) + 24|0);
          HEAP32[$5743>>2] = $5741;
          $5744 = HEAP32[$0>>2]|0;
          $5745 = (($5744) + 200|0);
          $5746 = HEAP32[$5745>>2]|0;
          $5747 = HEAP32[$pFrame88>>2]|0;
          $5748 = (($5747) + 16|0);
          HEAP32[$5748>>2] = $5746;
          $5749 = HEAP32[$0>>2]|0;
          $5750 = (($5749) + 196|0);
          $5751 = HEAP32[$5750>>2]|0;
          $5752 = HEAP32[$pFrame88>>2]|0;
          $5753 = (($5752) + 56|0);
          HEAP32[$5753>>2] = $5751;
          $5754 = HEAP32[$pFrame88>>2]|0;
          $5755 = (($5754) + 60|0);
          $5756 = HEAP32[$5755>>2]|0;
          $5757 = HEAP32[$pFrame88>>2]|0;
          $5758 = (($5757) + 72|0);
          $5759 = (($5758) + (($5756*40)|0)|0);
          HEAP32[$pEnd>>2] = $5759;
          $5760 = HEAP32[$pFrame88>>2]|0;
          $5761 = (($5760) + 72|0);
          HEAP32[$pMem87>>2] = $5761;
          while(1) {
           $5762 = HEAP32[$pMem87>>2]|0;
           $5763 = HEAP32[$pEnd>>2]|0;
           $5764 = ($5762|0)!=($5763|0);
           if (!($5764)) {
            break L845;
           }
           $5765 = HEAP32[$pMem87>>2]|0;
           $5766 = (($5765) + 8|0);
           HEAP16[$5766>>1] = 128;
           $5767 = HEAP32[$db>>2]|0;
           $5768 = HEAP32[$pMem87>>2]|0;
           $5769 = (($5768) + 32|0);
           HEAP32[$5769>>2] = $5767;
           $5770 = HEAP32[$pMem87>>2]|0;
           $5771 = (($5770) + 40|0);
           HEAP32[$pMem87>>2] = $5771;
          }
         } else {
          $5772 = HEAP32[$pRt>>2]|0;
          $5773 = HEAP32[$5772>>2]|0;
          HEAP32[$pFrame88>>2] = $5773;
         }
        } while(0);
        $5774 = HEAP32[$0>>2]|0;
        $5775 = (($5774) + 184|0);
        $5776 = HEAP32[$5775>>2]|0;
        $5777 = (($5776) + 1)|0;
        HEAP32[$5775>>2] = $5777;
        $5778 = HEAP32[$0>>2]|0;
        $5779 = (($5778) + 176|0);
        $5780 = HEAP32[$5779>>2]|0;
        $5781 = HEAP32[$pFrame88>>2]|0;
        $5782 = (($5781) + 4|0);
        HEAP32[$5782>>2] = $5780;
        $5783 = $lastRowid;
        $5784 = $5783;
        $5785 = HEAP32[$5784>>2]|0;
        $5786 = (($5783) + 4)|0;
        $5787 = $5786;
        $5788 = HEAP32[$5787>>2]|0;
        $5789 = HEAP32[$pFrame88>>2]|0;
        $5790 = (($5789) + 32|0);
        $5791 = $5790;
        $5792 = $5791;
        HEAP32[$5792>>2] = $5785;
        $5793 = (($5791) + 4)|0;
        $5794 = $5793;
        HEAP32[$5794>>2] = $5788;
        $5795 = HEAP32[$0>>2]|0;
        $5796 = (($5795) + 92|0);
        $5797 = HEAP32[$5796>>2]|0;
        $5798 = HEAP32[$pFrame88>>2]|0;
        $5799 = (($5798) + 68|0);
        HEAP32[$5799>>2] = $5797;
        $5800 = HEAP32[$0>>2]|0;
        $5801 = (($5800) + 92|0);
        HEAP32[$5801>>2] = 0;
        $5802 = HEAP32[$pFrame88>>2]|0;
        $5803 = HEAP32[$0>>2]|0;
        $5804 = (($5803) + 176|0);
        HEAP32[$5804>>2] = $5802;
        $5805 = HEAP32[$pFrame88>>2]|0;
        $5806 = (($5805) + 72|0);
        $5807 = (($5806) + -40|0);
        HEAP32[$aMem>>2] = $5807;
        $5808 = HEAP32[$0>>2]|0;
        $5809 = (($5808) + 8|0);
        HEAP32[$5809>>2] = $5807;
        $5810 = HEAP32[$pFrame88>>2]|0;
        $5811 = (($5810) + 60|0);
        $5812 = HEAP32[$5811>>2]|0;
        $5813 = HEAP32[$0>>2]|0;
        $5814 = (($5813) + 28|0);
        HEAP32[$5814>>2] = $5812;
        $5815 = HEAP32[$pFrame88>>2]|0;
        $5816 = (($5815) + 64|0);
        $5817 = HEAP32[$5816>>2]|0;
        $5818 = $5817&65535;
        $5819 = $5818&65535;
        $5820 = HEAP32[$0>>2]|0;
        $5821 = (($5820) + 36|0);
        HEAP32[$5821>>2] = $5819;
        $5822 = HEAP32[$0>>2]|0;
        $5823 = (($5822) + 28|0);
        $5824 = HEAP32[$5823>>2]|0;
        $5825 = (($5824) + 1)|0;
        $5826 = HEAP32[$aMem>>2]|0;
        $5827 = (($5826) + (($5825*40)|0)|0);
        $5828 = HEAP32[$0>>2]|0;
        $5829 = (($5828) + 56|0);
        HEAP32[$5829>>2] = $5827;
        $5830 = HEAP32[$pProgram>>2]|0;
        $5831 = HEAP32[$5830>>2]|0;
        HEAP32[$aOp>>2] = $5831;
        $5832 = HEAP32[$0>>2]|0;
        $5833 = (($5832) + 4|0);
        HEAP32[$5833>>2] = $5831;
        $5834 = HEAP32[$pProgram>>2]|0;
        $5835 = (($5834) + 4|0);
        $5836 = HEAP32[$5835>>2]|0;
        $5837 = HEAP32[$0>>2]|0;
        $5838 = (($5837) + 32|0);
        HEAP32[$5838>>2] = $5836;
        $5839 = HEAP32[$0>>2]|0;
        $5840 = (($5839) + 36|0);
        $5841 = HEAP32[$5840>>2]|0;
        $5842 = HEAP32[$0>>2]|0;
        $5843 = (($5842) + 56|0);
        $5844 = HEAP32[$5843>>2]|0;
        $5845 = (($5844) + ($5841<<2)|0);
        $5846 = HEAP32[$0>>2]|0;
        $5847 = (($5846) + 200|0);
        HEAP32[$5847>>2] = $5845;
        $5848 = HEAP32[$pProgram>>2]|0;
        $5849 = (($5848) + 16|0);
        $5850 = HEAP32[$5849>>2]|0;
        $5851 = HEAP32[$0>>2]|0;
        $5852 = (($5851) + 196|0);
        HEAP32[$5852>>2] = $5850;
        HEAP32[$pc>>2] = -1;
        $5853 = HEAP32[$0>>2]|0;
        $5854 = (($5853) + 200|0);
        $5855 = HEAP32[$5854>>2]|0;
        $5856 = HEAP32[$0>>2]|0;
        $5857 = (($5856) + 196|0);
        $5858 = HEAP32[$5857>>2]|0;
        _memset(($5855|0),0,($5858|0))|0;
        break;
       }
       case 132:  {
        $5859 = HEAP32[$0>>2]|0;
        $5860 = (($5859) + 176|0);
        $5861 = HEAP32[$5860>>2]|0;
        HEAP32[$pFrame90>>2] = $5861;
        $5862 = HEAP32[$pOp>>2]|0;
        $5863 = (($5862) + 4|0);
        $5864 = HEAP32[$5863>>2]|0;
        $5865 = HEAP32[$pFrame90>>2]|0;
        $5866 = (($5865) + 44|0);
        $5867 = HEAP32[$5866>>2]|0;
        $5868 = HEAP32[$pFrame90>>2]|0;
        $5869 = (($5868) + 8|0);
        $5870 = HEAP32[$5869>>2]|0;
        $5871 = (($5870) + (($5867*20)|0)|0);
        $5872 = (($5871) + 4|0);
        $5873 = HEAP32[$5872>>2]|0;
        $5874 = (($5864) + ($5873))|0;
        $5875 = HEAP32[$pFrame90>>2]|0;
        $5876 = (($5875) + 12|0);
        $5877 = HEAP32[$5876>>2]|0;
        $5878 = (($5877) + (($5874*40)|0)|0);
        HEAP32[$pIn>>2] = $5878;
        $5879 = HEAP32[$pOut>>2]|0;
        $5880 = HEAP32[$pIn>>2]|0;
        _sqlite3VdbeMemShallowCopy($5879,$5880,4096);
        break;
       }
       case 134:  {
        $5881 = HEAP32[$db>>2]|0;
        $5882 = (($5881) + 24|0);
        $5883 = HEAP32[$5882>>2]|0;
        $5884 = $5883 & 16777216;
        $5885 = ($5884|0)!=(0);
        $5886 = HEAP32[$pOp>>2]|0;
        if ($5885) {
         $5887 = (($5886) + 8|0);
         $5888 = HEAP32[$5887>>2]|0;
         $5889 = ($5888|0)<(0);
         $5890 = $5889 << 31 >> 31;
         $5891 = HEAP32[$db>>2]|0;
         $5892 = (($5891) + 512|0);
         $5893 = $5892;
         $5894 = $5893;
         $5895 = HEAP32[$5894>>2]|0;
         $5896 = (($5893) + 4)|0;
         $5897 = $5896;
         $5898 = HEAP32[$5897>>2]|0;
         $5899 = (_i64Add(($5895|0),($5898|0),($5888|0),($5890|0))|0);
         $5900 = tempRet0;
         $5901 = $5892;
         $5902 = $5901;
         HEAP32[$5902>>2] = $5899;
         $5903 = (($5901) + 4)|0;
         $5904 = $5903;
         HEAP32[$5904>>2] = $5900;
         break L21;
        }
        $5905 = (($5886) + 4|0);
        $5906 = HEAP32[$5905>>2]|0;
        $5907 = ($5906|0)!=(0);
        $5908 = HEAP32[$pOp>>2]|0;
        $5909 = (($5908) + 8|0);
        $5910 = HEAP32[$5909>>2]|0;
        $5911 = ($5910|0)<(0);
        $5912 = $5911 << 31 >> 31;
        if ($5907) {
         $5913 = HEAP32[$db>>2]|0;
         $5914 = (($5913) + 504|0);
         $5915 = $5914;
         $5916 = $5915;
         $5917 = HEAP32[$5916>>2]|0;
         $5918 = (($5915) + 4)|0;
         $5919 = $5918;
         $5920 = HEAP32[$5919>>2]|0;
         $5921 = (_i64Add(($5917|0),($5920|0),($5910|0),($5912|0))|0);
         $5922 = tempRet0;
         $5923 = $5914;
         $5924 = $5923;
         HEAP32[$5924>>2] = $5921;
         $5925 = (($5923) + 4)|0;
         $5926 = $5925;
         HEAP32[$5926>>2] = $5922;
         break L21;
        } else {
         $5927 = HEAP32[$0>>2]|0;
         $5928 = (($5927) + 144|0);
         $5929 = $5928;
         $5930 = $5929;
         $5931 = HEAP32[$5930>>2]|0;
         $5932 = (($5929) + 4)|0;
         $5933 = $5932;
         $5934 = HEAP32[$5933>>2]|0;
         $5935 = (_i64Add(($5931|0),($5934|0),($5910|0),($5912|0))|0);
         $5936 = tempRet0;
         $5937 = $5928;
         $5938 = $5937;
         HEAP32[$5938>>2] = $5935;
         $5939 = (($5937) + 4)|0;
         $5940 = $5939;
         HEAP32[$5940>>2] = $5936;
         break L21;
        }
        break;
       }
       case 135:  {
        $5941 = HEAP32[$pOp>>2]|0;
        $5942 = (($5941) + 4|0);
        $5943 = HEAP32[$5942>>2]|0;
        $5944 = ($5943|0)!=(0);
        if ($5944) {
         $5945 = HEAP32[$db>>2]|0;
         $5946 = (($5945) + 504|0);
         $5947 = $5946;
         $5948 = $5947;
         $5949 = HEAP32[$5948>>2]|0;
         $5950 = (($5947) + 4)|0;
         $5951 = $5950;
         $5952 = HEAP32[$5951>>2]|0;
         $5953 = ($5949|0)==(0);
         $5954 = ($5952|0)==(0);
         $5955 = $5953 & $5954;
         if (!($5955)) {
          break L21;
         }
         $5956 = HEAP32[$db>>2]|0;
         $5957 = (($5956) + 512|0);
         $5958 = $5957;
         $5959 = $5958;
         $5960 = HEAP32[$5959>>2]|0;
         $5961 = (($5958) + 4)|0;
         $5962 = $5961;
         $5963 = HEAP32[$5962>>2]|0;
         $5964 = ($5960|0)==(0);
         $5965 = ($5963|0)==(0);
         $5966 = $5964 & $5965;
         if (!($5966)) {
          break L21;
         }
         $5967 = HEAP32[$pOp>>2]|0;
         $5968 = (($5967) + 8|0);
         $5969 = HEAP32[$5968>>2]|0;
         $5970 = (($5969) - 1)|0;
         HEAP32[$pc>>2] = $5970;
         break L21;
        } else {
         $5971 = HEAP32[$0>>2]|0;
         $5972 = (($5971) + 144|0);
         $5973 = $5972;
         $5974 = $5973;
         $5975 = HEAP32[$5974>>2]|0;
         $5976 = (($5973) + 4)|0;
         $5977 = $5976;
         $5978 = HEAP32[$5977>>2]|0;
         $5979 = ($5975|0)==(0);
         $5980 = ($5978|0)==(0);
         $5981 = $5979 & $5980;
         if (!($5981)) {
          break L21;
         }
         $5982 = HEAP32[$db>>2]|0;
         $5983 = (($5982) + 512|0);
         $5984 = $5983;
         $5985 = $5984;
         $5986 = HEAP32[$5985>>2]|0;
         $5987 = (($5984) + 4)|0;
         $5988 = $5987;
         $5989 = HEAP32[$5988>>2]|0;
         $5990 = ($5986|0)==(0);
         $5991 = ($5989|0)==(0);
         $5992 = $5990 & $5991;
         if (!($5992)) {
          break L21;
         }
         $5993 = HEAP32[$pOp>>2]|0;
         $5994 = (($5993) + 8|0);
         $5995 = HEAP32[$5994>>2]|0;
         $5996 = (($5995) - 1)|0;
         HEAP32[$pc>>2] = $5996;
         break L21;
        }
        break;
       }
       case 136:  {
        $5997 = HEAP32[$0>>2]|0;
        $5998 = (($5997) + 176|0);
        $5999 = HEAP32[$5998>>2]|0;
        $6000 = ($5999|0)!=(0|0);
        if ($6000) {
         $6001 = HEAP32[$0>>2]|0;
         $6002 = (($6001) + 176|0);
         $6003 = HEAP32[$6002>>2]|0;
         HEAP32[$pFrame91>>2] = $6003;
         while(1) {
          $6004 = HEAP32[$pFrame91>>2]|0;
          $6005 = (($6004) + 4|0);
          $6006 = HEAP32[$6005>>2]|0;
          $6007 = ($6006|0)!=(0|0);
          if (!($6007)) {
           break;
          }
          $6008 = HEAP32[$pFrame91>>2]|0;
          $6009 = (($6008) + 4|0);
          $6010 = HEAP32[$6009>>2]|0;
          HEAP32[$pFrame91>>2] = $6010;
         }
         $6011 = HEAP32[$pOp>>2]|0;
         $6012 = (($6011) + 4|0);
         $6013 = HEAP32[$6012>>2]|0;
         $6014 = HEAP32[$pFrame91>>2]|0;
         $6015 = (($6014) + 12|0);
         $6016 = HEAP32[$6015>>2]|0;
         $6017 = (($6016) + (($6013*40)|0)|0);
         HEAP32[$pIn1>>2] = $6017;
        } else {
         $6018 = HEAP32[$pOp>>2]|0;
         $6019 = (($6018) + 4|0);
         $6020 = HEAP32[$6019>>2]|0;
         $6021 = HEAP32[$aMem>>2]|0;
         $6022 = (($6021) + (($6020*40)|0)|0);
         HEAP32[$pIn1>>2] = $6022;
        }
        $6023 = HEAP32[$pIn1>>2]|0;
        (_sqlite3VdbeMemIntegerify($6023)|0);
        $6024 = HEAP32[$pOp>>2]|0;
        $6025 = (($6024) + 8|0);
        $6026 = HEAP32[$6025>>2]|0;
        $6027 = HEAP32[$aMem>>2]|0;
        $6028 = (($6027) + (($6026*40)|0)|0);
        HEAP32[$pIn2>>2] = $6028;
        $6029 = HEAP32[$pIn2>>2]|0;
        (_sqlite3VdbeMemIntegerify($6029)|0);
        $6030 = HEAP32[$pIn1>>2]|0;
        $6031 = $6030;
        $6032 = $6031;
        $6033 = HEAP32[$6032>>2]|0;
        $6034 = (($6031) + 4)|0;
        $6035 = $6034;
        $6036 = HEAP32[$6035>>2]|0;
        $6037 = HEAP32[$pIn2>>2]|0;
        $6038 = $6037;
        $6039 = $6038;
        $6040 = HEAP32[$6039>>2]|0;
        $6041 = (($6038) + 4)|0;
        $6042 = $6041;
        $6043 = HEAP32[$6042>>2]|0;
        $6044 = ($6036|0)<($6043|0);
        $6045 = ($6033>>>0)<($6040>>>0);
        $6046 = ($6036|0)==($6043|0);
        $6047 = $6046 & $6045;
        $6048 = $6044 | $6047;
        if ($6048) {
         $6049 = HEAP32[$pIn2>>2]|0;
         $6050 = $6049;
         $6051 = $6050;
         $6052 = HEAP32[$6051>>2]|0;
         $6053 = (($6050) + 4)|0;
         $6054 = $6053;
         $6055 = HEAP32[$6054>>2]|0;
         $6056 = HEAP32[$pIn1>>2]|0;
         $6057 = $6056;
         $6058 = $6057;
         HEAP32[$6058>>2] = $6052;
         $6059 = (($6057) + 4)|0;
         $6060 = $6059;
         HEAP32[$6060>>2] = $6055;
        }
        break;
       }
       case 137:  {
        $6061 = HEAP32[$pOp>>2]|0;
        $6062 = (($6061) + 4|0);
        $6063 = HEAP32[$6062>>2]|0;
        $6064 = HEAP32[$aMem>>2]|0;
        $6065 = (($6064) + (($6063*40)|0)|0);
        HEAP32[$pIn1>>2] = $6065;
        $6066 = HEAP32[$pIn1>>2]|0;
        $6067 = $6066;
        $6068 = $6067;
        $6069 = HEAP32[$6068>>2]|0;
        $6070 = (($6067) + 4)|0;
        $6071 = $6070;
        $6072 = HEAP32[$6071>>2]|0;
        $6073 = ($6072|0)>(0);
        $6074 = ($6069>>>0)>(0);
        $6075 = ($6072|0)==(0);
        $6076 = $6075 & $6074;
        $6077 = $6073 | $6076;
        if ($6077) {
         $6078 = HEAP32[$pOp>>2]|0;
         $6079 = (($6078) + 8|0);
         $6080 = HEAP32[$6079>>2]|0;
         $6081 = (($6080) - 1)|0;
         HEAP32[$pc>>2] = $6081;
        }
        break;
       }
       case 138:  {
        $6082 = HEAP32[$pOp>>2]|0;
        $6083 = (($6082) + 4|0);
        $6084 = HEAP32[$6083>>2]|0;
        $6085 = HEAP32[$aMem>>2]|0;
        $6086 = (($6085) + (($6084*40)|0)|0);
        HEAP32[$pIn1>>2] = $6086;
        $6087 = HEAP32[$pOp>>2]|0;
        $6088 = (($6087) + 12|0);
        $6089 = HEAP32[$6088>>2]|0;
        $6090 = ($6089|0)<(0);
        $6091 = $6090 << 31 >> 31;
        $6092 = HEAP32[$pIn1>>2]|0;
        $6093 = $6092;
        $6094 = $6093;
        $6095 = HEAP32[$6094>>2]|0;
        $6096 = (($6093) + 4)|0;
        $6097 = $6096;
        $6098 = HEAP32[$6097>>2]|0;
        $6099 = (_i64Add(($6095|0),($6098|0),($6089|0),($6091|0))|0);
        $6100 = tempRet0;
        $6101 = $6092;
        $6102 = $6101;
        HEAP32[$6102>>2] = $6099;
        $6103 = (($6101) + 4)|0;
        $6104 = $6103;
        HEAP32[$6104>>2] = $6100;
        $6105 = HEAP32[$pIn1>>2]|0;
        $6106 = $6105;
        $6107 = $6106;
        $6108 = HEAP32[$6107>>2]|0;
        $6109 = (($6106) + 4)|0;
        $6110 = $6109;
        $6111 = HEAP32[$6110>>2]|0;
        $6112 = ($6111|0)<(0);
        if ($6112) {
         $6113 = HEAP32[$pOp>>2]|0;
         $6114 = (($6113) + 8|0);
         $6115 = HEAP32[$6114>>2]|0;
         $6116 = (($6115) - 1)|0;
         HEAP32[$pc>>2] = $6116;
        }
        break;
       }
       case 139:  {
        $6117 = HEAP32[$pOp>>2]|0;
        $6118 = (($6117) + 4|0);
        $6119 = HEAP32[$6118>>2]|0;
        $6120 = HEAP32[$aMem>>2]|0;
        $6121 = (($6120) + (($6119*40)|0)|0);
        HEAP32[$pIn1>>2] = $6121;
        $6122 = HEAP32[$pOp>>2]|0;
        $6123 = (($6122) + 12|0);
        $6124 = HEAP32[$6123>>2]|0;
        $6125 = ($6124|0)<(0);
        $6126 = $6125 << 31 >> 31;
        $6127 = HEAP32[$pIn1>>2]|0;
        $6128 = $6127;
        $6129 = $6128;
        $6130 = HEAP32[$6129>>2]|0;
        $6131 = (($6128) + 4)|0;
        $6132 = $6131;
        $6133 = HEAP32[$6132>>2]|0;
        $6134 = (_i64Add(($6130|0),($6133|0),($6124|0),($6126|0))|0);
        $6135 = tempRet0;
        $6136 = $6127;
        $6137 = $6136;
        HEAP32[$6137>>2] = $6134;
        $6138 = (($6136) + 4)|0;
        $6139 = $6138;
        HEAP32[$6139>>2] = $6135;
        $6140 = HEAP32[$pIn1>>2]|0;
        $6141 = $6140;
        $6142 = $6141;
        $6143 = HEAP32[$6142>>2]|0;
        $6144 = (($6141) + 4)|0;
        $6145 = $6144;
        $6146 = HEAP32[$6145>>2]|0;
        $6147 = ($6143|0)==(0);
        $6148 = ($6146|0)==(0);
        $6149 = $6147 & $6148;
        if ($6149) {
         $6150 = HEAP32[$pOp>>2]|0;
         $6151 = (($6150) + 8|0);
         $6152 = HEAP32[$6151>>2]|0;
         $6153 = (($6152) - 1)|0;
         HEAP32[$pc>>2] = $6153;
        }
        break;
       }
       case 10:  {
        $6154 = HEAP32[$pOp>>2]|0;
        $6155 = (($6154) + 3|0);
        $6156 = HEAP8[$6155>>0]|0;
        $6157 = $6156&255;
        HEAP32[$n92>>2] = $6157;
        $6158 = HEAP32[$pOp>>2]|0;
        $6159 = (($6158) + 8|0);
        $6160 = HEAP32[$6159>>2]|0;
        $6161 = HEAP32[$aMem>>2]|0;
        $6162 = (($6161) + (($6160*40)|0)|0);
        HEAP32[$pRec95>>2] = $6162;
        $6163 = HEAP32[$0>>2]|0;
        $6164 = (($6163) + 12|0);
        $6165 = HEAP32[$6164>>2]|0;
        HEAP32[$apVal98>>2] = $6165;
        HEAP32[$i93>>2] = 0;
        while(1) {
         $6166 = HEAP32[$i93>>2]|0;
         $6167 = HEAP32[$n92>>2]|0;
         $6168 = ($6166|0)<($6167|0);
         if (!($6168)) {
          break;
         }
         $6169 = HEAP32[$pRec95>>2]|0;
         $6170 = HEAP32[$i93>>2]|0;
         $6171 = HEAP32[$apVal98>>2]|0;
         $6172 = (($6171) + ($6170<<2)|0);
         HEAP32[$6172>>2] = $6169;
         $6173 = HEAP32[$i93>>2]|0;
         $6174 = (($6173) + 1)|0;
         HEAP32[$i93>>2] = $6174;
         $6175 = HEAP32[$pRec95>>2]|0;
         $6176 = (($6175) + 40|0);
         HEAP32[$pRec95>>2] = $6176;
        }
        $6177 = HEAP32[$pOp>>2]|0;
        $6178 = (($6177) + 16|0);
        $6179 = HEAP32[$6178>>2]|0;
        $6180 = (($ctx97) + 4|0);
        HEAP32[$6180>>2] = $6179;
        $6181 = HEAP32[$pOp>>2]|0;
        $6182 = (($6181) + 12|0);
        $6183 = HEAP32[$6182>>2]|0;
        $6184 = HEAP32[$aMem>>2]|0;
        $6185 = (($6184) + (($6183*40)|0)|0);
        HEAP32[$pMem94>>2] = $6185;
        $6186 = (($ctx97) + 8|0);
        HEAP32[$6186>>2] = $6185;
        $6187 = HEAP32[$pMem94>>2]|0;
        $6188 = (($6187) + 12|0);
        $6189 = HEAP32[$6188>>2]|0;
        $6190 = (($6189) + 1)|0;
        HEAP32[$6188>>2] = $6190;
        $6191 = HEAP32[$db>>2]|0;
        _sqlite3VdbeMemInit($t96,$6191,1);
        HEAP32[$ctx97>>2] = $t96;
        $6192 = (($ctx97) + 20|0);
        HEAP32[$6192>>2] = 0;
        $6193 = HEAP32[$0>>2]|0;
        $6194 = (($ctx97) + 12|0);
        HEAP32[$6194>>2] = $6193;
        $6195 = HEAP32[$pc>>2]|0;
        $6196 = (($ctx97) + 16|0);
        HEAP32[$6196>>2] = $6195;
        $6197 = (($ctx97) + 24|0);
        HEAP8[$6197>>0] = 0;
        $6198 = (($ctx97) + 4|0);
        $6199 = HEAP32[$6198>>2]|0;
        $6200 = (($6199) + 16|0);
        $6201 = HEAP32[$6200>>2]|0;
        $6202 = HEAP32[$n92>>2]|0;
        $6203 = HEAP32[$apVal98>>2]|0;
        FUNCTION_TABLE_viii[$6201 & 63]($ctx97,$6202,$6203);
        $6204 = (($ctx97) + 20|0);
        $6205 = HEAP32[$6204>>2]|0;
        $6206 = ($6205|0)!=(0);
        if ($6206) {
         $6207 = HEAP32[$0>>2]|0;
         $6208 = (($6207) + 44|0);
         $6209 = HEAP32[$db>>2]|0;
         $6210 = (_sqlite3_value_text($t96)|0);
         HEAP32[$vararg_buffer84>>2] = $6210;
         _sqlite3SetString($6208,$6209,1200,$vararg_buffer84);
         $6211 = (($ctx97) + 20|0);
         $6212 = HEAP32[$6211>>2]|0;
         HEAP32[$rc>>2] = $6212;
        }
        $6213 = (($ctx97) + 24|0);
        $6214 = HEAP8[$6213>>0]|0;
        $6215 = ($6214<<24>>24)!=(0);
        if ($6215) {
         $6216 = HEAP32[$pOp>>2]|0;
         $6217 = (($6216) + -20|0);
         $6218 = (($6217) + 4|0);
         $6219 = HEAP32[$6218>>2]|0;
         HEAP32[$i93>>2] = $6219;
         $6220 = HEAP32[$i93>>2]|0;
         $6221 = ($6220|0)!=(0);
         if ($6221) {
          $6222 = HEAP32[$i93>>2]|0;
          $6223 = HEAP32[$aMem>>2]|0;
          $6224 = (($6223) + (($6222*40)|0)|0);
          _sqlite3VdbeMemSetInt64($6224,1,0);
         }
        }
        _sqlite3VdbeMemRelease($t96);
        break;
       }
       case 140:  {
        $6225 = HEAP32[$pOp>>2]|0;
        $6226 = (($6225) + 4|0);
        $6227 = HEAP32[$6226>>2]|0;
        $6228 = HEAP32[$aMem>>2]|0;
        $6229 = (($6228) + (($6227*40)|0)|0);
        HEAP32[$pMem99>>2] = $6229;
        $6230 = HEAP32[$pMem99>>2]|0;
        $6231 = HEAP32[$pOp>>2]|0;
        $6232 = (($6231) + 16|0);
        $6233 = HEAP32[$6232>>2]|0;
        $6234 = (_sqlite3VdbeMemFinalize($6230,$6233)|0);
        HEAP32[$rc>>2] = $6234;
        $6235 = HEAP32[$rc>>2]|0;
        $6236 = ($6235|0)!=(0);
        if ($6236) {
         $6237 = HEAP32[$0>>2]|0;
         $6238 = (($6237) + 44|0);
         $6239 = HEAP32[$db>>2]|0;
         $6240 = HEAP32[$pMem99>>2]|0;
         $6241 = (_sqlite3_value_text($6240)|0);
         HEAP32[$vararg_buffer84>>2] = $6241;
         _sqlite3SetString($6238,$6239,1200,$vararg_buffer84);
        }
        $6242 = HEAP32[$pMem99>>2]|0;
        $6243 = HEAP8[$encoding>>0]|0;
        $6244 = $6243&255;
        (_sqlite3VdbeChangeEncoding($6242,$6244)|0);
        $6245 = HEAP32[$pMem99>>2]|0;
        $6246 = (_sqlite3VdbeMemTooBig($6245)|0);
        $6247 = ($6246|0)!=(0);
        if ($6247) {
         label = 870;
         break L11;
        }
        break;
       }
       case 12:  {
        $6279 = HEAP32[$pOp>>2]|0;
        $6280 = (($6279) + 12|0);
        $6281 = HEAP32[$6280>>2]|0;
        HEAP32[$eNew>>2] = $6281;
        $6282 = HEAP32[$pOp>>2]|0;
        $6283 = (($6282) + 4|0);
        $6284 = HEAP32[$6283>>2]|0;
        $6285 = HEAP32[$db>>2]|0;
        $6286 = (($6285) + 16|0);
        $6287 = HEAP32[$6286>>2]|0;
        $6288 = (($6287) + ($6284<<4)|0);
        $6289 = (($6288) + 4|0);
        $6290 = HEAP32[$6289>>2]|0;
        HEAP32[$pBt102>>2] = $6290;
        $6291 = HEAP32[$pBt102>>2]|0;
        $6292 = (_sqlite3BtreePager($6291)|0);
        HEAP32[$pPager>>2] = $6292;
        $6293 = HEAP32[$pPager>>2]|0;
        $6294 = (_sqlite3PagerGetJournalMode($6293)|0);
        HEAP32[$eOld>>2] = $6294;
        $6295 = HEAP32[$eNew>>2]|0;
        $6296 = ($6295|0)==(-1);
        if ($6296) {
         $6297 = HEAP32[$eOld>>2]|0;
         HEAP32[$eNew>>2] = $6297;
        }
        $6298 = HEAP32[$pPager>>2]|0;
        $6299 = (_sqlite3PagerOkToChangeJournalMode($6298)|0);
        $6300 = ($6299|0)!=(0);
        if (!($6300)) {
         $6301 = HEAP32[$eOld>>2]|0;
         HEAP32[$eNew>>2] = $6301;
        }
        $6302 = HEAP32[$pPager>>2]|0;
        $6303 = (_sqlite3PagerFilename($6302,1)|0);
        HEAP32[$zFilename>>2] = $6303;
        $6304 = HEAP32[$eNew>>2]|0;
        $6305 = ($6304|0)==(5);
        do {
         if ($6305) {
          $6306 = HEAP32[$zFilename>>2]|0;
          $6307 = (_sqlite3Strlen30($6306)|0);
          $6308 = ($6307|0)==(0);
          if (!($6308)) {
           $6309 = HEAP32[$pPager>>2]|0;
           $6310 = (_sqlite3PagerWalSupported($6309)|0);
           $6311 = ($6310|0)!=(0);
           if ($6311) {
            break;
           }
          }
          $6312 = HEAP32[$eOld>>2]|0;
          HEAP32[$eNew>>2] = $6312;
         }
        } while(0);
        $6313 = HEAP32[$eNew>>2]|0;
        $6314 = HEAP32[$eOld>>2]|0;
        $6315 = ($6313|0)!=($6314|0);
        do {
         if ($6315) {
          $6316 = HEAP32[$eOld>>2]|0;
          $6317 = ($6316|0)==(5);
          $6318 = HEAP32[$eNew>>2]|0;
          $6319 = ($6318|0)==(5);
          $or$cond24 = $6317 | $6319;
          if ($or$cond24) {
           $6320 = HEAP32[$db>>2]|0;
           $6321 = (($6320) + 62|0);
           $6322 = HEAP8[$6321>>0]|0;
           $6323 = ($6322<<24>>24)!=(0);
           if ($6323) {
            $6324 = HEAP32[$db>>2]|0;
            $6325 = (($6324) + 152|0);
            $6326 = HEAP32[$6325>>2]|0;
            $6327 = ($6326|0)>(1);
            if (!($6327)) {
             $6334 = HEAP32[$eOld>>2]|0;
             $6335 = ($6334|0)==(5);
             do {
              if ($6335) {
               $6336 = HEAP32[$pPager>>2]|0;
               $6337 = (_sqlite3PagerCloseWal($6336)|0);
               HEAP32[$rc>>2] = $6337;
               $6338 = HEAP32[$rc>>2]|0;
               $6339 = ($6338|0)==(0);
               if (!($6339)) {
                break;
               }
               $6340 = HEAP32[$pPager>>2]|0;
               $6341 = HEAP32[$eNew>>2]|0;
               (_sqlite3PagerSetJournalMode($6340,$6341)|0);
              } else {
               $6342 = HEAP32[$eOld>>2]|0;
               $6343 = ($6342|0)==(4);
               if (!($6343)) {
                break;
               }
               $6344 = HEAP32[$pPager>>2]|0;
               (_sqlite3PagerSetJournalMode($6344,2)|0);
              }
             } while(0);
             $6345 = HEAP32[$rc>>2]|0;
             $6346 = ($6345|0)==(0);
             if (!($6346)) {
              break;
             }
             $6347 = HEAP32[$pBt102>>2]|0;
             $6348 = HEAP32[$eNew>>2]|0;
             $6349 = ($6348|0)==(5);
             $6350 = $6349 ? 2 : 1;
             $6351 = (_sqlite3BtreeSetVersion($6347,$6350)|0);
             HEAP32[$rc>>2] = $6351;
             break;
            }
           }
           HEAP32[$rc>>2] = 1;
           $6328 = HEAP32[$0>>2]|0;
           $6329 = (($6328) + 44|0);
           $6330 = HEAP32[$db>>2]|0;
           $6331 = HEAP32[$eNew>>2]|0;
           $6332 = ($6331|0)==(5);
           $6333 = $6332 ? 31936 : 31944;
           HEAP32[$vararg_buffer84>>2] = $6333;
           _sqlite3SetString($6329,$6330,31880,$vararg_buffer84);
           break L21;
          }
         }
        } while(0);
        $6352 = HEAP32[$rc>>2]|0;
        $6353 = ($6352|0)!=(0);
        if ($6353) {
         $6354 = HEAP32[$eOld>>2]|0;
         HEAP32[$eNew>>2] = $6354;
        }
        $6355 = HEAP32[$pPager>>2]|0;
        $6356 = HEAP32[$eNew>>2]|0;
        $6357 = (_sqlite3PagerSetJournalMode($6355,$6356)|0);
        HEAP32[$eNew>>2] = $6357;
        $6358 = HEAP32[$pOp>>2]|0;
        $6359 = (($6358) + 8|0);
        $6360 = HEAP32[$6359>>2]|0;
        $6361 = HEAP32[$aMem>>2]|0;
        $6362 = (($6361) + (($6360*40)|0)|0);
        HEAP32[$pOut>>2] = $6362;
        $6363 = HEAP32[$pOut>>2]|0;
        $6364 = (($6363) + 8|0);
        HEAP16[$6364>>1] = 2562;
        $6365 = HEAP32[$eNew>>2]|0;
        $6366 = (_sqlite3JournalModename($6365)|0);
        $6367 = HEAP32[$pOut>>2]|0;
        $6368 = (($6367) + 16|0);
        HEAP32[$6368>>2] = $6366;
        $6369 = HEAP32[$pOut>>2]|0;
        $6370 = (($6369) + 16|0);
        $6371 = HEAP32[$6370>>2]|0;
        $6372 = (_sqlite3Strlen30($6371)|0);
        $6373 = HEAP32[$pOut>>2]|0;
        $6374 = (($6373) + 12|0);
        HEAP32[$6374>>2] = $6372;
        $6375 = HEAP32[$pOut>>2]|0;
        $6376 = (($6375) + 10|0);
        HEAP8[$6376>>0] = 1;
        $6377 = HEAP32[$pOut>>2]|0;
        $6378 = HEAP8[$encoding>>0]|0;
        $6379 = $6378&255;
        (_sqlite3VdbeChangeEncoding($6377,$6379)|0);
        break;
       }
       case 11:  {
        HEAP32[$aRes>>2] = 0;
        $6248 = (($aRes) + 8|0);
        HEAP32[$6248>>2] = -1;
        $6249 = (($aRes) + 4|0);
        HEAP32[$6249>>2] = -1;
        $6250 = HEAP32[$db>>2]|0;
        $6251 = HEAP32[$pOp>>2]|0;
        $6252 = (($6251) + 4|0);
        $6253 = HEAP32[$6252>>2]|0;
        $6254 = HEAP32[$pOp>>2]|0;
        $6255 = (($6254) + 8|0);
        $6256 = HEAP32[$6255>>2]|0;
        $6257 = (($aRes) + 4|0);
        $6258 = (($aRes) + 8|0);
        $6259 = (_sqlite3Checkpoint($6250,$6253,$6256,$6257,$6258)|0);
        HEAP32[$rc>>2] = $6259;
        $6260 = HEAP32[$rc>>2]|0;
        $6261 = ($6260|0)==(5);
        if ($6261) {
         HEAP32[$rc>>2] = 0;
         HEAP32[$aRes>>2] = 1;
        }
        HEAP32[$i100>>2] = 0;
        $6262 = HEAP32[$pOp>>2]|0;
        $6263 = (($6262) + 12|0);
        $6264 = HEAP32[$6263>>2]|0;
        $6265 = HEAP32[$aMem>>2]|0;
        $6266 = (($6265) + (($6264*40)|0)|0);
        HEAP32[$pMem101>>2] = $6266;
        while(1) {
         $6267 = HEAP32[$i100>>2]|0;
         $6268 = ($6267|0)<(3);
         if (!($6268)) {
          break L21;
         }
         $6269 = HEAP32[$pMem101>>2]|0;
         $6270 = HEAP32[$i100>>2]|0;
         $6271 = (($aRes) + ($6270<<2)|0);
         $6272 = HEAP32[$6271>>2]|0;
         $6273 = ($6272|0)<(0);
         $6274 = $6273 << 31 >> 31;
         _sqlite3VdbeMemSetInt64($6269,$6272,$6274);
         $6275 = HEAP32[$i100>>2]|0;
         $6276 = (($6275) + 1)|0;
         HEAP32[$i100>>2] = $6276;
         $6277 = HEAP32[$pMem101>>2]|0;
         $6278 = (($6277) + 40|0);
         HEAP32[$pMem101>>2] = $6278;
        }
        break;
       }
       case 141:  {
        $6384 = HEAP32[$pOp>>2]|0;
        $6385 = (($6384) + 4|0);
        $6386 = HEAP32[$6385>>2]|0;
        $6387 = HEAP32[$db>>2]|0;
        $6388 = (($6387) + 16|0);
        $6389 = HEAP32[$6388>>2]|0;
        $6390 = (($6389) + ($6386<<4)|0);
        $6391 = (($6390) + 4|0);
        $6392 = HEAP32[$6391>>2]|0;
        HEAP32[$pBt103>>2] = $6392;
        $6393 = HEAP32[$pBt103>>2]|0;
        $6394 = (_sqlite3BtreeIncrVacuum($6393)|0);
        HEAP32[$rc>>2] = $6394;
        $6395 = HEAP32[$rc>>2]|0;
        $6396 = ($6395|0)==(101);
        if ($6396) {
         $6397 = HEAP32[$pOp>>2]|0;
         $6398 = (($6397) + 8|0);
         $6399 = HEAP32[$6398>>2]|0;
         $6400 = (($6399) - 1)|0;
         HEAP32[$pc>>2] = $6400;
         HEAP32[$rc>>2] = 0;
        }
        break;
       }
       case 142:  {
        $6401 = HEAP32[$pOp>>2]|0;
        $6402 = (($6401) + 4|0);
        $6403 = HEAP32[$6402>>2]|0;
        $6404 = ($6403|0)!=(0);
        if ($6404) {
         $6406 = HEAP32[$0>>2]|0;
         $6407 = (($6406) + 88|0);
         $6408 = HEAP16[$6407>>1]|0;
         $6409 = $6408 & -33;
         $6410 = $6409 | 32;
         HEAP16[$6407>>1] = $6410;
         break L21;
        } else {
         $6405 = HEAP32[$db>>2]|0;
         _sqlite3ExpirePreparedStatements($6405);
         break L21;
        }
        break;
       }
       case 13:  {
        $6380 = HEAP32[$0>>2]|0;
        $6381 = (($6380) + 44|0);
        $6382 = HEAP32[$db>>2]|0;
        $6383 = (_sqlite3RunVacuum($6381,$6382)|0);
        HEAP32[$rc>>2] = $6383;
        break;
       }
       case 143:  {
        $6411 = HEAP32[$pOp>>2]|0;
        $6412 = (($6411) + 12|0);
        $6413 = HEAP32[$6412>>2]|0;
        $6414 = $6413&255;
        HEAP8[$isWriteLock>>0] = $6414;
        $6415 = HEAP8[$isWriteLock>>0]|0;
        $6416 = $6415&255;
        $6417 = ($6416|0)!=(0);
        if (!($6417)) {
         $6418 = HEAP32[$db>>2]|0;
         $6419 = (($6418) + 24|0);
         $6420 = HEAP32[$6419>>2]|0;
         $6421 = $6420 & 16384;
         $6422 = (0)==($6421|0);
         if (!($6422)) {
          break L21;
         }
        }
        $6423 = HEAP32[$pOp>>2]|0;
        $6424 = (($6423) + 4|0);
        $6425 = HEAP32[$6424>>2]|0;
        HEAP32[$p1104>>2] = $6425;
        $6426 = HEAP32[$p1104>>2]|0;
        $6427 = HEAP32[$db>>2]|0;
        $6428 = (($6427) + 16|0);
        $6429 = HEAP32[$6428>>2]|0;
        $6430 = (($6429) + ($6426<<4)|0);
        $6431 = (($6430) + 4|0);
        $6432 = HEAP32[$6431>>2]|0;
        $6433 = HEAP32[$pOp>>2]|0;
        $6434 = (($6433) + 8|0);
        $6435 = HEAP32[$6434>>2]|0;
        $6436 = HEAP8[$isWriteLock>>0]|0;
        $6437 = (_sqlite3BtreeLockTable($6432,$6435,$6436)|0);
        HEAP32[$rc>>2] = $6437;
        $6438 = HEAP32[$rc>>2]|0;
        $6439 = $6438 & 255;
        $6440 = ($6439|0)==(6);
        if ($6440) {
         $6441 = HEAP32[$pOp>>2]|0;
         $6442 = (($6441) + 16|0);
         $6443 = HEAP32[$6442>>2]|0;
         HEAP32[$z105>>2] = $6443;
         $6444 = HEAP32[$0>>2]|0;
         $6445 = (($6444) + 44|0);
         $6446 = HEAP32[$db>>2]|0;
         $6447 = HEAP32[$z105>>2]|0;
         HEAP32[$vararg_buffer84>>2] = $6447;
         _sqlite3SetString($6445,$6446,31952,$vararg_buffer84);
        }
        break;
       }
       case 145:  {
        $6460 = HEAP32[$db>>2]|0;
        $6461 = HEAP32[$pOp>>2]|0;
        $6462 = (($6461) + 4|0);
        $6463 = HEAP32[$6462>>2]|0;
        $6464 = HEAP32[$pOp>>2]|0;
        $6465 = (($6464) + 16|0);
        $6466 = HEAP32[$6465>>2]|0;
        $6467 = HEAP32[$0>>2]|0;
        $6468 = (($6467) + 44|0);
        $6469 = (_sqlite3VtabCallCreate($6460,$6463,$6466,$6468)|0);
        HEAP32[$rc>>2] = $6469;
        break;
       }
       case 146:  {
        $6470 = HEAP32[$0>>2]|0;
        $6471 = (($6470) + 88|0);
        $6472 = HEAP16[$6471>>1]|0;
        $6473 = $6472 & -13;
        $6474 = $6473 | 8;
        HEAP16[$6471>>1] = $6474;
        $6475 = HEAP32[$db>>2]|0;
        $6476 = HEAP32[$pOp>>2]|0;
        $6477 = (($6476) + 4|0);
        $6478 = HEAP32[$6477>>2]|0;
        $6479 = HEAP32[$pOp>>2]|0;
        $6480 = (($6479) + 16|0);
        $6481 = HEAP32[$6480>>2]|0;
        $6482 = (_sqlite3VtabCallDestroy($6475,$6478,$6481)|0);
        HEAP32[$rc>>2] = $6482;
        $6483 = HEAP32[$0>>2]|0;
        $6484 = (($6483) + 88|0);
        $6485 = HEAP16[$6484>>1]|0;
        $6486 = $6485 & -13;
        HEAP16[$6484>>1] = $6486;
        break;
       }
       case 144:  {
        $6448 = HEAP32[$pOp>>2]|0;
        $6449 = (($6448) + 16|0);
        $6450 = HEAP32[$6449>>2]|0;
        HEAP32[$pVTab>>2] = $6450;
        $6451 = HEAP32[$db>>2]|0;
        $6452 = HEAP32[$pVTab>>2]|0;
        $6453 = (_sqlite3VtabBegin($6451,$6452)|0);
        HEAP32[$rc>>2] = $6453;
        $6454 = HEAP32[$pVTab>>2]|0;
        $6455 = ($6454|0)!=(0|0);
        if ($6455) {
         $6456 = HEAP32[$0>>2]|0;
         $6457 = HEAP32[$pVTab>>2]|0;
         $6458 = (($6457) + 8|0);
         $6459 = HEAP32[$6458>>2]|0;
         _sqlite3VtabImportErrmsg($6456,$6459);
        }
        break;
       }
       case 147:  {
        HEAP32[$pCur106>>2] = 0;
        HEAP32[$pVtabCursor>>2] = 0;
        $6487 = HEAP32[$pOp>>2]|0;
        $6488 = (($6487) + 16|0);
        $6489 = HEAP32[$6488>>2]|0;
        $6490 = (($6489) + 8|0);
        $6491 = HEAP32[$6490>>2]|0;
        HEAP32[$pVtab107>>2] = $6491;
        $6492 = HEAP32[$pVtab107>>2]|0;
        $6493 = HEAP32[$6492>>2]|0;
        HEAP32[$pModule108>>2] = $6493;
        $6494 = HEAP32[$pModule108>>2]|0;
        $6495 = (($6494) + 24|0);
        $6496 = HEAP32[$6495>>2]|0;
        $6497 = HEAP32[$pVtab107>>2]|0;
        $6498 = (FUNCTION_TABLE_iii[$6496 & 63]($6497,$pVtabCursor)|0);
        HEAP32[$rc>>2] = $6498;
        $6499 = HEAP32[$0>>2]|0;
        $6500 = HEAP32[$pVtab107>>2]|0;
        _sqlite3VtabImportErrmsg($6499,$6500);
        $6501 = HEAP32[$rc>>2]|0;
        $6502 = (0)==($6501|0);
        if ($6502) {
         $6503 = HEAP32[$pVtab107>>2]|0;
         $6504 = HEAP32[$pVtabCursor>>2]|0;
         HEAP32[$6504>>2] = $6503;
         $6505 = HEAP32[$0>>2]|0;
         $6506 = HEAP32[$pOp>>2]|0;
         $6507 = (($6506) + 4|0);
         $6508 = HEAP32[$6507>>2]|0;
         $6509 = (_allocateCursor($6505,$6508,0,-1,0)|0);
         HEAP32[$pCur106>>2] = $6509;
         $6510 = HEAP32[$pCur106>>2]|0;
         $6511 = ($6510|0)!=(0|0);
         if ($6511) {
          $6512 = HEAP32[$pVtabCursor>>2]|0;
          $6513 = HEAP32[$pCur106>>2]|0;
          $6514 = (($6513) + 32|0);
          HEAP32[$6514>>2] = $6512;
          break L21;
         } else {
          $6515 = HEAP32[$db>>2]|0;
          $6516 = (($6515) + 64|0);
          HEAP8[$6516>>0] = 1;
          $6517 = HEAP32[$pModule108>>2]|0;
          $6518 = (($6517) + 28|0);
          $6519 = HEAP32[$6518>>2]|0;
          $6520 = HEAP32[$pVtabCursor>>2]|0;
          (FUNCTION_TABLE_ii[$6519 & 31]($6520)|0);
          break L21;
         }
        }
        break;
       }
       case 14:  {
        $6521 = HEAP32[$pOp>>2]|0;
        $6522 = (($6521) + 12|0);
        $6523 = HEAP32[$6522>>2]|0;
        $6524 = HEAP32[$aMem>>2]|0;
        $6525 = (($6524) + (($6523*40)|0)|0);
        HEAP32[$pQuery>>2] = $6525;
        $6526 = HEAP32[$pQuery>>2]|0;
        $6527 = (($6526) + 40|0);
        HEAP32[$pArgc>>2] = $6527;
        $6528 = HEAP32[$pOp>>2]|0;
        $6529 = (($6528) + 4|0);
        $6530 = HEAP32[$6529>>2]|0;
        $6531 = HEAP32[$0>>2]|0;
        $6532 = (($6531) + 56|0);
        $6533 = HEAP32[$6532>>2]|0;
        $6534 = (($6533) + ($6530<<2)|0);
        $6535 = HEAP32[$6534>>2]|0;
        HEAP32[$pCur112>>2] = $6535;
        $6536 = HEAP32[$pCur112>>2]|0;
        $6537 = (($6536) + 32|0);
        $6538 = HEAP32[$6537>>2]|0;
        HEAP32[$pVtabCursor110>>2] = $6538;
        $6539 = HEAP32[$pVtabCursor110>>2]|0;
        $6540 = HEAP32[$6539>>2]|0;
        HEAP32[$pVtab111>>2] = $6540;
        $6541 = HEAP32[$pVtab111>>2]|0;
        $6542 = HEAP32[$6541>>2]|0;
        HEAP32[$pModule109>>2] = $6542;
        $6543 = HEAP32[$pArgc>>2]|0;
        $6544 = $6543;
        $6545 = $6544;
        $6546 = HEAP32[$6545>>2]|0;
        $6547 = (($6544) + 4)|0;
        $6548 = $6547;
        $6549 = HEAP32[$6548>>2]|0;
        HEAP32[$nArg>>2] = $6546;
        $6550 = HEAP32[$pQuery>>2]|0;
        $6551 = $6550;
        $6552 = $6551;
        $6553 = HEAP32[$6552>>2]|0;
        $6554 = (($6551) + 4)|0;
        $6555 = $6554;
        $6556 = HEAP32[$6555>>2]|0;
        HEAP32[$iQuery>>2] = $6553;
        HEAP32[$res113>>2] = 0;
        $6557 = HEAP32[$0>>2]|0;
        $6558 = (($6557) + 12|0);
        $6559 = HEAP32[$6558>>2]|0;
        HEAP32[$apArg>>2] = $6559;
        HEAP32[$i114>>2] = 0;
        while(1) {
         $6560 = HEAP32[$i114>>2]|0;
         $6561 = HEAP32[$nArg>>2]|0;
         $6562 = ($6560|0)<($6561|0);
         if (!($6562)) {
          break;
         }
         $6563 = HEAP32[$i114>>2]|0;
         $6564 = (($6563) + 1)|0;
         $6565 = HEAP32[$pArgc>>2]|0;
         $6566 = (($6565) + (($6564*40)|0)|0);
         $6567 = HEAP32[$i114>>2]|0;
         $6568 = HEAP32[$apArg>>2]|0;
         $6569 = (($6568) + ($6567<<2)|0);
         HEAP32[$6569>>2] = $6566;
         $6570 = HEAP32[$i114>>2]|0;
         $6571 = (($6570) + 1)|0;
         HEAP32[$i114>>2] = $6571;
        }
        $6572 = HEAP32[$0>>2]|0;
        $6573 = (($6572) + 88|0);
        $6574 = HEAP16[$6573>>1]|0;
        $6575 = $6574 & -13;
        $6576 = $6575 | 4;
        HEAP16[$6573>>1] = $6576;
        $6577 = HEAP32[$pModule109>>2]|0;
        $6578 = (($6577) + 32|0);
        $6579 = HEAP32[$6578>>2]|0;
        $6580 = HEAP32[$pVtabCursor110>>2]|0;
        $6581 = HEAP32[$iQuery>>2]|0;
        $6582 = HEAP32[$pOp>>2]|0;
        $6583 = (($6582) + 16|0);
        $6584 = HEAP32[$6583>>2]|0;
        $6585 = HEAP32[$nArg>>2]|0;
        $6586 = HEAP32[$apArg>>2]|0;
        $6587 = (FUNCTION_TABLE_iiiiii[$6579 & 15]($6580,$6581,$6584,$6585,$6586)|0);
        HEAP32[$rc>>2] = $6587;
        $6588 = HEAP32[$0>>2]|0;
        $6589 = (($6588) + 88|0);
        $6590 = HEAP16[$6589>>1]|0;
        $6591 = $6590 & -13;
        HEAP16[$6589>>1] = $6591;
        $6592 = HEAP32[$0>>2]|0;
        $6593 = HEAP32[$pVtab111>>2]|0;
        _sqlite3VtabImportErrmsg($6592,$6593);
        $6594 = HEAP32[$rc>>2]|0;
        $6595 = ($6594|0)==(0);
        if ($6595) {
         $6596 = HEAP32[$pModule109>>2]|0;
         $6597 = (($6596) + 40|0);
         $6598 = HEAP32[$6597>>2]|0;
         $6599 = HEAP32[$pVtabCursor110>>2]|0;
         $6600 = (FUNCTION_TABLE_ii[$6598 & 31]($6599)|0);
         HEAP32[$res113>>2] = $6600;
        }
        $6601 = HEAP32[$res113>>2]|0;
        $6602 = ($6601|0)!=(0);
        if ($6602) {
         $6603 = HEAP32[$pOp>>2]|0;
         $6604 = (($6603) + 8|0);
         $6605 = HEAP32[$6604>>2]|0;
         $6606 = (($6605) - 1)|0;
         HEAP32[$pc>>2] = $6606;
        }
        $6607 = HEAP32[$pCur112>>2]|0;
        $6608 = (($6607) + 25|0);
        HEAP8[$6608>>0] = 0;
        break;
       }
       case 148:  {
        $6609 = HEAP32[$pOp>>2]|0;
        $6610 = (($6609) + 4|0);
        $6611 = HEAP32[$6610>>2]|0;
        $6612 = HEAP32[$0>>2]|0;
        $6613 = (($6612) + 56|0);
        $6614 = HEAP32[$6613>>2]|0;
        $6615 = (($6614) + ($6611<<2)|0);
        $6616 = HEAP32[$6615>>2]|0;
        HEAP32[$pCur118>>2] = $6616;
        $6617 = HEAP32[$pOp>>2]|0;
        $6618 = (($6617) + 12|0);
        $6619 = HEAP32[$6618>>2]|0;
        $6620 = HEAP32[$aMem>>2]|0;
        $6621 = (($6620) + (($6619*40)|0)|0);
        HEAP32[$pDest117>>2] = $6621;
        $6622 = HEAP32[$pCur118>>2]|0;
        $6623 = (($6622) + 25|0);
        $6624 = HEAP8[$6623>>0]|0;
        $6625 = ($6624<<24>>24)!=(0);
        if ($6625) {
         $6626 = HEAP32[$pDest117>>2]|0;
         _sqlite3VdbeMemSetNull($6626);
         break L21;
        }
        $6627 = HEAP32[$pCur118>>2]|0;
        $6628 = (($6627) + 32|0);
        $6629 = HEAP32[$6628>>2]|0;
        $6630 = HEAP32[$6629>>2]|0;
        HEAP32[$pVtab115>>2] = $6630;
        $6631 = HEAP32[$pVtab115>>2]|0;
        $6632 = HEAP32[$6631>>2]|0;
        HEAP32[$pModule116>>2] = $6632;
        ;HEAP32[$sContext+0>>2]=0|0;HEAP32[$sContext+4>>2]=0|0;HEAP32[$sContext+8>>2]=0|0;HEAP32[$sContext+12>>2]=0|0;HEAP32[$sContext+16>>2]=0|0;HEAP32[$sContext+20>>2]=0|0;HEAP32[$sContext+24>>2]=0|0;
        $6633 = HEAP32[$pDest117>>2]|0;
        HEAP32[$sContext>>2] = $6633;
        $6634 = HEAP32[$pDest117>>2]|0;
        $6635 = (($6634) + 8|0);
        $6636 = HEAP16[$6635>>1]|0;
        $6637 = $6636&65535;
        $6638 = $6637 & -16896;
        $6639 = $6638 | 1;
        $6640 = $6639&65535;
        $6641 = HEAP32[$pDest117>>2]|0;
        $6642 = (($6641) + 8|0);
        HEAP16[$6642>>1] = $6640;
        $6643 = HEAP32[$pModule116>>2]|0;
        $6644 = (($6643) + 44|0);
        $6645 = HEAP32[$6644>>2]|0;
        $6646 = HEAP32[$pCur118>>2]|0;
        $6647 = (($6646) + 32|0);
        $6648 = HEAP32[$6647>>2]|0;
        $6649 = HEAP32[$pOp>>2]|0;
        $6650 = (($6649) + 8|0);
        $6651 = HEAP32[$6650>>2]|0;
        $6652 = (FUNCTION_TABLE_iiii[$6645 & 31]($6648,$sContext,$6651)|0);
        HEAP32[$rc>>2] = $6652;
        $6653 = HEAP32[$0>>2]|0;
        $6654 = HEAP32[$pVtab115>>2]|0;
        _sqlite3VtabImportErrmsg($6653,$6654);
        $6655 = (($sContext) + 20|0);
        $6656 = HEAP32[$6655>>2]|0;
        $6657 = ($6656|0)!=(0);
        if ($6657) {
         $6658 = (($sContext) + 20|0);
         $6659 = HEAP32[$6658>>2]|0;
         HEAP32[$rc>>2] = $6659;
        }
        $6660 = HEAP32[$pDest117>>2]|0;
        $6661 = HEAP8[$encoding>>0]|0;
        $6662 = $6661&255;
        (_sqlite3VdbeChangeEncoding($6660,$6662)|0);
        $6663 = HEAP32[$pDest117>>2]|0;
        $6664 = (_sqlite3VdbeMemTooBig($6663)|0);
        $6665 = ($6664|0)!=(0);
        if ($6665) {
         label = 870;
         break L11;
        }
        break;
       }
       case 150:  {
        $6717 = HEAP32[$pOp>>2]|0;
        $6718 = (($6717) + 16|0);
        $6719 = HEAP32[$6718>>2]|0;
        $6720 = (($6719) + 8|0);
        $6721 = HEAP32[$6720>>2]|0;
        HEAP32[$pVtab123>>2] = $6721;
        $6722 = HEAP32[$pOp>>2]|0;
        $6723 = (($6722) + 4|0);
        $6724 = HEAP32[$6723>>2]|0;
        $6725 = HEAP32[$aMem>>2]|0;
        $6726 = (($6725) + (($6724*40)|0)|0);
        HEAP32[$pName>>2] = $6726;
        $6727 = HEAP32[$pName>>2]|0;
        $6728 = (_sqlite3VdbeChangeEncoding($6727,1)|0);
        HEAP32[$rc>>2] = $6728;
        $6729 = HEAP32[$rc>>2]|0;
        $6730 = ($6729|0)==(0);
        if ($6730) {
         $6731 = HEAP32[$pVtab123>>2]|0;
         $6732 = HEAP32[$6731>>2]|0;
         $6733 = (($6732) + 76|0);
         $6734 = HEAP32[$6733>>2]|0;
         $6735 = HEAP32[$pVtab123>>2]|0;
         $6736 = HEAP32[$pName>>2]|0;
         $6737 = (($6736) + 16|0);
         $6738 = HEAP32[$6737>>2]|0;
         $6739 = (FUNCTION_TABLE_iii[$6734 & 63]($6735,$6738)|0);
         HEAP32[$rc>>2] = $6739;
         $6740 = HEAP32[$0>>2]|0;
         $6741 = HEAP32[$pVtab123>>2]|0;
         _sqlite3VtabImportErrmsg($6740,$6741);
         $6742 = HEAP32[$0>>2]|0;
         $6743 = (($6742) + 88|0);
         $6744 = HEAP16[$6743>>1]|0;
         $6745 = $6744 & -33;
         HEAP16[$6743>>1] = $6745;
        }
        break;
       }
       case 149:  {
        HEAP32[$res121>>2] = 0;
        $6666 = HEAP32[$pOp>>2]|0;
        $6667 = (($6666) + 4|0);
        $6668 = HEAP32[$6667>>2]|0;
        $6669 = HEAP32[$0>>2]|0;
        $6670 = (($6669) + 56|0);
        $6671 = HEAP32[$6670>>2]|0;
        $6672 = (($6671) + ($6668<<2)|0);
        $6673 = HEAP32[$6672>>2]|0;
        HEAP32[$pCur122>>2] = $6673;
        $6674 = HEAP32[$pCur122>>2]|0;
        $6675 = (($6674) + 25|0);
        $6676 = HEAP8[$6675>>0]|0;
        $6677 = ($6676<<24>>24)!=(0);
        if (!($6677)) {
         $6678 = HEAP32[$pCur122>>2]|0;
         $6679 = (($6678) + 32|0);
         $6680 = HEAP32[$6679>>2]|0;
         $6681 = HEAP32[$6680>>2]|0;
         HEAP32[$pVtab119>>2] = $6681;
         $6682 = HEAP32[$pVtab119>>2]|0;
         $6683 = HEAP32[$6682>>2]|0;
         HEAP32[$pModule120>>2] = $6683;
         $6684 = HEAP32[$0>>2]|0;
         $6685 = (($6684) + 88|0);
         $6686 = HEAP16[$6685>>1]|0;
         $6687 = $6686 & -13;
         $6688 = $6687 | 4;
         HEAP16[$6685>>1] = $6688;
         $6689 = HEAP32[$pModule120>>2]|0;
         $6690 = (($6689) + 36|0);
         $6691 = HEAP32[$6690>>2]|0;
         $6692 = HEAP32[$pCur122>>2]|0;
         $6693 = (($6692) + 32|0);
         $6694 = HEAP32[$6693>>2]|0;
         $6695 = (FUNCTION_TABLE_ii[$6691 & 31]($6694)|0);
         HEAP32[$rc>>2] = $6695;
         $6696 = HEAP32[$0>>2]|0;
         $6697 = (($6696) + 88|0);
         $6698 = HEAP16[$6697>>1]|0;
         $6699 = $6698 & -13;
         HEAP16[$6697>>1] = $6699;
         $6700 = HEAP32[$0>>2]|0;
         $6701 = HEAP32[$pVtab119>>2]|0;
         _sqlite3VtabImportErrmsg($6700,$6701);
         $6702 = HEAP32[$rc>>2]|0;
         $6703 = ($6702|0)==(0);
         if ($6703) {
          $6704 = HEAP32[$pModule120>>2]|0;
          $6705 = (($6704) + 40|0);
          $6706 = HEAP32[$6705>>2]|0;
          $6707 = HEAP32[$pCur122>>2]|0;
          $6708 = (($6707) + 32|0);
          $6709 = HEAP32[$6708>>2]|0;
          $6710 = (FUNCTION_TABLE_ii[$6706 & 31]($6709)|0);
          HEAP32[$res121>>2] = $6710;
         }
         $6711 = HEAP32[$res121>>2]|0;
         $6712 = ($6711|0)!=(0);
         if ($6712) {
          label = 16;
         } else {
          $6713 = HEAP32[$pOp>>2]|0;
          $6714 = (($6713) + 8|0);
          $6715 = HEAP32[$6714>>2]|0;
          $6716 = (($6715) - 1)|0;
          HEAP32[$pc>>2] = $6716;
          label = 16;
         }
        }
        break;
       }
       case 15:  {
        $6746 = HEAP32[$pOp>>2]|0;
        $6747 = (($6746) + 16|0);
        $6748 = HEAP32[$6747>>2]|0;
        $6749 = (($6748) + 8|0);
        $6750 = HEAP32[$6749>>2]|0;
        HEAP32[$pVtab124>>2] = $6750;
        $6751 = HEAP32[$pVtab124>>2]|0;
        $6752 = HEAP32[$6751>>2]|0;
        HEAP32[$pModule125>>2] = $6752;
        $6753 = HEAP32[$pOp>>2]|0;
        $6754 = (($6753) + 8|0);
        $6755 = HEAP32[$6754>>2]|0;
        HEAP32[$nArg126>>2] = $6755;
        $6756 = HEAP32[$pModule125>>2]|0;
        $6757 = (($6756) + 52|0);
        $6758 = HEAP32[$6757>>2]|0;
        $6759 = ($6758|0)!=(0|0);
        if ($6759) {
         $6760 = HEAP32[$db>>2]|0;
         $6761 = (($6760) + 68|0);
         $6762 = HEAP8[$6761>>0]|0;
         HEAP8[$vtabOnConflict>>0] = $6762;
         $6763 = HEAP32[$0>>2]|0;
         $6764 = (($6763) + 12|0);
         $6765 = HEAP32[$6764>>2]|0;
         HEAP32[$apArg129>>2] = $6765;
         $6766 = HEAP32[$pOp>>2]|0;
         $6767 = (($6766) + 12|0);
         $6768 = HEAP32[$6767>>2]|0;
         $6769 = HEAP32[$aMem>>2]|0;
         $6770 = (($6769) + (($6768*40)|0)|0);
         HEAP32[$pX130>>2] = $6770;
         HEAP32[$i127>>2] = 0;
         while(1) {
          $6771 = HEAP32[$i127>>2]|0;
          $6772 = HEAP32[$nArg126>>2]|0;
          $6773 = ($6771|0)<($6772|0);
          if (!($6773)) {
           break;
          }
          $6774 = HEAP32[$pX130>>2]|0;
          $6775 = HEAP32[$i127>>2]|0;
          $6776 = HEAP32[$apArg129>>2]|0;
          $6777 = (($6776) + ($6775<<2)|0);
          HEAP32[$6777>>2] = $6774;
          $6778 = HEAP32[$pX130>>2]|0;
          $6779 = (($6778) + 40|0);
          HEAP32[$pX130>>2] = $6779;
          $6780 = HEAP32[$i127>>2]|0;
          $6781 = (($6780) + 1)|0;
          HEAP32[$i127>>2] = $6781;
         }
         $6782 = HEAP32[$pOp>>2]|0;
         $6783 = (($6782) + 3|0);
         $6784 = HEAP8[$6783>>0]|0;
         $6785 = HEAP32[$db>>2]|0;
         $6786 = (($6785) + 68|0);
         HEAP8[$6786>>0] = $6784;
         $6787 = HEAP32[$pModule125>>2]|0;
         $6788 = (($6787) + 52|0);
         $6789 = HEAP32[$6788>>2]|0;
         $6790 = HEAP32[$pVtab124>>2]|0;
         $6791 = HEAP32[$nArg126>>2]|0;
         $6792 = HEAP32[$apArg129>>2]|0;
         $6793 = (FUNCTION_TABLE_iiiii[$6789 & 7]($6790,$6791,$6792,$rowid128)|0);
         HEAP32[$rc>>2] = $6793;
         $6794 = HEAP8[$vtabOnConflict>>0]|0;
         $6795 = HEAP32[$db>>2]|0;
         $6796 = (($6795) + 68|0);
         HEAP8[$6796>>0] = $6794;
         $6797 = HEAP32[$0>>2]|0;
         $6798 = HEAP32[$pVtab124>>2]|0;
         _sqlite3VtabImportErrmsg($6797,$6798);
         $6799 = HEAP32[$rc>>2]|0;
         $6800 = ($6799|0)==(0);
         if ($6800) {
          $6801 = HEAP32[$pOp>>2]|0;
          $6802 = (($6801) + 4|0);
          $6803 = HEAP32[$6802>>2]|0;
          $6804 = ($6803|0)!=(0);
          if ($6804) {
           $6805 = $rowid128;
           $6806 = $6805;
           $6807 = HEAP32[$6806>>2]|0;
           $6808 = (($6805) + 4)|0;
           $6809 = $6808;
           $6810 = HEAP32[$6809>>2]|0;
           $6811 = $lastRowid;
           $6812 = $6811;
           HEAP32[$6812>>2] = $6807;
           $6813 = (($6811) + 4)|0;
           $6814 = $6813;
           HEAP32[$6814>>2] = $6810;
           $6815 = HEAP32[$db>>2]|0;
           $6816 = (($6815) + 32|0);
           $6817 = $6816;
           $6818 = $6817;
           HEAP32[$6818>>2] = $6807;
           $6819 = (($6817) + 4)|0;
           $6820 = $6819;
           HEAP32[$6820>>2] = $6810;
          }
         }
         $6821 = HEAP32[$rc>>2]|0;
         $6822 = $6821 & 255;
         $6823 = ($6822|0)==(19);
         if ($6823) {
          $6824 = HEAP32[$pOp>>2]|0;
          $6825 = (($6824) + 16|0);
          $6826 = HEAP32[$6825>>2]|0;
          $6827 = (($6826) + 16|0);
          $6828 = HEAP8[$6827>>0]|0;
          $6829 = $6828&255;
          $6830 = ($6829|0)!=(0);
          if ($6830) {
           $6831 = HEAP32[$pOp>>2]|0;
           $6832 = (($6831) + 3|0);
           $6833 = HEAP8[$6832>>0]|0;
           $6834 = $6833&255;
           $6835 = ($6834|0)==(4);
           if ($6835) {
            HEAP32[$rc>>2] = 0;
            break L21;
           }
           $6836 = HEAP32[$pOp>>2]|0;
           $6837 = (($6836) + 3|0);
           $6838 = HEAP8[$6837>>0]|0;
           $6839 = $6838&255;
           $6840 = ($6839|0)==(5);
           if ($6840) {
            $6846 = 2;
           } else {
            $6841 = HEAP32[$pOp>>2]|0;
            $6842 = (($6841) + 3|0);
            $6843 = HEAP8[$6842>>0]|0;
            $6844 = $6843&255;
            $6846 = $6844;
           }
           $6845 = $6846&255;
           $6847 = HEAP32[$0>>2]|0;
           $6848 = (($6847) + 86|0);
           HEAP8[$6848>>0] = $6845;
           break L21;
          }
         }
         $6849 = HEAP32[$0>>2]|0;
         $6850 = (($6849) + 92|0);
         $6851 = HEAP32[$6850>>2]|0;
         $6852 = (($6851) + 1)|0;
         HEAP32[$6850>>2] = $6852;
        }
        break;
       }
       case 153:  {
        $6901 = HEAP32[$pOp>>2]|0;
        $6902 = (($6901) + 8|0);
        $6903 = HEAP32[$6902>>2]|0;
        $6904 = ($6903|0)!=(0);
        if ($6904) {
         $6905 = HEAP32[$pOp>>2]|0;
         $6906 = (($6905) + 8|0);
         $6907 = HEAP32[$6906>>2]|0;
         $6908 = (($6907) - 1)|0;
         HEAP32[$pc>>2] = $6908;
        }
        $6909 = HEAP32[$db>>2]|0;
        $6910 = (($6909) + 172|0);
        $6911 = HEAP32[$6910>>2]|0;
        $6912 = ($6911|0)!=(0|0);
        if ($6912) {
         $6913 = HEAP32[$0>>2]|0;
         $6914 = (($6913) + 88|0);
         $6915 = HEAP16[$6914>>1]|0;
         $6916 = ($6915&65535) >>> 11;
         $6917 = $6916 & 1;
         $6918 = $6917&65535;
         $6919 = ($6918|0)!=(0);
         if (!($6919)) {
          $6920 = HEAP32[$pOp>>2]|0;
          $6921 = (($6920) + 16|0);
          $6922 = HEAP32[$6921>>2]|0;
          $6923 = ($6922|0)!=(0|0);
          if ($6923) {
           $6924 = HEAP32[$pOp>>2]|0;
           $6925 = (($6924) + 16|0);
           $6926 = HEAP32[$6925>>2]|0;
           $6930 = $6926;
          } else {
           $6927 = HEAP32[$0>>2]|0;
           $6928 = (($6927) + 168|0);
           $6929 = HEAP32[$6928>>2]|0;
           $6930 = $6929;
          }
          HEAP32[$zTrace>>2] = $6930;
          $6931 = ($6930|0)!=(0|0);
          if ($6931) {
           $6932 = HEAP32[$0>>2]|0;
           $6933 = HEAP32[$zTrace>>2]|0;
           $6934 = (_sqlite3VdbeExpandSql($6932,$6933)|0);
           HEAP32[$z132>>2] = $6934;
           $6935 = HEAP32[$db>>2]|0;
           $6936 = (($6935) + 172|0);
           $6937 = HEAP32[$6936>>2]|0;
           $6938 = HEAP32[$db>>2]|0;
           $6939 = (($6938) + 176|0);
           $6940 = HEAP32[$6939>>2]|0;
           $6941 = HEAP32[$z132>>2]|0;
           FUNCTION_TABLE_vii[$6937 & 7]($6940,$6941);
           $6942 = HEAP32[$db>>2]|0;
           $6943 = HEAP32[$z132>>2]|0;
           _sqlite3DbFree($6942,$6943);
          }
         }
        }
        break;
       }
       case 151:  {
        $6853 = HEAP32[$pOp>>2]|0;
        $6854 = (($6853) + 4|0);
        $6855 = HEAP32[$6854>>2]|0;
        $6856 = HEAP32[$db>>2]|0;
        $6857 = (($6856) + 16|0);
        $6858 = HEAP32[$6857>>2]|0;
        $6859 = (($6858) + ($6855<<4)|0);
        $6860 = (($6859) + 4|0);
        $6861 = HEAP32[$6860>>2]|0;
        $6862 = (_sqlite3BtreeLastPage($6861)|0);
        $6863 = HEAP32[$pOut>>2]|0;
        $6864 = $6863;
        $6865 = $6864;
        HEAP32[$6865>>2] = $6862;
        $6866 = (($6864) + 4)|0;
        $6867 = $6866;
        HEAP32[$6867>>2] = 0;
        break;
       }
       case 152:  {
        $6868 = HEAP32[$pOp>>2]|0;
        $6869 = (($6868) + 4|0);
        $6870 = HEAP32[$6869>>2]|0;
        $6871 = HEAP32[$db>>2]|0;
        $6872 = (($6871) + 16|0);
        $6873 = HEAP32[$6872>>2]|0;
        $6874 = (($6873) + ($6870<<4)|0);
        $6875 = (($6874) + 4|0);
        $6876 = HEAP32[$6875>>2]|0;
        HEAP32[$pBt131>>2] = $6876;
        HEAP32[$newMax>>2] = 0;
        $6877 = HEAP32[$pOp>>2]|0;
        $6878 = (($6877) + 12|0);
        $6879 = HEAP32[$6878>>2]|0;
        $6880 = ($6879|0)!=(0);
        if ($6880) {
         $6881 = HEAP32[$pBt131>>2]|0;
         $6882 = (_sqlite3BtreeLastPage($6881)|0);
         HEAP32[$newMax>>2] = $6882;
         $6883 = HEAP32[$newMax>>2]|0;
         $6884 = HEAP32[$pOp>>2]|0;
         $6885 = (($6884) + 12|0);
         $6886 = HEAP32[$6885>>2]|0;
         $6887 = ($6883>>>0)<($6886>>>0);
         if ($6887) {
          $6888 = HEAP32[$pOp>>2]|0;
          $6889 = (($6888) + 12|0);
          $6890 = HEAP32[$6889>>2]|0;
          HEAP32[$newMax>>2] = $6890;
         }
        }
        $6891 = HEAP32[$pBt131>>2]|0;
        $6892 = HEAP32[$newMax>>2]|0;
        $6893 = (_sqlite3BtreeMaxPageCount($6891,$6892)|0);
        $6894 = ($6893|0)<(0);
        $6895 = $6894 << 31 >> 31;
        $6896 = HEAP32[$pOut>>2]|0;
        $6897 = $6896;
        $6898 = $6897;
        HEAP32[$6898>>2] = $6893;
        $6899 = (($6897) + 4)|0;
        $6900 = $6899;
        HEAP32[$6900>>2] = $6895;
        break;
       }
       default: {
       }
       }
      } while(0);
      do {
       if ((label|0) == 28) {
        label = 0;
        $248 = HEAP32[$pOp>>2]|0;
        $249 = (($248) + 4|0);
        $250 = HEAP32[$249>>2]|0;
        $251 = ($250|0)==(0);
        if (!($251)) {
         label = 33;
         break L11;
        }
        $252 = HEAP32[$0>>2]|0;
        $253 = (($252) + 176|0);
        $254 = HEAP32[$253>>2]|0;
        $255 = ($254|0)!=(0|0);
        if (!($255)) {
         label = 33;
         break L11;
        }
        $256 = HEAP32[$0>>2]|0;
        $257 = (($256) + 176|0);
        $258 = HEAP32[$257>>2]|0;
        HEAP32[$pFrame>>2] = $258;
        $259 = HEAP32[$pFrame>>2]|0;
        $260 = (($259) + 4|0);
        $261 = HEAP32[$260>>2]|0;
        $262 = HEAP32[$0>>2]|0;
        $263 = (($262) + 176|0);
        HEAP32[$263>>2] = $261;
        $264 = HEAP32[$0>>2]|0;
        $265 = (($264) + 184|0);
        $266 = HEAP32[$265>>2]|0;
        $267 = (($266) + -1)|0;
        HEAP32[$265>>2] = $267;
        $268 = HEAP32[$db>>2]|0;
        $269 = HEAP32[$0>>2]|0;
        $270 = (($269) + 92|0);
        $271 = HEAP32[$270>>2]|0;
        _sqlite3VdbeSetChanges($268,$271);
        $272 = HEAP32[$pFrame>>2]|0;
        $273 = (_sqlite3VdbeFrameRestore($272)|0);
        HEAP32[$pc>>2] = $273;
        $274 = HEAP32[$db>>2]|0;
        $275 = (($274) + 32|0);
        $276 = $275;
        $277 = $276;
        $278 = HEAP32[$277>>2]|0;
        $279 = (($276) + 4)|0;
        $280 = $279;
        $281 = HEAP32[$280>>2]|0;
        $282 = $lastRowid;
        $283 = $282;
        HEAP32[$283>>2] = $278;
        $284 = (($282) + 4)|0;
        $285 = $284;
        HEAP32[$285>>2] = $281;
        $286 = HEAP32[$pOp>>2]|0;
        $287 = (($286) + 8|0);
        $288 = HEAP32[$287>>2]|0;
        $289 = ($288|0)==(4);
        if ($289) {
         $290 = HEAP32[$pc>>2]|0;
         $291 = HEAP32[$0>>2]|0;
         $292 = (($291) + 4|0);
         $293 = HEAP32[$292>>2]|0;
         $294 = (($293) + (($290*20)|0)|0);
         $295 = (($294) + 8|0);
         $296 = HEAP32[$295>>2]|0;
         $297 = (($296) - 1)|0;
         HEAP32[$pc>>2] = $297;
        }
        $298 = HEAP32[$0>>2]|0;
        $299 = (($298) + 4|0);
        $300 = HEAP32[$299>>2]|0;
        HEAP32[$aOp>>2] = $300;
        $301 = HEAP32[$0>>2]|0;
        $302 = (($301) + 8|0);
        $303 = HEAP32[$302>>2]|0;
        HEAP32[$aMem>>2] = $303;
       }
       else if ((label|0) == 57) {
        label = 0;
        $468 = HEAP32[$pOut>>2]|0;
        $469 = (($468) + 8|0);
        HEAP16[$469>>1] = 2562;
        $470 = HEAP32[$pOp>>2]|0;
        $471 = (($470) + 16|0);
        $472 = HEAP32[$471>>2]|0;
        $473 = HEAP32[$pOut>>2]|0;
        $474 = (($473) + 16|0);
        HEAP32[$474>>2] = $472;
        $475 = HEAP32[$pOp>>2]|0;
        $476 = (($475) + 4|0);
        $477 = HEAP32[$476>>2]|0;
        $478 = HEAP32[$pOut>>2]|0;
        $479 = (($478) + 12|0);
        HEAP32[$479>>2] = $477;
        $480 = HEAP8[$encoding>>0]|0;
        $481 = HEAP32[$pOut>>2]|0;
        $482 = (($481) + 10|0);
        HEAP8[$482>>0] = $480;
       }
       else if ((label|0) == 453) {
        label = 0;
        $3441 = HEAP32[$0>>2]|0;
        $3442 = (($3441) + 88|0);
        $3443 = HEAP16[$3442>>1]|0;
        $3444 = ($3443&65535) >>> 5;
        $3445 = $3444 & 1;
        $3446 = $3445&65535;
        $3447 = ($3446|0)!=(0);
        if ($3447) {
         HEAP32[$rc>>2] = 516;
         break;
        }
        HEAP32[$nField19>>2] = 0;
        HEAP32[$pKeyInfo20>>2] = 0;
        $3448 = HEAP32[$pOp>>2]|0;
        $3449 = (($3448) + 8|0);
        $3450 = HEAP32[$3449>>2]|0;
        HEAP32[$p221>>2] = $3450;
        $3451 = HEAP32[$pOp>>2]|0;
        $3452 = (($3451) + 12|0);
        $3453 = HEAP32[$3452>>2]|0;
        HEAP32[$iDb22>>2] = $3453;
        $3454 = HEAP32[$iDb22>>2]|0;
        $3455 = HEAP32[$db>>2]|0;
        $3456 = (($3455) + 16|0);
        $3457 = HEAP32[$3456>>2]|0;
        $3458 = (($3457) + ($3454<<4)|0);
        HEAP32[$pDb24>>2] = $3458;
        $3459 = HEAP32[$pDb24>>2]|0;
        $3460 = (($3459) + 4|0);
        $3461 = HEAP32[$3460>>2]|0;
        HEAP32[$pX>>2] = $3461;
        $3462 = HEAP32[$pOp>>2]|0;
        $3463 = HEAP8[$3462>>0]|0;
        $3464 = $3463&255;
        $3465 = ($3464|0)==(55);
        if ($3465) {
         HEAP32[$wrFlag>>2] = 1;
         $3466 = HEAP32[$pDb24>>2]|0;
         $3467 = (($3466) + 12|0);
         $3468 = HEAP32[$3467>>2]|0;
         $3469 = (($3468) + 76|0);
         $3470 = HEAP8[$3469>>0]|0;
         $3471 = $3470&255;
         $3472 = HEAP32[$0>>2]|0;
         $3473 = (($3472) + 87|0);
         $3474 = HEAP8[$3473>>0]|0;
         $3475 = $3474&255;
         $3476 = ($3471|0)<($3475|0);
         if ($3476) {
          $3477 = HEAP32[$pDb24>>2]|0;
          $3478 = (($3477) + 12|0);
          $3479 = HEAP32[$3478>>2]|0;
          $3480 = (($3479) + 76|0);
          $3481 = HEAP8[$3480>>0]|0;
          $3482 = HEAP32[$0>>2]|0;
          $3483 = (($3482) + 87|0);
          HEAP8[$3483>>0] = $3481;
         }
        } else {
         HEAP32[$wrFlag>>2] = 0;
        }
        $3484 = HEAP32[$pOp>>2]|0;
        $3485 = (($3484) + 3|0);
        $3486 = HEAP8[$3485>>0]|0;
        $3487 = $3486&255;
        $3488 = $3487 & 2;
        $3489 = ($3488|0)!=(0);
        if ($3489) {
         $3490 = HEAP32[$p221>>2]|0;
         $3491 = HEAP32[$aMem>>2]|0;
         $3492 = (($3491) + (($3490*40)|0)|0);
         HEAP32[$pIn2>>2] = $3492;
         $3493 = HEAP32[$pIn2>>2]|0;
         (_sqlite3VdbeMemIntegerify($3493)|0);
         $3494 = HEAP32[$pIn2>>2]|0;
         $3495 = $3494;
         $3496 = $3495;
         $3497 = HEAP32[$3496>>2]|0;
         $3498 = (($3495) + 4)|0;
         $3499 = $3498;
         $3500 = HEAP32[$3499>>2]|0;
         HEAP32[$p221>>2] = $3497;
         $3501 = HEAP32[$p221>>2]|0;
         $3502 = ($3501|0)<(2);
         if ($3502) {
          label = 461;
          break L11;
         }
        }
        $3504 = HEAP32[$pOp>>2]|0;
        $3505 = (($3504) + 1|0);
        $3506 = HEAP8[$3505>>0]|0;
        $3507 = $3506 << 24 >> 24;
        $3508 = ($3507|0)==(-6);
        $3509 = HEAP32[$pOp>>2]|0;
        if ($3508) {
         $3510 = (($3509) + 16|0);
         $3511 = HEAP32[$3510>>2]|0;
         HEAP32[$pKeyInfo20>>2] = $3511;
         $3512 = HEAP32[$pKeyInfo20>>2]|0;
         $3513 = (($3512) + 6|0);
         $3514 = HEAP16[$3513>>1]|0;
         $3515 = $3514&65535;
         $3516 = HEAP32[$pKeyInfo20>>2]|0;
         $3517 = (($3516) + 8|0);
         $3518 = HEAP16[$3517>>1]|0;
         $3519 = $3518&65535;
         $3520 = (($3515) + ($3519))|0;
         HEAP32[$nField19>>2] = $3520;
        } else {
         $3521 = (($3509) + 1|0);
         $3522 = HEAP8[$3521>>0]|0;
         $3523 = $3522 << 24 >> 24;
         $3524 = ($3523|0)==(-14);
         if ($3524) {
          $3525 = HEAP32[$pOp>>2]|0;
          $3526 = (($3525) + 16|0);
          $3527 = HEAP32[$3526>>2]|0;
          HEAP32[$nField19>>2] = $3527;
         }
        }
        $3528 = HEAP32[$0>>2]|0;
        $3529 = HEAP32[$pOp>>2]|0;
        $3530 = (($3529) + 4|0);
        $3531 = HEAP32[$3530>>2]|0;
        $3532 = HEAP32[$nField19>>2]|0;
        $3533 = HEAP32[$iDb22>>2]|0;
        $3534 = (_allocateCursor($3528,$3531,$3532,$3533,1)|0);
        HEAP32[$pCur23>>2] = $3534;
        $3535 = HEAP32[$pCur23>>2]|0;
        $3536 = ($3535|0)==(0|0);
        if ($3536) {
         label = 871;
         break L1;
        }
        $3537 = HEAP32[$pCur23>>2]|0;
        $3538 = (($3537) + 25|0);
        HEAP8[$3538>>0] = 1;
        $3539 = HEAP32[$pCur23>>2]|0;
        $3540 = (($3539) + 27|0);
        $3541 = HEAP8[$3540>>0]|0;
        $3542 = $3541 & -9;
        $3543 = $3542 | 8;
        HEAP8[$3540>>0] = $3543;
        $3544 = HEAP32[$p221>>2]|0;
        $3545 = HEAP32[$pCur23>>2]|0;
        $3546 = (($3545) + 28|0);
        HEAP32[$3546>>2] = $3544;
        $3547 = HEAP32[$pX>>2]|0;
        $3548 = HEAP32[$p221>>2]|0;
        $3549 = HEAP32[$wrFlag>>2]|0;
        $3550 = HEAP32[$pKeyInfo20>>2]|0;
        $3551 = HEAP32[$pCur23>>2]|0;
        $3552 = HEAP32[$3551>>2]|0;
        $3553 = (_sqlite3BtreeCursor($3547,$3548,$3549,$3550,$3552)|0);
        HEAP32[$rc>>2] = $3553;
        $3554 = HEAP32[$pKeyInfo20>>2]|0;
        $3555 = HEAP32[$pCur23>>2]|0;
        $3556 = (($3555) + 8|0);
        HEAP32[$3556>>2] = $3554;
        $3557 = HEAP32[$pCur23>>2]|0;
        $3558 = HEAP32[$3557>>2]|0;
        $3559 = HEAP32[$pOp>>2]|0;
        $3560 = (($3559) + 3|0);
        $3561 = HEAP8[$3560>>0]|0;
        $3562 = $3561&255;
        $3563 = $3562 & 1;
        _sqlite3BtreeCursorHints($3558,$3563);
        $3564 = HEAP32[$pOp>>2]|0;
        $3565 = (($3564) + 1|0);
        $3566 = HEAP8[$3565>>0]|0;
        $3567 = $3566 << 24 >> 24;
        $3568 = ($3567|0)!=(-6);
        $3569 = $3568&1;
        $3570 = HEAP32[$pCur23>>2]|0;
        $3571 = (($3570) + 27|0);
        $3572 = $3569&255;
        $3573 = HEAP8[$3571>>0]|0;
        $3574 = $3572 & 1;
        $3575 = ($3574 << 2)&255;
        $3576 = $3573 & -5;
        $3577 = $3576 | $3575;
        HEAP8[$3571>>0] = $3577;
       }
       else if ((label|0) == 623) {
        label = 0;
        $4894 = HEAP32[$pOp>>2]|0;
        $4895 = (($4894) + 4|0);
        $4896 = HEAP32[$4895>>2]|0;
        $4897 = HEAP32[$0>>2]|0;
        $4898 = (($4897) + 56|0);
        $4899 = HEAP32[$4898>>2]|0;
        $4900 = (($4899) + ($4896<<2)|0);
        $4901 = HEAP32[$4900>>2]|0;
        HEAP32[$pC63>>2] = $4901;
        HEAP32[$res65>>2] = 1;
        $4902 = HEAP32[$pC63>>2]|0;
        $4903 = (($4902) + 56|0);
        $4904 = HEAP32[$4903>>2]|0;
        $4905 = ($4904|0)!=(0|0);
        $4906 = HEAP32[$pC63>>2]|0;
        if ($4905) {
         $4907 = (_sqlite3VdbeSorterRewind($4906,$res65)|0);
         HEAP32[$rc>>2] = $4907;
        } else {
         $4908 = HEAP32[$4906>>2]|0;
         HEAP32[$pCrsr64>>2] = $4908;
         $4909 = HEAP32[$pCrsr64>>2]|0;
         $4910 = (_sqlite3BtreeFirst($4909,$res65)|0);
         HEAP32[$rc>>2] = $4910;
         $4911 = HEAP32[$pC63>>2]|0;
         $4912 = (($4911) + 26|0);
         HEAP8[$4912>>0] = 0;
         $4913 = HEAP32[$pC63>>2]|0;
         $4914 = (($4913) + 60|0);
         HEAP32[$4914>>2] = 0;
        }
        $4915 = HEAP32[$res65>>2]|0;
        $4916 = $4915&255;
        $4917 = HEAP32[$pC63>>2]|0;
        $4918 = (($4917) + 25|0);
        HEAP8[$4918>>0] = $4916;
        $4919 = HEAP32[$res65>>2]|0;
        $4920 = ($4919|0)!=(0);
        if ($4920) {
         $4921 = HEAP32[$pOp>>2]|0;
         $4922 = (($4921) + 8|0);
         $4923 = HEAP32[$4922>>2]|0;
         $4924 = (($4923) - 1)|0;
         HEAP32[$pc>>2] = $4924;
        }
       }
       else if ((label|0) == 630) {
        label = 0;
        $4945 = HEAP32[$pOp>>2]|0;
        $4946 = (($4945) + 4|0);
        $4947 = HEAP32[$4946>>2]|0;
        $4948 = HEAP32[$0>>2]|0;
        $4949 = (($4948) + 56|0);
        $4950 = HEAP32[$4949>>2]|0;
        $4951 = (($4950) + ($4947<<2)|0);
        $4952 = HEAP32[$4951>>2]|0;
        HEAP32[$pC66>>2] = $4952;
        $4953 = HEAP32[$pOp>>2]|0;
        $4954 = (($4953) + 12|0);
        $4955 = HEAP32[$4954>>2]|0;
        HEAP32[$res67>>2] = $4955;
        $4956 = HEAP32[$pOp>>2]|0;
        $4957 = (($4956) + 16|0);
        $4958 = HEAP32[$4957>>2]|0;
        $4959 = HEAP32[$pC66>>2]|0;
        $4960 = HEAP32[$4959>>2]|0;
        $4961 = (FUNCTION_TABLE_iii[$4958 & 63]($4960,$res67)|0);
        HEAP32[$rc>>2] = $4961;
        label = 631;
       }
      } while(0);
      do {
       if ((label|0) == 631) {
        label = 0;
        $4962 = HEAP32[$pC66>>2]|0;
        $4963 = (($4962) + 60|0);
        HEAP32[$4963>>2] = 0;
        $4964 = HEAP32[$res67>>2]|0;
        $4965 = ($4964|0)==(0);
        $4966 = HEAP32[$pC66>>2]|0;
        $4967 = (($4966) + 25|0);
        if ($4965) {
         HEAP8[$4967>>0] = 0;
         $4968 = HEAP32[$pOp>>2]|0;
         $4969 = (($4968) + 8|0);
         $4970 = HEAP32[$4969>>2]|0;
         $4971 = (($4970) - 1)|0;
         HEAP32[$pc>>2] = $4971;
         $4972 = HEAP32[$pOp>>2]|0;
         $4973 = (($4972) + 3|0);
         $4974 = HEAP8[$4973>>0]|0;
         $4975 = $4974&255;
         $4976 = HEAP32[$0>>2]|0;
         $4977 = (($4976) + 108|0);
         $4978 = (($4977) + ($4975<<2)|0);
         $4979 = HEAP32[$4978>>2]|0;
         $4980 = (($4979) + 1)|0;
         HEAP32[$4978>>2] = $4980;
         label = 16;
         break;
        } else {
         HEAP8[$4967>>0] = 1;
         label = 16;
         break;
        }
       }
      } while(0);
      if ((label|0) == 16) {
       label = 0;
       $105 = HEAP32[$db>>2]|0;
       $106 = (($105) + 240|0);
       $107 = HEAP32[$106>>2]|0;
       $108 = ($107|0)!=(0);
       if ($108) {
        break L3;
       }
       $109 = HEAP32[$db>>2]|0;
       $110 = (($109) + 292|0);
       $111 = HEAP32[$110>>2]|0;
       $112 = ($111|0)!=(0|0);
       if ($112) {
        $113 = HEAP32[$nVmStep>>2]|0;
        $114 = HEAP32[$nProgressLimit>>2]|0;
        $115 = ($113>>>0)>=($114>>>0);
        if ($115) {
         $116 = HEAP32[$nVmStep>>2]|0;
         $117 = HEAP32[$db>>2]|0;
         $118 = (($117) + 300|0);
         $119 = HEAP32[$118>>2]|0;
         $120 = (($116) + ($119))|0;
         $121 = HEAP32[$nVmStep>>2]|0;
         $122 = HEAP32[$db>>2]|0;
         $123 = (($122) + 300|0);
         $124 = HEAP32[$123>>2]|0;
         $125 = (($121>>>0) % ($124>>>0))&-1;
         $126 = (($120) - ($125))|0;
         HEAP32[$nProgressLimit>>2] = $126;
         $127 = HEAP32[$db>>2]|0;
         $128 = (($127) + 292|0);
         $129 = HEAP32[$128>>2]|0;
         $130 = HEAP32[$db>>2]|0;
         $131 = (($130) + 296|0);
         $132 = HEAP32[$131>>2]|0;
         $133 = (FUNCTION_TABLE_ii[$129 & 31]($132)|0);
         $134 = ($133|0)!=(0);
         if ($134) {
          label = 20;
          break;
         }
        }
       }
      }
      $6944 = HEAP32[$pc>>2]|0;
      $6945 = (($6944) + 1)|0;
      HEAP32[$pc>>2] = $6945;
     }
     switch (label|0) {
      case 20: {
       HEAP32[$rc>>2] = 9;
       break L1;
       break;
      }
      case 33: {
       $304 = HEAP32[$pOp>>2]|0;
       $305 = (($304) + 4|0);
       $306 = HEAP32[$305>>2]|0;
       $307 = HEAP32[$0>>2]|0;
       $308 = (($307) + 80|0);
       HEAP32[$308>>2] = $306;
       $309 = HEAP32[$pOp>>2]|0;
       $310 = (($309) + 8|0);
       $311 = HEAP32[$310>>2]|0;
       $312 = $311&255;
       $313 = HEAP32[$0>>2]|0;
       $314 = (($313) + 86|0);
       HEAP8[$314>>0] = $312;
       $315 = HEAP32[$pc>>2]|0;
       $316 = HEAP32[$0>>2]|0;
       $317 = (($316) + 76|0);
       HEAP32[$317>>2] = $315;
       $318 = HEAP32[$0>>2]|0;
       $319 = (($318) + 80|0);
       $320 = HEAP32[$319>>2]|0;
       $321 = ($320|0)!=(0);
       if ($321) {
        $322 = HEAP32[$pOp>>2]|0;
        $323 = (($322) + 3|0);
        $324 = HEAP8[$323>>0]|0;
        $325 = ($324<<24>>24)!=(0);
        if ($325) {
         $326 = HEAP32[$pOp>>2]|0;
         $327 = (($326) + 3|0);
         $328 = HEAP8[$327>>0]|0;
         $329 = $328&255;
         $330 = (($329) - 1)|0;
         $331 = (31264 + ($330<<2)|0);
         $332 = HEAP32[$331>>2]|0;
         HEAP32[$zType>>2] = $332;
        } else {
         HEAP32[$zType>>2] = 0;
        }
        HEAP32[$zLogFmt>>2] = 31328;
        $333 = HEAP32[$zType>>2]|0;
        $334 = ($333|0)!=(0|0);
        if ($334) {
         $335 = HEAP32[$pOp>>2]|0;
         $336 = (($335) + 16|0);
         $337 = HEAP32[$336>>2]|0;
         $338 = ($337|0)!=(0|0);
         if ($338) {
          $339 = HEAP32[$0>>2]|0;
          $340 = (($339) + 44|0);
          $341 = HEAP32[$db>>2]|0;
          $342 = HEAP32[$zType>>2]|0;
          $343 = HEAP32[$pOp>>2]|0;
          $344 = (($343) + 16|0);
          $345 = HEAP32[$344>>2]|0;
          HEAP32[$vararg_buffer84>>2] = $342;
          $vararg_ptr27 = (($vararg_buffer84) + 4|0);
          HEAP32[$vararg_ptr27>>2] = $345;
          _sqlite3SetString($340,$341,31352,$vararg_buffer84);
         } else {
          label = 40;
         }
        } else {
         label = 40;
        }
        do {
         if ((label|0) == 40) {
          $346 = HEAP32[$pOp>>2]|0;
          $347 = (($346) + 16|0);
          $348 = HEAP32[$347>>2]|0;
          $349 = ($348|0)!=(0|0);
          $350 = HEAP32[$0>>2]|0;
          $351 = (($350) + 44|0);
          $352 = HEAP32[$db>>2]|0;
          if ($349) {
           $353 = HEAP32[$pOp>>2]|0;
           $354 = (($353) + 16|0);
           $355 = HEAP32[$354>>2]|0;
           HEAP32[$vararg_buffer84>>2] = $355;
           _sqlite3SetString($351,$352,1200,$vararg_buffer84);
           break;
          } else {
           $356 = HEAP32[$zType>>2]|0;
           HEAP32[$vararg_buffer84>>2] = $356;
           _sqlite3SetString($351,$352,31384,$vararg_buffer84);
           break;
          }
         }
        } while(0);
        $357 = HEAP32[$pOp>>2]|0;
        $358 = (($357) + 4|0);
        $359 = HEAP32[$358>>2]|0;
        $360 = HEAP32[$zLogFmt>>2]|0;
        $361 = HEAP32[$pc>>2]|0;
        $362 = HEAP32[$0>>2]|0;
        $363 = (($362) + 168|0);
        $364 = HEAP32[$363>>2]|0;
        $365 = HEAP32[$0>>2]|0;
        $366 = (($365) + 44|0);
        $367 = HEAP32[$366>>2]|0;
        HEAP32[$vararg_buffer84>>2] = $361;
        $vararg_ptr37 = (($vararg_buffer84) + 4|0);
        HEAP32[$vararg_ptr37>>2] = $364;
        $vararg_ptr38 = (($vararg_buffer84) + 8|0);
        HEAP32[$vararg_ptr38>>2] = $367;
        _sqlite3_log($359,$360,$vararg_buffer84);
       }
       $368 = HEAP32[$0>>2]|0;
       $369 = (_sqlite3VdbeHalt($368)|0);
       HEAP32[$rc>>2] = $369;
       $370 = HEAP32[$rc>>2]|0;
       $371 = ($370|0)==(5);
       if ($371) {
        HEAP32[$rc>>2] = 5;
        $372 = HEAP32[$0>>2]|0;
        $373 = (($372) + 80|0);
        HEAP32[$373>>2] = 5;
        $6969 = $lastRowid;
        $6970 = $6969;
        $6971 = HEAP32[$6970>>2]|0;
        $6972 = (($6969) + 4)|0;
        $6973 = $6972;
        $6974 = HEAP32[$6973>>2]|0;
        $6975 = HEAP32[$db>>2]|0;
        $6976 = (($6975) + 32|0);
        $6977 = $6976;
        $6978 = $6977;
        HEAP32[$6978>>2] = $6971;
        $6979 = (($6977) + 4)|0;
        $6980 = $6979;
        HEAP32[$6980>>2] = $6974;
        $6981 = HEAP32[$nVmStep>>2]|0;
        $6982 = HEAP32[$0>>2]|0;
        $6983 = (($6982) + 108|0);
        $6984 = (($6983) + 16|0);
        $6985 = HEAP32[$6984>>2]|0;
        $6986 = (($6985) + ($6981))|0;
        HEAP32[$6984>>2] = $6986;
        $6987 = HEAP32[$rc>>2]|0;
        STACKTOP = sp;return ($6987|0);
       } else {
        $374 = HEAP32[$0>>2]|0;
        $375 = (($374) + 80|0);
        $376 = HEAP32[$375>>2]|0;
        $377 = ($376|0)!=(0);
        $378 = $377 ? 1 : 101;
        HEAP32[$rc>>2] = $378;
        $6969 = $lastRowid;
        $6970 = $6969;
        $6971 = HEAP32[$6970>>2]|0;
        $6972 = (($6969) + 4)|0;
        $6973 = $6972;
        $6974 = HEAP32[$6973>>2]|0;
        $6975 = HEAP32[$db>>2]|0;
        $6976 = (($6975) + 32|0);
        $6977 = $6976;
        $6978 = $6977;
        HEAP32[$6978>>2] = $6971;
        $6979 = (($6977) + 4)|0;
        $6980 = $6979;
        HEAP32[$6980>>2] = $6974;
        $6981 = HEAP32[$nVmStep>>2]|0;
        $6982 = HEAP32[$0>>2]|0;
        $6983 = (($6982) + 108|0);
        $6984 = (($6983) + 16|0);
        $6985 = HEAP32[$6984>>2]|0;
        $6986 = (($6985) + ($6981))|0;
        HEAP32[$6984>>2] = $6986;
        $6987 = HEAP32[$rc>>2]|0;
        STACKTOP = sp;return ($6987|0);
       }
       break;
      }
      case 76: {
       HEAP32[$rc>>2] = 9;
       break L1;
       break;
      }
      case 79: {
       $634 = HEAP32[$0>>2]|0;
       $635 = (($634) + 72|0);
       $636 = HEAP32[$635>>2]|0;
       $637 = (($636) + 2)|0;
       $638 = $637 | 1;
       $639 = HEAP32[$0>>2]|0;
       $640 = (($639) + 72|0);
       HEAP32[$640>>2] = $638;
       $641 = HEAP32[$pOp>>2]|0;
       $642 = (($641) + 4|0);
       $643 = HEAP32[$642>>2]|0;
       $644 = HEAP32[$aMem>>2]|0;
       $645 = (($644) + (($643*40)|0)|0);
       $646 = HEAP32[$0>>2]|0;
       $647 = (($646) + 20|0);
       HEAP32[$647>>2] = $645;
       HEAP32[$pMem>>2] = $645;
       HEAP32[$i>>2] = 0;
       while(1) {
        $648 = HEAP32[$i>>2]|0;
        $649 = HEAP32[$pOp>>2]|0;
        $650 = (($649) + 8|0);
        $651 = HEAP32[$650>>2]|0;
        $652 = ($648|0)<($651|0);
        if (!($652)) {
         break;
        }
        $653 = HEAP32[$i>>2]|0;
        $654 = HEAP32[$pMem>>2]|0;
        $655 = (($654) + (($653*40)|0)|0);
        $656 = (($655) + 8|0);
        $657 = HEAP16[$656>>1]|0;
        $658 = $657&65535;
        $659 = $658 & 4096;
        $660 = ($659|0)!=(0);
        if ($660) {
         $661 = HEAP32[$i>>2]|0;
         $662 = HEAP32[$pMem>>2]|0;
         $663 = (($662) + (($661*40)|0)|0);
         $664 = (_sqlite3VdbeMemMakeWriteable($663)|0);
         $665 = ($664|0)!=(0);
         if ($665) {
          label = 871;
          break L1;
         }
        }
        $666 = HEAP32[$i>>2]|0;
        $667 = HEAP32[$pMem>>2]|0;
        $668 = (($667) + (($666*40)|0)|0);
        (_sqlite3VdbeMemNulTerminate($668)|0);
        $669 = HEAP32[$i>>2]|0;
        $670 = (($669) + 1)|0;
        HEAP32[$i>>2] = $670;
       }
       $671 = HEAP32[$db>>2]|0;
       $672 = (($671) + 64|0);
       $673 = HEAP8[$672>>0]|0;
       $674 = ($673<<24>>24)!=(0);
       if ($674) {
        label = 871;
        break L1;
       }
       $675 = HEAP32[$pc>>2]|0;
       $676 = (($675) + 1)|0;
       $677 = HEAP32[$0>>2]|0;
       $678 = (($677) + 76|0);
       HEAP32[$678>>2] = $676;
       HEAP32[$rc>>2] = 100;
       $6969 = $lastRowid;
       $6970 = $6969;
       $6971 = HEAP32[$6970>>2]|0;
       $6972 = (($6969) + 4)|0;
       $6973 = $6972;
       $6974 = HEAP32[$6973>>2]|0;
       $6975 = HEAP32[$db>>2]|0;
       $6976 = (($6975) + 32|0);
       $6977 = $6976;
       $6978 = $6977;
       HEAP32[$6978>>2] = $6971;
       $6979 = (($6977) + 4)|0;
       $6980 = $6979;
       HEAP32[$6980>>2] = $6974;
       $6981 = HEAP32[$nVmStep>>2]|0;
       $6982 = HEAP32[$0>>2]|0;
       $6983 = (($6982) + 108|0);
       $6984 = (($6983) + 16|0);
       $6985 = HEAP32[$6984>>2]|0;
       $6986 = (($6985) + ($6981))|0;
       HEAP32[$6984>>2] = $6986;
       $6987 = HEAP32[$rc>>2]|0;
       STACKTOP = sp;return ($6987|0);
       break;
      }
      case 167: {
       HEAP32[$rc>>2] = 20;
       break;
      }
      case 381: {
       $2949 = HEAP32[$pc>>2]|0;
       $2950 = HEAP32[$0>>2]|0;
       $2951 = (($2950) + 76|0);
       HEAP32[$2951>>2] = $2949;
       $2952 = HEAP32[$db>>2]|0;
       $2953 = (($2952) + 62|0);
       HEAP8[$2953>>0] = 0;
       HEAP32[$rc>>2] = 5;
       $2954 = HEAP32[$0>>2]|0;
       $2955 = (($2954) + 80|0);
       HEAP32[$2955>>2] = 5;
       $6969 = $lastRowid;
       $6970 = $6969;
       $6971 = HEAP32[$6970>>2]|0;
       $6972 = (($6969) + 4)|0;
       $6973 = $6972;
       $6974 = HEAP32[$6973>>2]|0;
       $6975 = HEAP32[$db>>2]|0;
       $6976 = (($6975) + 32|0);
       $6977 = $6976;
       $6978 = $6977;
       HEAP32[$6978>>2] = $6971;
       $6979 = (($6977) + 4)|0;
       $6980 = $6979;
       HEAP32[$6980>>2] = $6974;
       $6981 = HEAP32[$nVmStep>>2]|0;
       $6982 = HEAP32[$0>>2]|0;
       $6983 = (($6982) + 108|0);
       $6984 = (($6983) + 16|0);
       $6985 = HEAP32[$6984>>2]|0;
       $6986 = (($6985) + ($6981))|0;
       HEAP32[$6984>>2] = $6986;
       $6987 = HEAP32[$rc>>2]|0;
       STACKTOP = sp;return ($6987|0);
       break;
      }
      case 409: {
       $3125 = HEAP32[$iRollback>>2]|0;
       $3126 = ($3125|0)!=(0);
       if ($3126) {
        $3127 = HEAP32[$db>>2]|0;
        _sqlite3RollbackAll($3127,516);
        $3128 = HEAP32[$db>>2]|0;
        $3129 = (($3128) + 62|0);
        HEAP8[$3129>>0] = 1;
       } else {
        $3130 = HEAP32[$0>>2]|0;
        $3131 = (_sqlite3VdbeCheckFk($3130,1)|0);
        HEAP32[$rc>>2] = $3131;
        $3132 = ($3131|0)!=(0);
        if ($3132) {
         $6969 = $lastRowid;
         $6970 = $6969;
         $6971 = HEAP32[$6970>>2]|0;
         $6972 = (($6969) + 4)|0;
         $6973 = $6972;
         $6974 = HEAP32[$6973>>2]|0;
         $6975 = HEAP32[$db>>2]|0;
         $6976 = (($6975) + 32|0);
         $6977 = $6976;
         $6978 = $6977;
         HEAP32[$6978>>2] = $6971;
         $6979 = (($6977) + 4)|0;
         $6980 = $6979;
         HEAP32[$6980>>2] = $6974;
         $6981 = HEAP32[$nVmStep>>2]|0;
         $6982 = HEAP32[$0>>2]|0;
         $6983 = (($6982) + 108|0);
         $6984 = (($6983) + 16|0);
         $6985 = HEAP32[$6984>>2]|0;
         $6986 = (($6985) + ($6981))|0;
         HEAP32[$6984>>2] = $6986;
         $6987 = HEAP32[$rc>>2]|0;
         STACKTOP = sp;return ($6987|0);
        }
        $3133 = HEAP32[$desiredAutoCommit>>2]|0;
        $3134 = $3133&255;
        $3135 = HEAP32[$db>>2]|0;
        $3136 = (($3135) + 62|0);
        HEAP8[$3136>>0] = $3134;
        $3137 = HEAP32[$0>>2]|0;
        $3138 = (_sqlite3VdbeHalt($3137)|0);
        $3139 = ($3138|0)==(5);
        if ($3139) {
         $3140 = HEAP32[$pc>>2]|0;
         $3141 = HEAP32[$0>>2]|0;
         $3142 = (($3141) + 76|0);
         HEAP32[$3142>>2] = $3140;
         $3143 = HEAP32[$desiredAutoCommit>>2]|0;
         $3144 = (1 - ($3143))|0;
         $3145 = $3144&255;
         $3146 = HEAP32[$db>>2]|0;
         $3147 = (($3146) + 62|0);
         HEAP8[$3147>>0] = $3145;
         HEAP32[$rc>>2] = 5;
         $3148 = HEAP32[$0>>2]|0;
         $3149 = (($3148) + 80|0);
         HEAP32[$3149>>2] = 5;
         $6969 = $lastRowid;
         $6970 = $6969;
         $6971 = HEAP32[$6970>>2]|0;
         $6972 = (($6969) + 4)|0;
         $6973 = $6972;
         $6974 = HEAP32[$6973>>2]|0;
         $6975 = HEAP32[$db>>2]|0;
         $6976 = (($6975) + 32|0);
         $6977 = $6976;
         $6978 = $6977;
         HEAP32[$6978>>2] = $6971;
         $6979 = (($6977) + 4)|0;
         $6980 = $6979;
         HEAP32[$6980>>2] = $6974;
         $6981 = HEAP32[$nVmStep>>2]|0;
         $6982 = HEAP32[$0>>2]|0;
         $6983 = (($6982) + 108|0);
         $6984 = (($6983) + 16|0);
         $6985 = HEAP32[$6984>>2]|0;
         $6986 = (($6985) + ($6981))|0;
         HEAP32[$6984>>2] = $6986;
         $6987 = HEAP32[$rc>>2]|0;
         STACKTOP = sp;return ($6987|0);
        }
       }
       $3150 = HEAP32[$db>>2]|0;
       _sqlite3CloseSavepoints($3150);
       $3151 = HEAP32[$0>>2]|0;
       $3152 = (($3151) + 80|0);
       $3153 = HEAP32[$3152>>2]|0;
       $3154 = ($3153|0)==(0);
       if ($3154) {
        HEAP32[$rc>>2] = 101;
        $6969 = $lastRowid;
        $6970 = $6969;
        $6971 = HEAP32[$6970>>2]|0;
        $6972 = (($6969) + 4)|0;
        $6973 = $6972;
        $6974 = HEAP32[$6973>>2]|0;
        $6975 = HEAP32[$db>>2]|0;
        $6976 = (($6975) + 32|0);
        $6977 = $6976;
        $6978 = $6977;
        HEAP32[$6978>>2] = $6971;
        $6979 = (($6977) + 4)|0;
        $6980 = $6979;
        HEAP32[$6980>>2] = $6974;
        $6981 = HEAP32[$nVmStep>>2]|0;
        $6982 = HEAP32[$0>>2]|0;
        $6983 = (($6982) + 108|0);
        $6984 = (($6983) + 16|0);
        $6985 = HEAP32[$6984>>2]|0;
        $6986 = (($6985) + ($6981))|0;
        HEAP32[$6984>>2] = $6986;
        $6987 = HEAP32[$rc>>2]|0;
        STACKTOP = sp;return ($6987|0);
       } else {
        HEAP32[$rc>>2] = 1;
        $6969 = $lastRowid;
        $6970 = $6969;
        $6971 = HEAP32[$6970>>2]|0;
        $6972 = (($6969) + 4)|0;
        $6973 = $6972;
        $6974 = HEAP32[$6973>>2]|0;
        $6975 = HEAP32[$db>>2]|0;
        $6976 = (($6975) + 32|0);
        $6977 = $6976;
        $6978 = $6977;
        HEAP32[$6978>>2] = $6971;
        $6979 = (($6977) + 4)|0;
        $6980 = $6979;
        HEAP32[$6980>>2] = $6974;
        $6981 = HEAP32[$nVmStep>>2]|0;
        $6982 = HEAP32[$0>>2]|0;
        $6983 = (($6982) + 108|0);
        $6984 = (($6983) + 16|0);
        $6985 = HEAP32[$6984>>2]|0;
        $6986 = (($6985) + ($6981))|0;
        HEAP32[$6984>>2] = $6986;
        $6987 = HEAP32[$rc>>2]|0;
        STACKTOP = sp;return ($6987|0);
       }
       break;
      }
      case 422: {
       HEAP32[$rc>>2] = 8;
       break;
      }
      case 425: {
       $3191 = HEAP32[$pc>>2]|0;
       $3192 = HEAP32[$0>>2]|0;
       $3193 = (($3192) + 76|0);
       HEAP32[$3193>>2] = $3191;
       HEAP32[$rc>>2] = 5;
       $3194 = HEAP32[$0>>2]|0;
       $3195 = (($3194) + 80|0);
       HEAP32[$3195>>2] = 5;
       $6969 = $lastRowid;
       $6970 = $6969;
       $6971 = HEAP32[$6970>>2]|0;
       $6972 = (($6969) + 4)|0;
       $6973 = $6972;
       $6974 = HEAP32[$6973>>2]|0;
       $6975 = HEAP32[$db>>2]|0;
       $6976 = (($6975) + 32|0);
       $6977 = $6976;
       $6978 = $6977;
       HEAP32[$6978>>2] = $6971;
       $6979 = (($6977) + 4)|0;
       $6980 = $6979;
       HEAP32[$6980>>2] = $6974;
       $6981 = HEAP32[$nVmStep>>2]|0;
       $6982 = HEAP32[$0>>2]|0;
       $6983 = (($6982) + 108|0);
       $6984 = (($6983) + 16|0);
       $6985 = HEAP32[$6984>>2]|0;
       $6986 = (($6985) + ($6981))|0;
       HEAP32[$6984>>2] = $6986;
       $6987 = HEAP32[$rc>>2]|0;
       STACKTOP = sp;return ($6987|0);
       break;
      }
      case 461: {
       $3503 = (_sqlite3CorruptError(72320)|0);
       HEAP32[$rc>>2] = $3503;
       break;
      }
      case 557: {
       HEAP32[$rc>>2] = 13;
       break;
      }
      case 566: {
       HEAP32[$rc>>2] = 13;
       break;
      }
      case 869: {
       $6969 = $lastRowid;
       $6970 = $6969;
       $6971 = HEAP32[$6970>>2]|0;
       $6972 = (($6969) + 4)|0;
       $6973 = $6972;
       $6974 = HEAP32[$6973>>2]|0;
       $6975 = HEAP32[$db>>2]|0;
       $6976 = (($6975) + 32|0);
       $6977 = $6976;
       $6978 = $6977;
       HEAP32[$6978>>2] = $6971;
       $6979 = (($6977) + 4)|0;
       $6980 = $6979;
       HEAP32[$6980>>2] = $6974;
       $6981 = HEAP32[$nVmStep>>2]|0;
       $6982 = HEAP32[$0>>2]|0;
       $6983 = (($6982) + 108|0);
       $6984 = (($6983) + 16|0);
       $6985 = HEAP32[$6984>>2]|0;
       $6986 = (($6985) + ($6981))|0;
       HEAP32[$6984>>2] = $6986;
       $6987 = HEAP32[$rc>>2]|0;
       STACKTOP = sp;return ($6987|0);
       break;
      }
      case 870: {
       $6988 = HEAP32[$0>>2]|0;
       $6989 = (($6988) + 44|0);
       $6990 = HEAP32[$db>>2]|0;
       _sqlite3SetString($6989,$6990,1176,$vararg_buffer84);
       HEAP32[$rc>>2] = 18;
       break L1;
       break;
      }
     }
     $6996 = HEAP32[$db>>2]|0;
     $6997 = (($6996) + 64|0);
     $6998 = HEAP8[$6997>>0]|0;
     $6999 = ($6998<<24>>24)!=(0);
     if ($6999) {
      HEAP32[$rc>>2] = 7;
     }
     $7000 = HEAP32[$rc>>2]|0;
     $7001 = ($7000|0)!=(3082);
     if (!($7001)) {
      break L1;
     }
     $7002 = HEAP32[$0>>2]|0;
     $7003 = (($7002) + 44|0);
     $7004 = HEAP32[$db>>2]|0;
     $7005 = HEAP32[$rc>>2]|0;
     $7006 = (_sqlite3ErrStr($7005)|0);
     HEAP32[$vararg_buffer84>>2] = $7006;
     _sqlite3SetString($7003,$7004,1200,$vararg_buffer84);
     break L1;
    }
   } while(0);
   HEAP32[$rc>>2] = 9;
   $7007 = HEAP32[$rc>>2]|0;
   $7008 = HEAP32[$0>>2]|0;
   $7009 = (($7008) + 80|0);
   HEAP32[$7009>>2] = $7007;
   $7010 = HEAP32[$0>>2]|0;
   $7011 = (($7010) + 44|0);
   $7012 = HEAP32[$db>>2]|0;
   $7013 = HEAP32[$rc>>2]|0;
   $7014 = (_sqlite3ErrStr($7013)|0);
   HEAP32[$vararg_buffer84>>2] = $7014;
   _sqlite3SetString($7011,$7012,1200,$vararg_buffer84);
  }
 } while(0);
 if ((label|0) == 871) {
  $6991 = HEAP32[$db>>2]|0;
  $6992 = (($6991) + 64|0);
  HEAP8[$6992>>0] = 1;
  $6993 = HEAP32[$0>>2]|0;
  $6994 = (($6993) + 44|0);
  $6995 = HEAP32[$db>>2]|0;
  _sqlite3SetString($6994,$6995,5400,$vararg_buffer84);
  HEAP32[$rc>>2] = 7;
 }
 $6946 = HEAP32[$rc>>2]|0;
 $6947 = HEAP32[$0>>2]|0;
 $6948 = (($6947) + 80|0);
 HEAP32[$6948>>2] = $6946;
 $6949 = HEAP32[$rc>>2]|0;
 $6950 = HEAP32[$pc>>2]|0;
 $6951 = HEAP32[$0>>2]|0;
 $6952 = (($6951) + 168|0);
 $6953 = HEAP32[$6952>>2]|0;
 $6954 = HEAP32[$0>>2]|0;
 $6955 = (($6954) + 44|0);
 $6956 = HEAP32[$6955>>2]|0;
 HEAP32[$vararg_buffer84>>2] = $6950;
 $vararg_ptr75 = (($vararg_buffer84) + 4|0);
 HEAP32[$vararg_ptr75>>2] = $6953;
 $vararg_ptr76 = (($vararg_buffer84) + 8|0);
 HEAP32[$vararg_ptr76>>2] = $6956;
 _sqlite3_log($6949,31984,$vararg_buffer84);
 $6957 = HEAP32[$0>>2]|0;
 (_sqlite3VdbeHalt($6957)|0);
 $6958 = HEAP32[$rc>>2]|0;
 $6959 = ($6958|0)==(3082);
 if ($6959) {
  $6960 = HEAP32[$db>>2]|0;
  $6961 = (($6960) + 64|0);
  HEAP8[$6961>>0] = 1;
 }
 HEAP32[$rc>>2] = 1;
 $6962 = HEAP8[$resetSchemaOnFault>>0]|0;
 $6963 = $6962&255;
 $6964 = ($6963|0)>(0);
 if (!($6964)) {
  $6969 = $lastRowid;
  $6970 = $6969;
  $6971 = HEAP32[$6970>>2]|0;
  $6972 = (($6969) + 4)|0;
  $6973 = $6972;
  $6974 = HEAP32[$6973>>2]|0;
  $6975 = HEAP32[$db>>2]|0;
  $6976 = (($6975) + 32|0);
  $6977 = $6976;
  $6978 = $6977;
  HEAP32[$6978>>2] = $6971;
  $6979 = (($6977) + 4)|0;
  $6980 = $6979;
  HEAP32[$6980>>2] = $6974;
  $6981 = HEAP32[$nVmStep>>2]|0;
  $6982 = HEAP32[$0>>2]|0;
  $6983 = (($6982) + 108|0);
  $6984 = (($6983) + 16|0);
  $6985 = HEAP32[$6984>>2]|0;
  $6986 = (($6985) + ($6981))|0;
  HEAP32[$6984>>2] = $6986;
  $6987 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($6987|0);
 }
 $6965 = HEAP32[$db>>2]|0;
 $6966 = HEAP8[$resetSchemaOnFault>>0]|0;
 $6967 = $6966&255;
 $6968 = (($6967) - 1)|0;
 _sqlite3ResetOneSchema($6965,$6968);
 $6969 = $lastRowid;
 $6970 = $6969;
 $6971 = HEAP32[$6970>>2]|0;
 $6972 = (($6969) + 4)|0;
 $6973 = $6972;
 $6974 = HEAP32[$6973>>2]|0;
 $6975 = HEAP32[$db>>2]|0;
 $6976 = (($6975) + 32|0);
 $6977 = $6976;
 $6978 = $6977;
 HEAP32[$6978>>2] = $6971;
 $6979 = (($6977) + 4)|0;
 $6980 = $6979;
 HEAP32[$6980>>2] = $6974;
 $6981 = HEAP32[$nVmStep>>2]|0;
 $6982 = HEAP32[$0>>2]|0;
 $6983 = (($6982) + 108|0);
 $6984 = (($6983) + 16|0);
 $6985 = HEAP32[$6984>>2]|0;
 $6986 = (($6985) + ($6981))|0;
 HEAP32[$6984>>2] = $6986;
 $6987 = HEAP32[$rc>>2]|0;
 STACKTOP = sp;return ($6987|0);
}
function _doWalCallbacks($db) {
 $db = $db|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $5 = 0;
 var $6 = 0, $7 = 0, $8 = 0, $9 = 0, $i = 0, $nEntry = 0, $or$cond = 0, $or$cond3 = 0, $pBt = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 16|0;
 $rc = sp + 12|0;
 $i = sp + 8|0;
 $pBt = sp + 4|0;
 $nEntry = sp;
 HEAP32[$0>>2] = $db;
 HEAP32[$rc>>2] = 0;
 HEAP32[$i>>2] = 0;
 while(1) {
  $1 = HEAP32[$i>>2]|0;
  $2 = HEAP32[$0>>2]|0;
  $3 = (($2) + 20|0);
  $4 = HEAP32[$3>>2]|0;
  $5 = ($1|0)<($4|0);
  if (!($5)) {
   break;
  }
  $6 = HEAP32[$i>>2]|0;
  $7 = HEAP32[$0>>2]|0;
  $8 = (($7) + 16|0);
  $9 = HEAP32[$8>>2]|0;
  $10 = (($9) + ($6<<4)|0);
  $11 = (($10) + 4|0);
  $12 = HEAP32[$11>>2]|0;
  HEAP32[$pBt>>2] = $12;
  $13 = HEAP32[$pBt>>2]|0;
  $14 = ($13|0)!=(0|0);
  if ($14) {
   $15 = HEAP32[$pBt>>2]|0;
   $16 = (_sqlite3BtreePager($15)|0);
   $17 = (_sqlite3PagerWalCallback($16)|0);
   HEAP32[$nEntry>>2] = $17;
   $18 = HEAP32[$0>>2]|0;
   $19 = (($18) + 212|0);
   $20 = HEAP32[$19>>2]|0;
   $21 = ($20|0)!=(0|0);
   $22 = HEAP32[$nEntry>>2]|0;
   $23 = ($22|0)>(0);
   $or$cond = $21 & $23;
   $24 = HEAP32[$rc>>2]|0;
   $25 = ($24|0)==(0);
   $or$cond3 = $or$cond & $25;
   if ($or$cond3) {
    $26 = HEAP32[$0>>2]|0;
    $27 = (($26) + 212|0);
    $28 = HEAP32[$27>>2]|0;
    $29 = HEAP32[$0>>2]|0;
    $30 = (($29) + 216|0);
    $31 = HEAP32[$30>>2]|0;
    $32 = HEAP32[$0>>2]|0;
    $33 = HEAP32[$i>>2]|0;
    $34 = HEAP32[$0>>2]|0;
    $35 = (($34) + 16|0);
    $36 = HEAP32[$35>>2]|0;
    $37 = (($36) + ($33<<4)|0);
    $38 = HEAP32[$37>>2]|0;
    $39 = HEAP32[$nEntry>>2]|0;
    $40 = (FUNCTION_TABLE_iiiii[$28 & 7]($31,$32,$38,$39)|0);
    HEAP32[$rc>>2] = $40;
   }
  }
  $41 = HEAP32[$i>>2]|0;
  $42 = (($41) + 1)|0;
  HEAP32[$i>>2] = $42;
 }
 $43 = HEAP32[$rc>>2]|0;
 STACKTOP = sp;return ($43|0);
}
function _sqlite3VdbeTransferError($p) {
 $p = $p|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $db = 0, $mallocFailed = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $db = sp + 4|0;
 $rc = sp;
 $mallocFailed = sp + 12|0;
 HEAP32[$0>>2] = $p;
 $1 = HEAP32[$0>>2]|0;
 $2 = HEAP32[$1>>2]|0;
 HEAP32[$db>>2] = $2;
 $3 = HEAP32[$0>>2]|0;
 $4 = (($3) + 80|0);
 $5 = HEAP32[$4>>2]|0;
 HEAP32[$rc>>2] = $5;
 $6 = HEAP32[$0>>2]|0;
 $7 = (($6) + 44|0);
 $8 = HEAP32[$7>>2]|0;
 $9 = ($8|0)!=(0|0);
 $10 = HEAP32[$db>>2]|0;
 if (!($9)) {
  $33 = HEAP32[$rc>>2]|0;
  _sqlite3Error($10,$33);
  $34 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($34|0);
 }
 $11 = (($10) + 64|0);
 $12 = HEAP8[$11>>0]|0;
 HEAP8[$mallocFailed>>0] = $12;
 _sqlite3BeginBenignMalloc();
 $13 = HEAP32[$db>>2]|0;
 $14 = (($13) + 232|0);
 $15 = HEAP32[$14>>2]|0;
 $16 = ($15|0)==(0|0);
 if ($16) {
  $17 = HEAP32[$db>>2]|0;
  $18 = (_sqlite3ValueNew($17)|0);
  $19 = HEAP32[$db>>2]|0;
  $20 = (($19) + 232|0);
  HEAP32[$20>>2] = $18;
 }
 $21 = HEAP32[$db>>2]|0;
 $22 = (($21) + 232|0);
 $23 = HEAP32[$22>>2]|0;
 $24 = HEAP32[$0>>2]|0;
 $25 = (($24) + 44|0);
 $26 = HEAP32[$25>>2]|0;
 _sqlite3ValueSetStr($23,-1,$26,1,(-1));
 _sqlite3EndBenignMalloc();
 $27 = HEAP8[$mallocFailed>>0]|0;
 $28 = HEAP32[$db>>2]|0;
 $29 = (($28) + 64|0);
 HEAP8[$29>>0] = $27;
 $30 = HEAP32[$rc>>2]|0;
 $31 = HEAP32[$db>>2]|0;
 $32 = (($31) + 52|0);
 HEAP32[$32>>2] = $30;
 $34 = HEAP32[$rc>>2]|0;
 STACKTOP = sp;return ($34|0);
}
function _sqlite3PagerWalCallback($pPager) {
 $pPager = $pPager|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 HEAP32[$0>>2] = $pPager;
 $1 = HEAP32[$0>>2]|0;
 $2 = (($1) + 208|0);
 $3 = HEAP32[$2>>2]|0;
 $4 = (_sqlite3WalCallback($3)|0);
 STACKTOP = sp;return ($4|0);
}
function _sqlite3WalCallback($pWal) {
 $pWal = $pWal|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $ret = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $ret = sp;
 HEAP32[$0>>2] = $pWal;
 HEAP32[$ret>>2] = 0;
 $1 = HEAP32[$0>>2]|0;
 $2 = ($1|0)!=(0|0);
 if (!($2)) {
  $8 = HEAP32[$ret>>2]|0;
  STACKTOP = sp;return ($8|0);
 }
 $3 = HEAP32[$0>>2]|0;
 $4 = (($3) + 12|0);
 $5 = HEAP32[$4>>2]|0;
 HEAP32[$ret>>2] = $5;
 $6 = HEAP32[$0>>2]|0;
 $7 = (($6) + 12|0);
 HEAP32[$7>>2] = 0;
 $8 = HEAP32[$ret>>2]|0;
 STACKTOP = sp;return ($8|0);
}
function _sqlite3VdbeSetChanges($db,$nChange) {
 $db = $db|0;
 $nChange = $nChange|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $1 = sp;
 HEAP32[$0>>2] = $db;
 HEAP32[$1>>2] = $nChange;
 $2 = HEAP32[$1>>2]|0;
 $3 = HEAP32[$0>>2]|0;
 $4 = (($3) + 80|0);
 HEAP32[$4>>2] = $2;
 $5 = HEAP32[$1>>2]|0;
 $6 = HEAP32[$0>>2]|0;
 $7 = (($6) + 84|0);
 $8 = HEAP32[$7>>2]|0;
 $9 = (($8) + ($5))|0;
 HEAP32[$7>>2] = $9;
 STACKTOP = sp;return;
}
function _sqlite3VdbeFrameRestore($pFrame) {
 $pFrame = $pFrame|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $7 = 0, $8 = 0, $9 = 0, $v = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $v = sp;
 HEAP32[$0>>2] = $pFrame;
 $1 = HEAP32[$0>>2]|0;
 $2 = HEAP32[$1>>2]|0;
 HEAP32[$v>>2] = $2;
 $3 = HEAP32[$0>>2]|0;
 $4 = (($3) + 16|0);
 $5 = HEAP32[$4>>2]|0;
 $6 = HEAP32[$v>>2]|0;
 $7 = (($6) + 200|0);
 HEAP32[$7>>2] = $5;
 $8 = HEAP32[$0>>2]|0;
 $9 = (($8) + 56|0);
 $10 = HEAP32[$9>>2]|0;
 $11 = HEAP32[$v>>2]|0;
 $12 = (($11) + 196|0);
 HEAP32[$12>>2] = $10;
 $13 = HEAP32[$0>>2]|0;
 $14 = (($13) + 8|0);
 $15 = HEAP32[$14>>2]|0;
 $16 = HEAP32[$v>>2]|0;
 $17 = (($16) + 4|0);
 HEAP32[$17>>2] = $15;
 $18 = HEAP32[$0>>2]|0;
 $19 = (($18) + 48|0);
 $20 = HEAP32[$19>>2]|0;
 $21 = HEAP32[$v>>2]|0;
 $22 = (($21) + 32|0);
 HEAP32[$22>>2] = $20;
 $23 = HEAP32[$0>>2]|0;
 $24 = (($23) + 12|0);
 $25 = HEAP32[$24>>2]|0;
 $26 = HEAP32[$v>>2]|0;
 $27 = (($26) + 8|0);
 HEAP32[$27>>2] = $25;
 $28 = HEAP32[$0>>2]|0;
 $29 = (($28) + 52|0);
 $30 = HEAP32[$29>>2]|0;
 $31 = HEAP32[$v>>2]|0;
 $32 = (($31) + 28|0);
 HEAP32[$32>>2] = $30;
 $33 = HEAP32[$0>>2]|0;
 $34 = (($33) + 20|0);
 $35 = HEAP32[$34>>2]|0;
 $36 = HEAP32[$v>>2]|0;
 $37 = (($36) + 56|0);
 HEAP32[$37>>2] = $35;
 $38 = HEAP32[$0>>2]|0;
 $39 = (($38) + 40|0);
 $40 = HEAP32[$39>>2]|0;
 $41 = HEAP32[$v>>2]|0;
 $42 = (($41) + 36|0);
 HEAP32[$42>>2] = $40;
 $43 = HEAP32[$0>>2]|0;
 $44 = (($43) + 32|0);
 $45 = $44;
 $46 = $45;
 $47 = HEAP32[$46>>2]|0;
 $48 = (($45) + 4)|0;
 $49 = $48;
 $50 = HEAP32[$49>>2]|0;
 $51 = HEAP32[$v>>2]|0;
 $52 = HEAP32[$51>>2]|0;
 $53 = (($52) + 32|0);
 $54 = $53;
 $55 = $54;
 HEAP32[$55>>2] = $47;
 $56 = (($54) + 4)|0;
 $57 = $56;
 HEAP32[$57>>2] = $50;
 $58 = HEAP32[$0>>2]|0;
 $59 = (($58) + 68|0);
 $60 = HEAP32[$59>>2]|0;
 $61 = HEAP32[$v>>2]|0;
 $62 = (($61) + 92|0);
 HEAP32[$62>>2] = $60;
 $63 = HEAP32[$0>>2]|0;
 $64 = (($63) + 44|0);
 $65 = HEAP32[$64>>2]|0;
 STACKTOP = sp;return ($65|0);
}
function _sqlite3VdbeHalt($p) {
 $p = $p|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0;
 var $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0, $188 = 0;
 var $189 = 0, $19 = 0, $190 = 0, $191 = 0, $192 = 0, $193 = 0, $194 = 0, $195 = 0, $196 = 0, $197 = 0, $198 = 0, $199 = 0, $2 = 0, $20 = 0, $200 = 0, $201 = 0, $202 = 0, $203 = 0, $204 = 0, $205 = 0;
 var $206 = 0, $207 = 0, $208 = 0, $209 = 0, $21 = 0, $210 = 0, $211 = 0, $212 = 0, $213 = 0, $214 = 0, $215 = 0, $216 = 0, $217 = 0, $218 = 0, $219 = 0, $22 = 0, $220 = 0, $221 = 0, $222 = 0, $223 = 0;
 var $224 = 0, $225 = 0, $226 = 0, $227 = 0, $228 = 0, $229 = 0, $23 = 0, $230 = 0, $231 = 0, $232 = 0, $233 = 0, $234 = 0, $235 = 0, $236 = 0, $237 = 0, $238 = 0, $239 = 0, $24 = 0, $240 = 0, $241 = 0;
 var $242 = 0, $243 = 0, $244 = 0, $245 = 0, $246 = 0, $247 = 0, $248 = 0, $249 = 0, $25 = 0, $250 = 0, $251 = 0, $252 = 0, $253 = 0, $254 = 0, $255 = 0, $256 = 0, $257 = 0, $258 = 0, $259 = 0, $26 = 0;
 var $260 = 0, $261 = 0, $262 = 0, $263 = 0, $264 = 0, $265 = 0, $266 = 0, $267 = 0, $268 = 0, $269 = 0, $27 = 0, $270 = 0, $271 = 0, $272 = 0, $273 = 0, $274 = 0, $275 = 0, $28 = 0, $29 = 0, $3 = 0;
 var $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0;
 var $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0;
 var $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0;
 var $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $db = 0, $eStatementOp = 0, $isSpecialError = 0, $mrc = 0;
 var $or$cond = 0, $or$cond3 = 0, $or$cond5 = 0, $or$cond7 = 0, $or$cond9 = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 24|0;
 $1 = sp + 20|0;
 $rc = sp + 16|0;
 $db = sp + 12|0;
 $mrc = sp + 8|0;
 $eStatementOp = sp + 4|0;
 $isSpecialError = sp;
 HEAP32[$1>>2] = $p;
 $2 = HEAP32[$1>>2]|0;
 $3 = HEAP32[$2>>2]|0;
 HEAP32[$db>>2] = $3;
 $4 = HEAP32[$1>>2]|0;
 $5 = HEAP32[$4>>2]|0;
 $6 = (($5) + 64|0);
 $7 = HEAP8[$6>>0]|0;
 $8 = ($7<<24>>24)!=(0);
 if ($8) {
  $9 = HEAP32[$1>>2]|0;
  $10 = (($9) + 80|0);
  HEAP32[$10>>2] = 7;
 }
 $11 = HEAP32[$1>>2]|0;
 $12 = (($11) + 200|0);
 $13 = HEAP32[$12>>2]|0;
 $14 = ($13|0)!=(0|0);
 if ($14) {
  $15 = HEAP32[$1>>2]|0;
  $16 = (($15) + 200|0);
  $17 = HEAP32[$16>>2]|0;
  $18 = HEAP32[$1>>2]|0;
  $19 = (($18) + 196|0);
  $20 = HEAP32[$19>>2]|0;
  _memset(($17|0),0,($20|0))|0;
 }
 $21 = HEAP32[$1>>2]|0;
 _closeAllCursors($21);
 $22 = HEAP32[$1>>2]|0;
 $23 = (($22) + 40|0);
 $24 = HEAP32[$23>>2]|0;
 $25 = ($24|0)!=(-1108210269);
 if ($25) {
  HEAP32[$0>>2] = 0;
  $275 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($275|0);
 }
 $26 = HEAP32[$1>>2]|0;
 $27 = (($26) + 76|0);
 $28 = HEAP32[$27>>2]|0;
 $29 = ($28|0)>=(0);
 if ($29) {
  $30 = HEAP32[$1>>2]|0;
  $31 = (($30) + 88|0);
  $32 = HEAP16[$31>>1]|0;
  $33 = ($32&65535) >>> 9;
  $34 = $33 & 1;
  $35 = $34&65535;
  $36 = ($35|0)!=(0);
  if ($36) {
   HEAP32[$eStatementOp>>2] = 0;
   $37 = HEAP32[$1>>2]|0;
   $38 = (($37) + 80|0);
   $39 = HEAP32[$38>>2]|0;
   $40 = $39 & 255;
   HEAP32[$mrc>>2] = $40;
   $41 = HEAP32[$mrc>>2]|0;
   $42 = ($41|0)==(7);
   $43 = HEAP32[$mrc>>2]|0;
   $44 = ($43|0)==(10);
   $or$cond = $42 | $44;
   $45 = HEAP32[$mrc>>2]|0;
   $46 = ($45|0)==(9);
   $or$cond3 = $or$cond | $46;
   if ($or$cond3) {
    $50 = 1;
   } else {
    $47 = HEAP32[$mrc>>2]|0;
    $48 = ($47|0)==(13);
    $50 = $48;
   }
   $49 = $50&1;
   HEAP32[$isSpecialError>>2] = $49;
   $51 = HEAP32[$isSpecialError>>2]|0;
   $52 = ($51|0)!=(0);
   do {
    if ($52) {
     $53 = HEAP32[$1>>2]|0;
     $54 = (($53) + 88|0);
     $55 = HEAP16[$54>>1]|0;
     $56 = ($55&65535) >>> 8;
     $57 = $56 & 1;
     $58 = $57&65535;
     $59 = ($58|0)==(0);
     $60 = HEAP32[$mrc>>2]|0;
     $61 = ($60|0)!=(9);
     $or$cond5 = $59 | $61;
     if ($or$cond5) {
      $62 = HEAP32[$mrc>>2]|0;
      $63 = ($62|0)==(7);
      $64 = HEAP32[$mrc>>2]|0;
      $65 = ($64|0)==(13);
      $or$cond7 = $63 | $65;
      if ($or$cond7) {
       $66 = HEAP32[$1>>2]|0;
       $67 = (($66) + 88|0);
       $68 = HEAP16[$67>>1]|0;
       $69 = ($68&65535) >>> 7;
       $70 = $69 & 1;
       $71 = $70&65535;
       $72 = ($71|0)!=(0);
       if ($72) {
        HEAP32[$eStatementOp>>2] = 2;
        break;
       }
      }
      $73 = HEAP32[$db>>2]|0;
      _sqlite3RollbackAll($73,516);
      $74 = HEAP32[$db>>2]|0;
      _sqlite3CloseSavepoints($74);
      $75 = HEAP32[$db>>2]|0;
      $76 = (($75) + 62|0);
      HEAP8[$76>>0] = 1;
     }
    }
   } while(0);
   $77 = HEAP32[$1>>2]|0;
   $78 = (($77) + 80|0);
   $79 = HEAP32[$78>>2]|0;
   $80 = ($79|0)==(0);
   if ($80) {
    $81 = HEAP32[$1>>2]|0;
    (_sqlite3VdbeCheckFk($81,0)|0);
   }
   $82 = HEAP32[$db>>2]|0;
   $83 = (($82) + 304|0);
   $84 = HEAP32[$83>>2]|0;
   $85 = ($84|0)>(0);
   if ($85) {
    $86 = HEAP32[$db>>2]|0;
    $87 = (($86) + 328|0);
    $88 = HEAP32[$87>>2]|0;
    $89 = ($88|0)==(0|0);
    if ($89) {
     label = 38;
    } else {
     label = 21;
    }
   } else {
    label = 21;
   }
   if ((label|0) == 21) {
    $90 = HEAP32[$db>>2]|0;
    $91 = (($90) + 62|0);
    $92 = HEAP8[$91>>0]|0;
    $93 = $92&255;
    $94 = ($93|0)!=(0);
    if ($94) {
     $95 = HEAP32[$db>>2]|0;
     $96 = (($95) + 156|0);
     $97 = HEAP32[$96>>2]|0;
     $98 = HEAP32[$1>>2]|0;
     $99 = (($98) + 88|0);
     $100 = HEAP16[$99>>1]|0;
     $101 = ($100&65535) >>> 8;
     $102 = $101 & 1;
     $103 = $102&65535;
     $104 = ($103|0)==(0);
     $105 = $104&1;
     $106 = ($97|0)==($105|0);
     if ($106) {
      $107 = HEAP32[$1>>2]|0;
      $108 = (($107) + 80|0);
      $109 = HEAP32[$108>>2]|0;
      $110 = ($109|0)==(0);
      if ($110) {
       label = 25;
      } else {
       $111 = HEAP32[$1>>2]|0;
       $112 = (($111) + 86|0);
       $113 = HEAP8[$112>>0]|0;
       $114 = $113&255;
       $115 = ($114|0)!=(3);
       $116 = HEAP32[$isSpecialError>>2]|0;
       $117 = ($116|0)!=(0);
       $or$cond9 = $115 | $117;
       if ($or$cond9) {
        $164 = HEAP32[$db>>2]|0;
        _sqlite3RollbackAll($164,0);
       } else {
        label = 25;
       }
      }
      do {
       if ((label|0) == 25) {
        $118 = HEAP32[$1>>2]|0;
        $119 = (_sqlite3VdbeCheckFk($118,1)|0);
        HEAP32[$rc>>2] = $119;
        $120 = HEAP32[$rc>>2]|0;
        $121 = ($120|0)!=(0);
        do {
         if ($121) {
          $122 = HEAP32[$1>>2]|0;
          $123 = (($122) + 88|0);
          $124 = HEAP16[$123>>1]|0;
          $125 = ($124&65535) >>> 8;
          $126 = $125 & 1;
          $127 = $126&65535;
          $128 = ($127|0)!=(0);
          if (!($128)) {
           HEAP32[$rc>>2] = 787;
           break;
          }
          HEAP32[$0>>2] = 1;
          $275 = HEAP32[$0>>2]|0;
          STACKTOP = sp;return ($275|0);
         } else {
          $129 = HEAP32[$db>>2]|0;
          $130 = HEAP32[$1>>2]|0;
          $131 = (_vdbeCommit($129,$130)|0);
          HEAP32[$rc>>2] = $131;
         }
        } while(0);
        $132 = HEAP32[$rc>>2]|0;
        $133 = ($132|0)==(5);
        if ($133) {
         $134 = HEAP32[$1>>2]|0;
         $135 = (($134) + 88|0);
         $136 = HEAP16[$135>>1]|0;
         $137 = ($136&65535) >>> 8;
         $138 = $137 & 1;
         $139 = $138&65535;
         $140 = ($139|0)!=(0);
         if ($140) {
          HEAP32[$0>>2] = 5;
          $275 = HEAP32[$0>>2]|0;
          STACKTOP = sp;return ($275|0);
         }
        }
        $141 = HEAP32[$rc>>2]|0;
        $142 = ($141|0)!=(0);
        if ($142) {
         $143 = HEAP32[$rc>>2]|0;
         $144 = HEAP32[$1>>2]|0;
         $145 = (($144) + 80|0);
         HEAP32[$145>>2] = $143;
         $146 = HEAP32[$db>>2]|0;
         _sqlite3RollbackAll($146,0);
         break;
        } else {
         $147 = HEAP32[$db>>2]|0;
         $148 = (($147) + 504|0);
         $149 = $148;
         $150 = $149;
         HEAP32[$150>>2] = 0;
         $151 = (($149) + 4)|0;
         $152 = $151;
         HEAP32[$152>>2] = 0;
         $153 = HEAP32[$db>>2]|0;
         $154 = (($153) + 512|0);
         $155 = $154;
         $156 = $155;
         HEAP32[$156>>2] = 0;
         $157 = (($155) + 4)|0;
         $158 = $157;
         HEAP32[$158>>2] = 0;
         $159 = HEAP32[$db>>2]|0;
         $160 = (($159) + 24|0);
         $161 = HEAP32[$160>>2]|0;
         $162 = $161 & -16777217;
         HEAP32[$160>>2] = $162;
         $163 = HEAP32[$db>>2]|0;
         _sqlite3CommitInternalChanges($163);
         break;
        }
       }
      } while(0);
      $165 = HEAP32[$db>>2]|0;
      $166 = (($165) + 500|0);
      HEAP32[$166>>2] = 0;
     } else {
      label = 38;
     }
    } else {
     label = 38;
    }
   }
   do {
    if ((label|0) == 38) {
     $167 = HEAP32[$eStatementOp>>2]|0;
     $168 = ($167|0)==(0);
     if ($168) {
      $169 = HEAP32[$1>>2]|0;
      $170 = (($169) + 80|0);
      $171 = HEAP32[$170>>2]|0;
      $172 = ($171|0)==(0);
      if (!($172)) {
       $173 = HEAP32[$1>>2]|0;
       $174 = (($173) + 86|0);
       $175 = HEAP8[$174>>0]|0;
       $176 = $175&255;
       $177 = ($176|0)==(3);
       if (!($177)) {
        $178 = HEAP32[$1>>2]|0;
        $179 = (($178) + 86|0);
        $180 = HEAP8[$179>>0]|0;
        $181 = $180&255;
        $182 = ($181|0)==(2);
        if ($182) {
         HEAP32[$eStatementOp>>2] = 2;
         break;
        } else {
         $183 = HEAP32[$db>>2]|0;
         _sqlite3RollbackAll($183,516);
         $184 = HEAP32[$db>>2]|0;
         _sqlite3CloseSavepoints($184);
         $185 = HEAP32[$db>>2]|0;
         $186 = (($185) + 62|0);
         HEAP8[$186>>0] = 1;
         break;
        }
       }
      }
      HEAP32[$eStatementOp>>2] = 1;
     }
    }
   } while(0);
   $187 = HEAP32[$eStatementOp>>2]|0;
   $188 = ($187|0)!=(0);
   if ($188) {
    $189 = HEAP32[$1>>2]|0;
    $190 = HEAP32[$eStatementOp>>2]|0;
    $191 = (_sqlite3VdbeCloseStatement($189,$190)|0);
    HEAP32[$rc>>2] = $191;
    $192 = HEAP32[$rc>>2]|0;
    $193 = ($192|0)!=(0);
    if ($193) {
     $194 = HEAP32[$1>>2]|0;
     $195 = (($194) + 80|0);
     $196 = HEAP32[$195>>2]|0;
     $197 = ($196|0)==(0);
     if ($197) {
      label = 49;
     } else {
      $198 = HEAP32[$1>>2]|0;
      $199 = (($198) + 80|0);
      $200 = HEAP32[$199>>2]|0;
      $201 = $200 & 255;
      $202 = ($201|0)==(19);
      if ($202) {
       label = 49;
      }
     }
     if ((label|0) == 49) {
      $203 = HEAP32[$rc>>2]|0;
      $204 = HEAP32[$1>>2]|0;
      $205 = (($204) + 80|0);
      HEAP32[$205>>2] = $203;
      $206 = HEAP32[$db>>2]|0;
      $207 = HEAP32[$1>>2]|0;
      $208 = (($207) + 44|0);
      $209 = HEAP32[$208>>2]|0;
      _sqlite3DbFree($206,$209);
      $210 = HEAP32[$1>>2]|0;
      $211 = (($210) + 44|0);
      HEAP32[$211>>2] = 0;
     }
     $212 = HEAP32[$db>>2]|0;
     _sqlite3RollbackAll($212,516);
     $213 = HEAP32[$db>>2]|0;
     _sqlite3CloseSavepoints($213);
     $214 = HEAP32[$db>>2]|0;
     $215 = (($214) + 62|0);
     HEAP8[$215>>0] = 1;
    }
   }
   $216 = HEAP32[$1>>2]|0;
   $217 = (($216) + 88|0);
   $218 = HEAP16[$217>>1]|0;
   $219 = ($218&65535) >>> 4;
   $220 = $219 & 1;
   $221 = $220&65535;
   $222 = ($221|0)!=(0);
   if ($222) {
    $223 = HEAP32[$eStatementOp>>2]|0;
    $224 = ($223|0)!=(2);
    $225 = HEAP32[$db>>2]|0;
    if ($224) {
     $226 = HEAP32[$1>>2]|0;
     $227 = (($226) + 92|0);
     $228 = HEAP32[$227>>2]|0;
     _sqlite3VdbeSetChanges($225,$228);
    } else {
     _sqlite3VdbeSetChanges($225,0);
    }
    $229 = HEAP32[$1>>2]|0;
    $230 = (($229) + 92|0);
    HEAP32[$230>>2] = 0;
   }
  }
 }
 $231 = HEAP32[$1>>2]|0;
 $232 = (($231) + 76|0);
 $233 = HEAP32[$232>>2]|0;
 $234 = ($233|0)>=(0);
 if ($234) {
  $235 = HEAP32[$db>>2]|0;
  $236 = (($235) + 148|0);
  $237 = HEAP32[$236>>2]|0;
  $238 = (($237) + -1)|0;
  HEAP32[$236>>2] = $238;
  $239 = HEAP32[$1>>2]|0;
  $240 = (($239) + 88|0);
  $241 = HEAP16[$240>>1]|0;
  $242 = ($241&65535) >>> 8;
  $243 = $242 & 1;
  $244 = $243&65535;
  $245 = ($244|0)!=(0);
  if (!($245)) {
   $246 = HEAP32[$db>>2]|0;
   $247 = (($246) + 156|0);
   $248 = HEAP32[$247>>2]|0;
   $249 = (($248) + -1)|0;
   HEAP32[$247>>2] = $249;
  }
  $250 = HEAP32[$1>>2]|0;
  $251 = (($250) + 88|0);
  $252 = HEAP16[$251>>1]|0;
  $253 = ($252&65535) >>> 9;
  $254 = $253 & 1;
  $255 = $254&65535;
  $256 = ($255|0)!=(0);
  if ($256) {
   $257 = HEAP32[$db>>2]|0;
   $258 = (($257) + 152|0);
   $259 = HEAP32[$258>>2]|0;
   $260 = (($259) + -1)|0;
   HEAP32[$258>>2] = $260;
  }
 }
 $261 = HEAP32[$1>>2]|0;
 $262 = (($261) + 40|0);
 HEAP32[$262>>2] = 1369188723;
 $263 = HEAP32[$1>>2]|0;
 $264 = HEAP32[$263>>2]|0;
 $265 = (($264) + 64|0);
 $266 = HEAP8[$265>>0]|0;
 $267 = ($266<<24>>24)!=(0);
 if ($267) {
  $268 = HEAP32[$1>>2]|0;
  $269 = (($268) + 80|0);
  HEAP32[$269>>2] = 7;
 }
 $270 = HEAP32[$1>>2]|0;
 $271 = (($270) + 80|0);
 $272 = HEAP32[$271>>2]|0;
 $273 = ($272|0)==(5);
 $274 = $273 ? 5 : 0;
 HEAP32[$0>>2] = $274;
 $275 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($275|0);
}
function _sqlite3VdbeMemTooBig($p) {
 $p = $p|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $n = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $1 = sp + 4|0;
 $n = sp;
 HEAP32[$1>>2] = $p;
 $2 = HEAP32[$1>>2]|0;
 $3 = (($2) + 8|0);
 $4 = HEAP16[$3>>1]|0;
 $5 = $4&65535;
 $6 = $5 & 18;
 $7 = ($6|0)!=(0);
 if (!($7)) {
  HEAP32[$0>>2] = 0;
  $29 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($29|0);
 }
 $8 = HEAP32[$1>>2]|0;
 $9 = (($8) + 12|0);
 $10 = HEAP32[$9>>2]|0;
 HEAP32[$n>>2] = $10;
 $11 = HEAP32[$1>>2]|0;
 $12 = (($11) + 8|0);
 $13 = HEAP16[$12>>1]|0;
 $14 = $13&65535;
 $15 = $14 & 16384;
 $16 = ($15|0)!=(0);
 if ($16) {
  $17 = HEAP32[$1>>2]|0;
  $18 = HEAP32[$17>>2]|0;
  $19 = HEAP32[$n>>2]|0;
  $20 = (($19) + ($18))|0;
  HEAP32[$n>>2] = $20;
 }
 $21 = HEAP32[$n>>2]|0;
 $22 = HEAP32[$1>>2]|0;
 $23 = (($22) + 32|0);
 $24 = HEAP32[$23>>2]|0;
 $25 = (($24) + 88|0);
 $26 = HEAP32[$25>>2]|0;
 $27 = ($21|0)>($26|0);
 $28 = $27&1;
 HEAP32[$0>>2] = $28;
 $29 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($29|0);
}
function _sqlite3VdbeCheckFk($p,$deferred) {
 $p = $p|0;
 $deferred = $deferred|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $db = 0, $vararg_buffer = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $vararg_buffer = sp;
 $0 = sp + 16|0;
 $1 = sp + 12|0;
 $2 = sp + 8|0;
 $db = sp + 4|0;
 HEAP32[$1>>2] = $p;
 HEAP32[$2>>2] = $deferred;
 $3 = HEAP32[$1>>2]|0;
 $4 = HEAP32[$3>>2]|0;
 HEAP32[$db>>2] = $4;
 $5 = HEAP32[$2>>2]|0;
 $6 = ($5|0)!=(0);
 if ($6) {
  $7 = HEAP32[$db>>2]|0;
  $8 = (($7) + 504|0);
  $9 = $8;
  $10 = $9;
  $11 = HEAP32[$10>>2]|0;
  $12 = (($9) + 4)|0;
  $13 = $12;
  $14 = HEAP32[$13>>2]|0;
  $15 = HEAP32[$db>>2]|0;
  $16 = (($15) + 512|0);
  $17 = $16;
  $18 = $17;
  $19 = HEAP32[$18>>2]|0;
  $20 = (($17) + 4)|0;
  $21 = $20;
  $22 = HEAP32[$21>>2]|0;
  $23 = (_i64Add(($11|0),($14|0),($19|0),($22|0))|0);
  $24 = tempRet0;
  $25 = ($24|0)>(0);
  $26 = ($23>>>0)>(0);
  $27 = ($24|0)==(0);
  $28 = $27 & $26;
  $29 = $25 | $28;
  if (!($29)) {
   label = 3;
  }
 } else {
  label = 3;
 }
 do {
  if ((label|0) == 3) {
   $30 = HEAP32[$2>>2]|0;
   $31 = ($30|0)!=(0);
   if (!($31)) {
    $32 = HEAP32[$1>>2]|0;
    $33 = (($32) + 144|0);
    $34 = $33;
    $35 = $34;
    $36 = HEAP32[$35>>2]|0;
    $37 = (($34) + 4)|0;
    $38 = $37;
    $39 = HEAP32[$38>>2]|0;
    $40 = ($39|0)>(0);
    $41 = ($36>>>0)>(0);
    $42 = ($39|0)==(0);
    $43 = $42 & $41;
    $44 = $40 | $43;
    if ($44) {
     break;
    }
   }
   HEAP32[$0>>2] = 0;
   $52 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($52|0);
  }
 } while(0);
 $45 = HEAP32[$1>>2]|0;
 $46 = (($45) + 80|0);
 HEAP32[$46>>2] = 787;
 $47 = HEAP32[$1>>2]|0;
 $48 = (($47) + 86|0);
 HEAP8[$48>>0] = 2;
 $49 = HEAP32[$1>>2]|0;
 $50 = (($49) + 44|0);
 $51 = HEAP32[$db>>2]|0;
 _sqlite3SetString($50,$51,24400,$vararg_buffer);
 HEAP32[$0>>2] = 1;
 $52 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($52|0);
}
function _sqlite3VdbeCloseStatement($p,$eOp) {
 $p = $p|0;
 $eOp = $eOp|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0;
 var $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $db = 0, $i = 0, $iSavepoint = 0, $pBt = 0, $rc = 0, $rc2 = 0;
 var label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 28|0;
 $1 = sp + 24|0;
 $db = sp + 20|0;
 $rc = sp + 16|0;
 $i = sp + 12|0;
 $iSavepoint = sp + 8|0;
 $rc2 = sp + 4|0;
 $pBt = sp;
 HEAP32[$0>>2] = $p;
 HEAP32[$1>>2] = $eOp;
 $2 = HEAP32[$0>>2]|0;
 $3 = HEAP32[$2>>2]|0;
 HEAP32[$db>>2] = $3;
 HEAP32[$rc>>2] = 0;
 $4 = HEAP32[$db>>2]|0;
 $5 = (($4) + 500|0);
 $6 = HEAP32[$5>>2]|0;
 $7 = ($6|0)!=(0);
 if (!($7)) {
  $93 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($93|0);
 }
 $8 = HEAP32[$0>>2]|0;
 $9 = (($8) + 104|0);
 $10 = HEAP32[$9>>2]|0;
 $11 = ($10|0)!=(0);
 if (!($11)) {
  $93 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($93|0);
 }
 $12 = HEAP32[$0>>2]|0;
 $13 = (($12) + 104|0);
 $14 = HEAP32[$13>>2]|0;
 $15 = (($14) - 1)|0;
 HEAP32[$iSavepoint>>2] = $15;
 HEAP32[$i>>2] = 0;
 while(1) {
  $16 = HEAP32[$i>>2]|0;
  $17 = HEAP32[$db>>2]|0;
  $18 = (($17) + 20|0);
  $19 = HEAP32[$18>>2]|0;
  $20 = ($16|0)<($19|0);
  if (!($20)) {
   break;
  }
  HEAP32[$rc2>>2] = 0;
  $21 = HEAP32[$i>>2]|0;
  $22 = HEAP32[$db>>2]|0;
  $23 = (($22) + 16|0);
  $24 = HEAP32[$23>>2]|0;
  $25 = (($24) + ($21<<4)|0);
  $26 = (($25) + 4|0);
  $27 = HEAP32[$26>>2]|0;
  HEAP32[$pBt>>2] = $27;
  $28 = HEAP32[$pBt>>2]|0;
  $29 = ($28|0)!=(0|0);
  if ($29) {
   $30 = HEAP32[$1>>2]|0;
   $31 = ($30|0)==(2);
   if ($31) {
    $32 = HEAP32[$pBt>>2]|0;
    $33 = HEAP32[$iSavepoint>>2]|0;
    $34 = (_sqlite3BtreeSavepoint($32,2,$33)|0);
    HEAP32[$rc2>>2] = $34;
   }
   $35 = HEAP32[$rc2>>2]|0;
   $36 = ($35|0)==(0);
   if ($36) {
    $37 = HEAP32[$pBt>>2]|0;
    $38 = HEAP32[$iSavepoint>>2]|0;
    $39 = (_sqlite3BtreeSavepoint($37,1,$38)|0);
    HEAP32[$rc2>>2] = $39;
   }
   $40 = HEAP32[$rc>>2]|0;
   $41 = ($40|0)==(0);
   if ($41) {
    $42 = HEAP32[$rc2>>2]|0;
    HEAP32[$rc>>2] = $42;
   }
  }
  $43 = HEAP32[$i>>2]|0;
  $44 = (($43) + 1)|0;
  HEAP32[$i>>2] = $44;
 }
 $45 = HEAP32[$db>>2]|0;
 $46 = (($45) + 500|0);
 $47 = HEAP32[$46>>2]|0;
 $48 = (($47) + -1)|0;
 HEAP32[$46>>2] = $48;
 $49 = HEAP32[$0>>2]|0;
 $50 = (($49) + 104|0);
 HEAP32[$50>>2] = 0;
 $51 = HEAP32[$rc>>2]|0;
 $52 = ($51|0)==(0);
 if ($52) {
  $53 = HEAP32[$1>>2]|0;
  $54 = ($53|0)==(2);
  if ($54) {
   $55 = HEAP32[$db>>2]|0;
   $56 = HEAP32[$iSavepoint>>2]|0;
   $57 = (_sqlite3VtabSavepoint($55,2,$56)|0);
   HEAP32[$rc>>2] = $57;
  }
  $58 = HEAP32[$rc>>2]|0;
  $59 = ($58|0)==(0);
  if ($59) {
   $60 = HEAP32[$db>>2]|0;
   $61 = HEAP32[$iSavepoint>>2]|0;
   $62 = (_sqlite3VtabSavepoint($60,1,$61)|0);
   HEAP32[$rc>>2] = $62;
  }
 }
 $63 = HEAP32[$1>>2]|0;
 $64 = ($63|0)==(2);
 if (!($64)) {
  $93 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($93|0);
 }
 $65 = HEAP32[$0>>2]|0;
 $66 = (($65) + 152|0);
 $67 = $66;
 $68 = $67;
 $69 = HEAP32[$68>>2]|0;
 $70 = (($67) + 4)|0;
 $71 = $70;
 $72 = HEAP32[$71>>2]|0;
 $73 = HEAP32[$db>>2]|0;
 $74 = (($73) + 504|0);
 $75 = $74;
 $76 = $75;
 HEAP32[$76>>2] = $69;
 $77 = (($75) + 4)|0;
 $78 = $77;
 HEAP32[$78>>2] = $72;
 $79 = HEAP32[$0>>2]|0;
 $80 = (($79) + 160|0);
 $81 = $80;
 $82 = $81;
 $83 = HEAP32[$82>>2]|0;
 $84 = (($81) + 4)|0;
 $85 = $84;
 $86 = HEAP32[$85>>2]|0;
 $87 = HEAP32[$db>>2]|0;
 $88 = (($87) + 512|0);
 $89 = $88;
 $90 = $89;
 HEAP32[$90>>2] = $83;
 $91 = (($89) + 4)|0;
 $92 = $91;
 HEAP32[$92>>2] = $86;
 $93 = HEAP32[$rc>>2]|0;
 STACKTOP = sp;return ($93|0);
}
function _sqlite3VdbeMemNulTerminate($pMem) {
 $pMem = $pMem|0;
 var $0 = 0, $1 = 0, $10 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $1 = sp;
 HEAP32[$1>>2] = $pMem;
 $2 = HEAP32[$1>>2]|0;
 $3 = (($2) + 8|0);
 $4 = HEAP16[$3>>1]|0;
 $5 = $4&65535;
 $6 = $5 & 514;
 $7 = ($6|0)!=(2);
 if ($7) {
  HEAP32[$0>>2] = 0;
  $10 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($10|0);
 } else {
  $8 = HEAP32[$1>>2]|0;
  $9 = (_vdbeMemAddTerminator($8)|0);
  HEAP32[$0>>2] = $9;
  $10 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($10|0);
 }
 return 0|0;
}
function _numericType($pMem) {
 $pMem = $pMem|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0;
 var sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $1 = sp;
 HEAP32[$1>>2] = $pMem;
 $2 = HEAP32[$1>>2]|0;
 $3 = (($2) + 8|0);
 $4 = HEAP16[$3>>1]|0;
 $5 = $4&65535;
 $6 = $5 & 12;
 $7 = ($6|0)!=(0);
 $8 = HEAP32[$1>>2]|0;
 $9 = (($8) + 8|0);
 $10 = HEAP16[$9>>1]|0;
 $11 = $10&65535;
 do {
  if ($7) {
   $12 = $11 & 12;
   $13 = $12&65535;
   HEAP16[$0>>1] = $13;
  } else {
   $14 = $11 & 18;
   $15 = ($14|0)!=(0);
   if ($15) {
    $16 = HEAP32[$1>>2]|0;
    $17 = (_computeNumericType($16)|0);
    HEAP16[$0>>1] = $17;
    break;
   } else {
    HEAP16[$0>>1] = 0;
    break;
   }
  }
 } while(0);
 $18 = HEAP16[$0>>1]|0;
 STACKTOP = sp;return ($18|0);
}
function _sqlite3SubInt64($pA,$0,$1) {
 $pA = $pA|0;
 $0 = $0|0;
 $1 = $1|0;
 var $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0;
 var $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0;
 var $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $2 = sp + 12|0;
 $3 = sp + 8|0;
 $4 = sp;
 HEAP32[$3>>2] = $pA;
 $5 = $4;
 $6 = $5;
 HEAP32[$6>>2] = $0;
 $7 = (($5) + 4)|0;
 $8 = $7;
 HEAP32[$8>>2] = $1;
 $9 = $4;
 $10 = $9;
 $11 = HEAP32[$10>>2]|0;
 $12 = (($9) + 4)|0;
 $13 = $12;
 $14 = HEAP32[$13>>2]|0;
 $15 = ($11|0)==(0);
 $16 = ($14|0)==(-2147483648);
 $17 = $15 & $16;
 $18 = HEAP32[$3>>2]|0;
 if (!($17)) {
  $49 = $4;
  $50 = $49;
  $51 = HEAP32[$50>>2]|0;
  $52 = (($49) + 4)|0;
  $53 = $52;
  $54 = HEAP32[$53>>2]|0;
  $55 = (_i64Subtract(0,0,($51|0),($54|0))|0);
  $56 = tempRet0;
  $57 = (_sqlite3AddInt64($18,$55,$56)|0);
  HEAP32[$2>>2] = $57;
  $58 = HEAP32[$2>>2]|0;
  STACKTOP = sp;return ($58|0);
 }
 $19 = $18;
 $20 = $19;
 $21 = HEAP32[$20>>2]|0;
 $22 = (($19) + 4)|0;
 $23 = $22;
 $24 = HEAP32[$23>>2]|0;
 $25 = ($24|0)>(0);
 $26 = ($21>>>0)>=(0);
 $27 = ($24|0)==(0);
 $28 = $27 & $26;
 $29 = $25 | $28;
 if ($29) {
  HEAP32[$2>>2] = 1;
  $58 = HEAP32[$2>>2]|0;
  STACKTOP = sp;return ($58|0);
 } else {
  $30 = $4;
  $31 = $30;
  $32 = HEAP32[$31>>2]|0;
  $33 = (($30) + 4)|0;
  $34 = $33;
  $35 = HEAP32[$34>>2]|0;
  $36 = HEAP32[$3>>2]|0;
  $37 = $36;
  $38 = $37;
  $39 = HEAP32[$38>>2]|0;
  $40 = (($37) + 4)|0;
  $41 = $40;
  $42 = HEAP32[$41>>2]|0;
  $43 = (_i64Subtract(($39|0),($42|0),($32|0),($35|0))|0);
  $44 = tempRet0;
  $45 = $36;
  $46 = $45;
  HEAP32[$46>>2] = $43;
  $47 = (($45) + 4)|0;
  $48 = $47;
  HEAP32[$48>>2] = $44;
  HEAP32[$2>>2] = 0;
  $58 = HEAP32[$2>>2]|0;
  STACKTOP = sp;return ($58|0);
 }
 return 0|0;
}
function _sqlite3MulInt64($pA,$0,$1) {
 $pA = $pA|0;
 $0 = $0|0;
 $1 = $1|0;
 var $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0, $116 = 0, $117 = 0;
 var $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0, $134 = 0, $135 = 0;
 var $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0, $152 = 0, $153 = 0;
 var $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0, $170 = 0, $171 = 0;
 var $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0, $188 = 0, $189 = 0, $19 = 0;
 var $190 = 0, $191 = 0, $192 = 0, $193 = 0, $194 = 0, $195 = 0, $196 = 0, $197 = 0, $198 = 0, $199 = 0, $2 = 0, $20 = 0, $200 = 0, $201 = 0, $202 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0;
 var $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0;
 var $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0;
 var $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0;
 var $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0;
 var $99 = 0, $iA = 0, $iA0 = 0, $iA1 = 0, $iB0 = 0, $iB1 = 0, $or$cond = 0, $r = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 64|0;
 $2 = sp + 60|0;
 $3 = sp + 56|0;
 $4 = sp + 48|0;
 $iA = sp + 40|0;
 $iA1 = sp + 32|0;
 $iA0 = sp + 24|0;
 $iB1 = sp + 16|0;
 $iB0 = sp + 8|0;
 $r = sp;
 HEAP32[$3>>2] = $pA;
 $5 = $4;
 $6 = $5;
 HEAP32[$6>>2] = $0;
 $7 = (($5) + 4)|0;
 $8 = $7;
 HEAP32[$8>>2] = $1;
 $9 = HEAP32[$3>>2]|0;
 $10 = $9;
 $11 = $10;
 $12 = HEAP32[$11>>2]|0;
 $13 = (($10) + 4)|0;
 $14 = $13;
 $15 = HEAP32[$14>>2]|0;
 $16 = $iA;
 $17 = $16;
 HEAP32[$17>>2] = $12;
 $18 = (($16) + 4)|0;
 $19 = $18;
 HEAP32[$19>>2] = $15;
 $20 = $iA;
 $21 = $20;
 $22 = HEAP32[$21>>2]|0;
 $23 = (($20) + 4)|0;
 $24 = $23;
 $25 = HEAP32[$24>>2]|0;
 $26 = (___divdi3(($22|0),($25|0),0,1)|0);
 $27 = tempRet0;
 $28 = $iA1;
 $29 = $28;
 HEAP32[$29>>2] = $26;
 $30 = (($28) + 4)|0;
 $31 = $30;
 HEAP32[$31>>2] = $27;
 $32 = $iA;
 $33 = $32;
 $34 = HEAP32[$33>>2]|0;
 $35 = (($32) + 4)|0;
 $36 = $35;
 $37 = HEAP32[$36>>2]|0;
 $38 = (___remdi3(($34|0),($37|0),0,1)|0);
 $39 = tempRet0;
 $40 = $iA0;
 $41 = $40;
 HEAP32[$41>>2] = $38;
 $42 = (($40) + 4)|0;
 $43 = $42;
 HEAP32[$43>>2] = $39;
 $44 = $4;
 $45 = $44;
 $46 = HEAP32[$45>>2]|0;
 $47 = (($44) + 4)|0;
 $48 = $47;
 $49 = HEAP32[$48>>2]|0;
 $50 = (___divdi3(($46|0),($49|0),0,1)|0);
 $51 = tempRet0;
 $52 = $iB1;
 $53 = $52;
 HEAP32[$53>>2] = $50;
 $54 = (($52) + 4)|0;
 $55 = $54;
 HEAP32[$55>>2] = $51;
 $56 = $4;
 $57 = $56;
 $58 = HEAP32[$57>>2]|0;
 $59 = (($56) + 4)|0;
 $60 = $59;
 $61 = HEAP32[$60>>2]|0;
 $62 = (___remdi3(($58|0),($61|0),0,1)|0);
 $63 = tempRet0;
 $64 = $iB0;
 $65 = $64;
 HEAP32[$65>>2] = $62;
 $66 = (($64) + 4)|0;
 $67 = $66;
 HEAP32[$67>>2] = $63;
 $68 = $iA1;
 $69 = $68;
 $70 = HEAP32[$69>>2]|0;
 $71 = (($68) + 4)|0;
 $72 = $71;
 $73 = HEAP32[$72>>2]|0;
 $74 = ($70|0)==(0);
 $75 = ($73|0)==(0);
 $76 = $74 & $75;
 $77 = $iB1;
 $78 = $77;
 $79 = HEAP32[$78>>2]|0;
 $80 = (($77) + 4)|0;
 $81 = $80;
 $82 = HEAP32[$81>>2]|0;
 $83 = ($79|0)==(0);
 $84 = ($82|0)==(0);
 $85 = $83 & $84;
 do {
  if ($76) {
   if (!($85)) {
    $105 = $iA0;
    $106 = $105;
    $107 = HEAP32[$106>>2]|0;
    $108 = (($105) + 4)|0;
    $109 = $108;
    $110 = HEAP32[$109>>2]|0;
    $111 = $iB1;
    $112 = $111;
    $113 = HEAP32[$112>>2]|0;
    $114 = (($111) + 4)|0;
    $115 = $114;
    $116 = HEAP32[$115>>2]|0;
    $117 = (___muldi3(($107|0),($110|0),($113|0),($116|0))|0);
    $118 = tempRet0;
    $119 = $r;
    $120 = $119;
    HEAP32[$120>>2] = $117;
    $121 = (($119) + 4)|0;
    $122 = $121;
    HEAP32[$122>>2] = $118;
    break;
   }
   $86 = $4;
   $87 = $86;
   $88 = HEAP32[$87>>2]|0;
   $89 = (($86) + 4)|0;
   $90 = $89;
   $91 = HEAP32[$90>>2]|0;
   $92 = HEAP32[$3>>2]|0;
   $93 = $92;
   $94 = $93;
   $95 = HEAP32[$94>>2]|0;
   $96 = (($93) + 4)|0;
   $97 = $96;
   $98 = HEAP32[$97>>2]|0;
   $99 = (___muldi3(($95|0),($98|0),($88|0),($91|0))|0);
   $100 = tempRet0;
   $101 = $92;
   $102 = $101;
   HEAP32[$102>>2] = $99;
   $103 = (($101) + 4)|0;
   $104 = $103;
   HEAP32[$104>>2] = $100;
   HEAP32[$2>>2] = 0;
   $202 = HEAP32[$2>>2]|0;
   STACKTOP = sp;return ($202|0);
  } else {
   if ($85) {
    $123 = $iA1;
    $124 = $123;
    $125 = HEAP32[$124>>2]|0;
    $126 = (($123) + 4)|0;
    $127 = $126;
    $128 = HEAP32[$127>>2]|0;
    $129 = $iB0;
    $130 = $129;
    $131 = HEAP32[$130>>2]|0;
    $132 = (($129) + 4)|0;
    $133 = $132;
    $134 = HEAP32[$133>>2]|0;
    $135 = (___muldi3(($125|0),($128|0),($131|0),($134|0))|0);
    $136 = tempRet0;
    $137 = $r;
    $138 = $137;
    HEAP32[$138>>2] = $135;
    $139 = (($137) + 4)|0;
    $140 = $139;
    HEAP32[$140>>2] = $136;
    break;
   }
   HEAP32[$2>>2] = 1;
   $202 = HEAP32[$2>>2]|0;
   STACKTOP = sp;return ($202|0);
  }
 } while(0);
 $141 = $r;
 $142 = $141;
 $143 = HEAP32[$142>>2]|0;
 $144 = (($141) + 4)|0;
 $145 = $144;
 $146 = HEAP32[$145>>2]|0;
 $147 = ($146|0)<(-1);
 $148 = ($143>>>0)<(2147483648);
 $149 = ($146|0)==(-1);
 $150 = $149 & $148;
 $151 = $147 | $150;
 $152 = $r;
 $153 = $152;
 $154 = HEAP32[$153>>2]|0;
 $155 = (($152) + 4)|0;
 $156 = $155;
 $157 = HEAP32[$156>>2]|0;
 $158 = ($157|0)>(0);
 $159 = ($154>>>0)>=(2147483648);
 $160 = ($157|0)==(0);
 $161 = $160 & $159;
 $162 = $158 | $161;
 $or$cond = $151 | $162;
 if ($or$cond) {
  HEAP32[$2>>2] = 1;
  $202 = HEAP32[$2>>2]|0;
  STACKTOP = sp;return ($202|0);
 }
 $163 = $r;
 $164 = $163;
 $165 = HEAP32[$164>>2]|0;
 $166 = (($163) + 4)|0;
 $167 = $166;
 $168 = HEAP32[$167>>2]|0;
 $169 = (___muldi3(($165|0),($168|0),0,1)|0);
 $170 = tempRet0;
 $171 = $r;
 $172 = $171;
 HEAP32[$172>>2] = $169;
 $173 = (($171) + 4)|0;
 $174 = $173;
 HEAP32[$174>>2] = $170;
 $175 = $iA0;
 $176 = $175;
 $177 = HEAP32[$176>>2]|0;
 $178 = (($175) + 4)|0;
 $179 = $178;
 $180 = HEAP32[$179>>2]|0;
 $181 = $iB0;
 $182 = $181;
 $183 = HEAP32[$182>>2]|0;
 $184 = (($181) + 4)|0;
 $185 = $184;
 $186 = HEAP32[$185>>2]|0;
 $187 = (___muldi3(($177|0),($180|0),($183|0),($186|0))|0);
 $188 = tempRet0;
 $189 = (_sqlite3AddInt64($r,$187,$188)|0);
 $190 = ($189|0)!=(0);
 if ($190) {
  HEAP32[$2>>2] = 1;
  $202 = HEAP32[$2>>2]|0;
  STACKTOP = sp;return ($202|0);
 } else {
  $191 = $r;
  $192 = $191;
  $193 = HEAP32[$192>>2]|0;
  $194 = (($191) + 4)|0;
  $195 = $194;
  $196 = HEAP32[$195>>2]|0;
  $197 = HEAP32[$3>>2]|0;
  $198 = $197;
  $199 = $198;
  HEAP32[$199>>2] = $193;
  $200 = (($198) + 4)|0;
  $201 = $200;
  HEAP32[$201>>2] = $196;
  HEAP32[$2>>2] = 0;
  $202 = HEAP32[$2>>2]|0;
  STACKTOP = sp;return ($202|0);
 }
 return 0|0;
}
function _sqlite3VdbeDeleteAuxData($pVdbe,$iOp,$mask) {
 $pVdbe = $pVdbe|0;
 $iOp = $iOp|0;
 $mask = $mask|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $pAux = 0, $pp = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 16|0;
 $1 = sp + 12|0;
 $2 = sp + 8|0;
 $pp = sp + 4|0;
 $pAux = sp;
 HEAP32[$0>>2] = $pVdbe;
 HEAP32[$1>>2] = $iOp;
 HEAP32[$2>>2] = $mask;
 $3 = HEAP32[$0>>2]|0;
 $4 = (($3) + 204|0);
 HEAP32[$pp>>2] = $4;
 L1: while(1) {
  $5 = HEAP32[$pp>>2]|0;
  $6 = HEAP32[$5>>2]|0;
  $7 = ($6|0)!=(0|0);
  if (!($7)) {
   break;
  }
  $8 = HEAP32[$pp>>2]|0;
  $9 = HEAP32[$8>>2]|0;
  HEAP32[$pAux>>2] = $9;
  $10 = HEAP32[$1>>2]|0;
  $11 = ($10|0)<(0);
  do {
   if (!($11)) {
    $12 = HEAP32[$pAux>>2]|0;
    $13 = HEAP32[$12>>2]|0;
    $14 = HEAP32[$1>>2]|0;
    $15 = ($13|0)==($14|0);
    if ($15) {
     $16 = HEAP32[$pAux>>2]|0;
     $17 = (($16) + 4|0);
     $18 = HEAP32[$17>>2]|0;
     $19 = ($18|0)>(31);
     if ($19) {
      break;
     }
     $20 = HEAP32[$2>>2]|0;
     $21 = HEAP32[$pAux>>2]|0;
     $22 = (($21) + 4|0);
     $23 = HEAP32[$22>>2]|0;
     $24 = 1 << $23;
     $25 = $20 & $24;
     $26 = ($25|0)!=(0);
     if (!($26)) {
      break;
     }
    }
    $44 = HEAP32[$pAux>>2]|0;
    $45 = (($44) + 16|0);
    HEAP32[$pp>>2] = $45;
    continue L1;
   }
  } while(0);
  $27 = HEAP32[$pAux>>2]|0;
  $28 = (($27) + 12|0);
  $29 = HEAP32[$28>>2]|0;
  $30 = ($29|0)!=(0|0);
  if ($30) {
   $31 = HEAP32[$pAux>>2]|0;
   $32 = (($31) + 12|0);
   $33 = HEAP32[$32>>2]|0;
   $34 = HEAP32[$pAux>>2]|0;
   $35 = (($34) + 8|0);
   $36 = HEAP32[$35>>2]|0;
   FUNCTION_TABLE_vi[$33 & 31]($36);
  }
  $37 = HEAP32[$pAux>>2]|0;
  $38 = (($37) + 16|0);
  $39 = HEAP32[$38>>2]|0;
  $40 = HEAP32[$pp>>2]|0;
  HEAP32[$40>>2] = $39;
  $41 = HEAP32[$0>>2]|0;
  $42 = HEAP32[$41>>2]|0;
  $43 = HEAP32[$pAux>>2]|0;
  _sqlite3DbFree($42,$43);
 }
 STACKTOP = sp;return;
}
function _sqlite3VdbeCursorMoveto($p) {
 $p = $p|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $1 = sp;
 HEAP32[$1>>2] = $p;
 $2 = HEAP32[$1>>2]|0;
 $3 = (($2) + 26|0);
 $4 = HEAP8[$3>>0]|0;
 $5 = ($4<<24>>24)!=(0);
 $6 = HEAP32[$1>>2]|0;
 do {
  if ($5) {
   $7 = (_handleDeferredMoveto($6)|0);
   HEAP32[$0>>2] = $7;
  } else {
   $8 = HEAP32[$6>>2]|0;
   $9 = ($8|0)!=(0|0);
   if ($9) {
    $10 = HEAP32[$1>>2]|0;
    $11 = HEAP32[$10>>2]|0;
    $12 = (_sqlite3BtreeCursorHasMoved($11)|0);
    $13 = ($12|0)!=(0);
    if ($13) {
     $14 = HEAP32[$1>>2]|0;
     $15 = (_handleMovedCursor($14)|0);
     HEAP32[$0>>2] = $15;
     break;
    }
   }
   HEAP32[$0>>2] = 0;
  }
 } while(0);
 $16 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($16|0);
}
function _sqlite3BtreeKeyFetch($pCur,$pAmt) {
 $pCur = $pCur|0;
 $pAmt = $pAmt|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $1 = sp;
 HEAP32[$0>>2] = $pCur;
 HEAP32[$1>>2] = $pAmt;
 $2 = HEAP32[$0>>2]|0;
 $3 = HEAP32[$1>>2]|0;
 $4 = (_fetchPayload($2,$3)|0);
 STACKTOP = sp;return ($4|0);
}
function _sqlite3BtreeDataSize($pCur,$pSize) {
 $pCur = $pCur|0;
 $pSize = $pSize|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $iPage = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $1 = sp + 4|0;
 $iPage = sp;
 HEAP32[$0>>2] = $pCur;
 HEAP32[$1>>2] = $pSize;
 $2 = HEAP32[$0>>2]|0;
 $3 = (($2) + 24|0);
 $4 = (($3) + 20|0);
 $5 = HEAP16[$4>>1]|0;
 $6 = $5&65535;
 $7 = ($6|0)==(0);
 if (!($7)) {
  $31 = HEAP32[$0>>2]|0;
  $32 = (($31) + 24|0);
  $33 = (($32) + 12|0);
  $34 = HEAP32[$33>>2]|0;
  $35 = HEAP32[$1>>2]|0;
  HEAP32[$35>>2] = $34;
  STACKTOP = sp;return 0;
 }
 $8 = HEAP32[$0>>2]|0;
 $9 = (($8) + 76|0);
 $10 = HEAP16[$9>>1]|0;
 $11 = $10 << 16 >> 16;
 HEAP32[$iPage>>2] = $11;
 $12 = HEAP32[$iPage>>2]|0;
 $13 = HEAP32[$0>>2]|0;
 $14 = (($13) + 120|0);
 $15 = (($14) + ($12<<2)|0);
 $16 = HEAP32[$15>>2]|0;
 $17 = HEAP32[$iPage>>2]|0;
 $18 = HEAP32[$0>>2]|0;
 $19 = (($18) + 78|0);
 $20 = (($19) + ($17<<1)|0);
 $21 = HEAP16[$20>>1]|0;
 $22 = $21&65535;
 $23 = HEAP32[$0>>2]|0;
 $24 = (($23) + 24|0);
 _btreeParseCell($16,$22,$24);
 $25 = HEAP32[$0>>2]|0;
 $26 = (($25) + 72|0);
 $27 = HEAP8[$26>>0]|0;
 $28 = $27&255;
 $29 = $28 | 2;
 $30 = $29&255;
 HEAP8[$26>>0] = $30;
 $31 = HEAP32[$0>>2]|0;
 $32 = (($31) + 24|0);
 $33 = (($32) + 12|0);
 $34 = HEAP32[$33>>2]|0;
 $35 = HEAP32[$1>>2]|0;
 HEAP32[$35>>2] = $34;
 STACKTOP = sp;return 0;
}
function _sqlite3BtreeDataFetch($pCur,$pAmt) {
 $pCur = $pCur|0;
 $pAmt = $pAmt|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $1 = sp;
 HEAP32[$0>>2] = $pCur;
 HEAP32[$1>>2] = $pAmt;
 $2 = HEAP32[$0>>2]|0;
 $3 = HEAP32[$1>>2]|0;
 $4 = (_fetchPayload($2,$3)|0);
 STACKTOP = sp;return ($4|0);
}
function _sqlite3VdbeMemFromBtree($pCur,$offset,$amt,$key,$pMem) {
 $pCur = $pCur|0;
 $offset = $offset|0;
 $amt = $amt|0;
 $key = $key|0;
 $pMem = $pMem|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $7 = 0;
 var $8 = 0, $9 = 0, $available = 0, $rc = 0, $zData = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 28|0;
 $1 = sp + 24|0;
 $2 = sp + 20|0;
 $3 = sp + 16|0;
 $4 = sp + 12|0;
 $zData = sp + 8|0;
 $available = sp + 4|0;
 $rc = sp;
 HEAP32[$0>>2] = $pCur;
 HEAP32[$1>>2] = $offset;
 HEAP32[$2>>2] = $amt;
 HEAP32[$3>>2] = $key;
 HEAP32[$4>>2] = $pMem;
 HEAP32[$available>>2] = 0;
 HEAP32[$rc>>2] = 0;
 $5 = HEAP32[$3>>2]|0;
 $6 = ($5|0)!=(0);
 $7 = HEAP32[$0>>2]|0;
 if ($6) {
  $8 = (_sqlite3BtreeKeyFetch($7,$available)|0);
  HEAP32[$zData>>2] = $8;
 } else {
  $9 = (_sqlite3BtreeDataFetch($7,$available)|0);
  HEAP32[$zData>>2] = $9;
 }
 $10 = HEAP32[$1>>2]|0;
 $11 = HEAP32[$2>>2]|0;
 $12 = (($10) + ($11))|0;
 $13 = HEAP32[$available>>2]|0;
 $14 = ($12>>>0)<=($13>>>0);
 if ($14) {
  $15 = HEAP32[$1>>2]|0;
  $16 = HEAP32[$zData>>2]|0;
  $17 = (($16) + ($15)|0);
  $18 = HEAP32[$4>>2]|0;
  $19 = (($18) + 16|0);
  HEAP32[$19>>2] = $17;
  $20 = HEAP32[$4>>2]|0;
  $21 = (($20) + 8|0);
  HEAP16[$21>>1] = 4112;
  $22 = HEAP32[$2>>2]|0;
  $23 = HEAP32[$4>>2]|0;
  $24 = (($23) + 12|0);
  HEAP32[$24>>2] = $22;
  $61 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($61|0);
 }
 $25 = HEAP32[$4>>2]|0;
 $26 = (($25) + 8|0);
 HEAP16[$26>>1] = 1;
 $27 = HEAP32[$4>>2]|0;
 $28 = HEAP32[$2>>2]|0;
 $29 = (($28) + 2)|0;
 $30 = (_sqlite3VdbeMemClearAndResize($27,$29)|0);
 HEAP32[$rc>>2] = $30;
 $31 = (0)==($30|0);
 if (!($31)) {
  $61 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($61|0);
 }
 $32 = HEAP32[$3>>2]|0;
 $33 = ($32|0)!=(0);
 $34 = HEAP32[$0>>2]|0;
 $35 = HEAP32[$1>>2]|0;
 $36 = HEAP32[$2>>2]|0;
 $37 = HEAP32[$4>>2]|0;
 $38 = (($37) + 16|0);
 $39 = HEAP32[$38>>2]|0;
 if ($33) {
  $40 = (_sqlite3BtreeKey($34,$35,$36,$39)|0);
  HEAP32[$rc>>2] = $40;
 } else {
  $41 = (_sqlite3BtreeData($34,$35,$36,$39)|0);
  HEAP32[$rc>>2] = $41;
 }
 $42 = HEAP32[$rc>>2]|0;
 $43 = ($42|0)==(0);
 if ($43) {
  $44 = HEAP32[$2>>2]|0;
  $45 = HEAP32[$4>>2]|0;
  $46 = (($45) + 16|0);
  $47 = HEAP32[$46>>2]|0;
  $48 = (($47) + ($44)|0);
  HEAP8[$48>>0] = 0;
  $49 = HEAP32[$2>>2]|0;
  $50 = (($49) + 1)|0;
  $51 = HEAP32[$4>>2]|0;
  $52 = (($51) + 16|0);
  $53 = HEAP32[$52>>2]|0;
  $54 = (($53) + ($50)|0);
  HEAP8[$54>>0] = 0;
  $55 = HEAP32[$4>>2]|0;
  $56 = (($55) + 8|0);
  HEAP16[$56>>1] = 528;
  $57 = HEAP32[$2>>2]|0;
  $58 = HEAP32[$4>>2]|0;
  $59 = (($58) + 12|0);
  HEAP32[$59>>2] = $57;
  $61 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($61|0);
 } else {
  $60 = HEAP32[$4>>2]|0;
  _sqlite3VdbeMemRelease($60);
  $61 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($61|0);
 }
 return 0|0;
}
function _sqlite3VdbeSerialType($pMem,$file_format) {
 $pMem = $pMem|0;
 $file_format = $file_format|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0;
 var $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0;
 var $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0;
 var $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0;
 var $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0;
 var $99 = 0, $flags = 0, $i = 0, $n = 0, $or$cond = 0, $u = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $0 = sp + 32|0;
 $1 = sp + 28|0;
 $2 = sp + 24|0;
 $flags = sp + 20|0;
 $n = sp + 16|0;
 $i = sp + 8|0;
 $u = sp;
 HEAP32[$1>>2] = $pMem;
 HEAP32[$2>>2] = $file_format;
 $3 = HEAP32[$1>>2]|0;
 $4 = (($3) + 8|0);
 $5 = HEAP16[$4>>1]|0;
 $6 = $5&65535;
 HEAP32[$flags>>2] = $6;
 $7 = HEAP32[$flags>>2]|0;
 $8 = $7 & 1;
 $9 = ($8|0)!=(0);
 if ($9) {
  HEAP32[$0>>2] = 0;
  $160 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($160|0);
 }
 $10 = HEAP32[$flags>>2]|0;
 $11 = $10 & 4;
 $12 = ($11|0)!=(0);
 if (!($12)) {
  $139 = HEAP32[$flags>>2]|0;
  $140 = $139 & 8;
  $141 = ($140|0)!=(0);
  if ($141) {
   HEAP32[$0>>2] = 7;
   $160 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($160|0);
  }
  $142 = HEAP32[$1>>2]|0;
  $143 = (($142) + 12|0);
  $144 = HEAP32[$143>>2]|0;
  HEAP32[$n>>2] = $144;
  $145 = HEAP32[$flags>>2]|0;
  $146 = $145 & 16384;
  $147 = ($146|0)!=(0);
  if ($147) {
   $148 = HEAP32[$1>>2]|0;
   $149 = HEAP32[$148>>2]|0;
   $150 = HEAP32[$n>>2]|0;
   $151 = (($150) + ($149))|0;
   HEAP32[$n>>2] = $151;
  }
  $152 = HEAP32[$n>>2]|0;
  $153 = $152<<1;
  $154 = (($153) + 12)|0;
  $155 = HEAP32[$flags>>2]|0;
  $156 = $155 & 2;
  $157 = ($156|0)!=(0);
  $158 = $157&1;
  $159 = (($154) + ($158))|0;
  HEAP32[$0>>2] = $159;
  $160 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($160|0);
 }
 $13 = HEAP32[$1>>2]|0;
 $14 = $13;
 $15 = $14;
 $16 = HEAP32[$15>>2]|0;
 $17 = (($14) + 4)|0;
 $18 = $17;
 $19 = HEAP32[$18>>2]|0;
 $20 = $i;
 $21 = $20;
 HEAP32[$21>>2] = $16;
 $22 = (($20) + 4)|0;
 $23 = $22;
 HEAP32[$23>>2] = $19;
 $24 = $i;
 $25 = $24;
 $26 = HEAP32[$25>>2]|0;
 $27 = (($24) + 4)|0;
 $28 = $27;
 $29 = HEAP32[$28>>2]|0;
 $30 = ($29|0)<(0);
 $31 = $i;
 $32 = $31;
 $33 = HEAP32[$32>>2]|0;
 $34 = (($31) + 4)|0;
 $35 = $34;
 $36 = HEAP32[$35>>2]|0;
 do {
  if ($30) {
   $37 = ($36|0)<(-32768);
   $38 = ($33>>>0)<(1);
   $39 = ($36|0)==(-32768);
   $40 = $39 & $38;
   $41 = $37 | $40;
   if (!($41)) {
    $42 = $i;
    $43 = $42;
    $44 = HEAP32[$43>>2]|0;
    $45 = (($42) + 4)|0;
    $46 = $45;
    $47 = HEAP32[$46>>2]|0;
    $48 = (_i64Subtract(0,0,($44|0),($47|0))|0);
    $49 = tempRet0;
    $50 = $u;
    $51 = $50;
    HEAP32[$51>>2] = $48;
    $52 = (($50) + 4)|0;
    $53 = $52;
    HEAP32[$53>>2] = $49;
    break;
   }
   HEAP32[$0>>2] = 6;
   $160 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($160|0);
  } else {
   $54 = $u;
   $55 = $54;
   HEAP32[$55>>2] = $33;
   $56 = (($54) + 4)|0;
   $57 = $56;
   HEAP32[$57>>2] = $36;
  }
 } while(0);
 $58 = $u;
 $59 = $58;
 $60 = HEAP32[$59>>2]|0;
 $61 = (($58) + 4)|0;
 $62 = $61;
 $63 = HEAP32[$62>>2]|0;
 $64 = ($63>>>0)<(0);
 $65 = ($60>>>0)<=(127);
 $66 = ($63|0)==(0);
 $67 = $66 & $65;
 $68 = $64 | $67;
 if ($68) {
  $69 = $i;
  $70 = $69;
  $71 = HEAP32[$70>>2]|0;
  $72 = (($69) + 4)|0;
  $73 = $72;
  $74 = HEAP32[$73>>2]|0;
  $75 = $71 & 1;
  $76 = $i;
  $77 = $76;
  $78 = HEAP32[$77>>2]|0;
  $79 = (($76) + 4)|0;
  $80 = $79;
  $81 = HEAP32[$80>>2]|0;
  $82 = ($75|0)==($78|0);
  $83 = (0)==($81|0);
  $84 = $82 & $83;
  $85 = HEAP32[$2>>2]|0;
  $86 = ($85|0)>=(4);
  $or$cond = $84 & $86;
  if ($or$cond) {
   $87 = $u;
   $88 = $87;
   $89 = HEAP32[$88>>2]|0;
   $90 = (($87) + 4)|0;
   $91 = $90;
   $92 = HEAP32[$91>>2]|0;
   $93 = (8 + ($89))|0;
   $94 = $93;
  } else {
   $94 = 1;
  }
  HEAP32[$0>>2] = $94;
  $160 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($160|0);
 }
 $95 = $u;
 $96 = $95;
 $97 = HEAP32[$96>>2]|0;
 $98 = (($95) + 4)|0;
 $99 = $98;
 $100 = HEAP32[$99>>2]|0;
 $101 = ($100>>>0)<(0);
 $102 = ($97>>>0)<=(32767);
 $103 = ($100|0)==(0);
 $104 = $103 & $102;
 $105 = $101 | $104;
 if ($105) {
  HEAP32[$0>>2] = 2;
  $160 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($160|0);
 }
 $106 = $u;
 $107 = $106;
 $108 = HEAP32[$107>>2]|0;
 $109 = (($106) + 4)|0;
 $110 = $109;
 $111 = HEAP32[$110>>2]|0;
 $112 = ($111>>>0)<(0);
 $113 = ($108>>>0)<=(8388607);
 $114 = ($111|0)==(0);
 $115 = $114 & $113;
 $116 = $112 | $115;
 if ($116) {
  HEAP32[$0>>2] = 3;
  $160 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($160|0);
 }
 $117 = $u;
 $118 = $117;
 $119 = HEAP32[$118>>2]|0;
 $120 = (($117) + 4)|0;
 $121 = $120;
 $122 = HEAP32[$121>>2]|0;
 $123 = ($122>>>0)<(0);
 $124 = ($119>>>0)<=(2147483647);
 $125 = ($122|0)==(0);
 $126 = $125 & $124;
 $127 = $123 | $126;
 if ($127) {
  HEAP32[$0>>2] = 4;
  $160 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($160|0);
 }
 $128 = $u;
 $129 = $128;
 $130 = HEAP32[$129>>2]|0;
 $131 = (($128) + 4)|0;
 $132 = $131;
 $133 = HEAP32[$132>>2]|0;
 $134 = ($133>>>0)<(32767);
 $135 = ($130>>>0)<=(4294967295);
 $136 = ($133|0)==(32767);
 $137 = $136 & $135;
 $138 = $134 | $137;
 if ($138) {
  HEAP32[$0>>2] = 5;
  $160 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($160|0);
 } else {
  HEAP32[$0>>2] = 6;
  $160 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($160|0);
 }
 return 0|0;
}
function _sqlite3PutVarint($p,$0,$1) {
 $p = $p|0;
 $0 = $0|0;
 $1 = $1|0;
 var $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0;
 var $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0;
 var $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0;
 var $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $2 = sp + 12|0;
 $3 = sp + 8|0;
 $4 = sp;
 HEAP32[$3>>2] = $p;
 $5 = $4;
 $6 = $5;
 HEAP32[$6>>2] = $0;
 $7 = (($5) + 4)|0;
 $8 = $7;
 HEAP32[$8>>2] = $1;
 $9 = $4;
 $10 = $9;
 $11 = HEAP32[$10>>2]|0;
 $12 = (($9) + 4)|0;
 $13 = $12;
 $14 = HEAP32[$13>>2]|0;
 $15 = ($14>>>0)<(0);
 $16 = ($11>>>0)<=(127);
 $17 = ($14|0)==(0);
 $18 = $17 & $16;
 $19 = $15 | $18;
 $20 = $4;
 $21 = $20;
 $22 = HEAP32[$21>>2]|0;
 $23 = (($20) + 4)|0;
 $24 = $23;
 $25 = HEAP32[$24>>2]|0;
 if ($19) {
  $26 = $22 & 127;
  $27 = $26&255;
  $28 = HEAP32[$3>>2]|0;
  HEAP8[$28>>0] = $27;
  HEAP32[$2>>2] = 1;
  $64 = HEAP32[$2>>2]|0;
  STACKTOP = sp;return ($64|0);
 }
 $29 = ($25>>>0)<(0);
 $30 = ($22>>>0)<=(16383);
 $31 = ($25|0)==(0);
 $32 = $31 & $30;
 $33 = $29 | $32;
 if ($33) {
  $34 = $4;
  $35 = $34;
  $36 = HEAP32[$35>>2]|0;
  $37 = (($34) + 4)|0;
  $38 = $37;
  $39 = HEAP32[$38>>2]|0;
  $40 = (_bitshift64Lshr(($36|0),($39|0),7)|0);
  $41 = tempRet0;
  $42 = $40 & 127;
  $43 = $42 | 128;
  $44 = $43&255;
  $45 = HEAP32[$3>>2]|0;
  HEAP8[$45>>0] = $44;
  $46 = $4;
  $47 = $46;
  $48 = HEAP32[$47>>2]|0;
  $49 = (($46) + 4)|0;
  $50 = $49;
  $51 = HEAP32[$50>>2]|0;
  $52 = $48 & 127;
  $53 = $52&255;
  $54 = HEAP32[$3>>2]|0;
  $55 = (($54) + 1|0);
  HEAP8[$55>>0] = $53;
  HEAP32[$2>>2] = 2;
  $64 = HEAP32[$2>>2]|0;
  STACKTOP = sp;return ($64|0);
 } else {
  $56 = HEAP32[$3>>2]|0;
  $57 = $4;
  $58 = $57;
  $59 = HEAP32[$58>>2]|0;
  $60 = (($57) + 4)|0;
  $61 = $60;
  $62 = HEAP32[$61>>2]|0;
  $63 = (_putVarint64($56,$59,$62)|0);
  HEAP32[$2>>2] = $63;
  $64 = HEAP32[$2>>2]|0;
  STACKTOP = sp;return ($64|0);
 }
 return 0|0;
}
function _sqlite3VdbeSerialPut($buf,$pMem,$serial_type) {
 $buf = $buf|0;
 $pMem = $pMem|0;
 $serial_type = $serial_type|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $7 = 0;
 var $8 = 0, $9 = 0, $i = 0, $len = 0, $or$cond = 0, $v = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 28|0;
 $1 = sp + 24|0;
 $2 = sp + 20|0;
 $3 = sp + 16|0;
 $len = sp + 12|0;
 $v = sp;
 $i = sp + 8|0;
 HEAP32[$1>>2] = $buf;
 HEAP32[$2>>2] = $pMem;
 HEAP32[$3>>2] = $serial_type;
 $4 = HEAP32[$3>>2]|0;
 $5 = ($4>>>0)<=(7);
 $6 = HEAP32[$3>>2]|0;
 $7 = ($6>>>0)>(0);
 $or$cond = $5 & $7;
 $8 = HEAP32[$3>>2]|0;
 if (!($or$cond)) {
  $51 = ($8>>>0)>=(12);
  if ($51) {
   $52 = HEAP32[$2>>2]|0;
   $53 = (($52) + 12|0);
   $54 = HEAP32[$53>>2]|0;
   HEAP32[$len>>2] = $54;
   $55 = HEAP32[$1>>2]|0;
   $56 = HEAP32[$2>>2]|0;
   $57 = (($56) + 16|0);
   $58 = HEAP32[$57>>2]|0;
   $59 = HEAP32[$len>>2]|0;
   _memcpy(($55|0),($58|0),($59|0))|0;
   $60 = HEAP32[$len>>2]|0;
   HEAP32[$0>>2] = $60;
   $61 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($61|0);
  } else {
   HEAP32[$0>>2] = 0;
   $61 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($61|0);
  }
 }
 $9 = ($8|0)==(7);
 if ($9) {
  $10 = HEAP32[$2>>2]|0;
  ;HEAP32[$v+0>>2]=HEAP32[$10+0>>2]|0;HEAP32[$v+4>>2]=HEAP32[$10+4>>2]|0;
 } else {
  $11 = HEAP32[$2>>2]|0;
  $12 = $11;
  $13 = $12;
  $14 = HEAP32[$13>>2]|0;
  $15 = (($12) + 4)|0;
  $16 = $15;
  $17 = HEAP32[$16>>2]|0;
  $18 = $v;
  $19 = $18;
  HEAP32[$19>>2] = $14;
  $20 = (($18) + 4)|0;
  $21 = $20;
  HEAP32[$21>>2] = $17;
 }
 $22 = HEAP32[$3>>2]|0;
 $23 = (_sqlite3VdbeSerialTypeLen($22)|0);
 HEAP32[$i>>2] = $23;
 HEAP32[$len>>2] = $23;
 while(1) {
  $24 = $v;
  $25 = $24;
  $26 = HEAP32[$25>>2]|0;
  $27 = (($24) + 4)|0;
  $28 = $27;
  $29 = HEAP32[$28>>2]|0;
  $30 = $26 & 255;
  $31 = $30&255;
  $32 = HEAP32[$i>>2]|0;
  $33 = (($32) + -1)|0;
  HEAP32[$i>>2] = $33;
  $34 = HEAP32[$1>>2]|0;
  $35 = (($34) + ($33)|0);
  HEAP8[$35>>0] = $31;
  $36 = $v;
  $37 = $36;
  $38 = HEAP32[$37>>2]|0;
  $39 = (($36) + 4)|0;
  $40 = $39;
  $41 = HEAP32[$40>>2]|0;
  $42 = (_bitshift64Lshr(($38|0),($41|0),8)|0);
  $43 = tempRet0;
  $44 = $v;
  $45 = $44;
  HEAP32[$45>>2] = $42;
  $46 = (($44) + 4)|0;
  $47 = $46;
  HEAP32[$47>>2] = $43;
  $48 = HEAP32[$i>>2]|0;
  $49 = ($48|0)!=(0);
  if (!($49)) {
   break;
  }
 }
 $50 = HEAP32[$len>>2]|0;
 HEAP32[$0>>2] = $50;
 $61 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($61|0);
}
function _sqlite3BtreeCount($pCur,$pnEntry) {
 $pCur = $pCur|0;
 $pnEntry = $pnEntry|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0;
 var $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0;
 var $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0;
 var $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0;
 var $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0;
 var $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $iIdx = 0, $nEntry = 0, $pPage = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 28|0;
 $1 = sp + 24|0;
 $2 = sp + 20|0;
 $nEntry = sp;
 $rc = sp + 16|0;
 $iIdx = sp + 12|0;
 $pPage = sp + 8|0;
 HEAP32[$1>>2] = $pCur;
 HEAP32[$2>>2] = $pnEntry;
 $3 = $nEntry;
 $4 = $3;
 HEAP32[$4>>2] = 0;
 $5 = (($3) + 4)|0;
 $6 = $5;
 HEAP32[$6>>2] = 0;
 $7 = HEAP32[$1>>2]|0;
 $8 = (($7) + 60|0);
 $9 = HEAP32[$8>>2]|0;
 $10 = ($9|0)==(0);
 if ($10) {
  $11 = HEAP32[$2>>2]|0;
  $12 = $11;
  $13 = $12;
  HEAP32[$13>>2] = 0;
  $14 = (($12) + 4)|0;
  $15 = $14;
  HEAP32[$15>>2] = 0;
  HEAP32[$0>>2] = 0;
  $169 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($169|0);
 }
 $16 = HEAP32[$1>>2]|0;
 $17 = (_moveToRoot($16)|0);
 HEAP32[$rc>>2] = $17;
 L5: while(1) {
  $18 = HEAP32[$rc>>2]|0;
  $19 = ($18|0)==(0);
  if (!($19)) {
   label = 16;
   break;
  }
  $20 = HEAP32[$1>>2]|0;
  $21 = (($20) + 76|0);
  $22 = HEAP16[$21>>1]|0;
  $23 = $22 << 16 >> 16;
  $24 = HEAP32[$1>>2]|0;
  $25 = (($24) + 120|0);
  $26 = (($25) + ($23<<2)|0);
  $27 = HEAP32[$26>>2]|0;
  HEAP32[$pPage>>2] = $27;
  $28 = HEAP32[$pPage>>2]|0;
  $29 = (($28) + 5|0);
  $30 = HEAP8[$29>>0]|0;
  $31 = $30&255;
  $32 = ($31|0)!=(0);
  if ($32) {
   label = 7;
  } else {
   $33 = HEAP32[$pPage>>2]|0;
   $34 = (($33) + 2|0);
   $35 = HEAP8[$34>>0]|0;
   $36 = ($35<<24>>24)!=(0);
   if (!($36)) {
    label = 7;
   }
  }
  if ((label|0) == 7) {
   label = 0;
   $37 = HEAP32[$pPage>>2]|0;
   $38 = (($37) + 18|0);
   $39 = HEAP16[$38>>1]|0;
   $40 = $39&65535;
   $41 = $nEntry;
   $42 = $41;
   $43 = HEAP32[$42>>2]|0;
   $44 = (($41) + 4)|0;
   $45 = $44;
   $46 = HEAP32[$45>>2]|0;
   $47 = (_i64Add(($43|0),($46|0),($40|0),0)|0);
   $48 = tempRet0;
   $49 = $nEntry;
   $50 = $49;
   HEAP32[$50>>2] = $47;
   $51 = (($49) + 4)|0;
   $52 = $51;
   HEAP32[$52>>2] = $48;
  }
  $53 = HEAP32[$pPage>>2]|0;
  $54 = (($53) + 5|0);
  $55 = HEAP8[$54>>0]|0;
  $56 = ($55<<24>>24)!=(0);
  if ($56) {
   while(1) {
    $57 = HEAP32[$1>>2]|0;
    $58 = (($57) + 76|0);
    $59 = HEAP16[$58>>1]|0;
    $60 = $59 << 16 >> 16;
    $61 = ($60|0)==(0);
    if ($61) {
     label = 10;
     break L5;
    }
    $73 = HEAP32[$1>>2]|0;
    _moveToParent($73);
    $74 = HEAP32[$1>>2]|0;
    $75 = (($74) + 76|0);
    $76 = HEAP16[$75>>1]|0;
    $77 = $76 << 16 >> 16;
    $78 = HEAP32[$1>>2]|0;
    $79 = (($78) + 78|0);
    $80 = (($79) + ($77<<1)|0);
    $81 = HEAP16[$80>>1]|0;
    $82 = $81&65535;
    $83 = HEAP32[$1>>2]|0;
    $84 = (($83) + 76|0);
    $85 = HEAP16[$84>>1]|0;
    $86 = $85 << 16 >> 16;
    $87 = HEAP32[$1>>2]|0;
    $88 = (($87) + 120|0);
    $89 = (($88) + ($86<<2)|0);
    $90 = HEAP32[$89>>2]|0;
    $91 = (($90) + 18|0);
    $92 = HEAP16[$91>>1]|0;
    $93 = $92&65535;
    $94 = ($82|0)>=($93|0);
    if (!($94)) {
     break;
    }
   }
   $95 = HEAP32[$1>>2]|0;
   $96 = (($95) + 76|0);
   $97 = HEAP16[$96>>1]|0;
   $98 = $97 << 16 >> 16;
   $99 = HEAP32[$1>>2]|0;
   $100 = (($99) + 78|0);
   $101 = (($100) + ($98<<1)|0);
   $102 = HEAP16[$101>>1]|0;
   $103 = (($102) + 1)<<16>>16;
   HEAP16[$101>>1] = $103;
   $104 = HEAP32[$1>>2]|0;
   $105 = (($104) + 76|0);
   $106 = HEAP16[$105>>1]|0;
   $107 = $106 << 16 >> 16;
   $108 = HEAP32[$1>>2]|0;
   $109 = (($108) + 120|0);
   $110 = (($109) + ($107<<2)|0);
   $111 = HEAP32[$110>>2]|0;
   HEAP32[$pPage>>2] = $111;
  }
  $112 = HEAP32[$1>>2]|0;
  $113 = (($112) + 76|0);
  $114 = HEAP16[$113>>1]|0;
  $115 = $114 << 16 >> 16;
  $116 = HEAP32[$1>>2]|0;
  $117 = (($116) + 78|0);
  $118 = (($117) + ($115<<1)|0);
  $119 = HEAP16[$118>>1]|0;
  $120 = $119&65535;
  HEAP32[$iIdx>>2] = $120;
  $121 = HEAP32[$iIdx>>2]|0;
  $122 = HEAP32[$pPage>>2]|0;
  $123 = (($122) + 18|0);
  $124 = HEAP16[$123>>1]|0;
  $125 = $124&65535;
  $126 = ($121|0)==($125|0);
  $127 = HEAP32[$1>>2]|0;
  $128 = HEAP32[$pPage>>2]|0;
  if ($126) {
   $129 = (($128) + 6|0);
   $130 = HEAP8[$129>>0]|0;
   $131 = $130&255;
   $132 = (($131) + 8)|0;
   $133 = HEAP32[$pPage>>2]|0;
   $134 = (($133) + 56|0);
   $135 = HEAP32[$134>>2]|0;
   $136 = (($135) + ($132)|0);
   $137 = (_sqlite3Get4byte($136)|0);
   $138 = (_moveToChild($127,$137)|0);
   HEAP32[$rc>>2] = $138;
   continue;
  } else {
   $139 = (($128) + 56|0);
   $140 = HEAP32[$139>>2]|0;
   $141 = HEAP32[$pPage>>2]|0;
   $142 = (($141) + 20|0);
   $143 = HEAP16[$142>>1]|0;
   $144 = $143&65535;
   $145 = HEAP32[$iIdx>>2]|0;
   $146 = $145<<1;
   $147 = HEAP32[$pPage>>2]|0;
   $148 = (($147) + 64|0);
   $149 = HEAP32[$148>>2]|0;
   $150 = (($149) + ($146)|0);
   $151 = HEAP8[$150>>0]|0;
   $152 = $151&255;
   $153 = $152 << 8;
   $154 = HEAP32[$iIdx>>2]|0;
   $155 = $154<<1;
   $156 = HEAP32[$pPage>>2]|0;
   $157 = (($156) + 64|0);
   $158 = HEAP32[$157>>2]|0;
   $159 = (($158) + ($155)|0);
   $160 = (($159) + 1|0);
   $161 = HEAP8[$160>>0]|0;
   $162 = $161&255;
   $163 = $153 | $162;
   $164 = $144 & $163;
   $165 = (($140) + ($164)|0);
   $166 = (_sqlite3Get4byte($165)|0);
   $167 = (_moveToChild($127,$166)|0);
   HEAP32[$rc>>2] = $167;
   continue;
  }
 }
 if ((label|0) == 10) {
  $62 = $nEntry;
  $63 = $62;
  $64 = HEAP32[$63>>2]|0;
  $65 = (($62) + 4)|0;
  $66 = $65;
  $67 = HEAP32[$66>>2]|0;
  $68 = HEAP32[$2>>2]|0;
  $69 = $68;
  $70 = $69;
  HEAP32[$70>>2] = $64;
  $71 = (($69) + 4)|0;
  $72 = $71;
  HEAP32[$72>>2] = $67;
  HEAP32[$0>>2] = 0;
  $169 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($169|0);
 }
 else if ((label|0) == 16) {
  $168 = HEAP32[$rc>>2]|0;
  HEAP32[$0>>2] = $168;
  $169 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($169|0);
 }
 return 0|0;
}
function _sqlite3VtabSavepoint($db,$op,$iSavepoint) {
 $db = $db|0;
 $op = $op|0;
 $iSavepoint = $iSavepoint|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $7 = 0, $8 = 0;
 var $9 = 0, $i = 0, $pMod = 0, $pVTab = 0, $rc = 0, $xMethod = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 28|0;
 $1 = sp + 24|0;
 $2 = sp + 20|0;
 $rc = sp + 16|0;
 $i = sp + 12|0;
 $pVTab = sp + 8|0;
 $pMod = sp + 4|0;
 $xMethod = sp;
 HEAP32[$0>>2] = $db;
 HEAP32[$1>>2] = $op;
 HEAP32[$2>>2] = $iSavepoint;
 HEAP32[$rc>>2] = 0;
 $3 = HEAP32[$0>>2]|0;
 $4 = (($3) + 328|0);
 $5 = HEAP32[$4>>2]|0;
 $6 = ($5|0)!=(0|0);
 if (!($6)) {
  $60 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($60|0);
 }
 HEAP32[$i>>2] = 0;
 while(1) {
  $7 = HEAP32[$rc>>2]|0;
  $8 = ($7|0)==(0);
  if (!($8)) {
   label = 15;
   break;
  }
  $9 = HEAP32[$i>>2]|0;
  $10 = HEAP32[$0>>2]|0;
  $11 = (($10) + 304|0);
  $12 = HEAP32[$11>>2]|0;
  $13 = ($9|0)<($12|0);
  if (!($13)) {
   label = 15;
   break;
  }
  $14 = HEAP32[$i>>2]|0;
  $15 = HEAP32[$0>>2]|0;
  $16 = (($15) + 328|0);
  $17 = HEAP32[$16>>2]|0;
  $18 = (($17) + ($14<<2)|0);
  $19 = HEAP32[$18>>2]|0;
  HEAP32[$pVTab>>2] = $19;
  $20 = HEAP32[$pVTab>>2]|0;
  $21 = (($20) + 4|0);
  $22 = HEAP32[$21>>2]|0;
  $23 = HEAP32[$22>>2]|0;
  HEAP32[$pMod>>2] = $23;
  $24 = HEAP32[$pVTab>>2]|0;
  $25 = (($24) + 8|0);
  $26 = HEAP32[$25>>2]|0;
  $27 = ($26|0)!=(0|0);
  if ($27) {
   $28 = HEAP32[$pMod>>2]|0;
   $29 = HEAP32[$28>>2]|0;
   $30 = ($29|0)>=(2);
   if ($30) {
    $31 = HEAP32[$1>>2]|0;
    if ((($31|0) == 0)) {
     $32 = HEAP32[$pMod>>2]|0;
     $33 = (($32) + 80|0);
     $34 = HEAP32[$33>>2]|0;
     HEAP32[$xMethod>>2] = $34;
     $35 = HEAP32[$2>>2]|0;
     $36 = (($35) + 1)|0;
     $37 = HEAP32[$pVTab>>2]|0;
     $38 = (($37) + 20|0);
     HEAP32[$38>>2] = $36;
    } else if ((($31|0) == 2)) {
     $39 = HEAP32[$pMod>>2]|0;
     $40 = (($39) + 88|0);
     $41 = HEAP32[$40>>2]|0;
     HEAP32[$xMethod>>2] = $41;
    } else {
     $42 = HEAP32[$pMod>>2]|0;
     $43 = (($42) + 84|0);
     $44 = HEAP32[$43>>2]|0;
     HEAP32[$xMethod>>2] = $44;
    }
    $45 = HEAP32[$xMethod>>2]|0;
    $46 = ($45|0)!=(0|0);
    if ($46) {
     $47 = HEAP32[$pVTab>>2]|0;
     $48 = (($47) + 20|0);
     $49 = HEAP32[$48>>2]|0;
     $50 = HEAP32[$2>>2]|0;
     $51 = ($49|0)>($50|0);
     if ($51) {
      $52 = HEAP32[$xMethod>>2]|0;
      $53 = HEAP32[$pVTab>>2]|0;
      $54 = (($53) + 8|0);
      $55 = HEAP32[$54>>2]|0;
      $56 = HEAP32[$2>>2]|0;
      $57 = (FUNCTION_TABLE_iii[$52 & 63]($55,$56)|0);
      HEAP32[$rc>>2] = $57;
     }
    }
   }
  }
  $58 = HEAP32[$i>>2]|0;
  $59 = (($58) + 1)|0;
  HEAP32[$i>>2] = $59;
 }
 if ((label|0) == 15) {
  $60 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($60|0);
 }
 return 0|0;
}
function _sqlite3BtreeTripAllCursors($pBtree,$errCode,$writeOnly) {
 $pBtree = $pBtree|0;
 $errCode = $errCode|0;
 $writeOnly = $writeOnly|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $i = 0;
 var $p = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 20|0;
 $1 = sp + 16|0;
 $2 = sp + 12|0;
 $p = sp + 8|0;
 $rc = sp + 4|0;
 $i = sp;
 HEAP32[$0>>2] = $pBtree;
 HEAP32[$1>>2] = $errCode;
 HEAP32[$2>>2] = $writeOnly;
 HEAP32[$rc>>2] = 0;
 $3 = HEAP32[$0>>2]|0;
 $4 = ($3|0)!=(0|0);
 if (!($4)) {
  $58 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($58|0);
 }
 $5 = HEAP32[$0>>2]|0;
 _sqlite3BtreeEnter($5);
 $6 = HEAP32[$0>>2]|0;
 $7 = (($6) + 4|0);
 $8 = HEAP32[$7>>2]|0;
 $9 = (($8) + 8|0);
 $10 = HEAP32[$9>>2]|0;
 HEAP32[$p>>2] = $10;
 while(1) {
  $11 = HEAP32[$p>>2]|0;
  $12 = ($11|0)!=(0|0);
  if (!($12)) {
   label = 14;
   break;
  }
  $13 = HEAP32[$2>>2]|0;
  $14 = ($13|0)!=(0);
  if ($14) {
   $15 = HEAP32[$p>>2]|0;
   $16 = (($15) + 72|0);
   $17 = HEAP8[$16>>0]|0;
   $18 = $17&255;
   $19 = $18 & 1;
   $20 = ($19|0)==(0);
   if ($20) {
    $21 = HEAP32[$p>>2]|0;
    $22 = (($21) + 73|0);
    $23 = HEAP8[$22>>0]|0;
    $24 = $23&255;
    $25 = ($24|0)==(1);
    if ($25) {
     $26 = HEAP32[$p>>2]|0;
     $27 = (_saveCursorPosition($26)|0);
     HEAP32[$rc>>2] = $27;
     $28 = HEAP32[$rc>>2]|0;
     $29 = ($28|0)!=(0);
     if ($29) {
      break;
     }
    }
   } else {
    label = 9;
   }
  } else {
   label = 9;
  }
  if ((label|0) == 9) {
   label = 0;
   $32 = HEAP32[$p>>2]|0;
   _sqlite3BtreeClearCursor($32);
   $33 = HEAP32[$p>>2]|0;
   $34 = (($33) + 73|0);
   HEAP8[$34>>0] = 4;
   $35 = HEAP32[$1>>2]|0;
   $36 = HEAP32[$p>>2]|0;
   $37 = (($36) + 68|0);
   HEAP32[$37>>2] = $35;
  }
  HEAP32[$i>>2] = 0;
  while(1) {
   $38 = HEAP32[$i>>2]|0;
   $39 = HEAP32[$p>>2]|0;
   $40 = (($39) + 76|0);
   $41 = HEAP16[$40>>1]|0;
   $42 = $41 << 16 >> 16;
   $43 = ($38|0)<=($42|0);
   if (!($43)) {
    break;
   }
   $44 = HEAP32[$i>>2]|0;
   $45 = HEAP32[$p>>2]|0;
   $46 = (($45) + 120|0);
   $47 = (($46) + ($44<<2)|0);
   $48 = HEAP32[$47>>2]|0;
   _releasePage($48);
   $49 = HEAP32[$i>>2]|0;
   $50 = HEAP32[$p>>2]|0;
   $51 = (($50) + 120|0);
   $52 = (($51) + ($49<<2)|0);
   HEAP32[$52>>2] = 0;
   $53 = HEAP32[$i>>2]|0;
   $54 = (($53) + 1)|0;
   HEAP32[$i>>2] = $54;
  }
  $55 = HEAP32[$p>>2]|0;
  $56 = (($55) + 8|0);
  $57 = HEAP32[$56>>2]|0;
  HEAP32[$p>>2] = $57;
 }
 if ((label|0) == 14) {
  $58 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($58|0);
 }
 $30 = HEAP32[$0>>2]|0;
 $31 = HEAP32[$rc>>2]|0;
 (_sqlite3BtreeTripAllCursors($30,$31,0)|0);
 $58 = HEAP32[$rc>>2]|0;
 STACKTOP = sp;return ($58|0);
}
function _sqlite3BtreeSavepoint($p,$op,$iSavepoint) {
 $p = $p|0;
 $op = $op|0;
 $iSavepoint = $iSavepoint|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $5 = 0, $6 = 0;
 var $7 = 0, $8 = 0, $9 = 0, $pBt = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 16|0;
 $1 = sp + 12|0;
 $2 = sp + 8|0;
 $rc = sp + 4|0;
 $pBt = sp;
 HEAP32[$0>>2] = $p;
 HEAP32[$1>>2] = $op;
 HEAP32[$2>>2] = $iSavepoint;
 HEAP32[$rc>>2] = 0;
 $3 = HEAP32[$0>>2]|0;
 $4 = ($3|0)!=(0|0);
 if (!($4)) {
  $42 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($42|0);
 }
 $5 = HEAP32[$0>>2]|0;
 $6 = (($5) + 8|0);
 $7 = HEAP8[$6>>0]|0;
 $8 = $7&255;
 $9 = ($8|0)==(2);
 if (!($9)) {
  $42 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($42|0);
 }
 $10 = HEAP32[$0>>2]|0;
 $11 = (($10) + 4|0);
 $12 = HEAP32[$11>>2]|0;
 HEAP32[$pBt>>2] = $12;
 $13 = HEAP32[$0>>2]|0;
 _sqlite3BtreeEnter($13);
 $14 = HEAP32[$pBt>>2]|0;
 $15 = HEAP32[$14>>2]|0;
 $16 = HEAP32[$1>>2]|0;
 $17 = HEAP32[$2>>2]|0;
 $18 = (_sqlite3PagerSavepoint($15,$16,$17)|0);
 HEAP32[$rc>>2] = $18;
 $19 = HEAP32[$rc>>2]|0;
 $20 = ($19|0)==(0);
 if (!($20)) {
  $42 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($42|0);
 }
 $21 = HEAP32[$2>>2]|0;
 $22 = ($21|0)<(0);
 if ($22) {
  $23 = HEAP32[$pBt>>2]|0;
  $24 = (($23) + 22|0);
  $25 = HEAP16[$24>>1]|0;
  $26 = $25&65535;
  $27 = $26 & 8;
  $28 = ($27|0)!=(0);
  if ($28) {
   $29 = HEAP32[$pBt>>2]|0;
   $30 = (($29) + 44|0);
   HEAP32[$30>>2] = 0;
  }
 }
 $31 = HEAP32[$pBt>>2]|0;
 $32 = (_newDatabase($31)|0);
 HEAP32[$rc>>2] = $32;
 $33 = HEAP32[$pBt>>2]|0;
 $34 = (($33) + 12|0);
 $35 = HEAP32[$34>>2]|0;
 $36 = (($35) + 56|0);
 $37 = HEAP32[$36>>2]|0;
 $38 = (($37) + 28|0);
 $39 = (_sqlite3Get4byte($38)|0);
 $40 = HEAP32[$pBt>>2]|0;
 $41 = (($40) + 44|0);
 HEAP32[$41>>2] = $39;
 $42 = HEAP32[$rc>>2]|0;
 STACKTOP = sp;return ($42|0);
}
function _sqlite3BtreeBeginStmt($p,$iStatement) {
 $p = $p|0;
 $iStatement = $iStatement|0;
 var $0 = 0, $1 = 0, $10 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $pBt = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $1 = sp + 8|0;
 $rc = sp + 4|0;
 $pBt = sp;
 HEAP32[$0>>2] = $p;
 HEAP32[$1>>2] = $iStatement;
 $2 = HEAP32[$0>>2]|0;
 $3 = (($2) + 4|0);
 $4 = HEAP32[$3>>2]|0;
 HEAP32[$pBt>>2] = $4;
 $5 = HEAP32[$0>>2]|0;
 _sqlite3BtreeEnter($5);
 $6 = HEAP32[$pBt>>2]|0;
 $7 = HEAP32[$6>>2]|0;
 $8 = HEAP32[$1>>2]|0;
 $9 = (_sqlite3PagerOpenSavepoint($7,$8)|0);
 HEAP32[$rc>>2] = $9;
 $10 = HEAP32[$rc>>2]|0;
 STACKTOP = sp;return ($10|0);
}
function _allocateCursor($p,$iCur,$nField,$iDb,$isBtreeCursor) {
 $p = $p|0;
 $iCur = $iCur|0;
 $nField = $nField|0;
 $iDb = $iDb|0;
 $isBtreeCursor = $isBtreeCursor|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0;
 var $9 = 0, $nByte = 0, $pCx = 0, $pMem = 0, dest = 0, label = 0, sp = 0, stop = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 28|0;
 $1 = sp + 24|0;
 $2 = sp + 20|0;
 $3 = sp + 16|0;
 $4 = sp + 12|0;
 $pMem = sp + 8|0;
 $nByte = sp + 4|0;
 $pCx = sp;
 HEAP32[$0>>2] = $p;
 HEAP32[$1>>2] = $iCur;
 HEAP32[$2>>2] = $nField;
 HEAP32[$3>>2] = $iDb;
 HEAP32[$4>>2] = $isBtreeCursor;
 $5 = HEAP32[$0>>2]|0;
 $6 = (($5) + 28|0);
 $7 = HEAP32[$6>>2]|0;
 $8 = HEAP32[$1>>2]|0;
 $9 = (($7) - ($8))|0;
 $10 = HEAP32[$0>>2]|0;
 $11 = (($10) + 8|0);
 $12 = HEAP32[$11>>2]|0;
 $13 = (($12) + (($9*40)|0)|0);
 HEAP32[$pMem>>2] = $13;
 HEAP32[$pCx>>2] = 0;
 $14 = HEAP32[$2>>2]|0;
 $15 = $14<<3;
 $16 = (88 + ($15))|0;
 $17 = HEAP32[$4>>2]|0;
 $18 = ($17|0)!=(0);
 if ($18) {
  $19 = (_sqlite3BtreeCursorSize()|0);
  $21 = $19;
 } else {
  $21 = 0;
 }
 $20 = (($16) + ($21))|0;
 HEAP32[$nByte>>2] = $20;
 $22 = HEAP32[$1>>2]|0;
 $23 = HEAP32[$0>>2]|0;
 $24 = (($23) + 56|0);
 $25 = HEAP32[$24>>2]|0;
 $26 = (($25) + ($22<<2)|0);
 $27 = HEAP32[$26>>2]|0;
 $28 = ($27|0)!=(0|0);
 if ($28) {
  $29 = HEAP32[$0>>2]|0;
  $30 = HEAP32[$1>>2]|0;
  $31 = HEAP32[$0>>2]|0;
  $32 = (($31) + 56|0);
  $33 = HEAP32[$32>>2]|0;
  $34 = (($33) + ($30<<2)|0);
  $35 = HEAP32[$34>>2]|0;
  _sqlite3VdbeFreeCursor($29,$35);
  $36 = HEAP32[$1>>2]|0;
  $37 = HEAP32[$0>>2]|0;
  $38 = (($37) + 56|0);
  $39 = HEAP32[$38>>2]|0;
  $40 = (($39) + ($36<<2)|0);
  HEAP32[$40>>2] = 0;
 }
 $41 = HEAP32[$pMem>>2]|0;
 $42 = HEAP32[$nByte>>2]|0;
 $43 = (_sqlite3VdbeMemClearAndResize($41,$42)|0);
 $44 = (0)==($43|0);
 if (!($44)) {
  $80 = HEAP32[$pCx>>2]|0;
  STACKTOP = sp;return ($80|0);
 }
 $45 = HEAP32[$pMem>>2]|0;
 $46 = (($45) + 16|0);
 $47 = HEAP32[$46>>2]|0;
 HEAP32[$pCx>>2] = $47;
 $48 = HEAP32[$1>>2]|0;
 $49 = HEAP32[$0>>2]|0;
 $50 = (($49) + 56|0);
 $51 = HEAP32[$50>>2]|0;
 $52 = (($51) + ($48<<2)|0);
 HEAP32[$52>>2] = $47;
 $53 = HEAP32[$pCx>>2]|0;
 dest=$53+0|0; stop=dest+88|0; do { HEAP32[dest>>2]=0|0; dest=dest+4|0; } while ((dest|0) < (stop|0));
 $54 = HEAP32[$3>>2]|0;
 $55 = $54&255;
 $56 = HEAP32[$pCx>>2]|0;
 $57 = (($56) + 24|0);
 HEAP8[$57>>0] = $55;
 $58 = HEAP32[$2>>2]|0;
 $59 = $58&65535;
 $60 = HEAP32[$pCx>>2]|0;
 $61 = (($60) + 20|0);
 HEAP16[$61>>1] = $59;
 $62 = HEAP32[$2>>2]|0;
 $63 = HEAP32[$pCx>>2]|0;
 $64 = (($63) + 84|0);
 $65 = (($64) + ($62<<2)|0);
 $66 = HEAP32[$pCx>>2]|0;
 $67 = (($66) + 80|0);
 HEAP32[$67>>2] = $65;
 $68 = HEAP32[$4>>2]|0;
 $69 = ($68|0)!=(0);
 if (!($69)) {
  $80 = HEAP32[$pCx>>2]|0;
  STACKTOP = sp;return ($80|0);
 }
 $70 = HEAP32[$2>>2]|0;
 $71 = $70<<3;
 $72 = (88 + ($71))|0;
 $73 = HEAP32[$pMem>>2]|0;
 $74 = (($73) + 16|0);
 $75 = HEAP32[$74>>2]|0;
 $76 = (($75) + ($72)|0);
 $77 = HEAP32[$pCx>>2]|0;
 HEAP32[$77>>2] = $76;
 $78 = HEAP32[$pCx>>2]|0;
 $79 = HEAP32[$78>>2]|0;
 _sqlite3BtreeCursorZero($79);
 $80 = HEAP32[$pCx>>2]|0;
 STACKTOP = sp;return ($80|0);
}
function _sqlite3BtreeCursor($p,$iTable,$wrFlag,$pKeyInfo,$pCur) {
 $p = $p|0;
 $iTable = $iTable|0;
 $wrFlag = $wrFlag|0;
 $pKeyInfo = $pKeyInfo|0;
 $pCur = $pCur|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 20|0;
 $1 = sp + 16|0;
 $2 = sp + 12|0;
 $3 = sp + 8|0;
 $4 = sp + 4|0;
 $rc = sp;
 HEAP32[$0>>2] = $p;
 HEAP32[$1>>2] = $iTable;
 HEAP32[$2>>2] = $wrFlag;
 HEAP32[$3>>2] = $pKeyInfo;
 HEAP32[$4>>2] = $pCur;
 $5 = HEAP32[$0>>2]|0;
 _sqlite3BtreeEnter($5);
 $6 = HEAP32[$0>>2]|0;
 $7 = HEAP32[$1>>2]|0;
 $8 = HEAP32[$2>>2]|0;
 $9 = HEAP32[$3>>2]|0;
 $10 = HEAP32[$4>>2]|0;
 $11 = (_btreeCursor($6,$7,$8,$9,$10)|0);
 HEAP32[$rc>>2] = $11;
 $12 = HEAP32[$rc>>2]|0;
 STACKTOP = sp;return ($12|0);
}
function _sqlite3BtreeCursorHints($pCsr,$mask) {
 $pCsr = $pCsr|0;
 $mask = $mask|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $1 = sp;
 HEAP32[$0>>2] = $pCsr;
 HEAP32[$1>>2] = $mask;
 $2 = HEAP32[$1>>2]|0;
 $3 = $2&255;
 $4 = HEAP32[$0>>2]|0;
 $5 = (($4) + 74|0);
 HEAP8[$5>>0] = $3;
 STACKTOP = sp;return;
}
function _sqlite3BtreeCreateTable($p,$piTable,$flags) {
 $p = $p|0;
 $piTable = $piTable|0;
 $flags = $flags|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $1 = sp + 8|0;
 $2 = sp + 4|0;
 $rc = sp;
 HEAP32[$0>>2] = $p;
 HEAP32[$1>>2] = $piTable;
 HEAP32[$2>>2] = $flags;
 $3 = HEAP32[$0>>2]|0;
 _sqlite3BtreeEnter($3);
 $4 = HEAP32[$0>>2]|0;
 $5 = HEAP32[$1>>2]|0;
 $6 = HEAP32[$2>>2]|0;
 $7 = (_btreeCreateTable($4,$5,$6)|0);
 HEAP32[$rc>>2] = $7;
 $8 = HEAP32[$rc>>2]|0;
 STACKTOP = sp;return ($8|0);
}
function _sqlite3VdbeSorterInit($db,$nField,$pCsr) {
 $db = $db|0;
 $nField = $nField|0;
 $pCsr = $pCsr|0;
 var $$ = 0, $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0;
 var $115 = 0, $116 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0;
 var $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0;
 var $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0;
 var $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0;
 var $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $i = 0, $mxCache = 0;
 var $pKeyInfo = 0, $pSorter = 0, $pTask = 0, $pgsz = 0, $rc = 0, $sz = 0, $szKeyInfo = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $0 = sp + 44|0;
 $1 = sp + 40|0;
 $2 = sp + 36|0;
 $pgsz = sp + 32|0;
 $i = sp + 28|0;
 $mxCache = sp + 24|0;
 $pSorter = sp + 20|0;
 $pKeyInfo = sp + 16|0;
 $szKeyInfo = sp + 12|0;
 $sz = sp + 8|0;
 $rc = sp + 4|0;
 $pTask = sp;
 HEAP32[$0>>2] = $db;
 HEAP32[$1>>2] = $nField;
 HEAP32[$2>>2] = $pCsr;
 HEAP32[$rc>>2] = 0;
 $3 = HEAP32[$2>>2]|0;
 $4 = (($3) + 8|0);
 $5 = HEAP32[$4>>2]|0;
 $6 = (($5) + 6|0);
 $7 = HEAP16[$6>>1]|0;
 $8 = $7&65535;
 $9 = (($8) - 1)|0;
 $10 = $9<<2;
 $11 = (24 + ($10))|0;
 HEAP32[$szKeyInfo>>2] = $11;
 HEAP32[$sz>>2] = 128;
 $12 = HEAP32[$0>>2]|0;
 $13 = HEAP32[$sz>>2]|0;
 $14 = HEAP32[$szKeyInfo>>2]|0;
 $15 = (($13) + ($14))|0;
 $16 = ($15|0)<(0);
 $17 = $16 << 31 >> 31;
 $18 = (_sqlite3DbMallocZero($12,$15,$17)|0);
 HEAP32[$pSorter>>2] = $18;
 $19 = HEAP32[$pSorter>>2]|0;
 $20 = HEAP32[$2>>2]|0;
 $21 = (($20) + 56|0);
 HEAP32[$21>>2] = $19;
 $22 = HEAP32[$pSorter>>2]|0;
 $23 = ($22|0)==(0|0);
 if ($23) {
  HEAP32[$rc>>2] = 7;
  $116 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($116|0);
 }
 $24 = HEAP32[$pSorter>>2]|0;
 $25 = HEAP32[$sz>>2]|0;
 $26 = (($24) + ($25)|0);
 HEAP32[$pKeyInfo>>2] = $26;
 $27 = HEAP32[$pSorter>>2]|0;
 $28 = (($27) + 28|0);
 HEAP32[$28>>2] = $26;
 $29 = HEAP32[$pKeyInfo>>2]|0;
 $30 = HEAP32[$2>>2]|0;
 $31 = (($30) + 8|0);
 $32 = HEAP32[$31>>2]|0;
 $33 = HEAP32[$szKeyInfo>>2]|0;
 _memcpy(($29|0),($32|0),($33|0))|0;
 $34 = HEAP32[$pKeyInfo>>2]|0;
 $35 = (($34) + 12|0);
 HEAP32[$35>>2] = 0;
 $36 = HEAP32[$1>>2]|0;
 $37 = ($36|0)!=(0);
 if ($37) {
  $38 = HEAP32[$1>>2]|0;
  $39 = $38&65535;
  $40 = HEAP32[$pKeyInfo>>2]|0;
  $41 = (($40) + 6|0);
  HEAP16[$41>>1] = $39;
 }
 $42 = HEAP32[$0>>2]|0;
 $43 = (($42) + 16|0);
 $44 = HEAP32[$43>>2]|0;
 $45 = (($44) + 4|0);
 $46 = HEAP32[$45>>2]|0;
 $47 = (_sqlite3BtreeGetPageSize($46)|0);
 HEAP32[$pgsz>>2] = $47;
 $48 = HEAP32[$pSorter>>2]|0;
 $49 = (($48) + 12|0);
 HEAP32[$49>>2] = $47;
 $50 = HEAP32[$pSorter>>2]|0;
 $51 = (($50) + 59|0);
 HEAP8[$51>>0] = 1;
 $52 = HEAP32[$pSorter>>2]|0;
 $53 = (($52) + 59|0);
 $54 = HEAP8[$53>>0]|0;
 $55 = $54&255;
 $56 = ($55|0)>(1);
 $57 = $56&1;
 $58 = $57&255;
 $59 = HEAP32[$pSorter>>2]|0;
 $60 = (($59) + 57|0);
 HEAP8[$60>>0] = $58;
 $61 = HEAP32[$0>>2]|0;
 $62 = HEAP32[$pSorter>>2]|0;
 $63 = (($62) + 24|0);
 HEAP32[$63>>2] = $61;
 HEAP32[$i>>2] = 0;
 while(1) {
  $64 = HEAP32[$i>>2]|0;
  $65 = HEAP32[$pSorter>>2]|0;
  $66 = (($65) + 59|0);
  $67 = HEAP8[$66>>0]|0;
  $68 = $67&255;
  $69 = ($64|0)<($68|0);
  if (!($69)) {
   break;
  }
  $70 = HEAP32[$i>>2]|0;
  $71 = HEAP32[$pSorter>>2]|0;
  $72 = (($71) + 64|0);
  $73 = (($72) + ($70<<6)|0);
  HEAP32[$pTask>>2] = $73;
  $74 = HEAP32[$pSorter>>2]|0;
  $75 = HEAP32[$pTask>>2]|0;
  $76 = (($75) + 8|0);
  HEAP32[$76>>2] = $74;
  $77 = HEAP32[$i>>2]|0;
  $78 = (($77) + 1)|0;
  HEAP32[$i>>2] = $78;
 }
 $79 = HEAP32[$0>>2]|0;
 $80 = (_sqlite3TempInMemory($79)|0);
 $81 = ($80|0)!=(0);
 if ($81) {
  $116 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($116|0);
 }
 $82 = HEAP32[$pgsz>>2]|0;
 $83 = ($82*10)|0;
 $84 = HEAP32[$pSorter>>2]|0;
 HEAP32[$84>>2] = $83;
 $85 = HEAP32[$0>>2]|0;
 $86 = (($85) + 16|0);
 $87 = HEAP32[$86>>2]|0;
 $88 = (($87) + 12|0);
 $89 = HEAP32[$88>>2]|0;
 $90 = (($89) + 80|0);
 $91 = HEAP32[$90>>2]|0;
 HEAP32[$mxCache>>2] = $91;
 $92 = HEAP32[$mxCache>>2]|0;
 $93 = ($92|0)<(10);
 $$ = $93 ? 10 : $91;
 HEAP32[$mxCache>>2] = $$;
 $94 = HEAP32[$mxCache>>2]|0;
 $95 = HEAP32[$pgsz>>2]|0;
 $96 = Math_imul($94, $95)|0;
 $97 = HEAP32[$pSorter>>2]|0;
 $98 = (($97) + 4|0);
 HEAP32[$98>>2] = $96;
 $99 = HEAP32[((38640 + 192|0))>>2]|0;
 $100 = ($99|0)==(0|0);
 if (!($100)) {
  $116 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($116|0);
 }
 $101 = HEAP32[$pgsz>>2]|0;
 $102 = HEAP32[$pSorter>>2]|0;
 $103 = (($102) + 52|0);
 HEAP32[$103>>2] = $101;
 $104 = HEAP32[$pgsz>>2]|0;
 $105 = ($104|0)<(0);
 $106 = $105 << 31 >> 31;
 $107 = (_sqlite3Malloc($104,$106)|0);
 $108 = HEAP32[$pSorter>>2]|0;
 $109 = (($108) + 36|0);
 $110 = (($109) + 4|0);
 HEAP32[$110>>2] = $107;
 $111 = HEAP32[$pSorter>>2]|0;
 $112 = (($111) + 36|0);
 $113 = (($112) + 4|0);
 $114 = HEAP32[$113>>2]|0;
 $115 = ($114|0)!=(0|0);
 if ($115) {
  $116 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($116|0);
 }
 HEAP32[$rc>>2] = 7;
 $116 = HEAP32[$rc>>2]|0;
 STACKTOP = sp;return ($116|0);
}
function _sqlite3VdbeFreeCursor($p,$pCx) {
 $p = $p|0;
 $pCx = $pCx|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0;
 var $pModule = 0, $pVtabCursor = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $1 = sp + 8|0;
 $pVtabCursor = sp + 4|0;
 $pModule = sp;
 HEAP32[$0>>2] = $p;
 HEAP32[$1>>2] = $pCx;
 $2 = HEAP32[$1>>2]|0;
 $3 = ($2|0)==(0|0);
 if ($3) {
  STACKTOP = sp;return;
 }
 $4 = HEAP32[$0>>2]|0;
 $5 = HEAP32[$4>>2]|0;
 $6 = HEAP32[$1>>2]|0;
 _sqlite3VdbeSorterClose($5,$6);
 $7 = HEAP32[$1>>2]|0;
 $8 = (($7) + 4|0);
 $9 = HEAP32[$8>>2]|0;
 $10 = ($9|0)!=(0|0);
 $11 = HEAP32[$1>>2]|0;
 if ($10) {
  $12 = (($11) + 4|0);
  $13 = HEAP32[$12>>2]|0;
  (_sqlite3BtreeClose($13)|0);
  STACKTOP = sp;return;
 }
 $14 = HEAP32[$11>>2]|0;
 $15 = ($14|0)!=(0|0);
 $16 = HEAP32[$1>>2]|0;
 if ($15) {
  $17 = HEAP32[$16>>2]|0;
  (_sqlite3BtreeCloseCursor($17)|0);
  STACKTOP = sp;return;
 }
 $18 = (($16) + 32|0);
 $19 = HEAP32[$18>>2]|0;
 $20 = ($19|0)!=(0|0);
 if (!($20)) {
  STACKTOP = sp;return;
 }
 $21 = HEAP32[$1>>2]|0;
 $22 = (($21) + 32|0);
 $23 = HEAP32[$22>>2]|0;
 HEAP32[$pVtabCursor>>2] = $23;
 $24 = HEAP32[$pVtabCursor>>2]|0;
 $25 = HEAP32[$24>>2]|0;
 $26 = HEAP32[$25>>2]|0;
 HEAP32[$pModule>>2] = $26;
 $27 = HEAP32[$0>>2]|0;
 $28 = (($27) + 88|0);
 $29 = HEAP16[$28>>1]|0;
 $30 = $29 & -13;
 $31 = $30 | 4;
 HEAP16[$28>>1] = $31;
 $32 = HEAP32[$pModule>>2]|0;
 $33 = (($32) + 28|0);
 $34 = HEAP32[$33>>2]|0;
 $35 = HEAP32[$pVtabCursor>>2]|0;
 (FUNCTION_TABLE_ii[$34 & 31]($35)|0);
 $36 = HEAP32[$0>>2]|0;
 $37 = (($36) + 88|0);
 $38 = HEAP16[$37>>1]|0;
 $39 = $38 & -13;
 HEAP16[$37>>1] = $39;
 STACKTOP = sp;return;
}
function _sqlite3BtreeEof($pCur) {
 $pCur = $pCur|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 HEAP32[$0>>2] = $pCur;
 $1 = HEAP32[$0>>2]|0;
 $2 = (($1) + 73|0);
 $3 = HEAP8[$2>>0]|0;
 $4 = $3&255;
 $5 = (1)!=($4|0);
 $6 = $5&1;
 STACKTOP = sp;return ($6|0);
}
function _sqlite3BtreeLast($pCur,$pRes) {
 $pCur = $pCur|0;
 $pRes = $pRes|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $rc = 0, label = 0;
 var sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $1 = sp + 8|0;
 $2 = sp + 4|0;
 $rc = sp;
 HEAP32[$1>>2] = $pCur;
 HEAP32[$2>>2] = $pRes;
 $3 = HEAP32[$1>>2]|0;
 $4 = (($3) + 73|0);
 $5 = HEAP8[$4>>0]|0;
 $6 = $5&255;
 $7 = (1)==($6|0);
 if ($7) {
  $8 = HEAP32[$1>>2]|0;
  $9 = (($8) + 72|0);
  $10 = HEAP8[$9>>0]|0;
  $11 = $10&255;
  $12 = $11 & 8;
  $13 = ($12|0)!=(0);
  if ($13) {
   HEAP32[$0>>2] = 0;
   $37 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($37|0);
  }
 }
 $14 = HEAP32[$1>>2]|0;
 $15 = (_moveToRoot($14)|0);
 HEAP32[$rc>>2] = $15;
 $16 = HEAP32[$rc>>2]|0;
 $17 = ($16|0)==(0);
 do {
  if ($17) {
   $18 = HEAP32[$1>>2]|0;
   $19 = (($18) + 73|0);
   $20 = HEAP8[$19>>0]|0;
   $21 = $20&255;
   $22 = (0)==($21|0);
   $23 = HEAP32[$2>>2]|0;
   if ($22) {
    HEAP32[$23>>2] = 1;
    break;
   }
   HEAP32[$23>>2] = 0;
   $24 = HEAP32[$1>>2]|0;
   $25 = (_moveToRightmost($24)|0);
   HEAP32[$rc>>2] = $25;
   $26 = HEAP32[$rc>>2]|0;
   $27 = ($26|0)==(0);
   $28 = HEAP32[$1>>2]|0;
   $29 = (($28) + 72|0);
   $30 = HEAP8[$29>>0]|0;
   $31 = $30&255;
   if ($27) {
    $32 = $31 | 8;
    $33 = $32&255;
    HEAP8[$29>>0] = $33;
    break;
   } else {
    $34 = $31 & -9;
    $35 = $34&255;
    HEAP8[$29>>0] = $35;
    break;
   }
  }
 } while(0);
 $36 = HEAP32[$rc>>2]|0;
 HEAP32[$0>>2] = $36;
 $37 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($37|0);
}
function _sqlite3BtreeInsert($pCur,$pKey,$0,$1,$pData,$nData,$nZero,$appendBias,$seekResult) {
 $pCur = $pCur|0;
 $pKey = $pKey|0;
 $0 = $0|0;
 $1 = $1|0;
 $pData = $pData|0;
 $nData = $nData|0;
 $nZero = $nZero|0;
 $appendBias = $appendBias|0;
 $seekResult = $seekResult|0;
 var $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0, $116 = 0, $117 = 0;
 var $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0, $134 = 0, $135 = 0;
 var $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0, $152 = 0, $153 = 0;
 var $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0, $170 = 0, $171 = 0;
 var $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0, $188 = 0, $189 = 0, $19 = 0;
 var $190 = 0, $191 = 0, $192 = 0, $193 = 0, $194 = 0, $195 = 0, $196 = 0, $197 = 0, $198 = 0, $199 = 0, $2 = 0, $20 = 0, $200 = 0, $201 = 0, $202 = 0, $203 = 0, $204 = 0, $205 = 0, $206 = 0, $207 = 0;
 var $208 = 0, $209 = 0, $21 = 0, $210 = 0, $211 = 0, $212 = 0, $213 = 0, $214 = 0, $215 = 0, $216 = 0, $217 = 0, $218 = 0, $219 = 0, $22 = 0, $220 = 0, $221 = 0, $222 = 0, $223 = 0, $224 = 0, $225 = 0;
 var $226 = 0, $227 = 0, $228 = 0, $229 = 0, $23 = 0, $230 = 0, $231 = 0, $232 = 0, $233 = 0, $234 = 0, $235 = 0, $236 = 0, $237 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0;
 var $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0;
 var $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0;
 var $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0;
 var $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $idx = 0, $loc = 0, $newCell = 0, $oldCell = 0;
 var $or$cond = 0, $p = 0, $pBt = 0, $pPage = 0, $rc = 0, $szNew = 0, $szOld = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 80|0;
 $2 = sp + 72|0;
 $3 = sp + 68|0;
 $4 = sp + 64|0;
 $5 = sp;
 $6 = sp + 60|0;
 $7 = sp + 56|0;
 $8 = sp + 52|0;
 $9 = sp + 48|0;
 $10 = sp + 44|0;
 $rc = sp + 40|0;
 $loc = sp + 36|0;
 $szNew = sp + 32|0;
 $idx = sp + 28|0;
 $pPage = sp + 24|0;
 $p = sp + 20|0;
 $pBt = sp + 16|0;
 $oldCell = sp + 12|0;
 $newCell = sp + 8|0;
 $szOld = sp + 76|0;
 HEAP32[$3>>2] = $pCur;
 HEAP32[$4>>2] = $pKey;
 $11 = $5;
 $12 = $11;
 HEAP32[$12>>2] = $0;
 $13 = (($11) + 4)|0;
 $14 = $13;
 HEAP32[$14>>2] = $1;
 HEAP32[$6>>2] = $pData;
 HEAP32[$7>>2] = $nData;
 HEAP32[$8>>2] = $nZero;
 HEAP32[$9>>2] = $appendBias;
 HEAP32[$10>>2] = $seekResult;
 $15 = HEAP32[$10>>2]|0;
 HEAP32[$loc>>2] = $15;
 HEAP32[$szNew>>2] = 0;
 $16 = HEAP32[$3>>2]|0;
 $17 = HEAP32[$16>>2]|0;
 HEAP32[$p>>2] = $17;
 $18 = HEAP32[$p>>2]|0;
 $19 = (($18) + 4|0);
 $20 = HEAP32[$19>>2]|0;
 HEAP32[$pBt>>2] = $20;
 HEAP32[$newCell>>2] = 0;
 $21 = HEAP32[$3>>2]|0;
 $22 = (($21) + 73|0);
 $23 = HEAP8[$22>>0]|0;
 $24 = $23&255;
 $25 = ($24|0)==(4);
 if ($25) {
  $26 = HEAP32[$3>>2]|0;
  $27 = (($26) + 68|0);
  $28 = HEAP32[$27>>2]|0;
  HEAP32[$2>>2] = $28;
  $237 = HEAP32[$2>>2]|0;
  STACKTOP = sp;return ($237|0);
 }
 $29 = HEAP32[$pBt>>2]|0;
 $30 = HEAP32[$3>>2]|0;
 $31 = (($30) + 60|0);
 $32 = HEAP32[$31>>2]|0;
 $33 = HEAP32[$3>>2]|0;
 $34 = (_saveAllCursors($29,$32,$33)|0);
 HEAP32[$rc>>2] = $34;
 $35 = HEAP32[$rc>>2]|0;
 $36 = ($35|0)!=(0);
 if ($36) {
  $37 = HEAP32[$rc>>2]|0;
  HEAP32[$2>>2] = $37;
  $237 = HEAP32[$2>>2]|0;
  STACKTOP = sp;return ($237|0);
 }
 $38 = HEAP32[$3>>2]|0;
 $39 = (($38) + 16|0);
 $40 = HEAP32[$39>>2]|0;
 $41 = ($40|0)==(0|0);
 if ($41) {
  $42 = HEAP32[$p>>2]|0;
  $43 = $5;
  $44 = $43;
  $45 = HEAP32[$44>>2]|0;
  $46 = (($43) + 4)|0;
  $47 = $46;
  $48 = HEAP32[$47>>2]|0;
  _invalidateIncrblobCursors($42,$45,$48,0);
  $49 = HEAP32[$3>>2]|0;
  $50 = (($49) + 72|0);
  $51 = HEAP8[$50>>0]|0;
  $52 = $51&255;
  $53 = $52 & 2;
  $54 = ($53|0)!=(0);
  $55 = $5;
  $56 = $55;
  $57 = HEAP32[$56>>2]|0;
  $58 = (($55) + 4)|0;
  $59 = $58;
  $60 = HEAP32[$59>>2]|0;
  $61 = ($60|0)>(0);
  $62 = ($57>>>0)>(0);
  $63 = ($60|0)==(0);
  $64 = $63 & $62;
  $65 = $61 | $64;
  $or$cond = $54 & $65;
  if ($or$cond) {
   $66 = HEAP32[$3>>2]|0;
   $67 = (($66) + 24|0);
   $68 = $67;
   $69 = $68;
   $70 = HEAP32[$69>>2]|0;
   $71 = (($68) + 4)|0;
   $72 = $71;
   $73 = HEAP32[$72>>2]|0;
   $74 = $5;
   $75 = $74;
   $76 = HEAP32[$75>>2]|0;
   $77 = (($74) + 4)|0;
   $78 = $77;
   $79 = HEAP32[$78>>2]|0;
   $80 = (_i64Subtract(($76|0),($79|0),1,0)|0);
   $81 = tempRet0;
   $82 = ($70|0)==($80|0);
   $83 = ($73|0)==($81|0);
   $84 = $82 & $83;
   if ($84) {
    HEAP32[$loc>>2] = -1;
   }
  }
 }
 $85 = HEAP32[$loc>>2]|0;
 $86 = ($85|0)!=(0);
 if (!($86)) {
  $87 = HEAP32[$3>>2]|0;
  $88 = HEAP32[$4>>2]|0;
  $89 = $5;
  $90 = $89;
  $91 = HEAP32[$90>>2]|0;
  $92 = (($89) + 4)|0;
  $93 = $92;
  $94 = HEAP32[$93>>2]|0;
  $95 = HEAP32[$9>>2]|0;
  $96 = (_btreeMoveto($87,$88,$91,$94,$95,$loc)|0);
  HEAP32[$rc>>2] = $96;
  $97 = HEAP32[$rc>>2]|0;
  $98 = ($97|0)!=(0);
  if ($98) {
   $99 = HEAP32[$rc>>2]|0;
   HEAP32[$2>>2] = $99;
   $237 = HEAP32[$2>>2]|0;
   STACKTOP = sp;return ($237|0);
  }
 }
 $100 = HEAP32[$3>>2]|0;
 $101 = (($100) + 76|0);
 $102 = HEAP16[$101>>1]|0;
 $103 = $102 << 16 >> 16;
 $104 = HEAP32[$3>>2]|0;
 $105 = (($104) + 120|0);
 $106 = (($105) + ($103<<2)|0);
 $107 = HEAP32[$106>>2]|0;
 HEAP32[$pPage>>2] = $107;
 $108 = HEAP32[$pBt>>2]|0;
 $109 = (($108) + 80|0);
 $110 = HEAP32[$109>>2]|0;
 HEAP32[$newCell>>2] = $110;
 $111 = HEAP32[$pPage>>2]|0;
 $112 = HEAP32[$newCell>>2]|0;
 $113 = HEAP32[$4>>2]|0;
 $114 = $5;
 $115 = $114;
 $116 = HEAP32[$115>>2]|0;
 $117 = (($114) + 4)|0;
 $118 = $117;
 $119 = HEAP32[$118>>2]|0;
 $120 = HEAP32[$6>>2]|0;
 $121 = HEAP32[$7>>2]|0;
 $122 = HEAP32[$8>>2]|0;
 $123 = (_fillInCell($111,$112,$113,$116,$119,$120,$121,$122,$szNew)|0);
 HEAP32[$rc>>2] = $123;
 $124 = HEAP32[$rc>>2]|0;
 $125 = ($124|0)!=(0);
 do {
  if (!($125)) {
   $126 = HEAP32[$3>>2]|0;
   $127 = (($126) + 76|0);
   $128 = HEAP16[$127>>1]|0;
   $129 = $128 << 16 >> 16;
   $130 = HEAP32[$3>>2]|0;
   $131 = (($130) + 78|0);
   $132 = (($131) + ($129<<1)|0);
   $133 = HEAP16[$132>>1]|0;
   $134 = $133&65535;
   HEAP32[$idx>>2] = $134;
   $135 = HEAP32[$loc>>2]|0;
   $136 = ($135|0)==(0);
   if ($136) {
    $137 = HEAP32[$pPage>>2]|0;
    $138 = (($137) + 68|0);
    $139 = HEAP32[$138>>2]|0;
    $140 = (_sqlite3PagerWrite($139)|0);
    HEAP32[$rc>>2] = $140;
    $141 = HEAP32[$rc>>2]|0;
    $142 = ($141|0)!=(0);
    if ($142) {
     break;
    }
    $143 = HEAP32[$pPage>>2]|0;
    $144 = (($143) + 56|0);
    $145 = HEAP32[$144>>2]|0;
    $146 = HEAP32[$pPage>>2]|0;
    $147 = (($146) + 20|0);
    $148 = HEAP16[$147>>1]|0;
    $149 = $148&65535;
    $150 = HEAP32[$idx>>2]|0;
    $151 = $150<<1;
    $152 = HEAP32[$pPage>>2]|0;
    $153 = (($152) + 64|0);
    $154 = HEAP32[$153>>2]|0;
    $155 = (($154) + ($151)|0);
    $156 = HEAP8[$155>>0]|0;
    $157 = $156&255;
    $158 = $157 << 8;
    $159 = HEAP32[$idx>>2]|0;
    $160 = $159<<1;
    $161 = HEAP32[$pPage>>2]|0;
    $162 = (($161) + 64|0);
    $163 = HEAP32[$162>>2]|0;
    $164 = (($163) + ($160)|0);
    $165 = (($164) + 1|0);
    $166 = HEAP8[$165>>0]|0;
    $167 = $166&255;
    $168 = $158 | $167;
    $169 = $149 & $168;
    $170 = (($145) + ($169)|0);
    HEAP32[$oldCell>>2] = $170;
    $171 = HEAP32[$pPage>>2]|0;
    $172 = (($171) + 5|0);
    $173 = HEAP8[$172>>0]|0;
    $174 = ($173<<24>>24)!=(0);
    if (!($174)) {
     $175 = HEAP32[$newCell>>2]|0;
     $176 = HEAP32[$oldCell>>2]|0;
     ;HEAP8[$175+0>>0]=HEAP8[$176+0>>0]|0;HEAP8[$175+1>>0]=HEAP8[$176+1>>0]|0;HEAP8[$175+2>>0]=HEAP8[$176+2>>0]|0;HEAP8[$175+3>>0]=HEAP8[$176+3>>0]|0;
    }
    $177 = HEAP32[$pPage>>2]|0;
    $178 = HEAP32[$oldCell>>2]|0;
    $179 = (_clearCell($177,$178,$szOld)|0);
    HEAP32[$rc>>2] = $179;
    $180 = HEAP32[$pPage>>2]|0;
    $181 = HEAP32[$idx>>2]|0;
    $182 = HEAP16[$szOld>>1]|0;
    $183 = $182&65535;
    _dropCell($180,$181,$183,$rc);
    $184 = HEAP32[$rc>>2]|0;
    $185 = ($184|0)!=(0);
    if ($185) {
     break;
    }
   } else {
    $186 = HEAP32[$loc>>2]|0;
    $187 = ($186|0)<(0);
    if ($187) {
     $188 = HEAP32[$pPage>>2]|0;
     $189 = (($188) + 18|0);
     $190 = HEAP16[$189>>1]|0;
     $191 = $190&65535;
     $192 = ($191|0)>(0);
     if ($192) {
      $193 = HEAP32[$3>>2]|0;
      $194 = (($193) + 76|0);
      $195 = HEAP16[$194>>1]|0;
      $196 = $195 << 16 >> 16;
      $197 = HEAP32[$3>>2]|0;
      $198 = (($197) + 78|0);
      $199 = (($198) + ($196<<1)|0);
      $200 = HEAP16[$199>>1]|0;
      $201 = (($200) + 1)<<16>>16;
      HEAP16[$199>>1] = $201;
      $202 = $201&65535;
      HEAP32[$idx>>2] = $202;
     }
    }
   }
   $203 = HEAP32[$pPage>>2]|0;
   $204 = HEAP32[$idx>>2]|0;
   $205 = HEAP32[$newCell>>2]|0;
   $206 = HEAP32[$szNew>>2]|0;
   _insertCell($203,$204,$205,$206,0,0,$rc);
   $207 = HEAP32[$3>>2]|0;
   $208 = (($207) + 24|0);
   $209 = (($208) + 20|0);
   HEAP16[$209>>1] = 0;
   $210 = HEAP32[$rc>>2]|0;
   $211 = ($210|0)==(0);
   if ($211) {
    $212 = HEAP32[$pPage>>2]|0;
    $213 = (($212) + 1|0);
    $214 = HEAP8[$213>>0]|0;
    $215 = $214&255;
    $216 = ($215|0)!=(0);
    if ($216) {
     $217 = HEAP32[$3>>2]|0;
     $218 = (($217) + 72|0);
     $219 = HEAP8[$218>>0]|0;
     $220 = $219&255;
     $221 = $220 & -3;
     $222 = $221&255;
     HEAP8[$218>>0] = $222;
     $223 = HEAP32[$3>>2]|0;
     $224 = (_balance($223)|0);
     HEAP32[$rc>>2] = $224;
     $225 = HEAP32[$3>>2]|0;
     $226 = (($225) + 76|0);
     $227 = HEAP16[$226>>1]|0;
     $228 = $227 << 16 >> 16;
     $229 = HEAP32[$3>>2]|0;
     $230 = (($229) + 120|0);
     $231 = (($230) + ($228<<2)|0);
     $232 = HEAP32[$231>>2]|0;
     $233 = (($232) + 1|0);
     HEAP8[$233>>0] = 0;
     $234 = HEAP32[$3>>2]|0;
     $235 = (($234) + 73|0);
     HEAP8[$235>>0] = 0;
    }
   }
  }
 } while(0);
 $236 = HEAP32[$rc>>2]|0;
 HEAP32[$2>>2] = $236;
 $237 = HEAP32[$2>>2]|0;
 STACKTOP = sp;return ($237|0);
}
function _sqlite3BtreeDelete($pCur) {
 $pCur = $pCur|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0;
 var $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0, $188 = 0;
 var $189 = 0, $19 = 0, $190 = 0, $191 = 0, $192 = 0, $193 = 0, $194 = 0, $195 = 0, $196 = 0, $197 = 0, $198 = 0, $199 = 0, $2 = 0, $20 = 0, $200 = 0, $201 = 0, $202 = 0, $203 = 0, $204 = 0, $205 = 0;
 var $206 = 0, $207 = 0, $208 = 0, $209 = 0, $21 = 0, $210 = 0, $211 = 0, $212 = 0, $213 = 0, $214 = 0, $215 = 0, $216 = 0, $217 = 0, $218 = 0, $219 = 0, $22 = 0, $220 = 0, $221 = 0, $222 = 0, $223 = 0;
 var $224 = 0, $225 = 0, $226 = 0, $227 = 0, $228 = 0, $229 = 0, $23 = 0, $230 = 0, $231 = 0, $232 = 0, $233 = 0, $234 = 0, $235 = 0, $236 = 0, $237 = 0, $238 = 0, $239 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0;
 var $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0;
 var $iCellDepth = 0, $iCellIdx = 0, $n = 0, $nCell = 0, $notUsed = 0, $p = 0, $pBt = 0, $pCell = 0, $pLeaf = 0, $pPage = 0, $pTmp = 0, $rc = 0, $szCell = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 64|0;
 $0 = sp + 52|0;
 $1 = sp + 48|0;
 $p = sp + 44|0;
 $pBt = sp + 40|0;
 $rc = sp + 36|0;
 $pPage = sp + 32|0;
 $pCell = sp + 28|0;
 $iCellIdx = sp + 24|0;
 $iCellDepth = sp + 20|0;
 $szCell = sp + 56|0;
 $notUsed = sp + 16|0;
 $pLeaf = sp + 12|0;
 $nCell = sp + 8|0;
 $n = sp + 4|0;
 $pTmp = sp;
 HEAP32[$1>>2] = $pCur;
 $2 = HEAP32[$1>>2]|0;
 $3 = HEAP32[$2>>2]|0;
 HEAP32[$p>>2] = $3;
 $4 = HEAP32[$p>>2]|0;
 $5 = (($4) + 4|0);
 $6 = HEAP32[$5>>2]|0;
 HEAP32[$pBt>>2] = $6;
 $7 = HEAP32[$1>>2]|0;
 $8 = (($7) + 76|0);
 $9 = HEAP16[$8>>1]|0;
 $10 = $9 << 16 >> 16;
 $11 = HEAP32[$1>>2]|0;
 $12 = (($11) + 78|0);
 $13 = (($12) + ($10<<1)|0);
 $14 = HEAP16[$13>>1]|0;
 $15 = $14&65535;
 $16 = HEAP32[$1>>2]|0;
 $17 = (($16) + 76|0);
 $18 = HEAP16[$17>>1]|0;
 $19 = $18 << 16 >> 16;
 $20 = HEAP32[$1>>2]|0;
 $21 = (($20) + 120|0);
 $22 = (($21) + ($19<<2)|0);
 $23 = HEAP32[$22>>2]|0;
 $24 = (($23) + 18|0);
 $25 = HEAP16[$24>>1]|0;
 $26 = $25&65535;
 $27 = ($15|0)>=($26|0);
 if (!($27)) {
  $28 = HEAP32[$1>>2]|0;
  $29 = (($28) + 73|0);
  $30 = HEAP8[$29>>0]|0;
  $31 = $30&255;
  $32 = ($31|0)!=(1);
  if (!($32)) {
   $33 = HEAP32[$1>>2]|0;
   $34 = (($33) + 76|0);
   $35 = HEAP16[$34>>1]|0;
   $36 = $35 << 16 >> 16;
   HEAP32[$iCellDepth>>2] = $36;
   $37 = HEAP32[$iCellDepth>>2]|0;
   $38 = HEAP32[$1>>2]|0;
   $39 = (($38) + 78|0);
   $40 = (($39) + ($37<<1)|0);
   $41 = HEAP16[$40>>1]|0;
   $42 = $41&65535;
   HEAP32[$iCellIdx>>2] = $42;
   $43 = HEAP32[$iCellDepth>>2]|0;
   $44 = HEAP32[$1>>2]|0;
   $45 = (($44) + 120|0);
   $46 = (($45) + ($43<<2)|0);
   $47 = HEAP32[$46>>2]|0;
   HEAP32[$pPage>>2] = $47;
   $48 = HEAP32[$pPage>>2]|0;
   $49 = (($48) + 56|0);
   $50 = HEAP32[$49>>2]|0;
   $51 = HEAP32[$pPage>>2]|0;
   $52 = (($51) + 20|0);
   $53 = HEAP16[$52>>1]|0;
   $54 = $53&65535;
   $55 = HEAP32[$iCellIdx>>2]|0;
   $56 = $55<<1;
   $57 = HEAP32[$pPage>>2]|0;
   $58 = (($57) + 64|0);
   $59 = HEAP32[$58>>2]|0;
   $60 = (($59) + ($56)|0);
   $61 = HEAP8[$60>>0]|0;
   $62 = $61&255;
   $63 = $62 << 8;
   $64 = HEAP32[$iCellIdx>>2]|0;
   $65 = $64<<1;
   $66 = HEAP32[$pPage>>2]|0;
   $67 = (($66) + 64|0);
   $68 = HEAP32[$67>>2]|0;
   $69 = (($68) + ($65)|0);
   $70 = (($69) + 1|0);
   $71 = HEAP8[$70>>0]|0;
   $72 = $71&255;
   $73 = $63 | $72;
   $74 = $54 & $73;
   $75 = (($50) + ($74)|0);
   HEAP32[$pCell>>2] = $75;
   $76 = HEAP32[$pPage>>2]|0;
   $77 = (($76) + 5|0);
   $78 = HEAP8[$77>>0]|0;
   $79 = ($78<<24>>24)!=(0);
   if (!($79)) {
    HEAP32[$notUsed>>2] = 0;
    $80 = HEAP32[$1>>2]|0;
    $81 = (_sqlite3BtreePrevious($80,$notUsed)|0);
    HEAP32[$rc>>2] = $81;
    $82 = HEAP32[$rc>>2]|0;
    $83 = ($82|0)!=(0);
    if ($83) {
     $84 = HEAP32[$rc>>2]|0;
     HEAP32[$0>>2] = $84;
     $239 = HEAP32[$0>>2]|0;
     STACKTOP = sp;return ($239|0);
    }
   }
   $85 = HEAP32[$pBt>>2]|0;
   $86 = HEAP32[$1>>2]|0;
   $87 = (($86) + 60|0);
   $88 = HEAP32[$87>>2]|0;
   $89 = HEAP32[$1>>2]|0;
   $90 = (_saveAllCursors($85,$88,$89)|0);
   HEAP32[$rc>>2] = $90;
   $91 = HEAP32[$rc>>2]|0;
   $92 = ($91|0)!=(0);
   if ($92) {
    $93 = HEAP32[$rc>>2]|0;
    HEAP32[$0>>2] = $93;
    $239 = HEAP32[$0>>2]|0;
    STACKTOP = sp;return ($239|0);
   }
   $94 = HEAP32[$1>>2]|0;
   $95 = (($94) + 16|0);
   $96 = HEAP32[$95>>2]|0;
   $97 = ($96|0)==(0|0);
   if ($97) {
    $98 = HEAP32[$p>>2]|0;
    $99 = HEAP32[$1>>2]|0;
    $100 = (($99) + 24|0);
    $101 = $100;
    $102 = $101;
    $103 = HEAP32[$102>>2]|0;
    $104 = (($101) + 4)|0;
    $105 = $104;
    $106 = HEAP32[$105>>2]|0;
    _invalidateIncrblobCursors($98,$103,$106,0);
   }
   $107 = HEAP32[$pPage>>2]|0;
   $108 = (($107) + 68|0);
   $109 = HEAP32[$108>>2]|0;
   $110 = (_sqlite3PagerWrite($109)|0);
   HEAP32[$rc>>2] = $110;
   $111 = HEAP32[$rc>>2]|0;
   $112 = ($111|0)!=(0);
   if ($112) {
    $113 = HEAP32[$rc>>2]|0;
    HEAP32[$0>>2] = $113;
    $239 = HEAP32[$0>>2]|0;
    STACKTOP = sp;return ($239|0);
   }
   $114 = HEAP32[$pPage>>2]|0;
   $115 = HEAP32[$pCell>>2]|0;
   $116 = (_clearCell($114,$115,$szCell)|0);
   HEAP32[$rc>>2] = $116;
   $117 = HEAP32[$pPage>>2]|0;
   $118 = HEAP32[$iCellIdx>>2]|0;
   $119 = HEAP16[$szCell>>1]|0;
   $120 = $119&65535;
   _dropCell($117,$118,$120,$rc);
   $121 = HEAP32[$rc>>2]|0;
   $122 = ($121|0)!=(0);
   if ($122) {
    $123 = HEAP32[$rc>>2]|0;
    HEAP32[$0>>2] = $123;
    $239 = HEAP32[$0>>2]|0;
    STACKTOP = sp;return ($239|0);
   }
   $124 = HEAP32[$pPage>>2]|0;
   $125 = (($124) + 5|0);
   $126 = HEAP8[$125>>0]|0;
   $127 = ($126<<24>>24)!=(0);
   if (!($127)) {
    $128 = HEAP32[$1>>2]|0;
    $129 = (($128) + 76|0);
    $130 = HEAP16[$129>>1]|0;
    $131 = $130 << 16 >> 16;
    $132 = HEAP32[$1>>2]|0;
    $133 = (($132) + 120|0);
    $134 = (($133) + ($131<<2)|0);
    $135 = HEAP32[$134>>2]|0;
    HEAP32[$pLeaf>>2] = $135;
    $136 = HEAP32[$iCellDepth>>2]|0;
    $137 = (($136) + 1)|0;
    $138 = HEAP32[$1>>2]|0;
    $139 = (($138) + 120|0);
    $140 = (($139) + ($137<<2)|0);
    $141 = HEAP32[$140>>2]|0;
    $142 = (($141) + 72|0);
    $143 = HEAP32[$142>>2]|0;
    HEAP32[$n>>2] = $143;
    $144 = HEAP32[$pLeaf>>2]|0;
    $145 = (($144) + 56|0);
    $146 = HEAP32[$145>>2]|0;
    $147 = HEAP32[$pLeaf>>2]|0;
    $148 = (($147) + 20|0);
    $149 = HEAP16[$148>>1]|0;
    $150 = $149&65535;
    $151 = HEAP32[$pLeaf>>2]|0;
    $152 = (($151) + 18|0);
    $153 = HEAP16[$152>>1]|0;
    $154 = $153&65535;
    $155 = (($154) - 1)|0;
    $156 = $155<<1;
    $157 = HEAP32[$pLeaf>>2]|0;
    $158 = (($157) + 64|0);
    $159 = HEAP32[$158>>2]|0;
    $160 = (($159) + ($156)|0);
    $161 = HEAP8[$160>>0]|0;
    $162 = $161&255;
    $163 = $162 << 8;
    $164 = HEAP32[$pLeaf>>2]|0;
    $165 = (($164) + 18|0);
    $166 = HEAP16[$165>>1]|0;
    $167 = $166&65535;
    $168 = (($167) - 1)|0;
    $169 = $168<<1;
    $170 = HEAP32[$pLeaf>>2]|0;
    $171 = (($170) + 64|0);
    $172 = HEAP32[$171>>2]|0;
    $173 = (($172) + ($169)|0);
    $174 = (($173) + 1|0);
    $175 = HEAP8[$174>>0]|0;
    $176 = $175&255;
    $177 = $163 | $176;
    $178 = $150 & $177;
    $179 = (($146) + ($178)|0);
    HEAP32[$pCell>>2] = $179;
    $180 = HEAP32[$pLeaf>>2]|0;
    $181 = HEAP32[$pCell>>2]|0;
    $182 = (_cellSizePtr($180,$181)|0);
    $183 = $182&65535;
    HEAP32[$nCell>>2] = $183;
    $184 = HEAP32[$pBt>>2]|0;
    $185 = (($184) + 80|0);
    $186 = HEAP32[$185>>2]|0;
    HEAP32[$pTmp>>2] = $186;
    $187 = HEAP32[$pLeaf>>2]|0;
    $188 = (($187) + 68|0);
    $189 = HEAP32[$188>>2]|0;
    $190 = (_sqlite3PagerWrite($189)|0);
    HEAP32[$rc>>2] = $190;
    $191 = HEAP32[$pPage>>2]|0;
    $192 = HEAP32[$iCellIdx>>2]|0;
    $193 = HEAP32[$pCell>>2]|0;
    $194 = (($193) + -4|0);
    $195 = HEAP32[$nCell>>2]|0;
    $196 = (($195) + 4)|0;
    $197 = HEAP32[$pTmp>>2]|0;
    $198 = HEAP32[$n>>2]|0;
    _insertCell($191,$192,$194,$196,$197,$198,$rc);
    $199 = HEAP32[$pLeaf>>2]|0;
    $200 = HEAP32[$pLeaf>>2]|0;
    $201 = (($200) + 18|0);
    $202 = HEAP16[$201>>1]|0;
    $203 = $202&65535;
    $204 = (($203) - 1)|0;
    $205 = HEAP32[$nCell>>2]|0;
    _dropCell($199,$204,$205,$rc);
    $206 = HEAP32[$rc>>2]|0;
    $207 = ($206|0)!=(0);
    if ($207) {
     $208 = HEAP32[$rc>>2]|0;
     HEAP32[$0>>2] = $208;
     $239 = HEAP32[$0>>2]|0;
     STACKTOP = sp;return ($239|0);
    }
   }
   $209 = HEAP32[$1>>2]|0;
   $210 = (_balance($209)|0);
   HEAP32[$rc>>2] = $210;
   $211 = HEAP32[$rc>>2]|0;
   $212 = ($211|0)==(0);
   if ($212) {
    $213 = HEAP32[$1>>2]|0;
    $214 = (($213) + 76|0);
    $215 = HEAP16[$214>>1]|0;
    $216 = $215 << 16 >> 16;
    $217 = HEAP32[$iCellDepth>>2]|0;
    $218 = ($216|0)>($217|0);
    if ($218) {
     while(1) {
      $219 = HEAP32[$1>>2]|0;
      $220 = (($219) + 76|0);
      $221 = HEAP16[$220>>1]|0;
      $222 = $221 << 16 >> 16;
      $223 = HEAP32[$iCellDepth>>2]|0;
      $224 = ($222|0)>($223|0);
      $225 = HEAP32[$1>>2]|0;
      if (!($224)) {
       break;
      }
      $226 = (($225) + 76|0);
      $227 = HEAP16[$226>>1]|0;
      $228 = (($227) + -1)<<16>>16;
      HEAP16[$226>>1] = $228;
      $229 = $227 << 16 >> 16;
      $230 = HEAP32[$1>>2]|0;
      $231 = (($230) + 120|0);
      $232 = (($231) + ($229<<2)|0);
      $233 = HEAP32[$232>>2]|0;
      _releasePage($233);
     }
     $234 = (_balance($225)|0);
     HEAP32[$rc>>2] = $234;
    }
   }
   $235 = HEAP32[$rc>>2]|0;
   $236 = ($235|0)==(0);
   if ($236) {
    $237 = HEAP32[$1>>2]|0;
    (_moveToRoot($237)|0);
   }
   $238 = HEAP32[$rc>>2]|0;
   HEAP32[$0>>2] = $238;
   $239 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($239|0);
  }
 }
 HEAP32[$0>>2] = 1;
 $239 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($239|0);
}
function _sqlite3VdbeSorterCompare($pCsr,$pVal,$nKeyCol,$pRes) {
 $pCsr = $pCsr|0;
 $pVal = $pVal|0;
 $nKeyCol = $nKeyCol|0;
 $pRes = $pRes|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $i = 0, $nKey = 0;
 var $p = 0, $pKey = 0, $pKeyInfo = 0, $pSorter = 0, $r2 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $0 = sp + 44|0;
 $1 = sp + 40|0;
 $2 = sp + 36|0;
 $3 = sp + 32|0;
 $4 = sp + 28|0;
 $pSorter = sp + 24|0;
 $r2 = sp + 20|0;
 $pKeyInfo = sp + 16|0;
 $i = sp + 12|0;
 $pKey = sp + 8|0;
 $nKey = sp + 4|0;
 $p = sp;
 HEAP32[$1>>2] = $pCsr;
 HEAP32[$2>>2] = $pVal;
 HEAP32[$3>>2] = $nKeyCol;
 HEAP32[$4>>2] = $pRes;
 $5 = HEAP32[$1>>2]|0;
 $6 = (($5) + 56|0);
 $7 = HEAP32[$6>>2]|0;
 HEAP32[$pSorter>>2] = $7;
 $8 = HEAP32[$pSorter>>2]|0;
 $9 = (($8) + 32|0);
 $10 = HEAP32[$9>>2]|0;
 HEAP32[$r2>>2] = $10;
 $11 = HEAP32[$1>>2]|0;
 $12 = (($11) + 8|0);
 $13 = HEAP32[$12>>2]|0;
 HEAP32[$pKeyInfo>>2] = $13;
 $14 = HEAP32[$r2>>2]|0;
 $15 = ($14|0)==(0|0);
 do {
  if ($15) {
   $16 = HEAP32[$pKeyInfo>>2]|0;
   $17 = (_sqlite3VdbeAllocUnpackedRecord($16,0,0,$p)|0);
   $18 = HEAP32[$pSorter>>2]|0;
   $19 = (($18) + 32|0);
   HEAP32[$19>>2] = $17;
   HEAP32[$r2>>2] = $17;
   $20 = HEAP32[$r2>>2]|0;
   $21 = ($20|0)==(0|0);
   if (!($21)) {
    $22 = HEAP32[$3>>2]|0;
    $23 = $22&65535;
    $24 = HEAP32[$r2>>2]|0;
    $25 = (($24) + 4|0);
    HEAP16[$25>>1] = $23;
    break;
   }
   HEAP32[$0>>2] = 7;
   $57 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($57|0);
  }
 } while(0);
 $26 = HEAP32[$pSorter>>2]|0;
 $27 = (_vdbeSorterRowkey($26,$nKey)|0);
 HEAP32[$pKey>>2] = $27;
 $28 = HEAP32[$pKeyInfo>>2]|0;
 $29 = HEAP32[$nKey>>2]|0;
 $30 = HEAP32[$pKey>>2]|0;
 $31 = HEAP32[$r2>>2]|0;
 _sqlite3VdbeRecordUnpack($28,$29,$30,$31);
 HEAP32[$i>>2] = 0;
 while(1) {
  $32 = HEAP32[$i>>2]|0;
  $33 = HEAP32[$3>>2]|0;
  $34 = ($32|0)<($33|0);
  if (!($34)) {
   label = 10;
   break;
  }
  $35 = HEAP32[$i>>2]|0;
  $36 = HEAP32[$r2>>2]|0;
  $37 = (($36) + 8|0);
  $38 = HEAP32[$37>>2]|0;
  $39 = (($38) + (($35*40)|0)|0);
  $40 = (($39) + 8|0);
  $41 = HEAP16[$40>>1]|0;
  $42 = $41&65535;
  $43 = $42 & 1;
  $44 = ($43|0)!=(0);
  if ($44) {
   label = 8;
   break;
  }
  $46 = HEAP32[$i>>2]|0;
  $47 = (($46) + 1)|0;
  HEAP32[$i>>2] = $47;
 }
 if ((label|0) == 8) {
  $45 = HEAP32[$4>>2]|0;
  HEAP32[$45>>2] = -1;
  HEAP32[$0>>2] = 0;
  $57 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($57|0);
 }
 else if ((label|0) == 10) {
  $48 = HEAP32[$2>>2]|0;
  $49 = (($48) + 12|0);
  $50 = HEAP32[$49>>2]|0;
  $51 = HEAP32[$2>>2]|0;
  $52 = (($51) + 16|0);
  $53 = HEAP32[$52>>2]|0;
  $54 = HEAP32[$r2>>2]|0;
  $55 = (_sqlite3VdbeRecordCompare($50,$53,$54)|0);
  $56 = HEAP32[$4>>2]|0;
  HEAP32[$56>>2] = $55;
  HEAP32[$0>>2] = 0;
  $57 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($57|0);
 }
 return 0|0;
}
function _sqlite3VdbeSorterRowkey($pCsr,$pOut) {
 $pCsr = $pCsr|0;
 $pOut = $pOut|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $nKey = 0, $pKey = 0, $pSorter = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 20|0;
 $1 = sp + 16|0;
 $2 = sp + 12|0;
 $pSorter = sp + 8|0;
 $pKey = sp + 4|0;
 $nKey = sp;
 HEAP32[$1>>2] = $pCsr;
 HEAP32[$2>>2] = $pOut;
 $3 = HEAP32[$1>>2]|0;
 $4 = (($3) + 56|0);
 $5 = HEAP32[$4>>2]|0;
 HEAP32[$pSorter>>2] = $5;
 $6 = HEAP32[$pSorter>>2]|0;
 $7 = (_vdbeSorterRowkey($6,$nKey)|0);
 HEAP32[$pKey>>2] = $7;
 $8 = HEAP32[$2>>2]|0;
 $9 = HEAP32[$nKey>>2]|0;
 $10 = (_sqlite3VdbeMemClearAndResize($8,$9)|0);
 $11 = ($10|0)!=(0);
 if ($11) {
  HEAP32[$0>>2] = 7;
  $29 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($29|0);
 } else {
  $12 = HEAP32[$nKey>>2]|0;
  $13 = HEAP32[$2>>2]|0;
  $14 = (($13) + 12|0);
  HEAP32[$14>>2] = $12;
  $15 = HEAP32[$2>>2]|0;
  $16 = (($15) + 8|0);
  $17 = HEAP16[$16>>1]|0;
  $18 = $17&65535;
  $19 = $18 & -16896;
  $20 = $19 | 16;
  $21 = $20&65535;
  $22 = HEAP32[$2>>2]|0;
  $23 = (($22) + 8|0);
  HEAP16[$23>>1] = $21;
  $24 = HEAP32[$2>>2]|0;
  $25 = (($24) + 16|0);
  $26 = HEAP32[$25>>2]|0;
  $27 = HEAP32[$pKey>>2]|0;
  $28 = HEAP32[$nKey>>2]|0;
  _memcpy(($26|0),($27|0),($28|0))|0;
  HEAP32[$0>>2] = 0;
  $29 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($29|0);
 }
 return 0|0;
}
function _sqlite3VtabImportErrmsg($p,$pVtab) {
 $p = $p|0;
 $pVtab = $pVtab|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0;
 var $db = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $1 = sp + 4|0;
 $db = sp;
 HEAP32[$0>>2] = $p;
 HEAP32[$1>>2] = $pVtab;
 $2 = HEAP32[$0>>2]|0;
 $3 = HEAP32[$2>>2]|0;
 HEAP32[$db>>2] = $3;
 $4 = HEAP32[$db>>2]|0;
 $5 = HEAP32[$0>>2]|0;
 $6 = (($5) + 44|0);
 $7 = HEAP32[$6>>2]|0;
 _sqlite3DbFree($4,$7);
 $8 = HEAP32[$db>>2]|0;
 $9 = HEAP32[$1>>2]|0;
 $10 = (($9) + 8|0);
 $11 = HEAP32[$10>>2]|0;
 $12 = (_sqlite3DbStrDup($8,$11)|0);
 $13 = HEAP32[$0>>2]|0;
 $14 = (($13) + 44|0);
 HEAP32[$14>>2] = $12;
 $15 = HEAP32[$1>>2]|0;
 $16 = (($15) + 8|0);
 $17 = HEAP32[$16>>2]|0;
 _sqlite3_free($17);
 $18 = HEAP32[$1>>2]|0;
 $19 = (($18) + 8|0);
 HEAP32[$19>>2] = 0;
 STACKTOP = sp;return;
}
function _sqlite3VdbeCursorRestore($p) {
 $p = $p|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $1 = sp;
 HEAP32[$1>>2] = $p;
 $2 = HEAP32[$1>>2]|0;
 $3 = HEAP32[$2>>2]|0;
 $4 = (_sqlite3BtreeCursorHasMoved($3)|0);
 $5 = ($4|0)!=(0);
 if ($5) {
  $6 = HEAP32[$1>>2]|0;
  $7 = (_handleMovedCursor($6)|0);
  HEAP32[$0>>2] = $7;
  $8 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($8|0);
 } else {
  HEAP32[$0>>2] = 0;
  $8 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($8|0);
 }
 return 0|0;
}
function _sqlite3VdbeSorterRewind($pCsr,$pbEof) {
 $pCsr = $pCsr|0;
 $pbEof = $pbEof|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $pSorter = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 16|0;
 $1 = sp + 12|0;
 $2 = sp + 8|0;
 $pSorter = sp + 4|0;
 $rc = sp;
 HEAP32[$1>>2] = $pCsr;
 HEAP32[$2>>2] = $pbEof;
 $3 = HEAP32[$1>>2]|0;
 $4 = (($3) + 56|0);
 $5 = HEAP32[$4>>2]|0;
 HEAP32[$pSorter>>2] = $5;
 HEAP32[$rc>>2] = 0;
 $6 = HEAP32[$pSorter>>2]|0;
 $7 = (($6) + 56|0);
 $8 = HEAP8[$7>>0]|0;
 $9 = $8&255;
 $10 = ($9|0)==(0);
 $11 = HEAP32[$pSorter>>2]|0;
 if (!($10)) {
  $22 = (_vdbeSorterFlushPMA($11)|0);
  HEAP32[$rc>>2] = $22;
  $23 = HEAP32[$rc>>2]|0;
  HEAP32[$rc>>2] = $23;
  $24 = HEAP32[$rc>>2]|0;
  $25 = ($24|0)==(0);
  if ($25) {
   $26 = HEAP32[$pSorter>>2]|0;
   $27 = (_vdbeSorterSetupMerge($26)|0);
   HEAP32[$rc>>2] = $27;
   $28 = HEAP32[$2>>2]|0;
   HEAP32[$28>>2] = 0;
  }
  $29 = HEAP32[$rc>>2]|0;
  HEAP32[$0>>2] = $29;
  $30 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($30|0);
 }
 $12 = (($11) + 36|0);
 $13 = HEAP32[$12>>2]|0;
 $14 = ($13|0)!=(0|0);
 $15 = HEAP32[$2>>2]|0;
 if ($14) {
  HEAP32[$15>>2] = 0;
  $16 = HEAP32[$pSorter>>2]|0;
  $17 = (($16) + 64|0);
  $18 = HEAP32[$pSorter>>2]|0;
  $19 = (($18) + 36|0);
  $20 = (_vdbeSorterSort($17,$19)|0);
  HEAP32[$rc>>2] = $20;
 } else {
  HEAP32[$15>>2] = 1;
 }
 $21 = HEAP32[$rc>>2]|0;
 HEAP32[$0>>2] = $21;
 $30 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($30|0);
}
function _sqlite3BtreeFirst($pCur,$pRes) {
 $pCur = $pCur|0;
 $pRes = $pRes|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $1 = sp + 4|0;
 $rc = sp;
 HEAP32[$0>>2] = $pCur;
 HEAP32[$1>>2] = $pRes;
 $2 = HEAP32[$0>>2]|0;
 $3 = (_moveToRoot($2)|0);
 HEAP32[$rc>>2] = $3;
 $4 = HEAP32[$rc>>2]|0;
 $5 = ($4|0)==(0);
 do {
  if ($5) {
   $6 = HEAP32[$0>>2]|0;
   $7 = (($6) + 73|0);
   $8 = HEAP8[$7>>0]|0;
   $9 = $8&255;
   $10 = ($9|0)==(0);
   $11 = HEAP32[$1>>2]|0;
   if ($10) {
    HEAP32[$11>>2] = 1;
    break;
   } else {
    HEAP32[$11>>2] = 0;
    $12 = HEAP32[$0>>2]|0;
    $13 = (_moveToLeftmost($12)|0);
    HEAP32[$rc>>2] = $13;
    break;
   }
  }
 } while(0);
 $14 = HEAP32[$rc>>2]|0;
 STACKTOP = sp;return ($14|0);
}
function _sqlite3VdbeSorterNext($db,$pCsr,$pbEof) {
 $db = $db|0;
 $pCsr = $pCsr|0;
 $pbEof = $pbEof|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $pFree = 0;
 var $pSorter = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 20|0;
 $1 = sp + 16|0;
 $2 = sp + 12|0;
 $pSorter = sp + 8|0;
 $rc = sp + 4|0;
 $pFree = sp;
 HEAP32[$0>>2] = $db;
 HEAP32[$1>>2] = $pCsr;
 HEAP32[$2>>2] = $pbEof;
 $3 = HEAP32[$1>>2]|0;
 $4 = (($3) + 56|0);
 $5 = HEAP32[$4>>2]|0;
 HEAP32[$pSorter>>2] = $5;
 $6 = HEAP32[$pSorter>>2]|0;
 $7 = (($6) + 56|0);
 $8 = HEAP8[$7>>0]|0;
 $9 = ($8<<24>>24)!=(0);
 $10 = HEAP32[$pSorter>>2]|0;
 if ($9) {
  $11 = (($10) + 20|0);
  $12 = HEAP32[$11>>2]|0;
  $13 = HEAP32[$2>>2]|0;
  $14 = (_vdbeMergeEngineStep($12,$13)|0);
  HEAP32[$rc>>2] = $14;
  $38 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($38|0);
 }
 $15 = (($10) + 36|0);
 $16 = HEAP32[$15>>2]|0;
 HEAP32[$pFree>>2] = $16;
 $17 = HEAP32[$pFree>>2]|0;
 $18 = (($17) + 4|0);
 $19 = HEAP32[$18>>2]|0;
 $20 = HEAP32[$pSorter>>2]|0;
 $21 = (($20) + 36|0);
 HEAP32[$21>>2] = $19;
 $22 = HEAP32[$pFree>>2]|0;
 $23 = (($22) + 4|0);
 HEAP32[$23>>2] = 0;
 $24 = HEAP32[$pSorter>>2]|0;
 $25 = (($24) + 36|0);
 $26 = (($25) + 4|0);
 $27 = HEAP32[$26>>2]|0;
 $28 = ($27|0)==(0|0);
 if ($28) {
  $29 = HEAP32[$0>>2]|0;
  $30 = HEAP32[$pFree>>2]|0;
  _vdbeSorterRecordFree($29,$30);
 }
 $31 = HEAP32[$pSorter>>2]|0;
 $32 = (($31) + 36|0);
 $33 = HEAP32[$32>>2]|0;
 $34 = ($33|0)!=(0|0);
 $35 = $34 ^ 1;
 $36 = $35&1;
 $37 = HEAP32[$2>>2]|0;
 HEAP32[$37>>2] = $36;
 HEAP32[$rc>>2] = 0;
 $38 = HEAP32[$rc>>2]|0;
 STACKTOP = sp;return ($38|0);
}
function _sqlite3VdbeSorterWrite($pCsr,$pVal) {
 $pCsr = $pCsr|0;
 $pVal = $pVal|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0;
 var $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0, $188 = 0;
 var $189 = 0, $19 = 0, $190 = 0, $191 = 0, $192 = 0, $193 = 0, $194 = 0, $195 = 0, $196 = 0, $197 = 0, $198 = 0, $199 = 0, $2 = 0, $20 = 0, $200 = 0, $201 = 0, $202 = 0, $203 = 0, $204 = 0, $205 = 0;
 var $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0;
 var $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0;
 var $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0;
 var $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0;
 var $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $aNew = 0, $bFlush = 0, $nMin = 0, $nNew = 0, $nPMA = 0, $nReq = 0, $pNew = 0, $pSorter = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $0 = sp + 44|0;
 $1 = sp + 40|0;
 $2 = sp + 36|0;
 $pSorter = sp + 32|0;
 $rc = sp + 28|0;
 $pNew = sp + 24|0;
 $bFlush = sp + 20|0;
 $nReq = sp + 16|0;
 $nPMA = sp + 12|0;
 $nMin = sp + 8|0;
 $aNew = sp + 4|0;
 $nNew = sp;
 HEAP32[$1>>2] = $pCsr;
 HEAP32[$2>>2] = $pVal;
 $3 = HEAP32[$1>>2]|0;
 $4 = (($3) + 56|0);
 $5 = HEAP32[$4>>2]|0;
 HEAP32[$pSorter>>2] = $5;
 HEAP32[$rc>>2] = 0;
 $6 = HEAP32[$2>>2]|0;
 $7 = (($6) + 12|0);
 $8 = HEAP32[$7>>2]|0;
 $9 = (($8) + 8)|0;
 HEAP32[$nReq>>2] = $9;
 $10 = HEAP32[$2>>2]|0;
 $11 = (($10) + 12|0);
 $12 = HEAP32[$11>>2]|0;
 $13 = HEAP32[$2>>2]|0;
 $14 = (($13) + 12|0);
 $15 = HEAP32[$14>>2]|0;
 $16 = ($15|0)<(0);
 $17 = $16 << 31 >> 31;
 $18 = (_sqlite3VarintLen($15,$17)|0);
 $19 = (($12) + ($18))|0;
 HEAP32[$nPMA>>2] = $19;
 $20 = HEAP32[$pSorter>>2]|0;
 $21 = (($20) + 4|0);
 $22 = HEAP32[$21>>2]|0;
 $23 = ($22|0)!=(0);
 if ($23) {
  $24 = HEAP32[$pSorter>>2]|0;
  $25 = (($24) + 36|0);
  $26 = (($25) + 4|0);
  $27 = HEAP32[$26>>2]|0;
  $28 = ($27|0)!=(0|0);
  $29 = HEAP32[$pSorter>>2]|0;
  if ($28) {
   $30 = (($29) + 48|0);
   $31 = HEAP32[$30>>2]|0;
   $32 = ($31|0)!=(0);
   if ($32) {
    $33 = HEAP32[$pSorter>>2]|0;
    $34 = (($33) + 48|0);
    $35 = HEAP32[$34>>2]|0;
    $36 = HEAP32[$nReq>>2]|0;
    $37 = (($35) + ($36))|0;
    $38 = HEAP32[$pSorter>>2]|0;
    $39 = (($38) + 4|0);
    $40 = HEAP32[$39>>2]|0;
    $41 = ($37|0)>($40|0);
    $43 = $41;
   } else {
    $43 = 0;
   }
   $42 = $43&1;
   HEAP32[$bFlush>>2] = $42;
  } else {
   $44 = (($29) + 36|0);
   $45 = (($44) + 8|0);
   $46 = HEAP32[$45>>2]|0;
   $47 = HEAP32[$pSorter>>2]|0;
   $48 = (($47) + 4|0);
   $49 = HEAP32[$48>>2]|0;
   $50 = ($46|0)>($49|0);
   if ($50) {
    $61 = 1;
   } else {
    $51 = HEAP32[$pSorter>>2]|0;
    $52 = (($51) + 36|0);
    $53 = (($52) + 8|0);
    $54 = HEAP32[$53>>2]|0;
    $55 = HEAP32[$pSorter>>2]|0;
    $56 = HEAP32[$55>>2]|0;
    $57 = ($54|0)>($56|0);
    if ($57) {
     $58 = (_sqlite3HeapNearlyFull()|0);
     $59 = ($58|0)!=(0);
     $61 = $59;
    } else {
     $61 = 0;
    }
   }
   $60 = $61&1;
   HEAP32[$bFlush>>2] = $60;
  }
  $62 = HEAP32[$bFlush>>2]|0;
  $63 = ($62|0)!=(0);
  if ($63) {
   $64 = HEAP32[$pSorter>>2]|0;
   $65 = (_vdbeSorterFlushPMA($64)|0);
   HEAP32[$rc>>2] = $65;
   $66 = HEAP32[$pSorter>>2]|0;
   $67 = (($66) + 36|0);
   $68 = (($67) + 8|0);
   HEAP32[$68>>2] = 0;
   $69 = HEAP32[$pSorter>>2]|0;
   $70 = (($69) + 48|0);
   HEAP32[$70>>2] = 0;
  }
 }
 $71 = HEAP32[$nPMA>>2]|0;
 $72 = HEAP32[$pSorter>>2]|0;
 $73 = (($72) + 36|0);
 $74 = (($73) + 8|0);
 $75 = HEAP32[$74>>2]|0;
 $76 = (($75) + ($71))|0;
 HEAP32[$74>>2] = $76;
 $77 = HEAP32[$nPMA>>2]|0;
 $78 = HEAP32[$pSorter>>2]|0;
 $79 = (($78) + 8|0);
 $80 = HEAP32[$79>>2]|0;
 $81 = ($77|0)>($80|0);
 if ($81) {
  $82 = HEAP32[$nPMA>>2]|0;
  $83 = HEAP32[$pSorter>>2]|0;
  $84 = (($83) + 8|0);
  HEAP32[$84>>2] = $82;
 }
 $85 = HEAP32[$pSorter>>2]|0;
 $86 = (($85) + 36|0);
 $87 = (($86) + 4|0);
 $88 = HEAP32[$87>>2]|0;
 $89 = ($88|0)!=(0|0);
 do {
  if ($89) {
   $90 = HEAP32[$pSorter>>2]|0;
   $91 = (($90) + 48|0);
   $92 = HEAP32[$91>>2]|0;
   $93 = HEAP32[$nReq>>2]|0;
   $94 = (($92) + ($93))|0;
   HEAP32[$nMin>>2] = $94;
   $95 = HEAP32[$nMin>>2]|0;
   $96 = HEAP32[$pSorter>>2]|0;
   $97 = (($96) + 52|0);
   $98 = HEAP32[$97>>2]|0;
   $99 = ($95|0)>($98|0);
   do {
    if ($99) {
     $100 = HEAP32[$pSorter>>2]|0;
     $101 = (($100) + 52|0);
     $102 = HEAP32[$101>>2]|0;
     $103 = $102<<1;
     HEAP32[$nNew>>2] = $103;
     while(1) {
      $104 = HEAP32[$nNew>>2]|0;
      $105 = HEAP32[$nMin>>2]|0;
      $106 = ($104|0)<($105|0);
      $107 = HEAP32[$nNew>>2]|0;
      if (!($106)) {
       break;
      }
      $108 = $107<<1;
      HEAP32[$nNew>>2] = $108;
     }
     $109 = HEAP32[$pSorter>>2]|0;
     $110 = (($109) + 4|0);
     $111 = HEAP32[$110>>2]|0;
     $112 = ($107|0)>($111|0);
     if ($112) {
      $113 = HEAP32[$pSorter>>2]|0;
      $114 = (($113) + 4|0);
      $115 = HEAP32[$114>>2]|0;
      HEAP32[$nNew>>2] = $115;
     }
     $116 = HEAP32[$nNew>>2]|0;
     $117 = HEAP32[$nMin>>2]|0;
     $118 = ($116|0)<($117|0);
     if ($118) {
      $119 = HEAP32[$nMin>>2]|0;
      HEAP32[$nNew>>2] = $119;
     }
     $120 = HEAP32[$pSorter>>2]|0;
     $121 = (($120) + 36|0);
     $122 = (($121) + 4|0);
     $123 = HEAP32[$122>>2]|0;
     $124 = HEAP32[$nNew>>2]|0;
     $125 = ($124|0)<(0);
     $126 = $125 << 31 >> 31;
     $127 = (_sqlite3Realloc($123,$124,$126)|0);
     HEAP32[$aNew>>2] = $127;
     $128 = HEAP32[$aNew>>2]|0;
     $129 = ($128|0)!=(0|0);
     if ($129) {
      $130 = HEAP32[$aNew>>2]|0;
      $131 = HEAP32[$pSorter>>2]|0;
      $132 = (($131) + 36|0);
      $133 = HEAP32[$132>>2]|0;
      $134 = HEAP32[$pSorter>>2]|0;
      $135 = (($134) + 36|0);
      $136 = (($135) + 4|0);
      $137 = HEAP32[$136>>2]|0;
      $138 = $133;
      $139 = $137;
      $140 = (($138) - ($139))|0;
      $141 = (($130) + ($140)|0);
      $142 = HEAP32[$pSorter>>2]|0;
      $143 = (($142) + 36|0);
      HEAP32[$143>>2] = $141;
      $144 = HEAP32[$aNew>>2]|0;
      $145 = HEAP32[$pSorter>>2]|0;
      $146 = (($145) + 36|0);
      $147 = (($146) + 4|0);
      HEAP32[$147>>2] = $144;
      $148 = HEAP32[$nNew>>2]|0;
      $149 = HEAP32[$pSorter>>2]|0;
      $150 = (($149) + 52|0);
      HEAP32[$150>>2] = $148;
      break;
     }
     HEAP32[$0>>2] = 7;
     $205 = HEAP32[$0>>2]|0;
     STACKTOP = sp;return ($205|0);
    }
   } while(0);
   $151 = HEAP32[$pSorter>>2]|0;
   $152 = (($151) + 48|0);
   $153 = HEAP32[$152>>2]|0;
   $154 = HEAP32[$pSorter>>2]|0;
   $155 = (($154) + 36|0);
   $156 = (($155) + 4|0);
   $157 = HEAP32[$156>>2]|0;
   $158 = (($157) + ($153)|0);
   HEAP32[$pNew>>2] = $158;
   $159 = HEAP32[$nReq>>2]|0;
   $160 = (($159) + 7)|0;
   $161 = $160 & -8;
   $162 = HEAP32[$pSorter>>2]|0;
   $163 = (($162) + 48|0);
   $164 = HEAP32[$163>>2]|0;
   $165 = (($164) + ($161))|0;
   HEAP32[$163>>2] = $165;
   $166 = HEAP32[$pSorter>>2]|0;
   $167 = (($166) + 36|0);
   $168 = HEAP32[$167>>2]|0;
   $169 = HEAP32[$pSorter>>2]|0;
   $170 = (($169) + 36|0);
   $171 = (($170) + 4|0);
   $172 = HEAP32[$171>>2]|0;
   $173 = $168;
   $174 = $172;
   $175 = (($173) - ($174))|0;
   $176 = HEAP32[$pNew>>2]|0;
   $177 = (($176) + 4|0);
   HEAP32[$177>>2] = $175;
  } else {
   $178 = HEAP32[$nReq>>2]|0;
   $179 = ($178|0)<(0);
   $180 = $179 << 31 >> 31;
   $181 = (_sqlite3Malloc($178,$180)|0);
   HEAP32[$pNew>>2] = $181;
   $182 = HEAP32[$pNew>>2]|0;
   $183 = ($182|0)==(0|0);
   if (!($183)) {
    $184 = HEAP32[$pSorter>>2]|0;
    $185 = (($184) + 36|0);
    $186 = HEAP32[$185>>2]|0;
    $187 = HEAP32[$pNew>>2]|0;
    $188 = (($187) + 4|0);
    HEAP32[$188>>2] = $186;
    break;
   }
   HEAP32[$0>>2] = 7;
   $205 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($205|0);
  }
 } while(0);
 $189 = HEAP32[$pNew>>2]|0;
 $190 = (($189) + 8|0);
 $191 = HEAP32[$2>>2]|0;
 $192 = (($191) + 16|0);
 $193 = HEAP32[$192>>2]|0;
 $194 = HEAP32[$2>>2]|0;
 $195 = (($194) + 12|0);
 $196 = HEAP32[$195>>2]|0;
 _memcpy(($190|0),($193|0),($196|0))|0;
 $197 = HEAP32[$2>>2]|0;
 $198 = (($197) + 12|0);
 $199 = HEAP32[$198>>2]|0;
 $200 = HEAP32[$pNew>>2]|0;
 HEAP32[$200>>2] = $199;
 $201 = HEAP32[$pNew>>2]|0;
 $202 = HEAP32[$pSorter>>2]|0;
 $203 = (($202) + 36|0);
 HEAP32[$203>>2] = $201;
 $204 = HEAP32[$rc>>2]|0;
 HEAP32[$0>>2] = $204;
 $205 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($205|0);
}
function _sqlite3VdbeIdxRowid($db,$pCur,$rowid) {
 $db = $db|0;
 $pCur = $pCur|0;
 $rowid = $rowid|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0;
 var $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $9 = 0, $lenRowid = 0, $m = 0, $nCellKey = 0, $or$cond = 0, $or$cond3 = 0, $rc = 0, $szHdr = 0, $typeRowid = 0, $v = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 128|0;
 $0 = sp + 116|0;
 $1 = sp + 112|0;
 $2 = sp + 108|0;
 $3 = sp + 104|0;
 $nCellKey = sp + 80|0;
 $rc = sp + 100|0;
 $szHdr = sp + 96|0;
 $typeRowid = sp + 92|0;
 $lenRowid = sp + 88|0;
 $m = sp + 40|0;
 $v = sp;
 HEAP32[$1>>2] = $db;
 HEAP32[$2>>2] = $pCur;
 HEAP32[$3>>2] = $rowid;
 $4 = $nCellKey;
 $5 = $4;
 HEAP32[$5>>2] = 0;
 $6 = (($4) + 4)|0;
 $7 = $6;
 HEAP32[$7>>2] = 0;
 $8 = HEAP32[$2>>2]|0;
 (_sqlite3BtreeKeySize($8,$nCellKey)|0);
 $9 = HEAP32[$1>>2]|0;
 _sqlite3VdbeMemInit($m,$9,0);
 $10 = HEAP32[$2>>2]|0;
 $11 = $nCellKey;
 $12 = $11;
 $13 = HEAP32[$12>>2]|0;
 $14 = (($11) + 4)|0;
 $15 = $14;
 $16 = HEAP32[$15>>2]|0;
 $17 = (_sqlite3VdbeMemFromBtree($10,0,$13,1,$m)|0);
 HEAP32[$rc>>2] = $17;
 $18 = HEAP32[$rc>>2]|0;
 $19 = ($18|0)!=(0);
 if ($19) {
  $20 = HEAP32[$rc>>2]|0;
  HEAP32[$0>>2] = $20;
  $85 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($85|0);
 }
 $21 = (($m) + 16|0);
 $22 = HEAP32[$21>>2]|0;
 $23 = HEAP8[$22>>0]|0;
 $24 = $23&255;
 $25 = ($24|0)<(128);
 $26 = (($m) + 16|0);
 $27 = HEAP32[$26>>2]|0;
 if ($25) {
  $28 = HEAP8[$27>>0]|0;
  $29 = $28&255;
  HEAP32[$szHdr>>2] = $29;
 } else {
  (_sqlite3GetVarint32($27,$szHdr)|0);
 }
 $30 = HEAP32[$szHdr>>2]|0;
 $31 = ($30>>>0)<(3);
 if (!($31)) {
  $32 = HEAP32[$szHdr>>2]|0;
  $33 = (($m) + 12|0);
  $34 = HEAP32[$33>>2]|0;
  $35 = ($32|0)>($34|0);
  if (!($35)) {
   $36 = HEAP32[$szHdr>>2]|0;
   $37 = (($36) - 1)|0;
   $38 = (($m) + 16|0);
   $39 = HEAP32[$38>>2]|0;
   $40 = (($39) + ($37)|0);
   $41 = HEAP8[$40>>0]|0;
   $42 = $41&255;
   $43 = ($42|0)<(128);
   $44 = HEAP32[$szHdr>>2]|0;
   $45 = (($44) - 1)|0;
   $46 = (($m) + 16|0);
   $47 = HEAP32[$46>>2]|0;
   $48 = (($47) + ($45)|0);
   if ($43) {
    $49 = HEAP8[$48>>0]|0;
    $50 = $49&255;
    HEAP32[$typeRowid>>2] = $50;
   } else {
    (_sqlite3GetVarint32($48,$typeRowid)|0);
   }
   $51 = HEAP32[$typeRowid>>2]|0;
   $52 = ($51>>>0)<(1);
   $53 = HEAP32[$typeRowid>>2]|0;
   $54 = ($53>>>0)>(9);
   $or$cond = $52 | $54;
   $55 = HEAP32[$typeRowid>>2]|0;
   $56 = ($55|0)==(7);
   $or$cond3 = $or$cond | $56;
   if (!($or$cond3)) {
    $57 = HEAP32[$typeRowid>>2]|0;
    $58 = (_sqlite3VdbeSerialTypeLen($57)|0);
    HEAP32[$lenRowid>>2] = $58;
    $59 = (($m) + 12|0);
    $60 = HEAP32[$59>>2]|0;
    $61 = HEAP32[$szHdr>>2]|0;
    $62 = HEAP32[$lenRowid>>2]|0;
    $63 = (($61) + ($62))|0;
    $64 = ($60>>>0)<($63>>>0);
    if (!($64)) {
     $65 = (($m) + 12|0);
     $66 = HEAP32[$65>>2]|0;
     $67 = HEAP32[$lenRowid>>2]|0;
     $68 = (($66) - ($67))|0;
     $69 = (($m) + 16|0);
     $70 = HEAP32[$69>>2]|0;
     $71 = (($70) + ($68)|0);
     $72 = HEAP32[$typeRowid>>2]|0;
     (_sqlite3VdbeSerialGet($71,$72,$v)|0);
     $73 = $v;
     $74 = $73;
     $75 = HEAP32[$74>>2]|0;
     $76 = (($73) + 4)|0;
     $77 = $76;
     $78 = HEAP32[$77>>2]|0;
     $79 = HEAP32[$3>>2]|0;
     $80 = $79;
     $81 = $80;
     HEAP32[$81>>2] = $75;
     $82 = (($80) + 4)|0;
     $83 = $82;
     HEAP32[$83>>2] = $78;
     _sqlite3VdbeMemRelease($m);
     HEAP32[$0>>2] = 0;
     $85 = HEAP32[$0>>2]|0;
     STACKTOP = sp;return ($85|0);
    }
   }
  }
 }
 _sqlite3VdbeMemRelease($m);
 $84 = (_sqlite3CorruptError(67174)|0);
 HEAP32[$0>>2] = $84;
 $85 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($85|0);
}
function _sqlite3VdbeIdxKeyCompare($db,$pC,$pUnpacked,$res) {
 $db = $db|0;
 $pC = $pC|0;
 $pUnpacked = $pUnpacked|0;
 $res = $res|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $m = 0, $nCellKey = 0, $or$cond = 0;
 var $pCur = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 80|0;
 $0 = sp + 72|0;
 $1 = sp + 68|0;
 $2 = sp + 64|0;
 $3 = sp + 60|0;
 $4 = sp + 56|0;
 $nCellKey = sp + 40|0;
 $rc = sp + 52|0;
 $pCur = sp + 48|0;
 $m = sp;
 HEAP32[$1>>2] = $db;
 HEAP32[$2>>2] = $pC;
 HEAP32[$3>>2] = $pUnpacked;
 HEAP32[$4>>2] = $res;
 $5 = $nCellKey;
 $6 = $5;
 HEAP32[$6>>2] = 0;
 $7 = (($5) + 4)|0;
 $8 = $7;
 HEAP32[$8>>2] = 0;
 $9 = HEAP32[$2>>2]|0;
 $10 = HEAP32[$9>>2]|0;
 HEAP32[$pCur>>2] = $10;
 $11 = HEAP32[$pCur>>2]|0;
 (_sqlite3BtreeKeySize($11,$nCellKey)|0);
 $12 = $nCellKey;
 $13 = $12;
 $14 = HEAP32[$13>>2]|0;
 $15 = (($12) + 4)|0;
 $16 = $15;
 $17 = HEAP32[$16>>2]|0;
 $18 = ($17|0)<(0);
 $19 = ($14>>>0)<=(0);
 $20 = ($17|0)==(0);
 $21 = $20 & $19;
 $22 = $18 | $21;
 $23 = $nCellKey;
 $24 = $23;
 $25 = HEAP32[$24>>2]|0;
 $26 = (($23) + 4)|0;
 $27 = $26;
 $28 = HEAP32[$27>>2]|0;
 $29 = ($28|0)>(0);
 $30 = ($25>>>0)>(2147483647);
 $31 = ($28|0)==(0);
 $32 = $31 & $30;
 $33 = $29 | $32;
 $or$cond = $22 | $33;
 if ($or$cond) {
  $34 = HEAP32[$4>>2]|0;
  HEAP32[$34>>2] = 0;
  $35 = (_sqlite3CorruptError(67206)|0);
  HEAP32[$0>>2] = $35;
  $56 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($56|0);
 }
 $36 = HEAP32[$1>>2]|0;
 _sqlite3VdbeMemInit($m,$36,0);
 $37 = HEAP32[$2>>2]|0;
 $38 = HEAP32[$37>>2]|0;
 $39 = $nCellKey;
 $40 = $39;
 $41 = HEAP32[$40>>2]|0;
 $42 = (($39) + 4)|0;
 $43 = $42;
 $44 = HEAP32[$43>>2]|0;
 $45 = (_sqlite3VdbeMemFromBtree($38,0,$41,1,$m)|0);
 HEAP32[$rc>>2] = $45;
 $46 = HEAP32[$rc>>2]|0;
 $47 = ($46|0)!=(0);
 if ($47) {
  $48 = HEAP32[$rc>>2]|0;
  HEAP32[$0>>2] = $48;
  $56 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($56|0);
 } else {
  $49 = (($m) + 12|0);
  $50 = HEAP32[$49>>2]|0;
  $51 = (($m) + 16|0);
  $52 = HEAP32[$51>>2]|0;
  $53 = HEAP32[$3>>2]|0;
  $54 = (_sqlite3VdbeRecordCompare($50,$52,$53)|0);
  $55 = HEAP32[$4>>2]|0;
  HEAP32[$55>>2] = $54;
  _sqlite3VdbeMemRelease($m);
  HEAP32[$0>>2] = 0;
  $56 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($56|0);
 }
 return 0|0;
}
function _sqlite3BtreeDropTable($p,$iTable,$piMoved) {
 $p = $p|0;
 $iTable = $iTable|0;
 $piMoved = $piMoved|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $1 = sp + 8|0;
 $2 = sp + 4|0;
 $rc = sp;
 HEAP32[$0>>2] = $p;
 HEAP32[$1>>2] = $iTable;
 HEAP32[$2>>2] = $piMoved;
 $3 = HEAP32[$0>>2]|0;
 _sqlite3BtreeEnter($3);
 $4 = HEAP32[$0>>2]|0;
 $5 = HEAP32[$1>>2]|0;
 $6 = HEAP32[$2>>2]|0;
 $7 = (_btreeDropTable($4,$5,$6)|0);
 HEAP32[$rc>>2] = $7;
 $8 = HEAP32[$rc>>2]|0;
 STACKTOP = sp;return ($8|0);
}
function _sqlite3RootPageMoved($db,$iDb,$iFrom,$iTo) {
 $db = $db|0;
 $iDb = $iDb|0;
 $iFrom = $iFrom|0;
 $iTo = $iTo|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $pDb = 0, $pElem = 0, $pHash = 0, $pIdx = 0, $pTab = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $0 = sp + 32|0;
 $1 = sp + 28|0;
 $2 = sp + 24|0;
 $3 = sp + 20|0;
 $pElem = sp + 16|0;
 $pHash = sp + 12|0;
 $pDb = sp + 8|0;
 $pTab = sp + 4|0;
 $pIdx = sp;
 HEAP32[$0>>2] = $db;
 HEAP32[$1>>2] = $iDb;
 HEAP32[$2>>2] = $iFrom;
 HEAP32[$3>>2] = $iTo;
 $4 = HEAP32[$1>>2]|0;
 $5 = HEAP32[$0>>2]|0;
 $6 = (($5) + 16|0);
 $7 = HEAP32[$6>>2]|0;
 $8 = (($7) + ($4<<4)|0);
 HEAP32[$pDb>>2] = $8;
 $9 = HEAP32[$pDb>>2]|0;
 $10 = (($9) + 12|0);
 $11 = HEAP32[$10>>2]|0;
 $12 = (($11) + 8|0);
 HEAP32[$pHash>>2] = $12;
 $13 = HEAP32[$pHash>>2]|0;
 $14 = (($13) + 8|0);
 $15 = HEAP32[$14>>2]|0;
 HEAP32[$pElem>>2] = $15;
 while(1) {
  $16 = HEAP32[$pElem>>2]|0;
  $17 = ($16|0)!=(0|0);
  if (!($17)) {
   break;
  }
  $18 = HEAP32[$pElem>>2]|0;
  $19 = (($18) + 8|0);
  $20 = HEAP32[$19>>2]|0;
  HEAP32[$pTab>>2] = $20;
  $21 = HEAP32[$pTab>>2]|0;
  $22 = (($21) + 32|0);
  $23 = HEAP32[$22>>2]|0;
  $24 = HEAP32[$2>>2]|0;
  $25 = ($23|0)==($24|0);
  if ($25) {
   $26 = HEAP32[$3>>2]|0;
   $27 = HEAP32[$pTab>>2]|0;
   $28 = (($27) + 32|0);
   HEAP32[$28>>2] = $26;
  }
  $29 = HEAP32[$pElem>>2]|0;
  $30 = HEAP32[$29>>2]|0;
  HEAP32[$pElem>>2] = $30;
 }
 $31 = HEAP32[$pDb>>2]|0;
 $32 = (($31) + 12|0);
 $33 = HEAP32[$32>>2]|0;
 $34 = (($33) + 24|0);
 HEAP32[$pHash>>2] = $34;
 $35 = HEAP32[$pHash>>2]|0;
 $36 = (($35) + 8|0);
 $37 = HEAP32[$36>>2]|0;
 HEAP32[$pElem>>2] = $37;
 while(1) {
  $38 = HEAP32[$pElem>>2]|0;
  $39 = ($38|0)!=(0|0);
  if (!($39)) {
   break;
  }
  $40 = HEAP32[$pElem>>2]|0;
  $41 = (($40) + 8|0);
  $42 = HEAP32[$41>>2]|0;
  HEAP32[$pIdx>>2] = $42;
  $43 = HEAP32[$pIdx>>2]|0;
  $44 = (($43) + 44|0);
  $45 = HEAP32[$44>>2]|0;
  $46 = HEAP32[$2>>2]|0;
  $47 = ($45|0)==($46|0);
  if ($47) {
   $48 = HEAP32[$3>>2]|0;
   $49 = HEAP32[$pIdx>>2]|0;
   $50 = (($49) + 44|0);
   HEAP32[$50>>2] = $48;
  }
  $51 = HEAP32[$pElem>>2]|0;
  $52 = HEAP32[$51>>2]|0;
  HEAP32[$pElem>>2] = $52;
 }
 STACKTOP = sp;return;
}
function _sqlite3BtreeClearTable($p,$iTable,$pnChange) {
 $p = $p|0;
 $iTable = $iTable|0;
 $pnChange = $pnChange|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $pBt = 0, $rc = 0;
 var label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 16|0;
 $1 = sp + 12|0;
 $2 = sp + 8|0;
 $rc = sp + 4|0;
 $pBt = sp;
 HEAP32[$0>>2] = $p;
 HEAP32[$1>>2] = $iTable;
 HEAP32[$2>>2] = $pnChange;
 $3 = HEAP32[$0>>2]|0;
 $4 = (($3) + 4|0);
 $5 = HEAP32[$4>>2]|0;
 HEAP32[$pBt>>2] = $5;
 $6 = HEAP32[$0>>2]|0;
 _sqlite3BtreeEnter($6);
 $7 = HEAP32[$pBt>>2]|0;
 $8 = HEAP32[$1>>2]|0;
 $9 = (_saveAllCursors($7,$8,0)|0);
 HEAP32[$rc>>2] = $9;
 $10 = HEAP32[$rc>>2]|0;
 $11 = (0)==($10|0);
 if (!($11)) {
  $17 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($17|0);
 }
 $12 = HEAP32[$0>>2]|0;
 _invalidateIncrblobCursors($12,0,0,1);
 $13 = HEAP32[$pBt>>2]|0;
 $14 = HEAP32[$1>>2]|0;
 $15 = HEAP32[$2>>2]|0;
 $16 = (_clearDatabasePage($13,$14,0,$15)|0);
 HEAP32[$rc>>2] = $16;
 $17 = HEAP32[$rc>>2]|0;
 STACKTOP = sp;return ($17|0);
}
function _sqlite3VdbeSorterReset($db,$pSorter) {
 $db = $db|0;
 $pSorter = $pSorter|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $i = 0, $pTask = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $1 = sp + 8|0;
 $i = sp + 4|0;
 $pTask = sp;
 HEAP32[$0>>2] = $db;
 HEAP32[$1>>2] = $pSorter;
 $2 = HEAP32[$1>>2]|0;
 $3 = (($2) + 20|0);
 $4 = HEAP32[$3>>2]|0;
 _vdbeMergeEngineFree($4);
 $5 = HEAP32[$1>>2]|0;
 $6 = (($5) + 20|0);
 HEAP32[$6>>2] = 0;
 HEAP32[$i>>2] = 0;
 while(1) {
  $7 = HEAP32[$i>>2]|0;
  $8 = HEAP32[$1>>2]|0;
  $9 = (($8) + 59|0);
  $10 = HEAP8[$9>>0]|0;
  $11 = $10&255;
  $12 = ($7|0)<($11|0);
  if (!($12)) {
   break;
  }
  $13 = HEAP32[$i>>2]|0;
  $14 = HEAP32[$1>>2]|0;
  $15 = (($14) + 64|0);
  $16 = (($15) + ($13<<6)|0);
  HEAP32[$pTask>>2] = $16;
  $17 = HEAP32[$0>>2]|0;
  $18 = HEAP32[$pTask>>2]|0;
  _vdbeSortSubtaskCleanup($17,$18);
  $19 = HEAP32[$i>>2]|0;
  $20 = (($19) + 1)|0;
  HEAP32[$i>>2] = $20;
 }
 $21 = HEAP32[$1>>2]|0;
 $22 = (($21) + 36|0);
 $23 = (($22) + 4|0);
 $24 = HEAP32[$23>>2]|0;
 $25 = ($24|0)==(0|0);
 if ($25) {
  $26 = HEAP32[$1>>2]|0;
  $27 = (($26) + 36|0);
  $28 = HEAP32[$27>>2]|0;
  _vdbeSorterRecordFree(0,$28);
 }
 $29 = HEAP32[$1>>2]|0;
 $30 = (($29) + 36|0);
 HEAP32[$30>>2] = 0;
 $31 = HEAP32[$1>>2]|0;
 $32 = (($31) + 36|0);
 $33 = (($32) + 8|0);
 HEAP32[$33>>2] = 0;
 $34 = HEAP32[$1>>2]|0;
 $35 = (($34) + 56|0);
 HEAP8[$35>>0] = 0;
 $36 = HEAP32[$1>>2]|0;
 $37 = (($36) + 48|0);
 HEAP32[$37>>2] = 0;
 $38 = HEAP32[$1>>2]|0;
 $39 = (($38) + 8|0);
 HEAP32[$39>>2] = 0;
 $40 = HEAP32[$0>>2]|0;
 $41 = HEAP32[$1>>2]|0;
 $42 = (($41) + 32|0);
 $43 = HEAP32[$42>>2]|0;
 _sqlite3DbFree($40,$43);
 $44 = HEAP32[$1>>2]|0;
 $45 = (($44) + 32|0);
 HEAP32[$45>>2] = 0;
 STACKTOP = sp;return;
}
function _sqlite3BtreeClearTableOfCursor($pCur) {
 $pCur = $pCur|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 HEAP32[$0>>2] = $pCur;
 $1 = HEAP32[$0>>2]|0;
 $2 = HEAP32[$1>>2]|0;
 $3 = HEAP32[$0>>2]|0;
 $4 = (($3) + 60|0);
 $5 = HEAP32[$4>>2]|0;
 $6 = (_sqlite3BtreeClearTable($2,$5,0)|0);
 STACKTOP = sp;return ($6|0);
}
function _sqlite3UnlinkAndDeleteTable($db,$iDb,$zTabName) {
 $db = $db|0;
 $iDb = $iDb|0;
 $zTabName = $zTabName|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0;
 var $p = 0, $pDb = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 16|0;
 $1 = sp + 12|0;
 $2 = sp + 8|0;
 $p = sp + 4|0;
 $pDb = sp;
 HEAP32[$0>>2] = $db;
 HEAP32[$1>>2] = $iDb;
 HEAP32[$2>>2] = $zTabName;
 $3 = HEAP32[$1>>2]|0;
 $4 = HEAP32[$0>>2]|0;
 $5 = (($4) + 16|0);
 $6 = HEAP32[$5>>2]|0;
 $7 = (($6) + ($3<<4)|0);
 HEAP32[$pDb>>2] = $7;
 $8 = HEAP32[$pDb>>2]|0;
 $9 = (($8) + 12|0);
 $10 = HEAP32[$9>>2]|0;
 $11 = (($10) + 8|0);
 $12 = HEAP32[$2>>2]|0;
 $13 = (_sqlite3HashInsert($11,$12,0)|0);
 HEAP32[$p>>2] = $13;
 $14 = HEAP32[$0>>2]|0;
 $15 = HEAP32[$p>>2]|0;
 _sqlite3DeleteTable($14,$15);
 $16 = HEAP32[$0>>2]|0;
 $17 = (($16) + 24|0);
 $18 = HEAP32[$17>>2]|0;
 $19 = $18 | 2;
 HEAP32[$17>>2] = $19;
 STACKTOP = sp;return;
}
function _sqlite3UnlinkAndDeleteIndex($db,$iDb,$zIdxName) {
 $db = $db|0;
 $iDb = $iDb|0;
 $zIdxName = $zIdxName|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $7 = 0;
 var $8 = 0, $9 = 0, $p = 0, $pHash = 0, $pIndex = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 20|0;
 $1 = sp + 16|0;
 $2 = sp + 12|0;
 $pIndex = sp + 8|0;
 $pHash = sp + 4|0;
 $p = sp;
 HEAP32[$0>>2] = $db;
 HEAP32[$1>>2] = $iDb;
 HEAP32[$2>>2] = $zIdxName;
 $3 = HEAP32[$1>>2]|0;
 $4 = HEAP32[$0>>2]|0;
 $5 = (($4) + 16|0);
 $6 = HEAP32[$5>>2]|0;
 $7 = (($6) + ($3<<4)|0);
 $8 = (($7) + 12|0);
 $9 = HEAP32[$8>>2]|0;
 $10 = (($9) + 24|0);
 HEAP32[$pHash>>2] = $10;
 $11 = HEAP32[$pHash>>2]|0;
 $12 = HEAP32[$2>>2]|0;
 $13 = (_sqlite3HashInsert($11,$12,0)|0);
 HEAP32[$pIndex>>2] = $13;
 $14 = HEAP32[$pIndex>>2]|0;
 $15 = ($14|0)!=(0|0);
 if (!($15)) {
  $57 = HEAP32[$0>>2]|0;
  $58 = (($57) + 24|0);
  $59 = HEAP32[$58>>2]|0;
  $60 = $59 | 2;
  HEAP32[$58>>2] = $60;
  STACKTOP = sp;return;
 }
 $16 = HEAP32[$pIndex>>2]|0;
 $17 = (($16) + 12|0);
 $18 = HEAP32[$17>>2]|0;
 $19 = (($18) + 8|0);
 $20 = HEAP32[$19>>2]|0;
 $21 = HEAP32[$pIndex>>2]|0;
 $22 = ($20|0)==($21|0);
 $23 = HEAP32[$pIndex>>2]|0;
 if ($22) {
  $24 = (($23) + 20|0);
  $25 = HEAP32[$24>>2]|0;
  $26 = HEAP32[$pIndex>>2]|0;
  $27 = (($26) + 12|0);
  $28 = HEAP32[$27>>2]|0;
  $29 = (($28) + 8|0);
  HEAP32[$29>>2] = $25;
 } else {
  $30 = (($23) + 12|0);
  $31 = HEAP32[$30>>2]|0;
  $32 = (($31) + 8|0);
  $33 = HEAP32[$32>>2]|0;
  HEAP32[$p>>2] = $33;
  while(1) {
   $34 = HEAP32[$p>>2]|0;
   $35 = ($34|0)!=(0|0);
   if ($35) {
    $36 = HEAP32[$p>>2]|0;
    $37 = (($36) + 20|0);
    $38 = HEAP32[$37>>2]|0;
    $39 = HEAP32[$pIndex>>2]|0;
    $40 = ($38|0)!=($39|0);
    $61 = $40;
   } else {
    $61 = 0;
   }
   $41 = HEAP32[$p>>2]|0;
   if (!($61)) {
    break;
   }
   $42 = (($41) + 20|0);
   $43 = HEAP32[$42>>2]|0;
   HEAP32[$p>>2] = $43;
  }
  $44 = ($41|0)!=(0|0);
  if ($44) {
   $45 = HEAP32[$p>>2]|0;
   $46 = (($45) + 20|0);
   $47 = HEAP32[$46>>2]|0;
   $48 = HEAP32[$pIndex>>2]|0;
   $49 = ($47|0)==($48|0);
   if ($49) {
    $50 = HEAP32[$pIndex>>2]|0;
    $51 = (($50) + 20|0);
    $52 = HEAP32[$51>>2]|0;
    $53 = HEAP32[$p>>2]|0;
    $54 = (($53) + 20|0);
    HEAP32[$54>>2] = $52;
   }
  }
 }
 $55 = HEAP32[$0>>2]|0;
 $56 = HEAP32[$pIndex>>2]|0;
 _freeIndex($55,$56);
 $57 = HEAP32[$0>>2]|0;
 $58 = (($57) + 24|0);
 $59 = HEAP32[$58>>2]|0;
 $60 = $59 | 2;
 HEAP32[$58>>2] = $60;
 STACKTOP = sp;return;
}
function _sqlite3UnlinkAndDeleteTrigger($db,$iDb,$zName) {
 $db = $db|0;
 $iDb = $iDb|0;
 $zName = $zName|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $5 = 0, $6 = 0, $7 = 0;
 var $8 = 0, $9 = 0, $pHash = 0, $pTab = 0, $pTrigger = 0, $pp = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 24|0;
 $1 = sp + 20|0;
 $2 = sp + 16|0;
 $pTrigger = sp + 12|0;
 $pHash = sp + 8|0;
 $pTab = sp + 4|0;
 $pp = sp;
 HEAP32[$0>>2] = $db;
 HEAP32[$1>>2] = $iDb;
 HEAP32[$2>>2] = $zName;
 $3 = HEAP32[$1>>2]|0;
 $4 = HEAP32[$0>>2]|0;
 $5 = (($4) + 16|0);
 $6 = HEAP32[$5>>2]|0;
 $7 = (($6) + ($3<<4)|0);
 $8 = (($7) + 12|0);
 $9 = HEAP32[$8>>2]|0;
 $10 = (($9) + 40|0);
 HEAP32[$pHash>>2] = $10;
 $11 = HEAP32[$pHash>>2]|0;
 $12 = HEAP32[$2>>2]|0;
 $13 = (_sqlite3HashInsert($11,$12,0)|0);
 HEAP32[$pTrigger>>2] = $13;
 $14 = HEAP32[$pTrigger>>2]|0;
 $15 = ($14|0)!=(0|0);
 if (!($15)) {
  STACKTOP = sp;return;
 }
 $16 = HEAP32[$pTrigger>>2]|0;
 $17 = (($16) + 20|0);
 $18 = HEAP32[$17>>2]|0;
 $19 = HEAP32[$pTrigger>>2]|0;
 $20 = (($19) + 24|0);
 $21 = HEAP32[$20>>2]|0;
 $22 = ($18|0)==($21|0);
 if ($22) {
  $23 = HEAP32[$pTrigger>>2]|0;
  $24 = (_tableOfTrigger($23)|0);
  HEAP32[$pTab>>2] = $24;
  $25 = HEAP32[$pTab>>2]|0;
  $26 = (($25) + 64|0);
  HEAP32[$pp>>2] = $26;
  while(1) {
   $27 = HEAP32[$pp>>2]|0;
   $28 = HEAP32[$27>>2]|0;
   $29 = HEAP32[$pTrigger>>2]|0;
   $30 = ($28|0)!=($29|0);
   $31 = HEAP32[$pp>>2]|0;
   $32 = HEAP32[$31>>2]|0;
   $33 = (($32) + 32|0);
   if (!($30)) {
    break;
   }
   HEAP32[$pp>>2] = $33;
  }
  $34 = HEAP32[$33>>2]|0;
  $35 = HEAP32[$pp>>2]|0;
  HEAP32[$35>>2] = $34;
 }
 $36 = HEAP32[$0>>2]|0;
 $37 = HEAP32[$pTrigger>>2]|0;
 _sqlite3DeleteTrigger($36,$37);
 $38 = HEAP32[$0>>2]|0;
 $39 = (($38) + 24|0);
 $40 = HEAP32[$39>>2]|0;
 $41 = $40 | 2;
 HEAP32[$39>>2] = $41;
 STACKTOP = sp;return;
}
function _sqlite3BtreeIntegrityCheck($p,$aRoot,$nRoot,$mxErr,$pnErr) {
 $p = $p|0;
 $aRoot = $aRoot|0;
 $nRoot = $nRoot|0;
 $mxErr = $mxErr|0;
 $pnErr = $pnErr|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $17 = 0, $18 = 0;
 var $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0;
 var $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0;
 var $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0;
 var $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0;
 var $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $i = 0, $nRef = 0, $pBt = 0, $sCheck = 0, $vararg_buffer4 = 0, $vararg_ptr7 = 0, $zErr = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 224|0;
 $vararg_buffer4 = sp;
 $0 = sp + 108|0;
 $1 = sp + 104|0;
 $2 = sp + 100|0;
 $3 = sp + 96|0;
 $4 = sp + 92|0;
 $5 = sp + 88|0;
 $i = sp + 84|0;
 $nRef = sp + 80|0;
 $sCheck = sp + 12|0;
 $pBt = sp + 8|0;
 $zErr = sp + 112|0;
 HEAP32[$1>>2] = $p;
 HEAP32[$2>>2] = $aRoot;
 HEAP32[$3>>2] = $nRoot;
 HEAP32[$4>>2] = $mxErr;
 HEAP32[$5>>2] = $pnErr;
 $6 = HEAP32[$1>>2]|0;
 $7 = (($6) + 4|0);
 $8 = HEAP32[$7>>2]|0;
 HEAP32[$pBt>>2] = $8;
 $9 = HEAP32[$1>>2]|0;
 _sqlite3BtreeEnter($9);
 $10 = HEAP32[$pBt>>2]|0;
 $11 = HEAP32[$10>>2]|0;
 $12 = (_sqlite3PagerRefcount($11)|0);
 HEAP32[$nRef>>2] = $12;
 $13 = HEAP32[$pBt>>2]|0;
 HEAP32[$sCheck>>2] = $13;
 $14 = HEAP32[$pBt>>2]|0;
 $15 = HEAP32[$14>>2]|0;
 $16 = (($sCheck) + 4|0);
 HEAP32[$16>>2] = $15;
 $17 = HEAP32[$sCheck>>2]|0;
 $18 = (_btreePagecount($17)|0);
 $19 = (($sCheck) + 12|0);
 HEAP32[$19>>2] = $18;
 $20 = HEAP32[$4>>2]|0;
 $21 = (($sCheck) + 16|0);
 HEAP32[$21>>2] = $20;
 $22 = (($sCheck) + 20|0);
 HEAP32[$22>>2] = 0;
 $23 = (($sCheck) + 24|0);
 HEAP32[$23>>2] = 0;
 $24 = (($sCheck) + 28|0);
 HEAP32[$24>>2] = 0;
 $25 = (($sCheck) + 32|0);
 HEAP32[$25>>2] = 0;
 $26 = (($sCheck) + 36|0);
 HEAP32[$26>>2] = 0;
 $27 = HEAP32[$5>>2]|0;
 HEAP32[$27>>2] = 0;
 $28 = (($sCheck) + 12|0);
 $29 = HEAP32[$28>>2]|0;
 $30 = ($29|0)==(0);
 if ($30) {
  HEAP32[$0>>2] = 0;
  $168 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($168|0);
 }
 $31 = (($sCheck) + 12|0);
 $32 = HEAP32[$31>>2]|0;
 $33 = (($32>>>0) / 8)&-1;
 $34 = (($33) + 1)|0;
 $35 = (_sqlite3MallocZero($34,0)|0);
 $36 = (($sCheck) + 8|0);
 HEAP32[$36>>2] = $35;
 $37 = (($sCheck) + 8|0);
 $38 = HEAP32[$37>>2]|0;
 $39 = ($38|0)!=(0|0);
 if (!($39)) {
  $40 = HEAP32[$5>>2]|0;
  HEAP32[$40>>2] = 1;
  HEAP32[$0>>2] = 0;
  $168 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($168|0);
 }
 $41 = HEAP32[1128>>2]|0;
 $42 = HEAP32[$pBt>>2]|0;
 $43 = (($42) + 32|0);
 $44 = HEAP32[$43>>2]|0;
 $45 = (($41>>>0) / ($44>>>0))&-1;
 $46 = (($45) + 1)|0;
 HEAP32[$i>>2] = $46;
 $47 = HEAP32[$i>>2]|0;
 $48 = (($sCheck) + 12|0);
 $49 = HEAP32[$48>>2]|0;
 $50 = ($47>>>0)<=($49>>>0);
 if ($50) {
  $51 = HEAP32[$i>>2]|0;
  _setPageReferenced($sCheck,$51);
 }
 $52 = (($sCheck) + 40|0);
 _sqlite3StrAccumInit($52,$zErr,100,1000000000);
 $53 = (($sCheck) + 40|0);
 $54 = (($53) + 24|0);
 HEAP8[$54>>0] = 2;
 $55 = (($sCheck) + 28|0);
 HEAP32[$55>>2] = 33320;
 $56 = HEAP32[$pBt>>2]|0;
 $57 = (($56) + 12|0);
 $58 = HEAP32[$57>>2]|0;
 $59 = (($58) + 56|0);
 $60 = HEAP32[$59>>2]|0;
 $61 = (($60) + 32|0);
 $62 = (_sqlite3Get4byte($61)|0);
 $63 = HEAP32[$pBt>>2]|0;
 $64 = (($63) + 12|0);
 $65 = HEAP32[$64>>2]|0;
 $66 = (($65) + 56|0);
 $67 = HEAP32[$66>>2]|0;
 $68 = (($67) + 36|0);
 $69 = (_sqlite3Get4byte($68)|0);
 _checkList($sCheck,1,$62,$69);
 $70 = (($sCheck) + 28|0);
 HEAP32[$70>>2] = 0;
 HEAP32[$i>>2] = 0;
 while(1) {
  $71 = HEAP32[$i>>2]|0;
  $72 = HEAP32[$3>>2]|0;
  $73 = ($71|0)<($72|0);
  if (!($73)) {
   break;
  }
  $74 = (($sCheck) + 16|0);
  $75 = HEAP32[$74>>2]|0;
  $76 = ($75|0)!=(0);
  if (!($76)) {
   break;
  }
  $77 = HEAP32[$i>>2]|0;
  $78 = HEAP32[$2>>2]|0;
  $79 = (($78) + ($77<<2)|0);
  $80 = HEAP32[$79>>2]|0;
  $81 = ($80|0)==(0);
  if (!($81)) {
   $82 = HEAP32[$pBt>>2]|0;
   $83 = (($82) + 17|0);
   $84 = HEAP8[$83>>0]|0;
   $85 = $84&255;
   $86 = ($85|0)!=(0);
   if ($86) {
    $87 = HEAP32[$i>>2]|0;
    $88 = HEAP32[$2>>2]|0;
    $89 = (($88) + ($87<<2)|0);
    $90 = HEAP32[$89>>2]|0;
    $91 = ($90|0)>(1);
    if ($91) {
     $92 = HEAP32[$i>>2]|0;
     $93 = HEAP32[$2>>2]|0;
     $94 = (($93) + ($92<<2)|0);
     $95 = HEAP32[$94>>2]|0;
     _checkPtrmap($sCheck,$95,1,0);
    }
   }
   $96 = (($sCheck) + 28|0);
   HEAP32[$96>>2] = 33336;
   $97 = HEAP32[$i>>2]|0;
   $98 = HEAP32[$2>>2]|0;
   $99 = (($98) + ($97<<2)|0);
   $100 = HEAP32[$99>>2]|0;
   (_checkTreePage($sCheck,$100,0,0)|0);
   $101 = (($sCheck) + 28|0);
   HEAP32[$101>>2] = 0;
  }
  $102 = HEAP32[$i>>2]|0;
  $103 = (($102) + 1)|0;
  HEAP32[$i>>2] = $103;
 }
 HEAP32[$i>>2] = 1;
 while(1) {
  $104 = HEAP32[$i>>2]|0;
  $105 = (($sCheck) + 12|0);
  $106 = HEAP32[$105>>2]|0;
  $107 = ($104>>>0)<=($106>>>0);
  if (!($107)) {
   break;
  }
  $108 = (($sCheck) + 16|0);
  $109 = HEAP32[$108>>2]|0;
  $110 = ($109|0)!=(0);
  if (!($110)) {
   break;
  }
  $111 = HEAP32[$i>>2]|0;
  $112 = (_getPageReferenced($sCheck,$111)|0);
  $113 = ($112|0)==(0);
  do {
   if ($113) {
    $114 = HEAP32[$pBt>>2]|0;
    $115 = HEAP32[$i>>2]|0;
    $116 = (_ptrmapPageno($114,$115)|0);
    $117 = HEAP32[$i>>2]|0;
    $118 = ($116|0)!=($117|0);
    if (!($118)) {
     $119 = HEAP32[$pBt>>2]|0;
     $120 = (($119) + 17|0);
     $121 = HEAP8[$120>>0]|0;
     $122 = ($121<<24>>24)!=(0);
     if ($122) {
      break;
     }
    }
    $123 = HEAP32[$i>>2]|0;
    HEAP32[$vararg_buffer4>>2] = $123;
    _checkAppendMsg($sCheck,33360,$vararg_buffer4);
   }
  } while(0);
  $124 = HEAP32[$i>>2]|0;
  $125 = (_getPageReferenced($sCheck,$124)|0);
  $126 = ($125|0)!=(0);
  if ($126) {
   $127 = HEAP32[$pBt>>2]|0;
   $128 = HEAP32[$i>>2]|0;
   $129 = (_ptrmapPageno($127,$128)|0);
   $130 = HEAP32[$i>>2]|0;
   $131 = ($129|0)==($130|0);
   if ($131) {
    $132 = HEAP32[$pBt>>2]|0;
    $133 = (($132) + 17|0);
    $134 = HEAP8[$133>>0]|0;
    $135 = $134&255;
    $136 = ($135|0)!=(0);
    if ($136) {
     $137 = HEAP32[$i>>2]|0;
     HEAP32[$vararg_buffer4>>2] = $137;
     _checkAppendMsg($sCheck,33384,$vararg_buffer4);
    }
   }
  }
  $138 = HEAP32[$i>>2]|0;
  $139 = (($138) + 1)|0;
  HEAP32[$i>>2] = $139;
 }
 $140 = HEAP32[$nRef>>2]|0;
 $141 = HEAP32[$pBt>>2]|0;
 $142 = HEAP32[$141>>2]|0;
 $143 = (_sqlite3PagerRefcount($142)|0);
 $144 = ($140|0)!=($143|0);
 if ($144) {
  $145 = HEAP32[$nRef>>2]|0;
  $146 = HEAP32[$pBt>>2]|0;
  $147 = HEAP32[$146>>2]|0;
  $148 = (_sqlite3PagerRefcount($147)|0);
  HEAP32[$vararg_buffer4>>2] = $145;
  $vararg_ptr7 = (($vararg_buffer4) + 4|0);
  HEAP32[$vararg_ptr7>>2] = $148;
  _checkAppendMsg($sCheck,33424,$vararg_buffer4);
 }
 $149 = (($sCheck) + 8|0);
 $150 = HEAP32[$149>>2]|0;
 _sqlite3_free($150);
 $151 = (($sCheck) + 24|0);
 $152 = HEAP32[$151>>2]|0;
 $153 = ($152|0)!=(0);
 if ($153) {
  $154 = (($sCheck) + 40|0);
  _sqlite3StrAccumReset($154);
  $155 = (($sCheck) + 20|0);
  $156 = HEAP32[$155>>2]|0;
  $157 = (($156) + 1)|0;
  $158 = HEAP32[$5>>2]|0;
  HEAP32[$158>>2] = $157;
  HEAP32[$0>>2] = 0;
  $168 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($168|0);
 }
 $159 = (($sCheck) + 20|0);
 $160 = HEAP32[$159>>2]|0;
 $161 = HEAP32[$5>>2]|0;
 HEAP32[$161>>2] = $160;
 $162 = (($sCheck) + 20|0);
 $163 = HEAP32[$162>>2]|0;
 $164 = ($163|0)==(0);
 if ($164) {
  $165 = (($sCheck) + 40|0);
  _sqlite3StrAccumReset($165);
 }
 $166 = (($sCheck) + 40|0);
 $167 = (_sqlite3StrAccumFinish($166)|0);
 HEAP32[$0>>2] = $167;
 $168 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($168|0);
}
function _sqlite3VdbeMemSetRowSet($pMem) {
 $pMem = $pMem|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $db = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $db = sp;
 HEAP32[$0>>2] = $pMem;
 $1 = HEAP32[$0>>2]|0;
 $2 = (($1) + 32|0);
 $3 = HEAP32[$2>>2]|0;
 HEAP32[$db>>2] = $3;
 $4 = HEAP32[$0>>2]|0;
 _sqlite3VdbeMemRelease($4);
 $5 = HEAP32[$db>>2]|0;
 $6 = (_sqlite3DbMallocRaw($5,64,0)|0);
 $7 = HEAP32[$0>>2]|0;
 $8 = (($7) + 20|0);
 HEAP32[$8>>2] = $6;
 $9 = HEAP32[$db>>2]|0;
 $10 = (($9) + 64|0);
 $11 = HEAP8[$10>>0]|0;
 $12 = ($11<<24>>24)!=(0);
 if ($12) {
  $13 = HEAP32[$0>>2]|0;
  $14 = (($13) + 8|0);
  HEAP16[$14>>1] = 1;
  $15 = HEAP32[$0>>2]|0;
  $16 = (($15) + 24|0);
  HEAP32[$16>>2] = 0;
  STACKTOP = sp;return;
 } else {
  $17 = HEAP32[$db>>2]|0;
  $18 = HEAP32[$0>>2]|0;
  $19 = (($18) + 20|0);
  $20 = HEAP32[$19>>2]|0;
  $21 = (_sqlite3DbMallocSize($17,$20)|0);
  $22 = HEAP32[$0>>2]|0;
  $23 = (($22) + 24|0);
  HEAP32[$23>>2] = $21;
  $24 = HEAP32[$db>>2]|0;
  $25 = HEAP32[$0>>2]|0;
  $26 = (($25) + 20|0);
  $27 = HEAP32[$26>>2]|0;
  $28 = HEAP32[$0>>2]|0;
  $29 = (($28) + 24|0);
  $30 = HEAP32[$29>>2]|0;
  $31 = (_sqlite3RowSetInit($24,$27,$30)|0);
  $32 = HEAP32[$0>>2]|0;
  HEAP32[$32>>2] = $31;
  $33 = HEAP32[$0>>2]|0;
  $34 = (($33) + 8|0);
  HEAP16[$34>>1] = 32;
  STACKTOP = sp;return;
 }
}
function _sqlite3RowSetInsert($p,$0,$1) {
 $p = $p|0;
 $0 = $0|0;
 $1 = $1|0;
 var $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0;
 var $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0;
 var $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0;
 var $65 = 0, $66 = 0, $67 = 0, $68 = 0, $7 = 0, $8 = 0, $9 = 0, $pEntry = 0, $pLast = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $2 = sp + 16|0;
 $3 = sp;
 $pEntry = sp + 12|0;
 $pLast = sp + 8|0;
 HEAP32[$2>>2] = $p;
 $4 = $3;
 $5 = $4;
 HEAP32[$5>>2] = $0;
 $6 = (($4) + 4)|0;
 $7 = $6;
 HEAP32[$7>>2] = $1;
 $8 = HEAP32[$2>>2]|0;
 $9 = (_rowSetEntryAlloc($8)|0);
 HEAP32[$pEntry>>2] = $9;
 $10 = HEAP32[$pEntry>>2]|0;
 $11 = ($10|0)==(0|0);
 if ($11) {
  STACKTOP = sp;return;
 }
 $12 = $3;
 $13 = $12;
 $14 = HEAP32[$13>>2]|0;
 $15 = (($12) + 4)|0;
 $16 = $15;
 $17 = HEAP32[$16>>2]|0;
 $18 = HEAP32[$pEntry>>2]|0;
 $19 = $18;
 $20 = $19;
 HEAP32[$20>>2] = $14;
 $21 = (($19) + 4)|0;
 $22 = $21;
 HEAP32[$22>>2] = $17;
 $23 = HEAP32[$pEntry>>2]|0;
 $24 = (($23) + 8|0);
 HEAP32[$24>>2] = 0;
 $25 = HEAP32[$2>>2]|0;
 $26 = (($25) + 12|0);
 $27 = HEAP32[$26>>2]|0;
 HEAP32[$pLast>>2] = $27;
 $28 = HEAP32[$pLast>>2]|0;
 $29 = ($28|0)!=(0|0);
 if ($29) {
  $30 = HEAP32[$2>>2]|0;
  $31 = (($30) + 26|0);
  $32 = HEAP16[$31>>1]|0;
  $33 = $32&65535;
  $34 = $33 & 1;
  $35 = ($34|0)!=(0);
  if ($35) {
   $36 = $3;
   $37 = $36;
   $38 = HEAP32[$37>>2]|0;
   $39 = (($36) + 4)|0;
   $40 = $39;
   $41 = HEAP32[$40>>2]|0;
   $42 = HEAP32[$pLast>>2]|0;
   $43 = $42;
   $44 = $43;
   $45 = HEAP32[$44>>2]|0;
   $46 = (($43) + 4)|0;
   $47 = $46;
   $48 = HEAP32[$47>>2]|0;
   $49 = ($41|0)<($48|0);
   $50 = ($38>>>0)<=($45>>>0);
   $51 = ($41|0)==($48|0);
   $52 = $51 & $50;
   $53 = $49 | $52;
   if ($53) {
    $54 = HEAP32[$2>>2]|0;
    $55 = (($54) + 26|0);
    $56 = HEAP16[$55>>1]|0;
    $57 = $56&65535;
    $58 = $57 & -2;
    $59 = $58&65535;
    HEAP16[$55>>1] = $59;
   }
  }
  $60 = HEAP32[$pEntry>>2]|0;
  $61 = HEAP32[$pLast>>2]|0;
  $62 = (($61) + 8|0);
  HEAP32[$62>>2] = $60;
 } else {
  $63 = HEAP32[$pEntry>>2]|0;
  $64 = HEAP32[$2>>2]|0;
  $65 = (($64) + 8|0);
  HEAP32[$65>>2] = $63;
 }
 $66 = HEAP32[$pEntry>>2]|0;
 $67 = HEAP32[$2>>2]|0;
 $68 = (($67) + 12|0);
 HEAP32[$68>>2] = $66;
 STACKTOP = sp;return;
}
function _sqlite3RowSetNext($p,$pRowid) {
 $p = $p|0;
 $pRowid = $pRowid|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0;
 var $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $1 = sp + 4|0;
 $2 = sp;
 HEAP32[$1>>2] = $p;
 HEAP32[$2>>2] = $pRowid;
 $3 = HEAP32[$1>>2]|0;
 $4 = (($3) + 26|0);
 $5 = HEAP16[$4>>1]|0;
 $6 = $5&65535;
 $7 = $6 & 2;
 $8 = ($7|0)==(0);
 if ($8) {
  $9 = HEAP32[$1>>2]|0;
  _rowSetToList($9);
 }
 $10 = HEAP32[$1>>2]|0;
 $11 = (($10) + 8|0);
 $12 = HEAP32[$11>>2]|0;
 $13 = ($12|0)!=(0|0);
 if (!($13)) {
  HEAP32[$0>>2] = 0;
  $40 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($40|0);
 }
 $14 = HEAP32[$1>>2]|0;
 $15 = (($14) + 8|0);
 $16 = HEAP32[$15>>2]|0;
 $17 = $16;
 $18 = $17;
 $19 = HEAP32[$18>>2]|0;
 $20 = (($17) + 4)|0;
 $21 = $20;
 $22 = HEAP32[$21>>2]|0;
 $23 = HEAP32[$2>>2]|0;
 $24 = $23;
 $25 = $24;
 HEAP32[$25>>2] = $19;
 $26 = (($24) + 4)|0;
 $27 = $26;
 HEAP32[$27>>2] = $22;
 $28 = HEAP32[$1>>2]|0;
 $29 = (($28) + 8|0);
 $30 = HEAP32[$29>>2]|0;
 $31 = (($30) + 8|0);
 $32 = HEAP32[$31>>2]|0;
 $33 = HEAP32[$1>>2]|0;
 $34 = (($33) + 8|0);
 HEAP32[$34>>2] = $32;
 $35 = HEAP32[$1>>2]|0;
 $36 = (($35) + 8|0);
 $37 = HEAP32[$36>>2]|0;
 $38 = ($37|0)==(0|0);
 if ($38) {
  $39 = HEAP32[$1>>2]|0;
  _sqlite3RowSetClear($39);
 }
 HEAP32[$0>>2] = 1;
 $40 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($40|0);
}
function _sqlite3RowSetTest($pRowSet,$iBatch,$0,$1) {
 $pRowSet = $pRowSet|0;
 $iBatch = $iBatch|0;
 $0 = $0|0;
 $1 = $1|0;
 var $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0, $116 = 0, $117 = 0;
 var $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0, $134 = 0, $135 = 0;
 var $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0;
 var $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0;
 var $p = 0, $pAux = 0, $pTail = 0, $pTree = 0, $ppPrevTree = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $2 = sp + 36|0;
 $3 = sp + 32|0;
 $4 = sp + 28|0;
 $5 = sp;
 $p = sp + 24|0;
 $pTree = sp + 20|0;
 $ppPrevTree = sp + 16|0;
 $pAux = sp + 12|0;
 $pTail = sp + 8|0;
 HEAP32[$3>>2] = $pRowSet;
 HEAP32[$4>>2] = $iBatch;
 $6 = $5;
 $7 = $6;
 HEAP32[$7>>2] = $0;
 $8 = (($6) + 4)|0;
 $9 = $8;
 HEAP32[$9>>2] = $1;
 $10 = HEAP32[$4>>2]|0;
 $11 = HEAP32[$3>>2]|0;
 $12 = (($11) + 28|0);
 $13 = HEAP32[$12>>2]|0;
 $14 = ($10|0)!=($13|0);
 if ($14) {
  $15 = HEAP32[$3>>2]|0;
  $16 = (($15) + 8|0);
  $17 = HEAP32[$16>>2]|0;
  HEAP32[$p>>2] = $17;
  $18 = HEAP32[$p>>2]|0;
  $19 = ($18|0)!=(0|0);
  if ($19) {
   $20 = HEAP32[$3>>2]|0;
   $21 = (($20) + 20|0);
   HEAP32[$ppPrevTree>>2] = $21;
   $22 = HEAP32[$3>>2]|0;
   $23 = (($22) + 26|0);
   $24 = HEAP16[$23>>1]|0;
   $25 = $24&65535;
   $26 = $25 & 1;
   $27 = ($26|0)==(0);
   if ($27) {
    $28 = HEAP32[$p>>2]|0;
    $29 = (_rowSetEntrySort($28)|0);
    HEAP32[$p>>2] = $29;
   }
   $30 = HEAP32[$3>>2]|0;
   $31 = (($30) + 20|0);
   $32 = HEAP32[$31>>2]|0;
   HEAP32[$pTree>>2] = $32;
   while(1) {
    $33 = HEAP32[$pTree>>2]|0;
    $34 = ($33|0)!=(0|0);
    if (!($34)) {
     break;
    }
    $35 = HEAP32[$pTree>>2]|0;
    $36 = (($35) + 8|0);
    HEAP32[$ppPrevTree>>2] = $36;
    $37 = HEAP32[$pTree>>2]|0;
    $38 = (($37) + 12|0);
    $39 = HEAP32[$38>>2]|0;
    $40 = ($39|0)==(0|0);
    if ($40) {
     label = 8;
     break;
    }
    $45 = HEAP32[$pTree>>2]|0;
    $46 = (($45) + 12|0);
    $47 = HEAP32[$46>>2]|0;
    _rowSetTreeToList($47,$pAux,$pTail);
    $48 = HEAP32[$pTree>>2]|0;
    $49 = (($48) + 12|0);
    HEAP32[$49>>2] = 0;
    $50 = HEAP32[$pAux>>2]|0;
    $51 = HEAP32[$p>>2]|0;
    $52 = (_rowSetEntryMerge($50,$51)|0);
    HEAP32[$p>>2] = $52;
    $53 = HEAP32[$pTree>>2]|0;
    $54 = (($53) + 8|0);
    $55 = HEAP32[$54>>2]|0;
    HEAP32[$pTree>>2] = $55;
   }
   if ((label|0) == 8) {
    $41 = HEAP32[$p>>2]|0;
    $42 = (_rowSetListToTree($41)|0);
    $43 = HEAP32[$pTree>>2]|0;
    $44 = (($43) + 12|0);
    HEAP32[$44>>2] = $42;
   }
   $56 = HEAP32[$pTree>>2]|0;
   $57 = ($56|0)==(0|0);
   if ($57) {
    $58 = HEAP32[$3>>2]|0;
    $59 = (_rowSetEntryAlloc($58)|0);
    HEAP32[$pTree>>2] = $59;
    $60 = HEAP32[$ppPrevTree>>2]|0;
    HEAP32[$60>>2] = $59;
    $61 = HEAP32[$pTree>>2]|0;
    $62 = ($61|0)!=(0|0);
    if ($62) {
     $63 = HEAP32[$pTree>>2]|0;
     $64 = $63;
     $65 = $64;
     HEAP32[$65>>2] = 0;
     $66 = (($64) + 4)|0;
     $67 = $66;
     HEAP32[$67>>2] = 0;
     $68 = HEAP32[$pTree>>2]|0;
     $69 = (($68) + 8|0);
     HEAP32[$69>>2] = 0;
     $70 = HEAP32[$p>>2]|0;
     $71 = (_rowSetListToTree($70)|0);
     $72 = HEAP32[$pTree>>2]|0;
     $73 = (($72) + 12|0);
     HEAP32[$73>>2] = $71;
    }
   }
   $74 = HEAP32[$3>>2]|0;
   $75 = (($74) + 8|0);
   HEAP32[$75>>2] = 0;
   $76 = HEAP32[$3>>2]|0;
   $77 = (($76) + 12|0);
   HEAP32[$77>>2] = 0;
   $78 = HEAP32[$3>>2]|0;
   $79 = (($78) + 26|0);
   $80 = HEAP16[$79>>1]|0;
   $81 = $80&65535;
   $82 = $81 | 1;
   $83 = $82&65535;
   HEAP16[$79>>1] = $83;
  }
  $84 = HEAP32[$4>>2]|0;
  $85 = HEAP32[$3>>2]|0;
  $86 = (($85) + 28|0);
  HEAP32[$86>>2] = $84;
 }
 $87 = HEAP32[$3>>2]|0;
 $88 = (($87) + 20|0);
 $89 = HEAP32[$88>>2]|0;
 HEAP32[$pTree>>2] = $89;
 L21: while(1) {
  $90 = HEAP32[$pTree>>2]|0;
  $91 = ($90|0)!=(0|0);
  if (!($91)) {
   label = 25;
   break;
  }
  $92 = HEAP32[$pTree>>2]|0;
  $93 = (($92) + 12|0);
  $94 = HEAP32[$93>>2]|0;
  HEAP32[$p>>2] = $94;
  while(1) {
   $95 = HEAP32[$p>>2]|0;
   $96 = ($95|0)!=(0|0);
   if (!($96)) {
    break;
   }
   $97 = HEAP32[$p>>2]|0;
   $98 = $97;
   $99 = $98;
   $100 = HEAP32[$99>>2]|0;
   $101 = (($98) + 4)|0;
   $102 = $101;
   $103 = HEAP32[$102>>2]|0;
   $104 = $5;
   $105 = $104;
   $106 = HEAP32[$105>>2]|0;
   $107 = (($104) + 4)|0;
   $108 = $107;
   $109 = HEAP32[$108>>2]|0;
   $110 = ($103|0)<($109|0);
   $111 = ($100>>>0)<($106>>>0);
   $112 = ($103|0)==($109|0);
   $113 = $112 & $111;
   $114 = $110 | $113;
   $115 = HEAP32[$p>>2]|0;
   if ($114) {
    $116 = (($115) + 8|0);
    $117 = HEAP32[$116>>2]|0;
    HEAP32[$p>>2] = $117;
    continue;
   }
   $118 = $115;
   $119 = $118;
   $120 = HEAP32[$119>>2]|0;
   $121 = (($118) + 4)|0;
   $122 = $121;
   $123 = HEAP32[$122>>2]|0;
   $124 = $5;
   $125 = $124;
   $126 = HEAP32[$125>>2]|0;
   $127 = (($124) + 4)|0;
   $128 = $127;
   $129 = HEAP32[$128>>2]|0;
   $130 = ($123|0)>($129|0);
   $131 = ($120>>>0)>($126>>>0);
   $132 = ($123|0)==($129|0);
   $133 = $132 & $131;
   $134 = $130 | $133;
   if (!($134)) {
    label = 23;
    break L21;
   }
   $135 = HEAP32[$p>>2]|0;
   $136 = (($135) + 12|0);
   $137 = HEAP32[$136>>2]|0;
   HEAP32[$p>>2] = $137;
  }
  $138 = HEAP32[$pTree>>2]|0;
  $139 = (($138) + 8|0);
  $140 = HEAP32[$139>>2]|0;
  HEAP32[$pTree>>2] = $140;
 }
 if ((label|0) == 23) {
  HEAP32[$2>>2] = 1;
  $141 = HEAP32[$2>>2]|0;
  STACKTOP = sp;return ($141|0);
 }
 else if ((label|0) == 25) {
  HEAP32[$2>>2] = 0;
  $141 = HEAP32[$2>>2]|0;
  STACKTOP = sp;return ($141|0);
 }
 return 0|0;
}
function _sqlite3PagerOkToChangeJournalMode($pPager) {
 $pPager = $pPager|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $3 = 0;
 var $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $1 = sp;
 HEAP32[$1>>2] = $pPager;
 $2 = HEAP32[$1>>2]|0;
 $3 = (($2) + 16|0);
 $4 = HEAP8[$3>>0]|0;
 $5 = $4&255;
 $6 = ($5|0)>=(3);
 if ($6) {
  HEAP32[$0>>2] = 0;
  $25 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($25|0);
 }
 $7 = HEAP32[$1>>2]|0;
 $8 = (($7) + 64|0);
 $9 = HEAP32[$8>>2]|0;
 $10 = HEAP32[$9>>2]|0;
 $11 = ($10|0)!=(0|0);
 if ($11) {
  $12 = HEAP32[$1>>2]|0;
  $13 = (($12) + 72|0);
  $14 = $13;
  $15 = $14;
  $16 = HEAP32[$15>>2]|0;
  $17 = (($14) + 4)|0;
  $18 = $17;
  $19 = HEAP32[$18>>2]|0;
  $20 = ($19|0)>(0);
  $21 = ($16>>>0)>(0);
  $22 = ($19|0)==(0);
  $23 = $22 & $21;
  $24 = $20 | $23;
  if ($24) {
   HEAP32[$0>>2] = 0;
   $25 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($25|0);
  }
 }
 HEAP32[$0>>2] = 1;
 $25 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($25|0);
}
function _sqlite3PagerWalSupported($pPager) {
 $pPager = $pPager|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $pMethods = 0;
 var label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $pMethods = sp;
 HEAP32[$0>>2] = $pPager;
 $1 = HEAP32[$0>>2]|0;
 $2 = (($1) + 60|0);
 $3 = HEAP32[$2>>2]|0;
 $4 = HEAP32[$3>>2]|0;
 HEAP32[$pMethods>>2] = $4;
 $5 = HEAP32[$0>>2]|0;
 $6 = (($5) + 4|0);
 $7 = HEAP8[$6>>0]|0;
 $8 = $7&255;
 $9 = ($8|0)!=(0);
 if ($9) {
  $18 = 1;
  $17 = $18&1;
  STACKTOP = sp;return ($17|0);
 }
 $10 = HEAP32[$pMethods>>2]|0;
 $11 = HEAP32[$10>>2]|0;
 $12 = ($11|0)>=(2);
 if (!($12)) {
  $18 = 0;
  $17 = $18&1;
  STACKTOP = sp;return ($17|0);
 }
 $13 = HEAP32[$pMethods>>2]|0;
 $14 = (($13) + 52|0);
 $15 = HEAP32[$14>>2]|0;
 $16 = ($15|0)!=(0|0);
 $18 = $16;
 $17 = $18&1;
 STACKTOP = sp;return ($17|0);
}
function _sqlite3PagerCloseWal($pPager) {
 $pPager = $pPager|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $logexists = 0, $or$cond = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $rc = sp + 4|0;
 $logexists = sp;
 HEAP32[$0>>2] = $pPager;
 HEAP32[$rc>>2] = 0;
 $1 = HEAP32[$0>>2]|0;
 $2 = (($1) + 208|0);
 $3 = HEAP32[$2>>2]|0;
 $4 = ($3|0)!=(0|0);
 if (!($4)) {
  HEAP32[$logexists>>2] = 0;
  $5 = HEAP32[$0>>2]|0;
  $6 = (_pagerLockDb($5,1)|0);
  HEAP32[$rc>>2] = $6;
  $7 = HEAP32[$rc>>2]|0;
  $8 = ($7|0)==(0);
  if ($8) {
   $9 = HEAP32[$0>>2]|0;
   $10 = HEAP32[$9>>2]|0;
   $11 = HEAP32[$0>>2]|0;
   $12 = (($11) + 212|0);
   $13 = HEAP32[$12>>2]|0;
   $14 = (_sqlite3OsAccess($10,$13,0,$logexists)|0);
   HEAP32[$rc>>2] = $14;
  }
  $15 = HEAP32[$rc>>2]|0;
  $16 = ($15|0)==(0);
  $17 = HEAP32[$logexists>>2]|0;
  $18 = ($17|0)!=(0);
  $or$cond = $16 & $18;
  if ($or$cond) {
   $19 = HEAP32[$0>>2]|0;
   $20 = (_pagerOpenWal($19)|0);
   HEAP32[$rc>>2] = $20;
  }
 }
 $21 = HEAP32[$rc>>2]|0;
 $22 = ($21|0)==(0);
 if (!($22)) {
  $48 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($48|0);
 }
 $23 = HEAP32[$0>>2]|0;
 $24 = (($23) + 208|0);
 $25 = HEAP32[$24>>2]|0;
 $26 = ($25|0)!=(0|0);
 if (!($26)) {
  $48 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($48|0);
 }
 $27 = HEAP32[$0>>2]|0;
 $28 = (_pagerExclusiveLock($27)|0);
 HEAP32[$rc>>2] = $28;
 $29 = HEAP32[$rc>>2]|0;
 $30 = ($29|0)==(0);
 if (!($30)) {
  $48 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($48|0);
 }
 $31 = HEAP32[$0>>2]|0;
 $32 = (($31) + 208|0);
 $33 = HEAP32[$32>>2]|0;
 $34 = HEAP32[$0>>2]|0;
 $35 = (($34) + 9|0);
 $36 = HEAP8[$35>>0]|0;
 $37 = $36&255;
 $38 = HEAP32[$0>>2]|0;
 $39 = (($38) + 152|0);
 $40 = HEAP32[$39>>2]|0;
 $41 = HEAP32[$0>>2]|0;
 $42 = (($41) + 200|0);
 $43 = HEAP32[$42>>2]|0;
 $44 = (_sqlite3WalClose($33,$37,$40,$43)|0);
 HEAP32[$rc>>2] = $44;
 $45 = HEAP32[$0>>2]|0;
 $46 = (($45) + 208|0);
 HEAP32[$46>>2] = 0;
 $47 = HEAP32[$0>>2]|0;
 _pagerFixMaplimit($47);
 $48 = HEAP32[$rc>>2]|0;
 STACKTOP = sp;return ($48|0);
}
function _sqlite3PagerSetJournalMode($pPager,$eMode) {
 $pPager = $pPager|0;
 $eMode = $eMode|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0;
 var $9 = 0, $eOld = 0, $or$cond = 0, $or$cond3 = 0, $rc = 0, $state = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 12|0;
 $1 = sp + 8|0;
 $eOld = sp + 16|0;
 $rc = sp + 4|0;
 $state = sp;
 HEAP32[$0>>2] = $pPager;
 HEAP32[$1>>2] = $eMode;
 $2 = HEAP32[$0>>2]|0;
 $3 = (($2) + 5|0);
 $4 = HEAP8[$3>>0]|0;
 HEAP8[$eOld>>0] = $4;
 $5 = HEAP32[$0>>2]|0;
 $6 = (($5) + 15|0);
 $7 = HEAP8[$6>>0]|0;
 $8 = ($7<<24>>24)!=(0);
 if ($8) {
  $9 = HEAP32[$1>>2]|0;
  $10 = ($9|0)!=(4);
  $11 = HEAP32[$1>>2]|0;
  $12 = ($11|0)!=(2);
  $or$cond = $10 & $12;
  if ($or$cond) {
   $13 = HEAP8[$eOld>>0]|0;
   $14 = $13&255;
   HEAP32[$1>>2] = $14;
  }
 }
 $15 = HEAP32[$1>>2]|0;
 $16 = HEAP8[$eOld>>0]|0;
 $17 = $16&255;
 $18 = ($15|0)!=($17|0);
 do {
  if ($18) {
   $19 = HEAP32[$1>>2]|0;
   $20 = $19&255;
   $21 = HEAP32[$0>>2]|0;
   $22 = (($21) + 5|0);
   HEAP8[$22>>0] = $20;
   $23 = HEAP32[$0>>2]|0;
   $24 = (($23) + 4|0);
   $25 = HEAP8[$24>>0]|0;
   $26 = ($25<<24>>24)!=(0);
   if (!($26)) {
    $27 = HEAP8[$eOld>>0]|0;
    $28 = $27&255;
    $29 = $28 & 5;
    $30 = ($29|0)==(1);
    if ($30) {
     $31 = HEAP32[$1>>2]|0;
     $32 = $31 & 1;
     $33 = ($32|0)==(0);
     if ($33) {
      $34 = HEAP32[$0>>2]|0;
      $35 = (($34) + 64|0);
      $36 = HEAP32[$35>>2]|0;
      (_sqlite3OsClose($36)|0);
      $37 = HEAP32[$0>>2]|0;
      $38 = (($37) + 17|0);
      $39 = HEAP8[$38>>0]|0;
      $40 = $39&255;
      $41 = ($40|0)>=(2);
      if ($41) {
       $42 = HEAP32[$0>>2]|0;
       $43 = HEAP32[$42>>2]|0;
       $44 = HEAP32[$0>>2]|0;
       $45 = (($44) + 172|0);
       $46 = HEAP32[$45>>2]|0;
       (_sqlite3OsDelete($43,$46,0)|0);
       break;
      }
      HEAP32[$rc>>2] = 0;
      $47 = HEAP32[$0>>2]|0;
      $48 = (($47) + 16|0);
      $49 = HEAP8[$48>>0]|0;
      $50 = $49&255;
      HEAP32[$state>>2] = $50;
      $51 = HEAP32[$state>>2]|0;
      $52 = ($51|0)==(0);
      if ($52) {
       $53 = HEAP32[$0>>2]|0;
       $54 = (_sqlite3PagerSharedLock($53)|0);
       HEAP32[$rc>>2] = $54;
      }
      $55 = HEAP32[$0>>2]|0;
      $56 = (($55) + 16|0);
      $57 = HEAP8[$56>>0]|0;
      $58 = $57&255;
      $59 = ($58|0)==(1);
      if ($59) {
       $60 = HEAP32[$0>>2]|0;
       $61 = (_pagerLockDb($60,2)|0);
       HEAP32[$rc>>2] = $61;
      }
      $62 = HEAP32[$rc>>2]|0;
      $63 = ($62|0)==(0);
      if ($63) {
       $64 = HEAP32[$0>>2]|0;
       $65 = HEAP32[$64>>2]|0;
       $66 = HEAP32[$0>>2]|0;
       $67 = (($66) + 172|0);
       $68 = HEAP32[$67>>2]|0;
       (_sqlite3OsDelete($65,$68,0)|0);
      }
      $69 = HEAP32[$rc>>2]|0;
      $70 = ($69|0)==(0);
      $71 = HEAP32[$state>>2]|0;
      $72 = ($71|0)==(1);
      $or$cond3 = $70 & $72;
      if ($or$cond3) {
       $73 = HEAP32[$0>>2]|0;
       (_pagerUnlockDb($73,1)|0);
       break;
      }
      $74 = HEAP32[$state>>2]|0;
      $75 = ($74|0)==(0);
      if ($75) {
       $76 = HEAP32[$0>>2]|0;
       _pager_unlock($76);
      }
     }
    }
   }
  }
 } while(0);
 $77 = HEAP32[$0>>2]|0;
 $78 = (($77) + 5|0);
 $79 = HEAP8[$78>>0]|0;
 $80 = $79&255;
 STACKTOP = sp;return ($80|0);
}
function _sqlite3RunVacuum($pzErrMsg,$db) {
 $pzErrMsg = $pzErrMsg|0;
 $db = $db|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0;
 var $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0, $188 = 0;
 var $189 = 0, $19 = 0, $190 = 0, $191 = 0, $192 = 0, $193 = 0, $194 = 0, $195 = 0, $196 = 0, $197 = 0, $198 = 0, $199 = 0, $2 = 0, $20 = 0, $200 = 0, $201 = 0, $202 = 0, $203 = 0, $204 = 0, $205 = 0;
 var $206 = 0, $207 = 0, $208 = 0, $209 = 0, $21 = 0, $210 = 0, $211 = 0, $212 = 0, $213 = 0, $214 = 0, $215 = 0, $216 = 0, $217 = 0, $218 = 0, $219 = 0, $22 = 0, $220 = 0, $221 = 0, $222 = 0, $223 = 0;
 var $224 = 0, $225 = 0, $226 = 0, $227 = 0, $228 = 0, $229 = 0, $23 = 0, $230 = 0, $231 = 0, $232 = 0, $233 = 0, $234 = 0, $235 = 0, $236 = 0, $237 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0;
 var $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0;
 var $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0;
 var $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0;
 var $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $i = 0, $isMemDb = 0;
 var $meta = 0, $nDb = 0, $nRes = 0, $pDb = 0, $pMain = 0, $pTemp = 0, $rc = 0, $saved_flags = 0, $saved_nChange = 0, $saved_nTotalChange = 0, $saved_xTrace = 0, $vararg_buffer1 = 0, $zSql = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 80|0;
 $vararg_buffer1 = sp;
 $0 = sp + 68|0;
 $1 = sp + 64|0;
 $2 = sp + 60|0;
 $rc = sp + 56|0;
 $pMain = sp + 52|0;
 $pTemp = sp + 48|0;
 $zSql = sp + 44|0;
 $saved_flags = sp + 40|0;
 $saved_nChange = sp + 36|0;
 $saved_nTotalChange = sp + 32|0;
 $saved_xTrace = sp + 28|0;
 $pDb = sp + 24|0;
 $isMemDb = sp + 20|0;
 $nRes = sp + 16|0;
 $nDb = sp + 12|0;
 $meta = sp + 8|0;
 $i = sp + 4|0;
 HEAP32[$1>>2] = $pzErrMsg;
 HEAP32[$2>>2] = $db;
 HEAP32[$rc>>2] = 0;
 HEAP32[$zSql>>2] = 0;
 HEAP32[$pDb>>2] = 0;
 $3 = HEAP32[$2>>2]|0;
 $4 = (($3) + 62|0);
 $5 = HEAP8[$4>>0]|0;
 $6 = ($5<<24>>24)!=(0);
 if (!($6)) {
  $7 = HEAP32[$1>>2]|0;
  $8 = HEAP32[$2>>2]|0;
  _sqlite3SetString($7,$8,32072,$vararg_buffer1);
  HEAP32[$0>>2] = 1;
  $237 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($237|0);
 }
 $9 = HEAP32[$2>>2]|0;
 $10 = (($9) + 148|0);
 $11 = HEAP32[$10>>2]|0;
 $12 = ($11|0)>(1);
 if ($12) {
  $13 = HEAP32[$1>>2]|0;
  $14 = HEAP32[$2>>2]|0;
  _sqlite3SetString($13,$14,32112,$vararg_buffer1);
  HEAP32[$0>>2] = 1;
  $237 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($237|0);
 }
 $15 = HEAP32[$2>>2]|0;
 $16 = (($15) + 24|0);
 $17 = HEAP32[$16>>2]|0;
 HEAP32[$saved_flags>>2] = $17;
 $18 = HEAP32[$2>>2]|0;
 $19 = (($18) + 80|0);
 $20 = HEAP32[$19>>2]|0;
 HEAP32[$saved_nChange>>2] = $20;
 $21 = HEAP32[$2>>2]|0;
 $22 = (($21) + 84|0);
 $23 = HEAP32[$22>>2]|0;
 HEAP32[$saved_nTotalChange>>2] = $23;
 $24 = HEAP32[$2>>2]|0;
 $25 = (($24) + 172|0);
 $26 = HEAP32[$25>>2]|0;
 HEAP32[$saved_xTrace>>2] = $26;
 $27 = HEAP32[$2>>2]|0;
 $28 = (($27) + 24|0);
 $29 = HEAP32[$28>>2]|0;
 $30 = $29 | 2107392;
 HEAP32[$28>>2] = $30;
 $31 = HEAP32[$2>>2]|0;
 $32 = (($31) + 24|0);
 $33 = HEAP32[$32>>2]|0;
 $34 = $33 & -655361;
 HEAP32[$32>>2] = $34;
 $35 = HEAP32[$2>>2]|0;
 $36 = (($35) + 172|0);
 HEAP32[$36>>2] = 0;
 $37 = HEAP32[$2>>2]|0;
 $38 = (($37) + 16|0);
 $39 = HEAP32[$38>>2]|0;
 $40 = (($39) + 4|0);
 $41 = HEAP32[$40>>2]|0;
 HEAP32[$pMain>>2] = $41;
 $42 = HEAP32[$pMain>>2]|0;
 $43 = (_sqlite3BtreePager($42)|0);
 $44 = (_sqlite3PagerIsMemdb($43)|0);
 HEAP32[$isMemDb>>2] = $44;
 $45 = HEAP32[$2>>2]|0;
 $46 = (($45) + 20|0);
 $47 = HEAP32[$46>>2]|0;
 HEAP32[$nDb>>2] = $47;
 $48 = HEAP32[$2>>2]|0;
 $49 = (_sqlite3TempInMemory($48)|0);
 $50 = ($49|0)!=(0);
 if ($50) {
  HEAP32[$zSql>>2] = 32160;
 } else {
  HEAP32[$zSql>>2] = 32192;
 }
 $51 = HEAP32[$2>>2]|0;
 $52 = HEAP32[$1>>2]|0;
 $53 = HEAP32[$zSql>>2]|0;
 $54 = (_execSql($51,$52,$53)|0);
 HEAP32[$rc>>2] = $54;
 $55 = HEAP32[$2>>2]|0;
 $56 = (($55) + 20|0);
 $57 = HEAP32[$56>>2]|0;
 $58 = HEAP32[$nDb>>2]|0;
 $59 = ($57|0)>($58|0);
 if ($59) {
  $60 = HEAP32[$2>>2]|0;
  $61 = (($60) + 20|0);
  $62 = HEAP32[$61>>2]|0;
  $63 = (($62) - 1)|0;
  $64 = HEAP32[$2>>2]|0;
  $65 = (($64) + 16|0);
  $66 = HEAP32[$65>>2]|0;
  $67 = (($66) + ($63<<4)|0);
  HEAP32[$pDb>>2] = $67;
 }
 $68 = HEAP32[$rc>>2]|0;
 $69 = ($68|0)!=(0);
 L16: do {
  if (!($69)) {
   $70 = HEAP32[$2>>2]|0;
   $71 = (($70) + 20|0);
   $72 = HEAP32[$71>>2]|0;
   $73 = (($72) - 1)|0;
   $74 = HEAP32[$2>>2]|0;
   $75 = (($74) + 16|0);
   $76 = HEAP32[$75>>2]|0;
   $77 = (($76) + ($73<<4)|0);
   $78 = (($77) + 4|0);
   $79 = HEAP32[$78>>2]|0;
   HEAP32[$pTemp>>2] = $79;
   $80 = HEAP32[$pTemp>>2]|0;
   (_sqlite3BtreeCommit($80)|0);
   $81 = HEAP32[$pMain>>2]|0;
   $82 = (_sqlite3BtreeGetReserve($81)|0);
   HEAP32[$nRes>>2] = $82;
   $83 = HEAP32[$2>>2]|0;
   $84 = HEAP32[$1>>2]|0;
   $85 = (_execSql($83,$84,32216)|0);
   HEAP32[$rc>>2] = $85;
   $86 = HEAP32[$rc>>2]|0;
   $87 = ($86|0)!=(0);
   if (!($87)) {
    $88 = HEAP32[$2>>2]|0;
    $89 = HEAP32[$1>>2]|0;
    $90 = (_execSql($88,$89,32256)|0);
    HEAP32[$rc>>2] = $90;
    $91 = HEAP32[$rc>>2]|0;
    $92 = ($91|0)!=(0);
    if (!($92)) {
     $93 = HEAP32[$pMain>>2]|0;
     $94 = (_sqlite3BtreeBeginTrans($93,2)|0);
     HEAP32[$rc>>2] = $94;
     $95 = HEAP32[$rc>>2]|0;
     $96 = ($95|0)!=(0);
     if (!($96)) {
      $97 = HEAP32[$pMain>>2]|0;
      $98 = (_sqlite3BtreePager($97)|0);
      $99 = (_sqlite3PagerGetJournalMode($98)|0);
      $100 = ($99|0)==(5);
      if ($100) {
       $101 = HEAP32[$2>>2]|0;
       $102 = (($101) + 72|0);
       HEAP32[$102>>2] = 0;
      }
      $103 = HEAP32[$pTemp>>2]|0;
      $104 = HEAP32[$pMain>>2]|0;
      $105 = (_sqlite3BtreeGetPageSize($104)|0);
      $106 = HEAP32[$nRes>>2]|0;
      $107 = (_sqlite3BtreeSetPageSize($103,$105,$106,0)|0);
      $108 = ($107|0)!=(0);
      do {
       if (!($108)) {
        $109 = HEAP32[$isMemDb>>2]|0;
        $110 = ($109|0)!=(0);
        if (!($110)) {
         $111 = HEAP32[$pTemp>>2]|0;
         $112 = HEAP32[$2>>2]|0;
         $113 = (($112) + 72|0);
         $114 = HEAP32[$113>>2]|0;
         $115 = HEAP32[$nRes>>2]|0;
         $116 = (_sqlite3BtreeSetPageSize($111,$114,$115,0)|0);
         $117 = ($116|0)!=(0);
         if ($117) {
          break;
         }
        }
        $118 = HEAP32[$2>>2]|0;
        $119 = (($118) + 64|0);
        $120 = HEAP8[$119>>0]|0;
        $121 = $120&255;
        $122 = ($121|0)!=(0);
        if (!($122)) {
         $123 = HEAP32[$pTemp>>2]|0;
         $124 = HEAP32[$2>>2]|0;
         $125 = (($124) + 66|0);
         $126 = HEAP8[$125>>0]|0;
         $127 = $126 << 24 >> 24;
         $128 = ($127|0)>=(0);
         if ($128) {
          $129 = HEAP32[$2>>2]|0;
          $130 = (($129) + 66|0);
          $131 = HEAP8[$130>>0]|0;
          $132 = $131 << 24 >> 24;
          $135 = $132;
         } else {
          $133 = HEAP32[$pMain>>2]|0;
          $134 = (_sqlite3BtreeGetAutoVacuum($133)|0);
          $135 = $134;
         }
         (_sqlite3BtreeSetAutoVacuum($123,$135)|0);
         $136 = HEAP32[$2>>2]|0;
         $137 = HEAP32[$1>>2]|0;
         $138 = (_execExecSql($136,$137,32264)|0);
         HEAP32[$rc>>2] = $138;
         $139 = HEAP32[$rc>>2]|0;
         $140 = ($139|0)!=(0);
         if ($140) {
          break L16;
         }
         $141 = HEAP32[$2>>2]|0;
         $142 = HEAP32[$1>>2]|0;
         $143 = (_execExecSql($141,$142,32416)|0);
         HEAP32[$rc>>2] = $143;
         $144 = HEAP32[$rc>>2]|0;
         $145 = ($144|0)!=(0);
         if ($145) {
          break L16;
         }
         $146 = HEAP32[$2>>2]|0;
         $147 = HEAP32[$1>>2]|0;
         $148 = (_execExecSql($146,$147,32520)|0);
         HEAP32[$rc>>2] = $148;
         $149 = HEAP32[$rc>>2]|0;
         $150 = ($149|0)!=(0);
         if ($150) {
          break L16;
         }
         $151 = HEAP32[$2>>2]|0;
         $152 = HEAP32[$1>>2]|0;
         $153 = (_execExecSql($151,$152,32640)|0);
         HEAP32[$rc>>2] = $153;
         $154 = HEAP32[$rc>>2]|0;
         $155 = ($154|0)!=(0);
         if ($155) {
          break L16;
         }
         $156 = HEAP32[$2>>2]|0;
         $157 = HEAP32[$1>>2]|0;
         $158 = (_execExecSql($156,$157,32840)|0);
         HEAP32[$rc>>2] = $158;
         $159 = HEAP32[$rc>>2]|0;
         $160 = ($159|0)!=(0);
         if ($160) {
          break L16;
         }
         $161 = HEAP32[$2>>2]|0;
         $162 = HEAP32[$1>>2]|0;
         $163 = (_execExecSql($161,$162,32960)|0);
         HEAP32[$rc>>2] = $163;
         $164 = HEAP32[$rc>>2]|0;
         $165 = ($164|0)!=(0);
         if ($165) {
          break L16;
         }
         $166 = HEAP32[$2>>2]|0;
         $167 = HEAP32[$1>>2]|0;
         $168 = (_execSql($166,$167,33120)|0);
         HEAP32[$rc>>2] = $168;
         $169 = HEAP32[$rc>>2]|0;
         $170 = ($169|0)!=(0);
         if ($170) {
          break L16;
         }
         HEAP32[$i>>2] = 0;
         while(1) {
          $171 = HEAP32[$i>>2]|0;
          $172 = ($171|0)<(10);
          $173 = HEAP32[$pMain>>2]|0;
          if (!($172)) {
           break;
          }
          $174 = HEAP32[$i>>2]|0;
          $175 = (33304 + ($174)|0);
          $176 = HEAP8[$175>>0]|0;
          $177 = $176&255;
          _sqlite3BtreeGetMeta($173,$177,$meta);
          $178 = HEAP32[$pTemp>>2]|0;
          $179 = HEAP32[$i>>2]|0;
          $180 = (33304 + ($179)|0);
          $181 = HEAP8[$180>>0]|0;
          $182 = $181&255;
          $183 = HEAP32[$meta>>2]|0;
          $184 = HEAP32[$i>>2]|0;
          $185 = (($184) + 1)|0;
          $186 = (33304 + ($185)|0);
          $187 = HEAP8[$186>>0]|0;
          $188 = $187&255;
          $189 = (($183) + ($188))|0;
          $190 = (_sqlite3BtreeUpdateMeta($178,$182,$189)|0);
          HEAP32[$rc>>2] = $190;
          $191 = HEAP32[$rc>>2]|0;
          $192 = ($191|0)!=(0);
          if ($192) {
           break L16;
          }
          $193 = HEAP32[$i>>2]|0;
          $194 = (($193) + 2)|0;
          HEAP32[$i>>2] = $194;
         }
         $195 = HEAP32[$pTemp>>2]|0;
         $196 = (_sqlite3BtreeCopyFile($173,$195)|0);
         HEAP32[$rc>>2] = $196;
         $197 = HEAP32[$rc>>2]|0;
         $198 = ($197|0)!=(0);
         if ($198) {
          break L16;
         }
         $199 = HEAP32[$pTemp>>2]|0;
         $200 = (_sqlite3BtreeCommit($199)|0);
         HEAP32[$rc>>2] = $200;
         $201 = HEAP32[$rc>>2]|0;
         $202 = ($201|0)!=(0);
         if ($202) {
          break L16;
         }
         $203 = HEAP32[$pMain>>2]|0;
         $204 = HEAP32[$pTemp>>2]|0;
         $205 = (_sqlite3BtreeGetAutoVacuum($204)|0);
         (_sqlite3BtreeSetAutoVacuum($203,$205)|0);
         $206 = HEAP32[$pMain>>2]|0;
         $207 = HEAP32[$pTemp>>2]|0;
         $208 = (_sqlite3BtreeGetPageSize($207)|0);
         $209 = HEAP32[$nRes>>2]|0;
         $210 = (_sqlite3BtreeSetPageSize($206,$208,$209,1)|0);
         HEAP32[$rc>>2] = $210;
         break L16;
        }
       }
      } while(0);
      HEAP32[$rc>>2] = 7;
     }
    }
   }
  }
 } while(0);
 $211 = HEAP32[$saved_flags>>2]|0;
 $212 = HEAP32[$2>>2]|0;
 $213 = (($212) + 24|0);
 HEAP32[$213>>2] = $211;
 $214 = HEAP32[$saved_nChange>>2]|0;
 $215 = HEAP32[$2>>2]|0;
 $216 = (($215) + 80|0);
 HEAP32[$216>>2] = $214;
 $217 = HEAP32[$saved_nTotalChange>>2]|0;
 $218 = HEAP32[$2>>2]|0;
 $219 = (($218) + 84|0);
 HEAP32[$219>>2] = $217;
 $220 = HEAP32[$saved_xTrace>>2]|0;
 $221 = HEAP32[$2>>2]|0;
 $222 = (($221) + 172|0);
 HEAP32[$222>>2] = $220;
 $223 = HEAP32[$pMain>>2]|0;
 (_sqlite3BtreeSetPageSize($223,-1,-1,1)|0);
 $224 = HEAP32[$2>>2]|0;
 $225 = (($224) + 62|0);
 HEAP8[$225>>0] = 1;
 $226 = HEAP32[$pDb>>2]|0;
 $227 = ($226|0)!=(0|0);
 if ($227) {
  $228 = HEAP32[$pDb>>2]|0;
  $229 = (($228) + 4|0);
  $230 = HEAP32[$229>>2]|0;
  (_sqlite3BtreeClose($230)|0);
  $231 = HEAP32[$pDb>>2]|0;
  $232 = (($231) + 4|0);
  HEAP32[$232>>2] = 0;
  $233 = HEAP32[$pDb>>2]|0;
  $234 = (($233) + 12|0);
  HEAP32[$234>>2] = 0;
 }
 $235 = HEAP32[$2>>2]|0;
 _sqlite3ResetAllSchemasOfConnection($235);
 $236 = HEAP32[$rc>>2]|0;
 HEAP32[$0>>2] = $236;
 $237 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($237|0);
}
function _sqlite3BtreeIncrVacuum($p) {
 $p = $p|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $nFin = 0, $nFree = 0, $nOrig = 0, $pBt = 0, $rc = 0;
 var label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 20|0;
 $rc = sp + 16|0;
 $pBt = sp + 12|0;
 $nOrig = sp + 8|0;
 $nFree = sp + 4|0;
 $nFin = sp;
 HEAP32[$0>>2] = $p;
 $1 = HEAP32[$0>>2]|0;
 $2 = (($1) + 4|0);
 $3 = HEAP32[$2>>2]|0;
 HEAP32[$pBt>>2] = $3;
 $4 = HEAP32[$0>>2]|0;
 _sqlite3BtreeEnter($4);
 $5 = HEAP32[$pBt>>2]|0;
 $6 = (($5) + 17|0);
 $7 = HEAP8[$6>>0]|0;
 $8 = ($7<<24>>24)!=(0);
 if (!($8)) {
  HEAP32[$rc>>2] = 101;
  $54 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($54|0);
 }
 $9 = HEAP32[$pBt>>2]|0;
 $10 = (_btreePagecount($9)|0);
 HEAP32[$nOrig>>2] = $10;
 $11 = HEAP32[$pBt>>2]|0;
 $12 = (($11) + 12|0);
 $13 = HEAP32[$12>>2]|0;
 $14 = (($13) + 56|0);
 $15 = HEAP32[$14>>2]|0;
 $16 = (($15) + 36|0);
 $17 = (_sqlite3Get4byte($16)|0);
 HEAP32[$nFree>>2] = $17;
 $18 = HEAP32[$pBt>>2]|0;
 $19 = HEAP32[$nOrig>>2]|0;
 $20 = HEAP32[$nFree>>2]|0;
 $21 = (_finalDbSize($18,$19,$20)|0);
 HEAP32[$nFin>>2] = $21;
 $22 = HEAP32[$nOrig>>2]|0;
 $23 = HEAP32[$nFin>>2]|0;
 $24 = ($22>>>0)<($23>>>0);
 if ($24) {
  $25 = (_sqlite3CorruptError(55422)|0);
  HEAP32[$rc>>2] = $25;
  $54 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($54|0);
 }
 $26 = HEAP32[$nFree>>2]|0;
 $27 = ($26>>>0)>(0);
 if (!($27)) {
  HEAP32[$rc>>2] = 101;
  $54 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($54|0);
 }
 $28 = HEAP32[$pBt>>2]|0;
 $29 = (_saveAllCursors($28,0,0)|0);
 HEAP32[$rc>>2] = $29;
 $30 = HEAP32[$rc>>2]|0;
 $31 = ($30|0)==(0);
 if ($31) {
  $32 = HEAP32[$pBt>>2]|0;
  _invalidateAllOverflowCache($32);
  $33 = HEAP32[$pBt>>2]|0;
  $34 = HEAP32[$nFin>>2]|0;
  $35 = HEAP32[$nOrig>>2]|0;
  $36 = (_incrVacuumStep($33,$34,$35,0)|0);
  HEAP32[$rc>>2] = $36;
 }
 $37 = HEAP32[$rc>>2]|0;
 $38 = ($37|0)==(0);
 if (!($38)) {
  $54 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($54|0);
 }
 $39 = HEAP32[$pBt>>2]|0;
 $40 = (($39) + 12|0);
 $41 = HEAP32[$40>>2]|0;
 $42 = (($41) + 68|0);
 $43 = HEAP32[$42>>2]|0;
 $44 = (_sqlite3PagerWrite($43)|0);
 HEAP32[$rc>>2] = $44;
 $45 = HEAP32[$pBt>>2]|0;
 $46 = (($45) + 12|0);
 $47 = HEAP32[$46>>2]|0;
 $48 = (($47) + 56|0);
 $49 = HEAP32[$48>>2]|0;
 $50 = (($49) + 28|0);
 $51 = HEAP32[$pBt>>2]|0;
 $52 = (($51) + 44|0);
 $53 = HEAP32[$52>>2]|0;
 _sqlite3Put4byte($50,$53);
 $54 = HEAP32[$rc>>2]|0;
 STACKTOP = sp;return ($54|0);
}
function _sqlite3BtreeLockTable($p,$iTab,$isWriteLock) {
 $p = $p|0;
 $iTab = $iTab|0;
 $isWriteLock = $isWriteLock|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0;
 var $7 = 0, $8 = 0, $9 = 0, $lockType = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $1 = sp + 4|0;
 $2 = sp + 13|0;
 $rc = sp;
 $lockType = sp + 12|0;
 HEAP32[$0>>2] = $p;
 HEAP32[$1>>2] = $iTab;
 HEAP8[$2>>0] = $isWriteLock;
 HEAP32[$rc>>2] = 0;
 $3 = HEAP32[$0>>2]|0;
 $4 = (($3) + 9|0);
 $5 = HEAP8[$4>>0]|0;
 $6 = ($5<<24>>24)!=(0);
 if (!($6)) {
  $22 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($22|0);
 }
 $7 = HEAP8[$2>>0]|0;
 $8 = $7&255;
 $9 = (1 + ($8))|0;
 $10 = $9&255;
 HEAP8[$lockType>>0] = $10;
 $11 = HEAP32[$0>>2]|0;
 _sqlite3BtreeEnter($11);
 $12 = HEAP32[$0>>2]|0;
 $13 = HEAP32[$1>>2]|0;
 $14 = HEAP8[$lockType>>0]|0;
 $15 = (_querySharedCacheTableLock($12,$13,$14)|0);
 HEAP32[$rc>>2] = $15;
 $16 = HEAP32[$rc>>2]|0;
 $17 = ($16|0)==(0);
 if (!($17)) {
  $22 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($22|0);
 }
 $18 = HEAP32[$0>>2]|0;
 $19 = HEAP32[$1>>2]|0;
 $20 = HEAP8[$lockType>>0]|0;
 $21 = (_setSharedCacheTableLock($18,$19,$20)|0);
 HEAP32[$rc>>2] = $21;
 $22 = HEAP32[$rc>>2]|0;
 STACKTOP = sp;return ($22|0);
}
function _sqlite3VtabBegin($db,$pVTab) {
 $db = $db|0;
 $pVTab = $pVTab|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $i = 0, $pModule = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 20|0;
 $1 = sp + 16|0;
 $2 = sp + 12|0;
 $rc = sp + 8|0;
 $pModule = sp + 4|0;
 $i = sp;
 HEAP32[$1>>2] = $db;
 HEAP32[$2>>2] = $pVTab;
 HEAP32[$rc>>2] = 0;
 $3 = HEAP32[$1>>2]|0;
 $4 = (($3) + 304|0);
 $5 = HEAP32[$4>>2]|0;
 $6 = ($5|0)>(0);
 if ($6) {
  $7 = HEAP32[$1>>2]|0;
  $8 = (($7) + 328|0);
  $9 = HEAP32[$8>>2]|0;
  $10 = ($9|0)==(0|0);
  if ($10) {
   HEAP32[$0>>2] = 6;
   $52 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($52|0);
  }
 }
 $11 = HEAP32[$2>>2]|0;
 $12 = ($11|0)!=(0|0);
 if (!($12)) {
  HEAP32[$0>>2] = 0;
  $52 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($52|0);
 }
 $13 = HEAP32[$2>>2]|0;
 $14 = (($13) + 8|0);
 $15 = HEAP32[$14>>2]|0;
 $16 = HEAP32[$15>>2]|0;
 HEAP32[$pModule>>2] = $16;
 $17 = HEAP32[$pModule>>2]|0;
 $18 = (($17) + 56|0);
 $19 = HEAP32[$18>>2]|0;
 $20 = ($19|0)!=(0|0);
 if ($20) {
  HEAP32[$i>>2] = 0;
  while(1) {
   $21 = HEAP32[$i>>2]|0;
   $22 = HEAP32[$1>>2]|0;
   $23 = (($22) + 304|0);
   $24 = HEAP32[$23>>2]|0;
   $25 = ($21|0)<($24|0);
   if (!($25)) {
    break;
   }
   $26 = HEAP32[$i>>2]|0;
   $27 = HEAP32[$1>>2]|0;
   $28 = (($27) + 328|0);
   $29 = HEAP32[$28>>2]|0;
   $30 = (($29) + ($26<<2)|0);
   $31 = HEAP32[$30>>2]|0;
   $32 = HEAP32[$2>>2]|0;
   $33 = ($31|0)==($32|0);
   if ($33) {
    label = 10;
    break;
   }
   $34 = HEAP32[$i>>2]|0;
   $35 = (($34) + 1)|0;
   HEAP32[$i>>2] = $35;
  }
  if ((label|0) == 10) {
   HEAP32[$0>>2] = 0;
   $52 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($52|0);
  }
  $36 = HEAP32[$1>>2]|0;
  $37 = (_growVTrans($36)|0);
  HEAP32[$rc>>2] = $37;
  $38 = HEAP32[$rc>>2]|0;
  $39 = ($38|0)==(0);
  if ($39) {
   $40 = HEAP32[$pModule>>2]|0;
   $41 = (($40) + 56|0);
   $42 = HEAP32[$41>>2]|0;
   $43 = HEAP32[$2>>2]|0;
   $44 = (($43) + 8|0);
   $45 = HEAP32[$44>>2]|0;
   $46 = (FUNCTION_TABLE_ii[$42 & 31]($45)|0);
   HEAP32[$rc>>2] = $46;
   $47 = HEAP32[$rc>>2]|0;
   $48 = ($47|0)==(0);
   if ($48) {
    $49 = HEAP32[$1>>2]|0;
    $50 = HEAP32[$2>>2]|0;
    _addToVTrans($49,$50);
   }
  }
 }
 $51 = HEAP32[$rc>>2]|0;
 HEAP32[$0>>2] = $51;
 $52 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($52|0);
}
function _sqlite3VtabCallCreate($db,$iDb,$zTab,$pzErr) {
 $db = $db|0;
 $iDb = $iDb|0;
 $zTab = $zTab|0;
 $pzErr = $pzErr|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $pMod = 0, $pTab = 0, $rc = 0, $vararg_buffer = 0, $zMod = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $vararg_buffer = sp;
 $0 = sp + 32|0;
 $1 = sp + 28|0;
 $2 = sp + 24|0;
 $3 = sp + 20|0;
 $rc = sp + 16|0;
 $pTab = sp + 12|0;
 $pMod = sp + 8|0;
 $zMod = sp + 4|0;
 HEAP32[$0>>2] = $db;
 HEAP32[$1>>2] = $iDb;
 HEAP32[$2>>2] = $zTab;
 HEAP32[$3>>2] = $pzErr;
 HEAP32[$rc>>2] = 0;
 $4 = HEAP32[$0>>2]|0;
 $5 = HEAP32[$2>>2]|0;
 $6 = HEAP32[$1>>2]|0;
 $7 = HEAP32[$0>>2]|0;
 $8 = (($7) + 16|0);
 $9 = HEAP32[$8>>2]|0;
 $10 = (($9) + ($6<<4)|0);
 $11 = HEAP32[$10>>2]|0;
 $12 = (_sqlite3FindTable($4,$5,$11)|0);
 HEAP32[$pTab>>2] = $12;
 $13 = HEAP32[$pTab>>2]|0;
 $14 = (($13) + 56|0);
 $15 = HEAP32[$14>>2]|0;
 $16 = HEAP32[$15>>2]|0;
 HEAP32[$zMod>>2] = $16;
 $17 = HEAP32[$0>>2]|0;
 $18 = (($17) + 308|0);
 $19 = HEAP32[$zMod>>2]|0;
 $20 = (_sqlite3HashFind($18,$19)|0);
 HEAP32[$pMod>>2] = $20;
 $21 = HEAP32[$pMod>>2]|0;
 $22 = ($21|0)!=(0|0);
 $23 = HEAP32[$0>>2]|0;
 if ($22) {
  $27 = HEAP32[$pTab>>2]|0;
  $28 = HEAP32[$pMod>>2]|0;
  $29 = HEAP32[$pMod>>2]|0;
  $30 = HEAP32[$29>>2]|0;
  $31 = (($30) + 4|0);
  $32 = HEAP32[$31>>2]|0;
  $33 = HEAP32[$3>>2]|0;
  $34 = (_vtabCallConstructor($23,$27,$28,$32,$33)|0);
  HEAP32[$rc>>2] = $34;
 } else {
  $24 = HEAP32[$zMod>>2]|0;
  HEAP32[$vararg_buffer>>2] = $24;
  $25 = (_sqlite3MPrintf($23,17944,$vararg_buffer)|0);
  $26 = HEAP32[$3>>2]|0;
  HEAP32[$26>>2] = $25;
  HEAP32[$rc>>2] = 1;
 }
 $35 = HEAP32[$rc>>2]|0;
 $36 = ($35|0)==(0);
 if (!($36)) {
  $49 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($49|0);
 }
 $37 = HEAP32[$0>>2]|0;
 $38 = HEAP32[$pTab>>2]|0;
 $39 = (_sqlite3GetVTable($37,$38)|0);
 $40 = ($39|0)!=(0|0);
 if (!($40)) {
  $49 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($49|0);
 }
 $41 = HEAP32[$0>>2]|0;
 $42 = (_growVTrans($41)|0);
 HEAP32[$rc>>2] = $42;
 $43 = HEAP32[$rc>>2]|0;
 $44 = ($43|0)==(0);
 if (!($44)) {
  $49 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($49|0);
 }
 $45 = HEAP32[$0>>2]|0;
 $46 = HEAP32[$0>>2]|0;
 $47 = HEAP32[$pTab>>2]|0;
 $48 = (_sqlite3GetVTable($46,$47)|0);
 _addToVTrans($45,$48);
 $49 = HEAP32[$rc>>2]|0;
 STACKTOP = sp;return ($49|0);
}
function _sqlite3VtabCallDestroy($db,$iDb,$zTab) {
 $db = $db|0;
 $iDb = $iDb|0;
 $zTab = $zTab|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $p = 0;
 var $pTab = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 20|0;
 $1 = sp + 16|0;
 $2 = sp + 12|0;
 $rc = sp + 8|0;
 $pTab = sp + 4|0;
 $p = sp;
 HEAP32[$0>>2] = $db;
 HEAP32[$1>>2] = $iDb;
 HEAP32[$2>>2] = $zTab;
 HEAP32[$rc>>2] = 0;
 $3 = HEAP32[$0>>2]|0;
 $4 = HEAP32[$2>>2]|0;
 $5 = HEAP32[$1>>2]|0;
 $6 = HEAP32[$0>>2]|0;
 $7 = (($6) + 16|0);
 $8 = HEAP32[$7>>2]|0;
 $9 = (($8) + ($5<<4)|0);
 $10 = HEAP32[$9>>2]|0;
 $11 = (_sqlite3FindTable($3,$4,$10)|0);
 HEAP32[$pTab>>2] = $11;
 $12 = HEAP32[$pTab>>2]|0;
 $13 = ($12|0)!=(0|0);
 if (!($13)) {
  $38 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($38|0);
 }
 $14 = HEAP32[$pTab>>2]|0;
 $15 = (($14) + 60|0);
 $16 = HEAP32[$15>>2]|0;
 $17 = ($16|0)!=(0|0);
 if (!($17)) {
  $38 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($38|0);
 }
 $18 = HEAP32[$0>>2]|0;
 $19 = HEAP32[$pTab>>2]|0;
 $20 = (_vtabDisconnectAll($18,$19)|0);
 HEAP32[$p>>2] = $20;
 $21 = HEAP32[$p>>2]|0;
 $22 = (($21) + 4|0);
 $23 = HEAP32[$22>>2]|0;
 $24 = HEAP32[$23>>2]|0;
 $25 = (($24) + 20|0);
 $26 = HEAP32[$25>>2]|0;
 $27 = HEAP32[$p>>2]|0;
 $28 = (($27) + 8|0);
 $29 = HEAP32[$28>>2]|0;
 $30 = (FUNCTION_TABLE_ii[$26 & 31]($29)|0);
 HEAP32[$rc>>2] = $30;
 $31 = HEAP32[$rc>>2]|0;
 $32 = ($31|0)==(0);
 if (!($32)) {
  $38 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($38|0);
 }
 $33 = HEAP32[$p>>2]|0;
 $34 = (($33) + 8|0);
 HEAP32[$34>>2] = 0;
 $35 = HEAP32[$pTab>>2]|0;
 $36 = (($35) + 60|0);
 HEAP32[$36>>2] = 0;
 $37 = HEAP32[$p>>2]|0;
 _sqlite3VtabUnlock($37);
 $38 = HEAP32[$rc>>2]|0;
 STACKTOP = sp;return ($38|0);
}
function _sqlite3BtreeMaxPageCount($p,$mxPage) {
 $p = $p|0;
 $mxPage = $mxPage|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $n = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $1 = sp + 4|0;
 $n = sp;
 HEAP32[$0>>2] = $p;
 HEAP32[$1>>2] = $mxPage;
 $2 = HEAP32[$0>>2]|0;
 _sqlite3BtreeEnter($2);
 $3 = HEAP32[$0>>2]|0;
 $4 = (($3) + 4|0);
 $5 = HEAP32[$4>>2]|0;
 $6 = HEAP32[$5>>2]|0;
 $7 = HEAP32[$1>>2]|0;
 $8 = (_sqlite3PagerMaxPageCount($6,$7)|0);
 HEAP32[$n>>2] = $8;
 $9 = HEAP32[$n>>2]|0;
 STACKTOP = sp;return ($9|0);
}
function _sqlite3VdbeExpandSql($p,$zRawSql) {
 $p = $p|0;
 $zRawSql = $zRawSql|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0;
 var $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0;
 var $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0;
 var $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0;
 var $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0.0, $97 = 0, $98 = 0, $99 = 0, $db = 0, $enc = 0, $i = 0, $idx = 0;
 var $n = 0, $nOut = 0, $nOut1 = 0, $nToken = 0, $nextIndex = 0, $out = 0, $pVar = 0, $utf8 = 0, $vararg_buffer11 = 0, $vararg_ptr7 = 0, $zBase = 0, $zStart = 0, dest = 0, label = 0, sp = 0, stop = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 240|0;
 $vararg_buffer11 = sp + 40|0;
 $0 = sp + 120|0;
 $1 = sp + 116|0;
 $db = sp + 112|0;
 $idx = sp + 108|0;
 $nextIndex = sp + 104|0;
 $n = sp + 100|0;
 $nToken = sp + 96|0;
 $i = sp + 92|0;
 $pVar = sp + 88|0;
 $out = sp + 60|0;
 $zBase = sp + 128|0;
 $zStart = sp + 56|0;
 $nOut = sp + 52|0;
 $enc = sp + 124|0;
 $utf8 = sp;
 $nOut1 = sp + 48|0;
 HEAP32[$0>>2] = $p;
 HEAP32[$1>>2] = $zRawSql;
 HEAP32[$idx>>2] = 0;
 HEAP32[$nextIndex>>2] = 1;
 $2 = HEAP32[$0>>2]|0;
 $3 = HEAP32[$2>>2]|0;
 HEAP32[$db>>2] = $3;
 $4 = HEAP32[$db>>2]|0;
 $5 = (($4) + 88|0);
 $6 = HEAP32[$5>>2]|0;
 _sqlite3StrAccumInit($out,$zBase,100,$6);
 $7 = HEAP32[$db>>2]|0;
 HEAP32[$out>>2] = $7;
 $8 = HEAP32[$db>>2]|0;
 $9 = (($8) + 160|0);
 $10 = HEAP32[$9>>2]|0;
 $11 = ($10|0)>(1);
 if ($11) {
  while(1) {
   $12 = HEAP32[$1>>2]|0;
   $13 = HEAP8[$12>>0]|0;
   $14 = ($13<<24>>24)!=(0);
   if (!($14)) {
    break;
   }
   $15 = HEAP32[$1>>2]|0;
   HEAP32[$zStart>>2] = $15;
   while(1) {
    $16 = HEAP32[$1>>2]|0;
    $17 = (($16) + 1|0);
    HEAP32[$1>>2] = $17;
    $18 = HEAP8[$16>>0]|0;
    $19 = $18 << 24 >> 24;
    $20 = ($19|0)!=(10);
    if (!($20)) {
     break;
    }
    $21 = HEAP32[$1>>2]|0;
    $22 = HEAP8[$21>>0]|0;
    $23 = $22 << 24 >> 24;
    $24 = ($23|0)!=(0);
    if (!($24)) {
     break;
    }
   }
   _sqlite3StrAccumAppend($out,32016,3);
   $25 = HEAP32[$zStart>>2]|0;
   $26 = HEAP32[$1>>2]|0;
   $27 = HEAP32[$zStart>>2]|0;
   $28 = $26;
   $29 = $27;
   $30 = (($28) - ($29))|0;
   _sqlite3StrAccumAppend($out,$25,$30);
  }
  $155 = (_sqlite3StrAccumFinish($out)|0);
  STACKTOP = sp;return ($155|0);
 }
 while(1) {
  $31 = HEAP32[$1>>2]|0;
  $32 = HEAP8[$31>>0]|0;
  $33 = ($32<<24>>24)!=(0);
  if (!($33)) {
   label = 31;
   break;
  }
  $34 = HEAP32[$1>>2]|0;
  $35 = (_findNextHostParameter($34,$nToken)|0);
  HEAP32[$n>>2] = $35;
  $36 = HEAP32[$1>>2]|0;
  $37 = HEAP32[$n>>2]|0;
  _sqlite3StrAccumAppend($out,$36,$37);
  $38 = HEAP32[$n>>2]|0;
  $39 = HEAP32[$1>>2]|0;
  $40 = (($39) + ($38)|0);
  HEAP32[$1>>2] = $40;
  $41 = HEAP32[$nToken>>2]|0;
  $42 = ($41|0)==(0);
  if ($42) {
   label = 31;
   break;
  }
  $43 = HEAP32[$1>>2]|0;
  $44 = HEAP8[$43>>0]|0;
  $45 = $44 << 24 >> 24;
  $46 = ($45|0)==(63);
  do {
   if ($46) {
    $47 = HEAP32[$nToken>>2]|0;
    $48 = ($47|0)>(1);
    if ($48) {
     $49 = HEAP32[$1>>2]|0;
     $50 = (($49) + 1|0);
     (_sqlite3GetInt32($50,$idx)|0);
     break;
    } else {
     $51 = HEAP32[$nextIndex>>2]|0;
     HEAP32[$idx>>2] = $51;
     break;
    }
   } else {
    $52 = HEAP32[$0>>2]|0;
    $53 = HEAP32[$1>>2]|0;
    $54 = HEAP32[$nToken>>2]|0;
    $55 = (_sqlite3VdbeParameterIndex($52,$53,$54)|0);
    HEAP32[$idx>>2] = $55;
   }
  } while(0);
  $56 = HEAP32[$nToken>>2]|0;
  $57 = HEAP32[$1>>2]|0;
  $58 = (($57) + ($56)|0);
  HEAP32[$1>>2] = $58;
  $59 = HEAP32[$idx>>2]|0;
  $60 = (($59) + 1)|0;
  HEAP32[$nextIndex>>2] = $60;
  $61 = HEAP32[$idx>>2]|0;
  $62 = (($61) - 1)|0;
  $63 = HEAP32[$0>>2]|0;
  $64 = (($63) + 60|0);
  $65 = HEAP32[$64>>2]|0;
  $66 = (($65) + (($62*40)|0)|0);
  HEAP32[$pVar>>2] = $66;
  $67 = HEAP32[$pVar>>2]|0;
  $68 = (($67) + 8|0);
  $69 = HEAP16[$68>>1]|0;
  $70 = $69&65535;
  $71 = $70 & 1;
  $72 = ($71|0)!=(0);
  if ($72) {
   _sqlite3StrAccumAppend($out,9840,4);
   continue;
  }
  $73 = HEAP32[$pVar>>2]|0;
  $74 = (($73) + 8|0);
  $75 = HEAP16[$74>>1]|0;
  $76 = $75&65535;
  $77 = $76 & 4;
  $78 = ($77|0)!=(0);
  $79 = HEAP32[$pVar>>2]|0;
  if ($78) {
   $80 = $79;
   $81 = $80;
   $82 = HEAP32[$81>>2]|0;
   $83 = (($80) + 4)|0;
   $84 = $83;
   $85 = HEAP32[$84>>2]|0;
   $86 = $vararg_buffer11;
   $87 = $86;
   HEAP32[$87>>2] = $82;
   $88 = (($86) + 4)|0;
   $89 = $88;
   HEAP32[$89>>2] = $85;
   _sqlite3XPrintf($out,0,9456,$vararg_buffer11);
   continue;
  }
  $90 = (($79) + 8|0);
  $91 = HEAP16[$90>>1]|0;
  $92 = $91&65535;
  $93 = $92 & 8;
  $94 = ($93|0)!=(0);
  $95 = HEAP32[$pVar>>2]|0;
  if ($94) {
   $96 = +HEAPF64[$95>>3];
   HEAPF64[tempDoublePtr>>3]=$96;HEAP32[$vararg_buffer11>>2]=HEAP32[tempDoublePtr>>2];HEAP32[$vararg_buffer11+4>>2]=HEAP32[tempDoublePtr+4>>2];
   _sqlite3XPrintf($out,0,9808,$vararg_buffer11);
   continue;
  }
  $97 = (($95) + 8|0);
  $98 = HEAP16[$97>>1]|0;
  $99 = $98&65535;
  $100 = $99 & 2;
  $101 = ($100|0)!=(0);
  if ($101) {
   $102 = HEAP32[$db>>2]|0;
   $103 = (($102) + 16|0);
   $104 = HEAP32[$103>>2]|0;
   $105 = (($104) + 12|0);
   $106 = HEAP32[$105>>2]|0;
   $107 = (($106) + 77|0);
   $108 = HEAP8[$107>>0]|0;
   HEAP8[$enc>>0] = $108;
   $109 = HEAP8[$enc>>0]|0;
   $110 = $109&255;
   $111 = ($110|0)!=(1);
   if ($111) {
    dest=$utf8+0|0; stop=dest+40|0; do { HEAP32[dest>>2]=0|0; dest=dest+4|0; } while ((dest|0) < (stop|0));
    $112 = HEAP32[$db>>2]|0;
    $113 = (($utf8) + 32|0);
    HEAP32[$113>>2] = $112;
    $114 = HEAP32[$pVar>>2]|0;
    $115 = (($114) + 16|0);
    $116 = HEAP32[$115>>2]|0;
    $117 = HEAP32[$pVar>>2]|0;
    $118 = (($117) + 12|0);
    $119 = HEAP32[$118>>2]|0;
    $120 = HEAP8[$enc>>0]|0;
    (_sqlite3VdbeMemSetStr($utf8,$116,$119,$120,0)|0);
    (_sqlite3VdbeChangeEncoding($utf8,1)|0);
    HEAP32[$pVar>>2] = $utf8;
   }
   $121 = HEAP32[$pVar>>2]|0;
   $122 = (($121) + 12|0);
   $123 = HEAP32[$122>>2]|0;
   HEAP32[$nOut>>2] = $123;
   $124 = HEAP32[$nOut>>2]|0;
   $125 = HEAP32[$pVar>>2]|0;
   $126 = (($125) + 16|0);
   $127 = HEAP32[$126>>2]|0;
   HEAP32[$vararg_buffer11>>2] = $124;
   $vararg_ptr7 = (($vararg_buffer11) + 4|0);
   HEAP32[$vararg_ptr7>>2] = $127;
   _sqlite3XPrintf($out,0,32024,$vararg_buffer11);
   $128 = HEAP8[$enc>>0]|0;
   $129 = $128&255;
   $130 = ($129|0)!=(1);
   if (!($130)) {
    continue;
   }
   _sqlite3VdbeMemRelease($utf8);
   continue;
  }
  $131 = HEAP32[$pVar>>2]|0;
  $132 = (($131) + 8|0);
  $133 = HEAP16[$132>>1]|0;
  $134 = $133&65535;
  $135 = $134 & 16384;
  $136 = ($135|0)!=(0);
  if ($136) {
   $137 = HEAP32[$pVar>>2]|0;
   $138 = HEAP32[$137>>2]|0;
   HEAP32[$vararg_buffer11>>2] = $138;
   _sqlite3XPrintf($out,0,32032,$vararg_buffer11);
   continue;
  }
  _sqlite3StrAccumAppend($out,32048,2);
  $139 = HEAP32[$pVar>>2]|0;
  $140 = (($139) + 12|0);
  $141 = HEAP32[$140>>2]|0;
  HEAP32[$nOut1>>2] = $141;
  HEAP32[$i>>2] = 0;
  while(1) {
   $142 = HEAP32[$i>>2]|0;
   $143 = HEAP32[$nOut1>>2]|0;
   $144 = ($142|0)<($143|0);
   if (!($144)) {
    break;
   }
   $145 = HEAP32[$i>>2]|0;
   $146 = HEAP32[$pVar>>2]|0;
   $147 = (($146) + 16|0);
   $148 = HEAP32[$147>>2]|0;
   $149 = (($148) + ($145)|0);
   $150 = HEAP8[$149>>0]|0;
   $151 = $150 << 24 >> 24;
   $152 = $151 & 255;
   HEAP32[$vararg_buffer11>>2] = $152;
   _sqlite3XPrintf($out,0,32056,$vararg_buffer11);
   $153 = HEAP32[$i>>2]|0;
   $154 = (($153) + 1)|0;
   HEAP32[$i>>2] = $154;
  }
  _sqlite3StrAccumAppend($out,32064,1);
 }
 if ((label|0) == 31) {
  $155 = (_sqlite3StrAccumFinish($out)|0);
  STACKTOP = sp;return ($155|0);
 }
 return 0|0;
}
function _findNextHostParameter($zSql,$pnToken) {
 $zSql = $zSql|0;
 $pnToken = $pnToken|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $n = 0, $nTotal = 0;
 var $tokenType = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 16|0;
 $1 = sp + 12|0;
 $tokenType = sp + 8|0;
 $nTotal = sp + 4|0;
 $n = sp;
 HEAP32[$0>>2] = $zSql;
 HEAP32[$1>>2] = $pnToken;
 HEAP32[$nTotal>>2] = 0;
 $2 = HEAP32[$1>>2]|0;
 HEAP32[$2>>2] = 0;
 while(1) {
  $3 = HEAP32[$0>>2]|0;
  $4 = HEAP8[$3>>0]|0;
  $5 = ($4<<24>>24)!=(0);
  if (!($5)) {
   label = 6;
   break;
  }
  $6 = HEAP32[$0>>2]|0;
  $7 = (_sqlite3GetToken($6,$tokenType)|0);
  HEAP32[$n>>2] = $7;
  $8 = HEAP32[$tokenType>>2]|0;
  $9 = ($8|0)==(135);
  $10 = HEAP32[$n>>2]|0;
  if ($9) {
   break;
  }
  $12 = HEAP32[$nTotal>>2]|0;
  $13 = (($12) + ($10))|0;
  HEAP32[$nTotal>>2] = $13;
  $14 = HEAP32[$n>>2]|0;
  $15 = HEAP32[$0>>2]|0;
  $16 = (($15) + ($14)|0);
  HEAP32[$0>>2] = $16;
 }
 if ((label|0) == 6) {
  $17 = HEAP32[$nTotal>>2]|0;
  STACKTOP = sp;return ($17|0);
 }
 $11 = HEAP32[$1>>2]|0;
 HEAP32[$11>>2] = $10;
 $17 = HEAP32[$nTotal>>2]|0;
 STACKTOP = sp;return ($17|0);
}
function _sqlite3PagerMaxPageCount($pPager,$mxPage) {
 $pPager = $pPager|0;
 $mxPage = $mxPage|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $1 = sp;
 HEAP32[$0>>2] = $pPager;
 HEAP32[$1>>2] = $mxPage;
 $2 = HEAP32[$1>>2]|0;
 $3 = ($2|0)>(0);
 if ($3) {
  $4 = HEAP32[$1>>2]|0;
  $5 = HEAP32[$0>>2]|0;
  $6 = (($5) + 156|0);
  HEAP32[$6>>2] = $4;
 }
 $7 = HEAP32[$0>>2]|0;
 $8 = (($7) + 156|0);
 $9 = HEAP32[$8>>2]|0;
 STACKTOP = sp;return ($9|0);
}
function _growVTrans($db) {
 $db = $db|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $ARRAY_INCR = 0, $aVTrans = 0, $nBytes = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 16|0;
 $1 = sp + 12|0;
 $ARRAY_INCR = sp + 8|0;
 $aVTrans = sp + 4|0;
 $nBytes = sp;
 HEAP32[$1>>2] = $db;
 HEAP32[$ARRAY_INCR>>2] = 5;
 $2 = HEAP32[$1>>2]|0;
 $3 = (($2) + 304|0);
 $4 = HEAP32[$3>>2]|0;
 $5 = (($4|0) % 5)&-1;
 $6 = ($5|0)==(0);
 do {
  if ($6) {
   $7 = HEAP32[$1>>2]|0;
   $8 = (($7) + 304|0);
   $9 = HEAP32[$8>>2]|0;
   $10 = (($9) + 5)|0;
   $11 = $10<<2;
   HEAP32[$nBytes>>2] = $11;
   $12 = HEAP32[$1>>2]|0;
   $13 = HEAP32[$1>>2]|0;
   $14 = (($13) + 328|0);
   $15 = HEAP32[$14>>2]|0;
   $16 = HEAP32[$nBytes>>2]|0;
   $17 = ($16|0)<(0);
   $18 = $17 << 31 >> 31;
   $19 = (_sqlite3DbRealloc($12,$15,$16,$18)|0);
   HEAP32[$aVTrans>>2] = $19;
   $20 = HEAP32[$aVTrans>>2]|0;
   $21 = ($20|0)!=(0|0);
   if ($21) {
    $22 = HEAP32[$1>>2]|0;
    $23 = (($22) + 304|0);
    $24 = HEAP32[$23>>2]|0;
    $25 = HEAP32[$aVTrans>>2]|0;
    $26 = (($25) + ($24<<2)|0);
    ;HEAP32[$26+0>>2]=0|0;HEAP32[$26+4>>2]=0|0;HEAP32[$26+8>>2]=0|0;HEAP32[$26+12>>2]=0|0;HEAP32[$26+16>>2]=0|0;
    $27 = HEAP32[$aVTrans>>2]|0;
    $28 = HEAP32[$1>>2]|0;
    $29 = (($28) + 328|0);
    HEAP32[$29>>2] = $27;
    break;
   }
   HEAP32[$0>>2] = 7;
   $30 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($30|0);
  }
 } while(0);
 HEAP32[$0>>2] = 0;
 $30 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($30|0);
}
function _addToVTrans($db,$pVTab) {
 $db = $db|0;
 $pVTab = $pVTab|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $1 = sp;
 HEAP32[$0>>2] = $db;
 HEAP32[$1>>2] = $pVTab;
 $2 = HEAP32[$1>>2]|0;
 $3 = HEAP32[$0>>2]|0;
 $4 = (($3) + 304|0);
 $5 = HEAP32[$4>>2]|0;
 $6 = (($5) + 1)|0;
 HEAP32[$4>>2] = $6;
 $7 = HEAP32[$0>>2]|0;
 $8 = (($7) + 328|0);
 $9 = HEAP32[$8>>2]|0;
 $10 = (($9) + ($5<<2)|0);
 HEAP32[$10>>2] = $2;
 $11 = HEAP32[$1>>2]|0;
 _sqlite3VtabLock($11);
 STACKTOP = sp;return;
}
function _setSharedCacheTableLock($p,$iTable,$eLock) {
 $p = $p|0;
 $iTable = $iTable|0;
 $eLock = $eLock|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $pBt = 0, $pIter = 0, $pLock = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 20|0;
 $1 = sp + 16|0;
 $2 = sp + 12|0;
 $3 = sp + 24|0;
 $pBt = sp + 8|0;
 $pLock = sp + 4|0;
 $pIter = sp;
 HEAP32[$1>>2] = $p;
 HEAP32[$2>>2] = $iTable;
 HEAP8[$3>>0] = $eLock;
 $4 = HEAP32[$1>>2]|0;
 $5 = (($4) + 4|0);
 $6 = HEAP32[$5>>2]|0;
 HEAP32[$pBt>>2] = $6;
 HEAP32[$pLock>>2] = 0;
 $7 = HEAP32[$pBt>>2]|0;
 $8 = (($7) + 72|0);
 $9 = HEAP32[$8>>2]|0;
 HEAP32[$pIter>>2] = $9;
 while(1) {
  $10 = HEAP32[$pIter>>2]|0;
  $11 = ($10|0)!=(0|0);
  if (!($11)) {
   break;
  }
  $12 = HEAP32[$pIter>>2]|0;
  $13 = (($12) + 4|0);
  $14 = HEAP32[$13>>2]|0;
  $15 = HEAP32[$2>>2]|0;
  $16 = ($14|0)==($15|0);
  if ($16) {
   $17 = HEAP32[$pIter>>2]|0;
   $18 = HEAP32[$17>>2]|0;
   $19 = HEAP32[$1>>2]|0;
   $20 = ($18|0)==($19|0);
   if ($20) {
    label = 5;
    break;
   }
  }
  $22 = HEAP32[$pIter>>2]|0;
  $23 = (($22) + 12|0);
  $24 = HEAP32[$23>>2]|0;
  HEAP32[$pIter>>2] = $24;
 }
 if ((label|0) == 5) {
  $21 = HEAP32[$pIter>>2]|0;
  HEAP32[$pLock>>2] = $21;
 }
 $25 = HEAP32[$pLock>>2]|0;
 $26 = ($25|0)!=(0|0);
 do {
  if (!($26)) {
   $27 = (_sqlite3MallocZero(16,0)|0);
   HEAP32[$pLock>>2] = $27;
   $28 = HEAP32[$pLock>>2]|0;
   $29 = ($28|0)!=(0|0);
   if ($29) {
    $30 = HEAP32[$2>>2]|0;
    $31 = HEAP32[$pLock>>2]|0;
    $32 = (($31) + 4|0);
    HEAP32[$32>>2] = $30;
    $33 = HEAP32[$1>>2]|0;
    $34 = HEAP32[$pLock>>2]|0;
    HEAP32[$34>>2] = $33;
    $35 = HEAP32[$pBt>>2]|0;
    $36 = (($35) + 72|0);
    $37 = HEAP32[$36>>2]|0;
    $38 = HEAP32[$pLock>>2]|0;
    $39 = (($38) + 12|0);
    HEAP32[$39>>2] = $37;
    $40 = HEAP32[$pLock>>2]|0;
    $41 = HEAP32[$pBt>>2]|0;
    $42 = (($41) + 72|0);
    HEAP32[$42>>2] = $40;
    break;
   }
   HEAP32[$0>>2] = 7;
   $53 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($53|0);
  }
 } while(0);
 $43 = HEAP8[$3>>0]|0;
 $44 = $43&255;
 $45 = HEAP32[$pLock>>2]|0;
 $46 = (($45) + 8|0);
 $47 = HEAP8[$46>>0]|0;
 $48 = $47&255;
 $49 = ($44|0)>($48|0);
 if ($49) {
  $50 = HEAP8[$3>>0]|0;
  $51 = HEAP32[$pLock>>2]|0;
  $52 = (($51) + 8|0);
  HEAP8[$52>>0] = $50;
 }
 HEAP32[$0>>2] = 0;
 $53 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($53|0);
}
function _finalDbSize($pBt,$nOrig,$nFree) {
 $pBt = $pBt|0;
 $nOrig = $nOrig|0;
 $nFree = $nFree|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $nEntry = 0, $nFin = 0, $nPtrmap = 0;
 var label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 20|0;
 $1 = sp + 16|0;
 $2 = sp + 12|0;
 $nEntry = sp + 8|0;
 $nPtrmap = sp + 4|0;
 $nFin = sp;
 HEAP32[$0>>2] = $pBt;
 HEAP32[$1>>2] = $nOrig;
 HEAP32[$2>>2] = $nFree;
 $3 = HEAP32[$0>>2]|0;
 $4 = (($3) + 36|0);
 $5 = HEAP32[$4>>2]|0;
 $6 = (($5>>>0) / 5)&-1;
 HEAP32[$nEntry>>2] = $6;
 $7 = HEAP32[$2>>2]|0;
 $8 = HEAP32[$1>>2]|0;
 $9 = (($7) - ($8))|0;
 $10 = HEAP32[$0>>2]|0;
 $11 = HEAP32[$1>>2]|0;
 $12 = (_ptrmapPageno($10,$11)|0);
 $13 = (($9) + ($12))|0;
 $14 = HEAP32[$nEntry>>2]|0;
 $15 = (($13) + ($14))|0;
 $16 = HEAP32[$nEntry>>2]|0;
 $17 = (($15>>>0) / ($16>>>0))&-1;
 HEAP32[$nPtrmap>>2] = $17;
 $18 = HEAP32[$1>>2]|0;
 $19 = HEAP32[$2>>2]|0;
 $20 = (($18) - ($19))|0;
 $21 = HEAP32[$nPtrmap>>2]|0;
 $22 = (($20) - ($21))|0;
 HEAP32[$nFin>>2] = $22;
 $23 = HEAP32[$1>>2]|0;
 $24 = HEAP32[1128>>2]|0;
 $25 = HEAP32[$0>>2]|0;
 $26 = (($25) + 32|0);
 $27 = HEAP32[$26>>2]|0;
 $28 = (($24>>>0) / ($27>>>0))&-1;
 $29 = (($28) + 1)|0;
 $30 = ($23>>>0)>($29>>>0);
 if ($30) {
  $31 = HEAP32[$nFin>>2]|0;
  $32 = HEAP32[1128>>2]|0;
  $33 = HEAP32[$0>>2]|0;
  $34 = (($33) + 32|0);
  $35 = HEAP32[$34>>2]|0;
  $36 = (($32>>>0) / ($35>>>0))&-1;
  $37 = (($36) + 1)|0;
  $38 = ($31>>>0)<($37>>>0);
  if ($38) {
   $39 = HEAP32[$nFin>>2]|0;
   $40 = (($39) + -1)|0;
   HEAP32[$nFin>>2] = $40;
  }
 }
 while(1) {
  $41 = HEAP32[$0>>2]|0;
  $42 = HEAP32[$nFin>>2]|0;
  $43 = (_ptrmapPageno($41,$42)|0);
  $44 = HEAP32[$nFin>>2]|0;
  $45 = ($43|0)==($44|0);
  if ($45) {
   $56 = 1;
  } else {
   $46 = HEAP32[$nFin>>2]|0;
   $47 = HEAP32[1128>>2]|0;
   $48 = HEAP32[$0>>2]|0;
   $49 = (($48) + 32|0);
   $50 = HEAP32[$49>>2]|0;
   $51 = (($47>>>0) / ($50>>>0))&-1;
   $52 = (($51) + 1)|0;
   $53 = ($46|0)==($52|0);
   $56 = $53;
  }
  $54 = HEAP32[$nFin>>2]|0;
  if (!($56)) {
   break;
  }
  $55 = (($54) + -1)|0;
  HEAP32[$nFin>>2] = $55;
 }
 STACKTOP = sp;return ($54|0);
}
function _invalidateAllOverflowCache($pBt) {
 $pBt = $pBt|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $p = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $p = sp;
 HEAP32[$0>>2] = $pBt;
 $1 = HEAP32[$0>>2]|0;
 $2 = (($1) + 8|0);
 $3 = HEAP32[$2>>2]|0;
 HEAP32[$p>>2] = $3;
 while(1) {
  $4 = HEAP32[$p>>2]|0;
  $5 = ($4|0)!=(0|0);
  if (!($5)) {
   break;
  }
  $6 = HEAP32[$p>>2]|0;
  $7 = (($6) + 72|0);
  $8 = HEAP8[$7>>0]|0;
  $9 = $8&255;
  $10 = $9 & -5;
  $11 = $10&255;
  HEAP8[$7>>0] = $11;
  $12 = HEAP32[$p>>2]|0;
  $13 = (($12) + 8|0);
  $14 = HEAP32[$13>>2]|0;
  HEAP32[$p>>2] = $14;
 }
 STACKTOP = sp;return;
}
function _incrVacuumStep($pBt,$nFin,$iLastPg,$bCommit) {
 $pBt = $pBt|0;
 $nFin = $nFin|0;
 $iLastPg = $iLastPg|0;
 $bCommit = $bCommit|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0;
 var $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0;
 var $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0;
 var $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0;
 var $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0;
 var $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $eMode = 0, $eType = 0, $iFreePg = 0, $iFreePg1 = 0, $iNear = 0, $iPtrPage = 0, $nFreeList = 0, $pFreePg = 0, $pFreePg2 = 0, $pLastPg = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 64|0;
 $0 = sp + 52|0;
 $1 = sp + 48|0;
 $2 = sp + 44|0;
 $3 = sp + 40|0;
 $4 = sp + 36|0;
 $nFreeList = sp + 32|0;
 $rc = sp + 28|0;
 $eType = sp + 57|0;
 $iPtrPage = sp + 24|0;
 $iFreePg = sp + 20|0;
 $pFreePg = sp + 16|0;
 $iFreePg1 = sp + 12|0;
 $pLastPg = sp + 8|0;
 $eMode = sp + 56|0;
 $iNear = sp + 4|0;
 $pFreePg2 = sp;
 HEAP32[$1>>2] = $pBt;
 HEAP32[$2>>2] = $nFin;
 HEAP32[$3>>2] = $iLastPg;
 HEAP32[$4>>2] = $bCommit;
 $5 = HEAP32[$1>>2]|0;
 $6 = HEAP32[$3>>2]|0;
 $7 = (_ptrmapPageno($5,$6)|0);
 $8 = HEAP32[$3>>2]|0;
 $9 = ($7|0)==($8|0);
 do {
  if (!($9)) {
   $10 = HEAP32[$3>>2]|0;
   $11 = HEAP32[1128>>2]|0;
   $12 = HEAP32[$1>>2]|0;
   $13 = (($12) + 32|0);
   $14 = HEAP32[$13>>2]|0;
   $15 = (($11>>>0) / ($14>>>0))&-1;
   $16 = (($15) + 1)|0;
   $17 = ($10|0)!=($16|0);
   if ($17) {
    $18 = HEAP32[$1>>2]|0;
    $19 = (($18) + 12|0);
    $20 = HEAP32[$19>>2]|0;
    $21 = (($20) + 56|0);
    $22 = HEAP32[$21>>2]|0;
    $23 = (($22) + 36|0);
    $24 = (_sqlite3Get4byte($23)|0);
    HEAP32[$nFreeList>>2] = $24;
    $25 = HEAP32[$nFreeList>>2]|0;
    $26 = ($25|0)==(0);
    if ($26) {
     HEAP32[$0>>2] = 101;
     $105 = HEAP32[$0>>2]|0;
     STACKTOP = sp;return ($105|0);
    }
    $27 = HEAP32[$1>>2]|0;
    $28 = HEAP32[$3>>2]|0;
    $29 = (_ptrmapGet($27,$28,$eType,$iPtrPage)|0);
    HEAP32[$rc>>2] = $29;
    $30 = HEAP32[$rc>>2]|0;
    $31 = ($30|0)!=(0);
    if ($31) {
     $32 = HEAP32[$rc>>2]|0;
     HEAP32[$0>>2] = $32;
     $105 = HEAP32[$0>>2]|0;
     STACKTOP = sp;return ($105|0);
    }
    $33 = HEAP8[$eType>>0]|0;
    $34 = $33&255;
    $35 = ($34|0)==(1);
    if ($35) {
     $36 = (_sqlite3CorruptError(55307)|0);
     HEAP32[$0>>2] = $36;
     $105 = HEAP32[$0>>2]|0;
     STACKTOP = sp;return ($105|0);
    }
    $37 = HEAP8[$eType>>0]|0;
    $38 = $37&255;
    $39 = ($38|0)==(2);
    if ($39) {
     $40 = HEAP32[$4>>2]|0;
     $41 = ($40|0)==(0);
     if (!($41)) {
      break;
     }
     $42 = HEAP32[$1>>2]|0;
     $43 = HEAP32[$3>>2]|0;
     $44 = (_allocateBtreePage($42,$pFreePg,$iFreePg,$43,1)|0);
     HEAP32[$rc>>2] = $44;
     $45 = HEAP32[$rc>>2]|0;
     $46 = ($45|0)!=(0);
     if (!($46)) {
      $48 = HEAP32[$pFreePg>>2]|0;
      _releasePage($48);
      break;
     }
     $47 = HEAP32[$rc>>2]|0;
     HEAP32[$0>>2] = $47;
     $105 = HEAP32[$0>>2]|0;
     STACKTOP = sp;return ($105|0);
    }
    HEAP8[$eMode>>0] = 0;
    HEAP32[$iNear>>2] = 0;
    $49 = HEAP32[$1>>2]|0;
    $50 = HEAP32[$3>>2]|0;
    $51 = (_btreeGetPage($49,$50,$pLastPg,0)|0);
    HEAP32[$rc>>2] = $51;
    $52 = HEAP32[$rc>>2]|0;
    $53 = ($52|0)!=(0);
    if ($53) {
     $54 = HEAP32[$rc>>2]|0;
     HEAP32[$0>>2] = $54;
     $105 = HEAP32[$0>>2]|0;
     STACKTOP = sp;return ($105|0);
    }
    $55 = HEAP32[$4>>2]|0;
    $56 = ($55|0)==(0);
    if ($56) {
     HEAP8[$eMode>>0] = 2;
     $57 = HEAP32[$2>>2]|0;
     HEAP32[$iNear>>2] = $57;
    }
    while(1) {
     $58 = HEAP32[$1>>2]|0;
     $59 = HEAP32[$iNear>>2]|0;
     $60 = HEAP8[$eMode>>0]|0;
     $61 = (_allocateBtreePage($58,$pFreePg2,$iFreePg1,$59,$60)|0);
     HEAP32[$rc>>2] = $61;
     $62 = HEAP32[$rc>>2]|0;
     $63 = ($62|0)!=(0);
     if ($63) {
      label = 19;
      break;
     }
     $66 = HEAP32[$pFreePg2>>2]|0;
     _releasePage($66);
     $67 = HEAP32[$4>>2]|0;
     $68 = ($67|0)!=(0);
     if (!($68)) {
      break;
     }
     $69 = HEAP32[$iFreePg1>>2]|0;
     $70 = HEAP32[$2>>2]|0;
     $71 = ($69>>>0)>($70>>>0);
     if (!($71)) {
      break;
     }
    }
    if ((label|0) == 19) {
     $64 = HEAP32[$pLastPg>>2]|0;
     _releasePage($64);
     $65 = HEAP32[$rc>>2]|0;
     HEAP32[$0>>2] = $65;
     $105 = HEAP32[$0>>2]|0;
     STACKTOP = sp;return ($105|0);
    }
    $72 = HEAP32[$1>>2]|0;
    $73 = HEAP32[$pLastPg>>2]|0;
    $74 = HEAP8[$eType>>0]|0;
    $75 = HEAP32[$iPtrPage>>2]|0;
    $76 = HEAP32[$iFreePg1>>2]|0;
    $77 = HEAP32[$4>>2]|0;
    $78 = (_relocatePage($72,$73,$74,$75,$76,$77)|0);
    HEAP32[$rc>>2] = $78;
    $79 = HEAP32[$pLastPg>>2]|0;
    _releasePage($79);
    $80 = HEAP32[$rc>>2]|0;
    $81 = ($80|0)!=(0);
    if ($81) {
     $82 = HEAP32[$rc>>2]|0;
     HEAP32[$0>>2] = $82;
     $105 = HEAP32[$0>>2]|0;
     STACKTOP = sp;return ($105|0);
    }
   }
  }
 } while(0);
 $83 = HEAP32[$4>>2]|0;
 $84 = ($83|0)==(0);
 if ($84) {
  while(1) {
   $85 = HEAP32[$3>>2]|0;
   $86 = (($85) + -1)|0;
   HEAP32[$3>>2] = $86;
   $87 = HEAP32[$3>>2]|0;
   $88 = HEAP32[1128>>2]|0;
   $89 = HEAP32[$1>>2]|0;
   $90 = (($89) + 32|0);
   $91 = HEAP32[$90>>2]|0;
   $92 = (($88>>>0) / ($91>>>0))&-1;
   $93 = (($92) + 1)|0;
   $94 = ($87|0)==($93|0);
   if ($94) {
    continue;
   }
   $95 = HEAP32[$1>>2]|0;
   $96 = HEAP32[$3>>2]|0;
   $97 = (_ptrmapPageno($95,$96)|0);
   $98 = HEAP32[$3>>2]|0;
   $99 = ($97|0)==($98|0);
   if (!($99)) {
    break;
   }
  }
  $100 = HEAP32[$1>>2]|0;
  $101 = (($100) + 19|0);
  HEAP8[$101>>0] = 1;
  $102 = HEAP32[$3>>2]|0;
  $103 = HEAP32[$1>>2]|0;
  $104 = (($103) + 44|0);
  HEAP32[$104>>2] = $102;
 }
 HEAP32[$0>>2] = 0;
 $105 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($105|0);
}
function _allocateBtreePage($pBt,$ppPage,$pPgno,$nearby,$eMode) {
 $pBt = $pBt|0;
 $ppPage = $ppPage|0;
 $pPgno = $pPgno|0;
 $nearby = $nearby|0;
 $eMode = $eMode|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0;
 var $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0, $188 = 0;
 var $189 = 0, $19 = 0, $190 = 0, $191 = 0, $192 = 0, $193 = 0, $194 = 0, $195 = 0, $196 = 0, $197 = 0, $198 = 0, $199 = 0, $2 = 0, $20 = 0, $200 = 0, $201 = 0, $202 = 0, $203 = 0, $204 = 0, $205 = 0;
 var $206 = 0, $207 = 0, $208 = 0, $209 = 0, $21 = 0, $210 = 0, $211 = 0, $212 = 0, $213 = 0, $214 = 0, $215 = 0, $216 = 0, $217 = 0, $218 = 0, $219 = 0, $22 = 0, $220 = 0, $221 = 0, $222 = 0, $223 = 0;
 var $224 = 0, $225 = 0, $226 = 0, $227 = 0, $228 = 0, $229 = 0, $23 = 0, $230 = 0, $231 = 0, $232 = 0, $233 = 0, $234 = 0, $235 = 0, $236 = 0, $237 = 0, $238 = 0, $239 = 0, $24 = 0, $240 = 0, $241 = 0;
 var $242 = 0, $243 = 0, $244 = 0, $245 = 0, $246 = 0, $247 = 0, $248 = 0, $249 = 0, $25 = 0, $250 = 0, $251 = 0, $252 = 0, $253 = 0, $254 = 0, $255 = 0, $256 = 0, $257 = 0, $258 = 0, $259 = 0, $26 = 0;
 var $260 = 0, $261 = 0, $262 = 0, $263 = 0, $264 = 0, $265 = 0, $266 = 0, $267 = 0, $268 = 0, $269 = 0, $27 = 0, $270 = 0, $271 = 0, $272 = 0, $273 = 0, $274 = 0, $275 = 0, $276 = 0, $277 = 0, $278 = 0;
 var $279 = 0, $28 = 0, $280 = 0, $281 = 0, $282 = 0, $283 = 0, $284 = 0, $285 = 0, $286 = 0, $287 = 0, $288 = 0, $289 = 0, $29 = 0, $290 = 0, $291 = 0, $292 = 0, $293 = 0, $294 = 0, $295 = 0, $296 = 0;
 var $297 = 0, $298 = 0, $299 = 0, $3 = 0, $30 = 0, $300 = 0, $301 = 0, $302 = 0, $303 = 0, $304 = 0, $305 = 0, $306 = 0, $307 = 0, $308 = 0, $309 = 0, $31 = 0, $310 = 0, $311 = 0, $312 = 0, $313 = 0;
 var $314 = 0, $315 = 0, $316 = 0, $317 = 0, $318 = 0, $319 = 0, $32 = 0, $320 = 0, $321 = 0, $322 = 0, $323 = 0, $324 = 0, $325 = 0, $326 = 0, $327 = 0, $328 = 0, $329 = 0, $33 = 0, $330 = 0, $331 = 0;
 var $332 = 0, $333 = 0, $334 = 0, $335 = 0, $336 = 0, $337 = 0, $338 = 0, $339 = 0, $34 = 0, $340 = 0, $341 = 0, $342 = 0, $343 = 0, $344 = 0, $345 = 0, $346 = 0, $347 = 0, $348 = 0, $349 = 0, $35 = 0;
 var $350 = 0, $351 = 0, $352 = 0, $353 = 0, $354 = 0, $355 = 0, $356 = 0, $357 = 0, $358 = 0, $359 = 0, $36 = 0, $360 = 0, $361 = 0, $362 = 0, $363 = 0, $364 = 0, $365 = 0, $366 = 0, $367 = 0, $368 = 0;
 var $369 = 0, $37 = 0, $370 = 0, $371 = 0, $372 = 0, $373 = 0, $374 = 0, $375 = 0, $376 = 0, $377 = 0, $378 = 0, $379 = 0, $38 = 0, $380 = 0, $381 = 0, $382 = 0, $383 = 0, $384 = 0, $385 = 0, $386 = 0;
 var $387 = 0, $388 = 0, $389 = 0, $39 = 0, $390 = 0, $391 = 0, $392 = 0, $393 = 0, $394 = 0, $395 = 0, $396 = 0, $397 = 0, $398 = 0, $399 = 0, $4 = 0, $40 = 0, $400 = 0, $401 = 0, $402 = 0, $403 = 0;
 var $404 = 0, $405 = 0, $406 = 0, $407 = 0, $408 = 0, $409 = 0, $41 = 0, $410 = 0, $411 = 0, $412 = 0, $413 = 0, $414 = 0, $415 = 0, $416 = 0, $417 = 0, $418 = 0, $419 = 0, $42 = 0, $420 = 0, $421 = 0;
 var $422 = 0, $423 = 0, $424 = 0, $425 = 0, $426 = 0, $427 = 0, $428 = 0, $429 = 0, $43 = 0, $430 = 0, $431 = 0, $432 = 0, $433 = 0, $434 = 0, $435 = 0, $436 = 0, $437 = 0, $438 = 0, $439 = 0, $44 = 0;
 var $440 = 0, $441 = 0, $442 = 0, $443 = 0, $444 = 0, $445 = 0, $446 = 0, $447 = 0, $448 = 0, $449 = 0, $45 = 0, $450 = 0, $451 = 0, $452 = 0, $453 = 0, $454 = 0, $455 = 0, $456 = 0, $457 = 0, $458 = 0;
 var $459 = 0, $46 = 0, $460 = 0, $461 = 0, $462 = 0, $463 = 0, $464 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0;
 var $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0;
 var $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0;
 var $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $aData = 0, $bNoContent = 0, $closest = 0, $d2 = 0, $dist = 0, $eType = 0, $i = 0, $iNewTrunk = 0, $iPage = 0, $iTrunk = 0, $k = 0, $mxPage = 0, $n = 0, $noContent = 0, $or$cond = 0;
 var $pNewTrunk = 0, $pPage1 = 0, $pPg = 0, $pPrevTrunk = 0, $pTrunk = 0, $rc = 0, $searchList = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 112|0;
 $0 = sp + 92|0;
 $1 = sp + 88|0;
 $2 = sp + 84|0;
 $3 = sp + 80|0;
 $4 = sp + 76|0;
 $5 = sp + 98|0;
 $pPage1 = sp + 72|0;
 $rc = sp + 68|0;
 $n = sp + 64|0;
 $k = sp + 60|0;
 $pTrunk = sp + 56|0;
 $pPrevTrunk = sp + 52|0;
 $mxPage = sp + 48|0;
 $iTrunk = sp + 44|0;
 $searchList = sp + 97|0;
 $eType = sp + 96|0;
 $pNewTrunk = sp + 40|0;
 $iNewTrunk = sp + 36|0;
 $closest = sp + 32|0;
 $iPage = sp + 28|0;
 $aData = sp + 24|0;
 $i = sp + 20|0;
 $dist = sp + 16|0;
 $d2 = sp + 12|0;
 $noContent = sp + 8|0;
 $bNoContent = sp + 4|0;
 $pPg = sp;
 HEAP32[$1>>2] = $pBt;
 HEAP32[$2>>2] = $ppPage;
 HEAP32[$3>>2] = $pPgno;
 HEAP32[$4>>2] = $nearby;
 HEAP8[$5>>0] = $eMode;
 HEAP32[$pTrunk>>2] = 0;
 HEAP32[$pPrevTrunk>>2] = 0;
 $6 = HEAP32[$1>>2]|0;
 $7 = (($6) + 12|0);
 $8 = HEAP32[$7>>2]|0;
 HEAP32[$pPage1>>2] = $8;
 $9 = HEAP32[$1>>2]|0;
 $10 = (_btreePagecount($9)|0);
 HEAP32[$mxPage>>2] = $10;
 $11 = HEAP32[$pPage1>>2]|0;
 $12 = (($11) + 56|0);
 $13 = HEAP32[$12>>2]|0;
 $14 = (($13) + 36|0);
 $15 = (_sqlite3Get4byte($14)|0);
 HEAP32[$n>>2] = $15;
 $16 = HEAP32[$n>>2]|0;
 $17 = HEAP32[$mxPage>>2]|0;
 $18 = ($16>>>0)>=($17>>>0);
 if ($18) {
  $19 = (_sqlite3CorruptError(57337)|0);
  HEAP32[$0>>2] = $19;
  $464 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($464|0);
 }
 $20 = HEAP32[$n>>2]|0;
 $21 = ($20>>>0)>(0);
 L5: do {
  if ($21) {
   HEAP8[$searchList>>0] = 0;
   $22 = HEAP8[$5>>0]|0;
   $23 = $22&255;
   $24 = ($23|0)==(1);
   do {
    if ($24) {
     $25 = HEAP32[$4>>2]|0;
     $26 = HEAP32[$mxPage>>2]|0;
     $27 = ($25>>>0)<=($26>>>0);
     if ($27) {
      $28 = HEAP32[$1>>2]|0;
      $29 = HEAP32[$4>>2]|0;
      $30 = (_ptrmapGet($28,$29,$eType,0)|0);
      HEAP32[$rc>>2] = $30;
      $31 = HEAP32[$rc>>2]|0;
      $32 = ($31|0)!=(0);
      if ($32) {
       $33 = HEAP32[$rc>>2]|0;
       HEAP32[$0>>2] = $33;
       $464 = HEAP32[$0>>2]|0;
       STACKTOP = sp;return ($464|0);
      } else {
       $34 = HEAP8[$eType>>0]|0;
       $35 = $34&255;
       $36 = ($35|0)==(2);
       if (!($36)) {
        break;
       }
       HEAP8[$searchList>>0] = 1;
       break;
      }
     }
    } else {
     $37 = HEAP8[$5>>0]|0;
     $38 = $37&255;
     $39 = ($38|0)==(2);
     if ($39) {
      HEAP8[$searchList>>0] = 1;
     }
    }
   } while(0);
   $40 = HEAP32[$pPage1>>2]|0;
   $41 = (($40) + 68|0);
   $42 = HEAP32[$41>>2]|0;
   $43 = (_sqlite3PagerWrite($42)|0);
   HEAP32[$rc>>2] = $43;
   $44 = HEAP32[$rc>>2]|0;
   $45 = ($44|0)!=(0);
   if ($45) {
    $46 = HEAP32[$rc>>2]|0;
    HEAP32[$0>>2] = $46;
    $464 = HEAP32[$0>>2]|0;
    STACKTOP = sp;return ($464|0);
   }
   $47 = HEAP32[$pPage1>>2]|0;
   $48 = (($47) + 56|0);
   $49 = HEAP32[$48>>2]|0;
   $50 = (($49) + 36|0);
   $51 = HEAP32[$n>>2]|0;
   $52 = (($51) - 1)|0;
   _sqlite3Put4byte($50,$52);
   L22: while(1) {
    $53 = HEAP32[$pTrunk>>2]|0;
    HEAP32[$pPrevTrunk>>2] = $53;
    $54 = HEAP32[$pPrevTrunk>>2]|0;
    $55 = ($54|0)!=(0|0);
    if ($55) {
     $56 = HEAP32[$pPrevTrunk>>2]|0;
     $57 = (($56) + 56|0);
     $58 = HEAP32[$57>>2]|0;
     $59 = (_sqlite3Get4byte($58)|0);
     HEAP32[$iTrunk>>2] = $59;
    } else {
     $60 = HEAP32[$pPage1>>2]|0;
     $61 = (($60) + 56|0);
     $62 = HEAP32[$61>>2]|0;
     $63 = (($62) + 32|0);
     $64 = (_sqlite3Get4byte($63)|0);
     HEAP32[$iTrunk>>2] = $64;
    }
    $65 = HEAP32[$iTrunk>>2]|0;
    $66 = HEAP32[$mxPage>>2]|0;
    $67 = ($65>>>0)>($66>>>0);
    if ($67) {
     $68 = (_sqlite3CorruptError(57386)|0);
     HEAP32[$rc>>2] = $68;
    } else {
     $69 = HEAP32[$1>>2]|0;
     $70 = HEAP32[$iTrunk>>2]|0;
     $71 = (_btreeGetPage($69,$70,$pTrunk,0)|0);
     HEAP32[$rc>>2] = $71;
    }
    $72 = HEAP32[$rc>>2]|0;
    $73 = ($72|0)!=(0);
    if ($73) {
     label = 22;
     break;
    }
    $74 = HEAP32[$pTrunk>>2]|0;
    $75 = (($74) + 56|0);
    $76 = HEAP32[$75>>2]|0;
    $77 = (($76) + 4|0);
    $78 = (_sqlite3Get4byte($77)|0);
    HEAP32[$k>>2] = $78;
    $79 = HEAP32[$k>>2]|0;
    $80 = ($79|0)!=(0);
    $81 = HEAP8[$searchList>>0]|0;
    $82 = ($81<<24>>24)!=(0);
    $or$cond = $80 | $82;
    L33: do {
     if ($or$cond) {
      $100 = HEAP32[$k>>2]|0;
      $101 = HEAP32[$1>>2]|0;
      $102 = (($101) + 36|0);
      $103 = HEAP32[$102>>2]|0;
      $104 = (($103>>>0) / 4)&-1;
      $105 = (($104) - 2)|0;
      $106 = ($100>>>0)>($105>>>0);
      if ($106) {
       label = 27;
       break L22;
      }
      $108 = HEAP8[$searchList>>0]|0;
      $109 = $108&255;
      $110 = ($109|0)!=(0);
      do {
       if ($110) {
        $111 = HEAP32[$4>>2]|0;
        $112 = HEAP32[$iTrunk>>2]|0;
        $113 = ($111|0)==($112|0);
        if (!($113)) {
         $114 = HEAP32[$iTrunk>>2]|0;
         $115 = HEAP32[$4>>2]|0;
         $116 = ($114>>>0)<($115>>>0);
         if (!($116)) {
          break;
         }
         $117 = HEAP8[$5>>0]|0;
         $118 = $117&255;
         $119 = ($118|0)==(2);
         if (!($119)) {
          break;
         }
        }
        $120 = HEAP32[$iTrunk>>2]|0;
        $121 = HEAP32[$3>>2]|0;
        HEAP32[$121>>2] = $120;
        $122 = HEAP32[$pTrunk>>2]|0;
        $123 = HEAP32[$2>>2]|0;
        HEAP32[$123>>2] = $122;
        HEAP8[$searchList>>0] = 0;
        $124 = HEAP32[$pTrunk>>2]|0;
        $125 = (($124) + 68|0);
        $126 = HEAP32[$125>>2]|0;
        $127 = (_sqlite3PagerWrite($126)|0);
        HEAP32[$rc>>2] = $127;
        $128 = HEAP32[$rc>>2]|0;
        $129 = ($128|0)!=(0);
        if ($129) {
         break L5;
        }
        $130 = HEAP32[$k>>2]|0;
        $131 = ($130|0)==(0);
        do {
         if ($131) {
          $132 = HEAP32[$pPrevTrunk>>2]|0;
          $133 = ($132|0)!=(0|0);
          if (!($133)) {
           $134 = HEAP32[$pPage1>>2]|0;
           $135 = (($134) + 56|0);
           $136 = HEAP32[$135>>2]|0;
           $137 = (($136) + 32|0);
           $138 = HEAP32[$pTrunk>>2]|0;
           $139 = (($138) + 56|0);
           $140 = HEAP32[$139>>2]|0;
           ;HEAP8[$137+0>>0]=HEAP8[$140+0>>0]|0;HEAP8[$137+1>>0]=HEAP8[$140+1>>0]|0;HEAP8[$137+2>>0]=HEAP8[$140+2>>0]|0;HEAP8[$137+3>>0]=HEAP8[$140+3>>0]|0;
           break;
          }
          $141 = HEAP32[$pPrevTrunk>>2]|0;
          $142 = (($141) + 68|0);
          $143 = HEAP32[$142>>2]|0;
          $144 = (_sqlite3PagerWrite($143)|0);
          HEAP32[$rc>>2] = $144;
          $145 = HEAP32[$rc>>2]|0;
          $146 = ($145|0)!=(0);
          if ($146) {
           break L5;
          }
          $147 = HEAP32[$pPrevTrunk>>2]|0;
          $148 = (($147) + 56|0);
          $149 = HEAP32[$148>>2]|0;
          $150 = HEAP32[$pTrunk>>2]|0;
          $151 = (($150) + 56|0);
          $152 = HEAP32[$151>>2]|0;
          ;HEAP8[$149+0>>0]=HEAP8[$152+0>>0]|0;HEAP8[$149+1>>0]=HEAP8[$152+1>>0]|0;HEAP8[$149+2>>0]=HEAP8[$152+2>>0]|0;HEAP8[$149+3>>0]=HEAP8[$152+3>>0]|0;
         } else {
          $153 = HEAP32[$pTrunk>>2]|0;
          $154 = (($153) + 56|0);
          $155 = HEAP32[$154>>2]|0;
          $156 = (($155) + 8|0);
          $157 = (_sqlite3Get4byte($156)|0);
          HEAP32[$iNewTrunk>>2] = $157;
          $158 = HEAP32[$iNewTrunk>>2]|0;
          $159 = HEAP32[$mxPage>>2]|0;
          $160 = ($158>>>0)>($159>>>0);
          if ($160) {
           label = 39;
           break L22;
          }
          $162 = HEAP32[$1>>2]|0;
          $163 = HEAP32[$iNewTrunk>>2]|0;
          $164 = (_btreeGetPage($162,$163,$pNewTrunk,0)|0);
          HEAP32[$rc>>2] = $164;
          $165 = HEAP32[$rc>>2]|0;
          $166 = ($165|0)!=(0);
          if ($166) {
           break L5;
          }
          $167 = HEAP32[$pNewTrunk>>2]|0;
          $168 = (($167) + 68|0);
          $169 = HEAP32[$168>>2]|0;
          $170 = (_sqlite3PagerWrite($169)|0);
          HEAP32[$rc>>2] = $170;
          $171 = HEAP32[$rc>>2]|0;
          $172 = ($171|0)!=(0);
          $173 = HEAP32[$pNewTrunk>>2]|0;
          if ($172) {
           label = 42;
           break L22;
          }
          $174 = (($173) + 56|0);
          $175 = HEAP32[$174>>2]|0;
          $176 = HEAP32[$pTrunk>>2]|0;
          $177 = (($176) + 56|0);
          $178 = HEAP32[$177>>2]|0;
          ;HEAP8[$175+0>>0]=HEAP8[$178+0>>0]|0;HEAP8[$175+1>>0]=HEAP8[$178+1>>0]|0;HEAP8[$175+2>>0]=HEAP8[$178+2>>0]|0;HEAP8[$175+3>>0]=HEAP8[$178+3>>0]|0;
          $179 = HEAP32[$pNewTrunk>>2]|0;
          $180 = (($179) + 56|0);
          $181 = HEAP32[$180>>2]|0;
          $182 = (($181) + 4|0);
          $183 = HEAP32[$k>>2]|0;
          $184 = (($183) - 1)|0;
          _sqlite3Put4byte($182,$184);
          $185 = HEAP32[$pNewTrunk>>2]|0;
          $186 = (($185) + 56|0);
          $187 = HEAP32[$186>>2]|0;
          $188 = (($187) + 8|0);
          $189 = HEAP32[$pTrunk>>2]|0;
          $190 = (($189) + 56|0);
          $191 = HEAP32[$190>>2]|0;
          $192 = (($191) + 12|0);
          $193 = HEAP32[$k>>2]|0;
          $194 = (($193) - 1)|0;
          $195 = $194<<2;
          _memcpy(($188|0),($192|0),($195|0))|0;
          $196 = HEAP32[$pNewTrunk>>2]|0;
          _releasePage($196);
          $197 = HEAP32[$pPrevTrunk>>2]|0;
          $198 = ($197|0)!=(0|0);
          if (!($198)) {
           $199 = HEAP32[$pPage1>>2]|0;
           $200 = (($199) + 56|0);
           $201 = HEAP32[$200>>2]|0;
           $202 = (($201) + 32|0);
           $203 = HEAP32[$iNewTrunk>>2]|0;
           _sqlite3Put4byte($202,$203);
           break;
          }
          $204 = HEAP32[$pPrevTrunk>>2]|0;
          $205 = (($204) + 68|0);
          $206 = HEAP32[$205>>2]|0;
          $207 = (_sqlite3PagerWrite($206)|0);
          HEAP32[$rc>>2] = $207;
          $208 = HEAP32[$rc>>2]|0;
          $209 = ($208|0)!=(0);
          if ($209) {
           break L5;
          }
          $210 = HEAP32[$pPrevTrunk>>2]|0;
          $211 = (($210) + 56|0);
          $212 = HEAP32[$211>>2]|0;
          $213 = HEAP32[$iNewTrunk>>2]|0;
          _sqlite3Put4byte($212,$213);
         }
        } while(0);
        HEAP32[$pTrunk>>2] = 0;
        break L33;
       }
      } while(0);
      $214 = HEAP32[$k>>2]|0;
      $215 = ($214>>>0)>(0);
      if ($215) {
       $216 = HEAP32[$pTrunk>>2]|0;
       $217 = (($216) + 56|0);
       $218 = HEAP32[$217>>2]|0;
       HEAP32[$aData>>2] = $218;
       $219 = HEAP32[$4>>2]|0;
       $220 = ($219>>>0)>(0);
       HEAP32[$closest>>2] = 0;
       L60: do {
        if ($220) {
         $221 = HEAP8[$5>>0]|0;
         $222 = $221&255;
         $223 = ($222|0)==(2);
         if ($223) {
          HEAP32[$i>>2] = 0;
          while(1) {
           $224 = HEAP32[$i>>2]|0;
           $225 = HEAP32[$k>>2]|0;
           $226 = ($224>>>0)<($225>>>0);
           if (!($226)) {
            break L60;
           }
           $227 = HEAP32[$i>>2]|0;
           $228 = $227<<2;
           $229 = (8 + ($228))|0;
           $230 = HEAP32[$aData>>2]|0;
           $231 = (($230) + ($229)|0);
           $232 = (_sqlite3Get4byte($231)|0);
           HEAP32[$iPage>>2] = $232;
           $233 = HEAP32[$iPage>>2]|0;
           $234 = HEAP32[$4>>2]|0;
           $235 = ($233>>>0)<=($234>>>0);
           $236 = HEAP32[$i>>2]|0;
           if ($235) {
            break;
           }
           $237 = (($236) + 1)|0;
           HEAP32[$i>>2] = $237;
          }
          HEAP32[$closest>>2] = $236;
          break;
         } else {
          $238 = HEAP32[$aData>>2]|0;
          $239 = (($238) + 8|0);
          $240 = (_sqlite3Get4byte($239)|0);
          $241 = HEAP32[$4>>2]|0;
          $242 = (($240) - ($241))|0;
          $243 = (_sqlite3AbsInt32($242)|0);
          HEAP32[$dist>>2] = $243;
          HEAP32[$i>>2] = 1;
          while(1) {
           $244 = HEAP32[$i>>2]|0;
           $245 = HEAP32[$k>>2]|0;
           $246 = ($244>>>0)<($245>>>0);
           if (!($246)) {
            break L60;
           }
           $247 = HEAP32[$i>>2]|0;
           $248 = $247<<2;
           $249 = (8 + ($248))|0;
           $250 = HEAP32[$aData>>2]|0;
           $251 = (($250) + ($249)|0);
           $252 = (_sqlite3Get4byte($251)|0);
           $253 = HEAP32[$4>>2]|0;
           $254 = (($252) - ($253))|0;
           $255 = (_sqlite3AbsInt32($254)|0);
           HEAP32[$d2>>2] = $255;
           $256 = HEAP32[$d2>>2]|0;
           $257 = HEAP32[$dist>>2]|0;
           $258 = ($256|0)<($257|0);
           if ($258) {
            $259 = HEAP32[$i>>2]|0;
            HEAP32[$closest>>2] = $259;
            $260 = HEAP32[$d2>>2]|0;
            HEAP32[$dist>>2] = $260;
           }
           $261 = HEAP32[$i>>2]|0;
           $262 = (($261) + 1)|0;
           HEAP32[$i>>2] = $262;
          }
         }
        }
       } while(0);
       $263 = HEAP32[$closest>>2]|0;
       $264 = $263<<2;
       $265 = (8 + ($264))|0;
       $266 = HEAP32[$aData>>2]|0;
       $267 = (($266) + ($265)|0);
       $268 = (_sqlite3Get4byte($267)|0);
       HEAP32[$iPage>>2] = $268;
       $269 = HEAP32[$iPage>>2]|0;
       $270 = HEAP32[$mxPage>>2]|0;
       $271 = ($269>>>0)>($270>>>0);
       if ($271) {
        label = 62;
        break L22;
       }
       $273 = HEAP8[$searchList>>0]|0;
       $274 = ($273<<24>>24)!=(0);
       if ($274) {
        $275 = HEAP32[$iPage>>2]|0;
        $276 = HEAP32[$4>>2]|0;
        $277 = ($275|0)==($276|0);
        if (!($277)) {
         $278 = HEAP32[$iPage>>2]|0;
         $279 = HEAP32[$4>>2]|0;
         $280 = ($278>>>0)<($279>>>0);
         if (!($280)) {
          break;
         }
         $281 = HEAP8[$5>>0]|0;
         $282 = $281&255;
         $283 = ($282|0)==(2);
         if (!($283)) {
          break;
         }
        }
       }
       $284 = HEAP32[$iPage>>2]|0;
       $285 = HEAP32[$3>>2]|0;
       HEAP32[$285>>2] = $284;
       $286 = HEAP32[$pTrunk>>2]|0;
       $287 = (($286) + 68|0);
       $288 = HEAP32[$287>>2]|0;
       $289 = (_sqlite3PagerWrite($288)|0);
       HEAP32[$rc>>2] = $289;
       $290 = HEAP32[$rc>>2]|0;
       $291 = ($290|0)!=(0);
       if ($291) {
        break L5;
       }
       $292 = HEAP32[$closest>>2]|0;
       $293 = HEAP32[$k>>2]|0;
       $294 = (($293) - 1)|0;
       $295 = ($292>>>0)<($294>>>0);
       if ($295) {
        $296 = HEAP32[$closest>>2]|0;
        $297 = $296<<2;
        $298 = (8 + ($297))|0;
        $299 = HEAP32[$aData>>2]|0;
        $300 = (($299) + ($298)|0);
        $301 = HEAP32[$k>>2]|0;
        $302 = $301<<2;
        $303 = (4 + ($302))|0;
        $304 = HEAP32[$aData>>2]|0;
        $305 = (($304) + ($303)|0);
        ;HEAP8[$300+0>>0]=HEAP8[$305+0>>0]|0;HEAP8[$300+1>>0]=HEAP8[$305+1>>0]|0;HEAP8[$300+2>>0]=HEAP8[$305+2>>0]|0;HEAP8[$300+3>>0]=HEAP8[$305+3>>0]|0;
       }
       $306 = HEAP32[$aData>>2]|0;
       $307 = (($306) + 4|0);
       $308 = HEAP32[$k>>2]|0;
       $309 = (($308) - 1)|0;
       _sqlite3Put4byte($307,$309);
       $310 = HEAP32[$1>>2]|0;
       $311 = HEAP32[$3>>2]|0;
       $312 = HEAP32[$311>>2]|0;
       $313 = (_btreeGetHasContent($310,$312)|0);
       $314 = ($313|0)!=(0);
       $315 = $314 ^ 1;
       $316 = $315 ? 1 : 0;
       HEAP32[$noContent>>2] = $316;
       $317 = HEAP32[$1>>2]|0;
       $318 = HEAP32[$3>>2]|0;
       $319 = HEAP32[$318>>2]|0;
       $320 = HEAP32[$2>>2]|0;
       $321 = HEAP32[$noContent>>2]|0;
       $322 = (_btreeGetPage($317,$319,$320,$321)|0);
       HEAP32[$rc>>2] = $322;
       $323 = HEAP32[$rc>>2]|0;
       $324 = ($323|0)==(0);
       do {
        if ($324) {
         $325 = HEAP32[$2>>2]|0;
         $326 = HEAP32[$325>>2]|0;
         $327 = (($326) + 68|0);
         $328 = HEAP32[$327>>2]|0;
         $329 = (_sqlite3PagerWrite($328)|0);
         HEAP32[$rc>>2] = $329;
         $330 = HEAP32[$rc>>2]|0;
         $331 = ($330|0)!=(0);
         if (!($331)) {
          break;
         }
         $332 = HEAP32[$2>>2]|0;
         $333 = HEAP32[$332>>2]|0;
         _releasePage($333);
        }
       } while(0);
       HEAP8[$searchList>>0] = 0;
      }
     } else {
      $83 = HEAP32[$pTrunk>>2]|0;
      $84 = (($83) + 68|0);
      $85 = HEAP32[$84>>2]|0;
      $86 = (_sqlite3PagerWrite($85)|0);
      HEAP32[$rc>>2] = $86;
      $87 = HEAP32[$rc>>2]|0;
      $88 = ($87|0)!=(0);
      if ($88) {
       break L5;
      }
      $89 = HEAP32[$iTrunk>>2]|0;
      $90 = HEAP32[$3>>2]|0;
      HEAP32[$90>>2] = $89;
      $91 = HEAP32[$pPage1>>2]|0;
      $92 = (($91) + 56|0);
      $93 = HEAP32[$92>>2]|0;
      $94 = (($93) + 32|0);
      $95 = HEAP32[$pTrunk>>2]|0;
      $96 = (($95) + 56|0);
      $97 = HEAP32[$96>>2]|0;
      ;HEAP8[$94+0>>0]=HEAP8[$97+0>>0]|0;HEAP8[$94+1>>0]=HEAP8[$97+1>>0]|0;HEAP8[$94+2>>0]=HEAP8[$97+2>>0]|0;HEAP8[$94+3>>0]=HEAP8[$97+3>>0]|0;
      $98 = HEAP32[$pTrunk>>2]|0;
      $99 = HEAP32[$2>>2]|0;
      HEAP32[$99>>2] = $98;
      HEAP32[$pTrunk>>2] = 0;
     }
    } while(0);
    $334 = HEAP32[$pPrevTrunk>>2]|0;
    _releasePage($334);
    HEAP32[$pPrevTrunk>>2] = 0;
    $335 = HEAP8[$searchList>>0]|0;
    $336 = ($335<<24>>24)!=(0);
    if (!($336)) {
     break L5;
    }
   }
   if ((label|0) == 22) {
    HEAP32[$pTrunk>>2] = 0;
    break;
   }
   else if ((label|0) == 27) {
    $107 = (_sqlite3CorruptError(57414)|0);
    HEAP32[$rc>>2] = $107;
    break;
   }
   else if ((label|0) == 39) {
    $161 = (_sqlite3CorruptError(57448)|0);
    HEAP32[$rc>>2] = $161;
    break;
   }
   else if ((label|0) == 42) {
    _releasePage($173);
    break;
   }
   else if ((label|0) == 62) {
    $272 = (_sqlite3CorruptError(57513)|0);
    HEAP32[$rc>>2] = $272;
    break;
   }
  } else {
   $337 = HEAP32[$1>>2]|0;
   $338 = (($337) + 19|0);
   $339 = HEAP8[$338>>0]|0;
   $340 = $339&255;
   $341 = (0)==($340|0);
   $342 = $341 ? 1 : 0;
   HEAP32[$bNoContent>>2] = $342;
   $343 = HEAP32[$1>>2]|0;
   $344 = (($343) + 12|0);
   $345 = HEAP32[$344>>2]|0;
   $346 = (($345) + 68|0);
   $347 = HEAP32[$346>>2]|0;
   $348 = (_sqlite3PagerWrite($347)|0);
   HEAP32[$rc>>2] = $348;
   $349 = HEAP32[$rc>>2]|0;
   $350 = ($349|0)!=(0);
   if ($350) {
    $351 = HEAP32[$rc>>2]|0;
    HEAP32[$0>>2] = $351;
    $464 = HEAP32[$0>>2]|0;
    STACKTOP = sp;return ($464|0);
   }
   $352 = HEAP32[$1>>2]|0;
   $353 = (($352) + 44|0);
   $354 = HEAP32[$353>>2]|0;
   $355 = (($354) + 1)|0;
   HEAP32[$353>>2] = $355;
   $356 = HEAP32[$1>>2]|0;
   $357 = (($356) + 44|0);
   $358 = HEAP32[$357>>2]|0;
   $359 = HEAP32[1128>>2]|0;
   $360 = HEAP32[$1>>2]|0;
   $361 = (($360) + 32|0);
   $362 = HEAP32[$361>>2]|0;
   $363 = (($359>>>0) / ($362>>>0))&-1;
   $364 = (($363) + 1)|0;
   $365 = ($358|0)==($364|0);
   if ($365) {
    $366 = HEAP32[$1>>2]|0;
    $367 = (($366) + 44|0);
    $368 = HEAP32[$367>>2]|0;
    $369 = (($368) + 1)|0;
    HEAP32[$367>>2] = $369;
   }
   $370 = HEAP32[$1>>2]|0;
   $371 = (($370) + 17|0);
   $372 = HEAP8[$371>>0]|0;
   $373 = $372&255;
   $374 = ($373|0)!=(0);
   do {
    if ($374) {
     $375 = HEAP32[$1>>2]|0;
     $376 = HEAP32[$1>>2]|0;
     $377 = (($376) + 44|0);
     $378 = HEAP32[$377>>2]|0;
     $379 = (_ptrmapPageno($375,$378)|0);
     $380 = HEAP32[$1>>2]|0;
     $381 = (($380) + 44|0);
     $382 = HEAP32[$381>>2]|0;
     $383 = ($379|0)==($382|0);
     if ($383) {
      HEAP32[$pPg>>2] = 0;
      $384 = HEAP32[$1>>2]|0;
      $385 = HEAP32[$1>>2]|0;
      $386 = (($385) + 44|0);
      $387 = HEAP32[$386>>2]|0;
      $388 = HEAP32[$bNoContent>>2]|0;
      $389 = (_btreeGetPage($384,$387,$pPg,$388)|0);
      HEAP32[$rc>>2] = $389;
      $390 = HEAP32[$rc>>2]|0;
      $391 = ($390|0)==(0);
      if ($391) {
       $392 = HEAP32[$pPg>>2]|0;
       $393 = (($392) + 68|0);
       $394 = HEAP32[$393>>2]|0;
       $395 = (_sqlite3PagerWrite($394)|0);
       HEAP32[$rc>>2] = $395;
       $396 = HEAP32[$pPg>>2]|0;
       _releasePage($396);
      }
      $397 = HEAP32[$rc>>2]|0;
      $398 = ($397|0)!=(0);
      if ($398) {
       $399 = HEAP32[$rc>>2]|0;
       HEAP32[$0>>2] = $399;
       $464 = HEAP32[$0>>2]|0;
       STACKTOP = sp;return ($464|0);
      } else {
       $400 = HEAP32[$1>>2]|0;
       $401 = (($400) + 44|0);
       $402 = HEAP32[$401>>2]|0;
       $403 = (($402) + 1)|0;
       HEAP32[$401>>2] = $403;
       $404 = HEAP32[$1>>2]|0;
       $405 = (($404) + 44|0);
       $406 = HEAP32[$405>>2]|0;
       $407 = HEAP32[1128>>2]|0;
       $408 = HEAP32[$1>>2]|0;
       $409 = (($408) + 32|0);
       $410 = HEAP32[$409>>2]|0;
       $411 = (($407>>>0) / ($410>>>0))&-1;
       $412 = (($411) + 1)|0;
       $413 = ($406|0)==($412|0);
       if (!($413)) {
        break;
       }
       $414 = HEAP32[$1>>2]|0;
       $415 = (($414) + 44|0);
       $416 = HEAP32[$415>>2]|0;
       $417 = (($416) + 1)|0;
       HEAP32[$415>>2] = $417;
       break;
      }
     }
    }
   } while(0);
   $418 = HEAP32[$1>>2]|0;
   $419 = (($418) + 12|0);
   $420 = HEAP32[$419>>2]|0;
   $421 = (($420) + 56|0);
   $422 = HEAP32[$421>>2]|0;
   $423 = (($422) + 28|0);
   $424 = HEAP32[$1>>2]|0;
   $425 = (($424) + 44|0);
   $426 = HEAP32[$425>>2]|0;
   _sqlite3Put4byte($423,$426);
   $427 = HEAP32[$1>>2]|0;
   $428 = (($427) + 44|0);
   $429 = HEAP32[$428>>2]|0;
   $430 = HEAP32[$3>>2]|0;
   HEAP32[$430>>2] = $429;
   $431 = HEAP32[$1>>2]|0;
   $432 = HEAP32[$3>>2]|0;
   $433 = HEAP32[$432>>2]|0;
   $434 = HEAP32[$2>>2]|0;
   $435 = HEAP32[$bNoContent>>2]|0;
   $436 = (_btreeGetPage($431,$433,$434,$435)|0);
   HEAP32[$rc>>2] = $436;
   $437 = HEAP32[$rc>>2]|0;
   $438 = ($437|0)!=(0);
   if ($438) {
    $439 = HEAP32[$rc>>2]|0;
    HEAP32[$0>>2] = $439;
    $464 = HEAP32[$0>>2]|0;
    STACKTOP = sp;return ($464|0);
   } else {
    $440 = HEAP32[$2>>2]|0;
    $441 = HEAP32[$440>>2]|0;
    $442 = (($441) + 68|0);
    $443 = HEAP32[$442>>2]|0;
    $444 = (_sqlite3PagerWrite($443)|0);
    HEAP32[$rc>>2] = $444;
    $445 = HEAP32[$rc>>2]|0;
    $446 = ($445|0)!=(0);
    if (!($446)) {
     break;
    }
    $447 = HEAP32[$2>>2]|0;
    $448 = HEAP32[$447>>2]|0;
    _releasePage($448);
    break;
   }
  }
 } while(0);
 $449 = HEAP32[$pTrunk>>2]|0;
 _releasePage($449);
 $450 = HEAP32[$pPrevTrunk>>2]|0;
 _releasePage($450);
 $451 = HEAP32[$rc>>2]|0;
 $452 = ($451|0)==(0);
 $453 = HEAP32[$2>>2]|0;
 do {
  if ($452) {
   $454 = HEAP32[$453>>2]|0;
   $455 = (($454) + 68|0);
   $456 = HEAP32[$455>>2]|0;
   $457 = (_sqlite3PagerPageRefcount($456)|0);
   $458 = ($457|0)>(1);
   $459 = HEAP32[$2>>2]|0;
   $460 = HEAP32[$459>>2]|0;
   if (!($458)) {
    HEAP8[$460>>0] = 0;
    break;
   }
   _releasePage($460);
   $461 = HEAP32[$2>>2]|0;
   HEAP32[$461>>2] = 0;
   $462 = (_sqlite3CorruptError(57612)|0);
   HEAP32[$0>>2] = $462;
   $464 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($464|0);
  } else {
   HEAP32[$453>>2] = 0;
  }
 } while(0);
 $463 = HEAP32[$rc>>2]|0;
 HEAP32[$0>>2] = $463;
 $464 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($464|0);
}
function _relocatePage($pBt,$pDbPage,$eType,$iPtrPage,$iFreePage,$isCommit) {
 $pBt = $pBt|0;
 $pDbPage = $pDbPage|0;
 $eType = $eType|0;
 $iPtrPage = $iPtrPage|0;
 $iFreePage = $iFreePage|0;
 $isCommit = $isCommit|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $8 = 0, $9 = 0, $iDbPage = 0, $nextOvfl = 0;
 var $pPager = 0, $pPtrPage = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $0 = sp + 40|0;
 $1 = sp + 36|0;
 $2 = sp + 32|0;
 $3 = sp + 44|0;
 $4 = sp + 28|0;
 $5 = sp + 24|0;
 $6 = sp + 20|0;
 $pPtrPage = sp + 16|0;
 $iDbPage = sp + 12|0;
 $pPager = sp + 8|0;
 $rc = sp + 4|0;
 $nextOvfl = sp;
 HEAP32[$1>>2] = $pBt;
 HEAP32[$2>>2] = $pDbPage;
 HEAP8[$3>>0] = $eType;
 HEAP32[$4>>2] = $iPtrPage;
 HEAP32[$5>>2] = $iFreePage;
 HEAP32[$6>>2] = $isCommit;
 $7 = HEAP32[$2>>2]|0;
 $8 = (($7) + 72|0);
 $9 = HEAP32[$8>>2]|0;
 HEAP32[$iDbPage>>2] = $9;
 $10 = HEAP32[$1>>2]|0;
 $11 = HEAP32[$10>>2]|0;
 HEAP32[$pPager>>2] = $11;
 $12 = HEAP32[$pPager>>2]|0;
 $13 = HEAP32[$2>>2]|0;
 $14 = (($13) + 68|0);
 $15 = HEAP32[$14>>2]|0;
 $16 = HEAP32[$5>>2]|0;
 $17 = HEAP32[$6>>2]|0;
 $18 = (_sqlite3PagerMovepage($12,$15,$16,$17)|0);
 HEAP32[$rc>>2] = $18;
 $19 = HEAP32[$rc>>2]|0;
 $20 = ($19|0)!=(0);
 if ($20) {
  $21 = HEAP32[$rc>>2]|0;
  HEAP32[$0>>2] = $21;
  $77 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($77|0);
 }
 $22 = HEAP32[$5>>2]|0;
 $23 = HEAP32[$2>>2]|0;
 $24 = (($23) + 72|0);
 HEAP32[$24>>2] = $22;
 $25 = HEAP8[$3>>0]|0;
 $26 = $25&255;
 $27 = ($26|0)==(5);
 if ($27) {
  label = 5;
 } else {
  $28 = HEAP8[$3>>0]|0;
  $29 = $28&255;
  $30 = ($29|0)==(1);
  if ($30) {
   label = 5;
  } else {
   $36 = HEAP32[$2>>2]|0;
   $37 = (($36) + 56|0);
   $38 = HEAP32[$37>>2]|0;
   $39 = (_sqlite3Get4byte($38)|0);
   HEAP32[$nextOvfl>>2] = $39;
   $40 = HEAP32[$nextOvfl>>2]|0;
   $41 = ($40|0)!=(0);
   if ($41) {
    $42 = HEAP32[$1>>2]|0;
    $43 = HEAP32[$nextOvfl>>2]|0;
    $44 = HEAP32[$5>>2]|0;
    _ptrmapPut($42,$43,4,$44,$rc);
    $45 = HEAP32[$rc>>2]|0;
    $46 = ($45|0)!=(0);
    if ($46) {
     $47 = HEAP32[$rc>>2]|0;
     HEAP32[$0>>2] = $47;
     $77 = HEAP32[$0>>2]|0;
     STACKTOP = sp;return ($77|0);
    }
   }
  }
 }
 if ((label|0) == 5) {
  $31 = HEAP32[$2>>2]|0;
  $32 = (_setChildPtrmaps($31)|0);
  HEAP32[$rc>>2] = $32;
  $33 = HEAP32[$rc>>2]|0;
  $34 = ($33|0)!=(0);
  if ($34) {
   $35 = HEAP32[$rc>>2]|0;
   HEAP32[$0>>2] = $35;
   $77 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($77|0);
  }
 }
 $48 = HEAP8[$3>>0]|0;
 $49 = $48&255;
 $50 = ($49|0)!=(1);
 do {
  if ($50) {
   $51 = HEAP32[$1>>2]|0;
   $52 = HEAP32[$4>>2]|0;
   $53 = (_btreeGetPage($51,$52,$pPtrPage,0)|0);
   HEAP32[$rc>>2] = $53;
   $54 = HEAP32[$rc>>2]|0;
   $55 = ($54|0)!=(0);
   if ($55) {
    $56 = HEAP32[$rc>>2]|0;
    HEAP32[$0>>2] = $56;
    $77 = HEAP32[$0>>2]|0;
    STACKTOP = sp;return ($77|0);
   }
   $57 = HEAP32[$pPtrPage>>2]|0;
   $58 = (($57) + 68|0);
   $59 = HEAP32[$58>>2]|0;
   $60 = (_sqlite3PagerWrite($59)|0);
   HEAP32[$rc>>2] = $60;
   $61 = HEAP32[$rc>>2]|0;
   $62 = ($61|0)!=(0);
   $63 = HEAP32[$pPtrPage>>2]|0;
   if ($62) {
    _releasePage($63);
    $64 = HEAP32[$rc>>2]|0;
    HEAP32[$0>>2] = $64;
    $77 = HEAP32[$0>>2]|0;
    STACKTOP = sp;return ($77|0);
   } else {
    $65 = HEAP32[$iDbPage>>2]|0;
    $66 = HEAP32[$5>>2]|0;
    $67 = HEAP8[$3>>0]|0;
    $68 = (_modifyPagePointer($63,$65,$66,$67)|0);
    HEAP32[$rc>>2] = $68;
    $69 = HEAP32[$pPtrPage>>2]|0;
    _releasePage($69);
    $70 = HEAP32[$rc>>2]|0;
    $71 = ($70|0)==(0);
    if (!($71)) {
     break;
    }
    $72 = HEAP32[$1>>2]|0;
    $73 = HEAP32[$5>>2]|0;
    $74 = HEAP8[$3>>0]|0;
    $75 = HEAP32[$4>>2]|0;
    _ptrmapPut($72,$73,$74,$75,$rc);
    break;
   }
  }
 } while(0);
 $76 = HEAP32[$rc>>2]|0;
 HEAP32[$0>>2] = $76;
 $77 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($77|0);
}
function _sqlite3PagerMovepage($pPager,$pPg,$pgno,$isCommit) {
 $pPager = $pPager|0;
 $pPg = $pPg|0;
 $pgno = $pgno|0;
 $isCommit = $isCommit|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0;
 var $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0;
 var $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0;
 var $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0;
 var $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0;
 var $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $needSyncPgno = 0, $or$cond = 0, $origPgno = 0, $pPgHdr = 0, $pPgOld = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $0 = sp + 36|0;
 $1 = sp + 32|0;
 $2 = sp + 28|0;
 $3 = sp + 24|0;
 $4 = sp + 20|0;
 $pPgOld = sp + 16|0;
 $needSyncPgno = sp + 12|0;
 $rc = sp + 8|0;
 $origPgno = sp + 4|0;
 $pPgHdr = sp;
 HEAP32[$1>>2] = $pPager;
 HEAP32[$2>>2] = $pPg;
 HEAP32[$3>>2] = $pgno;
 HEAP32[$4>>2] = $isCommit;
 HEAP32[$needSyncPgno>>2] = 0;
 $5 = HEAP32[$1>>2]|0;
 $6 = (($5) + 15|0);
 $7 = HEAP8[$6>>0]|0;
 $8 = ($7<<24>>24)!=(0);
 if ($8) {
  $9 = HEAP32[$2>>2]|0;
  $10 = (_sqlite3PagerWrite($9)|0);
  HEAP32[$rc>>2] = $10;
  $11 = HEAP32[$rc>>2]|0;
  $12 = ($11|0)!=(0);
  if ($12) {
   $13 = HEAP32[$rc>>2]|0;
   HEAP32[$0>>2] = $13;
   $110 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($110|0);
  }
 }
 $14 = HEAP32[$2>>2]|0;
 $15 = (($14) + 24|0);
 $16 = HEAP16[$15>>1]|0;
 $17 = $16&65535;
 $18 = $17 & 2;
 $19 = ($18|0)!=(0);
 if ($19) {
  $20 = HEAP32[$2>>2]|0;
  $21 = (_subjRequiresPage($20)|0);
  $22 = ($21|0)!=(0);
  if ($22) {
   $23 = HEAP32[$2>>2]|0;
   $24 = (_subjournalPage($23)|0);
   HEAP32[$rc>>2] = $24;
   $25 = (0)!=($24|0);
   if ($25) {
    $26 = HEAP32[$rc>>2]|0;
    HEAP32[$0>>2] = $26;
    $110 = HEAP32[$0>>2]|0;
    STACKTOP = sp;return ($110|0);
   }
  }
 }
 $27 = HEAP32[$2>>2]|0;
 $28 = (($27) + 24|0);
 $29 = HEAP16[$28>>1]|0;
 $30 = $29&65535;
 $31 = $30 & 4;
 $32 = ($31|0)==(0);
 $33 = HEAP32[$4>>2]|0;
 $34 = ($33|0)!=(0);
 $or$cond = $32 | $34;
 if (!($or$cond)) {
  $35 = HEAP32[$2>>2]|0;
  $36 = (($35) + 20|0);
  $37 = HEAP32[$36>>2]|0;
  HEAP32[$needSyncPgno>>2] = $37;
 }
 $38 = HEAP32[$2>>2]|0;
 $39 = (($38) + 24|0);
 $40 = HEAP16[$39>>1]|0;
 $41 = $40&65535;
 $42 = $41 & -5;
 $43 = $42&65535;
 HEAP16[$39>>1] = $43;
 $44 = HEAP32[$1>>2]|0;
 $45 = HEAP32[$3>>2]|0;
 $46 = (_sqlite3PagerLookup($44,$45)|0);
 HEAP32[$pPgOld>>2] = $46;
 $47 = HEAP32[$pPgOld>>2]|0;
 $48 = ($47|0)!=(0|0);
 do {
  if ($48) {
   $49 = HEAP32[$pPgOld>>2]|0;
   $50 = (($49) + 24|0);
   $51 = HEAP16[$50>>1]|0;
   $52 = $51&65535;
   $53 = $52 & 4;
   $54 = HEAP32[$2>>2]|0;
   $55 = (($54) + 24|0);
   $56 = HEAP16[$55>>1]|0;
   $57 = $56&65535;
   $58 = $57 | $53;
   $59 = $58&65535;
   HEAP16[$55>>1] = $59;
   $60 = HEAP32[$1>>2]|0;
   $61 = (($60) + 15|0);
   $62 = HEAP8[$61>>0]|0;
   $63 = ($62<<24>>24)!=(0);
   $64 = HEAP32[$pPgOld>>2]|0;
   if ($63) {
    $65 = HEAP32[$1>>2]|0;
    $66 = (($65) + 24|0);
    $67 = HEAP32[$66>>2]|0;
    $68 = (($67) + 1)|0;
    _sqlite3PcacheMove($64,$68);
    break;
   } else {
    _sqlite3PcacheDrop($64);
    break;
   }
  }
 } while(0);
 $69 = HEAP32[$2>>2]|0;
 $70 = (($69) + 20|0);
 $71 = HEAP32[$70>>2]|0;
 HEAP32[$origPgno>>2] = $71;
 $72 = HEAP32[$2>>2]|0;
 $73 = HEAP32[$3>>2]|0;
 _sqlite3PcacheMove($72,$73);
 $74 = HEAP32[$2>>2]|0;
 _sqlite3PcacheMakeDirty($74);
 $75 = HEAP32[$1>>2]|0;
 $76 = (($75) + 15|0);
 $77 = HEAP8[$76>>0]|0;
 $78 = ($77<<24>>24)!=(0);
 if ($78) {
  $79 = HEAP32[$pPgOld>>2]|0;
  $80 = HEAP32[$origPgno>>2]|0;
  _sqlite3PcacheMove($79,$80);
  $81 = HEAP32[$pPgOld>>2]|0;
  _sqlite3PagerUnrefNotNull($81);
 }
 $82 = HEAP32[$needSyncPgno>>2]|0;
 $83 = ($82|0)!=(0);
 do {
  if ($83) {
   $84 = HEAP32[$1>>2]|0;
   $85 = HEAP32[$needSyncPgno>>2]|0;
   $86 = (_sqlite3PagerAcquire($84,$85,$pPgHdr,0)|0);
   HEAP32[$rc>>2] = $86;
   $87 = HEAP32[$rc>>2]|0;
   $88 = ($87|0)!=(0);
   if (!($88)) {
    $102 = HEAP32[$pPgHdr>>2]|0;
    $103 = (($102) + 24|0);
    $104 = HEAP16[$103>>1]|0;
    $105 = $104&65535;
    $106 = $105 | 4;
    $107 = $106&65535;
    HEAP16[$103>>1] = $107;
    $108 = HEAP32[$pPgHdr>>2]|0;
    _sqlite3PcacheMakeDirty($108);
    $109 = HEAP32[$pPgHdr>>2]|0;
    _sqlite3PagerUnrefNotNull($109);
    break;
   }
   $89 = HEAP32[$needSyncPgno>>2]|0;
   $90 = HEAP32[$1>>2]|0;
   $91 = (($90) + 28|0);
   $92 = HEAP32[$91>>2]|0;
   $93 = ($89>>>0)<=($92>>>0);
   if ($93) {
    $94 = HEAP32[$1>>2]|0;
    $95 = (($94) + 56|0);
    $96 = HEAP32[$95>>2]|0;
    $97 = HEAP32[$needSyncPgno>>2]|0;
    $98 = HEAP32[$1>>2]|0;
    $99 = (($98) + 200|0);
    $100 = HEAP32[$99>>2]|0;
    _sqlite3BitvecClear($96,$97,$100);
   }
   $101 = HEAP32[$rc>>2]|0;
   HEAP32[$0>>2] = $101;
   $110 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($110|0);
  }
 } while(0);
 HEAP32[$0>>2] = 0;
 $110 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($110|0);
}
function _setChildPtrmaps($pPage) {
 $pPage = $pPage|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $9 = 0;
 var $childPgno = 0, $childPgno1 = 0, $i = 0, $isInitOrig = 0, $nCell = 0, $pBt = 0, $pCell = 0, $pgno = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $0 = sp + 32|0;
 $i = sp + 28|0;
 $nCell = sp + 24|0;
 $rc = sp + 20|0;
 $pBt = sp + 16|0;
 $isInitOrig = sp + 36|0;
 $pgno = sp + 12|0;
 $pCell = sp + 8|0;
 $childPgno = sp + 4|0;
 $childPgno1 = sp;
 HEAP32[$0>>2] = $pPage;
 $1 = HEAP32[$0>>2]|0;
 $2 = (($1) + 52|0);
 $3 = HEAP32[$2>>2]|0;
 HEAP32[$pBt>>2] = $3;
 $4 = HEAP32[$0>>2]|0;
 $5 = HEAP8[$4>>0]|0;
 HEAP8[$isInitOrig>>0] = $5;
 $6 = HEAP32[$0>>2]|0;
 $7 = (($6) + 72|0);
 $8 = HEAP32[$7>>2]|0;
 HEAP32[$pgno>>2] = $8;
 $9 = HEAP32[$0>>2]|0;
 $10 = (_btreeInitPage($9)|0);
 HEAP32[$rc>>2] = $10;
 $11 = HEAP32[$rc>>2]|0;
 $12 = ($11|0)!=(0);
 if ($12) {
  $77 = HEAP8[$isInitOrig>>0]|0;
  $78 = HEAP32[$0>>2]|0;
  HEAP8[$78>>0] = $77;
  $79 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($79|0);
 }
 $13 = HEAP32[$0>>2]|0;
 $14 = (($13) + 18|0);
 $15 = HEAP16[$14>>1]|0;
 $16 = $15&65535;
 HEAP32[$nCell>>2] = $16;
 HEAP32[$i>>2] = 0;
 while(1) {
  $17 = HEAP32[$i>>2]|0;
  $18 = HEAP32[$nCell>>2]|0;
  $19 = ($17|0)<($18|0);
  $20 = HEAP32[$0>>2]|0;
  if (!($19)) {
   break;
  }
  $21 = (($20) + 56|0);
  $22 = HEAP32[$21>>2]|0;
  $23 = HEAP32[$0>>2]|0;
  $24 = (($23) + 20|0);
  $25 = HEAP16[$24>>1]|0;
  $26 = $25&65535;
  $27 = HEAP32[$i>>2]|0;
  $28 = $27<<1;
  $29 = HEAP32[$0>>2]|0;
  $30 = (($29) + 64|0);
  $31 = HEAP32[$30>>2]|0;
  $32 = (($31) + ($28)|0);
  $33 = HEAP8[$32>>0]|0;
  $34 = $33&255;
  $35 = $34 << 8;
  $36 = HEAP32[$i>>2]|0;
  $37 = $36<<1;
  $38 = HEAP32[$0>>2]|0;
  $39 = (($38) + 64|0);
  $40 = HEAP32[$39>>2]|0;
  $41 = (($40) + ($37)|0);
  $42 = (($41) + 1|0);
  $43 = HEAP8[$42>>0]|0;
  $44 = $43&255;
  $45 = $35 | $44;
  $46 = $26 & $45;
  $47 = (($22) + ($46)|0);
  HEAP32[$pCell>>2] = $47;
  $48 = HEAP32[$0>>2]|0;
  $49 = HEAP32[$pCell>>2]|0;
  _ptrmapPutOvflPtr($48,$49,$rc);
  $50 = HEAP32[$0>>2]|0;
  $51 = (($50) + 5|0);
  $52 = HEAP8[$51>>0]|0;
  $53 = ($52<<24>>24)!=(0);
  if (!($53)) {
   $54 = HEAP32[$pCell>>2]|0;
   $55 = (_sqlite3Get4byte($54)|0);
   HEAP32[$childPgno>>2] = $55;
   $56 = HEAP32[$pBt>>2]|0;
   $57 = HEAP32[$childPgno>>2]|0;
   $58 = HEAP32[$pgno>>2]|0;
   _ptrmapPut($56,$57,5,$58,$rc);
  }
  $59 = HEAP32[$i>>2]|0;
  $60 = (($59) + 1)|0;
  HEAP32[$i>>2] = $60;
 }
 $61 = (($20) + 5|0);
 $62 = HEAP8[$61>>0]|0;
 $63 = ($62<<24>>24)!=(0);
 if ($63) {
  $77 = HEAP8[$isInitOrig>>0]|0;
  $78 = HEAP32[$0>>2]|0;
  HEAP8[$78>>0] = $77;
  $79 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($79|0);
 }
 $64 = HEAP32[$0>>2]|0;
 $65 = (($64) + 6|0);
 $66 = HEAP8[$65>>0]|0;
 $67 = $66&255;
 $68 = (($67) + 8)|0;
 $69 = HEAP32[$0>>2]|0;
 $70 = (($69) + 56|0);
 $71 = HEAP32[$70>>2]|0;
 $72 = (($71) + ($68)|0);
 $73 = (_sqlite3Get4byte($72)|0);
 HEAP32[$childPgno1>>2] = $73;
 $74 = HEAP32[$pBt>>2]|0;
 $75 = HEAP32[$childPgno1>>2]|0;
 $76 = HEAP32[$pgno>>2]|0;
 _ptrmapPut($74,$75,5,$76,$rc);
 $77 = HEAP8[$isInitOrig>>0]|0;
 $78 = HEAP32[$0>>2]|0;
 HEAP8[$78>>0] = $77;
 $79 = HEAP32[$rc>>2]|0;
 STACKTOP = sp;return ($79|0);
}
function _ptrmapPut($pBt,$key,$eType,$parent,$pRC) {
 $pBt = $pBt|0;
 $key = $key|0;
 $eType = $eType|0;
 $parent = $parent|0;
 $pRC = $pRC|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $7 = 0, $8 = 0, $9 = 0, $iPtrmap = 0, $offset = 0, $pDbPage = 0, $pPtrmap = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $0 = sp + 32|0;
 $1 = sp + 28|0;
 $2 = sp + 36|0;
 $3 = sp + 24|0;
 $4 = sp + 20|0;
 $pDbPage = sp + 16|0;
 $pPtrmap = sp + 12|0;
 $iPtrmap = sp + 8|0;
 $offset = sp + 4|0;
 $rc = sp;
 HEAP32[$0>>2] = $pBt;
 HEAP32[$1>>2] = $key;
 HEAP8[$2>>0] = $eType;
 HEAP32[$3>>2] = $parent;
 HEAP32[$4>>2] = $pRC;
 $5 = HEAP32[$4>>2]|0;
 $6 = HEAP32[$5>>2]|0;
 $7 = ($6|0)!=(0);
 if ($7) {
  STACKTOP = sp;return;
 }
 $8 = HEAP32[$1>>2]|0;
 $9 = ($8|0)==(0);
 if ($9) {
  $10 = (_sqlite3CorruptError(53057)|0);
  $11 = HEAP32[$4>>2]|0;
  HEAP32[$11>>2] = $10;
  STACKTOP = sp;return;
 }
 $12 = HEAP32[$0>>2]|0;
 $13 = HEAP32[$1>>2]|0;
 $14 = (_ptrmapPageno($12,$13)|0);
 HEAP32[$iPtrmap>>2] = $14;
 $15 = HEAP32[$0>>2]|0;
 $16 = HEAP32[$15>>2]|0;
 $17 = HEAP32[$iPtrmap>>2]|0;
 $18 = (_sqlite3PagerAcquire($16,$17,$pDbPage,0)|0);
 HEAP32[$rc>>2] = $18;
 $19 = HEAP32[$rc>>2]|0;
 $20 = ($19|0)!=(0);
 if ($20) {
  $21 = HEAP32[$rc>>2]|0;
  $22 = HEAP32[$4>>2]|0;
  HEAP32[$22>>2] = $21;
  STACKTOP = sp;return;
 }
 $23 = HEAP32[$1>>2]|0;
 $24 = HEAP32[$iPtrmap>>2]|0;
 $25 = (($23) - ($24))|0;
 $26 = (($25) - 1)|0;
 $27 = ($26*5)|0;
 HEAP32[$offset>>2] = $27;
 $28 = HEAP32[$offset>>2]|0;
 $29 = ($28|0)<(0);
 do {
  if ($29) {
   $30 = (_sqlite3CorruptError(53068)|0);
   $31 = HEAP32[$4>>2]|0;
   HEAP32[$31>>2] = $30;
  } else {
   $32 = HEAP32[$pDbPage>>2]|0;
   $33 = (_sqlite3PagerGetData($32)|0);
   HEAP32[$pPtrmap>>2] = $33;
   $34 = HEAP8[$2>>0]|0;
   $35 = $34&255;
   $36 = HEAP32[$offset>>2]|0;
   $37 = HEAP32[$pPtrmap>>2]|0;
   $38 = (($37) + ($36)|0);
   $39 = HEAP8[$38>>0]|0;
   $40 = $39&255;
   $41 = ($35|0)!=($40|0);
   if (!($41)) {
    $42 = HEAP32[$offset>>2]|0;
    $43 = (($42) + 1)|0;
    $44 = HEAP32[$pPtrmap>>2]|0;
    $45 = (($44) + ($43)|0);
    $46 = (_sqlite3Get4byte($45)|0);
    $47 = HEAP32[$3>>2]|0;
    $48 = ($46|0)!=($47|0);
    if (!($48)) {
     break;
    }
   }
   $49 = HEAP32[$pDbPage>>2]|0;
   $50 = (_sqlite3PagerWrite($49)|0);
   HEAP32[$rc>>2] = $50;
   $51 = HEAP32[$4>>2]|0;
   HEAP32[$51>>2] = $50;
   $52 = HEAP32[$rc>>2]|0;
   $53 = ($52|0)==(0);
   if ($53) {
    $54 = HEAP8[$2>>0]|0;
    $55 = HEAP32[$offset>>2]|0;
    $56 = HEAP32[$pPtrmap>>2]|0;
    $57 = (($56) + ($55)|0);
    HEAP8[$57>>0] = $54;
    $58 = HEAP32[$offset>>2]|0;
    $59 = (($58) + 1)|0;
    $60 = HEAP32[$pPtrmap>>2]|0;
    $61 = (($60) + ($59)|0);
    $62 = HEAP32[$3>>2]|0;
    _sqlite3Put4byte($61,$62);
   }
  }
 } while(0);
 $63 = HEAP32[$pDbPage>>2]|0;
 _sqlite3PagerUnref($63);
 STACKTOP = sp;return;
}
function _modifyPagePointer($pPage,$iFrom,$iTo,$eType) {
 $pPage = $pPage|0;
 $iFrom = $iFrom|0;
 $iTo = $iTo|0;
 $eType = $eType|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0;
 var $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0;
 var $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0;
 var $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0;
 var $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $i = 0, $info = 0, $isInitOrig = 0, $nCell = 0, $pCell = 0, label = 0;
 var sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 64|0;
 $0 = sp + 48|0;
 $1 = sp + 44|0;
 $2 = sp + 40|0;
 $3 = sp + 36|0;
 $4 = sp + 53|0;
 $isInitOrig = sp + 52|0;
 $i = sp + 32|0;
 $nCell = sp + 28|0;
 $pCell = sp + 24|0;
 $info = sp;
 HEAP32[$1>>2] = $pPage;
 HEAP32[$2>>2] = $iFrom;
 HEAP32[$3>>2] = $iTo;
 HEAP8[$4>>0] = $eType;
 $5 = HEAP8[$4>>0]|0;
 $6 = $5&255;
 $7 = ($6|0)==(4);
 $8 = HEAP32[$1>>2]|0;
 do {
  if ($7) {
   $9 = (($8) + 56|0);
   $10 = HEAP32[$9>>2]|0;
   $11 = (_sqlite3Get4byte($10)|0);
   $12 = HEAP32[$2>>2]|0;
   $13 = ($11|0)!=($12|0);
   if (!($13)) {
    $15 = HEAP32[$1>>2]|0;
    $16 = (($15) + 56|0);
    $17 = HEAP32[$16>>2]|0;
    $18 = HEAP32[$3>>2]|0;
    _sqlite3Put4byte($17,$18);
    break;
   }
   $14 = (_sqlite3CorruptError(55139)|0);
   HEAP32[$0>>2] = $14;
   $133 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($133|0);
  } else {
   $19 = HEAP8[$8>>0]|0;
   HEAP8[$isInitOrig>>0] = $19;
   $20 = HEAP32[$1>>2]|0;
   (_btreeInitPage($20)|0);
   $21 = HEAP32[$1>>2]|0;
   $22 = (($21) + 18|0);
   $23 = HEAP16[$22>>1]|0;
   $24 = $23&65535;
   HEAP32[$nCell>>2] = $24;
   HEAP32[$i>>2] = 0;
   while(1) {
    $25 = HEAP32[$i>>2]|0;
    $26 = HEAP32[$nCell>>2]|0;
    $27 = ($25|0)<($26|0);
    if (!($27)) {
     break;
    }
    $28 = HEAP32[$1>>2]|0;
    $29 = (($28) + 56|0);
    $30 = HEAP32[$29>>2]|0;
    $31 = HEAP32[$1>>2]|0;
    $32 = (($31) + 20|0);
    $33 = HEAP16[$32>>1]|0;
    $34 = $33&65535;
    $35 = HEAP32[$i>>2]|0;
    $36 = $35<<1;
    $37 = HEAP32[$1>>2]|0;
    $38 = (($37) + 64|0);
    $39 = HEAP32[$38>>2]|0;
    $40 = (($39) + ($36)|0);
    $41 = HEAP8[$40>>0]|0;
    $42 = $41&255;
    $43 = $42 << 8;
    $44 = HEAP32[$i>>2]|0;
    $45 = $44<<1;
    $46 = HEAP32[$1>>2]|0;
    $47 = (($46) + 64|0);
    $48 = HEAP32[$47>>2]|0;
    $49 = (($48) + ($45)|0);
    $50 = (($49) + 1|0);
    $51 = HEAP8[$50>>0]|0;
    $52 = $51&255;
    $53 = $43 | $52;
    $54 = $34 & $53;
    $55 = (($30) + ($54)|0);
    HEAP32[$pCell>>2] = $55;
    $56 = HEAP8[$4>>0]|0;
    $57 = $56&255;
    $58 = ($57|0)==(3);
    if ($58) {
     $59 = HEAP32[$1>>2]|0;
     $60 = HEAP32[$pCell>>2]|0;
     _btreeParseCellPtr($59,$60,$info);
     $61 = (($info) + 18|0);
     $62 = HEAP16[$61>>1]|0;
     $63 = $62&65535;
     $64 = ($63|0)!=(0);
     if ($64) {
      $65 = HEAP32[$pCell>>2]|0;
      $66 = (($info) + 18|0);
      $67 = HEAP16[$66>>1]|0;
      $68 = $67&65535;
      $69 = (($65) + ($68)|0);
      $70 = (($69) + 3|0);
      $71 = HEAP32[$1>>2]|0;
      $72 = (($71) + 56|0);
      $73 = HEAP32[$72>>2]|0;
      $74 = HEAP32[$1>>2]|0;
      $75 = (($74) + 20|0);
      $76 = HEAP16[$75>>1]|0;
      $77 = $76&65535;
      $78 = (($73) + ($77)|0);
      $79 = ($70>>>0)<=($78>>>0);
      if ($79) {
       $80 = HEAP32[$2>>2]|0;
       $81 = (($info) + 18|0);
       $82 = HEAP16[$81>>1]|0;
       $83 = $82&65535;
       $84 = HEAP32[$pCell>>2]|0;
       $85 = (($84) + ($83)|0);
       $86 = (_sqlite3Get4byte($85)|0);
       $87 = ($80|0)==($86|0);
       if ($87) {
        label = 11;
        break;
       }
      }
     }
    } else {
     $94 = HEAP32[$pCell>>2]|0;
     $95 = (_sqlite3Get4byte($94)|0);
     $96 = HEAP32[$2>>2]|0;
     $97 = ($95|0)==($96|0);
     if ($97) {
      label = 13;
      break;
     }
    }
    $100 = HEAP32[$i>>2]|0;
    $101 = (($100) + 1)|0;
    HEAP32[$i>>2] = $101;
   }
   if ((label|0) == 11) {
    $88 = (($info) + 18|0);
    $89 = HEAP16[$88>>1]|0;
    $90 = $89&65535;
    $91 = HEAP32[$pCell>>2]|0;
    $92 = (($91) + ($90)|0);
    $93 = HEAP32[$3>>2]|0;
    _sqlite3Put4byte($92,$93);
   }
   else if ((label|0) == 13) {
    $98 = HEAP32[$pCell>>2]|0;
    $99 = HEAP32[$3>>2]|0;
    _sqlite3Put4byte($98,$99);
   }
   $102 = HEAP32[$i>>2]|0;
   $103 = HEAP32[$nCell>>2]|0;
   $104 = ($102|0)==($103|0);
   do {
    if ($104) {
     $105 = HEAP8[$4>>0]|0;
     $106 = $105&255;
     $107 = ($106|0)!=(5);
     if (!($107)) {
      $108 = HEAP32[$1>>2]|0;
      $109 = (($108) + 6|0);
      $110 = HEAP8[$109>>0]|0;
      $111 = $110&255;
      $112 = (($111) + 8)|0;
      $113 = HEAP32[$1>>2]|0;
      $114 = (($113) + 56|0);
      $115 = HEAP32[$114>>2]|0;
      $116 = (($115) + ($112)|0);
      $117 = (_sqlite3Get4byte($116)|0);
      $118 = HEAP32[$2>>2]|0;
      $119 = ($117|0)!=($118|0);
      if (!($119)) {
       $121 = HEAP32[$1>>2]|0;
       $122 = (($121) + 6|0);
       $123 = HEAP8[$122>>0]|0;
       $124 = $123&255;
       $125 = (($124) + 8)|0;
       $126 = HEAP32[$1>>2]|0;
       $127 = (($126) + 56|0);
       $128 = HEAP32[$127>>2]|0;
       $129 = (($128) + ($125)|0);
       $130 = HEAP32[$3>>2]|0;
       _sqlite3Put4byte($129,$130);
       break;
      }
     }
     $120 = (_sqlite3CorruptError(55173)|0);
     HEAP32[$0>>2] = $120;
     $133 = HEAP32[$0>>2]|0;
     STACKTOP = sp;return ($133|0);
    }
   } while(0);
   $131 = HEAP8[$isInitOrig>>0]|0;
   $132 = HEAP32[$1>>2]|0;
   HEAP8[$132>>0] = $131;
  }
 } while(0);
 HEAP32[$0>>2] = 0;
 $133 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($133|0);
}
function _ptrmapPutOvflPtr($pPage,$pCell,$pRC) {
 $pPage = $pPage|0;
 $pCell = $pCell|0;
 $pRC = $pRC|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $3 = 0, $4 = 0;
 var $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $info = 0, $ovfl = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $0 = sp + 36|0;
 $1 = sp + 32|0;
 $2 = sp + 28|0;
 $info = sp;
 $ovfl = sp + 24|0;
 HEAP32[$0>>2] = $pPage;
 HEAP32[$1>>2] = $pCell;
 HEAP32[$2>>2] = $pRC;
 $3 = HEAP32[$2>>2]|0;
 $4 = HEAP32[$3>>2]|0;
 $5 = ($4|0)!=(0);
 if ($5) {
  STACKTOP = sp;return;
 }
 $6 = HEAP32[$0>>2]|0;
 $7 = HEAP32[$1>>2]|0;
 _btreeParseCellPtr($6,$7,$info);
 $8 = (($info) + 18|0);
 $9 = HEAP16[$8>>1]|0;
 $10 = ($9<<16>>16)!=(0);
 if (!($10)) {
  STACKTOP = sp;return;
 }
 $11 = (($info) + 18|0);
 $12 = HEAP16[$11>>1]|0;
 $13 = $12&65535;
 $14 = HEAP32[$1>>2]|0;
 $15 = (($14) + ($13)|0);
 $16 = (_sqlite3Get4byte($15)|0);
 HEAP32[$ovfl>>2] = $16;
 $17 = HEAP32[$0>>2]|0;
 $18 = (($17) + 52|0);
 $19 = HEAP32[$18>>2]|0;
 $20 = HEAP32[$ovfl>>2]|0;
 $21 = HEAP32[$0>>2]|0;
 $22 = (($21) + 72|0);
 $23 = HEAP32[$22>>2]|0;
 $24 = HEAP32[$2>>2]|0;
 _ptrmapPut($19,$20,3,$23,$24);
 STACKTOP = sp;return;
}
function _sqlite3PcacheMove($p,$newPgno) {
 $p = $p|0;
 $newPgno = $newPgno|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $pCache = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $1 = sp + 4|0;
 $pCache = sp;
 HEAP32[$0>>2] = $p;
 HEAP32[$1>>2] = $newPgno;
 $2 = HEAP32[$0>>2]|0;
 $3 = (($2) + 28|0);
 $4 = HEAP32[$3>>2]|0;
 HEAP32[$pCache>>2] = $4;
 $5 = HEAP32[((38640 + 140|0))>>2]|0;
 $6 = HEAP32[$pCache>>2]|0;
 $7 = (($6) + 40|0);
 $8 = HEAP32[$7>>2]|0;
 $9 = HEAP32[$0>>2]|0;
 $10 = HEAP32[$9>>2]|0;
 $11 = HEAP32[$0>>2]|0;
 $12 = (($11) + 20|0);
 $13 = HEAP32[$12>>2]|0;
 $14 = HEAP32[$1>>2]|0;
 FUNCTION_TABLE_viiii[$5 & 3]($8,$10,$13,$14);
 $15 = HEAP32[$1>>2]|0;
 $16 = HEAP32[$0>>2]|0;
 $17 = (($16) + 20|0);
 HEAP32[$17>>2] = $15;
 $18 = HEAP32[$0>>2]|0;
 $19 = (($18) + 24|0);
 $20 = HEAP16[$19>>1]|0;
 $21 = $20&65535;
 $22 = $21 & 2;
 $23 = ($22|0)!=(0);
 if (!($23)) {
  STACKTOP = sp;return;
 }
 $24 = HEAP32[$0>>2]|0;
 $25 = (($24) + 24|0);
 $26 = HEAP16[$25>>1]|0;
 $27 = $26&65535;
 $28 = $27 & 4;
 $29 = ($28|0)!=(0);
 if (!($29)) {
  STACKTOP = sp;return;
 }
 $30 = HEAP32[$0>>2]|0;
 _pcacheManageDirtyList($30,3);
 STACKTOP = sp;return;
}
function _btreeGetHasContent($pBt,$pgno) {
 $pBt = $pBt|0;
 $pgno = $pgno|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $p = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $1 = sp + 4|0;
 $p = sp;
 HEAP32[$0>>2] = $pBt;
 HEAP32[$1>>2] = $pgno;
 $2 = HEAP32[$0>>2]|0;
 $3 = (($2) + 60|0);
 $4 = HEAP32[$3>>2]|0;
 HEAP32[$p>>2] = $4;
 $5 = HEAP32[$p>>2]|0;
 $6 = ($5|0)!=(0|0);
 if (!($6)) {
  $16 = 0;
  $15 = $16&1;
  STACKTOP = sp;return ($15|0);
 }
 $7 = HEAP32[$1>>2]|0;
 $8 = HEAP32[$p>>2]|0;
 $9 = (_sqlite3BitvecSize($8)|0);
 $10 = ($7>>>0)>($9>>>0);
 if ($10) {
  $16 = 1;
  $15 = $16&1;
  STACKTOP = sp;return ($15|0);
 }
 $11 = HEAP32[$p>>2]|0;
 $12 = HEAP32[$1>>2]|0;
 $13 = (_sqlite3BitvecTest($11,$12)|0);
 $14 = ($13|0)!=(0);
 $16 = $14;
 $15 = $16&1;
 STACKTOP = sp;return ($15|0);
}
function _sqlite3PagerIsMemdb($pPager) {
 $pPager = $pPager|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 HEAP32[$0>>2] = $pPager;
 $1 = HEAP32[$0>>2]|0;
 $2 = (($1) + 15|0);
 $3 = HEAP8[$2>>0]|0;
 $4 = $3&255;
 STACKTOP = sp;return ($4|0);
}
function _execSql($db,$pzErrMsg,$zSql) {
 $db = $db|0;
 $pzErrMsg = $pzErrMsg|0;
 $zSql = $zSql|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0;
 var $8 = 0, $9 = 0, $pStmt = 0, $vararg_buffer = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $vararg_buffer = sp;
 $0 = sp + 20|0;
 $1 = sp + 16|0;
 $2 = sp + 12|0;
 $3 = sp + 8|0;
 $pStmt = sp + 4|0;
 HEAP32[$1>>2] = $db;
 HEAP32[$2>>2] = $pzErrMsg;
 HEAP32[$3>>2] = $zSql;
 $4 = HEAP32[$3>>2]|0;
 $5 = ($4|0)!=(0|0);
 if (!($5)) {
  HEAP32[$0>>2] = 7;
  $21 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($21|0);
 }
 $6 = HEAP32[$1>>2]|0;
 $7 = HEAP32[$3>>2]|0;
 $8 = (_sqlite3_prepare($6,$7,-1,$pStmt,0)|0);
 $9 = (0)!=($8|0);
 if ($9) {
  $10 = HEAP32[$2>>2]|0;
  $11 = HEAP32[$1>>2]|0;
  $12 = HEAP32[$1>>2]|0;
  $13 = (_sqlite3_errmsg($12)|0);
  _sqlite3SetString($10,$11,$13,$vararg_buffer);
  $14 = HEAP32[$1>>2]|0;
  $15 = (_sqlite3_errcode($14)|0);
  HEAP32[$0>>2] = $15;
  $21 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($21|0);
 } else {
  $16 = HEAP32[$pStmt>>2]|0;
  (_sqlite3_step($16)|0);
  $17 = HEAP32[$1>>2]|0;
  $18 = HEAP32[$pStmt>>2]|0;
  $19 = HEAP32[$2>>2]|0;
  $20 = (_vacuumFinalize($17,$18,$19)|0);
  HEAP32[$0>>2] = $20;
  $21 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($21|0);
 }
 return 0|0;
}
function _sqlite3BtreeGetReserve($p) {
 $p = $p|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $n = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $n = sp;
 HEAP32[$0>>2] = $p;
 $1 = HEAP32[$0>>2]|0;
 _sqlite3BtreeEnter($1);
 $2 = HEAP32[$0>>2]|0;
 $3 = (($2) + 4|0);
 $4 = HEAP32[$3>>2]|0;
 $5 = (($4) + 32|0);
 $6 = HEAP32[$5>>2]|0;
 $7 = HEAP32[$0>>2]|0;
 $8 = (($7) + 4|0);
 $9 = HEAP32[$8>>2]|0;
 $10 = (($9) + 36|0);
 $11 = HEAP32[$10>>2]|0;
 $12 = (($6) - ($11))|0;
 HEAP32[$n>>2] = $12;
 $13 = HEAP32[$n>>2]|0;
 STACKTOP = sp;return ($13|0);
}
function _execExecSql($db,$pzErrMsg,$zSql) {
 $db = $db|0;
 $pzErrMsg = $pzErrMsg|0;
 $zSql = $zSql|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $pStmt = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 20|0;
 $1 = sp + 16|0;
 $2 = sp + 12|0;
 $3 = sp + 8|0;
 $pStmt = sp + 4|0;
 $rc = sp;
 HEAP32[$1>>2] = $db;
 HEAP32[$2>>2] = $pzErrMsg;
 HEAP32[$3>>2] = $zSql;
 $4 = HEAP32[$1>>2]|0;
 $5 = HEAP32[$3>>2]|0;
 $6 = (_sqlite3_prepare($4,$5,-1,$pStmt,0)|0);
 HEAP32[$rc>>2] = $6;
 $7 = HEAP32[$rc>>2]|0;
 $8 = ($7|0)!=(0);
 if ($8) {
  $9 = HEAP32[$rc>>2]|0;
  HEAP32[$0>>2] = $9;
  $27 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($27|0);
 }
 while(1) {
  $10 = HEAP32[$pStmt>>2]|0;
  $11 = (_sqlite3_step($10)|0);
  $12 = (100)==($11|0);
  $13 = HEAP32[$1>>2]|0;
  if (!($12)) {
   label = 6;
   break;
  }
  $14 = HEAP32[$2>>2]|0;
  $15 = HEAP32[$pStmt>>2]|0;
  $16 = (_sqlite3_column_text($15,0)|0);
  $17 = (_execSql($13,$14,$16)|0);
  HEAP32[$rc>>2] = $17;
  $18 = HEAP32[$rc>>2]|0;
  $19 = ($18|0)!=(0);
  if ($19) {
   label = 5;
   break;
  }
 }
 if ((label|0) == 5) {
  $20 = HEAP32[$1>>2]|0;
  $21 = HEAP32[$pStmt>>2]|0;
  $22 = HEAP32[$2>>2]|0;
  (_vacuumFinalize($20,$21,$22)|0);
  $23 = HEAP32[$rc>>2]|0;
  HEAP32[$0>>2] = $23;
  $27 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($27|0);
 }
 else if ((label|0) == 6) {
  $24 = HEAP32[$pStmt>>2]|0;
  $25 = HEAP32[$2>>2]|0;
  $26 = (_vacuumFinalize($13,$24,$25)|0);
  HEAP32[$0>>2] = $26;
  $27 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($27|0);
 }
 return 0|0;
}
function _sqlite3BtreeCopyFile($pTo,$pFrom) {
 $pTo = $pTo|0;
 $pFrom = $pFrom|0;
 var $$ = 0, $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0;
 var $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0;
 var $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $b = 0, $nByte = 0, $pFd = 0, $rc = 0, dest = 0, label = 0, sp = 0, stop = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 80|0;
 $0 = sp + 68|0;
 $1 = sp + 64|0;
 $rc = sp + 60|0;
 $pFd = sp + 56|0;
 $b = sp + 8|0;
 $nByte = sp;
 HEAP32[$0>>2] = $pTo;
 HEAP32[$1>>2] = $pFrom;
 $2 = HEAP32[$0>>2]|0;
 _sqlite3BtreeEnter($2);
 $3 = HEAP32[$1>>2]|0;
 _sqlite3BtreeEnter($3);
 $4 = HEAP32[$0>>2]|0;
 $5 = (_sqlite3BtreePager($4)|0);
 $6 = (_sqlite3PagerFile($5)|0);
 HEAP32[$pFd>>2] = $6;
 $7 = HEAP32[$pFd>>2]|0;
 $8 = HEAP32[$7>>2]|0;
 $9 = ($8|0)!=(0|0);
 if ($9) {
  $10 = HEAP32[$1>>2]|0;
  $11 = (_sqlite3BtreeGetPageSize($10)|0);
  $12 = ($11|0)<(0);
  $13 = $12 << 31 >> 31;
  $14 = HEAP32[$1>>2]|0;
  $15 = (_sqlite3BtreeLastPage($14)|0);
  $16 = (___muldi3(($11|0),($13|0),($15|0),0)|0);
  $17 = tempRet0;
  $18 = $nByte;
  $19 = $18;
  HEAP32[$19>>2] = $16;
  $20 = (($18) + 4)|0;
  $21 = $20;
  HEAP32[$21>>2] = $17;
  $22 = HEAP32[$pFd>>2]|0;
  $23 = (_sqlite3OsFileControl($22,11,$nByte)|0);
  HEAP32[$rc>>2] = $23;
  $24 = HEAP32[$rc>>2]|0;
  $25 = ($24|0)==(12);
  $$ = $25 ? 0 : $23;
  HEAP32[$rc>>2] = $$;
  $26 = HEAP32[$rc>>2]|0;
  $27 = ($26|0)!=(0);
  if ($27) {
   $50 = HEAP32[$rc>>2]|0;
   STACKTOP = sp;return ($50|0);
  }
 }
 dest=$b+0|0; stop=dest+48|0; do { HEAP32[dest>>2]=0|0; dest=dest+4|0; } while ((dest|0) < (stop|0));
 $28 = HEAP32[$1>>2]|0;
 $29 = HEAP32[$28>>2]|0;
 $30 = (($b) + 20|0);
 HEAP32[$30>>2] = $29;
 $31 = HEAP32[$1>>2]|0;
 $32 = (($b) + 24|0);
 HEAP32[$32>>2] = $31;
 $33 = HEAP32[$0>>2]|0;
 $34 = (($b) + 4|0);
 HEAP32[$34>>2] = $33;
 $35 = (($b) + 16|0);
 HEAP32[$35>>2] = 1;
 (_sqlite3_backup_step($b,2147483647)|0);
 $36 = (_sqlite3_backup_finish($b)|0);
 HEAP32[$rc>>2] = $36;
 $37 = HEAP32[$rc>>2]|0;
 $38 = ($37|0)==(0);
 if ($38) {
  $39 = HEAP32[$0>>2]|0;
  $40 = (($39) + 4|0);
  $41 = HEAP32[$40>>2]|0;
  $42 = (($41) + 22|0);
  $43 = HEAP16[$42>>1]|0;
  $44 = $43&65535;
  $45 = $44 & -3;
  $46 = $45&65535;
  HEAP16[$42>>1] = $46;
  $50 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($50|0);
 } else {
  $47 = (($b) + 4|0);
  $48 = HEAP32[$47>>2]|0;
  $49 = (_sqlite3BtreePager($48)|0);
  _sqlite3PagerClearCache($49);
  $50 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($50|0);
 }
 return 0|0;
}
function _sqlite3PagerClearCache($pPager) {
 $pPager = $pPager|0;
 var $0 = 0, $1 = 0, $10 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 HEAP32[$0>>2] = $pPager;
 $1 = HEAP32[$0>>2]|0;
 $2 = (($1) + 15|0);
 $3 = HEAP8[$2>>0]|0;
 $4 = ($3<<24>>24)!=(0);
 if ($4) {
  STACKTOP = sp;return;
 }
 $5 = HEAP32[$0>>2]|0;
 $6 = (($5) + 12|0);
 $7 = HEAP8[$6>>0]|0;
 $8 = $7&255;
 $9 = ($8|0)==(0);
 if (!($9)) {
  STACKTOP = sp;return;
 }
 $10 = HEAP32[$0>>2]|0;
 _pager_reset($10);
 STACKTOP = sp;return;
}
function _vacuumFinalize($db,$pStmt,$pzErrMsg) {
 $db = $db|0;
 $pStmt = $pStmt|0;
 $pzErrMsg = $pzErrMsg|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $rc = 0, $vararg_buffer = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $vararg_buffer = sp;
 $0 = sp + 16|0;
 $1 = sp + 12|0;
 $2 = sp + 8|0;
 $rc = sp + 4|0;
 HEAP32[$0>>2] = $db;
 HEAP32[$1>>2] = $pStmt;
 HEAP32[$2>>2] = $pzErrMsg;
 $3 = HEAP32[$1>>2]|0;
 $4 = (_sqlite3VdbeFinalize($3)|0);
 HEAP32[$rc>>2] = $4;
 $5 = HEAP32[$rc>>2]|0;
 $6 = ($5|0)!=(0);
 if (!($6)) {
  $11 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($11|0);
 }
 $7 = HEAP32[$2>>2]|0;
 $8 = HEAP32[$0>>2]|0;
 $9 = HEAP32[$0>>2]|0;
 $10 = (_sqlite3_errmsg($9)|0);
 _sqlite3SetString($7,$8,$10,$vararg_buffer);
 $11 = HEAP32[$rc>>2]|0;
 STACKTOP = sp;return ($11|0);
}
function _sqlite3PagerSharedLock($pPager) {
 $pPager = $pPager|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0;
 var $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0;
 var $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0;
 var $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0;
 var $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $bExists = 0, $bHotJournal = 0, $dbFileVers = 0, $f = 0;
 var $fout = 0, $nPage = 0, $or$cond = 0, $or$cond3 = 0, $or$cond5 = 0, $pVfs = 0, $rc = 0, dest = 0, label = 0, sp = 0, stop = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 64|0;
 $0 = sp + 32|0;
 $1 = sp + 28|0;
 $rc = sp + 24|0;
 $bHotJournal = sp + 20|0;
 $pVfs = sp + 16|0;
 $bExists = sp + 12|0;
 $fout = sp + 8|0;
 $f = sp + 4|0;
 $nPage = sp;
 $dbFileVers = sp + 40|0;
 HEAP32[$1>>2] = $pPager;
 HEAP32[$rc>>2] = 0;
 $2 = HEAP32[$1>>2]|0;
 $3 = (($2) + 15|0);
 $4 = HEAP8[$3>>0]|0;
 $5 = $4&255;
 $6 = ($5|0)!=(0);
 if ($6) {
  $7 = HEAP32[$1>>2]|0;
  $8 = (($7) + 40|0);
  $9 = HEAP32[$8>>2]|0;
  $10 = ($9|0)!=(0);
  if ($10) {
   $11 = HEAP32[$1>>2]|0;
   $12 = (($11) + 40|0);
   $13 = HEAP32[$12>>2]|0;
   HEAP32[$0>>2] = $13;
   $155 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($155|0);
  }
 }
 $14 = HEAP32[$1>>2]|0;
 $15 = (_pagerUseWal($14)|0);
 $16 = ($15|0)!=(0);
 L6: do {
  if ($16) {
   label = 36;
  } else {
   $17 = HEAP32[$1>>2]|0;
   $18 = (($17) + 16|0);
   $19 = HEAP8[$18>>0]|0;
   $20 = $19&255;
   $21 = ($20|0)==(0);
   if ($21) {
    HEAP32[$bHotJournal>>2] = 1;
    $22 = HEAP32[$1>>2]|0;
    $23 = (_pager_wait_on_lock($22,1)|0);
    HEAP32[$rc>>2] = $23;
    $24 = HEAP32[$rc>>2]|0;
    $25 = ($24|0)!=(0);
    if (!($25)) {
     $26 = HEAP32[$1>>2]|0;
     $27 = (($26) + 17|0);
     $28 = HEAP8[$27>>0]|0;
     $29 = $28&255;
     $30 = ($29|0)<=(1);
     if ($30) {
      $31 = HEAP32[$1>>2]|0;
      $32 = (_hasHotJournal($31,$bHotJournal)|0);
      HEAP32[$rc>>2] = $32;
     }
     $33 = HEAP32[$rc>>2]|0;
     $34 = ($33|0)!=(0);
     if (!($34)) {
      $35 = HEAP32[$bHotJournal>>2]|0;
      $36 = ($35|0)!=(0);
      if ($36) {
       $37 = HEAP32[$1>>2]|0;
       $38 = (($37) + 14|0);
       $39 = HEAP8[$38>>0]|0;
       $40 = ($39<<24>>24)!=(0);
       if ($40) {
        HEAP32[$rc>>2] = 776;
        break;
       }
       $41 = HEAP32[$1>>2]|0;
       $42 = (_pagerLockDb($41,4)|0);
       HEAP32[$rc>>2] = $42;
       $43 = HEAP32[$rc>>2]|0;
       $44 = ($43|0)!=(0);
       if ($44) {
        break;
       }
       $45 = HEAP32[$1>>2]|0;
       $46 = (($45) + 64|0);
       $47 = HEAP32[$46>>2]|0;
       $48 = HEAP32[$47>>2]|0;
       $49 = ($48|0)!=(0|0);
       if (!($49)) {
        $50 = HEAP32[$1>>2]|0;
        $51 = HEAP32[$50>>2]|0;
        HEAP32[$pVfs>>2] = $51;
        $52 = HEAP32[$pVfs>>2]|0;
        $53 = HEAP32[$1>>2]|0;
        $54 = (($53) + 172|0);
        $55 = HEAP32[$54>>2]|0;
        $56 = (_sqlite3OsAccess($52,$55,0,$bExists)|0);
        HEAP32[$rc>>2] = $56;
        $57 = HEAP32[$rc>>2]|0;
        $58 = ($57|0)==(0);
        $59 = HEAP32[$bExists>>2]|0;
        $60 = ($59|0)!=(0);
        $or$cond = $58 & $60;
        if ($or$cond) {
         HEAP32[$fout>>2] = 0;
         HEAP32[$f>>2] = 2050;
         $61 = HEAP32[$pVfs>>2]|0;
         $62 = HEAP32[$1>>2]|0;
         $63 = (($62) + 172|0);
         $64 = HEAP32[$63>>2]|0;
         $65 = HEAP32[$1>>2]|0;
         $66 = (($65) + 64|0);
         $67 = HEAP32[$66>>2]|0;
         $68 = HEAP32[$f>>2]|0;
         $69 = (_sqlite3OsOpen($61,$64,$67,$68,$fout)|0);
         HEAP32[$rc>>2] = $69;
         $70 = HEAP32[$rc>>2]|0;
         $71 = ($70|0)==(0);
         if ($71) {
          $72 = HEAP32[$fout>>2]|0;
          $73 = $72 & 1;
          $74 = ($73|0)!=(0);
          if ($74) {
           $75 = (_sqlite3CantopenError(45936)|0);
           HEAP32[$rc>>2] = $75;
           $76 = HEAP32[$1>>2]|0;
           $77 = (($76) + 64|0);
           $78 = HEAP32[$77>>2]|0;
           (_sqlite3OsClose($78)|0);
          }
         }
        }
       }
       $79 = HEAP32[$1>>2]|0;
       $80 = (($79) + 64|0);
       $81 = HEAP32[$80>>2]|0;
       $82 = HEAP32[$81>>2]|0;
       $83 = ($82|0)!=(0|0);
       $84 = HEAP32[$1>>2]|0;
       if ($83) {
        $85 = (_pagerSyncHotJournal($84)|0);
        HEAP32[$rc>>2] = $85;
        $86 = HEAP32[$rc>>2]|0;
        $87 = ($86|0)==(0);
        if ($87) {
         $88 = HEAP32[$1>>2]|0;
         $89 = (_pager_playback($88,1)|0);
         HEAP32[$rc>>2] = $89;
         $90 = HEAP32[$1>>2]|0;
         $91 = (($90) + 16|0);
         HEAP8[$91>>0] = 0;
        }
       } else {
        $92 = (($84) + 4|0);
        $93 = HEAP8[$92>>0]|0;
        $94 = ($93<<24>>24)!=(0);
        if (!($94)) {
         $95 = HEAP32[$1>>2]|0;
         (_pagerUnlockDb($95,1)|0);
        }
       }
       $96 = HEAP32[$rc>>2]|0;
       $97 = ($96|0)!=(0);
       if ($97) {
        $98 = HEAP32[$1>>2]|0;
        $99 = HEAP32[$rc>>2]|0;
        (_pager_error($98,$99)|0);
        break;
       }
      }
      $100 = HEAP32[$1>>2]|0;
      $101 = (($100) + 12|0);
      $102 = HEAP8[$101>>0]|0;
      $103 = ($102<<24>>24)!=(0);
      do {
       if (!($103)) {
        $104 = HEAP32[$1>>2]|0;
        $105 = (($104) + 88|0);
        $106 = HEAP32[$105>>2]|0;
        $107 = ($106|0)!=(0|0);
        if (!($107)) {
         $108 = HEAP32[$1>>2]|0;
         $109 = (($108) + 204|0);
         $110 = HEAP32[$109>>2]|0;
         $111 = (_sqlite3PcachePagecount($110)|0);
         $112 = ($111|0)>(0);
         if (!($112)) {
          break;
         }
        }
        HEAP32[$nPage>>2] = 0;
        $113 = HEAP32[$1>>2]|0;
        $114 = (_pagerPagecount($113,$nPage)|0);
        HEAP32[$rc>>2] = $114;
        $115 = HEAP32[$rc>>2]|0;
        $116 = ($115|0)!=(0);
        if ($116) {
         break L6;
        }
        $117 = HEAP32[$nPage>>2]|0;
        $118 = ($117>>>0)>(0);
        if ($118) {
         $119 = HEAP32[$1>>2]|0;
         $120 = (($119) + 60|0);
         $121 = HEAP32[$120>>2]|0;
         $122 = (_sqlite3OsRead($121,$dbFileVers,16,24,0)|0);
         HEAP32[$rc>>2] = $122;
         $123 = HEAP32[$rc>>2]|0;
         $124 = ($123|0)!=(0);
         $125 = HEAP32[$rc>>2]|0;
         $126 = ($125|0)!=(522);
         $or$cond3 = $124 & $126;
         if ($or$cond3) {
          break L6;
         }
        } else {
         dest=$dbFileVers+0|0; stop=dest+16|0; do { HEAP8[dest>>0]=0|0; dest=dest+1|0; } while ((dest|0) < (stop|0));
        }
        $127 = HEAP32[$1>>2]|0;
        $128 = (($127) + 100|0);
        $129 = (_memcmp($128,$dbFileVers,16)|0);
        $130 = ($129|0)!=(0);
        if ($130) {
         $131 = HEAP32[$1>>2]|0;
         _pager_reset($131);
        }
       }
      } while(0);
      $132 = HEAP32[$1>>2]|0;
      $133 = (_pagerOpenWalIfPresent($132)|0);
      HEAP32[$rc>>2] = $133;
      label = 36;
     }
    }
   } else {
    label = 36;
   }
  }
 } while(0);
 if ((label|0) == 36) {
  $134 = HEAP32[$1>>2]|0;
  $135 = (_pagerUseWal($134)|0);
  $136 = ($135|0)!=(0);
  if ($136) {
   $137 = HEAP32[$1>>2]|0;
   $138 = (_pagerBeginReadTransaction($137)|0);
   HEAP32[$rc>>2] = $138;
  }
  $139 = HEAP32[$1>>2]|0;
  $140 = (($139) + 16|0);
  $141 = HEAP8[$140>>0]|0;
  $142 = $141&255;
  $143 = ($142|0)==(0);
  $144 = HEAP32[$rc>>2]|0;
  $145 = ($144|0)==(0);
  $or$cond5 = $143 & $145;
  if ($or$cond5) {
   $146 = HEAP32[$1>>2]|0;
   $147 = HEAP32[$1>>2]|0;
   $148 = (($147) + 24|0);
   $149 = (_pagerPagecount($146,$148)|0);
   HEAP32[$rc>>2] = $149;
  }
 }
 $150 = HEAP32[$rc>>2]|0;
 $151 = ($150|0)!=(0);
 $152 = HEAP32[$1>>2]|0;
 if ($151) {
  _pager_unlock($152);
 } else {
  $153 = (($152) + 16|0);
  HEAP8[$153>>0] = 1;
 }
 $154 = HEAP32[$rc>>2]|0;
 HEAP32[$0>>2] = $154;
 $155 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($155|0);
}
function _hasHotJournal($pPager,$pExists) {
 $pPager = $pPager|0;
 $pExists = $pExists|0;
 var $$ = 0, $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0;
 var $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0;
 var $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0;
 var $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0;
 var $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $9 = 0, $exists = 0, $f = 0, $first = 0, $jrnlOpen = 0, $locked = 0, $nPage = 0, $or$cond = 0, $or$cond3 = 0, $or$cond5 = 0, $pVfs = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $0 = sp + 32|0;
 $1 = sp + 28|0;
 $pVfs = sp + 24|0;
 $rc = sp + 20|0;
 $exists = sp + 16|0;
 $jrnlOpen = sp + 12|0;
 $locked = sp + 8|0;
 $nPage = sp + 4|0;
 $f = sp;
 $first = sp + 36|0;
 HEAP32[$0>>2] = $pPager;
 HEAP32[$1>>2] = $pExists;
 $2 = HEAP32[$0>>2]|0;
 $3 = HEAP32[$2>>2]|0;
 HEAP32[$pVfs>>2] = $3;
 HEAP32[$rc>>2] = 0;
 HEAP32[$exists>>2] = 1;
 $4 = HEAP32[$0>>2]|0;
 $5 = (($4) + 64|0);
 $6 = HEAP32[$5>>2]|0;
 $7 = HEAP32[$6>>2]|0;
 $8 = ($7|0)!=(0|0);
 $9 = $8 ^ 1;
 $10 = $9 ^ 1;
 $11 = $10&1;
 HEAP32[$jrnlOpen>>2] = $11;
 $12 = HEAP32[$1>>2]|0;
 HEAP32[$12>>2] = 0;
 $13 = HEAP32[$jrnlOpen>>2]|0;
 $14 = ($13|0)!=(0);
 if (!($14)) {
  $15 = HEAP32[$pVfs>>2]|0;
  $16 = HEAP32[$0>>2]|0;
  $17 = (($16) + 172|0);
  $18 = HEAP32[$17>>2]|0;
  $19 = (_sqlite3OsAccess($15,$18,0,$exists)|0);
  HEAP32[$rc>>2] = $19;
 }
 $20 = HEAP32[$rc>>2]|0;
 $21 = ($20|0)==(0);
 $22 = HEAP32[$exists>>2]|0;
 $23 = ($22|0)!=(0);
 $or$cond = $21 & $23;
 if (!($or$cond)) {
  $84 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($84|0);
 }
 HEAP32[$locked>>2] = 0;
 $24 = HEAP32[$0>>2]|0;
 $25 = (($24) + 60|0);
 $26 = HEAP32[$25>>2]|0;
 $27 = (_sqlite3OsCheckReservedLock($26,$locked)|0);
 HEAP32[$rc>>2] = $27;
 $28 = HEAP32[$rc>>2]|0;
 $29 = ($28|0)!=(0);
 $30 = HEAP32[$locked>>2]|0;
 $31 = ($30|0)!=(0);
 $or$cond3 = $29 | $31;
 if ($or$cond3) {
  $84 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($84|0);
 }
 $32 = HEAP32[$0>>2]|0;
 $33 = (_pagerPagecount($32,$nPage)|0);
 HEAP32[$rc>>2] = $33;
 $34 = HEAP32[$rc>>2]|0;
 $35 = ($34|0)==(0);
 if (!($35)) {
  $84 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($84|0);
 }
 $36 = HEAP32[$nPage>>2]|0;
 $37 = ($36|0)!=(0);
 $38 = HEAP32[$jrnlOpen>>2]|0;
 $39 = ($38|0)!=(0);
 $or$cond5 = $37 | $39;
 if (!($or$cond5)) {
  _sqlite3BeginBenignMalloc();
  $40 = HEAP32[$0>>2]|0;
  $41 = (_pagerLockDb($40,2)|0);
  $42 = ($41|0)==(0);
  if ($42) {
   $43 = HEAP32[$pVfs>>2]|0;
   $44 = HEAP32[$0>>2]|0;
   $45 = (($44) + 172|0);
   $46 = HEAP32[$45>>2]|0;
   (_sqlite3OsDelete($43,$46,0)|0);
   $47 = HEAP32[$0>>2]|0;
   $48 = (($47) + 4|0);
   $49 = HEAP8[$48>>0]|0;
   $50 = ($49<<24>>24)!=(0);
   if (!($50)) {
    $51 = HEAP32[$0>>2]|0;
    (_pagerUnlockDb($51,1)|0);
   }
  }
  _sqlite3EndBenignMalloc();
  $84 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($84|0);
 }
 $52 = HEAP32[$jrnlOpen>>2]|0;
 $53 = ($52|0)!=(0);
 if (!($53)) {
  HEAP32[$f>>2] = 2049;
  $54 = HEAP32[$pVfs>>2]|0;
  $55 = HEAP32[$0>>2]|0;
  $56 = (($55) + 172|0);
  $57 = HEAP32[$56>>2]|0;
  $58 = HEAP32[$0>>2]|0;
  $59 = (($58) + 64|0);
  $60 = HEAP32[$59>>2]|0;
  $61 = HEAP32[$f>>2]|0;
  $62 = (_sqlite3OsOpen($54,$57,$60,$61,$f)|0);
  HEAP32[$rc>>2] = $62;
 }
 $63 = HEAP32[$rc>>2]|0;
 $64 = ($63|0)==(0);
 if ($64) {
  HEAP8[$first>>0] = 0;
  $65 = HEAP32[$0>>2]|0;
  $66 = (($65) + 64|0);
  $67 = HEAP32[$66>>2]|0;
  $68 = (_sqlite3OsRead($67,$first,1,0,0)|0);
  HEAP32[$rc>>2] = $68;
  $69 = HEAP32[$rc>>2]|0;
  $70 = ($69|0)==(522);
  $$ = $70 ? 0 : $68;
  HEAP32[$rc>>2] = $$;
  $71 = HEAP32[$jrnlOpen>>2]|0;
  $72 = ($71|0)!=(0);
  if (!($72)) {
   $73 = HEAP32[$0>>2]|0;
   $74 = (($73) + 64|0);
   $75 = HEAP32[$74>>2]|0;
   (_sqlite3OsClose($75)|0);
  }
  $76 = HEAP8[$first>>0]|0;
  $77 = $76&255;
  $78 = ($77|0)!=(0);
  $79 = $78&1;
  $80 = HEAP32[$1>>2]|0;
  HEAP32[$80>>2] = $79;
  $84 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($84|0);
 } else {
  $81 = HEAP32[$rc>>2]|0;
  $82 = ($81|0)==(14);
  if (!($82)) {
   $84 = HEAP32[$rc>>2]|0;
   STACKTOP = sp;return ($84|0);
  }
  $83 = HEAP32[$1>>2]|0;
  HEAP32[$83>>2] = 1;
  HEAP32[$rc>>2] = 0;
  $84 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($84|0);
 }
 return 0|0;
}
function _sqlite3PcachePagecount($pCache) {
 $pCache = $pCache|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 HEAP32[$0>>2] = $pCache;
 $1 = HEAP32[((38640 + 128|0))>>2]|0;
 $2 = HEAP32[$0>>2]|0;
 $3 = (($2) + 40|0);
 $4 = HEAP32[$3>>2]|0;
 $5 = (FUNCTION_TABLE_ii[$1 & 31]($4)|0);
 STACKTOP = sp;return ($5|0);
}
function _pagerPagecount($pPager,$pnPage) {
 $pPager = $pPager|0;
 $pnPage = $pnPage|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $n = 0, $nPage = 0;
 var $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 24|0;
 $1 = sp + 20|0;
 $2 = sp + 16|0;
 $nPage = sp + 12|0;
 $n = sp;
 $rc = sp + 8|0;
 HEAP32[$1>>2] = $pPager;
 HEAP32[$2>>2] = $pnPage;
 $3 = HEAP32[$1>>2]|0;
 $4 = (($3) + 208|0);
 $5 = HEAP32[$4>>2]|0;
 $6 = (_sqlite3WalDbsize($5)|0);
 HEAP32[$nPage>>2] = $6;
 $7 = HEAP32[$nPage>>2]|0;
 $8 = ($7|0)==(0);
 if ($8) {
  $9 = $n;
  $10 = $9;
  HEAP32[$10>>2] = 0;
  $11 = (($9) + 4)|0;
  $12 = $11;
  HEAP32[$12>>2] = 0;
  $13 = HEAP32[$1>>2]|0;
  $14 = (($13) + 60|0);
  $15 = HEAP32[$14>>2]|0;
  $16 = HEAP32[$15>>2]|0;
  $17 = ($16|0)!=(0|0);
  if ($17) {
   $18 = HEAP32[$1>>2]|0;
   $19 = (($18) + 60|0);
   $20 = HEAP32[$19>>2]|0;
   $21 = (_sqlite3OsFileSize($20,$n)|0);
   HEAP32[$rc>>2] = $21;
   $22 = HEAP32[$rc>>2]|0;
   $23 = ($22|0)!=(0);
   if ($23) {
    $24 = HEAP32[$rc>>2]|0;
    HEAP32[$0>>2] = $24;
    $57 = HEAP32[$0>>2]|0;
    STACKTOP = sp;return ($57|0);
   }
  }
  $25 = $n;
  $26 = $25;
  $27 = HEAP32[$26>>2]|0;
  $28 = (($25) + 4)|0;
  $29 = $28;
  $30 = HEAP32[$29>>2]|0;
  $31 = HEAP32[$1>>2]|0;
  $32 = (($31) + 152|0);
  $33 = HEAP32[$32>>2]|0;
  $34 = ($33|0)<(0);
  $35 = $34 << 31 >> 31;
  $36 = (_i64Add(($27|0),($30|0),($33|0),($35|0))|0);
  $37 = tempRet0;
  $38 = (_i64Subtract(($36|0),($37|0),1,0)|0);
  $39 = tempRet0;
  $40 = HEAP32[$1>>2]|0;
  $41 = (($40) + 152|0);
  $42 = HEAP32[$41>>2]|0;
  $43 = ($42|0)<(0);
  $44 = $43 << 31 >> 31;
  $45 = (___divdi3(($38|0),($39|0),($42|0),($44|0))|0);
  $46 = tempRet0;
  HEAP32[$nPage>>2] = $45;
 }
 $47 = HEAP32[$nPage>>2]|0;
 $48 = HEAP32[$1>>2]|0;
 $49 = (($48) + 156|0);
 $50 = HEAP32[$49>>2]|0;
 $51 = ($47>>>0)>($50>>>0);
 if ($51) {
  $52 = HEAP32[$nPage>>2]|0;
  $53 = HEAP32[$1>>2]|0;
  $54 = (($53) + 156|0);
  HEAP32[$54>>2] = $52;
 }
 $55 = HEAP32[$nPage>>2]|0;
 $56 = HEAP32[$2>>2]|0;
 HEAP32[$56>>2] = $55;
 HEAP32[$0>>2] = 0;
 $57 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($57|0);
}
function _pagerOpenWalIfPresent($pPager) {
 $pPager = $pPager|0;
 var $$ = 0, $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0;
 var $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $isWal = 0, $nPage = 0, $rc = 0;
 var label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 16|0;
 $1 = sp + 12|0;
 $rc = sp + 8|0;
 $isWal = sp + 4|0;
 $nPage = sp;
 HEAP32[$1>>2] = $pPager;
 HEAP32[$rc>>2] = 0;
 $2 = HEAP32[$1>>2]|0;
 $3 = (($2) + 12|0);
 $4 = HEAP8[$3>>0]|0;
 $5 = ($4<<24>>24)!=(0);
 do {
  if (!($5)) {
   $6 = HEAP32[$1>>2]|0;
   $7 = (_pagerPagecount($6,$nPage)|0);
   HEAP32[$rc>>2] = $7;
   $8 = HEAP32[$rc>>2]|0;
   $9 = ($8|0)!=(0);
   if ($9) {
    $10 = HEAP32[$rc>>2]|0;
    HEAP32[$0>>2] = $10;
    $35 = HEAP32[$0>>2]|0;
    STACKTOP = sp;return ($35|0);
   }
   $11 = HEAP32[$nPage>>2]|0;
   $12 = ($11|0)==(0);
   $13 = HEAP32[$1>>2]|0;
   $14 = HEAP32[$13>>2]|0;
   $15 = HEAP32[$1>>2]|0;
   $16 = (($15) + 212|0);
   $17 = HEAP32[$16>>2]|0;
   if ($12) {
    $18 = (_sqlite3OsDelete($14,$17,0)|0);
    HEAP32[$rc>>2] = $18;
    $19 = HEAP32[$rc>>2]|0;
    $20 = ($19|0)==(5898);
    $$ = $20 ? 0 : $18;
    HEAP32[$rc>>2] = $$;
    HEAP32[$isWal>>2] = 0;
   } else {
    $21 = (_sqlite3OsAccess($14,$17,0,$isWal)|0);
    HEAP32[$rc>>2] = $21;
   }
   $22 = HEAP32[$rc>>2]|0;
   $23 = ($22|0)==(0);
   if ($23) {
    $24 = HEAP32[$isWal>>2]|0;
    $25 = ($24|0)!=(0);
    $26 = HEAP32[$1>>2]|0;
    if ($25) {
     $27 = (_sqlite3PagerOpenWal($26,0)|0);
     HEAP32[$rc>>2] = $27;
     break;
    }
    $28 = (($26) + 5|0);
    $29 = HEAP8[$28>>0]|0;
    $30 = $29&255;
    $31 = ($30|0)==(5);
    if ($31) {
     $32 = HEAP32[$1>>2]|0;
     $33 = (($32) + 5|0);
     HEAP8[$33>>0] = 0;
    }
   }
  }
 } while(0);
 $34 = HEAP32[$rc>>2]|0;
 HEAP32[$0>>2] = $34;
 $35 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($35|0);
}
function _pagerBeginReadTransaction($pPager) {
 $pPager = $pPager|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $changed = 0, $or$cond = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $rc = sp + 4|0;
 $changed = sp;
 HEAP32[$0>>2] = $pPager;
 HEAP32[$changed>>2] = 0;
 $1 = HEAP32[$0>>2]|0;
 $2 = (($1) + 208|0);
 $3 = HEAP32[$2>>2]|0;
 _sqlite3WalEndReadTransaction($3);
 $4 = HEAP32[$0>>2]|0;
 $5 = (($4) + 208|0);
 $6 = HEAP32[$5>>2]|0;
 $7 = (_sqlite3WalBeginReadTransaction($6,$changed)|0);
 HEAP32[$rc>>2] = $7;
 $8 = HEAP32[$rc>>2]|0;
 $9 = ($8|0)!=(0);
 $10 = HEAP32[$changed>>2]|0;
 $11 = ($10|0)!=(0);
 $or$cond = $9 | $11;
 if (!($or$cond)) {
  $13 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($13|0);
 }
 $12 = HEAP32[$0>>2]|0;
 _pager_reset($12);
 $13 = HEAP32[$rc>>2]|0;
 STACKTOP = sp;return ($13|0);
}
function _sqlite3WalBeginReadTransaction($pWal,$pChanged) {
 $pWal = $pWal|0;
 $pChanged = $pChanged|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $cnt = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $1 = sp + 8|0;
 $rc = sp + 4|0;
 $cnt = sp;
 HEAP32[$0>>2] = $pWal;
 HEAP32[$1>>2] = $pChanged;
 HEAP32[$cnt>>2] = 0;
 while(1) {
  $2 = HEAP32[$0>>2]|0;
  $3 = HEAP32[$1>>2]|0;
  $4 = HEAP32[$cnt>>2]|0;
  $5 = (($4) + 1)|0;
  HEAP32[$cnt>>2] = $5;
  $6 = (_walTryBeginRead($2,$3,0,$5)|0);
  HEAP32[$rc>>2] = $6;
  $7 = HEAP32[$rc>>2]|0;
  $8 = ($7|0)==(-1);
  if (!($8)) {
   break;
  }
 }
 $9 = HEAP32[$rc>>2]|0;
 STACKTOP = sp;return ($9|0);
}
function _sqlite3PagerOpenWal($pPager,$pbOpen) {
 $pPager = $pPager|0;
 $pbOpen = $pbOpen|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $1 = sp + 8|0;
 $2 = sp + 4|0;
 $rc = sp;
 HEAP32[$1>>2] = $pPager;
 HEAP32[$2>>2] = $pbOpen;
 HEAP32[$rc>>2] = 0;
 $3 = HEAP32[$1>>2]|0;
 $4 = (($3) + 12|0);
 $5 = HEAP8[$4>>0]|0;
 $6 = ($5<<24>>24)!=(0);
 do {
  if ($6) {
   label = 7;
  } else {
   $7 = HEAP32[$1>>2]|0;
   $8 = (($7) + 208|0);
   $9 = HEAP32[$8>>2]|0;
   $10 = ($9|0)!=(0|0);
   if ($10) {
    label = 7;
   } else {
    $11 = HEAP32[$1>>2]|0;
    $12 = (_sqlite3PagerWalSupported($11)|0);
    $13 = ($12|0)!=(0);
    if ($13) {
     $14 = HEAP32[$1>>2]|0;
     $15 = (($14) + 64|0);
     $16 = HEAP32[$15>>2]|0;
     (_sqlite3OsClose($16)|0);
     $17 = HEAP32[$1>>2]|0;
     $18 = (_pagerOpenWal($17)|0);
     HEAP32[$rc>>2] = $18;
     $19 = HEAP32[$rc>>2]|0;
     $20 = ($19|0)==(0);
     if (!($20)) {
      break;
     }
     $21 = HEAP32[$1>>2]|0;
     $22 = (($21) + 5|0);
     HEAP8[$22>>0] = 5;
     $23 = HEAP32[$1>>2]|0;
     $24 = (($23) + 16|0);
     HEAP8[$24>>0] = 0;
     break;
    } else {
     HEAP32[$0>>2] = 14;
     $27 = HEAP32[$0>>2]|0;
     STACKTOP = sp;return ($27|0);
    }
   }
  }
 } while(0);
 if ((label|0) == 7) {
  $25 = HEAP32[$2>>2]|0;
  HEAP32[$25>>2] = 1;
 }
 $26 = HEAP32[$rc>>2]|0;
 HEAP32[$0>>2] = $26;
 $27 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($27|0);
}
function _pagerOpenWal($pPager) {
 $pPager = $pPager|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $rc = sp;
 HEAP32[$0>>2] = $pPager;
 HEAP32[$rc>>2] = 0;
 $1 = HEAP32[$0>>2]|0;
 $2 = (($1) + 4|0);
 $3 = HEAP8[$2>>0]|0;
 $4 = ($3<<24>>24)!=(0);
 if ($4) {
  $5 = HEAP32[$0>>2]|0;
  $6 = (_pagerExclusiveLock($5)|0);
  HEAP32[$rc>>2] = $6;
 }
 $7 = HEAP32[$rc>>2]|0;
 $8 = ($7|0)==(0);
 if (!($8)) {
  $32 = HEAP32[$0>>2]|0;
  _pagerFixMaplimit($32);
  $33 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($33|0);
 }
 $9 = HEAP32[$0>>2]|0;
 $10 = HEAP32[$9>>2]|0;
 $11 = HEAP32[$0>>2]|0;
 $12 = (($11) + 60|0);
 $13 = HEAP32[$12>>2]|0;
 $14 = HEAP32[$0>>2]|0;
 $15 = (($14) + 212|0);
 $16 = HEAP32[$15>>2]|0;
 $17 = HEAP32[$0>>2]|0;
 $18 = (($17) + 4|0);
 $19 = HEAP8[$18>>0]|0;
 $20 = $19&255;
 $21 = HEAP32[$0>>2]|0;
 $22 = (($21) + 160|0);
 $23 = $22;
 $24 = $23;
 $25 = HEAP32[$24>>2]|0;
 $26 = (($23) + 4)|0;
 $27 = $26;
 $28 = HEAP32[$27>>2]|0;
 $29 = HEAP32[$0>>2]|0;
 $30 = (($29) + 208|0);
 $31 = (_sqlite3WalOpen($10,$13,$16,$20,$25,$28,$30)|0);
 HEAP32[$rc>>2] = $31;
 $32 = HEAP32[$0>>2]|0;
 _pagerFixMaplimit($32);
 $33 = HEAP32[$rc>>2]|0;
 STACKTOP = sp;return ($33|0);
}
function _pagerExclusiveLock($pPager) {
 $pPager = $pPager|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $rc = sp;
 HEAP32[$0>>2] = $pPager;
 $1 = HEAP32[$0>>2]|0;
 $2 = (_pagerLockDb($1,4)|0);
 HEAP32[$rc>>2] = $2;
 $3 = HEAP32[$rc>>2]|0;
 $4 = ($3|0)!=(0);
 if (!($4)) {
  $6 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($6|0);
 }
 $5 = HEAP32[$0>>2]|0;
 (_pagerUnlockDb($5,1)|0);
 $6 = HEAP32[$rc>>2]|0;
 STACKTOP = sp;return ($6|0);
}
function _sqlite3WalOpen($pVfs,$pDbFd,$zWalName,$bNoShm,$0,$1,$ppWal) {
 $pVfs = $pVfs|0;
 $pDbFd = $pDbFd|0;
 $zWalName = $zWalName|0;
 $bNoShm = $bNoShm|0;
 $0 = $0|0;
 $1 = $1|0;
 $ppWal = $ppWal|0;
 var $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0;
 var $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0;
 var $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0;
 var $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0;
 var $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $flags = 0, $iDC = 0, $pRet = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $2 = sp + 44|0;
 $3 = sp + 40|0;
 $4 = sp + 36|0;
 $5 = sp + 32|0;
 $6 = sp + 28|0;
 $7 = sp;
 $8 = sp + 24|0;
 $rc = sp + 20|0;
 $pRet = sp + 16|0;
 $flags = sp + 12|0;
 $iDC = sp + 8|0;
 HEAP32[$3>>2] = $pVfs;
 HEAP32[$4>>2] = $pDbFd;
 HEAP32[$5>>2] = $zWalName;
 HEAP32[$6>>2] = $bNoShm;
 $9 = $7;
 $10 = $9;
 HEAP32[$10>>2] = $0;
 $11 = (($9) + 4)|0;
 $12 = $11;
 HEAP32[$12>>2] = $1;
 HEAP32[$8>>2] = $ppWal;
 $13 = HEAP32[$8>>2]|0;
 HEAP32[$13>>2] = 0;
 $14 = HEAP32[$3>>2]|0;
 $15 = (($14) + 4|0);
 $16 = HEAP32[$15>>2]|0;
 $17 = (112 + ($16))|0;
 $18 = (_sqlite3MallocZero($17,0)|0);
 HEAP32[$pRet>>2] = $18;
 $19 = HEAP32[$pRet>>2]|0;
 $20 = ($19|0)!=(0|0);
 if (!($20)) {
  HEAP32[$2>>2] = 7;
  $93 = HEAP32[$2>>2]|0;
  STACKTOP = sp;return ($93|0);
 }
 $21 = HEAP32[$3>>2]|0;
 $22 = HEAP32[$pRet>>2]|0;
 HEAP32[$22>>2] = $21;
 $23 = HEAP32[$pRet>>2]|0;
 $24 = (($23) + 112|0);
 $25 = HEAP32[$pRet>>2]|0;
 $26 = (($25) + 8|0);
 HEAP32[$26>>2] = $24;
 $27 = HEAP32[$4>>2]|0;
 $28 = HEAP32[$pRet>>2]|0;
 $29 = (($28) + 4|0);
 HEAP32[$29>>2] = $27;
 $30 = HEAP32[$pRet>>2]|0;
 $31 = (($30) + 40|0);
 HEAP16[$31>>1] = -1;
 $32 = $7;
 $33 = $32;
 $34 = HEAP32[$33>>2]|0;
 $35 = (($32) + 4)|0;
 $36 = $35;
 $37 = HEAP32[$36>>2]|0;
 $38 = HEAP32[$pRet>>2]|0;
 $39 = (($38) + 16|0);
 $40 = $39;
 $41 = $40;
 HEAP32[$41>>2] = $34;
 $42 = (($40) + 4)|0;
 $43 = $42;
 HEAP32[$43>>2] = $37;
 $44 = HEAP32[$5>>2]|0;
 $45 = HEAP32[$pRet>>2]|0;
 $46 = (($45) + 100|0);
 HEAP32[$46>>2] = $44;
 $47 = HEAP32[$pRet>>2]|0;
 $48 = (($47) + 48|0);
 HEAP8[$48>>0] = 1;
 $49 = HEAP32[$pRet>>2]|0;
 $50 = (($49) + 49|0);
 HEAP8[$50>>0] = 1;
 $51 = HEAP32[$6>>2]|0;
 $52 = ($51|0)!=(0);
 $53 = $52 ? 2 : 0;
 $54 = $53&255;
 $55 = HEAP32[$pRet>>2]|0;
 $56 = (($55) + 43|0);
 HEAP8[$56>>0] = $54;
 HEAP32[$flags>>2] = 524294;
 $57 = HEAP32[$3>>2]|0;
 $58 = HEAP32[$5>>2]|0;
 $59 = HEAP32[$pRet>>2]|0;
 $60 = (($59) + 8|0);
 $61 = HEAP32[$60>>2]|0;
 $62 = HEAP32[$flags>>2]|0;
 $63 = (_sqlite3OsOpen($57,$58,$61,$62,$flags)|0);
 HEAP32[$rc>>2] = $63;
 $64 = HEAP32[$rc>>2]|0;
 $65 = ($64|0)==(0);
 if ($65) {
  $66 = HEAP32[$flags>>2]|0;
  $67 = $66 & 1;
  $68 = ($67|0)!=(0);
  if ($68) {
   $69 = HEAP32[$pRet>>2]|0;
   $70 = (($69) + 46|0);
   HEAP8[$70>>0] = 1;
  }
 }
 $71 = HEAP32[$rc>>2]|0;
 $72 = ($71|0)!=(0);
 if ($72) {
  $73 = HEAP32[$pRet>>2]|0;
  _walIndexClose($73,0);
  $74 = HEAP32[$pRet>>2]|0;
  $75 = (($74) + 8|0);
  $76 = HEAP32[$75>>2]|0;
  (_sqlite3OsClose($76)|0);
  $77 = HEAP32[$pRet>>2]|0;
  _sqlite3_free($77);
 } else {
  $78 = HEAP32[$4>>2]|0;
  $79 = (_sqlite3OsDeviceCharacteristics($78)|0);
  HEAP32[$iDC>>2] = $79;
  $80 = HEAP32[$iDC>>2]|0;
  $81 = $80 & 1024;
  $82 = ($81|0)!=(0);
  if ($82) {
   $83 = HEAP32[$pRet>>2]|0;
   $84 = (($83) + 48|0);
   HEAP8[$84>>0] = 0;
  }
  $85 = HEAP32[$iDC>>2]|0;
  $86 = $85 & 4096;
  $87 = ($86|0)!=(0);
  if ($87) {
   $88 = HEAP32[$pRet>>2]|0;
   $89 = (($88) + 49|0);
   HEAP8[$89>>0] = 0;
  }
  $90 = HEAP32[$pRet>>2]|0;
  $91 = HEAP32[$8>>2]|0;
  HEAP32[$91>>2] = $90;
 }
 $92 = HEAP32[$rc>>2]|0;
 HEAP32[$2>>2] = $92;
 $93 = HEAP32[$2>>2]|0;
 STACKTOP = sp;return ($93|0);
}
function _sqlite3WalDbsize($pWal) {
 $pWal = $pWal|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $1 = sp;
 HEAP32[$1>>2] = $pWal;
 $2 = HEAP32[$1>>2]|0;
 $3 = ($2|0)!=(0|0);
 if ($3) {
  $4 = HEAP32[$1>>2]|0;
  $5 = (($4) + 40|0);
  $6 = HEAP16[$5>>1]|0;
  $7 = $6 << 16 >> 16;
  $8 = ($7|0)>=(0);
  if ($8) {
   $9 = HEAP32[$1>>2]|0;
   $10 = (($9) + 52|0);
   $11 = (($10) + 20|0);
   $12 = HEAP32[$11>>2]|0;
   HEAP32[$0>>2] = $12;
   $13 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($13|0);
  }
 }
 HEAP32[$0>>2] = 0;
 $13 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($13|0);
}
function _sqlite3OsCheckReservedLock($id,$pResOut) {
 $id = $id|0;
 $pResOut = $pResOut|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $1 = sp;
 HEAP32[$0>>2] = $id;
 HEAP32[$1>>2] = $pResOut;
 $2 = HEAP32[$0>>2]|0;
 $3 = HEAP32[$2>>2]|0;
 $4 = (($3) + 36|0);
 $5 = HEAP32[$4>>2]|0;
 $6 = HEAP32[$0>>2]|0;
 $7 = HEAP32[$1>>2]|0;
 $8 = (FUNCTION_TABLE_iii[$5 & 63]($6,$7)|0);
 STACKTOP = sp;return ($8|0);
}
function _rowSetEntrySort($pIn) {
 $pIn = $pIn|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $aBucket = 0, $i = 0, $pNext = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 176|0;
 $0 = sp + 168|0;
 $i = sp + 164|0;
 $pNext = sp + 160|0;
 $aBucket = sp;
 HEAP32[$0>>2] = $pIn;
 _memset(($aBucket|0),0,160)|0;
 while(1) {
  $1 = HEAP32[$0>>2]|0;
  $2 = ($1|0)!=(0|0);
  if (!($2)) {
   break;
  }
  $3 = HEAP32[$0>>2]|0;
  $4 = (($3) + 8|0);
  $5 = HEAP32[$4>>2]|0;
  HEAP32[$pNext>>2] = $5;
  $6 = HEAP32[$0>>2]|0;
  $7 = (($6) + 8|0);
  HEAP32[$7>>2] = 0;
  HEAP32[$i>>2] = 0;
  while(1) {
   $8 = HEAP32[$i>>2]|0;
   $9 = (($aBucket) + ($8<<2)|0);
   $10 = HEAP32[$9>>2]|0;
   $11 = ($10|0)!=(0|0);
   if (!($11)) {
    break;
   }
   $12 = HEAP32[$i>>2]|0;
   $13 = (($aBucket) + ($12<<2)|0);
   $14 = HEAP32[$13>>2]|0;
   $15 = HEAP32[$0>>2]|0;
   $16 = (_rowSetEntryMerge($14,$15)|0);
   HEAP32[$0>>2] = $16;
   $17 = HEAP32[$i>>2]|0;
   $18 = (($aBucket) + ($17<<2)|0);
   HEAP32[$18>>2] = 0;
   $19 = HEAP32[$i>>2]|0;
   $20 = (($19) + 1)|0;
   HEAP32[$i>>2] = $20;
  }
  $21 = HEAP32[$0>>2]|0;
  $22 = HEAP32[$i>>2]|0;
  $23 = (($aBucket) + ($22<<2)|0);
  HEAP32[$23>>2] = $21;
  $24 = HEAP32[$pNext>>2]|0;
  HEAP32[$0>>2] = $24;
 }
 HEAP32[$0>>2] = 0;
 HEAP32[$i>>2] = 0;
 while(1) {
  $25 = HEAP32[$i>>2]|0;
  $26 = ($25>>>0)<(40);
  $27 = HEAP32[$0>>2]|0;
  if (!($26)) {
   break;
  }
  $28 = HEAP32[$i>>2]|0;
  $29 = (($aBucket) + ($28<<2)|0);
  $30 = HEAP32[$29>>2]|0;
  $31 = (_rowSetEntryMerge($27,$30)|0);
  HEAP32[$0>>2] = $31;
  $32 = HEAP32[$i>>2]|0;
  $33 = (($32) + 1)|0;
  HEAP32[$i>>2] = $33;
 }
 STACKTOP = sp;return ($27|0);
}
function _rowSetListToTree($pList) {
 $pList = $pList|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $3 = 0, $4 = 0;
 var $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $iDepth = 0, $p = 0, $pLeft = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $iDepth = sp + 8|0;
 $p = sp + 4|0;
 $pLeft = sp;
 HEAP32[$0>>2] = $pList;
 $1 = HEAP32[$0>>2]|0;
 HEAP32[$p>>2] = $1;
 $2 = HEAP32[$p>>2]|0;
 $3 = (($2) + 8|0);
 $4 = HEAP32[$3>>2]|0;
 HEAP32[$0>>2] = $4;
 $5 = HEAP32[$p>>2]|0;
 $6 = (($5) + 8|0);
 HEAP32[$6>>2] = 0;
 $7 = HEAP32[$p>>2]|0;
 $8 = (($7) + 12|0);
 HEAP32[$8>>2] = 0;
 HEAP32[$iDepth>>2] = 1;
 while(1) {
  $9 = HEAP32[$0>>2]|0;
  $10 = ($9|0)!=(0|0);
  $11 = HEAP32[$p>>2]|0;
  if (!($10)) {
   break;
  }
  HEAP32[$pLeft>>2] = $11;
  $12 = HEAP32[$0>>2]|0;
  HEAP32[$p>>2] = $12;
  $13 = HEAP32[$p>>2]|0;
  $14 = (($13) + 8|0);
  $15 = HEAP32[$14>>2]|0;
  HEAP32[$0>>2] = $15;
  $16 = HEAP32[$pLeft>>2]|0;
  $17 = HEAP32[$p>>2]|0;
  $18 = (($17) + 12|0);
  HEAP32[$18>>2] = $16;
  $19 = HEAP32[$iDepth>>2]|0;
  $20 = (_rowSetNDeepTree($0,$19)|0);
  $21 = HEAP32[$p>>2]|0;
  $22 = (($21) + 8|0);
  HEAP32[$22>>2] = $20;
  $23 = HEAP32[$iDepth>>2]|0;
  $24 = (($23) + 1)|0;
  HEAP32[$iDepth>>2] = $24;
 }
 STACKTOP = sp;return ($11|0);
}
function _rowSetTreeToList($pIn,$ppFirst,$ppLast) {
 $pIn = $pIn|0;
 $ppFirst = $ppFirst|0;
 $ppLast = $ppLast|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $3 = 0;
 var $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $p = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $1 = sp + 8|0;
 $2 = sp + 4|0;
 $p = sp;
 HEAP32[$0>>2] = $pIn;
 HEAP32[$1>>2] = $ppFirst;
 HEAP32[$2>>2] = $ppLast;
 $3 = HEAP32[$0>>2]|0;
 $4 = (($3) + 12|0);
 $5 = HEAP32[$4>>2]|0;
 $6 = ($5|0)!=(0|0);
 $7 = HEAP32[$0>>2]|0;
 if ($6) {
  $8 = (($7) + 12|0);
  $9 = HEAP32[$8>>2]|0;
  $10 = HEAP32[$1>>2]|0;
  _rowSetTreeToList($9,$10,$p);
  $11 = HEAP32[$0>>2]|0;
  $12 = HEAP32[$p>>2]|0;
  $13 = (($12) + 8|0);
  HEAP32[$13>>2] = $11;
 } else {
  $14 = HEAP32[$1>>2]|0;
  HEAP32[$14>>2] = $7;
 }
 $15 = HEAP32[$0>>2]|0;
 $16 = (($15) + 8|0);
 $17 = HEAP32[$16>>2]|0;
 $18 = ($17|0)!=(0|0);
 $19 = HEAP32[$0>>2]|0;
 if ($18) {
  $20 = (($19) + 8|0);
  $21 = HEAP32[$20>>2]|0;
  $22 = HEAP32[$0>>2]|0;
  $23 = (($22) + 8|0);
  $24 = HEAP32[$2>>2]|0;
  _rowSetTreeToList($21,$23,$24);
  STACKTOP = sp;return;
 } else {
  $25 = HEAP32[$2>>2]|0;
  HEAP32[$25>>2] = $19;
  STACKTOP = sp;return;
 }
}
function _rowSetEntryMerge($pA,$pB) {
 $pA = $pA|0;
 $pB = $pB|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $8 = 0, $9 = 0, $head = 0, $pTail = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 24|0;
 $1 = sp + 20|0;
 $head = sp;
 $pTail = sp + 16|0;
 HEAP32[$0>>2] = $pA;
 HEAP32[$1>>2] = $pB;
 HEAP32[$pTail>>2] = $head;
 while(1) {
  $2 = HEAP32[$0>>2]|0;
  $3 = ($2|0)!=(0|0);
  if ($3) {
   $4 = HEAP32[$1>>2]|0;
   $5 = ($4|0)!=(0|0);
   $74 = $5;
  } else {
   $74 = 0;
  }
  $6 = HEAP32[$0>>2]|0;
  if (!($74)) {
   break;
  }
  $7 = $6;
  $8 = $7;
  $9 = HEAP32[$8>>2]|0;
  $10 = (($7) + 4)|0;
  $11 = $10;
  $12 = HEAP32[$11>>2]|0;
  $13 = HEAP32[$1>>2]|0;
  $14 = $13;
  $15 = $14;
  $16 = HEAP32[$15>>2]|0;
  $17 = (($14) + 4)|0;
  $18 = $17;
  $19 = HEAP32[$18>>2]|0;
  $20 = ($12|0)<($19|0);
  $21 = ($9>>>0)<($16>>>0);
  $22 = ($12|0)==($19|0);
  $23 = $22 & $21;
  $24 = $20 | $23;
  if ($24) {
   $25 = HEAP32[$0>>2]|0;
   $26 = HEAP32[$pTail>>2]|0;
   $27 = (($26) + 8|0);
   HEAP32[$27>>2] = $25;
   $28 = HEAP32[$0>>2]|0;
   $29 = (($28) + 8|0);
   $30 = HEAP32[$29>>2]|0;
   HEAP32[$0>>2] = $30;
   $31 = HEAP32[$pTail>>2]|0;
   $32 = (($31) + 8|0);
   $33 = HEAP32[$32>>2]|0;
   HEAP32[$pTail>>2] = $33;
   continue;
  }
  $34 = HEAP32[$1>>2]|0;
  $35 = $34;
  $36 = $35;
  $37 = HEAP32[$36>>2]|0;
  $38 = (($35) + 4)|0;
  $39 = $38;
  $40 = HEAP32[$39>>2]|0;
  $41 = HEAP32[$0>>2]|0;
  $42 = $41;
  $43 = $42;
  $44 = HEAP32[$43>>2]|0;
  $45 = (($42) + 4)|0;
  $46 = $45;
  $47 = HEAP32[$46>>2]|0;
  $48 = ($40|0)<($47|0);
  $49 = ($37>>>0)<($44>>>0);
  $50 = ($40|0)==($47|0);
  $51 = $50 & $49;
  $52 = $48 | $51;
  if ($52) {
   $53 = HEAP32[$1>>2]|0;
   $54 = HEAP32[$pTail>>2]|0;
   $55 = (($54) + 8|0);
   HEAP32[$55>>2] = $53;
   $56 = HEAP32[$1>>2]|0;
   $57 = (($56) + 8|0);
   $58 = HEAP32[$57>>2]|0;
   HEAP32[$1>>2] = $58;
   $59 = HEAP32[$pTail>>2]|0;
   $60 = (($59) + 8|0);
   $61 = HEAP32[$60>>2]|0;
   HEAP32[$pTail>>2] = $61;
   continue;
  } else {
   $62 = HEAP32[$0>>2]|0;
   $63 = (($62) + 8|0);
   $64 = HEAP32[$63>>2]|0;
   HEAP32[$0>>2] = $64;
   continue;
  }
 }
 $65 = ($6|0)!=(0|0);
 if ($65) {
  $66 = HEAP32[$0>>2]|0;
  $67 = HEAP32[$pTail>>2]|0;
  $68 = (($67) + 8|0);
  HEAP32[$68>>2] = $66;
  $72 = (($head) + 8|0);
  $73 = HEAP32[$72>>2]|0;
  STACKTOP = sp;return ($73|0);
 } else {
  $69 = HEAP32[$1>>2]|0;
  $70 = HEAP32[$pTail>>2]|0;
  $71 = (($70) + 8|0);
  HEAP32[$71>>2] = $69;
  $72 = (($head) + 8|0);
  $73 = HEAP32[$72>>2]|0;
  STACKTOP = sp;return ($73|0);
 }
 return 0|0;
}
function _rowSetEntryAlloc($p) {
 $p = $p|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $pNew = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $1 = sp + 4|0;
 $pNew = sp;
 HEAP32[$1>>2] = $p;
 $2 = HEAP32[$1>>2]|0;
 $3 = (($2) + 24|0);
 $4 = HEAP16[$3>>1]|0;
 $5 = $4&65535;
 $6 = ($5|0)==(0);
 do {
  if ($6) {
   $7 = HEAP32[$1>>2]|0;
   $8 = (($7) + 4|0);
   $9 = HEAP32[$8>>2]|0;
   $10 = (_sqlite3DbMallocRaw($9,1016,0)|0);
   HEAP32[$pNew>>2] = $10;
   $11 = HEAP32[$pNew>>2]|0;
   $12 = ($11|0)==(0|0);
   if (!($12)) {
    $13 = HEAP32[$1>>2]|0;
    $14 = HEAP32[$13>>2]|0;
    $15 = HEAP32[$pNew>>2]|0;
    HEAP32[$15>>2] = $14;
    $16 = HEAP32[$pNew>>2]|0;
    $17 = HEAP32[$1>>2]|0;
    HEAP32[$17>>2] = $16;
    $18 = HEAP32[$pNew>>2]|0;
    $19 = (($18) + 8|0);
    $20 = HEAP32[$1>>2]|0;
    $21 = (($20) + 16|0);
    HEAP32[$21>>2] = $19;
    $22 = HEAP32[$1>>2]|0;
    $23 = (($22) + 24|0);
    HEAP16[$23>>1] = 63;
    break;
   }
   HEAP32[$0>>2] = 0;
   $32 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($32|0);
  }
 } while(0);
 $24 = HEAP32[$1>>2]|0;
 $25 = (($24) + 24|0);
 $26 = HEAP16[$25>>1]|0;
 $27 = (($26) + -1)<<16>>16;
 HEAP16[$25>>1] = $27;
 $28 = HEAP32[$1>>2]|0;
 $29 = (($28) + 16|0);
 $30 = HEAP32[$29>>2]|0;
 $31 = (($30) + 16|0);
 HEAP32[$29>>2] = $31;
 HEAP32[$0>>2] = $30;
 $32 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($32|0);
}
function _rowSetNDeepTree($ppList,$iDepth) {
 $ppList = $ppList|0;
 $iDepth = $iDepth|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0;
 var $9 = 0, $p = 0, $pLeft = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 16|0;
 $1 = sp + 12|0;
 $2 = sp + 8|0;
 $p = sp + 4|0;
 $pLeft = sp;
 HEAP32[$1>>2] = $ppList;
 HEAP32[$2>>2] = $iDepth;
 $3 = HEAP32[$1>>2]|0;
 $4 = HEAP32[$3>>2]|0;
 $5 = ($4|0)==(0|0);
 if ($5) {
  HEAP32[$0>>2] = 0;
  $40 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($40|0);
 }
 $6 = HEAP32[$2>>2]|0;
 $7 = ($6|0)==(1);
 $8 = HEAP32[$1>>2]|0;
 if ($7) {
  $9 = HEAP32[$8>>2]|0;
  HEAP32[$p>>2] = $9;
  $10 = HEAP32[$p>>2]|0;
  $11 = (($10) + 8|0);
  $12 = HEAP32[$11>>2]|0;
  $13 = HEAP32[$1>>2]|0;
  HEAP32[$13>>2] = $12;
  $14 = HEAP32[$p>>2]|0;
  $15 = (($14) + 8|0);
  HEAP32[$15>>2] = 0;
  $16 = HEAP32[$p>>2]|0;
  $17 = (($16) + 12|0);
  HEAP32[$17>>2] = 0;
  $18 = HEAP32[$p>>2]|0;
  HEAP32[$0>>2] = $18;
  $40 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($40|0);
 }
 $19 = HEAP32[$2>>2]|0;
 $20 = (($19) - 1)|0;
 $21 = (_rowSetNDeepTree($8,$20)|0);
 HEAP32[$pLeft>>2] = $21;
 $22 = HEAP32[$1>>2]|0;
 $23 = HEAP32[$22>>2]|0;
 HEAP32[$p>>2] = $23;
 $24 = HEAP32[$p>>2]|0;
 $25 = ($24|0)==(0|0);
 $26 = HEAP32[$pLeft>>2]|0;
 if ($25) {
  HEAP32[$0>>2] = $26;
  $40 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($40|0);
 } else {
  $27 = HEAP32[$p>>2]|0;
  $28 = (($27) + 12|0);
  HEAP32[$28>>2] = $26;
  $29 = HEAP32[$p>>2]|0;
  $30 = (($29) + 8|0);
  $31 = HEAP32[$30>>2]|0;
  $32 = HEAP32[$1>>2]|0;
  HEAP32[$32>>2] = $31;
  $33 = HEAP32[$1>>2]|0;
  $34 = HEAP32[$2>>2]|0;
  $35 = (($34) - 1)|0;
  $36 = (_rowSetNDeepTree($33,$35)|0);
  $37 = HEAP32[$p>>2]|0;
  $38 = (($37) + 8|0);
  HEAP32[$38>>2] = $36;
  $39 = HEAP32[$p>>2]|0;
  HEAP32[$0>>2] = $39;
  $40 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($40|0);
 }
 return 0|0;
}
function _rowSetToList($p) {
 $p = $p|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0;
 var sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 HEAP32[$0>>2] = $p;
 $1 = HEAP32[$0>>2]|0;
 $2 = (($1) + 26|0);
 $3 = HEAP16[$2>>1]|0;
 $4 = $3&65535;
 $5 = $4 & 1;
 $6 = ($5|0)==(0);
 if ($6) {
  $7 = HEAP32[$0>>2]|0;
  $8 = (($7) + 8|0);
  $9 = HEAP32[$8>>2]|0;
  $10 = (_rowSetEntrySort($9)|0);
  $11 = HEAP32[$0>>2]|0;
  $12 = (($11) + 8|0);
  HEAP32[$12>>2] = $10;
 }
 $13 = HEAP32[$0>>2]|0;
 $14 = (($13) + 26|0);
 $15 = HEAP16[$14>>1]|0;
 $16 = $15&65535;
 $17 = $16 | 2;
 $18 = $17&65535;
 HEAP16[$14>>1] = $18;
 STACKTOP = sp;return;
}
function _sqlite3RowSetInit($db,$pSpace,$N) {
 $db = $db|0;
 $pSpace = $pSpace|0;
 $N = $N|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $p = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $1 = sp + 8|0;
 $2 = sp + 4|0;
 $p = sp;
 HEAP32[$0>>2] = $db;
 HEAP32[$1>>2] = $pSpace;
 HEAP32[$2>>2] = $N;
 $3 = HEAP32[$1>>2]|0;
 HEAP32[$p>>2] = $3;
 $4 = HEAP32[$p>>2]|0;
 HEAP32[$4>>2] = 0;
 $5 = HEAP32[$0>>2]|0;
 $6 = HEAP32[$p>>2]|0;
 $7 = (($6) + 4|0);
 HEAP32[$7>>2] = $5;
 $8 = HEAP32[$p>>2]|0;
 $9 = (($8) + 8|0);
 HEAP32[$9>>2] = 0;
 $10 = HEAP32[$p>>2]|0;
 $11 = (($10) + 12|0);
 HEAP32[$11>>2] = 0;
 $12 = HEAP32[$p>>2]|0;
 $13 = (($12) + 20|0);
 HEAP32[$13>>2] = 0;
 $14 = HEAP32[$p>>2]|0;
 $15 = (($14) + 32|0);
 $16 = HEAP32[$p>>2]|0;
 $17 = (($16) + 16|0);
 HEAP32[$17>>2] = $15;
 $18 = HEAP32[$2>>2]|0;
 $19 = (($18) - 32)|0;
 $20 = (($19>>>0) / 16)&-1;
 $21 = $20&65535;
 $22 = HEAP32[$p>>2]|0;
 $23 = (($22) + 24|0);
 HEAP16[$23>>1] = $21;
 $24 = HEAP32[$p>>2]|0;
 $25 = (($24) + 26|0);
 HEAP16[$25>>1] = 1;
 $26 = HEAP32[$p>>2]|0;
 $27 = (($26) + 28|0);
 HEAP32[$27>>2] = 0;
 $28 = HEAP32[$p>>2]|0;
 STACKTOP = sp;return ($28|0);
}
function _sqlite3PagerRefcount($pPager) {
 $pPager = $pPager|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 HEAP32[$0>>2] = $pPager;
 $1 = HEAP32[$0>>2]|0;
 $2 = (($1) + 204|0);
 $3 = HEAP32[$2>>2]|0;
 $4 = (_sqlite3PcacheRefCount($3)|0);
 STACKTOP = sp;return ($4|0);
}
function _setPageReferenced($pCheck,$iPg) {
 $pCheck = $pCheck|0;
 $iPg = $iPg|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $1 = sp;
 HEAP32[$0>>2] = $pCheck;
 HEAP32[$1>>2] = $iPg;
 $2 = HEAP32[$1>>2]|0;
 $3 = $2 & 7;
 $4 = 1 << $3;
 $5 = HEAP32[$1>>2]|0;
 $6 = (($5>>>0) / 8)&-1;
 $7 = HEAP32[$0>>2]|0;
 $8 = (($7) + 8|0);
 $9 = HEAP32[$8>>2]|0;
 $10 = (($9) + ($6)|0);
 $11 = HEAP8[$10>>0]|0;
 $12 = $11&255;
 $13 = $12 | $4;
 $14 = $13&255;
 HEAP8[$10>>0] = $14;
 STACKTOP = sp;return;
}
function _checkList($pCheck,$isFreeList,$iPage,$N) {
 $pCheck = $pCheck|0;
 $isFreeList = $isFreeList|0;
 $iPage = $iPage|0;
 $N = $N|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0;
 var $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $expected = 0, $i = 0, $iFirst = 0, $iFreePage = 0;
 var $n = 0, $or$cond = 0, $pOvflData = 0, $pOvflPage = 0, $vararg_buffer7 = 0, $vararg_ptr2 = 0, $vararg_ptr3 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 64|0;
 $vararg_buffer7 = sp;
 $0 = sp + 52|0;
 $1 = sp + 48|0;
 $2 = sp + 44|0;
 $3 = sp + 40|0;
 $i = sp + 36|0;
 $expected = sp + 32|0;
 $iFirst = sp + 28|0;
 $pOvflPage = sp + 24|0;
 $pOvflData = sp + 20|0;
 $n = sp + 16|0;
 $iFreePage = sp + 12|0;
 HEAP32[$0>>2] = $pCheck;
 HEAP32[$1>>2] = $isFreeList;
 HEAP32[$2>>2] = $iPage;
 HEAP32[$3>>2] = $N;
 $4 = HEAP32[$3>>2]|0;
 HEAP32[$expected>>2] = $4;
 $5 = HEAP32[$2>>2]|0;
 HEAP32[$iFirst>>2] = $5;
 while(1) {
  $6 = HEAP32[$3>>2]|0;
  $7 = (($6) + -1)|0;
  HEAP32[$3>>2] = $7;
  $8 = ($6|0)>(0);
  if (!($8)) {
   label = 23;
   break;
  }
  $9 = HEAP32[$0>>2]|0;
  $10 = (($9) + 16|0);
  $11 = HEAP32[$10>>2]|0;
  $12 = ($11|0)!=(0);
  if (!($12)) {
   label = 23;
   break;
  }
  $13 = HEAP32[$2>>2]|0;
  $14 = ($13|0)<(1);
  $15 = HEAP32[$0>>2]|0;
  if ($14) {
   label = 5;
   break;
  }
  $20 = HEAP32[$2>>2]|0;
  $21 = (_checkRef($15,$20)|0);
  $22 = ($21|0)!=(0);
  if ($22) {
   label = 23;
   break;
  }
  $23 = HEAP32[$0>>2]|0;
  $24 = (($23) + 4|0);
  $25 = HEAP32[$24>>2]|0;
  $26 = HEAP32[$2>>2]|0;
  $27 = (_sqlite3PagerAcquire($25,$26,$pOvflPage,0)|0);
  $28 = ($27|0)!=(0);
  if ($28) {
   label = 8;
   break;
  }
  $31 = HEAP32[$pOvflPage>>2]|0;
  $32 = (_sqlite3PagerGetData($31)|0);
  HEAP32[$pOvflData>>2] = $32;
  $33 = HEAP32[$1>>2]|0;
  $34 = ($33|0)!=(0);
  do {
   if ($34) {
    $35 = HEAP32[$pOvflData>>2]|0;
    $36 = (($35) + 4|0);
    $37 = (_sqlite3Get4byte($36)|0);
    HEAP32[$n>>2] = $37;
    $38 = HEAP32[$0>>2]|0;
    $39 = HEAP32[$38>>2]|0;
    $40 = (($39) + 17|0);
    $41 = HEAP8[$40>>0]|0;
    $42 = ($41<<24>>24)!=(0);
    if ($42) {
     $43 = HEAP32[$0>>2]|0;
     $44 = HEAP32[$2>>2]|0;
     _checkPtrmap($43,$44,2,0);
    }
    $45 = HEAP32[$n>>2]|0;
    $46 = HEAP32[$0>>2]|0;
    $47 = HEAP32[$46>>2]|0;
    $48 = (($47) + 36|0);
    $49 = HEAP32[$48>>2]|0;
    $50 = (($49|0) / 4)&-1;
    $51 = (($50) - 2)|0;
    $52 = ($45|0)>($51|0);
    if ($52) {
     $53 = HEAP32[$0>>2]|0;
     $54 = HEAP32[$2>>2]|0;
     HEAP32[$vararg_buffer7>>2] = $54;
     _checkAppendMsg($53,34368,$vararg_buffer7);
     $55 = HEAP32[$3>>2]|0;
     $56 = (($55) + -1)|0;
     HEAP32[$3>>2] = $56;
     break;
    }
    HEAP32[$i>>2] = 0;
    while(1) {
     $57 = HEAP32[$i>>2]|0;
     $58 = HEAP32[$n>>2]|0;
     $59 = ($57|0)<($58|0);
     if (!($59)) {
      break;
     }
     $60 = HEAP32[$i>>2]|0;
     $61 = $60<<2;
     $62 = (8 + ($61))|0;
     $63 = HEAP32[$pOvflData>>2]|0;
     $64 = (($63) + ($62)|0);
     $65 = (_sqlite3Get4byte($64)|0);
     HEAP32[$iFreePage>>2] = $65;
     $66 = HEAP32[$0>>2]|0;
     $67 = HEAP32[$66>>2]|0;
     $68 = (($67) + 17|0);
     $69 = HEAP8[$68>>0]|0;
     $70 = ($69<<24>>24)!=(0);
     if ($70) {
      $71 = HEAP32[$0>>2]|0;
      $72 = HEAP32[$iFreePage>>2]|0;
      _checkPtrmap($71,$72,2,0);
     }
     $73 = HEAP32[$0>>2]|0;
     $74 = HEAP32[$iFreePage>>2]|0;
     (_checkRef($73,$74)|0);
     $75 = HEAP32[$i>>2]|0;
     $76 = (($75) + 1)|0;
     HEAP32[$i>>2] = $76;
    }
    $77 = HEAP32[$n>>2]|0;
    $78 = HEAP32[$3>>2]|0;
    $79 = (($78) - ($77))|0;
    HEAP32[$3>>2] = $79;
   } else {
    $80 = HEAP32[$0>>2]|0;
    $81 = HEAP32[$80>>2]|0;
    $82 = (($81) + 17|0);
    $83 = HEAP8[$82>>0]|0;
    $84 = $83&255;
    $85 = ($84|0)!=(0);
    $86 = HEAP32[$3>>2]|0;
    $87 = ($86|0)>(0);
    $or$cond = $85 & $87;
    if ($or$cond) {
     $88 = HEAP32[$pOvflData>>2]|0;
     $89 = (_sqlite3Get4byte($88)|0);
     HEAP32[$i>>2] = $89;
     $90 = HEAP32[$0>>2]|0;
     $91 = HEAP32[$i>>2]|0;
     $92 = HEAP32[$2>>2]|0;
     _checkPtrmap($90,$91,4,$92);
    }
   }
  } while(0);
  $93 = HEAP32[$pOvflData>>2]|0;
  $94 = (_sqlite3Get4byte($93)|0);
  HEAP32[$2>>2] = $94;
  $95 = HEAP32[$pOvflPage>>2]|0;
  _sqlite3PagerUnref($95);
 }
 if ((label|0) == 5) {
  $16 = HEAP32[$3>>2]|0;
  $17 = (($16) + 1)|0;
  $18 = HEAP32[$expected>>2]|0;
  $19 = HEAP32[$iFirst>>2]|0;
  HEAP32[$vararg_buffer7>>2] = $17;
  $vararg_ptr2 = (($vararg_buffer7) + 4|0);
  HEAP32[$vararg_ptr2>>2] = $18;
  $vararg_ptr3 = (($vararg_buffer7) + 8|0);
  HEAP32[$vararg_ptr3>>2] = $19;
  _checkAppendMsg($15,34280,$vararg_buffer7);
  STACKTOP = sp;return;
 }
 else if ((label|0) == 8) {
  $29 = HEAP32[$0>>2]|0;
  $30 = HEAP32[$2>>2]|0;
  HEAP32[$vararg_buffer7>>2] = $30;
  _checkAppendMsg($29,34344,$vararg_buffer7);
  STACKTOP = sp;return;
 }
 else if ((label|0) == 23) {
  STACKTOP = sp;return;
 }
}
function _checkPtrmap($pCheck,$iChild,$eType,$iParent) {
 $pCheck = $pCheck|0;
 $iChild = $iChild|0;
 $eType = $eType|0;
 $iParent = $iParent|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $ePtrmapType = 0, $iPtrmapParent = 0, $or$cond = 0, $rc = 0, $vararg_buffer2 = 0, $vararg_ptr5 = 0;
 var $vararg_ptr6 = 0, $vararg_ptr7 = 0, $vararg_ptr8 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $vararg_buffer2 = sp;
 $0 = sp + 36|0;
 $1 = sp + 32|0;
 $2 = sp + 41|0;
 $3 = sp + 28|0;
 $rc = sp + 24|0;
 $ePtrmapType = sp + 40|0;
 $iPtrmapParent = sp + 20|0;
 HEAP32[$0>>2] = $pCheck;
 HEAP32[$1>>2] = $iChild;
 HEAP8[$2>>0] = $eType;
 HEAP32[$3>>2] = $iParent;
 $4 = HEAP32[$0>>2]|0;
 $5 = HEAP32[$4>>2]|0;
 $6 = HEAP32[$1>>2]|0;
 $7 = (_ptrmapGet($5,$6,$ePtrmapType,$iPtrmapParent)|0);
 HEAP32[$rc>>2] = $7;
 $8 = HEAP32[$rc>>2]|0;
 $9 = ($8|0)!=(0);
 if ($9) {
  $10 = HEAP32[$rc>>2]|0;
  $11 = ($10|0)==(7);
  $12 = HEAP32[$rc>>2]|0;
  $13 = ($12|0)==(3082);
  $or$cond = $11 | $13;
  if ($or$cond) {
   $14 = HEAP32[$0>>2]|0;
   $15 = (($14) + 24|0);
   HEAP32[$15>>2] = 1;
  }
  $16 = HEAP32[$0>>2]|0;
  $17 = HEAP32[$1>>2]|0;
  HEAP32[$vararg_buffer2>>2] = $17;
  _checkAppendMsg($16,34192,$vararg_buffer2);
  STACKTOP = sp;return;
 }
 $18 = HEAP8[$ePtrmapType>>0]|0;
 $19 = $18&255;
 $20 = HEAP8[$2>>0]|0;
 $21 = $20&255;
 $22 = ($19|0)!=($21|0);
 if (!($22)) {
  $23 = HEAP32[$iPtrmapParent>>2]|0;
  $24 = HEAP32[$3>>2]|0;
  $25 = ($23|0)!=($24|0);
  if (!($25)) {
   STACKTOP = sp;return;
  }
 }
 $26 = HEAP32[$0>>2]|0;
 $27 = HEAP32[$1>>2]|0;
 $28 = HEAP8[$2>>0]|0;
 $29 = $28&255;
 $30 = HEAP32[$3>>2]|0;
 $31 = HEAP8[$ePtrmapType>>0]|0;
 $32 = $31&255;
 $33 = HEAP32[$iPtrmapParent>>2]|0;
 HEAP32[$vararg_buffer2>>2] = $27;
 $vararg_ptr5 = (($vararg_buffer2) + 4|0);
 HEAP32[$vararg_ptr5>>2] = $29;
 $vararg_ptr6 = (($vararg_buffer2) + 8|0);
 HEAP32[$vararg_ptr6>>2] = $30;
 $vararg_ptr7 = (($vararg_buffer2) + 12|0);
 HEAP32[$vararg_ptr7>>2] = $32;
 $vararg_ptr8 = (($vararg_buffer2) + 16|0);
 HEAP32[$vararg_ptr8>>2] = $33;
 _checkAppendMsg($26,34224,$vararg_buffer2);
 STACKTOP = sp;return;
}
function _checkTreePage($pCheck,$iPage,$pnParentMinKey,$pnParentMaxKey) {
 $pCheck = $pCheck|0;
 $iPage = $iPage|0;
 $pnParentMinKey = $pnParentMinKey|0;
 $pnParentMaxKey = $pnParentMaxKey|0;
 var $$nMaxKey = 0, $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0;
 var $115 = 0, $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0;
 var $133 = 0, $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0;
 var $151 = 0, $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0;
 var $17 = 0, $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0;
 var $188 = 0, $189 = 0, $19 = 0, $190 = 0, $191 = 0, $192 = 0, $193 = 0, $194 = 0, $195 = 0, $196 = 0, $197 = 0, $198 = 0, $199 = 0, $2 = 0, $20 = 0, $200 = 0, $201 = 0, $202 = 0, $203 = 0, $204 = 0;
 var $205 = 0, $206 = 0, $207 = 0, $208 = 0, $209 = 0, $21 = 0, $210 = 0, $211 = 0, $212 = 0, $213 = 0, $214 = 0, $215 = 0, $216 = 0, $217 = 0, $218 = 0, $219 = 0, $22 = 0, $220 = 0, $221 = 0, $222 = 0;
 var $223 = 0, $224 = 0, $225 = 0, $226 = 0, $227 = 0, $228 = 0, $229 = 0, $23 = 0, $230 = 0, $231 = 0, $232 = 0, $233 = 0, $234 = 0, $235 = 0, $236 = 0, $237 = 0, $238 = 0, $239 = 0, $24 = 0, $240 = 0;
 var $241 = 0, $242 = 0, $243 = 0, $244 = 0, $245 = 0, $246 = 0, $247 = 0, $248 = 0, $249 = 0, $25 = 0, $250 = 0, $251 = 0, $252 = 0, $253 = 0, $254 = 0, $255 = 0, $256 = 0, $257 = 0, $258 = 0, $259 = 0;
 var $26 = 0, $260 = 0, $261 = 0, $262 = 0, $263 = 0, $264 = 0, $265 = 0, $266 = 0, $267 = 0, $268 = 0, $269 = 0, $27 = 0, $270 = 0, $271 = 0, $272 = 0, $273 = 0, $274 = 0, $275 = 0, $276 = 0, $277 = 0;
 var $278 = 0, $279 = 0, $28 = 0, $280 = 0, $281 = 0, $282 = 0, $283 = 0, $284 = 0, $285 = 0, $286 = 0, $287 = 0, $288 = 0, $289 = 0, $29 = 0, $290 = 0, $291 = 0, $292 = 0, $293 = 0, $294 = 0, $295 = 0;
 var $296 = 0, $297 = 0, $298 = 0, $299 = 0, $3 = 0, $30 = 0, $300 = 0, $301 = 0, $302 = 0, $303 = 0, $304 = 0, $305 = 0, $306 = 0, $307 = 0, $308 = 0, $309 = 0, $31 = 0, $310 = 0, $311 = 0, $312 = 0;
 var $313 = 0, $314 = 0, $315 = 0, $316 = 0, $317 = 0, $318 = 0, $319 = 0, $32 = 0, $320 = 0, $321 = 0, $322 = 0, $323 = 0, $324 = 0, $325 = 0, $326 = 0, $327 = 0, $328 = 0, $329 = 0, $33 = 0, $330 = 0;
 var $331 = 0, $332 = 0, $333 = 0, $334 = 0, $335 = 0, $336 = 0, $337 = 0, $338 = 0, $339 = 0, $34 = 0, $340 = 0, $341 = 0, $342 = 0, $343 = 0, $344 = 0, $345 = 0, $346 = 0, $347 = 0, $348 = 0, $349 = 0;
 var $35 = 0, $350 = 0, $351 = 0, $352 = 0, $353 = 0, $354 = 0, $355 = 0, $356 = 0, $357 = 0, $358 = 0, $359 = 0, $36 = 0, $360 = 0, $361 = 0, $362 = 0, $363 = 0, $364 = 0, $365 = 0, $366 = 0, $367 = 0;
 var $368 = 0, $369 = 0, $37 = 0, $370 = 0, $371 = 0, $372 = 0, $373 = 0, $374 = 0, $375 = 0, $376 = 0, $377 = 0, $378 = 0, $379 = 0, $38 = 0, $380 = 0, $381 = 0, $382 = 0, $383 = 0, $384 = 0, $385 = 0;
 var $386 = 0, $387 = 0, $388 = 0, $389 = 0, $39 = 0, $390 = 0, $391 = 0, $392 = 0, $393 = 0, $394 = 0, $395 = 0, $396 = 0, $397 = 0, $398 = 0, $399 = 0, $4 = 0, $40 = 0, $400 = 0, $401 = 0, $402 = 0;
 var $403 = 0, $404 = 0, $405 = 0, $406 = 0, $407 = 0, $408 = 0, $409 = 0, $41 = 0, $410 = 0, $411 = 0, $412 = 0, $413 = 0, $414 = 0, $415 = 0, $416 = 0, $417 = 0, $418 = 0, $419 = 0, $42 = 0, $420 = 0;
 var $421 = 0, $422 = 0, $423 = 0, $424 = 0, $425 = 0, $426 = 0, $427 = 0, $428 = 0, $429 = 0, $43 = 0, $430 = 0, $431 = 0, $432 = 0, $433 = 0, $434 = 0, $435 = 0, $436 = 0, $437 = 0, $438 = 0, $439 = 0;
 var $44 = 0, $440 = 0, $441 = 0, $442 = 0, $443 = 0, $444 = 0, $445 = 0, $446 = 0, $447 = 0, $448 = 0, $449 = 0, $45 = 0, $450 = 0, $451 = 0, $452 = 0, $453 = 0, $454 = 0, $455 = 0, $456 = 0, $457 = 0;
 var $458 = 0, $459 = 0, $46 = 0, $460 = 0, $461 = 0, $462 = 0, $463 = 0, $464 = 0, $465 = 0, $466 = 0, $467 = 0, $468 = 0, $469 = 0, $47 = 0, $470 = 0, $471 = 0, $472 = 0, $473 = 0, $474 = 0, $475 = 0;
 var $476 = 0, $477 = 0, $478 = 0, $479 = 0, $48 = 0, $480 = 0, $481 = 0, $482 = 0, $483 = 0, $484 = 0, $485 = 0, $486 = 0, $487 = 0, $488 = 0, $489 = 0, $49 = 0, $490 = 0, $491 = 0, $492 = 0, $493 = 0;
 var $494 = 0, $495 = 0, $496 = 0, $497 = 0, $498 = 0, $499 = 0, $5 = 0, $50 = 0, $500 = 0, $501 = 0, $502 = 0, $503 = 0, $504 = 0, $505 = 0, $506 = 0, $507 = 0, $508 = 0, $509 = 0, $51 = 0, $510 = 0;
 var $511 = 0, $512 = 0, $513 = 0, $514 = 0, $515 = 0, $516 = 0, $517 = 0, $518 = 0, $519 = 0, $52 = 0, $520 = 0, $521 = 0, $522 = 0, $523 = 0, $524 = 0, $525 = 0, $526 = 0, $527 = 0, $528 = 0, $529 = 0;
 var $53 = 0, $530 = 0, $531 = 0, $532 = 0, $533 = 0, $534 = 0, $535 = 0, $536 = 0, $537 = 0, $538 = 0, $539 = 0, $54 = 0, $540 = 0, $541 = 0, $542 = 0, $543 = 0, $544 = 0, $545 = 0, $546 = 0, $547 = 0;
 var $548 = 0, $549 = 0, $55 = 0, $550 = 0, $551 = 0, $552 = 0, $553 = 0, $554 = 0, $555 = 0, $556 = 0, $557 = 0, $558 = 0, $559 = 0, $56 = 0, $560 = 0, $561 = 0, $562 = 0, $563 = 0, $564 = 0, $565 = 0;
 var $566 = 0, $567 = 0, $568 = 0, $569 = 0, $57 = 0, $570 = 0, $571 = 0, $572 = 0, $573 = 0, $574 = 0, $575 = 0, $576 = 0, $577 = 0, $578 = 0, $579 = 0, $58 = 0, $580 = 0, $581 = 0, $582 = 0, $583 = 0;
 var $584 = 0, $585 = 0, $586 = 0, $587 = 0, $588 = 0, $589 = 0, $59 = 0, $590 = 0, $591 = 0, $592 = 0, $593 = 0, $594 = 0, $595 = 0, $596 = 0, $597 = 0, $598 = 0, $599 = 0, $6 = 0, $60 = 0, $600 = 0;
 var $601 = 0, $602 = 0, $603 = 0, $604 = 0, $605 = 0, $606 = 0, $607 = 0, $608 = 0, $609 = 0, $61 = 0, $610 = 0, $611 = 0, $612 = 0, $613 = 0, $614 = 0, $615 = 0, $616 = 0, $617 = 0, $618 = 0, $619 = 0;
 var $62 = 0, $620 = 0, $621 = 0, $622 = 0, $623 = 0, $624 = 0, $625 = 0, $626 = 0, $627 = 0, $628 = 0, $629 = 0, $63 = 0, $630 = 0, $631 = 0, $632 = 0, $633 = 0, $634 = 0, $635 = 0, $636 = 0, $637 = 0;
 var $638 = 0, $639 = 0, $64 = 0, $640 = 0, $641 = 0, $642 = 0, $643 = 0, $644 = 0, $645 = 0, $646 = 0, $647 = 0, $648 = 0, $649 = 0, $65 = 0, $650 = 0, $651 = 0, $652 = 0, $653 = 0, $654 = 0, $655 = 0;
 var $656 = 0, $657 = 0, $658 = 0, $659 = 0, $66 = 0, $660 = 0, $661 = 0, $662 = 0, $663 = 0, $664 = 0, $665 = 0, $666 = 0, $667 = 0, $668 = 0, $669 = 0, $67 = 0, $670 = 0, $671 = 0, $672 = 0, $673 = 0;
 var $674 = 0, $675 = 0, $676 = 0, $677 = 0, $678 = 0, $679 = 0, $68 = 0, $680 = 0, $681 = 0, $682 = 0, $683 = 0, $684 = 0, $685 = 0, $686 = 0, $687 = 0, $688 = 0, $689 = 0, $69 = 0, $690 = 0, $691 = 0;
 var $692 = 0, $693 = 0, $694 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0;
 var $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $cellStart = 0, $cnt = 0, $contentOffset = 0, $d2 = 0;
 var $data = 0, $depth = 0, $hdr = 0, $hit = 0, $i = 0, $info = 0, $j = 0, $j2 = 0, $nCell = 0, $nMaxKey = 0, $nMaxKey$ = 0, $nMinKey = 0, $nPage = 0, $pBt = 0, $pCell = 0, $pPage = 0, $pc = 0, $pgno = 0, $pgnoOvfl = 0, $rc = 0;
 var $saved_v1 = 0, $saved_v2 = 0, $saved_zPfx = 0, $size = 0, $size1 = 0, $sz = 0, $usableSize = 0, $vararg_buffer34 = 0, $vararg_ptr13 = 0, $vararg_ptr17 = 0, $vararg_ptr21 = 0, $vararg_ptr25 = 0, $vararg_ptr29 = 0, $vararg_ptr33 = 0, $vararg_ptr37 = 0, $vararg_ptr38 = 0, $vararg_ptr7 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 192|0;
 $vararg_buffer34 = sp + 40|0;
 $0 = sp + 180|0;
 $1 = sp + 176|0;
 $2 = sp + 172|0;
 $3 = sp + 168|0;
 $4 = sp + 164|0;
 $pPage = sp + 160|0;
 $i = sp + 156|0;
 $rc = sp + 152|0;
 $depth = sp + 148|0;
 $d2 = sp + 144|0;
 $pgno = sp + 140|0;
 $cnt = sp + 136|0;
 $hdr = sp + 132|0;
 $cellStart = sp + 128|0;
 $nCell = sp + 124|0;
 $data = sp + 120|0;
 $pBt = sp + 116|0;
 $usableSize = sp + 112|0;
 $hit = sp + 108|0;
 $nMinKey = sp + 32|0;
 $nMaxKey = sp + 24|0;
 $saved_zPfx = sp + 104|0;
 $saved_v1 = sp + 100|0;
 $saved_v2 = sp + 96|0;
 $pCell = sp + 92|0;
 $sz = sp + 88|0;
 $info = sp;
 $nPage = sp + 84|0;
 $pgnoOvfl = sp + 80|0;
 $contentOffset = sp + 76|0;
 $pc = sp + 72|0;
 $size = sp + 68|0;
 $j = sp + 64|0;
 $size1 = sp + 60|0;
 $j2 = sp + 56|0;
 HEAP32[$1>>2] = $pCheck;
 HEAP32[$2>>2] = $iPage;
 HEAP32[$3>>2] = $pnParentMinKey;
 HEAP32[$4>>2] = $pnParentMaxKey;
 HEAP32[$hit>>2] = 0;
 $5 = $nMinKey;
 $6 = $5;
 HEAP32[$6>>2] = 0;
 $7 = (($5) + 4)|0;
 $8 = $7;
 HEAP32[$8>>2] = 0;
 $9 = $nMaxKey;
 $10 = $9;
 HEAP32[$10>>2] = 0;
 $11 = (($9) + 4)|0;
 $12 = $11;
 HEAP32[$12>>2] = 0;
 $13 = HEAP32[$1>>2]|0;
 $14 = (($13) + 28|0);
 $15 = HEAP32[$14>>2]|0;
 HEAP32[$saved_zPfx>>2] = $15;
 $16 = HEAP32[$1>>2]|0;
 $17 = (($16) + 32|0);
 $18 = HEAP32[$17>>2]|0;
 HEAP32[$saved_v1>>2] = $18;
 $19 = HEAP32[$1>>2]|0;
 $20 = (($19) + 36|0);
 $21 = HEAP32[$20>>2]|0;
 HEAP32[$saved_v2>>2] = $21;
 $22 = HEAP32[$1>>2]|0;
 $23 = HEAP32[$22>>2]|0;
 HEAP32[$pBt>>2] = $23;
 $24 = HEAP32[$pBt>>2]|0;
 $25 = (($24) + 36|0);
 $26 = HEAP32[$25>>2]|0;
 HEAP32[$usableSize>>2] = $26;
 $27 = HEAP32[$2>>2]|0;
 $28 = ($27|0)==(0);
 if ($28) {
  HEAP32[$0>>2] = 0;
  $694 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($694|0);
 }
 $29 = HEAP32[$1>>2]|0;
 $30 = HEAP32[$2>>2]|0;
 $31 = (_checkRef($29,$30)|0);
 $32 = ($31|0)!=(0);
 if ($32) {
  HEAP32[$0>>2] = 0;
  $694 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($694|0);
 }
 $33 = HEAP32[$1>>2]|0;
 $34 = (($33) + 28|0);
 HEAP32[$34>>2] = 33496;
 $35 = HEAP32[$2>>2]|0;
 $36 = HEAP32[$1>>2]|0;
 $37 = (($36) + 32|0);
 HEAP32[$37>>2] = $35;
 $38 = HEAP32[$pBt>>2]|0;
 $39 = HEAP32[$2>>2]|0;
 $40 = (_btreeGetPage($38,$39,$pPage,0)|0);
 HEAP32[$rc>>2] = $40;
 $41 = ($40|0)!=(0);
 do {
  if ($41) {
   $42 = HEAP32[$1>>2]|0;
   $43 = HEAP32[$rc>>2]|0;
   HEAP32[$vararg_buffer34>>2] = $43;
   _checkAppendMsg($42,33512,$vararg_buffer34);
   HEAP32[$depth>>2] = -1;
  } else {
   $44 = HEAP32[$pPage>>2]|0;
   HEAP8[$44>>0] = 0;
   $45 = HEAP32[$pPage>>2]|0;
   $46 = (_btreeInitPage($45)|0);
   HEAP32[$rc>>2] = $46;
   $47 = ($46|0)!=(0);
   if ($47) {
    $48 = HEAP32[$1>>2]|0;
    $49 = HEAP32[$rc>>2]|0;
    HEAP32[$vararg_buffer34>>2] = $49;
    _checkAppendMsg($48,33552,$vararg_buffer34);
    $50 = HEAP32[$pPage>>2]|0;
    _releasePage($50);
    HEAP32[$depth>>2] = -1;
    break;
   }
   HEAP32[$depth>>2] = 0;
   HEAP32[$i>>2] = 0;
   while(1) {
    $51 = HEAP32[$i>>2]|0;
    $52 = HEAP32[$pPage>>2]|0;
    $53 = (($52) + 18|0);
    $54 = HEAP16[$53>>1]|0;
    $55 = $54&65535;
    $56 = ($51|0)<($55|0);
    if (!($56)) {
     break;
    }
    $57 = HEAP32[$1>>2]|0;
    $58 = (($57) + 16|0);
    $59 = HEAP32[$58>>2]|0;
    $60 = ($59|0)!=(0);
    if (!($60)) {
     break;
    }
    $61 = HEAP32[$1>>2]|0;
    $62 = (($61) + 28|0);
    HEAP32[$62>>2] = 33592;
    $63 = HEAP32[$2>>2]|0;
    $64 = HEAP32[$1>>2]|0;
    $65 = (($64) + 32|0);
    HEAP32[$65>>2] = $63;
    $66 = HEAP32[$i>>2]|0;
    $67 = HEAP32[$1>>2]|0;
    $68 = (($67) + 36|0);
    HEAP32[$68>>2] = $66;
    $69 = HEAP32[$pPage>>2]|0;
    $70 = (($69) + 56|0);
    $71 = HEAP32[$70>>2]|0;
    $72 = HEAP32[$pPage>>2]|0;
    $73 = (($72) + 20|0);
    $74 = HEAP16[$73>>1]|0;
    $75 = $74&65535;
    $76 = HEAP32[$i>>2]|0;
    $77 = $76<<1;
    $78 = HEAP32[$pPage>>2]|0;
    $79 = (($78) + 64|0);
    $80 = HEAP32[$79>>2]|0;
    $81 = (($80) + ($77)|0);
    $82 = HEAP8[$81>>0]|0;
    $83 = $82&255;
    $84 = $83 << 8;
    $85 = HEAP32[$i>>2]|0;
    $86 = $85<<1;
    $87 = HEAP32[$pPage>>2]|0;
    $88 = (($87) + 64|0);
    $89 = HEAP32[$88>>2]|0;
    $90 = (($89) + ($86)|0);
    $91 = (($90) + 1|0);
    $92 = HEAP8[$91>>0]|0;
    $93 = $92&255;
    $94 = $84 | $93;
    $95 = $75 & $94;
    $96 = (($71) + ($95)|0);
    HEAP32[$pCell>>2] = $96;
    $97 = HEAP32[$pPage>>2]|0;
    $98 = HEAP32[$pCell>>2]|0;
    _btreeParseCellPtr($97,$98,$info);
    $99 = (($info) + 12|0);
    $100 = HEAP32[$99>>2]|0;
    HEAP32[$sz>>2] = $100;
    $101 = HEAP32[$pPage>>2]|0;
    $102 = (($101) + 2|0);
    $103 = HEAP8[$102>>0]|0;
    $104 = ($103<<24>>24)!=(0);
    if ($104) {
     $105 = HEAP32[$i>>2]|0;
     $106 = ($105|0)==(0);
     $107 = $info;
     $108 = $107;
     $109 = HEAP32[$108>>2]|0;
     $110 = (($107) + 4)|0;
     $111 = $110;
     $112 = HEAP32[$111>>2]|0;
     if ($106) {
      $113 = $nMaxKey;
      $114 = $113;
      HEAP32[$114>>2] = $109;
      $115 = (($113) + 4)|0;
      $116 = $115;
      HEAP32[$116>>2] = $112;
      $117 = $nMinKey;
      $118 = $117;
      HEAP32[$118>>2] = $109;
      $119 = (($117) + 4)|0;
      $120 = $119;
      HEAP32[$120>>2] = $112;
     } else {
      $121 = $nMaxKey;
      $122 = $121;
      $123 = HEAP32[$122>>2]|0;
      $124 = (($121) + 4)|0;
      $125 = $124;
      $126 = HEAP32[$125>>2]|0;
      $127 = ($112|0)<($126|0);
      $128 = ($109>>>0)<=($123>>>0);
      $129 = ($112|0)==($126|0);
      $130 = $129 & $128;
      $131 = $127 | $130;
      if ($131) {
       $132 = HEAP32[$1>>2]|0;
       $133 = $info;
       $134 = $133;
       $135 = HEAP32[$134>>2]|0;
       $136 = (($133) + 4)|0;
       $137 = $136;
       $138 = HEAP32[$137>>2]|0;
       $139 = $nMaxKey;
       $140 = $139;
       $141 = HEAP32[$140>>2]|0;
       $142 = (($139) + 4)|0;
       $143 = $142;
       $144 = HEAP32[$143>>2]|0;
       $145 = $vararg_buffer34;
       $146 = $145;
       HEAP32[$146>>2] = $135;
       $147 = (($145) + 4)|0;
       $148 = $147;
       HEAP32[$148>>2] = $138;
       $vararg_ptr7 = (($vararg_buffer34) + 8|0);
       $149 = $vararg_ptr7;
       $150 = $149;
       HEAP32[$150>>2] = $141;
       $151 = (($149) + 4)|0;
       $152 = $151;
       HEAP32[$152>>2] = $144;
       _checkAppendMsg($132,33624,$vararg_buffer34);
      }
     }
     $153 = $info;
     $154 = $153;
     $155 = HEAP32[$154>>2]|0;
     $156 = (($153) + 4)|0;
     $157 = $156;
     $158 = HEAP32[$157>>2]|0;
     $159 = $nMaxKey;
     $160 = $159;
     HEAP32[$160>>2] = $155;
     $161 = (($159) + 4)|0;
     $162 = $161;
     HEAP32[$162>>2] = $158;
    }
    $163 = HEAP32[$sz>>2]|0;
    $164 = (($info) + 16|0);
    $165 = HEAP16[$164>>1]|0;
    $166 = $165&65535;
    $167 = ($163>>>0)>($166>>>0);
    if ($167) {
     $168 = (($info) + 18|0);
     $169 = HEAP16[$168>>1]|0;
     $170 = $169&65535;
     $171 = HEAP32[$pCell>>2]|0;
     $172 = (($171) + ($170)|0);
     $173 = HEAP32[$pBt>>2]|0;
     $174 = (($173) + 36|0);
     $175 = HEAP32[$174>>2]|0;
     $176 = HEAP32[$pPage>>2]|0;
     $177 = (($176) + 56|0);
     $178 = HEAP32[$177>>2]|0;
     $179 = (($178) + ($175)|0);
     $180 = ($172>>>0)<=($179>>>0);
     if ($180) {
      $181 = HEAP32[$sz>>2]|0;
      $182 = (($info) + 16|0);
      $183 = HEAP16[$182>>1]|0;
      $184 = $183&65535;
      $185 = (($181) - ($184))|0;
      $186 = HEAP32[$usableSize>>2]|0;
      $187 = (($185) + ($186))|0;
      $188 = (($187) - 5)|0;
      $189 = HEAP32[$usableSize>>2]|0;
      $190 = (($189) - 4)|0;
      $191 = (($188>>>0) / ($190>>>0))&-1;
      HEAP32[$nPage>>2] = $191;
      $192 = (($info) + 18|0);
      $193 = HEAP16[$192>>1]|0;
      $194 = $193&65535;
      $195 = HEAP32[$pCell>>2]|0;
      $196 = (($195) + ($194)|0);
      $197 = (_sqlite3Get4byte($196)|0);
      HEAP32[$pgnoOvfl>>2] = $197;
      $198 = HEAP32[$pBt>>2]|0;
      $199 = (($198) + 17|0);
      $200 = HEAP8[$199>>0]|0;
      $201 = ($200<<24>>24)!=(0);
      if ($201) {
       $202 = HEAP32[$1>>2]|0;
       $203 = HEAP32[$pgnoOvfl>>2]|0;
       $204 = HEAP32[$2>>2]|0;
       _checkPtrmap($202,$203,3,$204);
      }
      $205 = HEAP32[$1>>2]|0;
      $206 = HEAP32[$pgnoOvfl>>2]|0;
      $207 = HEAP32[$nPage>>2]|0;
      _checkList($205,0,$206,$207);
     }
    }
    $208 = HEAP32[$pPage>>2]|0;
    $209 = (($208) + 5|0);
    $210 = HEAP8[$209>>0]|0;
    $211 = ($210<<24>>24)!=(0);
    if (!($211)) {
     $212 = HEAP32[$pCell>>2]|0;
     $213 = (_sqlite3Get4byte($212)|0);
     HEAP32[$pgno>>2] = $213;
     $214 = HEAP32[$pBt>>2]|0;
     $215 = (($214) + 17|0);
     $216 = HEAP8[$215>>0]|0;
     $217 = ($216<<24>>24)!=(0);
     if ($217) {
      $218 = HEAP32[$1>>2]|0;
      $219 = HEAP32[$pgno>>2]|0;
      $220 = HEAP32[$2>>2]|0;
      _checkPtrmap($218,$219,5,$220);
     }
     $221 = HEAP32[$1>>2]|0;
     $222 = HEAP32[$pgno>>2]|0;
     $223 = HEAP32[$i>>2]|0;
     $224 = ($223|0)==(0);
     $$nMaxKey = $224 ? 0 : $nMaxKey;
     $225 = (_checkTreePage($221,$222,$nMinKey,$$nMaxKey)|0);
     HEAP32[$d2>>2] = $225;
     $226 = HEAP32[$i>>2]|0;
     $227 = ($226|0)>(0);
     if ($227) {
      $228 = HEAP32[$d2>>2]|0;
      $229 = HEAP32[$depth>>2]|0;
      $230 = ($228|0)!=($229|0);
      if ($230) {
       $231 = HEAP32[$1>>2]|0;
       _checkAppendMsg($231,33672,$vararg_buffer34);
      }
     }
     $232 = HEAP32[$d2>>2]|0;
     HEAP32[$depth>>2] = $232;
    }
    $233 = HEAP32[$i>>2]|0;
    $234 = (($233) + 1)|0;
    HEAP32[$i>>2] = $234;
   }
   $235 = HEAP32[$pPage>>2]|0;
   $236 = (($235) + 5|0);
   $237 = HEAP8[$236>>0]|0;
   $238 = ($237<<24>>24)!=(0);
   if (!($238)) {
    $239 = HEAP32[$pPage>>2]|0;
    $240 = (($239) + 6|0);
    $241 = HEAP8[$240>>0]|0;
    $242 = $241&255;
    $243 = (($242) + 8)|0;
    $244 = HEAP32[$pPage>>2]|0;
    $245 = (($244) + 56|0);
    $246 = HEAP32[$245>>2]|0;
    $247 = (($246) + ($243)|0);
    $248 = (_sqlite3Get4byte($247)|0);
    HEAP32[$pgno>>2] = $248;
    $249 = HEAP32[$1>>2]|0;
    $250 = (($249) + 28|0);
    HEAP32[$250>>2] = 33704;
    $251 = HEAP32[$2>>2]|0;
    $252 = HEAP32[$1>>2]|0;
    $253 = (($252) + 32|0);
    HEAP32[$253>>2] = $251;
    $254 = HEAP32[$pBt>>2]|0;
    $255 = (($254) + 17|0);
    $256 = HEAP8[$255>>0]|0;
    $257 = ($256<<24>>24)!=(0);
    if ($257) {
     $258 = HEAP32[$1>>2]|0;
     $259 = HEAP32[$pgno>>2]|0;
     $260 = HEAP32[$2>>2]|0;
     _checkPtrmap($258,$259,5,$260);
    }
    $261 = HEAP32[$1>>2]|0;
    $262 = HEAP32[$pgno>>2]|0;
    $263 = HEAP32[$pPage>>2]|0;
    $264 = (($263) + 18|0);
    $265 = HEAP16[$264>>1]|0;
    $266 = ($265<<16>>16)!=(0);
    $nMaxKey$ = $266 ? $nMaxKey : 0;
    (_checkTreePage($261,$262,0,$nMaxKey$)|0);
   }
   $267 = HEAP32[$1>>2]|0;
   $268 = (($267) + 28|0);
   HEAP32[$268>>2] = 33496;
   $269 = HEAP32[$2>>2]|0;
   $270 = HEAP32[$1>>2]|0;
   $271 = (($270) + 32|0);
   HEAP32[$271>>2] = $269;
   $272 = HEAP32[$pPage>>2]|0;
   $273 = (($272) + 5|0);
   $274 = HEAP8[$273>>0]|0;
   $275 = $274&255;
   $276 = ($275|0)!=(0);
   do {
    if ($276) {
     $277 = HEAP32[$pPage>>2]|0;
     $278 = (($277) + 2|0);
     $279 = HEAP8[$278>>0]|0;
     $280 = $279&255;
     $281 = ($280|0)!=(0);
     if ($281) {
      $282 = HEAP32[$3>>2]|0;
      $283 = ($282|0)!=(0|0);
      $284 = HEAP32[$4>>2]|0;
      $285 = ($284|0)!=(0|0);
      if (!($283)) {
       if (!($285)) {
        break;
       }
       $417 = $nMinKey;
       $418 = $417;
       $419 = HEAP32[$418>>2]|0;
       $420 = (($417) + 4)|0;
       $421 = $420;
       $422 = HEAP32[$421>>2]|0;
       $423 = HEAP32[$4>>2]|0;
       $424 = $423;
       $425 = $424;
       $426 = HEAP32[$425>>2]|0;
       $427 = (($424) + 4)|0;
       $428 = $427;
       $429 = HEAP32[$428>>2]|0;
       $430 = ($422|0)<($429|0);
       $431 = ($419>>>0)<=($426>>>0);
       $432 = ($422|0)==($429|0);
       $433 = $432 & $431;
       $434 = $430 | $433;
       if (!($434)) {
        break;
       }
       $435 = HEAP32[$1>>2]|0;
       $436 = $nMinKey;
       $437 = $436;
       $438 = HEAP32[$437>>2]|0;
       $439 = (($436) + 4)|0;
       $440 = $439;
       $441 = HEAP32[$440>>2]|0;
       $442 = HEAP32[$4>>2]|0;
       $443 = $442;
       $444 = $443;
       $445 = HEAP32[$444>>2]|0;
       $446 = (($443) + 4)|0;
       $447 = $446;
       $448 = HEAP32[$447>>2]|0;
       $449 = $vararg_buffer34;
       $450 = $449;
       HEAP32[$450>>2] = $438;
       $451 = (($449) + 4)|0;
       $452 = $451;
       HEAP32[$452>>2] = $441;
       $vararg_ptr25 = (($vararg_buffer34) + 8|0);
       $453 = $vararg_ptr25;
       $454 = $453;
       HEAP32[$454>>2] = $445;
       $455 = (($453) + 4)|0;
       $456 = $455;
       HEAP32[$456>>2] = $448;
       _checkAppendMsg($435,33928,$vararg_buffer34);
       break;
      }
      if (!($285)) {
       $286 = $nMaxKey;
       $287 = $286;
       $288 = HEAP32[$287>>2]|0;
       $289 = (($286) + 4)|0;
       $290 = $289;
       $291 = HEAP32[$290>>2]|0;
       $292 = HEAP32[$3>>2]|0;
       $293 = $292;
       $294 = $293;
       $295 = HEAP32[$294>>2]|0;
       $296 = (($293) + 4)|0;
       $297 = $296;
       $298 = HEAP32[$297>>2]|0;
       $299 = ($291|0)>($298|0);
       $300 = ($288>>>0)>($295>>>0);
       $301 = ($291|0)==($298|0);
       $302 = $301 & $300;
       $303 = $299 | $302;
       if (!($303)) {
        break;
       }
       $304 = HEAP32[$1>>2]|0;
       $305 = $nMaxKey;
       $306 = $305;
       $307 = HEAP32[$306>>2]|0;
       $308 = (($305) + 4)|0;
       $309 = $308;
       $310 = HEAP32[$309>>2]|0;
       $311 = HEAP32[$3>>2]|0;
       $312 = $311;
       $313 = $312;
       $314 = HEAP32[$313>>2]|0;
       $315 = (($312) + 4)|0;
       $316 = $315;
       $317 = HEAP32[$316>>2]|0;
       $318 = $vararg_buffer34;
       $319 = $318;
       HEAP32[$319>>2] = $307;
       $320 = (($318) + 4)|0;
       $321 = $320;
       HEAP32[$321>>2] = $310;
       $vararg_ptr13 = (($vararg_buffer34) + 8|0);
       $322 = $vararg_ptr13;
       $323 = $322;
       HEAP32[$323>>2] = $314;
       $324 = (($322) + 4)|0;
       $325 = $324;
       HEAP32[$325>>2] = $317;
       _checkAppendMsg($304,33736,$vararg_buffer34);
       break;
      }
      $326 = $nMinKey;
      $327 = $326;
      $328 = HEAP32[$327>>2]|0;
      $329 = (($326) + 4)|0;
      $330 = $329;
      $331 = HEAP32[$330>>2]|0;
      $332 = HEAP32[$3>>2]|0;
      $333 = $332;
      $334 = $333;
      $335 = HEAP32[$334>>2]|0;
      $336 = (($333) + 4)|0;
      $337 = $336;
      $338 = HEAP32[$337>>2]|0;
      $339 = ($331|0)<($338|0);
      $340 = ($328>>>0)<=($335>>>0);
      $341 = ($331|0)==($338|0);
      $342 = $341 & $340;
      $343 = $339 | $342;
      if ($343) {
       $344 = HEAP32[$1>>2]|0;
       $345 = $nMinKey;
       $346 = $345;
       $347 = HEAP32[$346>>2]|0;
       $348 = (($345) + 4)|0;
       $349 = $348;
       $350 = HEAP32[$349>>2]|0;
       $351 = HEAP32[$3>>2]|0;
       $352 = $351;
       $353 = $352;
       $354 = HEAP32[$353>>2]|0;
       $355 = (($352) + 4)|0;
       $356 = $355;
       $357 = HEAP32[$356>>2]|0;
       $358 = $vararg_buffer34;
       $359 = $358;
       HEAP32[$359>>2] = $347;
       $360 = (($358) + 4)|0;
       $361 = $360;
       HEAP32[$361>>2] = $350;
       $vararg_ptr17 = (($vararg_buffer34) + 8|0);
       $362 = $vararg_ptr17;
       $363 = $362;
       HEAP32[$363>>2] = $354;
       $364 = (($362) + 4)|0;
       $365 = $364;
       HEAP32[$365>>2] = $357;
       _checkAppendMsg($344,33800,$vararg_buffer34);
      }
      $366 = $nMaxKey;
      $367 = $366;
      $368 = HEAP32[$367>>2]|0;
      $369 = (($366) + 4)|0;
      $370 = $369;
      $371 = HEAP32[$370>>2]|0;
      $372 = HEAP32[$4>>2]|0;
      $373 = $372;
      $374 = $373;
      $375 = HEAP32[$374>>2]|0;
      $376 = (($373) + 4)|0;
      $377 = $376;
      $378 = HEAP32[$377>>2]|0;
      $379 = ($371|0)>($378|0);
      $380 = ($368>>>0)>($375>>>0);
      $381 = ($371|0)==($378|0);
      $382 = $381 & $380;
      $383 = $379 | $382;
      if ($383) {
       $384 = HEAP32[$1>>2]|0;
       $385 = $nMaxKey;
       $386 = $385;
       $387 = HEAP32[$386>>2]|0;
       $388 = (($385) + 4)|0;
       $389 = $388;
       $390 = HEAP32[$389>>2]|0;
       $391 = HEAP32[$4>>2]|0;
       $392 = $391;
       $393 = $392;
       $394 = HEAP32[$393>>2]|0;
       $395 = (($392) + 4)|0;
       $396 = $395;
       $397 = HEAP32[$396>>2]|0;
       $398 = $vararg_buffer34;
       $399 = $398;
       HEAP32[$399>>2] = $387;
       $400 = (($398) + 4)|0;
       $401 = $400;
       HEAP32[$401>>2] = $390;
       $vararg_ptr21 = (($vararg_buffer34) + 8|0);
       $402 = $vararg_ptr21;
       $403 = $402;
       HEAP32[$403>>2] = $394;
       $404 = (($402) + 4)|0;
       $405 = $404;
       HEAP32[$405>>2] = $397;
       _checkAppendMsg($384,33864,$vararg_buffer34);
      }
      $406 = $nMaxKey;
      $407 = $406;
      $408 = HEAP32[$407>>2]|0;
      $409 = (($406) + 4)|0;
      $410 = $409;
      $411 = HEAP32[$410>>2]|0;
      $412 = HEAP32[$3>>2]|0;
      $413 = $412;
      $414 = $413;
      HEAP32[$414>>2] = $408;
      $415 = (($413) + 4)|0;
      $416 = $415;
      HEAP32[$416>>2] = $411;
     }
    }
   } while(0);
   $457 = HEAP32[$pPage>>2]|0;
   $458 = (($457) + 56|0);
   $459 = HEAP32[$458>>2]|0;
   HEAP32[$data>>2] = $459;
   $460 = HEAP32[$pPage>>2]|0;
   $461 = (($460) + 6|0);
   $462 = HEAP8[$461>>0]|0;
   $463 = $462&255;
   HEAP32[$hdr>>2] = $463;
   $464 = HEAP32[$pBt>>2]|0;
   $465 = (($464) + 32|0);
   $466 = HEAP32[$465>>2]|0;
   $467 = (_sqlite3PageMalloc($466)|0);
   HEAP32[$hit>>2] = $467;
   $468 = HEAP32[$1>>2]|0;
   $469 = (($468) + 28|0);
   HEAP32[$469>>2] = 0;
   $470 = HEAP32[$hit>>2]|0;
   $471 = ($470|0)==(0|0);
   if ($471) {
    $472 = HEAP32[$1>>2]|0;
    $473 = (($472) + 24|0);
    HEAP32[$473>>2] = 1;
   } else {
    $474 = HEAP32[$hdr>>2]|0;
    $475 = (($474) + 5)|0;
    $476 = HEAP32[$data>>2]|0;
    $477 = (($476) + ($475)|0);
    $478 = HEAP8[$477>>0]|0;
    $479 = $478&255;
    $480 = $479 << 8;
    $481 = HEAP32[$hdr>>2]|0;
    $482 = (($481) + 5)|0;
    $483 = HEAP32[$data>>2]|0;
    $484 = (($483) + ($482)|0);
    $485 = (($484) + 1|0);
    $486 = HEAP8[$485>>0]|0;
    $487 = $486&255;
    $488 = $480 | $487;
    $489 = (($488) - 1)|0;
    $490 = $489 & 65535;
    $491 = (($490) + 1)|0;
    HEAP32[$contentOffset>>2] = $491;
    $492 = HEAP32[$hit>>2]|0;
    $493 = HEAP32[$contentOffset>>2]|0;
    $494 = (($492) + ($493)|0);
    $495 = HEAP32[$usableSize>>2]|0;
    $496 = HEAP32[$contentOffset>>2]|0;
    $497 = (($495) - ($496))|0;
    _memset(($494|0),0,($497|0))|0;
    $498 = HEAP32[$hit>>2]|0;
    $499 = HEAP32[$contentOffset>>2]|0;
    _memset(($498|0),1,($499|0))|0;
    $500 = HEAP32[$hdr>>2]|0;
    $501 = (($500) + 3)|0;
    $502 = HEAP32[$data>>2]|0;
    $503 = (($502) + ($501)|0);
    $504 = HEAP8[$503>>0]|0;
    $505 = $504&255;
    $506 = $505 << 8;
    $507 = HEAP32[$hdr>>2]|0;
    $508 = (($507) + 3)|0;
    $509 = HEAP32[$data>>2]|0;
    $510 = (($509) + ($508)|0);
    $511 = (($510) + 1|0);
    $512 = HEAP8[$511>>0]|0;
    $513 = $512&255;
    $514 = $506 | $513;
    HEAP32[$nCell>>2] = $514;
    $515 = HEAP32[$hdr>>2]|0;
    $516 = (($515) + 12)|0;
    $517 = HEAP32[$pPage>>2]|0;
    $518 = (($517) + 5|0);
    $519 = HEAP8[$518>>0]|0;
    $520 = $519&255;
    $521 = $520<<2;
    $522 = (($516) - ($521))|0;
    HEAP32[$cellStart>>2] = $522;
    HEAP32[$i>>2] = 0;
    while(1) {
     $523 = HEAP32[$i>>2]|0;
     $524 = HEAP32[$nCell>>2]|0;
     $525 = ($523|0)<($524|0);
     if (!($525)) {
      break;
     }
     $526 = HEAP32[$cellStart>>2]|0;
     $527 = HEAP32[$i>>2]|0;
     $528 = $527<<1;
     $529 = (($526) + ($528))|0;
     $530 = HEAP32[$data>>2]|0;
     $531 = (($530) + ($529)|0);
     $532 = HEAP8[$531>>0]|0;
     $533 = $532&255;
     $534 = $533 << 8;
     $535 = HEAP32[$cellStart>>2]|0;
     $536 = HEAP32[$i>>2]|0;
     $537 = $536<<1;
     $538 = (($535) + ($537))|0;
     $539 = HEAP32[$data>>2]|0;
     $540 = (($539) + ($538)|0);
     $541 = (($540) + 1|0);
     $542 = HEAP8[$541>>0]|0;
     $543 = $542&255;
     $544 = $534 | $543;
     HEAP32[$pc>>2] = $544;
     HEAP32[$size>>2] = 65536;
     $545 = HEAP32[$pc>>2]|0;
     $546 = HEAP32[$usableSize>>2]|0;
     $547 = (($546) - 4)|0;
     $548 = ($545|0)<=($547|0);
     if ($548) {
      $549 = HEAP32[$pPage>>2]|0;
      $550 = HEAP32[$pc>>2]|0;
      $551 = HEAP32[$data>>2]|0;
      $552 = (($551) + ($550)|0);
      $553 = (_cellSizePtr($549,$552)|0);
      $554 = $553&65535;
      HEAP32[$size>>2] = $554;
     }
     $555 = HEAP32[$pc>>2]|0;
     $556 = HEAP32[$size>>2]|0;
     $557 = (($555) + ($556))|0;
     $558 = (($557) - 1)|0;
     $559 = HEAP32[$usableSize>>2]|0;
     $560 = ($558|0)>=($559|0);
     L79: do {
      if ($560) {
       $561 = HEAP32[$1>>2]|0;
       $562 = (($561) + 28|0);
       HEAP32[$562>>2] = 0;
       $563 = HEAP32[$1>>2]|0;
       $564 = HEAP32[$i>>2]|0;
       $565 = HEAP32[$2>>2]|0;
       HEAP32[$vararg_buffer34>>2] = $564;
       $vararg_ptr29 = (($vararg_buffer34) + 4|0);
       HEAP32[$vararg_ptr29>>2] = $565;
       _checkAppendMsg($563,33992,$vararg_buffer34);
      } else {
       $566 = HEAP32[$pc>>2]|0;
       $567 = HEAP32[$size>>2]|0;
       $568 = (($566) + ($567))|0;
       $569 = (($568) - 1)|0;
       HEAP32[$j>>2] = $569;
       while(1) {
        $570 = HEAP32[$j>>2]|0;
        $571 = HEAP32[$pc>>2]|0;
        $572 = ($570|0)>=($571|0);
        if (!($572)) {
         break L79;
        }
        $573 = HEAP32[$j>>2]|0;
        $574 = HEAP32[$hit>>2]|0;
        $575 = (($574) + ($573)|0);
        $576 = HEAP8[$575>>0]|0;
        $577 = (($576) + 1)<<24>>24;
        HEAP8[$575>>0] = $577;
        $578 = HEAP32[$j>>2]|0;
        $579 = (($578) + -1)|0;
        HEAP32[$j>>2] = $579;
       }
      }
     } while(0);
     $580 = HEAP32[$i>>2]|0;
     $581 = (($580) + 1)|0;
     HEAP32[$i>>2] = $581;
    }
    $582 = HEAP32[$hdr>>2]|0;
    $583 = (($582) + 1)|0;
    $584 = HEAP32[$data>>2]|0;
    $585 = (($584) + ($583)|0);
    $586 = HEAP8[$585>>0]|0;
    $587 = $586&255;
    $588 = $587 << 8;
    $589 = HEAP32[$hdr>>2]|0;
    $590 = (($589) + 1)|0;
    $591 = HEAP32[$data>>2]|0;
    $592 = (($591) + ($590)|0);
    $593 = (($592) + 1|0);
    $594 = HEAP8[$593>>0]|0;
    $595 = $594&255;
    $596 = $588 | $595;
    HEAP32[$i>>2] = $596;
    while(1) {
     $597 = HEAP32[$i>>2]|0;
     $598 = ($597|0)>(0);
     if (!($598)) {
      break;
     }
     $599 = HEAP32[$i>>2]|0;
     $600 = (($599) + 2)|0;
     $601 = HEAP32[$data>>2]|0;
     $602 = (($601) + ($600)|0);
     $603 = HEAP8[$602>>0]|0;
     $604 = $603&255;
     $605 = $604 << 8;
     $606 = HEAP32[$i>>2]|0;
     $607 = (($606) + 2)|0;
     $608 = HEAP32[$data>>2]|0;
     $609 = (($608) + ($607)|0);
     $610 = (($609) + 1|0);
     $611 = HEAP8[$610>>0]|0;
     $612 = $611&255;
     $613 = $605 | $612;
     HEAP32[$size1>>2] = $613;
     $614 = HEAP32[$i>>2]|0;
     $615 = HEAP32[$size1>>2]|0;
     $616 = (($614) + ($615))|0;
     $617 = (($616) - 1)|0;
     HEAP32[$j2>>2] = $617;
     while(1) {
      $618 = HEAP32[$j2>>2]|0;
      $619 = HEAP32[$i>>2]|0;
      $620 = ($618|0)>=($619|0);
      if (!($620)) {
       break;
      }
      $621 = HEAP32[$j2>>2]|0;
      $622 = HEAP32[$hit>>2]|0;
      $623 = (($622) + ($621)|0);
      $624 = HEAP8[$623>>0]|0;
      $625 = (($624) + 1)<<24>>24;
      HEAP8[$623>>0] = $625;
      $626 = HEAP32[$j2>>2]|0;
      $627 = (($626) + -1)|0;
      HEAP32[$j2>>2] = $627;
     }
     $628 = HEAP32[$i>>2]|0;
     $629 = HEAP32[$data>>2]|0;
     $630 = (($629) + ($628)|0);
     $631 = HEAP8[$630>>0]|0;
     $632 = $631&255;
     $633 = $632 << 8;
     $634 = HEAP32[$i>>2]|0;
     $635 = HEAP32[$data>>2]|0;
     $636 = (($635) + ($634)|0);
     $637 = (($636) + 1|0);
     $638 = HEAP8[$637>>0]|0;
     $639 = $638&255;
     $640 = $633 | $639;
     HEAP32[$j2>>2] = $640;
     $641 = HEAP32[$j2>>2]|0;
     HEAP32[$i>>2] = $641;
    }
    HEAP32[$cnt>>2] = 0;
    HEAP32[$i>>2] = 0;
    while(1) {
     $642 = HEAP32[$i>>2]|0;
     $643 = HEAP32[$usableSize>>2]|0;
     $644 = ($642|0)<($643|0);
     if (!($644)) {
      break;
     }
     $645 = HEAP32[$i>>2]|0;
     $646 = HEAP32[$hit>>2]|0;
     $647 = (($646) + ($645)|0);
     $648 = HEAP8[$647>>0]|0;
     $649 = $648 << 24 >> 24;
     $650 = ($649|0)==(0);
     if ($650) {
      $651 = HEAP32[$cnt>>2]|0;
      $652 = (($651) + 1)|0;
      HEAP32[$cnt>>2] = $652;
     } else {
      $653 = HEAP32[$i>>2]|0;
      $654 = HEAP32[$hit>>2]|0;
      $655 = (($654) + ($653)|0);
      $656 = HEAP8[$655>>0]|0;
      $657 = $656 << 24 >> 24;
      $658 = ($657|0)>(1);
      if ($658) {
       label = 72;
       break;
      }
     }
     $662 = HEAP32[$i>>2]|0;
     $663 = (($662) + 1)|0;
     HEAP32[$i>>2] = $663;
    }
    if ((label|0) == 72) {
     $659 = HEAP32[$1>>2]|0;
     $660 = HEAP32[$i>>2]|0;
     $661 = HEAP32[$2>>2]|0;
     HEAP32[$vararg_buffer34>>2] = $660;
     $vararg_ptr33 = (($vararg_buffer34) + 4|0);
     HEAP32[$vararg_ptr33>>2] = $661;
     _checkAppendMsg($659,34040,$vararg_buffer34);
    }
    $664 = HEAP32[$cnt>>2]|0;
    $665 = HEAP32[$hdr>>2]|0;
    $666 = (($665) + 7)|0;
    $667 = HEAP32[$data>>2]|0;
    $668 = (($667) + ($666)|0);
    $669 = HEAP8[$668>>0]|0;
    $670 = $669&255;
    $671 = ($664|0)!=($670|0);
    if ($671) {
     $672 = HEAP32[$1>>2]|0;
     $673 = HEAP32[$cnt>>2]|0;
     $674 = HEAP32[$hdr>>2]|0;
     $675 = (($674) + 7)|0;
     $676 = HEAP32[$data>>2]|0;
     $677 = (($676) + ($675)|0);
     $678 = HEAP8[$677>>0]|0;
     $679 = $678&255;
     $680 = HEAP32[$2>>2]|0;
     HEAP32[$vararg_buffer34>>2] = $673;
     $vararg_ptr37 = (($vararg_buffer34) + 4|0);
     HEAP32[$vararg_ptr37>>2] = $679;
     $vararg_ptr38 = (($vararg_buffer34) + 8|0);
     HEAP32[$vararg_ptr38>>2] = $680;
     _checkAppendMsg($672,34080,$vararg_buffer34);
    }
   }
   $681 = HEAP32[$hit>>2]|0;
   _sqlite3PageFree($681);
   $682 = HEAP32[$pPage>>2]|0;
   _releasePage($682);
  }
 } while(0);
 $683 = HEAP32[$saved_zPfx>>2]|0;
 $684 = HEAP32[$1>>2]|0;
 $685 = (($684) + 28|0);
 HEAP32[$685>>2] = $683;
 $686 = HEAP32[$saved_v1>>2]|0;
 $687 = HEAP32[$1>>2]|0;
 $688 = (($687) + 32|0);
 HEAP32[$688>>2] = $686;
 $689 = HEAP32[$saved_v2>>2]|0;
 $690 = HEAP32[$1>>2]|0;
 $691 = (($690) + 36|0);
 HEAP32[$691>>2] = $689;
 $692 = HEAP32[$depth>>2]|0;
 $693 = (($692) + 1)|0;
 HEAP32[$0>>2] = $693;
 $694 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($694|0);
}
function _getPageReferenced($pCheck,$iPg) {
 $pCheck = $pCheck|0;
 $iPg = $iPg|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $1 = sp;
 HEAP32[$0>>2] = $pCheck;
 HEAP32[$1>>2] = $iPg;
 $2 = HEAP32[$1>>2]|0;
 $3 = (($2>>>0) / 8)&-1;
 $4 = HEAP32[$0>>2]|0;
 $5 = (($4) + 8|0);
 $6 = HEAP32[$5>>2]|0;
 $7 = (($6) + ($3)|0);
 $8 = HEAP8[$7>>0]|0;
 $9 = $8&255;
 $10 = HEAP32[$1>>2]|0;
 $11 = $10 & 7;
 $12 = 1 << $11;
 $13 = $9 & $12;
 STACKTOP = sp;return ($13|0);
}
function _checkAppendMsg($pCheck,$zFormat,$varargs) {
 $pCheck = $pCheck|0;
 $zFormat = $zFormat|0;
 $varargs = $varargs|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $ap = 0, $vararg_buffer = 0, $vararg_ptr1 = 0, $zBuf = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 240|0;
 $vararg_buffer = sp;
 $0 = sp + 28|0;
 $1 = sp + 24|0;
 $ap = sp + 8|0;
 $zBuf = sp + 32|0;
 HEAP32[$0>>2] = $pCheck;
 HEAP32[$1>>2] = $zFormat;
 $2 = HEAP32[$0>>2]|0;
 $3 = (($2) + 16|0);
 $4 = HEAP32[$3>>2]|0;
 $5 = ($4|0)!=(0);
 if (!($5)) {
  STACKTOP = sp;return;
 }
 $6 = HEAP32[$0>>2]|0;
 $7 = (($6) + 16|0);
 $8 = HEAP32[$7>>2]|0;
 $9 = (($8) + -1)|0;
 HEAP32[$7>>2] = $9;
 $10 = HEAP32[$0>>2]|0;
 $11 = (($10) + 20|0);
 $12 = HEAP32[$11>>2]|0;
 $13 = (($12) + 1)|0;
 HEAP32[$11>>2] = $13;
 HEAP32[$ap>>2] = $varargs;
 $14 = HEAP32[$0>>2]|0;
 $15 = (($14) + 40|0);
 $16 = (($15) + 12|0);
 $17 = HEAP32[$16>>2]|0;
 $18 = ($17|0)!=(0);
 if ($18) {
  $19 = HEAP32[$0>>2]|0;
  $20 = (($19) + 40|0);
  _sqlite3StrAccumAppend($20,33488,1);
 }
 $21 = HEAP32[$0>>2]|0;
 $22 = (($21) + 28|0);
 $23 = HEAP32[$22>>2]|0;
 $24 = ($23|0)!=(0|0);
 if ($24) {
  $25 = HEAP32[$0>>2]|0;
  $26 = (($25) + 28|0);
  $27 = HEAP32[$26>>2]|0;
  $28 = HEAP32[$0>>2]|0;
  $29 = (($28) + 32|0);
  $30 = HEAP32[$29>>2]|0;
  $31 = HEAP32[$0>>2]|0;
  $32 = (($31) + 36|0);
  $33 = HEAP32[$32>>2]|0;
  HEAP32[$vararg_buffer>>2] = $30;
  $vararg_ptr1 = (($vararg_buffer) + 4|0);
  HEAP32[$vararg_ptr1>>2] = $33;
  (_sqlite3_snprintf(200,$zBuf,$27,$vararg_buffer)|0);
  $34 = HEAP32[$0>>2]|0;
  $35 = (($34) + 40|0);
  _sqlite3StrAccumAppendAll($35,$zBuf);
 }
 $36 = HEAP32[$0>>2]|0;
 $37 = (($36) + 40|0);
 $38 = HEAP32[$1>>2]|0;
 _sqlite3VXPrintf($37,1,$38,$ap);
 $39 = HEAP32[$0>>2]|0;
 $40 = (($39) + 40|0);
 $41 = (($40) + 25|0);
 $42 = HEAP8[$41>>0]|0;
 $43 = $42&255;
 $44 = ($43|0)==(1);
 if (!($44)) {
  STACKTOP = sp;return;
 }
 $45 = HEAP32[$0>>2]|0;
 $46 = (($45) + 24|0);
 HEAP32[$46>>2] = 1;
 STACKTOP = sp;return;
}
function _checkRef($pCheck,$iPage) {
 $pCheck = $pCheck|0;
 $iPage = $iPage|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $vararg_buffer1 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $vararg_buffer1 = sp;
 $0 = sp + 12|0;
 $1 = sp + 8|0;
 $2 = sp + 4|0;
 HEAP32[$1>>2] = $pCheck;
 HEAP32[$2>>2] = $iPage;
 $3 = HEAP32[$2>>2]|0;
 $4 = ($3|0)==(0);
 do {
  if ($4) {
   HEAP32[$0>>2] = 1;
  } else {
   $5 = HEAP32[$2>>2]|0;
   $6 = HEAP32[$1>>2]|0;
   $7 = (($6) + 12|0);
   $8 = HEAP32[$7>>2]|0;
   $9 = ($5>>>0)>($8>>>0);
   $10 = HEAP32[$1>>2]|0;
   $11 = HEAP32[$2>>2]|0;
   if ($9) {
    HEAP32[$vararg_buffer1>>2] = $11;
    _checkAppendMsg($10,34136,$vararg_buffer1);
    HEAP32[$0>>2] = 1;
    break;
   }
   $12 = (_getPageReferenced($10,$11)|0);
   $13 = ($12|0)!=(0);
   $14 = HEAP32[$1>>2]|0;
   $15 = HEAP32[$2>>2]|0;
   if ($13) {
    HEAP32[$vararg_buffer1>>2] = $15;
    _checkAppendMsg($14,34160,$vararg_buffer1);
    HEAP32[$0>>2] = 1;
    break;
   } else {
    _setPageReferenced($14,$15);
    HEAP32[$0>>2] = 0;
    break;
   }
  }
 } while(0);
 $16 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($16|0);
}
function _cellSizePtr($pPage,$pCell) {
 $pPage = $pPage|0;
 $pCell = $pCell|0;
 var $$ = 0, $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0;
 var $115 = 0, $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0;
 var $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0;
 var $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0;
 var $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0;
 var $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0;
 var $98 = 0, $99 = 0, $minLocal = 0, $nSize = 0, $pEnd = 0, $pIter = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 24|0;
 $1 = sp + 20|0;
 $2 = sp + 16|0;
 $pIter = sp + 12|0;
 $pEnd = sp + 8|0;
 $nSize = sp + 4|0;
 $minLocal = sp;
 HEAP32[$1>>2] = $pPage;
 HEAP32[$2>>2] = $pCell;
 $3 = HEAP32[$2>>2]|0;
 $4 = HEAP32[$1>>2]|0;
 $5 = (($4) + 7|0);
 $6 = HEAP8[$5>>0]|0;
 $7 = $6&255;
 $8 = (($3) + ($7)|0);
 HEAP32[$pIter>>2] = $8;
 $9 = HEAP32[$1>>2]|0;
 $10 = (($9) + 4|0);
 $11 = HEAP8[$10>>0]|0;
 $12 = ($11<<24>>24)!=(0);
 $13 = HEAP32[$pIter>>2]|0;
 if ($12) {
  $14 = (($13) + 9|0);
  HEAP32[$pEnd>>2] = $14;
  while(1) {
   $15 = HEAP32[$pIter>>2]|0;
   $16 = (($15) + 1|0);
   HEAP32[$pIter>>2] = $16;
   $17 = HEAP8[$15>>0]|0;
   $18 = $17&255;
   $19 = $18 & 128;
   $20 = ($19|0)!=(0);
   if (!($20)) {
    break;
   }
   $21 = HEAP32[$pIter>>2]|0;
   $22 = HEAP32[$pEnd>>2]|0;
   $23 = ($21>>>0)<($22>>>0);
   if (!($23)) {
    break;
   }
  }
  $24 = HEAP32[$pIter>>2]|0;
  $25 = HEAP32[$2>>2]|0;
  $26 = $24;
  $27 = $25;
  $28 = (($26) - ($27))|0;
  $29 = $28&65535;
  HEAP16[$0>>1] = $29;
  $120 = HEAP16[$0>>1]|0;
  STACKTOP = sp;return ($120|0);
 }
 $30 = HEAP8[$13>>0]|0;
 $31 = $30&255;
 HEAP32[$nSize>>2] = $31;
 $32 = HEAP32[$nSize>>2]|0;
 $33 = ($32>>>0)>=(128);
 L9: do {
  if ($33) {
   $34 = HEAP32[$pIter>>2]|0;
   $35 = (($34) + 9|0);
   HEAP32[$pEnd>>2] = $35;
   $36 = HEAP32[$nSize>>2]|0;
   $37 = $36 & 127;
   HEAP32[$nSize>>2] = $37;
   while(1) {
    $38 = HEAP32[$nSize>>2]|0;
    $39 = $38 << 7;
    $40 = HEAP32[$pIter>>2]|0;
    $41 = (($40) + 1|0);
    HEAP32[$pIter>>2] = $41;
    $42 = HEAP8[$41>>0]|0;
    $43 = $42&255;
    $44 = $43 & 127;
    $45 = $39 | $44;
    HEAP32[$nSize>>2] = $45;
    $46 = HEAP32[$pIter>>2]|0;
    $47 = HEAP8[$46>>0]|0;
    $48 = $47&255;
    $49 = ($48|0)>=(128);
    if (!($49)) {
     break L9;
    }
    $50 = HEAP32[$pIter>>2]|0;
    $51 = HEAP32[$pEnd>>2]|0;
    $52 = ($50>>>0)<($51>>>0);
    if (!($52)) {
     break;
    }
   }
  }
 } while(0);
 $53 = HEAP32[$pIter>>2]|0;
 $54 = (($53) + 1|0);
 HEAP32[$pIter>>2] = $54;
 $55 = HEAP32[$1>>2]|0;
 $56 = (($55) + 2|0);
 $57 = HEAP8[$56>>0]|0;
 $58 = ($57<<24>>24)!=(0);
 L15: do {
  if ($58) {
   $59 = HEAP32[$pIter>>2]|0;
   $60 = (($59) + 9|0);
   HEAP32[$pEnd>>2] = $60;
   while(1) {
    $61 = HEAP32[$pIter>>2]|0;
    $62 = (($61) + 1|0);
    HEAP32[$pIter>>2] = $62;
    $63 = HEAP8[$61>>0]|0;
    $64 = $63&255;
    $65 = $64 & 128;
    $66 = ($65|0)!=(0);
    if (!($66)) {
     break L15;
    }
    $67 = HEAP32[$pIter>>2]|0;
    $68 = HEAP32[$pEnd>>2]|0;
    $69 = ($67>>>0)<($68>>>0);
    if (!($69)) {
     break;
    }
   }
  }
 } while(0);
 $70 = HEAP32[$nSize>>2]|0;
 $71 = HEAP32[$1>>2]|0;
 $72 = (($71) + 10|0);
 $73 = HEAP16[$72>>1]|0;
 $74 = $73&65535;
 $75 = ($70>>>0)<=($74>>>0);
 if ($75) {
  $76 = HEAP32[$pIter>>2]|0;
  $77 = HEAP32[$2>>2]|0;
  $78 = $76;
  $79 = $77;
  $80 = (($78) - ($79))|0;
  $81 = HEAP32[$nSize>>2]|0;
  $82 = (($81) + ($80))|0;
  HEAP32[$nSize>>2] = $82;
  $83 = HEAP32[$nSize>>2]|0;
  $84 = ($83>>>0)<(4);
  $$ = $84 ? 4 : $82;
  HEAP32[$nSize>>2] = $$;
 } else {
  $85 = HEAP32[$1>>2]|0;
  $86 = (($85) + 12|0);
  $87 = HEAP16[$86>>1]|0;
  $88 = $87&65535;
  HEAP32[$minLocal>>2] = $88;
  $89 = HEAP32[$minLocal>>2]|0;
  $90 = HEAP32[$nSize>>2]|0;
  $91 = HEAP32[$minLocal>>2]|0;
  $92 = (($90) - ($91))|0;
  $93 = HEAP32[$1>>2]|0;
  $94 = (($93) + 52|0);
  $95 = HEAP32[$94>>2]|0;
  $96 = (($95) + 36|0);
  $97 = HEAP32[$96>>2]|0;
  $98 = (($97) - 4)|0;
  $99 = (($92>>>0) % ($98>>>0))&-1;
  $100 = (($89) + ($99))|0;
  HEAP32[$nSize>>2] = $100;
  $101 = HEAP32[$nSize>>2]|0;
  $102 = HEAP32[$1>>2]|0;
  $103 = (($102) + 10|0);
  $104 = HEAP16[$103>>1]|0;
  $105 = $104&65535;
  $106 = ($101>>>0)>($105>>>0);
  if ($106) {
   $107 = HEAP32[$minLocal>>2]|0;
   HEAP32[$nSize>>2] = $107;
  }
  $108 = HEAP32[$pIter>>2]|0;
  $109 = HEAP32[$2>>2]|0;
  $110 = $108;
  $111 = $109;
  $112 = (($110) - ($111))|0;
  $113 = $112&65535;
  $114 = $113&65535;
  $115 = (4 + ($114))|0;
  $116 = HEAP32[$nSize>>2]|0;
  $117 = (($116) + ($115))|0;
  HEAP32[$nSize>>2] = $117;
 }
 $118 = HEAP32[$nSize>>2]|0;
 $119 = $118&65535;
 HEAP16[$0>>1] = $119;
 $120 = HEAP16[$0>>1]|0;
 STACKTOP = sp;return ($120|0);
}
function _vdbeMergeEngineFree($pMerger) {
 $pMerger = $pMerger|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $i = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $i = sp;
 HEAP32[$0>>2] = $pMerger;
 $1 = HEAP32[$0>>2]|0;
 $2 = ($1|0)!=(0|0);
 if (!($2)) {
  $14 = HEAP32[$0>>2]|0;
  _sqlite3_free($14);
  STACKTOP = sp;return;
 }
 HEAP32[$i>>2] = 0;
 while(1) {
  $3 = HEAP32[$i>>2]|0;
  $4 = HEAP32[$0>>2]|0;
  $5 = HEAP32[$4>>2]|0;
  $6 = ($3|0)<($5|0);
  if (!($6)) {
   break;
  }
  $7 = HEAP32[$i>>2]|0;
  $8 = HEAP32[$0>>2]|0;
  $9 = (($8) + 12|0);
  $10 = HEAP32[$9>>2]|0;
  $11 = (($10) + (($7*56)|0)|0);
  _vdbePmaReaderClear($11);
  $12 = HEAP32[$i>>2]|0;
  $13 = (($12) + 1)|0;
  HEAP32[$i>>2] = $13;
 }
 $14 = HEAP32[$0>>2]|0;
 _sqlite3_free($14);
 STACKTOP = sp;return;
}
function _vdbeSortSubtaskCleanup($db,$pTask) {
 $db = $db|0;
 $pTask = $pTask|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $1 = sp;
 HEAP32[$0>>2] = $db;
 HEAP32[$1>>2] = $pTask;
 $2 = HEAP32[$0>>2]|0;
 $3 = HEAP32[$1>>2]|0;
 $4 = (($3) + 12|0);
 $5 = HEAP32[$4>>2]|0;
 _sqlite3DbFree($2,$5);
 $6 = HEAP32[$1>>2]|0;
 $7 = (($6) + 12|0);
 HEAP32[$7>>2] = 0;
 $8 = HEAP32[$1>>2]|0;
 $9 = (($8) + 16|0);
 $10 = HEAP32[$9>>2]|0;
 _vdbeSorterRecordFree(0,$10);
 $11 = HEAP32[$1>>2]|0;
 $12 = (($11) + 16|0);
 HEAP32[$12>>2] = 0;
 $13 = HEAP32[$1>>2]|0;
 $14 = (($13) + 32|0);
 $15 = HEAP32[$14>>2]|0;
 $16 = ($15|0)!=(0|0);
 if ($16) {
  $17 = HEAP32[$1>>2]|0;
  $18 = (($17) + 32|0);
  $19 = HEAP32[$18>>2]|0;
  (_sqlite3OsCloseFree($19)|0);
  $20 = HEAP32[$1>>2]|0;
  $21 = (($20) + 32|0);
  HEAP32[$21>>2] = 0;
  $22 = HEAP32[$1>>2]|0;
  $23 = (($22) + 32|0);
  $24 = (($23) + 8|0);
  $25 = $24;
  $26 = $25;
  HEAP32[$26>>2] = 0;
  $27 = (($25) + 4)|0;
  $28 = $27;
  HEAP32[$28>>2] = 0;
 }
 $29 = HEAP32[$1>>2]|0;
 $30 = (($29) + 48|0);
 $31 = HEAP32[$30>>2]|0;
 $32 = ($31|0)!=(0|0);
 if (!($32)) {
  STACKTOP = sp;return;
 }
 $33 = HEAP32[$1>>2]|0;
 $34 = (($33) + 48|0);
 $35 = HEAP32[$34>>2]|0;
 (_sqlite3OsCloseFree($35)|0);
 $36 = HEAP32[$1>>2]|0;
 $37 = (($36) + 48|0);
 HEAP32[$37>>2] = 0;
 $38 = HEAP32[$1>>2]|0;
 $39 = (($38) + 48|0);
 $40 = (($39) + 8|0);
 $41 = $40;
 $42 = $41;
 HEAP32[$42>>2] = 0;
 $43 = (($41) + 4)|0;
 $44 = $43;
 HEAP32[$44>>2] = 0;
 STACKTOP = sp;return;
}
function _vdbeSorterRecordFree($db,$pRecord) {
 $db = $db|0;
 $pRecord = $pRecord|0;
 var $0 = 0, $1 = 0, $10 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $p = 0, $pNext = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $1 = sp + 8|0;
 $p = sp + 4|0;
 $pNext = sp;
 HEAP32[$0>>2] = $db;
 HEAP32[$1>>2] = $pRecord;
 $2 = HEAP32[$1>>2]|0;
 HEAP32[$p>>2] = $2;
 while(1) {
  $3 = HEAP32[$p>>2]|0;
  $4 = ($3|0)!=(0|0);
  if (!($4)) {
   break;
  }
  $5 = HEAP32[$p>>2]|0;
  $6 = (($5) + 4|0);
  $7 = HEAP32[$6>>2]|0;
  HEAP32[$pNext>>2] = $7;
  $8 = HEAP32[$0>>2]|0;
  $9 = HEAP32[$p>>2]|0;
  _sqlite3DbFree($8,$9);
  $10 = HEAP32[$pNext>>2]|0;
  HEAP32[$p>>2] = $10;
 }
 STACKTOP = sp;return;
}
function _sqlite3OsCloseFree($pFile) {
 $pFile = $pFile|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $rc = sp;
 HEAP32[$0>>2] = $pFile;
 HEAP32[$rc>>2] = 0;
 $1 = HEAP32[$0>>2]|0;
 $2 = (_sqlite3OsClose($1)|0);
 HEAP32[$rc>>2] = $2;
 $3 = HEAP32[$0>>2]|0;
 _sqlite3_free($3);
 $4 = HEAP32[$rc>>2]|0;
 STACKTOP = sp;return ($4|0);
}
function _vdbePmaReaderClear($pReadr) {
 $pReadr = $pReadr|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0;
 var $9 = 0, dest = 0, label = 0, sp = 0, stop = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 HEAP32[$0>>2] = $pReadr;
 $1 = HEAP32[$0>>2]|0;
 $2 = (($1) + 28|0);
 $3 = HEAP32[$2>>2]|0;
 _sqlite3_free($3);
 $4 = HEAP32[$0>>2]|0;
 $5 = (($4) + 36|0);
 $6 = HEAP32[$5>>2]|0;
 _sqlite3_free($6);
 $7 = HEAP32[$0>>2]|0;
 $8 = (($7) + 44|0);
 $9 = HEAP32[$8>>2]|0;
 $10 = ($9|0)!=(0|0);
 if ($10) {
  $11 = HEAP32[$0>>2]|0;
  $12 = (($11) + 24|0);
  $13 = HEAP32[$12>>2]|0;
  $14 = HEAP32[$0>>2]|0;
  $15 = (($14) + 44|0);
  $16 = HEAP32[$15>>2]|0;
  (_sqlite3OsUnfetch($13,0,0,$16)|0);
 }
 $17 = HEAP32[$0>>2]|0;
 $18 = (($17) + 48|0);
 $19 = HEAP32[$18>>2]|0;
 _vdbeIncrFree($19);
 $20 = HEAP32[$0>>2]|0;
 dest=$20+0|0; stop=dest+56|0; do { HEAP32[dest>>2]=0|0; dest=dest+4|0; } while ((dest|0) < (stop|0));
 STACKTOP = sp;return;
}
function _vdbeIncrFree($pIncr) {
 $pIncr = $pIncr|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 HEAP32[$0>>2] = $pIncr;
 $1 = HEAP32[$0>>2]|0;
 $2 = ($1|0)!=(0|0);
 if (!($2)) {
  STACKTOP = sp;return;
 }
 $3 = HEAP32[$0>>2]|0;
 $4 = (($3) + 4|0);
 $5 = HEAP32[$4>>2]|0;
 _vdbeMergeEngineFree($5);
 $6 = HEAP32[$0>>2]|0;
 _sqlite3_free($6);
 STACKTOP = sp;return;
}
function _invalidateIncrblobCursors($pBtree,$0,$1,$isClearTable) {
 $pBtree = $pBtree|0;
 $0 = $0|0;
 $1 = $1|0;
 $isClearTable = $isClearTable|0;
 var $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0;
 var $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0;
 var $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $p = 0, $pBt = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $2 = sp + 20|0;
 $3 = sp;
 $4 = sp + 16|0;
 $p = sp + 12|0;
 $pBt = sp + 8|0;
 HEAP32[$2>>2] = $pBtree;
 $5 = $3;
 $6 = $5;
 HEAP32[$6>>2] = $0;
 $7 = (($5) + 4)|0;
 $8 = $7;
 HEAP32[$8>>2] = $1;
 HEAP32[$4>>2] = $isClearTable;
 $9 = HEAP32[$2>>2]|0;
 $10 = (($9) + 4|0);
 $11 = HEAP32[$10>>2]|0;
 HEAP32[$pBt>>2] = $11;
 $12 = HEAP32[$pBt>>2]|0;
 $13 = (($12) + 8|0);
 $14 = HEAP32[$13>>2]|0;
 HEAP32[$p>>2] = $14;
 while(1) {
  $15 = HEAP32[$p>>2]|0;
  $16 = ($15|0)!=(0|0);
  if (!($16)) {
   break;
  }
  $17 = HEAP32[$p>>2]|0;
  $18 = (($17) + 72|0);
  $19 = HEAP8[$18>>0]|0;
  $20 = $19&255;
  $21 = $20 & 16;
  $22 = ($21|0)!=(0);
  do {
   if ($22) {
    $23 = HEAP32[$4>>2]|0;
    $24 = ($23|0)!=(0);
    if (!($24)) {
     $25 = HEAP32[$p>>2]|0;
     $26 = (($25) + 24|0);
     $27 = $26;
     $28 = $27;
     $29 = HEAP32[$28>>2]|0;
     $30 = (($27) + 4)|0;
     $31 = $30;
     $32 = HEAP32[$31>>2]|0;
     $33 = $3;
     $34 = $33;
     $35 = HEAP32[$34>>2]|0;
     $36 = (($33) + 4)|0;
     $37 = $36;
     $38 = HEAP32[$37>>2]|0;
     $39 = ($29|0)==($35|0);
     $40 = ($32|0)==($38|0);
     $41 = $39 & $40;
     if (!($41)) {
      break;
     }
    }
    $42 = HEAP32[$p>>2]|0;
    $43 = (($42) + 73|0);
    HEAP8[$43>>0] = 0;
   }
  } while(0);
  $44 = HEAP32[$p>>2]|0;
  $45 = (($44) + 8|0);
  $46 = HEAP32[$45>>2]|0;
  HEAP32[$p>>2] = $46;
 }
 STACKTOP = sp;return;
}
function _clearDatabasePage($pBt,$pgno,$freePageFlag,$pnChange) {
 $pBt = $pBt|0;
 $pgno = $pgno|0;
 $freePageFlag = $freePageFlag|0;
 $pnChange = $pnChange|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $12 = 0;
 var $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0;
 var $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0;
 var $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0;
 var $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0;
 var $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $hdr = 0, $i = 0, $pCell = 0, $pPage = 0, $rc = 0;
 var $szCell = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $0 = sp + 36|0;
 $1 = sp + 32|0;
 $2 = sp + 28|0;
 $3 = sp + 24|0;
 $4 = sp + 20|0;
 $pPage = sp + 16|0;
 $rc = sp + 12|0;
 $pCell = sp + 8|0;
 $i = sp + 4|0;
 $hdr = sp;
 $szCell = sp + 40|0;
 HEAP32[$1>>2] = $pBt;
 HEAP32[$2>>2] = $pgno;
 HEAP32[$3>>2] = $freePageFlag;
 HEAP32[$4>>2] = $pnChange;
 $5 = HEAP32[$2>>2]|0;
 $6 = HEAP32[$1>>2]|0;
 $7 = (_btreePagecount($6)|0);
 $8 = ($5>>>0)>($7>>>0);
 if ($8) {
  $9 = (_sqlite3CorruptError(59756)|0);
  HEAP32[$0>>2] = $9;
  $114 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($114|0);
 }
 $10 = HEAP32[$1>>2]|0;
 $11 = HEAP32[$2>>2]|0;
 $12 = (_getAndInitPage($10,$11,$pPage,0)|0);
 HEAP32[$rc>>2] = $12;
 $13 = HEAP32[$rc>>2]|0;
 $14 = ($13|0)!=(0);
 if ($14) {
  $15 = HEAP32[$rc>>2]|0;
  HEAP32[$0>>2] = $15;
  $114 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($114|0);
 }
 $16 = HEAP32[$pPage>>2]|0;
 $17 = (($16) + 6|0);
 $18 = HEAP8[$17>>0]|0;
 $19 = $18&255;
 HEAP32[$hdr>>2] = $19;
 HEAP32[$i>>2] = 0;
 while(1) {
  $20 = HEAP32[$i>>2]|0;
  $21 = HEAP32[$pPage>>2]|0;
  $22 = (($21) + 18|0);
  $23 = HEAP16[$22>>1]|0;
  $24 = $23&65535;
  $25 = ($20|0)<($24|0);
  $26 = HEAP32[$pPage>>2]|0;
  if (!($25)) {
   label = 11;
   break;
  }
  $27 = (($26) + 56|0);
  $28 = HEAP32[$27>>2]|0;
  $29 = HEAP32[$pPage>>2]|0;
  $30 = (($29) + 20|0);
  $31 = HEAP16[$30>>1]|0;
  $32 = $31&65535;
  $33 = HEAP32[$i>>2]|0;
  $34 = $33<<1;
  $35 = HEAP32[$pPage>>2]|0;
  $36 = (($35) + 64|0);
  $37 = HEAP32[$36>>2]|0;
  $38 = (($37) + ($34)|0);
  $39 = HEAP8[$38>>0]|0;
  $40 = $39&255;
  $41 = $40 << 8;
  $42 = HEAP32[$i>>2]|0;
  $43 = $42<<1;
  $44 = HEAP32[$pPage>>2]|0;
  $45 = (($44) + 64|0);
  $46 = HEAP32[$45>>2]|0;
  $47 = (($46) + ($43)|0);
  $48 = (($47) + 1|0);
  $49 = HEAP8[$48>>0]|0;
  $50 = $49&255;
  $51 = $41 | $50;
  $52 = $32 & $51;
  $53 = (($28) + ($52)|0);
  HEAP32[$pCell>>2] = $53;
  $54 = HEAP32[$pPage>>2]|0;
  $55 = (($54) + 5|0);
  $56 = HEAP8[$55>>0]|0;
  $57 = ($56<<24>>24)!=(0);
  if (!($57)) {
   $58 = HEAP32[$1>>2]|0;
   $59 = HEAP32[$pCell>>2]|0;
   $60 = (_sqlite3Get4byte($59)|0);
   $61 = HEAP32[$4>>2]|0;
   $62 = (_clearDatabasePage($58,$60,1,$61)|0);
   HEAP32[$rc>>2] = $62;
   $63 = HEAP32[$rc>>2]|0;
   $64 = ($63|0)!=(0);
   if ($64) {
    break;
   }
  }
  $65 = HEAP32[$pPage>>2]|0;
  $66 = HEAP32[$pCell>>2]|0;
  $67 = (_clearCell($65,$66,$szCell)|0);
  HEAP32[$rc>>2] = $67;
  $68 = HEAP32[$rc>>2]|0;
  $69 = ($68|0)!=(0);
  if ($69) {
   break;
  }
  $70 = HEAP32[$i>>2]|0;
  $71 = (($70) + 1)|0;
  HEAP32[$i>>2] = $71;
 }
 do {
  if ((label|0) == 11) {
   $72 = (($26) + 5|0);
   $73 = HEAP8[$72>>0]|0;
   $74 = ($73<<24>>24)!=(0);
   if ($74) {
    $87 = HEAP32[$4>>2]|0;
    $88 = ($87|0)!=(0|0);
    if ($88) {
     $89 = HEAP32[$pPage>>2]|0;
     $90 = (($89) + 18|0);
     $91 = HEAP16[$90>>1]|0;
     $92 = $91&65535;
     $93 = HEAP32[$4>>2]|0;
     $94 = HEAP32[$93>>2]|0;
     $95 = (($94) + ($92))|0;
     HEAP32[$93>>2] = $95;
    }
   } else {
    $75 = HEAP32[$1>>2]|0;
    $76 = HEAP32[$hdr>>2]|0;
    $77 = (($76) + 8)|0;
    $78 = HEAP32[$pPage>>2]|0;
    $79 = (($78) + 56|0);
    $80 = HEAP32[$79>>2]|0;
    $81 = (($80) + ($77)|0);
    $82 = (_sqlite3Get4byte($81)|0);
    $83 = HEAP32[$4>>2]|0;
    $84 = (_clearDatabasePage($75,$82,1,$83)|0);
    HEAP32[$rc>>2] = $84;
    $85 = HEAP32[$rc>>2]|0;
    $86 = ($85|0)!=(0);
    if ($86) {
     break;
    }
   }
   $96 = HEAP32[$3>>2]|0;
   $97 = ($96|0)!=(0);
   $98 = HEAP32[$pPage>>2]|0;
   if ($97) {
    _freePage($98,$rc);
    break;
   }
   $99 = (($98) + 68|0);
   $100 = HEAP32[$99>>2]|0;
   $101 = (_sqlite3PagerWrite($100)|0);
   HEAP32[$rc>>2] = $101;
   $102 = ($101|0)==(0);
   if ($102) {
    $103 = HEAP32[$pPage>>2]|0;
    $104 = HEAP32[$hdr>>2]|0;
    $105 = HEAP32[$pPage>>2]|0;
    $106 = (($105) + 56|0);
    $107 = HEAP32[$106>>2]|0;
    $108 = (($107) + ($104)|0);
    $109 = HEAP8[$108>>0]|0;
    $110 = $109&255;
    $111 = $110 | 8;
    _zeroPage($103,$111);
   }
  }
 } while(0);
 $112 = HEAP32[$pPage>>2]|0;
 _releasePage($112);
 $113 = HEAP32[$rc>>2]|0;
 HEAP32[$0>>2] = $113;
 $114 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($114|0);
}
function _clearCell($pPage,$pCell,$pnSize) {
 $pPage = $pPage|0;
 $pCell = $pCell|0;
 $pnSize = $pnSize|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0;
 var $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $iNext = 0, $info = 0, $nOvfl = 0;
 var $ovflPageSize = 0, $ovflPgno = 0, $pBt = 0, $pOvfl = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 80|0;
 $0 = sp + 64|0;
 $1 = sp + 60|0;
 $2 = sp + 56|0;
 $3 = sp + 52|0;
 $pBt = sp + 48|0;
 $info = sp;
 $ovflPgno = sp + 44|0;
 $rc = sp + 40|0;
 $nOvfl = sp + 36|0;
 $ovflPageSize = sp + 32|0;
 $iNext = sp + 28|0;
 $pOvfl = sp + 24|0;
 HEAP32[$1>>2] = $pPage;
 HEAP32[$2>>2] = $pCell;
 HEAP32[$3>>2] = $pnSize;
 $4 = HEAP32[$1>>2]|0;
 $5 = (($4) + 52|0);
 $6 = HEAP32[$5>>2]|0;
 HEAP32[$pBt>>2] = $6;
 $7 = HEAP32[$1>>2]|0;
 $8 = HEAP32[$2>>2]|0;
 _btreeParseCellPtr($7,$8,$info);
 $9 = (($info) + 20|0);
 $10 = HEAP16[$9>>1]|0;
 $11 = HEAP32[$3>>2]|0;
 HEAP16[$11>>1] = $10;
 $12 = (($info) + 18|0);
 $13 = HEAP16[$12>>1]|0;
 $14 = $13&65535;
 $15 = ($14|0)==(0);
 if ($15) {
  HEAP32[$0>>2] = 0;
  $96 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($96|0);
 }
 $16 = HEAP32[$2>>2]|0;
 $17 = (($info) + 18|0);
 $18 = HEAP16[$17>>1]|0;
 $19 = $18&65535;
 $20 = (($16) + ($19)|0);
 $21 = (($20) + 3|0);
 $22 = HEAP32[$1>>2]|0;
 $23 = (($22) + 56|0);
 $24 = HEAP32[$23>>2]|0;
 $25 = HEAP32[$1>>2]|0;
 $26 = (($25) + 20|0);
 $27 = HEAP16[$26>>1]|0;
 $28 = $27&65535;
 $29 = (($24) + ($28)|0);
 $30 = ($21>>>0)>($29>>>0);
 if ($30) {
  $31 = (_sqlite3CorruptError(57786)|0);
  HEAP32[$0>>2] = $31;
  $96 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($96|0);
 }
 $32 = (($info) + 18|0);
 $33 = HEAP16[$32>>1]|0;
 $34 = $33&65535;
 $35 = HEAP32[$2>>2]|0;
 $36 = (($35) + ($34)|0);
 $37 = (_sqlite3Get4byte($36)|0);
 HEAP32[$ovflPgno>>2] = $37;
 $38 = HEAP32[$pBt>>2]|0;
 $39 = (($38) + 36|0);
 $40 = HEAP32[$39>>2]|0;
 $41 = (($40) - 4)|0;
 HEAP32[$ovflPageSize>>2] = $41;
 $42 = (($info) + 12|0);
 $43 = HEAP32[$42>>2]|0;
 $44 = (($info) + 16|0);
 $45 = HEAP16[$44>>1]|0;
 $46 = $45&65535;
 $47 = (($43) - ($46))|0;
 $48 = HEAP32[$ovflPageSize>>2]|0;
 $49 = (($47) + ($48))|0;
 $50 = (($49) - 1)|0;
 $51 = HEAP32[$ovflPageSize>>2]|0;
 $52 = (($50>>>0) / ($51>>>0))&-1;
 HEAP32[$nOvfl>>2] = $52;
 while(1) {
  $53 = HEAP32[$nOvfl>>2]|0;
  $54 = (($53) + -1)|0;
  HEAP32[$nOvfl>>2] = $54;
  $55 = ($53|0)!=(0);
  if (!($55)) {
   label = 23;
   break;
  }
  HEAP32[$iNext>>2] = 0;
  HEAP32[$pOvfl>>2] = 0;
  $56 = HEAP32[$ovflPgno>>2]|0;
  $57 = ($56>>>0)<(2);
  if ($57) {
   label = 9;
   break;
  }
  $58 = HEAP32[$ovflPgno>>2]|0;
  $59 = HEAP32[$pBt>>2]|0;
  $60 = (_btreePagecount($59)|0);
  $61 = ($58>>>0)>($60>>>0);
  if ($61) {
   label = 9;
   break;
  }
  $63 = HEAP32[$nOvfl>>2]|0;
  $64 = ($63|0)!=(0);
  if ($64) {
   $65 = HEAP32[$pBt>>2]|0;
   $66 = HEAP32[$ovflPgno>>2]|0;
   $67 = (_getOverflowPage($65,$66,$pOvfl,$iNext)|0);
   HEAP32[$rc>>2] = $67;
   $68 = HEAP32[$rc>>2]|0;
   $69 = ($68|0)!=(0);
   if ($69) {
    label = 12;
    break;
   }
  }
  $71 = HEAP32[$pOvfl>>2]|0;
  $72 = ($71|0)!=(0|0);
  if ($72) {
   label = 15;
  } else {
   $73 = HEAP32[$pBt>>2]|0;
   $74 = HEAP32[$ovflPgno>>2]|0;
   $75 = (_btreePageLookup($73,$74)|0);
   HEAP32[$pOvfl>>2] = $75;
   $76 = ($75|0)!=(0|0);
   if ($76) {
    label = 15;
   } else {
    label = 17;
   }
  }
  if ((label|0) == 15) {
   label = 0;
   $77 = HEAP32[$pOvfl>>2]|0;
   $78 = (($77) + 68|0);
   $79 = HEAP32[$78>>2]|0;
   $80 = (_sqlite3PagerPageRefcount($79)|0);
   $81 = ($80|0)!=(1);
   if ($81) {
    $82 = (_sqlite3CorruptError(57820)|0);
    HEAP32[$rc>>2] = $82;
   } else {
    label = 17;
   }
  }
  if ((label|0) == 17) {
   label = 0;
   $83 = HEAP32[$pBt>>2]|0;
   $84 = HEAP32[$pOvfl>>2]|0;
   $85 = HEAP32[$ovflPgno>>2]|0;
   $86 = (_freePage2($83,$84,$85)|0);
   HEAP32[$rc>>2] = $86;
  }
  $87 = HEAP32[$pOvfl>>2]|0;
  $88 = ($87|0)!=(0|0);
  if ($88) {
   $89 = HEAP32[$pOvfl>>2]|0;
   $90 = (($89) + 68|0);
   $91 = HEAP32[$90>>2]|0;
   _sqlite3PagerUnref($91);
  }
  $92 = HEAP32[$rc>>2]|0;
  $93 = ($92|0)!=(0);
  if ($93) {
   label = 21;
   break;
  }
  $95 = HEAP32[$iNext>>2]|0;
  HEAP32[$ovflPgno>>2] = $95;
 }
 if ((label|0) == 9) {
  $62 = (_sqlite3CorruptError(57800)|0);
  HEAP32[$0>>2] = $62;
  $96 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($96|0);
 }
 else if ((label|0) == 12) {
  $70 = HEAP32[$rc>>2]|0;
  HEAP32[$0>>2] = $70;
  $96 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($96|0);
 }
 else if ((label|0) == 21) {
  $94 = HEAP32[$rc>>2]|0;
  HEAP32[$0>>2] = $94;
  $96 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($96|0);
 }
 else if ((label|0) == 23) {
  HEAP32[$0>>2] = 0;
  $96 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($96|0);
 }
 return 0|0;
}
function _freePage($pPage,$pRC) {
 $pPage = $pPage|0;
 $pRC = $pRC|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $1 = sp;
 HEAP32[$0>>2] = $pPage;
 HEAP32[$1>>2] = $pRC;
 $2 = HEAP32[$1>>2]|0;
 $3 = HEAP32[$2>>2]|0;
 $4 = ($3|0)==(0);
 if (!($4)) {
  STACKTOP = sp;return;
 }
 $5 = HEAP32[$0>>2]|0;
 $6 = (($5) + 52|0);
 $7 = HEAP32[$6>>2]|0;
 $8 = HEAP32[$0>>2]|0;
 $9 = HEAP32[$0>>2]|0;
 $10 = (($9) + 72|0);
 $11 = HEAP32[$10>>2]|0;
 $12 = (_freePage2($7,$8,$11)|0);
 $13 = HEAP32[$1>>2]|0;
 HEAP32[$13>>2] = $12;
 STACKTOP = sp;return;
}
function _zeroPage($pPage,$flags) {
 $pPage = $pPage|0;
 $flags = $flags|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0;
 var $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0;
 var $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0;
 var $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0;
 var $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0;
 var $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $data = 0, $first = 0, $hdr = 0, $pBt = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 12|0;
 $1 = sp + 8|0;
 $data = sp + 4|0;
 $pBt = sp;
 $hdr = sp + 18|0;
 $first = sp + 16|0;
 HEAP32[$0>>2] = $pPage;
 HEAP32[$1>>2] = $flags;
 $2 = HEAP32[$0>>2]|0;
 $3 = (($2) + 56|0);
 $4 = HEAP32[$3>>2]|0;
 HEAP32[$data>>2] = $4;
 $5 = HEAP32[$0>>2]|0;
 $6 = (($5) + 52|0);
 $7 = HEAP32[$6>>2]|0;
 HEAP32[$pBt>>2] = $7;
 $8 = HEAP32[$0>>2]|0;
 $9 = (($8) + 6|0);
 $10 = HEAP8[$9>>0]|0;
 HEAP8[$hdr>>0] = $10;
 $11 = HEAP32[$pBt>>2]|0;
 $12 = (($11) + 22|0);
 $13 = HEAP16[$12>>1]|0;
 $14 = $13&65535;
 $15 = $14 & 4;
 $16 = ($15|0)!=(0);
 if ($16) {
  $17 = HEAP8[$hdr>>0]|0;
  $18 = $17&255;
  $19 = HEAP32[$data>>2]|0;
  $20 = (($19) + ($18)|0);
  $21 = HEAP32[$pBt>>2]|0;
  $22 = (($21) + 36|0);
  $23 = HEAP32[$22>>2]|0;
  $24 = HEAP8[$hdr>>0]|0;
  $25 = $24&255;
  $26 = (($23) - ($25))|0;
  _memset(($20|0),0,($26|0))|0;
 }
 $27 = HEAP32[$1>>2]|0;
 $28 = $27&255;
 $29 = HEAP8[$hdr>>0]|0;
 $30 = $29&255;
 $31 = HEAP32[$data>>2]|0;
 $32 = (($31) + ($30)|0);
 HEAP8[$32>>0] = $28;
 $33 = HEAP8[$hdr>>0]|0;
 $34 = $33&255;
 $35 = HEAP32[$1>>2]|0;
 $36 = $35 & 8;
 $37 = ($36|0)==(0);
 $38 = $37 ? 12 : 8;
 $39 = (($34) + ($38))|0;
 $40 = $39&65535;
 HEAP16[$first>>1] = $40;
 $41 = HEAP8[$hdr>>0]|0;
 $42 = $41&255;
 $43 = (($42) + 1)|0;
 $44 = HEAP32[$data>>2]|0;
 $45 = (($44) + ($43)|0);
 ;HEAP8[$45+0>>0]=0|0;HEAP8[$45+1>>0]=0|0;HEAP8[$45+2>>0]=0|0;HEAP8[$45+3>>0]=0|0;
 $46 = HEAP8[$hdr>>0]|0;
 $47 = $46&255;
 $48 = (($47) + 7)|0;
 $49 = HEAP32[$data>>2]|0;
 $50 = (($49) + ($48)|0);
 HEAP8[$50>>0] = 0;
 $51 = HEAP32[$pBt>>2]|0;
 $52 = (($51) + 36|0);
 $53 = HEAP32[$52>>2]|0;
 $54 = $53 >>> 8;
 $55 = $54&255;
 $56 = HEAP8[$hdr>>0]|0;
 $57 = $56&255;
 $58 = (($57) + 5)|0;
 $59 = HEAP32[$data>>2]|0;
 $60 = (($59) + ($58)|0);
 HEAP8[$60>>0] = $55;
 $61 = HEAP32[$pBt>>2]|0;
 $62 = (($61) + 36|0);
 $63 = HEAP32[$62>>2]|0;
 $64 = $63&255;
 $65 = HEAP8[$hdr>>0]|0;
 $66 = $65&255;
 $67 = (($66) + 5)|0;
 $68 = HEAP32[$data>>2]|0;
 $69 = (($68) + ($67)|0);
 $70 = (($69) + 1|0);
 HEAP8[$70>>0] = $64;
 $71 = HEAP32[$pBt>>2]|0;
 $72 = (($71) + 36|0);
 $73 = HEAP32[$72>>2]|0;
 $74 = HEAP16[$first>>1]|0;
 $75 = $74&65535;
 $76 = (($73) - ($75))|0;
 $77 = $76&65535;
 $78 = HEAP32[$0>>2]|0;
 $79 = (($78) + 16|0);
 HEAP16[$79>>1] = $77;
 $80 = HEAP32[$0>>2]|0;
 $81 = HEAP32[$1>>2]|0;
 (_decodeFlags($80,$81)|0);
 $82 = HEAP16[$first>>1]|0;
 $83 = HEAP32[$0>>2]|0;
 $84 = (($83) + 14|0);
 HEAP16[$84>>1] = $82;
 $85 = HEAP32[$pBt>>2]|0;
 $86 = (($85) + 36|0);
 $87 = HEAP32[$86>>2]|0;
 $88 = HEAP32[$data>>2]|0;
 $89 = (($88) + ($87)|0);
 $90 = HEAP32[$0>>2]|0;
 $91 = (($90) + 60|0);
 HEAP32[$91>>2] = $89;
 $92 = HEAP16[$first>>1]|0;
 $93 = $92&65535;
 $94 = HEAP32[$data>>2]|0;
 $95 = (($94) + ($93)|0);
 $96 = HEAP32[$0>>2]|0;
 $97 = (($96) + 64|0);
 HEAP32[$97>>2] = $95;
 $98 = HEAP32[$0>>2]|0;
 $99 = (($98) + 1|0);
 HEAP8[$99>>0] = 0;
 $100 = HEAP32[$pBt>>2]|0;
 $101 = (($100) + 32|0);
 $102 = HEAP32[$101>>2]|0;
 $103 = (($102) - 1)|0;
 $104 = $103&65535;
 $105 = HEAP32[$0>>2]|0;
 $106 = (($105) + 20|0);
 HEAP16[$106>>1] = $104;
 $107 = HEAP32[$0>>2]|0;
 $108 = (($107) + 18|0);
 HEAP16[$108>>1] = 0;
 $109 = HEAP32[$0>>2]|0;
 HEAP8[$109>>0] = 1;
 STACKTOP = sp;return;
}
function _freePage2($pBt,$pMemPage,$iPage) {
 $pBt = $pBt|0;
 $pMemPage = $pMemPage|0;
 $iPage = $iPage|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0;
 var $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0;
 var $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0;
 var $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0;
 var $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0;
 var $98 = 0, $99 = 0, $iTrunk = 0, $nFree = 0, $nLeaf = 0, $pPage = 0, $pPage1 = 0, $pTrunk = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $0 = sp + 36|0;
 $1 = sp + 32|0;
 $2 = sp + 28|0;
 $pTrunk = sp + 24|0;
 $iTrunk = sp + 20|0;
 $pPage1 = sp + 16|0;
 $pPage = sp + 12|0;
 $rc = sp + 8|0;
 $nFree = sp + 4|0;
 $nLeaf = sp;
 HEAP32[$0>>2] = $pBt;
 HEAP32[$1>>2] = $pMemPage;
 HEAP32[$2>>2] = $iPage;
 HEAP32[$pTrunk>>2] = 0;
 HEAP32[$iTrunk>>2] = 0;
 $3 = HEAP32[$0>>2]|0;
 $4 = (($3) + 12|0);
 $5 = HEAP32[$4>>2]|0;
 HEAP32[$pPage1>>2] = $5;
 $6 = HEAP32[$1>>2]|0;
 $7 = ($6|0)!=(0|0);
 if ($7) {
  $8 = HEAP32[$1>>2]|0;
  HEAP32[$pPage>>2] = $8;
  $9 = HEAP32[$pPage>>2]|0;
  $10 = (($9) + 68|0);
  $11 = HEAP32[$10>>2]|0;
  _sqlite3PagerRef($11);
 } else {
  $12 = HEAP32[$0>>2]|0;
  $13 = HEAP32[$2>>2]|0;
  $14 = (_btreePageLookup($12,$13)|0);
  HEAP32[$pPage>>2] = $14;
 }
 $15 = HEAP32[$pPage1>>2]|0;
 $16 = (($15) + 68|0);
 $17 = HEAP32[$16>>2]|0;
 $18 = (_sqlite3PagerWrite($17)|0);
 HEAP32[$rc>>2] = $18;
 $19 = HEAP32[$rc>>2]|0;
 $20 = ($19|0)!=(0);
 do {
  if (!($20)) {
   $21 = HEAP32[$pPage1>>2]|0;
   $22 = (($21) + 56|0);
   $23 = HEAP32[$22>>2]|0;
   $24 = (($23) + 36|0);
   $25 = (_sqlite3Get4byte($24)|0);
   HEAP32[$nFree>>2] = $25;
   $26 = HEAP32[$pPage1>>2]|0;
   $27 = (($26) + 56|0);
   $28 = HEAP32[$27>>2]|0;
   $29 = (($28) + 36|0);
   $30 = HEAP32[$nFree>>2]|0;
   $31 = (($30) + 1)|0;
   _sqlite3Put4byte($29,$31);
   $32 = HEAP32[$0>>2]|0;
   $33 = (($32) + 22|0);
   $34 = HEAP16[$33>>1]|0;
   $35 = $34&65535;
   $36 = $35 & 4;
   $37 = ($36|0)!=(0);
   if ($37) {
    $38 = HEAP32[$pPage>>2]|0;
    $39 = ($38|0)!=(0|0);
    if (!($39)) {
     $40 = HEAP32[$0>>2]|0;
     $41 = HEAP32[$2>>2]|0;
     $42 = (_btreeGetPage($40,$41,$pPage,0)|0);
     HEAP32[$rc>>2] = $42;
     $43 = ($42|0)!=(0);
     if ($43) {
      break;
     }
    }
    $44 = HEAP32[$pPage>>2]|0;
    $45 = (($44) + 68|0);
    $46 = HEAP32[$45>>2]|0;
    $47 = (_sqlite3PagerWrite($46)|0);
    HEAP32[$rc>>2] = $47;
    $48 = ($47|0)!=(0);
    if ($48) {
     break;
    }
    $49 = HEAP32[$pPage>>2]|0;
    $50 = (($49) + 56|0);
    $51 = HEAP32[$50>>2]|0;
    $52 = HEAP32[$pPage>>2]|0;
    $53 = (($52) + 52|0);
    $54 = HEAP32[$53>>2]|0;
    $55 = (($54) + 32|0);
    $56 = HEAP32[$55>>2]|0;
    _memset(($51|0),0,($56|0))|0;
   }
   $57 = HEAP32[$0>>2]|0;
   $58 = (($57) + 17|0);
   $59 = HEAP8[$58>>0]|0;
   $60 = ($59<<24>>24)!=(0);
   if ($60) {
    $61 = HEAP32[$0>>2]|0;
    $62 = HEAP32[$2>>2]|0;
    _ptrmapPut($61,$62,2,0,$rc);
    $63 = HEAP32[$rc>>2]|0;
    $64 = ($63|0)!=(0);
    if ($64) {
     break;
    }
   }
   $65 = HEAP32[$nFree>>2]|0;
   $66 = ($65|0)!=(0);
   if ($66) {
    $67 = HEAP32[$pPage1>>2]|0;
    $68 = (($67) + 56|0);
    $69 = HEAP32[$68>>2]|0;
    $70 = (($69) + 32|0);
    $71 = (_sqlite3Get4byte($70)|0);
    HEAP32[$iTrunk>>2] = $71;
    $72 = HEAP32[$0>>2]|0;
    $73 = HEAP32[$iTrunk>>2]|0;
    $74 = (_btreeGetPage($72,$73,$pTrunk,0)|0);
    HEAP32[$rc>>2] = $74;
    $75 = HEAP32[$rc>>2]|0;
    $76 = ($75|0)!=(0);
    if ($76) {
     break;
    }
    $77 = HEAP32[$pTrunk>>2]|0;
    $78 = (($77) + 56|0);
    $79 = HEAP32[$78>>2]|0;
    $80 = (($79) + 4|0);
    $81 = (_sqlite3Get4byte($80)|0);
    HEAP32[$nLeaf>>2] = $81;
    $82 = HEAP32[$nLeaf>>2]|0;
    $83 = HEAP32[$0>>2]|0;
    $84 = (($83) + 36|0);
    $85 = HEAP32[$84>>2]|0;
    $86 = (($85>>>0) / 4)&-1;
    $87 = (($86) - 2)|0;
    $88 = ($82>>>0)>($87>>>0);
    if ($88) {
     $89 = (_sqlite3CorruptError(57698)|0);
     HEAP32[$rc>>2] = $89;
     break;
    }
    $90 = HEAP32[$nLeaf>>2]|0;
    $91 = HEAP32[$0>>2]|0;
    $92 = (($91) + 36|0);
    $93 = HEAP32[$92>>2]|0;
    $94 = (($93>>>0) / 4)&-1;
    $95 = (($94) - 8)|0;
    $96 = ($90>>>0)<($95>>>0);
    if ($96) {
     $97 = HEAP32[$pTrunk>>2]|0;
     $98 = (($97) + 68|0);
     $99 = HEAP32[$98>>2]|0;
     $100 = (_sqlite3PagerWrite($99)|0);
     HEAP32[$rc>>2] = $100;
     $101 = HEAP32[$rc>>2]|0;
     $102 = ($101|0)==(0);
     if (!($102)) {
      break;
     }
     $103 = HEAP32[$pTrunk>>2]|0;
     $104 = (($103) + 56|0);
     $105 = HEAP32[$104>>2]|0;
     $106 = (($105) + 4|0);
     $107 = HEAP32[$nLeaf>>2]|0;
     $108 = (($107) + 1)|0;
     _sqlite3Put4byte($106,$108);
     $109 = HEAP32[$nLeaf>>2]|0;
     $110 = $109<<2;
     $111 = (8 + ($110))|0;
     $112 = HEAP32[$pTrunk>>2]|0;
     $113 = (($112) + 56|0);
     $114 = HEAP32[$113>>2]|0;
     $115 = (($114) + ($111)|0);
     $116 = HEAP32[$2>>2]|0;
     _sqlite3Put4byte($115,$116);
     $117 = HEAP32[$pPage>>2]|0;
     $118 = ($117|0)!=(0|0);
     if ($118) {
      $119 = HEAP32[$0>>2]|0;
      $120 = (($119) + 22|0);
      $121 = HEAP16[$120>>1]|0;
      $122 = $121&65535;
      $123 = $122 & 4;
      $124 = ($123|0)==(0);
      if ($124) {
       $125 = HEAP32[$pPage>>2]|0;
       $126 = (($125) + 68|0);
       $127 = HEAP32[$126>>2]|0;
       _sqlite3PagerDontWrite($127);
      }
     }
     $128 = HEAP32[$0>>2]|0;
     $129 = HEAP32[$2>>2]|0;
     $130 = (_btreeSetHasContent($128,$129)|0);
     HEAP32[$rc>>2] = $130;
     break;
    }
   }
   $131 = HEAP32[$pPage>>2]|0;
   $132 = ($131|0)==(0|0);
   if ($132) {
    $133 = HEAP32[$0>>2]|0;
    $134 = HEAP32[$2>>2]|0;
    $135 = (_btreeGetPage($133,$134,$pPage,0)|0);
    HEAP32[$rc>>2] = $135;
    $136 = (0)!=($135|0);
    if ($136) {
     break;
    }
   }
   $137 = HEAP32[$pPage>>2]|0;
   $138 = (($137) + 68|0);
   $139 = HEAP32[$138>>2]|0;
   $140 = (_sqlite3PagerWrite($139)|0);
   HEAP32[$rc>>2] = $140;
   $141 = HEAP32[$rc>>2]|0;
   $142 = ($141|0)!=(0);
   if (!($142)) {
    $143 = HEAP32[$pPage>>2]|0;
    $144 = (($143) + 56|0);
    $145 = HEAP32[$144>>2]|0;
    $146 = HEAP32[$iTrunk>>2]|0;
    _sqlite3Put4byte($145,$146);
    $147 = HEAP32[$pPage>>2]|0;
    $148 = (($147) + 56|0);
    $149 = HEAP32[$148>>2]|0;
    $150 = (($149) + 4|0);
    _sqlite3Put4byte($150,0);
    $151 = HEAP32[$pPage1>>2]|0;
    $152 = (($151) + 56|0);
    $153 = HEAP32[$152>>2]|0;
    $154 = (($153) + 32|0);
    $155 = HEAP32[$2>>2]|0;
    _sqlite3Put4byte($154,$155);
   }
  }
 } while(0);
 $156 = HEAP32[$pPage>>2]|0;
 $157 = ($156|0)!=(0|0);
 if (!($157)) {
  $159 = HEAP32[$pPage>>2]|0;
  _releasePage($159);
  $160 = HEAP32[$pTrunk>>2]|0;
  _releasePage($160);
  $161 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($161|0);
 }
 $158 = HEAP32[$pPage>>2]|0;
 HEAP8[$158>>0] = 0;
 $159 = HEAP32[$pPage>>2]|0;
 _releasePage($159);
 $160 = HEAP32[$pTrunk>>2]|0;
 _releasePage($160);
 $161 = HEAP32[$rc>>2]|0;
 STACKTOP = sp;return ($161|0);
}
function _sqlite3PagerRef($pPg) {
 $pPg = $pPg|0;
 var $0 = 0, $1 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 HEAP32[$0>>2] = $pPg;
 $1 = HEAP32[$0>>2]|0;
 _sqlite3PcacheRef($1);
 STACKTOP = sp;return;
}
function _btreePageLookup($pBt,$pgno) {
 $pBt = $pBt|0;
 $pgno = $pgno|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $pDbPage = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $1 = sp + 8|0;
 $2 = sp + 4|0;
 $pDbPage = sp;
 HEAP32[$1>>2] = $pBt;
 HEAP32[$2>>2] = $pgno;
 $3 = HEAP32[$1>>2]|0;
 $4 = HEAP32[$3>>2]|0;
 $5 = HEAP32[$2>>2]|0;
 $6 = (_sqlite3PagerLookup($4,$5)|0);
 HEAP32[$pDbPage>>2] = $6;
 $7 = HEAP32[$pDbPage>>2]|0;
 $8 = ($7|0)!=(0|0);
 if ($8) {
  $9 = HEAP32[$pDbPage>>2]|0;
  $10 = HEAP32[$2>>2]|0;
  $11 = HEAP32[$1>>2]|0;
  $12 = (_btreePageFromDbPage($9,$10,$11)|0);
  HEAP32[$0>>2] = $12;
  $13 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($13|0);
 } else {
  HEAP32[$0>>2] = 0;
  $13 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($13|0);
 }
 return 0|0;
}
function _sqlite3PagerDontWrite($pPg) {
 $pPg = $pPg|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0;
 var $pPager = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $pPager = sp;
 HEAP32[$0>>2] = $pPg;
 $1 = HEAP32[$0>>2]|0;
 $2 = (($1) + 16|0);
 $3 = HEAP32[$2>>2]|0;
 HEAP32[$pPager>>2] = $3;
 $4 = HEAP32[$0>>2]|0;
 $5 = (($4) + 24|0);
 $6 = HEAP16[$5>>1]|0;
 $7 = $6&65535;
 $8 = $7 & 2;
 $9 = ($8|0)!=(0);
 if (!($9)) {
  STACKTOP = sp;return;
 }
 $10 = HEAP32[$pPager>>2]|0;
 $11 = (($10) + 96|0);
 $12 = HEAP32[$11>>2]|0;
 $13 = ($12|0)==(0);
 if (!($13)) {
  STACKTOP = sp;return;
 }
 $14 = HEAP32[$0>>2]|0;
 $15 = (($14) + 24|0);
 $16 = HEAP16[$15>>1]|0;
 $17 = $16&65535;
 $18 = $17 | 32;
 $19 = $18&65535;
 HEAP16[$15>>1] = $19;
 STACKTOP = sp;return;
}
function _btreeSetHasContent($pBt,$pgno) {
 $pBt = $pBt|0;
 $pgno = $pgno|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $1 = sp + 4|0;
 $rc = sp;
 HEAP32[$0>>2] = $pBt;
 HEAP32[$1>>2] = $pgno;
 HEAP32[$rc>>2] = 0;
 $2 = HEAP32[$0>>2]|0;
 $3 = (($2) + 60|0);
 $4 = HEAP32[$3>>2]|0;
 $5 = ($4|0)!=(0|0);
 if (!($5)) {
  $6 = HEAP32[$0>>2]|0;
  $7 = (($6) + 44|0);
  $8 = HEAP32[$7>>2]|0;
  $9 = (_sqlite3BitvecCreate($8)|0);
  $10 = HEAP32[$0>>2]|0;
  $11 = (($10) + 60|0);
  HEAP32[$11>>2] = $9;
  $12 = HEAP32[$0>>2]|0;
  $13 = (($12) + 60|0);
  $14 = HEAP32[$13>>2]|0;
  $15 = ($14|0)!=(0|0);
  if (!($15)) {
   HEAP32[$rc>>2] = 7;
  }
 }
 $16 = HEAP32[$rc>>2]|0;
 $17 = ($16|0)==(0);
 if (!($17)) {
  $29 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($29|0);
 }
 $18 = HEAP32[$1>>2]|0;
 $19 = HEAP32[$0>>2]|0;
 $20 = (($19) + 60|0);
 $21 = HEAP32[$20>>2]|0;
 $22 = (_sqlite3BitvecSize($21)|0);
 $23 = ($18>>>0)<=($22>>>0);
 if (!($23)) {
  $29 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($29|0);
 }
 $24 = HEAP32[$0>>2]|0;
 $25 = (($24) + 60|0);
 $26 = HEAP32[$25>>2]|0;
 $27 = HEAP32[$1>>2]|0;
 $28 = (_sqlite3BitvecSet($26,$27)|0);
 HEAP32[$rc>>2] = $28;
 $29 = HEAP32[$rc>>2]|0;
 STACKTOP = sp;return ($29|0);
}
function _sqlite3PcacheRef($p) {
 $p = $p|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 HEAP32[$0>>2] = $p;
 $1 = HEAP32[$0>>2]|0;
 $2 = (($1) + 26|0);
 $3 = HEAP16[$2>>1]|0;
 $4 = (($3) + 1)<<16>>16;
 HEAP16[$2>>1] = $4;
 STACKTOP = sp;return;
}
function _btreeDropTable($p,$iTable,$piMoved) {
 $p = $p|0;
 $iTable = $iTable|0;
 $piMoved = $piMoved|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0;
 var $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $maxRootPgno = 0, $pBt = 0, $pMove = 0, $pPage = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $0 = sp + 32|0;
 $1 = sp + 28|0;
 $2 = sp + 24|0;
 $3 = sp + 20|0;
 $rc = sp + 16|0;
 $pPage = sp + 12|0;
 $pBt = sp + 8|0;
 $maxRootPgno = sp + 4|0;
 $pMove = sp;
 HEAP32[$1>>2] = $p;
 HEAP32[$2>>2] = $iTable;
 HEAP32[$3>>2] = $piMoved;
 HEAP32[$pPage>>2] = 0;
 $4 = HEAP32[$1>>2]|0;
 $5 = (($4) + 4|0);
 $6 = HEAP32[$5>>2]|0;
 HEAP32[$pBt>>2] = $6;
 $7 = HEAP32[$pBt>>2]|0;
 $8 = (($7) + 8|0);
 $9 = HEAP32[$8>>2]|0;
 $10 = ($9|0)!=(0|0);
 if ($10) {
  HEAP32[$0>>2] = 262;
  $89 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($89|0);
 }
 $11 = HEAP32[$pBt>>2]|0;
 $12 = HEAP32[$2>>2]|0;
 $13 = (_btreeGetPage($11,$12,$pPage,0)|0);
 HEAP32[$rc>>2] = $13;
 $14 = HEAP32[$rc>>2]|0;
 $15 = ($14|0)!=(0);
 if ($15) {
  $16 = HEAP32[$rc>>2]|0;
  HEAP32[$0>>2] = $16;
  $89 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($89|0);
 }
 $17 = HEAP32[$1>>2]|0;
 $18 = HEAP32[$2>>2]|0;
 $19 = (_sqlite3BtreeClearTable($17,$18,0)|0);
 HEAP32[$rc>>2] = $19;
 $20 = HEAP32[$rc>>2]|0;
 $21 = ($20|0)!=(0);
 if ($21) {
  $22 = HEAP32[$pPage>>2]|0;
  _releasePage($22);
  $23 = HEAP32[$rc>>2]|0;
  HEAP32[$0>>2] = $23;
  $89 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($89|0);
 }
 $24 = HEAP32[$3>>2]|0;
 HEAP32[$24>>2] = 0;
 $25 = HEAP32[$2>>2]|0;
 $26 = ($25>>>0)>(1);
 do {
  if ($26) {
   $27 = HEAP32[$pBt>>2]|0;
   $28 = (($27) + 17|0);
   $29 = HEAP8[$28>>0]|0;
   $30 = ($29<<24>>24)!=(0);
   if (!($30)) {
    $84 = HEAP32[$pPage>>2]|0;
    _freePage($84,$rc);
    $85 = HEAP32[$pPage>>2]|0;
    _releasePage($85);
    break;
   }
   $31 = HEAP32[$1>>2]|0;
   _sqlite3BtreeGetMeta($31,4,$maxRootPgno);
   $32 = HEAP32[$2>>2]|0;
   $33 = HEAP32[$maxRootPgno>>2]|0;
   $34 = ($32|0)==($33|0);
   $35 = HEAP32[$pPage>>2]|0;
   do {
    if ($34) {
     _freePage($35,$rc);
     $36 = HEAP32[$pPage>>2]|0;
     _releasePage($36);
     $37 = HEAP32[$rc>>2]|0;
     $38 = ($37|0)!=(0);
     if ($38) {
      $39 = HEAP32[$rc>>2]|0;
      HEAP32[$0>>2] = $39;
      $89 = HEAP32[$0>>2]|0;
      STACKTOP = sp;return ($89|0);
     }
    } else {
     _releasePage($35);
     $40 = HEAP32[$pBt>>2]|0;
     $41 = HEAP32[$maxRootPgno>>2]|0;
     $42 = (_btreeGetPage($40,$41,$pMove,0)|0);
     HEAP32[$rc>>2] = $42;
     $43 = HEAP32[$rc>>2]|0;
     $44 = ($43|0)!=(0);
     if ($44) {
      $45 = HEAP32[$rc>>2]|0;
      HEAP32[$0>>2] = $45;
      $89 = HEAP32[$0>>2]|0;
      STACKTOP = sp;return ($89|0);
     }
     $46 = HEAP32[$pBt>>2]|0;
     $47 = HEAP32[$pMove>>2]|0;
     $48 = HEAP32[$2>>2]|0;
     $49 = (_relocatePage($46,$47,1,0,$48,0)|0);
     HEAP32[$rc>>2] = $49;
     $50 = HEAP32[$pMove>>2]|0;
     _releasePage($50);
     $51 = HEAP32[$rc>>2]|0;
     $52 = ($51|0)!=(0);
     if ($52) {
      $53 = HEAP32[$rc>>2]|0;
      HEAP32[$0>>2] = $53;
      $89 = HEAP32[$0>>2]|0;
      STACKTOP = sp;return ($89|0);
     }
     HEAP32[$pMove>>2] = 0;
     $54 = HEAP32[$pBt>>2]|0;
     $55 = HEAP32[$maxRootPgno>>2]|0;
     $56 = (_btreeGetPage($54,$55,$pMove,0)|0);
     HEAP32[$rc>>2] = $56;
     $57 = HEAP32[$pMove>>2]|0;
     _freePage($57,$rc);
     $58 = HEAP32[$pMove>>2]|0;
     _releasePage($58);
     $59 = HEAP32[$rc>>2]|0;
     $60 = ($59|0)!=(0);
     if (!($60)) {
      $62 = HEAP32[$maxRootPgno>>2]|0;
      $63 = HEAP32[$3>>2]|0;
      HEAP32[$63>>2] = $62;
      break;
     }
     $61 = HEAP32[$rc>>2]|0;
     HEAP32[$0>>2] = $61;
     $89 = HEAP32[$0>>2]|0;
     STACKTOP = sp;return ($89|0);
    }
   } while(0);
   $64 = HEAP32[$maxRootPgno>>2]|0;
   $65 = (($64) + -1)|0;
   HEAP32[$maxRootPgno>>2] = $65;
   while(1) {
    $66 = HEAP32[$maxRootPgno>>2]|0;
    $67 = HEAP32[1128>>2]|0;
    $68 = HEAP32[$pBt>>2]|0;
    $69 = (($68) + 32|0);
    $70 = HEAP32[$69>>2]|0;
    $71 = (($67>>>0) / ($70>>>0))&-1;
    $72 = (($71) + 1)|0;
    $73 = ($66|0)==($72|0);
    if (!($73)) {
     $74 = HEAP32[$pBt>>2]|0;
     $75 = HEAP32[$maxRootPgno>>2]|0;
     $76 = (_ptrmapPageno($74,$75)|0);
     $77 = HEAP32[$maxRootPgno>>2]|0;
     $78 = ($76|0)==($77|0);
     if (!($78)) {
      break;
     }
    }
    $79 = HEAP32[$maxRootPgno>>2]|0;
    $80 = (($79) + -1)|0;
    HEAP32[$maxRootPgno>>2] = $80;
   }
   $81 = HEAP32[$1>>2]|0;
   $82 = HEAP32[$maxRootPgno>>2]|0;
   $83 = (_sqlite3BtreeUpdateMeta($81,4,$82)|0);
   HEAP32[$rc>>2] = $83;
  } else {
   $86 = HEAP32[$pPage>>2]|0;
   _zeroPage($86,9);
   $87 = HEAP32[$pPage>>2]|0;
   _releasePage($87);
  }
 } while(0);
 $88 = HEAP32[$rc>>2]|0;
 HEAP32[$0>>2] = $88;
 $89 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($89|0);
}
function _vdbeSorterFlushPMA($pSorter) {
 $pSorter = $pSorter|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 HEAP32[$0>>2] = $pSorter;
 $1 = HEAP32[$0>>2]|0;
 $2 = (($1) + 56|0);
 HEAP8[$2>>0] = 1;
 $3 = HEAP32[$0>>2]|0;
 $4 = (($3) + 64|0);
 $5 = HEAP32[$0>>2]|0;
 $6 = (($5) + 36|0);
 $7 = (_vdbeSorterListToPMA($4,$6)|0);
 STACKTOP = sp;return ($7|0);
}
function _vdbeSorterListToPMA($pTask,$pList) {
 $pTask = $pTask|0;
 $pList = $pList|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $8 = 0, $9 = 0, $db = 0, $p = 0, $pNext = 0, $rc = 0, $writer = 0;
 var dest = 0, label = 0, sp = 0, stop = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 64|0;
 $0 = sp + 60|0;
 $1 = sp + 56|0;
 $db = sp + 52|0;
 $rc = sp + 48|0;
 $writer = sp;
 $p = sp + 44|0;
 $pNext = sp + 40|0;
 HEAP32[$0>>2] = $pTask;
 HEAP32[$1>>2] = $pList;
 $2 = HEAP32[$0>>2]|0;
 $3 = (($2) + 8|0);
 $4 = HEAP32[$3>>2]|0;
 $5 = (($4) + 24|0);
 $6 = HEAP32[$5>>2]|0;
 HEAP32[$db>>2] = $6;
 HEAP32[$rc>>2] = 0;
 dest=$writer+0|0; stop=dest+40|0; do { HEAP32[dest>>2]=0|0; dest=dest+4|0; } while ((dest|0) < (stop|0));
 $7 = HEAP32[$0>>2]|0;
 $8 = (($7) + 32|0);
 $9 = HEAP32[$8>>2]|0;
 $10 = ($9|0)==(0|0);
 if ($10) {
  $11 = HEAP32[$db>>2]|0;
  $12 = HEAP32[$0>>2]|0;
  $13 = (($12) + 32|0);
  $14 = (_vdbeSorterOpenTempFile($11,0,0,$13)|0);
  HEAP32[$rc>>2] = $14;
 }
 $15 = HEAP32[$rc>>2]|0;
 $16 = ($15|0)==(0);
 if ($16) {
  $17 = HEAP32[$0>>2]|0;
  $18 = HEAP32[$1>>2]|0;
  $19 = (_vdbeSorterSort($17,$18)|0);
  HEAP32[$rc>>2] = $19;
 }
 $20 = HEAP32[$rc>>2]|0;
 $21 = ($20|0)==(0);
 if (!($21)) {
  $74 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($74|0);
 }
 HEAP32[$pNext>>2] = 0;
 $22 = HEAP32[$0>>2]|0;
 $23 = (($22) + 32|0);
 $24 = HEAP32[$23>>2]|0;
 $25 = HEAP32[$0>>2]|0;
 $26 = (($25) + 8|0);
 $27 = HEAP32[$26>>2]|0;
 $28 = (($27) + 12|0);
 $29 = HEAP32[$28>>2]|0;
 $30 = HEAP32[$0>>2]|0;
 $31 = (($30) + 32|0);
 $32 = (($31) + 8|0);
 $33 = $32;
 $34 = $33;
 $35 = HEAP32[$34>>2]|0;
 $36 = (($33) + 4)|0;
 $37 = $36;
 $38 = HEAP32[$37>>2]|0;
 _vdbePmaWriterInit($24,$writer,$29,$35,$38);
 $39 = HEAP32[$0>>2]|0;
 $40 = (($39) + 28|0);
 $41 = HEAP32[$40>>2]|0;
 $42 = (($41) + 1)|0;
 HEAP32[$40>>2] = $42;
 $43 = HEAP32[$1>>2]|0;
 $44 = (($43) + 8|0);
 $45 = HEAP32[$44>>2]|0;
 $46 = ($45|0)<(0);
 $47 = $46 << 31 >> 31;
 _vdbePmaWriteVarint($writer,$45,$47);
 $48 = HEAP32[$1>>2]|0;
 $49 = HEAP32[$48>>2]|0;
 HEAP32[$p>>2] = $49;
 while(1) {
  $50 = HEAP32[$p>>2]|0;
  $51 = ($50|0)!=(0|0);
  $52 = HEAP32[$p>>2]|0;
  if (!($51)) {
   break;
  }
  $53 = (($52) + 4|0);
  $54 = HEAP32[$53>>2]|0;
  HEAP32[$pNext>>2] = $54;
  $55 = HEAP32[$p>>2]|0;
  $56 = HEAP32[$55>>2]|0;
  $57 = ($56|0)<(0);
  $58 = $57 << 31 >> 31;
  _vdbePmaWriteVarint($writer,$56,$58);
  $59 = HEAP32[$p>>2]|0;
  $60 = (($59) + 8|0);
  $61 = HEAP32[$p>>2]|0;
  $62 = HEAP32[$61>>2]|0;
  _vdbePmaWriteBlob($writer,$60,$62);
  $63 = HEAP32[$1>>2]|0;
  $64 = (($63) + 4|0);
  $65 = HEAP32[$64>>2]|0;
  $66 = ($65|0)==(0|0);
  if ($66) {
   $67 = HEAP32[$p>>2]|0;
   _sqlite3_free($67);
  }
  $68 = HEAP32[$pNext>>2]|0;
  HEAP32[$p>>2] = $68;
 }
 $69 = HEAP32[$1>>2]|0;
 HEAP32[$69>>2] = $52;
 $70 = HEAP32[$0>>2]|0;
 $71 = (($70) + 32|0);
 $72 = (($71) + 8|0);
 $73 = (_vdbePmaWriterFinish($writer,$72)|0);
 HEAP32[$rc>>2] = $73;
 $74 = HEAP32[$rc>>2]|0;
 STACKTOP = sp;return ($74|0);
}
function _vdbeSorterOpenTempFile($db,$0,$1,$ppFd) {
 $db = $db|0;
 $0 = $0|0;
 $1 = $1|0;
 $ppFd = $ppFd|0;
 var $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0;
 var $max = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $2 = sp + 24|0;
 $3 = sp + 8|0;
 $4 = sp + 20|0;
 $rc = sp + 16|0;
 $max = sp;
 HEAP32[$2>>2] = $db;
 $5 = $3;
 $6 = $5;
 HEAP32[$6>>2] = $0;
 $7 = (($5) + 4)|0;
 $8 = $7;
 HEAP32[$8>>2] = $1;
 HEAP32[$4>>2] = $ppFd;
 $9 = HEAP32[$2>>2]|0;
 $10 = HEAP32[$9>>2]|0;
 $11 = HEAP32[$4>>2]|0;
 $12 = (_sqlite3OsOpenMalloc($10,0,$11,4126,$rc)|0);
 HEAP32[$rc>>2] = $12;
 $13 = HEAP32[$rc>>2]|0;
 $14 = ($13|0)==(0);
 if (!($14)) {
  $21 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($21|0);
 }
 $15 = $max;
 $16 = $15;
 HEAP32[$16>>2] = 0;
 $17 = (($15) + 4)|0;
 $18 = $17;
 HEAP32[$18>>2] = 0;
 $19 = HEAP32[$4>>2]|0;
 $20 = HEAP32[$19>>2]|0;
 _sqlite3OsFileControlHint($20,18,$max);
 $21 = HEAP32[$rc>>2]|0;
 STACKTOP = sp;return ($21|0);
}
function _vdbeSorterSort($pTask,$pList) {
 $pTask = $pTask|0;
 $pList = $pList|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $8 = 0, $9 = 0, $aSlot = 0, $i = 0, $p = 0, $pNext = 0;
 var $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 28|0;
 $1 = sp + 24|0;
 $2 = sp + 20|0;
 $i = sp + 16|0;
 $aSlot = sp + 12|0;
 $p = sp + 8|0;
 $rc = sp + 4|0;
 $pNext = sp;
 HEAP32[$1>>2] = $pTask;
 HEAP32[$2>>2] = $pList;
 $3 = HEAP32[$1>>2]|0;
 $4 = (_vdbeSortAllocUnpacked($3)|0);
 HEAP32[$rc>>2] = $4;
 $5 = HEAP32[$rc>>2]|0;
 $6 = ($5|0)!=(0);
 if ($6) {
  $7 = HEAP32[$rc>>2]|0;
  HEAP32[$0>>2] = $7;
  $75 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($75|0);
 }
 $8 = (_sqlite3MallocZero(256,0)|0);
 HEAP32[$aSlot>>2] = $8;
 $9 = HEAP32[$aSlot>>2]|0;
 $10 = ($9|0)!=(0|0);
 if (!($10)) {
  HEAP32[$0>>2] = 7;
  $75 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($75|0);
 }
 $11 = HEAP32[$2>>2]|0;
 $12 = HEAP32[$11>>2]|0;
 HEAP32[$p>>2] = $12;
 while(1) {
  $13 = HEAP32[$p>>2]|0;
  $14 = ($13|0)!=(0|0);
  if (!($14)) {
   break;
  }
  $15 = HEAP32[$2>>2]|0;
  $16 = (($15) + 4|0);
  $17 = HEAP32[$16>>2]|0;
  $18 = ($17|0)!=(0|0);
  $19 = HEAP32[$p>>2]|0;
  do {
   if ($18) {
    $20 = HEAP32[$2>>2]|0;
    $21 = (($20) + 4|0);
    $22 = HEAP32[$21>>2]|0;
    $23 = ($19|0)==($22|0);
    if ($23) {
     HEAP32[$pNext>>2] = 0;
     break;
    } else {
     $24 = HEAP32[$p>>2]|0;
     $25 = (($24) + 4|0);
     $26 = HEAP32[$25>>2]|0;
     $27 = HEAP32[$2>>2]|0;
     $28 = (($27) + 4|0);
     $29 = HEAP32[$28>>2]|0;
     $30 = (($29) + ($26)|0);
     HEAP32[$pNext>>2] = $30;
     break;
    }
   } else {
    $31 = (($19) + 4|0);
    $32 = HEAP32[$31>>2]|0;
    HEAP32[$pNext>>2] = $32;
   }
  } while(0);
  $33 = HEAP32[$p>>2]|0;
  $34 = (($33) + 4|0);
  HEAP32[$34>>2] = 0;
  HEAP32[$i>>2] = 0;
  while(1) {
   $35 = HEAP32[$i>>2]|0;
   $36 = HEAP32[$aSlot>>2]|0;
   $37 = (($36) + ($35<<2)|0);
   $38 = HEAP32[$37>>2]|0;
   $39 = ($38|0)!=(0|0);
   if (!($39)) {
    break;
   }
   $40 = HEAP32[$1>>2]|0;
   $41 = HEAP32[$p>>2]|0;
   $42 = HEAP32[$i>>2]|0;
   $43 = HEAP32[$aSlot>>2]|0;
   $44 = (($43) + ($42<<2)|0);
   $45 = HEAP32[$44>>2]|0;
   _vdbeSorterMerge($40,$41,$45,$p);
   $46 = HEAP32[$i>>2]|0;
   $47 = HEAP32[$aSlot>>2]|0;
   $48 = (($47) + ($46<<2)|0);
   HEAP32[$48>>2] = 0;
   $49 = HEAP32[$i>>2]|0;
   $50 = (($49) + 1)|0;
   HEAP32[$i>>2] = $50;
  }
  $51 = HEAP32[$p>>2]|0;
  $52 = HEAP32[$i>>2]|0;
  $53 = HEAP32[$aSlot>>2]|0;
  $54 = (($53) + ($52<<2)|0);
  HEAP32[$54>>2] = $51;
  $55 = HEAP32[$pNext>>2]|0;
  HEAP32[$p>>2] = $55;
 }
 HEAP32[$p>>2] = 0;
 HEAP32[$i>>2] = 0;
 while(1) {
  $56 = HEAP32[$i>>2]|0;
  $57 = ($56|0)<(64);
  if (!($57)) {
   break;
  }
  $58 = HEAP32[$1>>2]|0;
  $59 = HEAP32[$p>>2]|0;
  $60 = HEAP32[$i>>2]|0;
  $61 = HEAP32[$aSlot>>2]|0;
  $62 = (($61) + ($60<<2)|0);
  $63 = HEAP32[$62>>2]|0;
  _vdbeSorterMerge($58,$59,$63,$p);
  $64 = HEAP32[$i>>2]|0;
  $65 = (($64) + 1)|0;
  HEAP32[$i>>2] = $65;
 }
 $66 = HEAP32[$p>>2]|0;
 $67 = HEAP32[$2>>2]|0;
 HEAP32[$67>>2] = $66;
 $68 = HEAP32[$aSlot>>2]|0;
 _sqlite3_free($68);
 $69 = HEAP32[$1>>2]|0;
 $70 = (($69) + 12|0);
 $71 = HEAP32[$70>>2]|0;
 $72 = (($71) + 7|0);
 $73 = HEAP8[$72>>0]|0;
 $74 = $73&255;
 HEAP32[$0>>2] = $74;
 $75 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($75|0);
}
function _vdbePmaWriterInit($pFd,$p,$nBuf,$0,$1) {
 $pFd = $pFd|0;
 $p = $p|0;
 $nBuf = $nBuf|0;
 $0 = $0|0;
 $1 = $1|0;
 var $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0;
 var $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0;
 var $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $7 = 0, $8 = 0, $9 = 0;
 var dest = 0, label = 0, sp = 0, stop = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $2 = sp + 16|0;
 $3 = sp + 12|0;
 $4 = sp + 8|0;
 $5 = sp;
 HEAP32[$2>>2] = $pFd;
 HEAP32[$3>>2] = $p;
 HEAP32[$4>>2] = $nBuf;
 $6 = $5;
 $7 = $6;
 HEAP32[$7>>2] = $0;
 $8 = (($6) + 4)|0;
 $9 = $8;
 HEAP32[$9>>2] = $1;
 $10 = HEAP32[$3>>2]|0;
 dest=$10+0|0; stop=dest+40|0; do { HEAP32[dest>>2]=0|0; dest=dest+4|0; } while ((dest|0) < (stop|0));
 $11 = HEAP32[$4>>2]|0;
 $12 = ($11|0)<(0);
 $13 = $12 << 31 >> 31;
 $14 = (_sqlite3Malloc($11,$13)|0);
 $15 = HEAP32[$3>>2]|0;
 $16 = (($15) + 4|0);
 HEAP32[$16>>2] = $14;
 $17 = HEAP32[$3>>2]|0;
 $18 = (($17) + 4|0);
 $19 = HEAP32[$18>>2]|0;
 $20 = ($19|0)!=(0|0);
 if ($20) {
  $22 = $5;
  $23 = $22;
  $24 = HEAP32[$23>>2]|0;
  $25 = (($22) + 4)|0;
  $26 = $25;
  $27 = HEAP32[$26>>2]|0;
  $28 = HEAP32[$4>>2]|0;
  $29 = ($28|0)<(0);
  $30 = $29 << 31 >> 31;
  $31 = (___remdi3(($24|0),($27|0),($28|0),($30|0))|0);
  $32 = tempRet0;
  $33 = HEAP32[$3>>2]|0;
  $34 = (($33) + 12|0);
  HEAP32[$34>>2] = $31;
  $35 = HEAP32[$3>>2]|0;
  $36 = (($35) + 16|0);
  HEAP32[$36>>2] = $31;
  $37 = $5;
  $38 = $37;
  $39 = HEAP32[$38>>2]|0;
  $40 = (($37) + 4)|0;
  $41 = $40;
  $42 = HEAP32[$41>>2]|0;
  $43 = HEAP32[$3>>2]|0;
  $44 = (($43) + 12|0);
  $45 = HEAP32[$44>>2]|0;
  $46 = ($45|0)<(0);
  $47 = $46 << 31 >> 31;
  $48 = (_i64Subtract(($39|0),($42|0),($45|0),($47|0))|0);
  $49 = tempRet0;
  $50 = HEAP32[$3>>2]|0;
  $51 = (($50) + 24|0);
  $52 = $51;
  $53 = $52;
  HEAP32[$53>>2] = $48;
  $54 = (($52) + 4)|0;
  $55 = $54;
  HEAP32[$55>>2] = $49;
  $56 = HEAP32[$4>>2]|0;
  $57 = HEAP32[$3>>2]|0;
  $58 = (($57) + 8|0);
  HEAP32[$58>>2] = $56;
  $59 = HEAP32[$2>>2]|0;
  $60 = HEAP32[$3>>2]|0;
  $61 = (($60) + 32|0);
  HEAP32[$61>>2] = $59;
  STACKTOP = sp;return;
 } else {
  $21 = HEAP32[$3>>2]|0;
  HEAP32[$21>>2] = 7;
  STACKTOP = sp;return;
 }
}
function _vdbePmaWriteVarint($p,$0,$1) {
 $p = $p|0;
 $0 = $0|0;
 $1 = $1|0;
 var $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $aByte = 0, $nByte = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $2 = sp + 12|0;
 $3 = sp;
 $nByte = sp + 8|0;
 $aByte = sp + 16|0;
 HEAP32[$2>>2] = $p;
 $4 = $3;
 $5 = $4;
 HEAP32[$5>>2] = $0;
 $6 = (($4) + 4)|0;
 $7 = $6;
 HEAP32[$7>>2] = $1;
 $8 = $3;
 $9 = $8;
 $10 = HEAP32[$9>>2]|0;
 $11 = (($8) + 4)|0;
 $12 = $11;
 $13 = HEAP32[$12>>2]|0;
 $14 = (_sqlite3PutVarint($aByte,$10,$13)|0);
 HEAP32[$nByte>>2] = $14;
 $15 = HEAP32[$2>>2]|0;
 $16 = HEAP32[$nByte>>2]|0;
 _vdbePmaWriteBlob($15,$aByte,$16);
 STACKTOP = sp;return;
}
function _vdbePmaWriteBlob($p,$pData,$nData) {
 $p = $p|0;
 $pData = $pData|0;
 $nData = $nData|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0;
 var $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0;
 var $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0;
 var $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0;
 var $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0;
 var $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $nCopy = 0, $nRem = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 16|0;
 $1 = sp + 12|0;
 $2 = sp + 8|0;
 $nRem = sp + 4|0;
 $nCopy = sp;
 HEAP32[$0>>2] = $p;
 HEAP32[$1>>2] = $pData;
 HEAP32[$2>>2] = $nData;
 $3 = HEAP32[$2>>2]|0;
 HEAP32[$nRem>>2] = $3;
 while(1) {
  $4 = HEAP32[$nRem>>2]|0;
  $5 = ($4|0)>(0);
  if (!($5)) {
   label = 9;
   break;
  }
  $6 = HEAP32[$0>>2]|0;
  $7 = HEAP32[$6>>2]|0;
  $8 = ($7|0)==(0);
  if (!($8)) {
   label = 9;
   break;
  }
  $9 = HEAP32[$nRem>>2]|0;
  HEAP32[$nCopy>>2] = $9;
  $10 = HEAP32[$nCopy>>2]|0;
  $11 = HEAP32[$0>>2]|0;
  $12 = (($11) + 8|0);
  $13 = HEAP32[$12>>2]|0;
  $14 = HEAP32[$0>>2]|0;
  $15 = (($14) + 16|0);
  $16 = HEAP32[$15>>2]|0;
  $17 = (($13) - ($16))|0;
  $18 = ($10|0)>($17|0);
  if ($18) {
   $19 = HEAP32[$0>>2]|0;
   $20 = (($19) + 8|0);
   $21 = HEAP32[$20>>2]|0;
   $22 = HEAP32[$0>>2]|0;
   $23 = (($22) + 16|0);
   $24 = HEAP32[$23>>2]|0;
   $25 = (($21) - ($24))|0;
   HEAP32[$nCopy>>2] = $25;
  }
  $26 = HEAP32[$0>>2]|0;
  $27 = (($26) + 16|0);
  $28 = HEAP32[$27>>2]|0;
  $29 = HEAP32[$0>>2]|0;
  $30 = (($29) + 4|0);
  $31 = HEAP32[$30>>2]|0;
  $32 = (($31) + ($28)|0);
  $33 = HEAP32[$2>>2]|0;
  $34 = HEAP32[$nRem>>2]|0;
  $35 = (($33) - ($34))|0;
  $36 = HEAP32[$1>>2]|0;
  $37 = (($36) + ($35)|0);
  $38 = HEAP32[$nCopy>>2]|0;
  _memcpy(($32|0),($37|0),($38|0))|0;
  $39 = HEAP32[$nCopy>>2]|0;
  $40 = HEAP32[$0>>2]|0;
  $41 = (($40) + 16|0);
  $42 = HEAP32[$41>>2]|0;
  $43 = (($42) + ($39))|0;
  HEAP32[$41>>2] = $43;
  $44 = HEAP32[$0>>2]|0;
  $45 = (($44) + 16|0);
  $46 = HEAP32[$45>>2]|0;
  $47 = HEAP32[$0>>2]|0;
  $48 = (($47) + 8|0);
  $49 = HEAP32[$48>>2]|0;
  $50 = ($46|0)==($49|0);
  if ($50) {
   $51 = HEAP32[$0>>2]|0;
   $52 = (($51) + 32|0);
   $53 = HEAP32[$52>>2]|0;
   $54 = HEAP32[$0>>2]|0;
   $55 = (($54) + 12|0);
   $56 = HEAP32[$55>>2]|0;
   $57 = HEAP32[$0>>2]|0;
   $58 = (($57) + 4|0);
   $59 = HEAP32[$58>>2]|0;
   $60 = (($59) + ($56)|0);
   $61 = HEAP32[$0>>2]|0;
   $62 = (($61) + 16|0);
   $63 = HEAP32[$62>>2]|0;
   $64 = HEAP32[$0>>2]|0;
   $65 = (($64) + 12|0);
   $66 = HEAP32[$65>>2]|0;
   $67 = (($63) - ($66))|0;
   $68 = HEAP32[$0>>2]|0;
   $69 = (($68) + 24|0);
   $70 = $69;
   $71 = $70;
   $72 = HEAP32[$71>>2]|0;
   $73 = (($70) + 4)|0;
   $74 = $73;
   $75 = HEAP32[$74>>2]|0;
   $76 = HEAP32[$0>>2]|0;
   $77 = (($76) + 12|0);
   $78 = HEAP32[$77>>2]|0;
   $79 = ($78|0)<(0);
   $80 = $79 << 31 >> 31;
   $81 = (_i64Add(($72|0),($75|0),($78|0),($80|0))|0);
   $82 = tempRet0;
   $83 = (_sqlite3OsWrite($53,$60,$67,$81,$82)|0);
   $84 = HEAP32[$0>>2]|0;
   HEAP32[$84>>2] = $83;
   $85 = HEAP32[$0>>2]|0;
   $86 = (($85) + 16|0);
   HEAP32[$86>>2] = 0;
   $87 = HEAP32[$0>>2]|0;
   $88 = (($87) + 12|0);
   HEAP32[$88>>2] = 0;
   $89 = HEAP32[$0>>2]|0;
   $90 = (($89) + 8|0);
   $91 = HEAP32[$90>>2]|0;
   $92 = ($91|0)<(0);
   $93 = $92 << 31 >> 31;
   $94 = HEAP32[$0>>2]|0;
   $95 = (($94) + 24|0);
   $96 = $95;
   $97 = $96;
   $98 = HEAP32[$97>>2]|0;
   $99 = (($96) + 4)|0;
   $100 = $99;
   $101 = HEAP32[$100>>2]|0;
   $102 = (_i64Add(($98|0),($101|0),($91|0),($93|0))|0);
   $103 = tempRet0;
   $104 = $95;
   $105 = $104;
   HEAP32[$105>>2] = $102;
   $106 = (($104) + 4)|0;
   $107 = $106;
   HEAP32[$107>>2] = $103;
  }
  $108 = HEAP32[$nCopy>>2]|0;
  $109 = HEAP32[$nRem>>2]|0;
  $110 = (($109) - ($108))|0;
  HEAP32[$nRem>>2] = $110;
 }
 if ((label|0) == 9) {
  STACKTOP = sp;return;
 }
}
function _vdbePmaWriterFinish($p,$piEof) {
 $p = $p|0;
 $piEof = $piEof|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $8 = 0, $9 = 0, $rc = 0, dest = 0, label = 0;
 var sp = 0, stop = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $1 = sp + 4|0;
 $rc = sp;
 HEAP32[$0>>2] = $p;
 HEAP32[$1>>2] = $piEof;
 $2 = HEAP32[$0>>2]|0;
 $3 = HEAP32[$2>>2]|0;
 $4 = ($3|0)==(0);
 if ($4) {
  $5 = HEAP32[$0>>2]|0;
  $6 = (($5) + 4|0);
  $7 = HEAP32[$6>>2]|0;
  $8 = ($7|0)!=(0|0);
  if ($8) {
   $9 = HEAP32[$0>>2]|0;
   $10 = (($9) + 16|0);
   $11 = HEAP32[$10>>2]|0;
   $12 = HEAP32[$0>>2]|0;
   $13 = (($12) + 12|0);
   $14 = HEAP32[$13>>2]|0;
   $15 = ($11|0)>($14|0);
   if ($15) {
    $16 = HEAP32[$0>>2]|0;
    $17 = (($16) + 32|0);
    $18 = HEAP32[$17>>2]|0;
    $19 = HEAP32[$0>>2]|0;
    $20 = (($19) + 12|0);
    $21 = HEAP32[$20>>2]|0;
    $22 = HEAP32[$0>>2]|0;
    $23 = (($22) + 4|0);
    $24 = HEAP32[$23>>2]|0;
    $25 = (($24) + ($21)|0);
    $26 = HEAP32[$0>>2]|0;
    $27 = (($26) + 16|0);
    $28 = HEAP32[$27>>2]|0;
    $29 = HEAP32[$0>>2]|0;
    $30 = (($29) + 12|0);
    $31 = HEAP32[$30>>2]|0;
    $32 = (($28) - ($31))|0;
    $33 = HEAP32[$0>>2]|0;
    $34 = (($33) + 24|0);
    $35 = $34;
    $36 = $35;
    $37 = HEAP32[$36>>2]|0;
    $38 = (($35) + 4)|0;
    $39 = $38;
    $40 = HEAP32[$39>>2]|0;
    $41 = HEAP32[$0>>2]|0;
    $42 = (($41) + 12|0);
    $43 = HEAP32[$42>>2]|0;
    $44 = ($43|0)<(0);
    $45 = $44 << 31 >> 31;
    $46 = (_i64Add(($37|0),($40|0),($43|0),($45|0))|0);
    $47 = tempRet0;
    $48 = (_sqlite3OsWrite($18,$25,$32,$46,$47)|0);
    $49 = HEAP32[$0>>2]|0;
    HEAP32[$49>>2] = $48;
   }
  }
 }
 $50 = HEAP32[$0>>2]|0;
 $51 = (($50) + 24|0);
 $52 = $51;
 $53 = $52;
 $54 = HEAP32[$53>>2]|0;
 $55 = (($52) + 4)|0;
 $56 = $55;
 $57 = HEAP32[$56>>2]|0;
 $58 = HEAP32[$0>>2]|0;
 $59 = (($58) + 16|0);
 $60 = HEAP32[$59>>2]|0;
 $61 = ($60|0)<(0);
 $62 = $61 << 31 >> 31;
 $63 = (_i64Add(($54|0),($57|0),($60|0),($62|0))|0);
 $64 = tempRet0;
 $65 = HEAP32[$1>>2]|0;
 $66 = $65;
 $67 = $66;
 HEAP32[$67>>2] = $63;
 $68 = (($66) + 4)|0;
 $69 = $68;
 HEAP32[$69>>2] = $64;
 $70 = HEAP32[$0>>2]|0;
 $71 = (($70) + 4|0);
 $72 = HEAP32[$71>>2]|0;
 _sqlite3_free($72);
 $73 = HEAP32[$0>>2]|0;
 $74 = HEAP32[$73>>2]|0;
 HEAP32[$rc>>2] = $74;
 $75 = HEAP32[$0>>2]|0;
 dest=$75+0|0; stop=dest+40|0; do { HEAP32[dest>>2]=0|0; dest=dest+4|0; } while ((dest|0) < (stop|0));
 $76 = HEAP32[$rc>>2]|0;
 STACKTOP = sp;return ($76|0);
}
function _vdbeSortAllocUnpacked($pTask) {
 $pTask = $pTask|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $pFree = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $1 = sp + 4|0;
 $pFree = sp;
 HEAP32[$1>>2] = $pTask;
 $2 = HEAP32[$1>>2]|0;
 $3 = (($2) + 12|0);
 $4 = HEAP32[$3>>2]|0;
 $5 = ($4|0)==(0|0);
 do {
  if ($5) {
   $6 = HEAP32[$1>>2]|0;
   $7 = (($6) + 8|0);
   $8 = HEAP32[$7>>2]|0;
   $9 = (($8) + 28|0);
   $10 = HEAP32[$9>>2]|0;
   $11 = (_sqlite3VdbeAllocUnpackedRecord($10,0,0,$pFree)|0);
   $12 = HEAP32[$1>>2]|0;
   $13 = (($12) + 12|0);
   HEAP32[$13>>2] = $11;
   $14 = HEAP32[$pFree>>2]|0;
   $15 = ($14|0)==(0|0);
   if (!($15)) {
    $16 = HEAP32[$1>>2]|0;
    $17 = (($16) + 8|0);
    $18 = HEAP32[$17>>2]|0;
    $19 = (($18) + 28|0);
    $20 = HEAP32[$19>>2]|0;
    $21 = (($20) + 6|0);
    $22 = HEAP16[$21>>1]|0;
    $23 = HEAP32[$1>>2]|0;
    $24 = (($23) + 12|0);
    $25 = HEAP32[$24>>2]|0;
    $26 = (($25) + 4|0);
    HEAP16[$26>>1] = $22;
    $27 = HEAP32[$1>>2]|0;
    $28 = (($27) + 12|0);
    $29 = HEAP32[$28>>2]|0;
    $30 = (($29) + 7|0);
    HEAP8[$30>>0] = 0;
    break;
   }
   HEAP32[$0>>2] = 7;
   $31 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($31|0);
  }
 } while(0);
 HEAP32[$0>>2] = 0;
 $31 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($31|0);
}
function _vdbeSorterMerge($pTask,$p1,$p2,$ppOut) {
 $pTask = $pTask|0;
 $p1 = $p1|0;
 $p2 = $p2|0;
 $ppOut = $ppOut|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $or$cond = 0, $pFinal = 0, $pVal2 = 0, $pp = 0, $res = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 28|0;
 $1 = sp + 24|0;
 $2 = sp + 20|0;
 $3 = sp + 16|0;
 $pFinal = sp + 12|0;
 $pp = sp + 8|0;
 $pVal2 = sp + 4|0;
 $res = sp;
 HEAP32[$0>>2] = $pTask;
 HEAP32[$1>>2] = $p1;
 HEAP32[$2>>2] = $p2;
 HEAP32[$3>>2] = $ppOut;
 HEAP32[$pFinal>>2] = 0;
 HEAP32[$pp>>2] = $pFinal;
 $4 = HEAP32[$2>>2]|0;
 $5 = ($4|0)!=(0|0);
 if ($5) {
  $6 = HEAP32[$2>>2]|0;
  $7 = (($6) + 8|0);
  $8 = $7;
 } else {
  $8 = 0;
 }
 HEAP32[$pVal2>>2] = $8;
 while(1) {
  $9 = HEAP32[$1>>2]|0;
  $10 = ($9|0)!=(0|0);
  $11 = HEAP32[$2>>2]|0;
  $12 = ($11|0)!=(0|0);
  $or$cond = $10 & $12;
  if (!($or$cond)) {
   label = 9;
   break;
  }
  $13 = HEAP32[$0>>2]|0;
  $14 = HEAP32[$1>>2]|0;
  $15 = (($14) + 8|0);
  $16 = HEAP32[$1>>2]|0;
  $17 = HEAP32[$16>>2]|0;
  $18 = HEAP32[$pVal2>>2]|0;
  $19 = HEAP32[$2>>2]|0;
  $20 = HEAP32[$19>>2]|0;
  $21 = (_vdbeSorterCompare($13,$15,$17,$18,$20)|0);
  HEAP32[$res>>2] = $21;
  $22 = HEAP32[$res>>2]|0;
  $23 = ($22|0)<=(0);
  if ($23) {
   $24 = HEAP32[$1>>2]|0;
   $25 = HEAP32[$pp>>2]|0;
   HEAP32[$25>>2] = $24;
   $26 = HEAP32[$1>>2]|0;
   $27 = (($26) + 4|0);
   HEAP32[$pp>>2] = $27;
   $28 = HEAP32[$1>>2]|0;
   $29 = (($28) + 4|0);
   $30 = HEAP32[$29>>2]|0;
   HEAP32[$1>>2] = $30;
   HEAP32[$pVal2>>2] = 0;
   continue;
  }
  $31 = HEAP32[$2>>2]|0;
  $32 = HEAP32[$pp>>2]|0;
  HEAP32[$32>>2] = $31;
  $33 = HEAP32[$2>>2]|0;
  $34 = (($33) + 4|0);
  HEAP32[$pp>>2] = $34;
  $35 = HEAP32[$2>>2]|0;
  $36 = (($35) + 4|0);
  $37 = HEAP32[$36>>2]|0;
  HEAP32[$2>>2] = $37;
  $38 = HEAP32[$2>>2]|0;
  $39 = ($38|0)==(0|0);
  if ($39) {
   label = 9;
   break;
  }
  $40 = HEAP32[$2>>2]|0;
  $41 = (($40) + 8|0);
  HEAP32[$pVal2>>2] = $41;
 }
 if ((label|0) == 9) {
  $42 = HEAP32[$1>>2]|0;
  $43 = ($42|0)!=(0|0);
  $44 = HEAP32[$1>>2]|0;
  $45 = HEAP32[$2>>2]|0;
  $46 = $43 ? $44 : $45;
  $47 = HEAP32[$pp>>2]|0;
  HEAP32[$47>>2] = $46;
  $48 = HEAP32[$pFinal>>2]|0;
  $49 = HEAP32[$3>>2]|0;
  HEAP32[$49>>2] = $48;
  STACKTOP = sp;return;
 }
}
function _vdbeSorterCompare($pTask,$pKey1,$nKey1,$pKey2,$nKey2) {
 $pTask = $pTask|0;
 $pKey1 = $pKey1|0;
 $nKey1 = $nKey1|0;
 $pKey2 = $pKey2|0;
 $nKey2 = $nKey2|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0;
 var $8 = 0, $9 = 0, $r2 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 20|0;
 $1 = sp + 16|0;
 $2 = sp + 12|0;
 $3 = sp + 8|0;
 $4 = sp + 4|0;
 $r2 = sp;
 HEAP32[$0>>2] = $pTask;
 HEAP32[$1>>2] = $pKey1;
 HEAP32[$2>>2] = $nKey1;
 HEAP32[$3>>2] = $pKey2;
 HEAP32[$4>>2] = $nKey2;
 $5 = HEAP32[$0>>2]|0;
 $6 = (($5) + 12|0);
 $7 = HEAP32[$6>>2]|0;
 HEAP32[$r2>>2] = $7;
 $8 = HEAP32[$3>>2]|0;
 $9 = ($8|0)!=(0|0);
 if ($9) {
  $10 = HEAP32[$0>>2]|0;
  $11 = (($10) + 8|0);
  $12 = HEAP32[$11>>2]|0;
  $13 = (($12) + 28|0);
  $14 = HEAP32[$13>>2]|0;
  $15 = HEAP32[$4>>2]|0;
  $16 = HEAP32[$3>>2]|0;
  $17 = HEAP32[$r2>>2]|0;
  _sqlite3VdbeRecordUnpack($14,$15,$16,$17);
 }
 $18 = HEAP32[$2>>2]|0;
 $19 = HEAP32[$1>>2]|0;
 $20 = HEAP32[$r2>>2]|0;
 $21 = (_sqlite3VdbeRecordCompare($18,$19,$20)|0);
 STACKTOP = sp;return ($21|0);
}
function _sqlite3OsOpenMalloc($pVfs,$zFile,$ppFile,$flags,$pOutFlags) {
 $pVfs = $pVfs|0;
 $zFile = $zFile|0;
 $ppFile = $ppFile|0;
 $flags = $flags|0;
 $pOutFlags = $pOutFlags|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $3 = 0, $4 = 0, $5 = 0;
 var $6 = 0, $7 = 0, $8 = 0, $9 = 0, $pFile = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 24|0;
 $1 = sp + 20|0;
 $2 = sp + 16|0;
 $3 = sp + 12|0;
 $4 = sp + 8|0;
 $rc = sp + 4|0;
 $pFile = sp;
 HEAP32[$0>>2] = $pVfs;
 HEAP32[$1>>2] = $zFile;
 HEAP32[$2>>2] = $ppFile;
 HEAP32[$3>>2] = $flags;
 HEAP32[$4>>2] = $pOutFlags;
 HEAP32[$rc>>2] = 7;
 $5 = HEAP32[$0>>2]|0;
 $6 = (($5) + 4|0);
 $7 = HEAP32[$6>>2]|0;
 $8 = ($7|0)<(0);
 $9 = $8 << 31 >> 31;
 $10 = (_sqlite3MallocZero($7,$9)|0);
 HEAP32[$pFile>>2] = $10;
 $11 = HEAP32[$pFile>>2]|0;
 $12 = ($11|0)!=(0|0);
 if (!($12)) {
  $23 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($23|0);
 }
 $13 = HEAP32[$0>>2]|0;
 $14 = HEAP32[$1>>2]|0;
 $15 = HEAP32[$pFile>>2]|0;
 $16 = HEAP32[$3>>2]|0;
 $17 = HEAP32[$4>>2]|0;
 $18 = (_sqlite3OsOpen($13,$14,$15,$16,$17)|0);
 HEAP32[$rc>>2] = $18;
 $19 = HEAP32[$rc>>2]|0;
 $20 = ($19|0)!=(0);
 $21 = HEAP32[$pFile>>2]|0;
 if ($20) {
  _sqlite3_free($21);
  $23 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($23|0);
 } else {
  $22 = HEAP32[$2>>2]|0;
  HEAP32[$22>>2] = $21;
  $23 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($23|0);
 }
 return 0|0;
}
function _vdbeMergeEngineStep($pMerger,$pbEof) {
 $pMerger = $pMerger|0;
 $pbEof = $pbEof|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0;
 var $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0;
 var $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0;
 var $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0;
 var $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0;
 var $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $i = 0, $iPrev = 0, $iRes = 0, $pKey2 = 0, $pReadr1 = 0, $pReadr2 = 0, $pTask = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $0 = sp + 36|0;
 $1 = sp + 32|0;
 $rc = sp + 28|0;
 $iPrev = sp + 24|0;
 $pTask = sp + 20|0;
 $i = sp + 16|0;
 $pReadr1 = sp + 12|0;
 $pReadr2 = sp + 8|0;
 $pKey2 = sp + 4|0;
 $iRes = sp;
 HEAP32[$0>>2] = $pMerger;
 HEAP32[$1>>2] = $pbEof;
 $2 = HEAP32[$0>>2]|0;
 $3 = (($2) + 8|0);
 $4 = HEAP32[$3>>2]|0;
 $5 = (($4) + 4|0);
 $6 = HEAP32[$5>>2]|0;
 HEAP32[$iPrev>>2] = $6;
 $7 = HEAP32[$0>>2]|0;
 $8 = (($7) + 4|0);
 $9 = HEAP32[$8>>2]|0;
 HEAP32[$pTask>>2] = $9;
 $10 = HEAP32[$iPrev>>2]|0;
 $11 = HEAP32[$0>>2]|0;
 $12 = (($11) + 12|0);
 $13 = HEAP32[$12>>2]|0;
 $14 = (($13) + (($10*56)|0)|0);
 $15 = (_vdbePmaReaderNext($14)|0);
 HEAP32[$rc>>2] = $15;
 $16 = HEAP32[$rc>>2]|0;
 $17 = ($16|0)==(0);
 if ($17) {
  $18 = HEAP32[$iPrev>>2]|0;
  $19 = $18 & 65534;
  $20 = HEAP32[$0>>2]|0;
  $21 = (($20) + 12|0);
  $22 = HEAP32[$21>>2]|0;
  $23 = (($22) + (($19*56)|0)|0);
  HEAP32[$pReadr1>>2] = $23;
  $24 = HEAP32[$iPrev>>2]|0;
  $25 = $24 | 1;
  $26 = HEAP32[$0>>2]|0;
  $27 = (($26) + 12|0);
  $28 = HEAP32[$27>>2]|0;
  $29 = (($28) + (($25*56)|0)|0);
  HEAP32[$pReadr2>>2] = $29;
  $30 = HEAP32[$pReadr2>>2]|0;
  $31 = (($30) + 32|0);
  $32 = HEAP32[$31>>2]|0;
  HEAP32[$pKey2>>2] = $32;
  $33 = HEAP32[$0>>2]|0;
  $34 = HEAP32[$33>>2]|0;
  $35 = HEAP32[$iPrev>>2]|0;
  $36 = (($34) + ($35))|0;
  $37 = (($36|0) / 2)&-1;
  HEAP32[$i>>2] = $37;
  while(1) {
   $38 = HEAP32[$i>>2]|0;
   $39 = ($38|0)>(0);
   if (!($39)) {
    break;
   }
   $40 = HEAP32[$pReadr1>>2]|0;
   $41 = (($40) + 24|0);
   $42 = HEAP32[$41>>2]|0;
   $43 = ($42|0)==(0|0);
   do {
    if ($43) {
     HEAP32[$iRes>>2] = 1;
    } else {
     $44 = HEAP32[$pReadr2>>2]|0;
     $45 = (($44) + 24|0);
     $46 = HEAP32[$45>>2]|0;
     $47 = ($46|0)==(0|0);
     if ($47) {
      HEAP32[$iRes>>2] = -1;
      break;
     } else {
      $48 = HEAP32[$pTask>>2]|0;
      $49 = HEAP32[$pReadr1>>2]|0;
      $50 = (($49) + 32|0);
      $51 = HEAP32[$50>>2]|0;
      $52 = HEAP32[$pReadr1>>2]|0;
      $53 = (($52) + 20|0);
      $54 = HEAP32[$53>>2]|0;
      $55 = HEAP32[$pKey2>>2]|0;
      $56 = HEAP32[$pReadr2>>2]|0;
      $57 = (($56) + 20|0);
      $58 = HEAP32[$57>>2]|0;
      $59 = (_vdbeSorterCompare($48,$51,$54,$55,$58)|0);
      HEAP32[$iRes>>2] = $59;
      break;
     }
    }
   } while(0);
   $60 = HEAP32[$iRes>>2]|0;
   $61 = ($60|0)<(0);
   do {
    if ($61) {
     label = 12;
    } else {
     $62 = HEAP32[$iRes>>2]|0;
     $63 = ($62|0)==(0);
     if ($63) {
      $64 = HEAP32[$pReadr1>>2]|0;
      $65 = HEAP32[$pReadr2>>2]|0;
      $66 = ($64>>>0)<($65>>>0);
      if ($66) {
       label = 12;
       break;
      }
     }
     $94 = HEAP32[$pReadr1>>2]|0;
     $95 = (($94) + 24|0);
     $96 = HEAP32[$95>>2]|0;
     $97 = ($96|0)!=(0|0);
     if ($97) {
      HEAP32[$pKey2>>2] = 0;
     }
     $98 = HEAP32[$pReadr2>>2]|0;
     $99 = HEAP32[$0>>2]|0;
     $100 = (($99) + 12|0);
     $101 = HEAP32[$100>>2]|0;
     $102 = $98;
     $103 = $101;
     $104 = (($102) - ($103))|0;
     $105 = (($104|0) / 56)&-1;
     $106 = HEAP32[$i>>2]|0;
     $107 = HEAP32[$0>>2]|0;
     $108 = (($107) + 8|0);
     $109 = HEAP32[$108>>2]|0;
     $110 = (($109) + ($106<<2)|0);
     HEAP32[$110>>2] = $105;
     $111 = HEAP32[$i>>2]|0;
     $112 = $111 ^ 1;
     $113 = HEAP32[$0>>2]|0;
     $114 = (($113) + 8|0);
     $115 = HEAP32[$114>>2]|0;
     $116 = (($115) + ($112<<2)|0);
     $117 = HEAP32[$116>>2]|0;
     $118 = HEAP32[$0>>2]|0;
     $119 = (($118) + 12|0);
     $120 = HEAP32[$119>>2]|0;
     $121 = (($120) + (($117*56)|0)|0);
     HEAP32[$pReadr1>>2] = $121;
    }
   } while(0);
   if ((label|0) == 12) {
    label = 0;
    $67 = HEAP32[$pReadr1>>2]|0;
    $68 = HEAP32[$0>>2]|0;
    $69 = (($68) + 12|0);
    $70 = HEAP32[$69>>2]|0;
    $71 = $67;
    $72 = $70;
    $73 = (($71) - ($72))|0;
    $74 = (($73|0) / 56)&-1;
    $75 = HEAP32[$i>>2]|0;
    $76 = HEAP32[$0>>2]|0;
    $77 = (($76) + 8|0);
    $78 = HEAP32[$77>>2]|0;
    $79 = (($78) + ($75<<2)|0);
    HEAP32[$79>>2] = $74;
    $80 = HEAP32[$i>>2]|0;
    $81 = $80 ^ 1;
    $82 = HEAP32[$0>>2]|0;
    $83 = (($82) + 8|0);
    $84 = HEAP32[$83>>2]|0;
    $85 = (($84) + ($81<<2)|0);
    $86 = HEAP32[$85>>2]|0;
    $87 = HEAP32[$0>>2]|0;
    $88 = (($87) + 12|0);
    $89 = HEAP32[$88>>2]|0;
    $90 = (($89) + (($86*56)|0)|0);
    HEAP32[$pReadr2>>2] = $90;
    $91 = HEAP32[$pReadr2>>2]|0;
    $92 = (($91) + 32|0);
    $93 = HEAP32[$92>>2]|0;
    HEAP32[$pKey2>>2] = $93;
   }
   $122 = HEAP32[$i>>2]|0;
   $123 = (($122|0) / 2)&-1;
   HEAP32[$i>>2] = $123;
  }
  $124 = HEAP32[$0>>2]|0;
  $125 = (($124) + 8|0);
  $126 = HEAP32[$125>>2]|0;
  $127 = (($126) + 4|0);
  $128 = HEAP32[$127>>2]|0;
  $129 = HEAP32[$0>>2]|0;
  $130 = (($129) + 12|0);
  $131 = HEAP32[$130>>2]|0;
  $132 = (($131) + (($128*56)|0)|0);
  $133 = (($132) + 24|0);
  $134 = HEAP32[$133>>2]|0;
  $135 = ($134|0)==(0|0);
  $136 = $135&1;
  $137 = HEAP32[$1>>2]|0;
  HEAP32[$137>>2] = $136;
 }
 $138 = HEAP32[$rc>>2]|0;
 $139 = ($138|0)==(0);
 if ($139) {
  $140 = HEAP32[$pTask>>2]|0;
  $141 = (($140) + 12|0);
  $142 = HEAP32[$141>>2]|0;
  $143 = (($142) + 7|0);
  $144 = HEAP8[$143>>0]|0;
  $145 = $144&255;
  $147 = $145;
  STACKTOP = sp;return ($147|0);
 } else {
  $146 = HEAP32[$rc>>2]|0;
  $147 = $146;
  STACKTOP = sp;return ($147|0);
 }
 return 0|0;
}
function _vdbePmaReaderNext($pReadr) {
 $pReadr = $pReadr|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0;
 var $81 = 0, $82 = 0, $9 = 0, $bEof = 0, $nRec = 0, $pIncr = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 24|0;
 $1 = sp + 20|0;
 $rc = sp + 16|0;
 $nRec = sp;
 $pIncr = sp + 12|0;
 $bEof = sp + 8|0;
 HEAP32[$1>>2] = $pReadr;
 HEAP32[$rc>>2] = 0;
 $2 = $nRec;
 $3 = $2;
 HEAP32[$3>>2] = 0;
 $4 = (($2) + 4)|0;
 $5 = $4;
 HEAP32[$5>>2] = 0;
 $6 = HEAP32[$1>>2]|0;
 $7 = $6;
 $8 = $7;
 $9 = HEAP32[$8>>2]|0;
 $10 = (($7) + 4)|0;
 $11 = $10;
 $12 = HEAP32[$11>>2]|0;
 $13 = HEAP32[$1>>2]|0;
 $14 = (($13) + 8|0);
 $15 = $14;
 $16 = $15;
 $17 = HEAP32[$16>>2]|0;
 $18 = (($15) + 4)|0;
 $19 = $18;
 $20 = HEAP32[$19>>2]|0;
 $21 = ($12|0)>($20|0);
 $22 = ($9>>>0)>=($17>>>0);
 $23 = ($12|0)==($20|0);
 $24 = $23 & $22;
 $25 = $21 | $24;
 if ($25) {
  $26 = HEAP32[$1>>2]|0;
  $27 = (($26) + 48|0);
  $28 = HEAP32[$27>>2]|0;
  HEAP32[$pIncr>>2] = $28;
  HEAP32[$bEof>>2] = 1;
  $29 = HEAP32[$pIncr>>2]|0;
  $30 = ($29|0)!=(0|0);
  if ($30) {
   $31 = HEAP32[$pIncr>>2]|0;
   $32 = (_vdbeIncrSwap($31)|0);
   HEAP32[$rc>>2] = $32;
   $33 = HEAP32[$rc>>2]|0;
   $34 = ($33|0)==(0);
   if ($34) {
    $35 = HEAP32[$pIncr>>2]|0;
    $36 = (($35) + 20|0);
    $37 = HEAP32[$36>>2]|0;
    $38 = ($37|0)==(0);
    if ($38) {
     $39 = HEAP32[$pIncr>>2]|0;
     $40 = HEAP32[$39>>2]|0;
     $41 = HEAP32[$1>>2]|0;
     $42 = HEAP32[$pIncr>>2]|0;
     $43 = (($42) + 32|0);
     $44 = HEAP32[$pIncr>>2]|0;
     $45 = (($44) + 8|0);
     $46 = $45;
     $47 = $46;
     $48 = HEAP32[$47>>2]|0;
     $49 = (($46) + 4)|0;
     $50 = $49;
     $51 = HEAP32[$50>>2]|0;
     $52 = (_vdbePmaReaderSeek($40,$41,$43,$48,$51)|0);
     HEAP32[$rc>>2] = $52;
     HEAP32[$bEof>>2] = 0;
    }
   }
  }
  $53 = HEAP32[$bEof>>2]|0;
  $54 = ($53|0)!=(0);
  if ($54) {
   $55 = HEAP32[$1>>2]|0;
   _vdbePmaReaderClear($55);
   $56 = HEAP32[$rc>>2]|0;
   HEAP32[$0>>2] = $56;
   $82 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($82|0);
  }
 }
 $57 = HEAP32[$rc>>2]|0;
 $58 = ($57|0)==(0);
 if ($58) {
  $59 = HEAP32[$1>>2]|0;
  $60 = (_vdbePmaReadVarint($59,$nRec)|0);
  HEAP32[$rc>>2] = $60;
 }
 $61 = HEAP32[$rc>>2]|0;
 $62 = ($61|0)==(0);
 if ($62) {
  $63 = $nRec;
  $64 = $63;
  $65 = HEAP32[$64>>2]|0;
  $66 = (($63) + 4)|0;
  $67 = $66;
  $68 = HEAP32[$67>>2]|0;
  $69 = HEAP32[$1>>2]|0;
  $70 = (($69) + 20|0);
  HEAP32[$70>>2] = $65;
  $71 = HEAP32[$1>>2]|0;
  $72 = $nRec;
  $73 = $72;
  $74 = HEAP32[$73>>2]|0;
  $75 = (($72) + 4)|0;
  $76 = $75;
  $77 = HEAP32[$76>>2]|0;
  $78 = HEAP32[$1>>2]|0;
  $79 = (($78) + 32|0);
  $80 = (_vdbePmaReadBlob($71,$74,$79)|0);
  HEAP32[$rc>>2] = $80;
 }
 $81 = HEAP32[$rc>>2]|0;
 HEAP32[$0>>2] = $81;
 $82 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($82|0);
}
function _vdbeIncrSwap($pIncr) {
 $pIncr = $pIncr|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $rc = sp;
 HEAP32[$0>>2] = $pIncr;
 HEAP32[$rc>>2] = 0;
 $1 = HEAP32[$0>>2]|0;
 $2 = (_vdbeIncrPopulate($1)|0);
 HEAP32[$rc>>2] = $2;
 $3 = HEAP32[$0>>2]|0;
 $4 = (($3) + 32|0);
 $5 = HEAP32[$0>>2]|0;
 $6 = (($5) + 32|0);
 $7 = (($6) + 16|0);
 ;HEAP32[$4+0>>2]=HEAP32[$7+0>>2]|0;HEAP32[$4+4>>2]=HEAP32[$7+4>>2]|0;HEAP32[$4+8>>2]=HEAP32[$7+8>>2]|0;HEAP32[$4+12>>2]=HEAP32[$7+12>>2]|0;
 $8 = HEAP32[$0>>2]|0;
 $9 = (($8) + 32|0);
 $10 = (($9) + 8|0);
 $11 = $10;
 $12 = $11;
 $13 = HEAP32[$12>>2]|0;
 $14 = (($11) + 4)|0;
 $15 = $14;
 $16 = HEAP32[$15>>2]|0;
 $17 = HEAP32[$0>>2]|0;
 $18 = (($17) + 8|0);
 $19 = $18;
 $20 = $19;
 $21 = HEAP32[$20>>2]|0;
 $22 = (($19) + 4)|0;
 $23 = $22;
 $24 = HEAP32[$23>>2]|0;
 $25 = ($13|0)==($21|0);
 $26 = ($16|0)==($24|0);
 $27 = $25 & $26;
 if (!($27)) {
  $30 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($30|0);
 }
 $28 = HEAP32[$0>>2]|0;
 $29 = (($28) + 20|0);
 HEAP32[$29>>2] = 1;
 $30 = HEAP32[$rc>>2]|0;
 STACKTOP = sp;return ($30|0);
}
function _vdbePmaReaderSeek($pTask,$pReadr,$pFile,$0,$1) {
 $pTask = $pTask|0;
 $pReadr = $pReadr|0;
 $pFile = $pFile|0;
 $0 = $0|0;
 $1 = $1|0;
 var $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0, $116 = 0, $117 = 0;
 var $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0, $134 = 0, $135 = 0;
 var $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0, $152 = 0, $153 = 0;
 var $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0;
 var $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0;
 var $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0;
 var $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0;
 var $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0;
 var $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $iBuf = 0, $nRead = 0, $or$cond = 0, $pgsz = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $2 = sp + 36|0;
 $3 = sp + 32|0;
 $4 = sp + 28|0;
 $5 = sp + 24|0;
 $6 = sp;
 $rc = sp + 20|0;
 $pgsz = sp + 16|0;
 $iBuf = sp + 12|0;
 $nRead = sp + 8|0;
 HEAP32[$3>>2] = $pTask;
 HEAP32[$4>>2] = $pReadr;
 HEAP32[$5>>2] = $pFile;
 $7 = $6;
 $8 = $7;
 HEAP32[$8>>2] = $0;
 $9 = (($7) + 4)|0;
 $10 = $9;
 HEAP32[$10>>2] = $1;
 HEAP32[$rc>>2] = 0;
 $11 = (_sqlite3FaultSim(201)|0);
 $12 = ($11|0)!=(0);
 if ($12) {
  HEAP32[$2>>2] = 266;
  $166 = HEAP32[$2>>2]|0;
  STACKTOP = sp;return ($166|0);
 }
 $13 = HEAP32[$4>>2]|0;
 $14 = (($13) + 44|0);
 $15 = HEAP32[$14>>2]|0;
 $16 = ($15|0)!=(0|0);
 if ($16) {
  $17 = HEAP32[$4>>2]|0;
  $18 = (($17) + 24|0);
  $19 = HEAP32[$18>>2]|0;
  $20 = HEAP32[$4>>2]|0;
  $21 = (($20) + 44|0);
  $22 = HEAP32[$21>>2]|0;
  (_sqlite3OsUnfetch($19,0,0,$22)|0);
  $23 = HEAP32[$4>>2]|0;
  $24 = (($23) + 44|0);
  HEAP32[$24>>2] = 0;
 }
 $25 = $6;
 $26 = $25;
 $27 = HEAP32[$26>>2]|0;
 $28 = (($25) + 4)|0;
 $29 = $28;
 $30 = HEAP32[$29>>2]|0;
 $31 = HEAP32[$4>>2]|0;
 $32 = $31;
 $33 = $32;
 HEAP32[$33>>2] = $27;
 $34 = (($32) + 4)|0;
 $35 = $34;
 HEAP32[$35>>2] = $30;
 $36 = HEAP32[$5>>2]|0;
 $37 = (($36) + 8|0);
 $38 = $37;
 $39 = $38;
 $40 = HEAP32[$39>>2]|0;
 $41 = (($38) + 4)|0;
 $42 = $41;
 $43 = HEAP32[$42>>2]|0;
 $44 = HEAP32[$4>>2]|0;
 $45 = (($44) + 8|0);
 $46 = $45;
 $47 = $46;
 HEAP32[$47>>2] = $40;
 $48 = (($46) + 4)|0;
 $49 = $48;
 HEAP32[$49>>2] = $43;
 $50 = HEAP32[$5>>2]|0;
 $51 = HEAP32[$50>>2]|0;
 $52 = HEAP32[$4>>2]|0;
 $53 = (($52) + 24|0);
 HEAP32[$53>>2] = $51;
 $54 = HEAP32[$3>>2]|0;
 $55 = HEAP32[$5>>2]|0;
 $56 = HEAP32[$4>>2]|0;
 $57 = (($56) + 44|0);
 $58 = (_vdbeSorterMapFile($54,$55,$57)|0);
 HEAP32[$rc>>2] = $58;
 $59 = HEAP32[$rc>>2]|0;
 $60 = ($59|0)==(0);
 if ($60) {
  $61 = HEAP32[$4>>2]|0;
  $62 = (($61) + 44|0);
  $63 = HEAP32[$62>>2]|0;
  $64 = ($63|0)==(0|0);
  if ($64) {
   $65 = HEAP32[$3>>2]|0;
   $66 = (($65) + 8|0);
   $67 = HEAP32[$66>>2]|0;
   $68 = (($67) + 12|0);
   $69 = HEAP32[$68>>2]|0;
   HEAP32[$pgsz>>2] = $69;
   $70 = HEAP32[$4>>2]|0;
   $71 = $70;
   $72 = $71;
   $73 = HEAP32[$72>>2]|0;
   $74 = (($71) + 4)|0;
   $75 = $74;
   $76 = HEAP32[$75>>2]|0;
   $77 = HEAP32[$pgsz>>2]|0;
   $78 = ($77|0)<(0);
   $79 = $78 << 31 >> 31;
   $80 = (___remdi3(($73|0),($76|0),($77|0),($79|0))|0);
   $81 = tempRet0;
   HEAP32[$iBuf>>2] = $80;
   $82 = HEAP32[$4>>2]|0;
   $83 = (($82) + 36|0);
   $84 = HEAP32[$83>>2]|0;
   $85 = ($84|0)==(0|0);
   if ($85) {
    $86 = HEAP32[$pgsz>>2]|0;
    $87 = ($86|0)<(0);
    $88 = $87 << 31 >> 31;
    $89 = (_sqlite3Malloc($86,$88)|0);
    $90 = HEAP32[$4>>2]|0;
    $91 = (($90) + 36|0);
    HEAP32[$91>>2] = $89;
    $92 = HEAP32[$4>>2]|0;
    $93 = (($92) + 36|0);
    $94 = HEAP32[$93>>2]|0;
    $95 = ($94|0)==(0|0);
    if ($95) {
     HEAP32[$rc>>2] = 7;
    }
    $96 = HEAP32[$pgsz>>2]|0;
    $97 = HEAP32[$4>>2]|0;
    $98 = (($97) + 40|0);
    HEAP32[$98>>2] = $96;
   }
   $99 = HEAP32[$rc>>2]|0;
   $100 = ($99|0)==(0);
   $101 = HEAP32[$iBuf>>2]|0;
   $102 = ($101|0)!=(0);
   $or$cond = $100 & $102;
   if ($or$cond) {
    $103 = HEAP32[$pgsz>>2]|0;
    $104 = HEAP32[$iBuf>>2]|0;
    $105 = (($103) - ($104))|0;
    HEAP32[$nRead>>2] = $105;
    $106 = HEAP32[$4>>2]|0;
    $107 = $106;
    $108 = $107;
    $109 = HEAP32[$108>>2]|0;
    $110 = (($107) + 4)|0;
    $111 = $110;
    $112 = HEAP32[$111>>2]|0;
    $113 = HEAP32[$nRead>>2]|0;
    $114 = ($113|0)<(0);
    $115 = $114 << 31 >> 31;
    $116 = (_i64Add(($109|0),($112|0),($113|0),($115|0))|0);
    $117 = tempRet0;
    $118 = HEAP32[$4>>2]|0;
    $119 = (($118) + 8|0);
    $120 = $119;
    $121 = $120;
    $122 = HEAP32[$121>>2]|0;
    $123 = (($120) + 4)|0;
    $124 = $123;
    $125 = HEAP32[$124>>2]|0;
    $126 = ($117|0)>($125|0);
    $127 = ($116>>>0)>($122>>>0);
    $128 = ($117|0)==($125|0);
    $129 = $128 & $127;
    $130 = $126 | $129;
    if ($130) {
     $131 = HEAP32[$4>>2]|0;
     $132 = (($131) + 8|0);
     $133 = $132;
     $134 = $133;
     $135 = HEAP32[$134>>2]|0;
     $136 = (($133) + 4)|0;
     $137 = $136;
     $138 = HEAP32[$137>>2]|0;
     $139 = HEAP32[$4>>2]|0;
     $140 = $139;
     $141 = $140;
     $142 = HEAP32[$141>>2]|0;
     $143 = (($140) + 4)|0;
     $144 = $143;
     $145 = HEAP32[$144>>2]|0;
     $146 = (_i64Subtract(($135|0),($138|0),($142|0),($145|0))|0);
     $147 = tempRet0;
     HEAP32[$nRead>>2] = $146;
    }
    $148 = HEAP32[$4>>2]|0;
    $149 = (($148) + 24|0);
    $150 = HEAP32[$149>>2]|0;
    $151 = HEAP32[$iBuf>>2]|0;
    $152 = HEAP32[$4>>2]|0;
    $153 = (($152) + 36|0);
    $154 = HEAP32[$153>>2]|0;
    $155 = (($154) + ($151)|0);
    $156 = HEAP32[$nRead>>2]|0;
    $157 = HEAP32[$4>>2]|0;
    $158 = $157;
    $159 = $158;
    $160 = HEAP32[$159>>2]|0;
    $161 = (($158) + 4)|0;
    $162 = $161;
    $163 = HEAP32[$162>>2]|0;
    $164 = (_sqlite3OsRead($150,$155,$156,$160,$163)|0);
    HEAP32[$rc>>2] = $164;
   }
  }
 }
 $165 = HEAP32[$rc>>2]|0;
 HEAP32[$2>>2] = $165;
 $166 = HEAP32[$2>>2]|0;
 STACKTOP = sp;return ($166|0);
}
function _vdbePmaReadVarint($p,$pnOut) {
 $p = $p|0;
 $pnOut = $pnOut|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0;
 var $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $9 = 0, $a = 0, $aVarint = 0, $i = 0, $iBuf = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $0 = sp + 24|0;
 $1 = sp + 20|0;
 $2 = sp + 16|0;
 $iBuf = sp + 12|0;
 $aVarint = sp + 32|0;
 $a = sp + 8|0;
 $i = sp + 4|0;
 $rc = sp;
 HEAP32[$1>>2] = $p;
 HEAP32[$2>>2] = $pnOut;
 $3 = HEAP32[$1>>2]|0;
 $4 = (($3) + 44|0);
 $5 = HEAP32[$4>>2]|0;
 $6 = ($5|0)!=(0|0);
 $7 = HEAP32[$1>>2]|0;
 $8 = $7;
 $9 = $8;
 $10 = HEAP32[$9>>2]|0;
 $11 = (($8) + 4)|0;
 $12 = $11;
 $13 = HEAP32[$12>>2]|0;
 do {
  if ($6) {
   $14 = HEAP32[$1>>2]|0;
   $15 = (($14) + 44|0);
   $16 = HEAP32[$15>>2]|0;
   $17 = (($16) + ($10)|0);
   $18 = HEAP32[$2>>2]|0;
   $19 = (_sqlite3GetVarint($17,$18)|0);
   $20 = $19&255;
   $21 = HEAP32[$1>>2]|0;
   $22 = $21;
   $23 = $22;
   $24 = HEAP32[$23>>2]|0;
   $25 = (($22) + 4)|0;
   $26 = $25;
   $27 = HEAP32[$26>>2]|0;
   $28 = (_i64Add(($24|0),($27|0),($20|0),0)|0);
   $29 = tempRet0;
   $30 = $21;
   $31 = $30;
   HEAP32[$31>>2] = $28;
   $32 = (($30) + 4)|0;
   $33 = $32;
   HEAP32[$33>>2] = $29;
  } else {
   $34 = HEAP32[$1>>2]|0;
   $35 = (($34) + 40|0);
   $36 = HEAP32[$35>>2]|0;
   $37 = ($36|0)<(0);
   $38 = $37 << 31 >> 31;
   $39 = (___remdi3(($10|0),($13|0),($36|0),($38|0))|0);
   $40 = tempRet0;
   HEAP32[$iBuf>>2] = $39;
   $41 = HEAP32[$iBuf>>2]|0;
   $42 = ($41|0)!=(0);
   if ($42) {
    $43 = HEAP32[$1>>2]|0;
    $44 = (($43) + 40|0);
    $45 = HEAP32[$44>>2]|0;
    $46 = HEAP32[$iBuf>>2]|0;
    $47 = (($45) - ($46))|0;
    $48 = ($47|0)>=(9);
    if ($48) {
     $49 = HEAP32[$iBuf>>2]|0;
     $50 = HEAP32[$1>>2]|0;
     $51 = (($50) + 36|0);
     $52 = HEAP32[$51>>2]|0;
     $53 = (($52) + ($49)|0);
     $54 = HEAP32[$2>>2]|0;
     $55 = (_sqlite3GetVarint($53,$54)|0);
     $56 = $55&255;
     $57 = HEAP32[$1>>2]|0;
     $58 = $57;
     $59 = $58;
     $60 = HEAP32[$59>>2]|0;
     $61 = (($58) + 4)|0;
     $62 = $61;
     $63 = HEAP32[$62>>2]|0;
     $64 = (_i64Add(($60|0),($63|0),($56|0),0)|0);
     $65 = tempRet0;
     $66 = $57;
     $67 = $66;
     HEAP32[$67>>2] = $64;
     $68 = (($66) + 4)|0;
     $69 = $68;
     HEAP32[$69>>2] = $65;
     break;
    }
   }
   HEAP32[$i>>2] = 0;
   while(1) {
    $70 = HEAP32[$1>>2]|0;
    $71 = (_vdbePmaReadBlob($70,1,$a)|0);
    HEAP32[$rc>>2] = $71;
    $72 = HEAP32[$rc>>2]|0;
    $73 = ($72|0)!=(0);
    if ($73) {
     break;
    }
    $75 = HEAP32[$a>>2]|0;
    $76 = HEAP8[$75>>0]|0;
    $77 = HEAP32[$i>>2]|0;
    $78 = (($77) + 1)|0;
    HEAP32[$i>>2] = $78;
    $79 = $77 & 15;
    $80 = (($aVarint) + ($79)|0);
    HEAP8[$80>>0] = $76;
    $81 = HEAP32[$a>>2]|0;
    $82 = HEAP8[$81>>0]|0;
    $83 = $82&255;
    $84 = $83 & 128;
    $85 = ($84|0)!=(0);
    if (!($85)) {
     label = 10;
     break;
    }
   }
   if ((label|0) == 10) {
    $86 = HEAP32[$2>>2]|0;
    (_sqlite3GetVarint($aVarint,$86)|0);
    break;
   }
   $74 = HEAP32[$rc>>2]|0;
   HEAP32[$0>>2] = $74;
   $87 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($87|0);
  }
 } while(0);
 HEAP32[$0>>2] = 0;
 $87 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($87|0);
}
function _vdbePmaReadBlob($p,$nByte,$ppOut) {
 $p = $p|0;
 $nByte = $nByte|0;
 $ppOut = $ppOut|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0;
 var $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0, $188 = 0;
 var $189 = 0, $19 = 0, $190 = 0, $191 = 0, $192 = 0, $193 = 0, $194 = 0, $195 = 0, $196 = 0, $197 = 0, $198 = 0, $199 = 0, $2 = 0, $20 = 0, $200 = 0, $201 = 0, $202 = 0, $203 = 0, $204 = 0, $205 = 0;
 var $206 = 0, $207 = 0, $208 = 0, $209 = 0, $21 = 0, $210 = 0, $211 = 0, $212 = 0, $213 = 0, $214 = 0, $215 = 0, $216 = 0, $217 = 0, $218 = 0, $219 = 0, $22 = 0, $220 = 0, $221 = 0, $222 = 0, $223 = 0;
 var $224 = 0, $225 = 0, $226 = 0, $227 = 0, $228 = 0, $229 = 0, $23 = 0, $230 = 0, $231 = 0, $232 = 0, $233 = 0, $234 = 0, $235 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0;
 var $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0;
 var $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0;
 var $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0;
 var $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $aNew = 0, $aNext = 0, $iBuf = 0, $nAvail = 0;
 var $nCopy = 0, $nNew = 0, $nRead = 0, $nRem = 0, $rc = 0, $rc1 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 64|0;
 $0 = sp + 52|0;
 $1 = sp + 48|0;
 $2 = sp + 44|0;
 $3 = sp + 40|0;
 $iBuf = sp + 36|0;
 $nAvail = sp + 32|0;
 $nRead = sp + 28|0;
 $rc = sp + 24|0;
 $nRem = sp + 20|0;
 $aNew = sp + 16|0;
 $nNew = sp + 12|0;
 $rc1 = sp + 8|0;
 $nCopy = sp + 4|0;
 $aNext = sp;
 HEAP32[$1>>2] = $p;
 HEAP32[$2>>2] = $nByte;
 HEAP32[$3>>2] = $ppOut;
 $4 = HEAP32[$1>>2]|0;
 $5 = (($4) + 44|0);
 $6 = HEAP32[$5>>2]|0;
 $7 = ($6|0)!=(0|0);
 $8 = HEAP32[$1>>2]|0;
 $9 = $8;
 $10 = $9;
 $11 = HEAP32[$10>>2]|0;
 $12 = (($9) + 4)|0;
 $13 = $12;
 $14 = HEAP32[$13>>2]|0;
 if ($7) {
  $15 = HEAP32[$1>>2]|0;
  $16 = (($15) + 44|0);
  $17 = HEAP32[$16>>2]|0;
  $18 = (($17) + ($11)|0);
  $19 = HEAP32[$3>>2]|0;
  HEAP32[$19>>2] = $18;
  $20 = HEAP32[$2>>2]|0;
  $21 = ($20|0)<(0);
  $22 = $21 << 31 >> 31;
  $23 = HEAP32[$1>>2]|0;
  $24 = $23;
  $25 = $24;
  $26 = HEAP32[$25>>2]|0;
  $27 = (($24) + 4)|0;
  $28 = $27;
  $29 = HEAP32[$28>>2]|0;
  $30 = (_i64Add(($26|0),($29|0),($20|0),($22|0))|0);
  $31 = tempRet0;
  $32 = $23;
  $33 = $32;
  HEAP32[$33>>2] = $30;
  $34 = (($32) + 4)|0;
  $35 = $34;
  HEAP32[$35>>2] = $31;
  HEAP32[$0>>2] = 0;
  $235 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($235|0);
 }
 $36 = HEAP32[$1>>2]|0;
 $37 = (($36) + 40|0);
 $38 = HEAP32[$37>>2]|0;
 $39 = ($38|0)<(0);
 $40 = $39 << 31 >> 31;
 $41 = (___remdi3(($11|0),($14|0),($38|0),($40|0))|0);
 $42 = tempRet0;
 HEAP32[$iBuf>>2] = $41;
 $43 = HEAP32[$iBuf>>2]|0;
 $44 = ($43|0)==(0);
 if ($44) {
  $45 = HEAP32[$1>>2]|0;
  $46 = (($45) + 8|0);
  $47 = $46;
  $48 = $47;
  $49 = HEAP32[$48>>2]|0;
  $50 = (($47) + 4)|0;
  $51 = $50;
  $52 = HEAP32[$51>>2]|0;
  $53 = HEAP32[$1>>2]|0;
  $54 = $53;
  $55 = $54;
  $56 = HEAP32[$55>>2]|0;
  $57 = (($54) + 4)|0;
  $58 = $57;
  $59 = HEAP32[$58>>2]|0;
  $60 = (_i64Subtract(($49|0),($52|0),($56|0),($59|0))|0);
  $61 = tempRet0;
  $62 = HEAP32[$1>>2]|0;
  $63 = (($62) + 40|0);
  $64 = HEAP32[$63>>2]|0;
  $65 = ($64|0)<(0);
  $66 = $65 << 31 >> 31;
  $67 = ($61|0)>($66|0);
  $68 = ($60>>>0)>($64>>>0);
  $69 = ($61|0)==($66|0);
  $70 = $69 & $68;
  $71 = $67 | $70;
  $72 = HEAP32[$1>>2]|0;
  if ($71) {
   $73 = (($72) + 40|0);
   $74 = HEAP32[$73>>2]|0;
   HEAP32[$nRead>>2] = $74;
  } else {
   $75 = (($72) + 8|0);
   $76 = $75;
   $77 = $76;
   $78 = HEAP32[$77>>2]|0;
   $79 = (($76) + 4)|0;
   $80 = $79;
   $81 = HEAP32[$80>>2]|0;
   $82 = HEAP32[$1>>2]|0;
   $83 = $82;
   $84 = $83;
   $85 = HEAP32[$84>>2]|0;
   $86 = (($83) + 4)|0;
   $87 = $86;
   $88 = HEAP32[$87>>2]|0;
   $89 = (_i64Subtract(($78|0),($81|0),($85|0),($88|0))|0);
   $90 = tempRet0;
   HEAP32[$nRead>>2] = $89;
  }
  $91 = HEAP32[$1>>2]|0;
  $92 = (($91) + 24|0);
  $93 = HEAP32[$92>>2]|0;
  $94 = HEAP32[$1>>2]|0;
  $95 = (($94) + 36|0);
  $96 = HEAP32[$95>>2]|0;
  $97 = HEAP32[$nRead>>2]|0;
  $98 = HEAP32[$1>>2]|0;
  $99 = $98;
  $100 = $99;
  $101 = HEAP32[$100>>2]|0;
  $102 = (($99) + 4)|0;
  $103 = $102;
  $104 = HEAP32[$103>>2]|0;
  $105 = (_sqlite3OsRead($93,$96,$97,$101,$104)|0);
  HEAP32[$rc>>2] = $105;
  $106 = HEAP32[$rc>>2]|0;
  $107 = ($106|0)!=(0);
  if ($107) {
   $108 = HEAP32[$rc>>2]|0;
   HEAP32[$0>>2] = $108;
   $235 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($235|0);
  }
 }
 $109 = HEAP32[$1>>2]|0;
 $110 = (($109) + 40|0);
 $111 = HEAP32[$110>>2]|0;
 $112 = HEAP32[$iBuf>>2]|0;
 $113 = (($111) - ($112))|0;
 HEAP32[$nAvail>>2] = $113;
 $114 = HEAP32[$2>>2]|0;
 $115 = HEAP32[$nAvail>>2]|0;
 $116 = ($114|0)<=($115|0);
 do {
  if ($116) {
   $117 = HEAP32[$iBuf>>2]|0;
   $118 = HEAP32[$1>>2]|0;
   $119 = (($118) + 36|0);
   $120 = HEAP32[$119>>2]|0;
   $121 = (($120) + ($117)|0);
   $122 = HEAP32[$3>>2]|0;
   HEAP32[$122>>2] = $121;
   $123 = HEAP32[$2>>2]|0;
   $124 = ($123|0)<(0);
   $125 = $124 << 31 >> 31;
   $126 = HEAP32[$1>>2]|0;
   $127 = $126;
   $128 = $127;
   $129 = HEAP32[$128>>2]|0;
   $130 = (($127) + 4)|0;
   $131 = $130;
   $132 = HEAP32[$131>>2]|0;
   $133 = (_i64Add(($129|0),($132|0),($123|0),($125|0))|0);
   $134 = tempRet0;
   $135 = $126;
   $136 = $135;
   HEAP32[$136>>2] = $133;
   $137 = (($135) + 4)|0;
   $138 = $137;
   HEAP32[$138>>2] = $134;
  } else {
   $139 = HEAP32[$1>>2]|0;
   $140 = (($139) + 16|0);
   $141 = HEAP32[$140>>2]|0;
   $142 = HEAP32[$2>>2]|0;
   $143 = ($141|0)<($142|0);
   do {
    if ($143) {
     $144 = HEAP32[$1>>2]|0;
     $145 = (($144) + 16|0);
     $146 = HEAP32[$145>>2]|0;
     $147 = $146<<1;
     $148 = (128)>($147|0);
     if ($148) {
      $153 = 128;
     } else {
      $149 = HEAP32[$1>>2]|0;
      $150 = (($149) + 16|0);
      $151 = HEAP32[$150>>2]|0;
      $152 = $151<<1;
      $153 = $152;
     }
     HEAP32[$nNew>>2] = $153;
     while(1) {
      $154 = HEAP32[$2>>2]|0;
      $155 = HEAP32[$nNew>>2]|0;
      $156 = ($154|0)>($155|0);
      if (!($156)) {
       break;
      }
      $157 = HEAP32[$nNew>>2]|0;
      $158 = $157<<1;
      HEAP32[$nNew>>2] = $158;
     }
     $159 = HEAP32[$1>>2]|0;
     $160 = (($159) + 28|0);
     $161 = HEAP32[$160>>2]|0;
     $162 = HEAP32[$nNew>>2]|0;
     $163 = ($162|0)<(0);
     $164 = $163 << 31 >> 31;
     $165 = (_sqlite3Realloc($161,$162,$164)|0);
     HEAP32[$aNew>>2] = $165;
     $166 = HEAP32[$aNew>>2]|0;
     $167 = ($166|0)!=(0|0);
     if ($167) {
      $168 = HEAP32[$nNew>>2]|0;
      $169 = HEAP32[$1>>2]|0;
      $170 = (($169) + 16|0);
      HEAP32[$170>>2] = $168;
      $171 = HEAP32[$aNew>>2]|0;
      $172 = HEAP32[$1>>2]|0;
      $173 = (($172) + 28|0);
      HEAP32[$173>>2] = $171;
      break;
     }
     HEAP32[$0>>2] = 7;
     $235 = HEAP32[$0>>2]|0;
     STACKTOP = sp;return ($235|0);
    }
   } while(0);
   $174 = HEAP32[$1>>2]|0;
   $175 = (($174) + 28|0);
   $176 = HEAP32[$175>>2]|0;
   $177 = HEAP32[$iBuf>>2]|0;
   $178 = HEAP32[$1>>2]|0;
   $179 = (($178) + 36|0);
   $180 = HEAP32[$179>>2]|0;
   $181 = (($180) + ($177)|0);
   $182 = HEAP32[$nAvail>>2]|0;
   _memcpy(($176|0),($181|0),($182|0))|0;
   $183 = HEAP32[$nAvail>>2]|0;
   $184 = ($183|0)<(0);
   $185 = $184 << 31 >> 31;
   $186 = HEAP32[$1>>2]|0;
   $187 = $186;
   $188 = $187;
   $189 = HEAP32[$188>>2]|0;
   $190 = (($187) + 4)|0;
   $191 = $190;
   $192 = HEAP32[$191>>2]|0;
   $193 = (_i64Add(($189|0),($192|0),($183|0),($185|0))|0);
   $194 = tempRet0;
   $195 = $186;
   $196 = $195;
   HEAP32[$196>>2] = $193;
   $197 = (($195) + 4)|0;
   $198 = $197;
   HEAP32[$198>>2] = $194;
   $199 = HEAP32[$2>>2]|0;
   $200 = HEAP32[$nAvail>>2]|0;
   $201 = (($199) - ($200))|0;
   HEAP32[$nRem>>2] = $201;
   while(1) {
    $202 = HEAP32[$nRem>>2]|0;
    $203 = ($202|0)>(0);
    if (!($203)) {
     label = 27;
     break;
    }
    $204 = HEAP32[$nRem>>2]|0;
    HEAP32[$nCopy>>2] = $204;
    $205 = HEAP32[$nRem>>2]|0;
    $206 = HEAP32[$1>>2]|0;
    $207 = (($206) + 40|0);
    $208 = HEAP32[$207>>2]|0;
    $209 = ($205|0)>($208|0);
    if ($209) {
     $210 = HEAP32[$1>>2]|0;
     $211 = (($210) + 40|0);
     $212 = HEAP32[$211>>2]|0;
     HEAP32[$nCopy>>2] = $212;
    }
    $213 = HEAP32[$1>>2]|0;
    $214 = HEAP32[$nCopy>>2]|0;
    $215 = (_vdbePmaReadBlob($213,$214,$aNext)|0);
    HEAP32[$rc1>>2] = $215;
    $216 = HEAP32[$rc1>>2]|0;
    $217 = ($216|0)!=(0);
    if ($217) {
     break;
    }
    $219 = HEAP32[$2>>2]|0;
    $220 = HEAP32[$nRem>>2]|0;
    $221 = (($219) - ($220))|0;
    $222 = HEAP32[$1>>2]|0;
    $223 = (($222) + 28|0);
    $224 = HEAP32[$223>>2]|0;
    $225 = (($224) + ($221)|0);
    $226 = HEAP32[$aNext>>2]|0;
    $227 = HEAP32[$nCopy>>2]|0;
    _memcpy(($225|0),($226|0),($227|0))|0;
    $228 = HEAP32[$nCopy>>2]|0;
    $229 = HEAP32[$nRem>>2]|0;
    $230 = (($229) - ($228))|0;
    HEAP32[$nRem>>2] = $230;
   }
   if ((label|0) == 27) {
    $231 = HEAP32[$1>>2]|0;
    $232 = (($231) + 28|0);
    $233 = HEAP32[$232>>2]|0;
    $234 = HEAP32[$3>>2]|0;
    HEAP32[$234>>2] = $233;
    break;
   }
   $218 = HEAP32[$rc1>>2]|0;
   HEAP32[$0>>2] = $218;
   $235 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($235|0);
  }
 } while(0);
 HEAP32[$0>>2] = 0;
 $235 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($235|0);
}
function _vdbeSorterMapFile($pTask,$pFile,$pp) {
 $pTask = $pTask|0;
 $pFile = $pFile|0;
 $pp = $pp|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $5 = 0, $6 = 0;
 var $7 = 0, $8 = 0, $9 = 0, $pFd = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 16|0;
 $1 = sp + 12|0;
 $2 = sp + 8|0;
 $rc = sp + 4|0;
 $pFd = sp;
 HEAP32[$0>>2] = $pTask;
 HEAP32[$1>>2] = $pFile;
 HEAP32[$2>>2] = $pp;
 HEAP32[$rc>>2] = 0;
 $3 = HEAP32[$1>>2]|0;
 $4 = (($3) + 8|0);
 $5 = $4;
 $6 = $5;
 $7 = HEAP32[$6>>2]|0;
 $8 = (($5) + 4)|0;
 $9 = $8;
 $10 = HEAP32[$9>>2]|0;
 $11 = HEAP32[$0>>2]|0;
 $12 = (($11) + 8|0);
 $13 = HEAP32[$12>>2]|0;
 $14 = (($13) + 24|0);
 $15 = HEAP32[$14>>2]|0;
 $16 = (($15) + 136|0);
 $17 = HEAP32[$16>>2]|0;
 $18 = ($17|0)<(0);
 $19 = $18 << 31 >> 31;
 $20 = ($10|0)<($19|0);
 $21 = ($7>>>0)<=($17>>>0);
 $22 = ($10|0)==($19|0);
 $23 = $22 & $21;
 $24 = $20 | $23;
 if (!($24)) {
  $42 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($42|0);
 }
 $25 = HEAP32[$1>>2]|0;
 $26 = HEAP32[$25>>2]|0;
 HEAP32[$pFd>>2] = $26;
 $27 = HEAP32[$pFd>>2]|0;
 $28 = HEAP32[$27>>2]|0;
 $29 = HEAP32[$28>>2]|0;
 $30 = ($29|0)>=(3);
 if (!($30)) {
  $42 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($42|0);
 }
 $31 = HEAP32[$pFd>>2]|0;
 $32 = HEAP32[$1>>2]|0;
 $33 = (($32) + 8|0);
 $34 = $33;
 $35 = $34;
 $36 = HEAP32[$35>>2]|0;
 $37 = (($34) + 4)|0;
 $38 = $37;
 $39 = HEAP32[$38>>2]|0;
 $40 = HEAP32[$2>>2]|0;
 $41 = (_sqlite3OsFetch($31,0,0,$36,$40)|0);
 HEAP32[$rc>>2] = $41;
 $42 = HEAP32[$rc>>2]|0;
 STACKTOP = sp;return ($42|0);
}
function _sqlite3OsFetch($id,$0,$1,$iAmt,$pp) {
 $id = $id|0;
 $0 = $0|0;
 $1 = $1|0;
 $iAmt = $iAmt|0;
 $pp = $pp|0;
 var $10 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $2 = sp + 16|0;
 $3 = sp;
 $4 = sp + 12|0;
 $5 = sp + 8|0;
 HEAP32[$2>>2] = $id;
 $6 = $3;
 $7 = $6;
 HEAP32[$7>>2] = $0;
 $8 = (($6) + 4)|0;
 $9 = $8;
 HEAP32[$9>>2] = $1;
 HEAP32[$4>>2] = $iAmt;
 HEAP32[$5>>2] = $pp;
 $10 = HEAP32[$5>>2]|0;
 HEAP32[$10>>2] = 0;
 STACKTOP = sp;return 0;
}
function _vdbeIncrPopulate($pIncr) {
 $pIncr = $pIncr|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0;
 var $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0;
 var $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0;
 var $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0;
 var $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0;
 var $96 = 0, $97 = 0, $98 = 0, $99 = 0, $dummy = 0, $iEof = 0, $iStart = 0, $nKey = 0, $pMerger = 0, $pOut = 0, $pReader = 0, $pTask = 0, $rc = 0, $rc2 = 0, $writer = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 96|0;
 $0 = sp + 88|0;
 $rc = sp + 84|0;
 $rc2 = sp + 80|0;
 $iStart = sp + 48|0;
 $pOut = sp + 76|0;
 $pTask = sp + 72|0;
 $pMerger = sp + 68|0;
 $writer = sp + 8|0;
 $dummy = sp + 64|0;
 $pReader = sp + 60|0;
 $nKey = sp + 56|0;
 $iEof = sp;
 HEAP32[$0>>2] = $pIncr;
 HEAP32[$rc>>2] = 0;
 $1 = HEAP32[$0>>2]|0;
 $2 = (($1) + 8|0);
 $3 = $2;
 $4 = $3;
 $5 = HEAP32[$4>>2]|0;
 $6 = (($3) + 4)|0;
 $7 = $6;
 $8 = HEAP32[$7>>2]|0;
 $9 = $iStart;
 $10 = $9;
 HEAP32[$10>>2] = $5;
 $11 = (($9) + 4)|0;
 $12 = $11;
 HEAP32[$12>>2] = $8;
 $13 = HEAP32[$0>>2]|0;
 $14 = (($13) + 32|0);
 $15 = (($14) + 16|0);
 HEAP32[$pOut>>2] = $15;
 $16 = HEAP32[$0>>2]|0;
 $17 = HEAP32[$16>>2]|0;
 HEAP32[$pTask>>2] = $17;
 $18 = HEAP32[$0>>2]|0;
 $19 = (($18) + 4|0);
 $20 = HEAP32[$19>>2]|0;
 HEAP32[$pMerger>>2] = $20;
 $21 = HEAP32[$pOut>>2]|0;
 $22 = HEAP32[$21>>2]|0;
 $23 = HEAP32[$pTask>>2]|0;
 $24 = (($23) + 8|0);
 $25 = HEAP32[$24>>2]|0;
 $26 = (($25) + 12|0);
 $27 = HEAP32[$26>>2]|0;
 $28 = $iStart;
 $29 = $28;
 $30 = HEAP32[$29>>2]|0;
 $31 = (($28) + 4)|0;
 $32 = $31;
 $33 = HEAP32[$32>>2]|0;
 _vdbePmaWriterInit($22,$writer,$27,$30,$33);
 while(1) {
  $34 = HEAP32[$rc>>2]|0;
  $35 = ($34|0)==(0);
  if (!($35)) {
   break;
  }
  $36 = HEAP32[$pMerger>>2]|0;
  $37 = (($36) + 8|0);
  $38 = HEAP32[$37>>2]|0;
  $39 = (($38) + 4|0);
  $40 = HEAP32[$39>>2]|0;
  $41 = HEAP32[$pMerger>>2]|0;
  $42 = (($41) + 12|0);
  $43 = HEAP32[$42>>2]|0;
  $44 = (($43) + (($40*56)|0)|0);
  HEAP32[$pReader>>2] = $44;
  $45 = HEAP32[$pReader>>2]|0;
  $46 = (($45) + 20|0);
  $47 = HEAP32[$46>>2]|0;
  HEAP32[$nKey>>2] = $47;
  $48 = (($writer) + 24|0);
  $49 = $48;
  $50 = $49;
  $51 = HEAP32[$50>>2]|0;
  $52 = (($49) + 4)|0;
  $53 = $52;
  $54 = HEAP32[$53>>2]|0;
  $55 = (($writer) + 16|0);
  $56 = HEAP32[$55>>2]|0;
  $57 = ($56|0)<(0);
  $58 = $57 << 31 >> 31;
  $59 = (_i64Add(($51|0),($54|0),($56|0),($58|0))|0);
  $60 = tempRet0;
  $61 = $iEof;
  $62 = $61;
  HEAP32[$62>>2] = $59;
  $63 = (($61) + 4)|0;
  $64 = $63;
  HEAP32[$64>>2] = $60;
  $65 = HEAP32[$pReader>>2]|0;
  $66 = (($65) + 24|0);
  $67 = HEAP32[$66>>2]|0;
  $68 = ($67|0)==(0|0);
  if ($68) {
   break;
  }
  $69 = $iEof;
  $70 = $69;
  $71 = HEAP32[$70>>2]|0;
  $72 = (($69) + 4)|0;
  $73 = $72;
  $74 = HEAP32[$73>>2]|0;
  $75 = HEAP32[$nKey>>2]|0;
  $76 = ($75|0)<(0);
  $77 = $76 << 31 >> 31;
  $78 = (_i64Add(($71|0),($74|0),($75|0),($77|0))|0);
  $79 = tempRet0;
  $80 = HEAP32[$nKey>>2]|0;
  $81 = ($80|0)<(0);
  $82 = $81 << 31 >> 31;
  $83 = (_sqlite3VarintLen($80,$82)|0);
  $84 = ($83|0)<(0);
  $85 = $84 << 31 >> 31;
  $86 = (_i64Add(($78|0),($79|0),($83|0),($85|0))|0);
  $87 = tempRet0;
  $88 = $iStart;
  $89 = $88;
  $90 = HEAP32[$89>>2]|0;
  $91 = (($88) + 4)|0;
  $92 = $91;
  $93 = HEAP32[$92>>2]|0;
  $94 = HEAP32[$0>>2]|0;
  $95 = (($94) + 16|0);
  $96 = HEAP32[$95>>2]|0;
  $97 = ($96|0)<(0);
  $98 = $97 << 31 >> 31;
  $99 = (_i64Add(($90|0),($93|0),($96|0),($98|0))|0);
  $100 = tempRet0;
  $101 = ($87|0)>($100|0);
  $102 = ($86>>>0)>($99>>>0);
  $103 = ($87|0)==($100|0);
  $104 = $103 & $102;
  $105 = $101 | $104;
  if ($105) {
   break;
  }
  $106 = HEAP32[$nKey>>2]|0;
  $107 = ($106|0)<(0);
  $108 = $107 << 31 >> 31;
  _vdbePmaWriteVarint($writer,$106,$108);
  $109 = HEAP32[$pReader>>2]|0;
  $110 = (($109) + 32|0);
  $111 = HEAP32[$110>>2]|0;
  $112 = HEAP32[$nKey>>2]|0;
  _vdbePmaWriteBlob($writer,$111,$112);
  $113 = HEAP32[$0>>2]|0;
  $114 = (($113) + 4|0);
  $115 = HEAP32[$114>>2]|0;
  $116 = (_vdbeMergeEngineStep($115,$dummy)|0);
  HEAP32[$rc>>2] = $116;
 }
 $117 = HEAP32[$pOut>>2]|0;
 $118 = (($117) + 8|0);
 $119 = (_vdbePmaWriterFinish($writer,$118)|0);
 HEAP32[$rc2>>2] = $119;
 $120 = HEAP32[$rc>>2]|0;
 $121 = ($120|0)==(0);
 if (!($121)) {
  $123 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($123|0);
 }
 $122 = HEAP32[$rc2>>2]|0;
 HEAP32[$rc>>2] = $122;
 $123 = HEAP32[$rc>>2]|0;
 STACKTOP = sp;return ($123|0);
}
function _vdbeSorterSetupMerge($pSorter) {
 $pSorter = $pSorter|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $pMain = 0, $pTask0 = 0, $rc = 0;
 var label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $rc = sp + 8|0;
 $pTask0 = sp + 4|0;
 $pMain = sp;
 HEAP32[$0>>2] = $pSorter;
 $1 = HEAP32[$0>>2]|0;
 $2 = (($1) + 64|0);
 HEAP32[$pTask0>>2] = $2;
 HEAP32[$pMain>>2] = 0;
 $3 = HEAP32[$0>>2]|0;
 $4 = (_vdbeSorterMergeTreeBuild($3,$pMain)|0);
 HEAP32[$rc>>2] = $4;
 $5 = HEAP32[$rc>>2]|0;
 $6 = ($5|0)==(0);
 if ($6) {
  $7 = HEAP32[$pTask0>>2]|0;
  $8 = HEAP32[$pMain>>2]|0;
  $9 = (_vdbeMergeEngineInit($7,$8,0)|0);
  HEAP32[$rc>>2] = $9;
  $10 = HEAP32[$pMain>>2]|0;
  $11 = HEAP32[$0>>2]|0;
  $12 = (($11) + 20|0);
  HEAP32[$12>>2] = $10;
  HEAP32[$pMain>>2] = 0;
 }
 $13 = HEAP32[$rc>>2]|0;
 $14 = ($13|0)!=(0);
 if (!($14)) {
  $16 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($16|0);
 }
 $15 = HEAP32[$pMain>>2]|0;
 _vdbeMergeEngineFree($15);
 $16 = HEAP32[$rc>>2]|0;
 STACKTOP = sp;return ($16|0);
}
function _vdbeSorterMergeTreeBuild($pSorter,$ppOut) {
 $pSorter = $pSorter|0;
 $ppOut = $ppOut|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $8 = 0, $9 = 0, $i = 0, $iReadOff = 0;
 var $iSeq = 0, $iTask = 0, $nDepth = 0, $nReader = 0, $or$cond = 0, $pMain = 0, $pMerger = 0, $pRoot = 0, $pTask = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 64|0;
 $0 = sp + 52|0;
 $1 = sp + 48|0;
 $pMain = sp + 44|0;
 $rc = sp + 40|0;
 $iTask = sp + 36|0;
 $pTask = sp + 32|0;
 $pRoot = sp + 28|0;
 $nDepth = sp + 24|0;
 $iReadOff = sp;
 $i = sp + 20|0;
 $iSeq = sp + 16|0;
 $pMerger = sp + 12|0;
 $nReader = sp + 8|0;
 HEAP32[$0>>2] = $pSorter;
 HEAP32[$1>>2] = $ppOut;
 HEAP32[$pMain>>2] = 0;
 HEAP32[$rc>>2] = 0;
 HEAP32[$iTask>>2] = 0;
 while(1) {
  $2 = HEAP32[$rc>>2]|0;
  $3 = ($2|0)==(0);
  if (!($3)) {
   break;
  }
  $4 = HEAP32[$iTask>>2]|0;
  $5 = HEAP32[$0>>2]|0;
  $6 = (($5) + 59|0);
  $7 = HEAP8[$6>>0]|0;
  $8 = $7&255;
  $9 = ($4|0)<($8|0);
  if (!($9)) {
   break;
  }
  $10 = HEAP32[$iTask>>2]|0;
  $11 = HEAP32[$0>>2]|0;
  $12 = (($11) + 64|0);
  $13 = (($12) + ($10<<6)|0);
  HEAP32[$pTask>>2] = $13;
  HEAP32[$pRoot>>2] = 0;
  $14 = HEAP32[$pTask>>2]|0;
  $15 = (($14) + 28|0);
  $16 = HEAP32[$15>>2]|0;
  $17 = (_vdbeSorterTreeDepth($16)|0);
  HEAP32[$nDepth>>2] = $17;
  $18 = $iReadOff;
  $19 = $18;
  HEAP32[$19>>2] = 0;
  $20 = (($18) + 4)|0;
  $21 = $20;
  HEAP32[$21>>2] = 0;
  $22 = HEAP32[$pTask>>2]|0;
  $23 = (($22) + 28|0);
  $24 = HEAP32[$23>>2]|0;
  $25 = ($24|0)<=(16);
  L5: do {
   if ($25) {
    $26 = HEAP32[$pTask>>2]|0;
    $27 = HEAP32[$pTask>>2]|0;
    $28 = (($27) + 28|0);
    $29 = HEAP32[$28>>2]|0;
    $30 = (_vdbeMergeEngineLevel0($26,$29,$iReadOff,$pRoot)|0);
    HEAP32[$rc>>2] = $30;
   } else {
    HEAP32[$iSeq>>2] = 0;
    $31 = (_vdbeMergeEngineNew(16)|0);
    HEAP32[$pRoot>>2] = $31;
    $32 = HEAP32[$pRoot>>2]|0;
    $33 = ($32|0)==(0|0);
    if ($33) {
     HEAP32[$rc>>2] = 7;
    }
    HEAP32[$i>>2] = 0;
    while(1) {
     $34 = HEAP32[$i>>2]|0;
     $35 = HEAP32[$pTask>>2]|0;
     $36 = (($35) + 28|0);
     $37 = HEAP32[$36>>2]|0;
     $38 = ($34|0)<($37|0);
     $39 = HEAP32[$rc>>2]|0;
     $40 = ($39|0)==(0);
     $or$cond = $38 & $40;
     if (!($or$cond)) {
      break L5;
     }
     HEAP32[$pMerger>>2] = 0;
     $41 = HEAP32[$pTask>>2]|0;
     $42 = (($41) + 28|0);
     $43 = HEAP32[$42>>2]|0;
     $44 = HEAP32[$i>>2]|0;
     $45 = (($43) - ($44))|0;
     $46 = ($45|0)<(16);
     if ($46) {
      $47 = HEAP32[$pTask>>2]|0;
      $48 = (($47) + 28|0);
      $49 = HEAP32[$48>>2]|0;
      $50 = HEAP32[$i>>2]|0;
      $51 = (($49) - ($50))|0;
      $52 = $51;
     } else {
      $52 = 16;
     }
     HEAP32[$nReader>>2] = $52;
     $53 = HEAP32[$pTask>>2]|0;
     $54 = HEAP32[$nReader>>2]|0;
     $55 = (_vdbeMergeEngineLevel0($53,$54,$iReadOff,$pMerger)|0);
     HEAP32[$rc>>2] = $55;
     $56 = HEAP32[$rc>>2]|0;
     $57 = ($56|0)==(0);
     if ($57) {
      $58 = HEAP32[$pTask>>2]|0;
      $59 = HEAP32[$nDepth>>2]|0;
      $60 = HEAP32[$iSeq>>2]|0;
      $61 = (($60) + 1)|0;
      HEAP32[$iSeq>>2] = $61;
      $62 = HEAP32[$pRoot>>2]|0;
      $63 = HEAP32[$pMerger>>2]|0;
      $64 = (_vdbeSorterAddToTree($58,$59,$60,$62,$63)|0);
      HEAP32[$rc>>2] = $64;
     }
     $65 = HEAP32[$i>>2]|0;
     $66 = (($65) + 16)|0;
     HEAP32[$i>>2] = $66;
    }
   }
  } while(0);
  $67 = HEAP32[$rc>>2]|0;
  $68 = ($67|0)==(0);
  $69 = HEAP32[$pRoot>>2]|0;
  if ($68) {
   HEAP32[$pMain>>2] = $69;
  } else {
   _vdbeMergeEngineFree($69);
  }
  $70 = HEAP32[$iTask>>2]|0;
  $71 = (($70) + 1)|0;
  HEAP32[$iTask>>2] = $71;
 }
 $72 = HEAP32[$rc>>2]|0;
 $73 = ($72|0)!=(0);
 if (!($73)) {
  $75 = HEAP32[$pMain>>2]|0;
  $76 = HEAP32[$1>>2]|0;
  HEAP32[$76>>2] = $75;
  $77 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($77|0);
 }
 $74 = HEAP32[$pMain>>2]|0;
 _vdbeMergeEngineFree($74);
 HEAP32[$pMain>>2] = 0;
 $75 = HEAP32[$pMain>>2]|0;
 $76 = HEAP32[$1>>2]|0;
 HEAP32[$76>>2] = $75;
 $77 = HEAP32[$rc>>2]|0;
 STACKTOP = sp;return ($77|0);
}
function _vdbeMergeEngineInit($pTask,$pMerger,$eMode) {
 $pTask = $pTask|0;
 $pMerger = $pMerger|0;
 $eMode = $eMode|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $i = 0;
 var $nTree = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 24|0;
 $1 = sp + 20|0;
 $2 = sp + 16|0;
 $3 = sp + 12|0;
 $rc = sp + 8|0;
 $i = sp + 4|0;
 $nTree = sp;
 HEAP32[$1>>2] = $pTask;
 HEAP32[$2>>2] = $pMerger;
 HEAP32[$3>>2] = $eMode;
 HEAP32[$rc>>2] = 0;
 $4 = HEAP32[$2>>2]|0;
 $5 = HEAP32[$4>>2]|0;
 HEAP32[$nTree>>2] = $5;
 $6 = HEAP32[$1>>2]|0;
 $7 = HEAP32[$2>>2]|0;
 $8 = (($7) + 4|0);
 HEAP32[$8>>2] = $6;
 HEAP32[$i>>2] = 0;
 while(1) {
  $9 = HEAP32[$i>>2]|0;
  $10 = HEAP32[$nTree>>2]|0;
  $11 = ($9|0)<($10|0);
  if (!($11)) {
   break;
  }
  $12 = HEAP32[$i>>2]|0;
  $13 = HEAP32[$2>>2]|0;
  $14 = (($13) + 12|0);
  $15 = HEAP32[$14>>2]|0;
  $16 = (($15) + (($12*56)|0)|0);
  $17 = (_vdbePmaReaderIncrMergeInit($16,0)|0);
  HEAP32[$rc>>2] = $17;
  $18 = HEAP32[$rc>>2]|0;
  $19 = ($18|0)!=(0);
  if ($19) {
   label = 4;
   break;
  }
  $21 = HEAP32[$i>>2]|0;
  $22 = (($21) + 1)|0;
  HEAP32[$i>>2] = $22;
 }
 if ((label|0) == 4) {
  $20 = HEAP32[$rc>>2]|0;
  HEAP32[$0>>2] = $20;
  $38 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($38|0);
 }
 $23 = HEAP32[$2>>2]|0;
 $24 = HEAP32[$23>>2]|0;
 $25 = (($24) - 1)|0;
 HEAP32[$i>>2] = $25;
 while(1) {
  $26 = HEAP32[$i>>2]|0;
  $27 = ($26|0)>(0);
  if (!($27)) {
   break;
  }
  $28 = HEAP32[$2>>2]|0;
  $29 = HEAP32[$i>>2]|0;
  _vdbeMergeEngineCompare($28,$29);
  $30 = HEAP32[$i>>2]|0;
  $31 = (($30) + -1)|0;
  HEAP32[$i>>2] = $31;
 }
 $32 = HEAP32[$1>>2]|0;
 $33 = (($32) + 12|0);
 $34 = HEAP32[$33>>2]|0;
 $35 = (($34) + 7|0);
 $36 = HEAP8[$35>>0]|0;
 $37 = $36&255;
 HEAP32[$0>>2] = $37;
 $38 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($38|0);
}
function _vdbePmaReaderIncrMergeInit($pReadr,$eMode) {
 $pReadr = $pReadr|0;
 $eMode = $eMode|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0;
 var $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $db = 0, $mxSz = 0, $pIncr = 0, $pTask = 0, $rc = 0;
 var label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 24|0;
 $1 = sp + 20|0;
 $rc = sp + 16|0;
 $pIncr = sp + 12|0;
 $pTask = sp + 8|0;
 $db = sp + 4|0;
 $mxSz = sp;
 HEAP32[$0>>2] = $pReadr;
 HEAP32[$1>>2] = $eMode;
 HEAP32[$rc>>2] = 0;
 $2 = HEAP32[$0>>2]|0;
 $3 = (($2) + 48|0);
 $4 = HEAP32[$3>>2]|0;
 HEAP32[$pIncr>>2] = $4;
 $5 = HEAP32[$pIncr>>2]|0;
 $6 = ($5|0)!=(0|0);
 if (!($6)) {
  $94 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($94|0);
 }
 $7 = HEAP32[$pIncr>>2]|0;
 $8 = HEAP32[$7>>2]|0;
 HEAP32[$pTask>>2] = $8;
 $9 = HEAP32[$pTask>>2]|0;
 $10 = (($9) + 8|0);
 $11 = HEAP32[$10>>2]|0;
 $12 = (($11) + 24|0);
 $13 = HEAP32[$12>>2]|0;
 HEAP32[$db>>2] = $13;
 $14 = HEAP32[$pTask>>2]|0;
 $15 = HEAP32[$pIncr>>2]|0;
 $16 = (($15) + 4|0);
 $17 = HEAP32[$16>>2]|0;
 $18 = HEAP32[$1>>2]|0;
 $19 = (_vdbeMergeEngineInit($14,$17,$18)|0);
 HEAP32[$rc>>2] = $19;
 $20 = HEAP32[$rc>>2]|0;
 $21 = ($20|0)==(0);
 if ($21) {
  $22 = HEAP32[$pIncr>>2]|0;
  $23 = (($22) + 16|0);
  $24 = HEAP32[$23>>2]|0;
  HEAP32[$mxSz>>2] = $24;
  $25 = HEAP32[$pTask>>2]|0;
  $26 = (($25) + 48|0);
  $27 = HEAP32[$26>>2]|0;
  $28 = ($27|0)==(0|0);
  if ($28) {
   $29 = HEAP32[$db>>2]|0;
   $30 = HEAP32[$pTask>>2]|0;
   $31 = (($30) + 48|0);
   $32 = (($31) + 8|0);
   $33 = $32;
   $34 = $33;
   $35 = HEAP32[$34>>2]|0;
   $36 = (($33) + 4)|0;
   $37 = $36;
   $38 = HEAP32[$37>>2]|0;
   $39 = HEAP32[$pTask>>2]|0;
   $40 = (($39) + 48|0);
   $41 = (_vdbeSorterOpenTempFile($29,$35,$38,$40)|0);
   HEAP32[$rc>>2] = $41;
   $42 = HEAP32[$pTask>>2]|0;
   $43 = (($42) + 48|0);
   $44 = (($43) + 8|0);
   $45 = $44;
   $46 = $45;
   HEAP32[$46>>2] = 0;
   $47 = (($45) + 4)|0;
   $48 = $47;
   HEAP32[$48>>2] = 0;
  }
  $49 = HEAP32[$rc>>2]|0;
  $50 = ($49|0)==(0);
  if ($50) {
   $51 = HEAP32[$pTask>>2]|0;
   $52 = (($51) + 48|0);
   $53 = HEAP32[$52>>2]|0;
   $54 = HEAP32[$pIncr>>2]|0;
   $55 = (($54) + 32|0);
   $56 = (($55) + 16|0);
   HEAP32[$56>>2] = $53;
   $57 = HEAP32[$pTask>>2]|0;
   $58 = (($57) + 48|0);
   $59 = (($58) + 8|0);
   $60 = $59;
   $61 = $60;
   $62 = HEAP32[$61>>2]|0;
   $63 = (($60) + 4)|0;
   $64 = $63;
   $65 = HEAP32[$64>>2]|0;
   $66 = HEAP32[$pIncr>>2]|0;
   $67 = (($66) + 8|0);
   $68 = $67;
   $69 = $68;
   HEAP32[$69>>2] = $62;
   $70 = (($68) + 4)|0;
   $71 = $70;
   HEAP32[$71>>2] = $65;
   $72 = HEAP32[$mxSz>>2]|0;
   $73 = ($72|0)<(0);
   $74 = $73 << 31 >> 31;
   $75 = HEAP32[$pTask>>2]|0;
   $76 = (($75) + 48|0);
   $77 = (($76) + 8|0);
   $78 = $77;
   $79 = $78;
   $80 = HEAP32[$79>>2]|0;
   $81 = (($78) + 4)|0;
   $82 = $81;
   $83 = HEAP32[$82>>2]|0;
   $84 = (_i64Add(($80|0),($83|0),($72|0),($74|0))|0);
   $85 = tempRet0;
   $86 = $77;
   $87 = $86;
   HEAP32[$87>>2] = $84;
   $88 = (($86) + 4)|0;
   $89 = $88;
   HEAP32[$89>>2] = $85;
  }
 }
 $90 = HEAP32[$rc>>2]|0;
 $91 = ($90|0)==(0);
 if (!($91)) {
  $94 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($94|0);
 }
 $92 = HEAP32[$0>>2]|0;
 $93 = (_vdbePmaReaderNext($92)|0);
 HEAP32[$rc>>2] = $93;
 $94 = HEAP32[$rc>>2]|0;
 STACKTOP = sp;return ($94|0);
}
function _vdbeMergeEngineCompare($pMerger,$iOut) {
 $pMerger = $pMerger|0;
 $iOut = $iOut|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $8 = 0, $9 = 0, $i1 = 0, $i2 = 0, $iRes = 0, $p1 = 0, $p2 = 0;
 var $res = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 28|0;
 $1 = sp + 24|0;
 $i1 = sp + 20|0;
 $i2 = sp + 16|0;
 $iRes = sp + 12|0;
 $p1 = sp + 8|0;
 $p2 = sp + 4|0;
 $res = sp;
 HEAP32[$0>>2] = $pMerger;
 HEAP32[$1>>2] = $iOut;
 $2 = HEAP32[$1>>2]|0;
 $3 = HEAP32[$0>>2]|0;
 $4 = HEAP32[$3>>2]|0;
 $5 = (($4|0) / 2)&-1;
 $6 = ($2|0)>=($5|0);
 $7 = HEAP32[$1>>2]|0;
 if ($6) {
  $8 = HEAP32[$0>>2]|0;
  $9 = HEAP32[$8>>2]|0;
  $10 = (($9|0) / 2)&-1;
  $11 = (($7) - ($10))|0;
  $12 = $11<<1;
  HEAP32[$i1>>2] = $12;
  $13 = HEAP32[$i1>>2]|0;
  $14 = (($13) + 1)|0;
  HEAP32[$i2>>2] = $14;
 } else {
  $15 = $7<<1;
  $16 = HEAP32[$0>>2]|0;
  $17 = (($16) + 8|0);
  $18 = HEAP32[$17>>2]|0;
  $19 = (($18) + ($15<<2)|0);
  $20 = HEAP32[$19>>2]|0;
  HEAP32[$i1>>2] = $20;
  $21 = HEAP32[$1>>2]|0;
  $22 = $21<<1;
  $23 = (($22) + 1)|0;
  $24 = HEAP32[$0>>2]|0;
  $25 = (($24) + 8|0);
  $26 = HEAP32[$25>>2]|0;
  $27 = (($26) + ($23<<2)|0);
  $28 = HEAP32[$27>>2]|0;
  HEAP32[$i2>>2] = $28;
 }
 $29 = HEAP32[$i1>>2]|0;
 $30 = HEAP32[$0>>2]|0;
 $31 = (($30) + 12|0);
 $32 = HEAP32[$31>>2]|0;
 $33 = (($32) + (($29*56)|0)|0);
 HEAP32[$p1>>2] = $33;
 $34 = HEAP32[$i2>>2]|0;
 $35 = HEAP32[$0>>2]|0;
 $36 = (($35) + 12|0);
 $37 = HEAP32[$36>>2]|0;
 $38 = (($37) + (($34*56)|0)|0);
 HEAP32[$p2>>2] = $38;
 $39 = HEAP32[$p1>>2]|0;
 $40 = (($39) + 24|0);
 $41 = HEAP32[$40>>2]|0;
 $42 = ($41|0)==(0|0);
 do {
  if ($42) {
   $43 = HEAP32[$i2>>2]|0;
   HEAP32[$iRes>>2] = $43;
  } else {
   $44 = HEAP32[$p2>>2]|0;
   $45 = (($44) + 24|0);
   $46 = HEAP32[$45>>2]|0;
   $47 = ($46|0)==(0|0);
   if ($47) {
    $48 = HEAP32[$i1>>2]|0;
    HEAP32[$iRes>>2] = $48;
    break;
   }
   $49 = HEAP32[$0>>2]|0;
   $50 = (($49) + 4|0);
   $51 = HEAP32[$50>>2]|0;
   $52 = HEAP32[$p1>>2]|0;
   $53 = (($52) + 32|0);
   $54 = HEAP32[$53>>2]|0;
   $55 = HEAP32[$p1>>2]|0;
   $56 = (($55) + 20|0);
   $57 = HEAP32[$56>>2]|0;
   $58 = HEAP32[$p2>>2]|0;
   $59 = (($58) + 32|0);
   $60 = HEAP32[$59>>2]|0;
   $61 = HEAP32[$p2>>2]|0;
   $62 = (($61) + 20|0);
   $63 = HEAP32[$62>>2]|0;
   $64 = (_vdbeSorterCompare($51,$54,$57,$60,$63)|0);
   HEAP32[$res>>2] = $64;
   $65 = HEAP32[$res>>2]|0;
   $66 = ($65|0)<=(0);
   if ($66) {
    $67 = HEAP32[$i1>>2]|0;
    HEAP32[$iRes>>2] = $67;
    break;
   } else {
    $68 = HEAP32[$i2>>2]|0;
    HEAP32[$iRes>>2] = $68;
    break;
   }
  }
 } while(0);
 $69 = HEAP32[$iRes>>2]|0;
 $70 = HEAP32[$1>>2]|0;
 $71 = HEAP32[$0>>2]|0;
 $72 = (($71) + 8|0);
 $73 = HEAP32[$72>>2]|0;
 $74 = (($73) + ($70<<2)|0);
 HEAP32[$74>>2] = $69;
 STACKTOP = sp;return;
}
function _vdbeSorterTreeDepth($nPMA) {
 $nPMA = $nPMA|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $nDepth = 0, $nDiv = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $nDepth = sp + 8|0;
 $nDiv = sp;
 HEAP32[$0>>2] = $nPMA;
 HEAP32[$nDepth>>2] = 0;
 $1 = $nDiv;
 $2 = $1;
 HEAP32[$2>>2] = 16;
 $3 = (($1) + 4)|0;
 $4 = $3;
 HEAP32[$4>>2] = 0;
 while(1) {
  $5 = $nDiv;
  $6 = $5;
  $7 = HEAP32[$6>>2]|0;
  $8 = (($5) + 4)|0;
  $9 = $8;
  $10 = HEAP32[$9>>2]|0;
  $11 = HEAP32[$0>>2]|0;
  $12 = ($11|0)<(0);
  $13 = $12 << 31 >> 31;
  $14 = ($10|0)<($13|0);
  $15 = ($7>>>0)<($11>>>0);
  $16 = ($10|0)==($13|0);
  $17 = $16 & $15;
  $18 = $14 | $17;
  if (!($18)) {
   break;
  }
  $19 = $nDiv;
  $20 = $19;
  $21 = HEAP32[$20>>2]|0;
  $22 = (($19) + 4)|0;
  $23 = $22;
  $24 = HEAP32[$23>>2]|0;
  $25 = (___muldi3(($21|0),($24|0),16,0)|0);
  $26 = tempRet0;
  $27 = $nDiv;
  $28 = $27;
  HEAP32[$28>>2] = $25;
  $29 = (($27) + 4)|0;
  $30 = $29;
  HEAP32[$30>>2] = $26;
  $31 = HEAP32[$nDepth>>2]|0;
  $32 = (($31) + 1)|0;
  HEAP32[$nDepth>>2] = $32;
 }
 $33 = HEAP32[$nDepth>>2]|0;
 STACKTOP = sp;return ($33|0);
}
function _vdbeMergeEngineLevel0($pTask,$nPMA,$piOffset,$ppOut) {
 $pTask = $pTask|0;
 $nPMA = $nPMA|0;
 $piOffset = $piOffset|0;
 $ppOut = $ppOut|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $8 = 0, $9 = 0, $i = 0, $iOff = 0, $nDummy = 0, $or$cond = 0, $pNew = 0, $pReadr = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $0 = sp + 44|0;
 $1 = sp + 40|0;
 $2 = sp + 36|0;
 $3 = sp + 32|0;
 $pNew = sp + 28|0;
 $iOff = sp + 8|0;
 $i = sp + 24|0;
 $rc = sp + 20|0;
 $nDummy = sp;
 $pReadr = sp + 16|0;
 HEAP32[$0>>2] = $pTask;
 HEAP32[$1>>2] = $nPMA;
 HEAP32[$2>>2] = $piOffset;
 HEAP32[$3>>2] = $ppOut;
 $4 = HEAP32[$2>>2]|0;
 $5 = $4;
 $6 = $5;
 $7 = HEAP32[$6>>2]|0;
 $8 = (($5) + 4)|0;
 $9 = $8;
 $10 = HEAP32[$9>>2]|0;
 $11 = $iOff;
 $12 = $11;
 HEAP32[$12>>2] = $7;
 $13 = (($11) + 4)|0;
 $14 = $13;
 HEAP32[$14>>2] = $10;
 HEAP32[$rc>>2] = 0;
 $15 = HEAP32[$1>>2]|0;
 $16 = (_vdbeMergeEngineNew($15)|0);
 HEAP32[$pNew>>2] = $16;
 $17 = HEAP32[$3>>2]|0;
 HEAP32[$17>>2] = $16;
 $18 = HEAP32[$pNew>>2]|0;
 $19 = ($18|0)==(0|0);
 if ($19) {
  HEAP32[$rc>>2] = 7;
 }
 HEAP32[$i>>2] = 0;
 while(1) {
  $20 = HEAP32[$i>>2]|0;
  $21 = HEAP32[$1>>2]|0;
  $22 = ($20|0)<($21|0);
  $23 = HEAP32[$rc>>2]|0;
  $24 = ($23|0)==(0);
  $or$cond = $22 & $24;
  if (!($or$cond)) {
   break;
  }
  $25 = HEAP32[$i>>2]|0;
  $26 = HEAP32[$pNew>>2]|0;
  $27 = (($26) + 12|0);
  $28 = HEAP32[$27>>2]|0;
  $29 = (($28) + (($25*56)|0)|0);
  HEAP32[$pReadr>>2] = $29;
  $30 = HEAP32[$0>>2]|0;
  $31 = HEAP32[$0>>2]|0;
  $32 = (($31) + 32|0);
  $33 = $iOff;
  $34 = $33;
  $35 = HEAP32[$34>>2]|0;
  $36 = (($33) + 4)|0;
  $37 = $36;
  $38 = HEAP32[$37>>2]|0;
  $39 = HEAP32[$pReadr>>2]|0;
  $40 = (_vdbePmaReaderInit($30,$32,$35,$38,$39,$nDummy)|0);
  HEAP32[$rc>>2] = $40;
  $41 = HEAP32[$pReadr>>2]|0;
  $42 = (($41) + 8|0);
  $43 = $42;
  $44 = $43;
  $45 = HEAP32[$44>>2]|0;
  $46 = (($43) + 4)|0;
  $47 = $46;
  $48 = HEAP32[$47>>2]|0;
  $49 = $iOff;
  $50 = $49;
  HEAP32[$50>>2] = $45;
  $51 = (($49) + 4)|0;
  $52 = $51;
  HEAP32[$52>>2] = $48;
  $53 = HEAP32[$i>>2]|0;
  $54 = (($53) + 1)|0;
  HEAP32[$i>>2] = $54;
 }
 $55 = HEAP32[$rc>>2]|0;
 $56 = ($55|0)!=(0);
 if (!($56)) {
  $59 = $iOff;
  $60 = $59;
  $61 = HEAP32[$60>>2]|0;
  $62 = (($59) + 4)|0;
  $63 = $62;
  $64 = HEAP32[$63>>2]|0;
  $65 = HEAP32[$2>>2]|0;
  $66 = $65;
  $67 = $66;
  HEAP32[$67>>2] = $61;
  $68 = (($66) + 4)|0;
  $69 = $68;
  HEAP32[$69>>2] = $64;
  $70 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($70|0);
 }
 $57 = HEAP32[$pNew>>2]|0;
 _vdbeMergeEngineFree($57);
 $58 = HEAP32[$3>>2]|0;
 HEAP32[$58>>2] = 0;
 $59 = $iOff;
 $60 = $59;
 $61 = HEAP32[$60>>2]|0;
 $62 = (($59) + 4)|0;
 $63 = $62;
 $64 = HEAP32[$63>>2]|0;
 $65 = HEAP32[$2>>2]|0;
 $66 = $65;
 $67 = $66;
 HEAP32[$67>>2] = $61;
 $68 = (($66) + 4)|0;
 $69 = $68;
 HEAP32[$69>>2] = $64;
 $70 = HEAP32[$rc>>2]|0;
 STACKTOP = sp;return ($70|0);
}
function _vdbeMergeEngineNew($nReader) {
 $nReader = $nReader|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $N = 0, $nByte = 0, $pNew = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $N = sp + 8|0;
 $nByte = sp + 4|0;
 $pNew = sp;
 HEAP32[$0>>2] = $nReader;
 HEAP32[$N>>2] = 2;
 while(1) {
  $1 = HEAP32[$N>>2]|0;
  $2 = HEAP32[$0>>2]|0;
  $3 = ($1|0)<($2|0);
  $4 = HEAP32[$N>>2]|0;
  if (!($3)) {
   break;
  }
  $5 = HEAP32[$N>>2]|0;
  $6 = (($5) + ($4))|0;
  HEAP32[$N>>2] = $6;
 }
 $7 = ($4*60)|0;
 $8 = (16 + ($7))|0;
 HEAP32[$nByte>>2] = $8;
 $9 = (_sqlite3FaultSim(100)|0);
 $10 = ($9|0)!=(0);
 if ($10) {
  $15 = 0;
 } else {
  $11 = HEAP32[$nByte>>2]|0;
  $12 = ($11|0)<(0);
  $13 = $12 << 31 >> 31;
  $14 = (_sqlite3MallocZero($11,$13)|0);
  $15 = $14;
 }
 HEAP32[$pNew>>2] = $15;
 $16 = HEAP32[$pNew>>2]|0;
 $17 = ($16|0)!=(0|0);
 if (!($17)) {
  $33 = HEAP32[$pNew>>2]|0;
  STACKTOP = sp;return ($33|0);
 }
 $18 = HEAP32[$N>>2]|0;
 $19 = HEAP32[$pNew>>2]|0;
 HEAP32[$19>>2] = $18;
 $20 = HEAP32[$pNew>>2]|0;
 $21 = (($20) + 4|0);
 HEAP32[$21>>2] = 0;
 $22 = HEAP32[$pNew>>2]|0;
 $23 = (($22) + 16|0);
 $24 = HEAP32[$pNew>>2]|0;
 $25 = (($24) + 12|0);
 HEAP32[$25>>2] = $23;
 $26 = HEAP32[$N>>2]|0;
 $27 = HEAP32[$pNew>>2]|0;
 $28 = (($27) + 12|0);
 $29 = HEAP32[$28>>2]|0;
 $30 = (($29) + (($26*56)|0)|0);
 $31 = HEAP32[$pNew>>2]|0;
 $32 = (($31) + 8|0);
 HEAP32[$32>>2] = $30;
 $33 = HEAP32[$pNew>>2]|0;
 STACKTOP = sp;return ($33|0);
}
function _vdbeSorterAddToTree($pTask,$nDepth,$iSeq,$pRoot,$pLeaf) {
 $pTask = $pTask|0;
 $nDepth = $nDepth|0;
 $iSeq = $iSeq|0;
 $pRoot = $pRoot|0;
 $pLeaf = $pLeaf|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $7 = 0, $8 = 0, $9 = 0, $i = 0, $iIter = 0, $nDiv = 0, $or$cond = 0, $p = 0, $pIncr = 0, $pNew = 0, $pReadr = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 64|0;
 $0 = sp + 48|0;
 $1 = sp + 44|0;
 $2 = sp + 40|0;
 $3 = sp + 36|0;
 $4 = sp + 32|0;
 $rc = sp + 28|0;
 $nDiv = sp + 24|0;
 $i = sp + 20|0;
 $p = sp + 16|0;
 $pIncr = sp + 12|0;
 $iIter = sp + 8|0;
 $pReadr = sp + 4|0;
 $pNew = sp;
 HEAP32[$0>>2] = $pTask;
 HEAP32[$1>>2] = $nDepth;
 HEAP32[$2>>2] = $iSeq;
 HEAP32[$3>>2] = $pRoot;
 HEAP32[$4>>2] = $pLeaf;
 HEAP32[$rc>>2] = 0;
 HEAP32[$nDiv>>2] = 1;
 $5 = HEAP32[$3>>2]|0;
 HEAP32[$p>>2] = $5;
 $6 = HEAP32[$0>>2]|0;
 $7 = HEAP32[$4>>2]|0;
 $8 = (_vdbeIncrMergerNew($6,$7,$pIncr)|0);
 HEAP32[$rc>>2] = $8;
 HEAP32[$i>>2] = 1;
 while(1) {
  $9 = HEAP32[$i>>2]|0;
  $10 = HEAP32[$1>>2]|0;
  $11 = ($9|0)<($10|0);
  if (!($11)) {
   break;
  }
  $12 = HEAP32[$nDiv>>2]|0;
  $13 = $12<<4;
  HEAP32[$nDiv>>2] = $13;
  $14 = HEAP32[$i>>2]|0;
  $15 = (($14) + 1)|0;
  HEAP32[$i>>2] = $15;
 }
 HEAP32[$i>>2] = 1;
 while(1) {
  $16 = HEAP32[$i>>2]|0;
  $17 = HEAP32[$1>>2]|0;
  $18 = ($16|0)<($17|0);
  $19 = HEAP32[$rc>>2]|0;
  $20 = ($19|0)==(0);
  $or$cond = $18 & $20;
  if (!($or$cond)) {
   break;
  }
  $21 = HEAP32[$2>>2]|0;
  $22 = HEAP32[$nDiv>>2]|0;
  $23 = (($21|0) / ($22|0))&-1;
  $24 = (($23|0) % 16)&-1;
  HEAP32[$iIter>>2] = $24;
  $25 = HEAP32[$iIter>>2]|0;
  $26 = HEAP32[$p>>2]|0;
  $27 = (($26) + 12|0);
  $28 = HEAP32[$27>>2]|0;
  $29 = (($28) + (($25*56)|0)|0);
  HEAP32[$pReadr>>2] = $29;
  $30 = HEAP32[$pReadr>>2]|0;
  $31 = (($30) + 48|0);
  $32 = HEAP32[$31>>2]|0;
  $33 = ($32|0)==(0|0);
  do {
   if ($33) {
    $34 = (_vdbeMergeEngineNew(16)|0);
    HEAP32[$pNew>>2] = $34;
    $35 = HEAP32[$pNew>>2]|0;
    $36 = ($35|0)==(0|0);
    if ($36) {
     HEAP32[$rc>>2] = 7;
     break;
    } else {
     $37 = HEAP32[$0>>2]|0;
     $38 = HEAP32[$pNew>>2]|0;
     $39 = HEAP32[$pReadr>>2]|0;
     $40 = (($39) + 48|0);
     $41 = (_vdbeIncrMergerNew($37,$38,$40)|0);
     HEAP32[$rc>>2] = $41;
     break;
    }
   }
  } while(0);
  $42 = HEAP32[$rc>>2]|0;
  $43 = ($42|0)==(0);
  if ($43) {
   $44 = HEAP32[$pReadr>>2]|0;
   $45 = (($44) + 48|0);
   $46 = HEAP32[$45>>2]|0;
   $47 = (($46) + 4|0);
   $48 = HEAP32[$47>>2]|0;
   HEAP32[$p>>2] = $48;
   $49 = HEAP32[$nDiv>>2]|0;
   $50 = (($49|0) / 16)&-1;
   HEAP32[$nDiv>>2] = $50;
  }
  $51 = HEAP32[$i>>2]|0;
  $52 = (($51) + 1)|0;
  HEAP32[$i>>2] = $52;
 }
 $53 = HEAP32[$rc>>2]|0;
 $54 = ($53|0)==(0);
 $55 = HEAP32[$pIncr>>2]|0;
 if ($54) {
  $56 = HEAP32[$2>>2]|0;
  $57 = (($56|0) % 16)&-1;
  $58 = HEAP32[$p>>2]|0;
  $59 = (($58) + 12|0);
  $60 = HEAP32[$59>>2]|0;
  $61 = (($60) + (($57*56)|0)|0);
  $62 = (($61) + 48|0);
  HEAP32[$62>>2] = $55;
  $63 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($63|0);
 } else {
  _vdbeIncrFree($55);
  $63 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($63|0);
 }
 return 0|0;
}
function _vdbeIncrMergerNew($pTask,$pMerger,$ppOut) {
 $pTask = $pTask|0;
 $pMerger = $pMerger|0;
 $ppOut = $ppOut|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $7 = 0, $8 = 0;
 var $9 = 0, $pIncr = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 16|0;
 $1 = sp + 12|0;
 $2 = sp + 8|0;
 $rc = sp + 4|0;
 $pIncr = sp;
 HEAP32[$0>>2] = $pTask;
 HEAP32[$1>>2] = $pMerger;
 HEAP32[$2>>2] = $ppOut;
 HEAP32[$rc>>2] = 0;
 $3 = (_sqlite3FaultSim(100)|0);
 $4 = ($3|0)!=(0);
 if ($4) {
  $7 = 0;
 } else {
  $5 = (_sqlite3MallocZero(64,0)|0);
  $7 = $5;
 }
 $6 = HEAP32[$2>>2]|0;
 HEAP32[$6>>2] = $7;
 HEAP32[$pIncr>>2] = $7;
 $8 = HEAP32[$pIncr>>2]|0;
 $9 = ($8|0)!=(0|0);
 $10 = HEAP32[$1>>2]|0;
 if (!($9)) {
  _vdbeMergeEngineFree($10);
  HEAP32[$rc>>2] = 7;
  $60 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($60|0);
 }
 $11 = HEAP32[$pIncr>>2]|0;
 $12 = (($11) + 4|0);
 HEAP32[$12>>2] = $10;
 $13 = HEAP32[$0>>2]|0;
 $14 = HEAP32[$pIncr>>2]|0;
 HEAP32[$14>>2] = $13;
 $15 = HEAP32[$0>>2]|0;
 $16 = (($15) + 8|0);
 $17 = HEAP32[$16>>2]|0;
 $18 = (($17) + 8|0);
 $19 = HEAP32[$18>>2]|0;
 $20 = (($19) + 9)|0;
 $21 = HEAP32[$0>>2]|0;
 $22 = (($21) + 8|0);
 $23 = HEAP32[$22>>2]|0;
 $24 = (($23) + 4|0);
 $25 = HEAP32[$24>>2]|0;
 $26 = (($25|0) / 2)&-1;
 $27 = ($20|0)>($26|0);
 $28 = HEAP32[$0>>2]|0;
 $29 = (($28) + 8|0);
 $30 = HEAP32[$29>>2]|0;
 if ($27) {
  $31 = (($30) + 8|0);
  $32 = HEAP32[$31>>2]|0;
  $33 = (($32) + 9)|0;
  $39 = $33;
 } else {
  $34 = (($30) + 4|0);
  $35 = HEAP32[$34>>2]|0;
  $36 = (($35|0) / 2)&-1;
  $39 = $36;
 }
 $37 = HEAP32[$pIncr>>2]|0;
 $38 = (($37) + 16|0);
 HEAP32[$38>>2] = $39;
 $40 = HEAP32[$pIncr>>2]|0;
 $41 = (($40) + 16|0);
 $42 = HEAP32[$41>>2]|0;
 $43 = ($42|0)<(0);
 $44 = $43 << 31 >> 31;
 $45 = HEAP32[$0>>2]|0;
 $46 = (($45) + 48|0);
 $47 = (($46) + 8|0);
 $48 = $47;
 $49 = $48;
 $50 = HEAP32[$49>>2]|0;
 $51 = (($48) + 4)|0;
 $52 = $51;
 $53 = HEAP32[$52>>2]|0;
 $54 = (_i64Add(($50|0),($53|0),($42|0),($44|0))|0);
 $55 = tempRet0;
 $56 = $47;
 $57 = $56;
 HEAP32[$57>>2] = $54;
 $58 = (($56) + 4)|0;
 $59 = $58;
 HEAP32[$59>>2] = $55;
 $60 = HEAP32[$rc>>2]|0;
 STACKTOP = sp;return ($60|0);
}
function _vdbePmaReaderInit($pTask,$pFile,$0,$1,$pReadr,$pnByte) {
 $pTask = $pTask|0;
 $pFile = $pFile|0;
 $0 = $0|0;
 $1 = $1|0;
 $pReadr = $pReadr|0;
 $pnByte = $pnByte|0;
 var $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0;
 var $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0;
 var $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0;
 var $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $8 = 0, $9 = 0, $nByte = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $2 = sp + 32|0;
 $3 = sp + 28|0;
 $4 = sp + 8|0;
 $5 = sp + 24|0;
 $6 = sp + 20|0;
 $rc = sp + 16|0;
 $nByte = sp;
 HEAP32[$2>>2] = $pTask;
 HEAP32[$3>>2] = $pFile;
 $7 = $4;
 $8 = $7;
 HEAP32[$8>>2] = $0;
 $9 = (($7) + 4)|0;
 $10 = $9;
 HEAP32[$10>>2] = $1;
 HEAP32[$5>>2] = $pReadr;
 HEAP32[$6>>2] = $pnByte;
 $11 = HEAP32[$2>>2]|0;
 $12 = HEAP32[$5>>2]|0;
 $13 = HEAP32[$3>>2]|0;
 $14 = $4;
 $15 = $14;
 $16 = HEAP32[$15>>2]|0;
 $17 = (($14) + 4)|0;
 $18 = $17;
 $19 = HEAP32[$18>>2]|0;
 $20 = (_vdbePmaReaderSeek($11,$12,$13,$16,$19)|0);
 HEAP32[$rc>>2] = $20;
 $21 = HEAP32[$rc>>2]|0;
 $22 = ($21|0)==(0);
 if ($22) {
  $23 = HEAP32[$5>>2]|0;
  $24 = (_vdbePmaReadVarint($23,$nByte)|0);
  HEAP32[$rc>>2] = $24;
  $25 = HEAP32[$5>>2]|0;
  $26 = $25;
  $27 = $26;
  $28 = HEAP32[$27>>2]|0;
  $29 = (($26) + 4)|0;
  $30 = $29;
  $31 = HEAP32[$30>>2]|0;
  $32 = $nByte;
  $33 = $32;
  $34 = HEAP32[$33>>2]|0;
  $35 = (($32) + 4)|0;
  $36 = $35;
  $37 = HEAP32[$36>>2]|0;
  $38 = (_i64Add(($28|0),($31|0),($34|0),($37|0))|0);
  $39 = tempRet0;
  $40 = HEAP32[$5>>2]|0;
  $41 = (($40) + 8|0);
  $42 = $41;
  $43 = $42;
  HEAP32[$43>>2] = $38;
  $44 = (($42) + 4)|0;
  $45 = $44;
  HEAP32[$45>>2] = $39;
  $46 = $nByte;
  $47 = $46;
  $48 = HEAP32[$47>>2]|0;
  $49 = (($46) + 4)|0;
  $50 = $49;
  $51 = HEAP32[$50>>2]|0;
  $52 = HEAP32[$6>>2]|0;
  $53 = $52;
  $54 = $53;
  $55 = HEAP32[$54>>2]|0;
  $56 = (($53) + 4)|0;
  $57 = $56;
  $58 = HEAP32[$57>>2]|0;
  $59 = (_i64Add(($55|0),($58|0),($48|0),($51|0))|0);
  $60 = tempRet0;
  $61 = $52;
  $62 = $61;
  HEAP32[$62>>2] = $59;
  $63 = (($61) + 4)|0;
  $64 = $63;
  HEAP32[$64>>2] = $60;
 }
 $65 = HEAP32[$rc>>2]|0;
 $66 = ($65|0)==(0);
 if (!($66)) {
  $69 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($69|0);
 }
 $67 = HEAP32[$5>>2]|0;
 $68 = (_vdbePmaReaderNext($67)|0);
 HEAP32[$rc>>2] = $68;
 $69 = HEAP32[$rc>>2]|0;
 STACKTOP = sp;return ($69|0);
}
function _sqlite3BtreeCursorHasMoved($pCur) {
 $pCur = $pCur|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 HEAP32[$0>>2] = $pCur;
 $1 = HEAP32[$0>>2]|0;
 $2 = (($1) + 73|0);
 $3 = HEAP8[$2>>0]|0;
 $4 = $3&255;
 $5 = ($4|0)!=(1);
 $6 = $5&1;
 STACKTOP = sp;return ($6|0);
}
function _handleMovedCursor($p) {
 $p = $p|0;
 var $0 = 0, $1 = 0, $10 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $isDifferentRow = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $isDifferentRow = sp + 4|0;
 $rc = sp;
 HEAP32[$0>>2] = $p;
 $1 = HEAP32[$0>>2]|0;
 $2 = HEAP32[$1>>2]|0;
 $3 = (_sqlite3BtreeCursorRestore($2,$isDifferentRow)|0);
 HEAP32[$rc>>2] = $3;
 $4 = HEAP32[$0>>2]|0;
 $5 = (($4) + 60|0);
 HEAP32[$5>>2] = 0;
 $6 = HEAP32[$isDifferentRow>>2]|0;
 $7 = ($6|0)!=(0);
 if (!($7)) {
  $10 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($10|0);
 }
 $8 = HEAP32[$0>>2]|0;
 $9 = (($8) + 25|0);
 HEAP8[$9>>0] = 1;
 $10 = HEAP32[$rc>>2]|0;
 STACKTOP = sp;return ($10|0);
}
function _sqlite3BtreeCursorRestore($pCur,$pDifferentRow) {
 $pCur = $pCur|0;
 $pDifferentRow = $pDifferentRow|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $1 = sp + 8|0;
 $2 = sp + 4|0;
 $rc = sp;
 HEAP32[$1>>2] = $pCur;
 HEAP32[$2>>2] = $pDifferentRow;
 $3 = HEAP32[$1>>2]|0;
 $4 = (($3) + 73|0);
 $5 = HEAP8[$4>>0]|0;
 $6 = $5&255;
 $7 = ($6|0)>=(3);
 if ($7) {
  $8 = HEAP32[$1>>2]|0;
  $9 = (_btreeRestoreCursorPosition($8)|0);
  $10 = $9;
 } else {
  $10 = 0;
 }
 HEAP32[$rc>>2] = $10;
 $11 = HEAP32[$rc>>2]|0;
 $12 = ($11|0)!=(0);
 if ($12) {
  $13 = HEAP32[$2>>2]|0;
  HEAP32[$13>>2] = 1;
  $14 = HEAP32[$rc>>2]|0;
  HEAP32[$0>>2] = $14;
  $26 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($26|0);
 }
 $15 = HEAP32[$1>>2]|0;
 $16 = (($15) + 73|0);
 $17 = HEAP8[$16>>0]|0;
 $18 = $17&255;
 $19 = ($18|0)!=(1);
 if ($19) {
  label = 7;
 } else {
  $20 = HEAP32[$1>>2]|0;
  $21 = (($20) + 68|0);
  $22 = HEAP32[$21>>2]|0;
  $23 = ($22|0)!=(0);
  if ($23) {
   label = 7;
  } else {
   $25 = HEAP32[$2>>2]|0;
   HEAP32[$25>>2] = 0;
  }
 }
 if ((label|0) == 7) {
  $24 = HEAP32[$2>>2]|0;
  HEAP32[$24>>2] = 1;
 }
 HEAP32[$0>>2] = 0;
 $26 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($26|0);
}
function _vdbeSorterRowkey($pSorter,$pnKey) {
 $pSorter = $pSorter|0;
 $pnKey = $pnKey|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $pKey = 0, $pReader = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $1 = sp + 8|0;
 $pKey = sp + 4|0;
 $pReader = sp;
 HEAP32[$0>>2] = $pSorter;
 HEAP32[$1>>2] = $pnKey;
 $2 = HEAP32[$0>>2]|0;
 $3 = (($2) + 56|0);
 $4 = HEAP8[$3>>0]|0;
 $5 = ($4<<24>>24)!=(0);
 $6 = HEAP32[$0>>2]|0;
 if ($5) {
  $7 = (($6) + 20|0);
  $8 = HEAP32[$7>>2]|0;
  $9 = (($8) + 8|0);
  $10 = HEAP32[$9>>2]|0;
  $11 = (($10) + 4|0);
  $12 = HEAP32[$11>>2]|0;
  $13 = HEAP32[$0>>2]|0;
  $14 = (($13) + 20|0);
  $15 = HEAP32[$14>>2]|0;
  $16 = (($15) + 12|0);
  $17 = HEAP32[$16>>2]|0;
  $18 = (($17) + (($12*56)|0)|0);
  HEAP32[$pReader>>2] = $18;
  $19 = HEAP32[$pReader>>2]|0;
  $20 = (($19) + 20|0);
  $21 = HEAP32[$20>>2]|0;
  $22 = HEAP32[$1>>2]|0;
  HEAP32[$22>>2] = $21;
  $23 = HEAP32[$pReader>>2]|0;
  $24 = (($23) + 32|0);
  $25 = HEAP32[$24>>2]|0;
  HEAP32[$pKey>>2] = $25;
  $34 = HEAP32[$pKey>>2]|0;
  STACKTOP = sp;return ($34|0);
 } else {
  $26 = (($6) + 36|0);
  $27 = HEAP32[$26>>2]|0;
  $28 = HEAP32[$27>>2]|0;
  $29 = HEAP32[$1>>2]|0;
  HEAP32[$29>>2] = $28;
  $30 = HEAP32[$0>>2]|0;
  $31 = (($30) + 36|0);
  $32 = HEAP32[$31>>2]|0;
  $33 = (($32) + 8|0);
  HEAP32[$pKey>>2] = $33;
  $34 = HEAP32[$pKey>>2]|0;
  STACKTOP = sp;return ($34|0);
 }
 return 0|0;
}
function _dropCell($pPage,$idx,$sz,$pRC) {
 $pPage = $pPage|0;
 $idx = $idx|0;
 $sz = $sz|0;
 $pRC = $pRC|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0;
 var $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0;
 var $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0;
 var $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0;
 var $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0;
 var $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $data = 0, $hdr = 0, $pc = 0, $ptr = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $0 = sp + 32|0;
 $1 = sp + 28|0;
 $2 = sp + 24|0;
 $3 = sp + 20|0;
 $pc = sp + 16|0;
 $data = sp + 12|0;
 $ptr = sp + 8|0;
 $rc = sp + 4|0;
 $hdr = sp;
 HEAP32[$0>>2] = $pPage;
 HEAP32[$1>>2] = $idx;
 HEAP32[$2>>2] = $sz;
 HEAP32[$3>>2] = $pRC;
 $4 = HEAP32[$3>>2]|0;
 $5 = HEAP32[$4>>2]|0;
 $6 = ($5|0)!=(0);
 if ($6) {
  STACKTOP = sp;return;
 }
 $7 = HEAP32[$0>>2]|0;
 $8 = (($7) + 56|0);
 $9 = HEAP32[$8>>2]|0;
 HEAP32[$data>>2] = $9;
 $10 = HEAP32[$1>>2]|0;
 $11 = $10<<1;
 $12 = HEAP32[$0>>2]|0;
 $13 = (($12) + 64|0);
 $14 = HEAP32[$13>>2]|0;
 $15 = (($14) + ($11)|0);
 HEAP32[$ptr>>2] = $15;
 $16 = HEAP32[$ptr>>2]|0;
 $17 = HEAP8[$16>>0]|0;
 $18 = $17&255;
 $19 = $18 << 8;
 $20 = HEAP32[$ptr>>2]|0;
 $21 = (($20) + 1|0);
 $22 = HEAP8[$21>>0]|0;
 $23 = $22&255;
 $24 = $19 | $23;
 HEAP32[$pc>>2] = $24;
 $25 = HEAP32[$0>>2]|0;
 $26 = (($25) + 6|0);
 $27 = HEAP8[$26>>0]|0;
 $28 = $27&255;
 HEAP32[$hdr>>2] = $28;
 $29 = HEAP32[$pc>>2]|0;
 $30 = HEAP32[$hdr>>2]|0;
 $31 = (($30) + 5)|0;
 $32 = HEAP32[$data>>2]|0;
 $33 = (($32) + ($31)|0);
 $34 = HEAP8[$33>>0]|0;
 $35 = $34&255;
 $36 = $35 << 8;
 $37 = HEAP32[$hdr>>2]|0;
 $38 = (($37) + 5)|0;
 $39 = HEAP32[$data>>2]|0;
 $40 = (($39) + ($38)|0);
 $41 = (($40) + 1|0);
 $42 = HEAP8[$41>>0]|0;
 $43 = $42&255;
 $44 = $36 | $43;
 $45 = ($29>>>0)<($44>>>0);
 if (!($45)) {
  $46 = HEAP32[$pc>>2]|0;
  $47 = HEAP32[$2>>2]|0;
  $48 = (($46) + ($47))|0;
  $49 = HEAP32[$0>>2]|0;
  $50 = (($49) + 52|0);
  $51 = HEAP32[$50>>2]|0;
  $52 = (($51) + 36|0);
  $53 = HEAP32[$52>>2]|0;
  $54 = ($48>>>0)>($53>>>0);
  if (!($54)) {
   $57 = HEAP32[$0>>2]|0;
   $58 = HEAP32[$pc>>2]|0;
   $59 = $58&65535;
   $60 = HEAP32[$2>>2]|0;
   $61 = $60&65535;
   $62 = (_freeSpace($57,$59,$61)|0);
   HEAP32[$rc>>2] = $62;
   $63 = HEAP32[$rc>>2]|0;
   $64 = ($63|0)!=(0);
   if ($64) {
    $65 = HEAP32[$rc>>2]|0;
    $66 = HEAP32[$3>>2]|0;
    HEAP32[$66>>2] = $65;
    STACKTOP = sp;return;
   } else {
    $67 = HEAP32[$0>>2]|0;
    $68 = (($67) + 18|0);
    $69 = HEAP16[$68>>1]|0;
    $70 = (($69) + -1)<<16>>16;
    HEAP16[$68>>1] = $70;
    $71 = HEAP32[$ptr>>2]|0;
    $72 = HEAP32[$ptr>>2]|0;
    $73 = (($72) + 2|0);
    $74 = HEAP32[$0>>2]|0;
    $75 = (($74) + 18|0);
    $76 = HEAP16[$75>>1]|0;
    $77 = $76&65535;
    $78 = HEAP32[$1>>2]|0;
    $79 = (($77) - ($78))|0;
    $80 = $79<<1;
    _memmove(($71|0),($73|0),($80|0))|0;
    $81 = HEAP32[$0>>2]|0;
    $82 = (($81) + 18|0);
    $83 = HEAP16[$82>>1]|0;
    $84 = $83&65535;
    $85 = $84 >> 8;
    $86 = $85&255;
    $87 = HEAP32[$hdr>>2]|0;
    $88 = (($87) + 3)|0;
    $89 = HEAP32[$data>>2]|0;
    $90 = (($89) + ($88)|0);
    HEAP8[$90>>0] = $86;
    $91 = HEAP32[$0>>2]|0;
    $92 = (($91) + 18|0);
    $93 = HEAP16[$92>>1]|0;
    $94 = $93&255;
    $95 = HEAP32[$hdr>>2]|0;
    $96 = (($95) + 3)|0;
    $97 = HEAP32[$data>>2]|0;
    $98 = (($97) + ($96)|0);
    $99 = (($98) + 1|0);
    HEAP8[$99>>0] = $94;
    $100 = HEAP32[$0>>2]|0;
    $101 = (($100) + 16|0);
    $102 = HEAP16[$101>>1]|0;
    $103 = $102&65535;
    $104 = (($103) + 2)|0;
    $105 = $104&65535;
    HEAP16[$101>>1] = $105;
    STACKTOP = sp;return;
   }
  }
 }
 $55 = (_sqlite3CorruptError(58057)|0);
 $56 = HEAP32[$3>>2]|0;
 HEAP32[$56>>2] = $55;
 STACKTOP = sp;return;
}
function _insertCell($pPage,$i,$pCell,$sz,$pTemp,$iChild,$pRC) {
 $pPage = $pPage|0;
 $i = $i|0;
 $pCell = $pCell|0;
 $sz = $sz|0;
 $pTemp = $pTemp|0;
 $iChild = $iChild|0;
 $pRC = $pRC|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0;
 var $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0;
 var $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0;
 var $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0;
 var $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0;
 var $98 = 0, $99 = 0, $cellOffset = 0, $data = 0, $end = 0, $idx = 0, $ins = 0, $j = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 64|0;
 $0 = sp + 52|0;
 $1 = sp + 48|0;
 $2 = sp + 44|0;
 $3 = sp + 40|0;
 $4 = sp + 36|0;
 $5 = sp + 32|0;
 $6 = sp + 28|0;
 $idx = sp + 24|0;
 $j = sp + 20|0;
 $end = sp + 16|0;
 $ins = sp + 12|0;
 $cellOffset = sp + 8|0;
 $data = sp + 4|0;
 $rc = sp;
 HEAP32[$0>>2] = $pPage;
 HEAP32[$1>>2] = $i;
 HEAP32[$2>>2] = $pCell;
 HEAP32[$3>>2] = $sz;
 HEAP32[$4>>2] = $pTemp;
 HEAP32[$5>>2] = $iChild;
 HEAP32[$6>>2] = $pRC;
 HEAP32[$idx>>2] = 0;
 $7 = HEAP32[$6>>2]|0;
 $8 = HEAP32[$7>>2]|0;
 $9 = ($8|0)!=(0);
 if ($9) {
  STACKTOP = sp;return;
 }
 $10 = HEAP32[$0>>2]|0;
 $11 = (($10) + 1|0);
 $12 = HEAP8[$11>>0]|0;
 $13 = $12&255;
 $14 = ($13|0)!=(0);
 if (!($14)) {
  $15 = HEAP32[$3>>2]|0;
  $16 = (($15) + 2)|0;
  $17 = HEAP32[$0>>2]|0;
  $18 = (($17) + 16|0);
  $19 = HEAP16[$18>>1]|0;
  $20 = $19&65535;
  $21 = ($16|0)>($20|0);
  if (!($21)) {
   $48 = HEAP32[$0>>2]|0;
   $49 = (($48) + 68|0);
   $50 = HEAP32[$49>>2]|0;
   $51 = (_sqlite3PagerWrite($50)|0);
   HEAP32[$rc>>2] = $51;
   $52 = HEAP32[$rc>>2]|0;
   $53 = ($52|0)!=(0);
   if ($53) {
    $54 = HEAP32[$rc>>2]|0;
    $55 = HEAP32[$6>>2]|0;
    HEAP32[$55>>2] = $54;
    STACKTOP = sp;return;
   }
   $56 = HEAP32[$0>>2]|0;
   $57 = (($56) + 56|0);
   $58 = HEAP32[$57>>2]|0;
   HEAP32[$data>>2] = $58;
   $59 = HEAP32[$0>>2]|0;
   $60 = (($59) + 14|0);
   $61 = HEAP16[$60>>1]|0;
   $62 = $61&65535;
   HEAP32[$cellOffset>>2] = $62;
   $63 = HEAP32[$cellOffset>>2]|0;
   $64 = HEAP32[$0>>2]|0;
   $65 = (($64) + 18|0);
   $66 = HEAP16[$65>>1]|0;
   $67 = $66&65535;
   $68 = $67<<1;
   $69 = (($63) + ($68))|0;
   HEAP32[$end>>2] = $69;
   $70 = HEAP32[$cellOffset>>2]|0;
   $71 = HEAP32[$1>>2]|0;
   $72 = $71<<1;
   $73 = (($70) + ($72))|0;
   HEAP32[$ins>>2] = $73;
   $74 = HEAP32[$0>>2]|0;
   $75 = HEAP32[$3>>2]|0;
   $76 = (_allocateSpace($74,$75,$idx)|0);
   HEAP32[$rc>>2] = $76;
   $77 = HEAP32[$rc>>2]|0;
   $78 = ($77|0)!=(0);
   if ($78) {
    $79 = HEAP32[$rc>>2]|0;
    $80 = HEAP32[$6>>2]|0;
    HEAP32[$80>>2] = $79;
    STACKTOP = sp;return;
   }
   $81 = HEAP32[$0>>2]|0;
   $82 = (($81) + 18|0);
   $83 = HEAP16[$82>>1]|0;
   $84 = (($83) + 1)<<16>>16;
   HEAP16[$82>>1] = $84;
   $85 = HEAP32[$3>>2]|0;
   $86 = (2 + ($85))|0;
   $87 = $86&65535;
   $88 = $87&65535;
   $89 = HEAP32[$0>>2]|0;
   $90 = (($89) + 16|0);
   $91 = HEAP16[$90>>1]|0;
   $92 = $91&65535;
   $93 = (($92) - ($88))|0;
   $94 = $93&65535;
   HEAP16[$90>>1] = $94;
   $95 = HEAP32[$idx>>2]|0;
   $96 = HEAP32[$data>>2]|0;
   $97 = (($96) + ($95)|0);
   $98 = HEAP32[$2>>2]|0;
   $99 = HEAP32[$3>>2]|0;
   _memcpy(($97|0),($98|0),($99|0))|0;
   $100 = HEAP32[$5>>2]|0;
   $101 = ($100|0)!=(0);
   if ($101) {
    $102 = HEAP32[$idx>>2]|0;
    $103 = HEAP32[$data>>2]|0;
    $104 = (($103) + ($102)|0);
    $105 = HEAP32[$5>>2]|0;
    _sqlite3Put4byte($104,$105);
   }
   $106 = HEAP32[$ins>>2]|0;
   $107 = (($106) + 2)|0;
   $108 = HEAP32[$data>>2]|0;
   $109 = (($108) + ($107)|0);
   $110 = HEAP32[$ins>>2]|0;
   $111 = HEAP32[$data>>2]|0;
   $112 = (($111) + ($110)|0);
   $113 = HEAP32[$end>>2]|0;
   $114 = HEAP32[$ins>>2]|0;
   $115 = (($113) - ($114))|0;
   _memmove(($109|0),($112|0),($115|0))|0;
   $116 = HEAP32[$idx>>2]|0;
   $117 = $116 >> 8;
   $118 = $117&255;
   $119 = HEAP32[$ins>>2]|0;
   $120 = HEAP32[$data>>2]|0;
   $121 = (($120) + ($119)|0);
   HEAP8[$121>>0] = $118;
   $122 = HEAP32[$idx>>2]|0;
   $123 = $122&255;
   $124 = HEAP32[$ins>>2]|0;
   $125 = HEAP32[$data>>2]|0;
   $126 = (($125) + ($124)|0);
   $127 = (($126) + 1|0);
   HEAP8[$127>>0] = $123;
   $128 = HEAP32[$0>>2]|0;
   $129 = (($128) + 18|0);
   $130 = HEAP16[$129>>1]|0;
   $131 = $130&65535;
   $132 = $131 >> 8;
   $133 = $132&255;
   $134 = HEAP32[$0>>2]|0;
   $135 = (($134) + 6|0);
   $136 = HEAP8[$135>>0]|0;
   $137 = $136&255;
   $138 = (($137) + 3)|0;
   $139 = HEAP32[$data>>2]|0;
   $140 = (($139) + ($138)|0);
   HEAP8[$140>>0] = $133;
   $141 = HEAP32[$0>>2]|0;
   $142 = (($141) + 18|0);
   $143 = HEAP16[$142>>1]|0;
   $144 = $143&255;
   $145 = HEAP32[$0>>2]|0;
   $146 = (($145) + 6|0);
   $147 = HEAP8[$146>>0]|0;
   $148 = $147&255;
   $149 = (($148) + 3)|0;
   $150 = HEAP32[$data>>2]|0;
   $151 = (($150) + ($149)|0);
   $152 = (($151) + 1|0);
   HEAP8[$152>>0] = $144;
   $153 = HEAP32[$0>>2]|0;
   $154 = (($153) + 52|0);
   $155 = HEAP32[$154>>2]|0;
   $156 = (($155) + 17|0);
   $157 = HEAP8[$156>>0]|0;
   $158 = ($157<<24>>24)!=(0);
   if (!($158)) {
    STACKTOP = sp;return;
   }
   $159 = HEAP32[$0>>2]|0;
   $160 = HEAP32[$2>>2]|0;
   $161 = HEAP32[$6>>2]|0;
   _ptrmapPutOvflPtr($159,$160,$161);
   STACKTOP = sp;return;
  }
 }
 $22 = HEAP32[$4>>2]|0;
 $23 = ($22|0)!=(0|0);
 if ($23) {
  $24 = HEAP32[$4>>2]|0;
  $25 = HEAP32[$2>>2]|0;
  $26 = HEAP32[$3>>2]|0;
  _memcpy(($24|0),($25|0),($26|0))|0;
  $27 = HEAP32[$4>>2]|0;
  HEAP32[$2>>2] = $27;
 }
 $28 = HEAP32[$5>>2]|0;
 $29 = ($28|0)!=(0);
 if ($29) {
  $30 = HEAP32[$2>>2]|0;
  $31 = HEAP32[$5>>2]|0;
  _sqlite3Put4byte($30,$31);
 }
 $32 = HEAP32[$0>>2]|0;
 $33 = (($32) + 1|0);
 $34 = HEAP8[$33>>0]|0;
 $35 = (($34) + 1)<<24>>24;
 HEAP8[$33>>0] = $35;
 $36 = $34&255;
 HEAP32[$j>>2] = $36;
 $37 = HEAP32[$2>>2]|0;
 $38 = HEAP32[$j>>2]|0;
 $39 = HEAP32[$0>>2]|0;
 $40 = (($39) + 32|0);
 $41 = (($40) + ($38<<2)|0);
 HEAP32[$41>>2] = $37;
 $42 = HEAP32[$1>>2]|0;
 $43 = $42&65535;
 $44 = HEAP32[$j>>2]|0;
 $45 = HEAP32[$0>>2]|0;
 $46 = (($45) + 22|0);
 $47 = (($46) + ($44<<1)|0);
 HEAP16[$47>>1] = $43;
 STACKTOP = sp;return;
}
function _balance($pCur) {
 $pCur = $pCur|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0;
 var $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0;
 var $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0;
 var $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0;
 var $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0;
 var $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $aBalanceQuickSpace = 0, $iIdx = 0, $iPage = 0, $nMin = 0, $pFree = 0, $pPage = 0, $pParent = 0, $pSpace = 0, $rc = 0, label = 0;
 var sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 64|0;
 $0 = sp + 32|0;
 $rc = sp + 28|0;
 $nMin = sp + 24|0;
 $aBalanceQuickSpace = sp + 36|0;
 $pFree = sp + 20|0;
 $iPage = sp + 16|0;
 $pPage = sp + 12|0;
 $pParent = sp + 8|0;
 $iIdx = sp + 4|0;
 $pSpace = sp;
 HEAP32[$0>>2] = $pCur;
 HEAP32[$rc>>2] = 0;
 $1 = HEAP32[$0>>2]|0;
 $2 = (($1) + 4|0);
 $3 = HEAP32[$2>>2]|0;
 $4 = (($3) + 36|0);
 $5 = HEAP32[$4>>2]|0;
 $6 = $5<<1;
 $7 = (($6>>>0) / 3)&-1;
 HEAP32[$nMin>>2] = $7;
 HEAP32[$pFree>>2] = 0;
 while(1) {
  $8 = HEAP32[$0>>2]|0;
  $9 = (($8) + 76|0);
  $10 = HEAP16[$9>>1]|0;
  $11 = $10 << 16 >> 16;
  HEAP32[$iPage>>2] = $11;
  $12 = HEAP32[$iPage>>2]|0;
  $13 = HEAP32[$0>>2]|0;
  $14 = (($13) + 120|0);
  $15 = (($14) + ($12<<2)|0);
  $16 = HEAP32[$15>>2]|0;
  HEAP32[$pPage>>2] = $16;
  $17 = HEAP32[$iPage>>2]|0;
  $18 = ($17|0)==(0);
  $19 = HEAP32[$pPage>>2]|0;
  $20 = (($19) + 1|0);
  $21 = HEAP8[$20>>0]|0;
  if ($18) {
   $22 = ($21<<24>>24)!=(0);
   if (!($22)) {
    break;
   }
   $23 = HEAP32[$pPage>>2]|0;
   $24 = HEAP32[$0>>2]|0;
   $25 = (($24) + 120|0);
   $26 = (($25) + 4|0);
   $27 = (_balance_deeper($23,$26)|0);
   HEAP32[$rc>>2] = $27;
   $28 = HEAP32[$rc>>2]|0;
   $29 = ($28|0)==(0);
   if ($29) {
    $30 = HEAP32[$0>>2]|0;
    $31 = (($30) + 76|0);
    HEAP16[$31>>1] = 1;
    $32 = HEAP32[$0>>2]|0;
    $33 = (($32) + 78|0);
    HEAP16[$33>>1] = 0;
    $34 = HEAP32[$0>>2]|0;
    $35 = (($34) + 78|0);
    $36 = (($35) + 2|0);
    HEAP16[$36>>1] = 0;
   }
  } else {
   $37 = $21&255;
   $38 = ($37|0)==(0);
   if ($38) {
    $39 = HEAP32[$pPage>>2]|0;
    $40 = (($39) + 16|0);
    $41 = HEAP16[$40>>1]|0;
    $42 = $41&65535;
    $43 = HEAP32[$nMin>>2]|0;
    $44 = ($42|0)<=($43|0);
    if ($44) {
     break;
    }
   }
   $45 = HEAP32[$iPage>>2]|0;
   $46 = (($45) - 1)|0;
   $47 = HEAP32[$0>>2]|0;
   $48 = (($47) + 120|0);
   $49 = (($48) + ($46<<2)|0);
   $50 = HEAP32[$49>>2]|0;
   HEAP32[$pParent>>2] = $50;
   $51 = HEAP32[$iPage>>2]|0;
   $52 = (($51) - 1)|0;
   $53 = HEAP32[$0>>2]|0;
   $54 = (($53) + 78|0);
   $55 = (($54) + ($52<<1)|0);
   $56 = HEAP16[$55>>1]|0;
   $57 = $56&65535;
   HEAP32[$iIdx>>2] = $57;
   $58 = HEAP32[$pParent>>2]|0;
   $59 = (($58) + 68|0);
   $60 = HEAP32[$59>>2]|0;
   $61 = (_sqlite3PagerWrite($60)|0);
   HEAP32[$rc>>2] = $61;
   $62 = HEAP32[$rc>>2]|0;
   $63 = ($62|0)==(0);
   do {
    if ($63) {
     $64 = HEAP32[$pPage>>2]|0;
     $65 = (($64) + 3|0);
     $66 = HEAP8[$65>>0]|0;
     $67 = $66&255;
     $68 = ($67|0)!=(0);
     if ($68) {
      $69 = HEAP32[$pPage>>2]|0;
      $70 = (($69) + 1|0);
      $71 = HEAP8[$70>>0]|0;
      $72 = $71&255;
      $73 = ($72|0)==(1);
      if ($73) {
       $74 = HEAP32[$pPage>>2]|0;
       $75 = (($74) + 22|0);
       $76 = HEAP16[$75>>1]|0;
       $77 = $76&65535;
       $78 = HEAP32[$pPage>>2]|0;
       $79 = (($78) + 18|0);
       $80 = HEAP16[$79>>1]|0;
       $81 = $80&65535;
       $82 = ($77|0)==($81|0);
       if ($82) {
        $83 = HEAP32[$pParent>>2]|0;
        $84 = (($83) + 72|0);
        $85 = HEAP32[$84>>2]|0;
        $86 = ($85|0)!=(1);
        if ($86) {
         $87 = HEAP32[$pParent>>2]|0;
         $88 = (($87) + 18|0);
         $89 = HEAP16[$88>>1]|0;
         $90 = $89&65535;
         $91 = HEAP32[$iIdx>>2]|0;
         $92 = ($90|0)==($91|0);
         if ($92) {
          $93 = HEAP32[$pParent>>2]|0;
          $94 = HEAP32[$pPage>>2]|0;
          $95 = (_balance_quick($93,$94,$aBalanceQuickSpace)|0);
          HEAP32[$rc>>2] = $95;
          break;
         }
        }
       }
      }
     }
     $96 = HEAP32[$0>>2]|0;
     $97 = (($96) + 4|0);
     $98 = HEAP32[$97>>2]|0;
     $99 = (($98) + 32|0);
     $100 = HEAP32[$99>>2]|0;
     $101 = (_sqlite3PageMalloc($100)|0);
     HEAP32[$pSpace>>2] = $101;
     $102 = HEAP32[$pParent>>2]|0;
     $103 = HEAP32[$iIdx>>2]|0;
     $104 = HEAP32[$pSpace>>2]|0;
     $105 = HEAP32[$iPage>>2]|0;
     $106 = ($105|0)==(1);
     $107 = $106&1;
     $108 = HEAP32[$0>>2]|0;
     $109 = (($108) + 74|0);
     $110 = HEAP8[$109>>0]|0;
     $111 = $110&255;
     $112 = (_balance_nonroot($102,$103,$104,$107,$111)|0);
     HEAP32[$rc>>2] = $112;
     $113 = HEAP32[$pFree>>2]|0;
     $114 = ($113|0)!=(0|0);
     if ($114) {
      $115 = HEAP32[$pFree>>2]|0;
      _sqlite3PageFree($115);
     }
     $116 = HEAP32[$pSpace>>2]|0;
     HEAP32[$pFree>>2] = $116;
    }
   } while(0);
   $117 = HEAP32[$pPage>>2]|0;
   $118 = (($117) + 1|0);
   HEAP8[$118>>0] = 0;
   $119 = HEAP32[$pPage>>2]|0;
   _releasePage($119);
   $120 = HEAP32[$0>>2]|0;
   $121 = (($120) + 76|0);
   $122 = HEAP16[$121>>1]|0;
   $123 = (($122) + -1)<<16>>16;
   HEAP16[$121>>1] = $123;
  }
  $124 = HEAP32[$rc>>2]|0;
  $125 = ($124|0)==(0);
  if (!($125)) {
   break;
  }
 }
 $126 = HEAP32[$pFree>>2]|0;
 $127 = ($126|0)!=(0|0);
 if (!($127)) {
  $129 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($129|0);
 }
 $128 = HEAP32[$pFree>>2]|0;
 _sqlite3PageFree($128);
 $129 = HEAP32[$rc>>2]|0;
 STACKTOP = sp;return ($129|0);
}
function _balance_deeper($pRoot,$ppChild) {
 $pRoot = $pRoot|0;
 $ppChild = $ppChild|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $8 = 0, $9 = 0, $pBt = 0, $pChild = 0, $pgnoChild = 0, $rc = 0;
 var label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 24|0;
 $1 = sp + 20|0;
 $2 = sp + 16|0;
 $rc = sp + 12|0;
 $pChild = sp + 8|0;
 $pgnoChild = sp + 4|0;
 $pBt = sp;
 HEAP32[$1>>2] = $pRoot;
 HEAP32[$2>>2] = $ppChild;
 HEAP32[$pChild>>2] = 0;
 HEAP32[$pgnoChild>>2] = 0;
 $3 = HEAP32[$1>>2]|0;
 $4 = (($3) + 52|0);
 $5 = HEAP32[$4>>2]|0;
 HEAP32[$pBt>>2] = $5;
 $6 = HEAP32[$1>>2]|0;
 $7 = (($6) + 68|0);
 $8 = HEAP32[$7>>2]|0;
 $9 = (_sqlite3PagerWrite($8)|0);
 HEAP32[$rc>>2] = $9;
 $10 = HEAP32[$rc>>2]|0;
 $11 = ($10|0)==(0);
 if ($11) {
  $12 = HEAP32[$pBt>>2]|0;
  $13 = HEAP32[$1>>2]|0;
  $14 = (($13) + 72|0);
  $15 = HEAP32[$14>>2]|0;
  $16 = (_allocateBtreePage($12,$pChild,$pgnoChild,$15,0)|0);
  HEAP32[$rc>>2] = $16;
  $17 = HEAP32[$1>>2]|0;
  $18 = HEAP32[$pChild>>2]|0;
  _copyNodeContent($17,$18,$rc);
  $19 = HEAP32[$pBt>>2]|0;
  $20 = (($19) + 17|0);
  $21 = HEAP8[$20>>0]|0;
  $22 = ($21<<24>>24)!=(0);
  if ($22) {
   $23 = HEAP32[$pBt>>2]|0;
   $24 = HEAP32[$pgnoChild>>2]|0;
   $25 = HEAP32[$1>>2]|0;
   $26 = (($25) + 72|0);
   $27 = HEAP32[$26>>2]|0;
   _ptrmapPut($23,$24,5,$27,$rc);
  }
 }
 $28 = HEAP32[$rc>>2]|0;
 $29 = ($28|0)!=(0);
 if ($29) {
  $30 = HEAP32[$2>>2]|0;
  HEAP32[$30>>2] = 0;
  $31 = HEAP32[$pChild>>2]|0;
  _releasePage($31);
  $32 = HEAP32[$rc>>2]|0;
  HEAP32[$0>>2] = $32;
  $75 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($75|0);
 } else {
  $33 = HEAP32[$pChild>>2]|0;
  $34 = (($33) + 22|0);
  $35 = HEAP32[$1>>2]|0;
  $36 = (($35) + 22|0);
  $37 = HEAP32[$1>>2]|0;
  $38 = (($37) + 1|0);
  $39 = HEAP8[$38>>0]|0;
  $40 = $39&255;
  $41 = $40<<1;
  _memcpy(($34|0),($36|0),($41|0))|0;
  $42 = HEAP32[$pChild>>2]|0;
  $43 = (($42) + 32|0);
  $44 = HEAP32[$1>>2]|0;
  $45 = (($44) + 32|0);
  $46 = HEAP32[$1>>2]|0;
  $47 = (($46) + 1|0);
  $48 = HEAP8[$47>>0]|0;
  $49 = $48&255;
  $50 = $49<<2;
  _memcpy(($43|0),($45|0),($50|0))|0;
  $51 = HEAP32[$1>>2]|0;
  $52 = (($51) + 1|0);
  $53 = HEAP8[$52>>0]|0;
  $54 = HEAP32[$pChild>>2]|0;
  $55 = (($54) + 1|0);
  HEAP8[$55>>0] = $53;
  $56 = HEAP32[$1>>2]|0;
  $57 = HEAP32[$pChild>>2]|0;
  $58 = (($57) + 56|0);
  $59 = HEAP32[$58>>2]|0;
  $60 = HEAP8[$59>>0]|0;
  $61 = $60&255;
  $62 = $61 & -9;
  _zeroPage($56,$62);
  $63 = HEAP32[$1>>2]|0;
  $64 = (($63) + 6|0);
  $65 = HEAP8[$64>>0]|0;
  $66 = $65&255;
  $67 = (($66) + 8)|0;
  $68 = HEAP32[$1>>2]|0;
  $69 = (($68) + 56|0);
  $70 = HEAP32[$69>>2]|0;
  $71 = (($70) + ($67)|0);
  $72 = HEAP32[$pgnoChild>>2]|0;
  _sqlite3Put4byte($71,$72);
  $73 = HEAP32[$pChild>>2]|0;
  $74 = HEAP32[$2>>2]|0;
  HEAP32[$74>>2] = $73;
  HEAP32[$0>>2] = 0;
  $75 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($75|0);
 }
 return 0|0;
}
function _balance_quick($pParent,$pPage,$pSpace) {
 $pParent = $pParent|0;
 $pPage = $pPage|0;
 $pSpace = $pSpace|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $14 = 0, $15 = 0;
 var $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0;
 var $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0;
 var $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0;
 var $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0;
 var $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $pBt = 0, $pCell = 0, $pNew = 0, $pOut = 0, $pStop = 0, $pgnoNew = 0, $rc = 0, $szCell = 0;
 var label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $0 = sp + 40|0;
 $1 = sp + 36|0;
 $2 = sp + 32|0;
 $3 = sp + 28|0;
 $pBt = sp + 24|0;
 $pNew = sp + 20|0;
 $rc = sp + 16|0;
 $pgnoNew = sp + 12|0;
 $pOut = sp + 8|0;
 $pCell = sp + 4|0;
 $szCell = sp + 44|0;
 $pStop = sp;
 HEAP32[$1>>2] = $pParent;
 HEAP32[$2>>2] = $pPage;
 HEAP32[$3>>2] = $pSpace;
 $4 = HEAP32[$2>>2]|0;
 $5 = (($4) + 52|0);
 $6 = HEAP32[$5>>2]|0;
 HEAP32[$pBt>>2] = $6;
 $7 = HEAP32[$2>>2]|0;
 $8 = (($7) + 18|0);
 $9 = HEAP16[$8>>1]|0;
 $10 = $9&65535;
 $11 = ($10|0)==(0);
 if ($11) {
  $12 = (_sqlite3CorruptError(58255)|0);
  HEAP32[$0>>2] = $12;
  $131 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($131|0);
 }
 $13 = HEAP32[$pBt>>2]|0;
 $14 = (_allocateBtreePage($13,$pNew,$pgnoNew,0,0)|0);
 HEAP32[$rc>>2] = $14;
 $15 = HEAP32[$rc>>2]|0;
 $16 = ($15|0)==(0);
 if ($16) {
  $17 = HEAP32[$3>>2]|0;
  $18 = (($17) + 4|0);
  HEAP32[$pOut>>2] = $18;
  $19 = HEAP32[$2>>2]|0;
  $20 = (($19) + 32|0);
  $21 = HEAP32[$20>>2]|0;
  HEAP32[$pCell>>2] = $21;
  $22 = HEAP32[$2>>2]|0;
  $23 = HEAP32[$pCell>>2]|0;
  $24 = (_cellSizePtr($22,$23)|0);
  HEAP16[$szCell>>1] = $24;
  $25 = HEAP32[$pNew>>2]|0;
  _zeroPage($25,13);
  $26 = HEAP32[$pNew>>2]|0;
  _assemblePage($26,1,$pCell,$szCell);
  $27 = HEAP32[$pBt>>2]|0;
  $28 = (($27) + 17|0);
  $29 = HEAP8[$28>>0]|0;
  $30 = ($29<<24>>24)!=(0);
  if ($30) {
   $31 = HEAP32[$pBt>>2]|0;
   $32 = HEAP32[$pgnoNew>>2]|0;
   $33 = HEAP32[$1>>2]|0;
   $34 = (($33) + 72|0);
   $35 = HEAP32[$34>>2]|0;
   _ptrmapPut($31,$32,5,$35,$rc);
   $36 = HEAP16[$szCell>>1]|0;
   $37 = $36&65535;
   $38 = HEAP32[$pNew>>2]|0;
   $39 = (($38) + 12|0);
   $40 = HEAP16[$39>>1]|0;
   $41 = $40&65535;
   $42 = ($37|0)>($41|0);
   if ($42) {
    $43 = HEAP32[$pNew>>2]|0;
    $44 = HEAP32[$pCell>>2]|0;
    _ptrmapPutOvflPtr($43,$44,$rc);
   }
  }
  $45 = HEAP32[$2>>2]|0;
  $46 = (($45) + 56|0);
  $47 = HEAP32[$46>>2]|0;
  $48 = HEAP32[$2>>2]|0;
  $49 = (($48) + 20|0);
  $50 = HEAP16[$49>>1]|0;
  $51 = $50&65535;
  $52 = HEAP32[$2>>2]|0;
  $53 = (($52) + 18|0);
  $54 = HEAP16[$53>>1]|0;
  $55 = $54&65535;
  $56 = (($55) - 1)|0;
  $57 = $56<<1;
  $58 = HEAP32[$2>>2]|0;
  $59 = (($58) + 64|0);
  $60 = HEAP32[$59>>2]|0;
  $61 = (($60) + ($57)|0);
  $62 = HEAP8[$61>>0]|0;
  $63 = $62&255;
  $64 = $63 << 8;
  $65 = HEAP32[$2>>2]|0;
  $66 = (($65) + 18|0);
  $67 = HEAP16[$66>>1]|0;
  $68 = $67&65535;
  $69 = (($68) - 1)|0;
  $70 = $69<<1;
  $71 = HEAP32[$2>>2]|0;
  $72 = (($71) + 64|0);
  $73 = HEAP32[$72>>2]|0;
  $74 = (($73) + ($70)|0);
  $75 = (($74) + 1|0);
  $76 = HEAP8[$75>>0]|0;
  $77 = $76&255;
  $78 = $64 | $77;
  $79 = $51 & $78;
  $80 = (($47) + ($79)|0);
  HEAP32[$pCell>>2] = $80;
  $81 = HEAP32[$pCell>>2]|0;
  $82 = (($81) + 9|0);
  HEAP32[$pStop>>2] = $82;
  while(1) {
   $83 = HEAP32[$pCell>>2]|0;
   $84 = (($83) + 1|0);
   HEAP32[$pCell>>2] = $84;
   $85 = HEAP8[$83>>0]|0;
   $86 = $85&255;
   $87 = $86 & 128;
   $88 = ($87|0)!=(0);
   if (!($88)) {
    break;
   }
   $89 = HEAP32[$pCell>>2]|0;
   $90 = HEAP32[$pStop>>2]|0;
   $91 = ($89>>>0)<($90>>>0);
   if (!($91)) {
    break;
   }
  }
  $92 = HEAP32[$pCell>>2]|0;
  $93 = (($92) + 9|0);
  HEAP32[$pStop>>2] = $93;
  while(1) {
   $94 = HEAP32[$pCell>>2]|0;
   $95 = (($94) + 1|0);
   HEAP32[$pCell>>2] = $95;
   $96 = HEAP8[$94>>0]|0;
   $97 = HEAP32[$pOut>>2]|0;
   $98 = (($97) + 1|0);
   HEAP32[$pOut>>2] = $98;
   HEAP8[$97>>0] = $96;
   $99 = $96&255;
   $100 = $99 & 128;
   $101 = ($100|0)!=(0);
   if (!($101)) {
    break;
   }
   $102 = HEAP32[$pCell>>2]|0;
   $103 = HEAP32[$pStop>>2]|0;
   $104 = ($102>>>0)<($103>>>0);
   if (!($104)) {
    break;
   }
  }
  $105 = HEAP32[$1>>2]|0;
  $106 = HEAP32[$1>>2]|0;
  $107 = (($106) + 18|0);
  $108 = HEAP16[$107>>1]|0;
  $109 = $108&65535;
  $110 = HEAP32[$3>>2]|0;
  $111 = HEAP32[$pOut>>2]|0;
  $112 = HEAP32[$3>>2]|0;
  $113 = $111;
  $114 = $112;
  $115 = (($113) - ($114))|0;
  $116 = HEAP32[$2>>2]|0;
  $117 = (($116) + 72|0);
  $118 = HEAP32[$117>>2]|0;
  _insertCell($105,$109,$110,$115,0,$118,$rc);
  $119 = HEAP32[$1>>2]|0;
  $120 = (($119) + 6|0);
  $121 = HEAP8[$120>>0]|0;
  $122 = $121&255;
  $123 = (($122) + 8)|0;
  $124 = HEAP32[$1>>2]|0;
  $125 = (($124) + 56|0);
  $126 = HEAP32[$125>>2]|0;
  $127 = (($126) + ($123)|0);
  $128 = HEAP32[$pgnoNew>>2]|0;
  _sqlite3Put4byte($127,$128);
  $129 = HEAP32[$pNew>>2]|0;
  _releasePage($129);
 }
 $130 = HEAP32[$rc>>2]|0;
 HEAP32[$0>>2] = $130;
 $131 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($131|0);
}
function _balance_nonroot($pParent,$iParentIdx,$aOvflSpace,$isRoot,$bBulk) {
 $pParent = $pParent|0;
 $iParentIdx = $iParentIdx|0;
 $aOvflSpace = $aOvflSpace|0;
 $isRoot = $isRoot|0;
 $bBulk = $bBulk|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $1000 = 0, $1001 = 0, $1002 = 0, $1003 = 0, $1004 = 0, $1005 = 0, $1006 = 0, $1007 = 0, $1008 = 0, $1009 = 0, $101 = 0, $1010 = 0, $1011 = 0, $1012 = 0, $1013 = 0, $1014 = 0;
 var $1015 = 0, $1016 = 0, $1017 = 0, $1018 = 0, $1019 = 0, $102 = 0, $1020 = 0, $1021 = 0, $1022 = 0, $1023 = 0, $1024 = 0, $1025 = 0, $1026 = 0, $1027 = 0, $1028 = 0, $1029 = 0, $103 = 0, $1030 = 0, $1031 = 0, $1032 = 0;
 var $1033 = 0, $1034 = 0, $1035 = 0, $1036 = 0, $1037 = 0, $1038 = 0, $1039 = 0, $104 = 0, $1040 = 0, $1041 = 0, $1042 = 0, $1043 = 0, $1044 = 0, $1045 = 0, $1046 = 0, $1047 = 0, $1048 = 0, $1049 = 0, $105 = 0, $1050 = 0;
 var $1051 = 0, $1052 = 0, $1053 = 0, $1054 = 0, $1055 = 0, $1056 = 0, $1057 = 0, $1058 = 0, $1059 = 0, $106 = 0, $1060 = 0, $1061 = 0, $1062 = 0, $1063 = 0, $1064 = 0, $1065 = 0, $1066 = 0, $1067 = 0, $1068 = 0, $1069 = 0;
 var $107 = 0, $1070 = 0, $1071 = 0, $1072 = 0, $1073 = 0, $1074 = 0, $1075 = 0, $1076 = 0, $1077 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0, $116 = 0, $117 = 0;
 var $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0, $134 = 0, $135 = 0;
 var $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0, $152 = 0, $153 = 0;
 var $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0, $170 = 0, $171 = 0;
 var $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0, $188 = 0, $189 = 0, $19 = 0;
 var $190 = 0, $191 = 0, $192 = 0, $193 = 0, $194 = 0, $195 = 0, $196 = 0, $197 = 0, $198 = 0, $199 = 0, $2 = 0, $20 = 0, $200 = 0, $201 = 0, $202 = 0, $203 = 0, $204 = 0, $205 = 0, $206 = 0, $207 = 0;
 var $208 = 0, $209 = 0, $21 = 0, $210 = 0, $211 = 0, $212 = 0, $213 = 0, $214 = 0, $215 = 0, $216 = 0, $217 = 0, $218 = 0, $219 = 0, $22 = 0, $220 = 0, $221 = 0, $222 = 0, $223 = 0, $224 = 0, $225 = 0;
 var $226 = 0, $227 = 0, $228 = 0, $229 = 0, $23 = 0, $230 = 0, $231 = 0, $232 = 0, $233 = 0, $234 = 0, $235 = 0, $236 = 0, $237 = 0, $238 = 0, $239 = 0, $24 = 0, $240 = 0, $241 = 0, $242 = 0, $243 = 0;
 var $244 = 0, $245 = 0, $246 = 0, $247 = 0, $248 = 0, $249 = 0, $25 = 0, $250 = 0, $251 = 0, $252 = 0, $253 = 0, $254 = 0, $255 = 0, $256 = 0, $257 = 0, $258 = 0, $259 = 0, $26 = 0, $260 = 0, $261 = 0;
 var $262 = 0, $263 = 0, $264 = 0, $265 = 0, $266 = 0, $267 = 0, $268 = 0, $269 = 0, $27 = 0, $270 = 0, $271 = 0, $272 = 0, $273 = 0, $274 = 0, $275 = 0, $276 = 0, $277 = 0, $278 = 0, $279 = 0, $28 = 0;
 var $280 = 0, $281 = 0, $282 = 0, $283 = 0, $284 = 0, $285 = 0, $286 = 0, $287 = 0, $288 = 0, $289 = 0, $29 = 0, $290 = 0, $291 = 0, $292 = 0, $293 = 0, $294 = 0, $295 = 0, $296 = 0, $297 = 0, $298 = 0;
 var $299 = 0, $3 = 0, $30 = 0, $300 = 0, $301 = 0, $302 = 0, $303 = 0, $304 = 0, $305 = 0, $306 = 0, $307 = 0, $308 = 0, $309 = 0, $31 = 0, $310 = 0, $311 = 0, $312 = 0, $313 = 0, $314 = 0, $315 = 0;
 var $316 = 0, $317 = 0, $318 = 0, $319 = 0, $32 = 0, $320 = 0, $321 = 0, $322 = 0, $323 = 0, $324 = 0, $325 = 0, $326 = 0, $327 = 0, $328 = 0, $329 = 0, $33 = 0, $330 = 0, $331 = 0, $332 = 0, $333 = 0;
 var $334 = 0, $335 = 0, $336 = 0, $337 = 0, $338 = 0, $339 = 0, $34 = 0, $340 = 0, $341 = 0, $342 = 0, $343 = 0, $344 = 0, $345 = 0, $346 = 0, $347 = 0, $348 = 0, $349 = 0, $35 = 0, $350 = 0, $351 = 0;
 var $352 = 0, $353 = 0, $354 = 0, $355 = 0, $356 = 0, $357 = 0, $358 = 0, $359 = 0, $36 = 0, $360 = 0, $361 = 0, $362 = 0, $363 = 0, $364 = 0, $365 = 0, $366 = 0, $367 = 0, $368 = 0, $369 = 0, $37 = 0;
 var $370 = 0, $371 = 0, $372 = 0, $373 = 0, $374 = 0, $375 = 0, $376 = 0, $377 = 0, $378 = 0, $379 = 0, $38 = 0, $380 = 0, $381 = 0, $382 = 0, $383 = 0, $384 = 0, $385 = 0, $386 = 0, $387 = 0, $388 = 0;
 var $389 = 0, $39 = 0, $390 = 0, $391 = 0, $392 = 0, $393 = 0, $394 = 0, $395 = 0, $396 = 0, $397 = 0, $398 = 0, $399 = 0, $4 = 0, $40 = 0, $400 = 0, $401 = 0, $402 = 0, $403 = 0, $404 = 0, $405 = 0;
 var $406 = 0, $407 = 0, $408 = 0, $409 = 0, $41 = 0, $410 = 0, $411 = 0, $412 = 0, $413 = 0, $414 = 0, $415 = 0, $416 = 0, $417 = 0, $418 = 0, $419 = 0, $42 = 0, $420 = 0, $421 = 0, $422 = 0, $423 = 0;
 var $424 = 0, $425 = 0, $426 = 0, $427 = 0, $428 = 0, $429 = 0, $43 = 0, $430 = 0, $431 = 0, $432 = 0, $433 = 0, $434 = 0, $435 = 0, $436 = 0, $437 = 0, $438 = 0, $439 = 0, $44 = 0, $440 = 0, $441 = 0;
 var $442 = 0, $443 = 0, $444 = 0, $445 = 0, $446 = 0, $447 = 0, $448 = 0, $449 = 0, $45 = 0, $450 = 0, $451 = 0, $452 = 0, $453 = 0, $454 = 0, $455 = 0, $456 = 0, $457 = 0, $458 = 0, $459 = 0, $46 = 0;
 var $460 = 0, $461 = 0, $462 = 0, $463 = 0, $464 = 0, $465 = 0, $466 = 0, $467 = 0, $468 = 0, $469 = 0, $47 = 0, $470 = 0, $471 = 0, $472 = 0, $473 = 0, $474 = 0, $475 = 0, $476 = 0, $477 = 0, $478 = 0;
 var $479 = 0, $48 = 0, $480 = 0, $481 = 0, $482 = 0, $483 = 0, $484 = 0, $485 = 0, $486 = 0, $487 = 0, $488 = 0, $489 = 0, $49 = 0, $490 = 0, $491 = 0, $492 = 0, $493 = 0, $494 = 0, $495 = 0, $496 = 0;
 var $497 = 0, $498 = 0, $499 = 0, $5 = 0, $50 = 0, $500 = 0, $501 = 0, $502 = 0, $503 = 0, $504 = 0, $505 = 0, $506 = 0, $507 = 0, $508 = 0, $509 = 0, $51 = 0, $510 = 0, $511 = 0, $512 = 0, $513 = 0;
 var $514 = 0, $515 = 0, $516 = 0, $517 = 0, $518 = 0, $519 = 0, $52 = 0, $520 = 0, $521 = 0, $522 = 0, $523 = 0, $524 = 0, $525 = 0, $526 = 0, $527 = 0, $528 = 0, $529 = 0, $53 = 0, $530 = 0, $531 = 0;
 var $532 = 0, $533 = 0, $534 = 0, $535 = 0, $536 = 0, $537 = 0, $538 = 0, $539 = 0, $54 = 0, $540 = 0, $541 = 0, $542 = 0, $543 = 0, $544 = 0, $545 = 0, $546 = 0, $547 = 0, $548 = 0, $549 = 0, $55 = 0;
 var $550 = 0, $551 = 0, $552 = 0, $553 = 0, $554 = 0, $555 = 0, $556 = 0, $557 = 0, $558 = 0, $559 = 0, $56 = 0, $560 = 0, $561 = 0, $562 = 0, $563 = 0, $564 = 0, $565 = 0, $566 = 0, $567 = 0, $568 = 0;
 var $569 = 0, $57 = 0, $570 = 0, $571 = 0, $572 = 0, $573 = 0, $574 = 0, $575 = 0, $576 = 0, $577 = 0, $578 = 0, $579 = 0, $58 = 0, $580 = 0, $581 = 0, $582 = 0, $583 = 0, $584 = 0, $585 = 0, $586 = 0;
 var $587 = 0, $588 = 0, $589 = 0, $59 = 0, $590 = 0, $591 = 0, $592 = 0, $593 = 0, $594 = 0, $595 = 0, $596 = 0, $597 = 0, $598 = 0, $599 = 0, $6 = 0, $60 = 0, $600 = 0, $601 = 0, $602 = 0, $603 = 0;
 var $604 = 0, $605 = 0, $606 = 0, $607 = 0, $608 = 0, $609 = 0, $61 = 0, $610 = 0, $611 = 0, $612 = 0, $613 = 0, $614 = 0, $615 = 0, $616 = 0, $617 = 0, $618 = 0, $619 = 0, $62 = 0, $620 = 0, $621 = 0;
 var $622 = 0, $623 = 0, $624 = 0, $625 = 0, $626 = 0, $627 = 0, $628 = 0, $629 = 0, $63 = 0, $630 = 0, $631 = 0, $632 = 0, $633 = 0, $634 = 0, $635 = 0, $636 = 0, $637 = 0, $638 = 0, $639 = 0, $64 = 0;
 var $640 = 0, $641 = 0, $642 = 0, $643 = 0, $644 = 0, $645 = 0, $646 = 0, $647 = 0, $648 = 0, $649 = 0, $65 = 0, $650 = 0, $651 = 0, $652 = 0, $653 = 0, $654 = 0, $655 = 0, $656 = 0, $657 = 0, $658 = 0;
 var $659 = 0, $66 = 0, $660 = 0, $661 = 0, $662 = 0, $663 = 0, $664 = 0, $665 = 0, $666 = 0, $667 = 0, $668 = 0, $669 = 0, $67 = 0, $670 = 0, $671 = 0, $672 = 0, $673 = 0, $674 = 0, $675 = 0, $676 = 0;
 var $677 = 0, $678 = 0, $679 = 0, $68 = 0, $680 = 0, $681 = 0, $682 = 0, $683 = 0, $684 = 0, $685 = 0, $686 = 0, $687 = 0, $688 = 0, $689 = 0, $69 = 0, $690 = 0, $691 = 0, $692 = 0, $693 = 0, $694 = 0;
 var $695 = 0, $696 = 0, $697 = 0, $698 = 0, $699 = 0, $7 = 0, $70 = 0, $700 = 0, $701 = 0, $702 = 0, $703 = 0, $704 = 0, $705 = 0, $706 = 0, $707 = 0, $708 = 0, $709 = 0, $71 = 0, $710 = 0, $711 = 0;
 var $712 = 0, $713 = 0, $714 = 0, $715 = 0, $716 = 0, $717 = 0, $718 = 0, $719 = 0, $72 = 0, $720 = 0, $721 = 0, $722 = 0, $723 = 0, $724 = 0, $725 = 0, $726 = 0, $727 = 0, $728 = 0, $729 = 0, $73 = 0;
 var $730 = 0, $731 = 0, $732 = 0, $733 = 0, $734 = 0, $735 = 0, $736 = 0, $737 = 0, $738 = 0, $739 = 0, $74 = 0, $740 = 0, $741 = 0, $742 = 0, $743 = 0, $744 = 0, $745 = 0, $746 = 0, $747 = 0, $748 = 0;
 var $749 = 0, $75 = 0, $750 = 0, $751 = 0, $752 = 0, $753 = 0, $754 = 0, $755 = 0, $756 = 0, $757 = 0, $758 = 0, $759 = 0, $76 = 0, $760 = 0, $761 = 0, $762 = 0, $763 = 0, $764 = 0, $765 = 0, $766 = 0;
 var $767 = 0, $768 = 0, $769 = 0, $77 = 0, $770 = 0, $771 = 0, $772 = 0, $773 = 0, $774 = 0, $775 = 0, $776 = 0, $777 = 0, $778 = 0, $779 = 0, $78 = 0, $780 = 0, $781 = 0, $782 = 0, $783 = 0, $784 = 0;
 var $785 = 0, $786 = 0, $787 = 0, $788 = 0, $789 = 0, $79 = 0, $790 = 0, $791 = 0, $792 = 0, $793 = 0, $794 = 0, $795 = 0, $796 = 0, $797 = 0, $798 = 0, $799 = 0, $8 = 0, $80 = 0, $800 = 0, $801 = 0;
 var $802 = 0, $803 = 0, $804 = 0, $805 = 0, $806 = 0, $807 = 0, $808 = 0, $809 = 0, $81 = 0, $810 = 0, $811 = 0, $812 = 0, $813 = 0, $814 = 0, $815 = 0, $816 = 0, $817 = 0, $818 = 0, $819 = 0, $82 = 0;
 var $820 = 0, $821 = 0, $822 = 0, $823 = 0, $824 = 0, $825 = 0, $826 = 0, $827 = 0, $828 = 0, $829 = 0, $83 = 0, $830 = 0, $831 = 0, $832 = 0, $833 = 0, $834 = 0, $835 = 0, $836 = 0, $837 = 0, $838 = 0;
 var $839 = 0, $84 = 0, $840 = 0, $841 = 0, $842 = 0, $843 = 0, $844 = 0, $845 = 0, $846 = 0, $847 = 0, $848 = 0, $849 = 0, $85 = 0, $850 = 0, $851 = 0, $852 = 0, $853 = 0, $854 = 0, $855 = 0, $856 = 0;
 var $857 = 0, $858 = 0, $859 = 0, $86 = 0, $860 = 0, $861 = 0, $862 = 0, $863 = 0, $864 = 0, $865 = 0, $866 = 0, $867 = 0, $868 = 0, $869 = 0, $87 = 0, $870 = 0, $871 = 0, $872 = 0, $873 = 0, $874 = 0;
 var $875 = 0, $876 = 0, $877 = 0, $878 = 0, $879 = 0, $88 = 0, $880 = 0, $881 = 0, $882 = 0, $883 = 0, $884 = 0, $885 = 0, $886 = 0, $887 = 0, $888 = 0, $889 = 0, $89 = 0, $890 = 0, $891 = 0, $892 = 0;
 var $893 = 0, $894 = 0, $895 = 0, $896 = 0, $897 = 0, $898 = 0, $899 = 0, $9 = 0, $90 = 0, $900 = 0, $901 = 0, $902 = 0, $903 = 0, $904 = 0, $905 = 0, $906 = 0, $907 = 0, $908 = 0, $909 = 0, $91 = 0;
 var $910 = 0, $911 = 0, $912 = 0, $913 = 0, $914 = 0, $915 = 0, $916 = 0, $917 = 0, $918 = 0, $919 = 0, $92 = 0, $920 = 0, $921 = 0, $922 = 0, $923 = 0, $924 = 0, $925 = 0, $926 = 0, $927 = 0, $928 = 0;
 var $929 = 0, $93 = 0, $930 = 0, $931 = 0, $932 = 0, $933 = 0, $934 = 0, $935 = 0, $936 = 0, $937 = 0, $938 = 0, $939 = 0, $94 = 0, $940 = 0, $941 = 0, $942 = 0, $943 = 0, $944 = 0, $945 = 0, $946 = 0;
 var $947 = 0, $948 = 0, $949 = 0, $95 = 0, $950 = 0, $951 = 0, $952 = 0, $953 = 0, $954 = 0, $955 = 0, $956 = 0, $957 = 0, $958 = 0, $959 = 0, $96 = 0, $960 = 0, $961 = 0, $962 = 0, $963 = 0, $964 = 0;
 var $965 = 0, $966 = 0, $967 = 0, $968 = 0, $969 = 0, $97 = 0, $970 = 0, $971 = 0, $972 = 0, $973 = 0, $974 = 0, $975 = 0, $976 = 0, $977 = 0, $978 = 0, $979 = 0, $98 = 0, $980 = 0, $981 = 0, $982 = 0;
 var $983 = 0, $984 = 0, $985 = 0, $986 = 0, $987 = 0, $988 = 0, $989 = 0, $99 = 0, $990 = 0, $991 = 0, $992 = 0, $993 = 0, $994 = 0, $995 = 0, $996 = 0, $997 = 0, $998 = 0, $999 = 0, $aData = 0, $aSpace1 = 0;
 var $apCell = 0, $apCopy = 0, $apDiv = 0, $apNew = 0, $apOld = 0, $cellOffset = 0, $cntNew = 0, $d = 0, $i = 0, $iNextOld = 0, $iOff = 0, $iOverflow = 0, $iOvflSpace = 0, $iSpace1 = 0, $info = 0, $isDivider = 0, $j = 0, $k = 0, $key = 0, $leafCorrection = 0;
 var $leafData = 0, $limit = 0, $maskPage = 0, $minI = 0, $minV = 0, $nCell = 0, $nMaxCells = 0, $nNew = 0, $nOld = 0, $nOverflow = 0, $nxDiv = 0, $or$cond = 0, $pBt = 0, $pCell = 0, $pNew = 0, $pNew1 = 0, $pNew4 = 0, $pOld = 0, $pOld5 = 0, $pRight = 0;
 var $pT = 0, $pTemp = 0, $pTemp2 = 0, $pageFlags = 0, $pgno = 0, $r = 0, $rc = 0, $subtotal = 0, $sz = 0, $sz3 = 0, $szCell = 0, $szLeft = 0, $szNew = 0, $szRight = 0, $szScratch = 0, $usableSpace = 0, $zChild = 0, dest = 0, label = 0, sp = 0;
 var src = 0, stop = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 352|0;
 $0 = sp + 328|0;
 $1 = sp + 324|0;
 $2 = sp + 320|0;
 $3 = sp + 316|0;
 $4 = sp + 312|0;
 $5 = sp + 308|0;
 $pBt = sp + 304|0;
 $nCell = sp + 300|0;
 $nMaxCells = sp + 296|0;
 $nNew = sp + 292|0;
 $nOld = sp + 288|0;
 $i = sp + 284|0;
 $j = sp + 280|0;
 $k = sp + 276|0;
 $nxDiv = sp + 272|0;
 $rc = sp + 268|0;
 $leafCorrection = sp + 338|0;
 $leafData = sp + 264|0;
 $usableSpace = sp + 260|0;
 $pageFlags = sp + 256|0;
 $subtotal = sp + 252|0;
 $iSpace1 = sp + 248|0;
 $iOvflSpace = sp + 244|0;
 $szScratch = sp + 240|0;
 $apOld = sp + 228|0;
 $apCopy = sp + 216|0;
 $apNew = sp + 196|0;
 $pRight = sp + 192|0;
 $apDiv = sp + 184|0;
 $cntNew = sp + 160|0;
 $szNew = sp + 140|0;
 $apCell = sp + 136|0;
 $szCell = sp + 132|0;
 $aSpace1 = sp + 128|0;
 $pgno = sp + 124|0;
 $iOff = sp + 120|0;
 $limit = sp + 116|0;
 $pOld = sp + 112|0;
 $aData = sp + 108|0;
 $maskPage = sp + 336|0;
 $cellOffset = sp + 334|0;
 $sz = sp + 332|0;
 $pTemp = sp + 104|0;
 $szRight = sp + 100|0;
 $szLeft = sp + 96|0;
 $r = sp + 92|0;
 $d = sp + 88|0;
 $pNew = sp + 84|0;
 $minV = sp + 80|0;
 $minI = sp + 76|0;
 $pT = sp + 72|0;
 $pNew1 = sp + 68|0;
 $pCell = sp + 64|0;
 $pTemp2 = sp + 60|0;
 $sz3 = sp + 56|0;
 $info = sp;
 $zChild = sp + 52|0;
 $pNew4 = sp + 48|0;
 $pOld5 = sp + 44|0;
 $nOverflow = sp + 40|0;
 $iNextOld = sp + 36|0;
 $iOverflow = sp + 32|0;
 $isDivider = sp + 28|0;
 $key = sp + 24|0;
 HEAP32[$1>>2] = $pParent;
 HEAP32[$2>>2] = $iParentIdx;
 HEAP32[$3>>2] = $aOvflSpace;
 HEAP32[$4>>2] = $isRoot;
 HEAP32[$5>>2] = $bBulk;
 HEAP32[$nCell>>2] = 0;
 HEAP32[$nMaxCells>>2] = 0;
 HEAP32[$nNew>>2] = 0;
 HEAP32[$rc>>2] = 0;
 HEAP32[$iSpace1>>2] = 0;
 HEAP32[$iOvflSpace>>2] = 0;
 HEAP32[$apCell>>2] = 0;
 $6 = HEAP32[$1>>2]|0;
 $7 = (($6) + 52|0);
 $8 = HEAP32[$7>>2]|0;
 HEAP32[$pBt>>2] = $8;
 $9 = HEAP32[$3>>2]|0;
 $10 = ($9|0)!=(0|0);
 if (!($10)) {
  HEAP32[$0>>2] = 7;
  $1077 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($1077|0);
 }
 $11 = HEAP32[$1>>2]|0;
 $12 = (($11) + 1|0);
 $13 = HEAP8[$12>>0]|0;
 $14 = $13&255;
 $15 = HEAP32[$1>>2]|0;
 $16 = (($15) + 18|0);
 $17 = HEAP16[$16>>1]|0;
 $18 = $17&65535;
 $19 = (($14) + ($18))|0;
 HEAP32[$i>>2] = $19;
 $20 = HEAP32[$i>>2]|0;
 $21 = ($20|0)<(2);
 if ($21) {
  HEAP32[$nxDiv>>2] = 0;
 } else {
  $22 = HEAP32[$2>>2]|0;
  $23 = ($22|0)==(0);
  do {
   if ($23) {
    HEAP32[$nxDiv>>2] = 0;
   } else {
    $24 = HEAP32[$2>>2]|0;
    $25 = HEAP32[$i>>2]|0;
    $26 = ($24|0)==($25|0);
    if ($26) {
     $27 = HEAP32[$i>>2]|0;
     $28 = (($27) - 2)|0;
     $29 = HEAP32[$5>>2]|0;
     $30 = (($28) + ($29))|0;
     HEAP32[$nxDiv>>2] = $30;
     break;
    } else {
     $31 = HEAP32[$2>>2]|0;
     $32 = (($31) - 1)|0;
     HEAP32[$nxDiv>>2] = $32;
     break;
    }
   }
  } while(0);
  $33 = HEAP32[$5>>2]|0;
  $34 = (2 - ($33))|0;
  HEAP32[$i>>2] = $34;
 }
 $35 = HEAP32[$i>>2]|0;
 $36 = (($35) + 1)|0;
 HEAP32[$nOld>>2] = $36;
 $37 = HEAP32[$i>>2]|0;
 $38 = HEAP32[$nxDiv>>2]|0;
 $39 = (($37) + ($38))|0;
 $40 = HEAP32[$1>>2]|0;
 $41 = (($40) + 1|0);
 $42 = HEAP8[$41>>0]|0;
 $43 = $42&255;
 $44 = (($39) - ($43))|0;
 $45 = HEAP32[$1>>2]|0;
 $46 = (($45) + 18|0);
 $47 = HEAP16[$46>>1]|0;
 $48 = $47&65535;
 $49 = ($44|0)==($48|0);
 $50 = HEAP32[$1>>2]|0;
 if ($49) {
  $51 = (($50) + 6|0);
  $52 = HEAP8[$51>>0]|0;
  $53 = $52&255;
  $54 = (($53) + 8)|0;
  $55 = HEAP32[$1>>2]|0;
  $56 = (($55) + 56|0);
  $57 = HEAP32[$56>>2]|0;
  $58 = (($57) + ($54)|0);
  HEAP32[$pRight>>2] = $58;
 } else {
  $59 = (($50) + 56|0);
  $60 = HEAP32[$59>>2]|0;
  $61 = HEAP32[$1>>2]|0;
  $62 = (($61) + 20|0);
  $63 = HEAP16[$62>>1]|0;
  $64 = $63&65535;
  $65 = HEAP32[$i>>2]|0;
  $66 = HEAP32[$nxDiv>>2]|0;
  $67 = (($65) + ($66))|0;
  $68 = HEAP32[$1>>2]|0;
  $69 = (($68) + 1|0);
  $70 = HEAP8[$69>>0]|0;
  $71 = $70&255;
  $72 = (($67) - ($71))|0;
  $73 = $72<<1;
  $74 = HEAP32[$1>>2]|0;
  $75 = (($74) + 64|0);
  $76 = HEAP32[$75>>2]|0;
  $77 = (($76) + ($73)|0);
  $78 = HEAP8[$77>>0]|0;
  $79 = $78&255;
  $80 = $79 << 8;
  $81 = HEAP32[$i>>2]|0;
  $82 = HEAP32[$nxDiv>>2]|0;
  $83 = (($81) + ($82))|0;
  $84 = HEAP32[$1>>2]|0;
  $85 = (($84) + 1|0);
  $86 = HEAP8[$85>>0]|0;
  $87 = $86&255;
  $88 = (($83) - ($87))|0;
  $89 = $88<<1;
  $90 = HEAP32[$1>>2]|0;
  $91 = (($90) + 64|0);
  $92 = HEAP32[$91>>2]|0;
  $93 = (($92) + ($89)|0);
  $94 = (($93) + 1|0);
  $95 = HEAP8[$94>>0]|0;
  $96 = $95&255;
  $97 = $80 | $96;
  $98 = $64 & $97;
  $99 = (($60) + ($98)|0);
  HEAP32[$pRight>>2] = $99;
 }
 $100 = HEAP32[$pRight>>2]|0;
 $101 = (_sqlite3Get4byte($100)|0);
 HEAP32[$pgno>>2] = $101;
 while(1) {
  $102 = HEAP32[$pBt>>2]|0;
  $103 = HEAP32[$pgno>>2]|0;
  $104 = HEAP32[$i>>2]|0;
  $105 = (($apOld) + ($104<<2)|0);
  $106 = (_getAndInitPage($102,$103,$105,0)|0);
  HEAP32[$rc>>2] = $106;
  $107 = HEAP32[$rc>>2]|0;
  $108 = ($107|0)!=(0);
  if ($108) {
   label = 16;
   break;
  }
  $112 = HEAP32[$i>>2]|0;
  $113 = (($apOld) + ($112<<2)|0);
  $114 = HEAP32[$113>>2]|0;
  $115 = (($114) + 18|0);
  $116 = HEAP16[$115>>1]|0;
  $117 = $116&65535;
  $118 = (1 + ($117))|0;
  $119 = HEAP32[$i>>2]|0;
  $120 = (($apOld) + ($119<<2)|0);
  $121 = HEAP32[$120>>2]|0;
  $122 = (($121) + 1|0);
  $123 = HEAP8[$122>>0]|0;
  $124 = $123&255;
  $125 = (($118) + ($124))|0;
  $126 = HEAP32[$nMaxCells>>2]|0;
  $127 = (($126) + ($125))|0;
  HEAP32[$nMaxCells>>2] = $127;
  $128 = HEAP32[$i>>2]|0;
  $129 = (($128) + -1)|0;
  HEAP32[$i>>2] = $129;
  $130 = ($128|0)==(0);
  if ($130) {
   label = 26;
   break;
  }
  $131 = HEAP32[$i>>2]|0;
  $132 = HEAP32[$nxDiv>>2]|0;
  $133 = (($131) + ($132))|0;
  $134 = HEAP32[$1>>2]|0;
  $135 = (($134) + 22|0);
  $136 = HEAP16[$135>>1]|0;
  $137 = $136&65535;
  $138 = ($133|0)==($137|0);
  if ($138) {
   $139 = HEAP32[$1>>2]|0;
   $140 = (($139) + 1|0);
   $141 = HEAP8[$140>>0]|0;
   $142 = $141&255;
   $143 = ($142|0)!=(0);
   if ($143) {
    $144 = HEAP32[$1>>2]|0;
    $145 = (($144) + 32|0);
    $146 = HEAP32[$145>>2]|0;
    $147 = HEAP32[$i>>2]|0;
    $148 = (($apDiv) + ($147<<2)|0);
    HEAP32[$148>>2] = $146;
    $149 = HEAP32[$i>>2]|0;
    $150 = (($apDiv) + ($149<<2)|0);
    $151 = HEAP32[$150>>2]|0;
    $152 = (_sqlite3Get4byte($151)|0);
    HEAP32[$pgno>>2] = $152;
    $153 = HEAP32[$1>>2]|0;
    $154 = HEAP32[$i>>2]|0;
    $155 = (($apDiv) + ($154<<2)|0);
    $156 = HEAP32[$155>>2]|0;
    $157 = (_cellSizePtr($153,$156)|0);
    $158 = $157&65535;
    $159 = HEAP32[$i>>2]|0;
    $160 = (($szNew) + ($159<<2)|0);
    HEAP32[$160>>2] = $158;
    $161 = HEAP32[$1>>2]|0;
    $162 = (($161) + 1|0);
    HEAP8[$162>>0] = 0;
    continue;
   }
  }
  $163 = HEAP32[$1>>2]|0;
  $164 = (($163) + 56|0);
  $165 = HEAP32[$164>>2]|0;
  $166 = HEAP32[$1>>2]|0;
  $167 = (($166) + 20|0);
  $168 = HEAP16[$167>>1]|0;
  $169 = $168&65535;
  $170 = HEAP32[$i>>2]|0;
  $171 = HEAP32[$nxDiv>>2]|0;
  $172 = (($170) + ($171))|0;
  $173 = HEAP32[$1>>2]|0;
  $174 = (($173) + 1|0);
  $175 = HEAP8[$174>>0]|0;
  $176 = $175&255;
  $177 = (($172) - ($176))|0;
  $178 = $177<<1;
  $179 = HEAP32[$1>>2]|0;
  $180 = (($179) + 64|0);
  $181 = HEAP32[$180>>2]|0;
  $182 = (($181) + ($178)|0);
  $183 = HEAP8[$182>>0]|0;
  $184 = $183&255;
  $185 = $184 << 8;
  $186 = HEAP32[$i>>2]|0;
  $187 = HEAP32[$nxDiv>>2]|0;
  $188 = (($186) + ($187))|0;
  $189 = HEAP32[$1>>2]|0;
  $190 = (($189) + 1|0);
  $191 = HEAP8[$190>>0]|0;
  $192 = $191&255;
  $193 = (($188) - ($192))|0;
  $194 = $193<<1;
  $195 = HEAP32[$1>>2]|0;
  $196 = (($195) + 64|0);
  $197 = HEAP32[$196>>2]|0;
  $198 = (($197) + ($194)|0);
  $199 = (($198) + 1|0);
  $200 = HEAP8[$199>>0]|0;
  $201 = $200&255;
  $202 = $185 | $201;
  $203 = $169 & $202;
  $204 = (($165) + ($203)|0);
  $205 = HEAP32[$i>>2]|0;
  $206 = (($apDiv) + ($205<<2)|0);
  HEAP32[$206>>2] = $204;
  $207 = HEAP32[$i>>2]|0;
  $208 = (($apDiv) + ($207<<2)|0);
  $209 = HEAP32[$208>>2]|0;
  $210 = (_sqlite3Get4byte($209)|0);
  HEAP32[$pgno>>2] = $210;
  $211 = HEAP32[$1>>2]|0;
  $212 = HEAP32[$i>>2]|0;
  $213 = (($apDiv) + ($212<<2)|0);
  $214 = HEAP32[$213>>2]|0;
  $215 = (_cellSizePtr($211,$214)|0);
  $216 = $215&65535;
  $217 = HEAP32[$i>>2]|0;
  $218 = (($szNew) + ($217<<2)|0);
  HEAP32[$218>>2] = $216;
  $219 = HEAP32[$pBt>>2]|0;
  $220 = (($219) + 22|0);
  $221 = HEAP16[$220>>1]|0;
  $222 = $221&65535;
  $223 = $222 & 4;
  $224 = ($223|0)!=(0);
  if ($224) {
   $225 = HEAP32[$i>>2]|0;
   $226 = (($apDiv) + ($225<<2)|0);
   $227 = HEAP32[$226>>2]|0;
   $228 = $227;
   $229 = HEAP32[$1>>2]|0;
   $230 = (($229) + 56|0);
   $231 = HEAP32[$230>>2]|0;
   $232 = $231;
   $233 = (($228) - ($232))|0;
   HEAP32[$iOff>>2] = $233;
   $234 = HEAP32[$iOff>>2]|0;
   $235 = HEAP32[$i>>2]|0;
   $236 = (($szNew) + ($235<<2)|0);
   $237 = HEAP32[$236>>2]|0;
   $238 = (($234) + ($237))|0;
   $239 = HEAP32[$pBt>>2]|0;
   $240 = (($239) + 36|0);
   $241 = HEAP32[$240>>2]|0;
   $242 = ($238|0)>($241|0);
   if ($242) {
    label = 23;
    break;
   }
   $247 = HEAP32[$iOff>>2]|0;
   $248 = HEAP32[$3>>2]|0;
   $249 = (($248) + ($247)|0);
   $250 = HEAP32[$i>>2]|0;
   $251 = (($apDiv) + ($250<<2)|0);
   $252 = HEAP32[$251>>2]|0;
   $253 = HEAP32[$i>>2]|0;
   $254 = (($szNew) + ($253<<2)|0);
   $255 = HEAP32[$254>>2]|0;
   _memcpy(($249|0),($252|0),($255|0))|0;
   $256 = HEAP32[$i>>2]|0;
   $257 = (($apDiv) + ($256<<2)|0);
   $258 = HEAP32[$257>>2]|0;
   $259 = HEAP32[$1>>2]|0;
   $260 = (($259) + 56|0);
   $261 = HEAP32[$260>>2]|0;
   $262 = $258;
   $263 = $261;
   $264 = (($262) - ($263))|0;
   $265 = HEAP32[$3>>2]|0;
   $266 = (($265) + ($264)|0);
   $267 = HEAP32[$i>>2]|0;
   $268 = (($apDiv) + ($267<<2)|0);
   HEAP32[$268>>2] = $266;
  }
  $269 = HEAP32[$1>>2]|0;
  $270 = HEAP32[$i>>2]|0;
  $271 = HEAP32[$nxDiv>>2]|0;
  $272 = (($270) + ($271))|0;
  $273 = HEAP32[$1>>2]|0;
  $274 = (($273) + 1|0);
  $275 = HEAP8[$274>>0]|0;
  $276 = $275&255;
  $277 = (($272) - ($276))|0;
  $278 = HEAP32[$i>>2]|0;
  $279 = (($szNew) + ($278<<2)|0);
  $280 = HEAP32[$279>>2]|0;
  _dropCell($269,$277,$280,$rc);
 }
 L32: do {
  if ((label|0) == 16) {
   $109 = HEAP32[$i>>2]|0;
   $110 = (($109) + 1)|0;
   $111 = $110<<2;
   _memset(($apOld|0),0,($111|0))|0;
  }
  else if ((label|0) == 23) {
   $243 = (_sqlite3CorruptError(58592)|0);
   HEAP32[$rc>>2] = $243;
   $244 = HEAP32[$i>>2]|0;
   $245 = (($244) + 1)|0;
   $246 = $245<<2;
   _memset(($apOld|0),0,($246|0))|0;
  }
  else if ((label|0) == 26) {
   $281 = HEAP32[$nMaxCells>>2]|0;
   $282 = (($281) + 3)|0;
   $283 = $282 & -4;
   HEAP32[$nMaxCells>>2] = $283;
   $284 = HEAP32[$pBt>>2]|0;
   $285 = (($284) + 32|0);
   $286 = HEAP32[$285>>2]|0;
   $287 = (($286) + 80)|0;
   HEAP32[$k>>2] = $287;
   $288 = HEAP32[$nMaxCells>>2]|0;
   $289 = $288<<2;
   $290 = HEAP32[$nMaxCells>>2]|0;
   $291 = $290<<1;
   $292 = (($289) + ($291))|0;
   $293 = HEAP32[$pBt>>2]|0;
   $294 = (($293) + 32|0);
   $295 = HEAP32[$294>>2]|0;
   $296 = (($292) + ($295))|0;
   $297 = HEAP32[$k>>2]|0;
   $298 = HEAP32[$nOld>>2]|0;
   $299 = Math_imul($297, $298)|0;
   $300 = (($296) + ($299))|0;
   HEAP32[$szScratch>>2] = $300;
   $301 = HEAP32[$szScratch>>2]|0;
   $302 = (_sqlite3ScratchMalloc($301)|0);
   HEAP32[$apCell>>2] = $302;
   $303 = HEAP32[$apCell>>2]|0;
   $304 = ($303|0)==(0|0);
   if ($304) {
    HEAP32[$rc>>2] = 7;
    break;
   }
   $305 = HEAP32[$nMaxCells>>2]|0;
   $306 = HEAP32[$apCell>>2]|0;
   $307 = (($306) + ($305<<2)|0);
   HEAP32[$szCell>>2] = $307;
   $308 = HEAP32[$nMaxCells>>2]|0;
   $309 = HEAP32[$szCell>>2]|0;
   $310 = (($309) + ($308<<1)|0);
   HEAP32[$aSpace1>>2] = $310;
   $311 = HEAP32[$apOld>>2]|0;
   $312 = (($311) + 5|0);
   $313 = HEAP8[$312>>0]|0;
   $314 = $313&255;
   $315 = $314<<2;
   $316 = $315&65535;
   HEAP16[$leafCorrection>>1] = $316;
   $317 = HEAP32[$apOld>>2]|0;
   $318 = (($317) + 3|0);
   $319 = HEAP8[$318>>0]|0;
   $320 = $319&255;
   HEAP32[$leafData>>2] = $320;
   HEAP32[$i>>2] = 0;
   while(1) {
    $321 = HEAP32[$i>>2]|0;
    $322 = HEAP32[$nOld>>2]|0;
    $323 = ($321|0)<($322|0);
    $324 = HEAP32[$pBt>>2]|0;
    if (!($323)) {
     break;
    }
    $325 = (($324) + 32|0);
    $326 = HEAP32[$325>>2]|0;
    $327 = HEAP32[$k>>2]|0;
    $328 = HEAP32[$i>>2]|0;
    $329 = Math_imul($327, $328)|0;
    $330 = (($326) + ($329))|0;
    $331 = HEAP32[$aSpace1>>2]|0;
    $332 = (($331) + ($330)|0);
    $333 = HEAP32[$i>>2]|0;
    $334 = (($apCopy) + ($333<<2)|0);
    HEAP32[$334>>2] = $332;
    HEAP32[$pOld>>2] = $332;
    $335 = HEAP32[$pOld>>2]|0;
    $336 = HEAP32[$i>>2]|0;
    $337 = (($apOld) + ($336<<2)|0);
    $338 = HEAP32[$337>>2]|0;
    dest=$335+0|0; src=$338+0|0; stop=dest+76|0; do { HEAP32[dest>>2]=HEAP32[src>>2]|0; dest=dest+4|0; src=src+4|0; } while ((dest|0) < (stop|0));
    $339 = HEAP32[$pOld>>2]|0;
    $340 = (($339) + 76|0);
    $341 = HEAP32[$pOld>>2]|0;
    $342 = (($341) + 56|0);
    HEAP32[$342>>2] = $340;
    $343 = HEAP32[$pOld>>2]|0;
    $344 = (($343) + 56|0);
    $345 = HEAP32[$344>>2]|0;
    $346 = HEAP32[$i>>2]|0;
    $347 = (($apOld) + ($346<<2)|0);
    $348 = HEAP32[$347>>2]|0;
    $349 = (($348) + 56|0);
    $350 = HEAP32[$349>>2]|0;
    $351 = HEAP32[$pBt>>2]|0;
    $352 = (($351) + 32|0);
    $353 = HEAP32[$352>>2]|0;
    _memcpy(($345|0),($350|0),($353|0))|0;
    $354 = HEAP32[$pOld>>2]|0;
    $355 = (($354) + 18|0);
    $356 = HEAP16[$355>>1]|0;
    $357 = $356&65535;
    $358 = HEAP32[$pOld>>2]|0;
    $359 = (($358) + 1|0);
    $360 = HEAP8[$359>>0]|0;
    $361 = $360&255;
    $362 = (($357) + ($361))|0;
    HEAP32[$limit>>2] = $362;
    $363 = HEAP32[$pOld>>2]|0;
    $364 = (($363) + 1|0);
    $365 = HEAP8[$364>>0]|0;
    $366 = $365&255;
    $367 = ($366|0)>(0);
    L42: do {
     if ($367) {
      HEAP32[$j>>2] = 0;
      while(1) {
       $368 = HEAP32[$j>>2]|0;
       $369 = HEAP32[$limit>>2]|0;
       $370 = ($368|0)<($369|0);
       if (!($370)) {
        break L42;
       }
       $371 = HEAP32[$pOld>>2]|0;
       $372 = HEAP32[$j>>2]|0;
       $373 = (_findOverflowCell($371,$372)|0);
       $374 = HEAP32[$nCell>>2]|0;
       $375 = HEAP32[$apCell>>2]|0;
       $376 = (($375) + ($374<<2)|0);
       HEAP32[$376>>2] = $373;
       $377 = HEAP32[$pOld>>2]|0;
       $378 = HEAP32[$nCell>>2]|0;
       $379 = HEAP32[$apCell>>2]|0;
       $380 = (($379) + ($378<<2)|0);
       $381 = HEAP32[$380>>2]|0;
       $382 = (_cellSizePtr($377,$381)|0);
       $383 = HEAP32[$nCell>>2]|0;
       $384 = HEAP32[$szCell>>2]|0;
       $385 = (($384) + ($383<<1)|0);
       HEAP16[$385>>1] = $382;
       $386 = HEAP32[$nCell>>2]|0;
       $387 = (($386) + 1)|0;
       HEAP32[$nCell>>2] = $387;
       $388 = HEAP32[$j>>2]|0;
       $389 = (($388) + 1)|0;
       HEAP32[$j>>2] = $389;
      }
     } else {
      $390 = HEAP32[$pOld>>2]|0;
      $391 = (($390) + 56|0);
      $392 = HEAP32[$391>>2]|0;
      HEAP32[$aData>>2] = $392;
      $393 = HEAP32[$pOld>>2]|0;
      $394 = (($393) + 20|0);
      $395 = HEAP16[$394>>1]|0;
      HEAP16[$maskPage>>1] = $395;
      $396 = HEAP32[$pOld>>2]|0;
      $397 = (($396) + 14|0);
      $398 = HEAP16[$397>>1]|0;
      HEAP16[$cellOffset>>1] = $398;
      HEAP32[$j>>2] = 0;
      while(1) {
       $399 = HEAP32[$j>>2]|0;
       $400 = HEAP32[$limit>>2]|0;
       $401 = ($399|0)<($400|0);
       if (!($401)) {
        break L42;
       }
       $402 = HEAP32[$aData>>2]|0;
       $403 = HEAP16[$maskPage>>1]|0;
       $404 = $403&65535;
       $405 = HEAP32[$aData>>2]|0;
       $406 = HEAP16[$cellOffset>>1]|0;
       $407 = $406&65535;
       $408 = HEAP32[$j>>2]|0;
       $409 = $408<<1;
       $410 = (($407) + ($409))|0;
       $411 = (($405) + ($410)|0);
       $412 = HEAP8[$411>>0]|0;
       $413 = $412&255;
       $414 = $413 << 8;
       $415 = HEAP32[$aData>>2]|0;
       $416 = HEAP16[$cellOffset>>1]|0;
       $417 = $416&65535;
       $418 = HEAP32[$j>>2]|0;
       $419 = $418<<1;
       $420 = (($417) + ($419))|0;
       $421 = (($415) + ($420)|0);
       $422 = (($421) + 1|0);
       $423 = HEAP8[$422>>0]|0;
       $424 = $423&255;
       $425 = $414 | $424;
       $426 = $404 & $425;
       $427 = (($402) + ($426)|0);
       $428 = HEAP32[$nCell>>2]|0;
       $429 = HEAP32[$apCell>>2]|0;
       $430 = (($429) + ($428<<2)|0);
       HEAP32[$430>>2] = $427;
       $431 = HEAP32[$pOld>>2]|0;
       $432 = HEAP32[$nCell>>2]|0;
       $433 = HEAP32[$apCell>>2]|0;
       $434 = (($433) + ($432<<2)|0);
       $435 = HEAP32[$434>>2]|0;
       $436 = (_cellSizePtr($431,$435)|0);
       $437 = HEAP32[$nCell>>2]|0;
       $438 = HEAP32[$szCell>>2]|0;
       $439 = (($438) + ($437<<1)|0);
       HEAP16[$439>>1] = $436;
       $440 = HEAP32[$nCell>>2]|0;
       $441 = (($440) + 1)|0;
       HEAP32[$nCell>>2] = $441;
       $442 = HEAP32[$j>>2]|0;
       $443 = (($442) + 1)|0;
       HEAP32[$j>>2] = $443;
      }
     }
    } while(0);
    $444 = HEAP32[$i>>2]|0;
    $445 = HEAP32[$nOld>>2]|0;
    $446 = (($445) - 1)|0;
    $447 = ($444|0)>=($446|0);
    $448 = HEAP32[$leafData>>2]|0;
    $449 = ($448|0)!=(0);
    $or$cond = $447 | $449;
    if (!($or$cond)) {
     $450 = HEAP32[$i>>2]|0;
     $451 = (($szNew) + ($450<<2)|0);
     $452 = HEAP32[$451>>2]|0;
     $453 = $452&65535;
     HEAP16[$sz>>1] = $453;
     $454 = HEAP16[$sz>>1]|0;
     $455 = HEAP32[$nCell>>2]|0;
     $456 = HEAP32[$szCell>>2]|0;
     $457 = (($456) + ($455<<1)|0);
     HEAP16[$457>>1] = $454;
     $458 = HEAP32[$iSpace1>>2]|0;
     $459 = HEAP32[$aSpace1>>2]|0;
     $460 = (($459) + ($458)|0);
     HEAP32[$pTemp>>2] = $460;
     $461 = HEAP16[$sz>>1]|0;
     $462 = $461&65535;
     $463 = HEAP32[$iSpace1>>2]|0;
     $464 = (($463) + ($462))|0;
     HEAP32[$iSpace1>>2] = $464;
     $465 = HEAP32[$pTemp>>2]|0;
     $466 = HEAP32[$i>>2]|0;
     $467 = (($apDiv) + ($466<<2)|0);
     $468 = HEAP32[$467>>2]|0;
     $469 = HEAP16[$sz>>1]|0;
     $470 = $469&65535;
     _memcpy(($465|0),($468|0),($470|0))|0;
     $471 = HEAP32[$pTemp>>2]|0;
     $472 = HEAP16[$leafCorrection>>1]|0;
     $473 = $472&65535;
     $474 = (($471) + ($473)|0);
     $475 = HEAP32[$nCell>>2]|0;
     $476 = HEAP32[$apCell>>2]|0;
     $477 = (($476) + ($475<<2)|0);
     HEAP32[$477>>2] = $474;
     $478 = HEAP32[$nCell>>2]|0;
     $479 = HEAP32[$szCell>>2]|0;
     $480 = (($479) + ($478<<1)|0);
     $481 = HEAP16[$480>>1]|0;
     $482 = $481&65535;
     $483 = HEAP16[$leafCorrection>>1]|0;
     $484 = $483&65535;
     $485 = (($482) - ($484))|0;
     $486 = $485&65535;
     $487 = HEAP32[$nCell>>2]|0;
     $488 = HEAP32[$szCell>>2]|0;
     $489 = (($488) + ($487<<1)|0);
     HEAP16[$489>>1] = $486;
     $490 = HEAP32[$pOld>>2]|0;
     $491 = (($490) + 5|0);
     $492 = HEAP8[$491>>0]|0;
     $493 = ($492<<24>>24)!=(0);
     $494 = HEAP32[$nCell>>2]|0;
     if ($493) {
      $502 = HEAP32[$szCell>>2]|0;
      $503 = (($502) + ($494<<1)|0);
      $504 = HEAP16[$503>>1]|0;
      $505 = $504&65535;
      $506 = ($505|0)<(4);
      if ($506) {
       $507 = HEAP32[$nCell>>2]|0;
       $508 = HEAP32[$szCell>>2]|0;
       $509 = (($508) + ($507<<1)|0);
       HEAP16[$509>>1] = 4;
      }
     } else {
      $495 = HEAP32[$apCell>>2]|0;
      $496 = (($495) + ($494<<2)|0);
      $497 = HEAP32[$496>>2]|0;
      $498 = HEAP32[$pOld>>2]|0;
      $499 = (($498) + 56|0);
      $500 = HEAP32[$499>>2]|0;
      $501 = (($500) + 8|0);
      ;HEAP8[$497+0>>0]=HEAP8[$501+0>>0]|0;HEAP8[$497+1>>0]=HEAP8[$501+1>>0]|0;HEAP8[$497+2>>0]=HEAP8[$501+2>>0]|0;HEAP8[$497+3>>0]=HEAP8[$501+3>>0]|0;
     }
     $510 = HEAP32[$nCell>>2]|0;
     $511 = (($510) + 1)|0;
     HEAP32[$nCell>>2] = $511;
    }
    $512 = HEAP32[$i>>2]|0;
    $513 = (($512) + 1)|0;
    HEAP32[$i>>2] = $513;
   }
   $514 = (($324) + 36|0);
   $515 = HEAP32[$514>>2]|0;
   $516 = (($515) - 12)|0;
   $517 = HEAP16[$leafCorrection>>1]|0;
   $518 = $517&65535;
   $519 = (($516) + ($518))|0;
   HEAP32[$usableSpace>>2] = $519;
   HEAP32[$i>>2] = 0;
   HEAP32[$k>>2] = 0;
   HEAP32[$subtotal>>2] = 0;
   while(1) {
    $520 = HEAP32[$i>>2]|0;
    $521 = HEAP32[$nCell>>2]|0;
    $522 = ($520|0)<($521|0);
    if (!($522)) {
     break;
    }
    $523 = HEAP32[$i>>2]|0;
    $524 = HEAP32[$szCell>>2]|0;
    $525 = (($524) + ($523<<1)|0);
    $526 = HEAP16[$525>>1]|0;
    $527 = $526&65535;
    $528 = (($527) + 2)|0;
    $529 = HEAP32[$subtotal>>2]|0;
    $530 = (($529) + ($528))|0;
    HEAP32[$subtotal>>2] = $530;
    $531 = HEAP32[$subtotal>>2]|0;
    $532 = HEAP32[$usableSpace>>2]|0;
    $533 = ($531|0)>($532|0);
    if ($533) {
     $534 = HEAP32[$subtotal>>2]|0;
     $535 = HEAP32[$i>>2]|0;
     $536 = HEAP32[$szCell>>2]|0;
     $537 = (($536) + ($535<<1)|0);
     $538 = HEAP16[$537>>1]|0;
     $539 = $538&65535;
     $540 = (($534) - ($539))|0;
     $541 = HEAP32[$k>>2]|0;
     $542 = (($szNew) + ($541<<2)|0);
     HEAP32[$542>>2] = $540;
     $543 = HEAP32[$i>>2]|0;
     $544 = HEAP32[$k>>2]|0;
     $545 = (($cntNew) + ($544<<2)|0);
     HEAP32[$545>>2] = $543;
     $546 = HEAP32[$leafData>>2]|0;
     $547 = ($546|0)!=(0);
     if ($547) {
      $548 = HEAP32[$i>>2]|0;
      $549 = (($548) + -1)|0;
      HEAP32[$i>>2] = $549;
     }
     HEAP32[$subtotal>>2] = 0;
     $550 = HEAP32[$k>>2]|0;
     $551 = (($550) + 1)|0;
     HEAP32[$k>>2] = $551;
     $552 = HEAP32[$k>>2]|0;
     $553 = ($552|0)>(4);
     if ($553) {
      label = 50;
      break;
     }
    }
    $555 = HEAP32[$i>>2]|0;
    $556 = (($555) + 1)|0;
    HEAP32[$i>>2] = $556;
   }
   if ((label|0) == 50) {
    $554 = (_sqlite3CorruptError(58731)|0);
    HEAP32[$rc>>2] = $554;
    break;
   }
   $557 = HEAP32[$subtotal>>2]|0;
   $558 = HEAP32[$k>>2]|0;
   $559 = (($szNew) + ($558<<2)|0);
   HEAP32[$559>>2] = $557;
   $560 = HEAP32[$nCell>>2]|0;
   $561 = HEAP32[$k>>2]|0;
   $562 = (($cntNew) + ($561<<2)|0);
   HEAP32[$562>>2] = $560;
   $563 = HEAP32[$k>>2]|0;
   $564 = (($563) + 1)|0;
   HEAP32[$k>>2] = $564;
   $565 = HEAP32[$k>>2]|0;
   $566 = (($565) - 1)|0;
   HEAP32[$i>>2] = $566;
   while(1) {
    $567 = HEAP32[$i>>2]|0;
    $568 = ($567|0)>(0);
    if (!($568)) {
     break;
    }
    $569 = HEAP32[$i>>2]|0;
    $570 = (($szNew) + ($569<<2)|0);
    $571 = HEAP32[$570>>2]|0;
    HEAP32[$szRight>>2] = $571;
    $572 = HEAP32[$i>>2]|0;
    $573 = (($572) - 1)|0;
    $574 = (($szNew) + ($573<<2)|0);
    $575 = HEAP32[$574>>2]|0;
    HEAP32[$szLeft>>2] = $575;
    $576 = HEAP32[$i>>2]|0;
    $577 = (($576) - 1)|0;
    $578 = (($cntNew) + ($577<<2)|0);
    $579 = HEAP32[$578>>2]|0;
    $580 = (($579) - 1)|0;
    HEAP32[$r>>2] = $580;
    $581 = HEAP32[$r>>2]|0;
    $582 = (($581) + 1)|0;
    $583 = HEAP32[$leafData>>2]|0;
    $584 = (($582) - ($583))|0;
    HEAP32[$d>>2] = $584;
    while(1) {
     $585 = HEAP32[$szRight>>2]|0;
     $586 = ($585|0)==(0);
     if (!($586)) {
      $587 = HEAP32[$5>>2]|0;
      $588 = ($587|0)!=(0);
      if ($588) {
       break;
      }
      $589 = HEAP32[$szRight>>2]|0;
      $590 = HEAP32[$d>>2]|0;
      $591 = HEAP32[$szCell>>2]|0;
      $592 = (($591) + ($590<<1)|0);
      $593 = HEAP16[$592>>1]|0;
      $594 = $593&65535;
      $595 = (($589) + ($594))|0;
      $596 = (($595) + 2)|0;
      $597 = HEAP32[$szLeft>>2]|0;
      $598 = HEAP32[$r>>2]|0;
      $599 = HEAP32[$szCell>>2]|0;
      $600 = (($599) + ($598<<1)|0);
      $601 = HEAP16[$600>>1]|0;
      $602 = $601&65535;
      $603 = (($602) + 2)|0;
      $604 = (($597) - ($603))|0;
      $605 = ($596|0)<=($604|0);
      if (!($605)) {
       break;
      }
     }
     $606 = HEAP32[$d>>2]|0;
     $607 = HEAP32[$szCell>>2]|0;
     $608 = (($607) + ($606<<1)|0);
     $609 = HEAP16[$608>>1]|0;
     $610 = $609&65535;
     $611 = (($610) + 2)|0;
     $612 = HEAP32[$szRight>>2]|0;
     $613 = (($612) + ($611))|0;
     HEAP32[$szRight>>2] = $613;
     $614 = HEAP32[$r>>2]|0;
     $615 = HEAP32[$szCell>>2]|0;
     $616 = (($615) + ($614<<1)|0);
     $617 = HEAP16[$616>>1]|0;
     $618 = $617&65535;
     $619 = (($618) + 2)|0;
     $620 = HEAP32[$szLeft>>2]|0;
     $621 = (($620) - ($619))|0;
     HEAP32[$szLeft>>2] = $621;
     $622 = HEAP32[$i>>2]|0;
     $623 = (($622) - 1)|0;
     $624 = (($cntNew) + ($623<<2)|0);
     $625 = HEAP32[$624>>2]|0;
     $626 = (($625) + -1)|0;
     HEAP32[$624>>2] = $626;
     $627 = HEAP32[$i>>2]|0;
     $628 = (($627) - 1)|0;
     $629 = (($cntNew) + ($628<<2)|0);
     $630 = HEAP32[$629>>2]|0;
     $631 = (($630) - 1)|0;
     HEAP32[$r>>2] = $631;
     $632 = HEAP32[$r>>2]|0;
     $633 = (($632) + 1)|0;
     $634 = HEAP32[$leafData>>2]|0;
     $635 = (($633) - ($634))|0;
     HEAP32[$d>>2] = $635;
    }
    $636 = HEAP32[$szRight>>2]|0;
    $637 = HEAP32[$i>>2]|0;
    $638 = (($szNew) + ($637<<2)|0);
    HEAP32[$638>>2] = $636;
    $639 = HEAP32[$szLeft>>2]|0;
    $640 = HEAP32[$i>>2]|0;
    $641 = (($640) - 1)|0;
    $642 = (($szNew) + ($641<<2)|0);
    HEAP32[$642>>2] = $639;
    $643 = HEAP32[$i>>2]|0;
    $644 = (($643) + -1)|0;
    HEAP32[$i>>2] = $644;
   }
   $645 = HEAP32[$apOld>>2]|0;
   $646 = (($645) + 72|0);
   $647 = HEAP32[$646>>2]|0;
   $648 = ($647>>>0)<=(1);
   if ($648) {
    $649 = (_sqlite3CorruptError(58793)|0);
    HEAP32[$rc>>2] = $649;
    break;
   }
   $650 = HEAP32[$apOld>>2]|0;
   $651 = (($650) + 56|0);
   $652 = HEAP32[$651>>2]|0;
   $653 = HEAP8[$652>>0]|0;
   $654 = $653&255;
   HEAP32[$pageFlags>>2] = $654;
   HEAP32[$i>>2] = 0;
   while(1) {
    $655 = HEAP32[$i>>2]|0;
    $656 = HEAP32[$k>>2]|0;
    $657 = ($655|0)<($656|0);
    if (!($657)) {
     break;
    }
    $658 = HEAP32[$i>>2]|0;
    $659 = HEAP32[$nOld>>2]|0;
    $660 = ($658|0)<($659|0);
    if ($660) {
     $661 = HEAP32[$i>>2]|0;
     $662 = (($apOld) + ($661<<2)|0);
     $663 = HEAP32[$662>>2]|0;
     $664 = HEAP32[$i>>2]|0;
     $665 = (($apNew) + ($664<<2)|0);
     HEAP32[$665>>2] = $663;
     HEAP32[$pNew>>2] = $663;
     $666 = HEAP32[$i>>2]|0;
     $667 = (($apOld) + ($666<<2)|0);
     HEAP32[$667>>2] = 0;
     $668 = HEAP32[$pNew>>2]|0;
     $669 = (($668) + 68|0);
     $670 = HEAP32[$669>>2]|0;
     $671 = (_sqlite3PagerWrite($670)|0);
     HEAP32[$rc>>2] = $671;
     $672 = HEAP32[$nNew>>2]|0;
     $673 = (($672) + 1)|0;
     HEAP32[$nNew>>2] = $673;
     $674 = HEAP32[$rc>>2]|0;
     $675 = ($674|0)!=(0);
     if ($675) {
      break L32;
     }
    } else {
     $676 = HEAP32[$pBt>>2]|0;
     $677 = HEAP32[$5>>2]|0;
     $678 = ($677|0)!=(0);
     $679 = HEAP32[$pgno>>2]|0;
     $680 = $678 ? 1 : $679;
     $681 = (_allocateBtreePage($676,$pNew,$pgno,$680,0)|0);
     HEAP32[$rc>>2] = $681;
     $682 = HEAP32[$rc>>2]|0;
     $683 = ($682|0)!=(0);
     if ($683) {
      break L32;
     }
     $684 = HEAP32[$pNew>>2]|0;
     $685 = HEAP32[$i>>2]|0;
     $686 = (($apNew) + ($685<<2)|0);
     HEAP32[$686>>2] = $684;
     $687 = HEAP32[$nNew>>2]|0;
     $688 = (($687) + 1)|0;
     HEAP32[$nNew>>2] = $688;
     $689 = HEAP32[$pBt>>2]|0;
     $690 = (($689) + 17|0);
     $691 = HEAP8[$690>>0]|0;
     $692 = ($691<<24>>24)!=(0);
     if ($692) {
      $693 = HEAP32[$pBt>>2]|0;
      $694 = HEAP32[$pNew>>2]|0;
      $695 = (($694) + 72|0);
      $696 = HEAP32[$695>>2]|0;
      $697 = HEAP32[$1>>2]|0;
      $698 = (($697) + 72|0);
      $699 = HEAP32[$698>>2]|0;
      _ptrmapPut($693,$696,5,$699,$rc);
      $700 = HEAP32[$rc>>2]|0;
      $701 = ($700|0)!=(0);
      if ($701) {
       break L32;
      }
     }
    }
    $702 = HEAP32[$i>>2]|0;
    $703 = (($702) + 1)|0;
    HEAP32[$i>>2] = $703;
   }
   while(1) {
    $704 = HEAP32[$i>>2]|0;
    $705 = HEAP32[$nOld>>2]|0;
    $706 = ($704|0)<($705|0);
    if (!($706)) {
     break;
    }
    $707 = HEAP32[$i>>2]|0;
    $708 = (($apOld) + ($707<<2)|0);
    $709 = HEAP32[$708>>2]|0;
    _freePage($709,$rc);
    $710 = HEAP32[$rc>>2]|0;
    $711 = ($710|0)!=(0);
    if ($711) {
     break L32;
    }
    $712 = HEAP32[$i>>2]|0;
    $713 = (($apOld) + ($712<<2)|0);
    $714 = HEAP32[$713>>2]|0;
    _releasePage($714);
    $715 = HEAP32[$i>>2]|0;
    $716 = (($apOld) + ($715<<2)|0);
    HEAP32[$716>>2] = 0;
    $717 = HEAP32[$i>>2]|0;
    $718 = (($717) + 1)|0;
    HEAP32[$i>>2] = $718;
   }
   HEAP32[$i>>2] = 0;
   while(1) {
    $719 = HEAP32[$i>>2]|0;
    $720 = HEAP32[$k>>2]|0;
    $721 = (($720) - 1)|0;
    $722 = ($719|0)<($721|0);
    if (!($722)) {
     break;
    }
    $723 = HEAP32[$i>>2]|0;
    $724 = (($apNew) + ($723<<2)|0);
    $725 = HEAP32[$724>>2]|0;
    $726 = (($725) + 72|0);
    $727 = HEAP32[$726>>2]|0;
    HEAP32[$minV>>2] = $727;
    $728 = HEAP32[$i>>2]|0;
    HEAP32[$minI>>2] = $728;
    $729 = HEAP32[$i>>2]|0;
    $730 = (($729) + 1)|0;
    HEAP32[$j>>2] = $730;
    while(1) {
     $731 = HEAP32[$j>>2]|0;
     $732 = HEAP32[$k>>2]|0;
     $733 = ($731|0)<($732|0);
     if (!($733)) {
      break;
     }
     $734 = HEAP32[$j>>2]|0;
     $735 = (($apNew) + ($734<<2)|0);
     $736 = HEAP32[$735>>2]|0;
     $737 = (($736) + 72|0);
     $738 = HEAP32[$737>>2]|0;
     $739 = HEAP32[$minV>>2]|0;
     $740 = ($738>>>0)<($739>>>0);
     if ($740) {
      $741 = HEAP32[$j>>2]|0;
      HEAP32[$minI>>2] = $741;
      $742 = HEAP32[$j>>2]|0;
      $743 = (($apNew) + ($742<<2)|0);
      $744 = HEAP32[$743>>2]|0;
      $745 = (($744) + 72|0);
      $746 = HEAP32[$745>>2]|0;
      HEAP32[$minV>>2] = $746;
     }
     $747 = HEAP32[$j>>2]|0;
     $748 = (($747) + 1)|0;
     HEAP32[$j>>2] = $748;
    }
    $749 = HEAP32[$minI>>2]|0;
    $750 = HEAP32[$i>>2]|0;
    $751 = ($749|0)>($750|0);
    if ($751) {
     $752 = HEAP32[$i>>2]|0;
     $753 = (($apNew) + ($752<<2)|0);
     $754 = HEAP32[$753>>2]|0;
     HEAP32[$pT>>2] = $754;
     $755 = HEAP32[$minI>>2]|0;
     $756 = (($apNew) + ($755<<2)|0);
     $757 = HEAP32[$756>>2]|0;
     $758 = HEAP32[$i>>2]|0;
     $759 = (($apNew) + ($758<<2)|0);
     HEAP32[$759>>2] = $757;
     $760 = HEAP32[$pT>>2]|0;
     $761 = HEAP32[$minI>>2]|0;
     $762 = (($apNew) + ($761<<2)|0);
     HEAP32[$762>>2] = $760;
    }
    $763 = HEAP32[$i>>2]|0;
    $764 = (($763) + 1)|0;
    HEAP32[$i>>2] = $764;
   }
   $765 = HEAP32[$pRight>>2]|0;
   $766 = HEAP32[$nNew>>2]|0;
   $767 = (($766) - 1)|0;
   $768 = (($apNew) + ($767<<2)|0);
   $769 = HEAP32[$768>>2]|0;
   $770 = (($769) + 72|0);
   $771 = HEAP32[$770>>2]|0;
   _sqlite3Put4byte($765,$771);
   HEAP32[$j>>2] = 0;
   HEAP32[$i>>2] = 0;
   while(1) {
    $772 = HEAP32[$i>>2]|0;
    $773 = HEAP32[$nNew>>2]|0;
    $774 = ($772|0)<($773|0);
    if (!($774)) {
     break;
    }
    $775 = HEAP32[$i>>2]|0;
    $776 = (($apNew) + ($775<<2)|0);
    $777 = HEAP32[$776>>2]|0;
    HEAP32[$pNew1>>2] = $777;
    $778 = HEAP32[$pNew1>>2]|0;
    $779 = HEAP32[$pageFlags>>2]|0;
    _zeroPage($778,$779);
    $780 = HEAP32[$pNew1>>2]|0;
    $781 = HEAP32[$i>>2]|0;
    $782 = (($cntNew) + ($781<<2)|0);
    $783 = HEAP32[$782>>2]|0;
    $784 = HEAP32[$j>>2]|0;
    $785 = (($783) - ($784))|0;
    $786 = HEAP32[$j>>2]|0;
    $787 = HEAP32[$apCell>>2]|0;
    $788 = (($787) + ($786<<2)|0);
    $789 = HEAP32[$j>>2]|0;
    $790 = HEAP32[$szCell>>2]|0;
    $791 = (($790) + ($789<<1)|0);
    _assemblePage($780,$785,$788,$791);
    $792 = HEAP32[$i>>2]|0;
    $793 = (($cntNew) + ($792<<2)|0);
    $794 = HEAP32[$793>>2]|0;
    HEAP32[$j>>2] = $794;
    $795 = HEAP32[$j>>2]|0;
    $796 = HEAP32[$nCell>>2]|0;
    $797 = ($795|0)<($796|0);
    if ($797) {
     $798 = HEAP32[$j>>2]|0;
     $799 = HEAP32[$apCell>>2]|0;
     $800 = (($799) + ($798<<2)|0);
     $801 = HEAP32[$800>>2]|0;
     HEAP32[$pCell>>2] = $801;
     $802 = HEAP32[$j>>2]|0;
     $803 = HEAP32[$szCell>>2]|0;
     $804 = (($803) + ($802<<1)|0);
     $805 = HEAP16[$804>>1]|0;
     $806 = $805&65535;
     $807 = HEAP16[$leafCorrection>>1]|0;
     $808 = $807&65535;
     $809 = (($806) + ($808))|0;
     HEAP32[$sz3>>2] = $809;
     $810 = HEAP32[$iOvflSpace>>2]|0;
     $811 = HEAP32[$3>>2]|0;
     $812 = (($811) + ($810)|0);
     HEAP32[$pTemp2>>2] = $812;
     $813 = HEAP32[$pNew1>>2]|0;
     $814 = (($813) + 5|0);
     $815 = HEAP8[$814>>0]|0;
     $816 = ($815<<24>>24)!=(0);
     do {
      if ($816) {
       $822 = HEAP32[$leafData>>2]|0;
       $823 = ($822|0)!=(0);
       if ($823) {
        $824 = HEAP32[$j>>2]|0;
        $825 = (($824) + -1)|0;
        HEAP32[$j>>2] = $825;
        $826 = HEAP32[$pNew1>>2]|0;
        $827 = HEAP32[$j>>2]|0;
        $828 = HEAP32[$apCell>>2]|0;
        $829 = (($828) + ($827<<2)|0);
        $830 = HEAP32[$829>>2]|0;
        _btreeParseCellPtr($826,$830,$info);
        $831 = HEAP32[$pTemp2>>2]|0;
        HEAP32[$pCell>>2] = $831;
        $832 = HEAP32[$pCell>>2]|0;
        $833 = (($832) + 4|0);
        $834 = $info;
        $835 = $834;
        $836 = HEAP32[$835>>2]|0;
        $837 = (($834) + 4)|0;
        $838 = $837;
        $839 = HEAP32[$838>>2]|0;
        $840 = (_sqlite3PutVarint($833,$836,$839)|0);
        $841 = (4 + ($840))|0;
        HEAP32[$sz3>>2] = $841;
        HEAP32[$pTemp2>>2] = 0;
        break;
       }
       $842 = HEAP32[$pCell>>2]|0;
       $843 = (($842) + -4|0);
       HEAP32[$pCell>>2] = $843;
       $844 = HEAP32[$j>>2]|0;
       $845 = HEAP32[$szCell>>2]|0;
       $846 = (($845) + ($844<<1)|0);
       $847 = HEAP16[$846>>1]|0;
       $848 = $847&65535;
       $849 = ($848|0)==(4);
       if ($849) {
        $850 = HEAP32[$1>>2]|0;
        $851 = HEAP32[$pCell>>2]|0;
        $852 = (_cellSizePtr($850,$851)|0);
        $853 = $852&65535;
        HEAP32[$sz3>>2] = $853;
       }
      } else {
       $817 = HEAP32[$pNew1>>2]|0;
       $818 = (($817) + 56|0);
       $819 = HEAP32[$818>>2]|0;
       $820 = (($819) + 8|0);
       $821 = HEAP32[$pCell>>2]|0;
       ;HEAP8[$820+0>>0]=HEAP8[$821+0>>0]|0;HEAP8[$820+1>>0]=HEAP8[$821+1>>0]|0;HEAP8[$820+2>>0]=HEAP8[$821+2>>0]|0;HEAP8[$820+3>>0]=HEAP8[$821+3>>0]|0;
      }
     } while(0);
     $854 = HEAP32[$sz3>>2]|0;
     $855 = HEAP32[$iOvflSpace>>2]|0;
     $856 = (($855) + ($854))|0;
     HEAP32[$iOvflSpace>>2] = $856;
     $857 = HEAP32[$1>>2]|0;
     $858 = HEAP32[$nxDiv>>2]|0;
     $859 = HEAP32[$pCell>>2]|0;
     $860 = HEAP32[$sz3>>2]|0;
     $861 = HEAP32[$pTemp2>>2]|0;
     $862 = HEAP32[$pNew1>>2]|0;
     $863 = (($862) + 72|0);
     $864 = HEAP32[$863>>2]|0;
     _insertCell($857,$858,$859,$860,$861,$864,$rc);
     $865 = HEAP32[$rc>>2]|0;
     $866 = ($865|0)!=(0);
     if ($866) {
      break L32;
     }
     $867 = HEAP32[$j>>2]|0;
     $868 = (($867) + 1)|0;
     HEAP32[$j>>2] = $868;
     $869 = HEAP32[$nxDiv>>2]|0;
     $870 = (($869) + 1)|0;
     HEAP32[$nxDiv>>2] = $870;
    }
    $871 = HEAP32[$i>>2]|0;
    $872 = (($871) + 1)|0;
    HEAP32[$i>>2] = $872;
   }
   $873 = HEAP32[$pageFlags>>2]|0;
   $874 = $873 & 8;
   $875 = ($874|0)==(0);
   if ($875) {
    $876 = HEAP32[$nOld>>2]|0;
    $877 = (($876) - 1)|0;
    $878 = (($apCopy) + ($877<<2)|0);
    $879 = HEAP32[$878>>2]|0;
    $880 = (($879) + 56|0);
    $881 = HEAP32[$880>>2]|0;
    $882 = (($881) + 8|0);
    HEAP32[$zChild>>2] = $882;
    $883 = HEAP32[$nNew>>2]|0;
    $884 = (($883) - 1)|0;
    $885 = (($apNew) + ($884<<2)|0);
    $886 = HEAP32[$885>>2]|0;
    $887 = (($886) + 56|0);
    $888 = HEAP32[$887>>2]|0;
    $889 = (($888) + 8|0);
    $890 = HEAP32[$zChild>>2]|0;
    ;HEAP8[$889+0>>0]=HEAP8[$890+0>>0]|0;HEAP8[$889+1>>0]=HEAP8[$890+1>>0]|0;HEAP8[$889+2>>0]=HEAP8[$890+2>>0]|0;HEAP8[$889+3>>0]=HEAP8[$890+3>>0]|0;
   }
   $891 = HEAP32[$4>>2]|0;
   $892 = ($891|0)!=(0);
   if ($892) {
    $893 = HEAP32[$1>>2]|0;
    $894 = (($893) + 18|0);
    $895 = HEAP16[$894>>1]|0;
    $896 = $895&65535;
    $897 = ($896|0)==(0);
    if ($897) {
     $898 = HEAP32[$1>>2]|0;
     $899 = (($898) + 6|0);
     $900 = HEAP8[$899>>0]|0;
     $901 = $900&255;
     $902 = HEAP32[$apNew>>2]|0;
     $903 = (($902) + 16|0);
     $904 = HEAP16[$903>>1]|0;
     $905 = $904&65535;
     $906 = ($901|0)<=($905|0);
     if ($906) {
      $907 = HEAP32[$apNew>>2]|0;
      $908 = HEAP32[$1>>2]|0;
      _copyNodeContent($907,$908,$rc);
      $909 = HEAP32[$apNew>>2]|0;
      _freePage($909,$rc);
      break;
     }
    }
   }
   $910 = HEAP32[$pBt>>2]|0;
   $911 = (($910) + 17|0);
   $912 = HEAP8[$911>>0]|0;
   $913 = ($912<<24>>24)!=(0);
   if ($913) {
    $914 = HEAP32[$apNew>>2]|0;
    HEAP32[$pNew4>>2] = $914;
    $915 = HEAP32[$apCopy>>2]|0;
    HEAP32[$pOld5>>2] = $915;
    $916 = HEAP32[$pOld5>>2]|0;
    $917 = (($916) + 1|0);
    $918 = HEAP8[$917>>0]|0;
    $919 = $918&255;
    HEAP32[$nOverflow>>2] = $919;
    $920 = HEAP32[$pOld5>>2]|0;
    $921 = (($920) + 18|0);
    $922 = HEAP16[$921>>1]|0;
    $923 = $922&65535;
    $924 = HEAP32[$nOverflow>>2]|0;
    $925 = (($923) + ($924))|0;
    HEAP32[$iNextOld>>2] = $925;
    $926 = HEAP32[$nOverflow>>2]|0;
    $927 = ($926|0)!=(0);
    if ($927) {
     $928 = HEAP32[$pOld5>>2]|0;
     $929 = (($928) + 22|0);
     $930 = HEAP16[$929>>1]|0;
     $931 = $930&65535;
     $932 = $931;
    } else {
     $932 = -1;
    }
    HEAP32[$iOverflow>>2] = $932;
    HEAP32[$j>>2] = 0;
    HEAP32[$k>>2] = 0;
    HEAP32[$i>>2] = 0;
    while(1) {
     $933 = HEAP32[$i>>2]|0;
     $934 = HEAP32[$nCell>>2]|0;
     $935 = ($933|0)<($934|0);
     if (!($935)) {
      break;
     }
     HEAP32[$isDivider>>2] = 0;
     while(1) {
      $936 = HEAP32[$i>>2]|0;
      $937 = HEAP32[$iNextOld>>2]|0;
      $938 = ($936|0)==($937|0);
      if (!($938)) {
       break;
      }
      $939 = HEAP32[$j>>2]|0;
      $940 = (($939) + 1)|0;
      HEAP32[$j>>2] = $940;
      $941 = (($apCopy) + ($940<<2)|0);
      $942 = HEAP32[$941>>2]|0;
      HEAP32[$pOld5>>2] = $942;
      $943 = HEAP32[$i>>2]|0;
      $944 = HEAP32[$leafData>>2]|0;
      $945 = ($944|0)!=(0);
      $946 = $945 ^ 1;
      $947 = $946&1;
      $948 = (($943) + ($947))|0;
      $949 = HEAP32[$pOld5>>2]|0;
      $950 = (($949) + 18|0);
      $951 = HEAP16[$950>>1]|0;
      $952 = $951&65535;
      $953 = (($948) + ($952))|0;
      $954 = HEAP32[$pOld5>>2]|0;
      $955 = (($954) + 1|0);
      $956 = HEAP8[$955>>0]|0;
      $957 = $956&255;
      $958 = (($953) + ($957))|0;
      HEAP32[$iNextOld>>2] = $958;
      $959 = HEAP32[$pOld5>>2]|0;
      $960 = (($959) + 1|0);
      $961 = HEAP8[$960>>0]|0;
      $962 = ($961<<24>>24)!=(0);
      if ($962) {
       $963 = HEAP32[$pOld5>>2]|0;
       $964 = (($963) + 1|0);
       $965 = HEAP8[$964>>0]|0;
       $966 = $965&255;
       HEAP32[$nOverflow>>2] = $966;
       $967 = HEAP32[$i>>2]|0;
       $968 = HEAP32[$leafData>>2]|0;
       $969 = ($968|0)!=(0);
       $970 = $969 ^ 1;
       $971 = $970&1;
       $972 = (($967) + ($971))|0;
       $973 = HEAP32[$pOld5>>2]|0;
       $974 = (($973) + 22|0);
       $975 = HEAP16[$974>>1]|0;
       $976 = $975&65535;
       $977 = (($972) + ($976))|0;
       HEAP32[$iOverflow>>2] = $977;
      }
      $978 = HEAP32[$leafData>>2]|0;
      $979 = ($978|0)!=(0);
      $980 = $979 ^ 1;
      $981 = $980&1;
      HEAP32[$isDivider>>2] = $981;
     }
     $982 = HEAP32[$i>>2]|0;
     $983 = HEAP32[$iOverflow>>2]|0;
     $984 = ($982|0)==($983|0);
     if ($984) {
      HEAP32[$isDivider>>2] = 1;
      $985 = HEAP32[$nOverflow>>2]|0;
      $986 = (($985) + -1)|0;
      HEAP32[$nOverflow>>2] = $986;
      $987 = ($986|0)>(0);
      if ($987) {
       $988 = HEAP32[$iOverflow>>2]|0;
       $989 = (($988) + 1)|0;
       HEAP32[$iOverflow>>2] = $989;
      }
     }
     $990 = HEAP32[$i>>2]|0;
     $991 = HEAP32[$k>>2]|0;
     $992 = (($cntNew) + ($991<<2)|0);
     $993 = HEAP32[$992>>2]|0;
     $994 = ($990|0)==($993|0);
     if ($994) {
      $995 = HEAP32[$k>>2]|0;
      $996 = (($995) + 1)|0;
      HEAP32[$k>>2] = $996;
      $997 = (($apNew) + ($996<<2)|0);
      $998 = HEAP32[$997>>2]|0;
      HEAP32[$pNew4>>2] = $998;
      $999 = HEAP32[$leafData>>2]|0;
      $1000 = ($999|0)!=(0);
      if ($1000) {
       label = 116;
      }
     } else {
      label = 116;
     }
     do {
      if ((label|0) == 116) {
       label = 0;
       $1001 = HEAP32[$isDivider>>2]|0;
       $1002 = ($1001|0)!=(0);
       if (!($1002)) {
        $1003 = HEAP32[$pOld5>>2]|0;
        $1004 = (($1003) + 72|0);
        $1005 = HEAP32[$1004>>2]|0;
        $1006 = HEAP32[$pNew4>>2]|0;
        $1007 = (($1006) + 72|0);
        $1008 = HEAP32[$1007>>2]|0;
        $1009 = ($1005|0)!=($1008|0);
        if (!($1009)) {
         break;
        }
       }
       $1010 = HEAP16[$leafCorrection>>1]|0;
       $1011 = ($1010<<16>>16)!=(0);
       if (!($1011)) {
        $1012 = HEAP32[$pBt>>2]|0;
        $1013 = HEAP32[$i>>2]|0;
        $1014 = HEAP32[$apCell>>2]|0;
        $1015 = (($1014) + ($1013<<2)|0);
        $1016 = HEAP32[$1015>>2]|0;
        $1017 = (_sqlite3Get4byte($1016)|0);
        $1018 = HEAP32[$pNew4>>2]|0;
        $1019 = (($1018) + 72|0);
        $1020 = HEAP32[$1019>>2]|0;
        _ptrmapPut($1012,$1017,5,$1020,$rc);
       }
       $1021 = HEAP32[$i>>2]|0;
       $1022 = HEAP32[$szCell>>2]|0;
       $1023 = (($1022) + ($1021<<1)|0);
       $1024 = HEAP16[$1023>>1]|0;
       $1025 = $1024&65535;
       $1026 = HEAP32[$pNew4>>2]|0;
       $1027 = (($1026) + 12|0);
       $1028 = HEAP16[$1027>>1]|0;
       $1029 = $1028&65535;
       $1030 = ($1025|0)>($1029|0);
       if ($1030) {
        $1031 = HEAP32[$pNew4>>2]|0;
        $1032 = HEAP32[$i>>2]|0;
        $1033 = HEAP32[$apCell>>2]|0;
        $1034 = (($1033) + ($1032<<2)|0);
        $1035 = HEAP32[$1034>>2]|0;
        _ptrmapPutOvflPtr($1031,$1035,$rc);
       }
      }
     } while(0);
     $1036 = HEAP32[$i>>2]|0;
     $1037 = (($1036) + 1)|0;
     HEAP32[$i>>2] = $1037;
    }
    $1038 = HEAP16[$leafCorrection>>1]|0;
    $1039 = ($1038<<16>>16)!=(0);
    if (!($1039)) {
     HEAP32[$i>>2] = 0;
     while(1) {
      $1040 = HEAP32[$i>>2]|0;
      $1041 = HEAP32[$nNew>>2]|0;
      $1042 = ($1040|0)<($1041|0);
      if (!($1042)) {
       break L32;
      }
      $1043 = HEAP32[$i>>2]|0;
      $1044 = (($apNew) + ($1043<<2)|0);
      $1045 = HEAP32[$1044>>2]|0;
      $1046 = (($1045) + 56|0);
      $1047 = HEAP32[$1046>>2]|0;
      $1048 = (($1047) + 8|0);
      $1049 = (_sqlite3Get4byte($1048)|0);
      HEAP32[$key>>2] = $1049;
      $1050 = HEAP32[$pBt>>2]|0;
      $1051 = HEAP32[$key>>2]|0;
      $1052 = HEAP32[$i>>2]|0;
      $1053 = (($apNew) + ($1052<<2)|0);
      $1054 = HEAP32[$1053>>2]|0;
      $1055 = (($1054) + 72|0);
      $1056 = HEAP32[$1055>>2]|0;
      _ptrmapPut($1050,$1051,5,$1056,$rc);
      $1057 = HEAP32[$i>>2]|0;
      $1058 = (($1057) + 1)|0;
      HEAP32[$i>>2] = $1058;
     }
    }
   }
  }
 } while(0);
 $1059 = HEAP32[$apCell>>2]|0;
 _sqlite3ScratchFree($1059);
 HEAP32[$i>>2] = 0;
 while(1) {
  $1060 = HEAP32[$i>>2]|0;
  $1061 = HEAP32[$nOld>>2]|0;
  $1062 = ($1060|0)<($1061|0);
  if (!($1062)) {
   break;
  }
  $1063 = HEAP32[$i>>2]|0;
  $1064 = (($apOld) + ($1063<<2)|0);
  $1065 = HEAP32[$1064>>2]|0;
  _releasePage($1065);
  $1066 = HEAP32[$i>>2]|0;
  $1067 = (($1066) + 1)|0;
  HEAP32[$i>>2] = $1067;
 }
 HEAP32[$i>>2] = 0;
 while(1) {
  $1068 = HEAP32[$i>>2]|0;
  $1069 = HEAP32[$nNew>>2]|0;
  $1070 = ($1068|0)<($1069|0);
  if (!($1070)) {
   break;
  }
  $1071 = HEAP32[$i>>2]|0;
  $1072 = (($apNew) + ($1071<<2)|0);
  $1073 = HEAP32[$1072>>2]|0;
  _releasePage($1073);
  $1074 = HEAP32[$i>>2]|0;
  $1075 = (($1074) + 1)|0;
  HEAP32[$i>>2] = $1075;
 }
 $1076 = HEAP32[$rc>>2]|0;
 HEAP32[$0>>2] = $1076;
 $1077 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($1077|0);
}
function _findOverflowCell($pPage,$iCell) {
 $pPage = $pPage|0;
 $iCell = $iCell|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0;
 var $i = 0, $k = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 16|0;
 $1 = sp + 12|0;
 $2 = sp + 8|0;
 $i = sp + 4|0;
 $k = sp;
 HEAP32[$1>>2] = $pPage;
 HEAP32[$2>>2] = $iCell;
 $3 = HEAP32[$1>>2]|0;
 $4 = (($3) + 1|0);
 $5 = HEAP8[$4>>0]|0;
 $6 = $5&255;
 $7 = (($6) - 1)|0;
 HEAP32[$i>>2] = $7;
 while(1) {
  $8 = HEAP32[$i>>2]|0;
  $9 = ($8|0)>=(0);
  if (!($9)) {
   label = 8;
   break;
  }
  $10 = HEAP32[$i>>2]|0;
  $11 = HEAP32[$1>>2]|0;
  $12 = (($11) + 22|0);
  $13 = (($12) + ($10<<1)|0);
  $14 = HEAP16[$13>>1]|0;
  $15 = $14&65535;
  HEAP32[$k>>2] = $15;
  $16 = HEAP32[$k>>2]|0;
  $17 = HEAP32[$2>>2]|0;
  $18 = ($16|0)<=($17|0);
  if ($18) {
   $19 = HEAP32[$k>>2]|0;
   $20 = HEAP32[$2>>2]|0;
   $21 = ($19|0)==($20|0);
   if ($21) {
    label = 5;
    break;
   }
   $27 = HEAP32[$2>>2]|0;
   $28 = (($27) + -1)|0;
   HEAP32[$2>>2] = $28;
  }
  $29 = HEAP32[$i>>2]|0;
  $30 = (($29) + -1)|0;
  HEAP32[$i>>2] = $30;
 }
 if ((label|0) == 5) {
  $22 = HEAP32[$i>>2]|0;
  $23 = HEAP32[$1>>2]|0;
  $24 = (($23) + 32|0);
  $25 = (($24) + ($22<<2)|0);
  $26 = HEAP32[$25>>2]|0;
  HEAP32[$0>>2] = $26;
  $59 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($59|0);
 }
 else if ((label|0) == 8) {
  $31 = HEAP32[$1>>2]|0;
  $32 = (($31) + 56|0);
  $33 = HEAP32[$32>>2]|0;
  $34 = HEAP32[$1>>2]|0;
  $35 = (($34) + 20|0);
  $36 = HEAP16[$35>>1]|0;
  $37 = $36&65535;
  $38 = HEAP32[$2>>2]|0;
  $39 = $38<<1;
  $40 = HEAP32[$1>>2]|0;
  $41 = (($40) + 64|0);
  $42 = HEAP32[$41>>2]|0;
  $43 = (($42) + ($39)|0);
  $44 = HEAP8[$43>>0]|0;
  $45 = $44&255;
  $46 = $45 << 8;
  $47 = HEAP32[$2>>2]|0;
  $48 = $47<<1;
  $49 = HEAP32[$1>>2]|0;
  $50 = (($49) + 64|0);
  $51 = HEAP32[$50>>2]|0;
  $52 = (($51) + ($48)|0);
  $53 = (($52) + 1|0);
  $54 = HEAP8[$53>>0]|0;
  $55 = $54&255;
  $56 = $46 | $55;
  $57 = $37 & $56;
  $58 = (($33) + ($57)|0);
  HEAP32[$0>>2] = $58;
  $59 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($59|0);
 }
 return 0|0;
}
function _assemblePage($pPage,$nCell,$apCell,$aSize) {
 $pPage = $pPage|0;
 $nCell = $nCell|0;
 $apCell = $apCell|0;
 $aSize = $aSize|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0;
 var $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0;
 var $cellbody = 0, $data = 0, $hdr = 0, $i = 0, $nUsable = 0, $pCellptr = 0, $sz = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $0 = sp + 36|0;
 $1 = sp + 32|0;
 $2 = sp + 28|0;
 $3 = sp + 24|0;
 $i = sp + 20|0;
 $pCellptr = sp + 16|0;
 $cellbody = sp + 12|0;
 $data = sp + 8|0;
 $hdr = sp + 4|0;
 $nUsable = sp;
 $sz = sp + 40|0;
 HEAP32[$0>>2] = $pPage;
 HEAP32[$1>>2] = $nCell;
 HEAP32[$2>>2] = $apCell;
 HEAP32[$3>>2] = $aSize;
 $4 = HEAP32[$0>>2]|0;
 $5 = (($4) + 56|0);
 $6 = HEAP32[$5>>2]|0;
 HEAP32[$data>>2] = $6;
 $7 = HEAP32[$0>>2]|0;
 $8 = (($7) + 6|0);
 $9 = HEAP8[$8>>0]|0;
 $10 = $9&255;
 HEAP32[$hdr>>2] = $10;
 $11 = HEAP32[$0>>2]|0;
 $12 = (($11) + 52|0);
 $13 = HEAP32[$12>>2]|0;
 $14 = (($13) + 36|0);
 $15 = HEAP32[$14>>2]|0;
 HEAP32[$nUsable>>2] = $15;
 $16 = HEAP32[$1>>2]|0;
 $17 = $16<<1;
 $18 = HEAP32[$0>>2]|0;
 $19 = (($18) + 64|0);
 $20 = HEAP32[$19>>2]|0;
 $21 = (($20) + ($17)|0);
 HEAP32[$pCellptr>>2] = $21;
 $22 = HEAP32[$nUsable>>2]|0;
 HEAP32[$cellbody>>2] = $22;
 $23 = HEAP32[$1>>2]|0;
 $24 = (($23) - 1)|0;
 HEAP32[$i>>2] = $24;
 while(1) {
  $25 = HEAP32[$i>>2]|0;
  $26 = ($25|0)>=(0);
  if (!($26)) {
   break;
  }
  $27 = HEAP32[$i>>2]|0;
  $28 = HEAP32[$3>>2]|0;
  $29 = (($28) + ($27<<1)|0);
  $30 = HEAP16[$29>>1]|0;
  HEAP16[$sz>>1] = $30;
  $31 = HEAP32[$pCellptr>>2]|0;
  $32 = (($31) + -2|0);
  HEAP32[$pCellptr>>2] = $32;
  $33 = HEAP16[$sz>>1]|0;
  $34 = $33&65535;
  $35 = HEAP32[$cellbody>>2]|0;
  $36 = (($35) - ($34))|0;
  HEAP32[$cellbody>>2] = $36;
  $37 = HEAP32[$cellbody>>2]|0;
  $38 = $37 >> 8;
  $39 = $38&255;
  $40 = HEAP32[$pCellptr>>2]|0;
  HEAP8[$40>>0] = $39;
  $41 = HEAP32[$cellbody>>2]|0;
  $42 = $41&255;
  $43 = HEAP32[$pCellptr>>2]|0;
  $44 = (($43) + 1|0);
  HEAP8[$44>>0] = $42;
  $45 = HEAP32[$cellbody>>2]|0;
  $46 = HEAP32[$data>>2]|0;
  $47 = (($46) + ($45)|0);
  $48 = HEAP32[$i>>2]|0;
  $49 = HEAP32[$2>>2]|0;
  $50 = (($49) + ($48<<2)|0);
  $51 = HEAP32[$50>>2]|0;
  $52 = HEAP16[$sz>>1]|0;
  $53 = $52&65535;
  _memcpy(($47|0),($51|0),($53|0))|0;
  $54 = HEAP32[$i>>2]|0;
  $55 = (($54) + -1)|0;
  HEAP32[$i>>2] = $55;
 }
 $56 = HEAP32[$1>>2]|0;
 $57 = $56 >> 8;
 $58 = $57&255;
 $59 = HEAP32[$hdr>>2]|0;
 $60 = (($59) + 3)|0;
 $61 = HEAP32[$data>>2]|0;
 $62 = (($61) + ($60)|0);
 HEAP8[$62>>0] = $58;
 $63 = HEAP32[$1>>2]|0;
 $64 = $63&255;
 $65 = HEAP32[$hdr>>2]|0;
 $66 = (($65) + 3)|0;
 $67 = HEAP32[$data>>2]|0;
 $68 = (($67) + ($66)|0);
 $69 = (($68) + 1|0);
 HEAP8[$69>>0] = $64;
 $70 = HEAP32[$cellbody>>2]|0;
 $71 = $70 >> 8;
 $72 = $71&255;
 $73 = HEAP32[$hdr>>2]|0;
 $74 = (($73) + 5)|0;
 $75 = HEAP32[$data>>2]|0;
 $76 = (($75) + ($74)|0);
 HEAP8[$76>>0] = $72;
 $77 = HEAP32[$cellbody>>2]|0;
 $78 = $77&255;
 $79 = HEAP32[$hdr>>2]|0;
 $80 = (($79) + 5)|0;
 $81 = HEAP32[$data>>2]|0;
 $82 = (($81) + ($80)|0);
 $83 = (($82) + 1|0);
 HEAP8[$83>>0] = $78;
 $84 = HEAP32[$1>>2]|0;
 $85 = $84<<1;
 $86 = HEAP32[$nUsable>>2]|0;
 $87 = (($85) + ($86))|0;
 $88 = HEAP32[$cellbody>>2]|0;
 $89 = (($87) - ($88))|0;
 $90 = HEAP32[$0>>2]|0;
 $91 = (($90) + 16|0);
 $92 = HEAP16[$91>>1]|0;
 $93 = $92&65535;
 $94 = (($93) - ($89))|0;
 $95 = $94&65535;
 HEAP16[$91>>1] = $95;
 $96 = HEAP32[$1>>2]|0;
 $97 = $96&65535;
 $98 = HEAP32[$0>>2]|0;
 $99 = (($98) + 18|0);
 HEAP16[$99>>1] = $97;
 STACKTOP = sp;return;
}
function _copyNodeContent($pFrom,$pTo,$pRC) {
 $pFrom = $pFrom|0;
 $pTo = $pTo|0;
 $pRC = $pRC|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $9 = 0;
 var $aFrom = 0, $aTo = 0, $iData = 0, $iFromHdr = 0, $iToHdr = 0, $pBt = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $0 = sp + 36|0;
 $1 = sp + 32|0;
 $2 = sp + 28|0;
 $pBt = sp + 24|0;
 $aFrom = sp + 20|0;
 $aTo = sp + 16|0;
 $iFromHdr = sp + 12|0;
 $iToHdr = sp + 8|0;
 $rc = sp + 4|0;
 $iData = sp;
 HEAP32[$0>>2] = $pFrom;
 HEAP32[$1>>2] = $pTo;
 HEAP32[$2>>2] = $pRC;
 $3 = HEAP32[$2>>2]|0;
 $4 = HEAP32[$3>>2]|0;
 $5 = ($4|0)==(0);
 if (!($5)) {
  STACKTOP = sp;return;
 }
 $6 = HEAP32[$0>>2]|0;
 $7 = (($6) + 52|0);
 $8 = HEAP32[$7>>2]|0;
 HEAP32[$pBt>>2] = $8;
 $9 = HEAP32[$0>>2]|0;
 $10 = (($9) + 56|0);
 $11 = HEAP32[$10>>2]|0;
 HEAP32[$aFrom>>2] = $11;
 $12 = HEAP32[$1>>2]|0;
 $13 = (($12) + 56|0);
 $14 = HEAP32[$13>>2]|0;
 HEAP32[$aTo>>2] = $14;
 $15 = HEAP32[$0>>2]|0;
 $16 = (($15) + 6|0);
 $17 = HEAP8[$16>>0]|0;
 $18 = $17&255;
 HEAP32[$iFromHdr>>2] = $18;
 $19 = HEAP32[$1>>2]|0;
 $20 = (($19) + 72|0);
 $21 = HEAP32[$20>>2]|0;
 $22 = ($21|0)==(1);
 $23 = $22 ? 100 : 0;
 HEAP32[$iToHdr>>2] = $23;
 $24 = HEAP32[$iFromHdr>>2]|0;
 $25 = (($24) + 5)|0;
 $26 = HEAP32[$aFrom>>2]|0;
 $27 = (($26) + ($25)|0);
 $28 = HEAP8[$27>>0]|0;
 $29 = $28&255;
 $30 = $29 << 8;
 $31 = HEAP32[$iFromHdr>>2]|0;
 $32 = (($31) + 5)|0;
 $33 = HEAP32[$aFrom>>2]|0;
 $34 = (($33) + ($32)|0);
 $35 = (($34) + 1|0);
 $36 = HEAP8[$35>>0]|0;
 $37 = $36&255;
 $38 = $30 | $37;
 HEAP32[$iData>>2] = $38;
 $39 = HEAP32[$iData>>2]|0;
 $40 = HEAP32[$aTo>>2]|0;
 $41 = (($40) + ($39)|0);
 $42 = HEAP32[$iData>>2]|0;
 $43 = HEAP32[$aFrom>>2]|0;
 $44 = (($43) + ($42)|0);
 $45 = HEAP32[$pBt>>2]|0;
 $46 = (($45) + 36|0);
 $47 = HEAP32[$46>>2]|0;
 $48 = HEAP32[$iData>>2]|0;
 $49 = (($47) - ($48))|0;
 _memcpy(($41|0),($44|0),($49|0))|0;
 $50 = HEAP32[$iToHdr>>2]|0;
 $51 = HEAP32[$aTo>>2]|0;
 $52 = (($51) + ($50)|0);
 $53 = HEAP32[$iFromHdr>>2]|0;
 $54 = HEAP32[$aFrom>>2]|0;
 $55 = (($54) + ($53)|0);
 $56 = HEAP32[$0>>2]|0;
 $57 = (($56) + 14|0);
 $58 = HEAP16[$57>>1]|0;
 $59 = $58&65535;
 $60 = HEAP32[$0>>2]|0;
 $61 = (($60) + 18|0);
 $62 = HEAP16[$61>>1]|0;
 $63 = $62&65535;
 $64 = $63<<1;
 $65 = (($59) + ($64))|0;
 _memcpy(($52|0),($55|0),($65|0))|0;
 $66 = HEAP32[$1>>2]|0;
 HEAP8[$66>>0] = 0;
 $67 = HEAP32[$1>>2]|0;
 $68 = (_btreeInitPage($67)|0);
 HEAP32[$rc>>2] = $68;
 $69 = HEAP32[$rc>>2]|0;
 $70 = ($69|0)!=(0);
 if ($70) {
  $71 = HEAP32[$rc>>2]|0;
  $72 = HEAP32[$2>>2]|0;
  HEAP32[$72>>2] = $71;
  STACKTOP = sp;return;
 }
 $73 = HEAP32[$pBt>>2]|0;
 $74 = (($73) + 17|0);
 $75 = HEAP8[$74>>0]|0;
 $76 = ($75<<24>>24)!=(0);
 if (!($76)) {
  STACKTOP = sp;return;
 }
 $77 = HEAP32[$1>>2]|0;
 $78 = (_setChildPtrmaps($77)|0);
 $79 = HEAP32[$2>>2]|0;
 HEAP32[$79>>2] = $78;
 STACKTOP = sp;return;
}
function _allocateSpace($pPage,$nByte,$pIdx) {
 $pPage = $pPage|0;
 $nByte = $nByte|0;
 $pIdx = $pIdx|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0;
 var $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0, $188 = 0;
 var $189 = 0, $19 = 0, $190 = 0, $191 = 0, $192 = 0, $193 = 0, $194 = 0, $195 = 0, $196 = 0, $197 = 0, $198 = 0, $199 = 0, $2 = 0, $20 = 0, $200 = 0, $201 = 0, $202 = 0, $203 = 0, $204 = 0, $205 = 0;
 var $206 = 0, $207 = 0, $208 = 0, $209 = 0, $21 = 0, $210 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0;
 var $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0;
 var $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0;
 var $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0;
 var $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $addr = 0, $data = 0, $gap = 0, $hdr = 0, $pc = 0, $rc = 0, $size = 0, $top = 0, $usableSize = 0;
 var $x = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 64|0;
 $0 = sp + 52|0;
 $1 = sp + 48|0;
 $2 = sp + 44|0;
 $3 = sp + 40|0;
 $hdr = sp + 36|0;
 $data = sp + 32|0;
 $top = sp + 28|0;
 $gap = sp + 24|0;
 $rc = sp + 20|0;
 $usableSize = sp + 16|0;
 $pc = sp + 12|0;
 $addr = sp + 8|0;
 $size = sp + 4|0;
 $x = sp;
 HEAP32[$1>>2] = $pPage;
 HEAP32[$2>>2] = $nByte;
 HEAP32[$3>>2] = $pIdx;
 $4 = HEAP32[$1>>2]|0;
 $5 = (($4) + 6|0);
 $6 = HEAP8[$5>>0]|0;
 $7 = $6&255;
 HEAP32[$hdr>>2] = $7;
 $8 = HEAP32[$1>>2]|0;
 $9 = (($8) + 56|0);
 $10 = HEAP32[$9>>2]|0;
 HEAP32[$data>>2] = $10;
 $11 = HEAP32[$1>>2]|0;
 $12 = (($11) + 52|0);
 $13 = HEAP32[$12>>2]|0;
 $14 = (($13) + 36|0);
 $15 = HEAP32[$14>>2]|0;
 HEAP32[$usableSize>>2] = $15;
 $16 = HEAP32[$1>>2]|0;
 $17 = (($16) + 14|0);
 $18 = HEAP16[$17>>1]|0;
 $19 = $18&65535;
 $20 = HEAP32[$1>>2]|0;
 $21 = (($20) + 18|0);
 $22 = HEAP16[$21>>1]|0;
 $23 = $22&65535;
 $24 = $23<<1;
 $25 = (($19) + ($24))|0;
 HEAP32[$gap>>2] = $25;
 $26 = HEAP32[$hdr>>2]|0;
 $27 = (($26) + 5)|0;
 $28 = HEAP32[$data>>2]|0;
 $29 = (($28) + ($27)|0);
 $30 = HEAP8[$29>>0]|0;
 $31 = $30&255;
 $32 = $31 << 8;
 $33 = HEAP32[$hdr>>2]|0;
 $34 = (($33) + 5)|0;
 $35 = HEAP32[$data>>2]|0;
 $36 = (($35) + ($34)|0);
 $37 = (($36) + 1|0);
 $38 = HEAP8[$37>>0]|0;
 $39 = $38&255;
 $40 = $32 | $39;
 HEAP32[$top>>2] = $40;
 $41 = HEAP32[$gap>>2]|0;
 $42 = HEAP32[$top>>2]|0;
 $43 = ($41|0)>($42|0);
 do {
  if ($43) {
   $44 = HEAP32[$top>>2]|0;
   $45 = ($44|0)==(0);
   if ($45) {
    HEAP32[$top>>2] = 65536;
    break;
   }
   $46 = (_sqlite3CorruptError(53453)|0);
   HEAP32[$0>>2] = $46;
   $210 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($210|0);
  }
 } while(0);
 $47 = HEAP32[$gap>>2]|0;
 $48 = (($47) + 2)|0;
 $49 = HEAP32[$top>>2]|0;
 $50 = ($48|0)<=($49|0);
 L8: do {
  if ($50) {
   $51 = HEAP32[$hdr>>2]|0;
   $52 = (($51) + 1)|0;
   $53 = HEAP32[$data>>2]|0;
   $54 = (($53) + ($52)|0);
   $55 = HEAP8[$54>>0]|0;
   $56 = $55&255;
   $57 = ($56|0)!=(0);
   if (!($57)) {
    $58 = HEAP32[$hdr>>2]|0;
    $59 = (($58) + 2)|0;
    $60 = HEAP32[$data>>2]|0;
    $61 = (($60) + ($59)|0);
    $62 = HEAP8[$61>>0]|0;
    $63 = $62&255;
    $64 = ($63|0)!=(0);
    if (!($64)) {
     label = 22;
     break;
    }
   }
   $65 = HEAP32[$hdr>>2]|0;
   $66 = (($65) + 1)|0;
   HEAP32[$addr>>2] = $66;
   while(1) {
    $67 = HEAP32[$addr>>2]|0;
    $68 = HEAP32[$data>>2]|0;
    $69 = (($68) + ($67)|0);
    $70 = HEAP8[$69>>0]|0;
    $71 = $70&255;
    $72 = $71 << 8;
    $73 = HEAP32[$addr>>2]|0;
    $74 = HEAP32[$data>>2]|0;
    $75 = (($74) + ($73)|0);
    $76 = (($75) + 1|0);
    $77 = HEAP8[$76>>0]|0;
    $78 = $77&255;
    $79 = $72 | $78;
    HEAP32[$pc>>2] = $79;
    $80 = ($79|0)>(0);
    if (!($80)) {
     label = 22;
     break L8;
    }
    $81 = HEAP32[$pc>>2]|0;
    $82 = HEAP32[$usableSize>>2]|0;
    $83 = (($82) - 4)|0;
    $84 = ($81|0)>($83|0);
    if ($84) {
     label = 12;
     break;
    }
    $85 = HEAP32[$pc>>2]|0;
    $86 = HEAP32[$addr>>2]|0;
    $87 = (($86) + 4)|0;
    $88 = ($85|0)<($87|0);
    if ($88) {
     label = 12;
     break;
    }
    $90 = HEAP32[$pc>>2]|0;
    $91 = (($90) + 2)|0;
    $92 = HEAP32[$data>>2]|0;
    $93 = (($92) + ($91)|0);
    $94 = HEAP8[$93>>0]|0;
    $95 = $94&255;
    $96 = $95 << 8;
    $97 = HEAP32[$pc>>2]|0;
    $98 = (($97) + 2)|0;
    $99 = HEAP32[$data>>2]|0;
    $100 = (($99) + ($98)|0);
    $101 = (($100) + 1|0);
    $102 = HEAP8[$101>>0]|0;
    $103 = $102&255;
    $104 = $96 | $103;
    HEAP32[$size>>2] = $104;
    $105 = HEAP32[$size>>2]|0;
    $106 = HEAP32[$2>>2]|0;
    $107 = ($105|0)>=($106|0);
    if ($107) {
     break;
    }
    $161 = HEAP32[$pc>>2]|0;
    HEAP32[$addr>>2] = $161;
   }
   if ((label|0) == 12) {
    $89 = (_sqlite3CorruptError(53469)|0);
    HEAP32[$0>>2] = $89;
    $210 = HEAP32[$0>>2]|0;
    STACKTOP = sp;return ($210|0);
   }
   $108 = HEAP32[$size>>2]|0;
   $109 = HEAP32[$2>>2]|0;
   $110 = (($108) - ($109))|0;
   HEAP32[$x>>2] = $110;
   $111 = HEAP32[$x>>2]|0;
   $112 = ($111|0)<(4);
   do {
    if ($112) {
     $113 = HEAP32[$hdr>>2]|0;
     $114 = (($113) + 7)|0;
     $115 = HEAP32[$data>>2]|0;
     $116 = (($115) + ($114)|0);
     $117 = HEAP8[$116>>0]|0;
     $118 = $117&255;
     $119 = ($118|0)>=(60);
     if ($119) {
      label = 23;
      break L8;
     }
     $120 = HEAP32[$addr>>2]|0;
     $121 = HEAP32[$data>>2]|0;
     $122 = (($121) + ($120)|0);
     $123 = HEAP32[$pc>>2]|0;
     $124 = HEAP32[$data>>2]|0;
     $125 = (($124) + ($123)|0);
     ;HEAP8[$122+0>>0]=HEAP8[$125+0>>0]|0;HEAP8[$122+1>>0]=HEAP8[$125+1>>0]|0;
     $126 = HEAP32[$x>>2]|0;
     $127 = $126&255;
     $128 = $127&255;
     $129 = HEAP32[$hdr>>2]|0;
     $130 = (($129) + 7)|0;
     $131 = HEAP32[$data>>2]|0;
     $132 = (($131) + ($130)|0);
     $133 = HEAP8[$132>>0]|0;
     $134 = $133&255;
     $135 = (($134) + ($128))|0;
     $136 = $135&255;
     HEAP8[$132>>0] = $136;
    } else {
     $137 = HEAP32[$size>>2]|0;
     $138 = HEAP32[$pc>>2]|0;
     $139 = (($137) + ($138))|0;
     $140 = HEAP32[$usableSize>>2]|0;
     $141 = ($139|0)>($140|0);
     if (!($141)) {
      $143 = HEAP32[$x>>2]|0;
      $144 = $143 >> 8;
      $145 = $144&255;
      $146 = HEAP32[$pc>>2]|0;
      $147 = (($146) + 2)|0;
      $148 = HEAP32[$data>>2]|0;
      $149 = (($148) + ($147)|0);
      HEAP8[$149>>0] = $145;
      $150 = HEAP32[$x>>2]|0;
      $151 = $150&255;
      $152 = HEAP32[$pc>>2]|0;
      $153 = (($152) + 2)|0;
      $154 = HEAP32[$data>>2]|0;
      $155 = (($154) + ($153)|0);
      $156 = (($155) + 1|0);
      HEAP8[$156>>0] = $151;
      break;
     }
     $142 = (_sqlite3CorruptError(53483)|0);
     HEAP32[$0>>2] = $142;
     $210 = HEAP32[$0>>2]|0;
     STACKTOP = sp;return ($210|0);
    }
   } while(0);
   $157 = HEAP32[$pc>>2]|0;
   $158 = HEAP32[$x>>2]|0;
   $159 = (($157) + ($158))|0;
   $160 = HEAP32[$3>>2]|0;
   HEAP32[$160>>2] = $159;
   HEAP32[$0>>2] = 0;
   $210 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($210|0);
  } else {
   label = 22;
  }
 } while(0);
 if ((label|0) == 22) {
  $162 = HEAP32[$gap>>2]|0;
  $163 = (($162) + 2)|0;
  $164 = HEAP32[$2>>2]|0;
  $165 = (($163) + ($164))|0;
  $166 = HEAP32[$top>>2]|0;
  $167 = ($165|0)>($166|0);
  if ($167) {
   label = 23;
  }
 }
 do {
  if ((label|0) == 23) {
   $168 = HEAP32[$1>>2]|0;
   $169 = (_defragmentPage($168)|0);
   HEAP32[$rc>>2] = $169;
   $170 = HEAP32[$rc>>2]|0;
   $171 = ($170|0)!=(0);
   if (!($171)) {
    $173 = HEAP32[$hdr>>2]|0;
    $174 = (($173) + 5)|0;
    $175 = HEAP32[$data>>2]|0;
    $176 = (($175) + ($174)|0);
    $177 = HEAP8[$176>>0]|0;
    $178 = $177&255;
    $179 = $178 << 8;
    $180 = HEAP32[$hdr>>2]|0;
    $181 = (($180) + 5)|0;
    $182 = HEAP32[$data>>2]|0;
    $183 = (($182) + ($181)|0);
    $184 = (($183) + 1|0);
    $185 = HEAP8[$184>>0]|0;
    $186 = $185&255;
    $187 = $179 | $186;
    $188 = (($187) - 1)|0;
    $189 = $188 & 65535;
    $190 = (($189) + 1)|0;
    HEAP32[$top>>2] = $190;
    break;
   }
   $172 = HEAP32[$rc>>2]|0;
   HEAP32[$0>>2] = $172;
   $210 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($210|0);
  }
 } while(0);
 $191 = HEAP32[$2>>2]|0;
 $192 = HEAP32[$top>>2]|0;
 $193 = (($192) - ($191))|0;
 HEAP32[$top>>2] = $193;
 $194 = HEAP32[$top>>2]|0;
 $195 = $194 >> 8;
 $196 = $195&255;
 $197 = HEAP32[$hdr>>2]|0;
 $198 = (($197) + 5)|0;
 $199 = HEAP32[$data>>2]|0;
 $200 = (($199) + ($198)|0);
 HEAP8[$200>>0] = $196;
 $201 = HEAP32[$top>>2]|0;
 $202 = $201&255;
 $203 = HEAP32[$hdr>>2]|0;
 $204 = (($203) + 5)|0;
 $205 = HEAP32[$data>>2]|0;
 $206 = (($205) + ($204)|0);
 $207 = (($206) + 1|0);
 HEAP8[$207>>0] = $202;
 $208 = HEAP32[$top>>2]|0;
 $209 = HEAP32[$3>>2]|0;
 HEAP32[$209>>2] = $208;
 HEAP32[$0>>2] = 0;
 $210 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($210|0);
}
function _defragmentPage($pPage) {
 $pPage = $pPage|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0;
 var $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0;
 var $cbrk = 0, $cellOffset = 0, $data = 0, $hdr = 0, $i = 0, $iCellFirst = 0, $iCellLast = 0, $nCell = 0, $pAddr = 0, $pc = 0, $size = 0, $temp = 0, $usableSize = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 64|0;
 $0 = sp + 56|0;
 $1 = sp + 52|0;
 $i = sp + 48|0;
 $pc = sp + 44|0;
 $hdr = sp + 40|0;
 $size = sp + 36|0;
 $usableSize = sp + 32|0;
 $cellOffset = sp + 28|0;
 $cbrk = sp + 24|0;
 $nCell = sp + 20|0;
 $data = sp + 16|0;
 $temp = sp + 12|0;
 $iCellFirst = sp + 8|0;
 $iCellLast = sp + 4|0;
 $pAddr = sp;
 HEAP32[$1>>2] = $pPage;
 $2 = HEAP32[$1>>2]|0;
 $3 = (($2) + 52|0);
 $4 = HEAP32[$3>>2]|0;
 $5 = HEAP32[$4>>2]|0;
 $6 = (_sqlite3PagerTempSpace($5)|0);
 HEAP32[$temp>>2] = $6;
 $7 = HEAP32[$1>>2]|0;
 $8 = (($7) + 56|0);
 $9 = HEAP32[$8>>2]|0;
 HEAP32[$data>>2] = $9;
 $10 = HEAP32[$1>>2]|0;
 $11 = (($10) + 6|0);
 $12 = HEAP8[$11>>0]|0;
 $13 = $12&255;
 HEAP32[$hdr>>2] = $13;
 $14 = HEAP32[$1>>2]|0;
 $15 = (($14) + 14|0);
 $16 = HEAP16[$15>>1]|0;
 $17 = $16&65535;
 HEAP32[$cellOffset>>2] = $17;
 $18 = HEAP32[$1>>2]|0;
 $19 = (($18) + 18|0);
 $20 = HEAP16[$19>>1]|0;
 $21 = $20&65535;
 HEAP32[$nCell>>2] = $21;
 $22 = HEAP32[$1>>2]|0;
 $23 = (($22) + 52|0);
 $24 = HEAP32[$23>>2]|0;
 $25 = (($24) + 36|0);
 $26 = HEAP32[$25>>2]|0;
 HEAP32[$usableSize>>2] = $26;
 $27 = HEAP32[$hdr>>2]|0;
 $28 = (($27) + 5)|0;
 $29 = HEAP32[$data>>2]|0;
 $30 = (($29) + ($28)|0);
 $31 = HEAP8[$30>>0]|0;
 $32 = $31&255;
 $33 = $32 << 8;
 $34 = HEAP32[$hdr>>2]|0;
 $35 = (($34) + 5)|0;
 $36 = HEAP32[$data>>2]|0;
 $37 = (($36) + ($35)|0);
 $38 = (($37) + 1|0);
 $39 = HEAP8[$38>>0]|0;
 $40 = $39&255;
 $41 = $33 | $40;
 HEAP32[$cbrk>>2] = $41;
 $42 = HEAP32[$cbrk>>2]|0;
 $43 = HEAP32[$temp>>2]|0;
 $44 = (($43) + ($42)|0);
 $45 = HEAP32[$cbrk>>2]|0;
 $46 = HEAP32[$data>>2]|0;
 $47 = (($46) + ($45)|0);
 $48 = HEAP32[$usableSize>>2]|0;
 $49 = HEAP32[$cbrk>>2]|0;
 $50 = (($48) - ($49))|0;
 _memcpy(($44|0),($47|0),($50|0))|0;
 $51 = HEAP32[$usableSize>>2]|0;
 HEAP32[$cbrk>>2] = $51;
 $52 = HEAP32[$cellOffset>>2]|0;
 $53 = HEAP32[$nCell>>2]|0;
 $54 = $53<<1;
 $55 = (($52) + ($54))|0;
 HEAP32[$iCellFirst>>2] = $55;
 $56 = HEAP32[$usableSize>>2]|0;
 $57 = (($56) - 4)|0;
 HEAP32[$iCellLast>>2] = $57;
 HEAP32[$i>>2] = 0;
 while(1) {
  $58 = HEAP32[$i>>2]|0;
  $59 = HEAP32[$nCell>>2]|0;
  $60 = ($58|0)<($59|0);
  if (!($60)) {
   label = 10;
   break;
  }
  $61 = HEAP32[$cellOffset>>2]|0;
  $62 = HEAP32[$i>>2]|0;
  $63 = $62<<1;
  $64 = (($61) + ($63))|0;
  $65 = HEAP32[$data>>2]|0;
  $66 = (($65) + ($64)|0);
  HEAP32[$pAddr>>2] = $66;
  $67 = HEAP32[$pAddr>>2]|0;
  $68 = HEAP8[$67>>0]|0;
  $69 = $68&255;
  $70 = $69 << 8;
  $71 = HEAP32[$pAddr>>2]|0;
  $72 = (($71) + 1|0);
  $73 = HEAP8[$72>>0]|0;
  $74 = $73&255;
  $75 = $70 | $74;
  HEAP32[$pc>>2] = $75;
  $76 = HEAP32[$pc>>2]|0;
  $77 = HEAP32[$iCellFirst>>2]|0;
  $78 = ($76|0)<($77|0);
  if ($78) {
   label = 5;
   break;
  }
  $79 = HEAP32[$pc>>2]|0;
  $80 = HEAP32[$iCellLast>>2]|0;
  $81 = ($79|0)>($80|0);
  if ($81) {
   label = 5;
   break;
  }
  $83 = HEAP32[$1>>2]|0;
  $84 = HEAP32[$pc>>2]|0;
  $85 = HEAP32[$temp>>2]|0;
  $86 = (($85) + ($84)|0);
  $87 = (_cellSizePtr($83,$86)|0);
  $88 = $87&65535;
  HEAP32[$size>>2] = $88;
  $89 = HEAP32[$size>>2]|0;
  $90 = HEAP32[$cbrk>>2]|0;
  $91 = (($90) - ($89))|0;
  HEAP32[$cbrk>>2] = $91;
  $92 = HEAP32[$cbrk>>2]|0;
  $93 = HEAP32[$iCellFirst>>2]|0;
  $94 = ($92|0)<($93|0);
  if ($94) {
   label = 8;
   break;
  }
  $95 = HEAP32[$pc>>2]|0;
  $96 = HEAP32[$size>>2]|0;
  $97 = (($95) + ($96))|0;
  $98 = HEAP32[$usableSize>>2]|0;
  $99 = ($97|0)>($98|0);
  if ($99) {
   label = 8;
   break;
  }
  $101 = HEAP32[$cbrk>>2]|0;
  $102 = HEAP32[$data>>2]|0;
  $103 = (($102) + ($101)|0);
  $104 = HEAP32[$pc>>2]|0;
  $105 = HEAP32[$temp>>2]|0;
  $106 = (($105) + ($104)|0);
  $107 = HEAP32[$size>>2]|0;
  _memcpy(($103|0),($106|0),($107|0))|0;
  $108 = HEAP32[$cbrk>>2]|0;
  $109 = $108 >> 8;
  $110 = $109&255;
  $111 = HEAP32[$pAddr>>2]|0;
  HEAP8[$111>>0] = $110;
  $112 = HEAP32[$cbrk>>2]|0;
  $113 = $112&255;
  $114 = HEAP32[$pAddr>>2]|0;
  $115 = (($114) + 1|0);
  HEAP8[$115>>0] = $113;
  $116 = HEAP32[$i>>2]|0;
  $117 = (($116) + 1)|0;
  HEAP32[$i>>2] = $117;
 }
 if ((label|0) == 5) {
  $82 = (_sqlite3CorruptError(53381)|0);
  HEAP32[$0>>2] = $82;
  $159 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($159|0);
 }
 else if ((label|0) == 8) {
  $100 = (_sqlite3CorruptError(53393)|0);
  HEAP32[$0>>2] = $100;
  $159 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($159|0);
 }
 else if ((label|0) == 10) {
  $118 = HEAP32[$cbrk>>2]|0;
  $119 = $118 >> 8;
  $120 = $119&255;
  $121 = HEAP32[$hdr>>2]|0;
  $122 = (($121) + 5)|0;
  $123 = HEAP32[$data>>2]|0;
  $124 = (($123) + ($122)|0);
  HEAP8[$124>>0] = $120;
  $125 = HEAP32[$cbrk>>2]|0;
  $126 = $125&255;
  $127 = HEAP32[$hdr>>2]|0;
  $128 = (($127) + 5)|0;
  $129 = HEAP32[$data>>2]|0;
  $130 = (($129) + ($128)|0);
  $131 = (($130) + 1|0);
  HEAP8[$131>>0] = $126;
  $132 = HEAP32[$hdr>>2]|0;
  $133 = (($132) + 1)|0;
  $134 = HEAP32[$data>>2]|0;
  $135 = (($134) + ($133)|0);
  HEAP8[$135>>0] = 0;
  $136 = HEAP32[$hdr>>2]|0;
  $137 = (($136) + 2)|0;
  $138 = HEAP32[$data>>2]|0;
  $139 = (($138) + ($137)|0);
  HEAP8[$139>>0] = 0;
  $140 = HEAP32[$hdr>>2]|0;
  $141 = (($140) + 7)|0;
  $142 = HEAP32[$data>>2]|0;
  $143 = (($142) + ($141)|0);
  HEAP8[$143>>0] = 0;
  $144 = HEAP32[$iCellFirst>>2]|0;
  $145 = HEAP32[$data>>2]|0;
  $146 = (($145) + ($144)|0);
  $147 = HEAP32[$cbrk>>2]|0;
  $148 = HEAP32[$iCellFirst>>2]|0;
  $149 = (($147) - ($148))|0;
  _memset(($146|0),0,($149|0))|0;
  $150 = HEAP32[$cbrk>>2]|0;
  $151 = HEAP32[$iCellFirst>>2]|0;
  $152 = (($150) - ($151))|0;
  $153 = HEAP32[$1>>2]|0;
  $154 = (($153) + 16|0);
  $155 = HEAP16[$154>>1]|0;
  $156 = $155&65535;
  $157 = ($152|0)!=($156|0);
  if ($157) {
   $158 = (_sqlite3CorruptError(53410)|0);
   HEAP32[$0>>2] = $158;
   $159 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($159|0);
  } else {
   HEAP32[$0>>2] = 0;
   $159 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($159|0);
  }
 }
 return 0|0;
}
function _sqlite3PagerTempSpace($pPager) {
 $pPager = $pPager|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 HEAP32[$0>>2] = $pPager;
 $1 = HEAP32[$0>>2]|0;
 $2 = (($1) + 200|0);
 $3 = HEAP32[$2>>2]|0;
 STACKTOP = sp;return ($3|0);
}
function _freeSpace($pPage,$iStart,$iSize) {
 $pPage = $pPage|0;
 $iStart = $iStart|0;
 $iSize = $iSize|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0;
 var $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0, $188 = 0;
 var $189 = 0, $19 = 0, $190 = 0, $191 = 0, $192 = 0, $193 = 0, $194 = 0, $195 = 0, $196 = 0, $197 = 0, $198 = 0, $199 = 0, $2 = 0, $20 = 0, $200 = 0, $201 = 0, $202 = 0, $203 = 0, $204 = 0, $205 = 0;
 var $206 = 0, $207 = 0, $208 = 0, $209 = 0, $21 = 0, $210 = 0, $211 = 0, $212 = 0, $213 = 0, $214 = 0, $215 = 0, $216 = 0, $217 = 0, $218 = 0, $219 = 0, $22 = 0, $220 = 0, $221 = 0, $222 = 0, $223 = 0;
 var $224 = 0, $225 = 0, $226 = 0, $227 = 0, $228 = 0, $229 = 0, $23 = 0, $230 = 0, $231 = 0, $232 = 0, $233 = 0, $234 = 0, $235 = 0, $236 = 0, $237 = 0, $238 = 0, $239 = 0, $24 = 0, $240 = 0, $241 = 0;
 var $242 = 0, $243 = 0, $244 = 0, $245 = 0, $246 = 0, $247 = 0, $248 = 0, $249 = 0, $25 = 0, $250 = 0, $251 = 0, $252 = 0, $253 = 0, $254 = 0, $255 = 0, $256 = 0, $257 = 0, $258 = 0, $259 = 0, $26 = 0;
 var $260 = 0, $261 = 0, $262 = 0, $263 = 0, $264 = 0, $265 = 0, $266 = 0, $267 = 0, $268 = 0, $269 = 0, $27 = 0, $270 = 0, $271 = 0, $272 = 0, $273 = 0, $274 = 0, $275 = 0, $276 = 0, $277 = 0, $278 = 0;
 var $279 = 0, $28 = 0, $280 = 0, $281 = 0, $282 = 0, $283 = 0, $284 = 0, $285 = 0, $286 = 0, $287 = 0, $288 = 0, $289 = 0, $29 = 0, $290 = 0, $291 = 0, $292 = 0, $293 = 0, $294 = 0, $295 = 0, $296 = 0;
 var $297 = 0, $298 = 0, $299 = 0, $3 = 0, $30 = 0, $300 = 0, $301 = 0, $302 = 0, $303 = 0, $304 = 0, $305 = 0, $306 = 0, $307 = 0, $308 = 0, $309 = 0, $31 = 0, $310 = 0, $311 = 0, $312 = 0, $313 = 0;
 var $314 = 0, $315 = 0, $316 = 0, $317 = 0, $318 = 0, $319 = 0, $32 = 0, $320 = 0, $321 = 0, $322 = 0, $323 = 0, $324 = 0, $325 = 0, $326 = 0, $327 = 0, $328 = 0, $329 = 0, $33 = 0, $330 = 0, $331 = 0;
 var $332 = 0, $333 = 0, $334 = 0, $335 = 0, $336 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0;
 var $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0;
 var $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0;
 var $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $data = 0, $hdr = 0, $iEnd = 0;
 var $iFreeBlk = 0, $iLast = 0, $iOrigSize = 0, $iPtr = 0, $iPtrEnd = 0, $nFrag = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $0 = sp + 20|0;
 $1 = sp + 16|0;
 $2 = sp + 32|0;
 $3 = sp + 30|0;
 $iPtr = sp + 28|0;
 $iFreeBlk = sp + 26|0;
 $hdr = sp + 35|0;
 $nFrag = sp + 34|0;
 $iOrigSize = sp + 24|0;
 $iLast = sp + 12|0;
 $iEnd = sp + 8|0;
 $data = sp + 4|0;
 $iPtrEnd = sp;
 HEAP32[$1>>2] = $pPage;
 HEAP16[$2>>1] = $iStart;
 HEAP16[$3>>1] = $iSize;
 HEAP8[$nFrag>>0] = 0;
 $4 = HEAP16[$3>>1]|0;
 HEAP16[$iOrigSize>>1] = $4;
 $5 = HEAP32[$1>>2]|0;
 $6 = (($5) + 52|0);
 $7 = HEAP32[$6>>2]|0;
 $8 = (($7) + 36|0);
 $9 = HEAP32[$8>>2]|0;
 $10 = (($9) - 4)|0;
 HEAP32[$iLast>>2] = $10;
 $11 = HEAP16[$2>>1]|0;
 $12 = $11&65535;
 $13 = HEAP16[$3>>1]|0;
 $14 = $13&65535;
 $15 = (($12) + ($14))|0;
 HEAP32[$iEnd>>2] = $15;
 $16 = HEAP32[$1>>2]|0;
 $17 = (($16) + 56|0);
 $18 = HEAP32[$17>>2]|0;
 HEAP32[$data>>2] = $18;
 $19 = HEAP32[$1>>2]|0;
 $20 = (($19) + 52|0);
 $21 = HEAP32[$20>>2]|0;
 $22 = (($21) + 22|0);
 $23 = HEAP16[$22>>1]|0;
 $24 = $23&65535;
 $25 = $24 & 4;
 $26 = ($25|0)!=(0);
 if ($26) {
  $27 = HEAP16[$2>>1]|0;
  $28 = $27&65535;
  $29 = HEAP32[$data>>2]|0;
  $30 = (($29) + ($28)|0);
  $31 = HEAP16[$3>>1]|0;
  $32 = $31&65535;
  _memset(($30|0),0,($32|0))|0;
 }
 $33 = HEAP32[$1>>2]|0;
 $34 = (($33) + 6|0);
 $35 = HEAP8[$34>>0]|0;
 HEAP8[$hdr>>0] = $35;
 $36 = HEAP8[$hdr>>0]|0;
 $37 = $36&255;
 $38 = (($37) + 1)|0;
 $39 = $38&65535;
 HEAP16[$iPtr>>1] = $39;
 $40 = HEAP16[$iPtr>>1]|0;
 $41 = $40&65535;
 $42 = (($41) + 1)|0;
 $43 = HEAP32[$data>>2]|0;
 $44 = (($43) + ($42)|0);
 $45 = HEAP8[$44>>0]|0;
 $46 = $45&255;
 $47 = ($46|0)==(0);
 if ($47) {
  $48 = HEAP16[$iPtr>>1]|0;
  $49 = $48&65535;
  $50 = HEAP32[$data>>2]|0;
  $51 = (($50) + ($49)|0);
  $52 = HEAP8[$51>>0]|0;
  $53 = $52&255;
  $54 = ($53|0)==(0);
  if ($54) {
   HEAP16[$iFreeBlk>>1] = 0;
  } else {
   label = 6;
  }
 } else {
  label = 6;
 }
 do {
  if ((label|0) == 6) {
   while(1) {
    label = 0;
    $55 = HEAP16[$iPtr>>1]|0;
    $56 = $55&65535;
    $57 = HEAP32[$data>>2]|0;
    $58 = (($57) + ($56)|0);
    $59 = HEAP8[$58>>0]|0;
    $60 = $59&255;
    $61 = $60 << 8;
    $62 = HEAP16[$iPtr>>1]|0;
    $63 = $62&65535;
    $64 = HEAP32[$data>>2]|0;
    $65 = (($64) + ($63)|0);
    $66 = (($65) + 1|0);
    $67 = HEAP8[$66>>0]|0;
    $68 = $67&255;
    $69 = $61 | $68;
    $70 = $69&65535;
    HEAP16[$iFreeBlk>>1] = $70;
    $71 = $70&65535;
    $72 = ($71|0)>(0);
    if ($72) {
     $73 = HEAP16[$iFreeBlk>>1]|0;
     $74 = $73&65535;
     $75 = HEAP16[$2>>1]|0;
     $76 = $75&65535;
     $77 = ($74|0)<($76|0);
     $336 = $77;
    } else {
     $336 = 0;
    }
    $78 = HEAP16[$iFreeBlk>>1]|0;
    $79 = $78&65535;
    if (!($336)) {
     break;
    }
    $80 = HEAP16[$iPtr>>1]|0;
    $81 = $80&65535;
    $82 = (($81) + 4)|0;
    $83 = ($79|0)<($82|0);
    if ($83) {
     label = 10;
     break;
    }
    $85 = HEAP16[$iFreeBlk>>1]|0;
    HEAP16[$iPtr>>1] = $85;
    label = 6;
   }
   if ((label|0) == 10) {
    $84 = (_sqlite3CorruptError(53568)|0);
    HEAP32[$0>>2] = $84;
    $335 = HEAP32[$0>>2]|0;
    STACKTOP = sp;return ($335|0);
   }
   $86 = HEAP32[$iLast>>2]|0;
   $87 = ($79>>>0)>($86>>>0);
   if ($87) {
    $88 = (_sqlite3CorruptError(53571)|0);
    HEAP32[$0>>2] = $88;
    $335 = HEAP32[$0>>2]|0;
    STACKTOP = sp;return ($335|0);
   }
   $89 = HEAP16[$iFreeBlk>>1]|0;
   $90 = $89&65535;
   $91 = ($90|0)!=(0);
   do {
    if ($91) {
     $92 = HEAP32[$iEnd>>2]|0;
     $93 = (($92) + 3)|0;
     $94 = HEAP16[$iFreeBlk>>1]|0;
     $95 = $94&65535;
     $96 = ($93>>>0)>=($95>>>0);
     if ($96) {
      $97 = HEAP16[$iFreeBlk>>1]|0;
      $98 = $97&65535;
      $99 = HEAP32[$iEnd>>2]|0;
      $100 = (($98) - ($99))|0;
      $101 = $100&255;
      HEAP8[$nFrag>>0] = $101;
      $102 = HEAP32[$iEnd>>2]|0;
      $103 = HEAP16[$iFreeBlk>>1]|0;
      $104 = $103&65535;
      $105 = ($102>>>0)>($104>>>0);
      if (!($105)) {
       $107 = HEAP16[$iFreeBlk>>1]|0;
       $108 = $107&65535;
       $109 = HEAP16[$iFreeBlk>>1]|0;
       $110 = $109&65535;
       $111 = (($110) + 2)|0;
       $112 = HEAP32[$data>>2]|0;
       $113 = (($112) + ($111)|0);
       $114 = HEAP8[$113>>0]|0;
       $115 = $114&255;
       $116 = $115 << 8;
       $117 = HEAP16[$iFreeBlk>>1]|0;
       $118 = $117&65535;
       $119 = (($118) + 2)|0;
       $120 = HEAP32[$data>>2]|0;
       $121 = (($120) + ($119)|0);
       $122 = (($121) + 1|0);
       $123 = HEAP8[$122>>0]|0;
       $124 = $123&255;
       $125 = $116 | $124;
       $126 = (($108) + ($125))|0;
       HEAP32[$iEnd>>2] = $126;
       $127 = HEAP32[$iEnd>>2]|0;
       $128 = HEAP16[$2>>1]|0;
       $129 = $128&65535;
       $130 = (($127) - ($129))|0;
       $131 = $130&65535;
       HEAP16[$3>>1] = $131;
       $132 = HEAP16[$iFreeBlk>>1]|0;
       $133 = $132&65535;
       $134 = HEAP32[$data>>2]|0;
       $135 = (($134) + ($133)|0);
       $136 = HEAP8[$135>>0]|0;
       $137 = $136&255;
       $138 = $137 << 8;
       $139 = HEAP16[$iFreeBlk>>1]|0;
       $140 = $139&65535;
       $141 = HEAP32[$data>>2]|0;
       $142 = (($141) + ($140)|0);
       $143 = (($142) + 1|0);
       $144 = HEAP8[$143>>0]|0;
       $145 = $144&255;
       $146 = $138 | $145;
       $147 = $146&65535;
       HEAP16[$iFreeBlk>>1] = $147;
       break;
      }
      $106 = (_sqlite3CorruptError(53582)|0);
      HEAP32[$0>>2] = $106;
      $335 = HEAP32[$0>>2]|0;
      STACKTOP = sp;return ($335|0);
     }
    }
   } while(0);
   $148 = HEAP16[$iPtr>>1]|0;
   $149 = $148&65535;
   $150 = HEAP8[$hdr>>0]|0;
   $151 = $150&255;
   $152 = (($151) + 1)|0;
   $153 = ($149|0)>($152|0);
   do {
    if ($153) {
     $154 = HEAP16[$iPtr>>1]|0;
     $155 = $154&65535;
     $156 = HEAP16[$iPtr>>1]|0;
     $157 = $156&65535;
     $158 = (($157) + 2)|0;
     $159 = HEAP32[$data>>2]|0;
     $160 = (($159) + ($158)|0);
     $161 = HEAP8[$160>>0]|0;
     $162 = $161&255;
     $163 = $162 << 8;
     $164 = HEAP16[$iPtr>>1]|0;
     $165 = $164&65535;
     $166 = (($165) + 2)|0;
     $167 = HEAP32[$data>>2]|0;
     $168 = (($167) + ($166)|0);
     $169 = (($168) + 1|0);
     $170 = HEAP8[$169>>0]|0;
     $171 = $170&255;
     $172 = $163 | $171;
     $173 = (($155) + ($172))|0;
     HEAP32[$iPtrEnd>>2] = $173;
     $174 = HEAP32[$iPtrEnd>>2]|0;
     $175 = (($174) + 3)|0;
     $176 = HEAP16[$2>>1]|0;
     $177 = $176&65535;
     $178 = ($175|0)>=($177|0);
     if ($178) {
      $179 = HEAP32[$iPtrEnd>>2]|0;
      $180 = HEAP16[$2>>1]|0;
      $181 = $180&65535;
      $182 = ($179|0)>($181|0);
      if (!($182)) {
       $184 = HEAP16[$2>>1]|0;
       $185 = $184&65535;
       $186 = HEAP32[$iPtrEnd>>2]|0;
       $187 = (($185) - ($186))|0;
       $188 = HEAP8[$nFrag>>0]|0;
       $189 = $188&255;
       $190 = (($189) + ($187))|0;
       $191 = $190&255;
       HEAP8[$nFrag>>0] = $191;
       $192 = HEAP32[$iEnd>>2]|0;
       $193 = HEAP16[$iPtr>>1]|0;
       $194 = $193&65535;
       $195 = (($192) - ($194))|0;
       $196 = $195&65535;
       HEAP16[$3>>1] = $196;
       $197 = HEAP16[$iPtr>>1]|0;
       HEAP16[$2>>1] = $197;
       break;
      }
      $183 = (_sqlite3CorruptError(53595)|0);
      HEAP32[$0>>2] = $183;
      $335 = HEAP32[$0>>2]|0;
      STACKTOP = sp;return ($335|0);
     }
    }
   } while(0);
   $198 = HEAP8[$nFrag>>0]|0;
   $199 = $198&255;
   $200 = HEAP8[$hdr>>0]|0;
   $201 = $200&255;
   $202 = (($201) + 7)|0;
   $203 = HEAP32[$data>>2]|0;
   $204 = (($203) + ($202)|0);
   $205 = HEAP8[$204>>0]|0;
   $206 = $205&255;
   $207 = ($199|0)>($206|0);
   if (!($207)) {
    $209 = HEAP8[$nFrag>>0]|0;
    $210 = $209&255;
    $211 = HEAP8[$hdr>>0]|0;
    $212 = $211&255;
    $213 = (($212) + 7)|0;
    $214 = HEAP32[$data>>2]|0;
    $215 = (($214) + ($213)|0);
    $216 = HEAP8[$215>>0]|0;
    $217 = $216&255;
    $218 = (($217) - ($210))|0;
    $219 = $218&255;
    HEAP8[$215>>0] = $219;
    break;
   }
   $208 = (_sqlite3CorruptError(53601)|0);
   HEAP32[$0>>2] = $208;
   $335 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($335|0);
  }
 } while(0);
 $220 = HEAP16[$2>>1]|0;
 $221 = $220&65535;
 $222 = HEAP8[$hdr>>0]|0;
 $223 = $222&255;
 $224 = (($223) + 5)|0;
 $225 = HEAP32[$data>>2]|0;
 $226 = (($225) + ($224)|0);
 $227 = HEAP8[$226>>0]|0;
 $228 = $227&255;
 $229 = $228 << 8;
 $230 = HEAP8[$hdr>>0]|0;
 $231 = $230&255;
 $232 = (($231) + 5)|0;
 $233 = HEAP32[$data>>2]|0;
 $234 = (($233) + ($232)|0);
 $235 = (($234) + 1|0);
 $236 = HEAP8[$235>>0]|0;
 $237 = $236&255;
 $238 = $229 | $237;
 $239 = ($221|0)==($238|0);
 do {
  if ($239) {
   $240 = HEAP16[$iPtr>>1]|0;
   $241 = $240&65535;
   $242 = HEAP8[$hdr>>0]|0;
   $243 = $242&255;
   $244 = (($243) + 1)|0;
   $245 = ($241|0)!=($244|0);
   if (!($245)) {
    $247 = HEAP16[$iFreeBlk>>1]|0;
    $248 = $247&65535;
    $249 = $248 >> 8;
    $250 = $249&255;
    $251 = HEAP8[$hdr>>0]|0;
    $252 = $251&255;
    $253 = (($252) + 1)|0;
    $254 = HEAP32[$data>>2]|0;
    $255 = (($254) + ($253)|0);
    HEAP8[$255>>0] = $250;
    $256 = HEAP16[$iFreeBlk>>1]|0;
    $257 = $256&255;
    $258 = HEAP8[$hdr>>0]|0;
    $259 = $258&255;
    $260 = (($259) + 1)|0;
    $261 = HEAP32[$data>>2]|0;
    $262 = (($261) + ($260)|0);
    $263 = (($262) + 1|0);
    HEAP8[$263>>0] = $257;
    $264 = HEAP32[$iEnd>>2]|0;
    $265 = $264 >>> 8;
    $266 = $265&255;
    $267 = HEAP8[$hdr>>0]|0;
    $268 = $267&255;
    $269 = (($268) + 5)|0;
    $270 = HEAP32[$data>>2]|0;
    $271 = (($270) + ($269)|0);
    HEAP8[$271>>0] = $266;
    $272 = HEAP32[$iEnd>>2]|0;
    $273 = $272&255;
    $274 = HEAP8[$hdr>>0]|0;
    $275 = $274&255;
    $276 = (($275) + 5)|0;
    $277 = HEAP32[$data>>2]|0;
    $278 = (($277) + ($276)|0);
    $279 = (($278) + 1|0);
    HEAP8[$279>>0] = $273;
    break;
   }
   $246 = (_sqlite3CorruptError(53608)|0);
   HEAP32[$0>>2] = $246;
   $335 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($335|0);
  } else {
   $280 = HEAP16[$2>>1]|0;
   $281 = $280&65535;
   $282 = $281 >> 8;
   $283 = $282&255;
   $284 = HEAP16[$iPtr>>1]|0;
   $285 = $284&65535;
   $286 = HEAP32[$data>>2]|0;
   $287 = (($286) + ($285)|0);
   HEAP8[$287>>0] = $283;
   $288 = HEAP16[$2>>1]|0;
   $289 = $288&255;
   $290 = HEAP16[$iPtr>>1]|0;
   $291 = $290&65535;
   $292 = HEAP32[$data>>2]|0;
   $293 = (($292) + ($291)|0);
   $294 = (($293) + 1|0);
   HEAP8[$294>>0] = $289;
   $295 = HEAP16[$iFreeBlk>>1]|0;
   $296 = $295&65535;
   $297 = $296 >> 8;
   $298 = $297&255;
   $299 = HEAP16[$2>>1]|0;
   $300 = $299&65535;
   $301 = HEAP32[$data>>2]|0;
   $302 = (($301) + ($300)|0);
   HEAP8[$302>>0] = $298;
   $303 = HEAP16[$iFreeBlk>>1]|0;
   $304 = $303&255;
   $305 = HEAP16[$2>>1]|0;
   $306 = $305&65535;
   $307 = HEAP32[$data>>2]|0;
   $308 = (($307) + ($306)|0);
   $309 = (($308) + 1|0);
   HEAP8[$309>>0] = $304;
   $310 = HEAP16[$3>>1]|0;
   $311 = $310&65535;
   $312 = $311 >> 8;
   $313 = $312&255;
   $314 = HEAP16[$2>>1]|0;
   $315 = $314&65535;
   $316 = (($315) + 2)|0;
   $317 = HEAP32[$data>>2]|0;
   $318 = (($317) + ($316)|0);
   HEAP8[$318>>0] = $313;
   $319 = HEAP16[$3>>1]|0;
   $320 = $319&255;
   $321 = HEAP16[$2>>1]|0;
   $322 = $321&65535;
   $323 = (($322) + 2)|0;
   $324 = HEAP32[$data>>2]|0;
   $325 = (($324) + ($323)|0);
   $326 = (($325) + 1|0);
   HEAP8[$326>>0] = $320;
  }
 } while(0);
 $327 = HEAP16[$iOrigSize>>1]|0;
 $328 = $327&65535;
 $329 = HEAP32[$1>>2]|0;
 $330 = (($329) + 16|0);
 $331 = HEAP16[$330>>1]|0;
 $332 = $331&65535;
 $333 = (($332) + ($328))|0;
 $334 = $333&65535;
 HEAP16[$330>>1] = $334;
 HEAP32[$0>>2] = 0;
 $335 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($335|0);
}
function _fillInCell($pPage,$pCell,$pKey,$0,$1,$pData,$nData,$nZero,$pnSize) {
 $pPage = $pPage|0;
 $pCell = $pCell|0;
 $pKey = $pKey|0;
 $0 = $0|0;
 $1 = $1|0;
 $pData = $pData|0;
 $nData = $nData|0;
 $nZero = $nZero|0;
 $pnSize = $pnSize|0;
 var $$ = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0, $116 = 0;
 var $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0, $134 = 0;
 var $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0, $152 = 0;
 var $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0, $170 = 0;
 var $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0, $188 = 0, $189 = 0;
 var $19 = 0, $190 = 0, $191 = 0, $192 = 0, $193 = 0, $194 = 0, $195 = 0, $196 = 0, $197 = 0, $198 = 0, $199 = 0, $2 = 0, $20 = 0, $200 = 0, $201 = 0, $202 = 0, $203 = 0, $204 = 0, $205 = 0, $206 = 0;
 var $207 = 0, $208 = 0, $209 = 0, $21 = 0, $210 = 0, $211 = 0, $212 = 0, $213 = 0, $214 = 0, $215 = 0, $216 = 0, $217 = 0, $218 = 0, $219 = 0, $22 = 0, $220 = 0, $221 = 0, $222 = 0, $223 = 0, $224 = 0;
 var $225 = 0, $226 = 0, $227 = 0, $228 = 0, $229 = 0, $23 = 0, $230 = 0, $231 = 0, $232 = 0, $233 = 0, $234 = 0, $235 = 0, $236 = 0, $237 = 0, $238 = 0, $239 = 0, $24 = 0, $240 = 0, $241 = 0, $242 = 0;
 var $243 = 0, $244 = 0, $245 = 0, $246 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0;
 var $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0;
 var $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0;
 var $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0;
 var $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $eType = 0, $mn = 0, $n = 0, $nHeader = 0, $nPayload = 0, $nSrc = 0, $or$cond = 0, $or$cond3 = 0, $pBt = 0, $pOvfl = 0, $pPayload = 0, $pPrior = 0, $pSrc = 0, $pToRelease = 0;
 var $pgnoOvfl = 0, $pgnoPtrmap = 0, $rc = 0, $spaceLeft = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 112|0;
 $2 = sp + 96|0;
 $3 = sp + 92|0;
 $4 = sp + 88|0;
 $5 = sp + 84|0;
 $6 = sp;
 $7 = sp + 80|0;
 $8 = sp + 76|0;
 $9 = sp + 72|0;
 $10 = sp + 68|0;
 $nPayload = sp + 64|0;
 $pSrc = sp + 60|0;
 $nSrc = sp + 56|0;
 $n = sp + 52|0;
 $rc = sp + 48|0;
 $spaceLeft = sp + 44|0;
 $pOvfl = sp + 40|0;
 $pToRelease = sp + 36|0;
 $pPrior = sp + 32|0;
 $pPayload = sp + 28|0;
 $pBt = sp + 24|0;
 $pgnoOvfl = sp + 20|0;
 $nHeader = sp + 16|0;
 $mn = sp + 12|0;
 $pgnoPtrmap = sp + 8|0;
 $eType = sp + 100|0;
 HEAP32[$3>>2] = $pPage;
 HEAP32[$4>>2] = $pCell;
 HEAP32[$5>>2] = $pKey;
 $11 = $6;
 $12 = $11;
 HEAP32[$12>>2] = $0;
 $13 = (($11) + 4)|0;
 $14 = $13;
 HEAP32[$14>>2] = $1;
 HEAP32[$7>>2] = $pData;
 HEAP32[$8>>2] = $nData;
 HEAP32[$9>>2] = $nZero;
 HEAP32[$10>>2] = $pnSize;
 HEAP32[$pOvfl>>2] = 0;
 HEAP32[$pToRelease>>2] = 0;
 $15 = HEAP32[$3>>2]|0;
 $16 = (($15) + 52|0);
 $17 = HEAP32[$16>>2]|0;
 HEAP32[$pBt>>2] = $17;
 HEAP32[$pgnoOvfl>>2] = 0;
 $18 = HEAP32[$3>>2]|0;
 $19 = (($18) + 7|0);
 $20 = HEAP8[$19>>0]|0;
 $21 = $20&255;
 HEAP32[$nHeader>>2] = $21;
 $22 = HEAP32[$8>>2]|0;
 $23 = HEAP32[$9>>2]|0;
 $24 = (($22) + ($23))|0;
 HEAP32[$nPayload>>2] = $24;
 $25 = HEAP32[$3>>2]|0;
 $26 = (($25) + 3|0);
 $27 = HEAP8[$26>>0]|0;
 $28 = ($27<<24>>24)!=(0);
 if ($28) {
  $29 = HEAP32[$nPayload>>2]|0;
  $30 = ($29>>>0)<(128);
  if ($30) {
   $31 = HEAP32[$nPayload>>2]|0;
   $32 = $31&255;
   $33 = HEAP32[$nHeader>>2]|0;
   $34 = HEAP32[$4>>2]|0;
   $35 = (($34) + ($33)|0);
   HEAP8[$35>>0] = $32;
   $44 = 1;
  } else {
   $36 = HEAP32[$nHeader>>2]|0;
   $37 = HEAP32[$4>>2]|0;
   $38 = (($37) + ($36)|0);
   $39 = HEAP32[$nPayload>>2]|0;
   $40 = ($39|0)<(0);
   $41 = $40 << 31 >> 31;
   $42 = (_sqlite3PutVarint($38,$39,$41)|0);
   $44 = $42;
  }
  $43 = $44&255;
  $45 = $43&255;
  $46 = HEAP32[$nHeader>>2]|0;
  $47 = (($46) + ($45))|0;
  HEAP32[$nHeader>>2] = $47;
 }
 $48 = HEAP32[$nHeader>>2]|0;
 $49 = HEAP32[$4>>2]|0;
 $50 = (($49) + ($48)|0);
 $51 = $6;
 $52 = $51;
 $53 = HEAP32[$52>>2]|0;
 $54 = (($51) + 4)|0;
 $55 = $54;
 $56 = HEAP32[$55>>2]|0;
 $57 = (_sqlite3PutVarint($50,$53,$56)|0);
 $58 = HEAP32[$nHeader>>2]|0;
 $59 = (($58) + ($57))|0;
 HEAP32[$nHeader>>2] = $59;
 $60 = HEAP32[$3>>2]|0;
 $61 = (($60) + 2|0);
 $62 = HEAP8[$61>>0]|0;
 $63 = ($62<<24>>24)!=(0);
 do {
  if ($63) {
   $64 = HEAP32[$7>>2]|0;
   HEAP32[$pSrc>>2] = $64;
   $65 = HEAP32[$8>>2]|0;
   HEAP32[$nSrc>>2] = $65;
   HEAP32[$8>>2] = 0;
  } else {
   $66 = $6;
   $67 = $66;
   $68 = HEAP32[$67>>2]|0;
   $69 = (($66) + 4)|0;
   $70 = $69;
   $71 = HEAP32[$70>>2]|0;
   $72 = ($71|0)>(0);
   $73 = ($68>>>0)>(2147483647);
   $74 = ($71|0)==(0);
   $75 = $74 & $73;
   $76 = $72 | $75;
   $77 = HEAP32[$5>>2]|0;
   $78 = ($77|0)==(0|0);
   $or$cond = $76 | $78;
   if (!($or$cond)) {
    $80 = $6;
    $81 = $80;
    $82 = HEAP32[$81>>2]|0;
    $83 = (($80) + 4)|0;
    $84 = $83;
    $85 = HEAP32[$84>>2]|0;
    HEAP32[$nPayload>>2] = $82;
    $86 = HEAP32[$5>>2]|0;
    HEAP32[$pSrc>>2] = $86;
    $87 = $6;
    $88 = $87;
    $89 = HEAP32[$88>>2]|0;
    $90 = (($87) + 4)|0;
    $91 = $90;
    $92 = HEAP32[$91>>2]|0;
    HEAP32[$nSrc>>2] = $89;
    break;
   }
   $79 = (_sqlite3CorruptError(57891)|0);
   HEAP32[$2>>2] = $79;
   $246 = HEAP32[$2>>2]|0;
   STACKTOP = sp;return ($246|0);
  }
 } while(0);
 $93 = HEAP32[$nPayload>>2]|0;
 $94 = HEAP32[$3>>2]|0;
 $95 = (($94) + 10|0);
 $96 = HEAP16[$95>>1]|0;
 $97 = $96&65535;
 $98 = ($93|0)<=($97|0);
 if ($98) {
  $99 = HEAP32[$nHeader>>2]|0;
  $100 = HEAP32[$nPayload>>2]|0;
  $101 = (($99) + ($100))|0;
  HEAP32[$n>>2] = $101;
  $102 = HEAP32[$n>>2]|0;
  $103 = ($102|0)<(4);
  $$ = $103 ? 4 : $101;
  HEAP32[$n>>2] = $$;
  $104 = HEAP32[$n>>2]|0;
  $105 = HEAP32[$10>>2]|0;
  HEAP32[$105>>2] = $104;
  $106 = HEAP32[$nPayload>>2]|0;
  HEAP32[$spaceLeft>>2] = $106;
  $107 = HEAP32[$4>>2]|0;
  HEAP32[$pPrior>>2] = $107;
 } else {
  $108 = HEAP32[$3>>2]|0;
  $109 = (($108) + 12|0);
  $110 = HEAP16[$109>>1]|0;
  $111 = $110&65535;
  HEAP32[$mn>>2] = $111;
  $112 = HEAP32[$mn>>2]|0;
  $113 = HEAP32[$nPayload>>2]|0;
  $114 = HEAP32[$mn>>2]|0;
  $115 = (($113) - ($114))|0;
  $116 = HEAP32[$3>>2]|0;
  $117 = (($116) + 52|0);
  $118 = HEAP32[$117>>2]|0;
  $119 = (($118) + 36|0);
  $120 = HEAP32[$119>>2]|0;
  $121 = (($120) - 4)|0;
  $122 = (($115>>>0) % ($121>>>0))&-1;
  $123 = (($112) + ($122))|0;
  HEAP32[$n>>2] = $123;
  $124 = HEAP32[$n>>2]|0;
  $125 = HEAP32[$3>>2]|0;
  $126 = (($125) + 10|0);
  $127 = HEAP16[$126>>1]|0;
  $128 = $127&65535;
  $129 = ($124|0)>($128|0);
  if ($129) {
   $130 = HEAP32[$mn>>2]|0;
   HEAP32[$n>>2] = $130;
  }
  $131 = HEAP32[$n>>2]|0;
  HEAP32[$spaceLeft>>2] = $131;
  $132 = HEAP32[$n>>2]|0;
  $133 = HEAP32[$nHeader>>2]|0;
  $134 = (($132) + ($133))|0;
  $135 = (($134) + 4)|0;
  $136 = HEAP32[$10>>2]|0;
  HEAP32[$136>>2] = $135;
  $137 = HEAP32[$nHeader>>2]|0;
  $138 = HEAP32[$n>>2]|0;
  $139 = (($137) + ($138))|0;
  $140 = HEAP32[$4>>2]|0;
  $141 = (($140) + ($139)|0);
  HEAP32[$pPrior>>2] = $141;
 }
 $142 = HEAP32[$nHeader>>2]|0;
 $143 = HEAP32[$4>>2]|0;
 $144 = (($143) + ($142)|0);
 HEAP32[$pPayload>>2] = $144;
 while(1) {
  $145 = HEAP32[$nPayload>>2]|0;
  $146 = ($145|0)>(0);
  if (!($146)) {
   label = 37;
   break;
  }
  $147 = HEAP32[$spaceLeft>>2]|0;
  $148 = ($147|0)==(0);
  if ($148) {
   $149 = HEAP32[$pgnoOvfl>>2]|0;
   HEAP32[$pgnoPtrmap>>2] = $149;
   $150 = HEAP32[$pBt>>2]|0;
   $151 = (($150) + 17|0);
   $152 = HEAP8[$151>>0]|0;
   $153 = ($152<<24>>24)!=(0);
   if ($153) {
    while(1) {
     $154 = HEAP32[$pgnoOvfl>>2]|0;
     $155 = (($154) + 1)|0;
     HEAP32[$pgnoOvfl>>2] = $155;
     $156 = HEAP32[$pBt>>2]|0;
     $157 = HEAP32[$pgnoOvfl>>2]|0;
     $158 = (_ptrmapPageno($156,$157)|0);
     $159 = HEAP32[$pgnoOvfl>>2]|0;
     $160 = ($158|0)==($159|0);
     if ($160) {
      continue;
     }
     $161 = HEAP32[$pgnoOvfl>>2]|0;
     $162 = HEAP32[1128>>2]|0;
     $163 = HEAP32[$pBt>>2]|0;
     $164 = (($163) + 32|0);
     $165 = HEAP32[$164>>2]|0;
     $166 = (($162>>>0) / ($165>>>0))&-1;
     $167 = (($166) + 1)|0;
     $168 = ($161|0)==($167|0);
     if (!($168)) {
      break;
     }
    }
   }
   $169 = HEAP32[$pBt>>2]|0;
   $170 = HEAP32[$pgnoOvfl>>2]|0;
   $171 = (_allocateBtreePage($169,$pOvfl,$pgnoOvfl,$170,0)|0);
   HEAP32[$rc>>2] = $171;
   $172 = HEAP32[$pBt>>2]|0;
   $173 = (($172) + 17|0);
   $174 = HEAP8[$173>>0]|0;
   $175 = $174&255;
   $176 = ($175|0)!=(0);
   $177 = HEAP32[$rc>>2]|0;
   $178 = ($177|0)==(0);
   $or$cond3 = $176 & $178;
   if ($or$cond3) {
    $179 = HEAP32[$pgnoPtrmap>>2]|0;
    $180 = ($179|0)!=(0);
    $181 = $180 ? 4 : 3;
    $182 = $181&255;
    HEAP8[$eType>>0] = $182;
    $183 = HEAP32[$pBt>>2]|0;
    $184 = HEAP32[$pgnoOvfl>>2]|0;
    $185 = HEAP8[$eType>>0]|0;
    $186 = HEAP32[$pgnoPtrmap>>2]|0;
    _ptrmapPut($183,$184,$185,$186,$rc);
    $187 = HEAP32[$rc>>2]|0;
    $188 = ($187|0)!=(0);
    if ($188) {
     $189 = HEAP32[$pOvfl>>2]|0;
     _releasePage($189);
    }
   }
   $190 = HEAP32[$rc>>2]|0;
   $191 = ($190|0)!=(0);
   if ($191) {
    label = 26;
    break;
   }
   $194 = HEAP32[$pPrior>>2]|0;
   $195 = HEAP32[$pgnoOvfl>>2]|0;
   _sqlite3Put4byte($194,$195);
   $196 = HEAP32[$pToRelease>>2]|0;
   _releasePage($196);
   $197 = HEAP32[$pOvfl>>2]|0;
   HEAP32[$pToRelease>>2] = $197;
   $198 = HEAP32[$pOvfl>>2]|0;
   $199 = (($198) + 56|0);
   $200 = HEAP32[$199>>2]|0;
   HEAP32[$pPrior>>2] = $200;
   $201 = HEAP32[$pPrior>>2]|0;
   _sqlite3Put4byte($201,0);
   $202 = HEAP32[$pOvfl>>2]|0;
   $203 = (($202) + 56|0);
   $204 = HEAP32[$203>>2]|0;
   $205 = (($204) + 4|0);
   HEAP32[$pPayload>>2] = $205;
   $206 = HEAP32[$pBt>>2]|0;
   $207 = (($206) + 36|0);
   $208 = HEAP32[$207>>2]|0;
   $209 = (($208) - 4)|0;
   HEAP32[$spaceLeft>>2] = $209;
  }
  $210 = HEAP32[$nPayload>>2]|0;
  HEAP32[$n>>2] = $210;
  $211 = HEAP32[$n>>2]|0;
  $212 = HEAP32[$spaceLeft>>2]|0;
  $213 = ($211|0)>($212|0);
  if ($213) {
   $214 = HEAP32[$spaceLeft>>2]|0;
   HEAP32[$n>>2] = $214;
  }
  $215 = HEAP32[$nSrc>>2]|0;
  $216 = ($215|0)>(0);
  if ($216) {
   $217 = HEAP32[$n>>2]|0;
   $218 = HEAP32[$nSrc>>2]|0;
   $219 = ($217|0)>($218|0);
   if ($219) {
    $220 = HEAP32[$nSrc>>2]|0;
    HEAP32[$n>>2] = $220;
   }
   $221 = HEAP32[$pPayload>>2]|0;
   $222 = HEAP32[$pSrc>>2]|0;
   $223 = HEAP32[$n>>2]|0;
   _memcpy(($221|0),($222|0),($223|0))|0;
  } else {
   $224 = HEAP32[$pPayload>>2]|0;
   $225 = HEAP32[$n>>2]|0;
   _memset(($224|0),0,($225|0))|0;
  }
  $226 = HEAP32[$n>>2]|0;
  $227 = HEAP32[$nPayload>>2]|0;
  $228 = (($227) - ($226))|0;
  HEAP32[$nPayload>>2] = $228;
  $229 = HEAP32[$n>>2]|0;
  $230 = HEAP32[$pPayload>>2]|0;
  $231 = (($230) + ($229)|0);
  HEAP32[$pPayload>>2] = $231;
  $232 = HEAP32[$n>>2]|0;
  $233 = HEAP32[$pSrc>>2]|0;
  $234 = (($233) + ($232)|0);
  HEAP32[$pSrc>>2] = $234;
  $235 = HEAP32[$n>>2]|0;
  $236 = HEAP32[$nSrc>>2]|0;
  $237 = (($236) - ($235))|0;
  HEAP32[$nSrc>>2] = $237;
  $238 = HEAP32[$n>>2]|0;
  $239 = HEAP32[$spaceLeft>>2]|0;
  $240 = (($239) - ($238))|0;
  HEAP32[$spaceLeft>>2] = $240;
  $241 = HEAP32[$nSrc>>2]|0;
  $242 = ($241|0)==(0);
  if (!($242)) {
   continue;
  }
  $243 = HEAP32[$8>>2]|0;
  HEAP32[$nSrc>>2] = $243;
  $244 = HEAP32[$7>>2]|0;
  HEAP32[$pSrc>>2] = $244;
 }
 if ((label|0) == 26) {
  $192 = HEAP32[$pToRelease>>2]|0;
  _releasePage($192);
  $193 = HEAP32[$rc>>2]|0;
  HEAP32[$2>>2] = $193;
  $246 = HEAP32[$2>>2]|0;
  STACKTOP = sp;return ($246|0);
 }
 else if ((label|0) == 37) {
  $245 = HEAP32[$pToRelease>>2]|0;
  _releasePage($245);
  HEAP32[$2>>2] = 0;
  $246 = HEAP32[$2>>2]|0;
  STACKTOP = sp;return ($246|0);
 }
 return 0|0;
}
function _sqlite3VdbeSorterClose($db,$pCsr) {
 $db = $db|0;
 $pCsr = $pCsr|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $pSorter = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $1 = sp + 4|0;
 $pSorter = sp;
 HEAP32[$0>>2] = $db;
 HEAP32[$1>>2] = $pCsr;
 $2 = HEAP32[$1>>2]|0;
 $3 = (($2) + 56|0);
 $4 = HEAP32[$3>>2]|0;
 HEAP32[$pSorter>>2] = $4;
 $5 = HEAP32[$pSorter>>2]|0;
 $6 = ($5|0)!=(0|0);
 if (!($6)) {
  STACKTOP = sp;return;
 }
 $7 = HEAP32[$0>>2]|0;
 $8 = HEAP32[$pSorter>>2]|0;
 _sqlite3VdbeSorterReset($7,$8);
 $9 = HEAP32[$pSorter>>2]|0;
 $10 = (($9) + 36|0);
 $11 = (($10) + 4|0);
 $12 = HEAP32[$11>>2]|0;
 _sqlite3_free($12);
 $13 = HEAP32[$0>>2]|0;
 $14 = HEAP32[$pSorter>>2]|0;
 _sqlite3DbFree($13,$14);
 $15 = HEAP32[$1>>2]|0;
 $16 = (($15) + 56|0);
 HEAP32[$16>>2] = 0;
 STACKTOP = sp;return;
}
function _btreeCreateTable($p,$piTable,$createTabFlags) {
 $p = $p|0;
 $piTable = $piTable|0;
 $createTabFlags = $createTabFlags|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0;
 var $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0;
 var $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0;
 var $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0;
 var $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $eType = 0, $iPtrPage = 0;
 var $pBt = 0, $pPageMove = 0, $pRoot = 0, $pgnoMove = 0, $pgnoRoot = 0, $ptfFlags = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 64|0;
 $0 = sp + 44|0;
 $1 = sp + 40|0;
 $2 = sp + 36|0;
 $3 = sp + 32|0;
 $pBt = sp + 28|0;
 $pRoot = sp + 24|0;
 $pgnoRoot = sp + 20|0;
 $rc = sp + 16|0;
 $ptfFlags = sp + 12|0;
 $pgnoMove = sp + 8|0;
 $pPageMove = sp + 4|0;
 $eType = sp + 48|0;
 $iPtrPage = sp;
 HEAP32[$1>>2] = $p;
 HEAP32[$2>>2] = $piTable;
 HEAP32[$3>>2] = $createTabFlags;
 $4 = HEAP32[$1>>2]|0;
 $5 = (($4) + 4|0);
 $6 = HEAP32[$5>>2]|0;
 HEAP32[$pBt>>2] = $6;
 $7 = HEAP32[$pBt>>2]|0;
 $8 = (($7) + 17|0);
 $9 = HEAP8[$8>>0]|0;
 $10 = ($9<<24>>24)!=(0);
 $11 = HEAP32[$pBt>>2]|0;
 if ($10) {
  _invalidateAllOverflowCache($11);
  $12 = HEAP32[$1>>2]|0;
  _sqlite3BtreeGetMeta($12,4,$pgnoRoot);
  $13 = HEAP32[$pgnoRoot>>2]|0;
  $14 = (($13) + 1)|0;
  HEAP32[$pgnoRoot>>2] = $14;
  while(1) {
   $15 = HEAP32[$pgnoRoot>>2]|0;
   $16 = HEAP32[$pBt>>2]|0;
   $17 = HEAP32[$pgnoRoot>>2]|0;
   $18 = (_ptrmapPageno($16,$17)|0);
   $19 = ($15|0)==($18|0);
   if (!($19)) {
    $20 = HEAP32[$pgnoRoot>>2]|0;
    $21 = HEAP32[1128>>2]|0;
    $22 = HEAP32[$pBt>>2]|0;
    $23 = (($22) + 32|0);
    $24 = HEAP32[$23>>2]|0;
    $25 = (($21>>>0) / ($24>>>0))&-1;
    $26 = (($25) + 1)|0;
    $27 = ($20|0)==($26|0);
    if (!($27)) {
     break;
    }
   }
   $28 = HEAP32[$pgnoRoot>>2]|0;
   $29 = (($28) + 1)|0;
   HEAP32[$pgnoRoot>>2] = $29;
  }
  $30 = HEAP32[$pBt>>2]|0;
  $31 = HEAP32[$pgnoRoot>>2]|0;
  $32 = (_allocateBtreePage($30,$pPageMove,$pgnoMove,$31,1)|0);
  HEAP32[$rc>>2] = $32;
  $33 = HEAP32[$rc>>2]|0;
  $34 = ($33|0)!=(0);
  if ($34) {
   $35 = HEAP32[$rc>>2]|0;
   HEAP32[$0>>2] = $35;
   $117 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($117|0);
  }
  $36 = HEAP32[$pgnoMove>>2]|0;
  $37 = HEAP32[$pgnoRoot>>2]|0;
  $38 = ($36|0)!=($37|0);
  if ($38) {
   HEAP8[$eType>>0] = 0;
   HEAP32[$iPtrPage>>2] = 0;
   $39 = HEAP32[$pBt>>2]|0;
   $40 = (_saveAllCursors($39,0,0)|0);
   HEAP32[$rc>>2] = $40;
   $41 = HEAP32[$pPageMove>>2]|0;
   _releasePage($41);
   $42 = HEAP32[$rc>>2]|0;
   $43 = ($42|0)!=(0);
   if ($43) {
    $44 = HEAP32[$rc>>2]|0;
    HEAP32[$0>>2] = $44;
    $117 = HEAP32[$0>>2]|0;
    STACKTOP = sp;return ($117|0);
   }
   $45 = HEAP32[$pBt>>2]|0;
   $46 = HEAP32[$pgnoRoot>>2]|0;
   $47 = (_btreeGetPage($45,$46,$pRoot,0)|0);
   HEAP32[$rc>>2] = $47;
   $48 = HEAP32[$rc>>2]|0;
   $49 = ($48|0)!=(0);
   if ($49) {
    $50 = HEAP32[$rc>>2]|0;
    HEAP32[$0>>2] = $50;
    $117 = HEAP32[$0>>2]|0;
    STACKTOP = sp;return ($117|0);
   }
   $51 = HEAP32[$pBt>>2]|0;
   $52 = HEAP32[$pgnoRoot>>2]|0;
   $53 = (_ptrmapGet($51,$52,$eType,$iPtrPage)|0);
   HEAP32[$rc>>2] = $53;
   $54 = HEAP8[$eType>>0]|0;
   $55 = $54&255;
   $56 = ($55|0)==(1);
   if ($56) {
    label = 15;
   } else {
    $57 = HEAP8[$eType>>0]|0;
    $58 = $57&255;
    $59 = ($58|0)==(2);
    if ($59) {
     label = 15;
    }
   }
   if ((label|0) == 15) {
    $60 = (_sqlite3CorruptError(59666)|0);
    HEAP32[$rc>>2] = $60;
   }
   $61 = HEAP32[$rc>>2]|0;
   $62 = ($61|0)!=(0);
   if ($62) {
    $63 = HEAP32[$pRoot>>2]|0;
    _releasePage($63);
    $64 = HEAP32[$rc>>2]|0;
    HEAP32[$0>>2] = $64;
    $117 = HEAP32[$0>>2]|0;
    STACKTOP = sp;return ($117|0);
   }
   $65 = HEAP32[$pBt>>2]|0;
   $66 = HEAP32[$pRoot>>2]|0;
   $67 = HEAP8[$eType>>0]|0;
   $68 = HEAP32[$iPtrPage>>2]|0;
   $69 = HEAP32[$pgnoMove>>2]|0;
   $70 = (_relocatePage($65,$66,$67,$68,$69,0)|0);
   HEAP32[$rc>>2] = $70;
   $71 = HEAP32[$pRoot>>2]|0;
   _releasePage($71);
   $72 = HEAP32[$rc>>2]|0;
   $73 = ($72|0)!=(0);
   if ($73) {
    $74 = HEAP32[$rc>>2]|0;
    HEAP32[$0>>2] = $74;
    $117 = HEAP32[$0>>2]|0;
    STACKTOP = sp;return ($117|0);
   }
   $75 = HEAP32[$pBt>>2]|0;
   $76 = HEAP32[$pgnoRoot>>2]|0;
   $77 = (_btreeGetPage($75,$76,$pRoot,0)|0);
   HEAP32[$rc>>2] = $77;
   $78 = HEAP32[$rc>>2]|0;
   $79 = ($78|0)!=(0);
   if ($79) {
    $80 = HEAP32[$rc>>2]|0;
    HEAP32[$0>>2] = $80;
    $117 = HEAP32[$0>>2]|0;
    STACKTOP = sp;return ($117|0);
   }
   $81 = HEAP32[$pRoot>>2]|0;
   $82 = (($81) + 68|0);
   $83 = HEAP32[$82>>2]|0;
   $84 = (_sqlite3PagerWrite($83)|0);
   HEAP32[$rc>>2] = $84;
   $85 = HEAP32[$rc>>2]|0;
   $86 = ($85|0)!=(0);
   if ($86) {
    $87 = HEAP32[$pRoot>>2]|0;
    _releasePage($87);
    $88 = HEAP32[$rc>>2]|0;
    HEAP32[$0>>2] = $88;
    $117 = HEAP32[$0>>2]|0;
    STACKTOP = sp;return ($117|0);
   }
  } else {
   $89 = HEAP32[$pPageMove>>2]|0;
   HEAP32[$pRoot>>2] = $89;
  }
  $90 = HEAP32[$pBt>>2]|0;
  $91 = HEAP32[$pgnoRoot>>2]|0;
  _ptrmapPut($90,$91,1,0,$rc);
  $92 = HEAP32[$rc>>2]|0;
  $93 = ($92|0)!=(0);
  if ($93) {
   $94 = HEAP32[$pRoot>>2]|0;
   _releasePage($94);
   $95 = HEAP32[$rc>>2]|0;
   HEAP32[$0>>2] = $95;
   $117 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($117|0);
  }
  $96 = HEAP32[$1>>2]|0;
  $97 = HEAP32[$pgnoRoot>>2]|0;
  $98 = (_sqlite3BtreeUpdateMeta($96,4,$97)|0);
  HEAP32[$rc>>2] = $98;
  $99 = HEAP32[$rc>>2]|0;
  $100 = ($99|0)!=(0);
  if ($100) {
   $101 = HEAP32[$pRoot>>2]|0;
   _releasePage($101);
   $102 = HEAP32[$rc>>2]|0;
   HEAP32[$0>>2] = $102;
   $117 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($117|0);
  }
 } else {
  $103 = (_allocateBtreePage($11,$pRoot,$pgnoRoot,1,0)|0);
  HEAP32[$rc>>2] = $103;
  $104 = HEAP32[$rc>>2]|0;
  $105 = ($104|0)!=(0);
  if ($105) {
   $106 = HEAP32[$rc>>2]|0;
   HEAP32[$0>>2] = $106;
   $117 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($117|0);
  }
 }
 $107 = HEAP32[$3>>2]|0;
 $108 = $107 & 1;
 $109 = ($108|0)!=(0);
 if ($109) {
  HEAP32[$ptfFlags>>2] = 13;
 } else {
  HEAP32[$ptfFlags>>2] = 10;
 }
 $110 = HEAP32[$pRoot>>2]|0;
 $111 = HEAP32[$ptfFlags>>2]|0;
 _zeroPage($110,$111);
 $112 = HEAP32[$pRoot>>2]|0;
 $113 = (($112) + 68|0);
 $114 = HEAP32[$113>>2]|0;
 _sqlite3PagerUnref($114);
 $115 = HEAP32[$pgnoRoot>>2]|0;
 $116 = HEAP32[$2>>2]|0;
 HEAP32[$116>>2] = $115;
 HEAP32[$0>>2] = 0;
 $117 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($117|0);
}
function _btreeCursor($p,$iTable,$wrFlag,$pKeyInfo,$pCur) {
 $p = $p|0;
 $iTable = $iTable|0;
 $wrFlag = $wrFlag|0;
 $pKeyInfo = $pKeyInfo|0;
 $pCur = $pCur|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $7 = 0, $8 = 0, $9 = 0, $pBt = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 24|0;
 $1 = sp + 20|0;
 $2 = sp + 16|0;
 $3 = sp + 12|0;
 $4 = sp + 8|0;
 $5 = sp + 4|0;
 $pBt = sp;
 HEAP32[$1>>2] = $p;
 HEAP32[$2>>2] = $iTable;
 HEAP32[$3>>2] = $wrFlag;
 HEAP32[$4>>2] = $pKeyInfo;
 HEAP32[$5>>2] = $pCur;
 $6 = HEAP32[$1>>2]|0;
 $7 = (($6) + 4|0);
 $8 = HEAP32[$7>>2]|0;
 HEAP32[$pBt>>2] = $8;
 $9 = HEAP32[$3>>2]|0;
 $10 = ($9|0)!=(0);
 if ($10) {
  $11 = HEAP32[$pBt>>2]|0;
  $12 = (($11) + 22|0);
  $13 = HEAP16[$12>>1]|0;
  $14 = $13&65535;
  $15 = $14 & 1;
  $16 = ($15|0)!=(0);
  if ($16) {
   HEAP32[$0>>2] = 8;
   $65 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($65|0);
  }
 }
 $17 = HEAP32[$3>>2]|0;
 $18 = ($17|0)!=(0);
 if ($18) {
  $19 = HEAP32[$pBt>>2]|0;
  _allocateTempSpace($19);
  $20 = HEAP32[$pBt>>2]|0;
  $21 = (($20) + 80|0);
  $22 = HEAP32[$21>>2]|0;
  $23 = ($22|0)==(0|0);
  if ($23) {
   HEAP32[$0>>2] = 7;
   $65 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($65|0);
  }
 }
 $24 = HEAP32[$2>>2]|0;
 $25 = ($24|0)==(1);
 if ($25) {
  $26 = HEAP32[$pBt>>2]|0;
  $27 = (_btreePagecount($26)|0);
  $28 = ($27|0)==(0);
  if ($28) {
   HEAP32[$2>>2] = 0;
  }
 }
 $29 = HEAP32[$2>>2]|0;
 $30 = HEAP32[$5>>2]|0;
 $31 = (($30) + 60|0);
 HEAP32[$31>>2] = $29;
 $32 = HEAP32[$5>>2]|0;
 $33 = (($32) + 76|0);
 HEAP16[$33>>1] = -1;
 $34 = HEAP32[$4>>2]|0;
 $35 = HEAP32[$5>>2]|0;
 $36 = (($35) + 16|0);
 HEAP32[$36>>2] = $34;
 $37 = HEAP32[$1>>2]|0;
 $38 = HEAP32[$5>>2]|0;
 HEAP32[$38>>2] = $37;
 $39 = HEAP32[$pBt>>2]|0;
 $40 = HEAP32[$5>>2]|0;
 $41 = (($40) + 4|0);
 HEAP32[$41>>2] = $39;
 $42 = HEAP32[$3>>2]|0;
 $43 = $42&255;
 $44 = HEAP32[$5>>2]|0;
 $45 = (($44) + 72|0);
 HEAP8[$45>>0] = $43;
 $46 = HEAP32[$pBt>>2]|0;
 $47 = (($46) + 8|0);
 $48 = HEAP32[$47>>2]|0;
 $49 = HEAP32[$5>>2]|0;
 $50 = (($49) + 8|0);
 HEAP32[$50>>2] = $48;
 $51 = HEAP32[$5>>2]|0;
 $52 = (($51) + 8|0);
 $53 = HEAP32[$52>>2]|0;
 $54 = ($53|0)!=(0|0);
 if ($54) {
  $55 = HEAP32[$5>>2]|0;
  $56 = HEAP32[$5>>2]|0;
  $57 = (($56) + 8|0);
  $58 = HEAP32[$57>>2]|0;
  $59 = (($58) + 12|0);
  HEAP32[$59>>2] = $55;
 }
 $60 = HEAP32[$5>>2]|0;
 $61 = HEAP32[$pBt>>2]|0;
 $62 = (($61) + 8|0);
 HEAP32[$62>>2] = $60;
 $63 = HEAP32[$5>>2]|0;
 $64 = (($63) + 73|0);
 HEAP8[$64>>0] = 0;
 HEAP32[$0>>2] = 0;
 $65 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($65|0);
}
function _allocateTempSpace($pBt) {
 $pBt = $pBt|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0;
 var $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 HEAP32[$0>>2] = $pBt;
 $1 = HEAP32[$0>>2]|0;
 $2 = (($1) + 80|0);
 $3 = HEAP32[$2>>2]|0;
 $4 = ($3|0)!=(0|0);
 if ($4) {
  STACKTOP = sp;return;
 }
 $5 = HEAP32[$0>>2]|0;
 $6 = (($5) + 32|0);
 $7 = HEAP32[$6>>2]|0;
 $8 = (_sqlite3PageMalloc($7)|0);
 $9 = HEAP32[$0>>2]|0;
 $10 = (($9) + 80|0);
 HEAP32[$10>>2] = $8;
 $11 = HEAP32[$0>>2]|0;
 $12 = (($11) + 80|0);
 $13 = HEAP32[$12>>2]|0;
 $14 = ($13|0)!=(0|0);
 if (!($14)) {
  STACKTOP = sp;return;
 }
 $15 = HEAP32[$0>>2]|0;
 $16 = (($15) + 80|0);
 $17 = HEAP32[$16>>2]|0;
 ;HEAP8[$17+0>>0]=0|0;HEAP8[$17+1>>0]=0|0;HEAP8[$17+2>>0]=0|0;HEAP8[$17+3>>0]=0|0;HEAP8[$17+4>>0]=0|0;HEAP8[$17+5>>0]=0|0;HEAP8[$17+6>>0]=0|0;HEAP8[$17+7>>0]=0|0;
 $18 = HEAP32[$0>>2]|0;
 $19 = (($18) + 80|0);
 $20 = HEAP32[$19>>2]|0;
 $21 = (($20) + 4|0);
 HEAP32[$19>>2] = $21;
 STACKTOP = sp;return;
}
function _sqlite3BtreeCursorSize() {
 var label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = sp;return 200;
}
function _sqlite3BtreeCursorZero($p) {
 $p = $p|0;
 var $0 = 0, $1 = 0, dest = 0, label = 0, sp = 0, stop = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 HEAP32[$0>>2] = $p;
 $1 = HEAP32[$0>>2]|0;
 dest=$1+0|0; stop=dest+76|0; do { HEAP32[dest>>2]=0|0; dest=dest+4|0; } while ((dest|0) < (stop|0));
 STACKTOP = sp;return;
}
function _sqlite3PagerOpenSavepoint($pPager,$nSavepoint) {
 $pPager = $pPager|0;
 $nSavepoint = $nSavepoint|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0;
 var $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0;
 var $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0;
 var $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0;
 var $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0;
 var $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $aNew = 0, $ii = 0, $nCurrent = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 24|0;
 $1 = sp + 20|0;
 $2 = sp + 16|0;
 $rc = sp + 12|0;
 $nCurrent = sp + 8|0;
 $ii = sp + 4|0;
 $aNew = sp;
 HEAP32[$1>>2] = $pPager;
 HEAP32[$2>>2] = $nSavepoint;
 HEAP32[$rc>>2] = 0;
 $3 = HEAP32[$1>>2]|0;
 $4 = (($3) + 96|0);
 $5 = HEAP32[$4>>2]|0;
 HEAP32[$nCurrent>>2] = $5;
 $6 = HEAP32[$2>>2]|0;
 $7 = HEAP32[$nCurrent>>2]|0;
 $8 = ($6|0)>($7|0);
 L1: do {
  if ($8) {
   $9 = HEAP32[$1>>2]|0;
   $10 = (($9) + 6|0);
   $11 = HEAP8[$10>>0]|0;
   $12 = $11&255;
   $13 = ($12|0)!=(0);
   if ($13) {
    $14 = HEAP32[$1>>2]|0;
    $15 = (($14) + 92|0);
    $16 = HEAP32[$15>>2]|0;
    $17 = HEAP32[$2>>2]|0;
    $18 = ($17*48)|0;
    $19 = (_sqlite3Realloc($16,$18,0)|0);
    HEAP32[$aNew>>2] = $19;
    $20 = HEAP32[$aNew>>2]|0;
    $21 = ($20|0)!=(0|0);
    if (!($21)) {
     HEAP32[$0>>2] = 7;
     $124 = HEAP32[$0>>2]|0;
     STACKTOP = sp;return ($124|0);
    }
    $22 = HEAP32[$nCurrent>>2]|0;
    $23 = HEAP32[$aNew>>2]|0;
    $24 = (($23) + (($22*48)|0)|0);
    $25 = HEAP32[$2>>2]|0;
    $26 = HEAP32[$nCurrent>>2]|0;
    $27 = (($25) - ($26))|0;
    $28 = ($27*48)|0;
    _memset(($24|0),0,($28|0))|0;
    $29 = HEAP32[$aNew>>2]|0;
    $30 = HEAP32[$1>>2]|0;
    $31 = (($30) + 92|0);
    HEAP32[$31>>2] = $29;
    $32 = HEAP32[$nCurrent>>2]|0;
    HEAP32[$ii>>2] = $32;
    while(1) {
     $33 = HEAP32[$ii>>2]|0;
     $34 = HEAP32[$2>>2]|0;
     $35 = ($33|0)<($34|0);
     if (!($35)) {
      break L1;
     }
     $36 = HEAP32[$1>>2]|0;
     $37 = (($36) + 24|0);
     $38 = HEAP32[$37>>2]|0;
     $39 = HEAP32[$ii>>2]|0;
     $40 = HEAP32[$aNew>>2]|0;
     $41 = (($40) + (($39*48)|0)|0);
     $42 = (($41) + 20|0);
     HEAP32[$42>>2] = $38;
     $43 = HEAP32[$1>>2]|0;
     $44 = (($43) + 64|0);
     $45 = HEAP32[$44>>2]|0;
     $46 = HEAP32[$45>>2]|0;
     $47 = ($46|0)!=(0|0);
     if ($47) {
      $48 = HEAP32[$1>>2]|0;
      $49 = (($48) + 72|0);
      $50 = $49;
      $51 = $50;
      $52 = HEAP32[$51>>2]|0;
      $53 = (($50) + 4)|0;
      $54 = $53;
      $55 = HEAP32[$54>>2]|0;
      $56 = ($55|0)>(0);
      $57 = ($52>>>0)>(0);
      $58 = ($55|0)==(0);
      $59 = $58 & $57;
      $60 = $56 | $59;
      if ($60) {
       $61 = HEAP32[$1>>2]|0;
       $62 = (($61) + 72|0);
       $63 = $62;
       $64 = $63;
       $65 = HEAP32[$64>>2]|0;
       $66 = (($63) + 4)|0;
       $67 = $66;
       $68 = HEAP32[$67>>2]|0;
       $69 = HEAP32[$ii>>2]|0;
       $70 = HEAP32[$aNew>>2]|0;
       $71 = (($70) + (($69*48)|0)|0);
       $72 = $71;
       $73 = $72;
       HEAP32[$73>>2] = $65;
       $74 = (($72) + 4)|0;
       $75 = $74;
       HEAP32[$75>>2] = $68;
      } else {
       label = 10;
      }
     } else {
      label = 10;
     }
     if ((label|0) == 10) {
      label = 0;
      $76 = HEAP32[$1>>2]|0;
      $77 = (($76) + 148|0);
      $78 = HEAP32[$77>>2]|0;
      $79 = HEAP32[$ii>>2]|0;
      $80 = HEAP32[$aNew>>2]|0;
      $81 = (($80) + (($79*48)|0)|0);
      $82 = $81;
      $83 = $82;
      HEAP32[$83>>2] = $78;
      $84 = (($82) + 4)|0;
      $85 = $84;
      HEAP32[$85>>2] = 0;
     }
     $86 = HEAP32[$1>>2]|0;
     $87 = (($86) + 52|0);
     $88 = HEAP32[$87>>2]|0;
     $89 = HEAP32[$ii>>2]|0;
     $90 = HEAP32[$aNew>>2]|0;
     $91 = (($90) + (($89*48)|0)|0);
     $92 = (($91) + 24|0);
     HEAP32[$92>>2] = $88;
     $93 = HEAP32[$1>>2]|0;
     $94 = (($93) + 24|0);
     $95 = HEAP32[$94>>2]|0;
     $96 = (_sqlite3BitvecCreate($95)|0);
     $97 = HEAP32[$ii>>2]|0;
     $98 = HEAP32[$aNew>>2]|0;
     $99 = (($98) + (($97*48)|0)|0);
     $100 = (($99) + 16|0);
     HEAP32[$100>>2] = $96;
     $101 = HEAP32[$ii>>2]|0;
     $102 = HEAP32[$aNew>>2]|0;
     $103 = (($102) + (($101*48)|0)|0);
     $104 = (($103) + 16|0);
     $105 = HEAP32[$104>>2]|0;
     $106 = ($105|0)!=(0|0);
     if (!($106)) {
      break;
     }
     $107 = HEAP32[$1>>2]|0;
     $108 = (_pagerUseWal($107)|0);
     $109 = ($108|0)!=(0);
     if ($109) {
      $110 = HEAP32[$1>>2]|0;
      $111 = (($110) + 208|0);
      $112 = HEAP32[$111>>2]|0;
      $113 = HEAP32[$ii>>2]|0;
      $114 = HEAP32[$aNew>>2]|0;
      $115 = (($114) + (($113*48)|0)|0);
      $116 = (($115) + 28|0);
      _sqlite3WalSavepoint($112,$116);
     }
     $117 = HEAP32[$ii>>2]|0;
     $118 = (($117) + 1)|0;
     $119 = HEAP32[$1>>2]|0;
     $120 = (($119) + 96|0);
     HEAP32[$120>>2] = $118;
     $121 = HEAP32[$ii>>2]|0;
     $122 = (($121) + 1)|0;
     HEAP32[$ii>>2] = $122;
    }
    HEAP32[$0>>2] = 7;
    $124 = HEAP32[$0>>2]|0;
    STACKTOP = sp;return ($124|0);
   }
  }
 } while(0);
 $123 = HEAP32[$rc>>2]|0;
 HEAP32[$0>>2] = $123;
 $124 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($124|0);
}
function _sqlite3WalSavepoint($pWal,$aWalData) {
 $pWal = $pWal|0;
 $aWalData = $aWalData|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $3 = 0, $4 = 0;
 var $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $1 = sp;
 HEAP32[$0>>2] = $pWal;
 HEAP32[$1>>2] = $aWalData;
 $2 = HEAP32[$0>>2]|0;
 $3 = (($2) + 52|0);
 $4 = (($3) + 16|0);
 $5 = HEAP32[$4>>2]|0;
 $6 = HEAP32[$1>>2]|0;
 HEAP32[$6>>2] = $5;
 $7 = HEAP32[$0>>2]|0;
 $8 = (($7) + 52|0);
 $9 = (($8) + 24|0);
 $10 = HEAP32[$9>>2]|0;
 $11 = HEAP32[$1>>2]|0;
 $12 = (($11) + 4|0);
 HEAP32[$12>>2] = $10;
 $13 = HEAP32[$0>>2]|0;
 $14 = (($13) + 52|0);
 $15 = (($14) + 24|0);
 $16 = (($15) + 4|0);
 $17 = HEAP32[$16>>2]|0;
 $18 = HEAP32[$1>>2]|0;
 $19 = (($18) + 8|0);
 HEAP32[$19>>2] = $17;
 $20 = HEAP32[$0>>2]|0;
 $21 = (($20) + 104|0);
 $22 = HEAP32[$21>>2]|0;
 $23 = HEAP32[$1>>2]|0;
 $24 = (($23) + 12|0);
 HEAP32[$24>>2] = $22;
 STACKTOP = sp;return;
}
function _newDatabase($pBt) {
 $pBt = $pBt|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0;
 var $81 = 0, $9 = 0, $data = 0, $pP1 = 0, $rc = 0, dest = 0, label = 0, sp = 0, src = 0, stop = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 16|0;
 $1 = sp + 12|0;
 $pP1 = sp + 8|0;
 $data = sp + 4|0;
 $rc = sp;
 HEAP32[$1>>2] = $pBt;
 $2 = HEAP32[$1>>2]|0;
 $3 = (($2) + 44|0);
 $4 = HEAP32[$3>>2]|0;
 $5 = ($4>>>0)>(0);
 if ($5) {
  HEAP32[$0>>2] = 0;
  $81 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($81|0);
 }
 $6 = HEAP32[$1>>2]|0;
 $7 = (($6) + 12|0);
 $8 = HEAP32[$7>>2]|0;
 HEAP32[$pP1>>2] = $8;
 $9 = HEAP32[$pP1>>2]|0;
 $10 = (($9) + 56|0);
 $11 = HEAP32[$10>>2]|0;
 HEAP32[$data>>2] = $11;
 $12 = HEAP32[$pP1>>2]|0;
 $13 = (($12) + 68|0);
 $14 = HEAP32[$13>>2]|0;
 $15 = (_sqlite3PagerWrite($14)|0);
 HEAP32[$rc>>2] = $15;
 $16 = HEAP32[$rc>>2]|0;
 $17 = ($16|0)!=(0);
 if ($17) {
  $18 = HEAP32[$rc>>2]|0;
  HEAP32[$0>>2] = $18;
  $81 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($81|0);
 } else {
  $19 = HEAP32[$data>>2]|0;
  dest=$19+0|0; src=34408+0|0; stop=dest+16|0; do { HEAP8[dest>>0]=HEAP8[src>>0]|0; dest=dest+1|0; src=src+1|0; } while ((dest|0) < (stop|0));
  $20 = HEAP32[$1>>2]|0;
  $21 = (($20) + 32|0);
  $22 = HEAP32[$21>>2]|0;
  $23 = $22 >>> 8;
  $24 = $23 & 255;
  $25 = $24&255;
  $26 = HEAP32[$data>>2]|0;
  $27 = (($26) + 16|0);
  HEAP8[$27>>0] = $25;
  $28 = HEAP32[$1>>2]|0;
  $29 = (($28) + 32|0);
  $30 = HEAP32[$29>>2]|0;
  $31 = $30 >>> 16;
  $32 = $31 & 255;
  $33 = $32&255;
  $34 = HEAP32[$data>>2]|0;
  $35 = (($34) + 17|0);
  HEAP8[$35>>0] = $33;
  $36 = HEAP32[$data>>2]|0;
  $37 = (($36) + 18|0);
  HEAP8[$37>>0] = 1;
  $38 = HEAP32[$data>>2]|0;
  $39 = (($38) + 19|0);
  HEAP8[$39>>0] = 1;
  $40 = HEAP32[$1>>2]|0;
  $41 = (($40) + 32|0);
  $42 = HEAP32[$41>>2]|0;
  $43 = HEAP32[$1>>2]|0;
  $44 = (($43) + 36|0);
  $45 = HEAP32[$44>>2]|0;
  $46 = (($42) - ($45))|0;
  $47 = $46&255;
  $48 = HEAP32[$data>>2]|0;
  $49 = (($48) + 20|0);
  HEAP8[$49>>0] = $47;
  $50 = HEAP32[$data>>2]|0;
  $51 = (($50) + 21|0);
  HEAP8[$51>>0] = 64;
  $52 = HEAP32[$data>>2]|0;
  $53 = (($52) + 22|0);
  HEAP8[$53>>0] = 32;
  $54 = HEAP32[$data>>2]|0;
  $55 = (($54) + 23|0);
  HEAP8[$55>>0] = 32;
  $56 = HEAP32[$data>>2]|0;
  $57 = (($56) + 24|0);
  dest=$57+0|0; stop=dest+76|0; do { HEAP8[dest>>0]=0|0; dest=dest+1|0; } while ((dest|0) < (stop|0));
  $58 = HEAP32[$pP1>>2]|0;
  _zeroPage($58,13);
  $59 = HEAP32[$1>>2]|0;
  $60 = (($59) + 22|0);
  $61 = HEAP16[$60>>1]|0;
  $62 = $61&65535;
  $63 = $62 | 2;
  $64 = $63&65535;
  HEAP16[$60>>1] = $64;
  $65 = HEAP32[$data>>2]|0;
  $66 = (($65) + 52|0);
  $67 = HEAP32[$1>>2]|0;
  $68 = (($67) + 17|0);
  $69 = HEAP8[$68>>0]|0;
  $70 = $69&255;
  _sqlite3Put4byte($66,$70);
  $71 = HEAP32[$data>>2]|0;
  $72 = (($71) + 64|0);
  $73 = HEAP32[$1>>2]|0;
  $74 = (($73) + 18|0);
  $75 = HEAP8[$74>>0]|0;
  $76 = $75&255;
  _sqlite3Put4byte($72,$76);
  $77 = HEAP32[$1>>2]|0;
  $78 = (($77) + 44|0);
  HEAP32[$78>>2] = 1;
  $79 = HEAP32[$data>>2]|0;
  $80 = (($79) + 31|0);
  HEAP8[$80>>0] = 1;
  HEAP32[$0>>2] = 0;
  $81 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($81|0);
 }
 return 0|0;
}
function _putVarint64($p,$0,$1) {
 $p = $p|0;
 $0 = $0|0;
 $1 = $1|0;
 var $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0, $116 = 0, $117 = 0;
 var $118 = 0, $119 = 0, $12 = 0, $120 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0;
 var $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0;
 var $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0;
 var $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0;
 var $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $buf = 0;
 var $i = 0, $j = 0, $n = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $2 = sp + 24|0;
 $3 = sp + 20|0;
 $4 = sp;
 $i = sp + 16|0;
 $j = sp + 12|0;
 $n = sp + 8|0;
 $buf = sp + 28|0;
 HEAP32[$3>>2] = $p;
 $5 = $4;
 $6 = $5;
 HEAP32[$6>>2] = $0;
 $7 = (($5) + 4)|0;
 $8 = $7;
 HEAP32[$8>>2] = $1;
 $9 = $4;
 $10 = $9;
 $11 = HEAP32[$10>>2]|0;
 $12 = (($9) + 4)|0;
 $13 = $12;
 $14 = HEAP32[$13>>2]|0;
 $15 = $14 & -16777216;
 $16 = (0)!=(0);
 $17 = ($15|0)!=(0);
 $18 = $16 | $17;
 if ($18) {
  $19 = $4;
  $20 = $19;
  $21 = HEAP32[$20>>2]|0;
  $22 = (($19) + 4)|0;
  $23 = $22;
  $24 = HEAP32[$23>>2]|0;
  $25 = $21&255;
  $26 = HEAP32[$3>>2]|0;
  $27 = (($26) + 8|0);
  HEAP8[$27>>0] = $25;
  $28 = $4;
  $29 = $28;
  $30 = HEAP32[$29>>2]|0;
  $31 = (($28) + 4)|0;
  $32 = $31;
  $33 = HEAP32[$32>>2]|0;
  $34 = (_bitshift64Lshr(($30|0),($33|0),8)|0);
  $35 = tempRet0;
  $36 = $4;
  $37 = $36;
  HEAP32[$37>>2] = $34;
  $38 = (($36) + 4)|0;
  $39 = $38;
  HEAP32[$39>>2] = $35;
  HEAP32[$i>>2] = 7;
  while(1) {
   $40 = HEAP32[$i>>2]|0;
   $41 = ($40|0)>=(0);
   if (!($41)) {
    break;
   }
   $42 = $4;
   $43 = $42;
   $44 = HEAP32[$43>>2]|0;
   $45 = (($42) + 4)|0;
   $46 = $45;
   $47 = HEAP32[$46>>2]|0;
   $48 = $44 & 127;
   $49 = $48 | 128;
   $50 = $49&255;
   $51 = HEAP32[$i>>2]|0;
   $52 = HEAP32[$3>>2]|0;
   $53 = (($52) + ($51)|0);
   HEAP8[$53>>0] = $50;
   $54 = $4;
   $55 = $54;
   $56 = HEAP32[$55>>2]|0;
   $57 = (($54) + 4)|0;
   $58 = $57;
   $59 = HEAP32[$58>>2]|0;
   $60 = (_bitshift64Lshr(($56|0),($59|0),7)|0);
   $61 = tempRet0;
   $62 = $4;
   $63 = $62;
   HEAP32[$63>>2] = $60;
   $64 = (($62) + 4)|0;
   $65 = $64;
   HEAP32[$65>>2] = $61;
   $66 = HEAP32[$i>>2]|0;
   $67 = (($66) + -1)|0;
   HEAP32[$i>>2] = $67;
  }
  HEAP32[$2>>2] = 9;
  $120 = HEAP32[$2>>2]|0;
  STACKTOP = sp;return ($120|0);
 }
 HEAP32[$n>>2] = 0;
 while(1) {
  $68 = $4;
  $69 = $68;
  $70 = HEAP32[$69>>2]|0;
  $71 = (($68) + 4)|0;
  $72 = $71;
  $73 = HEAP32[$72>>2]|0;
  $74 = $70 & 127;
  $75 = $74 | 128;
  $76 = $75&255;
  $77 = HEAP32[$n>>2]|0;
  $78 = (($77) + 1)|0;
  HEAP32[$n>>2] = $78;
  $79 = (($buf) + ($77)|0);
  HEAP8[$79>>0] = $76;
  $80 = $4;
  $81 = $80;
  $82 = HEAP32[$81>>2]|0;
  $83 = (($80) + 4)|0;
  $84 = $83;
  $85 = HEAP32[$84>>2]|0;
  $86 = (_bitshift64Lshr(($82|0),($85|0),7)|0);
  $87 = tempRet0;
  $88 = $4;
  $89 = $88;
  HEAP32[$89>>2] = $86;
  $90 = (($88) + 4)|0;
  $91 = $90;
  HEAP32[$91>>2] = $87;
  $92 = $4;
  $93 = $92;
  $94 = HEAP32[$93>>2]|0;
  $95 = (($92) + 4)|0;
  $96 = $95;
  $97 = HEAP32[$96>>2]|0;
  $98 = ($94|0)!=(0);
  $99 = ($97|0)!=(0);
  $100 = $98 | $99;
  if (!($100)) {
   break;
  }
 }
 $101 = HEAP8[$buf>>0]|0;
 $102 = $101&255;
 $103 = $102 & 127;
 $104 = $103&255;
 HEAP8[$buf>>0] = $104;
 HEAP32[$i>>2] = 0;
 $105 = HEAP32[$n>>2]|0;
 $106 = (($105) - 1)|0;
 HEAP32[$j>>2] = $106;
 while(1) {
  $107 = HEAP32[$j>>2]|0;
  $108 = ($107|0)>=(0);
  if (!($108)) {
   break;
  }
  $109 = HEAP32[$j>>2]|0;
  $110 = (($buf) + ($109)|0);
  $111 = HEAP8[$110>>0]|0;
  $112 = HEAP32[$i>>2]|0;
  $113 = HEAP32[$3>>2]|0;
  $114 = (($113) + ($112)|0);
  HEAP8[$114>>0] = $111;
  $115 = HEAP32[$j>>2]|0;
  $116 = (($115) + -1)|0;
  HEAP32[$j>>2] = $116;
  $117 = HEAP32[$i>>2]|0;
  $118 = (($117) + 1)|0;
  HEAP32[$i>>2] = $118;
 }
 $119 = HEAP32[$n>>2]|0;
 HEAP32[$2>>2] = $119;
 $120 = HEAP32[$2>>2]|0;
 STACKTOP = sp;return ($120|0);
}
function _fetchPayload($pCur,$pAmt) {
 $pCur = $pCur|0;
 $pAmt = $pAmt|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $1 = sp;
 HEAP32[$0>>2] = $pCur;
 HEAP32[$1>>2] = $pAmt;
 $2 = HEAP32[$0>>2]|0;
 $3 = (($2) + 24|0);
 $4 = (($3) + 16|0);
 $5 = HEAP16[$4>>1]|0;
 $6 = $5&65535;
 $7 = HEAP32[$1>>2]|0;
 HEAP32[$7>>2] = $6;
 $8 = HEAP32[$0>>2]|0;
 $9 = (($8) + 24|0);
 $10 = (($9) + 8|0);
 $11 = HEAP32[$10>>2]|0;
 STACKTOP = sp;return ($11|0);
}
function _handleDeferredMoveto($p) {
 $p = $p|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $3 = 0, $4 = 0, $5 = 0;
 var $6 = 0, $7 = 0, $8 = 0, $9 = 0, $rc = 0, $res = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $1 = sp + 8|0;
 $res = sp + 4|0;
 $rc = sp;
 HEAP32[$1>>2] = $p;
 $2 = HEAP32[$1>>2]|0;
 $3 = HEAP32[$2>>2]|0;
 $4 = HEAP32[$1>>2]|0;
 $5 = (($4) + 48|0);
 $6 = $5;
 $7 = $6;
 $8 = HEAP32[$7>>2]|0;
 $9 = (($6) + 4)|0;
 $10 = $9;
 $11 = HEAP32[$10>>2]|0;
 $12 = (_sqlite3BtreeMovetoUnpacked($3,0,$8,$11,0,$res)|0);
 HEAP32[$rc>>2] = $12;
 $13 = HEAP32[$rc>>2]|0;
 $14 = ($13|0)!=(0);
 if ($14) {
  $15 = HEAP32[$rc>>2]|0;
  HEAP32[$0>>2] = $15;
  $23 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($23|0);
 }
 $16 = HEAP32[$res>>2]|0;
 $17 = ($16|0)!=(0);
 if ($17) {
  $18 = (_sqlite3CorruptError(65954)|0);
  HEAP32[$0>>2] = $18;
  $23 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($23|0);
 } else {
  $19 = HEAP32[$1>>2]|0;
  $20 = (($19) + 26|0);
  HEAP8[$20>>0] = 0;
  $21 = HEAP32[$1>>2]|0;
  $22 = (($21) + 60|0);
  HEAP32[$22>>2] = 0;
  HEAP32[$0>>2] = 0;
  $23 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($23|0);
 }
 return 0|0;
}
function _computeNumericType($pMem) {
 $pMem = $pMem|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $1 = sp;
 HEAP32[$1>>2] = $pMem;
 $2 = HEAP32[$1>>2]|0;
 $3 = (($2) + 16|0);
 $4 = HEAP32[$3>>2]|0;
 $5 = HEAP32[$1>>2]|0;
 $6 = HEAP32[$1>>2]|0;
 $7 = (($6) + 12|0);
 $8 = HEAP32[$7>>2]|0;
 $9 = HEAP32[$1>>2]|0;
 $10 = (($9) + 10|0);
 $11 = HEAP8[$10>>0]|0;
 $12 = (_sqlite3AtoF($4,$5,$8,$11)|0);
 $13 = ($12|0)==(0);
 if ($13) {
  HEAP16[$0>>1] = 0;
  $26 = HEAP16[$0>>1]|0;
  STACKTOP = sp;return ($26|0);
 }
 $14 = HEAP32[$1>>2]|0;
 $15 = (($14) + 16|0);
 $16 = HEAP32[$15>>2]|0;
 $17 = HEAP32[$1>>2]|0;
 $18 = HEAP32[$1>>2]|0;
 $19 = (($18) + 12|0);
 $20 = HEAP32[$19>>2]|0;
 $21 = HEAP32[$1>>2]|0;
 $22 = (($21) + 10|0);
 $23 = HEAP8[$22>>0]|0;
 $24 = (_sqlite3Atoi64($16,$17,$20,$23)|0);
 $25 = ($24|0)==(0);
 if ($25) {
  HEAP16[$0>>1] = 4;
  $26 = HEAP16[$0>>1]|0;
  STACKTOP = sp;return ($26|0);
 } else {
  HEAP16[$0>>1] = 8;
  $26 = HEAP16[$0>>1]|0;
  STACKTOP = sp;return ($26|0);
 }
 return 0|0;
}
function _vdbeMemAddTerminator($pMem) {
 $pMem = $pMem|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $1 = sp;
 HEAP32[$1>>2] = $pMem;
 $2 = HEAP32[$1>>2]|0;
 $3 = HEAP32[$1>>2]|0;
 $4 = (($3) + 12|0);
 $5 = HEAP32[$4>>2]|0;
 $6 = (($5) + 2)|0;
 $7 = (_sqlite3VdbeMemGrow($2,$6,1)|0);
 $8 = ($7|0)!=(0);
 if ($8) {
  HEAP32[$0>>2] = 7;
  $30 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($30|0);
 } else {
  $9 = HEAP32[$1>>2]|0;
  $10 = (($9) + 12|0);
  $11 = HEAP32[$10>>2]|0;
  $12 = HEAP32[$1>>2]|0;
  $13 = (($12) + 16|0);
  $14 = HEAP32[$13>>2]|0;
  $15 = (($14) + ($11)|0);
  HEAP8[$15>>0] = 0;
  $16 = HEAP32[$1>>2]|0;
  $17 = (($16) + 12|0);
  $18 = HEAP32[$17>>2]|0;
  $19 = (($18) + 1)|0;
  $20 = HEAP32[$1>>2]|0;
  $21 = (($20) + 16|0);
  $22 = HEAP32[$21>>2]|0;
  $23 = (($22) + ($19)|0);
  HEAP8[$23>>0] = 0;
  $24 = HEAP32[$1>>2]|0;
  $25 = (($24) + 8|0);
  $26 = HEAP16[$25>>1]|0;
  $27 = $26&65535;
  $28 = $27 | 512;
  $29 = $28&65535;
  HEAP16[$25>>1] = $29;
  HEAP32[$0>>2] = 0;
  $30 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($30|0);
 }
 return 0|0;
}
function _closeAllCursors($p) {
 $p = $p|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $8 = 0, $9 = 0, $i = 0, $pC = 0, $pDel = 0, $pFrame = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 16|0;
 $pFrame = sp + 12|0;
 $i = sp + 8|0;
 $pC = sp + 4|0;
 $pDel = sp;
 HEAP32[$0>>2] = $p;
 $1 = HEAP32[$0>>2]|0;
 $2 = (($1) + 176|0);
 $3 = HEAP32[$2>>2]|0;
 $4 = ($3|0)!=(0|0);
 if ($4) {
  $5 = HEAP32[$0>>2]|0;
  $6 = (($5) + 176|0);
  $7 = HEAP32[$6>>2]|0;
  HEAP32[$pFrame>>2] = $7;
  while(1) {
   $8 = HEAP32[$pFrame>>2]|0;
   $9 = (($8) + 4|0);
   $10 = HEAP32[$9>>2]|0;
   $11 = ($10|0)!=(0|0);
   $12 = HEAP32[$pFrame>>2]|0;
   if (!($11)) {
    break;
   }
   $13 = (($12) + 4|0);
   $14 = HEAP32[$13>>2]|0;
   HEAP32[$pFrame>>2] = $14;
  }
  (_sqlite3VdbeFrameRestore($12)|0);
  $15 = HEAP32[$0>>2]|0;
  $16 = (($15) + 176|0);
  HEAP32[$16>>2] = 0;
  $17 = HEAP32[$0>>2]|0;
  $18 = (($17) + 184|0);
  HEAP32[$18>>2] = 0;
 }
 $19 = HEAP32[$0>>2]|0;
 $20 = (($19) + 56|0);
 $21 = HEAP32[$20>>2]|0;
 $22 = ($21|0)!=(0|0);
 L8: do {
  if ($22) {
   HEAP32[$i>>2] = 0;
   while(1) {
    $23 = HEAP32[$i>>2]|0;
    $24 = HEAP32[$0>>2]|0;
    $25 = (($24) + 36|0);
    $26 = HEAP32[$25>>2]|0;
    $27 = ($23|0)<($26|0);
    if (!($27)) {
     break L8;
    }
    $28 = HEAP32[$i>>2]|0;
    $29 = HEAP32[$0>>2]|0;
    $30 = (($29) + 56|0);
    $31 = HEAP32[$30>>2]|0;
    $32 = (($31) + ($28<<2)|0);
    $33 = HEAP32[$32>>2]|0;
    HEAP32[$pC>>2] = $33;
    $34 = HEAP32[$pC>>2]|0;
    $35 = ($34|0)!=(0|0);
    if ($35) {
     $36 = HEAP32[$0>>2]|0;
     $37 = HEAP32[$pC>>2]|0;
     _sqlite3VdbeFreeCursor($36,$37);
     $38 = HEAP32[$i>>2]|0;
     $39 = HEAP32[$0>>2]|0;
     $40 = (($39) + 56|0);
     $41 = HEAP32[$40>>2]|0;
     $42 = (($41) + ($38<<2)|0);
     HEAP32[$42>>2] = 0;
    }
    $43 = HEAP32[$i>>2]|0;
    $44 = (($43) + 1)|0;
    HEAP32[$i>>2] = $44;
   }
  }
 } while(0);
 $45 = HEAP32[$0>>2]|0;
 $46 = (($45) + 8|0);
 $47 = HEAP32[$46>>2]|0;
 $48 = ($47|0)!=(0|0);
 if ($48) {
  $49 = HEAP32[$0>>2]|0;
  $50 = (($49) + 8|0);
  $51 = HEAP32[$50>>2]|0;
  $52 = (($51) + 40|0);
  $53 = HEAP32[$0>>2]|0;
  $54 = (($53) + 28|0);
  $55 = HEAP32[$54>>2]|0;
  _releaseMemArray($52,$55);
 }
 while(1) {
  $56 = HEAP32[$0>>2]|0;
  $57 = (($56) + 180|0);
  $58 = HEAP32[$57>>2]|0;
  $59 = ($58|0)!=(0|0);
  $60 = HEAP32[$0>>2]|0;
  if (!($59)) {
   break;
  }
  $61 = (($60) + 180|0);
  $62 = HEAP32[$61>>2]|0;
  HEAP32[$pDel>>2] = $62;
  $63 = HEAP32[$pDel>>2]|0;
  $64 = (($63) + 4|0);
  $65 = HEAP32[$64>>2]|0;
  $66 = HEAP32[$0>>2]|0;
  $67 = (($66) + 180|0);
  HEAP32[$67>>2] = $65;
  $68 = HEAP32[$pDel>>2]|0;
  _sqlite3VdbeFrameDelete($68);
 }
 $69 = (($60) + 204|0);
 $70 = HEAP32[$69>>2]|0;
 $71 = ($70|0)!=(0|0);
 if (!($71)) {
  STACKTOP = sp;return;
 }
 $72 = HEAP32[$0>>2]|0;
 _sqlite3VdbeDeleteAuxData($72,-1,0);
 STACKTOP = sp;return;
}
function _vdbeCommit($db,$p) {
 $db = $db|0;
 $p = $p|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0;
 var $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0, $188 = 0;
 var $189 = 0, $19 = 0, $190 = 0, $191 = 0, $192 = 0, $193 = 0, $194 = 0, $195 = 0, $196 = 0, $197 = 0, $198 = 0, $199 = 0, $2 = 0, $20 = 0, $200 = 0, $201 = 0, $202 = 0, $203 = 0, $204 = 0, $205 = 0;
 var $206 = 0, $207 = 0, $208 = 0, $209 = 0, $21 = 0, $210 = 0, $211 = 0, $212 = 0, $213 = 0, $214 = 0, $215 = 0, $216 = 0, $217 = 0, $218 = 0, $219 = 0, $22 = 0, $220 = 0, $221 = 0, $222 = 0, $223 = 0;
 var $224 = 0, $225 = 0, $226 = 0, $227 = 0, $228 = 0, $229 = 0, $23 = 0, $230 = 0, $231 = 0, $232 = 0, $233 = 0, $234 = 0, $235 = 0, $236 = 0, $237 = 0, $238 = 0, $239 = 0, $24 = 0, $240 = 0, $241 = 0;
 var $242 = 0, $243 = 0, $244 = 0, $245 = 0, $246 = 0, $247 = 0, $248 = 0, $249 = 0, $25 = 0, $250 = 0, $251 = 0, $252 = 0, $253 = 0, $254 = 0, $255 = 0, $256 = 0, $257 = 0, $258 = 0, $259 = 0, $26 = 0;
 var $260 = 0, $261 = 0, $262 = 0, $263 = 0, $264 = 0, $265 = 0, $266 = 0, $267 = 0, $268 = 0, $269 = 0, $27 = 0, $270 = 0, $271 = 0, $272 = 0, $273 = 0, $274 = 0, $275 = 0, $276 = 0, $277 = 0, $278 = 0;
 var $279 = 0, $28 = 0, $280 = 0, $281 = 0, $282 = 0, $283 = 0, $284 = 0, $285 = 0, $286 = 0, $287 = 0, $288 = 0, $289 = 0, $29 = 0, $290 = 0, $291 = 0, $292 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0;
 var $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0;
 var $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0;
 var $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0;
 var $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $i = 0, $iRandom = 0, $nMainFile = 0, $nTrans = 0, $needSync = 0, $needXcommit = 0, $offset = 0;
 var $or$cond = 0, $or$cond3 = 0, $pBt = 0, $pBt1 = 0, $pBt2 = 0, $pBt3 = 0, $pBt4 = 0, $pBt5 = 0, $pMaster = 0, $pVfs = 0, $rc = 0, $res = 0, $retryCount = 0, $vararg_buffer10 = 0, $vararg_ptr13 = 0, $zFile = 0, $zMainFile = 0, $zMaster = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 112|0;
 $vararg_buffer10 = sp + 8|0;
 $0 = sp + 104|0;
 $1 = sp + 100|0;
 $2 = sp + 96|0;
 $i = sp + 92|0;
 $nTrans = sp + 88|0;
 $rc = sp + 84|0;
 $needXcommit = sp + 80|0;
 $pBt = sp + 76|0;
 $pBt1 = sp + 72|0;
 $pBt2 = sp + 68|0;
 $pVfs = sp + 64|0;
 $needSync = sp + 60|0;
 $zMaster = sp + 56|0;
 $zMainFile = sp + 52|0;
 $pMaster = sp + 48|0;
 $offset = sp;
 $res = sp + 44|0;
 $retryCount = sp + 40|0;
 $nMainFile = sp + 36|0;
 $iRandom = sp + 32|0;
 $pBt3 = sp + 28|0;
 $zFile = sp + 24|0;
 $pBt4 = sp + 20|0;
 $pBt5 = sp + 16|0;
 HEAP32[$1>>2] = $db;
 HEAP32[$2>>2] = $p;
 HEAP32[$nTrans>>2] = 0;
 HEAP32[$rc>>2] = 0;
 HEAP32[$needXcommit>>2] = 0;
 $3 = HEAP32[$1>>2]|0;
 $4 = HEAP32[$2>>2]|0;
 $5 = (_sqlite3VtabSync($3,$4)|0);
 HEAP32[$rc>>2] = $5;
 HEAP32[$i>>2] = 0;
 while(1) {
  $6 = HEAP32[$rc>>2]|0;
  $7 = ($6|0)==(0);
  if (!($7)) {
   break;
  }
  $8 = HEAP32[$i>>2]|0;
  $9 = HEAP32[$1>>2]|0;
  $10 = (($9) + 20|0);
  $11 = HEAP32[$10>>2]|0;
  $12 = ($8|0)<($11|0);
  if (!($12)) {
   break;
  }
  $13 = HEAP32[$i>>2]|0;
  $14 = HEAP32[$1>>2]|0;
  $15 = (($14) + 16|0);
  $16 = HEAP32[$15>>2]|0;
  $17 = (($16) + ($13<<4)|0);
  $18 = (($17) + 4|0);
  $19 = HEAP32[$18>>2]|0;
  HEAP32[$pBt>>2] = $19;
  $20 = HEAP32[$pBt>>2]|0;
  $21 = (_sqlite3BtreeIsInTrans($20)|0);
  $22 = ($21|0)!=(0);
  if ($22) {
   HEAP32[$needXcommit>>2] = 1;
   $23 = HEAP32[$i>>2]|0;
   $24 = ($23|0)!=(1);
   if ($24) {
    $25 = HEAP32[$nTrans>>2]|0;
    $26 = (($25) + 1)|0;
    HEAP32[$nTrans>>2] = $26;
   }
   $27 = HEAP32[$pBt>>2]|0;
   _sqlite3BtreeEnter($27);
   $28 = HEAP32[$pBt>>2]|0;
   $29 = (_sqlite3BtreePager($28)|0);
   $30 = (_sqlite3PagerExclusiveLock($29)|0);
   HEAP32[$rc>>2] = $30;
  }
  $31 = HEAP32[$i>>2]|0;
  $32 = (($31) + 1)|0;
  HEAP32[$i>>2] = $32;
 }
 $33 = HEAP32[$rc>>2]|0;
 $34 = ($33|0)!=(0);
 if ($34) {
  $35 = HEAP32[$rc>>2]|0;
  HEAP32[$0>>2] = $35;
  $292 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($292|0);
 }
 $36 = HEAP32[$needXcommit>>2]|0;
 $37 = ($36|0)!=(0);
 if ($37) {
  $38 = HEAP32[$1>>2]|0;
  $39 = (($38) + 192|0);
  $40 = HEAP32[$39>>2]|0;
  $41 = ($40|0)!=(0|0);
  if ($41) {
   $42 = HEAP32[$1>>2]|0;
   $43 = (($42) + 192|0);
   $44 = HEAP32[$43>>2]|0;
   $45 = HEAP32[$1>>2]|0;
   $46 = (($45) + 188|0);
   $47 = HEAP32[$46>>2]|0;
   $48 = (FUNCTION_TABLE_ii[$44 & 31]($47)|0);
   HEAP32[$rc>>2] = $48;
   $49 = HEAP32[$rc>>2]|0;
   $50 = ($49|0)!=(0);
   if ($50) {
    HEAP32[$0>>2] = 531;
    $292 = HEAP32[$0>>2]|0;
    STACKTOP = sp;return ($292|0);
   }
  }
 }
 $51 = HEAP32[$1>>2]|0;
 $52 = (($51) + 16|0);
 $53 = HEAP32[$52>>2]|0;
 $54 = (($53) + 4|0);
 $55 = HEAP32[$54>>2]|0;
 $56 = (_sqlite3BtreeGetFilename($55)|0);
 $57 = (_sqlite3Strlen30($56)|0);
 $58 = (0)==($57|0);
 $59 = HEAP32[$nTrans>>2]|0;
 $60 = ($59|0)<=(1);
 $or$cond = $58 | $60;
 if ($or$cond) {
  HEAP32[$i>>2] = 0;
  while(1) {
   $61 = HEAP32[$rc>>2]|0;
   $62 = ($61|0)==(0);
   if (!($62)) {
    break;
   }
   $63 = HEAP32[$i>>2]|0;
   $64 = HEAP32[$1>>2]|0;
   $65 = (($64) + 20|0);
   $66 = HEAP32[$65>>2]|0;
   $67 = ($63|0)<($66|0);
   if (!($67)) {
    break;
   }
   $68 = HEAP32[$i>>2]|0;
   $69 = HEAP32[$1>>2]|0;
   $70 = (($69) + 16|0);
   $71 = HEAP32[$70>>2]|0;
   $72 = (($71) + ($68<<4)|0);
   $73 = (($72) + 4|0);
   $74 = HEAP32[$73>>2]|0;
   HEAP32[$pBt1>>2] = $74;
   $75 = HEAP32[$pBt1>>2]|0;
   $76 = ($75|0)!=(0|0);
   if ($76) {
    $77 = HEAP32[$pBt1>>2]|0;
    $78 = (_sqlite3BtreeCommitPhaseOne($77,0)|0);
    HEAP32[$rc>>2] = $78;
   }
   $79 = HEAP32[$i>>2]|0;
   $80 = (($79) + 1)|0;
   HEAP32[$i>>2] = $80;
  }
  HEAP32[$i>>2] = 0;
  while(1) {
   $81 = HEAP32[$rc>>2]|0;
   $82 = ($81|0)==(0);
   if (!($82)) {
    break;
   }
   $83 = HEAP32[$i>>2]|0;
   $84 = HEAP32[$1>>2]|0;
   $85 = (($84) + 20|0);
   $86 = HEAP32[$85>>2]|0;
   $87 = ($83|0)<($86|0);
   if (!($87)) {
    break;
   }
   $88 = HEAP32[$i>>2]|0;
   $89 = HEAP32[$1>>2]|0;
   $90 = (($89) + 16|0);
   $91 = HEAP32[$90>>2]|0;
   $92 = (($91) + ($88<<4)|0);
   $93 = (($92) + 4|0);
   $94 = HEAP32[$93>>2]|0;
   HEAP32[$pBt2>>2] = $94;
   $95 = HEAP32[$pBt2>>2]|0;
   $96 = ($95|0)!=(0|0);
   if ($96) {
    $97 = HEAP32[$pBt2>>2]|0;
    $98 = (_sqlite3BtreeCommitPhaseTwo($97,0)|0);
    HEAP32[$rc>>2] = $98;
   }
   $99 = HEAP32[$i>>2]|0;
   $100 = (($99) + 1)|0;
   HEAP32[$i>>2] = $100;
  }
  $101 = HEAP32[$rc>>2]|0;
  $102 = ($101|0)==(0);
  if ($102) {
   $103 = HEAP32[$1>>2]|0;
   (_sqlite3VtabCommit($103)|0);
  }
 } else {
  $104 = HEAP32[$1>>2]|0;
  $105 = HEAP32[$104>>2]|0;
  HEAP32[$pVfs>>2] = $105;
  HEAP32[$needSync>>2] = 0;
  HEAP32[$zMaster>>2] = 0;
  $106 = HEAP32[$1>>2]|0;
  $107 = (($106) + 16|0);
  $108 = HEAP32[$107>>2]|0;
  $109 = (($108) + 4|0);
  $110 = HEAP32[$109>>2]|0;
  $111 = (_sqlite3BtreeGetFilename($110)|0);
  HEAP32[$zMainFile>>2] = $111;
  HEAP32[$pMaster>>2] = 0;
  $112 = $offset;
  $113 = $112;
  HEAP32[$113>>2] = 0;
  $114 = (($112) + 4)|0;
  $115 = $114;
  HEAP32[$115>>2] = 0;
  HEAP32[$retryCount>>2] = 0;
  $116 = HEAP32[$zMainFile>>2]|0;
  $117 = (_sqlite3Strlen30($116)|0);
  HEAP32[$nMainFile>>2] = $117;
  $118 = HEAP32[$1>>2]|0;
  $119 = HEAP32[$zMainFile>>2]|0;
  HEAP32[$vararg_buffer10>>2] = $119;
  $120 = (_sqlite3MPrintf($118,34424,$vararg_buffer10)|0);
  HEAP32[$zMaster>>2] = $120;
  $121 = HEAP32[$zMaster>>2]|0;
  $122 = ($121|0)==(0|0);
  if ($122) {
   HEAP32[$0>>2] = 7;
   $292 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($292|0);
  }
  while(1) {
   $123 = HEAP32[$retryCount>>2]|0;
   $124 = ($123|0)!=(0);
   if ($124) {
    $125 = HEAP32[$retryCount>>2]|0;
    $126 = ($125|0)>(100);
    if ($126) {
     label = 34;
     break;
    }
    $130 = HEAP32[$retryCount>>2]|0;
    $131 = ($130|0)==(1);
    if ($131) {
     $132 = HEAP32[$zMaster>>2]|0;
     HEAP32[$vararg_buffer10>>2] = $132;
     _sqlite3_log(13,34456,$vararg_buffer10);
    }
   }
   $133 = HEAP32[$retryCount>>2]|0;
   $134 = (($133) + 1)|0;
   HEAP32[$retryCount>>2] = $134;
   _sqlite3_randomness(4,$iRandom);
   $135 = HEAP32[$nMainFile>>2]|0;
   $136 = HEAP32[$zMaster>>2]|0;
   $137 = (($136) + ($135)|0);
   $138 = HEAP32[$iRandom>>2]|0;
   $139 = $138 >>> 8;
   $140 = $139 & 16777215;
   $141 = HEAP32[$iRandom>>2]|0;
   $142 = $141 & 255;
   HEAP32[$vararg_buffer10>>2] = $140;
   $vararg_ptr13 = (($vararg_buffer10) + 4|0);
   HEAP32[$vararg_ptr13>>2] = $142;
   (_sqlite3_snprintf(13,$137,34472,$vararg_buffer10)|0);
   $143 = HEAP32[$pVfs>>2]|0;
   $144 = HEAP32[$zMaster>>2]|0;
   $145 = (_sqlite3OsAccess($143,$144,0,$res)|0);
   HEAP32[$rc>>2] = $145;
   $146 = HEAP32[$rc>>2]|0;
   $147 = ($146|0)==(0);
   $148 = HEAP32[$res>>2]|0;
   $149 = ($148|0)!=(0);
   $or$cond3 = $147 & $149;
   if (!($or$cond3)) {
    break;
   }
  }
  if ((label|0) == 34) {
   $127 = HEAP32[$zMaster>>2]|0;
   HEAP32[$vararg_buffer10>>2] = $127;
   _sqlite3_log(13,34440,$vararg_buffer10);
   $128 = HEAP32[$pVfs>>2]|0;
   $129 = HEAP32[$zMaster>>2]|0;
   (_sqlite3OsDelete($128,$129,0)|0);
  }
  $150 = HEAP32[$rc>>2]|0;
  $151 = ($150|0)==(0);
  if ($151) {
   $152 = HEAP32[$pVfs>>2]|0;
   $153 = HEAP32[$zMaster>>2]|0;
   $154 = (_sqlite3OsOpenMalloc($152,$153,$pMaster,16406,0)|0);
   HEAP32[$rc>>2] = $154;
  }
  $155 = HEAP32[$rc>>2]|0;
  $156 = ($155|0)!=(0);
  if ($156) {
   $157 = HEAP32[$1>>2]|0;
   $158 = HEAP32[$zMaster>>2]|0;
   _sqlite3DbFree($157,$158);
   $159 = HEAP32[$rc>>2]|0;
   HEAP32[$0>>2] = $159;
   $292 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($292|0);
  }
  HEAP32[$i>>2] = 0;
  while(1) {
   $160 = HEAP32[$i>>2]|0;
   $161 = HEAP32[$1>>2]|0;
   $162 = (($161) + 20|0);
   $163 = HEAP32[$162>>2]|0;
   $164 = ($160|0)<($163|0);
   if (!($164)) {
    break;
   }
   $165 = HEAP32[$i>>2]|0;
   $166 = HEAP32[$1>>2]|0;
   $167 = (($166) + 16|0);
   $168 = HEAP32[$167>>2]|0;
   $169 = (($168) + ($165<<4)|0);
   $170 = (($169) + 4|0);
   $171 = HEAP32[$170>>2]|0;
   HEAP32[$pBt3>>2] = $171;
   $172 = HEAP32[$pBt3>>2]|0;
   $173 = (_sqlite3BtreeIsInTrans($172)|0);
   $174 = ($173|0)!=(0);
   if ($174) {
    $175 = HEAP32[$pBt3>>2]|0;
    $176 = (_sqlite3BtreeGetJournalname($175)|0);
    HEAP32[$zFile>>2] = $176;
    $177 = HEAP32[$zFile>>2]|0;
    $178 = ($177|0)==(0|0);
    if (!($178)) {
     $179 = HEAP32[$needSync>>2]|0;
     $180 = ($179|0)!=(0);
     if (!($180)) {
      $181 = HEAP32[$pBt3>>2]|0;
      $182 = (_sqlite3BtreeSyncDisabled($181)|0);
      $183 = ($182|0)!=(0);
      if (!($183)) {
       HEAP32[$needSync>>2] = 1;
      }
     }
     $184 = HEAP32[$pMaster>>2]|0;
     $185 = HEAP32[$zFile>>2]|0;
     $186 = HEAP32[$zFile>>2]|0;
     $187 = (_sqlite3Strlen30($186)|0);
     $188 = (($187) + 1)|0;
     $189 = $offset;
     $190 = $189;
     $191 = HEAP32[$190>>2]|0;
     $192 = (($189) + 4)|0;
     $193 = $192;
     $194 = HEAP32[$193>>2]|0;
     $195 = (_sqlite3OsWrite($184,$185,$188,$191,$194)|0);
     HEAP32[$rc>>2] = $195;
     $196 = HEAP32[$zFile>>2]|0;
     $197 = (_sqlite3Strlen30($196)|0);
     $198 = (($197) + 1)|0;
     $199 = ($198|0)<(0);
     $200 = $199 << 31 >> 31;
     $201 = $offset;
     $202 = $201;
     $203 = HEAP32[$202>>2]|0;
     $204 = (($201) + 4)|0;
     $205 = $204;
     $206 = HEAP32[$205>>2]|0;
     $207 = (_i64Add(($203|0),($206|0),($198|0),($200|0))|0);
     $208 = tempRet0;
     $209 = $offset;
     $210 = $209;
     HEAP32[$210>>2] = $207;
     $211 = (($209) + 4)|0;
     $212 = $211;
     HEAP32[$212>>2] = $208;
     $213 = HEAP32[$rc>>2]|0;
     $214 = ($213|0)!=(0);
     if ($214) {
      label = 50;
      break;
     }
    }
   }
   $221 = HEAP32[$i>>2]|0;
   $222 = (($221) + 1)|0;
   HEAP32[$i>>2] = $222;
  }
  if ((label|0) == 50) {
   $215 = HEAP32[$pMaster>>2]|0;
   (_sqlite3OsCloseFree($215)|0);
   $216 = HEAP32[$pVfs>>2]|0;
   $217 = HEAP32[$zMaster>>2]|0;
   (_sqlite3OsDelete($216,$217,0)|0);
   $218 = HEAP32[$1>>2]|0;
   $219 = HEAP32[$zMaster>>2]|0;
   _sqlite3DbFree($218,$219);
   $220 = HEAP32[$rc>>2]|0;
   HEAP32[$0>>2] = $220;
   $292 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($292|0);
  }
  $223 = HEAP32[$needSync>>2]|0;
  $224 = ($223|0)!=(0);
  if ($224) {
   $225 = HEAP32[$pMaster>>2]|0;
   $226 = (_sqlite3OsDeviceCharacteristics($225)|0);
   $227 = $226 & 1024;
   $228 = (0)==($227|0);
   if ($228) {
    $229 = HEAP32[$pMaster>>2]|0;
    $230 = (_sqlite3OsSync($229,2)|0);
    HEAP32[$rc>>2] = $230;
    $231 = (0)!=($230|0);
    if ($231) {
     $232 = HEAP32[$pMaster>>2]|0;
     (_sqlite3OsCloseFree($232)|0);
     $233 = HEAP32[$pVfs>>2]|0;
     $234 = HEAP32[$zMaster>>2]|0;
     (_sqlite3OsDelete($233,$234,0)|0);
     $235 = HEAP32[$1>>2]|0;
     $236 = HEAP32[$zMaster>>2]|0;
     _sqlite3DbFree($235,$236);
     $237 = HEAP32[$rc>>2]|0;
     HEAP32[$0>>2] = $237;
     $292 = HEAP32[$0>>2]|0;
     STACKTOP = sp;return ($292|0);
    }
   }
  }
  HEAP32[$i>>2] = 0;
  while(1) {
   $238 = HEAP32[$rc>>2]|0;
   $239 = ($238|0)==(0);
   if (!($239)) {
    break;
   }
   $240 = HEAP32[$i>>2]|0;
   $241 = HEAP32[$1>>2]|0;
   $242 = (($241) + 20|0);
   $243 = HEAP32[$242>>2]|0;
   $244 = ($240|0)<($243|0);
   if (!($244)) {
    break;
   }
   $245 = HEAP32[$i>>2]|0;
   $246 = HEAP32[$1>>2]|0;
   $247 = (($246) + 16|0);
   $248 = HEAP32[$247>>2]|0;
   $249 = (($248) + ($245<<4)|0);
   $250 = (($249) + 4|0);
   $251 = HEAP32[$250>>2]|0;
   HEAP32[$pBt4>>2] = $251;
   $252 = HEAP32[$pBt4>>2]|0;
   $253 = ($252|0)!=(0|0);
   if ($253) {
    $254 = HEAP32[$pBt4>>2]|0;
    $255 = HEAP32[$zMaster>>2]|0;
    $256 = (_sqlite3BtreeCommitPhaseOne($254,$255)|0);
    HEAP32[$rc>>2] = $256;
   }
   $257 = HEAP32[$i>>2]|0;
   $258 = (($257) + 1)|0;
   HEAP32[$i>>2] = $258;
  }
  $259 = HEAP32[$pMaster>>2]|0;
  (_sqlite3OsCloseFree($259)|0);
  $260 = HEAP32[$rc>>2]|0;
  $261 = ($260|0)!=(0);
  if ($261) {
   $262 = HEAP32[$1>>2]|0;
   $263 = HEAP32[$zMaster>>2]|0;
   _sqlite3DbFree($262,$263);
   $264 = HEAP32[$rc>>2]|0;
   HEAP32[$0>>2] = $264;
   $292 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($292|0);
  }
  $265 = HEAP32[$pVfs>>2]|0;
  $266 = HEAP32[$zMaster>>2]|0;
  $267 = (_sqlite3OsDelete($265,$266,1)|0);
  HEAP32[$rc>>2] = $267;
  $268 = HEAP32[$1>>2]|0;
  $269 = HEAP32[$zMaster>>2]|0;
  _sqlite3DbFree($268,$269);
  HEAP32[$zMaster>>2] = 0;
  $270 = HEAP32[$rc>>2]|0;
  $271 = ($270|0)!=(0);
  if ($271) {
   $272 = HEAP32[$rc>>2]|0;
   HEAP32[$0>>2] = $272;
   $292 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($292|0);
  }
  _sqlite3BeginBenignMalloc();
  HEAP32[$i>>2] = 0;
  while(1) {
   $273 = HEAP32[$i>>2]|0;
   $274 = HEAP32[$1>>2]|0;
   $275 = (($274) + 20|0);
   $276 = HEAP32[$275>>2]|0;
   $277 = ($273|0)<($276|0);
   if (!($277)) {
    break;
   }
   $278 = HEAP32[$i>>2]|0;
   $279 = HEAP32[$1>>2]|0;
   $280 = (($279) + 16|0);
   $281 = HEAP32[$280>>2]|0;
   $282 = (($281) + ($278<<4)|0);
   $283 = (($282) + 4|0);
   $284 = HEAP32[$283>>2]|0;
   HEAP32[$pBt5>>2] = $284;
   $285 = HEAP32[$pBt5>>2]|0;
   $286 = ($285|0)!=(0|0);
   if ($286) {
    $287 = HEAP32[$pBt5>>2]|0;
    (_sqlite3BtreeCommitPhaseTwo($287,1)|0);
   }
   $288 = HEAP32[$i>>2]|0;
   $289 = (($288) + 1)|0;
   HEAP32[$i>>2] = $289;
  }
  _sqlite3EndBenignMalloc();
  $290 = HEAP32[$1>>2]|0;
  (_sqlite3VtabCommit($290)|0);
 }
 $291 = HEAP32[$rc>>2]|0;
 HEAP32[$0>>2] = $291;
 $292 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($292|0);
}
function _sqlite3VtabSync($db,$p) {
 $db = $db|0;
 $p = $p|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $aVTrans = 0, $i = 0;
 var $pVtab = 0, $rc = 0, $x = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 24|0;
 $1 = sp + 20|0;
 $i = sp + 16|0;
 $rc = sp + 12|0;
 $aVTrans = sp + 8|0;
 $x = sp + 4|0;
 $pVtab = sp;
 HEAP32[$0>>2] = $db;
 HEAP32[$1>>2] = $p;
 HEAP32[$rc>>2] = 0;
 $2 = HEAP32[$0>>2]|0;
 $3 = (($2) + 328|0);
 $4 = HEAP32[$3>>2]|0;
 HEAP32[$aVTrans>>2] = $4;
 $5 = HEAP32[$0>>2]|0;
 $6 = (($5) + 328|0);
 HEAP32[$6>>2] = 0;
 HEAP32[$i>>2] = 0;
 while(1) {
  $7 = HEAP32[$rc>>2]|0;
  $8 = ($7|0)==(0);
  if (!($8)) {
   label = 8;
   break;
  }
  $9 = HEAP32[$i>>2]|0;
  $10 = HEAP32[$0>>2]|0;
  $11 = (($10) + 304|0);
  $12 = HEAP32[$11>>2]|0;
  $13 = ($9|0)<($12|0);
  if (!($13)) {
   label = 8;
   break;
  }
  $14 = HEAP32[$i>>2]|0;
  $15 = HEAP32[$aVTrans>>2]|0;
  $16 = (($15) + ($14<<2)|0);
  $17 = HEAP32[$16>>2]|0;
  $18 = (($17) + 8|0);
  $19 = HEAP32[$18>>2]|0;
  HEAP32[$pVtab>>2] = $19;
  $20 = HEAP32[$pVtab>>2]|0;
  $21 = ($20|0)!=(0|0);
  if ($21) {
   $22 = HEAP32[$pVtab>>2]|0;
   $23 = HEAP32[$22>>2]|0;
   $24 = (($23) + 60|0);
   $25 = HEAP32[$24>>2]|0;
   HEAP32[$x>>2] = $25;
   $26 = ($25|0)!=(0|0);
   if ($26) {
    $27 = HEAP32[$x>>2]|0;
    $28 = HEAP32[$pVtab>>2]|0;
    $29 = (FUNCTION_TABLE_ii[$27 & 31]($28)|0);
    HEAP32[$rc>>2] = $29;
    $30 = HEAP32[$1>>2]|0;
    $31 = HEAP32[$pVtab>>2]|0;
    _sqlite3VtabImportErrmsg($30,$31);
   }
  }
  $32 = HEAP32[$i>>2]|0;
  $33 = (($32) + 1)|0;
  HEAP32[$i>>2] = $33;
 }
 if ((label|0) == 8) {
  $34 = HEAP32[$aVTrans>>2]|0;
  $35 = HEAP32[$0>>2]|0;
  $36 = (($35) + 328|0);
  HEAP32[$36>>2] = $34;
  $37 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($37|0);
 }
 return 0|0;
}
function _sqlite3VtabCommit($db) {
 $db = $db|0;
 var $0 = 0, $1 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 HEAP32[$0>>2] = $db;
 $1 = HEAP32[$0>>2]|0;
 _callFinaliser($1,64);
 STACKTOP = sp;return 0;
}
function _sqlite3BtreeGetJournalname($p) {
 $p = $p|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 HEAP32[$0>>2] = $p;
 $1 = HEAP32[$0>>2]|0;
 $2 = (($1) + 4|0);
 $3 = HEAP32[$2>>2]|0;
 $4 = HEAP32[$3>>2]|0;
 $5 = (_sqlite3PagerJournalname($4)|0);
 STACKTOP = sp;return ($5|0);
}
function _sqlite3BtreeSyncDisabled($p) {
 $p = $p|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $pBt = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $pBt = sp + 4|0;
 $rc = sp;
 HEAP32[$0>>2] = $p;
 $1 = HEAP32[$0>>2]|0;
 $2 = (($1) + 4|0);
 $3 = HEAP32[$2>>2]|0;
 HEAP32[$pBt>>2] = $3;
 $4 = HEAP32[$0>>2]|0;
 _sqlite3BtreeEnter($4);
 $5 = HEAP32[$pBt>>2]|0;
 $6 = HEAP32[$5>>2]|0;
 $7 = (_sqlite3PagerNosync($6)|0);
 HEAP32[$rc>>2] = $7;
 $8 = HEAP32[$rc>>2]|0;
 STACKTOP = sp;return ($8|0);
}
function _sqlite3PagerNosync($pPager) {
 $pPager = $pPager|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 HEAP32[$0>>2] = $pPager;
 $1 = HEAP32[$0>>2]|0;
 $2 = (($1) + 7|0);
 $3 = HEAP8[$2>>0]|0;
 $4 = $3&255;
 STACKTOP = sp;return ($4|0);
}
function _sqlite3PagerJournalname($pPager) {
 $pPager = $pPager|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 HEAP32[$0>>2] = $pPager;
 $1 = HEAP32[$0>>2]|0;
 $2 = (($1) + 172|0);
 $3 = HEAP32[$2>>2]|0;
 STACKTOP = sp;return ($3|0);
}
function _sqlite3VdbeFrameDelete($p) {
 $p = $p|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $aMem = 0, $apCsr = 0, $i = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $i = sp + 8|0;
 $aMem = sp + 4|0;
 $apCsr = sp;
 HEAP32[$0>>2] = $p;
 $1 = HEAP32[$0>>2]|0;
 $2 = (($1) + 72|0);
 HEAP32[$aMem>>2] = $2;
 $3 = HEAP32[$0>>2]|0;
 $4 = (($3) + 60|0);
 $5 = HEAP32[$4>>2]|0;
 $6 = HEAP32[$aMem>>2]|0;
 $7 = (($6) + (($5*40)|0)|0);
 HEAP32[$apCsr>>2] = $7;
 HEAP32[$i>>2] = 0;
 while(1) {
  $8 = HEAP32[$i>>2]|0;
  $9 = HEAP32[$0>>2]|0;
  $10 = (($9) + 64|0);
  $11 = HEAP32[$10>>2]|0;
  $12 = ($8|0)<($11|0);
  if (!($12)) {
   break;
  }
  $13 = HEAP32[$0>>2]|0;
  $14 = HEAP32[$13>>2]|0;
  $15 = HEAP32[$i>>2]|0;
  $16 = HEAP32[$apCsr>>2]|0;
  $17 = (($16) + ($15<<2)|0);
  $18 = HEAP32[$17>>2]|0;
  _sqlite3VdbeFreeCursor($14,$18);
  $19 = HEAP32[$i>>2]|0;
  $20 = (($19) + 1)|0;
  HEAP32[$i>>2] = $20;
 }
 $21 = HEAP32[$aMem>>2]|0;
 $22 = HEAP32[$0>>2]|0;
 $23 = (($22) + 60|0);
 $24 = HEAP32[$23>>2]|0;
 _releaseMemArray($21,$24);
 $25 = HEAP32[$0>>2]|0;
 $26 = HEAP32[$25>>2]|0;
 $27 = HEAP32[$26>>2]|0;
 $28 = HEAP32[$0>>2]|0;
 _sqlite3DbFree($27,$28);
 STACKTOP = sp;return;
}
function _sqlite3OpcodeName($i) {
 $i = $i|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 HEAP32[$0>>2] = $i;
 $1 = HEAP32[$0>>2]|0;
 $2 = (34600 + ($1<<2)|0);
 $3 = HEAP32[$2>>2]|0;
 STACKTOP = sp;return ($3|0);
}
function _displayP4($pOp,$zTemp,$nTemp) {
 $pOp = $pOp|0;
 $zTemp = $zTemp|0;
 $nTemp = $nTemp|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0.0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0.0, $167 = 0, $168 = 0, $169 = 0, $17 = 0;
 var $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0, $188 = 0;
 var $189 = 0, $19 = 0, $190 = 0, $191 = 0, $192 = 0, $193 = 0, $194 = 0, $195 = 0, $196 = 0, $197 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0;
 var $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0;
 var $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0;
 var $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0;
 var $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $i = 0, $j = 0;
 var $n = 0, $pColl = 0, $pColl1 = 0, $pDef = 0, $pKeyInfo = 0, $pMem = 0, $pVtab = 0, $vararg_buffer31 = 0, $vararg_ptr28 = 0, $vararg_ptr7 = 0, $zColl = 0, $zP4 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 64|0;
 $vararg_buffer31 = sp;
 $0 = sp + 60|0;
 $1 = sp + 56|0;
 $2 = sp + 52|0;
 $zP4 = sp + 48|0;
 $i = sp + 44|0;
 $j = sp + 40|0;
 $pKeyInfo = sp + 36|0;
 $pColl = sp + 32|0;
 $zColl = sp + 28|0;
 $n = sp + 24|0;
 $pColl1 = sp + 20|0;
 $pDef = sp + 16|0;
 $pMem = sp + 12|0;
 $pVtab = sp + 8|0;
 HEAP32[$0>>2] = $pOp;
 HEAP32[$1>>2] = $zTemp;
 HEAP32[$2>>2] = $nTemp;
 $3 = HEAP32[$1>>2]|0;
 HEAP32[$zP4>>2] = $3;
 $4 = HEAP32[$0>>2]|0;
 $5 = (($4) + 1|0);
 $6 = HEAP8[$5>>0]|0;
 $7 = $6 << 24 >> 24;
 do {
  switch ($7|0) {
  case -5:  {
   $90 = HEAP32[$0>>2]|0;
   $91 = (($90) + 16|0);
   $92 = HEAP32[$91>>2]|0;
   HEAP32[$pDef>>2] = $92;
   $93 = HEAP32[$2>>2]|0;
   $94 = HEAP32[$1>>2]|0;
   $95 = HEAP32[$pDef>>2]|0;
   $96 = (($95) + 24|0);
   $97 = HEAP32[$96>>2]|0;
   $98 = HEAP32[$pDef>>2]|0;
   $99 = HEAP16[$98>>1]|0;
   $100 = $99 << 16 >> 16;
   HEAP32[$vararg_buffer31>>2] = $97;
   $vararg_ptr7 = (($vararg_buffer31) + 4|0);
   HEAP32[$vararg_ptr7>>2] = $100;
   (_sqlite3_snprintf($93,$94,34536,$vararg_buffer31)|0);
   $197 = HEAP32[$zP4>>2]|0;
   STACKTOP = sp;return ($197|0);
   break;
  }
  case -18:  {
   $187 = HEAP32[$2>>2]|0;
   $188 = HEAP32[$1>>2]|0;
   (_sqlite3_snprintf($187,$188,34592,$vararg_buffer31)|0);
   $197 = HEAP32[$zP4>>2]|0;
   STACKTOP = sp;return ($197|0);
   break;
  }
  case -19:  {
   $189 = HEAP32[$1>>2]|0;
   HEAP8[$189>>0] = 0;
   $197 = HEAP32[$zP4>>2]|0;
   STACKTOP = sp;return ($197|0);
   break;
  }
  case -8:  {
   $127 = HEAP32[$0>>2]|0;
   $128 = (($127) + 16|0);
   $129 = HEAP32[$128>>2]|0;
   HEAP32[$pMem>>2] = $129;
   $130 = HEAP32[$pMem>>2]|0;
   $131 = (($130) + 8|0);
   $132 = HEAP16[$131>>1]|0;
   $133 = $132&65535;
   $134 = $133 & 2;
   $135 = ($134|0)!=(0);
   $136 = HEAP32[$pMem>>2]|0;
   if ($135) {
    $137 = (($136) + 16|0);
    $138 = HEAP32[$137>>2]|0;
    HEAP32[$zP4>>2] = $138;
    $197 = HEAP32[$zP4>>2]|0;
    STACKTOP = sp;return ($197|0);
   }
   $139 = (($136) + 8|0);
   $140 = HEAP16[$139>>1]|0;
   $141 = $140&65535;
   $142 = $141 & 4;
   $143 = ($142|0)!=(0);
   if ($143) {
    $144 = HEAP32[$2>>2]|0;
    $145 = HEAP32[$1>>2]|0;
    $146 = HEAP32[$pMem>>2]|0;
    $147 = $146;
    $148 = $147;
    $149 = HEAP32[$148>>2]|0;
    $150 = (($147) + 4)|0;
    $151 = $150;
    $152 = HEAP32[$151>>2]|0;
    $153 = $vararg_buffer31;
    $154 = $153;
    HEAP32[$154>>2] = $149;
    $155 = (($153) + 4)|0;
    $156 = $155;
    HEAP32[$156>>2] = $152;
    (_sqlite3_snprintf($144,$145,9456,$vararg_buffer31)|0);
    $197 = HEAP32[$zP4>>2]|0;
    STACKTOP = sp;return ($197|0);
   }
   $157 = HEAP32[$pMem>>2]|0;
   $158 = (($157) + 8|0);
   $159 = HEAP16[$158>>1]|0;
   $160 = $159&65535;
   $161 = $160 & 8;
   $162 = ($161|0)!=(0);
   if ($162) {
    $163 = HEAP32[$2>>2]|0;
    $164 = HEAP32[$1>>2]|0;
    $165 = HEAP32[$pMem>>2]|0;
    $166 = +HEAPF64[$165>>3];
    HEAPF64[tempDoublePtr>>3]=$166;HEAP32[$vararg_buffer31>>2]=HEAP32[tempDoublePtr>>2];HEAP32[$vararg_buffer31+4>>2]=HEAP32[tempDoublePtr+4>>2];
    (_sqlite3_snprintf($163,$164,9448,$vararg_buffer31)|0);
    $197 = HEAP32[$zP4>>2]|0;
    STACKTOP = sp;return ($197|0);
   }
   $167 = HEAP32[$pMem>>2]|0;
   $168 = (($167) + 8|0);
   $169 = HEAP16[$168>>1]|0;
   $170 = $169&65535;
   $171 = $170 & 1;
   $172 = ($171|0)!=(0);
   if ($172) {
    $173 = HEAP32[$2>>2]|0;
    $174 = HEAP32[$1>>2]|0;
    (_sqlite3_snprintf($173,$174,9840,$vararg_buffer31)|0);
    $197 = HEAP32[$zP4>>2]|0;
    STACKTOP = sp;return ($197|0);
   } else {
    HEAP32[$zP4>>2] = 34552;
    $197 = HEAP32[$zP4>>2]|0;
    STACKTOP = sp;return ($197|0);
   }
   break;
  }
  case -13:  {
   $101 = HEAP32[$2>>2]|0;
   $102 = HEAP32[$1>>2]|0;
   $103 = HEAP32[$0>>2]|0;
   $104 = (($103) + 16|0);
   $105 = HEAP32[$104>>2]|0;
   $106 = $105;
   $107 = $106;
   $108 = HEAP32[$107>>2]|0;
   $109 = (($106) + 4)|0;
   $110 = $109;
   $111 = HEAP32[$110>>2]|0;
   $112 = $vararg_buffer31;
   $113 = $112;
   HEAP32[$113>>2] = $108;
   $114 = (($112) + 4)|0;
   $115 = $114;
   HEAP32[$115>>2] = $111;
   (_sqlite3_snprintf($101,$102,9456,$vararg_buffer31)|0);
   $197 = HEAP32[$zP4>>2]|0;
   STACKTOP = sp;return ($197|0);
   break;
  }
  case -15:  {
   $185 = HEAP32[$2>>2]|0;
   $186 = HEAP32[$1>>2]|0;
   (_sqlite3_snprintf($185,$186,34576,$vararg_buffer31)|0);
   $197 = HEAP32[$zP4>>2]|0;
   STACKTOP = sp;return ($197|0);
   break;
  }
  case -6:  {
   $8 = HEAP32[$0>>2]|0;
   $9 = (($8) + 16|0);
   $10 = HEAP32[$9>>2]|0;
   HEAP32[$pKeyInfo>>2] = $10;
   $11 = HEAP32[$2>>2]|0;
   $12 = HEAP32[$1>>2]|0;
   $13 = HEAP32[$pKeyInfo>>2]|0;
   $14 = (($13) + 6|0);
   $15 = HEAP16[$14>>1]|0;
   $16 = $15&65535;
   HEAP32[$vararg_buffer31>>2] = $16;
   (_sqlite3_snprintf($11,$12,34496,$vararg_buffer31)|0);
   $17 = HEAP32[$1>>2]|0;
   $18 = (_sqlite3Strlen30($17)|0);
   HEAP32[$i>>2] = $18;
   HEAP32[$j>>2] = 0;
   while(1) {
    $19 = HEAP32[$j>>2]|0;
    $20 = HEAP32[$pKeyInfo>>2]|0;
    $21 = (($20) + 6|0);
    $22 = HEAP16[$21>>1]|0;
    $23 = $22&65535;
    $24 = ($19|0)<($23|0);
    if (!($24)) {
     break;
    }
    $25 = HEAP32[$j>>2]|0;
    $26 = HEAP32[$pKeyInfo>>2]|0;
    $27 = (($26) + 20|0);
    $28 = (($27) + ($25<<2)|0);
    $29 = HEAP32[$28>>2]|0;
    HEAP32[$pColl>>2] = $29;
    $30 = HEAP32[$pColl>>2]|0;
    $31 = ($30|0)!=(0|0);
    if ($31) {
     $32 = HEAP32[$pColl>>2]|0;
     $33 = HEAP32[$32>>2]|0;
     $34 = $33;
    } else {
     $34 = 34504;
    }
    HEAP32[$zColl>>2] = $34;
    $35 = HEAP32[$zColl>>2]|0;
    $36 = (_sqlite3Strlen30($35)|0);
    HEAP32[$n>>2] = $36;
    $37 = HEAP32[$n>>2]|0;
    $38 = ($37|0)==(6);
    if ($38) {
     $39 = HEAP32[$zColl>>2]|0;
     $40 = (_memcmp($39,3320,6)|0);
     $41 = ($40|0)==(0);
     if ($41) {
      HEAP32[$zColl>>2] = 34512;
      HEAP32[$n>>2] = 1;
     }
    }
    $42 = HEAP32[$i>>2]|0;
    $43 = HEAP32[$n>>2]|0;
    $44 = (($42) + ($43))|0;
    $45 = HEAP32[$2>>2]|0;
    $46 = (($45) - 6)|0;
    $47 = ($44|0)>($46|0);
    $48 = HEAP32[$i>>2]|0;
    if ($47) {
     label = 10;
     break;
    }
    $51 = (($48) + 1)|0;
    HEAP32[$i>>2] = $51;
    $52 = HEAP32[$1>>2]|0;
    $53 = (($52) + ($48)|0);
    HEAP8[$53>>0] = 44;
    $54 = HEAP32[$j>>2]|0;
    $55 = HEAP32[$pKeyInfo>>2]|0;
    $56 = (($55) + 16|0);
    $57 = HEAP32[$56>>2]|0;
    $58 = (($57) + ($54)|0);
    $59 = HEAP8[$58>>0]|0;
    $60 = ($59<<24>>24)!=(0);
    if ($60) {
     $61 = HEAP32[$i>>2]|0;
     $62 = (($61) + 1)|0;
     HEAP32[$i>>2] = $62;
     $63 = HEAP32[$1>>2]|0;
     $64 = (($63) + ($61)|0);
     HEAP8[$64>>0] = 45;
    }
    $65 = HEAP32[$i>>2]|0;
    $66 = HEAP32[$1>>2]|0;
    $67 = (($66) + ($65)|0);
    $68 = HEAP32[$zColl>>2]|0;
    $69 = HEAP32[$n>>2]|0;
    $70 = (($69) + 1)|0;
    _memcpy(($67|0),($68|0),($70|0))|0;
    $71 = HEAP32[$n>>2]|0;
    $72 = HEAP32[$i>>2]|0;
    $73 = (($72) + ($71))|0;
    HEAP32[$i>>2] = $73;
    $74 = HEAP32[$j>>2]|0;
    $75 = (($74) + 1)|0;
    HEAP32[$j>>2] = $75;
   }
   if ((label|0) == 10) {
    $49 = HEAP32[$1>>2]|0;
    $50 = (($49) + ($48)|0);
    ;HEAP8[$50+0>>0]=HEAP8[34520+0>>0]|0;HEAP8[$50+1>>0]=HEAP8[34520+1>>0]|0;HEAP8[$50+2>>0]=HEAP8[34520+2>>0]|0;HEAP8[$50+3>>0]=HEAP8[34520+3>>0]|0;
   }
   $76 = HEAP32[$i>>2]|0;
   $77 = (($76) + 1)|0;
   HEAP32[$i>>2] = $77;
   $78 = HEAP32[$1>>2]|0;
   $79 = (($78) + ($76)|0);
   HEAP8[$79>>0] = 41;
   $80 = HEAP32[$i>>2]|0;
   $81 = HEAP32[$1>>2]|0;
   $82 = (($81) + ($80)|0);
   HEAP8[$82>>0] = 0;
   $197 = HEAP32[$zP4>>2]|0;
   STACKTOP = sp;return ($197|0);
   break;
  }
  case -14:  {
   $116 = HEAP32[$2>>2]|0;
   $117 = HEAP32[$1>>2]|0;
   $118 = HEAP32[$0>>2]|0;
   $119 = (($118) + 16|0);
   $120 = HEAP32[$119>>2]|0;
   HEAP32[$vararg_buffer31>>2] = $120;
   (_sqlite3_snprintf($116,$117,34544,$vararg_buffer31)|0);
   $197 = HEAP32[$zP4>>2]|0;
   STACKTOP = sp;return ($197|0);
   break;
  }
  case -12:  {
   $121 = HEAP32[$2>>2]|0;
   $122 = HEAP32[$1>>2]|0;
   $123 = HEAP32[$0>>2]|0;
   $124 = (($123) + 16|0);
   $125 = HEAP32[$124>>2]|0;
   $126 = +HEAPF64[$125>>3];
   HEAPF64[tempDoublePtr>>3]=$126;HEAP32[$vararg_buffer31>>2]=HEAP32[tempDoublePtr>>2];HEAP32[$vararg_buffer31+4>>2]=HEAP32[tempDoublePtr+4>>2];
   (_sqlite3_snprintf($121,$122,9448,$vararg_buffer31)|0);
   $197 = HEAP32[$zP4>>2]|0;
   STACKTOP = sp;return ($197|0);
   break;
  }
  case -10:  {
   $175 = HEAP32[$0>>2]|0;
   $176 = (($175) + 16|0);
   $177 = HEAP32[$176>>2]|0;
   $178 = (($177) + 8|0);
   $179 = HEAP32[$178>>2]|0;
   HEAP32[$pVtab>>2] = $179;
   $180 = HEAP32[$2>>2]|0;
   $181 = HEAP32[$1>>2]|0;
   $182 = HEAP32[$pVtab>>2]|0;
   $183 = HEAP32[$pVtab>>2]|0;
   $184 = HEAP32[$183>>2]|0;
   HEAP32[$vararg_buffer31>>2] = $182;
   $vararg_ptr28 = (($vararg_buffer31) + 4|0);
   HEAP32[$vararg_ptr28>>2] = $184;
   (_sqlite3_snprintf($180,$181,34560,$vararg_buffer31)|0);
   $197 = HEAP32[$zP4>>2]|0;
   STACKTOP = sp;return ($197|0);
   break;
  }
  case -4:  {
   $83 = HEAP32[$0>>2]|0;
   $84 = (($83) + 16|0);
   $85 = HEAP32[$84>>2]|0;
   HEAP32[$pColl1>>2] = $85;
   $86 = HEAP32[$2>>2]|0;
   $87 = HEAP32[$1>>2]|0;
   $88 = HEAP32[$pColl1>>2]|0;
   $89 = HEAP32[$88>>2]|0;
   HEAP32[$vararg_buffer31>>2] = $89;
   (_sqlite3_snprintf($86,$87,34528,$vararg_buffer31)|0);
   $197 = HEAP32[$zP4>>2]|0;
   STACKTOP = sp;return ($197|0);
   break;
  }
  default: {
   $190 = HEAP32[$0>>2]|0;
   $191 = (($190) + 16|0);
   $192 = HEAP32[$191>>2]|0;
   HEAP32[$zP4>>2] = $192;
   $193 = HEAP32[$zP4>>2]|0;
   $194 = ($193|0)==(0|0);
   if (!($194)) {
    $197 = HEAP32[$zP4>>2]|0;
    STACKTOP = sp;return ($197|0);
   }
   $195 = HEAP32[$1>>2]|0;
   HEAP32[$zP4>>2] = $195;
   $196 = HEAP32[$1>>2]|0;
   HEAP8[$196>>0] = 0;
   $197 = HEAP32[$zP4>>2]|0;
   STACKTOP = sp;return ($197|0);
  }
  }
 } while(0);
 return 0|0;
}
function _vdbeReleaseAndSetInt64($pMem,$0,$1) {
 $pMem = $pMem|0;
 $0 = $0|0;
 $1 = $1|0;
 var $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0;
 var label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $2 = sp + 8|0;
 $3 = sp;
 HEAP32[$2>>2] = $pMem;
 $4 = $3;
 $5 = $4;
 HEAP32[$5>>2] = $0;
 $6 = (($4) + 4)|0;
 $7 = $6;
 HEAP32[$7>>2] = $1;
 $8 = HEAP32[$2>>2]|0;
 _sqlite3VdbeMemSetNull($8);
 $9 = $3;
 $10 = $9;
 $11 = HEAP32[$10>>2]|0;
 $12 = (($9) + 4)|0;
 $13 = $12;
 $14 = HEAP32[$13>>2]|0;
 $15 = HEAP32[$2>>2]|0;
 $16 = $15;
 $17 = $16;
 HEAP32[$17>>2] = $11;
 $18 = (($16) + 4)|0;
 $19 = $18;
 HEAP32[$19>>2] = $14;
 $20 = HEAP32[$2>>2]|0;
 $21 = (($20) + 8|0);
 HEAP16[$21>>1] = 4;
 STACKTOP = sp;return;
}
function _sqlite3VdbeMemHandleBom($pMem) {
 $pMem = $pMem|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $8 = 0, $9 = 0, $b1 = 0, $b2 = 0, $bom = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $rc = sp;
 $bom = sp + 10|0;
 $b1 = sp + 9|0;
 $b2 = sp + 8|0;
 HEAP32[$0>>2] = $pMem;
 HEAP32[$rc>>2] = 0;
 HEAP8[$bom>>0] = 0;
 $1 = HEAP32[$0>>2]|0;
 $2 = (($1) + 12|0);
 $3 = HEAP32[$2>>2]|0;
 $4 = ($3|0)>(1);
 if ($4) {
  $5 = HEAP32[$0>>2]|0;
  $6 = (($5) + 16|0);
  $7 = HEAP32[$6>>2]|0;
  $8 = HEAP8[$7>>0]|0;
  HEAP8[$b1>>0] = $8;
  $9 = HEAP32[$0>>2]|0;
  $10 = (($9) + 16|0);
  $11 = HEAP32[$10>>2]|0;
  $12 = (($11) + 1|0);
  $13 = HEAP8[$12>>0]|0;
  HEAP8[$b2>>0] = $13;
  $14 = HEAP8[$b1>>0]|0;
  $15 = $14&255;
  $16 = ($15|0)==(254);
  if ($16) {
   $17 = HEAP8[$b2>>0]|0;
   $18 = $17&255;
   $19 = ($18|0)==(255);
   if ($19) {
    HEAP8[$bom>>0] = 3;
   }
  }
  $20 = HEAP8[$b1>>0]|0;
  $21 = $20&255;
  $22 = ($21|0)==(255);
  if ($22) {
   $23 = HEAP8[$b2>>0]|0;
   $24 = $23&255;
   $25 = ($24|0)==(254);
   if ($25) {
    HEAP8[$bom>>0] = 2;
   }
  }
 }
 $26 = HEAP8[$bom>>0]|0;
 $27 = ($26<<24>>24)!=(0);
 if (!($27)) {
  $70 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($70|0);
 }
 $28 = HEAP32[$0>>2]|0;
 $29 = (_sqlite3VdbeMemMakeWriteable($28)|0);
 HEAP32[$rc>>2] = $29;
 $30 = HEAP32[$rc>>2]|0;
 $31 = ($30|0)==(0);
 if (!($31)) {
  $70 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($70|0);
 }
 $32 = HEAP32[$0>>2]|0;
 $33 = (($32) + 12|0);
 $34 = HEAP32[$33>>2]|0;
 $35 = (($34) - 2)|0;
 HEAP32[$33>>2] = $35;
 $36 = HEAP32[$0>>2]|0;
 $37 = (($36) + 16|0);
 $38 = HEAP32[$37>>2]|0;
 $39 = HEAP32[$0>>2]|0;
 $40 = (($39) + 16|0);
 $41 = HEAP32[$40>>2]|0;
 $42 = (($41) + 2|0);
 $43 = HEAP32[$0>>2]|0;
 $44 = (($43) + 12|0);
 $45 = HEAP32[$44>>2]|0;
 _memmove(($38|0),($42|0),($45|0))|0;
 $46 = HEAP32[$0>>2]|0;
 $47 = (($46) + 12|0);
 $48 = HEAP32[$47>>2]|0;
 $49 = HEAP32[$0>>2]|0;
 $50 = (($49) + 16|0);
 $51 = HEAP32[$50>>2]|0;
 $52 = (($51) + ($48)|0);
 HEAP8[$52>>0] = 0;
 $53 = HEAP32[$0>>2]|0;
 $54 = (($53) + 12|0);
 $55 = HEAP32[$54>>2]|0;
 $56 = (($55) + 1)|0;
 $57 = HEAP32[$0>>2]|0;
 $58 = (($57) + 16|0);
 $59 = HEAP32[$58>>2]|0;
 $60 = (($59) + ($56)|0);
 HEAP8[$60>>0] = 0;
 $61 = HEAP32[$0>>2]|0;
 $62 = (($61) + 8|0);
 $63 = HEAP16[$62>>1]|0;
 $64 = $63&65535;
 $65 = $64 | 512;
 $66 = $65&65535;
 HEAP16[$62>>1] = $66;
 $67 = HEAP8[$bom>>0]|0;
 $68 = HEAP32[$0>>2]|0;
 $69 = (($68) + 10|0);
 HEAP8[$69>>0] = $67;
 $70 = HEAP32[$rc>>2]|0;
 STACKTOP = sp;return ($70|0);
}
function _valueToText($pVal,$enc) {
 $pVal = $pVal|0;
 $enc = $enc|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $7 = 0, $8 = 0;
 var $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $1 = sp;
 $2 = sp + 8|0;
 HEAP32[$1>>2] = $pVal;
 HEAP8[$2>>0] = $enc;
 $3 = HEAP32[$1>>2]|0;
 $4 = (($3) + 8|0);
 $5 = HEAP16[$4>>1]|0;
 $6 = $5&65535;
 $7 = $6 & 18;
 $8 = ($7|0)!=(0);
 $9 = HEAP32[$1>>2]|0;
 if ($8) {
  $10 = (($9) + 8|0);
  $11 = HEAP16[$10>>1]|0;
  $12 = $11&65535;
  $13 = $12 | 2;
  $14 = $13&65535;
  HEAP16[$10>>1] = $14;
  $15 = HEAP32[$1>>2]|0;
  $16 = (($15) + 8|0);
  $17 = HEAP16[$16>>1]|0;
  $18 = $17&65535;
  $19 = $18 & 16384;
  $20 = ($19|0)!=(0);
  if ($20) {
   $21 = HEAP32[$1>>2]|0;
   (_sqlite3VdbeMemExpandBlob($21)|0);
  }
  $22 = HEAP32[$1>>2]|0;
  $23 = (($22) + 10|0);
  $24 = HEAP8[$23>>0]|0;
  $25 = $24&255;
  $26 = HEAP8[$2>>0]|0;
  $27 = $26&255;
  $28 = $27 & -9;
  $29 = ($25|0)!=($28|0);
  if ($29) {
   $30 = HEAP32[$1>>2]|0;
   $31 = HEAP8[$2>>0]|0;
   $32 = $31&255;
   $33 = $32 & -9;
   (_sqlite3VdbeChangeEncoding($30,$33)|0);
  }
  $34 = HEAP8[$2>>0]|0;
  $35 = $34&255;
  $36 = $35 & 8;
  $37 = ($36|0)!=(0);
  if ($37) {
   $38 = HEAP32[$1>>2]|0;
   $39 = (($38) + 16|0);
   $40 = HEAP32[$39>>2]|0;
   $41 = $40;
   $42 = 1 & $41;
   $43 = (1)==($42|0);
   if ($43) {
    $44 = HEAP32[$1>>2]|0;
    $45 = (_sqlite3VdbeMemMakeWriteable($44)|0);
    $46 = ($45|0)!=(0);
    if ($46) {
     HEAP32[$0>>2] = 0;
     $60 = HEAP32[$0>>2]|0;
     STACKTOP = sp;return ($60|0);
    }
   }
  }
  $47 = HEAP32[$1>>2]|0;
  (_sqlite3VdbeMemNulTerminate($47)|0);
 } else {
  $48 = HEAP8[$2>>0]|0;
  (_sqlite3VdbeMemStringify($9,$48,0)|0);
 }
 $49 = HEAP32[$1>>2]|0;
 $50 = (($49) + 10|0);
 $51 = HEAP8[$50>>0]|0;
 $52 = $51&255;
 $53 = HEAP8[$2>>0]|0;
 $54 = $53&255;
 $55 = $54 & -9;
 $56 = ($52|0)==($55|0);
 if ($56) {
  $57 = HEAP32[$1>>2]|0;
  $58 = (($57) + 16|0);
  $59 = HEAP32[$58>>2]|0;
  HEAP32[$0>>2] = $59;
  $60 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($60|0);
 } else {
  HEAP32[$0>>2] = 0;
  $60 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($60|0);
 }
 return 0|0;
}
function _vdbeMemClear($p) {
 $p = $p|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0;
 var $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 HEAP32[$0>>2] = $p;
 $1 = HEAP32[$0>>2]|0;
 $2 = (($1) + 8|0);
 $3 = HEAP16[$2>>1]|0;
 $4 = $3&65535;
 $5 = $4 & 9312;
 $6 = ($5|0)!=(0);
 if ($6) {
  $7 = HEAP32[$0>>2]|0;
  _vdbeMemClearExternAndSetNull($7);
 }
 $8 = HEAP32[$0>>2]|0;
 $9 = (($8) + 24|0);
 $10 = HEAP32[$9>>2]|0;
 $11 = ($10|0)!=(0);
 if (!($11)) {
  $20 = HEAP32[$0>>2]|0;
  $21 = (($20) + 16|0);
  HEAP32[$21>>2] = 0;
  STACKTOP = sp;return;
 }
 $12 = HEAP32[$0>>2]|0;
 $13 = (($12) + 32|0);
 $14 = HEAP32[$13>>2]|0;
 $15 = HEAP32[$0>>2]|0;
 $16 = (($15) + 20|0);
 $17 = HEAP32[$16>>2]|0;
 _sqlite3DbFree($14,$17);
 $18 = HEAP32[$0>>2]|0;
 $19 = (($18) + 24|0);
 HEAP32[$19>>2] = 0;
 $20 = HEAP32[$0>>2]|0;
 $21 = (($20) + 16|0);
 HEAP32[$21>>2] = 0;
 STACKTOP = sp;return;
}
function _Cleanup($p) {
 $p = $p|0;
 var $0 = 0, $1 = 0, $10 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $db = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $db = sp;
 HEAP32[$0>>2] = $p;
 $1 = HEAP32[$0>>2]|0;
 $2 = HEAP32[$1>>2]|0;
 HEAP32[$db>>2] = $2;
 $3 = HEAP32[$db>>2]|0;
 $4 = HEAP32[$0>>2]|0;
 $5 = (($4) + 44|0);
 $6 = HEAP32[$5>>2]|0;
 _sqlite3DbFree($3,$6);
 $7 = HEAP32[$0>>2]|0;
 $8 = (($7) + 44|0);
 HEAP32[$8>>2] = 0;
 $9 = HEAP32[$0>>2]|0;
 $10 = (($9) + 20|0);
 HEAP32[$10>>2] = 0;
 STACKTOP = sp;return;
}
function _apiOomError($db) {
 $db = $db|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 HEAP32[$0>>2] = $db;
 $1 = HEAP32[$0>>2]|0;
 $2 = (($1) + 64|0);
 HEAP8[$2>>0] = 0;
 $3 = HEAP32[$0>>2]|0;
 _sqlite3Error($3,7);
 STACKTOP = sp;return 7;
}
function _vdbeFreeOpArray($db,$aOp,$nOp) {
 $db = $db|0;
 $aOp = $aOp|0;
 $nOp = $nOp|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0;
 var $7 = 0, $8 = 0, $9 = 0, $pOp = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $1 = sp + 8|0;
 $2 = sp + 4|0;
 $pOp = sp;
 HEAP32[$0>>2] = $db;
 HEAP32[$1>>2] = $aOp;
 HEAP32[$2>>2] = $nOp;
 $3 = HEAP32[$1>>2]|0;
 $4 = ($3|0)!=(0|0);
 if (!($4)) {
  $21 = HEAP32[$0>>2]|0;
  $22 = HEAP32[$1>>2]|0;
  _sqlite3DbFree($21,$22);
  STACKTOP = sp;return;
 }
 $5 = HEAP32[$1>>2]|0;
 HEAP32[$pOp>>2] = $5;
 while(1) {
  $6 = HEAP32[$pOp>>2]|0;
  $7 = HEAP32[$2>>2]|0;
  $8 = HEAP32[$1>>2]|0;
  $9 = (($8) + (($7*20)|0)|0);
  $10 = ($6>>>0)<($9>>>0);
  if (!($10)) {
   break;
  }
  $11 = HEAP32[$0>>2]|0;
  $12 = HEAP32[$pOp>>2]|0;
  $13 = (($12) + 1|0);
  $14 = HEAP8[$13>>0]|0;
  $15 = $14 << 24 >> 24;
  $16 = HEAP32[$pOp>>2]|0;
  $17 = (($16) + 16|0);
  $18 = HEAP32[$17>>2]|0;
  _freeP4($11,$15,$18);
  $19 = HEAP32[$pOp>>2]|0;
  $20 = (($19) + 20|0);
  HEAP32[$pOp>>2] = $20;
 }
 $21 = HEAP32[$0>>2]|0;
 $22 = HEAP32[$1>>2]|0;
 _sqlite3DbFree($21,$22);
 STACKTOP = sp;return;
}
function _btreeClearHasContent($pBt) {
 $pBt = $pBt|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 HEAP32[$0>>2] = $pBt;
 $1 = HEAP32[$0>>2]|0;
 $2 = (($1) + 60|0);
 $3 = HEAP32[$2>>2]|0;
 _sqlite3BitvecDestroy($3);
 $4 = HEAP32[$0>>2]|0;
 $5 = (($4) + 60|0);
 HEAP32[$5>>2] = 0;
 STACKTOP = sp;return;
}
function _btreeEndTransaction($p) {
 $p = $p|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $db = 0;
 var $pBt = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $pBt = sp + 4|0;
 $db = sp;
 HEAP32[$0>>2] = $p;
 $1 = HEAP32[$0>>2]|0;
 $2 = (($1) + 4|0);
 $3 = HEAP32[$2>>2]|0;
 HEAP32[$pBt>>2] = $3;
 $4 = HEAP32[$0>>2]|0;
 $5 = HEAP32[$4>>2]|0;
 HEAP32[$db>>2] = $5;
 $6 = HEAP32[$pBt>>2]|0;
 $7 = (($6) + 19|0);
 HEAP8[$7>>0] = 0;
 $8 = HEAP32[$0>>2]|0;
 $9 = (($8) + 8|0);
 $10 = HEAP8[$9>>0]|0;
 $11 = $10&255;
 $12 = ($11|0)>(0);
 if ($12) {
  $13 = HEAP32[$db>>2]|0;
  $14 = (($13) + 152|0);
  $15 = HEAP32[$14>>2]|0;
  $16 = ($15|0)>(1);
  if ($16) {
   $17 = HEAP32[$0>>2]|0;
   _downgradeAllSharedCacheTableLocks($17);
   $18 = HEAP32[$0>>2]|0;
   $19 = (($18) + 8|0);
   HEAP8[$19>>0] = 1;
   STACKTOP = sp;return;
  }
 }
 $20 = HEAP32[$0>>2]|0;
 $21 = (($20) + 8|0);
 $22 = HEAP8[$21>>0]|0;
 $23 = $22&255;
 $24 = ($23|0)!=(0);
 if ($24) {
  $25 = HEAP32[$0>>2]|0;
  _clearAllSharedCacheTableLocks($25);
  $26 = HEAP32[$pBt>>2]|0;
  $27 = (($26) + 40|0);
  $28 = HEAP32[$27>>2]|0;
  $29 = (($28) + -1)|0;
  HEAP32[$27>>2] = $29;
  $30 = HEAP32[$pBt>>2]|0;
  $31 = (($30) + 40|0);
  $32 = HEAP32[$31>>2]|0;
  $33 = (0)==($32|0);
  if ($33) {
   $34 = HEAP32[$pBt>>2]|0;
   $35 = (($34) + 20|0);
   HEAP8[$35>>0] = 0;
  }
 }
 $36 = HEAP32[$0>>2]|0;
 $37 = (($36) + 8|0);
 HEAP8[$37>>0] = 0;
 $38 = HEAP32[$pBt>>2]|0;
 _unlockBtreeIfUnused($38);
 STACKTOP = sp;return;
}
function _downgradeAllSharedCacheTableLocks($p) {
 $p = $p|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $pBt = 0, $pLock = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $pBt = sp + 4|0;
 $pLock = sp;
 HEAP32[$0>>2] = $p;
 $1 = HEAP32[$0>>2]|0;
 $2 = (($1) + 4|0);
 $3 = HEAP32[$2>>2]|0;
 HEAP32[$pBt>>2] = $3;
 $4 = HEAP32[$pBt>>2]|0;
 $5 = (($4) + 76|0);
 $6 = HEAP32[$5>>2]|0;
 $7 = HEAP32[$0>>2]|0;
 $8 = ($6|0)==($7|0);
 if (!($8)) {
  STACKTOP = sp;return;
 }
 $9 = HEAP32[$pBt>>2]|0;
 $10 = (($9) + 76|0);
 HEAP32[$10>>2] = 0;
 $11 = HEAP32[$pBt>>2]|0;
 $12 = (($11) + 22|0);
 $13 = HEAP16[$12>>1]|0;
 $14 = $13&65535;
 $15 = $14 & -97;
 $16 = $15&65535;
 HEAP16[$12>>1] = $16;
 $17 = HEAP32[$pBt>>2]|0;
 $18 = (($17) + 72|0);
 $19 = HEAP32[$18>>2]|0;
 HEAP32[$pLock>>2] = $19;
 while(1) {
  $20 = HEAP32[$pLock>>2]|0;
  $21 = ($20|0)!=(0|0);
  if (!($21)) {
   break;
  }
  $22 = HEAP32[$pLock>>2]|0;
  $23 = (($22) + 8|0);
  HEAP8[$23>>0] = 1;
  $24 = HEAP32[$pLock>>2]|0;
  $25 = (($24) + 12|0);
  $26 = HEAP32[$25>>2]|0;
  HEAP32[$pLock>>2] = $26;
 }
 STACKTOP = sp;return;
}
function _clearAllSharedCacheTableLocks($p) {
 $p = $p|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $pBt = 0, $pLock = 0, $ppIter = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $pBt = sp + 8|0;
 $ppIter = sp + 4|0;
 $pLock = sp;
 HEAP32[$0>>2] = $p;
 $1 = HEAP32[$0>>2]|0;
 $2 = (($1) + 4|0);
 $3 = HEAP32[$2>>2]|0;
 HEAP32[$pBt>>2] = $3;
 $4 = HEAP32[$pBt>>2]|0;
 $5 = (($4) + 72|0);
 HEAP32[$ppIter>>2] = $5;
 while(1) {
  $6 = HEAP32[$ppIter>>2]|0;
  $7 = HEAP32[$6>>2]|0;
  $8 = ($7|0)!=(0|0);
  if (!($8)) {
   break;
  }
  $9 = HEAP32[$ppIter>>2]|0;
  $10 = HEAP32[$9>>2]|0;
  HEAP32[$pLock>>2] = $10;
  $11 = HEAP32[$pLock>>2]|0;
  $12 = HEAP32[$11>>2]|0;
  $13 = HEAP32[$0>>2]|0;
  $14 = ($12|0)==($13|0);
  $15 = HEAP32[$pLock>>2]|0;
  $16 = (($15) + 12|0);
  if (!($14)) {
   HEAP32[$ppIter>>2] = $16;
   continue;
  }
  $17 = HEAP32[$16>>2]|0;
  $18 = HEAP32[$ppIter>>2]|0;
  HEAP32[$18>>2] = $17;
  $19 = HEAP32[$pLock>>2]|0;
  $20 = (($19) + 4|0);
  $21 = HEAP32[$20>>2]|0;
  $22 = ($21|0)!=(1);
  if (!($22)) {
   continue;
  }
  $23 = HEAP32[$pLock>>2]|0;
  _sqlite3_free($23);
 }
 $24 = HEAP32[$pBt>>2]|0;
 $25 = (($24) + 76|0);
 $26 = HEAP32[$25>>2]|0;
 $27 = HEAP32[$0>>2]|0;
 $28 = ($26|0)==($27|0);
 $29 = HEAP32[$pBt>>2]|0;
 if ($28) {
  $30 = (($29) + 76|0);
  HEAP32[$30>>2] = 0;
  $31 = HEAP32[$pBt>>2]|0;
  $32 = (($31) + 22|0);
  $33 = HEAP16[$32>>1]|0;
  $34 = $33&65535;
  $35 = $34 & -97;
  $36 = $35&65535;
  HEAP16[$32>>1] = $36;
  STACKTOP = sp;return;
 }
 $37 = (($29) + 40|0);
 $38 = HEAP32[$37>>2]|0;
 $39 = ($38|0)==(2);
 if (!($39)) {
  STACKTOP = sp;return;
 }
 $40 = HEAP32[$pBt>>2]|0;
 $41 = (($40) + 22|0);
 $42 = HEAP16[$41>>1]|0;
 $43 = $42&65535;
 $44 = $43 & -65;
 $45 = $44&65535;
 HEAP16[$41>>1] = $45;
 STACKTOP = sp;return;
}
function _autoVacuumCommit($pBt) {
 $pBt = $pBt|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0;
 var $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $iFree = 0, $nFin = 0;
 var $nFree = 0, $nOrig = 0, $or$cond = 0, $or$cond3 = 0, $or$cond5 = 0, $pPager = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 28|0;
 $1 = sp + 24|0;
 $rc = sp + 20|0;
 $pPager = sp + 16|0;
 $nFin = sp + 12|0;
 $nFree = sp + 8|0;
 $iFree = sp + 4|0;
 $nOrig = sp;
 HEAP32[$1>>2] = $pBt;
 HEAP32[$rc>>2] = 0;
 $2 = HEAP32[$1>>2]|0;
 $3 = HEAP32[$2>>2]|0;
 HEAP32[$pPager>>2] = $3;
 $4 = HEAP32[$1>>2]|0;
 _invalidateAllOverflowCache($4);
 $5 = HEAP32[$1>>2]|0;
 $6 = (($5) + 18|0);
 $7 = HEAP8[$6>>0]|0;
 $8 = ($7<<24>>24)!=(0);
 do {
  if (!($8)) {
   $9 = HEAP32[$1>>2]|0;
   $10 = (_btreePagecount($9)|0);
   HEAP32[$nOrig>>2] = $10;
   $11 = HEAP32[$1>>2]|0;
   $12 = HEAP32[$nOrig>>2]|0;
   $13 = (_ptrmapPageno($11,$12)|0);
   $14 = HEAP32[$nOrig>>2]|0;
   $15 = ($13|0)==($14|0);
   if (!($15)) {
    $16 = HEAP32[$nOrig>>2]|0;
    $17 = HEAP32[1128>>2]|0;
    $18 = HEAP32[$1>>2]|0;
    $19 = (($18) + 32|0);
    $20 = HEAP32[$19>>2]|0;
    $21 = (($17>>>0) / ($20>>>0))&-1;
    $22 = (($21) + 1)|0;
    $23 = ($16|0)==($22|0);
    if (!($23)) {
     $25 = HEAP32[$1>>2]|0;
     $26 = (($25) + 12|0);
     $27 = HEAP32[$26>>2]|0;
     $28 = (($27) + 56|0);
     $29 = HEAP32[$28>>2]|0;
     $30 = (($29) + 36|0);
     $31 = (_sqlite3Get4byte($30)|0);
     HEAP32[$nFree>>2] = $31;
     $32 = HEAP32[$1>>2]|0;
     $33 = HEAP32[$nOrig>>2]|0;
     $34 = HEAP32[$nFree>>2]|0;
     $35 = (_finalDbSize($32,$33,$34)|0);
     HEAP32[$nFin>>2] = $35;
     $36 = HEAP32[$nFin>>2]|0;
     $37 = HEAP32[$nOrig>>2]|0;
     $38 = ($36>>>0)>($37>>>0);
     if ($38) {
      $39 = (_sqlite3CorruptError(55475)|0);
      HEAP32[$0>>2] = $39;
      $97 = HEAP32[$0>>2]|0;
      STACKTOP = sp;return ($97|0);
     }
     $40 = HEAP32[$nFin>>2]|0;
     $41 = HEAP32[$nOrig>>2]|0;
     $42 = ($40>>>0)<($41>>>0);
     if ($42) {
      $43 = HEAP32[$1>>2]|0;
      $44 = (_saveAllCursors($43,0,0)|0);
      HEAP32[$rc>>2] = $44;
     }
     $45 = HEAP32[$nOrig>>2]|0;
     HEAP32[$iFree>>2] = $45;
     while(1) {
      $46 = HEAP32[$iFree>>2]|0;
      $47 = HEAP32[$nFin>>2]|0;
      $48 = ($46>>>0)>($47>>>0);
      $49 = HEAP32[$rc>>2]|0;
      $50 = ($49|0)==(0);
      $or$cond = $48 & $50;
      if (!($or$cond)) {
       break;
      }
      $51 = HEAP32[$1>>2]|0;
      $52 = HEAP32[$nFin>>2]|0;
      $53 = HEAP32[$iFree>>2]|0;
      $54 = (_incrVacuumStep($51,$52,$53,1)|0);
      HEAP32[$rc>>2] = $54;
      $55 = HEAP32[$iFree>>2]|0;
      $56 = (($55) + -1)|0;
      HEAP32[$iFree>>2] = $56;
     }
     $57 = HEAP32[$rc>>2]|0;
     $58 = ($57|0)==(101);
     $59 = HEAP32[$rc>>2]|0;
     $60 = ($59|0)==(0);
     $or$cond3 = $58 | $60;
     $61 = HEAP32[$nFree>>2]|0;
     $62 = ($61>>>0)>(0);
     $or$cond5 = $or$cond3 & $62;
     if ($or$cond5) {
      $63 = HEAP32[$1>>2]|0;
      $64 = (($63) + 12|0);
      $65 = HEAP32[$64>>2]|0;
      $66 = (($65) + 68|0);
      $67 = HEAP32[$66>>2]|0;
      $68 = (_sqlite3PagerWrite($67)|0);
      HEAP32[$rc>>2] = $68;
      $69 = HEAP32[$1>>2]|0;
      $70 = (($69) + 12|0);
      $71 = HEAP32[$70>>2]|0;
      $72 = (($71) + 56|0);
      $73 = HEAP32[$72>>2]|0;
      $74 = (($73) + 32|0);
      _sqlite3Put4byte($74,0);
      $75 = HEAP32[$1>>2]|0;
      $76 = (($75) + 12|0);
      $77 = HEAP32[$76>>2]|0;
      $78 = (($77) + 56|0);
      $79 = HEAP32[$78>>2]|0;
      $80 = (($79) + 36|0);
      _sqlite3Put4byte($80,0);
      $81 = HEAP32[$1>>2]|0;
      $82 = (($81) + 12|0);
      $83 = HEAP32[$82>>2]|0;
      $84 = (($83) + 56|0);
      $85 = HEAP32[$84>>2]|0;
      $86 = (($85) + 28|0);
      $87 = HEAP32[$nFin>>2]|0;
      _sqlite3Put4byte($86,$87);
      $88 = HEAP32[$1>>2]|0;
      $89 = (($88) + 19|0);
      HEAP8[$89>>0] = 1;
      $90 = HEAP32[$nFin>>2]|0;
      $91 = HEAP32[$1>>2]|0;
      $92 = (($91) + 44|0);
      HEAP32[$92>>2] = $90;
     }
     $93 = HEAP32[$rc>>2]|0;
     $94 = ($93|0)!=(0);
     if (!($94)) {
      break;
     }
     $95 = HEAP32[$pPager>>2]|0;
     (_sqlite3PagerRollback($95)|0);
     break;
    }
   }
   $24 = (_sqlite3CorruptError(55470)|0);
   HEAP32[$0>>2] = $24;
   $97 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($97|0);
  }
 } while(0);
 $96 = HEAP32[$rc>>2]|0;
 HEAP32[$0>>2] = $96;
 $97 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($97|0);
}
function _sqlite3PagerCommitPhaseTwo($pPager) {
 $pPager = $pPager|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $1 = sp + 4|0;
 $rc = sp;
 HEAP32[$1>>2] = $pPager;
 HEAP32[$rc>>2] = 0;
 $2 = HEAP32[$1>>2]|0;
 $3 = (($2) + 40|0);
 $4 = HEAP32[$3>>2]|0;
 $5 = ($4|0)!=(0);
 $6 = HEAP32[$1>>2]|0;
 if ($5) {
  $7 = (($6) + 40|0);
  $8 = HEAP32[$7>>2]|0;
  HEAP32[$0>>2] = $8;
  $34 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($34|0);
 }
 $9 = (($6) + 16|0);
 $10 = HEAP8[$9>>0]|0;
 $11 = $10&255;
 $12 = ($11|0)==(2);
 if ($12) {
  $13 = HEAP32[$1>>2]|0;
  $14 = (($13) + 4|0);
  $15 = HEAP8[$14>>0]|0;
  $16 = $15&255;
  $17 = ($16|0)!=(0);
  if ($17) {
   $18 = HEAP32[$1>>2]|0;
   $19 = (($18) + 5|0);
   $20 = HEAP8[$19>>0]|0;
   $21 = $20&255;
   $22 = ($21|0)==(1);
   if ($22) {
    $23 = HEAP32[$1>>2]|0;
    $24 = (($23) + 16|0);
    HEAP8[$24>>0] = 1;
    HEAP32[$0>>2] = 0;
    $34 = HEAP32[$0>>2]|0;
    STACKTOP = sp;return ($34|0);
   }
  }
 }
 $25 = HEAP32[$1>>2]|0;
 $26 = HEAP32[$1>>2]|0;
 $27 = (($26) + 19|0);
 $28 = HEAP8[$27>>0]|0;
 $29 = $28&255;
 $30 = (_pager_end_transaction($25,$29,1)|0);
 HEAP32[$rc>>2] = $30;
 $31 = HEAP32[$1>>2]|0;
 $32 = HEAP32[$rc>>2]|0;
 $33 = (_pager_error($31,$32)|0);
 HEAP32[$0>>2] = $33;
 $34 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($34|0);
}
function _pager_incr_changecounter($pPager,$isDirectMode) {
 $pPager = $pPager|0;
 $isDirectMode = $isDirectMode|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0;
 var $7 = 0, $8 = 0, $9 = 0, $pPgHdr = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $1 = sp + 8|0;
 $rc = sp + 4|0;
 $pPgHdr = sp;
 HEAP32[$0>>2] = $pPager;
 HEAP32[$1>>2] = $isDirectMode;
 HEAP32[$rc>>2] = 0;
 $2 = HEAP32[$0>>2]|0;
 $3 = (($2) + 18|0);
 $4 = HEAP8[$3>>0]|0;
 $5 = ($4<<24>>24)!=(0);
 if ($5) {
  $22 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($22|0);
 }
 $6 = HEAP32[$0>>2]|0;
 $7 = (($6) + 24|0);
 $8 = HEAP32[$7>>2]|0;
 $9 = ($8>>>0)>(0);
 if (!($9)) {
  $22 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($22|0);
 }
 $10 = HEAP32[$0>>2]|0;
 $11 = (_sqlite3PagerAcquire($10,1,$pPgHdr,0)|0);
 HEAP32[$rc>>2] = $11;
 $12 = HEAP32[$rc>>2]|0;
 $13 = ($12|0)==(0);
 if ($13) {
  $14 = HEAP32[$pPgHdr>>2]|0;
  $15 = (_sqlite3PagerWrite($14)|0);
  HEAP32[$rc>>2] = $15;
 }
 $16 = HEAP32[$rc>>2]|0;
 $17 = ($16|0)==(0);
 if ($17) {
  $18 = HEAP32[$pPgHdr>>2]|0;
  _pager_write_changecounter($18);
  $19 = HEAP32[$0>>2]|0;
  $20 = (($19) + 18|0);
  HEAP8[$20>>0] = 1;
 }
 $21 = HEAP32[$pPgHdr>>2]|0;
 _sqlite3PagerUnref($21);
 $22 = HEAP32[$rc>>2]|0;
 STACKTOP = sp;return ($22|0);
}
function _writeMasterJournal($pPager,$zMaster) {
 $pPager = $pPager|0;
 $zMaster = $zMaster|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0;
 var $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0, $188 = 0;
 var $189 = 0, $19 = 0, $190 = 0, $191 = 0, $192 = 0, $193 = 0, $194 = 0, $195 = 0, $196 = 0, $197 = 0, $198 = 0, $199 = 0, $2 = 0, $20 = 0, $200 = 0, $201 = 0, $202 = 0, $203 = 0, $204 = 0, $21 = 0;
 var $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0;
 var $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0;
 var $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0;
 var $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0;
 var $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $cksum = 0, $iHdrOff = 0, $jrnlSize = 0, $nMaster = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $0 = sp + 36|0;
 $1 = sp + 32|0;
 $2 = sp + 28|0;
 $rc = sp + 24|0;
 $nMaster = sp + 20|0;
 $iHdrOff = sp + 8|0;
 $jrnlSize = sp;
 $cksum = sp + 16|0;
 HEAP32[$1>>2] = $pPager;
 HEAP32[$2>>2] = $zMaster;
 HEAP32[$cksum>>2] = 0;
 $3 = HEAP32[$2>>2]|0;
 $4 = ($3|0)!=(0|0);
 if ($4) {
  $5 = HEAP32[$1>>2]|0;
  $6 = (($5) + 5|0);
  $7 = HEAP8[$6>>0]|0;
  $8 = $7&255;
  $9 = ($8|0)==(4);
  if (!($9)) {
   $10 = HEAP32[$1>>2]|0;
   $11 = (($10) + 64|0);
   $12 = HEAP32[$11>>2]|0;
   $13 = HEAP32[$12>>2]|0;
   $14 = ($13|0)!=(0|0);
   if ($14) {
    $15 = HEAP32[$1>>2]|0;
    $16 = (($15) + 19|0);
    HEAP8[$16>>0] = 1;
    HEAP32[$nMaster>>2] = 0;
    while(1) {
     $17 = HEAP32[$nMaster>>2]|0;
     $18 = HEAP32[$2>>2]|0;
     $19 = (($18) + ($17)|0);
     $20 = HEAP8[$19>>0]|0;
     $21 = ($20<<24>>24)!=(0);
     if (!($21)) {
      break;
     }
     $22 = HEAP32[$nMaster>>2]|0;
     $23 = HEAP32[$2>>2]|0;
     $24 = (($23) + ($22)|0);
     $25 = HEAP8[$24>>0]|0;
     $26 = $25 << 24 >> 24;
     $27 = HEAP32[$cksum>>2]|0;
     $28 = (($27) + ($26))|0;
     HEAP32[$cksum>>2] = $28;
     $29 = HEAP32[$nMaster>>2]|0;
     $30 = (($29) + 1)|0;
     HEAP32[$nMaster>>2] = $30;
    }
    $31 = HEAP32[$1>>2]|0;
    $32 = (($31) + 8|0);
    $33 = HEAP8[$32>>0]|0;
    $34 = ($33<<24>>24)!=(0);
    if ($34) {
     $35 = HEAP32[$1>>2]|0;
     $36 = (_journalHdrOffset($35)|0);
     $37 = tempRet0;
     $38 = HEAP32[$1>>2]|0;
     $39 = (($38) + 72|0);
     $40 = $39;
     $41 = $40;
     HEAP32[$41>>2] = $36;
     $42 = (($40) + 4)|0;
     $43 = $42;
     HEAP32[$43>>2] = $37;
    }
    $44 = HEAP32[$1>>2]|0;
    $45 = (($44) + 72|0);
    $46 = $45;
    $47 = $46;
    $48 = HEAP32[$47>>2]|0;
    $49 = (($46) + 4)|0;
    $50 = $49;
    $51 = HEAP32[$50>>2]|0;
    $52 = $iHdrOff;
    $53 = $52;
    HEAP32[$53>>2] = $48;
    $54 = (($52) + 4)|0;
    $55 = $54;
    HEAP32[$55>>2] = $51;
    $56 = HEAP32[$1>>2]|0;
    $57 = (($56) + 64|0);
    $58 = HEAP32[$57>>2]|0;
    $59 = $iHdrOff;
    $60 = $59;
    $61 = HEAP32[$60>>2]|0;
    $62 = (($59) + 4)|0;
    $63 = $62;
    $64 = HEAP32[$63>>2]|0;
    $65 = HEAP32[1128>>2]|0;
    $66 = HEAP32[$1>>2]|0;
    $67 = (($66) + 152|0);
    $68 = HEAP32[$67>>2]|0;
    $69 = (($65|0) / ($68|0))&-1;
    $70 = (($69) + 1)|0;
    $71 = (_write32bits($58,$61,$64,$70)|0);
    HEAP32[$rc>>2] = $71;
    $72 = (0)!=($71|0);
    if (!($72)) {
     $73 = HEAP32[$1>>2]|0;
     $74 = (($73) + 64|0);
     $75 = HEAP32[$74>>2]|0;
     $76 = HEAP32[$2>>2]|0;
     $77 = HEAP32[$nMaster>>2]|0;
     $78 = $iHdrOff;
     $79 = $78;
     $80 = HEAP32[$79>>2]|0;
     $81 = (($78) + 4)|0;
     $82 = $81;
     $83 = HEAP32[$82>>2]|0;
     $84 = (_i64Add(($80|0),($83|0),4,0)|0);
     $85 = tempRet0;
     $86 = (_sqlite3OsWrite($75,$76,$77,$84,$85)|0);
     HEAP32[$rc>>2] = $86;
     $87 = (0)!=($86|0);
     if (!($87)) {
      $88 = HEAP32[$1>>2]|0;
      $89 = (($88) + 64|0);
      $90 = HEAP32[$89>>2]|0;
      $91 = $iHdrOff;
      $92 = $91;
      $93 = HEAP32[$92>>2]|0;
      $94 = (($91) + 4)|0;
      $95 = $94;
      $96 = HEAP32[$95>>2]|0;
      $97 = (_i64Add(($93|0),($96|0),4,0)|0);
      $98 = tempRet0;
      $99 = HEAP32[$nMaster>>2]|0;
      $100 = ($99|0)<(0);
      $101 = $100 << 31 >> 31;
      $102 = (_i64Add(($97|0),($98|0),($99|0),($101|0))|0);
      $103 = tempRet0;
      $104 = HEAP32[$nMaster>>2]|0;
      $105 = (_write32bits($90,$102,$103,$104)|0);
      HEAP32[$rc>>2] = $105;
      $106 = (0)!=($105|0);
      if (!($106)) {
       $107 = HEAP32[$1>>2]|0;
       $108 = (($107) + 64|0);
       $109 = HEAP32[$108>>2]|0;
       $110 = $iHdrOff;
       $111 = $110;
       $112 = HEAP32[$111>>2]|0;
       $113 = (($110) + 4)|0;
       $114 = $113;
       $115 = HEAP32[$114>>2]|0;
       $116 = (_i64Add(($112|0),($115|0),4,0)|0);
       $117 = tempRet0;
       $118 = HEAP32[$nMaster>>2]|0;
       $119 = ($118|0)<(0);
       $120 = $119 << 31 >> 31;
       $121 = (_i64Add(($116|0),($117|0),($118|0),($120|0))|0);
       $122 = tempRet0;
       $123 = (_i64Add(($121|0),($122|0),4,0)|0);
       $124 = tempRet0;
       $125 = HEAP32[$cksum>>2]|0;
       $126 = (_write32bits($109,$123,$124,$125)|0);
       HEAP32[$rc>>2] = $126;
       $127 = (0)!=($126|0);
       if (!($127)) {
        $128 = HEAP32[$1>>2]|0;
        $129 = (($128) + 64|0);
        $130 = HEAP32[$129>>2]|0;
        $131 = $iHdrOff;
        $132 = $131;
        $133 = HEAP32[$132>>2]|0;
        $134 = (($131) + 4)|0;
        $135 = $134;
        $136 = HEAP32[$135>>2]|0;
        $137 = (_i64Add(($133|0),($136|0),4,0)|0);
        $138 = tempRet0;
        $139 = HEAP32[$nMaster>>2]|0;
        $140 = ($139|0)<(0);
        $141 = $140 << 31 >> 31;
        $142 = (_i64Add(($137|0),($138|0),($139|0),($141|0))|0);
        $143 = tempRet0;
        $144 = (_i64Add(($142|0),($143|0),8,0)|0);
        $145 = tempRet0;
        $146 = (_sqlite3OsWrite($130,4384,8,$144,$145)|0);
        HEAP32[$rc>>2] = $146;
        $147 = (0)!=($146|0);
        if (!($147)) {
         $149 = HEAP32[$nMaster>>2]|0;
         $150 = (($149) + 20)|0;
         $151 = ($150|0)<(0);
         $152 = $151 << 31 >> 31;
         $153 = HEAP32[$1>>2]|0;
         $154 = (($153) + 72|0);
         $155 = $154;
         $156 = $155;
         $157 = HEAP32[$156>>2]|0;
         $158 = (($155) + 4)|0;
         $159 = $158;
         $160 = HEAP32[$159>>2]|0;
         $161 = (_i64Add(($157|0),($160|0),($150|0),($152|0))|0);
         $162 = tempRet0;
         $163 = $154;
         $164 = $163;
         HEAP32[$164>>2] = $161;
         $165 = (($163) + 4)|0;
         $166 = $165;
         HEAP32[$166>>2] = $162;
         $167 = HEAP32[$1>>2]|0;
         $168 = (($167) + 64|0);
         $169 = HEAP32[$168>>2]|0;
         $170 = (_sqlite3OsFileSize($169,$jrnlSize)|0);
         HEAP32[$rc>>2] = $170;
         $171 = (0)==($170|0);
         if ($171) {
          $172 = $jrnlSize;
          $173 = $172;
          $174 = HEAP32[$173>>2]|0;
          $175 = (($172) + 4)|0;
          $176 = $175;
          $177 = HEAP32[$176>>2]|0;
          $178 = HEAP32[$1>>2]|0;
          $179 = (($178) + 72|0);
          $180 = $179;
          $181 = $180;
          $182 = HEAP32[$181>>2]|0;
          $183 = (($180) + 4)|0;
          $184 = $183;
          $185 = HEAP32[$184>>2]|0;
          $186 = ($177|0)>($185|0);
          $187 = ($174>>>0)>($182>>>0);
          $188 = ($177|0)==($185|0);
          $189 = $188 & $187;
          $190 = $186 | $189;
          if ($190) {
           $191 = HEAP32[$1>>2]|0;
           $192 = (($191) + 64|0);
           $193 = HEAP32[$192>>2]|0;
           $194 = HEAP32[$1>>2]|0;
           $195 = (($194) + 72|0);
           $196 = $195;
           $197 = $196;
           $198 = HEAP32[$197>>2]|0;
           $199 = (($196) + 4)|0;
           $200 = $199;
           $201 = HEAP32[$200>>2]|0;
           $202 = (_sqlite3OsTruncate($193,$198,$201)|0);
           HEAP32[$rc>>2] = $202;
          }
         }
         $203 = HEAP32[$rc>>2]|0;
         HEAP32[$0>>2] = $203;
         $204 = HEAP32[$0>>2]|0;
         STACKTOP = sp;return ($204|0);
        }
       }
      }
     }
    }
    $148 = HEAP32[$rc>>2]|0;
    HEAP32[$0>>2] = $148;
    $204 = HEAP32[$0>>2]|0;
    STACKTOP = sp;return ($204|0);
   }
  }
 }
 HEAP32[$0>>2] = 0;
 $204 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($204|0);
}
function _pagerWriteLargeSector($pPg) {
 $pPg = $pPg|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0;
 var $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0;
 var $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0;
 var $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0;
 var $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $ii = 0, $nPage = 0, $nPageCount = 0, $nPagePerSector = 0, $needSync = 0;
 var $or$cond = 0, $or$cond3 = 0, $pPage = 0, $pPage1 = 0, $pPager = 0, $pg = 0, $pg1 = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $0 = sp + 44|0;
 $rc = sp + 40|0;
 $nPageCount = sp + 36|0;
 $pg1 = sp + 32|0;
 $nPage = sp + 28|0;
 $ii = sp + 24|0;
 $needSync = sp + 20|0;
 $pPager = sp + 16|0;
 $nPagePerSector = sp + 12|0;
 $pg = sp + 8|0;
 $pPage = sp + 4|0;
 $pPage1 = sp;
 HEAP32[$0>>2] = $pPg;
 HEAP32[$rc>>2] = 0;
 HEAP32[$nPage>>2] = 0;
 HEAP32[$needSync>>2] = 0;
 $1 = HEAP32[$0>>2]|0;
 $2 = (($1) + 16|0);
 $3 = HEAP32[$2>>2]|0;
 HEAP32[$pPager>>2] = $3;
 $4 = HEAP32[$pPager>>2]|0;
 $5 = (($4) + 148|0);
 $6 = HEAP32[$5>>2]|0;
 $7 = HEAP32[$pPager>>2]|0;
 $8 = (($7) + 152|0);
 $9 = HEAP32[$8>>2]|0;
 $10 = (($6>>>0) / ($9>>>0))&-1;
 HEAP32[$nPagePerSector>>2] = $10;
 $11 = HEAP32[$pPager>>2]|0;
 $12 = (($11) + 20|0);
 $13 = HEAP8[$12>>0]|0;
 $14 = $13&255;
 $15 = $14 | 4;
 $16 = $15&255;
 HEAP8[$12>>0] = $16;
 $17 = HEAP32[$0>>2]|0;
 $18 = (($17) + 20|0);
 $19 = HEAP32[$18>>2]|0;
 $20 = (($19) - 1)|0;
 $21 = HEAP32[$nPagePerSector>>2]|0;
 $22 = (($21) - 1)|0;
 $23 = $22 ^ -1;
 $24 = $20 & $23;
 $25 = (($24) + 1)|0;
 HEAP32[$pg1>>2] = $25;
 $26 = HEAP32[$pPager>>2]|0;
 $27 = (($26) + 24|0);
 $28 = HEAP32[$27>>2]|0;
 HEAP32[$nPageCount>>2] = $28;
 $29 = HEAP32[$0>>2]|0;
 $30 = (($29) + 20|0);
 $31 = HEAP32[$30>>2]|0;
 $32 = HEAP32[$nPageCount>>2]|0;
 $33 = ($31>>>0)>($32>>>0);
 do {
  if ($33) {
   $34 = HEAP32[$0>>2]|0;
   $35 = (($34) + 20|0);
   $36 = HEAP32[$35>>2]|0;
   $37 = HEAP32[$pg1>>2]|0;
   $38 = (($36) - ($37))|0;
   $39 = (($38) + 1)|0;
   HEAP32[$nPage>>2] = $39;
  } else {
   $40 = HEAP32[$pg1>>2]|0;
   $41 = HEAP32[$nPagePerSector>>2]|0;
   $42 = (($40) + ($41))|0;
   $43 = (($42) - 1)|0;
   $44 = HEAP32[$nPageCount>>2]|0;
   $45 = ($43>>>0)>($44>>>0);
   if ($45) {
    $46 = HEAP32[$nPageCount>>2]|0;
    $47 = (($46) + 1)|0;
    $48 = HEAP32[$pg1>>2]|0;
    $49 = (($47) - ($48))|0;
    HEAP32[$nPage>>2] = $49;
    break;
   } else {
    $50 = HEAP32[$nPagePerSector>>2]|0;
    HEAP32[$nPage>>2] = $50;
    break;
   }
  }
 } while(0);
 HEAP32[$ii>>2] = 0;
 while(1) {
  $51 = HEAP32[$ii>>2]|0;
  $52 = HEAP32[$nPage>>2]|0;
  $53 = ($51|0)<($52|0);
  $54 = HEAP32[$rc>>2]|0;
  $55 = ($54|0)==(0);
  $or$cond = $53 & $55;
  if (!($or$cond)) {
   break;
  }
  $56 = HEAP32[$pg1>>2]|0;
  $57 = HEAP32[$ii>>2]|0;
  $58 = (($56) + ($57))|0;
  HEAP32[$pg>>2] = $58;
  $59 = HEAP32[$pg>>2]|0;
  $60 = HEAP32[$0>>2]|0;
  $61 = (($60) + 20|0);
  $62 = HEAP32[$61>>2]|0;
  $63 = ($59|0)==($62|0);
  if ($63) {
   label = 10;
  } else {
   $64 = HEAP32[$pPager>>2]|0;
   $65 = (($64) + 56|0);
   $66 = HEAP32[$65>>2]|0;
   $67 = HEAP32[$pg>>2]|0;
   $68 = (_sqlite3BitvecTest($66,$67)|0);
   $69 = ($68|0)!=(0);
   if ($69) {
    $92 = HEAP32[$pPager>>2]|0;
    $93 = HEAP32[$pg>>2]|0;
    $94 = (_sqlite3PagerLookup($92,$93)|0);
    HEAP32[$pPage>>2] = $94;
    $95 = ($94|0)!=(0|0);
    if ($95) {
     $96 = HEAP32[$pPage>>2]|0;
     $97 = (($96) + 24|0);
     $98 = HEAP16[$97>>1]|0;
     $99 = $98&65535;
     $100 = $99 & 4;
     $101 = ($100|0)!=(0);
     if ($101) {
      HEAP32[$needSync>>2] = 1;
     }
     $102 = HEAP32[$pPage>>2]|0;
     _sqlite3PagerUnrefNotNull($102);
    }
   } else {
    label = 10;
   }
  }
  if ((label|0) == 10) {
   label = 0;
   $70 = HEAP32[$pg>>2]|0;
   $71 = HEAP32[1128>>2]|0;
   $72 = HEAP32[$pPager>>2]|0;
   $73 = (($72) + 152|0);
   $74 = HEAP32[$73>>2]|0;
   $75 = (($71|0) / ($74|0))&-1;
   $76 = (($75) + 1)|0;
   $77 = ($70|0)!=($76|0);
   if ($77) {
    $78 = HEAP32[$pPager>>2]|0;
    $79 = HEAP32[$pg>>2]|0;
    $80 = (_sqlite3PagerAcquire($78,$79,$pPage,0)|0);
    HEAP32[$rc>>2] = $80;
    $81 = HEAP32[$rc>>2]|0;
    $82 = ($81|0)==(0);
    if ($82) {
     $83 = HEAP32[$pPage>>2]|0;
     $84 = (_pager_write($83)|0);
     HEAP32[$rc>>2] = $84;
     $85 = HEAP32[$pPage>>2]|0;
     $86 = (($85) + 24|0);
     $87 = HEAP16[$86>>1]|0;
     $88 = $87&65535;
     $89 = $88 & 4;
     $90 = ($89|0)!=(0);
     if ($90) {
      HEAP32[$needSync>>2] = 1;
     }
     $91 = HEAP32[$pPage>>2]|0;
     _sqlite3PagerUnrefNotNull($91);
    }
   }
  }
  $103 = HEAP32[$ii>>2]|0;
  $104 = (($103) + 1)|0;
  HEAP32[$ii>>2] = $104;
 }
 $105 = HEAP32[$rc>>2]|0;
 $106 = ($105|0)==(0);
 $107 = HEAP32[$needSync>>2]|0;
 $108 = ($107|0)!=(0);
 $or$cond3 = $106 & $108;
 if (!($or$cond3)) {
  $128 = HEAP32[$pPager>>2]|0;
  $129 = (($128) + 20|0);
  $130 = HEAP8[$129>>0]|0;
  $131 = $130&255;
  $132 = $131 & -5;
  $133 = $132&255;
  HEAP8[$129>>0] = $133;
  $134 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($134|0);
 }
 HEAP32[$ii>>2] = 0;
 while(1) {
  $109 = HEAP32[$ii>>2]|0;
  $110 = HEAP32[$nPage>>2]|0;
  $111 = ($109|0)<($110|0);
  if (!($111)) {
   break;
  }
  $112 = HEAP32[$pPager>>2]|0;
  $113 = HEAP32[$pg1>>2]|0;
  $114 = HEAP32[$ii>>2]|0;
  $115 = (($113) + ($114))|0;
  $116 = (_sqlite3PagerLookup($112,$115)|0);
  HEAP32[$pPage1>>2] = $116;
  $117 = HEAP32[$pPage1>>2]|0;
  $118 = ($117|0)!=(0|0);
  if ($118) {
   $119 = HEAP32[$pPage1>>2]|0;
   $120 = (($119) + 24|0);
   $121 = HEAP16[$120>>1]|0;
   $122 = $121&65535;
   $123 = $122 | 4;
   $124 = $123&65535;
   HEAP16[$120>>1] = $124;
   $125 = HEAP32[$pPage1>>2]|0;
   _sqlite3PagerUnrefNotNull($125);
  }
  $126 = HEAP32[$ii>>2]|0;
  $127 = (($126) + 1)|0;
  HEAP32[$ii>>2] = $127;
 }
 $128 = HEAP32[$pPager>>2]|0;
 $129 = (($128) + 20|0);
 $130 = HEAP8[$129>>0]|0;
 $131 = $130&255;
 $132 = $131 & -5;
 $133 = $132&255;
 HEAP8[$129>>0] = $133;
 $134 = HEAP32[$rc>>2]|0;
 STACKTOP = sp;return ($134|0);
}
function _pager_write($pPg) {
 $pPg = $pPg|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0;
 var $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0, $188 = 0;
 var $189 = 0, $19 = 0, $190 = 0, $191 = 0, $192 = 0, $193 = 0, $194 = 0, $195 = 0, $196 = 0, $197 = 0, $198 = 0, $199 = 0, $2 = 0, $20 = 0, $200 = 0, $201 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0;
 var $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0;
 var $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0;
 var $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0;
 var $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0;
 var $98 = 0, $99 = 0, $cksum = 0, $iOff = 0, $inJournal = 0, $pData2 = 0, $pPager = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $0 = sp + 32|0;
 $1 = sp + 28|0;
 $pPager = sp + 24|0;
 $rc = sp + 20|0;
 $inJournal = sp + 16|0;
 $cksum = sp + 12|0;
 $pData2 = sp + 8|0;
 $iOff = sp;
 HEAP32[$1>>2] = $pPg;
 $2 = HEAP32[$1>>2]|0;
 $3 = (($2) + 16|0);
 $4 = HEAP32[$3>>2]|0;
 HEAP32[$pPager>>2] = $4;
 HEAP32[$rc>>2] = 0;
 $5 = HEAP32[$pPager>>2]|0;
 $6 = (($5) + 16|0);
 $7 = HEAP8[$6>>0]|0;
 $8 = $7&255;
 $9 = ($8|0)==(2);
 if ($9) {
  $10 = HEAP32[$pPager>>2]|0;
  $11 = (_pager_open_journal($10)|0);
  HEAP32[$rc>>2] = $11;
  $12 = HEAP32[$rc>>2]|0;
  $13 = ($12|0)!=(0);
  if ($13) {
   $14 = HEAP32[$rc>>2]|0;
   HEAP32[$0>>2] = $14;
   $201 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($201|0);
  }
 }
 $15 = HEAP32[$1>>2]|0;
 _sqlite3PcacheMakeDirty($15);
 $16 = HEAP32[$pPager>>2]|0;
 $17 = HEAP32[$1>>2]|0;
 $18 = (_pageInJournal($16,$17)|0);
 HEAP32[$inJournal>>2] = $18;
 $19 = HEAP32[$inJournal>>2]|0;
 $20 = ($19|0)!=(0);
 if ($20) {
  $21 = HEAP32[$pPager>>2]|0;
  $22 = (($21) + 96|0);
  $23 = HEAP32[$22>>2]|0;
  $24 = ($23|0)==(0);
  if (!($24)) {
   $25 = HEAP32[$1>>2]|0;
   $26 = (_subjRequiresPage($25)|0);
   $27 = ($26|0)!=(0);
   if ($27) {
    label = 7;
   }
  }
 } else {
  label = 7;
 }
 if ((label|0) == 7) {
  $28 = HEAP32[$inJournal>>2]|0;
  $29 = ($28|0)!=(0);
  do {
   if (!($29)) {
    $30 = HEAP32[$pPager>>2]|0;
    $31 = (_pagerUseWal($30)|0);
    $32 = ($31|0)!=(0);
    if (!($32)) {
     $33 = HEAP32[$1>>2]|0;
     $34 = (($33) + 20|0);
     $35 = HEAP32[$34>>2]|0;
     $36 = HEAP32[$pPager>>2]|0;
     $37 = (($36) + 28|0);
     $38 = HEAP32[$37>>2]|0;
     $39 = ($35>>>0)<=($38>>>0);
     if ($39) {
      $40 = HEAP32[$pPager>>2]|0;
      $41 = (($40) + 64|0);
      $42 = HEAP32[$41>>2]|0;
      $43 = HEAP32[$42>>2]|0;
      $44 = ($43|0)!=(0|0);
      if ($44) {
       $45 = HEAP32[$pPager>>2]|0;
       $46 = (($45) + 72|0);
       $47 = $46;
       $48 = $47;
       $49 = HEAP32[$48>>2]|0;
       $50 = (($47) + 4)|0;
       $51 = $50;
       $52 = HEAP32[$51>>2]|0;
       $53 = $iOff;
       $54 = $53;
       HEAP32[$54>>2] = $49;
       $55 = (($53) + 4)|0;
       $56 = $55;
       HEAP32[$56>>2] = $52;
       $57 = HEAP32[$1>>2]|0;
       $58 = (($57) + 4|0);
       $59 = HEAP32[$58>>2]|0;
       HEAP32[$pData2>>2] = $59;
       $60 = HEAP32[$pPager>>2]|0;
       $61 = HEAP32[$pData2>>2]|0;
       $62 = (_pager_cksum($60,$61)|0);
       HEAP32[$cksum>>2] = $62;
       $63 = HEAP32[$1>>2]|0;
       $64 = (($63) + 24|0);
       $65 = HEAP16[$64>>1]|0;
       $66 = $65&65535;
       $67 = $66 | 4;
       $68 = $67&65535;
       HEAP16[$64>>1] = $68;
       $69 = HEAP32[$pPager>>2]|0;
       $70 = (($69) + 64|0);
       $71 = HEAP32[$70>>2]|0;
       $72 = $iOff;
       $73 = $72;
       $74 = HEAP32[$73>>2]|0;
       $75 = (($72) + 4)|0;
       $76 = $75;
       $77 = HEAP32[$76>>2]|0;
       $78 = HEAP32[$1>>2]|0;
       $79 = (($78) + 20|0);
       $80 = HEAP32[$79>>2]|0;
       $81 = (_write32bits($71,$74,$77,$80)|0);
       HEAP32[$rc>>2] = $81;
       $82 = HEAP32[$rc>>2]|0;
       $83 = ($82|0)!=(0);
       if ($83) {
        $84 = HEAP32[$rc>>2]|0;
        HEAP32[$0>>2] = $84;
        $201 = HEAP32[$0>>2]|0;
        STACKTOP = sp;return ($201|0);
       }
       $85 = HEAP32[$pPager>>2]|0;
       $86 = (($85) + 64|0);
       $87 = HEAP32[$86>>2]|0;
       $88 = HEAP32[$pData2>>2]|0;
       $89 = HEAP32[$pPager>>2]|0;
       $90 = (($89) + 152|0);
       $91 = HEAP32[$90>>2]|0;
       $92 = $iOff;
       $93 = $92;
       $94 = HEAP32[$93>>2]|0;
       $95 = (($92) + 4)|0;
       $96 = $95;
       $97 = HEAP32[$96>>2]|0;
       $98 = (_i64Add(($94|0),($97|0),4,0)|0);
       $99 = tempRet0;
       $100 = (_sqlite3OsWrite($87,$88,$91,$98,$99)|0);
       HEAP32[$rc>>2] = $100;
       $101 = HEAP32[$rc>>2]|0;
       $102 = ($101|0)!=(0);
       if ($102) {
        $103 = HEAP32[$rc>>2]|0;
        HEAP32[$0>>2] = $103;
        $201 = HEAP32[$0>>2]|0;
        STACKTOP = sp;return ($201|0);
       }
       $104 = HEAP32[$pPager>>2]|0;
       $105 = (($104) + 64|0);
       $106 = HEAP32[$105>>2]|0;
       $107 = $iOff;
       $108 = $107;
       $109 = HEAP32[$108>>2]|0;
       $110 = (($107) + 4)|0;
       $111 = $110;
       $112 = HEAP32[$111>>2]|0;
       $113 = HEAP32[$pPager>>2]|0;
       $114 = (($113) + 152|0);
       $115 = HEAP32[$114>>2]|0;
       $116 = ($115|0)<(0);
       $117 = $116 << 31 >> 31;
       $118 = (_i64Add(($109|0),($112|0),($115|0),($117|0))|0);
       $119 = tempRet0;
       $120 = (_i64Add(($118|0),($119|0),4,0)|0);
       $121 = tempRet0;
       $122 = HEAP32[$cksum>>2]|0;
       $123 = (_write32bits($106,$120,$121,$122)|0);
       HEAP32[$rc>>2] = $123;
       $124 = HEAP32[$rc>>2]|0;
       $125 = ($124|0)!=(0);
       if ($125) {
        $126 = HEAP32[$rc>>2]|0;
        HEAP32[$0>>2] = $126;
        $201 = HEAP32[$0>>2]|0;
        STACKTOP = sp;return ($201|0);
       }
       $127 = HEAP32[$pPager>>2]|0;
       $128 = (($127) + 152|0);
       $129 = HEAP32[$128>>2]|0;
       $130 = (8 + ($129))|0;
       $131 = ($130|0)<(0);
       $132 = $131 << 31 >> 31;
       $133 = HEAP32[$pPager>>2]|0;
       $134 = (($133) + 72|0);
       $135 = $134;
       $136 = $135;
       $137 = HEAP32[$136>>2]|0;
       $138 = (($135) + 4)|0;
       $139 = $138;
       $140 = HEAP32[$139>>2]|0;
       $141 = (_i64Add(($137|0),($140|0),($130|0),($132|0))|0);
       $142 = tempRet0;
       $143 = $134;
       $144 = $143;
       HEAP32[$144>>2] = $141;
       $145 = (($143) + 4)|0;
       $146 = $145;
       HEAP32[$146>>2] = $142;
       $147 = HEAP32[$pPager>>2]|0;
       $148 = (($147) + 44|0);
       $149 = HEAP32[$148>>2]|0;
       $150 = (($149) + 1)|0;
       HEAP32[$148>>2] = $150;
       $151 = HEAP32[$pPager>>2]|0;
       $152 = (($151) + 56|0);
       $153 = HEAP32[$152>>2]|0;
       $154 = HEAP32[$1>>2]|0;
       $155 = (($154) + 20|0);
       $156 = HEAP32[$155>>2]|0;
       $157 = (_sqlite3BitvecSet($153,$156)|0);
       HEAP32[$rc>>2] = $157;
       $158 = HEAP32[$pPager>>2]|0;
       $159 = HEAP32[$1>>2]|0;
       $160 = (($159) + 20|0);
       $161 = HEAP32[$160>>2]|0;
       $162 = (_addToSavepointBitvecs($158,$161)|0);
       $163 = HEAP32[$rc>>2]|0;
       $164 = $163 | $162;
       HEAP32[$rc>>2] = $164;
       $165 = HEAP32[$rc>>2]|0;
       $166 = ($165|0)!=(0);
       if (!($166)) {
        break;
       }
       $167 = HEAP32[$rc>>2]|0;
       HEAP32[$0>>2] = $167;
       $201 = HEAP32[$0>>2]|0;
       STACKTOP = sp;return ($201|0);
      }
     }
     $168 = HEAP32[$pPager>>2]|0;
     $169 = (($168) + 16|0);
     $170 = HEAP8[$169>>0]|0;
     $171 = $170&255;
     $172 = ($171|0)!=(4);
     if ($172) {
      $173 = HEAP32[$1>>2]|0;
      $174 = (($173) + 24|0);
      $175 = HEAP16[$174>>1]|0;
      $176 = $175&65535;
      $177 = $176 | 4;
      $178 = $177&65535;
      HEAP16[$174>>1] = $178;
     }
    }
   }
  } while(0);
  $179 = HEAP32[$pPager>>2]|0;
  $180 = (($179) + 96|0);
  $181 = HEAP32[$180>>2]|0;
  $182 = ($181|0)>(0);
  if ($182) {
   $183 = HEAP32[$1>>2]|0;
   $184 = (_subjRequiresPage($183)|0);
   $185 = ($184|0)!=(0);
   if ($185) {
    $186 = HEAP32[$1>>2]|0;
    $187 = (_subjournalPage($186)|0);
    HEAP32[$rc>>2] = $187;
   }
  }
 }
 $188 = HEAP32[$pPager>>2]|0;
 $189 = (($188) + 24|0);
 $190 = HEAP32[$189>>2]|0;
 $191 = HEAP32[$1>>2]|0;
 $192 = (($191) + 20|0);
 $193 = HEAP32[$192>>2]|0;
 $194 = ($190>>>0)<($193>>>0);
 if ($194) {
  $195 = HEAP32[$1>>2]|0;
  $196 = (($195) + 20|0);
  $197 = HEAP32[$196>>2]|0;
  $198 = HEAP32[$pPager>>2]|0;
  $199 = (($198) + 24|0);
  HEAP32[$199>>2] = $197;
 }
 $200 = HEAP32[$rc>>2]|0;
 HEAP32[$0>>2] = $200;
 $201 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($201|0);
}
function _pager_open_journal($pPager) {
 $pPager = $pPager|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0;
 var $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $flags = 0, $pVfs = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 16|0;
 $1 = sp + 12|0;
 $rc = sp + 8|0;
 $pVfs = sp + 4|0;
 $flags = sp;
 HEAP32[$1>>2] = $pPager;
 HEAP32[$rc>>2] = 0;
 $2 = HEAP32[$1>>2]|0;
 $3 = HEAP32[$2>>2]|0;
 HEAP32[$pVfs>>2] = $3;
 $4 = HEAP32[$1>>2]|0;
 $5 = (($4) + 40|0);
 $6 = HEAP32[$5>>2]|0;
 $7 = ($6|0)!=(0);
 $8 = HEAP32[$1>>2]|0;
 if ($7) {
  $9 = (($8) + 40|0);
  $10 = HEAP32[$9>>2]|0;
  HEAP32[$0>>2] = $10;
  $89 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($89|0);
 }
 $11 = (_pagerUseWal($8)|0);
 $12 = ($11|0)!=(0);
 if (!($12)) {
  $13 = HEAP32[$1>>2]|0;
  $14 = (($13) + 5|0);
  $15 = HEAP8[$14>>0]|0;
  $16 = $15&255;
  $17 = ($16|0)!=(2);
  if ($17) {
   $18 = HEAP32[$1>>2]|0;
   $19 = (($18) + 24|0);
   $20 = HEAP32[$19>>2]|0;
   $21 = (_sqlite3BitvecCreate($20)|0);
   $22 = HEAP32[$1>>2]|0;
   $23 = (($22) + 56|0);
   HEAP32[$23>>2] = $21;
   $24 = HEAP32[$1>>2]|0;
   $25 = (($24) + 56|0);
   $26 = HEAP32[$25>>2]|0;
   $27 = ($26|0)==(0|0);
   if ($27) {
    HEAP32[$0>>2] = 7;
    $89 = HEAP32[$0>>2]|0;
    STACKTOP = sp;return ($89|0);
   }
   $28 = HEAP32[$1>>2]|0;
   $29 = (($28) + 64|0);
   $30 = HEAP32[$29>>2]|0;
   $31 = HEAP32[$30>>2]|0;
   $32 = ($31|0)!=(0|0);
   do {
    if (!($32)) {
     $33 = HEAP32[$1>>2]|0;
     $34 = (($33) + 5|0);
     $35 = HEAP8[$34>>0]|0;
     $36 = $35&255;
     $37 = ($36|0)==(4);
     $38 = HEAP32[$1>>2]|0;
     if ($37) {
      $39 = (($38) + 64|0);
      $40 = HEAP32[$39>>2]|0;
      _sqlite3MemJournalOpen($40);
      break;
     }
     $41 = (($38) + 12|0);
     $42 = HEAP8[$41>>0]|0;
     $43 = $42&255;
     $44 = ($43|0)!=(0);
     $45 = $44 ? 4104 : 2048;
     $46 = 6 | $45;
     HEAP32[$flags>>2] = $46;
     $47 = HEAP32[$1>>2]|0;
     $48 = (_databaseIsUnmoved($47)|0);
     HEAP32[$rc>>2] = $48;
     $49 = HEAP32[$rc>>2]|0;
     $50 = ($49|0)==(0);
     if ($50) {
      $51 = HEAP32[$pVfs>>2]|0;
      $52 = HEAP32[$1>>2]|0;
      $53 = (($52) + 172|0);
      $54 = HEAP32[$53>>2]|0;
      $55 = HEAP32[$1>>2]|0;
      $56 = (($55) + 64|0);
      $57 = HEAP32[$56>>2]|0;
      $58 = HEAP32[$flags>>2]|0;
      $59 = (_sqlite3OsOpen($51,$54,$57,$58,0)|0);
      HEAP32[$rc>>2] = $59;
     }
    }
   } while(0);
   $60 = HEAP32[$rc>>2]|0;
   $61 = ($60|0)==(0);
   if ($61) {
    $62 = HEAP32[$1>>2]|0;
    $63 = (($62) + 44|0);
    HEAP32[$63>>2] = 0;
    $64 = HEAP32[$1>>2]|0;
    $65 = (($64) + 72|0);
    $66 = $65;
    $67 = $66;
    HEAP32[$67>>2] = 0;
    $68 = (($66) + 4)|0;
    $69 = $68;
    HEAP32[$69>>2] = 0;
    $70 = HEAP32[$1>>2]|0;
    $71 = (($70) + 19|0);
    HEAP8[$71>>0] = 0;
    $72 = HEAP32[$1>>2]|0;
    $73 = (($72) + 80|0);
    $74 = $73;
    $75 = $74;
    HEAP32[$75>>2] = 0;
    $76 = (($74) + 4)|0;
    $77 = $76;
    HEAP32[$77>>2] = 0;
    $78 = HEAP32[$1>>2]|0;
    $79 = (_writeJournalHdr($78)|0);
    HEAP32[$rc>>2] = $79;
   }
  }
 }
 $80 = HEAP32[$rc>>2]|0;
 $81 = ($80|0)!=(0);
 $82 = HEAP32[$1>>2]|0;
 if ($81) {
  $83 = (($82) + 56|0);
  $84 = HEAP32[$83>>2]|0;
  _sqlite3BitvecDestroy($84);
  $85 = HEAP32[$1>>2]|0;
  $86 = (($85) + 56|0);
  HEAP32[$86>>2] = 0;
 } else {
  $87 = (($82) + 16|0);
  HEAP8[$87>>0] = 3;
 }
 $88 = HEAP32[$rc>>2]|0;
 HEAP32[$0>>2] = $88;
 $89 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($89|0);
}
function _pageInJournal($pPager,$pPg) {
 $pPager = $pPager|0;
 $pPg = $pPg|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $1 = sp;
 HEAP32[$0>>2] = $pPager;
 HEAP32[$1>>2] = $pPg;
 $2 = HEAP32[$0>>2]|0;
 $3 = (($2) + 56|0);
 $4 = HEAP32[$3>>2]|0;
 $5 = HEAP32[$1>>2]|0;
 $6 = (($5) + 20|0);
 $7 = HEAP32[$6>>2]|0;
 $8 = (_sqlite3BitvecTest($4,$7)|0);
 STACKTOP = sp;return ($8|0);
}
function _databaseIsUnmoved($pPager) {
 $pPager = $pPager|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0;
 var $8 = 0, $9 = 0, $bHasMoved = 0, $or$cond = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $1 = sp + 8|0;
 $bHasMoved = sp + 4|0;
 $rc = sp;
 HEAP32[$1>>2] = $pPager;
 HEAP32[$bHasMoved>>2] = 0;
 $2 = HEAP32[$1>>2]|0;
 $3 = (($2) + 12|0);
 $4 = HEAP8[$3>>0]|0;
 $5 = ($4<<24>>24)!=(0);
 if ($5) {
  HEAP32[$0>>2] = 0;
  $21 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($21|0);
 }
 $6 = HEAP32[$1>>2]|0;
 $7 = (($6) + 24|0);
 $8 = HEAP32[$7>>2]|0;
 $9 = ($8|0)==(0);
 if ($9) {
  HEAP32[$0>>2] = 0;
  $21 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($21|0);
 }
 $10 = HEAP32[$1>>2]|0;
 $11 = (($10) + 60|0);
 $12 = HEAP32[$11>>2]|0;
 $13 = (_sqlite3OsFileControl($12,20,$bHasMoved)|0);
 HEAP32[$rc>>2] = $13;
 $14 = HEAP32[$rc>>2]|0;
 $15 = ($14|0)==(12);
 if ($15) {
  HEAP32[$rc>>2] = 0;
 } else {
  $16 = HEAP32[$rc>>2]|0;
  $17 = ($16|0)==(0);
  $18 = HEAP32[$bHasMoved>>2]|0;
  $19 = ($18|0)!=(0);
  $or$cond = $17 & $19;
  if ($or$cond) {
   HEAP32[$rc>>2] = 1032;
  }
 }
 $20 = HEAP32[$rc>>2]|0;
 HEAP32[$0>>2] = $20;
 $21 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($21|0);
}
function _pagerAcquireMapPage($pPager,$pgno,$pData,$ppPage) {
 $pPager = $pPager|0;
 $pgno = $pgno|0;
 $pData = $pData|0;
 $ppPage = $ppPage|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $8 = 0, $9 = 0, $p = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 20|0;
 $1 = sp + 16|0;
 $2 = sp + 12|0;
 $3 = sp + 8|0;
 $4 = sp + 4|0;
 $p = sp;
 HEAP32[$1>>2] = $pPager;
 HEAP32[$2>>2] = $pgno;
 HEAP32[$3>>2] = $pData;
 HEAP32[$4>>2] = $ppPage;
 $5 = HEAP32[$1>>2]|0;
 $6 = (($5) + 136|0);
 $7 = HEAP32[$6>>2]|0;
 $8 = ($7|0)!=(0|0);
 $9 = HEAP32[$1>>2]|0;
 do {
  if ($8) {
   $10 = (($9) + 136|0);
   $11 = HEAP32[$10>>2]|0;
   HEAP32[$p>>2] = $11;
   $12 = HEAP32[$4>>2]|0;
   HEAP32[$12>>2] = $11;
   $13 = HEAP32[$p>>2]|0;
   $14 = (($13) + 12|0);
   $15 = HEAP32[$14>>2]|0;
   $16 = HEAP32[$1>>2]|0;
   $17 = (($16) + 136|0);
   HEAP32[$17>>2] = $15;
   $18 = HEAP32[$p>>2]|0;
   $19 = (($18) + 12|0);
   HEAP32[$19>>2] = 0;
   $20 = HEAP32[$p>>2]|0;
   $21 = (($20) + 8|0);
   $22 = HEAP32[$21>>2]|0;
   $23 = HEAP32[$1>>2]|0;
   $24 = (($23) + 140|0);
   $25 = HEAP16[$24>>1]|0;
   $26 = $25&65535;
   _memset(($22|0),0,($26|0))|0;
  } else {
   $27 = (($9) + 140|0);
   $28 = HEAP16[$27>>1]|0;
   $29 = $28&65535;
   $30 = (40 + ($29))|0;
   $31 = (_sqlite3MallocZero($30,0)|0);
   HEAP32[$p>>2] = $31;
   $32 = HEAP32[$4>>2]|0;
   HEAP32[$32>>2] = $31;
   $33 = HEAP32[$p>>2]|0;
   $34 = ($33|0)==(0|0);
   if (!($34)) {
    $48 = HEAP32[$p>>2]|0;
    $49 = (($48) + 40|0);
    $50 = HEAP32[$p>>2]|0;
    $51 = (($50) + 8|0);
    HEAP32[$51>>2] = $49;
    $52 = HEAP32[$p>>2]|0;
    $53 = (($52) + 24|0);
    HEAP16[$53>>1] = 64;
    $54 = HEAP32[$p>>2]|0;
    $55 = (($54) + 26|0);
    HEAP16[$55>>1] = 1;
    $56 = HEAP32[$1>>2]|0;
    $57 = HEAP32[$p>>2]|0;
    $58 = (($57) + 16|0);
    HEAP32[$58>>2] = $56;
    break;
   }
   $35 = HEAP32[$1>>2]|0;
   $36 = (($35) + 60|0);
   $37 = HEAP32[$36>>2]|0;
   $38 = HEAP32[$2>>2]|0;
   $39 = (($38) - 1)|0;
   $40 = HEAP32[$1>>2]|0;
   $41 = (($40) + 152|0);
   $42 = HEAP32[$41>>2]|0;
   $43 = ($42|0)<(0);
   $44 = $43 << 31 >> 31;
   $45 = (___muldi3(($39|0),0,($42|0),($44|0))|0);
   $46 = tempRet0;
   $47 = HEAP32[$3>>2]|0;
   (_sqlite3OsUnfetch($37,$45,$46,$47)|0);
   HEAP32[$0>>2] = 7;
   $69 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($69|0);
  }
 } while(0);
 $59 = HEAP32[$2>>2]|0;
 $60 = HEAP32[$p>>2]|0;
 $61 = (($60) + 20|0);
 HEAP32[$61>>2] = $59;
 $62 = HEAP32[$3>>2]|0;
 $63 = HEAP32[$p>>2]|0;
 $64 = (($63) + 4|0);
 HEAP32[$64>>2] = $62;
 $65 = HEAP32[$1>>2]|0;
 $66 = (($65) + 120|0);
 $67 = HEAP32[$66>>2]|0;
 $68 = (($67) + 1)|0;
 HEAP32[$66>>2] = $68;
 HEAP32[$0>>2] = 0;
 $69 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($69|0);
}
function _sqlite3PcacheFetchStress($pCache,$pgno,$ppPage) {
 $pCache = $pCache|0;
 $pgno = $pgno|0;
 $ppPage = $ppPage|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $8 = 0, $9 = 0, $or$cond = 0, $pPg = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 20|0;
 $1 = sp + 16|0;
 $2 = sp + 12|0;
 $3 = sp + 8|0;
 $pPg = sp + 4|0;
 $rc = sp;
 HEAP32[$1>>2] = $pCache;
 HEAP32[$2>>2] = $pgno;
 HEAP32[$3>>2] = $ppPage;
 $4 = HEAP32[$1>>2]|0;
 $5 = (($4) + 29|0);
 $6 = HEAP8[$5>>0]|0;
 $7 = $6&255;
 $8 = ($7|0)==(2);
 if ($8) {
  HEAP32[$0>>2] = 0;
  $72 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($72|0);
 }
 $9 = HEAP32[$1>>2]|0;
 $10 = (($9) + 8|0);
 $11 = HEAP32[$10>>2]|0;
 HEAP32[$pPg>>2] = $11;
 while(1) {
  $12 = HEAP32[$pPg>>2]|0;
  $13 = ($12|0)!=(0|0);
  if (!($13)) {
   break;
  }
  $14 = HEAP32[$pPg>>2]|0;
  $15 = (($14) + 26|0);
  $16 = HEAP16[$15>>1]|0;
  $17 = $16 << 16 >> 16;
  $18 = ($17|0)!=(0);
  if (!($18)) {
   $19 = HEAP32[$pPg>>2]|0;
   $20 = (($19) + 24|0);
   $21 = HEAP16[$20>>1]|0;
   $22 = $21&65535;
   $23 = $22 & 4;
   $24 = ($23|0)!=(0);
   if (!($24)) {
    break;
   }
  }
  $25 = HEAP32[$pPg>>2]|0;
  $26 = (($25) + 36|0);
  $27 = HEAP32[$26>>2]|0;
  HEAP32[$pPg>>2] = $27;
 }
 $28 = HEAP32[$pPg>>2]|0;
 $29 = HEAP32[$1>>2]|0;
 $30 = (($29) + 8|0);
 HEAP32[$30>>2] = $28;
 $31 = HEAP32[$pPg>>2]|0;
 $32 = ($31|0)!=(0|0);
 L12: do {
  if (!($32)) {
   $33 = HEAP32[$1>>2]|0;
   $34 = (($33) + 4|0);
   $35 = HEAP32[$34>>2]|0;
   HEAP32[$pPg>>2] = $35;
   while(1) {
    $36 = HEAP32[$pPg>>2]|0;
    $37 = ($36|0)!=(0|0);
    if (!($37)) {
     break L12;
    }
    $38 = HEAP32[$pPg>>2]|0;
    $39 = (($38) + 26|0);
    $40 = HEAP16[$39>>1]|0;
    $41 = $40 << 16 >> 16;
    $42 = ($41|0)!=(0);
    if (!($42)) {
     break L12;
    }
    $43 = HEAP32[$pPg>>2]|0;
    $44 = (($43) + 36|0);
    $45 = HEAP32[$44>>2]|0;
    HEAP32[$pPg>>2] = $45;
   }
  }
 } while(0);
 $46 = HEAP32[$pPg>>2]|0;
 $47 = ($46|0)!=(0|0);
 if ($47) {
  $48 = HEAP32[$1>>2]|0;
  $49 = (($48) + 32|0);
  $50 = HEAP32[$49>>2]|0;
  $51 = HEAP32[$1>>2]|0;
  $52 = (($51) + 36|0);
  $53 = HEAP32[$52>>2]|0;
  $54 = HEAP32[$pPg>>2]|0;
  $55 = (FUNCTION_TABLE_iii[$50 & 63]($53,$54)|0);
  HEAP32[$rc>>2] = $55;
  $56 = HEAP32[$rc>>2]|0;
  $57 = ($56|0)!=(0);
  $58 = HEAP32[$rc>>2]|0;
  $59 = ($58|0)!=(5);
  $or$cond = $57 & $59;
  if ($or$cond) {
   $60 = HEAP32[$rc>>2]|0;
   HEAP32[$0>>2] = $60;
   $72 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($72|0);
  }
 }
 $61 = HEAP32[((38640 + 132|0))>>2]|0;
 $62 = HEAP32[$1>>2]|0;
 $63 = (($62) + 40|0);
 $64 = HEAP32[$63>>2]|0;
 $65 = HEAP32[$2>>2]|0;
 $66 = (FUNCTION_TABLE_iiii[$61 & 31]($64,$65,2)|0);
 $67 = HEAP32[$3>>2]|0;
 HEAP32[$67>>2] = $66;
 $68 = HEAP32[$3>>2]|0;
 $69 = HEAP32[$68>>2]|0;
 $70 = ($69|0)==(0|0);
 $71 = $70 ? 7 : 0;
 HEAP32[$0>>2] = $71;
 $72 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($72|0);
}
function _lockBtree($pBt) {
 $pBt = $pBt|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0;
 var $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0, $188 = 0;
 var $189 = 0, $19 = 0, $190 = 0, $191 = 0, $192 = 0, $193 = 0, $194 = 0, $195 = 0, $196 = 0, $197 = 0, $198 = 0, $199 = 0, $2 = 0, $20 = 0, $200 = 0, $201 = 0, $202 = 0, $203 = 0, $204 = 0, $205 = 0;
 var $206 = 0, $207 = 0, $208 = 0, $209 = 0, $21 = 0, $210 = 0, $211 = 0, $212 = 0, $213 = 0, $214 = 0, $215 = 0, $216 = 0, $217 = 0, $218 = 0, $219 = 0, $22 = 0, $220 = 0, $221 = 0, $222 = 0, $23 = 0;
 var $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0;
 var $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0;
 var $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0;
 var $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0;
 var $97 = 0, $98 = 0, $99 = 0, $isOpen = 0, $nPage = 0, $nPageFile = 0, $nPageHeader = 0, $or$cond = 0, $or$cond3 = 0, $pPage1 = 0, $page1 = 0, $pageSize = 0, $rc = 0, $usableSize = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $0 = sp + 40|0;
 $1 = sp + 36|0;
 $rc = sp + 32|0;
 $pPage1 = sp + 28|0;
 $nPage = sp + 24|0;
 $nPageFile = sp + 20|0;
 $nPageHeader = sp + 16|0;
 $pageSize = sp + 12|0;
 $usableSize = sp + 8|0;
 $page1 = sp + 4|0;
 $isOpen = sp;
 HEAP32[$1>>2] = $pBt;
 HEAP32[$nPageFile>>2] = 0;
 $2 = HEAP32[$1>>2]|0;
 $3 = HEAP32[$2>>2]|0;
 $4 = (_sqlite3PagerSharedLock($3)|0);
 HEAP32[$rc>>2] = $4;
 $5 = HEAP32[$rc>>2]|0;
 $6 = ($5|0)!=(0);
 if ($6) {
  $7 = HEAP32[$rc>>2]|0;
  HEAP32[$0>>2] = $7;
  $222 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($222|0);
 }
 $8 = HEAP32[$1>>2]|0;
 $9 = (_btreeGetPage($8,1,$pPage1,0)|0);
 HEAP32[$rc>>2] = $9;
 $10 = HEAP32[$rc>>2]|0;
 $11 = ($10|0)!=(0);
 if ($11) {
  $12 = HEAP32[$rc>>2]|0;
  HEAP32[$0>>2] = $12;
  $222 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($222|0);
 }
 $13 = HEAP32[$pPage1>>2]|0;
 $14 = (($13) + 56|0);
 $15 = HEAP32[$14>>2]|0;
 $16 = (($15) + 28|0);
 $17 = (_sqlite3Get4byte($16)|0);
 HEAP32[$nPageHeader>>2] = $17;
 HEAP32[$nPage>>2] = $17;
 $18 = HEAP32[$1>>2]|0;
 $19 = HEAP32[$18>>2]|0;
 _sqlite3PagerPagecount($19,$nPageFile);
 $20 = HEAP32[$nPage>>2]|0;
 $21 = ($20|0)==(0);
 if ($21) {
  label = 7;
 } else {
  $22 = HEAP32[$pPage1>>2]|0;
  $23 = (($22) + 56|0);
  $24 = HEAP32[$23>>2]|0;
  $25 = (($24) + 24|0);
  $26 = HEAP32[$pPage1>>2]|0;
  $27 = (($26) + 56|0);
  $28 = HEAP32[$27>>2]|0;
  $29 = (($28) + 92|0);
  $30 = (_memcmp($25,$29,4)|0);
  $31 = ($30|0)!=(0);
  if ($31) {
   label = 7;
  }
 }
 if ((label|0) == 7) {
  $32 = HEAP32[$nPageFile>>2]|0;
  HEAP32[$nPage>>2] = $32;
 }
 $33 = HEAP32[$nPage>>2]|0;
 $34 = ($33|0)>(0);
 L14: do {
  if ($34) {
   $35 = HEAP32[$pPage1>>2]|0;
   $36 = (($35) + 56|0);
   $37 = HEAP32[$36>>2]|0;
   HEAP32[$page1>>2] = $37;
   HEAP32[$rc>>2] = 26;
   $38 = HEAP32[$page1>>2]|0;
   $39 = (_memcmp($38,34408,16)|0);
   $40 = ($39|0)!=(0);
   L16: do {
    if (!($40)) {
     $41 = HEAP32[$page1>>2]|0;
     $42 = (($41) + 18|0);
     $43 = HEAP8[$42>>0]|0;
     $44 = $43&255;
     $45 = ($44|0)>(2);
     if ($45) {
      $46 = HEAP32[$1>>2]|0;
      $47 = (($46) + 22|0);
      $48 = HEAP16[$47>>1]|0;
      $49 = $48&65535;
      $50 = $49 | 1;
      $51 = $50&65535;
      HEAP16[$47>>1] = $51;
     }
     $52 = HEAP32[$page1>>2]|0;
     $53 = (($52) + 19|0);
     $54 = HEAP8[$53>>0]|0;
     $55 = $54&255;
     $56 = ($55|0)>(2);
     if (!($56)) {
      $57 = HEAP32[$page1>>2]|0;
      $58 = (($57) + 19|0);
      $59 = HEAP8[$58>>0]|0;
      $60 = $59&255;
      $61 = ($60|0)==(2);
      do {
       if ($61) {
        $62 = HEAP32[$1>>2]|0;
        $63 = (($62) + 22|0);
        $64 = HEAP16[$63>>1]|0;
        $65 = $64&65535;
        $66 = $65 & 16;
        $67 = ($66|0)==(0);
        if ($67) {
         HEAP32[$isOpen>>2] = 0;
         $68 = HEAP32[$1>>2]|0;
         $69 = HEAP32[$68>>2]|0;
         $70 = (_sqlite3PagerOpenWal($69,$isOpen)|0);
         HEAP32[$rc>>2] = $70;
         $71 = HEAP32[$rc>>2]|0;
         $72 = ($71|0)!=(0);
         if ($72) {
          break L16;
         }
         $73 = HEAP32[$isOpen>>2]|0;
         $74 = ($73|0)==(0);
         if (!($74)) {
          HEAP32[$rc>>2] = 26;
          break;
         }
         $75 = HEAP32[$pPage1>>2]|0;
         _releasePage($75);
         HEAP32[$0>>2] = 0;
         $222 = HEAP32[$0>>2]|0;
         STACKTOP = sp;return ($222|0);
        }
       }
      } while(0);
      $76 = HEAP32[$page1>>2]|0;
      $77 = (($76) + 21|0);
      $78 = (_memcmp($77,37104,3)|0);
      $79 = ($78|0)!=(0);
      if (!($79)) {
       $80 = HEAP32[$page1>>2]|0;
       $81 = (($80) + 16|0);
       $82 = HEAP8[$81>>0]|0;
       $83 = $82&255;
       $84 = $83 << 8;
       $85 = HEAP32[$page1>>2]|0;
       $86 = (($85) + 17|0);
       $87 = HEAP8[$86>>0]|0;
       $88 = $87&255;
       $89 = $88 << 16;
       $90 = $84 | $89;
       HEAP32[$pageSize>>2] = $90;
       $91 = HEAP32[$pageSize>>2]|0;
       $92 = (($91) - 1)|0;
       $93 = HEAP32[$pageSize>>2]|0;
       $94 = $92 & $93;
       $95 = ($94|0)!=(0);
       $96 = HEAP32[$pageSize>>2]|0;
       $97 = ($96>>>0)>(65536);
       $or$cond = $95 | $97;
       $98 = HEAP32[$pageSize>>2]|0;
       $99 = ($98>>>0)<=(256);
       $or$cond3 = $or$cond | $99;
       if (!($or$cond3)) {
        $100 = HEAP32[$pageSize>>2]|0;
        $101 = HEAP32[$page1>>2]|0;
        $102 = (($101) + 20|0);
        $103 = HEAP8[$102>>0]|0;
        $104 = $103&255;
        $105 = (($100) - ($104))|0;
        HEAP32[$usableSize>>2] = $105;
        $106 = HEAP32[$pageSize>>2]|0;
        $107 = HEAP32[$1>>2]|0;
        $108 = (($107) + 32|0);
        $109 = HEAP32[$108>>2]|0;
        $110 = ($106|0)!=($109|0);
        if ($110) {
         $111 = HEAP32[$pPage1>>2]|0;
         _releasePage($111);
         $112 = HEAP32[$usableSize>>2]|0;
         $113 = HEAP32[$1>>2]|0;
         $114 = (($113) + 36|0);
         HEAP32[$114>>2] = $112;
         $115 = HEAP32[$pageSize>>2]|0;
         $116 = HEAP32[$1>>2]|0;
         $117 = (($116) + 32|0);
         HEAP32[$117>>2] = $115;
         $118 = HEAP32[$1>>2]|0;
         _freeTempSpace($118);
         $119 = HEAP32[$1>>2]|0;
         $120 = HEAP32[$119>>2]|0;
         $121 = HEAP32[$1>>2]|0;
         $122 = (($121) + 32|0);
         $123 = HEAP32[$pageSize>>2]|0;
         $124 = HEAP32[$usableSize>>2]|0;
         $125 = (($123) - ($124))|0;
         $126 = (_sqlite3PagerSetPagesize($120,$122,$125)|0);
         HEAP32[$rc>>2] = $126;
         $127 = HEAP32[$rc>>2]|0;
         HEAP32[$0>>2] = $127;
         $222 = HEAP32[$0>>2]|0;
         STACKTOP = sp;return ($222|0);
        }
        $128 = HEAP32[$1>>2]|0;
        $129 = (($128) + 4|0);
        $130 = HEAP32[$129>>2]|0;
        $131 = (($130) + 24|0);
        $132 = HEAP32[$131>>2]|0;
        $133 = $132 & 65536;
        $134 = ($133|0)==(0);
        if ($134) {
         $135 = HEAP32[$nPage>>2]|0;
         $136 = HEAP32[$nPageFile>>2]|0;
         $137 = ($135|0)>($136|0);
         if ($137) {
          $138 = (_sqlite3CorruptError(54754)|0);
          HEAP32[$rc>>2] = $138;
          break;
         }
        }
        $139 = HEAP32[$usableSize>>2]|0;
        $140 = ($139>>>0)<(480);
        if (!($140)) {
         $141 = HEAP32[$pageSize>>2]|0;
         $142 = HEAP32[$1>>2]|0;
         $143 = (($142) + 32|0);
         HEAP32[$143>>2] = $141;
         $144 = HEAP32[$usableSize>>2]|0;
         $145 = HEAP32[$1>>2]|0;
         $146 = (($145) + 36|0);
         HEAP32[$146>>2] = $144;
         $147 = HEAP32[$page1>>2]|0;
         $148 = (($147) + 52|0);
         $149 = (_sqlite3Get4byte($148)|0);
         $150 = ($149|0)!=(0);
         $151 = $150 ? 1 : 0;
         $152 = $151&255;
         $153 = HEAP32[$1>>2]|0;
         $154 = (($153) + 17|0);
         HEAP8[$154>>0] = $152;
         $155 = HEAP32[$page1>>2]|0;
         $156 = (($155) + 64|0);
         $157 = (_sqlite3Get4byte($156)|0);
         $158 = ($157|0)!=(0);
         $159 = $158 ? 1 : 0;
         $160 = $159&255;
         $161 = HEAP32[$1>>2]|0;
         $162 = (($161) + 18|0);
         HEAP8[$162>>0] = $160;
         break L14;
        }
       }
      }
     }
    }
   } while(0);
   $218 = HEAP32[$pPage1>>2]|0;
   _releasePage($218);
   $219 = HEAP32[$1>>2]|0;
   $220 = (($219) + 12|0);
   HEAP32[$220>>2] = 0;
   $221 = HEAP32[$rc>>2]|0;
   HEAP32[$0>>2] = $221;
   $222 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($222|0);
  }
 } while(0);
 $163 = HEAP32[$1>>2]|0;
 $164 = (($163) + 36|0);
 $165 = HEAP32[$164>>2]|0;
 $166 = (($165) - 12)|0;
 $167 = $166<<6;
 $168 = (($167>>>0) / 255)&-1;
 $169 = (($168) - 23)|0;
 $170 = $169&65535;
 $171 = HEAP32[$1>>2]|0;
 $172 = (($171) + 24|0);
 HEAP16[$172>>1] = $170;
 $173 = HEAP32[$1>>2]|0;
 $174 = (($173) + 36|0);
 $175 = HEAP32[$174>>2]|0;
 $176 = (($175) - 12)|0;
 $177 = $176<<5;
 $178 = (($177>>>0) / 255)&-1;
 $179 = (($178) - 23)|0;
 $180 = $179&65535;
 $181 = HEAP32[$1>>2]|0;
 $182 = (($181) + 26|0);
 HEAP16[$182>>1] = $180;
 $183 = HEAP32[$1>>2]|0;
 $184 = (($183) + 36|0);
 $185 = HEAP32[$184>>2]|0;
 $186 = (($185) - 35)|0;
 $187 = $186&65535;
 $188 = HEAP32[$1>>2]|0;
 $189 = (($188) + 28|0);
 HEAP16[$189>>1] = $187;
 $190 = HEAP32[$1>>2]|0;
 $191 = (($190) + 36|0);
 $192 = HEAP32[$191>>2]|0;
 $193 = (($192) - 12)|0;
 $194 = $193<<5;
 $195 = (($194>>>0) / 255)&-1;
 $196 = (($195) - 23)|0;
 $197 = $196&65535;
 $198 = HEAP32[$1>>2]|0;
 $199 = (($198) + 30|0);
 HEAP16[$199>>1] = $197;
 $200 = HEAP32[$1>>2]|0;
 $201 = (($200) + 24|0);
 $202 = HEAP16[$201>>1]|0;
 $203 = $202&65535;
 $204 = ($203|0)>(127);
 $205 = HEAP32[$1>>2]|0;
 if ($204) {
  $206 = (($205) + 21|0);
  HEAP8[$206>>0] = 127;
 } else {
  $207 = (($205) + 24|0);
  $208 = HEAP16[$207>>1]|0;
  $209 = $208&255;
  $210 = HEAP32[$1>>2]|0;
  $211 = (($210) + 21|0);
  HEAP8[$211>>0] = $209;
 }
 $212 = HEAP32[$pPage1>>2]|0;
 $213 = HEAP32[$1>>2]|0;
 $214 = (($213) + 12|0);
 HEAP32[$214>>2] = $212;
 $215 = HEAP32[$nPage>>2]|0;
 $216 = HEAP32[$1>>2]|0;
 $217 = (($216) + 44|0);
 HEAP32[$217>>2] = $215;
 HEAP32[$0>>2] = 0;
 $222 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($222|0);
}
function _sqlite3PagerBegin($pPager,$exFlag,$subjInMemory) {
 $pPager = $pPager|0;
 $exFlag = $exFlag|0;
 $subjInMemory = $subjInMemory|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $8 = 0, $9 = 0, $or$cond = 0;
 var $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 16|0;
 $1 = sp + 12|0;
 $2 = sp + 8|0;
 $3 = sp + 4|0;
 $rc = sp;
 HEAP32[$1>>2] = $pPager;
 HEAP32[$2>>2] = $exFlag;
 HEAP32[$3>>2] = $subjInMemory;
 HEAP32[$rc>>2] = 0;
 $4 = HEAP32[$1>>2]|0;
 $5 = (($4) + 40|0);
 $6 = HEAP32[$5>>2]|0;
 $7 = ($6|0)!=(0);
 if ($7) {
  $8 = HEAP32[$1>>2]|0;
  $9 = (($8) + 40|0);
  $10 = HEAP32[$9>>2]|0;
  HEAP32[$0>>2] = $10;
  $78 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($78|0);
 }
 $11 = HEAP32[$3>>2]|0;
 $12 = $11&255;
 $13 = HEAP32[$1>>2]|0;
 $14 = (($13) + 21|0);
 HEAP8[$14>>0] = $12;
 $15 = HEAP32[$1>>2]|0;
 $16 = (($15) + 16|0);
 $17 = HEAP8[$16>>0]|0;
 $18 = $17&255;
 $19 = ($18|0)==(1);
 if ($19) {
  $20 = HEAP32[$1>>2]|0;
  $21 = (_pagerUseWal($20)|0);
  $22 = ($21|0)!=(0);
  $23 = HEAP32[$1>>2]|0;
  if ($22) {
   $24 = (($23) + 4|0);
   $25 = HEAP8[$24>>0]|0;
   $26 = $25&255;
   $27 = ($26|0)!=(0);
   do {
    if ($27) {
     $28 = HEAP32[$1>>2]|0;
     $29 = (($28) + 208|0);
     $30 = HEAP32[$29>>2]|0;
     $31 = (_sqlite3WalExclusiveMode($30,-1)|0);
     $32 = ($31|0)!=(0);
     if ($32) {
      $33 = HEAP32[$1>>2]|0;
      $34 = (_pagerLockDb($33,4)|0);
      HEAP32[$rc>>2] = $34;
      $35 = HEAP32[$rc>>2]|0;
      $36 = ($35|0)!=(0);
      if (!($36)) {
       $38 = HEAP32[$1>>2]|0;
       $39 = (($38) + 208|0);
       $40 = HEAP32[$39>>2]|0;
       (_sqlite3WalExclusiveMode($40,1)|0);
       break;
      }
      $37 = HEAP32[$rc>>2]|0;
      HEAP32[$0>>2] = $37;
      $78 = HEAP32[$0>>2]|0;
      STACKTOP = sp;return ($78|0);
     }
    }
   } while(0);
   $41 = HEAP32[$1>>2]|0;
   $42 = (($41) + 208|0);
   $43 = HEAP32[$42>>2]|0;
   $44 = (_sqlite3WalBeginWriteTransaction($43)|0);
   HEAP32[$rc>>2] = $44;
  } else {
   $45 = (_pagerLockDb($23,2)|0);
   HEAP32[$rc>>2] = $45;
   $46 = HEAP32[$rc>>2]|0;
   $47 = ($46|0)==(0);
   $48 = HEAP32[$2>>2]|0;
   $49 = ($48|0)!=(0);
   $or$cond = $47 & $49;
   if ($or$cond) {
    $50 = HEAP32[$1>>2]|0;
    $51 = (_pager_wait_on_lock($50,4)|0);
    HEAP32[$rc>>2] = $51;
   }
  }
  $52 = HEAP32[$rc>>2]|0;
  $53 = ($52|0)==(0);
  if ($53) {
   $54 = HEAP32[$1>>2]|0;
   $55 = (($54) + 16|0);
   HEAP8[$55>>0] = 2;
   $56 = HEAP32[$1>>2]|0;
   $57 = (($56) + 24|0);
   $58 = HEAP32[$57>>2]|0;
   $59 = HEAP32[$1>>2]|0;
   $60 = (($59) + 36|0);
   HEAP32[$60>>2] = $58;
   $61 = HEAP32[$1>>2]|0;
   $62 = (($61) + 24|0);
   $63 = HEAP32[$62>>2]|0;
   $64 = HEAP32[$1>>2]|0;
   $65 = (($64) + 32|0);
   HEAP32[$65>>2] = $63;
   $66 = HEAP32[$1>>2]|0;
   $67 = (($66) + 24|0);
   $68 = HEAP32[$67>>2]|0;
   $69 = HEAP32[$1>>2]|0;
   $70 = (($69) + 28|0);
   HEAP32[$70>>2] = $68;
   $71 = HEAP32[$1>>2]|0;
   $72 = (($71) + 72|0);
   $73 = $72;
   $74 = $73;
   HEAP32[$74>>2] = 0;
   $75 = (($73) + 4)|0;
   $76 = $75;
   HEAP32[$76>>2] = 0;
  }
 }
 $77 = HEAP32[$rc>>2]|0;
 HEAP32[$0>>2] = $77;
 $78 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($78|0);
}
function _sqlite3WalBeginWriteTransaction($pWal) {
 $pWal = $pWal|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $3 = 0, $4 = 0, $5 = 0;
 var $6 = 0, $7 = 0, $8 = 0, $9 = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $1 = sp + 4|0;
 $rc = sp;
 HEAP32[$1>>2] = $pWal;
 $2 = HEAP32[$1>>2]|0;
 $3 = (($2) + 46|0);
 $4 = HEAP8[$3>>0]|0;
 $5 = ($4<<24>>24)!=(0);
 if ($5) {
  HEAP32[$0>>2] = 8;
  $23 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($23|0);
 }
 $6 = HEAP32[$1>>2]|0;
 $7 = (_walLockExclusive($6,0,1)|0);
 HEAP32[$rc>>2] = $7;
 $8 = HEAP32[$rc>>2]|0;
 $9 = ($8|0)!=(0);
 if ($9) {
  $10 = HEAP32[$rc>>2]|0;
  HEAP32[$0>>2] = $10;
  $23 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($23|0);
 }
 $11 = HEAP32[$1>>2]|0;
 $12 = (($11) + 44|0);
 HEAP8[$12>>0] = 1;
 $13 = HEAP32[$1>>2]|0;
 $14 = (($13) + 52|0);
 $15 = HEAP32[$1>>2]|0;
 $16 = (_walIndexHdr($15)|0);
 $17 = (_memcmp($14,$16,48)|0);
 $18 = ($17|0)!=(0);
 if ($18) {
  $19 = HEAP32[$1>>2]|0;
  _walUnlockExclusive($19,0,1);
  $20 = HEAP32[$1>>2]|0;
  $21 = (($20) + 44|0);
  HEAP8[$21>>0] = 0;
  HEAP32[$rc>>2] = 517;
 }
 $22 = HEAP32[$rc>>2]|0;
 HEAP32[$0>>2] = $22;
 $23 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($23|0);
}
function _sqlite3ValueSetNull($p) {
 $p = $p|0;
 var $0 = 0, $1 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 HEAP32[$0>>2] = $p;
 $1 = HEAP32[$0>>2]|0;
 _sqlite3VdbeMemSetNull($1);
 STACKTOP = sp;return;
}
function _dotlockIoFinderImpl($z,$p) {
 $z = $z|0;
 $p = $p|0;
 var $0 = 0, $1 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $1 = sp;
 HEAP32[$0>>2] = $z;
 HEAP32[$1>>2] = $p;
 STACKTOP = sp;return (37112|0);
}
function _dotlockClose($id) {
 $id = $id|0;
 var $0 = 0, $1 = 0, $10 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $pFile = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $rc = sp + 4|0;
 $pFile = sp;
 HEAP32[$0>>2] = $id;
 HEAP32[$rc>>2] = 0;
 $1 = HEAP32[$0>>2]|0;
 $2 = ($1|0)!=(0|0);
 if (!($2)) {
  $10 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($10|0);
 }
 $3 = HEAP32[$0>>2]|0;
 HEAP32[$pFile>>2] = $3;
 $4 = HEAP32[$0>>2]|0;
 (_dotlockUnlock($4,0)|0);
 $5 = HEAP32[$pFile>>2]|0;
 $6 = (($5) + 24|0);
 $7 = HEAP32[$6>>2]|0;
 _sqlite3_free($7);
 $8 = HEAP32[$0>>2]|0;
 $9 = (_closeUnixFile($8)|0);
 HEAP32[$rc>>2] = $9;
 $10 = HEAP32[$rc>>2]|0;
 STACKTOP = sp;return ($10|0);
}
function _unixRead($id,$pBuf,$amt,$0,$1) {
 $id = $id|0;
 $pBuf = $pBuf|0;
 $amt = $amt|0;
 $0 = $0|0;
 $1 = $1|0;
 var $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0;
 var $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $got = 0, $pFile = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $2 = sp + 28|0;
 $3 = sp + 24|0;
 $4 = sp + 20|0;
 $5 = sp + 16|0;
 $6 = sp;
 $pFile = sp + 12|0;
 $got = sp + 8|0;
 HEAP32[$3>>2] = $id;
 HEAP32[$4>>2] = $pBuf;
 HEAP32[$5>>2] = $amt;
 $7 = $6;
 $8 = $7;
 HEAP32[$8>>2] = $0;
 $9 = (($7) + 4)|0;
 $10 = $9;
 HEAP32[$10>>2] = $1;
 $11 = HEAP32[$3>>2]|0;
 HEAP32[$pFile>>2] = $11;
 $12 = HEAP32[$pFile>>2]|0;
 $13 = $6;
 $14 = $13;
 $15 = HEAP32[$14>>2]|0;
 $16 = (($13) + 4)|0;
 $17 = $16;
 $18 = HEAP32[$17>>2]|0;
 $19 = HEAP32[$4>>2]|0;
 $20 = HEAP32[$5>>2]|0;
 $21 = (_seekAndRead($12,$15,$18,$19,$20)|0);
 HEAP32[$got>>2] = $21;
 $22 = HEAP32[$got>>2]|0;
 $23 = HEAP32[$5>>2]|0;
 $24 = ($22|0)==($23|0);
 if ($24) {
  HEAP32[$2>>2] = 0;
  $35 = HEAP32[$2>>2]|0;
  STACKTOP = sp;return ($35|0);
 }
 $25 = HEAP32[$got>>2]|0;
 $26 = ($25|0)<(0);
 if ($26) {
  HEAP32[$2>>2] = 266;
  $35 = HEAP32[$2>>2]|0;
  STACKTOP = sp;return ($35|0);
 } else {
  $27 = HEAP32[$pFile>>2]|0;
  $28 = (($27) + 20|0);
  HEAP32[$28>>2] = 0;
  $29 = HEAP32[$got>>2]|0;
  $30 = HEAP32[$4>>2]|0;
  $31 = (($30) + ($29)|0);
  $32 = HEAP32[$5>>2]|0;
  $33 = HEAP32[$got>>2]|0;
  $34 = (($32) - ($33))|0;
  _memset(($31|0),0,($34|0))|0;
  HEAP32[$2>>2] = 522;
  $35 = HEAP32[$2>>2]|0;
  STACKTOP = sp;return ($35|0);
 }
 return 0|0;
}
function _unixWrite($id,$pBuf,$amt,$0,$1) {
 $id = $id|0;
 $pBuf = $pBuf|0;
 $amt = $amt|0;
 $0 = $0|0;
 $1 = $1|0;
 var $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0;
 var $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0;
 var $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $pFile = 0, $wrote = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $2 = sp + 28|0;
 $3 = sp + 24|0;
 $4 = sp + 20|0;
 $5 = sp + 16|0;
 $6 = sp;
 $pFile = sp + 12|0;
 $wrote = sp + 8|0;
 HEAP32[$3>>2] = $id;
 HEAP32[$4>>2] = $pBuf;
 HEAP32[$5>>2] = $amt;
 $7 = $6;
 $8 = $7;
 HEAP32[$8>>2] = $0;
 $9 = (($7) + 4)|0;
 $10 = $9;
 HEAP32[$10>>2] = $1;
 $11 = HEAP32[$3>>2]|0;
 HEAP32[$pFile>>2] = $11;
 HEAP32[$wrote>>2] = 0;
 while(1) {
  $12 = HEAP32[$5>>2]|0;
  $13 = ($12|0)>(0);
  if (!($13)) {
   break;
  }
  $14 = HEAP32[$pFile>>2]|0;
  $15 = $6;
  $16 = $15;
  $17 = HEAP32[$16>>2]|0;
  $18 = (($15) + 4)|0;
  $19 = $18;
  $20 = HEAP32[$19>>2]|0;
  $21 = HEAP32[$4>>2]|0;
  $22 = HEAP32[$5>>2]|0;
  $23 = (_seekAndWrite($14,$17,$20,$21,$22)|0);
  HEAP32[$wrote>>2] = $23;
  $24 = ($23|0)>(0);
  if (!($24)) {
   break;
  }
  $25 = HEAP32[$wrote>>2]|0;
  $26 = HEAP32[$5>>2]|0;
  $27 = (($26) - ($25))|0;
  HEAP32[$5>>2] = $27;
  $28 = HEAP32[$wrote>>2]|0;
  $29 = ($28|0)<(0);
  $30 = $29 << 31 >> 31;
  $31 = $6;
  $32 = $31;
  $33 = HEAP32[$32>>2]|0;
  $34 = (($31) + 4)|0;
  $35 = $34;
  $36 = HEAP32[$35>>2]|0;
  $37 = (_i64Add(($33|0),($36|0),($28|0),($30|0))|0);
  $38 = tempRet0;
  $39 = $6;
  $40 = $39;
  HEAP32[$40>>2] = $37;
  $41 = (($39) + 4)|0;
  $42 = $41;
  HEAP32[$42>>2] = $38;
  $43 = HEAP32[$wrote>>2]|0;
  $44 = HEAP32[$4>>2]|0;
  $45 = (($44) + ($43)|0);
  HEAP32[$4>>2] = $45;
 }
 $46 = HEAP32[$5>>2]|0;
 $47 = ($46|0)>(0);
 if (!($47)) {
  HEAP32[$2>>2] = 0;
  $56 = HEAP32[$2>>2]|0;
  STACKTOP = sp;return ($56|0);
 }
 $48 = HEAP32[$wrote>>2]|0;
 $49 = ($48|0)<(0);
 if ($49) {
  $50 = HEAP32[$pFile>>2]|0;
  $51 = (($50) + 20|0);
  $52 = HEAP32[$51>>2]|0;
  $53 = ($52|0)!=(28);
  if ($53) {
   HEAP32[$2>>2] = 778;
   $56 = HEAP32[$2>>2]|0;
   STACKTOP = sp;return ($56|0);
  }
 }
 $54 = HEAP32[$pFile>>2]|0;
 $55 = (($54) + 20|0);
 HEAP32[$55>>2] = 0;
 HEAP32[$2>>2] = 13;
 $56 = HEAP32[$2>>2]|0;
 STACKTOP = sp;return ($56|0);
}
function _unixTruncate($id,$0,$1) {
 $id = $id|0;
 $0 = $0|0;
 $1 = $1|0;
 var $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0;
 var $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0;
 var $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0;
 var $65 = 0, $66 = 0, $67 = 0, $7 = 0, $8 = 0, $9 = 0, $pFile = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $2 = sp + 20|0;
 $3 = sp + 16|0;
 $4 = sp;
 $pFile = sp + 12|0;
 $rc = sp + 8|0;
 HEAP32[$3>>2] = $id;
 $5 = $4;
 $6 = $5;
 HEAP32[$6>>2] = $0;
 $7 = (($5) + 4)|0;
 $8 = $7;
 HEAP32[$8>>2] = $1;
 $9 = HEAP32[$3>>2]|0;
 HEAP32[$pFile>>2] = $9;
 $10 = HEAP32[$pFile>>2]|0;
 $11 = (($10) + 40|0);
 $12 = HEAP32[$11>>2]|0;
 $13 = ($12|0)>(0);
 if ($13) {
  $14 = $4;
  $15 = $14;
  $16 = HEAP32[$15>>2]|0;
  $17 = (($14) + 4)|0;
  $18 = $17;
  $19 = HEAP32[$18>>2]|0;
  $20 = HEAP32[$pFile>>2]|0;
  $21 = (($20) + 40|0);
  $22 = HEAP32[$21>>2]|0;
  $23 = ($22|0)<(0);
  $24 = $23 << 31 >> 31;
  $25 = (_i64Add(($16|0),($19|0),($22|0),($24|0))|0);
  $26 = tempRet0;
  $27 = (_i64Subtract(($25|0),($26|0),1,0)|0);
  $28 = tempRet0;
  $29 = HEAP32[$pFile>>2]|0;
  $30 = (($29) + 40|0);
  $31 = HEAP32[$30>>2]|0;
  $32 = ($31|0)<(0);
  $33 = $32 << 31 >> 31;
  $34 = (___divdi3(($27|0),($28|0),($31|0),($33|0))|0);
  $35 = tempRet0;
  $36 = HEAP32[$pFile>>2]|0;
  $37 = (($36) + 40|0);
  $38 = HEAP32[$37>>2]|0;
  $39 = ($38|0)<(0);
  $40 = $39 << 31 >> 31;
  $41 = (___muldi3(($34|0),($35|0),($38|0),($40|0))|0);
  $42 = tempRet0;
  $43 = $4;
  $44 = $43;
  HEAP32[$44>>2] = $41;
  $45 = (($43) + 4)|0;
  $46 = $45;
  HEAP32[$46>>2] = $42;
 }
 $47 = HEAP32[$pFile>>2]|0;
 $48 = (($47) + 12|0);
 $49 = HEAP32[$48>>2]|0;
 $50 = $4;
 $51 = $50;
 $52 = HEAP32[$51>>2]|0;
 $53 = (($50) + 4)|0;
 $54 = $53;
 $55 = HEAP32[$54>>2]|0;
 $56 = (_robust_ftruncate($49,$52,$55)|0);
 HEAP32[$rc>>2] = $56;
 $57 = HEAP32[$rc>>2]|0;
 $58 = ($57|0)!=(0);
 if ($58) {
  $59 = (___errno_location()|0);
  $60 = HEAP32[$59>>2]|0;
  $61 = HEAP32[$pFile>>2]|0;
  $62 = (($61) + 20|0);
  HEAP32[$62>>2] = $60;
  $63 = HEAP32[$pFile>>2]|0;
  $64 = (($63) + 32|0);
  $65 = HEAP32[$64>>2]|0;
  $66 = (_unixLogErrorAtLine(1546,37568,$65,28429)|0);
  HEAP32[$2>>2] = $66;
  $67 = HEAP32[$2>>2]|0;
  STACKTOP = sp;return ($67|0);
 } else {
  HEAP32[$2>>2] = 0;
  $67 = HEAP32[$2>>2]|0;
  STACKTOP = sp;return ($67|0);
 }
 return 0|0;
}
function _unixSync($id,$flags) {
 $id = $id|0;
 $flags = $flags|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $dirfd = 0, $isDataOnly = 0, $isFullsync = 0, $or$cond = 0, $pFile = 0, $rc = 0;
 var label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 28|0;
 $1 = sp + 24|0;
 $2 = sp + 20|0;
 $rc = sp + 16|0;
 $pFile = sp + 12|0;
 $isDataOnly = sp + 8|0;
 $isFullsync = sp + 4|0;
 $dirfd = sp;
 HEAP32[$1>>2] = $id;
 HEAP32[$2>>2] = $flags;
 $3 = HEAP32[$1>>2]|0;
 HEAP32[$pFile>>2] = $3;
 $4 = HEAP32[$2>>2]|0;
 $5 = $4 & 16;
 HEAP32[$isDataOnly>>2] = $5;
 $6 = HEAP32[$2>>2]|0;
 $7 = $6 & 15;
 $8 = ($7|0)==(3);
 $9 = $8&1;
 HEAP32[$isFullsync>>2] = $9;
 $10 = HEAP32[$pFile>>2]|0;
 $11 = (($10) + 12|0);
 $12 = HEAP32[$11>>2]|0;
 $13 = HEAP32[$isFullsync>>2]|0;
 $14 = HEAP32[$isDataOnly>>2]|0;
 $15 = (_full_fsync($12,$13,$14)|0);
 HEAP32[$rc>>2] = $15;
 $16 = HEAP32[$rc>>2]|0;
 $17 = ($16|0)!=(0);
 if ($17) {
  $18 = (___errno_location()|0);
  $19 = HEAP32[$18>>2]|0;
  $20 = HEAP32[$pFile>>2]|0;
  $21 = (($20) + 20|0);
  HEAP32[$21>>2] = $19;
  $22 = HEAP32[$pFile>>2]|0;
  $23 = (($22) + 32|0);
  $24 = HEAP32[$23>>2]|0;
  $25 = (_unixLogErrorAtLine(1034,38000,$24,28385)|0);
  HEAP32[$0>>2] = $25;
  $53 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($53|0);
 }
 $26 = HEAP32[$pFile>>2]|0;
 $27 = (($26) + 18|0);
 $28 = HEAP16[$27>>1]|0;
 $29 = $28&65535;
 $30 = $29 & 8;
 $31 = ($30|0)!=(0);
 if ($31) {
  $32 = HEAP32[((37192 + 208|0))>>2]|0;
  $33 = HEAP32[$pFile>>2]|0;
  $34 = (($33) + 32|0);
  $35 = HEAP32[$34>>2]|0;
  $36 = (FUNCTION_TABLE_iii[$32 & 63]($35,$dirfd)|0);
  HEAP32[$rc>>2] = $36;
  $37 = HEAP32[$rc>>2]|0;
  $38 = ($37|0)==(0);
  $39 = HEAP32[$dirfd>>2]|0;
  $40 = ($39|0)>=(0);
  $or$cond = $38 & $40;
  if ($or$cond) {
   $41 = HEAP32[$dirfd>>2]|0;
   (_full_fsync($41,0,0)|0);
   $42 = HEAP32[$pFile>>2]|0;
   $43 = HEAP32[$dirfd>>2]|0;
   _robust_close($42,$43,28399);
  } else {
   $44 = HEAP32[$rc>>2]|0;
   $45 = ($44|0)==(14);
   if ($45) {
    HEAP32[$rc>>2] = 0;
   }
  }
  $46 = HEAP32[$pFile>>2]|0;
  $47 = (($46) + 18|0);
  $48 = HEAP16[$47>>1]|0;
  $49 = $48&65535;
  $50 = $49 & -9;
  $51 = $50&65535;
  HEAP16[$47>>1] = $51;
 }
 $52 = HEAP32[$rc>>2]|0;
 HEAP32[$0>>2] = $52;
 $53 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($53|0);
}
function _unixFileSize($id,$pSize) {
 $id = $id|0;
 $pSize = $pSize|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $buf = 0;
 var $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 96|0;
 $0 = sp + 88|0;
 $1 = sp + 84|0;
 $2 = sp + 80|0;
 $rc = sp + 76|0;
 $buf = sp;
 HEAP32[$1>>2] = $id;
 HEAP32[$2>>2] = $pSize;
 $3 = HEAP32[((37192 + 64|0))>>2]|0;
 $4 = HEAP32[$1>>2]|0;
 $5 = (($4) + 12|0);
 $6 = HEAP32[$5>>2]|0;
 $7 = (FUNCTION_TABLE_iii[$3 & 63]($6,$buf)|0);
 HEAP32[$rc>>2] = $7;
 $8 = HEAP32[$rc>>2]|0;
 $9 = ($8|0)!=(0);
 if ($9) {
  $10 = (___errno_location()|0);
  $11 = HEAP32[$10>>2]|0;
  $12 = HEAP32[$1>>2]|0;
  $13 = (($12) + 20|0);
  HEAP32[$13>>2] = $11;
  HEAP32[$0>>2] = 1802;
  $38 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($38|0);
 }
 $14 = (($buf) + 36|0);
 $15 = HEAP32[$14>>2]|0;
 $16 = ($15|0)<(0);
 $17 = $16 << 31 >> 31;
 $18 = HEAP32[$2>>2]|0;
 $19 = $18;
 $20 = $19;
 HEAP32[$20>>2] = $15;
 $21 = (($19) + 4)|0;
 $22 = $21;
 HEAP32[$22>>2] = $17;
 $23 = HEAP32[$2>>2]|0;
 $24 = $23;
 $25 = $24;
 $26 = HEAP32[$25>>2]|0;
 $27 = (($24) + 4)|0;
 $28 = $27;
 $29 = HEAP32[$28>>2]|0;
 $30 = ($26|0)==(1);
 $31 = ($29|0)==(0);
 $32 = $30 & $31;
 if ($32) {
  $33 = HEAP32[$2>>2]|0;
  $34 = $33;
  $35 = $34;
  HEAP32[$35>>2] = 0;
  $36 = (($34) + 4)|0;
  $37 = $36;
  HEAP32[$37>>2] = 0;
 }
 HEAP32[$0>>2] = 0;
 $38 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($38|0);
}
function _dotlockLock($id,$eFileLock) {
 $id = $id|0;
 $eFileLock = $eFileLock|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $5 = 0, $6 = 0, $7 = 0;
 var $8 = 0, $9 = 0, $or$cond = 0, $pFile = 0, $rc = 0, $tErrno = 0, $zLockFile = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 24|0;
 $1 = sp + 20|0;
 $2 = sp + 16|0;
 $pFile = sp + 12|0;
 $zLockFile = sp + 8|0;
 $rc = sp + 4|0;
 $tErrno = sp;
 HEAP32[$1>>2] = $id;
 HEAP32[$2>>2] = $eFileLock;
 $3 = HEAP32[$1>>2]|0;
 HEAP32[$pFile>>2] = $3;
 $4 = HEAP32[$pFile>>2]|0;
 $5 = (($4) + 24|0);
 $6 = HEAP32[$5>>2]|0;
 HEAP32[$zLockFile>>2] = $6;
 HEAP32[$rc>>2] = 0;
 $7 = HEAP32[$pFile>>2]|0;
 $8 = (($7) + 16|0);
 $9 = HEAP8[$8>>0]|0;
 $10 = $9&255;
 $11 = ($10|0)>(0);
 if ($11) {
  $12 = HEAP32[$2>>2]|0;
  $13 = $12&255;
  $14 = HEAP32[$pFile>>2]|0;
  $15 = (($14) + 16|0);
  HEAP8[$15>>0] = $13;
  $16 = HEAP32[$zLockFile>>2]|0;
  (_utimes(($16|0),(0|0))|0);
  HEAP32[$0>>2] = 0;
  $41 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($41|0);
 }
 $17 = HEAP32[((37192 + 220|0))>>2]|0;
 $18 = HEAP32[$zLockFile>>2]|0;
 $19 = (FUNCTION_TABLE_iii[$17 & 63]($18,511)|0);
 HEAP32[$rc>>2] = $19;
 $20 = HEAP32[$rc>>2]|0;
 $21 = ($20|0)<(0);
 if (!($21)) {
  $36 = HEAP32[$2>>2]|0;
  $37 = $36&255;
  $38 = HEAP32[$pFile>>2]|0;
  $39 = (($38) + 16|0);
  HEAP8[$39>>0] = $37;
  $40 = HEAP32[$rc>>2]|0;
  HEAP32[$0>>2] = $40;
  $41 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($41|0);
 }
 $22 = (___errno_location()|0);
 $23 = HEAP32[$22>>2]|0;
 HEAP32[$tErrno>>2] = $23;
 $24 = HEAP32[$tErrno>>2]|0;
 $25 = (17)==($24|0);
 if ($25) {
  HEAP32[$rc>>2] = 5;
 } else {
  $26 = HEAP32[$tErrno>>2]|0;
  $27 = (_sqliteErrorFromPosixError($26,3850)|0);
  HEAP32[$rc>>2] = $27;
  $28 = HEAP32[$rc>>2]|0;
  $29 = ($28|0)!=(0);
  $30 = HEAP32[$rc>>2]|0;
  $31 = ($30|0)!=(5);
  $or$cond = $29 & $31;
  if ($or$cond) {
   $32 = HEAP32[$tErrno>>2]|0;
   $33 = HEAP32[$pFile>>2]|0;
   $34 = (($33) + 20|0);
   HEAP32[$34>>2] = $32;
  }
 }
 $35 = HEAP32[$rc>>2]|0;
 HEAP32[$0>>2] = $35;
 $41 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($41|0);
}
function _dotlockUnlock($id,$eFileLock) {
 $id = $id|0;
 $eFileLock = $eFileLock|0;
 var $$ = 0, $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0;
 var $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0;
 var $44 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $or$cond = 0, $pFile = 0, $rc = 0, $tErrno = 0, $zLockFile = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 24|0;
 $1 = sp + 20|0;
 $2 = sp + 16|0;
 $pFile = sp + 12|0;
 $zLockFile = sp + 8|0;
 $rc = sp + 4|0;
 $tErrno = sp;
 HEAP32[$1>>2] = $id;
 HEAP32[$2>>2] = $eFileLock;
 $3 = HEAP32[$1>>2]|0;
 HEAP32[$pFile>>2] = $3;
 $4 = HEAP32[$pFile>>2]|0;
 $5 = (($4) + 24|0);
 $6 = HEAP32[$5>>2]|0;
 HEAP32[$zLockFile>>2] = $6;
 $7 = HEAP32[$pFile>>2]|0;
 $8 = (($7) + 16|0);
 $9 = HEAP8[$8>>0]|0;
 $10 = $9&255;
 $11 = HEAP32[$2>>2]|0;
 $12 = ($10|0)==($11|0);
 if ($12) {
  HEAP32[$0>>2] = 0;
  $44 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($44|0);
 }
 $13 = HEAP32[$2>>2]|0;
 $14 = ($13|0)==(1);
 if ($14) {
  $15 = HEAP32[$pFile>>2]|0;
  $16 = (($15) + 16|0);
  HEAP8[$16>>0] = 1;
  HEAP32[$0>>2] = 0;
  $44 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($44|0);
 }
 $17 = HEAP32[((37192 + 232|0))>>2]|0;
 $18 = HEAP32[$zLockFile>>2]|0;
 $19 = (FUNCTION_TABLE_ii[$17 & 31]($18)|0);
 HEAP32[$rc>>2] = $19;
 $20 = HEAP32[$rc>>2]|0;
 $21 = ($20|0)<(0);
 if ($21) {
  $22 = (___errno_location()|0);
  $23 = HEAP32[$22>>2]|0;
  $24 = ($23|0)==(20);
  if ($24) {
   $25 = HEAP32[((37192 + 196|0))>>2]|0;
   $26 = HEAP32[$zLockFile>>2]|0;
   $27 = (FUNCTION_TABLE_ii[$25 & 31]($26)|0);
   HEAP32[$rc>>2] = $27;
  }
 }
 $28 = HEAP32[$rc>>2]|0;
 $29 = ($28|0)<(0);
 if (!($29)) {
  $42 = HEAP32[$pFile>>2]|0;
  $43 = (($42) + 16|0);
  HEAP8[$43>>0] = 0;
  HEAP32[$0>>2] = 0;
  $44 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($44|0);
 }
 $30 = (___errno_location()|0);
 $31 = HEAP32[$30>>2]|0;
 HEAP32[$tErrno>>2] = $31;
 HEAP32[$rc>>2] = 0;
 $32 = HEAP32[$tErrno>>2]|0;
 $33 = (2)!=($32|0);
 $$ = $33 ? 2058 : 0;
 HEAP32[$rc>>2] = $$;
 $34 = HEAP32[$rc>>2]|0;
 $35 = ($34|0)!=(0);
 $36 = HEAP32[$rc>>2]|0;
 $37 = ($36|0)!=(5);
 $or$cond = $35 & $37;
 if ($or$cond) {
  $38 = HEAP32[$tErrno>>2]|0;
  $39 = HEAP32[$pFile>>2]|0;
  $40 = (($39) + 20|0);
  HEAP32[$40>>2] = $38;
 }
 $41 = HEAP32[$rc>>2]|0;
 HEAP32[$0>>2] = $41;
 $44 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($44|0);
}
function _dotlockCheckReservedLock($id,$pResOut) {
 $id = $id|0;
 $pResOut = $pResOut|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $pFile = 0;
 var $rc = 0, $reserved = 0, $zLockFile = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 20|0;
 $1 = sp + 16|0;
 $rc = sp + 12|0;
 $reserved = sp + 8|0;
 $pFile = sp + 4|0;
 $zLockFile = sp;
 HEAP32[$0>>2] = $id;
 HEAP32[$1>>2] = $pResOut;
 HEAP32[$rc>>2] = 0;
 HEAP32[$reserved>>2] = 0;
 $2 = HEAP32[$0>>2]|0;
 HEAP32[$pFile>>2] = $2;
 $3 = HEAP32[$pFile>>2]|0;
 $4 = (($3) + 16|0);
 $5 = HEAP8[$4>>0]|0;
 $6 = $5&255;
 $7 = ($6|0)>(1);
 if ($7) {
  HEAP32[$reserved>>2] = 1;
  $16 = HEAP32[$reserved>>2]|0;
  $17 = HEAP32[$1>>2]|0;
  HEAP32[$17>>2] = $16;
  $18 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($18|0);
 } else {
  $8 = HEAP32[$pFile>>2]|0;
  $9 = (($8) + 24|0);
  $10 = HEAP32[$9>>2]|0;
  HEAP32[$zLockFile>>2] = $10;
  $11 = HEAP32[((37192 + 28|0))>>2]|0;
  $12 = HEAP32[$zLockFile>>2]|0;
  $13 = (FUNCTION_TABLE_iii[$11 & 63]($12,0)|0);
  $14 = ($13|0)==(0);
  $15 = $14&1;
  HEAP32[$reserved>>2] = $15;
  $16 = HEAP32[$reserved>>2]|0;
  $17 = HEAP32[$1>>2]|0;
  HEAP32[$17>>2] = $16;
  $18 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($18|0);
 }
 return 0|0;
}
function _unixFileControl($id,$op,$pArg) {
 $id = $id|0;
 $op = $op|0;
 $pArg = $pArg|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0;
 var $pFile = 0, $rc = 0, $vararg_buffer = 0, $zTFile = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $vararg_buffer = sp;
 $0 = sp + 28|0;
 $1 = sp + 24|0;
 $2 = sp + 20|0;
 $3 = sp + 16|0;
 $pFile = sp + 12|0;
 $rc = sp + 8|0;
 $zTFile = sp + 4|0;
 HEAP32[$1>>2] = $id;
 HEAP32[$2>>2] = $op;
 HEAP32[$3>>2] = $pArg;
 $4 = HEAP32[$1>>2]|0;
 HEAP32[$pFile>>2] = $4;
 $5 = HEAP32[$2>>2]|0;
 do {
  switch ($5|0) {
  case 16:  {
   $40 = HEAP32[$pFile>>2]|0;
   $41 = (($40) + 4|0);
   $42 = HEAP32[$41>>2]|0;
   $43 = (($42) + 8|0);
   $44 = HEAP32[$43>>2]|0;
   $45 = (_sqlite3_malloc($44)|0);
   HEAP32[$zTFile>>2] = $45;
   $46 = HEAP32[$zTFile>>2]|0;
   $47 = ($46|0)!=(0|0);
   if ($47) {
    $48 = HEAP32[$pFile>>2]|0;
    $49 = (($48) + 4|0);
    $50 = HEAP32[$49>>2]|0;
    $51 = (($50) + 8|0);
    $52 = HEAP32[$51>>2]|0;
    $53 = HEAP32[$zTFile>>2]|0;
    (_unixGetTempname($52,$53)|0);
    $54 = HEAP32[$zTFile>>2]|0;
    $55 = HEAP32[$3>>2]|0;
    HEAP32[$55>>2] = $54;
   }
   HEAP32[$0>>2] = 0;
   break;
  }
  case 1:  {
   $6 = HEAP32[$pFile>>2]|0;
   $7 = (($6) + 16|0);
   $8 = HEAP8[$7>>0]|0;
   $9 = $8&255;
   $10 = HEAP32[$3>>2]|0;
   HEAP32[$10>>2] = $9;
   HEAP32[$0>>2] = 0;
   break;
  }
  case 12:  {
   $33 = HEAP32[$pFile>>2]|0;
   $34 = (($33) + 4|0);
   $35 = HEAP32[$34>>2]|0;
   $36 = (($35) + 16|0);
   $37 = HEAP32[$36>>2]|0;
   HEAP32[$vararg_buffer>>2] = $37;
   $38 = (_sqlite3_mprintf(1200,$vararg_buffer)|0);
   $39 = HEAP32[$3>>2]|0;
   HEAP32[$39>>2] = $38;
   HEAP32[$0>>2] = 0;
   break;
  }
  case 20:  {
   $56 = HEAP32[$pFile>>2]|0;
   $57 = (_fileHasMoved($56)|0);
   $58 = HEAP32[$3>>2]|0;
   HEAP32[$58>>2] = $57;
   HEAP32[$0>>2] = 0;
   break;
  }
  case 4:  {
   $11 = HEAP32[$pFile>>2]|0;
   $12 = (($11) + 20|0);
   $13 = HEAP32[$12>>2]|0;
   $14 = HEAP32[$3>>2]|0;
   HEAP32[$14>>2] = $13;
   HEAP32[$0>>2] = 0;
   break;
  }
  case 5:  {
   $19 = HEAP32[$pFile>>2]|0;
   $20 = HEAP32[$3>>2]|0;
   $21 = $20;
   $22 = $21;
   $23 = HEAP32[$22>>2]|0;
   $24 = (($21) + 4)|0;
   $25 = $24;
   $26 = HEAP32[$25>>2]|0;
   $27 = (_fcntlSizeHint($19,$23,$26)|0);
   HEAP32[$rc>>2] = $27;
   $28 = HEAP32[$rc>>2]|0;
   HEAP32[$0>>2] = $28;
   break;
  }
  case 10:  {
   $29 = HEAP32[$pFile>>2]|0;
   $30 = HEAP32[$3>>2]|0;
   _unixModeBit($29,4,$30);
   HEAP32[$0>>2] = 0;
   break;
  }
  case 13:  {
   $31 = HEAP32[$pFile>>2]|0;
   $32 = HEAP32[$3>>2]|0;
   _unixModeBit($31,16,$32);
   HEAP32[$0>>2] = 0;
   break;
  }
  case 6:  {
   $15 = HEAP32[$3>>2]|0;
   $16 = HEAP32[$15>>2]|0;
   $17 = HEAP32[$pFile>>2]|0;
   $18 = (($17) + 40|0);
   HEAP32[$18>>2] = $16;
   HEAP32[$0>>2] = 0;
   break;
  }
  default: {
   HEAP32[$0>>2] = 12;
  }
  }
 } while(0);
 $59 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($59|0);
}
function _unixSectorSize($NotUsed) {
 $NotUsed = $NotUsed|0;
 var $0 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 HEAP32[$0>>2] = $NotUsed;
 STACKTOP = sp;return 4096;
}
function _unixDeviceCharacteristics($id) {
 $id = $id|0;
 var $0 = 0, $1 = 0, $10 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $p = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $p = sp + 4|0;
 $rc = sp;
 HEAP32[$0>>2] = $id;
 $1 = HEAP32[$0>>2]|0;
 HEAP32[$p>>2] = $1;
 HEAP32[$rc>>2] = 0;
 $2 = HEAP32[$p>>2]|0;
 $3 = (($2) + 18|0);
 $4 = HEAP16[$3>>1]|0;
 $5 = $4&65535;
 $6 = $5 & 16;
 $7 = ($6|0)!=(0);
 if (!($7)) {
  $10 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($10|0);
 }
 $8 = HEAP32[$rc>>2]|0;
 $9 = $8 | 4096;
 HEAP32[$rc>>2] = $9;
 $10 = HEAP32[$rc>>2]|0;
 STACKTOP = sp;return ($10|0);
}
function _unixShmLock($fd,$ofst,$n,$flags) {
 $fd = $fd|0;
 $ofst = $ofst|0;
 $n = $n|0;
 $flags = $flags|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0;
 var $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0;
 var $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0;
 var $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0;
 var $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0;
 var $97 = 0, $98 = 0, $99 = 0, $allMask = 0, $allShared = 0, $mask = 0, $p = 0, $pDbFd = 0, $pShmNode = 0, $pX = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $0 = sp + 32|0;
 $1 = sp + 28|0;
 $2 = sp + 24|0;
 $3 = sp + 20|0;
 $pDbFd = sp + 16|0;
 $p = sp + 12|0;
 $pX = sp + 8|0;
 $pShmNode = sp + 4|0;
 $rc = sp;
 $mask = sp + 40|0;
 $allMask = sp + 38|0;
 $allShared = sp + 36|0;
 HEAP32[$0>>2] = $fd;
 HEAP32[$1>>2] = $ofst;
 HEAP32[$2>>2] = $n;
 HEAP32[$3>>2] = $flags;
 $4 = HEAP32[$0>>2]|0;
 HEAP32[$pDbFd>>2] = $4;
 $5 = HEAP32[$pDbFd>>2]|0;
 $6 = (($5) + 36|0);
 $7 = HEAP32[$6>>2]|0;
 HEAP32[$p>>2] = $7;
 $8 = HEAP32[$p>>2]|0;
 $9 = HEAP32[$8>>2]|0;
 HEAP32[$pShmNode>>2] = $9;
 HEAP32[$rc>>2] = 0;
 $10 = HEAP32[$1>>2]|0;
 $11 = HEAP32[$2>>2]|0;
 $12 = (($10) + ($11))|0;
 $13 = 1 << $12;
 $14 = HEAP32[$1>>2]|0;
 $15 = 1 << $14;
 $16 = (($13) - ($15))|0;
 $17 = $16&65535;
 HEAP16[$mask>>1] = $17;
 $18 = HEAP32[$3>>2]|0;
 $19 = $18 & 1;
 $20 = ($19|0)!=(0);
 if ($20) {
  HEAP16[$allMask>>1] = 0;
  $21 = HEAP32[$pShmNode>>2]|0;
  $22 = (($21) + 32|0);
  $23 = HEAP32[$22>>2]|0;
  HEAP32[$pX>>2] = $23;
  while(1) {
   $24 = HEAP32[$pX>>2]|0;
   $25 = ($24|0)!=(0|0);
   if (!($25)) {
    break;
   }
   $26 = HEAP32[$pX>>2]|0;
   $27 = HEAP32[$p>>2]|0;
   $28 = ($26|0)==($27|0);
   if (!($28)) {
    $29 = HEAP32[$pX>>2]|0;
    $30 = (($29) + 10|0);
    $31 = HEAP16[$30>>1]|0;
    $32 = $31&65535;
    $33 = HEAP16[$allMask>>1]|0;
    $34 = $33&65535;
    $35 = $34 | $32;
    $36 = $35&65535;
    HEAP16[$allMask>>1] = $36;
   }
   $37 = HEAP32[$pX>>2]|0;
   $38 = (($37) + 4|0);
   $39 = HEAP32[$38>>2]|0;
   HEAP32[$pX>>2] = $39;
  }
  $40 = HEAP16[$mask>>1]|0;
  $41 = $40&65535;
  $42 = HEAP16[$allMask>>1]|0;
  $43 = $42&65535;
  $44 = $41 & $43;
  $45 = ($44|0)==(0);
  if ($45) {
   $46 = HEAP32[$pShmNode>>2]|0;
   $47 = HEAP32[$1>>2]|0;
   $48 = (($47) + 120)|0;
   $49 = HEAP32[$2>>2]|0;
   $50 = (_unixShmSystemLock($46,2,$48,$49)|0);
   HEAP32[$rc>>2] = $50;
  } else {
   HEAP32[$rc>>2] = 0;
  }
  $51 = HEAP32[$rc>>2]|0;
  $52 = ($51|0)==(0);
  if (!($52)) {
   $162 = HEAP32[$rc>>2]|0;
   STACKTOP = sp;return ($162|0);
  }
  $53 = HEAP16[$mask>>1]|0;
  $54 = $53&65535;
  $55 = $54 ^ -1;
  $56 = HEAP32[$p>>2]|0;
  $57 = (($56) + 12|0);
  $58 = HEAP16[$57>>1]|0;
  $59 = $58&65535;
  $60 = $59 & $55;
  $61 = $60&65535;
  HEAP16[$57>>1] = $61;
  $62 = HEAP16[$mask>>1]|0;
  $63 = $62&65535;
  $64 = $63 ^ -1;
  $65 = HEAP32[$p>>2]|0;
  $66 = (($65) + 10|0);
  $67 = HEAP16[$66>>1]|0;
  $68 = $67&65535;
  $69 = $68 & $64;
  $70 = $69&65535;
  HEAP16[$66>>1] = $70;
  $162 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($162|0);
 }
 $71 = HEAP32[$3>>2]|0;
 $72 = $71 & 4;
 $73 = ($72|0)!=(0);
 if (!($73)) {
  $121 = HEAP32[$pShmNode>>2]|0;
  $122 = (($121) + 32|0);
  $123 = HEAP32[$122>>2]|0;
  HEAP32[$pX>>2] = $123;
  while(1) {
   $124 = HEAP32[$pX>>2]|0;
   $125 = ($124|0)!=(0|0);
   if (!($125)) {
    break;
   }
   $126 = HEAP32[$pX>>2]|0;
   $127 = (($126) + 12|0);
   $128 = HEAP16[$127>>1]|0;
   $129 = $128&65535;
   $130 = HEAP16[$mask>>1]|0;
   $131 = $130&65535;
   $132 = $129 & $131;
   $133 = ($132|0)!=(0);
   if ($133) {
    label = 28;
    break;
   }
   $134 = HEAP32[$pX>>2]|0;
   $135 = (($134) + 10|0);
   $136 = HEAP16[$135>>1]|0;
   $137 = $136&65535;
   $138 = HEAP16[$mask>>1]|0;
   $139 = $138&65535;
   $140 = $137 & $139;
   $141 = ($140|0)!=(0);
   if ($141) {
    label = 28;
    break;
   }
   $142 = HEAP32[$pX>>2]|0;
   $143 = (($142) + 4|0);
   $144 = HEAP32[$143>>2]|0;
   HEAP32[$pX>>2] = $144;
  }
  if ((label|0) == 28) {
   HEAP32[$rc>>2] = 5;
  }
  $145 = HEAP32[$rc>>2]|0;
  $146 = ($145|0)==(0);
  if (!($146)) {
   $162 = HEAP32[$rc>>2]|0;
   STACKTOP = sp;return ($162|0);
  }
  $147 = HEAP32[$pShmNode>>2]|0;
  $148 = HEAP32[$1>>2]|0;
  $149 = (($148) + 120)|0;
  $150 = HEAP32[$2>>2]|0;
  $151 = (_unixShmSystemLock($147,1,$149,$150)|0);
  HEAP32[$rc>>2] = $151;
  $152 = HEAP32[$rc>>2]|0;
  $153 = ($152|0)==(0);
  if (!($153)) {
   $162 = HEAP32[$rc>>2]|0;
   STACKTOP = sp;return ($162|0);
  }
  $154 = HEAP16[$mask>>1]|0;
  $155 = $154&65535;
  $156 = HEAP32[$p>>2]|0;
  $157 = (($156) + 12|0);
  $158 = HEAP16[$157>>1]|0;
  $159 = $158&65535;
  $160 = $159 | $155;
  $161 = $160&65535;
  HEAP16[$157>>1] = $161;
  $162 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($162|0);
 }
 HEAP16[$allShared>>1] = 0;
 $74 = HEAP32[$pShmNode>>2]|0;
 $75 = (($74) + 32|0);
 $76 = HEAP32[$75>>2]|0;
 HEAP32[$pX>>2] = $76;
 while(1) {
  $77 = HEAP32[$pX>>2]|0;
  $78 = ($77|0)!=(0|0);
  if (!($78)) {
   break;
  }
  $79 = HEAP32[$pX>>2]|0;
  $80 = (($79) + 12|0);
  $81 = HEAP16[$80>>1]|0;
  $82 = $81&65535;
  $83 = HEAP16[$mask>>1]|0;
  $84 = $83&65535;
  $85 = $82 & $84;
  $86 = ($85|0)!=(0);
  if ($86) {
   label = 16;
   break;
  }
  $87 = HEAP32[$pX>>2]|0;
  $88 = (($87) + 10|0);
  $89 = HEAP16[$88>>1]|0;
  $90 = $89&65535;
  $91 = HEAP16[$allShared>>1]|0;
  $92 = $91&65535;
  $93 = $92 | $90;
  $94 = $93&65535;
  HEAP16[$allShared>>1] = $94;
  $95 = HEAP32[$pX>>2]|0;
  $96 = (($95) + 4|0);
  $97 = HEAP32[$96>>2]|0;
  HEAP32[$pX>>2] = $97;
 }
 if ((label|0) == 16) {
  HEAP32[$rc>>2] = 5;
 }
 $98 = HEAP32[$rc>>2]|0;
 $99 = ($98|0)==(0);
 do {
  if ($99) {
   $100 = HEAP16[$allShared>>1]|0;
   $101 = $100&65535;
   $102 = HEAP16[$mask>>1]|0;
   $103 = $102&65535;
   $104 = $101 & $103;
   $105 = ($104|0)==(0);
   if ($105) {
    $106 = HEAP32[$pShmNode>>2]|0;
    $107 = HEAP32[$1>>2]|0;
    $108 = (($107) + 120)|0;
    $109 = HEAP32[$2>>2]|0;
    $110 = (_unixShmSystemLock($106,0,$108,$109)|0);
    HEAP32[$rc>>2] = $110;
    break;
   } else {
    HEAP32[$rc>>2] = 0;
    break;
   }
  }
 } while(0);
 $111 = HEAP32[$rc>>2]|0;
 $112 = ($111|0)==(0);
 if (!($112)) {
  $162 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($162|0);
 }
 $113 = HEAP16[$mask>>1]|0;
 $114 = $113&65535;
 $115 = HEAP32[$p>>2]|0;
 $116 = (($115) + 10|0);
 $117 = HEAP16[$116>>1]|0;
 $118 = $117&65535;
 $119 = $118 | $114;
 $120 = $119&65535;
 HEAP16[$116>>1] = $120;
 $162 = HEAP32[$rc>>2]|0;
 STACKTOP = sp;return ($162|0);
}
function _unixShmBarrier($fd) {
 $fd = $fd|0;
 var $0 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 HEAP32[$0>>2] = $fd;
 _unixEnterMutex();
 _unixLeaveMutex();
 STACKTOP = sp;return;
}
function _unixShmUnmap($fd,$deleteFlag) {
 $fd = $fd|0;
 $deleteFlag = $deleteFlag|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $p = 0, $pDbFd = 0, $pShmNode = 0, $pp = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 24|0;
 $1 = sp + 20|0;
 $2 = sp + 16|0;
 $p = sp + 12|0;
 $pShmNode = sp + 8|0;
 $pp = sp + 4|0;
 $pDbFd = sp;
 HEAP32[$1>>2] = $fd;
 HEAP32[$2>>2] = $deleteFlag;
 $3 = HEAP32[$1>>2]|0;
 HEAP32[$pDbFd>>2] = $3;
 $4 = HEAP32[$pDbFd>>2]|0;
 $5 = (($4) + 36|0);
 $6 = HEAP32[$5>>2]|0;
 HEAP32[$p>>2] = $6;
 $7 = HEAP32[$p>>2]|0;
 $8 = ($7|0)==(0|0);
 if ($8) {
  HEAP32[$0>>2] = 0;
  $46 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($46|0);
 }
 $9 = HEAP32[$p>>2]|0;
 $10 = HEAP32[$9>>2]|0;
 HEAP32[$pShmNode>>2] = $10;
 $11 = HEAP32[$pShmNode>>2]|0;
 $12 = (($11) + 32|0);
 HEAP32[$pp>>2] = $12;
 while(1) {
  $13 = HEAP32[$pp>>2]|0;
  $14 = HEAP32[$13>>2]|0;
  $15 = HEAP32[$p>>2]|0;
  $16 = ($14|0)!=($15|0);
  if (!($16)) {
   break;
  }
  $17 = HEAP32[$pp>>2]|0;
  $18 = HEAP32[$17>>2]|0;
  $19 = (($18) + 4|0);
  HEAP32[$pp>>2] = $19;
 }
 $20 = HEAP32[$p>>2]|0;
 $21 = (($20) + 4|0);
 $22 = HEAP32[$21>>2]|0;
 $23 = HEAP32[$pp>>2]|0;
 HEAP32[$23>>2] = $22;
 $24 = HEAP32[$p>>2]|0;
 _sqlite3_free($24);
 $25 = HEAP32[$pDbFd>>2]|0;
 $26 = (($25) + 36|0);
 HEAP32[$26>>2] = 0;
 _unixEnterMutex();
 $27 = HEAP32[$pShmNode>>2]|0;
 $28 = (($27) + 28|0);
 $29 = HEAP32[$28>>2]|0;
 $30 = (($29) + -1)|0;
 HEAP32[$28>>2] = $30;
 $31 = HEAP32[$pShmNode>>2]|0;
 $32 = (($31) + 28|0);
 $33 = HEAP32[$32>>2]|0;
 $34 = ($33|0)==(0);
 if ($34) {
  $35 = HEAP32[$2>>2]|0;
  $36 = ($35|0)!=(0);
  if ($36) {
   $37 = HEAP32[$pShmNode>>2]|0;
   $38 = (($37) + 12|0);
   $39 = HEAP32[$38>>2]|0;
   $40 = ($39|0)>=(0);
   if ($40) {
    $41 = HEAP32[((37192 + 196|0))>>2]|0;
    $42 = HEAP32[$pShmNode>>2]|0;
    $43 = (($42) + 8|0);
    $44 = HEAP32[$43>>2]|0;
    (FUNCTION_TABLE_ii[$41 & 31]($44)|0);
   }
  }
  $45 = HEAP32[$pDbFd>>2]|0;
  _unixShmPurge($45);
 }
 _unixLeaveMutex();
 HEAP32[$0>>2] = 0;
 $46 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($46|0);
}
function _unixFetch($fd,$0,$1,$nAmt,$pp) {
 $fd = $fd|0;
 $0 = $0|0;
 $1 = $1|0;
 $nAmt = $nAmt|0;
 $pp = $pp|0;
 var $10 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $2 = sp + 16|0;
 $3 = sp;
 $4 = sp + 12|0;
 $5 = sp + 8|0;
 HEAP32[$2>>2] = $fd;
 $6 = $3;
 $7 = $6;
 HEAP32[$7>>2] = $0;
 $8 = (($6) + 4)|0;
 $9 = $8;
 HEAP32[$9>>2] = $1;
 HEAP32[$4>>2] = $nAmt;
 HEAP32[$5>>2] = $pp;
 $10 = HEAP32[$5>>2]|0;
 HEAP32[$10>>2] = 0;
 STACKTOP = sp;return 0;
}
function _unixUnfetch($fd,$0,$1,$p) {
 $fd = $fd|0;
 $0 = $0|0;
 $1 = $1|0;
 $p = $p|0;
 var $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $2 = sp + 12|0;
 $3 = sp;
 $4 = sp + 8|0;
 HEAP32[$2>>2] = $fd;
 $5 = $3;
 $6 = $5;
 HEAP32[$6>>2] = $0;
 $7 = (($5) + 4)|0;
 $8 = $7;
 HEAP32[$8>>2] = $1;
 HEAP32[$4>>2] = $p;
 STACKTOP = sp;return 0;
}
function _unixEnterMutex() {
 var label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = sp;return;
}
function _unixShmPurge($pFd) {
 $pFd = $pFd|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $i = 0, $nShmPerMap = 0;
 var $p = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $p = sp + 8|0;
 $nShmPerMap = sp + 4|0;
 $i = sp;
 HEAP32[$0>>2] = $pFd;
 $1 = HEAP32[$0>>2]|0;
 $2 = (($1) + 8|0);
 $3 = HEAP32[$2>>2]|0;
 $4 = (($3) + 20|0);
 $5 = HEAP32[$4>>2]|0;
 HEAP32[$p>>2] = $5;
 $6 = HEAP32[$p>>2]|0;
 $7 = ($6|0)!=(0|0);
 if (!($7)) {
  STACKTOP = sp;return;
 }
 $8 = HEAP32[$p>>2]|0;
 $9 = (($8) + 28|0);
 $10 = HEAP32[$9>>2]|0;
 $11 = ($10|0)==(0);
 if (!($11)) {
  STACKTOP = sp;return;
 }
 $12 = (_unixShmRegionPerMap()|0);
 HEAP32[$nShmPerMap>>2] = $12;
 HEAP32[$i>>2] = 0;
 while(1) {
  $13 = HEAP32[$i>>2]|0;
  $14 = HEAP32[$p>>2]|0;
  $15 = (($14) + 20|0);
  $16 = HEAP16[$15>>1]|0;
  $17 = $16&65535;
  $18 = ($13|0)<($17|0);
  $19 = HEAP32[$p>>2]|0;
  if (!($18)) {
   break;
  }
  $20 = (($19) + 12|0);
  $21 = HEAP32[$20>>2]|0;
  $22 = ($21|0)>=(0);
  if ($22) {
   $23 = HEAP32[((37192 + 268|0))>>2]|0;
   $24 = HEAP32[$i>>2]|0;
   $25 = HEAP32[$p>>2]|0;
   $26 = (($25) + 24|0);
   $27 = HEAP32[$26>>2]|0;
   $28 = (($27) + ($24<<2)|0);
   $29 = HEAP32[$28>>2]|0;
   $30 = HEAP32[$p>>2]|0;
   $31 = (($30) + 16|0);
   $32 = HEAP32[$31>>2]|0;
   (FUNCTION_TABLE_iii[$23 & 63]($29,$32)|0);
  } else {
   $33 = HEAP32[$i>>2]|0;
   $34 = HEAP32[$p>>2]|0;
   $35 = (($34) + 24|0);
   $36 = HEAP32[$35>>2]|0;
   $37 = (($36) + ($33<<2)|0);
   $38 = HEAP32[$37>>2]|0;
   _sqlite3_free($38);
  }
  $39 = HEAP32[$nShmPerMap>>2]|0;
  $40 = HEAP32[$i>>2]|0;
  $41 = (($40) + ($39))|0;
  HEAP32[$i>>2] = $41;
 }
 $42 = (($19) + 24|0);
 $43 = HEAP32[$42>>2]|0;
 _sqlite3_free($43);
 $44 = HEAP32[$p>>2]|0;
 $45 = (($44) + 12|0);
 $46 = HEAP32[$45>>2]|0;
 $47 = ($46|0)>=(0);
 if ($47) {
  $48 = HEAP32[$0>>2]|0;
  $49 = HEAP32[$p>>2]|0;
  $50 = (($49) + 12|0);
  $51 = HEAP32[$50>>2]|0;
  _robust_close($48,$51,28999);
  $52 = HEAP32[$p>>2]|0;
  $53 = (($52) + 12|0);
  HEAP32[$53>>2] = -1;
 }
 $54 = HEAP32[$p>>2]|0;
 $55 = HEAP32[$54>>2]|0;
 $56 = (($55) + 20|0);
 HEAP32[$56>>2] = 0;
 $57 = HEAP32[$p>>2]|0;
 _sqlite3_free($57);
 STACKTOP = sp;return;
}
function _unixLeaveMutex() {
 var label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = sp;return;
}
function _unixShmRegionPerMap() {
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $pgsz = 0, $shmsz = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $shmsz = sp + 4|0;
 $pgsz = sp;
 HEAP32[$shmsz>>2] = 32768;
 $1 = HEAP32[((37192 + 292|0))>>2]|0;
 $2 = (FUNCTION_TABLE_i[$1 & 1]()|0);
 HEAP32[$pgsz>>2] = $2;
 $3 = HEAP32[$pgsz>>2]|0;
 $4 = HEAP32[$shmsz>>2]|0;
 $5 = ($3|0)<($4|0);
 if ($5) {
  HEAP32[$0>>2] = 1;
  $9 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($9|0);
 } else {
  $6 = HEAP32[$pgsz>>2]|0;
  $7 = HEAP32[$shmsz>>2]|0;
  $8 = (($6|0) / ($7|0))&-1;
  HEAP32[$0>>2] = $8;
  $9 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($9|0);
 }
 return 0|0;
}
function _robust_close($pFile,$h,$lineno) {
 $pFile = $pFile|0;
 $h = $h|0;
 $lineno = $lineno|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $1 = sp + 4|0;
 $2 = sp;
 HEAP32[$0>>2] = $pFile;
 HEAP32[$1>>2] = $h;
 HEAP32[$2>>2] = $lineno;
 $3 = HEAP32[((37192 + 16|0))>>2]|0;
 $4 = HEAP32[$1>>2]|0;
 $5 = (FUNCTION_TABLE_ii[$3 & 31]($4)|0);
 $6 = ($5|0)!=(0);
 if (!($6)) {
  STACKTOP = sp;return;
 }
 $7 = HEAP32[$0>>2]|0;
 $8 = ($7|0)!=(0|0);
 if ($8) {
  $9 = HEAP32[$0>>2]|0;
  $10 = (($9) + 32|0);
  $11 = HEAP32[$10>>2]|0;
  $13 = $11;
 } else {
  $13 = 0;
 }
 $12 = HEAP32[$2>>2]|0;
 (_unixLogErrorAtLine(4106,37496,$13,$12)|0);
 STACKTOP = sp;return;
}
function _unixLogErrorAtLine($errcode,$zFunc,$zPath,$iLine) {
 $errcode = $errcode|0;
 $zFunc = $zFunc|0;
 $zPath = $zPath|0;
 $iLine = $iLine|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $iErrno = 0, $vararg_buffer = 0, $vararg_ptr1 = 0;
 var $vararg_ptr2 = 0, $vararg_ptr3 = 0, $vararg_ptr4 = 0, $zErr = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $vararg_buffer = sp;
 $0 = sp + 40|0;
 $1 = sp + 36|0;
 $2 = sp + 32|0;
 $3 = sp + 28|0;
 $zErr = sp + 24|0;
 $iErrno = sp + 20|0;
 HEAP32[$0>>2] = $errcode;
 HEAP32[$1>>2] = $zFunc;
 HEAP32[$2>>2] = $zPath;
 HEAP32[$3>>2] = $iLine;
 $4 = (___errno_location()|0);
 $5 = HEAP32[$4>>2]|0;
 HEAP32[$iErrno>>2] = $5;
 $6 = HEAP32[$iErrno>>2]|0;
 $7 = (_strerror(($6|0))|0);
 HEAP32[$zErr>>2] = $7;
 $8 = HEAP32[$2>>2]|0;
 $9 = ($8|0)==(0|0);
 if ($9) {
  HEAP32[$2>>2] = 1216;
 }
 $10 = HEAP32[$0>>2]|0;
 $11 = HEAP32[$3>>2]|0;
 $12 = HEAP32[$iErrno>>2]|0;
 $13 = HEAP32[$1>>2]|0;
 $14 = HEAP32[$2>>2]|0;
 $15 = HEAP32[$zErr>>2]|0;
 HEAP32[$vararg_buffer>>2] = $11;
 $vararg_ptr1 = (($vararg_buffer) + 4|0);
 HEAP32[$vararg_ptr1>>2] = $12;
 $vararg_ptr2 = (($vararg_buffer) + 8|0);
 HEAP32[$vararg_ptr2>>2] = $13;
 $vararg_ptr3 = (($vararg_buffer) + 12|0);
 HEAP32[$vararg_ptr3>>2] = $14;
 $vararg_ptr4 = (($vararg_buffer) + 16|0);
 HEAP32[$vararg_ptr4>>2] = $15;
 _sqlite3_log($10,37504,$vararg_buffer);
 $16 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($16|0);
}
function _posixOpen($zFile,$flags,$mode) {
 $zFile = $zFile|0;
 $flags = $flags|0;
 $mode = $mode|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $vararg_buffer = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $vararg_buffer = sp;
 $0 = sp + 12|0;
 $1 = sp + 8|0;
 $2 = sp + 4|0;
 HEAP32[$0>>2] = $zFile;
 HEAP32[$1>>2] = $flags;
 HEAP32[$2>>2] = $mode;
 $3 = HEAP32[$0>>2]|0;
 $4 = HEAP32[$1>>2]|0;
 $5 = HEAP32[$2>>2]|0;
 HEAP32[$vararg_buffer>>2] = $5;
 $6 = (_open(($3|0),($4|0),($vararg_buffer|0))|0);
 STACKTOP = sp;return ($6|0);
}
function _openDirectory($zFilename,$pFd) {
 $zFilename = $zFilename|0;
 $pFd = $pFd|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $3 = 0, $4 = 0;
 var $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $fd = 0, $ii = 0, $vararg_buffer = 0, $zDirname = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 544|0;
 $vararg_buffer = sp;
 $0 = sp + 16|0;
 $1 = sp + 12|0;
 $ii = sp + 8|0;
 $fd = sp + 4|0;
 $zDirname = sp + 20|0;
 HEAP32[$0>>2] = $zFilename;
 HEAP32[$1>>2] = $pFd;
 HEAP32[$fd>>2] = -1;
 $2 = HEAP32[$0>>2]|0;
 HEAP32[$vararg_buffer>>2] = $2;
 (_sqlite3_snprintf(512,$zDirname,1200,$vararg_buffer)|0);
 $3 = (_strlen(($zDirname|0))|0);
 HEAP32[$ii>>2] = $3;
 while(1) {
  $4 = HEAP32[$ii>>2]|0;
  $5 = ($4|0)>(1);
  if (!($5)) {
   break;
  }
  $6 = HEAP32[$ii>>2]|0;
  $7 = (($zDirname) + ($6)|0);
  $8 = HEAP8[$7>>0]|0;
  $9 = $8 << 24 >> 24;
  $10 = ($9|0)!=(47);
  if (!($10)) {
   break;
  }
  $11 = HEAP32[$ii>>2]|0;
  $12 = (($11) + -1)|0;
  HEAP32[$ii>>2] = $12;
 }
 $13 = HEAP32[$ii>>2]|0;
 $14 = ($13|0)>(0);
 if ($14) {
  $15 = HEAP32[$ii>>2]|0;
  $16 = (($zDirname) + ($15)|0);
  HEAP8[$16>>0] = 0;
  $17 = (_robust_open($zDirname,0,0)|0);
  HEAP32[$fd>>2] = $17;
 }
 $18 = HEAP32[$fd>>2]|0;
 $19 = HEAP32[$1>>2]|0;
 HEAP32[$19>>2] = $18;
 $20 = HEAP32[$fd>>2]|0;
 $21 = ($20|0)>=(0);
 if ($21) {
  $24 = 0;
  STACKTOP = sp;return ($24|0);
 }
 $22 = (_sqlite3CantopenError(28344)|0);
 $23 = (_unixLogErrorAtLine($22,37536,$zDirname,28344)|0);
 $24 = $23;
 STACKTOP = sp;return ($24|0);
}
function _posixFchown($fd,$uid,$gid) {
 $fd = $fd|0;
 $uid = $uid|0;
 $gid = $gid|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $1 = sp + 4|0;
 $2 = sp;
 HEAP32[$0>>2] = $fd;
 HEAP32[$1>>2] = $uid;
 HEAP32[$2>>2] = $gid;
 $3 = (_geteuid()|0);
 $4 = ($3|0)!=(0);
 if ($4) {
  $9 = 0;
  STACKTOP = sp;return ($9|0);
 }
 $5 = HEAP32[$0>>2]|0;
 $6 = HEAP32[$1>>2]|0;
 $7 = HEAP32[$2>>2]|0;
 $8 = (_fchown(($5|0),($6|0),($7|0))|0);
 $9 = $8;
 STACKTOP = sp;return ($9|0);
}
function _unixGetpagesize() {
 var $0 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 $0 = (_sysconf(30)|0);
 STACKTOP = sp;return ($0|0);
}
function _robust_open($z,$f,$m) {
 $z = $z|0;
 $f = $f|0;
 $m = $m|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $fd = 0, $m2 = 0, $or$cond = 0, $statbuf = 0, $vararg_buffer = 0, $vararg_ptr2 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 112|0;
 $vararg_buffer = sp;
 $0 = sp + 100|0;
 $1 = sp + 96|0;
 $2 = sp + 92|0;
 $fd = sp + 88|0;
 $m2 = sp + 84|0;
 $statbuf = sp + 8|0;
 HEAP32[$0>>2] = $z;
 HEAP32[$1>>2] = $f;
 HEAP32[$2>>2] = $m;
 $3 = HEAP32[$2>>2]|0;
 $4 = ($3|0)!=(0);
 $5 = HEAP32[$2>>2]|0;
 $6 = $4 ? $5 : 420;
 HEAP32[$m2>>2] = $6;
 while(1) {
  $7 = HEAP32[((37192 + 4|0))>>2]|0;
  $8 = HEAP32[$0>>2]|0;
  $9 = HEAP32[$1>>2]|0;
  $10 = $9 | 524288;
  $11 = HEAP32[$m2>>2]|0;
  $12 = (FUNCTION_TABLE_iiii[$7 & 31]($8,$10,$11)|0);
  HEAP32[$fd>>2] = $12;
  $13 = HEAP32[$fd>>2]|0;
  $14 = ($13|0)<(0);
  if ($14) {
   $15 = (___errno_location()|0);
   $16 = HEAP32[$15>>2]|0;
   $17 = ($16|0)==(4);
   if ($17) {
    continue;
   } else {
    break;
   }
  }
  $18 = HEAP32[$fd>>2]|0;
  $19 = ($18|0)>=(3);
  if ($19) {
   break;
  }
  $20 = HEAP32[((37192 + 16|0))>>2]|0;
  $21 = HEAP32[$fd>>2]|0;
  (FUNCTION_TABLE_ii[$20 & 31]($21)|0);
  $22 = HEAP32[$0>>2]|0;
  $23 = HEAP32[$fd>>2]|0;
  HEAP32[$vararg_buffer>>2] = $22;
  $vararg_ptr2 = (($vararg_buffer) + 4|0);
  HEAP32[$vararg_ptr2>>2] = $23;
  _sqlite3_log(28,37760,$vararg_buffer);
  HEAP32[$fd>>2] = -1;
  $24 = HEAP32[((37192 + 4|0))>>2]|0;
  $25 = HEAP32[$1>>2]|0;
  $26 = HEAP32[$2>>2]|0;
  $27 = (FUNCTION_TABLE_iiii[$24 & 31](37808,$25,$26)|0);
  $28 = ($27|0)<(0);
  if ($28) {
   break;
  }
 }
 $29 = HEAP32[$fd>>2]|0;
 $30 = ($29|0)>=(0);
 $31 = HEAP32[$2>>2]|0;
 $32 = ($31|0)!=(0);
 $or$cond = $30 & $32;
 if (!($or$cond)) {
  $48 = HEAP32[$fd>>2]|0;
  STACKTOP = sp;return ($48|0);
 }
 $33 = HEAP32[((37192 + 64|0))>>2]|0;
 $34 = HEAP32[$fd>>2]|0;
 $35 = (FUNCTION_TABLE_iii[$33 & 63]($34,$statbuf)|0);
 $36 = ($35|0)==(0);
 if (!($36)) {
  $48 = HEAP32[$fd>>2]|0;
  STACKTOP = sp;return ($48|0);
 }
 $37 = (($statbuf) + 36|0);
 $38 = HEAP32[$37>>2]|0;
 $39 = ($38|0)==(0);
 if (!($39)) {
  $48 = HEAP32[$fd>>2]|0;
  STACKTOP = sp;return ($48|0);
 }
 $40 = (($statbuf) + 12|0);
 $41 = HEAP32[$40>>2]|0;
 $42 = $41 & 511;
 $43 = HEAP32[$2>>2]|0;
 $44 = ($42|0)!=($43|0);
 if (!($44)) {
  $48 = HEAP32[$fd>>2]|0;
  STACKTOP = sp;return ($48|0);
 }
 $45 = HEAP32[((37192 + 172|0))>>2]|0;
 $46 = HEAP32[$fd>>2]|0;
 $47 = HEAP32[$2>>2]|0;
 (FUNCTION_TABLE_iii[$45 & 63]($46,$47)|0);
 $48 = HEAP32[$fd>>2]|0;
 STACKTOP = sp;return ($48|0);
}
function _unixShmSystemLock($pShmNode,$lockType,$ofst,$n) {
 $pShmNode = $pShmNode|0;
 $lockType = $lockType|0;
 $ofst = $ofst|0;
 $n = $n|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $3 = 0, $4 = 0, $5 = 0;
 var $6 = 0, $7 = 0, $8 = 0, $9 = 0, $f = 0, $rc = 0, $vararg_buffer = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $vararg_buffer = sp;
 $0 = sp + 36|0;
 $1 = sp + 32|0;
 $2 = sp + 28|0;
 $3 = sp + 24|0;
 $f = sp + 8|0;
 $rc = sp + 4|0;
 HEAP32[$0>>2] = $pShmNode;
 HEAP32[$1>>2] = $lockType;
 HEAP32[$2>>2] = $ofst;
 HEAP32[$3>>2] = $n;
 HEAP32[$rc>>2] = 0;
 $4 = HEAP32[$0>>2]|0;
 $5 = (($4) + 12|0);
 $6 = HEAP32[$5>>2]|0;
 $7 = ($6|0)>=(0);
 if (!($7)) {
  $23 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($23|0);
 }
 ;HEAP32[$f+0>>2]=0|0;HEAP32[$f+4>>2]=0|0;HEAP32[$f+8>>2]=0|0;HEAP32[$f+12>>2]=0|0;
 $8 = HEAP32[$1>>2]|0;
 $9 = $8&65535;
 HEAP16[$f>>1] = $9;
 $10 = (($f) + 2|0);
 HEAP16[$10>>1] = 0;
 $11 = HEAP32[$2>>2]|0;
 $12 = (($f) + 4|0);
 HEAP32[$12>>2] = $11;
 $13 = HEAP32[$3>>2]|0;
 $14 = (($f) + 8|0);
 HEAP32[$14>>2] = $13;
 $15 = HEAP32[((37192 + 88|0))>>2]|0;
 $16 = HEAP32[$0>>2]|0;
 $17 = (($16) + 12|0);
 $18 = HEAP32[$17>>2]|0;
 HEAP32[$vararg_buffer>>2] = $f;
 $19 = (FUNCTION_TABLE_iiii[$15 & 31]($18,13,$vararg_buffer)|0);
 HEAP32[$rc>>2] = $19;
 $20 = HEAP32[$rc>>2]|0;
 $21 = ($20|0)!=(-1);
 $22 = $21 ? 0 : 5;
 HEAP32[$rc>>2] = $22;
 $23 = HEAP32[$rc>>2]|0;
 STACKTOP = sp;return ($23|0);
}
function _fcntlSizeHint($pFile,$0,$1) {
 $pFile = $pFile|0;
 $0 = $0|0;
 $1 = $1|0;
 var $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0, $116 = 0, $117 = 0;
 var $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0, $134 = 0, $135 = 0;
 var $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0;
 var $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0;
 var $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0;
 var $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0;
 var $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0;
 var $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $buf = 0, $iWrite = 0, $nBlk = 0, $nSize = 0, $nWrite = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 128|0;
 $2 = sp + 112|0;
 $3 = sp + 108|0;
 $4 = sp + 16|0;
 $nSize = sp + 8|0;
 $buf = sp + 32|0;
 $nBlk = sp + 28|0;
 $iWrite = sp;
 $nWrite = sp + 24|0;
 HEAP32[$3>>2] = $pFile;
 $5 = $4;
 $6 = $5;
 HEAP32[$6>>2] = $0;
 $7 = (($5) + 4)|0;
 $8 = $7;
 HEAP32[$8>>2] = $1;
 $9 = HEAP32[$3>>2]|0;
 $10 = (($9) + 40|0);
 $11 = HEAP32[$10>>2]|0;
 $12 = ($11|0)>(0);
 L1: do {
  if ($12) {
   $13 = HEAP32[((37192 + 64|0))>>2]|0;
   $14 = HEAP32[$3>>2]|0;
   $15 = (($14) + 12|0);
   $16 = HEAP32[$15>>2]|0;
   $17 = (FUNCTION_TABLE_iii[$13 & 63]($16,$buf)|0);
   $18 = ($17|0)!=(0);
   if ($18) {
    HEAP32[$2>>2] = 1802;
    $147 = HEAP32[$2>>2]|0;
    STACKTOP = sp;return ($147|0);
   }
   $19 = $4;
   $20 = $19;
   $21 = HEAP32[$20>>2]|0;
   $22 = (($19) + 4)|0;
   $23 = $22;
   $24 = HEAP32[$23>>2]|0;
   $25 = HEAP32[$3>>2]|0;
   $26 = (($25) + 40|0);
   $27 = HEAP32[$26>>2]|0;
   $28 = ($27|0)<(0);
   $29 = $28 << 31 >> 31;
   $30 = (_i64Add(($21|0),($24|0),($27|0),($29|0))|0);
   $31 = tempRet0;
   $32 = (_i64Subtract(($30|0),($31|0),1,0)|0);
   $33 = tempRet0;
   $34 = HEAP32[$3>>2]|0;
   $35 = (($34) + 40|0);
   $36 = HEAP32[$35>>2]|0;
   $37 = ($36|0)<(0);
   $38 = $37 << 31 >> 31;
   $39 = (___divdi3(($32|0),($33|0),($36|0),($38|0))|0);
   $40 = tempRet0;
   $41 = HEAP32[$3>>2]|0;
   $42 = (($41) + 40|0);
   $43 = HEAP32[$42>>2]|0;
   $44 = ($43|0)<(0);
   $45 = $44 << 31 >> 31;
   $46 = (___muldi3(($39|0),($40|0),($43|0),($45|0))|0);
   $47 = tempRet0;
   $48 = $nSize;
   $49 = $48;
   HEAP32[$49>>2] = $46;
   $50 = (($48) + 4)|0;
   $51 = $50;
   HEAP32[$51>>2] = $47;
   $52 = $nSize;
   $53 = $52;
   $54 = HEAP32[$53>>2]|0;
   $55 = (($52) + 4)|0;
   $56 = $55;
   $57 = HEAP32[$56>>2]|0;
   $58 = (($buf) + 36|0);
   $59 = HEAP32[$58>>2]|0;
   $60 = ($59|0)<(0);
   $61 = $60 << 31 >> 31;
   $62 = ($57|0)>($61|0);
   $63 = ($54>>>0)>($59>>>0);
   $64 = ($57|0)==($61|0);
   $65 = $64 & $63;
   $66 = $62 | $65;
   if ($66) {
    $67 = (($buf) + 40|0);
    $68 = HEAP32[$67>>2]|0;
    HEAP32[$nBlk>>2] = $68;
    $69 = HEAP32[$3>>2]|0;
    $70 = (($69) + 12|0);
    $71 = HEAP32[$70>>2]|0;
    $72 = $nSize;
    $73 = $72;
    $74 = HEAP32[$73>>2]|0;
    $75 = (($72) + 4)|0;
    $76 = $75;
    $77 = HEAP32[$76>>2]|0;
    $78 = (_robust_ftruncate($71,$74,$77)|0);
    $79 = ($78|0)!=(0);
    if ($79) {
     $80 = (___errno_location()|0);
     $81 = HEAP32[$80>>2]|0;
     $82 = HEAP32[$3>>2]|0;
     $83 = (($82) + 20|0);
     HEAP32[$83>>2] = $81;
     $84 = HEAP32[$3>>2]|0;
     $85 = (($84) + 32|0);
     $86 = HEAP32[$85>>2]|0;
     $87 = (_unixLogErrorAtLine(1546,37568,$86,28530)|0);
     HEAP32[$2>>2] = $87;
     $147 = HEAP32[$2>>2]|0;
     STACKTOP = sp;return ($147|0);
    }
    $88 = (($buf) + 36|0);
    $89 = HEAP32[$88>>2]|0;
    $90 = HEAP32[$nBlk>>2]|0;
    $91 = $90<<1;
    $92 = (($89) + ($91))|0;
    $93 = (($92) - 1)|0;
    $94 = HEAP32[$nBlk>>2]|0;
    $95 = (($93|0) / ($94|0))&-1;
    $96 = HEAP32[$nBlk>>2]|0;
    $97 = Math_imul($95, $96)|0;
    $98 = (($97) - 1)|0;
    $99 = ($98|0)<(0);
    $100 = $99 << 31 >> 31;
    $101 = $iWrite;
    $102 = $101;
    HEAP32[$102>>2] = $98;
    $103 = (($101) + 4)|0;
    $104 = $103;
    HEAP32[$104>>2] = $100;
    while(1) {
     $105 = $iWrite;
     $106 = $105;
     $107 = HEAP32[$106>>2]|0;
     $108 = (($105) + 4)|0;
     $109 = $108;
     $110 = HEAP32[$109>>2]|0;
     $111 = $nSize;
     $112 = $111;
     $113 = HEAP32[$112>>2]|0;
     $114 = (($111) + 4)|0;
     $115 = $114;
     $116 = HEAP32[$115>>2]|0;
     $117 = ($110|0)<($116|0);
     $118 = ($107>>>0)<($113>>>0);
     $119 = ($110|0)==($116|0);
     $120 = $119 & $118;
     $121 = $117 | $120;
     if (!($121)) {
      break L1;
     }
     $122 = HEAP32[$3>>2]|0;
     $123 = $iWrite;
     $124 = $123;
     $125 = HEAP32[$124>>2]|0;
     $126 = (($123) + 4)|0;
     $127 = $126;
     $128 = HEAP32[$127>>2]|0;
     $129 = (_seekAndWrite($122,$125,$128,1216,1)|0);
     HEAP32[$nWrite>>2] = $129;
     $130 = HEAP32[$nWrite>>2]|0;
     $131 = ($130|0)!=(1);
     if ($131) {
      break;
     }
     $132 = HEAP32[$nBlk>>2]|0;
     $133 = ($132|0)<(0);
     $134 = $133 << 31 >> 31;
     $135 = $iWrite;
     $136 = $135;
     $137 = HEAP32[$136>>2]|0;
     $138 = (($135) + 4)|0;
     $139 = $138;
     $140 = HEAP32[$139>>2]|0;
     $141 = (_i64Add(($137|0),($140|0),($132|0),($134|0))|0);
     $142 = tempRet0;
     $143 = $iWrite;
     $144 = $143;
     HEAP32[$144>>2] = $141;
     $145 = (($143) + 4)|0;
     $146 = $145;
     HEAP32[$146>>2] = $142;
    }
    HEAP32[$2>>2] = 778;
    $147 = HEAP32[$2>>2]|0;
    STACKTOP = sp;return ($147|0);
   }
  }
 } while(0);
 HEAP32[$2>>2] = 0;
 $147 = HEAP32[$2>>2]|0;
 STACKTOP = sp;return ($147|0);
}
function _unixModeBit($pFile,$mask,$pArg) {
 $pFile = $pFile|0;
 $mask = $mask|0;
 $pArg = $pArg|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $1 = sp + 8|0;
 $2 = sp;
 HEAP32[$0>>2] = $pFile;
 HEAP8[$1>>0] = $mask;
 HEAP32[$2>>2] = $pArg;
 $3 = HEAP32[$2>>2]|0;
 $4 = HEAP32[$3>>2]|0;
 $5 = ($4|0)<(0);
 if ($5) {
  $6 = HEAP32[$0>>2]|0;
  $7 = (($6) + 18|0);
  $8 = HEAP16[$7>>1]|0;
  $9 = $8&65535;
  $10 = HEAP8[$1>>0]|0;
  $11 = $10&255;
  $12 = $9 & $11;
  $13 = ($12|0)!=(0);
  $14 = $13&1;
  $15 = HEAP32[$2>>2]|0;
  HEAP32[$15>>2] = $14;
  STACKTOP = sp;return;
 }
 $16 = HEAP32[$2>>2]|0;
 $17 = HEAP32[$16>>2]|0;
 $18 = ($17|0)==(0);
 $19 = HEAP8[$1>>0]|0;
 $20 = $19&255;
 if ($18) {
  $21 = $20 ^ -1;
  $22 = HEAP32[$0>>2]|0;
  $23 = (($22) + 18|0);
  $24 = HEAP16[$23>>1]|0;
  $25 = $24&65535;
  $26 = $25 & $21;
  $27 = $26&65535;
  HEAP16[$23>>1] = $27;
  STACKTOP = sp;return;
 } else {
  $28 = HEAP32[$0>>2]|0;
  $29 = (($28) + 18|0);
  $30 = HEAP16[$29>>1]|0;
  $31 = $30&65535;
  $32 = $31 | $20;
  $33 = $32&65535;
  HEAP16[$29>>1] = $33;
  STACKTOP = sp;return;
 }
}
function _unixGetTempname($nBuf,$zBuf) {
 $nBuf = $nBuf|0;
 $zBuf = $zBuf|0;
 var $$ = 0, $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0;
 var $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0;
 var $44 = 0, $45 = 0, $46 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $i = 0, $j = 0, $vararg_buffer = 0, $zDir = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $vararg_buffer = sp;
 $0 = sp + 24|0;
 $1 = sp + 20|0;
 $2 = sp + 16|0;
 $i = sp + 12|0;
 $j = sp + 8|0;
 $zDir = sp + 4|0;
 HEAP32[$1>>2] = $nBuf;
 HEAP32[$2>>2] = $zBuf;
 $3 = (_unixTempFileDir()|0);
 HEAP32[$zDir>>2] = $3;
 $4 = HEAP32[$zDir>>2]|0;
 $5 = ($4|0)==(0|0);
 $$ = $5 ? 19120 : $3;
 HEAP32[$zDir>>2] = $$;
 $6 = HEAP32[$zDir>>2]|0;
 $7 = (_strlen(($6|0))|0);
 $8 = (($7) + 7)|0;
 $9 = (($8) + 18)|0;
 $10 = HEAP32[$1>>2]|0;
 $11 = ($9>>>0)>=($10>>>0);
 if ($11) {
  HEAP32[$0>>2] = 1;
  $46 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($46|0);
 }
 while(1) {
  $12 = HEAP32[$1>>2]|0;
  $13 = (($12) - 18)|0;
  $14 = HEAP32[$2>>2]|0;
  $15 = HEAP32[$zDir>>2]|0;
  HEAP32[$vararg_buffer>>2] = $15;
  (_sqlite3_snprintf($13,$14,37888,$vararg_buffer)|0);
  $16 = HEAP32[$2>>2]|0;
  $17 = (_strlen(($16|0))|0);
  HEAP32[$j>>2] = $17;
  $18 = HEAP32[$j>>2]|0;
  $19 = HEAP32[$2>>2]|0;
  $20 = (($19) + ($18)|0);
  _sqlite3_randomness(15,$20);
  HEAP32[$i>>2] = 0;
  while(1) {
   $21 = HEAP32[$i>>2]|0;
   $22 = ($21>>>0)<(15);
   $23 = HEAP32[$j>>2]|0;
   $24 = HEAP32[$2>>2]|0;
   $25 = (($24) + ($23)|0);
   if (!($22)) {
    break;
   }
   $26 = HEAP8[$25>>0]|0;
   $27 = $26&255;
   $28 = (($27>>>0) % 62)&-1;
   $29 = (37824 + ($28)|0);
   $30 = HEAP8[$29>>0]|0;
   $31 = HEAP32[$j>>2]|0;
   $32 = HEAP32[$2>>2]|0;
   $33 = (($32) + ($31)|0);
   HEAP8[$33>>0] = $30;
   $34 = HEAP32[$i>>2]|0;
   $35 = (($34) + 1)|0;
   HEAP32[$i>>2] = $35;
   $36 = HEAP32[$j>>2]|0;
   $37 = (($36) + 1)|0;
   HEAP32[$j>>2] = $37;
  }
  HEAP8[$25>>0] = 0;
  $38 = HEAP32[$j>>2]|0;
  $39 = (($38) + 1)|0;
  $40 = HEAP32[$2>>2]|0;
  $41 = (($40) + ($39)|0);
  HEAP8[$41>>0] = 0;
  $42 = HEAP32[((37192 + 28|0))>>2]|0;
  $43 = HEAP32[$2>>2]|0;
  $44 = (FUNCTION_TABLE_iii[$42 & 63]($43,0)|0);
  $45 = ($44|0)==(0);
  if (!($45)) {
   break;
  }
 }
 HEAP32[$0>>2] = 0;
 $46 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($46|0);
}
function _fileHasMoved($pFile) {
 $pFile = $pFile|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0;
 var $9 = 0, $buf = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 80|0;
 $0 = sp + 76|0;
 $buf = sp;
 HEAP32[$0>>2] = $pFile;
 $1 = HEAP32[$0>>2]|0;
 $2 = (($1) + 8|0);
 $3 = HEAP32[$2>>2]|0;
 $4 = ($3|0)!=(0|0);
 if (!($4)) {
  $20 = 0;
  $19 = $20&1;
  STACKTOP = sp;return ($19|0);
 }
 $5 = HEAP32[((37192 + 52|0))>>2]|0;
 $6 = HEAP32[$0>>2]|0;
 $7 = (($6) + 32|0);
 $8 = HEAP32[$7>>2]|0;
 $9 = (FUNCTION_TABLE_iii[$5 & 63]($8,$buf)|0);
 $10 = ($9|0)!=(0);
 if ($10) {
  $20 = 1;
  $19 = $20&1;
  STACKTOP = sp;return ($19|0);
 }
 $11 = (($buf) + 72|0);
 $12 = HEAP32[$11>>2]|0;
 $13 = HEAP32[$0>>2]|0;
 $14 = (($13) + 8|0);
 $15 = HEAP32[$14>>2]|0;
 $16 = (($15) + 4|0);
 $17 = HEAP32[$16>>2]|0;
 $18 = ($12|0)!=($17|0);
 $20 = $18;
 $19 = $20&1;
 STACKTOP = sp;return ($19|0);
}
function _unixTempFileDir() {
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $buf = 0, $i = 0, $zDir = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 96|0;
 $i = sp + 80|0;
 $buf = sp + 4|0;
 $zDir = sp;
 HEAP32[$zDir>>2] = 0;
 $0 = HEAP32[1568>>2]|0;
 HEAP32[37904>>2] = $0;
 $1 = HEAP32[((37904 + 4|0))>>2]|0;
 $2 = ($1|0)!=(0|0);
 if (!($2)) {
  $3 = (_getenv((37976|0))|0);
  HEAP32[((37904 + 4|0))>>2] = $3;
 }
 $4 = HEAP32[((37904 + 8|0))>>2]|0;
 $5 = ($4|0)!=(0|0);
 if (!($5)) {
  $6 = (_getenv((37992|0))|0);
  HEAP32[((37904 + 8|0))>>2] = $6;
 }
 HEAP32[$i>>2] = 0;
 while(1) {
  $7 = HEAP32[$i>>2]|0;
  $8 = ($7>>>0)<(7);
  if (!($8)) {
   label = 12;
   break;
  }
  $9 = HEAP32[$zDir>>2]|0;
  $10 = ($9|0)==(0|0);
  if (!($10)) {
   $11 = HEAP32[((37192 + 52|0))>>2]|0;
   $12 = HEAP32[$zDir>>2]|0;
   $13 = (FUNCTION_TABLE_iii[$11 & 63]($12,$buf)|0);
   $14 = ($13|0)!=(0);
   if (!($14)) {
    $15 = (($buf) + 12|0);
    $16 = HEAP32[$15>>2]|0;
    $17 = $16 & 61440;
    $18 = ($17|0)==(16384);
    if ($18) {
     $19 = HEAP32[((37192 + 28|0))>>2]|0;
     $20 = HEAP32[$zDir>>2]|0;
     $21 = (FUNCTION_TABLE_iii[$19 & 63]($20,7)|0);
     $22 = ($21|0)!=(0);
     if (!($22)) {
      label = 12;
      break;
     }
    }
   }
  }
  $23 = HEAP32[$i>>2]|0;
  $24 = (($23) + 1)|0;
  HEAP32[$i>>2] = $24;
  $25 = (37904 + ($23<<2)|0);
  $26 = HEAP32[$25>>2]|0;
  HEAP32[$zDir>>2] = $26;
 }
 if ((label|0) == 12) {
  $27 = HEAP32[$zDir>>2]|0;
  STACKTOP = sp;return ($27|0);
 }
 return 0|0;
}
function _robust_ftruncate($h,$0,$1) {
 $h = $h|0;
 $0 = $0|0;
 $1 = $1|0;
 var $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0;
 var $9 = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $2 = sp + 12|0;
 $3 = sp;
 $rc = sp + 8|0;
 HEAP32[$2>>2] = $h;
 $4 = $3;
 $5 = $4;
 HEAP32[$5>>2] = $0;
 $6 = (($4) + 4)|0;
 $7 = $6;
 HEAP32[$7>>2] = $1;
 while(1) {
  $8 = HEAP32[((37192 + 76|0))>>2]|0;
  $9 = HEAP32[$2>>2]|0;
  $10 = $3;
  $11 = $10;
  $12 = HEAP32[$11>>2]|0;
  $13 = (($10) + 4)|0;
  $14 = $13;
  $15 = HEAP32[$14>>2]|0;
  $16 = (FUNCTION_TABLE_iii[$8 & 63]($9,$12)|0);
  HEAP32[$rc>>2] = $16;
  $17 = HEAP32[$rc>>2]|0;
  $18 = ($17|0)<(0);
  if (!($18)) {
   label = 4;
   break;
  }
  $19 = (___errno_location()|0);
  $20 = HEAP32[$19>>2]|0;
  $21 = ($20|0)==(4);
  if (!($21)) {
   label = 4;
   break;
  }
 }
 if ((label|0) == 4) {
  $22 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($22|0);
 }
 return 0|0;
}
function _seekAndWrite($id,$0,$1,$pBuf,$cnt) {
 $id = $id|0;
 $0 = $0|0;
 $1 = $1|0;
 $pBuf = $pBuf|0;
 $cnt = $cnt|0;
 var $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0;
 var $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $2 = sp + 16|0;
 $3 = sp;
 $4 = sp + 12|0;
 $5 = sp + 8|0;
 HEAP32[$2>>2] = $id;
 $6 = $3;
 $7 = $6;
 HEAP32[$7>>2] = $0;
 $8 = (($6) + 4)|0;
 $9 = $8;
 HEAP32[$9>>2] = $1;
 HEAP32[$4>>2] = $pBuf;
 HEAP32[$5>>2] = $cnt;
 $10 = HEAP32[$2>>2]|0;
 $11 = (($10) + 12|0);
 $12 = HEAP32[$11>>2]|0;
 $13 = $3;
 $14 = $13;
 $15 = HEAP32[$14>>2]|0;
 $16 = (($13) + 4)|0;
 $17 = $16;
 $18 = HEAP32[$17>>2]|0;
 $19 = HEAP32[$4>>2]|0;
 $20 = HEAP32[$5>>2]|0;
 $21 = HEAP32[$2>>2]|0;
 $22 = (($21) + 20|0);
 $23 = (_seekAndWriteFd($12,$15,$18,$19,$20,$22)|0);
 STACKTOP = sp;return ($23|0);
}
function _seekAndWriteFd($fd,$0,$1,$pBuf,$nBuf,$piErrno) {
 $fd = $fd|0;
 $0 = $0|0;
 $1 = $1|0;
 $pBuf = $pBuf|0;
 $nBuf = $nBuf|0;
 $piErrno = $piErrno|0;
 var $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0;
 var $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0;
 var $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0;
 var $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $8 = 0, $9 = 0, $iSeek = 0, $or$cond = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $2 = sp + 36|0;
 $3 = sp + 32|0;
 $4 = sp + 8|0;
 $5 = sp + 28|0;
 $6 = sp + 24|0;
 $7 = sp + 20|0;
 $rc = sp + 16|0;
 $iSeek = sp;
 HEAP32[$3>>2] = $fd;
 $8 = $4;
 $9 = $8;
 HEAP32[$9>>2] = $0;
 $10 = (($8) + 4)|0;
 $11 = $10;
 HEAP32[$11>>2] = $1;
 HEAP32[$5>>2] = $pBuf;
 HEAP32[$6>>2] = $nBuf;
 HEAP32[$7>>2] = $piErrno;
 HEAP32[$rc>>2] = 0;
 $12 = HEAP32[$6>>2]|0;
 $13 = $12 & 131071;
 HEAP32[$6>>2] = $13;
 while(1) {
  $14 = HEAP32[$3>>2]|0;
  $15 = $4;
  $16 = $15;
  $17 = HEAP32[$16>>2]|0;
  $18 = (($15) + 4)|0;
  $19 = $18;
  $20 = HEAP32[$19>>2]|0;
  $21 = (_lseek(($14|0),($17|0),0)|0);
  $22 = ($21|0)<(0);
  $23 = $22 << 31 >> 31;
  $24 = $iSeek;
  $25 = $24;
  HEAP32[$25>>2] = $21;
  $26 = (($24) + 4)|0;
  $27 = $26;
  HEAP32[$27>>2] = $23;
  $28 = $iSeek;
  $29 = $28;
  $30 = HEAP32[$29>>2]|0;
  $31 = (($28) + 4)|0;
  $32 = $31;
  $33 = HEAP32[$32>>2]|0;
  $34 = $4;
  $35 = $34;
  $36 = HEAP32[$35>>2]|0;
  $37 = (($34) + 4)|0;
  $38 = $37;
  $39 = HEAP32[$38>>2]|0;
  $40 = ($30|0)!=($36|0);
  $41 = ($33|0)!=($39|0);
  $42 = $40 | $41;
  if ($42) {
   break;
  }
  $58 = HEAP32[((37192 + 136|0))>>2]|0;
  $59 = HEAP32[$3>>2]|0;
  $60 = HEAP32[$5>>2]|0;
  $61 = HEAP32[$6>>2]|0;
  $62 = (FUNCTION_TABLE_iiii[$58 & 31]($59,$60,$61)|0);
  HEAP32[$rc>>2] = $62;
  $63 = HEAP32[$rc>>2]|0;
  $64 = ($63|0)<(0);
  if (!($64)) {
   label = 10;
   break;
  }
  $65 = (___errno_location()|0);
  $66 = HEAP32[$65>>2]|0;
  $67 = ($66|0)==(4);
  if (!($67)) {
   label = 10;
   break;
  }
 }
 if ((label|0) == 10) {
  $68 = HEAP32[$rc>>2]|0;
  $69 = ($68|0)<(0);
  $70 = HEAP32[$7>>2]|0;
  $71 = ($70|0)!=(0|0);
  $or$cond = $69 & $71;
  if ($or$cond) {
   $72 = (___errno_location()|0);
   $73 = HEAP32[$72>>2]|0;
   $74 = HEAP32[$7>>2]|0;
   HEAP32[$74>>2] = $73;
  }
  $75 = HEAP32[$rc>>2]|0;
  HEAP32[$2>>2] = $75;
  $76 = HEAP32[$2>>2]|0;
  STACKTOP = sp;return ($76|0);
 }
 $43 = HEAP32[$7>>2]|0;
 $44 = ($43|0)!=(0|0);
 if ($44) {
  $45 = $iSeek;
  $46 = $45;
  $47 = HEAP32[$46>>2]|0;
  $48 = (($45) + 4)|0;
  $49 = $48;
  $50 = HEAP32[$49>>2]|0;
  $51 = ($47|0)==(-1);
  $52 = ($50|0)==(-1);
  $53 = $51 & $52;
  if ($53) {
   $54 = (___errno_location()|0);
   $55 = HEAP32[$54>>2]|0;
   $57 = $55;
  } else {
   $57 = 0;
  }
  $56 = HEAP32[$7>>2]|0;
  HEAP32[$56>>2] = $57;
 }
 HEAP32[$2>>2] = -1;
 $76 = HEAP32[$2>>2]|0;
 STACKTOP = sp;return ($76|0);
}
function _sqliteErrorFromPosixError($posixError,$sqliteIOErr) {
 $posixError = $posixError|0;
 $sqliteIOErr = $sqliteIOErr|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $or$cond = 0, $or$cond3 = 0, $or$cond5 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $1 = sp + 4|0;
 $2 = sp;
 HEAP32[$1>>2] = $posixError;
 HEAP32[$2>>2] = $sqliteIOErr;
 $3 = HEAP32[$1>>2]|0;
 switch ($3|0) {
 case 1:  {
  label = 5;
  break;
 }
 case 13:  {
  $4 = HEAP32[$2>>2]|0;
  $5 = ($4|0)==(3850);
  $6 = HEAP32[$2>>2]|0;
  $7 = ($6|0)==(2058);
  $or$cond = $5 | $7;
  $8 = HEAP32[$2>>2]|0;
  $9 = ($8|0)==(2314);
  $or$cond3 = $or$cond | $9;
  $10 = HEAP32[$2>>2]|0;
  $11 = ($10|0)==(3594);
  $or$cond5 = $or$cond3 | $11;
  if ($or$cond5) {
   HEAP32[$0>>2] = 5;
  } else {
   label = 5;
  }
  break;
 }
 case 37: case 4: case 16: case 110: case 11:  {
  HEAP32[$0>>2] = 5;
  break;
 }
 default: {
  $12 = HEAP32[$2>>2]|0;
  HEAP32[$0>>2] = $12;
 }
 }
 if ((label|0) == 5) {
  HEAP32[$0>>2] = 3;
 }
 $13 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($13|0);
}
function _full_fsync($fd,$fullSync,$dataOnly) {
 $fd = $fd|0;
 $fullSync = $fullSync|0;
 $dataOnly = $dataOnly|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $1 = sp + 8|0;
 $2 = sp + 4|0;
 $rc = sp;
 HEAP32[$0>>2] = $fd;
 HEAP32[$1>>2] = $fullSync;
 HEAP32[$2>>2] = $dataOnly;
 $3 = HEAP32[$0>>2]|0;
 $4 = (_fsync(($3|0))|0);
 HEAP32[$rc>>2] = $4;
 $5 = HEAP32[$rc>>2]|0;
 STACKTOP = sp;return ($5|0);
}
function _seekAndRead($id,$0,$1,$pBuf,$cnt) {
 $id = $id|0;
 $0 = $0|0;
 $1 = $1|0;
 $pBuf = $pBuf|0;
 $cnt = $cnt|0;
 var $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0;
 var $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0;
 var $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0;
 var $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0;
 var $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0;
 var $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $got = 0, $newOffset = 0, $prior = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $2 = sp + 36|0;
 $3 = sp + 32|0;
 $4 = sp + 8|0;
 $5 = sp + 28|0;
 $6 = sp + 24|0;
 $got = sp + 20|0;
 $prior = sp + 16|0;
 $newOffset = sp;
 HEAP32[$3>>2] = $id;
 $7 = $4;
 $8 = $7;
 HEAP32[$8>>2] = $0;
 $9 = (($7) + 4)|0;
 $10 = $9;
 HEAP32[$10>>2] = $1;
 HEAP32[$5>>2] = $pBuf;
 HEAP32[$6>>2] = $cnt;
 HEAP32[$prior>>2] = 0;
 $11 = HEAP32[$6>>2]|0;
 $12 = $11 & 131071;
 HEAP32[$6>>2] = $12;
 while(1) {
  $13 = HEAP32[$3>>2]|0;
  $14 = (($13) + 12|0);
  $15 = HEAP32[$14>>2]|0;
  $16 = $4;
  $17 = $16;
  $18 = HEAP32[$17>>2]|0;
  $19 = (($16) + 4)|0;
  $20 = $19;
  $21 = HEAP32[$20>>2]|0;
  $22 = (_lseek(($15|0),($18|0),0)|0);
  $23 = ($22|0)<(0);
  $24 = $23 << 31 >> 31;
  $25 = $newOffset;
  $26 = $25;
  HEAP32[$26>>2] = $22;
  $27 = (($25) + 4)|0;
  $28 = $27;
  HEAP32[$28>>2] = $24;
  $29 = $newOffset;
  $30 = $29;
  $31 = HEAP32[$30>>2]|0;
  $32 = (($29) + 4)|0;
  $33 = $32;
  $34 = HEAP32[$33>>2]|0;
  $35 = $4;
  $36 = $35;
  $37 = HEAP32[$36>>2]|0;
  $38 = (($35) + 4)|0;
  $39 = $38;
  $40 = HEAP32[$39>>2]|0;
  $41 = ($31|0)!=($37|0);
  $42 = ($34|0)!=($40|0);
  $43 = $41 | $42;
  if ($43) {
   label = 3;
   break;
  }
  $59 = HEAP32[((37192 + 100|0))>>2]|0;
  $60 = HEAP32[$3>>2]|0;
  $61 = (($60) + 12|0);
  $62 = HEAP32[$61>>2]|0;
  $63 = HEAP32[$5>>2]|0;
  $64 = HEAP32[$6>>2]|0;
  $65 = (FUNCTION_TABLE_iiii[$59 & 31]($62,$63,$64)|0);
  HEAP32[$got>>2] = $65;
  $66 = HEAP32[$got>>2]|0;
  $67 = HEAP32[$6>>2]|0;
  $68 = ($66|0)==($67|0);
  if ($68) {
   break;
  }
  $69 = HEAP32[$got>>2]|0;
  $70 = ($69|0)<(0);
  if ($70) {
   $71 = (___errno_location()|0);
   $72 = HEAP32[$71>>2]|0;
   $73 = ($72|0)==(4);
   if (!($73)) {
    label = 11;
    break;
   }
   HEAP32[$got>>2] = 1;
  } else {
   $78 = HEAP32[$got>>2]|0;
   $79 = ($78|0)>(0);
   if ($79) {
    $80 = HEAP32[$got>>2]|0;
    $81 = HEAP32[$6>>2]|0;
    $82 = (($81) - ($80))|0;
    HEAP32[$6>>2] = $82;
    $83 = HEAP32[$got>>2]|0;
    $84 = ($83|0)<(0);
    $85 = $84 << 31 >> 31;
    $86 = $4;
    $87 = $86;
    $88 = HEAP32[$87>>2]|0;
    $89 = (($86) + 4)|0;
    $90 = $89;
    $91 = HEAP32[$90>>2]|0;
    $92 = (_i64Add(($88|0),($91|0),($83|0),($85|0))|0);
    $93 = tempRet0;
    $94 = $4;
    $95 = $94;
    HEAP32[$95>>2] = $92;
    $96 = (($94) + 4)|0;
    $97 = $96;
    HEAP32[$97>>2] = $93;
    $98 = HEAP32[$got>>2]|0;
    $99 = HEAP32[$prior>>2]|0;
    $100 = (($99) + ($98))|0;
    HEAP32[$prior>>2] = $100;
    $101 = HEAP32[$got>>2]|0;
    $102 = HEAP32[$5>>2]|0;
    $103 = (($102) + ($101)|0);
    HEAP32[$5>>2] = $103;
   }
  }
  $104 = HEAP32[$got>>2]|0;
  $105 = ($104|0)>(0);
  if (!($105)) {
   break;
  }
 }
 if ((label|0) == 3) {
  $44 = $newOffset;
  $45 = $44;
  $46 = HEAP32[$45>>2]|0;
  $47 = (($44) + 4)|0;
  $48 = $47;
  $49 = HEAP32[$48>>2]|0;
  $50 = ($46|0)==(-1);
  $51 = ($49|0)==(-1);
  $52 = $50 & $51;
  if ($52) {
   $53 = (___errno_location()|0);
   $54 = HEAP32[$53>>2]|0;
   $55 = HEAP32[$3>>2]|0;
   $56 = (($55) + 20|0);
   HEAP32[$56>>2] = $54;
  } else {
   $57 = HEAP32[$3>>2]|0;
   $58 = (($57) + 20|0);
   HEAP32[$58>>2] = 0;
  }
  HEAP32[$2>>2] = -1;
  $109 = HEAP32[$2>>2]|0;
  STACKTOP = sp;return ($109|0);
 }
 else if ((label|0) == 11) {
  HEAP32[$prior>>2] = 0;
  $74 = (___errno_location()|0);
  $75 = HEAP32[$74>>2]|0;
  $76 = HEAP32[$3>>2]|0;
  $77 = (($76) + 20|0);
  HEAP32[$77>>2] = $75;
 }
 $106 = HEAP32[$got>>2]|0;
 $107 = HEAP32[$prior>>2]|0;
 $108 = (($106) + ($107))|0;
 HEAP32[$2>>2] = $108;
 $109 = HEAP32[$2>>2]|0;
 STACKTOP = sp;return ($109|0);
}
function _closeUnixFile($id) {
 $id = $id|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $pFile = 0, dest = 0, label = 0, sp = 0;
 var stop = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $pFile = sp;
 HEAP32[$0>>2] = $id;
 $1 = HEAP32[$0>>2]|0;
 HEAP32[$pFile>>2] = $1;
 $2 = HEAP32[$pFile>>2]|0;
 $3 = (($2) + 12|0);
 $4 = HEAP32[$3>>2]|0;
 $5 = ($4|0)>=(0);
 if ($5) {
  $6 = HEAP32[$pFile>>2]|0;
  $7 = HEAP32[$pFile>>2]|0;
  $8 = (($7) + 12|0);
  $9 = HEAP32[$8>>2]|0;
  _robust_close($6,$9,26735);
  $10 = HEAP32[$pFile>>2]|0;
  $11 = (($10) + 12|0);
  HEAP32[$11>>2] = -1;
 }
 $12 = HEAP32[$pFile>>2]|0;
 $13 = (($12) + 28|0);
 $14 = HEAP32[$13>>2]|0;
 _sqlite3_free($14);
 $15 = HEAP32[$pFile>>2]|0;
 dest=$15+0|0; stop=dest+44|0; do { HEAP32[dest>>2]=0|0; dest=dest+4|0; } while ((dest|0) < (stop|0));
 STACKTOP = sp;return 0;
}
function _nolockIoFinderImpl($z,$p) {
 $z = $z|0;
 $p = $p|0;
 var $0 = 0, $1 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $1 = sp;
 HEAP32[$0>>2] = $z;
 HEAP32[$1>>2] = $p;
 STACKTOP = sp;return (38016|0);
}
function _nolockClose($id) {
 $id = $id|0;
 var $0 = 0, $1 = 0, $2 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 HEAP32[$0>>2] = $id;
 $1 = HEAP32[$0>>2]|0;
 $2 = (_closeUnixFile($1)|0);
 STACKTOP = sp;return ($2|0);
}
function _nolockLock($NotUsed,$NotUsed2) {
 $NotUsed = $NotUsed|0;
 $NotUsed2 = $NotUsed2|0;
 var $0 = 0, $1 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $1 = sp;
 HEAP32[$0>>2] = $NotUsed;
 HEAP32[$1>>2] = $NotUsed2;
 STACKTOP = sp;return 0;
}
function _nolockUnlock($NotUsed,$NotUsed2) {
 $NotUsed = $NotUsed|0;
 $NotUsed2 = $NotUsed2|0;
 var $0 = 0, $1 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $1 = sp;
 HEAP32[$0>>2] = $NotUsed;
 HEAP32[$1>>2] = $NotUsed2;
 STACKTOP = sp;return 0;
}
function _nolockCheckReservedLock($NotUsed,$pResOut) {
 $NotUsed = $NotUsed|0;
 $pResOut = $pResOut|0;
 var $0 = 0, $1 = 0, $2 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $1 = sp;
 HEAP32[$0>>2] = $NotUsed;
 HEAP32[$1>>2] = $pResOut;
 $2 = HEAP32[$1>>2]|0;
 HEAP32[$2>>2] = 0;
 STACKTOP = sp;return 0;
}
function _findReusableFd($zPath,$flags) {
 $zPath = $zPath|0;
 $flags = $flags|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $pInode = 0, $pUnused = 0, $pp = 0, $sStat = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 96|0;
 $0 = sp + 92|0;
 $1 = sp + 88|0;
 $pUnused = sp + 84|0;
 $sStat = sp + 8|0;
 $pInode = sp + 4|0;
 $pp = sp;
 HEAP32[$0>>2] = $zPath;
 HEAP32[$1>>2] = $flags;
 HEAP32[$pUnused>>2] = 0;
 $2 = HEAP32[((37192 + 52|0))>>2]|0;
 $3 = HEAP32[$0>>2]|0;
 $4 = (FUNCTION_TABLE_iii[$2 & 63]($3,$sStat)|0);
 $5 = (0)==($4|0);
 if (!($5)) {
  $45 = HEAP32[$pUnused>>2]|0;
  STACKTOP = sp;return ($45|0);
 }
 _unixEnterMutex();
 $6 = HEAP32[38352>>2]|0;
 HEAP32[$pInode>>2] = $6;
 while(1) {
  $7 = HEAP32[$pInode>>2]|0;
  $8 = ($7|0)!=(0|0);
  if ($8) {
   $9 = HEAP32[$pInode>>2]|0;
   $10 = HEAP32[$9>>2]|0;
   $11 = HEAP32[$sStat>>2]|0;
   $12 = ($10|0)!=($11|0);
   if ($12) {
    $46 = 1;
   } else {
    $13 = HEAP32[$pInode>>2]|0;
    $14 = (($13) + 4|0);
    $15 = HEAP32[$14>>2]|0;
    $16 = (($sStat) + 72|0);
    $17 = HEAP32[$16>>2]|0;
    $18 = ($15|0)!=($17|0);
    $46 = $18;
   }
  } else {
   $46 = 0;
  }
  $19 = HEAP32[$pInode>>2]|0;
  if (!($46)) {
   break;
  }
  $20 = (($19) + 32|0);
  $21 = HEAP32[$20>>2]|0;
  HEAP32[$pInode>>2] = $21;
 }
 $22 = ($19|0)!=(0|0);
 if ($22) {
  $23 = HEAP32[$pInode>>2]|0;
  $24 = (($23) + 28|0);
  HEAP32[$pp>>2] = $24;
  while(1) {
   $25 = HEAP32[$pp>>2]|0;
   $26 = HEAP32[$25>>2]|0;
   $27 = ($26|0)!=(0|0);
   if (!($27)) {
    break;
   }
   $28 = HEAP32[$pp>>2]|0;
   $29 = HEAP32[$28>>2]|0;
   $30 = (($29) + 4|0);
   $31 = HEAP32[$30>>2]|0;
   $32 = HEAP32[$1>>2]|0;
   $33 = ($31|0)!=($32|0);
   if (!($33)) {
    break;
   }
   $34 = HEAP32[$pp>>2]|0;
   $35 = HEAP32[$34>>2]|0;
   $36 = (($35) + 8|0);
   HEAP32[$pp>>2] = $36;
  }
  $37 = HEAP32[$pp>>2]|0;
  $38 = HEAP32[$37>>2]|0;
  HEAP32[$pUnused>>2] = $38;
  $39 = HEAP32[$pUnused>>2]|0;
  $40 = ($39|0)!=(0|0);
  if ($40) {
   $41 = HEAP32[$pUnused>>2]|0;
   $42 = (($41) + 8|0);
   $43 = HEAP32[$42>>2]|0;
   $44 = HEAP32[$pp>>2]|0;
   HEAP32[$44>>2] = $43;
  }
 }
 _unixLeaveMutex();
 $45 = HEAP32[$pUnused>>2]|0;
 STACKTOP = sp;return ($45|0);
}
function _findCreateFileMode($zPath,$flags,$pMode,$pUid,$pGid) {
 $zPath = $zPath|0;
 $flags = $flags|0;
 $pMode = $pMode|0;
 $pUid = $pUid|0;
 $pGid = $pGid|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $5 = 0;
 var $6 = 0, $7 = 0, $8 = 0, $9 = 0, $nDb = 0, $rc = 0, $sStat = 0, $zDb = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 624|0;
 $0 = sp + 100|0;
 $1 = sp + 96|0;
 $2 = sp + 92|0;
 $3 = sp + 88|0;
 $4 = sp + 84|0;
 $rc = sp + 80|0;
 $zDb = sp + 104|0;
 $nDb = sp + 76|0;
 $sStat = sp;
 HEAP32[$0>>2] = $zPath;
 HEAP32[$1>>2] = $flags;
 HEAP32[$2>>2] = $pMode;
 HEAP32[$3>>2] = $pUid;
 HEAP32[$4>>2] = $pGid;
 HEAP32[$rc>>2] = 0;
 $5 = HEAP32[$2>>2]|0;
 HEAP32[$5>>2] = 0;
 $6 = HEAP32[$3>>2]|0;
 HEAP32[$6>>2] = 0;
 $7 = HEAP32[$4>>2]|0;
 HEAP32[$7>>2] = 0;
 $8 = HEAP32[$1>>2]|0;
 $9 = $8 & 526336;
 $10 = ($9|0)!=(0);
 if (!($10)) {
  $39 = HEAP32[$1>>2]|0;
  $40 = $39 & 8;
  $41 = ($40|0)!=(0);
  if (!($41)) {
   $43 = HEAP32[$rc>>2]|0;
   STACKTOP = sp;return ($43|0);
  }
  $42 = HEAP32[$2>>2]|0;
  HEAP32[$42>>2] = 384;
  $43 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($43|0);
 }
 $11 = HEAP32[$0>>2]|0;
 $12 = (_sqlite3Strlen30($11)|0);
 $13 = (($12) - 1)|0;
 HEAP32[$nDb>>2] = $13;
 while(1) {
  $14 = HEAP32[$nDb>>2]|0;
  $15 = HEAP32[$0>>2]|0;
  $16 = (($15) + ($14)|0);
  $17 = HEAP8[$16>>0]|0;
  $18 = $17 << 24 >> 24;
  $19 = ($18|0)!=(45);
  if (!($19)) {
   break;
  }
  $20 = HEAP32[$nDb>>2]|0;
  $21 = (($20) + -1)|0;
  HEAP32[$nDb>>2] = $21;
 }
 $22 = HEAP32[$0>>2]|0;
 $23 = HEAP32[$nDb>>2]|0;
 _memcpy(($zDb|0),($22|0),($23|0))|0;
 $24 = HEAP32[$nDb>>2]|0;
 $25 = (($zDb) + ($24)|0);
 HEAP8[$25>>0] = 0;
 $26 = HEAP32[((37192 + 52|0))>>2]|0;
 $27 = (FUNCTION_TABLE_iii[$26 & 63]($zDb,$sStat)|0);
 $28 = (0)==($27|0);
 if ($28) {
  $29 = (($sStat) + 12|0);
  $30 = HEAP32[$29>>2]|0;
  $31 = $30 & 511;
  $32 = HEAP32[$2>>2]|0;
  HEAP32[$32>>2] = $31;
  $33 = (($sStat) + 20|0);
  $34 = HEAP32[$33>>2]|0;
  $35 = HEAP32[$3>>2]|0;
  HEAP32[$35>>2] = $34;
  $36 = (($sStat) + 24|0);
  $37 = HEAP32[$36>>2]|0;
  $38 = HEAP32[$4>>2]|0;
  HEAP32[$38>>2] = $37;
  $43 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($43|0);
 } else {
  HEAP32[$rc>>2] = 1802;
  $43 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($43|0);
 }
 return 0|0;
}
function _fillInUnixFile($pVfs,$h,$pId,$zFilename,$ctrlFlags) {
 $pVfs = $pVfs|0;
 $h = $h|0;
 $pId = $pId|0;
 $zFilename = $zFilename|0;
 $ctrlFlags = $ctrlFlags|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0;
 var $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $nFilename = 0, $pLockingStyle = 0, $pNew = 0, $rc = 0, $vararg_buffer = 0, $zLockFile = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $vararg_buffer = sp;
 $0 = sp + 40|0;
 $1 = sp + 36|0;
 $2 = sp + 32|0;
 $3 = sp + 28|0;
 $4 = sp + 24|0;
 $pLockingStyle = sp + 20|0;
 $pNew = sp + 16|0;
 $rc = sp + 12|0;
 $zLockFile = sp + 8|0;
 $nFilename = sp + 4|0;
 HEAP32[$0>>2] = $pVfs;
 HEAP32[$1>>2] = $h;
 HEAP32[$2>>2] = $pId;
 HEAP32[$3>>2] = $zFilename;
 HEAP32[$4>>2] = $ctrlFlags;
 $5 = HEAP32[$2>>2]|0;
 HEAP32[$pNew>>2] = $5;
 HEAP32[$rc>>2] = 0;
 $6 = HEAP32[$1>>2]|0;
 $7 = HEAP32[$pNew>>2]|0;
 $8 = (($7) + 12|0);
 HEAP32[$8>>2] = $6;
 $9 = HEAP32[$0>>2]|0;
 $10 = HEAP32[$pNew>>2]|0;
 $11 = (($10) + 4|0);
 HEAP32[$11>>2] = $9;
 $12 = HEAP32[$3>>2]|0;
 $13 = HEAP32[$pNew>>2]|0;
 $14 = (($13) + 32|0);
 HEAP32[$14>>2] = $12;
 $15 = HEAP32[$4>>2]|0;
 $16 = $15&255;
 $17 = $16&255;
 $18 = HEAP32[$pNew>>2]|0;
 $19 = (($18) + 18|0);
 HEAP16[$19>>1] = $17;
 $20 = HEAP32[$4>>2]|0;
 $21 = $20 & 64;
 $22 = ($21|0)!=(0);
 $23 = HEAP32[$3>>2]|0;
 $24 = $22 ? $23 : 0;
 $25 = (_sqlite3_uri_boolean($24,38136,1)|0);
 $26 = ($25|0)!=(0);
 if ($26) {
  $27 = HEAP32[$pNew>>2]|0;
  $28 = (($27) + 18|0);
  $29 = HEAP16[$28>>1]|0;
  $30 = $29&65535;
  $31 = $30 | 16;
  $32 = $31&65535;
  HEAP16[$28>>1] = $32;
 }
 $33 = HEAP32[$0>>2]|0;
 $34 = (($33) + 16|0);
 $35 = HEAP32[$34>>2]|0;
 $36 = (_strcmp($35,1112)|0);
 $37 = ($36|0)==(0);
 if ($37) {
  $38 = HEAP32[$pNew>>2]|0;
  $39 = (($38) + 18|0);
  $40 = HEAP16[$39>>1]|0;
  $41 = $40&65535;
  $42 = $41 | 1;
  $43 = $42&65535;
  HEAP16[$39>>1] = $43;
 }
 $44 = HEAP32[$4>>2]|0;
 $45 = $44 & 128;
 $46 = ($45|0)!=(0);
 if ($46) {
  HEAP32[$pLockingStyle>>2] = 38016;
 } else {
  $47 = HEAP32[$0>>2]|0;
  $48 = (($47) + 20|0);
  $49 = HEAP32[$48>>2]|0;
  $50 = HEAP32[$49>>2]|0;
  $51 = HEAP32[$3>>2]|0;
  $52 = HEAP32[$pNew>>2]|0;
  $53 = (FUNCTION_TABLE_iii[$50 & 63]($51,$52)|0);
  HEAP32[$pLockingStyle>>2] = $53;
 }
 $54 = HEAP32[$pLockingStyle>>2]|0;
 $55 = ($54|0)==(38144|0);
 if ($55) {
  _unixEnterMutex();
  $56 = HEAP32[$pNew>>2]|0;
  $57 = HEAP32[$pNew>>2]|0;
  $58 = (($57) + 8|0);
  $59 = (_findInodeInfo($56,$58)|0);
  HEAP32[$rc>>2] = $59;
  $60 = HEAP32[$rc>>2]|0;
  $61 = ($60|0)!=(0);
  if ($61) {
   $62 = HEAP32[$pNew>>2]|0;
   $63 = HEAP32[$1>>2]|0;
   _robust_close($62,$63,30114);
   HEAP32[$1>>2] = -1;
  }
  _unixLeaveMutex();
 } else {
  $64 = HEAP32[$pLockingStyle>>2]|0;
  $65 = ($64|0)==(37112|0);
  if ($65) {
   $66 = HEAP32[$3>>2]|0;
   $67 = (_strlen(($66|0))|0);
   $68 = (($67) + 6)|0;
   HEAP32[$nFilename>>2] = $68;
   $69 = HEAP32[$nFilename>>2]|0;
   $70 = (_sqlite3_malloc($69)|0);
   HEAP32[$zLockFile>>2] = $70;
   $71 = HEAP32[$zLockFile>>2]|0;
   $72 = ($71|0)==(0|0);
   if ($72) {
    HEAP32[$rc>>2] = 7;
   } else {
    $73 = HEAP32[$nFilename>>2]|0;
    $74 = HEAP32[$zLockFile>>2]|0;
    $75 = HEAP32[$3>>2]|0;
    HEAP32[$vararg_buffer>>2] = $75;
    (_sqlite3_snprintf($73,$74,38224,$vararg_buffer)|0);
   }
   $76 = HEAP32[$zLockFile>>2]|0;
   $77 = HEAP32[$pNew>>2]|0;
   $78 = (($77) + 24|0);
   HEAP32[$78>>2] = $76;
  }
 }
 $79 = HEAP32[$pNew>>2]|0;
 $80 = (($79) + 20|0);
 HEAP32[$80>>2] = 0;
 $81 = HEAP32[$rc>>2]|0;
 $82 = ($81|0)!=(0);
 if (!($82)) {
  $87 = HEAP32[$pLockingStyle>>2]|0;
  $88 = HEAP32[$pNew>>2]|0;
  HEAP32[$88>>2] = $87;
  $89 = HEAP32[$pNew>>2]|0;
  _verifyDbFile($89);
  $90 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($90|0);
 }
 $83 = HEAP32[$1>>2]|0;
 $84 = ($83|0)>=(0);
 if (!($84)) {
  $90 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($90|0);
 }
 $85 = HEAP32[$pNew>>2]|0;
 $86 = HEAP32[$1>>2]|0;
 _robust_close($85,$86,30199);
 $90 = HEAP32[$rc>>2]|0;
 STACKTOP = sp;return ($90|0);
}
function _findInodeInfo($pFile,$ppInode) {
 $pFile = $pFile|0;
 $ppInode = $ppInode|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $fd = 0;
 var $fileId = 0, $pInode = 0, $rc = 0, $statbuf = 0, dest = 0, label = 0, sp = 0, stop = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 112|0;
 $0 = sp + 104|0;
 $1 = sp + 100|0;
 $2 = sp + 96|0;
 $rc = sp + 92|0;
 $fd = sp + 88|0;
 $fileId = sp + 80|0;
 $statbuf = sp + 4|0;
 $pInode = sp;
 HEAP32[$1>>2] = $pFile;
 HEAP32[$2>>2] = $ppInode;
 HEAP32[$pInode>>2] = 0;
 $3 = HEAP32[$1>>2]|0;
 $4 = (($3) + 12|0);
 $5 = HEAP32[$4>>2]|0;
 HEAP32[$fd>>2] = $5;
 $6 = HEAP32[((37192 + 64|0))>>2]|0;
 $7 = HEAP32[$fd>>2]|0;
 $8 = (FUNCTION_TABLE_iii[$6 & 63]($7,$statbuf)|0);
 HEAP32[$rc>>2] = $8;
 $9 = HEAP32[$rc>>2]|0;
 $10 = ($9|0)!=(0);
 if ($10) {
  $11 = (___errno_location()|0);
  $12 = HEAP32[$11>>2]|0;
  $13 = HEAP32[$1>>2]|0;
  $14 = (($13) + 20|0);
  HEAP32[$14>>2] = $12;
  $15 = HEAP32[$1>>2]|0;
  $16 = (($15) + 20|0);
  $17 = HEAP32[$16>>2]|0;
  $18 = ($17|0)==(75);
  if ($18) {
   HEAP32[$0>>2] = 22;
   $57 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($57|0);
  } else {
   HEAP32[$0>>2] = 10;
   $57 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($57|0);
  }
 }
 ;HEAP32[$fileId+0>>2]=0|0;HEAP32[$fileId+4>>2]=0|0;
 $19 = HEAP32[$statbuf>>2]|0;
 HEAP32[$fileId>>2] = $19;
 $20 = (($statbuf) + 72|0);
 $21 = HEAP32[$20>>2]|0;
 $22 = (($fileId) + 4|0);
 HEAP32[$22>>2] = $21;
 $23 = HEAP32[38352>>2]|0;
 HEAP32[$pInode>>2] = $23;
 while(1) {
  $24 = HEAP32[$pInode>>2]|0;
  $25 = ($24|0)!=(0|0);
  if ($25) {
   $26 = HEAP32[$pInode>>2]|0;
   $27 = (_memcmp($fileId,$26,8)|0);
   $28 = ($27|0)!=(0);
   $58 = $28;
  } else {
   $58 = 0;
  }
  $29 = HEAP32[$pInode>>2]|0;
  if (!($58)) {
   break;
  }
  $30 = (($29) + 32|0);
  $31 = HEAP32[$30>>2]|0;
  HEAP32[$pInode>>2] = $31;
 }
 $32 = ($29|0)==(0|0);
 if ($32) {
  $33 = (_sqlite3_malloc(40)|0);
  HEAP32[$pInode>>2] = $33;
  $34 = HEAP32[$pInode>>2]|0;
  $35 = ($34|0)==(0|0);
  if ($35) {
   HEAP32[$0>>2] = 7;
   $57 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($57|0);
  }
  $36 = HEAP32[$pInode>>2]|0;
  dest=$36+0|0; stop=dest+40|0; do { HEAP32[dest>>2]=0|0; dest=dest+4|0; } while ((dest|0) < (stop|0));
  $37 = HEAP32[$pInode>>2]|0;
  ;HEAP32[$37+0>>2]=HEAP32[$fileId+0>>2]|0;HEAP32[$37+4>>2]=HEAP32[$fileId+4>>2]|0;
  $38 = HEAP32[$pInode>>2]|0;
  $39 = (($38) + 16|0);
  HEAP32[$39>>2] = 1;
  $40 = HEAP32[38352>>2]|0;
  $41 = HEAP32[$pInode>>2]|0;
  $42 = (($41) + 32|0);
  HEAP32[$42>>2] = $40;
  $43 = HEAP32[$pInode>>2]|0;
  $44 = (($43) + 36|0);
  HEAP32[$44>>2] = 0;
  $45 = HEAP32[38352>>2]|0;
  $46 = ($45|0)!=(0|0);
  if ($46) {
   $47 = HEAP32[$pInode>>2]|0;
   $48 = HEAP32[38352>>2]|0;
   $49 = (($48) + 36|0);
   HEAP32[$49>>2] = $47;
  }
  $50 = HEAP32[$pInode>>2]|0;
  HEAP32[38352>>2] = $50;
 } else {
  $51 = HEAP32[$pInode>>2]|0;
  $52 = (($51) + 16|0);
  $53 = HEAP32[$52>>2]|0;
  $54 = (($53) + 1)|0;
  HEAP32[$52>>2] = $54;
 }
 $55 = HEAP32[$pInode>>2]|0;
 $56 = HEAP32[$2>>2]|0;
 HEAP32[$56>>2] = $55;
 HEAP32[$0>>2] = 0;
 $57 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($57|0);
}
function _verifyDbFile($pFile) {
 $pFile = $pFile|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $7 = 0, $8 = 0, $9 = 0, $buf = 0, $rc = 0, $vararg_buffer7 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 96|0;
 $vararg_buffer7 = sp;
 $0 = sp + 84|0;
 $buf = sp + 8|0;
 $rc = sp + 4|0;
 HEAP32[$0>>2] = $pFile;
 $1 = HEAP32[$0>>2]|0;
 $2 = (($1) + 18|0);
 $3 = HEAP16[$2>>1]|0;
 $4 = $3&65535;
 $5 = $4 & 256;
 $6 = ($5|0)!=(0);
 if ($6) {
  STACKTOP = sp;return;
 }
 $7 = HEAP32[((37192 + 64|0))>>2]|0;
 $8 = HEAP32[$0>>2]|0;
 $9 = (($8) + 12|0);
 $10 = HEAP32[$9>>2]|0;
 $11 = (FUNCTION_TABLE_iii[$7 & 63]($10,$buf)|0);
 HEAP32[$rc>>2] = $11;
 $12 = HEAP32[$rc>>2]|0;
 $13 = ($12|0)!=(0);
 if ($13) {
  $14 = HEAP32[$0>>2]|0;
  $15 = (($14) + 32|0);
  $16 = HEAP32[$15>>2]|0;
  HEAP32[$vararg_buffer7>>2] = $16;
  _sqlite3_log(28,38232,$vararg_buffer7);
  $17 = HEAP32[$0>>2]|0;
  $18 = (($17) + 18|0);
  $19 = HEAP16[$18>>1]|0;
  $20 = $19&65535;
  $21 = $20 | 256;
  $22 = $21&65535;
  HEAP16[$18>>1] = $22;
  STACKTOP = sp;return;
 }
 $23 = (($buf) + 16|0);
 $24 = HEAP32[$23>>2]|0;
 $25 = ($24|0)==(0);
 if ($25) {
  $26 = HEAP32[$0>>2]|0;
  $27 = (($26) + 18|0);
  $28 = HEAP16[$27>>1]|0;
  $29 = $28&65535;
  $30 = $29 & 32;
  $31 = ($30|0)==(0);
  if ($31) {
   $32 = HEAP32[$0>>2]|0;
   $33 = (($32) + 32|0);
   $34 = HEAP32[$33>>2]|0;
   HEAP32[$vararg_buffer7>>2] = $34;
   _sqlite3_log(28,38256,$vararg_buffer7);
   $35 = HEAP32[$0>>2]|0;
   $36 = (($35) + 18|0);
   $37 = HEAP16[$36>>1]|0;
   $38 = $37&65535;
   $39 = $38 | 256;
   $40 = $39&65535;
   HEAP16[$36>>1] = $40;
   STACKTOP = sp;return;
  }
 }
 $41 = (($buf) + 16|0);
 $42 = HEAP32[$41>>2]|0;
 $43 = ($42>>>0)>(1);
 $44 = HEAP32[$0>>2]|0;
 if ($43) {
  $45 = (($44) + 32|0);
  $46 = HEAP32[$45>>2]|0;
  HEAP32[$vararg_buffer7>>2] = $46;
  _sqlite3_log(28,38288,$vararg_buffer7);
  $47 = HEAP32[$0>>2]|0;
  $48 = (($47) + 18|0);
  $49 = HEAP16[$48>>1]|0;
  $50 = $49&65535;
  $51 = $50 | 256;
  $52 = $51&65535;
  HEAP16[$48>>1] = $52;
  STACKTOP = sp;return;
 }
 $53 = (_fileHasMoved($44)|0);
 $54 = ($53|0)!=(0);
 if (!($54)) {
  STACKTOP = sp;return;
 }
 $55 = HEAP32[$0>>2]|0;
 $56 = (($55) + 32|0);
 $57 = HEAP32[$56>>2]|0;
 HEAP32[$vararg_buffer7>>2] = $57;
 _sqlite3_log(28,38320,$vararg_buffer7);
 $58 = HEAP32[$0>>2]|0;
 $59 = (($58) + 18|0);
 $60 = HEAP16[$59>>1]|0;
 $61 = $60&65535;
 $62 = $61 | 256;
 $63 = $62&65535;
 HEAP16[$59>>1] = $63;
 STACKTOP = sp;return;
}
function _unixClose($id) {
 $id = $id|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $pFile = 0;
 var $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $rc = sp + 4|0;
 $pFile = sp;
 HEAP32[$0>>2] = $id;
 HEAP32[$rc>>2] = 0;
 $1 = HEAP32[$0>>2]|0;
 HEAP32[$pFile>>2] = $1;
 $2 = HEAP32[$pFile>>2]|0;
 _verifyDbFile($2);
 $3 = HEAP32[$0>>2]|0;
 (_unixUnlock($3,0)|0);
 _unixEnterMutex();
 $4 = HEAP32[$pFile>>2]|0;
 $5 = (($4) + 8|0);
 $6 = HEAP32[$5>>2]|0;
 $7 = ($6|0)!=(0|0);
 if ($7) {
  $8 = HEAP32[$pFile>>2]|0;
  $9 = (($8) + 8|0);
  $10 = HEAP32[$9>>2]|0;
  $11 = (($10) + 24|0);
  $12 = HEAP32[$11>>2]|0;
  $13 = ($12|0)!=(0);
  if ($13) {
   $14 = HEAP32[$pFile>>2]|0;
   _setPendingFd($14);
  }
 }
 $15 = HEAP32[$pFile>>2]|0;
 _releaseInodeInfo($15);
 $16 = HEAP32[$0>>2]|0;
 $17 = (_closeUnixFile($16)|0);
 HEAP32[$rc>>2] = $17;
 _unixLeaveMutex();
 $18 = HEAP32[$rc>>2]|0;
 STACKTOP = sp;return ($18|0);
}
function _unixLock($id,$eFileLock) {
 $id = $id|0;
 $eFileLock = $eFileLock|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0;
 var $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0;
 var $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0;
 var $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0;
 var $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0;
 var $98 = 0, $99 = 0, $lock = 0, $or$cond = 0, $or$cond3 = 0, $pFile = 0, $pInode = 0, $rc = 0, $tErrno = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $0 = sp + 44|0;
 $1 = sp + 40|0;
 $2 = sp + 36|0;
 $rc = sp + 32|0;
 $pFile = sp + 28|0;
 $pInode = sp + 24|0;
 $lock = sp + 8|0;
 $tErrno = sp;
 HEAP32[$1>>2] = $id;
 HEAP32[$2>>2] = $eFileLock;
 HEAP32[$rc>>2] = 0;
 $3 = HEAP32[$1>>2]|0;
 HEAP32[$pFile>>2] = $3;
 HEAP32[$tErrno>>2] = 0;
 $4 = HEAP32[$pFile>>2]|0;
 $5 = (($4) + 16|0);
 $6 = HEAP8[$5>>0]|0;
 $7 = $6&255;
 $8 = HEAP32[$2>>2]|0;
 $9 = ($7|0)>=($8|0);
 if ($9) {
  HEAP32[$0>>2] = 0;
  $161 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($161|0);
 }
 _unixEnterMutex();
 $10 = HEAP32[$pFile>>2]|0;
 $11 = (($10) + 8|0);
 $12 = HEAP32[$11>>2]|0;
 HEAP32[$pInode>>2] = $12;
 $13 = HEAP32[$pFile>>2]|0;
 $14 = (($13) + 16|0);
 $15 = HEAP8[$14>>0]|0;
 $16 = $15&255;
 $17 = HEAP32[$pInode>>2]|0;
 $18 = (($17) + 12|0);
 $19 = HEAP8[$18>>0]|0;
 $20 = $19&255;
 $21 = ($16|0)!=($20|0);
 if ($21) {
  $22 = HEAP32[$pInode>>2]|0;
  $23 = (($22) + 12|0);
  $24 = HEAP8[$23>>0]|0;
  $25 = $24&255;
  $26 = ($25|0)>=(3);
  $27 = HEAP32[$2>>2]|0;
  $28 = ($27|0)>(1);
  $or$cond = $26 | $28;
  if ($or$cond) {
   HEAP32[$rc>>2] = 5;
  } else {
   label = 6;
  }
 } else {
  label = 6;
 }
 L8: do {
  if ((label|0) == 6) {
   $29 = HEAP32[$2>>2]|0;
   $30 = ($29|0)==(1);
   do {
    if ($30) {
     $31 = HEAP32[$pInode>>2]|0;
     $32 = (($31) + 12|0);
     $33 = HEAP8[$32>>0]|0;
     $34 = $33&255;
     $35 = ($34|0)==(1);
     if (!($35)) {
      $36 = HEAP32[$pInode>>2]|0;
      $37 = (($36) + 12|0);
      $38 = HEAP8[$37>>0]|0;
      $39 = $38&255;
      $40 = ($39|0)==(2);
      if (!($40)) {
       break;
      }
     }
     $41 = HEAP32[$pFile>>2]|0;
     $42 = (($41) + 16|0);
     HEAP8[$42>>0] = 1;
     $43 = HEAP32[$pInode>>2]|0;
     $44 = (($43) + 8|0);
     $45 = HEAP32[$44>>2]|0;
     $46 = (($45) + 1)|0;
     HEAP32[$44>>2] = $46;
     $47 = HEAP32[$pInode>>2]|0;
     $48 = (($47) + 24|0);
     $49 = HEAP32[$48>>2]|0;
     $50 = (($49) + 1)|0;
     HEAP32[$48>>2] = $50;
     break L8;
    }
   } while(0);
   $51 = (($lock) + 8|0);
   HEAP32[$51>>2] = 1;
   $52 = (($lock) + 2|0);
   HEAP16[$52>>1] = 0;
   $53 = HEAP32[$2>>2]|0;
   $54 = ($53|0)==(1);
   if ($54) {
    label = 13;
   } else {
    $55 = HEAP32[$2>>2]|0;
    $56 = ($55|0)==(4);
    if ($56) {
     $57 = HEAP32[$pFile>>2]|0;
     $58 = (($57) + 16|0);
     $59 = HEAP8[$58>>0]|0;
     $60 = $59&255;
     $61 = ($60|0)<(3);
     if ($61) {
      label = 13;
     }
    }
   }
   if ((label|0) == 13) {
    $62 = HEAP32[$2>>2]|0;
    $63 = ($62|0)==(1);
    $64 = $63 ? 0 : 1;
    $65 = $64&65535;
    HEAP16[$lock>>1] = $65;
    $66 = HEAP32[1128>>2]|0;
    $67 = (($lock) + 4|0);
    HEAP32[$67>>2] = $66;
    $68 = HEAP32[$pFile>>2]|0;
    $69 = (_unixFileLock($68,$lock)|0);
    $70 = ($69|0)!=(0);
    if ($70) {
     $71 = (___errno_location()|0);
     $72 = HEAP32[$71>>2]|0;
     HEAP32[$tErrno>>2] = $72;
     $73 = HEAP32[$tErrno>>2]|0;
     $74 = (_sqliteErrorFromPosixError($73,3850)|0);
     HEAP32[$rc>>2] = $74;
     $75 = HEAP32[$rc>>2]|0;
     $76 = ($75|0)!=(5);
     if (!($76)) {
      break;
     }
     $77 = HEAP32[$tErrno>>2]|0;
     $78 = HEAP32[$pFile>>2]|0;
     $79 = (($78) + 20|0);
     HEAP32[$79>>2] = $77;
     break;
    }
   }
   $80 = HEAP32[$2>>2]|0;
   $81 = ($80|0)==(1);
   do {
    if ($81) {
     $82 = HEAP32[1128>>2]|0;
     $83 = (($82) + 2)|0;
     $84 = (($lock) + 4|0);
     HEAP32[$84>>2] = $83;
     $85 = (($lock) + 8|0);
     HEAP32[$85>>2] = 510;
     $86 = HEAP32[$pFile>>2]|0;
     $87 = (_unixFileLock($86,$lock)|0);
     $88 = ($87|0)!=(0);
     if ($88) {
      $89 = (___errno_location()|0);
      $90 = HEAP32[$89>>2]|0;
      HEAP32[$tErrno>>2] = $90;
      $91 = HEAP32[$tErrno>>2]|0;
      $92 = (_sqliteErrorFromPosixError($91,3850)|0);
      HEAP32[$rc>>2] = $92;
     }
     $93 = HEAP32[1128>>2]|0;
     $94 = (($lock) + 4|0);
     HEAP32[$94>>2] = $93;
     $95 = (($lock) + 8|0);
     HEAP32[$95>>2] = 1;
     HEAP16[$lock>>1] = 2;
     $96 = HEAP32[$pFile>>2]|0;
     $97 = (_unixFileLock($96,$lock)|0);
     $98 = ($97|0)!=(0);
     $99 = HEAP32[$rc>>2]|0;
     $100 = ($99|0)==(0);
     $or$cond3 = $98 & $100;
     if ($or$cond3) {
      $101 = (___errno_location()|0);
      $102 = HEAP32[$101>>2]|0;
      HEAP32[$tErrno>>2] = $102;
      HEAP32[$rc>>2] = 2058;
     }
     $103 = HEAP32[$rc>>2]|0;
     $104 = ($103|0)!=(0);
     if (!($104)) {
      $110 = HEAP32[$pFile>>2]|0;
      $111 = (($110) + 16|0);
      HEAP8[$111>>0] = 1;
      $112 = HEAP32[$pInode>>2]|0;
      $113 = (($112) + 24|0);
      $114 = HEAP32[$113>>2]|0;
      $115 = (($114) + 1)|0;
      HEAP32[$113>>2] = $115;
      $116 = HEAP32[$pInode>>2]|0;
      $117 = (($116) + 8|0);
      HEAP32[$117>>2] = 1;
      break;
     }
     $105 = HEAP32[$rc>>2]|0;
     $106 = ($105|0)!=(5);
     if (!($106)) {
      break L8;
     }
     $107 = HEAP32[$tErrno>>2]|0;
     $108 = HEAP32[$pFile>>2]|0;
     $109 = (($108) + 20|0);
     HEAP32[$109>>2] = $107;
     break L8;
    } else {
     $118 = HEAP32[$2>>2]|0;
     $119 = ($118|0)==(4);
     if ($119) {
      $120 = HEAP32[$pInode>>2]|0;
      $121 = (($120) + 8|0);
      $122 = HEAP32[$121>>2]|0;
      $123 = ($122|0)>(1);
      if ($123) {
       HEAP32[$rc>>2] = 5;
       break;
      }
     }
     HEAP16[$lock>>1] = 1;
     $124 = HEAP32[$2>>2]|0;
     $125 = ($124|0)==(2);
     $126 = HEAP32[1128>>2]|0;
     if ($125) {
      $127 = (($126) + 1)|0;
      $128 = (($lock) + 4|0);
      HEAP32[$128>>2] = $127;
      $129 = (($lock) + 8|0);
      HEAP32[$129>>2] = 1;
     } else {
      $130 = (($126) + 2)|0;
      $131 = (($lock) + 4|0);
      HEAP32[$131>>2] = $130;
      $132 = (($lock) + 8|0);
      HEAP32[$132>>2] = 510;
     }
     $133 = HEAP32[$pFile>>2]|0;
     $134 = (_unixFileLock($133,$lock)|0);
     $135 = ($134|0)!=(0);
     if ($135) {
      $136 = (___errno_location()|0);
      $137 = HEAP32[$136>>2]|0;
      HEAP32[$tErrno>>2] = $137;
      $138 = HEAP32[$tErrno>>2]|0;
      $139 = (_sqliteErrorFromPosixError($138,3850)|0);
      HEAP32[$rc>>2] = $139;
      $140 = HEAP32[$rc>>2]|0;
      $141 = ($140|0)!=(5);
      if ($141) {
       $142 = HEAP32[$tErrno>>2]|0;
       $143 = HEAP32[$pFile>>2]|0;
       $144 = (($143) + 20|0);
       HEAP32[$144>>2] = $142;
      }
     }
    }
   } while(0);
   $145 = HEAP32[$rc>>2]|0;
   $146 = ($145|0)==(0);
   $147 = HEAP32[$2>>2]|0;
   if ($146) {
    $148 = $147&255;
    $149 = HEAP32[$pFile>>2]|0;
    $150 = (($149) + 16|0);
    HEAP8[$150>>0] = $148;
    $151 = HEAP32[$2>>2]|0;
    $152 = $151&255;
    $153 = HEAP32[$pInode>>2]|0;
    $154 = (($153) + 12|0);
    HEAP8[$154>>0] = $152;
    break;
   }
   $155 = ($147|0)==(4);
   if ($155) {
    $156 = HEAP32[$pFile>>2]|0;
    $157 = (($156) + 16|0);
    HEAP8[$157>>0] = 3;
    $158 = HEAP32[$pInode>>2]|0;
    $159 = (($158) + 12|0);
    HEAP8[$159>>0] = 3;
   }
  }
 } while(0);
 _unixLeaveMutex();
 $160 = HEAP32[$rc>>2]|0;
 HEAP32[$0>>2] = $160;
 $161 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($161|0);
}
function _unixUnlock($id,$eFileLock) {
 $id = $id|0;
 $eFileLock = $eFileLock|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $1 = sp;
 HEAP32[$0>>2] = $id;
 HEAP32[$1>>2] = $eFileLock;
 $2 = HEAP32[$0>>2]|0;
 $3 = HEAP32[$1>>2]|0;
 $4 = (_posixUnlock($2,$3,0)|0);
 STACKTOP = sp;return ($4|0);
}
function _unixCheckReservedLock($id,$pResOut) {
 $id = $id|0;
 $pResOut = $pResOut|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $lock = 0;
 var $pFile = 0, $rc = 0, $reserved = 0, $vararg_buffer = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $vararg_buffer = sp;
 $0 = sp + 40|0;
 $1 = sp + 36|0;
 $rc = sp + 32|0;
 $reserved = sp + 28|0;
 $pFile = sp + 24|0;
 $lock = sp + 8|0;
 HEAP32[$0>>2] = $id;
 HEAP32[$1>>2] = $pResOut;
 HEAP32[$rc>>2] = 0;
 HEAP32[$reserved>>2] = 0;
 $2 = HEAP32[$0>>2]|0;
 HEAP32[$pFile>>2] = $2;
 _unixEnterMutex();
 $3 = HEAP32[$pFile>>2]|0;
 $4 = (($3) + 8|0);
 $5 = HEAP32[$4>>2]|0;
 $6 = (($5) + 12|0);
 $7 = HEAP8[$6>>0]|0;
 $8 = $7&255;
 $9 = ($8|0)>(1);
 if ($9) {
  HEAP32[$reserved>>2] = 1;
 }
 $10 = HEAP32[$reserved>>2]|0;
 $11 = ($10|0)!=(0);
 do {
  if (!($11)) {
   $12 = HEAP32[$pFile>>2]|0;
   $13 = (($12) + 8|0);
   $14 = HEAP32[$13>>2]|0;
   $15 = (($14) + 13|0);
   $16 = HEAP8[$15>>0]|0;
   $17 = ($16<<24>>24)!=(0);
   if (!($17)) {
    $18 = (($lock) + 2|0);
    HEAP16[$18>>1] = 0;
    $19 = HEAP32[1128>>2]|0;
    $20 = (($19) + 1)|0;
    $21 = (($lock) + 4|0);
    HEAP32[$21>>2] = $20;
    $22 = (($lock) + 8|0);
    HEAP32[$22>>2] = 1;
    HEAP16[$lock>>1] = 1;
    $23 = HEAP32[((37192 + 88|0))>>2]|0;
    $24 = HEAP32[$pFile>>2]|0;
    $25 = (($24) + 12|0);
    $26 = HEAP32[$25>>2]|0;
    HEAP32[$vararg_buffer>>2] = $lock;
    $27 = (FUNCTION_TABLE_iiii[$23 & 31]($26,12,$vararg_buffer)|0);
    $28 = ($27|0)!=(0);
    if ($28) {
     HEAP32[$rc>>2] = 3594;
     $29 = (___errno_location()|0);
     $30 = HEAP32[$29>>2]|0;
     $31 = HEAP32[$pFile>>2]|0;
     $32 = (($31) + 20|0);
     HEAP32[$32>>2] = $30;
     break;
    }
    $33 = HEAP16[$lock>>1]|0;
    $34 = $33 << 16 >> 16;
    $35 = ($34|0)!=(2);
    if ($35) {
     HEAP32[$reserved>>2] = 1;
    }
   }
  }
 } while(0);
 _unixLeaveMutex();
 $36 = HEAP32[$reserved>>2]|0;
 $37 = HEAP32[$1>>2]|0;
 HEAP32[$37>>2] = $36;
 $38 = HEAP32[$rc>>2]|0;
 STACKTOP = sp;return ($38|0);
}
function _unixShmMap($fd,$iRegion,$szRegion,$bExtend,$pp) {
 $fd = $fd|0;
 $iRegion = $iRegion|0;
 $szRegion = $szRegion|0;
 $bExtend = $bExtend|0;
 $pp = $pp|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0;
 var $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0, $188 = 0;
 var $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0;
 var $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0;
 var $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0;
 var $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0;
 var $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $apNew = 0, $i = 0, $iPg = 0, $nByte = 0, $nMap = 0, $nReqRegion = 0, $nShmPerMap = 0, $or$cond = 0, $p = 0, $pDbFd = 0, $pMem = 0;
 var $pShmNode = 0, $rc = 0, $sStat = 0, $zFile = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 160|0;
 $0 = sp + 148|0;
 $1 = sp + 144|0;
 $2 = sp + 140|0;
 $3 = sp + 136|0;
 $4 = sp + 132|0;
 $5 = sp + 128|0;
 $pDbFd = sp + 124|0;
 $p = sp + 120|0;
 $pShmNode = sp + 116|0;
 $rc = sp + 112|0;
 $nShmPerMap = sp + 108|0;
 $nReqRegion = sp + 104|0;
 $apNew = sp + 100|0;
 $nByte = sp + 96|0;
 $sStat = sp + 20|0;
 $iPg = sp + 16|0;
 $zFile = sp + 12|0;
 $nMap = sp + 8|0;
 $i = sp + 4|0;
 $pMem = sp;
 HEAP32[$1>>2] = $fd;
 HEAP32[$2>>2] = $iRegion;
 HEAP32[$3>>2] = $szRegion;
 HEAP32[$4>>2] = $bExtend;
 HEAP32[$5>>2] = $pp;
 $6 = HEAP32[$1>>2]|0;
 HEAP32[$pDbFd>>2] = $6;
 HEAP32[$rc>>2] = 0;
 $7 = (_unixShmRegionPerMap()|0);
 HEAP32[$nShmPerMap>>2] = $7;
 $8 = HEAP32[$pDbFd>>2]|0;
 $9 = (($8) + 36|0);
 $10 = HEAP32[$9>>2]|0;
 $11 = ($10|0)==(0|0);
 if ($11) {
  $12 = HEAP32[$pDbFd>>2]|0;
  $13 = (_unixOpenSharedMemory($12)|0);
  HEAP32[$rc>>2] = $13;
  $14 = HEAP32[$rc>>2]|0;
  $15 = ($14|0)!=(0);
  if ($15) {
   $16 = HEAP32[$rc>>2]|0;
   HEAP32[$0>>2] = $16;
   $188 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($188|0);
  }
 }
 $17 = HEAP32[$pDbFd>>2]|0;
 $18 = (($17) + 36|0);
 $19 = HEAP32[$18>>2]|0;
 HEAP32[$p>>2] = $19;
 $20 = HEAP32[$p>>2]|0;
 $21 = HEAP32[$20>>2]|0;
 HEAP32[$pShmNode>>2] = $21;
 $22 = HEAP32[$2>>2]|0;
 $23 = HEAP32[$nShmPerMap>>2]|0;
 $24 = (($22) + ($23))|0;
 $25 = HEAP32[$nShmPerMap>>2]|0;
 $26 = (($24|0) / ($25|0))&-1;
 $27 = HEAP32[$nShmPerMap>>2]|0;
 $28 = Math_imul($26, $27)|0;
 HEAP32[$nReqRegion>>2] = $28;
 $29 = HEAP32[$pShmNode>>2]|0;
 $30 = (($29) + 20|0);
 $31 = HEAP16[$30>>1]|0;
 $32 = $31&65535;
 $33 = HEAP32[$nReqRegion>>2]|0;
 $34 = ($32|0)<($33|0);
 L6: do {
  if ($34) {
   $35 = HEAP32[$nReqRegion>>2]|0;
   $36 = HEAP32[$3>>2]|0;
   $37 = Math_imul($35, $36)|0;
   HEAP32[$nByte>>2] = $37;
   $38 = HEAP32[$3>>2]|0;
   $39 = HEAP32[$pShmNode>>2]|0;
   $40 = (($39) + 16|0);
   HEAP32[$40>>2] = $38;
   $41 = HEAP32[$pShmNode>>2]|0;
   $42 = (($41) + 12|0);
   $43 = HEAP32[$42>>2]|0;
   $44 = ($43|0)>=(0);
   L8: do {
    if ($44) {
     $45 = HEAP32[((37192 + 64|0))>>2]|0;
     $46 = HEAP32[$pShmNode>>2]|0;
     $47 = (($46) + 12|0);
     $48 = HEAP32[$47>>2]|0;
     $49 = (FUNCTION_TABLE_iii[$45 & 63]($48,$sStat)|0);
     $50 = ($49|0)!=(0);
     if ($50) {
      HEAP32[$rc>>2] = 4874;
      break L6;
     }
     $51 = (($sStat) + 36|0);
     $52 = HEAP32[$51>>2]|0;
     $53 = HEAP32[$nByte>>2]|0;
     $54 = ($52|0)<($53|0);
     if ($54) {
      $55 = HEAP32[$4>>2]|0;
      $56 = ($55|0)!=(0);
      if (!($56)) {
       break L6;
      }
      $57 = (($sStat) + 36|0);
      $58 = HEAP32[$57>>2]|0;
      $59 = (($58|0) / 4096)&-1;
      HEAP32[$iPg>>2] = $59;
      while(1) {
       $60 = HEAP32[$iPg>>2]|0;
       $61 = HEAP32[$nByte>>2]|0;
       $62 = (($61|0) / 4096)&-1;
       $63 = ($60|0)<($62|0);
       if (!($63)) {
        break L8;
       }
       $64 = HEAP32[$pShmNode>>2]|0;
       $65 = (($64) + 12|0);
       $66 = HEAP32[$65>>2]|0;
       $67 = HEAP32[$iPg>>2]|0;
       $68 = $67<<12;
       $69 = (($68) + 4096)|0;
       $70 = (($69) - 1)|0;
       $71 = ($70|0)<(0);
       $72 = $71 << 31 >> 31;
       $73 = (_seekAndWriteFd($66,$70,$72,1216,1,0)|0);
       $74 = ($73|0)!=(1);
       if ($74) {
        break;
       }
       $80 = HEAP32[$iPg>>2]|0;
       $81 = (($80) + 1)|0;
       HEAP32[$iPg>>2] = $81;
      }
      $75 = HEAP32[$pShmNode>>2]|0;
      $76 = (($75) + 8|0);
      $77 = HEAP32[$76>>2]|0;
      HEAP32[$zFile>>2] = $77;
      $78 = HEAP32[$zFile>>2]|0;
      $79 = (_unixLogErrorAtLine(4874,37616,$78,29258)|0);
      HEAP32[$rc>>2] = $79;
      break L6;
     }
    }
   } while(0);
   $82 = HEAP32[$pShmNode>>2]|0;
   $83 = (($82) + 24|0);
   $84 = HEAP32[$83>>2]|0;
   $85 = HEAP32[$nReqRegion>>2]|0;
   $86 = $85<<2;
   $87 = (_sqlite3_realloc($84,$86)|0);
   HEAP32[$apNew>>2] = $87;
   $88 = HEAP32[$apNew>>2]|0;
   $89 = ($88|0)!=(0|0);
   if (!($89)) {
    HEAP32[$rc>>2] = 3082;
    break;
   }
   $90 = HEAP32[$apNew>>2]|0;
   $91 = HEAP32[$pShmNode>>2]|0;
   $92 = (($91) + 24|0);
   HEAP32[$92>>2] = $90;
   while(1) {
    $93 = HEAP32[$pShmNode>>2]|0;
    $94 = (($93) + 20|0);
    $95 = HEAP16[$94>>1]|0;
    $96 = $95&65535;
    $97 = HEAP32[$nReqRegion>>2]|0;
    $98 = ($96|0)<($97|0);
    if (!($98)) {
     break L6;
    }
    $99 = HEAP32[$3>>2]|0;
    $100 = HEAP32[$nShmPerMap>>2]|0;
    $101 = Math_imul($99, $100)|0;
    HEAP32[$nMap>>2] = $101;
    $102 = HEAP32[$pShmNode>>2]|0;
    $103 = (($102) + 12|0);
    $104 = HEAP32[$103>>2]|0;
    $105 = ($104|0)>=(0);
    if ($105) {
     $106 = HEAP32[((37192 + 256|0))>>2]|0;
     $107 = HEAP32[$nMap>>2]|0;
     $108 = HEAP32[$pShmNode>>2]|0;
     $109 = (($108) + 22|0);
     $110 = HEAP8[$109>>0]|0;
     $111 = $110&255;
     $112 = ($111|0)!=(0);
     $113 = $112 ? 1 : 3;
     $114 = HEAP32[$pShmNode>>2]|0;
     $115 = (($114) + 12|0);
     $116 = HEAP32[$115>>2]|0;
     $117 = HEAP32[$3>>2]|0;
     $118 = ($117|0)<(0);
     $119 = $118 << 31 >> 31;
     $120 = HEAP32[$pShmNode>>2]|0;
     $121 = (($120) + 20|0);
     $122 = HEAP16[$121>>1]|0;
     $123 = $122&65535;
     $124 = (___muldi3(($117|0),($119|0),($123|0),0)|0);
     $125 = tempRet0;
     $126 = (FUNCTION_TABLE_iiiiiii[$106 & 1](0,$107,$113,1,$116,$124)|0);
     HEAP32[$pMem>>2] = $126;
     $127 = HEAP32[$pMem>>2]|0;
     $128 = ($127|0)==((-1)|0);
     if ($128) {
      label = 21;
      break;
     }
    } else {
     $133 = HEAP32[$3>>2]|0;
     $134 = (_sqlite3_malloc($133)|0);
     HEAP32[$pMem>>2] = $134;
     $135 = HEAP32[$pMem>>2]|0;
     $136 = ($135|0)==(0|0);
     if ($136) {
      label = 23;
      break;
     }
     $137 = HEAP32[$pMem>>2]|0;
     $138 = HEAP32[$3>>2]|0;
     _memset(($137|0),0,($138|0))|0;
    }
    HEAP32[$i>>2] = 0;
    while(1) {
     $139 = HEAP32[$i>>2]|0;
     $140 = HEAP32[$nShmPerMap>>2]|0;
     $141 = ($139|0)<($140|0);
     if (!($141)) {
      break;
     }
     $142 = HEAP32[$3>>2]|0;
     $143 = HEAP32[$i>>2]|0;
     $144 = Math_imul($142, $143)|0;
     $145 = HEAP32[$pMem>>2]|0;
     $146 = (($145) + ($144)|0);
     $147 = HEAP32[$pShmNode>>2]|0;
     $148 = (($147) + 20|0);
     $149 = HEAP16[$148>>1]|0;
     $150 = $149&65535;
     $151 = HEAP32[$i>>2]|0;
     $152 = (($150) + ($151))|0;
     $153 = HEAP32[$pShmNode>>2]|0;
     $154 = (($153) + 24|0);
     $155 = HEAP32[$154>>2]|0;
     $156 = (($155) + ($152<<2)|0);
     HEAP32[$156>>2] = $146;
     $157 = HEAP32[$i>>2]|0;
     $158 = (($157) + 1)|0;
     HEAP32[$i>>2] = $158;
    }
    $159 = HEAP32[$nShmPerMap>>2]|0;
    $160 = HEAP32[$pShmNode>>2]|0;
    $161 = (($160) + 20|0);
    $162 = HEAP16[$161>>1]|0;
    $163 = $162&65535;
    $164 = (($163) + ($159))|0;
    $165 = $164&65535;
    HEAP16[$161>>1] = $165;
   }
   if ((label|0) == 21) {
    $129 = HEAP32[$pShmNode>>2]|0;
    $130 = (($129) + 8|0);
    $131 = HEAP32[$130>>2]|0;
    $132 = (_unixLogErrorAtLine(5386,37720,$131,29285)|0);
    HEAP32[$rc>>2] = $132;
    break;
   }
   else if ((label|0) == 23) {
    HEAP32[$rc>>2] = 7;
    break;
   }
  }
 } while(0);
 $166 = HEAP32[$pShmNode>>2]|0;
 $167 = (($166) + 20|0);
 $168 = HEAP16[$167>>1]|0;
 $169 = $168&65535;
 $170 = HEAP32[$2>>2]|0;
 $171 = ($169|0)>($170|0);
 if ($171) {
  $172 = HEAP32[$2>>2]|0;
  $173 = HEAP32[$pShmNode>>2]|0;
  $174 = (($173) + 24|0);
  $175 = HEAP32[$174>>2]|0;
  $176 = (($175) + ($172<<2)|0);
  $177 = HEAP32[$176>>2]|0;
  $178 = HEAP32[$5>>2]|0;
  HEAP32[$178>>2] = $177;
 } else {
  $179 = HEAP32[$5>>2]|0;
  HEAP32[$179>>2] = 0;
 }
 $180 = HEAP32[$pShmNode>>2]|0;
 $181 = (($180) + 22|0);
 $182 = HEAP8[$181>>0]|0;
 $183 = $182&255;
 $184 = ($183|0)!=(0);
 $185 = HEAP32[$rc>>2]|0;
 $186 = ($185|0)==(0);
 $or$cond = $184 & $186;
 if ($or$cond) {
  HEAP32[$rc>>2] = 8;
 }
 $187 = HEAP32[$rc>>2]|0;
 HEAP32[$0>>2] = $187;
 $188 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($188|0);
}
function _unixOpenSharedMemory($pDbFd) {
 $pDbFd = $pDbFd|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0;
 var $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0;
 var $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0;
 var $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0;
 var $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $nShmFilename = 0, $openFlags = 0, $p = 0, $pInode = 0;
 var $pShmNode = 0, $rc = 0, $sStat = 0, $vararg_buffer = 0, $zShmFilename = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 128|0;
 $vararg_buffer = sp;
 $0 = sp + 112|0;
 $1 = sp + 108|0;
 $p = sp + 104|0;
 $pShmNode = sp + 100|0;
 $rc = sp + 96|0;
 $pInode = sp + 92|0;
 $zShmFilename = sp + 88|0;
 $nShmFilename = sp + 84|0;
 $sStat = sp + 8|0;
 $openFlags = sp + 4|0;
 HEAP32[$1>>2] = $pDbFd;
 HEAP32[$p>>2] = 0;
 $2 = (_sqlite3_malloc(16)|0);
 HEAP32[$p>>2] = $2;
 $3 = HEAP32[$p>>2]|0;
 $4 = ($3|0)==(0|0);
 if ($4) {
  HEAP32[$0>>2] = 7;
  $135 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($135|0);
 }
 $5 = HEAP32[$p>>2]|0;
 ;HEAP32[$5+0>>2]=0|0;HEAP32[$5+4>>2]=0|0;HEAP32[$5+8>>2]=0|0;HEAP32[$5+12>>2]=0|0;
 _unixEnterMutex();
 $6 = HEAP32[$1>>2]|0;
 $7 = (($6) + 8|0);
 $8 = HEAP32[$7>>2]|0;
 HEAP32[$pInode>>2] = $8;
 $9 = HEAP32[$pInode>>2]|0;
 $10 = (($9) + 20|0);
 $11 = HEAP32[$10>>2]|0;
 HEAP32[$pShmNode>>2] = $11;
 $12 = HEAP32[$pShmNode>>2]|0;
 $13 = ($12|0)==(0|0);
 L5: do {
  if ($13) {
   $14 = HEAP32[((37192 + 64|0))>>2]|0;
   $15 = HEAP32[$1>>2]|0;
   $16 = (($15) + 12|0);
   $17 = HEAP32[$16>>2]|0;
   $18 = (FUNCTION_TABLE_iii[$14 & 63]($17,$sStat)|0);
   $19 = ($18|0)!=(0);
   if ($19) {
    $20 = HEAP32[$pInode>>2]|0;
    $21 = (($20) + 13|0);
    $22 = HEAP8[$21>>0]|0;
    $23 = $22&255;
    $24 = ($23|0)==(0);
    if ($24) {
     HEAP32[$rc>>2] = 1802;
    } else {
     label = 7;
    }
   } else {
    label = 7;
   }
   do {
    if ((label|0) == 7) {
     $25 = HEAP32[$1>>2]|0;
     $26 = (($25) + 32|0);
     $27 = HEAP32[$26>>2]|0;
     $28 = (_strlen(($27|0))|0);
     $29 = (6 + ($28))|0;
     HEAP32[$nShmFilename>>2] = $29;
     $30 = HEAP32[$nShmFilename>>2]|0;
     $31 = (36 + ($30))|0;
     $32 = (_sqlite3_malloc($31)|0);
     HEAP32[$pShmNode>>2] = $32;
     $33 = HEAP32[$pShmNode>>2]|0;
     $34 = ($33|0)==(0|0);
     if ($34) {
      HEAP32[$rc>>2] = 7;
      break;
     }
     $35 = HEAP32[$pShmNode>>2]|0;
     $36 = HEAP32[$nShmFilename>>2]|0;
     $37 = (36 + ($36))|0;
     _memset(($35|0),0,($37|0))|0;
     $38 = HEAP32[$pShmNode>>2]|0;
     $39 = (($38) + 36|0);
     $40 = HEAP32[$pShmNode>>2]|0;
     $41 = (($40) + 8|0);
     HEAP32[$41>>2] = $39;
     HEAP32[$zShmFilename>>2] = $39;
     $42 = HEAP32[$nShmFilename>>2]|0;
     $43 = HEAP32[$zShmFilename>>2]|0;
     $44 = HEAP32[$1>>2]|0;
     $45 = (($44) + 32|0);
     $46 = HEAP32[$45>>2]|0;
     HEAP32[$vararg_buffer>>2] = $46;
     (_sqlite3_snprintf($42,$43,38360,$vararg_buffer)|0);
     $47 = HEAP32[$pShmNode>>2]|0;
     $48 = (($47) + 12|0);
     HEAP32[$48>>2] = -1;
     $49 = HEAP32[$pShmNode>>2]|0;
     $50 = HEAP32[$1>>2]|0;
     $51 = (($50) + 8|0);
     $52 = HEAP32[$51>>2]|0;
     $53 = (($52) + 20|0);
     HEAP32[$53>>2] = $49;
     $54 = HEAP32[$1>>2]|0;
     $55 = (($54) + 8|0);
     $56 = HEAP32[$55>>2]|0;
     $57 = HEAP32[$pShmNode>>2]|0;
     HEAP32[$57>>2] = $56;
     $58 = HEAP32[$pShmNode>>2]|0;
     $59 = (($58) + 4|0);
     HEAP32[$59>>2] = (8);
     $60 = HEAP32[$pShmNode>>2]|0;
     $61 = (($60) + 4|0);
     $62 = HEAP32[$61>>2]|0;
     $63 = ($62|0)==(0|0);
     if ($63) {
      HEAP32[$rc>>2] = 7;
      break;
     }
     $64 = HEAP32[$pInode>>2]|0;
     $65 = (($64) + 13|0);
     $66 = HEAP8[$65>>0]|0;
     $67 = $66&255;
     $68 = ($67|0)==(0);
     if (!($68)) {
      break L5;
     }
     HEAP32[$openFlags>>2] = 66;
     $69 = HEAP32[$1>>2]|0;
     $70 = (($69) + 32|0);
     $71 = HEAP32[$70>>2]|0;
     $72 = (_sqlite3_uri_boolean($71,38368,0)|0);
     $73 = ($72|0)!=(0);
     if ($73) {
      HEAP32[$openFlags>>2] = 0;
      $74 = HEAP32[$pShmNode>>2]|0;
      $75 = (($74) + 22|0);
      HEAP8[$75>>0] = 1;
     }
     $76 = HEAP32[$zShmFilename>>2]|0;
     $77 = HEAP32[$openFlags>>2]|0;
     $78 = (($sStat) + 12|0);
     $79 = HEAP32[$78>>2]|0;
     $80 = $79 & 511;
     $81 = (_robust_open($76,$77,$80)|0);
     $82 = HEAP32[$pShmNode>>2]|0;
     $83 = (($82) + 12|0);
     HEAP32[$83>>2] = $81;
     $84 = HEAP32[$pShmNode>>2]|0;
     $85 = (($84) + 12|0);
     $86 = HEAP32[$85>>2]|0;
     $87 = ($86|0)<(0);
     if ($87) {
      $88 = (_sqlite3CantopenError(29111)|0);
      $89 = HEAP32[$zShmFilename>>2]|0;
      $90 = (_unixLogErrorAtLine($88,37536,$89,29111)|0);
      HEAP32[$rc>>2] = $90;
      break;
     }
     $91 = HEAP32[((37192 + 244|0))>>2]|0;
     $92 = HEAP32[$pShmNode>>2]|0;
     $93 = (($92) + 12|0);
     $94 = HEAP32[$93>>2]|0;
     $95 = (($sStat) + 20|0);
     $96 = HEAP32[$95>>2]|0;
     $97 = (($sStat) + 24|0);
     $98 = HEAP32[$97>>2]|0;
     (FUNCTION_TABLE_iiii[$91 & 31]($94,$96,$98)|0);
     HEAP32[$rc>>2] = 0;
     $99 = HEAP32[$pShmNode>>2]|0;
     $100 = (_unixShmSystemLock($99,1,128,1)|0);
     $101 = ($100|0)==(0);
     if ($101) {
      $102 = HEAP32[$pShmNode>>2]|0;
      $103 = (($102) + 12|0);
      $104 = HEAP32[$103>>2]|0;
      $105 = (_robust_ftruncate($104,0,0)|0);
      $106 = ($105|0)!=(0);
      if ($106) {
       $107 = HEAP32[$zShmFilename>>2]|0;
       $108 = (_unixLogErrorAtLine(4618,37568,$107,29127)|0);
       HEAP32[$rc>>2] = $108;
      }
     }
     $109 = HEAP32[$rc>>2]|0;
     $110 = ($109|0)==(0);
     if ($110) {
      $111 = HEAP32[$pShmNode>>2]|0;
      $112 = (_unixShmSystemLock($111,0,128,1)|0);
      HEAP32[$rc>>2] = $112;
     }
     $113 = HEAP32[$rc>>2]|0;
     $114 = ($113|0)!=(0);
     if (!($114)) {
      break L5;
     }
    }
   } while(0);
   $132 = HEAP32[$1>>2]|0;
   _unixShmPurge($132);
   $133 = HEAP32[$p>>2]|0;
   _sqlite3_free($133);
   _unixLeaveMutex();
   $134 = HEAP32[$rc>>2]|0;
   HEAP32[$0>>2] = $134;
   $135 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($135|0);
  }
 } while(0);
 $115 = HEAP32[$pShmNode>>2]|0;
 $116 = HEAP32[$p>>2]|0;
 HEAP32[$116>>2] = $115;
 $117 = HEAP32[$pShmNode>>2]|0;
 $118 = (($117) + 28|0);
 $119 = HEAP32[$118>>2]|0;
 $120 = (($119) + 1)|0;
 HEAP32[$118>>2] = $120;
 $121 = HEAP32[$p>>2]|0;
 $122 = HEAP32[$1>>2]|0;
 $123 = (($122) + 36|0);
 HEAP32[$123>>2] = $121;
 _unixLeaveMutex();
 $124 = HEAP32[$pShmNode>>2]|0;
 $125 = (($124) + 32|0);
 $126 = HEAP32[$125>>2]|0;
 $127 = HEAP32[$p>>2]|0;
 $128 = (($127) + 4|0);
 HEAP32[$128>>2] = $126;
 $129 = HEAP32[$p>>2]|0;
 $130 = HEAP32[$pShmNode>>2]|0;
 $131 = (($130) + 32|0);
 HEAP32[$131>>2] = $129;
 HEAP32[$0>>2] = 0;
 $135 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($135|0);
}
function _posixUnlock($id,$eFileLock,$handleNFSUnlock) {
 $id = $id|0;
 $eFileLock = $eFileLock|0;
 $handleNFSUnlock = $handleNFSUnlock|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0;
 var $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $9 = 0, $lock = 0, $pFile = 0, $pInode = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $0 = sp + 44|0;
 $1 = sp + 40|0;
 $2 = sp + 36|0;
 $3 = sp + 32|0;
 $pFile = sp + 28|0;
 $pInode = sp + 24|0;
 $lock = sp + 8|0;
 $rc = sp;
 HEAP32[$1>>2] = $id;
 HEAP32[$2>>2] = $eFileLock;
 HEAP32[$3>>2] = $handleNFSUnlock;
 $4 = HEAP32[$1>>2]|0;
 HEAP32[$pFile>>2] = $4;
 HEAP32[$rc>>2] = 0;
 $5 = HEAP32[$pFile>>2]|0;
 $6 = (($5) + 16|0);
 $7 = HEAP8[$6>>0]|0;
 $8 = $7&255;
 $9 = HEAP32[$2>>2]|0;
 $10 = ($8|0)<=($9|0);
 if ($10) {
  HEAP32[$0>>2] = 0;
  $88 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($88|0);
 }
 _unixEnterMutex();
 $11 = HEAP32[$pFile>>2]|0;
 $12 = (($11) + 8|0);
 $13 = HEAP32[$12>>2]|0;
 HEAP32[$pInode>>2] = $13;
 $14 = HEAP32[$pFile>>2]|0;
 $15 = (($14) + 16|0);
 $16 = HEAP8[$15>>0]|0;
 $17 = $16&255;
 $18 = ($17|0)>(1);
 do {
  if ($18) {
   $19 = HEAP32[$2>>2]|0;
   $20 = ($19|0)==(1);
   if ($20) {
    HEAP16[$lock>>1] = 0;
    $21 = (($lock) + 2|0);
    HEAP16[$21>>1] = 0;
    $22 = HEAP32[1128>>2]|0;
    $23 = (($22) + 2)|0;
    $24 = (($lock) + 4|0);
    HEAP32[$24>>2] = $23;
    $25 = (($lock) + 8|0);
    HEAP32[$25>>2] = 510;
    $26 = HEAP32[$pFile>>2]|0;
    $27 = (_unixFileLock($26,$lock)|0);
    $28 = ($27|0)!=(0);
    if ($28) {
     HEAP32[$rc>>2] = 2314;
     $29 = (___errno_location()|0);
     $30 = HEAP32[$29>>2]|0;
     $31 = HEAP32[$pFile>>2]|0;
     $32 = (($31) + 20|0);
     HEAP32[$32>>2] = $30;
     break;
    }
   }
   HEAP16[$lock>>1] = 2;
   $33 = (($lock) + 2|0);
   HEAP16[$33>>1] = 0;
   $34 = HEAP32[1128>>2]|0;
   $35 = (($lock) + 4|0);
   HEAP32[$35>>2] = $34;
   $36 = (($lock) + 8|0);
   HEAP32[$36>>2] = 2;
   $37 = HEAP32[$pFile>>2]|0;
   $38 = (_unixFileLock($37,$lock)|0);
   $39 = ($38|0)==(0);
   if ($39) {
    $40 = HEAP32[$pInode>>2]|0;
    $41 = (($40) + 12|0);
    HEAP8[$41>>0] = 1;
    label = 10;
    break;
   } else {
    HEAP32[$rc>>2] = 2058;
    $42 = (___errno_location()|0);
    $43 = HEAP32[$42>>2]|0;
    $44 = HEAP32[$pFile>>2]|0;
    $45 = (($44) + 20|0);
    HEAP32[$45>>2] = $43;
    break;
   }
  } else {
   label = 10;
  }
 } while(0);
 if ((label|0) == 10) {
  $46 = HEAP32[$2>>2]|0;
  $47 = ($46|0)==(0);
  if ($47) {
   $48 = HEAP32[$pInode>>2]|0;
   $49 = (($48) + 8|0);
   $50 = HEAP32[$49>>2]|0;
   $51 = (($50) + -1)|0;
   HEAP32[$49>>2] = $51;
   $52 = HEAP32[$pInode>>2]|0;
   $53 = (($52) + 8|0);
   $54 = HEAP32[$53>>2]|0;
   $55 = ($54|0)==(0);
   do {
    if ($55) {
     HEAP16[$lock>>1] = 2;
     $56 = (($lock) + 2|0);
     HEAP16[$56>>1] = 0;
     $57 = (($lock) + 8|0);
     HEAP32[$57>>2] = 0;
     $58 = (($lock) + 4|0);
     HEAP32[$58>>2] = 0;
     $59 = HEAP32[$pFile>>2]|0;
     $60 = (_unixFileLock($59,$lock)|0);
     $61 = ($60|0)==(0);
     if ($61) {
      $62 = HEAP32[$pInode>>2]|0;
      $63 = (($62) + 12|0);
      HEAP8[$63>>0] = 0;
      break;
     } else {
      HEAP32[$rc>>2] = 2058;
      $64 = (___errno_location()|0);
      $65 = HEAP32[$64>>2]|0;
      $66 = HEAP32[$pFile>>2]|0;
      $67 = (($66) + 20|0);
      HEAP32[$67>>2] = $65;
      $68 = HEAP32[$pInode>>2]|0;
      $69 = (($68) + 12|0);
      HEAP8[$69>>0] = 0;
      $70 = HEAP32[$pFile>>2]|0;
      $71 = (($70) + 16|0);
      HEAP8[$71>>0] = 0;
      break;
     }
    }
   } while(0);
   $72 = HEAP32[$pInode>>2]|0;
   $73 = (($72) + 24|0);
   $74 = HEAP32[$73>>2]|0;
   $75 = (($74) + -1)|0;
   HEAP32[$73>>2] = $75;
   $76 = HEAP32[$pInode>>2]|0;
   $77 = (($76) + 24|0);
   $78 = HEAP32[$77>>2]|0;
   $79 = ($78|0)==(0);
   if ($79) {
    $80 = HEAP32[$pFile>>2]|0;
    _closePendingFds($80);
   }
  }
 }
 _unixLeaveMutex();
 $81 = HEAP32[$rc>>2]|0;
 $82 = ($81|0)==(0);
 if ($82) {
  $83 = HEAP32[$2>>2]|0;
  $84 = $83&255;
  $85 = HEAP32[$pFile>>2]|0;
  $86 = (($85) + 16|0);
  HEAP8[$86>>0] = $84;
 }
 $87 = HEAP32[$rc>>2]|0;
 HEAP32[$0>>2] = $87;
 $88 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($88|0);
}
function _unixFileLock($pFile,$pLock) {
 $pFile = $pFile|0;
 $pLock = $pLock|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $lock = 0, $pInode = 0, $rc = 0, $vararg_buffer1 = 0, label = 0;
 var sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $vararg_buffer1 = sp;
 $0 = sp + 40|0;
 $1 = sp + 36|0;
 $2 = sp + 32|0;
 $rc = sp + 28|0;
 $pInode = sp + 24|0;
 $lock = sp + 8|0;
 HEAP32[$1>>2] = $pFile;
 HEAP32[$2>>2] = $pLock;
 $3 = HEAP32[$1>>2]|0;
 $4 = (($3) + 8|0);
 $5 = HEAP32[$4>>2]|0;
 HEAP32[$pInode>>2] = $5;
 $6 = HEAP32[$1>>2]|0;
 $7 = (($6) + 18|0);
 $8 = HEAP16[$7>>1]|0;
 $9 = $8&65535;
 $10 = $9 & 1;
 $11 = ($10|0)!=(0);
 if ($11) {
  label = 3;
 } else {
  $12 = HEAP32[$pInode>>2]|0;
  $13 = (($12) + 13|0);
  $14 = HEAP8[$13>>0]|0;
  $15 = $14&255;
  $16 = ($15|0)!=(0);
  if ($16) {
   label = 3;
  } else {
   label = 9;
  }
 }
 do {
  if ((label|0) == 3) {
   $17 = HEAP32[$1>>2]|0;
   $18 = (($17) + 18|0);
   $19 = HEAP16[$18>>1]|0;
   $20 = $19&65535;
   $21 = $20 & 2;
   $22 = ($21|0)==(0);
   if ($22) {
    $23 = HEAP32[$pInode>>2]|0;
    $24 = (($23) + 13|0);
    $25 = HEAP8[$24>>0]|0;
    $26 = $25&255;
    $27 = ($26|0)==(0);
    if (!($27)) {
     HEAP32[$rc>>2] = 0;
     break;
    }
    $28 = (($lock) + 2|0);
    HEAP16[$28>>1] = 0;
    $29 = HEAP32[1128>>2]|0;
    $30 = (($29) + 2)|0;
    $31 = (($lock) + 4|0);
    HEAP32[$31>>2] = $30;
    $32 = (($lock) + 8|0);
    HEAP32[$32>>2] = 510;
    HEAP16[$lock>>1] = 1;
    $33 = HEAP32[((37192 + 88|0))>>2]|0;
    $34 = HEAP32[$1>>2]|0;
    $35 = (($34) + 12|0);
    $36 = HEAP32[$35>>2]|0;
    HEAP32[$vararg_buffer1>>2] = $lock;
    $37 = (FUNCTION_TABLE_iiii[$33 & 31]($36,13,$vararg_buffer1)|0);
    HEAP32[$rc>>2] = $37;
    $38 = HEAP32[$rc>>2]|0;
    $39 = ($38|0)<(0);
    if (!($39)) {
     $41 = HEAP32[$pInode>>2]|0;
     $42 = (($41) + 13|0);
     HEAP8[$42>>0] = 1;
     $43 = HEAP32[$pInode>>2]|0;
     $44 = (($43) + 24|0);
     $45 = HEAP32[$44>>2]|0;
     $46 = (($45) + 1)|0;
     HEAP32[$44>>2] = $46;
     break;
    }
    $40 = HEAP32[$rc>>2]|0;
    HEAP32[$0>>2] = $40;
    $54 = HEAP32[$0>>2]|0;
    STACKTOP = sp;return ($54|0);
   } else {
    label = 9;
   }
  }
 } while(0);
 if ((label|0) == 9) {
  $47 = HEAP32[((37192 + 88|0))>>2]|0;
  $48 = HEAP32[$1>>2]|0;
  $49 = (($48) + 12|0);
  $50 = HEAP32[$49>>2]|0;
  $51 = HEAP32[$2>>2]|0;
  HEAP32[$vararg_buffer1>>2] = $51;
  $52 = (FUNCTION_TABLE_iiii[$47 & 31]($50,13,$vararg_buffer1)|0);
  HEAP32[$rc>>2] = $52;
 }
 $53 = HEAP32[$rc>>2]|0;
 HEAP32[$0>>2] = $53;
 $54 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($54|0);
}
function _closePendingFds($pFile) {
 $pFile = $pFile|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $p = 0;
 var $pInode = 0, $pNext = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $pInode = sp + 8|0;
 $p = sp + 4|0;
 $pNext = sp;
 HEAP32[$0>>2] = $pFile;
 $1 = HEAP32[$0>>2]|0;
 $2 = (($1) + 8|0);
 $3 = HEAP32[$2>>2]|0;
 HEAP32[$pInode>>2] = $3;
 $4 = HEAP32[$pInode>>2]|0;
 $5 = (($4) + 28|0);
 $6 = HEAP32[$5>>2]|0;
 HEAP32[$p>>2] = $6;
 while(1) {
  $7 = HEAP32[$p>>2]|0;
  $8 = ($7|0)!=(0|0);
  if (!($8)) {
   break;
  }
  $9 = HEAP32[$p>>2]|0;
  $10 = (($9) + 8|0);
  $11 = HEAP32[$10>>2]|0;
  HEAP32[$pNext>>2] = $11;
  $12 = HEAP32[$0>>2]|0;
  $13 = HEAP32[$p>>2]|0;
  $14 = HEAP32[$13>>2]|0;
  _robust_close($12,$14,25997);
  $15 = HEAP32[$p>>2]|0;
  _sqlite3_free($15);
  $16 = HEAP32[$pNext>>2]|0;
  HEAP32[$p>>2] = $16;
 }
 $17 = HEAP32[$pInode>>2]|0;
 $18 = (($17) + 28|0);
 HEAP32[$18>>2] = 0;
 STACKTOP = sp;return;
}
function _setPendingFd($pFile) {
 $pFile = $pFile|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $p = 0;
 var $pInode = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $pInode = sp + 4|0;
 $p = sp;
 HEAP32[$0>>2] = $pFile;
 $1 = HEAP32[$0>>2]|0;
 $2 = (($1) + 8|0);
 $3 = HEAP32[$2>>2]|0;
 HEAP32[$pInode>>2] = $3;
 $4 = HEAP32[$0>>2]|0;
 $5 = (($4) + 28|0);
 $6 = HEAP32[$5>>2]|0;
 HEAP32[$p>>2] = $6;
 $7 = HEAP32[$pInode>>2]|0;
 $8 = (($7) + 28|0);
 $9 = HEAP32[$8>>2]|0;
 $10 = HEAP32[$p>>2]|0;
 $11 = (($10) + 8|0);
 HEAP32[$11>>2] = $9;
 $12 = HEAP32[$p>>2]|0;
 $13 = HEAP32[$pInode>>2]|0;
 $14 = (($13) + 28|0);
 HEAP32[$14>>2] = $12;
 $15 = HEAP32[$0>>2]|0;
 $16 = (($15) + 12|0);
 HEAP32[$16>>2] = -1;
 $17 = HEAP32[$0>>2]|0;
 $18 = (($17) + 28|0);
 HEAP32[$18>>2] = 0;
 STACKTOP = sp;return;
}
function _releaseInodeInfo($pFile) {
 $pFile = $pFile|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $pInode = 0, label = 0;
 var sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $pInode = sp;
 HEAP32[$0>>2] = $pFile;
 $1 = HEAP32[$0>>2]|0;
 $2 = (($1) + 8|0);
 $3 = HEAP32[$2>>2]|0;
 HEAP32[$pInode>>2] = $3;
 $4 = HEAP32[$pInode>>2]|0;
 $5 = ($4|0)!=(0|0);
 if (!($5)) {
  STACKTOP = sp;return;
 }
 $6 = HEAP32[$pInode>>2]|0;
 $7 = (($6) + 16|0);
 $8 = HEAP32[$7>>2]|0;
 $9 = (($8) + -1)|0;
 HEAP32[$7>>2] = $9;
 $10 = HEAP32[$pInode>>2]|0;
 $11 = (($10) + 16|0);
 $12 = HEAP32[$11>>2]|0;
 $13 = ($12|0)==(0);
 if (!($13)) {
  STACKTOP = sp;return;
 }
 $14 = HEAP32[$0>>2]|0;
 _closePendingFds($14);
 $15 = HEAP32[$pInode>>2]|0;
 $16 = (($15) + 36|0);
 $17 = HEAP32[$16>>2]|0;
 $18 = ($17|0)!=(0|0);
 $19 = HEAP32[$pInode>>2]|0;
 $20 = (($19) + 32|0);
 $21 = HEAP32[$20>>2]|0;
 if ($18) {
  $22 = HEAP32[$pInode>>2]|0;
  $23 = (($22) + 36|0);
  $24 = HEAP32[$23>>2]|0;
  $25 = (($24) + 32|0);
  HEAP32[$25>>2] = $21;
 } else {
  HEAP32[38352>>2] = $21;
 }
 $26 = HEAP32[$pInode>>2]|0;
 $27 = (($26) + 32|0);
 $28 = HEAP32[$27>>2]|0;
 $29 = ($28|0)!=(0|0);
 if ($29) {
  $30 = HEAP32[$pInode>>2]|0;
  $31 = (($30) + 36|0);
  $32 = HEAP32[$31>>2]|0;
  $33 = HEAP32[$pInode>>2]|0;
  $34 = (($33) + 32|0);
  $35 = HEAP32[$34>>2]|0;
  $36 = (($35) + 36|0);
  HEAP32[$36>>2] = $32;
 }
 $37 = HEAP32[$pInode>>2]|0;
 _sqlite3_free($37);
 STACKTOP = sp;return;
}
function _posixIoFinderImpl($z,$p) {
 $z = $z|0;
 $p = $p|0;
 var $0 = 0, $1 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $1 = sp;
 HEAP32[$0>>2] = $z;
 HEAP32[$1>>2] = $p;
 STACKTOP = sp;return (38144|0);
}
function _getIntArg($p) {
 $p = $p|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 $1 = sp + 8|0;
 HEAP32[$1>>2] = $p;
 $2 = HEAP32[$1>>2]|0;
 $3 = HEAP32[$2>>2]|0;
 $4 = HEAP32[$1>>2]|0;
 $5 = (($4) + 4|0);
 $6 = HEAP32[$5>>2]|0;
 $7 = ($3|0)<=($6|0);
 if ($7) {
  $8 = $0;
  $9 = $8;
  HEAP32[$9>>2] = 0;
  $10 = (($8) + 4)|0;
  $11 = $10;
  HEAP32[$11>>2] = 0;
  $27 = $0;
  $28 = $27;
  $29 = HEAP32[$28>>2]|0;
  $30 = (($27) + 4)|0;
  $31 = $30;
  $32 = HEAP32[$31>>2]|0;
  tempRet0 = $32;
  STACKTOP = sp;return ($29|0);
 } else {
  $12 = HEAP32[$1>>2]|0;
  $13 = (($12) + 4|0);
  $14 = HEAP32[$13>>2]|0;
  $15 = (($14) + 1)|0;
  HEAP32[$13>>2] = $15;
  $16 = HEAP32[$1>>2]|0;
  $17 = (($16) + 8|0);
  $18 = HEAP32[$17>>2]|0;
  $19 = (($18) + ($14<<2)|0);
  $20 = HEAP32[$19>>2]|0;
  $21 = (_sqlite3_value_int64($20)|0);
  $22 = tempRet0;
  $23 = $0;
  $24 = $23;
  HEAP32[$24>>2] = $21;
  $25 = (($23) + 4)|0;
  $26 = $25;
  HEAP32[$26>>2] = $22;
  $27 = $0;
  $28 = $27;
  $29 = HEAP32[$28>>2]|0;
  $30 = (($27) + 4)|0;
  $31 = $30;
  $32 = HEAP32[$31>>2]|0;
  tempRet0 = $32;
  STACKTOP = sp;return ($29|0);
 }
 return 0|0;
}
function _getDoubleArg($p) {
 $p = $p|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0.0, $18 = 0.0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0;
 var sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 $1 = sp + 8|0;
 HEAP32[$1>>2] = $p;
 $2 = HEAP32[$1>>2]|0;
 $3 = HEAP32[$2>>2]|0;
 $4 = HEAP32[$1>>2]|0;
 $5 = (($4) + 4|0);
 $6 = HEAP32[$5>>2]|0;
 $7 = ($3|0)<=($6|0);
 if ($7) {
  HEAPF64[$0>>3] = 0.0;
  $18 = +HEAPF64[$0>>3];
  STACKTOP = sp;return (+$18);
 } else {
  $8 = HEAP32[$1>>2]|0;
  $9 = (($8) + 4|0);
  $10 = HEAP32[$9>>2]|0;
  $11 = (($10) + 1)|0;
  HEAP32[$9>>2] = $11;
  $12 = HEAP32[$1>>2]|0;
  $13 = (($12) + 8|0);
  $14 = HEAP32[$13>>2]|0;
  $15 = (($14) + ($10<<2)|0);
  $16 = HEAP32[$15>>2]|0;
  $17 = (+_sqlite3_value_double($16));
  HEAPF64[$0>>3] = $17;
  $18 = +HEAPF64[$0>>3];
  STACKTOP = sp;return (+$18);
 }
 return +0;
}
function _et_getdigit($val,$cnt) {
 $val = $val|0;
 $cnt = $cnt|0;
 var $0 = 0, $1 = 0, $10 = 0.0, $11 = 0, $12 = 0, $13 = 0.0, $14 = 0, $15 = 0, $16 = 0, $17 = 0.0, $18 = 0.0, $19 = 0.0, $2 = 0, $20 = 0.0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $3 = 0, $4 = 0;
 var $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $d = 0, $digit = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 20|0;
 $1 = sp + 16|0;
 $2 = sp + 12|0;
 $digit = sp + 8|0;
 $d = sp;
 HEAP32[$1>>2] = $val;
 HEAP32[$2>>2] = $cnt;
 $3 = HEAP32[$2>>2]|0;
 $4 = HEAP32[$3>>2]|0;
 $5 = ($4|0)<=(0);
 if ($5) {
  HEAP8[$0>>0] = 48;
  $24 = HEAP8[$0>>0]|0;
  STACKTOP = sp;return ($24|0);
 } else {
  $6 = HEAP32[$2>>2]|0;
  $7 = HEAP32[$6>>2]|0;
  $8 = (($7) + -1)|0;
  HEAP32[$6>>2] = $8;
  $9 = HEAP32[$1>>2]|0;
  $10 = +HEAPF64[$9>>3];
  $11 = (~~(($10)));
  HEAP32[$digit>>2] = $11;
  $12 = HEAP32[$digit>>2]|0;
  $13 = (+($12|0));
  HEAPF64[$d>>3] = $13;
  $14 = HEAP32[$digit>>2]|0;
  $15 = (($14) + 48)|0;
  HEAP32[$digit>>2] = $15;
  $16 = HEAP32[$1>>2]|0;
  $17 = +HEAPF64[$16>>3];
  $18 = +HEAPF64[$d>>3];
  $19 = $17 - $18;
  $20 = $19 * 10.0;
  $21 = HEAP32[$1>>2]|0;
  HEAPF64[$21>>3] = $20;
  $22 = HEAP32[$digit>>2]|0;
  $23 = $22&255;
  HEAP8[$0>>0] = $23;
  $24 = HEAP8[$0>>0]|0;
  STACKTOP = sp;return ($24|0);
 }
 return 0|0;
}
function _getTextArg($p) {
 $p = $p|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0;
 var sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $1 = sp;
 HEAP32[$1>>2] = $p;
 $2 = HEAP32[$1>>2]|0;
 $3 = HEAP32[$2>>2]|0;
 $4 = HEAP32[$1>>2]|0;
 $5 = (($4) + 4|0);
 $6 = HEAP32[$5>>2]|0;
 $7 = ($3|0)<=($6|0);
 if ($7) {
  HEAP32[$0>>2] = 0;
  $18 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($18|0);
 } else {
  $8 = HEAP32[$1>>2]|0;
  $9 = (($8) + 4|0);
  $10 = HEAP32[$9>>2]|0;
  $11 = (($10) + 1)|0;
  HEAP32[$9>>2] = $11;
  $12 = HEAP32[$1>>2]|0;
  $13 = (($12) + 8|0);
  $14 = HEAP32[$13>>2]|0;
  $15 = (($14) + ($10<<2)|0);
  $16 = HEAP32[$15>>2]|0;
  $17 = (_sqlite3_value_text($16)|0);
  HEAP32[$0>>2] = $17;
  $18 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($18|0);
 }
 return 0|0;
}
function _sqlite3AppendChar($p,$N,$c) {
 $p = $p|0;
 $N = $N|0;
 $c = $c|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $1 = sp;
 $2 = sp + 8|0;
 HEAP32[$0>>2] = $p;
 HEAP32[$1>>2] = $N;
 HEAP8[$2>>0] = $c;
 $3 = HEAP32[$0>>2]|0;
 $4 = (($3) + 12|0);
 $5 = HEAP32[$4>>2]|0;
 $6 = HEAP32[$1>>2]|0;
 $7 = (($5) + ($6))|0;
 $8 = HEAP32[$0>>2]|0;
 $9 = (($8) + 16|0);
 $10 = HEAP32[$9>>2]|0;
 $11 = ($7|0)>=($10|0);
 if ($11) {
  $12 = HEAP32[$0>>2]|0;
  $13 = HEAP32[$1>>2]|0;
  $14 = (_sqlite3StrAccumEnlarge($12,$13)|0);
  HEAP32[$1>>2] = $14;
  $15 = ($14|0)<=(0);
  if ($15) {
   STACKTOP = sp;return;
  }
 }
 while(1) {
  $16 = HEAP32[$1>>2]|0;
  $17 = (($16) + -1)|0;
  HEAP32[$1>>2] = $17;
  $18 = ($16|0)>(0);
  if (!($18)) {
   break;
  }
  $19 = HEAP8[$2>>0]|0;
  $20 = HEAP32[$0>>2]|0;
  $21 = (($20) + 12|0);
  $22 = HEAP32[$21>>2]|0;
  $23 = (($22) + 1)|0;
  HEAP32[$21>>2] = $23;
  $24 = HEAP32[$0>>2]|0;
  $25 = (($24) + 8|0);
  $26 = HEAP32[$25>>2]|0;
  $27 = (($26) + ($22)|0);
  HEAP8[$27>>0] = $19;
 }
 STACKTOP = sp;return;
}
function _sqlite3StatusValue($op) {
 $op = $op|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 HEAP32[$0>>2] = $op;
 $1 = HEAP32[$0>>2]|0;
 $2 = (48 + ($1<<2)|0);
 $3 = HEAP32[$2>>2]|0;
 STACKTOP = sp;return ($3|0);
}
function _sqlite3MallocAlarm($nByte) {
 $nByte = $nByte|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0;
 var $7 = 0, $8 = 0, $9 = 0, $nowUsed = 0, $pArg = 0, $xCallback = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 16|0;
 $xCallback = sp + 12|0;
 $nowUsed = sp;
 $pArg = sp + 8|0;
 HEAP32[$0>>2] = $nByte;
 $1 = HEAP32[((136 + 16|0))>>2]|0;
 $2 = ($1|0)==(0|0);
 if ($2) {
  STACKTOP = sp;return;
 }
 $3 = HEAP32[((136 + 16|0))>>2]|0;
 HEAP32[$xCallback>>2] = $3;
 $4 = (_sqlite3StatusValue(0)|0);
 $5 = ($4|0)<(0);
 $6 = $5 << 31 >> 31;
 $7 = $nowUsed;
 $8 = $7;
 HEAP32[$8>>2] = $4;
 $9 = (($7) + 4)|0;
 $10 = $9;
 HEAP32[$10>>2] = $6;
 $11 = HEAP32[((136 + 20|0))>>2]|0;
 HEAP32[$pArg>>2] = $11;
 HEAP32[((136 + 16|0))>>2] = 0;
 $12 = HEAP32[$xCallback>>2]|0;
 $13 = HEAP32[$pArg>>2]|0;
 $14 = $nowUsed;
 $15 = $14;
 $16 = HEAP32[$15>>2]|0;
 $17 = (($14) + 4)|0;
 $18 = $17;
 $19 = HEAP32[$18>>2]|0;
 $20 = HEAP32[$0>>2]|0;
 FUNCTION_TABLE_viiii[$12 & 3]($13,$16,$19,$20);
 $21 = HEAP32[$xCallback>>2]|0;
 HEAP32[((136 + 16|0))>>2] = $21;
 $22 = HEAP32[$pArg>>2]|0;
 HEAP32[((136 + 20|0))>>2] = $22;
 STACKTOP = sp;return;
}
function _measureAllocationSize($db,$p) {
 $db = $db|0;
 $p = $p|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $1 = sp;
 HEAP32[$0>>2] = $db;
 HEAP32[$1>>2] = $p;
 $2 = HEAP32[$0>>2]|0;
 $3 = HEAP32[$1>>2]|0;
 $4 = (_sqlite3DbMallocSize($2,$3)|0);
 $5 = HEAP32[$0>>2]|0;
 $6 = (($5) + 520|0);
 $7 = HEAP32[$6>>2]|0;
 $8 = HEAP32[$7>>2]|0;
 $9 = (($8) + ($4))|0;
 HEAP32[$7>>2] = $9;
 STACKTOP = sp;return;
}
function _mallocWithAlarm($n,$pp) {
 $n = $n|0;
 $pp = $pp|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0;
 var $nFull = 0, $nUsed = 0, $p = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 16|0;
 $1 = sp + 12|0;
 $nFull = sp + 8|0;
 $p = sp + 4|0;
 $nUsed = sp;
 HEAP32[$0>>2] = $n;
 HEAP32[$1>>2] = $pp;
 $2 = HEAP32[((38640 + 52|0))>>2]|0;
 $3 = HEAP32[$0>>2]|0;
 $4 = (FUNCTION_TABLE_ii[$2 & 31]($3)|0);
 HEAP32[$nFull>>2] = $4;
 $5 = HEAP32[$0>>2]|0;
 _sqlite3StatusSet(5,$5);
 $6 = HEAP32[((136 + 16|0))>>2]|0;
 $7 = ($6|0)!=(0|0);
 do {
  if ($7) {
   $8 = (_sqlite3StatusValue(0)|0);
   HEAP32[$nUsed>>2] = $8;
   $9 = HEAP32[$nUsed>>2]|0;
   $10 = ($9|0)<(0);
   $11 = $10 << 31 >> 31;
   $12 = ((136 + 8|0));
   $13 = $12;
   $14 = HEAP32[$13>>2]|0;
   $15 = (($12) + 4)|0;
   $16 = $15;
   $17 = HEAP32[$16>>2]|0;
   $18 = HEAP32[$nFull>>2]|0;
   $19 = ($18|0)<(0);
   $20 = $19 << 31 >> 31;
   $21 = (_i64Subtract(($14|0),($17|0),($18|0),($20|0))|0);
   $22 = tempRet0;
   $23 = ($11|0)>($22|0);
   $24 = ($9>>>0)>=($21>>>0);
   $25 = ($11|0)==($22|0);
   $26 = $25 & $24;
   $27 = $23 | $26;
   if ($27) {
    HEAP32[((136 + 36|0))>>2] = 1;
    $28 = HEAP32[$nFull>>2]|0;
    _sqlite3MallocAlarm($28);
    break;
   } else {
    HEAP32[((136 + 36|0))>>2] = 0;
    break;
   }
  }
 } while(0);
 $29 = HEAP32[((38640 + 36|0))>>2]|0;
 $30 = HEAP32[$nFull>>2]|0;
 $31 = (FUNCTION_TABLE_ii[$29 & 31]($30)|0);
 HEAP32[$p>>2] = $31;
 $32 = HEAP32[$p>>2]|0;
 $33 = ($32|0)!=(0|0);
 if (!($33)) {
  $37 = HEAP32[$p>>2]|0;
  $38 = HEAP32[$1>>2]|0;
  HEAP32[$38>>2] = $37;
  $39 = HEAP32[$nFull>>2]|0;
  STACKTOP = sp;return ($39|0);
 }
 $34 = HEAP32[$p>>2]|0;
 $35 = (_sqlite3MallocSize($34)|0);
 HEAP32[$nFull>>2] = $35;
 $36 = HEAP32[$nFull>>2]|0;
 _sqlite3StatusAdd(0,$36);
 _sqlite3StatusAdd(9,1);
 $37 = HEAP32[$p>>2]|0;
 $38 = HEAP32[$1>>2]|0;
 HEAP32[$38>>2] = $37;
 $39 = HEAP32[$nFull>>2]|0;
 STACKTOP = sp;return ($39|0);
}
function _malloc($bytes) {
 $bytes = $bytes|0;
 var $$pre = 0, $$pre$i = 0, $$pre$i$i = 0, $$pre$i23$i = 0, $$pre$i25 = 0, $$pre$phi$i$iZ2D = 0, $$pre$phi$i24$iZ2D = 0, $$pre$phi$i26Z2D = 0, $$pre$phi$iZ2D = 0, $$pre$phi59$i$iZ2D = 0, $$pre$phiZ2D = 0, $$pre105 = 0, $$pre58$i$i = 0, $$rsize$0$i = 0, $$rsize$3$i = 0, $$sum = 0, $$sum$i$i = 0, $$sum$i$i$i = 0, $$sum$i12$i = 0, $$sum$i13$i = 0;
 var $$sum$i16$i = 0, $$sum$i19$i = 0, $$sum$i2338 = 0, $$sum$i32 = 0, $$sum$i39 = 0, $$sum1 = 0, $$sum1$i = 0, $$sum1$i$i = 0, $$sum1$i14$i = 0, $$sum1$i20$i = 0, $$sum1$i24 = 0, $$sum10 = 0, $$sum10$i = 0, $$sum10$i$i = 0, $$sum10$pre$i$i = 0, $$sum102$i = 0, $$sum103$i = 0, $$sum104$i = 0, $$sum105$i = 0, $$sum106$i = 0;
 var $$sum107$i = 0, $$sum108$i = 0, $$sum109$i = 0, $$sum11$i = 0, $$sum11$i$i = 0, $$sum11$i22$i = 0, $$sum110$i = 0, $$sum111$i = 0, $$sum1112 = 0, $$sum112$i = 0, $$sum113$i = 0, $$sum114$i = 0, $$sum115$i = 0, $$sum12$i = 0, $$sum12$i$i = 0, $$sum13$i = 0, $$sum13$i$i = 0, $$sum14$i$i = 0, $$sum14$pre$i = 0, $$sum15$i = 0;
 var $$sum15$i$i = 0, $$sum16$i = 0, $$sum16$i$i = 0, $$sum17$i = 0, $$sum17$i$i = 0, $$sum18$i = 0, $$sum1819$i$i = 0, $$sum2 = 0, $$sum2$i = 0, $$sum2$i$i = 0, $$sum2$i$i$i = 0, $$sum2$i15$i = 0, $$sum2$i17$i = 0, $$sum2$i21$i = 0, $$sum2$pre$i = 0, $$sum20$i$i = 0, $$sum21$i$i = 0, $$sum22$i$i = 0, $$sum23$i$i = 0, $$sum24$i$i = 0;
 var $$sum25$i$i = 0, $$sum26$pre$i$i = 0, $$sum27$i$i = 0, $$sum28$i$i = 0, $$sum29$i$i = 0, $$sum3$i = 0, $$sum3$i$i = 0, $$sum3$i27 = 0, $$sum30$i$i = 0, $$sum3132$i$i = 0, $$sum34$i$i = 0, $$sum3536$i$i = 0, $$sum3738$i$i = 0, $$sum39$i$i = 0, $$sum4 = 0, $$sum4$i = 0, $$sum4$i28 = 0, $$sum40$i$i = 0, $$sum41$i$i = 0, $$sum42$i$i = 0;
 var $$sum5$i = 0, $$sum5$i$i = 0, $$sum56 = 0, $$sum6$i = 0, $$sum67$i$i = 0, $$sum7$i = 0, $$sum8$i = 0, $$sum8$pre = 0, $$sum9 = 0, $$sum9$i = 0, $$sum9$i$i = 0, $$tsize$1$i = 0, $$v$0$i = 0, $0 = 0, $1 = 0, $10 = 0, $100 = 0, $1000 = 0, $1001 = 0, $1002 = 0;
 var $1003 = 0, $1004 = 0, $1005 = 0, $1006 = 0, $1007 = 0, $1008 = 0, $1009 = 0, $101 = 0, $1010 = 0, $1011 = 0, $1012 = 0, $1013 = 0, $1014 = 0, $1015 = 0, $1016 = 0, $1017 = 0, $1018 = 0, $1019 = 0, $102 = 0, $1020 = 0;
 var $1021 = 0, $1022 = 0, $1023 = 0, $1024 = 0, $1025 = 0, $1026 = 0, $1027 = 0, $1028 = 0, $1029 = 0, $103 = 0, $1030 = 0, $1031 = 0, $1032 = 0, $1033 = 0, $1034 = 0, $1035 = 0, $1036 = 0, $1037 = 0, $1038 = 0, $1039 = 0;
 var $104 = 0, $1040 = 0, $1041 = 0, $1042 = 0, $1043 = 0, $1044 = 0, $1045 = 0, $1046 = 0, $1047 = 0, $1048 = 0, $1049 = 0, $105 = 0, $1050 = 0, $1051 = 0, $1052 = 0, $1053 = 0, $1054 = 0, $1055 = 0, $1056 = 0, $1057 = 0;
 var $1058 = 0, $1059 = 0, $106 = 0, $1060 = 0, $1061 = 0, $1062 = 0, $1063 = 0, $1064 = 0, $1065 = 0, $1066 = 0, $1067 = 0, $1068 = 0, $1069 = 0, $107 = 0, $1070 = 0, $1071 = 0, $1072 = 0, $1073 = 0, $1074 = 0, $108 = 0;
 var $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0, $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0;
 var $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0, $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0;
 var $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0, $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0;
 var $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0, $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0;
 var $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0, $188 = 0, $189 = 0, $19 = 0, $190 = 0, $191 = 0, $192 = 0, $193 = 0, $194 = 0, $195 = 0, $196 = 0, $197 = 0, $198 = 0, $199 = 0;
 var $2 = 0, $20 = 0, $200 = 0, $201 = 0, $202 = 0, $203 = 0, $204 = 0, $205 = 0, $206 = 0, $207 = 0, $208 = 0, $209 = 0, $21 = 0, $210 = 0, $211 = 0, $212 = 0, $213 = 0, $214 = 0, $215 = 0, $216 = 0;
 var $217 = 0, $218 = 0, $219 = 0, $22 = 0, $220 = 0, $221 = 0, $222 = 0, $223 = 0, $224 = 0, $225 = 0, $226 = 0, $227 = 0, $228 = 0, $229 = 0, $23 = 0, $230 = 0, $231 = 0, $232 = 0, $233 = 0, $234 = 0;
 var $235 = 0, $236 = 0, $237 = 0, $238 = 0, $239 = 0, $24 = 0, $240 = 0, $241 = 0, $242 = 0, $243 = 0, $244 = 0, $245 = 0, $246 = 0, $247 = 0, $248 = 0, $249 = 0, $25 = 0, $250 = 0, $251 = 0, $252 = 0;
 var $253 = 0, $254 = 0, $255 = 0, $256 = 0, $257 = 0, $258 = 0, $259 = 0, $26 = 0, $260 = 0, $261 = 0, $262 = 0, $263 = 0, $264 = 0, $265 = 0, $266 = 0, $267 = 0, $268 = 0, $269 = 0, $27 = 0, $270 = 0;
 var $271 = 0, $272 = 0, $273 = 0, $274 = 0, $275 = 0, $276 = 0, $277 = 0, $278 = 0, $279 = 0, $28 = 0, $280 = 0, $281 = 0, $282 = 0, $283 = 0, $284 = 0, $285 = 0, $286 = 0, $287 = 0, $288 = 0, $289 = 0;
 var $29 = 0, $290 = 0, $291 = 0, $292 = 0, $293 = 0, $294 = 0, $295 = 0, $296 = 0, $297 = 0, $298 = 0, $299 = 0, $3 = 0, $30 = 0, $300 = 0, $301 = 0, $302 = 0, $303 = 0, $304 = 0, $305 = 0, $306 = 0;
 var $307 = 0, $308 = 0, $309 = 0, $31 = 0, $310 = 0, $311 = 0, $312 = 0, $313 = 0, $314 = 0, $315 = 0, $316 = 0, $317 = 0, $318 = 0, $319 = 0, $32 = 0, $320 = 0, $321 = 0, $322 = 0, $323 = 0, $324 = 0;
 var $325 = 0, $326 = 0, $327 = 0, $328 = 0, $329 = 0, $33 = 0, $330 = 0, $331 = 0, $332 = 0, $333 = 0, $334 = 0, $335 = 0, $336 = 0, $337 = 0, $338 = 0, $339 = 0, $34 = 0, $340 = 0, $341 = 0, $342 = 0;
 var $343 = 0, $344 = 0, $345 = 0, $346 = 0, $347 = 0, $348 = 0, $349 = 0, $35 = 0, $350 = 0, $351 = 0, $352 = 0, $353 = 0, $354 = 0, $355 = 0, $356 = 0, $357 = 0, $358 = 0, $359 = 0, $36 = 0, $360 = 0;
 var $361 = 0, $362 = 0, $363 = 0, $364 = 0, $365 = 0, $366 = 0, $367 = 0, $368 = 0, $369 = 0, $37 = 0, $370 = 0, $371 = 0, $372 = 0, $373 = 0, $374 = 0, $375 = 0, $376 = 0, $377 = 0, $378 = 0, $379 = 0;
 var $38 = 0, $380 = 0, $381 = 0, $382 = 0, $383 = 0, $384 = 0, $385 = 0, $386 = 0, $387 = 0, $388 = 0, $389 = 0, $39 = 0, $390 = 0, $391 = 0, $392 = 0, $393 = 0, $394 = 0, $395 = 0, $396 = 0, $397 = 0;
 var $398 = 0, $399 = 0, $4 = 0, $40 = 0, $400 = 0, $401 = 0, $402 = 0, $403 = 0, $404 = 0, $405 = 0, $406 = 0, $407 = 0, $408 = 0, $409 = 0, $41 = 0, $410 = 0, $411 = 0, $412 = 0, $413 = 0, $414 = 0;
 var $415 = 0, $416 = 0, $417 = 0, $418 = 0, $419 = 0, $42 = 0, $420 = 0, $421 = 0, $422 = 0, $423 = 0, $424 = 0, $425 = 0, $426 = 0, $427 = 0, $428 = 0, $429 = 0, $43 = 0, $430 = 0, $431 = 0, $432 = 0;
 var $433 = 0, $434 = 0, $435 = 0, $436 = 0, $437 = 0, $438 = 0, $439 = 0, $44 = 0, $440 = 0, $441 = 0, $442 = 0, $443 = 0, $444 = 0, $445 = 0, $446 = 0, $447 = 0, $448 = 0, $449 = 0, $45 = 0, $450 = 0;
 var $451 = 0, $452 = 0, $453 = 0, $454 = 0, $455 = 0, $456 = 0, $457 = 0, $458 = 0, $459 = 0, $46 = 0, $460 = 0, $461 = 0, $462 = 0, $463 = 0, $464 = 0, $465 = 0, $466 = 0, $467 = 0, $468 = 0, $469 = 0;
 var $47 = 0, $470 = 0, $471 = 0, $472 = 0, $473 = 0, $474 = 0, $475 = 0, $476 = 0, $477 = 0, $478 = 0, $479 = 0, $48 = 0, $480 = 0, $481 = 0, $482 = 0, $483 = 0, $484 = 0, $485 = 0, $486 = 0, $487 = 0;
 var $488 = 0, $489 = 0, $49 = 0, $490 = 0, $491 = 0, $492 = 0, $493 = 0, $494 = 0, $495 = 0, $496 = 0, $497 = 0, $498 = 0, $499 = 0, $5 = 0, $50 = 0, $500 = 0, $501 = 0, $502 = 0, $503 = 0, $504 = 0;
 var $505 = 0, $506 = 0, $507 = 0, $508 = 0, $509 = 0, $51 = 0, $510 = 0, $511 = 0, $512 = 0, $513 = 0, $514 = 0, $515 = 0, $516 = 0, $517 = 0, $518 = 0, $519 = 0, $52 = 0, $520 = 0, $521 = 0, $522 = 0;
 var $523 = 0, $524 = 0, $525 = 0, $526 = 0, $527 = 0, $528 = 0, $529 = 0, $53 = 0, $530 = 0, $531 = 0, $532 = 0, $533 = 0, $534 = 0, $535 = 0, $536 = 0, $537 = 0, $538 = 0, $539 = 0, $54 = 0, $540 = 0;
 var $541 = 0, $542 = 0, $543 = 0, $544 = 0, $545 = 0, $546 = 0, $547 = 0, $548 = 0, $549 = 0, $55 = 0, $550 = 0, $551 = 0, $552 = 0, $553 = 0, $554 = 0, $555 = 0, $556 = 0, $557 = 0, $558 = 0, $559 = 0;
 var $56 = 0, $560 = 0, $561 = 0, $562 = 0, $563 = 0, $564 = 0, $565 = 0, $566 = 0, $567 = 0, $568 = 0, $569 = 0, $57 = 0, $570 = 0, $571 = 0, $572 = 0, $573 = 0, $574 = 0, $575 = 0, $576 = 0, $577 = 0;
 var $578 = 0, $579 = 0, $58 = 0, $580 = 0, $581 = 0, $582 = 0, $583 = 0, $584 = 0, $585 = 0, $586 = 0, $587 = 0, $588 = 0, $589 = 0, $59 = 0, $590 = 0, $591 = 0, $592 = 0, $593 = 0, $594 = 0, $595 = 0;
 var $596 = 0, $597 = 0, $598 = 0, $599 = 0, $6 = 0, $60 = 0, $600 = 0, $601 = 0, $602 = 0, $603 = 0, $604 = 0, $605 = 0, $606 = 0, $607 = 0, $608 = 0, $609 = 0, $61 = 0, $610 = 0, $611 = 0, $612 = 0;
 var $613 = 0, $614 = 0, $615 = 0, $616 = 0, $617 = 0, $618 = 0, $619 = 0, $62 = 0, $620 = 0, $621 = 0, $622 = 0, $623 = 0, $624 = 0, $625 = 0, $626 = 0, $627 = 0, $628 = 0, $629 = 0, $63 = 0, $630 = 0;
 var $631 = 0, $632 = 0, $633 = 0, $634 = 0, $635 = 0, $636 = 0, $637 = 0, $638 = 0, $639 = 0, $64 = 0, $640 = 0, $641 = 0, $642 = 0, $643 = 0, $644 = 0, $645 = 0, $646 = 0, $647 = 0, $648 = 0, $649 = 0;
 var $65 = 0, $650 = 0, $651 = 0, $652 = 0, $653 = 0, $654 = 0, $655 = 0, $656 = 0, $657 = 0, $658 = 0, $659 = 0, $66 = 0, $660 = 0, $661 = 0, $662 = 0, $663 = 0, $664 = 0, $665 = 0, $666 = 0, $667 = 0;
 var $668 = 0, $669 = 0, $67 = 0, $670 = 0, $671 = 0, $672 = 0, $673 = 0, $674 = 0, $675 = 0, $676 = 0, $677 = 0, $678 = 0, $679 = 0, $68 = 0, $680 = 0, $681 = 0, $682 = 0, $683 = 0, $684 = 0, $685 = 0;
 var $686 = 0, $687 = 0, $688 = 0, $689 = 0, $69 = 0, $690 = 0, $691 = 0, $692 = 0, $693 = 0, $694 = 0, $695 = 0, $696 = 0, $697 = 0, $698 = 0, $699 = 0, $7 = 0, $70 = 0, $700 = 0, $701 = 0, $702 = 0;
 var $703 = 0, $704 = 0, $705 = 0, $706 = 0, $707 = 0, $708 = 0, $709 = 0, $71 = 0, $710 = 0, $711 = 0, $712 = 0, $713 = 0, $714 = 0, $715 = 0, $716 = 0, $717 = 0, $718 = 0, $719 = 0, $72 = 0, $720 = 0;
 var $721 = 0, $722 = 0, $723 = 0, $724 = 0, $725 = 0, $726 = 0, $727 = 0, $728 = 0, $729 = 0, $73 = 0, $730 = 0, $731 = 0, $732 = 0, $733 = 0, $734 = 0, $735 = 0, $736 = 0, $737 = 0, $738 = 0, $739 = 0;
 var $74 = 0, $740 = 0, $741 = 0, $742 = 0, $743 = 0, $744 = 0, $745 = 0, $746 = 0, $747 = 0, $748 = 0, $749 = 0, $75 = 0, $750 = 0, $751 = 0, $752 = 0, $753 = 0, $754 = 0, $755 = 0, $756 = 0, $757 = 0;
 var $758 = 0, $759 = 0, $76 = 0, $760 = 0, $761 = 0, $762 = 0, $763 = 0, $764 = 0, $765 = 0, $766 = 0, $767 = 0, $768 = 0, $769 = 0, $77 = 0, $770 = 0, $771 = 0, $772 = 0, $773 = 0, $774 = 0, $775 = 0;
 var $776 = 0, $777 = 0, $778 = 0, $779 = 0, $78 = 0, $780 = 0, $781 = 0, $782 = 0, $783 = 0, $784 = 0, $785 = 0, $786 = 0, $787 = 0, $788 = 0, $789 = 0, $79 = 0, $790 = 0, $791 = 0, $792 = 0, $793 = 0;
 var $794 = 0, $795 = 0, $796 = 0, $797 = 0, $798 = 0, $799 = 0, $8 = 0, $80 = 0, $800 = 0, $801 = 0, $802 = 0, $803 = 0, $804 = 0, $805 = 0, $806 = 0, $807 = 0, $808 = 0, $809 = 0, $81 = 0, $810 = 0;
 var $811 = 0, $812 = 0, $813 = 0, $814 = 0, $815 = 0, $816 = 0, $817 = 0, $818 = 0, $819 = 0, $82 = 0, $820 = 0, $821 = 0, $822 = 0, $823 = 0, $824 = 0, $825 = 0, $826 = 0, $827 = 0, $828 = 0, $829 = 0;
 var $83 = 0, $830 = 0, $831 = 0, $832 = 0, $833 = 0, $834 = 0, $835 = 0, $836 = 0, $837 = 0, $838 = 0, $839 = 0, $84 = 0, $840 = 0, $841 = 0, $842 = 0, $843 = 0, $844 = 0, $845 = 0, $846 = 0, $847 = 0;
 var $848 = 0, $849 = 0, $85 = 0, $850 = 0, $851 = 0, $852 = 0, $853 = 0, $854 = 0, $855 = 0, $856 = 0, $857 = 0, $858 = 0, $859 = 0, $86 = 0, $860 = 0, $861 = 0, $862 = 0, $863 = 0, $864 = 0, $865 = 0;
 var $866 = 0, $867 = 0, $868 = 0, $869 = 0, $87 = 0, $870 = 0, $871 = 0, $872 = 0, $873 = 0, $874 = 0, $875 = 0, $876 = 0, $877 = 0, $878 = 0, $879 = 0, $88 = 0, $880 = 0, $881 = 0, $882 = 0, $883 = 0;
 var $884 = 0, $885 = 0, $886 = 0, $887 = 0, $888 = 0, $889 = 0, $89 = 0, $890 = 0, $891 = 0, $892 = 0, $893 = 0, $894 = 0, $895 = 0, $896 = 0, $897 = 0, $898 = 0, $899 = 0, $9 = 0, $90 = 0, $900 = 0;
 var $901 = 0, $902 = 0, $903 = 0, $904 = 0, $905 = 0, $906 = 0, $907 = 0, $908 = 0, $909 = 0, $91 = 0, $910 = 0, $911 = 0, $912 = 0, $913 = 0, $914 = 0, $915 = 0, $916 = 0, $917 = 0, $918 = 0, $919 = 0;
 var $92 = 0, $920 = 0, $921 = 0, $922 = 0, $923 = 0, $924 = 0, $925 = 0, $926 = 0, $927 = 0, $928 = 0, $929 = 0, $93 = 0, $930 = 0, $931 = 0, $932 = 0, $933 = 0, $934 = 0, $935 = 0, $936 = 0, $937 = 0;
 var $938 = 0, $939 = 0, $94 = 0, $940 = 0, $941 = 0, $942 = 0, $943 = 0, $944 = 0, $945 = 0, $946 = 0, $947 = 0, $948 = 0, $949 = 0, $95 = 0, $950 = 0, $951 = 0, $952 = 0, $953 = 0, $954 = 0, $955 = 0;
 var $956 = 0, $957 = 0, $958 = 0, $959 = 0, $96 = 0, $960 = 0, $961 = 0, $962 = 0, $963 = 0, $964 = 0, $965 = 0, $966 = 0, $967 = 0, $968 = 0, $969 = 0, $97 = 0, $970 = 0, $971 = 0, $972 = 0, $973 = 0;
 var $974 = 0, $975 = 0, $976 = 0, $977 = 0, $978 = 0, $979 = 0, $98 = 0, $980 = 0, $981 = 0, $982 = 0, $983 = 0, $984 = 0, $985 = 0, $986 = 0, $987 = 0, $988 = 0, $989 = 0, $99 = 0, $990 = 0, $991 = 0;
 var $992 = 0, $993 = 0, $994 = 0, $995 = 0, $996 = 0, $997 = 0, $998 = 0, $999 = 0, $F$0$i$i = 0, $F1$0$i = 0, $F4$0 = 0, $F4$0$i$i = 0, $F5$0$i = 0, $I1$0$c$i$i = 0, $I1$0$i$i = 0, $I7$0$i = 0, $I7$0$i$i = 0, $K12$027$i = 0, $K2$015$i$i = 0, $K8$053$i$i = 0;
 var $R$0$i = 0, $R$0$i$i = 0, $R$0$i18 = 0, $R$1$i = 0, $R$1$i$i = 0, $R$1$i20 = 0, $RP$0$i = 0, $RP$0$i$i = 0, $RP$0$i17 = 0, $T$0$lcssa$i = 0, $T$0$lcssa$i$i = 0, $T$0$lcssa$i26$i = 0, $T$014$i$i = 0, $T$026$i = 0, $T$052$i$i = 0, $br$0$i = 0, $br$030$i = 0, $cond$i = 0, $cond$i$i = 0, $cond$i21 = 0;
 var $exitcond$i$i = 0, $i$02$i$i = 0, $idx$0$i = 0, $mem$0 = 0, $nb$0 = 0, $oldfirst$0$i$i = 0, $or$cond$i = 0, $or$cond$i$i = 0, $or$cond$i27$i = 0, $or$cond$i29 = 0, $or$cond1$i = 0, $or$cond19$i = 0, $or$cond2$i = 0, $or$cond24$i = 0, $or$cond3$i = 0, $or$cond4$i = 0, $or$cond47$i = 0, $or$cond5$i = 0, $or$cond6$i = 0, $or$cond8$i = 0;
 var $qsize$0$i$i = 0, $rsize$0$i = 0, $rsize$0$i15 = 0, $rsize$1$i = 0, $rsize$2$i = 0, $rsize$3$lcssa$i = 0, $rsize$331$i = 0, $rst$0$i = 0, $rst$1$i = 0, $sizebits$0$i = 0, $sp$0$i$i = 0, $sp$0$i$i$i = 0, $sp$073$i = 0, $sp$166$i = 0, $ssize$0$i = 0, $ssize$1$i = 0, $ssize$129$i = 0, $ssize$2$i = 0, $t$0$i = 0, $t$0$i14 = 0;
 var $t$1$i = 0, $t$2$ph$i = 0, $t$2$v$3$i = 0, $t$230$i = 0, $tbase$245$i = 0, $tsize$03141$i = 0, $tsize$1$i = 0, $tsize$244$i = 0, $v$0$i = 0, $v$0$i16 = 0, $v$1$i = 0, $v$2$i = 0, $v$3$lcssa$i = 0, $v$332$i = 0, label = 0, sp = 0;
 sp = STACKTOP;
 $0 = ($bytes>>>0)<(245);
 do {
  if ($0) {
   $1 = ($bytes>>>0)<(11);
   if ($1) {
    $5 = 16;
   } else {
    $2 = (($bytes) + 11)|0;
    $3 = $2 & -8;
    $5 = $3;
   }
   $4 = $5 >>> 3;
   $6 = HEAP32[39000>>2]|0;
   $7 = $6 >>> $4;
   $8 = $7 & 3;
   $9 = ($8|0)==(0);
   if (!($9)) {
    $10 = $7 & 1;
    $11 = $10 ^ 1;
    $12 = (($11) + ($4))|0;
    $13 = $12 << 1;
    $14 = ((39000 + ($13<<2)|0) + 40|0);
    $$sum10 = (($13) + 2)|0;
    $15 = ((39000 + ($$sum10<<2)|0) + 40|0);
    $16 = HEAP32[$15>>2]|0;
    $17 = (($16) + 8|0);
    $18 = HEAP32[$17>>2]|0;
    $19 = ($14|0)==($18|0);
    do {
     if ($19) {
      $20 = 1 << $12;
      $21 = $20 ^ -1;
      $22 = $6 & $21;
      HEAP32[39000>>2] = $22;
     } else {
      $23 = HEAP32[((39000 + 16|0))>>2]|0;
      $24 = ($18>>>0)<($23>>>0);
      if ($24) {
       _abort();
       // unreachable;
      }
      $25 = (($18) + 12|0);
      $26 = HEAP32[$25>>2]|0;
      $27 = ($26|0)==($16|0);
      if ($27) {
       HEAP32[$25>>2] = $14;
       HEAP32[$15>>2] = $18;
       break;
      } else {
       _abort();
       // unreachable;
      }
     }
    } while(0);
    $28 = $12 << 3;
    $29 = $28 | 3;
    $30 = (($16) + 4|0);
    HEAP32[$30>>2] = $29;
    $$sum1112 = $28 | 4;
    $31 = (($16) + ($$sum1112)|0);
    $32 = HEAP32[$31>>2]|0;
    $33 = $32 | 1;
    HEAP32[$31>>2] = $33;
    $mem$0 = $17;
    STACKTOP = sp;return ($mem$0|0);
   }
   $34 = HEAP32[((39000 + 8|0))>>2]|0;
   $35 = ($5>>>0)>($34>>>0);
   if ($35) {
    $36 = ($7|0)==(0);
    if (!($36)) {
     $37 = $7 << $4;
     $38 = 2 << $4;
     $39 = (0 - ($38))|0;
     $40 = $38 | $39;
     $41 = $37 & $40;
     $42 = (0 - ($41))|0;
     $43 = $41 & $42;
     $44 = (($43) + -1)|0;
     $45 = $44 >>> 12;
     $46 = $45 & 16;
     $47 = $44 >>> $46;
     $48 = $47 >>> 5;
     $49 = $48 & 8;
     $50 = $49 | $46;
     $51 = $47 >>> $49;
     $52 = $51 >>> 2;
     $53 = $52 & 4;
     $54 = $50 | $53;
     $55 = $51 >>> $53;
     $56 = $55 >>> 1;
     $57 = $56 & 2;
     $58 = $54 | $57;
     $59 = $55 >>> $57;
     $60 = $59 >>> 1;
     $61 = $60 & 1;
     $62 = $58 | $61;
     $63 = $59 >>> $61;
     $64 = (($62) + ($63))|0;
     $65 = $64 << 1;
     $66 = ((39000 + ($65<<2)|0) + 40|0);
     $$sum4 = (($65) + 2)|0;
     $67 = ((39000 + ($$sum4<<2)|0) + 40|0);
     $68 = HEAP32[$67>>2]|0;
     $69 = (($68) + 8|0);
     $70 = HEAP32[$69>>2]|0;
     $71 = ($66|0)==($70|0);
     do {
      if ($71) {
       $72 = 1 << $64;
       $73 = $72 ^ -1;
       $74 = $6 & $73;
       HEAP32[39000>>2] = $74;
       $89 = $34;
      } else {
       $75 = HEAP32[((39000 + 16|0))>>2]|0;
       $76 = ($70>>>0)<($75>>>0);
       if ($76) {
        _abort();
        // unreachable;
       }
       $77 = (($70) + 12|0);
       $78 = HEAP32[$77>>2]|0;
       $79 = ($78|0)==($68|0);
       if ($79) {
        HEAP32[$77>>2] = $66;
        HEAP32[$67>>2] = $70;
        $$pre = HEAP32[((39000 + 8|0))>>2]|0;
        $89 = $$pre;
        break;
       } else {
        _abort();
        // unreachable;
       }
      }
     } while(0);
     $80 = $64 << 3;
     $81 = (($80) - ($5))|0;
     $82 = $5 | 3;
     $83 = (($68) + 4|0);
     HEAP32[$83>>2] = $82;
     $84 = (($68) + ($5)|0);
     $85 = $81 | 1;
     $$sum56 = $5 | 4;
     $86 = (($68) + ($$sum56)|0);
     HEAP32[$86>>2] = $85;
     $87 = (($68) + ($80)|0);
     HEAP32[$87>>2] = $81;
     $88 = ($89|0)==(0);
     if (!($88)) {
      $90 = HEAP32[((39000 + 20|0))>>2]|0;
      $91 = $89 >>> 3;
      $92 = $91 << 1;
      $93 = ((39000 + ($92<<2)|0) + 40|0);
      $94 = HEAP32[39000>>2]|0;
      $95 = 1 << $91;
      $96 = $94 & $95;
      $97 = ($96|0)==(0);
      if ($97) {
       $98 = $94 | $95;
       HEAP32[39000>>2] = $98;
       $$sum8$pre = (($92) + 2)|0;
       $$pre105 = ((39000 + ($$sum8$pre<<2)|0) + 40|0);
       $$pre$phiZ2D = $$pre105;$F4$0 = $93;
      } else {
       $$sum9 = (($92) + 2)|0;
       $99 = ((39000 + ($$sum9<<2)|0) + 40|0);
       $100 = HEAP32[$99>>2]|0;
       $101 = HEAP32[((39000 + 16|0))>>2]|0;
       $102 = ($100>>>0)<($101>>>0);
       if ($102) {
        _abort();
        // unreachable;
       } else {
        $$pre$phiZ2D = $99;$F4$0 = $100;
       }
      }
      HEAP32[$$pre$phiZ2D>>2] = $90;
      $103 = (($F4$0) + 12|0);
      HEAP32[$103>>2] = $90;
      $104 = (($90) + 8|0);
      HEAP32[$104>>2] = $F4$0;
      $105 = (($90) + 12|0);
      HEAP32[$105>>2] = $93;
     }
     HEAP32[((39000 + 8|0))>>2] = $81;
     HEAP32[((39000 + 20|0))>>2] = $84;
     $mem$0 = $69;
     STACKTOP = sp;return ($mem$0|0);
    }
    $106 = HEAP32[((39000 + 4|0))>>2]|0;
    $107 = ($106|0)==(0);
    if ($107) {
     $nb$0 = $5;
    } else {
     $108 = (0 - ($106))|0;
     $109 = $106 & $108;
     $110 = (($109) + -1)|0;
     $111 = $110 >>> 12;
     $112 = $111 & 16;
     $113 = $110 >>> $112;
     $114 = $113 >>> 5;
     $115 = $114 & 8;
     $116 = $115 | $112;
     $117 = $113 >>> $115;
     $118 = $117 >>> 2;
     $119 = $118 & 4;
     $120 = $116 | $119;
     $121 = $117 >>> $119;
     $122 = $121 >>> 1;
     $123 = $122 & 2;
     $124 = $120 | $123;
     $125 = $121 >>> $123;
     $126 = $125 >>> 1;
     $127 = $126 & 1;
     $128 = $124 | $127;
     $129 = $125 >>> $127;
     $130 = (($128) + ($129))|0;
     $131 = ((39000 + ($130<<2)|0) + 304|0);
     $132 = HEAP32[$131>>2]|0;
     $133 = (($132) + 4|0);
     $134 = HEAP32[$133>>2]|0;
     $135 = $134 & -8;
     $136 = (($135) - ($5))|0;
     $rsize$0$i = $136;$t$0$i = $132;$v$0$i = $132;
     while(1) {
      $137 = (($t$0$i) + 16|0);
      $138 = HEAP32[$137>>2]|0;
      $139 = ($138|0)==(0|0);
      if ($139) {
       $140 = (($t$0$i) + 20|0);
       $141 = HEAP32[$140>>2]|0;
       $142 = ($141|0)==(0|0);
       if ($142) {
        break;
       } else {
        $144 = $141;
       }
      } else {
       $144 = $138;
      }
      $143 = (($144) + 4|0);
      $145 = HEAP32[$143>>2]|0;
      $146 = $145 & -8;
      $147 = (($146) - ($5))|0;
      $148 = ($147>>>0)<($rsize$0$i>>>0);
      $$rsize$0$i = $148 ? $147 : $rsize$0$i;
      $$v$0$i = $148 ? $144 : $v$0$i;
      $rsize$0$i = $$rsize$0$i;$t$0$i = $144;$v$0$i = $$v$0$i;
     }
     $149 = HEAP32[((39000 + 16|0))>>2]|0;
     $150 = ($v$0$i>>>0)<($149>>>0);
     if ($150) {
      _abort();
      // unreachable;
     }
     $151 = (($v$0$i) + ($5)|0);
     $152 = ($v$0$i>>>0)<($151>>>0);
     if (!($152)) {
      _abort();
      // unreachable;
     }
     $153 = (($v$0$i) + 24|0);
     $154 = HEAP32[$153>>2]|0;
     $155 = (($v$0$i) + 12|0);
     $156 = HEAP32[$155>>2]|0;
     $157 = ($156|0)==($v$0$i|0);
     do {
      if ($157) {
       $167 = (($v$0$i) + 20|0);
       $168 = HEAP32[$167>>2]|0;
       $169 = ($168|0)==(0|0);
       if ($169) {
        $170 = (($v$0$i) + 16|0);
        $171 = HEAP32[$170>>2]|0;
        $172 = ($171|0)==(0|0);
        if ($172) {
         $R$1$i = 0;
         break;
        } else {
         $R$0$i = $171;$RP$0$i = $170;
        }
       } else {
        $R$0$i = $168;$RP$0$i = $167;
       }
       while(1) {
        $173 = (($R$0$i) + 20|0);
        $174 = HEAP32[$173>>2]|0;
        $175 = ($174|0)==(0|0);
        if (!($175)) {
         $R$0$i = $174;$RP$0$i = $173;
         continue;
        }
        $176 = (($R$0$i) + 16|0);
        $177 = HEAP32[$176>>2]|0;
        $178 = ($177|0)==(0|0);
        if ($178) {
         break;
        } else {
         $R$0$i = $177;$RP$0$i = $176;
        }
       }
       $179 = ($RP$0$i>>>0)<($149>>>0);
       if ($179) {
        _abort();
        // unreachable;
       } else {
        HEAP32[$RP$0$i>>2] = 0;
        $R$1$i = $R$0$i;
        break;
       }
      } else {
       $158 = (($v$0$i) + 8|0);
       $159 = HEAP32[$158>>2]|0;
       $160 = ($159>>>0)<($149>>>0);
       if ($160) {
        _abort();
        // unreachable;
       }
       $161 = (($159) + 12|0);
       $162 = HEAP32[$161>>2]|0;
       $163 = ($162|0)==($v$0$i|0);
       if (!($163)) {
        _abort();
        // unreachable;
       }
       $164 = (($156) + 8|0);
       $165 = HEAP32[$164>>2]|0;
       $166 = ($165|0)==($v$0$i|0);
       if ($166) {
        HEAP32[$161>>2] = $156;
        HEAP32[$164>>2] = $159;
        $R$1$i = $156;
        break;
       } else {
        _abort();
        // unreachable;
       }
      }
     } while(0);
     $180 = ($154|0)==(0|0);
     do {
      if (!($180)) {
       $181 = (($v$0$i) + 28|0);
       $182 = HEAP32[$181>>2]|0;
       $183 = ((39000 + ($182<<2)|0) + 304|0);
       $184 = HEAP32[$183>>2]|0;
       $185 = ($v$0$i|0)==($184|0);
       if ($185) {
        HEAP32[$183>>2] = $R$1$i;
        $cond$i = ($R$1$i|0)==(0|0);
        if ($cond$i) {
         $186 = 1 << $182;
         $187 = $186 ^ -1;
         $188 = HEAP32[((39000 + 4|0))>>2]|0;
         $189 = $188 & $187;
         HEAP32[((39000 + 4|0))>>2] = $189;
         break;
        }
       } else {
        $190 = HEAP32[((39000 + 16|0))>>2]|0;
        $191 = ($154>>>0)<($190>>>0);
        if ($191) {
         _abort();
         // unreachable;
        }
        $192 = (($154) + 16|0);
        $193 = HEAP32[$192>>2]|0;
        $194 = ($193|0)==($v$0$i|0);
        if ($194) {
         HEAP32[$192>>2] = $R$1$i;
        } else {
         $195 = (($154) + 20|0);
         HEAP32[$195>>2] = $R$1$i;
        }
        $196 = ($R$1$i|0)==(0|0);
        if ($196) {
         break;
        }
       }
       $197 = HEAP32[((39000 + 16|0))>>2]|0;
       $198 = ($R$1$i>>>0)<($197>>>0);
       if ($198) {
        _abort();
        // unreachable;
       }
       $199 = (($R$1$i) + 24|0);
       HEAP32[$199>>2] = $154;
       $200 = (($v$0$i) + 16|0);
       $201 = HEAP32[$200>>2]|0;
       $202 = ($201|0)==(0|0);
       do {
        if (!($202)) {
         $203 = ($201>>>0)<($197>>>0);
         if ($203) {
          _abort();
          // unreachable;
         } else {
          $204 = (($R$1$i) + 16|0);
          HEAP32[$204>>2] = $201;
          $205 = (($201) + 24|0);
          HEAP32[$205>>2] = $R$1$i;
          break;
         }
        }
       } while(0);
       $206 = (($v$0$i) + 20|0);
       $207 = HEAP32[$206>>2]|0;
       $208 = ($207|0)==(0|0);
       if (!($208)) {
        $209 = HEAP32[((39000 + 16|0))>>2]|0;
        $210 = ($207>>>0)<($209>>>0);
        if ($210) {
         _abort();
         // unreachable;
        } else {
         $211 = (($R$1$i) + 20|0);
         HEAP32[$211>>2] = $207;
         $212 = (($207) + 24|0);
         HEAP32[$212>>2] = $R$1$i;
         break;
        }
       }
      }
     } while(0);
     $213 = ($rsize$0$i>>>0)<(16);
     if ($213) {
      $214 = (($rsize$0$i) + ($5))|0;
      $215 = $214 | 3;
      $216 = (($v$0$i) + 4|0);
      HEAP32[$216>>2] = $215;
      $$sum4$i = (($214) + 4)|0;
      $217 = (($v$0$i) + ($$sum4$i)|0);
      $218 = HEAP32[$217>>2]|0;
      $219 = $218 | 1;
      HEAP32[$217>>2] = $219;
     } else {
      $220 = $5 | 3;
      $221 = (($v$0$i) + 4|0);
      HEAP32[$221>>2] = $220;
      $222 = $rsize$0$i | 1;
      $$sum$i39 = $5 | 4;
      $223 = (($v$0$i) + ($$sum$i39)|0);
      HEAP32[$223>>2] = $222;
      $$sum1$i = (($rsize$0$i) + ($5))|0;
      $224 = (($v$0$i) + ($$sum1$i)|0);
      HEAP32[$224>>2] = $rsize$0$i;
      $225 = HEAP32[((39000 + 8|0))>>2]|0;
      $226 = ($225|0)==(0);
      if (!($226)) {
       $227 = HEAP32[((39000 + 20|0))>>2]|0;
       $228 = $225 >>> 3;
       $229 = $228 << 1;
       $230 = ((39000 + ($229<<2)|0) + 40|0);
       $231 = HEAP32[39000>>2]|0;
       $232 = 1 << $228;
       $233 = $231 & $232;
       $234 = ($233|0)==(0);
       if ($234) {
        $235 = $231 | $232;
        HEAP32[39000>>2] = $235;
        $$sum2$pre$i = (($229) + 2)|0;
        $$pre$i = ((39000 + ($$sum2$pre$i<<2)|0) + 40|0);
        $$pre$phi$iZ2D = $$pre$i;$F1$0$i = $230;
       } else {
        $$sum3$i = (($229) + 2)|0;
        $236 = ((39000 + ($$sum3$i<<2)|0) + 40|0);
        $237 = HEAP32[$236>>2]|0;
        $238 = HEAP32[((39000 + 16|0))>>2]|0;
        $239 = ($237>>>0)<($238>>>0);
        if ($239) {
         _abort();
         // unreachable;
        } else {
         $$pre$phi$iZ2D = $236;$F1$0$i = $237;
        }
       }
       HEAP32[$$pre$phi$iZ2D>>2] = $227;
       $240 = (($F1$0$i) + 12|0);
       HEAP32[$240>>2] = $227;
       $241 = (($227) + 8|0);
       HEAP32[$241>>2] = $F1$0$i;
       $242 = (($227) + 12|0);
       HEAP32[$242>>2] = $230;
      }
      HEAP32[((39000 + 8|0))>>2] = $rsize$0$i;
      HEAP32[((39000 + 20|0))>>2] = $151;
     }
     $243 = (($v$0$i) + 8|0);
     $mem$0 = $243;
     STACKTOP = sp;return ($mem$0|0);
    }
   } else {
    $nb$0 = $5;
   }
  } else {
   $244 = ($bytes>>>0)>(4294967231);
   if ($244) {
    $nb$0 = -1;
   } else {
    $245 = (($bytes) + 11)|0;
    $246 = $245 & -8;
    $247 = HEAP32[((39000 + 4|0))>>2]|0;
    $248 = ($247|0)==(0);
    if ($248) {
     $nb$0 = $246;
    } else {
     $249 = (0 - ($246))|0;
     $250 = $245 >>> 8;
     $251 = ($250|0)==(0);
     if ($251) {
      $idx$0$i = 0;
     } else {
      $252 = ($246>>>0)>(16777215);
      if ($252) {
       $idx$0$i = 31;
      } else {
       $253 = (($250) + 1048320)|0;
       $254 = $253 >>> 16;
       $255 = $254 & 8;
       $256 = $250 << $255;
       $257 = (($256) + 520192)|0;
       $258 = $257 >>> 16;
       $259 = $258 & 4;
       $260 = $259 | $255;
       $261 = $256 << $259;
       $262 = (($261) + 245760)|0;
       $263 = $262 >>> 16;
       $264 = $263 & 2;
       $265 = $260 | $264;
       $266 = (14 - ($265))|0;
       $267 = $261 << $264;
       $268 = $267 >>> 15;
       $269 = (($266) + ($268))|0;
       $270 = $269 << 1;
       $271 = (($269) + 7)|0;
       $272 = $246 >>> $271;
       $273 = $272 & 1;
       $274 = $273 | $270;
       $idx$0$i = $274;
      }
     }
     $275 = ((39000 + ($idx$0$i<<2)|0) + 304|0);
     $276 = HEAP32[$275>>2]|0;
     $277 = ($276|0)==(0|0);
     L126: do {
      if ($277) {
       $rsize$2$i = $249;$t$1$i = 0;$v$2$i = 0;
      } else {
       $278 = ($idx$0$i|0)==(31);
       if ($278) {
        $282 = 0;
       } else {
        $279 = $idx$0$i >>> 1;
        $280 = (25 - ($279))|0;
        $282 = $280;
       }
       $281 = $246 << $282;
       $rsize$0$i15 = $249;$rst$0$i = 0;$sizebits$0$i = $281;$t$0$i14 = $276;$v$0$i16 = 0;
       while(1) {
        $283 = (($t$0$i14) + 4|0);
        $284 = HEAP32[$283>>2]|0;
        $285 = $284 & -8;
        $286 = (($285) - ($246))|0;
        $287 = ($286>>>0)<($rsize$0$i15>>>0);
        if ($287) {
         $288 = ($285|0)==($246|0);
         if ($288) {
          $rsize$2$i = $286;$t$1$i = $t$0$i14;$v$2$i = $t$0$i14;
          break L126;
         } else {
          $rsize$1$i = $286;$v$1$i = $t$0$i14;
         }
        } else {
         $rsize$1$i = $rsize$0$i15;$v$1$i = $v$0$i16;
        }
        $289 = (($t$0$i14) + 20|0);
        $290 = HEAP32[$289>>2]|0;
        $291 = $sizebits$0$i >>> 31;
        $292 = ((($t$0$i14) + ($291<<2)|0) + 16|0);
        $293 = HEAP32[$292>>2]|0;
        $294 = ($290|0)==(0|0);
        $295 = ($290|0)==($293|0);
        $or$cond19$i = $294 | $295;
        $rst$1$i = $or$cond19$i ? $rst$0$i : $290;
        $296 = ($293|0)==(0|0);
        $297 = $sizebits$0$i << 1;
        if ($296) {
         $rsize$2$i = $rsize$1$i;$t$1$i = $rst$1$i;$v$2$i = $v$1$i;
         break;
        } else {
         $rsize$0$i15 = $rsize$1$i;$rst$0$i = $rst$1$i;$sizebits$0$i = $297;$t$0$i14 = $293;$v$0$i16 = $v$1$i;
        }
       }
      }
     } while(0);
     $298 = ($t$1$i|0)==(0|0);
     $299 = ($v$2$i|0)==(0|0);
     $or$cond$i = $298 & $299;
     if ($or$cond$i) {
      $300 = 2 << $idx$0$i;
      $301 = (0 - ($300))|0;
      $302 = $300 | $301;
      $303 = $247 & $302;
      $304 = ($303|0)==(0);
      if ($304) {
       $nb$0 = $246;
       break;
      }
      $305 = (0 - ($303))|0;
      $306 = $303 & $305;
      $307 = (($306) + -1)|0;
      $308 = $307 >>> 12;
      $309 = $308 & 16;
      $310 = $307 >>> $309;
      $311 = $310 >>> 5;
      $312 = $311 & 8;
      $313 = $312 | $309;
      $314 = $310 >>> $312;
      $315 = $314 >>> 2;
      $316 = $315 & 4;
      $317 = $313 | $316;
      $318 = $314 >>> $316;
      $319 = $318 >>> 1;
      $320 = $319 & 2;
      $321 = $317 | $320;
      $322 = $318 >>> $320;
      $323 = $322 >>> 1;
      $324 = $323 & 1;
      $325 = $321 | $324;
      $326 = $322 >>> $324;
      $327 = (($325) + ($326))|0;
      $328 = ((39000 + ($327<<2)|0) + 304|0);
      $329 = HEAP32[$328>>2]|0;
      $t$2$ph$i = $329;
     } else {
      $t$2$ph$i = $t$1$i;
     }
     $330 = ($t$2$ph$i|0)==(0|0);
     if ($330) {
      $rsize$3$lcssa$i = $rsize$2$i;$v$3$lcssa$i = $v$2$i;
     } else {
      $rsize$331$i = $rsize$2$i;$t$230$i = $t$2$ph$i;$v$332$i = $v$2$i;
      while(1) {
       $331 = (($t$230$i) + 4|0);
       $332 = HEAP32[$331>>2]|0;
       $333 = $332 & -8;
       $334 = (($333) - ($246))|0;
       $335 = ($334>>>0)<($rsize$331$i>>>0);
       $$rsize$3$i = $335 ? $334 : $rsize$331$i;
       $t$2$v$3$i = $335 ? $t$230$i : $v$332$i;
       $336 = (($t$230$i) + 16|0);
       $337 = HEAP32[$336>>2]|0;
       $338 = ($337|0)==(0|0);
       if (!($338)) {
        $rsize$331$i = $$rsize$3$i;$t$230$i = $337;$v$332$i = $t$2$v$3$i;
        continue;
       }
       $339 = (($t$230$i) + 20|0);
       $340 = HEAP32[$339>>2]|0;
       $341 = ($340|0)==(0|0);
       if ($341) {
        $rsize$3$lcssa$i = $$rsize$3$i;$v$3$lcssa$i = $t$2$v$3$i;
        break;
       } else {
        $rsize$331$i = $$rsize$3$i;$t$230$i = $340;$v$332$i = $t$2$v$3$i;
       }
      }
     }
     $342 = ($v$3$lcssa$i|0)==(0|0);
     if ($342) {
      $nb$0 = $246;
     } else {
      $343 = HEAP32[((39000 + 8|0))>>2]|0;
      $344 = (($343) - ($246))|0;
      $345 = ($rsize$3$lcssa$i>>>0)<($344>>>0);
      if ($345) {
       $346 = HEAP32[((39000 + 16|0))>>2]|0;
       $347 = ($v$3$lcssa$i>>>0)<($346>>>0);
       if ($347) {
        _abort();
        // unreachable;
       }
       $348 = (($v$3$lcssa$i) + ($246)|0);
       $349 = ($v$3$lcssa$i>>>0)<($348>>>0);
       if (!($349)) {
        _abort();
        // unreachable;
       }
       $350 = (($v$3$lcssa$i) + 24|0);
       $351 = HEAP32[$350>>2]|0;
       $352 = (($v$3$lcssa$i) + 12|0);
       $353 = HEAP32[$352>>2]|0;
       $354 = ($353|0)==($v$3$lcssa$i|0);
       do {
        if ($354) {
         $364 = (($v$3$lcssa$i) + 20|0);
         $365 = HEAP32[$364>>2]|0;
         $366 = ($365|0)==(0|0);
         if ($366) {
          $367 = (($v$3$lcssa$i) + 16|0);
          $368 = HEAP32[$367>>2]|0;
          $369 = ($368|0)==(0|0);
          if ($369) {
           $R$1$i20 = 0;
           break;
          } else {
           $R$0$i18 = $368;$RP$0$i17 = $367;
          }
         } else {
          $R$0$i18 = $365;$RP$0$i17 = $364;
         }
         while(1) {
          $370 = (($R$0$i18) + 20|0);
          $371 = HEAP32[$370>>2]|0;
          $372 = ($371|0)==(0|0);
          if (!($372)) {
           $R$0$i18 = $371;$RP$0$i17 = $370;
           continue;
          }
          $373 = (($R$0$i18) + 16|0);
          $374 = HEAP32[$373>>2]|0;
          $375 = ($374|0)==(0|0);
          if ($375) {
           break;
          } else {
           $R$0$i18 = $374;$RP$0$i17 = $373;
          }
         }
         $376 = ($RP$0$i17>>>0)<($346>>>0);
         if ($376) {
          _abort();
          // unreachable;
         } else {
          HEAP32[$RP$0$i17>>2] = 0;
          $R$1$i20 = $R$0$i18;
          break;
         }
        } else {
         $355 = (($v$3$lcssa$i) + 8|0);
         $356 = HEAP32[$355>>2]|0;
         $357 = ($356>>>0)<($346>>>0);
         if ($357) {
          _abort();
          // unreachable;
         }
         $358 = (($356) + 12|0);
         $359 = HEAP32[$358>>2]|0;
         $360 = ($359|0)==($v$3$lcssa$i|0);
         if (!($360)) {
          _abort();
          // unreachable;
         }
         $361 = (($353) + 8|0);
         $362 = HEAP32[$361>>2]|0;
         $363 = ($362|0)==($v$3$lcssa$i|0);
         if ($363) {
          HEAP32[$358>>2] = $353;
          HEAP32[$361>>2] = $356;
          $R$1$i20 = $353;
          break;
         } else {
          _abort();
          // unreachable;
         }
        }
       } while(0);
       $377 = ($351|0)==(0|0);
       do {
        if (!($377)) {
         $378 = (($v$3$lcssa$i) + 28|0);
         $379 = HEAP32[$378>>2]|0;
         $380 = ((39000 + ($379<<2)|0) + 304|0);
         $381 = HEAP32[$380>>2]|0;
         $382 = ($v$3$lcssa$i|0)==($381|0);
         if ($382) {
          HEAP32[$380>>2] = $R$1$i20;
          $cond$i21 = ($R$1$i20|0)==(0|0);
          if ($cond$i21) {
           $383 = 1 << $379;
           $384 = $383 ^ -1;
           $385 = HEAP32[((39000 + 4|0))>>2]|0;
           $386 = $385 & $384;
           HEAP32[((39000 + 4|0))>>2] = $386;
           break;
          }
         } else {
          $387 = HEAP32[((39000 + 16|0))>>2]|0;
          $388 = ($351>>>0)<($387>>>0);
          if ($388) {
           _abort();
           // unreachable;
          }
          $389 = (($351) + 16|0);
          $390 = HEAP32[$389>>2]|0;
          $391 = ($390|0)==($v$3$lcssa$i|0);
          if ($391) {
           HEAP32[$389>>2] = $R$1$i20;
          } else {
           $392 = (($351) + 20|0);
           HEAP32[$392>>2] = $R$1$i20;
          }
          $393 = ($R$1$i20|0)==(0|0);
          if ($393) {
           break;
          }
         }
         $394 = HEAP32[((39000 + 16|0))>>2]|0;
         $395 = ($R$1$i20>>>0)<($394>>>0);
         if ($395) {
          _abort();
          // unreachable;
         }
         $396 = (($R$1$i20) + 24|0);
         HEAP32[$396>>2] = $351;
         $397 = (($v$3$lcssa$i) + 16|0);
         $398 = HEAP32[$397>>2]|0;
         $399 = ($398|0)==(0|0);
         do {
          if (!($399)) {
           $400 = ($398>>>0)<($394>>>0);
           if ($400) {
            _abort();
            // unreachable;
           } else {
            $401 = (($R$1$i20) + 16|0);
            HEAP32[$401>>2] = $398;
            $402 = (($398) + 24|0);
            HEAP32[$402>>2] = $R$1$i20;
            break;
           }
          }
         } while(0);
         $403 = (($v$3$lcssa$i) + 20|0);
         $404 = HEAP32[$403>>2]|0;
         $405 = ($404|0)==(0|0);
         if (!($405)) {
          $406 = HEAP32[((39000 + 16|0))>>2]|0;
          $407 = ($404>>>0)<($406>>>0);
          if ($407) {
           _abort();
           // unreachable;
          } else {
           $408 = (($R$1$i20) + 20|0);
           HEAP32[$408>>2] = $404;
           $409 = (($404) + 24|0);
           HEAP32[$409>>2] = $R$1$i20;
           break;
          }
         }
        }
       } while(0);
       $410 = ($rsize$3$lcssa$i>>>0)<(16);
       L204: do {
        if ($410) {
         $411 = (($rsize$3$lcssa$i) + ($246))|0;
         $412 = $411 | 3;
         $413 = (($v$3$lcssa$i) + 4|0);
         HEAP32[$413>>2] = $412;
         $$sum18$i = (($411) + 4)|0;
         $414 = (($v$3$lcssa$i) + ($$sum18$i)|0);
         $415 = HEAP32[$414>>2]|0;
         $416 = $415 | 1;
         HEAP32[$414>>2] = $416;
        } else {
         $417 = $246 | 3;
         $418 = (($v$3$lcssa$i) + 4|0);
         HEAP32[$418>>2] = $417;
         $419 = $rsize$3$lcssa$i | 1;
         $$sum$i2338 = $246 | 4;
         $420 = (($v$3$lcssa$i) + ($$sum$i2338)|0);
         HEAP32[$420>>2] = $419;
         $$sum1$i24 = (($rsize$3$lcssa$i) + ($246))|0;
         $421 = (($v$3$lcssa$i) + ($$sum1$i24)|0);
         HEAP32[$421>>2] = $rsize$3$lcssa$i;
         $422 = $rsize$3$lcssa$i >>> 3;
         $423 = ($rsize$3$lcssa$i>>>0)<(256);
         if ($423) {
          $424 = $422 << 1;
          $425 = ((39000 + ($424<<2)|0) + 40|0);
          $426 = HEAP32[39000>>2]|0;
          $427 = 1 << $422;
          $428 = $426 & $427;
          $429 = ($428|0)==(0);
          do {
           if ($429) {
            $430 = $426 | $427;
            HEAP32[39000>>2] = $430;
            $$sum14$pre$i = (($424) + 2)|0;
            $$pre$i25 = ((39000 + ($$sum14$pre$i<<2)|0) + 40|0);
            $$pre$phi$i26Z2D = $$pre$i25;$F5$0$i = $425;
           } else {
            $$sum17$i = (($424) + 2)|0;
            $431 = ((39000 + ($$sum17$i<<2)|0) + 40|0);
            $432 = HEAP32[$431>>2]|0;
            $433 = HEAP32[((39000 + 16|0))>>2]|0;
            $434 = ($432>>>0)<($433>>>0);
            if (!($434)) {
             $$pre$phi$i26Z2D = $431;$F5$0$i = $432;
             break;
            }
            _abort();
            // unreachable;
           }
          } while(0);
          HEAP32[$$pre$phi$i26Z2D>>2] = $348;
          $435 = (($F5$0$i) + 12|0);
          HEAP32[$435>>2] = $348;
          $$sum15$i = (($246) + 8)|0;
          $436 = (($v$3$lcssa$i) + ($$sum15$i)|0);
          HEAP32[$436>>2] = $F5$0$i;
          $$sum16$i = (($246) + 12)|0;
          $437 = (($v$3$lcssa$i) + ($$sum16$i)|0);
          HEAP32[$437>>2] = $425;
          break;
         }
         $438 = $rsize$3$lcssa$i >>> 8;
         $439 = ($438|0)==(0);
         if ($439) {
          $I7$0$i = 0;
         } else {
          $440 = ($rsize$3$lcssa$i>>>0)>(16777215);
          if ($440) {
           $I7$0$i = 31;
          } else {
           $441 = (($438) + 1048320)|0;
           $442 = $441 >>> 16;
           $443 = $442 & 8;
           $444 = $438 << $443;
           $445 = (($444) + 520192)|0;
           $446 = $445 >>> 16;
           $447 = $446 & 4;
           $448 = $447 | $443;
           $449 = $444 << $447;
           $450 = (($449) + 245760)|0;
           $451 = $450 >>> 16;
           $452 = $451 & 2;
           $453 = $448 | $452;
           $454 = (14 - ($453))|0;
           $455 = $449 << $452;
           $456 = $455 >>> 15;
           $457 = (($454) + ($456))|0;
           $458 = $457 << 1;
           $459 = (($457) + 7)|0;
           $460 = $rsize$3$lcssa$i >>> $459;
           $461 = $460 & 1;
           $462 = $461 | $458;
           $I7$0$i = $462;
          }
         }
         $463 = ((39000 + ($I7$0$i<<2)|0) + 304|0);
         $$sum2$i = (($246) + 28)|0;
         $464 = (($v$3$lcssa$i) + ($$sum2$i)|0);
         HEAP32[$464>>2] = $I7$0$i;
         $$sum3$i27 = (($246) + 16)|0;
         $465 = (($v$3$lcssa$i) + ($$sum3$i27)|0);
         $$sum4$i28 = (($246) + 20)|0;
         $466 = (($v$3$lcssa$i) + ($$sum4$i28)|0);
         HEAP32[$466>>2] = 0;
         HEAP32[$465>>2] = 0;
         $467 = HEAP32[((39000 + 4|0))>>2]|0;
         $468 = 1 << $I7$0$i;
         $469 = $467 & $468;
         $470 = ($469|0)==(0);
         if ($470) {
          $471 = $467 | $468;
          HEAP32[((39000 + 4|0))>>2] = $471;
          HEAP32[$463>>2] = $348;
          $$sum5$i = (($246) + 24)|0;
          $472 = (($v$3$lcssa$i) + ($$sum5$i)|0);
          HEAP32[$472>>2] = $463;
          $$sum6$i = (($246) + 12)|0;
          $473 = (($v$3$lcssa$i) + ($$sum6$i)|0);
          HEAP32[$473>>2] = $348;
          $$sum7$i = (($246) + 8)|0;
          $474 = (($v$3$lcssa$i) + ($$sum7$i)|0);
          HEAP32[$474>>2] = $348;
          break;
         }
         $475 = HEAP32[$463>>2]|0;
         $476 = ($I7$0$i|0)==(31);
         if ($476) {
          $484 = 0;
         } else {
          $477 = $I7$0$i >>> 1;
          $478 = (25 - ($477))|0;
          $484 = $478;
         }
         $479 = (($475) + 4|0);
         $480 = HEAP32[$479>>2]|0;
         $481 = $480 & -8;
         $482 = ($481|0)==($rsize$3$lcssa$i|0);
         L224: do {
          if ($482) {
           $T$0$lcssa$i = $475;
          } else {
           $483 = $rsize$3$lcssa$i << $484;
           $K12$027$i = $483;$T$026$i = $475;
           while(1) {
            $491 = $K12$027$i >>> 31;
            $492 = ((($T$026$i) + ($491<<2)|0) + 16|0);
            $487 = HEAP32[$492>>2]|0;
            $493 = ($487|0)==(0|0);
            if ($493) {
             break;
            }
            $485 = $K12$027$i << 1;
            $486 = (($487) + 4|0);
            $488 = HEAP32[$486>>2]|0;
            $489 = $488 & -8;
            $490 = ($489|0)==($rsize$3$lcssa$i|0);
            if ($490) {
             $T$0$lcssa$i = $487;
             break L224;
            } else {
             $K12$027$i = $485;$T$026$i = $487;
            }
           }
           $494 = HEAP32[((39000 + 16|0))>>2]|0;
           $495 = ($492>>>0)<($494>>>0);
           if ($495) {
            _abort();
            // unreachable;
           } else {
            HEAP32[$492>>2] = $348;
            $$sum11$i = (($246) + 24)|0;
            $496 = (($v$3$lcssa$i) + ($$sum11$i)|0);
            HEAP32[$496>>2] = $T$026$i;
            $$sum12$i = (($246) + 12)|0;
            $497 = (($v$3$lcssa$i) + ($$sum12$i)|0);
            HEAP32[$497>>2] = $348;
            $$sum13$i = (($246) + 8)|0;
            $498 = (($v$3$lcssa$i) + ($$sum13$i)|0);
            HEAP32[$498>>2] = $348;
            break L204;
           }
          }
         } while(0);
         $499 = (($T$0$lcssa$i) + 8|0);
         $500 = HEAP32[$499>>2]|0;
         $501 = HEAP32[((39000 + 16|0))>>2]|0;
         $502 = ($T$0$lcssa$i>>>0)>=($501>>>0);
         $503 = ($500>>>0)>=($501>>>0);
         $or$cond24$i = $502 & $503;
         if ($or$cond24$i) {
          $504 = (($500) + 12|0);
          HEAP32[$504>>2] = $348;
          HEAP32[$499>>2] = $348;
          $$sum8$i = (($246) + 8)|0;
          $505 = (($v$3$lcssa$i) + ($$sum8$i)|0);
          HEAP32[$505>>2] = $500;
          $$sum9$i = (($246) + 12)|0;
          $506 = (($v$3$lcssa$i) + ($$sum9$i)|0);
          HEAP32[$506>>2] = $T$0$lcssa$i;
          $$sum10$i = (($246) + 24)|0;
          $507 = (($v$3$lcssa$i) + ($$sum10$i)|0);
          HEAP32[$507>>2] = 0;
          break;
         } else {
          _abort();
          // unreachable;
         }
        }
       } while(0);
       $508 = (($v$3$lcssa$i) + 8|0);
       $mem$0 = $508;
       STACKTOP = sp;return ($mem$0|0);
      } else {
       $nb$0 = $246;
      }
     }
    }
   }
  }
 } while(0);
 $509 = HEAP32[((39000 + 8|0))>>2]|0;
 $510 = ($509>>>0)<($nb$0>>>0);
 if (!($510)) {
  $511 = (($509) - ($nb$0))|0;
  $512 = HEAP32[((39000 + 20|0))>>2]|0;
  $513 = ($511>>>0)>(15);
  if ($513) {
   $514 = (($512) + ($nb$0)|0);
   HEAP32[((39000 + 20|0))>>2] = $514;
   HEAP32[((39000 + 8|0))>>2] = $511;
   $515 = $511 | 1;
   $$sum2 = (($nb$0) + 4)|0;
   $516 = (($512) + ($$sum2)|0);
   HEAP32[$516>>2] = $515;
   $517 = (($512) + ($509)|0);
   HEAP32[$517>>2] = $511;
   $518 = $nb$0 | 3;
   $519 = (($512) + 4|0);
   HEAP32[$519>>2] = $518;
  } else {
   HEAP32[((39000 + 8|0))>>2] = 0;
   HEAP32[((39000 + 20|0))>>2] = 0;
   $520 = $509 | 3;
   $521 = (($512) + 4|0);
   HEAP32[$521>>2] = $520;
   $$sum1 = (($509) + 4)|0;
   $522 = (($512) + ($$sum1)|0);
   $523 = HEAP32[$522>>2]|0;
   $524 = $523 | 1;
   HEAP32[$522>>2] = $524;
  }
  $525 = (($512) + 8|0);
  $mem$0 = $525;
  STACKTOP = sp;return ($mem$0|0);
 }
 $526 = HEAP32[((39000 + 12|0))>>2]|0;
 $527 = ($526>>>0)>($nb$0>>>0);
 if ($527) {
  $528 = (($526) - ($nb$0))|0;
  HEAP32[((39000 + 12|0))>>2] = $528;
  $529 = HEAP32[((39000 + 24|0))>>2]|0;
  $530 = (($529) + ($nb$0)|0);
  HEAP32[((39000 + 24|0))>>2] = $530;
  $531 = $528 | 1;
  $$sum = (($nb$0) + 4)|0;
  $532 = (($529) + ($$sum)|0);
  HEAP32[$532>>2] = $531;
  $533 = $nb$0 | 3;
  $534 = (($529) + 4|0);
  HEAP32[$534>>2] = $533;
  $535 = (($529) + 8|0);
  $mem$0 = $535;
  STACKTOP = sp;return ($mem$0|0);
 }
 $536 = HEAP32[39472>>2]|0;
 $537 = ($536|0)==(0);
 do {
  if ($537) {
   $538 = (_sysconf(30)|0);
   $539 = (($538) + -1)|0;
   $540 = $539 & $538;
   $541 = ($540|0)==(0);
   if ($541) {
    HEAP32[((39472 + 8|0))>>2] = $538;
    HEAP32[((39472 + 4|0))>>2] = $538;
    HEAP32[((39472 + 12|0))>>2] = -1;
    HEAP32[((39472 + 16|0))>>2] = -1;
    HEAP32[((39472 + 20|0))>>2] = 0;
    HEAP32[((39000 + 444|0))>>2] = 0;
    $542 = (_time((0|0))|0);
    $543 = $542 & -16;
    $544 = $543 ^ 1431655768;
    HEAP32[39472>>2] = $544;
    break;
   } else {
    _abort();
    // unreachable;
   }
  }
 } while(0);
 $545 = (($nb$0) + 48)|0;
 $546 = HEAP32[((39472 + 8|0))>>2]|0;
 $547 = (($nb$0) + 47)|0;
 $548 = (($546) + ($547))|0;
 $549 = (0 - ($546))|0;
 $550 = $548 & $549;
 $551 = ($550>>>0)>($nb$0>>>0);
 if (!($551)) {
  $mem$0 = 0;
  STACKTOP = sp;return ($mem$0|0);
 }
 $552 = HEAP32[((39000 + 440|0))>>2]|0;
 $553 = ($552|0)==(0);
 if (!($553)) {
  $554 = HEAP32[((39000 + 432|0))>>2]|0;
  $555 = (($554) + ($550))|0;
  $556 = ($555>>>0)<=($554>>>0);
  $557 = ($555>>>0)>($552>>>0);
  $or$cond1$i = $556 | $557;
  if ($or$cond1$i) {
   $mem$0 = 0;
   STACKTOP = sp;return ($mem$0|0);
  }
 }
 $558 = HEAP32[((39000 + 444|0))>>2]|0;
 $559 = $558 & 4;
 $560 = ($559|0)==(0);
 L266: do {
  if ($560) {
   $561 = HEAP32[((39000 + 24|0))>>2]|0;
   $562 = ($561|0)==(0|0);
   L268: do {
    if ($562) {
     label = 181;
    } else {
     $sp$0$i$i = ((39000 + 448|0));
     while(1) {
      $563 = HEAP32[$sp$0$i$i>>2]|0;
      $564 = ($563>>>0)>($561>>>0);
      if (!($564)) {
       $565 = (($sp$0$i$i) + 4|0);
       $566 = HEAP32[$565>>2]|0;
       $567 = (($563) + ($566)|0);
       $568 = ($567>>>0)>($561>>>0);
       if ($568) {
        break;
       }
      }
      $569 = (($sp$0$i$i) + 8|0);
      $570 = HEAP32[$569>>2]|0;
      $571 = ($570|0)==(0|0);
      if ($571) {
       label = 181;
       break L268;
      } else {
       $sp$0$i$i = $570;
      }
     }
     $572 = ($sp$0$i$i|0)==(0|0);
     if ($572) {
      label = 181;
     } else {
      $595 = HEAP32[((39000 + 12|0))>>2]|0;
      $596 = (($548) - ($595))|0;
      $597 = $596 & $549;
      $598 = ($597>>>0)<(2147483647);
      if ($598) {
       $599 = (_sbrk(($597|0))|0);
       $600 = HEAP32[$sp$0$i$i>>2]|0;
       $601 = HEAP32[$565>>2]|0;
       $602 = (($600) + ($601)|0);
       $603 = ($599|0)==($602|0);
       if ($603) {
        $br$0$i = $599;$ssize$1$i = $597;
        label = 190;
       } else {
        $br$030$i = $599;$ssize$129$i = $597;
        label = 191;
       }
      } else {
       $tsize$03141$i = 0;
      }
     }
    }
   } while(0);
   do {
    if ((label|0) == 181) {
     $573 = (_sbrk(0)|0);
     $574 = ($573|0)==((-1)|0);
     if ($574) {
      $tsize$03141$i = 0;
     } else {
      $575 = $573;
      $576 = HEAP32[((39472 + 4|0))>>2]|0;
      $577 = (($576) + -1)|0;
      $578 = $577 & $575;
      $579 = ($578|0)==(0);
      if ($579) {
       $ssize$0$i = $550;
      } else {
       $580 = (($577) + ($575))|0;
       $581 = (0 - ($576))|0;
       $582 = $580 & $581;
       $583 = (($550) - ($575))|0;
       $584 = (($583) + ($582))|0;
       $ssize$0$i = $584;
      }
      $585 = HEAP32[((39000 + 432|0))>>2]|0;
      $586 = (($585) + ($ssize$0$i))|0;
      $587 = ($ssize$0$i>>>0)>($nb$0>>>0);
      $588 = ($ssize$0$i>>>0)<(2147483647);
      $or$cond$i29 = $587 & $588;
      if ($or$cond$i29) {
       $589 = HEAP32[((39000 + 440|0))>>2]|0;
       $590 = ($589|0)==(0);
       if (!($590)) {
        $591 = ($586>>>0)<=($585>>>0);
        $592 = ($586>>>0)>($589>>>0);
        $or$cond2$i = $591 | $592;
        if ($or$cond2$i) {
         $tsize$03141$i = 0;
         break;
        }
       }
       $593 = (_sbrk(($ssize$0$i|0))|0);
       $594 = ($593|0)==($573|0);
       if ($594) {
        $br$0$i = $573;$ssize$1$i = $ssize$0$i;
        label = 190;
       } else {
        $br$030$i = $593;$ssize$129$i = $ssize$0$i;
        label = 191;
       }
      } else {
       $tsize$03141$i = 0;
      }
     }
    }
   } while(0);
   L288: do {
    if ((label|0) == 190) {
     $604 = ($br$0$i|0)==((-1)|0);
     if ($604) {
      $tsize$03141$i = $ssize$1$i;
     } else {
      $tbase$245$i = $br$0$i;$tsize$244$i = $ssize$1$i;
      label = 201;
      break L266;
     }
    }
    else if ((label|0) == 191) {
     $605 = (0 - ($ssize$129$i))|0;
     $606 = ($br$030$i|0)!=((-1)|0);
     $607 = ($ssize$129$i>>>0)<(2147483647);
     $or$cond5$i = $606 & $607;
     $608 = ($545>>>0)>($ssize$129$i>>>0);
     $or$cond4$i = $or$cond5$i & $608;
     do {
      if ($or$cond4$i) {
       $609 = HEAP32[((39472 + 8|0))>>2]|0;
       $610 = (($547) - ($ssize$129$i))|0;
       $611 = (($610) + ($609))|0;
       $612 = (0 - ($609))|0;
       $613 = $611 & $612;
       $614 = ($613>>>0)<(2147483647);
       if ($614) {
        $615 = (_sbrk(($613|0))|0);
        $616 = ($615|0)==((-1)|0);
        if ($616) {
         (_sbrk(($605|0))|0);
         $tsize$03141$i = 0;
         break L288;
        } else {
         $617 = (($613) + ($ssize$129$i))|0;
         $ssize$2$i = $617;
         break;
        }
       } else {
        $ssize$2$i = $ssize$129$i;
       }
      } else {
       $ssize$2$i = $ssize$129$i;
      }
     } while(0);
     $618 = ($br$030$i|0)==((-1)|0);
     if ($618) {
      $tsize$03141$i = 0;
     } else {
      $tbase$245$i = $br$030$i;$tsize$244$i = $ssize$2$i;
      label = 201;
      break L266;
     }
    }
   } while(0);
   $619 = HEAP32[((39000 + 444|0))>>2]|0;
   $620 = $619 | 4;
   HEAP32[((39000 + 444|0))>>2] = $620;
   $tsize$1$i = $tsize$03141$i;
   label = 198;
  } else {
   $tsize$1$i = 0;
   label = 198;
  }
 } while(0);
 if ((label|0) == 198) {
  $621 = ($550>>>0)<(2147483647);
  if ($621) {
   $622 = (_sbrk(($550|0))|0);
   $623 = (_sbrk(0)|0);
   $624 = ($622|0)!=((-1)|0);
   $625 = ($623|0)!=((-1)|0);
   $or$cond3$i = $624 & $625;
   $626 = ($622>>>0)<($623>>>0);
   $or$cond6$i = $or$cond3$i & $626;
   if ($or$cond6$i) {
    $627 = $623;
    $628 = $622;
    $629 = (($627) - ($628))|0;
    $630 = (($nb$0) + 40)|0;
    $631 = ($629>>>0)>($630>>>0);
    $$tsize$1$i = $631 ? $629 : $tsize$1$i;
    if ($631) {
     $tbase$245$i = $622;$tsize$244$i = $$tsize$1$i;
     label = 201;
    }
   }
  }
 }
 if ((label|0) == 201) {
  $632 = HEAP32[((39000 + 432|0))>>2]|0;
  $633 = (($632) + ($tsize$244$i))|0;
  HEAP32[((39000 + 432|0))>>2] = $633;
  $634 = HEAP32[((39000 + 436|0))>>2]|0;
  $635 = ($633>>>0)>($634>>>0);
  if ($635) {
   HEAP32[((39000 + 436|0))>>2] = $633;
  }
  $636 = HEAP32[((39000 + 24|0))>>2]|0;
  $637 = ($636|0)==(0|0);
  L308: do {
   if ($637) {
    $638 = HEAP32[((39000 + 16|0))>>2]|0;
    $639 = ($638|0)==(0|0);
    $640 = ($tbase$245$i>>>0)<($638>>>0);
    $or$cond8$i = $639 | $640;
    if ($or$cond8$i) {
     HEAP32[((39000 + 16|0))>>2] = $tbase$245$i;
    }
    HEAP32[((39000 + 448|0))>>2] = $tbase$245$i;
    HEAP32[((39000 + 452|0))>>2] = $tsize$244$i;
    HEAP32[((39000 + 460|0))>>2] = 0;
    $641 = HEAP32[39472>>2]|0;
    HEAP32[((39000 + 36|0))>>2] = $641;
    HEAP32[((39000 + 32|0))>>2] = -1;
    $i$02$i$i = 0;
    while(1) {
     $642 = $i$02$i$i << 1;
     $643 = ((39000 + ($642<<2)|0) + 40|0);
     $$sum$i$i = (($642) + 3)|0;
     $644 = ((39000 + ($$sum$i$i<<2)|0) + 40|0);
     HEAP32[$644>>2] = $643;
     $$sum1$i$i = (($642) + 2)|0;
     $645 = ((39000 + ($$sum1$i$i<<2)|0) + 40|0);
     HEAP32[$645>>2] = $643;
     $646 = (($i$02$i$i) + 1)|0;
     $exitcond$i$i = ($646|0)==(32);
     if ($exitcond$i$i) {
      break;
     } else {
      $i$02$i$i = $646;
     }
    }
    $647 = (($tsize$244$i) + -40)|0;
    $648 = (($tbase$245$i) + 8|0);
    $649 = $648;
    $650 = $649 & 7;
    $651 = ($650|0)==(0);
    if ($651) {
     $655 = 0;
    } else {
     $652 = (0 - ($649))|0;
     $653 = $652 & 7;
     $655 = $653;
    }
    $654 = (($tbase$245$i) + ($655)|0);
    $656 = (($647) - ($655))|0;
    HEAP32[((39000 + 24|0))>>2] = $654;
    HEAP32[((39000 + 12|0))>>2] = $656;
    $657 = $656 | 1;
    $$sum$i12$i = (($655) + 4)|0;
    $658 = (($tbase$245$i) + ($$sum$i12$i)|0);
    HEAP32[$658>>2] = $657;
    $$sum2$i$i = (($tsize$244$i) + -36)|0;
    $659 = (($tbase$245$i) + ($$sum2$i$i)|0);
    HEAP32[$659>>2] = 40;
    $660 = HEAP32[((39472 + 16|0))>>2]|0;
    HEAP32[((39000 + 28|0))>>2] = $660;
   } else {
    $sp$073$i = ((39000 + 448|0));
    while(1) {
     $661 = HEAP32[$sp$073$i>>2]|0;
     $662 = (($sp$073$i) + 4|0);
     $663 = HEAP32[$662>>2]|0;
     $664 = (($661) + ($663)|0);
     $665 = ($tbase$245$i|0)==($664|0);
     if ($665) {
      label = 213;
      break;
     }
     $666 = (($sp$073$i) + 8|0);
     $667 = HEAP32[$666>>2]|0;
     $668 = ($667|0)==(0|0);
     if ($668) {
      break;
     } else {
      $sp$073$i = $667;
     }
    }
    if ((label|0) == 213) {
     $669 = (($sp$073$i) + 12|0);
     $670 = HEAP32[$669>>2]|0;
     $671 = $670 & 8;
     $672 = ($671|0)==(0);
     if ($672) {
      $673 = ($636>>>0)>=($661>>>0);
      $674 = ($636>>>0)<($tbase$245$i>>>0);
      $or$cond47$i = $673 & $674;
      if ($or$cond47$i) {
       $675 = (($663) + ($tsize$244$i))|0;
       HEAP32[$662>>2] = $675;
       $676 = HEAP32[((39000 + 12|0))>>2]|0;
       $677 = (($676) + ($tsize$244$i))|0;
       $678 = (($636) + 8|0);
       $679 = $678;
       $680 = $679 & 7;
       $681 = ($680|0)==(0);
       if ($681) {
        $685 = 0;
       } else {
        $682 = (0 - ($679))|0;
        $683 = $682 & 7;
        $685 = $683;
       }
       $684 = (($636) + ($685)|0);
       $686 = (($677) - ($685))|0;
       HEAP32[((39000 + 24|0))>>2] = $684;
       HEAP32[((39000 + 12|0))>>2] = $686;
       $687 = $686 | 1;
       $$sum$i16$i = (($685) + 4)|0;
       $688 = (($636) + ($$sum$i16$i)|0);
       HEAP32[$688>>2] = $687;
       $$sum2$i17$i = (($677) + 4)|0;
       $689 = (($636) + ($$sum2$i17$i)|0);
       HEAP32[$689>>2] = 40;
       $690 = HEAP32[((39472 + 16|0))>>2]|0;
       HEAP32[((39000 + 28|0))>>2] = $690;
       break;
      }
     }
    }
    $691 = HEAP32[((39000 + 16|0))>>2]|0;
    $692 = ($tbase$245$i>>>0)<($691>>>0);
    if ($692) {
     HEAP32[((39000 + 16|0))>>2] = $tbase$245$i;
     $756 = $tbase$245$i;
    } else {
     $756 = $691;
    }
    $693 = (($tbase$245$i) + ($tsize$244$i)|0);
    $sp$166$i = ((39000 + 448|0));
    while(1) {
     $694 = HEAP32[$sp$166$i>>2]|0;
     $695 = ($694|0)==($693|0);
     if ($695) {
      label = 223;
      break;
     }
     $696 = (($sp$166$i) + 8|0);
     $697 = HEAP32[$696>>2]|0;
     $698 = ($697|0)==(0|0);
     if ($698) {
      break;
     } else {
      $sp$166$i = $697;
     }
    }
    if ((label|0) == 223) {
     $699 = (($sp$166$i) + 12|0);
     $700 = HEAP32[$699>>2]|0;
     $701 = $700 & 8;
     $702 = ($701|0)==(0);
     if ($702) {
      HEAP32[$sp$166$i>>2] = $tbase$245$i;
      $703 = (($sp$166$i) + 4|0);
      $704 = HEAP32[$703>>2]|0;
      $705 = (($704) + ($tsize$244$i))|0;
      HEAP32[$703>>2] = $705;
      $706 = (($tbase$245$i) + 8|0);
      $707 = $706;
      $708 = $707 & 7;
      $709 = ($708|0)==(0);
      if ($709) {
       $713 = 0;
      } else {
       $710 = (0 - ($707))|0;
       $711 = $710 & 7;
       $713 = $711;
      }
      $712 = (($tbase$245$i) + ($713)|0);
      $$sum102$i = (($tsize$244$i) + 8)|0;
      $714 = (($tbase$245$i) + ($$sum102$i)|0);
      $715 = $714;
      $716 = $715 & 7;
      $717 = ($716|0)==(0);
      if ($717) {
       $720 = 0;
      } else {
       $718 = (0 - ($715))|0;
       $719 = $718 & 7;
       $720 = $719;
      }
      $$sum103$i = (($720) + ($tsize$244$i))|0;
      $721 = (($tbase$245$i) + ($$sum103$i)|0);
      $722 = $721;
      $723 = $712;
      $724 = (($722) - ($723))|0;
      $$sum$i19$i = (($713) + ($nb$0))|0;
      $725 = (($tbase$245$i) + ($$sum$i19$i)|0);
      $726 = (($724) - ($nb$0))|0;
      $727 = $nb$0 | 3;
      $$sum1$i20$i = (($713) + 4)|0;
      $728 = (($tbase$245$i) + ($$sum1$i20$i)|0);
      HEAP32[$728>>2] = $727;
      $729 = ($721|0)==($636|0);
      L345: do {
       if ($729) {
        $730 = HEAP32[((39000 + 12|0))>>2]|0;
        $731 = (($730) + ($726))|0;
        HEAP32[((39000 + 12|0))>>2] = $731;
        HEAP32[((39000 + 24|0))>>2] = $725;
        $732 = $731 | 1;
        $$sum42$i$i = (($$sum$i19$i) + 4)|0;
        $733 = (($tbase$245$i) + ($$sum42$i$i)|0);
        HEAP32[$733>>2] = $732;
       } else {
        $734 = HEAP32[((39000 + 20|0))>>2]|0;
        $735 = ($721|0)==($734|0);
        if ($735) {
         $736 = HEAP32[((39000 + 8|0))>>2]|0;
         $737 = (($736) + ($726))|0;
         HEAP32[((39000 + 8|0))>>2] = $737;
         HEAP32[((39000 + 20|0))>>2] = $725;
         $738 = $737 | 1;
         $$sum40$i$i = (($$sum$i19$i) + 4)|0;
         $739 = (($tbase$245$i) + ($$sum40$i$i)|0);
         HEAP32[$739>>2] = $738;
         $$sum41$i$i = (($737) + ($$sum$i19$i))|0;
         $740 = (($tbase$245$i) + ($$sum41$i$i)|0);
         HEAP32[$740>>2] = $737;
         break;
        }
        $$sum2$i21$i = (($tsize$244$i) + 4)|0;
        $$sum104$i = (($$sum2$i21$i) + ($720))|0;
        $741 = (($tbase$245$i) + ($$sum104$i)|0);
        $742 = HEAP32[$741>>2]|0;
        $743 = $742 & 3;
        $744 = ($743|0)==(1);
        if ($744) {
         $745 = $742 & -8;
         $746 = $742 >>> 3;
         $747 = ($742>>>0)<(256);
         L352: do {
          if ($747) {
           $$sum3738$i$i = $720 | 8;
           $$sum114$i = (($$sum3738$i$i) + ($tsize$244$i))|0;
           $748 = (($tbase$245$i) + ($$sum114$i)|0);
           $749 = HEAP32[$748>>2]|0;
           $$sum39$i$i = (($tsize$244$i) + 12)|0;
           $$sum115$i = (($$sum39$i$i) + ($720))|0;
           $750 = (($tbase$245$i) + ($$sum115$i)|0);
           $751 = HEAP32[$750>>2]|0;
           $752 = $746 << 1;
           $753 = ((39000 + ($752<<2)|0) + 40|0);
           $754 = ($749|0)==($753|0);
           do {
            if (!($754)) {
             $755 = ($749>>>0)<($756>>>0);
             if ($755) {
              _abort();
              // unreachable;
             }
             $757 = (($749) + 12|0);
             $758 = HEAP32[$757>>2]|0;
             $759 = ($758|0)==($721|0);
             if ($759) {
              break;
             }
             _abort();
             // unreachable;
            }
           } while(0);
           $760 = ($751|0)==($749|0);
           if ($760) {
            $761 = 1 << $746;
            $762 = $761 ^ -1;
            $763 = HEAP32[39000>>2]|0;
            $764 = $763 & $762;
            HEAP32[39000>>2] = $764;
            break;
           }
           $765 = ($751|0)==($753|0);
           do {
            if ($765) {
             $$pre58$i$i = (($751) + 8|0);
             $$pre$phi59$i$iZ2D = $$pre58$i$i;
            } else {
             $766 = ($751>>>0)<($756>>>0);
             if ($766) {
              _abort();
              // unreachable;
             }
             $767 = (($751) + 8|0);
             $768 = HEAP32[$767>>2]|0;
             $769 = ($768|0)==($721|0);
             if ($769) {
              $$pre$phi59$i$iZ2D = $767;
              break;
             }
             _abort();
             // unreachable;
            }
           } while(0);
           $770 = (($749) + 12|0);
           HEAP32[$770>>2] = $751;
           HEAP32[$$pre$phi59$i$iZ2D>>2] = $749;
          } else {
           $$sum34$i$i = $720 | 24;
           $$sum105$i = (($$sum34$i$i) + ($tsize$244$i))|0;
           $771 = (($tbase$245$i) + ($$sum105$i)|0);
           $772 = HEAP32[$771>>2]|0;
           $$sum5$i$i = (($tsize$244$i) + 12)|0;
           $$sum106$i = (($$sum5$i$i) + ($720))|0;
           $773 = (($tbase$245$i) + ($$sum106$i)|0);
           $774 = HEAP32[$773>>2]|0;
           $775 = ($774|0)==($721|0);
           do {
            if ($775) {
             $$sum67$i$i = $720 | 16;
             $$sum112$i = (($$sum2$i21$i) + ($$sum67$i$i))|0;
             $785 = (($tbase$245$i) + ($$sum112$i)|0);
             $786 = HEAP32[$785>>2]|0;
             $787 = ($786|0)==(0|0);
             if ($787) {
              $$sum113$i = (($$sum67$i$i) + ($tsize$244$i))|0;
              $788 = (($tbase$245$i) + ($$sum113$i)|0);
              $789 = HEAP32[$788>>2]|0;
              $790 = ($789|0)==(0|0);
              if ($790) {
               $R$1$i$i = 0;
               break;
              } else {
               $R$0$i$i = $789;$RP$0$i$i = $788;
              }
             } else {
              $R$0$i$i = $786;$RP$0$i$i = $785;
             }
             while(1) {
              $791 = (($R$0$i$i) + 20|0);
              $792 = HEAP32[$791>>2]|0;
              $793 = ($792|0)==(0|0);
              if (!($793)) {
               $R$0$i$i = $792;$RP$0$i$i = $791;
               continue;
              }
              $794 = (($R$0$i$i) + 16|0);
              $795 = HEAP32[$794>>2]|0;
              $796 = ($795|0)==(0|0);
              if ($796) {
               break;
              } else {
               $R$0$i$i = $795;$RP$0$i$i = $794;
              }
             }
             $797 = ($RP$0$i$i>>>0)<($756>>>0);
             if ($797) {
              _abort();
              // unreachable;
             } else {
              HEAP32[$RP$0$i$i>>2] = 0;
              $R$1$i$i = $R$0$i$i;
              break;
             }
            } else {
             $$sum3536$i$i = $720 | 8;
             $$sum107$i = (($$sum3536$i$i) + ($tsize$244$i))|0;
             $776 = (($tbase$245$i) + ($$sum107$i)|0);
             $777 = HEAP32[$776>>2]|0;
             $778 = ($777>>>0)<($756>>>0);
             if ($778) {
              _abort();
              // unreachable;
             }
             $779 = (($777) + 12|0);
             $780 = HEAP32[$779>>2]|0;
             $781 = ($780|0)==($721|0);
             if (!($781)) {
              _abort();
              // unreachable;
             }
             $782 = (($774) + 8|0);
             $783 = HEAP32[$782>>2]|0;
             $784 = ($783|0)==($721|0);
             if ($784) {
              HEAP32[$779>>2] = $774;
              HEAP32[$782>>2] = $777;
              $R$1$i$i = $774;
              break;
             } else {
              _abort();
              // unreachable;
             }
            }
           } while(0);
           $798 = ($772|0)==(0|0);
           if ($798) {
            break;
           }
           $$sum30$i$i = (($tsize$244$i) + 28)|0;
           $$sum108$i = (($$sum30$i$i) + ($720))|0;
           $799 = (($tbase$245$i) + ($$sum108$i)|0);
           $800 = HEAP32[$799>>2]|0;
           $801 = ((39000 + ($800<<2)|0) + 304|0);
           $802 = HEAP32[$801>>2]|0;
           $803 = ($721|0)==($802|0);
           do {
            if ($803) {
             HEAP32[$801>>2] = $R$1$i$i;
             $cond$i$i = ($R$1$i$i|0)==(0|0);
             if (!($cond$i$i)) {
              break;
             }
             $804 = 1 << $800;
             $805 = $804 ^ -1;
             $806 = HEAP32[((39000 + 4|0))>>2]|0;
             $807 = $806 & $805;
             HEAP32[((39000 + 4|0))>>2] = $807;
             break L352;
            } else {
             $808 = HEAP32[((39000 + 16|0))>>2]|0;
             $809 = ($772>>>0)<($808>>>0);
             if ($809) {
              _abort();
              // unreachable;
             }
             $810 = (($772) + 16|0);
             $811 = HEAP32[$810>>2]|0;
             $812 = ($811|0)==($721|0);
             if ($812) {
              HEAP32[$810>>2] = $R$1$i$i;
             } else {
              $813 = (($772) + 20|0);
              HEAP32[$813>>2] = $R$1$i$i;
             }
             $814 = ($R$1$i$i|0)==(0|0);
             if ($814) {
              break L352;
             }
            }
           } while(0);
           $815 = HEAP32[((39000 + 16|0))>>2]|0;
           $816 = ($R$1$i$i>>>0)<($815>>>0);
           if ($816) {
            _abort();
            // unreachable;
           }
           $817 = (($R$1$i$i) + 24|0);
           HEAP32[$817>>2] = $772;
           $$sum3132$i$i = $720 | 16;
           $$sum109$i = (($$sum3132$i$i) + ($tsize$244$i))|0;
           $818 = (($tbase$245$i) + ($$sum109$i)|0);
           $819 = HEAP32[$818>>2]|0;
           $820 = ($819|0)==(0|0);
           do {
            if (!($820)) {
             $821 = ($819>>>0)<($815>>>0);
             if ($821) {
              _abort();
              // unreachable;
             } else {
              $822 = (($R$1$i$i) + 16|0);
              HEAP32[$822>>2] = $819;
              $823 = (($819) + 24|0);
              HEAP32[$823>>2] = $R$1$i$i;
              break;
             }
            }
           } while(0);
           $$sum110$i = (($$sum2$i21$i) + ($$sum3132$i$i))|0;
           $824 = (($tbase$245$i) + ($$sum110$i)|0);
           $825 = HEAP32[$824>>2]|0;
           $826 = ($825|0)==(0|0);
           if ($826) {
            break;
           }
           $827 = HEAP32[((39000 + 16|0))>>2]|0;
           $828 = ($825>>>0)<($827>>>0);
           if ($828) {
            _abort();
            // unreachable;
           } else {
            $829 = (($R$1$i$i) + 20|0);
            HEAP32[$829>>2] = $825;
            $830 = (($825) + 24|0);
            HEAP32[$830>>2] = $R$1$i$i;
            break;
           }
          }
         } while(0);
         $$sum9$i$i = $745 | $720;
         $$sum111$i = (($$sum9$i$i) + ($tsize$244$i))|0;
         $831 = (($tbase$245$i) + ($$sum111$i)|0);
         $832 = (($745) + ($726))|0;
         $oldfirst$0$i$i = $831;$qsize$0$i$i = $832;
        } else {
         $oldfirst$0$i$i = $721;$qsize$0$i$i = $726;
        }
        $833 = (($oldfirst$0$i$i) + 4|0);
        $834 = HEAP32[$833>>2]|0;
        $835 = $834 & -2;
        HEAP32[$833>>2] = $835;
        $836 = $qsize$0$i$i | 1;
        $$sum10$i$i = (($$sum$i19$i) + 4)|0;
        $837 = (($tbase$245$i) + ($$sum10$i$i)|0);
        HEAP32[$837>>2] = $836;
        $$sum11$i22$i = (($qsize$0$i$i) + ($$sum$i19$i))|0;
        $838 = (($tbase$245$i) + ($$sum11$i22$i)|0);
        HEAP32[$838>>2] = $qsize$0$i$i;
        $839 = $qsize$0$i$i >>> 3;
        $840 = ($qsize$0$i$i>>>0)<(256);
        if ($840) {
         $841 = $839 << 1;
         $842 = ((39000 + ($841<<2)|0) + 40|0);
         $843 = HEAP32[39000>>2]|0;
         $844 = 1 << $839;
         $845 = $843 & $844;
         $846 = ($845|0)==(0);
         do {
          if ($846) {
           $847 = $843 | $844;
           HEAP32[39000>>2] = $847;
           $$sum26$pre$i$i = (($841) + 2)|0;
           $$pre$i23$i = ((39000 + ($$sum26$pre$i$i<<2)|0) + 40|0);
           $$pre$phi$i24$iZ2D = $$pre$i23$i;$F4$0$i$i = $842;
          } else {
           $$sum29$i$i = (($841) + 2)|0;
           $848 = ((39000 + ($$sum29$i$i<<2)|0) + 40|0);
           $849 = HEAP32[$848>>2]|0;
           $850 = HEAP32[((39000 + 16|0))>>2]|0;
           $851 = ($849>>>0)<($850>>>0);
           if (!($851)) {
            $$pre$phi$i24$iZ2D = $848;$F4$0$i$i = $849;
            break;
           }
           _abort();
           // unreachable;
          }
         } while(0);
         HEAP32[$$pre$phi$i24$iZ2D>>2] = $725;
         $852 = (($F4$0$i$i) + 12|0);
         HEAP32[$852>>2] = $725;
         $$sum27$i$i = (($$sum$i19$i) + 8)|0;
         $853 = (($tbase$245$i) + ($$sum27$i$i)|0);
         HEAP32[$853>>2] = $F4$0$i$i;
         $$sum28$i$i = (($$sum$i19$i) + 12)|0;
         $854 = (($tbase$245$i) + ($$sum28$i$i)|0);
         HEAP32[$854>>2] = $842;
         break;
        }
        $855 = $qsize$0$i$i >>> 8;
        $856 = ($855|0)==(0);
        do {
         if ($856) {
          $I7$0$i$i = 0;
         } else {
          $857 = ($qsize$0$i$i>>>0)>(16777215);
          if ($857) {
           $I7$0$i$i = 31;
           break;
          }
          $858 = (($855) + 1048320)|0;
          $859 = $858 >>> 16;
          $860 = $859 & 8;
          $861 = $855 << $860;
          $862 = (($861) + 520192)|0;
          $863 = $862 >>> 16;
          $864 = $863 & 4;
          $865 = $864 | $860;
          $866 = $861 << $864;
          $867 = (($866) + 245760)|0;
          $868 = $867 >>> 16;
          $869 = $868 & 2;
          $870 = $865 | $869;
          $871 = (14 - ($870))|0;
          $872 = $866 << $869;
          $873 = $872 >>> 15;
          $874 = (($871) + ($873))|0;
          $875 = $874 << 1;
          $876 = (($874) + 7)|0;
          $877 = $qsize$0$i$i >>> $876;
          $878 = $877 & 1;
          $879 = $878 | $875;
          $I7$0$i$i = $879;
         }
        } while(0);
        $880 = ((39000 + ($I7$0$i$i<<2)|0) + 304|0);
        $$sum12$i$i = (($$sum$i19$i) + 28)|0;
        $881 = (($tbase$245$i) + ($$sum12$i$i)|0);
        HEAP32[$881>>2] = $I7$0$i$i;
        $$sum13$i$i = (($$sum$i19$i) + 16)|0;
        $882 = (($tbase$245$i) + ($$sum13$i$i)|0);
        $$sum14$i$i = (($$sum$i19$i) + 20)|0;
        $883 = (($tbase$245$i) + ($$sum14$i$i)|0);
        HEAP32[$883>>2] = 0;
        HEAP32[$882>>2] = 0;
        $884 = HEAP32[((39000 + 4|0))>>2]|0;
        $885 = 1 << $I7$0$i$i;
        $886 = $884 & $885;
        $887 = ($886|0)==(0);
        if ($887) {
         $888 = $884 | $885;
         HEAP32[((39000 + 4|0))>>2] = $888;
         HEAP32[$880>>2] = $725;
         $$sum15$i$i = (($$sum$i19$i) + 24)|0;
         $889 = (($tbase$245$i) + ($$sum15$i$i)|0);
         HEAP32[$889>>2] = $880;
         $$sum16$i$i = (($$sum$i19$i) + 12)|0;
         $890 = (($tbase$245$i) + ($$sum16$i$i)|0);
         HEAP32[$890>>2] = $725;
         $$sum17$i$i = (($$sum$i19$i) + 8)|0;
         $891 = (($tbase$245$i) + ($$sum17$i$i)|0);
         HEAP32[$891>>2] = $725;
         break;
        }
        $892 = HEAP32[$880>>2]|0;
        $893 = ($I7$0$i$i|0)==(31);
        if ($893) {
         $901 = 0;
        } else {
         $894 = $I7$0$i$i >>> 1;
         $895 = (25 - ($894))|0;
         $901 = $895;
        }
        $896 = (($892) + 4|0);
        $897 = HEAP32[$896>>2]|0;
        $898 = $897 & -8;
        $899 = ($898|0)==($qsize$0$i$i|0);
        L441: do {
         if ($899) {
          $T$0$lcssa$i26$i = $892;
         } else {
          $900 = $qsize$0$i$i << $901;
          $K8$053$i$i = $900;$T$052$i$i = $892;
          while(1) {
           $908 = $K8$053$i$i >>> 31;
           $909 = ((($T$052$i$i) + ($908<<2)|0) + 16|0);
           $904 = HEAP32[$909>>2]|0;
           $910 = ($904|0)==(0|0);
           if ($910) {
            break;
           }
           $902 = $K8$053$i$i << 1;
           $903 = (($904) + 4|0);
           $905 = HEAP32[$903>>2]|0;
           $906 = $905 & -8;
           $907 = ($906|0)==($qsize$0$i$i|0);
           if ($907) {
            $T$0$lcssa$i26$i = $904;
            break L441;
           } else {
            $K8$053$i$i = $902;$T$052$i$i = $904;
           }
          }
          $911 = HEAP32[((39000 + 16|0))>>2]|0;
          $912 = ($909>>>0)<($911>>>0);
          if ($912) {
           _abort();
           // unreachable;
          } else {
           HEAP32[$909>>2] = $725;
           $$sum23$i$i = (($$sum$i19$i) + 24)|0;
           $913 = (($tbase$245$i) + ($$sum23$i$i)|0);
           HEAP32[$913>>2] = $T$052$i$i;
           $$sum24$i$i = (($$sum$i19$i) + 12)|0;
           $914 = (($tbase$245$i) + ($$sum24$i$i)|0);
           HEAP32[$914>>2] = $725;
           $$sum25$i$i = (($$sum$i19$i) + 8)|0;
           $915 = (($tbase$245$i) + ($$sum25$i$i)|0);
           HEAP32[$915>>2] = $725;
           break L345;
          }
         }
        } while(0);
        $916 = (($T$0$lcssa$i26$i) + 8|0);
        $917 = HEAP32[$916>>2]|0;
        $918 = HEAP32[((39000 + 16|0))>>2]|0;
        $919 = ($T$0$lcssa$i26$i>>>0)>=($918>>>0);
        $920 = ($917>>>0)>=($918>>>0);
        $or$cond$i27$i = $919 & $920;
        if ($or$cond$i27$i) {
         $921 = (($917) + 12|0);
         HEAP32[$921>>2] = $725;
         HEAP32[$916>>2] = $725;
         $$sum20$i$i = (($$sum$i19$i) + 8)|0;
         $922 = (($tbase$245$i) + ($$sum20$i$i)|0);
         HEAP32[$922>>2] = $917;
         $$sum21$i$i = (($$sum$i19$i) + 12)|0;
         $923 = (($tbase$245$i) + ($$sum21$i$i)|0);
         HEAP32[$923>>2] = $T$0$lcssa$i26$i;
         $$sum22$i$i = (($$sum$i19$i) + 24)|0;
         $924 = (($tbase$245$i) + ($$sum22$i$i)|0);
         HEAP32[$924>>2] = 0;
         break;
        } else {
         _abort();
         // unreachable;
        }
       }
      } while(0);
      $$sum1819$i$i = $713 | 8;
      $925 = (($tbase$245$i) + ($$sum1819$i$i)|0);
      $mem$0 = $925;
      STACKTOP = sp;return ($mem$0|0);
     }
    }
    $sp$0$i$i$i = ((39000 + 448|0));
    while(1) {
     $926 = HEAP32[$sp$0$i$i$i>>2]|0;
     $927 = ($926>>>0)>($636>>>0);
     if (!($927)) {
      $928 = (($sp$0$i$i$i) + 4|0);
      $929 = HEAP32[$928>>2]|0;
      $930 = (($926) + ($929)|0);
      $931 = ($930>>>0)>($636>>>0);
      if ($931) {
       break;
      }
     }
     $932 = (($sp$0$i$i$i) + 8|0);
     $933 = HEAP32[$932>>2]|0;
     $sp$0$i$i$i = $933;
    }
    $$sum$i13$i = (($929) + -47)|0;
    $$sum1$i14$i = (($929) + -39)|0;
    $934 = (($926) + ($$sum1$i14$i)|0);
    $935 = $934;
    $936 = $935 & 7;
    $937 = ($936|0)==(0);
    if ($937) {
     $940 = 0;
    } else {
     $938 = (0 - ($935))|0;
     $939 = $938 & 7;
     $940 = $939;
    }
    $$sum2$i15$i = (($$sum$i13$i) + ($940))|0;
    $941 = (($926) + ($$sum2$i15$i)|0);
    $942 = (($636) + 16|0);
    $943 = ($941>>>0)<($942>>>0);
    $944 = $943 ? $636 : $941;
    $945 = (($944) + 8|0);
    $946 = (($tsize$244$i) + -40)|0;
    $947 = (($tbase$245$i) + 8|0);
    $948 = $947;
    $949 = $948 & 7;
    $950 = ($949|0)==(0);
    if ($950) {
     $954 = 0;
    } else {
     $951 = (0 - ($948))|0;
     $952 = $951 & 7;
     $954 = $952;
    }
    $953 = (($tbase$245$i) + ($954)|0);
    $955 = (($946) - ($954))|0;
    HEAP32[((39000 + 24|0))>>2] = $953;
    HEAP32[((39000 + 12|0))>>2] = $955;
    $956 = $955 | 1;
    $$sum$i$i$i = (($954) + 4)|0;
    $957 = (($tbase$245$i) + ($$sum$i$i$i)|0);
    HEAP32[$957>>2] = $956;
    $$sum2$i$i$i = (($tsize$244$i) + -36)|0;
    $958 = (($tbase$245$i) + ($$sum2$i$i$i)|0);
    HEAP32[$958>>2] = 40;
    $959 = HEAP32[((39472 + 16|0))>>2]|0;
    HEAP32[((39000 + 28|0))>>2] = $959;
    $960 = (($944) + 4|0);
    HEAP32[$960>>2] = 27;
    ;HEAP32[$945+0>>2]=HEAP32[((39000 + 448|0))+0>>2]|0;HEAP32[$945+4>>2]=HEAP32[((39000 + 448|0))+4>>2]|0;HEAP32[$945+8>>2]=HEAP32[((39000 + 448|0))+8>>2]|0;HEAP32[$945+12>>2]=HEAP32[((39000 + 448|0))+12>>2]|0;
    HEAP32[((39000 + 448|0))>>2] = $tbase$245$i;
    HEAP32[((39000 + 452|0))>>2] = $tsize$244$i;
    HEAP32[((39000 + 460|0))>>2] = 0;
    HEAP32[((39000 + 456|0))>>2] = $945;
    $961 = (($944) + 28|0);
    HEAP32[$961>>2] = 7;
    $962 = (($944) + 32|0);
    $963 = ($962>>>0)<($930>>>0);
    if ($963) {
     $965 = $961;
     while(1) {
      $964 = (($965) + 4|0);
      HEAP32[$964>>2] = 7;
      $966 = (($965) + 8|0);
      $967 = ($966>>>0)<($930>>>0);
      if ($967) {
       $965 = $964;
      } else {
       break;
      }
     }
    }
    $968 = ($944|0)==($636|0);
    if (!($968)) {
     $969 = $944;
     $970 = $636;
     $971 = (($969) - ($970))|0;
     $972 = (($636) + ($971)|0);
     $$sum3$i$i = (($971) + 4)|0;
     $973 = (($636) + ($$sum3$i$i)|0);
     $974 = HEAP32[$973>>2]|0;
     $975 = $974 & -2;
     HEAP32[$973>>2] = $975;
     $976 = $971 | 1;
     $977 = (($636) + 4|0);
     HEAP32[$977>>2] = $976;
     HEAP32[$972>>2] = $971;
     $978 = $971 >>> 3;
     $979 = ($971>>>0)<(256);
     if ($979) {
      $980 = $978 << 1;
      $981 = ((39000 + ($980<<2)|0) + 40|0);
      $982 = HEAP32[39000>>2]|0;
      $983 = 1 << $978;
      $984 = $982 & $983;
      $985 = ($984|0)==(0);
      do {
       if ($985) {
        $986 = $982 | $983;
        HEAP32[39000>>2] = $986;
        $$sum10$pre$i$i = (($980) + 2)|0;
        $$pre$i$i = ((39000 + ($$sum10$pre$i$i<<2)|0) + 40|0);
        $$pre$phi$i$iZ2D = $$pre$i$i;$F$0$i$i = $981;
       } else {
        $$sum11$i$i = (($980) + 2)|0;
        $987 = ((39000 + ($$sum11$i$i<<2)|0) + 40|0);
        $988 = HEAP32[$987>>2]|0;
        $989 = HEAP32[((39000 + 16|0))>>2]|0;
        $990 = ($988>>>0)<($989>>>0);
        if (!($990)) {
         $$pre$phi$i$iZ2D = $987;$F$0$i$i = $988;
         break;
        }
        _abort();
        // unreachable;
       }
      } while(0);
      HEAP32[$$pre$phi$i$iZ2D>>2] = $636;
      $991 = (($F$0$i$i) + 12|0);
      HEAP32[$991>>2] = $636;
      $992 = (($636) + 8|0);
      HEAP32[$992>>2] = $F$0$i$i;
      $993 = (($636) + 12|0);
      HEAP32[$993>>2] = $981;
      break;
     }
     $994 = $971 >>> 8;
     $995 = ($994|0)==(0);
     if ($995) {
      $I1$0$i$i = 0;
     } else {
      $996 = ($971>>>0)>(16777215);
      if ($996) {
       $I1$0$i$i = 31;
      } else {
       $997 = (($994) + 1048320)|0;
       $998 = $997 >>> 16;
       $999 = $998 & 8;
       $1000 = $994 << $999;
       $1001 = (($1000) + 520192)|0;
       $1002 = $1001 >>> 16;
       $1003 = $1002 & 4;
       $1004 = $1003 | $999;
       $1005 = $1000 << $1003;
       $1006 = (($1005) + 245760)|0;
       $1007 = $1006 >>> 16;
       $1008 = $1007 & 2;
       $1009 = $1004 | $1008;
       $1010 = (14 - ($1009))|0;
       $1011 = $1005 << $1008;
       $1012 = $1011 >>> 15;
       $1013 = (($1010) + ($1012))|0;
       $1014 = $1013 << 1;
       $1015 = (($1013) + 7)|0;
       $1016 = $971 >>> $1015;
       $1017 = $1016 & 1;
       $1018 = $1017 | $1014;
       $I1$0$i$i = $1018;
      }
     }
     $1019 = ((39000 + ($I1$0$i$i<<2)|0) + 304|0);
     $1020 = (($636) + 28|0);
     $I1$0$c$i$i = $I1$0$i$i;
     HEAP32[$1020>>2] = $I1$0$c$i$i;
     $1021 = (($636) + 20|0);
     HEAP32[$1021>>2] = 0;
     $1022 = (($636) + 16|0);
     HEAP32[$1022>>2] = 0;
     $1023 = HEAP32[((39000 + 4|0))>>2]|0;
     $1024 = 1 << $I1$0$i$i;
     $1025 = $1023 & $1024;
     $1026 = ($1025|0)==(0);
     if ($1026) {
      $1027 = $1023 | $1024;
      HEAP32[((39000 + 4|0))>>2] = $1027;
      HEAP32[$1019>>2] = $636;
      $1028 = (($636) + 24|0);
      HEAP32[$1028>>2] = $1019;
      $1029 = (($636) + 12|0);
      HEAP32[$1029>>2] = $636;
      $1030 = (($636) + 8|0);
      HEAP32[$1030>>2] = $636;
      break;
     }
     $1031 = HEAP32[$1019>>2]|0;
     $1032 = ($I1$0$i$i|0)==(31);
     if ($1032) {
      $1040 = 0;
     } else {
      $1033 = $I1$0$i$i >>> 1;
      $1034 = (25 - ($1033))|0;
      $1040 = $1034;
     }
     $1035 = (($1031) + 4|0);
     $1036 = HEAP32[$1035>>2]|0;
     $1037 = $1036 & -8;
     $1038 = ($1037|0)==($971|0);
     L493: do {
      if ($1038) {
       $T$0$lcssa$i$i = $1031;
      } else {
       $1039 = $971 << $1040;
       $K2$015$i$i = $1039;$T$014$i$i = $1031;
       while(1) {
        $1047 = $K2$015$i$i >>> 31;
        $1048 = ((($T$014$i$i) + ($1047<<2)|0) + 16|0);
        $1043 = HEAP32[$1048>>2]|0;
        $1049 = ($1043|0)==(0|0);
        if ($1049) {
         break;
        }
        $1041 = $K2$015$i$i << 1;
        $1042 = (($1043) + 4|0);
        $1044 = HEAP32[$1042>>2]|0;
        $1045 = $1044 & -8;
        $1046 = ($1045|0)==($971|0);
        if ($1046) {
         $T$0$lcssa$i$i = $1043;
         break L493;
        } else {
         $K2$015$i$i = $1041;$T$014$i$i = $1043;
        }
       }
       $1050 = HEAP32[((39000 + 16|0))>>2]|0;
       $1051 = ($1048>>>0)<($1050>>>0);
       if ($1051) {
        _abort();
        // unreachable;
       } else {
        HEAP32[$1048>>2] = $636;
        $1052 = (($636) + 24|0);
        HEAP32[$1052>>2] = $T$014$i$i;
        $1053 = (($636) + 12|0);
        HEAP32[$1053>>2] = $636;
        $1054 = (($636) + 8|0);
        HEAP32[$1054>>2] = $636;
        break L308;
       }
      }
     } while(0);
     $1055 = (($T$0$lcssa$i$i) + 8|0);
     $1056 = HEAP32[$1055>>2]|0;
     $1057 = HEAP32[((39000 + 16|0))>>2]|0;
     $1058 = ($T$0$lcssa$i$i>>>0)>=($1057>>>0);
     $1059 = ($1056>>>0)>=($1057>>>0);
     $or$cond$i$i = $1058 & $1059;
     if ($or$cond$i$i) {
      $1060 = (($1056) + 12|0);
      HEAP32[$1060>>2] = $636;
      HEAP32[$1055>>2] = $636;
      $1061 = (($636) + 8|0);
      HEAP32[$1061>>2] = $1056;
      $1062 = (($636) + 12|0);
      HEAP32[$1062>>2] = $T$0$lcssa$i$i;
      $1063 = (($636) + 24|0);
      HEAP32[$1063>>2] = 0;
      break;
     } else {
      _abort();
      // unreachable;
     }
    }
   }
  } while(0);
  $1064 = HEAP32[((39000 + 12|0))>>2]|0;
  $1065 = ($1064>>>0)>($nb$0>>>0);
  if ($1065) {
   $1066 = (($1064) - ($nb$0))|0;
   HEAP32[((39000 + 12|0))>>2] = $1066;
   $1067 = HEAP32[((39000 + 24|0))>>2]|0;
   $1068 = (($1067) + ($nb$0)|0);
   HEAP32[((39000 + 24|0))>>2] = $1068;
   $1069 = $1066 | 1;
   $$sum$i32 = (($nb$0) + 4)|0;
   $1070 = (($1067) + ($$sum$i32)|0);
   HEAP32[$1070>>2] = $1069;
   $1071 = $nb$0 | 3;
   $1072 = (($1067) + 4|0);
   HEAP32[$1072>>2] = $1071;
   $1073 = (($1067) + 8|0);
   $mem$0 = $1073;
   STACKTOP = sp;return ($mem$0|0);
  }
 }
 $1074 = (___errno_location()|0);
 HEAP32[$1074>>2] = 12;
 $mem$0 = 0;
 STACKTOP = sp;return ($mem$0|0);
}
function _free($mem) {
 $mem = $mem|0;
 var $$pre = 0, $$pre$phi66Z2D = 0, $$pre$phi68Z2D = 0, $$pre$phiZ2D = 0, $$pre65 = 0, $$pre67 = 0, $$sum = 0, $$sum16$pre = 0, $$sum17 = 0, $$sum18 = 0, $$sum19 = 0, $$sum2 = 0, $$sum20 = 0, $$sum2324 = 0, $$sum25 = 0, $$sum26 = 0, $$sum28 = 0, $$sum29 = 0, $$sum3 = 0, $$sum30 = 0;
 var $$sum31 = 0, $$sum32 = 0, $$sum33 = 0, $$sum34 = 0, $$sum35 = 0, $$sum36 = 0, $$sum37 = 0, $$sum5 = 0, $$sum67 = 0, $$sum8 = 0, $$sum9 = 0, $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0;
 var $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0, $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0;
 var $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0, $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0;
 var $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0, $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0;
 var $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0, $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0;
 var $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0, $188 = 0, $189 = 0, $19 = 0, $190 = 0, $191 = 0, $192 = 0, $193 = 0, $194 = 0, $195 = 0, $196 = 0;
 var $197 = 0, $198 = 0, $199 = 0, $2 = 0, $20 = 0, $200 = 0, $201 = 0, $202 = 0, $203 = 0, $204 = 0, $205 = 0, $206 = 0, $207 = 0, $208 = 0, $209 = 0, $21 = 0, $210 = 0, $211 = 0, $212 = 0, $213 = 0;
 var $214 = 0, $215 = 0, $216 = 0, $217 = 0, $218 = 0, $219 = 0, $22 = 0, $220 = 0, $221 = 0, $222 = 0, $223 = 0, $224 = 0, $225 = 0, $226 = 0, $227 = 0, $228 = 0, $229 = 0, $23 = 0, $230 = 0, $231 = 0;
 var $232 = 0, $233 = 0, $234 = 0, $235 = 0, $236 = 0, $237 = 0, $238 = 0, $239 = 0, $24 = 0, $240 = 0, $241 = 0, $242 = 0, $243 = 0, $244 = 0, $245 = 0, $246 = 0, $247 = 0, $248 = 0, $249 = 0, $25 = 0;
 var $250 = 0, $251 = 0, $252 = 0, $253 = 0, $254 = 0, $255 = 0, $256 = 0, $257 = 0, $258 = 0, $259 = 0, $26 = 0, $260 = 0, $261 = 0, $262 = 0, $263 = 0, $264 = 0, $265 = 0, $266 = 0, $267 = 0, $268 = 0;
 var $269 = 0, $27 = 0, $270 = 0, $271 = 0, $272 = 0, $273 = 0, $274 = 0, $275 = 0, $276 = 0, $277 = 0, $278 = 0, $279 = 0, $28 = 0, $280 = 0, $281 = 0, $282 = 0, $283 = 0, $284 = 0, $285 = 0, $286 = 0;
 var $287 = 0, $288 = 0, $289 = 0, $29 = 0, $290 = 0, $291 = 0, $292 = 0, $293 = 0, $294 = 0, $295 = 0, $296 = 0, $297 = 0, $298 = 0, $299 = 0, $3 = 0, $30 = 0, $300 = 0, $301 = 0, $302 = 0, $303 = 0;
 var $304 = 0, $305 = 0, $306 = 0, $307 = 0, $308 = 0, $309 = 0, $31 = 0, $310 = 0, $311 = 0, $312 = 0, $313 = 0, $314 = 0, $315 = 0, $316 = 0, $317 = 0, $318 = 0, $319 = 0, $32 = 0, $320 = 0, $321 = 0;
 var $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0;
 var $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0;
 var $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0;
 var $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $F16$0 = 0, $I18$0 = 0, $I18$0$c = 0, $K19$058 = 0, $R$0 = 0, $R$1 = 0, $R7$0 = 0;
 var $R7$1 = 0, $RP$0 = 0, $RP9$0 = 0, $T$0$lcssa = 0, $T$057 = 0, $cond = 0, $cond54 = 0, $or$cond = 0, $p$0 = 0, $psize$0 = 0, $psize$1 = 0, $sp$0$i = 0, $sp$0$in$i = 0, label = 0, sp = 0;
 sp = STACKTOP;
 $0 = ($mem|0)==(0|0);
 if ($0) {
  STACKTOP = sp;return;
 }
 $1 = (($mem) + -8|0);
 $2 = HEAP32[((39000 + 16|0))>>2]|0;
 $3 = ($1>>>0)<($2>>>0);
 if ($3) {
  _abort();
  // unreachable;
 }
 $4 = (($mem) + -4|0);
 $5 = HEAP32[$4>>2]|0;
 $6 = $5 & 3;
 $7 = ($6|0)==(1);
 if ($7) {
  _abort();
  // unreachable;
 }
 $8 = $5 & -8;
 $$sum = (($8) + -8)|0;
 $9 = (($mem) + ($$sum)|0);
 $10 = $5 & 1;
 $11 = ($10|0)==(0);
 do {
  if ($11) {
   $12 = HEAP32[$1>>2]|0;
   $13 = ($6|0)==(0);
   if ($13) {
    STACKTOP = sp;return;
   }
   $$sum2 = (-8 - ($12))|0;
   $14 = (($mem) + ($$sum2)|0);
   $15 = (($12) + ($8))|0;
   $16 = ($14>>>0)<($2>>>0);
   if ($16) {
    _abort();
    // unreachable;
   }
   $17 = HEAP32[((39000 + 20|0))>>2]|0;
   $18 = ($14|0)==($17|0);
   if ($18) {
    $$sum3 = (($8) + -4)|0;
    $103 = (($mem) + ($$sum3)|0);
    $104 = HEAP32[$103>>2]|0;
    $105 = $104 & 3;
    $106 = ($105|0)==(3);
    if (!($106)) {
     $p$0 = $14;$psize$0 = $15;
     break;
    }
    HEAP32[((39000 + 8|0))>>2] = $15;
    $107 = $104 & -2;
    HEAP32[$103>>2] = $107;
    $108 = $15 | 1;
    $$sum26 = (($$sum2) + 4)|0;
    $109 = (($mem) + ($$sum26)|0);
    HEAP32[$109>>2] = $108;
    HEAP32[$9>>2] = $15;
    STACKTOP = sp;return;
   }
   $19 = $12 >>> 3;
   $20 = ($12>>>0)<(256);
   if ($20) {
    $$sum36 = (($$sum2) + 8)|0;
    $21 = (($mem) + ($$sum36)|0);
    $22 = HEAP32[$21>>2]|0;
    $$sum37 = (($$sum2) + 12)|0;
    $23 = (($mem) + ($$sum37)|0);
    $24 = HEAP32[$23>>2]|0;
    $25 = $19 << 1;
    $26 = ((39000 + ($25<<2)|0) + 40|0);
    $27 = ($22|0)==($26|0);
    if (!($27)) {
     $28 = ($22>>>0)<($2>>>0);
     if ($28) {
      _abort();
      // unreachable;
     }
     $29 = (($22) + 12|0);
     $30 = HEAP32[$29>>2]|0;
     $31 = ($30|0)==($14|0);
     if (!($31)) {
      _abort();
      // unreachable;
     }
    }
    $32 = ($24|0)==($22|0);
    if ($32) {
     $33 = 1 << $19;
     $34 = $33 ^ -1;
     $35 = HEAP32[39000>>2]|0;
     $36 = $35 & $34;
     HEAP32[39000>>2] = $36;
     $p$0 = $14;$psize$0 = $15;
     break;
    }
    $37 = ($24|0)==($26|0);
    if ($37) {
     $$pre67 = (($24) + 8|0);
     $$pre$phi68Z2D = $$pre67;
    } else {
     $38 = ($24>>>0)<($2>>>0);
     if ($38) {
      _abort();
      // unreachable;
     }
     $39 = (($24) + 8|0);
     $40 = HEAP32[$39>>2]|0;
     $41 = ($40|0)==($14|0);
     if ($41) {
      $$pre$phi68Z2D = $39;
     } else {
      _abort();
      // unreachable;
     }
    }
    $42 = (($22) + 12|0);
    HEAP32[$42>>2] = $24;
    HEAP32[$$pre$phi68Z2D>>2] = $22;
    $p$0 = $14;$psize$0 = $15;
    break;
   }
   $$sum28 = (($$sum2) + 24)|0;
   $43 = (($mem) + ($$sum28)|0);
   $44 = HEAP32[$43>>2]|0;
   $$sum29 = (($$sum2) + 12)|0;
   $45 = (($mem) + ($$sum29)|0);
   $46 = HEAP32[$45>>2]|0;
   $47 = ($46|0)==($14|0);
   do {
    if ($47) {
     $$sum31 = (($$sum2) + 20)|0;
     $57 = (($mem) + ($$sum31)|0);
     $58 = HEAP32[$57>>2]|0;
     $59 = ($58|0)==(0|0);
     if ($59) {
      $$sum30 = (($$sum2) + 16)|0;
      $60 = (($mem) + ($$sum30)|0);
      $61 = HEAP32[$60>>2]|0;
      $62 = ($61|0)==(0|0);
      if ($62) {
       $R$1 = 0;
       break;
      } else {
       $R$0 = $61;$RP$0 = $60;
      }
     } else {
      $R$0 = $58;$RP$0 = $57;
     }
     while(1) {
      $63 = (($R$0) + 20|0);
      $64 = HEAP32[$63>>2]|0;
      $65 = ($64|0)==(0|0);
      if (!($65)) {
       $R$0 = $64;$RP$0 = $63;
       continue;
      }
      $66 = (($R$0) + 16|0);
      $67 = HEAP32[$66>>2]|0;
      $68 = ($67|0)==(0|0);
      if ($68) {
       break;
      } else {
       $R$0 = $67;$RP$0 = $66;
      }
     }
     $69 = ($RP$0>>>0)<($2>>>0);
     if ($69) {
      _abort();
      // unreachable;
     } else {
      HEAP32[$RP$0>>2] = 0;
      $R$1 = $R$0;
      break;
     }
    } else {
     $$sum35 = (($$sum2) + 8)|0;
     $48 = (($mem) + ($$sum35)|0);
     $49 = HEAP32[$48>>2]|0;
     $50 = ($49>>>0)<($2>>>0);
     if ($50) {
      _abort();
      // unreachable;
     }
     $51 = (($49) + 12|0);
     $52 = HEAP32[$51>>2]|0;
     $53 = ($52|0)==($14|0);
     if (!($53)) {
      _abort();
      // unreachable;
     }
     $54 = (($46) + 8|0);
     $55 = HEAP32[$54>>2]|0;
     $56 = ($55|0)==($14|0);
     if ($56) {
      HEAP32[$51>>2] = $46;
      HEAP32[$54>>2] = $49;
      $R$1 = $46;
      break;
     } else {
      _abort();
      // unreachable;
     }
    }
   } while(0);
   $70 = ($44|0)==(0|0);
   if ($70) {
    $p$0 = $14;$psize$0 = $15;
   } else {
    $$sum32 = (($$sum2) + 28)|0;
    $71 = (($mem) + ($$sum32)|0);
    $72 = HEAP32[$71>>2]|0;
    $73 = ((39000 + ($72<<2)|0) + 304|0);
    $74 = HEAP32[$73>>2]|0;
    $75 = ($14|0)==($74|0);
    if ($75) {
     HEAP32[$73>>2] = $R$1;
     $cond = ($R$1|0)==(0|0);
     if ($cond) {
      $76 = 1 << $72;
      $77 = $76 ^ -1;
      $78 = HEAP32[((39000 + 4|0))>>2]|0;
      $79 = $78 & $77;
      HEAP32[((39000 + 4|0))>>2] = $79;
      $p$0 = $14;$psize$0 = $15;
      break;
     }
    } else {
     $80 = HEAP32[((39000 + 16|0))>>2]|0;
     $81 = ($44>>>0)<($80>>>0);
     if ($81) {
      _abort();
      // unreachable;
     }
     $82 = (($44) + 16|0);
     $83 = HEAP32[$82>>2]|0;
     $84 = ($83|0)==($14|0);
     if ($84) {
      HEAP32[$82>>2] = $R$1;
     } else {
      $85 = (($44) + 20|0);
      HEAP32[$85>>2] = $R$1;
     }
     $86 = ($R$1|0)==(0|0);
     if ($86) {
      $p$0 = $14;$psize$0 = $15;
      break;
     }
    }
    $87 = HEAP32[((39000 + 16|0))>>2]|0;
    $88 = ($R$1>>>0)<($87>>>0);
    if ($88) {
     _abort();
     // unreachable;
    }
    $89 = (($R$1) + 24|0);
    HEAP32[$89>>2] = $44;
    $$sum33 = (($$sum2) + 16)|0;
    $90 = (($mem) + ($$sum33)|0);
    $91 = HEAP32[$90>>2]|0;
    $92 = ($91|0)==(0|0);
    do {
     if (!($92)) {
      $93 = ($91>>>0)<($87>>>0);
      if ($93) {
       _abort();
       // unreachable;
      } else {
       $94 = (($R$1) + 16|0);
       HEAP32[$94>>2] = $91;
       $95 = (($91) + 24|0);
       HEAP32[$95>>2] = $R$1;
       break;
      }
     }
    } while(0);
    $$sum34 = (($$sum2) + 20)|0;
    $96 = (($mem) + ($$sum34)|0);
    $97 = HEAP32[$96>>2]|0;
    $98 = ($97|0)==(0|0);
    if ($98) {
     $p$0 = $14;$psize$0 = $15;
    } else {
     $99 = HEAP32[((39000 + 16|0))>>2]|0;
     $100 = ($97>>>0)<($99>>>0);
     if ($100) {
      _abort();
      // unreachable;
     } else {
      $101 = (($R$1) + 20|0);
      HEAP32[$101>>2] = $97;
      $102 = (($97) + 24|0);
      HEAP32[$102>>2] = $R$1;
      $p$0 = $14;$psize$0 = $15;
      break;
     }
    }
   }
  } else {
   $p$0 = $1;$psize$0 = $8;
  }
 } while(0);
 $110 = ($p$0>>>0)<($9>>>0);
 if (!($110)) {
  _abort();
  // unreachable;
 }
 $$sum25 = (($8) + -4)|0;
 $111 = (($mem) + ($$sum25)|0);
 $112 = HEAP32[$111>>2]|0;
 $113 = $112 & 1;
 $114 = ($113|0)==(0);
 if ($114) {
  _abort();
  // unreachable;
 }
 $115 = $112 & 2;
 $116 = ($115|0)==(0);
 if ($116) {
  $117 = HEAP32[((39000 + 24|0))>>2]|0;
  $118 = ($9|0)==($117|0);
  if ($118) {
   $119 = HEAP32[((39000 + 12|0))>>2]|0;
   $120 = (($119) + ($psize$0))|0;
   HEAP32[((39000 + 12|0))>>2] = $120;
   HEAP32[((39000 + 24|0))>>2] = $p$0;
   $121 = $120 | 1;
   $122 = (($p$0) + 4|0);
   HEAP32[$122>>2] = $121;
   $123 = HEAP32[((39000 + 20|0))>>2]|0;
   $124 = ($p$0|0)==($123|0);
   if (!($124)) {
    STACKTOP = sp;return;
   }
   HEAP32[((39000 + 20|0))>>2] = 0;
   HEAP32[((39000 + 8|0))>>2] = 0;
   STACKTOP = sp;return;
  }
  $125 = HEAP32[((39000 + 20|0))>>2]|0;
  $126 = ($9|0)==($125|0);
  if ($126) {
   $127 = HEAP32[((39000 + 8|0))>>2]|0;
   $128 = (($127) + ($psize$0))|0;
   HEAP32[((39000 + 8|0))>>2] = $128;
   HEAP32[((39000 + 20|0))>>2] = $p$0;
   $129 = $128 | 1;
   $130 = (($p$0) + 4|0);
   HEAP32[$130>>2] = $129;
   $131 = (($p$0) + ($128)|0);
   HEAP32[$131>>2] = $128;
   STACKTOP = sp;return;
  }
  $132 = $112 & -8;
  $133 = (($132) + ($psize$0))|0;
  $134 = $112 >>> 3;
  $135 = ($112>>>0)<(256);
  do {
   if ($135) {
    $136 = (($mem) + ($8)|0);
    $137 = HEAP32[$136>>2]|0;
    $$sum2324 = $8 | 4;
    $138 = (($mem) + ($$sum2324)|0);
    $139 = HEAP32[$138>>2]|0;
    $140 = $134 << 1;
    $141 = ((39000 + ($140<<2)|0) + 40|0);
    $142 = ($137|0)==($141|0);
    if (!($142)) {
     $143 = HEAP32[((39000 + 16|0))>>2]|0;
     $144 = ($137>>>0)<($143>>>0);
     if ($144) {
      _abort();
      // unreachable;
     }
     $145 = (($137) + 12|0);
     $146 = HEAP32[$145>>2]|0;
     $147 = ($146|0)==($9|0);
     if (!($147)) {
      _abort();
      // unreachable;
     }
    }
    $148 = ($139|0)==($137|0);
    if ($148) {
     $149 = 1 << $134;
     $150 = $149 ^ -1;
     $151 = HEAP32[39000>>2]|0;
     $152 = $151 & $150;
     HEAP32[39000>>2] = $152;
     break;
    }
    $153 = ($139|0)==($141|0);
    if ($153) {
     $$pre65 = (($139) + 8|0);
     $$pre$phi66Z2D = $$pre65;
    } else {
     $154 = HEAP32[((39000 + 16|0))>>2]|0;
     $155 = ($139>>>0)<($154>>>0);
     if ($155) {
      _abort();
      // unreachable;
     }
     $156 = (($139) + 8|0);
     $157 = HEAP32[$156>>2]|0;
     $158 = ($157|0)==($9|0);
     if ($158) {
      $$pre$phi66Z2D = $156;
     } else {
      _abort();
      // unreachable;
     }
    }
    $159 = (($137) + 12|0);
    HEAP32[$159>>2] = $139;
    HEAP32[$$pre$phi66Z2D>>2] = $137;
   } else {
    $$sum5 = (($8) + 16)|0;
    $160 = (($mem) + ($$sum5)|0);
    $161 = HEAP32[$160>>2]|0;
    $$sum67 = $8 | 4;
    $162 = (($mem) + ($$sum67)|0);
    $163 = HEAP32[$162>>2]|0;
    $164 = ($163|0)==($9|0);
    do {
     if ($164) {
      $$sum9 = (($8) + 12)|0;
      $175 = (($mem) + ($$sum9)|0);
      $176 = HEAP32[$175>>2]|0;
      $177 = ($176|0)==(0|0);
      if ($177) {
       $$sum8 = (($8) + 8)|0;
       $178 = (($mem) + ($$sum8)|0);
       $179 = HEAP32[$178>>2]|0;
       $180 = ($179|0)==(0|0);
       if ($180) {
        $R7$1 = 0;
        break;
       } else {
        $R7$0 = $179;$RP9$0 = $178;
       }
      } else {
       $R7$0 = $176;$RP9$0 = $175;
      }
      while(1) {
       $181 = (($R7$0) + 20|0);
       $182 = HEAP32[$181>>2]|0;
       $183 = ($182|0)==(0|0);
       if (!($183)) {
        $R7$0 = $182;$RP9$0 = $181;
        continue;
       }
       $184 = (($R7$0) + 16|0);
       $185 = HEAP32[$184>>2]|0;
       $186 = ($185|0)==(0|0);
       if ($186) {
        break;
       } else {
        $R7$0 = $185;$RP9$0 = $184;
       }
      }
      $187 = HEAP32[((39000 + 16|0))>>2]|0;
      $188 = ($RP9$0>>>0)<($187>>>0);
      if ($188) {
       _abort();
       // unreachable;
      } else {
       HEAP32[$RP9$0>>2] = 0;
       $R7$1 = $R7$0;
       break;
      }
     } else {
      $165 = (($mem) + ($8)|0);
      $166 = HEAP32[$165>>2]|0;
      $167 = HEAP32[((39000 + 16|0))>>2]|0;
      $168 = ($166>>>0)<($167>>>0);
      if ($168) {
       _abort();
       // unreachable;
      }
      $169 = (($166) + 12|0);
      $170 = HEAP32[$169>>2]|0;
      $171 = ($170|0)==($9|0);
      if (!($171)) {
       _abort();
       // unreachable;
      }
      $172 = (($163) + 8|0);
      $173 = HEAP32[$172>>2]|0;
      $174 = ($173|0)==($9|0);
      if ($174) {
       HEAP32[$169>>2] = $163;
       HEAP32[$172>>2] = $166;
       $R7$1 = $163;
       break;
      } else {
       _abort();
       // unreachable;
      }
     }
    } while(0);
    $189 = ($161|0)==(0|0);
    if (!($189)) {
     $$sum18 = (($8) + 20)|0;
     $190 = (($mem) + ($$sum18)|0);
     $191 = HEAP32[$190>>2]|0;
     $192 = ((39000 + ($191<<2)|0) + 304|0);
     $193 = HEAP32[$192>>2]|0;
     $194 = ($9|0)==($193|0);
     if ($194) {
      HEAP32[$192>>2] = $R7$1;
      $cond54 = ($R7$1|0)==(0|0);
      if ($cond54) {
       $195 = 1 << $191;
       $196 = $195 ^ -1;
       $197 = HEAP32[((39000 + 4|0))>>2]|0;
       $198 = $197 & $196;
       HEAP32[((39000 + 4|0))>>2] = $198;
       break;
      }
     } else {
      $199 = HEAP32[((39000 + 16|0))>>2]|0;
      $200 = ($161>>>0)<($199>>>0);
      if ($200) {
       _abort();
       // unreachable;
      }
      $201 = (($161) + 16|0);
      $202 = HEAP32[$201>>2]|0;
      $203 = ($202|0)==($9|0);
      if ($203) {
       HEAP32[$201>>2] = $R7$1;
      } else {
       $204 = (($161) + 20|0);
       HEAP32[$204>>2] = $R7$1;
      }
      $205 = ($R7$1|0)==(0|0);
      if ($205) {
       break;
      }
     }
     $206 = HEAP32[((39000 + 16|0))>>2]|0;
     $207 = ($R7$1>>>0)<($206>>>0);
     if ($207) {
      _abort();
      // unreachable;
     }
     $208 = (($R7$1) + 24|0);
     HEAP32[$208>>2] = $161;
     $$sum19 = (($8) + 8)|0;
     $209 = (($mem) + ($$sum19)|0);
     $210 = HEAP32[$209>>2]|0;
     $211 = ($210|0)==(0|0);
     do {
      if (!($211)) {
       $212 = ($210>>>0)<($206>>>0);
       if ($212) {
        _abort();
        // unreachable;
       } else {
        $213 = (($R7$1) + 16|0);
        HEAP32[$213>>2] = $210;
        $214 = (($210) + 24|0);
        HEAP32[$214>>2] = $R7$1;
        break;
       }
      }
     } while(0);
     $$sum20 = (($8) + 12)|0;
     $215 = (($mem) + ($$sum20)|0);
     $216 = HEAP32[$215>>2]|0;
     $217 = ($216|0)==(0|0);
     if (!($217)) {
      $218 = HEAP32[((39000 + 16|0))>>2]|0;
      $219 = ($216>>>0)<($218>>>0);
      if ($219) {
       _abort();
       // unreachable;
      } else {
       $220 = (($R7$1) + 20|0);
       HEAP32[$220>>2] = $216;
       $221 = (($216) + 24|0);
       HEAP32[$221>>2] = $R7$1;
       break;
      }
     }
    }
   }
  } while(0);
  $222 = $133 | 1;
  $223 = (($p$0) + 4|0);
  HEAP32[$223>>2] = $222;
  $224 = (($p$0) + ($133)|0);
  HEAP32[$224>>2] = $133;
  $225 = HEAP32[((39000 + 20|0))>>2]|0;
  $226 = ($p$0|0)==($225|0);
  if ($226) {
   HEAP32[((39000 + 8|0))>>2] = $133;
   STACKTOP = sp;return;
  } else {
   $psize$1 = $133;
  }
 } else {
  $227 = $112 & -2;
  HEAP32[$111>>2] = $227;
  $228 = $psize$0 | 1;
  $229 = (($p$0) + 4|0);
  HEAP32[$229>>2] = $228;
  $230 = (($p$0) + ($psize$0)|0);
  HEAP32[$230>>2] = $psize$0;
  $psize$1 = $psize$0;
 }
 $231 = $psize$1 >>> 3;
 $232 = ($psize$1>>>0)<(256);
 if ($232) {
  $233 = $231 << 1;
  $234 = ((39000 + ($233<<2)|0) + 40|0);
  $235 = HEAP32[39000>>2]|0;
  $236 = 1 << $231;
  $237 = $235 & $236;
  $238 = ($237|0)==(0);
  if ($238) {
   $239 = $235 | $236;
   HEAP32[39000>>2] = $239;
   $$sum16$pre = (($233) + 2)|0;
   $$pre = ((39000 + ($$sum16$pre<<2)|0) + 40|0);
   $$pre$phiZ2D = $$pre;$F16$0 = $234;
  } else {
   $$sum17 = (($233) + 2)|0;
   $240 = ((39000 + ($$sum17<<2)|0) + 40|0);
   $241 = HEAP32[$240>>2]|0;
   $242 = HEAP32[((39000 + 16|0))>>2]|0;
   $243 = ($241>>>0)<($242>>>0);
   if ($243) {
    _abort();
    // unreachable;
   } else {
    $$pre$phiZ2D = $240;$F16$0 = $241;
   }
  }
  HEAP32[$$pre$phiZ2D>>2] = $p$0;
  $244 = (($F16$0) + 12|0);
  HEAP32[$244>>2] = $p$0;
  $245 = (($p$0) + 8|0);
  HEAP32[$245>>2] = $F16$0;
  $246 = (($p$0) + 12|0);
  HEAP32[$246>>2] = $234;
  STACKTOP = sp;return;
 }
 $247 = $psize$1 >>> 8;
 $248 = ($247|0)==(0);
 if ($248) {
  $I18$0 = 0;
 } else {
  $249 = ($psize$1>>>0)>(16777215);
  if ($249) {
   $I18$0 = 31;
  } else {
   $250 = (($247) + 1048320)|0;
   $251 = $250 >>> 16;
   $252 = $251 & 8;
   $253 = $247 << $252;
   $254 = (($253) + 520192)|0;
   $255 = $254 >>> 16;
   $256 = $255 & 4;
   $257 = $256 | $252;
   $258 = $253 << $256;
   $259 = (($258) + 245760)|0;
   $260 = $259 >>> 16;
   $261 = $260 & 2;
   $262 = $257 | $261;
   $263 = (14 - ($262))|0;
   $264 = $258 << $261;
   $265 = $264 >>> 15;
   $266 = (($263) + ($265))|0;
   $267 = $266 << 1;
   $268 = (($266) + 7)|0;
   $269 = $psize$1 >>> $268;
   $270 = $269 & 1;
   $271 = $270 | $267;
   $I18$0 = $271;
  }
 }
 $272 = ((39000 + ($I18$0<<2)|0) + 304|0);
 $273 = (($p$0) + 28|0);
 $I18$0$c = $I18$0;
 HEAP32[$273>>2] = $I18$0$c;
 $274 = (($p$0) + 20|0);
 HEAP32[$274>>2] = 0;
 $275 = (($p$0) + 16|0);
 HEAP32[$275>>2] = 0;
 $276 = HEAP32[((39000 + 4|0))>>2]|0;
 $277 = 1 << $I18$0;
 $278 = $276 & $277;
 $279 = ($278|0)==(0);
 L199: do {
  if ($279) {
   $280 = $276 | $277;
   HEAP32[((39000 + 4|0))>>2] = $280;
   HEAP32[$272>>2] = $p$0;
   $281 = (($p$0) + 24|0);
   HEAP32[$281>>2] = $272;
   $282 = (($p$0) + 12|0);
   HEAP32[$282>>2] = $p$0;
   $283 = (($p$0) + 8|0);
   HEAP32[$283>>2] = $p$0;
  } else {
   $284 = HEAP32[$272>>2]|0;
   $285 = ($I18$0|0)==(31);
   if ($285) {
    $293 = 0;
   } else {
    $286 = $I18$0 >>> 1;
    $287 = (25 - ($286))|0;
    $293 = $287;
   }
   $288 = (($284) + 4|0);
   $289 = HEAP32[$288>>2]|0;
   $290 = $289 & -8;
   $291 = ($290|0)==($psize$1|0);
   L205: do {
    if ($291) {
     $T$0$lcssa = $284;
    } else {
     $292 = $psize$1 << $293;
     $K19$058 = $292;$T$057 = $284;
     while(1) {
      $300 = $K19$058 >>> 31;
      $301 = ((($T$057) + ($300<<2)|0) + 16|0);
      $296 = HEAP32[$301>>2]|0;
      $302 = ($296|0)==(0|0);
      if ($302) {
       break;
      }
      $294 = $K19$058 << 1;
      $295 = (($296) + 4|0);
      $297 = HEAP32[$295>>2]|0;
      $298 = $297 & -8;
      $299 = ($298|0)==($psize$1|0);
      if ($299) {
       $T$0$lcssa = $296;
       break L205;
      } else {
       $K19$058 = $294;$T$057 = $296;
      }
     }
     $303 = HEAP32[((39000 + 16|0))>>2]|0;
     $304 = ($301>>>0)<($303>>>0);
     if ($304) {
      _abort();
      // unreachable;
     } else {
      HEAP32[$301>>2] = $p$0;
      $305 = (($p$0) + 24|0);
      HEAP32[$305>>2] = $T$057;
      $306 = (($p$0) + 12|0);
      HEAP32[$306>>2] = $p$0;
      $307 = (($p$0) + 8|0);
      HEAP32[$307>>2] = $p$0;
      break L199;
     }
    }
   } while(0);
   $308 = (($T$0$lcssa) + 8|0);
   $309 = HEAP32[$308>>2]|0;
   $310 = HEAP32[((39000 + 16|0))>>2]|0;
   $311 = ($T$0$lcssa>>>0)>=($310>>>0);
   $312 = ($309>>>0)>=($310>>>0);
   $or$cond = $311 & $312;
   if ($or$cond) {
    $313 = (($309) + 12|0);
    HEAP32[$313>>2] = $p$0;
    HEAP32[$308>>2] = $p$0;
    $314 = (($p$0) + 8|0);
    HEAP32[$314>>2] = $309;
    $315 = (($p$0) + 12|0);
    HEAP32[$315>>2] = $T$0$lcssa;
    $316 = (($p$0) + 24|0);
    HEAP32[$316>>2] = 0;
    break;
   } else {
    _abort();
    // unreachable;
   }
  }
 } while(0);
 $317 = HEAP32[((39000 + 32|0))>>2]|0;
 $318 = (($317) + -1)|0;
 HEAP32[((39000 + 32|0))>>2] = $318;
 $319 = ($318|0)==(0);
 if ($319) {
  $sp$0$in$i = ((39000 + 456|0));
 } else {
  STACKTOP = sp;return;
 }
 while(1) {
  $sp$0$i = HEAP32[$sp$0$in$i>>2]|0;
  $320 = ($sp$0$i|0)==(0|0);
  $321 = (($sp$0$i) + 8|0);
  if ($320) {
   break;
  } else {
   $sp$0$in$i = $321;
  }
 }
 HEAP32[((39000 + 32|0))>>2] = -1;
 STACKTOP = sp;return;
}
function _realloc($oldmem,$bytes) {
 $oldmem = $oldmem|0;
 $bytes = $bytes|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0;
 var $7 = 0, $8 = 0, $9 = 0, $mem$0 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 $0 = ($oldmem|0)==(0|0);
 do {
  if ($0) {
   $1 = (_malloc($bytes)|0);
   $mem$0 = $1;
  } else {
   $2 = ($bytes>>>0)>(4294967231);
   if ($2) {
    $3 = (___errno_location()|0);
    HEAP32[$3>>2] = 12;
    $mem$0 = 0;
    break;
   }
   $4 = ($bytes>>>0)<(11);
   if ($4) {
    $8 = 16;
   } else {
    $5 = (($bytes) + 11)|0;
    $6 = $5 & -8;
    $8 = $6;
   }
   $7 = (($oldmem) + -8|0);
   $9 = (_try_realloc_chunk($7,$8)|0);
   $10 = ($9|0)==(0|0);
   if (!($10)) {
    $11 = (($9) + 8|0);
    $mem$0 = $11;
    break;
   }
   $12 = (_malloc($bytes)|0);
   $13 = ($12|0)==(0|0);
   if ($13) {
    $mem$0 = 0;
   } else {
    $14 = (($oldmem) + -4|0);
    $15 = HEAP32[$14>>2]|0;
    $16 = $15 & -8;
    $17 = $15 & 3;
    $18 = ($17|0)==(0);
    $19 = $18 ? 8 : 4;
    $20 = (($16) - ($19))|0;
    $21 = ($20>>>0)<($bytes>>>0);
    $22 = $21 ? $20 : $bytes;
    _memcpy(($12|0),($oldmem|0),($22|0))|0;
    _free($oldmem);
    $mem$0 = $12;
   }
  }
 } while(0);
 STACKTOP = sp;return ($mem$0|0);
}
function _memcmp($vl,$vr,$n) {
 $vl = $vl|0;
 $vr = $vr|0;
 $n = $n|0;
 var $$02 = 0, $0 = 0, $1 = 0, $10 = 0, $11 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $l$03 = 0, $r$04 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 $0 = ($n|0)==(0);
 L1: do {
  if ($0) {
   $11 = 0;
  } else {
   $$02 = $n;$l$03 = $vl;$r$04 = $vr;
   while(1) {
    $1 = HEAP8[$l$03>>0]|0;
    $2 = HEAP8[$r$04>>0]|0;
    $3 = ($1<<24>>24)==($2<<24>>24);
    if (!($3)) {
     break;
    }
    $4 = (($$02) + -1)|0;
    $5 = (($l$03) + 1|0);
    $6 = (($r$04) + 1|0);
    $7 = ($4|0)==(0);
    if ($7) {
     $11 = 0;
     break L1;
    } else {
     $$02 = $4;$l$03 = $5;$r$04 = $6;
    }
   }
   $8 = $1&255;
   $9 = $2&255;
   $10 = (($8) - ($9))|0;
   $11 = $10;
  }
 } while(0);
 STACKTOP = sp;return ($11|0);
}
function _strcmp($l,$r) {
 $l = $l|0;
 $r = $r|0;
 var $$014 = 0, $$05 = 0, $$lcssa = 0, $$lcssa2 = 0, $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $or$cond = 0, $or$cond3 = 0, label = 0;
 var sp = 0;
 sp = STACKTOP;
 $0 = HEAP8[$l>>0]|0;
 $1 = HEAP8[$r>>0]|0;
 $2 = ($0<<24>>24)!=($1<<24>>24);
 $3 = ($0<<24>>24)==(0);
 $or$cond3 = $2 | $3;
 if ($or$cond3) {
  $$lcssa = $0;$$lcssa2 = $1;
 } else {
  $$014 = $l;$$05 = $r;
  while(1) {
   $4 = (($$014) + 1|0);
   $5 = (($$05) + 1|0);
   $6 = HEAP8[$4>>0]|0;
   $7 = HEAP8[$5>>0]|0;
   $8 = ($6<<24>>24)!=($7<<24>>24);
   $9 = ($6<<24>>24)==(0);
   $or$cond = $8 | $9;
   if ($or$cond) {
    $$lcssa = $6;$$lcssa2 = $7;
    break;
   } else {
    $$014 = $4;$$05 = $5;
   }
  }
 }
 $10 = $$lcssa&255;
 $11 = $$lcssa2&255;
 $12 = (($10) - ($11))|0;
 STACKTOP = sp;return ($12|0);
}
function _strncmp($_l,$_r,$n) {
 $_l = $_l|0;
 $_r = $_r|0;
 $n = $n|0;
 var $$02 = 0, $$07 = 0, $$07$in = 0, $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0;
 var $l$05 = 0, $or$cond = 0, $or$cond3 = 0, $r$0$lcssa = 0, $r$06 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 $0 = ($n|0)==(0);
 if ($0) {
  $$02 = 0;
  STACKTOP = sp;return ($$02|0);
 }
 $1 = HEAP8[$_l>>0]|0;
 $2 = ($1<<24>>24)==(0);
 L4: do {
  if ($2) {
   $13 = 0;$r$0$lcssa = $_r;
  } else {
   $$07$in = $n;$7 = $1;$l$05 = $_l;$r$06 = $_r;
   while(1) {
    $$07 = (($$07$in) + -1)|0;
    $3 = HEAP8[$r$06>>0]|0;
    $4 = ($3<<24>>24)!=(0);
    $5 = ($$07|0)!=(0);
    $or$cond = $4 & $5;
    $6 = ($7<<24>>24)==($3<<24>>24);
    $or$cond3 = $or$cond & $6;
    if (!($or$cond3)) {
     $13 = $7;$r$0$lcssa = $r$06;
     break L4;
    }
    $8 = (($l$05) + 1|0);
    $9 = (($r$06) + 1|0);
    $10 = HEAP8[$8>>0]|0;
    $11 = ($10<<24>>24)==(0);
    if ($11) {
     $13 = 0;$r$0$lcssa = $9;
     break;
    } else {
     $$07$in = $$07;$7 = $10;$l$05 = $8;$r$06 = $9;
    }
   }
  }
 } while(0);
 $12 = $13&255;
 $14 = HEAP8[$r$0$lcssa>>0]|0;
 $15 = $14&255;
 $16 = (($12) - ($15))|0;
 $$02 = $16;
 STACKTOP = sp;return ($$02|0);
}
function _try_realloc_chunk($p,$nb) {
 $p = $p|0;
 $nb = $nb|0;
 var $$pre = 0, $$pre$phiZ2D = 0, $$sum = 0, $$sum11 = 0, $$sum12 = 0, $$sum13 = 0, $$sum14 = 0, $$sum15 = 0, $$sum16 = 0, $$sum17 = 0, $$sum19 = 0, $$sum2 = 0, $$sum20 = 0, $$sum22 = 0, $$sum23 = 0, $$sum2728 = 0, $$sum3 = 0, $$sum4 = 0, $$sum5 = 0, $$sum78 = 0;
 var $$sum910 = 0, $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0;
 var $115 = 0, $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0;
 var $133 = 0, $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0;
 var $151 = 0, $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0;
 var $17 = 0, $170 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0;
 var $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0;
 var $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0;
 var $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0;
 var $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $R$0 = 0, $R$1 = 0, $RP$0 = 0, $cond = 0, $newp$0 = 0, $notlhs = 0, $notrhs = 0, $or$cond$not = 0;
 var $or$cond30 = 0, $storemerge = 0, $storemerge21 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 $0 = (($p) + 4|0);
 $1 = HEAP32[$0>>2]|0;
 $2 = $1 & -8;
 $3 = (($p) + ($2)|0);
 $4 = HEAP32[((39000 + 16|0))>>2]|0;
 $5 = $1 & 3;
 $notlhs = ($p>>>0)>=($4>>>0);
 $notrhs = ($5|0)!=(1);
 $or$cond$not = $notrhs & $notlhs;
 $6 = ($p>>>0)<($3>>>0);
 $or$cond30 = $or$cond$not & $6;
 if (!($or$cond30)) {
  _abort();
  // unreachable;
 }
 $$sum2728 = $2 | 4;
 $7 = (($p) + ($$sum2728)|0);
 $8 = HEAP32[$7>>2]|0;
 $9 = $8 & 1;
 $10 = ($9|0)==(0);
 if ($10) {
  _abort();
  // unreachable;
 }
 $11 = ($5|0)==(0);
 if ($11) {
  $12 = ($nb>>>0)<(256);
  if ($12) {
   $newp$0 = 0;
   STACKTOP = sp;return ($newp$0|0);
  }
  $13 = (($nb) + 4)|0;
  $14 = ($2>>>0)<($13>>>0);
  if (!($14)) {
   $15 = (($2) - ($nb))|0;
   $16 = HEAP32[((39472 + 8|0))>>2]|0;
   $17 = $16 << 1;
   $18 = ($15>>>0)>($17>>>0);
   if (!($18)) {
    $newp$0 = $p;
    STACKTOP = sp;return ($newp$0|0);
   }
  }
  $newp$0 = 0;
  STACKTOP = sp;return ($newp$0|0);
 }
 $19 = ($2>>>0)<($nb>>>0);
 if (!($19)) {
  $20 = (($2) - ($nb))|0;
  $21 = ($20>>>0)>(15);
  if (!($21)) {
   $newp$0 = $p;
   STACKTOP = sp;return ($newp$0|0);
  }
  $22 = (($p) + ($nb)|0);
  $23 = $1 & 1;
  $24 = $23 | $nb;
  $25 = $24 | 2;
  HEAP32[$0>>2] = $25;
  $$sum23 = (($nb) + 4)|0;
  $26 = (($p) + ($$sum23)|0);
  $27 = $20 | 3;
  HEAP32[$26>>2] = $27;
  $28 = HEAP32[$7>>2]|0;
  $29 = $28 | 1;
  HEAP32[$7>>2] = $29;
  _dispose_chunk($22,$20);
  $newp$0 = $p;
  STACKTOP = sp;return ($newp$0|0);
 }
 $30 = HEAP32[((39000 + 24|0))>>2]|0;
 $31 = ($3|0)==($30|0);
 if ($31) {
  $32 = HEAP32[((39000 + 12|0))>>2]|0;
  $33 = (($32) + ($2))|0;
  $34 = ($33>>>0)>($nb>>>0);
  if (!($34)) {
   $newp$0 = 0;
   STACKTOP = sp;return ($newp$0|0);
  }
  $35 = (($33) - ($nb))|0;
  $36 = (($p) + ($nb)|0);
  $37 = $1 & 1;
  $38 = $37 | $nb;
  $39 = $38 | 2;
  HEAP32[$0>>2] = $39;
  $$sum22 = (($nb) + 4)|0;
  $40 = (($p) + ($$sum22)|0);
  $41 = $35 | 1;
  HEAP32[$40>>2] = $41;
  HEAP32[((39000 + 24|0))>>2] = $36;
  HEAP32[((39000 + 12|0))>>2] = $35;
  $newp$0 = $p;
  STACKTOP = sp;return ($newp$0|0);
 }
 $42 = HEAP32[((39000 + 20|0))>>2]|0;
 $43 = ($3|0)==($42|0);
 if ($43) {
  $44 = HEAP32[((39000 + 8|0))>>2]|0;
  $45 = (($44) + ($2))|0;
  $46 = ($45>>>0)<($nb>>>0);
  if ($46) {
   $newp$0 = 0;
   STACKTOP = sp;return ($newp$0|0);
  }
  $47 = (($45) - ($nb))|0;
  $48 = ($47>>>0)>(15);
  if ($48) {
   $49 = (($p) + ($nb)|0);
   $50 = (($p) + ($45)|0);
   $51 = $1 & 1;
   $52 = $51 | $nb;
   $53 = $52 | 2;
   HEAP32[$0>>2] = $53;
   $$sum19 = (($nb) + 4)|0;
   $54 = (($p) + ($$sum19)|0);
   $55 = $47 | 1;
   HEAP32[$54>>2] = $55;
   HEAP32[$50>>2] = $47;
   $$sum20 = (($45) + 4)|0;
   $56 = (($p) + ($$sum20)|0);
   $57 = HEAP32[$56>>2]|0;
   $58 = $57 & -2;
   HEAP32[$56>>2] = $58;
   $storemerge = $49;$storemerge21 = $47;
  } else {
   $59 = $1 & 1;
   $60 = $59 | $45;
   $61 = $60 | 2;
   HEAP32[$0>>2] = $61;
   $$sum17 = (($45) + 4)|0;
   $62 = (($p) + ($$sum17)|0);
   $63 = HEAP32[$62>>2]|0;
   $64 = $63 | 1;
   HEAP32[$62>>2] = $64;
   $storemerge = 0;$storemerge21 = 0;
  }
  HEAP32[((39000 + 8|0))>>2] = $storemerge21;
  HEAP32[((39000 + 20|0))>>2] = $storemerge;
  $newp$0 = $p;
  STACKTOP = sp;return ($newp$0|0);
 }
 $65 = $8 & 2;
 $66 = ($65|0)==(0);
 if (!($66)) {
  $newp$0 = 0;
  STACKTOP = sp;return ($newp$0|0);
 }
 $67 = $8 & -8;
 $68 = (($67) + ($2))|0;
 $69 = ($68>>>0)<($nb>>>0);
 if ($69) {
  $newp$0 = 0;
  STACKTOP = sp;return ($newp$0|0);
 }
 $70 = (($68) - ($nb))|0;
 $71 = $8 >>> 3;
 $72 = ($8>>>0)<(256);
 do {
  if ($72) {
   $$sum15 = (($2) + 8)|0;
   $73 = (($p) + ($$sum15)|0);
   $74 = HEAP32[$73>>2]|0;
   $$sum16 = (($2) + 12)|0;
   $75 = (($p) + ($$sum16)|0);
   $76 = HEAP32[$75>>2]|0;
   $77 = $71 << 1;
   $78 = ((39000 + ($77<<2)|0) + 40|0);
   $79 = ($74|0)==($78|0);
   if (!($79)) {
    $80 = ($74>>>0)<($4>>>0);
    if ($80) {
     _abort();
     // unreachable;
    }
    $81 = (($74) + 12|0);
    $82 = HEAP32[$81>>2]|0;
    $83 = ($82|0)==($3|0);
    if (!($83)) {
     _abort();
     // unreachable;
    }
   }
   $84 = ($76|0)==($74|0);
   if ($84) {
    $85 = 1 << $71;
    $86 = $85 ^ -1;
    $87 = HEAP32[39000>>2]|0;
    $88 = $87 & $86;
    HEAP32[39000>>2] = $88;
    break;
   }
   $89 = ($76|0)==($78|0);
   if ($89) {
    $$pre = (($76) + 8|0);
    $$pre$phiZ2D = $$pre;
   } else {
    $90 = ($76>>>0)<($4>>>0);
    if ($90) {
     _abort();
     // unreachable;
    }
    $91 = (($76) + 8|0);
    $92 = HEAP32[$91>>2]|0;
    $93 = ($92|0)==($3|0);
    if ($93) {
     $$pre$phiZ2D = $91;
    } else {
     _abort();
     // unreachable;
    }
   }
   $94 = (($74) + 12|0);
   HEAP32[$94>>2] = $76;
   HEAP32[$$pre$phiZ2D>>2] = $74;
  } else {
   $$sum = (($2) + 24)|0;
   $95 = (($p) + ($$sum)|0);
   $96 = HEAP32[$95>>2]|0;
   $$sum2 = (($2) + 12)|0;
   $97 = (($p) + ($$sum2)|0);
   $98 = HEAP32[$97>>2]|0;
   $99 = ($98|0)==($3|0);
   do {
    if ($99) {
     $$sum4 = (($2) + 20)|0;
     $109 = (($p) + ($$sum4)|0);
     $110 = HEAP32[$109>>2]|0;
     $111 = ($110|0)==(0|0);
     if ($111) {
      $$sum3 = (($2) + 16)|0;
      $112 = (($p) + ($$sum3)|0);
      $113 = HEAP32[$112>>2]|0;
      $114 = ($113|0)==(0|0);
      if ($114) {
       $R$1 = 0;
       break;
      } else {
       $R$0 = $113;$RP$0 = $112;
      }
     } else {
      $R$0 = $110;$RP$0 = $109;
     }
     while(1) {
      $115 = (($R$0) + 20|0);
      $116 = HEAP32[$115>>2]|0;
      $117 = ($116|0)==(0|0);
      if (!($117)) {
       $R$0 = $116;$RP$0 = $115;
       continue;
      }
      $118 = (($R$0) + 16|0);
      $119 = HEAP32[$118>>2]|0;
      $120 = ($119|0)==(0|0);
      if ($120) {
       break;
      } else {
       $R$0 = $119;$RP$0 = $118;
      }
     }
     $121 = ($RP$0>>>0)<($4>>>0);
     if ($121) {
      _abort();
      // unreachable;
     } else {
      HEAP32[$RP$0>>2] = 0;
      $R$1 = $R$0;
      break;
     }
    } else {
     $$sum14 = (($2) + 8)|0;
     $100 = (($p) + ($$sum14)|0);
     $101 = HEAP32[$100>>2]|0;
     $102 = ($101>>>0)<($4>>>0);
     if ($102) {
      _abort();
      // unreachable;
     }
     $103 = (($101) + 12|0);
     $104 = HEAP32[$103>>2]|0;
     $105 = ($104|0)==($3|0);
     if (!($105)) {
      _abort();
      // unreachable;
     }
     $106 = (($98) + 8|0);
     $107 = HEAP32[$106>>2]|0;
     $108 = ($107|0)==($3|0);
     if ($108) {
      HEAP32[$103>>2] = $98;
      HEAP32[$106>>2] = $101;
      $R$1 = $98;
      break;
     } else {
      _abort();
      // unreachable;
     }
    }
   } while(0);
   $122 = ($96|0)==(0|0);
   if (!($122)) {
    $$sum11 = (($2) + 28)|0;
    $123 = (($p) + ($$sum11)|0);
    $124 = HEAP32[$123>>2]|0;
    $125 = ((39000 + ($124<<2)|0) + 304|0);
    $126 = HEAP32[$125>>2]|0;
    $127 = ($3|0)==($126|0);
    if ($127) {
     HEAP32[$125>>2] = $R$1;
     $cond = ($R$1|0)==(0|0);
     if ($cond) {
      $128 = 1 << $124;
      $129 = $128 ^ -1;
      $130 = HEAP32[((39000 + 4|0))>>2]|0;
      $131 = $130 & $129;
      HEAP32[((39000 + 4|0))>>2] = $131;
      break;
     }
    } else {
     $132 = HEAP32[((39000 + 16|0))>>2]|0;
     $133 = ($96>>>0)<($132>>>0);
     if ($133) {
      _abort();
      // unreachable;
     }
     $134 = (($96) + 16|0);
     $135 = HEAP32[$134>>2]|0;
     $136 = ($135|0)==($3|0);
     if ($136) {
      HEAP32[$134>>2] = $R$1;
     } else {
      $137 = (($96) + 20|0);
      HEAP32[$137>>2] = $R$1;
     }
     $138 = ($R$1|0)==(0|0);
     if ($138) {
      break;
     }
    }
    $139 = HEAP32[((39000 + 16|0))>>2]|0;
    $140 = ($R$1>>>0)<($139>>>0);
    if ($140) {
     _abort();
     // unreachable;
    }
    $141 = (($R$1) + 24|0);
    HEAP32[$141>>2] = $96;
    $$sum12 = (($2) + 16)|0;
    $142 = (($p) + ($$sum12)|0);
    $143 = HEAP32[$142>>2]|0;
    $144 = ($143|0)==(0|0);
    do {
     if (!($144)) {
      $145 = ($143>>>0)<($139>>>0);
      if ($145) {
       _abort();
       // unreachable;
      } else {
       $146 = (($R$1) + 16|0);
       HEAP32[$146>>2] = $143;
       $147 = (($143) + 24|0);
       HEAP32[$147>>2] = $R$1;
       break;
      }
     }
    } while(0);
    $$sum13 = (($2) + 20)|0;
    $148 = (($p) + ($$sum13)|0);
    $149 = HEAP32[$148>>2]|0;
    $150 = ($149|0)==(0|0);
    if (!($150)) {
     $151 = HEAP32[((39000 + 16|0))>>2]|0;
     $152 = ($149>>>0)<($151>>>0);
     if ($152) {
      _abort();
      // unreachable;
     } else {
      $153 = (($R$1) + 20|0);
      HEAP32[$153>>2] = $149;
      $154 = (($149) + 24|0);
      HEAP32[$154>>2] = $R$1;
      break;
     }
    }
   }
  }
 } while(0);
 $155 = ($70>>>0)<(16);
 if ($155) {
  $156 = $1 & 1;
  $157 = $68 | $156;
  $158 = $157 | 2;
  HEAP32[$0>>2] = $158;
  $$sum910 = $68 | 4;
  $159 = (($p) + ($$sum910)|0);
  $160 = HEAP32[$159>>2]|0;
  $161 = $160 | 1;
  HEAP32[$159>>2] = $161;
  $newp$0 = $p;
  STACKTOP = sp;return ($newp$0|0);
 } else {
  $162 = (($p) + ($nb)|0);
  $163 = $1 & 1;
  $164 = $163 | $nb;
  $165 = $164 | 2;
  HEAP32[$0>>2] = $165;
  $$sum5 = (($nb) + 4)|0;
  $166 = (($p) + ($$sum5)|0);
  $167 = $70 | 3;
  HEAP32[$166>>2] = $167;
  $$sum78 = $68 | 4;
  $168 = (($p) + ($$sum78)|0);
  $169 = HEAP32[$168>>2]|0;
  $170 = $169 | 1;
  HEAP32[$168>>2] = $170;
  _dispose_chunk($162,$70);
  $newp$0 = $p;
  STACKTOP = sp;return ($newp$0|0);
 }
 return 0|0;
}
function _dispose_chunk($p,$psize) {
 $p = $p|0;
 $psize = $psize|0;
 var $$0 = 0, $$02 = 0, $$1 = 0, $$pre = 0, $$pre$phi57Z2D = 0, $$pre$phi59Z2D = 0, $$pre$phiZ2D = 0, $$pre56 = 0, $$pre58 = 0, $$sum = 0, $$sum1 = 0, $$sum12$pre = 0, $$sum13 = 0, $$sum14 = 0, $$sum15 = 0, $$sum16 = 0, $$sum17 = 0, $$sum18 = 0, $$sum19 = 0, $$sum2 = 0;
 var $$sum20 = 0, $$sum22 = 0, $$sum23 = 0, $$sum24 = 0, $$sum25 = 0, $$sum26 = 0, $$sum27 = 0, $$sum28 = 0, $$sum29 = 0, $$sum3 = 0, $$sum30 = 0, $$sum31 = 0, $$sum4 = 0, $$sum5 = 0, $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0;
 var $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0, $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0;
 var $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0, $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0;
 var $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0, $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0;
 var $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0, $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0;
 var $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0, $188 = 0, $189 = 0, $19 = 0, $190 = 0, $191 = 0, $192 = 0, $193 = 0;
 var $194 = 0, $195 = 0, $196 = 0, $197 = 0, $198 = 0, $199 = 0, $2 = 0, $20 = 0, $200 = 0, $201 = 0, $202 = 0, $203 = 0, $204 = 0, $205 = 0, $206 = 0, $207 = 0, $208 = 0, $209 = 0, $21 = 0, $210 = 0;
 var $211 = 0, $212 = 0, $213 = 0, $214 = 0, $215 = 0, $216 = 0, $217 = 0, $218 = 0, $219 = 0, $22 = 0, $220 = 0, $221 = 0, $222 = 0, $223 = 0, $224 = 0, $225 = 0, $226 = 0, $227 = 0, $228 = 0, $229 = 0;
 var $23 = 0, $230 = 0, $231 = 0, $232 = 0, $233 = 0, $234 = 0, $235 = 0, $236 = 0, $237 = 0, $238 = 0, $239 = 0, $24 = 0, $240 = 0, $241 = 0, $242 = 0, $243 = 0, $244 = 0, $245 = 0, $246 = 0, $247 = 0;
 var $248 = 0, $249 = 0, $25 = 0, $250 = 0, $251 = 0, $252 = 0, $253 = 0, $254 = 0, $255 = 0, $256 = 0, $257 = 0, $258 = 0, $259 = 0, $26 = 0, $260 = 0, $261 = 0, $262 = 0, $263 = 0, $264 = 0, $265 = 0;
 var $266 = 0, $267 = 0, $268 = 0, $269 = 0, $27 = 0, $270 = 0, $271 = 0, $272 = 0, $273 = 0, $274 = 0, $275 = 0, $276 = 0, $277 = 0, $278 = 0, $279 = 0, $28 = 0, $280 = 0, $281 = 0, $282 = 0, $283 = 0;
 var $284 = 0, $285 = 0, $286 = 0, $287 = 0, $288 = 0, $289 = 0, $29 = 0, $290 = 0, $291 = 0, $292 = 0, $293 = 0, $294 = 0, $295 = 0, $296 = 0, $297 = 0, $298 = 0, $299 = 0, $3 = 0, $30 = 0, $300 = 0;
 var $301 = 0, $302 = 0, $303 = 0, $304 = 0, $305 = 0, $306 = 0, $307 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0;
 var $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0;
 var $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0;
 var $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0;
 var $98 = 0, $99 = 0, $F16$0 = 0, $I19$0 = 0, $I19$0$c = 0, $K20$050 = 0, $R$0 = 0, $R$1 = 0, $R7$0 = 0, $R7$1 = 0, $RP$0 = 0, $RP9$0 = 0, $T$0$lcssa = 0, $T$049 = 0, $cond = 0, $cond46 = 0, $or$cond = 0, label = 0, sp = 0;
 sp = STACKTOP;
 $0 = (($p) + ($psize)|0);
 $1 = (($p) + 4|0);
 $2 = HEAP32[$1>>2]|0;
 $3 = $2 & 1;
 $4 = ($3|0)==(0);
 do {
  if ($4) {
   $5 = HEAP32[$p>>2]|0;
   $6 = $2 & 3;
   $7 = ($6|0)==(0);
   if ($7) {
    STACKTOP = sp;return;
   }
   $8 = (0 - ($5))|0;
   $9 = (($p) + ($8)|0);
   $10 = (($5) + ($psize))|0;
   $11 = HEAP32[((39000 + 16|0))>>2]|0;
   $12 = ($9>>>0)<($11>>>0);
   if ($12) {
    _abort();
    // unreachable;
   }
   $13 = HEAP32[((39000 + 20|0))>>2]|0;
   $14 = ($9|0)==($13|0);
   if ($14) {
    $$sum = (($psize) + 4)|0;
    $99 = (($p) + ($$sum)|0);
    $100 = HEAP32[$99>>2]|0;
    $101 = $100 & 3;
    $102 = ($101|0)==(3);
    if (!($102)) {
     $$0 = $9;$$02 = $10;
     break;
    }
    HEAP32[((39000 + 8|0))>>2] = $10;
    $103 = $100 & -2;
    HEAP32[$99>>2] = $103;
    $104 = $10 | 1;
    $$sum20 = (4 - ($5))|0;
    $105 = (($p) + ($$sum20)|0);
    HEAP32[$105>>2] = $104;
    HEAP32[$0>>2] = $10;
    STACKTOP = sp;return;
   }
   $15 = $5 >>> 3;
   $16 = ($5>>>0)<(256);
   if ($16) {
    $$sum30 = (8 - ($5))|0;
    $17 = (($p) + ($$sum30)|0);
    $18 = HEAP32[$17>>2]|0;
    $$sum31 = (12 - ($5))|0;
    $19 = (($p) + ($$sum31)|0);
    $20 = HEAP32[$19>>2]|0;
    $21 = $15 << 1;
    $22 = ((39000 + ($21<<2)|0) + 40|0);
    $23 = ($18|0)==($22|0);
    if (!($23)) {
     $24 = ($18>>>0)<($11>>>0);
     if ($24) {
      _abort();
      // unreachable;
     }
     $25 = (($18) + 12|0);
     $26 = HEAP32[$25>>2]|0;
     $27 = ($26|0)==($9|0);
     if (!($27)) {
      _abort();
      // unreachable;
     }
    }
    $28 = ($20|0)==($18|0);
    if ($28) {
     $29 = 1 << $15;
     $30 = $29 ^ -1;
     $31 = HEAP32[39000>>2]|0;
     $32 = $31 & $30;
     HEAP32[39000>>2] = $32;
     $$0 = $9;$$02 = $10;
     break;
    }
    $33 = ($20|0)==($22|0);
    if ($33) {
     $$pre58 = (($20) + 8|0);
     $$pre$phi59Z2D = $$pre58;
    } else {
     $34 = ($20>>>0)<($11>>>0);
     if ($34) {
      _abort();
      // unreachable;
     }
     $35 = (($20) + 8|0);
     $36 = HEAP32[$35>>2]|0;
     $37 = ($36|0)==($9|0);
     if ($37) {
      $$pre$phi59Z2D = $35;
     } else {
      _abort();
      // unreachable;
     }
    }
    $38 = (($18) + 12|0);
    HEAP32[$38>>2] = $20;
    HEAP32[$$pre$phi59Z2D>>2] = $18;
    $$0 = $9;$$02 = $10;
    break;
   }
   $$sum22 = (24 - ($5))|0;
   $39 = (($p) + ($$sum22)|0);
   $40 = HEAP32[$39>>2]|0;
   $$sum23 = (12 - ($5))|0;
   $41 = (($p) + ($$sum23)|0);
   $42 = HEAP32[$41>>2]|0;
   $43 = ($42|0)==($9|0);
   do {
    if ($43) {
     $$sum24 = (16 - ($5))|0;
     $$sum25 = (($$sum24) + 4)|0;
     $53 = (($p) + ($$sum25)|0);
     $54 = HEAP32[$53>>2]|0;
     $55 = ($54|0)==(0|0);
     if ($55) {
      $56 = (($p) + ($$sum24)|0);
      $57 = HEAP32[$56>>2]|0;
      $58 = ($57|0)==(0|0);
      if ($58) {
       $R$1 = 0;
       break;
      } else {
       $R$0 = $57;$RP$0 = $56;
      }
     } else {
      $R$0 = $54;$RP$0 = $53;
     }
     while(1) {
      $59 = (($R$0) + 20|0);
      $60 = HEAP32[$59>>2]|0;
      $61 = ($60|0)==(0|0);
      if (!($61)) {
       $R$0 = $60;$RP$0 = $59;
       continue;
      }
      $62 = (($R$0) + 16|0);
      $63 = HEAP32[$62>>2]|0;
      $64 = ($63|0)==(0|0);
      if ($64) {
       break;
      } else {
       $R$0 = $63;$RP$0 = $62;
      }
     }
     $65 = ($RP$0>>>0)<($11>>>0);
     if ($65) {
      _abort();
      // unreachable;
     } else {
      HEAP32[$RP$0>>2] = 0;
      $R$1 = $R$0;
      break;
     }
    } else {
     $$sum29 = (8 - ($5))|0;
     $44 = (($p) + ($$sum29)|0);
     $45 = HEAP32[$44>>2]|0;
     $46 = ($45>>>0)<($11>>>0);
     if ($46) {
      _abort();
      // unreachable;
     }
     $47 = (($45) + 12|0);
     $48 = HEAP32[$47>>2]|0;
     $49 = ($48|0)==($9|0);
     if (!($49)) {
      _abort();
      // unreachable;
     }
     $50 = (($42) + 8|0);
     $51 = HEAP32[$50>>2]|0;
     $52 = ($51|0)==($9|0);
     if ($52) {
      HEAP32[$47>>2] = $42;
      HEAP32[$50>>2] = $45;
      $R$1 = $42;
      break;
     } else {
      _abort();
      // unreachable;
     }
    }
   } while(0);
   $66 = ($40|0)==(0|0);
   if ($66) {
    $$0 = $9;$$02 = $10;
   } else {
    $$sum26 = (28 - ($5))|0;
    $67 = (($p) + ($$sum26)|0);
    $68 = HEAP32[$67>>2]|0;
    $69 = ((39000 + ($68<<2)|0) + 304|0);
    $70 = HEAP32[$69>>2]|0;
    $71 = ($9|0)==($70|0);
    if ($71) {
     HEAP32[$69>>2] = $R$1;
     $cond = ($R$1|0)==(0|0);
     if ($cond) {
      $72 = 1 << $68;
      $73 = $72 ^ -1;
      $74 = HEAP32[((39000 + 4|0))>>2]|0;
      $75 = $74 & $73;
      HEAP32[((39000 + 4|0))>>2] = $75;
      $$0 = $9;$$02 = $10;
      break;
     }
    } else {
     $76 = HEAP32[((39000 + 16|0))>>2]|0;
     $77 = ($40>>>0)<($76>>>0);
     if ($77) {
      _abort();
      // unreachable;
     }
     $78 = (($40) + 16|0);
     $79 = HEAP32[$78>>2]|0;
     $80 = ($79|0)==($9|0);
     if ($80) {
      HEAP32[$78>>2] = $R$1;
     } else {
      $81 = (($40) + 20|0);
      HEAP32[$81>>2] = $R$1;
     }
     $82 = ($R$1|0)==(0|0);
     if ($82) {
      $$0 = $9;$$02 = $10;
      break;
     }
    }
    $83 = HEAP32[((39000 + 16|0))>>2]|0;
    $84 = ($R$1>>>0)<($83>>>0);
    if ($84) {
     _abort();
     // unreachable;
    }
    $85 = (($R$1) + 24|0);
    HEAP32[$85>>2] = $40;
    $$sum27 = (16 - ($5))|0;
    $86 = (($p) + ($$sum27)|0);
    $87 = HEAP32[$86>>2]|0;
    $88 = ($87|0)==(0|0);
    do {
     if (!($88)) {
      $89 = ($87>>>0)<($83>>>0);
      if ($89) {
       _abort();
       // unreachable;
      } else {
       $90 = (($R$1) + 16|0);
       HEAP32[$90>>2] = $87;
       $91 = (($87) + 24|0);
       HEAP32[$91>>2] = $R$1;
       break;
      }
     }
    } while(0);
    $$sum28 = (($$sum27) + 4)|0;
    $92 = (($p) + ($$sum28)|0);
    $93 = HEAP32[$92>>2]|0;
    $94 = ($93|0)==(0|0);
    if ($94) {
     $$0 = $9;$$02 = $10;
    } else {
     $95 = HEAP32[((39000 + 16|0))>>2]|0;
     $96 = ($93>>>0)<($95>>>0);
     if ($96) {
      _abort();
      // unreachable;
     } else {
      $97 = (($R$1) + 20|0);
      HEAP32[$97>>2] = $93;
      $98 = (($93) + 24|0);
      HEAP32[$98>>2] = $R$1;
      $$0 = $9;$$02 = $10;
      break;
     }
    }
   }
  } else {
   $$0 = $p;$$02 = $psize;
  }
 } while(0);
 $106 = HEAP32[((39000 + 16|0))>>2]|0;
 $107 = ($0>>>0)<($106>>>0);
 if ($107) {
  _abort();
  // unreachable;
 }
 $$sum1 = (($psize) + 4)|0;
 $108 = (($p) + ($$sum1)|0);
 $109 = HEAP32[$108>>2]|0;
 $110 = $109 & 2;
 $111 = ($110|0)==(0);
 if ($111) {
  $112 = HEAP32[((39000 + 24|0))>>2]|0;
  $113 = ($0|0)==($112|0);
  if ($113) {
   $114 = HEAP32[((39000 + 12|0))>>2]|0;
   $115 = (($114) + ($$02))|0;
   HEAP32[((39000 + 12|0))>>2] = $115;
   HEAP32[((39000 + 24|0))>>2] = $$0;
   $116 = $115 | 1;
   $117 = (($$0) + 4|0);
   HEAP32[$117>>2] = $116;
   $118 = HEAP32[((39000 + 20|0))>>2]|0;
   $119 = ($$0|0)==($118|0);
   if (!($119)) {
    STACKTOP = sp;return;
   }
   HEAP32[((39000 + 20|0))>>2] = 0;
   HEAP32[((39000 + 8|0))>>2] = 0;
   STACKTOP = sp;return;
  }
  $120 = HEAP32[((39000 + 20|0))>>2]|0;
  $121 = ($0|0)==($120|0);
  if ($121) {
   $122 = HEAP32[((39000 + 8|0))>>2]|0;
   $123 = (($122) + ($$02))|0;
   HEAP32[((39000 + 8|0))>>2] = $123;
   HEAP32[((39000 + 20|0))>>2] = $$0;
   $124 = $123 | 1;
   $125 = (($$0) + 4|0);
   HEAP32[$125>>2] = $124;
   $126 = (($$0) + ($123)|0);
   HEAP32[$126>>2] = $123;
   STACKTOP = sp;return;
  }
  $127 = $109 & -8;
  $128 = (($127) + ($$02))|0;
  $129 = $109 >>> 3;
  $130 = ($109>>>0)<(256);
  do {
   if ($130) {
    $$sum18 = (($psize) + 8)|0;
    $131 = (($p) + ($$sum18)|0);
    $132 = HEAP32[$131>>2]|0;
    $$sum19 = (($psize) + 12)|0;
    $133 = (($p) + ($$sum19)|0);
    $134 = HEAP32[$133>>2]|0;
    $135 = $129 << 1;
    $136 = ((39000 + ($135<<2)|0) + 40|0);
    $137 = ($132|0)==($136|0);
    if (!($137)) {
     $138 = ($132>>>0)<($106>>>0);
     if ($138) {
      _abort();
      // unreachable;
     }
     $139 = (($132) + 12|0);
     $140 = HEAP32[$139>>2]|0;
     $141 = ($140|0)==($0|0);
     if (!($141)) {
      _abort();
      // unreachable;
     }
    }
    $142 = ($134|0)==($132|0);
    if ($142) {
     $143 = 1 << $129;
     $144 = $143 ^ -1;
     $145 = HEAP32[39000>>2]|0;
     $146 = $145 & $144;
     HEAP32[39000>>2] = $146;
     break;
    }
    $147 = ($134|0)==($136|0);
    if ($147) {
     $$pre56 = (($134) + 8|0);
     $$pre$phi57Z2D = $$pre56;
    } else {
     $148 = ($134>>>0)<($106>>>0);
     if ($148) {
      _abort();
      // unreachable;
     }
     $149 = (($134) + 8|0);
     $150 = HEAP32[$149>>2]|0;
     $151 = ($150|0)==($0|0);
     if ($151) {
      $$pre$phi57Z2D = $149;
     } else {
      _abort();
      // unreachable;
     }
    }
    $152 = (($132) + 12|0);
    HEAP32[$152>>2] = $134;
    HEAP32[$$pre$phi57Z2D>>2] = $132;
   } else {
    $$sum2 = (($psize) + 24)|0;
    $153 = (($p) + ($$sum2)|0);
    $154 = HEAP32[$153>>2]|0;
    $$sum3 = (($psize) + 12)|0;
    $155 = (($p) + ($$sum3)|0);
    $156 = HEAP32[$155>>2]|0;
    $157 = ($156|0)==($0|0);
    do {
     if ($157) {
      $$sum5 = (($psize) + 20)|0;
      $167 = (($p) + ($$sum5)|0);
      $168 = HEAP32[$167>>2]|0;
      $169 = ($168|0)==(0|0);
      if ($169) {
       $$sum4 = (($psize) + 16)|0;
       $170 = (($p) + ($$sum4)|0);
       $171 = HEAP32[$170>>2]|0;
       $172 = ($171|0)==(0|0);
       if ($172) {
        $R7$1 = 0;
        break;
       } else {
        $R7$0 = $171;$RP9$0 = $170;
       }
      } else {
       $R7$0 = $168;$RP9$0 = $167;
      }
      while(1) {
       $173 = (($R7$0) + 20|0);
       $174 = HEAP32[$173>>2]|0;
       $175 = ($174|0)==(0|0);
       if (!($175)) {
        $R7$0 = $174;$RP9$0 = $173;
        continue;
       }
       $176 = (($R7$0) + 16|0);
       $177 = HEAP32[$176>>2]|0;
       $178 = ($177|0)==(0|0);
       if ($178) {
        break;
       } else {
        $R7$0 = $177;$RP9$0 = $176;
       }
      }
      $179 = ($RP9$0>>>0)<($106>>>0);
      if ($179) {
       _abort();
       // unreachable;
      } else {
       HEAP32[$RP9$0>>2] = 0;
       $R7$1 = $R7$0;
       break;
      }
     } else {
      $$sum17 = (($psize) + 8)|0;
      $158 = (($p) + ($$sum17)|0);
      $159 = HEAP32[$158>>2]|0;
      $160 = ($159>>>0)<($106>>>0);
      if ($160) {
       _abort();
       // unreachable;
      }
      $161 = (($159) + 12|0);
      $162 = HEAP32[$161>>2]|0;
      $163 = ($162|0)==($0|0);
      if (!($163)) {
       _abort();
       // unreachable;
      }
      $164 = (($156) + 8|0);
      $165 = HEAP32[$164>>2]|0;
      $166 = ($165|0)==($0|0);
      if ($166) {
       HEAP32[$161>>2] = $156;
       HEAP32[$164>>2] = $159;
       $R7$1 = $156;
       break;
      } else {
       _abort();
       // unreachable;
      }
     }
    } while(0);
    $180 = ($154|0)==(0|0);
    if (!($180)) {
     $$sum14 = (($psize) + 28)|0;
     $181 = (($p) + ($$sum14)|0);
     $182 = HEAP32[$181>>2]|0;
     $183 = ((39000 + ($182<<2)|0) + 304|0);
     $184 = HEAP32[$183>>2]|0;
     $185 = ($0|0)==($184|0);
     if ($185) {
      HEAP32[$183>>2] = $R7$1;
      $cond46 = ($R7$1|0)==(0|0);
      if ($cond46) {
       $186 = 1 << $182;
       $187 = $186 ^ -1;
       $188 = HEAP32[((39000 + 4|0))>>2]|0;
       $189 = $188 & $187;
       HEAP32[((39000 + 4|0))>>2] = $189;
       break;
      }
     } else {
      $190 = HEAP32[((39000 + 16|0))>>2]|0;
      $191 = ($154>>>0)<($190>>>0);
      if ($191) {
       _abort();
       // unreachable;
      }
      $192 = (($154) + 16|0);
      $193 = HEAP32[$192>>2]|0;
      $194 = ($193|0)==($0|0);
      if ($194) {
       HEAP32[$192>>2] = $R7$1;
      } else {
       $195 = (($154) + 20|0);
       HEAP32[$195>>2] = $R7$1;
      }
      $196 = ($R7$1|0)==(0|0);
      if ($196) {
       break;
      }
     }
     $197 = HEAP32[((39000 + 16|0))>>2]|0;
     $198 = ($R7$1>>>0)<($197>>>0);
     if ($198) {
      _abort();
      // unreachable;
     }
     $199 = (($R7$1) + 24|0);
     HEAP32[$199>>2] = $154;
     $$sum15 = (($psize) + 16)|0;
     $200 = (($p) + ($$sum15)|0);
     $201 = HEAP32[$200>>2]|0;
     $202 = ($201|0)==(0|0);
     do {
      if (!($202)) {
       $203 = ($201>>>0)<($197>>>0);
       if ($203) {
        _abort();
        // unreachable;
       } else {
        $204 = (($R7$1) + 16|0);
        HEAP32[$204>>2] = $201;
        $205 = (($201) + 24|0);
        HEAP32[$205>>2] = $R7$1;
        break;
       }
      }
     } while(0);
     $$sum16 = (($psize) + 20)|0;
     $206 = (($p) + ($$sum16)|0);
     $207 = HEAP32[$206>>2]|0;
     $208 = ($207|0)==(0|0);
     if (!($208)) {
      $209 = HEAP32[((39000 + 16|0))>>2]|0;
      $210 = ($207>>>0)<($209>>>0);
      if ($210) {
       _abort();
       // unreachable;
      } else {
       $211 = (($R7$1) + 20|0);
       HEAP32[$211>>2] = $207;
       $212 = (($207) + 24|0);
       HEAP32[$212>>2] = $R7$1;
       break;
      }
     }
    }
   }
  } while(0);
  $213 = $128 | 1;
  $214 = (($$0) + 4|0);
  HEAP32[$214>>2] = $213;
  $215 = (($$0) + ($128)|0);
  HEAP32[$215>>2] = $128;
  $216 = HEAP32[((39000 + 20|0))>>2]|0;
  $217 = ($$0|0)==($216|0);
  if ($217) {
   HEAP32[((39000 + 8|0))>>2] = $128;
   STACKTOP = sp;return;
  } else {
   $$1 = $128;
  }
 } else {
  $218 = $109 & -2;
  HEAP32[$108>>2] = $218;
  $219 = $$02 | 1;
  $220 = (($$0) + 4|0);
  HEAP32[$220>>2] = $219;
  $221 = (($$0) + ($$02)|0);
  HEAP32[$221>>2] = $$02;
  $$1 = $$02;
 }
 $222 = $$1 >>> 3;
 $223 = ($$1>>>0)<(256);
 if ($223) {
  $224 = $222 << 1;
  $225 = ((39000 + ($224<<2)|0) + 40|0);
  $226 = HEAP32[39000>>2]|0;
  $227 = 1 << $222;
  $228 = $226 & $227;
  $229 = ($228|0)==(0);
  if ($229) {
   $230 = $226 | $227;
   HEAP32[39000>>2] = $230;
   $$sum12$pre = (($224) + 2)|0;
   $$pre = ((39000 + ($$sum12$pre<<2)|0) + 40|0);
   $$pre$phiZ2D = $$pre;$F16$0 = $225;
  } else {
   $$sum13 = (($224) + 2)|0;
   $231 = ((39000 + ($$sum13<<2)|0) + 40|0);
   $232 = HEAP32[$231>>2]|0;
   $233 = HEAP32[((39000 + 16|0))>>2]|0;
   $234 = ($232>>>0)<($233>>>0);
   if ($234) {
    _abort();
    // unreachable;
   } else {
    $$pre$phiZ2D = $231;$F16$0 = $232;
   }
  }
  HEAP32[$$pre$phiZ2D>>2] = $$0;
  $235 = (($F16$0) + 12|0);
  HEAP32[$235>>2] = $$0;
  $236 = (($$0) + 8|0);
  HEAP32[$236>>2] = $F16$0;
  $237 = (($$0) + 12|0);
  HEAP32[$237>>2] = $225;
  STACKTOP = sp;return;
 }
 $238 = $$1 >>> 8;
 $239 = ($238|0)==(0);
 if ($239) {
  $I19$0 = 0;
 } else {
  $240 = ($$1>>>0)>(16777215);
  if ($240) {
   $I19$0 = 31;
  } else {
   $241 = (($238) + 1048320)|0;
   $242 = $241 >>> 16;
   $243 = $242 & 8;
   $244 = $238 << $243;
   $245 = (($244) + 520192)|0;
   $246 = $245 >>> 16;
   $247 = $246 & 4;
   $248 = $247 | $243;
   $249 = $244 << $247;
   $250 = (($249) + 245760)|0;
   $251 = $250 >>> 16;
   $252 = $251 & 2;
   $253 = $248 | $252;
   $254 = (14 - ($253))|0;
   $255 = $249 << $252;
   $256 = $255 >>> 15;
   $257 = (($254) + ($256))|0;
   $258 = $257 << 1;
   $259 = (($257) + 7)|0;
   $260 = $$1 >>> $259;
   $261 = $260 & 1;
   $262 = $261 | $258;
   $I19$0 = $262;
  }
 }
 $263 = ((39000 + ($I19$0<<2)|0) + 304|0);
 $264 = (($$0) + 28|0);
 $I19$0$c = $I19$0;
 HEAP32[$264>>2] = $I19$0$c;
 $265 = (($$0) + 20|0);
 HEAP32[$265>>2] = 0;
 $266 = (($$0) + 16|0);
 HEAP32[$266>>2] = 0;
 $267 = HEAP32[((39000 + 4|0))>>2]|0;
 $268 = 1 << $I19$0;
 $269 = $267 & $268;
 $270 = ($269|0)==(0);
 if ($270) {
  $271 = $267 | $268;
  HEAP32[((39000 + 4|0))>>2] = $271;
  HEAP32[$263>>2] = $$0;
  $272 = (($$0) + 24|0);
  HEAP32[$272>>2] = $263;
  $273 = (($$0) + 12|0);
  HEAP32[$273>>2] = $$0;
  $274 = (($$0) + 8|0);
  HEAP32[$274>>2] = $$0;
  STACKTOP = sp;return;
 }
 $275 = HEAP32[$263>>2]|0;
 $276 = ($I19$0|0)==(31);
 if ($276) {
  $284 = 0;
 } else {
  $277 = $I19$0 >>> 1;
  $278 = (25 - ($277))|0;
  $284 = $278;
 }
 $279 = (($275) + 4|0);
 $280 = HEAP32[$279>>2]|0;
 $281 = $280 & -8;
 $282 = ($281|0)==($$1|0);
 L194: do {
  if ($282) {
   $T$0$lcssa = $275;
  } else {
   $283 = $$1 << $284;
   $K20$050 = $283;$T$049 = $275;
   while(1) {
    $291 = $K20$050 >>> 31;
    $292 = ((($T$049) + ($291<<2)|0) + 16|0);
    $287 = HEAP32[$292>>2]|0;
    $293 = ($287|0)==(0|0);
    if ($293) {
     break;
    }
    $285 = $K20$050 << 1;
    $286 = (($287) + 4|0);
    $288 = HEAP32[$286>>2]|0;
    $289 = $288 & -8;
    $290 = ($289|0)==($$1|0);
    if ($290) {
     $T$0$lcssa = $287;
     break L194;
    } else {
     $K20$050 = $285;$T$049 = $287;
    }
   }
   $294 = HEAP32[((39000 + 16|0))>>2]|0;
   $295 = ($292>>>0)<($294>>>0);
   if ($295) {
    _abort();
    // unreachable;
   }
   HEAP32[$292>>2] = $$0;
   $296 = (($$0) + 24|0);
   HEAP32[$296>>2] = $T$049;
   $297 = (($$0) + 12|0);
   HEAP32[$297>>2] = $$0;
   $298 = (($$0) + 8|0);
   HEAP32[$298>>2] = $$0;
   STACKTOP = sp;return;
  }
 } while(0);
 $299 = (($T$0$lcssa) + 8|0);
 $300 = HEAP32[$299>>2]|0;
 $301 = HEAP32[((39000 + 16|0))>>2]|0;
 $302 = ($T$0$lcssa>>>0)>=($301>>>0);
 $303 = ($300>>>0)>=($301>>>0);
 $or$cond = $302 & $303;
 if (!($or$cond)) {
  _abort();
  // unreachable;
 }
 $304 = (($300) + 12|0);
 HEAP32[$304>>2] = $$0;
 HEAP32[$299>>2] = $$0;
 $305 = (($$0) + 8|0);
 HEAP32[$305>>2] = $300;
 $306 = (($$0) + 12|0);
 HEAP32[$306>>2] = $T$0$lcssa;
 $307 = (($$0) + 24|0);
 HEAP32[$307>>2] = 0;
 STACKTOP = sp;return;
}
function runPostSets() {
 
}
function _i64Subtract(a, b, c, d) {
    a = a|0; b = b|0; c = c|0; d = d|0;
    var l = 0, h = 0;
    l = (a - c)>>>0;
    h = (b - d)>>>0;
    h = (b - d - (((c>>>0) > (a>>>0))|0))>>>0; // Borrow one from high word to low word on underflow.
    return ((tempRet0 = h,l|0)|0);
}
function _memset(ptr, value, num) {
    ptr = ptr|0; value = value|0; num = num|0;
    var stop = 0, value4 = 0, stop4 = 0, unaligned = 0;
    stop = (ptr + num)|0;
    if ((num|0) >= 20) {
      // This is unaligned, but quite large, so work hard to get to aligned settings
      value = value & 0xff;
      unaligned = ptr & 3;
      value4 = value | (value << 8) | (value << 16) | (value << 24);
      stop4 = stop & ~3;
      if (unaligned) {
        unaligned = (ptr + 4 - unaligned)|0;
        while ((ptr|0) < (unaligned|0)) { // no need to check for stop, since we have large num
          HEAP8[((ptr)>>0)]=value;
          ptr = (ptr+1)|0;
        }
      }
      while ((ptr|0) < (stop4|0)) {
        HEAP32[((ptr)>>2)]=value4;
        ptr = (ptr+4)|0;
      }
    }
    while ((ptr|0) < (stop|0)) {
      HEAP8[((ptr)>>0)]=value;
      ptr = (ptr+1)|0;
    }
    return (ptr-num)|0;
}
function _bitshift64Shl(low, high, bits) {
    low = low|0; high = high|0; bits = bits|0;
    var ander = 0;
    if ((bits|0) < 32) {
      ander = ((1 << bits) - 1)|0;
      tempRet0 = (high << bits) | ((low&(ander << (32 - bits))) >>> (32 - bits));
      return low << bits;
    }
    tempRet0 = low << (bits - 32);
    return 0;
}
function _strlen(ptr) {
    ptr = ptr|0;
    var curr = 0;
    curr = ptr;
    while (((HEAP8[((curr)>>0)])|0)) {
      curr = (curr + 1)|0;
    }
    return (curr - ptr)|0;
}
function _i64Add(a, b, c, d) {
    /*
      x = a + b*2^32
      y = c + d*2^32
      result = l + h*2^32
    */
    a = a|0; b = b|0; c = c|0; d = d|0;
    var l = 0, h = 0;
    l = (a + c)>>>0;
    h = (b + d + (((l>>>0) < (a>>>0))|0))>>>0; // Add carry from low word to high word on overflow.
    return ((tempRet0 = h,l|0)|0);
}
function _bitshift64Lshr(low, high, bits) {
    low = low|0; high = high|0; bits = bits|0;
    var ander = 0;
    if ((bits|0) < 32) {
      ander = ((1 << bits) - 1)|0;
      tempRet0 = high >>> bits;
      return (low >>> bits) | ((high&ander) << (32 - bits));
    }
    tempRet0 = 0;
    return (high >>> (bits - 32))|0;
}
function _memcpy(dest, src, num) {

    dest = dest|0; src = src|0; num = num|0;
    var ret = 0;
    if ((num|0) >= 4096) return _emscripten_memcpy_big(dest|0, src|0, num|0)|0;
    ret = dest|0;
    if ((dest&3) == (src&3)) {
      while (dest & 3) {
        if ((num|0) == 0) return ret|0;
        HEAP8[((dest)>>0)]=((HEAP8[((src)>>0)])|0);
        dest = (dest+1)|0;
        src = (src+1)|0;
        num = (num-1)|0;
      }
      while ((num|0) >= 4) {
        HEAP32[((dest)>>2)]=((HEAP32[((src)>>2)])|0);
        dest = (dest+4)|0;
        src = (src+4)|0;
        num = (num-4)|0;
      }
    }
    while ((num|0) > 0) {
      HEAP8[((dest)>>0)]=((HEAP8[((src)>>0)])|0);
      dest = (dest+1)|0;
      src = (src+1)|0;
      num = (num-1)|0;
    }
    return ret|0;
}
function _memmove(dest, src, num) {
    dest = dest|0; src = src|0; num = num|0;
    var ret = 0;
    if (((src|0) < (dest|0)) & ((dest|0) < ((src + num)|0))) {
      // Unlikely case: Copy backwards in a safe manner
      ret = dest;
      src = (src + num)|0;
      dest = (dest + num)|0;
      while ((num|0) > 0) {
        dest = (dest - 1)|0;
        src = (src - 1)|0;
        num = (num - 1)|0;
        HEAP8[((dest)>>0)]=((HEAP8[((src)>>0)])|0);
      }
      dest = ret;
    } else {
      _memcpy(dest, src, num) | 0;
    }
    return dest | 0;
}
function _bitshift64Ashr(low, high, bits) {
    low = low|0; high = high|0; bits = bits|0;
    var ander = 0;
    if ((bits|0) < 32) {
      ander = ((1 << bits) - 1)|0;
      tempRet0 = high >> bits;
      return (low >>> bits) | ((high&ander) << (32 - bits));
    }
    tempRet0 = (high|0) < 0 ? -1 : 0;
    return (high >> (bits - 32))|0;
  }
function _llvm_ctlz_i32(x) {
    x = x|0;
    var ret = 0;
    ret = ((HEAP8[(((ctlz_i8)+(x >>> 24))>>0)])|0);
    if ((ret|0) < 8) return ret|0;
    ret = ((HEAP8[(((ctlz_i8)+((x >> 16)&0xff))>>0)])|0);
    if ((ret|0) < 8) return (ret + 8)|0;
    ret = ((HEAP8[(((ctlz_i8)+((x >> 8)&0xff))>>0)])|0);
    if ((ret|0) < 8) return (ret + 16)|0;
    return (((HEAP8[(((ctlz_i8)+(x&0xff))>>0)])|0) + 24)|0;
  }

function _llvm_cttz_i32(x) {
    x = x|0;
    var ret = 0;
    ret = ((HEAP8[(((cttz_i8)+(x & 0xff))>>0)])|0);
    if ((ret|0) < 8) return ret|0;
    ret = ((HEAP8[(((cttz_i8)+((x >> 8)&0xff))>>0)])|0);
    if ((ret|0) < 8) return (ret + 8)|0;
    ret = ((HEAP8[(((cttz_i8)+((x >> 16)&0xff))>>0)])|0);
    if ((ret|0) < 8) return (ret + 16)|0;
    return (((HEAP8[(((cttz_i8)+(x >>> 24))>>0)])|0) + 24)|0;
  }

// ======== compiled code from system/lib/compiler-rt , see readme therein
function ___muldsi3($a, $b) {
  $a = $a | 0;
  $b = $b | 0;
  var $1 = 0, $2 = 0, $3 = 0, $6 = 0, $8 = 0, $11 = 0, $12 = 0;
  $1 = $a & 65535;
  $2 = $b & 65535;
  $3 = Math_imul($2, $1) | 0;
  $6 = $a >>> 16;
  $8 = ($3 >>> 16) + (Math_imul($2, $6) | 0) | 0;
  $11 = $b >>> 16;
  $12 = Math_imul($11, $1) | 0;
  return (tempRet0 = (($8 >>> 16) + (Math_imul($11, $6) | 0) | 0) + ((($8 & 65535) + $12 | 0) >>> 16) | 0, 0 | ($8 + $12 << 16 | $3 & 65535)) | 0;
}
function ___divdi3($a$0, $a$1, $b$0, $b$1) {
  $a$0 = $a$0 | 0;
  $a$1 = $a$1 | 0;
  $b$0 = $b$0 | 0;
  $b$1 = $b$1 | 0;
  var $1$0 = 0, $1$1 = 0, $2$0 = 0, $2$1 = 0, $4$0 = 0, $4$1 = 0, $6$0 = 0, $7$0 = 0, $7$1 = 0, $8$0 = 0, $10$0 = 0;
  $1$0 = $a$1 >> 31 | (($a$1 | 0) < 0 ? -1 : 0) << 1;
  $1$1 = (($a$1 | 0) < 0 ? -1 : 0) >> 31 | (($a$1 | 0) < 0 ? -1 : 0) << 1;
  $2$0 = $b$1 >> 31 | (($b$1 | 0) < 0 ? -1 : 0) << 1;
  $2$1 = (($b$1 | 0) < 0 ? -1 : 0) >> 31 | (($b$1 | 0) < 0 ? -1 : 0) << 1;
  $4$0 = _i64Subtract($1$0 ^ $a$0, $1$1 ^ $a$1, $1$0, $1$1) | 0;
  $4$1 = tempRet0;
  $6$0 = _i64Subtract($2$0 ^ $b$0, $2$1 ^ $b$1, $2$0, $2$1) | 0;
  $7$0 = $2$0 ^ $1$0;
  $7$1 = $2$1 ^ $1$1;
  $8$0 = ___udivmoddi4($4$0, $4$1, $6$0, tempRet0, 0) | 0;
  $10$0 = _i64Subtract($8$0 ^ $7$0, tempRet0 ^ $7$1, $7$0, $7$1) | 0;
  return (tempRet0 = tempRet0, $10$0) | 0;
}
function ___remdi3($a$0, $a$1, $b$0, $b$1) {
  $a$0 = $a$0 | 0;
  $a$1 = $a$1 | 0;
  $b$0 = $b$0 | 0;
  $b$1 = $b$1 | 0;
  var $rem = 0, $1$0 = 0, $1$1 = 0, $2$0 = 0, $2$1 = 0, $4$0 = 0, $4$1 = 0, $6$0 = 0, $10$0 = 0, $10$1 = 0, __stackBase__ = 0;
  __stackBase__ = STACKTOP;
  STACKTOP = STACKTOP + 8 | 0;
  $rem = __stackBase__ | 0;
  $1$0 = $a$1 >> 31 | (($a$1 | 0) < 0 ? -1 : 0) << 1;
  $1$1 = (($a$1 | 0) < 0 ? -1 : 0) >> 31 | (($a$1 | 0) < 0 ? -1 : 0) << 1;
  $2$0 = $b$1 >> 31 | (($b$1 | 0) < 0 ? -1 : 0) << 1;
  $2$1 = (($b$1 | 0) < 0 ? -1 : 0) >> 31 | (($b$1 | 0) < 0 ? -1 : 0) << 1;
  $4$0 = _i64Subtract($1$0 ^ $a$0, $1$1 ^ $a$1, $1$0, $1$1) | 0;
  $4$1 = tempRet0;
  $6$0 = _i64Subtract($2$0 ^ $b$0, $2$1 ^ $b$1, $2$0, $2$1) | 0;
  ___udivmoddi4($4$0, $4$1, $6$0, tempRet0, $rem) | 0;
  $10$0 = _i64Subtract(HEAP32[$rem >> 2] ^ $1$0, HEAP32[$rem + 4 >> 2] ^ $1$1, $1$0, $1$1) | 0;
  $10$1 = tempRet0;
  STACKTOP = __stackBase__;
  return (tempRet0 = $10$1, $10$0) | 0;
}
function ___muldi3($a$0, $a$1, $b$0, $b$1) {
  $a$0 = $a$0 | 0;
  $a$1 = $a$1 | 0;
  $b$0 = $b$0 | 0;
  $b$1 = $b$1 | 0;
  var $x_sroa_0_0_extract_trunc = 0, $y_sroa_0_0_extract_trunc = 0, $1$0 = 0, $1$1 = 0, $2 = 0;
  $x_sroa_0_0_extract_trunc = $a$0;
  $y_sroa_0_0_extract_trunc = $b$0;
  $1$0 = ___muldsi3($x_sroa_0_0_extract_trunc, $y_sroa_0_0_extract_trunc) | 0;
  $1$1 = tempRet0;
  $2 = Math_imul($a$1, $y_sroa_0_0_extract_trunc) | 0;
  return (tempRet0 = ((Math_imul($b$1, $x_sroa_0_0_extract_trunc) | 0) + $2 | 0) + $1$1 | $1$1 & 0, 0 | $1$0 & -1) | 0;
}
function ___udivdi3($a$0, $a$1, $b$0, $b$1) {
  $a$0 = $a$0 | 0;
  $a$1 = $a$1 | 0;
  $b$0 = $b$0 | 0;
  $b$1 = $b$1 | 0;
  var $1$0 = 0;
  $1$0 = ___udivmoddi4($a$0, $a$1, $b$0, $b$1, 0) | 0;
  return (tempRet0 = tempRet0, $1$0) | 0;
}
function ___uremdi3($a$0, $a$1, $b$0, $b$1) {
  $a$0 = $a$0 | 0;
  $a$1 = $a$1 | 0;
  $b$0 = $b$0 | 0;
  $b$1 = $b$1 | 0;
  var $rem = 0, __stackBase__ = 0;
  __stackBase__ = STACKTOP;
  STACKTOP = STACKTOP + 8 | 0;
  $rem = __stackBase__ | 0;
  ___udivmoddi4($a$0, $a$1, $b$0, $b$1, $rem) | 0;
  STACKTOP = __stackBase__;
  return (tempRet0 = HEAP32[$rem + 4 >> 2] | 0, HEAP32[$rem >> 2] | 0) | 0;
}
function ___udivmoddi4($a$0, $a$1, $b$0, $b$1, $rem) {
  $a$0 = $a$0 | 0;
  $a$1 = $a$1 | 0;
  $b$0 = $b$0 | 0;
  $b$1 = $b$1 | 0;
  $rem = $rem | 0;
  var $n_sroa_0_0_extract_trunc = 0, $n_sroa_1_4_extract_shift$0 = 0, $n_sroa_1_4_extract_trunc = 0, $d_sroa_0_0_extract_trunc = 0, $d_sroa_1_4_extract_shift$0 = 0, $d_sroa_1_4_extract_trunc = 0, $4 = 0, $17 = 0, $37 = 0, $49 = 0, $51 = 0, $57 = 0, $58 = 0, $66 = 0, $78 = 0, $86 = 0, $88 = 0, $89 = 0, $91 = 0, $92 = 0, $95 = 0, $105 = 0, $117 = 0, $119 = 0, $125 = 0, $126 = 0, $130 = 0, $q_sroa_1_1_ph = 0, $q_sroa_0_1_ph = 0, $r_sroa_1_1_ph = 0, $r_sroa_0_1_ph = 0, $sr_1_ph = 0, $d_sroa_0_0_insert_insert99$0 = 0, $d_sroa_0_0_insert_insert99$1 = 0, $137$0 = 0, $137$1 = 0, $carry_0203 = 0, $sr_1202 = 0, $r_sroa_0_1201 = 0, $r_sroa_1_1200 = 0, $q_sroa_0_1199 = 0, $q_sroa_1_1198 = 0, $147 = 0, $149 = 0, $r_sroa_0_0_insert_insert42$0 = 0, $r_sroa_0_0_insert_insert42$1 = 0, $150$1 = 0, $151$0 = 0, $152 = 0, $154$0 = 0, $r_sroa_0_0_extract_trunc = 0, $r_sroa_1_4_extract_trunc = 0, $155 = 0, $carry_0_lcssa$0 = 0, $carry_0_lcssa$1 = 0, $r_sroa_0_1_lcssa = 0, $r_sroa_1_1_lcssa = 0, $q_sroa_0_1_lcssa = 0, $q_sroa_1_1_lcssa = 0, $q_sroa_0_0_insert_ext75$0 = 0, $q_sroa_0_0_insert_ext75$1 = 0, $q_sroa_0_0_insert_insert77$1 = 0, $_0$0 = 0, $_0$1 = 0;
  $n_sroa_0_0_extract_trunc = $a$0;
  $n_sroa_1_4_extract_shift$0 = $a$1;
  $n_sroa_1_4_extract_trunc = $n_sroa_1_4_extract_shift$0;
  $d_sroa_0_0_extract_trunc = $b$0;
  $d_sroa_1_4_extract_shift$0 = $b$1;
  $d_sroa_1_4_extract_trunc = $d_sroa_1_4_extract_shift$0;
  if (($n_sroa_1_4_extract_trunc | 0) == 0) {
    $4 = ($rem | 0) != 0;
    if (($d_sroa_1_4_extract_trunc | 0) == 0) {
      if ($4) {
        HEAP32[$rem >> 2] = ($n_sroa_0_0_extract_trunc >>> 0) % ($d_sroa_0_0_extract_trunc >>> 0);
        HEAP32[$rem + 4 >> 2] = 0;
      }
      $_0$1 = 0;
      $_0$0 = ($n_sroa_0_0_extract_trunc >>> 0) / ($d_sroa_0_0_extract_trunc >>> 0) >>> 0;
      return (tempRet0 = $_0$1, $_0$0) | 0;
    } else {
      if (!$4) {
        $_0$1 = 0;
        $_0$0 = 0;
        return (tempRet0 = $_0$1, $_0$0) | 0;
      }
      HEAP32[$rem >> 2] = $a$0 & -1;
      HEAP32[$rem + 4 >> 2] = $a$1 & 0;
      $_0$1 = 0;
      $_0$0 = 0;
      return (tempRet0 = $_0$1, $_0$0) | 0;
    }
  }
  $17 = ($d_sroa_1_4_extract_trunc | 0) == 0;
  do {
    if (($d_sroa_0_0_extract_trunc | 0) == 0) {
      if ($17) {
        if (($rem | 0) != 0) {
          HEAP32[$rem >> 2] = ($n_sroa_1_4_extract_trunc >>> 0) % ($d_sroa_0_0_extract_trunc >>> 0);
          HEAP32[$rem + 4 >> 2] = 0;
        }
        $_0$1 = 0;
        $_0$0 = ($n_sroa_1_4_extract_trunc >>> 0) / ($d_sroa_0_0_extract_trunc >>> 0) >>> 0;
        return (tempRet0 = $_0$1, $_0$0) | 0;
      }
      if (($n_sroa_0_0_extract_trunc | 0) == 0) {
        if (($rem | 0) != 0) {
          HEAP32[$rem >> 2] = 0;
          HEAP32[$rem + 4 >> 2] = ($n_sroa_1_4_extract_trunc >>> 0) % ($d_sroa_1_4_extract_trunc >>> 0);
        }
        $_0$1 = 0;
        $_0$0 = ($n_sroa_1_4_extract_trunc >>> 0) / ($d_sroa_1_4_extract_trunc >>> 0) >>> 0;
        return (tempRet0 = $_0$1, $_0$0) | 0;
      }
      $37 = $d_sroa_1_4_extract_trunc - 1 | 0;
      if (($37 & $d_sroa_1_4_extract_trunc | 0) == 0) {
        if (($rem | 0) != 0) {
          HEAP32[$rem >> 2] = 0 | $a$0 & -1;
          HEAP32[$rem + 4 >> 2] = $37 & $n_sroa_1_4_extract_trunc | $a$1 & 0;
        }
        $_0$1 = 0;
        $_0$0 = $n_sroa_1_4_extract_trunc >>> ((_llvm_cttz_i32($d_sroa_1_4_extract_trunc | 0) | 0) >>> 0);
        return (tempRet0 = $_0$1, $_0$0) | 0;
      }
      $49 = _llvm_ctlz_i32($d_sroa_1_4_extract_trunc | 0) | 0;
      $51 = $49 - (_llvm_ctlz_i32($n_sroa_1_4_extract_trunc | 0) | 0) | 0;
      if ($51 >>> 0 <= 30) {
        $57 = $51 + 1 | 0;
        $58 = 31 - $51 | 0;
        $sr_1_ph = $57;
        $r_sroa_0_1_ph = $n_sroa_1_4_extract_trunc << $58 | $n_sroa_0_0_extract_trunc >>> ($57 >>> 0);
        $r_sroa_1_1_ph = $n_sroa_1_4_extract_trunc >>> ($57 >>> 0);
        $q_sroa_0_1_ph = 0;
        $q_sroa_1_1_ph = $n_sroa_0_0_extract_trunc << $58;
        break;
      }
      if (($rem | 0) == 0) {
        $_0$1 = 0;
        $_0$0 = 0;
        return (tempRet0 = $_0$1, $_0$0) | 0;
      }
      HEAP32[$rem >> 2] = 0 | $a$0 & -1;
      HEAP32[$rem + 4 >> 2] = $n_sroa_1_4_extract_shift$0 | $a$1 & 0;
      $_0$1 = 0;
      $_0$0 = 0;
      return (tempRet0 = $_0$1, $_0$0) | 0;
    } else {
      if (!$17) {
        $117 = _llvm_ctlz_i32($d_sroa_1_4_extract_trunc | 0) | 0;
        $119 = $117 - (_llvm_ctlz_i32($n_sroa_1_4_extract_trunc | 0) | 0) | 0;
        if ($119 >>> 0 <= 31) {
          $125 = $119 + 1 | 0;
          $126 = 31 - $119 | 0;
          $130 = $119 - 31 >> 31;
          $sr_1_ph = $125;
          $r_sroa_0_1_ph = $n_sroa_0_0_extract_trunc >>> ($125 >>> 0) & $130 | $n_sroa_1_4_extract_trunc << $126;
          $r_sroa_1_1_ph = $n_sroa_1_4_extract_trunc >>> ($125 >>> 0) & $130;
          $q_sroa_0_1_ph = 0;
          $q_sroa_1_1_ph = $n_sroa_0_0_extract_trunc << $126;
          break;
        }
        if (($rem | 0) == 0) {
          $_0$1 = 0;
          $_0$0 = 0;
          return (tempRet0 = $_0$1, $_0$0) | 0;
        }
        HEAP32[$rem >> 2] = 0 | $a$0 & -1;
        HEAP32[$rem + 4 >> 2] = $n_sroa_1_4_extract_shift$0 | $a$1 & 0;
        $_0$1 = 0;
        $_0$0 = 0;
        return (tempRet0 = $_0$1, $_0$0) | 0;
      }
      $66 = $d_sroa_0_0_extract_trunc - 1 | 0;
      if (($66 & $d_sroa_0_0_extract_trunc | 0) != 0) {
        $86 = (_llvm_ctlz_i32($d_sroa_0_0_extract_trunc | 0) | 0) + 33 | 0;
        $88 = $86 - (_llvm_ctlz_i32($n_sroa_1_4_extract_trunc | 0) | 0) | 0;
        $89 = 64 - $88 | 0;
        $91 = 32 - $88 | 0;
        $92 = $91 >> 31;
        $95 = $88 - 32 | 0;
        $105 = $95 >> 31;
        $sr_1_ph = $88;
        $r_sroa_0_1_ph = $91 - 1 >> 31 & $n_sroa_1_4_extract_trunc >>> ($95 >>> 0) | ($n_sroa_1_4_extract_trunc << $91 | $n_sroa_0_0_extract_trunc >>> ($88 >>> 0)) & $105;
        $r_sroa_1_1_ph = $105 & $n_sroa_1_4_extract_trunc >>> ($88 >>> 0);
        $q_sroa_0_1_ph = $n_sroa_0_0_extract_trunc << $89 & $92;
        $q_sroa_1_1_ph = ($n_sroa_1_4_extract_trunc << $89 | $n_sroa_0_0_extract_trunc >>> ($95 >>> 0)) & $92 | $n_sroa_0_0_extract_trunc << $91 & $88 - 33 >> 31;
        break;
      }
      if (($rem | 0) != 0) {
        HEAP32[$rem >> 2] = $66 & $n_sroa_0_0_extract_trunc;
        HEAP32[$rem + 4 >> 2] = 0;
      }
      if (($d_sroa_0_0_extract_trunc | 0) == 1) {
        $_0$1 = $n_sroa_1_4_extract_shift$0 | $a$1 & 0;
        $_0$0 = 0 | $a$0 & -1;
        return (tempRet0 = $_0$1, $_0$0) | 0;
      } else {
        $78 = _llvm_cttz_i32($d_sroa_0_0_extract_trunc | 0) | 0;
        $_0$1 = 0 | $n_sroa_1_4_extract_trunc >>> ($78 >>> 0);
        $_0$0 = $n_sroa_1_4_extract_trunc << 32 - $78 | $n_sroa_0_0_extract_trunc >>> ($78 >>> 0) | 0;
        return (tempRet0 = $_0$1, $_0$0) | 0;
      }
    }
  } while (0);
  if (($sr_1_ph | 0) == 0) {
    $q_sroa_1_1_lcssa = $q_sroa_1_1_ph;
    $q_sroa_0_1_lcssa = $q_sroa_0_1_ph;
    $r_sroa_1_1_lcssa = $r_sroa_1_1_ph;
    $r_sroa_0_1_lcssa = $r_sroa_0_1_ph;
    $carry_0_lcssa$1 = 0;
    $carry_0_lcssa$0 = 0;
  } else {
    $d_sroa_0_0_insert_insert99$0 = 0 | $b$0 & -1;
    $d_sroa_0_0_insert_insert99$1 = $d_sroa_1_4_extract_shift$0 | $b$1 & 0;
    $137$0 = _i64Add($d_sroa_0_0_insert_insert99$0, $d_sroa_0_0_insert_insert99$1, -1, -1) | 0;
    $137$1 = tempRet0;
    $q_sroa_1_1198 = $q_sroa_1_1_ph;
    $q_sroa_0_1199 = $q_sroa_0_1_ph;
    $r_sroa_1_1200 = $r_sroa_1_1_ph;
    $r_sroa_0_1201 = $r_sroa_0_1_ph;
    $sr_1202 = $sr_1_ph;
    $carry_0203 = 0;
    while (1) {
      $147 = $q_sroa_0_1199 >>> 31 | $q_sroa_1_1198 << 1;
      $149 = $carry_0203 | $q_sroa_0_1199 << 1;
      $r_sroa_0_0_insert_insert42$0 = 0 | ($r_sroa_0_1201 << 1 | $q_sroa_1_1198 >>> 31);
      $r_sroa_0_0_insert_insert42$1 = $r_sroa_0_1201 >>> 31 | $r_sroa_1_1200 << 1 | 0;
      _i64Subtract($137$0, $137$1, $r_sroa_0_0_insert_insert42$0, $r_sroa_0_0_insert_insert42$1) | 0;
      $150$1 = tempRet0;
      $151$0 = $150$1 >> 31 | (($150$1 | 0) < 0 ? -1 : 0) << 1;
      $152 = $151$0 & 1;
      $154$0 = _i64Subtract($r_sroa_0_0_insert_insert42$0, $r_sroa_0_0_insert_insert42$1, $151$0 & $d_sroa_0_0_insert_insert99$0, ((($150$1 | 0) < 0 ? -1 : 0) >> 31 | (($150$1 | 0) < 0 ? -1 : 0) << 1) & $d_sroa_0_0_insert_insert99$1) | 0;
      $r_sroa_0_0_extract_trunc = $154$0;
      $r_sroa_1_4_extract_trunc = tempRet0;
      $155 = $sr_1202 - 1 | 0;
      if (($155 | 0) == 0) {
        break;
      } else {
        $q_sroa_1_1198 = $147;
        $q_sroa_0_1199 = $149;
        $r_sroa_1_1200 = $r_sroa_1_4_extract_trunc;
        $r_sroa_0_1201 = $r_sroa_0_0_extract_trunc;
        $sr_1202 = $155;
        $carry_0203 = $152;
      }
    }
    $q_sroa_1_1_lcssa = $147;
    $q_sroa_0_1_lcssa = $149;
    $r_sroa_1_1_lcssa = $r_sroa_1_4_extract_trunc;
    $r_sroa_0_1_lcssa = $r_sroa_0_0_extract_trunc;
    $carry_0_lcssa$1 = 0;
    $carry_0_lcssa$0 = $152;
  }
  $q_sroa_0_0_insert_ext75$0 = $q_sroa_0_1_lcssa;
  $q_sroa_0_0_insert_ext75$1 = 0;
  $q_sroa_0_0_insert_insert77$1 = $q_sroa_1_1_lcssa | $q_sroa_0_0_insert_ext75$1;
  if (($rem | 0) != 0) {
    HEAP32[$rem >> 2] = 0 | $r_sroa_0_1_lcssa;
    HEAP32[$rem + 4 >> 2] = $r_sroa_1_1_lcssa | 0;
  }
  $_0$1 = (0 | $q_sroa_0_0_insert_ext75$0) >>> 31 | $q_sroa_0_0_insert_insert77$1 << 1 | ($q_sroa_0_0_insert_ext75$1 << 1 | $q_sroa_0_0_insert_ext75$0 >>> 31) & 0 | $carry_0_lcssa$1;
  $_0$0 = ($q_sroa_0_0_insert_ext75$0 << 1 | 0 >>> 31) & -2 | $carry_0_lcssa$0;
  return (tempRet0 = $_0$1, $_0$0) | 0;
}
// =======================================================================



// EMSCRIPTEN_END_FUNCS

  
  function dynCall_iiii(index,a1,a2,a3) {
    index = index|0;
    a1=a1|0; a2=a2|0; a3=a3|0;
    return FUNCTION_TABLE_iiii[index&31](a1|0,a2|0,a3|0)|0;
  }


  function dynCall_i(index) {
    index = index|0;
    
    return FUNCTION_TABLE_i[index&1]()|0;
  }


  function dynCall_vi(index,a1) {
    index = index|0;
    a1=a1|0;
    FUNCTION_TABLE_vi[index&31](a1|0);
  }


  function dynCall_vii(index,a1,a2) {
    index = index|0;
    a1=a1|0; a2=a2|0;
    FUNCTION_TABLE_vii[index&7](a1|0,a2|0);
  }


  function dynCall_iiiiiii(index,a1,a2,a3,a4,a5,a6) {
    index = index|0;
    a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0; a5=a5|0; a6=a6|0;
    return FUNCTION_TABLE_iiiiiii[index&1](a1|0,a2|0,a3|0,a4|0,a5|0,a6|0)|0;
  }


  function dynCall_ii(index,a1) {
    index = index|0;
    a1=a1|0;
    return FUNCTION_TABLE_ii[index&31](a1|0)|0;
  }


  function dynCall_viii(index,a1,a2,a3) {
    index = index|0;
    a1=a1|0; a2=a2|0; a3=a3|0;
    FUNCTION_TABLE_viii[index&63](a1|0,a2|0,a3|0);
  }


  function dynCall_v(index) {
    index = index|0;
    
    FUNCTION_TABLE_v[index&0]();
  }


  function dynCall_iiiii(index,a1,a2,a3,a4) {
    index = index|0;
    a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0;
    return FUNCTION_TABLE_iiiii[index&7](a1|0,a2|0,a3|0,a4|0)|0;
  }


  function dynCall_viiiiii(index,a1,a2,a3,a4,a5,a6) {
    index = index|0;
    a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0; a5=a5|0; a6=a6|0;
    FUNCTION_TABLE_viiiiii[index&0](a1|0,a2|0,a3|0,a4|0,a5|0,a6|0);
  }


  function dynCall_iii(index,a1,a2) {
    index = index|0;
    a1=a1|0; a2=a2|0;
    return FUNCTION_TABLE_iii[index&63](a1|0,a2|0)|0;
  }


  function dynCall_iiiiii(index,a1,a2,a3,a4,a5) {
    index = index|0;
    a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0; a5=a5|0;
    return FUNCTION_TABLE_iiiiii[index&15](a1|0,a2|0,a3|0,a4|0,a5|0)|0;
  }


  function dynCall_viiii(index,a1,a2,a3,a4) {
    index = index|0;
    a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0;
    FUNCTION_TABLE_viiii[index&3](a1|0,a2|0,a3|0,a4|0);
  }

function b0(p0,p1,p2) { p0 = p0|0;p1 = p1|0;p2 = p2|0; abort(0);return 0; }
  function _fcntl__wrapper(p0,p1,p2) { p0 = p0|0;p1 = p1|0;p2 = p2|0; return _fcntl(p0|0,p1|0,p2|0)|0; }
  function _read__wrapper(p0,p1,p2) { p0 = p0|0;p1 = p1|0;p2 = p2|0; return _read(p0|0,p1|0,p2|0)|0; }
  function _write__wrapper(p0,p1,p2) { p0 = p0|0;p1 = p1|0;p2 = p2|0; return _write(p0|0,p1|0,p2|0)|0; }
  function b1() { ; abort(1);return 0; }
  function b2(p0) { p0 = p0|0; abort(2); }
  function b3(p0,p1) { p0 = p0|0;p1 = p1|0; abort(3); }
  function b4(p0,p1,p2,p3,p4,p5) { p0 = p0|0;p1 = p1|0;p2 = p2|0;p3 = p3|0;p4 = p4|0;p5 = p5|0; abort(4);return 0; }
  function _mmap__wrapper(p0,p1,p2,p3,p4,p5) { p0 = p0|0;p1 = p1|0;p2 = p2|0;p3 = p3|0;p4 = p4|0;p5 = p5|0; return _mmap(p0|0,p1|0,p2|0,p3|0,p4|0,p5|0)|0; }
  function b5(p0) { p0 = p0|0; abort(5);return 0; }
  function _close__wrapper(p0) { p0 = p0|0; return _close(p0|0)|0; }
  function _unlink__wrapper(p0) { p0 = p0|0; return _unlink(p0|0)|0; }
  function _rmdir__wrapper(p0) { p0 = p0|0; return _rmdir(p0|0)|0; }
  function b6(p0,p1,p2) { p0 = p0|0;p1 = p1|0;p2 = p2|0; abort(6); }
  function b7() { ; abort(7); }
  function b8(p0,p1,p2,p3) { p0 = p0|0;p1 = p1|0;p2 = p2|0;p3 = p3|0; abort(8);return 0; }
  function b9(p0,p1,p2,p3,p4,p5) { p0 = p0|0;p1 = p1|0;p2 = p2|0;p3 = p3|0;p4 = p4|0;p5 = p5|0; abort(9); }
  function b10(p0,p1) { p0 = p0|0;p1 = p1|0; abort(10);return 0; }
  function _access__wrapper(p0,p1) { p0 = p0|0;p1 = p1|0; return _access(p0|0,p1|0)|0; }
  function _getcwd__wrapper(p0,p1) { p0 = p0|0;p1 = p1|0; return _getcwd(p0|0,p1|0)|0; }
  function _stat__wrapper(p0,p1) { p0 = p0|0;p1 = p1|0; return _stat(p0|0,p1|0)|0; }
  function _fstat__wrapper(p0,p1) { p0 = p0|0;p1 = p1|0; return _fstat(p0|0,p1|0)|0; }
  function _ftruncate__wrapper(p0,p1) { p0 = p0|0;p1 = p1|0; return _ftruncate(p0|0,p1|0)|0; }
  function _fchmod__wrapper(p0,p1) { p0 = p0|0;p1 = p1|0; return _fchmod(p0|0,p1|0)|0; }
  function _mkdir__wrapper(p0,p1) { p0 = p0|0;p1 = p1|0; return _mkdir(p0|0,p1|0)|0; }
  function _munmap__wrapper(p0,p1) { p0 = p0|0;p1 = p1|0; return _munmap(p0|0,p1|0)|0; }
  function b11(p0,p1,p2,p3,p4) { p0 = p0|0;p1 = p1|0;p2 = p2|0;p3 = p3|0;p4 = p4|0; abort(11);return 0; }
  function b12(p0,p1,p2,p3) { p0 = p0|0;p1 = p1|0;p2 = p2|0;p3 = p3|0; abort(12); }
  // EMSCRIPTEN_END_FUNCS
  var FUNCTION_TABLE_iiii = [b0,_unixDelete,_unixRandomness,_unixGetLastError,_unixSetSystemCall,_memjrnlTruncate,_pcache1Create,_pcache1Fetch,_unixTruncate,_unixFileControl,_posixOpen,_fcntl__wrapper,_read__wrapper,_write__wrapper,_posixFchown,_vdbeRecordCompareInt,_vdbeRecordCompareString,_sqlite3VdbeRecordCompare,b0,b0,b0,b0,b0,b0,b0,b0,b0,b0,b0
  ,b0,b0,b0];
  var FUNCTION_TABLE_i = [b1,_unixGetpagesize];
  var FUNCTION_TABLE_vi = [b2,_pcache1Shutdown,_pcache1Destroy,_pcache1Shrink,_sqlite3MemFree,_sqlite3MemShutdown,_minMaxFinalize,_sumFinalize,_totalFinalize,_avgFinalize,_countFinalize,_groupConcatFinalize,_unixShmBarrier,_sqlite3_free,_pageReinit,_sqlite3SchemaClear,_stat4Destructor,b2,b2,b2,b2,b2,b2,b2,b2,b2,b2,b2,b2
  ,b2,b2,b2];
  var FUNCTION_TABLE_vii = [b3,_pcache1Cachesize,_pcache1Truncate,_selectPopWith,_selectAddSubqueryTypeInfo,b3,b3,b3];
  var FUNCTION_TABLE_iiiiiii = [b4,_mmap__wrapper];
  var FUNCTION_TABLE_ii = [b5,_memjrnlClose,_pcache1Init,_pcache1Pagecount,_sqlite3MemMalloc,_sqlite3MemSize,_sqlite3MemRoundup,_sqlite3MemInit,_dotlockClose,_unixSectorSize,_unixDeviceCharacteristics,_close__wrapper,_unlink__wrapper,_rmdir__wrapper,_nolockClose,_unixClose,_sqlite3MallocSize,_sqlite3_value_text,_btreeInvokeBusyHandler,b5,b5,b5,b5,b5,b5,b5,b5,b5,b5
  ,b5,b5,b5];
  var FUNCTION_TABLE_viii = [b6,_pcache1Unpin,_trimFunc,_minmaxFunc,_minmaxStep,_typeofFunc,_lengthFunc,_instrFunc,_substrFunc,_printfFunc,_unicodeFunc,_charFunc,_absFunc,_roundFunc,_upperFunc,_lowerFunc,_versionFunc,_hexFunc,_randomFunc,_randomBlob,_nullifFunc,_sourceidFunc,_errlogFunc,_compileoptionusedFunc,_compileoptiongetFunc,_quoteFunc,_last_insert_rowid,_changes,_total_changes
  ,_replaceFunc,_zeroblobFunc,_sumStep,_countStep,_groupConcatStep,_likeFunc,_renameTableFunc,_renameTriggerFunc,_renameParentFunc,_juliandayFunc,_dateFunc,_timeFunc,_datetimeFunc,_strftimeFunc,_ctimeFunc,_ctimestampFunc,_cdateFunc,_statInit,_statPush,_statGet,_detachFunc,_attachFunc,_sqlite3InvalidFunction,b6,b6,b6,b6,b6,b6,b6
  ,b6,b6,b6,b6,b6];
  var FUNCTION_TABLE_v = [b7];
  var FUNCTION_TABLE_iiiii = [b8,_unixAccess,_unixFullPathname,_unixShmLock,_unixUnfetch,_sqlite3WalDefaultHook,_sqlite3InitCallback,_analysisLoader];
  var FUNCTION_TABLE_viiiiii = [b9];
  var FUNCTION_TABLE_iii = [b10,_unixSleep,_unixCurrentTime,_unixCurrentTimeInt64,_unixGetSystemCall,_unixNextSystemCall,_posixIoFinderImpl,_nolockIoFinderImpl,_dotlockIoFinderImpl,_memjrnlSync,_memjrnlFileSize,_sqlite3MemRealloc,_unixSync,_unixFileSize,_dotlockLock,_dotlockUnlock,_dotlockCheckReservedLock,_unixShmUnmap,_access__wrapper,_getcwd__wrapper,_stat__wrapper,_fstat__wrapper,_ftruncate__wrapper,_fchmod__wrapper,_openDirectory,_mkdir__wrapper,_munmap__wrapper,_nolockLock,_nolockUnlock
  ,_nolockCheckReservedLock,_unixLock,_unixUnlock,_unixCheckReservedLock,_sqlite3Malloc,_sqliteDefaultBusyCallback,_pagerStress,_pagerUndoCallback,_exprWalkNoop,_convertCompoundSelectToSubquery,_selectExpander,_resolveExprStep,_resolveSelectStep,_incrAggDepth,_exprSrcCount,_exprNodeIsConstant,_selectNodeIsConstant,_sqlite3BtreeNext,_sqlite3BtreePrevious,_analyzeAggregate,_analyzeAggregatesInSelect,b10,b10,b10,b10,b10,b10,b10,b10,b10
  ,b10,b10,b10,b10,b10];
  var FUNCTION_TABLE_iiiiii = [b11,_unixOpen,_memjrnlRead,_memjrnlWrite,_unixRead,_unixWrite,_unixFetch,_unixShmMap,_binCollFunc,_nocaseCollatingFunc,b11,b11,b11,b11,b11,b11];
  var FUNCTION_TABLE_viiii = [b12,_pcache1Rekey,_softHeapLimitEnforcer,b12];

  return { _strlen: _strlen, _sqlite3_column_name: _sqlite3_column_name, _sqlite3_reset: _sqlite3_reset, _sqlite3_column_type: _sqlite3_column_type, _sqlite3_exec: _sqlite3_exec, _sqlite3_step: _sqlite3_step, _bitshift64Lshr: _bitshift64Lshr, _sqlite3_prepare_v2: _sqlite3_prepare_v2, _sqlite3_close_v2: _sqlite3_close_v2, _sqlite3_open: _sqlite3_open, _bitshift64Shl: _bitshift64Shl, _sqlite3_column_bytes: _sqlite3_column_bytes, _sqlite3_bind_int: _sqlite3_bind_int, _sqlite3_bind_blob: _sqlite3_bind_blob, _memset: _memset, _memcpy: _memcpy, _sqlite3_column_blob: _sqlite3_column_blob, _sqlite3_bind_parameter_index: _sqlite3_bind_parameter_index, _i64Subtract: _i64Subtract, _sqlite3_column_text: _sqlite3_column_text, _realloc: _realloc, _i64Add: _i64Add, _sqlite3_finalize: _sqlite3_finalize, _sqlite3_column_double: _sqlite3_column_double, _sqlite3_errmsg: _sqlite3_errmsg, _sqlite3_data_count: _sqlite3_data_count, _sqlite3_bind_text: _sqlite3_bind_text, _sqlite3_bind_double: _sqlite3_bind_double, _free: _free, _memmove: _memmove, _sqlite3_free: _sqlite3_free, _sqlite3_clear_bindings: _sqlite3_clear_bindings, _malloc: _malloc, runPostSets: runPostSets, stackAlloc: stackAlloc, stackSave: stackSave, stackRestore: stackRestore, setThrew: setThrew, setTempRet0: setTempRet0, getTempRet0: getTempRet0, dynCall_iiii: dynCall_iiii, dynCall_i: dynCall_i, dynCall_vi: dynCall_vi, dynCall_vii: dynCall_vii, dynCall_iiiiiii: dynCall_iiiiiii, dynCall_ii: dynCall_ii, dynCall_viii: dynCall_viii, dynCall_v: dynCall_v, dynCall_iiiii: dynCall_iiiii, dynCall_viiiiii: dynCall_viiiiii, dynCall_iii: dynCall_iii, dynCall_iiiiii: dynCall_iiiiii, dynCall_viiii: dynCall_viiii };
})
// EMSCRIPTEN_END_ASM
(Module.asmGlobalArg, Module.asmLibraryArg, buffer);
var _strlen = Module["_strlen"] = asm["_strlen"];
var _sqlite3_column_name = Module["_sqlite3_column_name"] = asm["_sqlite3_column_name"];
var _sqlite3_reset = Module["_sqlite3_reset"] = asm["_sqlite3_reset"];
var _sqlite3_column_type = Module["_sqlite3_column_type"] = asm["_sqlite3_column_type"];
var _sqlite3_exec = Module["_sqlite3_exec"] = asm["_sqlite3_exec"];
var _sqlite3_step = Module["_sqlite3_step"] = asm["_sqlite3_step"];
var _bitshift64Lshr = Module["_bitshift64Lshr"] = asm["_bitshift64Lshr"];
var _sqlite3_prepare_v2 = Module["_sqlite3_prepare_v2"] = asm["_sqlite3_prepare_v2"];
var _sqlite3_close_v2 = Module["_sqlite3_close_v2"] = asm["_sqlite3_close_v2"];
var _sqlite3_open = Module["_sqlite3_open"] = asm["_sqlite3_open"];
var _bitshift64Shl = Module["_bitshift64Shl"] = asm["_bitshift64Shl"];
var _sqlite3_column_bytes = Module["_sqlite3_column_bytes"] = asm["_sqlite3_column_bytes"];
var _sqlite3_bind_int = Module["_sqlite3_bind_int"] = asm["_sqlite3_bind_int"];
var _sqlite3_bind_blob = Module["_sqlite3_bind_blob"] = asm["_sqlite3_bind_blob"];
var _memset = Module["_memset"] = asm["_memset"];
var _memcpy = Module["_memcpy"] = asm["_memcpy"];
var _sqlite3_column_blob = Module["_sqlite3_column_blob"] = asm["_sqlite3_column_blob"];
var _sqlite3_bind_parameter_index = Module["_sqlite3_bind_parameter_index"] = asm["_sqlite3_bind_parameter_index"];
var _i64Subtract = Module["_i64Subtract"] = asm["_i64Subtract"];
var _sqlite3_column_text = Module["_sqlite3_column_text"] = asm["_sqlite3_column_text"];
var _realloc = Module["_realloc"] = asm["_realloc"];
var _i64Add = Module["_i64Add"] = asm["_i64Add"];
var _sqlite3_finalize = Module["_sqlite3_finalize"] = asm["_sqlite3_finalize"];
var _sqlite3_column_double = Module["_sqlite3_column_double"] = asm["_sqlite3_column_double"];
var _sqlite3_errmsg = Module["_sqlite3_errmsg"] = asm["_sqlite3_errmsg"];
var _sqlite3_data_count = Module["_sqlite3_data_count"] = asm["_sqlite3_data_count"];
var _sqlite3_bind_text = Module["_sqlite3_bind_text"] = asm["_sqlite3_bind_text"];
var _sqlite3_bind_double = Module["_sqlite3_bind_double"] = asm["_sqlite3_bind_double"];
var _free = Module["_free"] = asm["_free"];
var _memmove = Module["_memmove"] = asm["_memmove"];
var _sqlite3_free = Module["_sqlite3_free"] = asm["_sqlite3_free"];
var _sqlite3_clear_bindings = Module["_sqlite3_clear_bindings"] = asm["_sqlite3_clear_bindings"];
var _malloc = Module["_malloc"] = asm["_malloc"];
var runPostSets = Module["runPostSets"] = asm["runPostSets"];
var dynCall_iiii = Module["dynCall_iiii"] = asm["dynCall_iiii"];
var dynCall_i = Module["dynCall_i"] = asm["dynCall_i"];
var dynCall_vi = Module["dynCall_vi"] = asm["dynCall_vi"];
var dynCall_vii = Module["dynCall_vii"] = asm["dynCall_vii"];
var dynCall_iiiiiii = Module["dynCall_iiiiiii"] = asm["dynCall_iiiiiii"];
var dynCall_ii = Module["dynCall_ii"] = asm["dynCall_ii"];
var dynCall_viii = Module["dynCall_viii"] = asm["dynCall_viii"];
var dynCall_v = Module["dynCall_v"] = asm["dynCall_v"];
var dynCall_iiiii = Module["dynCall_iiiii"] = asm["dynCall_iiiii"];
var dynCall_viiiiii = Module["dynCall_viiiiii"] = asm["dynCall_viiiiii"];
var dynCall_iii = Module["dynCall_iii"] = asm["dynCall_iii"];
var dynCall_iiiiii = Module["dynCall_iiiiii"] = asm["dynCall_iiiiii"];
var dynCall_viiii = Module["dynCall_viiii"] = asm["dynCall_viiii"];

Runtime.stackAlloc = asm['stackAlloc'];
Runtime.stackSave = asm['stackSave'];
Runtime.stackRestore = asm['stackRestore'];
Runtime.setTempRet0 = asm['setTempRet0'];
Runtime.getTempRet0 = asm['getTempRet0'];


// TODO: strip out parts of this we do not need

//======= begin closure i64 code =======

// Copyright 2009 The Closure Library Authors. All Rights Reserved.
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//      http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS-IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

/**
 * @fileoverview Defines a Long class for representing a 64-bit two's-complement
 * integer value, which faithfully simulates the behavior of a Java "long". This
 * implementation is derived from LongLib in GWT.
 *
 */

var i64Math = (function() { // Emscripten wrapper
  var goog = { math: {} };


  /**
   * Constructs a 64-bit two's-complement integer, given its low and high 32-bit
   * values as *signed* integers.  See the from* functions below for more
   * convenient ways of constructing Longs.
   *
   * The internal representation of a long is the two given signed, 32-bit values.
   * We use 32-bit pieces because these are the size of integers on which
   * Javascript performs bit-operations.  For operations like addition and
   * multiplication, we split each number into 16-bit pieces, which can easily be
   * multiplied within Javascript's floating-point representation without overflow
   * or change in sign.
   *
   * In the algorithms below, we frequently reduce the negative case to the
   * positive case by negating the input(s) and then post-processing the result.
   * Note that we must ALWAYS check specially whether those values are MIN_VALUE
   * (-2^63) because -MIN_VALUE == MIN_VALUE (since 2^63 cannot be represented as
   * a positive number, it overflows back into a negative).  Not handling this
   * case would often result in infinite recursion.
   *
   * @param {number} low  The low (signed) 32 bits of the long.
   * @param {number} high  The high (signed) 32 bits of the long.
   * @constructor
   */
  goog.math.Long = function(low, high) {
    /**
     * @type {number}
     * @private
     */
    this.low_ = low | 0;  // force into 32 signed bits.

    /**
     * @type {number}
     * @private
     */
    this.high_ = high | 0;  // force into 32 signed bits.
  };


  // NOTE: Common constant values ZERO, ONE, NEG_ONE, etc. are defined below the
  // from* methods on which they depend.


  /**
   * A cache of the Long representations of small integer values.
   * @type {!Object}
   * @private
   */
  goog.math.Long.IntCache_ = {};


  /**
   * Returns a Long representing the given (32-bit) integer value.
   * @param {number} value The 32-bit integer in question.
   * @return {!goog.math.Long} The corresponding Long value.
   */
  goog.math.Long.fromInt = function(value) {
    if (-128 <= value && value < 128) {
      var cachedObj = goog.math.Long.IntCache_[value];
      if (cachedObj) {
        return cachedObj;
      }
    }

    var obj = new goog.math.Long(value | 0, value < 0 ? -1 : 0);
    if (-128 <= value && value < 128) {
      goog.math.Long.IntCache_[value] = obj;
    }
    return obj;
  };


  /**
   * Returns a Long representing the given value, provided that it is a finite
   * number.  Otherwise, zero is returned.
   * @param {number} value The number in question.
   * @return {!goog.math.Long} The corresponding Long value.
   */
  goog.math.Long.fromNumber = function(value) {
    if (isNaN(value) || !isFinite(value)) {
      return goog.math.Long.ZERO;
    } else if (value <= -goog.math.Long.TWO_PWR_63_DBL_) {
      return goog.math.Long.MIN_VALUE;
    } else if (value + 1 >= goog.math.Long.TWO_PWR_63_DBL_) {
      return goog.math.Long.MAX_VALUE;
    } else if (value < 0) {
      return goog.math.Long.fromNumber(-value).negate();
    } else {
      return new goog.math.Long(
          (value % goog.math.Long.TWO_PWR_32_DBL_) | 0,
          (value / goog.math.Long.TWO_PWR_32_DBL_) | 0);
    }
  };


  /**
   * Returns a Long representing the 64-bit integer that comes by concatenating
   * the given high and low bits.  Each is assumed to use 32 bits.
   * @param {number} lowBits The low 32-bits.
   * @param {number} highBits The high 32-bits.
   * @return {!goog.math.Long} The corresponding Long value.
   */
  goog.math.Long.fromBits = function(lowBits, highBits) {
    return new goog.math.Long(lowBits, highBits);
  };


  /**
   * Returns a Long representation of the given string, written using the given
   * radix.
   * @param {string} str The textual representation of the Long.
   * @param {number=} opt_radix The radix in which the text is written.
   * @return {!goog.math.Long} The corresponding Long value.
   */
  goog.math.Long.fromString = function(str, opt_radix) {
    if (str.length == 0) {
      throw Error('number format error: empty string');
    }

    var radix = opt_radix || 10;
    if (radix < 2 || 36 < radix) {
      throw Error('radix out of range: ' + radix);
    }

    if (str.charAt(0) == '-') {
      return goog.math.Long.fromString(str.substring(1), radix).negate();
    } else if (str.indexOf('-') >= 0) {
      throw Error('number format error: interior "-" character: ' + str);
    }

    // Do several (8) digits each time through the loop, so as to
    // minimize the calls to the very expensive emulated div.
    var radixToPower = goog.math.Long.fromNumber(Math.pow(radix, 8));

    var result = goog.math.Long.ZERO;
    for (var i = 0; i < str.length; i += 8) {
      var size = Math.min(8, str.length - i);
      var value = parseInt(str.substring(i, i + size), radix);
      if (size < 8) {
        var power = goog.math.Long.fromNumber(Math.pow(radix, size));
        result = result.multiply(power).add(goog.math.Long.fromNumber(value));
      } else {
        result = result.multiply(radixToPower);
        result = result.add(goog.math.Long.fromNumber(value));
      }
    }
    return result;
  };


  // NOTE: the compiler should inline these constant values below and then remove
  // these variables, so there should be no runtime penalty for these.


  /**
   * Number used repeated below in calculations.  This must appear before the
   * first call to any from* function below.
   * @type {number}
   * @private
   */
  goog.math.Long.TWO_PWR_16_DBL_ = 1 << 16;


  /**
   * @type {number}
   * @private
   */
  goog.math.Long.TWO_PWR_24_DBL_ = 1 << 24;


  /**
   * @type {number}
   * @private
   */
  goog.math.Long.TWO_PWR_32_DBL_ =
      goog.math.Long.TWO_PWR_16_DBL_ * goog.math.Long.TWO_PWR_16_DBL_;


  /**
   * @type {number}
   * @private
   */
  goog.math.Long.TWO_PWR_31_DBL_ =
      goog.math.Long.TWO_PWR_32_DBL_ / 2;


  /**
   * @type {number}
   * @private
   */
  goog.math.Long.TWO_PWR_48_DBL_ =
      goog.math.Long.TWO_PWR_32_DBL_ * goog.math.Long.TWO_PWR_16_DBL_;


  /**
   * @type {number}
   * @private
   */
  goog.math.Long.TWO_PWR_64_DBL_ =
      goog.math.Long.TWO_PWR_32_DBL_ * goog.math.Long.TWO_PWR_32_DBL_;


  /**
   * @type {number}
   * @private
   */
  goog.math.Long.TWO_PWR_63_DBL_ =
      goog.math.Long.TWO_PWR_64_DBL_ / 2;


  /** @type {!goog.math.Long} */
  goog.math.Long.ZERO = goog.math.Long.fromInt(0);


  /** @type {!goog.math.Long} */
  goog.math.Long.ONE = goog.math.Long.fromInt(1);


  /** @type {!goog.math.Long} */
  goog.math.Long.NEG_ONE = goog.math.Long.fromInt(-1);


  /** @type {!goog.math.Long} */
  goog.math.Long.MAX_VALUE =
      goog.math.Long.fromBits(0xFFFFFFFF | 0, 0x7FFFFFFF | 0);


  /** @type {!goog.math.Long} */
  goog.math.Long.MIN_VALUE = goog.math.Long.fromBits(0, 0x80000000 | 0);


  /**
   * @type {!goog.math.Long}
   * @private
   */
  goog.math.Long.TWO_PWR_24_ = goog.math.Long.fromInt(1 << 24);


  /** @return {number} The value, assuming it is a 32-bit integer. */
  goog.math.Long.prototype.toInt = function() {
    return this.low_;
  };


  /** @return {number} The closest floating-point representation to this value. */
  goog.math.Long.prototype.toNumber = function() {
    return this.high_ * goog.math.Long.TWO_PWR_32_DBL_ +
           this.getLowBitsUnsigned();
  };


  /**
   * @param {number=} opt_radix The radix in which the text should be written.
   * @return {string} The textual representation of this value.
   */
  goog.math.Long.prototype.toString = function(opt_radix) {
    var radix = opt_radix || 10;
    if (radix < 2 || 36 < radix) {
      throw Error('radix out of range: ' + radix);
    }

    if (this.isZero()) {
      return '0';
    }

    if (this.isNegative()) {
      if (this.equals(goog.math.Long.MIN_VALUE)) {
        // We need to change the Long value before it can be negated, so we remove
        // the bottom-most digit in this base and then recurse to do the rest.
        var radixLong = goog.math.Long.fromNumber(radix);
        var div = this.div(radixLong);
        var rem = div.multiply(radixLong).subtract(this);
        return div.toString(radix) + rem.toInt().toString(radix);
      } else {
        return '-' + this.negate().toString(radix);
      }
    }

    // Do several (6) digits each time through the loop, so as to
    // minimize the calls to the very expensive emulated div.
    var radixToPower = goog.math.Long.fromNumber(Math.pow(radix, 6));

    var rem = this;
    var result = '';
    while (true) {
      var remDiv = rem.div(radixToPower);
      var intval = rem.subtract(remDiv.multiply(radixToPower)).toInt();
      var digits = intval.toString(radix);

      rem = remDiv;
      if (rem.isZero()) {
        return digits + result;
      } else {
        while (digits.length < 6) {
          digits = '0' + digits;
        }
        result = '' + digits + result;
      }
    }
  };


  /** @return {number} The high 32-bits as a signed value. */
  goog.math.Long.prototype.getHighBits = function() {
    return this.high_;
  };


  /** @return {number} The low 32-bits as a signed value. */
  goog.math.Long.prototype.getLowBits = function() {
    return this.low_;
  };


  /** @return {number} The low 32-bits as an unsigned value. */
  goog.math.Long.prototype.getLowBitsUnsigned = function() {
    return (this.low_ >= 0) ?
        this.low_ : goog.math.Long.TWO_PWR_32_DBL_ + this.low_;
  };


  /**
   * @return {number} Returns the number of bits needed to represent the absolute
   *     value of this Long.
   */
  goog.math.Long.prototype.getNumBitsAbs = function() {
    if (this.isNegative()) {
      if (this.equals(goog.math.Long.MIN_VALUE)) {
        return 64;
      } else {
        return this.negate().getNumBitsAbs();
      }
    } else {
      var val = this.high_ != 0 ? this.high_ : this.low_;
      for (var bit = 31; bit > 0; bit--) {
        if ((val & (1 << bit)) != 0) {
          break;
        }
      }
      return this.high_ != 0 ? bit + 33 : bit + 1;
    }
  };


  /** @return {boolean} Whether this value is zero. */
  goog.math.Long.prototype.isZero = function() {
    return this.high_ == 0 && this.low_ == 0;
  };


  /** @return {boolean} Whether this value is negative. */
  goog.math.Long.prototype.isNegative = function() {
    return this.high_ < 0;
  };


  /** @return {boolean} Whether this value is odd. */
  goog.math.Long.prototype.isOdd = function() {
    return (this.low_ & 1) == 1;
  };


  /**
   * @param {goog.math.Long} other Long to compare against.
   * @return {boolean} Whether this Long equals the other.
   */
  goog.math.Long.prototype.equals = function(other) {
    return (this.high_ == other.high_) && (this.low_ == other.low_);
  };


  /**
   * @param {goog.math.Long} other Long to compare against.
   * @return {boolean} Whether this Long does not equal the other.
   */
  goog.math.Long.prototype.notEquals = function(other) {
    return (this.high_ != other.high_) || (this.low_ != other.low_);
  };


  /**
   * @param {goog.math.Long} other Long to compare against.
   * @return {boolean} Whether this Long is less than the other.
   */
  goog.math.Long.prototype.lessThan = function(other) {
    return this.compare(other) < 0;
  };


  /**
   * @param {goog.math.Long} other Long to compare against.
   * @return {boolean} Whether this Long is less than or equal to the other.
   */
  goog.math.Long.prototype.lessThanOrEqual = function(other) {
    return this.compare(other) <= 0;
  };


  /**
   * @param {goog.math.Long} other Long to compare against.
   * @return {boolean} Whether this Long is greater than the other.
   */
  goog.math.Long.prototype.greaterThan = function(other) {
    return this.compare(other) > 0;
  };


  /**
   * @param {goog.math.Long} other Long to compare against.
   * @return {boolean} Whether this Long is greater than or equal to the other.
   */
  goog.math.Long.prototype.greaterThanOrEqual = function(other) {
    return this.compare(other) >= 0;
  };


  /**
   * Compares this Long with the given one.
   * @param {goog.math.Long} other Long to compare against.
   * @return {number} 0 if they are the same, 1 if the this is greater, and -1
   *     if the given one is greater.
   */
  goog.math.Long.prototype.compare = function(other) {
    if (this.equals(other)) {
      return 0;
    }

    var thisNeg = this.isNegative();
    var otherNeg = other.isNegative();
    if (thisNeg && !otherNeg) {
      return -1;
    }
    if (!thisNeg && otherNeg) {
      return 1;
    }

    // at this point, the signs are the same, so subtraction will not overflow
    if (this.subtract(other).isNegative()) {
      return -1;
    } else {
      return 1;
    }
  };


  /** @return {!goog.math.Long} The negation of this value. */
  goog.math.Long.prototype.negate = function() {
    if (this.equals(goog.math.Long.MIN_VALUE)) {
      return goog.math.Long.MIN_VALUE;
    } else {
      return this.not().add(goog.math.Long.ONE);
    }
  };


  /**
   * Returns the sum of this and the given Long.
   * @param {goog.math.Long} other Long to add to this one.
   * @return {!goog.math.Long} The sum of this and the given Long.
   */
  goog.math.Long.prototype.add = function(other) {
    // Divide each number into 4 chunks of 16 bits, and then sum the chunks.

    var a48 = this.high_ >>> 16;
    var a32 = this.high_ & 0xFFFF;
    var a16 = this.low_ >>> 16;
    var a00 = this.low_ & 0xFFFF;

    var b48 = other.high_ >>> 16;
    var b32 = other.high_ & 0xFFFF;
    var b16 = other.low_ >>> 16;
    var b00 = other.low_ & 0xFFFF;

    var c48 = 0, c32 = 0, c16 = 0, c00 = 0;
    c00 += a00 + b00;
    c16 += c00 >>> 16;
    c00 &= 0xFFFF;
    c16 += a16 + b16;
    c32 += c16 >>> 16;
    c16 &= 0xFFFF;
    c32 += a32 + b32;
    c48 += c32 >>> 16;
    c32 &= 0xFFFF;
    c48 += a48 + b48;
    c48 &= 0xFFFF;
    return goog.math.Long.fromBits((c16 << 16) | c00, (c48 << 16) | c32);
  };


  /**
   * Returns the difference of this and the given Long.
   * @param {goog.math.Long} other Long to subtract from this.
   * @return {!goog.math.Long} The difference of this and the given Long.
   */
  goog.math.Long.prototype.subtract = function(other) {
    return this.add(other.negate());
  };


  /**
   * Returns the product of this and the given long.
   * @param {goog.math.Long} other Long to multiply with this.
   * @return {!goog.math.Long} The product of this and the other.
   */
  goog.math.Long.prototype.multiply = function(other) {
    if (this.isZero()) {
      return goog.math.Long.ZERO;
    } else if (other.isZero()) {
      return goog.math.Long.ZERO;
    }

    if (this.equals(goog.math.Long.MIN_VALUE)) {
      return other.isOdd() ? goog.math.Long.MIN_VALUE : goog.math.Long.ZERO;
    } else if (other.equals(goog.math.Long.MIN_VALUE)) {
      return this.isOdd() ? goog.math.Long.MIN_VALUE : goog.math.Long.ZERO;
    }

    if (this.isNegative()) {
      if (other.isNegative()) {
        return this.negate().multiply(other.negate());
      } else {
        return this.negate().multiply(other).negate();
      }
    } else if (other.isNegative()) {
      return this.multiply(other.negate()).negate();
    }

    // If both longs are small, use float multiplication
    if (this.lessThan(goog.math.Long.TWO_PWR_24_) &&
        other.lessThan(goog.math.Long.TWO_PWR_24_)) {
      return goog.math.Long.fromNumber(this.toNumber() * other.toNumber());
    }

    // Divide each long into 4 chunks of 16 bits, and then add up 4x4 products.
    // We can skip products that would overflow.

    var a48 = this.high_ >>> 16;
    var a32 = this.high_ & 0xFFFF;
    var a16 = this.low_ >>> 16;
    var a00 = this.low_ & 0xFFFF;

    var b48 = other.high_ >>> 16;
    var b32 = other.high_ & 0xFFFF;
    var b16 = other.low_ >>> 16;
    var b00 = other.low_ & 0xFFFF;

    var c48 = 0, c32 = 0, c16 = 0, c00 = 0;
    c00 += a00 * b00;
    c16 += c00 >>> 16;
    c00 &= 0xFFFF;
    c16 += a16 * b00;
    c32 += c16 >>> 16;
    c16 &= 0xFFFF;
    c16 += a00 * b16;
    c32 += c16 >>> 16;
    c16 &= 0xFFFF;
    c32 += a32 * b00;
    c48 += c32 >>> 16;
    c32 &= 0xFFFF;
    c32 += a16 * b16;
    c48 += c32 >>> 16;
    c32 &= 0xFFFF;
    c32 += a00 * b32;
    c48 += c32 >>> 16;
    c32 &= 0xFFFF;
    c48 += a48 * b00 + a32 * b16 + a16 * b32 + a00 * b48;
    c48 &= 0xFFFF;
    return goog.math.Long.fromBits((c16 << 16) | c00, (c48 << 16) | c32);
  };


  /**
   * Returns this Long divided by the given one.
   * @param {goog.math.Long} other Long by which to divide.
   * @return {!goog.math.Long} This Long divided by the given one.
   */
  goog.math.Long.prototype.div = function(other) {
    if (other.isZero()) {
      throw Error('division by zero');
    } else if (this.isZero()) {
      return goog.math.Long.ZERO;
    }

    if (this.equals(goog.math.Long.MIN_VALUE)) {
      if (other.equals(goog.math.Long.ONE) ||
          other.equals(goog.math.Long.NEG_ONE)) {
        return goog.math.Long.MIN_VALUE;  // recall that -MIN_VALUE == MIN_VALUE
      } else if (other.equals(goog.math.Long.MIN_VALUE)) {
        return goog.math.Long.ONE;
      } else {
        // At this point, we have |other| >= 2, so |this/other| < |MIN_VALUE|.
        var halfThis = this.shiftRight(1);
        var approx = halfThis.div(other).shiftLeft(1);
        if (approx.equals(goog.math.Long.ZERO)) {
          return other.isNegative() ? goog.math.Long.ONE : goog.math.Long.NEG_ONE;
        } else {
          var rem = this.subtract(other.multiply(approx));
          var result = approx.add(rem.div(other));
          return result;
        }
      }
    } else if (other.equals(goog.math.Long.MIN_VALUE)) {
      return goog.math.Long.ZERO;
    }

    if (this.isNegative()) {
      if (other.isNegative()) {
        return this.negate().div(other.negate());
      } else {
        return this.negate().div(other).negate();
      }
    } else if (other.isNegative()) {
      return this.div(other.negate()).negate();
    }

    // Repeat the following until the remainder is less than other:  find a
    // floating-point that approximates remainder / other *from below*, add this
    // into the result, and subtract it from the remainder.  It is critical that
    // the approximate value is less than or equal to the real value so that the
    // remainder never becomes negative.
    var res = goog.math.Long.ZERO;
    var rem = this;
    while (rem.greaterThanOrEqual(other)) {
      // Approximate the result of division. This may be a little greater or
      // smaller than the actual value.
      var approx = Math.max(1, Math.floor(rem.toNumber() / other.toNumber()));

      // We will tweak the approximate result by changing it in the 48-th digit or
      // the smallest non-fractional digit, whichever is larger.
      var log2 = Math.ceil(Math.log(approx) / Math.LN2);
      var delta = (log2 <= 48) ? 1 : Math.pow(2, log2 - 48);

      // Decrease the approximation until it is smaller than the remainder.  Note
      // that if it is too large, the product overflows and is negative.
      var approxRes = goog.math.Long.fromNumber(approx);
      var approxRem = approxRes.multiply(other);
      while (approxRem.isNegative() || approxRem.greaterThan(rem)) {
        approx -= delta;
        approxRes = goog.math.Long.fromNumber(approx);
        approxRem = approxRes.multiply(other);
      }

      // We know the answer can't be zero... and actually, zero would cause
      // infinite recursion since we would make no progress.
      if (approxRes.isZero()) {
        approxRes = goog.math.Long.ONE;
      }

      res = res.add(approxRes);
      rem = rem.subtract(approxRem);
    }
    return res;
  };


  /**
   * Returns this Long modulo the given one.
   * @param {goog.math.Long} other Long by which to mod.
   * @return {!goog.math.Long} This Long modulo the given one.
   */
  goog.math.Long.prototype.modulo = function(other) {
    return this.subtract(this.div(other).multiply(other));
  };


  /** @return {!goog.math.Long} The bitwise-NOT of this value. */
  goog.math.Long.prototype.not = function() {
    return goog.math.Long.fromBits(~this.low_, ~this.high_);
  };


  /**
   * Returns the bitwise-AND of this Long and the given one.
   * @param {goog.math.Long} other The Long with which to AND.
   * @return {!goog.math.Long} The bitwise-AND of this and the other.
   */
  goog.math.Long.prototype.and = function(other) {
    return goog.math.Long.fromBits(this.low_ & other.low_,
                                   this.high_ & other.high_);
  };


  /**
   * Returns the bitwise-OR of this Long and the given one.
   * @param {goog.math.Long} other The Long with which to OR.
   * @return {!goog.math.Long} The bitwise-OR of this and the other.
   */
  goog.math.Long.prototype.or = function(other) {
    return goog.math.Long.fromBits(this.low_ | other.low_,
                                   this.high_ | other.high_);
  };


  /**
   * Returns the bitwise-XOR of this Long and the given one.
   * @param {goog.math.Long} other The Long with which to XOR.
   * @return {!goog.math.Long} The bitwise-XOR of this and the other.
   */
  goog.math.Long.prototype.xor = function(other) {
    return goog.math.Long.fromBits(this.low_ ^ other.low_,
                                   this.high_ ^ other.high_);
  };


  /**
   * Returns this Long with bits shifted to the left by the given amount.
   * @param {number} numBits The number of bits by which to shift.
   * @return {!goog.math.Long} This shifted to the left by the given amount.
   */
  goog.math.Long.prototype.shiftLeft = function(numBits) {
    numBits &= 63;
    if (numBits == 0) {
      return this;
    } else {
      var low = this.low_;
      if (numBits < 32) {
        var high = this.high_;
        return goog.math.Long.fromBits(
            low << numBits,
            (high << numBits) | (low >>> (32 - numBits)));
      } else {
        return goog.math.Long.fromBits(0, low << (numBits - 32));
      }
    }
  };


  /**
   * Returns this Long with bits shifted to the right by the given amount.
   * @param {number} numBits The number of bits by which to shift.
   * @return {!goog.math.Long} This shifted to the right by the given amount.
   */
  goog.math.Long.prototype.shiftRight = function(numBits) {
    numBits &= 63;
    if (numBits == 0) {
      return this;
    } else {
      var high = this.high_;
      if (numBits < 32) {
        var low = this.low_;
        return goog.math.Long.fromBits(
            (low >>> numBits) | (high << (32 - numBits)),
            high >> numBits);
      } else {
        return goog.math.Long.fromBits(
            high >> (numBits - 32),
            high >= 0 ? 0 : -1);
      }
    }
  };


  /**
   * Returns this Long with bits shifted to the right by the given amount, with
   * the new top bits matching the current sign bit.
   * @param {number} numBits The number of bits by which to shift.
   * @return {!goog.math.Long} This shifted to the right by the given amount, with
   *     zeros placed into the new leading bits.
   */
  goog.math.Long.prototype.shiftRightUnsigned = function(numBits) {
    numBits &= 63;
    if (numBits == 0) {
      return this;
    } else {
      var high = this.high_;
      if (numBits < 32) {
        var low = this.low_;
        return goog.math.Long.fromBits(
            (low >>> numBits) | (high << (32 - numBits)),
            high >>> numBits);
      } else if (numBits == 32) {
        return goog.math.Long.fromBits(high, 0);
      } else {
        return goog.math.Long.fromBits(high >>> (numBits - 32), 0);
      }
    }
  };

  //======= begin jsbn =======

  var navigator = { appName: 'Modern Browser' }; // polyfill a little

  // Copyright (c) 2005  Tom Wu
  // All Rights Reserved.
  // http://www-cs-students.stanford.edu/~tjw/jsbn/

  /*
   * Copyright (c) 2003-2005  Tom Wu
   * All Rights Reserved.
   *
   * Permission is hereby granted, free of charge, to any person obtaining
   * a copy of this software and associated documentation files (the
   * "Software"), to deal in the Software without restriction, including
   * without limitation the rights to use, copy, modify, merge, publish,
   * distribute, sublicense, and/or sell copies of the Software, and to
   * permit persons to whom the Software is furnished to do so, subject to
   * the following conditions:
   *
   * The above copyright notice and this permission notice shall be
   * included in all copies or substantial portions of the Software.
   *
   * THE SOFTWARE IS PROVIDED "AS-IS" AND WITHOUT WARRANTY OF ANY KIND, 
   * EXPRESS, IMPLIED OR OTHERWISE, INCLUDING WITHOUT LIMITATION, ANY 
   * WARRANTY OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE.  
   *
   * IN NO EVENT SHALL TOM WU BE LIABLE FOR ANY SPECIAL, INCIDENTAL,
   * INDIRECT OR CONSEQUENTIAL DAMAGES OF ANY KIND, OR ANY DAMAGES WHATSOEVER
   * RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER OR NOT ADVISED OF
   * THE POSSIBILITY OF DAMAGE, AND ON ANY THEORY OF LIABILITY, ARISING OUT
   * OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
   *
   * In addition, the following condition applies:
   *
   * All redistributions must retain an intact copy of this copyright notice
   * and disclaimer.
   */

  // Basic JavaScript BN library - subset useful for RSA encryption.

  // Bits per digit
  var dbits;

  // JavaScript engine analysis
  var canary = 0xdeadbeefcafe;
  var j_lm = ((canary&0xffffff)==0xefcafe);

  // (public) Constructor
  function BigInteger(a,b,c) {
    if(a != null)
      if("number" == typeof a) this.fromNumber(a,b,c);
      else if(b == null && "string" != typeof a) this.fromString(a,256);
      else this.fromString(a,b);
  }

  // return new, unset BigInteger
  function nbi() { return new BigInteger(null); }

  // am: Compute w_j += (x*this_i), propagate carries,
  // c is initial carry, returns final carry.
  // c < 3*dvalue, x < 2*dvalue, this_i < dvalue
  // We need to select the fastest one that works in this environment.

  // am1: use a single mult and divide to get the high bits,
  // max digit bits should be 26 because
  // max internal value = 2*dvalue^2-2*dvalue (< 2^53)
  function am1(i,x,w,j,c,n) {
    while(--n >= 0) {
      var v = x*this[i++]+w[j]+c;
      c = Math.floor(v/0x4000000);
      w[j++] = v&0x3ffffff;
    }
    return c;
  }
  // am2 avoids a big mult-and-extract completely.
  // Max digit bits should be <= 30 because we do bitwise ops
  // on values up to 2*hdvalue^2-hdvalue-1 (< 2^31)
  function am2(i,x,w,j,c,n) {
    var xl = x&0x7fff, xh = x>>15;
    while(--n >= 0) {
      var l = this[i]&0x7fff;
      var h = this[i++]>>15;
      var m = xh*l+h*xl;
      l = xl*l+((m&0x7fff)<<15)+w[j]+(c&0x3fffffff);
      c = (l>>>30)+(m>>>15)+xh*h+(c>>>30);
      w[j++] = l&0x3fffffff;
    }
    return c;
  }
  // Alternately, set max digit bits to 28 since some
  // browsers slow down when dealing with 32-bit numbers.
  function am3(i,x,w,j,c,n) {
    var xl = x&0x3fff, xh = x>>14;
    while(--n >= 0) {
      var l = this[i]&0x3fff;
      var h = this[i++]>>14;
      var m = xh*l+h*xl;
      l = xl*l+((m&0x3fff)<<14)+w[j]+c;
      c = (l>>28)+(m>>14)+xh*h;
      w[j++] = l&0xfffffff;
    }
    return c;
  }
  if(j_lm && (navigator.appName == "Microsoft Internet Explorer")) {
    BigInteger.prototype.am = am2;
    dbits = 30;
  }
  else if(j_lm && (navigator.appName != "Netscape")) {
    BigInteger.prototype.am = am1;
    dbits = 26;
  }
  else { // Mozilla/Netscape seems to prefer am3
    BigInteger.prototype.am = am3;
    dbits = 28;
  }

  BigInteger.prototype.DB = dbits;
  BigInteger.prototype.DM = ((1<<dbits)-1);
  BigInteger.prototype.DV = (1<<dbits);

  var BI_FP = 52;
  BigInteger.prototype.FV = Math.pow(2,BI_FP);
  BigInteger.prototype.F1 = BI_FP-dbits;
  BigInteger.prototype.F2 = 2*dbits-BI_FP;

  // Digit conversions
  var BI_RM = "0123456789abcdefghijklmnopqrstuvwxyz";
  var BI_RC = new Array();
  var rr,vv;
  rr = "0".charCodeAt(0);
  for(vv = 0; vv <= 9; ++vv) BI_RC[rr++] = vv;
  rr = "a".charCodeAt(0);
  for(vv = 10; vv < 36; ++vv) BI_RC[rr++] = vv;
  rr = "A".charCodeAt(0);
  for(vv = 10; vv < 36; ++vv) BI_RC[rr++] = vv;

  function int2char(n) { return BI_RM.charAt(n); }
  function intAt(s,i) {
    var c = BI_RC[s.charCodeAt(i)];
    return (c==null)?-1:c;
  }

  // (protected) copy this to r
  function bnpCopyTo(r) {
    for(var i = this.t-1; i >= 0; --i) r[i] = this[i];
    r.t = this.t;
    r.s = this.s;
  }

  // (protected) set from integer value x, -DV <= x < DV
  function bnpFromInt(x) {
    this.t = 1;
    this.s = (x<0)?-1:0;
    if(x > 0) this[0] = x;
    else if(x < -1) this[0] = x+DV;
    else this.t = 0;
  }

  // return bigint initialized to value
  function nbv(i) { var r = nbi(); r.fromInt(i); return r; }

  // (protected) set from string and radix
  function bnpFromString(s,b) {
    var k;
    if(b == 16) k = 4;
    else if(b == 8) k = 3;
    else if(b == 256) k = 8; // byte array
    else if(b == 2) k = 1;
    else if(b == 32) k = 5;
    else if(b == 4) k = 2;
    else { this.fromRadix(s,b); return; }
    this.t = 0;
    this.s = 0;
    var i = s.length, mi = false, sh = 0;
    while(--i >= 0) {
      var x = (k==8)?s[i]&0xff:intAt(s,i);
      if(x < 0) {
        if(s.charAt(i) == "-") mi = true;
        continue;
      }
      mi = false;
      if(sh == 0)
        this[this.t++] = x;
      else if(sh+k > this.DB) {
        this[this.t-1] |= (x&((1<<(this.DB-sh))-1))<<sh;
        this[this.t++] = (x>>(this.DB-sh));
      }
      else
        this[this.t-1] |= x<<sh;
      sh += k;
      if(sh >= this.DB) sh -= this.DB;
    }
    if(k == 8 && (s[0]&0x80) != 0) {
      this.s = -1;
      if(sh > 0) this[this.t-1] |= ((1<<(this.DB-sh))-1)<<sh;
    }
    this.clamp();
    if(mi) BigInteger.ZERO.subTo(this,this);
  }

  // (protected) clamp off excess high words
  function bnpClamp() {
    var c = this.s&this.DM;
    while(this.t > 0 && this[this.t-1] == c) --this.t;
  }

  // (public) return string representation in given radix
  function bnToString(b) {
    if(this.s < 0) return "-"+this.negate().toString(b);
    var k;
    if(b == 16) k = 4;
    else if(b == 8) k = 3;
    else if(b == 2) k = 1;
    else if(b == 32) k = 5;
    else if(b == 4) k = 2;
    else return this.toRadix(b);
    var km = (1<<k)-1, d, m = false, r = "", i = this.t;
    var p = this.DB-(i*this.DB)%k;
    if(i-- > 0) {
      if(p < this.DB && (d = this[i]>>p) > 0) { m = true; r = int2char(d); }
      while(i >= 0) {
        if(p < k) {
          d = (this[i]&((1<<p)-1))<<(k-p);
          d |= this[--i]>>(p+=this.DB-k);
        }
        else {
          d = (this[i]>>(p-=k))&km;
          if(p <= 0) { p += this.DB; --i; }
        }
        if(d > 0) m = true;
        if(m) r += int2char(d);
      }
    }
    return m?r:"0";
  }

  // (public) -this
  function bnNegate() { var r = nbi(); BigInteger.ZERO.subTo(this,r); return r; }

  // (public) |this|
  function bnAbs() { return (this.s<0)?this.negate():this; }

  // (public) return + if this > a, - if this < a, 0 if equal
  function bnCompareTo(a) {
    var r = this.s-a.s;
    if(r != 0) return r;
    var i = this.t;
    r = i-a.t;
    if(r != 0) return (this.s<0)?-r:r;
    while(--i >= 0) if((r=this[i]-a[i]) != 0) return r;
    return 0;
  }

  // returns bit length of the integer x
  function nbits(x) {
    var r = 1, t;
    if((t=x>>>16) != 0) { x = t; r += 16; }
    if((t=x>>8) != 0) { x = t; r += 8; }
    if((t=x>>4) != 0) { x = t; r += 4; }
    if((t=x>>2) != 0) { x = t; r += 2; }
    if((t=x>>1) != 0) { x = t; r += 1; }
    return r;
  }

  // (public) return the number of bits in "this"
  function bnBitLength() {
    if(this.t <= 0) return 0;
    return this.DB*(this.t-1)+nbits(this[this.t-1]^(this.s&this.DM));
  }

  // (protected) r = this << n*DB
  function bnpDLShiftTo(n,r) {
    var i;
    for(i = this.t-1; i >= 0; --i) r[i+n] = this[i];
    for(i = n-1; i >= 0; --i) r[i] = 0;
    r.t = this.t+n;
    r.s = this.s;
  }

  // (protected) r = this >> n*DB
  function bnpDRShiftTo(n,r) {
    for(var i = n; i < this.t; ++i) r[i-n] = this[i];
    r.t = Math.max(this.t-n,0);
    r.s = this.s;
  }

  // (protected) r = this << n
  function bnpLShiftTo(n,r) {
    var bs = n%this.DB;
    var cbs = this.DB-bs;
    var bm = (1<<cbs)-1;
    var ds = Math.floor(n/this.DB), c = (this.s<<bs)&this.DM, i;
    for(i = this.t-1; i >= 0; --i) {
      r[i+ds+1] = (this[i]>>cbs)|c;
      c = (this[i]&bm)<<bs;
    }
    for(i = ds-1; i >= 0; --i) r[i] = 0;
    r[ds] = c;
    r.t = this.t+ds+1;
    r.s = this.s;
    r.clamp();
  }

  // (protected) r = this >> n
  function bnpRShiftTo(n,r) {
    r.s = this.s;
    var ds = Math.floor(n/this.DB);
    if(ds >= this.t) { r.t = 0; return; }
    var bs = n%this.DB;
    var cbs = this.DB-bs;
    var bm = (1<<bs)-1;
    r[0] = this[ds]>>bs;
    for(var i = ds+1; i < this.t; ++i) {
      r[i-ds-1] |= (this[i]&bm)<<cbs;
      r[i-ds] = this[i]>>bs;
    }
    if(bs > 0) r[this.t-ds-1] |= (this.s&bm)<<cbs;
    r.t = this.t-ds;
    r.clamp();
  }

  // (protected) r = this - a
  function bnpSubTo(a,r) {
    var i = 0, c = 0, m = Math.min(a.t,this.t);
    while(i < m) {
      c += this[i]-a[i];
      r[i++] = c&this.DM;
      c >>= this.DB;
    }
    if(a.t < this.t) {
      c -= a.s;
      while(i < this.t) {
        c += this[i];
        r[i++] = c&this.DM;
        c >>= this.DB;
      }
      c += this.s;
    }
    else {
      c += this.s;
      while(i < a.t) {
        c -= a[i];
        r[i++] = c&this.DM;
        c >>= this.DB;
      }
      c -= a.s;
    }
    r.s = (c<0)?-1:0;
    if(c < -1) r[i++] = this.DV+c;
    else if(c > 0) r[i++] = c;
    r.t = i;
    r.clamp();
  }

  // (protected) r = this * a, r != this,a (HAC 14.12)
  // "this" should be the larger one if appropriate.
  function bnpMultiplyTo(a,r) {
    var x = this.abs(), y = a.abs();
    var i = x.t;
    r.t = i+y.t;
    while(--i >= 0) r[i] = 0;
    for(i = 0; i < y.t; ++i) r[i+x.t] = x.am(0,y[i],r,i,0,x.t);
    r.s = 0;
    r.clamp();
    if(this.s != a.s) BigInteger.ZERO.subTo(r,r);
  }

  // (protected) r = this^2, r != this (HAC 14.16)
  function bnpSquareTo(r) {
    var x = this.abs();
    var i = r.t = 2*x.t;
    while(--i >= 0) r[i] = 0;
    for(i = 0; i < x.t-1; ++i) {
      var c = x.am(i,x[i],r,2*i,0,1);
      if((r[i+x.t]+=x.am(i+1,2*x[i],r,2*i+1,c,x.t-i-1)) >= x.DV) {
        r[i+x.t] -= x.DV;
        r[i+x.t+1] = 1;
      }
    }
    if(r.t > 0) r[r.t-1] += x.am(i,x[i],r,2*i,0,1);
    r.s = 0;
    r.clamp();
  }

  // (protected) divide this by m, quotient and remainder to q, r (HAC 14.20)
  // r != q, this != m.  q or r may be null.
  function bnpDivRemTo(m,q,r) {
    var pm = m.abs();
    if(pm.t <= 0) return;
    var pt = this.abs();
    if(pt.t < pm.t) {
      if(q != null) q.fromInt(0);
      if(r != null) this.copyTo(r);
      return;
    }
    if(r == null) r = nbi();
    var y = nbi(), ts = this.s, ms = m.s;
    var nsh = this.DB-nbits(pm[pm.t-1]);	// normalize modulus
    if(nsh > 0) { pm.lShiftTo(nsh,y); pt.lShiftTo(nsh,r); }
    else { pm.copyTo(y); pt.copyTo(r); }
    var ys = y.t;
    var y0 = y[ys-1];
    if(y0 == 0) return;
    var yt = y0*(1<<this.F1)+((ys>1)?y[ys-2]>>this.F2:0);
    var d1 = this.FV/yt, d2 = (1<<this.F1)/yt, e = 1<<this.F2;
    var i = r.t, j = i-ys, t = (q==null)?nbi():q;
    y.dlShiftTo(j,t);
    if(r.compareTo(t) >= 0) {
      r[r.t++] = 1;
      r.subTo(t,r);
    }
    BigInteger.ONE.dlShiftTo(ys,t);
    t.subTo(y,y);	// "negative" y so we can replace sub with am later
    while(y.t < ys) y[y.t++] = 0;
    while(--j >= 0) {
      // Estimate quotient digit
      var qd = (r[--i]==y0)?this.DM:Math.floor(r[i]*d1+(r[i-1]+e)*d2);
      if((r[i]+=y.am(0,qd,r,j,0,ys)) < qd) {	// Try it out
        y.dlShiftTo(j,t);
        r.subTo(t,r);
        while(r[i] < --qd) r.subTo(t,r);
      }
    }
    if(q != null) {
      r.drShiftTo(ys,q);
      if(ts != ms) BigInteger.ZERO.subTo(q,q);
    }
    r.t = ys;
    r.clamp();
    if(nsh > 0) r.rShiftTo(nsh,r);	// Denormalize remainder
    if(ts < 0) BigInteger.ZERO.subTo(r,r);
  }

  // (public) this mod a
  function bnMod(a) {
    var r = nbi();
    this.abs().divRemTo(a,null,r);
    if(this.s < 0 && r.compareTo(BigInteger.ZERO) > 0) a.subTo(r,r);
    return r;
  }

  // Modular reduction using "classic" algorithm
  function Classic(m) { this.m = m; }
  function cConvert(x) {
    if(x.s < 0 || x.compareTo(this.m) >= 0) return x.mod(this.m);
    else return x;
  }
  function cRevert(x) { return x; }
  function cReduce(x) { x.divRemTo(this.m,null,x); }
  function cMulTo(x,y,r) { x.multiplyTo(y,r); this.reduce(r); }
  function cSqrTo(x,r) { x.squareTo(r); this.reduce(r); }

  Classic.prototype.convert = cConvert;
  Classic.prototype.revert = cRevert;
  Classic.prototype.reduce = cReduce;
  Classic.prototype.mulTo = cMulTo;
  Classic.prototype.sqrTo = cSqrTo;

  // (protected) return "-1/this % 2^DB"; useful for Mont. reduction
  // justification:
  //         xy == 1 (mod m)
  //         xy =  1+km
  //   xy(2-xy) = (1+km)(1-km)
  // x[y(2-xy)] = 1-k^2m^2
  // x[y(2-xy)] == 1 (mod m^2)
  // if y is 1/x mod m, then y(2-xy) is 1/x mod m^2
  // should reduce x and y(2-xy) by m^2 at each step to keep size bounded.
  // JS multiply "overflows" differently from C/C++, so care is needed here.
  function bnpInvDigit() {
    if(this.t < 1) return 0;
    var x = this[0];
    if((x&1) == 0) return 0;
    var y = x&3;		// y == 1/x mod 2^2
    y = (y*(2-(x&0xf)*y))&0xf;	// y == 1/x mod 2^4
    y = (y*(2-(x&0xff)*y))&0xff;	// y == 1/x mod 2^8
    y = (y*(2-(((x&0xffff)*y)&0xffff)))&0xffff;	// y == 1/x mod 2^16
    // last step - calculate inverse mod DV directly;
    // assumes 16 < DB <= 32 and assumes ability to handle 48-bit ints
    y = (y*(2-x*y%this.DV))%this.DV;		// y == 1/x mod 2^dbits
    // we really want the negative inverse, and -DV < y < DV
    return (y>0)?this.DV-y:-y;
  }

  // Montgomery reduction
  function Montgomery(m) {
    this.m = m;
    this.mp = m.invDigit();
    this.mpl = this.mp&0x7fff;
    this.mph = this.mp>>15;
    this.um = (1<<(m.DB-15))-1;
    this.mt2 = 2*m.t;
  }

  // xR mod m
  function montConvert(x) {
    var r = nbi();
    x.abs().dlShiftTo(this.m.t,r);
    r.divRemTo(this.m,null,r);
    if(x.s < 0 && r.compareTo(BigInteger.ZERO) > 0) this.m.subTo(r,r);
    return r;
  }

  // x/R mod m
  function montRevert(x) {
    var r = nbi();
    x.copyTo(r);
    this.reduce(r);
    return r;
  }

  // x = x/R mod m (HAC 14.32)
  function montReduce(x) {
    while(x.t <= this.mt2)	// pad x so am has enough room later
      x[x.t++] = 0;
    for(var i = 0; i < this.m.t; ++i) {
      // faster way of calculating u0 = x[i]*mp mod DV
      var j = x[i]&0x7fff;
      var u0 = (j*this.mpl+(((j*this.mph+(x[i]>>15)*this.mpl)&this.um)<<15))&x.DM;
      // use am to combine the multiply-shift-add into one call
      j = i+this.m.t;
      x[j] += this.m.am(0,u0,x,i,0,this.m.t);
      // propagate carry
      while(x[j] >= x.DV) { x[j] -= x.DV; x[++j]++; }
    }
    x.clamp();
    x.drShiftTo(this.m.t,x);
    if(x.compareTo(this.m) >= 0) x.subTo(this.m,x);
  }

  // r = "x^2/R mod m"; x != r
  function montSqrTo(x,r) { x.squareTo(r); this.reduce(r); }

  // r = "xy/R mod m"; x,y != r
  function montMulTo(x,y,r) { x.multiplyTo(y,r); this.reduce(r); }

  Montgomery.prototype.convert = montConvert;
  Montgomery.prototype.revert = montRevert;
  Montgomery.prototype.reduce = montReduce;
  Montgomery.prototype.mulTo = montMulTo;
  Montgomery.prototype.sqrTo = montSqrTo;

  // (protected) true iff this is even
  function bnpIsEven() { return ((this.t>0)?(this[0]&1):this.s) == 0; }

  // (protected) this^e, e < 2^32, doing sqr and mul with "r" (HAC 14.79)
  function bnpExp(e,z) {
    if(e > 0xffffffff || e < 1) return BigInteger.ONE;
    var r = nbi(), r2 = nbi(), g = z.convert(this), i = nbits(e)-1;
    g.copyTo(r);
    while(--i >= 0) {
      z.sqrTo(r,r2);
      if((e&(1<<i)) > 0) z.mulTo(r2,g,r);
      else { var t = r; r = r2; r2 = t; }
    }
    return z.revert(r);
  }

  // (public) this^e % m, 0 <= e < 2^32
  function bnModPowInt(e,m) {
    var z;
    if(e < 256 || m.isEven()) z = new Classic(m); else z = new Montgomery(m);
    return this.exp(e,z);
  }

  // protected
  BigInteger.prototype.copyTo = bnpCopyTo;
  BigInteger.prototype.fromInt = bnpFromInt;
  BigInteger.prototype.fromString = bnpFromString;
  BigInteger.prototype.clamp = bnpClamp;
  BigInteger.prototype.dlShiftTo = bnpDLShiftTo;
  BigInteger.prototype.drShiftTo = bnpDRShiftTo;
  BigInteger.prototype.lShiftTo = bnpLShiftTo;
  BigInteger.prototype.rShiftTo = bnpRShiftTo;
  BigInteger.prototype.subTo = bnpSubTo;
  BigInteger.prototype.multiplyTo = bnpMultiplyTo;
  BigInteger.prototype.squareTo = bnpSquareTo;
  BigInteger.prototype.divRemTo = bnpDivRemTo;
  BigInteger.prototype.invDigit = bnpInvDigit;
  BigInteger.prototype.isEven = bnpIsEven;
  BigInteger.prototype.exp = bnpExp;

  // public
  BigInteger.prototype.toString = bnToString;
  BigInteger.prototype.negate = bnNegate;
  BigInteger.prototype.abs = bnAbs;
  BigInteger.prototype.compareTo = bnCompareTo;
  BigInteger.prototype.bitLength = bnBitLength;
  BigInteger.prototype.mod = bnMod;
  BigInteger.prototype.modPowInt = bnModPowInt;

  // "constants"
  BigInteger.ZERO = nbv(0);
  BigInteger.ONE = nbv(1);

  // jsbn2 stuff

  // (protected) convert from radix string
  function bnpFromRadix(s,b) {
    this.fromInt(0);
    if(b == null) b = 10;
    var cs = this.chunkSize(b);
    var d = Math.pow(b,cs), mi = false, j = 0, w = 0;
    for(var i = 0; i < s.length; ++i) {
      var x = intAt(s,i);
      if(x < 0) {
        if(s.charAt(i) == "-" && this.signum() == 0) mi = true;
        continue;
      }
      w = b*w+x;
      if(++j >= cs) {
        this.dMultiply(d);
        this.dAddOffset(w,0);
        j = 0;
        w = 0;
      }
    }
    if(j > 0) {
      this.dMultiply(Math.pow(b,j));
      this.dAddOffset(w,0);
    }
    if(mi) BigInteger.ZERO.subTo(this,this);
  }

  // (protected) return x s.t. r^x < DV
  function bnpChunkSize(r) { return Math.floor(Math.LN2*this.DB/Math.log(r)); }

  // (public) 0 if this == 0, 1 if this > 0
  function bnSigNum() {
    if(this.s < 0) return -1;
    else if(this.t <= 0 || (this.t == 1 && this[0] <= 0)) return 0;
    else return 1;
  }

  // (protected) this *= n, this >= 0, 1 < n < DV
  function bnpDMultiply(n) {
    this[this.t] = this.am(0,n-1,this,0,0,this.t);
    ++this.t;
    this.clamp();
  }

  // (protected) this += n << w words, this >= 0
  function bnpDAddOffset(n,w) {
    if(n == 0) return;
    while(this.t <= w) this[this.t++] = 0;
    this[w] += n;
    while(this[w] >= this.DV) {
      this[w] -= this.DV;
      if(++w >= this.t) this[this.t++] = 0;
      ++this[w];
    }
  }

  // (protected) convert to radix string
  function bnpToRadix(b) {
    if(b == null) b = 10;
    if(this.signum() == 0 || b < 2 || b > 36) return "0";
    var cs = this.chunkSize(b);
    var a = Math.pow(b,cs);
    var d = nbv(a), y = nbi(), z = nbi(), r = "";
    this.divRemTo(d,y,z);
    while(y.signum() > 0) {
      r = (a+z.intValue()).toString(b).substr(1) + r;
      y.divRemTo(d,y,z);
    }
    return z.intValue().toString(b) + r;
  }

  // (public) return value as integer
  function bnIntValue() {
    if(this.s < 0) {
      if(this.t == 1) return this[0]-this.DV;
      else if(this.t == 0) return -1;
    }
    else if(this.t == 1) return this[0];
    else if(this.t == 0) return 0;
    // assumes 16 < DB < 32
    return ((this[1]&((1<<(32-this.DB))-1))<<this.DB)|this[0];
  }

  // (protected) r = this + a
  function bnpAddTo(a,r) {
    var i = 0, c = 0, m = Math.min(a.t,this.t);
    while(i < m) {
      c += this[i]+a[i];
      r[i++] = c&this.DM;
      c >>= this.DB;
    }
    if(a.t < this.t) {
      c += a.s;
      while(i < this.t) {
        c += this[i];
        r[i++] = c&this.DM;
        c >>= this.DB;
      }
      c += this.s;
    }
    else {
      c += this.s;
      while(i < a.t) {
        c += a[i];
        r[i++] = c&this.DM;
        c >>= this.DB;
      }
      c += a.s;
    }
    r.s = (c<0)?-1:0;
    if(c > 0) r[i++] = c;
    else if(c < -1) r[i++] = this.DV+c;
    r.t = i;
    r.clamp();
  }

  BigInteger.prototype.fromRadix = bnpFromRadix;
  BigInteger.prototype.chunkSize = bnpChunkSize;
  BigInteger.prototype.signum = bnSigNum;
  BigInteger.prototype.dMultiply = bnpDMultiply;
  BigInteger.prototype.dAddOffset = bnpDAddOffset;
  BigInteger.prototype.toRadix = bnpToRadix;
  BigInteger.prototype.intValue = bnIntValue;
  BigInteger.prototype.addTo = bnpAddTo;

  //======= end jsbn =======

  // Emscripten wrapper
  var Wrapper = {
    abs: function(l, h) {
      var x = new goog.math.Long(l, h);
      var ret;
      if (x.isNegative()) {
        ret = x.negate();
      } else {
        ret = x;
      }
      HEAP32[tempDoublePtr>>2] = ret.low_;
      HEAP32[tempDoublePtr+4>>2] = ret.high_;
    },
    ensureTemps: function() {
      if (Wrapper.ensuredTemps) return;
      Wrapper.ensuredTemps = true;
      Wrapper.two32 = new BigInteger();
      Wrapper.two32.fromString('4294967296', 10);
      Wrapper.two64 = new BigInteger();
      Wrapper.two64.fromString('18446744073709551616', 10);
      Wrapper.temp1 = new BigInteger();
      Wrapper.temp2 = new BigInteger();
    },
    lh2bignum: function(l, h) {
      var a = new BigInteger();
      a.fromString(h.toString(), 10);
      var b = new BigInteger();
      a.multiplyTo(Wrapper.two32, b);
      var c = new BigInteger();
      c.fromString(l.toString(), 10);
      var d = new BigInteger();
      c.addTo(b, d);
      return d;
    },
    stringify: function(l, h, unsigned) {
      var ret = new goog.math.Long(l, h).toString();
      if (unsigned && ret[0] == '-') {
        // unsign slowly using jsbn bignums
        Wrapper.ensureTemps();
        var bignum = new BigInteger();
        bignum.fromString(ret, 10);
        ret = new BigInteger();
        Wrapper.two64.addTo(bignum, ret);
        ret = ret.toString(10);
      }
      return ret;
    },
    fromString: function(str, base, min, max, unsigned) {
      Wrapper.ensureTemps();
      var bignum = new BigInteger();
      bignum.fromString(str, base);
      var bigmin = new BigInteger();
      bigmin.fromString(min, 10);
      var bigmax = new BigInteger();
      bigmax.fromString(max, 10);
      if (unsigned && bignum.compareTo(BigInteger.ZERO) < 0) {
        var temp = new BigInteger();
        bignum.addTo(Wrapper.two64, temp);
        bignum = temp;
      }
      var error = false;
      if (bignum.compareTo(bigmin) < 0) {
        bignum = bigmin;
        error = true;
      } else if (bignum.compareTo(bigmax) > 0) {
        bignum = bigmax;
        error = true;
      }
      var ret = goog.math.Long.fromString(bignum.toString()); // min-max checks should have clamped this to a range goog.math.Long can handle well
      HEAP32[tempDoublePtr>>2] = ret.low_;
      HEAP32[tempDoublePtr+4>>2] = ret.high_;
      if (error) throw 'range error';
    }
  };
  return Wrapper;
})();

//======= end closure i64 code =======



// === Auto-generated postamble setup entry stuff ===

if (memoryInitializer) {
  if (typeof Module['locateFile'] === 'function') {
    memoryInitializer = Module['locateFile'](memoryInitializer);
  } else if (Module['memoryInitializerPrefixURL']) {
    memoryInitializer = Module['memoryInitializerPrefixURL'] + memoryInitializer;
  }
  if (ENVIRONMENT_IS_NODE || ENVIRONMENT_IS_SHELL) {
    var data = Module['readBinary'](memoryInitializer);
    HEAPU8.set(data, STATIC_BASE);
  } else {
    addRunDependency('memory initializer');
    Browser.asyncLoad(memoryInitializer, function(data) {
      HEAPU8.set(data, STATIC_BASE);
      removeRunDependency('memory initializer');
    }, function(data) {
      throw 'could not load memory initializer ' + memoryInitializer;
    });
  }
}

function ExitStatus(status) {
  this.name = "ExitStatus";
  this.message = "Program terminated with exit(" + status + ")";
  this.status = status;
};
ExitStatus.prototype = new Error();
ExitStatus.prototype.constructor = ExitStatus;

var initialStackTop;
var preloadStartTime = null;
var calledMain = false;

dependenciesFulfilled = function runCaller() {
  // If run has never been called, and we should call run (INVOKE_RUN is true, and Module.noInitialRun is not false)
  if (!Module['calledRun'] && shouldRunNow) run();
  if (!Module['calledRun']) dependenciesFulfilled = runCaller; // try this again later, after new deps are fulfilled
}

Module['callMain'] = Module.callMain = function callMain(args) {
  assert(runDependencies == 0, 'cannot call main when async dependencies remain! (listen on __ATMAIN__)');
  assert(__ATPRERUN__.length == 0, 'cannot call main when preRun functions remain to be called');

  args = args || [];

  ensureInitRuntime();

  var argc = args.length+1;
  function pad() {
    for (var i = 0; i < 4-1; i++) {
      argv.push(0);
    }
  }
  var argv = [allocate(intArrayFromString(Module['thisProgram']), 'i8', ALLOC_NORMAL) ];
  pad();
  for (var i = 0; i < argc-1; i = i + 1) {
    argv.push(allocate(intArrayFromString(args[i]), 'i8', ALLOC_NORMAL));
    pad();
  }
  argv.push(0);
  argv = allocate(argv, 'i32', ALLOC_NORMAL);

  initialStackTop = STACKTOP;

  try {

    var ret = Module['_main'](argc, argv, 0);


    // if we're not running an evented main loop, it's time to exit
    exit(ret);
  }
  catch(e) {
    if (e instanceof ExitStatus) {
      // exit() throws this once it's done to make sure execution
      // has been stopped completely
      return;
    } else if (e == 'SimulateInfiniteLoop') {
      // running an evented main loop, don't immediately exit
      Module['noExitRuntime'] = true;
      return;
    } else {
      if (e && typeof e === 'object' && e.stack) Module.printErr('exception thrown: ' + [e, e.stack]);
      throw e;
    }
  } finally {
    calledMain = true;
  }
}




function run(args) {
  args = args || Module['arguments'];

  if (preloadStartTime === null) preloadStartTime = Date.now();

  if (runDependencies > 0) {
    return;
  }

  preRun();

  if (runDependencies > 0) return; // a preRun added a dependency, run will be called later
  if (Module['calledRun']) return; // run may have just been called through dependencies being fulfilled just in this very frame

  function doRun() {
    if (Module['calledRun']) return; // run may have just been called while the async setStatus time below was happening
    Module['calledRun'] = true;

    if (ABORT) return; 

    ensureInitRuntime();

    preMain();

    if (ENVIRONMENT_IS_WEB && preloadStartTime !== null) {
      Module.printErr('pre-main prep time: ' + (Date.now() - preloadStartTime) + ' ms');
    }

    if (Module['_main'] && shouldRunNow) {
      Module['callMain'](args);
    }

    postRun();
  }

  if (Module['setStatus']) {
    Module['setStatus']('Running...');
    setTimeout(function() {
      setTimeout(function() {
        Module['setStatus']('');
      }, 1);
      doRun();
    }, 1);
  } else {
    doRun();
  }
}
Module['run'] = Module.run = run;

function exit(status) {
  if (Module['noExitRuntime']) {
    return;
  }

  ABORT = true;
  EXITSTATUS = status;
  STACKTOP = initialStackTop;

  // exit the runtime
  exitRuntime();

  if (ENVIRONMENT_IS_NODE) {
    // Work around a node.js bug where stdout buffer is not flushed at process exit:
    // Instead of process.exit() directly, wait for stdout flush event.
    // See https://github.com/joyent/node/issues/1669 and https://github.com/kripken/emscripten/issues/2582
    // Workaround is based on https://github.com/RReverser/acorn/commit/50ab143cecc9ed71a2d66f78b4aec3bb2e9844f6
    process['stdout']['once']('drain', function () {
      process['exit'](status);
    });
    console.log(' '); // Make sure to print something to force the drain event to occur, in case the stdout buffer was empty.
    // Work around another node bug where sometimes 'drain' is never fired - make another effort
    // to emit the exit status, after a significant delay (if node hasn't fired drain by then, give up)
    setTimeout(function() {
      process['exit'](status);
    }, 500);
  } else
  if (ENVIRONMENT_IS_SHELL && typeof quit === 'function') {
    quit(status);
  }
  // if we reach here, we must throw an exception to halt the current execution
  throw new ExitStatus(status);
}
Module['exit'] = Module.exit = exit;

function abort(text) {
  if (text) {
    Module.print(text);
    Module.printErr(text);
  }

  ABORT = true;
  EXITSTATUS = 1;

  var extra = '\nIf this abort() is unexpected, build with -s ASSERTIONS=1 which can give more information.';

  throw 'abort() at ' + stackTrace() + extra;
}
Module['abort'] = Module.abort = abort;

// {{PRE_RUN_ADDITIONS}}

if (Module['preInit']) {
  if (typeof Module['preInit'] == 'function') Module['preInit'] = [Module['preInit']];
  while (Module['preInit'].length > 0) {
    Module['preInit'].pop()();
  }
}

// shouldRunNow refers to calling main(), not run().
var shouldRunNow = true;
if (Module['noInitialRun']) {
  shouldRunNow = false;
}


run();

// {{POST_RUN_ADDITIONS}}






// {{MODULE_ADDITIONS}}



// Generated by CoffeeScript 1.7.1
var Database, NULL, Runtime, SQLite, Statement, apiTemp, i, sqlite3_bind_blob, sqlite3_bind_double, sqlite3_bind_int, sqlite3_bind_parameter_index, sqlite3_bind_text, sqlite3_clear_bindings, sqlite3_close_v2, sqlite3_column_blob, sqlite3_column_bytes, sqlite3_column_double, sqlite3_column_name, sqlite3_column_text, sqlite3_column_type, sqlite3_data_count, sqlite3_errmsg, sqlite3_exec, sqlite3_finalize, sqlite3_free, sqlite3_open, sqlite3_prepare_v2, sqlite3_prepare_v2_sqlptr, sqlite3_reset, sqlite3_step;

Runtime = Module['Runtime'];

apiTemp = Runtime.stackAlloc(4);

SQLite = {};


/* Represents an prepared statement.

Prepared statements allow you to have a template sql string,
that you can execute multiple times with different parameters.

You can't instantiate this class directly, you have to use a [Database](Database.html)
object in order to create a statement.

**Warning**: When you close a database (using db.close()), all its statements are
closed too and become unusable.

@see Database.html#prepare-dynamic
@see https://en.wikipedia.org/wiki/Prepared_statement
 */

Statement = (function() {
  function Statement(stmt, db) {
    this.stmt = stmt;
    this.db = db;
    this.pos = 1;
    this.allocatedmem = [];
  }


  /* Bind values to the parameters, after having reseted the statement
  
  	SQL statements can have parameters, named *'?', '?NNN', ':VVV', '@VVV', '$VVV'*,
  	where NNN is a number and VVV a string.
  	This function binds these parameters to the given values.
  
  	*Warning*: ':', '@', and '$' are included in the parameters names
  
  	 *# Binding values to named parameters
  	@example Bind values to named parameters
  		var stmt = db.prepare("UPDATE test SET a=@newval WHERE id BETWEEN $mini AND $maxi");
  		stmt.bind({$mini:10, $maxi:20, '@newval':5});
  	- Create a statement that contains parameters like '$VVV', ':VVV', '@VVV'
  	- Call Statement.bind with an object as parameter
  
  	 *# Binding values to parameters
  	@example Bind values to anonymous parameters
  		var stmt = db.prepare("UPDATE test SET a=? WHERE id BETWEEN ? AND ?");
  		stmt.bind([5, 10, 20]);
  	 - Create a statement that contains parameters like '?', '?NNN'
  	 - Call Statement.bind with an array as parameter
  
  	 *# Value types
  	Javascript type | SQLite type
  	--- | ---
  	number | REAL, INTEGER
  	boolean | INTEGER
  	string | TEXT
  	Array, Uint8Array | BLOB
  	null | NULL
  	@see http://www.sqlite.org/datatype3.html
  
  	@see http://www.sqlite.org/lang_expr.html#varparam
  	@param values [Array,Object] The values to bind
  	@return [Boolean] true if it worked
  	@throw [String] SQLite Error
   */

  Statement.prototype['bind'] = function(values) {
    this['reset']();
    if (Array.isArray(values)) {
      return this.bindFromArray(values);
    } else {
      return this.bindFromObject(values);
    }
  };


  /* Execute the statement, fetching the the next line of result,
  	that can be retrieved with [Statement.get()](#get-dynamic) .
  
  	@return [Boolean] true if a row of result available
  	@throw [String] SQLite Error
   */

  Statement.prototype['step'] = function() {
    var ret;
    if (!this.stmt) {
      throw "Statement closed";
    }
    this.pos = 1;
    switch (ret = sqlite3_step(this.stmt)) {
      case SQLite.ROW:
        return true;
      case SQLite.DONE:
        return false;
      default:
        return this.db.handleError(ret);
    }
  };

  Statement.prototype.getNumber = function(pos) {
    if (pos == null) {
      pos = this.pos++;
    }
    return sqlite3_column_double(this.stmt, pos);
  };

  Statement.prototype.getString = function(pos) {
    if (pos == null) {
      pos = this.pos++;
    }
    return sqlite3_column_text(this.stmt, pos);
  };

  Statement.prototype.getBlob = function(pos) {
    var i, ptr, result, size, _i;
    if (pos == null) {
      pos = this.pos++;
    }
    size = sqlite3_column_bytes(this.stmt, pos);
    ptr = sqlite3_column_blob(this.stmt, pos);
    result = new Uint8Array(size);
    for (i = _i = 0; 0 <= size ? _i < size : _i > size; i = 0 <= size ? ++_i : --_i) {
      result[i] = HEAP8[ptr + i];
    }
    return result;
  };


  /* Get one row of results of a statement.
  	If the first parameter is not provided, step must have been called before get.
  	@param [Array,Object] Optional: If set, the values will be bound to the statement, and it will be executed
  	@return [Array<String,Number,Uint8Array,null>] One row of result
  
  	@example Print all the rows of the table test to the console
  
  		var stmt = db.prepare("SELECT * FROM test");
  		while (stmt.step()) console.log(stmt.get());
   */

  Statement.prototype['get'] = function(params) {
    var field, _i, _ref, _results;
    if (params != null) {
      this['bind'](params) && this['step']();
    }
    _results = [];
    for (field = _i = 0, _ref = sqlite3_data_count(this.stmt); 0 <= _ref ? _i < _ref : _i > _ref; field = 0 <= _ref ? ++_i : --_i) {
      switch (sqlite3_column_type(this.stmt, field)) {
        case SQLite.INTEGER:
        case SQLite.FLOAT:
          _results.push(this.getNumber(field));
          break;
        case SQLite.TEXT:
          _results.push(this.getString(field));
          break;
        case SQLite.BLOB:
          _results.push(this.getBlob(field));
          break;
        default:
          _results.push(null);
      }
    }
    return _results;
  };


  /* Get the list of column names of a row of result of a statement.
  	@return [Array<String>] The names of the columns
  	@example
  
  		var stmt = db.prepare("SELECT 5 AS nbr, x'616200' AS data, NULL AS nothing;");
  		stmt.step(); // Execute the statement
  		console.log(stmt.getColumnNames()); // Will print ['nbr','data','nothing']
   */

  Statement.prototype['getColumnNames'] = function() {
    var i, _i, _ref, _results;
    _results = [];
    for (i = _i = 0, _ref = sqlite3_data_count(this.stmt); 0 <= _ref ? _i < _ref : _i > _ref; i = 0 <= _ref ? ++_i : --_i) {
      _results.push(sqlite3_column_name(this.stmt, i));
    }
    return _results;
  };


  /* Get one row of result as a javascript object, associating column names with
  	their value in the current row.
  	@param [Array,Object] Optional: If set, the values will be bound to the statement, and it will be executed
  	@return [Object] The row of result
  	@see [Statement.get](#get-dynamic)
  
  	@example
  
  		var stmt = db.prepare("SELECT 5 AS nbr, x'616200' AS data, NULL AS nothing;");
  		stmt.step(); // Execute the statement
  		console.log(stmt.getAsObject()); // Will print {nbr:5, data: Uint8Array([1,2,3]), nothing:null}
   */

  Statement.prototype['getAsObject'] = function(params) {
    var i, name, names, rowObject, values, _i, _len;
    values = this['get'](params);
    names = this['getColumnNames']();
    rowObject = {};
    for (i = _i = 0, _len = names.length; _i < _len; i = ++_i) {
      name = names[i];
      rowObject[name] = values[i];
    }
    return rowObject;
  };


  /* Shorthand for bind + step + reset
  	Bind the values, execute the statement, ignoring the rows it returns, and resets it
  	@param [Array,Object] Value to bind to the statement
   */

  Statement.prototype['run'] = function(values) {
    if (values != null) {
      this['bind'](values);
    }
    this['step']();
    return this['reset']();
  };

  Statement.prototype.bindString = function(string, pos) {
    var bytes, strptr;
    if (pos == null) {
      pos = this.pos++;
    }
    bytes = intArrayFromString(string);
    this.allocatedmem.push(strptr = allocate(bytes, 'i8', ALLOC_NORMAL));
    this.db.handleError(sqlite3_bind_text(this.stmt, pos, strptr, bytes.length - 1, 0));
    return true;
  };

  Statement.prototype.bindBlob = function(array, pos) {
    var blobptr;
    if (pos == null) {
      pos = this.pos++;
    }
    this.allocatedmem.push(blobptr = allocate(array, 'i8', ALLOC_NORMAL));
    this.db.handleError(sqlite3_bind_blob(this.stmt, pos, blobptr, array.length, 0));
    return true;
  };

  Statement.prototype.bindNumber = function(num, pos) {
    var bindfunc;
    if (pos == null) {
      pos = this.pos++;
    }
    bindfunc = num === (num | 0) ? sqlite3_bind_int : sqlite3_bind_double;
    this.db.handleError(bindfunc(this.stmt, pos, num));
    return true;
  };

  Statement.prototype.bindNull = function(pos) {
    if (pos == null) {
      pos = this.pos++;
    }
    return sqlite3_bind_blob(this.stmt, pos, 0, 0, 0) === SQLite.OK;
  };

  Statement.prototype.bindValue = function(val, pos) {
    if (pos == null) {
      pos = this.pos++;
    }
    switch (typeof val) {
      case "string":
        return this.bindString(val, pos);
      case "number":
      case "boolean":
        return this.bindNumber(val + 0, pos);
      case "object":
        if (val === null) {
          return this.bindNull(pos);
        } else if (val.length != null) {
          return this.bindBlob(val, pos);
        } else {
          throw "Wrong API use : tried to bind a value of an unknown type (" + val + ").";
        }
    }
  };


  /* Bind names and values of an object to the named parameters of the statement
  	@param [Object]
  	@private
  	@nodoc
   */

  Statement.prototype.bindFromObject = function(valuesObj) {
    var name, num, value;
    for (name in valuesObj) {
      value = valuesObj[name];
      num = sqlite3_bind_parameter_index(this.stmt, name);
      if (num !== 0) {
        this.bindValue(value, num);
      }
    }
    return true;
  };


  /* Bind values to numbered parameters
  	@param [Array]
  	@private
  	@nodoc
   */

  Statement.prototype.bindFromArray = function(values) {
    var num, value, _i, _len;
    for (num = _i = 0, _len = values.length; _i < _len; num = ++_i) {
      value = values[num];
      this.bindValue(value, num + 1);
    }
    return true;
  };


  /* Reset a statement, so that it's parameters can be bound to new values
  	It also clears all previous bindings, freeing the memory used by bound parameters.
   */

  Statement.prototype['reset'] = function() {
    this.freemem();
    return sqlite3_clear_bindings(this.stmt) === SQLite.OK && sqlite3_reset(this.stmt) === SQLite.OK;
  };


  /* Free the memory allocated during parameter binding
   */

  Statement.prototype.freemem = function() {
    var mem;
    while (mem = this.allocatedmem.pop()) {
      _free(mem);
    }
    return null;
  };


  /* Free the memory used by the statement
  	@return [Boolean] true in case of success
   */

  Statement.prototype['free'] = function() {
    var res;
    this.freemem();
    res = sqlite3_finalize(this.stmt) === SQLite.OK;
    delete this.db.statements[this.stmt];
    this.stmt = NULL;
    return res;
  };

  return Statement;

})();

Database = (function() {
  function Database(data) {
    this.filename = 'dbfile_' + (0xffffffff * Math.random() >>> 0);
    if (data != null) {
      FS.createDataFile('/', this.filename, data, true, true);
    }
    this.handleError(sqlite3_open(this.filename, apiTemp));
    this.db = getValue(apiTemp, 'i32');
    this.statements = {};
  }


  /* Execute an SQL query, ignoring the rows it returns.
  
  	@param sql [String] a string containing some SQL text to execute
  	@param params [Array] (*optional*) When the SQL statement contains placeholders, you can pass them in here. They will be bound to the statement before it is executed.
  
  	If you use the params argument, you **cannot** provide an sql string that contains several
  	queries (separated by ';')
  
  	@example Insert values in a table
  	    db.run("INSERT INTO test VALUES (:age, :name)", {':age':18, ':name':'John'});
  
  	@return [Database] The database object (usefull for method chaining)
   */

  Database.prototype['run'] = function(sql, params) {
    var stmt;
    if (!this.db) {
      throw "Database closed";
    }
    if (params) {
      stmt = this['prepare'](sql, params);
      stmt['step']();
      stmt['free']();
    } else {
      this.handleError(sqlite3_exec(this.db, sql, 0, 0, apiTemp));
    }
    return this;
  };


  /* Execute an SQL query, and returns the result.
  
  	This is a wrapper against Database.prepare, Statement.step, Statement.get,
  	and Statement.free.
  
  	The result is an array of result elements. There are as many result elements
  	as the number of statements in your sql string (statements are separated by a semicolon)
  
  	Each result element is an object with two properties:
  		'columns' : the name of the columns of the result (as returned by Statement.getColumnNames())
  		'values' : an array of rows. Each row is itself an array of values
  
  	 *# Example use
  	We have the following table, named *test* :
  
  	| id | age |  name  |
  	|:--:|:---:|:------:|
  	| 1  |  1  | Ling   |
  	| 2  |  18 | Paul   |
  	| 3  |  3  | Markus |
  
  
  	We query it like that:
  	```javascript
  	var db = new SQL.Database();
  	var res = db.exec("SELECT id FROM test; SELECT age,name FROM test;");
  	```
  
  	`res` is now :
  	```javascript
  		[
  		 	{columns: ['id'], values:[[1],[2],[3]]},
  		 	{columns: ['age','name'], values:[[1,'Ling'],[18,'Paul'],[3,'Markus']]}
  		]
  	```
  
  	@param sql [String] a string containing some SQL text to execute
  	@return [Array<QueryResults>] An array of results.
   */

  Database.prototype['exec'] = function(sql) {
    var curresult, nextSqlPtr, pStmt, pzTail, results, stack, stmt;
    if (!this.db) {
      throw "Database closed";
    }
    stack = Runtime.stackSave();
    nextSqlPtr = Runtime.stackAlloc(sql.length << 2 + 1);
    writeStringToMemory(sql, nextSqlPtr);
    pzTail = Runtime.stackAlloc(4);
    results = [];
    while (getValue(nextSqlPtr, 'i8') !== NULL) {
      setValue(apiTemp, 0, 'i32');
      setValue(pzTail, 0, 'i32');
      this.handleError(sqlite3_prepare_v2_sqlptr(this.db, nextSqlPtr, -1, apiTemp, pzTail));
      pStmt = getValue(apiTemp, 'i32');
      nextSqlPtr = getValue(pzTail, 'i32');
      if (pStmt === NULL) {
        continue;
      }
      stmt = new Statement(pStmt, this);
      curresult = null;
      while (stmt['step']()) {
        if (curresult === null) {
          curresult = {
            'columns': stmt['getColumnNames'](),
            'values': []
          };
          results.push(curresult);
        }
        curresult['values'].push(stmt['get']());
      }
      stmt['free']();
    }
    Runtime.stackRestore(stack);
    return results;
  };


  /* Execute an sql statement, and call a callback for each row of result.
  
  	**Currently** this method is synchronous, it will not return until the callback has
  	been called on every row of the result. But this might change.
  
  	@param sql [String] A string of SQL text. Can contain placeholders that will be
  	bound to the parameters given as the second argument
  	@param params [Array<String,Number,null,Uint8Array>] (*optional*) Parameters to bind 
  	to the query
  	@param callback [Function(Object)] A function that will be called on each row of result
  	@param done [Function] A function that will be called when all rows have been retrieved
  
  	@return [Database] The database object. Usefull for method chaining
  
  	@example Read values from a table
  	    db.each("SELECT name,age FROM users WHERE age >= $majority",
  	    				{$majority:18},
  	    				function(row){console.log(row.name + " is a grown-up.")}
  	    			);
   */

  Database.prototype['each'] = function(sql, params, callback, done) {
    var stmt;
    if (typeof params === 'function') {
      done = callback;
      callback = params;
      params = void 0;
    }
    stmt = this['prepare'](sql, params);
    while (stmt['step']()) {
      callback(stmt['getAsObject']());
    }
    stmt['free']();
    if (typeof done === 'function') {
      return done();
    }
  };


  /* Prepare an SQL statement
  	@param sql [String] a string of SQL, that can contain placeholders ('?', ':VVV', ':AAA', '@AAA')
  	@param params [Array] (*optional*) values to bind to placeholders
  	@return [Statement] the resulting statement
  	@throw [String] SQLite error
   */

  Database.prototype['prepare'] = function(sql, params) {
    var pStmt, stmt;
    setValue(apiTemp, 0, 'i32');
    this.handleError(sqlite3_prepare_v2(this.db, sql, -1, apiTemp, NULL));
    pStmt = getValue(apiTemp, 'i32');
    if (pStmt === NULL) {
      throw 'Nothing to prepare';
    }
    stmt = new Statement(pStmt, this);
    if (params != null) {
      stmt.bind(params);
    }
    this.statements[pStmt] = stmt;
    return stmt;
  };


  /* Exports the contents of the database to a binary array
  	@return [Uint8Array] An array of bytes of the SQLite3 database file
   */

  Database.prototype['export'] = function() {
    var binaryDb, stmt, _, _ref;
    _ref = this.statements;
    for (_ in _ref) {
      stmt = _ref[_];
      stmt['free']();
    }
    this.handleError(sqlite3_close_v2(this.db));
    binaryDb = FS.readFile(this.filename, {
      encoding: 'binary'
    });
    this.handleError(sqlite3_open(this.filename, apiTemp));
    this.db = getValue(apiTemp, 'i32');
    return binaryDb;
  };


  /* Close the database, and all associated prepared statements.
  
  	The memory associated to the database and all associated statements
  	will be freed.
  
  	**Warning**: A statement belonging to a database that has been closed cannot
  	be used anymore.
  
  	Databases **must** be closed, when you're finished with them, or the
  	memory consumption will grow forever
   */

  Database.prototype['close'] = function() {
    var stmt, _, _ref;
    _ref = this.statements;
    for (_ in _ref) {
      stmt = _ref[_];
      stmt['free']();
    }
    this.handleError(sqlite3_close_v2(this.db));
    FS.unlink('/' + this.filename);
    return this.db = null;
  };


  /* Analyze a result code, return null if no error occured, and throw
  	an error with a descriptive message otherwise
  	@nodoc
   */

  Database.prototype.handleError = function(returnCode) {
    var errmsg;
    if (returnCode === SQLite.OK) {
      return null;
    } else {
      errmsg = sqlite3_errmsg(this.db);
      throw new Error(errmsg);
    }
  };

  return Database;

})();

sqlite3_open = Module['cwrap']('sqlite3_open', 'number', ['string', 'number']);

sqlite3_close_v2 = Module['cwrap']('sqlite3_close_v2', 'number', ['number']);

sqlite3_exec = Module['cwrap']('sqlite3_exec', 'number', ['number', 'string', 'number', 'number', 'number']);

sqlite3_free = Module['cwrap']('sqlite3_free', '', ['number']);

sqlite3_prepare_v2 = Module['cwrap']('sqlite3_prepare_v2', 'number', ['number', 'string', 'number', 'number', 'number']);

sqlite3_prepare_v2_sqlptr = Module['cwrap']('sqlite3_prepare_v2', 'number', ['number', 'number', 'number', 'number', 'number']);

sqlite3_bind_text = Module['cwrap']('sqlite3_bind_text', 'number', ['number', 'number', 'number', 'number', 'number']);

sqlite3_bind_blob = Module['cwrap']('sqlite3_bind_blob', 'number', ['number', 'number', 'number', 'number', 'number']);

sqlite3_bind_double = Module['cwrap']('sqlite3_bind_double', 'number', ['number', 'number', 'number']);

sqlite3_bind_int = Module['cwrap']('sqlite3_bind_int', 'number', ['number', 'number', 'number']);

sqlite3_bind_parameter_index = Module['cwrap']('sqlite3_bind_parameter_index', 'number', ['number', 'string']);

sqlite3_step = Module['cwrap']('sqlite3_step', 'number', ['number']);

sqlite3_errmsg = Module['cwrap']('sqlite3_errmsg', 'string', ['number']);

sqlite3_data_count = Module['cwrap']('sqlite3_data_count', 'number', ['number']);

sqlite3_column_double = Module['cwrap']('sqlite3_column_double', 'number', ['number', 'number']);

sqlite3_column_text = Module['cwrap']('sqlite3_column_text', 'string', ['number', 'number']);

sqlite3_column_blob = Module['cwrap']('sqlite3_column_blob', 'number', ['number', 'number']);

sqlite3_column_bytes = Module['cwrap']('sqlite3_column_bytes', 'number', ['number', 'number']);

sqlite3_column_type = Module['cwrap']('sqlite3_column_type', 'number', ['number', 'number']);

sqlite3_column_name = Module['cwrap']('sqlite3_column_name', 'string', ['number', 'number']);

sqlite3_reset = Module['cwrap']('sqlite3_reset', 'number', ['number']);

sqlite3_clear_bindings = Module['cwrap']('sqlite3_clear_bindings', 'number', ['number']);

sqlite3_finalize = Module['cwrap']('sqlite3_finalize', 'number', ['number']);

this['SQL'] = {
  'Database': Database
};

for (i in this['SQL']) {
  Module[i] = this['SQL'][i];
}

NULL = 0;

SQLite.OK = 0;

SQLite.ERROR = 1;

SQLite.INTERNAL = 2;

SQLite.PERM = 3;

SQLite.ABORT = 4;

SQLite.BUSY = 5;

SQLite.LOCKED = 6;

SQLite.NOMEM = 7;

SQLite.READONLY = 8;

SQLite.INTERRUPT = 9;

SQLite.IOERR = 10;

SQLite.CORRUPT = 11;

SQLite.NOTFOUND = 12;

SQLite.FULL = 13;

SQLite.CANTOPEN = 14;

SQLite.PROTOCOL = 15;

SQLite.EMPTY = 16;

SQLite.SCHEMA = 17;

SQLite.TOOBIG = 18;

SQLite.CONSTRAINT = 19;

SQLite.MISMATCH = 20;

SQLite.MISUSE = 21;

SQLite.NOLFS = 22;

SQLite.AUTH = 23;

SQLite.FORMAT = 24;

SQLite.RANGE = 25;

SQLite.NOTADB = 26;

SQLite.NOTICE = 27;

SQLite.WARNING = 28;

SQLite.ROW = 100;

SQLite.DONE = 101;

SQLite.INTEGER = 1;

SQLite.FLOAT = 2;

SQLite.TEXT = 3;

SQLite.BLOB = 4;

SQLite.NULL = 5;

return this['SQL'];
})();
if (typeof module !== 'undefined') module.exports = SQL;
if (typeof define === 'function') define(SQL);
